from canvas_sdk.clients.sendgrid.structures.address import Address
from canvas_sdk.clients.sendgrid.structures.attachment import Attachment
from canvas_sdk.clients.sendgrid.structures.body_content import BodyContent
from canvas_sdk.clients.sendgrid.structures.criterion_datetime import CriterionDatetime
from canvas_sdk.clients.sendgrid.structures.email import Email
from canvas_sdk.clients.sendgrid.structures.email_event import EmailEvent
from canvas_sdk.clients.sendgrid.structures.event_webhook import EventWebhook
from canvas_sdk.clients.sendgrid.structures.event_webhook_record import EventWebhookRecord
from canvas_sdk.clients.sendgrid.structures.logged_email_criteria import LoggedEmailCriteria
from canvas_sdk.clients.sendgrid.structures.parse_setting import ParseSetting
from canvas_sdk.clients.sendgrid.structures.parsed_attachment import ParsedAttachment
from canvas_sdk.clients.sendgrid.structures.parsed_email import ParsedEmail
from canvas_sdk.clients.sendgrid.structures.parsed_envelope import ParsedEnvelope
from canvas_sdk.clients.sendgrid.structures.parsed_header import ParsedHeader
from canvas_sdk.clients.sendgrid.structures.recipient import Recipient
from canvas_sdk.clients.sendgrid.structures.request_failed import RequestFailed
from canvas_sdk.clients.sendgrid.structures.sent_email import SentEmail
from canvas_sdk.clients.sendgrid.structures.sent_email_detail import SentEmailDetail
from canvas_sdk.clients.sendgrid.structures.settings import Settings

__all__ = __exports__ = (
    "Address",
    "Attachment",
    "BodyContent",
    "CriterionDatetime",
    "Email",
    "EmailEvent",
    "EventWebhook",
    "EventWebhookRecord",
    "LoggedEmailCriteria",
    "ParseSetting",
    "ParsedAttachment",
    "ParsedEmail",
    "ParsedEnvelope",
    "ParsedHeader",
    "Recipient",
    "RequestFailed",
    "SentEmail",
    "SentEmailDetail",
    "Settings",
)
