from canvas_sdk.clients.extend_ai.structures.processor_result.insight import Insight
from canvas_sdk.clients.extend_ai.structures.processor_result.result_classification import (
    ResultClassification,
)
from canvas_sdk.clients.extend_ai.structures.processor_result.result_extraction import (
    ResultExtraction,
)
from canvas_sdk.clients.extend_ai.structures.processor_result.result_splitter import ResultSplitter
from canvas_sdk.clients.extend_ai.structures.processor_result.split import Split

__all__ = __exports__ = (
    "Insight",
    "ResultClassification",
    "ResultExtraction",
    "ResultSplitter",
    "Split",
)
