from ..value_set import ValueSet

class AdvancedIllness(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of  advanced illnesses..

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of advanced illness when combined with older age and indications of frailty. Codes cover a wide range of conditions: heart failure, end stage renal disease, liver disease, emphysema, dementia and neurodegenerative disorders, metastatic and sever cancers, respiratory failure, and pulmonary fibrosis..

    **Exclusion Criteria:** Exclude concepts that indicate pregnancy, fetal, juvenile, childhood or neonatal conditions, concepts that indicate acute, temporary, or procedure-related conditions..

    ** Used in:** CMS131v14
    """

    VALUE_SET_NAME = "Advanced Illness"
    OID = "2.16.840.1.113883.3.464.1003.110.12.1082"
    DEFINITION_VERSION = "20190315"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "A8100",  # Creutzfeldt-Jakob disease, unspecified
        "A8101",  # Variant Creutzfeldt-Jakob disease
        "A8109",  # Other Creutzfeldt-Jakob disease
        "C250",  # Malignant neoplasm of head of pancreas
        "C251",  # Malignant neoplasm of body of pancreas
        "C252",  # Malignant neoplasm of tail of pancreas
        "C253",  # Malignant neoplasm of pancreatic duct
        "C254",  # Malignant neoplasm of endocrine pancreas
        "C257",  # Malignant neoplasm of other parts of pancreas
        "C258",  # Malignant neoplasm of overlapping sites of pancreas
        "C259",  # Malignant neoplasm of pancreas, unspecified
        "C710",  # Malignant neoplasm of cerebrum, except lobes and ventricles
        "C711",  # Malignant neoplasm of frontal lobe
        "C712",  # Malignant neoplasm of temporal lobe
        "C713",  # Malignant neoplasm of parietal lobe
        "C714",  # Malignant neoplasm of occipital lobe
        "C715",  # Malignant neoplasm of cerebral ventricle
        "C716",  # Malignant neoplasm of cerebellum
        "C717",  # Malignant neoplasm of brain stem
        "C718",  # Malignant neoplasm of overlapping sites of brain
        "C719",  # Malignant neoplasm of brain, unspecified
        "C770",  # Secondary and unspecified malignant neoplasm of lymph nodes of head, face and neck
        "C771",  # Secondary and unspecified malignant neoplasm of intrathoracic lymph nodes
        "C772",  # Secondary and unspecified malignant neoplasm of intra-abdominal lymph nodes
        "C773",  # Secondary and unspecified malignant neoplasm of axilla and upper limb lymph nodes
        "C774",  # Secondary and unspecified malignant neoplasm of inguinal and lower limb lymph nodes
        "C775",  # Secondary and unspecified malignant neoplasm of intrapelvic lymph nodes
        "C778",  # Secondary and unspecified malignant neoplasm of lymph nodes of multiple regions
        "C779",  # Secondary and unspecified malignant neoplasm of lymph node, unspecified
        "C7800",  # Secondary malignant neoplasm of unspecified lung
        "C7801",  # Secondary malignant neoplasm of right lung
        "C7802",  # Secondary malignant neoplasm of left lung
        "C781",  # Secondary malignant neoplasm of mediastinum
        "C782",  # Secondary malignant neoplasm of pleura
        "C7830",  # Secondary malignant neoplasm of unspecified respiratory organ
        "C7839",  # Secondary malignant neoplasm of other respiratory organs
        "C784",  # Secondary malignant neoplasm of small intestine
        "C785",  # Secondary malignant neoplasm of large intestine and rectum
        "C786",  # Secondary malignant neoplasm of retroperitoneum and peritoneum
        "C787",  # Secondary malignant neoplasm of liver and intrahepatic bile duct
        "C7880",  # Secondary malignant neoplasm of unspecified digestive organ
        "C7889",  # Secondary malignant neoplasm of other digestive organs
        "C7900",  # Secondary malignant neoplasm of unspecified kidney and renal pelvis
        "C7901",  # Secondary malignant neoplasm of right kidney and renal pelvis
        "C7902",  # Secondary malignant neoplasm of left kidney and renal pelvis
        "C7910",  # Secondary malignant neoplasm of unspecified urinary organs
        "C7911",  # Secondary malignant neoplasm of bladder
        "C7919",  # Secondary malignant neoplasm of other urinary organs
        "C792",  # Secondary malignant neoplasm of skin
        "C7931",  # Secondary malignant neoplasm of brain
        "C7932",  # Secondary malignant neoplasm of cerebral meninges
        "C7940",  # Secondary malignant neoplasm of unspecified part of nervous system
        "C7949",  # Secondary malignant neoplasm of other parts of nervous system
        "C7951",  # Secondary malignant neoplasm of bone
        "C7952",  # Secondary malignant neoplasm of bone marrow
        "C7960",  # Secondary malignant neoplasm of unspecified ovary
        "C7961",  # Secondary malignant neoplasm of right ovary
        "C7962",  # Secondary malignant neoplasm of left ovary
        "C7963",  # Secondary malignant neoplasm of bilateral ovaries
        "C7970",  # Secondary malignant neoplasm of unspecified adrenal gland
        "C7971",  # Secondary malignant neoplasm of right adrenal gland
        "C7972",  # Secondary malignant neoplasm of left adrenal gland
        "C7981",  # Secondary malignant neoplasm of breast
        "C7982",  # Secondary malignant neoplasm of genital organs
        "C7989",  # Secondary malignant neoplasm of other specified sites
        "C799",  # Secondary malignant neoplasm of unspecified site
        "C9100",  # Acute lymphoblastic leukemia not having achieved remission
        "C9102",  # Acute lymphoblastic leukemia, in relapse
        "C9200",  # Acute myeloblastic leukemia, not having achieved remission
        "C9202",  # Acute myeloblastic leukemia, in relapse
        "C9300",  # Acute monoblastic/monocytic leukemia, not having achieved remission
        "C9302",  # Acute monoblastic/monocytic leukemia, in relapse
        "C9390",  # Monocytic leukemia, unspecified, not having achieved remission
        "C9392",  # Monocytic leukemia, unspecified in relapse
        "C93Z0",  # Other monocytic leukemia, not having achieved remission
        "C93Z2",  # Other monocytic leukemia, in relapse
        "C9430",  # Mast cell leukemia not having achieved remission
        "C9432",  # Mast cell leukemia, in relapse
        "F0150",  # Vascular dementia, unspecified severity, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F01511",  # Vascular dementia, unspecified severity, with agitation
        "F01518",  # Vascular dementia, unspecified severity, with other behavioral disturbance
        "F0152",  # Vascular dementia, unspecified severity, with psychotic disturbance
        "F0153",  # Vascular dementia, unspecified severity, with mood disturbance
        "F0154",  # Vascular dementia, unspecified severity, with anxiety
        "F01A0",  # Vascular dementia, mild, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F01A11",  # Vascular dementia, mild, with agitation
        "F01A18",  # Vascular dementia, mild, with other behavioral disturbance
        "F01A2",  # Vascular dementia, mild, with psychotic disturbance
        "F01A3",  # Vascular dementia, mild, with mood disturbance
        "F01A4",  # Vascular dementia, mild, with anxiety
        "F01B0",  # Vascular dementia, moderate, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F01B11",  # Vascular dementia, moderate, with agitation
        "F01B18",  # Vascular dementia, moderate, with other behavioral disturbance
        "F01B2",  # Vascular dementia, moderate, with psychotic disturbance
        "F01B3",  # Vascular dementia, moderate, with mood disturbance
        "F01B4",  # Vascular dementia, moderate, with anxiety
        "F01C0",  # Vascular dementia, severe, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F01C11",  # Vascular dementia, severe, with agitation
        "F01C18",  # Vascular dementia, severe, with other behavioral disturbance
        "F01C2",  # Vascular dementia, severe, with psychotic disturbance
        "F01C3",  # Vascular dementia, severe, with mood disturbance
        "F01C4",  # Vascular dementia, severe, with anxiety
        "F0280",  # Dementia in other diseases classified elsewhere, unspecified severity, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F02811",  # Dementia in other diseases classified elsewhere, unspecified severity, with agitation
        "F02818",  # Dementia in other diseases classified elsewhere, unspecified severity, with other behavioral disturbance
        "F0282",  # Dementia in other diseases classified elsewhere, unspecified severity, with psychotic disturbance
        "F0283",  # Dementia in other diseases classified elsewhere, unspecified severity, with mood disturbance
        "F0284",  # Dementia in other diseases classified elsewhere, unspecified severity, with anxiety
        "F02A0",  # Dementia in other diseases classified elsewhere, mild, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F02A11",  # Dementia in other diseases classified elsewhere, mild, with agitation
        "F02A18",  # Dementia in other diseases classified elsewhere, mild, with other behavioral disturbance
        "F02A2",  # Dementia in other diseases classified elsewhere, mild, with psychotic disturbance
        "F02A3",  # Dementia in other diseases classified elsewhere, mild, with mood disturbance
        "F02A4",  # Dementia in other diseases classified elsewhere, mild, with anxiety
        "F02B0",  # Dementia in other diseases classified elsewhere, moderate, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F02B11",  # Dementia in other diseases classified elsewhere, moderate, with agitation
        "F02B18",  # Dementia in other diseases classified elsewhere, moderate, with other behavioral disturbance
        "F02B2",  # Dementia in other diseases classified elsewhere, moderate, with psychotic disturbance
        "F02B3",  # Dementia in other diseases classified elsewhere, moderate, with mood disturbance
        "F02B4",  # Dementia in other diseases classified elsewhere, moderate, with anxiety
        "F02C0",  # Dementia in other diseases classified elsewhere, severe, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F02C11",  # Dementia in other diseases classified elsewhere, severe, with agitation
        "F02C18",  # Dementia in other diseases classified elsewhere, severe, with other behavioral disturbance
        "F02C2",  # Dementia in other diseases classified elsewhere, severe, with psychotic disturbance
        "F02C3",  # Dementia in other diseases classified elsewhere, severe, with mood disturbance
        "F02C4",  # Dementia in other diseases classified elsewhere, severe, with anxiety
        "F0390",  # Unspecified dementia, unspecified severity, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F03911",  # Unspecified dementia, unspecified severity, with agitation
        "F03918",  # Unspecified dementia, unspecified severity, with other behavioral disturbance
        "F0392",  # Unspecified dementia, unspecified severity, with psychotic disturbance
        "F0393",  # Unspecified dementia, unspecified severity, with mood disturbance
        "F0394",  # Unspecified dementia, unspecified severity, with anxiety
        "F03A0",  # Unspecified dementia, mild, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F03A11",  # Unspecified dementia, mild, with agitation
        "F03A18",  # Unspecified dementia, mild, with other behavioral disturbance
        "F03A2",  # Unspecified dementia, mild, with psychotic disturbance
        "F03A3",  # Unspecified dementia, mild, with mood disturbance
        "F03A4",  # Unspecified dementia, mild, with anxiety
        "F03B0",  # Unspecified dementia, moderate, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F03B11",  # Unspecified dementia, moderate, with agitation
        "F03B18",  # Unspecified dementia, moderate, with other behavioral disturbance
        "F03B2",  # Unspecified dementia, moderate, with psychotic disturbance
        "F03B3",  # Unspecified dementia, moderate, with mood disturbance
        "F03B4",  # Unspecified dementia, moderate, with anxiety
        "F03C0",  # Unspecified dementia, severe, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F03C11",  # Unspecified dementia, severe, with agitation
        "F03C18",  # Unspecified dementia, severe, with other behavioral disturbance
        "F03C2",  # Unspecified dementia, severe, with psychotic disturbance
        "F03C3",  # Unspecified dementia, severe, with mood disturbance
        "F03C4",  # Unspecified dementia, severe, with anxiety
        "F04",  # Amnestic disorder due to known physiological condition
        "F1027",  # Alcohol dependence with alcohol-induced persisting dementia
        "F1096",  # Alcohol use, unspecified with alcohol-induced persisting amnestic disorder
        "F1097",  # Alcohol use, unspecified with alcohol-induced persisting dementia
        "G10",  # Huntington's disease
        "G1221",  # Amyotrophic lateral sclerosis
        "G20A1",  # Parkinson's disease without dyskinesia, without mention of fluctuations
        "G20A2",  # Parkinson's disease without dyskinesia, with fluctuations
        "G20B1",  # Parkinson's disease with dyskinesia, without mention of fluctuations
        "G20B2",  # Parkinson's disease with dyskinesia, with fluctuations
        "G20C",  # Parkinsonism, unspecified
        "G300",  # Alzheimer's disease with early onset
        "G301",  # Alzheimer's disease with late onset
        "G308",  # Other Alzheimer's disease
        "G309",  # Alzheimer's disease, unspecified
        "G3101",  # Pick's disease
        "G3109",  # Other frontotemporal neurocognitive disorder
        "G3183",  # Neurocognitive disorder with Lewy bodies
        "G35",  # Multiple sclerosis
        "I0981",  # Rheumatic heart failure
        "I110",  # Hypertensive heart disease with heart failure
        "I120",  # Hypertensive chronic kidney disease with stage 5 chronic kidney disease or end stage renal disease
        "I130",  # Hypertensive heart and chronic kidney disease with heart failure and stage 1 through stage 4 chronic kidney disease, or unspecified chronic kidney disease
        "I1311",  # Hypertensive heart and chronic kidney disease without heart failure, with stage 5 chronic kidney disease, or end stage renal disease
        "I132",  # Hypertensive heart and chronic kidney disease with heart failure and with stage 5 chronic kidney disease, or end stage renal disease
        "I501",  # Left ventricular failure, unspecified
        "I5020",  # Unspecified systolic (congestive) heart failure
        "I5021",  # Acute systolic (congestive) heart failure
        "I5022",  # Chronic systolic (congestive) heart failure
        "I5023",  # Acute on chronic systolic (congestive) heart failure
        "I5030",  # Unspecified diastolic (congestive) heart failure
        "I5031",  # Acute diastolic (congestive) heart failure
        "I5032",  # Chronic diastolic (congestive) heart failure
        "I5033",  # Acute on chronic diastolic (congestive) heart failure
        "I5040",  # Unspecified combined systolic (congestive) and diastolic (congestive) heart failure
        "I5041",  # Acute combined systolic (congestive) and diastolic (congestive) heart failure
        "I5042",  # Chronic combined systolic (congestive) and diastolic (congestive) heart failure
        "I5043",  # Acute on chronic combined systolic (congestive) and diastolic (congestive) heart failure
        "I50810",  # Right heart failure, unspecified
        "I50811",  # Acute right heart failure
        "I50812",  # Chronic right heart failure
        "I50813",  # Acute on chronic right heart failure
        "I50814",  # Right heart failure due to left heart failure
        "I5082",  # Biventricular heart failure
        "I5083",  # High output heart failure
        "I5084",  # End stage heart failure
        "I5089",  # Other heart failure
        "I509",  # Heart failure, unspecified
        "J430",  # Unilateral pulmonary emphysema [MacLeod's syndrome]
        "J431",  # Panlobular emphysema
        "J432",  # Centrilobular emphysema
        "J438",  # Other emphysema
        "J439",  # Emphysema, unspecified
        "J684",  # Chronic respiratory conditions due to chemicals, gases, fumes and vapors
        "J8410",  # Pulmonary fibrosis, unspecified
        "J84112",  # Idiopathic pulmonary fibrosis
        "J84170",  # Interstitial lung disease with progressive fibrotic phenotype in diseases classified elsewhere
        "J84178",  # Other interstitial pulmonary diseases with fibrosis in diseases classified elsewhere
        "J9610",  # Chronic respiratory failure, unspecified whether with hypoxia or hypercapnia
        "J9611",  # Chronic respiratory failure with hypoxia
        "J9612",  # Chronic respiratory failure with hypercapnia
        "J9620",  # Acute and chronic respiratory failure, unspecified whether with hypoxia or hypercapnia
        "J9621",  # Acute and chronic respiratory failure with hypoxia
        "J9622",  # Acute and chronic respiratory failure with hypercapnia
        "J9690",  # Respiratory failure, unspecified, unspecified whether with hypoxia or hypercapnia
        "J9691",  # Respiratory failure, unspecified with hypoxia
        "J9692",  # Respiratory failure, unspecified with hypercapnia
        "J982",  # Interstitial emphysema
        "J983",  # Compensatory emphysema
        "K7010",  # Alcoholic hepatitis without ascites
        "K7011",  # Alcoholic hepatitis with ascites
        "K702",  # Alcoholic fibrosis and sclerosis of liver
        "K7030",  # Alcoholic cirrhosis of liver without ascites
        "K7031",  # Alcoholic cirrhosis of liver with ascites
        "K7040",  # Alcoholic hepatic failure without coma
        "K7041",  # Alcoholic hepatic failure with coma
        "K709",  # Alcoholic liver disease, unspecified
        "K7400",  # Hepatic fibrosis, unspecified
        "K7401",  # Hepatic fibrosis, early fibrosis
        "K7402",  # Hepatic fibrosis, advanced fibrosis
        "K741",  # Hepatic sclerosis
        "K742",  # Hepatic fibrosis with hepatic sclerosis
        "K744",  # Secondary biliary cirrhosis
        "K745",  # Biliary cirrhosis, unspecified
        "K7460",  # Unspecified cirrhosis of liver
        "K7469",  # Other cirrhosis of liver
        "N185",  # Chronic kidney disease, stage 5
        "N186",  # End stage renal disease
    }

    SNOMEDCT = {
        "1236965009",  # Metastatic renal cell carcinoma to kidney except renal pelvis (disorder)
        "1240357005",  # Metastatic renal cell carcinoma to kidney (disorder)
        "1254729000",  # Metastatic lobular carcinoma to breast (disorder)
        "1254730005",  # Metastatic lobular carcinoma to left female breast (disorder)
        "1254731009",  # Metastatic lobular carcinoma to right female breast (disorder)
        "1263997005",  # Metastatic pilomatrix carcinoma to skin (disorder)
        "1263998000",  # Metastatic cystic renal cell carcinoma to kidney (disorder)
        "1264257008",  # Metastatic apocrine adenocarcinoma to skin (disorder)
        "1264521002",  # Metastatic collecting duct carcinoma to kidney (disorder)
        "1264523004",  # Metastatic cholangiocarcinoma to perihilar bile duct (disorder)
        "1264524005",  # Metastatic cholangiocarcinoma to biliary tract (disorder)
        "1268355005",  # Metastatic eccrine porocarcinoma to skin (disorder)
        "781076008",  # Metastatic malignant neoplasm to colon and/or rectum (disorder)
        "1237491009",  # Metastatic carcinoma to male breast (disorder)
        "1264496004",  # Metastatic carcinoma to female breast (disorder)
        "1264498003",  # Metastatic infiltrating duct carcinoma to female breast (disorder)
        "233765002",  # Respiratory failure without hypercapnia (disorder)
        "705176003",  # Metastatic carcinoid tumor (disorder)
        "733185001",  # Dementia following injury caused by exposure to ionizing radiation (disorder)
        "94176003",  # Metastatic malignant neoplasm to areola of female breast (disorder)
        "94177007",  # Metastatic malignant neoplasm to areola of male breast (disorder)
        "94182000",  # Metastatic malignant neoplasm to axillary tail of female breast (disorder)
        "94244003",  # Metastatic malignant neoplasm to central portion of female breast (disorder)
        "94276008",  # Metastatic malignant neoplasm to ectopic female breast tissue (disorder)
        "94277004",  # Metastatic malignant neoplasm to ectopic male breast tissue (disorder)
        "94297009",  # Metastatic malignant neoplasm to female breast (disorder)
        "94385006",  # Metastatic malignant neoplasm to lower inner quadrant of female breast (disorder)
        "94387003",  # Metastatic malignant neoplasm to lower outer quadrant of female breast (disorder)
        "94401004",  # Metastatic malignant neoplasm to male breast (disorder)
        "94443006",  # Metastatic malignant neoplasm to nipple of female breast (disorder)
        "94444000",  # Metastatic malignant neoplasm to nipple of male breast (disorder)
        "94653000",  # Metastatic malignant neoplasm to upper inner quadrant of female breast (disorder)
        "94655007",  # Metastatic malignant neoplasm to upper outer quadrant of female breast (disorder)
        "100721000119109",  # High grade astrocytoma of brain (disorder)
        "100731000119107",  # Low grade astrocytoma of brain (disorder)
        "10091002",  # High output heart failure (disorder)
        "1010333003",  # Emphysema of left lung (disorder)
        "1010334009",  # Emphysema of right lung (disorder)
        "1010616001",  # Cirrhosis of liver due to classical cystic fibrosis (disorder)
        "101281000119107",  # Congestive heart failure due to cardiomyopathy (disorder)
        "101421000119107",  # Dementia due to Parkinson's disease (disorder)
        "1017196003",  # Interstitial pulmonary fibrosis due to inhalation of substance (disorder)
        "1017197007",  # Interstitial pulmonary fibrosis due to inhalation of drug (disorder)
        "10335000",  # Chronic right-sided heart failure (disorder)
        "10349009",  # Multi-infarct dementia with delirium (disorder)
        "103511000119103",  # Melanoma carcinomatosis (disorder)
        "103611000119102",  # Cirrhosis of liver due to hepatitis B (disorder)
        "104981000119104",  # Oligodendroglioma of cerebrum (disorder)
        "105041000119109",  # Metastatic squamous cell carcinoma to lung (disorder)
        "105051000119106",  # Metastatic undifferentiated large cell carcinoma to lung (disorder)
        "10532003",  # Primary degenerative dementia of the Alzheimer type, presenile onset, with depression (disorder)
        "105421000119105",  # Early onset Alzheimer's disease with behavioral disturbance (disorder)
        "106021000119105",  # Multi-infarct dementia due to atherosclerosis (disorder)
        "10633002",  # Acute congestive heart failure (disorder)
        "10672271000119100",  # Acute-on-chronic hypoxemic respiratory failure (disorder)
        "10676831000119101",  # Hypoxemic respiratory failure (disorder)
        "107571000119101",  # Oligodendroglioma of brain stem (disorder)
        "107581000119103",  # Astrocytoma of brain stem (disorder)
        "10762071000119109",  # Acute-on-chronic respiratory failure following trauma (disorder)
        "107811000119106",  # Metastatic small cell carcinoma to liver (disorder)
        "107831000119101",  # Metastatic squamous cell carcinoma to liver (disorder)
        "1079531000119106",  # Metastatic malignant neoplasm to left kidney (disorder)
        "1079541000119102",  # Metastatic malignant neoplasm to right kidney (disorder)
        "108101000119101",  # Malignant melanoma metastatic to kidney (disorder)
        "108131000119108",  # Small cell carcinoma metastatic to kidney (disorder)
        "108201000119105",  # Undifferentiated large cell carcinoma metastatic to kidney (disorder)
        "108211000119108",  # Secondary adenocarcinoma of kidney (disorder)
        "1082601000112109",  # Metastatic malignant neoplasm to viscera (disorder)
        "109819003",  # Obstructive biliary cirrhosis (disorder)
        "109848009",  # Overlapping malignant neoplasm of pancreas (disorder)
        "109912006",  # Overlapping malignant neoplasm of brain (disorder)
        "111283005",  # Chronic left-sided heart failure (disorder)
        "111411000119103",  # End stage renal disease due to hypertension (disorder)
        "113901000052102",  # Heart failure following administration of antineoplastic agent (disorder)
        "1141626005",  # Adenocarcinoma of pancreas with neuregulin 1 gene fusion (disorder)
        "11471000224106",  # Diffuse intrinsic pontine glioma (disorder)
        "1148551007",  # Infiltrating duct carcinoma of pancreas (disorder)
        "1148924004",  # Dementia due to deficiency of folic acid (disorder)
        "1155991005",  # Metastatic malignant neoplasm to leptomeninges (disorder)
        "1156413005",  # Gliomatosis cerebri (disorder)
        "1156414004",  # Giant cell glioblastoma of brain (disorder)
        "1156415003",  # Protoplasmic astrocytoma of brain (disorder)
        "1156416002",  # Gemistocytic astrocytoma of brain (disorder)
        "1156417006",  # Fibrillary astrocytoma of brain (disorder)
        "1156455001",  # Pleomorphic xanthoastrocytoma of brain (disorder)
        "1156456000",  # Pilomyxoid astrocytoma of brain (disorder)
        "1156460002",  # Desmoplastic nodular medulloblastoma of brain (disorder)
        "1156461003",  # Gliosarcoma of brain (disorder)
        "1156469001",  # Large cell medulloblastoma of brain (disorder)
        "1156471001",  # Choroid plexus carcinoma (disorder)
        "1156472008",  # Papillary tumor of pineal region (disorder)
        "1156789004",  # Autosomal dominant Alzheimer disease due to mutation of amyloid precursor protein (disorder)
        "1156798001",  # Autosomal dominant Alzheimer disease due to mutation of presenilin 2 (disorder)
        "1156800008",  # Autosomal dominant Alzheimer disease due to mutation of presenilin 1 (disorder)
        "1156822001",  # Autosomal recessive familial Parkinson disease (disorder)
        "1157060001",  # Diffuse astrocytoma of brain (disorder)
        "1163119007",  # Creutzfeldt Jakob disease following graft of dura (disorder)
        "1163147006",  # Chronic fibrosis of lung caused by Aspergillus (disorder)
        "116811000119106",  # Non-Hodgkin lymphoma of central nervous system metastatic to lymph node of lower limb (disorder)
        "116821000119104",  # Non-Hodgkin lymphoma of central nervous system metastatic to lymph node of upper limb (disorder)
        "11717361000119108",  # Metastatic malignant neoplasm to soft tissue of back (disorder)
        "1177007001",  # Fibrosis of lung due to pulmonary paracoccidioidomycosis (disorder)
        "1177062005",  # Sporadic fatal insomnia (disorder)
        "1179377000",  # Adenocarcinoma of pituitary gland (disorder)
        "1186612004",  # Metastatic hepatocellular carcinoma (disorder)
        "1186724002",  # HtrA serine peptidase 1-related autosomal dominant cerebral small vessel disease (disorder)
        "1186728004",  # Pediatric multiple sclerosis (disorder)
        "1186735007",  # Mixed neuroendocrine-non neuroendocrine neoplasm of pancreas (disorder)
        "1186869002",  # Neuroendocrine carcinoma of pancreas (disorder)
        "1186877003",  # Dementia due to vitamin E deficiency (disorder)
        "1186879000",  # Dementia due to thiamine deficiency (disorder)
        "1186880002",  # Dementia due to cobalamin deficiency (disorder)
        "1186881003",  # Dementia due to niacin deficiency (disorder)
        "1186883000",  # Dementia due to nutritional deficiency disorder (disorder)
        "1186887004",  # Dementia caused by manganese and/or manganese compound (disorder)
        "1187004001",  # Chronic traumatic encephalopathy (disorder)
        "1187126002",  # Integral membrane protein 2B related amyloidosis (disorder)
        "1187285005",  # Metastatic clear cell renal cell carcinoma (disorder)
        "1187519007",  # Malignant gastrinoma of pancreas (disorder)
        "1187521002",  # Malignant glucagonoma of pancreas (disorder)
        "1196839006",  # Primary embryonal carcinoma of brain (disorder)
        "1196844004",  # Primary malignant astrocytoma of brain (disorder)
        "1196860000",  # Primary anaplastic large cell medulloblastoma of brain (disorder)
        "1196898000",  # Primary malignant atypical teratoid rhabdoid neoplasm of brain (disorder)
        "1197279000",  # Primary malignant solid pseudopapillary neoplasm of pancreas (disorder)
        "1197283000",  # Primary mixed acinar endocrine-ductal carcinoma of pancreas (disorder)
        "1197285007",  # Metastatic squamous cell carcinoma to thoracic lymph node (disorder)
        "1197286008",  # Primary mixed acinar ductal carcinoma of pancreas (disorder)
        "1197292002",  # Metastatic squamous cell carcinoma of pelvic lymph node (disorder)
        "1197293007",  # Metastatic squamous cell carcinoma to lymph nodes of head and neck (disorder)
        "1197295000",  # Metastatic squamous cell carcinoma to lymph nodes of multiple sites (disorder)
        "1197297008",  # Metastatic squamous cell carcinoma to lymph node of lower limb (disorder)
        "1197298003",  # Primary malignant germ cell neoplasm of pineal gland (disorder)
        "1197299006",  # Metastatic squamous cell carcinoma to lymph node of upper limb (disorder)
        "1197310007",  # Metastatic adenocarcinoma to lymph node of lower limb (disorder)
        "1197311006",  # Metastatic adenocarcinoma to lymph node of upper limb (disorder)
        "1197312004",  # Metastatic adenocarcinoma to abdominal lymph node (disorder)
        "1197313009",  # Metastatic adenocarcinoma to lymph node of head and/or neck (disorder)
        "1197322005",  # Metastatic squamous cell carcinoma to abdominal lymph node (disorder)
        "1197323000",  # Metastatic carcinoma to lymph node of thorax (disorder)
        "1197324006",  # Metastatic malignant melanoma to skin (disorder)
        "1197325007",  # Metastatic carcinoma of lymph node of pelvis (disorder)
        "1197326008",  # Metastatic carcinoma to lymph node of head and neck (disorder)
        "1197327004",  # Metastatic carcinoma to lymph node of lower limb (disorder)
        "1197328009",  # Metastatic adenocarcinoma to thoracic lymph node (disorder)
        "1197329001",  # Metastatic carcinoma of abdominal lymph node (disorder)
        "1197330006",  # Metastatic adenocarcinoma to lymph nodes of multiple sites (disorder)
        "1197331005",  # Metastatic adenocarcinoma to pelvic lymph node (disorder)
        "1197332003",  # Primary malignant melanoma of central nervous system (disorder)
        "1197343006",  # Primary teratoma of brain (disorder)
        "1197523001",  # Autosomal dominant amyotrophic lateral sclerosis type 1 (disorder)
        "1197524007",  # Autosomal recessive amyotrophic lateral sclerosis type 1 (disorder)
        "1201863001",  # Amyotrophic lateral sclerosis type 1 (disorder)
        "1201947005",  # Juvenile amyotrophic lateral sclerosis type 2 (disorder)
        "1201950008",  # Amyotrophic lateral sclerosis type 3 (disorder)
        "1201961000",  # Juvenile amyotrophic lateral sclerosis type 5 (disorder)
        "1204200007",  # Left ventricular failure with normal ejection fraction due to valvular heart disease (disorder)
        "1204203009",  # Left ventricular failure with normal ejection fraction due to coronary arteriosclerosis (disorder)
        "1204204003",  # Left ventricular failure with normal ejection fraction due to myocarditis (disorder)
        "1204206001",  # Left ventricular failure with normal ejection fraction due to cardiomyopathy (disorder)
        "1204334005",  # Amyotrophic lateral sclerosis type 6 (disorder)
        "1204349002",  # Amyotrophic lateral sclerosis type 7 (disorder)
        "1204350002",  # Amyotrophic lateral sclerosis type 8 (disorder)
        "1204351003",  # Amyotrophic lateral sclerosis type 9 (disorder)
        "1204460007",  # Myocardial dysfunction with sepsis (disorder)
        "1204462004",  # Left ventricular failure with sepsis (disorder)
        "1204468000",  # Right ventricular failure with sepsis (disorder)
        "1208412003",  # Amyotrophic lateral sclerosis type 10 (disorder)
        "120851000119104",  # Systolic heart failure stage D (disorder)
        "120861000119102",  # Systolic heart failure stage C (disorder)
        "120871000119108",  # Systolic heart failure stage B (disorder)
        "1208744003",  # Non-functioning neuroendocrine neoplasm of pancreas (disorder)
        "1208745002",  # Serotonin-producing neuroendocrine neoplasm of pancreas (disorder)
        "120881000119106",  # Diastolic heart failure stage D (disorder)
        "1208843003",  # Right ventricular failure due to heart valve disorder (disorder)
        "1208846006",  # Right ventricular failure due to disorder of lung (disorder)
        "1208848007",  # Right ventricular failure due to disorder of pulmonary circulation (disorder)
        "1208850004",  # Right ventricular failure due to right ventricular infarction (disorder)
        "120891000119109",  # Diastolic heart failure stage C (disorder)
        "120901000119108",  # Diastolic heart failure stage B (disorder)
        "1217383007",  # RELA fusion-positive supratentorial ependymoma (disorder)
        "12241031000119108",  # Metastatic malignant neoplasm to left breast (disorder)
        "12241071000119106",  # Metastatic malignant neoplasm to right breast (disorder)
        "12246481000119106",  # Metastatic malignant neoplasm to bilateral kidneys (disorder)
        "12246561000119101",  # Metastatic malignant neoplasm to bilateral adrenal glands (disorder)
        "12246601000119101",  # Metastatic malignant neoplasm to bilateral lungs (disorder)
        "12246641000119104",  # Metastatic malignant neoplasm to bilateral breasts (disorder)
        "1230308005",  # Off-periods in Parkinson disease not responding to oral treatment (disorder)
        "12348006",  # Presenile dementia (disorder)
        "123604002",  # Toxic cirrhosis (disorder)
        "123606000",  # Cholangiolitic cirrhosis (disorder)
        "12368000",  # Secondary biliary cirrhosis (disorder)
        "1236931008",  # Metastatic carcinoma to body of pancreas (disorder)
        "1236940007",  # Metastatic carcinoma to tail of pancreas (disorder)
        "1236941006",  # Metastatic carcinoma to intrahepatic bile duct (disorder)
        "1236942004",  # Metastatic carcinoma to ventral surface of tongue (disorder)
        "1236943009",  # Metastatic carcinoma to anterior two-thirds tongue ventrum (disorder)
        "1236944003",  # Metastatic carcinoma to dorsal surface of anterior two-thirds of tongue (disorder)
        "1236945002",  # Metastatic carcinoma to midline of tongue (disorder)
        "1236947005",  # Metastatic carcinoma to dorsal surface of base of tongue (disorder)
        "1236948000",  # Metastatic carcinoma to fallopian tube (disorder)
        "1236953005",  # Metastatic carcinoma to bone, connective tissue, skin and breast (disorder)
        "1236954004",  # Metastatic carcinoma to vermilion border of upper lip (disorder)
        "1236955003",  # Metastatic carcinoma to frenum of lip (disorder)
        "1236956002",  # Metastatic carcinoma to frenum of lower lip (disorder)
        "1236957006",  # Metastatic carcinoma to frenum of upper lip (disorder)
        "1236958001",  # Metastatic carcinoma to commissure of lip (disorder)
        "1236960004",  # Metastatic carcinoma to lingual tonsil (disorder)
        "1236961000",  # Metastatic carcinoma to anterior part of floor of mouth (disorder)
        "1236962007",  # Metastatic carcinoma to lateral part of floor of mouth (disorder)
        "1236963002",  # Metastatic carcinoma to hard palate (disorder)
        "1236964008",  # Metastatic carcinoma to soft palate (disorder)
        "1236966005",  # Metastatic carcinoma to upper buccal sulcus (disorder)
        "1236967001",  # Metastatic carcinoma to lower buccal sulcus (disorder)
        "1236968006",  # Metastatic carcinoma to upper labial sulcus (disorder)
        "1236969003",  # Metastatic carcinoma to lower labial sulcus (disorder)
        "1237031003",  # Metastatic carcinoma to fundus of stomach (disorder)
        "1237032005",  # Metastatic carcinoma to thoracic part of esophagus (disorder)
        "1237033000",  # Metastatic carcinoma to sublingual gland (disorder)
        "1237034006",  # Metastatic carcinoma to submandibular gland (disorder)
        "1237035007",  # Metastatic carcinoma to parotid gland (disorder)
        "1237036008",  # Metastatic carcinoma to middle third of esophagus (disorder)
        "1237037004",  # Metastatic carcinoma to cervical part of esophagus (disorder)
        "1237038009",  # Metastatic carcinoma to abdominal part of esophagus (disorder)
        "1237039001",  # Metastatic carcinoma to upper third of esophagus (disorder)
        "1237041000",  # Metastatic carcinoma to pylorus (disorder)
        "1237042007",  # Metastatic carcinoma to pyloric antrum (disorder)
        "1237043002",  # Metastatic carcinoma to greater curve of stomach (disorder)
        "1237044008",  # Metastatic carcinoma to lesser curve of stomach (disorder)
        "1237045009",  # Metastatic carcinoma to body of stomach (disorder)
        "1237046005",  # Metastatic carcinoma to duodenum (disorder)
        "1237047001",  # Metastatic carcinoma to endocrine pancreas (disorder)
        "1237048006",  # Metastatic squamous cell carcinoma to bronchus (disorder)
        "1237050003",  # Metastatic carcinoma to cardia of stomach (disorder)
        "1237051004",  # Metastatic carcinoma to lower third of esophagus (disorder)
        "1237052006",  # Metastatic carcinoma to axillary tail of breast (disorder)
        "1237053001",  # Metastatic carcinoma to upper inner quadrant of breast (disorder)
        "1237055008",  # Metastatic carcinoma to upper outer quadrant of breast (disorder)
        "1237056009",  # Metastatic carcinoma to lower inner quadrant of breast (disorder)
        "1237057000",  # Metastatic carcinoma to lower outer quadrant of breast (disorder)
        "123716002",  # Latent cirrhosis (disorder)
        "123717006",  # Advanced cirrhosis (disorder)
        "1237341006",  # Intraductal tubulopapillary malignant neoplasm of pancreas (disorder)
        "1237372005",  # Metastatic carcinoma to vagina (disorder)
        "1237414000",  # Metastatic cystadenocarcinoma to kidney (disorder)
        "1237415004",  # Metastatic cystadenocarcinoma to ovary (disorder)
        "1237416003",  # Metastatic cystadenocarcinoma to pancreas (disorder)
        "1237422007",  # Metastatic carcinoma to prostate (disorder)
        "1237423002",  # Metastatic carcinoma to glans penis (disorder)
        "1237424008",  # Metastatic carcinoma to foreskin (disorder)
        "1237425009",  # Metastatic carcinoma to pituitary (disorder)
        "1237426005",  # Metastatic carcinoma to parathyroid (disorder)
        "1237427001",  # Metastatic carcinoma to cecum (disorder)
        "1237428006",  # Metastatic carcinoma to common bile duct (disorder)
        "1237454003",  # Metastatic carcinoma to splenic flexure (disorder)
        "1237455002",  # Metastatic carcinoma to descending colon (disorder)
        "1237456001",  # Metastatic carcinoma to hepatic flexure (disorder)
        "1237457005",  # Metastatic carcinoma to urinary bladder (disorder)
        "1237458000",  # Metastatic carcinoma to transverse colon (disorder)
        "1237460003",  # Metastatic carcinoma to ascending colon (disorder)
        "1237461004",  # Metastatic adenocarcinoma to retinal pigment epithelium (disorder)
        "1237465008",  # Metastatic adenocarcinoma to urinary bladder (disorder)
        "1237466009",  # Metastatic squamous cell carcinoma to bronchus of left upper lobe (disorder)
        "1237467000",  # Metastatic squamous cell carcinoma to bronchus of right upper lobe (disorder)
        "1237469002",  # Metastatic squamous cell carcinoma to bronchus of left lower lobe (disorder)
        "1237471002",  # Metastatic squamous cell carcinoma to bronchus of right lower lobe (disorder)
        "1237472009",  # Metastatic squamous cell carcinoma to bronchus of middle lobe of right lung (disorder)
        "1237473004",  # Metastatic carcinoma to head of pancreas (disorder)
        "1237474005",  # Metastatic carcinoma to bone of upper limb (disorder)
        "1237476007",  # Metastatic carcinoma to bone of lower limb (disorder)
        "1237477003",  # Metastatic carcinoma to skin of head (disorder)
        "1237478008",  # Metastatic carcinoma to skin of neck (disorder)
        "1237479000",  # Metastatic carcinoma to anal canal (disorder)
        "1237480002",  # Metastatic carcinoma to sigmoid colon (disorder)
        "1237481003",  # Metastatic carcinoma to cervix (disorder)
        "1237482005",  # Metastatic papillary adenocarcinoma to thyroid (disorder)
        "1237483000",  # Metastatic carcinoma to tongue (disorder)
        "1237484006",  # Metastatic carcinoma to colon (disorder)
        "1237485007",  # Metastatic carcinoma to rectosigmoid junction (disorder)
        "1237487004",  # Metastatic carcinoma to base of tongue (disorder)
        "1237488009",  # Metastatic carcinoma to palate (disorder)
        "1237489001",  # Metastatic carcinoma to tonsil (disorder)
        "1237490005",  # Metastatic carcinoma to genital organ (disorder)
        "1237492002",  # Metastatic carcinoma to endocervix (disorder)
        "1237493007",  # Metastatic carcinoma to exocervix (disorder)
        "1237494001",  # Metastatic carcinoma to extrahepatic bile duct (disorder)
        "1237495000",  # Metastatic carcinoma to glottis (disorder)
        "1237496004",  # Metastatic carcinoma to subglottis (disorder)
        "1237497008",  # Metastatic carcinoma to supraglottis (disorder)
        "1237498003",  # Metastatic carcinoma to penis (disorder)
        "1237516000",  # Metastatic carcinoma to lower lobe of lung (disorder)
        "1237517009",  # Metastatic carcinoma to pelvic bones, sacrum and coccyx (disorder)
        "1237518004",  # Metastatic carcinoma to main bronchus (disorder)
        "1237519007",  # Metastatic carcinoma to skin of lower limb (disorder)
        "1237520001",  # Metastatic carcinoma to skin of trunk (disorder)
        "1237521002",  # Metastatic carcinoma to skin of upper limb (disorder)
        "1237522009",  # Metastatic carcinoma to gallbladder (disorder)
        "1237523004",  # Metastatic carcinoma to vocal cord (disorder)
        "1237524005",  # Metastatic carcinoma to pancreas (disorder)
        "1237525006",  # Metastatic carcinoma to stomach (disorder)
        "1237526007",  # Metastatic carcinoma to bone of rib (disorder)
        "1237527003",  # Metastatic carcinoma to bone of sternum (disorder)
        "1237528008",  # Metastatic carcinoma to bone of clavicle (disorder)
        "1237529000",  # Metastatic carcinoma to ampulla of Vater (disorder)
        "1237530005",  # Metastatic adenocarcinoma to carotid body (disorder)
        "1237531009",  # Metastatic adenocarcinoma to parathyroid (disorder)
        "1237621006",  # Metastatic adenocarcinoma to Moll's gland (disorder)
        "1240346000",  # Metastatic adenocarcinoma to Meckel diverticulum (disorder)
        "1240351006",  # Metastatic carcinoma to lower gum (disorder)
        "1240352004",  # Metastatic carcinoma to upper gum (disorder)
        "1240353009",  # Metastatic squamous cell carcinoma to prostate (disorder)
        "1240355002",  # Metastatic squamous cell carcinoma to epididymis and spermatic cord (disorder)
        "1240359008",  # Metastatic small cell carcinoma to prostate (disorder)
        "1240362006",  # Metastatic small cell carcinoma (disorder)
        "1240363001",  # Metastatic small cell carcinoma to ovary (disorder)
        "1240370001",  # Metastatic squamous cell carcinoma to liver and intrahepatic biliary tract (disorder)
        "1240372009",  # Metastatic carcinoma to meibomian gland (disorder)
        "1251445000",  # Metastatic neuroblastoma to abdominothoracic structure (disorder)
        "1251454002",  # Metastatic neuroblastoma to adrenal gland (disorder)
        "1251456000",  # Metastatic neuroblastoma to brain (disorder)
        "1251457009",  # Metastatic neuroblastoma to central nervous system (disorder)
        "1251458004",  # Metastatic neuroblastoma to thorax (disorder)
        "1251459007",  # Metastatic neuroblastoma to paraspinal area (disorder)
        "1251460002",  # Metastatic neuroblastoma to pelvis (disorder)
        "1251465007",  # Metastatic extraskeletal osteosarcoma (disorder)
        "1251466008",  # Metastatic osteosarcoma to articular cartilage of clavicle (disorder)
        "1251467004",  # Metastatic osteosarcoma to articular cartilage of rib (disorder)
        "1251468009",  # Metastatic osteosarcoma to articular cartilage of sternum (disorder)
        "1251469001",  # Metastatic osteosarcoma to articular cartilage of vertebra (disorder)
        "1251470000",  # Metastatic osteosarcoma to bone (disorder)
        "1251471001",  # Metastatic osteosarcoma to bone of clavicle (disorder)
        "1251472008",  # Metastatic osteosarcoma to bone of head (disorder)
        "1251473003",  # Metastatic osteosarcoma to bone of rib (disorder)
        "1251474009",  # Metastatic osteosarcoma to bone of vertebra (disorder)
        "1251475005",  # Metastatic osteosarcoma to bone of sternum (disorder)
        "1251482009",  # Metastatic adenocarcinoma to pancreatic duct (disorder)
        "1251483004",  # Metastatic adenocarcinoma to paraganglia (disorder)
        "1251484005",  # Metastatic adenocarcinoma to pituitary gland (disorder)
        "1251485006",  # Metastatic adenocarcinoma to prostate (disorder)
        "1251486007",  # Metastatic carcinoma to liver (disorder)
        "1251487003",  # Metastatic carcinoma to biliary tract (disorder)
        "1251489000",  # Metastatic carcinoma to frenum linguae (disorder)
        "1251490009",  # Metastatic carcinoma to gallbladder and extrahepatic biliary tract (disorder)
        "1251491008",  # Metastatic carcinoma to larynx (disorder)
        "1251492001",  # Metastatic carcinoma to lip (disorder)
        "1251493006",  # Metastatic carcinoma to lip, oral cavity and/or pharynx (disorder)
        "1251494000",  # Metastatic carcinoma to retromolar area (disorder)
        "1251495004",  # Metastatic carcinoma to scapula (disorder)
        "1251496003",  # Metastatic carcinoma to uvula (disorder)
        "1251497007",  # Metastatic carcinoma to vermilion border of lower lip (disorder)
        "1251498002",  # Metastatic sarcoma to retroperitoneum (disorder)
        "1254689008",  # Metastatic adenoid cystic carcinoma to cervix uteri (disorder)
        "1254691000",  # Metastatic adenoid cystic carcinoma to left lacrimal gland (disorder)
        "1254723004",  # Metastatic adenoid cystic carcinoma to right lacrimal gland (disorder)
        "1254724005",  # Metastatic adenoid cystic carcinoma to oropharynx (disorder)
        "1254725006",  # Metastatic adenoid cystic carcinoma to parotid gland (disorder)
        "1254726007",  # Metastatic adenoid cystic carcinoma to salivary gland (disorder)
        "1254727003",  # Metastatic adenoid cystic carcinoma to submandibular gland (disorder)
        "1254728008",  # Metastatic adenoid cystic carcinoma to trachea (disorder)
        "1254732002",  # Metastatic large cell carcinoma to lung (disorder)
        "1254733007",  # Metastatic large cell carcinoma to right lung (disorder)
        "1254734001",  # Metastatic large cell carcinoma to left lung (disorder)
        "1254735000",  # Metastatic large cell carcinoma to bilateral lungs (disorder)
        "1254740008",  # Metastatic giant cell carcinoma to lung (disorder)
        "1254745003",  # Metastatic giant cell carcinoma to left lung (disorder)
        "1254746002",  # Metastatic giant cell carcinoma to right lung (disorder)
        "1254747006",  # Metastatic giant cell carcinoma to bilateral lungs (disorder)
        "1254748001",  # Metastatic clear cell squamous cell carcinoma to skin (disorder)
        "1254749009",  # Metastatic acantholytic squamous cell carcinoma (disorder)
        "1254750009",  # Metastatic acantholytic squamous cell carcinoma to skin (disorder)
        "1254752001",  # Metastatic verrucous carcinoma to oral cavity (disorder)
        "1254753006",  # Metastatic verrucous carcinoma to skin of sole of foot (disorder)
        "1254755004",  # Metastatic basosquamous cell carcinoma to skin (disorder)
        "1255358009",  # Oligometastatic malignant neoplasm (disorder)
        "1259083006",  # Disorder of autonomic nervous system due to multiple sclerosis (disorder)
        "1259121008",  # Amyotrophic lateral sclerosis, parkinsonism, dementia complex of West New Guinea (disorder)
        "1259122001",  # Amyotrophic lateral sclerosis with parkinsonism (disorder)
        "1259123006",  # Amyotrophic lateral sclerosis, parkinsonism, dementia complex of Kii Peninsula (disorder)
        "1259124000",  # Amyotrophic lateral sclerosis with frontotemporal dementia (disorder)
        "1259125004",  # Amyotrophic lateral sclerosis with multiple system atrophy (disorder)
        "1259126003",  # Amyotrophic lateral sclerosis with autonomic dysfunction (disorder)
        "1259127007",  # Amyotrophic lateral sclerosis with cerebellar dysfunction (disorder)
        "1259128002",  # Alzheimer disease with psychosis (disorder)
        "1259129005",  # Amyotrophic lateral sclerosis with spinocerebellar ataxia (disorder)
        "1259309001",  # Primary carcinoma of body of pancreas (disorder)
        "1259310006",  # Primary carcinoma of tail of pancreas (disorder)
        "1259311005",  # Primary cystadenocarcinoma of pancreas (disorder)
        "1259358002",  # Primary carcinoma of endocrine pancreas (disorder)
        "1259415000",  # Primary carcinoma of head of pancreas (disorder)
        "1259425005",  # Primary neuroblastoma of brain (disorder)
        "1259465009",  # Dementia due to hepatic failure (disorder)
        "1259467001",  # Dementia due to hypercalcemia (disorder)
        "1259469003",  # Dementia due to Gerstmann Straussler Scheinker syndrome (disorder)
        "1259471003",  # Dementia due to Hashimoto encephalopathy (disorder)
        "1259473000",  # Dementia due to fragile X syndrome (disorder)
        "1259476008",  # Dementia due to genetic disease (disorder)
        "1259478009",  # Dementia due to familial Creutzfeldt-Jakob disease (disorder)
        "1259480003",  # Dementia due to fatal familial insomnia (disorder)
        "1259485008",  # Dementia due to cerebral vasculitis (disorder)
        "1259488005",  # Dementia due to cerebral amyloid angiopathy (disorder)
        "1259492003",  # Dementia due to metastatic malignant neoplasm to brain (disorder)
        "1259494002",  # Dementia due to leukodystrophy (disorder)
        "1259496000",  # Dementia due to Lyme disease (disorder)
        "1259499007",  # Dementia due to hemorrhagic cerebral infarction due to hypertension (disorder)
        "1259501004",  # Dementia due to kuru (disorder)
        "1259503001",  # Dementia due to iatrogenic Creutzfeldt-Jakob disease (disorder)
        "1259511006",  # Dementia due to Wilson disease (disorder)
        "1259513009",  # Dementia due to Whipple disease (disorder)
        "1259517005",  # Dementia due to systemic lupus erythematosus (disorder)
        "1259519008",  # Dementia due to subacute sclerosing panencephalitis (disorder)
        "1259522005",  # Dementia due to variant Creutzfeldt-Jakob disease (disorder)
        "1259524006",  # Dementia due to trypanosomiasis (disorder)
        "1259529001",  # Dementia due to sporadic Creutzfeldt-Jakob disease (disorder)
        "1259531005",  # Dementia due to hypertensive encephalopathy (disorder)
        "1259539007",  # Primary carcinoma of pancreas (disorder)
        "1259579003",  # Dementia due to Behcet syndrome (disorder)
        "1259581001",  # Dementia due to celiac disease (disorder)
        "1259584009",  # Dementia due to and following dialysis (disorder)
        "1259586006",  # Dementia due to autoimmune encephalitis (disorder)
        "1259591007",  # Dementia due to acquired hypothyroidism (disorder)
        "1259629009",  # Primary adenocarcinoma of pituitary gland (disorder)
        "1259656006",  # Dementia due to renal failure (disorder)
        "1259661008",  # Dementia due to inflammatory disorder of musculoskeletal system (disorder)
        "1259663006",  # Dementia due to polyarteritis nodosa (disorder)
        "1259665004",  # Dementia due to progressive subcortical gliosis (disorder)
        "1259667007",  # Dementia due to paraneoplastic encephalitis (disorder)
        "1259675001",  # Dementia due to obstructive hydrocephalus (disorder)
        "1259677009",  # Dementia due to multiple system atrophy (disorder)
        "1259679007",  # Dementia due to atypical pantothenate kinase associated neurodegeneration (disorder)
        "1259682002",  # Primary infiltrating duct carcinoma of pancreas (disorder)
        "1259692005",  # Primary gliosarcoma of brain (disorder)
        "1259700000",  # Primary mucinous cystic neoplasm with invasive carcinoma of pancreas (disorder)
        "1259720004",  # Primary malignant glioma of brain (disorder)
        "1259722007",  # Primary glioblastoma multiforme of brain (disorder)
        "1259724008",  # Primary anaplastic astrocytoma of brain (disorder)
        "1259725009",  # Primary medulloblastoma of cerebellum (disorder)
        "1259730008",  # Primary oligodendroglioma of brain (disorder)
        "1259739009",  # Primary malignant glioma of hypothalamus (disorder)
        "1259740006",  # Primary malignant glioma of cerebellum (disorder)
        "1259742003",  # Primary malignant glioma of cerebrum (disorder)
        "1259743008",  # Primary astrocytoma of brain stem (disorder)
        "1259747009",  # Primary malignant insulinoma (disorder)
        "1259750007",  # Primary astrocytoma of cerebrum (disorder)
        "1259764007",  # Primary supratentorial primitive neuroectodermal tumor (disorder)
        "1259765008",  # Primary classic medulloblastoma (disorder)
        "1259769002",  # Primary malignant glioma of brainstem (disorder)
        "1259770001",  # Primary anaplastic glioma of brain (disorder)
        "1259798003",  # Primary oligodendroglioma of cerebrum (disorder)
        "1259799006",  # Primary malignant glucagonoma of pancreas (disorder)
        "1259800005",  # Primary malignant gastrinoma of pancreas (disorder)
        "1259812009",  # Primary astroblastoma of brain (disorder)
        "1259814005",  # Primary gemistocytic astrocytoma of brain (disorder)
        "1259815006",  # Primary giant cell glioblastoma of brain (disorder)
        "1259816007",  # Primary fibrillary astrocytoma of brain (disorder)
        "1259817003",  # Primary pleomorphic xanthoastrocytoma of brain (disorder)
        "1259818008",  # Primary pilomyxoid astrocytoma of brain (disorder)
        "1259990004",  # Dementia due to classical pantothenate kinase associated neurodegeneration (disorder)
        "1263484004",  # Metastatic endometrioid carcinoma to ovary (disorder)
        "1263555006",  # Logopenic non-amnestic Alzheimer disease (disorder)
        "1263585001",  # Frontal variant non-amnestic Alzheimer disease (disorder)
        "1263601000",  # Metastatic scirrhous carcinoma to breast (disorder)
        "1263639008",  # Metastatic transitional cell carcinoma to urinary bladder (disorder)
        "1263640005",  # Metastatic transitional cell carcinoma to corpus uteri (disorder)
        "1263641009",  # Metastatic transitional cell carcinoma to kidney (disorder)
        "1263642002",  # Metastatic transitional cell carcinoma to left kidney (disorder)
        "1263643007",  # Metastatic transitional cell carcinoma to right kidney (disorder)
        "1263651005",  # Metastatic mucoepidermoid carcinoma to skin (disorder)
        "1263652003",  # Metastatic mucoepidermoid carcinoma to parotid gland (disorder)
        "1263653008",  # Metastatic mucoepidermoid carcinoma to salivary gland (disorder)
        "1263654002",  # Metastatic mucoepidermoid carcinoma to submandibular gland (disorder)
        "1263655001",  # Metastatic transitional cell carcinoma to right renal pelvis (disorder)
        "1263656000",  # Metastatic transitional cell carcinoma to left renal pelvis (disorder)
        "1263657009",  # Metastatic transitional cell carcinoma to ureter (disorder)
        "1263658004",  # Metastatic transitional cell carcinoma to left ureter (disorder)
        "1263659007",  # Metastatic transitional cell carcinoma to renal pelvis (disorder)
        "1263660002",  # Metastatic transitional cell carcinoma to upper urinary tract (disorder)
        "1263786003",  # Metastatic basaloid squamous cell carcinoma to nasopharynx (disorder)
        "1263911005",  # Metastatic carcinoma ex pleomorphic adenoma to parotid gland (disorder)
        "1263912003",  # Metastatic carcinoma ex pleomorphic adenoma to lacrimal gland (disorder)
        "1263913008",  # Metastatic chromophobe renal cell carcinoma (disorder)
        "1263914002",  # Metastatic papillary renal cell carcinoma (disorder)
        "1263999008",  # Metastatic MiT family translocation renal cell carcinoma (disorder)
        "1264003007",  # Acute right ventricular failure following incision of heart (disorder)
        "1264066001",  # Metastatic keratinizing squamous cell carcinoma to nasopharynx (disorder)
        "1264067005",  # Metastatic acinar cell carcinoma to pancreas (disorder)
        "1264068000",  # Metastatic lymphoepithelial carcinoma (disorder)
        "1264115008",  # Metastatic metaplastic carcinoma to breast (disorder)
        "1264116009",  # Metastatic adenosquamous carcinoma (disorder)
        "1264117000",  # Metastatic adenosquamous carcinoma to cervix (disorder)
        "1264118005",  # Metastatic oxyphilic adenocarcinoma (disorder)
        "1264258003",  # Metastatic papillary squamous cell carcinoma (disorder)
        "1264357006",  # Metastatic mucinous cystadenocarcinoma to ovary (disorder)
        "1264358001",  # Metastatic mucinous cystadenocarcinoma to pancreas (disorder)
        "1264364008",  # Metastatic ceruminous adenocarcinoma to skin (disorder)
        "1264367001",  # Metastatic sebaceous adenocarcinoma (disorder)
        "1264369003",  # Metastatic sebaceous adenocarcinoma to eyelid (disorder)
        "1264495000",  # Metastatic carcinoma to breast (disorder)
        "1264497008",  # Metastatic infiltrating duct carcinoma to breast (disorder)
        "1264499006",  # Metastatic infiltrating duct carcinoma to pancreas (disorder)
        "1264500002",  # Metastatic infiltrating duct carcinoma to prostate (disorder)
        "1264501003",  # Metastatic papillary cystadenocarcinoma to kidney (disorder)
        "1264502005",  # Metastatic mucinous adenocarcinoma to gastrointestinal tract (disorder)
        "1268352008",  # Primary carcinoma of pituitary gland (disorder)
        "1268353003",  # Metastatic clear cell adenocarcinoma to ovary (disorder)
        "1268354009",  # Metastatic clear cell adenocarcinoma to peritoneum (disorder)
        "1268386004",  # Metastatic undifferentiated carcinoma to corpus uteri (disorder)
        "1268388003",  # Metastatic undifferentiated carcinoma to esophagus (disorder)
        "1268390002",  # Metastatic undifferentiated carcinoma to liver and intrahepatic biliary tract (disorder)
        "1268392005",  # Metastatic undifferentiated carcinoma to nasal sinus (disorder)
        "1268393000",  # Metastatic undifferentiated carcinoma to nasopharynx (disorder)
        "1268394006",  # Metastatic undifferentiated carcinoma to ovary (disorder)
        "1268395007",  # Metastatic undifferentiated carcinoma to stomach (disorder)
        "1268532006",  # Primary carcinoma of ampulla of Vater (disorder)
        "1268542008",  # Primary invasive intraductal papillary-mucinous carcinoma of pancreas (disorder)
        "1268546006",  # Primary mucinous cystadenocarcinoma of pancreas (disorder)
        "1268561007",  # Primary serous cystadenocarcinoma of pancreas (disorder)
        "1268563005",  # Primary acinar cell carcinoma of pancreas (disorder)
        "1268698000",  # Primary solid pseudopapillary carcinoma of pancreas (disorder)
        "1268702001",  # Primary choroid plexus carcinoma (disorder)
        "1268716006",  # Primary medulloblastoma (disorder)
        "1268888008",  # Primary pineal parenchymal tumor of intermediate differentiation (disorder)
        "1268911008",  # Primary pancreatoblastoma (disorder)
        "1268957005",  # Primary pineoblastoma (disorder)
        "1268961004",  # Primary diffuse astrocytoma of brain (disorder)
        "1268966009",  # Primary gliomatosis cerebri (disorder)
        "1269042005",  # Primary protoplasmic astrocytoma of brain (disorder)
        "1269047004",  # Primary papillary tumor of pineal region (disorder)
        "1269048009",  # Primary diffuse intrinsic pontine glioma (disorder)
        "1269051002",  # Primary desmoplastic nodular medulloblastoma of brain (disorder)
        "1269052009",  # Primary large cell medulloblastoma of brain (disorder)
        "1269121005",  # Metastatic rhabdomyosarcoma (disorder)
        "1269122003",  # Metastatic carcinoma to nasopharynx (disorder)
        "1269123008",  # Metastatic squamous cell carcinoma to rectum (disorder)
        "1269125001",  # Metastatic squamous cell carcinoma to small intestine (disorder)
        "1269126000",  # Metastatic squamous cell carcinoma to stomach (disorder)
        "1279841001",  # Undifferentiated carcinoma with osteoclast-like giant cells of pancreas (disorder)
        "1279882001",  # Metastatic embryonal rhabdomyosarcoma (disorder)
        "127991000119101",  # Hypertension concurrent and due to end stage renal disease on dialysis due to type 2 diabetes mellitus (disorder)
        "128001000119105",  # Hypertension concurrent and due to end stage renal disease on dialysis due to type 1 diabetes mellitus (disorder)
        "128462008",  # Metastatic malignant neoplasm (disorder)
        "128465005",  # Metastatic malignant neoplasm to articular cartilage (disorder)
        "1286863002",  # Metastatic malignant neoplasm to cecum and/or colon and/or rectum (disorder)
        "1286896000",  # Metastatic collecting duct renal cell carcinoma (disorder)
        "1286898004",  # Metastatic cystic renal cell carcinoma (disorder)
        "1287657002",  # Primary well-differentiated neuroendocrine tumor of ampulla of Vater (disorder)
        "1287665004",  # Primary well-differentiated neuroendocrine tumor of pancreas (disorder)
        "1290060007",  # Metastatic well-differentiated neuroendocrine tumor (disorder)
        "1290066001",  # Well-differentiated neuroendocrine tumor of ampulla of Vater (disorder)
        "1290080008",  # Neuroendocrine neoplasm of pancreas (disorder)
        "1290115008",  # Growth hormone releasing factor-secreting tumor of pancreas (disorder)
        "1290116009",  # Gastric inhibitory peptide-secreting tumor of pancreas (disorder)
        "1290117000",  # Parathyroid hormone-related peptide-secreting tumor of pancreas (disorder)
        "1290690001",  # Metastatic cholangiocarcinoma (disorder)
        "1290695006",  # Metastatic apocrine adenocarcinoma (disorder)
        "1290697003",  # Metastatic eccrine porocarcinoma (disorder)
        "1290698008",  # Metastatic pilomatrix carcinoma (disorder)
        "1290700004",  # Metastatic lobular carcinoma (disorder)
        "1296659009",  # Acute exacerbation of chronic heart failure (disorder)
        "1296851009",  # Metastatic malignant melanoma of unknown primary (disorder)
        "130121000119104",  # Dementia due to Rett syndrome (disorder)
        "1306321008",  # Metastatic malignant neoplasm to nipple (disorder)
        "1306322001",  # Metastatic malignant neoplasm to ectopic site of breast (disorder)
        "1306323006",  # Metastatic malignant neoplasm to areola (disorder)
        "1306543004",  # Metastatic malignant neoplasm to central portion of breast (disorder)
        "1306544005",  # Metastatic malignant neoplasm to lower inner quadrant of breast (disorder)
        "1306545006",  # Metastatic malignant neoplasm to lower outer quadrant of breast (disorder)
        "1306546007",  # Metastatic malignant neoplasm to upper inner quadrant of breast (disorder)
        "1306547003",  # Metastatic malignant neoplasm to upper outer quadrant of breast (disorder)
        "13274008",  # Atrophic fibrosis of lung (disorder)
        "1332483008",  # Metastatic balloon cell malignant melanoma (disorder)
        "1332484002",  # Metastatic anaplastic thyroid carcinoma (disorder)
        "1332485001",  # Metastatic endometrial stromal sarcoma (disorder)
        "1332486000",  # Metastatic amelanotic malignant melanoma to skin (disorder)
        "1335987009",  # Metastatic desmoplastic malignant melanoma (disorder)
        "1336170004",  # Metastatic spindle cell rhabdomyosarcoma (disorder)
        "1336171000",  # Metastatic synovial sarcoma (disorder)
        "1336172007",  # Metastatic ganglioneuroblastoma (disorder)
        "1336173002",  # Metastatic superficial spreading melanoma (disorder)
        "1336176005",  # Metastatic fibrosarcoma (disorder)
        "1336177001",  # Metastatic spindle cell melanoma (disorder)
        "1336182008",  # Metastatic myxofibrosarcoma (disorder)
        "1336185005",  # Metastatic pleomorphic rhabdomyosarcoma (disorder)
        "1336186006",  # Metastatic lentigo maligna melanoma (disorder)
        "1336188007",  # Metastatic glassy cell carcinoma (disorder)
        "1336190008",  # Metastatic inflammatory carcinoma (disorder)
        "1339031006",  # Dementia caused by ionizing radiation (disorder)
        "1339049006",  # Metastatic infantile fibrosarcoma (disorder)
        "1339050006",  # Metastatic malignant glioma (disorder)
        "1339051005",  # Metastatic germinoma (disorder)
        "135091000119106",  # Secondary neuroendocrine carcinoma of peritoneum (disorder)
        "13839000",  # Bernheim's syndrome (disorder)
        "14070001",  # Multi-infarct dementia with depression (disorder)
        "141991000119109",  # Delusions in Alzheimer's disease (disorder)
        "142001000119106",  # Depressed mood in Alzheimer's disease (disorder)
        "142011000119109",  # Alzheimer's disease co-occurrent with delirium (disorder)
        "143401000119106",  # Malignant carcinoid tumor of pancreas (disorder)
        "145501000119108",  # Metastatic malignant neoplasm to breast (disorder)
        "145511000119106",  # Metastatic malignant neoplasm to genital organ (disorder)
        "14700006",  # Bauxite fibrosis of lung (disorder)
        "14914851000119109",  # Metastatic Merkel cell carcinoma to skin (disorder)
        "14914931000119107",  # Metastatic Merkel cell carcinoma to liver (disorder)
        "153891000119101",  # End stage renal disease on dialysis due to hypertension (disorder)
        "153931000119109",  # Acute combined systolic and diastolic heart failure (disorder)
        "153941000119100",  # Chronic combined systolic and diastolic heart failure (disorder)
        "153951000119103",  # Acute on chronic combined systolic and diastolic heart failure (disorder)
        "15629541000119106",  # Congestive heart failure stage C due to ischemic cardiomyopathy (disorder)
        "15629591000119103",  # Congestive heart failure stage B due to ischemic cardiomyopathy (disorder)
        "15629641000119107",  # Systolic heart failure stage B due to ischemic cardiomyopathy (disorder)
        "15629741000119102",  # Systolic heart failure stage C due to ischemic cardiomyopathy (disorder)
        "15650751000119107",  # Primary sarcoma of pancreas (disorder)
        "15662003",  # Senile dementia (disorder)
        "15781000119107",  # Hypertensive heart AND chronic kidney disease with congestive heart failure (disorder)
        "1581000119101",  # Dementia of the Alzheimer type with behavioral disturbance (disorder)
        "1591000119103",  # Dementia with behavioral disturbance (disorder)
        "15930821000119105",  # Metastatic malignant neoplasm to bilateral ovaries (disorder)
        "15956181000119102",  # Metastatic adenocarcinoma to bilateral lungs (disorder)
        "15956221000119105",  # Metastatic adenocarcinoma to right lung (disorder)
        "15956261000119100",  # Metastatic adenocarcinoma to left lung (disorder)
        "15956661000119102",  # Metastatic small cell carcinoma to right lung (disorder)
        "15956701000119109",  # Metastatic small cell carcinoma to left lung (disorder)
        "15956741000119106",  # Metastatic small cell carcinoma to bilateral lungs (disorder)
        "15957141000119109",  # Metastatic squamous cell carcinoma to bilateral lungs (disorder)
        "15957181000119104",  # Metastatic squamous cell carcinoma to right lung (disorder)
        "15957221000119107",  # Metastatic squamous cell carcinoma to left lung (disorder)
        "15958181000119103",  # Metastatic adenocarcinoma to bilateral kidneys (disorder)
        "15958221000119106",  # Metastatic adenocarcinoma to left kidney (disorder)
        "15958261000119101",  # Metastatic adenocarcinoma to right kidney (disorder)
        "15958301000119109",  # Metastatic squamous cell carcinoma to right kidney (disorder)
        "15958381000119101",  # Metastatic squamous cell carcinoma to left kidney (disorder)
        "15958661000119103",  # Metastatic malignant melanoma to left kidney (disorder)
        "15958701000119105",  # Metastatic malignant melanoma to bilateral kidneys (disorder)
        "15958741000119107",  # Metastatic malignant melanoma to right kidney (disorder)
        "15964701000119109",  # Acute cor pulmonale co-occurrent and due to saddle embolus of pulmonary artery (disorder)
        "15977551000119100",  # Metastatic malignant neoplasm to bilateral eyes (disorder)
        "15977591000119105",  # Metastatic malignant neoplasm to right eye (disorder)
        "15977631000119105",  # Metastatic malignant neoplasm to left eye (disorder)
        "15999000",  # Mixed micro and macronodular cirrhosis (disorder)
        "16003001",  # Giant bullous emphysema (disorder)
        "16055271000119107",  # Respiratory failure following trauma (disorder)
        "16070004",  # Syphilitic cirrhosis (disorder)
        "16219201000119101",  # Behavioral disturbance co-occurrent and due to late onset Alzheimer dementia (disorder)
        "16253001",  # Dilated peripartum cardiomyopathy (disorder)
        "16276361000119109",  # Vascular dementia without behavioral disturbance (disorder)
        "1651000119109",  # Primary adenocarcinoma of pancreas (disorder)
        "1671000119100",  # Metastatic squamous cell carcinoma to lymph node (disorder)
        "1681000119102",  # Metastatic adenocarcinoma to lymph node (disorder)
        "16823941000119108",  # Primary malignant gastrointestinal stromal neoplasm of pancreas (disorder)
        "16838951000119100",  # Acute on chronic right-sided congestive heart failure (disorder)
        "16845301000119108",  # Primary glioblastoma multiforme of cerebellum (disorder)
        "16845341000119105",  # Primary glioblastoma multiforme of brainstem (disorder)
        "16846004",  # Obstructive emphysema (disorder)
        "17385007",  # Graphite fibrosis of lung (disorder)
        "1761006",  # Biliary cirrhosis (disorder)
        "187786003",  # Malignant neoplasm of sphincter of Oddi (disorder)
        "187791002",  # Malignant tumor of body of pancreas (disorder)
        "187792009",  # Malignant tumor of tail of pancreas (disorder)
        "187793004",  # Malignant tumor of pancreatic duct (disorder)
        "187794005",  # Malignant tumor of Islets of Langerhans (disorder)
        "187798008",  # Malignant neoplasm of ectopic pancreatic tissue (disorder)
        "188280007",  # Malignant neoplasm of cerebrum (excluding lobes and ventricles) (disorder)
        "188281006",  # Malignant neoplasm of basal ganglia (disorder)
        "188282004",  # Malignant neoplasm of cerebral cortex (disorder)
        "188283009",  # Malignant neoplasm of corpus striatum (disorder)
        "188285002",  # Malignant neoplasm of globus pallidus (disorder)
        "188286001",  # Malignant tumor of hypothalamus (disorder)
        "188287005",  # Malignant neoplasm of thalamus (disorder)
        "188289008",  # Malignant neoplasm of hippocampus (disorder)
        "188290004",  # Malignant neoplasm of uncus (disorder)
        "188292007",  # Malignant tumor of choroid plexus (disorder)
        "188293002",  # Malignant neoplasm of floor of cerebral ventricle (disorder)
        "188295009",  # Malignant neoplasm of cerebral peduncle (disorder)
        "188296005",  # Malignant neoplasm of medulla oblongata (disorder)
        "188297001",  # Malignant neoplasm of midbrain (disorder)
        "188298006",  # Malignant neoplasm of pons (disorder)
        "188301005",  # Malignant neoplasm of corpus callosum (disorder)
        "188302003",  # Malignant neoplasm of tapetum (disorder)
        "188308004",  # Malignant neoplasm of olfactory bulb (disorder)
        "188339002",  # Malignant neoplasm of pituitary gland and craniopharyngeal duct (disorder)
        "188340000",  # Malignant tumor of craniopharyngeal duct (disorder)
        "188445006",  # Metastatic malignant neoplasm to retroperitoneum and peritoneum (disorder)
        "188454009",  # Metastatic malignant neoplasm to skin of head (disorder)
        "188458007",  # Metastatic malignant neoplasm to skin of shoulder and arm (disorder)
        "188462001",  # Metastatic malignant neoplasm to brain and spinal cord (disorder)
        "188469005",  # Metastatic malignant neoplasm to cervix uteri (disorder)
        "188471005",  # Metastatic malignant neoplasm to epididymis and vas deferens (disorder)
        "188645002",  # Leukemic reticuloendotheliosis of lymph nodes of head, face and neck (disorder)
        "188648000",  # Leukemic reticuloendotheliosis of lymph nodes of axilla and upper limb (disorder)
        "188649008",  # Leukemic reticuloendotheliosis of lymph nodes of inguinal region and lower limb (disorder)
        "189814006",  # Pancreatoblastoma (disorder)
        "19090001000004101",  # Metastatic neoplasm of left basal ganglion (disorder)
        "191449005",  # Uncomplicated senile dementia (disorder)
        "191451009",  # Uncomplicated presenile dementia (disorder)
        "191452002",  # Presenile dementia with delirium (disorder)
        "191454001",  # Presenile dementia with paranoia (disorder)
        "191455000",  # Presenile dementia with depression (disorder)
        "191457008",  # Senile dementia with depressive or paranoid features (disorder)
        "191458003",  # Senile dementia with paranoia (disorder)
        "191459006",  # Senile dementia with depression (disorder)
        "191461002",  # Senile dementia with delirium (disorder)
        "191463004",  # Uncomplicated arteriosclerotic dementia (disorder)
        "191464005",  # Arteriosclerotic dementia with delirium (disorder)
        "191465006",  # Arteriosclerotic dementia with paranoia (disorder)
        "191466007",  # Arteriosclerotic dementia with depression (disorder)
        "191471000",  # Korsakov's alcoholic psychosis with peripheral neuritis (disorder)
        "191519005",  # Dementia associated with another disease (disorder)
        "192926004",  # Multiple sclerosis of the brainstem (disorder)
        "192927008",  # Multiple sclerosis of the spinal cord (disorder)
        "192929006",  # Exacerbation of multiple sclerosis (disorder)
        "194767001",  # Benign hypertensive heart disease with congestive cardiac failure (disorder)
        "194779001",  # Hypertensive heart and renal disease with (congestive) heart failure (disorder)
        "194781004",  # Hypertensive heart and renal disease with both (congestive) heart failure and renal failure (disorder)
        "195111005",  # Decompensated cardiac failure (disorder)
        "195112003",  # Compensated cardiac failure (disorder)
        "195114002",  # Acute left ventricular failure (disorder)
        "195957006",  # Chronic bullous emphysema (disorder)
        "195958001",  # Segmental bullous emphysema (disorder)
        "195959009",  # Zonal bullous emphysema (disorder)
        "196026004",  # Chronic emphysema caused by chemical fumes (disorder)
        "196028003",  # Chronic pulmonary fibrosis caused by chemical fumes (disorder)
        "196125002",  # Diffuse interstitial pulmonary fibrosis (disorder)
        "197291001",  # Unilobular portal cirrhosis (disorder)
        "197293003",  # Diffuse nodular cirrhosis (disorder)
        "197294009",  # Fatty portal cirrhosis (disorder)
        "197299004",  # Pigmentary portal cirrhosis (disorder)
        "197301006",  # Toxic portal cirrhosis (disorder)
        "197303009",  # Bacterial portal cirrhosis (disorder)
        "197305002",  # Syphilitic portal cirrhosis (disorder)
        "19943007",  # Cirrhosis of liver (disorder)
        "20091000175107",  # Acute on chronic hypoxemic and hypercapnic respiratory failure (disorder)
        "206586007",  # Congenital cardiac failure (disorder)
        "21861000",  # Micronodular cirrhosis (disorder)
        "22381000119105",  # Primary degenerative dementia (disorder)
        "230156002",  # Malignant meningitis (disorder)
        "230258005",  # Amyotrophic lateral sclerosis with dementia (disorder)
        "230265002",  # Familial Alzheimer's disease of early onset (disorder)
        "230266001",  # Non-familial Alzheimer's disease of early onset (disorder)
        "230267005",  # Familial Alzheimer's disease of late onset (disorder)
        "230268000",  # Non-familial Alzheimer's disease of late onset (disorder)
        "230269008",  # Focal Alzheimer's disease (disorder)
        "230271008",  # Pick's disease with Pick bodies (disorder)
        "230272001",  # Pick's disease with Pick cells and no Pick bodies (disorder)
        "230280008",  # Progressive aphasia in Alzheimer's disease (disorder)
        "230282000",  # Post-traumatic dementia (disorder)
        "230285003",  # Vascular dementia of acute onset (disorder)
        "230286002",  # Subcortical vascular dementia (disorder)
        "230287006",  # Mixed cortical and subcortical vascular dementia (disorder)
        "230288001",  # Semantic dementia (disorder)
        "230289009",  # Patchy dementia (disorder)
        "230299004",  # Juvenile onset Huntington's disease (disorder)
        "230300007",  # Late onset Huntington's disease (disorder)
        "230301006",  # Akinetic-rigid form of Huntington's disease (disorder)
        "230372003",  # Acute relapsing multiple sclerosis (disorder)
        "230499002",  # Sleep-related respiratory failure (disorder)
        "23341000119109",  # Congestive heart failure with right heart failure (disorder)
        "233674008",  # Pulmonary emphysema in alpha-1 primary immunodeficiency deficiency (disorder)
        "233675009",  # Toxic emphysema (disorder)
        "233677001",  # Scar emphysema (disorder)
        "233713004",  # Seasonal cryptogenic organizing pneumonia with biochemical cholestasis (disorder)
        "233726000",  # Localized pulmonary fibrosis (disorder)
        "233737004",  # Familial fibrous mediastinitis (disorder)
        "233758005",  # Mica pneumoconiosis (disorder)
        "233761006",  # Subacute silicosis (disorder)
        "233924009",  # Heart failure as a complication of care (disorder)
        "233940007",  # Pulmonary tumor embolism (disorder)
        "235875008",  # Alcoholic hepatitis (disorder)
        "235880004",  # Alcoholic fibrosis and sclerosis of liver (disorder)
        "235881000",  # Alcoholic hepatic failure (disorder)
        "235895002",  # Laennec's cirrhosis, non-alcoholic (disorder)
        "235896001",  # Infectious cirrhosis (disorder)
        "235897005",  # Hypoxia-associated cirrhosis (disorder)
        "235899008",  # Hepatic sclerosis (disorder)
        "235901004",  # Hepatic fibrosis with hepatic sclerosis (disorder)
        "235966007",  # Cystadenocarcinoma of pancreas (disorder)
        "236433006",  # Acute-on-chronic renal failure (disorder)
        "236434000",  # End stage renal failure untreated by renal replacement therapy (disorder)
        "236435004",  # End stage renal failure on dialysis (disorder)
        "236436003",  # End stage renal failure with renal transplant (disorder)
        "236512004",  # Leukemic infiltrate of kidney (disorder)
        "236513009",  # Lymphoma of kidney (disorder)
        "23958009",  # Vanishing lung (disorder)
        "241861008",  # Metastatic malignant neoplasm to nasopharynx (disorder)
        "2421000119107",  # Hallucinations co-occurrent and due to late onset dementia (disorder)
        "242862004",  # Metastatic malignant neoplasm to nasopharyngeal wall (disorder)
        "24700007",  # Multiple sclerosis (disorder)
        "24841007",  # Cardiorespiratory failure during AND/OR resulting from a procedure (disorder)
        "253005002",  # Vasoactive intestinal peptide-secreting tumor (disorder)
        "254609000",  # Carcinoma of ampulla of Vater (disorder)
        "254611009",  # Malignant tumor of endocrine pancreas (disorder)
        "254612002",  # Carcinoma of endocrine pancreas (disorder)
        "254938000",  # Astrocytoma of brain (disorder)
        "254940005",  # Oligodendroglioma of brain (disorder)
        "254955001",  # Pituitary carcinoma (disorder)
        "254969001",  # Malignant tumor of olfactory tract (disorder)
        "255039001",  # Pancreatic polypeptidoma (disorder)
        "255088001",  # Malignant tumor of exocrine pancreas (disorder)
        "255112006",  # Malignant tumor of pituitary and hypothalamus (disorder)
        "255118005",  # Metastatic lymphangitic carcinoma (disorder)
        "255119002",  # Lymphangitis carcinomatosa (disorder)
        "255121007",  # Carcinomatosis of peritoneal cavity (disorder)
        "255123005",  # Metastasis to nervous system and eye (disorder)
        "255124004",  # Metastasis to peripheral nerve (disorder)
        "25544003",  # Low output heart failure (disorder)
        "25772007",  # Multi-infarct dementia with delusions (disorder)
        "266355005",  # Bullous emphysema with collapse (disorder)
        "266356006",  # Atrophic (senile) emphysema (disorder)
        "266368002",  # Post-inflammatory pulmonary fibrosis (disorder)
        "266468003",  # Cirrhosis - non-alcoholic (disorder)
        "266469006",  # Multilobular portal cirrhosis (disorder)
        "266470007",  # Cardiac portal cirrhosis (disorder)
        "266471006",  # Juvenile portal cirrhosis (disorder)
        "26852004",  # Primary degenerative dementia of the Alzheimer type, senile onset, with depression (disorder)
        "268612007",  # Senile and presenile organic psychotic conditions (disorder)
        "26929004",  # Alzheimer's disease (disorder)
        "269473008",  # Metastatic malignant neoplasm to respiratory and digestive systems (disorder)
        "271440004",  # Cirrhosis secondary to cholestasis (disorder)
        "27156006",  # Posthepatitic cirrhosis (disorder)
        "274088005",  # Metastatic malignant neoplasm to unknown site (disorder)
        "275266006",  # Metastasis to digestive organs (disorder)
        "276259003",  # Respiratory arrest preceding cardiac arrest (disorder)
        "276826005",  # Malignant glioma of brain (disorder)
        "276828006",  # Glioblastoma multiforme of brain (disorder)
        "276836002",  # Primary cerebral lymphoma (disorder)
        "277461004",  # Anaplastic astrocytoma of brain (disorder)
        "277505007",  # Medulloblastoma of cerebellum (disorder)
        "278051002",  # Malignant lymphoma of thyroid gland (disorder)
        "278433008",  # Malignant infiltration of soft tissue (disorder)
        "278857002",  # Dementia of frontal lobe type (disorder)
        "281004",  # Dementia associated with alcoholism (disorder)
        "281560004",  # Neuroblastoma of brain (disorder)
        "285211000119102",  # Congestive heart failure as post-operative complication of cardiac surgery (disorder)
        "285221000119109",  # Congestive heart failure as post-operative complication of non-cardiac surgery (disorder)
        "285841000119104",  # Malignant hypertensive end stage renal disease (disorder)
        "286371000119107",  # Malignant hypertensive end stage renal disease on dialysis (disorder)
        "286902000",  # Secondary carcinoma of gastrointestinal tract (disorder)
        "288631000119104",  # Vascular dementia with behavioral disturbance (disorder)
        "2912004",  # Cystic-bullous disease of the lung (disorder)
        "293671000119109",  # Behavioral disturbance due to multi-infarct dementia (disorder)
        "30042003",  # Confluent fibrosis of lung (disorder)
        "303194003",  # Metastasis to head and neck lymph node (disorder)
        "303201005",  # Metastatic malignant neoplasm to multiple lymph nodes (disorder)
        "304603007",  # Variant Creutzfeldt-Jakob disease (disorder)
        "307226002",  # Metastatic adenocarcinoma of unknown origin (disorder)
        "307593001",  # Carcinomatosis (disorder)
        "307757001",  # Chronic alcoholic hepatitis (disorder)
        "31081000119101",  # Presenile dementia with delusions (disorder)
        "312991009",  # Senile dementia of the Lewy body type (disorder)
        "314206003",  # Refractory heart failure (disorder)
        "314408000",  # Leukemic infiltrate of choroid (disorder)
        "314418005",  # Leukemic infiltrate of retina (disorder)
        "314964006",  # Local recurrence of malignant tumor of pancreas (disorder)
        "31712002",  # Primary biliary cholangitis (disorder)
        "31898008",  # Paraseptal emphysema (disorder)
        "326072005",  # Carcinoma of head of pancreas (disorder)
        "33144001",  # Parasitic cirrhosis (disorder)
        "33325001",  # Compensatory emphysema (disorder)
        "350586651000119105",  # Chronic respiratory failure due to neuromuscular disease (disorder)
        "3514002",  # Peribronchial fibrosis of lung (disorder)
        "352701000119102",  # Primary malignant neoplasm of neck of pancreas (disorder)
        "353561000119103",  # Metastatic malignant neoplasm to right lung (disorder)
        "353571000119109",  # Metastatic malignant neoplasm to right adrenal gland (disorder)
        "353721000119100",  # Metastatic malignant neoplasm to left adrenal gland (disorder)
        "353741000119106",  # Metastatic malignant neoplasm to left lung (disorder)
        "354081000119101",  # Metastatic adenocarcinoma (disorder)
        "359617009",  # Acute right-sided heart failure (disorder)
        "359780007",  # Metastatic malignant neoplasm to lateral axillary lymph nodes (disorder)
        "359782004",  # Metastatic malignant neoplasm to apex of urinary bladder (disorder)
        "359785002",  # Metastatic malignant neoplasm to dome of urinary bladder (disorder)
        "359987004",  # Krukenberg tumor (disorder)
        "361194002",  # Perinatal pulmonary fibroplasia (disorder)
        "361195001",  # Pulmonary fibroplasia (disorder)
        "361196000",  # Idiopathic hilar fibrosis (disorder)
        "363368005",  # Carcinoma of body of pancreas (disorder)
        "363369002",  # Carcinoma of tail of pancreas (disorder)
        "363417006",  # Malignant tumor of ampulla of Vater (disorder)
        "363418001",  # Malignant tumor of pancreas (disorder)
        "363419009",  # Malignant tumor of head of pancreas (disorder)
        "363467004",  # Malignant neoplasm of frontal lobe (disorder)
        "363468009",  # Malignant neoplasm of temporal lobe (disorder)
        "363469001",  # Malignant neoplasm of parietal lobe (disorder)
        "363470000",  # Malignant neoplasm of occipital lobe (disorder)
        "363471001",  # Malignant neoplasm of cerebral ventricles (disorder)
        "363473003",  # Malignant neoplasm of brainstem (disorder)
        "363482009",  # Malignant tumor of pituitary gland (disorder)
        "363483004",  # Malignant tumor of pineal gland (disorder)
        "364006",  # Acute left-sided heart failure (disorder)
        "36599006",  # Chronic fibrosis of lung (disorder)
        "367363000",  # Right ventricular failure (disorder)
        "368471000119109",  # End stage renal disease on dialysis due to drug induced diabetes mellitus (disorder)
        "369488002",  # Metastatic malignant neoplasm to seminal vesicle (disorder)
        "369514009",  # Metastatic malignant neoplasm to left fallopian tube (disorder)
        "369521009",  # Metastatic malignant neoplasm to right fallopian tube (disorder)
        "369523007",  # Metastatic malignant neoplasm to left ovary (disorder)
        "369530001",  # Metastatic malignant neoplasm to right ovary (disorder)
        "369535006",  # Metastatic neoplasm of left broad ligament (disorder)
        "369536007",  # Metastatic neoplasm of right broad ligament (disorder)
        "371024007",  # Senile dementia with delusion (disorder)
        "371026009",  # Senile dementia with psychosis (disorder)
        "371139006",  # Early cirrhosis (disorder)
        "371967001",  # Primary malignant neoplasm of ampulla of Vater (disorder)
        "372003004",  # Primary malignant neoplasm of pancreas (disorder)
        "372093008",  # Metastatic malignant neoplasm to axillary tail of breast (disorder)
        "372119009",  # Primary malignant neoplasm of head of pancreas (disorder)
        "372142002",  # Carcinoma of pancreas (disorder)
        "37688005",  # Clonorchiasis with biliary cirrhosis (disorder)
        "39871006",  # Chronic respiratory failure (disorder)
        "399969009",  # Metastatic malignant neoplasm to blood vessel of upper arm (disorder)
        "400058002",  # Metastatic malignant neoplasm to blood vessel of lower leg (disorder)
        "402537005",  # Metastatic basal cell carcinoma (disorder)
        "402563000",  # Metastatic malignant melanoma with diffuse hypermelanosis (disorder)
        "402879006",  # T-cell leukemic infiltration of skin (disorder)
        "40312006",  # Pericarditis co-occurrent and due to tumor metastatic to pericardium (disorder)
        "403906006",  # Metastatic squamous cell carcinoma (disorder)
        "404090003",  # Malignant infiltration of oral cavity by underlying tumor (disorder)
        "404091004",  # Malignant infiltration of skin by underlying tumor (disorder)
        "404092006",  # Metastatic carcinoma to skin (disorder)
        "404093001",  # Sarcomatous metastasis in skin (disorder)
        "404094007",  # Metastasis involving oral cavity (disorder)
        "404122003",  # Leukemic infiltration of skin (chronic T-cell lymphocytic leukemia) (disorder)
        "404123008",  # Leukemic infiltration of skin (T-cell prolymphocytic leukemia) (disorder)
        "404124002",  # Leukemic infiltration of skin (T-cell lymphoblastic leukemia) (disorder)
        "404139001",  # Leukemic infiltration of skin in hairy-cell leukemia (disorder)
        "404154007",  # Leukemic infiltration of skin in monocytic leukemia (disorder)
        "404156009",  # Leukemic infiltration of skin (disorder)
        "404157000",  # Specific skin infiltration in Hodgkin's disease (disorder)
        "405570007",  # Pulmonary fibrosis due to and following radiotherapy (disorder)
        "405843009",  # Widespread metastatic malignant neoplastic disease (disorder)
        "40640008",  # Massive fibrosis of lung co-occurrent and due to silicosis (disorder)
        "409622000",  # Respiratory failure (disorder)
        "410430005",  # Cardiorespiratory arrest (disorder)
        "410431009",  # Cardiorespiratory failure (disorder)
        "41309000",  # Alcoholic liver damage (disorder)
        "414676007",  # Metastatic neuroblastoma of orbit proper (disorder)
        "416780008",  # Primary degenerative dementia of the Alzheimer type, presenile onset (disorder)
        "416975007",  # Primary degenerative dementia of the Alzheimer type, senile onset (disorder)
        "417996009",  # Systolic heart failure (disorder)
        "418304008",  # Diastolic heart failure (disorder)
        "418529003",  # Metastatic malignant neoplasm to lacrimal drainage structure (disorder)
        "419728003",  # Portal cirrhosis (disorder)
        "420054005",  # Alcoholic cirrhosis (disorder)
        "420614009",  # Organic dementia with acquired immunodeficiency syndrome (disorder)
        "421023003",  # Presenile dementia with acquired immunodeficiency syndrome (disorder)
        "421283008",  # Primary lymphoma of brain with acquired immunodeficiency syndrome (disorder)
        "421529006",  # Dementia with acquired immunodeficiency syndrome (disorder)
        "423032007",  # Leukemic infiltration of orbit (disorder)
        "423384009",  # Metastatic malignant neoplasm to lacrimal gland duct (disorder)
        "42343007",  # Congestive heart failure (disorder)
        "423595004",  # Adenocarcinoma carcinomatosis (disorder)
        "424052001",  # Small cell carcinoma carcinomatosis (disorder)
        "424151006",  # Anaplastic glioma of brain (disorder)
        "424276002",  # Malignant glioma of brainstem (disorder)
        "424404003",  # Decompensated chronic heart failure (disorder)
        "424954002",  # Undifferentiated large cell carcinomatosis (disorder)
        "425303004",  # Squamous cell carcinomatosis (disorder)
        "425390006",  # Dementia associated with Parkinson's Disease (disorder)
        "425413006",  # Drug-induced cirrhosis of liver (disorder)
        "425500002",  # Secondary progressive multiple sclerosis (disorder)
        "426263006",  # Congestive heart failure due to left ventricular systolic dysfunction (disorder)
        "426373005",  # Relapsing remitting multiple sclerosis (disorder)
        "426437004",  # Familial idiopathic pulmonary fibrosis (disorder)
        "426611007",  # Congestive heart failure due to valvular disease (disorder)
        "426896000",  # Chronic hypercapnic respiratory failure (disorder)
        "428051000124108",  # Mild dementia (disorder)
        "428061005",  # Malignant neoplasm of brain (disorder)
        "428173007",  # Chronic hypoxemic respiratory failure (disorder)
        "428351000124105",  # Severe dementia (disorder)
        "428700003",  # Primary progressive multiple sclerosis (disorder)
        "429033009",  # Malignant neoplasm of cerebrum (disorder)
        "429161000124103",  # Primary degenerative dementia of the Alzheimer type, senile onset, with behavioral disturbance (disorder)
        "429458009",  # Dementia due to Creutzfeldt Jakob disease (disorder)
        "429998004",  # Vascular dementia (disorder)
        "430771000124100",  # Moderate dementia (disorder)
        "433146000",  # Chronic kidney disease stage 5 (disorder)
        "43736008",  # Rheumatic left ventricular failure (disorder)
        "43904005",  # Macronodular cirrhosis (disorder)
        "439567002",  # Malignant multiple sclerosis (disorder)
        "44047000",  # Zieve's syndrome (disorder)
        "441481004",  # Chronic systolic heart failure (disorder)
        "441530006",  # Chronic diastolic heart failure (disorder)
        "442344002",  # Dementia due to Huntington chorea (disorder)
        "44313006",  # Right heart failure secondary to left heart failure (disorder)
        "443144000",  # Metastatic sarcoma (disorder)
        "443253003",  # Acute on chronic systolic heart failure (disorder)
        "443254009",  # Acute systolic heart failure (disorder)
        "443333004",  # Medulloblastoma (disorder)
        "443343001",  # Acute diastolic heart failure (disorder)
        "443344007",  # Acute on chronic diastolic heart failure (disorder)
        "443493003",  # Metastatic malignant melanoma (disorder)
        "446221000",  # Heart failure with normal ejection fraction (disorder)
        "448218008",  # Malignant neoplasm of cerebellopontine angle (disorder)
        "448248006",  # Malignant neoplasm of axial suprasellar region of brain (disorder)
        "448250003",  # Malignant teratoma of pineal region (disorder)
        "448863000",  # Malignant epithelial neoplasm of pineal gland (disorder)
        "448989001",  # Malignant epithelial neoplasm of brain (disorder)
        "449253005",  # Malignant epithelial neoplasm of hypothalamus (disorder)
        "449420002",  # Malignant neoplasm of cerebellum (disorder)
        "449630001",  # Metastatic malignant neoplasm to skin of lower leg (disorder)
        "449631002",  # Metastatic malignant neoplasm to skin of upper arm (disorder)
        "449632009",  # Metastatic malignant neoplasm to lower leg (disorder)
        "449633004",  # Metastatic malignant neoplasm to upper arm (disorder)
        "45256007",  # Cruveilhier-Baumgarten syndrome (disorder)
        "457721000124104",  # Metastatic non-small cell lung cancer (disorder)
        "458321000124102",  # Metastatic urothelial carcinoma (disorder)
        "458581000124106",  # Metastatic HER2 positive neoplasm of gastroesophageal junction (disorder)
        "459371000124108",  # Metastasis from gastroesophageal adenocarcinoma (disorder)
        "459381000124106",  # Metastasis from hormone-refractory prostate cancer (disorder)
        "459391000124109",  # Metastasis from human epidermal growth factor 2 positive carcinoma of breast (disorder)
        "459401000124106",  # Metastasis from pancreatic endocrine carcinoma (disorder)
        "459411000124109",  # Metastasis from collecting duct carcinoma of nipple (disorder)
        "459421000124101",  # Metastasis from malignant tumor of penis (disorder)
        "460561000124109",  # Progressive fibrosing interstitial lung disease (disorder)
        "46113002",  # Hypertensive heart failure (disorder)
        "46177005",  # End-stage renal disease (disorder)
        "471880001",  # Heart failure due to end stage congenital heart disease (disorder)
        "473419009",  # Intraductal papillary mucinous carcinoma in situ of pancreas (disorder)
        "4817008",  # Primary degenerative dementia of the Alzheimer type, senile onset, with delirium (disorder)
        "48447003",  # Chronic heart failure (disorder)
        "49049000",  # Parkinson's disease (disorder)
        "49584005",  # Acute cor pulmonale (disorder)
        "4981000",  # Panacinar emphysema (disorder)
        "50196008",  # Perialveolar fibrosis of lung (disorder)
        "50325005",  # Alcoholic fatty liver (disorder)
        "5148006",  # Hypertensive heart disease with congestive heart failure (disorder)
        "51615001",  # Fibrosis of lung (disorder)
        "51928006",  # General paresis - neurosyphilis (disorder)
        "52448006",  # Dementia (disorder)
        "536002",  # Glissonian cirrhosis (disorder)
        "5375005",  # Chronic left-sided congestive heart failure (disorder)
        "54502004",  # Primary degenerative dementia of the Alzheimer type, presenile onset, with delusions (disorder)
        "54867000",  # Rheumatoid arthritis with fibrosing alveolitis (disorder)
        "55009008",  # Primary degenerative dementia of the Alzheimer type, senile onset, with delusions (disorder)
        "56267009",  # Multi-infarct dementia (disorder)
        "56675007",  # Acute heart failure (disorder)
        "56841008",  # Massive fibrosis of lung (disorder)
        "57686001",  # Emphysematous bleb of lung (disorder)
        "58756001",  # Huntington's chorea (disorder)
        "59773008",  # Simple pulmonary alveolitis (disorder)
        "62377009",  # Postpartum cardiomyopathy (disorder)
        "6475002",  # Primary degenerative dementia of the Alzheimer type, presenile onset, uncomplicated (disorder)
        "65096006",  # Primary degenerative dementia of the Alzheimer type, presenile onset, with delirium (disorder)
        "66108005",  # Primary degenerative dementia of the Alzheimer type, senile onset, uncomplicated (disorder)
        "66989003",  # Chronic right-sided congestive heart failure (disorder)
        "67431000119105",  # Congestive heart failure stage D (disorder)
        "67441000119101",  # Congestive heart failure stage C (disorder)
        "67905004",  # Acute-on-chronic respiratory failure (disorder)
        "681621000119105",  # Primary adenocarcinoma of body of pancreas (disorder)
        "681721000119103",  # Primary adenocarcinoma of head of pancreas (disorder)
        "681831000119107",  # Primary adenocarcinoma of neck of pancreas (disorder)
        "681911000119108",  # Primary adenocarcinoma of pancreatic duct (disorder)
        "681971000119100",  # Primary adenocarcinoma of tail of pancreas (disorder)
        "682221000119103",  # Primary anaplastic astrocytoma of cerebrum (disorder)
        "682231000119100",  # Primary anaplastic astrocytoma of frontal lobe (disorder)
        "682241000119109",  # Primary anaplastic astrocytoma of occipital lobe (disorder)
        "682251000119106",  # Primary anaplastic astrocytoma of parietal lobe (disorder)
        "682261000119108",  # Primary anaplastic astrocytoma of temporal lobe (disorder)
        "682441000119105",  # Primary astrocytoma of cerebral ventricle (disorder)
        "682451000119107",  # Primary astrocytoma of frontal lobe (disorder)
        "682461000119109",  # Primary astrocytoma of occipital lobe (disorder)
        "682471000119103",  # Primary astrocytoma of parietal lobe (disorder)
        "682481000119100",  # Primary astrocytoma of temporal lobe (disorder)
        "68328006",  # Centriacinar emphysema (disorder)
        "683761000119107",  # Primary ependymoma of brain ventricle (disorder)
        "683771000119101",  # Primary ependymoma of frontal lobe (disorder)
        "683781000119103",  # Primary malignant ependymoma of occipital lobe (disorder)
        "683791000119100",  # Primary ependymoma of parietal lobe (disorder)
        "683801000119104",  # Primary ependymoma of temporal lobe (disorder)
        "684891000119108",  # Primary glioblastoma multiforme of cerebral ventricle (disorder)
        "684901000119107",  # Primary glioblastoma multiforme of cerebrum (disorder)
        "684911000119105",  # Primary glioblastoma multiforme of frontal lobe (disorder)
        "684921000119103",  # Primary glioblastoma multiforme of occipital lobe (disorder)
        "684931000119100",  # Primary glioblastoma multiforme of parietal lobe (disorder)
        "684941000119109",  # Primary glioblastoma multiforme of temporal lobe (disorder)
        "686421000119103",  # Primary malignant glioma of brain ventricle (disorder)
        "686431000119100",  # Primary malignant glioma of frontal lobe (disorder)
        "686441000119109",  # Primary malignant glioma of occipital lobe (disorder)
        "686451000119106",  # Primary malignant glioma of parietal lobe (disorder)
        "686461000119108",  # Primary malignant glioma of temporal lobe (disorder)
        "687331000119109",  # Primary oligodendroglioma of frontal lobe (disorder)
        "687341000119100",  # Primary oligodendroglioma of occipital lobe (disorder)
        "687351000119103",  # Primary oligodendroglioma of parietal lobe (disorder)
        "687361000119101",  # Primary oligodendroglioma of temporal lobe (disorder)
        "698296002",  # Acute exacerbation of chronic congestive heart failure (disorder)
        "698504006",  # Postoperative cardiopulmonary failure (disorder)
        "698594003",  # Symptomatic congestive heart failure (disorder)
        "698624003",  # Dementia associated with cerebral lipidosis (disorder)
        "698625002",  # Dementia associated with normal pressure hydrocephalus (disorder)
        "698626001",  # Dementia associated with multiple sclerosis (disorder)
        "698725008",  # Dementia associated with neurosyphilis (disorder)
        "698726009",  # Dementia associated with viral encephalitis (disorder)
        "698781002",  # Dementia associated with cerebral anoxia (disorder)
        "699189004",  # North American Indian childhood cirrhosis (disorder)
        "699318007",  # Supratentorial primitive neuroectodermal tumor (disorder)
        "699704002",  # Classic medulloblastoma (disorder)
        "700250006",  # Idiopathic pulmonary fibrosis (disorder)
        "700423003",  # Adenocarcinoma of pancreas (disorder)
        "702392008",  # Metastatic renal cell carcinoma (disorder)
        "703272007",  # Heart failure with reduced ejection fraction (disorder)
        "703273002",  # Heart failure with reduced ejection fraction due to coronary artery disease (disorder)
        "703274008",  # Heart failure with reduced ejection fraction due to myocarditis (disorder)
        "703275009",  # Heart failure with reduced ejection fraction due to cardiomyopathy (disorder)
        "703276005",  # Heart failure with reduced ejection fraction due to heart valve disease (disorder)
        "704152002",  # Metastatic neuroblastoma (disorder)
        "704667004",  # Hypertension concurrent and due to end stage renal disease on dialysis (disorder)
        "707324008",  # Anemia co-occurrent and due to end stage renal disease (disorder)
        "707434003",  # Pulmonary fibrosis due to Hermansky-Pudlak syndrome (disorder)
        "708030004",  # Pulmonary emphysema co-occurrent with fibrosis of lung (disorder)
        "709109004",  # Hypercapnic respiratory failure (disorder)
        "709285002",  # Metastatic malignant neoplasm to lumbosacral plexus (disorder)
        "70936005",  # Multi-infarct dementia, uncomplicated (disorder)
        "71193007",  # Fibrosis of lung caused by radiation (disorder)
        "712487000",  # End stage renal disease due to benign hypertension (disorder)
        "713060000",  # Sporadic Creutzfeldt-Jakob disease (disorder)
        "713181003",  # Chronic alcoholic liver disease (disorder)
        "713189001",  # Malignant insulinoma (disorder)
        "713325002",  # Primary cerebral lymphoma co-occurrent with human immunodeficiency virus infection (disorder)
        "713488003",  # Presenile dementia co-occurrent with human immunodeficiency virus infection (disorder)
        "713844000",  # Dementia co-occurrent with human immunodeficiency virus infection (disorder)
        "715401008",  # Primary biliary cirrhosis co-occurrent with systemic scleroderma (disorder)
        "715414009",  # Familial malignant neoplasm of pancreas (disorder)
        "715662009",  # Iatrogenic Jakob-Creutzfeldt disease (disorder)
        "715807002",  # Familial Creutzfeldt-Jakob (disorder)
        "715904005",  # Pineal parenchymal tumor of intermediate differentiation (disorder)
        "716107009",  # Early onset parkinsonism and intellectual disability syndrome (disorder)
        "716203000",  # Decompensated cirrhosis of liver (disorder)
        "716662004",  # Autosomal dominant late onset Parkinson disease (disorder)
        "717840005",  # Congestive heart failure stage B (disorder)
        "717919005",  # Neuroendocrine tumor of pancreas (disorder)
        "718685006",  # Orthostatic hypotension co-occurrent and due to Parkinson's disease (disorder)
        "71892000",   # Cardiac asthma (disorder)
        "719717006",  # Psychosis co-occurrent and due to Parkinson's disease (disorder)
        "721710005",  # Fibrosis of liver caused by alcohol (disorder)
        "721977007",  # Lung fibrosis, immunodeficiency, 46,XX gonadal dysgenesis syndrome (disorder)
        "722600006",  # Non-amnestic Alzheimer disease (disorder)
        "722671009",  # Metastatic malignant neoplasm of meninges (disorder)
        "722707001",  # Metastatic malignant neoplasm of peripheral nervous system (disorder)
        "722977005",  # Dementia co-occurrent and due to neurocysticercosis (disorder)
        "722979008",  # Dementia due to metabolic abnormality (disorder)
        "722980006",  # Dementia due to chromosomal anomaly (disorder)
        "722987009",  # Amyotrophic lateral sclerosis plus syndrome (disorder)
        "723123001",  # Ischemic vascular dementia (disorder)
        "723390000",  # Rapidly progressive dementia (disorder)
        "723829000",  # Pulmonary fibrosis, hepatic hyperplasia, bone marrow hypoplasia syndrome (disorder)
        "724003651000119103",  # Primary malignant glioma of thalamus (disorder)
        "724761004",  # Sporadic Parkinson disease (disorder)
        "724776007",  # Dementia due to disorder of central nervous system (disorder)
        "724778008",  # Progressive relapsing multiple sclerosis (disorder)
        "72481000119103",  # Congestive heart failure as early postoperative complication (disorder)
        "725416005",  # Cardiomyopathy due to cirrhosis of liver (disorder)
        "725898002",  # Delirium co-occurrent with dementia (disorder)
        "725938001",  # Cirrhosis of liver caused by methotrexate (disorder)
        "725939009",  # Cirrhosis of liver caused by amiodarone (disorder)
        "725940006",  # Cirrhosis of liver caused by methyldopa (disorder)
        "733028000",  # Multiple sclerosis, ichthyosis, factor VIII deficiency syndrome (disorder)
        "733184002",  # Dementia caused by heavy metal exposure (disorder)
        "733190003",  # Dementia due to primary malignant neoplasm of brain (disorder)
        "733191004",  # Dementia due to chronic subdural hematoma (disorder)
        "733192006",  # Dementia due to herpes encephalitis (disorder)
        "733193001",  # Dementia co-occurrent and due to progressive multifocal leukoencephalopathy (disorder)
        "733194007",  # Dementia co-occurrent and due to Down syndrome (disorder)
        "735386008",  # Acute hypoxemic respiratory failure co-occurrent with acute hypercapnic respiratory failure (disorder)
        "735733008",  # Cirrhosis of liver co-occurrent and due to primary sclerosing cholangitis (disorder)
        "735735001",  # Primary malignant neuroendocrine neoplasm of pancreas (disorder)
        "735757008",  # Primary ganglioneuroblastoma of brain (disorder)
        "7361000175106",  # Acute-on-chronic hypercapnic respiratory failure (disorder)
        "7371000175103",  # Reduced ejection fraction co-occurrent and due to acute on chronic heart failure (disorder)
        "7381000175100",  # Reduced ejection fraction co-occurrent and due to chronic heart failure (disorder)
        "7391000175102",  # Reduced ejection fraction co-occurrent and due to acute heart failure (disorder)
        "7401000175100",  # Acute on chronic heart failure co-occurrent with normal ejection fraction (disorder)
        "7411000175102",  # Chronic heart failure co-occurrent with normal ejection fraction (disorder)
        "7421000175106",  # Acute heart failure co-occurrent with normal ejection fraction (disorder)
        "74669004",  # Cardiac cirrhosis (disorder)
        "74960003",  # Acute left-sided congestive heart failure (disorder)
        "762350007",  # Dementia due to prion disease (disorder)
        "762351006",  # Dementia due to and following injury of head (disorder)
        "762457009",  # Astroblastoma of brain (disorder)
        "762707000",  # Subcortical dementia (disorder)
        "76301009",  # Florid cirrhosis (disorder)
        "766246000",  # Marburg acute multiple sclerosis (disorder)
        "767444009",  # Germline BRCA-mutated human epidermal growth factor receptor 2 negative metastatic carcinoma of breast (disorder)
        "767448007",  # Pineoblastoma (disorder)
        "770602005",  # Squamous cell carcinoma of exocrine pancreas (disorder)
        "770727008",  # Spinal muscular atrophy with respiratory distress type 2 (disorder)
        "771306007",  # Hereditary fibrosing poikiloderma, tendon contractures, myopathy, pulmonary fibrosis syndrome (disorder)
        "774069007",  # Protein kinase cAMP-dependent type I regulatory subunit beta-related neurodegenerative dementia with intermediate filaments (disorder)
        "776416004",  # Hyperuricemia, pulmonary hypertension, renal failure, alkalosis syndrome (disorder)
        "77690003",  # Interstitial emphysema of lung (disorder)
        "780821007",  # Invasive intraductal papillary-mucinous carcinoma of pancreas (disorder)
        "782697005",  # Solid pseudopapillary carcinoma of pancreas (disorder)
        "783161005",  # Familial dementia British type (disorder)
        "783258000",  # Familial dementia Danish type (disorder)
        "783706007",  # Serous cystadenocarcinoma of pancreas (disorder)
        "783771003",  # Acinar cell carcinoma of pancreas (disorder)
        "784341001",  # Amyotrophic lateral sclerosis type 4 (disorder)
        "78461000119105",  # Metastatic squamous cell carcinoma to skin (disorder)
        "785879009",  # Mucinous cystadenocarcinoma of pancreas (disorder)
        "78862003",  # Ayerza's syndrome (disorder)
        "788863007",  # Amnestic disorder caused by psychoactive substance (disorder)
        "788864001",  # Amnestic disorder caused by volatile solvent (disorder)
        "788898005",  # Dementia caused by volatile inhalant (disorder)
        "788899002",  # Dementia due to pellagra (disorder)
        "788950000",  # Heart failure with mid range ejection fraction (disorder)
        "789170003",  # Disinhibited behavior due to dementia (disorder)
        "789574002",  # Acute exacerbation of idiopathic pulmonary fibrosis (disorder)
        "792004",  # Jakob-Creutzfeldt disease (disorder)
        "792907004",  # Adenocarcinoma of pancreatic duct (disorder)
        "79341000119107",  # Mixed dementia (disorder)
        "79955004",  # Chronic cor pulmonale (disorder)
        "80479009",  # Acute right-sided congestive heart failure (disorder)
        "80614003",  # Prolonged pulmonary alveolitis (disorder)
        "816205008",  # Malignant pituitary blastoma (disorder)
        "816984002",  # Progressive multiple sclerosis (disorder)
        "82351000119105",  # Altered behavior due to Pick's disease (disorder)
        "82361000119107",  # Altered behavior in dementia due to Huntington chorea (disorder)
        "82371000119101",  # Dementia due to multiple sclerosis with altered behavior (disorder)
        "82381000119103",  # Epileptic dementia with behavioral disturbance (disorder)
        "82523003",  # Congestive rheumatic heart failure (disorder)
        "827186009",  # Metastatic malignant neoplasm to skin of hip and skin of lower leg (disorder)
        "82959004",  # Dementia paralytica juvenilis (disorder)
        "831000119103",  # Cirrhosis of liver due to chronic hepatitis C (disorder)
        "83105008",  # Malignant hypertensive heart disease with congestive heart failure (disorder)
        "83291003",  # Cor pulmonale (disorder)
        "833326008",  # Cortical vascular dementia (disorder)
        "836274002",  # Carcinomatosis of peritoneum (disorder)
        "836477007",  # Chronic emphysema caused by vapor (disorder)
        "836486002",  # Lymphomatous infiltrate of kidney (disorder)
        "838276009",  # Amyotrophic lateral sclerosis, parkinsonism, dementia complex (disorder)
        "840452004",  # Classical sporadic Creutzfeldt-Jakob disease (disorder)
        "840464007",  # Dementia due to carbon monoxide poisoning (disorder)
        "840465008",  # Dementia due to iron deficiency (disorder)
        "84114007",  # Heart failure (disorder)
        "846637007",  # Chronic pulmonary fibrosis caused by chemical vapors (disorder)
        "85232009",  # Left heart failure (disorder)
        "86044005",  # Amyotrophic lateral sclerosis (disorder)
        "860826006",  # Creutzfeldt-Jakob Disease caused by human growth hormone (disorder)
        "86454000",  # Postnecrotic cirrhosis (disorder)
        "866048009",  # Mucinous cystic neoplasm with invasive carcinoma of pancreas (disorder)
        "866052009",  # Mixed ductal-neuroendocrine carcinoma of pancreas (disorder)
        "86680006",  # Ruptured emphysematous bleb of lung (disorder)
        "87091000119101",  # Malignant glioma of cerebrum (disorder)
        "87111000119109",  # Malignant glioma of hypothalamus (disorder)
        "87121000119102",  # Malignant glioma of cerebellum (disorder)
        "871619002",  # Cirrhosis of liver due to and following cardiac procedure (disorder)
        "87317003",  # Respiratory arrest (disorder)
        "87433001",  # Pulmonary emphysema (disorder)
        "88805009",  # Chronic congestive heart failure (disorder)
        "89580002",  # Cryptogenic cirrhosis (disorder)
        "898208007",  # Heart failure due to thyrotoxicosis (disorder)
        "90099008",  # Subcortical leukoencephalopathy (disorder)
        "90117007",  # Tuberculous fibrosis of lung (disorder)
        "90610005",  # Interstitial pulmonary fibrosis of prematurity (disorder)
        "90688005",  # Chronic renal failure syndrome (disorder)
        "90771000119100",  # End stage renal disease on dialysis due to type 1 diabetes mellitus (disorder)
        "90791000119104",  # End stage renal disease on dialysis due to type 2 diabetes mellitus (disorder)
        "90811000119100",  # Low grade malignant glioma of brain (disorder)
        "90831000119105",  # Grade 4 malignant glioma of brain (disorder)
        "91181000119105",  # Squamous cell carcinoma metastatic to pleura (disorder)
        "91191000119108",  # Metastatic adenocarcinoma to pleura (disorder)
        "91211000119109",  # Metastatic small cell carcinoma to pleura (disorder)
        "91221000119102",  # Metastatic malignant melanoma to pleura (disorder)
        "91231000119104",  # Metastatic adenocarcinoma to skin (disorder)
        "91251000119105",  # Small cell carcinoma metastatic to skin (disorder)
        "91271000119101",  # Metastatic squamous cell carcinoma to bone (disorder)
        "91281000119103",  # Metastatic adenocarcinoma to bone (disorder)
        "92506005",  # Biventricular congestive heart failure (disorder)
        "93144003",  # Leukemic reticuloendotheliosis of intra-abdominal lymph nodes (disorder)
        "93145002",  # Leukemic reticuloendotheliosis of intrapelvic lymph nodes (disorder)
        "93146001",  # Leukemic reticuloendotheliosis of intrathoracic lymph nodes (disorder)
        "93150008",  # Leukemic reticuloendotheliosis of lymph nodes of multiple sites (disorder)
        "93715005",  # Primary malignant neoplasm of body of pancreas (disorder)
        "93726004",  # Primary malignant neoplasm of brain stem (disorder)
        "93727008",  # Primary malignant neoplasm of brain (disorder)
        "93746009",  # Primary malignant neoplasm of cerebellum (disorder)
        "93748005",  # Primary malignant neoplasm of cerebral ventricle (disorder)
        "93749002",  # Primary malignant neoplasm of cerebrum (disorder)
        "93768004",  # Primary malignant neoplasm of craniopharyngeal duct (disorder)
        "93807001",  # Primary malignant neoplasm of frontal lobe (disorder)
        "93843007",  # Primary malignant neoplasm of islets of Langerhans (disorder)
        "93928006",  # Primary malignant neoplasm of occipital lobe (disorder)
        "93939009",  # Primary malignant neoplasm of pancreatic duct (disorder)
        "93946000",  # Primary malignant neoplasm of parietal lobe (disorder)
        "94082003",  # Primary malignant neoplasm of tail of pancreas (disorder)
        "94086000",  # Primary malignant neoplasm of temporal lobe (disorder)
        "94152006",  # Metastatic malignant neoplasm to abdominal esophagus (disorder)
        "94153001",  # Metastatic malignant neoplasm to abducens nerve (disorder)
        "94154007",  # Metastatic malignant neoplasm to accessory nerve (disorder)
        "94155008",  # Metastatic malignant neoplasm to accessory sinus (disorder)
        "94156009",  # Metastatic malignant neoplasm of vestibulocochlear nerve (disorder)
        "94157000",  # Metastatic malignant neoplasm to acromion (disorder)
        "94158005",  # Metastatic malignant neoplasm to adenoid (disorder)
        "94159002",  # Metastatic malignant neoplasm to adnexa of skin (disorder)
        "94160007",  # Metastatic malignant neoplasm to adrenal cortex (disorder)
        "94161006",  # Metastatic malignant neoplasm to adrenal gland (disorder)
        "94162004",  # Metastatic malignant neoplasm to adrenal medulla (disorder)
        "94163009",  # Metastatic malignant neoplasm to alveolar ridge mucosa (disorder)
        "94164003",  # Metastatic malignant neoplasm to ampulla of Vater (disorder)
        "94165002",  # Metastatic malignant neoplasm to anal canal (disorder)
        "94166001",  # Metastatic malignant neoplasm to anterior aspect of epiglottis (disorder)
        "94167005",  # Metastatic malignant neoplasm to anterior mediastinum (disorder)
        "94168000",  # Metastatic malignant neoplasm to anterior portion of floor of mouth (disorder)
        "94169008",  # Metastatic malignant neoplasm to anterior two-thirds of tongue (disorder)
        "94170009",  # Metastatic malignant neoplasm to anterior wall of nasopharynx (disorder)
        "94171008",  # Metastatic malignant neoplasm to anterior wall of urinary bladder (disorder)
        "94172001",  # Metastatic malignant neoplasm to anus (disorder)
        "94173006",  # Metastatic malignant neoplasm to aortic body (disorder)
        "94175004",  # Metastatic malignant neoplasm to appendix (disorder)
        "94179005",  # Metastatic malignant neoplasm to ascending colon (disorder)
        "94180008",  # Metastatic malignant neoplasm to axilla (disorder)
        "94181007",  # Metastatic malignant neoplasm to axillary lymph nodes (disorder)
        "94183005",  # Metastatic malignant neoplasm to back (disorder)
        "94184004",  # Metastatic malignant neoplasm to base of tongue (disorder)
        "94185003",  # Metastatic malignant neoplasm to biliary tract (disorder)
        "94186002",  # Metastatic malignant neoplasm to urinary bladder (disorder)
        "94187006",  # Metastatic malignant neoplasm to blood vessel of abdomen (disorder)
        "94189009",  # Metastatic malignant neoplasm to blood vessel of axilla (disorder)
        "94190000",  # Metastatic malignant neoplasm to blood vessel of buttock (disorder)
        "94191001",  # Metastatic malignant neoplasm to blood vessel of face (disorder)
        "94192008",  # Metastatic malignant neoplasm to blood vessel of finger (disorder)
        "94193003",  # Metastatic malignant neoplasm to blood vessel of foot (disorder)
        "94194009",  # Metastatic malignant neoplasm to blood vessel of forearm (disorder)
        "94195005",  # Metastatic malignant neoplasm to blood vessel of hand (disorder)
        "94196006",  # Metastatic malignant neoplasm to blood vessel of head (disorder)
        "94197002",  # Metastatic malignant neoplasm to blood vessel of hip (disorder)
        "94198007",  # Metastatic malignant neoplasm to blood vessel of inguinal region (disorder)
        "94200001",  # Metastatic malignant neoplasm to blood vessel of lower limb (disorder)
        "94201002",  # Metastatic malignant neoplasm to blood vessel of neck (disorder)
        "94202009",  # Metastatic malignant neoplasm to blood vessel of pelvis (disorder)
        "94203004",  # Metastatic malignant neoplasm to blood vessel of perineum (disorder)
        "94204005",  # Metastatic malignant neoplasm to blood vessel of popliteal space (disorder)
        "94205006",  # Metastatic malignant neoplasm to blood vessel of shoulder (disorder)
        "94206007",  # Metastatic malignant neoplasm to blood vessel of thigh (disorder)
        "94207003",  # Metastatic malignant neoplasm to blood vessel of thorax (disorder)
        "94208008",  # Metastatic malignant neoplasm to blood vessel of toe (disorder)
        "94209000",  # Metastatic malignant neoplasm to blood vessel of trunk (disorder)
        "94210005",  # Metastatic malignant neoplasm to blood vessel of upper limb (disorder)
        "94211009",  # Metastatic malignant neoplasm to blood vessel (disorder)
        "94212002",  # Metastatic malignant neoplasm to body of pancreas (disorder)
        "94213007",  # Metastatic malignant neoplasm to body of penis (disorder)
        "94214001",  # Metastatic malignant neoplasm to body of stomach (disorder)
        "94215000",  # Metastatic malignant neoplasm to body of uterus (disorder)
        "94217008",  # Metastatic malignant neoplasm to bone marrow (disorder)
        "94218003",  # Metastatic malignant neoplasm to bone of face (disorder)
        "94219006",  # Metastatic malignant neoplasm to bone of lower limb (disorder)
        "94220000",  # Metastatic malignant neoplasm to bone of skull (disorder)
        "94221001",  # Metastatic malignant neoplasm to bone of upper limb (disorder)
        "94222008",  # Metastatic malignant neoplasm to bone (disorder)
        "94224009",  # Metastatic malignant neoplasm to brain stem (disorder)
        "94225005",  # Metastatic malignant neoplasm to brain (disorder)
        "94226006",  # Metastatic malignant neoplasm to broad ligament (disorder)
        "94227002",  # Metastatic malignant neoplasm to bronchopulmonary lymph nodes (disorder)
        "94228007",  # Metastatic malignant neoplasm to bronchus of left lower lobe (disorder)
        "94229004",  # Metastatic malignant neoplasm to bronchus of left upper lobe (disorder)
        "94230009",  # Metastatic malignant neoplasm to bronchus of right lower lobe (disorder)
        "94231008",  # Metastatic malignant neoplasm to bronchus of middle lobe of right lung (disorder)
        "94232001",  # Metastatic malignant neoplasm to bronchus of right upper lobe (disorder)
        "94233006",  # Metastatic malignant neoplasm to bronchus (disorder)
        "94234000",  # Metastatic malignant neoplasm to buccal mucosa (disorder)
        "94235004",  # Metastatic malignant neoplasm to cecum (disorder)
        "94236003",  # Metastatic malignant neoplasm to calcaneus (disorder)
        "94237007",  # Metastatic malignant neoplasm to cardia of stomach (disorder)
        "94238002",  # Metastatic malignant neoplasm to carina (disorder)
        "94239005",  # Metastatic malignant neoplasm to carotid body (disorder)
        "94240007",  # Metastatic malignant neoplasm to carpal bone (disorder)
        "94241006",  # Metastatic malignant neoplasm to cartilage of nose (disorder)
        "94242004",  # Metastatic malignant neoplasm to cauda equina (disorder)
        "94243009",  # Metastatic malignant neoplasm to central nervous system (disorder)
        "94245002",  # Metastatic malignant neoplasm to cerebellum (disorder)
        "94246001",  # Metastatic malignant neoplasm to cerebral meninges (disorder)
        "94247005",  # Metastatic malignant neoplasm to cerebral ventricle (disorder)
        "94248000",  # Metastatic malignant neoplasm to cerebrum (disorder)
        "94249008",  # Metastatic malignant neoplasm to cervical esophagus (disorder)
        "94250008",  # Metastatic malignant neoplasm to cervical vertebral column (disorder)
        "94252000",  # Metastatic malignant neoplasm to cheek (disorder)
        "94253005",  # Metastatic malignant neoplasm to chest wall (disorder)
        "94254004",  # Metastatic malignant neoplasm to choroid (disorder)
        "94255003",  # Metastatic malignant neoplasm to ciliary body (disorder)
        "94256002",  # Metastatic malignant neoplasm to clavicle (disorder)
        "94257006",  # Metastatic malignant neoplasm to clitoris (disorder)
        "94258001",  # Metastatic malignant neoplasm to coccygeal body (disorder)
        "94259009",  # Metastatic malignant neoplasm to coccyx (disorder)
        "94260004",  # Metastatic malignant neoplasm to colon (disorder)
        "94261000",  # Metastatic malignant neoplasm to commissure of lip (disorder)
        "94262007",  # Metastatic malignant neoplasm to common bile duct (disorder)
        "94263002",  # Metastatic malignant neoplasm to conjunctiva (disorder)
        "94264008",  # Metastatic malignant neoplasm to soft tissues (disorder)
        "94265009",  # Metastatic malignant neoplasm to cornea (disorder)
        "94266005",  # Metastatic malignant neoplasm to cranial nerve (disorder)
        "94267001",  # Metastatic malignant neoplasm to craniopharyngeal duct (disorder)
        "94268006",  # Metastatic malignant neoplasm to cubital lymph nodes (disorder)
        "94269003",  # Metastatic malignant neoplasm to cuboid (disorder)
        "94270002",  # Metastatic malignant neoplasm to cystic duct (disorder)
        "94271003",  # Metastatic malignant neoplasm to descending colon (disorder)
        "94272005",  # Metastatic malignant neoplasm to diaphragm (disorder)
        "94273000",  # Metastatic malignant neoplasm to dorsal surface of tongue (disorder)
        "94274006",  # Metastatic malignant neoplasm to thoracic vertebral column (disorder)
        "94275007",  # Metastatic malignant neoplasm to duodenum (disorder)
        "94278009",  # Metastatic malignant neoplasm to endocardium (disorder)
        "94279001",  # Metastatic malignant neoplasm to endocervix (disorder)
        "94280003",  # Metastatic malignant neoplasm to endocrine gland (disorder)
        "94281004",  # Metastatic malignant neoplasm to endometrium (disorder)
        "94282006",  # Metastatic malignant neoplasm to epicardium (disorder)
        "94283001",  # Metastatic malignant neoplasm to epididymis (disorder)
        "94284007",  # Metastatic malignant neoplasm to epiglottis (disorder)
        "94285008",  # Metastatic malignant neoplasm to epitrochlear lymph nodes (disorder)
        "94286009",  # Metastatic malignant neoplasm to esophagus (disorder)
        "94287000",  # Metastatic malignant neoplasm to ethmoid bone (disorder)
        "94288005",  # Metastatic malignant neoplasm to ethmoidal sinus (disorder)
        "94289002",  # Metastatic malignant neoplasm to eustachian tube (disorder)
        "94290006",  # Metastatic malignant neoplasm to exocervix (disorder)
        "94291005",  # Metastatic malignant neoplasm to extrahepatic bile ducts (disorder)
        "94292003",  # Metastatic malignant neoplasm to eye (disorder)
        "94293008",  # Metastatic malignant neoplasm to face (disorder)
        "94294002",  # Metastatic malignant neoplasm to facial nerve (disorder)
        "94295001",  # Metastatic malignant neoplasm to fallopian tube (disorder)
        "94296000",  # Metastatic malignant neoplasm to false vocal cord (disorder)
        "94298004",  # Metastatic malignant neoplasm to female genital organ (disorder)
        "94299007",  # Metastatic malignant neoplasm to femoral lymph nodes (disorder)
        "94300004",  # Metastatic malignant neoplasm to femur (disorder)
        "94301000",  # Metastatic malignant neoplasm to fibula (disorder)
        "94302007",  # Metastatic malignant neoplasm to first cuneiform bone of foot (disorder)
        "94303002",  # Metastatic malignant neoplasm to flank (disorder)
        "94304008",  # Metastatic malignant neoplasm to floor of mouth (disorder)
        "94305009",  # Metastatic malignant neoplasm to foot (disorder)
        "94306005",  # Metastatic malignant neoplasm to forearm (disorder)
        "94307001",  # Metastatic malignant neoplasm to prepuce (disorder)
        "94308006",  # Metastatic malignant neoplasm to frontal bone (disorder)
        "94309003",  # Metastatic malignant neoplasm to frontal lobe (disorder)
        "94310008",  # Metastatic malignant neoplasm to frontal sinus (disorder)
        "94311007",  # Metastatic malignant neoplasm to fundus of stomach (disorder)
        "94312000",  # Metastatic malignant neoplasm to gallbladder (disorder)
        "94313005",  # Metastatic malignant neoplasm to gastrointestinal tract (disorder)
        "94314004",  # Metastatic malignant neoplasm to gingival mucosa (disorder)
        "94315003",  # Metastatic malignant neoplasm to glans penis (disorder)
        "94316002",  # Metastatic malignant neoplasm to glomus jugulare (disorder)
        "94317006",  # Metastatic malignant neoplasm to glossopharyngeal nerve (disorder)
        "94318001",  # Metastatic malignant neoplasm to glottis (disorder)
        "94319009",  # Metastatic malignant neoplasm to great vessels (disorder)
        "94320003",  # Metastatic malignant neoplasm to greater curvature of stomach (disorder)
        "94321004",  # Metastatic malignant neoplasm to gum (disorder)
        "94322006",  # Metastatic malignant neoplasm to hamate bone (disorder)
        "94323001",  # Metastatic malignant neoplasm to hand (disorder)
        "94324007",  # Metastatic malignant neoplasm to hard palate (disorder)
        "94325008",  # Metastatic malignant neoplasm to head of pancreas (disorder)
        "94326009",  # Metastatic malignant neoplasm to head (disorder)
        "94327000",  # Metastatic malignant neoplasm to heart (disorder)
        "94328005",  # Metastatic malignant neoplasm to hepatic flexure of colon (disorder)
        "94329002",  # Metastatic malignant neoplasm to hilus of lung (disorder)
        "94330007",  # Metastatic malignant neoplasm to hypogastric lymph nodes (disorder)
        "94331006",  # Metastatic malignant neoplasm to hypoglossal nerve (disorder)
        "94332004",  # Metastatic malignant neoplasm to hypopharyngeal aspect of aryepiglottic fold (disorder)
        "94333009",  # Metastatic malignant neoplasm to hypopharyngeal aspect of interarytenoid fold (disorder)
        "94334003",  # Metastatic malignant neoplasm to hypopharynx (disorder)
        "94335002",  # Metastatic malignant neoplasm to ileum (disorder)
        "94336001",  # Metastatic malignant neoplasm to iliac lymph nodes (disorder)
        "94337005",  # Metastatic malignant neoplasm to ilium (disorder)
        "94338000",  # Metastatic malignant neoplasm to infraclavicular lymph nodes (disorder)
        "94339008",  # Metastatic malignant neoplasm to inguinal lymph nodes (disorder)
        "94340005",  # Metastatic malignant neoplasm to inguinal region (disorder)
        "94341009",  # Metastatic malignant neoplasm to inner aspect of lip (disorder)
        "94342002",  # Metastatic malignant neoplasm to inner aspect of lower lip (disorder)
        "94343007",  # Metastatic malignant neoplasm to inner aspect of upper lip (disorder)
        "94344001",  # Metastatic malignant neoplasm to intercostal lymph nodes (disorder)
        "94345000",  # Metastatic malignant neoplasm to intestinal lymph nodes (disorder)
        "94346004",  # Metastatic malignant neoplasm to intestinal tract (disorder)
        "94347008",  # Metastatic malignant neoplasm to intra-abdominal lymph nodes (disorder)
        "94348003",  # Metastatic malignant neoplasm to intra-abdominal organs (disorder)
        "94349006",  # Metastatic malignant neoplasm to intrahepatic bile ducts (disorder)
        "94350006",  # Metastatic malignant neoplasm to intrapelvic lymph nodes (disorder)
        "94351005",  # Metastatic malignant neoplasm to intrathoracic lymph nodes (disorder)
        "94352003",  # Metastatic malignant neoplasm to intrathoracic organs (disorder)
        "94353008",  # Metastatic malignant neoplasm to ischium (disorder)
        "94354002",  # Metastatic malignant neoplasm to islets of Langerhans (disorder)
        "94355001",  # Metastatic malignant neoplasm to isthmus of uterus (disorder)
        "94356000",  # Metastatic malignant neoplasm to jaw (disorder)
        "94357009",  # Metastatic malignant neoplasm to jejunum (disorder)
        "94358004",  # Metastatic malignant neoplasm to junctional region of epiglottis (disorder)
        "94359007",  # Metastatic malignant neoplasm to junctional zone of tongue (disorder)
        "94360002",  # Metastatic malignant neoplasm to kidney (disorder)
        "94361003",  # Metastatic malignant neoplasm to labia majora (disorder)
        "94362005",  # Metastatic malignant neoplasm to labia minora (disorder)
        "94364006",  # Metastatic malignant neoplasm to lacrimal gland (disorder)
        "94365007",  # Metastatic malignant neoplasm to large intestine (disorder)
        "94366008",  # Metastatic malignant neoplasm to laryngeal aspect of aryepiglottic fold (disorder)
        "94367004",  # Metastatic malignant neoplasm to laryngeal aspect of interarytenoid fold (disorder)
        "94368009",  # Metastatic malignant neoplasm to laryngeal commissure (disorder)
        "94369001",  # Metastatic malignant neoplasm to laryngeal surface of epiglottis (disorder)
        "94370000",  # Metastatic malignant neoplasm to larynx (disorder)
        "94371001",  # Metastatic malignant neoplasm to lateral portion floor of mouth (disorder)
        "94372008",  # Metastatic malignant neoplasm to lateral wall of nasopharynx (disorder)
        "94373003",  # Metastatic malignant neoplasm to lateral wall of oropharynx (disorder)
        "94374009",  # Metastatic malignant neoplasm to lateral wall of urinary bladder (disorder)
        "94375005",  # Metastatic malignant neoplasm to left lower lobe of lung (disorder)
        "94376006",  # Metastatic malignant neoplasm to left upper lobe of lung (disorder)
        "94378007",  # Metastatic malignant neoplasm to lesser curvature of stomach (disorder)
        "94379004",  # Metastatic malignant neoplasm to lingual tonsil (disorder)
        "94380001",  # Metastatic malignant neoplasm to lip (disorder)
        "94381002",  # Metastatic malignant neoplasm to liver (disorder)
        "94382009",  # Metastatic malignant neoplasm to long bone of lower limb (disorder)
        "94383004",  # Metastatic malignant neoplasm to long bone of upper limb (disorder)
        "94384005",  # Metastatic malignant neoplasm to lower gum (disorder)
        "94386007",  # Metastatic malignant neoplasm to lower limb (disorder)
        "94388008",  # Metastatic malignant neoplasm to lower third of esophagus (disorder)
        "94389000",  # Metastatic malignant neoplasm to lumbar vertebral column (disorder)
        "94390009",  # Metastatic malignant neoplasm to lunate bone (disorder)
        "94391008",  # Metastatic malignant neoplasm to lung (disorder)
        "94392001",  # Metastatic malignant neoplasm to lymph node (disorder)
        "94393006",  # Metastatic malignant neoplasm to lymph nodes of face (disorder)
        "94394000",  # Metastatic malignant neoplasm to lymph nodes of head (disorder)
        "94395004",  # Metastatic malignant neoplasm to lymph nodes of lower limb (disorder)
        "94396003",  # Metastatic malignant neoplasm to lymph nodes of multiple sites (disorder)
        "94397007",  # Metastatic malignant neoplasm to lymph nodes of neck (disorder)
        "94398002",  # Metastatic malignant neoplasm to lymph nodes of upper limb (disorder)
        "94399005",  # Metastatic malignant neoplasm to main bronchus (disorder)
        "94400003",  # Metastatic malignant neoplasm to major salivary gland (disorder)
        "94402006",  # Metastatic malignant neoplasm to male genital organ (disorder)
        "94403001",  # Metastatic malignant neoplasm to mandible (disorder)
        "94404007",  # Metastatic malignant neoplasm to mastoid air cells (disorder)
        "94405008",  # Metastatic malignant neoplasm to maxilla (disorder)
        "94406009",  # Metastatic malignant neoplasm to maxillary sinus (disorder)
        "94407000",  # Metastatic malignant neoplasm to Meckel's diverticulum (disorder)
        "94408005",  # Metastatic malignant neoplasm to mediastinal lymph nodes (disorder)
        "94409002",  # Metastatic malignant neoplasm to mediastinum (disorder)
        "94410007",  # Metastatic malignant neoplasm to mesenteric lymph nodes (disorder)
        "94411006",  # Metastatic malignant neoplasm to metacarpal bone (disorder)
        "94412004",  # Metastatic malignant neoplasm to metatarsal bone (disorder)
        "94413009",  # Metastatic malignant neoplasm to middle ear (disorder)
        "94414003",  # Metastatic malignant neoplasm to middle third of esophagus (disorder)
        "94415002",  # Metastatic malignant neoplasm to minor salivary gland (disorder)
        "94416001",  # Metastatic malignant neoplasm to mouth (disorder)
        "94417005",  # Metastatic malignant neoplasm to multiple endocrine glands (disorder)
        "94418000",  # Metastatic malignant neoplasm to muscle of abdomen (disorder)
        "94419008",  # Metastatic malignant neoplasm to muscle of buttock (disorder)
        "94420002",  # Metastatic malignant neoplasm to muscle of face (disorder)
        "94421003",  # Metastatic malignant neoplasm to muscle of head (disorder)
        "94422005",  # Metastatic malignant neoplasm to muscle of hip (disorder)
        "94423000",  # Metastatic malignant neoplasm to muscle of inguinal region (disorder)
        "94424006",  # Metastatic malignant neoplasm to muscle of lower limb (disorder)
        "94425007",  # Metastatic malignant neoplasm to muscle of neck (disorder)
        "94426008",  # Metastatic malignant neoplasm to muscle of pelvis (disorder)
        "94427004",  # Metastatic malignant neoplasm to muscle of perineum (disorder)
        "94428009",  # Metastatic malignant neoplasm to muscle of shoulder (disorder)
        "94429001",  # Metastatic malignant neoplasm to muscle of thorax (disorder)
        "94430006",  # Metastatic malignant neoplasm to muscle of trunk (disorder)
        "94431005",  # Metastatic malignant neoplasm to muscle of upper limb (disorder)
        "94432003",  # Metastatic malignant neoplasm to muscle (disorder)
        "94433008",  # Metastatic malignant neoplasm to myocardium (disorder)
        "94434002",  # Metastatic malignant neoplasm to myometrium (disorder)
        "94435001",  # Metastatic malignant neoplasm to nasal bone (disorder)
        "94436000",  # Metastatic malignant neoplasm to nasal cavity (disorder)
        "94437009",  # Metastatic malignant neoplasm to nasal concha (disorder)
        "94439007",  # Metastatic malignant neoplasm to navicular bone of foot (disorder)
        "94440009",  # Metastatic malignant neoplasm to scaphoid bone (disorder)
        "94441008",  # Metastatic malignant neoplasm to neck (disorder)
        "94442001",  # Metastatic malignant neoplasm to nervous system (disorder)
        "94445004",  # Metastatic malignant neoplasm to nose (disorder)
        "94446003",  # Metastatic malignant neoplasm to obturator lymph nodes (disorder)
        "94447007",  # Metastatic malignant neoplasm to occipital bone (disorder)
        "94448002",  # Metastatic malignant neoplasm to occipital lobe (disorder)
        "94449005",  # Metastatic malignant neoplasm to occipital lymph nodes (disorder)
        "94450005",  # Metastatic malignant neoplasm to oculomotor nerve (disorder)
        "94451009",  # Metastatic malignant neoplasm to olfactory nerve (disorder)
        "94452002",  # Metastatic malignant neoplasm to optic nerve (disorder)
        "94453007",  # Metastatic malignant neoplasm to orbit (disorder)
        "94454001",  # Metastatic malignant neoplasm to oropharynx (disorder)
        "94455000",  # Metastatic malignant neoplasm to ovary (disorder)
        "94456004",  # Metastatic malignant neoplasm to palate (disorder)
        "94457008",  # Metastatic malignant neoplasm to palatine bone (disorder)
        "94458003",  # Metastatic malignant neoplasm to tonsil (disorder)
        "94459006",  # Metastatic malignant neoplasm to pancreas (disorder)
        "94460001",  # Metastatic malignant neoplasm to pancreatic duct (disorder)
        "94461002",  # Metastatic malignant neoplasm to para-aortic body (disorder)
        "94462009",  # Metastatic malignant neoplasm to paraganglion (disorder)
        "94463004",  # Metastatic malignant neoplasm to paramammary lymph nodes (disorder)
        "94464005",  # Metastatic malignant neoplasm to parametrial lymph nodes (disorder)
        "94465006",  # Metastatic malignant neoplasm to parametrium (disorder)
        "94466007",  # Metastatic malignant neoplasm to pararectal lymph nodes (disorder)
        "94467003",  # Metastatic malignant neoplasm to parathyroid gland (disorder)
        "94468008",  # Metastatic malignant neoplasm to paraurethral glands (disorder)
        "94469000",  # Metastatic malignant neoplasm to paravaginal lymph nodes (disorder)
        "94470004",  # Metastatic malignant neoplasm to parietal bone (disorder)
        "94471000",  # Metastatic malignant neoplasm to parietal lobe (disorder)
        "94472007",  # Metastatic malignant neoplasm to parietal peritoneum (disorder)
        "94473002",  # Metastatic malignant neoplasm to parietal pleura (disorder)
        "94474008",  # Metastatic malignant neoplasm to parotid gland (disorder)
        "94475009",  # Metastatic malignant neoplasm to parotid lymph nodes (disorder)
        "94476005",  # Metastatic malignant neoplasm to patella (disorder)
        "94477001",  # Metastatic malignant neoplasm to pectoral axillary lymph nodes (disorder)
        "94478006",  # Metastatic malignant neoplasm to pelvic bone (disorder)
        "94479003",  # Metastatic malignant neoplasm to pelvic peritoneum (disorder)
        "94480000",  # Metastatic malignant neoplasm to pelvis (disorder)
        "94481001",  # Metastatic malignant neoplasm to penis (disorder)
        "94482008",  # Metastatic malignant neoplasm to periadrenal tissue (disorder)
        "94483003",  # Metastatic malignant neoplasm to perianal skin (disorder)
        "94484009",  # Metastatic malignant neoplasm to pericardium (disorder)
        "94485005",  # Metastatic malignant neoplasm to perirenal tissue (disorder)
        "94486006",  # Metastatic malignant neoplasm to phalanx of foot (disorder)
        "94487002",  # Metastatic malignant neoplasm to phalanx of hand (disorder)
        "94488007",  # Metastatic malignant neoplasm to pharynx (disorder)
        "94489004",  # Metastatic malignant neoplasm to pineal gland (disorder)
        "94490008",  # Metastatic malignant neoplasm to pisiform bone of hand (disorder)
        "94491007",  # Metastatic malignant neoplasm to pituitary gland (disorder)
        "94492000",  # Metastatic malignant neoplasm to placenta (disorder)
        "94493005",  # Metastatic malignant neoplasm to pleura (disorder)
        "94494004",  # Metastatic malignant neoplasm to popliteal lymph nodes (disorder)
        "94495003",  # Metastatic malignant neoplasm to postcricoid region (disorder)
        "94496002",  # Metastatic malignant neoplasm to posterior hypopharyngeal wall (disorder)
        "94497006",  # Metastatic malignant neoplasm to posterior mediastinum (disorder)
        "94498001",  # Metastatic malignant neoplasm to posterior wall of nasopharynx (disorder)
        "94499009",  # Metastatic malignant neoplasm to posterior wall of oropharynx (disorder)
        "94500000",  # Metastatic malignant neoplasm to posterior wall of urinary bladder (disorder)
        "94501001",  # Metastatic malignant neoplasm to preauricular lymph nodes (disorder)
        "94502008",  # Metastatic malignant neoplasm to presacral region (disorder)
        "94503003",  # Metastatic malignant neoplasm to prostate (disorder)
        "94504009",  # Metastatic malignant neoplasm to pubis (disorder)
        "94505005",  # Metastatic malignant neoplasm to pyloric antrum (disorder)
        "94506006",  # Metastatic malignant neoplasm to pylorus (disorder)
        "94507002",  # Metastatic malignant neoplasm to pyriform sinus (disorder)
        "94508007",  # Metastatic malignant neoplasm to radius (disorder)
        "94509004",  # Metastatic malignant neoplasm to rectosigmoid junction (disorder)
        "94510009",  # Metastatic malignant neoplasm to rectouterine pouch (disorder)
        "94511008",  # Metastatic malignant neoplasm to rectovaginal septum (disorder)
        "94512001",  # Metastatic malignant neoplasm to rectovesical septum (disorder)
        "94513006",  # Metastatic malignant neoplasm to rectum (disorder)
        "94514000",  # Metastatic malignant neoplasm to renal pelvis (disorder)
        "94515004",  # Metastatic malignant neoplasm to respiratory tract (disorder)
        "94516003",  # Metastatic malignant neoplasm to retina (disorder)
        "94517007",  # Metastatic malignant neoplasm to retrocecal tissue (disorder)
        "94518002",  # Metastatic malignant neoplasm to retromolar area (disorder)
        "94519005",  # Metastatic malignant neoplasm to retroperitoneal lymph nodes (disorder)
        "94520004",  # Metastatic malignant neoplasm to retropharyngeal lymph nodes (disorder)
        "94521000",  # Metastatic malignant neoplasm to rib (disorder)
        "94522007",  # Metastatic malignant neoplasm to right lower lobe of lung (disorder)
        "94523002",  # Metastatic malignant neoplasm to middle lobe of right lung (disorder)
        "94524008",  # Metastatic malignant neoplasm to right upper lobe of lung (disorder)
        "94525009",  # Metastatic malignant neoplasm to round ligament of uterus (disorder)
        "94526005",  # Metastatic malignant neoplasm to sacrococcygeal region (disorder)
        "94527001",  # Metastatic malignant neoplasm to sacrum (disorder)
        "94528006",  # Metastatic malignant neoplasm to scalene lymph nodes (disorder)
        "94529003",  # Metastatic malignant neoplasm to scapula (disorder)
        "94530008",  # Metastatic malignant neoplasm to sclera (disorder)
        "94531007",  # Metastatic malignant neoplasm to scrotum (disorder)
        "94532000",  # Metastatic malignant neoplasm to sebaceous gland (disorder)
        "94533005",  # Metastatic malignant neoplasm to second cuneiform bone of foot (disorder)
        "94534004",  # Metastatic malignant neoplasm to septum of nose (disorder)
        "94535003",  # Metastatic malignant neoplasm to short bone of lower limb (disorder)
        "94536002",  # Metastatic malignant neoplasm to short bone of upper limb (disorder)
        "94537006",  # Metastatic malignant neoplasm to shoulder (disorder)
        "94538001",  # Metastatic malignant neoplasm to sigmoid colon (disorder)
        "94539009",  # Metastatic malignant neoplasm to skin of abdomen (disorder)
        "94540006",  # Metastatic malignant neoplasm to skin of ankle (disorder)
        "94542003",  # Metastatic malignant neoplasm to skin of axilla (disorder)
        "94543008",  # Metastatic malignant neoplasm to skin of back (disorder)
        "94544002",  # Metastatic malignant neoplasm to skin of breast (disorder)
        "94545001",  # Metastatic malignant neoplasm to skin of buttock (disorder)
        "94546000",  # Metastatic malignant neoplasm to skin of cheek (disorder)
        "94547009",  # Metastatic malignant neoplasm to skin of chest (disorder)
        "94548004",  # Metastatic malignant neoplasm to skin of chin (disorder)
        "94549007",  # Metastatic malignant neoplasm to skin of ear (disorder)
        "94550007",  # Metastatic malignant neoplasm to skin of elbow (disorder)
        "94551006",  # Metastatic malignant neoplasm to skin of external auditory canal (disorder)
        "94552004",  # Metastatic malignant neoplasm to skin of eyebrow (disorder)
        "94553009",  # Metastatic malignant neoplasm to skin of eyelid (disorder)
        "94554003",  # Metastatic malignant neoplasm to skin of face (disorder)
        "94555002",  # Metastatic malignant neoplasm to skin of finger (disorder)
        "94556001",  # Metastatic malignant neoplasm to skin of foot (disorder)
        "94557005",  # Metastatic malignant neoplasm to skin of forearm (disorder)
        "94558000",  # Metastatic malignant neoplasm to skin of forehead (disorder)
        "94559008",  # Metastatic malignant neoplasm to skin of groin (disorder)
        "94560003",  # Metastatic malignant neoplasm to skin of hand (disorder)
        "94561004",  # Metastatic malignant neoplasm to skin of hip (disorder)
        "94562006",  # Metastatic malignant neoplasm to skin of knee (disorder)
        "94564007",  # Metastatic malignant neoplasm to skin of lip (disorder)
        "94565008",  # Metastatic malignant neoplasm to skin of lower limb (disorder)
        "94566009",  # Metastatic malignant neoplasm to skin of neck (disorder)
        "94567000",  # Metastatic malignant neoplasm to skin of nose (disorder)
        "94568005",  # Metastatic malignant neoplasm to skin of perineum (disorder)
        "94569002",  # Metastatic malignant neoplasm to skin of popliteal area (disorder)
        "94570001",  # Metastatic malignant neoplasm to skin of scalp (disorder)
        "94571002",  # Metastatic malignant neoplasm to skin of shoulder (disorder)
        "94572009",  # Metastatic malignant neoplasm to skin of temporal region (disorder)
        "94573004",  # Metastatic malignant neoplasm to skin of thigh (disorder)
        "94574005",  # Metastatic malignant neoplasm to skin of toe (disorder)
        "94575006",  # Metastatic malignant neoplasm to skin of trunk (disorder)
        "94577003",  # Metastatic malignant neoplasm to skin of upper limb (disorder)
        "94578008",  # Metastatic malignant neoplasm to skin of wrist (disorder)
        "94579000",  # Metastatic malignant neoplasm to skin (disorder)
        "94580002",  # Metastatic malignant neoplasm to small intestine (disorder)
        "94581003",  # Metastatic malignant neoplasm to soft palate (disorder)
        "94582005",  # Metastatic malignant neoplasm to soft tissues of abdomen (disorder)
        "94583000",  # Metastatic malignant neoplasm to soft tissues of axilla (disorder)
        "94584006",  # Metastatic malignant neoplasm to soft tissues of buttock (disorder)
        "94585007",  # Metastatic malignant neoplasm to soft tissues of face (disorder)
        "94586008",  # Metastatic malignant neoplasm to soft tissues of head (disorder)
        "94587004",  # Metastatic malignant neoplasm to soft tissues of hip (disorder)
        "94588009",  # Metastatic malignant neoplasm to soft tissues of inguinal region (disorder)
        "94589001",  # Metastatic malignant neoplasm to soft tissues of lower limb (disorder)
        "94590005",  # Metastatic malignant neoplasm to soft tissues of neck (disorder)
        "94591009",  # Metastatic malignant neoplasm to soft tissues of pelvis (disorder)
        "94592002",  # Metastatic malignant neoplasm to soft tissues of perineum (disorder)
        "94593007",  # Metastatic malignant neoplasm to soft tissues of shoulder (disorder)
        "94594001",  # Metastatic malignant neoplasm to soft tissues of thorax (disorder)
        "94595000",  # Metastatic malignant neoplasm to soft tissues of trunk (disorder)
        "94596004",  # Metastatic malignant neoplasm to soft tissues of upper limb (disorder)
        "94597008",  # Metastatic malignant neoplasm to spermatic cord (disorder)
        "94598003",  # Metastatic malignant neoplasm to sphenoid bone (disorder)
        "94599006",  # Metastatic malignant neoplasm to sphenoidal sinus (disorder)
        "94600009",  # Metastatic malignant neoplasm to spinal cord (disorder)
        "94601008",  # Metastatic malignant neoplasm to spinal meninges (disorder)
        "94602001",  # Metastatic malignant neoplasm to vertebral column (disorder)
        "94603006",  # Metastatic malignant neoplasm to spleen (disorder)
        "94604000",  # Metastatic malignant neoplasm to splenic flexure of colon (disorder)
        "94605004",  # Metastatic malignant neoplasm to sternum (disorder)
        "94606003",  # Metastatic malignant neoplasm to stomach (disorder)
        "94607007",  # Metastatic malignant neoplasm to subglottis (disorder)
        "94608002",  # Metastatic malignant neoplasm to sublingual gland (disorder)
        "94609005",  # Metastatic malignant neoplasm to submandibular lymph nodes (disorder)
        "94610000",  # Metastatic malignant neoplasm to submaxillary gland (disorder)
        "94611001",  # Metastatic malignant neoplasm to submental lymph nodes (disorder)
        "94612008",  # Metastatic malignant neoplasm to superficial inguinal lymph nodes (disorder)
        "94613003",  # Metastatic malignant neoplasm to superior wall of nasopharynx (disorder)
        "94614009",  # Metastatic malignant neoplasm to supraclavicular lymph nodes (disorder)
        "94615005",  # Metastatic malignant neoplasm to supraclavicular region (disorder)
        "94616006",  # Metastatic malignant neoplasm to supraglottis (disorder)
        "94617002",  # Metastatic malignant neoplasm to sweat gland (disorder)
        "94618007",  # Metastatic malignant neoplasm to tail of pancreas (disorder)
        "94619004",  # Metastatic malignant neoplasm to talus (disorder)
        "94620005",  # Metastatic malignant neoplasm to tarsal bone (disorder)
        "94621009",  # Metastatic malignant neoplasm to temporal bone (disorder)
        "94622002",  # Metastatic malignant neoplasm to temporal lobe (disorder)
        "94623007",  # Metastatic malignant neoplasm to testis (disorder)
        "94624001",  # Metastatic malignant neoplasm to the mesentery (disorder)
        "94625000",  # Metastatic malignant neoplasm to the mesocolon (disorder)
        "94626004",  # Metastatic malignant neoplasm to omentum (disorder)
        "94627008",  # Metastatic malignant neoplasm to peritoneum (disorder)
        "94628003",  # Metastatic malignant neoplasm to retroperitoneum (disorder)
        "94629006",  # Metastatic malignant neoplasm to thigh (disorder)
        "94630001",  # Metastatic malignant neoplasm to third cuneiform bone of foot (disorder)
        "94631002",  # Metastatic malignant neoplasm to thoracic esophagus (disorder)
        "94632009",  # Metastatic malignant neoplasm to thymus (disorder)
        "94633004",  # Metastatic malignant neoplasm to thyroglossal duct (disorder)
        "94634005",  # Metastatic malignant neoplasm to thyroid gland (disorder)
        "94635006",  # Metastatic malignant neoplasm to tibia (disorder)
        "94636007",  # Metastatic malignant neoplasm to tibial lymph nodes (disorder)
        "94637003",  # Metastatic malignant neoplasm to tip and lateral border of tongue (disorder)
        "94638008",  # Metastatic malignant neoplasm to tongue (disorder)
        "94639000",  # Metastatic malignant neoplasm to tonsillar fossa (disorder)
        "94640003",  # Metastatic malignant neoplasm to tonsillar pillar (disorder)
        "94641004",  # Metastatic malignant neoplasm to trachea (disorder)
        "94642006",  # Metastatic malignant neoplasm to tracheobronchial lymph nodes (disorder)
        "94643001",  # Metastatic malignant neoplasm to transverse colon (disorder)
        "94644007",  # Metastatic malignant neoplasm to trapezium (disorder)
        "94645008",  # Metastatic malignant neoplasm to trapezoid bone (disorder)
        "94646009",  # Metastatic malignant neoplasm to trigeminal nerve (disorder)
        "94647000",  # Metastatic malignant neoplasm to trigone of urinary bladder (disorder)
        "94648005",  # Metastatic malignant neoplasm to trochlear nerve (disorder)
        "94649002",  # Metastatic malignant neoplasm to trunk (disorder)
        "94650002",  # Metastatic malignant neoplasm to ulna (disorder)
        "94651003",  # Metastatic malignant neoplasm to undescended testis (disorder)
        "94652005",  # Metastatic malignant neoplasm to upper gum (disorder)
        "94654006",  # Metastatic malignant neoplasm to upper limb (disorder)
        "94656008",  # Metastatic malignant neoplasm to upper respiratory tract (disorder)
        "94657004",  # Metastatic malignant neoplasm to upper third of esophagus (disorder)
        "94658009",  # Metastatic malignant neoplasm to urachus (disorder)
        "94659001",  # Metastatic malignant neoplasm to ureter (disorder)
        "94660006",  # Metastatic malignant neoplasm to ureteric orifice of urinary bladder (disorder)
        "94661005",  # Metastatic malignant neoplasm to urethra (disorder)
        "94662003",  # Metastatic malignant neoplasm to urinary bladder neck (disorder)
        "94663008",  # Metastatic malignant neoplasm to urinary system (disorder)
        "94664002",  # Metastatic malignant neoplasm to uterine adnexa (disorder)
        "94665001",  # Metastatic malignant neoplasm to uterus (disorder)
        "94666000",  # Metastatic malignant neoplasm to uveal tract (disorder)
        "94667009",  # Metastatic malignant neoplasm to uvula (disorder)
        "94668004",  # Metastatic malignant neoplasm to vagina (disorder)
        "94669007",  # Metastatic malignant neoplasm to vagus nerve (disorder)
        "94670008",  # Metastatic malignant neoplasm to vallecula (disorder)
        "94671007",  # Metastatic malignant neoplasm to vas deferens (disorder)
        "94672000",  # Metastatic malignant neoplasm to ventral surface of tongue (disorder)
        "94673005",  # Metastatic malignant neoplasm to vermilion border of lip (disorder)
        "94674004",  # Metastatic malignant neoplasm to vermilion border of lower lip (disorder)
        "94675003",  # Metastatic malignant neoplasm to vermilion border of upper lip (disorder)
        "94676002",  # Metastatic malignant neoplasm to vestibule of mouth (disorder)
        "94677006",  # Metastatic malignant neoplasm to vestibule of nose (disorder)
        "94678001",  # Metastatic malignant neoplasm to visceral pleura (disorder)
        "94679009",  # Metastatic malignant neoplasm to vocal cord (disorder)
        "94680007",  # Metastatic malignant neoplasm to vomer (disorder)
        "94681006",  # Metastatic malignant neoplasm to vulva (disorder)
        "94682004",  # Metastatic malignant neoplasm to Waldeyer's ring (disorder)
        "94683009",  # Metastatic malignant neoplasm to zygomatic bone (disorder)
        "96311000119109",  # Exacerbation of congestive heart failure (disorder)
        "96901000119105",  # Prostate cancer metastatic to eye (disorder)
        "97051000119105",  # Malignant melanoma metastatic to pancreas (disorder)
        "971918681000119107",  # Chronic respiratory failure due to obstructive sleep apnea (disorder)
        "97751000119108",  # Altered behavior in Alzheimer's disease (disorder)
        "99131000119108",  # Astrocytoma of cerebrum (disorder)
    }


class Diabetes(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of  diabetes.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of type I or type II diabetes.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis of gestational diabetes or steroid-induced diabetes.

    ** Used in:** CMS131v14
    """
    VALUE_SET_NAME = "Diabetes"
    OID = "2.16.840.1.113883.3.464.1003.103.12.1001"
    DEFINITION_VERSION = "20250115"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "E1010",  # Type 1 diabetes mellitus with ketoacidosis without coma
        "E1011",  # Type 1 diabetes mellitus with ketoacidosis with coma
        "E1021",  # Type 1 diabetes mellitus with diabetic nephropathy
        "E1022",  # Type 1 diabetes mellitus with diabetic chronic kidney disease
        "E1029",  # Type 1 diabetes mellitus with other diabetic kidney complication
        "E10311",  # Type 1 diabetes mellitus with unspecified diabetic retinopathy with macular edema
        "E10319",  # Type 1 diabetes mellitus with unspecified diabetic retinopathy without macular edema
        "E103211",  # Type 1 diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, right eye
        "E103212",  # Type 1 diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, left eye
        "E103213",  # Type 1 diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, bilateral
        "E103219",  # Type 1 diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, unspecified eye
        "E103291",  # Type 1 diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, right eye
        "E103292",  # Type 1 diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, left eye
        "E103293",  # Type 1 diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, bilateral
        "E103299",  # Type 1 diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, unspecified eye
        "E103311",  # Type 1 diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, right eye
        "E103312",  # Type 1 diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, left eye
        "E103313",  # Type 1 diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, bilateral
        "E103319",  # Type 1 diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, unspecified eye
        "E103391",  # Type 1 diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, right eye
        "E103392",  # Type 1 diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, left eye
        "E103393",  # Type 1 diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, bilateral
        "E103399",  # Type 1 diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, unspecified eye
        "E103411",  # Type 1 diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, right eye
        "E103412",  # Type 1 diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, left eye
        "E103413",  # Type 1 diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, bilateral
        "E103419",  # Type 1 diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, unspecified eye
        "E103491",  # Type 1 diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, right eye
        "E103492",  # Type 1 diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, left eye
        "E103493",  # Type 1 diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, bilateral
        "E103499",  # Type 1 diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, unspecified eye
        "E103511",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with macular edema, right eye
        "E103512",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with macular edema, left eye
        "E103513",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with macular edema, bilateral
        "E103519",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with macular edema, unspecified eye
        "E103521",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, right eye
        "E103522",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, left eye
        "E103523",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, bilateral
        "E103529",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, unspecified eye
        "E103531",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, right eye
        "E103532",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, left eye
        "E103533",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, bilateral
        "E103539",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, unspecified eye
        "E103541",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, right eye
        "E103542",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, left eye
        "E103543",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, bilateral
        "E103549",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, unspecified eye
        "E103551",  # Type 1 diabetes mellitus with stable proliferative diabetic retinopathy, right eye
        "E103552",  # Type 1 diabetes mellitus with stable proliferative diabetic retinopathy, left eye
        "E103553",  # Type 1 diabetes mellitus with stable proliferative diabetic retinopathy, bilateral
        "E103559",  # Type 1 diabetes mellitus with stable proliferative diabetic retinopathy, unspecified eye
        "E103591",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy without macular edema, right eye
        "E103592",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy without macular edema, left eye
        "E103593",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy without macular edema, bilateral
        "E103599",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy without macular edema, unspecified eye
        "E1036",    # Type 1 diabetes mellitus with diabetic cataract
        "E1037X1",  # Type 1 diabetes mellitus with diabetic macular edema, resolved following treatment, right eye
        "E1037X2",  # Type 1 diabetes mellitus with diabetic macular edema, resolved following treatment, left eye
        "E1037X3",  # Type 1 diabetes mellitus with diabetic macular edema, resolved following treatment, bilateral
        "E1037X9",  # Type 1 diabetes mellitus with diabetic macular edema, resolved following treatment, unspecified eye
        "E1039",    # Type 1 diabetes mellitus with other diabetic ophthalmic complication
        "E1040",    # Type 1 diabetes mellitus with diabetic neuropathy, unspecified
        "E1041",    # Type 1 diabetes mellitus with diabetic mononeuropathy
        "E1042",    # Type 1 diabetes mellitus with diabetic polyneuropathy
        "E1043",    # Type 1 diabetes mellitus with diabetic autonomic (poly)neuropathy
        "E1044",    # Type 1 diabetes mellitus with diabetic amyotrophy
        "E1049",    # Type 1 diabetes mellitus with other diabetic neurological complication
        "E1051",    # Type 1 diabetes mellitus with diabetic peripheral angiopathy without gangrene
        "E1052",    # Type 1 diabetes mellitus with diabetic peripheral angiopathy with gangrene
        "E1059",    # Type 1 diabetes mellitus with other circulatory complications
        "E10610",   # Type 1 diabetes mellitus with diabetic neuropathic arthropathy
        "E10618",   # Type 1 diabetes mellitus with other diabetic arthropathy
        "E10620",   # Type 1 diabetes mellitus with diabetic dermatitis
        "E10621",   # Type 1 diabetes mellitus with foot ulcer
        "E10622",   # Type 1 diabetes mellitus with other skin ulcer
        "E10628",   # Type 1 diabetes mellitus with other skin complications
        "E10630",   # Type 1 diabetes mellitus with periodontal disease
        "E10638",   # Type 1 diabetes mellitus with other oral complications
        "E10641",   # Type 1 diabetes mellitus with hypoglycemia with coma
        "E10649",   # Type 1 diabetes mellitus with hypoglycemia without coma
        "E1065",    # Type 1 diabetes mellitus with hyperglycemia
        "E1069",    # Type 1 diabetes mellitus with other specified complication
        "E108",     # Type 1 diabetes mellitus with unspecified complications
        "E109",     # Type 1 diabetes mellitus without complications
        "E10A2",    # Type 1 diabetes mellitus, presymptomatic, Stage 2
        "E1100",    # Type 2 diabetes mellitus with hyperosmolarity without nonketotic hyperglycemic-hyperosmolar coma (NKHHC)
        "E1101",    # Type 2 diabetes mellitus with hyperosmolarity with coma
        "E1110",    # Type 2 diabetes mellitus with ketoacidosis without coma
        "E1111",    # Type 2 diabetes mellitus with ketoacidosis with coma
        "E1121",    # Type 2 diabetes mellitus with diabetic nephropathy
        "E1122",    # Type 2 diabetes mellitus with diabetic chronic kidney disease
        "E1129",    # Type 2 diabetes mellitus with other diabetic kidney complication
        "E11311",   # Type 2 diabetes mellitus with unspecified diabetic retinopathy with macular edema
        "E11319",   # Type 2 diabetes mellitus with unspecified diabetic retinopathy without macular edema
        "E113211",  # Type 2 diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, right eye
        "E113212",  # Type 2 diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, left eye
        "E113213",  # Type 2 diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, bilateral
        "E113219",  # Type 2 diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, unspecified eye
        "E113291",  # Type 2 diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, right eye
        "E113292",  # Type 2 diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, left eye
        "E113293",  # Type 2 diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, bilateral
        "E113299",  # Type 2 diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, unspecified eye
        "E113311",  # Type 2 diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, right eye
        "E113312",  # Type 2 diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, left eye
        "E113313",  # Type 2 diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, bilateral
        "E113319",  # Type 2 diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, unspecified eye
        "E113391",  # Type 2 diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, right eye
        "E113392",  # Type 2 diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, left eye
        "E113393",  # Type 2 diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, bilateral
        "E113399",  # Type 2 diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, unspecified eye
        "E113411",  # Type 2 diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, right eye
        "E113412",  # Type 2 diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, left eye
        "E113413",  # Type 2 diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, bilateral
        "E113419",  # Type 2 diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, unspecified eye
        "E113491",  # Type 2 diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, right eye
        "E113492",  # Type 2 diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, left eye
        "E113493",  # Type 2 diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, bilateral
        "E113499",  # Type 2 diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, unspecified eye
        "E113511",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with macular edema, right eye
        "E113512",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with macular edema, left eye
        "E113513",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with macular edema, bilateral
        "E113519",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with macular edema, unspecified eye
        "E113521",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, right eye
        "E113522",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, left eye
        "E113523",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, bilateral
        "E113529",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, unspecified eye
        "E113531",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, right eye
        "E113532",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, left eye
        "E113533",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, bilateral
        "E113539",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, unspecified eye
        "E113541",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, right eye
        "E113542",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, left eye
        "E113543",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, bilateral
        "E113549",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, unspecified eye
        "E113551",  # Type 2 diabetes mellitus with stable proliferative diabetic retinopathy, right eye
        "E113552",  # Type 2 diabetes mellitus with stable proliferative diabetic retinopathy, left eye
        "E113553",  # Type 2 diabetes mellitus with stable proliferative diabetic retinopathy, bilateral
        "E113559",  # Type 2 diabetes mellitus with stable proliferative diabetic retinopathy, unspecified eye
        "E113591",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy without macular edema, right eye
        "E113592",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy without macular edema, left eye
        "E113593",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy without macular edema, bilateral
        "E113599",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy without macular edema, unspecified eye
        "E1136",    # Type 2 diabetes mellitus with diabetic cataract
        "E1137X1",  # Type 2 diabetes mellitus with diabetic macular edema, resolved following treatment, right eye
        "E1137X2",  # Type 2 diabetes mellitus with diabetic macular edema, resolved following treatment, left eye
        "E1137X3",  # Type 2 diabetes mellitus with diabetic macular edema, resolved following treatment, bilateral
        "E1137X9",  # Type 2 diabetes mellitus with diabetic macular edema, resolved following treatment, unspecified eye
        "E1139",    # Type 2 diabetes mellitus with other diabetic ophthalmic complication
        "E1140",    # Type 2 diabetes mellitus with diabetic neuropathy, unspecified
        "E1141",    # Type 2 diabetes mellitus with diabetic mononeuropathy
        "E1142",    # Type 2 diabetes mellitus with diabetic polyneuropathy
        "E1143",    # Type 2 diabetes mellitus with diabetic autonomic (poly)neuropathy
        "E1144",    # Type 2 diabetes mellitus with diabetic amyotrophy
        "E1149",    # Type 2 diabetes mellitus with other diabetic neurological complication
        "E1151",    # Type 2 diabetes mellitus with diabetic peripheral angiopathy without gangrene
        "E1152",    # Type 2 diabetes mellitus with diabetic peripheral angiopathy with gangrene
        "E1159",    # Type 2 diabetes mellitus with other circulatory complications
        "E11610",   # Type 2 diabetes mellitus with diabetic neuropathic arthropathy
        "E11618",   # Type 2 diabetes mellitus with other diabetic arthropathy
        "E11620",   # Type 2 diabetes mellitus with diabetic dermatitis
        "E11621",   # Type 2 diabetes mellitus with foot ulcer
        "E11622",   # Type 2 diabetes mellitus with other skin ulcer
        "E11628",   # Type 2 diabetes mellitus with other skin complications
        "E11630",   # Type 2 diabetes mellitus with periodontal disease
        "E11638",   # Type 2 diabetes mellitus with other oral complications
        "E11641",   # Type 2 diabetes mellitus with hypoglycemia with coma
        "E11649",   # Type 2 diabetes mellitus with hypoglycemia without coma
        "E1165",    # Type 2 diabetes mellitus with hyperglycemia
        "E1169",    # Type 2 diabetes mellitus with other specified complication
        "E118",     # Type 2 diabetes mellitus with unspecified complications
        "E119",     # Type 2 diabetes mellitus without complications
        "E1300",    # Other specified diabetes mellitus with hyperosmolarity without nonketotic hyperglycemic-hyperosmolar coma (NKHHC)
        "E1301",    # Other specified diabetes mellitus with hyperosmolarity with coma
        "E1310",    # Other specified diabetes mellitus with ketoacidosis without coma
        "E1311",    # Other specified diabetes mellitus with ketoacidosis with coma
        "E1321",    # Other specified diabetes mellitus with diabetic nephropathy
        "E1322",    # Other specified diabetes mellitus with diabetic chronic kidney disease
        "E1329",    # Other specified diabetes mellitus with other diabetic kidney complication
        "E13311",   # Other specified diabetes mellitus with unspecified diabetic retinopathy with macular edema
        "E13319",   # Other specified diabetes mellitus with unspecified diabetic retinopathy without macular edema
        "E133211",  # Other specified diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, right eye
        "E133212",  # Other specified diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, left eye
        "E133213",  # Other specified diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, bilateral
        "E133219",  # Other specified diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, unspecified eye
        "E133291",  # Other specified diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, right eye
        "E133292",  # Other specified diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, left eye
        "E133293",  # Other specified diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, bilateral
        "E133299",  # Other specified diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, unspecified eye
        "E133311",  # Other specified diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, right eye
        "E133312",  # Other specified diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, left eye
        "E133313",  # Other specified diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, bilateral
        "E133319",  # Other specified diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, unspecified eye
        "E133391",  # Other specified diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, right eye
        "E133392",  # Other specified diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, left eye
        "E133393",  # Other specified diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, bilateral
        "E133399",  # Other specified diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, unspecified eye
        "E133411",  # Other specified diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, right eye
        "E133412",  # Other specified diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, left eye
        "E133413",  # Other specified diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, bilateral
        "E133419",  # Other specified diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, unspecified eye
        "E133491",  # Other specified diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, right eye
        "E133492",  # Other specified diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, left eye
        "E133493",  # Other specified diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, bilateral
        "E133499",  # Other specified diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, unspecified eye
        "E133511",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with macular edema, right eye
        "E133512",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with macular edema, left eye
        "E133513",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with macular edema, bilateral
        "E133519",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with macular edema, unspecified eye
        "E133521",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, right eye
        "E133522",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, left eye
        "E133523",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, bilateral
        "E133529",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, unspecified eye
        "E133531",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, right eye
        "E133532",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, left eye
        "E133533",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, bilateral
        "E133539",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, unspecified eye
        "E133541",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, right eye
        "E133542",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, left eye
        "E133543",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, bilateral
        "E133549",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, unspecified eye
        "E133551",  # Other specified diabetes mellitus with stable proliferative diabetic retinopathy, right eye
        "E133552",  # Other specified diabetes mellitus with stable proliferative diabetic retinopathy, left eye
        "E133553",  # Other specified diabetes mellitus with stable proliferative diabetic retinopathy, bilateral
        "E133559",  # Other specified diabetes mellitus with stable proliferative diabetic retinopathy, unspecified eye
        "E133591",  # Other specified diabetes mellitus with proliferative diabetic retinopathy without macular edema, right eye
        "E133592",  # Other specified diabetes mellitus with proliferative diabetic retinopathy without macular edema, left eye
        "E133593",  # Other specified diabetes mellitus with proliferative diabetic retinopathy without macular edema, bilateral
        "E133599",  # Other specified diabetes mellitus with proliferative diabetic retinopathy without macular edema, unspecified eye
        "E1336",    # Other specified diabetes mellitus with diabetic cataract
        "E1337X1",  # Other specified diabetes mellitus with diabetic macular edema, resolved following treatment, right eye
        "E1337X2",  # Other specified diabetes mellitus with diabetic macular edema, resolved following treatment, left eye
        "E1337X3",  # Other specified diabetes mellitus with diabetic macular edema, resolved following treatment, bilateral
        "E1337X9",  # Other specified diabetes mellitus with diabetic macular edema, resolved following treatment, unspecified eye
        "E1339",    # Other specified diabetes mellitus with other diabetic ophthalmic complication
        "E1340",    # Other specified diabetes mellitus with diabetic neuropathy, unspecified
        "E1341",    # Other specified diabetes mellitus with diabetic mononeuropathy
        "E1342",    # Other specified diabetes mellitus with diabetic polyneuropathy
        "E1343",    # Other specified diabetes mellitus with diabetic autonomic (poly)neuropathy
        "E1344",    # Other specified diabetes mellitus with diabetic amyotrophy
        "E1349",    # Other specified diabetes mellitus with other diabetic neurological complication
        "E1351",    # Other specified diabetes mellitus with diabetic peripheral angiopathy without gangrene
        "E1352",    # Other specified diabetes mellitus with diabetic peripheral angiopathy with gangrene
        "E1359",    # Other specified diabetes mellitus with other circulatory complications
        "E13610",   # Other specified diabetes mellitus with diabetic neuropathic arthropathy
        "E13618",   # Other specified diabetes mellitus with other diabetic arthropathy
        "E13620",   # Other specified diabetes mellitus with diabetic dermatitis
        "E13621",   # Other specified diabetes mellitus with foot ulcer
        "E13622",   # Other specified diabetes mellitus with other skin ulcer
        "E13628",   # Other specified diabetes mellitus with other skin complications
        "E13630",   # Other specified diabetes mellitus with periodontal disease
        "E13638",   # Other specified diabetes mellitus with other oral complications
        "E13641",   # Other specified diabetes mellitus with hypoglycemia with coma
        "E13649",   # Other specified diabetes mellitus with hypoglycemia without coma
        "E1365",    # Other specified diabetes mellitus with hyperglycemia
        "E1369",    # Other specified diabetes mellitus with other specified complication
        "E138",     # Other specified diabetes mellitus with unspecified complications
        "E139",     # Other specified diabetes mellitus without complications
        "O24011",   # Pre-existing type 1 diabetes mellitus, in pregnancy, first trimester
        "O24012",   # Pre-existing type 1 diabetes mellitus, in pregnancy, second trimester
        "O24013",   # Pre-existing type 1 diabetes mellitus, in pregnancy, third trimester
        "O24019",   # Pre-existing type 1 diabetes mellitus, in pregnancy, unspecified trimester
        "O2402",    # Pre-existing type 1 diabetes mellitus, in childbirth
        "O2403",    # Pre-existing type 1 diabetes mellitus, in the puerperium
        "O24111",   # Pre-existing type 2 diabetes mellitus, in pregnancy, first trimester
        "O24112",   # Pre-existing type 2 diabetes mellitus, in pregnancy, second trimester
        "O24113",   # Pre-existing type 2 diabetes mellitus, in pregnancy, third trimester
        "O24119",   # Pre-existing type 2 diabetes mellitus, in pregnancy, unspecified trimester
        "O2412",    # Pre-existing type 2 diabetes mellitus, in childbirth
        "O2413",    # Pre-existing type 2 diabetes mellitus, in the puerperium
        "O24311",   # Unspecified pre-existing diabetes mellitus in pregnancy, first trimester
        "O24312",   # Unspecified pre-existing diabetes mellitus in pregnancy, second trimester
        "O24313",   # Unspecified pre-existing diabetes mellitus in pregnancy, third trimester
        "O24319",   # Unspecified pre-existing diabetes mellitus in pregnancy, unspecified trimester
        "O2432",    # Unspecified pre-existing diabetes mellitus in childbirth
        "O2433",    # Unspecified pre-existing diabetes mellitus in the puerperium
        "O24811",   # Other pre-existing diabetes mellitus in pregnancy, first trimester
        "O24812",   # Other pre-existing diabetes mellitus in pregnancy, second trimester
        "O24813",   # Other pre-existing diabetes mellitus in pregnancy, third trimester
        "O24819",   # Other pre-existing diabetes mellitus in pregnancy, unspecified trimester
        "O2482",    # Other pre-existing diabetes mellitus in childbirth
        "O2483",    # Other pre-existing diabetes mellitus in the puerperium
    }

    SNOMEDCT = {
        "1003605004",  # Glaucoma suspect due to diabetes mellitus type 2 (disorder)
        "102621000119101",  # Ulcer of skin due to type 2 diabetes mellitus (disorder)
        "102781000119107",  # Sensory neuropathy due to type 1 diabetes mellitus (disorder)
        "103981000119101",  # Proliferative retinopathy following surgery due to diabetes mellitus (disorder)
        "104941000119109",  # Ischemia of retina due to type 1 diabetes mellitus (disorder)
        "104951000119106",  # Vitreous hemorrhage due to type 1 diabetes mellitus (disorder)
        "104961000119108",  # Ischemia of retina due to type 2 diabetes mellitus (disorder)
        "106281000119103",  # Pre-existing diabetes mellitus in mother complicating childbirth (disorder)
        "10656231000119100",  # Ulcer of skin of toe due to type 1 diabetes mellitus (disorder)
        "10656271000119102",  # Ulcer of skin of toe due to type 2 diabetes mellitus (disorder)
        "10660471000119109",  # Ulcer of left foot due to type 2 diabetes mellitus (disorder)
        "10661671000119102",  # Ulcer of right foot due to type 2 diabetes mellitus (disorder)
        "10754881000119104",  # Diabetes mellitus in mother complicating childbirth (disorder)
        "109171000119104",  # Retinal edema due to type 1 diabetes mellitus (disorder)
        "110141000119100",  # Ulcer of lower limb due to type 1 diabetes mellitus (disorder)
        "110171000119107",  # Ulcer of lower limb due to type 2 diabetes mellitus (disorder)
        "110181000119105",  # Peripheral sensory neuropathy due to type 2 diabetes mellitus (disorder)
        "111231000119109",  # Dyslipidemia with high density lipoprotein below reference range and triglyceride above reference range due to type 2 diabetes mellitus (disorder)
        "111307005",  # Leprechaunism syndrome (disorder)
        "111552007",  # Diabetes mellitus without complication (disorder)
        "111556005",  # Ketoacidosis without coma due to diabetes mellitus (disorder)
        "1144894004",  # Peripheral autonomic neuropathy due to diabetes mellitus (disorder)
        "11530004",  # Brittle diabetes mellitus (disorder)
        "1172389004",  # Disorder due to malnutrition related diabetes mellitus (disorder)
        "1196922005",  # Neovascular glaucoma due to diabetes mellitus type 2 (disorder)
        "1196923000",  # Neovascular glaucoma due to diabetes mellitus type 1 (disorder)
        "119831000119106",  # Hypoglycemia unawareness due to type 2 diabetes mellitus (disorder)
        "120711000119108",  # Hypoglycemic unawareness due to type 1 diabetes mellitus (disorder)
        "120731000119103",  # Hypoglycemia due to type 2 diabetes mellitus (disorder)
        "1217044000",  # Ketosis-resistant diabetes mellitus (disorder)
        "1217068008",  # Fulminant type 1 diabetes mellitus (disorder)
        "1217674007",  # Neurotrophic keratitis due to diabetes mellitus (disorder)
        "123763000",  # Houssay's syndrome (disorder)
        "1251386008",  # Disorder of skin due to diabetes mellitus (disorder)
        "1255271005",  # Juvenile-onset diabetes mellitus, central and peripheral neurodegeneration syndrome (disorder)
        "1259048007",  # Lesion of brachial plexus due to diabetes mellitus (disorder)
        "126534007",  # Mixed sensorimotor polyneuropathy due to diabetes mellitus (disorder)
        "126535008",  # Motor polyneuropathy due to diabetes mellitus (disorder)
        "127011001",  # Sensory neuropathy due to diabetes mellitus (disorder)
        "127012008",  # Lipoatrophic diabetes (disorder)
        "127013003",  # Disorder of kidney due to diabetes mellitus (disorder)
        "127014009",  # Peripheral angiopathy due to diabetes mellitus (disorder)
        "127991000119101",  # Hypertension concurrent and due to end stage renal disease on dialysis due to type 2 diabetes mellitus (disorder)
        "128001000119105",  # Hypertension concurrent and due to end stage renal disease on dialysis due to type 1 diabetes mellitus (disorder)
        "128201000119100",  # Neurogenic erectile dysfunction due to diabetes mellitus type 2 (disorder)
        "1306699009",  # Stable treated proliferative retinopathy due to diabetes mellitus (disorder)
        "1306700005",  # Stable treated proliferative retinopathy of bilateral eyes due to diabetes mellitus (disorder)
        "1332436006",  # Hypertension in chronic kidney disease stage 3B due to type 1 diabetes mellitus (disorder)
        "1332441003",  # Hypertension in chronic kidney disease stage 3A due to type 1 diabetes mellitus (disorder)
        "1332442005",  # Hypertension in chronic kidney disease stage 3 due to type 1 diabetes mellitus (disorder)
        "1332464001",  # Hypertension in chronic kidney disease stage 2 due to type 1 diabetes mellitus (disorder)
        "1332465000",  # Hypertension in chronic kidney disease stage 3B due to type 2 diabetes mellitus (disorder)
        "1332466004",  # Hypertension in chronic kidney disease stage 3A due to type 2 diabetes mellitus (disorder)
        "1332467008",  # Hypertension in chronic kidney disease stage 5 due to type 1 diabetes mellitus (disorder)
        "1332468003",  # Hypertension in chronic kidney disease stage 4 due to type 1 diabetes mellitus (disorder)
        "1332469006",  # Chronic kidney disease stage 3B due to type 2 diabetes mellitus (disorder)
        "1332470007",  # Chronic kidney disease stage 3A due to type 2 diabetes mellitus (disorder)
        "1332471006",  # Chronic kidney disease stage 3B due to type 1 diabetes mellitus (disorder)
        "1332472004",  # Chronic kidney disease stage 3A due to type 1 diabetes mellitus (disorder)
        "137931000119102",  # Hyperlipidemia due to type 2 diabetes mellitus (disorder)
        "137941000119106",  # Hyperlipidemia due to type 1 diabetes mellitus (disorder)
        "138881000119106",  # Mild nonproliferative retinopathy due to type 1 diabetes mellitus (disorder)
        "138891000119109",  # Moderate nonproliferative retinopathy due to type 1 diabetes mellitus (disorder)
        "138901000119108",  # Severe nonproliferative retinopathy due to diabetes mellitus type 1 (disorder)
        "138911000119106",  # Mild nonproliferative retinopathy due to type 2 diabetes mellitus (disorder)
        "138921000119104",  # Moderate nonproliferative retinopathy due to type 2 diabetes mellitus (disorder)
        "138941000119105",  # Severe nonproliferative retinopathy due to diabetes mellitus type 2 (disorder)
        "140101000119109",  # Hypertension in chronic kidney disease stage 5 due to type 2 diabetes mellitus (disorder)
        "140111000119107",  # Hypertension in chronic kidney disease stage 4 due to type 2 diabetes mellitus (disorder)
        "140121000119100",  # Hypertension in chronic kidney disease stage 3 due to type 2 diabetes mellitus (disorder)
        "140131000119102",  # Hypertension in chronic kidney disease stage 2 due to type 2 diabetes mellitus (disorder)
        "140381000119104",  # Neuropathic ulcer of toe due to type 2 diabetes mellitus (disorder)
        "140391000119101",  # Ulcer of toe due to type 2 diabetes mellitus (disorder)
        "140411000119101",  # Ischemic ankle ulcer due to diabetes mellitus type 2 (disorder)
        "140421000119108",  # Neuropathic ankle ulcer due to diabetes mellitus type 2 (disorder)
        "140521000119107",  # Ischemic ulcer of foot due to type 2 diabetes mellitus (disorder)
        "140531000119105",  # Neuropathic ulcer of foot due to type 2 diabetes mellitus (disorder)
        "1481000119100",  # Diabetes mellitus type 2 without retinopathy (disorder)
        "1491000119102",  # Vitreous hemorrhage due to type 2 diabetes mellitus (disorder)
        "1501000119109",  # Proliferative retinopathy due to type 2 diabetes mellitus (disorder)
        "1511000119107",  # Peripheral neuropathy due to type 2 diabetes mellitus (disorder)
        "1521000119100",  # Ulcer of foot due to type 2 diabetes mellitus (disorder)
        "1531000119102",  # Dermopathy due to type 2 diabetes mellitus (disorder)
        "1551000119108",  # Nonproliferative retinopathy due to type 2 diabetes mellitus (disorder)
        "1571000119104",  # Mixed hyperlipidemia due to type 1 diabetes mellitus (disorder)
        "157141000119108",  # Proteinuria due to type 2 diabetes mellitus (disorder)
        "15936501000119100",  # Inflammatory dermatosis due to type 2 diabetes mellitus (disorder)
        "164881000119109",  # Ulcer of foot due to type 1 diabetes mellitus (disorder)
        "164971000119101",  # Type 2 diabetes mellitus controlled by diet (disorder)
        "16697421000119103",  # Moderate nonproliferative retinopathy of left eye due to diabetes mellitus type 1 (disorder)
        "16697471000119102",  # Moderate nonproliferative retinopathy of right eye due to diabetes mellitus type 1 (disorder)
        "16697521000119104",  # Moderate nonproliferative retinopathy of bilateral eyes due to diabetes mellitus type 1 (disorder)
        "16745051000119107",  # Nonproliferative retinopathy of right eye due to diabetes mellitus type 2 (disorder)
        "16745131000119100",  # Nonproliferative retinopathy of left eye due to diabetes mellitus type 2 (disorder)
        "16745211000119100",  # Nonproliferative retinopathy of left eye due to diabetes mellitus type 1 (disorder)
        "16745411000119104",  # Mild nonproliferative retinopathy of bilateral eyes due to diabetes mellitus type 1 (disorder)
        "16745531000119108",  # Mild nonproliferative retinopathy of left eye due to diabetes mellitus type 2 (disorder)
        "16745611000119102",  # Mild nonproliferative retinopathy of right eye due to diabetes mellitus type 1 (disorder)
        "16745651000119101",  # Mild nonproliferative retinopathy of right eye due to diabetes mellitus type 2 (disorder)
        "16745891000119105",  # Mild nonproliferative retinopathy of left eye due to diabetes mellitus type 1 (disorder)
        "16745931000119102",  # Mild nonproliferative retinopathy of bilateral eyes due to diabetes mellitus type 2 (disorder)
        "16746131000119105",  # Moderate nonproliferative retinopathy of left eye due to diabetes mellitus type 2 (disorder)
        "16746261000119108",  # Moderate nonproliferative retinopathy of right eye due to diabetes mellitus type 2 (disorder)
        "16746341000119103",  # Moderate nonproliferative retinopathy of bilateral eyes due to diabetes mellitus type 2 (disorder)
        "16746661000119106",  # Nonproliferative retinopathy of bilateral eyes due to diabetes mellitus type 1 (disorder)
        "16746741000119102",  # Nonproliferative retinopathy of right eye due to diabetes mellitus type 1 (disorder)
        "16746901000119101",  # Severe nonproliferative retinopathy of left eye due to diabetes mellitus type 2 (disorder)
        "16746941000119104",  # Severe nonproliferative retinopathy of left eye due to diabetes mellitus type 1 (disorder)
        "16747021000119109",  # Severe nonproliferative retinopathy of bilateral eyes due to diabetes mellitus type 2 (disorder)
        "16747101000119101",  # Severe nonproliferative retinopathy of bilateral eyes due to diabetes mellitus type 1 (disorder)
        "16747141000119104",  # Severe nonproliferative retinopathy of right eye due to diabetes mellitus type 2 (disorder)
        "16747341000119101",  # Severe nonproliferative retinopathy of right eye due to diabetes mellitus type 1 (disorder)
        "16747661000119109",  # Nonproliferative retinopathy of bilateral eyes due to diabetes mellitus type 2 (disorder)
        "16747741000119100",  # Proliferative retinopathy of left eye due to diabetes mellitus type 1 (disorder)
        "16747901000119104",  # Proliferative retinopathy of left eye due to diabetes mellitus type 2 (disorder)
        "16748141000119100",  # Proliferative retinopathy of bilateral eyes due to diabetes mellitus type 2 (disorder)
        "16748741000119101",  # Proliferative retinopathy of right eye due to diabetes mellitus type 1 (disorder)
        "16749661000119102",  # Proliferative retinopathy of right eye due to diabetes mellitus type 2 (disorder)
        "16749781000119101",  # Proliferative retinopathy of bilateral eyes due to diabetes mellitus type 1 (disorder)
        "16891151000119103",  # Coronary artery disease due to type 2 diabetes mellitus (disorder)
        "16891191000119108",  # Coronary artery disease due to type 1 diabetes mellitus (disorder)
        "170766006",  # Loss of hypoglycemic warning due to diabetes mellitus (disorder)
        "18521000119106",  # Microalbuminuria due to type 1 diabetes mellitus (disorder)
        "190330002",  # Hyperosmolar coma due to type 1 diabetes mellitus (disorder)
        "190331003",  # Hyperosmolar coma due to type 2 diabetes mellitus (disorder)
        "190368000",  # Type I diabetes mellitus with ulcer (disorder)
        "190372001",  # Type I diabetes mellitus maturity onset (disorder)
        "190389009",  # Type II diabetes mellitus with ulcer (disorder)
        "190406000",  # Ketoacidosis due to malnutrition related diabetes mellitus (disorder)
        "193141005",  # Mononeuritis multiplex due to diabetes mellitus (disorder)
        "193183000",  # Acute painful neuropathy due to diabetes mellitus (disorder)
        "193184006",  # Chronic painful neuropathy due to diabetes mellitus (disorder)
        "193185007",  # Asymptomatic neuropathy due to diabetes mellitus (disorder)
        "193349004",  # Preproliferative retinopathy due to diabetes mellitus (disorder)
        "193350004",  # Advanced maculopathy due to diabetes mellitus (disorder)
        "193489006",  # Iritis due to diabetes mellitus (disorder)
        "19378003",  # Pseudotabes due to diabetes mellitus (disorder)
        "197605007",  # Nephrotic syndrome due to diabetes mellitus (disorder)
        "199223000",  # Diabetes mellitus during pregnancy, childbirth and the puerperium (disorder)
        "199229001",  # Pre-existing type 1 diabetes mellitus (disorder)
        "199230006",  # Pre-existing type 2 diabetes mellitus (disorder)
        "200687002",  # Cellulitis of foot due to diabetes mellitus (disorder)
        "201000119106",  # Disorder due to well controlled type 2 diabetes mellitus (disorder)
        "201250006",  # Ischemic ulcer of foot due to diabetes mellitus (disorder)
        "201251005",  # Neuropathic ulcer of foot due to diabetes mellitus (disorder)
        "201723002",  # Cheiroarthropathy due to diabetes mellitus (disorder)
        "201724008",  # Neuropathic arthropathy due to diabetes mellitus (disorder)
        "23045005",  # Insulin dependent diabetes mellitus type IA (disorder)
        "230572002",  # Neuropathy due to diabetes mellitus (disorder)
        "230574001",  # Acute painful polyneuropathy due to diabetes mellitus (disorder)
        "230575000",  # Chronic painful polyneuropathy due to diabetes mellitus (disorder)
        "230576004",  # Asymmetric polyneuropathy due to diabetes mellitus (disorder)
        "230577008",  # Mononeuropathy due to diabetes mellitus (disorder)
        "230579006",  # Thoracic radiculopathy due to diabetes mellitus (disorder)
        "232020009",  # Disorder of macula due to diabetes mellitus (disorder)
        "232021008",  # Proliferative retinopathy with optic disc neovascularization due to diabetes mellitus (disorder)
        "232022001",  # Proliferative retinopathy with neovascularization elsewhere than the optic disc due to diabetes mellitus (disorder)
        "232023006",  # Traction detachment of retina due to diabetes mellitus (disorder)
        "236499007",  # Microalbuminuric nephropathy due to diabetes mellitus (disorder)
        "236500003",  # Proteinuric nephropathy due to diabetes mellitus (disorder)
        "237599002",  # Insulin treated type 2 diabetes mellitus (disorder)
        "237604008",  # Maturity onset diabetes of the young, type 2 (disorder)
        "237620003",  # Abnormal metabolic state due to diabetes mellitus (disorder)
        "237621004",  # Severe hyperglycemia due to diabetes mellitus (disorder)
        "237627000",  # Pregnancy and type 2 diabetes mellitus (disorder)
        "237633009",  # Hypoglycemia due to diabetes mellitus (disorder)
        "237635002",  # Nocturnal hypoglycemia due to diabetes mellitus (disorder)
        "237651005",  # Insulin resistance - type A (disorder)
        "237652003",  # Insulin resistance - type B (disorder)
        "238981002",  # Disorder of soft tissue due to diabetes mellitus (disorder)
        "238982009",  # Dermopathy due to diabetes mellitus (disorder)
        "238983004",  # Thick skin syndrome due to diabetes mellitus (disorder)
        "238984005",  # Rubeosis faciei due to diabetes mellitus (disorder)
        "24203005",  # Extreme insulin resistance with acanthosis nigricans, hirsutism AND abnormal insulin receptors (disorder)
        "243421000119104",  # Proteinuria due to type 1 diabetes mellitus (disorder)
        "25093002",  # Disorder of eye due to diabetes mellitus (disorder)
        "25412000",  # Microaneurysm of retinal artery due to diabetes mellitus (disorder)
        "26298008",  # Ketoacidotic coma due to diabetes mellitus (disorder)
        "28032008",  # Insulin dependent diabetes mellitus type IB (disorder)
        "28331000119107",  # Retinal edema due to type 2 diabetes mellitus (disorder)
        "290002008",  # Brittle type 1 diabetes mellitus (disorder)
        "309426007",  # Glomerulopathy due to diabetes mellitus (disorder)
        "310387003",  # Intracapillary glomerulosclerosis of kidney due to diabetes mellitus (disorder)
        "310505005",  # Hyperosmolar non-ketotic state due to diabetes mellitus (disorder)
        "311782002",  # Advanced retinal disease due to diabetes mellitus (disorder)
        "31211000119101",  # Peripheral angiopathy due to type 1 diabetes mellitus (disorder)
        "312903003",  # Mild nonproliferative retinopathy due to diabetes mellitus (disorder)
        "312904009",  # Moderate nonproliferative retinopathy due to diabetes mellitus (disorder)
        "312905005",  # Severe nonproliferative retinopathy due to diabetes mellitus (disorder)
        "312906006",  # Non-high-risk proliferative retinopathy due to diabetes mellitus (disorder)
        "312907002",  # High risk proliferative retinopathy due to diabetes mellitus (disorder)
        "312908007",  # Quiescent proliferative retinopathy due to diabetes mellitus (disorder)
        "312909004",  # Proliferative retinopathy with iris neovascularization due to diabetes mellitus (disorder)
        "312910009",  # Vitreous hemorrhage due to diabetes mellitus (disorder)
        "312912001",  # Macular edema due to diabetes mellitus (disorder)
        "31321000119102",  # Diabetes mellitus type 1 without retinopathy (disorder)
        "313436004",  # Type II diabetes mellitus without complication (disorder)
        "314010006",  # Diffuse exudative maculopathy due to diabetes mellitus (disorder)
        "314011005",  # Focal exudative maculopathy due to diabetes mellitus (disorder)
        "314014002",  # Ischemic maculopathy due to diabetes mellitus (disorder)
        "314015001",  # Mixed maculopathy due to diabetes mellitus (disorder)
        "314537004",  # Optic papillopathy due to diabetes mellitus (disorder)
        "314893005",  # Arthropathy due to type 1 diabetes mellitus (disorder)
        "314902007",  # Peripheral angiopathy due to type 2 diabetes mellitus (disorder)
        "314903002",  # Arthropathy due to type 2 diabetes mellitus (disorder)
        "35777006",  # Mononeuropathy multiplex due to diabetes mellitus (disorder)
        "359611005",  # Neuropathy with neurologic complication due to diabetes mellitus (disorder)
        "359642000",  # Diabetes mellitus type 2 in nonobese (disorder)
        "361216007",  # Femoral mononeuropathy due to diabetes mellitus (disorder)
        "367211000119103",  # Inflammatory dermatosis due to type 1 diabetes mellitus (disorder)
        "367991000119101",  # Hyperglycemia due to type 1 diabetes mellitus (disorder)
        "368051000119109",  # Hyperglycemia due to type 2 diabetes mellitus (disorder)
        "368101000119109",  # Periodontal disease co-occurrent and due to diabetes mellitus type 2 (disorder)
        "368521000119107",  # Disorder of nerve co-occurrent and due to type 1 diabetes mellitus (disorder)
        "368541000119101",  # Cheiroarthropathy due to diabetes mellitus type 1 (disorder)
        "368551000119104",  # Dyslipidemia due to type 1 diabetes mellitus (disorder)
        "368561000119102",  # Hyperosmolarity due to type 1 diabetes mellitus (disorder)
        "368581000119106",  # Neuropathy due to type 2 diabetes mellitus (disorder)
        "368591000119109",  # Cheiroarthropathy due to diabetes mellitus type 2 (disorder)
        "371087003",  # Ulcer of foot due to diabetes mellitus (disorder)
        "38046004",  # Diffuse glomerulosclerosis of kidney due to diabetes mellitus (disorder)
        "38205001",  # Diarrhea due to diabetes mellitus (disorder)
        "39058009",  # Lumbosacral radiculoplexus neuropathy due to diabetes mellitus (disorder)
        "390834004",  # Nonproliferative retinopathy due to diabetes mellitus (disorder)
        "39127005",  # Symmetric proximal motor neuropathy due to diabetes mellitus (disorder)
        "39181008",  # Radiculoplexus neuropathy due to diabetes mellitus (disorder)
        "395204000",  # Hyperosmolar non-ketotic state due to type 2 diabetes mellitus (disorder)
        "398140007",  # Post hypoglycemic hyperglycemia due to diabetes mellitus (disorder)
        "399862001",  # High risk proliferative retinopathy without macular edema due to diabetes mellitus (disorder)
        "399863006",  # Very severe nonproliferative retinopathy without macular edema due to diabetes mellitus (disorder)
        "399864000",  # Macular edema not clinically significant due to diabetes mellitus (disorder)
        "399865004",  # Very severe proliferative retinopathy due to diabetes mellitus (disorder)
        "399866003",  # Venous beading of retina due to diabetes mellitus (disorder)
        "399868002",  # Intraretinal microvascular anomaly due to diabetes mellitus (disorder)
        "399869005",  # High risk proliferative retinopathy not amenable to photocoagulation due to diabetes mellitus (disorder)
        "399870006",  # Non-high-risk proliferative retinopathy with no macular edema due to diabetes mellitus (disorder)
        "399871005",  # Visually threatening retinopathy due to diabetes mellitus (disorder)
        "399872003",  # Severe nonproliferative retinopathy with clinically significant macular edema due to diabetes mellitus (disorder)
        "399873008",  # Severe nonproliferative retinopathy without macular edema due to diabetes mellitus (disorder)
        "399874002",  # High risk proliferative retinopathy with clinically significant macula edema due to diabetes mellitus (disorder)
        "399875001",  # Non-high-risk proliferative retinopathy with clinically significant macular edema due to diabetes mellitus (disorder)
        "399876000",  # Very severe nonproliferative retinopathy due to diabetes mellitus (disorder)
        "399877009",  # Very severe nonproliferative retinopathy with clinically significant macular edema due to diabetes mellitus (disorder)
        "402864004",  # Wet gangrene of foot due to diabetes mellitus (disorder)
        "405749004",  # Newly diagnosed diabetes (disorder)
        "419100001",  # Infection of foot due to diabetes mellitus (disorder)
        "41911000119107",  # Glaucoma due to type 2 diabetes mellitus (disorder)
        "420270002",  # Ketoacidosis due to type 1 diabetes mellitus (disorder)
        "420279001",  # Renal disorder due to type 2 diabetes mellitus (disorder)
        "420422005",  # Ketoacidosis due to diabetes mellitus (disorder)
        "420436000",  # Mononeuropathy due to type 2 diabetes mellitus (disorder)
        "420486006",  # Exudative maculopathy due to type 1 diabetes mellitus (disorder)
        "420514000",  # Persistent proteinuria due to type 1 diabetes mellitus (disorder)
        "420662003",  # Coma due to diabetes mellitus (disorder)
        "420683009",  # Disorder of nervous system due to malnutrition related diabetes mellitus (disorder)
        "420715001",  # Persistent microalbuminuria due to type 2 diabetes mellitus (disorder)
        "420756003",  # Cataract of eye due to diabetes mellitus type 2 (disorder)
        "420789003",  # Retinopathy due to type 1 diabetes mellitus (disorder)
        "420825003",  # Gangrene due to type 1 diabetes mellitus (disorder)
        "420868002",  # Disorder due to type 1 diabetes mellitus (disorder)
        "420918009",  # Mononeuropathy due to type 1 diabetes mellitus (disorder)
        "420996007",  # Coma due to malnutrition-related diabetes mellitus (disorder)
        "421075007",  # Ketoacidotic coma due to type 1 diabetes mellitus (disorder)
        "421256007",  # Disorder of eye due to malnutrition related diabetes mellitus (disorder)
        "421305000",  # Persistent microalbuminuria due to type 1 diabetes mellitus (disorder)
        "421326000",  # Disorder of nervous system due to type 2 diabetes mellitus (disorder)
        "421365002",  # Peripheral circulatory disorder due to type 1 diabetes mellitus (disorder)
        "421437000",  # Hypoglycemic coma due to type 1 diabetes mellitus (disorder)
        "421468001",  # Disorder of nervous system due to type 1 diabetes mellitus (disorder)
        "421631007",  # Gangrene due to type 2 diabetes mellitus (disorder)
        "421725003",  # Hypoglycemic coma due to diabetes mellitus (disorder)
        "421750000",  # Ketoacidosis due to type 2 diabetes mellitus (disorder)
        "421779007",  # Exudative maculopathy due to type 2 diabetes mellitus (disorder)
        "421847006",  # Ketoacidotic coma due to type 2 diabetes mellitus (disorder)
        "421893009",  # Renal disorder due to type 1 diabetes mellitus (disorder)
        "421895002",  # Peripheral vascular disorder due to diabetes mellitus (disorder)
        "421920002",  # Cataract of eye due to diabetes mellitus type 1 (disorder)
        "421966007",  # Non-ketotic non-hyperosmolar coma due to diabetes mellitus (disorder)
        "421986006",  # Persistent proteinuria due to type 2 diabetes mellitus (disorder)
        "422014003",  # Disorder due to type 2 diabetes mellitus (disorder)
        "422034002",  # Retinopathy due to type 2 diabetes mellitus (disorder)
        "422088007",  # Disorder of nervous system due to diabetes mellitus (disorder)
        "422099009",  # Disorder of eye due to type 2 diabetes mellitus (disorder)
        "422126006",  # Hyperosmolar coma due to diabetes mellitus (disorder)
        "422166005",  # Peripheral circulatory disorder due to type 2 diabetes mellitus (disorder)
        "422183001",  # Skin ulcer due to diabetes mellitus (disorder)
        "422275004",  # Gangrene due to diabetes mellitus (disorder)
        "424736006",  # Peripheral neuropathy due to diabetes mellitus (disorder)
        "425455002",  # Glomerulonephritis due to diabetes mellitus (disorder)
        "426875007",  # Latent autoimmune diabetes mellitus in adult (disorder)
        "426907004",  # Small vessel disease due to type 1 diabetes mellitus (disorder)
        "427027005",  # Lumbosacral radiculoplexus neuropathy due to type 2 diabetes mellitus (disorder)
        "427134009",  # Small vessel disease due to type 2 diabetes mellitus (disorder)
        "427571000",  # Lumbosacral radiculoplexus neuropathy due to type 1 diabetes mellitus (disorder)
        "427943001",  # Ophthalmoplegia due to diabetes mellitus (disorder)
        "428007007",  # Erectile dysfunction due to type 2 diabetes mellitus (disorder)
        "428896009",  # Hyperosmolality due to uncontrolled type 1 diabetes mellitus (disorder)
        "43959009",  # Cataract of eye due to diabetes mellitus (disorder)
        "44054006",  # Diabetes mellitus type 2 (disorder)
        "441656006",  # Hyperglycemic crisis due to diabetes mellitus (disorder)
        "443694000",  # Uncontrolled type 2 diabetes mellitus (disorder)
        "444073006",  # Uncontrolled type 1 diabetes mellitus (disorder)
        "444074000",  # Well controlled type 1 diabetes mellitus (disorder)
        "444110003",  # Well controlled type 2 diabetes mellitus (disorder)
        "445170001",  # Macroalbuminuric nephropathy due to diabetes mellitus (disorder)
        "445353002",  # Brittle type 2 diabetes mellitus (disorder)
        "46635009",  # Diabetes mellitus type 1 (disorder)
        "4783006",  # Maternal diabetes mellitus with hypoglycemia affecting fetus OR newborn (disorder)
        "4855003",  # Retinopathy due to diabetes mellitus (disorder)
        "48951005",  # Bullous disease due to diabetes mellitus (disorder)
        "49455004",  # Polyneuropathy due to diabetes mellitus (disorder)
        "50620007",  # Autonomic neuropathy due to diabetes mellitus (disorder)
        "530558861000132104",  # Atypical diabetes mellitus (disorder)
        "59276001",  # Proliferative retinopathy due to diabetes mellitus (disorder)
        "60951000119105",  # Blindness due to type 2 diabetes mellitus (disorder)
        "609561005",  # Maturity-onset diabetes of the young (disorder)
        "609562003",  # Maturity onset diabetes of the young, type 1 (disorder)
        "609563008",  # Pre-existing diabetes mellitus in pregnancy (disorder)
        "609564002",  # Pre-existing type 1 diabetes mellitus in pregnancy (disorder)
        "609566000",  # Pregnancy and type 1 diabetes mellitus (disorder)
        "609567009",  # Pre-existing type 2 diabetes mellitus in pregnancy (disorder)
        "609570008",  # Maturity-onset diabetes of the young, type 3 (disorder)
        "609571007",  # Maturity-onset diabetes of the young, type 4 (disorder)
        "609572000",  # Maturity-onset diabetes of the young, type 5 (disorder)
        "609573005",  # Maturity-onset diabetes of the young, type 6 (disorder)
        "609574004",  # Maturity-onset diabetes of the young, type 7 (disorder)
        "609575003",  # Maturity-onset diabetes of the young, type 8 (disorder)
        "609576002",  # Maturity-onset diabetes of the young, type 9 (disorder)
        "609577006",  # Maturity-onset diabetes of the young, type 10 (disorder)
        "609578001",  # Maturity-onset diabetes of the young, type 11 (disorder)
        "60961000119107",  # Nonproliferative diabetic retinopathy due to type 1 diabetes mellitus (disorder)
        "60971000119101",  # Proliferative retinopathy due to type 1 diabetes mellitus (disorder)
        "60981000119103",  # Glaucoma due to diabetes mellitus type 1 (disorder)
        "60991000119100",  # Blindness due to type 1 diabetes mellitus (disorder)
        "62260007",  # Pretibial pigmental patches due to diabetes mellitus (disorder)
        "63510008",  # Nodular glomerulosclerosis of kidney due to diabetes mellitus (disorder)
        "691000119103",  # Erectile dysfunction due to type 1 diabetes mellitus (disorder)
        "701000119103",  # Mixed hyperlipidemia due to type 2 diabetes mellitus (disorder)
        "703136005",  # Diabetes mellitus in remission (disorder)
        "703137001",  # Type I diabetes mellitus in remission (disorder)
        "703138006",  # Type II diabetes mellitus in remission (disorder)
        "706891008",  # Neuropathy due to unstable diabetes mellitus type 1 (disorder)
        "706894000",  # Retinopathy due to unstable diabetes mellitus type 1 (disorder)
        "707221002",  # Glomerulosclerosis of kidney due to diabetes mellitus (disorder)
        "709147009",  # Gingivitis co-occurrent with diabetes mellitus (disorder)
        "711000119100",  # Chronic kidney disease stage 5 due to type 2 diabetes mellitus (disorder)
        "712882000",  # Autonomic neuropathy due to type 1 diabetes mellitus (disorder)
        "712883005",  # Autonomic neuropathy due to type 2 diabetes mellitus (disorder)
        "713457002",  # Neovascular glaucoma due to diabetes mellitus (disorder)
        "713702000",  # Gastroparesis due to type 1 diabetes mellitus (disorder)
        "713703005",  # Gastroparesis due to type 2 diabetes mellitus (disorder)
        "713704004",  # Gastroparesis due to diabetes mellitus (disorder)
        "713705003",  # Polyneuropathy due to type 1 diabetes mellitus (disorder)
        "713706002",  # Polyneuropathy due to type 2 diabetes mellitus (disorder)
        "71421000119105",  # Hypertension in chronic kidney disease due to type 2 diabetes mellitus (disorder)
        "71441000119104",  # Nephrotic syndrome due to type 2 diabetes mellitus (disorder)
        "716362006",  # Gingival disease co-occurrent with diabetes mellitus (disorder)
        "71701000119105",  # Hypertension in chronic kidney disease due to type 1 diabetes mellitus (disorder)
        "71721000119101",  # Nephrotic syndrome due to type 1 diabetes mellitus (disorder)
        "71771000119100",  # Neuropathic arthropathy due to type 1 diabetes mellitus (disorder)
        "71791000119104",  # Peripheral neuropathy due to type 1 diabetes mellitus (disorder)
        "719216001",  # Hypoglycemic coma due to type 2 diabetes mellitus (disorder)
        "72021000119109",  # Dermopathy due to type 1 diabetes mellitus (disorder)
        "72031000119107",  # Severe malnutrition due to type 1 diabetes mellitus (disorder)
        "72041000119103",  # Osteomyelitis due to type 1 diabetes mellitus (disorder)
        "72051000119101",  # Severe malnutrition due to type 2 diabetes mellitus (disorder)
        "72061000119104",  # Osteomyelitis due to type 2 diabetes mellitus (disorder)
        "721000119107",  # Chronic kidney disease stage 4 due to type 2 diabetes mellitus (disorder)
        "721111000124107",  # Diabetes mellitus type 1 stage 1 (disorder)
        "721121000124104",  # Diabetes mellitus type 1 stage 2 (disorder)
        "721201000124104",  # Diabetes mellitus type 1 stage 3 (disorder)
        "721283000",  # Acidosis due to type 1 diabetes mellitus (disorder)
        "721284006",  # Acidosis due to type 2 diabetes mellitus (disorder)
        "72141000119104",  # Chronic ulcer of skin due to type 1 diabetes mellitus (disorder)
        "722206009",  # Pancreatic hypoplasia, diabetes mellitus, congenital heart disease syndrome (disorder)
        "722454003",  # Intellectual disability, craniofacial dysmorphism, hypogonadism, diabetes mellitus syndrome (disorder)
        "723074006",  # Renal papillary necrosis due to diabetes mellitus (disorder)
        "724136006",  # Mastopathy due to diabetes mellitus (disorder)
        "724876003",  # Lesion of skin due to diabetes mellitus (disorder)
        "731000119105",  # Chronic kidney disease stage 3 due to type 2 diabetes mellitus (disorder)
        "73211009",  # Diabetes mellitus (disorder)
        "733072002",  # Alaninuria, microcephaly, dwarfism, enamel hypoplasia, diabetes mellitus syndrome (disorder)
        "735200002",  # Absence of lower limb due to diabetes mellitus (disorder)
        "735537007",  # Hyperosmolar hyperglycemic coma due to diabetes mellitus without ketoacidosis (disorder)
        "735538002",  # Lactic acidosis due to diabetes mellitus (disorder)
        "735539005",  # Metabolic acidosis due to diabetes mellitus (disorder)
        "739681000",  # Disorder of eye due to type 1 diabetes mellitus (disorder)
        "741000119101",  # Chronic kidney disease stage 2 due to type 2 diabetes mellitus (disorder)
        "74627003",  # Complication due to diabetes mellitus (disorder)
        "751000119104",  # Chronic kidney disease stage 1 due to type 2 diabetes mellitus (disorder)
        "761000119102",  # Dyslipidemia due to type 2 diabetes mellitus (disorder)
        "762489000",  # Acute complication due to diabetes mellitus (disorder)
        "76751001",  # Diabetes mellitus in mother complicating pregnancy, childbirth AND/OR puerperium (disorder)
        "768792007",  # Cataract of right eye due to diabetes mellitus (disorder)
        "768793002",  # Cataract of left eye due to diabetes mellitus (disorder)
        "768794008",  # Cataract of bilateral eyes due to diabetes mellitus (disorder)
        "768797001",  # Iritis of right eye due to diabetes mellitus (disorder)
        "768798006",  # Iritis of left eye due to diabetes mellitus (disorder)
        "768799003",  # Iritis of bilateral eyes due to diabetes mellitus (disorder)
        "769181007",  # Preproliferative retinopathy of right eye due to diabetes mellitus (disorder)
        "769182000",  # Preproliferative retinopathy of left eye due to diabetes mellitus (disorder)
        "769183005",  # Mild nonproliferative retinopathy of right eye due to diabetes mellitus (disorder)
        "769184004",  # Mild nonproliferative retinopathy of left eye due to diabetes mellitus (disorder)
        "769185003",  # Moderate nonproliferative retinopathy of right eye due to diabetes mellitus (disorder)
        "769186002",  # Moderate nonproliferative retinopathy of left eye due to diabetes mellitus (disorder)
        "769187006",  # Severe nonproliferative retinopathy of right eye due to diabetes mellitus (disorder)
        "769188001",  # Severe nonproliferative retinopathy of left eye due to diabetes mellitus (disorder)
        "769190000",  # Very severe nonproliferative retinopathy of right eye due to diabetes mellitus (disorder)
        "769191001",  # Very severe nonproliferative retinopathy of left eye due to diabetes mellitus (disorder)
        "769217008",  # Macular edema of right eye due to diabetes mellitus (disorder)
        "769218003",  # Macular edema of left eye due to diabetes mellitus (disorder)
        "769219006",  # Macular edema due to type 1 diabetes mellitus (disorder)
        "769220000",  # Macular edema due to type 2 diabetes mellitus (disorder)
        "769221001",  # Clinically significant macular edema of right eye due to diabetes mellitus (disorder)
        "769222008",  # Clinically significant macular edema of left eye due to diabetes mellitus (disorder)
        "769244003",  # Disorder of right macula due to diabetes mellitus (disorder)
        "769245002",  # Disorder of left macula due to diabetes mellitus (disorder)
        "770094004",  # Cervical radiculoplexus neuropathy due to diabetes mellitus (disorder)
        "770095003",  # Cranial nerve palsy due to diabetes mellitus (disorder)
        "770096002",  # Erectile dysfunction due to diabetes mellitus (disorder)
        "770097006",  # Clinically significant macular edema due to diabetes mellitus (disorder)
        "770098001",  # Cranial nerve palsy due to type 1 diabetes mellitus (disorder)
        "770323005",  # Retinal edema due to diabetes mellitus (disorder)
        "770324004",  # Ischemia of retina due to diabetes mellitus (disorder)
        "770361008",  # Vitreous hemorrhage of right eye due to diabetes mellitus (disorder)
        "770362001",  # Vitreous hemorrhage of left eye due to diabetes mellitus (disorder)
        "770581008",  # Microaneurysm of right retinal artery due to diabetes mellitus (disorder)
        "770582001",  # Microaneurysm of left retinal artery due to diabetes mellitus (disorder)
        "770599000",  # Venous beading of right retina due to diabetes mellitus (disorder)
        "770600002",  # Venous beading of left retina due to diabetes mellitus (disorder)
        "770765001",  # Proliferative retinopathy of right eye due to diabetes mellitus (disorder)
        "770766000",  # Proliferative retinopathy of left eye due to diabetes mellitus (disorder)
        "771000119108",  # Chronic kidney disease due to type 2 diabetes mellitus (disorder)
        "781000119106",  # Neuropathic arthropathy due to type 2 diabetes mellitus (disorder)
        "788878000",  # Cardiomyopathy due to diabetes mellitus (disorder)
        "789542009",  # Neuropathy due to type 1 diabetes mellitus (disorder)
        "789562001",  # Ulcer of heel due to diabetes mellitus (disorder)
        "789567007",  # Ulcer of heel due to type 2 diabetes mellitus (disorder)
        "789568002",  # Ulcer of midfoot due to diabetes mellitus (disorder)
        "789569005",  # Neuropathic ulcer of heel due to type 2 diabetes mellitus (disorder)
        "789570006",  # Neuropathic ulcer of midfoot due to type 2 diabetes mellitus (disorder)
        "789571005",  # Ulcer of heel due to type 1 diabetes mellitus (disorder)
        "789572003",  # Ulcer of midfoot due to type 1 diabetes mellitus (disorder)
        "789585000",  # Sensory polyneuropathy due to diabetes mellitus (disorder)
        "791000119109",  # Angina due to type 2 diabetes mellitus (disorder)
        "792926007",  # Armanni-Ebstein kidney due to diabetes mellitus (disorder)
        "79554005",  # Asymmetric proximal motor neuropathy due to diabetes mellitus (disorder)
        "81531005",  # Diabetes mellitus type 2 in obese (disorder)
        "816177009",  # Nonproliferative retinopathy of left eye due to diabetes mellitus (disorder)
        "816178004",  # Nonproliferative retinopathy of right eye due to diabetes mellitus (disorder)
        "816961009",  # Stable treated proliferative retinopathy of right eye due to diabetes mellitus (disorder)
        "816962002",  # Stable treated proliferative retinopathy of left eye due to diabetes mellitus (disorder)
        "81830002",  # Mononeuropathy simplex due to diabetes mellitus (disorder)
        "822995009",  # Hyperglycemia due to diabetes mellitus (disorder)
        "82541000119100",  # Traction detachment of retina due to type 2 diabetes mellitus (disorder)
        "82551000119103",  # Rubeosis iridis due to type 2 diabetes mellitus (disorder)
        "82571000119107",  # Traction detachment of retina due to type 1 diabetes mellitus (disorder)
        "82581000119105",  # Rubeosis iridis due to type 1 diabetes mellitus (disorder)
        "82980005",  # Anemia due to diabetes mellitus (disorder)
        "84361000119102",  # Insulin reactive hypoglycemia due to type 2 diabetes mellitus (disorder)
        "84371000119108",  # Hypoglycemia due to type 1 diabetes mellitus (disorder)
        "860721006",  # Disorder of macula of bilateral eyes due to diabetes mellitus (disorder)
        "860798008",  # Glaucoma due to diabetes mellitus (disorder)
        "860883001",  # Generalized autonomic neuropathy due to diabetes mellitus (disorder)
        "860977000",  # Ulcer of right foot due to diabetes mellitus (disorder)
        "860978005",  # Ulcer of left foot due to diabetes mellitus (disorder)
        "860979002",  # Chronic ulcer of right foot due to diabetes mellitus (disorder)
        "860980004",  # Chronic ulcer of left foot due to diabetes mellitus (disorder)
        "870420005",  # Severe nonproliferative retinopathy with venous beading of retina due to diabetes mellitus (disorder)
        "870421009",  # Cystoid macular edema due to diabetes mellitus (disorder)
        "870528001",  # Newly diagnosed type 1 diabetes mellitus (disorder)
        "870529009",  # Persistent macular edema due to diabetes mellitus (disorder)
        "871778008",  # Centrally involved macular edema due to diabetes mellitus (disorder)
        "871781003",  # Non centrally involved macular edema due to diabetes mellitus (disorder)
        "87441000119104",  # Ulcer of ankle due to type 2 diabetes mellitus (disorder)
        "87461000119100",  # Ulcer of forefoot due to type 2 diabetes mellitus (disorder)
        "87471000119106",  # Ulcer of ankle due to type 1 diabetes mellitus (disorder)
        "87491000119107",  # Ulcer of forefoot due to type 1 diabetes mellitus (disorder)
        "87921000119104",  # Cranial nerve palsy due to type 2 diabetes mellitus (disorder)
        "890171006",  # Ketosis-prone diabetes mellitus (disorder)
        "90721000119101",  # Chronic kidney disease stage 1 due to type 1 diabetes mellitus (disorder)
        "90731000119103",  # Chronic kidney disease stage 2 due to type 1 diabetes mellitus (disorder)
        "90741000119107",  # Chronic kidney disease stage 3 due to type 1 diabetes mellitus (disorder)
        "90751000119109",  # Chronic kidney disease stage 4 due to type 1 diabetes mellitus (disorder)
        "90761000119106",  # Chronic kidney disease stage 5 due to type 1 diabetes mellitus (disorder)
        "90771000119100",  # End stage renal disease on dialysis due to type 1 diabetes mellitus (disorder)
        "90781000119102",  # Microalbuminuria due to type 2 diabetes mellitus (disorder)
        "90791000119104",  # End stage renal disease on dialysis due to type 2 diabetes mellitus (disorder)
        "96441000119101",  # Chronic kidney disease due to type 1 diabetes mellitus (disorder)
        "97331000119101",  # Macular edema and retinopathy due to type 2 diabetes mellitus (disorder)
        "97341000119105",  # Proliferative retinopathy with retinal edema due to type 2 diabetes mellitus (disorder)
        "97621000119107",  # Stasis ulcer due to type 2 diabetes mellitus (disorder)
        "9859006",  # Acanthosis nigricans due to type 2 diabetes mellitus (disorder)
    }


class DiabeticRetinopathy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of  diabetic retinopathy.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of diabetic retinopathy.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'

    ** Used in:** CMS131v14
    """
    VALUE_SET_NAME = "Diabetic Retinopathy"
    OID = "2.16.840.1.113883.3.526.3.327"
    DEFINITION_VERSION = "20210218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"


    ICD10CM = {
        "E08311",   # Diabetes mellitus due to underlying condition with unspecified diabetic retinopathy with macular edema
        "E08319",   # Diabetes mellitus due to underlying condition with unspecified diabetic retinopathy without macular edema
        "E083211",  # Diabetes mellitus due to underlying condition with mild nonproliferative diabetic retinopathy with macular edema, right eye
        "E083212",  # Diabetes mellitus due to underlying condition with mild nonproliferative diabetic retinopathy with macular edema, left eye
        "E083213",  # Diabetes mellitus due to underlying condition with mild nonproliferative diabetic retinopathy with macular edema, bilateral
        "E083291",  # Diabetes mellitus due to underlying condition with mild nonproliferative diabetic retinopathy without macular edema, right eye
        "E083292",  # Diabetes mellitus due to underlying condition with mild nonproliferative diabetic retinopathy without macular edema, left eye
        "E083293",  # Diabetes mellitus due to underlying condition with mild nonproliferative diabetic retinopathy without macular edema, bilateral
        "E083311",  # Diabetes mellitus due to underlying condition with moderate nonproliferative diabetic retinopathy with macular edema, right eye
        "E083312",  # Diabetes mellitus due to underlying condition with moderate nonproliferative diabetic retinopathy with macular edema, left eye
        "E083313",  # Diabetes mellitus due to underlying condition with moderate nonproliferative diabetic retinopathy with macular edema, bilateral
        "E083391",  # Diabetes mellitus due to underlying condition with moderate nonproliferative diabetic retinopathy without macular edema, right eye
        "E083392",  # Diabetes mellitus due to underlying condition with moderate nonproliferative diabetic retinopathy without macular edema, left eye
        "E083393",  # Diabetes mellitus due to underlying condition with moderate nonproliferative diabetic retinopathy without macular edema, bilateral
        "E083411",  # Diabetes mellitus due to underlying condition with severe nonproliferative diabetic retinopathy with macular edema, right eye
        "E083412",  # Diabetes mellitus due to underlying condition with severe nonproliferative diabetic retinopathy with macular edema, left eye
        "E083413",  # Diabetes mellitus due to underlying condition with severe nonproliferative diabetic retinopathy with macular edema, bilateral
        "E083491",  # Diabetes mellitus due to underlying condition with severe nonproliferative diabetic retinopathy without macular edema, right eye
        "E083492",  # Diabetes mellitus due to underlying condition with severe nonproliferative diabetic retinopathy without macular edema, left eye
        "E083493",  # Diabetes mellitus due to underlying condition with severe nonproliferative diabetic retinopathy without macular edema, bilateral
        "E083511",  # Diabetes mellitus due to underlying condition with proliferative diabetic retinopathy with macular edema, right eye
        "E083512",  # Diabetes mellitus due to underlying condition with proliferative diabetic retinopathy with macular edema, left eye
        "E083513",  # Diabetes mellitus due to underlying condition with proliferative diabetic retinopathy with macular edema, bilateral
        "E083521",  # Diabetes mellitus due to underlying condition with proliferative diabetic retinopathy with traction retinal detachment involving the macula, right eye
        "E083522",  # Diabetes mellitus due to underlying condition with proliferative diabetic retinopathy with traction retinal detachment involving the macula, left eye
        "E083523",  # Diabetes mellitus due to underlying condition with proliferative diabetic retinopathy with traction retinal detachment involving the macula, bilateral
        "E083531",  # Diabetes mellitus due to underlying condition with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, right eye
        "E083532",  # Diabetes mellitus due to underlying condition with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, left eye
        "E083533",  # Diabetes mellitus due to underlying condition with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, bilateral
        "E083541",  # Diabetes mellitus due to underlying condition with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, right eye
        "E083542",  # Diabetes mellitus due to underlying condition with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, left eye
        "E083543",  # Diabetes mellitus due to underlying condition with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, bilateral
        "E083551",  # Diabetes mellitus due to underlying condition with stable proliferative diabetic retinopathy, right eye
        "E083552",  # Diabetes mellitus due to underlying condition with stable proliferative diabetic retinopathy, left eye
        "E083553",  # Diabetes mellitus due to underlying condition with stable proliferative diabetic retinopathy, bilateral
        "E083591",  # Diabetes mellitus due to underlying condition with proliferative diabetic retinopathy without macular edema, right eye
        "E083592",  # Diabetes mellitus due to underlying condition with proliferative diabetic retinopathy without macular edema, left eye
        "E083593",  # Diabetes mellitus due to underlying condition with proliferative diabetic retinopathy without macular edema, bilateral
        "E09311",   # Drug or chemical induced diabetes mellitus with unspecified diabetic retinopathy with macular edema
        "E09319",   # Drug or chemical induced diabetes mellitus with unspecified diabetic retinopathy without macular edema
        "E093211",  # Drug or chemical induced diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, right eye
        "E093212",  # Drug or chemical induced diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, left eye
        "E093213",  # Drug or chemical induced diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, bilateral
        "E093291",  # Drug or chemical induced diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, right eye
        "E093292",  # Drug or chemical induced diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, left eye
        "E093293",  # Drug or chemical induced diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, bilateral
        "E093311",  # Drug or chemical induced diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, right eye
        "E093312",  # Drug or chemical induced diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, left eye
        "E093313",  # Drug or chemical induced diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, bilateral
        "E093391",  # Drug or chemical induced diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, right eye
        "E093392",  # Drug or chemical induced diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, left eye
        "E093393",  # Drug or chemical induced diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, bilateral
        "E093411",  # Drug or chemical induced diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, right eye
        "E093412",  # Drug or chemical induced diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, left eye
        "E093413",  # Drug or chemical induced diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, bilateral
        "E093491",  # Drug or chemical induced diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, right eye
        "E093492",  # Drug or chemical induced diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, left eye
        "E093493",  # Drug or chemical induced diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, bilateral
        "E093511",  # Drug or chemical induced diabetes mellitus with proliferative diabetic retinopathy with macular edema, right eye
        "E093512",  # Drug or chemical induced diabetes mellitus with proliferative diabetic retinopathy with macular edema, left eye
        "E093513",  # Drug or chemical induced diabetes mellitus with proliferative diabetic retinopathy with macular edema, bilateral
        "E093521",  # Drug or chemical induced diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, right eye
        "E093522",  # Drug or chemical induced diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, left eye
        "E093523",  # Drug or chemical induced diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, bilateral
        "E093531",  # Drug or chemical induced diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, right eye
        "E093532",  # Drug or chemical induced diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, left eye
        "E093533",  # Drug or chemical induced diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, bilateral
        "E093541",  # Drug or chemical induced diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, right eye
        "E093542",  # Drug or chemical induced diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, left eye
        "E093543",  # Drug or chemical induced diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, bilateral
        "E093551",  # Drug or chemical induced diabetes mellitus with stable proliferative diabetic retinopathy, right eye
        "E093552",  # Drug or chemical induced diabetes mellitus with stable proliferative diabetic retinopathy, left eye
        "E093553",  # Drug or chemical induced diabetes mellitus with stable proliferative diabetic retinopathy, bilateral
        "E093591",  # Drug or chemical induced diabetes mellitus with proliferative diabetic retinopathy without macular edema, right eye
        "E093592",  # Drug or chemical induced diabetes mellitus with proliferative diabetic retinopathy without macular edema, left eye
        "E093593",  # Drug or chemical induced diabetes mellitus with proliferative diabetic retinopathy without macular edema, bilateral
        "E10311",   # Type 1 diabetes mellitus with unspecified diabetic retinopathy with macular edema
        "E10319",   # Type 1 diabetes mellitus with unspecified diabetic retinopathy without macular edema
        "E103211",  # Type 1 diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, right eye
        "E103212",  # Type 1 diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, left eye
        "E103213",  # Type 1 diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, bilateral
        "E103291",  # Type 1 diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, right eye
        "E103292",  # Type 1 diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, left eye
        "E103293",  # Type 1 diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, bilateral
        "E103311",  # Type 1 diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, right eye
        "E103312",  # Type 1 diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, left eye
        "E103313",  # Type 1 diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, bilateral
        "E103391",  # Type 1 diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, right eye
        "E103392",  # Type 1 diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, left eye
        "E103393",  # Type 1 diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, bilateral
        "E103411",  # Type 1 diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, right eye
        "E103412",  # Type 1 diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, left eye
        "E103413",  # Type 1 diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, bilateral
        "E103491",  # Type 1 diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, right eye
        "E103492",  # Type 1 diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, left eye
        "E103493",  # Type 1 diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, bilateral
        "E103511",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with macular edema, right eye
        "E103512",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with macular edema, left eye
        "E103513",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with macular edema, bilateral
        "E103521",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, right eye
        "E103522",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, left eye
        "E103523",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, bilateral
        "E103531",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, right eye
        "E103532",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, left eye
        "E103533",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, bilateral
        "E103541",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, right eye
        "E103542",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, left eye
        "E103543",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, bilateral
        "E103551",  # Type 1 diabetes mellitus with stable proliferative diabetic retinopathy, right eye
        "E103552",  # Type 1 diabetes mellitus with stable proliferative diabetic retinopathy, left eye
        "E103553",  # Type 1 diabetes mellitus with stable proliferative diabetic retinopathy, bilateral
        "E103591",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy without macular edema, right eye
        "E103592",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy without macular edema, left eye
        "E103593",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy without macular edema, bilateral
        "E11311",   # Type 2 diabetes mellitus with unspecified diabetic retinopathy with macular edema
        "E11319",   # Type 2 diabetes mellitus with unspecified diabetic retinopathy without macular edema
        "E113211",  # Type 2 diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, right eye
        "E113212",  # Type 2 diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, left eye
        "E113213",  # Type 2 diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, bilateral
        "E113291",  # Type 2 diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, right eye
        "E113292",  # Type 2 diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, left eye
        "E113293",  # Type 2 diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, bilateral
        "E113311",  # Type 2 diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, right eye
        "E113312",  # Type 2 diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, left eye
        "E113313",  # Type 2 diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, bilateral
        "E113391",  # Type 2 diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, right eye
        "E113392",  # Type 2 diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, left eye
        "E113393",  # Type 2 diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, bilateral
        "E113411",  # Type 2 diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, right eye
        "E113412",  # Type 2 diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, left eye
        "E113413",  # Type 2 diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, bilateral
        "E113491",  # Type 2 diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, right eye
        "E113492",  # Type 2 diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, left eye
        "E113493",  # Type 2 diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, bilateral
        "E113511",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with macular edema, right eye
        "E113512",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with macular edema, left eye
        "E113513",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with macular edema, bilateral
        "E113521",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, right eye
        "E113522",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, left eye
        "E113523",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, bilateral
        "E113531",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, right eye
        "E113532",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, left eye
        "E113533",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, bilateral
        "E113541",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, right eye
        "E113542",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, left eye
        "E113543",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, bilateral
        "E113551",  # Type 2 diabetes mellitus with stable proliferative diabetic retinopathy, right eye
        "E113552",  # Type 2 diabetes mellitus with stable proliferative diabetic retinopathy, left eye
        "E113553",  # Type 2 diabetes mellitus with stable proliferative diabetic retinopathy, bilateral
        "E113591",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy without macular edema, right eye
        "E113592",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy without macular edema, left eye
        "E113593",   # Type 2 diabetes mellitus with proliferative diabetic retinopathy without macular edema, bilateral
        "E13311",    # Other specified diabetes mellitus with unspecified diabetic retinopathy with macular edema
        "E13319",    # Other specified diabetes mellitus with unspecified diabetic retinopathy without macular edema
        "E133211",   # Other specified diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, right eye
        "E133212",   # Other specified diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, left eye
        "E133213",   # Other specified diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, bilateral
        "E133291",   # Other specified diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, right eye
        "E133292",   # Other specified diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, left eye
        "E133293",   # Other specified diabetes mellitus with mild nonproliferative diabetic retinopathy without macular edema, bilateral
        "E133311",   # Other specified diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, right eye
        "E133312",   # Other specified diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, left eye
        "E133313",   # Other specified diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, bilateral
        "E133391",   # Other specified diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, right eye
        "E133392",   # Other specified diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, left eye
        "E133393",   # Other specified diabetes mellitus with moderate nonproliferative diabetic retinopathy without macular edema, bilateral
        "E133411",   # Other specified diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, right eye
        "E133412",   # Other specified diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, left eye
        "E133413",   # Other specified diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, bilateral
        "E133491",   # Other specified diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, right eye
        "E133492",   # Other specified diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, left eye
        "E133493",   # Other specified diabetes mellitus with severe nonproliferative diabetic retinopathy without macular edema, bilateral
        "E133511",   # Other specified diabetes mellitus with proliferative diabetic retinopathy with macular edema, right eye
        "E133512",   # Other specified diabetes mellitus with proliferative diabetic retinopathy with macular edema, left eye
        "E133513",   # Other specified diabetes mellitus with proliferative diabetic retinopathy with macular edema, bilateral
        "E133521",   # Other specified diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, right eye
        "E133522",   # Other specified diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, left eye
        "E133523",   # Other specified diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, bilateral
        "E133531",   # Other specified diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, right eye
        "E133532",   # Other specified diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, left eye
        "E133533",   # Other specified diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment not involving the macula, bilateral
        "E133541",   # Other specified diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, right eye
        "E133542",   # Other specified diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, left eye
        "E133543",   # Other specified diabetes mellitus with proliferative diabetic retinopathy with combined traction retinal detachment and rhegmatogenous retinal detachment, bilateral
        "E133551",   # Other specified diabetes mellitus with stable proliferative diabetic retinopathy, right eye
        "E133552",   # Other specified diabetes mellitus with stable proliferative diabetic retinopathy, left eye
        "E133553",   # Other specified diabetes mellitus with stable proliferative diabetic retinopathy, bilateral
        "E133591",   # Other specified diabetes mellitus with proliferative diabetic retinopathy without macular edema, right eye
        "E133592",   # Other specified diabetes mellitus with proliferative diabetic retinopathy without macular edema, left eye
        "E133593",   # Other specified diabetes mellitus with proliferative diabetic retinopathy without macular edema, bilateral
    }

    SNOMEDCT = {
        "1306699009",  # Stable treated proliferative retinopathy due to diabetes mellitus (disorder)
        "1306700005",  # Stable treated proliferative retinopathy of bilateral eyes due to diabetes mellitus (disorder)
        "193349004",   # Preproliferative retinopathy due to diabetes mellitus (disorder)
        "193350004",   # Advanced maculopathy due to diabetes mellitus (disorder)
        "232020009",   # Disorder of macula due to diabetes mellitus (disorder)
        "232021008",   # Proliferative retinopathy with optic disc neovascularization due to diabetes mellitus (disorder)
        "232022001",   # Proliferative retinopathy with neovascularization elsewhere than the optic disc due to diabetes mellitus (disorder)
        "232023006",   # Traction detachment of retina due to diabetes mellitus (disorder)
        "25412000",    # Microaneurysm of retinal artery due to diabetes mellitus (disorder)
        "311782002",   # Advanced retinal disease due to diabetes mellitus (disorder)
        "312903003",   # Mild nonproliferative retinopathy due to diabetes mellitus (disorder)
        "312904009",   # Moderate nonproliferative retinopathy due to diabetes mellitus (disorder)
        "312905005",   # Severe nonproliferative retinopathy due to diabetes mellitus (disorder)
        "312906006",   # Non-high-risk proliferative retinopathy due to diabetes mellitus (disorder)
        "312907002",   # High risk proliferative retinopathy due to diabetes mellitus (disorder)
        "312908007",   # Quiescent proliferative retinopathy due to diabetes mellitus (disorder)
        "312909004",   # Proliferative retinopathy with iris neovascularization due to diabetes mellitus (disorder)
        "312912001",   # Macular edema due to diabetes mellitus (disorder)
        "314010006",   # Diffuse exudative maculopathy due to diabetes mellitus (disorder)
        "314011005",   # Focal exudative maculopathy due to diabetes mellitus (disorder)
        "314014002",   # Ischemic maculopathy due to diabetes mellitus (disorder)
        "314015001",   # Mixed maculopathy due to diabetes mellitus (disorder)
        "390834004",   # Nonproliferative retinopathy due to diabetes mellitus (disorder)
        "399862001",   # High risk proliferative retinopathy without macular edema due to diabetes mellitus (disorder)
        "399863006",   # Very severe nonproliferative retinopathy without macular edema due to diabetes mellitus (disorder)
        "399864000",   # Macular edema not clinically significant due to diabetes mellitus (disorder)
        "399865004",   # Very severe proliferative retinopathy due to diabetes mellitus (disorder)
        "399866003",   # Venous beading of retina due to diabetes mellitus (disorder)
        "399868002",   # Intraretinal microvascular anomaly due to diabetes mellitus (disorder)
        "399869005",   # High risk proliferative retinopathy not amenable to photocoagulation due to diabetes mellitus (disorder)
        "399870006",   # Non-high-risk proliferative retinopathy with no macular edema due to diabetes mellitus (disorder)
        "399871005",   # Visually threatening retinopathy due to diabetes mellitus (disorder)
        "399872003",   # Severe nonproliferative retinopathy with clinically significant macular edema due to diabetes mellitus (disorder)
        "399873008",   # Severe nonproliferative retinopathy without macular edema due to diabetes mellitus (disorder)
        "399874002",   # High risk proliferative retinopathy with clinically significant macula edema due to diabetes mellitus (disorder)
        "399875001",   # Non-high-risk proliferative retinopathy with clinically significant macular edema due to diabetes mellitus (disorder)
        "399876000",   # Very severe nonproliferative retinopathy due to diabetes mellitus (disorder)
        "399877009",   # Very severe nonproliferative retinopathy with clinically significant macular edema due to diabetes mellitus (disorder)
        "420486006",   # Exudative maculopathy due to type 1 diabetes mellitus (disorder)
        "420789003",   # Retinopathy due to type 1 diabetes mellitus (disorder)
        "421779007",   # Exudative maculopathy due to type 2 diabetes mellitus (disorder)
        "422034002",   # Retinopathy due to type 2 diabetes mellitus (disorder)
        "4855003",     # Retinopathy due to diabetes mellitus (disorder)
        "59276001",    # Proliferative retinopathy due to diabetes mellitus (disorder)
        "870420005",   # Severe nonproliferative retinopathy with venous beading of retina due to diabetes mellitus (disorder)
    }


class FrailtyDiagnosis(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of potential indicators of frailty.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent that a diagnosis of potential frailty when coupled with older age: dependence on durable medical equipment for long-term use, limitation of activities of daily living due to disability, need for assistance with care, pressure ulcers, and falls.

    **Exclusion Criteria:** Exclude concepts that indicate potential frailty when associated with pregnancy, fetal, juvenile, childhood or neonatal conditions; codes that indicate temporary need of durable medical equipment; codes that indicate accidental falls caused by another person, being pushed/shoved/collided into, falls on ice due to ice skates, falls due to impact against a baby buggy/shopping cart/wheelbarrow.

    ** Used in:** CMS131v14
    """

    VALUE_SET_NAME = "Frailty Diagnosis"
    OID = "2.16.840.1.113883.3.464.1003.113.12.1074"
    DEFINITION_VERSION = "20190315"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "L89000",  # Pressure ulcer of unspecified elbow, unstageable
        "L89001",  # Pressure ulcer of unspecified elbow, stage 1
        "L89002",  # Pressure ulcer of unspecified elbow, stage 2
        "L89003",  # Pressure ulcer of unspecified elbow, stage 3
        "L89004",  # Pressure ulcer of unspecified elbow, stage 4
        "L89006",  # Pressure-induced deep tissue damage of unspecified elbow
        "L89009",  # Pressure ulcer of unspecified elbow, unspecified stage
        "L89010",  # Pressure ulcer of right elbow, unstageable
        "L89011",  # Pressure ulcer of right elbow, stage 1
        "L89012",  # Pressure ulcer of right elbow, stage 2
        "L89013",  # Pressure ulcer of right elbow, stage 3
        "L89014",  # Pressure ulcer of right elbow, stage 4
        "L89016",  # Pressure-induced deep tissue damage of right elbow
        "L89019",  # Pressure ulcer of right elbow, unspecified stage
        "L89020",  # Pressure ulcer of left elbow, unstageable
        "L89021",  # Pressure ulcer of left elbow, stage 1
        "L89022",  # Pressure ulcer of left elbow, stage 2
        "L89023",  # Pressure ulcer of left elbow, stage 3
        "L89024",  # Pressure ulcer of left elbow, stage 4
        "L89026",  # Pressure-induced deep tissue damage of left elbow
        "L89029",  # Pressure ulcer of left elbow, unspecified stage
        "L89100",  # Pressure ulcer of unspecified part of back, unstageable
        "L89101",  # Pressure ulcer of unspecified part of back, stage 1
        "L89102",  # Pressure ulcer of unspecified part of back, stage 2
        "L89103",  # Pressure ulcer of unspecified part of back, stage 3
        "L89104",  # Pressure ulcer of unspecified part of back, stage 4
        "L89106",  # Pressure-induced deep tissue damage of unspecified part of back
        "L89109",  # Pressure ulcer of unspecified part of back, unspecified stage
        "L89110",  # Pressure ulcer of right upper back, unstageable
        "L89111",  # Pressure ulcer of right upper back, stage 1
        "L89112",  # Pressure ulcer of right upper back, stage 2
        "L89113",  # Pressure ulcer of right upper back, stage 3
        "L89114",  # Pressure ulcer of right upper back, stage 4
        "L89116",  # Pressure-induced deep tissue damage of right upper back
        "L89119",  # Pressure ulcer of right upper back, unspecified stage
        "L89120",  # Pressure ulcer of left upper back, unstageable
        "L89121",  # Pressure ulcer of left upper back, stage 1
        "L89122",  # Pressure ulcer of left upper back, stage 2
        "L89123",  # Pressure ulcer of left upper back, stage 3
        "L89124",  # Pressure ulcer of left upper back, stage 4
        "L89126",  # Pressure-induced deep tissue damage of left upper back
        "L89129",  # Pressure ulcer of left upper back, unspecified stage
        "L89130",  # Pressure ulcer of right lower back, unstageable
        "L89131",  # Pressure ulcer of right lower back, stage 1
        "L89132",  # Pressure ulcer of right lower back, stage 2
        "L89133",  # Pressure ulcer of right lower back, stage 3
        "L89134",  # Pressure ulcer of right lower back, stage 4
        "L89136",  # Pressure-induced deep tissue damage of right lower back
        "L89139",  # Pressure ulcer of right lower back, unspecified stage
        "L89140",  # Pressure ulcer of left lower back, unstageable
        "L89141",  # Pressure ulcer of left lower back, stage 1
        "L89142",  # Pressure ulcer of left lower back, stage 2
        "L89143",  # Pressure ulcer of left lower back, stage 3
        "L89144",  # Pressure ulcer of left lower back, stage 4
        "L89146",  # Pressure-induced deep tissue damage of left lower back
        "L89149",  # Pressure ulcer of left lower back, unspecified stage
        "L89150",  # Pressure ulcer of sacral region, unstageable
        "L89151",  # Pressure ulcer of sacral region, stage 1
        "L89152",  # Pressure ulcer of sacral region, stage 2
        "L89153",  # Pressure ulcer of sacral region, stage 3
        "L89154",  # Pressure ulcer of sacral region, stage 4
        "L89156",  # Pressure-induced deep tissue damage of sacral region
        "L89159",  # Pressure ulcer of sacral region, unspecified stage
        "L89200",  # Pressure ulcer of unspecified hip, unstageable
        "L89201",  # Pressure ulcer of unspecified hip, stage 1
        "L89202",  # Pressure ulcer of unspecified hip, stage 2
        "L89203",  # Pressure ulcer of unspecified hip, stage 3
        "L89204",  # Pressure ulcer of unspecified hip, stage 4
        "L89206",  # Pressure-induced deep tissue damage of unspecified hip
        "L89209",  # Pressure ulcer of unspecified hip, unspecified stage
        "L89210",  # Pressure ulcer of right hip, unstageable
        "L89211",  # Pressure ulcer of right hip, stage 1
        "L89212",  # Pressure ulcer of right hip, stage 2
        "L89213",  # Pressure ulcer of right hip, stage 3
        "L89214",  # Pressure ulcer of right hip, stage 4
        "L89216",  # Pressure-induced deep tissue damage of right hip
        "L89219",  # Pressure ulcer of right hip, unspecified stage
        "L89220",  # Pressure ulcer of left hip, unstageable
        "L89221",  # Pressure ulcer of left hip, stage 1
        "L89222",  # Pressure ulcer of left hip, stage 2
        "L89223",  # Pressure ulcer of left hip, stage 3
        "L89224",  # Pressure ulcer of left hip, stage 4
        "L89226",  # Pressure-induced deep tissue damage of left hip
        "L89229",  # Pressure ulcer of left hip, unspecified stage
        "L89300",  # Pressure ulcer of unspecified buttock, unstageable
        "L89301",  # Pressure ulcer of unspecified buttock, stage 1
        "L89302",  # Pressure ulcer of unspecified buttock, stage 2
        "L89303",  # Pressure ulcer of unspecified buttock, stage 3
        "L89304",  # Pressure ulcer of unspecified buttock, stage 4
        "L89306",  # Pressure-induced deep tissue damage of unspecified buttock
        "L89309",  # Pressure ulcer of unspecified buttock, unspecified stage
        "L89310",  # Pressure ulcer of right buttock, unstageable
        "L89311",  # Pressure ulcer of right buttock, stage 1
        "L89312",  # Pressure ulcer of right buttock, stage 2
        "L89313",  # Pressure ulcer of right buttock, stage 3
        "L89314",  # Pressure ulcer of right buttock, stage 4
        "L89316",  # Pressure-induced deep tissue damage of right buttock
        "L89319",  # Pressure ulcer of right buttock, unspecified stage
        "L89320",  # Pressure ulcer of left buttock, unstageable
        "L89321",  # Pressure ulcer of left buttock, stage 1
        "L89322",  # Pressure ulcer of left buttock, stage 2
        "L89323",  # Pressure ulcer of left buttock, stage 3
        "L89324",  # Pressure ulcer of left buttock, stage 4
        "L89326",  # Pressure-induced deep tissue damage of left buttock
        "L89329",  # Pressure ulcer of left buttock, unspecified stage
        "L8940",   # Pressure ulcer of contiguous site of back, buttock and hip, unspecified stage
        "L8941",   # Pressure ulcer of contiguous site of back, buttock and hip, stage 1
        "L8942",   # Pressure ulcer of contiguous site of back, buttock and hip, stage 2
        "L8943",   # Pressure ulcer of contiguous site of back, buttock and hip, stage 3
        "L8944",   # Pressure ulcer of contiguous site of back, buttock and hip, stage 4
        "L8945",   # Pressure ulcer of contiguous site of back, buttock and hip, unstageable
        "L8946",   # Pressure-induced deep tissue damage of contiguous site of back, buttock and hip
        "L89500",  # Pressure ulcer of unspecified ankle, unstageable
        "L89501",  # Pressure ulcer of unspecified ankle, stage 1
        "L89502",  # Pressure ulcer of unspecified ankle, stage 2
        "L89503",  # Pressure ulcer of unspecified ankle, stage 3
        "L89504",  # Pressure ulcer of unspecified ankle, stage 4
        "L89506",  # Pressure-induced deep tissue damage of unspecified ankle
        "L89509",  # Pressure ulcer of unspecified ankle, unspecified stage
        "L89510",  # Pressure ulcer of right ankle, unstageable
        "L89511",  # Pressure ulcer of right ankle, stage 1
        "L89512",  # Pressure ulcer of right ankle, stage 2
        "L89513",  # Pressure ulcer of right ankle, stage 3
        "L89514",  # Pressure ulcer of right ankle, stage 4
        "L89516",  # Pressure-induced deep tissue damage of right ankle
        "L89519",  # Pressure ulcer of right ankle, unspecified stage
        "L89520",  # Pressure ulcer of left ankle, unstageable
        "L89521",  # Pressure ulcer of left ankle, stage 1
        "L89522",  # Pressure ulcer of left ankle, stage 2
        "L89523",  # Pressure ulcer of left ankle, stage 3
        "L89524",  # Pressure ulcer of left ankle, stage 4
        "L89526",  # Pressure-induced deep tissue damage of left ankle
        "L89529",   # Pressure ulcer of left ankle, unspecified stage
        "L89600",   # Pressure ulcer of unspecified heel, unstageable
        "L89601",   # Pressure ulcer of unspecified heel, stage 1
        "L89602",   # Pressure ulcer of unspecified heel, stage 2
        "L89603",   # Pressure ulcer of unspecified heel, stage 3
        "L89604",   # Pressure ulcer of unspecified heel, stage 4
        "L89606",   # Pressure-induced deep tissue damage of unspecified heel
        "L89609",   # Pressure ulcer of unspecified heel, unspecified stage
        "L89610",   # Pressure ulcer of right heel, unstageable
        "L89611",   # Pressure ulcer of right heel, stage 1
        "L89612",   # Pressure ulcer of right heel, stage 2
        "L89613",   # Pressure ulcer of right heel, stage 3
        "L89614",   # Pressure ulcer of right heel, stage 4
        "L89616",   # Pressure-induced deep tissue damage of right heel
        "L89619",   # Pressure ulcer of right heel, unspecified stage
        "L89620",   # Pressure ulcer of left heel, unstageable
        "L89621",   # Pressure ulcer of left heel, stage 1
        "L89622",   # Pressure ulcer of left heel, stage 2
        "L89623",   # Pressure ulcer of left heel, stage 3
        "L89624",   # Pressure ulcer of left heel, stage 4
        "L89626",   # Pressure-induced deep tissue damage of left heel
        "L89629",   # Pressure ulcer of left heel, unspecified stage
        "L89810",   # Pressure ulcer of head, unstageable
        "L89811",   # Pressure ulcer of head, stage 1
        "L89812",   # Pressure ulcer of head, stage 2
        "L89813",   # Pressure ulcer of head, stage 3
        "L89814",   # Pressure ulcer of head, stage 4
        "L89816",   # Pressure-induced deep tissue damage of head
        "L89819",   # Pressure ulcer of head, unspecified stage
        "L89890",   # Pressure ulcer of other site, unstageable
        "L89891",   # Pressure ulcer of other site, stage 1
        "L89892",   # Pressure ulcer of other site, stage 2
        "L89893",   # Pressure ulcer of other site, stage 3
        "L89894",   # Pressure ulcer of other site, stage 4
        "L89896",   # Pressure-induced deep tissue damage of other site
        "L89899",   # Pressure ulcer of other site, unspecified stage
        "L8990",    # Pressure ulcer of unspecified site, unspecified stage
        "L8991",    # Pressure ulcer of unspecified site, stage 1
        "L8992",    # Pressure ulcer of unspecified site, stage 2
        "L8993",    # Pressure ulcer of unspecified site, stage 3
        "L8994",    # Pressure ulcer of unspecified site, stage 4
        "L8995",    # Pressure ulcer of unspecified site, unstageable
        "L8996",    # Pressure-induced deep tissue damage of unspecified site
        "M6250",    # Muscle wasting and atrophy, not elsewhere classified, unspecified site
        "M6281",    # Muscle weakness (generalized)
        "M6284",    # Sarcopenia
        "R296",     # Repeated falls
        "W010XXA",  # Fall on same level from slipping, tripping and stumbling without subsequent striking against object, initial encounter
        "W010XXD",  # Fall on same level from slipping, tripping and stumbling without subsequent striking against object, subsequent encounter
        "W010XXS",  # Fall on same level from slipping, tripping and stumbling without subsequent striking against object, sequela
        "W0110XA",   # Fall on same level from slipping, tripping and stumbling with subsequent striking against unspecified object, initial encounter
        "W0110XD",   # Fall on same level from slipping, tripping and stumbling with subsequent striking against unspecified object, subsequent encounter
        "W0110XS",   # Fall on same level from slipping, tripping and stumbling with subsequent striking against unspecified object, sequela
        "W01110A",   # Fall on same level from slipping, tripping and stumbling with subsequent striking against sharp glass, initial encounter
        "W01110D",   # Fall on same level from slipping, tripping and stumbling with subsequent striking against sharp glass, subsequent encounter
        "W01110S",   # Fall on same level from slipping, tripping and stumbling with subsequent striking against sharp glass, sequela
        "W01111A",   # Fall on same level from slipping, tripping and stumbling with subsequent striking against power tool or machine, initial encounter
        "W01111D",   # Fall on same level from slipping, tripping and stumbling with subsequent striking against power tool or machine, subsequent encounter
        "W01111S",   # Fall on same level from slipping, tripping and stumbling with subsequent striking against power tool or machine, sequela
        "W01118A",   # Fall on same level from slipping, tripping and stumbling with subsequent striking against other sharp object, initial encounter
        "W01118D",   # Fall on same level from slipping, tripping and stumbling with subsequent striking against other sharp object, subsequent encounter
        "W01118S",   # Fall on same level from slipping, tripping and stumbling with subsequent striking against other sharp object, sequela
        "W01119A",   # Fall on same level from slipping, tripping and stumbling with subsequent striking against unspecified sharp object, initial encounter
        "W01119D",   # Fall on same level from slipping, tripping and stumbling with subsequent striking against unspecified sharp object, subsequent encounter
        "W01119S",   # Fall on same level from slipping, tripping and stumbling with subsequent striking against unspecified sharp object, sequela
        "W01190A",   # Fall on same level from slipping, tripping and stumbling with subsequent striking against furniture, initial encounter
        "W01190D",   # Fall on same level from slipping, tripping and stumbling with subsequent striking against furniture, subsequent encounter
        "W01190S",   # Fall on same level from slipping, tripping and stumbling with subsequent striking against furniture, sequela
        "W01198A",   # Fall on same level from slipping, tripping and stumbling with subsequent striking against other object, initial encounter
        "W01198D",   # Fall on same level from slipping, tripping and stumbling with subsequent striking against other object, subsequent encounter
        "W01198S",   # Fall on same level from slipping, tripping and stumbling with subsequent striking against other object, sequela
        "W06XXXA",   # Fall from bed, initial encounter
        "W06XXXD",   # Fall from bed, subsequent encounter
        "W06XXXS",   # Fall from bed, sequela
        "W07XXXA",   # Fall from chair, initial encounter
        "W07XXXD",   # Fall from chair, subsequent encounter
        "W07XXXS",   # Fall from chair, sequela
        "W08XXXA",   # Fall from other furniture, initial encounter
        "W08XXXD",   # Fall from other furniture, subsequent encounter
        "W08XXXS",   # Fall from other furniture, sequela
        "W100XXA",   # Fall (on)(from) escalator, initial encounter
        "W100XXD",   # Fall (on)(from) escalator, subsequent encounter
        "W100XXS",   # Fall (on)(from) escalator, sequela
        "W101XXA",   # Fall (on)(from) sidewalk curb, initial encounter
        "W101XXD",   # Fall (on)(from) sidewalk curb, subsequent encounter
        "W101XXS",   # Fall (on)(from) sidewalk curb, sequela
        "W102XXA",   # Fall (on)(from) incline, initial encounter
        "W102XXD",   # Fall (on)(from) incline, subsequent encounter
        "W102XXS",   # Fall (on)(from) incline, sequela
        "W108XXA",   # Fall (on) (from) other stairs and steps, initial encounter
        "W108XXD",   # Fall (on) (from) other stairs and steps, subsequent encounter
        "W108XXS",   # Fall (on) (from) other stairs and steps, sequela
        "W109XXA",   # Fall (on) (from) unspecified stairs and steps, initial encounter
        "W109XXD",   # Fall (on) (from) unspecified stairs and steps, subsequent encounter
        "W109XXS",   # Fall (on) (from) unspecified stairs and steps, sequela
        "W1800XA",   # Striking against unspecified object with subsequent fall, initial encounter
        "W1800XD",   # Striking against unspecified object with subsequent fall, subsequent encounter
        "W1800XS",   # Striking against unspecified object with subsequent fall, sequela
        "W1802XA",   # Striking against glass with subsequent fall, initial encounter
        "W1802XD",   # Striking against glass with subsequent fall, subsequent encounter
        "W1802XS",   # Striking against glass with subsequent fall, sequela
        "W1809XA",   # Striking against other object with subsequent fall, initial encounter
        "W1809XD",   # Striking against other object with subsequent fall, subsequent encounter
        "W1809XS",   # Striking against other object with subsequent fall, sequela
        "W1811XA",   # Fall from or off toilet without subsequent striking against object, initial encounter
        "W1811XD",   # Fall from or off toilet without subsequent striking against object, subsequent encounter
        "W1811XS",   # Fall from or off toilet without subsequent striking against object, sequela
        "W1812XA",   # Fall from or off toilet with subsequent striking against object, initial encounter
        "W1812XD",   # Fall from or off toilet with subsequent striking against object, subsequent encounter
        "W1812XS",   # Fall from or off toilet with subsequent striking against object, sequela
        "W182XXA",   # Fall in (into) shower or empty bathtub, initial encounter
        "W182XXD",   # Fall in (into) shower or empty bathtub, subsequent encounter
        "W182XXS",   # Fall in (into) shower or empty bathtub, sequela
        "W1830XA",   # Fall on same level, unspecified, initial encounter
        "W1830XD",   # Fall on same level, unspecified, subsequent encounter
        "W1830XS",   # Fall on same level, unspecified, sequela
        "W1831XA",   # Fall on same level due to stepping on an object, initial encounter
        "W1831XD",   # Fall on same level due to stepping on an object, subsequent encounter
        "W1831XS",   # Fall on same level due to stepping on an object, sequela
        "W1839XA",   # Other fall on same level, initial encounter
        "W1839XD",   # Other fall on same level, subsequent encounter
        "W1839XS",   # Other fall on same level, sequela
        "W19XXXA",   # Unspecified fall, initial encounter
        "W19XXXD",   # Unspecified fall, subsequent encounter
        "W19XXXS",   # Unspecified fall, sequela
        "Y92199",    # Unspecified place in other specified residential institution as the place of occurrence of the external cause
        "Z593",      # Problems related to living in residential institution
        "Z736",      # Limitation of activities due to disability
        "Z7401",     # Bed confinement status
        "Z7409",     # Other reduced mobility
        "Z741",      # Need for assistance with personal care
        "Z742",      # Need for assistance at home and no other household member able to render care
        "Z743",      # Need for continuous supervision
        "Z748",      # Other problems related to care provider dependency
        "Z749",      # Problem related to care provider dependency, unspecified
        "Z9181",     # History of falling
        "Z9911",     # Dependence on respirator [ventilator] status
        "Z993",      # Dependence on wheelchair
        "Z9981",     # Dependence on supplemental oxygen
        "Z9989",     # Dependence on other enabling machines and devices
    }

    SNOMEDCT = {
        "10637031000119106",  # Pressure injury of buttock stage I (disorder)
        "10637071000119109",  # Pressure injury of buttock stage II (disorder)
        "10637111000119102",  # Pressure injury of buttock stage III (disorder)
        "10637151000119101",  # Pressure injury of buttock stage IV (disorder)
        "129588001",          # Adult failure to thrive syndrome (disorder)
        "138371000119104",    # Paresis as late effect of poliomyelitis (disorder)
        "17886000",           # Fall from wheelchair (event)
        "20902002",           # Fall from bed (event)
        "214436006",          # Noncollision MVA involving fall down stairs of motor bus while boarding or alighting (event)
        "214437002",          # Noncollision MVA involving fall down stairs of motor bus while boarding or alighting, driver injured (event)
        "214438007",          # Noncollision MVA involving fall down stairs of motor bus while boarding or alighting, passenger injured (event)
        "214439004",          # Noncollision MVA involving fall down stairs of motor bus while boarding or alighting, motor cyclist injured (event)
        "214441003",          # Noncollision MVA involving fall down stairs of motor bus while boarding or alighting, occupant of tram injured (event)
        "214442005",          # Noncollision MVA involving fall down stairs of motor bus while boarding or alighting, rider of animal or occupant of animal-drawn vehicle injured (event)
        "214443000",          # Noncollision MVA involving fall down stairs of motor bus while boarding or alighting, pedal cyclist injured (event)
        "214444006",          # Noncollision MVA involving fall down stairs of motor bus while boarding or alighting, pedestrian injured (event)
        "217082002",          # Accidental fall (event)
        "217083007",          # Fall on or from stairs or steps (event)
        "217084001",          # Fall on or from escalator (event)
        "217086004",          # Fall from escalator (event)
        "217088003",          # Fall on or from stairs (event)
        "217090002",          # Fall from stairs (event)
        "217092005",          # Fall on or from steps (event)
        "217093000",          # Fall on steps (event)
        "217094006",          # Fall from steps (event)
        "217142006",          # Fall from chair or bed (event)
        "217154006",          # Fall on same level from slipping, tripping or stumbling (event)
        "217155007",          # Fall on same level from slipping (event)
        "217156008",          # Fall on same level from tripping (event)
        "217157004",          # Fall on same level from stumbling (event)
        "217158009",          # Fall on moving sidewalk (event)
        "217173005",          # Fall from bump against object (event)
        "225558004",          # Pressure injury of back (disorder)
        "225562005",          # Pressure injury of sacral region of back (disorder)
        "225563000",          # Pressure injury of buttock (disorder)
        "242109009",          # Fall down stairs of motor bus while boarding or alighting (event)
        "242389003",          # Fall due to wet surface (event)
        "242390007",          # Fall due to polished surface (event)
        "242391006",          # Fall due to discarded object (event)
        "242392004",          # Fall in bath or shower (event)
        "242395002",          # Fall due to trip on loose carpet (event)
        "242396001",          # Fall due to uneven surface indoors (event)
        "242413007",          # Fall from furniture (event)
        "242414001",          # Fall from stool (event)
        "242419006",          # Fall from toilet seat (event)
        "269699007",          # Fall on same level from impact against object (event)
        "274918000",          # Fall on same level due to nature of surface (event)
        "33036003",           # Fall on same level (event)
        "40104005",           # Fall in home (event)
        "414190009",          # Fall on stairs (event)
        "427849003",          # Fall on hard surface (event)
        "428484005",          # Organic failure to thrive (disorder)
        "429621003",          # Fall on concrete (event)
        "44188002",           # Fall in shower (event)
        "56307009",           # Fall from table (event)
        "67223001",           # Fall on escalator (event)
        "699214007",          # Pressure injury of hip (disorder)
        "699216009",          # Pressure injury of coccygeal region of back (disorder)
        "715504003",          # Spastic paraparesis co-occurrent with deafness (disorder)
        "74541001",           # Fall from bench (event)
        "763829004",          # Oculopharyngodistal myopathy (disorder)
        "823018004",          # Muscle paresis due to and following neuromuscular blockade (disorder)
        "83468000",           # Fall from chair (event)
        "8960001000004106",   # Pressure injury of ischial tuberosity region of right buttock (disorder)
        "90619006",           # Fall in bathtub (event)
        "92341000119107",     # Weakness of extremities as sequela of stroke (disorder)
    }


class HospiceDiagnosis(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts that indicate a patient is receiving hospice care or services.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent hospice care or services.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS131v14
    """

    VALUE_SET_NAME = "Hospice Diagnosis"
    OID = "2.16.840.1.113883.3.464.1003.1003"
    DEFINITION_VERSION = "20220818"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    SNOMEDCT = {
        "170935008",  # Full care by hospice (finding)
        "170936009",  # Shared care - hospice and general practitioner (finding)
        "305911006",  # 305911006
    }


class PalliativeCareDiagnosis(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts that indicate a patient is receiving palliative care or services.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent hospice care or services.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS131v14
    """

    VALUE_SET_NAME = "Palliative Care Diagnosis"
    OID = "2.16.840.1.113883.3.464.1003.1167"
    DEFINITION_VERSION = "20220820"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "Z515",  # Encounter for palliative care
    }

    SNOMEDCT = {
        "305686008",  # Seen by palliative care physician (finding)
        "305824005",  # Seen by palliative care medicine service (finding)
        "441874000",  # Seen by palliative care service (finding)
    }

class AtrialFibrillationOrFlutter(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of diagnoses to identify patients with a history of atrial fibrillation/flutter or a current finding of atrial fibrillation/flutter.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that identify a diagnosis of a history of atrial fibrillation/flutter or a current finding of atrial fibrillation/flutter.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Atrial Fibrillation or Flutter"
    OID = "2.16.840.1.113883.3.117.1.7.1.202"
    DEFINITION_VERSION = "20240209"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "I480",  # Paroxysmal atrial fibrillation
        "I4811",  # Longstanding persistent atrial fibrillation
        "I4819",  # Other persistent atrial fibrillation
        "I4820",  # Chronic atrial fibrillation, unspecified
        "I4821",  # Permanent atrial fibrillation
        "I483",  # Typical atrial flutter
        "I484",  # Atypical atrial flutter
        "I4891",  # Unspecified atrial fibrillation
        "I4892",  # Unspecified atrial flutter
    }

    SNOMEDCT = {
        "1010405004",  # Paroxysmal atrial fibrillation with rapid ventricular response (disorder)
        "120041000119109",  # Atrial fibrillation with rapid ventricular response (disorder)
        "15964901000119107",  # Atypical atrial flutter (disorder)
        "233910005",  # Lone atrial fibrillation (disorder)
        "233911009",  # Non-rheumatic atrial fibrillation (disorder)
        "280797561000119107",  # Permanent atrial fibrillation due to heart valve disorder (disorder)
        "282825002",  # Paroxysmal atrial fibrillation (disorder)
        "300996004",  # Controlled atrial fibrillation (disorder)
        "312442005",  # History of atrial fibrillation (situation)
        "313377641000119105",  # Atrial fibrillation due to heart valve disorder (disorder)
        "314208002",  # Rapid atrial fibrillation (disorder)
        "425615007",  # Chronic atrial flutter (disorder)
        "426749004",  # Chronic atrial fibrillation (disorder)
        "426814001",  # Transient cerebral ischemia due to atrial fibrillation (disorder)
        "427665004",  # Paroxysmal atrial flutter (disorder)
        "428076002",  # History of atrial flutter (situation)
        "440028005",  # Permanent atrial fibrillation (disorder)
        "440059007",  # Persistent atrial fibrillation (disorder)
        "467643831000119105",  # Paroxysmal atrial fibrillation due to heart valve disorder (disorder)
        "489609371000119104",  # Persistent atrial fibrillation due to heart valve disorder (disorder)
        "49436004",  # Atrial fibrillation (disorder)
        "5370000",  # Atrial flutter (disorder)
        "706923002",  # Longstanding persistent atrial fibrillation (disorder)
        "715395008",  # Familial atrial fibrillation (disorder)
        "720448006",  # Typical atrial flutter (disorder)
        "762247006",  # Preexcited atrial fibrillation (disorder)
    }

class VenousThromboembolism(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of venous thromboembolism (VTE), including a pulmonary embolism.

    **Data Element Scope:** This value set may use a model element related to Diagnosis. .

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of proximal venous thromboembolism (VTE) of popliteal veins, femoral/superficial femoral veins, deep femoral veins, iliofemoral veins, iliac veins and inferior vena cava and pulmonary embolism.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis of venous thromboembolism (VTE) that is chronic or history of, located in the upper extremities, and veins not listed in the inclusion criteria.
    """

    VALUE_SET_NAME = "Venous Thromboembolism"
    OID = "2.16.840.1.113883.3.117.1.7.1.279"
    DEFINITION_VERSION = "20210220"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "I2602",  # Saddle embolus of pulmonary artery with acute cor pulmonale
        "I2603",  # Cement embolism of pulmonary artery with acute cor pulmonale
        "I2604",  # Fat embolism of pulmonary artery with acute cor pulmonale
        "I2609",  # Other pulmonary embolism with acute cor pulmonale
        "I2692",  # Saddle embolus of pulmonary artery without acute cor pulmonale
        "I2693",  # Single subsegmental thrombotic pulmonary embolism without acute cor pulmonale
        "I2694",  # Multiple subsegmental thrombotic pulmonary emboli without acute cor pulmonale
        "I2695",  # Cement embolism of pulmonary artery without acute cor pulmonale
        "I2696",  # Fat embolism of pulmonary artery without acute cor pulmonale
        "I2699",  # Other pulmonary embolism without acute cor pulmonale
        "I8010",  # Phlebitis and thrombophlebitis of unspecified femoral vein
        "I8011",  # Phlebitis and thrombophlebitis of right femoral vein
        "I8012",  # Phlebitis and thrombophlebitis of left femoral vein
        "I8013",  # Phlebitis and thrombophlebitis of femoral vein, bilateral
        "I80201",  # Phlebitis and thrombophlebitis of unspecified deep vessels of right lower extremity
        "I80202",  # Phlebitis and thrombophlebitis of unspecified deep vessels of left lower extremity
        "I80203",  # Phlebitis and thrombophlebitis of unspecified deep vessels of lower extremities, bilateral
        "I80209",  # Phlebitis and thrombophlebitis of unspecified deep vessels of unspecified lower extremity
        "I80211",  # Phlebitis and thrombophlebitis of right iliac vein
        "I80212",  # Phlebitis and thrombophlebitis of left iliac vein
        "I80213",  # Phlebitis and thrombophlebitis of iliac vein, bilateral
        "I80219",  # Phlebitis and thrombophlebitis of unspecified iliac vein
        "I80221",  # Phlebitis and thrombophlebitis of right popliteal vein
        "I80222",  # Phlebitis and thrombophlebitis of left popliteal vein
        "I80223",  # Phlebitis and thrombophlebitis of popliteal vein, bilateral
        "I80229",  # Phlebitis and thrombophlebitis of unspecified popliteal vein
        "I80291",  # Phlebitis and thrombophlebitis of other deep vessels of right lower extremity
        "I80292",  # Phlebitis and thrombophlebitis of other deep vessels of left lower extremity
        "I80293",  # Phlebitis and thrombophlebitis of other deep vessels of lower extremity, bilateral
        "I80299",  # Phlebitis and thrombophlebitis of other deep vessels of unspecified lower extremity
        "I803",  # Phlebitis and thrombophlebitis of lower extremities, unspecified
        "I809",  # Phlebitis and thrombophlebitis of unspecified site
        "I82220",  # Acute embolism and thrombosis of inferior vena cava
        "I82290",  # Acute embolism and thrombosis of other thoracic veins
        "I82401",  # Acute embolism and thrombosis of unspecified deep veins of right lower extremity
        "I82402",  # Acute embolism and thrombosis of unspecified deep veins of left lower extremity
        "I82403",  # Acute embolism and thrombosis of unspecified deep veins of lower extremity, bilateral
        "I82409",  # Acute embolism and thrombosis of unspecified deep veins of unspecified lower extremity
        "I82411",  # Acute embolism and thrombosis of right femoral vein
        "I82412",  # Acute embolism and thrombosis of left femoral vein
        "I82413",  # Acute embolism and thrombosis of femoral vein, bilateral
        "I82419",  # Acute embolism and thrombosis of unspecified femoral vein
        "I82421",  # Acute embolism and thrombosis of right iliac vein
        "I82422",  # Acute embolism and thrombosis of left iliac vein
        "I82423",  # Acute embolism and thrombosis of iliac vein, bilateral
        "I82429",  # Acute embolism and thrombosis of unspecified iliac vein
        "I82431",  # Acute embolism and thrombosis of right popliteal vein
        "I82432",  # Acute embolism and thrombosis of left popliteal vein
        "I82433",  # Acute embolism and thrombosis of popliteal vein, bilateral
        "I82439",  # Acute embolism and thrombosis of unspecified popliteal vein
        "I824Y1",  # Acute embolism and thrombosis of unspecified deep veins of right proximal lower extremity
        "I824Y2",  # Acute embolism and thrombosis of unspecified deep veins of left proximal lower extremity
        "I824Y3",  # Acute embolism and thrombosis of unspecified deep veins of proximal lower extremity, bilateral
        "I824Y9",  # Acute embolism and thrombosis of unspecified deep veins of unspecified proximal lower extremity
        "I82890",  # Acute embolism and thrombosis of other specified veins
        "I8290",  # Acute embolism and thrombosis of unspecified vein
    }

    SNOMEDCT = {
        "1001000119102",  # Pulmonary embolism with pulmonary infarction (disorder)
        "11029002",  # Pulmonary apoplexy (disorder)
        "128053003",  # Deep venous thrombosis (disorder)
        "128055005",  # Deep venous thrombosis of pelvic vein (disorder)
        "132251000119101",  # Acute deep venous thrombosis of popliteal vein (disorder)
        "132291000119106",  # Acute deep venous thrombosis of femoral vein (disorder)
        "132301000119107",  # Acute deep venous thrombosis of iliofemoral vein (disorder)
        "134399007",  # Deep vein thrombosis of leg related to air travel (disorder)
        "15968821000119100",  # Thrombophlebitis of right iliac vein (disorder)
        "15968861000119105",  # Thrombophlebitis of left iliac vein (disorder)
        "15968901000119104",  # Thrombophlebitis of bilateral iliac veins (disorder)
        "16011871000119103",  # Thrombophlebitis of bilateral popliteal veins (disorder)
        "16011911000119100",  # Thrombophlebitis of left popliteal vein (disorder)
        "16011951000119104",  # Thrombophlebitis of right popliteal vein (disorder)
        "16011991000119109",  # Thrombophlebitis of right femoral vein (disorder)
        "16012031000119104",  # Thrombophlebitis of bilateral femoral veins (disorder)
        "16012071000119101",  # Thrombophlebitis of left femoral vein (disorder)
        "16012111000119108",  # Thrombophlebitis of right deep femoral vein (disorder)
        "16012151000119109",  # Thrombophlebitis of left deep femoral vein (disorder)
        "16012191000119104",  # Thrombophlebitis of bilateral deep femoral veins (disorder)
        "16750002",  # Deep thrombophlebitis (disorder)
        "1748006",  # Thrombophlebitis of deep femoral vein (disorder)
        "194883006",  # Postoperative pulmonary embolus (disorder)
        "195405004",  # Phlebitis of popliteal vein (disorder)
        "195410000",  # Thrombophlebitis of the femoral vein (disorder)
        "195411001",  # Thrombophlebitis of the popliteal vein (disorder)
        "195421009",  # Phlebitis of the common iliac vein (disorder)
        "195422002",  # Phlebitis of the internal iliac vein (disorder)
        "195423007",  # Phlebitis of the external iliac vein (disorder)
        "195425000",  # Thrombophlebitis of the common iliac vein (disorder)
        "195426004",  # Thrombophlebitis of the internal iliac vein (disorder)
        "195427008",  # Thrombophlebitis of the external iliac vein (disorder)
        "195437003",  # Embolism and thrombosis of the vena cava (disorder)
        "213220000",  # Postoperative deep vein thrombosis (disorder)
        "233935004",  # Pulmonary thromboembolism (disorder)
        "233936003",  # Acute massive pulmonary embolism (disorder)
        "234044007",  # Iliofemoral deep vein thrombosis (disorder)
        "234049002",  # Venous embolism (disorder)
        "25114006",  # Phlebitis of deep veins of lower extremity (disorder)
        "281595001",  # Thrombosis of inferior vena cava (disorder)
        "285381000119104",  # Acute deep vein thrombosis of bilateral femoral veins (disorder)
        "285391000119101",  # Acute deep venous thrombosis of left femoral vein (disorder)
        "285401000119104",  # Acute deep venous thrombosis of right femoral vein (disorder)
        "285441000119102",  # Acute deep venous thrombosis of bilateral ileofemoral veins (disorder)
        "285451000119100",  # Acute deep venous thrombosis of ileofemoral vein of left lower extremity (disorder)
        "285461000119103",  # Acute deep venous thrombosis of ileofemoral vein of right lower extremity (disorder)
        "285561000119102",  # Acute deep venous thrombosis of bilateral popliteal veins (disorder)
        "285571000119108",  # Acute deep venous thrombosis of popliteal vein of left leg (disorder)
        "285581000119106",  # Acute deep venous thrombosis of politeal vein of right leg (disorder)
        "293471000119106",  # Acute deep vein thrombosis of bilateral iliac veins (disorder)
        "293481000119109",  # Acute deep vein thrombosis of right iliac vein (disorder)
        "293491000119107",  # Acute deep vein thrombosis of left iliac vein (disorder)
        "312584000",  # Thrombosis of vein of trunk (disorder)
        "312588002",  # Phlebitis of lower limb vein (disorder)
        "31295000",  # Phlebitis of iliac vein (disorder)
        "35979000",  # Phlebitis of femoropopliteal vein (disorder)
        "40198004",  # Thrombophlebitis of deep veins of lower extremity (disorder)
        "404223003",  # Deep venous thrombosis of lower extremity (disorder)
        "427775006",  # Deep venous thrombosis of profunda femoris vein (disorder)
        "427776007",  # Thrombosis of the popliteal vein (disorder)
        "42861008",  # Thrombophlebitis of iliac vein (disorder)
        "429098002",  # Thromboembolism of vein (disorder)
        "441746000",  # Infarction of lung due to iatrogenic pulmonary embolism (disorder)
        "444325005",  # Deep vein thrombosis of bilateral lower extremities (disorder)
        "46253008",  # Thrombophlebitis of lower extremities (disorder)
        "50817002",  # Embolism of vena cava (disorder)
        "52496006",  # Thrombophlebitis of femoropopliteal vein (disorder)
        "57834008",  # Pulmonary artery thrombosis (disorder)
        "59282003",  # Pulmonary embolism (disorder)
        "64662007",  # Pulmonary infarction (disorder)
        "69357003",  # Pulmonary thrombosis (disorder)
        "703277001",  # Deep venous thrombosis of femoropopliteal vein (disorder)
        "74315008",  # Pulmonary microemboli (disorder)
        "762256003",  # Thrombosis of iliac vein (disorder)
        "77892009",  # Pulmonary venous thrombosis (disorder)
        "83727005",  # Hemorrhagic pulmonary infarction (disorder)
        "83938003",  # Thrombosis of vena cava (disorder)
        "95448006",  # Thrombosis of pelvic vein (disorder)
        "95449003",  # Thrombophlebitis of pelvic vein (disorder)
    }

class LimitedLifeExpectancy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of limited life expectancy, including poor prognosis or terminal illness.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis that indicates a poor prognosis of a patients health status such as terminal or pre-terminal illness.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Limited Life Expectancy"
    OID = "2.16.840.1.113883.3.526.3.1259"
    DEFINITION_VERSION = "20250402"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    SNOMEDCT = {
        "111947009",  # Functional disorder present, condition terminal (finding)
        "162607003",  # Terminal illness - early stage (finding)
        "162608008",  # Terminal illness - late stage (finding)
        "170969009",  # Prognosis bad (finding)
        "27143004",  # Patient status determination, pre-terminal (finding)
        "300936002",  # Terminal illness (finding)
    }

class PrediabetesBorderlineDiabetes(ValueSet):
    """
    **Clinical Focus:** This set of values addresses terms representing Prediabetes (Borderline Diabetes).

    **Data Element Scope:** The intent of this set of values is to identify patients who have a/an documented diagnosis of prediabetes.

    **Inclusion Criteria:** Includes GROUPING codes for Prediabetes (Borderline Diabetes).

    **Exclusion Criteria:** Terms not representative of observable encoded terms that may have values indicating Prediabetes (Borderline Diabetes).
    """

    VALUE_SET_NAME = "Prediabetes (Borderline Diabetes)"
    OID = "2.16.840.1.113762.1.4.1222.419"
    DEFINITION_VERSION = "20240621"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "R7301",  # Impaired fasting glucose
        "R7302",  # Impaired glucose tolerance (oral)
        "R7303",  # Prediabetes
    }

    SNOMEDCT = {
        "16896421000119107",  # Prediabetes in mother during pregnancy (disorder)
        "714628002",  # Prediabetes (finding)
    }

class Pregnancy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of pregnancy.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of pregnancy.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Pregnancy"
    OID = "2.16.840.1.113883.3.526.3.378"
    DEFINITION_VERSION = "20210218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "O0900",  # Supervision of pregnancy with history of infertility, unspecified trimester
        "O0901",  # Supervision of pregnancy with history of infertility, first trimester
        "O0902",  # Supervision of pregnancy with history of infertility, second trimester
        "O0903",  # Supervision of pregnancy with history of infertility, third trimester
        "O0910",  # Supervision of pregnancy with history of ectopic pregnancy, unspecified trimester
        "O0911",  # Supervision of pregnancy with history of ectopic pregnancy, first trimester
        "O0912",  # Supervision of pregnancy with history of ectopic pregnancy, second trimester
        "O0913",  # Supervision of pregnancy with history of ectopic pregnancy, third trimester
        "O09211",  # Supervision of pregnancy with history of pre-term labor, first trimester
        "O09212",  # Supervision of pregnancy with history of pre-term labor, second trimester
        "O09213",  # Supervision of pregnancy with history of pre-term labor, third trimester
        "O09219",  # Supervision of pregnancy with history of pre-term labor, unspecified trimester
        "O09291",  # Supervision of pregnancy with other poor reproductive or obstetric history, first trimester
        "O09292",  # Supervision of pregnancy with other poor reproductive or obstetric history, second trimester
        "O09293",  # Supervision of pregnancy with other poor reproductive or obstetric history, third trimester
        "O09299",  # Supervision of pregnancy with other poor reproductive or obstetric history, unspecified trimester
        "O0930",  # Supervision of pregnancy with insufficient antenatal care, unspecified trimester
        "O0931",  # Supervision of pregnancy with insufficient antenatal care, first trimester
        "O0932",  # Supervision of pregnancy with insufficient antenatal care, second trimester
        "O0933",  # Supervision of pregnancy with insufficient antenatal care, third trimester
        "O0940",  # Supervision of pregnancy with grand multiparity, unspecified trimester
        "O0941",  # Supervision of pregnancy with grand multiparity, first trimester
        "O0942",  # Supervision of pregnancy with grand multiparity, second trimester
        "O0943",  # Supervision of pregnancy with grand multiparity, third trimester
        "O09511",  # Supervision of elderly primigravida, first trimester
        "O09512",  # Supervision of elderly primigravida, second trimester
        "O09513",  # Supervision of elderly primigravida, third trimester
        "O09519",  # Supervision of elderly primigravida, unspecified trimester
        "O09521",  # Supervision of elderly multigravida, first trimester
        "O09522",  # Supervision of elderly multigravida, second trimester
        "O09523",  # Supervision of elderly multigravida, third trimester
        "O09529",  # Supervision of elderly multigravida, unspecified trimester
        "O09611",  # Supervision of young primigravida, first trimester
        "O09612",  # Supervision of young primigravida, second trimester
        "O09613",  # Supervision of young primigravida, third trimester
        "O09619",  # Supervision of young primigravida, unspecified trimester
        "O09621",  # Supervision of young multigravida, first trimester
        "O09622",  # Supervision of young multigravida, second trimester
        "O09623",  # Supervision of young multigravida, third trimester
        "O09629",  # Supervision of young multigravida, unspecified trimester
        "O0970",  # Supervision of high risk pregnancy due to social problems, unspecified trimester
        "O0971",  # Supervision of high risk pregnancy due to social problems, first trimester
        "O0972",  # Supervision of high risk pregnancy due to social problems, second trimester
        "O0973",  # Supervision of high risk pregnancy due to social problems, third trimester
        "O09811",  # Supervision of pregnancy resulting from assisted reproductive technology, first trimester
        "O09812",  # Supervision of pregnancy resulting from assisted reproductive technology, second trimester
        "O09813",  # Supervision of pregnancy resulting from assisted reproductive technology, third trimester
        "O09819",  # Supervision of pregnancy resulting from assisted reproductive technology, unspecified trimester
        "O09821",  # Supervision of pregnancy with history of in utero procedure during previous pregnancy, first trimester
        "O09822",  # Supervision of pregnancy with history of in utero procedure during previous pregnancy, second trimester
        "O09823",  # Supervision of pregnancy with history of in utero procedure during previous pregnancy, third trimester
        "O09829",  # Supervision of pregnancy with history of in utero procedure during previous pregnancy, unspecified trimester
        "O09891",  # Supervision of other high risk pregnancies, first trimester
        "O09892",  # Supervision of other high risk pregnancies, second trimester
        "O09893",  # Supervision of other high risk pregnancies, third trimester
        "O09899",  # Supervision of other high risk pregnancies, unspecified trimester
        "O0990",  # Supervision of high risk pregnancy, unspecified, unspecified trimester
        "O0991",  # Supervision of high risk pregnancy, unspecified, first trimester
        "O0992",  # Supervision of high risk pregnancy, unspecified, second trimester
        "O0993",  # Supervision of high risk pregnancy, unspecified, third trimester
        "O09A0",  # Supervision of pregnancy with history of molar pregnancy, unspecified trimester
        "O09A1",  # Supervision of pregnancy with history of molar pregnancy, first trimester
        "O09A2",  # Supervision of pregnancy with history of molar pregnancy, second trimester
        "O09A3",  # Supervision of pregnancy with history of molar pregnancy, third trimester
        "O10011",  # Pre-existing essential hypertension complicating pregnancy, first trimester
        "O10012",  # Pre-existing essential hypertension complicating pregnancy, second trimester
        "O10013",  # Pre-existing essential hypertension complicating pregnancy, third trimester
        "O10019",  # Pre-existing essential hypertension complicating pregnancy, unspecified trimester
        "O10111",  # Pre-existing hypertensive heart disease complicating pregnancy, first trimester
        "O10112",  # Pre-existing hypertensive heart disease complicating pregnancy, second trimester
        "O10113",  # Pre-existing hypertensive heart disease complicating pregnancy, third trimester
        "O10119",  # Pre-existing hypertensive heart disease complicating pregnancy, unspecified trimester
        "O10211",  # Pre-existing hypertensive chronic kidney disease complicating pregnancy, first trimester
        "O10212",  # Pre-existing hypertensive chronic kidney disease complicating pregnancy, second trimester
        "O10213",  # Pre-existing hypertensive chronic kidney disease complicating pregnancy, third trimester
        "O10219",  # Pre-existing hypertensive chronic kidney disease complicating pregnancy, unspecified trimester
        "O10311",  # Pre-existing hypertensive heart and chronic kidney disease complicating pregnancy, first trimester
        "O10312",  # Pre-existing hypertensive heart and chronic kidney disease complicating pregnancy, second trimester
        "O10313",  # Pre-existing hypertensive heart and chronic kidney disease complicating pregnancy, third trimester
        "O10319",  # Pre-existing hypertensive heart and chronic kidney disease complicating pregnancy, unspecified trimester
        "O10411",  # Pre-existing secondary hypertension complicating pregnancy, first trimester
        "O10412",  # Pre-existing secondary hypertension complicating pregnancy, second trimester
        "O10413",  # Pre-existing secondary hypertension complicating pregnancy, third trimester
        "O10419",  # Pre-existing secondary hypertension complicating pregnancy, unspecified trimester
        "O10911",  # Unspecified pre-existing hypertension complicating pregnancy, first trimester
        "O10912",  # Unspecified pre-existing hypertension complicating pregnancy, second trimester
        "O10913",  # Unspecified pre-existing hypertension complicating pregnancy, third trimester
        "O10919",  # Unspecified pre-existing hypertension complicating pregnancy, unspecified trimester
        "O111",  # Pre-existing hypertension with pre-eclampsia, first trimester
        "O112",  # Pre-existing hypertension with pre-eclampsia, second trimester
        "O113",  # Pre-existing hypertension with pre-eclampsia, third trimester
        "O119",  # Pre-existing hypertension with pre-eclampsia, unspecified trimester
        "O1200",  # Gestational edema, unspecified trimester
        "O1201",  # Gestational edema, first trimester
        "O1202",  # Gestational edema, second trimester
        "O1203",  # Gestational edema, third trimester
        "O1210",  # Gestational proteinuria, unspecified trimester
        "O1211",  # Gestational proteinuria, first trimester
        "O1212",  # Gestational proteinuria, second trimester
        "O1213",  # Gestational proteinuria, third trimester
        "O1220",  # Gestational edema with proteinuria, unspecified trimester
        "O1221",  # Gestational edema with proteinuria, first trimester
        "O1222",  # Gestational edema with proteinuria, second trimester
        "O1223",  # Gestational edema with proteinuria, third trimester
        "O131",  # Gestational [pregnancy-induced] hypertension without significant proteinuria, first trimester
        "O132",  # Gestational [pregnancy-induced] hypertension without significant proteinuria, second trimester
        "O133",  # Gestational [pregnancy-induced] hypertension without significant proteinuria, third trimester
        "O139",  # Gestational [pregnancy-induced] hypertension without significant proteinuria, unspecified trimester
        "O1400",  # Mild to moderate pre-eclampsia, unspecified trimester
        "O1402",  # Mild to moderate pre-eclampsia, second trimester
        "O1403",  # Mild to moderate pre-eclampsia, third trimester
        "O1410",  # Severe pre-eclampsia, unspecified trimester
        "O1412",  # Severe pre-eclampsia, second trimester
        "O1413",  # Severe pre-eclampsia, third trimester
        "O1420",  # HELLP syndrome (HELLP), unspecified trimester
        "O1422",  # HELLP syndrome (HELLP), second trimester
        "O1423",  # HELLP syndrome (HELLP), third trimester
        "O1490",  # Unspecified pre-eclampsia, unspecified trimester
        "O1492",  # Unspecified pre-eclampsia, second trimester
        "O1493",  # Unspecified pre-eclampsia, third trimester
        "O1500",  # Eclampsia complicating pregnancy, unspecified trimester
        "O1502",  # Eclampsia complicating pregnancy, second trimester
        "O1503",  # Eclampsia complicating pregnancy, third trimester
        "O161",  # Unspecified maternal hypertension, first trimester
        "O162",  # Unspecified maternal hypertension, second trimester
        "O163",  # Unspecified maternal hypertension, third trimester
        "O169",  # Unspecified maternal hypertension, unspecified trimester
        "O200",  # Threatened abortion
        "O208",  # Other hemorrhage in early pregnancy
        "O209",  # Hemorrhage in early pregnancy, unspecified
        "O210",  # Mild hyperemesis gravidarum
        "O211",  # Hyperemesis gravidarum with metabolic disturbance
        "O212",  # Late vomiting of pregnancy
        "O218",  # Other vomiting complicating pregnancy
        "O219",  # Vomiting of pregnancy, unspecified
        "O2200",  # Varicose veins of lower extremity in pregnancy, unspecified trimester
        "O2201",  # Varicose veins of lower extremity in pregnancy, first trimester
        "O2202",  # Varicose veins of lower extremity in pregnancy, second trimester
        "O2203",  # Varicose veins of lower extremity in pregnancy, third trimester
        "O2210",  # Genital varices in pregnancy, unspecified trimester
        "O2211",  # Genital varices in pregnancy, first trimester
        "O2212",  # Genital varices in pregnancy, second trimester
        "O2213",  # Genital varices in pregnancy, third trimester
        "O2220",  # Superficial thrombophlebitis in pregnancy, unspecified trimester
        "O2221",  # Superficial thrombophlebitis in pregnancy, first trimester
        "O2222",  # Superficial thrombophlebitis in pregnancy, second trimester
        "O2223",  # Superficial thrombophlebitis in pregnancy, third trimester
        "O2230",  # Deep phlebothrombosis in pregnancy, unspecified trimester
        "O2231",  # Deep phlebothrombosis in pregnancy, first trimester
        "O2232",  # Deep phlebothrombosis in pregnancy, second trimester
        "O2233",  # Deep phlebothrombosis in pregnancy, third trimester
        "O2240",  # Hemorrhoids in pregnancy, unspecified trimester
        "O2241",  # Hemorrhoids in pregnancy, first trimester
        "O2242",  # Hemorrhoids in pregnancy, second trimester
        "O2243",  # Hemorrhoids in pregnancy, third trimester
        "O2250",  # Cerebral venous thrombosis in pregnancy, unspecified trimester
        "O2251",  # Cerebral venous thrombosis in pregnancy, first trimester
        "O2252",  # Cerebral venous thrombosis in pregnancy, second trimester
        "O2253",  # Cerebral venous thrombosis in pregnancy, third trimester
        "O228X1",  # Other venous complications in pregnancy, first trimester
        "O228X2",  # Other venous complications in pregnancy, second trimester
        "O228X3",  # Other venous complications in pregnancy, third trimester
        "O228X9",  # Other venous complications in pregnancy, unspecified trimester
        "O2290",  # Venous complication in pregnancy, unspecified, unspecified trimester
        "O2291",  # Venous complication in pregnancy, unspecified, first trimester
        "O2292",  # Venous complication in pregnancy, unspecified, second trimester
        "O2293",  # Venous complication in pregnancy, unspecified, third trimester
        "O2300",  # Infections of kidney in pregnancy, unspecified trimester
        "O2301",  # Infections of kidney in pregnancy, first trimester
        "O2302",  # Infections of kidney in pregnancy, second trimester
        "O2303",  # Infections of kidney in pregnancy, third trimester
        "O2310",  # Infections of bladder in pregnancy, unspecified trimester
        "O2311",  # Infections of bladder in pregnancy, first trimester
        "O2312",  # Infections of bladder in pregnancy, second trimester
        "O2313",  # Infections of bladder in pregnancy, third trimester
        "O2320",  # Infections of urethra in pregnancy, unspecified trimester
        "O2321",  # Infections of urethra in pregnancy, first trimester
        "O2322",  # Infections of urethra in pregnancy, second trimester
        "O2323",  # Infections of urethra in pregnancy, third trimester
        "O2330",  # Infections of other parts of urinary tract in pregnancy, unspecified trimester
        "O2331",  # Infections of other parts of urinary tract in pregnancy, first trimester
        "O2332",  # Infections of other parts of urinary tract in pregnancy, second trimester
        "O2333",  # Infections of other parts of urinary tract in pregnancy, third trimester
        "O2340",  # Unspecified infection of urinary tract in pregnancy, unspecified trimester
        "O2341",  # Unspecified infection of urinary tract in pregnancy, first trimester
        "O2342",  # Unspecified infection of urinary tract in pregnancy, second trimester
        "O2343",  # Unspecified infection of urinary tract in pregnancy, third trimester
        "O23511",  # Infections of cervix in pregnancy, first trimester
        "O23512",  # Infections of cervix in pregnancy, second trimester
        "O23513",  # Infections of cervix in pregnancy, third trimester
        "O23519",  # Infections of cervix in pregnancy, unspecified trimester
        "O23521",  # Salpingo-oophoritis in pregnancy, first trimester
        "O23522",  # Salpingo-oophoritis in pregnancy, second trimester
        "O23523",  # Salpingo-oophoritis in pregnancy, third trimester
        "O23529",  # Salpingo-oophoritis in pregnancy, unspecified trimester
        "O23591",  # Infection of other part of genital tract in pregnancy, first trimester
        "O23592",  # Infection of other part of genital tract in pregnancy, second trimester
        "O23593",  # Infection of other part of genital tract in pregnancy, third trimester
        "O23599",  # Infection of other part of genital tract in pregnancy, unspecified trimester
        "O2390",  # Unspecified genitourinary tract infection in pregnancy, unspecified trimester
        "O2391",  # Unspecified genitourinary tract infection in pregnancy, first trimester
        "O2392",  # Unspecified genitourinary tract infection in pregnancy, second trimester
        "O2393",  # Unspecified genitourinary tract infection in pregnancy, third trimester
        "O24011",  # Pre-existing type 1 diabetes mellitus, in pregnancy, first trimester
        "O24012",  # Pre-existing type 1 diabetes mellitus, in pregnancy, second trimester
        "O24013",  # Pre-existing type 1 diabetes mellitus, in pregnancy, third trimester
        "O24019",  # Pre-existing type 1 diabetes mellitus, in pregnancy, unspecified trimester
        "O24111",  # Pre-existing type 2 diabetes mellitus, in pregnancy, first trimester
        "O24112",  # Pre-existing type 2 diabetes mellitus, in pregnancy, second trimester
        "O24113",  # Pre-existing type 2 diabetes mellitus, in pregnancy, third trimester
        "O24119",  # Pre-existing type 2 diabetes mellitus, in pregnancy, unspecified trimester
        "O24311",  # Unspecified pre-existing diabetes mellitus in pregnancy, first trimester
        "O24312",  # Unspecified pre-existing diabetes mellitus in pregnancy, second trimester
        "O24313",  # Unspecified pre-existing diabetes mellitus in pregnancy, third trimester
        "O24319",  # Unspecified pre-existing diabetes mellitus in pregnancy, unspecified trimester
        "O24410",  # Gestational diabetes mellitus in pregnancy, diet controlled
        "O24414",  # Gestational diabetes mellitus in pregnancy, insulin controlled
        "O24415",  # Gestational diabetes mellitus in pregnancy, controlled by oral hypoglycemic drugs
        "O24419",  # Gestational diabetes mellitus in pregnancy, unspecified control
        "O24811",  # Other pre-existing diabetes mellitus in pregnancy, first trimester
        "O24812",  # Other pre-existing diabetes mellitus in pregnancy, second trimester
        "O24813",  # Other pre-existing diabetes mellitus in pregnancy, third trimester
        "O24819",  # Other pre-existing diabetes mellitus in pregnancy, unspecified trimester
        "O24911",  # Unspecified diabetes mellitus in pregnancy, first trimester
        "O24912",  # Unspecified diabetes mellitus in pregnancy, second trimester
        "O24913",  # Unspecified diabetes mellitus in pregnancy, third trimester
        "O24919",  # Unspecified diabetes mellitus in pregnancy, unspecified trimester
        "O2510",  # Malnutrition in pregnancy, unspecified trimester
        "O2511",  # Malnutrition in pregnancy, first trimester
        "O2512",  # Malnutrition in pregnancy, second trimester
        "O2513",  # Malnutrition in pregnancy, third trimester
        "O2600",  # Excessive weight gain in pregnancy, unspecified trimester
        "O2601",  # Excessive weight gain in pregnancy, first trimester
        "O2602",  # Excessive weight gain in pregnancy, second trimester
        "O2603",  # Excessive weight gain in pregnancy, third trimester
        "O2610",  # Low weight gain in pregnancy, unspecified trimester
        "O2611",  # Low weight gain in pregnancy, first trimester
        "O2612",  # Low weight gain in pregnancy, second trimester
        "O2613",  # Low weight gain in pregnancy, third trimester
        "O2620",  # Pregnancy care for patient with recurrent pregnancy loss, unspecified trimester
        "O2621",  # Pregnancy care for patient with recurrent pregnancy loss, first trimester
        "O2622",  # Pregnancy care for patient with recurrent pregnancy loss, second trimester
        "O2623",  # Pregnancy care for patient with recurrent pregnancy loss, third trimester
        "O2630",  # Retained intrauterine contraceptive device in pregnancy, unspecified trimester
        "O2631",  # Retained intrauterine contraceptive device in pregnancy, first trimester
        "O2632",  # Retained intrauterine contraceptive device in pregnancy, second trimester
        "O2633",  # Retained intrauterine contraceptive device in pregnancy, third trimester
        "O2640",  # Herpes gestationis, unspecified trimester
        "O2641",  # Herpes gestationis, first trimester
        "O2642",  # Herpes gestationis, second trimester
        "O2643",  # Herpes gestationis, third trimester
        "O2650",  # Maternal hypotension syndrome, unspecified trimester
        "O2651",  # Maternal hypotension syndrome, first trimester
        "O2652",  # Maternal hypotension syndrome, second trimester
        "O2653",  # Maternal hypotension syndrome, third trimester
        "O26611",  # Liver and biliary tract disorders in pregnancy, first trimester
        "O26612",  # Liver and biliary tract disorders in pregnancy, second trimester
        "O26613",  # Liver and biliary tract disorders in pregnancy, third trimester
        "O26619",  # Liver and biliary tract disorders in pregnancy, unspecified trimester
        "O26641",  # Intrahepatic cholestasis of pregnancy, first trimester
        "O26642",  # Intrahepatic cholestasis of pregnancy, second trimester
        "O26643",  # Intrahepatic cholestasis of pregnancy, third trimester
        "O26649",  # Intrahepatic cholestasis of pregnancy, unspecified trimester
        "O26711",  # Subluxation of symphysis (pubis) in pregnancy, first trimester
        "O26712",  # Subluxation of symphysis (pubis) in pregnancy, second trimester
        "O26713",  # Subluxation of symphysis (pubis) in pregnancy, third trimester
        "O26719",  # Subluxation of symphysis (pubis) in pregnancy, unspecified trimester
        "O26811",  # Pregnancy related exhaustion and fatigue, first trimester
        "O26812",  # Pregnancy related exhaustion and fatigue, second trimester
        "O26813",  # Pregnancy related exhaustion and fatigue, third trimester
        "O26819",  # Pregnancy related exhaustion and fatigue, unspecified trimester
        "O26821",  # Pregnancy related peripheral neuritis, first trimester
        "O26822",  # Pregnancy related peripheral neuritis, second trimester
        "O26823",  # Pregnancy related peripheral neuritis, third trimester
        "O26829",  # Pregnancy related peripheral neuritis, unspecified trimester
        "O26831",  # Pregnancy related renal disease, first trimester
        "O26832",  # Pregnancy related renal disease, second trimester
        "O26833",  # Pregnancy related renal disease, third trimester
        "O26839",  # Pregnancy related renal disease, unspecified trimester
        "O26841",  # Uterine size-date discrepancy, first trimester
        "O26842",  # Uterine size-date discrepancy, second trimester
        "O26843",  # Uterine size-date discrepancy, third trimester
        "O26849",  # Uterine size-date discrepancy, unspecified trimester
        "O26851",  # Spotting complicating pregnancy, first trimester
        "O26852",  # Spotting complicating pregnancy, second trimester
        "O26853",  # Spotting complicating pregnancy, third trimester
        "O26859",  # Spotting complicating pregnancy, unspecified trimester
        "O2686",  # Pruritic urticarial papules and plaques of pregnancy (PUPPP)
        "O26872",  # Cervical shortening, second trimester
        "O26873",  # Cervical shortening, third trimester
        "O26879",  # Cervical shortening, unspecified trimester
        "O26891",  # Other specified pregnancy related conditions, first trimester
        "O26892",  # Other specified pregnancy related conditions, second trimester
        "O26893",  # Other specified pregnancy related conditions, third trimester
        "O26899",  # Other specified pregnancy related conditions, unspecified trimester
        "O2690",  # Pregnancy related conditions, unspecified, unspecified trimester
        "O2691",  # Pregnancy related conditions, unspecified, first trimester
        "O2692",  # Pregnancy related conditions, unspecified, second trimester
        "O2693",  # Pregnancy related conditions, unspecified, third trimester
        "O280",  # Abnormal hematological finding on antenatal screening of mother
        "O281",  # Abnormal biochemical finding on antenatal screening of mother
        "O282",  # Abnormal cytological finding on antenatal screening of mother
        "O283",  # Abnormal ultrasonic finding on antenatal screening of mother
        "O284",  # Abnormal radiological finding on antenatal screening of mother
        "O285",  # Abnormal chromosomal and genetic finding on antenatal screening of mother
        "O288",  # Other abnormal findings on antenatal screening of mother
        "O289",  # Unspecified abnormal findings on antenatal screening of mother
        "O29011",  # Aspiration pneumonitis due to anesthesia during pregnancy, first trimester
        "O29012",  # Aspiration pneumonitis due to anesthesia during pregnancy, second trimester
        "O29013",  # Aspiration pneumonitis due to anesthesia during pregnancy, third trimester
        "O29019",  # Aspiration pneumonitis due to anesthesia during pregnancy, unspecified trimester
        "O29021",  # Pressure collapse of lung due to anesthesia during pregnancy, first trimester
        "O29022",  # Pressure collapse of lung due to anesthesia during pregnancy, second trimester
        "O29023",  # Pressure collapse of lung due to anesthesia during pregnancy, third trimester
        "O29029",  # Pressure collapse of lung due to anesthesia during pregnancy, unspecified trimester
        "O29091",  # Other pulmonary complications of anesthesia during pregnancy, first trimester
        "O29092",  # Other pulmonary complications of anesthesia during pregnancy, second trimester
        "O29093",  # Other pulmonary complications of anesthesia during pregnancy, third trimester
        "O29099",  # Other pulmonary complications of anesthesia during pregnancy, unspecified trimester
        "O29111",  # Cardiac arrest due to anesthesia during pregnancy, first trimester
        "O29112",  # Cardiac arrest due to anesthesia during pregnancy, second trimester
        "O29113",  # Cardiac arrest due to anesthesia during pregnancy, third trimester
        "O29119",  # Cardiac arrest due to anesthesia during pregnancy, unspecified trimester
        "O29121",  # Cardiac failure due to anesthesia during pregnancy, first trimester
        "O29122",  # Cardiac failure due to anesthesia during pregnancy, second trimester
        "O29123",  # Cardiac failure due to anesthesia during pregnancy, third trimester
        "O29129",  # Cardiac failure due to anesthesia during pregnancy, unspecified trimester
        "O29191",  # Other cardiac complications of anesthesia during pregnancy, first trimester
        "O29192",  # Other cardiac complications of anesthesia during pregnancy, second trimester
        "O29193",  # Other cardiac complications of anesthesia during pregnancy, third trimester
        "O29199",  # Other cardiac complications of anesthesia during pregnancy, unspecified trimester
        "O29211",  # Cerebral anoxia due to anesthesia during pregnancy, first trimester
        "O29212",  # Cerebral anoxia due to anesthesia during pregnancy, second trimester
        "O29213",  # Cerebral anoxia due to anesthesia during pregnancy, third trimester
        "O29219",  # Cerebral anoxia due to anesthesia during pregnancy, unspecified trimester
        "O29291",  # Other central nervous system complications of anesthesia during pregnancy, first trimester
        "O29292",  # Other central nervous system complications of anesthesia during pregnancy, second trimester
        "O29293",  # Other central nervous system complications of anesthesia during pregnancy, third trimester
        "O29299",  # Other central nervous system complications of anesthesia during pregnancy, unspecified trimester
        "O293X1",  # Toxic reaction to local anesthesia during pregnancy, first trimester
        "O293X2",  # Toxic reaction to local anesthesia during pregnancy, second trimester
        "O293X3",  # Toxic reaction to local anesthesia during pregnancy, third trimester
        "O293X9",  # Toxic reaction to local anesthesia during pregnancy, unspecified trimester
        "O2940",  # Spinal and epidural anesthesia induced headache during pregnancy, unspecified trimester
        "O2941",  # Spinal and epidural anesthesia induced headache during pregnancy, first trimester
        "O2942",  # Spinal and epidural anesthesia induced headache during pregnancy, second trimester
        "O2943",  # Spinal and epidural anesthesia induced headache during pregnancy, third trimester
        "O295X1",  # Other complications of spinal and epidural anesthesia during pregnancy, first trimester
        "O295X2",  # Other complications of spinal and epidural anesthesia during pregnancy, second trimester
        "O295X3",  # Other complications of spinal and epidural anesthesia during pregnancy, third trimester
        "O295X9",  # Other complications of spinal and epidural anesthesia during pregnancy, unspecified trimester
        "O2960",  # Failed or difficult intubation for anesthesia during pregnancy, unspecified trimester
        "O2961",  # Failed or difficult intubation for anesthesia during pregnancy, first trimester
        "O2962",  # Failed or difficult intubation for anesthesia during pregnancy, second trimester
        "O2963",  # Failed or difficult intubation for anesthesia during pregnancy, third trimester
        "O298X1",  # Other complications of anesthesia during pregnancy, first trimester
        "O298X2",  # Other complications of anesthesia during pregnancy, second trimester
        "O298X3",  # Other complications of anesthesia during pregnancy, third trimester
        "O298X9",  # Other complications of anesthesia during pregnancy, unspecified trimester
        "O2990",  # Unspecified complication of anesthesia during pregnancy, unspecified trimester
        "O2991",  # Unspecified complication of anesthesia during pregnancy, first trimester
        "O2992",  # Unspecified complication of anesthesia during pregnancy, second trimester
        "O2993",  # Unspecified complication of anesthesia during pregnancy, third trimester
        "O30001",  # Twin pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, first trimester
        "O30002",  # Twin pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, second trimester
        "O30003",  # Twin pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, third trimester
        "O30009",  # Twin pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, unspecified trimester
        "O30011",  # Twin pregnancy, monochorionic/monoamniotic, first trimester
        "O30012",  # Twin pregnancy, monochorionic/monoamniotic, second trimester
        "O30013",  # Twin pregnancy, monochorionic/monoamniotic, third trimester
        "O30019",  # Twin pregnancy, monochorionic/monoamniotic, unspecified trimester
        "O30021",  # Conjoined twin pregnancy, first trimester
        "O30022",  # Conjoined twin pregnancy, second trimester
        "O30023",  # Conjoined twin pregnancy, third trimester
        "O30029",  # Conjoined twin pregnancy, unspecified trimester
        "O30031",  # Twin pregnancy, monochorionic/diamniotic, first trimester
        "O30032",  # Twin pregnancy, monochorionic/diamniotic, second trimester
        "O30033",  # Twin pregnancy, monochorionic/diamniotic, third trimester
        "O30039",  # Twin pregnancy, monochorionic/diamniotic, unspecified trimester
        "O30041",  # Twin pregnancy, dichorionic/diamniotic, first trimester
        "O30042",  # Twin pregnancy, dichorionic/diamniotic, second trimester
        "O30043",  # Twin pregnancy, dichorionic/diamniotic, third trimester
        "O30049",  # Twin pregnancy, dichorionic/diamniotic, unspecified trimester
        "O30091",  # Twin pregnancy, unable to determine number of placenta and number of amniotic sacs, first trimester
        "O30092",  # Twin pregnancy, unable to determine number of placenta and number of amniotic sacs, second trimester
        "O30093",  # Twin pregnancy, unable to determine number of placenta and number of amniotic sacs, third trimester
        "O30099",  # Twin pregnancy, unable to determine number of placenta and number of amniotic sacs, unspecified trimester
        "O30101",  # Triplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, first trimester
        "O30102",  # Triplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, second trimester
        "O30103",  # Triplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, third trimester
        "O30109",  # Triplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, unspecified trimester
        "O30111",  # Triplet pregnancy with two or more monochorionic fetuses, first trimester
        "O30112",  # Triplet pregnancy with two or more monochorionic fetuses, second trimester
        "O30113",  # Triplet pregnancy with two or more monochorionic fetuses, third trimester
        "O30119",  # Triplet pregnancy with two or more monochorionic fetuses, unspecified trimester
        "O30121",  # Triplet pregnancy with two or more monoamniotic fetuses, first trimester
        "O30122",  # Triplet pregnancy with two or more monoamniotic fetuses, second trimester
        "O30123",  # Triplet pregnancy with two or more monoamniotic fetuses, third trimester
        "O30129",  # Triplet pregnancy with two or more monoamniotic fetuses, unspecified trimester
        "O30131",  # Triplet pregnancy, trichorionic/triamniotic, first trimester
        "O30132",  # Triplet pregnancy, trichorionic/triamniotic, second trimester
        "O30133",  # Triplet pregnancy, trichorionic/triamniotic, third trimester
        "O30139",  # Triplet pregnancy, trichorionic/triamniotic, unspecified trimester
        "O30191",  # Triplet pregnancy, unable to determine number of placenta and number of amniotic sacs, first trimester
        "O30192",  # Triplet pregnancy, unable to determine number of placenta and number of amniotic sacs, second trimester
        "O30193",  # Triplet pregnancy, unable to determine number of placenta and number of amniotic sacs, third trimester
        "O30199",  # Triplet pregnancy, unable to determine number of placenta and number of amniotic sacs, unspecified trimester
        "O30201",  # Quadruplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, first trimester
        "O30202",  # Quadruplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, second trimester
        "O30203",  # Quadruplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, third trimester
        "O30209",  # Quadruplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, unspecified trimester
        "O30211",  # Quadruplet pregnancy with two or more monochorionic fetuses, first trimester
        "O30212",  # Quadruplet pregnancy with two or more monochorionic fetuses, second trimester
        "O30213",  # Quadruplet pregnancy with two or more monochorionic fetuses, third trimester
        "O30219",  # Quadruplet pregnancy with two or more monochorionic fetuses, unspecified trimester
        "O30221",  # Quadruplet pregnancy with two or more monoamniotic fetuses, first trimester
        "O30222",  # Quadruplet pregnancy with two or more monoamniotic fetuses, second trimester
        "O30223",  # Quadruplet pregnancy with two or more monoamniotic fetuses, third trimester
        "O30229",  # Quadruplet pregnancy with two or more monoamniotic fetuses, unspecified trimester
        "O30231",  # Quadruplet pregnancy, quadrachorionic/quadra-amniotic, first trimester
        "O30232",  # Quadruplet pregnancy, quadrachorionic/quadra-amniotic, second trimester
        "O30233",  # Quadruplet pregnancy, quadrachorionic/quadra-amniotic, third trimester
        "O30239",  # Quadruplet pregnancy, quadrachorionic/quadra-amniotic, unspecified trimester
        "O30291",  # Quadruplet pregnancy, unable to determine number of placenta and number of amniotic sacs, first trimester
        "O30292",  # Quadruplet pregnancy, unable to determine number of placenta and number of amniotic sacs, second trimester
        "O30293",  # Quadruplet pregnancy, unable to determine number of placenta and number of amniotic sacs, third trimester
        "O30299",  # Quadruplet pregnancy, unable to determine number of placenta and number of amniotic sacs, unspecified trimester
        "O30801",  # Other specified multiple gestation, unspecified number of placenta and unspecified number of amniotic sacs, first trimester
        "O30802",  # Other specified multiple gestation, unspecified number of placenta and unspecified number of amniotic sacs, second trimester
        "O30803",  # Other specified multiple gestation, unspecified number of placenta and unspecified number of amniotic sacs, third trimester
        "O30809",  # Other specified multiple gestation, unspecified number of placenta and unspecified number of amniotic sacs, unspecified trimester
        "O30811",  # Other specified multiple gestation with two or more monochorionic fetuses, first trimester
        "O30812",  # Other specified multiple gestation with two or more monochorionic fetuses, second trimester
        "O30813",  # Other specified multiple gestation with two or more monochorionic fetuses, third trimester
        "O30819",  # Other specified multiple gestation with two or more monochorionic fetuses, unspecified trimester
        "O30821",  # Other specified multiple gestation with two or more monoamniotic fetuses, first trimester
        "O30822",  # Other specified multiple gestation with two or more monoamniotic fetuses, second trimester
        "O30823",  # Other specified multiple gestation with two or more monoamniotic fetuses, third trimester
        "O30829",  # Other specified multiple gestation with two or more monoamniotic fetuses, unspecified trimester
        "O30831",  # Other specified multiple gestation, number of chorions and amnions are both equal to the number of fetuses, first trimester
        "O30832",  # Other specified multiple gestation, number of chorions and amnions are both equal to the number of fetuses, second trimester
        "O30833",  # Other specified multiple gestation, number of chorions and amnions are both equal to the number of fetuses, third trimester
        "O30839",  # Other specified multiple gestation, number of chorions and amnions are both equal to the number of fetuses, unspecified trimester
        "O30891",  # Other specified multiple gestation, unable to determine number of placenta and number of amniotic sacs, first trimester
        "O30892",  # Other specified multiple gestation, unable to determine number of placenta and number of amniotic sacs, second trimester
        "O30893",  # Other specified multiple gestation, unable to determine number of placenta and number of amniotic sacs, third trimester
        "O30899",  # Other specified multiple gestation, unable to determine number of placenta and number of amniotic sacs, unspecified trimester
        "O3090",  # Multiple gestation, unspecified, unspecified trimester
        "O3091",  # Multiple gestation, unspecified, first trimester
        "O3092",  # Multiple gestation, unspecified, second trimester
        "O3093",  # Multiple gestation, unspecified, third trimester
        "O3100X0",  # Papyraceous fetus, unspecified trimester, not applicable or unspecified
        "O3100X1",  # Papyraceous fetus, unspecified trimester, fetus 1
        "O3100X2",  # Papyraceous fetus, unspecified trimester, fetus 2
        "O3100X3",  # Papyraceous fetus, unspecified trimester, fetus 3
        "O3100X4",  # Papyraceous fetus, unspecified trimester, fetus 4
        "O3100X5",  # Papyraceous fetus, unspecified trimester, fetus 5
        "O3100X9",  # Papyraceous fetus, unspecified trimester, other fetus
        "O3101X0",  # Papyraceous fetus, first trimester, not applicable or unspecified
        "O3101X1",  # Papyraceous fetus, first trimester, fetus 1
        "O3101X2",  # Papyraceous fetus, first trimester, fetus 2
        "O3101X3",  # Papyraceous fetus, first trimester, fetus 3
        "O3101X4",  # Papyraceous fetus, first trimester, fetus 4
        "O3101X5",  # Papyraceous fetus, first trimester, fetus 5
        "O3101X9",  # Papyraceous fetus, first trimester, other fetus
        "O3102X0",  # Papyraceous fetus, second trimester, not applicable or unspecified
        "O3102X1",  # Papyraceous fetus, second trimester, fetus 1
        "O3102X2",  # Papyraceous fetus, second trimester, fetus 2
        "O3102X3",  # Papyraceous fetus, second trimester, fetus 3
        "O3102X4",  # Papyraceous fetus, second trimester, fetus 4
        "O3102X5",  # Papyraceous fetus, second trimester, fetus 5
        "O3102X9",  # Papyraceous fetus, second trimester, other fetus
        "O3103X0",  # Papyraceous fetus, third trimester, not applicable or unspecified
        "O3103X1",  # Papyraceous fetus, third trimester, fetus 1
        "O3103X2",  # Papyraceous fetus, third trimester, fetus 2
        "O3103X3",  # Papyraceous fetus, third trimester, fetus 3
        "O3103X4",  # Papyraceous fetus, third trimester, fetus 4
        "O3103X5",  # Papyraceous fetus, third trimester, fetus 5
        "O3103X9",  # Papyraceous fetus, third trimester, other fetus
        "O3110X0",  # Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, not applicable or unspecified
        "O3110X1",  # Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, fetus 1
        "O3110X2",  # Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, fetus 2
        "O3110X3",  # Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, fetus 3
        "O3110X4",  # Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, fetus 4
        "O3110X5",  # Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, fetus 5
        "O3110X9",  # Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, other fetus
        "O3111X0",  # Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, not applicable or unspecified
        "O3111X1",  # Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, fetus 1
        "O3111X2",  # Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, fetus 2
        "O3111X3",  # Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, fetus 3
        "O3111X4",  # Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, fetus 4
        "O3111X5",  # Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, fetus 5
        "O3111X9",  # Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, other fetus
        "O3112X0",  # Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, not applicable or unspecified
        "O3112X1",  # Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, fetus 1
        "O3112X2",  # Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, fetus 2
        "O3112X3",  # Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, fetus 3
        "O3112X4",  # Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, fetus 4
        "O3112X5",  # Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, fetus 5
        "O3112X9",  # Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, other fetus
        "O3113X0",  # Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, not applicable or unspecified
        "O3113X1",  # Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, fetus 1
        "O3113X2",  # Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, fetus 2
        "O3113X3",  # Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, fetus 3
        "O3113X4",  # Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, fetus 4
        "O3113X5",  # Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, fetus 5
        "O3113X9",  # Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, other fetus
        "O3120X0",  # Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, not applicable or unspecified
        "O3120X1",  # Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, fetus 1
        "O3120X2",  # Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, fetus 2
        "O3120X3",  # Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, fetus 3
        "O3120X4",  # Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, fetus 4
        "O3120X5",  # Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, fetus 5
        "O3120X9",  # Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, other fetus
        "O3121X0",  # Continuing pregnancy after intrauterine death of one fetus or more, first trimester, not applicable or unspecified
        "O3121X1",  # Continuing pregnancy after intrauterine death of one fetus or more, first trimester, fetus 1
        "O3121X2",  # Continuing pregnancy after intrauterine death of one fetus or more, first trimester, fetus 2
        "O3121X3",  # Continuing pregnancy after intrauterine death of one fetus or more, first trimester, fetus 3
        "O3121X4",  # Continuing pregnancy after intrauterine death of one fetus or more, first trimester, fetus 4
        "O3121X5",  # Continuing pregnancy after intrauterine death of one fetus or more, first trimester, fetus 5
        "O3121X9",  # Continuing pregnancy after intrauterine death of one fetus or more, first trimester, other fetus
        "O3122X0",  # Continuing pregnancy after intrauterine death of one fetus or more, second trimester, not applicable or unspecified
        "O3122X1",  # Continuing pregnancy after intrauterine death of one fetus or more, second trimester, fetus 1
        "O3122X2",  # Continuing pregnancy after intrauterine death of one fetus or more, second trimester, fetus 2
        "O3122X3",  # Continuing pregnancy after intrauterine death of one fetus or more, second trimester, fetus 3
        "O3122X4",  # Continuing pregnancy after intrauterine death of one fetus or more, second trimester, fetus 4
        "O3122X5",  # Continuing pregnancy after intrauterine death of one fetus or more, second trimester, fetus 5
        "O3122X9",  # Continuing pregnancy after intrauterine death of one fetus or more, second trimester, other fetus
        "O3123X0",  # Continuing pregnancy after intrauterine death of one fetus or more, third trimester, not applicable or unspecified
        "O3123X1",  # Continuing pregnancy after intrauterine death of one fetus or more, third trimester, fetus 1
        "O3123X2",  # Continuing pregnancy after intrauterine death of one fetus or more, third trimester, fetus 2
        "O3123X3",  # Continuing pregnancy after intrauterine death of one fetus or more, third trimester, fetus 3
        "O3123X4",  # Continuing pregnancy after intrauterine death of one fetus or more, third trimester, fetus 4
        "O3123X5",  # Continuing pregnancy after intrauterine death of one fetus or more, third trimester, fetus 5
        "O3123X9",  # Continuing pregnancy after intrauterine death of one fetus or more, third trimester, other fetus
        "O3130X0",  # Continuing pregnancy after elective fetal reduction of one fetus or more, unspecified trimester, not applicable or unspecified
        "O3130X1",  # Continuing pregnancy after elective fetal reduction of one fetus or more, unspecified trimester, fetus 1
        "O3130X2",  # Continuing pregnancy after elective fetal reduction of one fetus or more, unspecified trimester, fetus 2
        "O3130X3",  # Continuing pregnancy after elective fetal reduction of one fetus or more, unspecified trimester, fetus 3
        "O3130X4",  # Continuing pregnancy after elective fetal reduction of one fetus or more, unspecified trimester, fetus 4
        "O3130X5",  # Continuing pregnancy after elective fetal reduction of one fetus or more, unspecified trimester, fetus 5
        "O3130X9",  # Continuing pregnancy after elective fetal reduction of one fetus or more, unspecified trimester, other fetus
        "O3131X0",  # Continuing pregnancy after elective fetal reduction of one fetus or more, first trimester, not applicable or unspecified
        "O3131X1",  # Continuing pregnancy after elective fetal reduction of one fetus or more, first trimester, fetus 1
        "O3131X2",  # Continuing pregnancy after elective fetal reduction of one fetus or more, first trimester, fetus 2
        "O3131X3",  # Continuing pregnancy after elective fetal reduction of one fetus or more, first trimester, fetus 3
        "O3131X4",  # Continuing pregnancy after elective fetal reduction of one fetus or more, first trimester, fetus 4
        "O3131X5",  # Continuing pregnancy after elective fetal reduction of one fetus or more, first trimester, fetus 5
        "O3131X9",  # Continuing pregnancy after elective fetal reduction of one fetus or more, first trimester, other fetus
        "O3132X0",  # Continuing pregnancy after elective fetal reduction of one fetus or more, second trimester, not applicable or unspecified
        "O3132X1",  # Continuing pregnancy after elective fetal reduction of one fetus or more, second trimester, fetus 1
        "O3132X2",  # Continuing pregnancy after elective fetal reduction of one fetus or more, second trimester, fetus 2
        "O3132X3",  # Continuing pregnancy after elective fetal reduction of one fetus or more, second trimester, fetus 3
        "O3132X4",  # Continuing pregnancy after elective fetal reduction of one fetus or more, second trimester, fetus 4
        "O3132X5",  # Continuing pregnancy after elective fetal reduction of one fetus or more, second trimester, fetus 5
        "O3132X9",  # Continuing pregnancy after elective fetal reduction of one fetus or more, second trimester, other fetus
        "O3133X0",  # Continuing pregnancy after elective fetal reduction of one fetus or more, third trimester, not applicable or unspecified
        "O3133X1",  # Continuing pregnancy after elective fetal reduction of one fetus or more, third trimester, fetus 1
        "O3133X2",  # Continuing pregnancy after elective fetal reduction of one fetus or more, third trimester, fetus 2
        "O3133X3",  # Continuing pregnancy after elective fetal reduction of one fetus or more, third trimester, fetus 3
        "O3133X4",  # Continuing pregnancy after elective fetal reduction of one fetus or more, third trimester, fetus 4
        "O3133X5",  # Continuing pregnancy after elective fetal reduction of one fetus or more, third trimester, fetus 5
        "O3133X9",  # Continuing pregnancy after elective fetal reduction of one fetus or more, third trimester, other fetus
        "O318X10",  # Other complications specific to multiple gestation, first trimester, not applicable or unspecified
        "O318X11",  # Other complications specific to multiple gestation, first trimester, fetus 1
        "O318X12",  # Other complications specific to multiple gestation, first trimester, fetus 2
        "O318X13",  # Other complications specific to multiple gestation, first trimester, fetus 3
        "O318X14",  # Other complications specific to multiple gestation, first trimester, fetus 4
        "O318X15",  # Other complications specific to multiple gestation, first trimester, fetus 5
        "O318X19",  # Other complications specific to multiple gestation, first trimester, other fetus
        "O318X20",  # Other complications specific to multiple gestation, second trimester, not applicable or unspecified
        "O318X21",  # Other complications specific to multiple gestation, second trimester, fetus 1
        "O318X22",  # Other complications specific to multiple gestation, second trimester, fetus 2
        "O318X23",  # Other complications specific to multiple gestation, second trimester, fetus 3
        "O318X24",  # Other complications specific to multiple gestation, second trimester, fetus 4
        "O318X25",  # Other complications specific to multiple gestation, second trimester, fetus 5
        "O318X29",  # Other complications specific to multiple gestation, second trimester, other fetus
        "O318X30",  # Other complications specific to multiple gestation, third trimester, not applicable or unspecified
        "O318X31",  # Other complications specific to multiple gestation, third trimester, fetus 1
        "O318X32",  # Other complications specific to multiple gestation, third trimester, fetus 2
        "O318X33",  # Other complications specific to multiple gestation, third trimester, fetus 3
        "O318X34",  # Other complications specific to multiple gestation, third trimester, fetus 4
        "O318X35",  # Other complications specific to multiple gestation, third trimester, fetus 5
        "O318X39",  # Other complications specific to multiple gestation, third trimester, other fetus
        "O318X90",  # Other complications specific to multiple gestation, unspecified trimester, not applicable or unspecified
        "O318X91",  # Other complications specific to multiple gestation, unspecified trimester, fetus 1
        "O318X92",  # Other complications specific to multiple gestation, unspecified trimester, fetus 2
        "O318X93",  # Other complications specific to multiple gestation, unspecified trimester, fetus 3
        "O318X94",  # Other complications specific to multiple gestation, unspecified trimester, fetus 4
        "O318X95",  # Other complications specific to multiple gestation, unspecified trimester, fetus 5
        "O318X99",  # Other complications specific to multiple gestation, unspecified trimester, other fetus
        "O320XX0",  # Maternal care for unstable lie, not applicable or unspecified
        "O320XX1",  # Maternal care for unstable lie, fetus 1
        "O320XX2",  # Maternal care for unstable lie, fetus 2
        "O320XX3",  # Maternal care for unstable lie, fetus 3
        "O320XX4",  # Maternal care for unstable lie, fetus 4
        "O320XX5",  # Maternal care for unstable lie, fetus 5
        "O320XX9",  # Maternal care for unstable lie, other fetus
        "O321XX0",  # Maternal care for breech presentation, not applicable or unspecified
        "O321XX1",  # Maternal care for breech presentation, fetus 1
        "O321XX2",  # Maternal care for breech presentation, fetus 2
        "O321XX3",  # Maternal care for breech presentation, fetus 3
        "O321XX4",  # Maternal care for breech presentation, fetus 4
        "O321XX5",  # Maternal care for breech presentation, fetus 5
        "O321XX9",  # Maternal care for breech presentation, other fetus
        "O322XX0",  # Maternal care for transverse and oblique lie, not applicable or unspecified
        "O322XX1",  # Maternal care for transverse and oblique lie, fetus 1
        "O322XX2",  # Maternal care for transverse and oblique lie, fetus 2
        "O322XX3",  # Maternal care for transverse and oblique lie, fetus 3
        "O322XX4",  # Maternal care for transverse and oblique lie, fetus 4
        "O322XX5",  # Maternal care for transverse and oblique lie, fetus 5
        "O322XX9",  # Maternal care for transverse and oblique lie, other fetus
        "O323XX0",  # Maternal care for face, brow and chin presentation, not applicable or unspecified
        "O323XX1",  # Maternal care for face, brow and chin presentation, fetus 1
        "O323XX2",  # Maternal care for face, brow and chin presentation, fetus 2
        "O323XX3",  # Maternal care for face, brow and chin presentation, fetus 3
        "O323XX4",  # Maternal care for face, brow and chin presentation, fetus 4
        "O323XX5",  # Maternal care for face, brow and chin presentation, fetus 5
        "O323XX9",  # Maternal care for face, brow and chin presentation, other fetus
        "O324XX0",  # Maternal care for high head at term, not applicable or unspecified
        "O324XX1",  # Maternal care for high head at term, fetus 1
        "O324XX2",  # Maternal care for high head at term, fetus 2
        "O324XX3",  # Maternal care for high head at term, fetus 3
        "O324XX4",  # Maternal care for high head at term, fetus 4
        "O324XX5",  # Maternal care for high head at term, fetus 5
        "O324XX9",  # Maternal care for high head at term, other fetus
        "O326XX0",  # Maternal care for compound presentation, not applicable or unspecified
        "O326XX1",  # Maternal care for compound presentation, fetus 1
        "O326XX2",  # Maternal care for compound presentation, fetus 2
        "O326XX3",  # Maternal care for compound presentation, fetus 3
        "O326XX4",  # Maternal care for compound presentation, fetus 4
        "O326XX5",  # Maternal care for compound presentation, fetus 5
        "O326XX9",  # Maternal care for compound presentation, other fetus
        "O328XX0",  # Maternal care for other malpresentation of fetus, not applicable or unspecified
        "O328XX1",  # Maternal care for other malpresentation of fetus, fetus 1
        "O328XX2",  # Maternal care for other malpresentation of fetus, fetus 2
        "O328XX3",  # Maternal care for other malpresentation of fetus, fetus 3
        "O328XX4",  # Maternal care for other malpresentation of fetus, fetus 4
        "O328XX5",  # Maternal care for other malpresentation of fetus, fetus 5
        "O328XX9",  # Maternal care for other malpresentation of fetus, other fetus
        "O329XX0",  # Maternal care for malpresentation of fetus, unspecified, not applicable or unspecified
        "O329XX1",  # Maternal care for malpresentation of fetus, unspecified, fetus 1
        "O329XX2",  # Maternal care for malpresentation of fetus, unspecified, fetus 2
        "O329XX3",  # Maternal care for malpresentation of fetus, unspecified, fetus 3
        "O329XX4",  # Maternal care for malpresentation of fetus, unspecified, fetus 4
        "O329XX5",  # Maternal care for malpresentation of fetus, unspecified, fetus 5
        "O329XX9",  # Maternal care for malpresentation of fetus, unspecified, other fetus
        "O330",  # Maternal care for disproportion due to deformity of maternal pelvic bones
        "O331",  # Maternal care for disproportion due to generally contracted pelvis
        "O332",  # Maternal care for disproportion due to inlet contraction of pelvis
        "O333XX0",  # Maternal care for disproportion due to outlet contraction of pelvis, not applicable or unspecified
        "O333XX1",  # Maternal care for disproportion due to outlet contraction of pelvis, fetus 1
        "O333XX2",  # Maternal care for disproportion due to outlet contraction of pelvis, fetus 2
        "O333XX3",  # Maternal care for disproportion due to outlet contraction of pelvis, fetus 3
        "O333XX4",  # Maternal care for disproportion due to outlet contraction of pelvis, fetus 4
        "O333XX5",  # Maternal care for disproportion due to outlet contraction of pelvis, fetus 5
        "O333XX9",  # Maternal care for disproportion due to outlet contraction of pelvis, other fetus
        "O334XX0",  # Maternal care for disproportion of mixed maternal and fetal origin, not applicable or unspecified
        "O334XX1",  # Maternal care for disproportion of mixed maternal and fetal origin, fetus 1
        "O334XX2",  # Maternal care for disproportion of mixed maternal and fetal origin, fetus 2
        "O334XX3",  # Maternal care for disproportion of mixed maternal and fetal origin, fetus 3
        "O334XX4",  # Maternal care for disproportion of mixed maternal and fetal origin, fetus 4
        "O334XX5",  # Maternal care for disproportion of mixed maternal and fetal origin, fetus 5
        "O334XX9",  # Maternal care for disproportion of mixed maternal and fetal origin, other fetus
        "O335XX0",  # Maternal care for disproportion due to unusually large fetus, not applicable or unspecified
        "O335XX1",  # Maternal care for disproportion due to unusually large fetus, fetus 1
        "O335XX2",  # Maternal care for disproportion due to unusually large fetus, fetus 2
        "O335XX3",  # Maternal care for disproportion due to unusually large fetus, fetus 3
        "O335XX4",  # Maternal care for disproportion due to unusually large fetus, fetus 4
        "O335XX5",  # Maternal care for disproportion due to unusually large fetus, fetus 5
        "O335XX9",  # Maternal care for disproportion due to unusually large fetus, other fetus
        "O336XX0",  # Maternal care for disproportion due to hydrocephalic fetus, not applicable or unspecified
        "O336XX1",  # Maternal care for disproportion due to hydrocephalic fetus, fetus 1
        "O336XX2",  # Maternal care for disproportion due to hydrocephalic fetus, fetus 2
        "O336XX3",  # Maternal care for disproportion due to hydrocephalic fetus, fetus 3
        "O336XX4",  # Maternal care for disproportion due to hydrocephalic fetus, fetus 4
        "O336XX5",  # Maternal care for disproportion due to hydrocephalic fetus, fetus 5
        "O336XX9",  # Maternal care for disproportion due to hydrocephalic fetus, other fetus
        "O337XX0",  # Maternal care for disproportion due to other fetal deformities, not applicable or unspecified
        "O337XX1",  # Maternal care for disproportion due to other fetal deformities, fetus 1
        "O337XX2",  # Maternal care for disproportion due to other fetal deformities, fetus 2
        "O337XX3",  # Maternal care for disproportion due to other fetal deformities, fetus 3
        "O337XX4",  # Maternal care for disproportion due to other fetal deformities, fetus 4
        "O337XX5",  # Maternal care for disproportion due to other fetal deformities, fetus 5
        "O337XX9",  # Maternal care for disproportion due to other fetal deformities, other fetus
        "O338",  # Maternal care for disproportion of other origin
        "O339",  # Maternal care for disproportion, unspecified
        "O3400",  # Maternal care for unspecified congenital malformation of uterus, unspecified trimester
        "O3401",  # Maternal care for unspecified congenital malformation of uterus, first trimester
        "O3402",  # Maternal care for unspecified congenital malformation of uterus, second trimester
        "O3403",  # Maternal care for unspecified congenital malformation of uterus, third trimester
        "O3410",  # Maternal care for benign tumor of corpus uteri, unspecified trimester
        "O3411",  # Maternal care for benign tumor of corpus uteri, first trimester
        "O3412",  # Maternal care for benign tumor of corpus uteri, second trimester
        "O3413",  # Maternal care for benign tumor of corpus uteri, third trimester
        "O34211",  # Maternal care for low transverse scar from previous cesarean delivery
        "O34212",  # Maternal care for vertical scar from previous cesarean delivery
        "O34219",  # Maternal care for unspecified type scar from previous cesarean delivery
        "O3429",  # Maternal care due to uterine scar from other previous surgery
        "O3430",  # Maternal care for cervical incompetence, unspecified trimester
        "O3431",  # Maternal care for cervical incompetence, first trimester
        "O3432",  # Maternal care for cervical incompetence, second trimester
        "O3433",  # Maternal care for cervical incompetence, third trimester
        "O3440",  # Maternal care for other abnormalities of cervix, unspecified trimester
        "O3441",  # Maternal care for other abnormalities of cervix, first trimester
        "O3442",  # Maternal care for other abnormalities of cervix, second trimester
        "O3443",  # Maternal care for other abnormalities of cervix, third trimester
        "O34511",  # Maternal care for incarceration of gravid uterus, first trimester
        "O34512",  # Maternal care for incarceration of gravid uterus, second trimester
        "O34513",  # Maternal care for incarceration of gravid uterus, third trimester
        "O34519",  # Maternal care for incarceration of gravid uterus, unspecified trimester
        "O34521",  # Maternal care for prolapse of gravid uterus, first trimester
        "O34522",  # Maternal care for prolapse of gravid uterus, second trimester
        "O34523",  # Maternal care for prolapse of gravid uterus, third trimester
        "O34529",  # Maternal care for prolapse of gravid uterus, unspecified trimester
        "O34531",  # Maternal care for retroversion of gravid uterus, first trimester
        "O34532",  # Maternal care for retroversion of gravid uterus, second trimester
        "O34533",  # Maternal care for retroversion of gravid uterus, third trimester
        "O34539",  # Maternal care for retroversion of gravid uterus, unspecified trimester
        "O34591",  # Maternal care for other abnormalities of gravid uterus, first trimester
        "O34592",  # Maternal care for other abnormalities of gravid uterus, second trimester
        "O34593",  # Maternal care for other abnormalities of gravid uterus, third trimester
        "O34599",  # Maternal care for other abnormalities of gravid uterus, unspecified trimester
        "O3460",  # Maternal care for abnormality of vagina, unspecified trimester
        "O3461",  # Maternal care for abnormality of vagina, first trimester
        "O3462",  # Maternal care for abnormality of vagina, second trimester
        "O3463",  # Maternal care for abnormality of vagina, third trimester
        "O3470",  # Maternal care for abnormality of vulva and perineum, unspecified trimester
        "O3471",  # Maternal care for abnormality of vulva and perineum, first trimester
        "O3472",  # Maternal care for abnormality of vulva and perineum, second trimester
        "O3473",  # Maternal care for abnormality of vulva and perineum, third trimester
        "O3480",  # Maternal care for other abnormalities of pelvic organs, unspecified trimester
        "O3481",  # Maternal care for other abnormalities of pelvic organs, first trimester
        "O3482",  # Maternal care for other abnormalities of pelvic organs, second trimester
        "O3483",  # Maternal care for other abnormalities of pelvic organs, third trimester
        "O3490",  # Maternal care for abnormality of pelvic organ, unspecified, unspecified trimester
        "O3491",  # Maternal care for abnormality of pelvic organ, unspecified, first trimester
        "O3492",  # Maternal care for abnormality of pelvic organ, unspecified, second trimester
        "O3493",  # Maternal care for abnormality of pelvic organ, unspecified, third trimester
        "O3500X0",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, unspecified, not applicable or unspecified
        "O3500X1",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, unspecified, fetus 1
        "O3500X2",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, unspecified, fetus 2
        "O3500X3",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, unspecified, fetus 3
        "O3500X4",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, unspecified, fetus 4
        "O3500X5",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, unspecified, fetus 5
        "O3500X9",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, unspecified, other fetus
        "O3501X0",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, agenesis of the corpus callosum, not applicable or unspecified
        "O3501X1",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, agenesis of the corpus callosum, fetus 1
        "O3501X2",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, agenesis of the corpus callosum, fetus 2
        "O3501X3",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, agenesis of the corpus callosum, fetus 3
        "O3501X4",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, agenesis of the corpus callosum, fetus 4
        "O3501X5",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, agenesis of the corpus callosum, fetus 5
        "O3501X9",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, agenesis of the corpus callosum, other fetus
        "O3502X0",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, anencephaly, not applicable or unspecified
        "O3502X1",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, anencephaly, fetus 1
        "O3502X2",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, anencephaly, fetus 2
        "O3502X3",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, anencephaly, fetus 3
        "O3502X4",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, anencephaly, fetus 4
        "O3502X5",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, anencephaly, fetus 5
        "O3502X9",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, anencephaly, other fetus
        "O3503X0",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, choroid plexus cysts, not applicable or unspecified
        "O3503X1",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, choroid plexus cysts, fetus 1
        "O3503X2",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, choroid plexus cysts, fetus 2
        "O3503X3",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, choroid plexus cysts, fetus 3
        "O3503X4",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, choroid plexus cysts, fetus 4
        "O3503X5",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, choroid plexus cysts, fetus 5
        "O3503X9",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, choroid plexus cysts, other fetus
        "O3504X0",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, encephalocele, not applicable or unspecified
        "O3504X1",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, encephalocele, fetus 1
        "O3504X2",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, encephalocele, fetus 2
        "O3504X3",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, encephalocele, fetus 3
        "O3504X4",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, encephalocele, fetus 4
        "O3504X5",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, encephalocele, fetus 5
        "O3504X9",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, encephalocele, other fetus
        "O3505X0",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, holoprosencephaly, not applicable or unspecified
        "O3505X1",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, holoprosencephaly, fetus 1
        "O3505X2",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, holoprosencephaly, fetus 2
        "O3505X3",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, holoprosencephaly, fetus 3
        "O3505X4",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, holoprosencephaly, fetus 4
        "O3505X5",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, holoprosencephaly, fetus 5
        "O3505X9",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, holoprosencephaly, other fetus
        "O3506X0",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, hydrocephaly, not applicable or unspecified
        "O3506X1",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, hydrocephaly, fetus 1
        "O3506X2",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, hydrocephaly, fetus 2
        "O3506X3",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, hydrocephaly, fetus 3
        "O3506X4",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, hydrocephaly, fetus 4
        "O3506X5",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, hydrocephaly, fetus 5
        "O3506X9",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, hydrocephaly, other fetus
        "O3507X0",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, microcephaly, not applicable or unspecified
        "O3507X1",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, microcephaly, fetus 1
        "O3507X2",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, microcephaly, fetus 2
        "O3507X3",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, microcephaly, fetus 3
        "O3507X4",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, microcephaly, fetus 4
        "O3507X5",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, microcephaly, fetus 5
        "O3507X9",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, microcephaly, other fetus
        "O3508X0",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, spina bifida, not applicable or unspecified
        "O3508X1",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, spina bifida, fetus 1
        "O3508X2",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, spina bifida, fetus 2
        "O3508X3",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, spina bifida, fetus 3
        "O3508X4",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, spina bifida, fetus 4
        "O3508X5",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, spina bifida, fetus 5
        "O3508X9",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, spina bifida, other fetus
        "O3509X0",  # Maternal care for (suspected) other central nervous system malformation or damage in fetus, not applicable or unspecified
        "O3509X1",  # Maternal care for (suspected) other central nervous system malformation or damage in fetus, fetus 1
        "O3509X2",  # Maternal care for (suspected) other central nervous system malformation or damage in fetus, fetus 2
        "O3509X3",  # Maternal care for (suspected) other central nervous system malformation or damage in fetus, fetus 3
        "O3509X4",  # Maternal care for (suspected) other central nervous system malformation or damage in fetus, fetus 4
        "O3509X5",  # Maternal care for (suspected) other central nervous system malformation or damage in fetus, fetus 5
        "O3509X9",  # Maternal care for (suspected) other central nervous system malformation or damage in fetus, other fetus
        "O3510X0",  # Maternal care for (suspected) chromosomal abnormality in fetus, unspecified, not applicable or unspecified
        "O3510X1",  # Maternal care for (suspected) chromosomal abnormality in fetus, unspecified, fetus 1
        "O3510X2",  # Maternal care for (suspected) chromosomal abnormality in fetus, unspecified, fetus 2
        "O3510X3",  # Maternal care for (suspected) chromosomal abnormality in fetus, unspecified, fetus 3
        "O3510X4",  # Maternal care for (suspected) chromosomal abnormality in fetus, unspecified, fetus 4
        "O3510X5",  # Maternal care for (suspected) chromosomal abnormality in fetus, unspecified, fetus 5
        "O3510X9",  # Maternal care for (suspected) chromosomal abnormality in fetus, unspecified, other fetus
        "O3511X0",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 13, not applicable or unspecified
        "O3511X1",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 13, fetus 1
        "O3511X2",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 13, fetus 2
        "O3511X3",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 13, fetus 3
        "O3511X4",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 13, fetus 4
        "O3511X5",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 13, fetus 5
        "O3511X9",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 13, other fetus
        "O3512X0",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 18, not applicable or unspecified
        "O3512X1",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 18, fetus 1
        "O3512X2",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 18, fetus 2
        "O3512X3",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 18, fetus 3
        "O3512X4",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 18, fetus 4
        "O3512X5",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 18, fetus 5
        "O3512X9",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 18, other fetus
        "O3513X0",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 21, not applicable or unspecified
        "O3513X1",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 21, fetus 1
        "O3513X2",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 21, fetus 2
        "O3513X3",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 21, fetus 3
        "O3513X4",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 21, fetus 4
        "O3513X5",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 21, fetus 5
        "O3513X9",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 21, other fetus
        "O3514X0",  # Maternal care for (suspected) chromosomal abnormality in fetus, Turner Syndrome, not applicable or unspecified
        "O3514X1",  # Maternal care for (suspected) chromosomal abnormality in fetus, Turner Syndrome, fetus 1
        "O3514X2",  # Maternal care for (suspected) chromosomal abnormality in fetus, Turner Syndrome, fetus 2
        "O3514X3",  # Maternal care for (suspected) chromosomal abnormality in fetus, Turner Syndrome, fetus 3
        "O3514X4",  # Maternal care for (suspected) chromosomal abnormality in fetus, Turner Syndrome, fetus 4
        "O3514X5",  # Maternal care for (suspected) chromosomal abnormality in fetus, Turner Syndrome, fetus 5
        "O3514X9",  # Maternal care for (suspected) chromosomal abnormality in fetus, Turner Syndrome, other fetus
        "O3515X0",  # Maternal care for (suspected) chromosomal abnormality in fetus, sex chromosome abnormality, not applicable or unspecified
        "O3515X1",  # Maternal care for (suspected) chromosomal abnormality in fetus, sex chromosome abnormality, fetus 1
        "O3515X2",  # Maternal care for (suspected) chromosomal abnormality in fetus, sex chromosome abnormality, fetus 2
        "O3515X3",  # Maternal care for (suspected) chromosomal abnormality in fetus, sex chromosome abnormality, fetus 3
        "O3515X4",  # Maternal care for (suspected) chromosomal abnormality in fetus, sex chromosome abnormality, fetus 4
        "O3515X5",  # Maternal care for (suspected) chromosomal abnormality in fetus, sex chromosome abnormality, fetus 5
        "O3515X9",  # Maternal care for (suspected) chromosomal abnormality in fetus, sex chromosome abnormality, other fetus
        "O3519X0",  # Maternal care for (suspected) chromosomal abnormality in fetus, other chromosomal abnormality, not applicable or unspecified
        "O3519X1",  # Maternal care for (suspected) chromosomal abnormality in fetus, other chromosomal abnormality, fetus 1
        "O3519X2",  # Maternal care for (suspected) chromosomal abnormality in fetus, other chromosomal abnormality, fetus 2
        "O3519X3",  # Maternal care for (suspected) chromosomal abnormality in fetus, other chromosomal abnormality, fetus 3
        "O3519X4",  # Maternal care for (suspected) chromosomal abnormality in fetus, other chromosomal abnormality, fetus 4
        "O3519X5",  # Maternal care for (suspected) chromosomal abnormality in fetus, other chromosomal abnormality, fetus 5
        "O3519X9",  # Maternal care for (suspected) chromosomal abnormality in fetus, other chromosomal abnormality, other fetus
        "O352XX0",  # Maternal care for (suspected) hereditary disease in fetus, not applicable or unspecified
        "O352XX1",  # Maternal care for (suspected) hereditary disease in fetus, fetus 1
        "O352XX2",  # Maternal care for (suspected) hereditary disease in fetus, fetus 2
        "O352XX3",  # Maternal care for (suspected) hereditary disease in fetus, fetus 3
        "O352XX4",  # Maternal care for (suspected) hereditary disease in fetus, fetus 4
        "O352XX5",  # Maternal care for (suspected) hereditary disease in fetus, fetus 5
        "O352XX9",  # Maternal care for (suspected) hereditary disease in fetus, other fetus
        "O353XX0",  # Maternal care for (suspected) damage to fetus from viral disease in mother, not applicable or unspecified
        "O353XX1",  # Maternal care for (suspected) damage to fetus from viral disease in mother, fetus 1
        "O353XX2",  # Maternal care for (suspected) damage to fetus from viral disease in mother, fetus 2
        "O353XX3",  # Maternal care for (suspected) damage to fetus from viral disease in mother, fetus 3
        "O353XX4",  # Maternal care for (suspected) damage to fetus from viral disease in mother, fetus 4
        "O353XX5",  # Maternal care for (suspected) damage to fetus from viral disease in mother, fetus 5
        "O353XX9",  # Maternal care for (suspected) damage to fetus from viral disease in mother, other fetus
        "O354XX0",  # Maternal care for (suspected) damage to fetus from alcohol, not applicable or unspecified
        "O354XX1",  # Maternal care for (suspected) damage to fetus from alcohol, fetus 1
        "O354XX2",  # Maternal care for (suspected) damage to fetus from alcohol, fetus 2
        "O354XX3",  # Maternal care for (suspected) damage to fetus from alcohol, fetus 3
        "O354XX4",  # Maternal care for (suspected) damage to fetus from alcohol, fetus 4
        "O354XX5",  # Maternal care for (suspected) damage to fetus from alcohol, fetus 5
        "O354XX9",  # Maternal care for (suspected) damage to fetus from alcohol, other fetus
        "O355XX0",  # Maternal care for (suspected) damage to fetus by drugs, not applicable or unspecified
        "O355XX1",  # Maternal care for (suspected) damage to fetus by drugs, fetus 1
        "O355XX2",  # Maternal care for (suspected) damage to fetus by drugs, fetus 2
        "O355XX3",  # Maternal care for (suspected) damage to fetus by drugs, fetus 3
        "O355XX4",  # Maternal care for (suspected) damage to fetus by drugs, fetus 4
        "O355XX5",  # Maternal care for (suspected) damage to fetus by drugs, fetus 5
        "O355XX9",  # Maternal care for (suspected) damage to fetus by drugs, other fetus
        "O356XX0",  # Maternal care for (suspected) damage to fetus by radiation, not applicable or unspecified
        "O356XX1",  # Maternal care for (suspected) damage to fetus by radiation, fetus 1
        "O356XX2",  # Maternal care for (suspected) damage to fetus by radiation, fetus 2
        "O356XX3",  # Maternal care for (suspected) damage to fetus by radiation, fetus 3
        "O356XX4",  # Maternal care for (suspected) damage to fetus by radiation, fetus 4
        "O356XX5",  # Maternal care for (suspected) damage to fetus by radiation, fetus 5
        "O356XX9",  # Maternal care for (suspected) damage to fetus by radiation, other fetus
        "O357XX0",  # Maternal care for (suspected) damage to fetus by other medical procedures, not applicable or unspecified
        "O357XX1",  # Maternal care for (suspected) damage to fetus by other medical procedures, fetus 1
        "O357XX2",  # Maternal care for (suspected) damage to fetus by other medical procedures, fetus 2
        "O357XX3",  # Maternal care for (suspected) damage to fetus by other medical procedures, fetus 3
        "O357XX4",  # Maternal care for (suspected) damage to fetus by other medical procedures, fetus 4
        "O357XX5",  # Maternal care for (suspected) damage to fetus by other medical procedures, fetus 5
        "O357XX9",  # Maternal care for (suspected) damage to fetus by other medical procedures, other fetus
        "O358XX0",  # Maternal care for other (suspected) fetal abnormality and damage, not applicable or unspecified
        "O358XX1",  # Maternal care for other (suspected) fetal abnormality and damage, fetus 1
        "O358XX2",  # Maternal care for other (suspected) fetal abnormality and damage, fetus 2
        "O358XX3",  # Maternal care for other (suspected) fetal abnormality and damage, fetus 3
        "O358XX4",  # Maternal care for other (suspected) fetal abnormality and damage, fetus 4
        "O358XX5",  # Maternal care for other (suspected) fetal abnormality and damage, fetus 5
        "O358XX9",  # Maternal care for other (suspected) fetal abnormality and damage, other fetus
        "O359XX0",  # Maternal care for (suspected) fetal abnormality and damage, unspecified, not applicable or unspecified
        "O359XX1",  # Maternal care for (suspected) fetal abnormality and damage, unspecified, fetus 1
        "O359XX2",  # Maternal care for (suspected) fetal abnormality and damage, unspecified, fetus 2
        "O359XX3",  # Maternal care for (suspected) fetal abnormality and damage, unspecified, fetus 3
        "O359XX4",  # Maternal care for (suspected) fetal abnormality and damage, unspecified, fetus 4
        "O359XX5",  # Maternal care for (suspected) fetal abnormality and damage, unspecified, fetus 5
        "O359XX9",  # Maternal care for (suspected) fetal abnormality and damage, unspecified, other fetus
        "O35AXX0",  # Maternal care for other (suspected) fetal abnormality and damage, fetal facial anomalies, not applicable or unspecified
        "O35AXX1",  # Maternal care for other (suspected) fetal abnormality and damage, fetal facial anomalies, fetus 1
        "O35AXX2",  # Maternal care for other (suspected) fetal abnormality and damage, fetal facial anomalies, fetus 2
        "O35AXX3",  # Maternal care for other (suspected) fetal abnormality and damage, fetal facial anomalies, fetus 3
        "O35AXX4",  # Maternal care for other (suspected) fetal abnormality and damage, fetal facial anomalies, fetus 4
        "O35AXX5",  # Maternal care for other (suspected) fetal abnormality and damage, fetal facial anomalies, fetus 5
        "O35AXX9",  # Maternal care for other (suspected) fetal abnormality and damage, fetal facial anomalies, other fetus
        "O35BXX0",  # Maternal care for other (suspected) fetal abnormality and damage, fetal cardiac anomalies, not applicable or unspecified
        "O35BXX1",  # Maternal care for other (suspected) fetal abnormality and damage, fetal cardiac anomalies, fetus 1
        "O35BXX2",  # Maternal care for other (suspected) fetal abnormality and damage, fetal cardiac anomalies, fetus 2
        "O35BXX3",  # Maternal care for other (suspected) fetal abnormality and damage, fetal cardiac anomalies, fetus 3
        "O35BXX4",  # Maternal care for other (suspected) fetal abnormality and damage, fetal cardiac anomalies, fetus 4
        "O35BXX5",  # Maternal care for other (suspected) fetal abnormality and damage, fetal cardiac anomalies, fetus 5
        "O35BXX9",  # Maternal care for other (suspected) fetal abnormality and damage, fetal cardiac anomalies, other fetus
        "O35CXX0",  # Maternal care for other (suspected) fetal abnormality and damage, fetal pulmonary anomalies, not applicable or unspecified
        "O35CXX1",  # Maternal care for other (suspected) fetal abnormality and damage, fetal pulmonary anomalies, fetus 1
        "O35CXX2",  # Maternal care for other (suspected) fetal abnormality and damage, fetal pulmonary anomalies, fetus 2
        "O35CXX3",  # Maternal care for other (suspected) fetal abnormality and damage, fetal pulmonary anomalies, fetus 3
        "O35CXX4",  # Maternal care for other (suspected) fetal abnormality and damage, fetal pulmonary anomalies, fetus 4
        "O35CXX5",  # Maternal care for other (suspected) fetal abnormality and damage, fetal pulmonary anomalies, fetus 5
        "O35CXX9",  # Maternal care for other (suspected) fetal abnormality and damage, fetal pulmonary anomalies, other fetus
        "O35DXX0",  # Maternal care for other (suspected) fetal abnormality and damage, fetal gastrointestinal anomalies, not applicable or unspecified
        "O35DXX1",  # Maternal care for other (suspected) fetal abnormality and damage, fetal gastrointestinal anomalies, fetus 1
        "O35DXX2",  # Maternal care for other (suspected) fetal abnormality and damage, fetal gastrointestinal anomalies, fetus 2
        "O35DXX3",  # Maternal care for other (suspected) fetal abnormality and damage, fetal gastrointestinal anomalies, fetus 3
        "O35DXX4",  # Maternal care for other (suspected) fetal abnormality and damage, fetal gastrointestinal anomalies, fetus 4
        "O35DXX5",  # Maternal care for other (suspected) fetal abnormality and damage, fetal gastrointestinal anomalies, fetus 5
        "O35DXX9",  # Maternal care for other (suspected) fetal abnormality and damage, fetal gastrointestinal anomalies, other fetus
        "O35EXX0",  # Maternal care for other (suspected) fetal abnormality and damage, fetal genitourinary anomalies, not applicable or unspecified
        "O35EXX1",  # Maternal care for other (suspected) fetal abnormality and damage, fetal genitourinary anomalies, fetus 1
        "O35EXX2",  # Maternal care for other (suspected) fetal abnormality and damage, fetal genitourinary anomalies, fetus 2
        "O35EXX3",  # Maternal care for other (suspected) fetal abnormality and damage, fetal genitourinary anomalies, fetus 3
        "O35EXX4",  # Maternal care for other (suspected) fetal abnormality and damage, fetal genitourinary anomalies, fetus 4
        "O35EXX5",  # Maternal care for other (suspected) fetal abnormality and damage, fetal genitourinary anomalies, fetus 5
        "O35EXX9",  # Maternal care for other (suspected) fetal abnormality and damage, fetal genitourinary anomalies, other fetus
        "O35FXX0",  # Maternal care for other (suspected) fetal abnormality and damage, fetal musculoskeletal anomalies of trunk, not applicable or unspecified
        "O35FXX1",  # Maternal care for other (suspected) fetal abnormality and damage, fetal musculoskeletal anomalies of trunk, fetus 1
        "O35FXX2",  # Maternal care for other (suspected) fetal abnormality and damage, fetal musculoskeletal anomalies of trunk, fetus 2
        "O35FXX3",  # Maternal care for other (suspected) fetal abnormality and damage, fetal musculoskeletal anomalies of trunk, fetus 3
        "O35FXX4",  # Maternal care for other (suspected) fetal abnormality and damage, fetal musculoskeletal anomalies of trunk, fetus 4
        "O35FXX5",  # Maternal care for other (suspected) fetal abnormality and damage, fetal musculoskeletal anomalies of trunk, fetus 5
        "O35FXX9",  # Maternal care for other (suspected) fetal abnormality and damage, fetal musculoskeletal anomalies of trunk, other fetus
        "O35GXX0",  # Maternal care for other (suspected) fetal abnormality and damage, fetal upper extremities anomalies, not applicable or unspecified
        "O35GXX1",  # Maternal care for other (suspected) fetal abnormality and damage, fetal upper extremities anomalies, fetus 1
        "O35GXX2",  # Maternal care for other (suspected) fetal abnormality and damage, fetal upper extremities anomalies, fetus 2
        "O35GXX3",  # Maternal care for other (suspected) fetal abnormality and damage, fetal upper extremities anomalies, fetus 3
        "O35GXX4",  # Maternal care for other (suspected) fetal abnormality and damage, fetal upper extremities anomalies, fetus 4
        "O35GXX5",  # Maternal care for other (suspected) fetal abnormality and damage, fetal upper extremities anomalies, fetus 5
        "O35GXX9",  # Maternal care for other (suspected) fetal abnormality and damage, fetal upper extremities anomalies, other fetus
        "O35HXX0",  # Maternal care for other (suspected) fetal abnormality and damage, fetal lower extremities anomalies, not applicable or unspecified
        "O35HXX1",  # Maternal care for other (suspected) fetal abnormality and damage, fetal lower extremities anomalies, fetus 1
        "O35HXX2",  # Maternal care for other (suspected) fetal abnormality and damage, fetal lower extremities anomalies, fetus 2
        "O35HXX3",  # Maternal care for other (suspected) fetal abnormality and damage, fetal lower extremities anomalies, fetus 3
        "O35HXX4",  # Maternal care for other (suspected) fetal abnormality and damage, fetal lower extremities anomalies, fetus 4
        "O35HXX5",  # Maternal care for other (suspected) fetal abnormality and damage, fetal lower extremities anomalies, fetus 5
        "O35HXX9",  # Maternal care for other (suspected) fetal abnormality and damage, fetal lower extremities anomalies, other fetus
        "O360110",  # Maternal care for anti-D [Rh] antibodies, first trimester, not applicable or unspecified
        "O360111",  # Maternal care for anti-D [Rh] antibodies, first trimester, fetus 1
        "O360112",  # Maternal care for anti-D [Rh] antibodies, first trimester, fetus 2
        "O360113",  # Maternal care for anti-D [Rh] antibodies, first trimester, fetus 3
        "O360114",  # Maternal care for anti-D [Rh] antibodies, first trimester, fetus 4
        "O360115",  # Maternal care for anti-D [Rh] antibodies, first trimester, fetus 5
        "O360119",  # Maternal care for anti-D [Rh] antibodies, first trimester, other fetus
        "O360120",  # Maternal care for anti-D [Rh] antibodies, second trimester, not applicable or unspecified
        "O360121",  # Maternal care for anti-D [Rh] antibodies, second trimester, fetus 1
        "O360122",  # Maternal care for anti-D [Rh] antibodies, second trimester, fetus 2
        "O360123",  # Maternal care for anti-D [Rh] antibodies, second trimester, fetus 3
        "O360124",  # Maternal care for anti-D [Rh] antibodies, second trimester, fetus 4
        "O360125",  # Maternal care for anti-D [Rh] antibodies, second trimester, fetus 5
        "O360129",  # Maternal care for anti-D [Rh] antibodies, second trimester, other fetus
        "O360130",  # Maternal care for anti-D [Rh] antibodies, third trimester, not applicable or unspecified
        "O360131",  # Maternal care for anti-D [Rh] antibodies, third trimester, fetus 1
        "O360132",  # Maternal care for anti-D [Rh] antibodies, third trimester, fetus 2
        "O360133",  # Maternal care for anti-D [Rh] antibodies, third trimester, fetus 3
        "O360134",  # Maternal care for anti-D [Rh] antibodies, third trimester, fetus 4
        "O360135",  # Maternal care for anti-D [Rh] antibodies, third trimester, fetus 5
        "O360139",  # Maternal care for anti-D [Rh] antibodies, third trimester, other fetus
        "O360190",  # Maternal care for anti-D [Rh] antibodies, unspecified trimester, not applicable or unspecified
        "O360191",  # Maternal care for anti-D [Rh] antibodies, unspecified trimester, fetus 1
        "O360192",  # Maternal care for anti-D [Rh] antibodies, unspecified trimester, fetus 2
        "O360193",  # Maternal care for anti-D [Rh] antibodies, unspecified trimester, fetus 3
        "O360194",  # Maternal care for anti-D [Rh] antibodies, unspecified trimester, fetus 4
        "O360195",  # Maternal care for anti-D [Rh] antibodies, unspecified trimester, fetus 5
        "O360199",  # Maternal care for anti-D [Rh] antibodies, unspecified trimester, other fetus
        "O360910",  # Maternal care for other rhesus isoimmunization, first trimester, not applicable or unspecified
        "O360911",  # Maternal care for other rhesus isoimmunization, first trimester, fetus 1
        "O360912",  # Maternal care for other rhesus isoimmunization, first trimester, fetus 2
        "O360913",  # Maternal care for other rhesus isoimmunization, first trimester, fetus 3
        "O360914",  # Maternal care for other rhesus isoimmunization, first trimester, fetus 4
        "O360915",  # Maternal care for other rhesus isoimmunization, first trimester, fetus 5
        "O360919",  # Maternal care for other rhesus isoimmunization, first trimester, other fetus
        "O360920",  # Maternal care for other rhesus isoimmunization, second trimester, not applicable or unspecified
        "O360921",  # Maternal care for other rhesus isoimmunization, second trimester, fetus 1
        "O360922",  # Maternal care for other rhesus isoimmunization, second trimester, fetus 2
        "O360923",  # Maternal care for other rhesus isoimmunization, second trimester, fetus 3
        "O360924",  # Maternal care for other rhesus isoimmunization, second trimester, fetus 4
        "O360925",  # Maternal care for other rhesus isoimmunization, second trimester, fetus 5
        "O360929",  # Maternal care for other rhesus isoimmunization, second trimester, other fetus
        "O360930",  # Maternal care for other rhesus isoimmunization, third trimester, not applicable or unspecified
        "O360931",  # Maternal care for other rhesus isoimmunization, third trimester, fetus 1
        "O360932",  # Maternal care for other rhesus isoimmunization, third trimester, fetus 2
        "O360933",  # Maternal care for other rhesus isoimmunization, third trimester, fetus 3
        "O360934",  # Maternal care for other rhesus isoimmunization, third trimester, fetus 4
        "O360935",  # Maternal care for other rhesus isoimmunization, third trimester, fetus 5
        "O360939",  # Maternal care for other rhesus isoimmunization, third trimester, other fetus
        "O360990",  # Maternal care for other rhesus isoimmunization, unspecified trimester, not applicable or unspecified
        "O360991",  # Maternal care for other rhesus isoimmunization, unspecified trimester, fetus 1
        "O360992",  # Maternal care for other rhesus isoimmunization, unspecified trimester, fetus 2
        "O360993",  # Maternal care for other rhesus isoimmunization, unspecified trimester, fetus 3
        "O360994",  # Maternal care for other rhesus isoimmunization, unspecified trimester, fetus 4
        "O360995",  # Maternal care for other rhesus isoimmunization, unspecified trimester, fetus 5
        "O360999",  # Maternal care for other rhesus isoimmunization, unspecified trimester, other fetus
        "O361110",  # Maternal care for Anti-A sensitization, first trimester, not applicable or unspecified
        "O361111",  # Maternal care for Anti-A sensitization, first trimester, fetus 1
        "O361112",  # Maternal care for Anti-A sensitization, first trimester, fetus 2
        "O361113",  # Maternal care for Anti-A sensitization, first trimester, fetus 3
        "O361114",  # Maternal care for Anti-A sensitization, first trimester, fetus 4
        "O361115",  # Maternal care for Anti-A sensitization, first trimester, fetus 5
        "O361119",  # Maternal care for Anti-A sensitization, first trimester, other fetus
        "O361120",  # Maternal care for Anti-A sensitization, second trimester, not applicable or unspecified
        "O361121",  # Maternal care for Anti-A sensitization, second trimester, fetus 1
        "O361122",  # Maternal care for Anti-A sensitization, second trimester, fetus 2
        "O361123",  # Maternal care for Anti-A sensitization, second trimester, fetus 3
        "O361124",  # Maternal care for Anti-A sensitization, second trimester, fetus 4
        "O361125",  # Maternal care for Anti-A sensitization, second trimester, fetus 5
        "O361129",  # Maternal care for Anti-A sensitization, second trimester, other fetus
        "O361130",  # Maternal care for Anti-A sensitization, third trimester, not applicable or unspecified
        "O361131",  # Maternal care for Anti-A sensitization, third trimester, fetus 1
        "O361132",  # Maternal care for Anti-A sensitization, third trimester, fetus 2
        "O361133",  # Maternal care for Anti-A sensitization, third trimester, fetus 3
        "O361134",  # Maternal care for Anti-A sensitization, third trimester, fetus 4
        "O361135",  # Maternal care for Anti-A sensitization, third trimester, fetus 5
        "O361139",  # Maternal care for Anti-A sensitization, third trimester, other fetus
        "O361190",  # Maternal care for Anti-A sensitization, unspecified trimester, not applicable or unspecified
        "O361191",  # Maternal care for Anti-A sensitization, unspecified trimester, fetus 1
        "O361192",  # Maternal care for Anti-A sensitization, unspecified trimester, fetus 2
        "O361193",  # Maternal care for Anti-A sensitization, unspecified trimester, fetus 3
        "O361194",  # Maternal care for Anti-A sensitization, unspecified trimester, fetus 4
        "O361195",  # Maternal care for Anti-A sensitization, unspecified trimester, fetus 5
        "O361199",  # Maternal care for Anti-A sensitization, unspecified trimester, other fetus
        "O361910",  # Maternal care for other isoimmunization, first trimester, not applicable or unspecified
        "O361911",  # Maternal care for other isoimmunization, first trimester, fetus 1
        "O361912",  # Maternal care for other isoimmunization, first trimester, fetus 2
        "O361913",  # Maternal care for other isoimmunization, first trimester, fetus 3
        "O361914",  # Maternal care for other isoimmunization, first trimester, fetus 4
        "O361915",  # Maternal care for other isoimmunization, first trimester, fetus 5
        "O361919",  # Maternal care for other isoimmunization, first trimester, other fetus
        "O361920",  # Maternal care for other isoimmunization, second trimester, not applicable or unspecified
        "O361921",  # Maternal care for other isoimmunization, second trimester, fetus 1
        "O361922",  # Maternal care for other isoimmunization, second trimester, fetus 2
        "O361923",  # Maternal care for other isoimmunization, second trimester, fetus 3
        "O361924",  # Maternal care for other isoimmunization, second trimester, fetus 4
        "O361925",  # Maternal care for other isoimmunization, second trimester, fetus 5
        "O361929",  # Maternal care for other isoimmunization, second trimester, other fetus
        "O361930",  # Maternal care for other isoimmunization, third trimester, not applicable or unspecified
        "O361931",  # Maternal care for other isoimmunization, third trimester, fetus 1
        "O361932",  # Maternal care for other isoimmunization, third trimester, fetus 2
        "O361933",  # Maternal care for other isoimmunization, third trimester, fetus 3
        "O361934",  # Maternal care for other isoimmunization, third trimester, fetus 4
        "O361935",  # Maternal care for other isoimmunization, third trimester, fetus 5
        "O361939",  # Maternal care for other isoimmunization, third trimester, other fetus
        "O361990",  # Maternal care for other isoimmunization, unspecified trimester, not applicable or unspecified
        "O361991",  # Maternal care for other isoimmunization, unspecified trimester, fetus 1
        "O361992",  # Maternal care for other isoimmunization, unspecified trimester, fetus 2
        "O361993",  # Maternal care for other isoimmunization, unspecified trimester, fetus 3
        "O361994",  # Maternal care for other isoimmunization, unspecified trimester, fetus 4
        "O361995",  # Maternal care for other isoimmunization, unspecified trimester, fetus 5
        "O361999",  # Maternal care for other isoimmunization, unspecified trimester, other fetus
        "O3620X0",  # Maternal care for hydrops fetalis, unspecified trimester, not applicable or unspecified
        "O3620X1",  # Maternal care for hydrops fetalis, unspecified trimester, fetus 1
        "O3620X2",  # Maternal care for hydrops fetalis, unspecified trimester, fetus 2
        "O3620X3",  # Maternal care for hydrops fetalis, unspecified trimester, fetus 3
        "O3620X4",  # Maternal care for hydrops fetalis, unspecified trimester, fetus 4
        "O3620X5",  # Maternal care for hydrops fetalis, unspecified trimester, fetus 5
        "O3620X9",  # Maternal care for hydrops fetalis, unspecified trimester, other fetus
        "O3621X0",  # Maternal care for hydrops fetalis, first trimester, not applicable or unspecified
        "O3621X1",  # Maternal care for hydrops fetalis, first trimester, fetus 1
        "O3621X2",  # Maternal care for hydrops fetalis, first trimester, fetus 2
        "O3621X3",  # Maternal care for hydrops fetalis, first trimester, fetus 3
        "O3621X4",  # Maternal care for hydrops fetalis, first trimester, fetus 4
        "O3621X5",  # Maternal care for hydrops fetalis, first trimester, fetus 5
        "O3621X9",  # Maternal care for hydrops fetalis, first trimester, other fetus
        "O3622X0",  # Maternal care for hydrops fetalis, second trimester, not applicable or unspecified
        "O3622X1",  # Maternal care for hydrops fetalis, second trimester, fetus 1
        "O3622X2",  # Maternal care for hydrops fetalis, second trimester, fetus 2
        "O3622X3",  # Maternal care for hydrops fetalis, second trimester, fetus 3
        "O3622X4",  # Maternal care for hydrops fetalis, second trimester, fetus 4
        "O3622X5",  # Maternal care for hydrops fetalis, second trimester, fetus 5
        "O3622X9",  # Maternal care for hydrops fetalis, second trimester, other fetus
        "O3623X0",  # Maternal care for hydrops fetalis, third trimester, not applicable or unspecified
        "O3623X1",  # Maternal care for hydrops fetalis, third trimester, fetus 1
        "O3623X2",  # Maternal care for hydrops fetalis, third trimester, fetus 2
        "O3623X3",  # Maternal care for hydrops fetalis, third trimester, fetus 3
        "O3623X4",  # Maternal care for hydrops fetalis, third trimester, fetus 4
        "O3623X5",  # Maternal care for hydrops fetalis, third trimester, fetus 5
        "O3623X9",  # Maternal care for hydrops fetalis, third trimester, other fetus
        "O364XX0",  # Maternal care for intrauterine death, not applicable or unspecified
        "O364XX1",  # Maternal care for intrauterine death, fetus 1
        "O364XX2",  # Maternal care for intrauterine death, fetus 2
        "O364XX3",  # Maternal care for intrauterine death, fetus 3
        "O364XX4",  # Maternal care for intrauterine death, fetus 4
        "O364XX5",  # Maternal care for intrauterine death, fetus 5
        "O364XX9",  # Maternal care for intrauterine death, other fetus
        "O365110",  # Maternal care for known or suspected placental insufficiency, first trimester, not applicable or unspecified
        "O365111",  # Maternal care for known or suspected placental insufficiency, first trimester, fetus 1
        "O365112",  # Maternal care for known or suspected placental insufficiency, first trimester, fetus 2
        "O365113",  # Maternal care for known or suspected placental insufficiency, first trimester, fetus 3
        "O365114",  # Maternal care for known or suspected placental insufficiency, first trimester, fetus 4
        "O365115",  # Maternal care for known or suspected placental insufficiency, first trimester, fetus 5
        "O365119",  # Maternal care for known or suspected placental insufficiency, first trimester, other fetus
        "O365120",  # Maternal care for known or suspected placental insufficiency, second trimester, not applicable or unspecified
        "O365121",  # Maternal care for known or suspected placental insufficiency, second trimester, fetus 1
        "O365122",  # Maternal care for known or suspected placental insufficiency, second trimester, fetus 2
        "O365123",  # Maternal care for known or suspected placental insufficiency, second trimester, fetus 3
        "O365124",  # Maternal care for known or suspected placental insufficiency, second trimester, fetus 4
        "O365125",  # Maternal care for known or suspected placental insufficiency, second trimester, fetus 5
        "O365129",  # Maternal care for known or suspected placental insufficiency, second trimester, other fetus
        "O365130",  # Maternal care for known or suspected placental insufficiency, third trimester, not applicable or unspecified
        "O365131",  # Maternal care for known or suspected placental insufficiency, third trimester, fetus 1
        "O365132",  # Maternal care for known or suspected placental insufficiency, third trimester, fetus 2
        "O365133",  # Maternal care for known or suspected placental insufficiency, third trimester, fetus 3
        "O365134",  # Maternal care for known or suspected placental insufficiency, third trimester, fetus 4
        "O365135",  # Maternal care for known or suspected placental insufficiency, third trimester, fetus 5
        "O365139",  # Maternal care for known or suspected placental insufficiency, third trimester, other fetus
        "O365190",  # Maternal care for known or suspected placental insufficiency, unspecified trimester, not applicable or unspecified
        "O365191",  # Maternal care for known or suspected placental insufficiency, unspecified trimester, fetus 1
        "O365192",  # Maternal care for known or suspected placental insufficiency, unspecified trimester, fetus 2
        "O365193",  # Maternal care for known or suspected placental insufficiency, unspecified trimester, fetus 3
        "O365194",  # Maternal care for known or suspected placental insufficiency, unspecified trimester, fetus 4
        "O365195",  # Maternal care for known or suspected placental insufficiency, unspecified trimester, fetus 5
        "O365199",  # Maternal care for known or suspected placental insufficiency, unspecified trimester, other fetus
        "O365910",  # Maternal care for other known or suspected poor fetal growth, first trimester, not applicable or unspecified
        "O365911",  # Maternal care for other known or suspected poor fetal growth, first trimester, fetus 1
        "O365912",  # Maternal care for other known or suspected poor fetal growth, first trimester, fetus 2
        "O365913",  # Maternal care for other known or suspected poor fetal growth, first trimester, fetus 3
        "O365914",  # Maternal care for other known or suspected poor fetal growth, first trimester, fetus 4
        "O365915",  # Maternal care for other known or suspected poor fetal growth, first trimester, fetus 5
        "O365919",  # Maternal care for other known or suspected poor fetal growth, first trimester, other fetus
        "O365920",  # Maternal care for other known or suspected poor fetal growth, second trimester, not applicable or unspecified
        "O365921",  # Maternal care for other known or suspected poor fetal growth, second trimester, fetus 1
        "O365922",  # Maternal care for other known or suspected poor fetal growth, second trimester, fetus 2
        "O365923",  # Maternal care for other known or suspected poor fetal growth, second trimester, fetus 3
        "O365924",  # Maternal care for other known or suspected poor fetal growth, second trimester, fetus 4
        "O365925",  # Maternal care for other known or suspected poor fetal growth, second trimester, fetus 5
        "O365929",  # Maternal care for other known or suspected poor fetal growth, second trimester, other fetus
        "O365930",  # Maternal care for other known or suspected poor fetal growth, third trimester, not applicable or unspecified
        "O365931",  # Maternal care for other known or suspected poor fetal growth, third trimester, fetus 1
        "O365932",  # Maternal care for other known or suspected poor fetal growth, third trimester, fetus 2
        "O365933",  # Maternal care for other known or suspected poor fetal growth, third trimester, fetus 3
        "O365934",  # Maternal care for other known or suspected poor fetal growth, third trimester, fetus 4
        "O365935",  # Maternal care for other known or suspected poor fetal growth, third trimester, fetus 5
        "O365939",  # Maternal care for other known or suspected poor fetal growth, third trimester, other fetus
        "O365990",  # Maternal care for other known or suspected poor fetal growth, unspecified trimester, not applicable or unspecified
        "O365991",  # Maternal care for other known or suspected poor fetal growth, unspecified trimester, fetus 1
        "O365992",  # Maternal care for other known or suspected poor fetal growth, unspecified trimester, fetus 2
        "O365993",  # Maternal care for other known or suspected poor fetal growth, unspecified trimester, fetus 3
        "O365994",  # Maternal care for other known or suspected poor fetal growth, unspecified trimester, fetus 4
        "O365995",  # Maternal care for other known or suspected poor fetal growth, unspecified trimester, fetus 5
        "O365999",  # Maternal care for other known or suspected poor fetal growth, unspecified trimester, other fetus
        "O3660X0",  # Maternal care for excessive fetal growth, unspecified trimester, not applicable or unspecified
        "O3660X1",  # Maternal care for excessive fetal growth, unspecified trimester, fetus 1
        "O3660X2",  # Maternal care for excessive fetal growth, unspecified trimester, fetus 2
        "O3660X3",  # Maternal care for excessive fetal growth, unspecified trimester, fetus 3
        "O3660X4",  # Maternal care for excessive fetal growth, unspecified trimester, fetus 4
        "O3660X5",  # Maternal care for excessive fetal growth, unspecified trimester, fetus 5
        "O3660X9",  # Maternal care for excessive fetal growth, unspecified trimester, other fetus
        "O3661X0",  # Maternal care for excessive fetal growth, first trimester, not applicable or unspecified
        "O3661X1",  # Maternal care for excessive fetal growth, first trimester, fetus 1
        "O3661X2",  # Maternal care for excessive fetal growth, first trimester, fetus 2
        "O3661X3",  # Maternal care for excessive fetal growth, first trimester, fetus 3
        "O3661X4",  # Maternal care for excessive fetal growth, first trimester, fetus 4
        "O3661X5",  # Maternal care for excessive fetal growth, first trimester, fetus 5
        "O3661X9",  # Maternal care for excessive fetal growth, first trimester, other fetus
        "O3662X0",  # Maternal care for excessive fetal growth, second trimester, not applicable or unspecified
        "O3662X1",  # Maternal care for excessive fetal growth, second trimester, fetus 1
        "O3662X2",  # Maternal care for excessive fetal growth, second trimester, fetus 2
        "O3662X3",  # Maternal care for excessive fetal growth, second trimester, fetus 3
        "O3662X4",  # Maternal care for excessive fetal growth, second trimester, fetus 4
        "O3662X5",  # Maternal care for excessive fetal growth, second trimester, fetus 5
        "O3662X9",  # Maternal care for excessive fetal growth, second trimester, other fetus
        "O3663X0",  # Maternal care for excessive fetal growth, third trimester, not applicable or unspecified
        "O3663X1",  # Maternal care for excessive fetal growth, third trimester, fetus 1
        "O3663X2",  # Maternal care for excessive fetal growth, third trimester, fetus 2
        "O3663X3",  # Maternal care for excessive fetal growth, third trimester, fetus 3
        "O3663X4",  # Maternal care for excessive fetal growth, third trimester, fetus 4
        "O3663X5",  # Maternal care for excessive fetal growth, third trimester, fetus 5
        "O3663X9",  # Maternal care for excessive fetal growth, third trimester, other fetus
        "O3670X0",  # Maternal care for viable fetus in abdominal pregnancy, unspecified trimester, not applicable or unspecified
        "O3670X1",  # Maternal care for viable fetus in abdominal pregnancy, unspecified trimester, fetus 1
        "O3670X2",  # Maternal care for viable fetus in abdominal pregnancy, unspecified trimester, fetus 2
        "O3670X3",  # Maternal care for viable fetus in abdominal pregnancy, unspecified trimester, fetus 3
        "O3670X4",  # Maternal care for viable fetus in abdominal pregnancy, unspecified trimester, fetus 4
        "O3670X5",  # Maternal care for viable fetus in abdominal pregnancy, unspecified trimester, fetus 5
        "O3670X9",  # Maternal care for viable fetus in abdominal pregnancy, unspecified trimester, other fetus
        "O3671X0",  # Maternal care for viable fetus in abdominal pregnancy, first trimester, not applicable or unspecified
        "O3671X1",  # Maternal care for viable fetus in abdominal pregnancy, first trimester, fetus 1
        "O3671X2",  # Maternal care for viable fetus in abdominal pregnancy, first trimester, fetus 2
        "O3671X3",  # Maternal care for viable fetus in abdominal pregnancy, first trimester, fetus 3
        "O3671X4",  # Maternal care for viable fetus in abdominal pregnancy, first trimester, fetus 4
        "O3671X5",  # Maternal care for viable fetus in abdominal pregnancy, first trimester, fetus 5
        "O3671X9",  # Maternal care for viable fetus in abdominal pregnancy, first trimester, other fetus
        "O3672X0",  # Maternal care for viable fetus in abdominal pregnancy, second trimester, not applicable or unspecified
        "O3672X1",  # Maternal care for viable fetus in abdominal pregnancy, second trimester, fetus 1
        "O3672X2",  # Maternal care for viable fetus in abdominal pregnancy, second trimester, fetus 2
        "O3672X3",  # Maternal care for viable fetus in abdominal pregnancy, second trimester, fetus 3
        "O3672X4",  # Maternal care for viable fetus in abdominal pregnancy, second trimester, fetus 4
        "O3672X5",  # Maternal care for viable fetus in abdominal pregnancy, second trimester, fetus 5
        "O3672X9",  # Maternal care for viable fetus in abdominal pregnancy, second trimester, other fetus
        "O3673X0",  # Maternal care for viable fetus in abdominal pregnancy, third trimester, not applicable or unspecified
        "O3673X1",  # Maternal care for viable fetus in abdominal pregnancy, third trimester, fetus 1
        "O3673X2",  # Maternal care for viable fetus in abdominal pregnancy, third trimester, fetus 2
        "O3673X3",  # Maternal care for viable fetus in abdominal pregnancy, third trimester, fetus 3
        "O3673X4",  # Maternal care for viable fetus in abdominal pregnancy, third trimester, fetus 4
        "O3673X5",  # Maternal care for viable fetus in abdominal pregnancy, third trimester, fetus 5
        "O3673X9",  # Maternal care for viable fetus in abdominal pregnancy, third trimester, other fetus
        "O368120",  # Decreased fetal movements, second trimester, not applicable or unspecified
        "O368121",  # Decreased fetal movements, second trimester, fetus 1
        "O368122",  # Decreased fetal movements, second trimester, fetus 2
        "O368123",  # Decreased fetal movements, second trimester, fetus 3
        "O368124",  # Decreased fetal movements, second trimester, fetus 4
        "O368125",  # Decreased fetal movements, second trimester, fetus 5
        "O368129",  # Decreased fetal movements, second trimester, other fetus
        "O368130",  # Decreased fetal movements, third trimester, not applicable or unspecified
        "O368131",  # Decreased fetal movements, third trimester, fetus 1
        "O368132",  # Decreased fetal movements, third trimester, fetus 2
        "O368133",  # Decreased fetal movements, third trimester, fetus 3
        "O368134",  # Decreased fetal movements, third trimester, fetus 4
        "O368135",  # Decreased fetal movements, third trimester, fetus 5
        "O368139",  # Decreased fetal movements, third trimester, other fetus
        "O368190",  # Decreased fetal movements, unspecified trimester, not applicable or unspecified
        "O368191",  # Decreased fetal movements, unspecified trimester, fetus 1
        "O368192",  # Decreased fetal movements, unspecified trimester, fetus 2
        "O368193",  # Decreased fetal movements, unspecified trimester, fetus 3
        "O368194",  # Decreased fetal movements, unspecified trimester, fetus 4
        "O368195",  # Decreased fetal movements, unspecified trimester, fetus 5
        "O368199",  # Decreased fetal movements, unspecified trimester, other fetus
        "O368210",  # Fetal anemia and thrombocytopenia, first trimester, not applicable or unspecified
        "O368211",  # Fetal anemia and thrombocytopenia, first trimester, fetus 1
        "O368212",  # Fetal anemia and thrombocytopenia, first trimester, fetus 2
        "O368213",  # Fetal anemia and thrombocytopenia, first trimester, fetus 3
        "O368214",  # Fetal anemia and thrombocytopenia, first trimester, fetus 4
        "O368215",  # Fetal anemia and thrombocytopenia, first trimester, fetus 5
        "O368219",  # Fetal anemia and thrombocytopenia, first trimester, other fetus
        "O368220",  # Fetal anemia and thrombocytopenia, second trimester, not applicable or unspecified
        "O368221",  # Fetal anemia and thrombocytopenia, second trimester, fetus 1
        "O368222",  # Fetal anemia and thrombocytopenia, second trimester, fetus 2
        "O368223",  # Fetal anemia and thrombocytopenia, second trimester, fetus 3
        "O368224",  # Fetal anemia and thrombocytopenia, second trimester, fetus 4
        "O368225",  # Fetal anemia and thrombocytopenia, second trimester, fetus 5
        "O368229",  # Fetal anemia and thrombocytopenia, second trimester, other fetus
        "O368230",  # Fetal anemia and thrombocytopenia, third trimester, not applicable or unspecified
        "O368231",  # Fetal anemia and thrombocytopenia, third trimester, fetus 1
        "O368232",  # Fetal anemia and thrombocytopenia, third trimester, fetus 2
        "O368233",  # Fetal anemia and thrombocytopenia, third trimester, fetus 3
        "O368234",  # Fetal anemia and thrombocytopenia, third trimester, fetus 4
        "O368235",  # Fetal anemia and thrombocytopenia, third trimester, fetus 5
        "O368239",  # Fetal anemia and thrombocytopenia, third trimester, other fetus
        "O368290",  # Fetal anemia and thrombocytopenia, unspecified trimester, not applicable or unspecified
        "O368291",  # Fetal anemia and thrombocytopenia, unspecified trimester, fetus 1
        "O368292",  # Fetal anemia and thrombocytopenia, unspecified trimester, fetus 2
        "O368293",  # Fetal anemia and thrombocytopenia, unspecified trimester, fetus 3
        "O368294",  # Fetal anemia and thrombocytopenia, unspecified trimester, fetus 4
        "O368295",  # Fetal anemia and thrombocytopenia, unspecified trimester, fetus 5
        "O368299",  # Fetal anemia and thrombocytopenia, unspecified trimester, other fetus
        "O368310",  # Maternal care for abnormalities of the fetal heart rate or rhythm, first trimester, not applicable or unspecified
        "O368311",  # Maternal care for abnormalities of the fetal heart rate or rhythm, first trimester, fetus 1
        "O368312",  # Maternal care for abnormalities of the fetal heart rate or rhythm, first trimester, fetus 2
        "O368313",  # Maternal care for abnormalities of the fetal heart rate or rhythm, first trimester, fetus 3
        "O368314",  # Maternal care for abnormalities of the fetal heart rate or rhythm, first trimester, fetus 4
        "O368315",  # Maternal care for abnormalities of the fetal heart rate or rhythm, first trimester, fetus 5
        "O368319",  # Maternal care for abnormalities of the fetal heart rate or rhythm, first trimester, other fetus
        "O368320",  # Maternal care for abnormalities of the fetal heart rate or rhythm, second trimester, not applicable or unspecified
        "O368321",  # Maternal care for abnormalities of the fetal heart rate or rhythm, second trimester, fetus 1
        "O368322",  # Maternal care for abnormalities of the fetal heart rate or rhythm, second trimester, fetus 2
        "O368323",  # Maternal care for abnormalities of the fetal heart rate or rhythm, second trimester, fetus 3
        "O368324",  # Maternal care for abnormalities of the fetal heart rate or rhythm, second trimester, fetus 4
        "O368325",  # Maternal care for abnormalities of the fetal heart rate or rhythm, second trimester, fetus 5
        "O368329",  # Maternal care for abnormalities of the fetal heart rate or rhythm, second trimester, other fetus
        "O368330",  # Maternal care for abnormalities of the fetal heart rate or rhythm, third trimester, not applicable or unspecified
        "O368331",  # Maternal care for abnormalities of the fetal heart rate or rhythm, third trimester, fetus 1
        "O368332",  # Maternal care for abnormalities of the fetal heart rate or rhythm, third trimester, fetus 2
        "O368333",  # Maternal care for abnormalities of the fetal heart rate or rhythm, third trimester, fetus 3
        "O368334",  # Maternal care for abnormalities of the fetal heart rate or rhythm, third trimester, fetus 4
        "O368335",  # Maternal care for abnormalities of the fetal heart rate or rhythm, third trimester, fetus 5
        "O368339",  # Maternal care for abnormalities of the fetal heart rate or rhythm, third trimester, other fetus
        "O368390",  # Maternal care for abnormalities of the fetal heart rate or rhythm, unspecified trimester, not applicable or unspecified
        "O368391",  # Maternal care for abnormalities of the fetal heart rate or rhythm, unspecified trimester, fetus 1
        "O368392",  # Maternal care for abnormalities of the fetal heart rate or rhythm, unspecified trimester, fetus 2
        "O368393",  # Maternal care for abnormalities of the fetal heart rate or rhythm, unspecified trimester, fetus 3
        "O368394",  # Maternal care for abnormalities of the fetal heart rate or rhythm, unspecified trimester, fetus 4
        "O368395",  # Maternal care for abnormalities of the fetal heart rate or rhythm, unspecified trimester, fetus 5
        "O368399",  # Maternal care for abnormalities of the fetal heart rate or rhythm, unspecified trimester, other fetus
        "O368910",  # Maternal care for other specified fetal problems, first trimester, not applicable or unspecified
        "O368911",  # Maternal care for other specified fetal problems, first trimester, fetus 1
        "O368912",  # Maternal care for other specified fetal problems, first trimester, fetus 2
        "O368913",  # Maternal care for other specified fetal problems, first trimester, fetus 3
        "O368914",  # Maternal care for other specified fetal problems, first trimester, fetus 4
        "O368915",  # Maternal care for other specified fetal problems, first trimester, fetus 5
        "O368919",  # Maternal care for other specified fetal problems, first trimester, other fetus
        "O368920",  # Maternal care for other specified fetal problems, second trimester, not applicable or unspecified
        "O368921",  # Maternal care for other specified fetal problems, second trimester, fetus 1
        "O368922",  # Maternal care for other specified fetal problems, second trimester, fetus 2
        "O368923",  # Maternal care for other specified fetal problems, second trimester, fetus 3
        "O368924",  # Maternal care for other specified fetal problems, second trimester, fetus 4
        "O368925",  # Maternal care for other specified fetal problems, second trimester, fetus 5
        "O368929",  # Maternal care for other specified fetal problems, second trimester, other fetus
        "O368930",  # Maternal care for other specified fetal problems, third trimester, not applicable or unspecified
        "O368931",  # Maternal care for other specified fetal problems, third trimester, fetus 1
        "O368932",  # Maternal care for other specified fetal problems, third trimester, fetus 2
        "O368933",  # Maternal care for other specified fetal problems, third trimester, fetus 3
        "O368934",  # Maternal care for other specified fetal problems, third trimester, fetus 4
        "O368935",  # Maternal care for other specified fetal problems, third trimester, fetus 5
        "O368939",  # Maternal care for other specified fetal problems, third trimester, other fetus
        "O368990",  # Maternal care for other specified fetal problems, unspecified trimester, not applicable or unspecified
        "O368991",  # Maternal care for other specified fetal problems, unspecified trimester, fetus 1
        "O368992",  # Maternal care for other specified fetal problems, unspecified trimester, fetus 2
        "O368993",  # Maternal care for other specified fetal problems, unspecified trimester, fetus 3
        "O368994",  # Maternal care for other specified fetal problems, unspecified trimester, fetus 4
        "O368995",  # Maternal care for other specified fetal problems, unspecified trimester, fetus 5
        "O368999",  # Maternal care for other specified fetal problems, unspecified trimester, other fetus
        "O3690X0",  # Maternal care for fetal problem, unspecified, unspecified trimester, not applicable or unspecified
        "O3690X1",  # Maternal care for fetal problem, unspecified, unspecified trimester, fetus 1
        "O3690X2",  # Maternal care for fetal problem, unspecified, unspecified trimester, fetus 2
        "O3690X3",  # Maternal care for fetal problem, unspecified, unspecified trimester, fetus 3
        "O3690X4",  # Maternal care for fetal problem, unspecified, unspecified trimester, fetus 4
        "O3690X5",  # Maternal care for fetal problem, unspecified, unspecified trimester, fetus 5
        "O3690X9",  # Maternal care for fetal problem, unspecified, unspecified trimester, other fetus
        "O3691X0",  # Maternal care for fetal problem, unspecified, first trimester, not applicable or unspecified
        "O3691X1",  # Maternal care for fetal problem, unspecified, first trimester, fetus 1
        "O3691X2",  # Maternal care for fetal problem, unspecified, first trimester, fetus 2
        "O3691X3",  # Maternal care for fetal problem, unspecified, first trimester, fetus 3
        "O3691X4",  # Maternal care for fetal problem, unspecified, first trimester, fetus 4
        "O3691X5",  # Maternal care for fetal problem, unspecified, first trimester, fetus 5
        "O3691X9",  # Maternal care for fetal problem, unspecified, first trimester, other fetus
        "O3692X0",  # Maternal care for fetal problem, unspecified, second trimester, not applicable or unspecified
        "O3692X1",  # Maternal care for fetal problem, unspecified, second trimester, fetus 1
        "O3692X2",  # Maternal care for fetal problem, unspecified, second trimester, fetus 2
        "O3692X3",  # Maternal care for fetal problem, unspecified, second trimester, fetus 3
        "O3692X4",  # Maternal care for fetal problem, unspecified, second trimester, fetus 4
        "O3692X5",  # Maternal care for fetal problem, unspecified, second trimester, fetus 5
        "O3692X9",  # Maternal care for fetal problem, unspecified, second trimester, other fetus
        "O3693X0",  # Maternal care for fetal problem, unspecified, third trimester, not applicable or unspecified
        "O3693X1",  # Maternal care for fetal problem, unspecified, third trimester, fetus 1
        "O3693X2",  # Maternal care for fetal problem, unspecified, third trimester, fetus 2
        "O3693X3",  # Maternal care for fetal problem, unspecified, third trimester, fetus 3
        "O3693X4",  # Maternal care for fetal problem, unspecified, third trimester, fetus 4
        "O3693X5",  # Maternal care for fetal problem, unspecified, third trimester, fetus 5
        "O3693X9",  # Maternal care for fetal problem, unspecified, third trimester, other fetus
        "O401XX0",  # Polyhydramnios, first trimester, not applicable or unspecified
        "O401XX1",  # Polyhydramnios, first trimester, fetus 1
        "O401XX2",  # Polyhydramnios, first trimester, fetus 2
        "O401XX3",  # Polyhydramnios, first trimester, fetus 3
        "O401XX4",  # Polyhydramnios, first trimester, fetus 4
        "O401XX5",  # Polyhydramnios, first trimester, fetus 5
        "O401XX9",  # Polyhydramnios, first trimester, other fetus
        "O402XX0",  # Polyhydramnios, second trimester, not applicable or unspecified
        "O402XX1",  # Polyhydramnios, second trimester, fetus 1
        "O402XX2",  # Polyhydramnios, second trimester, fetus 2
        "O402XX3",  # Polyhydramnios, second trimester, fetus 3
        "O402XX4",  # Polyhydramnios, second trimester, fetus 4
        "O402XX5",  # Polyhydramnios, second trimester, fetus 5
        "O402XX9",  # Polyhydramnios, second trimester, other fetus
        "O403XX0",  # Polyhydramnios, third trimester, not applicable or unspecified
        "O403XX1",  # Polyhydramnios, third trimester, fetus 1
        "O403XX2",  # Polyhydramnios, third trimester, fetus 2
        "O403XX3",  # Polyhydramnios, third trimester, fetus 3
        "O403XX4",  # Polyhydramnios, third trimester, fetus 4
        "O403XX5",  # Polyhydramnios, third trimester, fetus 5
        "O403XX9",  # Polyhydramnios, third trimester, other fetus
        "O409XX0",  # Polyhydramnios, unspecified trimester, not applicable or unspecified
        "O409XX1",  # Polyhydramnios, unspecified trimester, fetus 1
        "O409XX2",  # Polyhydramnios, unspecified trimester, fetus 2
        "O409XX3",  # Polyhydramnios, unspecified trimester, fetus 3
        "O409XX4",  # Polyhydramnios, unspecified trimester, fetus 4
        "O409XX5",  # Polyhydramnios, unspecified trimester, fetus 5
        "O409XX9",  # Polyhydramnios, unspecified trimester, other fetus
        "O4100X0",  # Oligohydramnios, unspecified trimester, not applicable or unspecified
        "O4100X1",  # Oligohydramnios, unspecified trimester, fetus 1
        "O4100X2",  # Oligohydramnios, unspecified trimester, fetus 2
        "O4100X3",  # Oligohydramnios, unspecified trimester, fetus 3
        "O4100X4",  # Oligohydramnios, unspecified trimester, fetus 4
        "O4100X5",  # Oligohydramnios, unspecified trimester, fetus 5
        "O4100X9",  # Oligohydramnios, unspecified trimester, other fetus
        "O4101X0",  # Oligohydramnios, first trimester, not applicable or unspecified
        "O4101X1",  # Oligohydramnios, first trimester, fetus 1
        "O4101X2",  # Oligohydramnios, first trimester, fetus 2
        "O4101X3",  # Oligohydramnios, first trimester, fetus 3
        "O4101X4",  # Oligohydramnios, first trimester, fetus 4
        "O4101X5",  # Oligohydramnios, first trimester, fetus 5
        "O4101X9",  # Oligohydramnios, first trimester, other fetus
        "O4102X0",  # Oligohydramnios, second trimester, not applicable or unspecified
        "O4102X1",  # Oligohydramnios, second trimester, fetus 1
        "O4102X2",  # Oligohydramnios, second trimester, fetus 2
        "O4102X3",  # Oligohydramnios, second trimester, fetus 3
        "O4102X4",  # Oligohydramnios, second trimester, fetus 4
        "O4102X5",  # Oligohydramnios, second trimester, fetus 5
        "O4102X9",  # Oligohydramnios, second trimester, other fetus
        "O4103X0",  # Oligohydramnios, third trimester, not applicable or unspecified
        "O4103X1",  # Oligohydramnios, third trimester, fetus 1
        "O4103X2",  # Oligohydramnios, third trimester, fetus 2
        "O4103X3",  # Oligohydramnios, third trimester, fetus 3
        "O4103X4",  # Oligohydramnios, third trimester, fetus 4
        "O4103X5",  # Oligohydramnios, third trimester, fetus 5
        "O4103X9",  # Oligohydramnios, third trimester, other fetus
        "O411010",  # Infection of amniotic sac and membranes, unspecified, first trimester, not applicable or unspecified
        "O411011",  # Infection of amniotic sac and membranes, unspecified, first trimester, fetus 1
        "O411012",  # Infection of amniotic sac and membranes, unspecified, first trimester, fetus 2
        "O411013",  # Infection of amniotic sac and membranes, unspecified, first trimester, fetus 3
        "O411014",  # Infection of amniotic sac and membranes, unspecified, first trimester, fetus 4
        "O411015",  # Infection of amniotic sac and membranes, unspecified, first trimester, fetus 5
        "O411019",  # Infection of amniotic sac and membranes, unspecified, first trimester, other fetus
        "O411020",  # Infection of amniotic sac and membranes, unspecified, second trimester, not applicable or unspecified
        "O411021",  # Infection of amniotic sac and membranes, unspecified, second trimester, fetus 1
        "O411022",  # Infection of amniotic sac and membranes, unspecified, second trimester, fetus 2
        "O411023",  # Infection of amniotic sac and membranes, unspecified, second trimester, fetus 3
        "O411024",  # Infection of amniotic sac and membranes, unspecified, second trimester, fetus 4
        "O411025",  # Infection of amniotic sac and membranes, unspecified, second trimester, fetus 5
        "O411029",  # Infection of amniotic sac and membranes, unspecified, second trimester, other fetus
        "O411030",  # Infection of amniotic sac and membranes, unspecified, third trimester, not applicable or unspecified
        "O411031",  # Infection of amniotic sac and membranes, unspecified, third trimester, fetus 1
        "O411032",  # Infection of amniotic sac and membranes, unspecified, third trimester, fetus 2
        "O411033",  # Infection of amniotic sac and membranes, unspecified, third trimester, fetus 3
        "O411034",  # Infection of amniotic sac and membranes, unspecified, third trimester, fetus 4
        "O411035",  # Infection of amniotic sac and membranes, unspecified, third trimester, fetus 5
        "O411039",  # Infection of amniotic sac and membranes, unspecified, third trimester, other fetus
        "O411090",  # Infection of amniotic sac and membranes, unspecified, unspecified trimester, not applicable or unspecified
        "O411091",  # Infection of amniotic sac and membranes, unspecified, unspecified trimester, fetus 1
        "O411092",  # Infection of amniotic sac and membranes, unspecified, unspecified trimester, fetus 2
        "O411093",  # Infection of amniotic sac and membranes, unspecified, unspecified trimester, fetus 3
        "O411094",  # Infection of amniotic sac and membranes, unspecified, unspecified trimester, fetus 4
        "O411095",  # Infection of amniotic sac and membranes, unspecified, unspecified trimester, fetus 5
        "O411099",  # Infection of amniotic sac and membranes, unspecified, unspecified trimester, other fetus
        "O411210",  # Chorioamnionitis, first trimester, not applicable or unspecified
        "O411211",  # Chorioamnionitis, first trimester, fetus 1
        "O411212",  # Chorioamnionitis, first trimester, fetus 2
        "O411213",  # Chorioamnionitis, first trimester, fetus 3
        "O411214",  # Chorioamnionitis, first trimester, fetus 4
        "O411215",  # Chorioamnionitis, first trimester, fetus 5
        "O411219",  # Chorioamnionitis, first trimester, other fetus
        "O411220",  # Chorioamnionitis, second trimester, not applicable or unspecified
        "O411221",  # Chorioamnionitis, second trimester, fetus 1
        "O411222",  # Chorioamnionitis, second trimester, fetus 2
        "O411223",  # Chorioamnionitis, second trimester, fetus 3
        "O411224",  # Chorioamnionitis, second trimester, fetus 4
        "O411225",  # Chorioamnionitis, second trimester, fetus 5
        "O411229",  # Chorioamnionitis, second trimester, other fetus
        "O411230",  # Chorioamnionitis, third trimester, not applicable or unspecified
        "O411231",  # Chorioamnionitis, third trimester, fetus 1
        "O411232",  # Chorioamnionitis, third trimester, fetus 2
        "O411233",  # Chorioamnionitis, third trimester, fetus 3
        "O411234",  # Chorioamnionitis, third trimester, fetus 4
        "O411235",  # Chorioamnionitis, third trimester, fetus 5
        "O411239",  # Chorioamnionitis, third trimester, other fetus
        "O411290",  # Chorioamnionitis, unspecified trimester, not applicable or unspecified
        "O411291",  # Chorioamnionitis, unspecified trimester, fetus 1
        "O411292",  # Chorioamnionitis, unspecified trimester, fetus 2
        "O411293",  # Chorioamnionitis, unspecified trimester, fetus 3
        "O411294",  # Chorioamnionitis, unspecified trimester, fetus 4
        "O411295",  # Chorioamnionitis, unspecified trimester, fetus 5
        "O411299",  # Chorioamnionitis, unspecified trimester, other fetus
        "O411410",  # Placentitis, first trimester, not applicable or unspecified
        "O411411",  # Placentitis, first trimester, fetus 1
        "O411412",  # Placentitis, first trimester, fetus 2
        "O411413",  # Placentitis, first trimester, fetus 3
        "O411414",  # Placentitis, first trimester, fetus 4
        "O411415",  # Placentitis, first trimester, fetus 5
        "O411419",  # Placentitis, first trimester, other fetus
        "O411420",  # Placentitis, second trimester, not applicable or unspecified
        "O411421",  # Placentitis, second trimester, fetus 1
        "O411422",  # Placentitis, second trimester, fetus 2
        "O411423",  # Placentitis, second trimester, fetus 3
        "O411424",  # Placentitis, second trimester, fetus 4
        "O411425",  # Placentitis, second trimester, fetus 5
        "O411429",  # Placentitis, second trimester, other fetus
        "O411430",  # Placentitis, third trimester, not applicable or unspecified
        "O411431",  # Placentitis, third trimester, fetus 1
        "O411432",  # Placentitis, third trimester, fetus 2
        "O411433",  # Placentitis, third trimester, fetus 3
        "O411434",  # Placentitis, third trimester, fetus 4
        "O411435",  # Placentitis, third trimester, fetus 5
        "O411439",  # Placentitis, third trimester, other fetus
        "O411490",  # Placentitis, unspecified trimester, not applicable or unspecified
        "O411491",  # Placentitis, unspecified trimester, fetus 1
        "O411492",  # Placentitis, unspecified trimester, fetus 2
        "O411493",  # Placentitis, unspecified trimester, fetus 3
        "O411494",  # Placentitis, unspecified trimester, fetus 4
        "O411495",  # Placentitis, unspecified trimester, fetus 5
        "O411499",  # Placentitis, unspecified trimester, other fetus
        "O418X10",  # Other specified disorders of amniotic fluid and membranes, first trimester, not applicable or unspecified
        "O418X11",  # Other specified disorders of amniotic fluid and membranes, first trimester, fetus 1
        "O418X12",  # Other specified disorders of amniotic fluid and membranes, first trimester, fetus 2
        "O418X13",  # Other specified disorders of amniotic fluid and membranes, first trimester, fetus 3
        "O418X14",  # Other specified disorders of amniotic fluid and membranes, first trimester, fetus 4
        "O418X15",  # Other specified disorders of amniotic fluid and membranes, first trimester, fetus 5
        "O418X19",  # Other specified disorders of amniotic fluid and membranes, first trimester, other fetus
        "O418X20",  # Other specified disorders of amniotic fluid and membranes, second trimester, not applicable or unspecified
        "O418X21",  # Other specified disorders of amniotic fluid and membranes, second trimester, fetus 1
        "O418X22",  # Other specified disorders of amniotic fluid and membranes, second trimester, fetus 2
        "O418X23",  # Other specified disorders of amniotic fluid and membranes, second trimester, fetus 3
        "O418X24",  # Other specified disorders of amniotic fluid and membranes, second trimester, fetus 4
        "O418X25",  # Other specified disorders of amniotic fluid and membranes, second trimester, fetus 5
        "O418X29",  # Other specified disorders of amniotic fluid and membranes, second trimester, other fetus
        "O418X30",  # Other specified disorders of amniotic fluid and membranes, third trimester, not applicable or unspecified
        "O418X31",  # Other specified disorders of amniotic fluid and membranes, third trimester, fetus 1
        "O418X32",  # Other specified disorders of amniotic fluid and membranes, third trimester, fetus 2
        "O418X33",  # Other specified disorders of amniotic fluid and membranes, third trimester, fetus 3
        "O418X34",  # Other specified disorders of amniotic fluid and membranes, third trimester, fetus 4
        "O418X35",  # Other specified disorders of amniotic fluid and membranes, third trimester, fetus 5
        "O418X39",  # Other specified disorders of amniotic fluid and membranes, third trimester, other fetus
        "O418X90",  # Other specified disorders of amniotic fluid and membranes, unspecified trimester, not applicable or unspecified
        "O418X91",  # Other specified disorders of amniotic fluid and membranes, unspecified trimester, fetus 1
        "O418X92",  # Other specified disorders of amniotic fluid and membranes, unspecified trimester, fetus 2
        "O418X93",  # Other specified disorders of amniotic fluid and membranes, unspecified trimester, fetus 3
        "O418X94",  # Other specified disorders of amniotic fluid and membranes, unspecified trimester, fetus 4
        "O418X95",  # Other specified disorders of amniotic fluid and membranes, unspecified trimester, fetus 5
        "O418X99",  # Other specified disorders of amniotic fluid and membranes, unspecified trimester, other fetus
        "O4190X0",  # Disorder of amniotic fluid and membranes, unspecified, unspecified trimester, not applicable or unspecified
        "O4190X1",  # Disorder of amniotic fluid and membranes, unspecified, unspecified trimester, fetus 1
        "O4190X2",  # Disorder of amniotic fluid and membranes, unspecified, unspecified trimester, fetus 2
        "O4190X3",  # Disorder of amniotic fluid and membranes, unspecified, unspecified trimester, fetus 3
        "O4190X4",  # Disorder of amniotic fluid and membranes, unspecified, unspecified trimester, fetus 4
        "O4190X5",  # Disorder of amniotic fluid and membranes, unspecified, unspecified trimester, fetus 5
        "O4190X9",  # Disorder of amniotic fluid and membranes, unspecified, unspecified trimester, other fetus
        "O4191X0",  # Disorder of amniotic fluid and membranes, unspecified, first trimester, not applicable or unspecified
        "O4191X1",  # Disorder of amniotic fluid and membranes, unspecified, first trimester, fetus 1
        "O4191X2",  # Disorder of amniotic fluid and membranes, unspecified, first trimester, fetus 2
        "O4191X3",  # Disorder of amniotic fluid and membranes, unspecified, first trimester, fetus 3
        "O4191X4",  # Disorder of amniotic fluid and membranes, unspecified, first trimester, fetus 4
        "O4191X5",  # Disorder of amniotic fluid and membranes, unspecified, first trimester, fetus 5
        "O4191X9",  # Disorder of amniotic fluid and membranes, unspecified, first trimester, other fetus
        "O4192X0",  # Disorder of amniotic fluid and membranes, unspecified, second trimester, not applicable or unspecified
        "O4192X1",  # Disorder of amniotic fluid and membranes, unspecified, second trimester, fetus 1
        "O4192X2",  # Disorder of amniotic fluid and membranes, unspecified, second trimester, fetus 2
        "O4192X3",  # Disorder of amniotic fluid and membranes, unspecified, second trimester, fetus 3
        "O4192X4",  # Disorder of amniotic fluid and membranes, unspecified, second trimester, fetus 4
        "O4192X5",  # Disorder of amniotic fluid and membranes, unspecified, second trimester, fetus 5
        "O4192X9",  # Disorder of amniotic fluid and membranes, unspecified, second trimester, other fetus
        "O4193X0",  # Disorder of amniotic fluid and membranes, unspecified, third trimester, not applicable or unspecified
        "O4193X1",  # Disorder of amniotic fluid and membranes, unspecified, third trimester, fetus 1
        "O4193X2",  # Disorder of amniotic fluid and membranes, unspecified, third trimester, fetus 2
        "O4193X3",  # Disorder of amniotic fluid and membranes, unspecified, third trimester, fetus 3
        "O4193X4",  # Disorder of amniotic fluid and membranes, unspecified, third trimester, fetus 4
        "O4193X5",  # Disorder of amniotic fluid and membranes, unspecified, third trimester, fetus 5
        "O4193X9",  # Disorder of amniotic fluid and membranes, unspecified, third trimester, other fetus
        "O4200",  # Premature rupture of membranes, onset of labor within 24 hours of rupture, unspecified weeks of gestation
        "O42011",  # Preterm premature rupture of membranes, onset of labor within 24 hours of rupture, first trimester
        "O42012",  # Preterm premature rupture of membranes, onset of labor within 24 hours of rupture, second trimester
        "O42013",  # Preterm premature rupture of membranes, onset of labor within 24 hours of rupture, third trimester
        "O42019",  # Preterm premature rupture of membranes, onset of labor within 24 hours of rupture, unspecified trimester
        "O4202",  # Full-term premature rupture of membranes, onset of labor within 24 hours of rupture
        "O4210",  # Premature rupture of membranes, onset of labor more than 24 hours following rupture, unspecified weeks of gestation
        "O42111",  # Preterm premature rupture of membranes, onset of labor more than 24 hours following rupture, first trimester
        "O42112",  # Preterm premature rupture of membranes, onset of labor more than 24 hours following rupture, second trimester
        "O42113",  # Preterm premature rupture of membranes, onset of labor more than 24 hours following rupture, third trimester
        "O42119",  # Preterm premature rupture of membranes, onset of labor more than 24 hours following rupture, unspecified trimester
        "O4212",  # Full-term premature rupture of membranes, onset of labor more than 24 hours following rupture
        "O4290",  # Premature rupture of membranes, unspecified as to length of time between rupture and onset of labor, unspecified weeks of gestation
        "O42911",  # Preterm premature rupture of membranes, unspecified as to length of time between rupture and onset of labor, first trimester
        "O42912",  # Preterm premature rupture of membranes, unspecified as to length of time between rupture and onset of labor, second trimester
        "O42913",  # Preterm premature rupture of membranes, unspecified as to length of time between rupture and onset of labor, third trimester
        "O42919",  # Preterm premature rupture of membranes, unspecified as to length of time between rupture and onset of labor, unspecified trimester
        "O4292",  # Full-term premature rupture of membranes, unspecified as to length of time between rupture and onset of labor
        "O43011",  # Fetomaternal placental transfusion syndrome, first trimester
        "O43012",  # Fetomaternal placental transfusion syndrome, second trimester
        "O43013",  # Fetomaternal placental transfusion syndrome, third trimester
        "O43019",  # Fetomaternal placental transfusion syndrome, unspecified trimester
        "O43021",  # Fetus-to-fetus placental transfusion syndrome, first trimester
        "O43022",  # Fetus-to-fetus placental transfusion syndrome, second trimester
        "O43023",  # Fetus-to-fetus placental transfusion syndrome, third trimester
        "O43029",  # Fetus-to-fetus placental transfusion syndrome, unspecified trimester
        "O43101",  # Malformation of placenta, unspecified, first trimester
        "O43102",  # Malformation of placenta, unspecified, second trimester
        "O43103",  # Malformation of placenta, unspecified, third trimester
        "O43109",  # Malformation of placenta, unspecified, unspecified trimester
        "O43111",  # Circumvallate placenta, first trimester
        "O43112",  # Circumvallate placenta, second trimester
        "O43113",  # Circumvallate placenta, third trimester
        "O43119",  # Circumvallate placenta, unspecified trimester
        "O43121",  # Velamentous insertion of umbilical cord, first trimester
        "O43122",  # Velamentous insertion of umbilical cord, second trimester
        "O43123",  # Velamentous insertion of umbilical cord, third trimester
        "O43129",  # Velamentous insertion of umbilical cord, unspecified trimester
        "O43191",  # Other malformation of placenta, first trimester
        "O43192",  # Other malformation of placenta, second trimester
        "O43193",  # Other malformation of placenta, third trimester
        "O43199",  # Other malformation of placenta, unspecified trimester
        "O43211",  # Placenta accreta, first trimester
        "O43212",  # Placenta accreta, second trimester
        "O43213",  # Placenta accreta, third trimester
        "O43219",  # Placenta accreta, unspecified trimester
        "O43221",  # Placenta increta, first trimester
        "O43222",  # Placenta increta, second trimester
        "O43223",  # Placenta increta, third trimester
        "O43229",  # Placenta increta, unspecified trimester
        "O43231",  # Placenta percreta, first trimester
        "O43232",  # Placenta percreta, second trimester
        "O43233",  # Placenta percreta, third trimester
        "O43239",  # Placenta percreta, unspecified trimester
        "O43811",  # Placental infarction, first trimester
        "O43812",  # Placental infarction, second trimester
        "O43813",  # Placental infarction, third trimester
        "O43819",  # Placental infarction, unspecified trimester
        "O43891",  # Other placental disorders, first trimester
        "O43892",  # Other placental disorders, second trimester
        "O43893",  # Other placental disorders, third trimester
        "O43899",  # Other placental disorders, unspecified trimester
        "O4390",  # Unspecified placental disorder, unspecified trimester
        "O4391",  # Unspecified placental disorder, first trimester
        "O4392",  # Unspecified placental disorder, second trimester
        "O4393",  # Unspecified placental disorder, third trimester
        "O4400",  # Complete placenta previa NOS or without hemorrhage, unspecified trimester
        "O4401",  # Complete placenta previa NOS or without hemorrhage, first trimester
        "O4402",  # Complete placenta previa NOS or without hemorrhage, second trimester
        "O4403",  # Complete placenta previa NOS or without hemorrhage, third trimester
        "O4410",  # Complete placenta previa with hemorrhage, unspecified trimester
        "O4411",  # Complete placenta previa with hemorrhage, first trimester
        "O4412",  # Complete placenta previa with hemorrhage, second trimester
        "O4413",  # Complete placenta previa with hemorrhage, third trimester
        "O4420",  # Partial placenta previa NOS or without hemorrhage, unspecified trimester
        "O4421",  # Partial placenta previa NOS or without hemorrhage, first trimester
        "O4422",  # Partial placenta previa NOS or without hemorrhage, second trimester
        "O4423",  # Partial placenta previa NOS or without hemorrhage, third trimester
        "O4430",  # Partial placenta previa with hemorrhage, unspecified trimester
        "O4431",  # Partial placenta previa with hemorrhage, first trimester
        "O4432",  # Partial placenta previa with hemorrhage, second trimester
        "O4433",  # Partial placenta previa with hemorrhage, third trimester
        "O4440",  # Low lying placenta NOS or without hemorrhage, unspecified trimester
        "O4441",  # Low lying placenta NOS or without hemorrhage, first trimester
        "O4442",  # Low lying placenta NOS or without hemorrhage, second trimester
        "O4443",  # Low lying placenta NOS or without hemorrhage, third trimester
        "O4450",  # Low lying placenta with hemorrhage, unspecified trimester
        "O4451",  # Low lying placenta with hemorrhage, first trimester
        "O4452",  # Low lying placenta with hemorrhage, second trimester
        "O4453",  # Low lying placenta with hemorrhage, third trimester
        "O45001",  # Premature separation of placenta with coagulation defect, unspecified, first trimester
        "O45002",  # Premature separation of placenta with coagulation defect, unspecified, second trimester
        "O45003",  # Premature separation of placenta with coagulation defect, unspecified, third trimester
        "O45009",  # Premature separation of placenta with coagulation defect, unspecified, unspecified trimester
        "O45011",  # Premature separation of placenta with afibrinogenemia, first trimester
        "O45012",  # Premature separation of placenta with afibrinogenemia, second trimester
        "O45013",  # Premature separation of placenta with afibrinogenemia, third trimester
        "O45019",  # Premature separation of placenta with afibrinogenemia, unspecified trimester
        "O45021",  # Premature separation of placenta with disseminated intravascular coagulation, first trimester
        "O45022",  # Premature separation of placenta with disseminated intravascular coagulation, second trimester
        "O45023",  # Premature separation of placenta with disseminated intravascular coagulation, third trimester
        "O45029",  # Premature separation of placenta with disseminated intravascular coagulation, unspecified trimester
        "O45091",  # Premature separation of placenta with other coagulation defect, first trimester
        "O45092",  # Premature separation of placenta with other coagulation defect, second trimester
        "O45093",  # Premature separation of placenta with other coagulation defect, third trimester
        "O45099",  # Premature separation of placenta with other coagulation defect, unspecified trimester
        "O458X1",  # Other premature separation of placenta, first trimester
        "O458X2",  # Other premature separation of placenta, second trimester
        "O458X3",  # Other premature separation of placenta, third trimester
        "O458X9",  # Other premature separation of placenta, unspecified trimester
        "O4590",  # Premature separation of placenta, unspecified, unspecified trimester
        "O4591",  # Premature separation of placenta, unspecified, first trimester
        "O4592",  # Premature separation of placenta, unspecified, second trimester
        "O4593",  # Premature separation of placenta, unspecified, third trimester
        "O46001",  # Antepartum hemorrhage with coagulation defect, unspecified, first trimester
        "O46002",  # Antepartum hemorrhage with coagulation defect, unspecified, second trimester
        "O46003",  # Antepartum hemorrhage with coagulation defect, unspecified, third trimester
        "O46009",  # Antepartum hemorrhage with coagulation defect, unspecified, unspecified trimester
        "O46011",  # Antepartum hemorrhage with afibrinogenemia, first trimester
        "O46012",  # Antepartum hemorrhage with afibrinogenemia, second trimester
        "O46013",  # Antepartum hemorrhage with afibrinogenemia, third trimester
        "O46019",  # Antepartum hemorrhage with afibrinogenemia, unspecified trimester
        "O46021",  # Antepartum hemorrhage with disseminated intravascular coagulation, first trimester
        "O46022",  # Antepartum hemorrhage with disseminated intravascular coagulation, second trimester
        "O46023",  # Antepartum hemorrhage with disseminated intravascular coagulation, third trimester
        "O46029",  # Antepartum hemorrhage with disseminated intravascular coagulation, unspecified trimester
        "O46091",  # Antepartum hemorrhage with other coagulation defect, first trimester
        "O46092",  # Antepartum hemorrhage with other coagulation defect, second trimester
        "O46093",  # Antepartum hemorrhage with other coagulation defect, third trimester
        "O46099",  # Antepartum hemorrhage with other coagulation defect, unspecified trimester
        "O468X1",  # Other antepartum hemorrhage, first trimester
        "O468X2",  # Other antepartum hemorrhage, second trimester
        "O468X3",  # Other antepartum hemorrhage, third trimester
        "O468X9",  # Other antepartum hemorrhage, unspecified trimester
        "O4690",  # Antepartum hemorrhage, unspecified, unspecified trimester
        "O4691",  # Antepartum hemorrhage, unspecified, first trimester
        "O4692",  # Antepartum hemorrhage, unspecified, second trimester
        "O4693",  # Antepartum hemorrhage, unspecified, third trimester
        "O4700",  # False labor before 37 completed weeks of gestation, unspecified trimester
        "O4702",  # False labor before 37 completed weeks of gestation, second trimester
        "O4703",  # False labor before 37 completed weeks of gestation, third trimester
        "O471",  # False labor at or after 37 completed weeks of gestation
        "O479",  # False labor, unspecified
        "O480",  # Post-term pregnancy
        "O481",  # Prolonged pregnancy
        "O6000",  # Preterm labor without delivery, unspecified trimester
        "O6002",  # Preterm labor without delivery, second trimester
        "O6003",  # Preterm labor without delivery, third trimester
        "O7100",  # Rupture of uterus before onset of labor, unspecified trimester
        "O7102",  # Rupture of uterus before onset of labor, second trimester
        "O7103",  # Rupture of uterus before onset of labor, third trimester
        "O88011",  # Air embolism in pregnancy, first trimester
        "O88012",  # Air embolism in pregnancy, second trimester
        "O88013",  # Air embolism in pregnancy, third trimester
        "O88019",  # Air embolism in pregnancy, unspecified trimester
        "O88111",  # Amniotic fluid embolism in pregnancy, first trimester
        "O88112",  # Amniotic fluid embolism in pregnancy, second trimester
        "O88113",  # Amniotic fluid embolism in pregnancy, third trimester
        "O88119",  # Amniotic fluid embolism in pregnancy, unspecified trimester
        "O88211",  # Thromboembolism in pregnancy, first trimester
        "O88212",  # Thromboembolism in pregnancy, second trimester
        "O88213",  # Thromboembolism in pregnancy, third trimester
        "O88219",  # Thromboembolism in pregnancy, unspecified trimester
        "O88311",  # Pyemic and septic embolism in pregnancy, first trimester
        "O88312",  # Pyemic and septic embolism in pregnancy, second trimester
        "O88313",  # Pyemic and septic embolism in pregnancy, third trimester
        "O88319",  # Pyemic and septic embolism in pregnancy, unspecified trimester
        "O88811",  # Other embolism in pregnancy, first trimester
        "O88812",  # Other embolism in pregnancy, second trimester
        "O88813",  # Other embolism in pregnancy, third trimester
        "O88819",  # Other embolism in pregnancy, unspecified trimester
        "O903",  # Peripartum cardiomyopathy
        "O91011",  # Infection of nipple associated with pregnancy, first trimester
        "O91012",  # Infection of nipple associated with pregnancy, second trimester
        "O91013",  # Infection of nipple associated with pregnancy, third trimester
        "O91019",  # Infection of nipple associated with pregnancy, unspecified trimester
        "O91111",  # Abscess of breast associated with pregnancy, first trimester
        "O91112",  # Abscess of breast associated with pregnancy, second trimester
        "O91113",  # Abscess of breast associated with pregnancy, third trimester
        "O91119",  # Abscess of breast associated with pregnancy, unspecified trimester
        "O91211",  # Nonpurulent mastitis associated with pregnancy, first trimester
        "O91212",  # Nonpurulent mastitis associated with pregnancy, second trimester
        "O91213",  # Nonpurulent mastitis associated with pregnancy, third trimester
        "O91219",  # Nonpurulent mastitis associated with pregnancy, unspecified trimester
        "O92011",  # Retracted nipple associated with pregnancy, first trimester
        "O92012",  # Retracted nipple associated with pregnancy, second trimester
        "O92013",  # Retracted nipple associated with pregnancy, third trimester
        "O92019",  # Retracted nipple associated with pregnancy, unspecified trimester
        "O92111",  # Cracked nipple associated with pregnancy, first trimester
        "O92112",  # Cracked nipple associated with pregnancy, second trimester
        "O92113",  # Cracked nipple associated with pregnancy, third trimester
        "O92119",  # Cracked nipple associated with pregnancy, unspecified trimester
        "O9220",  # Unspecified disorder of breast associated with pregnancy and the puerperium
        "O9229",  # Other disorders of breast associated with pregnancy and the puerperium
        "O98011",  # Tuberculosis complicating pregnancy, first trimester
        "O98012",  # Tuberculosis complicating pregnancy, second trimester
        "O98013",  # Tuberculosis complicating pregnancy, third trimester
        "O98019",  # Tuberculosis complicating pregnancy, unspecified trimester
        "O98111",  # Syphilis complicating pregnancy, first trimester
        "O98112",  # Syphilis complicating pregnancy, second trimester
        "O98113",  # Syphilis complicating pregnancy, third trimester
        "O98119",  # Syphilis complicating pregnancy, unspecified trimester
        "O98211",  # Gonorrhea complicating pregnancy, first trimester
        "O98212",  # Gonorrhea complicating pregnancy, second trimester
        "O98213",  # Gonorrhea complicating pregnancy, third trimester
        "O98219",  # Gonorrhea complicating pregnancy, unspecified trimester
        "O98311",  # Other infections with a predominantly sexual mode of transmission complicating pregnancy, first trimester
        "O98312",  # Other infections with a predominantly sexual mode of transmission complicating pregnancy, second trimester
        "O98313",  # Other infections with a predominantly sexual mode of transmission complicating pregnancy, third trimester
        "O98319",  # Other infections with a predominantly sexual mode of transmission complicating pregnancy, unspecified trimester
        "O98411",  # Viral hepatitis complicating pregnancy, first trimester
        "O98412",  # Viral hepatitis complicating pregnancy, second trimester
        "O98413",  # Viral hepatitis complicating pregnancy, third trimester
        "O98419",  # Viral hepatitis complicating pregnancy, unspecified trimester
        "O98511",  # Other viral diseases complicating pregnancy, first trimester
        "O98512",  # Other viral diseases complicating pregnancy, second trimester
        "O98513",  # Other viral diseases complicating pregnancy, third trimester
        "O98519",  # Other viral diseases complicating pregnancy, unspecified trimester
        "O98611",  # Protozoal diseases complicating pregnancy, first trimester
        "O98612",  # Protozoal diseases complicating pregnancy, second trimester
        "O98613",  # Protozoal diseases complicating pregnancy, third trimester
        "O98619",  # Protozoal diseases complicating pregnancy, unspecified trimester
        "O98711",  # Human immunodeficiency virus [HIV] disease complicating pregnancy, first trimester
        "O98712",  # Human immunodeficiency virus [HIV] disease complicating pregnancy, second trimester
        "O98713",  # Human immunodeficiency virus [HIV] disease complicating pregnancy, third trimester
        "O98719",  # Human immunodeficiency virus [HIV] disease complicating pregnancy, unspecified trimester
        "O98811",  # Other maternal infectious and parasitic diseases complicating pregnancy, first trimester
        "O98812",  # Other maternal infectious and parasitic diseases complicating pregnancy, second trimester
        "O98813",  # Other maternal infectious and parasitic diseases complicating pregnancy, third trimester
        "O98819",  # Other maternal infectious and parasitic diseases complicating pregnancy, unspecified trimester
        "O98911",  # Unspecified maternal infectious and parasitic disease complicating pregnancy, first trimester
        "O98912",  # Unspecified maternal infectious and parasitic disease complicating pregnancy, second trimester
        "O98913",  # Unspecified maternal infectious and parasitic disease complicating pregnancy, third trimester
        "O98919",  # Unspecified maternal infectious and parasitic disease complicating pregnancy, unspecified trimester
        "O99011",  # Anemia complicating pregnancy, first trimester
        "O99012",  # Anemia complicating pregnancy, second trimester
        "O99013",  # Anemia complicating pregnancy, third trimester
        "O99019",  # Anemia complicating pregnancy, unspecified trimester
        "O99111",  # Other diseases of the blood and blood-forming organs and certain disorders involving the immune mechanism complicating pregnancy, first trimester
        "O99112",  # Other diseases of the blood and blood-forming organs and certain disorders involving the immune mechanism complicating pregnancy, second trimester
        "O99113",  # Other diseases of the blood and blood-forming organs and certain disorders involving the immune mechanism complicating pregnancy, third trimester
        "O99119",  # Other diseases of the blood and blood-forming organs and certain disorders involving the immune mechanism complicating pregnancy, unspecified trimester
        "O99210",  # Obesity complicating pregnancy, unspecified trimester
        "O99211",  # Obesity complicating pregnancy, first trimester
        "O99212",  # Obesity complicating pregnancy, second trimester
        "O99213",  # Obesity complicating pregnancy, third trimester
        "O99280",  # Endocrine, nutritional and metabolic diseases complicating pregnancy, unspecified trimester
        "O99281",  # Endocrine, nutritional and metabolic diseases complicating pregnancy, first trimester
        "O99282",  # Endocrine, nutritional and metabolic diseases complicating pregnancy, second trimester
        "O99283",  # Endocrine, nutritional and metabolic diseases complicating pregnancy, third trimester
        "O99310",  # Alcohol use complicating pregnancy, unspecified trimester
        "O99311",  # Alcohol use complicating pregnancy, first trimester
        "O99312",  # Alcohol use complicating pregnancy, second trimester
        "O99313",  # Alcohol use complicating pregnancy, third trimester
        "O99320",  # Drug use complicating pregnancy, unspecified trimester
        "O99321",  # Drug use complicating pregnancy, first trimester
        "O99322",  # Drug use complicating pregnancy, second trimester
        "O99323",  # Drug use complicating pregnancy, third trimester
        "O99330",  # Smoking (tobacco) complicating pregnancy, unspecified trimester
        "O99331",  # Smoking (tobacco) complicating pregnancy, first trimester
        "O99332",  # Smoking (tobacco) complicating pregnancy, second trimester
        "O99333",  # Smoking (tobacco) complicating pregnancy, third trimester
        "O99340",  # Other mental disorders complicating pregnancy, unspecified trimester
        "O99341",  # Other mental disorders complicating pregnancy, first trimester
        "O99342",  # Other mental disorders complicating pregnancy, second trimester
        "O99343",  # Other mental disorders complicating pregnancy, third trimester
        "O99350",  # Diseases of the nervous system complicating pregnancy, unspecified trimester
        "O99351",  # Diseases of the nervous system complicating pregnancy, first trimester
        "O99352",  # Diseases of the nervous system complicating pregnancy, second trimester
        "O99353",  # Diseases of the nervous system complicating pregnancy, third trimester
        "O99411",  # Diseases of the circulatory system complicating pregnancy, first trimester
        "O99412",  # Diseases of the circulatory system complicating pregnancy, second trimester
        "O99413",  # Diseases of the circulatory system complicating pregnancy, third trimester
        "O99419",  # Diseases of the circulatory system complicating pregnancy, unspecified trimester
        "O99511",  # Diseases of the respiratory system complicating pregnancy, first trimester
        "O99512",  # Diseases of the respiratory system complicating pregnancy, second trimester
        "O99513",  # Diseases of the respiratory system complicating pregnancy, third trimester
        "O99519",  # Diseases of the respiratory system complicating pregnancy, unspecified trimester
        "O99611",  # Diseases of the digestive system complicating pregnancy, first trimester
        "O99612",  # Diseases of the digestive system complicating pregnancy, second trimester
        "O99613",  # Diseases of the digestive system complicating pregnancy, third trimester
        "O99619",  # Diseases of the digestive system complicating pregnancy, unspecified trimester
        "O99711",  # Diseases of the skin and subcutaneous tissue complicating pregnancy, first trimester
        "O99712",  # Diseases of the skin and subcutaneous tissue complicating pregnancy, second trimester
        "O99713",  # Diseases of the skin and subcutaneous tissue complicating pregnancy, third trimester
        "O99719",  # Diseases of the skin and subcutaneous tissue complicating pregnancy, unspecified trimester
        "O99810",  # Abnormal glucose complicating pregnancy
        "O99820",  # Streptococcus B carrier state complicating pregnancy
        "O99830",  # Other infection carrier state complicating pregnancy
        "O99840",  # Bariatric surgery status complicating pregnancy, unspecified trimester
        "O99841",  # Bariatric surgery status complicating pregnancy, first trimester
        "O99842",  # Bariatric surgery status complicating pregnancy, second trimester
        "O99843",  # Bariatric surgery status complicating pregnancy, third trimester
        "O9A111",  # Malignant neoplasm complicating pregnancy, first trimester
        "O9A112",  # Malignant neoplasm complicating pregnancy, second trimester
        "O9A113",  # Malignant neoplasm complicating pregnancy, third trimester
        "O9A119",  # Malignant neoplasm complicating pregnancy, unspecified trimester
        "O9A211",  # Injury, poisoning and certain other consequences of external causes complicating pregnancy, first trimester
        "O9A212",  # Injury, poisoning and certain other consequences of external causes complicating pregnancy, second trimester
        "O9A213",  # Injury, poisoning and certain other consequences of external causes complicating pregnancy, third trimester
        "O9A219",  # Injury, poisoning and certain other consequences of external causes complicating pregnancy, unspecified trimester
        "O9A311",  # Physical abuse complicating pregnancy, first trimester
        "O9A312",  # Physical abuse complicating pregnancy, second trimester
        "O9A313",  # Physical abuse complicating pregnancy, third trimester
        "O9A319",  # Physical abuse complicating pregnancy, unspecified trimester
        "O9A411",  # Sexual abuse complicating pregnancy, first trimester
        "O9A412",  # Sexual abuse complicating pregnancy, second trimester
        "O9A413",  # Sexual abuse complicating pregnancy, third trimester
        "O9A419",  # Sexual abuse complicating pregnancy, unspecified trimester
        "O9A511",  # Psychological abuse complicating pregnancy, first trimester
        "O9A512",  # Psychological abuse complicating pregnancy, second trimester
        "O9A513",  # Psychological abuse complicating pregnancy, third trimester
        "O9A519",  # Psychological abuse complicating pregnancy, unspecified trimester
        "Z3201",  # Encounter for pregnancy test, result positive
        "Z331",  # Pregnant state, incidental
        "Z333",  # Pregnant state, gestational carrier
        "Z3400",  # Encounter for supervision of normal first pregnancy, unspecified trimester
        "Z3401",  # Encounter for supervision of normal first pregnancy, first trimester
        "Z3402",  # Encounter for supervision of normal first pregnancy, second trimester
        "Z3403",  # Encounter for supervision of normal first pregnancy, third trimester
        "Z3480",  # Encounter for supervision of other normal pregnancy, unspecified trimester
        "Z3481",  # Encounter for supervision of other normal pregnancy, first trimester
        "Z3482",  # Encounter for supervision of other normal pregnancy, second trimester
        "Z3483",  # Encounter for supervision of other normal pregnancy, third trimester
        "Z3490",  # Encounter for supervision of normal pregnancy, unspecified, unspecified trimester
        "Z3491",  # Encounter for supervision of normal pregnancy, unspecified, first trimester
        "Z3492",  # Encounter for supervision of normal pregnancy, unspecified, second trimester
        "Z3493",  # Encounter for supervision of normal pregnancy, unspecified, third trimester
        "Z3A01",  # Less than 8 weeks gestation of pregnancy
        "Z3A08",  # 8 weeks gestation of pregnancy
        "Z3A09",  # 9 weeks gestation of pregnancy
        "Z3A10",  # 10 weeks gestation of pregnancy
        "Z3A11",  # 11 weeks gestation of pregnancy
        "Z3A12",  # 12 weeks gestation of pregnancy
        "Z3A13",  # 13 weeks gestation of pregnancy
        "Z3A14",  # 14 weeks gestation of pregnancy
        "Z3A15",  # 15 weeks gestation of pregnancy
        "Z3A16",  # 16 weeks gestation of pregnancy
        "Z3A17",  # 17 weeks gestation of pregnancy
        "Z3A18",  # 18 weeks gestation of pregnancy
        "Z3A19",  # 19 weeks gestation of pregnancy
        "Z3A20",  # 20 weeks gestation of pregnancy
        "Z3A21",  # 21 weeks gestation of pregnancy
        "Z3A22",  # 22 weeks gestation of pregnancy
        "Z3A23",  # 23 weeks gestation of pregnancy
        "Z3A24",  # 24 weeks gestation of pregnancy
        "Z3A25",  # 25 weeks gestation of pregnancy
        "Z3A26",  # 26 weeks gestation of pregnancy
        "Z3A27",  # 27 weeks gestation of pregnancy
        "Z3A28",  # 28 weeks gestation of pregnancy
        "Z3A29",  # 29 weeks gestation of pregnancy
        "Z3A30",  # 30 weeks gestation of pregnancy
        "Z3A31",  # 31 weeks gestation of pregnancy
        "Z3A32",  # 32 weeks gestation of pregnancy
        "Z3A33",  # 33 weeks gestation of pregnancy
        "Z3A34",  # 34 weeks gestation of pregnancy
        "Z3A35",  # 35 weeks gestation of pregnancy
        "Z3A36",  # 36 weeks gestation of pregnancy
        "Z3A37",  # 37 weeks gestation of pregnancy
        "Z3A38",  # 38 weeks gestation of pregnancy
        "Z3A39",  # 39 weeks gestation of pregnancy
        "Z3A40",  # 40 weeks gestation of pregnancy
        "Z3A41",  # 41 weeks gestation of pregnancy
        "Z3A42",  # 42 weeks gestation of pregnancy
        "Z3A49",  # Greater than 42 weeks gestation of pregnancy
    }

    SNOMEDCT = {
        "10231000132102",  # In-vitro fertilization pregnancy (finding)
        "102872000",  # Pregnancy on oral contraceptive (finding)
        "102875003",  # Surrogate pregnancy (finding)
        "11082009",  # Abnormal pregnancy (finding)
        "127364007",  # Primigravida (finding)
        "1290152008",  # Singleton pregnancy (finding)
        "134781000119106",  # High risk pregnancy due to recurrent miscarriage (finding)
        "14080002",  # Illegitimate pregnancy, life event (finding)
        "14418008",  # Precocious pregnancy (finding)
        "16356006",  # Multiple pregnancy (finding)
        "169560008",  # Pregnant - urine test confirms (finding)
        "169561007",  # Pregnant - blood test confirms (finding)
        "169562000",  # Pregnant - vaginal examination confirms (finding)
        "169563005",  # Pregnant - on history (finding)
        "169564004",  # Pregnant - on abdominal palpation (finding)
        "169565003",  # Pregnant - planned (finding)
        "169566002",  # Pregnancy unplanned but wanted (finding)
        "169567006",  # Pregnancy unplanned and unwanted (finding)
        "169568001",  # Unplanned pregnancy unknown if child is wanted (finding)
        "199306007",  # Continuing pregnancy after abortion of one fetus or more (disorder)
        "22281000119101",  # Post-term pregnancy of 40 to 42 weeks (finding)
        "237233002",  # Concealed pregnancy (finding)
        "237238006",  # Pregnancy with uncertain dates (finding)
        "237239003",  # Low risk pregnancy (finding)
        "237240001",  # Teenage pregnancy (finding)
        "237241002",  # Viable pregnancy (finding)
        "248985009",  # Presentation of pregnancy (finding)
        "276367008",  # Wanted pregnancy (finding)
        "281307002",  # Uncertain viability of pregnancy (finding)
        "314204000",  # Early stage of pregnancy (finding)
        "35381000119101",  # Quadruplet pregnancy with loss of one or more fetuses (disorder)
        "36801000119105",  # Continuing triplet pregnancy after spontaneous abortion of one or more fetuses (disorder)
        "38720006",  # Septuplet pregnancy (finding)
        "41587001",  # Third trimester pregnancy (finding)
        "41991004",  # Angiectasis pregnancy (disorder)
        "429187001",  # Continuing pregnancy after intrauterine death of twin fetus (disorder)
        "43990006",  # Sextuplet pregnancy (finding)
        "442478007",  # Multiple pregnancy involving intrauterine pregnancy and tubal pregnancy (disorder)
        "444661007",  # High risk pregnancy due to history of preterm labor (finding)
        "45307008",  # Extrachorial pregnancy (finding)
        "457811000124103",  # Normal pregnancy in primigravida (finding)
        "457821000124106",  # Normal pregnancy in multigravida (finding)
        "459167000",  # Monochorionic twin pregnancy (finding)
        "459169002",  # Monochorionic diamniotic twin pregnancy with similar amniotic fluid volumes (finding)
        "459170001",  # Monochorionic diamniotic twin pregnancy with dissimilar amniotic fluid volumes (disorder)
        "47200007",  # High risk pregnancy (finding)
        "472321009",  # Continuing pregnancy after intrauterine death of one twin with intrauterine retention of dead twin (disorder)
        "567941000005109",  # Pregnancy with certain dates (finding)
        "57630001",  # First trimester pregnancy (finding)
        "58532003",  # Unwanted pregnancy (finding)
        "59466002",  # Second trimester pregnancy (finding)
        "60810003",  # Quadruplet pregnancy (finding)
        "64254006",  # Triplet pregnancy (finding)
        "65147003",  # Twin pregnancy (finding)
        "65727000",  # Intrauterine pregnancy (finding)
        "713575004",  # Dizygotic twin pregnancy (finding)
        "713576003",  # Monozygotic twin pregnancy (finding)
        "72892002",  # Normal pregnancy (finding)
        "77386006",  # Pregnancy (finding)
        "80997009",  # Quintuplet pregnancy (finding)
        "83074005",  # Unplanned pregnancy (finding)
        "87527008",  # Term pregnancy (finding)
        "90968009",  # Prolonged pregnancy (disorder)
        "9279009",  # Extra-amniotic pregnancy (finding)
    }

class Hiv(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of human immunodeficiency virus (HIV) disease.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of HIV and acquired immunodeficiency syndrome.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "HIV"
    OID = "2.16.840.1.113883.3.464.1003.120.12.1003"
    DEFINITION_VERSION = "20240106"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "B20",  # Human immunodeficiency virus [HIV] disease
        "B9735",  # Human immunodeficiency virus, type 2 [HIV 2] as the cause of diseases classified elsewhere
        "Z21",  # Asymptomatic human immunodeficiency virus [HIV] infection status
    }

    SNOMEDCT = {
        "91923005",  # Acquired immunodeficiency syndrome virus infection associated with pregnancy (disorder)
        "421510000",  # Isosporiasis associated with acquired immunodeficiency syndrome (disorder)
        "421666009",  # Toxoplasmosis associated with acquired immunodeficiency syndrome (disorder)
        "422074008",  # Coccidiosis associated with acquired immunodeficiency syndrome (disorder)
        "76981000119106",  # Human immunodeficiency virus (HIV) infection category B1 (disorder)
        "76991000119109",  # Human immunodeficiency virus (HIV) infection category B2 (disorder)
        "235009000",  # Human immunodeficiency virus-associated periodontitis (disorder)
        "860874006",  # Enteritis due to human immunodeficiency virus disease (disorder)
        "10746341000119109",  # Acquired immune deficiency syndrome complicating childbirth (disorder)
        "10755671000119100",  # Human immunodeficiency virus in mother complicating childbirth (disorder)
        "111880001",  # Acute human immunodeficiency virus infection (disorder)
        "1142045004",  # Primary human immunodeficiency virus infection (disorder)
        "1142055000",  # Human immunodeficiency virus modified skin disease (disorder)
        "1187000005",  # Infectious mononucleosis caused by human immunodeficiency virus (disorder)
        "1187001009",  # Infectious mononucleosis caused by human immunodeficiency virus type I (disorder)
        "1260096003",  # Wasting syndrome due to acquired immunodeficiency syndrome (disorder)
        "15928141000119107",  # Cognitive impairment co-occurrent and due to human immunodeficiency virus infection (disorder)
        "165816005",  # Human immunodeficiency virus detected (finding)
        "186706006",  # Human immunodeficiency virus infection constitutional disease (disorder)
        "186707002",  # Human immunodeficiency virus infection with neurological disease (disorder)
        "186708007",  # Human immunodeficiency virus infection with secondary clinical infectious disease (disorder)
        "230180003",  # Human immunodeficiency virus leukoencephalopathy (disorder)
        "230202002",  # Vacuolar myelopathy (disorder)
        "230598008",  # Neuropathy caused by human immunodeficiency virus (disorder)
        "235726002",  # Human immunodeficiency virus enteropathy (disorder)
        "236406007",  # Acquired immune deficiency syndrome-related nephropathy (disorder)
        "240103002",  # Human immunodeficiency virus myopathy (disorder)
        "276665006",  # Congenital acquired immune deficiency syndrome (disorder)
        "276666007",  # Congenital human immunodeficiency virus positive status syndrome (disorder)
        "281388009",  # Human immunodeficiency virus-related sclerosing cholangitis (disorder)
        "315019000",  # Aseptic meningitis due to human immunodeficiency virus infection (disorder)
        "397763006",  # Human immunodeficiency virus encephalopathy (disorder)
        "398329009",  # Human immunodeficiency virus encephalitis (disorder)
        "402901009",  # Oral hairy leukoplakia associated with human immunodeficiency virus disease (disorder)
        "405631006",  # Pediatric human immunodeficiency virus infection (disorder)
        "406109008",  # Human immunodeficiency virus detected by enzyme-linked immunosorbent assay (finding)
        "40780007",  # Human immunodeficiency virus I infection (disorder)
        "414376003",  # Hairy leukoplakia of tongue associated with human immunodeficiency virus disease (disorder)
        "414604009",  # Leukoplakia of tongue associated with human immunodeficiency virus disease (disorder)
        "416729007",  # Neutropenia with acquired immunodeficiency syndrome (disorder)
        "420244003",  # Encephalitis with acquired immunodeficiency syndrome (disorder)
        "420281004",  # Skin rash with acquired immunodeficiency syndrome (disorder)
        "420302007",  # Reticulosarcoma with acquired immunodeficiency syndrome (disorder)
        "420308006",  # Retinal vascular changes with acquired immunodeficiency syndrome (disorder)
        "420321004",  # Intestinal malabsorption with acquired immunodeficiency syndrome (disorder)
        "420384005",  # Hematopoietic system disease with acquired immunodeficiency syndrome (disorder)
        "420395004",  # Acute endocarditis with acquired immunodeficiency syndrome (disorder)
        "420403001",  # Pneumocystosis with acquired immunodeficiency syndrome (disorder)
        "420452002",  # Myelopathy with acquired immunodeficiency syndrome (disorder)
        "420524008",  # Kaposi's sarcoma with acquired immunodeficiency syndrome (disorder)
        "420543008",  # Anemia associated with acquired immunodeficiency syndrome (disorder)
        "420544002",  # Bacterial pneumonia with acquired immunodeficiency syndrome (disorder)
        "420549007",  # Salivary gland disease with acquired immunodeficiency syndrome (disorder)
        "420554003",  # Progressive multifocal leukoencephalopathy with acquired immunodeficiency syndrome (disorder)
        "420614009",  # Organic dementia with acquired immunodeficiency syndrome (disorder)
        "420658009",  # Radiculitis with acquired immunodeficiency syndrome (disorder)
        "420691000",  # Nutritional deficiency with acquired immunodeficiency syndrome (disorder)
        "420718004",  # Central nervous system demyelinating disease with acquired immunodeficiency syndrome (disorder)
        "420721002",  # Acquired immunodeficiency syndrome-associated disorder (disorder)
        "420764009",  # Salmonella infection with acquired immunodeficiency syndrome (disorder)
        "420774007",  # Organic brain syndrome with acquired immunodeficiency syndrome (disorder)
        "420787001",  # Pneumococcal pneumonia with acquired immunodeficiency syndrome (disorder)
        "420801006",  # Malaise with acquired immunodeficiency syndrome (disorder)
        "420818005",  # Mycobacteriosis with acquired immunodeficiency syndrome (disorder)
        "420877009",  # Dermatomycosis with acquired immunodeficiency syndrome (disorder)
        "420900006",  # Fatigue with acquired immunodeficiency syndrome (disorder)
        "420938005",  # Subacute endocarditis with acquired immunodeficiency syndrome (disorder)
        "420945005",  # Histoplasmosis with acquired immunodeficiency syndrome (disorder)
        "421020000",  # Microsporidiosis associated with acquired immunodeficiency syndrome (disorder)
        "421023003",  # Presenile dementia with acquired immunodeficiency syndrome (disorder)
        "421047005",  # Candidiasis of lung with acquired immunodeficiency syndrome (disorder)
        "421077004",  # Disseminated candidiasis with acquired immunodeficiency syndrome (disorder)
        "421102007",  # Aplastic anemia with acquired immunodeficiency syndrome (disorder)
        "421230000",  # Hepatomegaly with acquired immunodeficiency syndrome (disorder)
        "421272004",  # Subacute myocarditis with acquired immunodeficiency syndrome (disorder)
        "421283008",  # Primary lymphoma of brain with acquired immunodeficiency syndrome (disorder)
        "421312009",  # Agranulocytosis with acquired immunodeficiency syndrome (disorder)
        "421315006",  # Myelitis with acquired immunodeficiency syndrome (disorder)
        "421394009",  # Skin disorder with acquired immunodeficiency syndrome (disorder)
        "421403008",  # Cryptococcosis with acquired immunodeficiency syndrome (disorder)
        "421415007",  # Subacute adenoviral encephalitis with acquired immunodeficiency syndrome (disorder)
        "421431004",  # Nocardiosis with acquired immunodeficiency syndrome (disorder)
        "421454008",  # Infectious gastroenteritis with acquired immunodeficiency syndrome (disorder)
        "421460008",  # Retinopathy with acquired immunodeficiency syndrome (disorder)
        "421508002",  # Viral pneumonia with acquired immunodeficiency syndrome (disorder)
        "421529006",  # Dementia with acquired immunodeficiency syndrome (disorder)
        "421571007",  # Tuberculosis with acquired immunodeficiency syndrome (disorder)
        "421597001",  # Polyneuropathy with acquired immunodeficiency syndrome (disorder)
        "421660003",  # Failure to thrive in infant with acquired immunodeficiency syndrome (disorder)
        "421671002",  # Pneumonia with acquired immunodeficiency syndrome (disorder)
        "421695000",  # Abnormal weight loss associated with acquired immunodeficiency syndrome (finding)
        "421706001",  # Blindness with acquired immunodeficiency syndrome (disorder)
        "421708000",  # Infective arthritis with acquired immunodeficiency syndrome (disorder)
        "421710003",  # Candidiasis of mouth with acquired immunodeficiency syndrome (disorder)
        "421766003",  # Thrombocytopenia with acquired immunodeficiency syndrome (disorder)
        "421827003",  # Encephalopathy with acquired immunodeficiency syndrome (disorder)
        "421851008",  # Acquired hemolytic anemia associated with acquired immunodeficiency syndrome (disorder)
        "421874007",  # Respiratory disorder with acquired immunodeficiency syndrome (disorder)
        "421883002",  # Strongyloidiasis with acquired immunodeficiency syndrome (disorder)
        "421929001",  # Myocarditis with acquired immunodeficiency syndrome (disorder)
        "421983003",  # Noninfectious gastroenteritis with acquired immunodeficiency syndrome (disorder)
        "421998001",  # Central nervous disorder with acquired immunodeficiency syndrome (disorder)
        "422003001",  # Cachexia associated with acquired immunodeficiency syndrome (finding)
        "422012004",  # Neuritis with acquired immunodeficiency syndrome (disorder)
        "422089004",  # Encephalomyelitis with acquired immunodeficiency syndrome (disorder)
        "422127002",  # Herpes zoster with acquired immunodeficiency syndrome (disorder)
        "422136003",  # Neuralgia with acquired immunodeficiency syndrome (disorder)
        "422177004",  # Dyspnea with acquired immunodeficiency syndrome (disorder)
        "422189002",  # Low vision with acquired immunodeficiency syndrome (disorder)
        "422194002",  # Hyperhidrosis with acquired immunodeficiency syndrome (disorder)
        "422282000",  # Malignant neoplasm with acquired immunodeficiency syndrome (disorder)
        "422337001",  # Coccidioidomycosis with acquired immunodeficiency syndrome (disorder)
        "432218001",  # History of asymptomatic human immunodeficiency virus infection (situation)
        "442537007",  # Non-Hodgkin lymphoma associated with Human immunodeficiency virus infection (disorder)
        "442662004",  # Copies of human immunodeficiency virus ribonucleic acid above reference range (finding)
        "445945000",  # Infectious disease with acquired immune deficiency syndrome (disorder)
        "48794007",  # Human immunodeficiency virus infection with infectious mononucleosis-like syndrome (disorder)
        "52079000",  # Congenital human immunodeficiency virus infection (disorder)
        "62479008",  # Acquired immune deficiency syndrome (disorder)
        "697904001",  # Pulmonary arterial hypertension associated with human immunodeficiency virus infection (disorder)
        "697965002",  # Cholangitis with acquired immunodeficiency syndrome (disorder)
        "699433000",  # Human immunodeficiency virus carrier (finding)
        "700053002",  # Sepsis with acquired immunodeficiency syndrome (disorder)
        "713260006",  # Subacute adenoviral encephalitis co-occurrent with human immunodeficiency virus infection (disorder)
        "713275003",  # Splenomegaly co-occurrent with human immunodeficiency virus infection (disorder)
        "713278001",  # Neuralgia co-occurrent with human immunodeficiency virus infection (disorder)
        "713297001",  # Candidiasis of esophagus co-occurrent with human immunodeficiency virus infection (disorder)
        "713298006",  # Heart disease co-occurrent with human immunodeficiency virus infection (disorder)
        "713299003",  # Disorder of eye proper co-occurrent with human immunodeficiency virus infection (disorder)
        "713300006",  # Disorder of gastrointestinal tract co-occurrent with human immunodeficiency virus infection (disorder)
        "713316008",  # Eruption of skin co-occurrent with human immunodeficiency virus infection (disorder)
        "713318009",  # Myocarditis co-occurrent with human immunodeficiency virus infection (disorder)
        "713320007",  # Radiculitis co-occurrent with human immunodeficiency virus infection (disorder)
        "713325002",  # Primary cerebral lymphoma co-occurrent with human immunodeficiency virus infection (disorder)
        "713339002",  # Infection caused by Strongyloides co-occurrent with human immunodeficiency virus infection (disorder)
        "713340000",  # Disorder of skin co-occurrent with human immunodeficiency virus infection (disorder)
        "713341001",  # Myelitis co-occurrent with human immunodeficiency virus infection (disorder)
        "713342008",  # Infection caused by Salmonella co-occurrent with human immunodeficiency virus infection (disorder)
        "713349004",  # Anemia co-occurrent with human immunodeficiency virus infection (disorder)
        "713444005",  # Hemophagocytic syndrome with human immunodeficiency virus infection (disorder)
        "713445006",  # Disseminated infection caused by Strongyloides co-occurrent with human immunodeficiency virus infection (disorder)
        "713446007",  # Chronic infection caused by herpes simplex virus co-occurrent with human immunodeficiency virus infection (disorder)
        "713483007",  # Reticulosarcoma co-occurrent with human immunodeficiency virus infection (disorder)
        "713484001",  # Disorder of respiratory system co-occurrent with human immunodeficiency virus infection (disorder)
        "713487008",  # Progressive multifocal leukoencephalopathy co-occurrent with human immunodeficiency virus infection (disorder)
        "713488003",  # Presenile dementia co-occurrent with human immunodeficiency virus infection (disorder)
        "713489006",  # Polyneuropathy co-occurrent with human immunodeficiency virus infection (disorder)
        "713490002",  # Infection caused by Pneumocystis co-occurrent with human immunodeficiency virus infection (disorder)
        "713491003",  # Organic brain syndrome co-occurrent with human immunodeficiency virus infection (disorder)
        "713497004",  # Candidiasis of mouth co-occurrent with human immunodeficiency virus infection (disorder)
        "713503007",  # Disorder of spinal cord co-occurrent with human immunodeficiency virus infection (disorder)
        "713504001",  # Disorder of kidney co-occurrent with human immunodeficiency virus infection (disorder)
        "713505000",  # Gastrointestinal malabsorption syndrome co-occurrent with human immunodeficiency virus infection (disorder)
        "713506004",  # Neuritis co-occurrent with human immunodeficiency virus infection (disorder)
        "713507008",  # Lymphadenopathy co-occurrent with human immunodeficiency virus infection (disorder)
        "713508003",  # Aplastic anemia co-occurrent with human immunodeficiency virus infection (disorder)
        "713510001",  # Enlargement of liver co-occurrent with human immunodeficiency virus infection (disorder)
        "713511002",  # Acute endocarditis co-occurrent with human immunodeficiency virus infection (disorder)
        "713523008",  # Cardiomyopathy co-occurrent with human immunodeficiency virus infection (disorder)
        "713526000",  # Recurrent bacterial pneumonia co-occurrent with human immunodeficiency virus infection (disorder)
        "713527009",  # Disorder of peripheral nervous system co-occurrent with human immunodeficiency virus infection (disorder)
        "713530002",  # Agranulocytosis co-occurrent with human immunodeficiency virus infection (disorder)
        "713531003",  # Visual impairment co-occurrent with human immunodeficiency virus infection (disorder)
        "713532005",  # Infective arthritis co-occurrent with human immunodeficiency virus infection (disorder)
        "713533000",  # Acquired hemolytic anemia co-occurrent with human immunodeficiency virus infection (disorder)
        "713543002",  # Demyelinating disease of central nervous system co-occurrent with human immunodeficiency virus infection (disorder)
        "713544008",  # Bacterial pneumonia co-occurrent with human immunodeficiency virus infection (disorder)
        "713545009",  # Infection caused by Nocardia co-occurrent with human immunodeficiency virus infection (disorder)
        "713546005",  # Isosporiasis co-occurrent with human immunodeficiency virus infection (disorder)
        "713570009",  # Infectious gastroenteritis co-occurrent with human immunodeficiency virus infection (disorder)
        "713571008",  # Disorder of central nervous system co-occurrent with human immunodeficiency virus infection (disorder)
        "713572001",  # Malignant neoplastic disease co-occurrent with human immunodeficiency virus infection (disorder)
        "713695001",  # Nephrotic syndrome co-occurrent with human immunodeficiency virus infection (disorder)
        "713696000",  # Renal failure syndrome co-occurrent with human immunodeficiency virus infection (disorder)
        "713718006",  # Diffuse non-Hodgkin immunoblastic lymphoma co-occurrent with human immunodeficiency virus infection (disorder)
        "713722001",  # Infection caused by Cytomegalovirus co-occurrent with human immunodeficiency virus infection (disorder)
        "713729005",  # Infection caused by Coccidia co-occurrent with human immunodeficiency virus infection (disorder)
        "713730000",  # Infection caused by herpes simplex virus co-occurrent with human immunodeficiency virus infection (disorder)
        "713731001",  # Pyrexia of unknown origin co-occurrent with human immunodeficiency virus infection (disorder)
        "713732008",  # Infection caused by Aspergillus co-occurrent with human immunodeficiency virus infection (disorder)
        "713733003",  # Infection caused by herpes zoster virus co-occurrent with human immunodeficiency virus infection (disorder)
        "713734009",  # Infection caused by Dermatophyte co-occurrent with human immunodeficiency virus infection (disorder)
        "713742005",  # Human immunodeficiency virus antibody detected (finding)
        "713844000",  # Dementia co-occurrent with human immunodeficiency virus infection (disorder)
        "713845004",  # Infection caused by Cryptosporidium co-occurrent with human immunodeficiency virus infection (disorder)
        "713880000",  # Opportunistic mycosis co-occurrent with human immunodeficiency virus infection (disorder)
        "713881001",  # Infection caused by Microsporidia co-occurrent with human immunodeficiency virus infection (disorder)
        "713887002",  # Focal segmental glomerulosclerosis co-occurrent with human immunodeficiency virus infection (disorder)
        "713897006",  # Burkitt lymphoma co-occurrent with human immunodeficiency virus infection (disorder)
        "713964006",  # Multidermatomal infection caused by Herpes zoster co-occurrent with human immunodeficiency virus infection (disorder)
        "713967004",  # Disseminated atypical infection caused by Mycobacterium co-occurrent with human immunodeficiency virus infection (disorder)
        "714083007",  # Recurrent salmonella sepsis co-occurrent with human immunodeficiency virus infection (disorder)
        "714464009",  # Immune reconstitution inflammatory syndrome caused by human immunodeficiency virus infection (disorder)
        "719522009",  # Infection of upper respiratory tract caused by Candida co-occurrent with human immunodeficiency virus infection (disorder)
        "721166000",  # Human immunodeficiency virus complicating pregnancy childbirth and the puerperium (disorder)
        "722557007",  # Parkinsonism due to human immunodeficiency virus infection (disorder)
        "733834006",  # Invasive carcinoma of uterine cervix co-occurrent with human immunodeficiency virus infection (disorder)
        "733835007",  # Extrapulmonary tuberculosis co-occurrent with human immunodeficiency virus infection (disorder)
        "735521001",  # Human immunodeficiency virus World Health Organization 2007 stage 1 co-occurrent with tuberculosis (disorder)
        "735522008",  # Human immunodeficiency virus World Health Organization 2007 stage 1 co-occurrent with malaria (disorder)
        "735523003",  # Human immunodeficiency virus World Health Organization 2007 stage 2 co-occurrent with tuberculosis (disorder)
        "735524009",  # Human immunodeficiency virus World Health Organization 2007 stage 2 co-occurrent with malaria (disorder)
        "735525005",  # Human immunodeficiency virus World Health Organization 2007 stage 3 co-occurrent with tuberculosis (disorder)
        "735526006",  # Human immunodeficiency virus World Health Organization 2007 stage 3 co-occurrent with malaria (disorder)
        "735527002",  # Human immunodeficiency virus World Health Organization 2007 stage 4 co-occurrent with tuberculosis (disorder)
        "735528007",  # Human immunodeficiency virus World Health Organization 2007 stage 4 co-occurrent with malaria (disorder)
        "771119002",  # Infection caused by Coccidia co-occurrent with acquired immunodeficiency syndrome (disorder)
        "771126002",  # Infection caused by Toxoplasma gondii co-occurrent with acquired immunodeficiency syndrome (disorder)
        "771127006",  # Infection caused by Isospora co-occurrent with acquired immunodeficiency syndrome (disorder)
        "79019005",  # Human immunodeficiency virus II infection (disorder)
        "80191000119101",  # Symptomatic human immunodeficiency virus I infection (disorder)
        "81000119104",  # Symptomatic human immunodeficiency virus infection (disorder)
        "838338001",  # Bacterial infection due to human immunodeficiency virus infection (disorder)
        "838377003",  # Chronic hepatitis C co-occurrent with human immunodeficiency virus infection (disorder)
        "840442003",  # Encephalitis caused by human immunodeficiency virus type 2 (disorder)
        "840498003",  # Encephalitis caused by human immunodeficiency virus type 1 (disorder)
        "860871003",  # Eosinophilic folliculitis due to human immunodeficiency virus infection (disorder)
        "860872005",  # Esophagitis due to human immunodeficiency virus disease (disorder)
        "86406008",  # Human immunodeficiency virus infection (disorder)
        "866044006",  # Mycosis due to human immunodeficiency virus infection (disorder)
        "870271009",  # Pruritus due to human immunodeficiency virus infection (disorder)
        "870328002",  # Papular pruritic eruption of skin due to human immunodeficiency virus infection (disorder)
        "870344006",  # Seborrheic dermatitis due to human immunodeficiency virus infection (disorder)
        "87117006",  # Human immunodeficiency virus infection with acute lymphadenitis (disorder)
        "90681000119107",  # Asymptomatic human immunodeficiency virus A1 infection (disorder)
        "90691000119105",  # Asymptomatic human immunodeficiency virus A2 infection (disorder)
        "91947003",  # Asymptomatic human immunodeficiency virus infection (disorder)
        "91948008",  # Asymptomatic human immunodeficiency virus infection in pregnancy (disorder)
    }

class AnaphylacticReactionToDtapVaccine(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of an anaphylactic reaction to DTaP (diphtheria, tetanus, and acellular pertussis) vaccine.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of an anaphylactic reaction to DTaP vaccine.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Anaphylactic Reaction to DTaP Vaccine"
    OID = "2.16.840.1.113883.3.464.1003.199.12.1031"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    SNOMEDCT = {
        "428281000124107",  # Anaphylaxis caused by vaccine product containing only Clostridium tetani and Corynebacterium diphtheriae antigens (disorder)
        "428291000124105",  # Anaphylaxis caused by component of vaccine product containing low dose acellular Bordetella pertussis and Clostridium tetani and low dose Corynebacterium diphtheriae antigens (disorder)
    }

class DisordersOfTheImmuneSystem(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of conditions affecting the immune system.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of disorders of the immune system including disorders of the immunoglobulins and the complement system.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Disorders of the Immune System"
    OID = "2.16.840.1.113883.3.464.1003.120.12.1001"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "D800",  # Hereditary hypogammaglobulinemia
        "D801",  # Nonfamilial hypogammaglobulinemia
        "D802",  # Selective deficiency of immunoglobulin A [IgA]
        "D803",  # Selective deficiency of immunoglobulin G [IgG] subclasses
        "D804",  # Selective deficiency of immunoglobulin M [IgM]
        "D805",  # Immunodeficiency with increased immunoglobulin M [IgM]
        "D806",  # Antibody deficiency with near-normal immunoglobulins or with hyperimmunoglobulinemia
        "D807",  # Transient hypogammaglobulinemia of infancy
        "D808",  # Other immunodeficiencies with predominantly antibody defects
        "D809",  # Immunodeficiency with predominantly antibody defects, unspecified
        "D810",  # Severe combined immunodeficiency [SCID] with reticular dysgenesis
        "D811",  # Severe combined immunodeficiency [SCID] with low T- and B-cell numbers
        "D812",  # Severe combined immunodeficiency [SCID] with low or normal B-cell numbers
        "D814",  # Nezelof's syndrome
        "D816",  # Major histocompatibility complex class I deficiency
        "D817",  # Major histocompatibility complex class II deficiency
        "D8182",  # Activated Phosphoinositide 3-kinase Delta Syndrome [APDS]
        "D8189",  # Other combined immunodeficiencies
        "D819",  # Combined immunodeficiency, unspecified
        "D820",  # Wiskott-Aldrich syndrome
        "D821",  # Di George's syndrome
        "D822",  # Immunodeficiency with short-limbed stature
        "D823",  # Immunodeficiency following hereditary defective response to Epstein-Barr virus
        "D824",  # Hyperimmunoglobulin E [IgE] syndrome
        "D828",  # Immunodeficiency associated with other specified major defects
        "D829",  # Immunodeficiency associated with major defect, unspecified
        "D830",  # Common variable immunodeficiency with predominant abnormalities of B-cell numbers and function
        "D831",  # Common variable immunodeficiency with predominant immunoregulatory T-cell disorders
        "D832",  # Common variable immunodeficiency with autoantibodies to B- or T-cells
        "D838",  # Other common variable immunodeficiencies
        "D839",  # Common variable immunodeficiency, unspecified
        "D840",  # Lymphocyte function antigen-1 [LFA-1] defect
        "D841",  # Defects in the complement system
        "D8481",  # Immunodeficiency due to conditions classified elsewhere
        "D84821",  # Immunodeficiency due to drugs
        "D84822",  # Immunodeficiency due to external causes
        "D8489",  # Other immunodeficiencies
        "D849",  # Immunodeficiency, unspecified
        "D893",  # Immune reconstitution syndrome
        "D89810",  # Acute graft-versus-host disease
        "D89811",  # Chronic graft-versus-host disease
        "D89812",  # Acute on chronic graft-versus-host disease
        "D89813",  # Graft-versus-host disease, unspecified
        "D8982",  # Autoimmune lymphoproliferative syndrome [ALPS]
        "D89831",  # Cytokine release syndrome, grade 1
        "D89832",  # Cytokine release syndrome, grade 2
        "D89833",  # Cytokine release syndrome, grade 3
        "D89834",  # Cytokine release syndrome, grade 4
        "D89835",  # Cytokine release syndrome, grade 5
        "D89839",  # Cytokine release syndrome, grade unspecified
        "D8984",  # IgG4-related disease
        "D8989",  # Other specified disorders involving the immune mechanism, not elsewhere classified
        "D899",  # Disorder involving the immune mechanism, unspecified
    }

    SNOMEDCT = {
        "1144907001",  # Late acute graft-versus-host disease (disorder)
        "1144908006",  # Late chronic graft-versus-host disease (disorder)
        "1144963000",  # Acute on chronic graft-versus-host disease (disorder)
        "1162263002",  # Complement component 8 deficiency (disorder)
        "1177120001",  # Bronchiolitis obliterans syndrome due to and following allogeneic stem cell transplant (disorder)
        "1197361002",  # Autoimmune lymphoproliferative syndrome due to cytotoxic T-lymphocyte associated protein 4 haploinsufficiency (disorder)
        "1197362009",  # Signal transducer and activator of transcription 3-related early-onset multisystem autoimmune disease (disorder)
        "121121000119106",  # Acute graft-versus-host reaction following bone marrow transplant (disorder)
        "121131000119109",  # Chronic graft versus host disease after transplantation of bone marrow (disorder)
        "1268379007",  # Graft versus host disease of intestine (disorder)
        "1279887007",  # Complement hyperactivation, angiopathic thrombosis, protein losing enteropathy syndrome (disorder)
        "128631000119109",  # Chronic graft-versus-host disease following kidney transplant (disorder)
        "18827005",  # Complement abnormality (disorder)
        "190979003",  # Selective immunoglobulin A deficiency (disorder)
        "190980000",  # Selective immunoglobulin M deficiency (disorder)
        "190981001",  # Selective immunoglobulin G deficiency (disorder)
        "190996002",  # Severe combined immunodeficiency with reticular dysgenesis (disorder)
        "190997006",  # Severe combined immunodeficiency with low T- and B-cell numbers (disorder)
        "190998001",  # Severe combined immunodeficiency with low or normal B-cell numbers (disorder)
        "191001007",  # Major histocompatibility complex class I deficiency (disorder)
        "191002000",  # Major histocompatibility complex class II deficiency (disorder)
        "191011000",  # Common variable immunodeficiency with predominant abnormalities of B-cell numbers and functions (disorder)
        "191012007",  # Common variable immunodeficiency with predominant immunoregulatory T-cell disorders (disorder)
        "191013002",  # Common variable immunodeficiency with autoantibodies to B- or T-cells (disorder)
        "191018006",  # Lymphocyte function antigen-1 defect (disorder)
        "23238000",  # Common variable agammaglobulinemia (disorder)
        "234532001",  # Immunodeficiency disorder (disorder)
        "234562007",  # Anti-protein antibody deficiency (disorder)
        "234593008",  # Classical complement pathway abnormality (disorder)
        "234594002",  # Complement 1q deficiency (disorder)
        "234595001",  # Complement 1q beta chain deficiency (disorder)
        "234596000",  # Complement 1q dysfunction (disorder)
        "234597009",  # Complement 1r deficiency (disorder)
        "234598004",  # Complement 1s deficiency (disorder)
        "234599007",  # Complement 2 deficiency (disorder)
        "234600005",  # Complement 4 deficiency (disorder)
        "234601009",  # Complement 4A deficiency (disorder)
        "234602002",  # Complement 4B deficiency (disorder)
        "234604001",  # Alternative pathway deficiency (disorder)
        "234605000",  # Factor B deficiency (disorder)
        "234607008",  # Factor D deficiency (disorder)
        "234608003",  # Terminal component deficiency (disorder)
        "234609006",  # Complement 5 deficiency (disorder)
        "234611002",  # Complement 6 deficiency (disorder)
        "234612009",  # Complement 7 deficiency (disorder)
        "234613004",  # Combined complement 6 and 7 deficiencies (disorder)
        "234614005",  # Complement 8 beta chain deficiency (disorder)
        "234615006",  # Complement 8 beta chain dysfunction (disorder)
        "234616007",  # Complement 8 alpha-gamma deficiency (disorder)
        "234617003",  # Complement 9 deficiency (disorder)
        "234618008",  # Complement regulatory factor defect (disorder)
        "234619000",  # Hereditary C1 esterase inhibitor deficiency - deficient factor (disorder)
        "234620006",  # Hereditary C1 esterase inhibitor deficiency - dysfunctional factor (disorder)
        "234621005",  # Factor I deficiency (disorder)
        "234622003",  # Factor H deficiency (disorder)
        "234623008",  # Complement 4 binding protein deficiency (disorder)
        "234624002",  # Decay accelerating factor deficiency (disorder)
        "234625001",  # Homologous restriction factor deficiency (disorder)
        "234626000",  # Complement 5a inhibitor deficiency (disorder)
        "234627009",  # Anaphylotoxin inactivator deficiency (disorder)
        "234628004",  # Complement receptor deficiency (disorder)
        "234629007",  # Complement receptor 1 deficiency (disorder)
        "234630002",  # Complement receptor 3 deficiency (disorder)
        "234631003",  # Immunodeficiency with major anomalies (disorder)
        "234646005",  # Graft-versus-host disease (disorder)
        "24419001",  # Disorder of complement (disorder)
        "24743004",  # Complement deficiency disease (disorder)
        "263661007",  # Complement 5 dysfunction (disorder)
        "36070007",  # Wiskott-Aldrich syndrome (disorder)
        "39674000",  # Familial C3B inhibitor deficiency syndrome (disorder)
        "402355000",  # Acute graft-versus-host disease (disorder)
        "402356004",  # Chronic graft-versus-host disease (disorder)
        "402357008",  # Lichenoid chronic graft-versus-host disease (disorder)
        "402358003",  # Sclerodermoid chronic graft-versus-host disease (disorder)
        "402360001",  # Materno-fetal graft-versus-host disease (disorder)
        "403835002",  # X-linked hyper-immunoglobulin M syndrome (disorder)
        "403836001",  # Autosomal recessive hyperimmunoglobulin M syndrome (disorder)
        "403837005",  # Wiskott-Aldrich autosomal dominant variant syndrome (disorder)
        "414029004",  # Disorder of immune function (disorder)
        "426202004",  # Immune reconstitution syndrome (disorder)
        "427167008",  # Hereditary angioedema with normal C1 esterase inhibitor activity (disorder)
        "442459007",  # Combined immunodeficiency disease (disorder)
        "444547006",  # Graft versus host disease of skin (disorder)
        "449187006",  # Kappa light chain deficiency (disorder)
        "55602000",  # Nezelof's syndrome (disorder)
        "65880007",  # X-linked agammaglobulinemia (disorder)
        "702444009",  # Autoimmune lymphoproliferative syndrome (disorder)
        "719156006",  # X-linked intellectual disability with hypogammaglobulinemia and progressive neurological deterioration syndrome (disorder)
        "720853005",  # Severe combined immunodeficiency, microcephaly, growth retardation, sensitivity to ionizing radiation syndrome (disorder)
        "722067005",  # Severe combined immunodeficiency with hypereosinophilia (disorder)
        "724361001",  # Hepatic veno-occlusive disease with immunodeficiency syndrome (disorder)
        "762618008",  # Bronchiolitis obliterans syndrome due to and after lung transplantation (disorder)
        "763623001",  # Severe combined immunodeficiency due to cytidine 5-prime triphosphate synthetase 1 deficiency (disorder)
        "765188009",  # Severe combined immunodeficiency due to complete recombination-activating gene 1 and/or recombination-activating gene 2 deficiency (disorder)
        "771078002",  # Immunodeficiency due to mannan binding lectin serine peptidase 2 deficiency (disorder)
        "771443008",  # Complement component 3 deficiency (disorder)
        "778027003",  # Primary CD59 deficiency (disorder)
        "783007005",  # Recurrent Neisseria infection due to factor D deficiency (disorder)
        "783248004",  # Hyperimmunoglobulin M syndrome with susceptibility to opportunistic infection (disorder)
        "783249007",  # Hyperimmunoglobulin M syndrome without susceptibility to opportunistic infection (disorder)
        "783617001",  # Severe combined immunodeficiency due to lymphocyte-specific protein-tyrosine kinase deficiency (disorder)
        "783621008",  # Immunodeficiency with factor I anomaly (disorder)
        "789777007",  # Short-limb skeletal dysplasia with severe combined immunodeficiency (disorder)
        "81166004",  # Properdin deficiency disease (disorder)
        "82286005",  # Hyperimmunoglobulin M syndrome (disorder)
        "82966003",  # Hereditary angioedema (disorder)
        "846606007",  # Graft versus host disease of liver (disorder)
        "88714009",  # Transient hypogammaglobulinemia of infancy (disorder)
    }

class EncephalitisDueToDiphtheriaTetanusOrPertussisVaccine(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of an encephalitis due to DTaP, diphtheria, tetanus, or pertussis vaccine.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of encephalitis to diphtheria, tetanus or pertussis vaccine.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Encephalitis Due to Diphtheria, Tetanus or Pertussis Vaccine"
    OID = "2.16.840.1.113883.3.464.1003.1164"
    DEFINITION_VERSION = "20220208"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    SNOMEDCT = {
        "192710009",  # Post tetanus vaccination encephalitis (disorder)
        "192711008",  # Post diphtheria vaccination encephalitis (disorder)
        "192712001",  # Post pertussis vaccination encephalitis (disorder)
    }

class HepatitisA(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of Hepatitis A infection.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of Hepatitis A.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Hepatitis A"
    OID = "2.16.840.1.113883.3.464.1003.110.12.1024"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "B150",  # Hepatitis A with hepatic coma
        "B159",  # Hepatitis A without hepatic coma
    }

    SNOMEDCT = {
        "310875001",  # Hepatitis A - current infection (finding)
        "105801000119103",  # Immunity to hepatitis A determined by serology (finding)
        "111879004",  # Viral hepatitis A without hepatic coma (disorder)
        "16060001",  # Hepatic coma due to viral hepatitis A (disorder)
        "165997004",  # Hepatitis A detected (finding)
        "18917003",  # Acute fulminating type A viral hepatitis (disorder)
        "206373002",  # Congenital hepatitis A infection (disorder)
        "25102003",  # Acute type A viral hepatitis (disorder)
        "278971009",  # Hepatitis A immune (finding)
        "40468003",  # Viral hepatitis, type A (disorder)
        "424758008",  # Viral hepatitis A without hepatic coma, without hepatitis delta (disorder)
        "428030001",  # History of hepatitis A (situation)
        "43634002",  # Relapsing type A viral hepatitis (disorder)
        "79031007",  # Anicteric type A viral hepatitis (disorder)
    }

class HepatitisB(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of diagnoses for acute or chronic Hepatitis B infections.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of acute or chronic Hepatitis B.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Hepatitis B"
    OID = "2.16.840.1.113883.3.67.1.101.1.269"
    DEFINITION_VERSION = "20170825"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "B160",  # Acute hepatitis B with delta-agent with hepatic coma
        "B161",  # Acute hepatitis B with delta-agent without hepatic coma
        "B162",  # Acute hepatitis B without delta-agent with hepatic coma
        "B169",  # Acute hepatitis B without delta-agent and without hepatic coma
        "B180",  # Chronic viral hepatitis B with delta-agent
        "B181",  # Chronic viral hepatitis B without delta-agent
        "B1910",  # Unspecified viral hepatitis B without hepatic coma
        "B1911",  # Unspecified viral hepatitis B with hepatic coma
    }

    SNOMEDCT = {
        "1116000",  # Chronic aggressive type B viral hepatitis (disorder)
        "111891008",  # Viral hepatitis B without hepatic coma (disorder)
        "13265006",  # Acute fulminating type B viral hepatitis (disorder)
        "153091000119109",  # Hepatic coma due to chronic hepatitis B with delta agent (disorder)
        "186624004",  # Hepatic coma due to acute hepatitis B with delta agent (disorder)
        "186626002",  # Acute hepatitis B with delta-agent (coinfection) without hepatic coma (disorder)
        "186639003",  # Chronic viral hepatitis B without delta-agent (disorder)
        "235864009",  # Acute hepatitis B with hepatitis D (disorder)
        "235865005",  # Hepatitis D superinfection of hepatitis B carrier (disorder)
        "235869004",  # Chronic viral hepatitis B with hepatitis D (disorder)
        "235871004",  # Hepatitis B carrier (finding)
        "26206000",  # Hepatic coma due to viral hepatitis B (disorder)
        "38662009",  # Chronic persistent type B viral hepatitis (disorder)
        "424099008",  # Hepatic coma due to acute hepatitis B (disorder)
        "424340000",  # Hepatic coma due to chronic hepatitis B (disorder)
        "442134007",  # Hepatitis B associated with Human immunodeficiency virus infection (disorder)
        "442374005",  # Hepatitis B and hepatitis C (disorder)
        "446698005",  # Reactivation of hepatitis B viral hepatitis (disorder)
        "50167007",  # Chronic active type B viral hepatitis (disorder)
        "53425008",  # Anicteric type B viral hepatitis (disorder)
        "60498001",  # Congenital viral hepatitis B infection (disorder)
        "61977001",  # Chronic type B viral hepatitis (disorder)
        "66071002",  # Viral hepatitis type B (disorder)
        "76795007",  # Acute type B viral hepatitis (disorder)
        "838380002",  # Chronic hepatitis B co-occurrent with hepatitis C and hepatitis D (disorder)
    }

class Intussusception(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of intussusception.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of intussusception.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Intussusception"
    OID = "2.16.840.1.113883.3.464.1003.199.12.1056"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "K561",  # Intussusception
    }

    SNOMEDCT = {
        "111361008",  # Intussusception of cecum (disorder)
        "11578004",  # Intussusception of rectum (disorder)
        "17186003",  # Ileocolic intussusception (disorder)
        "197046003",  # Gastroduodenal intussusception (disorder)
        "197047007",  # Jejunogastric intussusception (disorder)
        "197049005",  # Ileocecal intussusception (disorder)
        "197050005",  # Chronic intussusception (disorder)
        "197051009",  # Compound intussusception (disorder)
        "197052002",  # Multiple intussusception (disorder)
        "197053007",  # Retrograde intussusception (disorder)
        "197054001",  # Secondary intussusception (disorder)
        "24610009",  # Sigmoidorectal intussusception (disorder)
        "26316005",  # Internal complete rectal prolapse with intussusception of rectosigmoid (disorder)
        "27673007",  # Intussusception of colon (disorder)
        "313326008",  # Internal intussusception of rectum (disorder)
        "313327004",  # Inflammatory cloacogenic polyp (disorder)
        "38397000",  # Gastroesophageal intussusception (disorder)
        "405558008",  # Intussusception of ileum (disorder)
        "49723003",  # Intussusception of intestine (disorder)
        "64976006",  # Cecocolic intussusception (disorder)
        "721651002",  # Intussusception of small intestine (disorder)
        "721676001",  # Intussusception of large intestine (disorder)
        "91446004",  # Intussusception of appendix (disorder)
    }

class MalignantNeoplasmOfLymphaticAndHematopoieticTissue(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of lymphoma and other cancers of the lymph and hematologic systems.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of lymphoma, leukemia and other malignant cancers of the lymphatic and hematologic systems.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Malignant Neoplasm of Lymphatic and Hematopoietic Tissue"
    OID = "2.16.840.1.113883.3.464.1003.108.12.1009"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "C8100",  # Nodular lymphocyte predominant Hodgkin lymphoma, unspecified site
        "C8101",  # Nodular lymphocyte predominant Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8102",  # Nodular lymphocyte predominant Hodgkin lymphoma, intrathoracic lymph nodes
        "C8103",  # Nodular lymphocyte predominant Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8104",  # Nodular lymphocyte predominant Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8105",  # Nodular lymphocyte predominant Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8106",  # Nodular lymphocyte predominant Hodgkin lymphoma, intrapelvic lymph nodes
        "C8107",  # Nodular lymphocyte predominant Hodgkin lymphoma, spleen
        "C8108",  # Nodular lymphocyte predominant Hodgkin lymphoma, lymph nodes of multiple sites
        "C8109",  # Nodular lymphocyte predominant Hodgkin lymphoma, extranodal and solid organ sites
        "C8110",  # Nodular sclerosis Hodgkin lymphoma, unspecified site
        "C8111",  # Nodular sclerosis Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8112",  # Nodular sclerosis Hodgkin lymphoma, intrathoracic lymph nodes
        "C8113",  # Nodular sclerosis Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8114",  # Nodular sclerosis Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8115",  # Nodular sclerosis Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8116",  # Nodular sclerosis Hodgkin lymphoma, intrapelvic lymph nodes
        "C8117",  # Nodular sclerosis Hodgkin lymphoma, spleen
        "C8118",  # Nodular sclerosis Hodgkin lymphoma, lymph nodes of multiple sites
        "C8119",  # Nodular sclerosis Hodgkin lymphoma, extranodal and solid organ sites
        "C8120",  # Mixed cellularity Hodgkin lymphoma, unspecified site
        "C8121",  # Mixed cellularity Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8122",  # Mixed cellularity Hodgkin lymphoma, intrathoracic lymph nodes
        "C8123",  # Mixed cellularity Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8124",  # Mixed cellularity Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8125",  # Mixed cellularity Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8126",  # Mixed cellularity Hodgkin lymphoma, intrapelvic lymph nodes
        "C8127",  # Mixed cellularity Hodgkin lymphoma, spleen
        "C8128",  # Mixed cellularity Hodgkin lymphoma, lymph nodes of multiple sites
        "C8129",  # Mixed cellularity Hodgkin lymphoma, extranodal and solid organ sites
        "C8130",  # Lymphocyte depleted Hodgkin lymphoma, unspecified site
        "C8131",  # Lymphocyte depleted Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8132",  # Lymphocyte depleted Hodgkin lymphoma, intrathoracic lymph nodes
        "C8133",  # Lymphocyte depleted Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8134",  # Lymphocyte depleted Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8135",  # Lymphocyte depleted Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8136",  # Lymphocyte depleted Hodgkin lymphoma, intrapelvic lymph nodes
        "C8137",  # Lymphocyte depleted Hodgkin lymphoma, spleen
        "C8138",  # Lymphocyte depleted Hodgkin lymphoma, lymph nodes of multiple sites
        "C8139",  # Lymphocyte depleted Hodgkin lymphoma, extranodal and solid organ sites
        "C8140",  # Lymphocyte-rich Hodgkin lymphoma, unspecified site
        "C8141",  # Lymphocyte-rich Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8142",  # Lymphocyte-rich Hodgkin lymphoma, intrathoracic lymph nodes
        "C8143",  # Lymphocyte-rich Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8144",  # Lymphocyte-rich Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8145",  # Lymphocyte-rich Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8146",  # Lymphocyte-rich Hodgkin lymphoma, intrapelvic lymph nodes
        "C8147",  # Lymphocyte-rich Hodgkin lymphoma, spleen
        "C8148",  # Lymphocyte-rich Hodgkin lymphoma, lymph nodes of multiple sites
        "C8149",  # Lymphocyte-rich Hodgkin lymphoma, extranodal and solid organ sites
        "C8170",  # Other Hodgkin lymphoma, unspecified site
        "C8171",  # Other Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8172",  # Other Hodgkin lymphoma, intrathoracic lymph nodes
        "C8173",  # Other Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8174",  # Other Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8175",  # Other Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8176",  # Other Hodgkin lymphoma, intrapelvic lymph nodes
        "C8177",  # Other Hodgkin lymphoma, spleen
        "C8178",  # Other Hodgkin lymphoma, lymph nodes of multiple sites
        "C8179",  # Other Hodgkin lymphoma, extranodal and solid organ sites
        "C8190",  # Hodgkin lymphoma, unspecified, unspecified site
        "C8191",  # Hodgkin lymphoma, unspecified, lymph nodes of head, face, and neck
        "C8192",  # Hodgkin lymphoma, unspecified, intrathoracic lymph nodes
        "C8193",  # Hodgkin lymphoma, unspecified, intra-abdominal lymph nodes
        "C8194",  # Hodgkin lymphoma, unspecified, lymph nodes of axilla and upper limb
        "C8195",  # Hodgkin lymphoma, unspecified, lymph nodes of inguinal region and lower limb
        "C8196",  # Hodgkin lymphoma, unspecified, intrapelvic lymph nodes
        "C8197",  # Hodgkin lymphoma, unspecified, spleen
        "C8198",  # Hodgkin lymphoma, unspecified, lymph nodes of multiple sites
        "C8199",  # Hodgkin lymphoma, unspecified, extranodal and solid organ sites
        "C8200",  # Follicular lymphoma grade I, unspecified site
        "C8201",  # Follicular lymphoma grade I, lymph nodes of head, face, and neck
        "C8202",  # Follicular lymphoma grade I, intrathoracic lymph nodes
        "C8203",  # Follicular lymphoma grade I, intra-abdominal lymph nodes
        "C8204",  # Follicular lymphoma grade I, lymph nodes of axilla and upper limb
        "C8205",  # Follicular lymphoma grade I, lymph nodes of inguinal region and lower limb
        "C8206",  # Follicular lymphoma grade I, intrapelvic lymph nodes
        "C8207",  # Follicular lymphoma grade I, spleen
        "C8208",  # Follicular lymphoma grade I, lymph nodes of multiple sites
        "C8209",  # Follicular lymphoma grade I, extranodal and solid organ sites
        "C8210",  # Follicular lymphoma grade II, unspecified site
        "C8211",  # Follicular lymphoma grade II, lymph nodes of head, face, and neck
        "C8212",  # Follicular lymphoma grade II, intrathoracic lymph nodes
        "C8213",  # Follicular lymphoma grade II, intra-abdominal lymph nodes
        "C8214",  # Follicular lymphoma grade II, lymph nodes of axilla and upper limb
        "C8215",  # Follicular lymphoma grade II, lymph nodes of inguinal region and lower limb
        "C8216",  # Follicular lymphoma grade II, intrapelvic lymph nodes
        "C8217",  # Follicular lymphoma grade II, spleen
        "C8218",  # Follicular lymphoma grade II, lymph nodes of multiple sites
        "C8219",  # Follicular lymphoma grade II, extranodal and solid organ sites
        "C8220",  # Follicular lymphoma grade III, unspecified, unspecified site
        "C8221",  # Follicular lymphoma grade III, unspecified, lymph nodes of head, face, and neck
        "C8222",  # Follicular lymphoma grade III, unspecified, intrathoracic lymph nodes
        "C8223",  # Follicular lymphoma grade III, unspecified, intra-abdominal lymph nodes
        "C8224",  # Follicular lymphoma grade III, unspecified, lymph nodes of axilla and upper limb
        "C8225",  # Follicular lymphoma grade III, unspecified, lymph nodes of inguinal region and lower limb
        "C8226",  # Follicular lymphoma grade III, unspecified, intrapelvic lymph nodes
        "C8227",  # Follicular lymphoma grade III, unspecified, spleen
        "C8228",  # Follicular lymphoma grade III, unspecified, lymph nodes of multiple sites
        "C8229",  # Follicular lymphoma grade III, unspecified, extranodal and solid organ sites
        "C8230",  # Follicular lymphoma grade IIIa, unspecified site
        "C8231",  # Follicular lymphoma grade IIIa, lymph nodes of head, face, and neck
        "C8232",  # Follicular lymphoma grade IIIa, intrathoracic lymph nodes
        "C8233",  # Follicular lymphoma grade IIIa, intra-abdominal lymph nodes
        "C8234",  # Follicular lymphoma grade IIIa, lymph nodes of axilla and upper limb
        "C8235",  # Follicular lymphoma grade IIIa, lymph nodes of inguinal region and lower limb
        "C8236",  # Follicular lymphoma grade IIIa, intrapelvic lymph nodes
        "C8237",  # Follicular lymphoma grade IIIa, spleen
        "C8238",  # Follicular lymphoma grade IIIa, lymph nodes of multiple sites
        "C8239",  # Follicular lymphoma grade IIIa, extranodal and solid organ sites
        "C8240",  # Follicular lymphoma grade IIIb, unspecified site
        "C8241",  # Follicular lymphoma grade IIIb, lymph nodes of head, face, and neck
        "C8242",  # Follicular lymphoma grade IIIb, intrathoracic lymph nodes
        "C8243",  # Follicular lymphoma grade IIIb, intra-abdominal lymph nodes
        "C8244",  # Follicular lymphoma grade IIIb, lymph nodes of axilla and upper limb
        "C8245",  # Follicular lymphoma grade IIIb, lymph nodes of inguinal region and lower limb
        "C8246",  # Follicular lymphoma grade IIIb, intrapelvic lymph nodes
        "C8247",  # Follicular lymphoma grade IIIb, spleen
        "C8248",  # Follicular lymphoma grade IIIb, lymph nodes of multiple sites
        "C8249",  # Follicular lymphoma grade IIIb, extranodal and solid organ sites
        "C8250",  # Diffuse follicle center lymphoma, unspecified site
        "C8251",  # Diffuse follicle center lymphoma, lymph nodes of head, face, and neck
        "C8252",  # Diffuse follicle center lymphoma, intrathoracic lymph nodes
        "C8253",  # Diffuse follicle center lymphoma, intra-abdominal lymph nodes
        "C8254",  # Diffuse follicle center lymphoma, lymph nodes of axilla and upper limb
        "C8255",  # Diffuse follicle center lymphoma, lymph nodes of inguinal region and lower limb
        "C8256",  # Diffuse follicle center lymphoma, intrapelvic lymph nodes
        "C8257",  # Diffuse follicle center lymphoma, spleen
        "C8258",  # Diffuse follicle center lymphoma, lymph nodes of multiple sites
        "C8259",  # Diffuse follicle center lymphoma, extranodal and solid organ sites
        "C8260",  # Cutaneous follicle center lymphoma, unspecified site
        "C8261",  # Cutaneous follicle center lymphoma, lymph nodes of head, face, and neck
        "C8262",  # Cutaneous follicle center lymphoma, intrathoracic lymph nodes
        "C8263",  # Cutaneous follicle center lymphoma, intra-abdominal lymph nodes
        "C8264",  # Cutaneous follicle center lymphoma, lymph nodes of axilla and upper limb
        "C8265",  # Cutaneous follicle center lymphoma, lymph nodes of inguinal region and lower limb
        "C8266",  # Cutaneous follicle center lymphoma, intrapelvic lymph nodes
        "C8267",  # Cutaneous follicle center lymphoma, spleen
        "C8268",  # Cutaneous follicle center lymphoma, lymph nodes of multiple sites
        "C8269",  # Cutaneous follicle center lymphoma, extranodal and solid organ sites
        "C8280",  # Other types of follicular lymphoma, unspecified site
        "C8281",  # Other types of follicular lymphoma, lymph nodes of head, face, and neck
        "C8282",  # Other types of follicular lymphoma, intrathoracic lymph nodes
        "C8283",  # Other types of follicular lymphoma, intra-abdominal lymph nodes
        "C8284",  # Other types of follicular lymphoma, lymph nodes of axilla and upper limb
        "C8285",  # Other types of follicular lymphoma, lymph nodes of inguinal region and lower limb
        "C8286",  # Other types of follicular lymphoma, intrapelvic lymph nodes
        "C8287",  # Other types of follicular lymphoma, spleen
        "C8288",  # Other types of follicular lymphoma, lymph nodes of multiple sites
        "C8289",  # Other types of follicular lymphoma, extranodal and solid organ sites
        "C8290",  # Follicular lymphoma, unspecified, unspecified site
        "C8291",  # Follicular lymphoma, unspecified, lymph nodes of head, face, and neck
        "C8292",  # Follicular lymphoma, unspecified, intrathoracic lymph nodes
        "C8293",  # Follicular lymphoma, unspecified, intra-abdominal lymph nodes
        "C8294",  # Follicular lymphoma, unspecified, lymph nodes of axilla and upper limb
        "C8295",  # Follicular lymphoma, unspecified, lymph nodes of inguinal region and lower limb
        "C8296",  # Follicular lymphoma, unspecified, intrapelvic lymph nodes
        "C8297",  # Follicular lymphoma, unspecified, spleen
        "C8298",  # Follicular lymphoma, unspecified, lymph nodes of multiple sites
        "C8299",  # Follicular lymphoma, unspecified, extranodal and solid organ sites
        "C8300",  # Small cell B-cell lymphoma, unspecified site
        "C8301",  # Small cell B-cell lymphoma, lymph nodes of head, face, and neck
        "C8302",  # Small cell B-cell lymphoma, intrathoracic lymph nodes
        "C8303",  # Small cell B-cell lymphoma, intra-abdominal lymph nodes
        "C8304",  # Small cell B-cell lymphoma, lymph nodes of axilla and upper limb
        "C8305",  # Small cell B-cell lymphoma, lymph nodes of inguinal region and lower limb
        "C8306",  # Small cell B-cell lymphoma, intrapelvic lymph nodes
        "C8307",  # Small cell B-cell lymphoma, spleen
        "C8308",  # Small cell B-cell lymphoma, lymph nodes of multiple sites
        "C8309",  # Small cell B-cell lymphoma, extranodal and solid organ sites
        "C8310",  # Mantle cell lymphoma, unspecified site
        "C8311",  # Mantle cell lymphoma, lymph nodes of head, face, and neck
        "C8312",  # Mantle cell lymphoma, intrathoracic lymph nodes
        "C8313",  # Mantle cell lymphoma, intra-abdominal lymph nodes
        "C8314",  # Mantle cell lymphoma, lymph nodes of axilla and upper limb
        "C8315",  # Mantle cell lymphoma, lymph nodes of inguinal region and lower limb
        "C8316",  # Mantle cell lymphoma, intrapelvic lymph nodes
        "C8317",  # Mantle cell lymphoma, spleen
        "C8318",  # Mantle cell lymphoma, lymph nodes of multiple sites
        "C8319",  # Mantle cell lymphoma, extranodal and solid organ sites
        "C8330",  # Diffuse large B-cell lymphoma, unspecified site
        "C8331",  # Diffuse large B-cell lymphoma, lymph nodes of head, face, and neck
        "C8332",  # Diffuse large B-cell lymphoma, intrathoracic lymph nodes
        "C8333",  # Diffuse large B-cell lymphoma, intra-abdominal lymph nodes
        "C8334",  # Diffuse large B-cell lymphoma, lymph nodes of axilla and upper limb
        "C8335",  # Diffuse large B-cell lymphoma, lymph nodes of inguinal region and lower limb
        "C8336",  # Diffuse large B-cell lymphoma, intrapelvic lymph nodes
        "C8337",  # Diffuse large B-cell lymphoma, spleen
        "C8338",  # Diffuse large B-cell lymphoma, lymph nodes of multiple sites
        "C8339",  # Diffuse large B-cell lymphoma, extranodal and solid organ sites
        "C8350",  # Lymphoblastic (diffuse) lymphoma, unspecified site
        "C8351",  # Lymphoblastic (diffuse) lymphoma, lymph nodes of head, face, and neck
        "C8352",  # Lymphoblastic (diffuse) lymphoma, intrathoracic lymph nodes
        "C8353",  # Lymphoblastic (diffuse) lymphoma, intra-abdominal lymph nodes
        "C8354",  # Lymphoblastic (diffuse) lymphoma, lymph nodes of axilla and upper limb
        "C8355",  # Lymphoblastic (diffuse) lymphoma, lymph nodes of inguinal region and lower limb
        "C8356",  # Lymphoblastic (diffuse) lymphoma, intrapelvic lymph nodes
        "C8357",  # Lymphoblastic (diffuse) lymphoma, spleen
        "C8358",  # Lymphoblastic (diffuse) lymphoma, lymph nodes of multiple sites
        "C8359",  # Lymphoblastic (diffuse) lymphoma, extranodal and solid organ sites
        "C8370",  # Burkitt lymphoma, unspecified site
        "C8371",  # Burkitt lymphoma, lymph nodes of head, face, and neck
        "C8372",  # Burkitt lymphoma, intrathoracic lymph nodes
        "C8373",  # Burkitt lymphoma, intra-abdominal lymph nodes
        "C8374",  # Burkitt lymphoma, lymph nodes of axilla and upper limb
        "C8375",  # Burkitt lymphoma, lymph nodes of inguinal region and lower limb
        "C8376",  # Burkitt lymphoma, intrapelvic lymph nodes
        "C8377",  # Burkitt lymphoma, spleen
        "C8378",  # Burkitt lymphoma, lymph nodes of multiple sites
        "C8379",  # Burkitt lymphoma, extranodal and solid organ sites
        "C8380",  # Other non-follicular lymphoma, unspecified site
        "C8381",  # Other non-follicular lymphoma, lymph nodes of head, face, and neck
        "C8382",  # Other non-follicular lymphoma, intrathoracic lymph nodes
        "C8383",  # Other non-follicular lymphoma, intra-abdominal lymph nodes
        "C8384",  # Other non-follicular lymphoma, lymph nodes of axilla and upper limb
        "C8385",  # Other non-follicular lymphoma, lymph nodes of inguinal region and lower limb
        "C8386",  # Other non-follicular lymphoma, intrapelvic lymph nodes
        "C8387",  # Other non-follicular lymphoma, spleen
        "C8388",  # Other non-follicular lymphoma, lymph nodes of multiple sites
        "C8389",  # Other non-follicular lymphoma, extranodal and solid organ sites
        "C8390",  # Non-follicular (diffuse) lymphoma, unspecified, unspecified site
        "C8391",  # Non-follicular (diffuse) lymphoma, unspecified, lymph nodes of head, face, and neck
        "C8392",  # Non-follicular (diffuse) lymphoma, unspecified, intrathoracic lymph nodes
        "C8393",  # Non-follicular (diffuse) lymphoma, unspecified, intra-abdominal lymph nodes
        "C8394",  # Non-follicular (diffuse) lymphoma, unspecified, lymph nodes of axilla and upper limb
        "C8395",  # Non-follicular (diffuse) lymphoma, unspecified, lymph nodes of inguinal region and lower limb
        "C8396",  # Non-follicular (diffuse) lymphoma, unspecified, intrapelvic lymph nodes
        "C8397",  # Non-follicular (diffuse) lymphoma, unspecified, spleen
        "C8398",  # Non-follicular (diffuse) lymphoma, unspecified, lymph nodes of multiple sites
        "C8399",  # Non-follicular (diffuse) lymphoma, unspecified, extranodal and solid organ sites
        "C8400",  # Mycosis fungoides, unspecified site
        "C8401",  # Mycosis fungoides, lymph nodes of head, face, and neck
        "C8402",  # Mycosis fungoides, intrathoracic lymph nodes
        "C8403",  # Mycosis fungoides, intra-abdominal lymph nodes
        "C8404",  # Mycosis fungoides, lymph nodes of axilla and upper limb
        "C8405",  # Mycosis fungoides, lymph nodes of inguinal region and lower limb
        "C8406",  # Mycosis fungoides, intrapelvic lymph nodes
        "C8407",  # Mycosis fungoides, spleen
        "C8408",  # Mycosis fungoides, lymph nodes of multiple sites
        "C8409",  # Mycosis fungoides, extranodal and solid organ sites
        "C8410",  # Sezary disease, unspecified site
        "C8411",  # Sezary disease, lymph nodes of head, face, and neck
        "C8412",  # Sezary disease, intrathoracic lymph nodes
        "C8413",  # Sezary disease, intra-abdominal lymph nodes
        "C8414",  # Sezary disease, lymph nodes of axilla and upper limb
        "C8415",  # Sezary disease, lymph nodes of inguinal region and lower limb
        "C8416",  # Sezary disease, intrapelvic lymph nodes
        "C8417",  # Sezary disease, spleen
        "C8418",  # Sezary disease, lymph nodes of multiple sites
        "C8419",  # Sezary disease, extranodal and solid organ sites
        "C8440",  # Peripheral T-cell lymphoma, not elsewhere classified, unspecified site
        "C8441",  # Peripheral T-cell lymphoma, not elsewhere classified, lymph nodes of head, face, and neck
        "C8442",  # Peripheral T-cell lymphoma, not elsewhere classified, intrathoracic lymph nodes
        "C8443",  # Peripheral T-cell lymphoma, not elsewhere classified, intra-abdominal lymph nodes
        "C8444",  # Peripheral T-cell lymphoma, not elsewhere classified, lymph nodes of axilla and upper limb
        "C8445",  # Peripheral T-cell lymphoma, not elsewhere classified, lymph nodes of inguinal region and lower limb
        "C8446",  # Peripheral T-cell lymphoma, not elsewhere classified, intrapelvic lymph nodes
        "C8447",  # Peripheral T-cell lymphoma, not elsewhere classified, spleen
        "C8448",  # Peripheral T-cell lymphoma, not elsewhere classified, lymph nodes of multiple sites
        "C8449",  # Peripheral T-cell lymphoma, not elsewhere classified, extranodal and solid organ sites
        "C8460",  # Anaplastic large cell lymphoma, ALK-positive, unspecified site
        "C8461",  # Anaplastic large cell lymphoma, ALK-positive, lymph nodes of head, face, and neck
        "C8462",  # Anaplastic large cell lymphoma, ALK-positive, intrathoracic lymph nodes
        "C8463",  # Anaplastic large cell lymphoma, ALK-positive, intra-abdominal lymph nodes
        "C8464",  # Anaplastic large cell lymphoma, ALK-positive, lymph nodes of axilla and upper limb
        "C8465",  # Anaplastic large cell lymphoma, ALK-positive, lymph nodes of inguinal region and lower limb
        "C8466",  # Anaplastic large cell lymphoma, ALK-positive, intrapelvic lymph nodes
        "C8467",  # Anaplastic large cell lymphoma, ALK-positive, spleen
        "C8468",  # Anaplastic large cell lymphoma, ALK-positive, lymph nodes of multiple sites
        "C8469",  # Anaplastic large cell lymphoma, ALK-positive, extranodal and solid organ sites
        "C8470",  # Anaplastic large cell lymphoma, ALK-negative, unspecified site
        "C8471",  # Anaplastic large cell lymphoma, ALK-negative, lymph nodes of head, face, and neck
        "C8472",  # Anaplastic large cell lymphoma, ALK-negative, intrathoracic lymph nodes
        "C8473",  # Anaplastic large cell lymphoma, ALK-negative, intra-abdominal lymph nodes
        "C8474",  # Anaplastic large cell lymphoma, ALK-negative, lymph nodes of axilla and upper limb
        "C8475",  # Anaplastic large cell lymphoma, ALK-negative, lymph nodes of inguinal region and lower limb
        "C8476",  # Anaplastic large cell lymphoma, ALK-negative, intrapelvic lymph nodes
        "C8477",  # Anaplastic large cell lymphoma, ALK-negative, spleen
        "C8478",  # Anaplastic large cell lymphoma, ALK-negative, lymph nodes of multiple sites
        "C8479",  # Anaplastic large cell lymphoma, ALK-negative, extranodal and solid organ sites
        "C847A",  # Anaplastic large cell lymphoma, ALK-negative, breast
        "C8490",  # Mature T/NK-cell lymphomas, unspecified, unspecified site
        "C8491",  # Mature T/NK-cell lymphomas, unspecified, lymph nodes of head, face, and neck
        "C8492",  # Mature T/NK-cell lymphomas, unspecified, intrathoracic lymph nodes
        "C8493",  # Mature T/NK-cell lymphomas, unspecified, intra-abdominal lymph nodes
        "C8494",  # Mature T/NK-cell lymphomas, unspecified, lymph nodes of axilla and upper limb
        "C8495",  # Mature T/NK-cell lymphomas, unspecified, lymph nodes of inguinal region and lower limb
        "C8496",  # Mature T/NK-cell lymphomas, unspecified, intrapelvic lymph nodes
        "C8497",  # Mature T/NK-cell lymphomas, unspecified, spleen
        "C8498",  # Mature T/NK-cell lymphomas, unspecified, lymph nodes of multiple sites
        "C8499",  # Mature T/NK-cell lymphomas, unspecified, extranodal and solid organ sites
        "C84A0",  # Cutaneous T-cell lymphoma, unspecified, unspecified site
        "C84A1",  # Cutaneous T-cell lymphoma, unspecified lymph nodes of head, face, and neck
        "C84A2",  # Cutaneous T-cell lymphoma, unspecified, intrathoracic lymph nodes
        "C84A3",  # Cutaneous T-cell lymphoma, unspecified, intra-abdominal lymph nodes
        "C84A4",  # Cutaneous T-cell lymphoma, unspecified, lymph nodes of axilla and upper limb
        "C84A5",  # Cutaneous T-cell lymphoma, unspecified, lymph nodes of inguinal region and lower limb
        "C84A6",  # Cutaneous T-cell lymphoma, unspecified, intrapelvic lymph nodes
        "C84A7",  # Cutaneous T-cell lymphoma, unspecified, spleen
        "C84A8",  # Cutaneous T-cell lymphoma, unspecified, lymph nodes of multiple sites
        "C84A9",  # Cutaneous T-cell lymphoma, unspecified, extranodal and solid organ sites
        "C84Z0",  # Other mature T/NK-cell lymphomas, unspecified site
        "C84Z1",  # Other mature T/NK-cell lymphomas, lymph nodes of head, face, and neck
        "C84Z2",  # Other mature T/NK-cell lymphomas, intrathoracic lymph nodes
        "C84Z3",  # Other mature T/NK-cell lymphomas, intra-abdominal lymph nodes
        "C84Z4",  # Other mature T/NK-cell lymphomas, lymph nodes of axilla and upper limb
        "C84Z5",  # Other mature T/NK-cell lymphomas, lymph nodes of inguinal region and lower limb
        "C84Z6",  # Other mature T/NK-cell lymphomas, intrapelvic lymph nodes
        "C84Z7",  # Other mature T/NK-cell lymphomas, spleen
        "C84Z8",  # Other mature T/NK-cell lymphomas, lymph nodes of multiple sites
        "C84Z9",  # Other mature T/NK-cell lymphomas, extranodal and solid organ sites
        "C8510",  # Unspecified B-cell lymphoma, unspecified site
        "C8511",  # Unspecified B-cell lymphoma, lymph nodes of head, face, and neck
        "C8512",  # Unspecified B-cell lymphoma, intrathoracic lymph nodes
        "C8513",  # Unspecified B-cell lymphoma, intra-abdominal lymph nodes
        "C8514",  # Unspecified B-cell lymphoma, lymph nodes of axilla and upper limb
        "C8515",  # Unspecified B-cell lymphoma, lymph nodes of inguinal region and lower limb
        "C8516",  # Unspecified B-cell lymphoma, intrapelvic lymph nodes
        "C8517",  # Unspecified B-cell lymphoma, spleen
        "C8518",  # Unspecified B-cell lymphoma, lymph nodes of multiple sites
        "C8519",  # Unspecified B-cell lymphoma, extranodal and solid organ sites
        "C8520",  # Mediastinal (thymic) large B-cell lymphoma, unspecified site
        "C8521",  # Mediastinal (thymic) large B-cell lymphoma, lymph nodes of head, face, and neck
        "C8522",  # Mediastinal (thymic) large B-cell lymphoma, intrathoracic lymph nodes
        "C8523",  # Mediastinal (thymic) large B-cell lymphoma, intra-abdominal lymph nodes
        "C8524",  # Mediastinal (thymic) large B-cell lymphoma, lymph nodes of axilla and upper limb
        "C8525",  # Mediastinal (thymic) large B-cell lymphoma, lymph nodes of inguinal region and lower limb
        "C8526",  # Mediastinal (thymic) large B-cell lymphoma, intrapelvic lymph nodes
        "C8527",  # Mediastinal (thymic) large B-cell lymphoma, spleen
        "C8528",  # Mediastinal (thymic) large B-cell lymphoma, lymph nodes of multiple sites
        "C8529",  # Mediastinal (thymic) large B-cell lymphoma, extranodal and solid organ sites
        "C8580",  # Other specified types of non-Hodgkin lymphoma, unspecified site
        "C8581",  # Other specified types of non-Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8582",  # Other specified types of non-Hodgkin lymphoma, intrathoracic lymph nodes
        "C8583",  # Other specified types of non-Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8584",  # Other specified types of non-Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8585",  # Other specified types of non-Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8586",  # Other specified types of non-Hodgkin lymphoma, intrapelvic lymph nodes
        "C8587",  # Other specified types of non-Hodgkin lymphoma, spleen
        "C8588",  # Other specified types of non-Hodgkin lymphoma, lymph nodes of multiple sites
        "C8589",  # Other specified types of non-Hodgkin lymphoma, extranodal and solid organ sites
        "C8590",  # Non-Hodgkin lymphoma, unspecified, unspecified site
        "C8591",  # Non-Hodgkin lymphoma, unspecified, lymph nodes of head, face, and neck
        "C8592",  # Non-Hodgkin lymphoma, unspecified, intrathoracic lymph nodes
        "C8593",  # Non-Hodgkin lymphoma, unspecified, intra-abdominal lymph nodes
        "C8594",  # Non-Hodgkin lymphoma, unspecified, lymph nodes of axilla and upper limb
        "C8595",  # Non-Hodgkin lymphoma, unspecified, lymph nodes of inguinal region and lower limb
        "C8596",  # Non-Hodgkin lymphoma, unspecified, intrapelvic lymph nodes
        "C8597",  # Non-Hodgkin lymphoma, unspecified, spleen
        "C8598",  # Non-Hodgkin lymphoma, unspecified, lymph nodes of multiple sites
        "C8599",  # Non-Hodgkin lymphoma, unspecified, extranodal and solid organ sites
        "C860",  # Extranodal NK/T-cell lymphoma, nasal type
        "C861",  # Hepatosplenic T-cell lymphoma
        "C862",  # Enteropathy-type (intestinal) T-cell lymphoma
        "C863",  # Subcutaneous panniculitis-like T-cell lymphoma
        "C864",  # Blastic NK-cell lymphoma
        "C865",  # Angioimmunoblastic T-cell lymphoma
        "C866",  # Primary cutaneous CD30-positive T-cell proliferations
        "C882",  # Heavy chain disease
        "C883",  # Immunoproliferative small intestinal disease
        "C884",  # Extranodal marginal zone B-cell lymphoma of mucosa-associated lymphoid tissue [MALT-lymphoma]
        "C888",  # Other malignant immunoproliferative diseases
        "C889",  # Malignant immunoproliferative disease, unspecified
        "C9000",  # Multiple myeloma not having achieved remission
        "C9001",  # Multiple myeloma in remission
        "C9002",  # Multiple myeloma in relapse
        "C9010",  # Plasma cell leukemia not having achieved remission
        "C9011",  # Plasma cell leukemia in remission
        "C9012",  # Plasma cell leukemia in relapse
        "C9020",  # Extramedullary plasmacytoma not having achieved remission
        "C9021",  # Extramedullary plasmacytoma in remission
        "C9022",  # Extramedullary plasmacytoma in relapse
        "C9030",  # Solitary plasmacytoma not having achieved remission
        "C9031",  # Solitary plasmacytoma in remission
        "C9032",  # Solitary plasmacytoma in relapse
        "C9100",  # Acute lymphoblastic leukemia not having achieved remission
        "C9101",  # Acute lymphoblastic leukemia, in remission
        "C9102",  # Acute lymphoblastic leukemia, in relapse
        "C9110",  # Chronic lymphocytic leukemia of B-cell type not having achieved remission
        "C9111",  # Chronic lymphocytic leukemia of B-cell type in remission
        "C9112",  # Chronic lymphocytic leukemia of B-cell type in relapse
        "C9130",  # Prolymphocytic leukemia of B-cell type not having achieved remission
        "C9131",  # Prolymphocytic leukemia of B-cell type, in remission
        "C9132",  # Prolymphocytic leukemia of B-cell type, in relapse
        "C9140",  # Hairy cell leukemia not having achieved remission
        "C9141",  # Hairy cell leukemia, in remission
        "C9142",  # Hairy cell leukemia, in relapse
        "C9150",  # Adult T-cell lymphoma/leukemia (HTLV-1-associated) not having achieved remission
        "C9151",  # Adult T-cell lymphoma/leukemia (HTLV-1-associated), in remission
        "C9152",  # Adult T-cell lymphoma/leukemia (HTLV-1-associated), in relapse
        "C9160",  # Prolymphocytic leukemia of T-cell type not having achieved remission
        "C9161",  # Prolymphocytic leukemia of T-cell type, in remission
        "C9162",  # Prolymphocytic leukemia of T-cell type, in relapse
        "C9190",  # Lymphoid leukemia, unspecified not having achieved remission
        "C9191",  # Lymphoid leukemia, unspecified, in remission
        "C9192",  # Lymphoid leukemia, unspecified, in relapse
        "C91A0",  # Mature B-cell leukemia Burkitt-type not having achieved remission
        "C91A1",  # Mature B-cell leukemia Burkitt-type, in remission
        "C91A2",  # Mature B-cell leukemia Burkitt-type, in relapse
        "C91Z0",  # Other lymphoid leukemia not having achieved remission
        "C91Z1",  # Other lymphoid leukemia, in remission
        "C91Z2",  # Other lymphoid leukemia, in relapse
        "C9200",  # Acute myeloblastic leukemia, not having achieved remission
        "C9201",  # Acute myeloblastic leukemia, in remission
        "C9202",  # Acute myeloblastic leukemia, in relapse
        "C9210",  # Chronic myeloid leukemia, BCR/ABL-positive, not having achieved remission
        "C9211",  # Chronic myeloid leukemia, BCR/ABL-positive, in remission
        "C9212",  # Chronic myeloid leukemia, BCR/ABL-positive, in relapse
        "C9220",  # Atypical chronic myeloid leukemia, BCR/ABL-negative, not having achieved remission
        "C9221",  # Atypical chronic myeloid leukemia, BCR/ABL-negative, in remission
        "C9222",  # Atypical chronic myeloid leukemia, BCR/ABL-negative, in relapse
        "C9230",  # Myeloid sarcoma, not having achieved remission
        "C9231",  # Myeloid sarcoma, in remission
        "C9232",  # Myeloid sarcoma, in relapse
        "C9240",  # Acute promyelocytic leukemia, not having achieved remission
        "C9241",  # Acute promyelocytic leukemia, in remission
        "C9242",  # Acute promyelocytic leukemia, in relapse
        "C9250",  # Acute myelomonocytic leukemia, not having achieved remission
        "C9251",  # Acute myelomonocytic leukemia, in remission
        "C9252",  # Acute myelomonocytic leukemia, in relapse
        "C9260",  # Acute myeloid leukemia with 11q23-abnormality not having achieved remission
        "C9261",  # Acute myeloid leukemia with 11q23-abnormality in remission
        "C9262",  # Acute myeloid leukemia with 11q23-abnormality in relapse
        "C9290",  # Myeloid leukemia, unspecified, not having achieved remission
        "C9291",  # Myeloid leukemia, unspecified in remission
        "C9292",  # Myeloid leukemia, unspecified in relapse
        "C92A0",  # Acute myeloid leukemia with multilineage dysplasia, not having achieved remission
        "C92A1",  # Acute myeloid leukemia with multilineage dysplasia, in remission
        "C92A2",  # Acute myeloid leukemia with multilineage dysplasia, in relapse
        "C92Z0",  # Other myeloid leukemia not having achieved remission
        "C92Z1",  # Other myeloid leukemia, in remission
        "C92Z2",  # Other myeloid leukemia, in relapse
        "C9300",  # Acute monoblastic/monocytic leukemia, not having achieved remission
        "C9301",  # Acute monoblastic/monocytic leukemia, in remission
        "C9302",  # Acute monoblastic/monocytic leukemia, in relapse
        "C9310",  # Chronic myelomonocytic leukemia not having achieved remission
        "C9311",  # Chronic myelomonocytic leukemia, in remission
        "C9312",  # Chronic myelomonocytic leukemia, in relapse
        "C9330",  # Juvenile myelomonocytic leukemia, not having achieved remission
        "C9331",  # Juvenile myelomonocytic leukemia, in remission
        "C9332",  # Juvenile myelomonocytic leukemia, in relapse
        "C9390",  # Monocytic leukemia, unspecified, not having achieved remission
        "C9391",  # Monocytic leukemia, unspecified in remission
        "C9392",  # Monocytic leukemia, unspecified in relapse
        "C93Z0",  # Other monocytic leukemia, not having achieved remission
        "C93Z1",  # Other monocytic leukemia, in remission
        "C93Z2",  # Other monocytic leukemia, in relapse
        "C9400",  # Acute erythroid leukemia, not having achieved remission
        "C9401",  # Acute erythroid leukemia, in remission
        "C9402",  # Acute erythroid leukemia, in relapse
        "C9420",  # Acute megakaryoblastic leukemia not having achieved remission
        "C9421",  # Acute megakaryoblastic leukemia, in remission
        "C9422",  # Acute megakaryoblastic leukemia, in relapse
        "C9430",  # Mast cell leukemia not having achieved remission
        "C9431",  # Mast cell leukemia, in remission
        "C9432",  # Mast cell leukemia, in relapse
        "C9480",  # Other specified leukemias not having achieved remission
        "C9481",  # Other specified leukemias, in remission
        "C9482",  # Other specified leukemias, in relapse
        "C9500",  # Acute leukemia of unspecified cell type not having achieved remission
        "C9501",  # Acute leukemia of unspecified cell type, in remission
        "C9502",  # Acute leukemia of unspecified cell type, in relapse
        "C9510",  # Chronic leukemia of unspecified cell type not having achieved remission
        "C9511",  # Chronic leukemia of unspecified cell type, in remission
        "C9512",  # Chronic leukemia of unspecified cell type, in relapse
        "C9590",  # Leukemia, unspecified not having achieved remission
        "C9591",  # Leukemia, unspecified, in remission
        "C9592",  # Leukemia, unspecified, in relapse
        "C960",  # Multifocal and multisystemic (disseminated) Langerhans-cell histiocytosis
        "C9620",  # Malignant mast cell neoplasm, unspecified
        "C9621",  # Aggressive systemic mastocytosis
        "C9622",  # Mast cell sarcoma
        "C9629",  # Other malignant mast cell neoplasm
        "C964",  # Sarcoma of dendritic cells (accessory cells)
        "C969",  # Malignant neoplasm of lymphoid, hematopoietic and related tissue, unspecified
        "C96A",  # Histiocytic sarcoma
        "C96Z",  # Other specified malignant neoplasms of lymphoid, hematopoietic and related tissue
    }

    SNOMEDCT = {
        "116691000119101",  # Marginal zone lymphoma of spleen (disorder)
        "1255740000",  # Letterer-Siwe disease of lymph nodes of head (disorder)
        "1255741001",  # Letterer-Siwe disease of lymph nodes of neck (disorder)
        "1255742008",  # Letterer-Siwe disease of lymph nodes of head and neck (disorder)
        "1255813007",  # Letterer-Siwe disease of lymph nodes of upper limb (disorder)
        "1255885008",  # Letterer-Siwe disease of lymph nodes of lower limb (disorder)
        "188679001",  # Diffuse non-Hodgkin's lymphoma undifferentiated (diffuse) (disorder)
        "303017006",  # Malignant lymphoma, convoluted cell type (disorder)
        "39795003",  # Hand-Schuller-Christian disease (disorder)
        "93133006",  # Letterer-Siwe disease of intra-abdominal lymph nodes (disorder)
        "93134000",  # Letterer-Siwe disease of intrapelvic lymph nodes (disorder)
        "93135004",  # Letterer-Siwe disease of intrathoracic lymph nodes (disorder)
        "93139005",  # Letterer-Siwe disease of lymph nodes of multiple sites (disorder)
        "93140007",  # Letterer-Siwe disease of spleen (disorder)
        "109962001",  # Diffuse non-Hodgkin's lymphoma (disorder)
        "109964000",  # Diffuse non-Hodgkin's lymphoma, undifferentiated (disorder)
        "109966003",  # Diffuse non-Hodgkin's lymphoma, immunoblastic (disorder)
        "109967007",  # Diffuse non-Hodgkin's lymphoma, small cleaved cell (disorder)
        "109968002",  # Diffuse non-Hodgkin's lymphoma, small cell (disorder)
        "109969005",  # Diffuse large B-cell malignant lymphoma (disorder)
        "109970006",  # Follicular non-Hodgkin's lymphoma, small cleaved cell (disorder)
        "109971005",  # Follicular non-Hodgkin's lymphoma, mixed small cleaved cell and large cell (disorder)
        "109972003",  # Follicular non-Hodgkin's lymphoma, large cell (disorder)
        "109975001",  # T-zone lymphoma (disorder)
        "109976000",  # Lymphoepithelioid lymphoma (disorder)
        "109977009",  # Peripheral T-cell lymphoma (disorder)
        "109978004",  # T-cell lymphoma (disorder)
        "109979007",  # B-cell lymphoma (disorder)
        "109980005",  # Malignant immunoproliferative disease (disorder)
        "109988003",  # Histiocytic sarcoma (disorder)
        "109989006",  # Multiple myeloma (disorder)
        "109992005",  # Polycythemia vera (disorder)
        "109993000",  # Chronic myeloproliferative disorder (clinical) (disorder)
        "109994006",  # Essential thrombocythemia (disorder)
        "109995007",  # Myelodysplastic syndrome (disorder)
        "110002002",  # Mast cell leukemia (disorder)
        "110004001",  # Acute promyelocytic leukemia, FAB M3 (disorder)
        "110005000",  # Acute myelomonocytic leukemia, FAB M4 (disorder)
        "110006004",  # Prolymphocytic leukemia (disorder)
        "110007008",  # Adult T-cell leukemia/lymphoma (disorder)
        "1148845007",  # Follicular lymphoma grade 3b (disorder)
        "1148847004",  # Follicular non-Hodgkin's lymphoma in situ (disorder)
        "1148851002",  # Follicular lymphoma grade 3a (disorder)
        "1148906001",  # Acute myeloid leukemia with t(8;21)(q22;q22) RUNX1-RUNX1T1 (disorder)
        "1153336009",  # Pediatric nodal marginal zone B cell lymphoma (disorder)
        "1153341001",  # Myeloproliferative hypereosinophilic syndrome (disorder)
        "1153346006",  # Primary non-Hodgkin malignant lymphoma of uveal tract (disorder)
        "1153347002",  # Primary non-Hodgkin malignant lymphoma of vitreoretinal tract (disorder)
        "1153348007",  # Primary effusion lymphoma due to human immune deficiency virus infection (disorder)
        "1153349004",  # Primary familial polycythemia due to erythropoietin receptor mutation (disorder)
        "1153354008",  # Malignant lymphoma of duodenum (disorder)
        "1153355009",  # Malignant lymphoma of esophagus (disorder)
        "1153356005",  # Lymphoma of anus (disorder)
        "1153357001",  # Lymphoma of appendix (disorder)
        "1153377008",  # Mixed phenotype acute leukemia with t(9;22) (q34;q11.2); BCR-ABL1 (disorder)
        "1153379006",  # Mixed phenotype acute leukemia with T-cell and myeloid lineage (disorder)
        "1153381008",  # Mixed phenotype acute leukemia with myeloid and B-cell lymphoid phenotypes (disorder)
        "1153382001",  # Malignant lymphoma of lacrimal gland (disorder)
        "1153383006",  # Malignant lymphoma of liver (disorder)
        "1156403002",  # Composite Hodgkin and non-Hodgkin lymphoma (disorder)
        "1156495004",  # Dendritic cell sarcoma (disorder)
        "1157157006",  # Acute myeloid leukemia with 11q23 abnormality (disorder)
        "1157162007",  # Intravascular large B-cell lymphoma (disorder)
        "1162264008",  # Malignant lymphoma of uveal tract (disorder)
        "1162531007",  # Acute myeloid leukemia in complete remission (disorder)
        "116711000119103",  # Marginal zone lymphoma of inguinal lymph node (disorder)
        "116741000119104",  # Marginal zone lymphoma of thoracic lymph node (disorder)
        "116811000119106",  # Non-Hodgkin lymphoma of central nervous system metastatic to lymph node of lower limb (disorder)
        "116821000119104",  # Non-Hodgkin lymphoma of central nervous system metastatic to lymph node of upper limb (disorder)
        "116841000119105",  # Marginal zone lymphoma of lymph nodes of multiple sites (disorder)
        "116871000119103",  # Mantle cell lymphoma of lymph nodes of multiple sites (disorder)
        "117061000119101",  # Peripheral T-cell lymphoma of spleen (disorder)
        "117091000119108",  # Peripheral T-cell lymphoma of axillary lymph node (disorder)
        "117111000119100",  # Peripheral T-cell lymphoma of thoracic lymph node (disorder)
        "117211000119105",  # Peripheral T-cell lymphoma of lymph nodes of multiple sites (disorder)
        "1172704005",  # High grade B-cell lymphoma with MYC and BCL2 and/or BCL6 rearrangements (disorder)
        "1177173001",  # Congenital progressive bone marrow failure, B-cell immunodeficiency, skeletal dysplasia syndrome (disorder)
        "118599009",  # Hodgkin's disease (disorder)
        "118600007",  # Malignant lymphoma (disorder)
        "118601006",  # Non-Hodgkin's lymphoma (disorder)
        "118602004",  # Hodgkin's granuloma (disorder)
        "118605002",  # Hodgkin lymphoma, nodular lymphocyte predominance (disorder)
        "118606001",  # Hodgkin's sarcoma (disorder)
        "118607005",  # Hodgkin lymphoma, lymphocyte-rich (disorder)
        "118608000",  # Hodgkin's disease, nodular sclerosis (disorder)
        "118609008",  # Hodgkin's disease, mixed cellularity (disorder)
        "118610003",  # Hodgkin's disease, lymphocytic depletion (disorder)
        "118611004",  # Sezary's disease (disorder)
        "118612006",  # Malignant histiocytosis (disorder)
        "118613001",  # Hairy cell leukemia (disorder)
        "118614007",  # Langerhans cell histiocytosis, disseminated (disorder)
        "118615008",  # Malignant mast cell tumor (disorder)
        "118617000",  # Burkitt's lymphoma (disorder)
        "118618005",  # Mycosis fungoides (disorder)
        "1187123005",  # Mixed phenotype acute leukemia (disorder)
        "1196900003",  # B cell lymphoma of small intestine (disorder)
        "1197344000",  # T-cell lymphoma of small intestine (disorder)
        "1217301006",  # Precursor cell lymphoblastic lymphoma (disorder)
        "12281000132104",  # Relapsing acute myeloid leukemia (disorder)
        "122881000119107",  # Chronic monocytic leukemia in relapse (disorder)
        "122901000119109",  # Myeloid leukemia in relapse (disorder)
        "12291000132102",  # Refractory acute myeloid leukemia (disorder)
        "122951000119108",  # Lymphoid leukemia in relapse (disorder)
        "122961000119105",  # Chronic lymphoid leukemia in relapse (disorder)
        "122981000119101",  # Plasma cell leukemia in relapse (disorder)
        "1229988003",  # Myeloid and/or lymphoid neoplasm with pericentriolar material 1-janus kinase 2 (disorder)
        "12301000132103",  # Acute lymphoid leukemia relapse (disorder)
        "12311000132101",  # Refractory acute lymphoid leukemia (disorder)
        "1237368006",  # Acute myeloid leukemia with BCR-ABL1 (disorder)
        "1237578005",  # Primary oculocerebral non-Hodgkin lymphoma (disorder)
        "123781000119107",  # Chronic leukemia in relapse (disorder)
        "1255270006",  # Lymphoplasmacytic lymphoma without immunoglobulin M production (disorder)
        "1255666008",  # Hodgkin lymphoma, lymphocyte depletion of lymph nodes of neck (disorder)
        "1255687003",  # Hodgkin lymphoma, lymphocyte depletion of lymph nodes of head and neck (disorder)
        "1255726001",  # Hodgkin granuloma of lymph nodes of head (disorder)
        "1255727005",  # Hodgkin granuloma of lymph nodes of neck (disorder)
        "1255728000",  # Hodgkin granuloma of lymph nodes of head and neck (disorder)
        "1255737000",  # Hodgkin disease, lymphocytic-histiocytic predominance of lymph nodes of head (disorder)
        "1255738005",  # Hodgkin disease, lymphocytic-histiocytic predominance of lymph nodes of neck (disorder)
        "1255739002",  # Hodgkin disease, lymphocytic-histiocytic predominance of lymph nodes of head and neck (disorder)
        "1255743003",  # Hodgkin sarcoma of lymph nodes of head (disorder)
        "1255744009",  # Hodgkin sarcoma of lymph nodes of neck (disorder)
        "1255745005",  # Hodgkin sarcoma of lymph nodes of head and neck (disorder)
        "1255746006",  # Malignant histiocytosis of lymph nodes of head (disorder)
        "1255747002",  # Malignant histiocytosis of lymph nodes of neck (disorder)
        "1255748007",  # Malignant histiocytosis of lymph nodes of head and neck (disorder)
        "1255749004",  # Burkitt tumor of lymph nodes of head (disorder)
        "1255750004",  # Burkitt tumor of lymph nodes of neck (disorder)
        "1255751000",  # Burkitt tumor of lymph nodes of head and neck (disorder)
        "1255752007",  # Mycosis fungoides of lymph nodes of head (disorder)
        "1255753002",  # Mycosis fungoides of lymph nodes of neck (disorder)
        "1255754008",  # Mycosis fungoides of lymph nodes of head and neck (disorder)
        "1255784000",  # Hodgkin paragranuloma of lymph nodes of head (disorder)
        "1255785004",  # Hodgkin paragranuloma of lymph nodes of neck (disorder)
        "1255786003",  # Hodgkin paragranuloma of lymph nodes of head and neck (disorder)
        "1255788002",  # Hodgkin disease, mixed cellularity of lymph nodes of head (disorder)
        "1255789005",  # Hodgkin disease, mixed cellularity of lymph nodes of neck (disorder)
        "1255790001",  # Hodgkin disease, mixed cellularity of lymph nodes of head and neck (disorder)
        "1255791002",  # Hodgkin disease, nodular sclerosis of lymph nodes of head and neck (disorder)
        "1255792009",  # Hodgkin disease, nodular sclerosis of lymph nodes of head (disorder)
        "1255793004",  # Hodgkin disease, nodular sclerosis of lymph nodes of neck (disorder)
        "1255795006",  # Malignant histiocytosis of lymph nodes of lower limb (disorder)
        "1255796007",  # Burkitt lymphoma of lymph nodes of lower limb (disorder)
        "1255797003",  # Hodgkin disease, nodular sclerosis of lymph nodes of lower limb (disorder)
        "1255798008",  # Hodgkin granuloma of lymph nodes of upper limb (disorder)
        "1255799000",  # Sezary disease of lymph nodes of head (disorder)
        "1255800001",  # Sezary disease of lymph nodes of neck (disorder)
        "1255801002",  # Sezary disease of lymph nodes of head and neck (disorder)
        "1255802009",  # Hodgkin disease of lymph nodes of head (disorder)
        "1255803004",  # Hodgkin disease of lymph nodes of neck (disorder)
        "1255804005",  # Hodgkin disease of lymph nodes of head and neck (disorder)
        "1255805006",  # Malignant lymphoma of lymph nodes of head (disorder)
        "1255806007",  # Malignant lymphoma of lymph nodes of neck (disorder)
        "1255807003",  # Malignant lymphoma of lymph nodes of head and neck (disorder)
        "1255808008",  # Hodgkin sarcoma of lymph nodes of upper limb (disorder)
        "1255811009",  # Hodgkin disease, mixed cellularity of lymph nodes of upper limb (disorder)
        "1255812002",  # Hodgkin disease, nodular sclerosis of lymph nodes of upper limb (disorder)
        "1255814001",  # Hodgkin disease, lymphocytic-histiocytic predominance of lymph nodes of upper limb (disorder)
        "1255815000",  # Malignant histiocytosis of lymph nodes of upper limb (disorder)
        "1255883001",  # Hodgkin sarcoma of lymph nodes of lower limb (disorder)
        "1255886009",  # Mycosis fungoides of lymph nodes of upper limb (disorder)
        "1255887000",  # Hodgkin's paragranuloma of lymph nodes of lower limb (disorder)
        "1255888005",  # Hodgkin paragranuloma of lymph nodes of upper limb (disorder)
        "1255890006",  # Hodgkin disease, lymphocytic depletion of lymph nodes of upper limb (disorder)
        "1255892003",  # Hodgkin disease, lymphocytic depletion of lymph nodes of lower limb (disorder)
        "1255894002",  # Hodgkin granuloma of lymph nodes of lower limb (disorder)
        "1255895001",  # Burkitt lymphoma of lymph nodes of upper limb (disorder)
        "1255897009",  # Hodgkin disease, lymphocytic-histiocytic predominance of lymph nodes of lower limb (disorder)
        "1255898004",  # Hodgkin disease, mixed cellularity of lymph nodes of lower limb (disorder)
        "1255902005",  # Hodgkin disease of lymph nodes of lower limb (disorder)
        "1255905007",  # Hodgkin disease of lymph nodes of upper limb (disorder)
        "1255906008",  # Malignant lymphoma of lymph nodes of upper limb (disorder)
        "1255907004",  # Malignant lymphoma of lymph nodes of lower limb (disorder)
        "1260217004",  # Myelodysplastic neoplasm with low blasts and SF3B1 mutation (disorder)
        "1260268001",  # Myelodysplastic syndrome with low blasts (disorder)
        "1260270005",  # Hypoplastic myelodysplastic syndrome (disorder)
        "1260406003",  # Myelodysplastic neoplasm with biallelic TP53 inactivation (disorder)
        "1264416003",  # Unclassifiable myelodysplastic syndrome (disorder)
        "1264458000",  # Acute undifferentiated leukemia (disorder)
        "127066000",  # Familial polycythemia vera (disorder)
        "127070008",  # Malignant histiocytic disorder (disorder)
        "127220001",  # Malignant lymphoma of lymph nodes (disorder)
        "127225006",  # Chronic myelomonocytic leukemia (disorder)
        "128875000",  # Primary cutaneous CD30 antigen positive large T-cell lymphoma (disorder)
        "13048006",  # Orbital lymphoma (disorder)
        "133751000119102",  # Lymphoma of colon (disorder)
        "143291000119104",  # Non-Hodgkin lymphoma in remission (disorder)
        "16775311000119107",  # Mycosis fungoides of facial lymph node (disorder)
        "187822008",  # Fibrosarcoma of spleen (disorder)
        "188487008",  # Lymphosarcoma and reticulosarcoma (disorder)
        "188489006",  # Reticulosarcoma of lymph nodes of head, face and neck (disorder)
        "188492005",  # Reticulosarcoma of lymph nodes of axilla and upper limb (disorder)
        "188493000",  # Reticulosarcoma of lymph nodes of inguinal region and lower limb (disorder)
        "188498009",  # Lymphosarcoma (disorder)
        "188500005",  # Lymphosarcoma of lymph nodes of head, face and neck (disorder)
        "188501009",  # Lymphosarcoma of intrathoracic lymph nodes (disorder)
        "188502002",  # Lymphosarcoma of intra-abdominal lymph nodes (disorder)
        "188503007",  # Lymphosarcoma of lymph nodes of axilla and upper limb (disorder)
        "188504001",  # Lymphosarcoma of lymph nodes of inguinal region and lower limb (disorder)
        "188505000",  # Lymphosarcoma of intrapelvic lymph nodes (disorder)
        "188506004",  # Lymphosarcoma of spleen (disorder)
        "188507008",  # Lymphosarcoma of lymph nodes of multiple sites (disorder)
        "188511002",  # Burkitt's lymphoma of intrathoracic lymph nodes (disorder)
        "188512009",  # Burkitt's lymphoma of intra-abdominal lymph nodes (disorder)
        "188515006",  # Burkitt's lymphoma of intrapelvic lymph nodes (disorder)
        "188516007",  # Burkitt's lymphoma of spleen (disorder)
        "188517003",  # Burkitt's lymphoma of lymph nodes of multiple sites (disorder)
        "188524002",  # Hodgkin's paragranuloma of intrathoracic lymph nodes (disorder)
        "188529007",  # Hodgkin's paragranuloma of intrapelvic lymph nodes (disorder)
        "188531003",  # Hodgkin's paragranuloma of lymph nodes of multiple sites (disorder)
        "188536008",  # Hodgkin's granuloma of intra-abdominal lymph nodes (disorder)
        "188541000",  # Hodgkin's granuloma of lymph nodes of multiple sites (disorder)
        "188551004",  # Hodgkin's sarcoma of lymph nodes of multiple sites (disorder)
        "188562004",  # Hodgkin's disease, lymphocytic-histiocytic predominance of lymph nodes of multiple sites (disorder)
        "188566001",  # Hodgkin's disease, nodular sclerosis of intrathoracic lymph nodes (disorder)
        "188567005",  # Hodgkin's disease, nodular sclerosis of intra-abdominal lymph nodes (disorder)
        "188570009",  # Hodgkin's disease, nodular sclerosis of intrapelvic lymph nodes (disorder)
        "188572001",  # Hodgkin's disease, nodular sclerosis of lymph nodes of multiple sites (disorder)
        "188576003",  # Hodgkin's disease, mixed cellularity of intrathoracic lymph nodes (disorder)
        "188577007",  # Hodgkin's disease, mixed cellularity of intra-abdominal lymph nodes (disorder)
        "188580008",  # Hodgkin's disease, mixed cellularity of intrapelvic lymph nodes (disorder)
        "188582000",  # Hodgkin's disease, mixed cellularity of lymph nodes of multiple sites (disorder)
        "188586002",  # Hodgkin's disease, lymphocytic depletion of intrathoracic lymph nodes (disorder)
        "188587006",  # Hodgkin's disease, lymphocytic depletion of intra-abdominal lymph nodes (disorder)
        "188591001",  # Hodgkin's disease, lymphocytic depletion of intrapelvic lymph nodes (disorder)
        "188592008",  # Hodgkin's disease, lymphocytic depletion of spleen (disorder)
        "188593003",  # Hodgkin's disease, lymphocytic depletion of lymph nodes of multiple sites (disorder)
        "188609000",  # Nodular lymphoma of lymph nodes of head, face and neck (disorder)
        "188612002",  # Nodular lymphoma of lymph nodes of axilla and upper limb (disorder)
        "188613007",  # Nodular lymphoma of lymph nodes of inguinal region and lower limb (disorder)
        "188627002",  # Mycosis fungoides of lymph nodes of multiple sites (disorder)
        "188631008",  # Sezary's disease of intrathoracic lymph nodes (disorder)
        "188632001",  # Sezary's disease of intra-abdominal lymph nodes (disorder)
        "188633006",  # Sezary's disease of lymph nodes of axilla and upper limb (disorder)
        "188634000",  # Sezary's disease of lymph nodes of inguinal region and lower limb (disorder)
        "188635004",  # Sezary's disease of intrapelvic lymph nodes (disorder)
        "188637007",  # Sezary's disease of lymph nodes of multiple sites (disorder)
        "188641006",  # Malignant histiocytosis of lymph nodes of axilla and upper limb (disorder)
        "188645002",  # Leukemic reticuloendotheliosis of lymph nodes of head, face and neck (disorder)
        "188648000",  # Leukemic reticuloendotheliosis of lymph nodes of axilla and upper limb (disorder)
        "188649008",  # Leukemic reticuloendotheliosis of lymph nodes of inguinal region and lower limb (disorder)
        "188662007",  # Mast cell malignancy of lymph nodes of head and neck (disorder)
        "188668006",  # Malignant mast cell sarcoma of spleen (disorder)
        "188669003",  # Mast cell malignancy of lymph nodes of multiple sites (disorder)
        "188672005",  # Follicular non-Hodgkin's mixed small cleaved and large cell lymphoma (disorder)
        "188674006",  # Diffuse malignant lymphoma - small non-cleaved cell (disorder)
        "188675007",  # Malignant lymphoma - small cleaved cell (disorder)
        "188676008",  # Malignant lymphoma - mixed small and large cell (disorder)
        "188718006",  # Malignant plasma cell neoplasm, extramedullary plasmacytoma (disorder)
        "188725004",  # Lymphoid leukemia (disorder)
        "188726003",  # Subacute lymphoid leukemia (disorder)
        "188728002",  # Aleukemic lymphoid leukemia (disorder)
        "188732008",  # Myeloid leukemia (disorder)
        "188733003",  # Chronic eosinophilic leukemia (disorder)
        "188734009",  # Chronic neutrophilic leukemia (disorder)
        "188737002",  # Chloroma (disorder)
        "188744006",  # Monocytic leukemia (disorder)
        "188754005",  # Megakaryocytic leukemia (disorder)
        "188768003",  # Myelomonocytic leukemia (disorder)
        "188770007",  # Subacute myelomonocytic leukemia (disorder)
        "190030009",  # Compound leukemias (disorder)
        "232075002",  # Lymphoma of retina (disorder)
        "236512004",  # Leukemic infiltrate of kidney (disorder)
        "236513009",  # Lymphoma of kidney (disorder)
        "240531002",  # African Burkitt's lymphoma (disorder)
        "254792006",  # Proliferating angioendotheliomatosis (disorder)
        "255191003",  # Localized malignant reticulohistiocytoma (disorder)
        "269475001",  # Malignant tumor of lymphoid, hemopoietic AND/OR related tissue (disorder)
        "269476000",  # Nodular lymphoma (disorder)
        "274905008",  # Malignant lymphoma - lymphocytic, intermediate differentiation (disorder)
        "276811008",  # Gastric lymphoma (disorder)
        "276815004",  # Lymphoma of intestine (disorder)
        "276836002",  # Primary cerebral lymphoma (disorder)
        "277474005",  # B-cell chronic lymphocytic leukemia variant (disorder)
        "277545003",  # T-cell chronic lymphocytic leukemia (disorder)
        "277549009",  # Chronic lymphocytic prolymphocytic leukemia syndrome (disorder)
        "277550009",  # Richter's syndrome (disorder)
        "277551008",  # Splenic lymphoma with villous lymphocytes (disorder)
        "277567002",  # T-cell prolymphocytic leukemia (disorder)
        "277568007",  # Hairy cell leukemia variant (disorder)
        "277569004",  # Large granular lymphocytic leukemia (disorder)
        "277570003",  # Lymphoma with spill (disorder)
        "277571004",  # B-cell acute lymphoblastic leukemia (disorder)
        "277572006",  # Precursor B-cell acute lymphoblastic leukemia (disorder)
        "277573001",  # Common acute lymphoblastic leukemia (disorder)
        "277574007",  # Null cell acute lymphoblastic leukemia (disorder)
        "277575008",  # T-lymphoblastic leukemia (disorder)
        "277579002",  # Light chain myeloma (disorder)
        "277580004",  # Non-secretory myeloma (disorder)
        "277587001",  # Juvenile chronic myeloid leukemia (disorder)
        "277589003",  # Myelodysplastic-myeloproliferative neoplasm with neutrophilia (disorder)
        "277597005",  # Myelodysplastic syndrome with isolated del(5q) (disorder)
        "277602003",  # Acute megakaryoblastic leukemia (disorder)
        "277604002",  # Malignant myeloid-lymphoid neoplasm (disorder)
        "277609007",  # Hodgkin's disease, lymphocytic predominance - diffuse (disorder)
        "277610002",  # Hodgkin's disease, nodular sclerosis - lymphocytic predominance (disorder)
        "277611003",  # Hodgkin's disease, nodular sclerosis - mixed cellularity (disorder)
        "277612005",  # Hodgkin's disease, nodular sclerosis - lymphocytic depletion (disorder)
        "277613000",  # Cutaneous/peripheral T-cell lymphoma (disorder)
        "277614006",  # Prethymic and thymic T-cell lymphoma/leukemia (disorder)
        "277615007",  # Low grade B-cell lymphoma (disorder)
        "277616008",  # Diffuse low grade B-cell lymphoma (disorder)
        "277617004",  # High grade B-cell lymphoma (disorder)
        "277618009",  # Follicular low grade B-cell lymphoma (disorder)
        "277619001",  # B-cell prolymphocytic leukemia (disorder)
        "277622004",  # Mucosa-associated lymphoma (disorder)
        "277623009",  # Monocytoid B-cell lymphoma (disorder)
        "277624003",  # Follicular malignant lymphoma - mixed cell type (disorder)
        "277625002",  # Follicular malignant lymphoma - small cleaved cell (disorder)
        "277626001",  # Diffuse high grade B-cell lymphoma (disorder)
        "277627005",  # Nodular high grade B-cell lymphoma (disorder)
        "277628000",  # Diffuse malignant lymphoma - large cleaved cell (disorder)
        "277629008",  # Diffuse malignant lymphoma - large non-cleaved cell (disorder)
        "277632006",  # Diffuse malignant lymphoma - centroblastic polymorphic (disorder)
        "277642008",  # Low grade T-cell lymphoma (disorder)
        "277643003",  # High grade T-cell lymphoma (disorder)
        "277654008",  # Enteropathy-associated T-cell lymphoma (disorder)
        "277664004",  # Malignant lymphoma of testis (disorder)
        "278051002",  # Malignant lymphoma of thyroid gland (disorder)
        "278052009",  # Malignant lymphoma of breast (disorder)
        "278189009",  # Hypergranular promyelocytic leukemia (disorder)
        "278453007",  # Acute biphenotypic leukemia (disorder)
        "285420006",  # Immunoglobulin A myeloma (disorder)
        "285421005",  # Immunoglobulin G myeloma (disorder)
        "285422003",  # Immunoglobulin D myeloma (disorder)
        "285769009",  # Acute promyelocytic leukemia - hypogranular variant (disorder)
        "285776004",  # Intermediate grade B-cell lymphoma (disorder)
        "302841002",  # Malignant lymphoma - small lymphocytic (disorder)
        "302842009",  # Diffuse malignant lymphoma - centroblastic (disorder)
        "302845006",  # Nodular malignant lymphoma, lymphocytic - well differentiated (disorder)
        "302848008",  # Nodular malignant lymphoma, lymphocytic - intermediate differentiation (disorder)
        "302855005",  # Subacute leukemia (disorder)
        "302856006",  # Aleukemic leukemia (disorder)
        "303055001",  # Malignant lymphoma, follicular center cell (disorder)
        "303056000",  # Malignant lymphoma, follicular center cell, cleaved (disorder)
        "303057009",  # Malignant lymphoma, follicular center cell, non-cleaved (disorder)
        "307340003",  # Monosomy 7 syndrome (disorder)
        "307341004",  # Atypical hairy cell leukemia (disorder)
        "307592006",  # Basophilic leukemia (disorder)
        "307617006",  # Neutrophilic leukemia (disorder)
        "307622006",  # Prolymphocytic lymphosarcoma (disorder)
        "307623001",  # Malignant lymphoma - lymphoplasmacytic (disorder)
        "307624007",  # Diffuse malignant lymphoma - centroblastic-centrocytic (disorder)
        "307625008",  # Malignant lymphoma - centrocytic (disorder)
        "307633009",  # Hodgkin's disease, lymphocytic depletion, diffuse fibrosis (disorder)
        "307634003",  # Hodgkin's disease, lymphocytic depletion, reticular type (disorder)
        "307635002",  # Hodgkin's disease, nodular sclerosis - cellular phase (disorder)
        "307636001",  # Malignant lymphoma, mixed lymphocytic-histiocytic, nodular (disorder)
        "307637005",  # Malignant lymphoma, centroblastic-centrocytic, follicular (disorder)
        "307646004",  # Malignant lymphoma, lymphocytic, poorly differentiated, nodular (disorder)
        "307647008",  # Malignant lymphoma, centroblastic type, follicular (disorder)
        "307649006",  # Microglioma (disorder)
        "307650006",  # Histiocytic medullary reticulosis (disorder)
        "308121000",  # Follicular lymphoma (disorder)
        "31047003",  # Lymphomatoid papulosis (disorder)
        "313427003",  # Lambda light chain myeloma (disorder)
        "350951000119101",  # B-cell lymphoma of intra-abdominal lymph nodes (disorder)
        "351211000119104",  # B-cell lymphoma of lymph nodes of multiple sites (disorder)
        "352251000119109",  # Small lymphocytic B-cell lymphoma of lymph nodes of multiple sites (disorder)
        "352411000119109",  # Small lymphocytic B-cell lymphoma of intra-abdominal lymph nodes (disorder)
        "352791000119108",  # Non-Hodgkin's lymphoma of lymph nodes of multiple sites (disorder)
        "353211000119103",  # Extramedullary plasmacytoma in remission (disorder)
        "354851000119101",  # Follicular non-Hodgkin's lymphoma of lymph nodes of multiple sites (disorder)
        "359631009",  # Acute myeloid leukemia, minimal differentiation, FAB M0 (disorder)
        "359640008",  # Acute myeloid leukemia without maturation, FAB M1 (disorder)
        "359648001",  # Acute myeloid leukemia with maturation, FAB M2 (disorder)
        "371012000",  # Acute lymphoblastic leukemia, transitional pre-B-cell (disorder)
        "371134001",  # Malignant lymphoma, large cell, polymorphous, immunoblastic (disorder)
        "373168002",  # Reticulosarcoma (disorder)
        "397008008",  # Aggressive lymphadenopathic mastocytosis with eosinophilia (disorder)
        "397009000",  # Mast cell malignancy (disorder)
        "397011009",  # Mast cell malignancy of lymph nodes (disorder)
        "397015000",  # Systemic mastocytosis with associated clonal hematological non-mast cell lineage disease (disorder)
        "398623004",  # Myelodysplastic syndrome with excess blasts (disorder)
        "400001003",  # Primary cutaneous lymphoma (disorder)
        "400122007",  # Primary cutaneous T-cell lymphoma (disorder)
        "402880009",  # Primary cutaneous large T-cell lymphoma (disorder)
        "402881008",  # Primary cutaneous B-cell lymphoma (disorder)
        "402882001",  # Hodgkin's disease affecting skin (disorder)
        "404103007",  # Lymphomatoid papulosis type A (CD-30 positive type) (disorder)
        "404104001",  # Lymphomatoid papulosis type B - mycosis fungoides-like (disorder)
        "404105000",  # Lymphomatoid papulosis type C (anaplastic large-cell lymphoma-like) (disorder)
        "404106004",  # Lymphomatoid papulosis with Hodgkin's disease (disorder)
        "404107008",  # Patch/plaque stage mycosis fungoides (disorder)
        "404108003",  # Poikilodermatous mycosis fungoides (disorder)
        "404109006",  # Follicular mucinosis type mycosis fungoides (disorder)
        "404110001",  # Hypomelanotic mycosis fungoides (disorder)
        "404111002",  # Lymphomatoid papulosis-associated mycosis fungoides (disorder)
        "404112009",  # Granulomatous mycosis fungoides (disorder)
        "404113004",  # Tumor stage mycosis fungoides (disorder)
        "404114005",  # Erythrodermic mycosis fungoides (disorder)
        "404115006",  # Bullous mycosis fungoides (disorder)
        "404116007",  # Mycosis fungoides with systemic infiltration (disorder)
        "404117003",  # Spongiotic mycosis fungoides (disorder)
        "404118008",  # Syringotropic mycosis fungoides (disorder)
        "404119000",  # Pagetoid reticulosis (disorder)
        "404120006",  # Localized pagetoid reticulosis (disorder)
        "404121005",  # Generalized pagetoid reticulosis (disorder)
        "404122003",  # Leukemic infiltration of skin (chronic T-cell lymphocytic leukemia) (disorder)
        "404123008",  # Leukemic infiltration of skin (T-cell prolymphocytic leukemia) (disorder)
        "404124002",  # Leukemic infiltration of skin (T-cell lymphoblastic leukemia) (disorder)
        "404126000",  # CD-30 positive pleomorphic large T-cell cutaneous lymphoma (disorder)
        "404127009",  # CD-30 positive T-immunoblastic cutaneous lymphoma (disorder)
        "404128004",  # CD-30 negative cutaneous T-cell lymphoma (disorder)
        "404129007",  # CD-30 negative anaplastic large T-cell cutaneous lymphoma (disorder)
        "404130002",  # CD-30 negative pleomorphic large T-cell cutaneous lymphoma (disorder)
        "404131003",  # CD-30 negative T-immunoblastic cutaneous lymphoma (disorder)
        "404133000",  # Subcutaneous panniculitic cutaneous T-cell lymphoma (disorder)
        "404134006",  # Anaplastic large T-cell systemic malignant lymphoma (disorder)
        "404135007",  # Angiocentric natural killer/T-cell malignant lymphoma involving skin (disorder)
        "404136008",  # Aggressive natural killer-cell leukemia involving skin (disorder)
        "404137004",  # Precursor B-cell lymphoblastic lymphoma involving skin (disorder)
        "404138009",  # Small lymphocytic B-cell lymphoma involving skin (disorder)
        "404139001",  # Leukemic infiltration of skin in hairy-cell leukemia (disorder)
        "404140004",  # Primary cutaneous marginal zone B-cell lymphoma (disorder)
        "404141000",  # Primary cutaneous immunocytoma (disorder)
        "404142007",  # Primary cutaneous plasmacytoma (disorder)
        "404143002",  # Primary cutaneous follicular center B-cell lymphoma (disorder)
        "404144008",  # Primary cutaneous diffuse large cell B-cell lymphoma (disorder)
        "404147001",  # Follicular center B-cell lymphoma (nodal/systemic with skin involvement) (disorder)
        "404148006",  # Diffuse large B-cell lymphoma (nodal/systemic with skin involvement) (disorder)
        "404149003",  # Lymphoplasmacytic B-cell lymphoma, nodal/systemic with skin involvement (disorder)
        "404150003",  # Mantle cell B-cell lymphoma (nodal/systemic with skin involvement) (disorder)
        "404151004",  # Leukemic infiltration of skin in myeloid leukemia (disorder)
        "404152006",  # Leukemic infiltration of skin in acute myeloid leukemia (disorder)
        "404153001",  # Leukemic infiltration of skin in chronic myeloid leukemia (disorder)
        "404154007",  # Leukemic infiltration of skin in monocytic leukemia (disorder)
        "404155008",  # Granulocytic sarcoma affecting skin (disorder)
        "404157000",  # Specific skin infiltration in Hodgkin's disease (disorder)
        "404169008",  # Malignant histiocytosis involving skin (disorder)
        "404172001",  # Mast cell leukemia affecting skin (disorder)
        "413389003",  # Accelerated phase chronic myeloid leukemia (disorder)
        "413441006",  # Acute monocytic leukemia (disorder)
        "413537009",  # Angioimmunoblastic T-cell lymphoma (disorder)
        "413587002",  # Smoldering myeloma (disorder)
        "413656006",  # Blastic phase chronic myeloid leukemia (disorder)
        "413842007",  # Chronic myeloid leukemia in lymphoid blast crisis (disorder)
        "413843002",  # Chronic myeloid leukemia in myeloid blast crisis (disorder)
        "413847001",  # Chronic phase chronic myeloid leukemia (disorder)
        "414166008",  # Extranodal natural killer/T-cell lymphoma, nasal type (disorder)
        "414553000",  # Kappa light chain myeloma (disorder)
        "414780005",  # Mucosa-associated lymphoid tissue lymphoma of orbit (disorder)
        "414785000",  # Multiple solitary plasmacytomas (disorder)
        "415110002",  # Plasma cell myeloma/plasmacytoma (disorder)
        "415112005",  # Plasmacytoma (disorder)
        "415283002",  # Myelodysplastic syndrome with excess blasts-1 (disorder)
        "415284008",  # Myelodysplastic syndrome with excess blasts-2 (disorder)
        "415287001",  # Relapsing chronic myeloid leukemia (disorder)
        "420302007",  # Reticulosarcoma with acquired immunodeficiency syndrome (disorder)
        "420519005",  # Malignant lymphoma of the eye region (disorder)
        "420788006",  # Intraocular non-Hodgkin malignant lymphoma (disorder)
        "421246008",  # T-lymphoblastic lymphoma (disorder)
        "421283008",  # Primary lymphoma of brain with acquired immunodeficiency syndrome (disorder)
        "422853008",  # Lymphoma of retroperitoneal space (disorder)
        "425657001",  # Osteosclerotic myeloma (disorder)
        "425688002",  # Philadelphia chromosome-positive acute lymphoblastic leukemia (disorder)
        "425869007",  # Acute promyelocytic leukemia, FAB M3, in remission (disorder)
        "425941003",  # Precursor B-cell acute lymphoblastic leukemia in remission (disorder)
        "426071002",  # Hodgkin's disease in remission (disorder)
        "426124006",  # Acute myeloid leukemia with maturation, FAB M2, in remission (disorder)
        "426217000",  # Aleukemic leukemia in remission (disorder)
        "426248008",  # Aleukemic lymphoid leukemia in remission (disorder)
        "426336007",  # Solitary osseous myeloma (disorder)
        "426370008",  # Subacute lymphoid leukemia in remission (disorder)
        "426642002",  # Erythroleukemia, FAB M6 in remission (disorder)
        "426885008",  # Hodgkin's disease, lymphocytic depletion of lymph nodes of head (disorder)
        "427056005",  # Subacute leukemia in remission (disorder)
        "427141003",  # Malignant lymphoma in remission (disorder)
        "427642009",  # T-lymphoblastic leukemia in remission (disorder)
        "427658007",  # Acute myelomonocytic leukemia, FAB M4, in remission (disorder)
        "430338009",  # Smoldering chronic lymphocytic leukemia (disorder)
        "440422002",  # Asymptomatic multiple myeloma (disorder)
        "441313008",  # Indolent multiple myeloma (disorder)
        "441559006",  # Mantle cell lymphoma of spleen (disorder)
        "441962003",  # Large cell lymphoma of intrapelvic lymph nodes (disorder)
        "442537007",  # Non-Hodgkin lymphoma associated with Human immunodeficiency virus infection (disorder)
        "443487006",  # Mantle cell lymphoma (disorder)
        "444597005",  # Extranodal marginal zone lymphoma of mucosa-associated lymphoid tissue of stomach (disorder)
        "444910004",  # Primary mediastinal (thymic) large B-cell lymphoma (disorder)
        "444911000",  # Acute myeloid leukemia with t(9:11)(p22;q23); MLLT3-MLL (disorder)
        "445105005",  # Blastic plasmacytoid dendritic cell neoplasm (disorder)
        "445227008",  # Juvenile myelomonocytic leukemia (disorder)
        "445269007",  # Extranodal marginal zone B-cell lymphoma of mucosa-associated lymphoid tissue (disorder)
        "445406001",  # Hepatosplenic T-cell lymphoma (disorder)
        "445448008",  # Acute myeloid leukemia with myelodysplasia-related changes (disorder)
        "445738007",  # Myelodysplastic/myeloproliferative disease (disorder)
        "447100004",  # Marginal zone lymphoma (disorder)
        "447596005",  # Myelodysplastic/myeloproliferative neoplasm, unclassifiable (disorder)
        "447656001",  # Lymphoma of pylorus of stomach (disorder)
        "447658000",  # Lymphoma of fundus of stomach (disorder)
        "447766003",  # Lymphoma of pyloric antrum of stomach (disorder)
        "447805007",  # Lymphoma of greater curvature of stomach (disorder)
        "447806008",  # Lymphoma of cardia of stomach (disorder)
        "447989004",  # Non-Hodgkin's lymphoma of extranodal site (disorder)
        "448212009",  # Anaplastic lymphoma kinase negative anaplastic large cell lymphoma (disorder)
        "448213004",  # Diffuse non-Hodgkin's lymphoma of prostate (disorder)
        "448217003",  # Follicular non-Hodgkin's lymphoma of prostate (disorder)
        "448220006",  # Non-Hodgkin's lymphoma of bone (disorder)
        "448231003",  # Follicular non-Hodgkin's lymphoma of nose (disorder)
        "448254007",  # Non-Hodgkin's lymphoma of central nervous system (disorder)
        "448269008",  # Lymphoma of lesser curvature of stomach (disorder)
        "448317000",  # Follicular non-Hodgkin's lymphoma of soft tissue (disorder)
        "448319002",  # Diffuse non-Hodgkin's lymphoma of nasopharynx (disorder)
        "448354009",  # Non-Hodgkin's lymphoma of intestine (disorder)
        "448371005",  # Non-Hodgkin's lymphoma of nasopharynx (disorder)
        "448372003",  # Non-Hodgkin's lymphoma of lung (disorder)
        "448376000",  # Non-Hodgkin's lymphoma of ovary (disorder)
        "448384001",  # Non-Hodgkin's lymphoma of nose (disorder)
        "448386004",  # Non-Hodgkin's lymphoma of oral cavity (disorder)
        "448387008",  # Non-Hodgkin's lymphoma of testis (disorder)
        "448447004",  # Non-Hodgkin's lymphoma of skin (disorder)
        "448465000",  # Diffuse non-Hodgkin's lymphoma of testis (disorder)
        "448468003",  # Diffuse non-Hodgkin's lymphoma of oral cavity (disorder)
        "448553002",  # Lymphoma of pelvis (disorder)
        "448555009",  # Lymphoma of body of stomach (disorder)
        "448560008",  # Diffuse non-Hodgkin's lymphoma of extranodal site (disorder)
        "448561007",  # Follicular non-Hodgkin's lymphoma of extranodal site (disorder)
        "448607004",  # Diffuse non-Hodgkin's lymphoma of uterine cervix (disorder)
        "448609001",  # Diffuse non-Hodgkin's lymphoma of ovary (disorder)
        "448663003",  # Diffuse non-Hodgkin's lymphoma of stomach (disorder)
        "448666006",  # Follicular non-Hodgkin's lymphoma of bone (disorder)
        "448672006",  # Follicular non-Hodgkin's lymphoma of lung (disorder)
        "448709005",  # Non-Hodgkin's lymphoma of stomach (disorder)
        "448738008",  # Non-Hodgkin's lymphoma of soft tissue (disorder)
        "448774004",  # Non-Hodgkin's lymphoma of uterine cervix (disorder)
        "448865007",  # Follicular non-Hodgkin's lymphoma of skin (disorder)
        "448867004",  # Diffuse non-Hodgkin's lymphoma of lung (disorder)
        "448995000",  # Follicular non-Hodgkin's lymphoma of central nervous system (disorder)
        "449053004",  # Lymphoma of lower esophagus (disorder)
        "449058008",  # Follicular non-Hodgkin's lymphoma of tonsil (disorder)
        "449059000",  # Follicular non-Hodgkin's lymphoma of uterine cervix (disorder)
        "449063007",  # Follicular non-Hodgkin's lymphoma of oral cavity (disorder)
        "449065000",  # Diffuse non-Hodgkin's lymphoma of nose (disorder)
        "449072004",  # Lymphoma of gastrointestinal tract (disorder)
        "449074003",  # Lymphoma of small intestine (disorder)
        "449075002",  # Lymphoma of cardioesophageal junction (disorder)
        "449173006",  # Diffuse non-Hodgkin's lymphoma of tonsil (disorder)
        "449176003",  # Diffuse non-Hodgkin's lymphoma of intestine (disorder)
        "449177007",  # Diffuse non-Hodgkin's lymphoma of bone (disorder)
        "449216004",  # Diffuse non-Hodgkin's lymphoma of soft tissue (disorder)
        "449217008",  # Diffuse non-Hodgkin's lymphoma of skin (disorder)
        "449218003",  # Lymphoma of sigmoid colon (disorder)
        "449219006",  # Follicular non-Hodgkin's lymphoma of nasopharynx (disorder)
        "449220000",  # Diffuse follicle center lymphoma (disorder)
        "449221001",  # Diffuse non-Hodgkin's lymphoma of central nervous system (disorder)
        "449222008",  # Follicular non-Hodgkin's lymphoma of stomach (disorder)
        "449292003",  # Non-Hodgkin's lymphoma of tonsil (disorder)
        "449307001",  # Follicular non-Hodgkin's lymphoma of ovary (disorder)
        "449318001",  # Non-Hodgkin's lymphoma of prostate (disorder)
        "449418000",  # Follicular non-Hodgkin's lymphoma of testis (disorder)
        "449419008",  # Follicular non-Hodgkin's lymphoma of intestine (disorder)
        "450521000124109",  # Marginal zone lymphoma of axillary lymph node (disorder)
        "452291000124109",  # Relapse multiple myeloma (disorder)
        "456911000124108",  # Diffuse non-Hodgkin's lymphoma Lugano stage III (disorder)
        "461311000124107",  # Refractory Hodgkin lymphoma (disorder)
        "58961005",  # Lethal midline granuloma (disorder)
        "61291000119103",  # Disorder of central nervous system co-occurrent and due to acute lymphoid leukemia in remission (disorder)
        "61301000119102",  # Disorder of central nervous system co-occurrent and due to acute lymphoid leukemia (disorder)
        "61311000119104",  # Acute myeloid leukemia, minimal differentiation, FAB M0 in remission (disorder)
        "61321000119106",  # Acute myeloid leukemia without maturation, FAB M1 in remission (disorder)
        "68261000119102",  # Diffuse non-Hodgkin's lymphoma Lugano stage I (disorder)
        "68271000119108",  # Diffuse non-Hodgkin's lymphoma Lugano stage II (disorder)
        "68291000119109",  # Diffuse non-Hodgkin's lymphoma Lugano stage IV (disorder)
        "686981000119108",  # Mycosis fungoides of inguinal lymph node (disorder)
        "686991000119106",  # Mycosis fungoides of lower limb lymph node (disorder)
        "699537002",  # Erdheim-Chester disease (disorder)
        "699657009",  # Hepatosplenic gamma-delta cell lymphoma (disorder)
        "699818003",  # T-cell large granular lymphocytic leukemia (disorder)
        "700057001",  # Emberger syndrome (disorder)
        "702446006",  # Core binding factor acute myeloid leukemia (disorder)
        "702476004",  # Therapy-related myelodysplastic syndrome (disorder)
        "702785000",  # Large cell anaplastic lymphoma T cell and Null cell type (disorder)
        "702786004",  # Follicular non-Hodgkin's lymphoma diffuse follicle center sub-type grade 1 (disorder)
        "702977001",  # Follicular non-Hodgkin's lymphoma diffuse follicle center cell sub-type grade 2 (disorder)
        "703387000",  # Acute myeloid leukemia with normal karyotype (disorder)
        "703626001",  # Anaplastic large cell lymphoma, T/Null cell, primary systemic type (disorder)
        "705061009",  # Childhood myelodysplastic syndrome (disorder)
        "709471005",  # Periodontitis co-occurrent with leukemia (disorder)
        "713325002",  # Primary cerebral lymphoma co-occurrent with human immunodeficiency virus infection (disorder)
        "713483007",  # Reticulosarcoma co-occurrent with human immunodeficiency virus infection (disorder)
        "713516007",  # Primary effusion lymphoma (disorder)
        "713718006",  # Diffuse non-Hodgkin immunoblastic lymphoma co-occurrent with human immunodeficiency virus infection (disorder)
        "713897006",  # Burkitt lymphoma co-occurrent with human immunodeficiency virus infection (disorder)
        "714251006",  # Philadelphia chromosome-negative precursor B-cell acute lymphoblastic leukemia (disorder)
        "714463003",  # Primary effusion lymphoma co-occurrent with infection caused by Human herpesvirus 8 (disorder)
        "715664005",  # Interdigitating dendritic cell sarcoma (disorder)
        "715950008",  # Anaplastic lymphoma kinase positive large B-cell lymphoma (disorder)
        "716788007",  # Epstein-Barr virus positive diffuse large B-cell lymphoma of elderly (disorder)
        "718200007",  # Primary pulmonary lymphoma (disorder)
        "721301004",  # Myeloid and lymphoid neoplasm with fibroblast growth factor receptor 1 abnormality (disorder)
        "721304007",  # Refractory thrombocytopenia (disorder)
        "721306009",  # Therapy related acute myeloid leukemia and myelodysplastic syndrome (disorder)
        "721308005",  # Acute leukemia of ambiguous lineage (disorder)
        "721310007",  # Aggressive natural killer-cell leukemia (disorder)
        "721313009",  # Indeterminate dendritic cell neoplasm (disorder)
        "721314003",  # Fibroblastic reticular cell neoplasm (disorder)
        "721555001",  # Follicular lymphoma of small intestine (disorder)
        "721762007",  # Adult T-cell leukemia/lymphoma of skin (disorder)
        "722795004",  # Meningeal leukemia (disorder)
        "722954005",  # B-cell lymphoma unclassifiable with features intermediate between classical Hodgkin lymphoma and diffuse large B-cell lymphoma (disorder)
        "723889003",  # B lymphoblastic leukemia lymphoma with t(9:22) (q34;q11.2); BCR-ABL 1 (disorder)
        "724642009",  # Myeloid neoplasm associated with beta-type platelet-derived growth factor receptor gene rearrangement (disorder)
        "724644005",  # Myeloid leukemia associated with Down syndrome (disorder)
        "724645006",  # T-cell histiocyte rich large B-cell lymphoma (disorder)
        "724647003",  # Diffuse large B-cell lymphoma co-occurrent with chronic inflammation caused by Epstein-Barr virus (disorder)
        "724648008",  # Plasmablastic lymphoma (disorder)
        "724649000",  # Langerhans cell sarcoma (disorder)
        "724650000",  # Primary follicular dendritic cell sarcoma (disorder)
        "725390002",  # Acute myeloid leukemia with t(8;16)(p11;p13) translocation (disorder)
        "725437002",  # Chronic lymphocytic leukemia genetic mutation variant (disorder)
        "726721002",  # Nodal marginal zone B-cell lymphoma (disorder)
        "733598001",  # Acute myeloid leukemia with t(6;9)(p23;q34) translocation (disorder)
        "733627006",  # Primary cutaneous gamma-delta-positive T-cell lymphoma (disorder)
        "734066005",  # Diffuse large B-cell lymphoma of central nervous system (disorder)
        "735332000",  # Primary cutaneous diffuse large cell B-cell lymphoma of lower extremity (disorder)
        "736322001",  # Pediatric follicular lymphoma (disorder)
        "738527001",  # Myeloid and/or lymphoid neoplasm associated with platelet derived growth factor receptor alpha rearrangement (disorder)
        "739301006",  # Osteoporosis co-occurrent and due to multiple myeloma (disorder)
        "762315004",  # Therapy related acute myeloid leukemia due to and following administration of antineoplastic agent (disorder)
        "762690000",  # Classical Hodgkin lymphoma (disorder)
        "763309005",  # Acute myeloid leukemia with nucleophosmin 1 somatic mutation (disorder)
        "763477007",  # Primary lymphoma of conjunctiva (disorder)
        "763666008",  # Splenic marginal zone B-cell lymphoma (disorder)
        "763719001",  # Hydroa vacciniforme-like lymphoma (disorder)
        "763796007",  # Megakaryoblastic acute myeloid leukemia with t(1;22)(p13;q13) (disorder)
        "763884007",  # Splenic diffuse red pulp small B-cell lymphoma (disorder)
        "764855007",  # Acute myeloid leukemia with CCAAT/enhancer binding protein alpha somatic mutation (disorder)
        "764940002",  # Inherited acute myeloid leukemia (disorder)
        "765136002",  # Primary cutaneous CD8 positive aggressive epidermotropic cytotoxic T-cell lymphoma (disorder)
        "765328000",  # Classic mycosis fungoides (disorder)
        "766045006",  # Acute myeloid leukemia and myelodysplastic syndrome related to alkylating agent (disorder)
        "766046007",  # Acute myeloid leukemia and myelodysplastic syndrome related to topoisomerase type 2 inhibitor (disorder)
        "766048008",  # Acute myeloid leukemia and myelodysplastic syndrome related to radiation (disorder)
        "766935007",  # Primary bone lymphoma (disorder)
        "773537001",  # Differentiation syndrome due to and following chemotherapy co-occurrent with acute promyelocytic leukemia (disorder)
        "773995001",  # Primary cutaneous anaplastic large cell lymphoma (disorder)
        "780817000",  # Undifferentiated myeloproliferative disease (disorder)
        "780844005",  # Acute myeloid leukemia with inv(3)(q21q26.2) or t(3;3)(q21;q26.2); RPN1-EVI1 (disorder)
        "783541009",  # Breast implant-associated anaplastic large-cell lymphoma (disorder)
        "783565007",  # Indolent T-cell lymphoproliferative disorder of gastrointestinal tract (disorder)
        "783615009",  # Erythropoietic uroporphyria associated with myeloid malignancy (disorder)
        "788874003",  # B-cell prolymphocytic leukemia in remission (disorder)
        "788972003",  # Juvenile myelomonocytic leukemia in remission (disorder)
        "830057003",  # Relapsing classical Hodgkin lymphoma (disorder)
        "836486002",  # Lymphomatous infiltrate of kidney (disorder)
        "838340006",  # B lymphoblastic leukemia lymphoma with t(5;14)(q31;q32); IL3-IGH (disorder)
        "838341005",  # B lymphoblastic leukemia lymphoma with t(v;11q23); MLL rearranged (disorder)
        "838342003",  # B lymphoblastic leukemia lymphoma with t(12;21) (p13;q22); TEL/AML1 (ETV6-RUNX1) (disorder)
        "838343008",  # B lymphoblastic leukemia lymphoma with t(1;19)(Q23;P13.3); E2A-PBX1 (TCF3/PBX1) (disorder)
        "838344002",  # B lymphoblastic leukemia lymphoma with hypodiploidy (disorder)
        "838346000",  # B lymphoblastic leukemia lymphoma with hyperdiploidy (disorder)
        "838355002",  # Acute myeloid leukemia with CBFB::MYH11 fusion (disorder)
        "840423002",  # Diffuse large B-cell lymphoma of small intestine (disorder)
        "840424008",  # Diffuse large B-cell lymphoma of stomach (disorder)
        "84811000119107",  # Follicular dendritic sarcoma of intraabdominal lymph node (disorder)
        "84821000119100",  # Primary follicular dendritic cell sarcoma of spleen (disorder)
        "84831000119102",  # Follicular dendritic sarcoma of lymph nodes of multiple sites (disorder)
        "866098005",  # Large B-cell lymphoma arising in HHV8-associated multicentric Castleman disease (disorder)
        "898197001",  # Adult T-cell leukemia/lymphoma in remission (disorder)
        "898198006",  # T-cell prolymphocytic leukemia in remission (disorder)
        "91854005",  # Acute leukemia in remission (disorder)
        "91855006",  # Acute leukemia, disease (disorder)
        "91856007",  # Acute lymphoid leukemia in remission (disorder)
        "91857003",  # Acute lymphoid leukemia, disease (disorder)
        "91858008",  # Acute monocytic leukemia in remission (disorder)
        "91860005",  # Acute myeloid leukemia in remission (disorder)
        "91861009",  # Acute myeloid leukemia (disorder)
        "92811003",  # Chronic leukemia in remission (disorder)
        "92812005",  # Chronic leukemia, disease (disorder)
        "92813000",  # Chronic lymphoid leukemia in remission (disorder)
        "92814006",  # Chronic lymphoid leukemia, disease (disorder)
        "92817004",  # Chronic myeloid leukemia in remission (disorder)
        "92818009",  # Chronic myeloid leukemia (disorder)
        "93142004",  # Leukemia in remission (disorder)
        "93143009",  # Leukemia, disease (disorder)
        "93144003",  # Leukemic reticuloendotheliosis of intra-abdominal lymph nodes (disorder)
        "93145002",  # Leukemic reticuloendotheliosis of intrapelvic lymph nodes (disorder)
        "93146001",  # Leukemic reticuloendotheliosis of intrathoracic lymph nodes (disorder)
        "93150008",  # Leukemic reticuloendotheliosis of lymph nodes of multiple sites (disorder)
        "93151007",  # Hairy cell leukemia of spleen (disorder)
        "93169003",  # Lymphoid leukemia in remission (disorder)
        "93182006",  # Malignant histiocytosis of intra-abdominal lymph nodes (disorder)
        "93183001",  # Malignant histiocytosis of intrapelvic lymph nodes (disorder)
        "93184007",  # Malignant histiocytosis of intrathoracic lymph nodes (disorder)
        "93188005",  # Malignant histiocytosis of lymph nodes of multiple sites (disorder)
        "93189002",  # Malignant histiocytosis of spleen (disorder)
        "93191005",  # Malignant lymphoma of intra-abdominal lymph nodes (disorder)
        "93192003",  # Malignant lymphoma of intrapelvic lymph nodes (disorder)
        "93193008",  # Malignant lymphoma of intrathoracic lymph nodes (disorder)
        "93197009",  # Malignant lymphoma of lymph nodes of multiple sites (disorder)
        "93198004",  # Malignant lymphoma of spleen (disorder)
        "93199007",  # Malignant lymphoma of extranodal AND/OR solid organ site (disorder)
        "93200005",  # Malignant mast cell tumor of intra-abdominal lymph nodes (disorder)
        "93201009",  # Malignant mast cell tumor of intrapelvic lymph nodes (disorder)
        "93202002",  # Malignant mast cell tumor of intrathoracic lymph nodes (disorder)
        "93451002",  # Erythroleukemia, FAB M6 (disorder)
        "93493001",  # Hodgkin's disease, lymphocytic-histiocytic predominance of intra-abdominal lymph nodes (disorder)
        "93494007",  # Hodgkin's disease, lymphocytic-histiocytic predominance of intrapelvic lymph nodes (disorder)
        "93495008",  # Hodgkin's disease, lymphocytic-histiocytic predominance of intrathoracic lymph nodes (disorder)
        "93500006",  # Hodgkin's disease, lymphocytic-histiocytic predominance of spleen (disorder)
        "93509007",  # Hodgkin's disease, mixed cellularity of spleen (disorder)
        "93518009",  # Hodgkin's disease, nodular sclerosis of spleen (disorder)
        "93520007",  # Hodgkin's disease of intra-abdominal lymph nodes (disorder)
        "93521006",  # Hodgkin's disease of intrapelvic lymph nodes (disorder)
        "93522004",  # Hodgkin's disease of intrathoracic lymph nodes (disorder)
        "93526001",  # Hodgkin's disease of lymph nodes of multiple sites (disorder)
        "93527005",  # Hodgkin's disease of spleen (disorder)
        "93530003",  # Hodgkin's granuloma of intrapelvic lymph nodes (disorder)
        "93531004",  # Hodgkin's granuloma of intrathoracic lymph nodes (disorder)
        "93536009",  # Hodgkin's granuloma of spleen (disorder)
        "93547002",  # Hodgkin's sarcoma of intra-abdominal lymph nodes (disorder)
        "93548007",  # Hodgkin's sarcoma of intrapelvic lymph nodes (disorder)
        "93549004",  # Hodgkin's sarcoma of intrathoracic lymph nodes (disorder)
        "93554008",  # Hodgkin's sarcoma of spleen (disorder)
        "94071006",  # Primary malignant neoplasm of spleen (disorder)
        "94148006",  # Megakaryocytic leukemia in remission (disorder)
        "94686001",  # Mixed cell type lymphosarcoma of intra-abdominal lymph nodes (disorder)
        "94687005",  # Mixed cell type lymphosarcoma of intrapelvic lymph nodes (disorder)
        "94688000",  # Mixed cell type lymphosarcoma of intrathoracic lymph nodes (disorder)
        "94690004",  # Mixed cell type lymphosarcoma of lymph nodes of head, face, and neck (disorder)
        "94704006",  # Multiple myeloma in remission (disorder)
        "94707004",  # Mycosis fungoides of intra-abdominal lymph nodes (disorder)
        "94708009",  # Mycosis fungoides of intrapelvic lymph nodes (disorder)
        "94709001",  # Mycosis fungoides of intrathoracic lymph nodes (disorder)
        "94714002",  # Mycosis fungoides of spleen (disorder)
        "94716000",  # Myeloid leukemia in remission (disorder)
        "94718004",  # Myeloid sarcoma in remission (disorder)
        "94719007",  # Myeloid sarcoma, disease (disorder)
        "95186006",  # Nodular lymphoma of intra-abdominal lymph nodes (disorder)
        "95187002",  # Nodular lymphoma of intrapelvic lymph nodes (disorder)
        "95188007",  # Nodular lymphoma of intrathoracic lymph nodes (disorder)
        "95192000",  # Nodular lymphoma of lymph nodes of multiple sites (disorder)
        "95193005",  # Nodular lymphoma of spleen (disorder)
        "95209008",  # Plasma cell leukemia in remission (disorder)
        "95210003",  # Plasma cell leukemia, disease (disorder)
        "95224004",  # Reticulosarcoma of intra-abdominal lymph nodes (disorder)
        "95225003",  # Reticulosarcoma of intrapelvic lymph nodes (disorder)
        "95226002",  # Reticulosarcoma of intrathoracic lymph nodes (disorder)
        "95230004",  # Reticulosarcoma of lymph nodes of multiple sites (disorder)
        "95231000",  # Reticulosarcoma of spleen (disorder)
        "95263006",  # Sezary's disease of spleen (disorder)
    }

class Measles(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of measles infections.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of measles infections.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Measles"
    OID = "2.16.840.1.113883.3.464.1003.110.12.1053"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "B050",  # Measles complicated by encephalitis
        "B051",  # Measles complicated by meningitis
        "B052",  # Measles complicated by pneumonia
        "B053",  # Measles complicated by otitis media
        "B054",  # Measles with intestinal complications
        "B0581",  # Measles keratitis and keratoconjunctivitis
        "B0589",  # Other measles complications
        "B059",  # Measles without complication
    }

    SNOMEDCT = {
        "111873003",  # Measles without complication (disorder)
        "105841000119101",  # Immunity to measles determined by serology (finding)
        "14189004",  # Measles (disorder)
        "161419000",  # History of measles (situation)
        "186561002",  # Meningitis caused by measles morbillivirus (disorder)
        "186562009",  # Disorder of intestine due to measles (disorder)
        "195900001",  # Pneumonia due to measles (disorder)
        "240483006",  # Atypical measles (disorder)
        "240484000",  # Modified measles (disorder)
        "28463004",  # Measles keratitis (disorder)
        "359686005",  # Van Bogaert's sclerosing leukoencephalitis (disorder)
        "371111005",  # Measles immune (finding)
        "38921001",  # Measles with complication (disorder)
        "406592004",  # Measles of the central nervous system (disorder)
        "417145006",  # Rubeola retinopathy (disorder)
        "424306000",  # Exanthem caused by measles virus (disorder)
        "60013002",  # Measles keratoconjunctivitis (disorder)
        "74918002",  # Measles myocarditis (disorder)
    }

class Mumps(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of mumps infections.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of mumps infections.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Mumps"
    OID = "2.16.840.1.113883.3.464.1003.110.12.1032"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "B260",  # Mumps orchitis
        "B261",  # Mumps meningitis
        "B262",  # Mumps encephalitis
        "B263",  # Mumps pancreatitis
        "B2681",  # Mumps hepatitis
        "B2682",  # Mumps myocarditis
        "B2683",  # Mumps nephritis
        "B2684",  # Mumps polyneuropathy
        "B2685",  # Mumps arthritis
        "B2689",  # Other mumps complications
        "B269",  # Mumps without complication
    }

    SNOMEDCT = {
        "74717002",  # Mumps with complication (disorder)
        "105821000119107",  # Immunity to mumps determined by serology (finding)
        "10665004",  # Mumps pancreatitis (disorder)
        "111870000",  # Mumps without complication (disorder)
        "1163539003",  # Dacryoadenitis caused by mumps virus (disorder)
        "161420006",  # History of mumps (situation)
        "17121006",  # Mumps nephritis (disorder)
        "235123001",  # Mumps acute sialadenitis (disorder)
        "236771002",  # Mumps epididymo-orchitis (disorder)
        "237443002",  # Mastitis of mumps (disorder)
        "240526004",  # Mumps parotitis (disorder)
        "240527008",  # Mumps submandibular sialadenitis (disorder)
        "240529006",  # Mumps sublingual sialadenitis (disorder)
        "31524007",  # Mumps polyneuropathy (disorder)
        "31646008",  # Mumps encephalitis (disorder)
        "36989005",  # Mumps (disorder)
        "371112003",  # Mumps immune (finding)
        "40099009",  # Meningoencephalitis caused by mumps virus (disorder)
        "44201003",  # Meningitis caused by mumps virus (disorder)
        "63462008",  # Mumps myocarditis (disorder)
        "72071001",  # Mumps thyroiditis (disorder)
        "75548002",  # Mumps oophoritis (disorder)
        "78580004",  # Mumps orchitis (disorder)
        "89231008",  # Mumps hepatitis (disorder)
        "89764009",  # Arthritis due to mumps (disorder)
    }

class Rubella(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of rubella infections.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of rubella infections.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Rubella"
    OID = "2.16.840.1.113883.3.464.1003.110.12.1037"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "B0600",  # Rubella with neurological complication, unspecified
        "B0601",  # Rubella encephalitis
        "B0602",  # Rubella meningitis
        "B0609",  # Other neurological complications of rubella
        "B0681",  # Rubella pneumonia
        "B0682",  # Rubella arthritis
        "B0689",  # Other rubella complications
        "B069",  # Rubella without complication
    }

    SNOMEDCT = {
        "165792000",  # Rubella antibody present - immune (finding)
        "406112006",  # Rubella enzyme-linked immunosorbent assay test result, greater than 10iu/ml rubella specific immunoglobulin G detected (finding)
        "406113001",  # Rubella enzyme-linked immunosorbent assay test result, greater than 15iu/ml rubella specific immunoglobulin G detected (finding)
        "10082001",  # Progressive rubella panencephalitis (disorder)
        "10759761000119100",  # Rubella in mother complicating childbirth (disorder)
        "1092361000119109",  # Rubella pneumonia (disorder)
        "128191000",  # Encephalomyelitis due to rubella (disorder)
        "13225007",  # Meningoencephalitis caused by Rubella virus (disorder)
        "161421005",  # History of rubella (situation)
        "186567003",  # Rubella with neurological complication (disorder)
        "186570004",  # Rubella deafness (disorder)
        "192689006",  # Rubella encephalitis (disorder)
        "19431000",  # Arthritis due to rubella (disorder)
        "231985001",  # Rubella retinopathy (disorder)
        "232312000",  # Endocochlear rubella (disorder)
        "240485004",  # Hemorrhagic rubella (disorder)
        "253227001",  # Rubella cataract (disorder)
        "36653000",  # Rubella (disorder)
        "51490003",  # Disease caused by Rubivirus (disorder)
        "64190005",  # Rubella myocarditis (disorder)
        "79303006",  # Expanded rubella syndrome (disorder)
    }

class SevereCombinedImmunodeficiency(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of severe combined immunodeficiency including both genetic and other causes.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of a severe combined immunodeficiency including genetic conditions and other causes.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Severe Combined Immunodeficiency"
    OID = "2.16.840.1.113883.3.464.1003.120.12.1007"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "D810",  # Severe combined immunodeficiency [SCID] with reticular dysgenesis
        "D811",  # Severe combined immunodeficiency [SCID] with low T- and B-cell numbers
        "D812",  # Severe combined immunodeficiency [SCID] with low or normal B-cell numbers
        "D819",  # Combined immunodeficiency, unspecified
    }

    SNOMEDCT = {
        "190993005",  # Autosomal recessive severe combined immunodeficiency (disorder)
        "111584000",  # Reticular dysgenesis (disorder)
        "111587007",  # Severe combined immunodeficiency due to absent interleukin-2 receptor (disorder)
        "1179284005",  # Severe combined immunodeficiency due to linker for activation of T cells deficiency (disorder)
        "1229940001",  # Autosomal recessive T-cell negative, B-cell positive severe combined immunodeficiency due to interleukin 7 receptor deficiency (disorder)
        "1229941002",  # Autosomal recessive T-cell negative, B-cell positive severe combined immunodeficiency due to protein tyrosine phosphatase receptor type C deficiency (disorder)
        "1229942009",  # Severe combined immunodeficiency due to coronin 1A deficiency (disorder)
        "190996002",  # Severe combined immunodeficiency with reticular dysgenesis (disorder)
        "190997006",  # Severe combined immunodeficiency with low T- and B-cell numbers (disorder)
        "190998001",  # Severe combined immunodeficiency with low or normal B-cell numbers (disorder)
        "191001007",  # Major histocompatibility complex class I deficiency (disorder)
        "191002000",  # Major histocompatibility complex class II deficiency (disorder)
        "22406001",  # Severe combined immunodeficiency due to absent lymphoid stem cells (disorder)
        "234570002",  # Severe combined immunodeficiency with maternofetal engraftment (disorder)
        "234571003",  # Warts, hypogammaglobulinemia, infections, and myelokathexis (disorder)
        "31323000",  # Severe combined immunodeficiency disease (disorder)
        "3439009",  # Severe combined immunodeficiency due to absent peripheral T cell maturation (disorder)
        "350353007",  # De Vaal's syndrome (disorder)
        "351287008",  # Reticular dysgenesis with congenital aleukocytosis (disorder)
        "362993009",  # Autosomal recessive severe combined immunodeficiency disease (disorder)
        "36980009",  # Severe combined immunodeficiency due to absent adenosine deaminase (disorder)
        "44940001",  # Adenosine deaminase deficiency (disorder)
        "45390000",  # Severe combined immunodeficiency due to absent interleukin-2 production (disorder)
        "49555001",  # Severe combined immunodeficiency due to absent T cell receptor (disorder)
        "55602000",  # Nezelof's syndrome (disorder)
        "715982006",  # Severe combined immunodeficiency due to deoxyribonucleic acid cross-link repair protein 1c deficiency (disorder)
        "716378008",  # Combined immunodeficiency due to Zeta-chain associated protein kinase 70 deficiency (disorder)
        "716871006",  # Severe combined immunodeficiency due to deoxyribonucleic acid dependent protein kinase catalytic subunit deficiency (disorder)
        "718107000",  # Severe combined immunodeficiency T-cell negative B-cell positive due to janus kinase-3 deficiency (disorder)
        "71904008",  # Severe combined immunodeficiency due to absent class II human leukocyte antigens (disorder)
        "720345008",  # Severe T-cell immunodeficiency, congenital alopecia, nail dystrophy syndrome (disorder)
        "720853005",  # Severe combined immunodeficiency, microcephaly, growth retardation, sensitivity to ionizing radiation syndrome (disorder)
        "720986005",  # Anhidrotic ectodermal dysplasia, immunodeficiency, osteopetrosis, lymphedema syndrome (disorder)
        "721977007",  # Lung fibrosis, immunodeficiency, 46,XX gonadal dysgenesis syndrome (disorder)
        "722067005",  # Severe combined immunodeficiency with hypereosinophilia (disorder)
        "724177005",  # Ligase 4 syndrome (disorder)
        "724361001",  # Hepatic veno-occlusive disease with immunodeficiency syndrome (disorder)
        "725135004",  # Combined immunodeficiency due to CD3gamma deficiency (disorder)
        "725136003",  # Immunodeficiency by defective expression of human leukocyte antigen class 1 (disorder)
        "725290000",  # Combined immunodeficiency due to partial recombination-activating gene 1 deficiency (disorder)
        "763623001",  # Severe combined immunodeficiency due to cytidine 5-prime triphosphate synthetase 1 deficiency (disorder)
        "765145001",  # T cell negative B cell positive severe combined immunodeficiency due to gamma chain deficiency (disorder)
        "765188009",  # Severe combined immunodeficiency due to complete recombination-activating gene 1 and/or recombination-activating gene 2 deficiency (disorder)
        "771517009",  # Severe combined immunodeficiency due to caspase recruitment domain family member 11 deficiency (disorder)
        "782751003",  # Severe combined immunodeficiency due to inhibitor of nuclear factor kappa B kinase subunit beta deficiency (disorder)
        "783617001",  # Severe combined immunodeficiency due to lymphocyte-specific protein-tyrosine kinase deficiency (disorder)
        "789777007",  # Short-limb skeletal dysplasia with severe combined immunodeficiency (disorder)
        "987840791000119102",  # Adenosine deaminase 2 deficiency (disorder)
    }

class VaricellaZoster(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of varicella zoster infections.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of varicella zoster infections.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Varicella Zoster"
    OID = "2.16.840.1.113883.3.464.1003.110.12.1039"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "B010",  # Varicella meningitis
        "B0111",  # Varicella encephalitis and encephalomyelitis
        "B0112",  # Varicella myelitis
        "B012",  # Varicella pneumonia
        "B0181",  # Varicella keratitis
        "B0189",  # Other varicella complications
        "B019",  # Varicella without complication
        "B020",  # Zoster encephalitis
        "B021",  # Zoster meningitis
        "B0221",  # Postherpetic geniculate ganglionitis
        "B0222",  # Postherpetic trigeminal neuralgia
        "B0223",  # Postherpetic polyneuropathy
        "B0224",  # Postherpetic myelitis
        "B0229",  # Other postherpetic nervous system involvement
        "B0230",  # Zoster ocular disease, unspecified
        "B0231",  # Zoster conjunctivitis
        "B0232",  # Zoster iridocyclitis
        "B0233",  # Zoster keratitis
        "B0234",  # Zoster scleritis
        "B0239",  # Other herpes zoster eye disease
        "B027",  # Disseminated zoster
        "B028",  # Zoster with other complications
        "B029",  # Zoster without complications
    }

    SNOMEDCT = {
        "186524006",  # Herpes zoster with ophthalmic complication (disorder)
        "49183009",  # Herpes zoster with complication (disorder)
        "723109003",  # Herpes zoster of skin and/or mucous membranes without complications (disorder)
        "10698009",  # Herpes zoster iridocyclitis (disorder)
        "1087131000119102",  # Scleritis due to herpes zoster (disorder)
        "111859007",  # Herpes zoster without complication (disorder)
        "111861003",  # Herpes zoster with nervous system complication (disorder)
        "1163465001",  # Dacryoadenitis due to herpes zoster (disorder)
        "1163483009",  # Infection of eyelid due to herpes zoster (disorder)
        "1179456002",  # Esophagitis caused by Human alphaherpesvirus 3 (disorder)
        "12551000132107",  # Cranial neuropathy due to Herpes zoster (disorder)
        "12561000132105",  # Bacterial infection co-occurrent and due to herpes zoster (disorder)
        "12571000132104",  # Pneumonia caused by Herpes zoster (disorder)
        "15678761000119105",  # Iridocyclitis of right eye due to herpes zoster (disorder)
        "15678801000119102",  # Iridocyclitis of left eye due to herpes zoster (disorder)
        "15678841000119100",  # Iridocyclitis of bilateral eyes caused by Human alphaherpesvirus 3 (disorder)
        "15680201000119106",  # Keratoconjunctivitis of right eye due to herpes zoster (disorder)
        "15680241000119108",  # Keratoconjunctivitis of left eye due to herpes zoster (disorder)
        "15681321000119100",  # Dermatitis of left eyelid due to herpes zoster (disorder)
        "15681401000119101",  # Dermatitis of right eyelid due to herpes zoster (disorder)
        "15685081000119102",  # Disorder of corneal epithelium of right eye caused by Human alphaherpesvirus 3 (disorder)
        "15685121000119100",  # Disorder of corneal epithelium of left eye caused by Human alphaherpesvirus 3 (disorder)
        "15685161000119105",  # Disorder of corneal epithelium of bilateral eyes caused by Human alphaherpesvirus 3 (disorder)
        "15936581000119108",  # Otitis externa of right external auditory canal caused by human herpesvirus 3 (disorder)
        "15936621000119108",  # Otitis externa of left external auditory canal caused by human herpesvirus 3 (disorder)
        "15989271000119107",  # Conjunctivitis of right eye caused by herpes zoster virus (disorder)
        "15989311000119107",  # Conjunctivitis of bilateral eyes caused by herpes zoster virus (disorder)
        "15989351000119108",  # Conjunctivitis of left eye caused by herpes zoster virus (disorder)
        "15991711000119108",  # Iritis of right eye caused by herpes zoster virus (disorder)
        "15991751000119109",  # Iritis of bilateral eyes caused by Human alphaherpesvirus 3 (disorder)
        "15991791000119104",  # Iritis of left eye caused by herpes zoster virus (disorder)
        "15992351000119104",  # Keratitis of bilateral eyes caused by Human herpesvirus 3 (disorder)
        "16000751000119105",  # Scleritis of bilateral eyes due to herpes zoster (disorder)
        "16000791000119100",  # Scleritis of left eye due to herpes zoster (disorder)
        "16000831000119106",  # Scleritis of right eye due to herpes zoster (disorder)
        "161423008",  # History of chickenpox (situation)
        "195911009",  # Chickenpox pneumonia (disorder)
        "21954000",  # Herpes zoster auricularis (disorder)
        "230176008",  # Herpes zoster encephalitis (disorder)
        "230198004",  # Varicella transverse myelitis (disorder)
        "230262004",  # Amyotrophy caused by herpes zoster (disorder)
        "230536009",  # Acute trigeminal herpes zoster (disorder)
        "232400003",  # Acute herpes zoster pharyngitis (disorder)
        "235059009",  # Herpes zoster infection of oral mucosa (disorder)
        "23737006",  # Chickenpox with complication (disorder)
        "240468001",  # Neurological varicella (disorder)
        "240470005",  # Herpes zoster involving thoracic dermatome (disorder)
        "240471009",  # Herpes zoster involving lumbar dermatome (disorder)
        "240472002",  # Maxillary herpes zoster infection (disorder)
        "240473007",  # Mandibular herpes zoster infection (disorder)
        "240474001",  # Multidermatomal herpes zoster infection (disorder)
        "24059009",  # Acute cerebellar ataxia caused by varicella (disorder)
        "309465005",  # Varicella-zoster virus infection (disorder)
        "31920006",  # Hemorrhagic pneumonia caused by varicella (disorder)
        "331071000119101",  # Herpes zoster iritis (disorder)
        "36292003",  # Otitis externa caused by Herpes zoster (disorder)
        "371113008",  # Varicella immune (finding)
        "38907003",  # Varicella (disorder)
        "397573005",  # Herpes zoster keratitis (disorder)
        "400020001",  # Herpes zoster dermatitis (disorder)
        "402897003",  # Herpes zoster involving cervical dermatome (disorder)
        "402898008",  # Herpes zoster involving sacral dermatome (disorder)
        "402899000",  # Anetoderma following varicella (disorder)
        "410500004",  # Herpes zoster involving tip of nose (disorder)
        "410509003",  # Herpes zoster conjunctivitis (disorder)
        "421029004",  # Herpes zoster retinitis (disorder)
        "422127002",  # Herpes zoster with acquired immunodeficiency syndrome (disorder)
        "422446008",  # Herpes zoster subepithelial infiltrates (disorder)
        "422471006",  # Herpes zoster acute retinal necrosis (disorder)
        "422666006",  # Disorder of corneal epithelium caused by Human alphaherpesvirus 3 (disorder)
        "423333008",  # Exanthem due to varicella (disorder)
        "423628002",  # Acute herpes zoster neuropathy (disorder)
        "424353002",  # Corneal haze caused by herpes zoster (disorder)
        "424435009",  # Herpes zoster pseudodendrites (disorder)
        "42448002",  # Herpes zoster keratoconjunctivitis (disorder)
        "424801004",  # Herpes zoster corneal endotheliitis (disorder)
        "424941009",  # Herpes zoster radiculitis (disorder)
        "425356002",  # Herpes zoster retinal pigment epithelial detachment (disorder)
        "426570007",  # Herpes zoster interstitial keratitis (disorder)
        "428633000",  # Exanthem due to herpes zoster (disorder)
        "4740000",  # Herpes zoster (disorder)
        "55560002",  # Disseminated herpes zoster (disorder)
        "681221000119108",  # History of herpes zoster (situation)
        "713250002",  # Gingival disease caused by varicella-zoster virus (disorder)
        "713733003",  # Infection caused by herpes zoster virus co-occurrent with human immunodeficiency virus infection (disorder)
        "713964006",  # Multidermatomal infection caused by Herpes zoster co-occurrent with human immunodeficiency virus infection (disorder)
        "715223009",  # Fetal varicella syndrome (disorder)
        "838357005",  # Acute glossopharyngeal neuropathy caused by Herpes zoster (disorder)
        "87513003",  # Herpes zoster ophthalmicus (disorder)
        "98541000119101",  # Herpes zoster myelitis (disorder)
    }

class CongenitalOrAcquiredAbsenceOfCervix(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent diagnoses related to absence of a cervix.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of an absence of cervix with or without remaining uterus, congenital or acquired absence of cervix, radical or total hysterectomy.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis of a partial hysterectomy, absence of uterus with remaining cervical stump.
    """

    VALUE_SET_NAME = "Congenital or Acquired Absence of Cervix"
    OID = "2.16.840.1.113883.3.464.1003.111.12.1016"
    DEFINITION_VERSION = "20240106"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "Q515",  # Agenesis and aplasia of cervix
        "Z90710",  # Acquired absence of both cervix and uterus
        "Z90712",  # Acquired absence of cervix with remaining uterus
    }

    SNOMEDCT = {
        "10738891000119107",  # History of total hysterectomy without abnormal cervical Papanicolaou smear (situation)
        "248911005",  # Uterine cervix absent (finding)
        "37687000",  # Congenital absence of cervix (disorder)
        "428078001",  # History of total hysterectomy (situation)
        "429290001",  # History of radical hysterectomy (situation)
        "429763009",  # History of total hysterectomy with bilateral salpingo-oophorectomy (situation)
        "473171009",  # History of vaginal hysterectomy (situation)
        "723171001",  # Acquired absence of cervix and uterus (disorder)
    }

class HistoryOfBilateralMastectomy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of a history of bilateral mastectomy.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of a history of a bilateral mastectomy.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis of a history of a unilateral or unspecified mastectomy.
    """

    VALUE_SET_NAME = "History of bilateral mastectomy"
    OID = "2.16.840.1.113883.3.464.1003.198.12.1068"
    DEFINITION_VERSION = "20171219"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "Z9013",  # Acquired absence of bilateral breasts and nipples
    }

    SNOMEDCT = {
        "136071000119101",  # History of bilateral prophylactic mastectomy (situation)
        "428529004",  # History of bilateral mastectomy (situation)
    }

class StatusPostLeftMastectomy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of a history of a left mastectomy.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of a left mastectomy.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis of a right, bilateral, or unspecified mastectomy.
    """

    VALUE_SET_NAME = "Status Post Left Mastectomy"
    OID = "2.16.840.1.113883.3.464.1003.198.12.1069"
    DEFINITION_VERSION = "20171216"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "Z9012",  # Acquired absence of left breast and nipple
    }

    SNOMEDCT = {
        "137671000119105",  # History of prophylactic mastectomy of left breast (situation)
        "429009003",  # History of left mastectomy (situation)
    }

class StatusPostRightMastectomy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of a history of a right mastectomy.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of a right mastectomy.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis of a left, bilateral, or unspecified mastectomy.
    """

    VALUE_SET_NAME = "Status Post Right Mastectomy"
    OID = "2.16.840.1.113883.3.464.1003.198.12.1070"
    DEFINITION_VERSION = "20171216"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "Z9011",  # Acquired absence of right breast and nipple
    }

    SNOMEDCT = {
        "137681000119108",  # History of prophylactic mastectomy of right breast (situation)
        "429242008",  # History of right mastectomy (situation)
    }

class UnilateralMastectomyUnspecifiedLaterality(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of a unilateral mastectomy where the side was not specified.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of a unilateral mastectomy where the side was unspecified.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis of a bilateral mastectomy or a unilateral mastectomy where the side is specified.
    """

    VALUE_SET_NAME = "Unilateral Mastectomy, Unspecified Laterality"
    OID = "2.16.840.1.113883.3.464.1003.198.12.1071"
    DEFINITION_VERSION = "20240112"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "Z9010",  # Acquired absence of unspecified breast and nipple
    }

    SNOMEDCT = {
        "22964006",  # Unilateral simple mastectomy (situation)
        "72269009",  # Unilateral modified radical mastectomy (situation)
        "73359007",  # Unilateral mastectomy (situation)
        "8115005",  # Unilateral radical mastectomy (situation)
        "88764002",  # Unilateral extended simple mastectomy (situation)
        "248802009",  # Absence of breast (finding)
    }

class MajorDepression(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of major depression.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of major depression including mild, moderate, and severe, with or without psychotic features. Includes major depression during pregnancy and major depression in partial remission.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis of major depression in remission or depression not specified as "major depression."
    """

    VALUE_SET_NAME = "Major Depression"
    OID = "2.16.840.1.113883.3.464.1003.105.12.1007"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "F320",  # Major depressive disorder, single episode, mild
        "F321",  # Major depressive disorder, single episode, moderate
        "F322",  # Major depressive disorder, single episode, severe without psychotic features
        "F323",  # Major depressive disorder, single episode, severe with psychotic features
        "F324",  # Major depressive disorder, single episode, in partial remission
        "F329",  # Major depressive disorder, single episode, unspecified
        "F330",  # Major depressive disorder, recurrent, mild
        "F331",  # Major depressive disorder, recurrent, moderate
        "F332",  # Major depressive disorder, recurrent severe without psychotic features
        "F333",  # Major depressive disorder, recurrent, severe with psychotic symptoms
        "F3341",  # Major depressive disorder, recurrent, in partial remission
        "F339",  # Major depressive disorder, recurrent, unspecified
    }

    SNOMEDCT = {
        "10811121000119102",  # Major depressive disorder in mother complicating childbirth (disorder)
        "10811161000119107",  # Major depressive disorder in mother complicating pregnancy (disorder)
        "14183003",  # Chronic major depressive disorder, single episode (disorder)
        "15193003",  # Severe recurrent major depression with psychotic features, mood-incongruent (disorder)
        "15639000",  # Moderate major depression, single episode (disorder)
        "16264621000119109",  # Recurrent mild major depressive disorder co-occurrent with anxiety (disorder)
        "16264821000119108",  # Recurrent severe major depressive disorder co-occurrent with anxiety (disorder)
        "16264901000119109",  # Recurrent moderate major depressive disorder co-occurrent with anxiety (disorder)
        "16265951000119109",  # Mild major depressive disorder co-occurrent with anxiety single episode (disorder)
        "16266831000119100",  # Moderate major depressive disorder co-occurrent with anxiety single episode (disorder)
        "16266991000119108",  # Severe major depressive disorder co-occurrent with anxiety single episode (disorder)
        "18818009",  # Moderate recurrent major depression (disorder)
        "191604000",  # Single major depressive episode, severe, with psychosis (disorder)
        "191610000",  # Recurrent major depressive episodes, mild (disorder)
        "191611001",  # Recurrent major depressive episodes, moderate (disorder)
        "191613003",  # Recurrent major depressive episodes, severe, with psychosis (disorder)
        "20250007",  # Severe major depression, single episode, with psychotic features, mood-incongruent (disorder)
        "251000119105",  # Severe major depression, single episode (disorder)
        "25922000",  # Major depressive disorder, single episode with postpartum onset (disorder)
        "2618002",  # Chronic recurrent major depressive disorder (disorder)
        "268621008",  # Recurrent major depressive episodes (disorder)
        "281000119103",  # Severe recurrent major depression (disorder)
        "28475009",  # Severe recurrent major depression with psychotic features (disorder)
        "319768000",  # Recurrent major depressive disorder with melancholic features (disorder)
        "320751009",  # Major depression, melancholic type (disorder)
        "33078009",  # Severe recurrent major depression with psychotic features, mood-congruent (disorder)
        "33736005",  # Severe major depression with psychotic features, mood-congruent (disorder)
        "36474008",  # Severe recurrent major depression without psychotic features (disorder)
        "36923009",  # Major depression, single episode (disorder)
        "370143000",  # Major depressive disorder (disorder)
        "38694004",  # Recurrent major depressive disorder with atypical features (disorder)
        "39809009",  # Recurrent major depressive disorder with catatonic features (disorder)
        "40379007",  # Mild recurrent major depression (disorder)
        "42925002",  # Major depressive disorder, single episode with atypical features (disorder)
        "430852001",  # Severe major depression, single episode, with psychotic features (disorder)
        "450714000",  # Severe major depression (disorder)
        "60099002",  # Severe major depression with psychotic features, mood-incongruent (disorder)
        "63778009",  # Major depressive disorder, single episode with melancholic features (disorder)
        "66344007",  # Recurrent major depression (disorder)
        "69392006",  # Major depressive disorder, single episode with catatonic features (disorder)
        "71336009",  # Recurrent major depressive disorder with postpartum onset (disorder)
        "719592004",  # Moderately severe major depression (disorder)
        "720451004",  # Minimal recurrent major depression (disorder)
        "720452006",  # Moderately severe recurrent major depression (disorder)
        "720453001",  # Moderately severe major depression single episode (disorder)
        "720454007",  # Minimal major depression single episode (disorder)
        "720455008",  # Minimal major depression (disorder)
        "726772006",  # Major depression with psychotic features (disorder)
        "73867007",  # Severe major depression with psychotic features (disorder)
        "75084000",  # Severe major depression without psychotic features (disorder)
        "76441001",  # Severe major depression, single episode, without psychotic features (disorder)
        "77911002",  # Severe major depression, single episode, with psychotic features, mood-congruent (disorder)
        "79298009",  # Mild major depression, single episode (disorder)
        "832007",  # Moderate major depression (disorder)
        "87512008",  # Mild major depression (disorder)
    }

class PainWarrantingFurtherInvestigationForProstateCancer(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent types of pain or conditions that could cause pain that could be associated with prostate cancer.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent types of pain or conditions that could cause pain that could be associated with prostate cancer.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Pain Warranting Further Investigation for Prostate Cancer"
    OID = "2.16.840.1.113883.3.526.3.451"
    DEFINITION_VERSION = "20200306"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "G893",  # Neoplasm related pain (acute) (chronic)
        "M25551",  # Pain in right hip
        "M25552",  # Pain in left hip
        "M25559",  # Pain in unspecified hip
        "M4327",  # Fusion of spine, lumbosacral region
        "M4328",  # Fusion of spine, sacral and sacrococcygeal region
        "M532X7",  # Spinal instabilities, lumbosacral region
        "M532X8",  # Spinal instabilities, sacral and sacrococcygeal region
        "M533",  # Sacrococcygeal disorders, not elsewhere classified
        "M5386",  # Other specified dorsopathies, lumbar region
        "M5387",  # Other specified dorsopathies, lumbosacral region
        "M5388",  # Other specified dorsopathies, sacral and sacrococcygeal region
        "M546",  # Pain in thoracic spine
        "M5489",  # Other dorsalgia
        "M549",  # Dorsalgia, unspecified
        "M859",  # Disorder of bone density and structure, unspecified
        "M899",  # Disorder of bone, unspecified
        "M949",  # Disorder of cartilage, unspecified
        "R0782",  # Intercostal pain
        "R0789",  # Other chest pain
        "R079",  # Chest pain, unspecified
        "R100",  # Acute abdomen
        "R109",  # Unspecified abdominal pain
    }

    SNOMEDCT = {
        "12584003",  # Bone pain (finding)
        "161977000",  # Costal margin chest pain (finding)
        "203508001",  # Bony pelvic pain (finding)
        "203509009",  # Clavicle pain (finding)
        "20793008",  # Scapulalgia (finding)
        "267981009",  # Pain in thoracic spine (finding)
        "282743009",  # Malignant bone pain (finding)
        "297217002",  # Rib pain (finding)
        "298731003",  # Pain of sternum (finding)
        "298740004",  # Rib tender (finding)
        "301773003",  # Pain in femur (finding)
        "3200003",  # Sacrocoxalgia (finding)
        "34789001",  # Pain in coccyx (finding)
        "36729000",  # Pain of prostate (finding)
        "426135001",  # Chronic prostatitis - chronic pelvic pain syndrome (disorder)
        "48926009",  # Pain in spine (finding)
        "49218002",  # Hip pain (finding)
        "89638008",  # Xiphodynia (finding)
    }

class ProstateCancer(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of diagnoses for prostate cancer.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of prostate cancer.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Prostate Cancer"
    OID = "2.16.840.1.113883.3.526.3.319"
    DEFINITION_VERSION = "20200306"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "C61",  # Malignant neoplasm of prostate
    }

    SNOMEDCT = {
        "1208457007",  # Small cell neuroendocrine carcinoma of prostate (disorder)
        "254900004",  # Carcinoma of prostate (disorder)
        "278060005",  # Endometrioid carcinoma of prostate (disorder)
        "314969001",  # Local recurrence of malignant tumor of prostate (disorder)
        "369485004",  # Malignant tumor involving prostate by direct extension from bladder (disorder)
        "399068003",  # Malignant tumor of prostate (disorder)
        "399490008",  # Adenocarcinoma of prostate (disorder)
        "399590005",  # Squamous cell carcinoma of prostate (disorder)
        "427492003",  # Hormone refractory prostate cancer (disorder)
        "93974005",  # Primary malignant neoplasm of prostate (disorder)
        "94503003",  # Metastatic malignant neoplasm to prostate (disorder)
    }

class MalignantNeoplasmOfColon(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of malignant neoplasms of the colon.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of malignant neoplasms of the colon, rectum and anus.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Malignant Neoplasm of Colon"
    OID = "2.16.840.1.113883.3.464.1003.108.12.1001"
    DEFINITION_VERSION = "20240112"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "C180",  # Malignant neoplasm of cecum
        "C181",  # Malignant neoplasm of appendix
        "C182",  # Malignant neoplasm of ascending colon
        "C183",  # Malignant neoplasm of hepatic flexure
        "C184",  # Malignant neoplasm of transverse colon
        "C185",  # Malignant neoplasm of splenic flexure
        "C186",  # Malignant neoplasm of descending colon
        "C187",  # Malignant neoplasm of sigmoid colon
        "C188",  # Malignant neoplasm of overlapping sites of colon
        "C189",  # Malignant neoplasm of colon, unspecified
        "C19",  # Malignant neoplasm of rectosigmoid junction
        "C20",  # Malignant neoplasm of rectum
        "C212",  # Malignant neoplasm of cloacogenic zone
        "C218",  # Malignant neoplasm of overlapping sites of rectum, anus and anal canal
        "C785",  # Secondary malignant neoplasm of large intestine and rectum
        "Z85038",  # Personal history of other malignant neoplasm of large intestine
        "Z85048",  # Personal history of other malignant neoplasm of rectum, rectosigmoid junction, and anus
    }

    SNOMEDCT = {
        "187758006",  # Malignant neoplasm of other specified sites of colon (disorder)
        "187757001",  # Malignant neoplasm, overlapping lesion of colon (disorder)
        "395705003",  # pTis: Carcinoma in situ, intraepithelial (colon/rectum) (finding)
        "285611007",  # Metastasis to colon of unknown primary (disorder)
        "285612000",  # Metastasis to rectum of unknown primary (disorder)
        "369455009",  # Malignant tumor involving rectum by separate metastasis from endometrium (disorder)
        "369456005",  # Malignant tumor involving rectum by separate metastasis from fallopian tube (disorder)
        "369457001",  # Malignant tumor involving rectum by separate metastasis from ovary (disorder)
        "369458006",  # Malignant tumor involving rectum by separate metastasis from prostate (disorder)
        "369459003",  # Malignant tumor involving rectum by separate metastasis from uterine cervix (disorder)
        "369460008",  # Malignant tumor involving rectum by separate metastasis from uterus (disorder)
        "369461007",  # Malignant tumor involving rectum by separate metastasis from vagina (disorder)
        "96981000119102",  # Primary malignant neoplasm of rectosigmoid junction metastatic to brain (disorder)
        "123691000119104",  # Malignant carcinoid tumor of sigmoid colon (disorder)
        "286771000119106",  # History of malignant carcinoid tumor of colon (situation)
        "286791000119107",  # History of malignant carcinoid tumor of rectum (situation)
        "713573006",  # Malignant carcinoid tumor of rectum (disorder)
        "726654006",  # Malignant carcinoid tumor of colon (disorder)
        "109838007",  # Overlapping malignant neoplasm of colon (disorder)
        "109839004",  # Overlapping malignant neoplasm of rectum, anus and anal canal (disorder)
        "109840002",  # Primary malignant neoplasm of cloacogenic zone (disorder)
        "10987871000119109",  # History of malignant neoplasm of rectosigmoid junction (situation)
        "1156783003",  # Leiomyosarcoma of rectum (disorder)
        "1156788007",  # Leiomyosarcoma of colon (disorder)
        "1156795003",  # Kaposi sarcoma of colon (disorder)
        "1156797006",  # Kaposi sarcoma of rectum (disorder)
        "1163568002",  # Primary malignant neoplasm of overlapping sites of colon (disorder)
        "1197354001",  # Primary malignant mesenchymal neoplasm of colon (disorder)
        "1197355000",  # Primary malignant mesenchymal neoplasm of rectum (disorder)
        "1197359006",  # Familial colorectal cancer type X (disorder)
        "1204448006",  # History of carcinoma of rectosigmoid junction (situation)
        "123701000119104",  # Malignant carcinoid tumor of descending colon (disorder)
        "123721000119108",  # Malignant carcinoid tumor of ascending colon (disorder)
        "1237454003",  # Metastatic carcinoma to splenic flexure (disorder)
        "1237455002",  # Metastatic carcinoma to descending colon (disorder)
        "1237456001",  # Metastatic carcinoma to hepatic flexure (disorder)
        "1237458000",  # Metastatic carcinoma to transverse colon (disorder)
        "1237460003",  # Metastatic carcinoma to ascending colon (disorder)
        "1237480002",  # Metastatic carcinoma to sigmoid colon (disorder)
        "1237484006",  # Metastatic carcinoma to colon (disorder)
        "1237485007",  # Metastatic carcinoma to rectosigmoid junction (disorder)
        "1259403004",  # Primary carcinoma of ascending colon (disorder)
        "1259404005",  # Primary carcinoma of transverse colon (disorder)
        "1259405006",  # Primary carcinoma of descending colon (disorder)
        "1259406007",  # Primary carcinoma of hepatic flexure (disorder)
        "1259407003",  # Primary carcinoma of splenic flexure (disorder)
        "1259432001",  # Primary carcinoma of sigmoid colon (disorder)
        "1259436003",  # Primary carcinoma of colon (disorder)
        "1259437007",  # Primary carcinoma of rectosigmoid junction (disorder)
        "1268633007",  # Primary squamous cell carcinoma of rectum (disorder)
        "1268635000",  # Primary squamous cell carcinoma of colon (disorder)
        "1269123008",  # Metastatic squamous cell carcinoma to rectum (disorder)
        "130381000119103",  # Primary malignant neuroendocrine neoplasm of ascending colon (disorder)
        "133751000119102",  # Lymphoma of colon (disorder)
        "16636051000119105",  # Primary malignant gastrointestinal stromal neoplasm of colon (disorder)
        "16636101000119105",  # Primary malignant gastrointestinal stromal neoplasm of rectum (disorder)
        "1701000119104",  # Primary adenocarcinoma of colon (disorder)
        "184881000119106",  # Primary adenocarcinoma of rectosigmoid junction (disorder)
        "187760008",  # Malignant neoplasm of rectum, rectosigmoid junction and anus (disorder)
        "254582000",  # Adenocarcinoma of rectum (disorder)
        "254586002",  # Malignant tumor of anorectal junction (disorder)
        "269533000",  # Carcinoma of colon (disorder)
        "269544008",  # Carcinoma of rectosigmoid junction (disorder)
        "276822007",  # Malignant melanoma of rectum (disorder)
        "285312008",  # Carcinoma of sigmoid colon (disorder)
        "301756000",  # Adenocarcinoma of sigmoid colon (disorder)
        "312111009",  # Carcinoma of ascending colon (disorder)
        "312112002",  # Carcinoma of transverse colon (disorder)
        "312113007",  # Carcinoma of descending colon (disorder)
        "312114001",  # Carcinoma of hepatic flexure (disorder)
        "312115000",  # Carcinoma of splenic flexure (disorder)
        "314965007",  # Local recurrence of malignant tumor of colon (disorder)
        "314966008",  # Local recurrence of malignant tumor of rectum (disorder)
        "315058005",  # Hereditary nonpolyposis colon cancer (disorder)
        "363351006",  # Malignant tumor of rectum (disorder)
        "363406005",  # Malignant neoplasm of colon (disorder)
        "363407001",  # Malignant tumor of hepatic flexure (disorder)
        "363408006",  # Malignant tumor of transverse colon (disorder)
        "363409003",  # Malignant tumor of descending colon (disorder)
        "363410008",  # Malignant tumor of sigmoid colon (disorder)
        "363412000",  # Malignant tumor of ascending colon (disorder)
        "363413005",  # Malignant tumor of splenic flexure (disorder)
        "363414004",  # Malignant tumor of rectosigmoid junction (disorder)
        "363491008",  # Malignant tumor of cloacogenic zone (disorder)
        "363510005",  # Malignant tumor of large intestine (disorder)
        "369448007",  # Malignant tumor involving rectum by direct extension from endometrium (disorder)
        "369449004",  # Malignant tumor involving rectum by direct extension from fallopian tube (disorder)
        "369450004",  # Malignant tumor involving rectum by direct extension from ovary (disorder)
        "369451000",  # Malignant tumor involving rectum by direct extension from prostate (disorder)
        "369452007",  # Malignant tumor involving rectum by direct extension from uterine cervix (disorder)
        "369453002",  # Malignant tumor involving rectum by direct extension from uterus (disorder)
        "369454008",  # Malignant tumor involving rectum by direct extension from vagina (disorder)
        "422375001",  # Carcinoma of colon, stage III (finding)
        "422581008",  # Carcinoma of colon, stage II (finding)
        "422985007",  # Carcinoma of colon, stage IV (finding)
        "425178004",  # Adenocarcinoma of rectosigmoid junction (disorder)
        "425213009",  # Carcinoma of colon, stage I (finding)
        "429084005",  # History of malignant neoplasm of rectum (situation)
        "429699009",  # History of malignant neoplasm of colon (situation)
        "443488001",  # Malignant neoplasm of anorectum (disorder)
        "447886005",  # Adenocarcinoma of anorectum (disorder)
        "448994001",  # Malignant epithelial neoplasm of upper rectum (disorder)
        "449218003",  # Lymphoma of sigmoid colon (disorder)
        "681601000119101",  # Primary adenocarcinoma of ascending colon (disorder)
        "681651000119102",  # Primary adenocarcinoma of descending colon (disorder)
        "721695008",  # Primary adenocarcinoma of ascending colon and right flexure (disorder)
        "721696009",  # Primary adenocarcinoma of transverse colon (disorder)
        "721697000",  # Primary neuroendocrine carcinoma of colon (disorder)
        "721698005",  # Primary malignant neuroendocrine neoplasm of colon (disorder)
        "721699002",  # Primary adenocarcinoma of descending colon and splenic flexure (disorder)
        "721700001",  # Primary malignant neuroendocrine neoplasm of rectum (disorder)
        "721701002",  # Primary neuroendocrine carcinoma of rectum (disorder)
        "737058005",  # Microsatellite instability-high colorectal cancer (disorder)
        "766979005",  # Squamous cell carcinoma of rectum (disorder)
        "766981007",  # Squamous cell carcinoma of colon (disorder)
        "93683002",  # Primary malignant neoplasm of ascending colon (disorder)
        "93761005",  # Primary malignant neoplasm of colon (disorder)
        "93771007",  # Primary malignant neoplasm of descending colon (disorder)
        "93826009",  # Primary malignant neoplasm of hepatic flexure of colon (disorder)
        "93980002",  # Primary malignant neoplasm of rectosigmoid junction (disorder)
        "93984006",  # Primary malignant neoplasm of rectum (disorder)
        "94006002",  # Primary malignant neoplasm of sigmoid colon (disorder)
        "94072004",  # Primary malignant neoplasm of splenic flexure of colon (disorder)
        "94105000",  # Primary malignant neoplasm of transverse colon (disorder)
        "94179005",  # Metastatic malignant neoplasm to ascending colon (disorder)
        "94260004",  # Metastatic malignant neoplasm to colon (disorder)
        "94271003",  # Metastatic malignant neoplasm to descending colon (disorder)
        "94328005",  # Metastatic malignant neoplasm to hepatic flexure of colon (disorder)
        "94509004",  # Metastatic malignant neoplasm to rectosigmoid junction (disorder)
        "94513006",  # Metastatic malignant neoplasm to rectum (disorder)
        "94538001",  # Metastatic malignant neoplasm to sigmoid colon (disorder)
        "94604000",  # Metastatic malignant neoplasm to splenic flexure of colon (disorder)
        "94643001",  # Metastatic malignant neoplasm to transverse colon (disorder)
        "96281000119107",  # Overlapping malignant neoplasm of colon and rectum (disorder)
    }

class AcuteAndSubacuteIridocyclitis(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of acute and subacute iridocyclitis.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of acute and subacute iridocyclitis, including primary and secondary iridocyclitis, and hypopyon.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Acute and Subacute Iridocyclitis"
    OID = "2.16.840.1.113883.3.526.3.1241"
    DEFINITION_VERSION = "20210210"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "A1854",  # Tuberculous iridocyclitis
        "A5143",  # Secondary syphilitic oculopathy
        "A5432",  # Gonococcal iridocyclitis
        "B0051",  # Herpesviral iridocyclitis
        "B0232",  # Zoster iridocyclitis
        "D8683",  # Sarcoid iridocyclitis
        "H16031",  # Corneal ulcer with hypopyon, right eye
        "H16032",  # Corneal ulcer with hypopyon, left eye
        "H16033",  # Corneal ulcer with hypopyon, bilateral
        "H2000",  # Unspecified acute and subacute iridocyclitis
        "H20011",  # Primary iridocyclitis, right eye
        "H20012",  # Primary iridocyclitis, left eye
        "H20013",  # Primary iridocyclitis, bilateral
        "H20021",  # Recurrent acute iridocyclitis, right eye
        "H20022",  # Recurrent acute iridocyclitis, left eye
        "H20023",  # Recurrent acute iridocyclitis, bilateral
        "H20031",  # Secondary infectious iridocyclitis, right eye
        "H20032",  # Secondary infectious iridocyclitis, left eye
        "H20033",  # Secondary infectious iridocyclitis, bilateral
        "H20041",  # Secondary noninfectious iridocyclitis, right eye
        "H20042",  # Secondary noninfectious iridocyclitis, left eye
        "H20043",  # Secondary noninfectious iridocyclitis, bilateral
        "H20051",  # Hypopyon, right eye
        "H20052",  # Hypopyon, left eye
        "H20053",  # Hypopyon, bilateral
    }

    SNOMEDCT = {
        "10698009",  # Herpes zoster iridocyclitis (disorder)
        "1092761000119105",  # Iridocyclitis caused by tuberculosis (disorder)
        "12630008",  # Primary iridocyclitis (disorder)
        "13608004",  # Herpes simplex iridocyclitis (disorder)
        "14427009",  # Subacute iritis (disorder)
        "15638451000119104",  # Inflammation of right iris (disorder)
        "15638491000119109",  # Inflammation of left iris (disorder)
        "15638531000119109",  # Iritis of bilateral eyes (disorder)
        "15678641000119101",  # Iridocyclitis of right eye caused by human herpes simplex virus (disorder)
        "15678681000119106",  # Iridocyclitis of left eye caused by human herpes simplex virus (disorder)
        "15678721000119100",  # Iridocyclitis of bilateral eyes caused by Human alphaherpesvirus 1 (disorder)
        "15678761000119105",  # Iridocyclitis of right eye due to herpes zoster (disorder)
        "15678801000119102",  # Iridocyclitis of left eye due to herpes zoster (disorder)
        "15678841000119100",  # Iridocyclitis of bilateral eyes caused by Human alphaherpesvirus 3 (disorder)
        "15679361000119103",  # Iridocyclitis of left eye (disorder)
        "15679401000119107",  # Iridocyclitis of right eye (disorder)
        "15679441000119109",  # Iridocyclitis of bilateral eyes (disorder)
        "15991591000119106",  # Iridocyclitis of left eye caused by Mycobacterium tuberculosis (disorder)
        "15991631000119106",  # Iridocyclitis of right eye caused by Mycobacterium tuberculosis (disorder)
        "15991671000119109",  # Iridocyclitis of bilateral eyes caused by Mycobacterium tuberculosis (disorder)
        "193487008",  # Secondary infected iridocyclitis (disorder)
        "193489006",  # Iritis due to diabetes mellitus (disorder)
        "20555001",  # Herpes simplex iritis (disorder)
        "220101000119108",  # Infectious secondary iritis (disorder)
        "231947004",  # Idiopathic inflammation of anterior uveal tract (disorder)
        "267619000",  # Non-infectious anterior uveitis (disorder)
        "29050005",  # Acute iritis (disorder)
        "29538005",  # Glaucomatocyclitic crisis (disorder)
        "30080002",  # Secondary syphilitic iridocyclitis (disorder)
        "331071000119101",  # Herpes zoster iritis (disorder)
        "332731000119108",  # Recurrent iridocyclitis of right eye (disorder)
        "332921000119107",  # Primary iridocyclitis of right eye (disorder)
        "334821000119108",  # Hypopyon of right eye (disorder)
        "336531000119103",  # Acute iritis of right eye (disorder)
        "336541000119107",  # Acute anterior uveitis of right eye (disorder)
        "336921000119106",  # Lens induced iritis (disorder)
        "338341000119103",  # Recurrent iridocyclitis of left eye (disorder)
        "338531000119102",  # Primary iridocyclitis of left eye (disorder)
        "340421000119101",  # Hypopyon of left eye (disorder)
        "342141000119102",  # Acute iritis of left eye (disorder)
        "342151000119100",  # Acute anterior uveitis of left eye (disorder)
        "342381000119109",  # Gonococcal iritis (disorder)
        "344111000119109",  # Recurrent iridocyclitis of bilateral eyes (disorder)
        "344301000119109",  # Primary iridocyclitis of bilateral eyes (disorder)
        "345761000119107",  # Hypopyon of bilateral eyes (disorder)
        "347301000119103",  # Acute iritis of bilateral eyes (disorder)
        "347311000119100",  # Acute anterior uveitis of bilateral eyes (disorder)
        "352941000119102",  # Sarcoid iridocyclitis (disorder)
        "35881000119106",  # Noninfectious secondary iritis (disorder)
        "410480004",  # Iritis in Behcet's syndrome (disorder)
        "410482007",  # Iritis with psoriatic arthritis (disorder)
        "410692006",  # Anterior uveitis (disorder)
        "413851004",  # Cidofovir-induced anterior uveitis (disorder)
        "414600000",  # Lepromatous anterior uveitis (disorder)
        "416666007",  # Anterior uveitis due to juvenile idiopathic arthritis (disorder)
        "417020006",  # Traumatic iritis (disorder)
        "420485005",  # Herpetic iridocyclitis (disorder)
        "43193009",  # Gouty iritis (disorder)
        "46916006",  # Secondary noninfectious iridocyclitis (disorder)
        "4927003",  # Acute anterior uveitis (disorder)
        "6395007",  # Hypopyon ulcer (disorder)
        "64015001",  # Subacute anterior uveitis (disorder)
        "65074000",  # Iritis (disorder)
        "675361000119103",  # Subacute iridocyclitis of bilateral eyes (disorder)
        "675371000119109",  # Subacute iridocyclitis of left eye (disorder)
        "675381000119107",  # Subacute iridocyclitis of right eye (disorder)
        "6869001",  # Recurrent iridocyclitis (disorder)
        "70461003",  # Lens-induced iridocyclitis (disorder)
        "733317009",  # Primary anterior uveitis (disorder)
        "77971008",  # Iridocyclitis (disorder)
        "87807004",  # Hypopyon (disorder)
        "89114005",  # Infectious secondary iridocyclitis (disorder)
        "9091006",  # Gonococcal iridocyclitis (disorder)
    }

class Amblyopia(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of amblyopia.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of amblyopia, including suspected amblyopia.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Amblyopia"
    OID = "2.16.840.1.113883.3.526.3.1448"
    DEFINITION_VERSION = "20210210"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H53001",  # Unspecified amblyopia, right eye
        "H53002",  # Unspecified amblyopia, left eye
        "H53003",  # Unspecified amblyopia, bilateral
        "H53011",  # Deprivation amblyopia, right eye
        "H53012",  # Deprivation amblyopia, left eye
        "H53013",  # Deprivation amblyopia, bilateral
        "H53021",  # Refractive amblyopia, right eye
        "H53022",  # Refractive amblyopia, left eye
        "H53023",  # Refractive amblyopia, bilateral
        "H53031",  # Strabismic amblyopia, right eye
        "H53032",  # Strabismic amblyopia, left eye
        "H53033",  # Strabismic amblyopia, bilateral
        "H53041",  # Amblyopia suspect, right eye
        "H53042",  # Amblyopia suspect, left eye
        "H53043",  # Amblyopia suspect, bilateral
    }

    SNOMEDCT = {
        "16697921000119109",  # Suspected amblyopia of bilateral eyes (situation)
        "16697961000119104",  # Suspected amblyopia of left eye (situation)
        "16698001000119102",  # Suspected amblyopia of right eye (situation)
        "193638002",  # Stimulus deprivation amblyopia (disorder)
        "232147001",  # Anisometropic amblyopia (disorder)
        "331961000119108",  # Strabismic amblyopia of right eye (disorder)
        "332701000119101",  # Refractive amblyopia of right eye (disorder)
        "335461000119102",  # Deprivation amblyopia of right eye (disorder)
        "336451000119108",  # Amblyopia of right eye (disorder)
        "337581000119100",  # Strabismic amblyopia of left eye (disorder)
        "338311000119102",  # Refractive amblyopia of left eye (disorder)
        "341071000119100",  # Deprivation amblyopia of left eye (disorder)
        "342061000119102",  # Amblyopia of left eye (disorder)
        "343371000119101",  # Strabismic amblyopia of bilateral eyes (disorder)
        "344081000119104",  # Refractive amblyopia of bilateral eyes (disorder)
        "346331000119107",  # Deprivation amblyopia of bilateral eyes (disorder)
        "347241000119107",  # Amblyopia of bilateral eyes (disorder)
        "35600002",  # Strabismic amblyopia (disorder)
        "361120000",  # Nutritional amblyopia (disorder)
        "387742006",  # Amblyopia (disorder)
        "387743001",  # Functional amblyopia (disorder)
        "396184003",  # Isometropic amblyopia (disorder)
        "410486005",  # Occlusion amblyopia (disorder)
        "70701004",  # Alcoholic amblyopia (disorder)
        "75834006",  # Strachan's syndrome (disorder)
        "90927000",  # Refractive amblyopia (disorder)
        "965003",  # Toxic amblyopia (disorder)
    }

class BurnConfinedToEyeAndAdnexa(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of a burn of the eye and/or adnexa.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of burns of various parts of the eye, or eye corrosion.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Burn Confined to Eye and Adnexa"
    OID = "2.16.840.1.113883.3.526.3.1409"
    DEFINITION_VERSION = "20210210"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "T2601XA",  # Burn of right eyelid and periocular area, initial encounter
        "T2602XA",  # Burn of left eyelid and periocular area, initial encounter
        "T2611XA",  # Burn of cornea and conjunctival sac, right eye, initial encounter
        "T2612XA",  # Burn of cornea and conjunctival sac, left eye, initial encounter
        "T2621XA",  # Burn with resulting rupture and destruction of right eyeball, initial encounter
        "T2622XA",  # Burn with resulting rupture and destruction of left eyeball, initial encounter
        "T2631XA",  # Burns of other specified parts of right eye and adnexa, initial encounter
        "T2632XA",  # Burns of other specified parts of left eye and adnexa, initial encounter
        "T2641XA",  # Burn of right eye and adnexa, part unspecified, initial encounter
        "T2642XA",  # Burn of left eye and adnexa, part unspecified, initial encounter
        "T2651XA",  # Corrosion of right eyelid and periocular area, initial encounter
        "T2652XA",  # Corrosion of left eyelid and periocular area, initial encounter
        "T2661XA",  # Corrosion of cornea and conjunctival sac, right eye, initial encounter
        "T2662XA",  # Corrosion of cornea and conjunctival sac, left eye, initial encounter
        "T2671XA",  # Corrosion with resulting rupture and destruction of right eyeball, initial encounter
        "T2672XA",  # Corrosion with resulting rupture and destruction of left eyeball, initial encounter
        "T2681XA",  # Corrosions of other specified parts of right eye and adnexa, initial encounter
        "T2682XA",  # Corrosions of other specified parts of left eye and adnexa, initial encounter
        "T2691XA",  # Corrosion of right eye and adnexa, part unspecified, initial encounter
        "T2692XA",  # Corrosion of left eye and adnexa, part unspecified, initial encounter
    }

    SNOMEDCT = {
        "110170006",  # Burn of periorbital region (disorder)
        "110171005",  # Epidermal burn of periorbital region (disorder)
        "110172003",  # Partial thickness burn of periorbital region (disorder)
        "110173008",  # Full thickness burn of periorbital region (disorder)
        "12081251000119109",  # Chemical burn of left conjunctival sac (disorder)
        "12081291000119104",  # Chemical burn of right conjunctival sac (disorder)
        "12081411000119109",  # Chemical burn of left cornea (disorder)
        "12081451000119105",  # Chemical burn of right cornea (disorder)
        "12081491000119100",  # Burn of cornea of left eye (disorder)
        "12081531000119100",  # Burn of cornea of right eye (disorder)
        "12081651000119107",  # Burn of conjunctival sac of left eye (disorder)
        "12081691000119102",  # Burn of conjunctival sac of right eye (disorder)
        "12081931000119106",  # Burn of left eye region (disorder)
        "12081971000119109",  # Burn of right eye region (disorder)
        "12114081000119101",  # Chemical burn of left eye (disorder)
        "12115361000119108",  # Chemical burn of right eye (disorder)
        "211659009",  # Chemical burn of eyelid region (disorder)
        "211661000",  # Alkaline chemical burn of cornea and conjunctival sac (disorder)
        "211662007",  # Acid chemical burn of cornea and conjunctival sac (disorder)
        "231819004",  # Thermal burn of eyelid (disorder)
        "231844007",  # Lacrimal punctum burn (disorder)
        "231867009",  # Thermal injury to conjunctiva (disorder)
        "231869007",  # Chemical burn injury to conjunctiva (disorder)
        "231884003",  # Burn of sclera (disorder)
        "231943000",  # Thermal and radiation injury of cornea (disorder)
        "231945007",  # Chemical injury to cornea (disorder)
        "232027007",  # Photocoagulation burn to retina (disorder)
        "232028002",  # Laser coagulation burn to retina (disorder)
        "232029005",  # Cryotherapy burn to retina (disorder)
        "269232009",  # Burn confined to eye and adnexa (disorder)
        "274204004",  # Burn of cornea (disorder)
        "274205003",  # Burn of eye region (disorder)
        "284537006",  # Eyelid burn (disorder)
        "284541005",  # Burn of conjunctiva (disorder)
        "287132002",  # Epidermal burn of eyelid (disorder)
        "287133007",  # Burn to eyelid - blister (disorder)
        "287134001",  # Full thickness burn of eyelid (disorder)
        "287137008",  # Burn to cornea - blister (disorder)
        "287138003",  # Burn to cornea - full thickness (disorder)
        "312646002",  # Burn of skin of eye region (disorder)
        "314531003",  # Burn of anterior eyeball segment (disorder)
        "38916004",  # Alkaline chemical burn of cornea AND/OR conjunctival sac (disorder)
        "40901000087106",  # Burn of left eye proper (disorder)
        "41171000087107",  # Burn of right eye proper (disorder)
        "418591009",  # Corneal wound burn (disorder)
        "423838005",  # Scleral surgical wound burn (disorder)
        "445832003",  # Chemical burn of eyelid (disorder)
        "445850005",  # Deep third degree burn of eye region (disorder)
        "446234004",  # Alkaline chemical burn of conjunctival sac (disorder)
        "446556000",  # Acid chemical burn of cornea (disorder)
        "446856005",  # Burn of conjuctival sac (disorder)
        "447094002",  # Alkaline chemical burn of cornea (disorder)
        "447095001",  # Acid chemical burn of conjunctival sac (disorder)
        "447333000",  # Burn of ocular adnexa (disorder)
        "473243005",  # Burn of eye region with burn of face (disorder)
        "473244004",  # Burn of eye region with epidermal burn of face (disorder)
        "473245003",  # Burn of eye region with partial thickness burn of face (disorder)
        "473246002",  # Burn of eye region with full thickness burn of face (disorder)
        "473248001",  # Burn of eye region with epidermal burn of head and/or neck (disorder)
        "473249009",  # Burn of eye region with partial thickness burn of head and/or neck (disorder)
        "473250009",  # Burn of eye region with full thickness burn of head and/or neck (disorder)
        "63542002",  # Acid chemical burn of cornea AND/OR conjunctival sac (disorder)
        "770555009",  # Burn of eye proper (disorder)
    }

class CataractCongenital(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of congenital cataract.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of congenital cataract.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Cataract Congenital"
    OID = "2.16.840.1.113883.3.526.3.1412"
    DEFINITION_VERSION = "20210210"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "Q120",  # Congenital cataract
    }

    SNOMEDCT = {
        "204128001",  # Congenital lamellar cataract (disorder)
        "204138006",  # Congenital blue dot cataract (disorder)
        "204139003",  # Congenital membranous cataract (disorder)
        "21590003",  # Congenital zonular cataract (disorder)
        "253223002",  # Congenital polar cataract (disorder)
        "253224008",  # Congenital anterior polar cataract (disorder)
        "253225009",  # Congenital posterior polar cataract (disorder)
        "253226005",  # Congenital sutural cataract (disorder)
        "253227001",  # Rubella cataract (disorder)
        "28550007",  # Congenital capsular cataract (disorder)
        "29590001",  # Congenital total cataract (disorder)
        "335831000119107",  # Congenital posterior subcapsular polar cataract of right eye (disorder)
        "335841000119103",  # Congenital nuclear cataract of right eye (disorder)
        "335851000119101",  # Congenital combined form cataract of right eye (disorder)
        "341441000119102",  # Congenital posterior subcapsular polar cataract of left eye (disorder)
        "341451000119100",  # Congenital nuclear cataract of left eye (disorder)
        "341461000119103",  # Congenital combined form cataract of left eye (disorder)
        "342821000119103",  # Congenital posterior subcapsular polar cataract (disorder)
        "346691000119104",  # Congenital posterior subcapsular polar cataract of bilateral eyes (disorder)
        "346701000119104",  # Congenital nuclear cataract of bilateral eyes (disorder)
        "346711000119101",  # Congenital combined form cataract of bilateral eyes (disorder)
        "346731000119106",  # Congenital anterior subcapsular polar cataract of bilateral eyes (disorder)
        "348321000119108",  # Congenital zonular cataract of right eye (disorder)
        "348771000119108",  # Congenital zonular cataract of left eye (disorder)
        "349281000119109",  # Congenital zonular cataract of bilateral eyes (disorder)
        "66499004",  # Congenital cortical cataract (disorder)
        "679911000119104",  # Congenital cortical cataract of left eye (disorder)
        "679931000119109",  # Congenital cortical cataract of right eye (disorder)
        "681041000119106",  # Congenital cortical cataract of bilateral eyes (disorder)
        "76562003",  # Congenital subcapsular cataract (disorder)
        "79410001",  # Congenital cataract (disorder)
    }

class CataractMatureOrHypermature(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of mature or hypermature cataract.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of mature or hypermature cataract, including senile or intumescent cataract.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Cataract Mature or Hypermature"
    OID = "2.16.840.1.113883.3.526.3.1413"
    DEFINITION_VERSION = "20210210"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H25011",  # Cortical age-related cataract, right eye
        "H25012",  # Cortical age-related cataract, left eye
        "H25013",  # Cortical age-related cataract, bilateral
        "H25091",  # Other age-related incipient cataract, right eye
        "H25092",  # Other age-related incipient cataract, left eye
        "H25093",  # Other age-related incipient cataract, bilateral
        "H25099",  # Other age-related incipient cataract, unspecified eye
        "H2511",  # Age-related nuclear cataract, right eye
        "H2512",  # Age-related nuclear cataract, left eye
        "H2513",  # Age-related nuclear cataract, bilateral
        "H25811",  # Combined forms of age-related cataract, right eye
        "H25812",  # Combined forms of age-related cataract, left eye
        "H25813",  # Combined forms of age-related cataract, bilateral
        "H2589",  # Other age-related cataract
        "H259",  # Unspecified age-related cataract
        "H269",  # Unspecified cataract
    }

    SNOMEDCT = {
        "193590000",  # Total, mature senile cataract (disorder)
        "217791000119107",  # Hypermature senile cataract (disorder)
        "267626000",  # Hypermature cataract (disorder)
        "347461000119107",  # Hypermature senile cataract of right eye (disorder)
        "347521000119103",  # Hypermature senile cataract of left eye (disorder)
        "347581000119104",  # Hypermature senile cataract of bilateral eyes (disorder)
        "849000",  # Mature cataract (disorder)
        "8656007",  # Total traumatic cataract (disorder)
        "95724003",  # Intumescent cataract (disorder)
    }

class CataractPosteriorPolar(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of posterior polar cataract.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of posterior polar cataract, including capsular or subcapsular cataract.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Cataract Posterior Polar"
    OID = "2.16.840.1.113883.3.526.3.1414"
    DEFINITION_VERSION = "20210210"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H25041",  # Posterior subcapsular polar age-related cataract, right eye
        "H25042",  # Posterior subcapsular polar age-related cataract, left eye
        "H25043",  # Posterior subcapsular polar age-related cataract, bilateral
    }

    SNOMEDCT = {
        "1078791000119109",  # Posterior subcapsular polar senile cataract of left eye (disorder)
        "1078801000119105",  # Posterior subcapsular polar senile cataract of right eye (disorder)
        "15639401000119105",  # Cataract of posterior subcapsule of right eye (disorder)
        "15639441000119107",  # Cataract of posterior subcapsule of left eye (disorder)
        "15639481000119102",  # Cataract of posterior subcapsule of bilateral eyes (disorder)
        "253225009",  # Congenital posterior polar cataract (disorder)
        "315353005",  # Posterior subcapsular cataract (disorder)
        "335831000119107",  # Congenital posterior subcapsular polar cataract of right eye (disorder)
        "341441000119102",  # Congenital posterior subcapsular polar cataract of left eye (disorder)
        "342821000119103",  # Congenital posterior subcapsular polar cataract (disorder)
        "34533008",  # Posterior subcapsular polar cataract (disorder)
        "346691000119104",  # Congenital posterior subcapsular polar cataract of bilateral eyes (disorder)
        "347561000119108",  # Posterior subcapsular polar senile cataract of bilateral eyes (disorder)
        "5318001",  # Posterior subcapsular polar senile cataract (disorder)
    }

class CataractSecondaryToOcularDisorders(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of cataract that is a secondary diagnosis to other ocular conditions.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of cataract secondary to degenerative or inflammatory ocular disorders, cataract with neovascularization, or subcapsular glaucomatous flecks.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Cataract Secondary to Ocular Disorders"
    OID = "2.16.840.1.113883.3.526.3.1410"
    DEFINITION_VERSION = "20210210"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H26211",  # Cataract with neovascularization, right eye
        "H26212",  # Cataract with neovascularization, left eye
        "H26213",  # Cataract with neovascularization, bilateral
        "H26221",  # Cataract secondary to ocular disorders (degenerative) (inflammatory), right eye
        "H26222",  # Cataract secondary to ocular disorders (degenerative) (inflammatory), left eye
        "H26223",  # Cataract secondary to ocular disorders (degenerative) (inflammatory), bilateral
    }

    SNOMEDCT = {
        "1217661001",  # Cataract due to degenerative disorder of eye (disorder)
        "15738161000119104",  # Cataract of right eye due to ocular disease (disorder)
        "15738201000119109",  # Cataract of left eye due to ocular disease (disorder)
        "15738241000119106",  # Cataract of left eye due to inflammatory disorder (disorder)
        "15738281000119101",  # Cataract of right eye due to inflammatory disorder (disorder)
        "15738321000119106",  # Cataract of bilateral eyes due to inflammatory disorder (disorder)
        "15990151000119103",  # Cataract due to and following ocular disease of bilateral eyes (disorder)
        "15990191000119108",  # Cataract due to and following ocular disease of left eye (disorder)
        "15990231000119104",  # Cataract due to and following ocular disease of right eye (disorder)
        "193600001",  # Cataract due to ocular disease (disorder)
        "193602009",  # Cataract due to inflammatory disorder (disorder)
        "331941000119109",  # Glaukomflecken of right eye (disorder)
        "337561000119109",  # Glaukomflecken of left eye (disorder)
        "343351000119105",  # Glaukomflecken of bilateral eyes (disorder)
        "67733005",  # Glaukomflecken (disorder)
    }

class CentralCornealUlcer(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of central corneal ulcer.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of central corneal ulcer or keratitis.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Central Corneal Ulcer"
    OID = "2.16.840.1.113883.3.526.3.1428"
    DEFINITION_VERSION = "20210210"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H16011",  # Central corneal ulcer, right eye
        "H16012",  # Central corneal ulcer, left eye
        "H16013",  # Central corneal ulcer, bilateral
    }

    SNOMEDCT = {
        "231901007",  # Neuropathic corneal ulcer (disorder)
        "332801000119108",  # Punctate keratitis of right eye (disorder)
        "338411000119106",  # Punctate keratitis of left eye (disorder)
        "344181000119103",  # Punctate keratitis of bilateral eyes (disorder)
        "347031000119107",  # Ulcer of bilateral central corneas (disorder)
        "397570008",  # Herpes simplex stromal keratitis (disorder)
        "397571007",  # Herpes simplex necrotizing stromal keratitis (disorder)
        "42513006",  # Punctate keratitis (disorder)
        "7426009",  # Central corneal ulcer (disorder)
        "95732006",  # Necrotizing keratitis (disorder)
        "95733001",  # Eosinophilic keratitis (disorder)
    }

class CertainTypesOfIridocyclitis(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of various types of iridocyclitis.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of lens-induced iridocyclitis, Fuchs' heterochromic cyclitis, Vogt-Koyanagi syndrome, and other types of iridocyclitis.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Certain Types of Iridocyclitis"
    OID = "2.16.840.1.113883.3.526.3.1415"
    DEFINITION_VERSION = "20210210"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H2021",  # Lens-induced iridocyclitis, right eye
        "H2022",  # Lens-induced iridocyclitis, left eye
        "H2023",  # Lens-induced iridocyclitis, bilateral
        "H20811",  # Fuchs' heterochromic cyclitis, right eye
        "H20812",  # Fuchs' heterochromic cyclitis, left eye
        "H20813",  # Fuchs' heterochromic cyclitis, bilateral
        "H20821",  # Vogt-Koyanagi syndrome, right eye
        "H20822",  # Vogt-Koyanagi syndrome, left eye
        "H20823",  # Vogt-Koyanagi syndrome, bilateral
        "H209",  # Unspecified iridocyclitis
    }

    SNOMEDCT = {
        "10698009",  # Herpes zoster iridocyclitis (disorder)
        "1092761000119105",  # Iridocyclitis caused by tuberculosis (disorder)
        "11226001",  # Fuchs' heterochromic cyclitis (disorder)
        "12630008",  # Primary iridocyclitis (disorder)
        "13608004",  # Herpes simplex iridocyclitis (disorder)
        "15678641000119101",  # Iridocyclitis of right eye caused by human herpes simplex virus (disorder)
        "15678681000119106",  # Iridocyclitis of left eye caused by human herpes simplex virus (disorder)
        "15678721000119100",  # Iridocyclitis of bilateral eyes caused by Human alphaherpesvirus 1 (disorder)
        "15678761000119105",  # Iridocyclitis of right eye due to herpes zoster (disorder)
        "15678801000119102",  # Iridocyclitis of left eye due to herpes zoster (disorder)
        "15678841000119100",  # Iridocyclitis of bilateral eyes caused by Human alphaherpesvirus 3 (disorder)
        "15679361000119103",  # Iridocyclitis of left eye (disorder)
        "15679401000119107",  # Iridocyclitis of right eye (disorder)
        "15679441000119109",  # Iridocyclitis of bilateral eyes (disorder)
        "15991591000119106",  # Iridocyclitis of left eye caused by Mycobacterium tuberculosis (disorder)
        "15991631000119106",  # Iridocyclitis of right eye caused by Mycobacterium tuberculosis (disorder)
        "15991671000119109",  # Iridocyclitis of bilateral eyes caused by Mycobacterium tuberculosis (disorder)
        "193487008",  # Secondary infected iridocyclitis (disorder)
        "23798003",  # Translucency of iris (disorder)
        "30080002",  # Secondary syphilitic iridocyclitis (disorder)
        "332731000119108",  # Recurrent iridocyclitis of right eye (disorder)
        "332921000119107",  # Primary iridocyclitis of right eye (disorder)
        "335061000119105",  # Fuchs' heterochromic cyclitis of right eye (disorder)
        "338341000119103",  # Recurrent iridocyclitis of left eye (disorder)
        "338531000119102",  # Primary iridocyclitis of left eye (disorder)
        "340671000119102",  # Fuchs' heterochromic cyclitis of left eye (disorder)
        "344111000119109",  # Recurrent iridocyclitis of bilateral eyes (disorder)
        "344301000119109",  # Primary iridocyclitis of bilateral eyes (disorder)
        "420485005",  # Herpetic iridocyclitis (disorder)
        "46916006",  # Secondary noninfectious iridocyclitis (disorder)
        "680291000119102",  # Fuchs' heterochromic cyclitis of bilateral eyes (disorder)
        "6869001",  # Recurrent iridocyclitis (disorder)
        "70461003",  # Lens-induced iridocyclitis (disorder)
        "77971008",  # Iridocyclitis (disorder)
        "89114005",  # Infectious secondary iridocyclitis (disorder)
        "9091006",  # Gonococcal iridocyclitis (disorder)
    }

class ChoroidalDegenerations(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of choroidal degenerations.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of degenerations of the choroid, including choroid atrophy and dystrophy, angioid streaks, and choroideremia.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Choroidal Degenerations"
    OID = "2.16.840.1.113883.3.526.3.1450"
    DEFINITION_VERSION = "20210210"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H31111",  # Age-related choroidal atrophy, right eye
        "H31112",  # Age-related choroidal atrophy, left eye
        "H31113",  # Age-related choroidal atrophy, bilateral
        "H31121",  # Diffuse secondary atrophy of choroid, right eye
        "H31122",  # Diffuse secondary atrophy of choroid, left eye
        "H31123",  # Diffuse secondary atrophy of choroid, bilateral
        "H3120",  # Hereditary choroidal dystrophy, unspecified
        "H3121",  # Choroideremia
        "H3122",  # Choroidal dystrophy (central areolar) (generalized) (peripapillary)
        "H3123",  # Gyrate atrophy, choroid
        "H3129",  # Other hereditary choroidal dystrophy
        "H3533",  # Angioid streaks of macula
    }

    SNOMEDCT = {
        "1231678007",  # Peripapillary choroidal dystrophy (disorder)
        "15989831000119105",  # Choroideremia of bilateral eyes (disorder)
        "15989911000119106",  # Angioid streaks of choroid of right eye (disorder)
        "15989951000119107",  # Angioid streaks of choroid of left eye (disorder)
        "15989991000119102",  # Angioid streaks of choroid of bilateral eyes (disorder)
        "193463006",  # Diffuse secondary choroid atrophy (disorder)
        "193466003",  # Partial circumpapillary choroid dystrophy (disorder)
        "193468002",  # Partial central choroid dystrophy (disorder)
        "19871000",  # Total generalized choroidal dystrophy (disorder)
        "231996009",  # Central areolar choroidal sclerosis (disorder)
        "231997000",  # Peripapillary choroidal sclerosis (disorder)
        "247149008",  # Macular choroidal sclerosis (disorder)
        "24950005",  # Partial generalized choroidal dystrophy (disorder)
        "312918002",  # Choroidal dystrophy (disorder)
        "312927001",  # Bietti's crystalline retinopathy (disorder)
        "335401000119103",  # Diffuse secondary atrophy of choroid of right eye (disorder)
        "336081000119106",  # Choroid degeneration of right eye (disorder)
        "336491000119103",  # Age-related atrophy of right choroid (disorder)
        "341011000119108",  # Diffuse secondary atrophy of choroid of left eye (disorder)
        "341691000119108",  # Choroid degeneration of left eye (disorder)
        "342101000119104",  # Age-related atrophy of left choroid (disorder)
        "346281000119104",  # Diffuse secondary atrophy of choroid of bilateral eyes (disorder)
        "346911000119103",  # Choroid degeneration of bilateral eyes (disorder)
        "347261000119106",  # Senile atrophy of choroid of bilateral eyes (disorder)
        "38513001",  # Senile atrophy of choroid (disorder)
        "406446000",  # Choroidal degeneration (disorder)
        "41250005",  # Diffuse choroidal sclerosis (disorder)
        "59753003",  # Total circumpapillary dystrophy of choroid (disorder)
        "74469006",  # Hereditary choroidal dystrophy (disorder)
        "75241009",  # Choroideremia (disorder)
        "86103006",  # Angioid streaks of choroid (disorder)
        "92820007",  # Circumpapillary dystrophy of choroid (disorder)
    }

class ChoroidalDetachment(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of choroidal detachment.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis choroidal detachment, or effusion.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Choroidal Detachment"
    OID = "2.16.840.1.113883.3.526.3.1451"
    DEFINITION_VERSION = "20210210"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H31401",  # Unspecified choroidal detachment, right eye
        "H31402",  # Unspecified choroidal detachment, left eye
        "H31403",  # Unspecified choroidal detachment, bilateral
        "H31411",  # Hemorrhagic choroidal detachment, right eye
        "H31412",  # Hemorrhagic choroidal detachment, left eye
        "H31413",  # Hemorrhagic choroidal detachment, bilateral
        "H31421",  # Serous choroidal detachment, right eye
        "H31422",  # Serous choroidal detachment, left eye
        "H31423",  # Serous choroidal detachment, bilateral
    }

    SNOMEDCT = {
        "19031009",  # Hemorrhagic choroidal detachment (disorder)
        "232000008",  # Choroidal effusion (disorder)
        "247198004",  # Annular choroidal detachment (disorder)
        "27871007",  # Choroidal detachment (disorder)
        "278805002",  # Uveal effusion syndrome (disorder)
        "332111000119102",  # Serous choroidal detachment of right eye (disorder)
        "334901000119109",  # Hemorrhagic detachment of right choroid (disorder)
        "336071000119108",  # Choroidal detachment of right eye (disorder)
        "337721000119107",  # Serous choroidal detachment of left eye (disorder)
        "340511000119109",  # Hemorrhagic detachment of left choroid (disorder)
        "341681000119105",  # Choroidal detachment of left eye (disorder)
        "343491000119109",  # Serous choroidal detachment of bilateral eyes (disorder)
        "346901000119101",  # Choroidal detachment of bilateral eyes (disorder)
        "778049009",  # Idiopathic uveal effusion syndrome (disorder)
        "78584008",  # Serous choroidal detachment (disorder)
    }

class ChoroidalHemorrhageAndRupture(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of choroidal hemorrhage and/or rupture.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of choroidal hemorrhage and rupture.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Choroidal Hemorrhage and Rupture"
    OID = "2.16.840.1.113883.3.526.3.1452"
    DEFINITION_VERSION = "20210210"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H31301",  # Unspecified choroidal hemorrhage, right eye
        "H31302",  # Unspecified choroidal hemorrhage, left eye
        "H31303",  # Unspecified choroidal hemorrhage, bilateral
        "H31311",  # Expulsive choroidal hemorrhage, right eye
        "H31312",  # Expulsive choroidal hemorrhage, left eye
        "H31313",  # Expulsive choroidal hemorrhage, bilateral
        "H31321",  # Choroidal rupture, right eye
        "H31322",  # Choroidal rupture, left eye
        "H31323",  # Choroidal rupture, bilateral
    }

    SNOMEDCT = {
        "11623000",  # Expulsive choroidal hemorrhage (disorder)
        "122003",  # Choroidal hemorrhage (disorder)
        "193474002",  # Choroidal hemorrhage and rupture (disorder)
        "2532009",  # Choroidal rupture (disorder)
        "335201000119104",  # Expulsive hemorrhage of choroid of right eye (disorder)
        "336051000119104",  # Rupture of choroid of right eye (disorder)
        "336061000119102",  # Hemorrhage of choroid of right eye (disorder)
        "340811000119106",  # Expulsive hemorrhage of choroid of left eye (disorder)
        "341661000119101",  # Rupture of choroid of left eye (disorder)
        "341671000119107",  # Hemorrhage of choroid of left eye (disorder)
        "346891000119100",  # Hemorrhage of choroid of bilateral eyes (disorder)
        "419596007",  # Suprachoroidal hemorrhage (disorder)
    }

class ChronicIridocyclitis(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of chronic iridocyclitis.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of chronic or recurrent iridocyclitis or tuberculous iridocyclitis.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Chronic Iridocyclitis"
    OID = "2.16.840.1.113883.3.526.3.1416"
    DEFINITION_VERSION = "20210210"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "A1854",  # Tuberculous iridocyclitis
        "H20021",  # Recurrent acute iridocyclitis, right eye
        "H20022",  # Recurrent acute iridocyclitis, left eye
        "H20023",  # Recurrent acute iridocyclitis, bilateral
        "H2011",  # Chronic iridocyclitis, right eye
        "H2012",  # Chronic iridocyclitis, left eye
        "H2013",  # Chronic iridocyclitis, bilateral
        "H209",  # Unspecified iridocyclitis
    }

    SNOMEDCT = {
        "186266008",  # Tuberculous chronic iridocyclitis (disorder)
        "332731000119108",  # Recurrent iridocyclitis of right eye (disorder)
        "338341000119103",  # Recurrent iridocyclitis of left eye (disorder)
        "344111000119109",  # Recurrent iridocyclitis of bilateral eyes (disorder)
        "416449002",  # Chronic iridocyclitis in young girls (disorder)
        "6869001",  # Recurrent iridocyclitis (disorder)
    }

class CloudyCornea(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of cloudy cornea.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of opacity of the cornea, adherent or corneal leukoma, and haziness of the cornea.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Cloudy Cornea"
    OID = "2.16.840.1.113883.3.526.3.1417"
    DEFINITION_VERSION = "20210210"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H1701",  # Adherent leukoma, right eye
        "H1702",  # Adherent leukoma, left eye
        "H1703",  # Adherent leukoma, bilateral
        "H1711",  # Central corneal opacity, right eye
        "H1712",  # Central corneal opacity, left eye
        "H1713",  # Central corneal opacity, bilateral
        "H17811",  # Minor opacity of cornea, right eye
        "H17812",  # Minor opacity of cornea, left eye
        "H17813",  # Minor opacity of cornea, bilateral
        "H17821",  # Peripheral opacity of cornea, right eye
        "H17822",  # Peripheral opacity of cornea, left eye
        "H17823",  # Peripheral opacity of cornea, bilateral
    }

    SNOMEDCT = {
        "1099031000119105",  # Opacity of cornea of bilateral eyes (disorder)
        "1099041000119101",  # Opacity of cornea of left eye (disorder)
        "1099051000119104",  # Opacity of cornea of right eye (disorder)
        "204149000",  # Congenital corneal opacity without visual deficit (disorder)
        "22135003",  # Corneal leukoma interfering with central vision (disorder)
        "30337003",  # Adherent leukoma (disorder)
        "333881000119109",  # Minor opacity of right cornea (disorder)
        "336501000119105",  # Adherent leukoma of right eye (disorder)
        "339481000119109",  # Minor opacity of left cornea (disorder)
        "342111000119101",  # Adherent leukoma of left eye (disorder)
        "344611000119101",  # Peripheral opacity of bilateral corneas (disorder)
        "347271000119100",  # Adherent leukoma of bilateral eyes (disorder)
        "413921009",  # Corneal subepithelial haze (disorder)
        "424353002",  # Corneal haze caused by herpes zoster (disorder)
        "425961009",  # Corneal haze caused by herpes simplex (disorder)
        "54359000",  # Congenital corneal opacity not interfering with vision (disorder)
        "55713007",  # Peripheral opacity of cornea (disorder)
        "56530002",  # Central opacity of cornea (disorder)
        "64634000",  # Corneal opacity (disorder)
        "66104007",  # Minor opacity of cornea (disorder)
        "72494008",  # Corneal nebula (disorder)
    }

class CornealEdema(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of corneal edema.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of corneal edema, including keratopathy.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Corneal Edema"
    OID = "2.16.840.1.113883.3.526.3.1418"
    DEFINITION_VERSION = "20210210"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H1811",  # Bullous keratopathy, right eye
        "H1812",  # Bullous keratopathy, left eye
        "H1813",  # Bullous keratopathy, bilateral
        "H1820",  # Unspecified corneal edema
        "H18221",  # Idiopathic corneal edema, right eye
        "H18222",  # Idiopathic corneal edema, left eye
        "H18223",  # Idiopathic corneal edema, bilateral
        "H18231",  # Secondary corneal edema, right eye
        "H18232",  # Secondary corneal edema, left eye
        "H18233",  # Secondary corneal edema, bilateral
        "H18421",  # Band keratopathy, right eye
        "H18422",  # Band keratopathy, left eye
        "H18423",  # Band keratopathy, bilateral
        "H1843",  # Other calcerous corneal degeneration
    }

    SNOMEDCT = {
        "129627002",  # Keratitis bullosa (disorder)
        "15736081000119108",  # Microcystic edema of cornea of left eye (disorder)
        "15736161000119100",  # Microcystic edema of cornea of right eye (disorder)
        "15736201000119105",  # Edema of cornea of left eye (disorder)
        "15736241000119107",  # Edema of cornea of bilateral eyes (disorder)
        "15736281000119102",  # Edema of cornea of right eye (disorder)
        "1794009",  # Idiopathic corneal edema (disorder)
        "231921008",  # Phakic corneal edema (disorder)
        "231922001",  # Pseudophakic corneal edema (disorder)
        "231923006",  # Aphakic corneal edema (disorder)
        "248503005",  # Fine corneal edema (disorder)
        "27194006",  # Corneal edema (disorder)
        "27536004",  # Secondary corneal edema (disorder)
        "336241000119105",  # Bullous keratopathy of right eye (disorder)
        "341851000119102",  # Bullous keratopathy of left eye (disorder)
        "347061000119104",  # Bullous keratopathy of bilateral eyes (disorder)
        "373428006",  # Corneal epithelial edema (finding)
        "373429003",  # Bullous corneal epithelial edema (finding)
        "373430008",  # Edema of corneal stroma (disorder)
        "49362009",  # Corneal edema caused by wearing of contact lenses (disorder)
        "57207003",  # Bullous keratopathy (disorder)
        "678991000119103",  # Pseudophakic bullous keratopathy of left eye (disorder)
        "679011000119100",  # Pseudophakic bullous keratopathy of right eye (disorder)
    }

class DegenerationOfMaculaAndPosteriorPole(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of degeneration of macula and posterior pole.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of macula and/or posterior pole degenerations, including age-related degenerations, macular cyst, maculopathy, macular puckering, macular hole, atrophy or dystrophy of macula.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Degeneration of Macula and Posterior Pole"
    OID = "2.16.840.1.113883.3.526.3.1453"
    DEFINITION_VERSION = "20210210"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H3530",  # Unspecified macular degeneration
        "H353110",  # Nonexudative age-related macular degeneration, right eye, stage unspecified
        "H353111",  # Nonexudative age-related macular degeneration, right eye, early dry stage
        "H353112",  # Nonexudative age-related macular degeneration, right eye, intermediate dry stage
        "H353113",  # Nonexudative age-related macular degeneration, right eye, advanced atrophic without subfoveal involvement
        "H353114",  # Nonexudative age-related macular degeneration, right eye, advanced atrophic with subfoveal involvement
        "H353120",  # Nonexudative age-related macular degeneration, left eye, stage unspecified
        "H353121",  # Nonexudative age-related macular degeneration, left eye, early dry stage
        "H353122",  # Nonexudative age-related macular degeneration, left eye, intermediate dry stage
        "H353123",  # Nonexudative age-related macular degeneration, left eye, advanced atrophic without subfoveal involvement
        "H353124",  # Nonexudative age-related macular degeneration, left eye, advanced atrophic with subfoveal involvement
        "H353130",  # Nonexudative age-related macular degeneration, bilateral, stage unspecified
        "H353131",  # Nonexudative age-related macular degeneration, bilateral, early dry stage
        "H353132",  # Nonexudative age-related macular degeneration, bilateral, intermediate dry stage
        "H353133",  # Nonexudative age-related macular degeneration, bilateral, advanced atrophic without subfoveal involvement
        "H353134",  # Nonexudative age-related macular degeneration, bilateral, advanced atrophic with subfoveal involvement
        "H353210",  # Exudative age-related macular degeneration, right eye, stage unspecified
        "H353211",  # Exudative age-related macular degeneration, right eye, with active choroidal neovascularization
        "H353212",  # Exudative age-related macular degeneration, right eye, with inactive choroidal neovascularization
        "H353213",  # Exudative age-related macular degeneration, right eye, with inactive scar
        "H353220",  # Exudative age-related macular degeneration, left eye, stage unspecified
        "H353221",  # Exudative age-related macular degeneration, left eye, with active choroidal neovascularization
        "H353222",  # Exudative age-related macular degeneration, left eye, with inactive choroidal neovascularization
        "H353223",  # Exudative age-related macular degeneration, left eye, with inactive scar
        "H353230",  # Exudative age-related macular degeneration, bilateral, stage unspecified
        "H353231",  # Exudative age-related macular degeneration, bilateral, with active choroidal neovascularization
        "H353232",  # Exudative age-related macular degeneration, bilateral, with inactive choroidal neovascularization
        "H353233",  # Exudative age-related macular degeneration, bilateral, with inactive scar
        "H35341",  # Macular cyst, hole, or pseudohole, right eye
        "H35342",  # Macular cyst, hole, or pseudohole, left eye
        "H35343",  # Macular cyst, hole, or pseudohole, bilateral
        "H35351",  # Cystoid macular degeneration, right eye
        "H35352",  # Cystoid macular degeneration, left eye
        "H35353",  # Cystoid macular degeneration, bilateral
        "H35361",  # Drusen (degenerative) of macula, right eye
        "H35362",  # Drusen (degenerative) of macula, left eye
        "H35363",  # Drusen (degenerative) of macula, bilateral
        "H35371",  # Puckering of macula, right eye
        "H35372",  # Puckering of macula, left eye
        "H35373",  # Puckering of macula, bilateral
        "H35381",  # Toxic maculopathy, right eye
        "H35382",  # Toxic maculopathy, left eye
        "H35383",  # Toxic maculopathy, bilateral
    }

    SNOMEDCT = {
        "1187639002",  # Martinique crinkled retinal pigment epitheliopathy (disorder)
        "1231253003",  # Macular hole due to degenerative progressive high myopia (disorder)
        "1285507004",  # Nonexudative neovascular age-related macular degeneration (disorder)
        "1285513008",  # Lamellar macular hole (disorder)
        "1285540008",  # Pachychoroid neovasculopathy (disorder)
        "1285589003",  # Nongeographic atrophy of macula lutea (disorder)
        "1296526000",  # Disciform macular degeneration (disorder)
        "1338026004",  # Disorder of macula of retina caused by radiation (disorder)
        "1338027008",  # Disorder of macula of retina due to and following radiotherapy (disorder)
        "14046000",  # Cystoid macular retinal degeneration (disorder)
        "15628531000119107",  # Fundus flavimaculatus of right eye (disorder)
        "15628571000119105",  # Fundus flavimaculatus of left eye (disorder)
        "15628611000119101",  # Fundus flavimaculatus of bilateral eyes (disorder)
        "15628771000119100",  # Forster-Fuchs spot of right eye (disorder)
        "15628811000119100",  # Forster-Fuchs spot of left eye (disorder)
        "15628851000119104",  # Forster-Fuchs spot of bilateral eyes (disorder)
        "15631531000119100",  # Scarred macula of bilateral eyes (disorder)
        "15631611000119106",  # Scarred macula of right eye (disorder)
        "15639131000119107",  # Macular edema of retina of right eye (disorder)
        "15639181000119108",  # Macular edema of retina of left eye (disorder)
        "15639231000119102",  # Macular edema of retina of bilateral eyes (disorder)
        "16320791000119106",  # Myopic macular degeneration of bilateral eyes (disorder)
        "16320831000119100",  # Myopic macular degeneration of right eye (disorder)
        "16320871000119102",  # Myopic macular degeneration of left eye (disorder)
        "18410006",  # Scarred macula (disorder)
        "193350004",  # Advanced maculopathy due to diabetes mellitus (disorder)
        "193387007",  # Cystoid macular edema (disorder)
        "193410003",  # Sorsby pseudoinflammatory fundus dystrophy (disorder)
        "232006002",  # Macular hole (disorder)
        "232013002",  # Macular retinoschisis (disorder)
        "232014008",  # Macular and peripheral retinoschisis (disorder)
        "232020009",  # Disorder of macula due to diabetes mellitus (disorder)
        "232049001",  # Adult vitelliform macular dystrophy (disorder)
        "232050001",  # Bull's eye macular dystrophy (disorder)
        "232051002",  # Pattern dystrophy of macula (disorder)
        "232071006",  # Occult neovascularization of macula (disorder)
        "232072004",  # Forster-Fuchs' spot (disorder)
        "247146001",  # Macular pigment deposit (disorder)
        "247147005",  # Macular diffuse atrophy (disorder)
        "247149008",  # Macular choroidal sclerosis (disorder)
        "247158001",  # Macular pigment epithelial detachment (disorder)
        "247159009",  # Macular retinal pigment epithelium tear (disorder)
        "247172005",  # Retinal macular break (disorder)
        "247200005",  # Choroidal atrophy (finding)
        "264633009",  # Drusen stage macular degeneration (disorder)
        "267612009",  # Degeneration of macula due to cyst, hole or pseudohole (disorder)
        "267718000",  # Age-related macular degeneration (disorder)
        "276436007",  # Hereditary macular dystrophy (disorder)
        "312701008",  # Impending macular hole (disorder)
        "312702001",  # Full thickness macular hole stage II (disorder)
        "312703006",  # Full thickness macular hole stage III (disorder)
        "312704000",  # Full thickness macular hole stage IV (disorder)
        "312898002",  # Myopic macular degeneration (disorder)
        "312899005",  # Fibrovascular macular scar (disorder)
        "312901001",  # Vitreomacular traction syndrome (disorder)
        "312902008",  # Macular pseudohole (disorder)
        "312911008",  # Clinically significant macular edema (disorder)
        "312912001",  # Macular edema due to diabetes mellitus (disorder)
        "312913006",  # Ischemic maculopathy (disorder)
        "312920004",  # Postoperative cystoid macular edema (disorder)
        "312921000",  # Autosomal dominant cystoid macular edema (disorder)
        "312922007",  # Uveitis related cystoid macular edema (disorder)
        "312925009",  # North Carolina macular dystrophy (disorder)
        "312999006",  # Disorder of macula of retina (disorder)
        "313002004",  # Acute macular neuroretinitis (disorder)
        "314010006",  # Diffuse exudative maculopathy due to diabetes mellitus (disorder)
        "314011005",  # Focal exudative maculopathy due to diabetes mellitus (disorder)
        "314014002",  # Ischemic maculopathy due to diabetes mellitus (disorder)
        "314015001",  # Mixed maculopathy due to diabetes mellitus (disorder)
        "314427006",  # Drusen plus pigment change stage macular degeneration (disorder)
        "331821000119106",  # Toxic disorder of macula lutea of right eye (disorder)
        "335531000119109",  # Cystoid macular degeneration of right retina (disorder)
        "337441000119109",  # Toxic disorder of macula lutea of left eye (disorder)
        "341141000119106",  # Cystoid macular degeneration of left retina (disorder)
        "343231000119106",  # Toxic maculopathy of bilateral eyes (disorder)
        "346401000119108",  # Cystoid macular degeneration of bilateral retinas (disorder)
        "37231002",  # Macular retinal edema (disorder)
        "399864000",  # Macular edema not clinically significant due to diabetes mellitus (disorder)
        "399872003",  # Severe nonproliferative retinopathy with clinically significant macular edema due to diabetes mellitus (disorder)
        "399874002",  # High risk proliferative retinopathy with clinically significant macula edema due to diabetes mellitus (disorder)
        "399875001",  # Non-high-risk proliferative retinopathy with clinically significant macular edema due to diabetes mellitus (disorder)
        "399877009",  # Very severe nonproliferative retinopathy with clinically significant macular edema due to diabetes mellitus (disorder)
        "4041005",  # Congenital anomaly of macula (disorder)
        "414173003",  # Exudative age-related macular degeneration (disorder)
        "414875008",  # Nonexudative age-related macular degeneration (disorder)
        "418918009",  # Macular infarction (disorder)
        "420030000",  # Serous macular detachment (disorder)
        "420486006",  # Exudative maculopathy due to type 1 diabetes mellitus (disorder)
        "421779007",  # Exudative maculopathy due to type 2 diabetes mellitus (disorder)
        "422338006",  # Degenerative disorder of macula (disorder)
        "423084001",  # Macular exudate (disorder)
        "425622004",  # Scar of macula following trauma (disorder)
        "429449002",  # Congenital hypoplasia of fovea centralis (disorder)
        "432789001",  # Noncystoid edema of macula of retina (disorder)
        "44115007",  # Toxic maculopathy (disorder)
        "443437004",  # Full thickness hole of macula lutea (disorder)
        "47673003",  # Fundus flavimaculatus (disorder)
        "677851000119101",  # Macular hole of bilateral eyes (disorder)
        "677861000119104",  # Full thickness macular hole of bilateral eyes (disorder)
        "677871000119105",  # Partial thickness macular hole of bilateral eyes (disorder)
        "677881000119108",  # Macular pseudohole of bilateral eyes (disorder)
        "677901000119105",  # Macular hole of left eye (disorder)
        "677911000119108",  # Full thickness macular hole of left eye (disorder)
        "677921000119101",  # Partial thickness macular hole of left eye (disorder)
        "677931000119103",  # Macular pseudohole of left eye (disorder)
        "677951000119109",  # Macular hole of right eye (disorder)
        "677961000119106",  # Full thickness macular hole of right eye (disorder)
        "677971000119100",  # Partial thickness macular hole of right eye (disorder)
        "677981000119102",  # Macular pseudohole of right eye (disorder)
        "70099003",  # Stargardt's disease (disorder)
        "725590001",  # Butterfly-shaped pigmentary macular dystrophy (disorder)
        "737273002",  # Degeneration of posterior pole of eye (disorder)
        "769244003",  # Disorder of right macula due to diabetes mellitus (disorder)
        "769245002",  # Disorder of left macula due to diabetes mellitus (disorder)
        "770097006",  # Clinically significant macular edema due to diabetes mellitus (disorder)
        "774010000",  # Scarred macula of left eye (disorder)
        "789220007",  # Atrophy of macula lutea (disorder)
        "860721006",  # Disorder of macula of bilateral eyes due to diabetes mellitus (disorder)
    }

class DegenerativeDisordersOfGlobe(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of degenerative disorders of globe.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of degenerative disorders of globe, including degenerative myopia, chalcosis, siderosis, and other degenerative diagnoses.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Degenerative Disorders of Globe"
    OID = "2.16.840.1.113883.3.526.3.1454"
    DEFINITION_VERSION = "20210214"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H4421",  # Degenerative myopia, right eye
        "H4422",  # Degenerative myopia, left eye
        "H4423",  # Degenerative myopia, bilateral
        "H442A1",  # Degenerative myopia with choroidal neovascularization, right eye
        "H442A2",  # Degenerative myopia with choroidal neovascularization, left eye
        "H442A3",  # Degenerative myopia with choroidal neovascularization, bilateral
        "H442B1",  # Degenerative myopia with macular hole, right eye
        "H442B2",  # Degenerative myopia with macular hole, left eye
        "H442B3",  # Degenerative myopia with macular hole, bilateral
        "H442C1",  # Degenerative myopia with retinal detachment, right eye
        "H442C2",  # Degenerative myopia with retinal detachment, left eye
        "H442C3",  # Degenerative myopia with retinal detachment, bilateral
        "H442D1",  # Degenerative myopia with foveoschisis, right eye
        "H442D2",  # Degenerative myopia with foveoschisis, left eye
        "H442D3",  # Degenerative myopia with foveoschisis, bilateral
        "H442E1",  # Degenerative myopia with other maculopathy, right eye
        "H442E2",  # Degenerative myopia with other maculopathy, left eye
        "H442E3",  # Degenerative myopia with other maculopathy, bilateral
        "H44311",  # Chalcosis, right eye
        "H44312",  # Chalcosis, left eye
        "H44313",  # Chalcosis, bilateral
        "H44321",  # Siderosis of eye, right eye
        "H44322",  # Siderosis of eye, left eye
        "H44323",  # Siderosis of eye, bilateral
        "H44391",  # Other degenerative disorders of globe, right eye
        "H44392",  # Other degenerative disorders of globe, left eye
        "H44393",  # Other degenerative disorders of globe, bilateral
    }

    SNOMEDCT = {
        "111521006",  # Corneal degeneration (disorder)
        "1208890001",  # Advanced exudative age related macular degeneration (disorder)
        "1208895006",  # Advanced age related macular degeneration (disorder)
        "1285507004",  # Nonexudative neovascular age-related macular degeneration (disorder)
        "1285555004",  # Peripheral exudative hemorrhagic chorioretinopathy (disorder)
        "1296526000",  # Disciform macular degeneration (disorder)
        "14046000",  # Cystoid macular retinal degeneration (disorder)
        "15628651000119100",  # Sorsby pseudoinflammatory fundus dystrophy of right eye (disorder)
        "15628691000119105",  # Sorsby pseudoinflammatory fundus dystrophy of left eye (disorder)
        "15628731000119103",  # Sorsby pseudoinflammatory fundus dystrophy of bilateral eyes (disorder)
        "15632931000119101",  # Bilateral age-related nonexudative macular degeneration (disorder)
        "15632971000119103",  # Age-related nonexudative macular degeneration of left eye (disorder)
        "15633011000119103",  # Age-related nonexudative macular degeneration of right eye (disorder)
        "15633051000119102",  # Age-related exudative macular degeneration of right eye (disorder)
        "15633091000119107",  # Age-related exudative macular degeneration of left eye (disorder)
        "15732881000119102",  # Generalized degeneration of retina of bilateral eyes (disorder)
        "15732921000119109",  # Generalized degeneration of retina of left eye (disorder)
        "15732961000119104",  # Generalized degeneration of retina of right eye (disorder)
        "15733361000119105",  # Peripheral retinal degeneration of left eye (disorder)
        "15733401000119101",  # Peripheral retinal degeneration of right eye (disorder)
        "15735721000119100",  # Degeneration of cornea of left eye (disorder)
        "15735801000119102",  # Degeneration of cornea of bilateral eyes (disorder)
        "16419701000119108",  # Vitreoretinal tuft of bilateral eyes (disorder)
        "16419741000119105",  # Vitreoretinal tuft of right eye (disorder)
        "16419781000119100",  # Vitreoretinal tuft of left eye (disorder)
        "193410003",  # Sorsby pseudoinflammatory fundus dystrophy (disorder)
        "193508003",  # Ciliary body degenerative changes (disorder)
        "231918006",  # Chalcosis of cornea (disorder)
        "231919003",  # Siderosis of cornea (disorder)
        "231927007",  # Fatty degeneration of cornea (disorder)
        "231929005",  # Corneal epithelial degeneration (disorder)
        "231944006",  # Labrador keratopathy (disorder)
        "232001007",  # Snail-track retinal degeneration (disorder)
        "232002000",  # Snowflake retinal degeneration (disorder)
        "232065000",  # Goldmann-Favre syndrome (disorder)
        "232076001",  # Amyloid of vitreous (disorder)
        "24210004",  # Congenital chorioretinal degeneration (disorder)
        "246877005",  # Bulbar conjunctival degeneration (disorder)
        "246937001",  # Microcystoid epithelial degeneration of cornea (disorder)
        "247177004",  # Chorioretinal degeneration (disorder)
        "247182006",  # Vitreoretinal degeneration (disorder)
        "247183001",  # Peripheral snowflake retinal degeneration (disorder)
        "25277000",  # Siderosis of eye (disorder)
        "264008",  # Blind hypertensive eye (disorder)
        "264633009",  # Drusen stage macular degeneration (disorder)
        "267612009",  # Degeneration of macula due to cyst, hole or pseudohole (disorder)
        "267718000",  # Age-related macular degeneration (disorder)
        "27971006",  # Terrien's marginal degeneration of cornea (disorder)
        "28143002",  # Phthisical cornea (disorder)
        "314010006",  # Diffuse exudative maculopathy due to diabetes mellitus (disorder)
        "314427006",  # Drusen plus pigment change stage macular degeneration (disorder)
        "331381000119100",  # Vitreous degeneration of right eye (disorder)
        "332441000119107",  # Paving stone retinal degeneration of right eye (disorder)
        "332481000119102",  # Lattice degeneration of right retina (disorder)
        "333241000119103",  # Degeneration of peripheral cornea of right eye (disorder)
        "333591000119109",  # Siderosis of right eye (disorder)
        "333661000119101",  # Nodular degeneration of right cornea (disorder)
        "335531000119109",  # Cystoid macular degeneration of right retina (disorder)
        "335881000119108",  # Degeneration of ciliary body of right eye (disorder)
        "336991000119108",  # Vitreous degeneration of left eye (disorder)
        "338051000119103",  # Paving stone retinal degeneration of left eye (disorder)
        "338091000119108",  # Lattice degeneration of left retina (disorder)
        "338851000119105",  # Degeneration of peripheral cornea of left eye (disorder)
        "339201000119102",  # Siderosis of left eye (disorder)
        "339261000119101",  # Nodular degeneration of left cornea (disorder)
        "341141000119106",  # Cystoid macular degeneration of left retina (disorder)
        "341491000119105",  # Degeneration of ciliary body of left eye (disorder)
        "343101000119108",  # Vitreous degeneration of bilateral eyes (disorder)
        "343821000119109",  # Paving stone retinal degeneration of bilateral eyes (disorder)
        "343841000119103",  # Microcystoid retinal degeneration of bilateral eyes (disorder)
        "343861000119104",  # Lattice degeneration of bilateral retinas (disorder)
        "344621000119108",  # Degeneration of peripheral cornea of bilateral eyes (disorder)
        "344951000119107",  # Siderosis of bilateral eyes (disorder)
        "345021000119104",  # Nodular degeneration of bilateral corneas (disorder)
        "346401000119108",  # Cystoid macular degeneration of bilateral retinas (disorder)
        "346741000119102",  # Degeneration of ciliary body of bilateral eyes (disorder)
        "347251000119109",  # Senile reticular retinal degeneration of bilateral eyes (disorder)
        "348071000119106",  # Posterior vitreous degeneration of right eye (disorder)
        "348521000119104",  # Posterior vitreous degeneration of left eye (disorder)
        "349111000119108",  # Posterior vitreous degeneration of bilateral eyes (disorder)
        "35545005",  # Secondary pigmentary retinal degeneration (disorder)
        "3577000",  # Retinal lattice degeneration (disorder)
        "36524008",  # Hyaline dystrophy of Bruch's membrane (disorder)
        "37075008",  # Microcystoid retinal degeneration (disorder)
        "40787005",  # Conjunctival degeneration (disorder)
        "414173003",  # Exudative age-related macular degeneration (disorder)
        "414875008",  # Nonexudative age-related macular degeneration (disorder)
        "417939004",  # Amyloid corneal degeneration (disorder)
        "418871005",  # Spheroidal degeneration of conjunctiva (disorder)
        "419051009",  # Polymorphic amyloid degeneration of cornea (disorder)
        "419648001",  # Senile furrow degeneration of cornea (disorder)
        "424358006",  # Vitreoretinal tuft (disorder)
        "428212003",  # Photoreceptor degeneration (disorder)
        "450921000124103",  # Degeneration of cornea of right eye (disorder)
        "54184008",  # Senile reticular retinal degeneration (disorder)
        "60189009",  # Vitreous degeneration (disorder)
        "61536007",  # Peripheral retinal degeneration (disorder)
        "62585004",  # Degenerative disorder of eye (disorder)
        "679171000119109",  # Salzmann's nodular degeneration of cornea of bilateral eyes (disorder)
        "679181000119107",  # Salzmann nodular degeneration of cornea of left eye (disorder)
        "679191000119105",  # Salzmann nodular degeneration of cornea of right eye (disorder)
        "679201000119108",  # Pellucid marginal degeneration of cornea of bilateral eyes (disorder)
        "679211000119106",  # Terrien's marginal degeneration of peripheral cornea of bilateral eyes (disorder)
        "679221000119104",  # Pellucid marginal degeneration of cornea of left eye (disorder)
        "679231000119101",  # Terrien's marginal degeneration of peripheral cornea of left eye (disorder)
        "679241000119105",  # Pellucid marginal degeneration of cornea of right eye (disorder)
        "679251000119107",  # Terrien's marginal degeneration of peripheral cornea of right eye (disorder)
        "69134001",  # Paving stone retinal degeneration (disorder)
        "72620002",  # Nodular degeneration of cornea (disorder)
        "737271000",  # Subconjunctival degeneration (disorder)
        "770791000",  # Autosomal dominant neovascular inflammatory vitreoretinopathy (disorder)
        "83231000119101",  # Optic nerve and photoreceptor degeneration (disorder)
        "88538005",  # Calcareous degeneration of cornea (disorder)
        "89182000",  # Peripheral degeneration of cornea (disorder)
        "95695004",  # Degeneration of retina (disorder)
        "95696003",  # Glaucomatous retinal degeneration (disorder)
        "95697007",  # Generalized retinal degeneration (disorder)
        "95699005",  # Multifocal retinal degeneration (disorder)
        "95700006",  # Sudden acquired retinal degeneration (disorder)
    }

class DiabeticMacularEdema(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of diabetic macular edema.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of diabetic macular edema or diabetic retinopathy with macular edema.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Diabetic Macular Edema"
    OID = "2.16.840.1.113883.3.526.3.1455"
    DEFINITION_VERSION = "20210214"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "E08311",  # Diabetes mellitus due to underlying condition with unspecified diabetic retinopathy with macular edema
        "E083211",  # Diabetes mellitus due to underlying condition with mild nonproliferative diabetic retinopathy with macular edema, right eye
        "E083212",  # Diabetes mellitus due to underlying condition with mild nonproliferative diabetic retinopathy with macular edema, left eye
        "E083213",  # Diabetes mellitus due to underlying condition with mild nonproliferative diabetic retinopathy with macular edema, bilateral
        "E083311",  # Diabetes mellitus due to underlying condition with moderate nonproliferative diabetic retinopathy with macular edema, right eye
        "E083312",  # Diabetes mellitus due to underlying condition with moderate nonproliferative diabetic retinopathy with macular edema, left eye
        "E083313",  # Diabetes mellitus due to underlying condition with moderate nonproliferative diabetic retinopathy with macular edema, bilateral
        "E083411",  # Diabetes mellitus due to underlying condition with severe nonproliferative diabetic retinopathy with macular edema, right eye
        "E083412",  # Diabetes mellitus due to underlying condition with severe nonproliferative diabetic retinopathy with macular edema, left eye
        "E083413",  # Diabetes mellitus due to underlying condition with severe nonproliferative diabetic retinopathy with macular edema, bilateral
        "E083511",  # Diabetes mellitus due to underlying condition with proliferative diabetic retinopathy with macular edema, right eye
        "E083512",  # Diabetes mellitus due to underlying condition with proliferative diabetic retinopathy with macular edema, left eye
        "E083513",  # Diabetes mellitus due to underlying condition with proliferative diabetic retinopathy with macular edema, bilateral
        "E083521",  # Diabetes mellitus due to underlying condition with proliferative diabetic retinopathy with traction retinal detachment involving the macula, right eye
        "E083522",  # Diabetes mellitus due to underlying condition with proliferative diabetic retinopathy with traction retinal detachment involving the macula, left eye
        "E083523",  # Diabetes mellitus due to underlying condition with proliferative diabetic retinopathy with traction retinal detachment involving the macula, bilateral
        "E0837X1",  # Diabetes mellitus due to underlying condition with diabetic macular edema, resolved following treatment, right eye
        "E0837X2",  # Diabetes mellitus due to underlying condition with diabetic macular edema, resolved following treatment, left eye
        "E0837X3",  # Diabetes mellitus due to underlying condition with diabetic macular edema, resolved following treatment, bilateral
        "E09311",  # Drug or chemical induced diabetes mellitus with unspecified diabetic retinopathy with macular edema
        "E093211",  # Drug or chemical induced diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, right eye
        "E093212",  # Drug or chemical induced diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, left eye
        "E093213",  # Drug or chemical induced diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, bilateral
        "E093311",  # Drug or chemical induced diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, right eye
        "E093312",  # Drug or chemical induced diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, left eye
        "E093313",  # Drug or chemical induced diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, bilateral
        "E093411",  # Drug or chemical induced diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, right eye
        "E093412",  # Drug or chemical induced diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, left eye
        "E093413",  # Drug or chemical induced diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, bilateral
        "E093511",  # Drug or chemical induced diabetes mellitus with proliferative diabetic retinopathy with macular edema, right eye
        "E093512",  # Drug or chemical induced diabetes mellitus with proliferative diabetic retinopathy with macular edema, left eye
        "E093513",  # Drug or chemical induced diabetes mellitus with proliferative diabetic retinopathy with macular edema, bilateral
        "E093521",  # Drug or chemical induced diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, right eye
        "E093522",  # Drug or chemical induced diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, left eye
        "E093523",  # Drug or chemical induced diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, bilateral
        "E0937X1",  # Drug or chemical induced diabetes mellitus with diabetic macular edema, resolved following treatment, right eye
        "E0937X2",  # Drug or chemical induced diabetes mellitus with diabetic macular edema, resolved following treatment, left eye
        "E0937X3",  # Drug or chemical induced diabetes mellitus with diabetic macular edema, resolved following treatment, bilateral
        "E10311",  # Type 1 diabetes mellitus with unspecified diabetic retinopathy with macular edema
        "E103211",  # Type 1 diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, right eye
        "E103212",  # Type 1 diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, left eye
        "E103213",  # Type 1 diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, bilateral
        "E103311",  # Type 1 diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, right eye
        "E103312",  # Type 1 diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, left eye
        "E103313",  # Type 1 diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, bilateral
        "E103411",  # Type 1 diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, right eye
        "E103412",  # Type 1 diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, left eye
        "E103413",  # Type 1 diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, bilateral
        "E103511",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with macular edema, right eye
        "E103512",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with macular edema, left eye
        "E103513",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with macular edema, bilateral
        "E103521",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, right eye
        "E103522",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, left eye
        "E103523",  # Type 1 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, bilateral
        "E1037X1",  # Type 1 diabetes mellitus with diabetic macular edema, resolved following treatment, right eye
        "E1037X2",  # Type 1 diabetes mellitus with diabetic macular edema, resolved following treatment, left eye
        "E1037X3",  # Type 1 diabetes mellitus with diabetic macular edema, resolved following treatment, bilateral
        "E11311",  # Type 2 diabetes mellitus with unspecified diabetic retinopathy with macular edema
        "E113211",  # Type 2 diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, right eye
        "E113212",  # Type 2 diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, left eye
        "E113213",  # Type 2 diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, bilateral
        "E113311",  # Type 2 diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, right eye
        "E113312",  # Type 2 diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, left eye
        "E113313",  # Type 2 diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, bilateral
        "E113411",  # Type 2 diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, right eye
        "E113412",  # Type 2 diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, left eye
        "E113413",  # Type 2 diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, bilateral
        "E113511",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with macular edema, right eye
        "E113512",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with macular edema, left eye
        "E113513",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with macular edema, bilateral
        "E113521",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, right eye
        "E113522",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, left eye
        "E113523",  # Type 2 diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, bilateral
        "E1137X1",  # Type 2 diabetes mellitus with diabetic macular edema, resolved following treatment, right eye
        "E1137X2",  # Type 2 diabetes mellitus with diabetic macular edema, resolved following treatment, left eye
        "E1137X3",  # Type 2 diabetes mellitus with diabetic macular edema, resolved following treatment, bilateral
        "E13311",  # Other specified diabetes mellitus with unspecified diabetic retinopathy with macular edema
        "E133211",  # Other specified diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, right eye
        "E133212",  # Other specified diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, left eye
        "E133213",  # Other specified diabetes mellitus with mild nonproliferative diabetic retinopathy with macular edema, bilateral
        "E133311",  # Other specified diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, right eye
        "E133312",  # Other specified diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, left eye
        "E133313",  # Other specified diabetes mellitus with moderate nonproliferative diabetic retinopathy with macular edema, bilateral
        "E133411",  # Other specified diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, right eye
        "E133412",  # Other specified diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, left eye
        "E133413",  # Other specified diabetes mellitus with severe nonproliferative diabetic retinopathy with macular edema, bilateral
        "E133511",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with macular edema, right eye
        "E133512",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with macular edema, left eye
        "E133513",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with macular edema, bilateral
        "E133521",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, right eye
        "E133522",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, left eye
        "E133523",  # Other specified diabetes mellitus with proliferative diabetic retinopathy with traction retinal detachment involving the macula, bilateral
        "E1337X1",  # Other specified diabetes mellitus with diabetic macular edema, resolved following treatment, right eye
        "E1337X2",  # Other specified diabetes mellitus with diabetic macular edema, resolved following treatment, left eye
        "E1337X3",  # Other specified diabetes mellitus with diabetic macular edema, resolved following treatment, bilateral
    }

    SNOMEDCT = {
        "312912001",  # Macular edema due to diabetes mellitus (disorder)
        "399864000",  # Macular edema not clinically significant due to diabetes mellitus (disorder)
        "399877009",  # Very severe nonproliferative retinopathy with clinically significant macular edema due to diabetes mellitus (disorder)
        "769217008",  # Macular edema of right eye due to diabetes mellitus (disorder)
        "769218003",  # Macular edema of left eye due to diabetes mellitus (disorder)
        "769219006",  # Macular edema due to type 1 diabetes mellitus (disorder)
        "769220000",  # Macular edema due to type 2 diabetes mellitus (disorder)
        "769221001",  # Clinically significant macular edema of right eye due to diabetes mellitus (disorder)
        "769222008",  # Clinically significant macular edema of left eye due to diabetes mellitus (disorder)
        "770097006",  # Clinically significant macular edema due to diabetes mellitus (disorder)
        "870421009",  # Cystoid macular edema due to diabetes mellitus (disorder)
        "870529009",  # Persistent macular edema due to diabetes mellitus (disorder)
        "871778008",  # Centrally involved macular edema due to diabetes mellitus (disorder)
        "871781003",  # Non centrally involved macular edema due to diabetes mellitus (disorder)
        "97331000119101",  # Macular edema and retinopathy due to type 2 diabetes mellitus (disorder)
    }

class DisordersOfCorneaIncludingCornealOpacity(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of corneal opacity and other corneal disorders.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of opacity of the cornea, adherent or corneal leukoma, scarring of the cornea, and corneal haze.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Disorders of Cornea Including Corneal Opacity"
    OID = "2.16.840.1.113883.3.526.3.1419"
    DEFINITION_VERSION = "20210210"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H1701",  # Adherent leukoma, right eye
        "H1702",  # Adherent leukoma, left eye
        "H1703",  # Adherent leukoma, bilateral
        "H1711",  # Central corneal opacity, right eye
        "H1712",  # Central corneal opacity, left eye
        "H1713",  # Central corneal opacity, bilateral
        "H1789",  # Other corneal scars and opacities
        "H179",  # Unspecified corneal scar and opacity
    }

    SNOMEDCT = {
        "1003921006",  # Sclerocornea of bilateral corneas (disorder)
        "1099031000119105",  # Opacity of cornea of bilateral eyes (disorder)
        "1099041000119101",  # Opacity of cornea of left eye (disorder)
        "1099051000119104",  # Opacity of cornea of right eye (disorder)
        "1144352007",  # Scar of cornea due to and following herpes zoster infection (disorder)
        "1217688006",  # Congenital corneal leukoma (disorder)
        "1231682009",  # Opacity of cornea due to and following traumatic injury (disorder)
        "13282008",  # Congenital corneal opacity interfering with vision (disorder)
        "15987391000119105",  # Congenital opacity of cornea of left eye (disorder)
        "15987431000119100",  # Congenital opacity of cornea of right eye (disorder)
        "15999271000119108",  # Scar of cornea of right eye (disorder)
        "15999311000119108",  # Scar of cornea of bilateral eyes (disorder)
        "15999351000119109",  # Scar of cornea of left eye (disorder)
        "16350000",  # Corneal macula interfering with central vision (disorder)
        "193795008",  # Corneal scars and opacities (disorder)
        "204148008",  # Congenital corneal opacity with visual deficit (disorder)
        "204149000",  # Congenital corneal opacity without visual deficit (disorder)
        "22135003",  # Corneal leukoma interfering with central vision (disorder)
        "246967007",  # Central posterior corneal opacity (finding)
        "268160006",  # Congenital corneal opacity (disorder)
        "30337003",  # Adherent leukoma (disorder)
        "314005007",  # Visually significant corneal scar (disorder)
        "333231000119107",  # Peripheral opacity of right cornea (disorder)
        "333881000119109",  # Minor opacity of right cornea (disorder)
        "336221000119104",  # Central opacity of right cornea (disorder)
        "336501000119105",  # Adherent leukoma of right eye (disorder)
        "338841000119108",  # Peripheral opacity of left cornea (disorder)
        "339481000119109",  # Minor opacity of left cornea (disorder)
        "341831000119108",  # Central opacity of left cornea (disorder)
        "342111000119101",  # Adherent leukoma of left eye (disorder)
        "344611000119101",  # Peripheral opacity of bilateral corneas (disorder)
        "347271000119100",  # Adherent leukoma of bilateral eyes (disorder)
        "370505008",  # Corneal facet (disorder)
        "413921009",  # Corneal subepithelial haze (disorder)
        "419496009",  # Congenital corneal keloid (disorder)
        "422947009",  # Corneal subepithelial scar (disorder)
        "424353002",  # Corneal haze caused by herpes zoster (disorder)
        "425961009",  # Corneal haze caused by herpes simplex (disorder)
        "54359000",  # Congenital corneal opacity not interfering with vision (disorder)
        "55713007",  # Peripheral opacity of cornea (disorder)
        "56530002",  # Central opacity of cornea (disorder)
        "64634000",  # Corneal opacity (disorder)
        "66104007",  # Minor opacity of cornea (disorder)
        "675331000119106",  # Leukoma of cornea of bilateral eyes (disorder)
        "675341000119102",  # Leukoma of cornea of left eye (disorder)
        "675351000119100",  # Leukoma of cornea of right eye (disorder)
        "704272003",  # Corneal leukoma (disorder)
        "72494008",  # Corneal nebula (disorder)
        "840489001",  # Congenital opacity of bilateral corneas (disorder)
        "95479005",  # Congenital sclerocornea (disorder)
        "95726001",  # Corneal scar (disorder)
    }

class DisordersOfOpticChiasm(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of disorders of optic chiasm.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of optic chiasm disorders or injuries or optic glioma.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Disorders of Optic Chiasm"
    OID = "2.16.840.1.113883.3.526.3.1457"
    DEFINITION_VERSION = "20210214"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H4741",  # Disorders of optic chiasm in (due to) inflammatory disorders
        "H4742",  # Disorders of optic chiasm in (due to) neoplasm
        "H4743",  # Disorders of optic chiasm in (due to) vascular disorders
        "H4749",  # Disorders of optic chiasm in (due to) other disorders
    }

    SNOMEDCT = {
        "194057002",  # Disorder of optic chiasm due to pituitary disorder (disorder)
        "194058007",  # Disorder of optic chiasm due to non-pituitary neoplasm (disorder)
        "194059004",  # Disorder of optic chiasm due to vascular disorder (disorder)
        "194060009",  # Inflammatory disorder of optic chiasm (disorder)
        "230517004",  # Compression of optic chiasm (disorder)
        "302902003",  # Infarction of optic chiasm (disorder)
        "404653000",  # Chiasmal glioma (disorder)
        "404664002",  # Malignant optic glioma (disorder)
        "431769004",  # Disorder of optic chiasm due to neoplasm (disorder)
        "64246009",  # Disorder of optic chiasm associated with non-pituitary neoplasm (disorder)
        "703429003",  # Malignant optic glioma of adulthood (disorder)
        "70476006",  # Optic chiasm disorder (disorder)
        "89654006",  # Disorder of optic chiasm associated with vascular disorder (disorder)
    }

class DisordersOfVisualCortex(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of disorders of visual cortex.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of a disorder of visual cortex, including inflammatory disorders, neoplasm, and cortical blindness.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Disorders of Visual Cortex"
    OID = "2.16.840.1.113883.3.526.3.1458"
    DEFINITION_VERSION = "20210214"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H47611",  # Cortical blindness, right side of brain
        "H47612",  # Cortical blindness, left side of brain
        "H47621",  # Disorders of visual cortex in (due to) inflammatory disorders, right side of brain
        "H47622",  # Disorders of visual cortex in (due to) inflammatory disorders, left side of brain
        "H47631",  # Disorders of visual cortex in (due to) neoplasm, right side of brain
        "H47632",  # Disorders of visual cortex in (due to) neoplasm, left side of brain
        "H47641",  # Disorders of visual cortex in (due to) vascular disorders, right side of brain
        "H47642",  # Disorders of visual cortex in (due to) vascular disorders, left side of brain
    }

    SNOMEDCT = {
        "1172900005",  # Progressive microcephaly, seizures, cortical blindness, developmental delay syndrome (disorder)
        "128329001",  # Disorder of visual cortex (disorder)
        "15913031000119100",  # Cortical blindness of bilateral sides of brain (disorder)
        "194068002",  # Visual cortex disorder due to neoplasm (disorder)
        "194069005",  # Visual cortex disorder due to vascular disorder (disorder)
        "194070006",  # Inflammatory disorder of visual cortex (disorder)
        "230526001",  # Compression of visual cortex (disorder)
        "302904002",  # Infarction of visual cortex (disorder)
        "342741000119103",  # Cortical blindness of right side of brain (disorder)
        "342751000119101",  # Cortical blindness of left side of brain (disorder)
        "39194005",  # Visual epilepsy (disorder)
        "413924001",  # Cortical visual impairment (disorder)
        "68574006",  # Cortical blindness (disorder)
        "732251003",  # Cortical blindness, intellectual disability, polydactyly syndrome (disorder)
        "773548008",  # Early-onset epileptic encephalopathy, cortical blindness, intellectual disability, facial dysmorphism syndrome (disorder)
    }

class DisseminatedChorioretinitisAndDisseminatedRetinochoroiditis(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of disseminated chorioretinitis and/or retinochoroiditis.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of disseminated chorioretinal inflammation, disseminated retinochoroiditis, and placoid pigment epitheliopathy.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Disseminated Chorioretinitis and Disseminated Retinochoroiditis"
    OID = "2.16.840.1.113883.3.526.3.1459"
    DEFINITION_VERSION = "20210214"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "A1853",  # Tuberculous chorioretinitis
        "H30101",  # Unspecified disseminated chorioretinal inflammation, right eye
        "H30102",  # Unspecified disseminated chorioretinal inflammation, left eye
        "H30103",  # Unspecified disseminated chorioretinal inflammation, bilateral
        "H30111",  # Disseminated chorioretinal inflammation of posterior pole, right eye
        "H30112",  # Disseminated chorioretinal inflammation of posterior pole, left eye
        "H30113",  # Disseminated chorioretinal inflammation of posterior pole, bilateral
        "H30121",  # Disseminated chorioretinal inflammation, peripheral right eye
        "H30122",  # Disseminated chorioretinal inflammation, peripheral, left eye
        "H30123",  # Disseminated chorioretinal inflammation, peripheral, bilateral
        "H30131",  # Disseminated chorioretinal inflammation, generalized, right eye
        "H30132",  # Disseminated chorioretinal inflammation, generalized, left eye
        "H30133",  # Disseminated chorioretinal inflammation, generalized, bilateral
        "H30141",  # Acute posterior multifocal placoid pigment epitheliopathy, right eye
        "H30142",  # Acute posterior multifocal placoid pigment epitheliopathy, left eye
        "H30143",  # Acute posterior multifocal placoid pigment epitheliopathy, bilateral
    }

    SNOMEDCT = {
        "14894002",  # Generalized disseminated choroiditis AND chorioretinitis (disorder)
        "193442009",  # Disseminated posterior pole chorioretinitis (disorder)
        "193443004",  # Disseminated peripheral chorioretinitis (disorder)
        "193444005",  # General disseminated chorioretinitis (disorder)
        "371099005",  # Generalized disseminated choroiditis AND chorioretinitis with acute necrosis (disorder)
        "48225000",  # Tuberculous disseminated chorioretinitis (disorder)
        "51579006",  # Disseminated choroiditis AND chorioretinitis, posterior pole (disorder)
        "61517002",  # Metastatic disseminated retinitis AND retinochoroiditis (disorder)
        "69811008",  # Peripheral disseminated choroiditis AND chorioretinitis (disorder)
        "77939001",  # Syphilitic disseminated retinochoroiditis (disorder)
        "78769001",  # Disseminated chorioretinitis (disorder)
    }

class FocalChorioretinitisAndFocalRetinochoroiditis(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of focal chorioretinitis and/or retinochoroiditis.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of focal chorioretinitis, choroiditis, retinochoroiditis and retinitis.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Focal Chorioretinitis and Focal Retinochoroiditis"
    OID = "2.16.840.1.113883.3.526.3.1460"
    DEFINITION_VERSION = "20210214"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H30001",  # Unspecified focal chorioretinal inflammation, right eye
        "H30002",  # Unspecified focal chorioretinal inflammation, left eye
        "H30003",  # Unspecified focal chorioretinal inflammation, bilateral
        "H30011",  # Focal chorioretinal inflammation, juxtapapillary, right eye
        "H30012",  # Focal chorioretinal inflammation, juxtapapillary, left eye
        "H30013",  # Focal chorioretinal inflammation, juxtapapillary, bilateral
        "H30021",  # Focal chorioretinal inflammation of posterior pole, right eye
        "H30022",  # Focal chorioretinal inflammation of posterior pole, left eye
        "H30023",  # Focal chorioretinal inflammation of posterior pole, bilateral
        "H30031",  # Focal chorioretinal inflammation, peripheral, right eye
        "H30032",  # Focal chorioretinal inflammation, peripheral, left eye
        "H30033",  # Focal chorioretinal inflammation, peripheral, bilateral
        "H30041",  # Focal chorioretinal inflammation, macular or paramacular, right eye
        "H30042",  # Focal chorioretinal inflammation, macular or paramacular, left eye
        "H30043",  # Focal chorioretinal inflammation, macular or paramacular, bilateral
    }

    SNOMEDCT = {
        "15847003",  # Focal chorioretinitis (disorder)
        "193434008",  # Focal juxtapapillary choroiditis (disorder)
        "193436005",  # Juxtapapillary focal chorioretinitis (disorder)
        "193438006",  # Peripheral focal retinochoroiditis (disorder)
        "333011000119108",  # Focal chorioretinitis of posterior pole of right eye (disorder)
        "333201000119100",  # Peripheral focal chorioretinitis of right eye (disorder)
        "334641000119103",  # Juxtapapillary focal chorioretinitis of right eye (disorder)
        "338621000119101",  # Focal chorioretinitis of posterior pole of left eye (disorder)
        "338811000119109",  # Peripheral focal chorioretinitis of left eye (disorder)
        "340241000119106",  # Juxtapapillary focal chorioretinitis of left eye (disorder)
        "344391000119103",  # Focal chorioretinitis of posterior pole of bilateral eyes (disorder)
        "344581000119108",  # Peripheral focal chorioretinitis of bilateral eyes (disorder)
        "345601000119100",  # Juxtapapillary focal chorioretinitis of bilateral eyes (disorder)
        "36535003",  # Peripheral focal retinitis AND retinochoroiditis (disorder)
        "373667006",  # Focal chorioretinitis of posterior pole of eye (disorder)
        "42031001",  # Macular focal retinitis AND retinochoroiditis (disorder)
        "46497009",  # Juxtapapillary focal retinitis AND retinochoroiditis (disorder)
        "56787009",  # Peripheral focal choroiditis AND chorioretinitis (disorder)
        "6313001",  # Juxtapapillary focal choroiditis AND chorioretinitis (disorder)
        "699515009",  # Focal retinitis of posterior pole of eye (disorder)
        "699763005",  # Focal choroiditis of posterior pole of eye (disorder)
        "88290000",  # Focal chorioretinitis due to acquired disseminated toxoplasmosis (disorder)
    }

class Glaucoma(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of diagnoses for glaucoma.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** This value set may use a model element related to Diagnosis.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Glaucoma"
    OID = "2.16.840.1.113883.3.526.3.1423"
    DEFINITION_VERSION = "20210218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H4010X0",  # Unspecified open-angle glaucoma, stage unspecified
        "H4010X1",  # Unspecified open-angle glaucoma, mild stage
        "H4010X2",  # Unspecified open-angle glaucoma, moderate stage
        "H4010X3",  # Unspecified open-angle glaucoma, severe stage
        "H4010X4",  # Unspecified open-angle glaucoma, indeterminate stage
        "H401110",  # Primary open-angle glaucoma, right eye, stage unspecified
        "H401111",  # Primary open-angle glaucoma, right eye, mild stage
        "H401112",  # Primary open-angle glaucoma, right eye, moderate stage
        "H401113",  # Primary open-angle glaucoma, right eye, severe stage
        "H401114",  # Primary open-angle glaucoma, right eye, indeterminate stage
        "H401120",  # Primary open-angle glaucoma, left eye, stage unspecified
        "H401121",  # Primary open-angle glaucoma, left eye, mild stage
        "H401122",  # Primary open-angle glaucoma, left eye, moderate stage
        "H401123",  # Primary open-angle glaucoma, left eye, severe stage
        "H401124",  # Primary open-angle glaucoma, left eye, indeterminate stage
        "H401130",  # Primary open-angle glaucoma, bilateral, stage unspecified
        "H401131",  # Primary open-angle glaucoma, bilateral, mild stage
        "H401132",  # Primary open-angle glaucoma, bilateral, moderate stage
        "H401133",  # Primary open-angle glaucoma, bilateral, severe stage
        "H401134",  # Primary open-angle glaucoma, bilateral, indeterminate stage
        "H401210",  # Low-tension glaucoma, right eye, stage unspecified
        "H401211",  # Low-tension glaucoma, right eye, mild stage
        "H401212",  # Low-tension glaucoma, right eye, moderate stage
        "H401213",  # Low-tension glaucoma, right eye, severe stage
        "H401214",  # Low-tension glaucoma, right eye, indeterminate stage
        "H401220",  # Low-tension glaucoma, left eye, stage unspecified
        "H401221",  # Low-tension glaucoma, left eye, mild stage
        "H401222",  # Low-tension glaucoma, left eye, moderate stage
        "H401223",  # Low-tension glaucoma, left eye, severe stage
        "H401224",  # Low-tension glaucoma, left eye, indeterminate stage
        "H401230",  # Low-tension glaucoma, bilateral, stage unspecified
        "H401231",  # Low-tension glaucoma, bilateral, mild stage
        "H401232",  # Low-tension glaucoma, bilateral, moderate stage
        "H401233",  # Low-tension glaucoma, bilateral, severe stage
        "H401234",  # Low-tension glaucoma, bilateral, indeterminate stage
        "H401310",  # Pigmentary glaucoma, right eye, stage unspecified
        "H401311",  # Pigmentary glaucoma, right eye, mild stage
        "H401312",  # Pigmentary glaucoma, right eye, moderate stage
        "H401313",  # Pigmentary glaucoma, right eye, severe stage
        "H401314",  # Pigmentary glaucoma, right eye, indeterminate stage
        "H401320",  # Pigmentary glaucoma, left eye, stage unspecified
        "H401321",  # Pigmentary glaucoma, left eye, mild stage
        "H401322",  # Pigmentary glaucoma, left eye, moderate stage
        "H401323",  # Pigmentary glaucoma, left eye, severe stage
        "H401324",  # Pigmentary glaucoma, left eye, indeterminate stage
        "H401330",  # Pigmentary glaucoma, bilateral, stage unspecified
        "H401331",  # Pigmentary glaucoma, bilateral, mild stage
        "H401332",  # Pigmentary glaucoma, bilateral, moderate stage
        "H401333",  # Pigmentary glaucoma, bilateral, severe stage
        "H401334",  # Pigmentary glaucoma, bilateral, indeterminate stage
        "H401410",  # Capsular glaucoma with pseudoexfoliation of lens, right eye, stage unspecified
        "H401411",  # Capsular glaucoma with pseudoexfoliation of lens, right eye, mild stage
        "H401412",  # Capsular glaucoma with pseudoexfoliation of lens, right eye, moderate stage
        "H401413",  # Capsular glaucoma with pseudoexfoliation of lens, right eye, severe stage
        "H401414",  # Capsular glaucoma with pseudoexfoliation of lens, right eye, indeterminate stage
        "H401420",  # Capsular glaucoma with pseudoexfoliation of lens, left eye, stage unspecified
        "H401421",  # Capsular glaucoma with pseudoexfoliation of lens, left eye, mild stage
        "H401422",  # Capsular glaucoma with pseudoexfoliation of lens, left eye, moderate stage
        "H401423",  # Capsular glaucoma with pseudoexfoliation of lens, left eye, severe stage
        "H401424",  # Capsular glaucoma with pseudoexfoliation of lens, left eye, indeterminate stage
        "H401430",  # Capsular glaucoma with pseudoexfoliation of lens, bilateral, stage unspecified
        "H401431",  # Capsular glaucoma with pseudoexfoliation of lens, bilateral, mild stage
        "H401432",  # Capsular glaucoma with pseudoexfoliation of lens, bilateral, moderate stage
        "H401433",  # Capsular glaucoma with pseudoexfoliation of lens, bilateral, severe stage
        "H401434",  # Capsular glaucoma with pseudoexfoliation of lens, bilateral, indeterminate stage
        "H40151",  # Residual stage of open-angle glaucoma, right eye
        "H40152",  # Residual stage of open-angle glaucoma, left eye
        "H40153",  # Residual stage of open-angle glaucoma, bilateral
        "H4020X0",  # Unspecified primary angle-closure glaucoma, stage unspecified
        "H4020X1",  # Unspecified primary angle-closure glaucoma, mild stage
        "H4020X2",  # Unspecified primary angle-closure glaucoma, moderate stage
        "H4020X3",  # Unspecified primary angle-closure glaucoma, severe stage
        "H4020X4",  # Unspecified primary angle-closure glaucoma, indeterminate stage
        "H40211",  # Acute angle-closure glaucoma, right eye
        "H40212",  # Acute angle-closure glaucoma, left eye
        "H40213",  # Acute angle-closure glaucoma, bilateral
        "H402210",  # Chronic angle-closure glaucoma, right eye, stage unspecified
        "H402211",  # Chronic angle-closure glaucoma, right eye, mild stage
        "H402212",  # Chronic angle-closure glaucoma, right eye, moderate stage
        "H402213",  # Chronic angle-closure glaucoma, right eye, severe stage
        "H402214",  # Chronic angle-closure glaucoma, right eye, indeterminate stage
        "H402220",  # Chronic angle-closure glaucoma, left eye, stage unspecified
        "H402221",  # Chronic angle-closure glaucoma, left eye, mild stage
        "H402222",  # Chronic angle-closure glaucoma, left eye, moderate stage
        "H402223",  # Chronic angle-closure glaucoma, left eye, severe stage
        "H402224",  # Chronic angle-closure glaucoma, left eye, indeterminate stage
        "H402230",  # Chronic angle-closure glaucoma, bilateral, stage unspecified
        "H402231",  # Chronic angle-closure glaucoma, bilateral, mild stage
        "H402232",  # Chronic angle-closure glaucoma, bilateral, moderate stage
        "H402233",  # Chronic angle-closure glaucoma, bilateral, severe stage
        "H402234",  # Chronic angle-closure glaucoma, bilateral, indeterminate stage
        "H40231",  # Intermittent angle-closure glaucoma, right eye
        "H40232",  # Intermittent angle-closure glaucoma, left eye
        "H40233",  # Intermittent angle-closure glaucoma, bilateral
        "H40241",  # Residual stage of angle-closure glaucoma, right eye
        "H40242",  # Residual stage of angle-closure glaucoma, left eye
        "H40243",  # Residual stage of angle-closure glaucoma, bilateral
        "H4031X0",  # Glaucoma secondary to eye trauma, right eye, stage unspecified
        "H4031X1",  # Glaucoma secondary to eye trauma, right eye, mild stage
        "H4031X2",  # Glaucoma secondary to eye trauma, right eye, moderate stage
        "H4031X3",  # Glaucoma secondary to eye trauma, right eye, severe stage
        "H4031X4",  # Glaucoma secondary to eye trauma, right eye, indeterminate stage
        "H4032X0",  # Glaucoma secondary to eye trauma, left eye, stage unspecified
        "H4032X1",  # Glaucoma secondary to eye trauma, left eye, mild stage
        "H4032X2",  # Glaucoma secondary to eye trauma, left eye, moderate stage
        "H4032X3",  # Glaucoma secondary to eye trauma, left eye, severe stage
        "H4032X4",  # Glaucoma secondary to eye trauma, left eye, indeterminate stage
        "H4033X0",  # Glaucoma secondary to eye trauma, bilateral, stage unspecified
        "H4033X1",  # Glaucoma secondary to eye trauma, bilateral, mild stage
        "H4033X2",  # Glaucoma secondary to eye trauma, bilateral, moderate stage
        "H4033X3",  # Glaucoma secondary to eye trauma, bilateral, severe stage
        "H4033X4",  # Glaucoma secondary to eye trauma, bilateral, indeterminate stage
        "H4041X0",  # Glaucoma secondary to eye inflammation, right eye, stage unspecified
        "H4041X1",  # Glaucoma secondary to eye inflammation, right eye, mild stage
        "H4041X2",  # Glaucoma secondary to eye inflammation, right eye, moderate stage
        "H4041X3",  # Glaucoma secondary to eye inflammation, right eye, severe stage
        "H4041X4",  # Glaucoma secondary to eye inflammation, right eye, indeterminate stage
        "H4042X0",  # Glaucoma secondary to eye inflammation, left eye, stage unspecified
        "H4042X1",  # Glaucoma secondary to eye inflammation, left eye, mild stage
        "H4042X2",  # Glaucoma secondary to eye inflammation, left eye, moderate stage
        "H4042X3",  # Glaucoma secondary to eye inflammation, left eye, severe stage
        "H4042X4",  # Glaucoma secondary to eye inflammation, left eye, indeterminate stage
        "H4043X0",  # Glaucoma secondary to eye inflammation, bilateral, stage unspecified
        "H4043X1",  # Glaucoma secondary to eye inflammation, bilateral, mild stage
        "H4043X2",  # Glaucoma secondary to eye inflammation, bilateral, moderate stage
        "H4043X3",  # Glaucoma secondary to eye inflammation, bilateral, severe stage
        "H4043X4",  # Glaucoma secondary to eye inflammation, bilateral, indeterminate stage
        "H4051X0",  # Glaucoma secondary to other eye disorders, right eye, stage unspecified
        "H4051X1",  # Glaucoma secondary to other eye disorders, right eye, mild stage
        "H4051X2",  # Glaucoma secondary to other eye disorders, right eye, moderate stage
        "H4051X3",  # Glaucoma secondary to other eye disorders, right eye, severe stage
        "H4051X4",  # Glaucoma secondary to other eye disorders, right eye, indeterminate stage
        "H4052X0",  # Glaucoma secondary to other eye disorders, left eye, stage unspecified
        "H4052X1",  # Glaucoma secondary to other eye disorders, left eye, mild stage
        "H4052X2",  # Glaucoma secondary to other eye disorders, left eye, moderate stage
        "H4052X3",  # Glaucoma secondary to other eye disorders, left eye, severe stage
        "H4052X4",  # Glaucoma secondary to other eye disorders, left eye, indeterminate stage
        "H4053X0",  # Glaucoma secondary to other eye disorders, bilateral, stage unspecified
        "H4053X1",  # Glaucoma secondary to other eye disorders, bilateral, mild stage
        "H4053X2",  # Glaucoma secondary to other eye disorders, bilateral, moderate stage
        "H4053X3",  # Glaucoma secondary to other eye disorders, bilateral, severe stage
        "H4053X4",  # Glaucoma secondary to other eye disorders, bilateral, indeterminate stage
        "H4061X0",  # Glaucoma secondary to drugs, right eye, stage unspecified
        "H4061X1",  # Glaucoma secondary to drugs, right eye, mild stage
        "H4061X2",  # Glaucoma secondary to drugs, right eye, moderate stage
        "H4061X3",  # Glaucoma secondary to drugs, right eye, severe stage
        "H4061X4",  # Glaucoma secondary to drugs, right eye, indeterminate stage
        "H4062X0",  # Glaucoma secondary to drugs, left eye, stage unspecified
        "H4062X1",  # Glaucoma secondary to drugs, left eye, mild stage
        "H4062X2",  # Glaucoma secondary to drugs, left eye, moderate stage
        "H4062X3",  # Glaucoma secondary to drugs, left eye, severe stage
        "H4062X4",  # Glaucoma secondary to drugs, left eye, indeterminate stage
        "H4063X0",  # Glaucoma secondary to drugs, bilateral, stage unspecified
        "H4063X1",  # Glaucoma secondary to drugs, bilateral, mild stage
        "H4063X2",  # Glaucoma secondary to drugs, bilateral, moderate stage
        "H4063X3",  # Glaucoma secondary to drugs, bilateral, severe stage
        "H4063X4",  # Glaucoma secondary to drugs, bilateral, indeterminate stage
        "H40811",  # Glaucoma with increased episcleral venous pressure, right eye
        "H40812",  # Glaucoma with increased episcleral venous pressure, left eye
        "H40813",  # Glaucoma with increased episcleral venous pressure, bilateral
        "H40821",  # Hypersecretion glaucoma, right eye
        "H40822",  # Hypersecretion glaucoma, left eye
        "H40823",  # Hypersecretion glaucoma, bilateral
        "H40831",  # Aqueous misdirection, right eye
        "H40832",  # Aqueous misdirection, left eye
        "H40833",  # Aqueous misdirection, bilateral
        "H4089",  # Other specified glaucoma
        "Q150",  # Congenital glaucoma
    }

    SNOMEDCT = {
        "1003657004",  # Congenital glaucoma of left eye (disorder)
        "1111391000000107",  # Open angle glaucoma due to ocular trauma (disorder)
        "1111401000000105",  # Open angle glaucoma due to neovascularization (disorder)
        "111513000",  # Advanced open-angle glaucoma (disorder)
        "111514006",  # Pseudoexfoliation glaucoma (disorder)
        "1137633000",  # Secondary angle-closure glaucoma due to iridocorneal endothelial syndrome (disorder)
        "12239301000119102",  # Bilateral open-angle glaucoma (disorder)
        "12239461000119106",  # Glaucoma of left eye (disorder)
        "12239501000119106",  # Glaucoma of right eye (disorder)
        "1285332001",  # Angle closure glaucoma caused by drug (disorder)
        "1285335004",  # Open angle glaucoma due to ocular surgery (disorder)
        "1285710008",  # Open angle glaucoma caused by drug (disorder)
        "1285711007",  # Open angle glaucoma of right eye caused by drug (disorder)
        "1285712000",  # Open angle glaucoma of left eye caused by drug (disorder)
        "1285713005",  # Open angle glaucoma of bilateral eyes caused by drug (disorder)
        "1285716002",  # Angle closure glaucoma of right eye caused by drug (disorder)
        "1285717006",  # Angle closure glaucoma of left eye caused by drug (disorder)
        "1285718001",  # Angle closure glaucoma of bilateral eyes caused by drug (disorder)
        "1303338003",  # Primary angle closure glaucoma of right eye (disorder)
        "1303339006",  # Primary angle closure glaucoma of left eye (disorder)
        "1303340008",  # Primary angle closure glaucoma of bilateral eyes (disorder)
        "1303454004",  # Open angle glaucoma of right eye due to ocular vascular disorder (disorder)
        "1303455003",  # Open angle glaucoma of left eye due to ocular vascular disorder (disorder)
        "1303456002",  # Open angle glaucoma of bilateral eyes due to ocular vascular disorder (disorder)
        "1303462007",  # Open angle glaucoma of bilateral eyes due to vascular disorder (disorder)
        "1303463002",  # Open angle glaucoma of left eye due to vascular disorder (disorder)
        "1303464008",  # Open angle glaucoma of right eye due to vascular disorder (disorder)
        "1303701001",  # Glaucoma of left eye due to intraocular neoplasm (disorder)
        "1303702008",  # Glaucoma of right eye caused by silicone oil (disorder)
        "1303704009",  # Glaucoma of bilateral eyes due to intraocular neoplasm (disorder)
        "1303706006",  # Glaucoma of right eye due to intraocular neoplasm (disorder)
        "1303707002",  # Glaucoma of left eye caused by silicone oil (disorder)
        "1303708007",  # Glaucoma of bilateral eyes caused by silicone oil (disorder)
        "1303889009",  # Open angle glaucoma of left eye due to systemic syndrome (disorder)
        "1303890000",  # Open angle glaucoma of right eye due to systemic syndrome (disorder)
        "1303891001",  # Open angle glaucoma of bilateral eyes due to systemic syndrome (disorder)
        "1303892008",  # Open angle glaucoma of right eye due to retinal detachment (disorder)
        "1303893003",  # Open angle glaucoma of left eye due to retinal detachment (disorder)
        "1303894009",  # Open angle glaucoma of bilateral eyes due to retinal detachment (disorder)
        "1303927008",  # Secondary open-angle glaucoma of right eye (disorder)
        "1303928003",  # Secondary open-angle glaucoma of left eye (disorder)
        "1303929006",  # Secondary open-angle glaucoma of bilateral eyes (disorder)
        "1304027002",  # Open angle glaucoma of left eye due to ocular trauma (disorder)
        "1304029004",  # Open angle glaucoma of right eye due to ocular trauma (disorder)
        "1304030009",  # Open angle glaucoma of bilateral eyes due to ocular trauma (disorder)
        "1304031008",  # Postoperative angle closure glaucoma of left eye (disorder)
        "1304032001",  # Postoperative angle closure glaucoma of bilateral eyes (disorder)
        "1304033006",  # Postoperative angle closure glaucoma of right eye (disorder)
        "1304034000",  # Open angle glaucoma of bilateral eyes due to neovascularization (disorder)
        "1304035004",  # Open angle glaucoma of left eye due to neovascularization (disorder)
        "1304036003",  # Open angle glaucoma of right eye due to neovascularization (disorder)
        "1304240002",  # Glaucoma of bilateral eyes (disorder)
        "1304266004",  # Secondary angle-closure glaucoma of right eye due to iridocorneal endothelial syndrome (disorder)
        "1304267008",  # Secondary angle-closure glaucoma of left eye due to iridocorneal endothelial syndrome (disorder)
        "1304268003",  # Secondary angle-closure glaucoma of bilateral eyes due to iridocorneal endothelial syndrome (disorder)
        "1304282003",  # Glaucoma of bilateral eyes due to hemolysis following intraocular hemorrhage (disorder)
        "1304284002",  # Glaucoma of left eye due to hemolysis following intraocular hemorrhage (disorder)
        "1304285001",  # Glaucoma of right eye due to hemolysis following intraocular hemorrhage (disorder)
        "1306287007",  # Primary open angle glaucoma of left eye (disorder)
        "1306288002",  # Primary open angle glaucoma of right eye (disorder)
        "1306289005",  # Primary open angle glaucoma of bilateral eyes (disorder)
        "1306298008",  # Angle closure glaucoma of right eye (disorder)
        "1306301007",  # Angle closure glaucoma of left eye (disorder)
        "1306328002",  # Angle closure glaucoma of right eye due to peripheral anterior synechiae (disorder)
        "1306329005",  # Angle closure glaucoma of left eye due to peripheral anterior synechiae (disorder)
        "1306330000",  # Angle closure glaucoma of bilateral eyes due to peripheral anterior synechiae (disorder)
        "1306331001",  # Secondary angle-closure glaucoma of right eye (disorder)
        "1306332008",  # Secondary angle closure glaucoma of left eye (disorder)
        "1306333003",  # Secondary angle closure glaucoma of bilateral eyes (disorder)
        "1306607000",  # Angle recession glaucoma of right eye (disorder)
        "1306746005",  # Ghost cell glaucoma of right eye due to and following intraocular hemorrhage (disorder)
        "1306749003",  # Ghost cell glaucoma of bilateral eyes due to and following intraocular hemorrhage (disorder)
        "1306750003",  # Ghost cell glaucoma of left eye due to and following intraocular hemorrhage (disorder)
        "1306768000",  # Phacomorphic angle closure glaucoma of left eye due to disorder of lens (disorder)
        "1306772001",  # Lens particle glaucoma of right eye (disorder)
        "1306773006",  # Lens particle glaucoma of left eye (disorder)
        "1306774000",  # Lens particle glaucoma of bilateral eyes (disorder)
        "1306777007",  # Phacomorphic angle closure glaucoma of bilateral eyes (disorder)
        "1306778002",  # Phacomorphic angle closure glaucoma of right eye due to disorder of lens (disorder)
        "1306782000",  # Glaucoma of right eye due to congenital malformation of angle of anterior chamber of eye (disorder)
        "1306787006",  # Glaucoma of left eye due to congenital malformation of angle of anterior chamber of eye (disorder)
        "1306789009",  # Glaucoma of bilateral eyes due to congenital malformation of angle of anterior chamber of eyes (disorder)
        "1306868005",  # Angle recession glaucoma of left eye (disorder)
        "1306869002",  # Angle recession glaucoma of bilateral eyes (disorder)
        "1306873004",  # Glaucoma of left eye with intraocular hemorrhage (disorder)
        "1306874005",  # Glaucoma of right eye with intraocular hemorrhage (disorder)
        "1306875006",  # Glaucoma of bilateral eyes with intraocular hemorrhage (disorder)
        "1306876007",  # Open angle glaucoma of bilateral eyes (disorder)
        "1306877003",  # Open angle glaucoma of right eye due to ocular surgery (disorder)
        "1306878008",  # Open angle glaucoma of left eye due to ocular surgery (disorder)
        "1306879000",  # Open angle glaucoma of bilateral eyes due to ocular surgery (disorder)
        "1306884006",  # Normal pressure primary open-angle glaucoma of left eye (disorder)
        "1306885007",  # Normal pressure primary open-angle glaucoma of right eye (disorder)
        "1306886008",  # Normal pressure primary open-angle glaucoma of bilateral eyes (disorder)
        "1331893009",  # Neovascular angle closure glaucoma of right eye (disorder)
        "1331894003",  # Neovascular angle closure glaucoma of left eye (disorder)
        "1331895002",  # Neovascular angle closure glaucoma of bilateral eyes (disorder)
        "1332297002",  # Glaucoma due to perforating injury of left eye (disorder)
        "1332298007",  # Glaucoma due to perforating injury of right eye (disorder)
        "1332299004",  # Glaucoma due to perforating injury of bilateral eyes (disorder)
        "1335974003",  # Glaucoma due to cyst of iris (disorder)
        "1335975002",  # Glaucoma due to cyst of ciliary body (disorder)
        "1335980006",  # Glaucoma due to cyst of ciliary body of right eye (disorder)
        "1335982003",  # Glaucoma due to cyst of ciliary body of left eye (disorder)
        "1335985001",  # Glaucoma due to cyst of ciliary body of bilateral eyes (disorder)
        "1336007002",  # Glaucoma due to cyst of iris of left eye (disorder)
        "1336008007",  # Glaucoma due to cyst of iris of right eye (disorder)
        "1336009004",  # Glaucoma due to cyst of iris of bilateral eyes (disorder)
        "1336022007",  # Glaucoma due to cyst of right eye proper (disorder)
        "1336023002",  # Glaucoma due to cyst of left eye proper (disorder)
        "1336024008",  # Glaucoma due to cyst of bilateral eyes (disorder)
        "1336042003",  # Glaucoma due to plateau iris (disorder)
        "15374009",  # Secondary glaucoma due to aphakia (disorder)
        "15633281000119103",  # Bilateral primary open angle glaucoma (disorder)
        "15673001000119103",  # Congenital glaucoma of bilateral eyes (disorder)
        "15673041000119101",  # Congenital glaucoma of right eye (disorder)
        "15679801000119105",  # Uveitis-glaucoma-hyphema syndrome of bilateral eyes (disorder)
        "15736681000119100",  # Narrow-angle glaucoma of right eye (disorder)
        "15736721000119106",  # Angle-closure glaucoma of bilateral eyes (disorder)
        "15736761000119101",  # Narrow angle glaucoma of left eye (disorder)
        "15738841000119105",  # Glaucoma of bilateral eyes due to combination of mechanisms (disorder)
        "15738881000119100",  # Glaucoma of left eye due to combination of mechanisms (disorder)
        "15738921000119107",  # Glaucoma of right eye due to combination of mechanisms (disorder)
        "15738961000119102",  # Glaucoma of left eye due to congenital anomaly of iris (disorder)
        "15739001000119109",  # Glaucoma of right eye due to congenital anomaly of iris (disorder)
        "15739041000119106",  # Glaucoma of bilateral eyes due to congenital anomaly of iris (disorder)
        "15739081000119101",  # Glaucoma of bilateral eyes due to anterior segment anomaly (disorder)
        "15739121000119104",  # Glaucoma of right eye due to anterior segment anomaly (disorder)
        "15739161000119109",  # Glaucoma of left eye due to anterior segment anomaly (disorder)
        "15739441000119101",  # Glaucoma of right eye due to trauma (disorder)
        "15739481000119106",  # Glaucoma of left eye due to trauma (disorder)
        "15739561000119101",  # Glaucoma of right eye due to lens disorder (disorder)
        "15739641000119104",  # Glaucoma of left eye due to lens disorder (disorder)
        "15995591000119109",  # Glaucoma of left eye due to plateau iris (disorder)
        "15995631000119109",  # Glaucoma of right eye due to plateau iris (disorder)
        "15996831000119101",  # Glaucoma caused by Onchocerca volvulus (disorder)
        "1654001",  # Open angle glaucoma caused by corticosteroid (disorder)
        "19144002",  # Absolute glaucoma (disorder)
        "19309007",  # Open angle glaucoma due to vascular disorder (disorder)
        "193548006",  # Steroid-induced glaucoma glaucomatous stage (disorder)
        "193552006",  # Glaucoma due to congenital malformation of angle of anterior chamber of eye (disorder)
        "193553001",  # Glaucoma due to iris anomaly (disorder)
        "193555008",  # Open angle glaucoma due to systemic syndrome (disorder)
        "193561006",  # Secondary angle-closure glaucoma with pupil block (disorder)
        "193562004",  # Open angle glaucoma due to ocular vascular disorder (disorder)
        "204113001",  # Congenital glaucoma (disorder)
        "206248004",  # Traumatic glaucoma due to birth trauma (disorder)
        "21571006",  # Secondary angle-closure glaucoma (disorder)
        "21928008",  # Secondary open-angle glaucoma (disorder)
        "232081005",  # Iatrogenic glaucoma (disorder)
        "232082003",  # Iatrogenic angle-closure glaucoma (disorder)
        "232083008",  # Glaucoma and corneal anomaly (disorder)
        "232086000",  # Neovascular angle closure glaucoma (disorder)
        "232087009",  # Glaucoma with intraocular hemorrhage (disorder)
        "232088004",  # Ghost cell glaucoma due to and following intraocular hemorrhage (disorder)
        "23986001",  # Glaucoma (disorder)
        "275477002",  # Glaucoma due to ocular cyst (disorder)
        "27735002",  # Glaucoma associated with tumors AND/OR cysts (disorder)
        "29369005",  # Hypersecretion glaucoma (disorder)
        "30041005",  # Acute angle-closure glaucoma (disorder)
        "314017009",  # Acute-on-chronic glaucoma (disorder)
        "314033007",  # Uveitic glaucoma (disorder)
        "314784002",  # Angle closure glaucoma due to peripheral anterior synechiae (disorder)
        "32893002",  # Phacolytic glaucoma (disorder)
        "333151000119108",  # Phacolytic glaucoma of right eye (disorder)
        "33647009",  # Anatomical narrow angle glaucoma (disorder)
        "336611000119109",  # Acute angle-closure glaucoma of right eye (disorder)
        "336631000119104",  # Absolute glaucoma right eye (disorder)
        "338761000119103",  # Phacolytic glaucoma of left eye (disorder)
        "342221000119104",  # Acute angle-closure glaucoma of left eye (disorder)
        "342241000119105",  # Absolute glaucoma left eye (disorder)
        "344531000119107",  # Phacolytic glaucoma of bilateral eyes (disorder)
        "34623005",  # Glaucoma with increased episcleral venous pressure (disorder)
        "347381000119106",  # Bilateral acute angle-closure glaucoma (disorder)
        "347401000119106",  # Bilateral absolute glaucoma (disorder)
        "370504007",  # Aqueous humor misdirect (disorder)
        "392029006",  # Uveitis-glaucoma-hyphema syndrome (disorder)
        "392030001",  # Glaucoma due to hemolysis following intraocular hemorrhage (disorder)
        "392288006",  # Primary angle-closure glaucoma (disorder)
        "392291006",  # Angle-closure glaucoma (disorder)
        "392300000",  # Phacomorphic angle closure glaucoma due to disorder of lens (disorder)
        "392352004",  # Angle recession glaucoma (disorder)
        "404648005",  # Lens particle glaucoma (disorder)
        "415176004",  # Primary congenital glaucoma (disorder)
        "41911000119107",  # Glaucoma due to type 2 diabetes mellitus (disorder)
        "444863008",  # Anatomical narrow angle glaucoma with borderline intraocular pressure (disorder)
        "45623002",  # Glaucoma due to anterior segment anomaly (disorder)
        "46168003",  # Pigmentary glaucoma (disorder)
        "50485007",  # Low tension glaucoma (disorder)
        "60981000119103",  # Glaucoma due to diabetes mellitus type 1 (disorder)
        "65460003",  # Intermittent angle-closure glaucoma (disorder)
        "66725002",  # Glaucoma due to perforating injury (disorder)
        "66747002",  # Glaucoma associated with ocular disorder (disorder)
        "68241007",  # Glaucoma due to ocular trauma (disorder)
        "698840003",  # Neovascular glaucoma due to hyphema (disorder)
        "713457002",  # Neovascular glaucoma due to diabetes mellitus (disorder)
        "715144004",  # Glaucoma caused by silicone oil (disorder)
        "716166002",  # Microcornea with glaucoma and absent frontal sinus syndrome (disorder)
        "721898008",  # Open angle glaucoma of bilateral eyes caused by corticosteroid (disorder)
        "722321001",  # Open angle glaucoma of left eye caused by corticosteroid (disorder)
        "722329004",  # Open angle glaucoma of right eye caused by corticosteroid (disorder)
        "77075001",  # Primary open angle glaucoma (disorder)
        "787051000",  # Open-angle glaucoma of left eye (disorder)
        "787052007",  # Open-angle glaucoma of right eye (disorder)
        "788946007",  # Glaucoma due to eye inflammation (disorder)
        "788947003",  # Glaucoma due to intraocular neoplasm (disorder)
        "84333006",  # Glaucoma due to lens disorder (disorder)
        "84494001",  # Open-angle glaucoma (disorder)
        "89215000",  # Postoperative angle-closure glaucoma (disorder)
        "92829008",  # Glaucoma due to combination of mechanisms (disorder)
        "95213001",  # Primary glaucoma due to combination of mechanisms (disorder)
        "95250000",  # Secondary glaucoma due to combination mechanisms (disorder)
        "95717004",  # Secondary glaucoma (disorder)
    }

class GlaucomaAssociatedWithCongenitalAnomaliesAndDystrophiesAndSystemicSyndromes(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of glaucoma associated with congenital anomalies, dystrophies, and systemic syndromes.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of glaucoma that may be due to congenital anomalies, dystrophies, or other systemic syndromes, including trauma, or a secondary diagnosis.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Glaucoma Associated with Congenital Anomalies and Dystrophies and Systemic Syndromes"
    OID = "2.16.840.1.113883.3.526.3.1461"
    DEFINITION_VERSION = "20210214"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H4031X0",  # Glaucoma secondary to eye trauma, right eye, stage unspecified
        "H4031X1",  # Glaucoma secondary to eye trauma, right eye, mild stage
        "H4031X2",  # Glaucoma secondary to eye trauma, right eye, moderate stage
        "H4031X3",  # Glaucoma secondary to eye trauma, right eye, severe stage
        "H4031X4",  # Glaucoma secondary to eye trauma, right eye, indeterminate stage
        "H4032X0",  # Glaucoma secondary to eye trauma, left eye, stage unspecified
        "H4032X1",  # Glaucoma secondary to eye trauma, left eye, mild stage
        "H4032X2",  # Glaucoma secondary to eye trauma, left eye, moderate stage
        "H4032X3",  # Glaucoma secondary to eye trauma, left eye, severe stage
        "H4032X4",  # Glaucoma secondary to eye trauma, left eye, indeterminate stage
        "H4033X0",  # Glaucoma secondary to eye trauma, bilateral, stage unspecified
        "H4033X1",  # Glaucoma secondary to eye trauma, bilateral, mild stage
        "H4033X2",  # Glaucoma secondary to eye trauma, bilateral, moderate stage
        "H4033X3",  # Glaucoma secondary to eye trauma, bilateral, severe stage
        "H4033X4",  # Glaucoma secondary to eye trauma, bilateral, indeterminate stage
        "H4041X0",  # Glaucoma secondary to eye inflammation, right eye, stage unspecified
        "H4041X1",  # Glaucoma secondary to eye inflammation, right eye, mild stage
        "H4041X2",  # Glaucoma secondary to eye inflammation, right eye, moderate stage
        "H4041X3",  # Glaucoma secondary to eye inflammation, right eye, severe stage
        "H4041X4",  # Glaucoma secondary to eye inflammation, right eye, indeterminate stage
        "H4042X0",  # Glaucoma secondary to eye inflammation, left eye, stage unspecified
        "H4042X1",  # Glaucoma secondary to eye inflammation, left eye, mild stage
        "H4042X2",  # Glaucoma secondary to eye inflammation, left eye, moderate stage
        "H4042X3",  # Glaucoma secondary to eye inflammation, left eye, severe stage
        "H4042X4",  # Glaucoma secondary to eye inflammation, left eye, indeterminate stage
        "H4043X0",  # Glaucoma secondary to eye inflammation, bilateral, stage unspecified
        "H4043X1",  # Glaucoma secondary to eye inflammation, bilateral, mild stage
        "H4043X2",  # Glaucoma secondary to eye inflammation, bilateral, moderate stage
        "H4043X3",  # Glaucoma secondary to eye inflammation, bilateral, severe stage
        "H4043X4",  # Glaucoma secondary to eye inflammation, bilateral, indeterminate stage
        "H4051X0",  # Glaucoma secondary to other eye disorders, right eye, stage unspecified
        "H4051X1",  # Glaucoma secondary to other eye disorders, right eye, mild stage
        "H4051X2",  # Glaucoma secondary to other eye disorders, right eye, moderate stage
        "H4051X3",  # Glaucoma secondary to other eye disorders, right eye, severe stage
        "H4051X4",  # Glaucoma secondary to other eye disorders, right eye, indeterminate stage
        "H4052X0",  # Glaucoma secondary to other eye disorders, left eye, stage unspecified
        "H4052X1",  # Glaucoma secondary to other eye disorders, left eye, mild stage
        "H4052X2",  # Glaucoma secondary to other eye disorders, left eye, moderate stage
        "H4052X3",  # Glaucoma secondary to other eye disorders, left eye, severe stage
        "H4052X4",  # Glaucoma secondary to other eye disorders, left eye, indeterminate stage
        "H4053X0",  # Glaucoma secondary to other eye disorders, bilateral, stage unspecified
        "H4053X1",  # Glaucoma secondary to other eye disorders, bilateral, mild stage
        "H4053X2",  # Glaucoma secondary to other eye disorders, bilateral, moderate stage
        "H4053X3",  # Glaucoma secondary to other eye disorders, bilateral, severe stage
        "H4053X4",  # Glaucoma secondary to other eye disorders, bilateral, indeterminate stage
        "H40811",  # Glaucoma with increased episcleral venous pressure, right eye
        "H40812",  # Glaucoma with increased episcleral venous pressure, left eye
        "H40813",  # Glaucoma with increased episcleral venous pressure, bilateral
        "H40821",  # Hypersecretion glaucoma, right eye
        "H40822",  # Hypersecretion glaucoma, left eye
        "H40823",  # Hypersecretion glaucoma, bilateral
        "H40831",  # Aqueous misdirection, right eye
        "H40832",  # Aqueous misdirection, left eye
        "H40833",  # Aqueous misdirection, bilateral
        "H4089",  # Other specified glaucoma
        "H409",  # Unspecified glaucoma
        "H42",  # Glaucoma in diseases classified elsewhere
    }

    SNOMEDCT = {
        "1003425007",  # Glaucoma due to Lowe syndrome (disorder)
        "1003428009",  # Glaucoma due to congenital anomaly of eye (disorder)
        "1003657004",  # Congenital glaucoma of left eye (disorder)
        "1111401000000105",  # Open angle glaucoma due to neovascularization (disorder)
        "1137633000",  # Secondary angle-closure glaucoma due to iridocorneal endothelial syndrome (disorder)
        "1231270006",  # Glaucoma following ocular onchocerciasis (disorder)
        "1231271005",  # Glaucoma due to and following retinopathy of prematurity (disorder)
        "1285643003",  # Neovascular glaucoma due to central retinal vein occlusion (disorder)
        "1303454004",  # Open angle glaucoma of right eye due to ocular vascular disorder (disorder)
        "1303455003",  # Open angle glaucoma of left eye due to ocular vascular disorder (disorder)
        "1303456002",  # Open angle glaucoma of bilateral eyes due to ocular vascular disorder (disorder)
        "1303462007",  # Open angle glaucoma of bilateral eyes due to vascular disorder (disorder)
        "1303463002",  # Open angle glaucoma of left eye due to vascular disorder (disorder)
        "1303464008",  # Open angle glaucoma of right eye due to vascular disorder (disorder)
        "1303889009",  # Open angle glaucoma of left eye due to systemic syndrome (disorder)
        "1303890000",  # Open angle glaucoma of right eye due to systemic syndrome (disorder)
        "1303891001",  # Open angle glaucoma of bilateral eyes due to systemic syndrome (disorder)
        "1304034000",  # Open angle glaucoma of bilateral eyes due to neovascularization (disorder)
        "1304035004",  # Open angle glaucoma of left eye due to neovascularization (disorder)
        "1304036003",  # Open angle glaucoma of right eye due to neovascularization (disorder)
        "1304266004",  # Secondary angle-closure glaucoma of right eye due to iridocorneal endothelial syndrome (disorder)
        "1304267008",  # Secondary angle-closure glaucoma of left eye due to iridocorneal endothelial syndrome (disorder)
        "1304268003",  # Secondary angle-closure glaucoma of bilateral eyes due to iridocorneal endothelial syndrome (disorder)
        "1306607000",  # Angle recession glaucoma of right eye (disorder)
        "1306768000",  # Phacomorphic angle closure glaucoma of left eye due to disorder of lens (disorder)
        "1306772001",  # Lens particle glaucoma of right eye (disorder)
        "1306773006",  # Lens particle glaucoma of left eye (disorder)
        "1306774000",  # Lens particle glaucoma of bilateral eyes (disorder)
        "1306778002",  # Phacomorphic angle closure glaucoma of right eye due to disorder of lens (disorder)
        "1306782000",  # Glaucoma of right eye due to congenital malformation of angle of anterior chamber of eye (disorder)
        "1306787006",  # Glaucoma of left eye due to congenital malformation of angle of anterior chamber of eye (disorder)
        "1306789009",  # Glaucoma of bilateral eyes due to congenital malformation of angle of anterior chamber of eyes (disorder)
        "1306868005",  # Angle recession glaucoma of left eye (disorder)
        "1306869002",  # Angle recession glaucoma of bilateral eyes (disorder)
        "1331893009",  # Neovascular angle closure glaucoma of right eye (disorder)
        "1331894003",  # Neovascular angle closure glaucoma of left eye (disorder)
        "1331895002",  # Neovascular angle closure glaucoma of bilateral eyes (disorder)
        "15673001000119103",  # Congenital glaucoma of bilateral eyes (disorder)
        "15673041000119101",  # Congenital glaucoma of right eye (disorder)
        "15679801000119105",  # Uveitis-glaucoma-hyphema syndrome of bilateral eyes (disorder)
        "15738961000119102",  # Glaucoma of left eye due to congenital anomaly of iris (disorder)
        "15739001000119109",  # Glaucoma of right eye due to congenital anomaly of iris (disorder)
        "15739041000119106",  # Glaucoma of bilateral eyes due to congenital anomaly of iris (disorder)
        "15739081000119101",  # Glaucoma of bilateral eyes due to anterior segment anomaly (disorder)
        "15739121000119104",  # Glaucoma of right eye due to anterior segment anomaly (disorder)
        "15739161000119109",  # Glaucoma of left eye due to anterior segment anomaly (disorder)
        "15739561000119101",  # Glaucoma of right eye due to lens disorder (disorder)
        "15739641000119104",  # Glaucoma of left eye due to lens disorder (disorder)
        "19309007",  # Open angle glaucoma due to vascular disorder (disorder)
        "193552006",  # Glaucoma due to congenital malformation of angle of anterior chamber of eye (disorder)
        "193553001",  # Glaucoma due to iris anomaly (disorder)
        "193555008",  # Open angle glaucoma due to systemic syndrome (disorder)
        "193556009",  # Glaucoma in endocrine, nutritional and metabolic diseases (disorder)
        "193561006",  # Secondary angle-closure glaucoma with pupil block (disorder)
        "193562004",  # Open angle glaucoma due to ocular vascular disorder (disorder)
        "204113001",  # Congenital glaucoma (disorder)
        "206248004",  # Traumatic glaucoma due to birth trauma (disorder)
        "232083008",  # Glaucoma and corneal anomaly (disorder)
        "232086000",  # Neovascular angle closure glaucoma (disorder)
        "314033007",  # Uveitic glaucoma (disorder)
        "32893002",  # Phacolytic glaucoma (disorder)
        "331471000119105",  # Uveitic glaucoma of right eye (disorder)
        "333111000119107",  # Pigmentary glaucoma of right eye (disorder)
        "333151000119108",  # Phacolytic glaucoma of right eye (disorder)
        "337081000119109",  # Uveitic glaucoma of left eye (disorder)
        "338721000119108",  # Pigmentary glaucoma of left eye (disorder)
        "338761000119103",  # Phacolytic glaucoma of left eye (disorder)
        "344491000119107",  # Pigmentary glaucoma of bilateral eyes (disorder)
        "344531000119107",  # Phacolytic glaucoma of bilateral eyes (disorder)
        "370504007",  # Aqueous humor misdirect (disorder)
        "392300000",  # Phacomorphic angle closure glaucoma due to disorder of lens (disorder)
        "392352004",  # Angle recession glaucoma (disorder)
        "404648005",  # Lens particle glaucoma (disorder)
        "415176004",  # Primary congenital glaucoma (disorder)
        "418435001",  # Macular corneal dystrophy Type II (disorder)
        "45623002",  # Glaucoma due to anterior segment anomaly (disorder)
        "46168003",  # Pigmentary glaucoma (disorder)
        "66725002",  # Glaucoma due to perforating injury (disorder)
        "66747002",  # Glaucoma associated with ocular disorder (disorder)
        "713457002",  # Neovascular glaucoma due to diabetes mellitus (disorder)
        "716166002",  # Microcornea with glaucoma and absent frontal sinus syndrome (disorder)
        "788946007",  # Glaucoma due to eye inflammation (disorder)
        "84333006",  # Glaucoma due to lens disorder (disorder)
        "860798008",  # Glaucoma due to diabetes mellitus (disorder)
        "890423003",  # Secondary childhood glaucoma following congenital cataract surgery (disorder)
        "897585008",  # Congenital primary glaucoma of bilateral eyes (disorder)
        "95717004",  # Secondary glaucoma (disorder)
    }

class HereditaryChoroidalDystrophies(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of hereditary choroidal dystrophies.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of hereditary choroidal dystrophies, including choroideremia and atrophy.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Hereditary Choroidal Dystrophies"
    OID = "2.16.840.1.113883.3.526.3.1462"
    DEFINITION_VERSION = "20210214"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H3120",  # Hereditary choroidal dystrophy, unspecified
        "H3121",  # Choroideremia
        "H3122",  # Choroidal dystrophy (central areolar) (generalized) (peripapillary)
        "H3123",  # Gyrate atrophy, choroid
        "H3129",  # Other hereditary choroidal dystrophy
    }

    SNOMEDCT = {
        "1231678007",  # Peripapillary choroidal dystrophy (disorder)
        "15989831000119105",  # Choroideremia of bilateral eyes (disorder)
        "193466003",  # Partial circumpapillary choroid dystrophy (disorder)
        "193468002",  # Partial central choroid dystrophy (disorder)
        "19871000",  # Total generalized choroidal dystrophy (disorder)
        "231996009",  # Central areolar choroidal sclerosis (disorder)
        "231997000",  # Peripapillary choroidal sclerosis (disorder)
        "247200005",  # Choroidal atrophy (finding)
        "24950005",  # Partial generalized choroidal dystrophy (disorder)
        "312918002",  # Choroidal dystrophy (disorder)
        "314467007",  # Gyrate atrophy (disorder)
        "41250005",  # Diffuse choroidal sclerosis (disorder)
        "59753003",  # Total circumpapillary dystrophy of choroid (disorder)
        "715984007",  # Cerebellar ataxia with hypogonadism and choroidal dystrophy syndrome (disorder)
        "717761005",  # Choroideremia with deafness and obesity syndrome (disorder)
        "720850008",  # Choroidal atrophy and alopecia syndrome (disorder)
        "733110004",  # Van den Bosch syndrome (disorder)
        "74469006",  # Hereditary choroidal dystrophy (disorder)
        "75241009",  # Choroideremia (disorder)
        "92820007",  # Circumpapillary dystrophy of choroid (disorder)
    }

class HereditaryCornealDystrophies(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of hereditary corneal dystrophies.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of hereditary corneal dystrophies, including congenital, epithelial/juvenile, granular, lattice or macular corneal dystrophies.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Hereditary Corneal Dystrophies"
    OID = "2.16.840.1.113883.3.526.3.1424"
    DEFINITION_VERSION = "20210214"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H18511",  # Endothelial corneal dystrophy, right eye
        "H18512",  # Endothelial corneal dystrophy, left eye
        "H18513",  # Endothelial corneal dystrophy, bilateral
        "H18521",  # Epithelial (juvenile) corneal dystrophy, right eye
        "H18522",  # Epithelial (juvenile) corneal dystrophy, left eye
        "H18523",  # Epithelial (juvenile) corneal dystrophy, bilateral
        "H18531",  # Granular corneal dystrophy, right eye
        "H18532",  # Granular corneal dystrophy, left eye
        "H18533",  # Granular corneal dystrophy, bilateral
        "H18541",  # Lattice corneal dystrophy, right eye
        "H18542",  # Lattice corneal dystrophy, left eye
        "H18543",  # Lattice corneal dystrophy, bilateral
        "H18551",  # Macular corneal dystrophy, right eye
        "H18552",  # Macular corneal dystrophy, left eye
        "H18553",  # Macular corneal dystrophy, bilateral
        "H18591",  # Other hereditary corneal dystrophies, right eye
        "H18592",  # Other hereditary corneal dystrophies, left eye
        "H18593",  # Other hereditary corneal dystrophies, bilateral
    }

    SNOMEDCT = {
        "129623003",  # Iridocorneal endothelial syndrome (disorder)
        "1674008",  # Juvenile epithelial corneal dystrophy (disorder)
        "193837009",  # Combined corneal dystrophy (disorder)
        "193839007",  # Fuchs' corneal dystrophy (disorder)
        "231927007",  # Fatty degeneration of cornea (disorder)
        "231930000",  # Reis-Bucklers' corneal dystrophy (disorder)
        "231931001",  # Stromal corneal dystrophy (disorder)
        "231933003",  # Lattice corneal dystrophy, isolated form (disorder)
        "231934009",  # Pre-descemet's corneal dystrophy (disorder)
        "29504002",  # Posterior polymorphous corneal dystrophy (disorder)
        "361199007",  # Lattice corneal dystrophy (disorder)
        "373426005",  # Epithelial basement membrane dystrophy (disorder)
        "39662004",  # Crystalline corneal dystrophy (disorder)
        "397568004",  # Granular corneal dystrophy type II (disorder)
        "404633004",  # Cogan-Reese syndrome (disorder)
        "416960004",  # Corneal endothelial dystrophy (disorder)
        "417065002",  # Thiel-Behnke corneal dystrophy (disorder)
        "417183007",  # Fleck corneal dystrophy (disorder)
        "417395001",  # Congenital hereditary endothelial dystrophy type 2 (disorder)
        "417651000",  # Congenital hereditary endothelial dystrophy (disorder)
        "418054005",  # Macular corneal dystrophy Type I (disorder)
        "418435001",  # Macular corneal dystrophy Type II (disorder)
        "419039007",  # Granular corneal dystrophy type I (disorder)
        "419197009",  # Lattice corneal dystrophy Type I (disorder)
        "419395007",  # Schnyder crystalline cornea dystrophy (disorder)
        "419900000",  # Gelatinous droplike corneal dystrophy (disorder)
        "430888006",  # Dystrophy of anterior cornea (disorder)
        "45283008",  # Granular corneal dystrophy (disorder)
        "5587004",  # Corneal dystrophy (disorder)
        "60258001",  # Macular corneal dystrophy (disorder)
        "680491000119103",  # Iridocorneal endothelial syndrome of bilateral eyes (disorder)
        "680501000119105",  # Iridocorneal endothelial syndrome of right eye (disorder)
        "680511000119108",  # Iridocorneal endothelial syndrome of left eye (disorder)
        "702359002",  # Congenital stromal corneal dystrophy (disorder)
        "75663006",  # Ring-like corneal dystrophy (disorder)
        "77797009",  # Hereditary corneal dystrophy (disorder)
        "95488001",  # Congenital macular corneal dystrophy (disorder)
    }

class HereditaryRetinalDystrophies(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of hereditary retinal dystrophies.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of hereditary retinal or vitreoretinal dystrophies, including Usher syndrome.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Hereditary Retinal Dystrophies"
    OID = "2.16.840.1.113883.3.526.3.1463"
    DEFINITION_VERSION = "20210219"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H3550",  # Unspecified hereditary retinal dystrophy
        "H3551",  # Vitreoretinal dystrophy
        "H3552",  # Pigmentary retinal dystrophy
        "H3553",  # Other dystrophies primarily involving the sensory retina
        "H3554",  # Dystrophies primarily involving the retinal pigment epithelium
        "H3689",  # Other retinal disorders in diseases classified elsewhere
    }

    SNOMEDCT = {
        "1010610007",  # Retinitis pigmentosa-deafness syndrome type 3 (disorder)
        "1172605003",  # Retinitis pigmentosa, hearing loss, premature aging, short stature, facial dysmorphism syndrome (disorder)
        "1220597000",  # Retinitis pigmentosa, juvenile cataract, short stature, intellectual disability syndrome (disorder)
        "15059000",  # Retinal dystrophy in systemic lipidosis (disorder)
        "15699201000119104",  # Retinitis pigmentosa of left eye (disorder)
        "15699241000119102",  # Retinitis pigmentosa of right eye (disorder)
        "15699281000119107",  # Retinitis pigmentosa of bilateral eyes (disorder)
        "192788009",  # Retinal dystrophy in cerebroretinal lipidosis (disorder)
        "193400007",  # Hereditary retinal dystrophies in lipidoses (disorder)
        "193409008",  # Hyaline retinal dystrophy (disorder)
        "193415008",  # Tapetoretinal dystrophy (disorder)
        "232052009",  # Autosomal dominant retinitis pigmentosa (disorder)
        "232053004",  # Autosomal recessive retinitis pigmentosa (disorder)
        "232054005",  # X-linked retinitis pigmentosa (disorder)
        "232055006",  # X-linked retinitis pigmentosa heterozygote (disorder)
        "232056007",  # Progressive rod dystrophy (disorder)
        "232057003",  # Usher syndrome type 1 (disorder)
        "232058008",  # Usher syndrome type 2 (disorder)
        "237611007",  # Muscular atrophy, ataxia, retinitis pigmentosa, and diabetes mellitus (disorder)
        "237984008",  # Neurogenic muscle weakness, ataxia and retinitis pigmentosa (disorder)
        "267613004",  # Progressive cone dystrophy (without rod involvement) (disorder)
        "28835009",  # Retinitis pigmentosa (disorder)
        "312917007",  # Cone dystrophy (disorder)
        "313003009",  # Rod dystrophy (disorder)
        "41799005",  # Hereditary retinal dystrophy (disorder)
        "514793471000119104",  # Usher syndrome type 1F (disorder)
        "57838006",  # Retinitis pigmentosa-deafness syndrome (disorder)
        "698847000",  # Hereditary retinal dystrophy primarily involving retinal pigment epithelium (disorder)
        "698848005",  # Hereditary retinal dystrophy primarily involving sensory retina (disorder)
        "719282008",  # Primary ciliary dyskinesia and retinitis pigmentosa syndrome (disorder)
        "723452007",  # Polyneuropathy, hearing loss, ataxia, retinitis pigmentosa, cataract syndrome (disorder)
        "764939004",  # Fundus albipunctatus (disorder)
        "79556007",  # Vitreoretinal dystrophy (disorder)
        "80328002",  # Progressive cone-rod dystrophy (disorder)
    }

class HypotonyOfEye(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of hypotony of eye.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of hypotony of eye.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Hypotony of Eye"
    OID = "2.16.840.1.113883.3.526.3.1426"
    DEFINITION_VERSION = "20210214"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H4440",  # Unspecified hypotony of eye
        "H44411",  # Flat anterior chamber hypotony of right eye
        "H44412",  # Flat anterior chamber hypotony of left eye
        "H44413",  # Flat anterior chamber hypotony of eye, bilateral
        "H44421",  # Hypotony of right eye due to ocular fistula
        "H44422",  # Hypotony of left eye due to ocular fistula
        "H44423",  # Hypotony of eye due to ocular fistula, bilateral
        "H44431",  # Hypotony of eye due to other ocular disorders, right eye
        "H44432",  # Hypotony of eye due to other ocular disorders, left eye
        "H44433",  # Hypotony of eye due to other ocular disorders, bilateral
        "H44441",  # Primary hypotony of right eye
        "H44442",  # Primary hypotony of left eye
        "H44443",  # Primary hypotony of eye, bilateral
    }

    SNOMEDCT = {
        "19721008",  # Hypotony of eye (disorder)
        "2251002",  # Primary hypotony of eye (disorder)
        "424497004",  # Optic disc edema due to hypotony (disorder)
        "53885001",  # Hypotony of eye co-occurrent and due to ocular fistula (disorder)
        "677291000119105",  # Hypotony of bilateral eyes due to ocular fistula (disorder)
        "677301000119106",  # Hypotony of left eye due to ocular fistula (disorder)
        "677311000119109",  # Hypotony of right eye due to ocular fistula (disorder)
        "700280001",  # Flat anterior chamber of eye with hypotony (disorder)
        "95255005",  # Secondary hypotony of eye (disorder)
    }

class InjuryToOpticNerveAndPathways(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of injury to optic nerve and optic pathways.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of injuries to optic nerve, optic chiasm, optic tract, optic pathways and optic visual cortex.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Injury to Optic Nerve and Pathways"
    OID = "2.16.840.1.113883.3.526.3.1427"
    DEFINITION_VERSION = "20210214"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "S04011A",  # Injury of optic nerve, right eye, initial encounter
        "S04012A",  # Injury of optic nerve, left eye, initial encounter
        "S0402XA",  # Injury of optic chiasm, initial encounter
        "S04031A",  # Injury of optic tract and pathways, right side, initial encounter
        "S04032A",  # Injury of optic tract and pathways, left side, initial encounter
        "S04041A",  # Injury of visual cortex, right side, initial encounter
        "S04042A",  # Injury of visual cortex, left side, initial encounter
    }

    SNOMEDCT = {
        "1339033009",  # Disorder of optic nerve caused by radiation (disorder)
        "212141007",  # Optic nerve and pathway injury (disorder)
        "212142000",  # Visual cortex injury (disorder)
        "230513000",  # Avulsion of optic nerve (disorder)
        "230519001",  # Injury of optic tract (disorder)
        "48233004",  # Traumatic optic nerve injury (disorder)
        "69820004",  # Injury of optic chiasm (disorder)
        "735646005",  # Injury of bilateral optic nerves (disorder)
        "735648006",  # Injury of bilateral optic pathways (disorder)
        "735649003",  # Injury of bilateral optic tracts (disorder)
        "736321008",  # Injury of both visual cortices (disorder)
        "762357005",  # Traumatic injury of left optic nerve (disorder)
        "762358000",  # Traumatic injury of right optic nerve (disorder)
        "762362006",  # Traumatic injury of left optic tract (disorder)
        "762363001",  # Traumatic injury of right optic tract (disorder)
        "762436001",  # Injury of left visual cortex (disorder)
        "762617003",  # Injury of right visual cortex (disorder)
        "80499003",  # Traumatic injury of visual pathways (disorder)
    }

class MacularScarOfPosteriorPolar(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of macular scar of posterior polar.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of macular or posterior pole scarring.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Macular Scar of Posterior Polar"
    OID = "2.16.840.1.113883.3.526.3.1559"
    DEFINITION_VERSION = "20210218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H31011",  # Macula scars of posterior pole (postinflammatory) (post-traumatic), right eye
        "H31012",  # Macula scars of posterior pole (postinflammatory) (post-traumatic), left eye
        "H31013",  # Macula scars of posterior pole (postinflammatory) (post-traumatic), bilateral
    }

    SNOMEDCT = {
        "1137552000",  # Scar of posterior pole of retina of left eye (disorder)
        "1137553005",  # Scar of posterior pole of retina of right eye (disorder)
        "1137554004",  # Scar of posterior pole of retina of bilateral eyes (disorder)
        "1222888000",  # Scar of posterior pole of retina (disorder)
        "15631531000119100",  # Scarred macula of bilateral eyes (disorder)
        "15631611000119106",  # Scarred macula of right eye (disorder)
        "15710081000119102",  # Scar of macula of right eye following trauma (disorder)
        "15710121000119100",  # Scar of macula of bilateral eyes following trauma (disorder)
        "15710161000119105",  # Scar of macula of left eye following trauma (disorder)
        "18410006",  # Scarred macula (disorder)
        "30537002",  # Peripheral scar of posterior pole of retina of eye (disorder)
        "344341000119106",  # Postinflammatory chorioretinal scar of bilateral eyes (disorder)
        "425622004",  # Scar of macula following trauma (disorder)
        "737273002",  # Degeneration of posterior pole of eye (disorder)
        "774010000",  # Scarred macula of left eye (disorder)
    }

class MorgagnianCataract(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of morgagnian cataract.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of a morgagnian cataract.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Morgagnian Cataract"
    OID = "2.16.840.1.113883.3.526.3.1558"
    DEFINITION_VERSION = "20210214"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H2521",  # Age-related cataract, morgagnian type, right eye
        "H2522",  # Age-related cataract, morgagnian type, left eye
        "H2523",  # Age-related cataract, morgagnian type, bilateral
    }

    SNOMEDCT = {
        "264443002",  # Morgagnian cataract (disorder)
    }

class NystagmusAndOtherIrregularEyeMovements(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of diagnoses for nystagmus and other irregular eye movements.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of nystagmus.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Nystagmus and Other Irregular Eye Movements"
    OID = "2.16.840.1.113883.3.526.3.1465"
    DEFINITION_VERSION = "20210308"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H5500",  # Unspecified nystagmus
        "H5501",  # Congenital nystagmus
        "H5502",  # Latent nystagmus
        "H5503",  # Visual deprivation nystagmus
        "H5504",  # Dissociated nystagmus
        "H5509",  # Other forms of nystagmus
        "H5581",  # Deficient saccadic eye movements
        "H5582",  # Deficient smooth pursuit eye movements
        "H5589",  # Other irregular eye movements
    }

    SNOMEDCT = {
        "1010460006",  # Uniocular nystagmus of right eye (disorder)
        "1010461005",  # Uniocular nystagmus of left eye (disorder)
        "111533001",  # Vertical nystagmus (disorder)
        "1208860008",  # Acquired periodic alternating nystagmus (disorder)
        "1217636004",  # Cyclovergence nystagmus (disorder)
        "1217639006",  # Congenital periodic alternating nystagmus (disorder)
        "1217642000",  # Centripetal nystagmus (disorder)
        "1217662008",  # Disjunctive pendular nystagmus (disorder)
        "1220537002",  # Gaze-evoked nystagmus (disorder)
        "194360001",  # Central positional nystagmus (disorder)
        "232124002",  # Voluntary nystagmus (disorder)
        "246758003",  # Mixed nystagmus (disorder)
        "246759006",  # Irregular nystagmus (disorder)
        "246761002",  # Uniocular nystagmus (disorder)
        "246762009",  # Gaze paretic nystagmus (disorder)
        "246764005",  # Postural nystagmus (disorder)
        "267664003",  # Nystagmus and other irregular eye movements (disorder)
        "278465006",  # Spontaneous nystagmus (disorder)
        "278467003",  # Neck positional nystagmus (disorder)
        "278468008",  # Pressure-induced nystagmus (disorder)
        "278469000",  # Sound intensity-induced nystagmus (disorder)
        "285764004",  # Manifest nystagmus (disorder)
        "285765003",  # Manifest-latent nystagmus (disorder)
        "29356006",  # End-position nystagmus (disorder)
        "307670000",  # Congenital nystagmus with sensory abnormality (disorder)
        "307671001",  # Congenital nystagmus without sensory abnormality (disorder)
        "307672008",  # Oculopalatal myoclonus (disorder)
        "307673003",  # Peripheral vestibular nystagmus (disorder)
        "307674009",  # Benign paroxysmal positional vertigo nystagmus (disorder)
        "307675005",  # Central vestibular nystagmus (disorder)
        "307676006",  # Downbeat nystagmus (disorder)
        "307677002",  # Upbeat nystagmus (disorder)
        "307678007",  # Torsional central vestibular nystagmus (disorder)
        "307679004",  # Hemi-seesaw nystagmus (disorder)
        "307680001",  # Abducting nystagmus (disorder)
        "307700006",  # Ataxic nystagmus (disorder)
        "314935002",  # Acquired nystagmus (disorder)
        "3327002",  # Jerk nystagmus (disorder)
        "35743001",  # Pendular nystagmus (disorder)
        "359687001",  # Acquired pendular nystagmus (disorder)
        "373590007",  # Convergence retraction nystagmus (disorder)
        "39737009",  # Symptomatic nystagmus (disorder)
        "400948003",  # Spasmus nutans (disorder)
        "400949006",  # Divergence retraction nystagmus (disorder)
        "404676002",  # Bruns nystagmus (disorder)
        "404677006",  # Elliptical nystagmus (disorder)
        "40782004",  # Hennebert's syndrome (disorder)
        "414489005",  # Infantile nystagmus syndrome (disorder)
        "418472007",  # Heartbeat nystagmus (disorder)
        "431951000124101",  # Idiopathic nystagmus (disorder)
        "44526006",  # Rotary nystagmus (disorder)
        "45339001",  # Visual deprivation nystagmus (disorder)
        "46888001",  # Nystagmus associated with disorder of the vestibular system (disorder)
        "563001",  # Nystagmus (disorder)
        "64635004",  # Congenital nystagmus (disorder)
        "68610001",  # Latent nystagmus (disorder)
        "6881000119109",  # Sensory nystagmus (disorder)
        "733334001",  # Perverted nystagmus (disorder)
        "733335000",  # Eyelid nystagmus (disorder)
        "762438000",  # Physiological nystagmus (disorder)
        "762439008",  # Torsional nystagmus (disorder)
        "783562005",  # Syndactyly, nystagmus syndrome due to 2q31.1 microduplication (disorder)
        "8134005",  # Periodic alternating nystagmus (disorder)
        "81756001",  # Horizontal nystagmus (disorder)
        "83655001",  # Rebound nystagmus (disorder)
        "9074007",  # Retraction nystagmus (disorder)
        "9520006",  # Dissociated nystagmus (disorder)
        "95777008",  # Temporary nystagmus (disorder)
        "95778003",  # Permanent nystagmus (disorder)
        "95779006",  # Unidirectional nystagmus (disorder)
        "95780009",  # Multidirectional nystagmus (disorder)
        "95781008",  # Conjugate nystagmus (disorder)
        "95782001",  # See-saw nystagmus (disorder)
        "95783006",  # Rotational nystagmus (disorder)
        "95784000",  # Convergence nystagmus (disorder)
        "95785004",  # Positional nystagmus (disorder)
        "95786003",  # Fatigable positional nystagmus (disorder)
        "95787007",  # Non-fatigable positional nystagmus (disorder)
        "95788002",  # Positional nystagmus, variable eye direction (disorder)
        "95789005",  # Nystagmus produced by neck rotation (disorder)
        "95790001",  # Fixation nystagmus (disorder)
        "95791002",  # Nystagmus enhanced when fixation removed (disorder)
        "95792009",  # Nystagmus unchanged when fixation removed (disorder)
        "95793004",  # Nystagmus inhibited when fixation removed (disorder)
    }

class OpenWoundOfEyeball(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of open wound of the eyeball.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of choroidal rupture, traumatic enucleation of eye, and avulsion, contusion, laceration, penetration and/or rupture of eye.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Open Wound of Eyeball"
    OID = "2.16.840.1.113883.3.526.3.1430"
    DEFINITION_VERSION = "20210406"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H31321",  # Choroidal rupture, right eye
        "H31322",  # Choroidal rupture, left eye
        "H31323",  # Choroidal rupture, bilateral
        "S0511XA",  # Contusion of eyeball and orbital tissues, right eye, initial encounter
        "S0512XA",  # Contusion of eyeball and orbital tissues, left eye, initial encounter
        "S0521XA",  # Ocular laceration and rupture with prolapse or loss of intraocular tissue, right eye, initial encounter
        "S0522XA",  # Ocular laceration and rupture with prolapse or loss of intraocular tissue, left eye, initial encounter
        "S0531XA",  # Ocular laceration without prolapse or loss of intraocular tissue, right eye, initial encounter
        "S0532XA",  # Ocular laceration without prolapse or loss of intraocular tissue, left eye, initial encounter
        "S0551XA",  # Penetrating wound with foreign body of right eyeball, initial encounter
        "S0552XA",  # Penetrating wound with foreign body of left eyeball, initial encounter
        "S0561XA",  # Penetrating wound without foreign body of right eyeball, initial encounter
        "S0562XA",  # Penetrating wound without foreign body of left eyeball, initial encounter
        "S0571XA",  # Avulsion of right eye, initial encounter
        "S0572XA",  # Avulsion of left eye, initial encounter
        "S058X1A",  # Other injuries of right eye and orbit, initial encounter
        "S058X2A",  # Other injuries of left eye and orbit, initial encounter
    }

    SNOMEDCT = {
        "10942751000119100",  # Commotio retinae of right retina (disorder)
        "10942791000119105",  # Commotio retinae of left retina (disorder)
        "110262009",  # Contusion of periorbital region (disorder)
        "12193006",  # Avulsion of eye (disorder)
        "125687005",  # Contusion of ocular adnexa (disorder)
        "1869002",  # Rupture of sphincter of pupil (disorder)
        "210285009",  # Ocular laceration with intraocular prolapse (disorder)
        "211483007",  # Contusion of globe of eye (disorder)
        "21549001",  # Superficial laceration of cornea (disorder)
        "231791008",  # Rupture of globe (disorder)
        "231795004",  # Laceration of globe of eye (disorder)
        "231952009",  # Tears of iris stroma (disorder)
        "23293006",  # Rupture of eye with partial loss of intraocular tissue (disorder)
        "2532009",  # Choroidal rupture (disorder)
        "3018008",  # Penetration of eyeball with magnetic foreign body (disorder)
        "315296002",  # Penetrating wound of eye (disorder)
        "346301000119100",  # Rupture of Descemet membrane of bilateral eyes (disorder)
        "367423000",  # Contusion of eye (disorder)
        "37911007",  # Ocular laceration with prolapse AND/OR exposure of intraocular tissue (disorder)
        "416179002",  # Contusion of eye AND ocular adnexa (disorder)
        "41658006",  # Traumatic enucleation of eye (disorder)
        "416786002",  # Penetrating injury of lens (disorder)
        "416931008",  # Corneoscleral laceration (disorder)
        "417654008",  # Contusion of ocular adnexa and periocular tissues (disorder)
        "419743001",  # Ruptured Descemet membrane due to birth trauma (disorder)
        "42241000087103",  # Penetrating wound of right eye proper due to foreign body (disorder)
        "42312000",  # Rupture in Descemet's membrane (disorder)
        "42697001",  # Contusion of orbital tissues (disorder)
        "44199000",  # Penetration of eyeball with nonmagnetic foreign body (disorder)
        "722613000",  # Penetrating wound of eyeball without foreign body (disorder)
        "735657000",  # Penetrating injury of both eyeballs (disorder)
        "85100000",  # Open wound of eyeball (disorder)
        "870312007",  # Penetrating wound of eyeball with foreign body (disorder)
        "95718009",  # Rupture of lens (disorder)
        "95725002",  # Corneal laceration (disorder)
    }

class OpticAtrophy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of diagnoses for optic atrophy.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of optic atrophy, which may include atrophy of the optic disc.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Optic Atrophy"
    OID = "2.16.840.1.113883.3.526.3.1466"
    DEFINITION_VERSION = "20210218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H4720",  # Unspecified optic atrophy
        "H47211",  # Primary optic atrophy, right eye
        "H47212",  # Primary optic atrophy, left eye
        "H47213",  # Primary optic atrophy, bilateral
        "H4722",  # Hereditary optic atrophy
        "H47231",  # Glaucomatous optic atrophy, right eye
        "H47232",  # Glaucomatous optic atrophy, left eye
        "H47233",  # Glaucomatous optic atrophy, bilateral
        "H47291",  # Other optic atrophy, right eye
        "H47292",  # Other optic atrophy, left eye
        "H47293",  # Other optic atrophy, bilateral
    }

    SNOMEDCT = {
        "111527005",  # Partial optic atrophy (disorder)
        "1207009",  # Glaucomatous atrophy of optic disc (disorder)
        "123615007",  # Secondary optic atrophy (disorder)
        "128203003",  # Hereditary motor and sensory neuropathy with optic atrophy (disorder)
        "194043004",  # Optic atrophy secondary to retinal disease (disorder)
        "19448008",  # Optic atrophy associated with retinal dystrophy (disorder)
        "2065009",  # Dominant hereditary optic atrophy (disorder)
        "21098003",  # Primary optic atrophy (disorder)
        "22386003",  # Syphilitic optic atrophy (disorder)
        "230511003",  # Optic atrophy secondary to papilledema (disorder)
        "239028001",  # Odontotrichomelic syndrome (disorder)
        "24003004",  # Postinflammatory optic atrophy (disorder)
        "247219000",  # Atrophy of sector of optic disc (disorder)
        "26360005",  # Hereditary optic atrophy (disorder)
        "312944002",  # Compressive optic atrophy (disorder)
        "334941000119106",  # Atrophy of optic disc of right eye due to glaucoma (disorder)
        "340551000119105",  # Atrophy of optic disc of left eye due to glaucoma (disorder)
        "345861000119103",  # Atrophy of optic disc of bilateral eyes due to glaucoma (disorder)
        "348051000119102",  # Atrophy of right optic nerve following inflammation (disorder)
        "348501000119108",  # Atrophy of left optic nerve following inflammation (disorder)
        "359683002",  # Complete optic atrophy (disorder)
        "58610003",  # Leber's optic atrophy (disorder)
        "680991000119108",  # Atrophy of bilateral optic nerves following inflammation (disorder)
        "70694009",  # Wolfram syndrome (disorder)
        "715374003",  # Autosomal dominant optic atrophy plus syndrome (disorder)
        "717336005",  # Autosomal dominant optic atrophy classic form (disorder)
        "717975006",  # Autosomal dominant optic atrophy and peripheral neuropathy syndrome (disorder)
        "719517009",  # Autosomal dominant optic atrophy and cataract (disorder)
        "76976005",  # Optic atrophy (disorder)
        "838307002",  # Childhood-onset autosomal dominant optic atrophy (disorder)
        "838345001",  # Autosomal recessive optic atrophy type 6 (disorder)
        "870319003",  # Optic atrophy due to late syphilis (disorder)
        "95774001",  # Atrophy of optic disc (disorder)
        "95775000",  # Retrobulbar optic nerve atrophy (disorder)
    }

class OpticNeuritis(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of optic neuritis.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of optic neuritis, neuropathy, neuroretinitis, and papillitis.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Optic Neuritis"
    OID = "2.16.840.1.113883.3.526.3.1467"
    DEFINITION_VERSION = "20210218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H4601",  # Optic papillitis, right eye
        "H4602",  # Optic papillitis, left eye
        "H4603",  # Optic papillitis, bilateral
        "H4611",  # Retrobulbar neuritis, right eye
        "H4612",  # Retrobulbar neuritis, left eye
        "H4613",  # Retrobulbar neuritis, bilateral
        "H462",  # Nutritional optic neuropathy
        "H463",  # Toxic optic neuropathy
        "H468",  # Other optic neuritis
        "H469",  # Unspecified optic neuritis
    }

    SNOMEDCT = {
        "15631011000119102",  # Neuritis of right optic nerve (disorder)
        "15631051000119101",  # Neuritis of left optic nerve (disorder)
        "230507009",  # Retrobulbar neuritis (disorder)
        "25044007",  # Neuromyelitis optica (disorder)
        "26125006",  # Toxic optic neuropathy (disorder)
        "2691000124105",  # Recurrent optic neuritis (disorder)
        "314537004",  # Optic papillopathy due to diabetes mellitus (disorder)
        "314538009",  # Tobacco related optic neuropathy (disorder)
        "314539001",  # Alcohol related optic neuropathy (disorder)
        "314540004",  # Drug induced optic neuropathy (disorder)
        "36276008",  # Syphilitic retrobulbar neuritis (disorder)
        "410471004",  # Neuroretinitis (disorder)
        "410472006",  # Bartonella henselae neuroretinitis (disorder)
        "416913007",  # Toxoplasma neuroretinitis (disorder)
        "420920007",  # Optic neuropathy due to vitamin B12 deficiency (disorder)
        "432521000124106",  # Bilateral optic neuritis (disorder)
        "51604006",  # Acute retrobulbar neuritis (disorder)
        "66760008",  # Optic neuritis (disorder)
        "73221001",  # Optic papillitis (disorder)
        "73431005",  # Meningococcal optic neuritis (disorder)
        "82108004",  # Disorder of optic nerve due to nutritional deficiency (disorder)
    }

class OtherBackgroundRetinopathyAndRetinalVascularChanges(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of other background retinopathy and retinal vascular changes.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of various retinal changes and retinopathies, including exudative retinopathy, retinal neovascularization, arteriolitis, arteritis, endarteritis, phlebitis, and vasculitis.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Other Background Retinopathy and Retinal Vascular Changes"
    OID = "2.16.840.1.113883.3.526.3.1469"
    DEFINITION_VERSION = "20210218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H35021",  # Exudative retinopathy, right eye
        "H35022",  # Exudative retinopathy, left eye
        "H35023",  # Exudative retinopathy, bilateral
        "H35051",  # Retinal neovascularization, unspecified, right eye
        "H35052",  # Retinal neovascularization, unspecified, left eye
        "H35053",  # Retinal neovascularization, unspecified, bilateral
        "H35061",  # Retinal vasculitis, right eye
        "H35062",  # Retinal vasculitis, left eye
        "H35063",  # Retinal vasculitis, bilateral
    }

    SNOMEDCT = {
        "11255009",  # Retinal arteritis (disorder)
        "247099009",  # Optic disc neovascularization (disorder)
        "247100001",  # Peripheral retinal neovascularization (disorder)
        "25506007",  # Exudative retinopathy (disorder)
        "314265001",  # Classic choroidal neovascular membrane (disorder)
        "332421000119101",  # Retinal vasculitis of right eye (disorder)
        "338031000119109",  # Retinal vasculitis of left eye (disorder)
        "343801000119100",  # Retinal vasculitis of bilateral eyes (disorder)
        "414297000",  # Frosted branch angiitis (disorder)
        "416703007",  # Retinal vasculitis due to polyarteritis nodosa (disorder)
        "417019000",  # Retinal arteriolitis (disorder)
        "417303004",  # Retinal vasculitis due to systemic lupus erythematosus (disorder)
        "417544004",  # Segmental retinal periarteritis (disorder)
        "54122009",  # Eales' disease (disorder)
        "61267008",  # Retinal neovascularization (disorder)
        "677651000119102",  # Neovascularization of choroid of left eye (disorder)
        "677681000119109",  # Neovascularization of choroid of right eye (disorder)
        "7089008",  # Retinal phlebitis (disorder)
        "71719003",  # Thrombophlebitis of retinal vein (disorder)
        "75971007",  # Choroidal retinal neovascularization (disorder)
        "77628002",  # Retinal vasculitis (disorder)
        "86191000",  # Retinal endarteritis (disorder)
    }

class OtherDisordersOfOpticNerve(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis for other disorders of optic nerve.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of ischemic optic neuropathy.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Other Disorders of Optic Nerve"
    OID = "2.16.840.1.113883.3.526.3.1471"
    DEFINITION_VERSION = "20210218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H47011",  # Ischemic optic neuropathy, right eye
        "H47012",  # Ischemic optic neuropathy, left eye
        "H47013",  # Ischemic optic neuropathy, bilateral
    }

    SNOMEDCT = {
        "1231226002",  # Non-arteritic posterior ischemic optic neuropathy (disorder)
        "14357004",  # Ischemic optic neuropathy (disorder)
        "15731601000119100",  # Anterior ischemic optic neuropathy of left eye (disorder)
        "15731641000119103",  # Anterior ischemic optic neuropathy of right eye (disorder)
        "16376191000119109",  # Arteritic anterior ischemic optic neuropathy of bilateral eyes (disorder)
        "16376241000119109",  # Arteritic anterior ischemic optic neuropathy of left eye (disorder)
        "16376391000119106",  # Arteritic anterior ischemic optic neuropathy of right eye (disorder)
        "230508004",  # Arteritic ischemic optic neuropathy (disorder)
        "230509007",  # Non-arteritic ischemic optic neuropathy (disorder)
        "334651000119101",  # Ischemic optic neuropathy of right eye (disorder)
        "340251000119108",  # Ischemic optic neuropathy of left eye (disorder)
        "345611000119102",  # Ischemic optic neuropathy of bilateral eyes (disorder)
        "404659001",  # Anterior ischemic optic neuropathy (disorder)
        "415147002",  # Posterior ischemic optic neuropathy (disorder)
        "733506009",  # Arteritic anterior ischemic optic neuropathy (disorder)
        "838324008",  # Autoimmune optic neuropathy (disorder)
        "838363001",  # Arteritic anterior ischemic optic neuropathy due to giant cell arteritis (disorder)
        "860808008",  # Idiopathic non-arteritic ischemic optic neuropathy (disorder)
        "860884007",  # Idiopathic non-arteritic posterior ischemic optic neuropathy (disorder)
    }

class OtherEndophthalmitis(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of other endophthalmitis.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of ophthalmia nodosa, parasitic cyst or endophthalmitis, panuveitis, sympathetic uveitis, or other endophthalmitis diagnoses.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Other Endophthalmitis"
    OID = "2.16.840.1.113883.3.526.3.1473"
    DEFINITION_VERSION = "20210218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H16241",  # Ophthalmia nodosa, right eye
        "H16242",  # Ophthalmia nodosa, left eye
        "H16243",  # Ophthalmia nodosa, bilateral
        "H21331",  # Parasitic cyst of iris, ciliary body or anterior chamber, right eye
        "H21332",  # Parasitic cyst of iris, ciliary body or anterior chamber, left eye
        "H21333",  # Parasitic cyst of iris, ciliary body or anterior chamber, bilateral
        "H33121",  # Parasitic cyst of retina, right eye
        "H33122",  # Parasitic cyst of retina, left eye
        "H33123",  # Parasitic cyst of retina, bilateral
        "H44111",  # Panuveitis, right eye
        "H44112",  # Panuveitis, left eye
        "H44113",  # Panuveitis, bilateral
        "H44121",  # Parasitic endophthalmitis, unspecified, right eye
        "H44122",  # Parasitic endophthalmitis, unspecified, left eye
        "H44123",  # Parasitic endophthalmitis, unspecified, bilateral
        "H44131",  # Sympathetic uveitis, right eye
        "H44132",  # Sympathetic uveitis, left eye
        "H44133",  # Sympathetic uveitis, bilateral
        "H4419",  # Other endophthalmitis
    }

    SNOMEDCT = {
        "1176990003",  # Panuveitis caused by Fungi (disorder)
        "1177028005",  # Panuveitis caused by Bacteria (disorder)
        "1231680001",  # Panuveitis due to sarcoidosis (disorder)
        "12371008",  # Ophthalmia nodosa (disorder)
        "231992006",  # Onchocerca chorioretinitis (disorder)
        "240842000",  # Ocular onchocerciasis (disorder)
        "333391000119101",  # Panuveitis of right eye (disorder)
        "333581000119106",  # Ophthalmia nodosa of right eye (disorder)
        "339001000119107",  # Panuveitis of left eye (disorder)
        "339191000119100",  # Ophthalmia nodosa of left eye (disorder)
        "344751000119109",  # Panuveitis of bilateral eyes (disorder)
        "344941000119105",  # Ophthalmia nodosa of bilateral eyes (disorder)
        "415737009",  # Toxocara endophthalmitis (disorder)
        "57100005",  # Parasitic endophthalmitis (disorder)
        "733337008",  # Noninfectious panuveitis (disorder)
        "75315001",  # Sympathetic uveitis (disorder)
        "75614007",  # Panuveitis (disorder)
        "766933000",  # Idiopathic panuveitis (disorder)
        "789116000",  # Panuveitis due to infectious disease (disorder)
        "870329005",  # Panuveitis caused by virus (disorder)
        "870330000",  # Panuveitis caused by parasite (disorder)
        "870331001",  # Panuveitis caused by Treponema pallidum (disorder)
        "870332008",  # Panuveitis caused by Herpesvirus (disorder)
        "870333003",  # Panuveitis caused by Mycobacterium tuberculosis (disorder)
        "870335005",  # Panuveitis caused by Borrelia burgdorferi (disorder)
    }

class OtherProliferativeRetinopathy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of other proliferative retinopathy.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of premature retinopathy.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Other Proliferative Retinopathy"
    OID = "2.16.840.1.113883.3.526.3.1480"
    DEFINITION_VERSION = "20210218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H35101",  # Retinopathy of prematurity, unspecified, right eye
        "H35102",  # Retinopathy of prematurity, unspecified, left eye
        "H35103",  # Retinopathy of prematurity, unspecified, bilateral
        "H35111",  # Retinopathy of prematurity, stage 0, right eye
        "H35112",  # Retinopathy of prematurity, stage 0, left eye
        "H35113",  # Retinopathy of prematurity, stage 0, bilateral
        "H35121",  # Retinopathy of prematurity, stage 1, right eye
        "H35122",  # Retinopathy of prematurity, stage 1, left eye
        "H35123",  # Retinopathy of prematurity, stage 1, bilateral
        "H35131",  # Retinopathy of prematurity, stage 2, right eye
        "H35132",  # Retinopathy of prematurity, stage 2, left eye
        "H35133",  # Retinopathy of prematurity, stage 2, bilateral
        "H35141",  # Retinopathy of prematurity, stage 3, right eye
        "H35142",  # Retinopathy of prematurity, stage 3, left eye
        "H35143",  # Retinopathy of prematurity, stage 3, bilateral
        "H35151",  # Retinopathy of prematurity, stage 4, right eye
        "H35152",  # Retinopathy of prematurity, stage 4, left eye
        "H35153",  # Retinopathy of prematurity, stage 4, bilateral
        "H35161",  # Retinopathy of prematurity, stage 5, right eye
        "H35162",  # Retinopathy of prematurity, stage 5, left eye
        "H35163",  # Retinopathy of prematurity, stage 5, bilateral
        "H35171",  # Retrolental fibroplasia, right eye
        "H35172",  # Retrolental fibroplasia, left eye
        "H35173",  # Retrolental fibroplasia, bilateral
    }

    SNOMEDCT = {
        "332351000119103",  # Retinopathy of prematurity stage 5 of right eye (disorder)
        "332361000119101",  # Retinopathy of prematurity of right eye stage 4 - subtotal retinal detachment (disorder)
        "332371000119107",  # Retinopathy of prematurity of right eye stage 3 - ridge with extraretinal fibrovascular proliferation (disorder)
        "332381000119105",  # Retinopathy of prematurity of right eye stage 2 - intraretinal ridge (disorder)
        "332391000119108",  # Retinopathy of prematurity of right eye stage 1 - demarcation line (disorder)
        "332411000119108",  # Retinopathy of prematurity of right eye (disorder)
        "337961000119104",  # Retinopathy of prematurity stage 5 of left eye (disorder)
        "337971000119105",  # Retinopathy of prematurity of left eye stage 4 - subtotal retinal detachment (disorder)
        "337981000119108",  # Retinopathy of prematurity of left eye stage 3 - ridge with extraretinal fibrovascular proliferation (disorder)
        "337991000119106",  # Retinopathy of prematurity of left eye stage 2 - intraretinal ridge (disorder)
        "338001000119102",  # Retinopathy of prematurity of left eye stage 1 - demarcation line (disorder)
        "338021000119106",  # Retinopathy of prematurity of left eye (disorder)
        "343731000119100",  # Retinopathy of prematurity stage 5 of bilateral eyes (disorder)
        "343741000119109",  # Retinopathy of prematurity of bilateral eyes stage 4 - subtotal retinal detachment (disorder)
        "343751000119106",  # Retinopathy of prematurity of bilateral eyes stage 3 - ridge with extraretinal fibrovascular proliferation (disorder)
        "343761000119108",  # Retinopathy of prematurity of bilateral eyes stage 2 - intraretinal ridge (disorder)
        "343771000119102",  # Retinopathy of prematurity of bilateral eyes stage 1 - demarcation line (disorder)
        "343791000119101",  # Retinopathy of prematurity of bilateral eyes (disorder)
        "408847006",  # Retinopathy of prematurity stage 1 - demarcation line (disorder)
        "408848001",  # Retinopathy of prematurity stage 2 - intraretinal ridge (disorder)
        "408849009",  # Retinopathy of prematurity stage 3 - ridge with extraretinal fibrovascular proliferation (disorder)
        "408850009",  # Retinopathy of prematurity stage 4 - subtotal retinal detachment (disorder)
        "408851008",  # Retinopathy of prematurity stage 5 - total retinal detachment (disorder)
        "415297005",  # Retinopathy of prematurity (disorder)
    }

class OtherAndUnspecifiedFormsOfChorioretinitisAndRetinochoroiditis(ValueSet):
    """
    **Clinical Focus:** Includes concepts that represent a diagnosis of posterior cyclitis, chorioretinal inflammation, other chorioretinitis or retinitis or retinochoroiditis.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes only relevant concepts associated with a diagnosis of posterior cyclitis, chorioretinal inflammation, other chorioretinitis or retinitis or retinochoroiditis.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Other and Unspecified Forms of Chorioretinitis and Retinochoroiditis"
    OID = "2.16.840.1.113883.3.526.3.1468"
    DEFINITION_VERSION = "20210219"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H3021",  # Posterior cyclitis, right eye
        "H3022",  # Posterior cyclitis, left eye
        "H3023",  # Posterior cyclitis, bilateral
        "H30811",  # Harada's disease, right eye
        "H30812",  # Harada's disease, left eye
        "H30813",  # Harada's disease, bilateral
        "H30891",  # Other chorioretinal inflammations, right eye
        "H30892",  # Other chorioretinal inflammations, left eye
        "H30893",  # Other chorioretinal inflammations, bilateral
        "H3091",  # Unspecified chorioretinal inflammation, right eye
        "H3092",  # Unspecified chorioretinal inflammation, left eye
        "H3093",  # Unspecified chorioretinal inflammation, bilateral
    }

    SNOMEDCT = {
        "14894002",  # Generalized disseminated choroiditis AND chorioretinitis (disorder)
        "15677801000119103",  # Chorioretinitis of right eye due to secondary syphilis (disorder)
        "15677881000119106",  # Chorioretinitis of left eye due to secondary syphilis (disorder)
        "15685401000119104",  # Retinitis of bilateral eyes caused by Toxoplasma gondii (disorder)
        "15685441000119102",  # Retinitis of bilateral eyes caused by Treponema pallidum (disorder)
        "15685801000119101",  # Retinitis of right eye (disorder)
        "15685841000119104",  # Retinitis of left eye (disorder)
        "15685881000119109",  # Retinitis of bilateral eyes (disorder)
        "15847003",  # Focal chorioretinitis (disorder)
        "15992631000119101",  # Neuroretinitis of right eye (disorder)
        "15992671000119103",  # Neuroretinitis of left eye (disorder)
        "15992711000119104",  # Neuroretinitis of bilateral eyes (disorder)
        "15993391000119102",  # Retinitis of bilateral eyes due to secondary syphilis (disorder)
        "186263000",  # Tuberculous chorioretinitis (disorder)
        "187039009",  # Histoplasma capsulatum with retinitis (disorder)
        "187194004",  # Toxoplasmosis chorioretinitis (disorder)
        "193434008",  # Focal juxtapapillary choroiditis (disorder)
        "193436005",  # Juxtapapillary focal chorioretinitis (disorder)
        "193438006",  # Peripheral focal retinochoroiditis (disorder)
        "193442009",  # Disseminated posterior pole chorioretinitis (disorder)
        "193443004",  # Disseminated peripheral chorioretinitis (disorder)
        "193444005",  # General disseminated chorioretinitis (disorder)
        "193497004",  # Vogt-Koyanagi-Harada disease (disorder)
        "22455005",  # Cytomegaloviral retinitis (disorder)
        "231981005",  # Birdshot chorioretinitis (disorder)
        "231984002",  # Viral retinitis (disorder)
        "231988004",  # Bacterial chorioretinitis (disorder)
        "231990003",  # Candida retinitis (disorder)
        "231992006",  # Onchocerca chorioretinitis (disorder)
        "231994007",  # Toxocara chorioretinitis (disorder)
        "312491004",  # Serpiginous choroiditis (disorder)
        "312930008",  # Sarcoid chorioretinitis (disorder)
        "312934004",  # Syphilitic chorioretinitis (disorder)
        "312955002",  # Tertiary syphilitic chorioretinitis (disorder)
        "314031009",  # Acute toxoplasmosis chorioretinitis (disorder)
        "314032002",  # Inactive toxoplasmosis chorioretinitis (finding)
        "314428001",  # Posterior cyclitis (disorder)
        "345241000119101",  # Macular focal choroiditis of bilateral eyes (disorder)
        "345251000119104",  # Macular focal chorioretinitis of bilateral eyes (disorder)
        "345971000119104",  # Generalized disseminated chorioretinitis of bilateral eyes (disorder)
        "346251000119106",  # Disseminated chorioretinitis of bilateral eyes (disorder)
        "359668007",  # Focal retinitis (disorder)
        "359698000",  # Parasitic chorioretinitis (disorder)
        "359707007",  # Fungal chorioretinitis (disorder)
        "36535003",  # Peripheral focal retinitis AND retinochoroiditis (disorder)
        "371099005",  # Generalized disseminated choroiditis AND chorioretinitis with acute necrosis (disorder)
        "373667006",  # Focal chorioretinitis of posterior pole of eye (disorder)
        "399463004",  # Retinitis (disorder)
        "410470003",  # Syphilitic retinitis (disorder)
        "414783007",  # Multifocal choroiditis (disorder)
        "415218004",  # Punctate outer retinal toxoplasmosis (disorder)
        "416589006",  # Toxoplasma retinitis (disorder)
        "416877003",  # Cryptococcal chorioretinitis (disorder)
        "417405005",  # Chorioretinitis with coccidioidmycosis (disorder)
        "417547006",  # Cryptococcal retinitis (disorder)
        "42031001",  # Macular focal retinitis AND retinochoroiditis (disorder)
        "421029004",  # Herpes zoster retinitis (disorder)
        "441854007",  # Reactivation of toxoplasmosis chorioretinitis (disorder)
        "46497009",  # Juxtapapillary focal retinitis AND retinochoroiditis (disorder)
        "46627006",  # Chorioretinitis (disorder)
        "48225000",  # Tuberculous disseminated chorioretinitis (disorder)
        "51579006",  # Disseminated choroiditis AND chorioretinitis, posterior pole (disorder)
        "56787009",  # Peripheral focal choroiditis AND chorioretinitis (disorder)
        "61517002",  # Metastatic disseminated retinitis AND retinochoroiditis (disorder)
        "61922005",  # Disseminated retinitis (disorder)
        "6313001",  # Juxtapapillary focal choroiditis AND chorioretinitis (disorder)
        "66281009",  # Chorioretinitis due to secondary syphilis (disorder)
        "69811008",  # Peripheral disseminated choroiditis AND chorioretinitis (disorder)
        "699515009",  # Focal retinitis of posterior pole of eye (disorder)
        "699763005",  # Focal choroiditis of posterior pole of eye (disorder)
        "711616006",  # Chorioretinitis caused by Cytomegalovirus (disorder)
        "77628002",  # Retinal vasculitis (disorder)
        "77939001",  # Syphilitic disseminated retinochoroiditis (disorder)
        "78769001",  # Disseminated chorioretinitis (disorder)
        "866099002",  # Leber idiopathic stellate neuroretinitis (disorder)
        "866133000",  # Infectious neuroretinitis (disorder)
        "88290000",  # Focal chorioretinitis due to acquired disseminated toxoplasmosis (disorder)
        "95682009",  # Granulomatous chorioretinitis (disorder)
        "95685006",  # Granulomatous retinitis (disorder)
        "9941009",  # Congenital syphilitic choroiditis (disorder)
    }

class PathologicMyopia(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis for pathologic myopia.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of degenerative disorders of eye, Forster-Fuchs' spot, malignant or severe myopia, and myopic macular degeneration.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Pathologic Myopia"
    OID = "2.16.840.1.113883.3.526.3.1432"
    DEFINITION_VERSION = "20210218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H4421",  # Degenerative myopia, right eye
        "H4422",  # Degenerative myopia, left eye
        "H4423",  # Degenerative myopia, bilateral
        "H442A1",  # Degenerative myopia with choroidal neovascularization, right eye
        "H442A2",  # Degenerative myopia with choroidal neovascularization, left eye
        "H442A3",  # Degenerative myopia with choroidal neovascularization, bilateral
        "H442B1",  # Degenerative myopia with macular hole, right eye
        "H442B2",  # Degenerative myopia with macular hole, left eye
        "H442B3",  # Degenerative myopia with macular hole, bilateral
        "H442C1",  # Degenerative myopia with retinal detachment, right eye
        "H442C2",  # Degenerative myopia with retinal detachment, left eye
        "H442C3",  # Degenerative myopia with retinal detachment, bilateral
        "H442D1",  # Degenerative myopia with foveoschisis, right eye
        "H442D2",  # Degenerative myopia with foveoschisis, left eye
        "H442D3",  # Degenerative myopia with foveoschisis, bilateral
        "H442E1",  # Degenerative myopia with other maculopathy, right eye
        "H442E2",  # Degenerative myopia with other maculopathy, left eye
        "H442E3",  # Degenerative myopia with other maculopathy, bilateral
        "H4430",  # Unspecified degenerative disorder of globe
    }

    SNOMEDCT = {
        "15628771000119100",  # Forster-Fuchs spot of right eye (disorder)
        "15628811000119100",  # Forster-Fuchs spot of left eye (disorder)
        "15628851000119104",  # Forster-Fuchs spot of bilateral eyes (disorder)
        "16320791000119106",  # Myopic macular degeneration of bilateral eyes (disorder)
        "16320831000119100",  # Myopic macular degeneration of right eye (disorder)
        "16320871000119102",  # Myopic macular degeneration of left eye (disorder)
        "16837401000119105",  # Neovascularization of choroid of bilateral eyes due to degenerative myopia (disorder)
        "16837461000119106",  # Neovascularization of choroid of right eye due to degenerative myopia (disorder)
        "16837521000119102",  # Neovascularization of choroid of left eye due to degenerative myopia (disorder)
        "232072004",  # Forster-Fuchs' spot (disorder)
        "232133000",  # Simple myopia (disorder)
        "232134006",  # Congenital axial myopia (disorder)
        "232135007",  # Index myopia (disorder)
        "312898002",  # Myopic macular degeneration (disorder)
        "32022003",  # Degenerative progressive high myopia (disorder)
        "333701000119108",  # Myopia of right eye (disorder)
        "335471000119108",  # Degenerative progressive high myopia of right eye (disorder)
        "339301000119109",  # Myopia of left eye (disorder)
        "341081000119102",  # Degenerative progressive high myopia of left eye (disorder)
        "34187009",  # Severe myopia (disorder)
        "345061000119109",  # Myopia of bilateral eyes (disorder)
        "346341000119103",  # Degenerative progressive high myopia of bilateral eyes (disorder)
        "408757002",  # Axial myopia (disorder)
        "57190000",  # Myopia (disorder)
        "718718009",  # X-linked cone dysfunction syndrome with myopia (disorder)
        "733087007",  # Polydactyly myopia syndrome (disorder)
        "773771008",  # Rare isolated myopia (disorder)
        "846621001",  # Degenerative myopia of left eye (disorder)
        "846622008",  # Degenerative myopia of right eye (disorder)
    }

class PosteriorLenticonus(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis for posterior lenticonus.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of posterior lenticonus, including spherophakia.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Posterior Lenticonus"
    OID = "2.16.840.1.113883.3.526.3.1433"
    DEFINITION_VERSION = "20210314"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "Q122",  # Coloboma of lens
        "Q124",  # Spherophakia
        "Q128",  # Other congenital lens malformations
    }

    SNOMEDCT = {
        "1230066003",  # Isolated microspherophakia (disorder)
        "15987871000119100",  # Congenital coloboma of right lens (disorder)
        "15987911000119102",  # Congenital coloboma of left lens (disorder)
        "204134008",  # Coloboma of lens (disorder)
        "33521009",  # Congenital anomaly of lens shape (disorder)
        "416671000",  # Microspherophakia (disorder)
        "418653000",  # Secondary lens coloboma (disorder)
        "419544009",  # Primary lens coloboma (disorder)
        "4465002",  # Spherophakia (disorder)
        "773690008",  # Microcornea, posterior megalolenticonus, persistent fetal vasculature, coloboma syndrome (disorder)
        "840458000",  # Coloboma of bilateral lenses (disorder)
        "897576008",  # Spherophakia of bilateral lenses (disorder)
        "95482000",  # Posterior lenticonus (disorder)
        "95483005",  # Posterior lentiglobus (disorder)
    }

class PriorPenetratingKeratoplasty(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of penetrating keratoplasty.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of keratoconus.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Prior Penetrating Keratoplasty"
    OID = "2.16.840.1.113883.3.526.3.1475"
    DEFINITION_VERSION = "20210218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H18601",  # Keratoconus, unspecified, right eye
        "H18602",  # Keratoconus, unspecified, left eye
        "H18603",  # Keratoconus, unspecified, bilateral
        "H18611",  # Keratoconus, stable, right eye
        "H18612",  # Keratoconus, stable, left eye
        "H18613",  # Keratoconus, stable, bilateral
        "H18621",  # Keratoconus, unstable, right eye
        "H18622",  # Keratoconus, unstable, left eye
        "H18623",  # Keratoconus, unstable, bilateral
    }

    SNOMEDCT = {
        "111523009",  # Acute hydrops keratoconus (disorder)
        "193844000",  # Keratoconus, stable condition (disorder)
        "314758008",  # Posterior keratoconus (disorder)
        "334551000119102",  # Unstable keratoconus of right eye (disorder)
        "334561000119100",  # Stable keratoconus of right eye (disorder)
        "334571000119106",  # Keratoconus of right cornea (disorder)
        "340151000119101",  # Unstable keratoconus of left eye (disorder)
        "340161000119104",  # Stable keratoconus of left eye (disorder)
        "340171000119105",  # Keratoconus of left cornea (disorder)
        "345511000119103",  # Unstable keratoconus of bilateral eyes (disorder)
        "345521000119105",  # Stable keratoconus of bilateral eyes (disorder)
        "345531000119108",  # Keratoconus of bilateral corneas (disorder)
        "348371000119109",  # Acute hydrops keratoconus of right eye (disorder)
        "348821000119101",  # Acute hydrops keratoconus of left eye (disorder)
        "349321000119104",  # Acute hydrops keratoconus of bilateral eyes (disorder)
        "63216003",  # Congenital keratoconus (disorder)
        "65636009",  # Keratoconus (disorder)
        "700281002",  # Unstable keratoconus (disorder)
        "75340008",  # Congenital keratoconus posticus circumscriptus (disorder)
    }

class PurulentEndophthalmitis(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of diagnoses for purulent endophthalmitis.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of purulent endophthalmitis, panophthalmitis, vitreous abscess, and other types of endophthalmitis.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Purulent Endophthalmitis"
    OID = "2.16.840.1.113883.3.526.3.1477"
    DEFINITION_VERSION = "20210218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H44001",  # Unspecified purulent endophthalmitis, right eye
        "H44002",  # Unspecified purulent endophthalmitis, left eye
        "H44003",  # Unspecified purulent endophthalmitis, bilateral
        "H44011",  # Panophthalmitis (acute), right eye
        "H44012",  # Panophthalmitis (acute), left eye
        "H44013",  # Panophthalmitis (acute), bilateral
        "H44021",  # Vitreous abscess (chronic), right eye
        "H44022",  # Vitreous abscess (chronic), left eye
        "H44023",  # Vitreous abscess (chronic), bilateral
    }

    SNOMEDCT = {
        "1099241000119105",  # Acute endophthalmitis of bilateral eyes (disorder)
        "1208910001",  # Acute bacterial endophthalmitis following surgical procedure (disorder)
        "1217670003",  # Chronic endophthalmitis caused by Bacteria following surgical procedure (disorder)
        "1231258007",  # Endogenous endophthalmitis caused by Aspergillus (disorder)
        "1231260009",  # Endogenous endophthalmitis caused by Candida (disorder)
        "1231276000",  # Exogenous endophthalmitis caused by Fungi (disorder)
        "1231279007",  # Endogenous endophthalmitis caused by Fungi (disorder)
        "1231280005",  # Endogenous endophthalmitis caused by Bacteria (disorder)
        "1231646004",  # Bacterial endophthalmitis due to and following traumatic injury (disorder)
        "1231700006",  # Endophthalmitis caused by Bacteria following surgical procedure (disorder)
        "1285506008",  # Endophthalmitis following glaucoma drainage implant surgery (disorder)
        "13978000",  # Chronic endophthalmitis (disorder)
        "1493002",  # Acute endophthalmitis (disorder)
        "15678401000119100",  # Chronic endophthalmitis of bilateral eyes (disorder)
        "15678441000119103",  # Chronic endophthalmitis of left eye (disorder)
        "15678481000119108",  # Chronic endophthalmitis of right eye (disorder)
        "15681561000119104",  # Endophthalmitis of right eye (disorder)
        "15681681000119107",  # Endophthalmitis of bilateral eyes (disorder)
        "15681721000119101",  # Endophthalmitis of left eye (disorder)
        "1847009",  # Endophthalmitis (disorder)
        "314544008",  # Postoperative endophthalmitis (disorder)
        "314545009",  # Acute postoperative endophthalmitis (disorder)
        "314546005",  # Chronic postoperative endophthalmitis (disorder)
        "314547001",  # Post-traumatic endophthalmitis (disorder)
        "314548006",  # Endogenous endophthalmitis (disorder)
        "332761000119100",  # Purulent endophthalmitis of right eye (disorder)
        "338371000119105",  # Purulent endophthalmitis of left eye (disorder)
        "348381000119107",  # Acute endophthalmitis of right eye (disorder)
        "348831000119103",  # Acute endophthalmitis of left eye (disorder)
        "410495002",  # Postoperative endophthalmitis caused by Propionibacterium acnes (disorder)
        "410496001",  # Postoperative sterile endophthalmitis (disorder)
        "410497005",  # Postoperative infectious endophthalmitis (disorder)
        "413583003",  # Aspergillus endophthalmitis (disorder)
        "413626002",  # Bacterial endophthalmitis (disorder)
        "413665004",  # Bleb associated endophthalmitis (disorder)
        "413746000",  # Candida endophthalmitis (disorder)
        "414303007",  # Fungal endophthalmitis (disorder)
        "414491002",  # Infectious endophthalmitis (disorder)
        "415182001",  # Propionibacterium acnes endophthalmitis (disorder)
        "415584007",  # Sterile endophthalmitis (disorder)
        "415737009",  # Toxocara endophthalmitis (disorder)
        "416296007",  # Cryptococcal endophthalmitis (disorder)
        "41720003",  # Purulent endophthalmitis (disorder)
        "711644007",  # Endophthalmitis caused by Acanthamoeba (disorder)
        "860836003",  # Endophthalmitis caused by Ascaris lumbricoides (disorder)
    }

class RetinalDetachmentWithRetinalDefect(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis for retinal detachment with a retinal defect.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of various types of retinal detachments.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Retinal Detachment with Retinal Defect"
    OID = "2.16.840.1.113883.3.526.3.1478"
    DEFINITION_VERSION = "20210218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H33001",  # Unspecified retinal detachment with retinal break, right eye
        "H33002",  # Unspecified retinal detachment with retinal break, left eye
        "H33003",  # Unspecified retinal detachment with retinal break, bilateral
        "H33011",  # Retinal detachment with single break, right eye
        "H33012",  # Retinal detachment with single break, left eye
        "H33013",  # Retinal detachment with single break, bilateral
        "H33021",  # Retinal detachment with multiple breaks, right eye
        "H33022",  # Retinal detachment with multiple breaks, left eye
        "H33023",  # Retinal detachment with multiple breaks, bilateral
        "H33031",  # Retinal detachment with giant retinal tear, right eye
        "H33032",  # Retinal detachment with giant retinal tear, left eye
        "H33033",  # Retinal detachment with giant retinal tear, bilateral
        "H33041",  # Retinal detachment with retinal dialysis, right eye
        "H33042",  # Retinal detachment with retinal dialysis, left eye
        "H33043",  # Retinal detachment with retinal dialysis, bilateral
        "H33051",  # Total retinal detachment, right eye
        "H33052",  # Total retinal detachment, left eye
        "H33053",  # Total retinal detachment, bilateral
        "H3341",  # Traction detachment of retina, right eye
        "H3342",  # Traction detachment of retina, left eye
        "H3343",  # Traction detachment of retina, bilateral
        "H338",  # Other retinal detachments
    }

    SNOMEDCT = {
        "1231625008",  # Tractional retinal detachment due to proliferative diabetic retinopathy (disorder)
        "1231650006",  # Combined rhegmatogenous and tractional retinal detachment due to proliferative diabetic retinopathy (disorder)
        "1231698002",  # Tractional retinal detachment with macula involvement due to proliferative diabetic retinopathy (disorder)
        "1231728000",  # Retinal detachment due to degenerative progressive high myopia (disorder)
        "1279846006",  # Detachment of retina of left eye with break of retina of left eye (disorder)
        "1279847002",  # Detachment of retina of right eye with break of retina of right eye (disorder)
        "1279848007",  # Detachment of retina with break of retina (disorder)
        "1281813008",  # Partial detachment of retina with single break of retina (disorder)
        "1281815001",  # Partial detachment of retina with multiple breaks of retina (disorder)
        "193319001",  # New partial retinal detachment with single defect (disorder)
        "193322004",  # New partial retinal detachment with retinal dialysis (disorder)
        "19620000",  # Rhegmatogenous retinal detachment (disorder)
        "232008001",  # Rhegmatogenous retinal detachment - macula on (disorder)
        "232009009",  # Rhegmatogenous retinal detachment - macula off (disorder)
        "232010004",  # Traction retinal detachment involving macula (disorder)
        "232011000",  # Traction retinal detachment sparing macula (disorder)
        "232012007",  # Combined traction and rhegmatogenous retinal detachment (disorder)
        "232023006",  # Traction detachment of retina due to diabetes mellitus (disorder)
        "232068003",  # Retinal pigment epithelial detachment with vascularization (disorder)
        "232069006",  # Retinal pigment epithelial detachment with tear of retinal pigment epithelium (disorder)
        "247165009",  # Retinal detachment - subretinal fluid (disorder)
        "331811000119104",  # Traction detachment of right retina (disorder)
        "332521000119102",  # Retinal detachment of right eye due to retinal tear of right eye (disorder)
        "337431000119100",  # Traction detachment of left retina (disorder)
        "338131000119105",  # Retinal detachment of left eye due to retinal tear of left eye (disorder)
        "343221000119108",  # Traction detachment of bilateral retinas (disorder)
        "34711008",  # Traction detachment of retina (disorder)
        "38579007",  # Partial recent retinal detachment with retinal dialysis (disorder)
        "38599001",  # Serous retinal detachment (disorder)
        "712506009",  # Rhegmatogenous retinal detachment with single break (disorder)
        "712507000",  # Rhegmatogenous retinal detachment with multiple breaks (disorder)
        "712508005",  # Rhegmatogenous retinal detachment due to retinal dialysis (disorder)
        "7219007",  # Traction detachment with vitreoretinal organization (disorder)
        "773727009",  # Autosomal dominant rhegmatogenous retinal detachment (disorder)
        "82541000119100",  # Traction detachment of retina due to type 2 diabetes mellitus (disorder)
        "82571000119107",  # Traction detachment of retina due to type 1 diabetes mellitus (disorder)
    }

class RetinalVascularOcclusion(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of retinal vascular occlusion.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of retinal vein or artery occlusion.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Retinal Vascular Occlusion"
    OID = "2.16.840.1.113883.3.526.3.1479"
    DEFINITION_VERSION = "20210218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H3411",  # Central retinal artery occlusion, right eye
        "H3412",  # Central retinal artery occlusion, left eye
        "H3413",  # Central retinal artery occlusion, bilateral
        "H34231",  # Retinal artery branch occlusion, right eye
        "H34232",  # Retinal artery branch occlusion, left eye
        "H34233",  # Retinal artery branch occlusion, bilateral
        "H348110",  # Central retinal vein occlusion, right eye, with macular edema
        "H348111",  # Central retinal vein occlusion, right eye, with retinal neovascularization
        "H348112",  # Central retinal vein occlusion, right eye, stable
        "H348120",  # Central retinal vein occlusion, left eye, with macular edema
        "H348121",  # Central retinal vein occlusion, left eye, with retinal neovascularization
        "H348122",  # Central retinal vein occlusion, left eye, stable
        "H348130",  # Central retinal vein occlusion, bilateral, with macular edema
        "H348131",  # Central retinal vein occlusion, bilateral, with retinal neovascularization
        "H348132",  # Central retinal vein occlusion, bilateral, stable
        "H348310",  # Tributary (branch) retinal vein occlusion, right eye, with macular edema
        "H348311",  # Tributary (branch) retinal vein occlusion, right eye, with retinal neovascularization
        "H348312",  # Tributary (branch) retinal vein occlusion, right eye, stable
        "H348320",  # Tributary (branch) retinal vein occlusion, left eye, with macular edema
        "H348321",  # Tributary (branch) retinal vein occlusion, left eye, with retinal neovascularization
        "H348322",  # Tributary (branch) retinal vein occlusion, left eye, stable
        "H348330",  # Tributary (branch) retinal vein occlusion, bilateral, with macular edema
        "H348331",  # Tributary (branch) retinal vein occlusion, bilateral, with retinal neovascularization
        "H348332",  # Tributary (branch) retinal vein occlusion, bilateral, stable
    }

    SNOMEDCT = {
        "15997911000119103",  # Vascular occlusion of retina of left eye (disorder)
        "15997951000119102",  # Vascular occlusion of retina of right eye (disorder)
        "15997991000119107",  # Vascular occlusion of retina of bilateral eyes (disorder)
        "16279401000119108",  # Occlusion of right cilioretinal artery (disorder)
        "16279441000119105",  # Occlusion of left cilioretinal artery (disorder)
        "16711111000119100",  # Retinal neovascularization due to occlusion of bilateral central retinal veins (disorder)
        "16711631000119103",  # Retinal neovascularization due to occlusion of branch of retinal vein of bilateral eyes (disorder)
        "232035005",  # Retinal artery occlusion (disorder)
        "232036006",  # Cilioretinal artery occlusion (disorder)
        "232037002",  # Branch macular artery occlusion (disorder)
        "232038007",  # Central retinal vein occlusion with neovascularization (disorder)
        "232039004",  # Central retinal vein occlusion with macular edema (disorder)
        "232040002",  # Central retinal vein occlusion - juvenile (disorder)
        "232041003",  # Neovascularization of eye due to juvenile central retinal vein occlusion (disorder)
        "232043000",  # Hemispheric retinal vein occlusion (disorder)
        "232044006",  # Neovascularization of eye due to hemispheric retinal vein occlusion (disorder)
        "232046008",  # Branch retinal vein occlusion with neovascularization (disorder)
        "232048009",  # Branch retinal vein occlusion with macular edema (disorder)
        "24596005",  # Venous retinal branch occlusion (disorder)
        "247121001",  # Macular branch retinal vein occlusion (disorder)
        "312997008",  # Central retinal vein occlusion - ischemic (disorder)
        "312998003",  # Central retinal vein occlusion - non-ischemic (disorder)
        "314000002",  # Branch retinal vein occlusion with no neovascularization (disorder)
        "331801000119102",  # Transient occlusion of right retinal artery (disorder)
        "333281000119108",  # Partial occlusion of right retinal artery (disorder)
        "336191000119105",  # Occlusion of right central retinal artery (disorder)
        "337421000119103",  # Transient occlusion of left retinal artery (disorder)
        "338891000119100",  # Partial occlusion of left retinal artery (disorder)
        "341801000119101",  # Occlusion of left central retinal artery (disorder)
        "344661000119103",  # Partial occlusion of retinal artery of bilateral eyes (disorder)
        "347011000119102",  # Occlusion of bilateral central retinal arteries (disorder)
        "38742007",  # Central retinal artery occlusion (disorder)
        "50821009",  # Arterial retinal branch occlusion (disorder)
        "65593009",  # Partial occlusion of retinal vein (disorder)
        "677771000119102",  # Occlusion of bilateral branch retinal arteries (disorder)
        "677781000119104",  # Occlusion of left branch retinal artery (disorder)
        "677791000119101",  # Occlusion of right branch retinal artery (disorder)
        "677821000119109",  # Partial occlusion of left retinal vein (disorder)
        "677831000119107",  # Partial occlusion of right retinal vein (disorder)
        "680941000119100",  # Partial occlusion of bilateral retinal veins (disorder)
        "68478007",  # Central retinal vein occlusion (disorder)
        "733325006",  # Combined occlusion by thrombus of retinal artery and retinal vein (disorder)
        "73757007",  # Retinal vascular occlusion (disorder)
        "776009",  # Partial occlusion of retinal artery (disorder)
        "786049007",  # Occlusion of branch of retinal vein of left eye (disorder)
        "786050007",  # Occlusion of branch of retinal vein of right eye (disorder)
        "786054003",  # Occlusion of central retinal vein of right eye (disorder)
        "786055002",  # Occlusion of central retinal vein of left eye (disorder)
        "87224000",  # Transient arterial retinal occlusion (disorder)
    }

class RetrolentalFibroplasias(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis for retrolental fibroplasias.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of retrolental fibroplasias, including retinopathy of prematurity.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Retrolental Fibroplasias"
    OID = "2.16.840.1.113883.3.526.3.1438"
    DEFINITION_VERSION = "20210218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H35171",  # Retrolental fibroplasia, right eye
        "H35172",  # Retrolental fibroplasia, left eye
        "H35173",  # Retrolental fibroplasia, bilateral
    }

    SNOMEDCT = {
        "332351000119103",  # Retinopathy of prematurity stage 5 of right eye (disorder)
        "332361000119101",  # Retinopathy of prematurity of right eye stage 4 - subtotal retinal detachment (disorder)
        "332371000119107",  # Retinopathy of prematurity of right eye stage 3 - ridge with extraretinal fibrovascular proliferation (disorder)
        "332381000119105",  # Retinopathy of prematurity of right eye stage 2 - intraretinal ridge (disorder)
        "332391000119108",  # Retinopathy of prematurity of right eye stage 1 - demarcation line (disorder)
        "332411000119108",  # Retinopathy of prematurity of right eye (disorder)
        "337961000119104",  # Retinopathy of prematurity stage 5 of left eye (disorder)
        "337971000119105",  # Retinopathy of prematurity of left eye stage 4 - subtotal retinal detachment (disorder)
        "337981000119108",  # Retinopathy of prematurity of left eye stage 3 - ridge with extraretinal fibrovascular proliferation (disorder)
        "337991000119106",  # Retinopathy of prematurity of left eye stage 2 - intraretinal ridge (disorder)
        "338001000119102",  # Retinopathy of prematurity of left eye stage 1 - demarcation line (disorder)
        "338021000119106",  # Retinopathy of prematurity of left eye (disorder)
        "343731000119100",  # Retinopathy of prematurity stage 5 of bilateral eyes (disorder)
        "343741000119109",  # Retinopathy of prematurity of bilateral eyes stage 4 - subtotal retinal detachment (disorder)
        "343751000119106",  # Retinopathy of prematurity of bilateral eyes stage 3 - ridge with extraretinal fibrovascular proliferation (disorder)
        "343761000119108",  # Retinopathy of prematurity of bilateral eyes stage 2 - intraretinal ridge (disorder)
        "343771000119102",  # Retinopathy of prematurity of bilateral eyes stage 1 - demarcation line (disorder)
        "343791000119101",  # Retinopathy of prematurity of bilateral eyes (disorder)
        "408847006",  # Retinopathy of prematurity stage 1 - demarcation line (disorder)
        "408848001",  # Retinopathy of prematurity stage 2 - intraretinal ridge (disorder)
        "408849009",  # Retinopathy of prematurity stage 3 - ridge with extraretinal fibrovascular proliferation (disorder)
        "408850009",  # Retinopathy of prematurity stage 4 - subtotal retinal detachment (disorder)
        "408851008",  # Retinopathy of prematurity stage 5 - total retinal detachment (disorder)
        "415297005",  # Retinopathy of prematurity (disorder)
    }

class Scleritis(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of diagnoses for scleritis.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of specific types of scleritis.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Scleritis"
    OID = "2.16.840.1.113762.1.4.1226.1"
    DEFINITION_VERSION = "20210308"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H15021",  # Brawny scleritis, right eye
        "H15022",  # Brawny scleritis, left eye
        "H15023",  # Brawny scleritis, bilateral
        "H15031",  # Posterior scleritis, right eye
        "H15032",  # Posterior scleritis, left eye
        "H15033",  # Posterior scleritis, bilateral
        "H15041",  # Scleritis with corneal involvement, right eye
        "H15042",  # Scleritis with corneal involvement, left eye
        "H15043",  # Scleritis with corneal involvement, bilateral
        "H15051",  # Scleromalacia perforans, right eye
        "H15052",  # Scleromalacia perforans, left eye
        "H15053",  # Scleromalacia perforans, bilateral
        "H15091",  # Other scleritis, right eye
        "H15092",  # Other scleritis, left eye
        "H15093",  # Other scleritis, bilateral
    }

    SNOMEDCT = {
        "1087131000119102",  # Scleritis due to herpes zoster (disorder)
        "1335863008",  # Idiopathic scleritis (disorder)
        "1335866000",  # Immune-mediated scleritis (disorder)
        "16000751000119105",  # Scleritis of bilateral eyes due to herpes zoster (disorder)
        "16000791000119100",  # Scleritis of left eye due to herpes zoster (disorder)
        "16000831000119106",  # Scleritis of right eye due to herpes zoster (disorder)
        "231876002",  # Non-pyogenic granulomatous scleritis (disorder)
        "26664005",  # Scleromalacia perforans (disorder)
        "267660007",  # Posterior scleritis (disorder)
        "27886001",  # Sclerosing keratitis (disorder)
        "332221000119100",  # Sclerosing keratitis of right eye (disorder)
        "332231000119102",  # Scleromalacia perforans of right eye (disorder)
        "332241000119106",  # Sclerokeratitis of right eye (disorder)
        "332261000119105",  # Scleritis of right eye (disorder)
        "332281000119101",  # Abscess of sclera of right eye (disorder)
        "332981000119106",  # Posterior scleritis of right eye (disorder)
        "336391000119107",  # Anterior scleritis of right eye (disorder)
        "337831000119103",  # Sclerosing keratitis of left eye (disorder)
        "337841000119107",  # Scleromalacia perforans of left eye (disorder)
        "337851000119109",  # Sclerokeratitis of left eye (disorder)
        "337871000119100",  # Scleritis of left eye (disorder)
        "337891000119104",  # Abscess of sclera of left eye (disorder)
        "338591000119103",  # Posterior scleritis of left eye (disorder)
        "342001000119103",  # Anterior scleritis of left eye (disorder)
        "343601000119101",  # Sclerosing keratitis of bilateral eyes (disorder)
        "343611000119103",  # Scleromalacia perforans of bilateral eyes (disorder)
        "343621000119105",  # Sclerokeratitis of bilateral eyes (disorder)
        "343641000119104",  # Scleritis of bilateral eyes (disorder)
        "344361000119105",  # Posterior scleritis of bilateral eyes (disorder)
        "347181000119109",  # Anterior scleritis of bilateral eyes (disorder)
        "410578007",  # Infective scleritis (disorder)
        "416879000",  # Scleritis due to granulomatosis with polyangiitis (disorder)
        "417290008",  # Sclerokeratitis (disorder)
        "42574005",  # Scleritis with corneal involvement (disorder)
        "50675000",  # Scleral abscess (disorder)
        "59165007",  # Rheumatoid arthritis with scleritis (disorder)
        "63454000",  # Anterior scleritis (disorder)
        "78370002",  # Scleritis (disorder)
        "91612009",  # Brawny scleritis (disorder)
        "95195003",  # Nodular scleritis (disorder)
        "95680001",  # Sclerouveitis (disorder)
        "95795006",  # Superficial scleritis (disorder)
        "95796007",  # Deep scleritis (disorder)
        "95797003",  # Necrotizing scleritis (disorder)
    }

class SeparationOfRetinalLayers(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of diagnoses for separation of the layers of the retina.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of serous chorioretinopathy and types of retinal detachment.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Separation of Retinal Layers"
    OID = "2.16.840.1.113883.3.526.3.1482"
    DEFINITION_VERSION = "20210218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H35711",  # Central serous chorioretinopathy, right eye
        "H35712",  # Central serous chorioretinopathy, left eye
        "H35713",  # Central serous chorioretinopathy, bilateral
        "H35721",  # Serous detachment of retinal pigment epithelium, right eye
        "H35722",  # Serous detachment of retinal pigment epithelium, left eye
        "H35723",  # Serous detachment of retinal pigment epithelium, bilateral
        "H35731",  # Hemorrhagic detachment of retinal pigment epithelium, right eye
        "H35732",  # Hemorrhagic detachment of retinal pigment epithelium, left eye
        "H35733",  # Hemorrhagic detachment of retinal pigment epithelium, bilateral
    }

    SNOMEDCT = {
        "1231625008",  # Tractional retinal detachment due to proliferative diabetic retinopathy (disorder)
        "1231728000",  # Retinal detachment due to degenerative progressive high myopia (disorder)
        "1255639008",  # Break of retina with schisis of retina (disorder)
        "1279846006",  # Detachment of retina of left eye with break of retina of left eye (disorder)
        "1279847002",  # Detachment of retina of right eye with break of retina of right eye (disorder)
        "1279848007",  # Detachment of retina with break of retina (disorder)
        "1281813008",  # Partial detachment of retina with single break of retina (disorder)
        "1281815001",  # Partial detachment of retina with multiple breaks of retina (disorder)
        "1306765002",  # Detachment of retina of right eye (disorder)
        "1306766001",  # Detachment of retina of left eye (disorder)
        "1306767005",  # Detachment of retina of bilateral eyes (disorder)
        "15733601000119104",  # Chronic partial detachment of retina of left eye (disorder)
        "15733681000119107",  # Chronic partial detachment of retina of right eye (disorder)
        "15734081000119102",  # Old total detachment of retinas of bilateral eyes (disorder)
        "15734121000119100",  # Chronic total detachment of retina of left eye (disorder)
        "15734161000119105",  # Chronic total detachment of retina of right eye (disorder)
        "15734321000119103",  # Acute partial detachment of retina of left eye (disorder)
        "15734361000119108",  # Old retinal detachment of left eye (disorder)
        "15734401000119104",  # Acute partial detachment of retina of right eye (disorder)
        "15734521000119107",  # Old retinal detachment of right eye (disorder)
        "15734561000119102",  # Old retinal detachment of bilateral eyes (disorder)
        "15734641000119100",  # Acute detachment of retina of left eye (disorder)
        "15734681000119105",  # Acute detachment of retina of right eye (disorder)
        "193319001",  # New partial retinal detachment with single defect (disorder)
        "193322004",  # New partial retinal detachment with retinal dialysis (disorder)
        "193323009",  # Recent total retinal detachment (disorder)
        "193324003",  # Recent subtotal retinal detachment (disorder)
        "193325002",  # Old partial retinal detachment (disorder)
        "193326001",  # Old total retinal detachment (disorder)
        "193327005",  # Old subtotal retinal detachment (disorder)
        "193329008",  # Retinoschisis and retinal cysts (disorder)
        "19620000",  # Rhegmatogenous retinal detachment (disorder)
        "232004004",  # Giant retinal tear (disorder)
        "232008001",  # Rhegmatogenous retinal detachment - macula on (disorder)
        "232009009",  # Rhegmatogenous retinal detachment - macula off (disorder)
        "232010004",  # Traction retinal detachment involving macula (disorder)
        "232011000",  # Traction retinal detachment sparing macula (disorder)
        "232012007",  # Combined traction and rhegmatogenous retinal detachment (disorder)
        "232015009",  # Solid retinal detachment (disorder)
        "232023006",  # Traction detachment of retina due to diabetes mellitus (disorder)
        "232034009",  # Central serous retinopathy with pit of optic disc (disorder)
        "247165009",  # Retinal detachment - subretinal fluid (disorder)
        "31181000119100",  # Old retinal detachment (disorder)
        "31191000119102",  # Recent retinal detachment (disorder)
        "312923002",  # Chronic central serous chorioretinopathy (disorder)
        "312924008",  # Inactive central serous chorioretinopathy (disorder)
        "312947009",  # Acute central serous chorioretinopathy (disorder)
        "312956001",  # Central serous chorioretinopathy (disorder)
        "312957005",  # Variant central serous chorioretinopathy (disorder)
        "314006008",  # Acute central serous retinopathy with subretinal fluid (disorder)
        "314007004",  # Chronic central serous retinopathy with diffuse retinal pigment epithelial detachment (disorder)
        "314008009",  # Inactive central serous retinopathy with focal retinal pigment epithelial detachment (disorder)
        "314009001",  # Central serous retinopathy with small retinal pigment epithelial detachment (disorder)
        "314494006",  # Pseudophakic retinal detachment (disorder)
        "331111000119108",  # Total retinal detachment (disorder)
        "331811000119104",  # Traction detachment of right retina (disorder)
        "331831000119109",  # Total detachment of right retina (disorder)
        "332341000119100",  # Schisis of right retina (disorder)
        "334891000119105",  # Hemorrhagic detachment of right retinal pigment epithelium (disorder)
        "335021000119100",  # Giant retinal tear of right eye (disorder)
        "337431000119100",  # Traction detachment of left retina (disorder)
        "337451000119106",  # Total detachment of left retina (disorder)
        "337951000119101",  # Schisis of left retina (disorder)
        "340501000119106",  # Hemorrhagic detachment of left retinal pigment epithelium (disorder)
        "340631000119100",  # Giant retinal tear of left eye (disorder)
        "345811000119101",  # Hemorrhagic detachment of bilateral retinal pigment epithelium (disorder)
        "345941000119106",  # Giant retinal tear of bilateral eyes (disorder)
        "34711008",  # Traction detachment of retina (disorder)
        "361130009",  # Peripheral retinoschisis (disorder)
        "38579007",  # Partial recent retinal detachment with retinal dialysis (disorder)
        "38599001",  # Serous retinal detachment (disorder)
        "42059000",  # Retinal detachment (disorder)
        "44268007",  # Retinoschisis (disorder)
        "51987004",  # Retinal detachment without retinal defect (disorder)
        "712507000",  # Rhegmatogenous retinal detachment with multiple breaks (disorder)
        "713345005",  # Partial retinal detachment (disorder)
        "7219007",  # Traction detachment with vitreoretinal organization (disorder)
        "723023003",  # Adult retinoschisis (disorder)
        "83405000",  # Flat retinoschisis (disorder)
        "86923008",  # Juvenile retinoschisis (disorder)
        "95493003",  # Congenital retinoschisis (disorder)
    }

class TraumaticCataract(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of diagnoses for traumatic cataract.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of traumatic cataract, including concussion cataract.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Traumatic Cataract"
    OID = "2.16.840.1.113883.3.526.3.1443"
    DEFINITION_VERSION = "20210219"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H26101",  # Unspecified traumatic cataract, right eye
        "H26102",  # Unspecified traumatic cataract, left eye
        "H26103",  # Unspecified traumatic cataract, bilateral
        "H26111",  # Localized traumatic opacities, right eye
        "H26112",  # Localized traumatic opacities, left eye
        "H26113",  # Localized traumatic opacities, bilateral
        "H26121",  # Partially resolved traumatic cataract, right eye
        "H26122",  # Partially resolved traumatic cataract, left eye
        "H26123",  # Partially resolved traumatic cataract, bilateral
        "H26131",  # Total traumatic cataract, right eye
        "H26132",  # Total traumatic cataract, left eye
        "H26133",  # Total traumatic cataract, bilateral
    }

    SNOMEDCT = {
        "193598007",  # Partial resolved traumatic cataract (disorder)
        "231967008",  # Concussion cataract (disorder)
        "231968003",  # Cataract following rupture of capsule (disorder)
        "331781000119101",  # Cataract of right eye due to and following trauma (disorder)
        "337401000119107",  # Cataract of left eye due to and following trauma (disorder)
        "343191000119102",  # Cataract of bilateral eyes due to and following trauma (disorder)
        "34361001",  # Traumatic cataract (disorder)
        "8656007",  # Total traumatic cataract (disorder)
        "866114005",  # Localized cataract opacities due to and following traumatic injury (disorder)
    }

class Uveitis(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of uveitis.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of uveitis.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Uveitis"
    OID = "2.16.840.1.113883.3.526.3.1444"
    DEFINITION_VERSION = "20210219"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H44111",  # Panuveitis, right eye
        "H44112",  # Panuveitis, left eye
        "H44113",  # Panuveitis, bilateral
        "H44131",  # Sympathetic uveitis, right eye
        "H44132",  # Sympathetic uveitis, left eye
        "H44133",  # Sympathetic uveitis, bilateral
    }

    SNOMEDCT = {
        "1144978007",  # Anterior uveitis due to sarcoidosis (disorder)
        "1156843000",  # Uveitis due to persistent onset antinuclear antibody negative juvenile idiopathic oligoarthritis (disorder)
        "1156844006",  # Uveitis due to persistent onset antinuclear antibody positive juvenile idiopathic oligoarthritis (disorder)
        "1156845007",  # Uveitis due to extended onset antinuclear antibody negative juvenile idiopathic oligoarthritis (disorder)
        "1156846008",  # Uveitis due to extended onset antinuclear antibody positive juvenile idiopathic oligoarthritis (disorder)
        "1163460006",  # Intermediate uveitis caused by Borrelia burgdorferi (disorder)
        "1176981005",  # Posterior uveitis caused by Mycobacterium tuberculosis complex (disorder)
        "1176982003",  # Posterior uveitis caused by virus (disorder)
        "1177018001",  # Anterior uveitis caused by Mycobacterium tuberculosis complex (disorder)
        "1177029002",  # Intermediate uveitis caused by Treponema pallidum (disorder)
        "1177031006",  # Intermediate uveitis caused by Mycobacterium tuberculosis complex (disorder)
        "1197180007",  # Recurrent anterior uveitis (disorder)
        "1208838005",  # Idiopathic acute inflammation of anterior uveal tract (disorder)
        "1216988003",  # Idiopathic subacute inflammation of anterior uveal tract (disorder)
        "1217647006",  # Idiopathic chronic anterior uveitis (disorder)
        "1231232007",  # Intermediate uveitis associated with multiple sclerosis (disorder)
        "1231259004",  # Uveitis due to juvenile idiopathic arthritis (disorder)
        "1231640005",  # Recurrent acute anterior uveitis (disorder)
        "1231665007",  # Intermediate uveitis due to sarcoidosis (disorder)
        "1231676006",  # Posterior uveitis due to Vogt Koyanagi Harada disease (disorder)
        "1231680001",  # Panuveitis due to sarcoidosis (disorder)
        "1269223003",  # Paraneoplastic uveitis (disorder)
        "128473001",  # Uveitis (disorder)
        "15630771000119104",  # Uveitis of right eye (disorder)
        "15630811000119104",  # Uveitis of left eye (disorder)
        "15630851000119103",  # Uveitis of bilateral eyes (disorder)
        "15678521000119108",  # Chronic inflammation of uveal tract of left eye (disorder)
        "15678561000119103",  # Chronic inflammation of uveal tract of bilateral eyes (disorder)
        "15678601000119103",  # Chronic inflammation of uveal tract of right eye (disorder)
        "16553002",  # Choroiditis (disorder)
        "186266008",  # Tuberculous chronic iridocyclitis (disorder)
        "186854007",  # Uveitis due to secondary syphilis (disorder)
        "231947004",  # Idiopathic inflammation of anterior uveal tract (disorder)
        "267619000",  # Non-infectious anterior uveitis (disorder)
        "314029000",  # Tuberculous uveitis (disorder)
        "314030005",  # Post-traumatic uveitis (disorder)
        "314429009",  # Intermediate uveitis (disorder)
        "314493000",  # Postoperative uveitis (disorder)
        "333391000119101",  # Panuveitis of right eye (disorder)
        "333401000119104",  # Panophthalmitis of right eye (disorder)
        "33382000",  # Panophthalmitis (disorder)
        "335971000119100",  # Chronic anterior uveitis of right eye (disorder)
        "336041000119101",  # Choroiditis of right eye (disorder)
        "336541000119107",  # Acute anterior uveitis of right eye (disorder)
        "339001000119107",  # Panuveitis of left eye (disorder)
        "339011000119105",  # Panophthalmitis of left eye (disorder)
        "341581000119106",  # Chronic anterior uveitis of left eye (disorder)
        "341651000119103",  # Choroiditis of left eye (disorder)
        "342151000119100",  # Acute anterior uveitis of left eye (disorder)
        "344751000119109",  # Panuveitis of bilateral eyes (disorder)
        "346801000119106",  # Chronic anterior uveitis of bilateral eyes (disorder)
        "346871000119101",  # Choroiditis of bilateral eyes (disorder)
        "347311000119100",  # Acute anterior uveitis of bilateral eyes (disorder)
        "370506009",  # Keratouveitis (disorder)
        "398155003",  # Chronic anterior uveitis (disorder)
        "410473001",  # Lyme uveitis (disorder)
        "410476009",  # Uveitis due to leptospirosis (disorder)
        "410477000",  # Infective uveitis (disorder)
        "410481000",  # Panuveitis in Behcet's syndrome (disorder)
        "410494003",  # Lens-induced uveitis (disorder)
        "410501000",  # Drug-induced uveitis (disorder)
        "410692006",  # Anterior uveitis (disorder)
        "413716003",  # Brucellosis uveitis (disorder)
        "413851004",  # Cidofovir-induced anterior uveitis (disorder)
        "414600000",  # Lepromatous anterior uveitis (disorder)
        "414689001",  # Metipranolol-induced anterior uveitis (disorder)
        "414784001",  # Multifocal choroiditis and panuveitis syndrome (disorder)
        "414996002",  # Pamidronic acid-induced uveitis (disorder)
        "415344002",  # Rifabutin-induced anterior uveitis (disorder)
        "415359008",  # Sarcoid uveitis (disorder)
        "415657000",  # Subretinal fibrosis and uveitis syndrome (disorder)
        "416666007",  # Anterior uveitis due to juvenile idiopathic arthritis (disorder)
        "417321001",  # Human T-cell Lymphoma Virus Type -1 associated uveitis (disorder)
        "418839003",  # Tubulointerstitial nephritis with uveitis syndrome (disorder)
        "428676003",  # Uveitis caused by ranibizumab (disorder)
        "43363007",  # Posterior uveitis (disorder)
        "434391000124109",  # Secondary infective uveitis (disorder)
        "444248002",  # Chronic inflammation of uveal tract (disorder)
        "4927003",  # Acute anterior uveitis (disorder)
        "64015001",  # Subacute anterior uveitis (disorder)
        "722607009",  # Optic disc swelling co-occurrent with uveitis (disorder)
        "733317009",  # Primary anterior uveitis (disorder)
        "733337008",  # Noninfectious panuveitis (disorder)
        "75315001",  # Sympathetic uveitis (disorder)
        "75614007",  # Panuveitis (disorder)
        "766933000",  # Idiopathic panuveitis (disorder)
        "773991005",  # Idiopathic posterior uveitis (disorder)
        "786077003",  # Ankylosing spondylitis co-occurrent with anterior uveitis (disorder)
        "788963006",  # Intermediate uveitis due to infectious disease (disorder)
        "789008006",  # Posterior uveitis due to infectious disease (disorder)
        "789116000",  # Panuveitis due to infectious disease (disorder)
        "86219005",  # Uveitis-rheumatoid arthritis syndrome (disorder)
        "870199008",  # Non-infectious uveitis of posterior segment of eye (disorder)
        "870273007",  # Posterior uveitis due to late syphilis (disorder)
        "870329005",  # Panuveitis caused by virus (disorder)
        "870331001",  # Panuveitis caused by Treponema pallidum (disorder)
        "870332008",  # Panuveitis caused by Herpesvirus (disorder)
        "870333003",  # Panuveitis caused by Mycobacterium tuberculosis (disorder)
        "870335005",  # Panuveitis caused by Borrelia burgdorferi (disorder)
        "95680001",  # Sclerouveitis (disorder)
    }

class VascularDisordersOfIrisAndCiliaryBody(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of vascular disorders of iris and ciliary body.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis for a vascular disorder of the iris and/or ciliary body, including rubeosis iridis.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Vascular Disorders of Iris and Ciliary Body"
    OID = "2.16.840.1.113883.3.526.3.1445"
    DEFINITION_VERSION = "20210214"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H211X1",  # Other vascular disorders of iris and ciliary body, right eye
        "H211X2",  # Other vascular disorders of iris and ciliary body, left eye
        "H211X3",  # Other vascular disorders of iris and ciliary body, bilateral
    }

    SNOMEDCT = {
        "193501009",  # Iris and ciliary body vascular disorder (disorder)
        "51995000",  # Rubeosis iridis (disorder)
        "678191000119101",  # Rubeosis iridis of bilateral eyes (disorder)
        "678201000119103",  # Rubeosis iridis of left eye (disorder)
        "678211000119100",  # Rubeosis iridis of right eye (disorder)
        "82551000119103",  # Rubeosis iridis due to type 2 diabetes mellitus (disorder)
        "82581000119105",  # Rubeosis iridis due to type 1 diabetes mellitus (disorder)
    }

class VisualFieldDefects(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of diagnoses for visual field defects.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of visual field defects, including scotoma and decreased field of vision.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Visual Field Defects"
    OID = "2.16.840.1.113883.3.526.3.1446"
    DEFINITION_VERSION = "20210220"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H53411",  # Scotoma involving central area, right eye
        "H53412",  # Scotoma involving central area, left eye
        "H53413",  # Scotoma involving central area, bilateral
        "H53421",  # Scotoma of blind spot area, right eye
        "H53422",  # Scotoma of blind spot area, left eye
        "H53423",  # Scotoma of blind spot area, bilateral
        "H53431",  # Sector or arcuate defects, right eye
        "H53432",  # Sector or arcuate defects, left eye
        "H53433",  # Sector or arcuate defects, bilateral
        "H53451",  # Other localized visual field defect, right eye
        "H53452",  # Other localized visual field defect, left eye
        "H53453",  # Other localized visual field defect, bilateral
        "H53461",  # Homonymous bilateral field defects, right side
        "H53462",  # Homonymous bilateral field defects, left side
        "H5347",  # Heteronymous bilateral field defects
        "H53481",  # Generalized contraction of visual field, right eye
        "H53482",  # Generalized contraction of visual field, left eye
        "H53483",  # Generalized contraction of visual field, bilateral
    }

    SNOMEDCT = {
        "1151008",  # Visual field constriction (finding)
        "12184005",  # Visual field defect (finding)
        "129625005",  # Ring scotoma (finding)
        "129626006",  # Quadrantic scotoma (finding)
        "15462009",  # Arcuate scotoma (finding)
        "15747321000119109",  # Scintillating scotoma of left eye (finding)
        "15747361000119104",  # Scintillating scotoma of right eye (finding)
        "15747401000119108",  # Scintillating scotoma of bilateral eyes (finding)
        "16058911000119101",  # Peripheral visual field defect of bilateral eyes (finding)
        "16058951000119100",  # Peripheral visual field defect of left eye (finding)
        "16058991000119105",  # Peripheral visual field defect of right eye (finding)
        "16059111000119107",  # Scotoma of right eye (finding)
        "16059151000119108",  # Scotoma of left eye (finding)
        "16059191000119103",  # Scotoma of bilateral eyes (finding)
        "193670002",  # Blind spot scotoma (finding)
        "193671003",  # Paracecal scotoma (finding)
        "193672005",  # Angioscotoma (finding)
        "19900004",  # Decreased peripheral vision (finding)
        "21273008",  # Relative scotoma (finding)
        "2228002",  # Scintillating scotoma (finding)
        "23388006",  # Visual field scotoma (finding)
        "24370009",  # Seidel scotoma (finding)
        "246656009",  # Loss of part of visual field (finding)
        "246658005",  # Seeing blob in vision (finding)
        "246659002",  # Peripheral scotoma (finding)
        "246660007",  # Peripheral ring scotoma (finding)
        "246661006",  # Local peripheral scotoma (finding)
        "246662004",  # Altitudinal scotoma (finding)
        "267628004",  # Generalized visual field constriction (finding)
        "33014001",  # Centrocecal scotoma (finding)
        "33048000",  # Peripheral visual field defect (finding)
        "332131000119107",  # Sector visual field defect of right eye (finding)
        "333371000119102",  # Paracentral scotoma of right eye (finding)
        "333381000119104",  # Paracecal scotoma of right eye (finding)
        "336151000119100",  # Centrocecal scotoma of right eye (finding)
        "336171000119109",  # Central scotoma of right eye (finding)
        "336371000119106",  # Arcuate visual field defect of right eye (finding)
        "337741000119101",  # Sector visual field defect of left eye (finding)
        "338981000119109",  # Paracentral scotoma of left eye (finding)
        "338991000119107",  # Paracecal scotoma of left eye (finding)
        "33970004",  # Enlarged blind spot (finding)
        "341761000119109",  # Centrocecal scotoma of left eye (finding)
        "341781000119100",  # Central scotoma of left eye (finding)
        "341981000119103",  # Arcuate visual field defect of left eye (finding)
        "343511000119104",  # Sector visual field defect of bilateral eyes (finding)
        "344731000119103",  # Paracentral scotoma of bilateral eyes (finding)
        "344741000119107",  # Paracecal scotoma of bilateral eyes (finding)
        "345121001",  # Heteronymous bilateral visual field defects (finding)
        "346971000119106",  # Centrocecal scotoma of bilateral eyes (finding)
        "346991000119107",  # Central scotoma of bilateral eyes (finding)
        "347161000119100",  # Arcuate visual field defect of bilateral eyes (finding)
        "370961005",  # Glaucomatous visual field defect (finding)
        "38950008",  # Central scotoma (finding)
        "404650002",  # Pericentral scotoma (finding)
        "416626008",  # Functional visual field loss (finding)
        "420990001",  # Tunnel visual field constriction (finding)
        "421561005",  # Concentric visual field constriction (finding)
        "445923001",  # Sector visual field defect (finding)
        "446020008",  # Arcuate visual field defect (finding)
        "4806007",  # Bjerrum scotoma (finding)
        "49073008",  # Absolute scotoma (finding)
        "59695000",  # Bilateral visual field constriction (finding)
        "64418005",  # Paracentral scotoma (finding)
        "65133002",  # Physiologic scotoma (finding)
        "678701000119106",  # Tunnel vision of bilateral eyes (finding)
        "678711000119109",  # Tunnel vision of left eye (finding)
        "678721000119102",  # Tunnel vision of right eye (finding)
        "733338003",  # Isolated peripheral scotoma (finding)
    }

class AllergyToAceInhibitorOrArb(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of diagnosis of allergy to an angiotensin-converting enzyme (ACE) inhibitor and angiotensin-receptor blocker (ARB).

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of allergy to an angiotensin-converting enzyme (ACE) inhibitor and angiotensin-receptor blocker (ARB).

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Allergy to ACE Inhibitor or ARB"
    OID = "2.16.840.1.113883.3.526.3.1211"
    DEFINITION_VERSION = "20170908"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    SNOMEDCT = {
        "18321000122100",  # Non-allergic hypersensitivity to benazepril (finding)
        "18361000122109",  # Non-allergic hypersensitivity to moexipril (finding)
        "609534005",  # Non-allergic hypersensitivity to losartan (finding)
        "609537003",  # Non-allergic hypersensitivity to angiotensin-converting enzyme inhibitor (finding)
        "609538008",  # Non-allergic hypersensitivity to captopril (finding)
        "609539000",  # Non-allergic hypersensitivity to cilazapril (finding)
        "609540003",  # Non-allergic hypersensitivity to enalapril (finding)
        "609541004",  # Non-allergic hypersensitivity to fosinopril (finding)
        "609542006",  # Non-allergic hypersensitivity to lisinopril (finding)
        "609543001",  # Non-allergic hypersensitivity to perindopril (finding)
        "609544007",  # Non-allergic hypersensitivity to quinapril (finding)
        "609545008",  # Non-allergic hypersensitivity to ramipril (finding)
        "609546009",  # Non-allergic hypersensitivity to trandolapril (finding)
        "609549002",  # Non-allergic hypersensitivity to angiotensin II receptor antagonist (finding)
        "609550002",  # History of angiotensin converting enzyme inhibitor pseudoallergy (situation)
    }

class HeartFailure(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of heart failure.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of heart failure.

    **Exclusion Criteria:** Excludes concepts that represent chronic and acute right-sided heart failure, right heart failure, right ventricular failure, cardiac insufficiency due to prosthesis and following cardiac surgery, or pulmonale.
    """

    VALUE_SET_NAME = "Heart Failure"
    OID = "2.16.840.1.113883.3.526.3.376"
    DEFINITION_VERSION = "20210218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "I110",  # Hypertensive heart disease with heart failure
        "I130",  # Hypertensive heart and chronic kidney disease with heart failure and stage 1 through stage 4 chronic kidney disease, or unspecified chronic kidney disease
        "I132",  # Hypertensive heart and chronic kidney disease with heart failure and with stage 5 chronic kidney disease, or end stage renal disease
        "I501",  # Left ventricular failure, unspecified
        "I5020",  # Unspecified systolic (congestive) heart failure
        "I5021",  # Acute systolic (congestive) heart failure
        "I5022",  # Chronic systolic (congestive) heart failure
        "I5023",  # Acute on chronic systolic (congestive) heart failure
        "I5030",  # Unspecified diastolic (congestive) heart failure
        "I5031",  # Acute diastolic (congestive) heart failure
        "I5032",  # Chronic diastolic (congestive) heart failure
        "I5033",  # Acute on chronic diastolic (congestive) heart failure
        "I5040",  # Unspecified combined systolic (congestive) and diastolic (congestive) heart failure
        "I5041",  # Acute combined systolic (congestive) and diastolic (congestive) heart failure
        "I5042",  # Chronic combined systolic (congestive) and diastolic (congestive) heart failure
        "I5043",  # Acute on chronic combined systolic (congestive) and diastolic (congestive) heart failure
        "I50814",  # Right heart failure due to left heart failure
        "I5082",  # Biventricular heart failure
        "I5083",  # High output heart failure
        "I5084",  # End stage heart failure
        "I5089",  # Other heart failure
        "I509",  # Heart failure, unspecified
    }

    SNOMEDCT = {
        "10091002",  # High output heart failure (disorder)
        "101281000119107",  # Congestive heart failure due to cardiomyopathy (disorder)
        "10633002",  # Acute congestive heart failure (disorder)
        "111283005",  # Chronic left-sided heart failure (disorder)
        "1204200007",  # Left ventricular failure with normal ejection fraction due to valvular heart disease (disorder)
        "1204203009",  # Left ventricular failure with normal ejection fraction due to coronary arteriosclerosis (disorder)
        "1204204003",  # Left ventricular failure with normal ejection fraction due to myocarditis (disorder)
        "1204206001",  # Left ventricular failure with normal ejection fraction due to cardiomyopathy (disorder)
        "1204462004",  # Left ventricular failure with sepsis (disorder)
        "120851000119104",  # Systolic heart failure stage D (disorder)
        "120861000119102",  # Systolic heart failure stage C (disorder)
        "120871000119108",  # Systolic heart failure stage B (disorder)
        "120881000119106",  # Diastolic heart failure stage D (disorder)
        "120891000119109",  # Diastolic heart failure stage C (disorder)
        "120901000119108",  # Diastolic heart failure stage B (disorder)
        "153931000119109",  # Acute combined systolic and diastolic heart failure (disorder)
        "153941000119100",  # Chronic combined systolic and diastolic heart failure (disorder)
        "153951000119103",  # Acute on chronic combined systolic and diastolic heart failure (disorder)
        "15629541000119106",  # Congestive heart failure stage C due to ischemic cardiomyopathy (disorder)
        "15629591000119103",  # Congestive heart failure stage B due to ischemic cardiomyopathy (disorder)
        "15629641000119107",  # Systolic heart failure stage B due to ischemic cardiomyopathy (disorder)
        "15629741000119102",  # Systolic heart failure stage C due to ischemic cardiomyopathy (disorder)
        "15781000119107",  # Hypertensive heart AND chronic kidney disease with congestive heart failure (disorder)
        "15964701000119109",  # Acute cor pulmonale co-occurrent and due to saddle embolus of pulmonary artery (disorder)
        "194767001",  # Benign hypertensive heart disease with congestive cardiac failure (disorder)
        "194779001",  # Hypertensive heart and renal disease with (congestive) heart failure (disorder)
        "194781004",  # Hypertensive heart and renal disease with both (congestive) heart failure and renal failure (disorder)
        "195111005",  # Decompensated cardiac failure (disorder)
        "195112003",  # Compensated cardiac failure (disorder)
        "195114002",  # Acute left ventricular failure (disorder)
        "206586007",  # Congenital cardiac failure (disorder)
        "23341000119109",  # Congestive heart failure with right heart failure (disorder)
        "233924009",  # Heart failure as a complication of care (disorder)
        "25544003",  # Low output heart failure (disorder)
        "314206003",  # Refractory heart failure (disorder)
        "364006",  # Acute left-sided heart failure (disorder)
        "410431009",  # Cardiorespiratory failure (disorder)
        "417996009",  # Systolic heart failure (disorder)
        "418304008",  # Diastolic heart failure (disorder)
        "42343007",  # Congestive heart failure (disorder)
        "424404003",  # Decompensated chronic heart failure (disorder)
        "426263006",  # Congestive heart failure due to left ventricular systolic dysfunction (disorder)
        "426611007",  # Congestive heart failure due to valvular disease (disorder)
        "43736008",  # Rheumatic left ventricular failure (disorder)
        "44088000",  # Low cardiac output syndrome (disorder)
        "441481004",  # Chronic systolic heart failure (disorder)
        "441530006",  # Chronic diastolic heart failure (disorder)
        "44313006",  # Right heart failure secondary to left heart failure (disorder)
        "443253003",  # Acute on chronic systolic heart failure (disorder)
        "443254009",  # Acute systolic heart failure (disorder)
        "443343001",  # Acute diastolic heart failure (disorder)
        "443344007",  # Acute on chronic diastolic heart failure (disorder)
        "46113002",  # Hypertensive heart failure (disorder)
        "471880001",  # Heart failure due to end stage congenital heart disease (disorder)
        "48447003",  # Chronic heart failure (disorder)
        "5148006",  # Hypertensive heart disease with congestive heart failure (disorder)
        "5375005",  # Chronic left-sided congestive heart failure (disorder)
        "56675007",  # Acute heart failure (disorder)
        "67431000119105",  # Congestive heart failure stage D (disorder)
        "67441000119101",  # Congestive heart failure stage C (disorder)
        "698594003",  # Symptomatic congestive heart failure (disorder)
        "703272007",  # Heart failure with reduced ejection fraction (disorder)
        "703273002",  # Heart failure with reduced ejection fraction due to coronary artery disease (disorder)
        "703274008",  # Heart failure with reduced ejection fraction due to myocarditis (disorder)
        "703275009",  # Heart failure with reduced ejection fraction due to cardiomyopathy (disorder)
        "703276005",  # Heart failure with reduced ejection fraction due to heart valve disease (disorder)
        "717840005",  # Congestive heart failure stage B (disorder)
        "72481000119103",  # Congestive heart failure as early postoperative complication (disorder)
        "74960003",  # Acute left-sided congestive heart failure (disorder)
        "82523003",  # Congestive rheumatic heart failure (disorder)
        "83105008",  # Malignant hypertensive heart disease with congestive heart failure (disorder)
        "84114007",  # Heart failure (disorder)
        "85232009",  # Left heart failure (disorder)
        "871617000",  # Low output heart failure due to and following Fontan operation (disorder)
        "88805009",  # Chronic congestive heart failure (disorder)
        "90727007",  # Pleural effusion due to congestive heart failure (disorder)
        "92506005",  # Biventricular congestive heart failure (disorder)
    }

class HeartTransplantComplications(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts related to complications associated with heart transplants.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent diagnoses related to heart transplant complications.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Heart Transplant Complications"
    OID = "2.16.840.1.113762.1.4.1178.56"
    DEFINITION_VERSION = "20220218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "I25750",  # Atherosclerosis of native coronary artery of transplanted heart with unstable angina
        "I25751",  # Atherosclerosis of native coronary artery of transplanted heart with angina pectoris with documented spasm
        "I25758",  # Atherosclerosis of native coronary artery of transplanted heart with other forms of angina pectoris
        "I25759",  # Atherosclerosis of native coronary artery of transplanted heart with unspecified angina pectoris
        "I25760",  # Atherosclerosis of bypass graft of coronary artery of transplanted heart with unstable angina
        "I25761",  # Atherosclerosis of bypass graft of coronary artery of transplanted heart with angina pectoris with documented spasm
        "I25768",  # Atherosclerosis of bypass graft of coronary artery of transplanted heart with other forms of angina pectoris
        "I25769",  # Atherosclerosis of bypass graft of coronary artery of transplanted heart with unspecified angina pectoris
        "I25811",  # Atherosclerosis of native coronary artery of transplanted heart without angina pectoris
        "I25812",  # Atherosclerosis of bypass graft of coronary artery of transplanted heart without angina pectoris
        "T8620",  # Unspecified complication of heart transplant
        "T8621",  # Heart transplant rejection
        "T8622",  # Heart transplant failure
        "T8623",  # Heart transplant infection
        "T86298",  # Other complications of heart transplant
        "T8630",  # Unspecified complication of heart-lung transplant
        "T8631",  # Heart-lung transplant rejection
        "T8632",  # Heart-lung transplant failure
        "T8633",  # Heart-lung transplant infection
        "T8639",  # Other complications of heart-lung transplant
    }

    SNOMEDCT = {
        "12224461000119108",  # Infection of transplanted heart (disorder)
        "122511000119103",  # Lymphoproliferative disorder following heart transplantation (disorder)
        "15960101000119104",  # Angina due to coronary arteriosclerosis in artery of transplanted heart (disorder)
        "15960341000119104",  # Unstable angina due to arteriosclerosis of coronary artery bypass graft of transplanted heart (disorder)
        "15960701000119100",  # Angina due to arteriosclerosis of coronary artery bypass graft of transplanted heart (disorder)
        "15960861000119108",  # Preinfarction syndrome due to coronary arteriosclerosis in artery of transplanted heart (disorder)
        "213151004",  # Heart transplant failure and rejection (disorder)
        "213152006",  # Heart-lung transplant failure and rejection (disorder)
        "233844002",  # Accelerated coronary artery disease in transplanted heart (disorder)
        "233933006",  # Cardiac transplant rejection (disorder)
        "233934000",  # Cardiac transplant failure (disorder)
        "429257001",  # Disorder of transplanted heart (disorder)
        "431186002",  # Accelerated rejection of cardiac transplant (disorder)
        "431896008",  # Chronic rejection of cardiac transplant (disorder)
        "432773004",  # Hyperacute rejection of cardiac transplant (disorder)
        "432843002",  # Acute rejection of cardiac transplant (disorder)
        "444855007",  # Arteriosclerosis of coronary artery bypass graft of transplanted heart (disorder)
        "471851005",  # Disorder of myocardium associated with rejection of cardiac transplant (disorder)
        "792842004",  # Coronary arteriosclerosis in artery of transplanted heart (disorder)
    }

class IntoleranceToAceInhibitorOrArb(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of diagnoses of intolerance to angiotensin-converting enzyme (ACE) inhibitor or angiotensin II receptor blockers (ARB).

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of intolerance to ACE inhibitor or ARB.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Intolerance to ACE Inhibitor or ARB"
    OID = "2.16.840.1.113883.3.526.3.1212"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    SNOMEDCT = {
        "293500009",  # Angiotensin-converting-enzyme inhibitor adverse reaction (disorder)
        "293501008",  # Captopril adverse reaction (disorder)
        "293502001",  # Lisinopril adverse reaction (disorder)
        "293503006",  # Quinapril adverse reaction (disorder)
        "293504000",  # Ramipril adverse reaction (disorder)
        "293505004",  # Cilazapril adverse reaction (disorder)
        "293506003",  # Trandolapril adverse reaction (disorder)
        "293507007",  # Enalapril adverse reaction (disorder)
        "293508002",  # Fosinopril adverse reaction (disorder)
        "293509005",  # Perindopril adverse reaction (disorder)
        "293513003",  # Adverse reaction caused by losartan (disorder)
        "371627004",  # Angiotensin converting enzyme inhibitor-aggravated angioedema (disorder)
        "407564000",  # Angiotensin converting enzyme inhibitor not tolerated (situation)
        "407565004",  # Angiotensin II receptor antagonist not tolerated (situation)
        "407590002",  # Angiotensin II receptor antagonist adverse reaction (disorder)
        "422593004",  # Acute renal failure caused by angiotensin-converting-enzyme inhibitor (disorder)
        "720672008",  # Adverse reaction caused by telmisartan (disorder)
        "720673003",  # Adverse reaction caused by valsartan (disorder)
        "720674009",  # Adverse reaction caused by olmesartan (disorder)
        "720675005",  # Adverse reaction caused by irbesartan (disorder)
        "720676006",  # Adverse reaction caused by eprosartan (disorder)
        "720677002",  # Adverse reaction caused by candesartan (disorder)
        "720678007",  # Adverse reaction caused by azilsartan (disorder)
    }

class LeftVentricularAssistDeviceComplications(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts related to complications associated with left ventricular assist devices.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent complications related to heart transplant.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Left Ventricular Assist Device Complications"
    OID = "2.16.840.1.113762.1.4.1178.58"
    DEFINITION_VERSION = "20220218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    SNOMEDCT = {
        "764680009",  # Infection of left ventricular assist device driveline (disorder)
        "765181003",  # Thrombosis caused by left ventricular assist device (disorder)
        "765465004",  # Thrombosis associated with left ventricular assist device (disorder)
    }

class ModerateOrSevereLvsd(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of moderate or severe left ventricular systolic dysfunction (LVSD).

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of moderate or severe LVSD.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Moderate or Severe LVSD"
    OID = "2.16.840.1.113883.3.526.3.1090"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    SNOMEDCT = {
        "981000124106",  # Moderate left ventricular systolic dysfunction (disorder)
        "991000124109",  # Severe left ventricular systolic dysfunction (disorder)
    }

class Narcolepsy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of narcolepsy.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of narcolepsy.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Narcolepsy"
    OID = "2.16.840.1.113883.3.464.1003.114.12.1011"
    DEFINITION_VERSION = "20240112"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "G47411",  # Narcolepsy with cataplexy
        "G47419",  # Narcolepsy without cataplexy
        "G47421",  # Narcolepsy in conditions classified elsewhere with cataplexy
        "G47429",  # Narcolepsy in conditions classified elsewhere without cataplexy
    }

    SNOMEDCT = {
        "193042000",  # Cataplexy and narcolepsy (disorder)
        "427426006",  # Sleep attack (disorder)
        "434241000124107",  # Secondary narcolepsy (disorder)
        "434251000124109",  # Secondary narcolepsy without cataplexy (disorder)
        "434261000124106",  # Secondary narcolepsy with cataplexy (disorder)
        "60380001",  # Narcolepsy (disorder)
        "735676003",  # Narcolepsy type 1 (disorder)
        "91521000119104",  # Narcolepsy without cataplexy (disorder)
    }

class SubstanceUseDisorder(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of substance use disorder.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of alcohol abuse and dependence and drug abuse and dependence.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis of alcohol and drug use (substance induced disorders).
    """

    VALUE_SET_NAME = "Substance Use Disorder"
    OID = "2.16.840.1.113883.3.464.1003.106.12.1001"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "F1010",  # Alcohol abuse, uncomplicated
        "F10120",  # Alcohol abuse with intoxication, uncomplicated
        "F10121",  # Alcohol abuse with intoxication delirium
        "F10129",  # Alcohol abuse with intoxication, unspecified
        "F10130",  # Alcohol abuse with withdrawal, uncomplicated
        "F10131",  # Alcohol abuse with withdrawal delirium
        "F10132",  # Alcohol abuse with withdrawal with perceptual disturbance
        "F10139",  # Alcohol abuse with withdrawal, unspecified
        "F1014",  # Alcohol abuse with alcohol-induced mood disorder
        "F10150",  # Alcohol abuse with alcohol-induced psychotic disorder with delusions
        "F10151",  # Alcohol abuse with alcohol-induced psychotic disorder with hallucinations
        "F10159",  # Alcohol abuse with alcohol-induced psychotic disorder, unspecified
        "F10180",  # Alcohol abuse with alcohol-induced anxiety disorder
        "F10181",  # Alcohol abuse with alcohol-induced sexual dysfunction
        "F10182",  # Alcohol abuse with alcohol-induced sleep disorder
        "F10188",  # Alcohol abuse with other alcohol-induced disorder
        "F1019",  # Alcohol abuse with unspecified alcohol-induced disorder
        "F1020",  # Alcohol dependence, uncomplicated
        "F10220",  # Alcohol dependence with intoxication, uncomplicated
        "F10221",  # Alcohol dependence with intoxication delirium
        "F10229",  # Alcohol dependence with intoxication, unspecified
        "F10230",  # Alcohol dependence with withdrawal, uncomplicated
        "F10231",  # Alcohol dependence with withdrawal delirium
        "F10232",  # Alcohol dependence with withdrawal with perceptual disturbance
        "F10239",  # Alcohol dependence with withdrawal, unspecified
        "F1024",  # Alcohol dependence with alcohol-induced mood disorder
        "F10250",  # Alcohol dependence with alcohol-induced psychotic disorder with delusions
        "F10251",  # Alcohol dependence with alcohol-induced psychotic disorder with hallucinations
        "F10259",  # Alcohol dependence with alcohol-induced psychotic disorder, unspecified
        "F1026",  # Alcohol dependence with alcohol-induced persisting amnestic disorder
        "F1027",  # Alcohol dependence with alcohol-induced persisting dementia
        "F10280",  # Alcohol dependence with alcohol-induced anxiety disorder
        "F10281",  # Alcohol dependence with alcohol-induced sexual dysfunction
        "F10282",  # Alcohol dependence with alcohol-induced sleep disorder
        "F10288",  # Alcohol dependence with other alcohol-induced disorder
        "F1029",  # Alcohol dependence with unspecified alcohol-induced disorder
        "F1110",  # Opioid abuse, uncomplicated
        "F11120",  # Opioid abuse with intoxication, uncomplicated
        "F11121",  # Opioid abuse with intoxication delirium
        "F11122",  # Opioid abuse with intoxication with perceptual disturbance
        "F11129",  # Opioid abuse with intoxication, unspecified
        "F1113",  # Opioid abuse with withdrawal
        "F1114",  # Opioid abuse with opioid-induced mood disorder
        "F11150",  # Opioid abuse with opioid-induced psychotic disorder with delusions
        "F11151",  # Opioid abuse with opioid-induced psychotic disorder with hallucinations
        "F11159",  # Opioid abuse with opioid-induced psychotic disorder, unspecified
        "F11181",  # Opioid abuse with opioid-induced sexual dysfunction
        "F11182",  # Opioid abuse with opioid-induced sleep disorder
        "F11188",  # Opioid abuse with other opioid-induced disorder
        "F1119",  # Opioid abuse with unspecified opioid-induced disorder
        "F1120",  # Opioid dependence, uncomplicated
        "F11220",  # Opioid dependence with intoxication, uncomplicated
        "F11221",  # Opioid dependence with intoxication delirium
        "F11222",  # Opioid dependence with intoxication with perceptual disturbance
        "F11229",  # Opioid dependence with intoxication, unspecified
        "F1123",  # Opioid dependence with withdrawal
        "F1124",  # Opioid dependence with opioid-induced mood disorder
        "F11250",  # Opioid dependence with opioid-induced psychotic disorder with delusions
        "F11251",  # Opioid dependence with opioid-induced psychotic disorder with hallucinations
        "F11259",  # Opioid dependence with opioid-induced psychotic disorder, unspecified
        "F11281",  # Opioid dependence with opioid-induced sexual dysfunction
        "F11282",  # Opioid dependence with opioid-induced sleep disorder
        "F11288",  # Opioid dependence with other opioid-induced disorder
        "F1129",  # Opioid dependence with unspecified opioid-induced disorder
        "F1210",  # Cannabis abuse, uncomplicated
        "F12120",  # Cannabis abuse with intoxication, uncomplicated
        "F12121",  # Cannabis abuse with intoxication delirium
        "F12122",  # Cannabis abuse with intoxication with perceptual disturbance
        "F12129",  # Cannabis abuse with intoxication, unspecified
        "F1213",  # Cannabis abuse with withdrawal
        "F12150",  # Cannabis abuse with psychotic disorder with delusions
        "F12151",  # Cannabis abuse with psychotic disorder with hallucinations
        "F12159",  # Cannabis abuse with psychotic disorder, unspecified
        "F12180",  # Cannabis abuse with cannabis-induced anxiety disorder
        "F12188",  # Cannabis abuse with other cannabis-induced disorder
        "F1219",  # Cannabis abuse with unspecified cannabis-induced disorder
        "F1220",  # Cannabis dependence, uncomplicated
        "F12220",  # Cannabis dependence with intoxication, uncomplicated
        "F12221",  # Cannabis dependence with intoxication delirium
        "F12222",  # Cannabis dependence with intoxication with perceptual disturbance
        "F12229",  # Cannabis dependence with intoxication, unspecified
        "F1223",  # Cannabis dependence with withdrawal
        "F12250",  # Cannabis dependence with psychotic disorder with delusions
        "F12251",  # Cannabis dependence with psychotic disorder with hallucinations
        "F12259",  # Cannabis dependence with psychotic disorder, unspecified
        "F12280",  # Cannabis dependence with cannabis-induced anxiety disorder
        "F12288",  # Cannabis dependence with other cannabis-induced disorder
        "F1229",  # Cannabis dependence with unspecified cannabis-induced disorder
        "F1310",  # Sedative, hypnotic or anxiolytic abuse, uncomplicated
        "F13120",  # Sedative, hypnotic or anxiolytic abuse with intoxication, uncomplicated
        "F13121",  # Sedative, hypnotic or anxiolytic abuse with intoxication delirium
        "F13129",  # Sedative, hypnotic or anxiolytic abuse with intoxication, unspecified
        "F13130",  # Sedative, hypnotic or anxiolytic abuse with withdrawal, uncomplicated
        "F13131",  # Sedative, hypnotic or anxiolytic abuse with withdrawal delirium
        "F13132",  # Sedative, hypnotic or anxiolytic abuse with withdrawal with perceptual disturbance
        "F13139",  # Sedative, hypnotic or anxiolytic abuse with withdrawal, unspecified
        "F1314",  # Sedative, hypnotic or anxiolytic abuse with sedative, hypnotic or anxiolytic-induced mood disorder
        "F13150",  # Sedative, hypnotic or anxiolytic abuse with sedative, hypnotic or anxiolytic-induced psychotic disorder with delusions
        "F13151",  # Sedative, hypnotic or anxiolytic abuse with sedative, hypnotic or anxiolytic-induced psychotic disorder with hallucinations
        "F13159",  # Sedative, hypnotic or anxiolytic abuse with sedative, hypnotic or anxiolytic-induced psychotic disorder, unspecified
        "F13180",  # Sedative, hypnotic or anxiolytic abuse with sedative, hypnotic or anxiolytic-induced anxiety disorder
        "F13181",  # Sedative, hypnotic or anxiolytic abuse with sedative, hypnotic or anxiolytic-induced sexual dysfunction
        "F13182",  # Sedative, hypnotic or anxiolytic abuse with sedative, hypnotic or anxiolytic-induced sleep disorder
        "F13188",  # Sedative, hypnotic or anxiolytic abuse with other sedative, hypnotic or anxiolytic-induced disorder
        "F1319",  # Sedative, hypnotic or anxiolytic abuse with unspecified sedative, hypnotic or anxiolytic-induced disorder
        "F1320",  # Sedative, hypnotic or anxiolytic dependence, uncomplicated
        "F13220",  # Sedative, hypnotic or anxiolytic dependence with intoxication, uncomplicated
        "F13221",  # Sedative, hypnotic or anxiolytic dependence with intoxication delirium
        "F13229",  # Sedative, hypnotic or anxiolytic dependence with intoxication, unspecified
        "F13230",  # Sedative, hypnotic or anxiolytic dependence with withdrawal, uncomplicated
        "F13231",  # Sedative, hypnotic or anxiolytic dependence with withdrawal delirium
        "F13232",  # Sedative, hypnotic or anxiolytic dependence with withdrawal with perceptual disturbance
        "F13239",  # Sedative, hypnotic or anxiolytic dependence with withdrawal, unspecified
        "F1324",  # Sedative, hypnotic or anxiolytic dependence with sedative, hypnotic or anxiolytic-induced mood disorder
        "F13250",  # Sedative, hypnotic or anxiolytic dependence with sedative, hypnotic or anxiolytic-induced psychotic disorder with delusions
        "F13251",  # Sedative, hypnotic or anxiolytic dependence with sedative, hypnotic or anxiolytic-induced psychotic disorder with hallucinations
        "F13259",  # Sedative, hypnotic or anxiolytic dependence with sedative, hypnotic or anxiolytic-induced psychotic disorder, unspecified
        "F1326",  # Sedative, hypnotic or anxiolytic dependence with sedative, hypnotic or anxiolytic-induced persisting amnestic disorder
        "F1327",  # Sedative, hypnotic or anxiolytic dependence with sedative, hypnotic or anxiolytic-induced persisting dementia
        "F13280",  # Sedative, hypnotic or anxiolytic dependence with sedative, hypnotic or anxiolytic-induced anxiety disorder
        "F13281",  # Sedative, hypnotic or anxiolytic dependence with sedative, hypnotic or anxiolytic-induced sexual dysfunction
        "F13282",  # Sedative, hypnotic or anxiolytic dependence with sedative, hypnotic or anxiolytic-induced sleep disorder
        "F13288",  # Sedative, hypnotic or anxiolytic dependence with other sedative, hypnotic or anxiolytic-induced disorder
        "F1329",  # Sedative, hypnotic or anxiolytic dependence with unspecified sedative, hypnotic or anxiolytic-induced disorder
        "F1410",  # Cocaine abuse, uncomplicated
        "F14120",  # Cocaine abuse with intoxication, uncomplicated
        "F14121",  # Cocaine abuse with intoxication with delirium
        "F14122",  # Cocaine abuse with intoxication with perceptual disturbance
        "F14129",  # Cocaine abuse with intoxication, unspecified
        "F1413",  # Cocaine abuse, unspecified with withdrawal
        "F1414",  # Cocaine abuse with cocaine-induced mood disorder
        "F14150",  # Cocaine abuse with cocaine-induced psychotic disorder with delusions
        "F14151",  # Cocaine abuse with cocaine-induced psychotic disorder with hallucinations
        "F14159",  # Cocaine abuse with cocaine-induced psychotic disorder, unspecified
        "F14180",  # Cocaine abuse with cocaine-induced anxiety disorder
        "F14181",  # Cocaine abuse with cocaine-induced sexual dysfunction
        "F14182",  # Cocaine abuse with cocaine-induced sleep disorder
        "F14188",  # Cocaine abuse with other cocaine-induced disorder
        "F1419",  # Cocaine abuse with unspecified cocaine-induced disorder
        "F1420",  # Cocaine dependence, uncomplicated
        "F14220",  # Cocaine dependence with intoxication, uncomplicated
        "F14221",  # Cocaine dependence with intoxication delirium
        "F14222",  # Cocaine dependence with intoxication with perceptual disturbance
        "F14229",  # Cocaine dependence with intoxication, unspecified
        "F1423",  # Cocaine dependence with withdrawal
        "F1424",  # Cocaine dependence with cocaine-induced mood disorder
        "F14250",  # Cocaine dependence with cocaine-induced psychotic disorder with delusions
        "F14251",  # Cocaine dependence with cocaine-induced psychotic disorder with hallucinations
        "F14259",  # Cocaine dependence with cocaine-induced psychotic disorder, unspecified
        "F14280",  # Cocaine dependence with cocaine-induced anxiety disorder
        "F14281",  # Cocaine dependence with cocaine-induced sexual dysfunction
        "F14282",  # Cocaine dependence with cocaine-induced sleep disorder
        "F14288",  # Cocaine dependence with other cocaine-induced disorder
        "F1429",  # Cocaine dependence with unspecified cocaine-induced disorder
        "F1510",  # Other stimulant abuse, uncomplicated
        "F15120",  # Other stimulant abuse with intoxication, uncomplicated
        "F15121",  # Other stimulant abuse with intoxication delirium
        "F15122",  # Other stimulant abuse with intoxication with perceptual disturbance
        "F15129",  # Other stimulant abuse with intoxication, unspecified
        "F1513",  # Other stimulant abuse with withdrawal
        "F1514",  # Other stimulant abuse with stimulant-induced mood disorder
        "F15150",  # Other stimulant abuse with stimulant-induced psychotic disorder with delusions
        "F15151",  # Other stimulant abuse with stimulant-induced psychotic disorder with hallucinations
        "F15159",  # Other stimulant abuse with stimulant-induced psychotic disorder, unspecified
        "F15180",  # Other stimulant abuse with stimulant-induced anxiety disorder
        "F15181",  # Other stimulant abuse with stimulant-induced sexual dysfunction
        "F15182",  # Other stimulant abuse with stimulant-induced sleep disorder
        "F15188",  # Other stimulant abuse with other stimulant-induced disorder
        "F1519",  # Other stimulant abuse with unspecified stimulant-induced disorder
        "F1520",  # Other stimulant dependence, uncomplicated
        "F15220",  # Other stimulant dependence with intoxication, uncomplicated
        "F15221",  # Other stimulant dependence with intoxication delirium
        "F15222",  # Other stimulant dependence with intoxication with perceptual disturbance
        "F15229",  # Other stimulant dependence with intoxication, unspecified
        "F1523",  # Other stimulant dependence with withdrawal
        "F1524",  # Other stimulant dependence with stimulant-induced mood disorder
        "F15250",  # Other stimulant dependence with stimulant-induced psychotic disorder with delusions
        "F15251",  # Other stimulant dependence with stimulant-induced psychotic disorder with hallucinations
        "F15259",  # Other stimulant dependence with stimulant-induced psychotic disorder, unspecified
        "F15280",  # Other stimulant dependence with stimulant-induced anxiety disorder
        "F15281",  # Other stimulant dependence with stimulant-induced sexual dysfunction
        "F15282",  # Other stimulant dependence with stimulant-induced sleep disorder
        "F15288",  # Other stimulant dependence with other stimulant-induced disorder
        "F1529",  # Other stimulant dependence with unspecified stimulant-induced disorder
        "F1610",  # Hallucinogen abuse, uncomplicated
        "F16120",  # Hallucinogen abuse with intoxication, uncomplicated
        "F16121",  # Hallucinogen abuse with intoxication with delirium
        "F16122",  # Hallucinogen abuse with intoxication with perceptual disturbance
        "F16129",  # Hallucinogen abuse with intoxication, unspecified
        "F1614",  # Hallucinogen abuse with hallucinogen-induced mood disorder
        "F16150",  # Hallucinogen abuse with hallucinogen-induced psychotic disorder with delusions
        "F16151",  # Hallucinogen abuse with hallucinogen-induced psychotic disorder with hallucinations
        "F16159",  # Hallucinogen abuse with hallucinogen-induced psychotic disorder, unspecified
        "F16180",  # Hallucinogen abuse with hallucinogen-induced anxiety disorder
        "F16183",  # Hallucinogen abuse with hallucinogen persisting perception disorder (flashbacks)
        "F16188",  # Hallucinogen abuse with other hallucinogen-induced disorder
        "F1619",  # Hallucinogen abuse with unspecified hallucinogen-induced disorder
        "F1620",  # Hallucinogen dependence, uncomplicated
        "F16220",  # Hallucinogen dependence with intoxication, uncomplicated
        "F16221",  # Hallucinogen dependence with intoxication with delirium
        "F16229",  # Hallucinogen dependence with intoxication, unspecified
        "F1624",  # Hallucinogen dependence with hallucinogen-induced mood disorder
        "F16250",  # Hallucinogen dependence with hallucinogen-induced psychotic disorder with delusions
        "F16251",  # Hallucinogen dependence with hallucinogen-induced psychotic disorder with hallucinations
        "F16259",  # Hallucinogen dependence with hallucinogen-induced psychotic disorder, unspecified
        "F16280",  # Hallucinogen dependence with hallucinogen-induced anxiety disorder
        "F16283",  # Hallucinogen dependence with hallucinogen persisting perception disorder (flashbacks)
        "F16288",  # Hallucinogen dependence with other hallucinogen-induced disorder
        "F1629",  # Hallucinogen dependence with unspecified hallucinogen-induced disorder
        "F1810",  # Inhalant abuse, uncomplicated
        "F18120",  # Inhalant abuse with intoxication, uncomplicated
        "F18121",  # Inhalant abuse with intoxication delirium
        "F18129",  # Inhalant abuse with intoxication, unspecified
        "F1814",  # Inhalant abuse with inhalant-induced mood disorder
        "F18150",  # Inhalant abuse with inhalant-induced psychotic disorder with delusions
        "F18151",  # Inhalant abuse with inhalant-induced psychotic disorder with hallucinations
        "F18159",  # Inhalant abuse with inhalant-induced psychotic disorder, unspecified
        "F1817",  # Inhalant abuse with inhalant-induced dementia
        "F18180",  # Inhalant abuse with inhalant-induced anxiety disorder
        "F18188",  # Inhalant abuse with other inhalant-induced disorder
        "F1819",  # Inhalant abuse with unspecified inhalant-induced disorder
        "F1820",  # Inhalant dependence, uncomplicated
        "F18220",  # Inhalant dependence with intoxication, uncomplicated
        "F18221",  # Inhalant dependence with intoxication delirium
        "F18229",  # Inhalant dependence with intoxication, unspecified
        "F1824",  # Inhalant dependence with inhalant-induced mood disorder
        "F18250",  # Inhalant dependence with inhalant-induced psychotic disorder with delusions
        "F18251",  # Inhalant dependence with inhalant-induced psychotic disorder with hallucinations
        "F18259",  # Inhalant dependence with inhalant-induced psychotic disorder, unspecified
        "F1827",  # Inhalant dependence with inhalant-induced dementia
        "F18280",  # Inhalant dependence with inhalant-induced anxiety disorder
        "F18288",  # Inhalant dependence with other inhalant-induced disorder
        "F1829",  # Inhalant dependence with unspecified inhalant-induced disorder
        "F1910",  # Other psychoactive substance abuse, uncomplicated
        "F19120",  # Other psychoactive substance abuse with intoxication, uncomplicated
        "F19121",  # Other psychoactive substance abuse with intoxication delirium
        "F19122",  # Other psychoactive substance abuse with intoxication with perceptual disturbances
        "F19129",  # Other psychoactive substance abuse with intoxication, unspecified
        "F19130",  # Other psychoactive substance abuse with withdrawal, uncomplicated
        "F19131",  # Other psychoactive substance abuse with withdrawal delirium
        "F19132",  # Other psychoactive substance abuse with withdrawal with perceptual disturbance
        "F19139",  # Other psychoactive substance abuse with withdrawal, unspecified
        "F1914",  # Other psychoactive substance abuse with psychoactive substance-induced mood disorder
        "F19150",  # Other psychoactive substance abuse with psychoactive substance-induced psychotic disorder with delusions
        "F19151",  # Other psychoactive substance abuse with psychoactive substance-induced psychotic disorder with hallucinations
        "F19159",  # Other psychoactive substance abuse with psychoactive substance-induced psychotic disorder, unspecified
        "F1916",  # Other psychoactive substance abuse with psychoactive substance-induced persisting amnestic disorder
        "F1917",  # Other psychoactive substance abuse with psychoactive substance-induced persisting dementia
        "F19180",  # Other psychoactive substance abuse with psychoactive substance-induced anxiety disorder
        "F19181",  # Other psychoactive substance abuse with psychoactive substance-induced sexual dysfunction
        "F19182",  # Other psychoactive substance abuse with psychoactive substance-induced sleep disorder
        "F19188",  # Other psychoactive substance abuse with other psychoactive substance-induced disorder
        "F1919",  # Other psychoactive substance abuse with unspecified psychoactive substance-induced disorder
        "F1920",  # Other psychoactive substance dependence, uncomplicated
        "F19220",  # Other psychoactive substance dependence with intoxication, uncomplicated
        "F19221",  # Other psychoactive substance dependence with intoxication delirium
        "F19222",  # Other psychoactive substance dependence with intoxication with perceptual disturbance
        "F19229",  # Other psychoactive substance dependence with intoxication, unspecified
        "F19230",  # Other psychoactive substance dependence with withdrawal, uncomplicated
        "F19231",  # Other psychoactive substance dependence with withdrawal delirium
        "F19232",  # Other psychoactive substance dependence with withdrawal with perceptual disturbance
        "F19239",  # Other psychoactive substance dependence with withdrawal, unspecified
        "F1924",  # Other psychoactive substance dependence with psychoactive substance-induced mood disorder
        "F19250",  # Other psychoactive substance dependence with psychoactive substance-induced psychotic disorder with delusions
        "F19251",  # Other psychoactive substance dependence with psychoactive substance-induced psychotic disorder with hallucinations
        "F19259",  # Other psychoactive substance dependence with psychoactive substance-induced psychotic disorder, unspecified
        "F1926",  # Other psychoactive substance dependence with psychoactive substance-induced persisting amnestic disorder
        "F1927",  # Other psychoactive substance dependence with psychoactive substance-induced persisting dementia
        "F19280",  # Other psychoactive substance dependence with psychoactive substance-induced anxiety disorder
        "F19281",  # Other psychoactive substance dependence with psychoactive substance-induced sexual dysfunction
        "F19282",  # Other psychoactive substance dependence with psychoactive substance-induced sleep disorder
        "F19288",  # Other psychoactive substance dependence with other psychoactive substance-induced disorder
        "F1929",  # Other psychoactive substance dependence with unspecified psychoactive substance-induced disorder
    }

    SNOMEDCT = {
        "416714005",  # Episodic flashbacks (disorder)
        "417143004",  # Short duration flashbacks (disorder)
        "520841000124109",  # Diamorphine abuse (disorder)
        "724662001",  # Sedative withdrawal syndrome without complication (disorder)
        "724664000",  # Anxiolytic withdrawal syndrome without complication (disorder)
        "724725009",  # Psychoactive substance withdrawal syndrome without complication (disorder)
        "85561006",  # Alcohol withdrawal syndrome without complication (disorder)
        "10327003",  # Cocaine-induced mood disorder (disorder)
        "10741871000119101",  # Alcohol dependence in pregnancy (disorder)
        "10755041000119100",  # Alcohol dependence in childbirth (disorder)
        "1081000119105",  # Opioid dependence, on agonist therapy (disorder)
        "11387009",  # Organic mental disorder caused by psychoactive substance (disorder)
        "1230071005",  # Amfetamine and/or amfetamine derivative abuse, episodic (disorder)
        "1230072003",  # Amfetamine and/or amfetamine derivative abuse, continuous (disorder)
        "1230084001",  # Nondependent psychostimulant abuse, continuous (disorder)
        "1230085000",  # Nondependent amfetamine and/or amfetamine derivative abuse, continuous (disorder)
        "1230086004",  # Nondependent psychostimulant abuse, episodic (disorder)
        "1230087008",  # Nondependent amfetamine and/or amfetamine derivative abuse, episodic (disorder)
        "1230088003",  # Nondependent amfetamine and/or amfetamine derivative abuse (disorder)
        "1230089006",  # Nondependent psychostimulant abuse (disorder)
        "1231160001",  # Hypnotic dependence, episodic (disorder)
        "1231162009",  # Hypnotic dependence, continuous (disorder)
        "1231319004",  # Methylenedioxymethamphetamine dependence (disorder)
        "1231327008",  # Amfetamine and/or amfetamine derivative dependence episodic (disorder)
        "1231328003",  # Amfetamine and/or amfetamine derivative dependence continuous (disorder)
        "1231333004",  # Psychostimulant dependence episodic (disorder)
        "1231336007",  # Psychostimulant dependence continuous (disorder)
        "12398281000119105",  # Methamphetamine withdrawal syndrome (disorder)
        "12398651000119100",  # Sleep disorder caused by methamphetamine (disorder)
        "1255013006",  # Combined opioid with non-opioid substance dependence (disorder)
        "1255015004",  # Combined opioid with non-opioid substance dependence, episodic (disorder)
        "1255018002",  # Combined opioid with non-opioid substance dependence, continuous (disorder)
        "125851000119106",  # Continuous sedative abuse (disorder)
        "135301000119103",  # Parasomnia caused by alcohol (disorder)
        "135311000119100",  # Insomnia caused by alcohol (disorder)
        "135321000119107",  # Hypersomnia caused by alcohol (disorder)
        "1383008",  # Hallucinogen mood disorder (disorder)
        "144981000119109",  # Nondependent intraveous amphetamine abuse (disorder)
        "145101000119102",  # Intravenous cocaine abuse (disorder)
        "145121000119106",  # Intravenous nondependent opioid abuse (disorder)
        "145841000119107",  # Episodic phencyclidine abuse (disorder)
        "14784000",  # Organic mental disorder caused by opioid (disorder)
        "15167005",  # Alcohol abuse (disorder)
        "15277004",  # Hallucinogen-induced anxiety disorder (disorder)
        "154211000119108",  # Chronic pancreatitis due to chronic alcoholism (disorder)
        "1686006",  # Sedative, hypnotic AND/OR anxiolytic-induced anxiety disorder (disorder)
        "18653004",  # Alcohol intoxication delirium (disorder)
        "18689007",  # Inhalant intoxication delirium (disorder)
        "191475009",  # Chronic alcoholic brain syndrome (disorder)
        "191476005",  # Alcohol withdrawal hallucinosis (disorder)
        "191478006",  # Alcoholic paranoia (disorder)
        "191480000",  # Alcohol withdrawal syndrome (disorder)
        "191811004",  # Continuous chronic alcoholism (disorder)
        "191812006",  # Episodic chronic alcoholism (disorder)
        "191819002",  # Continuous opioid dependence (disorder)
        "191820008",  # Episodic opioid dependence (disorder)
        "191831000",  # Cocaine dependence, continuous (disorder)
        "191832007",  # Cocaine dependence, episodic (disorder)
        "191837001",  # Cannabis dependence, continuous (disorder)
        "191838006",  # Cannabis dependence, episodic (disorder)
        "191849000",  # Hallucinogen dependence, continuous (disorder)
        "191850000",  # Hallucinogen dependence, episodic (disorder)
        "191853003",  # Glue sniffing dependence (disorder)
        "191855005",  # Glue sniffing dependence, continuous (disorder)
        "191856006",  # Glue sniffing dependence, episodic (disorder)
        "191882002",  # Nondependent alcohol abuse, continuous (disorder)
        "191883007",  # Nondependent alcohol abuse, episodic (disorder)
        "191891003",  # Nondependent cannabis abuse (disorder)
        "191893000",  # Nondependent cannabis abuse, continuous (disorder)
        "191894006",  # Nondependent cannabis abuse, episodic (disorder)
        "191899001",  # Nondependent hallucinogen abuse, continuous (disorder)
        "191900006",  # Nondependent hallucinogen abuse, episodic (disorder)
        "191909007",  # Nondependent opioid abuse (disorder)
        "191912005",  # Nondependent opioid abuse, continuous (disorder)
        "191913000",  # Nondependent opioid abuse, episodic (disorder)
        "191916008",  # Nondependent cocaine abuse (disorder)
        "191918009",  # Nondependent cocaine abuse, continuous (disorder)
        "191919001",  # Nondependent cocaine abuse, episodic (disorder)
        "19445006",  # Opioid-induced psychotic disorder with hallucinations (disorder)
        "1973000",  # Sedative, hypnotic AND/OR anxiolytic-induced psychotic disorder with delusions (disorder)
        "20385005",  # Psychotic disorder with delusions caused by opioid (disorder)
        "20876004",  # Inhalant-induced anxiety disorder (disorder)
        "21647008",  # Amphetamine dependence (disorder)
        "22574000",  # Cocaine-induced sleep disorder (disorder)
        "230443000",  # Narcotic withdrawal epilepsy (disorder)
        "231462006",  # Barbiturate abuse (disorder)
        "231468005",  # Lysergic acid diethylamide dependence (disorder)
        "231469002",  # Mescaline dependence (disorder)
        "231470001",  # Anxiolytic dependence (disorder)
        "231472009",  # Barbiturate dependence (disorder)
        "231473004",  # Benzodiazepine dependence (disorder)
        "231474005",  # Diazepam dependence (disorder)
        "231477003",  # Dependence on diamorphine (disorder)
        "231478008",  # Methadone dependence (disorder)
        "231479000",  # Morphine dependence (disorder)
        "231480002",  # Opium dependence (disorder)
        "23601000119102",  # Sedative withdrawal syndrome (disorder)
        "2403008",  # Psychoactive substance dependence (disorder)
        "25753007",  # Amphetamine-induced sleep disorder (disorder)
        "26714005",  # Cannabis-induced psychotic disorder with hallucinations (disorder)
        "268645007",  # Nondependent alcohol abuse (disorder)
        "268646008",  # Nondependent hallucinogen abuse (disorder)
        "275471001",  # Psychostimulant dependence (disorder)
        "27956007",  # Cocaine intoxication (disorder)
        "281004",  # Dementia associated with alcoholism (disorder)
        "284591009",  # Persistent alcohol abuse (disorder)
        "288021000119107",  # Disorder due to alcohol abuse (disorder)
        "288031000119105",  # Alcohol induced disorder co-occurrent and due to alcohol dependence (disorder)
        "288461000119105",  # Psychosis caused by inhalant (disorder)
        "28864000",  # Sedative, hypnotic AND/OR anxiolytic-induced mood disorder (disorder)
        "288851000119106",  # Opioid-induced mood disorder due to opioid abuse (disorder)
        "288861000119108",  # Opioid-induced mood disorder due to opioid dependence (disorder)
        "29733004",  # Mood disorder caused by opioid (disorder)
        "30491001",  # Cocaine delusional disorder (disorder)
        "308742005",  # Alcohol withdrawal-induced convulsion (disorder)
        "312098001",  # Sedative, hypnotic AND/OR anxiolytic-related disorder (disorder)
        "312936002",  # Organic mental disorder caused by anxiolytic agent (disorder)
        "31956009",  # Cocaine dependence (disorder)
        "32009006",  # Hallucinogen hallucinosis (disorder)
        "32358001",  # Amphetamine delusional disorder (disorder)
        "32875003",  # Inhalant-induced persisting dementia (disorder)
        "34111000119108",  # Nondependent amphetamine abuse (disorder)
        "34938008",  # Anxiety disorder caused by alcohol (disorder)
        "361151007",  # Delirium due to sedative withdrawal (disorder)
        "37344009",  # Cannabis abuse (disorder)
        "38247002",  # Hallucinogen dependence (disorder)
        "39807006",  # Cannabis intoxication delirium (disorder)
        "39951001",  # Cannabis-induced anxiety disorder (disorder)
        "40571009",  # Hallucinogen intoxication delirium (disorder)
        "425339005",  # Barbiturate withdrawal syndrome (disorder)
        "425885002",  # Continuous phencyclidine abuse (disorder)
        "426001001",  # Fentanyl dependence (disorder)
        "426095000",  # Continuous inhalant abuse (disorder)
        "426873000",  # Methamphetamine dependence (disorder)
        "427229002",  # Episodic inhalant abuse (disorder)
        "427327003",  # Sedative dependence (disorder)
        "428370001",  # Psychostimulant withdrawal syndrome (disorder)
        "43497001",  # Amfetamine-induced mood disorder (disorder)
        "441527004",  # Stimulant abuse (disorder)
        "442406005",  # Stimulant dependence (disorder)
        "46975003",  # Organic mental disorder caused by cocaine (disorder)
        "47664006",  # Sedative, hypnotic AND/OR anxiolytic-induced psychotic disorder with hallucinations (disorder)
        "5002000",  # Inhalant dependence (disorder)
        "50320000",  # Hallucinogen intoxication (disorder)
        "50933003",  # Hallucinogen delusional disorder (disorder)
        "51443000",  # Amphetamine-induced psychotic disorder with hallucinations (disorder)
        "51493001",  # Anxiety disorder caused by cocaine (disorder)
        "52866005",  # Opioid intoxication delirium (disorder)
        "53050002",  # Organic mental disorder caused by hallucinogen (disorder)
        "5444000",  # Sedative, hypnotic AND/OR anxiolytic intoxication delirium (disorder)
        "5602001",  # Opioid abuse (disorder)
        "57588009",  # Sedative, hypnotic AND/OR anxiolytic-induced sleep disorder (disorder)
        "58727001",  # Phencyclidine dependence (disorder)
        "59651006",  # Sedative, hypnotic AND/OR anxiolytic-induced persisting dementia (disorder)
        "61104008",  # Inhalant-induced organic mental disorder (disorder)
        "61144001",  # Alcohol-induced psychotic disorder with delusions (disorder)
        "6348008",  # Cocaine-induced psychotic disorder with hallucinations (disorder)
        "63649001",  # Cannabis delusional disorder (disorder)
        "63983005",  # Inhalant-induced psychotic disorder with hallucinations (disorder)
        "64386003",  # Sedative abuse (disorder)
        "66590003",  # Alcohol dependence (disorder)
        "699449003",  # Methamphetamine abuse (disorder)
        "70328006",  # Cocaine intoxication delirium (disorder)
        "70340006",  # Inhalant abuse (disorder)
        "703845008",  # Buprenorphine dependence (disorder)
        "703846009",  # Buprenorphine withdrawal syndrome (disorder)
        "703848005",  # Cannabis withdrawal syndrome (disorder)
        "703849002",  # Benzodiazepine withdrawal syndrome (disorder)
        "703850002",  # Delirium due to benzodiazepine withdrawal (disorder)
        "7052005",  # Alcohol hallucinosis (disorder)
        "70701004",  # Alcoholic amblyopia (disorder)
        "7071007",  # Phencyclidine abuse (disorder)
        "70932007",  # Sexual dysfunction caused by amfetamine (finding)
        "71328000",  # Sexual dysfunction caused by opioid (finding)
        "713583005",  # Mild alcohol dependence (disorder)
        "713862009",  # Severe alcohol dependence (disorder)
        "714829008",  # Moderate alcohol dependence (disorder)
        "7200002",  # Alcoholism (disorder)
        "723926008",  # Perceptual disturbances and seizures co-occurrent and due to alcohol withdrawal (disorder)
        "723927004",  # Psychotic disorder caused by alcohol with schizophreniform symptoms (disorder)
        "723936000",  # Psychotic disorder caused by cannabis (disorder)
        "723937009",  # Sleep disorder caused by cannabis (disorder)
        "724655005",  # Psychotic disorder caused by opioid (disorder)
        "724656006",  # Hypnotic dependence (disorder)
        "724660009",  # Hypnotic withdrawal syndrome (disorder)
        "724661008",  # Anxiolytic withdrawal syndrome (disorder)
        "724663006",  # Hypnotic withdrawal syndrome without complication (disorder)
        "724665004",  # Perceptual disturbances co-occurrent and due to sedative withdrawal (disorder)
        "724666003",  # Perceptual disturbances co-occurrent and due to hypnotic withdrawal (disorder)
        "724667007",  # Perceptual disturbances co-occurrent and due to anxiolytic withdrawal (disorder)
        "724668002",  # Seizure co-occurrent and due to sedative withdrawal (disorder)
        "724669005",  # Seizure co-occurrent and due to hypnotic withdrawal (disorder)
        "724670006",  # Seizure co-occurrent and due to anxiolytic withdrawal (disorder)
        "724671005",  # Perceptual disturbances and seizures co-occurrent and due to sedative withdrawal (disorder)
        "724672003",  # Perceptual disturbances and seizures co-occurrent and due to hypnotic withdrawal (disorder)
        "724675001",  # Psychotic disorder caused by anxiolytic (disorder)
        "724689006",  # Psychotic disorder caused by cocaine (disorder)
        "724690002",  # Mood disorder with depressive symptoms caused by cocaine (disorder)
        "724691003",  # Mood disorder with manic symptoms caused by cocaine (disorder)
        "724692005",  # Mood disorder with mixed depressive and manic symptoms caused by cocaine (disorder)
        "724693000",  # Obsessive compulsive disorder caused by cocaine (disorder)
        "724696008",  # Psychotic disorder caused by hallucinogen (disorder)
        "724701001",  # Volatile inhalant withdrawal (disorder)
        "724702008",  # Psychotic disorder caused by volatile inhalant (disorder)
        "724706006",  # Psychotic disorder caused by methylenedioxymethamphetamine (disorder)
        "724713006",  # Harmful use of ketamine (disorder)
        "724715004",  # Ketamine dependence (disorder)
        "724720004",  # Mood disorder caused by dissociative drug (disorder)
        "724726005",  # Perceptual disturbances co-occurrent and due to psychoactive substance withdrawal (disorder)
        "724727001",  # Seizure co-occurrent and due to psychoactive substance withdrawal (disorder)
        "724728006",  # Perceptual disturbances and seizures co-occurrent and due to psychoactive substance withdrawal (disorder)
        "724732000",  # Sleep disorder caused by psychoactive substance (disorder)
        "73097000",  # Alcohol amnestic disorder (disorder)
        "735750005",  # Psychotic disorder with schizophreniform symptoms caused by cocaine (disorder)
        "737338002",  # Synthetic cannabinoid withdrawal syndrome (disorder)
        "74851005",  # Hallucinogen abuse (disorder)
        "74934004",  # Psychoactive substance withdrawal syndrome (disorder)
        "75122001",  # Inhalant-induced psychotic disorder with delusions (disorder)
        "75544000",  # Opioid dependence (disorder)
        "762320004",  # Mood disorder with manic symptoms caused by opioid (disorder)
        "762321000",  # Mood disorder with depressive symptoms caused by opioid (disorder)
        "762322007",  # Mood disorder with mixed depressive and manic symptoms caused by opioid (disorder)
        "762324008",  # Delirium caused by stimulant (disorder)
        "762325009",  # Psychotic disorder caused by stimulant (disorder)
        "762326005",  # Psychotic disorder with hallucinations caused by stimulant (disorder)
        "762327001",  # Psychotic disorder with delusions caused by stimulant (disorder)
        "762333005",  # Sexual dysfunction caused by stimulant (disorder)
        "762334004",  # Sleep disorder caused by stimulant (disorder)
        "762335003",  # Mood disorder with manic symptoms caused by hallucinogen (disorder)
        "762336002",  # Mood disorder with depressive symptoms caused by hallucinogen (disorder)
        "762337006",  # Mood disorder with mixed depressive and manic symptoms caused by hallucinogen (disorder)
        "762338001",  # Mood disorder with manic symptoms caused by volatile inhalant (disorder)
        "762339009",  # Mood disorder with depressive symptoms caused by volatile inhalant (disorder)
        "762340006",  # Mood disorder with mixed depressive and manic symptoms caused by volatile inhalant (disorder)
        "762344002",  # Mood disorder with manic symptoms caused by dissociative drug (disorder)
        "762345001",  # Mood disorder with depressive symptoms caused by dissociative drug (disorder)
        "762346000",  # Mood disorder with mixed depressive and manic symptoms caused by dissociative drug (disorder)
        "762504005",  # Abuse of synthetic cathinone (disorder)
        "762505006",  # Synthetic cathinone dependence (disorder)
        "762517008",  # Sexual dysfunction caused by synthetic cathinone (disorder)
        "762672001",  # Synthetic cathinone withdrawal syndrome (disorder)
        "77355000",  # Organic mental disorder caused by cannabis (disorder)
        "78267003",  # Cocaine abuse (disorder)
        "78358001",  # Amphetamine withdrawal syndrome (disorder)
        "785277001",  # Harmful pattern of use of amfetamine and/or amfetamine derivative (disorder)
        "80868005",  # Cocaine withdrawal syndrome (disorder)
        "817962007",  # Chlordiazepoxide dependence (disorder)
        "82339009",  # Anxiety disorder caused by amfetamine (disorder)
        "83367009",  # Organic mental disorder caused by amphetamine and/or amfetamine derivative (disorder)
        "838527002",  # Amfetamine and/or amfetamine derivative dependence (disorder)
        "85005007",  # Cannabis dependence (disorder)
        "86325007",  # Non megaloblastic anemia due to alcoholism (disorder)
        "8635005",  # Alcohol withdrawal delirium (disorder)
        "86391000119101",  # Inhalant dependence, continuous (disorder)
        "86401000119104",  # Inhalant dependence, episodic (disorder)
        "87132004",  # Opioid withdrawal syndrome (disorder)
        "87810006",  # Megaloblastic anemia due to alcoholism (disorder)
        "8837000",  # Delirium caused by amphetamine (disorder)
        "88926005",  # Opioid-induced sleep disorder (disorder)
        "89451009",  # Inhalant-induced mood disorder (disorder)
        "91388009",  # Psychoactive substance abuse (disorder)
        "95661003",  # Ergotamine withdrawal syndrome (disorder)
        "97571000119109",  # Thrombocytopenia co-occurrent and due to alcoholism (disorder)
    }

class PrimaryOpenAngleGlaucoma(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of primary open-angle glaucoma (POAG).

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of primary open-angle glaucoma (POAG) and open-angle glaucoma.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis for 'unspecified eye.'
    """

    VALUE_SET_NAME = "Primary Open Angle Glaucoma"
    OID = "2.16.840.1.113883.3.526.3.326"
    DEFINITION_VERSION = "20210209"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H4010X0",  # Unspecified open-angle glaucoma, stage unspecified
        "H4010X1",  # Unspecified open-angle glaucoma, mild stage
        "H4010X2",  # Unspecified open-angle glaucoma, moderate stage
        "H4010X3",  # Unspecified open-angle glaucoma, severe stage
        "H4010X4",  # Unspecified open-angle glaucoma, indeterminate stage
        "H401110",  # Primary open-angle glaucoma, right eye, stage unspecified
        "H401111",  # Primary open-angle glaucoma, right eye, mild stage
        "H401112",  # Primary open-angle glaucoma, right eye, moderate stage
        "H401113",  # Primary open-angle glaucoma, right eye, severe stage
        "H401114",  # Primary open-angle glaucoma, right eye, indeterminate stage
        "H401120",  # Primary open-angle glaucoma, left eye, stage unspecified
        "H401121",  # Primary open-angle glaucoma, left eye, mild stage
        "H401122",  # Primary open-angle glaucoma, left eye, moderate stage
        "H401123",  # Primary open-angle glaucoma, left eye, severe stage
        "H401124",  # Primary open-angle glaucoma, left eye, indeterminate stage
        "H401130",  # Primary open-angle glaucoma, bilateral, stage unspecified
        "H401131",  # Primary open-angle glaucoma, bilateral, mild stage
        "H401132",  # Primary open-angle glaucoma, bilateral, moderate stage
        "H401133",  # Primary open-angle glaucoma, bilateral, severe stage
        "H401134",  # Primary open-angle glaucoma, bilateral, indeterminate stage
        "H401210",  # Low-tension glaucoma, right eye, stage unspecified
        "H401211",  # Low-tension glaucoma, right eye, mild stage
        "H401212",  # Low-tension glaucoma, right eye, moderate stage
        "H401213",  # Low-tension glaucoma, right eye, severe stage
        "H401214",  # Low-tension glaucoma, right eye, indeterminate stage
        "H401220",  # Low-tension glaucoma, left eye, stage unspecified
        "H401221",  # Low-tension glaucoma, left eye, mild stage
        "H401222",  # Low-tension glaucoma, left eye, moderate stage
        "H401223",  # Low-tension glaucoma, left eye, severe stage
        "H401224",  # Low-tension glaucoma, left eye, indeterminate stage
        "H401230",  # Low-tension glaucoma, bilateral, stage unspecified
        "H401231",  # Low-tension glaucoma, bilateral, mild stage
        "H401232",  # Low-tension glaucoma, bilateral, moderate stage
        "H401233",  # Low-tension glaucoma, bilateral, severe stage
        "H401234",  # Low-tension glaucoma, bilateral, indeterminate stage
        "H40151",  # Residual stage of open-angle glaucoma, right eye
        "H40152",  # Residual stage of open-angle glaucoma, left eye
        "H40153",  # Residual stage of open-angle glaucoma, bilateral
    }

    SNOMEDCT = {
        "111513000",  # Advanced open-angle glaucoma (disorder)
        "1231243004",  # High pressure primary open-angle glaucoma (disorder)
        "1306287007",  # Primary open angle glaucoma of left eye (disorder)
        "1306288002",  # Primary open angle glaucoma of right eye (disorder)
        "1306289005",  # Primary open angle glaucoma of bilateral eyes (disorder)
        "1306884006",  # Normal pressure primary open-angle glaucoma of left eye (disorder)
        "1306885007",  # Normal pressure primary open-angle glaucoma of right eye (disorder)
        "1306886008",  # Normal pressure primary open-angle glaucoma of bilateral eyes (disorder)
        "15633281000119103",  # Bilateral primary open angle glaucoma (disorder)
        "50485007",  # Low tension glaucoma (disorder)
        "77075001",  # Primary open angle glaucoma (disorder)
        "84494001",  # Open-angle glaucoma (disorder)
    }

class AllergyToBetaBlockerTherapy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of an allergy to beta blocker therapy.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of an allergy to beta blocker therapy.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Allergy to Beta Blocker Therapy"
    OID = "2.16.840.1.113883.3.526.3.1177"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    SNOMEDCT = {
        "293962009",  # Allergy to beta adrenergic receptor antagonist (finding)
        "293963004",  # Allergy to beta-1 adrenergic receptor antagonist (finding)
        "293964005",  # Allergy to acebutolol (finding)
        "293965006",  # Allergy to atenolol (finding)
        "293966007",  # Allergy to betaxolol (finding)
        "293967003",  # Allergy to bisoprolol (finding)
        "293968008",  # Allergy to celiprolol (finding)
        "293969000",  # Allergy to esmolol (finding)
        "293970004",  # Allergy to metoprolol (finding)
        "293972007",  # Allergy to nadolol (finding)
        "293973002",  # Allergy to pindolol (finding)
        "293974008",  # Allergy to carvedilol (finding)
        "293975009",  # Allergy to metipranolol (finding)
        "293976005",  # Allergy to carteolol (finding)
        "293977001",  # Allergy to labetalol (finding)
        "293978006",  # Allergy to levobunolol (finding)
        "293979003",  # Allergy to oxprenolol (finding)
        "293980000",  # Allergy to penbutolol (finding)
        "293981001",  # Allergy to practolol (finding)
        "293982008",  # Allergy to propranolol (finding)
        "293983003",  # Allergy to sotalol (finding)
        "293984009",  # Allergy to timolol (finding)
    }

class Arrhythmia(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of arrhythmia.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of arrhythmia.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Arrhythmia"
    OID = "2.16.840.1.113883.3.526.3.366"
    DEFINITION_VERSION = "20200306"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "I498",  # Other specified cardiac arrhythmias
        "I499",  # Cardiac arrhythmia, unspecified
    }

    SNOMEDCT = {
        "1010405004",  # Paroxysmal atrial fibrillation with rapid ventricular response (disorder)
        "10164001",  # Parasystole (disorder)
        "10626002",  # Multifocal premature ventricular complexes (disorder)
        "11157007",  # Ventricular bigeminy (disorder)
        "1142040009",  # Right atrial non-cavotricuspid isthmus dependent macro re-entrant atrial tachycardia (disorder)
        "1142041008",  # Non-scar mediated right atrial non-cavotricuspid isthmus dependent macro re-entrant atrial tachycardia (disorder)
        "1142057008",  # Anticlockwise cavotricuspid isthmus dependent macroreentry tachycardia (disorder)
        "1142064005",  # Antidromic atrioventricular reciprocating tachycardia utilizing atrio-ventricular accessory pathway with antegrade unidirectional conduction (disorder)
        "1142066007",  # Antidromic atrioventricular reciprocating tachycardia utilizing atrio-fascicular accessory pathway with antegrade unidirectional conduction (disorder)
        "1142067003",  # Non scar mediated left atrial non-cavotricuspid isthmus dependent macro re-entrant atrial tachycardia (disorder)
        "1142068008",  # Antidromic atrioventricular reciprocating tachycardia utilizing accessory pathway with antegrade unidirectional conduction (disorder)
        "1142069000",  # Scar mediated left atrial non-cavotricuspid isthmus dependent macro re-entrant atrial tachycardia (disorder)
        "1142082008",  # Idiopathic junctional ectopic tachycardia (disorder)
        "1142086006",  # Congenital junctional ectopic tachycardia (disorder)
        "1142090008",  # Clockwise cavotricuspid isthmus dependent macroreentry tachycardia (disorder)
        "1142093005",  # Left atrial non-cavotricuspid isthmus dependent macro re-entrant atrial tachycardia (disorder)
        "1142098001",  # Junctional ectopic tachycardia following surgical procedure on heart (disorder)
        "1142103004",  # Antidromic atrioventricular reciprocating tachycardia utilizing nodo-fascicular accessory pathway with antegrade unidirectional conduction (disorder)
        "1142105006",  # Typical atrioventricular nodal re-entry tachycardia (disorder)
        "1142114001",  # Scar mediated right atrial non-cavotricuspid isthmus dependent macro re-entrant atrial tachycardia (disorder)
        "1142117008",  # Atypical slow slow atrioventricular nodal re-entry tachycardia (disorder)
        "1142118003",  # Atypical atrioventricular nodal re-entry tachycardia (disorder)
        "1142119006",  # Atypical fast slow atrioventricular nodal re-entry tachycardia (disorder)
        "1142120000",  # Orthodromic atrioventricular reciprocating tachycardia utilizing manifest accessory pathway (disorder)
        "1142121001",  # Atrioventricular nodal reentry tachycardia with twin atrioventricular nodes (disorder)
        "1142122008",  # Antidromic atrioventricular reciprocating tachycardia utilizing accessory pathway with bidirectional conduction (disorder)
        "1142123003",  # Atrial premature depolarization with aberrant ventricular conduction (disorder)
        "1142124009",  # Atrial premature depolarization with normal conduction (disorder)
        "1142204001",  # Orthodromic atrioventricular reciprocating tachycardia utilizing concealed accessory pathway (disorder)
        "1156821008",  # Autosomal recessive sick sinus syndrome (disorder)
        "11849007",  # Atrioventricular junctional rhythm (disorder)
        "1197418004",  # Familial atrial tachyarrhythmia, infra-Hisian cardiac conduction disease (disorder)
        "120041000119109",  # Atrial fibrillation with rapid ventricular response (disorder)
        "1204129001",  # Catecholaminergic polymorphic ventricular tachycardia due to RYR2 mutation (disorder)
        "1204167003",  # Catecholaminergic polymorphic ventricular tachycardia due to calsequestrin mutation (disorder)
        "1208832006",  # Pacemaker mediated tachycardia (disorder)
        "1208861007",  # Long QT syndrome type 4 (disorder)
        "1208863005",  # Long QT syndrome type 5 (disorder)
        "1208864004",  # Long QT syndrome type 2 (disorder)
        "1208866002",  # Long QT syndrome type 3 (disorder)
        "1208868001",  # Long QT syndrome type 12 (disorder)
        "1208870005",  # Long QT syndrome type 13 (disorder)
        "1208881008",  # Long QT syndrome type 11 (disorder)
        "1208884000",  # Long QT syndrome type 10 (disorder)
        "1208909006",  # Long QT syndrome type 6 (disorder)
        "1208911002",  # Long QT syndrome type 9 (disorder)
        "1220643007",  # Regular wide QRS complex tachycardia (disorder)
        "1222539006",  # Three consecutive ventricular premature complexes (disorder)
        "1230096008",  # Timothy syndrome (disorder)
        "1230097004",  # Atypical Timothy syndrome (disorder)
        "1332373008",  # Agonal ventricular rhythm (disorder)
        "13395001",  # Sinoatrial arrest with nodal/ventricular escape (disorder)
        "13640000",  # Fusion beats (disorder)
        "15964901000119107",  # Atypical atrial flutter (disorder)
        "16415081000119104",  # Supraventricular tachycardia following acute myocardial infarction (disorder)
        "17366009",  # Atrial arrhythmia (disorder)
        "17869006",  # Anomalous atrioventricular excitation (disorder)
        "184004",  # Withdrawal arrhythmia (disorder)
        "195060002",  # Ventricular pre-excitation (disorder)
        "195069001",  # Paroxysmal atrial tachycardia (disorder)
        "195071001",  # Paroxysmal junctional tachycardia (disorder)
        "195072008",  # Paroxysmal nodal tachycardia (disorder)
        "195083004",  # Ventricular fibrillation and flutter (disorder)
        "20852007",  # Romano-Ward syndrome (disorder)
        "233891009",  # Sinoatrial node tachycardia (disorder)
        "233892002",  # Ectopic atrial tachycardia (disorder)
        "233893007",  # Re-entrant atrial tachycardia (disorder)
        "233894001",  # Incessant atrial tachycardia (disorder)
        "233895000",  # Ectopic atrioventricular node tachycardia (disorder)
        "233904005",  # Permanent junctional reciprocating tachycardia (disorder)
        "233910005",  # Lone atrial fibrillation (disorder)
        "233911009",  # Non-rheumatic atrial fibrillation (disorder)
        "233913007",  # Familial sick sinus syndrome (disorder)
        "233915000",  # Paroxysmal familial ventricular fibrillation (disorder)
        "233922008",  # Concealed accessory pathway (disorder)
        "233923003",  # Unidirectional retrograde accessory pathway (disorder)
        "234172002",  # Electromechanical dissociation (disorder)
        "251092003",  # Sinus arrest with atrial escape (disorder)
        "251093008",  # Sinus arrest with junctional escape (disorder)
        "251094002",  # Sinus arrest with ventricular escape (disorder)
        "251152003",  # Marked sinus arrhythmia (disorder)
        "251161003",  # Slow ventricular response (disorder)
        "251162005",  # Atrio-ventricular-junctional (nodal) bradycardia (disorder)
        "251163000",  # Atrio-ventricular junctional (nodal) arrest (disorder)
        "251164006",  # Junctional premature complex (disorder)
        "251165007",  # Atrioventricular junctional (nodal) tachycardia (disorder)
        "251167004",  # Aberrant premature complexes (disorder)
        "251168009",  # Supraventricular bigeminy (disorder)
        "251170000",  # Blocked premature atrial contraction (disorder)
        "251171001",  # Multiple atrial premature complexes (disorder)
        "251172008",  # Run of atrial premature complexes (disorder)
        "251173003",  # Atrial bigeminy (disorder)
        "251174009",  # Atrial trigeminy (disorder)
        "251175005",  # Ventricular premature complex (disorder)
        "251176006",  # Multiple premature ventricular complexes (disorder)
        "251177002",  # Run of ventricular premature complexes (disorder)
        "251179004",  # Multiple ventricular interpolated complexes (disorder)
        "251180001",  # Ventricular trigeminy (disorder)
        "251181002",  # Ventricular quadrigeminy (disorder)
        "251182009",  # Paired ventricular premature complexes (disorder)
        "251183004",  # Aberrantly conducted complex (disorder)
        "251186007",  # Ventricular escape complex (disorder)
        "251187003",  # Atrial escape complex (disorder)
        "251188008",  # Atrial parasystole (disorder)
        "253528005",  # Arrhythmogenic right ventricular dysplasia (disorder)
        "26950008",  # Chronic ectopic atrial tachycardia (disorder)
        "27337007",  # Unifocal premature ventricular complexes (disorder)
        "276796006",  # Atrial tachycardia (disorder)
        "284470004",  # Premature atrial contraction (disorder)
        "287057009",  # Atrial premature complex (disorder)
        "29320008",  # Ectopic rhythm (disorder)
        "300996004",  # Controlled atrial fibrillation (disorder)
        "309809007",  # Electromechanical dissociation with successful resuscitation (disorder)
        "33413000",  # Ectopic beats (disorder)
        "36083008",  # Sick sinus syndrome (disorder)
        "373905003",  # Jervell and Lange-Nielsen syndrome (disorder)
        "38274001",  # Interpolated ventricular premature complexes (disorder)
        "39260000",  # Nonparoxysmal atrioventricular nodal tachycardia (disorder)
        "39357005",  # Paroxysmal atrial tachycardia with block (disorder)
        "40593004",  # Fibrillation (disorder)
        "406461004",  # Ectopic atrial beats (disorder)
        "418341009",  # Atrioventricular conduction disorder (disorder)
        "418818005",  # Brugada syndrome (disorder)
        "419671004",  # Catecholaminergic polymorphic ventricular tachycardia (disorder)
        "419752005",  # Sinoatrial nodal reentrant tachycardia (disorder)
        "421869004",  # Bradyarrhythmia (disorder)
        "422348008",  # Andersen Tawil syndrome (disorder)
        "425615007",  # Chronic atrial flutter (disorder)
        "426749004",  # Chronic atrial fibrillation (disorder)
        "427665004",  # Paroxysmal atrial flutter (disorder)
        "429243003",  # Sustained ventricular fibrillation (disorder)
        "440059007",  # Persistent atrial fibrillation (disorder)
        "44103008",  # Ventricular arrhythmia (disorder)
        "442559009",  # Bundle branch reentrant ventricular tachycardia (disorder)
        "442917000",  # Familial long QT syndrome (disorder)
        "444605001",  # Symptomatic sinus bradycardia (disorder)
        "44602002",  # Persistent sinus bradycardia (disorder)
        "44808001",  # Conduction disorder of the heart (disorder)
        "450919004",  # Atrial standstill (disorder)
        "47830009",  # Junctional escape beats (disorder)
        "49044005",  # Severe sinus bradycardia (disorder)
        "49260003",  # Idioventricular rhythm (disorder)
        "49436004",  # Atrial fibrillation (disorder)
        "49710005",  # Sinus bradycardia (disorder)
        "49982000",  # Multifocal atrial tachycardia (disorder)
        "50799005",  # Atrioventricular dissociation (disorder)
        "5370000",  # Atrial flutter (disorder)
        "55475008",  # Lown-Ganong-Levine syndrome (disorder)
        "5609005",  # Sinus arrest (disorder)
        "59272004",  # Ventricular parasystole (disorder)
        "60423000",  # Sinus node dysfunction (disorder)
        "61277005",  # Accelerated idioventricular rhythm (disorder)
        "63232000",  # Multifocal premature beats (disorder)
        "63593006",  # Supraventricular premature beats (disorder)
        "65778007",  # Sinoatrial block (disorder)
        "6624005",  # Ventricular tachyarrhythmia (disorder)
        "69730002",  # Idiojunctional tachycardia (disorder)
        "698272007",  # Short QT syndrome (disorder)
        "699256006",  # Timothy syndrome type 1 (disorder)
        "706923002",  # Longstanding persistent atrial fibrillation (disorder)
        "710878005",  # Mahaim fibre tachycardia (disorder)
        "715395008",  # Familial atrial fibrillation (disorder)
        "715560009",  # Idiopathic neonatal atrial flutter (disorder)
        "715865008",  # Familial isolated arrhythmogenic right ventricular dysplasia (disorder)
        "71792006",  # Nodal rhythm disorder (disorder)
        "71908006",  # Ventricular fibrillation (disorder)
        "719907006",  # Timothy syndrome type 2 (disorder)
        "720448006",  # Typical atrial flutter (disorder)
        "723866006",  # Idiopathic ventricular fibrillation not Brugada type (disorder)
        "72654001",  # Supraventricular arrhythmia (disorder)
        "735682000",  # Macro re-entrant atrial tachycardia (disorder)
        "735684004",  # Non-cavotricuspid isthmus dependent atrial tachycardia (disorder)
        "74390002",  # Wolff-Parkinson-White pattern (disorder)
        "74615001",  # Tachycardia-bradycardia (disorder)
        "75532003",  # Ventricular escape beat (disorder)
        "762247006",  # Preexcited atrial fibrillation (disorder)
        "762534000",  # Supraventricular bradyarrhythmia (disorder)
        "770784003",  # Sinoatrial node dysfunction and deafness (disorder)
        "789039008",  # Scar mediated macro re-entrant atrial tachycardia (disorder)
        "81681009",  # Junctional premature beats (disorder)
        "81898007",  # Ventricular escape rhythm (disorder)
        "871686004",  # Multiple atrial ectopics (disorder)
        "871832005",  # Multifocal atrial ectopics (disorder)
        "88412007",  # Atrio-ventricular node arrhythmia (disorder)
    }

class Asthma(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of diagnoses for asthma.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of asthma.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Asthma"
    OID = "2.16.840.1.113883.3.526.3.362"
    DEFINITION_VERSION = "20200306"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "J4520",  # Mild intermittent asthma, uncomplicated
        "J4521",  # Mild intermittent asthma with (acute) exacerbation
        "J4522",  # Mild intermittent asthma with status asthmaticus
        "J4530",  # Mild persistent asthma, uncomplicated
        "J4531",  # Mild persistent asthma with (acute) exacerbation
        "J4532",  # Mild persistent asthma with status asthmaticus
        "J4540",  # Moderate persistent asthma, uncomplicated
        "J4541",  # Moderate persistent asthma with (acute) exacerbation
        "J4542",  # Moderate persistent asthma with status asthmaticus
        "J4550",  # Severe persistent asthma, uncomplicated
        "J4551",  # Severe persistent asthma with (acute) exacerbation
        "J4552",  # Severe persistent asthma with status asthmaticus
        "J45901",  # Unspecified asthma with (acute) exacerbation
        "J45902",  # Unspecified asthma with status asthmaticus
        "J45909",  # Unspecified asthma, uncomplicated
        "J45990",  # Exercise induced bronchospasm
        "J45991",  # Cough variant asthma
        "J45998",  # Other asthma
    }

    SNOMEDCT = {
        "10674791000119101",  # Acute exacerbation of intermittent allergic asthma (disorder)
        "10676271000119104",  # Acute exacerbation of moderate persistent allergic asthma (disorder)
        "11641008",  # Millers' asthma (disorder)
        "12428000",  # Intrinsic asthma without status asthmaticus (disorder)
        "13151001",  # Flax-dressers' disease (disorder)
        "195949008",  # Chronic asthmatic bronchitis (disorder)
        "195967001",  # Asthma (disorder)
        "195977004",  # Mixed asthma (disorder)
        "225057002",  # Brittle asthma (disorder)
        "233672007",  # Byssinosis grade 3 (disorder)
        "233678006",  # Childhood asthma (disorder)
        "233679003",  # Late onset asthma (disorder)
        "233683003",  # Hay fever with asthma (disorder)
        "233688007",  # Sulfite-induced asthma (disorder)
        "266361008",  # Non-allergic asthma (disorder)
        "281239006",  # Exacerbation of asthma (disorder)
        "31387002",  # Exercise-induced asthma (disorder)
        "370218001",  # Mild asthma (disorder)
        "370219009",  # Moderate asthma (disorder)
        "370220003",  # Occasional asthma (disorder)
        "370221004",  # Severe asthma (disorder)
        "389145006",  # Allergic asthma (disorder)
        "401000119107",  # Asthma with irreversible airway obstruction (disorder)
        "405944004",  # Asthmatic bronchitis (disorder)
        "407674008",  # Aspirin-induced asthma (disorder)
        "409663006",  # Cough variant asthma (disorder)
        "423889005",  # Non-immunoglobulin E mediated allergic asthma (disorder)
        "424643009",  # Immunoglobulin E-mediated allergic asthma (disorder)
        "425969006",  # Exacerbation of intermittent asthma (disorder)
        "426656000",  # Severe persistent asthma (disorder)
        "426979002",  # Mild persistent asthma (disorder)
        "427295004",  # Moderate persistent asthma (disorder)
        "427603009",  # Intermittent asthma (disorder)
        "427679007",  # Mild intermittent asthma (disorder)
        "442025000",  # Acute exacerbation of chronic asthmatic bronchitis (disorder)
        "55570000",  # Asthma without status asthmaticus (disorder)
        "56968009",  # Asthma caused by wood dust (disorder)
        "59786004",  # Weavers' cough (disorder)
        "63088003",  # Allergic asthma without status asthmaticus (disorder)
        "703953004",  # Allergic asthma caused by Dermatophagoides pteronyssinus (disorder)
        "703954005",  # Allergic asthma caused by Dermatophagoides farinae (disorder)
        "707445000",  # Exacerbation of mild persistent asthma (disorder)
        "707446004",  # Exacerbation of moderate persistent asthma (disorder)
        "707447008",  # Exacerbation of severe persistent asthma (disorder)
        "708090002",  # Acute severe exacerbation of asthma (disorder)
        "708093000",  # Acute exacerbation of immunoglobulin E-mediated allergic asthma (disorder)
        "708094006",  # Acute exacerbation of intrinsic asthma (disorder)
        "708095007",  # Acute severe exacerbation of immunoglobin E-mediated allergic asthma (disorder)
        "708096008",  # Acute severe exacerbation of intrinsic asthma (disorder)
        "735588005",  # Uncomplicated allergic asthma (disorder)
        "762521001",  # Exacerbation of allergic asthma (disorder)
        "782520007",  # Exacerbation of allergic asthma due to infection (disorder)
        "829976001",  # Thunderstorm asthma (disorder)
        "85761009",  # Byssinosis (disorder)
        "92807009",  # Chemical-induced asthma (disorder)
        "93432008",  # Drug-induced asthma (disorder)
    }

class AtrioventricularBlock(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of atrioventricular block.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of atrioventricular block.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Atrioventricular Block"
    OID = "2.16.840.1.113883.3.526.3.367"
    DEFINITION_VERSION = "20210218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "I440",  # Atrioventricular block, first degree
        "I441",  # Atrioventricular block, second degree
        "I442",  # Atrioventricular block, complete
        "I4430",  # Unspecified atrioventricular block
        "I4439",  # Other atrioventricular block
    }

    SNOMEDCT = {
        "102451000119107",  # Complete atrioventricular block as complication of atrioventricular nodal ablation (disorder)
        "195039008",  # Partial atrioventricular block (disorder)
        "195042002",  # Second degree atrioventricular block (disorder)
        "204383001",  # Congenital complete atrioventricular heart block (disorder)
        "204384007",  # Congenital incomplete atrioventricular heart block (disorder)
        "233917008",  # Atrioventricular block (disorder)
        "233918003",  # Postoperative complete heart block (disorder)
        "251114004",  # Intermittent second degree atrioventricular block (disorder)
        "270492004",  # First degree atrioventricular block (disorder)
        "27885002",  # Complete atrioventricular block (disorder)
        "28189009",  # Mobitz type II atrioventricular block (disorder)
        "283645003",  # Lev's syndrome (disorder)
        "284941000119107",  # High degree second degree atrioventricular block (disorder)
        "284951000119109",  # Postoperative atrioventricular block (disorder)
        "472809000",  # Atrioventricular block due to endocarditis (disorder)
        "54016002",  # Mobitz type I incomplete atrioventricular block (disorder)
        "733125004",  # Acquired complete atrioventricular block (disorder)
        "77221000",  # Incomplete atrioventricular block with atrioventricular response (disorder)
    }

class Bradycardia(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of bradycardia.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of bradycardia.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Bradycardia"
    OID = "2.16.840.1.113883.3.526.3.412"
    DEFINITION_VERSION = "20200306"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "I495",  # Sick sinus syndrome
        "I498",  # Other specified cardiac arrhythmias
        "R001",  # Bradycardia, unspecified
    }

    SNOMEDCT = {
        "1142110005",  # Sinus bradycardia caused by drug (disorder)
        "1142111009",  # Sinus bradycardia due to metabolic disease (disorder)
        "251162005",  # Atrio-ventricular-junctional (nodal) bradycardia (disorder)
        "29894000",  # Vagal autonomic bradycardia (disorder)
        "397841007",  # Drug-induced bradycardia (disorder)
        "444605001",  # Symptomatic sinus bradycardia (disorder)
        "44602002",  # Persistent sinus bradycardia (disorder)
        "49044005",  # Severe sinus bradycardia (disorder)
        "49710005",  # Sinus bradycardia (disorder)
    }

class CardiacPacerInSitu(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of cardiac pacer in situ.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of cardiac pacer in situ.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Cardiac Pacer in Situ"
    OID = "2.16.840.1.113883.3.526.3.368"
    DEFINITION_VERSION = "20210218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "Z950",  # Presence of cardiac pacemaker
        "Z95810",  # Presence of automatic (implantable) cardiac defibrillator
    }

    SNOMEDCT = {
        "441509002",  # Cardiac pacemaker in situ (finding)
        "441769002",  # Cardiac defibrillator in situ (finding)
        "443325000",  # Automatic implantable cardiac defibrillator in situ (finding)
        "703396000",  # Combination internal cardiac defibrillator and pacemaker in situ (finding)
        "86041000119107",  # Normally functioning cardiac pacemaker in situ (finding)
    }

class Hypotension(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of hypotension.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of hypotension.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Hypotension"
    OID = "2.16.840.1.113883.3.526.3.370"
    DEFINITION_VERSION = "20200306"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "I950",  # Idiopathic hypotension
        "I951",  # Orthostatic hypotension
        "I952",  # Hypotension due to drugs
        "I953",  # Hypotension of hemodialysis
        "I9581",  # Postprocedural hypotension
        "I9589",  # Other hypotension
        "I959",  # Hypotension, unspecified
        "O2650",  # Maternal hypotension syndrome, unspecified trimester
        "O2651",  # Maternal hypotension syndrome, first trimester
        "O2652",  # Maternal hypotension syndrome, second trimester
        "O2653",  # Maternal hypotension syndrome, third trimester
    }

    SNOMEDCT = {
        "195506001",  # Idiopathic hypotension (disorder)
        "200113008",  # Maternal hypotension syndrome with antenatal problem (disorder)
        "200114002",  # Maternal hypotension syndrome with postnatal problem (disorder)
        "230664009",  # Sympathotonic orthostatic hypotension (disorder)
        "234171009",  # Drug-induced hypotension (disorder)
        "286963007",  # Idiopathic chronic hypotension (disorder)
        "371073003",  # Postural orthostatic tachycardia syndrome (disorder)
        "408667000",  # Hemodialysis-associated hypotension (disorder)
        "408668005",  # Iatrogenic hypotension (disorder)
        "429561008",  # Exertional hypotension (disorder)
        "45007003",  # Low blood pressure (disorder)
        "61933008",  # Hyperadrenergic postural hypotension (disorder)
        "70247006",  # Hypoadrenergic postural hypotension (disorder)
        "75181005",  # Chronic orthostatic hypotension (disorder)
        "77545000",  # Chronic hypotension (disorder)
        "88887003",  # Maternal hypotension syndrome (disorder)
    }

class IntoleranceToBetaBlockerTherapy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of an intolerance to beta blocker therapy.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of an intolerance to beta blocker therapy.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Intolerance to Beta Blocker Therapy"
    OID = "2.16.840.1.113883.3.526.3.1178"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    SNOMEDCT = {
        "292419005",  # Beta-adrenoceptor blocking drug adverse reaction (disorder)
        "292420004",  # Cardioselective beta-blocker adverse reaction (disorder)
        "292421000",  # Acebutolol adverse reaction (disorder)
        "292424008",  # Betaxolol adverse reaction (disorder)
        "292425009",  # Bisoprolol adverse reaction (disorder)
        "292426005",  # Celiprolol adverse reaction (disorder)
        "292427001",  # Esmolol adverse reaction (disorder)
        "292428006",  # Metoprolol adverse reaction (disorder)
        "292430008",  # Nadolol adverse reaction (disorder)
        "292431007",  # Pindolol adverse reaction (disorder)
        "292432000",  # Carvedilol adverse reaction (disorder)
        "292433005",  # Metipranolol adverse reaction (disorder)
        "292434004",  # Carteolol adverse reaction (disorder)
        "292435003",  # Labetalol adverse reaction (disorder)
        "292436002",  # Levobunolol adverse reaction (disorder)
        "292437006",  # Oxprenolol adverse reaction (disorder)
        "292438001",  # Penbutolol adverse reaction (disorder)
        "292439009",  # Practolol adverse reaction (disorder)
        "292440006",  # Propranolol adverse reaction (disorder)
        "292441005",  # Sotalol adverse reaction (disorder)
        "292442003",  # Timolol adverse reaction (disorder)
        "418370000",  # Atenolol adverse reaction (disorder)
        "720665007",  # Adverse reaction caused by metoprolol succinate (disorder)
        "720666008",  # Adverse reaction caused by metoprolol tartrate (disorder)
        "772020009",  # Adverse reaction caused by nebivolol (disorder)
    }

class CoronaryArteryDiseaseNoMi(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a diagnosis of coronary artery disease without myocardial infarction.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of coronary artery disease without myocardial infarction.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Coronary Artery Disease No MI"
    OID = "2.16.840.1.113883.3.526.3.369"
    DEFINITION_VERSION = "20200306"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "I200",  # Unstable angina
        "I201",  # Angina pectoris with documented spasm
        "I202",  # Refractory angina pectoris
        "I2081",  # Angina pectoris with coronary microvascular dysfunction
        "I209",  # Angina pectoris, unspecified
        "I240",  # Acute coronary thrombosis not resulting in myocardial infarction
        "I2489",  # Other forms of acute ischemic heart disease
        "I249",  # Acute ischemic heart disease, unspecified
        "I2510",  # Atherosclerotic heart disease of native coronary artery without angina pectoris
        "I25110",  # Atherosclerotic heart disease of native coronary artery with unstable angina pectoris
        "I25111",  # Atherosclerotic heart disease of native coronary artery with angina pectoris with documented spasm
        "I25112",  # Atherosclerotic heart disease of native coronary artery with refractory angina pectoris
        "I25118",  # Atherosclerotic heart disease of native coronary artery with other forms of angina pectoris
        "I25119",  # Atherosclerotic heart disease of native coronary artery with unspecified angina pectoris
        "I255",  # Ischemic cardiomyopathy
        "I256",  # Silent myocardial ischemia
        "I25700",  # Atherosclerosis of coronary artery bypass graft(s), unspecified, with unstable angina pectoris
        "I25701",  # Atherosclerosis of coronary artery bypass graft(s), unspecified, with angina pectoris with documented spasm
        "I25702",  # Atherosclerosis of coronary artery bypass graft(s), unspecified, with refractory angina pectoris
        "I25708",  # Atherosclerosis of coronary artery bypass graft(s), unspecified, with other forms of angina pectoris
        "I25709",  # Atherosclerosis of coronary artery bypass graft(s), unspecified, with unspecified angina pectoris
        "I25710",  # Atherosclerosis of autologous vein coronary artery bypass graft(s) with unstable angina pectoris
        "I25711",  # Atherosclerosis of autologous vein coronary artery bypass graft(s) with angina pectoris with documented spasm
        "I25712",  # Atherosclerosis of autologous vein coronary artery bypass graft(s) with refractory angina pectoris
        "I25718",  # Atherosclerosis of autologous vein coronary artery bypass graft(s) with other forms of angina pectoris
        "I25719",  # Atherosclerosis of autologous vein coronary artery bypass graft(s) with unspecified angina pectoris
        "I25720",  # Atherosclerosis of autologous artery coronary artery bypass graft(s) with unstable angina pectoris
        "I25721",  # Atherosclerosis of autologous artery coronary artery bypass graft(s) with angina pectoris with documented spasm
        "I25722",  # Atherosclerosis of autologous artery coronary artery bypass graft(s) with refractory angina pectoris
        "I25728",  # Atherosclerosis of autologous artery coronary artery bypass graft(s) with other forms of angina pectoris
        "I25729",  # Atherosclerosis of autologous artery coronary artery bypass graft(s) with unspecified angina pectoris
        "I25730",  # Atherosclerosis of nonautologous biological coronary artery bypass graft(s) with unstable angina pectoris
        "I25731",  # Atherosclerosis of nonautologous biological coronary artery bypass graft(s) with angina pectoris with documented spasm
        "I25732",  # Atherosclerosis of nonautologous biological coronary artery bypass graft(s) with refractory angina pectoris
        "I25738",  # Atherosclerosis of nonautologous biological coronary artery bypass graft(s) with other forms of angina pectoris
        "I25739",  # Atherosclerosis of nonautologous biological coronary artery bypass graft(s) with unspecified angina pectoris
        "I25750",  # Atherosclerosis of native coronary artery of transplanted heart with unstable angina
        "I25751",  # Atherosclerosis of native coronary artery of transplanted heart with angina pectoris with documented spasm
        "I25752",  # Atherosclerosis of native coronary artery of transplanted heart with refractory angina pectoris
        "I25758",  # Atherosclerosis of native coronary artery of transplanted heart with other forms of angina pectoris
        "I25759",  # Atherosclerosis of native coronary artery of transplanted heart with unspecified angina pectoris
        "I25760",  # Atherosclerosis of bypass graft of coronary artery of transplanted heart with unstable angina
        "I25761",  # Atherosclerosis of bypass graft of coronary artery of transplanted heart with angina pectoris with documented spasm
        "I25762",  # Atherosclerosis of bypass graft of coronary artery of transplanted heart with refractory angina pectoris
        "I25768",  # Atherosclerosis of bypass graft of coronary artery of transplanted heart with other forms of angina pectoris
        "I25769",  # Atherosclerosis of bypass graft of coronary artery of transplanted heart with unspecified angina pectoris
        "I25790",  # Atherosclerosis of other coronary artery bypass graft(s) with unstable angina pectoris
        "I25791",  # Atherosclerosis of other coronary artery bypass graft(s) with angina pectoris with documented spasm
        "I25792",  # Atherosclerosis of other coronary artery bypass graft(s) with refractory angina pectoris
        "I25798",  # Atherosclerosis of other coronary artery bypass graft(s) with other forms of angina pectoris
        "I25799",  # Atherosclerosis of other coronary artery bypass graft(s) with unspecified angina pectoris
        "I25810",  # Atherosclerosis of coronary artery bypass graft(s) without angina pectoris
        "I25811",  # Atherosclerosis of native coronary artery of transplanted heart without angina pectoris
        "I25812",  # Atherosclerosis of bypass graft of coronary artery of transplanted heart without angina pectoris
        "I2582",  # Chronic total occlusion of coronary artery
        "I2583",  # Coronary atherosclerosis due to lipid rich plaque
        "I2589",  # Other forms of chronic ischemic heart disease
        "I259",  # Chronic ischemic heart disease, unspecified
        "Z951",  # Presence of aortocoronary bypass graft
        "Z955",  # Presence of coronary angioplasty implant and graft
        "Z9861",  # Coronary angioplasty status
    }

    SNOMEDCT = {
        "10365005",  # Right main coronary artery thrombosis (disorder)
        "11018701000119109",  # Coronary arteriosclerosis after percutaneous coronary angioplasty (disorder)
        "123641001",  # Left coronary artery occlusion (disorder)
        "123642008",  # Right coronary artery occlusion (disorder)
        "1254657004",  # Stenosis of proximal portion of right coronary artery (disorder)
        "1254658009",  # Stenosis of mid portion of right coronary artery (disorder)
        "1254659001",  # Stenosis of distal portion of right coronary artery (disorder)
        "1254660006",  # Stenosis of marginal branch of right coronary artery (disorder)
        "1254661005",  # Stenosis of posterior lateral branch of right coronary artery (disorder)
        "1254662003",  # Stenosis of posterior descending branch of right coronary artery (disorder)
        "1254663008",  # Stenosis of distal portion of anterior descending branch of left coronary artery (disorder)
        "1254664002",  # Stenosis of diagonal branch of anterior descending branch of left coronary artery (disorder)
        "1254665001",  # Stenosis of septal branch of anterior descending branch of left coronary artery (disorder)
        "1254666000",  # Stenosis of ostium of right coronary artery (disorder)
        "1255187002",  # Stenosis of obtuse marginal branch of circumflex branch of left coronary artery (disorder)
        "1255188007",  # Stenosis of left posterior lateral branch of circumflex branch of left coronary artery (disorder)
        "1255189004",  # Stenosis of left posterior descending circumflex coronary artery (disorder)
        "1255256006",  # Stenosis of proximal portion of circumflex branch of left coronary artery (disorder)
        "1255257002",  # Stenosis of mid portion of circumflex branch of left coronary artery (disorder)
        "1255258007",  # Stenosis of distal portion of circumflex branch of left coronary artery (disorder)
        "1255264000",  # Stenosis of coronary intermediate artery (disorder)
        "1255265004",  # Stenosis of proximal portion of anterior descending branch of left coronary artery (disorder)
        "1255266003",  # Stenosis of mid portion of anterior descending branch of left coronary artery (disorder)
        "1255569006",  # Stenosis of ostium of left main coronary artery (disorder)
        "1255621004",  # Stenosis of distal segment of left coronary artery main stem (disorder)
        "1255622006",  # Stenosis of mid segment of left coronary artery main stem (disorder)
        "1255623001",  # Stenosis of ostium of anterior descending branch of left coronary artery (disorder)
        "1255624007",  # Stenosis of ostium of circumflex branch of left coronary artery (disorder)
        "1269257006",  # Arteriosclerosis of nonautologous biological coronary artery bypass graft (disorder)
        "1293192009",  # Premature coronary artery disease (disorder)
        "139971000119106",  # Thrombosis of coronary artery bypass graft (disorder)
        "15960061000119102",  # Unstable angina co-occurrent and due to coronary arteriosclerosis (disorder)
        "15960341000119104",  # Unstable angina due to arteriosclerosis of coronary artery bypass graft of transplanted heart (disorder)
        "15960461000119105",  # Unstable angina due to arteriosclerosis of autologous arterial coronary artery bypass graft (disorder)
        "15960541000119107",  # Unstable angina due to arteriosclerosis of autologous vein coronary artery bypass graft (disorder)
        "15960701000119100",  # Angina due to arteriosclerosis of coronary artery bypass graft of transplanted heart (disorder)
        "15960741000119103",  # Preinfarction syndrome due to arteriosclerosis of nonautologous biological coronary artery bypass graft (disorder)
        "16025471000119107",  # Aneurysm of coronary artery due to and following acute febrile mucocutaneous lymph node syndrome (disorder)
        "16754391000119100",  # Stable angina due to coronary arteriosclerosis (disorder)
        "16891151000119103",  # Coronary artery disease due to type 2 diabetes mellitus (disorder)
        "19057007",  # Status anginosus (disorder)
        "194821006",  # Coronary thrombosis not resulting in myocardial infarction (disorder)
        "194823009",  # Acute coronary insufficiency (disorder)
        "194828000",  # Angina (disorder)
        "194842008",  # Single coronary vessel disease (disorder)
        "194843003",  # Double coronary vessel disease (disorder)
        "19830001000004106",  # Thrombosis of left circumflex artery (disorder)
        "204378009",  # Congenital coronary aneurysm (disorder)
        "21470009",  # Syncope anginosa (disorder)
        "233817007",  # Triple vessel disease of the heart (disorder)
        "233819005",  # Stable angina (disorder)
        "233821000",  # New onset angina (disorder)
        "233844002",  # Accelerated coronary artery disease in transplanted heart (disorder)
        "233970002",  # Coronary artery stenosis (disorder)
        "28248000",  # Left anterior descending coronary artery thrombosis (disorder)
        "29899005",  # Coronary artery embolism (disorder)
        "300995000",  # Exercise-induced angina (disorder)
        "314116003",  # Post infarct angina (disorder)
        "315025001",  # Refractory angina (disorder)
        "315348000",  # Asymptomatic coronary heart disease (disorder)
        "35928006",  # Nocturnal angina (disorder)
        "371803003",  # Multi vessel coronary artery disease (disorder)
        "371804009",  # Left main coronary artery disease (disorder)
        "371805005",  # Significant coronary bypass graft disease (disorder)
        "371806006",  # Progressive angina (disorder)
        "371807002",  # Atypical angina (disorder)
        "371808007",  # Recurrent angina status post percutaneous transluminal coronary angioplasty (disorder)
        "371809004",  # Recurrent angina following placement of coronary artery stent (disorder)
        "371810009",  # Recurrent angina status post coronary artery bypass graft (disorder)
        "371811008",  # Recurrent angina status post rotational atherectomy (disorder)
        "371812001",  # Recurrent angina status post directional coronary atherectomy (disorder)
        "398274000",  # Coronary artery thrombosis (disorder)
        "408546009",  # Coronary artery bypass graft occlusion (disorder)
        "41334000",  # Angina, class II (disorder)
        "420006002",  # Obliterative coronary artery disease (disorder)
        "421327009",  # Coronary artery stent thrombosis (disorder)
        "427919004",  # Coronary arteriosclerosis caused by radiation (disorder)
        "429245005",  # Recurrent coronary arteriosclerosis after percutaneous transluminal coronary angioplasty (disorder)
        "429559004",  # Typical angina (disorder)
        "443502000",  # Atherosclerosis of coronary artery (disorder)
        "44761000087104",  # Occlusion of distal portion of circumflex branch of left coronary artery (disorder)
        "44771000087108",  # Occlusion of mid portion of circumflex branch of left coronary artery (disorder)
        "44781000087105",  # Occlusion of obtuse marginal branch of circumflex branch of left coronary artery (disorder)
        "44791000087107",  # Occlusion of posterior lateral branch of circumflex branch of left coronary artery (disorder)
        "44801000087106",  # Occlusion of proximal portion of circumflex branch of left coronary artery (disorder)
        "4557003",  # Preinfarction syndrome (disorder)
        "473362006",  # Congenital stenosis of distal coronary artery (disorder)
        "50570003",  # Aneurysm of coronary vessels (disorder)
        "53741008",  # Coronary arteriosclerosis (disorder)
        "56276002",  # Left main coronary artery thrombosis (disorder)
        "59021001",  # Angina decubitus (disorder)
        "59062007",  # Coronary stricture (disorder)
        "61490001",  # Angina, class I (disorder)
        "62207008",  # Syphilitic ostial coronary disease (disorder)
        "63739005",  # Coronary occlusion (disorder)
        "67682002",  # Coronary artery atheroma (disorder)
        "74218008",  # Coronary artery arising from main pulmonary artery (disorder)
        "75398000",  # Anomalous origin of coronary artery (disorder)
        "791000119109",  # Angina due to type 2 diabetes mellitus (disorder)
        "792842004",  # Coronary arteriosclerosis in artery of transplanted heart (disorder)
        "85284003",  # Angina, class III (disorder)
        "87343002",  # Prinzmetal angina (disorder)
        "876856005",  # Stenosis of right coronary artery (disorder)
        "876857001",  # Stenosis of left coronary artery main stem (disorder)
        "876858006",  # Stenosis of circumflex branch of left coronary artery (disorder)
        "876859003",  # Stenosis of anterior descending branch of left coronary artery (disorder)
        "89323001",  # Angina, class IV (disorder)
        "896690007",  # Occlusion of circumflex branch of left coronary artery (disorder)
        "92517006",  # Calcific coronary arteriosclerosis (disorder)
    }

class InitialMyocardialInfarction(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of diagnoses for initial myocardial infarction.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent an initial diagnosis of myocardial infarction.

    **Exclusion Criteria:** Excludes concepts that represent subsequent myocardial infarction.
    """

    VALUE_SET_NAME = "Initial Myocardial Infarction"
    OID = "2.16.840.1.113883.3.526.3.403"
    DEFINITION_VERSION = "20240125"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "I2101",  # ST elevation (STEMI) myocardial infarction involving left main coronary artery
        "I2102",  # ST elevation (STEMI) myocardial infarction involving left anterior descending coronary artery
        "I2109",  # ST elevation (STEMI) myocardial infarction involving other coronary artery of anterior wall
        "I2111",  # ST elevation (STEMI) myocardial infarction involving right coronary artery
        "I2119",  # ST elevation (STEMI) myocardial infarction involving other coronary artery of inferior wall
        "I2121",  # ST elevation (STEMI) myocardial infarction involving left circumflex coronary artery
        "I2129",  # ST elevation (STEMI) myocardial infarction involving other sites
        "I213",  # ST elevation (STEMI) myocardial infarction of unspecified site
        "I214",  # Non-ST elevation (NSTEMI) myocardial infarction
        "I219",  # Acute myocardial infarction, unspecified
        "I21A9",  # Other myocardial infarction type
        "I21B",  # Myocardial infarction with coronary microvascular dysfunction
    }

    SNOMEDCT = {
        "10273003",  # Acute infarction of papillary muscle (disorder)
        "1163440003",  # Postoperative acute myocardial infarction (disorder)
        "1204151009",  # Acute inferior non-ST segment elevation myocardial infarction of right ventricle (disorder)
        "1204152002",  # Acute inferior non-ST segment elevation myocardial infarction (disorder)
        "1204154001",  # Acute anterior non-ST segment elevation myocardial infarction with right ventricular involvement (disorder)
        "1204155000",  # Acute anterior non-ST segment elevation myocardial infarction (disorder)
        "1204222000",  # Acute non-ST segment elevation myocardial infarction of right ventricle (disorder)
        "129574000",  # Postoperative myocardial infarction (disorder)
        "15713001000119100",  # Acute ST segment elevation myocardial infarction of atrium (disorder)
        "15990001",  # Acute myocardial infarction of posterolateral wall (disorder)
        "194802003",  # True posterior myocardial infarction (disorder)
        "194809007",  # Acute atrial infarction (disorder)
        "194856005",  # Subsequent myocardial infarction (disorder)
        "22298006",  # Myocardial infarction (disorder)
        "233835003",  # Acute widespread myocardial infarction (disorder)
        "233838001",  # Acute posterior myocardial infarction (disorder)
        "233843008",  # Silent myocardial infarction (disorder)
        "30277009",  # Rupture of ventricle due to acute myocardial infarction (disorder)
        "304914007",  # Acute Q wave myocardial infarction (disorder)
        "307140009",  # Acute non-Q wave infarction (disorder)
        "314207007",  # Non-Q wave myocardial infarction (disorder)
        "32574007",  # Past myocardial infarction diagnosed on electrocardiogram AND/OR other special investigation, but currently presenting no symptoms (disorder)
        "394710008",  # First myocardial infarction (disorder)
        "401303003",  # Acute ST segment elevation myocardial infarction (disorder)
        "401314000",  # Acute non-ST segment elevation myocardial infarction (disorder)
        "418044006",  # Myocardial infarction in recovery phase (disorder)
        "42531007",  # Microinfarct of heart (disorder)
        "428196007",  # Mixed myocardial ischemia and infarction (disorder)
        "428752002",  # Recent myocardial infarction (situation)
        "44811000087108",  # Acute ST segment elevation myocardial infarction due to occlusion of distal portion of circumflex branch of left coronary artery (disorder)
        "44821000087100",  # Acute ST segment elevation myocardial infarction due to occlusion of mid portion of circumflex branch of left coronary artery (disorder)
        "44831000087103",  # Acute ST segment elevation myocardial infarction due to occlusion of obtuse marginal branch of circumflex branch of left coronary artery (disorder)
        "44841000087109",  # Acute ST segment elevation myocardial infarction due to occlusion of posterior lateral branch of circumflex branch of left coronary artery (disorder)
        "44851000087107",  # Acute ST segment elevation myocardial infarction due to occlusion of proximal portion of circumflex branch of left coronary artery (disorder)
        "52035003",  # Acute anteroapical myocardial infarction (disorder)
        "54329005",  # Acute myocardial infarction of anterior wall (disorder)
        "57054005",  # Acute myocardial infarction (disorder)
        "58612006",  # Acute myocardial infarction of lateral wall (disorder)
        "62695002",  # Acute anteroseptal myocardial infarction (disorder)
        "65547006",  # Acute myocardial infarction of inferolateral wall (disorder)
        "70211005",  # Acute myocardial infarction of anterolateral wall (disorder)
        "703164000",  # Acute ST segment elevation myocardial infarction of anterior wall (disorder)
        "703165004",  # Acute ST segment elevation myocardial infarction of anterior wall involving right ventricle (disorder)
        "703212004",  # Acute myocardial infarction during procedure (disorder)
        "703213009",  # Acute ST segment elevation myocardial infarction of inferior wall (disorder)
        "703251009",  # Acute myocardial infarction of inferior wall involving right ventricle (disorder)
        "703252002",  # Acute myocardial infarction of anterior wall involving right ventricle (disorder)
        "703253007",  # Acute ST segment elevation myocardial infarction of inferior wall involving right ventricle (disorder)
        "70422006",  # Acute subendocardial infarction (disorder)
        "70998009",  # Acute myocardial infarction of basal inferior segment of left ventricle (disorder)
        "726499301000119105",  # Myocardial infarction due to atherothrombotic coronary artery disease (disorder)
        "73795002",  # Acute myocardial infarction of inferior wall (disorder)
        "76593002",  # Acute myocardial infarction of inferoposterior wall (disorder)
        "79009004",  # Acute myocardial infarction of septum (disorder)
        "836293000",  # Acute myocardial infarction of right ventricle (disorder)
        "836294006",  # Acute myocardial infarction of apex of heart (disorder)
        "836295007",  # Acute myocardial infarction of inferolateral wall with posterior extension (disorder)
        "840309000",  # Acute ST segment elevation myocardial infarction due to occlusion of proximal portion of anterior descending branch of left coronary artery (disorder)
        "840312002",  # Acute ST segment elevation myocardial infarction due to occlusion of mid portion of anterior descending branch of left coronary artery (disorder)
        "840316004",  # Acute ST segment elevation myocardial infarction due to occlusion of distal portion of anterior descending branch of left coronary artery (disorder)
        "840609007",  # Acute ST segment elevation myocardial infarction due to occlusion of anterior descending branch of left coronary artery (disorder)
        "840680009",  # Acute ST segment elevation myocardial infarction due to occlusion of septal branch of anterior descending branch of left coronary artery (disorder)
        "879955009",  # Myocardial infarction with non-obstructive coronary artery (disorder)
        "896689003",  # Acute myocardial infarction due to occlusion of circumflex branch of left coronary artery (disorder)
        "896691006",  # Acute ST segment elevation myocardial infarction due to occlusion of circumflex branch of left coronary artery (disorder)
        "896696001",  # Acute ST segment elevation myocardial infarction of apex of heart (disorder)
        "896697005",  # Acute ST segment elevation myocardial infarction of right ventricle (disorder)
    }

class AcutePharyngitis(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of bacterial or viral acute pharyngitis.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of either bacterial or viral acute pharyngitis.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Acute Pharyngitis"
    OID = "2.16.840.1.113883.3.464.1003.102.12.1011"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "J020",  # Streptococcal pharyngitis
        "J028",  # Acute pharyngitis due to other specified organisms
        "J029",  # Acute pharyngitis, unspecified
    }

    SNOMEDCT = {
        "1532007",  # Viral pharyngitis (disorder)
        "195655000",  # Acute gangrenous pharyngitis (disorder)
        "195656004",  # Acute phlegmonous pharyngitis (disorder)
        "195657008",  # Acute ulcerative pharyngitis (disorder)
        "195658003",  # Acute bacterial pharyngitis (disorder)
        "195659006",  # Acute pneumococcal pharyngitis (disorder)
        "195660001",  # Acute staphylococcal pharyngitis (disorder)
        "195662009",  # Acute viral pharyngitis (disorder)
        "232399005",  # Acute herpes simplex pharyngitis (disorder)
        "232400003",  # Acute herpes zoster pharyngitis (disorder)
        "363746003",  # Acute pharyngitis (disorder)
        "40766000",  # Enteroviral lymphonodular pharyngitis (disorder)
        "43878008",  # Streptococcal sore throat (disorder)
        "58031004",  # Suppurative pharyngitis (disorder)
    }

class AcuteTonsillitis(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of acute tonsillitis.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of either bacterial or viral acute tonsillitis.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Acute Tonsillitis"
    OID = "2.16.840.1.113883.3.464.1003.102.12.1012"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "J0300",  # Acute streptococcal tonsillitis, unspecified
        "J0301",  # Acute recurrent streptococcal tonsillitis
        "J0380",  # Acute tonsillitis due to other specified organisms
        "J0381",  # Acute recurrent tonsillitis due to other specified organisms
        "J0390",  # Acute tonsillitis, unspecified
        "J0391",  # Acute recurrent tonsillitis, unspecified
    }

    SNOMEDCT = {
        "10629231000119109",  # Recurrent acute streptococcal tonsillitis (disorder)
        "10629271000119107",  # Acute tonsillitis caused by Streptococcus (disorder)
        "17741008",  # Acute tonsillitis (disorder)
        "195666007",  # Acute erythematous tonsillitis (disorder)
        "195668008",  # Acute ulcerative tonsillitis (disorder)
        "195669000",  # Acute catarrhal tonsillitis (disorder)
        "195670004",  # Acute gangrenous tonsillitis (disorder)
        "195671000",  # Acute bacterial tonsillitis (disorder)
        "195672007",  # Acute pneumococcal tonsillitis (disorder)
        "195673002",  # Acute staphylococcal tonsillitis (disorder)
        "195676005",  # Acute viral tonsillitis (disorder)
        "195677001",  # Recurrent acute tonsillitis (disorder)
        "302911003",  # Acute lingual tonsillitis (disorder)
    }

class ComorbidConditionsForRespiratoryConditions(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of immunocompromising and chronic respiratory conditions, which increase susceptibility of contracting an acute bacterial infection.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of immunocompromising and chronic respiratory conditions, which increase susceptibility of contracting an acute bacterial infection.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis that is not associated with increased susceptibility of contracting an acute bacterial infection.
    """

    VALUE_SET_NAME = "Comorbid Conditions for Respiratory Conditions"
    OID = "2.16.840.1.113883.3.464.1003.102.12.1025"
    DEFINITION_VERSION = "20220928"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "A150",  # Tuberculosis of lung
        "A154",  # Tuberculosis of intrathoracic lymph nodes
        "A155",  # Tuberculosis of larynx, trachea and bronchus
        "A156",  # Tuberculous pleurisy
        "A157",  # Primary respiratory tuberculosis
        "A158",  # Other respiratory tuberculosis
        "A159",  # Respiratory tuberculosis unspecified
        "A170",  # Tuberculous meningitis
        "A171",  # Meningeal tuberculoma
        "A1781",  # Tuberculoma of brain and spinal cord
        "A1782",  # Tuberculous meningoencephalitis
        "A1783",  # Tuberculous neuritis
        "A1789",  # Other tuberculosis of nervous system
        "A179",  # Tuberculosis of nervous system, unspecified
        "A1801",  # Tuberculosis of spine
        "A1802",  # Tuberculous arthritis of other joints
        "A1803",  # Tuberculosis of other bones
        "A1809",  # Other musculoskeletal tuberculosis
        "A1810",  # Tuberculosis of genitourinary system, unspecified
        "A1811",  # Tuberculosis of kidney and ureter
        "A1812",  # Tuberculosis of bladder
        "A1813",  # Tuberculosis of other urinary organs
        "A1814",  # Tuberculosis of prostate
        "A1815",  # Tuberculosis of other male genital organs
        "A1816",  # Tuberculosis of cervix
        "A1817",  # Tuberculous female pelvic inflammatory disease
        "A1818",  # Tuberculosis of other female genital organs
        "A182",  # Tuberculous peripheral lymphadenopathy
        "A1831",  # Tuberculous peritonitis
        "A1832",  # Tuberculous enteritis
        "A1839",  # Retroperitoneal tuberculosis
        "A184",  # Tuberculosis of skin and subcutaneous tissue
        "A1850",  # Tuberculosis of eye, unspecified
        "A1851",  # Tuberculous episcleritis
        "A1852",  # Tuberculous keratitis
        "A1853",  # Tuberculous chorioretinitis
        "A1854",  # Tuberculous iridocyclitis
        "A1859",  # Other tuberculosis of eye
        "A186",  # Tuberculosis of (inner) (middle) ear
        "A187",  # Tuberculosis of adrenal glands
        "A1881",  # Tuberculosis of thyroid gland
        "A1882",  # Tuberculosis of other endocrine glands
        "A1883",  # Tuberculosis of digestive tract organs, not elsewhere classified
        "A1884",  # Tuberculosis of heart
        "A1885",  # Tuberculosis of spleen
        "A1889",  # Tuberculosis of other sites
        "A190",  # Acute miliary tuberculosis of a single specified site
        "A191",  # Acute miliary tuberculosis of multiple sites
        "A192",  # Acute miliary tuberculosis, unspecified
        "A198",  # Other miliary tuberculosis
        "A199",  # Miliary tuberculosis, unspecified
        "B4481",  # Allergic bronchopulmonary aspergillosis
        "C000",  # Malignant neoplasm of external upper lip
        "C001",  # Malignant neoplasm of external lower lip
        "C002",  # Malignant neoplasm of external lip, unspecified
        "C003",  # Malignant neoplasm of upper lip, inner aspect
        "C004",  # Malignant neoplasm of lower lip, inner aspect
        "C005",  # Malignant neoplasm of lip, unspecified, inner aspect
        "C006",  # Malignant neoplasm of commissure of lip, unspecified
        "C008",  # Malignant neoplasm of overlapping sites of lip
        "C009",  # Malignant neoplasm of lip, unspecified
        "C01",  # Malignant neoplasm of base of tongue
        "C020",  # Malignant neoplasm of dorsal surface of tongue
        "C021",  # Malignant neoplasm of border of tongue
        "C022",  # Malignant neoplasm of ventral surface of tongue
        "C023",  # Malignant neoplasm of anterior two-thirds of tongue, part unspecified
        "C024",  # Malignant neoplasm of lingual tonsil
        "C028",  # Malignant neoplasm of overlapping sites of tongue
        "C029",  # Malignant neoplasm of tongue, unspecified
        "C030",  # Malignant neoplasm of upper gum
        "C031",  # Malignant neoplasm of lower gum
        "C039",  # Malignant neoplasm of gum, unspecified
        "C040",  # Malignant neoplasm of anterior floor of mouth
        "C041",  # Malignant neoplasm of lateral floor of mouth
        "C048",  # Malignant neoplasm of overlapping sites of floor of mouth
        "C049",  # Malignant neoplasm of floor of mouth, unspecified
        "C050",  # Malignant neoplasm of hard palate
        "C051",  # Malignant neoplasm of soft palate
        "C052",  # Malignant neoplasm of uvula
        "C058",  # Malignant neoplasm of overlapping sites of palate
        "C059",  # Malignant neoplasm of palate, unspecified
        "C060",  # Malignant neoplasm of cheek mucosa
        "C061",  # Malignant neoplasm of vestibule of mouth
        "C062",  # Malignant neoplasm of retromolar area
        "C0680",  # Malignant neoplasm of overlapping sites of unspecified parts of mouth
        "C0689",  # Malignant neoplasm of overlapping sites of other parts of mouth
        "C069",  # Malignant neoplasm of mouth, unspecified
        "C07",  # Malignant neoplasm of parotid gland
        "C080",  # Malignant neoplasm of submandibular gland
        "C081",  # Malignant neoplasm of sublingual gland
        "C089",  # Malignant neoplasm of major salivary gland, unspecified
        "C090",  # Malignant neoplasm of tonsillar fossa
        "C091",  # Malignant neoplasm of tonsillar pillar (anterior) (posterior)
        "C098",  # Malignant neoplasm of overlapping sites of tonsil
        "C099",  # Malignant neoplasm of tonsil, unspecified
        "C100",  # Malignant neoplasm of vallecula
        "C101",  # Malignant neoplasm of anterior surface of epiglottis
        "C102",  # Malignant neoplasm of lateral wall of oropharynx
        "C103",  # Malignant neoplasm of posterior wall of oropharynx
        "C104",  # Malignant neoplasm of branchial cleft
        "C108",  # Malignant neoplasm of overlapping sites of oropharynx
        "C109",  # Malignant neoplasm of oropharynx, unspecified
        "C110",  # Malignant neoplasm of superior wall of nasopharynx
        "C111",  # Malignant neoplasm of posterior wall of nasopharynx
        "C112",  # Malignant neoplasm of lateral wall of nasopharynx
        "C113",  # Malignant neoplasm of anterior wall of nasopharynx
        "C118",  # Malignant neoplasm of overlapping sites of nasopharynx
        "C119",  # Malignant neoplasm of nasopharynx, unspecified
        "C12",  # Malignant neoplasm of pyriform sinus
        "C130",  # Malignant neoplasm of postcricoid region
        "C131",  # Malignant neoplasm of aryepiglottic fold, hypopharyngeal aspect
        "C132",  # Malignant neoplasm of posterior wall of hypopharynx
        "C138",  # Malignant neoplasm of overlapping sites of hypopharynx
        "C139",  # Malignant neoplasm of hypopharynx, unspecified
        "C140",  # Malignant neoplasm of pharynx, unspecified
        "C142",  # Malignant neoplasm of Waldeyer's ring
        "C148",  # Malignant neoplasm of overlapping sites of lip, oral cavity and pharynx
        "C153",  # Malignant neoplasm of upper third of esophagus
        "C154",  # Malignant neoplasm of middle third of esophagus
        "C155",  # Malignant neoplasm of lower third of esophagus
        "C158",  # Malignant neoplasm of overlapping sites of esophagus
        "C159",  # Malignant neoplasm of esophagus, unspecified
        "C160",  # Malignant neoplasm of cardia
        "C161",  # Malignant neoplasm of fundus of stomach
        "C162",  # Malignant neoplasm of body of stomach
        "C163",  # Malignant neoplasm of pyloric antrum
        "C164",  # Malignant neoplasm of pylorus
        "C165",  # Malignant neoplasm of lesser curvature of stomach, unspecified
        "C166",  # Malignant neoplasm of greater curvature of stomach, unspecified
        "C168",  # Malignant neoplasm of overlapping sites of stomach
        "C169",  # Malignant neoplasm of stomach, unspecified
        "C170",  # Malignant neoplasm of duodenum
        "C171",  # Malignant neoplasm of jejunum
        "C172",  # Malignant neoplasm of ileum
        "C173",  # Meckel's diverticulum, malignant
        "C178",  # Malignant neoplasm of overlapping sites of small intestine
        "C179",  # Malignant neoplasm of small intestine, unspecified
        "C180",  # Malignant neoplasm of cecum
        "C181",  # Malignant neoplasm of appendix
        "C182",  # Malignant neoplasm of ascending colon
        "C183",  # Malignant neoplasm of hepatic flexure
        "C184",  # Malignant neoplasm of transverse colon
        "C185",  # Malignant neoplasm of splenic flexure
        "C186",  # Malignant neoplasm of descending colon
        "C187",  # Malignant neoplasm of sigmoid colon
        "C188",  # Malignant neoplasm of overlapping sites of colon
        "C189",  # Malignant neoplasm of colon, unspecified
        "C19",  # Malignant neoplasm of rectosigmoid junction
        "C20",  # Malignant neoplasm of rectum
        "C210",  # Malignant neoplasm of anus, unspecified
        "C211",  # Malignant neoplasm of anal canal
        "C212",  # Malignant neoplasm of cloacogenic zone
        "C218",  # Malignant neoplasm of overlapping sites of rectum, anus and anal canal
        "C220",  # Liver cell carcinoma
        "C221",  # Intrahepatic bile duct carcinoma
        "C222",  # Hepatoblastoma
        "C223",  # Angiosarcoma of liver
        "C224",  # Other sarcomas of liver
        "C227",  # Other specified carcinomas of liver
        "C228",  # Malignant neoplasm of liver, primary, unspecified as to type
        "C229",  # Malignant neoplasm of liver, not specified as primary or secondary
        "C23",  # Malignant neoplasm of gallbladder
        "C240",  # Malignant neoplasm of extrahepatic bile duct
        "C241",  # Malignant neoplasm of ampulla of Vater
        "C248",  # Malignant neoplasm of overlapping sites of biliary tract
        "C249",  # Malignant neoplasm of biliary tract, unspecified
        "C250",  # Malignant neoplasm of head of pancreas
        "C251",  # Malignant neoplasm of body of pancreas
        "C252",  # Malignant neoplasm of tail of pancreas
        "C253",  # Malignant neoplasm of pancreatic duct
        "C254",  # Malignant neoplasm of endocrine pancreas
        "C257",  # Malignant neoplasm of other parts of pancreas
        "C258",  # Malignant neoplasm of overlapping sites of pancreas
        "C259",  # Malignant neoplasm of pancreas, unspecified
        "C260",  # Malignant neoplasm of intestinal tract, part unspecified
        "C261",  # Malignant neoplasm of spleen
        "C269",  # Malignant neoplasm of ill-defined sites within the digestive system
        "C300",  # Malignant neoplasm of nasal cavity
        "C301",  # Malignant neoplasm of middle ear
        "C310",  # Malignant neoplasm of maxillary sinus
        "C311",  # Malignant neoplasm of ethmoidal sinus
        "C312",  # Malignant neoplasm of frontal sinus
        "C313",  # Malignant neoplasm of sphenoid sinus
        "C318",  # Malignant neoplasm of overlapping sites of accessory sinuses
        "C319",  # Malignant neoplasm of accessory sinus, unspecified
        "C320",  # Malignant neoplasm of glottis
        "C321",  # Malignant neoplasm of supraglottis
        "C322",  # Malignant neoplasm of subglottis
        "C323",  # Malignant neoplasm of laryngeal cartilage
        "C328",  # Malignant neoplasm of overlapping sites of larynx
        "C329",  # Malignant neoplasm of larynx, unspecified
        "C33",  # Malignant neoplasm of trachea
        "C3400",  # Malignant neoplasm of unspecified main bronchus
        "C3401",  # Malignant neoplasm of right main bronchus
        "C3402",  # Malignant neoplasm of left main bronchus
        "C3410",  # Malignant neoplasm of upper lobe, unspecified bronchus or lung
        "C3411",  # Malignant neoplasm of upper lobe, right bronchus or lung
        "C3412",  # Malignant neoplasm of upper lobe, left bronchus or lung
        "C342",  # Malignant neoplasm of middle lobe, bronchus or lung
        "C3430",  # Malignant neoplasm of lower lobe, unspecified bronchus or lung
        "C3431",  # Malignant neoplasm of lower lobe, right bronchus or lung
        "C3432",  # Malignant neoplasm of lower lobe, left bronchus or lung
        "C3480",  # Malignant neoplasm of overlapping sites of unspecified bronchus and lung
        "C3481",  # Malignant neoplasm of overlapping sites of right bronchus and lung
        "C3482",  # Malignant neoplasm of overlapping sites of left bronchus and lung
        "C3490",  # Malignant neoplasm of unspecified part of unspecified bronchus or lung
        "C3491",  # Malignant neoplasm of unspecified part of right bronchus or lung
        "C3492",  # Malignant neoplasm of unspecified part of left bronchus or lung
        "C37",  # Malignant neoplasm of thymus
        "C380",  # Malignant neoplasm of heart
        "C381",  # Malignant neoplasm of anterior mediastinum
        "C382",  # Malignant neoplasm of posterior mediastinum
        "C383",  # Malignant neoplasm of mediastinum, part unspecified
        "C384",  # Malignant neoplasm of pleura
        "C388",  # Malignant neoplasm of overlapping sites of heart, mediastinum and pleura
        "C390",  # Malignant neoplasm of upper respiratory tract, part unspecified
        "C399",  # Malignant neoplasm of lower respiratory tract, part unspecified
        "C4000",  # Malignant neoplasm of scapula and long bones of unspecified upper limb
        "C4001",  # Malignant neoplasm of scapula and long bones of right upper limb
        "C4002",  # Malignant neoplasm of scapula and long bones of left upper limb
        "C4010",  # Malignant neoplasm of short bones of unspecified upper limb
        "C4011",  # Malignant neoplasm of short bones of right upper limb
        "C4012",  # Malignant neoplasm of short bones of left upper limb
        "C4020",  # Malignant neoplasm of long bones of unspecified lower limb
        "C4021",  # Malignant neoplasm of long bones of right lower limb
        "C4022",  # Malignant neoplasm of long bones of left lower limb
        "C4030",  # Malignant neoplasm of short bones of unspecified lower limb
        "C4031",  # Malignant neoplasm of short bones of right lower limb
        "C4032",  # Malignant neoplasm of short bones of left lower limb
        "C4080",  # Malignant neoplasm of overlapping sites of bone and articular cartilage of unspecified limb
        "C4081",  # Malignant neoplasm of overlapping sites of bone and articular cartilage of right limb
        "C4082",  # Malignant neoplasm of overlapping sites of bone and articular cartilage of left limb
        "C4090",  # Malignant neoplasm of unspecified bones and articular cartilage of unspecified limb
        "C4091",  # Malignant neoplasm of unspecified bones and articular cartilage of right limb
        "C4092",  # Malignant neoplasm of unspecified bones and articular cartilage of left limb
        "C410",  # Malignant neoplasm of bones of skull and face
        "C411",  # Malignant neoplasm of mandible
        "C412",  # Malignant neoplasm of vertebral column
        "C413",  # Malignant neoplasm of ribs, sternum and clavicle
        "C414",  # Malignant neoplasm of pelvic bones, sacrum and coccyx
        "C419",  # Malignant neoplasm of bone and articular cartilage, unspecified
        "C430",  # Malignant melanoma of lip
        "C4310",  # Malignant melanoma of unspecified eyelid, including canthus
        "C4311",  # Malignant melanoma of right eyelid, including canthus
        "C43111",  # Malignant melanoma of right upper eyelid, including canthus
        "C43112",  # Malignant melanoma of right lower eyelid, including canthus
        "C4312",  # Malignant melanoma of left eyelid, including canthus
        "C43121",  # Malignant melanoma of left upper eyelid, including canthus
        "C43122",  # Malignant melanoma of left lower eyelid, including canthus
        "C4320",  # Malignant melanoma of unspecified ear and external auricular canal
        "C4321",  # Malignant melanoma of right ear and external auricular canal
        "C4322",  # Malignant melanoma of left ear and external auricular canal
        "C4330",  # Malignant melanoma of unspecified part of face
        "C4331",  # Malignant melanoma of nose
        "C4339",  # Malignant melanoma of other parts of face
        "C434",  # Malignant melanoma of scalp and neck
        "C4351",  # Malignant melanoma of anal skin
        "C4352",  # Malignant melanoma of skin of breast
        "C4359",  # Malignant melanoma of other part of trunk
        "C4360",  # Malignant melanoma of unspecified upper limb, including shoulder
        "C4361",  # Malignant melanoma of right upper limb, including shoulder
        "C4362",  # Malignant melanoma of left upper limb, including shoulder
        "C4370",  # Malignant melanoma of unspecified lower limb, including hip
        "C4371",  # Malignant melanoma of right lower limb, including hip
        "C4372",  # Malignant melanoma of left lower limb, including hip
        "C438",  # Malignant melanoma of overlapping sites of skin
        "C439",  # Malignant melanoma of skin, unspecified
        "C4400",  # Unspecified malignant neoplasm of skin of lip
        "C4401",  # Basal cell carcinoma of skin of lip
        "C4402",  # Squamous cell carcinoma of skin of lip
        "C4409",  # Other specified malignant neoplasm of skin of lip
        "C44101",  # Unspecified malignant neoplasm of skin of unspecified eyelid, including canthus
        "C44102",  # Unspecified malignant neoplasm of skin of right eyelid, including canthus
        "C441021",  # Unspecified malignant neoplasm of skin of right upper eyelid, including canthus
        "C441022",  # Unspecified malignant neoplasm of skin of right lower eyelid, including canthus
        "C44109",  # Unspecified malignant neoplasm of skin of left eyelid, including canthus
        "C441091",  # Unspecified malignant neoplasm of skin of left upper eyelid, including canthus
        "C441092",  # Unspecified malignant neoplasm of skin of left lower eyelid, including canthus
        "C44111",  # Basal cell carcinoma of skin of unspecified eyelid, including canthus
        "C44112",  # Basal cell carcinoma of skin of right eyelid, including canthus
        "C441121",  # Basal cell carcinoma of skin of right upper eyelid, including canthus
        "C441122",  # Basal cell carcinoma of skin of right lower eyelid, including canthus
        "C44119",  # Basal cell carcinoma of skin of left eyelid, including canthus
        "C441191",  # Basal cell carcinoma of skin of left upper eyelid, including canthus
        "C441192",  # Basal cell carcinoma of skin of left lower eyelid, including canthus
        "C44121",  # Squamous cell carcinoma of skin of unspecified eyelid, including canthus
        "C44122",  # Squamous cell carcinoma of skin of right eyelid, including canthus
        "C441221",  # Squamous cell carcinoma of skin of right upper eyelid, including canthus
        "C441222",  # Squamous cell carcinoma of skin of right lower eyelid, including canthus
        "C44129",  # Squamous cell carcinoma of skin of left eyelid, including canthus
        "C441291",  # Squamous cell carcinoma of skin of left upper eyelid, including canthus
        "C441292",  # Squamous cell carcinoma of skin of left lower eyelid, including canthus
        "C44131",  # Sebaceous cell carcinoma of skin of unspecified eyelid, including canthus
        "C441321",  # Sebaceous cell carcinoma of skin of right upper eyelid, including canthus
        "C441322",  # Sebaceous cell carcinoma of skin of right lower eyelid, including canthus
        "C441391",  # Sebaceous cell carcinoma of skin of left upper eyelid, including canthus
        "C441392",  # Sebaceous cell carcinoma of skin of left lower eyelid, including canthus
        "C44191",  # Other specified malignant neoplasm of skin of unspecified eyelid, including canthus
        "C44192",  # Other specified malignant neoplasm of skin of right eyelid, including canthus
        "C441921",  # Other specified malignant neoplasm of skin of right upper eyelid, including canthus
        "C441922",  # Other specified malignant neoplasm of skin of right lower eyelid, including canthus
        "C44199",  # Other specified malignant neoplasm of skin of left eyelid, including canthus
        "C441991",  # Other specified malignant neoplasm of skin of left upper eyelid, including canthus
        "C441992",  # Other specified malignant neoplasm of skin of left lower eyelid, including canthus
        "C44201",  # Unspecified malignant neoplasm of skin of unspecified ear and external auricular canal
        "C44202",  # Unspecified malignant neoplasm of skin of right ear and external auricular canal
        "C44209",  # Unspecified malignant neoplasm of skin of left ear and external auricular canal
        "C44211",  # Basal cell carcinoma of skin of unspecified ear and external auricular canal
        "C44212",  # Basal cell carcinoma of skin of right ear and external auricular canal
        "C44219",  # Basal cell carcinoma of skin of left ear and external auricular canal
        "C44221",  # Squamous cell carcinoma of skin of unspecified ear and external auricular canal
        "C44222",  # Squamous cell carcinoma of skin of right ear and external auricular canal
        "C44229",  # Squamous cell carcinoma of skin of left ear and external auricular canal
        "C44291",  # Other specified malignant neoplasm of skin of unspecified ear and external auricular canal
        "C44292",  # Other specified malignant neoplasm of skin of right ear and external auricular canal
        "C44299",  # Other specified malignant neoplasm of skin of left ear and external auricular canal
        "C44300",  # Unspecified malignant neoplasm of skin of unspecified part of face
        "C44301",  # Unspecified malignant neoplasm of skin of nose
        "C44309",  # Unspecified malignant neoplasm of skin of other parts of face
        "C44310",  # Basal cell carcinoma of skin of unspecified parts of face
        "C44311",  # Basal cell carcinoma of skin of nose
        "C44319",  # Basal cell carcinoma of skin of other parts of face
        "C44320",  # Squamous cell carcinoma of skin of unspecified parts of face
        "C44321",  # Squamous cell carcinoma of skin of nose
        "C44329",  # Squamous cell carcinoma of skin of other parts of face
        "C44390",  # Other specified malignant neoplasm of skin of unspecified parts of face
        "C44391",  # Other specified malignant neoplasm of skin of nose
        "C44399",  # Other specified malignant neoplasm of skin of other parts of face
        "C4440",  # Unspecified malignant neoplasm of skin of scalp and neck
        "C4441",  # Basal cell carcinoma of skin of scalp and neck
        "C4442",  # Squamous cell carcinoma of skin of scalp and neck
        "C4449",  # Other specified malignant neoplasm of skin of scalp and neck
        "C44500",  # Unspecified malignant neoplasm of anal skin
        "C44501",  # Unspecified malignant neoplasm of skin of breast
        "C44509",  # Unspecified malignant neoplasm of skin of other part of trunk
        "C44510",  # Basal cell carcinoma of anal skin
        "C44511",  # Basal cell carcinoma of skin of breast
        "C44519",  # Basal cell carcinoma of skin of other part of trunk
        "C44520",  # Squamous cell carcinoma of anal skin
        "C44521",  # Squamous cell carcinoma of skin of breast
        "C44529",  # Squamous cell carcinoma of skin of other part of trunk
        "C44590",  # Other specified malignant neoplasm of anal skin
        "C44591",  # Other specified malignant neoplasm of skin of breast
        "C44599",  # Other specified malignant neoplasm of skin of other part of trunk
        "C44601",  # Unspecified malignant neoplasm of skin of unspecified upper limb, including shoulder
        "C44602",  # Unspecified malignant neoplasm of skin of right upper limb, including shoulder
        "C44609",  # Unspecified malignant neoplasm of skin of left upper limb, including shoulder
        "C44611",  # Basal cell carcinoma of skin of unspecified upper limb, including shoulder
        "C44612",  # Basal cell carcinoma of skin of right upper limb, including shoulder
        "C44619",  # Basal cell carcinoma of skin of left upper limb, including shoulder
        "C44621",  # Squamous cell carcinoma of skin of unspecified upper limb, including shoulder
        "C44622",  # Squamous cell carcinoma of skin of right upper limb, including shoulder
        "C44629",  # Squamous cell carcinoma of skin of left upper limb, including shoulder
        "C44691",  # Other specified malignant neoplasm of skin of unspecified upper limb, including shoulder
        "C44692",  # Other specified malignant neoplasm of skin of right upper limb, including shoulder
        "C44699",  # Other specified malignant neoplasm of skin of left upper limb, including shoulder
        "C44701",  # Unspecified malignant neoplasm of skin of unspecified lower limb, including hip
        "C44702",  # Unspecified malignant neoplasm of skin of right lower limb, including hip
        "C44709",  # Unspecified malignant neoplasm of skin of left lower limb, including hip
        "C44711",  # Basal cell carcinoma of skin of unspecified lower limb, including hip
        "C44712",  # Basal cell carcinoma of skin of right lower limb, including hip
        "C44719",  # Basal cell carcinoma of skin of left lower limb, including hip
        "C44721",  # Squamous cell carcinoma of skin of unspecified lower limb, including hip
        "C44722",  # Squamous cell carcinoma of skin of right lower limb, including hip
        "C44729",  # Squamous cell carcinoma of skin of left lower limb, including hip
        "C44791",  # Other specified malignant neoplasm of skin of unspecified lower limb, including hip
        "C44792",  # Other specified malignant neoplasm of skin of right lower limb, including hip
        "C44799",  # Other specified malignant neoplasm of skin of left lower limb, including hip
        "C4480",  # Unspecified malignant neoplasm of overlapping sites of skin
        "C4481",  # Basal cell carcinoma of overlapping sites of skin
        "C4482",  # Squamous cell carcinoma of overlapping sites of skin
        "C4489",  # Other specified malignant neoplasm of overlapping sites of skin
        "C4490",  # Unspecified malignant neoplasm of skin, unspecified
        "C4491",  # Basal cell carcinoma of skin, unspecified
        "C4492",  # Squamous cell carcinoma of skin, unspecified
        "C4499",  # Other specified malignant neoplasm of skin, unspecified
        "C450",  # Mesothelioma of pleura
        "C451",  # Mesothelioma of peritoneum
        "C452",  # Mesothelioma of pericardium
        "C457",  # Mesothelioma of other sites
        "C459",  # Mesothelioma, unspecified
        "C460",  # Kaposi's sarcoma of skin
        "C461",  # Kaposi's sarcoma of soft tissue
        "C462",  # Kaposi's sarcoma of palate
        "C463",  # Kaposi's sarcoma of lymph nodes
        "C464",  # Kaposi's sarcoma of gastrointestinal sites
        "C4650",  # Kaposi's sarcoma of unspecified lung
        "C4651",  # Kaposi's sarcoma of right lung
        "C4652",  # Kaposi's sarcoma of left lung
        "C467",  # Kaposi's sarcoma of other sites
        "C469",  # Kaposi's sarcoma, unspecified
        "C470",  # Malignant neoplasm of peripheral nerves of head, face and neck
        "C4710",  # Malignant neoplasm of peripheral nerves of unspecified upper limb, including shoulder
        "C4711",  # Malignant neoplasm of peripheral nerves of right upper limb, including shoulder
        "C4712",  # Malignant neoplasm of peripheral nerves of left upper limb, including shoulder
        "C4720",  # Malignant neoplasm of peripheral nerves of unspecified lower limb, including hip
        "C4721",  # Malignant neoplasm of peripheral nerves of right lower limb, including hip
        "C4722",  # Malignant neoplasm of peripheral nerves of left lower limb, including hip
        "C473",  # Malignant neoplasm of peripheral nerves of thorax
        "C474",  # Malignant neoplasm of peripheral nerves of abdomen
        "C475",  # Malignant neoplasm of peripheral nerves of pelvis
        "C476",  # Malignant neoplasm of peripheral nerves of trunk, unspecified
        "C478",  # Malignant neoplasm of overlapping sites of peripheral nerves and autonomic nervous system
        "C479",  # Malignant neoplasm of peripheral nerves and autonomic nervous system, unspecified
        "C480",  # Malignant neoplasm of retroperitoneum
        "C481",  # Malignant neoplasm of specified parts of peritoneum
        "C482",  # Malignant neoplasm of peritoneum, unspecified
        "C488",  # Malignant neoplasm of overlapping sites of retroperitoneum and peritoneum
        "C490",  # Malignant neoplasm of connective and soft tissue of head, face and neck
        "C4910",  # Malignant neoplasm of connective and soft tissue of unspecified upper limb, including shoulder
        "C4911",  # Malignant neoplasm of connective and soft tissue of right upper limb, including shoulder
        "C4912",  # Malignant neoplasm of connective and soft tissue of left upper limb, including shoulder
        "C4920",  # Malignant neoplasm of connective and soft tissue of unspecified lower limb, including hip
        "C4921",  # Malignant neoplasm of connective and soft tissue of right lower limb, including hip
        "C4922",  # Malignant neoplasm of connective and soft tissue of left lower limb, including hip
        "C493",  # Malignant neoplasm of connective and soft tissue of thorax
        "C494",  # Malignant neoplasm of connective and soft tissue of abdomen
        "C495",  # Malignant neoplasm of connective and soft tissue of pelvis
        "C496",  # Malignant neoplasm of connective and soft tissue of trunk, unspecified
        "C498",  # Malignant neoplasm of overlapping sites of connective and soft tissue
        "C499",  # Malignant neoplasm of connective and soft tissue, unspecified
        "C49A0",  # Gastrointestinal stromal tumor, unspecified site
        "C49A1",  # Gastrointestinal stromal tumor of esophagus
        "C49A2",  # Gastrointestinal stromal tumor of stomach
        "C49A3",  # Gastrointestinal stromal tumor of small intestine
        "C49A4",  # Gastrointestinal stromal tumor of large intestine
        "C49A5",  # Gastrointestinal stromal tumor of rectum
        "C49A9",  # Gastrointestinal stromal tumor of other sites
        "C4A0",  # Merkel cell carcinoma of lip
        "C4A10",  # Merkel cell carcinoma of unspecified eyelid, including canthus
        "C4A11",  # Merkel cell carcinoma of right eyelid, including canthus
        "C4A111",  # Merkel cell carcinoma of right upper eyelid, including canthus
        "C4A112",  # Merkel cell carcinoma of right lower eyelid, including canthus
        "C4A12",  # Merkel cell carcinoma of left eyelid, including canthus
        "C4A121",  # Merkel cell carcinoma of left upper eyelid, including canthus
        "C4A122",  # Merkel cell carcinoma of left lower eyelid, including canthus
        "C4A20",  # Merkel cell carcinoma of unspecified ear and external auricular canal
        "C4A21",  # Merkel cell carcinoma of right ear and external auricular canal
        "C4A22",  # Merkel cell carcinoma of left ear and external auricular canal
        "C4A30",  # Merkel cell carcinoma of unspecified part of face
        "C4A31",  # Merkel cell carcinoma of nose
        "C4A39",  # Merkel cell carcinoma of other parts of face
        "C4A4",  # Merkel cell carcinoma of scalp and neck
        "C4A51",  # Merkel cell carcinoma of anal skin
        "C4A52",  # Merkel cell carcinoma of skin of breast
        "C4A59",  # Merkel cell carcinoma of other part of trunk
        "C4A60",  # Merkel cell carcinoma of unspecified upper limb, including shoulder
        "C4A61",  # Merkel cell carcinoma of right upper limb, including shoulder
        "C4A62",  # Merkel cell carcinoma of left upper limb, including shoulder
        "C4A70",  # Merkel cell carcinoma of unspecified lower limb, including hip
        "C4A71",  # Merkel cell carcinoma of right lower limb, including hip
        "C4A72",  # Merkel cell carcinoma of left lower limb, including hip
        "C4A8",  # Merkel cell carcinoma of overlapping sites
        "C4A9",  # Merkel cell carcinoma, unspecified
        "C50011",  # Malignant neoplasm of nipple and areola, right female breast
        "C50012",  # Malignant neoplasm of nipple and areola, left female breast
        "C50019",  # Malignant neoplasm of nipple and areola, unspecified female breast
        "C50021",  # Malignant neoplasm of nipple and areola, right male breast
        "C50022",  # Malignant neoplasm of nipple and areola, left male breast
        "C50029",  # Malignant neoplasm of nipple and areola, unspecified male breast
        "C50111",  # Malignant neoplasm of central portion of right female breast
        "C50112",  # Malignant neoplasm of central portion of left female breast
        "C50119",  # Malignant neoplasm of central portion of unspecified female breast
        "C50121",  # Malignant neoplasm of central portion of right male breast
        "C50122",  # Malignant neoplasm of central portion of left male breast
        "C50129",  # Malignant neoplasm of central portion of unspecified male breast
        "C50211",  # Malignant neoplasm of upper-inner quadrant of right female breast
        "C50212",  # Malignant neoplasm of upper-inner quadrant of left female breast
        "C50219",  # Malignant neoplasm of upper-inner quadrant of unspecified female breast
        "C50221",  # Malignant neoplasm of upper-inner quadrant of right male breast
        "C50222",  # Malignant neoplasm of upper-inner quadrant of left male breast
        "C50229",  # Malignant neoplasm of upper-inner quadrant of unspecified male breast
        "C50311",  # Malignant neoplasm of lower-inner quadrant of right female breast
        "C50312",  # Malignant neoplasm of lower-inner quadrant of left female breast
        "C50319",  # Malignant neoplasm of lower-inner quadrant of unspecified female breast
        "C50321",  # Malignant neoplasm of lower-inner quadrant of right male breast
        "C50322",  # Malignant neoplasm of lower-inner quadrant of left male breast
        "C50329",  # Malignant neoplasm of lower-inner quadrant of unspecified male breast
        "C50411",  # Malignant neoplasm of upper-outer quadrant of right female breast
        "C50412",  # Malignant neoplasm of upper-outer quadrant of left female breast
        "C50419",  # Malignant neoplasm of upper-outer quadrant of unspecified female breast
        "C50421",  # Malignant neoplasm of upper-outer quadrant of right male breast
        "C50422",  # Malignant neoplasm of upper-outer quadrant of left male breast
        "C50429",  # Malignant neoplasm of upper-outer quadrant of unspecified male breast
        "C50511",  # Malignant neoplasm of lower-outer quadrant of right female breast
        "C50512",  # Malignant neoplasm of lower-outer quadrant of left female breast
        "C50519",  # Malignant neoplasm of lower-outer quadrant of unspecified female breast
        "C50521",  # Malignant neoplasm of lower-outer quadrant of right male breast
        "C50522",  # Malignant neoplasm of lower-outer quadrant of left male breast
        "C50529",  # Malignant neoplasm of lower-outer quadrant of unspecified male breast
        "C50611",  # Malignant neoplasm of axillary tail of right female breast
        "C50612",  # Malignant neoplasm of axillary tail of left female breast
        "C50619",  # Malignant neoplasm of axillary tail of unspecified female breast
        "C50621",  # Malignant neoplasm of axillary tail of right male breast
        "C50622",  # Malignant neoplasm of axillary tail of left male breast
        "C50629",  # Malignant neoplasm of axillary tail of unspecified male breast
        "C50811",  # Malignant neoplasm of overlapping sites of right female breast
        "C50812",  # Malignant neoplasm of overlapping sites of left female breast
        "C50819",  # Malignant neoplasm of overlapping sites of unspecified female breast
        "C50821",  # Malignant neoplasm of overlapping sites of right male breast
        "C50822",  # Malignant neoplasm of overlapping sites of left male breast
        "C50829",  # Malignant neoplasm of overlapping sites of unspecified male breast
        "C50911",  # Malignant neoplasm of unspecified site of right female breast
        "C50912",  # Malignant neoplasm of unspecified site of left female breast
        "C50919",  # Malignant neoplasm of unspecified site of unspecified female breast
        "C50921",  # Malignant neoplasm of unspecified site of right male breast
        "C50922",  # Malignant neoplasm of unspecified site of left male breast
        "C50929",  # Malignant neoplasm of unspecified site of unspecified male breast
        "C510",  # Malignant neoplasm of labium majus
        "C511",  # Malignant neoplasm of labium minus
        "C512",  # Malignant neoplasm of clitoris
        "C518",  # Malignant neoplasm of overlapping sites of vulva
        "C519",  # Malignant neoplasm of vulva, unspecified
        "C52",  # Malignant neoplasm of vagina
        "C530",  # Malignant neoplasm of endocervix
        "C531",  # Malignant neoplasm of exocervix
        "C538",  # Malignant neoplasm of overlapping sites of cervix uteri
        "C539",  # Malignant neoplasm of cervix uteri, unspecified
        "C540",  # Malignant neoplasm of isthmus uteri
        "C541",  # Malignant neoplasm of endometrium
        "C542",  # Malignant neoplasm of myometrium
        "C543",  # Malignant neoplasm of fundus uteri
        "C548",  # Malignant neoplasm of overlapping sites of corpus uteri
        "C549",  # Malignant neoplasm of corpus uteri, unspecified
        "C55",  # Malignant neoplasm of uterus, part unspecified
        "C561",  # Malignant neoplasm of right ovary
        "C562",  # Malignant neoplasm of left ovary
        "C563",  # Malignant neoplasm of bilateral ovaries
        "C569",  # Malignant neoplasm of unspecified ovary
        "C5700",  # Malignant neoplasm of unspecified fallopian tube
        "C5701",  # Malignant neoplasm of right fallopian tube
        "C5702",  # Malignant neoplasm of left fallopian tube
        "C5710",  # Malignant neoplasm of unspecified broad ligament
        "C5711",  # Malignant neoplasm of right broad ligament
        "C5712",  # Malignant neoplasm of left broad ligament
        "C5720",  # Malignant neoplasm of unspecified round ligament
        "C5721",  # Malignant neoplasm of right round ligament
        "C5722",  # Malignant neoplasm of left round ligament
        "C573",  # Malignant neoplasm of parametrium
        "C574",  # Malignant neoplasm of uterine adnexa, unspecified
        "C577",  # Malignant neoplasm of other specified female genital organs
        "C578",  # Malignant neoplasm of overlapping sites of female genital organs
        "C579",  # Malignant neoplasm of female genital organ, unspecified
        "C58",  # Malignant neoplasm of placenta
        "C600",  # Malignant neoplasm of prepuce
        "C601",  # Malignant neoplasm of glans penis
        "C602",  # Malignant neoplasm of body of penis
        "C608",  # Malignant neoplasm of overlapping sites of penis
        "C609",  # Malignant neoplasm of penis, unspecified
        "C61",  # Malignant neoplasm of prostate
        "C6200",  # Malignant neoplasm of unspecified undescended testis
        "C6201",  # Malignant neoplasm of undescended right testis
        "C6202",  # Malignant neoplasm of undescended left testis
        "C6210",  # Malignant neoplasm of unspecified descended testis
        "C6211",  # Malignant neoplasm of descended right testis
        "C6212",  # Malignant neoplasm of descended left testis
        "C6290",  # Malignant neoplasm of unspecified testis, unspecified whether descended or undescended
        "C6291",  # Malignant neoplasm of right testis, unspecified whether descended or undescended
        "C6292",  # Malignant neoplasm of left testis, unspecified whether descended or undescended
        "C6300",  # Malignant neoplasm of unspecified epididymis
        "C6301",  # Malignant neoplasm of right epididymis
        "C6302",  # Malignant neoplasm of left epididymis
        "C6310",  # Malignant neoplasm of unspecified spermatic cord
        "C6311",  # Malignant neoplasm of right spermatic cord
        "C6312",  # Malignant neoplasm of left spermatic cord
        "C632",  # Malignant neoplasm of scrotum
        "C637",  # Malignant neoplasm of other specified male genital organs
        "C638",  # Malignant neoplasm of overlapping sites of male genital organs
        "C639",  # Malignant neoplasm of male genital organ, unspecified
        "C641",  # Malignant neoplasm of right kidney, except renal pelvis
        "C642",  # Malignant neoplasm of left kidney, except renal pelvis
        "C649",  # Malignant neoplasm of unspecified kidney, except renal pelvis
        "C651",  # Malignant neoplasm of right renal pelvis
        "C652",  # Malignant neoplasm of left renal pelvis
        "C659",  # Malignant neoplasm of unspecified renal pelvis
        "C661",  # Malignant neoplasm of right ureter
        "C662",  # Malignant neoplasm of left ureter
        "C669",  # Malignant neoplasm of unspecified ureter
        "C670",  # Malignant neoplasm of trigone of bladder
        "C671",  # Malignant neoplasm of dome of bladder
        "C672",  # Malignant neoplasm of lateral wall of bladder
        "C673",  # Malignant neoplasm of anterior wall of bladder
        "C674",  # Malignant neoplasm of posterior wall of bladder
        "C675",  # Malignant neoplasm of bladder neck
        "C676",  # Malignant neoplasm of ureteric orifice
        "C677",  # Malignant neoplasm of urachus
        "C678",  # Malignant neoplasm of overlapping sites of bladder
        "C679",  # Malignant neoplasm of bladder, unspecified
        "C680",  # Malignant neoplasm of urethra
        "C681",  # Malignant neoplasm of paraurethral glands
        "C688",  # Malignant neoplasm of overlapping sites of urinary organs
        "C689",  # Malignant neoplasm of urinary organ, unspecified
        "C6900",  # Malignant neoplasm of unspecified conjunctiva
        "C6901",  # Malignant neoplasm of right conjunctiva
        "C6902",  # Malignant neoplasm of left conjunctiva
        "C6910",  # Malignant neoplasm of unspecified cornea
        "C6911",  # Malignant neoplasm of right cornea
        "C6912",  # Malignant neoplasm of left cornea
        "C6920",  # Malignant neoplasm of unspecified retina
        "C6921",  # Malignant neoplasm of right retina
        "C6922",  # Malignant neoplasm of left retina
        "C6930",  # Malignant neoplasm of unspecified choroid
        "C6931",  # Malignant neoplasm of right choroid
        "C6932",  # Malignant neoplasm of left choroid
        "C6940",  # Malignant neoplasm of unspecified ciliary body
        "C6941",  # Malignant neoplasm of right ciliary body
        "C6942",  # Malignant neoplasm of left ciliary body
        "C6950",  # Malignant neoplasm of unspecified lacrimal gland and duct
        "C6951",  # Malignant neoplasm of right lacrimal gland and duct
        "C6952",  # Malignant neoplasm of left lacrimal gland and duct
        "C6960",  # Malignant neoplasm of unspecified orbit
        "C6961",  # Malignant neoplasm of right orbit
        "C6962",  # Malignant neoplasm of left orbit
        "C6980",  # Malignant neoplasm of overlapping sites of unspecified eye and adnexa
        "C6981",  # Malignant neoplasm of overlapping sites of right eye and adnexa
        "C6982",  # Malignant neoplasm of overlapping sites of left eye and adnexa
        "C6990",  # Malignant neoplasm of unspecified site of unspecified eye
        "C6991",  # Malignant neoplasm of unspecified site of right eye
        "C6992",  # Malignant neoplasm of unspecified site of left eye
        "C700",  # Malignant neoplasm of cerebral meninges
        "C701",  # Malignant neoplasm of spinal meninges
        "C709",  # Malignant neoplasm of meninges, unspecified
        "C710",  # Malignant neoplasm of cerebrum, except lobes and ventricles
        "C711",  # Malignant neoplasm of frontal lobe
        "C712",  # Malignant neoplasm of temporal lobe
        "C713",  # Malignant neoplasm of parietal lobe
        "C714",  # Malignant neoplasm of occipital lobe
        "C715",  # Malignant neoplasm of cerebral ventricle
        "C716",  # Malignant neoplasm of cerebellum
        "C717",  # Malignant neoplasm of brain stem
        "C718",  # Malignant neoplasm of overlapping sites of brain
        "C719",  # Malignant neoplasm of brain, unspecified
        "C720",  # Malignant neoplasm of spinal cord
        "C721",  # Malignant neoplasm of cauda equina
        "C7220",  # Malignant neoplasm of unspecified olfactory nerve
        "C7221",  # Malignant neoplasm of right olfactory nerve
        "C7222",  # Malignant neoplasm of left olfactory nerve
        "C7230",  # Malignant neoplasm of unspecified optic nerve
        "C7231",  # Malignant neoplasm of right optic nerve
        "C7232",  # Malignant neoplasm of left optic nerve
        "C7240",  # Malignant neoplasm of unspecified acoustic nerve
        "C7241",  # Malignant neoplasm of right acoustic nerve
        "C7242",  # Malignant neoplasm of left acoustic nerve
        "C7250",  # Malignant neoplasm of unspecified cranial nerve
        "C7259",  # Malignant neoplasm of other cranial nerves
        "C729",  # Malignant neoplasm of central nervous system, unspecified
        "C73",  # Malignant neoplasm of thyroid gland
        "C7400",  # Malignant neoplasm of cortex of unspecified adrenal gland
        "C7401",  # Malignant neoplasm of cortex of right adrenal gland
        "C7402",  # Malignant neoplasm of cortex of left adrenal gland
        "C7410",  # Malignant neoplasm of medulla of unspecified adrenal gland
        "C7411",  # Malignant neoplasm of medulla of right adrenal gland
        "C7412",  # Malignant neoplasm of medulla of left adrenal gland
        "C7490",  # Malignant neoplasm of unspecified part of unspecified adrenal gland
        "C7491",  # Malignant neoplasm of unspecified part of right adrenal gland
        "C7492",  # Malignant neoplasm of unspecified part of left adrenal gland
        "C750",  # Malignant neoplasm of parathyroid gland
        "C751",  # Malignant neoplasm of pituitary gland
        "C752",  # Malignant neoplasm of craniopharyngeal duct
        "C753",  # Malignant neoplasm of pineal gland
        "C754",  # Malignant neoplasm of carotid body
        "C755",  # Malignant neoplasm of aortic body and other paraganglia
        "C758",  # Malignant neoplasm with pluriglandular involvement, unspecified
        "C759",  # Malignant neoplasm of endocrine gland, unspecified
        "C760",  # Malignant neoplasm of head, face and neck
        "C761",  # Malignant neoplasm of thorax
        "C762",  # Malignant neoplasm of abdomen
        "C763",  # Malignant neoplasm of pelvis
        "C7640",  # Malignant neoplasm of unspecified upper limb
        "C7641",  # Malignant neoplasm of right upper limb
        "C7642",  # Malignant neoplasm of left upper limb
        "C7650",  # Malignant neoplasm of unspecified lower limb
        "C7651",  # Malignant neoplasm of right lower limb
        "C7652",  # Malignant neoplasm of left lower limb
        "C768",  # Malignant neoplasm of other specified ill-defined sites
        "C770",  # Secondary and unspecified malignant neoplasm of lymph nodes of head, face and neck
        "C771",  # Secondary and unspecified malignant neoplasm of intrathoracic lymph nodes
        "C772",  # Secondary and unspecified malignant neoplasm of intra-abdominal lymph nodes
        "C773",  # Secondary and unspecified malignant neoplasm of axilla and upper limb lymph nodes
        "C774",  # Secondary and unspecified malignant neoplasm of inguinal and lower limb lymph nodes
        "C775",  # Secondary and unspecified malignant neoplasm of intrapelvic lymph nodes
        "C778",  # Secondary and unspecified malignant neoplasm of lymph nodes of multiple regions
        "C779",  # Secondary and unspecified malignant neoplasm of lymph node, unspecified
        "C7800",  # Secondary malignant neoplasm of unspecified lung
        "C7801",  # Secondary malignant neoplasm of right lung
        "C7802",  # Secondary malignant neoplasm of left lung
        "C781",  # Secondary malignant neoplasm of mediastinum
        "C782",  # Secondary malignant neoplasm of pleura
        "C7830",  # Secondary malignant neoplasm of unspecified respiratory organ
        "C7839",  # Secondary malignant neoplasm of other respiratory organs
        "C784",  # Secondary malignant neoplasm of small intestine
        "C785",  # Secondary malignant neoplasm of large intestine and rectum
        "C786",  # Secondary malignant neoplasm of retroperitoneum and peritoneum
        "C787",  # Secondary malignant neoplasm of liver and intrahepatic bile duct
        "C7880",  # Secondary malignant neoplasm of unspecified digestive organ
        "C7889",  # Secondary malignant neoplasm of other digestive organs
        "C7900",  # Secondary malignant neoplasm of unspecified kidney and renal pelvis
        "C7901",  # Secondary malignant neoplasm of right kidney and renal pelvis
        "C7902",  # Secondary malignant neoplasm of left kidney and renal pelvis
        "C7910",  # Secondary malignant neoplasm of unspecified urinary organs
        "C7911",  # Secondary malignant neoplasm of bladder
        "C7919",  # Secondary malignant neoplasm of other urinary organs
        "C792",  # Secondary malignant neoplasm of skin
        "C7931",  # Secondary malignant neoplasm of brain
        "C7932",  # Secondary malignant neoplasm of cerebral meninges
        "C7940",  # Secondary malignant neoplasm of unspecified part of nervous system
        "C7949",  # Secondary malignant neoplasm of other parts of nervous system
        "C7951",  # Secondary malignant neoplasm of bone
        "C7952",  # Secondary malignant neoplasm of bone marrow
        "C7960",  # Secondary malignant neoplasm of unspecified ovary
        "C7961",  # Secondary malignant neoplasm of right ovary
        "C7962",  # Secondary malignant neoplasm of left ovary
        "C7963",  # Secondary malignant neoplasm of bilateral ovaries
        "C7970",  # Secondary malignant neoplasm of unspecified adrenal gland
        "C7971",  # Secondary malignant neoplasm of right adrenal gland
        "C7972",  # Secondary malignant neoplasm of left adrenal gland
        "C7981",  # Secondary malignant neoplasm of breast
        "C7982",  # Secondary malignant neoplasm of genital organs
        "C7989",  # Secondary malignant neoplasm of other specified sites
        "C799",  # Secondary malignant neoplasm of unspecified site
        "C7A00",  # Malignant carcinoid tumor of unspecified site
        "C7A010",  # Malignant carcinoid tumor of the duodenum
        "C7A011",  # Malignant carcinoid tumor of the jejunum
        "C7A012",  # Malignant carcinoid tumor of the ileum
        "C7A019",  # Malignant carcinoid tumor of the small intestine, unspecified portion
        "C7A020",  # Malignant carcinoid tumor of the appendix
        "C7A021",  # Malignant carcinoid tumor of the cecum
        "C7A022",  # Malignant carcinoid tumor of the ascending colon
        "C7A023",  # Malignant carcinoid tumor of the transverse colon
        "C7A024",  # Malignant carcinoid tumor of the descending colon
        "C7A025",  # Malignant carcinoid tumor of the sigmoid colon
        "C7A026",  # Malignant carcinoid tumor of the rectum
        "C7A029",  # Malignant carcinoid tumor of the large intestine, unspecified portion
        "C7A090",  # Malignant carcinoid tumor of the bronchus and lung
        "C7A091",  # Malignant carcinoid tumor of the thymus
        "C7A092",  # Malignant carcinoid tumor of the stomach
        "C7A093",  # Malignant carcinoid tumor of the kidney
        "C7A094",  # Malignant carcinoid tumor of the foregut, unspecified
        "C7A095",  # Malignant carcinoid tumor of the midgut, unspecified
        "C7A096",  # Malignant carcinoid tumor of the hindgut, unspecified
        "C7A098",  # Malignant carcinoid tumors of other sites
        "C7A1",  # Malignant poorly differentiated neuroendocrine tumors
        "C7A8",  # Other malignant neuroendocrine tumors
        "C7B00",  # Secondary carcinoid tumors, unspecified site
        "C7B01",  # Secondary carcinoid tumors of distant lymph nodes
        "C7B02",  # Secondary carcinoid tumors of liver
        "C7B03",  # Secondary carcinoid tumors of bone
        "C7B04",  # Secondary carcinoid tumors of peritoneum
        "C7B09",  # Secondary carcinoid tumors of other sites
        "C7B1",  # Secondary Merkel cell carcinoma
        "C7B8",  # Other secondary neuroendocrine tumors
        "C800",  # Disseminated malignant neoplasm, unspecified
        "C801",  # Malignant (primary) neoplasm, unspecified
        "C802",  # Malignant neoplasm associated with transplanted organ
        "C8100",  # Nodular lymphocyte predominant Hodgkin lymphoma, unspecified site
        "C8101",  # Nodular lymphocyte predominant Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8102",  # Nodular lymphocyte predominant Hodgkin lymphoma, intrathoracic lymph nodes
        "C8103",  # Nodular lymphocyte predominant Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8104",  # Nodular lymphocyte predominant Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8105",  # Nodular lymphocyte predominant Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8106",  # Nodular lymphocyte predominant Hodgkin lymphoma, intrapelvic lymph nodes
        "C8107",  # Nodular lymphocyte predominant Hodgkin lymphoma, spleen
        "C8108",  # Nodular lymphocyte predominant Hodgkin lymphoma, lymph nodes of multiple sites
        "C8109",  # Nodular lymphocyte predominant Hodgkin lymphoma, extranodal and solid organ sites
        "C8110",  # Nodular sclerosis Hodgkin lymphoma, unspecified site
        "C8111",  # Nodular sclerosis Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8112",  # Nodular sclerosis Hodgkin lymphoma, intrathoracic lymph nodes
        "C8113",  # Nodular sclerosis Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8114",  # Nodular sclerosis Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8115",  # Nodular sclerosis Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8116",  # Nodular sclerosis Hodgkin lymphoma, intrapelvic lymph nodes
        "C8117",  # Nodular sclerosis Hodgkin lymphoma, spleen
        "C8118",  # Nodular sclerosis Hodgkin lymphoma, lymph nodes of multiple sites
        "C8119",  # Nodular sclerosis Hodgkin lymphoma, extranodal and solid organ sites
        "C8120",  # Mixed cellularity Hodgkin lymphoma, unspecified site
        "C8121",  # Mixed cellularity Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8122",  # Mixed cellularity Hodgkin lymphoma, intrathoracic lymph nodes
        "C8123",  # Mixed cellularity Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8124",  # Mixed cellularity Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8125",  # Mixed cellularity Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8126",  # Mixed cellularity Hodgkin lymphoma, intrapelvic lymph nodes
        "C8127",  # Mixed cellularity Hodgkin lymphoma, spleen
        "C8128",  # Mixed cellularity Hodgkin lymphoma, lymph nodes of multiple sites
        "C8129",  # Mixed cellularity Hodgkin lymphoma, extranodal and solid organ sites
        "C8130",  # Lymphocyte depleted Hodgkin lymphoma, unspecified site
        "C8131",  # Lymphocyte depleted Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8132",  # Lymphocyte depleted Hodgkin lymphoma, intrathoracic lymph nodes
        "C8133",  # Lymphocyte depleted Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8134",  # Lymphocyte depleted Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8135",  # Lymphocyte depleted Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8136",  # Lymphocyte depleted Hodgkin lymphoma, intrapelvic lymph nodes
        "C8137",  # Lymphocyte depleted Hodgkin lymphoma, spleen
        "C8138",  # Lymphocyte depleted Hodgkin lymphoma, lymph nodes of multiple sites
        "C8139",  # Lymphocyte depleted Hodgkin lymphoma, extranodal and solid organ sites
        "C8140",  # Lymphocyte-rich Hodgkin lymphoma, unspecified site
        "C8141",  # Lymphocyte-rich Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8142",  # Lymphocyte-rich Hodgkin lymphoma, intrathoracic lymph nodes
        "C8143",  # Lymphocyte-rich Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8144",  # Lymphocyte-rich Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8145",  # Lymphocyte-rich Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8146",  # Lymphocyte-rich Hodgkin lymphoma, intrapelvic lymph nodes
        "C8147",  # Lymphocyte-rich Hodgkin lymphoma, spleen
        "C8148",  # Lymphocyte-rich Hodgkin lymphoma, lymph nodes of multiple sites
        "C8149",  # Lymphocyte-rich Hodgkin lymphoma, extranodal and solid organ sites
        "C8170",  # Other Hodgkin lymphoma, unspecified site
        "C8171",  # Other Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8172",  # Other Hodgkin lymphoma, intrathoracic lymph nodes
        "C8173",  # Other Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8174",  # Other Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8175",  # Other Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8176",  # Other Hodgkin lymphoma, intrapelvic lymph nodes
        "C8177",  # Other Hodgkin lymphoma, spleen
        "C8178",  # Other Hodgkin lymphoma, lymph nodes of multiple sites
        "C8179",  # Other Hodgkin lymphoma, extranodal and solid organ sites
        "C8190",  # Hodgkin lymphoma, unspecified, unspecified site
        "C8191",  # Hodgkin lymphoma, unspecified, lymph nodes of head, face, and neck
        "C8192",  # Hodgkin lymphoma, unspecified, intrathoracic lymph nodes
        "C8193",  # Hodgkin lymphoma, unspecified, intra-abdominal lymph nodes
        "C8194",  # Hodgkin lymphoma, unspecified, lymph nodes of axilla and upper limb
        "C8195",  # Hodgkin lymphoma, unspecified, lymph nodes of inguinal region and lower limb
        "C8196",  # Hodgkin lymphoma, unspecified, intrapelvic lymph nodes
        "C8197",  # Hodgkin lymphoma, unspecified, spleen
        "C8198",  # Hodgkin lymphoma, unspecified, lymph nodes of multiple sites
        "C8199",  # Hodgkin lymphoma, unspecified, extranodal and solid organ sites
        "C8200",  # Follicular lymphoma grade I, unspecified site
        "C8201",  # Follicular lymphoma grade I, lymph nodes of head, face, and neck
        "C8202",  # Follicular lymphoma grade I, intrathoracic lymph nodes
        "C8203",  # Follicular lymphoma grade I, intra-abdominal lymph nodes
        "C8204",  # Follicular lymphoma grade I, lymph nodes of axilla and upper limb
        "C8205",  # Follicular lymphoma grade I, lymph nodes of inguinal region and lower limb
        "C8206",  # Follicular lymphoma grade I, intrapelvic lymph nodes
        "C8207",  # Follicular lymphoma grade I, spleen
        "C8208",  # Follicular lymphoma grade I, lymph nodes of multiple sites
        "C8209",  # Follicular lymphoma grade I, extranodal and solid organ sites
        "C8210",  # Follicular lymphoma grade II, unspecified site
        "C8211",  # Follicular lymphoma grade II, lymph nodes of head, face, and neck
        "C8212",  # Follicular lymphoma grade II, intrathoracic lymph nodes
        "C8213",  # Follicular lymphoma grade II, intra-abdominal lymph nodes
        "C8214",  # Follicular lymphoma grade II, lymph nodes of axilla and upper limb
        "C8215",  # Follicular lymphoma grade II, lymph nodes of inguinal region and lower limb
        "C8216",  # Follicular lymphoma grade II, intrapelvic lymph nodes
        "C8217",  # Follicular lymphoma grade II, spleen
        "C8218",  # Follicular lymphoma grade II, lymph nodes of multiple sites
        "C8219",  # Follicular lymphoma grade II, extranodal and solid organ sites
        "C8220",  # Follicular lymphoma grade III, unspecified, unspecified site
        "C8221",  # Follicular lymphoma grade III, unspecified, lymph nodes of head, face, and neck
        "C8222",  # Follicular lymphoma grade III, unspecified, intrathoracic lymph nodes
        "C8223",  # Follicular lymphoma grade III, unspecified, intra-abdominal lymph nodes
        "C8224",  # Follicular lymphoma grade III, unspecified, lymph nodes of axilla and upper limb
        "C8225",  # Follicular lymphoma grade III, unspecified, lymph nodes of inguinal region and lower limb
        "C8226",  # Follicular lymphoma grade III, unspecified, intrapelvic lymph nodes
        "C8227",  # Follicular lymphoma grade III, unspecified, spleen
        "C8228",  # Follicular lymphoma grade III, unspecified, lymph nodes of multiple sites
        "C8229",  # Follicular lymphoma grade III, unspecified, extranodal and solid organ sites
        "C8230",  # Follicular lymphoma grade IIIa, unspecified site
        "C8231",  # Follicular lymphoma grade IIIa, lymph nodes of head, face, and neck
        "C8232",  # Follicular lymphoma grade IIIa, intrathoracic lymph nodes
        "C8233",  # Follicular lymphoma grade IIIa, intra-abdominal lymph nodes
        "C8234",  # Follicular lymphoma grade IIIa, lymph nodes of axilla and upper limb
        "C8235",  # Follicular lymphoma grade IIIa, lymph nodes of inguinal region and lower limb
        "C8236",  # Follicular lymphoma grade IIIa, intrapelvic lymph nodes
        "C8237",  # Follicular lymphoma grade IIIa, spleen
        "C8238",  # Follicular lymphoma grade IIIa, lymph nodes of multiple sites
        "C8239",  # Follicular lymphoma grade IIIa, extranodal and solid organ sites
        "C8240",  # Follicular lymphoma grade IIIb, unspecified site
        "C8241",  # Follicular lymphoma grade IIIb, lymph nodes of head, face, and neck
        "C8242",  # Follicular lymphoma grade IIIb, intrathoracic lymph nodes
        "C8243",  # Follicular lymphoma grade IIIb, intra-abdominal lymph nodes
        "C8244",  # Follicular lymphoma grade IIIb, lymph nodes of axilla and upper limb
        "C8245",  # Follicular lymphoma grade IIIb, lymph nodes of inguinal region and lower limb
        "C8246",  # Follicular lymphoma grade IIIb, intrapelvic lymph nodes
        "C8247",  # Follicular lymphoma grade IIIb, spleen
        "C8248",  # Follicular lymphoma grade IIIb, lymph nodes of multiple sites
        "C8249",  # Follicular lymphoma grade IIIb, extranodal and solid organ sites
        "C8250",  # Diffuse follicle center lymphoma, unspecified site
        "C8251",  # Diffuse follicle center lymphoma, lymph nodes of head, face, and neck
        "C8252",  # Diffuse follicle center lymphoma, intrathoracic lymph nodes
        "C8253",  # Diffuse follicle center lymphoma, intra-abdominal lymph nodes
        "C8254",  # Diffuse follicle center lymphoma, lymph nodes of axilla and upper limb
        "C8255",  # Diffuse follicle center lymphoma, lymph nodes of inguinal region and lower limb
        "C8256",  # Diffuse follicle center lymphoma, intrapelvic lymph nodes
        "C8257",  # Diffuse follicle center lymphoma, spleen
        "C8258",  # Diffuse follicle center lymphoma, lymph nodes of multiple sites
        "C8259",  # Diffuse follicle center lymphoma, extranodal and solid organ sites
        "C8260",  # Cutaneous follicle center lymphoma, unspecified site
        "C8261",  # Cutaneous follicle center lymphoma, lymph nodes of head, face, and neck
        "C8262",  # Cutaneous follicle center lymphoma, intrathoracic lymph nodes
        "C8263",  # Cutaneous follicle center lymphoma, intra-abdominal lymph nodes
        "C8264",  # Cutaneous follicle center lymphoma, lymph nodes of axilla and upper limb
        "C8265",  # Cutaneous follicle center lymphoma, lymph nodes of inguinal region and lower limb
        "C8266",  # Cutaneous follicle center lymphoma, intrapelvic lymph nodes
        "C8267",  # Cutaneous follicle center lymphoma, spleen
        "C8268",  # Cutaneous follicle center lymphoma, lymph nodes of multiple sites
        "C8269",  # Cutaneous follicle center lymphoma, extranodal and solid organ sites
        "C8280",  # Other types of follicular lymphoma, unspecified site
        "C8281",  # Other types of follicular lymphoma, lymph nodes of head, face, and neck
        "C8282",  # Other types of follicular lymphoma, intrathoracic lymph nodes
        "C8283",  # Other types of follicular lymphoma, intra-abdominal lymph nodes
        "C8284",  # Other types of follicular lymphoma, lymph nodes of axilla and upper limb
        "C8285",  # Other types of follicular lymphoma, lymph nodes of inguinal region and lower limb
        "C8286",  # Other types of follicular lymphoma, intrapelvic lymph nodes
        "C8287",  # Other types of follicular lymphoma, spleen
        "C8288",  # Other types of follicular lymphoma, lymph nodes of multiple sites
        "C8289",  # Other types of follicular lymphoma, extranodal and solid organ sites
        "C8290",  # Follicular lymphoma, unspecified, unspecified site
        "C8291",  # Follicular lymphoma, unspecified, lymph nodes of head, face, and neck
        "C8292",  # Follicular lymphoma, unspecified, intrathoracic lymph nodes
        "C8293",  # Follicular lymphoma, unspecified, intra-abdominal lymph nodes
        "C8294",  # Follicular lymphoma, unspecified, lymph nodes of axilla and upper limb
        "C8295",  # Follicular lymphoma, unspecified, lymph nodes of inguinal region and lower limb
        "C8296",  # Follicular lymphoma, unspecified, intrapelvic lymph nodes
        "C8297",  # Follicular lymphoma, unspecified, spleen
        "C8298",  # Follicular lymphoma, unspecified, lymph nodes of multiple sites
        "C8299",  # Follicular lymphoma, unspecified, extranodal and solid organ sites
        "C8300",  # Small cell B-cell lymphoma, unspecified site
        "C8301",  # Small cell B-cell lymphoma, lymph nodes of head, face, and neck
        "C8302",  # Small cell B-cell lymphoma, intrathoracic lymph nodes
        "C8303",  # Small cell B-cell lymphoma, intra-abdominal lymph nodes
        "C8304",  # Small cell B-cell lymphoma, lymph nodes of axilla and upper limb
        "C8305",  # Small cell B-cell lymphoma, lymph nodes of inguinal region and lower limb
        "C8306",  # Small cell B-cell lymphoma, intrapelvic lymph nodes
        "C8307",  # Small cell B-cell lymphoma, spleen
        "C8308",  # Small cell B-cell lymphoma, lymph nodes of multiple sites
        "C8309",  # Small cell B-cell lymphoma, extranodal and solid organ sites
        "C8310",  # Mantle cell lymphoma, unspecified site
        "C8311",  # Mantle cell lymphoma, lymph nodes of head, face, and neck
        "C8312",  # Mantle cell lymphoma, intrathoracic lymph nodes
        "C8313",  # Mantle cell lymphoma, intra-abdominal lymph nodes
        "C8314",  # Mantle cell lymphoma, lymph nodes of axilla and upper limb
        "C8315",  # Mantle cell lymphoma, lymph nodes of inguinal region and lower limb
        "C8316",  # Mantle cell lymphoma, intrapelvic lymph nodes
        "C8317",  # Mantle cell lymphoma, spleen
        "C8318",  # Mantle cell lymphoma, lymph nodes of multiple sites
        "C8319",  # Mantle cell lymphoma, extranodal and solid organ sites
        "C8330",  # Diffuse large B-cell lymphoma, unspecified site
        "C8331",  # Diffuse large B-cell lymphoma, lymph nodes of head, face, and neck
        "C8332",  # Diffuse large B-cell lymphoma, intrathoracic lymph nodes
        "C8333",  # Diffuse large B-cell lymphoma, intra-abdominal lymph nodes
        "C8334",  # Diffuse large B-cell lymphoma, lymph nodes of axilla and upper limb
        "C8335",  # Diffuse large B-cell lymphoma, lymph nodes of inguinal region and lower limb
        "C8336",  # Diffuse large B-cell lymphoma, intrapelvic lymph nodes
        "C8337",  # Diffuse large B-cell lymphoma, spleen
        "C8338",  # Diffuse large B-cell lymphoma, lymph nodes of multiple sites
        "C8339",  # Diffuse large B-cell lymphoma, extranodal and solid organ sites
        "C8350",  # Lymphoblastic (diffuse) lymphoma, unspecified site
        "C8351",  # Lymphoblastic (diffuse) lymphoma, lymph nodes of head, face, and neck
        "C8352",  # Lymphoblastic (diffuse) lymphoma, intrathoracic lymph nodes
        "C8353",  # Lymphoblastic (diffuse) lymphoma, intra-abdominal lymph nodes
        "C8354",  # Lymphoblastic (diffuse) lymphoma, lymph nodes of axilla and upper limb
        "C8355",  # Lymphoblastic (diffuse) lymphoma, lymph nodes of inguinal region and lower limb
        "C8356",  # Lymphoblastic (diffuse) lymphoma, intrapelvic lymph nodes
        "C8357",  # Lymphoblastic (diffuse) lymphoma, spleen
        "C8358",  # Lymphoblastic (diffuse) lymphoma, lymph nodes of multiple sites
        "C8359",  # Lymphoblastic (diffuse) lymphoma, extranodal and solid organ sites
        "C8370",  # Burkitt lymphoma, unspecified site
        "C8371",  # Burkitt lymphoma, lymph nodes of head, face, and neck
        "C8372",  # Burkitt lymphoma, intrathoracic lymph nodes
        "C8373",  # Burkitt lymphoma, intra-abdominal lymph nodes
        "C8374",  # Burkitt lymphoma, lymph nodes of axilla and upper limb
        "C8375",  # Burkitt lymphoma, lymph nodes of inguinal region and lower limb
        "C8376",  # Burkitt lymphoma, intrapelvic lymph nodes
        "C8377",  # Burkitt lymphoma, spleen
        "C8378",  # Burkitt lymphoma, lymph nodes of multiple sites
        "C8379",  # Burkitt lymphoma, extranodal and solid organ sites
        "C8380",  # Other non-follicular lymphoma, unspecified site
        "C8381",  # Other non-follicular lymphoma, lymph nodes of head, face, and neck
        "C8382",  # Other non-follicular lymphoma, intrathoracic lymph nodes
        "C8383",  # Other non-follicular lymphoma, intra-abdominal lymph nodes
        "C8384",  # Other non-follicular lymphoma, lymph nodes of axilla and upper limb
        "C8385",  # Other non-follicular lymphoma, lymph nodes of inguinal region and lower limb
        "C8386",  # Other non-follicular lymphoma, intrapelvic lymph nodes
        "C8387",  # Other non-follicular lymphoma, spleen
        "C8388",  # Other non-follicular lymphoma, lymph nodes of multiple sites
        "C8389",  # Other non-follicular lymphoma, extranodal and solid organ sites
        "C8390",  # Non-follicular (diffuse) lymphoma, unspecified, unspecified site
        "C8391",  # Non-follicular (diffuse) lymphoma, unspecified, lymph nodes of head, face, and neck
        "C8392",  # Non-follicular (diffuse) lymphoma, unspecified, intrathoracic lymph nodes
        "C8393",  # Non-follicular (diffuse) lymphoma, unspecified, intra-abdominal lymph nodes
        "C8394",  # Non-follicular (diffuse) lymphoma, unspecified, lymph nodes of axilla and upper limb
        "C8395",  # Non-follicular (diffuse) lymphoma, unspecified, lymph nodes of inguinal region and lower limb
        "C8396",  # Non-follicular (diffuse) lymphoma, unspecified, intrapelvic lymph nodes
        "C8397",  # Non-follicular (diffuse) lymphoma, unspecified, spleen
        "C8398",  # Non-follicular (diffuse) lymphoma, unspecified, lymph nodes of multiple sites
        "C8399",  # Non-follicular (diffuse) lymphoma, unspecified, extranodal and solid organ sites
        "C8400",  # Mycosis fungoides, unspecified site
        "C8401",  # Mycosis fungoides, lymph nodes of head, face, and neck
        "C8402",  # Mycosis fungoides, intrathoracic lymph nodes
        "C8403",  # Mycosis fungoides, intra-abdominal lymph nodes
        "C8404",  # Mycosis fungoides, lymph nodes of axilla and upper limb
        "C8405",  # Mycosis fungoides, lymph nodes of inguinal region and lower limb
        "C8406",  # Mycosis fungoides, intrapelvic lymph nodes
        "C8407",  # Mycosis fungoides, spleen
        "C8408",  # Mycosis fungoides, lymph nodes of multiple sites
        "C8409",  # Mycosis fungoides, extranodal and solid organ sites
        "C8410",  # Sezary disease, unspecified site
        "C8411",  # Sezary disease, lymph nodes of head, face, and neck
        "C8412",  # Sezary disease, intrathoracic lymph nodes
        "C8413",  # Sezary disease, intra-abdominal lymph nodes
        "C8414",  # Sezary disease, lymph nodes of axilla and upper limb
        "C8415",  # Sezary disease, lymph nodes of inguinal region and lower limb
        "C8416",  # Sezary disease, intrapelvic lymph nodes
        "C8417",  # Sezary disease, spleen
        "C8418",  # Sezary disease, lymph nodes of multiple sites
        "C8419",  # Sezary disease, extranodal and solid organ sites
        "C8440",  # Peripheral T-cell lymphoma, not elsewhere classified, unspecified site
        "C8441",  # Peripheral T-cell lymphoma, not elsewhere classified, lymph nodes of head, face, and neck
        "C8442",  # Peripheral T-cell lymphoma, not elsewhere classified, intrathoracic lymph nodes
        "C8443",  # Peripheral T-cell lymphoma, not elsewhere classified, intra-abdominal lymph nodes
        "C8444",  # Peripheral T-cell lymphoma, not elsewhere classified, lymph nodes of axilla and upper limb
        "C8445",  # Peripheral T-cell lymphoma, not elsewhere classified, lymph nodes of inguinal region and lower limb
        "C8446",  # Peripheral T-cell lymphoma, not elsewhere classified, intrapelvic lymph nodes
        "C8447",  # Peripheral T-cell lymphoma, not elsewhere classified, spleen
        "C8448",  # Peripheral T-cell lymphoma, not elsewhere classified, lymph nodes of multiple sites
        "C8449",  # Peripheral T-cell lymphoma, not elsewhere classified, extranodal and solid organ sites
        "C8460",  # Anaplastic large cell lymphoma, ALK-positive, unspecified site
        "C8461",  # Anaplastic large cell lymphoma, ALK-positive, lymph nodes of head, face, and neck
        "C8462",  # Anaplastic large cell lymphoma, ALK-positive, intrathoracic lymph nodes
        "C8463",  # Anaplastic large cell lymphoma, ALK-positive, intra-abdominal lymph nodes
        "C8464",  # Anaplastic large cell lymphoma, ALK-positive, lymph nodes of axilla and upper limb
        "C8465",  # Anaplastic large cell lymphoma, ALK-positive, lymph nodes of inguinal region and lower limb
        "C8466",  # Anaplastic large cell lymphoma, ALK-positive, intrapelvic lymph nodes
        "C8467",  # Anaplastic large cell lymphoma, ALK-positive, spleen
        "C8468",  # Anaplastic large cell lymphoma, ALK-positive, lymph nodes of multiple sites
        "C8469",  # Anaplastic large cell lymphoma, ALK-positive, extranodal and solid organ sites
        "C8470",  # Anaplastic large cell lymphoma, ALK-negative, unspecified site
        "C8471",  # Anaplastic large cell lymphoma, ALK-negative, lymph nodes of head, face, and neck
        "C8472",  # Anaplastic large cell lymphoma, ALK-negative, intrathoracic lymph nodes
        "C8473",  # Anaplastic large cell lymphoma, ALK-negative, intra-abdominal lymph nodes
        "C8474",  # Anaplastic large cell lymphoma, ALK-negative, lymph nodes of axilla and upper limb
        "C8475",  # Anaplastic large cell lymphoma, ALK-negative, lymph nodes of inguinal region and lower limb
        "C8476",  # Anaplastic large cell lymphoma, ALK-negative, intrapelvic lymph nodes
        "C8477",  # Anaplastic large cell lymphoma, ALK-negative, spleen
        "C8478",  # Anaplastic large cell lymphoma, ALK-negative, lymph nodes of multiple sites
        "C8479",  # Anaplastic large cell lymphoma, ALK-negative, extranodal and solid organ sites
        "C847A",  # Anaplastic large cell lymphoma, ALK-negative, breast
        "C8490",  # Mature T/NK-cell lymphomas, unspecified, unspecified site
        "C8491",  # Mature T/NK-cell lymphomas, unspecified, lymph nodes of head, face, and neck
        "C8492",  # Mature T/NK-cell lymphomas, unspecified, intrathoracic lymph nodes
        "C8493",  # Mature T/NK-cell lymphomas, unspecified, intra-abdominal lymph nodes
        "C8494",  # Mature T/NK-cell lymphomas, unspecified, lymph nodes of axilla and upper limb
        "C8495",  # Mature T/NK-cell lymphomas, unspecified, lymph nodes of inguinal region and lower limb
        "C8496",  # Mature T/NK-cell lymphomas, unspecified, intrapelvic lymph nodes
        "C8497",  # Mature T/NK-cell lymphomas, unspecified, spleen
        "C8498",  # Mature T/NK-cell lymphomas, unspecified, lymph nodes of multiple sites
        "C8499",  # Mature T/NK-cell lymphomas, unspecified, extranodal and solid organ sites
        "C84A0",  # Cutaneous T-cell lymphoma, unspecified, unspecified site
        "C84A1",  # Cutaneous T-cell lymphoma, unspecified lymph nodes of head, face, and neck
        "C84A2",  # Cutaneous T-cell lymphoma, unspecified, intrathoracic lymph nodes
        "C84A3",  # Cutaneous T-cell lymphoma, unspecified, intra-abdominal lymph nodes
        "C84A4",  # Cutaneous T-cell lymphoma, unspecified, lymph nodes of axilla and upper limb
        "C84A5",  # Cutaneous T-cell lymphoma, unspecified, lymph nodes of inguinal region and lower limb
        "C84A6",  # Cutaneous T-cell lymphoma, unspecified, intrapelvic lymph nodes
        "C84A7",  # Cutaneous T-cell lymphoma, unspecified, spleen
        "C84A8",  # Cutaneous T-cell lymphoma, unspecified, lymph nodes of multiple sites
        "C84A9",  # Cutaneous T-cell lymphoma, unspecified, extranodal and solid organ sites
        "C84Z0",  # Other mature T/NK-cell lymphomas, unspecified site
        "C84Z1",  # Other mature T/NK-cell lymphomas, lymph nodes of head, face, and neck
        "C84Z2",  # Other mature T/NK-cell lymphomas, intrathoracic lymph nodes
        "C84Z3",  # Other mature T/NK-cell lymphomas, intra-abdominal lymph nodes
        "C84Z4",  # Other mature T/NK-cell lymphomas, lymph nodes of axilla and upper limb
        "C84Z5",  # Other mature T/NK-cell lymphomas, lymph nodes of inguinal region and lower limb
        "C84Z6",  # Other mature T/NK-cell lymphomas, intrapelvic lymph nodes
        "C84Z7",  # Other mature T/NK-cell lymphomas, spleen
        "C84Z8",  # Other mature T/NK-cell lymphomas, lymph nodes of multiple sites
        "C84Z9",  # Other mature T/NK-cell lymphomas, extranodal and solid organ sites
        "C8510",  # Unspecified B-cell lymphoma, unspecified site
        "C8511",  # Unspecified B-cell lymphoma, lymph nodes of head, face, and neck
        "C8512",  # Unspecified B-cell lymphoma, intrathoracic lymph nodes
        "C8513",  # Unspecified B-cell lymphoma, intra-abdominal lymph nodes
        "C8514",  # Unspecified B-cell lymphoma, lymph nodes of axilla and upper limb
        "C8515",  # Unspecified B-cell lymphoma, lymph nodes of inguinal region and lower limb
        "C8516",  # Unspecified B-cell lymphoma, intrapelvic lymph nodes
        "C8517",  # Unspecified B-cell lymphoma, spleen
        "C8518",  # Unspecified B-cell lymphoma, lymph nodes of multiple sites
        "C8519",  # Unspecified B-cell lymphoma, extranodal and solid organ sites
        "C8520",  # Mediastinal (thymic) large B-cell lymphoma, unspecified site
        "C8521",  # Mediastinal (thymic) large B-cell lymphoma, lymph nodes of head, face, and neck
        "C8522",  # Mediastinal (thymic) large B-cell lymphoma, intrathoracic lymph nodes
        "C8523",  # Mediastinal (thymic) large B-cell lymphoma, intra-abdominal lymph nodes
        "C8524",  # Mediastinal (thymic) large B-cell lymphoma, lymph nodes of axilla and upper limb
        "C8525",  # Mediastinal (thymic) large B-cell lymphoma, lymph nodes of inguinal region and lower limb
        "C8526",  # Mediastinal (thymic) large B-cell lymphoma, intrapelvic lymph nodes
        "C8527",  # Mediastinal (thymic) large B-cell lymphoma, spleen
        "C8528",  # Mediastinal (thymic) large B-cell lymphoma, lymph nodes of multiple sites
        "C8529",  # Mediastinal (thymic) large B-cell lymphoma, extranodal and solid organ sites
        "C8580",  # Other specified types of non-Hodgkin lymphoma, unspecified site
        "C8581",  # Other specified types of non-Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8582",  # Other specified types of non-Hodgkin lymphoma, intrathoracic lymph nodes
        "C8583",  # Other specified types of non-Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8584",  # Other specified types of non-Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8585",  # Other specified types of non-Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8586",  # Other specified types of non-Hodgkin lymphoma, intrapelvic lymph nodes
        "C8587",  # Other specified types of non-Hodgkin lymphoma, spleen
        "C8588",  # Other specified types of non-Hodgkin lymphoma, lymph nodes of multiple sites
        "C8589",  # Other specified types of non-Hodgkin lymphoma, extranodal and solid organ sites
        "C8590",  # Non-Hodgkin lymphoma, unspecified, unspecified site
        "C8591",  # Non-Hodgkin lymphoma, unspecified, lymph nodes of head, face, and neck
        "C8592",  # Non-Hodgkin lymphoma, unspecified, intrathoracic lymph nodes
        "C8593",  # Non-Hodgkin lymphoma, unspecified, intra-abdominal lymph nodes
        "C8594",  # Non-Hodgkin lymphoma, unspecified, lymph nodes of axilla and upper limb
        "C8595",  # Non-Hodgkin lymphoma, unspecified, lymph nodes of inguinal region and lower limb
        "C8596",  # Non-Hodgkin lymphoma, unspecified, intrapelvic lymph nodes
        "C8597",  # Non-Hodgkin lymphoma, unspecified, spleen
        "C8598",  # Non-Hodgkin lymphoma, unspecified, lymph nodes of multiple sites
        "C8599",  # Non-Hodgkin lymphoma, unspecified, extranodal and solid organ sites
        "C860",  # Extranodal NK/T-cell lymphoma, nasal type
        "C861",  # Hepatosplenic T-cell lymphoma
        "C862",  # Enteropathy-type (intestinal) T-cell lymphoma
        "C863",  # Subcutaneous panniculitis-like T-cell lymphoma
        "C864",  # Blastic NK-cell lymphoma
        "C865",  # Angioimmunoblastic T-cell lymphoma
        "C866",  # Primary cutaneous CD30-positive T-cell proliferations
        "C880",  # Waldenstrom macroglobulinemia
        "C882",  # Heavy chain disease
        "C883",  # Immunoproliferative small intestinal disease
        "C884",  # Extranodal marginal zone B-cell lymphoma of mucosa-associated lymphoid tissue [MALT-lymphoma]
        "C888",  # Other malignant immunoproliferative diseases
        "C889",  # Malignant immunoproliferative disease, unspecified
        "C9000",  # Multiple myeloma not having achieved remission
        "C9001",  # Multiple myeloma in remission
        "C9002",  # Multiple myeloma in relapse
        "C9010",  # Plasma cell leukemia not having achieved remission
        "C9011",  # Plasma cell leukemia in remission
        "C9012",  # Plasma cell leukemia in relapse
        "C9020",  # Extramedullary plasmacytoma not having achieved remission
        "C9021",  # Extramedullary plasmacytoma in remission
        "C9022",  # Extramedullary plasmacytoma in relapse
        "C9030",  # Solitary plasmacytoma not having achieved remission
        "C9031",  # Solitary plasmacytoma in remission
        "C9032",  # Solitary plasmacytoma in relapse
        "C9100",  # Acute lymphoblastic leukemia not having achieved remission
        "C9101",  # Acute lymphoblastic leukemia, in remission
        "C9102",  # Acute lymphoblastic leukemia, in relapse
        "C9110",  # Chronic lymphocytic leukemia of B-cell type not having achieved remission
        "C9111",  # Chronic lymphocytic leukemia of B-cell type in remission
        "C9112",  # Chronic lymphocytic leukemia of B-cell type in relapse
        "C9130",  # Prolymphocytic leukemia of B-cell type not having achieved remission
        "C9131",  # Prolymphocytic leukemia of B-cell type, in remission
        "C9132",  # Prolymphocytic leukemia of B-cell type, in relapse
        "C9140",  # Hairy cell leukemia not having achieved remission
        "C9141",  # Hairy cell leukemia, in remission
        "C9142",  # Hairy cell leukemia, in relapse
        "C9150",  # Adult T-cell lymphoma/leukemia (HTLV-1-associated) not having achieved remission
        "C9151",  # Adult T-cell lymphoma/leukemia (HTLV-1-associated), in remission
        "C9152",  # Adult T-cell lymphoma/leukemia (HTLV-1-associated), in relapse
        "C9160",  # Prolymphocytic leukemia of T-cell type not having achieved remission
        "C9161",  # Prolymphocytic leukemia of T-cell type, in remission
        "C9162",  # Prolymphocytic leukemia of T-cell type, in relapse
        "C9190",  # Lymphoid leukemia, unspecified not having achieved remission
        "C9191",  # Lymphoid leukemia, unspecified, in remission
        "C9192",  # Lymphoid leukemia, unspecified, in relapse
        "C91A0",  # Mature B-cell leukemia Burkitt-type not having achieved remission
        "C91A1",  # Mature B-cell leukemia Burkitt-type, in remission
        "C91A2",  # Mature B-cell leukemia Burkitt-type, in relapse
        "C91Z0",  # Other lymphoid leukemia not having achieved remission
        "C91Z1",  # Other lymphoid leukemia, in remission
        "C91Z2",  # Other lymphoid leukemia, in relapse
        "C9200",  # Acute myeloblastic leukemia, not having achieved remission
        "C9201",  # Acute myeloblastic leukemia, in remission
        "C9202",  # Acute myeloblastic leukemia, in relapse
        "C9210",  # Chronic myeloid leukemia, BCR/ABL-positive, not having achieved remission
        "C9211",  # Chronic myeloid leukemia, BCR/ABL-positive, in remission
        "C9212",  # Chronic myeloid leukemia, BCR/ABL-positive, in relapse
        "C9220",  # Atypical chronic myeloid leukemia, BCR/ABL-negative, not having achieved remission
        "C9221",  # Atypical chronic myeloid leukemia, BCR/ABL-negative, in remission
        "C9222",  # Atypical chronic myeloid leukemia, BCR/ABL-negative, in relapse
        "C9230",  # Myeloid sarcoma, not having achieved remission
        "C9231",  # Myeloid sarcoma, in remission
        "C9232",  # Myeloid sarcoma, in relapse
        "C9240",  # Acute promyelocytic leukemia, not having achieved remission
        "C9241",  # Acute promyelocytic leukemia, in remission
        "C9242",  # Acute promyelocytic leukemia, in relapse
        "C9250",  # Acute myelomonocytic leukemia, not having achieved remission
        "C9251",  # Acute myelomonocytic leukemia, in remission
        "C9252",  # Acute myelomonocytic leukemia, in relapse
        "C9260",  # Acute myeloid leukemia with 11q23-abnormality not having achieved remission
        "C9261",  # Acute myeloid leukemia with 11q23-abnormality in remission
        "C9262",  # Acute myeloid leukemia with 11q23-abnormality in relapse
        "C9290",  # Myeloid leukemia, unspecified, not having achieved remission
        "C9291",  # Myeloid leukemia, unspecified in remission
        "C9292",  # Myeloid leukemia, unspecified in relapse
        "C92A0",  # Acute myeloid leukemia with multilineage dysplasia, not having achieved remission
        "C92A1",  # Acute myeloid leukemia with multilineage dysplasia, in remission
        "C92A2",  # Acute myeloid leukemia with multilineage dysplasia, in relapse
        "C92Z0",  # Other myeloid leukemia not having achieved remission
        "C92Z1",  # Other myeloid leukemia, in remission
        "C92Z2",  # Other myeloid leukemia, in relapse
        "C9300",  # Acute monoblastic/monocytic leukemia, not having achieved remission
        "C9301",  # Acute monoblastic/monocytic leukemia, in remission
        "C9302",  # Acute monoblastic/monocytic leukemia, in relapse
        "C9310",  # Chronic myelomonocytic leukemia not having achieved remission
        "C9311",  # Chronic myelomonocytic leukemia, in remission
        "C9312",  # Chronic myelomonocytic leukemia, in relapse
        "C9330",  # Juvenile myelomonocytic leukemia, not having achieved remission
        "C9331",  # Juvenile myelomonocytic leukemia, in remission
        "C9332",  # Juvenile myelomonocytic leukemia, in relapse
        "C9390",  # Monocytic leukemia, unspecified, not having achieved remission
        "C9391",  # Monocytic leukemia, unspecified in remission
        "C9392",  # Monocytic leukemia, unspecified in relapse
        "C93Z0",  # Other monocytic leukemia, not having achieved remission
        "C93Z1",  # Other monocytic leukemia, in remission
        "C93Z2",  # Other monocytic leukemia, in relapse
        "C9400",  # Acute erythroid leukemia, not having achieved remission
        "C9401",  # Acute erythroid leukemia, in remission
        "C9402",  # Acute erythroid leukemia, in relapse
        "C9420",  # Acute megakaryoblastic leukemia not having achieved remission
        "C9421",  # Acute megakaryoblastic leukemia, in remission
        "C9422",  # Acute megakaryoblastic leukemia, in relapse
        "C9430",  # Mast cell leukemia not having achieved remission
        "C9431",  # Mast cell leukemia, in remission
        "C9432",  # Mast cell leukemia, in relapse
        "C9440",  # Acute panmyelosis with myelofibrosis not having achieved remission
        "C9441",  # Acute panmyelosis with myelofibrosis, in remission
        "C9442",  # Acute panmyelosis with myelofibrosis, in relapse
        "C946",  # Myelodysplastic disease, not elsewhere classified
        "C9480",  # Other specified leukemias not having achieved remission
        "C9481",  # Other specified leukemias, in remission
        "C9482",  # Other specified leukemias, in relapse
        "C9500",  # Acute leukemia of unspecified cell type not having achieved remission
        "C9501",  # Acute leukemia of unspecified cell type, in remission
        "C9502",  # Acute leukemia of unspecified cell type, in relapse
        "C9510",  # Chronic leukemia of unspecified cell type not having achieved remission
        "C9511",  # Chronic leukemia of unspecified cell type, in remission
        "C9512",  # Chronic leukemia of unspecified cell type, in relapse
        "C9590",  # Leukemia, unspecified not having achieved remission
        "C9591",  # Leukemia, unspecified, in remission
        "C9592",  # Leukemia, unspecified, in relapse
        "C960",  # Multifocal and multisystemic (disseminated) Langerhans-cell histiocytosis
        "C962",  # Malignant mast cell neoplasm
        "C9620",  # Malignant mast cell neoplasm, unspecified
        "C9621",  # Aggressive systemic mastocytosis
        "C9622",  # Mast cell sarcoma
        "C9629",  # Other malignant mast cell neoplasm
        "C964",  # Sarcoma of dendritic cells (accessory cells)
        "C965",  # Multifocal and unisystemic Langerhans-cell histiocytosis
        "C966",  # Unifocal Langerhans-cell histiocytosis
        "C969",  # Malignant neoplasm of lymphoid, hematopoietic and related tissue, unspecified
        "C96A",  # Histiocytic sarcoma
        "C96Z",  # Other specified malignant neoplasms of lymphoid, hematopoietic and related tissue
        "D5701",  # Hb-SS disease with acute chest syndrome
        "D5703",  # Hb-SS disease with cerebral vascular involvement
        "D5709",  # Hb-SS disease with crisis with other specified complication
        "D57211",  # Sickle-cell/Hb-C disease with acute chest syndrome
        "D57213",  # Sickle-cell/Hb-C disease with cerebral vascular involvement
        "D57218",  # Sickle-cell/Hb-C disease with crisis with other specified complication
        "D57411",  # Sickle-cell thalassemia, unspecified, with acute chest syndrome
        "D57413",  # Sickle-cell thalassemia, unspecified, with cerebral vascular involvement
        "D57418",  # Sickle-cell thalassemia, unspecified, with crisis with other specified complication
        "D57811",  # Other sickle-cell disorders with acute chest syndrome
        "D57813",  # Other sickle-cell disorders with cerebral vascular involvement
        "D57818",  # Other sickle-cell disorders with crisis with other specified complication
        "D61810",  # Antineoplastic chemotherapy induced pancytopenia
        "D61811",  # Other drug-induced pancytopenia
        "D61818",  # Other pancytopenia
        "D700",  # Congenital agranulocytosis
        "D701",  # Agranulocytosis secondary to cancer chemotherapy
        "D702",  # Other drug-induced agranulocytosis
        "D703",  # Neutropenia due to infection
        "D704",  # Cyclic neutropenia
        "D708",  # Other neutropenia
        "D709",  # Neutropenia, unspecified
        "D71",  # Functional disorders of polymorphonuclear neutrophils
        "D720",  # Genetic anomalies of leukocytes
        "D7581",  # Myelofibrosis
        "D761",  # Hemophagocytic lymphohistiocytosis
        "D762",  # Hemophagocytic syndrome, infection-associated
        "D763",  # Other histiocytosis syndromes
        "D860",  # Sarcoidosis of lung
        "D861",  # Sarcoidosis of lymph nodes
        "D862",  # Sarcoidosis of lung with sarcoidosis of lymph nodes
        "E840",  # Cystic fibrosis with pulmonary manifestations
        "E8411",  # Meconium ileus in cystic fibrosis
        "E8419",  # Cystic fibrosis with other intestinal manifestations
        "E848",  # Cystic fibrosis with other manifestations
        "E849",  # Cystic fibrosis, unspecified
        "J22",  # Unspecified acute lower respiratory infection
        "J410",  # Simple chronic bronchitis
        "J411",  # Mucopurulent chronic bronchitis
        "J418",  # Mixed simple and mucopurulent chronic bronchitis
        "J42",  # Unspecified chronic bronchitis
        "J470",  # Bronchiectasis with acute lower respiratory infection
        "J471",  # Bronchiectasis with (acute) exacerbation
        "J479",  # Bronchiectasis, uncomplicated
        "J60",  # Coalworker's pneumoconiosis
        "J61",  # Pneumoconiosis due to asbestos and other mineral fibers
        "J620",  # Pneumoconiosis due to talc dust
        "J628",  # Pneumoconiosis due to other dust containing silica
        "J630",  # Aluminosis (of lung)
        "J631",  # Bauxite fibrosis (of lung)
        "J632",  # Berylliosis
        "J633",  # Graphite fibrosis (of lung)
        "J634",  # Siderosis
        "J635",  # Stannosis
        "J636",  # Pneumoconiosis due to other specified inorganic dusts
        "J64",  # Unspecified pneumoconiosis
        "J65",  # Pneumoconiosis associated with tuberculosis
        "J660",  # Byssinosis
        "J661",  # Flax-dressers' disease
        "J662",  # Cannabinosis
        "J668",  # Airway disease due to other specific organic dusts
        "J670",  # Farmer's lung
        "J671",  # Bagassosis
        "J672",  # Bird fancier's lung
        "J673",  # Suberosis
        "J674",  # Maltworker's lung
        "J675",  # Mushroom-worker's lung
        "J676",  # Maple-bark-stripper's lung
        "J677",  # Air conditioner and humidifier lung
        "J678",  # Hypersensitivity pneumonitis due to other organic dusts
        "J679",  # Hypersensitivity pneumonitis due to unspecified organic dust
        "J680",  # Bronchitis and pneumonitis due to chemicals, gases, fumes and vapors
        "J681",  # Pulmonary edema due to chemicals, gases, fumes and vapors
        "J682",  # Upper respiratory inflammation due to chemicals, gases, fumes and vapors, not elsewhere classified
        "J683",  # Other acute and subacute respiratory conditions due to chemicals, gases, fumes and vapors
        "J689",  # Unspecified respiratory condition due to chemicals, gases, fumes and vapors
        "J690",  # Pneumonitis due to inhalation of food and vomit
        "J691",  # Pneumonitis due to inhalation of oils and essences
        "J698",  # Pneumonitis due to inhalation of other solids and liquids
        "J700",  # Acute pulmonary manifestations due to radiation
        "J701",  # Chronic and other pulmonary manifestations due to radiation
        "J702",  # Acute drug-induced interstitial lung disorders
        "J703",  # Chronic drug-induced interstitial lung disorders
        "J704",  # Drug-induced interstitial lung disorders, unspecified
        "J705",  # Respiratory conditions due to smoke inhalation
        "J708",  # Respiratory conditions due to other specified external agents
        "J709",  # Respiratory conditions due to unspecified external agent
        "J80",  # Acute respiratory distress syndrome
        "J810",  # Acute pulmonary edema
        "J811",  # Chronic pulmonary edema
        "J82",  # Pulmonary eosinophilia, not elsewhere classified
        "J8281",  # Chronic eosinophilic pneumonia
        "J8282",  # Acute eosinophilic pneumonia
        "J8283",  # Eosinophilic asthma
        "J8289",  # Other pulmonary eosinophilia, not elsewhere classified
        "J8401",  # Alveolar proteinosis
        "J8402",  # Pulmonary alveolar microlithiasis
        "J8403",  # Idiopathic pulmonary hemosiderosis
        "J8409",  # Other alveolar and parieto-alveolar conditions
        "J8410",  # Pulmonary fibrosis, unspecified
        "J84111",  # Idiopathic interstitial pneumonia, not otherwise specified
        "J84112",  # Idiopathic pulmonary fibrosis
        "J84113",  # Idiopathic non-specific interstitial pneumonitis
        "J84114",  # Acute interstitial pneumonitis
        "J84115",  # Respiratory bronchiolitis interstitial lung disease
        "J84116",  # Cryptogenic organizing pneumonia
        "J84117",  # Desquamative interstitial pneumonia
        "J8417",  # Other interstitial pulmonary diseases with fibrosis in diseases classified elsewhere
        "J84170",  # Interstitial lung disease with progressive fibrotic phenotype in diseases classified elsewhere
        "J84178",  # Other interstitial pulmonary diseases with fibrosis in diseases classified elsewhere
        "J842",  # Lymphoid interstitial pneumonia
        "J8481",  # Lymphangioleiomyomatosis
        "J8482",  # Adult pulmonary Langerhans cell histiocytosis
        "J8483",  # Surfactant mutations of the lung
        "J84841",  # Neuroendocrine cell hyperplasia of infancy
        "J84842",  # Pulmonary interstitial glycogenosis
        "J84843",  # Alveolar capillary dysplasia with vein misalignment
        "J84848",  # Other interstitial lung diseases of childhood
        "J8489",  # Other specified interstitial pulmonary diseases
        "J849",  # Interstitial pulmonary disease, unspecified
        "J850",  # Gangrene and necrosis of lung
        "J851",  # Abscess of lung with pneumonia
        "J852",  # Abscess of lung without pneumonia
        "J853",  # Abscess of mediastinum
        "J860",  # Pyothorax with fistula
        "J869",  # Pyothorax without fistula
        "J90",  # Pleural effusion, not elsewhere classified
        "J910",  # Malignant pleural effusion
        "J918",  # Pleural effusion in other conditions classified elsewhere
        "J920",  # Pleural plaque with presence of asbestos
        "J929",  # Pleural plaque without asbestos
        "J930",  # Spontaneous tension pneumothorax
        "J9311",  # Primary spontaneous pneumothorax
        "J9312",  # Secondary spontaneous pneumothorax
        "J9381",  # Chronic pneumothorax
        "J9382",  # Other air leak
        "J9383",  # Other pneumothorax
        "J939",  # Pneumothorax, unspecified
        "J940",  # Chylous effusion
        "J941",  # Fibrothorax
        "J942",  # Hemothorax
        "J948",  # Other specified pleural conditions
        "J949",  # Pleural condition, unspecified
        "J9500",  # Unspecified tracheostomy complication
        "J9501",  # Hemorrhage from tracheostomy stoma
        "J9502",  # Infection of tracheostomy stoma
        "J9503",  # Malfunction of tracheostomy stoma
        "J9504",  # Tracheo-esophageal fistula following tracheostomy
        "J9509",  # Other tracheostomy complication
        "J951",  # Acute pulmonary insufficiency following thoracic surgery
        "J952",  # Acute pulmonary insufficiency following nonthoracic surgery
        "J953",  # Chronic pulmonary insufficiency following surgery
        "J954",  # Chemical pneumonitis due to anesthesia
        "J955",  # Postprocedural subglottic stenosis
        "J9561",  # Intraoperative hemorrhage and hematoma of a respiratory system organ or structure complicating a respiratory system procedure
        "J9562",  # Intraoperative hemorrhage and hematoma of a respiratory system organ or structure complicating other procedure
        "J9571",  # Accidental puncture and laceration of a respiratory system organ or structure during a respiratory system procedure
        "J9572",  # Accidental puncture and laceration of a respiratory system organ or structure during other procedure
        "J95811",  # Postprocedural pneumothorax
        "J95812",  # Postprocedural air leak
        "J95821",  # Acute postprocedural respiratory failure
        "J95822",  # Acute and chronic postprocedural respiratory failure
        "J95830",  # Postprocedural hemorrhage of a respiratory system organ or structure following a respiratory system procedure
        "J95831",  # Postprocedural hemorrhage of a respiratory system organ or structure following other procedure
        "J9584",  # Transfusion-related acute lung injury (TRALI)
        "J95850",  # Mechanical complication of respirator
        "J95851",  # Ventilator associated pneumonia
        "J95859",  # Other complication of respirator [ventilator]
        "J95860",  # Postprocedural hematoma of a respiratory system organ or structure following a respiratory system procedure
        "J95861",  # Postprocedural hematoma of a respiratory system organ or structure following other procedure
        "J95862",  # Postprocedural seroma of a respiratory system organ or structure following a respiratory system procedure
        "J95863",  # Postprocedural seroma of a respiratory system organ or structure following other procedure
        "J9588",  # Other intraoperative complications of respiratory system, not elsewhere classified
        "J9589",  # Other postprocedural complications and disorders of respiratory system, not elsewhere classified
        "J9600",  # Acute respiratory failure, unspecified whether with hypoxia or hypercapnia
        "J9601",  # Acute respiratory failure with hypoxia
        "J9602",  # Acute respiratory failure with hypercapnia
        "J9610",  # Chronic respiratory failure, unspecified whether with hypoxia or hypercapnia
        "J9611",  # Chronic respiratory failure with hypoxia
        "J9612",  # Chronic respiratory failure with hypercapnia
        "J9620",  # Acute and chronic respiratory failure, unspecified whether with hypoxia or hypercapnia
        "J9621",  # Acute and chronic respiratory failure with hypoxia
        "J9622",  # Acute and chronic respiratory failure with hypercapnia
        "J9690",  # Respiratory failure, unspecified, unspecified whether with hypoxia or hypercapnia
        "J9691",  # Respiratory failure, unspecified with hypoxia
        "J9692",  # Respiratory failure, unspecified with hypercapnia
        "J9851",  # Mediastinitis
        "J986",  # Disorders of diaphragm
        "J99",  # Respiratory disorders in diseases classified elsewhere
        "M0510",  # Rheumatoid lung disease with rheumatoid arthritis of unspecified site
        "M05111",  # Rheumatoid lung disease with rheumatoid arthritis of right shoulder
        "M05112",  # Rheumatoid lung disease with rheumatoid arthritis of left shoulder
        "M05119",  # Rheumatoid lung disease with rheumatoid arthritis of unspecified shoulder
        "M05121",  # Rheumatoid lung disease with rheumatoid arthritis of right elbow
        "M05122",  # Rheumatoid lung disease with rheumatoid arthritis of left elbow
        "M05129",  # Rheumatoid lung disease with rheumatoid arthritis of unspecified elbow
        "M05131",  # Rheumatoid lung disease with rheumatoid arthritis of right wrist
        "M05132",  # Rheumatoid lung disease with rheumatoid arthritis of left wrist
        "M05139",  # Rheumatoid lung disease with rheumatoid arthritis of unspecified wrist
        "M05141",  # Rheumatoid lung disease with rheumatoid arthritis of right hand
        "M05142",  # Rheumatoid lung disease with rheumatoid arthritis of left hand
        "M05149",  # Rheumatoid lung disease with rheumatoid arthritis of unspecified hand
        "M05151",  # Rheumatoid lung disease with rheumatoid arthritis of right hip
        "M05152",  # Rheumatoid lung disease with rheumatoid arthritis of left hip
        "M05159",  # Rheumatoid lung disease with rheumatoid arthritis of unspecified hip
        "M05161",  # Rheumatoid lung disease with rheumatoid arthritis of right knee
        "M05162",  # Rheumatoid lung disease with rheumatoid arthritis of left knee
        "M05169",  # Rheumatoid lung disease with rheumatoid arthritis of unspecified knee
        "M05171",  # Rheumatoid lung disease with rheumatoid arthritis of right ankle and foot
        "M05172",  # Rheumatoid lung disease with rheumatoid arthritis of left ankle and foot
        "M05179",  # Rheumatoid lung disease with rheumatoid arthritis of unspecified ankle and foot
        "M0519",  # Rheumatoid lung disease with rheumatoid arthritis of multiple sites
        "M301",  # Polyarteritis with lung involvement [Churg-Strauss]
        "M3213",  # Lung involvement in systemic lupus erythematosus
        "M3301",  # Juvenile dermatomyositis with respiratory involvement
        "M3311",  # Other dermatomyositis with respiratory involvement
        "M3321",  # Polymyositis with respiratory involvement
        "M3391",  # Dermatopolymyositis, unspecified with respiratory involvement
        "M3481",  # Systemic sclerosis with lung involvement
        "M3502",  # Sjogren syndrome with lung involvement
        "O98011",  # Tuberculosis complicating pregnancy, first trimester
        "O98012",  # Tuberculosis complicating pregnancy, second trimester
        "O98013",  # Tuberculosis complicating pregnancy, third trimester
        "O98019",  # Tuberculosis complicating pregnancy, unspecified trimester
        "O9802",  # Tuberculosis complicating childbirth
        "O9803",  # Tuberculosis complicating the puerperium
        "P270",  # Wilson-Mikity syndrome
        "P271",  # Bronchopulmonary dysplasia originating in the perinatal period
        "P278",  # Other chronic respiratory diseases originating in the perinatal period
        "P279",  # Unspecified chronic respiratory disease originating in the perinatal period
        "Q2545",  # Double aortic arch
        "Q2547",  # Right aortic arch
        "Q2548",  # Anomalous origin of subclavian artery
        "Q300",  # Choanal atresia
        "Q301",  # Agenesis and underdevelopment of nose
        "Q302",  # Fissured, notched and cleft nose
        "Q303",  # Congenital perforated nasal septum
        "Q308",  # Other congenital malformations of nose
        "Q309",  # Congenital malformation of nose, unspecified
        "Q310",  # Web of larynx
        "Q311",  # Congenital subglottic stenosis
        "Q312",  # Laryngeal hypoplasia
        "Q313",  # Laryngocele
        "Q315",  # Congenital laryngomalacia
        "Q318",  # Other congenital malformations of larynx
        "Q319",  # Congenital malformation of larynx, unspecified
        "Q320",  # Congenital tracheomalacia
        "Q321",  # Other congenital malformations of trachea
        "Q322",  # Congenital bronchomalacia
        "Q323",  # Congenital stenosis of bronchus
        "Q324",  # Other congenital malformations of bronchus
        "Q330",  # Congenital cystic lung
        "Q331",  # Accessory lobe of lung
        "Q332",  # Sequestration of lung
        "Q333",  # Agenesis of lung
        "Q334",  # Congenital bronchiectasis
        "Q335",  # Ectopic tissue in lung
        "Q336",  # Congenital hypoplasia and dysplasia of lung
        "Q338",  # Other congenital malformations of lung
        "Q339",  # Congenital malformation of lung, unspecified
        "Q340",  # Anomaly of pleura
        "Q341",  # Congenital cyst of mediastinum
        "Q348",  # Other specified congenital malformations of respiratory system
        "Q349",  # Congenital malformation of respiratory system, unspecified
        "Q390",  # Atresia of esophagus without fistula
        "Q391",  # Atresia of esophagus with tracheo-esophageal fistula
        "Q392",  # Congenital tracheo-esophageal fistula without atresia
        "Q393",  # Congenital stenosis and stricture of esophagus
        "Q394",  # Esophageal web
        "Q8901",  # Asplenia (congenital)
        "Q893",  # Situs inversus
    }

    SNOMEDCT = {
        "12137005",  # Tuberculous enteritis (disorder)
        "109965004",  # Diffuse non-Hodgkin's lymphoma, lymphoblastic (disorder)
        "189509003",  # Refractory anemia without sideroblasts, so stated (disorder)
        "277651000",  # Peripheral T-cell lymphoma - pleomorphic small cell (disorder)
        "277653002",  # Peripheral T-cell lymphoma - pleomorphic medium and large cell (disorder)
        "721302006",  # Myelodysplastic/myeloproliferative neoplasm with ring sideroblasts and thrombocytosis (disorder)
        "107751000119102",  # Primary malignant mixed Mullerian neoplasm of endometrium (disorder)
        "109356001",  # Primary malignant neoplasm of unspecified site (disorder)
        "1156808001",  # Malignant mixed Mullerian neoplasm of corpus uteri (disorder)
        "188044004",  # Malignant melanoma of scalp and/or neck (disorder)
        "188478004",  # Malignant neoplasms of independent (primary) multiple sites (disorder)
        "188513004",  # Burkitt lymphoma of lymph nodes of axilla and upper limb (disorder)
        "188514005",  # Burkitt lymphoma of lymph nodes of inguinal region and lower limb (disorder)
        "188537004",  # Hodgkin's granuloma of lymph nodes of axilla and upper limb (disorder)
        "188538009",  # Hodgkin's granuloma of lymph nodes of inguinal region and lower limb (disorder)
        "188547001",  # Hodgkin's sarcoma of lymph nodes of axilla and upper limb (disorder)
        "188548006",  # Hodgkin's sarcoma of lymph nodes of inguinal region and lower limb (disorder)
        "188554007",  # Hodgkin's disease, lymphocytic-histiocytic predominance of lymph nodes of head, face and neck (disorder)
        "188558005",  # Hodgkin's disease, lymphocytic-histiocytic predominance of lymph nodes of axilla and upper limb (disorder)
        "188559002",  # Hodgkin's disease, lymphocytic-histiocytic predominance of lymph nodes of inguinal region and lower limb (disorder)
        "188565002",  # Hodgkin's disease, nodular sclerosis of lymph nodes of head, face and neck (disorder)
        "188568000",  # Hodgkin's disease, nodular sclerosis of lymph nodes of axilla and upper limb (disorder)
        "188569008",  # Hodgkin's disease, nodular sclerosis of lymph nodes of inguinal region and lower limb (disorder)
        "188575004",  # Hodgkin's disease, mixed cellularity of lymph nodes of head, face and neck (disorder)
        "188578002",  # Hodgkin's disease, mixed cellularity of lymph nodes of axilla and upper limb (disorder)
        "188579005",  # Hodgkin's disease, mixed cellularity of lymph nodes of inguinal region and lower limb (disorder)
        "188585003",  # Hodgkin's disease, lymphocytic depletion of lymph nodes of head, face and neck (disorder)
        "188589009",  # Hodgkin's disease, lymphocytic depletion of lymph nodes of axilla and upper limb (disorder)
        "188590000",  # Hodgkin's disease, lymphocytic depletion of lymph nodes of inguinal region and lower limb (disorder)
        "188642004",  # Malignant histiocytosis of lymph nodes of inguinal region and lower limb (disorder)
        "188665009",  # Mast cell malignancy of lymph nodes of axilla and upper limb (disorder)
        "188666005",  # Mast cell malignancy of lymph nodes of inguinal region and lower limb (disorder)
        "188691005",  # Malignant immunoproliferative small intestinal disease (disorder)
        "21504004",  # Pressure collapse of lung after anesthesia AND/OR sedation in labor AND/OR delivery (disorder)
        "254889004",  # Carcinoma of cervix stage 0 (disorder)
        "269616004",  # Secondary nodes - axilla/arm (disorder)
        "314988008",  # Metastasis from malignant tumor of skin (disorder)
        "314989000",  # Metastasis from malignant tumor of soft tissues (disorder)
        "314990009",  # Metastasis from malignant tumor of bone (disorder)
        "314991008",  # Metastasis from malignant tumor of adrenal gland (disorder)
        "314992001",  # Metastasis from malignant tumor of cervix (disorder)
        "314993006",  # Metastasis from malignant tumor of uterus (disorder)
        "314995004",  # Metastasis from malignant tumor of bladder (disorder)
        "314996003",  # Metastasis from malignant tumor of kidney (disorder)
        "314997007",  # Metastasis from malignant tumor of rectum (disorder)
        "314998002",  # Metastasis from malignant tumor of colon (disorder)
        "314999005",  # Metastasis from malignant tumor of pancreas (disorder)
        "315000005",  # Metastasis from malignant tumor of liver (disorder)
        "315001009",  # Metastasis from malignant tumor of gallbladder (disorder)
        "315002002",  # Metastasis from malignant tumor of stomach (disorder)
        "315003007",  # Metastasis from malignant tumor of esophagus (disorder)
        "315004001",  # Metastasis from malignant tumor of breast (disorder)
        "315005000",  # Metastasis from malignant tumor of bronchus (disorder)
        "315006004",  # Metastasis from malignant neoplasm of lung (disorder)
        "315007008",  # Metastasis from malignant tumor of thyroid (disorder)
        "315008003",  # Metastasis from malignant tumor of buccal cavity (disorder)
        "315009006",  # Metastasis from malignant tumor of tongue (disorder)
        "369602008",  # Malignant tumor involving an organ by separate metastasis from bladder (disorder)
        "369603003",  # Malignant tumor involving an organ by separate metastasis from endometrium (disorder)
        "369605005",  # Malignant tumor involving an organ by separate metastasis from ovary (disorder)
        "369606006",  # Malignant tumor involving an organ by separate metastasis from prostate (disorder)
        "369607002",  # Malignant tumor involving an organ by separate metastasis from uterine cervix (disorder)
        "369608007",  # Malignant tumor involving an organ by separate metastasis from uterus (disorder)
        "369609004",  # Malignant tumor involving an organ by separate metastasis from vagina (disorder)
        "87101000119106",  # Primary malignant glioma of brain (disorder)
        "92511007",  # Burkitt's tumor of lymph nodes of axilla AND/OR upper limb (disorder)
        "92512000",  # Burkitt's tumor of lymph nodes of head, face AND/OR neck (disorder)
        "92513005",  # Burkitt's tumor of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "92516002",  # Burkitt's tumor of extranodal AND/OR solid organ site (disorder)
        "93136003",  # Letterer-Siwe disease of lymph nodes of axilla AND/OR upper limb (disorder)
        "93137007",  # Letterer-Siwe disease of lymph nodes of head, face AND/OR neck (disorder)
        "93138002",  # Letterer-Siwe disease of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93141006",  # Letterer-Siwe disease of extranodal AND/OR solid organ site (disorder)
        "93185008",  # Malignant histiocytosis of lymph nodes of axilla AND/OR upper limb (disorder)
        "93186009",  # Malignant histiocytosis of lymph nodes of head, face AND/OR neck (disorder)
        "93187000",  # Malignant histiocytosis of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93190006",  # Malignant histiocytosis of extranodal AND/OR solid organ site (disorder)
        "93194002",  # Malignant lymphoma of lymph nodes of axilla AND/OR upper limb (disorder)
        "93195001",  # Malignant lymphoma of lymph nodes of head, face AND/OR neck (disorder)
        "93196000",  # Malignant lymphoma of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93203007",  # Malignant mast cell tumor of lymph nodes of axilla AND/OR upper limb (disorder)
        "93204001",  # Malignant mast cell tumor of lymph nodes of head, face AND/OR neck (disorder)
        "93205000",  # Malignant mast cell tumor of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93487009",  # Hodgkin's disease, lymphocytic depletion of lymph nodes of axilla AND/OR upper limb (disorder)
        "93488004",  # Hodgkin's disease, lymphocytic depletion of lymph nodes of head, face AND/OR neck (disorder)
        "93489007",  # Hodgkin's disease, lymphocytic depletion of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93492006",  # Hodgkin's disease, lymphocytic depletion of extranodal AND/OR solid organ site (disorder)
        "93496009",  # Hodgkin's disease, lymphocytic-histiocytic predominance of lymph nodes of axilla AND/OR upper limb (disorder)
        "93497000",  # Hodgkin's disease, lymphocytic-histiocytic predominance of lymph nodes of head, face AND/OR neck (disorder)
        "93498005",  # Hodgkin's disease, lymphocytic-histiocytic predominance of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93501005",  # Hodgkin's disease, lymphocytic-histiocytic predominance of extranodal AND/OR solid organ site (disorder)
        "93505001",  # Hodgkin's disease, mixed cellularity of lymph nodes of axilla AND/OR upper limb (disorder)
        "93506000",  # Hodgkin's disease, mixed cellularity of lymph nodes of head, face AND/OR neck (disorder)
        "93507009",  # Hodgkin's disease, mixed cellularity of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93510002",  # Hodgkin's disease, mixed cellularity of extranodal AND/OR solid organ site (disorder)
        "93514006",  # Hodgkin's disease, nodular sclerosis of lymph nodes of axilla AND/OR upper limb (disorder)
        "93515007",  # Hodgkin's disease, nodular sclerosis of lymph nodes of head, face AND/OR neck (disorder)
        "93516008",  # Hodgkin's disease, nodular sclerosis of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93519001",  # Hodgkin's disease, nodular sclerosis of extranodal AND/OR solid organ site (disorder)
        "93523009",  # Hodgkin's disease of lymph nodes of axilla AND/OR upper limb (disorder)
        "93524003",  # Hodgkin's disease of lymph nodes of head, face AND/OR neck (disorder)
        "93525002",  # Hodgkin's disease of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93528000",  # Hodgkin's disease of extranodal AND/OR solid organ site (disorder)
        "93532006",  # Hodgkin's granuloma of lymph nodes of axilla AND/OR upper limb (disorder)
        "93533001",  # Hodgkin's granuloma of lymph nodes of head, face AND/OR neck (disorder)
        "93534007",  # Hodgkin's granuloma of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93537000",  # Hodgkin's granuloma of extranodal AND/OR solid organ site (disorder)
        "93541001",  # Hodgkin's paragranuloma of lymph nodes of axilla AND/OR upper limb (disorder)
        "93542008",  # Hodgkin's paragranuloma of lymph nodes of head, face AND/OR neck (disorder)
        "93543003",  # Hodgkin's paragranuloma of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93546006",  # Hodgkin's paragranuloma of extranodal AND/OR solid organ site (disorder)
        "93550004",  # Hodgkin's sarcoma of lymph nodes of axilla AND/OR upper limb (disorder)
        "93551000",  # Hodgkin's sarcoma of lymph nodes of head, face AND/OR neck (disorder)
        "93552007",  # Hodgkin's sarcoma of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93555009",  # Hodgkin's sarcoma of extranodal AND/OR solid organ site (disorder)
        "93951006",  # Primary malignant neoplasm of pelvic bone (disorder)
        "94710006",  # Mycosis fungoides of lymph nodes of axilla AND/OR upper limb (disorder)
        "94711005",  # Mycosis fungoides of lymph nodes of head, face AND/OR neck (disorder)
        "94715001",  # Mycosis fungoides of extranodal AND/OR solid organ site (disorder)
        "95194004",  # Nodular lymphoma of extranodal AND/OR solid organ site (disorder)
        "95260009",  # Sezary's disease of lymph nodes of head, face AND/OR neck (disorder)
        "95264000",  # Sezary's disease of extranodal AND/OR solid organ site (disorder)
        "1082901000112103",  # Primary malignant neoplasm of breast with axillary lymph node invasion (disorder)
        "109996008",  # Myelodysplastic syndrome: Refractory anemia, without ringed sideroblasts, without excess blasts (disorder)
        "110000005",  # Refractory anemia with excess blasts in transformation (disorder)
        "1153345005",  # Myelodysplastic syndrome with single lineage dysplasia (disorder)
        "127961000119108",  # Aleukemic myeloid leukemia in relapse (disorder)
        "13351431000119102",  # Metastatic malignant neoplasm to lymph nodes of neck from primary malignant neoplasm of thyroid (disorder)
        "16260631000119101",  # Metastatic malignant neoplasm to lymph node from primary malignant neoplasm of female breast (disorder)
        "1661000119106",  # Metastasis to lung from adenocarcinoma (disorder)
        "1691000119104",  # Metastasis to liver from adenocarcinoma (disorder)
        "187991006",  # Malignant neoplasm of connective and soft tissue of upper limb and shoulder (disorder)
        "187992004",  # Malignant neoplasm of connective and soft tissue of shoulder (disorder)
        "187993009",  # Malignant neoplasm of connective and soft tissue, upper arm (disorder)
        "187994003",  # Malignant neoplasm of connective and soft tissue of forearm (disorder)
        "187995002",  # Malignant neoplasm of connective and soft tissue of hand (disorder)
        "187996001",  # Malignant neoplasm of connective and soft tissue of finger (disorder)
        "187997005",  # Malignant neoplasm of connective and soft tissue of thumb (disorder)
        "187999008",  # Malignant neoplasm of connective and soft tissue of hip and lower limb (disorder)
        "188000002",  # Malignant neoplasm of connective and soft tissue of hip (disorder)
        "188001003",  # Malignant neoplasm of connective and soft tissue of thigh and upper leg (disorder)
        "188002005",  # Malignant neoplasm of connective and soft tissue of popliteal space (disorder)
        "188003000",  # Malignant neoplasm of connective and soft tissue of lower leg (disorder)
        "188004006",  # Malignant neoplasm of connective and soft tissue of foot (disorder)
        "188005007",  # Malignant neoplasm of connective and soft tissue of toe (disorder)
        "188006008",  # Malignant neoplasm of connective and soft tissue of great toe (disorder)
        "188009001",  # Malignant neoplasm of connective and soft tissue of thorax (disorder)
        "188010006",  # Malignant neoplasm of connective and soft tissue of axilla (disorder)
        "188013008",  # Malignant neoplasm of connective and soft tissues of thoracic spine (disorder)
        "188015001",  # Malignant neoplasm of connective and soft tissue of abdomen (disorder)
        "188016000",  # Malignant neoplasm of connective and soft tissue of abdominal wall (disorder)
        "188017009",  # Malignant neoplasm of connective and soft tissues of lumbar spine (disorder)
        "188019007",  # Malignant neoplasm of connective and soft tissue of pelvis (disorder)
        "188020001",  # Malignant neoplasm of connective and soft tissue of buttock (disorder)
        "188021002",  # Malignant neoplasm of connective and soft tissue of inguinal region (disorder)
        "188022009",  # Malignant neoplasm of connective and soft tissue of perineum (disorder)
        "188736006",  # Subacute myeloid leukemia (disorder)
        "188741003",  # Aleukemic myeloid leukemia (disorder)
        "188745007",  # Chronic monocytic leukemia (disorder)
        "188746008",  # Subacute monocytic leukemia (disorder)
        "188748009",  # Aleukemic monocytic leukemia (disorder)
        "199179007",  # Maternal tuberculosis in the puerperium - baby delivered during current episode of care (disorder)
        "199180005",  # Maternal tuberculosis during pregnancy - baby not yet delivered (disorder)
        "199181009",  # Maternal tuberculosis in the puerperium - baby delivered during previous episode of care (disorder)
        "20224008",  # Delta heavy chain disease (disorder)
        "277601005",  # Acute monoblastic leukemia (disorder)
        "285598005",  # Metastasis to trachea of unknown primary (disorder)
        "285603002",  # Metastasis to bronchus of unknown primary (disorder)
        "285604008",  # Metastasis to lung of unknown primary (disorder)
        "285605009",  # Metastasis to pleura of unknown primary (disorder)
        "285606005",  # Metastasis to heart of unknown primary (disorder)
        "285607001",  # Metastasis to mediastinum of unknown primary (disorder)
        "285608006",  # Metastasis to thymus of unknown primary (disorder)
        "285609003",  # Metastasis to small intestine of unknown primary (disorder)
        "285610008",  # Metastasis to large intestine of unknown primary (disorder)
        "285611007",  # Metastasis to colon of unknown primary (disorder)
        "285612000",  # Metastasis to rectum of unknown primary (disorder)
        "285613005",  # Metastasis to liver of unknown primary (disorder)
        "285614004",  # Metastasis to pancreas of unknown primary (disorder)
        "285615003",  # Metastasis to spleen of unknown primary (disorder)
        "285616002",  # Metastasis to peritoneum of unknown primary (disorder)
        "285617006",  # Metastasis to retroperitoneum of unknown primary (disorder)
        "285618001",  # Metastasis to bone of unknown primary (disorder)
        "285619009",  # Metastasis to vertebral column of unknown primary (disorder)
        "285631006",  # Metastasis to skin of unknown primary (disorder)
        "285633009",  # Metastasis to soft tissue of unknown primary (disorder)
        "285634003",  # Metastasis to breast of unknown primary (disorder)
        "285635002",  # Metastasis to uterus of unknown primary (disorder)
        "285637005",  # Metastasis to ovary of unknown primary (disorder)
        "285638000",  # Metastasis to vagina of unknown primary (disorder)
        "285639008",  # Metastasis to kidney of unknown primary (disorder)
        "285640005",  # Metastasis to bladder of unknown primary (disorder)
        "285641009",  # Metastasis to brain of unknown primary (disorder)
        "285642002",  # Metastasis to eye of unknown primary (disorder)
        "285643007",  # Metastasis to adrenal gland of unknown primary (disorder)
        "285644001",  # Metastasis to lymph node of unknown primary (disorder)
        "285645000",  # Disseminated malignancy of unknown primary (disorder)
        "285839005",  # Acute myelomonocytic leukemia - eosinophilic variant (disorder)
        "302823005",  # Glucagonoma (disorder)
        "314987003",  # Metastasis from malignant melanoma of skin (disorder)
        "314994000",  # Metastasis from malignant tumor of prostate (disorder)
        "351251000119103",  # Acute myelofibrosis in remission (disorder)
        "369455009",  # Malignant tumor involving rectum by separate metastasis from endometrium (disorder)
        "369456005",  # Malignant tumor involving rectum by separate metastasis from fallopian tube (disorder)
        "369457001",  # Malignant tumor involving rectum by separate metastasis from ovary (disorder)
        "369458006",  # Malignant tumor involving rectum by separate metastasis from prostate (disorder)
        "369459003",  # Malignant tumor involving rectum by separate metastasis from uterine cervix (disorder)
        "369460008",  # Malignant tumor involving rectum by separate metastasis from uterus (disorder)
        "369461007",  # Malignant tumor involving rectum by separate metastasis from vagina (disorder)
        "369464004",  # Malignant tumor involving ureter by separate metastasis from bladder (disorder)
        "369467006",  # Malignant tumor involving urethra by separate metastasis from bladder (disorder)
        "369468001",  # Malignant tumor involving urethra by separate metastasis from prostate (disorder)
        "369476004",  # Malignant tumor involving bladder by separate metastasis from endometrium (disorder)
        "369477008",  # Malignant tumor involving bladder by separate metastasis from fallopian tube (disorder)
        "369478003",  # Malignant tumor involving bladder by separate metastasis from ovary (disorder)
        "369479006",  # Malignant tumor involving bladder by separate metastasis from prostate (disorder)
        "369480009",  # Malignant tumor involving bladder by separate metastasis from uterine cervix (disorder)
        "369481008",  # Malignant tumor involving bladder by separate metastasis from uterus (disorder)
        "369482001",  # Malignant tumor involving bladder by separate metastasis from vagina (disorder)
        "369484000",  # Malignant tumor involving vasa deferentia by separate metastasis from prostate (disorder)
        "369486003",  # Malignant tumor involving prostate by separate metastasis from bladder (disorder)
        "369491002",  # Malignant tumor involving seminal vesicle by separate metastasis from bladder (disorder)
        "369492009",  # Malignant tumor involving seminal vesicle by separate metastasis from prostate (disorder)
        "369500009",  # Malignant tumor involving uterine cervix by separate metastasis from fallopian tube (disorder)
        "369501008",  # Malignant tumor involving uterine cervix by separate metastasis from ovary (disorder)
        "369502001",  # Malignant tumor involving uterine corpus by separate metastasis from bladder (disorder)
        "369538008",  # Malignant tumor involving left broad ligament by metastasis from ovary (disorder)
        "369540003",  # Malignant tumor involving right broad ligament by metastasis from ovary (disorder)
        "369542006",  # Malignant tumor involving left fallopian tube by separate metastasis from endometrium (disorder)
        "369543001",  # Malignant tumor involving left fallopian tube by separate metastasis from ovary (disorder)
        "369544007",  # Malignant tumor involving left fallopian tube by separate metastasis from right fallopian tube (disorder)
        "369545008",  # Malignant tumor involving left fallopian tube by separate metastasis from uterus (disorder)
        "369546009",  # Malignant tumor involving left fallopian tube by separate metastasis from vagina (disorder)
        "369553000",  # Malignant tumor involving right fallopian tube by separate metastasis from endometrium (disorder)
        "369554006",  # Malignant tumor involving right fallopian tube by separate metastasis from left fallopian tube (disorder)
        "369555007",  # Malignant tumor involving right fallopian tube by separate metastasis from ovary (disorder)
        "369556008",  # Malignant tumor involving right fallopian tube by separate metastasis from uterine cervix (disorder)
        "369557004",  # Malignant tumor involving right fallopian tube by separate metastasis from uterus (disorder)
        "369558009",  # Malignant tumor involving right fallopian tube by separate metastasis from vagina (disorder)
        "369560006",  # Malignant tumor involving left ovary by separate metastasis from endometrium (disorder)
        "369561005",  # Malignant tumor involving left ovary by separate metastasis from fallopian tube (disorder)
        "369562003",  # Malignant tumor involving left ovary by separate metastasis from right ovary (disorder)
        "369563008",  # Malignant tumor involving left ovary by separate metastasis from uterine cervix (disorder)
        "369564002",  # Malignant tumor involving left ovary by separate metastasis from vagina (disorder)
        "369565001",  # Malignant tumor involving left ovary by separate metastasis uterus (disorder)
        "369568004",  # Malignant tumor involving right ovary by separate metastasis from endometrium (disorder)
        "369569007",  # Malignant tumor involving right ovary by separate metastasis from fallopian tube (disorder)
        "369570008",  # Malignant tumor involving right ovary by separate metastasis from left ovary (disorder)
        "369571007",  # Malignant tumor involving right ovary by separate metastasis from uterine cervix (disorder)
        "369572000",  # Malignant tumor involving right ovary by separate metastasis from uterus (disorder)
        "369573005",  # Malignant tumor involving right ovary by separate metastasis from vagina (disorder)
        "369574004",  # Malignant tumor involving uterine cervix by separate metastasis from vagina (disorder)
        "369575003",  # Malignant tumor involving uterine corpus by separate metastasis from fallopian tube (disorder)
        "369576002",  # Malignant tumor involving uterine corpus by separate metastasis from ovary (disorder)
        "369577006",  # Malignant tumor involving uterine corpus by separate metastasis from uterine cervix (disorder)
        "369578001",  # Malignant tumor involving uterine corpus by separate metastasis from vagina (disorder)
        "369581006",  # Malignant tumor involving vagina by separate metastasis from bladder (disorder)
        "369582004",  # Malignant tumor involving vagina by separate metastasis from endometrium (disorder)
        "369583009",  # Malignant tumor involving vagina by separate metastasis from fallopian tube (disorder)
        "369584003",  # Malignant tumor involving vagina by separate metastasis from ovary (disorder)
        "369585002",  # Malignant tumor involving vagina by separate metastasis from uterine cervix (disorder)
        "369586001",  # Malignant tumor involving vagina by separate metastasis from uterus (disorder)
        "369588000",  # Malignant tumor involving vulva by separate metastasis from endometrium (disorder)
        "369589008",  # Malignant tumor involving vulva by separate metastasis from fallopian tube (disorder)
        "369590004",  # Malignant tumor involving vulva by separate metastasis from ovary (disorder)
        "369591000",  # Malignant tumor involving vulva by separate metastasis from uterine cervix (disorder)
        "369592007",  # Malignant tumor involving vulva by separate metastasis from uterus (disorder)
        "369593002",  # Malignant tumor involving vulva by separate metastasis from vagina (disorder)
        "369604009",  # Malignant tumor involving an organ by separate metastasis from fallopian tube (disorder)
        "369610009",  # Malignant tumor involving left fallopian tube by separate metastasis from uterine cervix (disorder)
        "403920004",  # Minimal deviation malignant melanoma (disorder)
        "413442004",  # Acute monocytic/monoblastic leukemia (disorder)
        "415285009",  # Myelodysplastic syndrome with multilineage dysplasia (disorder)
        "415286005",  # Myelodysplastic syndrome with ring sideroblasts and multilineage dysplasia (disorder)
        "422782004",  # Primary malignant neoplasm of ovary, with widespread metastatic disease (disorder)
        "423610004",  # Rhabdomyosarcoma of connective or soft tissue (disorder)
        "423987006",  # Primary malignant neoplasm of vulva, with widespread metastatic disease (disorder)
        "424887002",  # Primary malignant neoplasm of thyroid gland, metastatic to bone (disorder)
        "425749006",  # Subacute myeloid leukemia in remission (disorder)
        "449108003",  # Philadelphia chromosome positive chronic myelogenous leukemia (disorder)
        "449386007",  # Philadelphia chromosome negative chronic myelogenous leukemia (disorder)
        "60620005",  # Epsilon heavy chain disease (disorder)
        "698646006",  # Acute monoblastic leukemia in remission (disorder)
        "712849003",  # Primary malignant neoplasm of prostate metastatic to bone (disorder)
        "721303001",  # Refractory neutropenia (disorder)
        "721305008",  # Acute myeloid leukemia due to recurrent genetic abnormality (disorder)
        "734522002",  # Acute myeloid leukemia with FMS-like tyrosine kinase-3 mutation (disorder)
        "770402000",  # Aleukemic mast cell leukemia (disorder)
        "871843002",  # Malignant neoplasm of connective and soft tissue of sacrococcygeal region (disorder)
        "96981000119102",  # Primary malignant neoplasm of rectosigmoid junction metastatic to brain (disorder)
        "133841000119105",  # Merkel cell carcinoma of face (disorder)
        "133871000119103",  # Merkel cell carcinoma of upper limb (disorder)
        "133881000119100",  # Merkel cell carcinoma of lower limb (disorder)
        "254980001",  # Malignant tumor of acoustic vestibular nerve (disorder)
        "351461000119100",  # Merkel cell carcinoma of right lower limb (disorder)
        "352071000119105",  # Merkel cell carcinoma of left lower limb (disorder)
        "771080008",  # Hereditary site-specific ovarian cancer syndrome (disorder)
        "781076008",  # Metastatic malignant neoplasm to colon and/or rectum (disorder)
        "781382000",  # Malignant neoplasm of colon and/or rectum (disorder)
        "1080091000119100",  # Infiltrating ductal carcinoma of axillary tail of left female breast (disorder)
        "1080101000119105",  # Infiltrating duct carcinoma of left female breast (disorder)
        "1080111000119108",  # Infiltrating ductal carcinoma of central portion of left female breast (disorder)
        "1080121000119101",  # Infiltrating ductal carcinoma of lower inner quadrant of left female breast (disorder)
        "1080131000119103",  # Infiltrating ductal carcinoma of lower outer quadrant of left female breast (disorder)
        "1080151000119109",  # Infiltrating ductal carcinoma of upper inner quadrant of left female breast (disorder)
        "1080161000119106",  # Infiltrating ductal carcinoma of upper outer quadrant of left female breast (disorder)
        "1080171000119100",  # Infiltrating ductal carcinoma of axillary tail of right female breast (disorder)
        "1080181000119102",  # Infiltrating duct carcinoma of right female breast (disorder)
        "1080191000119104",  # Infiltrating ductal carcinoma of central portion of right female breast (disorder)
        "1080201000119101",  # Infiltrating ductal carcinoma of lower inner quadrant of right female breast (disorder)
        "1080211000119103",  # Infiltrating ductal carcinoma of lower outer quadrant of right female breast (disorder)
        "1080231000119108",  # Infiltrating ductal carcinoma of upper inner quadrant of right female breast (disorder)
        "1080241000119104",  # Infiltrating ductal carcinoma of upper outer quadrant of right female breast (disorder)
        "1080261000119100",  # Infiltrating lobular carcinoma of left female breast (disorder)
        "1080341000119105",  # Infiltrating lobular carcinoma of right female breast (disorder)
        "1081551000119106",  # Recurrent primary malignant neoplasm of left female breast (disorder)
        "1081561000119108",  # Recurrent primary malignant neoplasm of right female breast (disorder)
        "109886000",  # Overlapping malignant neoplasm of female breast (disorder)
        "109887009",  # Overlapping malignant neoplasm of male breast (disorder)
        "116691000119101",  # Marginal zone lymphoma of spleen (disorder)
        "123631000119103",  # Malignant poorly differentiated neuroendocrine carcinoma (disorder)
        "133531000119104",  # Malignant neuroendocrine tumor (disorder)
        "15635761000119103",  # Infiltrating duct carcinoma of bilateral female breasts (disorder)
        "15635801000119106",  # Primary malignant neoplasm of bilateral female breasts (disorder)
        "186192006",  # Respiratory tuberculosis, bacteriologically and histologically confirmed (disorder)
        "186201000",  # Primary respiratory tuberculosis, confirmed bacteriologically and histologically (disorder)
        "188147009",  # Malignant neoplasm of nipple and areola of female breast (disorder)
        "188151006",  # Malignant neoplasm of central part of female breast (disorder)
        "188152004",  # Malignant neoplasm of upper-inner quadrant of female breast (disorder)
        "188153009",  # Malignant neoplasm of lower-inner quadrant of female breast (disorder)
        "188154003",  # Malignant neoplasm of upper-outer quadrant of female breast (disorder)
        "188155002",  # Malignant neoplasm of lower-outer quadrant of female breast (disorder)
        "188156001",  # Malignant neoplasm of axillary tail of female breast (disorder)
        "188159008",  # Malignant neoplasm of ectopic site of female breast (disorder)
        "188163001",  # Malignant neoplasm of nipple and areola of male breast (disorder)
        "188168005",  # Malignant neoplasm of ectopic site of male breast (disorder)
        "188679001",  # Diffuse non-Hodgkin's lymphoma undifferentiated (diffuse) (disorder)
        "253003009",  # Carcinoid bronchial adenoma (disorder)
        "254633006",  # Oat cell carcinoma of lung (disorder)
        "255038009",  # Growth hormone releasing factor-secreting tumor (disorder)
        "255040004",  # Parathyroid hormone-related peptide-secreting tumor (disorder)
        "255042007",  # Gastric inhibitory peptide-secreting tumor (disorder)
        "303017006",  # Malignant lymphoma, convoluted cell type (disorder)
        "353421000119109",  # Primary malignant neoplasm of axillary tail of left female breast (disorder)
        "353431000119107",  # Primary malignant neoplasm of female left breast (disorder)
        "353441000119103",  # Primary malignant neoplasm of central portion of female left breast (disorder)
        "353501000119104",  # Primary malignant neoplasm of axillary tail of right female breast (disorder)
        "353511000119101",  # Primary malignant neoplasm of female right breast (disorder)
        "353521000119108",  # Primary malignant neoplasm of central portion of female right breast (disorder)
        "354491000119109",  # Primary malignant neoplasm of left male breast (disorder)
        "354591000119108",  # Primary malignant neoplasm of right male breast (disorder)
        "372064008",  # Malignant neoplasm of female breast (disorder)
        "372095001",  # Malignant neoplasm of male breast (disorder)
        "372096000",  # Carcinoma of male breast (disorder)
        "396198006",  # Small cell carcinoma of prostate (disorder)
        "39795003",  # Hand-Schuller-Christian disease (disorder)
        "445238008",  # Malignant carcinoid tumor (disorder)
        "447782002",  # Malignant epithelial neoplasm of female breast (disorder)
        "447784001",  # Sarcoma of axillary tail of female breast (disorder)
        "448257000",  # Sarcoma of male breast (disorder)
        "448388003",  # Sarcoma of lower outer quadrant of female breast (disorder)
        "448408001",  # Sarcoma of upper inner quadrant of female breast (disorder)
        "448435005",  # Sarcoma lower inner quadrant of female breast (disorder)
        "448436006",  # Sarcoma of central portion of female breast (disorder)
        "448449001",  # Sarcoma of female breast (disorder)
        "448451002",  # Sarcoma upper outer quadrant of female breast (disorder)
        "448952004",  # Infiltrating duct carcinoma of female breast (disorder)
        "45221000119105",  # Primary invasive malignant neoplasm of female breast (disorder)
        "699356008",  # Endometrial stromal sarcoma (disorder)
        "705176003",  # Metastatic carcinoid tumor (disorder)
        "709517003",  # Malignant carcinoid tumor of small intestine (disorder)
        "709830006",  # Malignant carcinoid tumor of stomach (disorder)
        "713573006",  # Malignant carcinoid tumor of rectum (disorder)
        "713574000",  # Malignant carcinoid tumor of kidney (disorder)
        "717921000",  # Poorly-differentiated neuroendocrine carcinoma of thymus (disorder)
        "717922007",  # Well-differentiated neuroendocrine carcinoma of thymus (disorder)
        "726652005",  # Malignant carcinoid tumor of thymus (disorder)
        "726653000",  # Malignant carcinoid tumor of bronchus (disorder)
        "726654006",  # Malignant carcinoid tumor of colon (disorder)
        "783006001",  # Low-grade neuroendocrine neoplasm of corpus uteri (disorder)
        "789004008",  # Parafollicular cell carcinoma (disorder)
        "93133006",  # Letterer-Siwe disease of intra-abdominal lymph nodes (disorder)
        "93134000",  # Letterer-Siwe disease of intrapelvic lymph nodes (disorder)
        "93135004",  # Letterer-Siwe disease of intrathoracic lymph nodes (disorder)
        "93139005",  # Letterer-Siwe disease of lymph nodes of multiple sites (disorder)
        "93140007",  # Letterer-Siwe disease of spleen (disorder)
        "93680004",  # Primary malignant neoplasm of areola of female breast (disorder)
        "93681000",  # Primary malignant neoplasm of areola of male breast (disorder)
        "93745008",  # Primary malignant neoplasm of central portion of female breast (disorder)
        "93776002",  # Primary malignant neoplasm of ectopic female breast tissue (disorder)
        "93777006",  # Primary malignant neoplasm of ectopic male breast tissue (disorder)
        "93796005",  # Primary malignant neoplasm of female breast (disorder)
        "93874009",  # Primary malignant neoplasm of lower inner quadrant of female breast (disorder)
        "93876006",  # Primary malignant neoplasm of lower outer quadrant of female breast (disorder)
        "93884005",  # Primary malignant neoplasm of male breast (disorder)
        "93924008",  # Primary malignant neoplasm of nipple of female breast (disorder)
        "93925009",  # Primary malignant neoplasm of nipple of male breast (disorder)
        "94115006",  # Primary malignant neoplasm of upper inner quadrant of female breast (disorder)
        "94117003",  # Primary malignant neoplasm of upper outer quadrant of female breast (disorder)
        "94176003",  # Metastatic malignant neoplasm to areola of female breast (disorder)
        "94177007",  # Metastatic malignant neoplasm to areola of male breast (disorder)
        "94182000",  # Metastatic malignant neoplasm to axillary tail of female breast (disorder)
        "94244003",  # Metastatic malignant neoplasm to central portion of female breast (disorder)
        "94276008",  # Metastatic malignant neoplasm to ectopic female breast tissue (disorder)
        "94277004",  # Metastatic malignant neoplasm to ectopic male breast tissue (disorder)
        "94297009",  # Metastatic malignant neoplasm to female breast (disorder)
        "94385006",  # Metastatic malignant neoplasm to lower inner quadrant of female breast (disorder)
        "94387003",  # Metastatic malignant neoplasm to lower outer quadrant of female breast (disorder)
        "94401004",  # Metastatic malignant neoplasm to male breast (disorder)
        "94443006",  # Metastatic malignant neoplasm to nipple of female breast (disorder)
        "94444000",  # Metastatic malignant neoplasm to nipple of male breast (disorder)
        "94653000",  # Metastatic malignant neoplasm to upper inner quadrant of female breast (disorder)
        "94655007",  # Metastatic malignant neoplasm to upper outer quadrant of female breast (disorder)
        "96291000119105",  # Primary malignant inflammatory neoplasm of female breast (disorder)
        "10041001",  # Tracheal stenosis following tracheostomy (disorder)
        "100721000119109",  # High grade astrocytoma of brain (disorder)
        "100731000119107",  # Low grade astrocytoma of brain (disorder)
        "1010603001",  # Primary mucoepidermoid carcinoma of nasopharynx (disorder)
        "1010607000",  # Primary lymphoepithelial carcinoma of nasopharynx (disorder)
        "1010624006",  # Primary salivary gland type neoplasm of nasopharynx (disorder)
        "1010631005",  # Primary malignant melanoma of overlapping sites of accessory sinuses (disorder)
        "1010633008",  # Primary papillary adenocarcinoma of nasopharynx (disorder)
        "1010656004",  # Adenoid squamous cell carcinoma of nasopharynx (disorder)
        "1010669000",  # Primary squamous cell carcinoma of superior wall of nasopharynx (disorder)
        "1010674008",  # Basaloid squamous cell carcinoma of nasopharynx (disorder)
        "1010675009",  # Squamous cell carcinoma of posterior wall of nasopharynx (disorder)
        "1010677001",  # Primary squamous cell carcinomas of anterior wall of nasopharynx (disorder)
        "1010679003",  # Primary squamous cell carcinoma of lateral wall of nasopharynx (disorder)
        "1010681001",  # Primary squamous cell carcinoma of overlapping sites of nasopharynx (disorder)
        "101371000119101",  # Adamantinoma of long bone of lower limb (disorder)
        "103511000119103",  # Melanoma carcinomatosis (disorder)
        "104981000119104",  # Oligodendroglioma of cerebrum (disorder)
        "105041000119109",  # Metastatic squamous cell carcinoma to lung (disorder)
        "105051000119106",  # Metastatic undifferentiated large cell carcinoma to lung (disorder)
        "105111000119109",  # Primary squamous cell carcinoma of thymus (disorder)
        "105121000119102",  # Squamous cell carcinoma of vagina (disorder)
        "10528009",  # Lupus vulgaris (disorder)
        "10706006",  # Tuberculosis of central nervous system (disorder)
        "10708511000119108",  # Primary malignant neoplasm of uterus (disorder)
        "10737361000119109",  # Primary adenocarcinoma of right fallopian tube (disorder)
        "10737401000119100",  # Primary adenocarcinoma of left fallopian tube (disorder)
        "10737521000119108",  # Primary mucinous cystadenocarcinoma of right ovary (disorder)
        "10737561000119103",  # Primary mucinous cystadenocarcinoma of left ovary (disorder)
        "10737601000119103",  # Primary serous papillary cystadenocarcinoma of right ovary (disorder)
        "10737641000119101",  # Primary serous papillary cystadenocarcinoma of left ovary (disorder)
        "10737781000119101",  # Primary endometrioid carcinoma of right ovary (disorder)
        "10737821000119106",  # Primary endometrioid carcinoma of left ovary (disorder)
        "10737861000119101",  # Malignant germ cell neoplasm of right ovary (disorder)
        "10737911000119105",  # Malignant germ cell neoplasm of left ovary (disorder)
        "10745291000119103",  # Malignant neoplastic disease in mother complicating childbirth (disorder)
        "10749871000119100",  # Malignant neoplastic disease in pregnancy (disorder)
        "107571000119101",  # Oligodendroglioma of brain stem (disorder)
        "107581000119103",  # Astrocytoma of brain stem (disorder)
        "107591000119100",  # Primary squamous cell carcinoma of urethra (disorder)
        "107601000119107",  # Primary transitional cell carcinoma of urethra (disorder)
        "107691000119101",  # Non-seminomatous germ cell neoplasm of testis (disorder)
        "107771000119106",  # Primary malignant clear cell neoplasm of endometrium (disorder)
        "107791000119107",  # Primary adenosquamous carcinoma of endometrium (disorder)
        "107811000119106",  # Metastatic small cell carcinoma to liver (disorder)
        "107831000119101",  # Metastatic squamous cell carcinoma to liver (disorder)
        "1078881000119102",  # Primary adenocarcinoma of lower lobe of left lung (disorder)
        "1078891000119104",  # Primary adenocarcinoma of left main bronchus (disorder)
        "1078901000119100",  # Primary adenocarcinoma of upper lobe of left lung (disorder)
        "1078931000119107",  # Primary adenocarcinoma of lower lobe of right lung (disorder)
        "1078941000119103",  # Primary adenocarcinoma of right main bronchus (disorder)
        "1078951000119101",  # Primary adenocarcinoma of middle lobe of right lung (disorder)
        "1078961000119104",  # Primary adenocarcinoma of upper lobe of right lung (disorder)
        "1078971000119105",  # Adenoid cystic carcinoma of left lacrimal gland (disorder)
        "1078981000119108",  # Adenoid cystic carcinoma of right lacrimal gland (disorder)
        "1079011000119109",  # Primary angiosarcoma of left upper limb (disorder)
        "1079021000119102",  # Primary angiosarcoma of left hip region (disorder)
        "1079031000119104",  # Primary angiosarcoma of left lower limb (disorder)
        "1079051000119105",  # Primary angiosarcoma of right upper limb (disorder)
        "1079061000119107",  # Primary angiosarcoma of right hip region (disorder)
        "1079071000119101",  # Primary angiosarcoma of right lower limb (disorder)
        "1079091000119100",  # Primary basal cell carcinoma of left upper limb (disorder)
        "1079101000119105",  # Primary basal cell carcinoma of left ear (disorder)
        "1079111000119108",  # Primary basal cell carcinoma of left eyelid (disorder)
        "1079121000119101",  # Primary basal cell carcinoma of skin of left hip (disorder)
        "1079131000119103",  # Primary basal cell carcinoma of left lower limb (disorder)
        "1079141000119107",  # Primary basal cell carcinoma of skin of left shoulder (disorder)
        "1079151000119109",  # Primary basal cell carcinoma of right upper limb (disorder)
        "1079161000119106",  # Primary basal cell carcinoma of right ear (disorder)
        "1079171000119100",  # Primary basal cell carcinoma of right eyelid (disorder)
        "1079181000119102",  # Primary basal cell carcinoma of skin of right hip (disorder)
        "1079191000119104",  # Primary basal cell carcinoma of right lower limb (disorder)
        "1079201000119101",  # Primary basal cell carcinoma of skin of right shoulder (disorder)
        "1079531000119106",  # Metastatic malignant neoplasm to left kidney (disorder)
        "1079541000119102",  # Metastatic malignant neoplasm to right kidney (disorder)
        "1079551000119100",  # Primary chondrosarcoma of bone of left upper limb (disorder)
        "1079561000119103",  # Primary chondrosarcoma of bone of left foot (disorder)
        "1079571000119109",  # Primary chondrosarcoma of bone of left hand (disorder)
        "1079581000119107",  # Primary chondrosarcoma of bone of left lower limb (disorder)
        "1079591000119105",  # Primary chondrosarcoma of left scapula (disorder)
        "1079601000119103",  # Primary chondrosarcoma of bone of right upper limb (disorder)
        "1079611000119100",  # Primary chondrosarcoma of bone of right foot (disorder)
        "1079621000119107",  # Primary chondrosarcoma of bone of right hand (disorder)
        "1079631000119105",  # Primary chondrosarcoma of bone of right lower limb (disorder)
        "1079641000119101",  # Primary chondrosarcoma of right scapula (disorder)
        "1079991000119101",  # Primary Ewing sarcoma of bone of left upper limb (disorder)
        "1080001000119106",  # Primary Ewing sarcoma of bone of left foot (disorder)
        "1080021000119102",  # Primary Ewing sarcoma of bone of left lower limb (disorder)
        "1080041000119108",  # Primary Ewing sarcoma of bone of right upper limb (disorder)
        "1080051000119105",  # Primary Ewing sarcoma of bone of right foot (disorder)
        "1080071000119101",  # Primary Ewing sarcoma of bone of right lower limb (disorder)
        "1080451000119101",  # Primary large cell carcinoma of lower lobe of left lung (disorder)
        "1080471000119105",  # Primary large cell carcinoma of upper lobe of left lung (disorder)
        "1080501000119104",  # Primary large cell carcinoma of lower lobe of right lung (disorder)
        "1080531000119106",  # Primary large cell carcinoma of upper lobe of right lung (disorder)
        "1080541000119102",  # Primary leiomyosarcoma of left upper limb (disorder)
        "1080551000119100",  # Primary leiomyosarcoma of left hip region (disorder)
        "1080561000119103",  # Primary leiomyosarcoma of left kidney (disorder)
        "1080571000119109",  # Primary leiomyosarcoma of left lower limb (disorder)
        "1080581000119107",  # Primary leiomyosarcoma of left shoulder region (disorder)
        "1080591000119105",  # Primary leiomyosarcoma of right upper limb (disorder)
        "1080601000119103",  # Primary leiomyosarcoma of right hip region (disorder)
        "1080611000119100",  # Primary leiomyosarcoma of right kidney (disorder)
        "1080621000119107",  # Primary leiomyosarcoma of right lower limb (disorder)
        "1080631000119105",  # Primary leiomyosarcoma of right shoulder region (disorder)
        "1080681000119106",  # Primary liposarcoma of soft tissue of left upper limb (disorder)
        "1080691000119109",  # Primary liposarcoma of soft tissue of left hip (disorder)
        "1080701000119109",  # Primary liposarcoma of soft tissue of left lower limb (disorder)
        "1080711000119107",  # Primary liposarcoma of soft tissue of left shoulder (disorder)
        "1080721000119100",  # Primary liposarcoma of soft tissue of right upper limb (disorder)
        "1080731000119102",  # Primary liposarcoma of soft tissue of right hip (disorder)
        "1080741000119106",  # Primary liposarcoma of soft tissue of right lower limb (disorder)
        "1080751000119108",  # Primary liposarcoma of soft tissue of right shoulder (disorder)
        "1080941000119109",  # Malignant melanoma of left choroid (disorder)
        "1080981000119104",  # Malignant melanoma of right choroid (disorder)
        "108101000119101",  # Malignant melanoma metastatic to kidney (disorder)
        "1081271000119100",  # Primary osteosarcoma of bone of left upper limb (disorder)
        "1081281000119102",  # Primary osteosarcoma of bone of left foot (disorder)
        "1081291000119104",  # Primary osteosarcoma of bone of left hand (disorder)
        "1081301000119103",  # Primary osteosarcoma of bone of left lower limb (disorder)
        "108131000119108",  # Small cell carcinoma metastatic to kidney (disorder)
        "1081311000119100",  # Primary osteosarcoma of left scapula (disorder)
        "1081321000119107",  # Primary osteosarcoma of bone of right upper limb (disorder)
        "1081331000119105",  # Primary osteosarcoma of bone of right foot (disorder)
        "1081341000119101",  # Primary osteosarcoma of bone of right hand (disorder)
        "1081351000119104",  # Primary osteosarcoma of bone of right lower limb (disorder)
        "1081361000119102",  # Primary osteosarcoma of right scapula (disorder)
        "1081571000119102",  # Clear cell carcinoma of left kidney (disorder)
        "1081581000119104",  # Clear cell carcinoma of right kidney (disorder)
        "1081591000119101",  # Primary retinoblastoma of left retina (disorder)
        "1081601000119108",  # Primary retinoblastoma of right retina (disorder)
        "1081611000119106",  # Primary rhabdomyosarcoma of left upper limb (disorder)
        "1081631000119101",  # Primary rhabdomyosarcoma of left lower limb (disorder)
        "1081651000119107",  # Primary rhabdomyosarcoma of right upper limb (disorder)
        "1081671000119103",  # Primary rhabdomyosarcoma of right lower limb (disorder)
        "1081691000119102",  # Primary sarcoma of soft tissues of left upper limb (disorder)
        "1081701000119102",  # Primary sarcoma of soft tissues of left hip (disorder)
        "1081711000119104",  # Primary sarcoma of left lower limb (disorder)
        "1081721000119106",  # Primary sarcoma of soft tissues of left shoulder (disorder)
        "1081731000119109",  # Primary sarcoma of soft tissues of right upper limb (disorder)
        "1081741000119100",  # Primary sarcoma of soft tissues of right hip (disorder)
        "1081751000119103",  # Primary sarcoma of right lower limb (disorder)
        "1081761000119101",  # Primary sarcoma of soft tissues of right shoulder (disorder)
        "1081781000119105",  # Primary seminoma of left testis (disorder)
        "1081811000119107",  # Primary seminoma of right testis (disorder)
        "1081841000119106",  # Primary small cell carcinoma of lower lobe of left lung (disorder)
        "1081851000119108",  # Primary small cell carcinoma of left main bronchus (disorder)
        "1081861000119105",  # Primary small cell carcinoma of upper lobe of left lung (disorder)
        "1081891000119103",  # Primary small cell carcinoma of lower lobe of right lung (disorder)
        "1081901000119104",  # Primary small cell carcinoma of right main bronchus (disorder)
        "1081911000119101",  # Primary small cell carcinoma of middle lobe of right lung (disorder)
        "1081921000119108",  # Primary small cell carcinoma of upper lobe of right lung (disorder)
        "108201000119105",  # Undifferentiated large cell carcinoma metastatic to kidney (disorder)
        "1082051000119106",  # Primary squamous cell carcinoma of skin of left upper limb (disorder)
        "1082061000119108",  # Primary squamous cell carcinoma of conjunctiva of left eye (disorder)
        "1082071000119102",  # Primary squamous cell carcinoma of left ear (disorder)
        "1082091000119101",  # Primary squamous cell carcinoma of skin of left hip (disorder)
        "1082101000119106",  # Primary squamous cell carcinoma of skin of left lower limb (disorder)
        "108211000119108",  # Secondary adenocarcinoma of kidney (disorder)
        "1082111000119109",  # Primary squamous cell carcinoma of lower lobe of left lung (disorder)
        "1082121000119102",  # Primary squamous cell carcinoma of left main bronchus (disorder)
        "1082131000119104",  # Primary squamous cell carcinoma of skin of left shoulder (disorder)
        "1082141000119108",  # Primary squamous cell carcinoma of upper lobe of left lung (disorder)
        "1082171000119101",  # Primary squamous cell carcinoma of skin of right upper limb (disorder)
        "1082181000119103",  # Primary squamous cell carcinoma of conjunctiva of right eye (disorder)
        "1082191000119100",  # Primary squamous cell carcinoma of right ear (disorder)
        "1082211000119104",  # Primary squamous cell carcinoma of skin of right hip (disorder)
        "1082221000119106",  # Primary squamous cell carcinoma of skin of right lower limb (disorder)
        "1082231000119109",  # Primary squamous cell carcinoma of lower lobe of right lung (disorder)
        "1082241000119100",  # Primary squamous cell carcinoma of right main bronchus (disorder)
        "1082251000119103",  # Primary squamous cell carcinoma of middle lobe of right lung (disorder)
        "1082261000119101",  # Primary squamous cell carcinoma of skin of right shoulder (disorder)
        "1082271000119107",  # Primary squamous cell carcinoma of upper lobe of right lung (disorder)
        "1082281000119105",  # Transitional cell carcinoma of left renal pelvis (disorder)
        "1082291000119108",  # Transitional cell carcinoma of left ureter (disorder)
        "1082301000119109",  # Transitional cell carcinoma of right renal pelvis (disorder)
        "1082311000119107",  # Transitional cell carcinoma of right ureter (disorder)
        "1082321000119100",  # Nephroblastoma of left kidney (disorder)
        "1082331000119102",  # Nephroblastoma of right kidney (disorder)
        "1082601000112109",  # Metastatic malignant neoplasm to viscera (disorder)
        "1082701000112100",  # Locally advanced breast cancer (disorder)
        "10893003",  # Tuberculous adenitis (disorder)
        "109264009",  # Overlapping malignant neoplasm of skin (disorder)
        "1092771000119104",  # Tuberculous neuritis (disorder)
        "109347009",  # Overlapping malignant neoplasm of bone and articular cartilage (disorder)
        "109348004",  # Overlapping malignant neoplasm of bone and articular cartilage of limb (disorder)
        "109349007",  # Overlapping malignant neoplasm of soft tissues (disorder)
        "109359008",  # Primary malignant neoplasm of independent multiple sites (disorder)
        "109366009",  # Overlapping malignant neoplasm of accessory sinuses (disorder)
        "109367000",  # Overlapping malignant neoplasm of nasopharynx (disorder)
        "109368005",  # Overlapping malignant neoplasm of hypopharynx (disorder)
        "109369002",  # Overlapping malignant neoplasm of larynx (disorder)
        "109370001",  # Primary malignant neoplasm of laryngeal cartilage (disorder)
        "109371002",  # Overlapping malignant neoplasm of bronchus and lung (disorder)
        "109374005",  # Overlapping malignant neoplasm of mediastinum and pleura (disorder)
        "109378008",  # Mesothelioma (malignant, clinical disorder) (disorder)
        "109383000",  # Malignant mesothelioma of pericardium (disorder)
        "109384006",  # Overlapping malignant neoplasm of heart, mediastinum and pleura (disorder)
        "109385007",  # Kaposi's sarcoma (disorder)
        "109386008",  # Kaposi's sarcoma of skin (disorder)
        "109388009",  # Kaposi's sarcoma of palate (disorder)
        "109389001",  # Kaposi's sarcoma of gastrointestinal tract (disorder)
        "109390005",  # Kaposi's sarcoma of lung (disorder)
        "109391009",  # Kaposi's sarcoma of lymph nodes (disorder)
        "109392002",  # Kaposi's sarcoma of multiple organs (disorder)
        "10976631000119101",  # Leiomyosarcoma of skin of chest (disorder)
        "10976711000119104",  # Primary primitive neuroectodermal neoplasm of central nervous system (disorder)
        "109821008",  # Overlapping malignant neoplasm of gastrointestinal tract (disorder)
        "109822001",  # Overlapping malignant neoplasm of lip (disorder)
        "109823006",  # Overlapping malignant neoplasm of tongue (disorder)
        "109824000",  # Overlapping malignant neoplasm of major salivary gland (disorder)
        "109828002",  # Primary malignant neoplasm of salivary gland duct (disorder)
        "109830000",  # Overlapping malignant neoplasm of floor of mouth (disorder)
        "109831001",  # Overlapping malignant neoplasm of palate (disorder)
        "109832008",  # Overlapping malignant neoplasm of oropharynx (disorder)
        "109833003",  # Overlapping malignant neoplasm of lip, oral cavity and/or pharynx (disorder)
        "109834009",  # Primary malignant neoplasm of branchial cleft (disorder)
        "109835005",  # Overlapping malignant neoplasm of esophagus (disorder)
        "109836006",  # Overlapping malignant neoplasm of stomach (disorder)
        "109837002",  # Overlapping malignant neoplasm of small intestine (disorder)
        "109838007",  # Overlapping malignant neoplasm of colon (disorder)
        "109839004",  # Overlapping malignant neoplasm of rectum, anus and anal canal (disorder)
        "109840002",  # Primary malignant neoplasm of cloacogenic zone (disorder)
        "109841003",  # Liver cell carcinoma (disorder)
        "109842005",  # Intrahepatic bile duct carcinoma (disorder)
        "109843000",  # Hepatoblastoma (disorder)
        "109844006",  # Angiosarcoma of liver (disorder)
        "109847004",  # Overlapping malignant neoplasm of biliary tract (disorder)
        "109848009",  # Overlapping malignant neoplasm of pancreas (disorder)
        "109851002",  # Overlapping malignant neoplasm of retroperitoneum and peritoneum (disorder)
        "109853004",  # Malignant mesothelioma of peritoneum (disorder)
        "109854005",  # Malignant mesothelioma of parietal peritoneum (disorder)
        "109855006",  # Malignant mesothelioma of pelvic peritoneum (disorder)
        "109856007",  # Malignant mesothelioma of mesentery (disorder)
        "109858008",  # Malignant mesothelioma of omentum (disorder)
        "109870007",  # Overlapping malignant neoplasm of urinary system (disorder)
        "109874003",  # Overlapping malignant neoplasm of male genital organs (disorder)
        "109875002",  # Overlapping malignant neoplasm of penis (disorder)
        "109876001",  # Primary malignant neoplasm of descended testis (disorder)
        "109878000",  # Overlapping malignant neoplasm of female genital organs (disorder)
        "109879008",  # Overlapping malignant neoplasm of body of uterus (disorder)
        "109880006",  # Overlapping malignant neoplasm of uterine cervix (disorder)
        "109882003",  # Primary malignant neoplasm of fundus uteri (disorder)
        "109885001",  # Overlapping malignant neoplasm of vulva (disorder)
        "1098981000119101",  # Recurrent malignant neoplasm of prostate (disorder)
        "109911004",  # Overlapping malignant neoplasm of brain and other parts of the central nervous system (disorder)
        "109912006",  # Overlapping malignant neoplasm of brain (disorder)
        "109915008",  # Primary malignant neoplasm of meninges (disorder)
        "109918005",  # Primary malignant neoplasm of peripheral nerves and peripheral autonomic nervous system (disorder)
        "109919002",  # Overlapping malignant neoplasm of peripheral nerves and autonomic nervous system (disorder)
        "109921007",  # Primary malignant neoplasm of peripheral nerve of head, face and/or neck (disorder)
        "109923005",  # Primary malignant neoplasm of peripheral nerves of head (disorder)
        "109925003",  # Primary malignant neoplasm of peripheral nerves of face (disorder)
        "109927006",  # Primary malignant neoplasm of peripheral nerves of neck (disorder)
        "109931000",  # Primary malignant neoplasm of peripheral nerves of upper limb (disorder)
        "109933002",  # Primary malignant neoplasm of peripheral nerves of shoulder (disorder)
        "109937001",  # Primary malignant neoplasm of peripheral nerves of lower limb (disorder)
        "109939003",  # Primary malignant neoplasm of peripheral nerves of hip (disorder)
        "109941002",  # Primary malignant neoplasm of peripheral nerves of thorax (disorder)
        "109943004",  # Primary malignant neoplasm of peripheral nerves of abdomen (disorder)
        "109945006",  # Primary malignant neoplasm of peripheral nerves of pelvis region (disorder)
        "109947003",  # Primary malignant neoplasm of peripheral nerves of trunk (disorder)
        "109948008",  # Overlapping malignant neoplasm of eye and adnexa, primary (disorder)
        "109951001",  # Overlapping malignant neoplasm of multiple endocrine glands (disorder)
        "1099571000119100",  # Primary malignant neoplasm of left lacrimal sac (disorder)
        "1099581000119102",  # Primary malignant neoplasm of right lacrimal sac (disorder)
        "109962001",  # Diffuse non-Hodgkin's lymphoma (disorder)
        "109964000",  # Diffuse non-Hodgkin's lymphoma, undifferentiated (disorder)
        "109966003",  # Diffuse non-Hodgkin's lymphoma, immunoblastic (disorder)
        "109967007",  # Diffuse non-Hodgkin's lymphoma, small cleaved cell (disorder)
        "109968002",  # Diffuse non-Hodgkin's lymphoma, small cell (disorder)
        "109969005",  # Diffuse large B-cell malignant lymphoma (disorder)
        "109970006",  # Follicular non-Hodgkin's lymphoma, small cleaved cell (disorder)
        "109971005",  # Follicular non-Hodgkin's lymphoma, mixed small cleaved cell and large cell (disorder)
        "109972003",  # Follicular non-Hodgkin's lymphoma, large cell (disorder)
        "109975001",  # T-zone lymphoma (disorder)
        "109976000",  # Lymphoepithelioid lymphoma (disorder)
        "109977009",  # Peripheral T-cell lymphoma (disorder)
        "109978004",  # T-cell lymphoma (disorder)
        "109979007",  # B-cell lymphoma (disorder)
        "109980005",  # Malignant immunoproliferative disease (disorder)
        "109982002",  # Alpha heavy chain disease (disorder)
        "109984001",  # Gamma heavy chain disease (disorder)
        "109985000",  # Immunoproliferative small intestinal disease (disorder)
        "109988003",  # Histiocytic sarcoma (disorder)
        "109989006",  # Multiple myeloma (disorder)
        "109991003",  # Acute panmyelosis with myelofibrosis (disorder)
        "109992005",  # Polycythemia vera (disorder)
        "109994006",  # Essential thrombocythemia (disorder)
        "109995007",  # Myelodysplastic syndrome (disorder)
        "109998009",  # Myelodysplastic syndrome with ring sideroblasts and single lineage dysplasia (disorder)
        "110002002",  # Mast cell leukemia (disorder)
        "110004001",  # Acute promyelocytic leukemia, FAB M3 (disorder)
        "110005000",  # Acute myelomonocytic leukemia, FAB M4 (disorder)
        "110006004",  # Prolymphocytic leukemia (disorder)
        "110007008",  # Adult T-cell leukemia/lymphoma (disorder)
        "110013004",  # Overlapping malignant neoplasm of tonsil (disorder)
        "11010461000119101",  # Small cell carcinoma (disorder)
        "111466004",  # Perinatal partial atelectasis (disorder)
        "111825008",  # Tuberculosis of conjunctiva (disorder)
        "111830007",  # Tuberculosis of urinary organs (disorder)
        "111832004",  # Primary progressive tuberculosis (disorder)
        "1137644008",  # Leiomyosarcoma of fundus of stomach (disorder)
        "1141621000",  # Malignant epithelial neoplasm of pyriform fossa (disorder)
        "1141626005",  # Adenocarcinoma of pancreas with neuregulin 1 gene fusion (disorder)
        "1141627001",  # Non-small cell lung carcinoma with neuregulin 1 gene fusion (disorder)
        "1144252005",  # Malignant neoplasm of lateral border of tongue (disorder)
        "1144303008",  # Leiomyosarcoma of pylorus of stomach (disorder)
        "1144304002",  # Leiomyosarcoma of pyloric antrum of stomach (disorder)
        "1144305001",  # Leiomyosarcoma of gastric corpus structure (disorder)
        "1144306000",  # Leiomyosarcoma of lesser curvature of stomach (disorder)
        "1144307009",  # Leiomyosarcoma of greater curvature of stomach (disorder)
        "1144499001",  # Tuberculosis of pelvis and femur (disorder)
        "1144764001",  # Malignant neoplasm of esophagus with neuregulin 1 gene fusion (disorder)
        "1144891007",  # Primary cholangiocarcinoma of extrahepatic bile duct (disorder)
        "1144922006",  # Thymoma type AB (disorder)
        "1144928005",  # Thymoma type A (disorder)
        "1144945000",  # Adnexal carcinoma of vulva (disorder)
        "1145245004",  # Carcinoma of meibomian gland (disorder)
        "11471000224106",  # Diffuse intrinsic pontine glioma (disorder)
        "1148551007",  # Infiltrating duct carcinoma of pancreas (disorder)
        "1148845007",  # Follicular lymphoma grade 3b (disorder)
        "1148847004",  # Follicular non-Hodgkin's lymphoma in situ (disorder)
        "1148851002",  # Follicular lymphoma grade 3a (disorder)
        "1148852009",  # Malignant cystic nephroma (disorder)
        "1148901006",  # Squamous cell carcinoma of male genital (disorder)
        "1148904003",  # Spermatocytic seminoma of testis (disorder)
        "1148906001",  # Acute myeloid leukemia with t(8;21)(q22;q22) RUNX1-RUNX1T1 (disorder)
        "1148931000",  # Cystic renal cell carcinoma of kidney (disorder)
        "1149259003",  # Wound sinus due to and following tracheostomy (disorder)
        "1153336009",  # Pediatric nodal marginal zone B cell lymphoma (disorder)
        "1153346006",  # Primary non-Hodgkin malignant lymphoma of uveal tract (disorder)
        "1153347002",  # Primary non-Hodgkin malignant lymphoma of vitreoretinal tract (disorder)
        "1153348007",  # Primary effusion lymphoma due to human immune deficiency virus infection (disorder)
        "1153349004",  # Primary familial polycythemia due to erythropoietin receptor mutation (disorder)
        "1153354008",  # Malignant lymphoma of duodenum (disorder)
        "1153355009",  # Malignant lymphoma of esophagus (disorder)
        "1153356005",  # Lymphoma of anus (disorder)
        "1153357001",  # Lymphoma of appendix (disorder)
        "1153377008",  # Mixed phenotype acute leukemia with t(9;22) (q34;q11.2); BCR-ABL1 (disorder)
        "1153379006",  # Mixed phenotype acute leukemia with T-cell and myeloid lineage (disorder)
        "1153381008",  # Mixed phenotype acute leukemia with myeloid and B-cell lymphoid phenotypes (disorder)
        "1153382001",  # Malignant lymphoma of lacrimal gland (disorder)
        "1153383006",  # Malignant lymphoma of liver (disorder)
        "1153413001",  # Acute sequestration of spleen due to sickle cell thalassemia with crisis (disorder)
        "1153415008",  # Acute sequestration of spleen due to sickle cell hemoglobin C disease with crisis (disorder)
        "1153421007",  # Chondrosarcoma of sternum (disorder)
        "1153422000",  # Chondrosarcoma of mandible (disorder)
        "1153423005",  # Chondrosarcoma of rib (disorder)
        "1153424004",  # Chondrosarcoma of bone of pelvic wall (disorder)
        "1153425003",  # Chondrosarcoma of clavicle (disorder)
        "1153426002",  # Chondrosarcoma of skull (disorder)
        "1153427006",  # Chondrosarcoma of vertebral column (disorder)
        "1155991005",  # Metastatic malignant neoplasm to leptomeninges (disorder)
        "1156403002",  # Composite Hodgkin and non-Hodgkin lymphoma (disorder)
        "1156404008",  # Polyembryoma (disorder)
        "1156406005",  # Anaplastic oligodendroglioma of central nervous system (disorder)
        "1156409003",  # Anaplastic ependymoma of central nervous system (disorder)
        "1156410008",  # Anaplastic oligoastrocytoma of central nervous system (disorder)
        "1156413005",  # Gliomatosis cerebri (disorder)
        "1156414004",  # Giant cell glioblastoma of brain (disorder)
        "1156415003",  # Protoplasmic astrocytoma of brain (disorder)
        "1156416002",  # Gemistocytic astrocytoma of brain (disorder)
        "1156417006",  # Fibrillary astrocytoma of brain (disorder)
        "1156418001",  # Malignant rhabdoid tumor (disorder)
        "1156420003",  # Desmoplastic small round cell tumor (disorder)
        "1156452003",  # Mixed germ cell tumor (disorder)
        "1156453008",  # Chordoma (disorder)
        "1156454002",  # Embryonal carcinoma (disorder)
        "1156455001",  # Pleomorphic xanthoastrocytoma of brain (disorder)
        "1156456000",  # Pilomyxoid astrocytoma of brain (disorder)
        "1156459007",  # Anaplastic ganglioglioma of central nervous system (disorder)
        "1156460002",  # Desmoplastic nodular medulloblastoma of brain (disorder)
        "1156461003",  # Gliosarcoma of brain (disorder)
        "1156469001",  # Large cell medulloblastoma of brain (disorder)
        "1156471001",  # Choroid plexus carcinoma (disorder)
        "1156472008",  # Papillary tumor of pineal region (disorder)
        "1156495004",  # Dendritic cell sarcoma (disorder)
        "1156783003",  # Leiomyosarcoma of rectum (disorder)
        "1156785005",  # Leiomyosarcoma of duodenum (disorder)
        "1156786006",  # Leiomyosarcoma of esophagus (disorder)
        "1156788007",  # Leiomyosarcoma of colon (disorder)
        "1156795003",  # Kaposi sarcoma of colon (disorder)
        "1156797006",  # Kaposi sarcoma of rectum (disorder)
        "1156804004",  # Peripheral neuroectodermal neoplasm of corpus uteri (disorder)
        "1156806002",  # Primary malignant neoplasm of musculoskeletal system (disorder)
        "1156807006",  # Peripheral neuroectodermal neoplasm of cervix uteri (disorder)
        "1156809009",  # Malignant mesenchymal neoplasm of duodenum (disorder)
        "1156810004",  # Malignant mesenchymal neoplasm of esophagus (disorder)
        "1156811000",  # Malignant mesenchymal neoplasm of anus (disorder)
        "1156812007",  # Malignant mesenchymal neoplasm of appendix (disorder)
        "1156833003",  # Germinoma of central nervous system (disorder)
        "1157060001",  # Diffuse astrocytoma of brain (disorder)
        "1157061002",  # Gliosarcoma of central nervous system (disorder)
        "1157062009",  # Giant cell glioblastoma of central nervous system (disorder)
        "1157063004",  # Malignant neoplasm of vertebral column region (disorder)
        "1157064005",  # Gemistocytic astrocytoma of central nervous system (disorder)
        "1157067003",  # Malignant neoplasm of vertebra (disorder)
        "1157068008",  # Fibrillary astrocytoma of central nervous system (disorder)
        "1157070004",  # Pilomyxoid astrocytoma (disorder)
        "1157071000",  # Anaplastic ganglioglioma (disorder)
        "1157072007",  # Gliosarcoma of spinal cord (disorder)
        "1157141006",  # Astroblastoma of central nervous system (disorder)
        "1157157006",  # Acute myeloid leukemia with 11q23 abnormality (disorder)
        "1157162007",  # Intravascular large B-cell lymphoma (disorder)
        "1162264008",  # Malignant lymphoma of uveal tract (disorder)
        "1162280003",  # Primary rhabdomyosarcoma of isthmus of uterine body (disorder)
        "1162282006",  # Primary rhabdomyosarcoma of myometrium of corpus uteri (disorder)
        "1162284007",  # Primary rhabdomyosarcoma of endometrium of corpus uteri (disorder)
        "1162286009",  # Primary rhabdomyosarcoma of fundus uteri (disorder)
        "1162311007",  # Sarcoma of body of uterus (disorder)
        "1162346004",  # Osteosarcoma of sternum (disorder)
        "1162348003",  # Osteosarcoma of bone of vertebra (disorder)
        "1162349006",  # Osteosarcoma of bone of clavicle (disorder)
        "1162351005",  # Osteosarcoma of bone of rib (disorder)
        "1162352003",  # Osteosarcoma of articular cartilage of sternum (disorder)
        "1162353008",  # Osteosarcoma of articular cartilage of vertebra (disorder)
        "1162354002",  # Osteosarcoma of articular cartilage of rib (disorder)
        "1162361003",  # Osteosarcoma of articular cartilage of clavicle (disorder)
        "1162531007",  # Acute myeloid leukemia in complete remission (disorder)
        "1162855005",  # Carcinoma of gallbladder and extrahepatic biliary tract (disorder)
        "1163357003",  # Malignant neoplasm of oral cavity and lip and salivary gland (disorder)
        "1163568002",  # Primary malignant neoplasm of overlapping sites of colon (disorder)
        "116381000119105",  # Ganglioneuroblastoma (disorder)
        "116711000119103",  # Marginal zone lymphoma of inguinal lymph node (disorder)
        "116741000119104",  # Marginal zone lymphoma of thoracic lymph node (disorder)
        "11676005",  # Tuberculous leptomeningitis (disorder)
        "116811000119106",  # Non-Hodgkin lymphoma of central nervous system metastatic to lymph node of lower limb (disorder)
        "116821000119104",  # Non-Hodgkin lymphoma of central nervous system metastatic to lymph node of upper limb (disorder)
        "116841000119105",  # Marginal zone lymphoma of lymph nodes of multiple sites (disorder)
        "116871000119103",  # Mantle cell lymphoma of lymph nodes of multiple sites (disorder)
        "11689004",  # Linear atelectasis (disorder)
        "117061000119101",  # Peripheral T-cell lymphoma of spleen (disorder)
        "117091000119108",  # Peripheral T-cell lymphoma of axillary lymph node (disorder)
        "117111000119100",  # Peripheral T-cell lymphoma of thoracic lymph node (disorder)
        "11717361000119108",  # Metastatic malignant neoplasm to soft tissue of back (disorder)
        "117211000119105",  # Peripheral T-cell lymphoma of lymph nodes of multiple sites (disorder)
        "1172704005",  # High grade B-cell lymphoma with MYC and BCL2 and/or BCL6 rearrangements (disorder)
        "1172793008",  # Malignant peripheral nerve sheath tumor of gingival mucosa (disorder)
        "1177173001",  # Congenital progressive bone marrow failure, B-cell immunodeficiency, skeletal dysplasia syndrome (disorder)
        "1179375008",  # Adenocarcinoma of parathyroid gland (disorder)
        "1179377000",  # Adenocarcinoma of pituitary gland (disorder)
        "1179389003",  # Adenocarcinoma of paraganglia (disorder)
        "1179390007",  # Adenocarcinoma of carotid body (disorder)
        "1179762006",  # Malignant neoplasm of middle lobe of right lung (disorder)
        "118599009",  # Hodgkin's disease (disorder)
        "118600007",  # Malignant lymphoma (disorder)
        "118601006",  # Non-Hodgkin's lymphoma (disorder)
        "118602004",  # Hodgkin's granuloma (disorder)
        "118605002",  # Hodgkin lymphoma, nodular lymphocyte predominance (disorder)
        "118606001",  # Hodgkin's sarcoma (disorder)
        "118607005",  # Hodgkin lymphoma, lymphocyte-rich (disorder)
        "118608000",  # Hodgkin's disease, nodular sclerosis (disorder)
        "118609008",  # Hodgkin's disease, mixed cellularity (disorder)
        "118610003",  # Hodgkin's disease, lymphocytic depletion (disorder)
        "118611004",  # Sezary's disease (disorder)
        "118612006",  # Malignant histiocytosis (disorder)
        "118613001",  # Hairy cell leukemia (disorder)
        "118614007",  # Langerhans cell histiocytosis, disseminated (disorder)
        "118615008",  # Malignant mast cell tumor (disorder)
        "118617000",  # Burkitt's lymphoma (disorder)
        "118618005",  # Mycosis fungoides (disorder)
        "1186612004",  # Metastatic hepatocellular carcinoma (disorder)
        "1186614003",  # Primary Kaposi sarcoma of gastrointestinal tract (disorder)
        "1186615002",  # Primary Kaposi sarcoma of palate (disorder)
        "1186616001",  # Primary Kaposi sarcoma of skin (disorder)
        "1186629001",  # Primary fibrolamellar hepatocellular carcinoma (disorder)
        "1186630006",  # Primary liver cell carcinoma (disorder)
        "1186631005",  # Primary combined hepatocellular carcinoma and cholangiocarcinoma (disorder)
        "1186735007",  # Mixed neuroendocrine-non neuroendocrine neoplasm of pancreas (disorder)
        "1186869002",  # Neuroendocrine carcinoma of pancreas (disorder)
        "1187123005",  # Mixed phenotype acute leukemia (disorder)
        "1187285005",  # Metastatic clear cell renal cell carcinoma (disorder)
        "1187464007",  # Clear cell sarcoma of kidney (disorder)
        "1187519007",  # Malignant gastrinoma of pancreas (disorder)
        "1187521002",  # Malignant glucagonoma of pancreas (disorder)
        "1187622004",  # Nuclear protein in testis carcinoma (disorder)
        "12235561000119105",  # Large cell carcinoma of left lung (disorder)
        "12235601000119105",  # Large cell carcinoma of right lung (disorder)
        "12240951000119107",  # Squamous cell carcinoma of left lung (disorder)
        "12240991000119102",  # Squamous cell carcinoma of right lung (disorder)
        "12241031000119108",  # Metastatic malignant neoplasm to left breast (disorder)
        "12241071000119106",  # Metastatic malignant neoplasm to right breast (disorder)
        "12246481000119106",  # Metastatic malignant neoplasm to bilateral kidneys (disorder)
        "12246561000119101",  # Metastatic malignant neoplasm to bilateral adrenal glands (disorder)
        "12246601000119101",  # Metastatic malignant neoplasm to bilateral lungs (disorder)
        "12246641000119104",  # Metastatic malignant neoplasm to bilateral breasts (disorder)
        "12281000132104",  # Relapsing acute myeloid leukemia (disorder)
        "122881000119107",  # Chronic monocytic leukemia in relapse (disorder)
        "122901000119109",  # Myeloid leukemia in relapse (disorder)
        "12291000132102",  # Refractory acute myeloid leukemia (disorder)
        "122951000119108",  # Lymphoid leukemia in relapse (disorder)
        "122961000119105",  # Chronic lymphoid leukemia in relapse (disorder)
        "122981000119101",  # Plasma cell leukemia in relapse (disorder)
        "12301000132103",  # Acute lymphoid leukemia relapse (disorder)
        "12311000132101",  # Refractory acute lymphoid leukemia (disorder)
        "123313007",  # Alpha heavy chain disease, enteric form (disorder)
        "123593009",  # Complete atelectasis of left lung (disorder)
        "123594003",  # Complete atelectasis of right lung (disorder)
        "123661000119106",  # Malignant carcinoid tumor of lung (disorder)
        "123701000119104",  # Malignant carcinoid tumor of descending colon (disorder)
        "123712000",  # Acute atelectasis (disorder)
        "123713005",  # Chronic atelectasis (disorder)
        "123721000119108",  # Malignant carcinoid tumor of ascending colon (disorder)
        "123731000119106",  # Malignant carcinoid tumor of cecum (disorder)
        "123741000119102",  # Malignant carcinoid tumor of large intestine (disorder)
        "123751000119100",  # Malignant carcinoid tumor of jejunum (disorder)
        "123761000119103",  # Malignant carcinoid tumor of duodenum (disorder)
        "123781000119107",  # Chronic leukemia in relapse (disorder)
        "123842006",  # Endocervical adenocarcinoma (disorder)
        "123845008",  # Adenocarcinoma of endometrium (disorder)
        "1259003",  # Schistosis (disorder)
        "12677003",  # Tuberculosis of brain (disorder)
        "127066000",  # Familial polycythemia vera (disorder)
        "127070008",  # Malignant histiocytic disorder (disorder)
        "127218004",  # Reactive follicular hyperplasia in the elderly (disorder)
        "127220001",  # Malignant lymphoma of lymph nodes (disorder)
        "127225006",  # Chronic myelomonocytic leukemia (disorder)
        "128041000119107",  # Primary adenocarcinoma of distal third of esophagus (disorder)
        "128462008",  # Metastatic malignant neoplasm (disorder)
        "128465005",  # Metastatic malignant neoplasm to articular cartilage (disorder)
        "128466006",  # Primary malignant neoplasm of articular cartilage (disorder)
        "128875000",  # Primary cutaneous CD30 antigen positive large T-cell lymphoma (disorder)
        "129211000119108",  # Isoniazid resistant tuberculosis of lung (disorder)
        "12985005",  # Tuberculosis of seminal vesicle (disorder)
        "130351000119105",  # Primary malignant neuroendocrine neoplasm of large intestine (disorder)
        "130371000119101",  # Primary malignant neuroendocrine neoplasm of cecum (disorder)
        "130381000119103",  # Primary malignant neuroendocrine neoplasm of ascending colon (disorder)
        "13048006",  # Orbital lymphoma (disorder)
        "130891000119106",  # Chordoma of coccyx (disorder)
        "130911000119108",  # Chordoma of pelvis (disorder)
        "130941000119107",  # Primary chordoma of bone of skull (disorder)
        "133751000119102",  # Lymphoma of colon (disorder)
        "133851000119107",  # Merkel cell carcinoma of neck (disorder)
        "133861000119109",  # Merkel cell carcinoma of scalp (disorder)
        "133891000119102",  # Merkel cell carcinoma of trunk (disorder)
        "134312002",  # Odontogenic ghost cell carcinoma (disorder)
        "134421000",  # Pathologic fracture of bone at site of metastatic neoplasm (disorder)
        "134871000119100",  # Poorly differentiated malignant carcinoid tumor (disorder)
        "135091000119106",  # Secondary neuroendocrine carcinoma of peritoneum (disorder)
        "13570003",  # Tuberculous tenosynovitis (disorder)
        "135931000119106",  # Merkel cell carcinoma of buttock (disorder)
        "136641000119101",  # Torre-Muir syndrome co-occurrent with malignant sebaceous neoplasm (disorder)
        "14188007",  # Tuberculosis of hip (disorder)
        "143291000119104",  # Non-Hodgkin lymphoma in remission (disorder)
        "143401000119106",  # Malignant carcinoid tumor of pancreas (disorder)
        "143411000119109",  # Liposarcoma of retroperitoneum (disorder)
        "14527007",  # Tuberculous empyema (disorder)
        "145501000119108",  # Metastatic malignant neoplasm to breast (disorder)
        "145511000119106",  # Metastatic malignant neoplasm to genital organ (disorder)
        "145831000119103",  # Primary malignant germ cell neoplasm (disorder)
        "147101000119108",  # Primary malignant astrocytoma of central nervous system (disorder)
        "147131000119101",  # Glioblastoma multiforme of central nervous system (disorder)
        "148911000119107",  # Primary malignant neoplasm of abdomen (disorder)
        "15202009",  # Tuberculoma (disorder)
        "15284007",  # Tuberculosis of esophagus (disorder)
        "154283005",  # Pulmonary tuberculosis (disorder)
        "15635721000119108",  # Primary malignant neoplasm of both ovaries (disorder)
        "15636991000119103",  # Renal cell carcinoma of bilateral kidneys (disorder)
        "15930821000119105",  # Metastatic malignant neoplasm to bilateral ovaries (disorder)
        "15930861000119100",  # Carcinosarcoma of left ovary (disorder)
        "15930901000119106",  # Carcinosarcoma of right ovary (disorder)
        "15930941000119108",  # Carcinosarcoma of bilateral ovaries (disorder)
        "15931021000119108",  # Primary sarcoma of left ovary (disorder)
        "15931061000119103",  # Primary sarcoma of right ovary (disorder)
        "15949941000119101",  # Primary squamous cell carcinoma of skin of left breast (disorder)
        "15950021000119100",  # Primary squamous cell carcinoma of skin of right breast (disorder)
        "15950061000119105",  # Primary malignant neoplasm of skin of right breast (disorder)
        "15950101000119108",  # Primary basal cell carcinoma of skin of right breast (disorder)
        "15950141000119105",  # Primary basal cell carcinoma of skin of left breast (disorder)
        "15950221000119108",  # Primary malignant neoplasm of skin of left breast (disorder)
        "15955101000119101",  # Primary malignant neoplasm of right greater vestibular gland (disorder)
        "15955141000119104",  # Primary malignant neoplasm of left greater vestibular gland (disorder)
        "15955901000119104",  # Primary small cell carcinoma of right lung (disorder)
        "15955941000119102",  # Primary small cell carcinoma of left lung (disorder)
        "15956181000119102",  # Metastatic adenocarcinoma to bilateral lungs (disorder)
        "15956221000119105",  # Metastatic adenocarcinoma to right lung (disorder)
        "15956261000119100",  # Metastatic adenocarcinoma to left lung (disorder)
        "15956341000119105",  # Adenocarcinoma of left lung (disorder)
        "15956381000119100",  # Adenocarcinoma of right lung (disorder)
        "15956661000119102",  # Metastatic small cell carcinoma to right lung (disorder)
        "15956701000119109",  # Metastatic small cell carcinoma to left lung (disorder)
        "15956741000119106",  # Metastatic small cell carcinoma to bilateral lungs (disorder)
        "15957141000119109",  # Metastatic squamous cell carcinoma to bilateral lungs (disorder)
        "15957181000119104",  # Metastatic squamous cell carcinoma to right lung (disorder)
        "15957221000119107",  # Metastatic squamous cell carcinoma to left lung (disorder)
        "15957261000119102",  # Kaposi sarcoma of bilateral lungs (disorder)
        "15958021000119103",  # Primary sarcoma of left kidney (disorder)
        "15958101000119106",  # Primary sarcoma of right kidney (disorder)
        "15958181000119103",  # Metastatic adenocarcinoma to bilateral kidneys (disorder)
        "15958221000119106",  # Metastatic adenocarcinoma to left kidney (disorder)
        "15958261000119101",  # Metastatic adenocarcinoma to right kidney (disorder)
        "15958301000119109",  # Metastatic squamous cell carcinoma to right kidney (disorder)
        "15958381000119101",  # Metastatic squamous cell carcinoma to left kidney (disorder)
        "15958661000119103",  # Metastatic malignant melanoma to left kidney (disorder)
        "15958701000119105",  # Metastatic malignant melanoma to bilateral kidneys (disorder)
        "15958741000119107",  # Metastatic malignant melanoma to right kidney (disorder)
        "15959061000119101",  # Transitional cell carcinoma of left kidney (disorder)
        "15959101000119103",  # Transitional cell carcinoma of right kidney (disorder)
        "15977551000119100",  # Metastatic malignant neoplasm to bilateral eyes (disorder)
        "15977591000119105",  # Metastatic malignant neoplasm to right eye (disorder)
        "15977631000119105",  # Metastatic malignant neoplasm to left eye (disorder)
        "15984271000119108",  # Tuberculosis of bilateral ears (disorder)
        "16016231000119109",  # Tuberculosis of right knee joint (disorder)
        "16016311000119107",  # Tuberculosis of left knee joint (disorder)
        "16091131000119108",  # Primary angiosarcoma of skin of forehead (disorder)
        "16091171000119106",  # Angiosarcoma of skin of cheek (disorder)
        "16091251000119109",  # Primary angiosarcoma of skin of nose (disorder)
        "16091371000119108",  # Primary angiosarcoma of skin of temporal region (disorder)
        "16091411000119109",  # Primary angiosarcoma of skin of lip (disorder)
        "16218761000119102",  # Primary squamous cell carcinoma of skin of lip (disorder)
        "16228971000119104",  # Primary malignant neoplasm of soft tissue of left hip (disorder)
        "16229011000119100",  # Primary malignant neoplasm of soft tissue of right hip (disorder)
        "16260391000119109",  # Primary malignant neoplasm of soft tissues of left shoulder (disorder)
        "16260431000119104",  # Primary malignant neoplasm of soft tissues of right shoulder (disorder)
        "16409009",  # Tuberculosis of retroperitoneal lymph nodes (disorder)
        "16414611000119100",  # Primary malignant neoplasm of peripheral nerves of left upper limb (disorder)
        "1651000119109",  # Primary adenocarcinoma of pancreas (disorder)
        "16527441000119105",  # Primary non-small cell carcinoma of lower lobe of left lung (disorder)
        "16527481000119100",  # Primary non-small cell carcinoma of middle lobe of right lung (disorder)
        "16527521000119100",  # Primary non-small cell carcinoma of upper lobe of left lung (disorder)
        "16527561000119105",  # Primary non-small cell carcinoma of lower lobe of right lung (disorder)
        "16527601000119105",  # Primary non-small cell carcinoma of right lung (disorder)
        "16527641000119107",  # Primary non-small cell carcinoma of left lung (disorder)
        "16527721000119108",  # Primary non-small cell carcinoma of upper lobe of right lung (disorder)
        "16537000",  # Compression atelectasis (disorder)
        "16632002",  # Hemopneumothorax (disorder)
        "16636051000119105",  # Primary malignant gastrointestinal stromal neoplasm of colon (disorder)
        "16636101000119105",  # Primary malignant gastrointestinal stromal neoplasm of rectum (disorder)
        "16637101000119107",  # Primary sarcoma of small intestine (disorder)
        "16699241000119100",  # Kaposi sarcoma of toe (disorder)
        "1671000119100",  # Metastatic squamous cell carcinoma to lymph node (disorder)
        "16774551000119100",  # Primary basal cell carcinoma of skin of left hand (disorder)
        "16774591000119105",  # Primary basal cell carcinoma of skin of right hand (disorder)
        "16774671000119108",  # Primary basal cell carcinoma of skin of labium majus (disorder)
        "16775311000119107",  # Mycosis fungoides of facial lymph node (disorder)
        "16782241000119104",  # Primary squamous cell carcinoma of skin of left hand (disorder)
        "16782321000119104",  # Primary squamous cell carcinoma of skin of right hand (disorder)
        "1681000119102",  # Metastatic adenocarcinoma to lymph node (disorder)
        "16823941000119108",  # Primary malignant gastrointestinal stromal neoplasm of pancreas (disorder)
        "16845301000119108",  # Primary glioblastoma multiforme of cerebellum (disorder)
        "16845341000119105",  # Primary glioblastoma multiforme of brainstem (disorder)
        "16845381000119100",  # Primary chondrosarcoma of sternum (disorder)
        "16899941000119102",  # Primary squamous cell carcinoma of skin of anus (disorder)
        "16905141000119107",  # Primary malignant neoplasm of skin of right upper eyelid (disorder)
        "16905341000119105",  # Primary malignant neoplasm of skin of right lower eyelid (disorder)
        "16905381000119100",  # Primary basal cell carcinoma of left upper eyelid (disorder)
        "16905421000119109",  # Primary basal cell carcinoma of right upper eyelid (disorder)
        "16905461000119104",  # Primary basal cell carcinoma of right lower eyelid (disorder)
        "16905501000119104",  # Primary basal cell carcinoma of left lower eyelid (disorder)
        "16905541000119102",  # Primary malignant neoplasm of skin of left lower eyelid (disorder)
        "16905581000119107",  # Primary malignant neoplasm of skin of left upper eyelid (disorder)
        "16906481000119103",  # Primary squamous cell carcinoma of right upper eyelid (disorder)
        "16906521000119103",  # Primary squamous cell carcinoma of right lower eyelid (disorder)
        "16906561000119108",  # Primary squamous cell carcinoma of left upper eyelid (disorder)
        "16906601000119108",  # Primary squamous cell carcinoma of left lower eyelid (disorder)
        "1701000119104",  # Primary adenocarcinoma of colon (disorder)
        "17136009",  # Tuberculosis of knee (disorder)
        "17653001",  # Tuberculosis of bones and/or joints (disorder)
        "18121000119104",  # Primary squamous cell carcinoma of palatine tonsil (disorder)
        "182159002",  # Disseminated tuberculosis (disorder)
        "184741000119107",  # Primary adenocarcinoma of anus (disorder)
        "184781000119102",  # Primary adenocarcinoma of cervix uteri (disorder)
        "184801000119103",  # Primary adenocarcinoma of lacrimal gland (disorder)
        "184861000119102",  # Primary adenocarcinoma of nasopharynx (disorder)
        "184871000119108",  # Primary adenocarcinoma of pelvis (disorder)
        "184881000119106",  # Primary adenocarcinoma of rectosigmoid junction (disorder)
        "184891000119109",  # Primary adenocarcinoma of small intestine (disorder)
        "185113005",  # Tuberculosis of adrenal glands (disorder)
        "186172004",  # Tuberculous pleurisy in primary progressive tuberculosis (disorder)
        "186175002",  # Infiltrative lung tuberculosis (disorder)
        "186177005",  # Tuberculosis of lung with cavitation (disorder)
        "186178000",  # Tuberculosis of bronchus (disorder)
        "186182003",  # Tuberculosis of pleura (disorder)
        "186193001",  # Tuberculosis of lung, confirmed by sputum microscopy with or without culture (disorder)
        "186194007",  # Tuberculosis of lung, confirmed by culture only (disorder)
        "186195008",  # Tuberculosis of lung, confirmed histologically (disorder)
        "186199002",  # Tuberculosis of larynx, trachea and bronchus, confirmed bacteriologically and histologically (disorder)
        "186200004",  # Tuberculous pleurisy, confirmed bacteriologically and histologically (disorder)
        "186202007",  # Respiratory tuberculosis, not confirmed bacteriologically or histologically (disorder)
        "186203002",  # Tuberculosis of lung, bacteriologically and histologically negative (disorder)
        "186204008",  # Tuberculosis of lung, bacteriological and histological examination not done (disorder)
        "186217006",  # Tuberculous abscess of brain (disorder)
        "186225008",  # Tuberculosis of intestines, peritoneum and mesenteric glands (disorder)
        "186259007",  # Scrofulous tuberculous abscess (disorder)
        "186263000",  # Tuberculous chorioretinitis (disorder)
        "186266008",  # Tuberculous chronic iridocyclitis (disorder)
        "186267004",  # Tuberculous keratoconjunctivitis (disorder)
        "186269001",  # Tuberculosis of ear (disorder)
        "186270000",  # Tuberculous Addison's disease (disorder)
        "186272008",  # Tuberculosis of stomach (disorder)
        "186273003",  # Tuberculosis of liver (disorder)
        "186276006",  # Acute miliary tuberculosis (disorder)
        "186278007",  # Acute miliary tuberculosis of multiple sites (disorder)
        "187601000",  # Malignant neoplasm of upper lip, lipstick area (disorder)
        "187604008",  # Malignant neoplasm of lower lip, external (disorder)
        "187606005",  # Malignant tumor of upper labial mucosa (disorder)
        "187608006",  # Malignant tumor of frenum of upper lip (disorder)
        "187613005",  # Malignant neoplasm of lower lip, buccal aspect (disorder)
        "187614004",  # Malignant tumor of frenum of lower lip (disorder)
        "187622006",  # Malignant tumor of labial mucosa (disorder)
        "187631006",  # Malignant neoplasm of base of tongue dorsal surface (disorder)
        "187633009",  # Malignant neoplasm of dorsal surface of tongue (disorder)
        "187634003",  # Malignant tumor of anterior two-thirds of tongue - dorsal surface (disorder)
        "187635002",  # Malignant neoplasm of midline of tongue (disorder)
        "187637005",  # Malignant neoplasm of tongue, tip and lateral border (disorder)
        "187640005",  # Malignant tumor of anterior two-thirds of tongue - ventral surface (disorder)
        "187641009",  # Malignant tumor of frenum linguae (disorder)
        "187644001",  # Malignant tumor of junctional zone of tongue (disorder)
        "187652003",  # Malignant tumor of anterior floor of mouth (disorder)
        "187653008",  # Malignant tumor of lateral floor of mouth (disorder)
        "187658004",  # Malignant tumor of vestibule of mouth (disorder)
        "187659007",  # Malignant tumor of upper buccal sulcus (disorder)
        "187660002",  # Malignant tumor of lower buccal sulcus (disorder)
        "187661003",  # Malignant tumor of upper labial sulcus (disorder)
        "187662005",  # Malignant tumor of lower labial sulcus (disorder)
        "187666008",  # Malignant neoplasm of junction of hard and soft palate (disorder)
        "187675005",  # Malignant tumor of tonsillar pillar (disorder)
        "187681002",  # Malignant neoplasm of anterior epiglottis (disorder)
        "187682009",  # Malignant neoplasm of epiglottis, free border (disorder)
        "187683004",  # Malignant neoplasm of glossoepiglottic fold (disorder)
        "187685006",  # Malignant neoplasm of junctional region of epiglottis (disorder)
        "187688008",  # Malignant tumor of posterior wall of oropharynx (disorder)
        "187692001",  # Malignant tumor of nasopharynx (disorder)
        "187693006",  # Malignant tumor of posterior wall of nasopharynx (disorder)
        "187694000",  # Malignant tumor of adenoid (disorder)
        "187697007",  # Malignant tumor of pharyngeal recess (disorder)
        "187698002",  # Malignant tumor of opening of auditory tube (disorder)
        "187700006",  # Malignant tumor of anterior wall of nasopharynx (disorder)
        "187701005",  # Malignant neoplasm of floor of nasopharynx (disorder)
        "187708004",  # Malignant tumor aryepiglottic fold - hypopharyngeal aspect (disorder)
        "187709007",  # Malignant neoplasm of posterior pharynx (disorder)
        "187716008",  # Malignant tumor of Waldeyer's ring (disorder)
        "187722004",  # Malignant tumor of cervical part of esophagus (disorder)
        "187723009",  # Malignant tumor of thoracic part of esophagus (disorder)
        "187724003",  # Malignant tumor of abdominal part of esophagus (disorder)
        "187725002",  # Malignant tumor of upper third of esophagus (disorder)
        "187726001",  # Malignant tumor of middle third of esophagus (disorder)
        "187727005",  # Malignant tumor of lower third of esophagus (disorder)
        "187732006",  # Malignant tumor of cardia (disorder)
        "187733001",  # Malignant neoplasm of cardiac orifice of stomach (disorder)
        "187734007",  # Malignant neoplasm of cardioesophageal junction of stomach (disorder)
        "187736009",  # Malignant tumor of pylorus (disorder)
        "187738005",  # Malignant neoplasm of pyloric canal of stomach (disorder)
        "187740000",  # Malignant tumor of pyloric antrum (disorder)
        "187741001",  # Malignant tumor of fundus of stomach (disorder)
        "187742008",  # Malignant tumor of body of stomach (disorder)
        "187752007",  # Malignant tumor of Meckel's diverticulum (disorder)
        "187760008",  # Malignant neoplasm of rectum, rectosigmoid junction and anus (disorder)
        "187769009",  # Primary carcinoma of liver (disorder)
        "187773007",  # Malignant neoplasm of interlobular bile ducts (disorder)
        "187776004",  # Malignant neoplasm of intrahepatic canaliculi (disorder)
        "187777008",  # Malignant neoplasm of intrahepatic gall duct (disorder)
        "187784000",  # Malignant neoplasm of hepatic duct (disorder)
        "187786003",  # Malignant neoplasm of sphincter of Oddi (disorder)
        "187791002",  # Malignant tumor of body of pancreas (disorder)
        "187792009",  # Malignant tumor of tail of pancreas (disorder)
        "187793004",  # Malignant tumor of pancreatic duct (disorder)
        "187794005",  # Malignant tumor of Islets of Langerhans (disorder)
        "187798008",  # Malignant neoplasm of ectopic pancreatic tissue (disorder)
        "187801002",  # Malignant tumor of peritoneum and retroperitoneum (disorder)
        "187803004",  # Malignant neoplasm of perinephric tissue (disorder)
        "187804005",  # Malignant neoplasm of retrocecal tissue (disorder)
        "187809000",  # Malignant neoplasm of mesocolon (disorder)
        "187810005",  # Malignant neoplasm of mesocecum (disorder)
        "187811009",  # Malignant neoplasm of mesorectum (disorder)
        "187814001",  # Malignant neoplasm of the pouch of Douglas (disorder)
        "187821001",  # Angiosarcoma of spleen (disorder)
        "187822008",  # Fibrosarcoma of spleen (disorder)
        "187824009",  # Malignant neoplasm of overlapping sites of digestive system (disorder)
        "187828007",  # Malignant neoplasm of nasal cavities, middle ear and accessory sinuses (disorder)
        "187829004",  # Malignant neoplasm of cartilage of nose (disorder)
        "187830009",  # Malignant neoplasm of nasal conchae (disorder)
        "187831008",  # Malignant tumor of nasal vestibule (disorder)
        "187833006",  # Malignant neoplasm of auditory tube, middle ear and mastoid air cells (disorder)
        "187834000",  # Malignant tumor of Eustachian tube (disorder)
        "187835004",  # Malignant tumor of tympanic cavity (disorder)
        "187836003",  # Malignant tumor of tympanic antrum (disorder)
        "187841006",  # Malignant tumor of glottis (disorder)
        "187842004",  # Malignant tumor of supraglottis (disorder)
        "187843009",  # Malignant neoplasm of arytenoid cartilage (disorder)
        "187844003",  # Malignant neoplasm of cricoid cartilage (disorder)
        "187845002",  # Malignant neoplasm of cuneiform cartilage (disorder)
        "187846001",  # Malignant neoplasm of thyroid cartilage (disorder)
        "187853005",  # Malignant neoplasm of cartilage of trachea (disorder)
        "187854004",  # Malignant neoplasm of mucosa of trachea (disorder)
        "187857006",  # Malignant neoplasm of carina of bronchus (disorder)
        "187861000",  # Malignant neoplasm of upper lobe bronchus (disorder)
        "187862007",  # Malignant neoplasm of upper lobe of lung (disorder)
        "187865009",  # Malignant neoplasm of middle lobe bronchus (disorder)
        "187869003",  # Malignant neoplasm of lower lobe bronchus (disorder)
        "187870002",  # Malignant neoplasm of lower lobe of lung (disorder)
        "187900002",  # Malignant neoplasm of bones of skull and face (disorder)
        "187903000",  # Malignant neoplasm of malar bone (disorder)
        "187906008",  # Malignant neoplasm of orbital bone (disorder)
        "187916000",  # Malignant neoplasm of cervical vertebra (disorder)
        "187917009",  # Malignant neoplasm of thoracic vertebra (disorder)
        "187918004",  # Malignant neoplasm of lumbar vertebra (disorder)
        "187920001",  # Malignant neoplasm of ribs and/or sternum and/or clavicle (disorder)
        "187925006",  # Malignant neoplasm of costal cartilage (disorder)
        "187926007",  # Malignant neoplasm of costovertebral joint (disorder)
        "187927003",  # Malignant neoplasm of xiphoid process (disorder)
        "187929000",  # Malignant neoplasm of scapula and long bones of upper arm (disorder)
        "187932002",  # Malignant neoplasm of humerus (disorder)
        "187937008",  # Malignant neoplasm of carpal bone - scaphoid (disorder)
        "187938003",  # Malignant neoplasm of carpal bone - lunate (disorder)
        "187939006",  # Malignant neoplasm of carpal bone - triquetrum (disorder)
        "187940008",  # Malignant neoplasm of carpal bone - pisiform (disorder)
        "187941007",  # Malignant neoplasm of carpal bone - trapezium (disorder)
        "187942000",  # Malignant neoplasm of carpal bone - trapezoid (disorder)
        "187943005",  # Malignant neoplasm of carpal bone - capitate (disorder)
        "187944004",  # Malignant neoplasm of carpal bone - hamate (disorder)
        "187945003",  # Malignant neoplasm of first metacarpal bone (disorder)
        "187946002",  # Malignant neoplasm of second metacarpal bone (disorder)
        "187947006",  # Malignant neoplasm of third metacarpal bone (disorder)
        "187948001",  # Malignant neoplasm of fourth metacarpal bone (disorder)
        "187949009",  # Malignant neoplasm of fifth metacarpal bone (disorder)
        "187950009",  # Malignant neoplasm of phalanges of hand (disorder)
        "187952001",  # Malignant neoplasm of pelvic bones, sacrum and coccyx (disorder)
        "187956003",  # Malignant neoplasm of sacral vertebra (disorder)
        "187957007",  # Malignant neoplasm of coccygeal vertebra (disorder)
        "187967002",  # Malignant neoplasm of calcaneum (disorder)
        "187968007",  # Malignant neoplasm of medial cuneiform (disorder)
        "187969004",  # Malignant neoplasm of intermediate cuneiform (disorder)
        "187970003",  # Malignant neoplasm of lateral cuneiform (disorder)
        "187972006",  # Malignant neoplasm of navicular (disorder)
        "187973001",  # Malignant neoplasm of first metatarsal bone (disorder)
        "187974007",  # Malignant neoplasm of second metatarsal bone (disorder)
        "187975008",  # Malignant neoplasm of third metatarsal bone (disorder)
        "187976009",  # Malignant neoplasm of fourth metatarsal bone (disorder)
        "187977000",  # Malignant neoplasm of fifth metatarsal bone (disorder)
        "187978005",  # Malignant neoplasm of phalanges of foot (disorder)
        "187987001",  # Malignant neoplasm of cartilage of ear (disorder)
        "187988006",  # Malignant neoplasm of tarsus of eyelid (disorder)
        "187989003",  # Malignant neoplasm soft tissues of cervical spine (disorder)
        "188029000",  # Kaposi's sarcoma of soft tissue (disorder)
        "188030005",  # Malignant melanoma of lip (disorder)
        "188034001",  # Malignant melanoma of external auditory meatus (disorder)
        "188038003",  # Malignant melanoma of chin (disorder)
        "188040008",  # Malignant melanoma of forehead (disorder)
        "188045003",  # Malignant melanoma of scalp (disorder)
        "188046002",  # Malignant melanoma of neck (disorder)
        "188049009",  # Malignant melanoma of axilla (disorder)
        "188050009",  # Malignant melanoma of breast (disorder)
        "188051008",  # Malignant melanoma of buttock (disorder)
        "188052001",  # Malignant melanoma of groin (disorder)
        "188054000",  # Malignant melanoma of perineum (disorder)
        "188055004",  # Malignant melanoma of umbilicus (disorder)
        "188060000",  # Malignant melanoma of shoulder (disorder)
        "188061001",  # Malignant melanoma of upper arm (disorder)
        "188062008",  # Malignant melanoma of forearm (disorder)
        "188063003",  # Malignant melanoma of hand (disorder)
        "188064009",  # Malignant melanoma of finger (disorder)
        "188065005",  # Malignant melanoma of thumb (disorder)
        "188068007",  # Malignant melanoma of hip (disorder)
        "188069004",  # Malignant melanoma of thigh (disorder)
        "188070003",  # Malignant melanoma of knee (disorder)
        "188071004",  # Malignant melanoma of popliteal fossa area (disorder)
        "188072006",  # Malignant melanoma of lower leg (disorder)
        "188073001",  # Malignant melanoma of ankle (disorder)
        "188074007",  # Malignant melanoma of heel (disorder)
        "188075008",  # Malignant melanoma of foot (disorder)
        "188076009",  # Malignant melanoma of toe (disorder)
        "188077000",  # Malignant melanoma of great toe (disorder)
        "188089003",  # Malignant neoplasm of skin of ear and external auditory canal (disorder)
        "188090007",  # Malignant neoplasm of skin of auricle (ear) (disorder)
        "188091006",  # Malignant neoplasm of skin of external auditory meatus (disorder)
        "188095002",  # Malignant neoplasm of skin of cheek, external (disorder)
        "188099008",  # Malignant neoplasm of skin of nose (external) (disorder)
        "188100000",  # Malignant neoplasm of skin of temple (disorder)
        "188102008",  # Malignant neoplasm of scalp and/or skin of neck (disorder)
        "188103003",  # Malignant neoplasm of skin of scalp (disorder)
        "188107002",  # Malignant neoplasm of skin of axillary fold (disorder)
        "188110009",  # Malignant neoplasm of skin of abdominal wall (disorder)
        "188121000",  # Malignant neoplasm of skin of upper arm (disorder)
        "188122007",  # Malignant neoplasm of skin of forearm (disorder)
        "188125009",  # Malignant neoplasm of skin of thumb (disorder)
        "188132000",  # Malignant neoplasm of skin of popliteal fossa area (disorder)
        "188133005",  # Malignant neoplasm of skin of lower leg (disorder)
        "188135003",  # Malignant neoplasm of skin of heel (disorder)
        "188138001",  # Malignant neoplasm of skin of great toe (disorder)
        "188157005",  # Malignant neoplasm of overlapping sites of breast (disorder)
        "188176007",  # Malignant neoplasm of endocervical canal (disorder)
        "188177003",  # Malignant neoplasm of endocervical gland (disorder)
        "188180002",  # Primary malignant neoplasm of overlapping sites of cervix uteri (disorder)
        "188183000",  # Malignant neoplasm of cervical stump (disorder)
        "188184006",  # Malignant neoplasm of squamocolumnar junction of cervix (disorder)
        "188188009",  # Choriocarcinoma (disorder)
        "188189001",  # Malignant neoplasm of corpus uteri, excluding isthmus (disorder)
        "188190005",  # Malignant neoplasm of cornu of corpus uteri (disorder)
        "188191009",  # Malignant neoplasm of fundus of corpus uteri (disorder)
        "188192002",  # Malignant neoplasm of endometrium of corpus uteri (disorder)
        "188193007",  # Malignant neoplasm of myometrium of corpus uteri (disorder)
        "188195000",  # Malignant neoplasm of isthmus of uterine body (disorder)
        "188204000",  # Malignant neoplasm of round ligament (disorder)
        "188208002",  # Malignant neoplasm of Gartner's duct (disorder)
        "188209005",  # Malignant neoplasm of vaginal vault (disorder)
        "188211001",  # Malignant neoplasm of greater vestibular (Bartholin's) gland (disorder)
        "188219004",  # Malignant tumor of undescended testis (disorder)
        "188220005",  # Malignant tumor of ectopic testis (disorder)
        "188230001",  # Malignant tumor of body of penis (disorder)
        "188234005",  # Malignant tumor of seminal vesicle (disorder)
        "188235006",  # Malignant tumor of tunica vaginalis (disorder)
        "188239000",  # Malignant tumor of trigone of urinary bladder (disorder)
        "188240003",  # Malignant neoplasm of dome of urinary bladder (disorder)
        "188241004",  # Malignant neoplasm of lateral wall of urinary bladder (disorder)
        "188242006",  # Malignant neoplasm of anterior wall of urinary bladder (disorder)
        "188243001",  # Malignant neoplasm of posterior wall of urinary bladder (disorder)
        "188244007",  # Malignant neoplasm of neck of urinary bladder (disorder)
        "188245008",  # Malignant tumor of ureteric orifice (disorder)
        "188247000",  # Malignant neoplasm of overlapping sites of urinary bladder (disorder)
        "188250002",  # Malignant tumor of kidney parenchyma (disorder)
        "188252005",  # Malignant tumor of renal calyx (disorder)
        "188253000",  # Malignant tumor of pelviureteric junction (disorder)
        "188256008",  # Malignant neoplasm of overlapping sites of urinary organs (disorder)
        "188261005",  # Malignant neoplasm of eyeball excluding conjunctiva, cornea, retina and choroid (disorder)
        "188263008",  # Malignant tumor of ciliary body (disorder)
        "188264002",  # Malignant tumor of iris (disorder)
        "188265001",  # Malignant neoplasm of crystalline lens (disorder)
        "188266000",  # Malignant neoplasm of sclera (disorder)
        "188268004",  # Malignant neoplasm of connective tissue of orbit (disorder)
        "188269007",  # Malignant neoplasm of extraocular muscle of orbit (disorder)
        "188272000",  # Malignant tumor of lacrimal gland (disorder)
        "188273005",  # Malignant neoplasm of lacrimal sac (disorder)
        "188274004",  # Malignant neoplasm of nasolacrimal duct (disorder)
        "188280007",  # Malignant neoplasm of cerebrum (excluding lobes and ventricles) (disorder)
        "188281006",  # Malignant neoplasm of basal ganglia (disorder)
        "188282004",  # Malignant neoplasm of cerebral cortex (disorder)
        "188283009",  # Malignant neoplasm of corpus striatum (disorder)
        "188285002",  # Malignant neoplasm of globus pallidus (disorder)
        "188286001",  # Malignant tumor of hypothalamus (disorder)
        "188287005",  # Malignant neoplasm of thalamus (disorder)
        "188289008",  # Malignant neoplasm of hippocampus (disorder)
        "188290004",  # Malignant neoplasm of uncus (disorder)
        "188292007",  # Malignant tumor of choroid plexus (disorder)
        "188293002",  # Malignant neoplasm of floor of cerebral ventricle (disorder)
        "188295009",  # Malignant neoplasm of cerebral peduncle (disorder)
        "188296005",  # Malignant neoplasm of medulla oblongata (disorder)
        "188297001",  # Malignant neoplasm of midbrain (disorder)
        "188298006",  # Malignant neoplasm of pons (disorder)
        "188301005",  # Malignant neoplasm of corpus callosum (disorder)
        "188302003",  # Malignant neoplasm of tapetum (disorder)
        "188307009",  # Malignant tumor of cranial nerve (disorder)
        "188308004",  # Malignant neoplasm of olfactory bulb (disorder)
        "188312005",  # Malignant neoplasm of cerebral dura mater (disorder)
        "188313000",  # Malignant neoplasm of cerebral arachnoid mater (disorder)
        "188315007",  # Malignant neoplasm of cerebral pia mater (disorder)
        "188317004",  # Malignant neoplasm of spinal dura mater (disorder)
        "188318009",  # Malignant neoplasm of spinal arachnoid mater (disorder)
        "188319001",  # Malignant neoplasm of spinal pia mater (disorder)
        "188321006",  # Malignant neoplasm of peripheral nerves and autonomic nervous system (disorder)
        "188322004",  # Malignant neoplasm of peripheral nerves of head, face and neck (disorder)
        "188323009",  # Malignant neoplasm of peripheral nerves of upper limb, including shoulder (disorder)
        "188324003",  # Malignant neoplasm of peripheral nerves of lower limb, including hip (disorder)
        "188325002",  # Malignant neoplasm of peripheral nerve of thorax (disorder)
        "188326001",  # Malignant neoplasm of peripheral nerve of abdomen (disorder)
        "188327005",  # Malignant neoplasm of peripheral nerve of pelvis (disorder)
        "188339002",  # Malignant neoplasm of pituitary gland and craniopharyngeal duct (disorder)
        "188340000",  # Malignant tumor of craniopharyngeal duct (disorder)
        "188361007",  # Malignant neoplasm of thorax (disorder)
        "188366002",  # Malignant neoplasm of abdomen (disorder)
        "188445006",  # Metastatic malignant neoplasm to retroperitoneum and peritoneum (disorder)
        "188454009",  # Metastatic malignant neoplasm to skin of head (disorder)
        "188458007",  # Metastatic malignant neoplasm to skin of shoulder and arm (disorder)
        "188462001",  # Metastatic malignant neoplasm to brain and spinal cord (disorder)
        "188469005",  # Metastatic malignant neoplasm to cervix uteri (disorder)
        "188471005",  # Metastatic malignant neoplasm to epididymis and vas deferens (disorder)
        "188487008",  # Lymphosarcoma and reticulosarcoma (disorder)
        "188489006",  # Reticulosarcoma of lymph nodes of head, face and neck (disorder)
        "188492005",  # Reticulosarcoma of lymph nodes of axilla and upper limb (disorder)
        "188493000",  # Reticulosarcoma of lymph nodes of inguinal region and lower limb (disorder)
        "188498009",  # Lymphosarcoma (disorder)
        "188500005",  # Lymphosarcoma of lymph nodes of head, face and neck (disorder)
        "188501009",  # Lymphosarcoma of intrathoracic lymph nodes (disorder)
        "188502002",  # Lymphosarcoma of intra-abdominal lymph nodes (disorder)
        "188503007",  # Lymphosarcoma of lymph nodes of axilla and upper limb (disorder)
        "188504001",  # Lymphosarcoma of lymph nodes of inguinal region and lower limb (disorder)
        "188505000",  # Lymphosarcoma of intrapelvic lymph nodes (disorder)
        "188506004",  # Lymphosarcoma of spleen (disorder)
        "188507008",  # Lymphosarcoma of lymph nodes of multiple sites (disorder)
        "188511002",  # Burkitt's lymphoma of intrathoracic lymph nodes (disorder)
        "188512009",  # Burkitt's lymphoma of intra-abdominal lymph nodes (disorder)
        "188515006",  # Burkitt's lymphoma of intrapelvic lymph nodes (disorder)
        "188516007",  # Burkitt's lymphoma of spleen (disorder)
        "188517003",  # Burkitt's lymphoma of lymph nodes of multiple sites (disorder)
        "188524002",  # Hodgkin's paragranuloma of intrathoracic lymph nodes (disorder)
        "188529007",  # Hodgkin's paragranuloma of intrapelvic lymph nodes (disorder)
        "188531003",  # Hodgkin's paragranuloma of lymph nodes of multiple sites (disorder)
        "188536008",  # Hodgkin's granuloma of intra-abdominal lymph nodes (disorder)
        "188541000",  # Hodgkin's granuloma of lymph nodes of multiple sites (disorder)
        "188551004",  # Hodgkin's sarcoma of lymph nodes of multiple sites (disorder)
        "188562004",  # Hodgkin's disease, lymphocytic-histiocytic predominance of lymph nodes of multiple sites (disorder)
        "188566001",  # Hodgkin's disease, nodular sclerosis of intrathoracic lymph nodes (disorder)
        "188567005",  # Hodgkin's disease, nodular sclerosis of intra-abdominal lymph nodes (disorder)
        "188570009",  # Hodgkin's disease, nodular sclerosis of intrapelvic lymph nodes (disorder)
        "188572001",  # Hodgkin's disease, nodular sclerosis of lymph nodes of multiple sites (disorder)
        "188576003",  # Hodgkin's disease, mixed cellularity of intrathoracic lymph nodes (disorder)
        "188577007",  # Hodgkin's disease, mixed cellularity of intra-abdominal lymph nodes (disorder)
        "188580008",  # Hodgkin's disease, mixed cellularity of intrapelvic lymph nodes (disorder)
        "188582000",  # Hodgkin's disease, mixed cellularity of lymph nodes of multiple sites (disorder)
        "188586002",  # Hodgkin's disease, lymphocytic depletion of intrathoracic lymph nodes (disorder)
        "188587006",  # Hodgkin's disease, lymphocytic depletion of intra-abdominal lymph nodes (disorder)
        "188591001",  # Hodgkin's disease, lymphocytic depletion of intrapelvic lymph nodes (disorder)
        "188592008",  # Hodgkin's disease, lymphocytic depletion of spleen (disorder)
        "188593003",  # Hodgkin's disease, lymphocytic depletion of lymph nodes of multiple sites (disorder)
        "188609000",  # Nodular lymphoma of lymph nodes of head, face and neck (disorder)
        "188612002",  # Nodular lymphoma of lymph nodes of axilla and upper limb (disorder)
        "188613007",  # Nodular lymphoma of lymph nodes of inguinal region and lower limb (disorder)
        "188627002",  # Mycosis fungoides of lymph nodes of multiple sites (disorder)
        "188631008",  # Sezary's disease of intrathoracic lymph nodes (disorder)
        "188632001",  # Sezary's disease of intra-abdominal lymph nodes (disorder)
        "188633006",  # Sezary's disease of lymph nodes of axilla and upper limb (disorder)
        "188634000",  # Sezary's disease of lymph nodes of inguinal region and lower limb (disorder)
        "188635004",  # Sezary's disease of intrapelvic lymph nodes (disorder)
        "188637007",  # Sezary's disease of lymph nodes of multiple sites (disorder)
        "188641006",  # Malignant histiocytosis of lymph nodes of axilla and upper limb (disorder)
        "188645002",  # Leukemic reticuloendotheliosis of lymph nodes of head, face and neck (disorder)
        "188648000",  # Leukemic reticuloendotheliosis of lymph nodes of axilla and upper limb (disorder)
        "188649008",  # Leukemic reticuloendotheliosis of lymph nodes of inguinal region and lower limb (disorder)
        "188662007",  # Mast cell malignancy of lymph nodes of head and neck (disorder)
        "188668006",  # Malignant mast cell sarcoma of spleen (disorder)
        "188669003",  # Mast cell malignancy of lymph nodes of multiple sites (disorder)
        "188672005",  # Follicular non-Hodgkin's mixed small cleaved and large cell lymphoma (disorder)
        "188674006",  # Diffuse malignant lymphoma - small non-cleaved cell (disorder)
        "188675007",  # Malignant lymphoma - small cleaved cell (disorder)
        "188676008",  # Malignant lymphoma - mixed small and large cell (disorder)
        "188718006",  # Malignant plasma cell neoplasm, extramedullary plasmacytoma (disorder)
        "188725004",  # Lymphoid leukemia (disorder)
        "188726003",  # Subacute lymphoid leukemia (disorder)
        "188728002",  # Aleukemic lymphoid leukemia (disorder)
        "188732008",  # Myeloid leukemia (disorder)
        "188733003",  # Chronic eosinophilic leukemia (disorder)
        "188734009",  # Chronic neutrophilic leukemia (disorder)
        "188737002",  # Chloroma (disorder)
        "188744006",  # Monocytic leukemia (disorder)
        "188754005",  # Megakaryocytic leukemia (disorder)
        "188768003",  # Myelomonocytic leukemia (disorder)
        "188770007",  # Subacute myelomonocytic leukemia (disorder)
        "189814006",  # Pancreatoblastoma (disorder)
        "189815007",  # Pulmonary blastoma (disorder)
        "189847002",  # Malignant teratoma (disorder)
        "189924002",  # Pleomorphic xanthoastrocytoma (disorder)
        "190030009",  # Compound leukemias (disorder)
        "190710003",  # Tuberculous arthritis (disorder)
        "190818004",  # Waldenstrom macroglobulinemia (disorder)
        "19090001000004101",  # Metastatic neoplasm of left basal ganglion (disorder)
        "192681000119104",  # Malignant carcinoid tumor of appendix (disorder)
        "192701000119101",  # Malignant carcinoid tumor of ileum (disorder)
        "192745001",  # Tuberculous intracranial abscess (disorder)
        "194908003",  # Acute tuberculous pericarditis (disorder)
        "194949003",  # Acute myocarditis - tuberculous (disorder)
        "195953005",  # Mixed simple and mucopurulent chronic bronchitis (disorder)
        "195985008",  # Post-infective bronchiectasis (disorder)
        "196017002",  # Pneumoconiosis associated with tuberculosis (disorder)
        "196067009",  # Loculated empyema (disorder)
        "196102003",  # Spontaneous tension pneumothorax (disorder)
        "196107009",  # Abscess of lung and mediastinum (disorder)
        "196112005",  # Abscess of lung with pneumonia (disorder)
        "196133001",  # Lung disease with systemic sclerosis (disorder)
        "196136009",  # Lung disease co-occurrent with polymyositis (disorder)
        "196138005",  # Lung disease with systemic lupus erythematosus (disorder)
        "196143003",  # Postoperative atelectasis (disorder)
        "196188002",  # Acute pulmonary insufficiency following thoracic surgery (disorder)
        "196189005",  # Acute pulmonary insufficiency following nonthoracic surgery (disorder)
        "196190001",  # Chronic pulmonary insufficiency following surgery (disorder)
        "196191002",  # Postprocedural subglottic stenosis (disorder)
        "198241002",  # Female tuberculous pelvic inflammatory disease (disorder)
        "202994006",  # Tuberculous infection of tendon sheath (disorder)
        "203263006",  # Tuberculosis of cervical spine (disorder)
        "203264000",  # Tuberculosis of thoracic spine (disorder)
        "203265004",  # Tuberculosis of lumbar spine (disorder)
        "203267007",  # Tuberculosis of limb bones (disorder)
        "203269005",  # Tuberculosis of the upper arm bone (disorder)
        "203270006",  # Tuberculosis of the forearm bone (disorder)
        "203272003",  # Tuberculosis of the lower leg bone (disorder)
        "203274002",  # Tuberculosis of multiple limb bones (disorder)
        "203278004",  # Tuberculosis of bones of shoulder region (disorder)
        "203279007",  # Tuberculosis of the bones of the hand (disorder)
        "203280005",  # Tuberculosis of the bones of the ankle and/or foot (disorder)
        "203282002",  # Tuberculosis of the bones of multiple sites (disorder)
        "20372007",  # Tuberculosis of prostate (disorder)
        "208041000119100",  # Primary adenocarcinoma of endocervix (disorder)
        "213580002",  # Caseating tuberculoid granuloma (disorder)
        "21851000119103",  # Malignant pheochromocytoma (disorder)
        "22607003",  # Asbestosis (disorder)
        "226521000119108",  # Primary malignant neoplasm of nasopharynx (disorder)
        "22990009",  # Chronic tuberculosis (disorder)
        "230156002",  # Malignant meningitis (disorder)
        "23022004",  # Tuberculous bronchiectasis (disorder)
        "231829006",  # Malignant neoplasm of eyelid (disorder)
        "231831002",  # Squamous cell carcinoma of eyelid (disorder)
        "231832009",  # Basal cell carcinoma of eyelid (disorder)
        "231833004",  # Sebaceous adenocarcinoma of eyelid (disorder)
        "231834005",  # Malignant melanoma of eyelid (disorder)
        "231835006",  # Kaposi's sarcoma of eyelid (disorder)
        "232075002",  # Lymphoma of retina (disorder)
        "233628009",  # Acquired bronchiectasis (disorder)
        "233629001",  # Idiopathic bronchiectasis (disorder)
        "233631005",  # Toxin-induced bronchiectasis (disorder)
        "233634002",  # Post-lung transplantation bronchiectasis (disorder)
        "233649005",  # Tuberculous chylothorax (disorder)
        "233658003",  # Pleural plaque (disorder)
        "233659006",  # Asbestos-induced pleural plaque (disorder)
        "233749003",  # Complicated pneumoconiosis (disorder)
        "233751004",  # Metal pneumoconiosis (disorder)
        "233754007",  # Cerium pneumoconiosis (disorder)
        "233755008",  # Nickel pneumoconiosis (disorder)
        "233756009",  # Thorium pneumoconiosis (disorder)
        "233757000",  # Zirconium pneumoconiosis (disorder)
        "233758005",  # Mica pneumoconiosis (disorder)
        "233759002",  # Mixed mineral dust pneumoconiosis (disorder)
        "233760007",  # Acute silicosis (disorder)
        "233761006",  # Subacute silicosis (disorder)
        "233762004",  # Chronic silicosis (disorder)
        "233763009",  # Silicotuberculosis (disorder)
        "233794009",  # Tuberculous bronchopleural fistula (disorder)
        "233940007",  # Pulmonary tumor embolism (disorder)
        "234090004",  # Tuberculous mesenteric adenitis (disorder)
        "234941000119100",  # Malignant glioma of eye (disorder)
        "235067001",  # Oral tuberculosis (disorder)
        "235694001",  # Tuberculous duodenitis (disorder)
        "235966007",  # Cystadenocarcinoma of pancreas (disorder)
        "236512004",  # Leukemic infiltrate of kidney (disorder)
        "236513009",  # Lymphoma of kidney (disorder)
        "236602004",  # Tuberculous stricture of ureter (disorder)
        "236684001",  # Tuberculous urethritis (disorder)
        "236770001",  # Tuberculous epididymo-orchitis (disorder)
        "236811000119101",  # Paget disease of anal canal (disorder)
        "237445009",  # Tuberculosis of breast (disorder)
        "237832001",  # Gastrointestinal hormone-secreting endocrine tumor (disorder)
        "237833006",  # Carcinoid crisis (disorder)
        "238430004",  # Cutaneous complication of bacillus Calmette-Guerin immunization (disorder)
        "238431000",  # Bacillus Calmette-Guerin granuloma (disorder)
        "238432007",  # Bacillus Calmette-Guerin ulcer (disorder)
        "238636003",  # Malignant acanthosis nigricans (disorder)
        "238637007",  # Acanthosis palmaris (disorder)
        "240163000",  # Malignant neoplasm of nasopharyngeal wall (disorder)
        "240376003",  # Tuberculosis of gastrointestinal tract (disorder)
        "240379005",  # Tuberculosis of male genital organs (disorder)
        "240380008",  # Acute tuberculous ulcer (disorder)
        "240381007",  # Acute miliary cutaneous tuberculosis (disorder)
        "240382000",  # Cutaneous tuberculous gumma (disorder)
        "240531002",  # African Burkitt's lymphoma (disorder)
        "24111000119105",  # Primary squamous cell carcinoma of upper limb (disorder)
        "241861008",  # Metastatic malignant neoplasm to nasopharynx (disorder)
        "24235005",  # Tuberculous hydrothorax (disorder)
        "242862004",  # Metastatic malignant neoplasm to nasopharyngeal wall (disorder)
        "242951000119108",  # Primary squamous cell carcinoma of vermilion border of lip (disorder)
        "24810006",  # Tuberculosis orificialis of mouth (disorder)
        "25050002",  # Alpha heavy chain disease, respiratory form (disorder)
        "253000007",  # Neuroendocrine carcinoma (disorder)
        "253001006",  # Merkel cell carcinoma (disorder)
        "253005002",  # Vasoactive intestinal peptide-secreting tumor (disorder)
        "253017000",  # Klatskin's tumor (disorder)
        "253018005",  # Fibrolamellar hepatocellular carcinoma (disorder)
        "253042009",  # Myxoid dermatofibrosarcoma protuberans (disorder)
        "253096008",  # Peripheral neuroectodermal tumor (disorder)
        "254389005",  # Carcinoma of vermilion border of upper lip (disorder)
        "254390001",  # Carcinoma of vermilion border of lower lip (disorder)
        "254393004",  # Carcinoma of frenum of lip (disorder)
        "254398008",  # Carcinoma of frenum of upper lip (disorder)
        "254402004",  # Carcinoma of frenum of lower lip (disorder)
        "254404003",  # Carcinoma of commissure of lip (disorder)
        "254408000",  # Malignant tumor of anterior two-thirds of tongue - lateral margin (disorder)
        "254412006",  # Malignant tumor of tip of tongue (disorder)
        "254417000",  # Carcinoma of frenum linguae (disorder)
        "254423005",  # Carcinoma of lingual tonsil (disorder)
        "254424004",  # Carcinoma of upper gum (disorder)
        "254425003",  # Carcinoma of lower gum (disorder)
        "254427006",  # Carcinoma of anterior part of floor of mouth (disorder)
        "254431000",  # Carcinoma of lateral part of floor of mouth (disorder)
        "254434008",  # Carcinoma of hard palate (disorder)
        "254435009",  # Carcinoma of soft palate (disorder)
        "254436005",  # Carcinoma of uvula (disorder)
        "254437001",  # Squamous cell carcinoma of buccal mucosa (disorder)
        "254441002",  # Carcinoma of upper buccal sulcus (disorder)
        "254445006",  # Carcinoma of lower buccal sulcus (disorder)
        "254450000",  # Carcinoma of upper labial sulcus (disorder)
        "254454009",  # Carcinoma of lower labial sulcus (disorder)
        "254457002",  # Carcinoma of retromolar area (disorder)
        "254459004",  # Malignant tumor of anterior pillar of fauces (disorder)
        "254462001",  # Carcinoma of parotid gland (disorder)
        "254465004",  # Carcinoma of submandibular gland (disorder)
        "254466003",  # Carcinoma of sublingual gland (disorder)
        "254474002",  # Malignant tumor of nasal skeleton (disorder)
        "254478004",  # Malignant tumor of inferior turbinate (disorder)
        "254481009",  # Malignant tumor of middle turbinate (disorder)
        "254484001",  # Malignant tumor of posterior margin of nasal septum and choanae (disorder)
        "254503007",  # Malignant tumor of inferior surface of soft palate (disorder)
        "254509006",  # Malignant tumor of anterior commissure (disorder)
        "254513004",  # Malignant tumor of posterior commissure (disorder)
        "254517003",  # Malignant tumor of suprahyoid epiglottis (disorder)
        "254520006",  # Malignant tumor of infrahyoid epiglottis (disorder)
        "254526000",  # Malignant tumor of laryngeal ventricle (disorder)
        "254530002",  # Malignant tumor of parapharyngeal space (disorder)
        "254535007",  # Carcinoma of cervical part of esophagus (disorder)
        "254539001",  # Carcinoma of thoracic part of esophagus (disorder)
        "254543002",  # Carcinoma of abdominal part of esophagus (disorder)
        "254547001",  # Carcinoma of upper third of esophagus (disorder)
        "254549003",  # Carcinoma of middle third of esophagus (disorder)
        "254551004",  # Carcinoma of lower third of esophagus (disorder)
        "254553001",  # Carcinoma of cardia (disorder)
        "254555008",  # Carcinoma of fundus of stomach (disorder)
        "254557000",  # Carcinoma of body of stomach (disorder)
        "254559002",  # Carcinoma of pyloric antrum (disorder)
        "254561006",  # Carcinoma of pylorus (disorder)
        "254563009",  # Carcinoma of lesser curve of stomach (disorder)
        "254567005",  # Carcinoma of greater curve of stomach (disorder)
        "254570009",  # Carcinoma of duodenum (disorder)
        "254582000",  # Adenocarcinoma of rectum (disorder)
        "254586002",  # Malignant tumor of anorectal junction (disorder)
        "254601002",  # Sarcoma of liver (disorder)
        "254609000",  # Carcinoma of ampulla of Vater (disorder)
        "254611009",  # Malignant tumor of endocrine pancreas (disorder)
        "254612002",  # Carcinoma of endocrine pancreas (disorder)
        "254619006",  # Adenoid cystic carcinoma of trachea (disorder)
        "254620000",  # Squamous cell carcinoma of trachea (disorder)
        "254622008",  # Squamous cell carcinoma of bronchus (disorder)
        "254625005",  # Malignant tumor of lung parenchyma (disorder)
        "254626006",  # Adenocarcinoma of lung (disorder)
        "254629004",  # Large cell carcinoma of lung (disorder)
        "254631008",  # Giant cell carcinoma of lung (disorder)
        "254632001",  # Small cell carcinoma of lung (disorder)
        "254634000",  # Squamous cell carcinoma of lung (disorder)
        "254635004",  # Epithelioid hemangioendothelioma of lung (disorder)
        "254637007",  # Non-small cell lung cancer (disorder)
        "254638002",  # Pancoast tumor (disorder)
        "254645002",  # Malignant mesothelioma of pleura (disorder)
        "254650008",  # Malignant epithelial neoplasm of skin (disorder)
        "254651007",  # Squamous cell carcinoma of skin (disorder)
        "254652000",  # Clear cell squamous cell carcinoma of skin (disorder)
        "254653005",  # Spindle cell squamous carcinoma of skin (disorder)
        "254654004",  # Acantholytic squamous cell carcinoma of skin (disorder)
        "254655003",  # Plantar verrucous carcinoma (disorder)
        "254701007",  # Basal cell carcinoma of skin (disorder)
        "254702000",  # Basosquamous carcinoma of skin (disorder)
        "254703005",  # Fibroepithelioma of Pinkus (disorder)
        "254707006",  # Malignant skin tumor with eccrine differentiation (disorder)
        "254708001",  # Eccrine porocarcinoma of skin (disorder)
        "254709009",  # Digital papillary eccrine carcinoma of skin (disorder)
        "254710004",  # Basal cell carcinoma with eccrine differentiation (disorder)
        "254711000",  # Adenoid cystic eccrine carcinoma of skin (disorder)
        "254712007",  # Microcystic adnexal carcinoma of skin (disorder)
        "254713002",  # Mucoepidermoid carcinoma of skin (disorder)
        "254714008",  # Mucinous eccrine carcinoma of skin (disorder)
        "254726003",  # Malignant skin tumor with apocrine differentiation (disorder)
        "254727007",  # Extramammary Paget's disease of skin (disorder)
        "254748009",  # Cutaneous fibrosarcoma (disorder)
        "254764001",  # Peripheral neuroepithelioma (disorder)
        "254771006",  # Cutaneous leiomyosarcoma (disorder)
        "254792006",  # Proliferating angioendotheliomatosis (disorder)
        "254794007",  # Angiosarcoma of skin (disorder)
        "254797000",  # Malignant hemangiopericytoma of skin (disorder)
        "254800003",  # Epithelioid cell sarcoma of skin (disorder)
        "254820002",  # Follicular atrophoderma and basal cell epitheliomata (disorder)
        "254824006",  # Malignant tumor of mesothelial tissue (disorder)
        "254829001",  # Liposarcoma (disorder)
        "254837009",  # Malignant neoplasm of breast (disorder)
        "254838004",  # Carcinoma of breast (disorder)
        "254839007",  # Scirrhous carcinoma of breast (disorder)
        "254840009",  # Inflammatory carcinoma of breast (disorder)
        "254841008",  # Cancer en cuirasse (disorder)
        "254843006",  # Familial cancer of breast (disorder)
        "254844000",  # Malignant phyllodes tumor of breast (disorder)
        "254849005",  # Malignant epithelial tumor of ovary (disorder)
        "254850005",  # Serous papillary cystadenocarcinoma ovary (disorder)
        "254851009",  # Mucinous cystadenocarcinoma of ovary (disorder)
        "254852002",  # Endometrioid carcinoma ovary (disorder)
        "254856004",  # Undifferentiated carcinoma of ovary (disorder)
        "254860001",  # Malignant sex cord tumor of ovary (disorder)
        "254863004",  # Malignant granulosa cell tumour of ovary (disorder)
        "254869000",  # Malignant germ cell tumor of ovary (disorder)
        "254870004",  # Choriocarcinoma of ovary (disorder)
        "254872007",  # Embryonal carcinoma of ovary (disorder)
        "254874008",  # Malignant dysgerminoma of ovary (disorder)
        "254876005",  # Endodermal sinus tumor of ovary (disorder)
        "254877001",  # Sarcoma of uterus (disorder)
        "254878006",  # Endometrial carcinoma (disorder)
        "254886006",  # Squamous cell carcinoma of cervix (disorder)
        "254887002",  # Adenocarcinoma of cervix (disorder)
        "254888007",  # Adenosquamous carcinoma of cervix (disorder)
        "254893005",  # Carcinoma of vagina (disorder)
        "254895003",  # Squamous cell carcinoma of vulva (disorder)
        "254896002",  # Malignant melanoma of vulva (disorder)
        "254897006",  # Sarcoma of vulva (disorder)
        "254898001",  # Paget's disease of vulva (disorder)
        "254900004",  # Carcinoma of prostate (disorder)
        "254904008",  # Carcinoma of glans penis (disorder)
        "254908006",  # Malignant tumor of skin of penis (disorder)
        "254909003",  # Carcinoma of foreskin (disorder)
        "254912000",  # Regressed malignant testicular tumor (disorder)
        "254915003",  # Clear cell carcinoma of kidney (disorder)
        "254916002",  # Cystadenocarcinoma of kidney (disorder)
        "254917006",  # Papillary cystadenocarcinoma of kidney (disorder)
        "254918001",  # Sarcoma of kidney (disorder)
        "254934003",  # Malignant tumor of urethral stump (disorder)
        "254938000",  # Astrocytoma of brain (disorder)
        "254940005",  # Oligodendroglioma of brain (disorder)
        "254948003",  # Astrocytoma of spinal cord (disorder)
        "254950006",  # Oligodendroglioma of spinal cord (disorder)
        "254955001",  # Pituitary carcinoma (disorder)
        "254969001",  # Malignant tumor of olfactory tract (disorder)
        "254972008",  # Malignant tumor of optic nerve and sheath (disorder)
        "254973003",  # Malignant astrocytoma of optic nerve (disorder)
        "254974009",  # Malignant tumor of optic nerve sheath (disorder)
        "254975005",  # Malignant meningioma of optic nerve sheath (disorder)
        "254983004",  # Malignant tumor of spinal nerve and sheath (disorder)
        "254986007",  # Malignant tumor of peripheral nerve (disorder)
        "254987003",  # Adenoid cystic carcinoma of lacrimal gland (disorder)
        "254988008",  # Adenocarcinoma of lacrimal gland (disorder)
        "254989000",  # Carcinoma ex pleomorphic adenoma of lacrimal gland (disorder)
        "254990009",  # Mucoepidermoid tumor of lacrimal gland (disorder)
        "254993006",  # Liposarcoma of orbit (disorder)
        "254994000",  # Rhabdomyosarcoma of orbit (disorder)
        "254995004",  # Malignant hemangiopericytoma of orbit (disorder)
        "254996003",  # Malignant fibrous histiocytoma of orbit (disorder)
        "255003007",  # Squamous cell carcinoma of conjunctiva (disorder)
        "255004001",  # Malignant melanoma of conjunctiva (disorder)
        "255008003",  # Squamous cell carcinoma of cornea (disorder)
        "255012009",  # Malignant melanoma of iris (disorder)
        "255015006",  # Malignant melanoma of ciliary body (disorder)
        "255016007",  # Adenocarcinoma of pigmented epithelium of ciliary body (disorder)
        "255017003",  # Adenocarcinoma of non-pigmented epithelium of ciliary body (disorder)
        "255021005",  # Malignant melanoma of choroid (disorder)
        "255028004",  # Follicular thyroid carcinoma (disorder)
        "255029007",  # Papillary thyroid carcinoma (disorder)
        "255030002",  # Mixed follicular and papillary thyroid carcinoma (disorder)
        "255031003",  # Anaplastic thyroid carcinoma (disorder)
        "255032005",  # Medullary thyroid carcinoma (disorder)
        "255035007",  # Adrenal carcinoma (disorder)
        "255037004",  # Parathyroid carcinoma (disorder)
        "255039001",  # Pancreatic polypeptidoma (disorder)
        "255052006",  # Malignant neoplasm of unknown origin (disorder)
        "255056009",  # Malignant tumor of head and/or neck (disorder)
        "255066001",  # Carcinoma of genitourinary organ (disorder)
        "255067005",  # Sarcoma of bone and connective tissue (disorder)
        "255068000",  # Carcinoma of bone, connective tissue, skin and breast (disorder)
        "255069008",  # Carcinoma of lip, oral cavity and/or pharynx (disorder)
        "255070009",  # Overlapping malignant neoplasm of oral cavity and lip and salivary gland (disorder)
        "255071008",  # Squamous cell carcinoma of lip (disorder)
        "255072001",  # Malignant tumor of salivary gland (disorder)
        "255073006",  # Malignant tumor of ear, nose and throat (disorder)
        "255074000",  # Malignant tumor of nasal cavity and nasopharynx (disorder)
        "255075004",  # Malignant tumor of lateral nasal wall (disorder)
        "255077007",  # Malignant tumor of digestive organ (disorder)
        "255078002",  # Malignant tumor of esophagus, stomach and duodenum (disorder)
        "255081007",  # Carcinoma of cecum (disorder)
        "255084004",  # Squamous cell carcinoma of anal margin (disorder)
        "255086002",  # Carcinoma common bile duct (disorder)
        "255087006",  # Malignant polyp of biliary tract (disorder)
        "255088001",  # Malignant tumor of exocrine pancreas (disorder)
        "255090000",  # Malignant neoplasm of carpal bones (disorder)
        "255091001",  # Malignant neoplasm of metacarpal bones (disorder)
        "255093003",  # Malignant skin tumor with adnexal differentiation (disorder)
        "255096006",  # Malignant tumor of dermis (disorder)
        "255107005",  # Seminoma of testis (disorder)
        "255108000",  # Carcinoma of urinary bladder (disorder)
        "255109008",  # Transitional cell carcinoma of urinary bladder (disorder)
        "255110003",  # Adenocarcinoma of urinary bladder (disorder)
        "255111004",  # Squamous cell carcinoma of urinary bladder (disorder)
        "255112006",  # Malignant tumor of pituitary and hypothalamus (disorder)
        "255114007",  # Kaposi's sarcoma of conjunctiva (disorder)
        "255115008",  # Kaposi's sarcoma of cornea (disorder)
        "255118005",  # Metastatic lymphangitic carcinoma (disorder)
        "255119002",  # Lymphangitis carcinomatosa (disorder)
        "255121007",  # Carcinomatosis of peritoneal cavity (disorder)
        "255123005",  # Metastasis to nervous system and eye (disorder)
        "255124004",  # Metastasis to peripheral nerve (disorder)
        "255128001",  # Malignant infiltration of peripheral nerve (disorder)
        "255129009",  # Malignant infiltration of peripheral nerve plexus (disorder)
        "255191003",  # Localized malignant reticulohistiocytoma (disorder)
        "25629007",  # Acute tuberculosis (disorder)
        "25738004",  # Tuberculous synovitis (disorder)
        "25831001",  # Tuberculosis of nasal septum (disorder)
        "266366003",  # Lung disease due to external agents (disorder)
        "267317007",  # Tuberculosis in pregnancy (disorder)
        "26935004",  # Tuberculosis of thyroid gland (disorder)
        "269459004",  # Malignant tumor of lesser curve of stomach (disorder)
        "269460009",  # Malignant tumor of greater curve of stomach (disorder)
        "269463006",  # Malignant tumor of middle ear and mastoid (disorder)
        "269464000",  # Malignant neoplasm of upper lobe, bronchus or lung (disorder)
        "269467007",  # Malignant neoplasm of hand bones (disorder)
        "269469005",  # Malignant neoplasm of soft tissue (disorder)
        "269473008",  # Metastatic malignant neoplasm to respiratory and digestive systems (disorder)
        "269475001",  # Malignant tumor of lymphoid, hemopoietic AND/OR related tissue (disorder)
        "269476000",  # Nodular lymphoma (disorder)
        "269515006",  # Carcinoma of lip (disorder)
        "269516007",  # Tongue carcinoma (disorder)
        "269533000",  # Carcinoma of colon (disorder)
        "269544008",  # Carcinoma of rectosigmoid junction (disorder)
        "269578002",  # Malignant melanoma of head and neck (disorder)
        "269579005",  # Malignant melanoma of trunk (disorder)
        "269580008",  # Malignant melanoma of upper limb (disorder)
        "269581007",  # Malignant melanoma of lower limb (disorder)
        "271323007",  # Malignant neoplasm of lip, oral cavity and/or pharynx (disorder)
        "271423008",  # Tuberculosis of skin and subcutaneous tissue (disorder)
        "271467005",  # Malignant neoplasm of bone, connective tissue, skin and breast (disorder)
        "271468000",  # Malignant neoplasm of genitourinary organ (disorder)
        "271503005",  # Pleural empyema with fistula (disorder)
        "271504004",  # Pleural empyema with no fistula (disorder)
        "271568003",  # Malignant tumor of lower labial mucosa (disorder)
        "271943005",  # Carcinoma of base of tongue (disorder)
        "274084007",  # Palate carcinoma (disorder)
        "274085008",  # Tonsil carcinoma (disorder)
        "274087000",  # Malignant melanoma of eye (disorder)
        "274088005",  # Metastatic malignant neoplasm to unknown site (disorder)
        "274902006",  # Combined hepatocellular carcinoma and cholangiocarcinoma (disorder)
        "274905008",  # Malignant lymphoma - lymphocytic, intermediate differentiation (disorder)
        "275266006",  # Metastasis to digestive organs (disorder)
        "275394001",  # Carcinoma ventral surface of tongue (disorder)
        "275395000",  # Carcinoma anterior 2/3 tongue ventrum (disorder)
        "275396004",  # Carcinoma of anterior two-thirds of tongue - dorsal surface (disorder)
        "275397008",  # Carcinoma of midline of tongue (disorder)
        "275399006",  # Malignant tumor of lipstick area of lip (disorder)
        "275490009",  # Carcinoma of tongue base - dorsal surface (disorder)
        "276419004",  # Malignant tumor of corpus spongiosum (disorder)
        "276420005",  # Malignant tumor of corpus cavernosum (disorder)
        "276738009",  # Primary signet ring cell carcinoma of skin (disorder)
        "276750003",  # Malignant tumor of skin with pilar differentiation (disorder)
        "276797002",  # Malignant tumor of fibrous tissue (disorder)
        "276799004",  # Dermatofibrosarcoma protuberans (disorder)
        "276803003",  # Adenocarcinoma of esophagus (disorder)
        "276804009",  # Squamous cell carcinoma of esophagus (disorder)
        "276809004",  # Early gastric cancer (disorder)
        "276810009",  # Late gastric cancer (disorder)
        "276811008",  # Gastric lymphoma (disorder)
        "276815004",  # Lymphoma of intestine (disorder)
        "276821000",  # Malignant melanoma of anus (disorder)
        "276822007",  # Malignant melanoma of rectum (disorder)
        "276826005",  # Malignant glioma of brain (disorder)
        "276827001",  # Malignant glioma of spinal cord (disorder)
        "276828006",  # Glioblastoma multiforme of brain (disorder)
        "276829003",  # Glioblastoma multiforme of spinal cord (disorder)
        "276836002",  # Primary cerebral lymphoma (disorder)
        "276860003",  # Squamous cell carcinoma of scrotum (disorder)
        "276870001",  # Carcinoma of fallopian tube (disorder)
        "276876007",  # Carcinoma of Bartholin's gland (disorder)
        "276952000",  # Squamous cell carcinoma of tongue (disorder)
        "276953005",  # Squamous cell carcinoma of gum (disorder)
        "276954004",  # Squamous cell carcinoma of floor of mouth (disorder)
        "276962007",  # Squamous cell carcinoma of palate (disorder)
        "276975007",  # Carcinoma of larynx (disorder)
        "277156006",  # Malignant tumor of external ear (disorder)
        "277461004",  # Anaplastic astrocytoma of brain (disorder)
        "277474005",  # B-cell chronic lymphocytic leukemia variant (disorder)
        "277505007",  # Medulloblastoma of cerebellum (disorder)
        "277530005",  # Malignant melanoma of meninges (disorder)
        "277543005",  # Malignant white blood cell disorder (disorder)
        "277545003",  # T-cell chronic lymphocytic leukemia (disorder)
        "277549009",  # Chronic lymphocytic prolymphocytic leukemia syndrome (disorder)
        "277550009",  # Richter's syndrome (disorder)
        "277551008",  # Splenic lymphoma with villous lymphocytes (disorder)
        "277567002",  # T-cell prolymphocytic leukemia (disorder)
        "277568007",  # Hairy cell leukemia variant (disorder)
        "277569004",  # Large granular lymphocytic leukemia (disorder)
        "277570003",  # Lymphoma with spill (disorder)
        "277571004",  # B-cell acute lymphoblastic leukemia (disorder)
        "277572006",  # Precursor B-cell acute lymphoblastic leukemia (disorder)
        "277573001",  # Common acute lymphoblastic leukemia (disorder)
        "277574007",  # Null cell acute lymphoblastic leukemia (disorder)
        "277575008",  # T-lymphoblastic leukemia (disorder)
        "277579002",  # Light chain myeloma (disorder)
        "277580004",  # Non-secretory myeloma (disorder)
        "277587001",  # Juvenile chronic myeloid leukemia (disorder)
        "277589003",  # Myelodysplastic-myeloproliferative neoplasm with neutrophilia (disorder)
        "277597005",  # Myelodysplastic syndrome with isolated del(5q) (disorder)
        "277602003",  # Acute megakaryoblastic leukemia (disorder)
        "277604002",  # Malignant myeloid-lymphoid neoplasm (disorder)
        "277609007",  # Hodgkin's disease, lymphocytic predominance - diffuse (disorder)
        "277610002",  # Hodgkin's disease, nodular sclerosis - lymphocytic predominance (disorder)
        "277611003",  # Hodgkin's disease, nodular sclerosis - mixed cellularity (disorder)
        "277612005",  # Hodgkin's disease, nodular sclerosis - lymphocytic depletion (disorder)
        "277613000",  # Cutaneous/peripheral T-cell lymphoma (disorder)
        "277614006",  # Prethymic and thymic T-cell lymphoma/leukemia (disorder)
        "277615007",  # Low grade B-cell lymphoma (disorder)
        "277616008",  # Diffuse low grade B-cell lymphoma (disorder)
        "277617004",  # High grade B-cell lymphoma (disorder)
        "277618009",  # Follicular low grade B-cell lymphoma (disorder)
        "277619001",  # B-cell prolymphocytic leukemia (disorder)
        "277622004",  # Mucosa-associated lymphoma (disorder)
        "277623009",  # Monocytoid B-cell lymphoma (disorder)
        "277624003",  # Follicular malignant lymphoma - mixed cell type (disorder)
        "277625002",  # Follicular malignant lymphoma - small cleaved cell (disorder)
        "277626001",  # Diffuse high grade B-cell lymphoma (disorder)
        "277627005",  # Nodular high grade B-cell lymphoma (disorder)
        "277628000",  # Diffuse malignant lymphoma - large cleaved cell (disorder)
        "277629008",  # Diffuse malignant lymphoma - large non-cleaved cell (disorder)
        "277632006",  # Diffuse malignant lymphoma - centroblastic polymorphic (disorder)
        "277642008",  # Low grade T-cell lymphoma (disorder)
        "277643003",  # High grade T-cell lymphoma (disorder)
        "277654008",  # Enteropathy-associated T-cell lymphoma (disorder)
        "277664004",  # Malignant lymphoma of testis (disorder)
        "277782009",  # Local recurrence of malignant neoplasm of peritoneum (disorder)
        "278024000",  # Rhabdomyosarcoma of urinary bladder (disorder)
        "278042005",  # Malignant teratoma of mediastinum (disorder)
        "278043000",  # Malignant seminoma of mediastinum (disorder)
        "278044006",  # Malignant neuroma of mediastinum (disorder)
        "278046008",  # Sarcoma of urinary bladder (disorder)
        "278050001",  # Sarcoma of breast (disorder)
        "278051002",  # Malignant lymphoma of thyroid gland (disorder)
        "278052009",  # Malignant lymphoma of breast (disorder)
        "278054005",  # Lobular carcinoma of breast (disorder)
        "278055006",  # Malignant Leydig cell tumor of testis (disorder)
        "278060005",  # Endometrioid carcinoma of prostate (disorder)
        "278189009",  # Hypergranular promyelocytic leukemia (disorder)
        "278433008",  # Malignant infiltration of soft tissue (disorder)
        "278453007",  # Acute biphenotypic leukemia (disorder)
        "278491007",  # Mixed seminoma teratoma of testis (disorder)
        "28085001",  # Empyema with bronchopleural fistula (disorder)
        "280959007",  # Malignant tumor of lacrimal drainage structure (disorder)
        "281560004",  # Neuroblastoma of brain (disorder)
        "281562007",  # Adrenal neuroblastoma (disorder)
        "281563002",  # Thoracic neuroblastoma (disorder)
        "281564008",  # Pelvic neuroblastoma (disorder)
        "281565009",  # Paraspinal neuroblastoma (disorder)
        "281566005",  # Abdominothoracic neuroblastoma (disorder)
        "281623008",  # Genital tuberculosis (disorder)
        "281702006",  # Tibial adamantinoma (disorder)
        "28295001",  # Middle lobe syndrome (disorder)
        "28399005",  # Tuberculosis of spleen (disorder)
        "285307007",  # Squamous cell carcinoma of skin of upper lip (disorder)
        "285308002",  # Squamous cell carcinoma of skin of lower lip (disorder)
        "285309005",  # Squamous cell carcinoma of skin of cheek (disorder)
        "285310000",  # Carcinoma of anal canal (disorder)
        "285312008",  # Carcinoma of sigmoid colon (disorder)
        "285420006",  # Immunoglobulin A myeloma (disorder)
        "285421005",  # Immunoglobulin G myeloma (disorder)
        "285422003",  # Immunoglobulin D myeloma (disorder)
        "285432005",  # Carcinoma of cervix (disorder)
        "285769009",  # Acute promyelocytic leukemia - hypogranular variant (disorder)
        "285776004",  # Intermediate grade B-cell lymphoma (disorder)
        "286887005",  # Carcinoma liver and/or biliary system (disorder)
        "286889008",  # Carcinoma of upper limb bones/scapula (disorder)
        "286890004",  # Carcinoma of lower limb bones (disorder)
        "286891000",  # Carcinoma of skin of head/neck (disorder)
        "286893002",  # Carcinoma of breast - upper, inner quadrant (disorder)
        "286894008",  # Carcinoma of breast - lower, inner quadrant (disorder)
        "286895009",  # Carcinoma of upper outer quadrant of breast (disorder)
        "286896005",  # Carcinoma breast - lower, outer quadrant (disorder)
        "286897001",  # Carcinoma of breast - axillary tail (disorder)
        "286899003",  # Carcinoma of genital organ (disorder)
        "286902000",  # Secondary carcinoma of gastrointestinal tract (disorder)
        "29421000119105",  # Adenocarcinoma in adenomatous polyp (disorder)
        "29422001",  # Coal workers' pneumoconiosis (disorder)
        "29431000119108",  # Adenocarcinoma in villous adenoma (disorder)
        "297188009",  # Clotted hemothorax (disorder)
        "29731002",  # Tuberculous pneumothorax (disorder)
        "300988009",  # Transitional cell carcinoma of ureter (disorder)
        "301756000",  # Adenocarcinoma of sigmoid colon (disorder)
        "301836007",  # Tuberculosis of colon (disorder)
        "302131003",  # Tuberculosis of heart (disorder)
        "302815008",  # Malignant tumor of frenum of lip (disorder)
        "302816009",  # Malignant tumor of soft tissue of head, face and neck (disorder)
        "302841002",  # Malignant lymphoma - small lymphocytic (disorder)
        "302842009",  # Diffuse malignant lymphoma - centroblastic (disorder)
        "302845006",  # Nodular malignant lymphoma, lymphocytic - well differentiated (disorder)
        "302847003",  # Rhabdomyosarcoma (disorder)
        "302848008",  # Nodular malignant lymphoma, lymphocytic - intermediate differentiation (disorder)
        "302849000",  # Nephroblastoma (disorder)
        "302851001",  # Synovial sarcoma (disorder)
        "302855005",  # Subacute leukemia (disorder)
        "302856006",  # Aleukemic leukemia (disorder)
        "303012000",  # Malignant tumor of posterior wall of hypopharynx (disorder)
        "303055001",  # Malignant lymphoma, follicular center cell (disorder)
        "303056000",  # Malignant lymphoma, follicular center cell, cleaved (disorder)
        "303057009",  # Malignant lymphoma, follicular center cell, non-cleaved (disorder)
        "303194003",  # Metastasis to head and neck lymph node (disorder)
        "303201005",  # Metastatic malignant neoplasm to multiple lymph nodes (disorder)
        "30437004",  # Empyema with bronchocutaneous fistula (disorder)
        "304545002",  # Adenocarcinoma of ileum (disorder)
        "307202004",  # Hemorrhagic pleural effusion (disorder)
        "307216009",  # Perforated carcinoma of esophagus (disorder)
        "307219002",  # Retroperitoneal sarcoma (disorder)
        "307226002",  # Metastatic adenocarcinoma of unknown origin (disorder)
        "307340003",  # Monosomy 7 syndrome (disorder)
        "307341004",  # Atypical hairy cell leukemia (disorder)
        "307502000",  # Squamous cell carcinoma of mouth (disorder)
        "307576001",  # Osteosarcoma of bone (disorder)
        "307591004",  # Malignant mastocytosis (disorder)
        "307592006",  # Basophilic leukemia (disorder)
        "307593001",  # Carcinomatosis (disorder)
        "307599002",  # Sebaceous adenocarcinoma (disorder)
        "307601000",  # Pseudomyxoma peritonei (disorder)
        "307608006",  # Ewing's sarcoma of bone (disorder)
        "307609003",  # Adamantinoma of long bone (disorder)
        "307610008",  # Pilomatrix carcinoma of skin (disorder)
        "307617006",  # Neutrophilic leukemia (disorder)
        "307622006",  # Prolymphocytic lymphosarcoma (disorder)
        "307623001",  # Malignant lymphoma - lymphoplasmacytic (disorder)
        "307624007",  # Diffuse malignant lymphoma - centroblastic-centrocytic (disorder)
        "307625008",  # Malignant lymphoma - centrocytic (disorder)
        "307633009",  # Hodgkin's disease, lymphocytic depletion, diffuse fibrosis (disorder)
        "307634003",  # Hodgkin's disease, lymphocytic depletion, reticular type (disorder)
        "307635002",  # Hodgkin's disease, nodular sclerosis - cellular phase (disorder)
        "307636001",  # Malignant lymphoma, mixed lymphocytic-histiocytic, nodular (disorder)
        "307637005",  # Malignant lymphoma, centroblastic-centrocytic, follicular (disorder)
        "307646004",  # Malignant lymphoma, lymphocytic, poorly differentiated, nodular (disorder)
        "307647008",  # Malignant lymphoma, centroblastic type, follicular (disorder)
        "307649006",  # Microglioma (disorder)
        "307650006",  # Histiocytic medullary reticulosis (disorder)
        "307651005",  # Primary myelofibrosis (disorder)
        "308121000",  # Follicular lymphoma (disorder)
        "309245001",  # Adenocarcinoma of uterus (disorder)
        "31047003",  # Lymphomatoid papulosis (disorder)
        "310498001",  # Malignant melanoma of back (disorder)
        "310504009",  # Primary malignant neoplasm of unknown site (disorder)
        "310526005",  # Malignant tumor of soft tissue of back (disorder)
        "310599006",  # Malignant neoplasm of canthus (disorder)
        "31112008",  # Tuberculous meningoencephalitis (disorder)
        "311671000119100",  # Juvenile dermatomyositis co-occurrent with respiratory involvement (disorder)
        "311779007",  # Malignant neoplasm of skin of scapular region (disorder)
        "312104005",  # Cholangiocarcinoma of biliary tract (disorder)
        "312111009",  # Carcinoma of ascending colon (disorder)
        "312112002",  # Carcinoma of transverse colon (disorder)
        "312113007",  # Carcinoma of descending colon (disorder)
        "312114001",  # Carcinoma of hepatic flexure (disorder)
        "312115000",  # Carcinoma of splenic flexure (disorder)
        "312949007",  # Retinal pigment epithelial adenocarcinoma (disorder)
        "313248004",  # Malignant melanoma of chest wall (disorder)
        "313249007",  # Malignant neoplasm of upper eyelid (disorder)
        "313250007",  # Malignant neoplasm of lower eyelid (disorder)
        "313353007",  # Squamous cell carcinoma of bronchus in left lower lobe (disorder)
        "313354001",  # Squamous cell carcinoma of bronchus in left upper lobe (disorder)
        "313355000",  # Squamous cell carcinoma of bronchus in right lower lobe (disorder)
        "313356004",  # Squamous cell carcinoma of bronchus of middle lobe of right lung (disorder)
        "313357008",  # Squamous cell carcinoma of bronchus in right upper lobe (disorder)
        "313427003",  # Lambda light chain myeloma (disorder)
        "313428008",  # Seminoma of undescended testis (disorder)
        "313429000",  # Seminoma of descended testis (disorder)
        "314029000",  # Tuberculous uveitis (disorder)
        "314191009",  # Cystadenocarcinoma of ovary (disorder)
        "314408000",  # Leukemic infiltrate of choroid (disorder)
        "314418005",  # Leukemic infiltrate of retina (disorder)
        "314951005",  # Local recurrence of malignant tumor of tongue (disorder)
        "314952003",  # Local recurrence of malignant tumor of buccal cavity (disorder)
        "314953008",  # Local recurrence of malignant tumor of thyroid gland (disorder)
        "314954002",  # Local recurrence of malignant tumor of lung (disorder)
        "314955001",  # Local recurrence of malignant tumor of breast (disorder)
        "314960002",  # Local recurrence of malignant tumor of esophagus (disorder)
        "314961003",  # Local recurrence of malignant tumor of stomach (disorder)
        "314962005",  # Local recurrence of malignant tumor of gallbladder (disorder)
        "314963000",  # Local recurrence of malignant tumor of liver (disorder)
        "314964006",  # Local recurrence of malignant tumor of pancreas (disorder)
        "314965007",  # Local recurrence of malignant tumor of colon (disorder)
        "314966008",  # Local recurrence of malignant tumor of rectum (disorder)
        "314967004",  # Local recurrence of malignant tumor of kidney (disorder)
        "314968009",  # Local recurrence of malignant tumor of urinary bladder (disorder)
        "314969001",  # Local recurrence of malignant tumor of prostate (disorder)
        "314970000",  # Local recurrence of malignant tumor of cervix (disorder)
        "314973003",  # Local recurrence of malignant tumor of bone (disorder)
        "314974009",  # Local recurrence of malignant tumor of soft tissue (disorder)
        "314975005",  # Local recurrence of malignant tumor of skin (disorder)
        "315058005",  # Hereditary nonpolyposis colon cancer (disorder)
        "31892009",  # Hemothorax (disorder)
        "32139003",  # Mixed dust pneumoconiosis (disorder)
        "32268008",  # Tuberculosis of urinary bladder (disorder)
        "326072005",  # Carcinoma of head of pancreas (disorder)
        "326542006",  # Traumatic atelectasis (disorder)
        "328561000119107",  # Primary spontaneous pneumothorax (disorder)
        "328571000119101",  # Secondary spontaneous pneumothorax (disorder)
        "33072005",  # Traumatic hemothorax with open wound into thorax (disorder)
        "33325001",  # Compensatory emphysema (disorder)
        "33548005",  # Anthracosilicosis (disorder)
        "33581000119109",  # Primary malignant neoplasm of extradural spinal cord (disorder)
        "33631007",  # Pyopneumothorax (disorder)
        "33893009",  # Tuberculosis verrucosa cutis (disorder)
        "34004002",  # Siderosilicosis (disorder)
        "34286007",  # Empyema with hepatopleural fistula (disorder)
        "350621000119100",  # Primary malignant neoplasm of right vestibulocochlear nerve (disorder)
        "350631000119102",  # Primary malignant neoplasm of left vestibulocochlear nerve (disorder)
        "350681000119101",  # Primary malignant neoplasm of adrenal medulla of right adrenal gland (disorder)
        "350801000119105",  # Primary malignant neoplasm of left adrenal cortex (disorder)
        "350851000119109",  # Primary malignant neoplasm of adrenal medulla of left adrenal gland (disorder)
        "350861000119106",  # Primary malignant neoplasm of right adrenal gland (disorder)
        "350881000119102",  # Primary malignant neoplasm of right adrenal cortex (disorder)
        "350921000119109",  # Primary malignant neoplasm of left adrenal gland (disorder)
        "350951000119101",  # B-cell lymphoma of intra-abdominal lymph nodes (disorder)
        "351041000119109",  # Primary malignant neoplasm of left upper limb (disorder)
        "351051000119106",  # Primary malignant neoplasm of right upper limb (disorder)
        "351211000119104",  # B-cell lymphoma of lymph nodes of multiple sites (disorder)
        "352231000119103",  # Primary malignant neoplasm of soft tissue of left upper extremity (disorder)
        "352251000119109",  # Small lymphocytic B-cell lymphoma of lymph nodes of multiple sites (disorder)
        "352301000119103",  # Primary malignant neoplasm of soft tissue of right upper extremity (disorder)
        "352311000119100",  # Primary malignant neoplasm of soft tissue of right lower limb (disorder)
        "352321000119107",  # Primary malignant neoplasm of soft tissue of left lower limb (disorder)
        "352331000119105",  # Primary malignant neoplasm of left spermatic cord (disorder)
        "352341000119101",  # Primary malignant neoplasm of right spermatic cord (disorder)
        "352381000119106",  # Primary malignant neoplasm of skin of left upper limb (disorder)
        "352391000119109",  # Primary malignant neoplasm of skin of left ear (disorder)
        "352411000119109",  # Small lymphocytic B-cell lymphoma of intra-abdominal lymph nodes (disorder)
        "352421000119102",  # Primary malignant neoplasm of skin of left shoulder (disorder)
        "352431000119104",  # Primary malignant neoplasm of skin of right ear (disorder)
        "352461000119107",  # Primary malignant neoplasm of skin of right lower limb (disorder)
        "352471000119101",  # Primary malignant neoplasm of skin of right shoulder (disorder)
        "352481000119103",  # Primary malignant neoplasm of skin of left lower limb (disorder)
        "352701000119102",  # Primary malignant neoplasm of neck of pancreas (disorder)
        "352731000119109",  # Primary malignant neoplasm of peripheral nerves of left lower limb (disorder)
        "352751000119103",  # Primary malignant neoplasm of peripheral nerves of right upper limb (disorder)
        "352771000119107",  # Primary malignant neoplasm of peripheral nerves of right lower limb (disorder)
        "352791000119108",  # Non-Hodgkin's lymphoma of lymph nodes of multiple sites (disorder)
        "352911000119101",  # Primary malignant neoplasm of left optic nerve (disorder)
        "352921000119108",  # Primary malignant neoplasm of right optic nerve (disorder)
        "352961000119103",  # Primary malignant neoplasm of right renal pelvis (disorder)
        "353061000119107",  # Primary malignant neoplasm of retina of right eye (disorder)
        "353081000119103",  # Primary malignant neoplasm of retina of left eye (disorder)
        "353111000119108",  # Primary malignant neoplasm of left renal pelvis (disorder)
        "353561000119103",  # Metastatic malignant neoplasm to right lung (disorder)
        "353571000119109",  # Metastatic malignant neoplasm to right adrenal gland (disorder)
        "353661000119102",  # Overlapping primary malignant neoplasm of bone and articular cartilage of lower limb (disorder)
        "353671000119108",  # Overlapping primary malignant neoplasm of bone and articular cartilage of right upper limb (disorder)
        "353721000119100",  # Metastatic malignant neoplasm to left adrenal gland (disorder)
        "353741000119106",  # Metastatic malignant neoplasm to left lung (disorder)
        "353761000119105",  # Primary malignant neoplasm of cornea of left eye (disorder)
        "353791000119103",  # Primary malignant neoplasm of conjunctiva of right eye (disorder)
        "353801000119102",  # Primary malignant neoplasm of choroid of right eye (disorder)
        "353811000119104",  # Primary malignant neoplasm of cornea of right eye (disorder)
        "353841000119100",  # Primary malignant neoplasm of conjunctiva of left eye (disorder)
        "354081000119101",  # Metastatic adenocarcinoma (disorder)
        "354301000119106",  # Primary malignant neoplasm of choroid of left eye (disorder)
        "354331000119104",  # Primary malignant neoplasm of left lower limb (disorder)
        "354351000119105",  # Primary malignant neoplasm of left kidney (disorder)
        "354361000119107",  # Primary malignant neoplasm of right kidney (disorder)
        "354391000119100",  # Kaposi sarcoma of left lung (disorder)
        "354421000119107",  # Primary malignant neoplasm of right lower limb (disorder)
        "354461000119102",  # Kaposi sarcoma of right lung (disorder)
        "354621000119105",  # Overlapping primary malignant neoplasm of bone and articular cartilage of left upper limb (disorder)
        "354641000119104",  # Primary malignant neoplasm of bone of left upper limb (disorder)
        "354651000119102",  # Primary malignant neoplasm of bone of left lower limb (disorder)
        "354671000119106",  # Primary malignant neoplasm of bone of right upper limb (disorder)
        "354681000119109",  # Primary malignant neoplasm of bone of right lower limb (disorder)
        "354721000119103",  # Primary malignant neoplasm of left main bronchus (disorder)
        "354731000119100",  # Primary malignant neoplasm of right main bronchus (disorder)
        "354851000119101",  # Follicular non-Hodgkin's lymphoma of lymph nodes of multiple sites (disorder)
        "355141000119106",  # Primary malignant neoplasm of skin of right upper limb (disorder)
        "35585004",  # Tuberculous endometritis (disorder)
        "35786001",  # Tuberculoma of meninges (disorder)
        "35868009",  # Carcinoid syndrome (disorder)
        "359631009",  # Acute myeloid leukemia, minimal differentiation, FAB M0 (disorder)
        "359640008",  # Acute myeloid leukemia without maturation, FAB M1 (disorder)
        "359648001",  # Acute myeloid leukemia with maturation, FAB M2 (disorder)
        "359780007",  # Metastatic malignant neoplasm to lateral axillary lymph nodes (disorder)
        "359782004",  # Metastatic malignant neoplasm to apex of urinary bladder (disorder)
        "359785002",  # Metastatic malignant neoplasm to dome of urinary bladder (disorder)
        "35984006",  # Tuberculosis of vertebral column (disorder)
        "359987004",  # Krukenberg tumor (disorder)
        "36222008",  # Carcinoid heart disease (disorder)
        "363346000",  # Malignant neoplastic disease (disorder)
        "363348004",  # Malignant tumor of lip (disorder)
        "363349007",  # Malignant tumor of stomach (disorder)
        "363350007",  # Malignant tumor of cecum (disorder)
        "363351006",  # Malignant tumor of rectum (disorder)
        "363352004",  # Malignant tumor of anal canal (disorder)
        "363353009",  # Malignant tumor of gallbladder (disorder)
        "363354003",  # Malignant tumor of cervix (disorder)
        "363355002",  # Malignant tumor of adrenal gland (disorder)
        "363358000",  # Malignant tumor of lung (disorder)
        "363359008",  # Malignant tumor of middle ear (disorder)
        "363360003",  # Malignant tumor of anterior two-thirds of tongue (disorder)
        "363363001",  # Malignant tumor of soft tissue of shoulder (disorder)
        "363364007",  # Malignant tumor of soft tissue of hip (disorder)
        "363365008",  # Malignant tumor of soft tissue of thorax (disorder)
        "363366009",  # Malignant tumor of soft tissue of pelvis (disorder)
        "363367000",  # Malignant tumor of vulva (disorder)
        "363368005",  # Carcinoma of body of pancreas (disorder)
        "363369002",  # Carcinoma of tail of pancreas (disorder)
        "363370001",  # Malignant neoplasm of mesentery (disorder)
        "363372009",  # Malignant tumor of vermilion border of upper lip (disorder)
        "363373004",  # Malignant tumor of vermilion border of lower lip (disorder)
        "363374005",  # Malignant tumor of commissure of lip (disorder)
        "363375006",  # Malignant tumor of tongue (disorder)
        "363376007",  # Malignant tumor of base of tongue (disorder)
        "363377003",  # Malignant tumor of lingual tonsil (disorder)
        "363378008",  # Malignant tumor of major salivary gland (disorder)
        "363379000",  # Malignant tumor of parotid gland (disorder)
        "363380002",  # Malignant tumor of submandibular gland (disorder)
        "363381003",  # Malignant tumor of sublingual gland (disorder)
        "363382005",  # Malignant tumor of gum (disorder)
        "363383000",  # Malignant tumor of upper gingiva (disorder)
        "363384006",  # Malignant tumor of lower gingiva (disorder)
        "363385007",  # Malignant tumor of floor of mouth (disorder)
        "363386008",  # Malignant tumor of buccal mucosa (disorder)
        "363387004",  # Malignant tumor of hard palate (disorder)
        "363388009",  # Malignant tumor of soft palate (disorder)
        "363389001",  # Malignant tumor of uvula (disorder)
        "363390005",  # Malignant tumor of palate (disorder)
        "363391009",  # Malignant tumor of retromolar area (disorder)
        "363392002",  # Malignant tumor of oropharynx (disorder)
        "363393007",  # Malignant tumor of tonsil (disorder)
        "363394001",  # Malignant tumor of tonsillar fossa (disorder)
        "363395000",  # Malignant tumor of vallecula (disorder)
        "363396004",  # Malignant tumor of branchial cleft (disorder)
        "363398003",  # Malignant tumor of lateral wall of nasopharynx (disorder)
        "363399006",  # Malignant tumor of hypopharynx (disorder)
        "363400004",  # Malignant tumor of postcricoid region (disorder)
        "363401000",  # Malignant tumor of pyriform fossa (disorder)
        "363402007",  # Malignant tumor of esophagus (disorder)
        "363403002",  # Malignant tumor of duodenum (disorder)
        "363404008",  # Malignant tumor of jejunum (disorder)
        "363405009",  # Malignant tumor of ileum (disorder)
        "363406005",  # Malignant neoplasm of colon (disorder)
        "363407001",  # Malignant tumor of hepatic flexure (disorder)
        "363408006",  # Malignant tumor of transverse colon (disorder)
        "363409003",  # Malignant tumor of descending colon (disorder)
        "363410008",  # Malignant tumor of sigmoid colon (disorder)
        "363411007",  # Malignant neoplasm of appendix (disorder)
        "363412000",  # Malignant tumor of ascending colon (disorder)
        "363413005",  # Malignant tumor of splenic flexure (disorder)
        "363414004",  # Malignant tumor of rectosigmoid junction (disorder)
        "363415003",  # Malignant tumor of biliary tract (disorder)
        "363416002",  # Malignant tumor of extrahepatic bile duct (disorder)
        "363417006",  # Malignant tumor of ampulla of Vater (disorder)
        "363418001",  # Malignant tumor of pancreas (disorder)
        "363419009",  # Malignant tumor of head of pancreas (disorder)
        "363420003",  # Malignant retroperitoneal tumor (disorder)
        "363421004",  # Malignant neoplasm of omentum (disorder)
        "363422006",  # Malignant tumor of nasal cavity (disorder)
        "363423001",  # Malignant tumor of nasal septum (disorder)
        "363424007",  # Malignant tumor of mastoid air cells (disorder)
        "363425008",  # Malignant tumor of maxillary sinus (disorder)
        "363426009",  # Malignant tumor of ethmoid sinus (disorder)
        "363427000",  # Malignant tumor of frontal sinus (disorder)
        "363428005",  # Malignant tumor of sphenoid sinus (disorder)
        "363429002",  # Malignant tumor of larynx (disorder)
        "363430007",  # Malignant tumor of subglottis (disorder)
        "363431006",  # Malignant tumor of laryngeal cartilage (disorder)
        "363432004",  # Malignant tumor of trachea (disorder)
        "363433009",  # Malignant tumor of pleura (disorder)
        "363434003",  # Malignant tumor of thymus (disorder)
        "363435002",  # Malignant tumor of heart (disorder)
        "363436001",  # Malignant tumor of endocardium (disorder)
        "363437005",  # Malignant tumor of myocardium (disorder)
        "363438000",  # Malignant neoplasm of vertebral column (disorder)
        "363439008",  # Malignant tumor of soft tissue of head (disorder)
        "363440005",  # Malignant tumor of soft tissue of face (disorder)
        "363441009",  # Malignant tumor of soft tissue of neck (disorder)
        "363443007",  # Malignant tumor of ovary (disorder)
        "363444001",  # Malignant tumor of fallopian tube (disorder)
        "363445000",  # Malignant tumor of vagina (disorder)
        "363446004",  # Malignant neoplasm of labia majora (disorder)
        "363447008",  # Malignant neoplasm of labia minora (disorder)
        "363449006",  # Malignant tumor of testis (disorder)
        "363450006",  # Malignant tumor of foreskin (disorder)
        "363451005",  # Malignant tumor of glans penis (disorder)
        "363452003",  # Malignant tumor of epididymis (disorder)
        "363453008",  # Malignant tumor of spermatic cord (disorder)
        "363454002",  # Malignant tumor of scrotum (disorder)
        "363456000",  # Malignant tumor of urachus (disorder)
        "363457009",  # Malignant tumor of renal pelvis (disorder)
        "363458004",  # Malignant tumor of ureter (disorder)
        "363459007",  # Malignant tumor of urethra (disorder)
        "363460002",  # Malignant tumor of paraurethral gland (disorder)
        "363461003",  # Malignant tumor of eye (disorder)
        "363462005",  # Malignant tumor of orbit (disorder)
        "363463000",  # Malignant tumor of conjunctiva (disorder)
        "363464006",  # Malignant tumor of cornea (disorder)
        "363465007",  # Malignant tumor of retina (disorder)
        "363466008",  # Malignant tumor of choroid (disorder)
        "363467004",  # Malignant neoplasm of frontal lobe (disorder)
        "363468009",  # Malignant neoplasm of temporal lobe (disorder)
        "363469001",  # Malignant neoplasm of parietal lobe (disorder)
        "363470000",  # Malignant neoplasm of occipital lobe (disorder)
        "363471001",  # Malignant neoplasm of cerebral ventricles (disorder)
        "363473003",  # Malignant neoplasm of brainstem (disorder)
        "363474009",  # Malignant neoplasm of cerebral meninges (disorder)
        "363475005",  # Malignant tumor of spinal cord (disorder)
        "363476006",  # Malignant neoplasm of spinal meninges (disorder)
        "363477002",  # Malignant neoplasm of cauda equina (disorder)
        "363478007",  # Malignant tumor of thyroid gland (disorder)
        "363481002",  # Malignant tumor of parathyroid gland (disorder)
        "363482009",  # Malignant tumor of pituitary gland (disorder)
        "363483004",  # Malignant tumor of pineal gland (disorder)
        "363484005",  # Malignant tumor of pelvis (disorder)
        "363485006",  # Malignant tumor of minor salivary gland (disorder)
        "363486007",  # Malignant tumor of vocal cord (disorder)
        "363487003",  # Malignant tumor of aryepiglottic fold - laryngeal aspect (disorder)
        "363488008",  # Malignant tumor of false cord (disorder)
        "363489000",  # Malignant tumor of neck (disorder)
        "363490009",  # Malignant tumor of anus (disorder)
        "363491008",  # Malignant tumor of cloacogenic zone (disorder)
        "363492001",  # Malignant tumor of peritoneum (disorder)
        "363493006",  # Malignant tumor of bronchus (disorder)
        "363494000",  # Malignant tumor of mediastinum (disorder)
        "363495004",  # Malignant tumor of muscle (disorder)
        "363496003",  # Malignant tumor of soft tissue of abdomen (disorder)
        "363497007",  # Malignant tumor of meninges (disorder)
        "363498002",  # Malignant tumor of optic nerve (disorder)
        "363499005",  # Malignant tumor of spleen (disorder)
        "363500001",  # Multiple malignancy (disorder)
        "363501002",  # Malignant tumor of face (disorder)
        "363502009",  # Malignant tumor of axilla (disorder)
        "363503004",  # Malignant tumor of upper limb (disorder)
        "363504005",  # Malignant tumor of lower limb (disorder)
        "363505006",  # Malignant tumor of oral cavity (disorder)
        "363506007",  # Malignant tumor of nasal sinuses (disorder)
        "363507003",  # Malignant neoplasm of pharynx (disorder)
        "363508008",  # Malignant tumor of intestine (disorder)
        "363509000",  # Malignant tumor of small intestine (disorder)
        "363510005",  # Malignant tumor of large intestine (disorder)
        "363514001",  # Malignant tumor of female genital organ (disorder)
        "363515000",  # Malignant tumor of male genital organ (disorder)
        "363516004",  # Malignant tumor of penis (disorder)
        "363517008",  # Malignant tumor of urinary tract proper (disorder)
        "363518003",  # Malignant tumor of kidney (disorder)
        "363745004",  # Primary malignant neoplasm of gastrointestinal tract (disorder)
        "36696005",  # Kaolinosis (disorder)
        "369448007",  # Malignant tumor involving rectum by direct extension from endometrium (disorder)
        "369449004",  # Malignant tumor involving rectum by direct extension from fallopian tube (disorder)
        "369450004",  # Malignant tumor involving rectum by direct extension from ovary (disorder)
        "369451000",  # Malignant tumor involving rectum by direct extension from prostate (disorder)
        "369452007",  # Malignant tumor involving rectum by direct extension from uterine cervix (disorder)
        "369453002",  # Malignant tumor involving rectum by direct extension from uterus (disorder)
        "369454008",  # Malignant tumor involving rectum by direct extension from vagina (disorder)
        "369463005",  # Malignant tumor involving ureter by direct extension from bladder (disorder)
        "369465003",  # Malignant tumor involving urethra by direct extension from bladder (disorder)
        "369466002",  # Malignant tumor involving urethra by direct extension from prostate (disorder)
        "369469009",  # Malignant neoplasm involving urinary bladder by direct extension from endometrium (disorder)
        "369470005",  # Malignant neoplasm involving urinary bladder by direct extension from fallopian tube (disorder)
        "369471009",  # Malignant neoplasm involving urinary bladder by direct extension from ovary (disorder)
        "369472002",  # Malignant neoplasm involving urinary bladder by direct extension from prostate (disorder)
        "369473007",  # Malignant neoplasm involving urinary bladder by direct extension from uterine cervix (disorder)
        "369474001",  # Malignant neoplasm involving urinary bladder by direct extension from uterus (disorder)
        "369475000",  # Malignant neoplasm involving urinary bladder by direct extension from vagina (disorder)
        "369483006",  # Malignant tumor involving vasa deferentia by direct extension from prostate (disorder)
        "369485004",  # Malignant tumor involving prostate by direct extension from bladder (disorder)
        "369487007",  # Primary malignant neoplasm of seminal vesicle (disorder)
        "369488002",  # Metastatic malignant neoplasm to seminal vesicle (disorder)
        "369489005",  # Malignant tumor involving seminal vesicle by direct extension from bladder (disorder)
        "369490001",  # Malignant tumor involving seminal vesicle by direct extension from prostate (disorder)
        "369493004",  # Malignant tumor involving uterine corpus by direct extension from bladder (disorder)
        "369494005",  # Malignant tumor involving uterine corpus by direct extension from ovary (disorder)
        "369495006",  # Malignant tumor involving uterine corpus by direct extension from uterine cervix (disorder)
        "369496007",  # Malignant tumor involving uterine corpus by direct extension from vagina (disorder)
        "369497003",  # Malignant tumor involving uterine cervix by direct extension from fallopian tube (disorder)
        "369498008",  # Malignant tumor involving uterine cervix by direct extension from ovary (disorder)
        "369499000",  # Malignant tumor involving uterine cervix by direct extension from vagina (disorder)
        "369503006",  # Malignant tumor involving vagina by direct extension from uterus (disorder)
        "369504000",  # Malignant tumor involving vagina by direct extension from bladder (disorder)
        "369505004",  # Malignant tumor involving vagina by direct extension from endometrium (disorder)
        "369506003",  # Malignant tumor involving vagina by direct extension from fallopian tube (disorder)
        "369507007",  # Malignant tumor involving vulva by direct extension from endometrium (disorder)
        "369508002",  # Malignant tumor involving vulva by direct extension from fallopian tube (disorder)
        "369509005",  # Malignant tumor involving vulva by direct extension from ovary (disorder)
        "369510000",  # Malignant tumor involving vulva by direct extension from uterine cervix (disorder)
        "369511001",  # Malignant tumor involving vulva by direct extension from uterus (disorder)
        "369512008",  # Malignant tumor involving vagina by direct extension from ovary (disorder)
        "369513003",  # Primary malignant neoplasm of left fallopian tube (disorder)
        "369514009",  # Metastatic malignant neoplasm to left fallopian tube (disorder)
        "369515005",  # Malignant tumor involving left fallopian tube by direct extension from endometrium (disorder)
        "369516006",  # Malignant tumor involving left fallopian tube by direct extension from ovary (disorder)
        "369517002",  # Malignant tumor involving left fallopian tube by direct extension from right fallopian tube (disorder)
        "369518007",  # Malignant tumor involving left fallopian tube by direct extension from uterine cervix (disorder)
        "369519004",  # Malignant tumor involving left fallopian tube by direct extension from uterus (disorder)
        "369520005",  # Primary malignant neoplasm of right fallopian tube (disorder)
        "369521009",  # Metastatic malignant neoplasm to right fallopian tube (disorder)
        "369522002",  # Primary malignant neoplasm of left ovary (disorder)
        "369523007",  # Metastatic malignant neoplasm to left ovary (disorder)
        "369524001",  # Malignant tumor involving left ovary by direct extension from endometrium (disorder)
        "369525000",  # Malignant tumor involving left ovary by direct extension from fallopian tube (disorder)
        "369526004",  # Malignant tumor involving left ovary by direct extension from right ovary (disorder)
        "369527008",  # Malignant tumor involving left ovary by direct extension from uterine cervix (disorder)
        "369528003",  # Malignant tumor involving left ovary by direct extension from uterus (disorder)
        "369529006",  # Primary malignant neoplasm of right ovary (disorder)
        "369530001",  # Metastatic malignant neoplasm to right ovary (disorder)
        "369531002",  # Malignant tumor involving right ovary by direct extension from endometrium (disorder)
        "369532009",  # Malignant tumor involving right ovary by direct extension from fallopian tube (disorder)
        "369533004",  # Malignant tumor involving right ovary by direct extension from left ovary (disorder)
        "369534005",  # Malignant tumor involving right ovary by direct extension from uterine cervix (disorder)
        "369535006",  # Metastatic neoplasm of left broad ligament (disorder)
        "369536007",  # Metastatic neoplasm of right broad ligament (disorder)
        "369541004",  # Malignant tumor involving left fallopian tube by direct extension from vagina (disorder)
        "369547000",  # Malignant tumor involving right fallopian tube by direct extension from endometrium (disorder)
        "369548005",  # Malignant tumor involving right fallopian tube by direct extension from left fallopian tube (disorder)
        "369549002",  # Malignant tumor involving right fallopian tube by direct extension from ovary (disorder)
        "369550002",  # Malignant tumor involving right fallopian tube by direct extension from uterine cervix (disorder)
        "369551003",  # Malignant tumor involving right fallopian tube by direct extension from uterus (disorder)
        "369552005",  # Malignant tumor involving right fallopian tube by direct extension from vagina (disorder)
        "369559001",  # Malignant tumor involving left ovary by direct extension from vagina (disorder)
        "369566000",  # Malignant tumor involving right ovary by direct extension from uterus (disorder)
        "369567009",  # Malignant tumor involving right ovary by direct extension from vagina (disorder)
        "369579009",  # Malignant tumor involving uterine corpus by direct extension from fallopian tube (disorder)
        "369580007",  # Malignant tumor involving vagina by direct extension from uterine cervix (disorder)
        "369587005",  # Malignant tumor involving vulva by direct extension from vagina (disorder)
        "369594008",  # Malignant tumor involving an organ by direct extension from bladder (disorder)
        "369595009",  # Malignant tumor involving an organ by direct extension from endometrium (disorder)
        "369596005",  # Malignant tumor involving an organ by direct extension from fallopian tube (disorder)
        "369597001",  # Malignant tumor involving an organ by direct extension from ovary (disorder)
        "369598006",  # Malignant tumor involving an organ by direct extension from prostate (disorder)
        "369599003",  # Malignant tumor involving an organ by direct extension from uterine cervix (disorder)
        "369600000",  # Malignant tumor involving an organ by direct extension from uterus (disorder)
        "369601001",  # Malignant tumor involving an organ by direct extension from vagina (disorder)
        "370511006",  # Vaccine-induced fibrosarcoma (disorder)
        "370967009",  # Retinoblastoma (disorder)
        "370987005",  # Anaplastic astrocytoma of spinal cord (disorder)
        "371012000",  # Acute lymphoblastic leukemia, transitional pre-B-cell (disorder)
        "371134001",  # Malignant lymphoma, large cell, polymorphous, immunoblastic (disorder)
        "37133005",  # Tuberculous pyelitis (disorder)
        "371962007",  # Primary malignant neoplasm of abdominal esophagus (disorder)
        "371963002",  # Primary malignant neoplasm of adrenal cortex (disorder)
        "371964008",  # Malignant neoplasm of adrenal cortex (disorder)
        "371965009",  # Malignant neoplasm of adrenal medulla (disorder)
        "371966005",  # Primary malignant neoplasm of adrenal medulla (disorder)
        "371967001",  # Primary malignant neoplasm of ampulla of Vater (disorder)
        "371968006",  # Primary malignant neoplasm of anterior two-thirds of tongue (disorder)
        "371969003",  # Primary malignant neoplasm of apex of urinary bladder (disorder)
        "371970002",  # Primary malignant neoplasm of biliary tract (disorder)
        "371971003",  # Primary malignant neoplasm of body of uterus (disorder)
        "371972005",  # Malignant neoplasm of body of uterus (disorder)
        "371973000",  # Malignant neoplasm of uterus (disorder)
        "371974006",  # Malignant neoplasm of border of tongue (disorder)
        "371975007",  # Primary malignant neoplasm of border of tongue (disorder)
        "371976008",  # Primary malignant neoplasm of buccal mucosa (disorder)
        "371977004",  # Primary malignant neoplasm of cecum (disorder)
        "371978009",  # Primary malignant neoplasm of cervical esophagus (disorder)
        "371979001",  # Malignant neoplasm of clitoris (disorder)
        "371980003",  # Primary malignant neoplasm of clitoris (disorder)
        "371981004",  # Primary malignant neoplasm of commissure of lip (disorder)
        "371982006",  # Malignant neoplasm of endocrine gland (disorder)
        "371983001",  # Primary malignant neoplasm of endocrine gland (disorder)
        "371984007",  # Primary malignant neoplasm of esophagus (disorder)
        "371986009",  # Primary malignant neoplasm of eye (disorder)
        "371987000",  # Primary malignant neoplasm of fallopian tube (disorder)
        "371988005",  # Primary malignant neoplasm of false vocal cord (disorder)
        "371989002",  # Primary malignant neoplasm of glans penis (disorder)
        "371990006",  # Primary malignant neoplasm of gum (disorder)
        "371991005",  # Primary malignant neoplasm of hard palate (disorder)
        "371992003",  # Primary malignant neoplasm of intestinal tract (disorder)
        "371993008",  # Primary malignant neoplasm of lacrimal gland (disorder)
        "371994002",  # Primary malignant neoplasm of laryngeal aspect of aryepiglottic fold (disorder)
        "371995001",  # Primary malignant neoplasm of larynx (disorder)
        "371996000",  # Primary malignant neoplasm of lip (disorder)
        "371997009",  # Primary malignant neoplasm of lower gum (disorder)
        "371998004",  # Primary malignant neoplasm of lower third of esophagus (disorder)
        "371999007",  # Primary malignant neoplasm of middle third of esophagus (disorder)
        "372000001",  # Primary malignant neoplasm of minor salivary gland (disorder)
        "372001002",  # Primary malignant neoplasm of oral cavity (disorder)
        "372002009",  # Primary malignant neoplasm of palate (disorder)
        "372003004",  # Primary malignant neoplasm of pancreas (disorder)
        "372004005",  # Primary malignant neoplasm of parotid gland (disorder)
        "372005006",  # Primary malignant neoplasm of penis (disorder)
        "372006007",  # Primary malignant neoplasm of prepuce (disorder)
        "372007003",  # Primary malignant neoplasm of retrocecal tissue (disorder)
        "372008008",  # Primary malignant neoplasm of scaphoid bone (disorder)
        "372009000",  # Primary malignant neoplasm of scrotum (disorder)
        "372010005",  # Primary malignant neoplasm of soft tissues (disorder)
        "372011009",  # Malignant tumor of soft tissue of upper limb (disorder)
        "372012002",  # Primary malignant neoplasm of soft tissues of upper limb (disorder)
        "372013007",  # Primary malignant neoplasm of spermatic cord (disorder)
        "372014001",  # Primary malignant neoplasm of stomach (disorder)
        "372015000",  # Primary malignant neoplasm of the mesentery (disorder)
        "372016004",  # Primary malignant neoplasm of the peritoneum (disorder)
        "372017008",  # Primary malignant neoplasm of thoracic esophagus (disorder)
        "372018003",  # Malignant neoplasm of thoracic vertebral column (disorder)
        "372019006",  # Primary malignant neoplasm of thoracic vertebral column (disorder)
        "372020000",  # Primary malignant neoplasm of tonsil (disorder)
        "372021001",  # Primary malignant neoplasm of trapezium (disorder)
        "372022008",  # Primary malignant neoplasm of upper gum (disorder)
        "372023003",  # Primary malignant neoplasm of upper third of esophagus (disorder)
        "372024009",  # Primary malignant neoplasm of uterine cervix (disorder)
        "372025005",  # Primary malignant neoplasm of vagina (disorder)
        "372026006",  # Primary malignant neoplasm of vermilion border of lower lip (disorder)
        "372027002",  # Primary malignant neoplasm of vermilion border of upper lip (disorder)
        "372028007",  # Primary malignant neoplasm of vertebral column (disorder)
        "372030009",  # Primary malignant neoplasm of vocal cord (disorder)
        "372062007",  # Malignant neoplasm of central nervous system (disorder)
        "372063002",  # Malignant neoplasm of nervous system (disorder)
        "372065009",  # Malignant neoplasm of main bronchus (disorder)
        "372087000",  # Primary malignant neoplasm (disorder)
        "372092003",  # Primary malignant neoplasm of axillary tail of breast (disorder)
        "372093008",  # Metastatic malignant neoplasm to axillary tail of breast (disorder)
        "372094002",  # Malignant neoplasm of axillary tail of breast (disorder)
        "372097009",  # Malignant neoplasm of endocervix (disorder)
        "372098004",  # Carcinoma of endocervix (disorder)
        "372099007",  # Malignant neoplasm of exocervix (disorder)
        "372100004",  # Carcinoma of exocervix (disorder)
        "372101000",  # Carcinoma of extrahepatic bile duct (disorder)
        "372103002",  # Carcinoma of glottis (disorder)
        "372104008",  # Carcinoma of subglottis (disorder)
        "372105009",  # Carcinoma of supraglottis (disorder)
        "372106005",  # Carcinoma of penis (disorder)
        "372107001",  # Primary malignant neoplasm of ribs and/or sternum and/or clavicle (disorder)
        "372108006",  # Malignant neoplasm of bone of lower limb (disorder)
        "372110008",  # Primary malignant neoplasm of lower lobe, bronchus or lung (disorder)
        "372111007",  # Carcinoma of lower lobe of lung (disorder)
        "372115003",  # Primary malignant neoplasm of pelvic bones, sacrum and coccyx (disorder)
        "372116002",  # Carcinoma of pelvic bones, sacrum and coccyx (disorder)
        "372119009",  # Primary malignant neoplasm of head of pancreas (disorder)
        "372120003",  # Carcinoma of main bronchus (disorder)
        "372121004",  # Carcinoma of ribs and/or sternum and/or clavicle (disorder)
        "372122006",  # Malignant neoplasm of skin head and neck (disorder)
        "372123001",  # Primary malignant neoplasm of skin head and neck (disorder)
        "372124007",  # Malignant neoplasm of skin of lower limb (disorder)
        "372125008",  # Carcinoma of skin of lower limb (disorder)
        "372126009",  # Malignant neoplasm of skin of trunk (disorder)
        "372127000",  # Carcinoma of skin of trunk (disorder)
        "372128005",  # Malignant neoplasm of skin of upper limb (disorder)
        "372129002",  # Carcinoma of skin of upper limb (disorder)
        "372130007",  # Malignant neoplasm of skin (disorder)
        "372131006",  # Malignant neoplasm of upper limb bones and scapula (disorder)
        "372133009",  # Primary malignant neoplasm of upper limb bones and scapula (disorder)
        "372137005",  # Primary malignant neoplasm of breast (disorder)
        "372138000",  # Carcinoma of esophagus (disorder)
        "372139008",  # Primary malignant neoplasm of gallbladder (disorder)
        "372140005",  # Carcinoma of gallbladder (disorder)
        "372141009",  # Carcinoma of vocal cord (disorder)
        "372142002",  # Carcinoma of pancreas (disorder)
        "372143007",  # Carcinoma of stomach (disorder)
        "372244006",  # Malignant melanoma (disorder)
        "373080008",  # Malignant neoplasm of lower inner quadrant of breast (disorder)
        "373081007",  # Malignant neoplasm of lower outer quadrant of breast (disorder)
        "373082000",  # Malignant neoplasm of upper inner quadrant of breast (disorder)
        "373083005",  # Malignant neoplasm of upper outer quadrant of breast (disorder)
        "373088001",  # Primary malignant neoplasm of upper outer quadrant of breast (disorder)
        "373089009",  # Primary malignant neoplasm of upper inner quadrant of breast (disorder)
        "373090000",  # Primary malignant neoplasm of lower inner quadrant of breast (disorder)
        "373091001",  # Primary malignant neoplasm of lower outer quadrant of breast (disorder)
        "373168002",  # Reticulosarcoma (disorder)
        "373888000",  # Extrauterine adenocarcinoma (disorder)
        "37711000",  # Cadmium pneumonitis (disorder)
        "37981002",  # Allergic bronchopulmonary aspergillosis (disorder)
        "38115001",  # Tuberculosis of spinal meninges (disorder)
        "38279006",  # Tuberculosis of bone (disorder)
        "38420005",  # Tuberculosis of large intestine (disorder)
        "385478001",  # Adenoma malignum (disorder)
        "389216001",  # Diaphyseal medullary stenosis with bone malignancy (disorder)
        "39306006",  # Tuberculosis of anus (disorder)
        "393563007",  # Glioblastoma multiforme (disorder)
        "393573009",  # Hypereosinophilic syndrome (disorder)
        "397008008",  # Aggressive lymphadenopathic mastocytosis with eosinophilia (disorder)
        "397009000",  # Mast cell malignancy (disorder)
        "397011009",  # Mast cell malignancy of lymph nodes (disorder)
        "397015000",  # Systemic mastocytosis with associated clonal hematological non-mast cell lineage disease (disorder)
        "398623004",  # Myelodysplastic syndrome with excess blasts (disorder)
        "398640008",  # Rheumatoid arthritis with pneumoconiosis (disorder)
        "398670003",  # Pigmented dermatofibrosarcoma protuberans of skin (disorder)
        "398903003",  # Pleomorphic malignant fibrous histiocytoma of skin (disorder)
        "399068003",  # Malignant tumor of prostate (disorder)
        "399326009",  # Malignant neoplasm of urinary bladder (disorder)
        "399490008",  # Adenocarcinoma of prostate (disorder)
        "399590005",  # Squamous cell carcinoma of prostate (disorder)
        "399660006",  # Ring melanoma of ciliary body (disorder)
        "399884001",  # Primary malignant neoplasm of blood vessel of upper arm (disorder)
        "399897004",  # Basal cell carcinoma of postauricular skin (disorder)
        "399934007",  # Basal cell carcinoma of preauricular skin (disorder)
        "399954008",  # Basal cell carcinoma of earlobe (disorder)
        "399969009",  # Metastatic malignant neoplasm to blood vessel of upper arm (disorder)
        "400001003",  # Primary cutaneous lymphoma (disorder)
        "400057007",  # Basal cell carcinoma of pinnal sulcus (disorder)
        "400058002",  # Metastatic malignant neoplasm to blood vessel of lower leg (disorder)
        "400074007",  # Primary malignant neoplasm of blood vessel of lower leg (disorder)
        "400122007",  # Primary cutaneous T-cell lymphoma (disorder)
        "400173004",  # Eccrine carcinoma of skin (disorder)
        "400174005",  # Basal cell carcinoma of antihelix of ear (disorder)
        "400175006",  # Myxofibrosarcoma of skin (disorder)
        "40218008",  # Carbon electrode makers' pneumoconiosis (disorder)
        "402494003",  # Basal cell carcinoma of lower eyelid (disorder)
        "402495002",  # Basal cell carcinoma of medial canthus (disorder)
        "402496001",  # Basal cell carcinoma of lateral canthus (disorder)
        "402497005",  # Basal cell carcinoma of root of nose (disorder)
        "402498000",  # Basal cell carcinoma of dorsum of nose (disorder)
        "402499008",  # Basal cell carcinoma of lateral side wall of nose (disorder)
        "402500004",  # Basal cell carcinoma of ala nasi (disorder)
        "402501000",  # Basal cell carcinoma of supratip of nose (disorder)
        "402502007",  # Basal cell carcinoma of tip of nose (disorder)
        "402503002",  # Basal cell carcinoma of nasal columella (disorder)
        "402504008",  # Basal cell carcinoma of nasolabial groove (disorder)
        "402505009",  # Basal cell carcinoma of upper lip (disorder)
        "402506005",  # Basal cell carcinoma of lower lip (disorder)
        "402507001",  # Basal cell carcinoma of cheek (disorder)
        "402508006",  # Basal cell carcinoma of chin (disorder)
        "402509003",  # Basal cell carcinoma of neck (disorder)
        "402510008",  # Basal cell carcinoma of helix of ear (disorder)
        "402511007",  # Basal cell carcinoma of conchal bowl of ear (disorder)
        "402512000",  # Basal cell carcinoma of antitragus (disorder)
        "402513005",  # Basal cell carcinoma of tragus (disorder)
        "402514004",  # Basal cell carcinoma of obverse of pinna (disorder)
        "402515003",  # Basal cell carcinoma of anterior chest (disorder)
        "402516002",  # Basal cell carcinoma of abdomen (disorder)
        "402517006",  # Basal cell carcinoma of upper back (disorder)
        "402518001",  # Basal cell carcinoma of lower back (disorder)
        "402519009",  # Basal cell carcinoma of face (disorder)
        "402520003",  # Basal cell carcinoma of hand (disorder)
        "402521004",  # Basal cell carcinoma of upper extremity (disorder)
        "402522006",  # Basal cell carcinoma of lower extremity (disorder)
        "402523001",  # Basal cell carcinoma of truncal skin (disorder)
        "402524007",  # Basal cell carcinoma - adamantinoid (disorder)
        "402525008",  # Basal cell carcinoma - adenoid (disorder)
        "402526009",  # Basal cell carcinoma - follicular (disorder)
        "402527000",  # Basal cell carcinoma - infiltrative (disorder)
        "402528005",  # Basal cell carcinoma - keratotic (disorder)
        "402529002",  # Basal cell carcinoma - micronodular (disorder)
        "402530007",  # Basal cell carcinoma with matrical differentiation (disorder)
        "402531006",  # Basal cell carcinoma with granular cell change (disorder)
        "402532004",  # Basal cell carcinoma with monster cells (disorder)
        "402533009",  # Basal cell carcinoma with sebaceous differentiation (disorder)
        "402534003",  # Basal cell carcinoma with signet ring change (disorder)
        "402536001",  # Basal cell carcinoma - primary (disorder)
        "402537005",  # Metastatic basal cell carcinoma (disorder)
        "402538000",  # Recurrent basal cell carcinoma (disorder)
        "402539008",  # Basal cell carcinoma recurrent following excision (disorder)
        "402540005",  # Basal cell carcinoma recurrent following Mohs' excision (disorder)
        "402541009",  # Basal cell carcinoma recurrent following curettage (disorder)
        "402542002",  # Basal cell carcinoma recurrent following cryosurgery (disorder)
        "402543007",  # Basal cell carcinoma recurrent following radiotherapy (disorder)
        "402544001",  # Basal cell carcinoma - first recurrence (disorder)
        "402545000",  # Basal cell carcinoma - second recurrence (disorder)
        "402546004",  # Basal cell carcinoma - third recurrence (disorder)
        "402547008",  # Basal cell carcinoma - multiple recurrences (more than three times) (disorder)
        "402558004",  # Malignant melanoma (vertical growth phase) (disorder)
        "402559007",  # Congenital malignant melanoma (disorder)
        "402560002",  # Materno-fetal metastatic malignant melanoma (disorder)
        "402561003",  # Malignant melanoma of soft tissues (disorder)
        "402562005",  # Malignant melanoma animal-type (disorder)
        "402563000",  # Metastatic malignant melanoma with diffuse hypermelanosis (disorder)
        "402564006",  # Multiple primary malignant melanomata (disorder)
        "40258005",  # Tuberculosis of testis (disorder)
        "402636006",  # Malignant neoplasm of nail apparatus (disorder)
        "402652009",  # Malignant vascular tumor of skin (disorder)
        "402815007",  # Squamous cell carcinoma (disorder)
        "402816008",  # Squamous cell carcinoma of anogenital area (disorder)
        "402817004",  # Squamous cell carcinoma of nail apparatus (disorder)
        "402818009",  # Basal cell carcinoma of nose (disorder)
        "402819001",  # Basal cell carcinoma of skin of lip (disorder)
        "402820007",  # Basal cell carcinoma of ear (disorder)
        "402871009",  # Malignant neoplasm of subcutaneous fibrous tissue (disorder)
        "402873007",  # Malignant fibrohistiocytic tumor of skin (disorder)
        "402876004",  # Malignant tumor of nerve sheath origin (disorder)
        "402879006",  # T-cell leukemic infiltration of skin (disorder)
        "402880009",  # Primary cutaneous large T-cell lymphoma (disorder)
        "402881008",  # Primary cutaneous B-cell lymphoma (disorder)
        "402882001",  # Hodgkin's disease affecting skin (disorder)
        "402910001",  # Anogenital verrucous carcinoma of Buschke-Lowenstein (disorder)
        "402911002",  # Penile verrucous carcinoma of Buschke-Lowenstein (disorder)
        "402912009",  # Vulval verrucous carcinoma of Buschke-Lowenstein (disorder)
        "402973006",  # Bacillus Calmette-Guerin-induced lupus vulgaris (disorder)
        "402974000",  # Bacillus Calmette-Guerin-induced scrofuloderma (disorder)
        "402975004",  # Bacillus Calmette-Guerin-induced tuberculide (disorder)
        "40312006",  # Pericarditis co-occurrent and due to tumor metastatic to pericardium (disorder)
        "403458008",  # Localized skin involvement by breast carcinoma (disorder)
        "403468003",  # Squamous cell carcinoma of penis (disorder)
        "403711001",  # Primary malignant neoplasm of skin due to psoralen and longwave ultraviolet radiation therapy (disorder)
        "403712008",  # Psoralen and long-wave ultraviolet radiation therapy-associated basal cell carcinoma (disorder)
        "403713003",  # Psoralen and long-wave ultraviolet radiation therapy-associated squamous cell carcinoma (disorder)
        "403729006",  # Malignant neoplasm of skin caused by radiation (disorder)
        "403742006",  # Malignant neoplasm of skin caused by arsenic (disorder)
        "403889000",  # Verrucous carcinoma of oral cavity (disorder)
        "403890009",  # Squamous cell carcinoma of vulva due to lichen sclerosus (disorder)
        "403891008",  # Squamous cell carcinoma of scalp (disorder)
        "403892001",  # Squamous cell carcinoma of skin of face (disorder)
        "403893006",  # Squamous cell carcinoma of forehead (disorder)
        "403894000",  # Squamous cell carcinoma of skin of ear (disorder)
        "403895004",  # Squamous cell carcinoma of foot (disorder)
        "403896003",  # Squamous cell carcinoma of hand (disorder)
        "403897007",  # Squamous cell carcinoma of upper extremity (disorder)
        "403898002",  # Squamous cell carcinoma of skin of lower extremity (disorder)
        "403899005",  # Squamous cell carcinoma of skin of trunk (disorder)
        "403900000",  # Spindle cell squamous cell carcinoma (disorder)
        "403901001",  # Acantholytic squamous cell carcinoma (disorder)
        "403902008",  # Adenosquamous carcinoma (disorder)
        "403903003",  # Signet ring squamous cell carcinoma (disorder)
        "403904009",  # Verrucous squamous cell carcinoma (disorder)
        "403905005",  # Multiple squamous cell carcinomata (disorder)
        "403906006",  # Metastatic squamous cell carcinoma (disorder)
        "403909004",  # Pigmented basal cell carcinoma (disorder)
        "403910009",  # Circumscribed solid basal cell carcinoma (disorder)
        "403911008",  # Nodulo-ulcerative basal cell carcinoma (disorder)
        "403912001",  # Cystic basal cell carcinoma (disorder)
        "403913006",  # Sclerosing morpheic basal cell carcinoma (disorder)
        "403914000",  # Superficial basal cell carcinoma (disorder)
        "403915004",  # Basal cell carcinoma of scalp (disorder)
        "403916003",  # Basal cell carcinoma of forehead (disorder)
        "403917007",  # Basal cell carcinoma of temple (disorder)
        "403918002",  # Basal cell carcinoma of glabella (disorder)
        "403919005",  # Basal cell carcinoma of upper eyelid (disorder)
        "403921000",  # Borderline malignant melanoma (disorder)
        "403922007",  # Balloon cell malignant melanoma (disorder)
        "403923002",  # Spindle cell malignant melanoma (disorder)
        "403924008",  # Desmoplastic malignant melanoma (disorder)
        "403925009",  # Neurotropic malignant melanoma (disorder)
        "403926005",  # Malignant melanoma of oral cavity (disorder)
        "403927001",  # Malignant melanoma of nail apparatus (disorder)
        "403929003",  # Trichilemmal carcinoma (disorder)
        "403939009",  # Eccrine ductal carcinoma of skin (disorder)
        "403940006",  # Clear cell eccrine hidradenocarcinoma of skin (disorder)
        "403941005",  # Malignant cylindroma of skin (disorder)
        "403942003",  # Malignant eccrine spiradenoma of skin (disorder)
        "403943008",  # Malignant chondroid syringoma of skin (disorder)
        "403944002",  # Small cell eccrine carcinoma of skin (disorder)
        "403946000",  # Paget's disease of nipple (disorder)
        "403947009",  # Perianal Paget's disease (disorder)
        "403949007",  # Apocrine adenocarcinoma of skin (disorder)
        "403950007",  # Moll's gland adenocarcinoma (disorder)
        "403951006",  # Ceruminous gland adenocarcinoma of skin (disorder)
        "403954003",  # Mixed eccrine/pilar adnexal carcinoma of skin (disorder)
        "403955002",  # Undifferentiated adnexal carcinoma of skin (disorder)
        "403977003",  # Angiosarcoma (disorder)
        "403978008",  # Kaposi's sarcoma - sporadic (disorder)
        "403979000",  # Endemic Kaposi sarcoma (disorder)
        "403981003",  # Epithelioid hemangioendothelioma (disorder)
        "403986008",  # Lymphangiosarcoma (disorder)
        "403996004",  # Infantile fibrosarcoma (disorder)
        "404008001",  # Dermatofibrosarcoma protuberans with myoid differentiation (disorder)
        "404009009",  # Dermatofibrosarcoma protuberans with granular cell change (disorder)
        "404010004",  # Dermatofibrosarcoma protuberans with giant cell fibroblastoma (disorder)
        "404014008",  # Malignant fibrous histiocytoma of skin (disorder)
        "404015009",  # Superficial malignant fibrous histiocytoma of skin (disorder)
        "404016005",  # Giant cell malignant fibrous histiocytoma of skin (disorder)
        "404017001",  # Inflammatory malignant fibrous histiocytoma of skin (disorder)
        "404037002",  # Malignant peripheral nerve sheath tumor (disorder)
        "404038007",  # Epithelioid malignant nerve sheath tumor (disorder)
        "404039004",  # Melanotic malignant nerve sheath tumor (disorder)
        "404040002",  # Malignant Triton tumor (disorder)
        "404041003",  # Malignant granular cell tumor (disorder)
        "404045007",  # Epithelioid leiomyosarcoma of skin (disorder)
        "404046008",  # Myxoid leiomyosarcoma of skin (disorder)
        "404047004",  # Cutaneous leiomyosarcoma with granular cell change (disorder)
        "404051002",  # Embryonal rhabdomyosarcoma (disorder)
        "404052009",  # Botryoid rhabdomyosarcoma (disorder)
        "404053004",  # Alveolar rhabdomyosarcoma (disorder)
        "404054005",  # Pleomorphic rhabdomyosarcoma (disorder)
        "404055006",  # Spindle cell rhabdomyosarcoma (disorder)
        "404056007",  # Alveolar soft part sarcoma (disorder)
        "404067008",  # Adipocytic liposarcoma (disorder)
        "404068003",  # Sclerosing liposarcoma (disorder)
        "404069006",  # Myxoid liposarcoma (disorder)
        "404071006",  # Pleomorphic liposarcoma (disorder)
        "404072004",  # Dedifferentiated liposarcoma (disorder)
        "404073009",  # Spindle cell liposarcoma (disorder)
        "404077005",  # Extraskeletal osteosarcoma (disorder)
        "404079008",  # Extraskeletal myxoid chondrosarcoma (disorder)
        "404080006",  # Extraskeletal mesenchymal chondrosarcoma (disorder)
        "404081005",  # Yolk sac tumor (disorder)
        "404087009",  # Carcinosarcoma of skin (disorder)
        "404088004",  # Low-grade fibromyxoid sarcoma (disorder)
        "404089007",  # Extrarenal rhabdoid tumor (disorder)
        "404090003",  # Malignant infiltration of oral cavity by underlying tumor (disorder)
        "404091004",  # Malignant infiltration of skin by underlying tumor (disorder)
        "404092006",  # Metastatic carcinoma to skin (disorder)
        "404093001",  # Sarcomatous metastasis in skin (disorder)
        "404094007",  # Metastasis involving oral cavity (disorder)
        "404103007",  # Lymphomatoid papulosis type A (CD-30 positive type) (disorder)
        "404104001",  # Lymphomatoid papulosis type B - mycosis fungoides-like (disorder)
        "404105000",  # Lymphomatoid papulosis type C (anaplastic large-cell lymphoma-like) (disorder)
        "404106004",  # Lymphomatoid papulosis with Hodgkin's disease (disorder)
        "404107008",  # Patch/plaque stage mycosis fungoides (disorder)
        "404108003",  # Poikilodermatous mycosis fungoides (disorder)
        "404109006",  # Follicular mucinosis type mycosis fungoides (disorder)
        "404110001",  # Hypomelanotic mycosis fungoides (disorder)
        "404111002",  # Lymphomatoid papulosis-associated mycosis fungoides (disorder)
        "404112009",  # Granulomatous mycosis fungoides (disorder)
        "404113004",  # Tumor stage mycosis fungoides (disorder)
        "404114005",  # Erythrodermic mycosis fungoides (disorder)
        "404115006",  # Bullous mycosis fungoides (disorder)
        "404116007",  # Mycosis fungoides with systemic infiltration (disorder)
        "404117003",  # Spongiotic mycosis fungoides (disorder)
        "404118008",  # Syringotropic mycosis fungoides (disorder)
        "404119000",  # Pagetoid reticulosis (disorder)
        "404120006",  # Localized pagetoid reticulosis (disorder)
        "404121005",  # Generalized pagetoid reticulosis (disorder)
        "404122003",  # Leukemic infiltration of skin (chronic T-cell lymphocytic leukemia) (disorder)
        "404123008",  # Leukemic infiltration of skin (T-cell prolymphocytic leukemia) (disorder)
        "404124002",  # Leukemic infiltration of skin (T-cell lymphoblastic leukemia) (disorder)
        "404126000",  # CD-30 positive pleomorphic large T-cell cutaneous lymphoma (disorder)
        "404127009",  # CD-30 positive T-immunoblastic cutaneous lymphoma (disorder)
        "404128004",  # CD-30 negative cutaneous T-cell lymphoma (disorder)
        "404129007",  # CD-30 negative anaplastic large T-cell cutaneous lymphoma (disorder)
        "404130002",  # CD-30 negative pleomorphic large T-cell cutaneous lymphoma (disorder)
        "404131003",  # CD-30 negative T-immunoblastic cutaneous lymphoma (disorder)
        "404133000",  # Subcutaneous panniculitic cutaneous T-cell lymphoma (disorder)
        "404134006",  # Anaplastic large T-cell systemic malignant lymphoma (disorder)
        "404135007",  # Angiocentric natural killer/T-cell malignant lymphoma involving skin (disorder)
        "404136008",  # Aggressive natural killer-cell leukemia involving skin (disorder)
        "404137004",  # Precursor B-cell lymphoblastic lymphoma involving skin (disorder)
        "404138009",  # Small lymphocytic B-cell lymphoma involving skin (disorder)
        "404139001",  # Leukemic infiltration of skin in hairy-cell leukemia (disorder)
        "404140004",  # Primary cutaneous marginal zone B-cell lymphoma (disorder)
        "404141000",  # Primary cutaneous immunocytoma (disorder)
        "404142007",  # Primary cutaneous plasmacytoma (disorder)
        "404143002",  # Primary cutaneous follicular center B-cell lymphoma (disorder)
        "404144008",  # Primary cutaneous diffuse large cell B-cell lymphoma (disorder)
        "404147001",  # Follicular center B-cell lymphoma (nodal/systemic with skin involvement) (disorder)
        "404148006",  # Diffuse large B-cell lymphoma (nodal/systemic with skin involvement) (disorder)
        "404149003",  # Lymphoplasmacytic B-cell lymphoma, nodal/systemic with skin involvement (disorder)
        "404150003",  # Mantle cell B-cell lymphoma (nodal/systemic with skin involvement) (disorder)
        "404151004",  # Leukemic infiltration of skin in myeloid leukemia (disorder)
        "404152006",  # Leukemic infiltration of skin in acute myeloid leukemia (disorder)
        "404153001",  # Leukemic infiltration of skin in chronic myeloid leukemia (disorder)
        "404154007",  # Leukemic infiltration of skin in monocytic leukemia (disorder)
        "404155008",  # Granulocytic sarcoma affecting skin (disorder)
        "404156009",  # Leukemic infiltration of skin (disorder)
        "404157000",  # Specific skin infiltration in Hodgkin's disease (disorder)
        "404169008",  # Malignant histiocytosis involving skin (disorder)
        "404172001",  # Mast cell leukemia affecting skin (disorder)
        "404664002",  # Malignant optic glioma (disorder)
        "404810003",  # 5-hydroxytryptamine secreting neuroendocrine tumor (disorder)
        "405546008",  # Pericardial effusion co-occurrent and due to malignant neoplasm of pericardium (disorder)
        "405576001",  # Nontraumatic hemothorax (disorder)
        "405822008",  # Squamous cell carcinoma of larynx (disorder)
        "405843009",  # Widespread metastatic malignant neoplastic disease (disorder)
        "405945003",  # Malignant neoplasm of metatarsal bone of foot (disorder)
        "40640008",  # Massive fibrosis of lung co-occurrent and due to silicosis (disorder)
        "40779009",  # Discoid atelectasis (disorder)
        "408642003",  # Transitional cell carcinoma of kidney (disorder)
        "408643008",  # Infiltrating duct carcinoma of breast (disorder)
        "408644002",  # Adenocarcinoma of duodenum (disorder)
        "408645001",  # Adenocarcinoma of large intestine (disorder)
        "408646000",  # Adenocarcinoma of liver (disorder)
        "408647009",  # Adenocarcinoma of stomach (disorder)
        "408648004",  # Squamous cell carcinoma of epiglottis (disorder)
        "408649007",  # Squamous cell carcinoma of pharynx (disorder)
        "413389003",  # Accelerated phase chronic myeloid leukemia (disorder)
        "413441006",  # Acute monocytic leukemia (disorder)
        "413445002",  # Adenocarcinoma of appendix (disorder)
        "413446001",  # Adenocarcinoma of cecum (disorder)
        "4135001",  # 11p partial monosomy syndrome (disorder)
        "413537009",  # Angioimmunoblastic T-cell lymphoma (disorder)
        "413556004",  # Antibiotic resistant tuberculosis (disorder)
        "413587002",  # Smoldering myeloma (disorder)
        "413656006",  # Blastic phase chronic myeloid leukemia (disorder)
        "413842007",  # Chronic myeloid leukemia in lymphoid blast crisis (disorder)
        "413843002",  # Chronic myeloid leukemia in myeloid blast crisis (disorder)
        "413847001",  # Chronic phase chronic myeloid leukemia (disorder)
        "413852006",  # Ciprofloxacin resistant tuberculosis (disorder)
        "414146004",  # Ethambutol resistant tuberculosis (disorder)
        "414149006",  # Ethionamide resistant tuberculosis (disorder)
        "414166008",  # Extranodal natural killer/T-cell lymphoma, nasal type (disorder)
        "414546009",  # Isoniazid resistant tuberculosis (disorder)
        "414553000",  # Kappa light chain myeloma (disorder)
        "414676007",  # Metastatic neuroblastoma of orbit proper (disorder)
        "414780005",  # Mucosa-associated lymphoid tissue lymphoma of orbit (disorder)
        "414785000",  # Multiple solitary plasmacytomas (disorder)
        "415110002",  # Plasma cell myeloma/plasmacytoma (disorder)
        "415112005",  # Plasmacytoma (disorder)
        "415177008",  # Primary malignant neoplasm of ear, nose AND/OR throat (disorder)
        "415222009",  # Pyrazinamide resistant tuberculosis (disorder)
        "415283002",  # Myelodysplastic syndrome with excess blasts-1 (disorder)
        "415284008",  # Myelodysplastic syndrome with excess blasts-2 (disorder)
        "415287001",  # Relapsing chronic myeloid leukemia (disorder)
        "415345001",  # Rifampicin resistant tuberculosis (disorder)
        "415622003",  # Streptomycin resistant tuberculosis (disorder)
        "416265003",  # Tuberculoma of brain (disorder)
        "416484003",  # Sickle cell-hemoglobin E disease with crisis (disorder)
        "416769008",  # Malignant teratoma of testis (disorder)
        "416826005",  # Sickle cell-thalassemia disease with crisis (disorder)
        "416842003",  # Malignant sacral teratoma (disorder)
        "416901002",  # Malignant medulloepithelioma of ciliary body (disorder)
        "416903004",  # Tuberculoma of spinal cord (disorder)
        "417181009",  # Hormone receptor positive malignant neoplasm of breast (disorder)
        "417279003",  # Hemoglobin S sickling disorder with crisis (disorder)
        "417417007",  # Malignant teratoma of undescended testis (disorder)
        "417425009",  # Hemoglobin SS disease with crisis (disorder)
        "417484006",  # Tuberculous abscess of spinal cord (disorder)
        "417517009",  # Sickle cell-hemoglobin C disease with crisis (disorder)
        "417554000",  # Malignant teratoma of descended testis (disorder)
        "417570003",  # Gestational choriocarcinoma (disorder)
        "417748003",  # Sickle cell-hemoglobin D disease with crisis (disorder)
        "418372008",  # Squamous cell carcinoma of mucous membrane of lower lip (disorder)
        "418529003",  # Metastatic malignant neoplasm to lacrimal drainage structure (disorder)
        "419052002",  # Malignant tumor of urinary system (disorder)
        "419240004",  # Squamous cell carcinoma of mucous membrane of upper lip (disorder)
        "419842002",  # Squamous cell carcinoma of oral mucous membrane (disorder)
        "420302007",  # Reticulosarcoma with acquired immunodeficiency syndrome (disorder)
        "420519005",  # Malignant lymphoma of the eye region (disorder)
        "420524008",  # Kaposi's sarcoma with acquired immunodeficiency syndrome (disorder)
        "420788006",  # Intraocular non-Hodgkin malignant lymphoma (disorder)
        "420890002",  # T-lymphoblastic leukemia/lymphoblastic lymphoma (disorder)
        "421246008",  # T-lymphoblastic lymphoma (disorder)
        "421249001",  # Malignant tumor of vermilion border of lip (disorder)
        "421283008",  # Primary lymphoma of brain with acquired immunodeficiency syndrome (disorder)
        "421571007",  # Tuberculosis with acquired immunodeficiency syndrome (disorder)
        "422282000",  # Malignant neoplasm with acquired immunodeficiency syndrome (disorder)
        "422378004",  # Squamous cell carcinoma of bridge of nose (disorder)
        "422481005",  # Extreme drug resistant tuberculosis (disorder)
        "4225003",  # Tuberculosis of nose (disorder)
        "422541001",  # Undifferentiated carcinoma of nasopharynx (disorder)
        "422572002",  # Squamous cell carcinoma of chin (disorder)
        "422599000",  # Squamous cell carcinoma of back (disorder)
        "422676009",  # Squamous cell carcinoma of ala nasi (disorder)
        "422691006",  # Squamous cell carcinoma of nasopharynx (disorder)
        "422736007",  # Primary malignant neoplasm of peripheral nerve (disorder)
        "422758009",  # Carcinoma of nasal meatus (disorder)
        "422833009",  # Adenoid cystic carcinoma of salivary gland (disorder)
        "422853008",  # Lymphoma of retroperitoneal space (disorder)
        "422886007",  # Olfactory neuroblastoma (disorder)
        "422968005",  # Non-small cell carcinoma of lung, TNM stage 3 (disorder)
        "423005002",  # Primary malignant neoplasm of lacrimal gland duct (disorder)
        "423032007",  # Leukemic infiltration of orbit (disorder)
        "423038006",  # Polymorphous low grade adenocarcinoma of salivary gland (disorder)
        "423050000",  # Large cell carcinoma of lung, TNM stage 2 (disorder)
        "423092005",  # Multidrug resistant tuberculosis (disorder)
        "423106003",  # Adenocarcinoma of nasopharynx (disorder)
        "423121009",  # Non-small cell carcinoma of lung, TNM stage 4 (disorder)
        "423158009",  # Hurthle cell carcinoma of thyroid (disorder)
        "423189008",  # Adenoid cystic carcinoma of submandibular gland (disorder)
        "423195009",  # Primary malignant neoplasm of lacrimal drainage system (disorder)
        "423284006",  # Squamous cell carcinoma of skin of neck (disorder)
        "423294001",  # Idiopathic hypereosinophilic syndrome (disorder)
        "423295000",  # Squamous cell carcinoma of lung, TNM stage 1 (disorder)
        "423318000",  # Adenoid cystic carcinoma of oropharynx (disorder)
        "423325007",  # Basal cell carcinoma of external auditory canal (disorder)
        "423349005",  # Carcinoma of tip of nose (disorder)
        "423384009",  # Metastatic malignant neoplasm to lacrimal gland duct (disorder)
        "423424005",  # Mucoepidermoid carcinoma of submandibular gland (disorder)
        "423425006",  # Malignant neoplasm of skin of eyelid (disorder)
        "423463003",  # Basal cell carcinoma of back (disorder)
        "423464009",  # Squamous cell carcinoma of oropharynx (disorder)
        "423468007",  # Squamous cell carcinoma of lung, TNM stage 2 (disorder)
        "423506005",  # Squamous cell carcinoma of external auditory canal (disorder)
        "423535002",  # Basal cell carcinoma of chest wall (disorder)
        "423595004",  # Adenocarcinoma carcinomatosis (disorder)
        "423600008",  # Large cell carcinoma of lung, TNM stage 4 (disorder)
        "423607006",  # Adenocarcinoma of anus (disorder)
        "423615009",  # Adenoid cystic carcinoma of parotid gland (disorder)
        "423627007",  # Sarcoma of ovary (disorder)
        "423673009",  # Malignant melanoma of retina (disorder)
        "423691004",  # Malignant neoplasm of gum and contiguous sites (disorder)
        "423700001",  # Squamous cell carcinoma of auricle of ear (disorder)
        "423708008",  # Mucoepidermoid carcinoma of salivary gland (disorder)
        "423709000",  # Drug resistant tuberculosis (disorder)
        "423746001",  # Adenocarcinoma of pelvis (disorder)
        "423793008",  # Mucoepidermoid carcinoma of parotid gland (disorder)
        "423807009",  # Primary leiomyosarcoma (disorder)
        "423812005",  # Sarcoma of head and/or neck (disorder)
        "423829008",  # Invasive vulval Paget's disease (disorder)
        "423896007",  # Squamous cell carcinoma of tip of nose (disorder)
        "423973006",  # Carcinoma of uterine cervix, invasive (disorder)
        "423997002",  # Tuberculosis, extrapulmonary (disorder)
        "424052001",  # Small cell carcinoma carcinomatosis (disorder)
        "424132000",  # Non-small cell carcinoma of lung, TNM stage 1 (disorder)
        "424151006",  # Anaplastic glioma of brain (disorder)
        "424190005",  # Malignant melanoma of unknown origin (disorder)
        "424260006",  # Squamous cell carcinoma of nasolabial fold (disorder)
        "424276002",  # Malignant glioma of brainstem (disorder)
        "424334007",  # Malignant tumor of spinal cord, intramedullary (disorder)
        "42434002",  # Pneumohemothorax due to trauma (disorder)
        "424413001",  # Sarcoma (disorder)
        "424440001",  # Adenocarcinoma of small intestine (disorder)
        "424549003",  # Malignant tumor of spinal cord, extramedullary (disorder)
        "42458003",  # Hemothorax due to trauma (disorder)
        "424666005",  # Basal cell carcinoma of auricle of ear (disorder)
        "424779008",  # Primary Kaposi's sarcoma of oral cavity (disorder)
        "424849005",  # Primary sarcoma of tongue (disorder)
        "424938000",  # Large cell carcinoma of lung, TNM stage 1 (disorder)
        "424952003",  # Sarcoma of soft tissue (disorder)
        "424954002",  # Undifferentiated large cell carcinomatosis (disorder)
        "424970000",  # Large cell carcinoma of lung, TNM stage 3 (disorder)
        "425048006",  # Non-small cell carcinoma of lung, TNM stage 2 (disorder)
        "425066001",  # Carcinoma of urinary bladder, invasive (disorder)
        "425127006",  # Carcinoma ex pleomorphic adenoma of parotid gland (disorder)
        "425148008",  # Squamous cell carcinoma of temple (disorder)
        "425178004",  # Adenocarcinoma of rectosigmoid junction (disorder)
        "42522007",  # Abscess of mediastinum (disorder)
        "425225007",  # Malignant mixed tumor of salivary gland (disorder)
        "425230006",  # Squamous cell carcinoma of lung, TNM stage 3 (disorder)
        "425231005",  # Carcinoma of urinary bladder, superficial (disorder)
        "425303004",  # Squamous cell carcinomatosis (disorder)
        "425376008",  # Squamous cell carcinoma of lung, TNM stage 4 (disorder)
        "425657001",  # Osteosclerotic myeloma (disorder)
        "425688002",  # Philadelphia chromosome-positive acute lymphoblastic leukemia (disorder)
        "425869007",  # Acute promyelocytic leukemia, FAB M3, in remission (disorder)
        "425941003",  # Precursor B-cell acute lymphoblastic leukemia in remission (disorder)
        "426071002",  # Hodgkin's disease in remission (disorder)
        "426124006",  # Acute myeloid leukemia with maturation, FAB M2, in remission (disorder)
        "426217000",  # Aleukemic leukemia in remission (disorder)
        "426248008",  # Aleukemic lymphoid leukemia in remission (disorder)
        "426336007",  # Solitary osseous myeloma (disorder)
        "426370008",  # Subacute lymphoid leukemia in remission (disorder)
        "426642002",  # Erythroleukemia, FAB M6 in remission (disorder)
        "426885008",  # Hodgkin's disease, lymphocytic depletion of lymph nodes of head (disorder)
        "426964009",  # Non-small cell lung cancer, positive for epidermal growth factor receptor expression (disorder)
        "427038005",  # Non-small cell lung cancer, negative for epidermal growth factor receptor expression (disorder)
        "427056005",  # Subacute leukemia in remission (disorder)
        "427099000",  # Active tuberculosis (disorder)
        "427141003",  # Malignant lymphoma in remission (disorder)
        "427492003",  # Hormone refractory prostate cancer (disorder)
        "427642009",  # T-lymphoblastic leukemia in remission (disorder)
        "427658007",  # Acute myelomonocytic leukemia, FAB M4, in remission (disorder)
        "427685000",  # Human epidermal growth factor 2 positive carcinoma of breast (disorder)
        "428061005",  # Malignant neoplasm of brain (disorder)
        "428100006",  # Malignant neoplasm of thoracic cavity structure (disorder)
        "428281000",  # Malignant neoplasm of bone (disorder)
        "428322007",  # Malignant neoplasm of uterine adnexa (disorder)
        "428782008",  # Postoperative hemothorax (disorder)
        "428905002",  # Malignant neoplasm of gastrointestinal tract (disorder)
        "429033009",  # Malignant neoplasm of cerebrum (disorder)
        "42908004",  # Primary atelectasis, in perinatal period (disorder)
        "429114002",  # Malignant basal cell neoplasm of skin (disorder)
        "430338009",  # Smoldering chronic lymphocytic leukemia (disorder)
        "430556008",  # Malignant neoplasm of genital structure (disorder)
        "430621000",  # Malignant neoplasm of lower respiratory tract (disorder)
        "431396003",  # Human epidermal growth factor 2 negative carcinoma of breast (disorder)
        "431911000124102",  # Gnathic osteosarcoma (disorder)
        "432131000124100",  # Malignant mesothelioma of tunica vaginalis (disorder)
        "432328008",  # Neuroblastoma (disorder)
        "433067002",  # Adamantinoma of femur (disorder)
        "438946002",  # Siewert type I adenocarcinoma of esophagogastric junction (disorder)
        "439478008",  # Siewert type III adenocarcinoma of esophagogastric junction (disorder)
        "440173001",  # Nonsquamous nonsmall cell neoplasm of lung (disorder)
        "440206000",  # Hemoglobin SS disease with vasoocclusive crisis (disorder)
        "440397000",  # Primary osteosarcoma of pelvis (disorder)
        "440422002",  # Asymptomatic multiple myeloma (disorder)
        "440501006",  # Siewert type II adenocarcinoma of esophagogastric junction (disorder)
        "440525003",  # Primary small cell neoplasm of thymus (disorder)
        "440527006",  # Primary squamous cell carcinoma of naris (disorder)
        "441313008",  # Indolent multiple myeloma (disorder)
        "441535001",  # Adenocarcinoma of head and/or neck (disorder)
        "441559006",  # Mantle cell lymphoma of spleen (disorder)
        "441770001",  # Tuberculosis of external body orifice (disorder)
        "44182001",  # Avascular necrosis of bone caused by Mycobacterium tuberculosis complex (disorder)
        "441962003",  # Large cell lymphoma of intrapelvic lymph nodes (disorder)
        "442537007",  # Non-Hodgkin lymphoma associated with Human immunodeficiency virus infection (disorder)
        "443136000",  # Malignant neoplasm of skin of face (disorder)
        "443144000",  # Metastatic sarcoma (disorder)
        "44323002",  # Tuberculosis of kidney (disorder)
        "443250000",  # Malignant fibromatous neoplasm (disorder)
        "443261008",  # Oxyphilic adenocarcinoma (disorder)
        "443333004",  # Medulloblastoma (disorder)
        "443378001",  # Lady Windermere syndrome (disorder)
        "443439001",  # Malignant fibrous histiocytoma (disorder)
        "443487006",  # Mantle cell lymphoma (disorder)
        "443488001",  # Malignant neoplasm of anorectum (disorder)
        "443493003",  # Metastatic malignant melanoma (disorder)
        "443520009",  # Chondrosarcoma (disorder)
        "443648003",  # Malignant neoplasm of ear (disorder)
        "443675005",  # Seminoma (disorder)
        "443679004",  # Malignant neoplasm of skeletal system (disorder)
        "443719001",  # Leiomyosarcoma (disorder)
        "443936004",  # Oligodendroglioma (disorder)
        "443937008",  # Mixed glioma (disorder)
        "443961001",  # Malignant adenomatous neoplasm (disorder)
        "444108000",  # Acute sickle cell splenic sequestration crisis (disorder)
        "444374006",  # Type C thymoma (disorder)
        "4445009",  # Tuberculosis of genitourinary system (disorder)
        "444596001",  # Malignant thymoma (disorder)
        "444597005",  # Extranodal marginal zone lymphoma of mucosa-associated lymphoid tissue of stomach (disorder)
        "444604002",  # Carcinoma of breast with ductal and lobular features (disorder)
        "444712000",  # Mucinous carcinoma of breast (disorder)
        "444910004",  # Primary mediastinal (thymic) large B-cell lymphoma (disorder)
        "444911000",  # Acute myeloid leukemia with t(9:11)(p22;q23); MLLT3-MLL (disorder)
        "445227008",  # Juvenile myelomonocytic leukemia (disorder)
        "445269007",  # Extranodal marginal zone B-cell lymphoma of mucosa-associated lymphoid tissue (disorder)
        "445406001",  # Hepatosplenic T-cell lymphoma (disorder)
        "445448008",  # Acute myeloid leukemia with myelodysplasia-related changes (disorder)
        "44547005",  # Chalicosis (disorder)
        "44572005",  # Tuberculous peritonitis (disorder)
        "44599007",  # Traumatic hemothorax without open wound into thorax (disorder)
        "446022000",  # Malignant epithelial neoplasm of uterus (disorder)
        "446123007",  # Tuberculous abscess (disorder)
        "446189008",  # Primary malignant neoplasm of extrahepatic bile duct (disorder)
        "446543007",  # Tuberculous abscess of lung (disorder)
        "446753005",  # Tuberculosis of oropharynx (disorder)
        "446807009",  # Primary malignant neoplasm of perihilar bile duct (disorder)
        "446926000",  # Tuberculous abscess of sacrum (disorder)
        "446939001",  # Chordoma of clivus (disorder)
        "446946005",  # Reinfection pulmonary tuberculosis (disorder)
        "446986002",  # Tuberculous pleural effusion (disorder)
        "447006007",  # Relapse pulmonary tuberculosis (disorder)
        "447012002",  # Tuberculosis of spinal cord (disorder)
        "447100004",  # Marginal zone lymphoma (disorder)
        "447109003",  # Primary malignant neoplasm of intrahepatic bile duct (disorder)
        "447253004",  # Tuberculous arachnoiditis (disorder)
        "447266004",  # Sarcoma of endometrium (disorder)
        "447330002",  # Tuberculosis of abdomen (disorder)
        "447332005",  # Tuberculous abscess of epidural space (disorder)
        "447371008",  # Tuberculosis of ileum (disorder)
        "447389009",  # Leiomyosarcoma of uterus (disorder)
        "447390000",  # Adenosarcoma of uterus (disorder)
        "447399004",  # Tuberculous abscess of neck (disorder)
        "447416008",  # Primary malignant neoplasm of distal bile duct (disorder)
        "447656001",  # Lymphoma of pylorus of stomach (disorder)
        "447658000",  # Lymphoma of fundus of stomach (disorder)
        "447705002",  # Malignant germ cell neoplasm of posterior mediastinum (disorder)
        "447706001",  # Leiomyosarcoma of scalp (disorder)
        "447707005",  # Leiomyosarcoma of cardia of stomach (disorder)
        "447708000",  # Malignant germ cell neoplasm of anterior mediastinum (disorder)
        "447710003",  # Dermatofibrosarcoma protuberans of skin of shoulder (disorder)
        "447730004",  # Chordoma of sacrum (disorder)
        "447738006",  # Paget's disease of skin of scrotum (disorder)
        "447757002",  # Angiosarcoma of cheek (disorder)
        "447766003",  # Lymphoma of pyloric antrum of stomach (disorder)
        "447781009",  # Malignant epithelial neoplasm of peritoneum (disorder)
        "447783007",  # Sarcoma of peritoneum (disorder)
        "447785000",  # Leiomyosarcoma of stomach (disorder)
        "447792005",  # Chondrosarcoma of bone (disorder)
        "447800002",  # Adenocarcinoma of scrotum (disorder)
        "447804006",  # Leiomyosarcoma of connective tissue (disorder)
        "447805007",  # Lymphoma of greater curvature of stomach (disorder)
        "447806008",  # Lymphoma of cardia of stomach (disorder)
        "447882007",  # Malignant epithelial neoplasm of vulva (disorder)
        "447883002",  # Malignant neoplasm of carotid body (disorder)
        "447885009",  # Sarcoma of pelvis (disorder)
        "447886005",  # Adenocarcinoma of anorectum (disorder)
        "447949005",  # Malignant epithelial neoplasm of cheek (disorder)
        "447951009",  # Ewing's sarcoma of soft tissue (disorder)
        "447989004",  # Non-Hodgkin's lymphoma of extranodal site (disorder)
        "448139005",  # Fibrosarcoma of connective tissue (disorder)
        "448165009",  # Squamous cell carcinoma arising in chronic ulcer (disorder)
        "448212009",  # Anaplastic lymphoma kinase negative anaplastic large cell lymphoma (disorder)
        "448213004",  # Diffuse non-Hodgkin's lymphoma of prostate (disorder)
        "448214005",  # Malignant epithelial neoplasm of oropharynx (disorder)
        "448215006",  # Malignant epithelial neoplasm of renal pelvis (disorder)
        "448216007",  # Malignant epithelial neoplasm of thyroid (disorder)
        "448217003",  # Follicular non-Hodgkin's lymphoma of prostate (disorder)
        "448218008",  # Malignant neoplasm of cerebellopontine angle (disorder)
        "448220006",  # Non-Hodgkin's lymphoma of bone (disorder)
        "448221005",  # Sarcoma of fibula (disorder)
        "448229007",  # Leiomyosarcoma of lower esophagus (disorder)
        "448231003",  # Follicular non-Hodgkin's lymphoma of nose (disorder)
        "448233000",  # Malignant neoplasm of urinary organ (disorder)
        "448248006",  # Malignant neoplasm of axial suprasellar region of brain (disorder)
        "448250003",  # Malignant teratoma of pineal region (disorder)
        "448254007",  # Non-Hodgkin's lymphoma of central nervous system (disorder)
        "448258005",  # Sarcoma of mesentery (disorder)
        "448259002",  # Sarcoma of posterior mediastinum (disorder)
        "448269008",  # Lymphoma of lesser curvature of stomach (disorder)
        "448274000",  # Malignant neoplasm of connective tissue (disorder)
        "448296006",  # Dermatofibrosarcoma protuberans of skin of chest (disorder)
        "448299004",  # Malignant neoplasm of mastoid (disorder)
        "448314007",  # Malignant epithelial neoplasm of spinal cord (disorder)
        "448315008",  # Malignant epithelial neoplasm of anus (disorder)
        "448317000",  # Follicular non-Hodgkin's lymphoma of soft tissue (disorder)
        "448319002",  # Diffuse non-Hodgkin's lymphoma of nasopharynx (disorder)
        "448354009",  # Non-Hodgkin's lymphoma of intestine (disorder)
        "448370006",  # Round atelectasis (disorder)
        "448371005",  # Non-Hodgkin's lymphoma of nasopharynx (disorder)
        "448372003",  # Non-Hodgkin's lymphoma of lung (disorder)
        "448376000",  # Non-Hodgkin's lymphoma of ovary (disorder)
        "448377009",  # Sarcoma of sacrum (disorder)
        "448378004",  # Sarcoma of bone of foot (disorder)
        "448384001",  # Non-Hodgkin's lymphoma of nose (disorder)
        "448386004",  # Non-Hodgkin's lymphoma of oral cavity (disorder)
        "448387008",  # Non-Hodgkin's lymphoma of testis (disorder)
        "448401007",  # Choriocarcinoma of placenta (disorder)
        "448447004",  # Non-Hodgkin's lymphoma of skin (disorder)
        "448450001",  # Sarcoma of omentum (disorder)
        "448465000",  # Diffuse non-Hodgkin's lymphoma of testis (disorder)
        "448468003",  # Diffuse non-Hodgkin's lymphoma of oral cavity (disorder)
        "448509007",  # Transglottic malignant neoplasm of larynx (disorder)
        "448553002",  # Lymphoma of pelvis (disorder)
        "448554008",  # Liposarcoma of connective tissue (disorder)
        "448555009",  # Lymphoma of body of stomach (disorder)
        "448558006",  # Malignant neoplasm of maxillofacial bone (disorder)
        "448559003",  # Malignant teratoma of retroperitoneum (disorder)
        "448560008",  # Diffuse non-Hodgkin's lymphoma of extranodal site (disorder)
        "448561007",  # Follicular non-Hodgkin's lymphoma of extranodal site (disorder)
        "448607004",  # Diffuse non-Hodgkin's lymphoma of uterine cervix (disorder)
        "448609001",  # Diffuse non-Hodgkin's lymphoma of ovary (disorder)
        "448663003",  # Diffuse non-Hodgkin's lymphoma of stomach (disorder)
        "448664009",  # Malignant epithelial neoplasm of small intestine (disorder)
        "448665005",  # Malignant epithelial neoplasm of hypopharynx (disorder)
        "448666006",  # Follicular non-Hodgkin's lymphoma of bone (disorder)
        "448668007",  # Malignant neoplasm of mandible (disorder)
        "448669004",  # Malignant neoplasm of soft tissue of orbit (disorder)
        "448670003",  # Malignant neoplasm of posterior mediastinum (disorder)
        "448672006",  # Follicular non-Hodgkin's lymphoma of lung (disorder)
        "448674007",  # Malignant neoplasm of parametrium (disorder)
        "448675008",  # Malignant neoplasm of digestive system (disorder)
        "448709005",  # Non-Hodgkin's lymphoma of stomach (disorder)
        "448710000",  # Sarcoma of bone (disorder)
        "448711001",  # Sarcoma of back (disorder)
        "448712008",  # Sarcoma of femur (disorder)
        "448738008",  # Non-Hodgkin's lymphoma of soft tissue (disorder)
        "448774004",  # Non-Hodgkin's lymphoma of uterine cervix (disorder)
        "448775003",  # Sarcoma of radius (disorder)
        "448776002",  # Sarcoma of vertebra (disorder)
        "448863000",  # Malignant epithelial neoplasm of pineal gland (disorder)
        "448864006",  # Malignant epithelial neoplasm of ureter (disorder)
        "448865007",  # Follicular non-Hodgkin's lymphoma of skin (disorder)
        "448867004",  # Diffuse non-Hodgkin's lymphoma of lung (disorder)
        "448868009",  # Malignant neoplasm of lateral wall of oropharynx (disorder)
        "448882009",  # Malignant neoplasm of intraabdominal organ (disorder)
        "448930008",  # Squamous cell carcinoma of nose (disorder)
        "448931007",  # Squamous cell carcinoma of shoulder (disorder)
        "448954003",  # Malignant epithelial neoplasm of urethra (disorder)
        "448988009",  # Malignant epithelial neoplasm of nose (disorder)
        "448989001",  # Malignant epithelial neoplasm of brain (disorder)
        "448990005",  # Malignant epithelial neoplasm of nasal cavity (disorder)
        "448992002",  # Malignant epithelial neoplasm of appendix (disorder)
        "448993007",  # Malignant epithelial neoplasm of lung (disorder)
        "448994001",  # Malignant epithelial neoplasm of upper rectum (disorder)
        "448995000",  # Follicular non-Hodgkin's lymphoma of central nervous system (disorder)
        "449034009",  # Malignant neoplasm of anterior and lateral floor of mouth (disorder)
        "449053004",  # Lymphoma of lower esophagus (disorder)
        "449054005",  # Malignant epithelial neoplasm of fundus of uterus (disorder)
        "449055006",  # Leiomyosarcoma of cardioesophageal junction (disorder)
        "449058008",  # Follicular non-Hodgkin's lymphoma of tonsil (disorder)
        "449059000",  # Follicular non-Hodgkin's lymphoma of uterine cervix (disorder)
        "449063007",  # Follicular non-Hodgkin's lymphoma of oral cavity (disorder)
        "449065000",  # Diffuse non-Hodgkin's lymphoma of nose (disorder)
        "449066004",  # Malignant neoplasm of upper respiratory tract (disorder)
        "449067008",  # Malignant neoplasm of parietal pleura (disorder)
        "449072004",  # Lymphoma of gastrointestinal tract (disorder)
        "449073009",  # Malignant epithelial neoplasm of body of uterus (disorder)
        "449074003",  # Lymphoma of small intestine (disorder)
        "449075002",  # Lymphoma of cardioesophageal junction (disorder)
        "449077005",  # Malignant epithelial neoplasm of maxilla (disorder)
        "449096009",  # Malignant neoplasm of respiratory system (disorder)
        "449100005",  # Sarcoma of bone of pelvis (disorder)
        "449101009",  # Sarcoma of sternum (disorder)
        "449153001",  # Adenocarcinoma of lower esophagus (disorder)
        "449156009",  # Malignant epithelial neoplasm of floor of mouth (disorder)
        "449173006",  # Diffuse non-Hodgkin's lymphoma of tonsil (disorder)
        "449176003",  # Diffuse non-Hodgkin's lymphoma of intestine (disorder)
        "449177007",  # Diffuse non-Hodgkin's lymphoma of bone (disorder)
        "449206007",  # Sarcoma of tibia (disorder)
        "449207003",  # Sarcoma of humerus (disorder)
        "449208008",  # Sarcoma of coccyx (disorder)
        "449211009",  # Malignant epithelial neoplasm of face (disorder)
        "449216004",  # Diffuse non-Hodgkin's lymphoma of soft tissue (disorder)
        "449217008",  # Diffuse non-Hodgkin's lymphoma of skin (disorder)
        "449218003",  # Lymphoma of sigmoid colon (disorder)
        "449219006",  # Follicular non-Hodgkin's lymphoma of nasopharynx (disorder)
        "449220000",  # Diffuse follicle center lymphoma (disorder)
        "449221001",  # Diffuse non-Hodgkin's lymphoma of central nervous system (disorder)
        "449222008",  # Follicular non-Hodgkin's lymphoma of stomach (disorder)
        "449223003",  # Malignant neoplasm of alveolus of maxilla (disorder)
        "449224009",  # Malignant neoplasm of anterior mediastinum (disorder)
        "449248000",  # Nasopharyngeal carcinoma (disorder)
        "449253005",  # Malignant epithelial neoplasm of hypothalamus (disorder)
        "449254004",  # Malignant epithelial neoplasm of pharynx (disorder)
        "449259009",  # Malignant neoplasm of broad ligament of uterus (disorder)
        "449260004",  # Malignant neoplasm of alveolus dentalis (disorder)
        "449267001",  # Sarcoma of mandible (disorder)
        "449268006",  # Sarcoma of clavicle (disorder)
        "449269003",  # Sarcoma of rib (disorder)
        "449292003",  # Non-Hodgkin's lymphoma of tonsil (disorder)
        "449293008",  # Sarcoma of connective tissue (disorder)
        "449294002",  # Sarcoma of scapula (disorder)
        "449295001",  # Sarcoma of skull (disorder)
        "449307001",  # Follicular non-Hodgkin's lymphoma of ovary (disorder)
        "449308006",  # Malignant neoplasm of visceral pleura (disorder)
        "449309003",  # Malignant neoplasm of skin of scrotum (disorder)
        "449318001",  # Non-Hodgkin's lymphoma of prostate (disorder)
        "449377002",  # Malignant neoplasm of pelvic peritoneum (disorder)
        "449416001",  # Malignant epithelial neoplasm of skin of anus (disorder)
        "449417005",  # Malignant epithelial neoplasm of nasal septum (disorder)
        "449418000",  # Follicular non-Hodgkin's lymphoma of testis (disorder)
        "449419008",  # Follicular non-Hodgkin's lymphoma of intestine (disorder)
        "449420002",  # Malignant neoplasm of cerebellum (disorder)
        "449472007",  # Malignant epithelial neoplasm of alveolus dentalis (disorder)
        "449487002",  # Malignant epithelial neoplasm of mandible (disorder)
        "449497006",  # Sarcoma of pelvic peritoneum (disorder)
        "449578008",  # Malignant neoplasm of alveolus of mandible (disorder)
        "449627008",  # Malignant neoplasm of long bone of lower limb (disorder)
        "449628003",  # Malignant neoplasm of long bone of lower leg (disorder)
        "449630001",  # Metastatic malignant neoplasm to skin of lower leg (disorder)
        "449631002",  # Metastatic malignant neoplasm to skin of upper arm (disorder)
        "449632009",  # Metastatic malignant neoplasm to lower leg (disorder)
        "449633004",  # Metastatic malignant neoplasm to upper arm (disorder)
        "449634005",  # Primary malignant neoplasm of skin of lower leg (disorder)
        "449635006",  # Primary malignant neoplasm of lower leg (disorder)
        "449803009",  # Primary malignant neoplasm of dome of urinary bladder (disorder)
        "449925002",  # Tuberculosis of bone of upper limb (disorder)
        "4501007",  # Tuberculous ascites (disorder)
        "450521000124109",  # Marginal zone lymphoma of axillary lymph node (disorder)
        "452241000124100",  # Recurrent malignant neoplastic disease (disorder)
        "452291000124109",  # Relapse multiple myeloma (disorder)
        "454721000124105",  # Chemosensitive malignant neoplastic disease (disorder)
        "455733521000119103",  # Primary squamous cell carcinoma of skin of head and/or neck (disorder)
        "456911000124108",  # Diffuse non-Hodgkin's lymphoma Lugano stage III (disorder)
        "457721000124104",  # Metastatic non-small cell lung cancer (disorder)
        "458321000124102",  # Metastatic urothelial carcinoma (disorder)
        "458581000124106",  # Metastatic HER2 positive neoplasm of gastroesophageal junction (disorder)
        "459371000124108",  # Metastasis from gastroesophageal adenocarcinoma (disorder)
        "459381000124106",  # Metastasis from hormone-refractory prostate cancer (disorder)
        "459391000124109",  # Metastasis from human epidermal growth factor 2 positive carcinoma of breast (disorder)
        "459401000124106",  # Metastasis from pancreatic endocrine carcinoma (disorder)
        "459411000124109",  # Metastasis from collecting duct carcinoma of nipple (disorder)
        "459421000124101",  # Metastasis from malignant tumor of penis (disorder)
        "461311000124107",  # Refractory Hodgkin lymphoma (disorder)
        "461371000124103",  # Recurrent ovarian cancer (disorder)
        "46621007",  # Atelectasis (disorder)
        "47292005",  # Tuberculosis of myocardium (disorder)
        "473419009",  # Intraductal papillary mucinous carcinoma in situ of pancreas (disorder)
        "473426009",  # Primary tuberculous complex confirmed by culture (disorder)
        "473427000",  # Primary tuberculous complex confirmed histologically (disorder)
        "473428005",  # Primary tuberculous complex confirmed by microscopic examination (disorder)
        "473429002",  # Tuberculoma of spinal cord confirmed (disorder)
        "473430007",  # Primary tuberculous complex confirmed (disorder)
        "473431006",  # Primary tuberculous complex with bacteriological or histological examination results unknown (disorder)
        "47515009",  # Simple silicosis (disorder)
        "47604008",  # Miliary tuberculosis (disorder)
        "48225000",  # Tuberculous disseminated chorioretinitis (disorder)
        "48475001",  # Disorder of diaphragm (disorder)
        "49107007",  # Tuberculosis of eye (disorder)
        "49558004",  # Tuberculous salpingitis (disorder)
        "49807009",  # Tuberculous dactylitis (disorder)
        "49840000",  # Complicated silicosis (disorder)
        "50076003",  # Baritosis (disorder)
        "50589003",  # Silver polishers' lung disease (disorder)
        "50694005",  # Empyema with mediastinal fistula (disorder)
        "51014003",  # Reinfection tuberculosis (disorder)
        "51068008",  # Adult bronchiectasis (disorder)
        "51277007",  # Stannosis (disorder)
        "52721006",  # Tuberculosis of nasal sinus (disorder)
        "52987001",  # Tuberculosis of endocardium (disorder)
        "54084005",  # Cervical tuberculous lymphadenitis (disorder)
        "54959009",  # Perinatal atelectasis (disorder)
        "550991000124107",  # Malignant neoplasm in full remission (disorder)
        "551001000124108",  # Malignant neoplasm in partial remission (disorder)
        "55419007",  # Tuberculosis of glottis (disorder)
        "55448001",  # Tracheoesophageal fistula following tracheostomy (disorder)
        "56498009",  # Tuberculosis of nasopharynx (disorder)
        "56717001",  # Tuberculosis (disorder)
        "58374007",  # Acute diffuse tuberculosis (disorder)
        "58437007",  # Tuberculosis of meninges (disorder)
        "58525007",  # Obstructive atelectasis (disorder)
        "58554001",  # Empyema of pleura (disorder)
        "58691003",  # Antimony pneumoconiosis (disorder)
        "58764007",  # Tuberculous osteomyelitis (disorder)
        "58961005",  # Lethal midline granuloma (disorder)
        "59113005",  # Perinatal secondary atelectasis (disorder)
        "599006",  # Chronic pneumothorax (disorder)
        "60136008",  # Tuberculosis of intestine (disorder)
        "609515005",  # Epithelioid trophoblastic tumor (disorder)
        "61291000119103",  # Disorder of central nervous system co-occurrent and due to acute lymphoid leukemia in remission (disorder)
        "61301000119102",  # Disorder of central nervous system co-occurrent and due to acute lymphoid leukemia (disorder)
        "61311000119104",  # Acute myeloid leukemia, minimal differentiation, FAB M0 in remission (disorder)
        "61321000119106",  # Acute myeloid leukemia without maturation, FAB M1 in remission (disorder)
        "61448007",  # Focal atelectasis (disorder)
        "61471000119100",  # Basal cell carcinoma of naris (disorder)
        "61493004",  # Mu heavy chain disease (disorder)
        "62371005",  # Pulmonary siderosis (disorder)
        "62494007",  # Chronic miliary tuberculosis (disorder)
        "62497000",  # Stewart-Treves syndrome (disorder)
        "62819009",  # Tuberculous cervicitis (disorder)
        "63309002",  # Primary tuberculosis (disorder)
        "63808004",  # Tuberculous chancre (disorder)
        "64631008",  # Fullers' earth disease (disorder)
        "66151009",  # Perinatal complete pulmonary collapse (disorder)
        "66291003",  # Tuberculous phlyctenular keratoconjunctivitis (disorder)
        "66986005",  # Tuberculosis of skin (disorder)
        "67256000",  # Tuberculosis of pericardium (disorder)
        "67811000119102",  # Primary small cell malignant neoplasm of lung, TNM stage 1 (disorder)
        "67821000119109",  # Primary small cell malignant neoplasm of lung, TNM stage 2 (disorder)
        "67831000119107",  # Primary small cell malignant neoplasm of lung, TNM stage 3 (disorder)
        "67841000119103",  # Primary small cell malignant neoplasm of lung, TNM stage 4 (disorder)
        "68033004",  # Tracheostomy complication (disorder)
        "681591000119108",  # Primary adenocarcinoma of anterior wall of urinary bladder (disorder)
        "681601000119101",  # Primary adenocarcinoma of ascending colon (disorder)
        "681611000119103",  # Primary adenocarcinoma of neck of urinary bladder (disorder)
        "681621000119105",  # Primary adenocarcinoma of body of pancreas (disorder)
        "681631000119108",  # Primary adenocarcinoma of body of stomach (disorder)
        "681651000119102",  # Primary adenocarcinoma of descending colon (disorder)
        "681661000119100",  # Primary adenocarcinoma of dome of urinary bladder (disorder)
        "681681000119109",  # Primary adenocarcinoma of fundus of stomach (disorder)
        "681691000119107",  # Primary adenocarcinoma of gastrointestinal tract (disorder)
        "681701000119107",  # Primary adenocarcinoma of greater curvature of stomach (disorder)
        "681711000119105",  # Primary adenocarcinoma of head (disorder)
        "681721000119103",  # Primary adenocarcinoma of head of pancreas (disorder)
        "681751000119106",  # Primary adenocarcinoma of intrahepatic bile duct (disorder)
        "681781000119104",  # Primary adenocarcinoma of lesser curvature of stomach (disorder)
        "681821000119109",  # Primary adenocarcinoma of neck (disorder)
        "681831000119107",  # Primary adenocarcinoma of neck of pancreas (disorder)
        "681911000119108",  # Primary adenocarcinoma of pancreatic duct (disorder)
        "681941000119107",  # Primary adenocarcinoma of pylorus (disorder)
        "681971000119100",  # Primary adenocarcinoma of tail of pancreas (disorder)
        "681991000119104",  # Primary adenocarcinoma of trigone of urinary bladder (disorder)
        "682031000119109",  # Primary adenocarcinoma of urachus (disorder)
        "682041000119100",  # Primary adenocarcinoma of ureteral orifice (disorder)
        "682221000119103",  # Primary anaplastic astrocytoma of cerebrum (disorder)
        "682231000119100",  # Primary anaplastic astrocytoma of frontal lobe (disorder)
        "682241000119109",  # Primary anaplastic astrocytoma of occipital lobe (disorder)
        "682251000119106",  # Primary anaplastic astrocytoma of parietal lobe (disorder)
        "682261000119108",  # Primary anaplastic astrocytoma of temporal lobe (disorder)
        "682271000119102",  # Primary angiosarcoma of abdomen (disorder)
        "682291000119101",  # Primary angiosarcoma of axillary region (disorder)
        "682311000119102",  # Primary angiosarcoma of face (disorder)
        "682321000119109",  # Primary angiosarcoma of head (disorder)
        "682361000119104",  # Primary angiosarcoma of neck (disorder)
        "682381000119108",  # Primary angiosarcoma of pelvis (disorder)
        "682411000119106",  # Primary angiosarcoma of thorax (disorder)
        "682421000119104",  # Primary angiosarcoma of trunk (disorder)
        "682451000119107",  # Primary astrocytoma of frontal lobe (disorder)
        "682471000119103",  # Primary astrocytoma of parietal lobe (disorder)
        "682481000119100",  # Primary astrocytoma of temporal lobe (disorder)
        "682541000119106",  # Primary basal cell carcinoma of skin of perianal area (disorder)
        "68261000119102",  # Diffuse non-Hodgkin's lymphoma Lugano stage I (disorder)
        "68271000119108",  # Diffuse non-Hodgkin's lymphoma Lugano stage II (disorder)
        "68291000119109",  # Diffuse non-Hodgkin's lymphoma Lugano stage IV (disorder)
        "683031000119101",  # Primary carcinoma ex pleomorphic adenoma of salivary gland (disorder)
        "683041000119105",  # Primary carcinoma ex pleomorphic adenoma of submandibular gland (disorder)
        "683121000119103",  # Primary chondrosarcoma of mandible (disorder)
        "683131000119100",  # Primary chondrosarcoma of vertebral column (disorder)
        "683761000119107",  # Primary ependymoma of brain ventricle (disorder)
        "683791000119100",  # Primary ependymoma of parietal lobe (disorder)
        "683981000119101",  # Primary Ewing sarcoma of bone of spine (disorder)
        "683991000119103",  # Extensive stage primary small cell carcinoma of lung (disorder)
        "684901000119107",  # Primary glioblastoma multiforme of cerebrum (disorder)
        "684911000119105",  # Primary glioblastoma multiforme of frontal lobe (disorder)
        "684921000119103",  # Primary glioblastoma multiforme of occipital lobe (disorder)
        "684931000119100",  # Primary glioblastoma multiforme of parietal lobe (disorder)
        "684941000119109",  # Primary glioblastoma multiforme of temporal lobe (disorder)
        "685571000119105",  # Primary leiomyosarcoma of abdomen (disorder)
        "685601000119104",  # Primary leiomyosarcoma of buttock (disorder)
        "685611000119101",  # Primary leiomyosarcoma of face (disorder)
        "685621000119108",  # Primary leiomyosarcoma of head (disorder)
        "685641000119102",  # Primary leiomyosarcoma of inguinal region (disorder)
        "685671000119109",  # Primary leiomyosarcoma of neck (disorder)
        "685691000119105",  # Primary leiomyosarcoma of pelvis (disorder)
        "685701000119105",  # Primary leiomyosarcoma of perineum (disorder)
        "685721000119101",  # Primary leiomyosarcoma of thorax (disorder)
        "685731000119103",  # Primary leiomyosarcoma of trunk (disorder)
        "685831000119105",  # Primary liposarcoma of soft tissue of abdomen (disorder)
        "685851000119104",  # Primary liposarcoma of soft tissue of axilla (disorder)
        "685861000119102",  # Primary liposarcoma of soft tissue of buttock (disorder)
        "685901000119108",  # Primary liposarcoma of soft tissue of inguinal region (disorder)
        "685921000119104",  # Primary liposarcoma of soft tissue of neck (disorder)
        "685941000119105",  # Primary liposarcoma of soft tissue of pelvis (disorder)
        "685951000119107",  # Primary liposarcoma of soft tissue of perineum (disorder)
        "685971000119103",  # Primary liposarcoma of soft tissue of thorax (disorder)
        "685981000119100",  # Primary liposarcoma of soft tissue of trunk (disorder)
        "686431000119100",  # Primary malignant glioma of frontal lobe (disorder)
        "686441000119109",  # Primary malignant glioma of occipital lobe (disorder)
        "686451000119106",  # Primary malignant glioma of parietal lobe (disorder)
        "686461000119108",  # Primary malignant glioma of temporal lobe (disorder)
        "686591000119102",  # Primary malignant pheochromocytoma of left adrenal gland (disorder)
        "686601000119109",  # Primary malignant pheochromocytoma of right adrenal gland (disorder)
        "686981000119108",  # Mycosis fungoides of inguinal lymph node (disorder)
        "686991000119106",  # Mycosis fungoides of lower limb lymph node (disorder)
        "687331000119109",  # Primary oligodendroglioma of frontal lobe (disorder)
        "687351000119103",  # Primary oligodendroglioma of parietal lobe (disorder)
        "687361000119101",  # Primary oligodendroglioma of temporal lobe (disorder)
        "687401000119105",  # Primary osteosarcoma of mandible (disorder)
        "687421000119101",  # Primary osteosarcoma of vertebral column (disorder)
        "687771000119106",  # Primary rhabdomyosarcoma of abdomen (disorder)
        "687801000119108",  # Primary rhabdomyosarcoma of buttock (disorder)
        "687811000119106",  # Primary rhabdomyosarcoma of face (disorder)
        "687821000119104",  # Primary rhabdomyosarcoma of head (disorder)
        "687841000119105",  # Primary rhabdomyosarcoma of inguinal region (disorder)
        "687861000119109",  # Primary rhabdomyosarcoma of neck (disorder)
        "687881000119100",  # Primary rhabdomyosarcoma of pelvis (disorder)
        "687891000119102",  # Primary rhabdomyosarcoma of perineum (disorder)
        "687911000119100",  # Primary rhabdomyosarcoma of thorax (disorder)
        "687941000119101",  # Primary sarcoma of soft tissues of abdomen (disorder)
        "687961000119102",  # Primary sarcoma of axilla (disorder)
        "687971000119108",  # Primary sarcoma of buttock (disorder)
        "687981000119106",  # Sarcoma of face (disorder)
        "687991000119109",  # Primary sarcoma of head (disorder)
        "688011000119102",  # Primary sarcoma of inguinal region (disorder)
        "688031000119107",  # Primary sarcoma of neck (disorder)
        "688061000119104",  # Primary sarcoma of perineum (disorder)
        "688081000119108",  # Primary sarcoma of thorax (disorder)
        "688091000119106",  # Primary sarcoma of trunk (disorder)
        "688321000119107",  # Primary malignant neoplasm of skin of bridge of nose (disorder)
        "688381000119106",  # Primary malignant neoplasm of skin of ala nasi (disorder)
        "688411000119109",  # Primary malignant neoplasm of skin of tip of nose (disorder)
        "688671000119103",  # Primary squamous cell carcinoma of anterior floor of mouth (disorder)
        "688681000119100",  # Primary squamous cell carcinoma of body of tongue (disorder)
        "688691000119102",  # Primary squamous cell carcinoma of anterior wall of urinary bladder (disorder)
        "688711000119104",  # Primary squamous cell carcinoma of aryepiglottic fold (disorder)
        "688731000119109",  # Primary squamous cell carcinoma of neck of urinary bladder (disorder)
        "688751000119103",  # Primary squamous cell carcinoma of border of tongue (disorder)
        "688781000119105",  # Primary squamous cell carcinoma of commissure of lip (disorder)
        "688801000119109",  # Primary squamous cell carcinoma of dome of urinary bladder (disorder)
        "688811000119107",  # Primary squamous cell carcinoma of dorsal surface of tongue (disorder)
        "688861000119105",  # Primary squamous cell carcinoma of hard palate (disorder)
        "688891000119103",  # Primary squamous cell carcinoma of mucous membrane of lip (disorder)
        "688901000119104",  # Primary squamous cell carcinoma of mucous membrane of lower lip (disorder)
        "688911000119101",  # Primary squamous cell carcinoma of mucous membrane of upper lip (disorder)
        "688931000119106",  # Primary squamous cell carcinoma of lateral floor of mouth (disorder)
        "688941000119102",  # Primary squamous cell carcinoma of lateral wall of urinary bladder (disorder)
        "688981000119107",  # Primary squamous cell carcinoma of lower gum (disorder)
        "689211000119102",  # Primary squamous cell carcinoma of perianal skin (disorder)
        "689231000119107",  # Primary squamous cell carcinoma of posterior wall of urinary bladder (disorder)
        "689251000119101",  # Primary squamous cell carcinoma of posterior wall of nasopharynx (disorder)
        "689281000119108",  # Primary squamous cell carcinoma of retromolar area (disorder)
        "689311000119105",  # Primary squamous cell carcinoma of soft palate (disorder)
        "689361000119108",  # Primary squamous cell carcinoma of tonsillar fossa (disorder)
        "689371000119102",  # Primary squamous cell carcinoma of tonsillar pillar (disorder)
        "689401000119104",  # Primary squamous cell carcinoma of upper gum (disorder)
        "689441000119102",  # Primary squamous cell carcinoma of ureteral orifice (disorder)
        "689451000119100",  # Primary squamous cell carcinoma of uvula (disorder)
        "689471000119109",  # Primary squamous cell carcinoma of inferior surface of tongue (disorder)
        "689481000119107",  # Primary squamous cell carcinoma of vermilion border of lower lip (disorder)
        "689491000119105",  # Primary squamous cell carcinoma of vermilion border of upper lip (disorder)
        "689501000119103",  # Primary squamous cell carcinoma of vestibule of mouth (disorder)
        "689771000119103",  # Primary transitional cell carcinoma of anterior wall of urinary bladder (disorder)
        "689781000119100",  # Primary transitional cell carcinoma of neck of urinary bladder (disorder)
        "68979007",  # Heavy chain disease (disorder)
        "689791000119102",  # Primary transitional cell carcinoma of dome of urinary bladder (disorder)
        "689801000119101",  # Primary transitional cell carcinoma of lateral wall of urinary bladder (disorder)
        "689821000119105",  # Primary transitional cell carcinoma of posterior wall of urinary bladder (disorder)
        "689831000119108",  # Primary transitional cell carcinoma of trigone of urinary bladder (disorder)
        "689851000119102",  # Primary transitional cell carcinoma of ureteral orifice (disorder)
        "690121000119109",  # Primary basal cell carcinoma of skin of scrotum (disorder)
        "690141000119103",  # Primary basal cell carcinoma of skin of male genitalia (disorder)
        "69408002",  # Gorlin syndrome (disorder)
        "697993003",  # Undifferentiated carcinoma of nasal sinus (disorder)
        "698011002",  # Keratinizing squamous cell carcinoma of nasopharynx (disorder)
        "698040004",  # Malignant melanoma of nasal cavity (disorder)
        "698041000",  # Malignant melanoma of vestibule of mouth (disorder)
        "698042007",  # Malignant melanoma of tongue (disorder)
        "698043002",  # Malignant melanoma of floor of mouth (disorder)
        "698044008",  # Malignant melanoma of palatine arch (disorder)
        "698045009",  # Malignant melanoma of buccal mucosa (disorder)
        "698048006",  # Undifferentiated nonkeratinizing squamous cell carcinoma of nasopharynx (disorder)
        "698200003",  # Large cell Ewing sarcoma of bone (disorder)
        "698285005",  # Malignant melanoma of ethmoid sinus (disorder)
        "698286006",  # Malignant melanoma of palate (disorder)
        "698287002",  # Malignant melanoma of gum (disorder)
        "698288007",  # Malignant melanoma of maxillary sinus (disorder)
        "699028006",  # Primitive neuroectodermal tumor (disorder)
        "699317002",  # Paratesticular malignant neoplasm (disorder)
        "699318007",  # Supratentorial primitive neuroectodermal tumor (disorder)
        "699326004",  # Accidental puncture of pleura (disorder)
        "699354006",  # Sarcoma of orbit (disorder)
        "699355007",  # Leiomyosarcoma of orbit (disorder)
        "699357004",  # Low grade endometrial stromal sarcoma (disorder)
        "699358009",  # High grade endometrial stromal sarcoma (disorder)
        "699657009",  # Hepatosplenic gamma-delta cell lymphoma (disorder)
        "699659007",  # Glandular malignant peripheral nerve sheath tumor (disorder)
        "699704002",  # Classic medulloblastoma (disorder)
        "699818003",  # T-cell large granular lymphocytic leukemia (disorder)
        "700057001",  # Emberger syndrome (disorder)
        "700272008",  # Tuberculosis of respiratory system (disorder)
        "700273003",  # Isolated tracheobronchial tuberculosis (disorder)
        "700423003",  # Adenocarcinoma of pancreas (disorder)
        "700488005",  # Malignant sex cord tumor of testis (disorder)
        "702346005",  # Chromosome 11p11.2 deletion syndrome (disorder)
        "702368000",  # Carcinosarcoma of ovary (disorder)
        "702369008",  # Carcinosarcoma of uterus (disorder)
        "702391001",  # Renal cell carcinoma (disorder)
        "702392008",  # Metastatic renal cell carcinoma (disorder)
        "702405001",  # Malignant granulosa cell tumor of testis (disorder)
        "702446006",  # Core binding factor acute myeloid leukemia (disorder)
        "702467006",  # Malignant neoplasm of augmented urinary bladder (disorder)
        "702476004",  # Therapy-related myelodysplastic syndrome (disorder)
        "702785000",  # Large cell anaplastic lymphoma T cell and Null cell type (disorder)
        "702786004",  # Follicular non-Hodgkin's lymphoma diffuse follicle center sub-type grade 1 (disorder)
        "702977001",  # Follicular non-Hodgkin's lymphoma diffuse follicle center cell sub-type grade 2 (disorder)
        "703228009",  # Non-small cell lung cancer with mutation in epidermal growth factor receptor (disorder)
        "703230006",  # Non-small cell lung cancer without mutation in epidermal growth factor receptor (disorder)
        "703387000",  # Acute myeloid leukemia with normal karyotype (disorder)
        "70341005",  # Tuberculous laryngitis (disorder)
        "703429003",  # Malignant optic glioma of adulthood (disorder)
        "703625002",  # Kaposi sarcoma not associated with acquired immunodeficiency syndrome (disorder)
        "703626001",  # Anaplastic large cell lymphoma, T/Null cell, primary systemic type (disorder)
        "704152002",  # Metastatic neuroblastoma (disorder)
        "705061009",  # Childhood myelodysplastic syndrome (disorder)
        "706970001",  # Triple negative malignant neoplasm of breast (disorder)
        "707149004",  # Mycobacterial lymphadenitis (disorder)
        "707337006",  # Primary adenocarcinoma of accessory sinus (disorder)
        "707339009",  # Primary adenocarcinoma of maxillary sinus (disorder)
        "707342003",  # Primary adenocarcinoma of ethmoidal sinus (disorder)
        "707343008",  # Primary adenocarcinoma of frontal sinus (disorder)
        "707344002",  # Primary adenocarcinoma of sphenoidal sinus (disorder)
        "707345001",  # Primary carcinoma of accessory sinus (disorder)
        "707346000",  # Primary carcinoma of ethmoidal sinus (disorder)
        "707347009",  # Primary carcinoma of maxillary sinus (disorder)
        "707348004",  # Primary carcinoma of sphenoidal sinus (disorder)
        "707349007",  # Primary carcinoma of frontal sinus (disorder)
        "707350007",  # Malignant melanoma of accessory sinus (disorder)
        "707353009",  # Primary squamous cell carcinoma of accessory sinus (disorder)
        "707354003",  # Primary squamous cell carcinoma of maxillary sinus (disorder)
        "707355002",  # Primary squamous cell carcinoma of sphenoidal sinus (disorder)
        "707356001",  # Primary squamous cell carcinoma of frontal sinus (disorder)
        "707357005",  # Primary squamous cell carcinoma of laryngeal cartilage (disorder)
        "707358000",  # Primary squamous cell carcinoma of larynx (disorder)
        "707359008",  # Primary squamous cell carcinoma of ethmoidal sinus (disorder)
        "707360003",  # Primary lymphoepithelial carcinoma of larynx (disorder)
        "707361004",  # Malignant melanoma of frontal sinus (disorder)
        "707362006",  # Malignant melanoma of sphenoidal sinus (disorder)
        "707377003",  # Primary signet ring cell carcinoma of trachea (disorder)
        "707378008",  # Primary myoepithelial carcinoma of trachea (disorder)
        "707379000",  # Primary mucoepidermoid carcinoma of trachea (disorder)
        "707380002",  # Primary salivary gland type carcinoma of trachea (disorder)
        "707383000",  # Primary mucinous cystadenocarcinoma of trachea (disorder)
        "707384006",  # Primary solid carcinoma of trachea (disorder)
        "707385007",  # Primary undifferentiated carcinoma of trachea (disorder)
        "707386008",  # Primary acinar cell carcinoma of trachea (disorder)
        "707388009",  # Primary squamous cell carcinoma of trachea (disorder)
        "707389001",  # Primary clear cell squamous cell carcinoma of trachea (disorder)
        "707390005",  # Primary basaloid squamous cell carcinoma of trachea (disorder)
        "707391009",  # Primary papillary squamous cell carcinoma of trachea (disorder)
        "707392002",  # Primary giant cell carcinoma of trachea (disorder)
        "707393007",  # Primary adenosquamous carcinoma of trachea (disorder)
        "707394001",  # Primary spindle cell carcinoma of trachea (disorder)
        "707395000",  # Primary adenocarcinoma of hypopharynx (disorder)
        "707396004",  # Primary oxyphilic adenocarcinoma of oropharynx (disorder)
        "707397008",  # Primary basal cell adenocarcinoma of oropharynx (disorder)
        "707398003",  # Primary polymorphous low grade adenocarcinoma of oropharynx (disorder)
        "707399006",  # Primary papillary adenocarcinoma of oropharynx (disorder)
        "707400004",  # Primary mucinous adenocarcinoma of oropharynx (disorder)
        "707401000",  # Primary clear cell adenocarcinoma of oropharynx (disorder)
        "707402007",  # Primary adenocarcinoma of oropharynx (disorder)
        "707403002",  # Primary fetal adenocarcinoma of lung (disorder)
        "707404008",  # Primary mixed subtype adenocarcinoma of lung (disorder)
        "707405009",  # Primary adenosquamous carcinoma of lung (disorder)
        "707406005",  # Primary mucoepidermoid carcinoma of hypopharynx (disorder)
        "707407001",  # Primary signet ring cell carcinoma of lung (disorder)
        "707408006",  # Primary small cell non-keratinizing squamous cell carcinoma of lung (disorder)
        "707409003",  # Primary acinar cell carcinoma of lung (disorder)
        "707410008",  # Primary solid carcinoma of lung (disorder)
        "707411007",  # Primary papillary adenocarcinoma of lung (disorder)
        "707421004",  # Primary undifferentiated carcinoma of larynx (disorder)
        "707422006",  # Primary spindle cell squamous cell carcinoma of larynx (disorder)
        "707423001",  # Primary basaloid carcinoma of larynx (disorder)
        "707424007",  # Primary adenosquamous cell carcinoma of larynx (disorder)
        "707425008",  # Primary adenoid squamous cell carcinoma of larynx (disorder)
        "707426009",  # Primary papillary squamous cell carcinoma of larynx (disorder)
        "707427000",  # Primary verrucous carcinoma of larynx (disorder)
        "707429002",  # Overlapping squamous cell carcinoma of larynx (disorder)
        "707430007",  # Overlapping squamous cell carcinoma of laryngeal cartilage (disorder)
        "707451005",  # Primary adenocarcinoma of lung (disorder)
        "707452003",  # Primary mucinous adenocarcinoma of lung (disorder)
        "707453008",  # Primary clear cell squamous cell carcinoma of lung (disorder)
        "707454002",  # Primary basaloid squamous cell carcinoma of lung (disorder)
        "707455001",  # Primary papillary squamous cell carcinoma of lung (disorder)
        "707456000",  # Primary undifferentiated carcinoma of lung (disorder)
        "707457009",  # Primary spindle cell carcinoma of lung (disorder)
        "707458004",  # Primary pleomorphic carcinoma of lung (disorder)
        "707460002",  # Primary pseudosarcomatous carcinoma of lung (disorder)
        "707464006",  # Primary myoepithelial carcinoma of lung (disorder)
        "707465007",  # Primary mucoepidermoid carcinoma of lung (disorder)
        "707466008",  # Primary adenoid cystic carcinoma of lung (disorder)
        "707467004",  # Primary salivary gland type carcinoma of lung (disorder)
        "707468009",  # Primary mixed mucinous and non-mucinous bronchiolo-alveolar carcinoma of lung (disorder)
        "707469001",  # Primary non-mucinous bronchiolo-alveolar carcinoma of lung (disorder)
        "707470000",  # Primary mucinous bronchiolo-alveolar carcinoma of lung (disorder)
        "707471001",  # Primary clear cell adenocarcinoma of trachea (disorder)
        "707472008",  # Primary papillary adenocarcinoma of trachea (disorder)
        "707473003",  # Primary mucinous adenocarcinoma of trachea (disorder)
        "707475005",  # Primary adenocarcinoma of trachea (disorder)
        "707479004",  # Primary adenocarcinoma of subglottis (disorder)
        "707481002",  # Primary basaloid squamous cell carcinoma of hypopharynx (disorder)
        "707482009",  # Primary papillary squamous cell carcinoma of hypopharynx (disorder)
        "707483004",  # Primary undifferentiated carcinoma of hypopharynx (disorder)
        "707484005",  # Primary adenoid squamous cell carcinoma of hypopharynx (disorder)
        "707485006",  # Primary adenosquamous carcinoma of hypopharynx (disorder)
        "707486007",  # Primary basaloid carcinoma of hypopharynx (disorder)
        "707487003",  # Primary giant cell carcinoma of hypopharynx (disorder)
        "707489000",  # Primary spindle cell squamous cell carcinoma of hypopharynx (disorder)
        "707490009",  # Primary verrucous carcinoma of hypopharynx (disorder)
        "707491008",  # Primary lymphoepithelial carcinoma of hypopharynx (disorder)
        "707492001",  # Primary squamous cell carcinoma of hypopharynx (disorder)
        "707493006",  # Primary lymphoepithelial carcinoma of trachea (disorder)
        "707494000",  # Primary verrucous carcinoma of trachea (disorder)
        "707495004",  # Primary squamous cell adenoid carcinoma of trachea (disorder)
        "707528007",  # Primary squamous cell carcinoma of nasopharynx (disorder)
        "707529004",  # Overlapping squamous cell carcinoma of oropharynx (disorder)
        "707531008",  # Primary squamous cell carcinoma of branchial cleft (disorder)
        "707532001",  # Primary squamous cell carcinoma of posterior wall of oropharynx (disorder)
        "707535004",  # Primary squamous cell carcinoma of lateral wall of oropharynx (disorder)
        "707537007",  # Primary squamous cell carcinoma of anterior surface of epiglottis (disorder)
        "707538002",  # Primary squamous cell carcinoma of vallecula (disorder)
        "707539005",  # Primary adenoid cystic carcinoma of hypopharynx (disorder)
        "707575007",  # Primary squamous cell carcinoma of supraglottis (disorder)
        "707576008",  # Primary squamous cell carcinoma of subglottis (disorder)
        "707579001",  # Primary basaloid squamous cell carcinoma of oropharynx (disorder)
        "707580003",  # Primary basaloid carcinoma of oropharynx (disorder)
        "707581004",  # Primary papillary squamous cell carcinoma of oropharynx (disorder)
        "707582006",  # Primary spindle cell squamous cell carcinoma of oropharynx (disorder)
        "707583001",  # Primary adenosquamous carcinoma of oropharynx (disorder)
        "707584007",  # Primary lymphoepithelial carcinoma of oropharynx (disorder)
        "707585008",  # Primary squamous cell carcinoma of oropharynx (disorder)
        "707586009",  # Primary myoepithelial carcinoma of oropharynx (disorder)
        "707587000",  # Primary carcinoma ex pleomorphic adenoma of oropharynx (disorder)
        "707588005",  # Primary epithelial-myoepithelial carcinoma of oropharynx (disorder)
        "707589002",  # Primary cystadenocarcinoma of oropharynx (disorder)
        "707590006",  # Primary acinar cell carcinoma of oropharynx (disorder)
        "707591005",  # Primary mucoepidermoid carcinoma of oropharynx (disorder)
        "707592003",  # Primary infiltrating duct carcinoma of oropharynx (disorder)
        "707593008",  # Primary salivary gland-type tumor of oropharynx (disorder)
        "707595001",  # Primary mucinous cystadenocarcinoma of lung (disorder)
        "707596000",  # Primary carcinosarcoma of lung (disorder)
        "707627009",  # Primary salivary gland type carcinoma of hypopharynx (disorder)
        "707628004",  # Overlapping squamous cell carcinoma of hypopharynx (disorder)
        "707660007",  # Primary giant cell carcinoma of larynx (disorder)
        "707662004",  # Primary basaloid squamous cell carcinoma of larynx (disorder)
        "707664003",  # Primary squamous cell carcinoma of glottis (disorder)
        "707670009",  # Pleuropulmonary blastoma (disorder)
        "707671008",  # Pleuropulmonary blastoma type I (disorder)
        "707672001",  # Pleuropulmonary blastoma type II (disorder)
        "707673006",  # Pleuropulmonary blastoma type III (disorder)
        "707674000",  # Primary malignant epithelial neoplasm of trachea (disorder)
        "707686002",  # Primary squamous cell carcinoma of posterior wall of hypopharynx (disorder)
        "707697002",  # Primary squamous cell carcinoma of hypopharyngeal aspect of aryepiglottic fold (disorder)
        "707703001",  # Primary squamous cell carcinoma of postcricoid region (disorder)
        "707704007",  # Primary squamous cell carcinoma of pyriform sinus (disorder)
        "707705008",  # Nonkeratinizing carcinoma of the nasopharynx (disorder)
        "708025003",  # Pyopneumothorax following infection by Coccidioides (disorder)
        "708054009",  # Malignant neoplasm after immunosuppressive therapy (disorder)
        "708504008",  # Periosteal osteosarcoma of jaw (disorder)
        "708921005",  # Carcinoma of central portion of breast (disorder)
        "708971008",  # Diffuse sclerosing papillary thyroid carcinoma (disorder)
        "709031009",  # Malignant neoplasm of superior wall of nasopharynx (disorder)
        "70910003",  # Indolent systemic mastocytosis (disorder)
        "709285002",  # Metastatic malignant neoplasm to lumbosacral plexus (disorder)
        "709471005",  # Periodontitis co-occurrent with leukemia (disorder)
        "710106005",  # Extensively drug resistant tuberculosis (disorder)
        "710195004",  # Malignant odontogenic neoplasm of lower jaw (disorder)
        "710196003",  # Malignant odontogenic neoplasm of upper jaw (disorder)
        "71101000119106",  # Recurrent primary malignant neoplasm of vagina (disorder)
        "71111000119109",  # Recurrent primary malignant neoplasm of vulva (disorder)
        "711414003",  # Primary clear cell adenocarcinoma of lung (disorder)
        "712525007",  # Malignant neoplasm of short bone of lower limb (disorder)
        "712750007",  # Malignant neoplasm of chest wall (disorder)
        "713038003",  # Overlapping primary malignant neoplasm of bone and articular cartilage of upper limb (disorder)
        "713189001",  # Malignant insulinoma (disorder)
        "713290004",  # Malignant ameloblastoma of mandible (disorder)
        "713293002",  # Malignant germ cell neoplasm of mediastinum (disorder)
        "713325002",  # Primary cerebral lymphoma co-occurrent with human immunodeficiency virus infection (disorder)
        "713327005",  # Malignant meningioma of meninges of brain (disorder)
        "713483007",  # Reticulosarcoma co-occurrent with human immunodeficiency virus infection (disorder)
        "713516007",  # Primary effusion lymphoma (disorder)
        "713572001",  # Malignant neoplastic disease co-occurrent with human immunodeficiency virus infection (disorder)
        "713609000",  # Invasive carcinoma of breast (disorder)
        "713646001",  # Malignant germ cell tumor of testis (disorder)
        "713718006",  # Diffuse non-Hodgkin immunoblastic lymphoma co-occurrent with human immunodeficiency virus infection (disorder)
        "71375000",  # Tuberculosis of mastoid process (disorder)
        "713897006",  # Burkitt lymphoma co-occurrent with human immunodeficiency virus infection (disorder)
        "714251006",  # Philadelphia chromosome-negative precursor B-cell acute lymphoblastic leukemia (disorder)
        "714394004",  # Lupus vulgaris of nose (disorder)
        "714463003",  # Primary effusion lymphoma co-occurrent with infection caused by Human herpesvirus 8 (disorder)
        "715215007",  # Chromosome 11p13 deletion syndrome (disorder)
        "715412008",  # Familial malignant neoplasm of prostate (disorder)
        "715414009",  # Familial malignant neoplasm of pancreas (disorder)
        "715664005",  # Interdigitating dendritic cell sarcoma (disorder)
        "715901002",  # Ependymoblastoma (disorder)
        "715903004",  # Medulloepithelioma (disorder)
        "715904005",  # Pineal parenchymal tumor of intermediate differentiation (disorder)
        "715950008",  # Anaplastic lymphoma kinase positive large B-cell lymphoma (disorder)
        "716274007",  # Nodular basal cell carcinoma of skin (disorder)
        "716586009",  # Carcinoma of stomach due to Epstein-Barr virus disease (disorder)
        "716588005",  # Primary non-gestational choriocarcinoma of ovary (disorder)
        "716593008",  # Carcinoma of salivary gland type of breast (disorder)
        "716647001",  # Mixed oligoastrocytoma (disorder)
        "716648006",  # Embryonal sarcoma of liver (disorder)
        "716649003",  # Extraovarian primary peritoneal carcinoma (disorder)
        "716651004",  # Leiomyosarcoma of small intestine (disorder)
        "716652006",  # Primary hepatic neuroendocrine carcinoma (disorder)
        "716653001",  # Neuroendocrine carcinoma of thymus (disorder)
        "716655008",  # Aggressive systemic mastocytosis (disorder)
        "716657000",  # Familial papillary thyroid carcinoma with renal papillary neoplasia syndrome (disorder)
        "716659002",  # Squamous cell carcinoma of head and neck (disorder)
        "716788007",  # Epstein-Barr virus positive diffuse large B-cell lymphoma of elderly (disorder)
        "716855006",  # Theca steroid producing cell malignant neoplasm of ovary (disorder)
        "716859000",  # Hereditary diffuse carcinoma of stomach (disorder)
        "717697005",  # Tuberculous ulceration of vulva (disorder)
        "717731002",  # Basal cell carcinoma of vulva (disorder)
        "717734005",  # Papillary thyroid carcinoma with renal papillary neoplasia (disorder)
        "717735006",  # Renal cell carcinoma of kidney except renal pelvis (disorder)
        "717736007",  # Familial renal cell carcinoma (disorder)
        "717916003",  # Neuroendocrine carcinoma of appendix (disorder)
        "718200007",  # Primary pulmonary lymphoma (disorder)
        "718220008",  # Hereditary breast and ovarian cancer syndrome (disorder)
        "718604008",  # Small cell neuroendocrine carcinoma of urinary bladder (disorder)
        "719052006",  # Recurrent squamous cell carcinoma (disorder)
        "71943007",  # Chronic granulomatous tuberculosis (disorder)
        "720587009",  # Donor derived melanoma (disorder)
        "721304007",  # Refractory thrombocytopenia (disorder)
        "721306009",  # Therapy related acute myeloid leukemia and myelodysplastic syndrome (disorder)
        "721308005",  # Acute leukemia of ambiguous lineage (disorder)
        "721310007",  # Aggressive natural killer-cell leukemia (disorder)
        "721311006",  # Systemic Epstein-Barr virus positive T-cell lymphoproliferative disease of childhood (disorder)
        "721313009",  # Indeterminate dendritic cell neoplasm (disorder)
        "721314003",  # Fibroblastic reticular cell neoplasm (disorder)
        "721540005",  # Primary adnexal carcinoma of skin (disorder)
        "721541009",  # Primary malignant sarcoma of skin (disorder)
        "721546004",  # Primary adenocarcinoma of ciliary epithelium (disorder)
        "721547008",  # Primary adenocarcinoma of epithelium of iris (disorder)
        "721548003",  # Primary malignant neoplasm of lacrimal apparatus (disorder)
        "721549006",  # Primary adenocarcinoma of lacrimal apparatus (disorder)
        "721555001",  # Follicular lymphoma of small intestine (disorder)
        "721556000",  # Primary adenocarcinoma of palate (disorder)
        "721557009",  # Primary adenocarcinoma of parotid gland (disorder)
        "721558004",  # Primary adenocarcinoma of cystic duct (disorder)
        "721559007",  # Primary adenocarcinoma of common bile duct (disorder)
        "721560002",  # Primary adenocarcinoma of nasal cavity (disorder)
        "721561003",  # Primary adenocarcinoma of middle ear (disorder)
        "721562005",  # Primary adenocarcinoma of overlapping sites of retroperitoneum, peritoneum and omentum (disorder)
        "721563000",  # Primary malignant melanoma of vagina (disorder)
        "721567004",  # Primary malignant neoplasm of placenta (disorder)
        "721568009",  # Primary adenocarcinoma of parametrium (disorder)
        "721569001",  # Primary adenocarcinoma of uterine ligament (disorder)
        "721574009",  # Primary rhabdomyosarcoma of male genital organ (disorder)
        "721575005",  # Primary angiosarcoma of heart (disorder)
        "721576006",  # Primary angiosarcoma of breast (disorder)
        "721577002",  # Primary liposarcoma of soft tissue of limb (disorder)
        "721578007",  # Primary liposarcoma of male genital organ (disorder)
        "721579004",  # Primary synovial sarcoma of soft tissue of limb (disorder)
        "721580001",  # Primary myosarcoma of omentum (disorder)
        "721602000",  # Primary embryonal carcinoma of testis (disorder)
        "721603005",  # Primary choriocarcinoma of testis (disorder)
        "721604004",  # Primary squamous cell carcinoma of overlapping sites of male genital organ (disorder)
        "721605003",  # Primary squamous cell carcinoma of overlapping sites of accessory sinuses (disorder)
        "721606002",  # Primary adenocarcinoma of overlapping sites of accessory sinuses (disorder)
        "721607006",  # Primary undifferentiated carcinoma of oropharynx (disorder)
        "721617001",  # Primary adenocarcinoma of lower third of esophagus due to Barrett esophagus (disorder)
        "721618006",  # Primary squamous cell carcinoma of upper third of esophagus (disorder)
        "721619003",  # Primary squamous cell carcinoma of middle third of esophagus (disorder)
        "721620009",  # Primary squamous cell carcinoma of lower third of esophagus (disorder)
        "721621008",  # Primary squamous cell carcinoma of overlapping sites of esophagus (disorder)
        "721622001",  # Primary adenocarcinoma of upper third of esophagus (disorder)
        "721623006",  # Primary adenocarcinoma of middle third of esophagus (disorder)
        "721624000",  # Primary adenocarcinoma of overlapping sites of esophagus (disorder)
        "721625004",  # Primary neuroendocrine carcinoma of esophagus (disorder)
        "721626003",  # Primary malignant neuroendocrine neoplasm of esophagus (disorder)
        "721627007",  # Malignant melanoma of esophagus (disorder)
        "721628002",  # Primary adenocarcinoma of esophagogastric junction (disorder)
        "721629005",  # Linitis plastica of stomach (disorder)
        "721630000",  # Primary adenocarcinoma of cardia of stomach (disorder)
        "721632008",  # Primary adenocarcinoma of pyloric antrum of stomach (disorder)
        "721633003",  # Primary adenocarcinoma of overlapping sites of stomach (disorder)
        "721634009",  # Primary malignant neuroendocrine neoplasm of stomach (disorder)
        "721635005",  # Primary malignant neuroendocrine neoplasm of cardia of stomach (disorder)
        "721636006",  # Primary malignant neuroendocrine neoplasm of body of stomach (disorder)
        "721637002",  # Primary malignant neuroendocrine neoplasm of pyloric antrum of stomach (disorder)
        "721638007",  # Primary neuroendocrine carcinoma of stomach (disorder)
        "721639004",  # Primary neuroendocrine carcinoma of cardia of stomach (disorder)
        "721640002",  # Primary neuroendocrine carcinoma of body of stomach (disorder)
        "721641003",  # Primary neuroendocrine carcinoma of pyloric antrum of stomach (disorder)
        "721642005",  # Primary neuroendocrine carcinoma of overlapping sites of stomach (disorder)
        "721643000",  # Primary malignant mesenchymal neoplasm of stomach (disorder)
        "721644006",  # Primary malignant neuroendocrine neoplasm of duodenum (disorder)
        "721645007",  # Primary neuroendocrine carcinoma of duodenum (disorder)
        "721669006",  # Primary adenocarcinoma of overlapping sites of small intestine (disorder)
        "721670007",  # Primary malignant neuroendocrine neoplasm of small intestine (disorder)
        "721671006",  # Primary neuroendocrine carcinoma of small intestine (disorder)
        "721672004",  # Primary mucinous adenocarcinoma of appendix (disorder)
        "721673009",  # Primary malignant neuroendocrine neoplasm of appendix (disorder)
        "721695008",  # Primary adenocarcinoma of ascending colon and right flexure (disorder)
        "721696009",  # Primary adenocarcinoma of transverse colon (disorder)
        "721697000",  # Primary neuroendocrine carcinoma of colon (disorder)
        "721698005",  # Primary malignant neuroendocrine neoplasm of colon (disorder)
        "721699002",  # Primary adenocarcinoma of descending colon and splenic flexure (disorder)
        "721700001",  # Primary malignant neuroendocrine neoplasm of rectum (disorder)
        "721701002",  # Primary neuroendocrine carcinoma of rectum (disorder)
        "721709000",  # Primary cloacogenic carcinoma of anal canal (disorder)
        "721716004",  # Primary cholangiocarcinoma of intrahepatic biliary tract (disorder)
        "721718003",  # Primary adenocarcinoma of ampulla of Vater (disorder)
        "721725005",  # Primary adenocarcinoma of peritoneum (disorder)
        "721762007",  # Adult T-cell leukemia/lymphoma of skin (disorder)
        "722103009",  # Hormone sensitive prostate cancer (disorder)
        "722112006",  # Primary squamous cell carcinoma of ear (disorder)
        "722229001",  # Primary sarcoma of retroperitoneum (disorder)
        "722377004",  # Paraganglioma and gastric stromal sarcoma syndrome (disorder)
        "722425009",  # Reactive oxygen species 1 positive non-small cell lung cancer (disorder)
        "722444002",  # Primary giant cell sarcoma of retroperitoneum (disorder)
        "722445001",  # Primary giant cell sarcoma of peritoneum (disorder)
        "722509001",  # Primary rhabdomyosarcoma of oral cavity (disorder)
        "722510006",  # Primary rhabdomyosarcoma of pharynx (disorder)
        "722511005",  # Primary rhabdomyosarcoma of respiratory organ (disorder)
        "722512003",  # Primary rhabdomyosarcoma of intrathoracic organ (disorder)
        "722513008",  # Primary leiomyosarcoma of retroperitoneum (disorder)
        "722514002",  # Primary leiomyosarcoma of peritoneum (disorder)
        "722515001",  # Primary liposarcoma of retroperitoneum (disorder)
        "722516000",  # Primary liposarcoma of peritoneum (disorder)
        "722517009",  # Primary malignant nerve sheath neoplasm of peripheral nervous system structure (disorder)
        "722518004",  # Primary malignant nerve sheath neoplasm of autonomic nerve (disorder)
        "722519007",  # Primary sarcoma of peritoneum (disorder)
        "722524005",  # Primary invasive pleomorphic lobular carcinoma of breast (disorder)
        "722527003",  # Primary malignant neuroendocrine neoplasm of bronchus (disorder)
        "722528008",  # Primary malignant neuroendocrine neoplasm of lung (disorder)
        "722529000",  # Primary malignant epithelial neoplasm of nasopharynx (disorder)
        "722530005",  # Primary squamous cell carcinoma of pharyngeal tonsil (disorder)
        "722533007",  # Primary malignant neoplasm of esophagogastric junction (disorder)
        "722542000",  # Primary squamous cell carcinoma of anal canal (disorder)
        "722543005",  # Primary malignant melanoma of anal canal (disorder)
        "722665003",  # Primary malignant melanoma of cornea (disorder)
        "722666002",  # Primary squamous cell carcinoma of lacrimal apparatus (disorder)
        "722670005",  # Primary thymic carcinoma (disorder)
        "722671009",  # Metastatic malignant neoplasm of meninges (disorder)
        "722672002",  # Primary squamous cell carcinoma of base of tongue (disorder)
        "722673007",  # Primary squamous cell carcinoma of lingual tonsil (disorder)
        "722674001",  # Primary squamous cell carcinoma of parotid gland (disorder)
        "722676004",  # Primary squamous cell carcinoma of middle ear (disorder)
        "722677008",  # Primary mesothelioma of overlapping sites of retroperitoneum, peritoneum and omentum (disorder)
        "722678003",  # Primary squamous cell carcinoma of vagina (disorder)
        "722679006",  # Primary mucinous adenocarcinoma of endometrium (disorder)
        "722680009",  # Primary serous adenocarcinoma of endometrium (disorder)
        "722681008",  # Primary mixed adenocarcinoma of endometrium (disorder)
        "722682001",  # Primary small cell carcinoma of endometrium (disorder)
        "722683006",  # Primary neuroendocrine carcinoma of cervix uteri (disorder)
        "722684000",  # Primary low grade serous adenocarcinoma of ovary (disorder)
        "722685004",  # Primary high grade serous adenocarcinoma of ovary (disorder)
        "722686003",  # Primary serous carcinoma of uterine adnexa (disorder)
        "722687007",  # Primary mucinous carcinoma of uterine adnexa (disorder)
        "722688002",  # Malignant epithelial neoplasm (disorder)
        "72270005",  # Collagenous pneumoconiosis (disorder)
        "722707001",  # Metastatic malignant neoplasm of peripheral nervous system (disorder)
        "722712000",  # Occupational cancer of skin (disorder)
        "722714004",  # Primary mucoepidermoid carcinoma of lacrimal apparatus (disorder)
        "722718001",  # Primary malignant meningioma (disorder)
        "722795004",  # Meningeal leukemia (disorder)
        "722827008",  # Primary synovial sarcoma of respiratory organ (disorder)
        "722828003",  # Primary synovial sarcoma of intrathoracic organ (disorder)
        "722832009",  # Primary solid papillary carcinoma with invasion of breast (disorder)
        "722954005",  # B-cell lymphoma unclassifiable with features intermediate between classical Hodgkin lymphoma and diffuse large B-cell lymphoma (disorder)
        "723076008",  # Primary myxofibrosarcoma (disorder)
        "723077004",  # Primary myosarcoma of uterus (disorder)
        "723173003",  # Carcinosarcoma of uterine adnexa (disorder)
        "723182009",  # Primary squamous cell carcinoma of nasal cavity (disorder)
        "723265000",  # Primary squamous cell carcinoma of anus (disorder)
        "723281005",  # Primary malignant melanoma of anus (disorder)
        "723301009",  # Squamous non-small cell lung cancer (disorder)
        "723843005",  # Primary chondrosarcoma of bone of limb (disorder)
        "723844004",  # Primary chondrosarcoma of articular cartilage of limb (disorder)
        "723845003",  # Primary chondrosarcoma of articular cartilage of pelvis (disorder)
        "723846002",  # Primary chondrosarcoma of bone of rib (disorder)
        "723847006",  # Primary chondrosarcoma of articular cartilage of rib (disorder)
        "723848001",  # Primary osteosarcoma of bone of jaw (disorder)
        "723849009",  # Primary osteosarcoma of articular cartilage of jaw (disorder)
        "723850009",  # Primary osteosarcoma of bone of limb (disorder)
        "723851008",  # Primary osteosarcoma of articular cartilage of limb (disorder)
        "723852001",  # Primary osteosarcoma of articular cartilage of pelvis (disorder)
        "723853006",  # Primary Ewing sarcoma of bone of limb (disorder)
        "723854000",  # Primary Ewing sarcoma of articular cartilage of limb (disorder)
        "723855004",  # Primary Ewing sarcoma of articular cartilage of pelvis (disorder)
        "723856003",  # Primary Ewing sarcoma of articular cartilage of rib (disorder)
        "723879002",  # Primary adenocarcinoma of overlapping sites of urinary organ (disorder)
        "723887001",  # Primary Ewing sarcoma of bone of rib (disorder)
        "723889003",  # B lymphoblastic leukemia lymphoma with t(9:22) (q34;q11.2); BCR-ABL 1 (disorder)
        "724056005",  # Malignant neoplasm of lower lobe of right lung (disorder)
        "724058006",  # Malignant neoplasm of upper lobe of left lung (disorder)
        "724059003",  # Malignant neoplasm of lower lobe of left lung (disorder)
        "724060008",  # Malignant neoplasm of right upper lobe of lung (disorder)
        "724467001",  # Primary squamous cell carcinoma of overlapping sites of urinary organs (disorder)
        "724468006",  # Primary urothelial carcinoma of overlapping sites of urinary organs (disorder)
        "724536000",  # Primary adenocarcinoma of ileum (disorder)
        "724537009",  # Primary adenocarcinoma of jejunum (disorder)
        "724541008",  # Acute miliary tuberculosis of single site (disorder)
        "724552002",  # Primary poorly differentiated carcinoma of thyroid gland (disorder)
        "724553007",  # Primary undifferentiated carcinoma of thyroid gland (disorder)
        "724554001",  # Primary malignant epithelial neoplasm of endocrine gland (disorder)
        "724644005",  # Myeloid leukemia associated with Down syndrome (disorder)
        "724645006",  # T-cell histiocyte rich large B-cell lymphoma (disorder)
        "724647003",  # Diffuse large B-cell lymphoma co-occurrent with chronic inflammation caused by Epstein-Barr virus (disorder)
        "724648008",  # Plasmablastic lymphoma (disorder)
        "724649000",  # Langerhans cell sarcoma (disorder)
        "724650000",  # Primary follicular dendritic cell sarcoma (disorder)
        "724805000",  # Primary malignant neuroepitheliomatous neoplasm of peripheral nerve (disorder)
        "724806004",  # Primary malignant neuroepitheliomatous neoplasm of autonomic nervous system (disorder)
        "724866001",  # Primary malignant neoplasm of skin due to and following radiotherapy (disorder)
        "725390002",  # Acute myeloid leukemia with t(8;16)(p11;p13) translocation (disorder)
        "725437002",  # Chronic lymphocytic leukemia genetic mutation variant (disorder)
        "726721002",  # Nodal marginal zone B-cell lymphoma (disorder)
        "72807002",  # Tuberculous polyserositis (disorder)
        "72891000119103",  # Kaposi sarcoma of viscus (disorder)
        "732201008",  # Carcinosarcoma of endometrium (disorder)
        "733064004",  # Osteosarcoma, limb anomalies, erythroid macrocytosis syndrome (disorder)
        "733134009",  # Primary squamous cell carcinoma of paraurethral gland (disorder)
        "733135005",  # Primary urothelial carcinoma of paraurethral gland (disorder)
        "733136006",  # Primary adenocarcinoma of urethra (disorder)
        "733144006",  # Malignant epithelial neoplasm of bronchus (disorder)
        "733162002",  # Primary malignant neuroendocrine neoplasm of anus (disorder)
        "733163007",  # Primary malignant neuroendocrine neoplasm of anal canal (disorder)
        "733343005",  # Primary squamous cell carcinoma of oral cavity (disorder)
        "733344004",  # Primary squamous cell carcinoma of lip (disorder)
        "733345003",  # Primary squamous cell carcinoma of pharynx (disorder)
        "733346002",  # Primary mucinous cystic neoplasm with associated invasive carcinoma of perihilar bile duct (disorder)
        "733347006",  # Primary mucinous cystic neoplasm with associated invasive carcinoma of cystic duct (disorder)
        "733348001",  # Primary malignant neuroendocrine neoplasm of cystic duct (disorder)
        "733349009",  # Primary mucinous cystic neoplasm with associated invasive carcinoma of distal bile duct (disorder)
        "733350009",  # Primary malignant neuroendocrine neoplasm of distal bile duct (disorder)
        "733351008",  # Primary malignant neuroendocrine neoplasm of ampulla of Vater (disorder)
        "733352001",  # Primary mucinous cystic neoplasm with associated invasive carcinoma of biliary tract (disorder)
        "733353006",  # Primary malignant neuroendocrine neoplasm of biliary tract (disorder)
        "733354000",  # Primary adenocarcinoma of biliary tract (disorder)
        "733355004",  # Primary adenocarcinoma of digestive organ (disorder)
        "733356003",  # Primary mucinous carcinoma of digestive organ (disorder)
        "733357007",  # Primary squamous cell carcinoma of intrathoracic organ (disorder)
        "733358002",  # Primary squamous cell carcinoma of respiratory system (disorder)
        "733359005",  # Primary squamous cell carcinoma of endometrium (disorder)
        "733360000",  # Primary undifferentiated carcinoma of endometrium (disorder)
        "733361001",  # Primary mucinous adenocarcinoma of ovary (disorder)
        "733362008",  # Primary adenocarcinoma of paraurethral gland (disorder)
        "733470002",  # Collecting duct carcinoma of kidney (disorder)
        "733471003",  # Chromophobe renal cell carcinoma (disorder)
        "733497009",  # Chronic suppuration of bronchus (disorder)
        "733598001",  # Acute myeloid leukemia with t(6;9)(p23;q34) translocation (disorder)
        "733603009",  # Tubulocystic renal cell carcinoma (disorder)
        "733608000",  # Papillary renal cell carcinoma (disorder)
        "733627006",  # Primary cutaneous gamma-delta-positive T-cell lymphoma (disorder)
        "733834006",  # Invasive carcinoma of uterine cervix co-occurrent with human immunodeficiency virus infection (disorder)
        "733835007",  # Extrapulmonary tuberculosis co-occurrent with human immunodeficiency virus infection (disorder)
        "733926004",  # Ganglioneuroblastoma of central nervous system (disorder)
        "734015000",  # Clear cell papillary renal cell carcinoma (disorder)
        "734066005",  # Diffuse large B-cell lymphoma of central nervous system (disorder)
        "734099007",  # Neuroblastoma of central nervous system (disorder)
        "7343008",  # Liparitosis (disorder)
        "73452002",  # Abscess of lung (disorder)
        "73477001",  # Traumatic pneumohemothorax with open wound into thorax (disorder)
        "734884008",  # Diarrhea co-occurrent and due to carcinoid syndrome (disorder)
        "735332000",  # Primary cutaneous diffuse large cell B-cell lymphoma of lower extremity (disorder)
        "735385007",  # Microsatellite instability-high solid malignant tumor (disorder)
        "735450006",  # Primary malignant neuroepitheliomatous neoplasm of nasal cavity (disorder)
        "735521001",  # Human immunodeficiency virus World Health Organization 2007 stage 1 co-occurrent with tuberculosis (disorder)
        "735523003",  # Human immunodeficiency virus World Health Organization 2007 stage 2 co-occurrent with tuberculosis (disorder)
        "735525005",  # Human immunodeficiency virus World Health Organization 2007 stage 3 co-occurrent with tuberculosis (disorder)
        "735527002",  # Human immunodeficiency virus World Health Organization 2007 stage 4 co-occurrent with tuberculosis (disorder)
        "735678002",  # Primary chondrosarcoma of articular cartilage (disorder)
        "735679005",  # Primary chondrosarcoma of bone (disorder)
        "735680008",  # Primary osteosarcoma (disorder)
        "735718000",  # Neonatal malabsorption co-occurrent and due to gastrointestinal hormone-secreting endocrine tumor (disorder)
        "735735001",  # Primary malignant neuroendocrine neoplasm of pancreas (disorder)
        "735757008",  # Primary ganglioneuroblastoma of brain (disorder)
        "735916009",  # Primary malignant neuroepithelial neoplasm of retina (disorder)
        "735917000",  # Primary malignant neoplasm of ocular adnexa (disorder)
        "735918005",  # Primary malignant neoplasm of lacrimal caruncle (disorder)
        "735919002",  # Primary malignant neuroepithelial neoplasm of iris (disorder)
        "735920008",  # Primary malignant neuroepithelial neoplasm of ciliary body (disorder)
        "735921007",  # Primary malignant neuroepithelial neoplasm of orbit (disorder)
        "736322001",  # Pediatric follicular lymphoma (disorder)
        "737058005",  # Microsatellite instability-high colorectal cancer (disorder)
        "737308008",  # Primary adenocarcinoma of sublingual gland (disorder)
        "737309000",  # Primary adenocarcinoma of submandibular gland (disorder)
        "737310005",  # Primary squamous cell carcinoma of submandibular gland (disorder)
        "737311009",  # Primary squamous cell carcinoma of sublingual gland (disorder)
        "737312002",  # Primary malignant neuroendocrine neoplasm of jejunum (disorder)
        "737313007",  # Primary malignant neuroendocrine neoplasm of ileum (disorder)
        "738527001",  # Myeloid and/or lymphoid neoplasm associated with platelet derived growth factor receptor alpha rearrangement (disorder)
        "7391000119103",  # Primary adenoid cystic carcinoma of nasopharynx (disorder)
        "739301006",  # Osteoporosis co-occurrent and due to multiple myeloma (disorder)
        "74181004",  # Tuberculosis of female genital organs (disorder)
        "74387008",  # Tuberculosis of hilar lymph nodes (disorder)
        "7444001",  # Tuberculosis of rectum (disorder)
        "74610006",  # Erythema nodosum due to tuberculosis (disorder)
        "76157009",  # Bituminosis (disorder)
        "761958009",  # Malignant peripheral nerve sheath neoplasm with perineurial differentiation (disorder)
        "762315004",  # Therapy related acute myeloid leukemia due to and following administration of antineoplastic agent (disorder)
        "762457009",  # Astroblastoma of brain (disorder)
        "762458004",  # Primary endometrioid carcinoma of endometrium (disorder)
        "762665002",  # Primary malignant neuroendocrine neoplasm of perihilar bile duct (disorder)
        "762690000",  # Classical Hodgkin lymphoma (disorder)
        "763063001",  # Adenoid basal carcinoma of cervix uteri (disorder)
        "763064007",  # Adenoid cystic carcinoma of cervix uteri (disorder)
        "763101411000119100",  # Primary renal cell carcinoma of left kidney (disorder)
        "763131005",  # Clear cell adenocarcinoma of ovary (disorder)
        "763309005",  # Acute myeloid leukemia with nucleophosmin 1 somatic mutation (disorder)
        "763408003",  # Rhabdomyosarcoma of cervix uteri (disorder)
        "763409006",  # Rhabdomyosarcoma of corpus uteri (disorder)
        "763477007",  # Primary lymphoma of conjunctiva (disorder)
        "763479005",  # Metaplastic carcinoma of breast (disorder)
        "763666008",  # Splenic marginal zone B-cell lymphoma (disorder)
        "763719001",  # Hydroa vacciniforme-like lymphoma (disorder)
        "763771009",  # Leiomyosarcoma of cervix uteri (disorder)
        "763796007",  # Megakaryoblastic acute myeloid leukemia with t(1;22)(p13;q13) (disorder)
        "763884007",  # Splenic diffuse red pulp small B-cell lymphoma (disorder)
        "764694005",  # MiT family translocation renal cell carcinoma (disorder)
        "764735002",  # Squamous cell carcinoma of small intestine (disorder)
        "764737005",  # Squamous cell carcinoma of corpus uteri (disorder)
        "764845008",  # Adenocarcinoma of anal canal (disorder)
        "764846009",  # Adenocarcinoma of penis (disorder)
        "764847000",  # Adenosarcoma of cervix uteri (disorder)
        "764855007",  # Acute myeloid leukemia with CCAAT/enhancer binding protein alpha somatic mutation (disorder)
        "764856008",  # Acquired cystic disease associated renal cell carcinoma (disorder)
        "764938007",  # Lymphoepithelial carcinoma (disorder)
        "764940002",  # Inherited acute myeloid leukemia (disorder)
        "764951002",  # Carcinosarcoma of cervix uteri (disorder)
        "764952009",  # Carcinosarcoma of corpus uteri (disorder)
        "764961009",  # Hereditary primary clear cell renal cell carcinoma (disorder)
        "764990003",  # Renal mucinous tubular and spindle cell carcinoma (disorder)
        "765095002",  # Renal medullary carcinoma (disorder)
        "765136002",  # Primary cutaneous CD8 positive aggressive epidermotropic cytotoxic T-cell lymphoma (disorder)
        "765328000",  # Classic mycosis fungoides (disorder)
        "765740002",  # Adenosarcoma of corpus uteri (disorder)
        "765741003",  # Adenocarcinoma of gallbladder and extrahepatic biliary tract (disorder)
        "766045006",  # Acute myeloid leukemia and myelodysplastic syndrome related to alkylating agent (disorder)
        "766046007",  # Acute myeloid leukemia and myelodysplastic syndrome related to topoisomerase type 2 inhibitor (disorder)
        "766048008",  # Acute myeloid leukemia and myelodysplastic syndrome related to radiation (disorder)
        "766247009",  # Primitive neuroectodermal tumor of corpus uteri (disorder)
        "766248004",  # Primitive neuroectodermal tumor of cervix uteri (disorder)
        "76630008",  # Fibrinopurulent pleurisy (disorder)
        "766757006",  # Undifferentiated carcinoma of stomach (disorder)
        "766758001",  # Undifferentiated carcinoma of corpus uteri (disorder)
        "766759009",  # Rhabdomyosarcoma of vulva and vagina (disorder)
        "766930002",  # Glassy cell carcinoma of cervix uteri (disorder)
        "766935007",  # Primary bone lymphoma (disorder)
        "766978002",  # Squamous cell carcinoma of gallbladder and extrahepatic biliary tract (disorder)
        "766979005",  # Squamous cell carcinoma of rectum (disorder)
        "766980008",  # Squamous cell carcinoma of stomach (disorder)
        "766981007",  # Squamous cell carcinoma of colon (disorder)
        "767444009",  # Germline BRCA-mutated human epidermal growth factor receptor 2 negative metastatic carcinoma of breast (disorder)
        "767448007",  # Pineoblastoma (disorder)
        "77038006",  # Tuberculosis of peripheral lymph nodes (disorder)
        "770559003",  # Leiomyosarcoma of corpus uteri (disorder)
        "770601003",  # Small cell carcinoma of ovary (disorder)
        "770602005",  # Squamous cell carcinoma of exocrine pancreas (disorder)
        "770674007",  # Tubercular lesion of lung (disorder)
        "770684008",  # Squamous cell carcinoma of liver and intrahepatic biliary tract (disorder)
        "770685009",  # Undifferentiated carcinoma of liver and intrahepatic biliary tract (disorder)
        "770686005",  # Malignant germ cell neoplasm of vagina (disorder)
        "772992009",  # Primary differentiated carcinoma of thyroid gland (disorder)
        "773283006",  # Malignant germ cell neoplasm of cervix uteri (disorder)
        "773284000",  # Malignant germ cell neoplasm of corpus uteri (disorder)
        "773537001",  # Differentiation syndrome due to and following chemotherapy co-occurrent with acute promyelocytic leukemia (disorder)
        "773624006",  # Primary ameloblastic carcinoma (disorder)
        "773774000",  # High-grade neuroendocrine carcinoma of corpus uteri (disorder)
        "773775004",  # High-grade neuroendocrine carcinoma of cervix uteri (disorder)
        "773995001",  # Primary cutaneous anaplastic large cell lymphoma (disorder)
        "778066006",  # Carcinofibroma of corpus uteri (disorder)
        "780821007",  # Invasive intraductal papillary-mucinous carcinoma of pancreas (disorder)
        "780844005",  # Acute myeloid leukemia with inv(3)(q21q26.2) or t(3;3)(q21;q26.2); RPN1-EVI1 (disorder)
        "782672006",  # Extragonadal germinoma (disorder)
        "782697005",  # Solid pseudopapillary carcinoma of pancreas (disorder)
        "782722002",  # Global developmental delay, lung cysts, overgrowth, Wilms tumor syndrome (disorder)
        "782761005",  # Subacute invasive pulmonary aspergillosis (disorder)
        "782827000",  # Epithelioid sarcoma (disorder)
        "782915004",  # Acquired hemophagocytic lymphohistiocytosis associated with malignant disease (disorder)
        "783056006",  # Adenocarcinoma of paratestis (disorder)
        "783155007",  # Malignant epithelial neoplasm of salivary gland (disorder)
        "783183009",  # Salivary gland type carcinoma of esophagus (disorder)
        "783541009",  # Breast implant-associated anaplastic large-cell lymphoma (disorder)
        "783565007",  # Indolent T-cell lymphoproliferative disorder of gastrointestinal tract (disorder)
        "783615009",  # Erythropoietic uroporphyria associated with myeloid malignancy (disorder)
        "783704005",  # Undifferentiated carcinoma of esophagus (disorder)
        "783706007",  # Serous cystadenocarcinoma of pancreas (disorder)
        "783736003",  # Malignant melanoma of mucous membrane (disorder)
        "783771003",  # Acinar cell carcinoma of pancreas (disorder)
        "78411000119107",  # Ewing sarcoma of bone of pelvis (disorder)
        "78436002",  # Tuberculosis of intrathoracic lymph nodes (disorder)
        "78461000119105",  # Metastatic squamous cell carcinoma to skin (disorder)
        "785807007",  # Transitional cell carcinoma of corpus uteri (disorder)
        "785879009",  # Mucinous cystadenocarcinoma of pancreas (disorder)
        "786038001",  # Familial nonmedullary primary thyroid carcinoma (disorder)
        "786710002",  # Solid neoplasm with neurotrophic receptor tyrosine kinase gene fusion (disorder)
        "787091002",  # Adenocarcinoma of liver and intrahepatic biliary tract (disorder)
        "78887005",  # Seropurulent pleurisy (disorder)
        "788874003",  # B-cell prolymphocytic leukemia in remission (disorder)
        "788972003",  # Juvenile myelomonocytic leukemia in remission (disorder)
        "788979007",  # Malignant pilonidal cyst (disorder)
        "788982002",  # Mesothelioma of liver (disorder)
        "789051005",  # Squamous cell carcinoma in situ of skin caused by sunlight (disorder)
        "789056000",  # Tuberculosis of inner ear (disorder)
        "789057009",  # Tuberculosis of middle ear (disorder)
        "789689004",  # Malignant lymphomatoid granulomatosis (disorder)
        "789690008",  # Malignant lymphomatoid granulomatosis of lung (disorder)
        "789721005",  # Malignant middle ear paraganglioma (disorder)
        "79158003",  # Tuberculous interstitial keratitis (disorder)
        "79281000119106",  # Primary adenocarcinoma of skin (disorder)
        "792907004",  # Adenocarcinoma of pancreatic duct (disorder)
        "80003002",  # Tuberculous pneumonia (disorder)
        "80010008",  # Isolated bronchial tuberculosis (disorder)
        "80602006",  # Nodular tuberculosis of lung (disorder)
        "80914001",  # Anaplasia of cervix (disorder)
        "81133003",  # Tuberculosis of bones of lower extremity (disorder)
        "81359005",  # Tuberculosis of ureter (disorder)
        "81554001",  # Tuberculosis of lung with involvement of bronchus (disorder)
        "816204007",  # Nodular melanoma of mucous membrane (disorder)
        "816205008",  # Malignant pituitary blastoma (disorder)
        "818967003",  # Medulloepithelioma of central nervous system (disorder)
        "822969007",  # Acinar cell cystadenocarcinoma of lung (disorder)
        "822970008",  # Acinar cell cystadenocarcinoma of prostate (disorder)
        "823017009",  # Infiltrating duct carcinoma of prostate (disorder)
        "82495004",  # Tuberculosis of mediastinal lymph nodes (disorder)
        "8250007",  # Tuberculosis of subcutaneous cellular tissue (disorder)
        "82501000119102",  # Anaplastic astrocytoma of central nervous system (disorder)
        "82546001",  # Reactive immunoproliferative disease (disorder)
        "827162007",  # Malignant immature teratoma of ovary (disorder)
        "827186009",  # Metastatic malignant neoplasm to skin of hip and skin of lower leg (disorder)
        "82761000119108",  # Osteosarcoma of bone of upper limb (disorder)
        "82796002",  # Scrofuloderma (disorder)
        "830055006",  # Anaplastic lymphoma kinase fusion oncogene negative non-small cell lung cancer (disorder)
        "830057003",  # Relapsing classical Hodgkin lymphoma (disorder)
        "830060005",  # Reactive oxygen species 1 negative non-small cell lung cancer (disorder)
        "830150003",  # Malignant melanoma with B-Raf proto-oncogene, serine/threonine kinase V600E mutation (disorder)
        "830151004",  # Anaplastic lymphoma kinase fusion oncogene positive non-small cell lung cancer (disorder)
        "830196006",  # Chordoma of cervical spine (disorder)
        "833285007",  # Chordoma of lumbar spine (disorder)
        "833295000",  # Chordoma of thoracic spine (disorder)
        "836274002",  # Carcinomatosis of peritoneum (disorder)
        "836486002",  # Lymphomatous infiltrate of kidney (disorder)
        "83652003",  # Tuberculosis of epididymis (disorder)
        "83784000",  # Tuberculosis of mediastinum (disorder)
        "838340006",  # B lymphoblastic leukemia lymphoma with t(5;14)(q31;q32); IL3-IGH (disorder)
        "838341005",  # B lymphoblastic leukemia lymphoma with t(v;11q23); MLL rearranged (disorder)
        "838342003",  # B lymphoblastic leukemia lymphoma with t(12;21) (p13;q22); TEL/AML1 (ETV6-RUNX1) (disorder)
        "838343008",  # B lymphoblastic leukemia lymphoma with t(1;19)(Q23;P13.3); E2A-PBX1 (TCF3/PBX1) (disorder)
        "838344002",  # B lymphoblastic leukemia lymphoma with hypodiploidy (disorder)
        "838346000",  # B lymphoblastic leukemia lymphoma with hyperdiploidy (disorder)
        "838349007",  # Adenocarcinoma of Meckel diverticulum (disorder)
        "838355002",  # Acute myeloid leukemia with CBFB::MYH11 fusion (disorder)
        "840423002",  # Diffuse large B-cell lymphoma of small intestine (disorder)
        "840424008",  # Diffuse large B-cell lymphoma of stomach (disorder)
        "840573001",  # Carcinoma of epididymis and spermatic cord (disorder)
        "84194006",  # Tuberculous oophoritis (disorder)
        "84288007",  # Tuberculous episcleritis (disorder)
        "84811000119107",  # Follicular dendritic sarcoma of intraabdominal lymph node (disorder)
        "84821000119100",  # Primary follicular dendritic cell sarcoma of spleen (disorder)
        "84831000119102",  # Follicular dendritic sarcoma of lymph nodes of multiple sites (disorder)
        "85692003",  # Tuberculosis of tracheobronchial lymph nodes (disorder)
        "85940005",  # Tuberculous otitis media (disorder)
        "860803004",  # Hydrocephalus due to tuberculosis of brain (disorder)
        "860828007",  # Gastrointestinal stromal neoplasm of overlapping sites of small intestine (disorder)
        "860830009",  # Gastrointestinal stromal neoplasm of overlapping sites of stomach (disorder)
        "860831008",  # Epithelioid hemangioendothelioma of liver (disorder)
        "860832001",  # Extraskeletal myxoid chondrosarcoma of soft tissue of limb (disorder)
        "860923004",  # Primary osteosarcoma of articular cartilage of lower limb (disorder)
        "86263001",  # Cobaltosis (disorder)
        "865951006",  # Cholangiocarcinoma of perihilar bile duct (disorder)
        "865954003",  # Adenocarcinoma of breast (disorder)
        "866048009",  # Mucinous cystic neoplasm with invasive carcinoma of pancreas (disorder)
        "866050001",  # Mixed germ cell neoplasm of central nervous system (disorder)
        "866052009",  # Mixed ductal-neuroendocrine carcinoma of pancreas (disorder)
        "866068003",  # Malignant nerve sheath neoplasm of overlapping sites of peripheral nerves and autonomic nervous system (disorder)
        "866069006",  # Malignant nerve sheath neoplasm of peripheral nerve of abdomen (disorder)
        "866070007",  # Malignant mesenchymal neoplasm of bile duct (disorder)
        "866071006",  # Malignant mesenchymal neoplasm of gallbladder (disorder)
        "866072004",  # Malignant melanoma of orbit (disorder)
        "866073009",  # Malignant melanoma of overlapping sites of eye and adnexa (disorder)
        "866079008",  # Malignant melanoma of lacrimal gland (disorder)
        "866080006",  # Malignant germ cell neoplasm of heart (disorder)
        "866082003",  # Malignant melanoma of lacrimal drainage system (disorder)
        "866087009",  # Pathologic fracture of vertebra at site of metastatic neoplasm (disorder)
        "866093001",  # Malignant nerve sheath neoplasm of peripheral nerve of upper limb (disorder)
        "866094007",  # Malignant nerve sheath neoplasm of peripheral nerve of pelvis (disorder)
        "866095008",  # Malignant nerve sheath neoplasm of peripheral nerve of thorax (disorder)
        "866096009",  # Malignant nerve sheath neoplasm of peripheral nerve of head and neck (disorder)
        "866097000",  # Malignant nerve sheath neoplasm of peripheral nerve of lower limb (disorder)
        "866098005",  # Large B-cell lymphoma arising in HHV8-associated multicentric Castleman disease (disorder)
        "866109006",  # Lymphocytic hypereosinophilic syndrome (disorder)
        "866176003",  # Clear cell adenocarcinoma of peritoneum (disorder)
        "870318006",  # Osteosarcoma of bone of head (disorder)
        "870327007",  # Papillary squamous cell carcinoma of nasopharynx (disorder)
        "870355006",  # Neuroendocrine carcinoma of anus (disorder)
        "87091000119101",  # Malignant glioma of cerebrum (disorder)
        "87111000119109",  # Malignant glioma of hypothalamus (disorder)
        "87121000119102",  # Malignant glioma of cerebellum (disorder)
        "87131000119104",  # Primary extramedullary malignant tumor of spinal cord (disorder)
        "87141000119108",  # Primary malignant neoplasm of intramedullary spinal cord (disorder)
        "87151000119105",  # Malignant glioma of central nervous system (disorder)
        "878805009",  # Renal granular cell carcinoma (disorder)
        "878806005",  # Second primary cancer (disorder)
        "878807001",  # Human epidermal growth factor 2 positive gastric cancer (disorder)
        "878808006",  # Nongerminomatous germ cell tumor of central nervous system (disorder)
        "878857009",  # Malignant lymphomatoid granulomatosis grade 3 (disorder)
        "878858004",  # Malignant lymphomatoid granulomatosis grade 3 of lung (disorder)
        "87909002",  # Hard metal pneumoconiosis (disorder)
        "8832006",  # Tuberculosis orificialis of anus (disorder)
        "88356006",  # Primary tuberculous complex (disorder)
        "88687001",  # Manganese pneumonitis (disorder)
        "890528009",  # Primary malignant neoplasm of left lung (disorder)
        "890529001",  # Primary malignant neoplasm of right lung (disorder)
        "890534002",  # Primary malignant neoplasm of bilateral lungs (disorder)
        "894171571000119107",  # Primary renal cell carcinoma of right kidney (disorder)
        "895111002",  # Chordoma of vertebral column (disorder)
        "895345000",  # Primary malignant neoplasm of bilateral kidneys (disorder)
        "895354002",  # Primary malignant neoplasm of left ureter (disorder)
        "895355001",  # Primary malignant neoplasm of right ureter (disorder)
        "895356000",  # Primary malignant neoplasm of bilateral ureters (disorder)
        "895358004",  # Primary malignant neoplasm of left testis (disorder)
        "895359007",  # Primary malignant neoplasm of right testis (disorder)
        "895361003",  # Primary malignant neoplasm of bilateral testes (disorder)
        "898197001",  # Adult T-cell leukemia/lymphoma in remission (disorder)
        "898198006",  # T-cell prolymphocytic leukemia in remission (disorder)
        "90117007",  # Tuberculous fibrosis of lung (disorder)
        "90302003",  # Tuberculosis of cerebral meninges (disorder)
        "90623003",  # Aluminosis of lung (disorder)
        "90811000119100",  # Low grade malignant glioma of brain (disorder)
        "90831000119105",  # Grade 4 malignant glioma of brain (disorder)
        "91031000119108",  # Primary chondrosarcoma of bone of upper limb (disorder)
        "91036007",  # Complete atelectasis (disorder)
        "91041000119104",  # Ewing sarcoma of bone structure of upper limb (disorder)
        "91061000119100",  # Primary chondrosarcoma of bone of pelvis (disorder)
        "91081000119109",  # Primary chondrosarcoma of bone of lower limb (disorder)
        "91131000119109",  # Primary adenocarcinoma of vulva (disorder)
        "91141000119100",  # Primary spindle cell carcinoma of urinary bladder (disorder)
        "91151000119103",  # Primary squamous cell carcinoma of chest wall (disorder)
        "91161000119101",  # Primary adenocarcinoma of chest wall (disorder)
        "91171000119107",  # Primary undifferentiated large cell malignant neoplasm of chest wall (disorder)
        "91181000119105",  # Squamous cell carcinoma metastatic to pleura (disorder)
        "91191000119108",  # Metastatic adenocarcinoma to pleura (disorder)
        "91211000119109",  # Metastatic small cell carcinoma to pleura (disorder)
        "91221000119102",  # Metastatic malignant melanoma to pleura (disorder)
        "91231000119104",  # Metastatic adenocarcinoma to skin (disorder)
        "91251000119105",  # Small cell carcinoma metastatic to skin (disorder)
        "91271000119101",  # Metastatic squamous cell carcinoma to bone (disorder)
        "91281000119103",  # Metastatic adenocarcinoma to bone (disorder)
        "91421000119103",  # Primary undifferentiated large cell carcinoma (disorder)
        "91483004",  # Tuberculous encephalitis (disorder)
        "91854005",  # Acute leukemia in remission (disorder)
        "91855006",  # Acute leukemia, disease (disorder)
        "91856007",  # Acute lymphoid leukemia in remission (disorder)
        "91857003",  # Acute lymphoid leukemia, disease (disorder)
        "91858008",  # Acute monocytic leukemia in remission (disorder)
        "91860005",  # Acute myeloid leukemia in remission (disorder)
        "91861009",  # Acute myeloid leukemia (disorder)
        "92811003",  # Chronic leukemia in remission (disorder)
        "92812005",  # Chronic leukemia, disease (disorder)
        "92813000",  # Chronic lymphoid leukemia in remission (disorder)
        "92814006",  # Chronic lymphoid leukemia, disease (disorder)
        "92817004",  # Chronic myeloid leukemia in remission (disorder)
        "92818009",  # Chronic myeloid leukemia (disorder)
        "93142004",  # Leukemia in remission (disorder)
        "93143009",  # Leukemia, disease (disorder)
        "93144003",  # Leukemic reticuloendotheliosis of intra-abdominal lymph nodes (disorder)
        "93145002",  # Leukemic reticuloendotheliosis of intrapelvic lymph nodes (disorder)
        "93146001",  # Leukemic reticuloendotheliosis of intrathoracic lymph nodes (disorder)
        "93150008",  # Leukemic reticuloendotheliosis of lymph nodes of multiple sites (disorder)
        "93151007",  # Hairy cell leukemia of spleen (disorder)
        "93169003",  # Lymphoid leukemia in remission (disorder)
        "93182006",  # Malignant histiocytosis of intra-abdominal lymph nodes (disorder)
        "93183001",  # Malignant histiocytosis of intrapelvic lymph nodes (disorder)
        "93184007",  # Malignant histiocytosis of intrathoracic lymph nodes (disorder)
        "93188005",  # Malignant histiocytosis of lymph nodes of multiple sites (disorder)
        "93189002",  # Malignant histiocytosis of spleen (disorder)
        "93191005",  # Malignant lymphoma of intra-abdominal lymph nodes (disorder)
        "93192003",  # Malignant lymphoma of intrapelvic lymph nodes (disorder)
        "93193008",  # Malignant lymphoma of intrathoracic lymph nodes (disorder)
        "93197009",  # Malignant lymphoma of lymph nodes of multiple sites (disorder)
        "93198004",  # Malignant lymphoma of spleen (disorder)
        "93199007",  # Malignant lymphoma of extranodal AND/OR solid organ site (disorder)
        "93200005",  # Malignant mast cell tumor of intra-abdominal lymph nodes (disorder)
        "93201009",  # Malignant mast cell tumor of intrapelvic lymph nodes (disorder)
        "93202002",  # Malignant mast cell tumor of intrathoracic lymph nodes (disorder)
        "93451002",  # Erythroleukemia, FAB M6 (disorder)
        "93493001",  # Hodgkin's disease, lymphocytic-histiocytic predominance of intra-abdominal lymph nodes (disorder)
        "93494007",  # Hodgkin's disease, lymphocytic-histiocytic predominance of intrapelvic lymph nodes (disorder)
        "93495008",  # Hodgkin's disease, lymphocytic-histiocytic predominance of intrathoracic lymph nodes (disorder)
        "93500006",  # Hodgkin's disease, lymphocytic-histiocytic predominance of spleen (disorder)
        "93509007",  # Hodgkin's disease, mixed cellularity of spleen (disorder)
        "93518009",  # Hodgkin's disease, nodular sclerosis of spleen (disorder)
        "93520007",  # Hodgkin's disease of intra-abdominal lymph nodes (disorder)
        "93521006",  # Hodgkin's disease of intrapelvic lymph nodes (disorder)
        "93522004",  # Hodgkin's disease of intrathoracic lymph nodes (disorder)
        "93526001",  # Hodgkin's disease of lymph nodes of multiple sites (disorder)
        "93527005",  # Hodgkin's disease of spleen (disorder)
        "93530003",  # Hodgkin's granuloma of intrapelvic lymph nodes (disorder)
        "93531004",  # Hodgkin's granuloma of intrathoracic lymph nodes (disorder)
        "93536009",  # Hodgkin's granuloma of spleen (disorder)
        "93547002",  # Hodgkin's sarcoma of intra-abdominal lymph nodes (disorder)
        "93548007",  # Hodgkin's sarcoma of intrapelvic lymph nodes (disorder)
        "93549004",  # Hodgkin's sarcoma of intrathoracic lymph nodes (disorder)
        "93554008",  # Hodgkin's sarcoma of spleen (disorder)
        "93657007",  # Primary malignant neoplasm of abducens nerve (disorder)
        "93658002",  # Primary malignant neoplasm of accessory nerve (disorder)
        "93659005",  # Primary malignant neoplasm of accessory sinus (disorder)
        "93660000",  # Primary malignant neoplasm of vestibulocochlear nerve (disorder)
        "93661001",  # Primary malignant neoplasm of acromion (disorder)
        "93662008",  # Primary malignant neoplasm of adenoid (disorder)
        "93663003",  # Primary malignant neoplasm of skin with adnexal differentiation (disorder)
        "93665005",  # Primary malignant neoplasm of adrenal gland (disorder)
        "93667002",  # Primary malignant neoplasm of alveolar ridge mucosa (disorder)
        "93669004",  # Primary malignant neoplasm of anal canal (disorder)
        "93670003",  # Primary malignant neoplasm of anterior aspect of epiglottis (disorder)
        "93671004",  # Primary malignant neoplasm of anterior mediastinum (disorder)
        "93672006",  # Primary malignant neoplasm of anterior portion of floor of mouth (disorder)
        "93674007",  # Primary malignant neoplasm of anterior wall of nasopharynx (disorder)
        "93675008",  # Primary malignant neoplasm of anterior wall of urinary bladder (disorder)
        "93676009",  # Primary malignant neoplasm of anus (disorder)
        "93677000",  # Primary malignant neoplasm of aortic body (disorder)
        "93679002",  # Primary malignant neoplasm of appendix (disorder)
        "93682007",  # Primary malignant neoplasm of upper arm (disorder)
        "93683002",  # Primary malignant neoplasm of ascending colon (disorder)
        "93684008",  # Primary malignant neoplasm of axilla (disorder)
        "93686005",  # Primary malignant neoplasm of back (disorder)
        "93687001",  # Primary malignant neoplasm of base of tongue (disorder)
        "93689003",  # Primary malignant neoplasm of urinary bladder (disorder)
        "93690007",  # Primary malignant neoplasm of blood vessel of abdomen (disorder)
        "93692004",  # Primary malignant neoplasm of blood vessel of axilla (disorder)
        "93693009",  # Primary malignant neoplasm of blood vessel of buttock (disorder)
        "93694003",  # Primary malignant neoplasm of blood vessel of face (disorder)
        "93695002",  # Primary malignant neoplasm of blood vessel of finger (disorder)
        "93696001",  # Primary malignant neoplasm of blood vessel of foot (disorder)
        "93697005",  # Primary malignant neoplasm of blood vessel of forearm (disorder)
        "93698000",  # Primary malignant neoplasm of blood vessel of hand (disorder)
        "93699008",  # Primary malignant neoplasm of blood vessel of head (disorder)
        "93700009",  # Primary malignant neoplasm of blood vessel of hip (disorder)
        "93701008",  # Primary malignant neoplasm of blood vessel of inguinal region (disorder)
        "93703006",  # Primary malignant neoplasm of blood vessel of lower limb (disorder)
        "93704000",  # Primary malignant neoplasm of blood vessel of neck (disorder)
        "93705004",  # Primary malignant neoplasm of blood vessel of pelvis (disorder)
        "93706003",  # Primary malignant neoplasm of blood vessel of perineum (disorder)
        "93707007",  # Primary malignant neoplasm of blood vessel of popliteal space (disorder)
        "93708002",  # Primary malignant neoplasm of blood vessel of shoulder (disorder)
        "93709005",  # Primary malignant neoplasm of blood vessel of thigh (disorder)
        "93710000",  # Primary malignant neoplasm of blood vessel of thorax (disorder)
        "93711001",  # Primary malignant neoplasm of blood vessel of toe (disorder)
        "93712008",  # Primary malignant neoplasm of blood vessel of trunk (disorder)
        "93713003",  # Primary malignant neoplasm of blood vessel of upper limb (disorder)
        "93714009",  # Primary malignant neoplasm of blood vessel (disorder)
        "93715005",  # Primary malignant neoplasm of body of pancreas (disorder)
        "93716006",  # Primary malignant neoplasm of body of penis (disorder)
        "93717002",  # Primary malignant neoplasm of body of stomach (disorder)
        "93720005",  # Primary malignant neoplasm of bone marrow (disorder)
        "93721009",  # Primary malignant neoplasm of bone of face (disorder)
        "93722002",  # Primary malignant neoplasm of bone of lower limb (disorder)
        "93723007",  # Primary malignant neoplasm of bone of skull (disorder)
        "93724001",  # Primary malignant neoplasm of bone of upper limb (disorder)
        "93725000",  # Primary malignant neoplasm of bone (disorder)
        "93726004",  # Primary malignant neoplasm of brain stem (disorder)
        "93727008",  # Primary malignant neoplasm of brain (disorder)
        "93728003",  # Primary malignant neoplasm of broad ligament (disorder)
        "93729006",  # Primary malignant neoplasm of bronchus of left lower lobe (disorder)
        "93730001",  # Primary malignant neoplasm of bronchus of left upper lobe (disorder)
        "93731002",  # Primary malignant neoplasm of bronchus of right lower lobe (disorder)
        "93732009",  # Primary malignant neoplasm of bronchus of middle lobe of right lung (disorder)
        "93733004",  # Primary malignant neoplasm of bronchus of right upper lobe (disorder)
        "93734005",  # Primary malignant neoplasm of bronchus (disorder)
        "93737003",  # Primary malignant neoplasm of calcaneus (disorder)
        "93738008",  # Primary malignant neoplasm of cardia of stomach (disorder)
        "93739000",  # Primary malignant neoplasm of carina (disorder)
        "93740003",  # Primary malignant neoplasm of carotid body (disorder)
        "93741004",  # Primary malignant neoplasm of carpal bone (disorder)
        "93742006",  # Primary malignant neoplasm of cartilage of nose (disorder)
        "93743001",  # Primary malignant neoplasm of cauda equina (disorder)
        "93744007",  # Primary malignant neoplasm of central nervous system (disorder)
        "93746009",  # Primary malignant neoplasm of cerebellum (disorder)
        "93747000",  # Primary malignant neoplasm of cerebral meninges (disorder)
        "93748005",  # Primary malignant neoplasm of cerebral ventricle (disorder)
        "93749002",  # Primary malignant neoplasm of cerebrum (disorder)
        "93751003",  # Primary malignant neoplasm of cervical vertebral column (disorder)
        "93753000",  # Primary malignant neoplasm of cheek (disorder)
        "93754006",  # Primary malignant neoplasm of chest wall (disorder)
        "93755007",  # Primary malignant neoplasm of choroid (disorder)
        "93756008",  # Primary malignant neoplasm of ciliary body, primary (disorder)
        "93757004",  # Primary malignant neoplasm of clavicle (disorder)
        "93759001",  # Primary malignant neoplasm of coccygeal body (disorder)
        "93760006",  # Primary malignant neoplasm of coccyx (disorder)
        "93761005",  # Primary malignant neoplasm of colon (disorder)
        "93763008",  # Primary malignant neoplasm of common bile duct (disorder)
        "93764002",  # Primary malignant neoplasm of conjunctiva of eye (disorder)
        "93766000",  # Primary malignant neoplasm of cornea of eye (disorder)
        "93767009",  # Primary malignant neoplasm of cranial nerve (disorder)
        "93768004",  # Primary malignant neoplasm of craniopharyngeal duct (disorder)
        "93769007",  # Primary malignant neoplasm of cuboid (disorder)
        "93770008",  # Primary malignant neoplasm of cystic duct (disorder)
        "93771007",  # Primary malignant neoplasm of descending colon (disorder)
        "93772000",  # Primary malignant neoplasm of diaphragm (disorder)
        "93773005",  # Primary malignant neoplasm of dorsal surface of tongue (disorder)
        "93775003",  # Primary malignant neoplasm of duodenum (disorder)
        "93778001",  # Primary malignant neoplasm of endocardium (disorder)
        "93779009",  # Primary malignant neoplasm of endocervix (disorder)
        "93781006",  # Primary malignant neoplasm of endometrium (disorder)
        "93782004",  # Primary malignant neoplasm of epicardium (disorder)
        "93783009",  # Primary malignant neoplasm of epididymis (disorder)
        "93784003",  # Primary malignant neoplasm of epiglottis (disorder)
        "93786001",  # Primary malignant neoplasm of ethmoid bone (disorder)
        "93787005",  # Primary malignant neoplasm of ethmoidal sinus (disorder)
        "93788000",  # Primary malignant neoplasm of eustachian tube (disorder)
        "93789008",  # Primary malignant neoplasm of exocervix (disorder)
        "93792007",  # Primary malignant neoplasm of face (disorder)
        "93793002",  # Primary malignant neoplasm of facial nerve (disorder)
        "93797001",  # Primary malignant neoplasm of female genital organ (disorder)
        "93798006",  # Primary malignant neoplasm of femur (disorder)
        "93799003",  # Primary malignant neoplasm of fibula (disorder)
        "93800004",  # Primary malignant neoplasm of first cuneiform bone of foot (disorder)
        "93801000",  # Primary malignant neoplasm of flank (disorder)
        "93802007",  # Primary malignant neoplasm of floor of mouth (disorder)
        "93803002",  # Primary malignant neoplasm of foot (disorder)
        "93804008",  # Primary malignant neoplasm of forearm (disorder)
        "93806005",  # Primary malignant neoplasm of frontal bone (disorder)
        "93807001",  # Primary malignant neoplasm of frontal lobe (disorder)
        "93808006",  # Primary malignant neoplasm of frontal sinus (disorder)
        "93809003",  # Primary malignant neoplasm of fundus of stomach (disorder)
        "93812000",  # Primary malignant neoplasm of gingival mucosa (disorder)
        "93814004",  # Primary malignant neoplasm of glomus jugulare (disorder)
        "93815003",  # Primary malignant neoplasm of glossopharyngeal nerve (disorder)
        "93816002",  # Primary malignant neoplasm of glottis (disorder)
        "93817006",  # Primary malignant neoplasm of great vessels (disorder)
        "93818001",  # Primary malignant neoplasm of greater curvature of stomach (disorder)
        "93820003",  # Primary malignant neoplasm of hamate bone (disorder)
        "93821004",  # Primary malignant neoplasm of hand (disorder)
        "93824007",  # Primary malignant neoplasm of head (disorder)
        "93825008",  # Primary malignant neoplasm of heart (disorder)
        "93826009",  # Primary malignant neoplasm of hepatic flexure of colon (disorder)
        "93827000",  # Primary malignant neoplasm of hilus of lung (disorder)
        "93828005",  # Primary malignant neoplasm of hypoglossal nerve (disorder)
        "93829002",  # Primary malignant neoplasm of hypopharyngeal aspect of aryepiglottic fold (disorder)
        "93830007",  # Primary malignant neoplasm of hypopharyngeal aspect of interarytenoid fold (disorder)
        "93831006",  # Primary malignant neoplasm of hypopharynx (disorder)
        "93832004",  # Primary malignant neoplasm of ileum (disorder)
        "93833009",  # Primary malignant neoplasm of ilium (disorder)
        "93834003",  # Primary malignant neoplasm of inguinal region (disorder)
        "93835002",  # Primary malignant neoplasm of inner aspect of lip (disorder)
        "93836001",  # Primary malignant neoplasm of inner aspect of lower lip (disorder)
        "93837005",  # Primary malignant neoplasm of inner aspect of upper lip (disorder)
        "93839008",  # Primary malignant neoplasm of intra-abdominal organs (disorder)
        "93841009",  # Primary malignant neoplasm of intrathoracic organs (disorder)
        "93842002",  # Primary malignant neoplasm of ischium (disorder)
        "93843007",  # Primary malignant neoplasm of islets of Langerhans (disorder)
        "93844001",  # Primary malignant neoplasm of isthmus of uterus (disorder)
        "93845000",  # Primary malignant neoplasm of jaw (disorder)
        "93846004",  # Primary malignant neoplasm of jejunum (disorder)
        "93848003",  # Primary malignant neoplasm of junctional zone of tongue (disorder)
        "93849006",  # Primary malignant neoplasm of kidney (disorder)
        "93850006",  # Primary malignant neoplasm of labia majora (disorder)
        "93851005",  # Primary malignant neoplasm of labia minora (disorder)
        "93854002",  # Primary malignant neoplasm of large intestine (disorder)
        "93857009",  # Primary malignant neoplasm of laryngeal commissure (disorder)
        "93858004",  # Primary malignant neoplasm of laryngeal surface of epiglottis (disorder)
        "93860002",  # Primary malignant neoplasm of lateral portion of floor of mouth (disorder)
        "93861003",  # Primary malignant neoplasm of lateral wall of nasopharynx (disorder)
        "93862005",  # Primary malignant neoplasm of lateral wall of oropharynx (disorder)
        "93863000",  # Primary malignant neoplasm of lateral wall of urinary bladder (disorder)
        "93864006",  # Primary malignant neoplasm of lower lobe of left lung (disorder)
        "93865007",  # Primary malignant neoplasm of upper lobe of left lung (disorder)
        "93867004",  # Primary malignant neoplasm of lesser curvature of stomach (disorder)
        "93868009",  # Primary malignant neoplasm of lingual tonsil (disorder)
        "93870000",  # Malignant neoplasm of liver (disorder)
        "93871001",  # Primary malignant neoplasm of long bone of lower limb (disorder)
        "93872008",  # Primary malignant neoplasm of long bone of upper limb (disorder)
        "93875005",  # Primary malignant neoplasm of lower limb (disorder)
        "93878007",  # Primary malignant neoplasm of lumbar vertebral column (disorder)
        "93879004",  # Primary malignant neoplasm of lunate bone (disorder)
        "93880001",  # Primary malignant neoplasm of lung (disorder)
        "93882009",  # Primary malignant neoplasm of main bronchus (disorder)
        "93883004",  # Primary malignant neoplasm of major salivary gland (disorder)
        "93885006",  # Primary malignant neoplasm of male genital organ (disorder)
        "93886007",  # Primary malignant neoplasm of mandible (disorder)
        "93887003",  # Primary malignant neoplasm of mastoid air cells (disorder)
        "93888008",  # Primary malignant neoplasm of maxilla (disorder)
        "93889000",  # Primary malignant neoplasm of maxillary sinus (disorder)
        "93890009",  # Primary malignant neoplasm of Meckel's diverticulum (disorder)
        "93891008",  # Primary malignant neoplasm of mediastinum (disorder)
        "93892001",  # Primary malignant neoplasm of metacarpal bone (disorder)
        "93893006",  # Primary malignant neoplasm of metatarsal bone (disorder)
        "93894000",  # Primary malignant neoplasm of middle ear (disorder)
        "93898002",  # Primary malignant neoplasm of multiple endocrine glands (disorder)
        "93899005",  # Primary malignant neoplasm of muscle of abdomen (disorder)
        "93900000",  # Primary malignant neoplasm of muscle of buttock (disorder)
        "93901001",  # Primary malignant neoplasm of muscle of face (disorder)
        "93902008",  # Primary malignant neoplasm of muscle of head (disorder)
        "93903003",  # Primary malignant neoplasm of muscle of hip (disorder)
        "93904009",  # Primary malignant neoplasm of muscle of inguinal region (disorder)
        "93905005",  # Primary malignant neoplasm of muscle of lower limb (disorder)
        "93906006",  # Primary malignant neoplasm of muscle of neck (disorder)
        "93907002",  # Primary malignant neoplasm of muscle of pelvis (disorder)
        "93908007",  # Primary malignant neoplasm of muscle of perineum (disorder)
        "93909004",  # Primary malignant neoplasm of muscle of shoulder (disorder)
        "93910009",  # Primary malignant neoplasm of muscle of thorax (disorder)
        "93911008",  # Primary malignant neoplasm of muscle of trunk (disorder)
        "93912001",  # Primary malignant neoplasm of muscle of upper limb (disorder)
        "93913006",  # Primary malignant neoplasm of muscle (disorder)
        "93914000",  # Primary malignant neoplasm of myocardium (disorder)
        "93915004",  # Primary malignant neoplasm of myometrium (disorder)
        "93916003",  # Primary malignant neoplasm of nasal bone (disorder)
        "93917007",  # Primary malignant neoplasm of nasal cavity (disorder)
        "93918002",  # Primary malignant neoplasm of nasal concha (disorder)
        "93920004",  # Primary malignant neoplasm of navicular bone of foot (disorder)
        "93922007",  # Primary malignant neoplasm of neck (disorder)
        "93923002",  # Primary malignant neoplasm of nervous system (disorder)
        "93926005",  # Primary malignant neoplasm of nose (disorder)
        "93927001",  # Primary malignant neoplasm of occipital bone (disorder)
        "93928006",  # Primary malignant neoplasm of occipital lobe (disorder)
        "93929003",  # Primary malignant neoplasm of oculomotor nerve (disorder)
        "93930008",  # Primary malignant neoplasm of olfactory nerve (disorder)
        "93931007",  # Primary malignant neoplasm of optic nerve (disorder)
        "93932000",  # Primary malignant neoplasm of orbit (disorder)
        "93933005",  # Primary malignant neoplasm of oropharynx (disorder)
        "93934004",  # Primary malignant neoplasm of ovary (disorder)
        "93936002",  # Primary malignant neoplasm of palatine bone (disorder)
        "93939009",  # Primary malignant neoplasm of pancreatic duct (disorder)
        "93940006",  # Primary malignant neoplasm of para-aortic body (disorder)
        "93941005",  # Primary malignant neoplasm of paraganglion (disorder)
        "93942003",  # Primary malignant neoplasm of parametrium (disorder)
        "93943008",  # Primary malignant neoplasm of parathyroid gland (disorder)
        "93944002",  # Primary malignant neoplasm of paraurethral glands (disorder)
        "93945001",  # Primary malignant neoplasm of parietal bone (disorder)
        "93946000",  # Primary malignant neoplasm of parietal lobe (disorder)
        "93947009",  # Primary malignant neoplasm of parietal peritoneum (disorder)
        "93948004",  # Primary malignant neoplasm of parietal pleura (disorder)
        "93950007",  # Primary malignant neoplasm of patella (disorder)
        "93952004",  # Primary malignant neoplasm of pelvic peritoneum (disorder)
        "93953009",  # Primary malignant neoplasm of pelvis (disorder)
        "93955002",  # Primary malignant neoplasm of periadrenal tissue (disorder)
        "93956001",  # Primary malignant neoplasm of perianal skin (disorder)
        "93957005",  # Primary malignant neoplasm of pericardium (disorder)
        "93958000",  # Primary malignant neoplasm of perirenal tissue (disorder)
        "93959008",  # Primary malignant neoplasm of phalanx of foot (disorder)
        "93960003",  # Primary malignant neoplasm of phalanx of hand (disorder)
        "93961004",  # Primary malignant neoplasm of pharynx (disorder)
        "93962006",  # Primary malignant neoplasm of pineal gland (disorder)
        "93963001",  # Primary malignant neoplasm of pisiform bone of hand (disorder)
        "93964007",  # Primary malignant neoplasm of pituitary gland (disorder)
        "93966009",  # Primary malignant neoplasm of pleura (disorder)
        "93967000",  # Primary malignant neoplasm of postcricoid region (disorder)
        "93968005",  # Primary malignant neoplasm of posterior hypopharyngeal wall (disorder)
        "93969002",  # Primary malignant neoplasm of posterior mediastinum (disorder)
        "93970001",  # Primary malignant neoplasm of posterior wall of nasopharynx (disorder)
        "93971002",  # Primary malignant neoplasm of posterior wall of oropharynx (disorder)
        "93972009",  # Primary malignant neoplasm of posterior wall of urinary bladder (disorder)
        "93973004",  # Primary malignant neoplasm of presacral region (disorder)
        "93974005",  # Primary malignant neoplasm of prostate (disorder)
        "93975006",  # Primary malignant neoplasm of pubis (disorder)
        "93976007",  # Primary malignant neoplasm of pyloric antrum (disorder)
        "93977003",  # Primary malignant neoplasm of pylorus (disorder)
        "93978008",  # Primary malignant neoplasm of pyriform sinus (disorder)
        "93979000",  # Primary malignant neoplasm of radius (disorder)
        "93980002",  # Primary malignant neoplasm of rectosigmoid junction (disorder)
        "93981003",  # Primary malignant neoplasm of rectouterine pouch (disorder)
        "93982005",  # Primary malignant neoplasm of rectovaginal septum (disorder)
        "93983000",  # Primary malignant neoplasm of rectovesical septum (disorder)
        "93984006",  # Primary malignant neoplasm of rectum (disorder)
        "93985007",  # Primary malignant neoplasm of renal pelvis (disorder)
        "93986008",  # Primary malignant neoplasm of respiratory tract (disorder)
        "93987004",  # Primary malignant neoplasm of retina of eye (disorder)
        "93989001",  # Primary malignant neoplasm of retromolar area (disorder)
        "93990005",  # Primary malignant neoplasm of rib (disorder)
        "93991009",  # Primary malignant neoplasm of right lower lobe of lung (disorder)
        "93992002",  # Primary malignant neoplasm of middle lobe of right lung (disorder)
        "93993007",  # Primary malignant neoplasm of upper lobe of right lung (disorder)
        "93994001",  # Primary malignant neoplasm of round ligament of uterus (disorder)
        "93995000",  # Primary malignant neoplasm of sacrococcygeal region (disorder)
        "93996004",  # Primary malignant neoplasm of sacrum (disorder)
        "93997008",  # Primary malignant neoplasm of scapula (disorder)
        "93998003",  # Primary malignant neoplasm of sclera of eye (disorder)
        "94000008",  # Primary malignant neoplasm of sebaceous gland (disorder)
        "94001007",  # Primary malignant neoplasm of second cuneiform bone of foot (disorder)
        "94002000",  # Primary malignant neoplasm of septum of nose (disorder)
        "94003005",  # Primary malignant neoplasm of short bone of lower limb (disorder)
        "94004004",  # Primary malignant neoplasm of short bone of upper limb (disorder)
        "94005003",  # Primary malignant neoplasm of shoulder (disorder)
        "94006002",  # Primary malignant neoplasm of sigmoid colon (disorder)
        "94007006",  # Primary malignant neoplasm of skin of abdomen (disorder)
        "94008001",  # Primary malignant neoplasm of skin of ankle (disorder)
        "94010004",  # Primary malignant neoplasm of skin of axilla (disorder)
        "94011000",  # Primary malignant neoplasm of skin of back (disorder)
        "94012007",  # Primary malignant neoplasm of skin of breast (disorder)
        "94013002",  # Primary malignant neoplasm of skin of buttock (disorder)
        "94014008",  # Primary malignant neoplasm of skin of cheek (disorder)
        "94015009",  # Primary malignant neoplasm of skin of chest (disorder)
        "94016005",  # Primary malignant neoplasm of skin of chin (disorder)
        "94017001",  # Primary malignant neoplasm of skin of ear (disorder)
        "94018006",  # Primary malignant neoplasm of skin of elbow (disorder)
        "94019003",  # Primary malignant neoplasm of skin of external auditory canal (disorder)
        "94020009",  # Primary malignant neoplasm of skin of eyebrow (disorder)
        "94021008",  # Primary malignant neoplasm of skin of eyelid (disorder)
        "94022001",  # Primary malignant neoplasm of skin of face (disorder)
        "94023006",  # Primary malignant neoplasm of skin of finger (disorder)
        "94024000",  # Primary malignant neoplasm of skin of foot (disorder)
        "94025004",  # Primary malignant neoplasm of skin of forearm (disorder)
        "94026003",  # Primary malignant neoplasm of skin of forehead (disorder)
        "94027007",  # Primary malignant neoplasm of skin of groin (disorder)
        "94028002",  # Primary malignant neoplasm of skin of hand (disorder)
        "94029005",  # Primary malignant neoplasm of skin of hip (disorder)
        "94030000",  # Primary malignant neoplasm of skin of knee (disorder)
        "94032008",  # Primary malignant neoplasm of skin of lip (disorder)
        "94033003",  # Primary malignant neoplasm of skin of lower limb (disorder)
        "94034009",  # Primary malignant neoplasm of skin of neck (disorder)
        "94035005",  # Primary malignant neoplasm of skin of nose (disorder)
        "94036006",  # Primary malignant neoplasm of skin of perineum (disorder)
        "94037002",  # Primary malignant neoplasm of skin of popliteal area (disorder)
        "94038007",  # Primary malignant neoplasm of skin of scalp (disorder)
        "94039004",  # Primary malignant neoplasm of skin of shoulder (disorder)
        "94040002",  # Primary malignant neoplasm of skin of temporal region (disorder)
        "94041003",  # Primary malignant neoplasm of skin of thigh (disorder)
        "94042005",  # Primary malignant neoplasm of skin of toe (disorder)
        "94043000",  # Primary malignant neoplasm of skin of trunk (disorder)
        "94044006",  # Primary malignant neoplasm of skin of umbilicus (disorder)
        "94045007",  # Primary malignant neoplasm of skin of upper limb (disorder)
        "94046008",  # Primary malignant neoplasm of skin of wrist (disorder)
        "94047004",  # Primary malignant neoplasm of skin (disorder)
        "94048009",  # Primary malignant neoplasm of small intestine (disorder)
        "94049001",  # Primary malignant neoplasm of soft palate (disorder)
        "94050001",  # Primary malignant neoplasm of soft tissues of abdomen (disorder)
        "94051002",  # Primary malignant neoplasm of soft tissues of axilla (disorder)
        "94052009",  # Primary malignant neoplasm of soft tissues of buttock (disorder)
        "94053004",  # Primary malignant neoplasm of soft tissues of face (disorder)
        "94054005",  # Primary malignant neoplasm of soft tissues of head (disorder)
        "94055006",  # Primary malignant neoplasm of soft tissues of hip (disorder)
        "94056007",  # Primary malignant neoplasm of soft tissues of inguinal region (disorder)
        "94057003",  # Primary malignant neoplasm of soft tissues of lower limb (disorder)
        "94058008",  # Primary malignant neoplasm of soft tissues of neck (disorder)
        "94059000",  # Primary malignant neoplasm of soft tissues of pelvis (disorder)
        "94060005",  # Primary malignant neoplasm of soft tissues of perineum (disorder)
        "94061009",  # Primary malignant neoplasm of soft tissues of shoulder (disorder)
        "94062002",  # Primary malignant neoplasm of soft tissues of thorax (disorder)
        "94063007",  # Primary malignant neoplasm of soft tissues of trunk (disorder)
        "94066004",  # Primary malignant neoplasm of sphenoid bone (disorder)
        "94067008",  # Primary malignant neoplasm of sphenoidal sinus (disorder)
        "94068003",  # Primary malignant neoplasm of spinal cord (disorder)
        "94069006",  # Primary malignant neoplasm of spinal meninges (disorder)
        "94071006",  # Primary malignant neoplasm of spleen (disorder)
        "94072004",  # Primary malignant neoplasm of splenic flexure of colon (disorder)
        "94073009",  # Primary malignant neoplasm of sternum (disorder)
        "94075002",  # Primary malignant neoplasm of subglottis (disorder)
        "94076001",  # Primary malignant neoplasm of sublingual gland (disorder)
        "94077005",  # Primary malignant neoplasm of submaxillary gland (disorder)
        "94078000",  # Primary malignant neoplasm of superior wall of nasopharynx (disorder)
        "94079008",  # Primary malignant neoplasm of supraclavicular region (disorder)
        "94080006",  # Primary malignant neoplasm of supraglottis (disorder)
        "94081005",  # Primary malignant neoplasm of sweat gland (disorder)
        "94082003",  # Primary malignant neoplasm of tail of pancreas (disorder)
        "94083008",  # Primary malignant neoplasm of talus (disorder)
        "94084002",  # Primary malignant neoplasm of tarsal bone (disorder)
        "94085001",  # Primary malignant neoplasm of temporal bone (disorder)
        "94086000",  # Primary malignant neoplasm of temporal lobe (disorder)
        "94087009",  # Primary malignant neoplasm of testis (disorder)
        "94089007",  # Primary malignant neoplasm of the mesocolon (disorder)
        "94090003",  # Primary malignant neoplasm of omentum (disorder)
        "94092006",  # Primary malignant neoplasm of retroperitoneum (disorder)
        "94093001",  # Primary malignant neoplasm of thigh (disorder)
        "94094007",  # Primary malignant neoplasm of third cuneiform bone of foot (disorder)
        "94096009",  # Primary malignant neoplasm of thymus (disorder)
        "94097000",  # Primary malignant neoplasm of thyroglossal duct (disorder)
        "94098005",  # Primary malignant neoplasm of thyroid gland (disorder)
        "94099002",  # Primary malignant neoplasm of tibia (disorder)
        "94101009",  # Primary malignant neoplasm of tongue (disorder)
        "94102002",  # Primary malignant neoplasm of tonsillar fossa (disorder)
        "94103007",  # Primary malignant neoplasm of tonsillar pillar (disorder)
        "94104001",  # Primary malignant neoplasm of trachea (disorder)
        "94105000",  # Primary malignant neoplasm of transverse colon (disorder)
        "94107008",  # Primary malignant neoplasm of trapezoid bone (disorder)
        "94108003",  # Primary malignant neoplasm of trigeminal nerve (disorder)
        "94109006",  # Primary malignant neoplasm of trigone of urinary bladder (disorder)
        "94110001",  # Primary malignant neoplasm of trochlear nerve (disorder)
        "94111002",  # Primary malignant neoplasm of trunk (disorder)
        "94112009",  # Primary malignant neoplasm of ulna (disorder)
        "94113004",  # Primary malignant neoplasm of undescended testis (disorder)
        "94116007",  # Primary malignant neoplasm of upper limb (disorder)
        "94118008",  # Primary malignant neoplasm of upper respiratory tract (disorder)
        "94120006",  # Primary malignant neoplasm of urachus (disorder)
        "94121005",  # Primary malignant neoplasm of ureter (disorder)
        "94122003",  # Primary malignant neoplasm of ureteric orifice of urinary bladder (disorder)
        "94123008",  # Primary malignant neoplasm of urethra (disorder)
        "94124002",  # Primary malignant neoplasm of urinary bladder neck (disorder)
        "94125001",  # Primary malignant neoplasm of urinary system (disorder)
        "94126000",  # Primary malignant neoplasm of uterine adnexa (disorder)
        "94128004",  # Primary malignant neoplasm of uveal tract of eye (disorder)
        "94129007",  # Primary malignant neoplasm of uvula (disorder)
        "94131003",  # Primary malignant neoplasm of vagus nerve (disorder)
        "94132005",  # Primary malignant neoplasm of vallecula (disorder)
        "94133000",  # Primary malignant neoplasm of vas deferens (disorder)
        "94134006",  # Primary malignant neoplasm of ventral surface of tongue (disorder)
        "94135007",  # Primary malignant neoplasm of vermilion border of lip (disorder)
        "94138009",  # Primary malignant neoplasm of vestibule of mouth (disorder)
        "94139001",  # Primary malignant neoplasm of vestibule of nose (disorder)
        "94140004",  # Primary malignant neoplasm of visceral pleura (disorder)
        "94142007",  # Primary malignant neoplasm of vomer (disorder)
        "94143002",  # Primary malignant neoplasm of vulva (disorder)
        "94144008",  # Primary malignant neoplasm of Waldeyer's ring (disorder)
        "94145009",  # Primary malignant neoplasm of zygomatic bone (disorder)
        "94148006",  # Megakaryocytic leukemia in remission (disorder)
        "94152006",  # Metastatic malignant neoplasm to abdominal esophagus (disorder)
        "94153001",  # Metastatic malignant neoplasm to abducens nerve (disorder)
        "94154007",  # Metastatic malignant neoplasm to accessory nerve (disorder)
        "94155008",  # Metastatic malignant neoplasm to accessory sinus (disorder)
        "94156009",  # Metastatic malignant neoplasm of vestibulocochlear nerve (disorder)
        "94157000",  # Metastatic malignant neoplasm to acromion (disorder)
        "94158005",  # Metastatic malignant neoplasm to adenoid (disorder)
        "94159002",  # Metastatic malignant neoplasm to adnexa of skin (disorder)
        "94160007",  # Metastatic malignant neoplasm to adrenal cortex (disorder)
        "94161006",  # Metastatic malignant neoplasm to adrenal gland (disorder)
        "94162004",  # Metastatic malignant neoplasm to adrenal medulla (disorder)
        "94163009",  # Metastatic malignant neoplasm to alveolar ridge mucosa (disorder)
        "94164003",  # Metastatic malignant neoplasm to ampulla of Vater (disorder)
        "94165002",  # Metastatic malignant neoplasm to anal canal (disorder)
        "94166001",  # Metastatic malignant neoplasm to anterior aspect of epiglottis (disorder)
        "94167005",  # Metastatic malignant neoplasm to anterior mediastinum (disorder)
        "94168000",  # Metastatic malignant neoplasm to anterior portion of floor of mouth (disorder)
        "94169008",  # Metastatic malignant neoplasm to anterior two-thirds of tongue (disorder)
        "94170009",  # Metastatic malignant neoplasm to anterior wall of nasopharynx (disorder)
        "94171008",  # Metastatic malignant neoplasm to anterior wall of urinary bladder (disorder)
        "94172001",  # Metastatic malignant neoplasm to anus (disorder)
        "94173006",  # Metastatic malignant neoplasm to aortic body (disorder)
        "94175004",  # Metastatic malignant neoplasm to appendix (disorder)
        "94179005",  # Metastatic malignant neoplasm to ascending colon (disorder)
        "94180008",  # Metastatic malignant neoplasm to axilla (disorder)
        "94181007",  # Metastatic malignant neoplasm to axillary lymph nodes (disorder)
        "94183005",  # Metastatic malignant neoplasm to back (disorder)
        "94184004",  # Metastatic malignant neoplasm to base of tongue (disorder)
        "94185003",  # Metastatic malignant neoplasm to biliary tract (disorder)
        "94186002",  # Metastatic malignant neoplasm to urinary bladder (disorder)
        "94187006",  # Metastatic malignant neoplasm to blood vessel of abdomen (disorder)
        "94189009",  # Metastatic malignant neoplasm to blood vessel of axilla (disorder)
        "94190000",  # Metastatic malignant neoplasm to blood vessel of buttock (disorder)
        "94191001",  # Metastatic malignant neoplasm to blood vessel of face (disorder)
        "94192008",  # Metastatic malignant neoplasm to blood vessel of finger (disorder)
        "94193003",  # Metastatic malignant neoplasm to blood vessel of foot (disorder)
        "94194009",  # Metastatic malignant neoplasm to blood vessel of forearm (disorder)
        "94195005",  # Metastatic malignant neoplasm to blood vessel of hand (disorder)
        "94196006",  # Metastatic malignant neoplasm to blood vessel of head (disorder)
        "94197002",  # Metastatic malignant neoplasm to blood vessel of hip (disorder)
        "94198007",  # Metastatic malignant neoplasm to blood vessel of inguinal region (disorder)
        "94200001",  # Metastatic malignant neoplasm to blood vessel of lower limb (disorder)
        "94201002",  # Metastatic malignant neoplasm to blood vessel of neck (disorder)
        "94202009",  # Metastatic malignant neoplasm to blood vessel of pelvis (disorder)
        "94203004",  # Metastatic malignant neoplasm to blood vessel of perineum (disorder)
        "94204005",  # Metastatic malignant neoplasm to blood vessel of popliteal space (disorder)
        "94205006",  # Metastatic malignant neoplasm to blood vessel of shoulder (disorder)
        "94206007",  # Metastatic malignant neoplasm to blood vessel of thigh (disorder)
        "94207003",  # Metastatic malignant neoplasm to blood vessel of thorax (disorder)
        "94208008",  # Metastatic malignant neoplasm to blood vessel of toe (disorder)
        "94209000",  # Metastatic malignant neoplasm to blood vessel of trunk (disorder)
        "94210005",  # Metastatic malignant neoplasm to blood vessel of upper limb (disorder)
        "94211009",  # Metastatic malignant neoplasm to blood vessel (disorder)
        "94212002",  # Metastatic malignant neoplasm to body of pancreas (disorder)
        "94213007",  # Metastatic malignant neoplasm to body of penis (disorder)
        "94214001",  # Metastatic malignant neoplasm to body of stomach (disorder)
        "94215000",  # Metastatic malignant neoplasm to body of uterus (disorder)
        "94217008",  # Metastatic malignant neoplasm to bone marrow (disorder)
        "94218003",  # Metastatic malignant neoplasm to bone of face (disorder)
        "94219006",  # Metastatic malignant neoplasm to bone of lower limb (disorder)
        "94220000",  # Metastatic malignant neoplasm to bone of skull (disorder)
        "94221001",  # Metastatic malignant neoplasm to bone of upper limb (disorder)
        "94222008",  # Metastatic malignant neoplasm to bone (disorder)
        "94224009",  # Metastatic malignant neoplasm to brain stem (disorder)
        "94225005",  # Metastatic malignant neoplasm to brain (disorder)
        "94226006",  # Metastatic malignant neoplasm to broad ligament (disorder)
        "94227002",  # Metastatic malignant neoplasm to bronchopulmonary lymph nodes (disorder)
        "94228007",  # Metastatic malignant neoplasm to bronchus of left lower lobe (disorder)
        "94229004",  # Metastatic malignant neoplasm to bronchus of left upper lobe (disorder)
        "94230009",  # Metastatic malignant neoplasm to bronchus of right lower lobe (disorder)
        "94231008",  # Metastatic malignant neoplasm to bronchus of middle lobe of right lung (disorder)
        "94232001",  # Metastatic malignant neoplasm to bronchus of right upper lobe (disorder)
        "94233006",  # Metastatic malignant neoplasm to bronchus (disorder)
        "94234000",  # Metastatic malignant neoplasm to buccal mucosa (disorder)
        "94235004",  # Metastatic malignant neoplasm to cecum (disorder)
        "94236003",  # Metastatic malignant neoplasm to calcaneus (disorder)
        "94237007",  # Metastatic malignant neoplasm to cardia of stomach (disorder)
        "94238002",  # Metastatic malignant neoplasm to carina (disorder)
        "94239005",  # Metastatic malignant neoplasm to carotid body (disorder)
        "94240007",  # Metastatic malignant neoplasm to carpal bone (disorder)
        "94241006",  # Metastatic malignant neoplasm to cartilage of nose (disorder)
        "94242004",  # Metastatic malignant neoplasm to cauda equina (disorder)
        "94243009",  # Metastatic malignant neoplasm to central nervous system (disorder)
        "94245002",  # Metastatic malignant neoplasm to cerebellum (disorder)
        "94246001",  # Metastatic malignant neoplasm to cerebral meninges (disorder)
        "94247005",  # Metastatic malignant neoplasm to cerebral ventricle (disorder)
        "94248000",  # Metastatic malignant neoplasm to cerebrum (disorder)
        "94249008",  # Metastatic malignant neoplasm to cervical esophagus (disorder)
        "94250008",  # Metastatic malignant neoplasm to cervical vertebral column (disorder)
        "94252000",  # Metastatic malignant neoplasm to cheek (disorder)
        "94253005",  # Metastatic malignant neoplasm to chest wall (disorder)
        "94254004",  # Metastatic malignant neoplasm to choroid (disorder)
        "94255003",  # Metastatic malignant neoplasm to ciliary body (disorder)
        "94256002",  # Metastatic malignant neoplasm to clavicle (disorder)
        "94257006",  # Metastatic malignant neoplasm to clitoris (disorder)
        "94258001",  # Metastatic malignant neoplasm to coccygeal body (disorder)
        "94259009",  # Metastatic malignant neoplasm to coccyx (disorder)
        "94260004",  # Metastatic malignant neoplasm to colon (disorder)
        "94261000",  # Metastatic malignant neoplasm to commissure of lip (disorder)
        "94262007",  # Metastatic malignant neoplasm to common bile duct (disorder)
        "94263002",  # Metastatic malignant neoplasm to conjunctiva (disorder)
        "94264008",  # Metastatic malignant neoplasm to soft tissues (disorder)
        "94265009",  # Metastatic malignant neoplasm to cornea (disorder)
        "94266005",  # Metastatic malignant neoplasm to cranial nerve (disorder)
        "94267001",  # Metastatic malignant neoplasm to craniopharyngeal duct (disorder)
        "94268006",  # Metastatic malignant neoplasm to cubital lymph nodes (disorder)
        "94269003",  # Metastatic malignant neoplasm to cuboid (disorder)
        "94270002",  # Metastatic malignant neoplasm to cystic duct (disorder)
        "94271003",  # Metastatic malignant neoplasm to descending colon (disorder)
        "94272005",  # Metastatic malignant neoplasm to diaphragm (disorder)
        "94273000",  # Metastatic malignant neoplasm to dorsal surface of tongue (disorder)
        "94274006",  # Metastatic malignant neoplasm to thoracic vertebral column (disorder)
        "94275007",  # Metastatic malignant neoplasm to duodenum (disorder)
        "94278009",  # Metastatic malignant neoplasm to endocardium (disorder)
        "94279001",  # Metastatic malignant neoplasm to endocervix (disorder)
        "94280003",  # Metastatic malignant neoplasm to endocrine gland (disorder)
        "94281004",  # Metastatic malignant neoplasm to endometrium (disorder)
        "94282006",  # Metastatic malignant neoplasm to epicardium (disorder)
        "94283001",  # Metastatic malignant neoplasm to epididymis (disorder)
        "94284007",  # Metastatic malignant neoplasm to epiglottis (disorder)
        "94285008",  # Metastatic malignant neoplasm to epitrochlear lymph nodes (disorder)
        "94286009",  # Metastatic malignant neoplasm to esophagus (disorder)
        "94287000",  # Metastatic malignant neoplasm to ethmoid bone (disorder)
        "94288005",  # Metastatic malignant neoplasm to ethmoidal sinus (disorder)
        "94289002",  # Metastatic malignant neoplasm to eustachian tube (disorder)
        "94290006",  # Metastatic malignant neoplasm to exocervix (disorder)
        "94291000119103",  # Mucinous adenocarcinoma of gastrointestinal tract (disorder)
        "94291005",  # Metastatic malignant neoplasm to extrahepatic bile ducts (disorder)
        "94292003",  # Metastatic malignant neoplasm to eye (disorder)
        "94293008",  # Metastatic malignant neoplasm to face (disorder)
        "94294002",  # Metastatic malignant neoplasm to facial nerve (disorder)
        "94295001",  # Metastatic malignant neoplasm to fallopian tube (disorder)
        "94296000",  # Metastatic malignant neoplasm to false vocal cord (disorder)
        "94298004",  # Metastatic malignant neoplasm to female genital organ (disorder)
        "94299007",  # Metastatic malignant neoplasm to femoral lymph nodes (disorder)
        "94300004",  # Metastatic malignant neoplasm to femur (disorder)
        "94301000",  # Metastatic malignant neoplasm to fibula (disorder)
        "94302007",  # Metastatic malignant neoplasm to first cuneiform bone of foot (disorder)
        "94303002",  # Metastatic malignant neoplasm to flank (disorder)
        "94304008",  # Metastatic malignant neoplasm to floor of mouth (disorder)
        "94305009",  # Metastatic malignant neoplasm to foot (disorder)
        "94306005",  # Metastatic malignant neoplasm to forearm (disorder)
        "94307001",  # Metastatic malignant neoplasm to prepuce (disorder)
        "94308006",  # Metastatic malignant neoplasm to frontal bone (disorder)
        "94309003",  # Metastatic malignant neoplasm to frontal lobe (disorder)
        "94310008",  # Metastatic malignant neoplasm to frontal sinus (disorder)
        "94311007",  # Metastatic malignant neoplasm to fundus of stomach (disorder)
        "94312000",  # Metastatic malignant neoplasm to gallbladder (disorder)
        "94313005",  # Metastatic malignant neoplasm to gastrointestinal tract (disorder)
        "94314004",  # Metastatic malignant neoplasm to gingival mucosa (disorder)
        "94315003",  # Metastatic malignant neoplasm to glans penis (disorder)
        "94316002",  # Metastatic malignant neoplasm to glomus jugulare (disorder)
        "94317006",  # Metastatic malignant neoplasm to glossopharyngeal nerve (disorder)
        "94318001",  # Metastatic malignant neoplasm to glottis (disorder)
        "94319009",  # Metastatic malignant neoplasm to great vessels (disorder)
        "94320003",  # Metastatic malignant neoplasm to greater curvature of stomach (disorder)
        "94321004",  # Metastatic malignant neoplasm to gum (disorder)
        "94322006",  # Metastatic malignant neoplasm to hamate bone (disorder)
        "94323001",  # Metastatic malignant neoplasm to hand (disorder)
        "94324007",  # Metastatic malignant neoplasm to hard palate (disorder)
        "94325008",  # Metastatic malignant neoplasm to head of pancreas (disorder)
        "94326009",  # Metastatic malignant neoplasm to head (disorder)
        "94327000",  # Metastatic malignant neoplasm to heart (disorder)
        "94328005",  # Metastatic malignant neoplasm to hepatic flexure of colon (disorder)
        "94329002",  # Metastatic malignant neoplasm to hilus of lung (disorder)
        "94330007",  # Metastatic malignant neoplasm to hypogastric lymph nodes (disorder)
        "94331006",  # Metastatic malignant neoplasm to hypoglossal nerve (disorder)
        "94332004",  # Metastatic malignant neoplasm to hypopharyngeal aspect of aryepiglottic fold (disorder)
        "94333009",  # Metastatic malignant neoplasm to hypopharyngeal aspect of interarytenoid fold (disorder)
        "94334003",  # Metastatic malignant neoplasm to hypopharynx (disorder)
        "94335002",  # Metastatic malignant neoplasm to ileum (disorder)
        "94336001",  # Metastatic malignant neoplasm to iliac lymph nodes (disorder)
        "94337005",  # Metastatic malignant neoplasm to ilium (disorder)
        "94338000",  # Metastatic malignant neoplasm to infraclavicular lymph nodes (disorder)
        "94339008",  # Metastatic malignant neoplasm to inguinal lymph nodes (disorder)
        "94340005",  # Metastatic malignant neoplasm to inguinal region (disorder)
        "94341009",  # Metastatic malignant neoplasm to inner aspect of lip (disorder)
        "94342002",  # Metastatic malignant neoplasm to inner aspect of lower lip (disorder)
        "94343007",  # Metastatic malignant neoplasm to inner aspect of upper lip (disorder)
        "94344001",  # Metastatic malignant neoplasm to intercostal lymph nodes (disorder)
        "94345000",  # Metastatic malignant neoplasm to intestinal lymph nodes (disorder)
        "94346004",  # Metastatic malignant neoplasm to intestinal tract (disorder)
        "94347008",  # Metastatic malignant neoplasm to intra-abdominal lymph nodes (disorder)
        "94348003",  # Metastatic malignant neoplasm to intra-abdominal organs (disorder)
        "94349006",  # Metastatic malignant neoplasm to intrahepatic bile ducts (disorder)
        "94350006",  # Metastatic malignant neoplasm to intrapelvic lymph nodes (disorder)
        "94351005",  # Metastatic malignant neoplasm to intrathoracic lymph nodes (disorder)
        "94352003",  # Metastatic malignant neoplasm to intrathoracic organs (disorder)
        "94353008",  # Metastatic malignant neoplasm to ischium (disorder)
        "94354002",  # Metastatic malignant neoplasm to islets of Langerhans (disorder)
        "94355001",  # Metastatic malignant neoplasm to isthmus of uterus (disorder)
        "94356000",  # Metastatic malignant neoplasm to jaw (disorder)
        "94357009",  # Metastatic malignant neoplasm to jejunum (disorder)
        "94358004",  # Metastatic malignant neoplasm to junctional region of epiglottis (disorder)
        "94359007",  # Metastatic malignant neoplasm to junctional zone of tongue (disorder)
        "94360002",  # Metastatic malignant neoplasm to kidney (disorder)
        "94361003",  # Metastatic malignant neoplasm to labia majora (disorder)
        "94362005",  # Metastatic malignant neoplasm to labia minora (disorder)
        "94364006",  # Metastatic malignant neoplasm to lacrimal gland (disorder)
        "94365007",  # Metastatic malignant neoplasm to large intestine (disorder)
        "94366008",  # Metastatic malignant neoplasm to laryngeal aspect of aryepiglottic fold (disorder)
        "94367004",  # Metastatic malignant neoplasm to laryngeal aspect of interarytenoid fold (disorder)
        "94368009",  # Metastatic malignant neoplasm to laryngeal commissure (disorder)
        "94369001",  # Metastatic malignant neoplasm to laryngeal surface of epiglottis (disorder)
        "94370000",  # Metastatic malignant neoplasm to larynx (disorder)
        "94371001",  # Metastatic malignant neoplasm to lateral portion of floor of mouth (disorder)
        "94372008",  # Metastatic malignant neoplasm to lateral wall of nasopharynx (disorder)
        "94373003",  # Metastatic malignant neoplasm to lateral wall of oropharynx (disorder)
        "94374009",  # Metastatic malignant neoplasm to lateral wall of urinary bladder (disorder)
        "94375005",  # Metastatic malignant neoplasm to left lower lobe of lung (disorder)
        "94376006",  # Metastatic malignant neoplasm to left upper lobe of lung (disorder)
        "94378007",  # Metastatic malignant neoplasm to lesser curvature of stomach (disorder)
        "94379004",  # Metastatic malignant neoplasm to lingual tonsil (disorder)
        "94380001",  # Metastatic malignant neoplasm to lip (disorder)
        "94381002",  # Metastatic malignant neoplasm to liver (disorder)
        "94382009",  # Metastatic malignant neoplasm to long bone of lower limb (disorder)
        "94383004",  # Metastatic malignant neoplasm to long bone of upper limb (disorder)
        "94384005",  # Metastatic malignant neoplasm to lower gum (disorder)
        "94386007",  # Metastatic malignant neoplasm to lower limb (disorder)
        "94388008",  # Metastatic malignant neoplasm to lower third of esophagus (disorder)
        "94389000",  # Metastatic malignant neoplasm to lumbar vertebral column (disorder)
        "94390009",  # Metastatic malignant neoplasm to lunate bone (disorder)
        "94391008",  # Metastatic malignant neoplasm to lung (disorder)
        "94392001",  # Metastatic malignant neoplasm to lymph node (disorder)
        "94393006",  # Metastatic malignant neoplasm to lymph nodes of face (disorder)
        "94394000",  # Metastatic malignant neoplasm to lymph nodes of head (disorder)
        "94395004",  # Metastatic malignant neoplasm to lymph nodes of lower limb (disorder)
        "94396003",  # Metastatic malignant neoplasm to lymph nodes of multiple sites (disorder)
        "94397007",  # Metastatic malignant neoplasm to lymph nodes of neck (disorder)
        "94398002",  # Metastatic malignant neoplasm to lymph nodes of upper limb (disorder)
        "94399005",  # Metastatic malignant neoplasm to main bronchus (disorder)
        "94400003",  # Metastatic malignant neoplasm to major salivary gland (disorder)
        "94402006",  # Metastatic malignant neoplasm to male genital organ (disorder)
        "94403001",  # Metastatic malignant neoplasm to mandible (disorder)
        "94404007",  # Metastatic malignant neoplasm to mastoid air cells (disorder)
        "94405008",  # Metastatic malignant neoplasm to maxilla (disorder)
        "94406009",  # Metastatic malignant neoplasm to maxillary sinus (disorder)
        "94407000",  # Metastatic malignant neoplasm to Meckel's diverticulum (disorder)
        "94408005",  # Metastatic malignant neoplasm to mediastinal lymph nodes (disorder)
        "94409002",  # Metastatic malignant neoplasm to mediastinum (disorder)
        "94410007",  # Metastatic malignant neoplasm to mesenteric lymph nodes (disorder)
        "94411006",  # Metastatic malignant neoplasm to metacarpal bone (disorder)
        "94412004",  # Metastatic malignant neoplasm to metatarsal bone (disorder)
        "94413009",  # Metastatic malignant neoplasm to middle ear (disorder)
        "94414003",  # Metastatic malignant neoplasm to middle third of esophagus (disorder)
        "94415002",  # Metastatic malignant neoplasm to minor salivary gland (disorder)
        "94416001",  # Metastatic malignant neoplasm to mouth (disorder)
        "94417005",  # Metastatic malignant neoplasm to multiple endocrine glands (disorder)
        "94418000",  # Metastatic malignant neoplasm to muscle of abdomen (disorder)
        "94419008",  # Metastatic malignant neoplasm to muscle of buttock (disorder)
        "94420002",  # Metastatic malignant neoplasm to muscle of face (disorder)
        "94421003",  # Metastatic malignant neoplasm to muscle of head (disorder)
        "94422005",  # Metastatic malignant neoplasm to muscle of hip (disorder)
        "94423000",  # Metastatic malignant neoplasm to muscle of inguinal region (disorder)
        "94424006",  # Metastatic malignant neoplasm to muscle of lower limb (disorder)
        "94425007",  # Metastatic malignant neoplasm to muscle of neck (disorder)
        "94426008",  # Metastatic malignant neoplasm to muscle of pelvis (disorder)
        "94427004",  # Metastatic malignant neoplasm to muscle of perineum (disorder)
        "94428009",  # Metastatic malignant neoplasm to muscle of shoulder (disorder)
        "94429001",  # Metastatic malignant neoplasm to muscle of thorax (disorder)
        "94430006",  # Metastatic malignant neoplasm to muscle of trunk (disorder)
        "94431005",  # Metastatic malignant neoplasm to muscle of upper limb (disorder)
        "94432003",  # Metastatic malignant neoplasm to muscle (disorder)
        "94433008",  # Metastatic malignant neoplasm to myocardium (disorder)
        "94434002",  # Metastatic malignant neoplasm to myometrium (disorder)
        "94435001",  # Metastatic malignant neoplasm to nasal bone (disorder)
        "94436000",  # Metastatic malignant neoplasm to nasal cavity (disorder)
        "94437009",  # Metastatic malignant neoplasm to nasal concha (disorder)
        "94439007",  # Metastatic malignant neoplasm to navicular bone of foot (disorder)
        "94440009",  # Metastatic malignant neoplasm to scaphoid bone (disorder)
        "94441008",  # Metastatic malignant neoplasm to neck (disorder)
        "94442001",  # Metastatic malignant neoplasm to nervous system (disorder)
        "94445004",  # Metastatic malignant neoplasm to nose (disorder)
        "94446003",  # Metastatic malignant neoplasm to obturator lymph nodes (disorder)
        "94447007",  # Metastatic malignant neoplasm to occipital bone (disorder)
        "94448002",  # Metastatic malignant neoplasm to occipital lobe (disorder)
        "94449005",  # Metastatic malignant neoplasm to occipital lymph nodes (disorder)
        "94450005",  # Metastatic malignant neoplasm to oculomotor nerve (disorder)
        "94451009",  # Metastatic malignant neoplasm to olfactory nerve (disorder)
        "94452002",  # Metastatic malignant neoplasm to optic nerve (disorder)
        "94453007",  # Metastatic malignant neoplasm to orbit (disorder)
        "94454001",  # Metastatic malignant neoplasm to oropharynx (disorder)
        "94455000",  # Metastatic malignant neoplasm to ovary (disorder)
        "94456004",  # Metastatic malignant neoplasm to palate (disorder)
        "94457008",  # Metastatic malignant neoplasm to palatine bone (disorder)
        "94458003",  # Metastatic malignant neoplasm to tonsil (disorder)
        "94459006",  # Metastatic malignant neoplasm to pancreas (disorder)
        "94460001",  # Metastatic malignant neoplasm to pancreatic duct (disorder)
        "94461002",  # Metastatic malignant neoplasm to para-aortic body (disorder)
        "94462009",  # Metastatic malignant neoplasm to paraganglion (disorder)
        "94463004",  # Metastatic malignant neoplasm to paramammary lymph nodes (disorder)
        "94464005",  # Metastatic malignant neoplasm to parametrial lymph nodes (disorder)
        "94465006",  # Metastatic malignant neoplasm to parametrium (disorder)
        "94466007",  # Metastatic malignant neoplasm to pararectal lymph nodes (disorder)
        "94467003",  # Metastatic malignant neoplasm to parathyroid gland (disorder)
        "94468008",  # Metastatic malignant neoplasm to paraurethral glands (disorder)
        "94469000",  # Metastatic malignant neoplasm to paravaginal lymph nodes (disorder)
        "94470004",  # Metastatic malignant neoplasm to parietal bone (disorder)
        "94471000",  # Metastatic malignant neoplasm to parietal lobe (disorder)
        "94472007",  # Metastatic malignant neoplasm to parietal peritoneum (disorder)
        "94473002",  # Metastatic malignant neoplasm to parietal pleura (disorder)
        "94474008",  # Metastatic malignant neoplasm to parotid gland (disorder)
        "94475009",  # Metastatic malignant neoplasm to parotid lymph nodes (disorder)
        "94476005",  # Metastatic malignant neoplasm to patella (disorder)
        "94477001",  # Metastatic malignant neoplasm to pectoral axillary lymph nodes (disorder)
        "94478006",  # Metastatic malignant neoplasm to pelvic bone (disorder)
        "94479003",  # Metastatic malignant neoplasm to pelvic peritoneum (disorder)
        "94480000",  # Metastatic malignant neoplasm to pelvis (disorder)
        "94481001",  # Metastatic malignant neoplasm to penis (disorder)
        "94482008",  # Metastatic malignant neoplasm to periadrenal tissue (disorder)
        "94483003",  # Metastatic malignant neoplasm to perianal skin (disorder)
        "94484009",  # Metastatic malignant neoplasm to pericardium (disorder)
        "94485005",  # Metastatic malignant neoplasm to perirenal tissue (disorder)
        "94486006",  # Metastatic malignant neoplasm to phalanx of foot (disorder)
        "94487002",  # Metastatic malignant neoplasm to phalanx of hand (disorder)
        "94488007",  # Metastatic malignant neoplasm to pharynx (disorder)
        "94489004",  # Metastatic malignant neoplasm to pineal gland (disorder)
        "94490008",  # Metastatic malignant neoplasm to pisiform bone of hand (disorder)
        "94491007",  # Metastatic malignant neoplasm to pituitary gland (disorder)
        "94492000",  # Metastatic malignant neoplasm to placenta (disorder)
        "94493005",  # Metastatic malignant neoplasm to pleura (disorder)
        "94494004",  # Metastatic malignant neoplasm to popliteal lymph nodes (disorder)
        "94495003",  # Metastatic malignant neoplasm to postcricoid region (disorder)
        "94496002",  # Metastatic malignant neoplasm to posterior hypopharyngeal wall (disorder)
        "94497006",  # Metastatic malignant neoplasm to posterior mediastinum (disorder)
        "94498001",  # Metastatic malignant neoplasm to posterior wall of nasopharynx (disorder)
        "94499009",  # Metastatic malignant neoplasm to posterior wall of oropharynx (disorder)
        "94500000",  # Metastatic malignant neoplasm to posterior wall of urinary bladder (disorder)
        "94501001",  # Metastatic malignant neoplasm to preauricular lymph nodes (disorder)
        "94502008",  # Metastatic malignant neoplasm to presacral region (disorder)
        "94503003",  # Metastatic malignant neoplasm to prostate (disorder)
        "94504009",  # Metastatic malignant neoplasm to pubis (disorder)
        "94505005",  # Metastatic malignant neoplasm to pyloric antrum (disorder)
        "94506006",  # Metastatic malignant neoplasm to pylorus (disorder)
        "94507002",  # Metastatic malignant neoplasm to pyriform sinus (disorder)
        "94508007",  # Metastatic malignant neoplasm to radius (disorder)
        "94509004",  # Metastatic malignant neoplasm to rectosigmoid junction (disorder)
        "94510009",  # Metastatic malignant neoplasm to rectouterine pouch (disorder)
        "94511008",  # Metastatic malignant neoplasm to rectovaginal septum (disorder)
        "94512001",  # Metastatic malignant neoplasm to rectovesical septum (disorder)
        "94513006",  # Metastatic malignant neoplasm to rectum (disorder)
        "94514000",  # Metastatic malignant neoplasm to renal pelvis (disorder)
        "94515004",  # Metastatic malignant neoplasm to respiratory tract (disorder)
        "94516003",  # Metastatic malignant neoplasm to retina (disorder)
        "94517007",  # Metastatic malignant neoplasm to retrocecal tissue (disorder)
        "94518002",  # Metastatic malignant neoplasm to retromolar area (disorder)
        "94519005",  # Metastatic malignant neoplasm to retroperitoneal lymph nodes (disorder)
        "94520004",  # Metastatic malignant neoplasm to retropharyngeal lymph nodes (disorder)
        "94521000",  # Metastatic malignant neoplasm to rib (disorder)
        "94522007",  # Metastatic malignant neoplasm to right lower lobe of lung (disorder)
        "94523002",  # Metastatic malignant neoplasm to middle lobe of right lung (disorder)
        "94524008",  # Metastatic malignant neoplasm to right upper lobe of lung (disorder)
        "94525009",  # Metastatic malignant neoplasm to round ligament of uterus (disorder)
        "94526005",  # Metastatic malignant neoplasm to sacrococcygeal region (disorder)
        "94527001",  # Metastatic malignant neoplasm to sacrum (disorder)
        "94528006",  # Metastatic malignant neoplasm to scalene lymph nodes (disorder)
        "94529003",  # Metastatic malignant neoplasm to scapula (disorder)
        "94530008",  # Metastatic malignant neoplasm to sclera (disorder)
        "94531007",  # Metastatic malignant neoplasm to scrotum (disorder)
        "94532000",  # Metastatic malignant neoplasm to sebaceous gland (disorder)
        "94533005",  # Metastatic malignant neoplasm to second cuneiform bone of foot (disorder)
        "94534004",  # Metastatic malignant neoplasm to septum of nose (disorder)
        "94535003",  # Metastatic malignant neoplasm to short bone of lower limb (disorder)
        "94536002",  # Metastatic malignant neoplasm to short bone of upper limb (disorder)
        "94537006",  # Metastatic malignant neoplasm to shoulder (disorder)
        "94538001",  # Metastatic malignant neoplasm to sigmoid colon (disorder)
        "94539009",  # Metastatic malignant neoplasm to skin of abdomen (disorder)
        "94540006",  # Metastatic malignant neoplasm to skin of ankle (disorder)
        "94542003",  # Metastatic malignant neoplasm to skin of axilla (disorder)
        "94543008",  # Metastatic malignant neoplasm to skin of back (disorder)
        "94544002",  # Metastatic malignant neoplasm to skin of breast (disorder)
        "94545001",  # Metastatic malignant neoplasm to skin of buttock (disorder)
        "94546000",  # Metastatic malignant neoplasm to skin of cheek (disorder)
        "94547009",  # Metastatic malignant neoplasm to skin of chest (disorder)
        "94548004",  # Metastatic malignant neoplasm to skin of chin (disorder)
        "94549007",  # Metastatic malignant neoplasm to skin of ear (disorder)
        "94550007",  # Metastatic malignant neoplasm to skin of elbow (disorder)
        "94551006",  # Metastatic malignant neoplasm to skin of external auditory canal (disorder)
        "94552004",  # Metastatic malignant neoplasm to skin of eyebrow (disorder)
        "94553009",  # Metastatic malignant neoplasm to skin of eyelid (disorder)
        "94554003",  # Metastatic malignant neoplasm to skin of face (disorder)
        "94555002",  # Metastatic malignant neoplasm to skin of finger (disorder)
        "94556001",  # Metastatic malignant neoplasm to skin of foot (disorder)
        "94557005",  # Metastatic malignant neoplasm to skin of forearm (disorder)
        "94558000",  # Metastatic malignant neoplasm to skin of forehead (disorder)
        "94559008",  # Metastatic malignant neoplasm to skin of groin (disorder)
        "94560003",  # Metastatic malignant neoplasm to skin of hand (disorder)
        "94561004",  # Metastatic malignant neoplasm to skin of hip (disorder)
        "94562006",  # Metastatic malignant neoplasm to skin of knee (disorder)
        "94564007",  # Metastatic malignant neoplasm to skin of lip (disorder)
        "94565008",  # Metastatic malignant neoplasm to skin of lower limb (disorder)
        "94566009",  # Metastatic malignant neoplasm to skin of neck (disorder)
        "94567000",  # Metastatic malignant neoplasm to skin of nose (disorder)
        "94568005",  # Metastatic malignant neoplasm to skin of perineum (disorder)
        "94569002",  # Metastatic malignant neoplasm to skin of popliteal area (disorder)
        "94570001",  # Metastatic malignant neoplasm to skin of scalp (disorder)
        "94571002",  # Metastatic malignant neoplasm to skin of shoulder (disorder)
        "94572009",  # Metastatic malignant neoplasm to skin of temporal region (disorder)
        "94573004",  # Metastatic malignant neoplasm to skin of thigh (disorder)
        "94574005",  # Metastatic malignant neoplasm to skin of toe (disorder)
        "94575006",  # Metastatic malignant neoplasm to skin of trunk (disorder)
        "94576007",  # Metastatic malignant neoplasm to skin of umbilicus (disorder)
        "94577003",  # Metastatic malignant neoplasm to skin of upper limb (disorder)
        "94578008",  # Metastatic malignant neoplasm to skin of wrist (disorder)
        "94579000",  # Metastatic malignant neoplasm to skin (disorder)
        "94580002",  # Metastatic malignant neoplasm to small intestine (disorder)
        "94581003",  # Metastatic malignant neoplasm to soft palate (disorder)
        "94582005",  # Metastatic malignant neoplasm to soft tissues of abdomen (disorder)
        "94583000",  # Metastatic malignant neoplasm to soft tissues of axilla (disorder)
        "94584006",  # Metastatic malignant neoplasm to soft tissues of buttock (disorder)
        "94585007",  # Metastatic malignant neoplasm to soft tissues of face (disorder)
        "94586008",  # Metastatic malignant neoplasm to soft tissues of head (disorder)
        "94587004",  # Metastatic malignant neoplasm to soft tissues of hip (disorder)
        "94588009",  # Metastatic malignant neoplasm to soft tissues of inguinal region (disorder)
        "94589001",  # Metastatic malignant neoplasm to soft tissues of lower limb (disorder)
        "94590005",  # Metastatic malignant neoplasm to soft tissues of neck (disorder)
        "94591009",  # Metastatic malignant neoplasm to soft tissues of pelvis (disorder)
        "94592002",  # Metastatic malignant neoplasm to soft tissues of perineum (disorder)
        "94593007",  # Metastatic malignant neoplasm to soft tissues of shoulder (disorder)
        "94594001",  # Metastatic malignant neoplasm to soft tissues of thorax (disorder)
        "94595000",  # Metastatic malignant neoplasm to soft tissues of trunk (disorder)
        "94596004",  # Metastatic malignant neoplasm to soft tissues of upper limb (disorder)
        "94597008",  # Metastatic malignant neoplasm to spermatic cord (disorder)
        "94598003",  # Metastatic malignant neoplasm to sphenoid bone (disorder)
        "94599006",  # Metastatic malignant neoplasm to sphenoidal sinus (disorder)
        "94600009",  # Metastatic malignant neoplasm to spinal cord (disorder)
        "94601008",  # Metastatic malignant neoplasm to spinal meninges (disorder)
        "94602001",  # Metastatic malignant neoplasm to vertebral column (disorder)
        "94603006",  # Metastatic malignant neoplasm to spleen (disorder)
        "94604000",  # Metastatic malignant neoplasm to splenic flexure of colon (disorder)
        "94605004",  # Metastatic malignant neoplasm to sternum (disorder)
        "94606003",  # Metastatic malignant neoplasm to stomach (disorder)
        "94607007",  # Metastatic malignant neoplasm to subglottis (disorder)
        "94608002",  # Metastatic malignant neoplasm to sublingual gland (disorder)
        "94609005",  # Metastatic malignant neoplasm to submandibular lymph nodes (disorder)
        "94610000",  # Metastatic malignant neoplasm to submaxillary gland (disorder)
        "94611001",  # Metastatic malignant neoplasm to submental lymph nodes (disorder)
        "94612008",  # Metastatic malignant neoplasm to superficial inguinal lymph nodes (disorder)
        "94613003",  # Metastatic malignant neoplasm to superior wall of nasopharynx (disorder)
        "94614009",  # Metastatic malignant neoplasm to supraclavicular lymph nodes (disorder)
        "94615005",  # Metastatic malignant neoplasm to supraclavicular region (disorder)
        "94616006",  # Metastatic malignant neoplasm to supraglottis (disorder)
        "94617002",  # Metastatic malignant neoplasm to sweat gland (disorder)
        "94618007",  # Metastatic malignant neoplasm to tail of pancreas (disorder)
        "94619004",  # Metastatic malignant neoplasm to talus (disorder)
        "94620005",  # Metastatic malignant neoplasm to tarsal bone (disorder)
        "94621009",  # Metastatic malignant neoplasm to temporal bone (disorder)
        "94622002",  # Metastatic malignant neoplasm to temporal lobe (disorder)
        "94623007",  # Metastatic malignant neoplasm to testis (disorder)
        "94624001",  # Metastatic malignant neoplasm to the mesentery (disorder)
        "94625000",  # Metastatic malignant neoplasm to the mesocolon (disorder)
        "94626004",  # Metastatic malignant neoplasm to omentum (disorder)
        "94627008",  # Metastatic malignant neoplasm to peritoneum (disorder)
        "94628003",  # Metastatic malignant neoplasm to retroperitoneum (disorder)
        "94629006",  # Metastatic malignant neoplasm to thigh (disorder)
        "94630001",  # Metastatic malignant neoplasm to third cuneiform bone of foot (disorder)
        "94631002",  # Metastatic malignant neoplasm to thoracic esophagus (disorder)
        "94632009",  # Metastatic malignant neoplasm to thymus (disorder)
        "94633004",  # Metastatic malignant neoplasm to thyroglossal duct (disorder)
        "94634005",  # Metastatic malignant neoplasm to thyroid gland (disorder)
        "94635006",  # Metastatic malignant neoplasm to tibia (disorder)
        "94636007",  # Metastatic malignant neoplasm to tibial lymph nodes (disorder)
        "94637003",  # Metastatic malignant neoplasm to tip and lateral border of tongue (disorder)
        "94638008",  # Metastatic malignant neoplasm to tongue (disorder)
        "94639000",  # Metastatic malignant neoplasm to tonsillar fossa (disorder)
        "94640003",  # Metastatic malignant neoplasm to tonsillar pillar (disorder)
        "94641004",  # Metastatic malignant neoplasm to trachea (disorder)
        "94642006",  # Metastatic malignant neoplasm to tracheobronchial lymph nodes (disorder)
        "94643001",  # Metastatic malignant neoplasm to transverse colon (disorder)
        "94644007",  # Metastatic malignant neoplasm to trapezium (disorder)
        "94645008",  # Metastatic malignant neoplasm to trapezoid bone (disorder)
        "94646009",  # Metastatic malignant neoplasm to trigeminal nerve (disorder)
        "94647000",  # Metastatic malignant neoplasm to trigone of urinary bladder (disorder)
        "94648005",  # Metastatic malignant neoplasm to trochlear nerve (disorder)
        "94649002",  # Metastatic malignant neoplasm to trunk (disorder)
        "94650002",  # Metastatic malignant neoplasm to ulna (disorder)
        "94651003",  # Metastatic malignant neoplasm to undescended testis (disorder)
        "94652005",  # Metastatic malignant neoplasm to upper gum (disorder)
        "94654006",  # Metastatic malignant neoplasm to upper limb (disorder)
        "94656008",  # Metastatic malignant neoplasm to upper respiratory tract (disorder)
        "94657004",  # Metastatic malignant neoplasm to upper third of esophagus (disorder)
        "94658009",  # Metastatic malignant neoplasm to urachus (disorder)
        "94659001",  # Metastatic malignant neoplasm to ureter (disorder)
        "94660006",  # Metastatic malignant neoplasm to ureteric orifice of urinary bladder (disorder)
        "94661005",  # Metastatic malignant neoplasm to urethra (disorder)
        "94662003",  # Metastatic malignant neoplasm to urinary bladder neck (disorder)
        "94663008",  # Metastatic malignant neoplasm to urinary system (disorder)
        "94664002",  # Metastatic malignant neoplasm to uterine adnexa (disorder)
        "94665001",  # Metastatic malignant neoplasm to uterus (disorder)
        "94666000",  # Metastatic malignant neoplasm to uveal tract (disorder)
        "94667009",  # Metastatic malignant neoplasm to uvula (disorder)
        "94668004",  # Metastatic malignant neoplasm to vagina (disorder)
        "94669007",  # Metastatic malignant neoplasm to vagus nerve (disorder)
        "94670008",  # Metastatic malignant neoplasm to vallecula (disorder)
        "94671007",  # Metastatic malignant neoplasm to vas deferens (disorder)
        "94672000",  # Metastatic malignant neoplasm to ventral surface of tongue (disorder)
        "94673005",  # Metastatic malignant neoplasm to vermilion border of lip (disorder)
        "94674004",  # Metastatic malignant neoplasm to vermilion border of lower lip (disorder)
        "94675003",  # Metastatic malignant neoplasm to vermilion border of upper lip (disorder)
        "94676002",  # Metastatic malignant neoplasm to vestibule of mouth (disorder)
        "94677006",  # Metastatic malignant neoplasm to vestibule of nose (disorder)
        "94678001",  # Metastatic malignant neoplasm to visceral pleura (disorder)
        "94679009",  # Metastatic malignant neoplasm to vocal cord (disorder)
        "94680007",  # Metastatic malignant neoplasm to vomer (disorder)
        "94681006",  # Metastatic malignant neoplasm to vulva (disorder)
        "94682004",  # Metastatic malignant neoplasm to Waldeyer's ring (disorder)
        "94683009",  # Metastatic malignant neoplasm to zygomatic bone (disorder)
        "94686001",  # Mixed cell type lymphosarcoma of intra-abdominal lymph nodes (disorder)
        "94687005",  # Mixed cell type lymphosarcoma of intrapelvic lymph nodes (disorder)
        "94688000",  # Mixed cell type lymphosarcoma of intrathoracic lymph nodes (disorder)
        "94690004",  # Mixed cell type lymphosarcoma of lymph nodes of head, face, and neck (disorder)
        "94704006",  # Multiple myeloma in remission (disorder)
        "94707004",  # Mycosis fungoides of intra-abdominal lymph nodes (disorder)
        "94708009",  # Mycosis fungoides of intrapelvic lymph nodes (disorder)
        "94709001",  # Mycosis fungoides of intrathoracic lymph nodes (disorder)
        "94714002",  # Mycosis fungoides of spleen (disorder)
        "94716000",  # Myeloid leukemia in remission (disorder)
        "94718004",  # Myeloid sarcoma in remission (disorder)
        "94719007",  # Myeloid sarcoma, disease (disorder)
        "95186006",  # Nodular lymphoma of intra-abdominal lymph nodes (disorder)
        "95187002",  # Nodular lymphoma of intrapelvic lymph nodes (disorder)
        "95188007",  # Nodular lymphoma of intrathoracic lymph nodes (disorder)
        "95192000",  # Nodular lymphoma of lymph nodes of multiple sites (disorder)
        "95193005",  # Nodular lymphoma of spleen (disorder)
        "95209008",  # Plasma cell leukemia in remission (disorder)
        "95210003",  # Plasma cell leukemia, disease (disorder)
        "95214007",  # Primary malignant neoplasm of liver (disorder)
        "95224004",  # Reticulosarcoma of intra-abdominal lymph nodes (disorder)
        "95225003",  # Reticulosarcoma of intrapelvic lymph nodes (disorder)
        "95226002",  # Reticulosarcoma of intrathoracic lymph nodes (disorder)
        "95230004",  # Reticulosarcoma of lymph nodes of multiple sites (disorder)
        "95231000",  # Reticulosarcoma of spleen (disorder)
        "95263006",  # Sezary's disease of spleen (disorder)
        "9541000119105",  # Primary adenocarcinoma of gallbladder (disorder)
        "96281000119107",  # Overlapping malignant neoplasm of colon and rectum (disorder)
        "96901000119105",  # Prostate cancer metastatic to eye (disorder)
        "97051000119105",  # Malignant melanoma metastatic to pancreas (disorder)
        "98981000119103",  # Primary malignant neoplasm of ileocecal valve (disorder)
        "99101000119101",  # Primary adenocarcinoma of fallopian tube (disorder)
        "99111000119103",  # Primary adenocarcinoma of intestinal tract (disorder)
        "99121000119105",  # Primary adenocarcinoma of vagina (disorder)
        "99131000119108",  # Astrocytoma of cerebrum (disorder)
    }

class CompetingConditionsForRespiratoryConditions(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of competing conditions that may warrant treatment with an antibiotic medication for respiratory conditions.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of acute bacterial infections that may warrant treatment with an antibiotic medication.

    **Exclusion Criteria:** Excludes concepts that represent an acute diagnosis that does not warrant treatment with an antibiotic medication.
    """

    VALUE_SET_NAME = "Competing Conditions for Respiratory Conditions"
    OID = "2.16.840.1.113883.3.464.1003.102.12.1017"
    DEFINITION_VERSION = "20210220"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "A000",  # Cholera due to Vibrio cholerae 01, biovar cholerae
        "A001",  # Cholera due to Vibrio cholerae 01, biovar eltor
        "A009",  # Cholera, unspecified
        "A0100",  # Typhoid fever, unspecified
        "A0101",  # Typhoid meningitis
        "A0102",  # Typhoid fever with heart involvement
        "A0103",  # Typhoid pneumonia
        "A0104",  # Typhoid arthritis
        "A0105",  # Typhoid osteomyelitis
        "A0109",  # Typhoid fever with other complications
        "A011",  # Paratyphoid fever A
        "A012",  # Paratyphoid fever B
        "A013",  # Paratyphoid fever C
        "A014",  # Paratyphoid fever, unspecified
        "A020",  # Salmonella enteritis
        "A021",  # Salmonella sepsis
        "A0220",  # Localized salmonella infection, unspecified
        "A0221",  # Salmonella meningitis
        "A0222",  # Salmonella pneumonia
        "A0223",  # Salmonella arthritis
        "A0224",  # Salmonella osteomyelitis
        "A0225",  # Salmonella pyelonephritis
        "A0229",  # Salmonella with other localized infection
        "A028",  # Other specified salmonella infections
        "A029",  # Salmonella infection, unspecified
        "A030",  # Shigellosis due to Shigella dysenteriae
        "A031",  # Shigellosis due to Shigella flexneri
        "A032",  # Shigellosis due to Shigella boydii
        "A033",  # Shigellosis due to Shigella sonnei
        "A038",  # Other shigellosis
        "A039",  # Shigellosis, unspecified
        "A040",  # Enteropathogenic Escherichia coli infection
        "A041",  # Enterotoxigenic Escherichia coli infection
        "A042",  # Enteroinvasive Escherichia coli infection
        "A043",  # Enterohemorrhagic Escherichia coli infection
        "A044",  # Other intestinal Escherichia coli infections
        "A045",  # Campylobacter enteritis
        "A046",  # Enteritis due to Yersinia enterocolitica
        "A0471",  # Enterocolitis due to Clostridium difficile, recurrent
        "A0472",  # Enterocolitis due to Clostridium difficile, not specified as recurrent
        "A048",  # Other specified bacterial intestinal infections
        "A049",  # Bacterial intestinal infection, unspecified
        "A050",  # Foodborne staphylococcal intoxication
        "A051",  # Botulism food poisoning
        "A052",  # Foodborne Clostridium perfringens [Clostridium welchii] intoxication
        "A053",  # Foodborne Vibrio parahaemolyticus intoxication
        "A054",  # Foodborne Bacillus cereus intoxication
        "A055",  # Foodborne Vibrio vulnificus intoxication
        "A058",  # Other specified bacterial foodborne intoxications
        "A059",  # Bacterial foodborne intoxication, unspecified
        "A060",  # Acute amebic dysentery
        "A061",  # Chronic intestinal amebiasis
        "A062",  # Amebic nondysenteric colitis
        "A063",  # Ameboma of intestine
        "A064",  # Amebic liver abscess
        "A065",  # Amebic lung abscess
        "A066",  # Amebic brain abscess
        "A067",  # Cutaneous amebiasis
        "A0681",  # Amebic cystitis
        "A0682",  # Other amebic genitourinary infections
        "A0689",  # Other amebic infections
        "A069",  # Amebiasis, unspecified
        "A070",  # Balantidiasis
        "A071",  # Giardiasis [lambliasis]
        "A072",  # Cryptosporidiosis
        "A073",  # Isosporiasis
        "A074",  # Cyclosporiasis
        "A078",  # Other specified protozoal intestinal diseases
        "A079",  # Protozoal intestinal disease, unspecified
        "A080",  # Rotaviral enteritis
        "A0811",  # Acute gastroenteropathy due to Norwalk agent
        "A0819",  # Acute gastroenteropathy due to other small round viruses
        "A082",  # Adenoviral enteritis
        "A0831",  # Calicivirus enteritis
        "A0832",  # Astrovirus enteritis
        "A0839",  # Other viral enteritis
        "A084",  # Viral intestinal infection, unspecified
        "A088",  # Other specified intestinal infections
        "A09",  # Infectious gastroenteritis and colitis, unspecified
        "A3700",  # Whooping cough due to Bordetella pertussis without pneumonia
        "A3701",  # Whooping cough due to Bordetella pertussis with pneumonia
        "A3710",  # Whooping cough due to Bordetella parapertussis without pneumonia
        "A3711",  # Whooping cough due to Bordetella parapertussis with pneumonia
        "A3780",  # Whooping cough due to other Bordetella species without pneumonia
        "A3781",  # Whooping cough due to other Bordetella species with pneumonia
        "A3790",  # Whooping cough, unspecified species without pneumonia
        "A3791",  # Whooping cough, unspecified species with pneumonia
        "A440",  # Systemic bartonellosis
        "A441",  # Cutaneous and mucocutaneous bartonellosis
        "A448",  # Other forms of bartonellosis
        "A449",  # Bartonellosis, unspecified
        "A499",  # Bacterial infection, unspecified
        "A5001",  # Early congenital syphilitic oculopathy
        "A5002",  # Early congenital syphilitic osteochondropathy
        "A5003",  # Early congenital syphilitic pharyngitis
        "A5004",  # Early congenital syphilitic pneumonia
        "A5005",  # Early congenital syphilitic rhinitis
        "A5006",  # Early cutaneous congenital syphilis
        "A5007",  # Early mucocutaneous congenital syphilis
        "A5008",  # Early visceral congenital syphilis
        "A5009",  # Other early congenital syphilis, symptomatic
        "A501",  # Early congenital syphilis, latent
        "A502",  # Early congenital syphilis, unspecified
        "A5030",  # Late congenital syphilitic oculopathy, unspecified
        "A5031",  # Late congenital syphilitic interstitial keratitis
        "A5032",  # Late congenital syphilitic chorioretinitis
        "A5039",  # Other late congenital syphilitic oculopathy
        "A5040",  # Late congenital neurosyphilis, unspecified
        "A5041",  # Late congenital syphilitic meningitis
        "A5042",  # Late congenital syphilitic encephalitis
        "A5043",  # Late congenital syphilitic polyneuropathy
        "A5044",  # Late congenital syphilitic optic nerve atrophy
        "A5045",  # Juvenile general paresis
        "A5049",  # Other late congenital neurosyphilis
        "A5051",  # Clutton's joints
        "A5052",  # Hutchinson's teeth
        "A5053",  # Hutchinson's triad
        "A5054",  # Late congenital cardiovascular syphilis
        "A5055",  # Late congenital syphilitic arthropathy
        "A5056",  # Late congenital syphilitic osteochondropathy
        "A5057",  # Syphilitic saddle nose
        "A5059",  # Other late congenital syphilis, symptomatic
        "A506",  # Late congenital syphilis, latent
        "A507",  # Late congenital syphilis, unspecified
        "A509",  # Congenital syphilis, unspecified
        "A510",  # Primary genital syphilis
        "A511",  # Primary anal syphilis
        "A512",  # Primary syphilis of other sites
        "A5131",  # Condyloma latum
        "A5132",  # Syphilitic alopecia
        "A5139",  # Other secondary syphilis of skin
        "A5141",  # Secondary syphilitic meningitis
        "A5142",  # Secondary syphilitic female pelvic disease
        "A5143",  # Secondary syphilitic oculopathy
        "A5144",  # Secondary syphilitic nephritis
        "A5145",  # Secondary syphilitic hepatitis
        "A5146",  # Secondary syphilitic osteopathy
        "A5149",  # Other secondary syphilitic conditions
        "A515",  # Early syphilis, latent
        "A519",  # Early syphilis, unspecified
        "A5200",  # Cardiovascular syphilis, unspecified
        "A5201",  # Syphilitic aneurysm of aorta
        "A5202",  # Syphilitic aortitis
        "A5203",  # Syphilitic endocarditis
        "A5204",  # Syphilitic cerebral arteritis
        "A5205",  # Other cerebrovascular syphilis
        "A5206",  # Other syphilitic heart involvement
        "A5209",  # Other cardiovascular syphilis
        "A5210",  # Symptomatic neurosyphilis, unspecified
        "A5211",  # Tabes dorsalis
        "A5212",  # Other cerebrospinal syphilis
        "A5213",  # Late syphilitic meningitis
        "A5214",  # Late syphilitic encephalitis
        "A5215",  # Late syphilitic neuropathy
        "A5216",  # Charcot's arthropathy (tabetic)
        "A5217",  # General paresis
        "A5219",  # Other symptomatic neurosyphilis
        "A522",  # Asymptomatic neurosyphilis
        "A523",  # Neurosyphilis, unspecified
        "A5271",  # Late syphilitic oculopathy
        "A5272",  # Syphilis of lung and bronchus
        "A5273",  # Symptomatic late syphilis of other respiratory organs
        "A5274",  # Syphilis of liver and other viscera
        "A5275",  # Syphilis of kidney and ureter
        "A5276",  # Other genitourinary symptomatic late syphilis
        "A5277",  # Syphilis of bone and joint
        "A5278",  # Syphilis of other musculoskeletal tissue
        "A5279",  # Other symptomatic late syphilis
        "A528",  # Late syphilis, latent
        "A529",  # Late syphilis, unspecified
        "A530",  # Latent syphilis, unspecified as early or late
        "A539",  # Syphilis, unspecified
        "A5400",  # Gonococcal infection of lower genitourinary tract, unspecified
        "A5401",  # Gonococcal cystitis and urethritis, unspecified
        "A5402",  # Gonococcal vulvovaginitis, unspecified
        "A5403",  # Gonococcal cervicitis, unspecified
        "A5409",  # Other gonococcal infection of lower genitourinary tract
        "A541",  # Gonococcal infection of lower genitourinary tract with periurethral and accessory gland abscess
        "A5421",  # Gonococcal infection of kidney and ureter
        "A5422",  # Gonococcal prostatitis
        "A5423",  # Gonococcal infection of other male genital organs
        "A5424",  # Gonococcal female pelvic inflammatory disease
        "A5429",  # Other gonococcal genitourinary infections
        "A5430",  # Gonococcal infection of eye, unspecified
        "A5431",  # Gonococcal conjunctivitis
        "A5432",  # Gonococcal iridocyclitis
        "A5433",  # Gonococcal keratitis
        "A5439",  # Other gonococcal eye infection
        "A5440",  # Gonococcal infection of musculoskeletal system, unspecified
        "A5441",  # Gonococcal spondylopathy
        "A5442",  # Gonococcal arthritis
        "A5443",  # Gonococcal osteomyelitis
        "A5449",  # Gonococcal infection of other musculoskeletal tissue
        "A545",  # Gonococcal pharyngitis
        "A546",  # Gonococcal infection of anus and rectum
        "A5481",  # Gonococcal meningitis
        "A5482",  # Gonococcal brain abscess
        "A5483",  # Gonococcal heart infection
        "A5484",  # Gonococcal pneumonia
        "A5485",  # Gonococcal peritonitis
        "A5486",  # Gonococcal sepsis
        "A5489",  # Other gonococcal infections
        "A549",  # Gonococcal infection, unspecified
        "A55",  # Chlamydial lymphogranuloma (venereum)
        "A5600",  # Chlamydial infection of lower genitourinary tract, unspecified
        "A5601",  # Chlamydial cystitis and urethritis
        "A5602",  # Chlamydial vulvovaginitis
        "A5609",  # Other chlamydial infection of lower genitourinary tract
        "A5611",  # Chlamydial female pelvic inflammatory disease
        "A5619",  # Other chlamydial genitourinary infection
        "A562",  # Chlamydial infection of genitourinary tract, unspecified
        "A563",  # Chlamydial infection of anus and rectum
        "A564",  # Chlamydial infection of pharynx
        "A568",  # Sexually transmitted chlamydial infection of other sites
        "A57",  # Chancroid
        "A58",  # Granuloma inguinale
        "A5900",  # Urogenital trichomoniasis, unspecified
        "A5901",  # Trichomonal vulvovaginitis
        "A5902",  # Trichomonal prostatitis
        "A5903",  # Trichomonal cystitis and urethritis
        "A5909",  # Other urogenital trichomoniasis
        "A598",  # Trichomoniasis of other sites
        "A599",  # Trichomoniasis, unspecified
        "A630",  # Anogenital (venereal) warts
        "A638",  # Other specified predominantly sexually transmitted diseases
        "A64",  # Unspecified sexually transmitted disease
        "A690",  # Necrotizing ulcerative stomatitis
        "A691",  # Other Vincent's infections
        "A6920",  # Lyme disease, unspecified
        "A6921",  # Meningitis due to Lyme disease
        "A6922",  # Other neurologic disorders in Lyme disease
        "A6923",  # Arthritis due to Lyme disease
        "A6929",  # Other conditions associated with Lyme disease
        "A698",  # Other specified spirochetal infections
        "A699",  # Spirochetal infection, unspecified
        "B6000",  # Babesiosis, unspecified
        "B6001",  # Babesiosis due to Babesia microti
        "B6002",  # Babesiosis due to Babesia duncani
        "B6003",  # Babesiosis due to Babesia divergens
        "B6009",  # Other babesiosis
        "B6010",  # Acanthamebiasis, unspecified
        "B6011",  # Meningoencephalitis due to Acanthamoeba (culbertsoni)
        "B6012",  # Conjunctivitis due to Acanthamoeba
        "B6013",  # Keratoconjunctivitis due to Acanthamoeba
        "B6019",  # Other acanthamebic disease
        "B602",  # Naegleriasis
        "B608",  # Other specified protozoal diseases
        "B64",  # Unspecified protozoal disease
        "B781",  # Cutaneous strongyloidiasis
        "B9689",  # Other specified bacterial agents as the cause of diseases classified elsewhere
        "E832",  # Disorders of zinc metabolism
        "H66001",  # Acute suppurative otitis media without spontaneous rupture of ear drum, right ear
        "H66002",  # Acute suppurative otitis media without spontaneous rupture of ear drum, left ear
        "H66003",  # Acute suppurative otitis media without spontaneous rupture of ear drum, bilateral
        "H66004",  # Acute suppurative otitis media without spontaneous rupture of ear drum, recurrent, right ear
        "H66005",  # Acute suppurative otitis media without spontaneous rupture of ear drum, recurrent, left ear
        "H66006",  # Acute suppurative otitis media without spontaneous rupture of ear drum, recurrent, bilateral
        "H66007",  # Acute suppurative otitis media without spontaneous rupture of ear drum, recurrent, unspecified ear
        "H66009",  # Acute suppurative otitis media without spontaneous rupture of ear drum, unspecified ear
        "H66011",  # Acute suppurative otitis media with spontaneous rupture of ear drum, right ear
        "H66012",  # Acute suppurative otitis media with spontaneous rupture of ear drum, left ear
        "H66013",  # Acute suppurative otitis media with spontaneous rupture of ear drum, bilateral
        "H66014",  # Acute suppurative otitis media with spontaneous rupture of ear drum, recurrent, right ear
        "H66015",  # Acute suppurative otitis media with spontaneous rupture of ear drum, recurrent, left ear
        "H66016",  # Acute suppurative otitis media with spontaneous rupture of ear drum, recurrent, bilateral
        "H66017",  # Acute suppurative otitis media with spontaneous rupture of ear drum, recurrent, unspecified ear
        "H66019",  # Acute suppurative otitis media with spontaneous rupture of ear drum, unspecified ear
        "H6610",  # Chronic tubotympanic suppurative otitis media, unspecified
        "H6611",  # Chronic tubotympanic suppurative otitis media, right ear
        "H6612",  # Chronic tubotympanic suppurative otitis media, left ear
        "H6613",  # Chronic tubotympanic suppurative otitis media, bilateral
        "H6620",  # Chronic atticoantral suppurative otitis media, unspecified ear
        "H6621",  # Chronic atticoantral suppurative otitis media, right ear
        "H6622",  # Chronic atticoantral suppurative otitis media, left ear
        "H6623",  # Chronic atticoantral suppurative otitis media, bilateral
        "H663X1",  # Other chronic suppurative otitis media, right ear
        "H663X2",  # Other chronic suppurative otitis media, left ear
        "H663X3",  # Other chronic suppurative otitis media, bilateral
        "H663X9",  # Other chronic suppurative otitis media, unspecified ear
        "H6640",  # Suppurative otitis media, unspecified, unspecified ear
        "H6641",  # Suppurative otitis media, unspecified, right ear
        "H6642",  # Suppurative otitis media, unspecified, left ear
        "H6643",  # Suppurative otitis media, unspecified, bilateral
        "H6690",  # Otitis media, unspecified, unspecified ear
        "H6691",  # Otitis media, unspecified, right ear
        "H6692",  # Otitis media, unspecified, left ear
        "H6693",  # Otitis media, unspecified, bilateral
        "H671",  # Otitis media in diseases classified elsewhere, right ear
        "H672",  # Otitis media in diseases classified elsewhere, left ear
        "H673",  # Otitis media in diseases classified elsewhere, bilateral
        "H679",  # Otitis media in diseases classified elsewhere, unspecified ear
        "H70001",  # Acute mastoiditis without complications, right ear
        "H70002",  # Acute mastoiditis without complications, left ear
        "H70003",  # Acute mastoiditis without complications, bilateral
        "H70009",  # Acute mastoiditis without complications, unspecified ear
        "H70011",  # Subperiosteal abscess of mastoid, right ear
        "H70012",  # Subperiosteal abscess of mastoid, left ear
        "H70013",  # Subperiosteal abscess of mastoid, bilateral
        "H70019",  # Subperiosteal abscess of mastoid, unspecified ear
        "H70091",  # Acute mastoiditis with other complications, right ear
        "H70092",  # Acute mastoiditis with other complications, left ear
        "H70093",  # Acute mastoiditis with other complications, bilateral
        "H70099",  # Acute mastoiditis with other complications, unspecified ear
        "H7010",  # Chronic mastoiditis, unspecified ear
        "H7011",  # Chronic mastoiditis, right ear
        "H7012",  # Chronic mastoiditis, left ear
        "H7013",  # Chronic mastoiditis, bilateral
        "H70201",  # Unspecified petrositis, right ear
        "H70202",  # Unspecified petrositis, left ear
        "H70203",  # Unspecified petrositis, bilateral
        "H70209",  # Unspecified petrositis, unspecified ear
        "H70211",  # Acute petrositis, right ear
        "H70212",  # Acute petrositis, left ear
        "H70213",  # Acute petrositis, bilateral
        "H70219",  # Acute petrositis, unspecified ear
        "H70221",  # Chronic petrositis, right ear
        "H70222",  # Chronic petrositis, left ear
        "H70223",  # Chronic petrositis, bilateral
        "H70229",  # Chronic petrositis, unspecified ear
        "H70811",  # Postauricular fistula, right ear
        "H70812",  # Postauricular fistula, left ear
        "H70813",  # Postauricular fistula, bilateral
        "H70819",  # Postauricular fistula, unspecified ear
        "H70891",  # Other mastoiditis and related conditions, right ear
        "H70892",  # Other mastoiditis and related conditions, left ear
        "H70893",  # Other mastoiditis and related conditions, bilateral
        "H70899",  # Other mastoiditis and related conditions, unspecified ear
        "H7090",  # Unspecified mastoiditis, unspecified ear
        "H7091",  # Unspecified mastoiditis, right ear
        "H7092",  # Unspecified mastoiditis, left ear
        "H7093",  # Unspecified mastoiditis, bilateral
        "H9500",  # Recurrent cholesteatoma of postmastoidectomy cavity, unspecified ear
        "H9501",  # Recurrent cholesteatoma of postmastoidectomy cavity, right ear
        "H9502",  # Recurrent cholesteatoma of postmastoidectomy cavity, left ear
        "H9503",  # Recurrent cholesteatoma of postmastoidectomy cavity, bilateral ears
        "H95111",  # Chronic inflammation of postmastoidectomy cavity, right ear
        "H95112",  # Chronic inflammation of postmastoidectomy cavity, left ear
        "H95113",  # Chronic inflammation of postmastoidectomy cavity, bilateral ears
        "H95119",  # Chronic inflammation of postmastoidectomy cavity, unspecified ear
        "H95121",  # Granulation of postmastoidectomy cavity, right ear
        "H95122",  # Granulation of postmastoidectomy cavity, left ear
        "H95123",  # Granulation of postmastoidectomy cavity, bilateral ears
        "H95129",  # Granulation of postmastoidectomy cavity, unspecified ear
        "H95131",  # Mucosal cyst of postmastoidectomy cavity, right ear
        "H95132",  # Mucosal cyst of postmastoidectomy cavity, left ear
        "H95133",  # Mucosal cyst of postmastoidectomy cavity, bilateral ears
        "H95139",  # Mucosal cyst of postmastoidectomy cavity, unspecified ear
        "H95191",  # Other disorders following mastoidectomy, right ear
        "H95192",  # Other disorders following mastoidectomy, left ear
        "H95193",  # Other disorders following mastoidectomy, bilateral ears
        "H95199",  # Other disorders following mastoidectomy, unspecified ear
        "H9521",  # Intraoperative hemorrhage and hematoma of ear and mastoid process complicating a procedure on the ear and mastoid process
        "H9522",  # Intraoperative hemorrhage and hematoma of ear and mastoid process complicating other procedure
        "H9531",  # Accidental puncture and laceration of the ear and mastoid process during a procedure on the ear and mastoid process
        "H9532",  # Accidental puncture and laceration of the ear and mastoid process during other procedure
        "H9541",  # Postprocedural hemorrhage of ear and mastoid process following a procedure on the ear and mastoid process
        "H9542",  # Postprocedural hemorrhage of ear and mastoid process following other procedure
        "H9551",  # Postprocedural hematoma of ear and mastoid process following a procedure on the ear and mastoid process
        "H9552",  # Postprocedural hematoma of ear and mastoid process following other procedure
        "H9553",  # Postprocedural seroma of ear and mastoid process following a procedure on the ear and mastoid process
        "H9554",  # Postprocedural seroma of ear and mastoid process following other procedure
        "H95811",  # Postprocedural stenosis of right external ear canal
        "H95812",  # Postprocedural stenosis of left external ear canal
        "H95813",  # Postprocedural stenosis of external ear canal, bilateral
        "H95819",  # Postprocedural stenosis of unspecified external ear canal
        "H9588",  # Other intraoperative complications and disorders of the ear and mastoid process, not elsewhere classified
        "H9589",  # Other postprocedural complications and disorders of the ear and mastoid process, not elsewhere classified
        "J0100",  # Acute maxillary sinusitis, unspecified
        "J0101",  # Acute recurrent maxillary sinusitis
        "J0110",  # Acute frontal sinusitis, unspecified
        "J0111",  # Acute recurrent frontal sinusitis
        "J0120",  # Acute ethmoidal sinusitis, unspecified
        "J0121",  # Acute recurrent ethmoidal sinusitis
        "J0130",  # Acute sphenoidal sinusitis, unspecified
        "J0131",  # Acute recurrent sphenoidal sinusitis
        "J0140",  # Acute pansinusitis, unspecified
        "J0141",  # Acute recurrent pansinusitis
        "J0180",  # Other acute sinusitis
        "J0181",  # Other acute recurrent sinusitis
        "J0190",  # Acute sinusitis, unspecified
        "J0191",  # Acute recurrent sinusitis, unspecified
        "J0410",  # Acute tracheitis without obstruction
        "J0411",  # Acute tracheitis with obstruction
        "J042",  # Acute laryngotracheitis
        "J050",  # Acute obstructive laryngitis [croup]
        "J0510",  # Acute epiglottitis without obstruction
        "J0511",  # Acute epiglottitis with obstruction
        "J13",  # Pneumonia due to Streptococcus pneumoniae
        "J14",  # Pneumonia due to Hemophilus influenzae
        "J150",  # Pneumonia due to Klebsiella pneumoniae
        "J151",  # Pneumonia due to Pseudomonas
        "J1520",  # Pneumonia due to staphylococcus, unspecified
        "J15211",  # Pneumonia due to Methicillin susceptible Staphylococcus aureus
        "J15212",  # Pneumonia due to Methicillin resistant Staphylococcus aureus
        "J1529",  # Pneumonia due to other staphylococcus
        "J153",  # Pneumonia due to streptococcus, group B
        "J154",  # Pneumonia due to other streptococci
        "J155",  # Pneumonia due to Escherichia coli
        "J156",  # Pneumonia due to other Gram-negative bacteria
        "J157",  # Pneumonia due to Mycoplasma pneumoniae
        "J158",  # Pneumonia due to other specified bacteria
        "J159",  # Unspecified bacterial pneumonia
        "J160",  # Chlamydial pneumonia
        "J168",  # Pneumonia due to other specified infectious organisms
        "J17",  # Pneumonia in diseases classified elsewhere
        "J180",  # Bronchopneumonia, unspecified organism
        "J181",  # Lobar pneumonia, unspecified organism
        "J188",  # Other pneumonia, unspecified organism
        "J189",  # Pneumonia, unspecified organism
        "J200",  # Acute bronchitis due to Mycoplasma pneumoniae
        "J201",  # Acute bronchitis due to Hemophilus influenzae
        "J202",  # Acute bronchitis due to streptococcus
        "J320",  # Chronic maxillary sinusitis
        "J321",  # Chronic frontal sinusitis
        "J322",  # Chronic ethmoidal sinusitis
        "J323",  # Chronic sphenoidal sinusitis
        "J324",  # Chronic pansinusitis
        "J328",  # Other chronic sinusitis
        "J329",  # Chronic sinusitis, unspecified
        "J3501",  # Chronic tonsillitis
        "J3502",  # Chronic adenoiditis
        "J3503",  # Chronic tonsillitis and adenoiditis
        "J351",  # Hypertrophy of tonsils
        "J352",  # Hypertrophy of adenoids
        "J353",  # Hypertrophy of tonsils with hypertrophy of adenoids
        "J358",  # Other chronic diseases of tonsils and adenoids
        "J359",  # Chronic disease of tonsils and adenoids, unspecified
        "J387",  # Other diseases of larynx
        "J390",  # Retropharyngeal and parapharyngeal abscess
        "J391",  # Other abscess of pharynx
        "J392",  # Other diseases of pharynx
        "J398",  # Other specified diseases of upper respiratory tract
        "J399",  # Disease of upper respiratory tract, unspecified
        "K0520",  # Aggressive periodontitis, unspecified
        "K05211",  # Aggressive periodontitis, localized, slight
        "K05212",  # Aggressive periodontitis, localized, moderate
        "K05213",  # Aggressive periodontitis, localized, severe
        "K05219",  # Aggressive periodontitis, localized, unspecified severity
        "K05221",  # Aggressive periodontitis, generalized, slight
        "K05222",  # Aggressive periodontitis, generalized, moderate
        "K05223",  # Aggressive periodontitis, generalized, severe
        "K05229",  # Aggressive periodontitis, generalized, unspecified severity
        "K122",  # Cellulitis and abscess of mouth
        "L0100",  # Impetigo, unspecified
        "L0101",  # Non-bullous impetigo
        "L0102",  # Bockhart's impetigo
        "L0103",  # Bullous impetigo
        "L0109",  # Other impetigo
        "L011",  # Impetiginization of other dermatoses
        "L03011",  # Cellulitis of right finger
        "L03012",  # Cellulitis of left finger
        "L03019",  # Cellulitis of unspecified finger
        "L03021",  # Acute lymphangitis of right finger
        "L03022",  # Acute lymphangitis of left finger
        "L03029",  # Acute lymphangitis of unspecified finger
        "L03031",  # Cellulitis of right toe
        "L03032",  # Cellulitis of left toe
        "L03039",  # Cellulitis of unspecified toe
        "L03041",  # Acute lymphangitis of right toe
        "L03042",  # Acute lymphangitis of left toe
        "L03049",  # Acute lymphangitis of unspecified toe
        "L03111",  # Cellulitis of right axilla
        "L03112",  # Cellulitis of left axilla
        "L03113",  # Cellulitis of right upper limb
        "L03114",  # Cellulitis of left upper limb
        "L03115",  # Cellulitis of right lower limb
        "L03116",  # Cellulitis of left lower limb
        "L03119",  # Cellulitis of unspecified part of limb
        "L03121",  # Acute lymphangitis of right axilla
        "L03122",  # Acute lymphangitis of left axilla
        "L03123",  # Acute lymphangitis of right upper limb
        "L03124",  # Acute lymphangitis of left upper limb
        "L03125",  # Acute lymphangitis of right lower limb
        "L03126",  # Acute lymphangitis of left lower limb
        "L03129",  # Acute lymphangitis of unspecified part of limb
        "L03211",  # Cellulitis of face
        "L03212",  # Acute lymphangitis of face
        "L03213",  # Periorbital cellulitis
        "L03221",  # Cellulitis of neck
        "L03222",  # Acute lymphangitis of neck
        "L03311",  # Cellulitis of abdominal wall
        "L03312",  # Cellulitis of back [any part except buttock]
        "L03313",  # Cellulitis of chest wall
        "L03314",  # Cellulitis of groin
        "L03315",  # Cellulitis of perineum
        "L03316",  # Cellulitis of umbilicus
        "L03317",  # Cellulitis of buttock
        "L03319",  # Cellulitis of trunk, unspecified
        "L03321",  # Acute lymphangitis of abdominal wall
        "L03322",  # Acute lymphangitis of back [any part except buttock]
        "L03323",  # Acute lymphangitis of chest wall
        "L03324",  # Acute lymphangitis of groin
        "L03325",  # Acute lymphangitis of perineum
        "L03326",  # Acute lymphangitis of umbilicus
        "L03327",  # Acute lymphangitis of buttock
        "L03329",  # Acute lymphangitis of trunk, unspecified
        "L03811",  # Cellulitis of head [any part, except face]
        "L03818",  # Cellulitis of other sites
        "L03891",  # Acute lymphangitis of head [any part, except face]
        "L03898",  # Acute lymphangitis of other sites
        "L0390",  # Cellulitis, unspecified
        "L0391",  # Acute lymphangitis, unspecified
        "L040",  # Acute lymphadenitis of face, head and neck
        "L041",  # Acute lymphadenitis of trunk
        "L042",  # Acute lymphadenitis of upper limb
        "L043",  # Acute lymphadenitis of lower limb
        "L048",  # Acute lymphadenitis of other sites
        "L049",  # Acute lymphadenitis, unspecified
        "L081",  # Erythrasma
        "L0881",  # Pyoderma vegetans
        "L0882",  # Omphalitis not of newborn
        "L0889",  # Other specified local infections of the skin and subcutaneous tissue
        "L089",  # Local infection of the skin and subcutaneous tissue, unspecified
        "L88",  # Pyoderma gangrenosum
        "L928",  # Other granulomatous disorders of the skin and subcutaneous tissue
        "L980",  # Pyogenic granuloma
        "L983",  # Eosinophilic cellulitis [Wells]
        "M4620",  # Osteomyelitis of vertebra, site unspecified
        "M4621",  # Osteomyelitis of vertebra, occipito-atlanto-axial region
        "M4622",  # Osteomyelitis of vertebra, cervical region
        "M4623",  # Osteomyelitis of vertebra, cervicothoracic region
        "M4624",  # Osteomyelitis of vertebra, thoracic region
        "M4625",  # Osteomyelitis of vertebra, thoracolumbar region
        "M4626",  # Osteomyelitis of vertebra, lumbar region
        "M4627",  # Osteomyelitis of vertebra, lumbosacral region
        "M4628",  # Osteomyelitis of vertebra, sacral and sacrococcygeal region
        "M4630",  # Infection of intervertebral disc (pyogenic), site unspecified
        "M4631",  # Infection of intervertebral disc (pyogenic), occipito-atlanto-axial region
        "M4632",  # Infection of intervertebral disc (pyogenic), cervical region
        "M4633",  # Infection of intervertebral disc (pyogenic), cervicothoracic region
        "M4634",  # Infection of intervertebral disc (pyogenic), thoracic region
        "M4635",  # Infection of intervertebral disc (pyogenic), thoracolumbar region
        "M4636",  # Infection of intervertebral disc (pyogenic), lumbar region
        "M4637",  # Infection of intervertebral disc (pyogenic), lumbosacral region
        "M4638",  # Infection of intervertebral disc (pyogenic), sacral and sacrococcygeal region
        "M4639",  # Infection of intervertebral disc (pyogenic), multiple sites in spine
        "M8900",  # Algoneurodystrophy, unspecified site
        "M89011",  # Algoneurodystrophy, right shoulder
        "M89012",  # Algoneurodystrophy, left shoulder
        "M89019",  # Algoneurodystrophy, unspecified shoulder
        "M89021",  # Algoneurodystrophy, right upper arm
        "M89022",  # Algoneurodystrophy, left upper arm
        "M89029",  # Algoneurodystrophy, unspecified upper arm
        "M89031",  # Algoneurodystrophy, right forearm
        "M89032",  # Algoneurodystrophy, left forearm
        "M89039",  # Algoneurodystrophy, unspecified forearm
        "M89041",  # Algoneurodystrophy, right hand
        "M89042",  # Algoneurodystrophy, left hand
        "M89049",  # Algoneurodystrophy, unspecified hand
        "M89051",  # Algoneurodystrophy, right thigh
        "M89052",  # Algoneurodystrophy, left thigh
        "M89059",  # Algoneurodystrophy, unspecified thigh
        "M89061",  # Algoneurodystrophy, right lower leg
        "M89062",  # Algoneurodystrophy, left lower leg
        "M89069",  # Algoneurodystrophy, unspecified lower leg
        "M89071",  # Algoneurodystrophy, right ankle and foot
        "M89072",  # Algoneurodystrophy, left ankle and foot
        "M89079",  # Algoneurodystrophy, unspecified ankle and foot
        "M8908",  # Algoneurodystrophy, other site
        "M8909",  # Algoneurodystrophy, multiple sites
        "M89121",  # Complete physeal arrest, right proximal humerus
        "M89122",  # Complete physeal arrest, left proximal humerus
        "M89123",  # Partial physeal arrest, right proximal humerus
        "M89124",  # Partial physeal arrest, left proximal humerus
        "M89125",  # Complete physeal arrest, right distal humerus
        "M89126",  # Complete physeal arrest, left distal humerus
        "M89127",  # Partial physeal arrest, right distal humerus
        "M89128",  # Partial physeal arrest, left distal humerus
        "M89129",  # Physeal arrest, humerus, unspecified
        "M89131",  # Complete physeal arrest, right distal radius
        "M89132",  # Complete physeal arrest, left distal radius
        "M89133",  # Partial physeal arrest, right distal radius
        "M89134",  # Partial physeal arrest, left distal radius
        "M89138",  # Other physeal arrest of forearm
        "M89139",  # Physeal arrest, forearm, unspecified
        "M89151",  # Complete physeal arrest, right proximal femur
        "M89152",  # Complete physeal arrest, left proximal femur
        "M89153",  # Partial physeal arrest, right proximal femur
        "M89154",  # Partial physeal arrest, left proximal femur
        "M89155",  # Complete physeal arrest, right distal femur
        "M89156",  # Complete physeal arrest, left distal femur
        "M89157",  # Partial physeal arrest, right distal femur
        "M89158",  # Partial physeal arrest, left distal femur
        "M89159",  # Physeal arrest, femur, unspecified
        "M89160",  # Complete physeal arrest, right proximal tibia
        "M89161",  # Complete physeal arrest, left proximal tibia
        "M89162",  # Partial physeal arrest, right proximal tibia
        "M89163",  # Partial physeal arrest, left proximal tibia
        "M89164",  # Complete physeal arrest, right distal tibia
        "M89165",  # Complete physeal arrest, left distal tibia
        "M89166",  # Partial physeal arrest, right distal tibia
        "M89167",  # Partial physeal arrest, left distal tibia
        "M89168",  # Other physeal arrest of lower leg
        "M89169",  # Physeal arrest, lower leg, unspecified
        "M8918",  # Physeal arrest, other site
        "M8920",  # Other disorders of bone development and growth, unspecified site
        "M89211",  # Other disorders of bone development and growth, right shoulder
        "M89212",  # Other disorders of bone development and growth, left shoulder
        "M89219",  # Other disorders of bone development and growth, unspecified shoulder
        "M89221",  # Other disorders of bone development and growth, right humerus
        "M89222",  # Other disorders of bone development and growth, left humerus
        "M89229",  # Other disorders of bone development and growth, unspecified humerus
        "M89231",  # Other disorders of bone development and growth, right ulna
        "M89232",  # Other disorders of bone development and growth, left ulna
        "M89233",  # Other disorders of bone development and growth, right radius
        "M89234",  # Other disorders of bone development and growth, left radius
        "M89239",  # Other disorders of bone development and growth, unspecified ulna and radius
        "M89241",  # Other disorders of bone development and growth, right hand
        "M89242",  # Other disorders of bone development and growth, left hand
        "M89249",  # Other disorders of bone development and growth, unspecified hand
        "M89251",  # Other disorders of bone development and growth, right femur
        "M89252",  # Other disorders of bone development and growth, left femur
        "M89259",  # Other disorders of bone development and growth, unspecified femur
        "M89261",  # Other disorders of bone development and growth, right tibia
        "M89262",  # Other disorders of bone development and growth, left tibia
        "M89263",  # Other disorders of bone development and growth, right fibula
        "M89264",  # Other disorders of bone development and growth, left fibula
        "M89269",  # Other disorders of bone development and growth, unspecified lower leg
        "M89271",  # Other disorders of bone development and growth, right ankle and foot
        "M89272",  # Other disorders of bone development and growth, left ankle and foot
        "M89279",  # Other disorders of bone development and growth, unspecified ankle and foot
        "M8928",  # Other disorders of bone development and growth, other site
        "M8929",  # Other disorders of bone development and growth, multiple sites
        "M8940",  # Other hypertrophic osteoarthropathy, unspecified site
        "M89411",  # Other hypertrophic osteoarthropathy, right shoulder
        "M89412",  # Other hypertrophic osteoarthropathy, left shoulder
        "M89419",  # Other hypertrophic osteoarthropathy, unspecified shoulder
        "M89421",  # Other hypertrophic osteoarthropathy, right upper arm
        "M89422",  # Other hypertrophic osteoarthropathy, left upper arm
        "M89429",  # Other hypertrophic osteoarthropathy, unspecified upper arm
        "M89431",  # Other hypertrophic osteoarthropathy, right forearm
        "M89432",  # Other hypertrophic osteoarthropathy, left forearm
        "M89439",  # Other hypertrophic osteoarthropathy, unspecified forearm
        "M89441",  # Other hypertrophic osteoarthropathy, right hand
        "M89442",  # Other hypertrophic osteoarthropathy, left hand
        "M89449",  # Other hypertrophic osteoarthropathy, unspecified hand
        "M89451",  # Other hypertrophic osteoarthropathy, right thigh
        "M89452",  # Other hypertrophic osteoarthropathy, left thigh
        "M89459",  # Other hypertrophic osteoarthropathy, unspecified thigh
        "M89461",  # Other hypertrophic osteoarthropathy, right lower leg
        "M89462",  # Other hypertrophic osteoarthropathy, left lower leg
        "M89469",  # Other hypertrophic osteoarthropathy, unspecified lower leg
        "M89471",  # Other hypertrophic osteoarthropathy, right ankle and foot
        "M89472",  # Other hypertrophic osteoarthropathy, left ankle and foot
        "M89479",  # Other hypertrophic osteoarthropathy, unspecified ankle and foot
        "M8948",  # Other hypertrophic osteoarthropathy, other site
        "M8949",  # Other hypertrophic osteoarthropathy, multiple sites
        "M8950",  # Osteolysis, unspecified site
        "M89511",  # Osteolysis, right shoulder
        "M89512",  # Osteolysis, left shoulder
        "M89519",  # Osteolysis, unspecified shoulder
        "M89521",  # Osteolysis, right upper arm
        "M89522",  # Osteolysis, left upper arm
        "M89529",  # Osteolysis, unspecified upper arm
        "M89531",  # Osteolysis, right forearm
        "M89532",  # Osteolysis, left forearm
        "M89539",  # Osteolysis, unspecified forearm
        "M89541",  # Osteolysis, right hand
        "M89542",  # Osteolysis, left hand
        "M89549",  # Osteolysis, unspecified hand
        "M89551",  # Osteolysis, right thigh
        "M89552",  # Osteolysis, left thigh
        "M89559",  # Osteolysis, unspecified thigh
        "M89561",  # Osteolysis, right lower leg
        "M89562",  # Osteolysis, left lower leg
        "M89569",  # Osteolysis, unspecified lower leg
        "M89571",  # Osteolysis, right ankle and foot
        "M89572",  # Osteolysis, left ankle and foot
        "M89579",  # Osteolysis, unspecified ankle and foot
        "M8958",  # Osteolysis, other site
        "M8959",  # Osteolysis, multiple sites
        "M8960",  # Osteopathy after poliomyelitis, unspecified site
        "M89611",  # Osteopathy after poliomyelitis, right shoulder
        "M89612",  # Osteopathy after poliomyelitis, left shoulder
        "M89619",  # Osteopathy after poliomyelitis, unspecified shoulder
        "M89621",  # Osteopathy after poliomyelitis, right upper arm
        "M89622",  # Osteopathy after poliomyelitis, left upper arm
        "M89629",  # Osteopathy after poliomyelitis, unspecified upper arm
        "M89631",  # Osteopathy after poliomyelitis, right forearm
        "M89632",  # Osteopathy after poliomyelitis, left forearm
        "M89639",  # Osteopathy after poliomyelitis, unspecified forearm
        "M89641",  # Osteopathy after poliomyelitis, right hand
        "M89642",  # Osteopathy after poliomyelitis, left hand
        "M89649",  # Osteopathy after poliomyelitis, unspecified hand
        "M89651",  # Osteopathy after poliomyelitis, right thigh
        "M89652",  # Osteopathy after poliomyelitis, left thigh
        "M89659",  # Osteopathy after poliomyelitis, unspecified thigh
        "M89661",  # Osteopathy after poliomyelitis, right lower leg
        "M89662",  # Osteopathy after poliomyelitis, left lower leg
        "M89669",  # Osteopathy after poliomyelitis, unspecified lower leg
        "M89671",  # Osteopathy after poliomyelitis, right ankle and foot
        "M89672",  # Osteopathy after poliomyelitis, left ankle and foot
        "M89679",  # Osteopathy after poliomyelitis, unspecified ankle and foot
        "M8968",  # Osteopathy after poliomyelitis, other site
        "M8969",  # Osteopathy after poliomyelitis, multiple sites
        "M8970",  # Major osseous defect, unspecified site
        "M89711",  # Major osseous defect, right shoulder region
        "M89712",  # Major osseous defect, left shoulder region
        "M89719",  # Major osseous defect, unspecified shoulder region
        "M89721",  # Major osseous defect, right humerus
        "M89722",  # Major osseous defect, left humerus
        "M89729",  # Major osseous defect, unspecified humerus
        "M89731",  # Major osseous defect, right forearm
        "M89732",  # Major osseous defect, left forearm
        "M89739",  # Major osseous defect, unspecified forearm
        "M89741",  # Major osseous defect, right hand
        "M89742",  # Major osseous defect, left hand
        "M89749",  # Major osseous defect, unspecified hand
        "M89751",  # Major osseous defect, right pelvic region and thigh
        "M89752",  # Major osseous defect, left pelvic region and thigh
        "M89759",  # Major osseous defect, unspecified pelvic region and thigh
        "M89761",  # Major osseous defect, right lower leg
        "M89762",  # Major osseous defect, left lower leg
        "M89769",  # Major osseous defect, unspecified lower leg
        "M89771",  # Major osseous defect, right ankle and foot
        "M89772",  # Major osseous defect, left ankle and foot
        "M89779",  # Major osseous defect, unspecified ankle and foot
        "M8978",  # Major osseous defect, other site
        "M8979",  # Major osseous defect, multiple sites
        "M898X0",  # Other specified disorders of bone, multiple sites
        "M898X1",  # Other specified disorders of bone, shoulder
        "M898X2",  # Other specified disorders of bone, upper arm
        "M898X3",  # Other specified disorders of bone, forearm
        "M898X4",  # Other specified disorders of bone, hand
        "M898X5",  # Other specified disorders of bone, thigh
        "M898X6",  # Other specified disorders of bone, lower leg
        "M898X7",  # Other specified disorders of bone, ankle and foot
        "M898X8",  # Other specified disorders of bone, other site
        "M898X9",  # Other specified disorders of bone, unspecified site
        "M899",  # Disorder of bone, unspecified
        "M9080",  # Osteopathy in diseases classified elsewhere, unspecified site
        "M90811",  # Osteopathy in diseases classified elsewhere, right shoulder
        "M90812",  # Osteopathy in diseases classified elsewhere, left shoulder
        "M90819",  # Osteopathy in diseases classified elsewhere, unspecified shoulder
        "M90821",  # Osteopathy in diseases classified elsewhere, right upper arm
        "M90822",  # Osteopathy in diseases classified elsewhere, left upper arm
        "M90829",  # Osteopathy in diseases classified elsewhere, unspecified upper arm
        "M90831",  # Osteopathy in diseases classified elsewhere, right forearm
        "M90832",  # Osteopathy in diseases classified elsewhere, left forearm
        "M90839",  # Osteopathy in diseases classified elsewhere, unspecified forearm
        "M90841",  # Osteopathy in diseases classified elsewhere, right hand
        "M90842",  # Osteopathy in diseases classified elsewhere, left hand
        "M90849",  # Osteopathy in diseases classified elsewhere, unspecified hand
        "M90851",  # Osteopathy in diseases classified elsewhere, right thigh
        "M90852",  # Osteopathy in diseases classified elsewhere, left thigh
        "M90859",  # Osteopathy in diseases classified elsewhere, unspecified thigh
        "M90861",  # Osteopathy in diseases classified elsewhere, right lower leg
        "M90862",  # Osteopathy in diseases classified elsewhere, left lower leg
        "M90869",  # Osteopathy in diseases classified elsewhere, unspecified lower leg
        "M90871",  # Osteopathy in diseases classified elsewhere, right ankle and foot
        "M90872",  # Osteopathy in diseases classified elsewhere, left ankle and foot
        "M90879",  # Osteopathy in diseases classified elsewhere, unspecified ankle and foot
        "M9088",  # Osteopathy in diseases classified elsewhere, other site
        "M9089",  # Osteopathy in diseases classified elsewhere, multiple sites
        "N10",  # Acute pyelonephritis
        "N110",  # Nonobstructive reflux-associated chronic pyelonephritis
        "N111",  # Chronic obstructive pyelonephritis
        "N118",  # Other chronic tubulo-interstitial nephritis
        "N119",  # Chronic tubulo-interstitial nephritis, unspecified
        "N12",  # Tubulo-interstitial nephritis, not specified as acute or chronic
        "N130",  # Hydronephrosis with ureteropelvic junction obstruction
        "N136",  # Pyonephrosis
        "N151",  # Renal and perinephric abscess
        "N16",  # Renal tubulo-interstitial disorders in diseases classified elsewhere
        "N2884",  # Pyelitis cystica
        "N2885",  # Pyeloureteritis cystica
        "N2886",  # Ureteritis cystica
        "N3000",  # Acute cystitis without hematuria
        "N3001",  # Acute cystitis with hematuria
        "N3010",  # Interstitial cystitis (chronic) without hematuria
        "N3011",  # Interstitial cystitis (chronic) with hematuria
        "N3020",  # Other chronic cystitis without hematuria
        "N3021",  # Other chronic cystitis with hematuria
        "N3030",  # Trigonitis without hematuria
        "N3031",  # Trigonitis with hematuria
        "N3040",  # Irradiation cystitis without hematuria
        "N3041",  # Irradiation cystitis with hematuria
        "N3080",  # Other cystitis without hematuria
        "N3081",  # Other cystitis with hematuria
        "N3090",  # Cystitis, unspecified without hematuria
        "N3091",  # Cystitis, unspecified with hematuria
        "N390",  # Urinary tract infection, site not specified
        "N410",  # Acute prostatitis
        "N411",  # Chronic prostatitis
        "N412",  # Abscess of prostate
        "N413",  # Prostatocystitis
        "N414",  # Granulomatous prostatitis
        "N418",  # Other inflammatory diseases of prostate
        "N419",  # Inflammatory disease of prostate, unspecified
        "N7001",  # Acute salpingitis
        "N7002",  # Acute oophoritis
        "N7003",  # Acute salpingitis and oophoritis
        "N7011",  # Chronic salpingitis
        "N7012",  # Chronic oophoritis
        "N7013",  # Chronic salpingitis and oophoritis
        "N7091",  # Salpingitis, unspecified
        "N7092",  # Oophoritis, unspecified
        "N7093",  # Salpingitis and oophoritis, unspecified
        "N710",  # Acute inflammatory disease of uterus
        "N711",  # Chronic inflammatory disease of uterus
        "N719",  # Inflammatory disease of uterus, unspecified
        "N72",  # Inflammatory disease of cervix uteri
        "N730",  # Acute parametritis and pelvic cellulitis
        "N731",  # Chronic parametritis and pelvic cellulitis
        "N732",  # Unspecified parametritis and pelvic cellulitis
        "N733",  # Female acute pelvic peritonitis
        "N734",  # Female chronic pelvic peritonitis
        "N735",  # Female pelvic peritonitis, unspecified
        "N736",  # Female pelvic peritoneal adhesions (postinfective)
        "N738",  # Other specified female pelvic inflammatory diseases
        "N739",  # Female pelvic inflammatory disease, unspecified
        "N74",  # Female pelvic inflammatory disorders in diseases classified elsewhere
        "N750",  # Cyst of Bartholin's gland
        "N751",  # Abscess of Bartholin's gland
        "N758",  # Other diseases of Bartholin's gland
        "N759",  # Disease of Bartholin's gland, unspecified
        "N760",  # Acute vaginitis
        "N761",  # Subacute and chronic vaginitis
        "N762",  # Acute vulvitis
        "N763",  # Subacute and chronic vulvitis
        "N764",  # Abscess of vulva
        "N765",  # Ulceration of vagina
        "N766",  # Ulceration of vulva
        "N7681",  # Mucositis (ulcerative) of vagina and vulva
        "N7689",  # Other specified inflammation of vagina and vulva
        "N770",  # Ulceration of vulva in diseases classified elsewhere
        "N771",  # Vaginitis, vulvitis and vulvovaginitis in diseases classified elsewhere
        "Z202",  # Contact with and (suspected) exposure to infections with a predominantly sexual mode of transmission
        "Z224",  # Carrier of infections with a predominantly sexual mode of transmission
    }

    SNOMEDCT = {
        "10311000175107",  # Pneumonia caused by Enterobacter (disorder)
        "119371000119100",  # Enteritis of small intestine caused by enterotoxigenic Escherichia coli (disorder)
        "716790008",  # Infective dermatitis co-occurrent and due to human T-cell lymphotropic virus 1 infection (disorder)
        "721658008",  # Enteritis caused by protozoan (disorder)
        "733153004",  # Enteritis caused by Entamoeba histolytica (disorder)
        "860874006",  # Enteritis due to human immunodeficiency virus disease (disorder)
        "860885008",  # Enteritis caused by Cryptosporidium (disorder)
        "42004004",  # Aspiration pneumonia caused by inhalation of vomitus (disorder)
        "83608006",  # Aspiration pneumonia caused by inhalation of milk (disorder)
        "13095005",  # Primary symptomatic early syphilis (disorder)
        "13731006",  # Secondary symptomatic early syphilis (disorder)
        "199156006",  # Maternal syphilis during pregnancy - baby delivered (disorder)
        "199157002",  # Maternal syphilis in the puerperium - baby delivered during current episode of care (disorder)
        "199158007",  # Maternal syphilis during pregnancy - baby not yet delivered (disorder)
        "199159004",  # Maternal syphilis in the puerperium - baby delivered during previous episode of care (disorder)
        "199163006",  # Maternal gonorrhea during pregnancy - baby delivered (disorder)
        "199164000",  # Maternal gonorrhea in the puerperium - baby delivered during current episode of care (disorder)
        "199165004",  # Maternal gonorrhea during pregnancy - baby not yet delivered (disorder)
        "199166003",  # Maternal gonorrhea in the puerperium - baby delivered during previous episode of care (disorder)
        "367504009",  # Pelvic inflammatory disease with female sterility due to Chlamydia trachomatis (disorder)
        "57468008",  # Salpingo-oophoritis following molar AND/OR ectopic pregnancy (disorder)
        "72463008",  # Chancriform syndrome (disorder)
        "105656003",  # Disease caused by Enteromonadidae (disorder)
        "113291000119107",  # Enostosis of pelvis (disorder)
        "197760006",  # Chronic pyelonephritis without medullary necrosis (disorder)
        "199205008",  # Infections of bladder in pregnancy (disorder)
        "236626002",  # Chronic cystitis - culture negative (disorder)
        "236627006",  # Chronic nonspecific cystitis (disorder)
        "240762001",  # Cutaneous trichosporonosis (disorder)
        "402963009",  # Typhoid exanthem (disorder)
        "402964003",  # Paratyphoid exanthem (disorder)
        "718702003",  # Chronic symmetric plasma cell osteomyelitis (disorder)
        "105977003",  # Non-infectious pneumonia (disorder)
        "113331000119101",  # Enostosis of humerus (disorder)
        "16035391000119102",  # Osteolysis of right femur (disorder)
        "16035591000119105",  # Osteolysis of left femur (disorder)
        "195756009",  # Woakes' ethmoiditis (disorder)
        "312342009",  # Infective pneumonia (disorder)
        "335846001",  # Acute mastoiditis without complications (disorder)
        "35037009",  # Primary atypical interstitial pneumonia (disorder)
        "367671000119108",  # Hematuria co-occurrent and due to irradiation cystitis (disorder)
        "50804000",  # Catarrhal pneumonia (disorder)
        "71926009",  # Infective pneumonia acquired prenatally (disorder)
        "77798004",  # Systemic chromomycosis (disorder)
        "1010615002",  # Late syphilis of lung (disorder)
        "1010626008",  # Recurrent peritonsillar abscess (disorder)
        "1010634002",  # Pneumonia caused by Acinetobacter (disorder)
        "1010650005",  # Acute endogenous lipoid pneumonitis (disorder)
        "1010670004",  # Chronic endogenous lipoid pneumonitis (disorder)
        "102361000119104",  # Chronic pneumonia (disorder)
        "102453009",  # Peritonsillar cellulitis (disorder)
        "10271000175107",  # Pneumonia caused by Enterococcus faecium (disorder)
        "10281000175105",  # Pneumonia caused by Enterococcus faecalis (disorder)
        "10291000175108",  # Pneumonia caused by Enterococcus (disorder)
        "10301000175109",  # Pneumonia caused by Serratia (disorder)
        "10345003",  # Primary syphilis of tonsils (disorder)
        "1052300001",  # Proctocolitis due to secondary syphilis (disorder)
        "1052319006",  # Secondary syphilis of nervous system (disorder)
        "1052321001",  # Secondary syphilis of digestive system (disorder)
        "1052322008",  # Secondary syphilis of eye (disorder)
        "1052323003",  # Secondary syphilis of musculoskeletal system (disorder)
        "105645003",  # Disease caused by Acanthamoebidae (disorder)
        "10624991000119103",  # Bronchopneumonia caused by Achromobacter (disorder)
        "10625031000119102",  # Bronchopneumonia caused by anaerobic bacteria (disorder)
        "10625071000119104",  # Bronchopneumonia caused by bacteria (disorder)
        "10625111000119106",  # Bronchopneumonia caused by Escherichia coli (disorder)
        "10625151000119107",  # Bronchopneumonia caused by Group A Streptococcus (disorder)
        "10625191000119102",  # Bronchopneumonia caused by Group B Streptococcus (disorder)
        "10625231000119106",  # Bronchopneumonia caused by Haemophilus influenzae (disorder)
        "10625311000119109",  # Bronchopneumonia caused by Klebsiella pneumoniae (disorder)
        "10625351000119105",  # Bronchopneumonia caused by methicillin resistant Staphylococcus aureus (disorder)
        "10625391000119100",  # Bronchopneumonia caused by methicillin susceptible Staphylococcus aureus (disorder)
        "10625431000119105",  # Bronchopneumonia caused by Mycoplasma pneumoniae (disorder)
        "10625471000119108",  # Bronchopneumonia caused by Proteus mirabilis (disorder)
        "10625511000119104",  # Bronchopneumonia caused by Pseudomonas (disorder)
        "10625591000119108",  # Bronchopneumonia caused by Staphylococcus (disorder)
        "10625631000119108",  # Bronchopneumonia caused by Staphylococcus aureus (disorder)
        "10625671000119106",  # Bronchopneumonia caused by Streptococcus (disorder)
        "10625711000119105",  # Bronchopneumonia caused by Streptococcus pneumoniae (disorder)
        "10628871000119101",  # Gastroenteritis caused by Influenza virus (disorder)
        "10628911000119103",  # Gastroenteritis caused by Influenza A virus (disorder)
        "106371000119109",  # Osteopathy co-occurrent and due to non active poliomyelitis of lower leg (disorder)
        "10662001",  # Nonvenomous insect bite of face with infection (disorder)
        "10671005",  # Blister of scapular region with infection (disorder)
        "10679007",  # Infection caused by Giardia lamblia (disorder)
        "10754031000119105",  # Gonorrhea in mother complicating childbirth (disorder)
        "10759841000119105",  # Sexually transmitted infectious disease in mother complicating childbirth (disorder)
        "10759921000119107",  # Syphilis in mother complicating childbirth (disorder)
        "1077931000119106",  # Osteolysis of right distal clavicle (disorder)
        "1077941000119102",  # Osteolysis of left distal clavicle (disorder)
        "1082561000119104",  # Recurrent acute suppurative otitis media with spontaneous rupture of ear drum (disorder)
        "1082721000119101",  # Pneumonia caused by Ascaris (disorder)
        "1082731000119103",  # Enteritis caused by Astrovirus (disorder)
        "1082841000119101",  # Acquired deformity of bilateral pierced ear lobes (disorder)
        "1082931000119101",  # Acute inflammation of bilateral mastoids (disorder)
        "1083061000119100",  # Spontaneous rupture of tympanic membranes of bilateral ears co-occurrent and due to recurrent acute suppurative otitis media (disorder)
        "1083101000119102",  # Acute suppurative otitis media of bilateral ears (disorder)
        "1083111000119104",  # Spontaneous rupture of bilateral tympanic membranes co-occurrent and due to acute suppurative otitis media (disorder)
        "1083321000119108",  # Chronic inflammation of bilateral mastoids (disorder)
        "108365000",  # Infection of skin (disorder)
        "1084011000119100",  # Inflammation of bilateral mastoids (disorder)
        "1084821000119103",  # Cystitis caused by Chlamydia (disorder)
        "1085006",  # Candidiasis of vulva (disorder)
        "1086071000119103",  # Diphtheria tubulointerstitial nephropathy (disorder)
        "1087001000119105",  # Infection of anus and rectum caused by Neisseria gonorrhoeae (disorder)
        "1087021000119101",  # Periurethral abscess caused by Neisseria gonorrhoeae (disorder)
        "1087041000119107",  # Infection of kidney and ureter caused by Neisseria gonorrhoeae (disorder)
        "1087051000119109",  # Gonococcal osteomyelitis (disorder)
        "1087061000119106",  # Gonococcal pneumonia (disorder)
        "1087951000119105",  # Acquired deformity of left pierced ear lobe (disorder)
        "1088041000119106",  # Acute inflammation of left mastoid (disorder)
        "1088181000119100",  # Spontaneous rupture of tympanic membrane of left ear co-occurrent and due to recurrent acute suppurative otitis media (disorder)
        "1088221000119108",  # Acute suppurative otitis media of left ear (disorder)
        "1088231000119106",  # Spontaneous rupture of left tympanic membrane co-occurrent and due to acute suppurative otitis media (disorder)
        "1088451000119104",  # Chronic inflammation of left mastoid (disorder)
        "1088811000119102",  # Stenosis of left external auditory canal due to and following procedure (disorder)
        "1089071000119109",  # Inflammation of left mastoid (disorder)
        "1090571000119108",  # Acquired deformity of right pierced ear lobe (disorder)
        "1090661000119101",  # Acute inflammation of right mastoid (disorder)
        "1090791000119102",  # Spontaneous rupture of tympanic membrane of right ear co-occurrent and due to recurrent acute suppurative otitis media (disorder)
        "1090831000119108",  # Acute suppurative otitis media of right ear (disorder)
        "1091061000119103",  # Chronic inflammation of right mastoid (disorder)
        "1091421000119101",  # Stenosis of right external auditory canal due to and following procedure (disorder)
        "1091681000119104",  # Inflammation of right mastoid (disorder)
        "10920005",  # Abrasion and/or friction burn of lip with infection (disorder)
        "1092371000119103",  # Salmonella pyelonephritis (disorder)
        "1092831000119109",  # Toxoplasma tubulointerstitial nephropathy (disorder)
        "1092951000119106",  # Pneumonia caused by Bordetella parapertussis (disorder)
        "109362006",  # Cellulitis of parapharyngeal space (disorder)
        "109436001",  # Moon molar teeth (disorder)
        "109802000",  # Abscess of uvula of palate (disorder)
        "109909008",  # Acute suppurative mastoiditis (disorder)
        "109910003",  # Chronic suppurative mastoiditis (disorder)
        "10995009",  # Abrasion and/or friction burn of scapular region with infection (disorder)
        "110081000119109",  # Bacterial vaginosis in pregnancy (disorder)
        "110091000119107",  # Candidiasis of vagina in pregnancy (disorder)
        "1102005",  # Intraerythrocytic parasitosis caused by Nuttallia (disorder)
        "1107004",  # Early latent syphilis, positive serology, negative cerebrospinal fluid, with relapse after treatment (disorder)
        "111003001",  # Sporotrichotic gumma (disorder)
        "111347003",  # Exostosis of jaw (disorder)
        "111409009",  # Ulcerative cystitis (disorder)
        "111538005",  # Acute mastoiditis with complication (disorder)
        "111705004",  # Blister of trunk with infection (disorder)
        "111707007",  # Abrasion and/or friction burn of chest wall with infection (disorder)
        "111711001",  # Blister of shoulder with infection (disorder)
        "111712008",  # Abrasion and/or friction burn of elbow with infection (disorder)
        "111796005",  # Madura foot caused by Streptomyces somaliensis (disorder)
        "111806005",  # Acute gonococcal prostatitis (disorder)
        "111807001",  # Gonococcal endophthalmia (disorder)
        "111839008",  # Intestinal infection caused by Escherichia coli (disorder)
        "111843007",  # Viral gastroenteritis (disorder)
        "111852003",  # Vaccinia (disorder)
        "111898002",  # Chronic mycotic otitis externa (disorder)
        "111899005",  # Cutaneous aspergillosis (disorder)
        "111900000",  # Pneumonia in aspergillosis (disorder)
        "111901001",  # Bronchocentric granulomatosis (disorder)
        "111911008",  # Chronic intestinal amebiasis without abscess (disorder)
        "111912001",  # Acute amebic dysentery without abscess (disorder)
        "111921000",  # Trichobilharzia cercarial dermatitis (disorder)
        "111923002",  # Gastrodiscoidiasis (disorder)
        "112121000119105",  # Venereal disease caused by Chlamydia trachomatis (disorder)
        "11251000",  # Cystitis caused by radiation (disorder)
        "11258006",  # Tinea barbae caused by Trichophyton mentagrophytes (disorder)
        "113271000119106",  # Enostosis of talus (disorder)
        "113281000119109",  # Enostosis of fibula (disorder)
        "113301000119108",  # Enostosis of femur (disorder)
        "113311000119106",  # Enostosis of tibia (disorder)
        "113321000119104",  # Enostosis of ulna (disorder)
        "113341000119105",  # Enostosis of radius (disorder)
        "113351000119107",  # Enostosis of vertebra (disorder)
        "11338007",  # Syphilitic episcleritis (disorder)
        "1141788003",  # Bacillary peliosis caused by Bartonella quintana (disorder)
        "1141789006",  # Bacillary peliosis caused by Bartonella henselae (disorder)
        "1142031005",  # Optic atrophy due to late congenital syphilis (disorder)
        "1142036000",  # Neonatal infection of skin caused by Pseudomonas (disorder)
        "1142043006",  # Regurgitation of pulmonary valve due to late syphilis (disorder)
        "1142053007",  # Neonatal cephalic pustulosis caused by Malassezia (disorder)
        "1142055000",  # Human immunodeficiency virus modified skin disease (disorder)
        "1142092000",  # Late congenital syphilitic osteochondropathy (disorder)
        "1142095003",  # Late congenital syphilis of cardiovascular system (disorder)
        "1142099009",  # Secondary syphilis of synovium (disorder)
        "1142226000",  # Infection of intestine caused by Salmonella group O:2 (disorder)
        "1142227009",  # Arthritis of knee joint caused by Borrelia burgdorferi (disorder)
        "1144504001",  # Poliomyelitis osteopathy of pelvis and femur (disorder)
        "1144976006",  # Asymptomatic prostatitis (disorder)
        "1148585008",  # Infection of intestine caused by Vibrio parahaemolyticus (disorder)
        "1148699004",  # Inflammation of small intestine caused by Calicivirus (disorder)
        "114881000119108",  # Maternal gonorrhea during pregnancy (disorder)
        "1148932007",  # Cystitis caused by virus (disorder)
        "1149093006",  # Pneumonia caused by vancomycin resistant Enterococcus (disorder)
        "1153389005",  # Epiphyseal arrest due to disorder of endocrine system (disorder)
        "1153390001",  # Epiphyseal arrest due to kidney disease (disorder)
        "11625007",  # Torus mandibularis (disorder)
        "11673571000119101",  # Exostosis of skull (disorder)
        "11773411000119102",  # Infected abrasion of skin of right thumb (disorder)
        "11773491000119106",  # Infected abrasion of skin of left thumb (disorder)
        "11773571000119109",  # Infected wound of right elbow due to non venomous insect bite (disorder)
        "11773651000119104",  # Infected wound of left elbow due to non venomous insect bite (disorder)
        "11779241000119101",  # Infected abrasion of skin of right forearm (disorder)
        "11779281000119106",  # Infected abrasion of skin of left forearm (disorder)
        "11779481000119105",  # Infected abrasion of skin of right elbow region (disorder)
        "11779521000119105",  # Infected abrasion of skin of left elbow region (disorder)
        "11779561000119100",  # Infected wound of right forearm due to non venomous insect bite (disorder)
        "11779601000119100",  # Infected wound of left forearm due to non venomous insect bite (disorder)
        "11779641000119103",  # Infected abrasion of skin of lesser toe of right foot (disorder)
        "11783401000119109",  # Infected wound of right hip due to nonvenomous insect bite (disorder)
        "11783441000119106",  # Infected wound of left hip due to nonvenomous insect bite (disorder)
        "11783641000119108",  # Infected abrasion of skin of lesser toe of left foot (disorder)
        "11783881000119104",  # Nonvenomous insect bite of right hand with infection (disorder)
        "11783921000119106",  # Nonvenomous insect bite of left hand with infection (disorder)
        "11783961000119101",  # Infected wound of right thigh due to nonvenomous insect bite (disorder)
        "11784001000119100",  # Infected wound of left thigh due to nonvenomous insect bite (disorder)
        "11784041000119103",  # Infected wound of right wrist due to non venomous insect bite (disorder)
        "11784081000119108",  # Infected wound of left wrist due to non venomous insect bite (disorder)
        "11784121000119105",  # Infected abrasion of skin of finger of right hand (disorder)
        "11784161000119100",  # Infected abrasion of skin of right index finger (disorder)
        "11784201000119105",  # Infected abrasion of skin of right little finger (disorder)
        "11784241000119107",  # Infected abrasion of skin of right middle finger (disorder)
        "11784281000119102",  # Infected abrasion of skin of right ring finger (disorder)
        "11784321000119107",  # Infected abrasion of skin of finger of left hand (disorder)
        "11784361000119102",  # Infected abrasion of skin of left index finger (disorder)
        "11784401000119106",  # Infected abrasion of skin of left little finger (disorder)
        "11784441000119108",  # Infected abrasion of skin of left middle finger (disorder)
        "11784481000119103",  # Infected abrasion of skin of left ring finger (disorder)
        "11784521000119103",  # Infected abrasion of skin of right hand (disorder)
        "11784561000119108",  # Infected abrasion of skin of left hand (disorder)
        "11784601000119108",  # Infected abrasion of skin of right wrist region (disorder)
        "11784641000119105",  # Infected abrasion of skin of left wrist region (disorder)
        "11793501000119109",  # Infected abrasion of skin of left foot (disorder)
        "11797701000119109",  # Infected abrasion of skin of sacral region (disorder)
        "11808821000119105",  # Infected abrasion of skin of right hip region (disorder)
        "11808861000119100",  # Infected abrasion of skin of left hip region (disorder)
        "11809221000119101",  # Infected wound of low back due to nonvenomous insect bite (disorder)
        "11809421000119102",  # Infected abrasion of skin of right lower leg (disorder)
        "11809461000119107",  # Infected abrasion of skin of left lower leg (disorder)
        "11809501000119107",  # Infected abrasion of skin of right knee region (disorder)
        "11809541000119109",  # Infected abrasion of skin of left knee region (disorder)
        "11809581000119104",  # Infected abrasion of skin of right thigh (disorder)
        "11809621000119104",  # Infected abrasion of skin of left thigh (disorder)
        "11809661000119109",  # Infected wound of right knee due to nonvenomous insect bite (disorder)
        "11809701000119102",  # Infected wound of left knee due to nonvenomous insect bite (disorder)
        "11822921000119105",  # Infected abrasion of skin of right upper arm (disorder)
        "11822961000119100",  # Infected abrasion of skin of left upper arm (disorder)
        "11823161000119108",  # Infected abrasion of skin of right shoulder region (disorder)
        "11823201000119103",  # Infected abrasion of skin of left shoulder region (disorder)
        "11823321000119101",  # Infected wound of right upper arm due to non venomous insect bite (disorder)
        "11823361000119106",  # Infected wound of left upper arm due to non venomous insect bite (disorder)
        "11837001000119102",  # Infected wound of right lower leg due to nonvenomous insect bite (disorder)
        "11837041000119100",  # Infected wound of left lower leg due to nonvenomous insect bite (disorder)
        "11840006",  # Traveler's diarrhea (disorder)
        "11843951000119105",  # Infected abrasion of skin of right ear (disorder)
        "11843991000119100",  # Infected abrasion of skin of left ear (disorder)
        "11848301000119100",  # Infected wound of right eyelid due to nonvenomous insect bite (disorder)
        "11848381000119108",  # Infected wound of left eyelid due to nonvenomous insect bite (disorder)
        "11848501000119109",  # Infected abrasion of skin of right eyelid (disorder)
        "11848581000119101",  # Infected abrasion of skin of left eyelid (disorder)
        "11848981000119109",  # Infected abrasion of skin of right breast (disorder)
        "11849021000119103",  # Infected abrasion of skin of left breast (disorder)
        "11862161000119109",  # Infected abrasion of skin of right ankle region (disorder)
        "11862201000119104",  # Infected abrasion of skin of left ankle region (disorder)
        "11862241000119102",  # Infected abrasion of skin of right great toe (disorder)
        "11862281000119107",  # Infected abrasion of skin of left great toe (disorder)
        "11862321000119102",  # Infected abrasion of skin of right foot (disorder)
        "11862521000119106",  # Infected wound of right foot due to nonvenomous insect bite (disorder)
        "11862561000119101",  # Infected wound of left foot due to nonvenomous insect bite (disorder)
        "11906007",  # Chronic gonococcal vulvovaginitis (disorder)
        "12008008",  # Palmar papilloma of yaws (disorder)
        "120639003",  # Hantavirus pulmonary syndrome (disorder)
        "12232008",  # Syphilitic aneurysm of aorta (disorder)
        "12241911000119102",  # Plantar wart of left foot (disorder)
        "12241951000119101",  # Plantar wart of right foot (disorder)
        "12277551000119101",  # Bilateral plantar wart (disorder)
        "123587001",  # Acute bronchopneumonia (disorder)
        "123588006",  # Confluent bronchopneumonia with abscess formation (disorder)
        "123589003",  # Necrotizing bronchopneumonia (disorder)
        "123590007",  # Focal pneumonia (disorder)
        "123591006",  # Confluent pneumonia (disorder)
        "123611003",  # Subacute pyelonephritis (disorder)
        "12373006",  # Chronic gonococcal bartholinitis (disorder)
        "123754002",  # Focal pyelonephritis (disorder)
        "123755001",  # Diffuse pyelonephritis (disorder)
        "123812008",  # Zinc excess (disorder)
        "12435008",  # Abrasion and/or friction burn of anus with infection (disorder)
        "12463005",  # Infectious gastroenteritis (disorder)
        "124691000119101",  # Pneumonia caused by methicillin resistant Staphylococcus aureus (disorder)
        "12602008",  # Non-hereditary acrodermatitis enteropathica (disorder)
        "127601000119103",  # Disseminated Acanthamoeba infection (disorder)
        "12804006",  # Tinea cruris caused by Trichophyton rubrum (disorder)
        "1282001",  # Perichondritis of larynx (disorder)
        "128711000119106",  # Pneumonia caused by methicillin susceptible Staphylococcus aureus (disorder)
        "128936008",  # Bacterial infection of skin (disorder)
        "128937004",  # Viral infection of skin (disorder)
        "128938009",  # Parasitic infection of skin (disorder)
        "128939001",  # Infection of subcutaneous tissue (disorder)
        "129451001",  # Respiratory bronchiolitis associated interstitial lung disease (disorder)
        "129452008",  # Nonspecific interstitial pneumonitis (disorder)
        "129668006",  # Ulcer caused by Treponema vincentii (disorder)
        "13087006",  # Necrosis of larynx (disorder)
        "13125003",  # Chronic myometritis (disorder)
        "13177009",  # Cellulitis of nasopharynx (disorder)
        "13285005",  # Cystitis cystica (disorder)
        "13310005",  # Taboparesis (disorder)
        "133211000119107",  # Acquired deformity of pierced ear lobe (disorder)
        "13338002",  # Verruca palmaris (disorder)
        "13471002",  # Infection caused by Trichuris ovis (disorder)
        "13641000119106",  # Viral colitis (disorder)
        "14096004",  # Nonvenomous insect bite of scapular region with infection (disorder)
        "142961000119108",  # Gastroenteritis caused by Influenza A virus subtype H1N1 (disorder)
        "14386001",  # Indeterminate leprosy (disorder)
        "14560005",  # Dermal mycosis (disorder)
        "14948001",  # Acute suppurative otitis media without spontaneous rupture of ear drum (disorder)
        "15033003",  # Peritonsillar abscess (disorder)
        "15034009",  # Nonvenomous insect bite of breast with infection (disorder)
        "151004",  # Gonococcal meningitis (disorder)
        "1515008",  # Gorham's disease (disorder)
        "15207003",  # Zoonotic form of cutaneous leishmaniasis (disorder)
        "15470004",  # Toxic diffuse goiter with acropachy (disorder)
        "15628003",  # Gonorrhea (disorder)
        "15634005",  # Palmar hyperkeratosis of yaws (disorder)
        "15640081000119104",  # Bilateral bone spur of calcaneum (disorder)
        "15677801000119103",  # Chorioretinitis of right eye due to secondary syphilis (disorder)
        "15677841000119101",  # Chorioretinitis of bilateral eyes due to secondary syphilis (disorder)
        "15677881000119106",  # Chorioretinitis of left eye due to secondary syphilis (disorder)
        "15679481000119104",  # Iritis of right eye caused by Treponema pallidum (disorder)
        "15679561000119109",  # Iritis of left eye caused by Treponema pallidum (disorder)
        "15679681000119101",  # Iritis of bilateral eyes caused by Treponema pallidum (disorder)
        "15681361000119105",  # Infection of skin of bilateral eyelids due to herpes zoster (disorder)
        "15685441000119102",  # Retinitis of bilateral eyes caused by Treponema pallidum (disorder)
        "15693201000119102",  # Keratitis of bilateral eyes caused by Acanthamoeba (disorder)
        "15693241000119100",  # Keratitis of left eye caused by Acanthamoeba (disorder)
        "15693281000119105",  # Keratitis of right eye caused by Acanthamoeba (disorder)
        "15698841000119105",  # Ulcer of right cornea caused by Acanthamoeba (disorder)
        "15698881000119100",  # Ulcer of left cornea caused by Acanthamoeba (disorder)
        "15715841000119107",  # Enostosis of left tibia (disorder)
        "15715881000119102",  # Enostosis of right tibia (disorder)
        "15716561000119104",  # Bone island of right femur (disorder)
        "15716601000119104",  # Bone island of left femur (disorder)
        "15845006",  # Buruli ulcer (disorder)
        "15936661000119103",  # Erysipelas of bilateral lower limbs (disorder)
        "15936701000119105",  # Erysipelas of right lower limb (disorder)
        "15936741000119107",  # Erysipelas of left lower limb (disorder)
        "15965501000119100",  # Abscess of right greater vestibular gland (disorder)
        "15965541000119103",  # Abscess of left greater vestibular gland (disorder)
        "15981911000119105",  # Chronic bilateral maxillary sinusitis (disorder)
        "15981951000119106",  # Chronic right maxillary sinusitis (disorder)
        "15981991000119101",  # Chronic left maxillary sinusitis (disorder)
        "15993391000119102",  # Retinitis of bilateral eyes due to secondary syphilis (disorder)
        "16015471000119102",  # Osteopathy of right foot following poliomyelitis (disorder)
        "16015551000119107",  # Osteopathy of bilateral feet following poliomyelitis (disorder)
        "16015631000119102",  # Osteopathy of left foot following poliomyelitis (disorder)
        "16035631000119105",  # Osteolysis of left foot (disorder)
        "16035831000119109",  # Osteolysis of right ankle (disorder)
        "16035991000119101",  # Osteolysis of bilateral hands (disorder)
        "16036000",  # Acute empyema of frontal sinus (disorder)
        "16036071000119105",  # Osteolysis of left ankle (disorder)
        "16036111000119103",  # Osteolysis of right foot (disorder)
        "16070004",  # Syphilitic cirrhosis (disorder)
        "1621000119101",  # Chlamydia trachomatis infection in pregnancy (disorder)
        "16217981000119107",  # Infection of upper genitourinary tract caused by Neisseria gonorrheae (disorder)
        "16271000119108",  # Pyelonephritis in pregnancy (disorder)
        "16291381000119103",  # Infection of subungual skin of toe of bilateral feet due to ingrown toenail (disorder)
        "16314371000119109",  # Steroid-modified tinea infection of lower limb (disorder)
        "16314421000119103",  # Steroid-modified tinea infection of groin (disorder)
        "16314471000119102",  # Steroid-modified tinea infection of foot (disorder)
        "16358007",  # Chronic disease of tonsils AND/OR adenoids (disorder)
        "16429005",  # Cutaneous larva migrans caused by Ancylostoma braziliense (disorder)
        "16442005",  # Leontiasis ossium (disorder)
        "16681000",  # Candidal otitis externa (disorder)
        "17039000",  # Blister of abdominal wall with infection (disorder)
        "17198008",  # Nonvenomous insect bite of ear with infection (disorder)
        "17250009",  # Blister of interscapular region with infection (disorder)
        "17273001",  # Mucocutaneous cryptococcosis (disorder)
        "17305005",  # Acute gonorrhea of genitourinary tract (disorder)
        "17384006",  # Nonvenomous insect bite of toe with infection (disorder)
        "17723000",  # Infection caused by Naegleria (disorder)
        "17794004",  # Recurrent cutaneous leishmaniasis (disorder)
        "18081009",  # Enteric campylobacteriosis (disorder)
        "18099001",  # Retropharyngeal abscess (disorder)
        "181007",  # Hemorrhagic bronchopneumonia (disorder)
        "18229003",  # Suppurative colitis (disorder)
        "185367005",  # Microsporosis nigra (disorder)
        "186116005",  # Acute amebic dysentery (disorder)
        "186150001",  # Enteritis caused by rotavirus (disorder)
        "186289000",  # Tinea nigra (disorder)
        "186357007",  # Streptococcal sore throat with scarlatina (disorder)
        "186394003",  # Actinomycosis of cecum (disorder)
        "186397005",  # Madura foot - actinomycotic (disorder)
        "186431008",  # Infection caused by Clostridioides difficile (disorder)
        "186535001",  # Eczema herpeticum (disorder)
        "186668002",  # Gammaherpesviral mononucleosis (disorder)
        "186807008",  # Asian desert cutaneous leishmaniasis (disorder)
        "186825001",  # Systemic bartonellosis (disorder)
        "186826000",  # Cutaneous and mucocutaneous bartonellosis (disorder)
        "186833000",  # Early congenital syphilis - latent (disorder)
        "186842007",  # Late congenital syphilitic oculopathy (disorder)
        "186846005",  # Early symptomatic syphilis (disorder)
        "186847001",  # Primary genital syphilis (disorder)
        "186850003",  # Secondary syphilis of skin and mucous membrane (disorder)
        "186854007",  # Uveitis due to secondary syphilis (disorder)
        "186861006",  # Secondary syphilis relapse (disorder)
        "186863009",  # Acute secondary syphilitic meningitis (disorder)
        "186867005",  # Latent early syphilis (disorder)
        "186868000",  # Serological relapse after treatment of latent early syphilis (disorder)
        "186875004",  # Syphilitic endocarditis of mitral valve (disorder)
        "186877007",  # Syphilitic endocarditis of tricuspid valve (disorder)
        "186878002",  # Syphilitic endocarditis of pulmonary valve (disorder)
        "186893003",  # Rupture of syphilitic cerebral aneurysm (disorder)
        "186903006",  # Late latent syphilis (disorder)
        "186915005",  # Chronic gonorrhea lower genitourinary tract (disorder)
        "186931002",  # Gonococcal anal infection (disorder)
        "186939000",  # Gonococcal peritonitis (disorder)
        "186946009",  # Lymphogranuloma venereum (disorder)
        "187052004",  # Pulmonary African histoplasmosis (disorder)
        "187065008",  # Cutaneous blastomycosis (disorder)
        "187081003",  # Subcutaneous pheohyphomycotic abscess and cyst (disorder)
        "187089001",  # Actinomycotic mycetoma (disorder)
        "187115002",  # Cutaneous schistosomiasis (disorder)
        "187171000",  # Ascariasis with intestinal complications (disorder)
        "187174008",  # Intestinal strongyloidiasis (disorder)
        "187187002",  # Intestinal angiostrongyliasis (disorder)
        "188463006",  # Chlamydial pelvic inflammatory disease (disorder)
        "18849004",  # Infection caused by Trichostrongylus axei (disorder)
        "18900005",  # Periporitis (disorder)
        "18901009",  # Cutaneous diphtheria (disorder)
        "19108007",  # Nonvenomous insect bite of trunk with infection (disorder)
        "19138001",  # Epidermodysplasia verruciformis (disorder)
        "191727003",  # Post measles pneumonia (disorder)
        "192008",  # Congenital syphilitic hepatomegaly (disorder)
        "19206003",  # Syphilitic acoustic neuritis (disorder)
        "19213003",  # Infectious diarrheal disease (disorder)
        "192647003",  # Secondary syphilitic meningitis (disorder)
        "19290004",  # Clutton's joints (disorder)
        "19362000",  # Cutaneous larva migrans (disorder)
        "193786000",  # Keratitis due to syphilis (disorder)
        "193922006",  # Parasitic eyelid infestation (disorder)
        "194202008",  # Infective otitis externa due to erysipelas (disorder)
        "194203003",  # Infective otitis externa due to impetigo (disorder)
        "194208007",  # Chronic otitis externa due to moniliasis (disorder)
        "194281003",  # Acute suppurative otitis media (disorder)
        "194907008",  # Acute syphilitic pericarditis (disorder)
        "194910001",  # Acute gonococcal pericarditis (disorder)
        "194947001",  # Acute myocarditis caused by Treponema pallidum (disorder)
        "195720006",  # Acute streptococcal bronchitis (disorder)
        "195721005",  # Acute bronchitis caused by Haemophilus influenzae (disorder)
        "195798007",  # Chronic adenotonsillitis (disorder)
        "195834004",  # Pharynx or nasopharynx edema (disorder)
        "195866009",  # Laryngeal pachydermia (disorder)
        "195878008",  # Pneumonia and influenza (disorder)
        "195886008",  # Group B streptococcal pneumonia (disorder)
        "195888009",  # Proteus pneumonia (disorder)
        "195889001",  # Legionella pneumonia (disorder)
        "195896004",  # Pneumonia caused by pleuropneumonia-like organism (disorder)
        "195902009",  # Anthrax pneumonia (disorder)
        "195904005",  # Pneumonia with coccidioidomycosis (disorder)
        "195908008",  # Actinomycotic pneumonia (disorder)
        "195909000",  # Nocardial pneumonia (disorder)
        "197143009",  # Megacolon in Chagas' disease (disorder)
        "197305002",  # Syphilitic portal cirrhosis (disorder)
        "197347003",  # Hepatitis in late syphilis (disorder)
        "197348008",  # Hepatitis in secondary syphilis (disorder)
        "197757004",  # Late syphilis of kidney (disorder)
        "197761005",  # Chronic pyelonephritis with medullary necrosis (disorder)
        "197763008",  # Chronic pyonephrosis (disorder)
        "197764002",  # Non-obstructive reflux-associated chronic pyelonephritis (disorder)
        "197768004",  # Acute pyelonephritis without medullary necrosis (disorder)
        "197769007",  # Acute pyelonephritis with medullary necrosis (disorder)
        "197770008",  # Acute pyonephrosis (disorder)
        "197782004",  # Pyelonephritis associated with another disorder (disorder)
        "197834003",  # Chronic interstitial cystitis (disorder)
        "197837005",  # Submucous cystitis (disorder)
        "197845000",  # Cystitis associated with another disorder (disorder)
        "197848003",  # Gonococcal cystitis (disorder)
        "197849006",  # Cystitis caused by Candida (disorder)
        "197850006",  # Trichomonal cystitis (disorder)
        "197853008",  # Recurrent cystitis (disorder)
        "197963001",  # Prostatitis associated with another disorder (disorder)
        "197964007",  # Actinomycotic prostatitis (disorder)
        "197965008",  # Blastomycotic prostatitis (disorder)
        "197966009",  # Syphilitic prostatitis (disorder)
        "197967000",  # Gonococcal prostatitis (disorder)
        "197968005",  # Prostatitis caused by Candida (disorder)
        "198130006",  # Female pelvic inflammatory disease (disorder)
        "198141008",  # Chronic perioophoritis (disorder)
        "198159006",  # Chronic parametritis and pelvic cellulitis (disorder)
        "198175009",  # Female syphilitic pelvic inflammatory disease (disorder)
        "198186006",  # Subacute pyometra (disorder)
        "198189004",  # Chronic pyometra (disorder)
        "198235009",  # Carbuncle of vagina (disorder)
        "19824006",  # Infection of skin and/or subcutaneous tissue (disorder)
        "198242009",  # Female gonococcal pelvic inflammatory disease (disorder)
        "19905009",  # Chronic prostatitis (disorder)
        "199154009",  # Maternal syphilis during pregnancy, childbirth and the puerperium (disorder)
        "199161008",  # Maternal gonorrhea during pregnancy, childbirth and the puerperium (disorder)
        "200635001",  # Finger web space infection (disorder)
        "200649005",  # Cellulitis and abscess of submandibular region (disorder)
        "200721005",  # Pyogenic progressive granuloma (disorder)
        "200722003",  # Pyogenic granuloma (disorder)
        "201060008",  # Disorder of upper respiratory system (disorder)
        "201517005",  # Seronegative arthritis secondary to Whipple's disease (disorder)
        "202756004",  # Infection of intervertebral disc - pyogenic (disorder)
        "202883008",  # Posterior calcaneal exostosis (disorder)
        "202933002",  # Bursitis caused by Treponema pallidum (disorder)
        "203157008",  # Acute osteomyelitis of cervical spine (disorder)
        "203158003",  # Acute osteomyelitis of thoracic spine (disorder)
        "203159006",  # Acute osteomyelitis of lumbar spine (disorder)
        "203193005",  # Chronic osteomyelitis of cervical spine (disorder)
        "203194004",  # Chronic osteomyelitis of thoracic spine (disorder)
        "203195003",  # Chronic osteomyelitis of lumbar spine (disorder)
        "203198001",  # Brodie's abscess of cervical spine (disorder)
        "203199009",  # Brodie's abscess of thoracic spine (disorder)
        "203200007",  # Brodie's abscess of lumbar spine (disorder)
        "203241002",  # Osteomyelitis of vertebra (disorder)
        "20325007",  # Tinea barbae caused by Microsporum (disorder)
        "203284001",  # Osteopathy from poliomyelitis (disorder)
        "203286004",  # Poliomyelitis osteopathy of the shoulder region (disorder)
        "203287008",  # Poliomyelitis osteopathy of the upper arm (disorder)
        "203288003",  # Poliomyelitis osteopathy of the forearm (disorder)
        "203289006",  # Poliomyelitis osteopathy of the hand (disorder)
        "203291003",  # Poliomyelitis osteopathy of the lower leg (disorder)
        "203292005",  # Poliomyelitis osteopathy of the ankle and/or foot (disorder)
        "203294006",  # Poliomyelitis osteopathy of multiple sites (disorder)
        "203296008",  # Subacute osteomyelitis of cervical spine (disorder)
        "203297004",  # Subacute osteomyelitis of thoracic spine (disorder)
        "203298009",  # Subacute osteomyelitis of lumbar spine (disorder)
        "203517001",  # Complete epiphyseal arrest (disorder)
        "203518006",  # Partial epiphyseal arrest (disorder)
        "203522001",  # Osteolysis (disorder)
        "204919003",  # Erythematous pinta plaque (disorder)
        "206283000",  # Congenital staphylococcal pneumonia (disorder)
        "206284006",  # Congenital group A hemolytic streptococcal pneumonia (disorder)
        "206285007",  # Congenital group B hemolytic streptococcal pneumonia (disorder)
        "206286008",  # Congenital Escherichia coli pneumonia (disorder)
        "206287004",  # Congenital pseudomonal pneumonia (disorder)
        "206289001",  # Congenital chlamydial pneumonia (disorder)
        "206356004",  # Neonatal candidiasis of perineum (disorder)
        "206358003",  # Neonatal candidiasis of intestine (disorder)
        "206445003",  # Malarial pigment deposition (disorder)
        "206554002",  # Neonatal skin infection (disorder)
        "206598002",  # Relapsing fever of Southwest United States AND/OR Mexico (disorder)
        "20735004",  # Syphilitic aortitis (disorder)
        "20860008",  # Infestation of intestine caused by fly larvae (disorder)
        "20943002",  # Acute inflammation of cervix uteri caused by Neisseria gonorrhoeae (disorder)
        "209516003",  # Relapsing fever of North Africa (disorder)
        "21061004",  # Babesiosis (disorder)
        "210992005",  # Infected abrasion of skin of face (disorder)
        "210993000",  # Infected abrasion of skin of neck (disorder)
        "210994006",  # Infected abrasion of scalp (disorder)
        "211012003",  # Insect bite, nonvenomous, of head, infected (disorder)
        "211053006",  # Abrasion and/or friction burn of trunk, infected (disorder)
        "211054000",  # Infected abrasion of skin of interscapular region of back (disorder)
        "211055004",  # Infected abrasion of skin of chest (disorder)
        "211056003",  # Infected abrasion of skin of breast (disorder)
        "211057007",  # Infected abrasion of skin of anterior abdominal wall (disorder)
        "211058002",  # Infected abrasion of skin of back (disorder)
        "211059005",  # Infected abrasion of skin of buttock (disorder)
        "211060000",  # Infected abrasion of anus (disorder)
        "211061001",  # Infected abrasion of skin of flank (disorder)
        "211062008",  # Infected abrasion of skin of groin (disorder)
        "211063003",  # Infected abrasion of perineum (disorder)
        "211064009",  # Infected abrasion of skin of penis (disorder)
        "211145001",  # Splinter of flank without major open wound, infected (disorder)
        "211196004",  # Abrasion and/or friction burn of shoulder and/or upper arm, infected (disorder)
        "211197008",  # Infected abrasion of skin of shoulder (disorder)
        "211198003",  # Infected abrasion of skin of scapular region of back (disorder)
        "211199006",  # Infected abrasion of skin of axilla (disorder)
        "211200009",  # Infected abrasion of skin of upper arm (disorder)
        "211220005",  # Insect bite, nonvenomous, of shoulder and upper arm, infected (disorder)
        "211256008",  # Abrasion and/or friction burn of forearm, infected (disorder)
        "211257004",  # Infected abrasion of skin of elbow (disorder)
        "211258009",  # Infected abrasion of skin of forearm (disorder)
        "211259001",  # Infected abrasion of skin of wrist (disorder)
        "211306000",  # Abrasion and/or friction burn of hand, infected (disorder)
        "211320007",  # Abrasion and/or friction burn of finger, infected (disorder)
        "211336009",  # Abrasion and/or friction burn of lower limb, infected (disorder)
        "211337000",  # Infected abrasion of skin of hip (disorder)
        "211338005",  # Infected abrasion of skin of thigh (disorder)
        "211339002",  # Infected abrasion of skin of knee (disorder)
        "211340000",  # Infected abrasion of skin of lower leg (disorder)
        "211341001",  # Infected abrasion of skin of ankle (disorder)
        "211364004",  # Insect bite, nonvenomous, of lower limb, infected (disorder)
        "211365003",  # Insect bite, nonvenomous, of knee, infected (disorder)
        "211406001",  # Infected abrasion of skin of foot (disorder)
        "211407005",  # Infected abrasion of skin of toe (disorder)
        "2129002",  # Edema of pharynx (disorder)
        "214600002",  # Tinea profunda (disorder)
        "21523006",  # Syphilitic gumma of central nervous system (disorder)
        "21560005",  # Lepromatous leprosy (disorder)
        "21955004",  # Trichomonal cervicitis (disorder)
        "22064009",  # Pinta (disorder)
        "22277006",  # Blister of groin with infection (disorder)
        "22386003",  # Syphilitic optic atrophy (disorder)
        "22500005",  # Infection by Oesophagostomum (disorder)
        "22557000",  # Bullous cystitis (disorder)
        "22754005",  # Staphylococcal pneumonia (disorder)
        "22905009",  # Infection caused by Heterophyes heterophyes (disorder)
        "22922006",  # Infection caused by Physaloptera (disorder)
        "22999005",  # Blister of thigh with infection (disorder)
        "230150008",  # Meningitis in Lyme disease (disorder)
        "230152000",  # Late congenital syphilitic meningitis (disorder)
        "230182006",  # Late syphilitic encephalitis (disorder)
        "230194002",  # Neuro - Whipple's disease (disorder)
        "230402000",  # Panmural cystitis (disorder)
        "230563005",  # Late congenital syphilitic polyneuropathy (disorder)
        "230605007",  # Lyme mononeuritis multiplex (disorder)
        "230735006",  # Syphilitic cerebral arteritis (disorder)
        "23166004",  # Cellulitis of pharynx (disorder)
        "231801008",  # Bacterial dermatitis of eyelid (disorder)
        "231803006",  # Viral dermatitis of eyelid (disorder)
        "231804000",  # Fungal dermatitis of eyelid (disorder)
        "231858009",  # Gonococcal conjunctivitis (disorder)
        "231896005",  # Acanthamoeba keratitis (disorder)
        "23191004",  # Early yaws (disorder)
        "23196009",  # Frambeside of early yaws (disorder)
        "232222008",  # Acute viral otitis externa (disorder)
        "232223003",  # Acute fungal otitis externa (disorder)
        "232224009",  # Chronic infective otitis externa (disorder)
        "232226006",  # Chronic viral otitis externa (disorder)
        "232251007",  # Recurrent acute suppurative otitis media (disorder)
        "232268006",  # Acute mastoiditis with neck abscess (disorder)
        "232269003",  # Acute mastoiditis with facial paralysis (disorder)
        "232270002",  # Acute mastoiditis with labyrinthitis (disorder)
        "232271003",  # Intracranial disorder due to and following acute mastoiditis (disorder)
        "232313005",  # Endocochlear syphilis (disorder)
        "232314004",  # Lyme disease of inner ear (disorder)
        "232367004",  # Nasal syphilis (disorder)
        "232369001",  # Granulomatosis with polyangiitis of nose (disorder)
        "232393006",  # Chronic anterior ethmoidal sinusitis (disorder)
        "232394000",  # Chronic posterior ethmoidal sinusitis (disorder)
        "232395004",  # Chronic panethmoidal sinusitis (disorder)
        "232396003",  # Chronic osteomeatal disease (disorder)
        "232397007",  # Chronic frontoethmoidal sinusitis (disorder)
        "232404007",  # Acute pharyngeal candidiasis (disorder)
        "232410007",  # Vallecular cyst (disorder)
        "232411006",  # Edema of uvula (disorder)
        "232418000",  # Acute infection of tonsillar remnant (disorder)
        "232420002",  # Chronic adenoiditis (disorder)
        "232423000",  # Tonsillar retention cyst (disorder)
        "232460001",  # Granulomatosis with polyangiitis of larynx (disorder)
        "233599001",  # Acute mycoplasmal bronchitis (disorder)
        "233604007",  # Pneumonia (disorder)
        "233607000",  # Pneumococcal pneumonia (disorder)
        "233608005",  # Meningococcal pneumonia (disorder)
        "233609002",  # Pneumonia caused by Chlamydiaceae (disorder)
        "233610007",  # Neonatal chlamydial pneumonia (disorder)
        "233613009",  # Fungal pneumonia (disorder)
        "233614003",  # Pulmonary mucormycosis (disorder)
        "233617005",  # Hemorrhagic pneumonia (disorder)
        "233618000",  # Mycobacterial pneumonia (disorder)
        "233619008",  # Neonatal pneumonia (disorder)
        "233620002",  # Pneumonia due to parasitic infestation (disorder)
        "233621003",  # Rickettsial pneumonia (disorder)
        "233713004",  # Seasonal cryptogenic organizing pneumonia with biochemical cholestasis (disorder)
        "233728004",  # Cholesterol pneumonitis (disorder)
        "233730002",  # Lupus pneumonitis (disorder)
        "233731003",  # Traumatic pneumonitis (disorder)
        "233849007",  # Syphilitic valve disease (disorder)
        "234998009",  # Orofacial granulomatosis of gingivae (disorder)
        "235000001",  # Granulomatosis with polyangiitis of gingiva (disorder)
        "235002009",  # Gingival pyogenic granuloma (disorder)
        "23502006",  # Lyme disease (disorder)
        "235032001",  # Syphilitic oral leukoplakia (disorder)
        "235062007",  # Oral syphilis (disorder)
        "235064008",  # Syphilitic gumma of oral cavity (disorder)
        "235065009",  # Syphilitic oral snail track ulcer (disorder)
        "235231000119100",  # Osteophyte of bone (disorder)
        "235604009",  # Protozoal esophagitis (disorder)
        "235612001",  # Protozoal ulcer of esophagus (disorder)
        "235726002",  # Human immunodeficiency virus enteropathy (disorder)
        "235747003",  # Amebic colitis (disorder)
        "235748008",  # Fulminant amebic colitis (disorder)
        "235749000",  # Cytomegaloviral colitis (disorder)
        "235764000",  # Infective proctitis (disorder)
        "235770006",  # Acute suppurative appendicitis (disorder)
        "235834001",  # Infection-induced ileus (disorder)
        "235861001",  # Abscess gonococcal (disorder)
        "235863003",  # Gonococcal hepatitis (disorder)
        "236063005",  # Adenoviral gastroenteritis (disorder)
        "236066002",  # Prototheca diarrhea (disorder)
        "236302005",  # Acute interstitial pneumonitis (disorder)
        "236373001",  # Emphysematous pyelonephritis (disorder)
        "236374007",  # Chronic infective interstitial nephritis (disorder)
        "236375008",  # Emphysematous pyelitis (disorder)
        "236378005",  # Candida pyelonephritis (disorder)
        "236590008",  # Cytomegalovirus-induced glomerulonephritis (disorder)
        "236599009",  # Schistosomiasis of ureter (disorder)
        "236600007",  # Candida ureteritis (disorder)
        "236620008",  # Infective cystitis (disorder)
        "236622000",  # Chronic infective cystitis (disorder)
        "236624004",  # Acute cystitis - culture-negative (disorder)
        "236625003",  # Recurrent cystitis - culture-negative (disorder)
        "236628001",  # Acute radiation cystitis (disorder)
        "236629009",  # Chronic radiation cystitis (disorder)
        "236630004",  # Chemical cystitis (disorder)
        "236672003",  # Chlamydial prostatitis (disorder)
        "236673008",  # Non-bacterial prostatitis (disorder)
        "236682002",  # Inflammation of urethra caused by Neisseria gonorrhoeae (disorder)
        "236687008",  # Gonococcal urethral abscess (disorder)
        "236702008",  # Acute culture positive cystitis (disorder)
        "236703003",  # Recurrent infective cystitis (disorder)
        "236766009",  # Gonococcal epididymitis (disorder)
        "236772009",  # Gonococcal epididymo-orchitis (disorder)
        "237037006",  # Acute pelvic inflammatory disease (disorder)
        "237038001",  # Gonococcal salpingitis (disorder)
        "237039009",  # Chlamydial salpingitis (disorder)
        "237042003",  # Gonococcal perihepatitis (disorder)
        "237044002",  # Chronic pelvic inflammatory disease (disorder)
        "237045001",  # Chronic parametritis (disorder)
        "237046000",  # Gonococcal tubo-ovarian abscess (disorder)
        "237069002",  # Gonococcal endometritis (disorder)
        "237070001",  # Uterine candidiasis (disorder)
        "237083000",  # Inflammation of cervix uteri caused by Neisseria gonorrhoeae (disorder)
        "237084006",  # Inflammation of cervix uteri caused by Chlamydia (disorder)
        "237085007",  # Actinomycotic cervicitis (disorder)
        "237091009",  # Infective vaginitis (disorder)
        "237094001",  # Streptococcal vulvovaginitis (disorder)
        "237095000",  # Gonococcal vulvovaginitis (disorder)
        "237096004",  # Neonatal gonococcal vulvovaginitis (disorder)
        "237097008",  # Chlamydial vulvovaginitis (disorder)
        "237104007",  # Trichomonal vulvitis (disorder)
        "237106009",  # Chlamydial bartholinitis (disorder)
        "237109002",  # Vulval warts (disorder)
        "237110007",  # Condyloma acuminata of vulva (disorder)
        "237124006",  # Hematopyometra (disorder)
        "237446005",  # Syphilis of breast (disorder)
        "237447001",  # Primary syphilis of nipple (disorder)
        "237461006",  # Infected gland of Montgomery (disorder)
        "23754003",  # Pyelonephritis due to kidney stone (disorder)
        "237826006",  # Thyroid acropachy (disorder)
        "238367006",  # Pox virus infection of skin (disorder)
        "238368001",  # Eczema vaccinatum (disorder)
        "238370005",  # Infectious mononucleosis exanthem (disorder)
        "238372002",  # Chlamydial dermatological disorders (disorder)
        "238379006",  # Staphylococcal folliculitis of scalp (disorder)
        "238380009",  # Gram-negative folliculitis (disorder)
        "238381008",  # Staphylococcal granuloma (disorder)
        "238392009",  # Sycosis cruris (disorder)
        "238409008",  # Infection of toe web (disorder)
        "238410003",  # Staphylococcal intertrigo (disorder)
        "238411004",  # Streptococcal intertrigo (disorder)
        "238414007",  # Staphylococcal botryomycosis (disorder)
        "238415008",  # Cutaneous botryomycosis (disorder)
        "238417000",  # Non-pyogenic bacterial infection of skin (disorder)
        "238418005",  # Erythrasma (disorder)
        "238419002",  # Gonococcal lymphangitis of penis (disorder)
        "238421007",  # Fusobacterium infection of skin (disorder)
        "238423005",  # Desert sore (disorder)
        "238424004",  # Arcanobacterium pyogenes infection of skin (disorder)
        "238425003",  # Meningococcal rash (disorder)
        "238426002",  # Moraxella infection of skin (disorder)
        "238435009",  # Steroid-modified tinea infection (disorder)
        "238436005",  # Cutaneous alternariosis (disorder)
        "238438006",  # Scytalidium hyalinum infection of skin (disorder)
        "238440001",  # Primary cutaneous blastomycosis (disorder)
        "238442009",  # Subcutaneous zygomycosis (disorder)
        "238447003",  # Nematode infestation of skin (disorder)
        "238448008",  # Streptocerciasis (disorder)
        "238449000",  # Amebiasis of skin (disorder)
        "238450000",  # Protozoal infestation of skin (disorder)
        "23874000",  # Chronic amebiasis (disorder)
        "238815002",  # Infectious mononucleosis ampicillin reaction (disorder)
        "238931006",  # Eosinophilic cellulitis (disorder)
        "238985006",  # Bullous pyoderma gangrenosum (disorder)
        "2390000",  # Acute gonococcal vulvovaginitis (disorder)
        "239187005",  # Skin graft infection (disorder)
        "239190004",  # Infection of skin donor site (disorder)
        "239297008",  # Lymphomatoid granulomatosis of lung (disorder)
        "23949002",  # Capillaria philippinensis infection (disorder)
        "23975003",  # Chronic gonococcal seminal vesiculitis (disorder)
        "239825002",  # Tabetic joint (disorder)
        "239940004",  # Lymphomatoid granulomatosis (disorder)
        "240039005",  # Gonococcal tenosynovitis (disorder)
        "240040007",  # Infective panniculitis (disorder)
        "240349003",  # Cholera caused by Vibrio cholerae O1 Classical biotype (disorder)
        "240350003",  # Cholera - non-O1 group vibrio (disorder)
        "240351004",  # Cholera - O139 group Vibrio cholerae (disorder)
        "240357000",  # Clostridial enteric disease (disorder)
        "240363009",  # Toxic megacolon due to amebic infection (disorder)
        "240365002",  # Intestinal flagellate infection (disorder)
        "240367005",  # Dientamoeba fragilis infection (disorder)
        "240368000",  # Retortamonas intestinalis infection (disorder)
        "240369008",  # Intestinal ciliate infection (disorder)
        "240370009",  # Cryptosporidiosis (disorder)
        "240371008",  # Chronic intestinal cryptosporidiasis (disorder)
        "240372001",  # Cyclosporiasis (disorder)
        "240374000",  # Chronic intestinal isosporiasis (disorder)
        "240375004",  # Intestinal microsporidiosis (disorder)
        "240387006",  # Pulmonary glanders (disorder)
        "240391001",  # Pulmonary melioidosis (disorder)
        "240394009",  # Cutaneous erysipeloid (disorder)
        "240395005",  # Localized cutaneous erysipeloid (disorder)
        "240396006",  # Disseminated cutaneous erysipeloid (disorder)
        "240397002",  # Cutaneous pasteurellosis (disorder)
        "240401005",  # Neural type tuberculoid leprosy (disorder)
        "240404002",  # Diffuse lepromatous leprosy (disorder)
        "240416008",  # Cutaneous mycobacterium marinum infection (disorder)
        "240417004",  # Fish tank granuloma (disorder)
        "240418009",  # Cutaneous mycobacterium haemophilium infection (disorder)
        "240419001",  # Hyperimmune cutaneous reaction caused by atypical mycobacteria (disorder)
        "240420007",  # Hyperimmune cutaneous reaction caused by mycobacterium avium-intracellulare (disorder)
        "240421006",  # Hyperimmune cutaneous reaction caused by mycobacterium kansasii (disorder)
        "240424003",  # Cutaneous streptococcal ulcer (disorder)
        "240425002",  # Facial erysipelas (disorder)
        "240435008",  # Gas gangrene of skin (disorder)
        "240438005",  # Primary cutaneous actinomycosis (disorder)
        "240449004",  # Cutaneous involvement legionellosis (disorder)
        "240452007",  # Asymptomatic bartonellosis (disorder)
        "240453002",  # Oroya fever (disorder)
        "240455009",  # Verruga peruana (disorder)
        "240456005",  # Miliary bartonellosis (disorder)
        "240457001",  # Nodular bartonellosis (disorder)
        "240458006",  # Neurobartonellosis (disorder)
        "240467006",  # Infection of skin of genitalia caused by Molluscum contagiosum (disorder)
        "240482001",  # Enteroviral exanthem (disorder)
        "240533004",  # Facial wart (disorder)
        "240534005",  # Hand wart (disorder)
        "240535006",  # Subungual wart (disorder)
        "240537003",  # Digitate wart (disorder)
        "240538008",  # Butcher's wart (disorder)
        "240539000",  # Plane wart (disorder)
        "240540003",  # Mosaic plantar wart (disorder)
        "240541004",  # Myrmecia plantar wart (disorder)
        "240542006",  # Anogenital warts (disorder)
        "240545008",  # Anogenital papillomaviral intraepithelial neoplasia (disorder)
        "240546009",  # Coxsackie virus exanthem (disorder)
        "240552005",  # Juvenile tabes dorsalis (disorder)
        "240554006",  # Hutchinson's triad (disorder)
        "240555007",  # Acquired syphilis (disorder)
        "240556008",  # Primary extragenital syphilis (disorder)
        "240557004",  # Secondary syphilis (disorder)
        "240558009",  # Macular syphilide (disorder)
        "240560006",  # Papular syphilide (disorder)
        "240561005",  # Corona veneris (disorder)
        "240562003",  # Pustular syphilide (disorder)
        "240563008",  # Syphilitic mucosal ulceration (disorder)
        "240564002",  # Secondary neurosyphilis (disorder)
        "240565001",  # Asymptomatic secondary neurosyphilis (disorder)
        "240566000",  # Gummatous neurosyphilis (disorder)
        "240567009",  # Syphilitic coronary artery disease (disorder)
        "240568004",  # Meningovascular syphilis - quaternary stage (disorder)
        "240569007",  # Syphilitic polyneuropathy (disorder)
        "240571007",  # Neonatal gonococcal infection (disorder)
        "240572000",  # Gonorrhea with local complication (disorder)
        "240573005",  # Gonococcal Bartholin's gland abscess (disorder)
        "240574004",  # Inflammation of paraurethral gland caused by Neisseria gonorrhoeae (disorder)
        "240575003",  # Gonococcal Tysonitis (disorder)
        "240576002",  # Inflammation of bulbourethral gland caused by Neisseria gonorrhoeae (disorder)
        "240577006",  # Inflammation of Littre gland caused by Neisseria gonorrhoeae (disorder)
        "240578001",  # Gonococcal Littre gland abscess (disorder)
        "240579009",  # Gonococcal paraurethral gland abscess (disorder)
        "240581006",  # Gonococcal female pelvic infection (disorder)
        "240582004",  # Gonococcal synovitis (disorder)
        "240583009",  # Cutaneous gonorrhea (disorder)
        "240584003",  # Gonococcal cellulitis (disorder)
        "240585002",  # Chancroid - anogenital ulcer (disorder)
        "240586001",  # Chancroid - extragenital ulcer (disorder)
        "240587005",  # Chancroid - latent infection (disorder)
        "240597001",  # Anal warts (disorder)
        "240598006",  # Penile warts (disorder)
        "240602008",  # Early lymphogranuloma venereum (disorder)
        "240603003",  # Late lymphogranuloma venereum (disorder)
        "240604009",  # Latent lymphogranuloma venereum (disorder)
        "240606006",  # Donovanosis - anogenital ulcer (disorder)
        "240608007",  # Donovanosis - non-genital lesion (disorder)
        "240624008",  # Bacillary peliosis caused by Bartonella (disorder)
        "240635003",  # Pneumonia caused by Leishmania (disorder)
        "240637006",  # Cutaneous leishmaniasis (disorder)
        "240638001",  # Leishmania recidiva (disorder)
        "240639009",  # Indian anthroponotic cutaneous leishmaniasis (disorder)
        "240641005",  # Classical zoonotic cutaneous leishmaniasis (disorder)
        "240642003",  # Middle Eastern zoonotic cutaneous leishmaniasis (disorder)
        "240643008",  # Indian zoonotic cutaneous leishmaniasis (disorder)
        "240644002",  # Sahelian zoonotic cutaneous leishmaniasis (disorder)
        "240645001",  # Diffuse cutaneous New World leishmaniasis (disorder)
        "240648004",  # Bush yaws (disorder)
        "240650007",  # Lupoid leishmaniasis (disorder)
        "240654003",  # Chagoma (disorder)
        "240656001",  # Chagas' exanthem (disorder)
        "240657005",  # Subcutaneous lipochagoma (disorder)
        "240659008",  # Chagas' mega disease (disorder)
        "240664007",  # Amebic abscess of skin (disorder)
        "240666009",  # Cutaneous toxoplasmosis (disorder)
        "240669002",  # Congenital Lyme disease (disorder)
        "240670001",  # Skeletal Lyme disease (disorder)
        "240671002",  # Lyme erosive synovitis (disorder)
        "240673004",  # Leptospiral rash (disorder)
        "240676007",  # Macular yaws (disorder)
        "240677003",  # Maculopapular yaws (disorder)
        "240678008",  # Micropapular yaws (disorder)
        "240679000",  # Papular yaws (disorder)
        "240685007",  # Mixed cutaneous pinta lesion (disorder)
        "240696004",  # Tinea faciei (disorder)
        "240700007",  # Fungal infection of hair (disorder)
        "240708000",  # Penile candidiasis (disorder)
        "240709008",  # Perineal candidiasis (disorder)
        "240710003",  # Granuloma gluteale infantum (disorder)
        "240711004",  # Diaper candidiasis (disorder)
        "240713001",  # Candidiasis of finger web (disorder)
        "240724004",  # Cutaneous cryptococcosis (disorder)
        "240727006",  # Cutaneous coccidioidomycosis (disorder)
        "240728001",  # Subcutaneous coccidioidomycosis (disorder)
        "240736005",  # Classical histoplasmosis intestinal ulceration (disorder)
        "240741002",  # Acute pulmonary African histoplasmosis (disorder)
        "240742009",  # Chronic pulmonary African histoplasmosis (disorder)
        "240744005",  # Cutaneous histoplasmosis (disorder)
        "240745006",  # Primary cutaneous histoplasmosis (disorder)
        "240746007",  # Disseminated cutaneous histoplasmosis (disorder)
        "240747003",  # Chronic pulmonary blastomycosis (disorder)
        "240748008",  # Disseminated cutaneous blastomycosis (disorder)
        "240750000",  # Primary lobomycotic nodule (disorder)
        "240751001",  # Chronic coalescent cutaneous lobomycosis (disorder)
        "240753003",  # Cutaneous rhinosporidiosis (disorder)
        "240755005",  # Polypoid cutaneous rhinosporidiosis (disorder)
        "240756006",  # Ulcerated cutaneous rhinosporidiosis (disorder)
        "240758007",  # Primary cutaneous sporotrichoid nodule (disorder)
        "240759004",  # Disseminated cutaneous sporotrichosis (disorder)
        "240765004",  # Cutaneous chromoblastomycosis (disorder)
        "240766003",  # Primary chromoblastomycotic nodule (disorder)
        "240767007",  # Cutaneous chromoblastomycotic abscess (disorder)
        "240768002",  # Lymphocutaneous chromoblastomycosis (disorder)
        "240769005",  # Carcinomatous transformation of chromoblastomycosis (disorder)
        "240770006",  # Phaeohyphomycosis of skin (disorder)
        "240771005",  # Cyst of subcutaneous tissue caused by Dematiaceae (disorder)
        "240774002",  # Cutaneous hyalohyphomycosis (disorder)
        "240776000",  # Subcutaneous pseudallescheriosis (disorder)
        "240778004",  # Cutaneous mucormycosis (disorder)
        "240779007",  # Necrotizing cutaneous mucormycosis (disorder)
        "240786004",  # Cutaneous basidiobolomycosis (disorder)
        "240796008",  # Intestinal schistosomiasis (disorder)
        "240802005",  # Intestinal fluke infection (disorder)
        "240803000",  # Haplorchiasis (disorder)
        "240804006",  # Fibricoliasis (disorder)
        "240806008",  # Phaneropsuliasis (disorder)
        "240813008",  # Cutaneous fluke infection (disorder)
        "240814002",  # Poikilorchiasis (disorder)
        "240835000",  # Subcutaneous filariasis (disorder)
        "240836004",  # Onchodermatitis (disorder)
        "240838003",  # Onchocercal subcutaneous nodule (disorder)
        "240839006",  # Onchocercal depigmentation (disorder)
        "240840008",  # Onchocercal lichenification (disorder)
        "240841007",  # Sowda (disorder)
        "240843005",  # Calabar swelling (disorder)
        "240847006",  # Guinea-worm ulcer (disorder)
        "240851008",  # Subcutaneous dirofilariasis (disorder)
        "240854000",  # Ancylostomal cutaneous larva migrans (disorder)
        "240855004",  # Strongyloides cutaneous larva currens (disorder)
        "240856003",  # Strongyloidal ground itch (disorder)
        "240857007",  # Strongyloidal cutaneous larva migrans (disorder)
        "240860000",  # Gnathostomal cutaneous larva migrans (disorder)
        "240861001",  # Larval ascariasis (disorder)
        "240863003",  # Migratory ascariasis (disorder)
        "240864009",  # Biliary ascariasis (disorder)
        "240865005",  # Ascaris peritoneal granuloma (disorder)
        "240869004",  # Capillaria hepatica hepatitis (disorder)
        "240870003",  # Capillaria aerophila infection (disorder)
        "240871004",  # Capillaria aerophila chest infection (disorder)
        "240872006",  # Ternideniasis (disorder)
        "240877000",  # Infestation of skin caused by fly larvae (disorder)
        "240879002",  # Infestation of subcutaneous tissue caused by fly larvae (disorder)
        "240894003",  # Demodex folliculitis (disorder)
        "240897005",  # Dermatitis due to Cheyletiella infestation (disorder)
        "24347001",  # Cellulitis of vocal cords (disorder)
        "2435008",  # Ascariasis (disorder)
        "24620004",  # Hemolytic anemia due to babesiosis (disorder)
        "24752008",  # Infantile cortical hyperostosis (disorder)
        "24789006",  # Viral gastroenteritis caused by Norwalk-like agent (disorder)
        "24868007",  # Acute gonococcal cystitis (disorder)
        "249457005",  # Supraglottic ulcer (disorder)
        "249458000",  # Supraglottic abscess (disorder)
        "249463001",  # Subglottic abscess (disorder)
        "249464007",  # Subglottic ulcer (disorder)
        "249465008",  # Subglottic necrosis (disorder)
        "2513003",  # Tinea capitis caused by Trichophyton (disorder)
        "25188002",  # Pediculosis corporis (disorder)
        "2523007",  # Salmonella pneumonia (disorder)
        "254044004",  # Multiple congenital exostosis (disorder)
        "254147009",  # Idiopathic osteolyses (disorder)
        "254149007",  # Carpal-tarsal osteolysis with nephropathy (disorder)
        "254151006",  # Winchester syndrome (disorder)
        "254152004",  # Torg type osteolysis (disorder)
        "254153009",  # Familial expansile osteolysis (disorder)
        "254671003",  # Infection of sebaceous cyst (disorder)
        "254676008",  # Warty dyskeratoma (disorder)
        "254899009",  # Cutaneous glanders (disorder)
        "25626000",  # Cutaneous infectious disease caused by Mycobacteria (disorder)
        "25726008",  # Nonvenomous insect bite of groin with infection (disorder)
        "25956006",  # Tinea pedis caused by Trichophyton (disorder)
        "26039008",  # Neurosyphilis (disorder)
        "26067004",  # Madura foot caused by Streptomyces (disorder)
        "26135000",  # Syphilitic encephalitis (disorder)
        "26151008",  # Graves' disease with acropachy AND with thyrotoxic crisis (disorder)
        "2622007",  # Infected ulcer of skin (disorder)
        "262461007",  # Infection caused by Bartonella bacilliformis (disorder)
        "26249004",  # Intestinal infection caused by Cestoda and/or Trematoda and/or Phylum Nemata (disorder)
        "262781009",  # Kaposi's varicelliform eruption caused by vaccinia virus (disorder)
        "26535009",  # Carbuncle of vulva (disorder)
        "266071000",  # Intestinal infectious disease (disorder)
        "266077001",  # Staphylococcal gastrointestinal tract infection (disorder)
        "266090008",  # Ileocecal actinomycosis (disorder)
        "266108008",  # Enteroviral vesicular stomatitis with exanthem (disorder)
        "266113007",  # Genital warts (disorder)
        "266114001",  # Perianal warts (disorder)
        "266123003",  # Bartonellosis (disorder)
        "266125005",  # Early congenital syphilis with symptoms (disorder)
        "266126006",  # Late congenital neurosyphilis (disorder)
        "266127002",  # Primary syphilis (disorder)
        "266128007",  # Rash of secondary syphilis (disorder)
        "266130009",  # Acquired syphilis - early latent (disorder)
        "266133006",  # Late quaternary neurosyphilis (disorder)
        "266136003",  # Acquired syphilis - late latent (disorder)
        "266143009",  # Chancroid (disorder)
        "266152000",  # Deep seated dermatophytosis (disorder)
        "266157006",  # Perianal candidiasis (disorder)
        "266158001",  # Candida intertrigo (disorder)
        "266160004",  # Intestinal taenia solium infection (disorder)
        "266162007",  # Enterobiasis (disorder)
        "266350000",  # Pneumococcal lobar pneumonia (disorder)
        "266404004",  # Chronic chemical respiratory disease (disorder)
        "26652002",  # Abrasion and/or friction burn of scalp with infection (disorder)
        "267665002",  # Acute infective otitis externa (disorder)
        "26826005",  # Amebic appendicitis (disorder)
        "268276007",  # Congenital exostosis (disorder)
        "269202001",  # Abrasion and/or friction burn of foot and/or toe, infected (disorder)
        "269206003",  # Insect bite, nonvenomous, of foot and toe, infected (disorder)
        "27023008",  # Infective dermatitis of eyelid (disorder)
        "270550006",  # Pelvic inflammatory disease caused by Metamycoplasma hominis (disorder)
        "27075004",  # Venereal disease in mother complicating pregnancy, childbirth AND/OR puerperium (disorder)
        "271111006",  # Traumatic blister of head, infected (disorder)
        "271112004",  # Traumatic blister of face, infected (disorder)
        "271113009",  # Traumatic blister of neck, infected (disorder)
        "271114003",  # Traumatic blister of scalp, infected (disorder)
        "271132003",  # Traumatic blister of interscapular region, infected (disorder)
        "271133008",  # Traumatic blister of chest wall, infected (disorder)
        "271134002",  # Traumatic blister of breast, infected (disorder)
        "271135001",  # Traumatic blister of abdominal wall, infected (disorder)
        "271136000",  # Traumatic blister of back, infected (disorder)
        "271138004",  # Traumatic blister of buttock, infected (disorder)
        "271139007",  # Traumatic blister of anus, infected (disorder)
        "271140009",  # Traumatic blister of flank, infected (disorder)
        "271141008",  # Traumatic blister of groin, infected (disorder)
        "271142001",  # Traumatic blister of perineum, infected (disorder)
        "271143006",  # Traumatic blister of penis, infected (disorder)
        "271153007",  # Traumatic blister of shoulder and upper arm, infected (disorder)
        "271154001",  # Traumatic blister of shoulder, infected (disorder)
        "271155000",  # Traumatic blister of scapular region, infected (disorder)
        "271156004",  # Traumatic blister of axilla with infection (disorder)
        "271157008",  # Traumatic blister of upper arm, infected (disorder)
        "271165006",  # Traumatic blister of elbow, infected (disorder)
        "271166007",  # Traumatic blister of forearm, infected (disorder)
        "271167003",  # Traumatic blister of wrist, infected (disorder)
        "271170004",  # Traumatic blister of hand, infected (disorder)
        "271172007",  # Traumatic blister of finger, infected (disorder)
        "271180000",  # Traumatic blister of lower limb, infected (disorder)
        "271181001",  # Traumatic blister of hip, infected (disorder)
        "271182008",  # Traumatic blister of thigh, infected (disorder)
        "271183003",  # Traumatic blister of knee, infected (disorder)
        "271184009",  # Traumatic blister of lower leg, infected (disorder)
        "271185005",  # Traumatic blister of ankle, infected (disorder)
        "271191007",  # Traumatic blister of foot, infected (disorder)
        "271192000",  # Traumatic blister of toe, infected (disorder)
        "271449003",  # Multiple yaws papillomata (disorder)
        "271488001",  # Mycoplasmal pelvic inflammatory disease (disorder)
        "272006008",  # Gonococcal arthritis dermatitis syndrome (disorder)
        "272262003",  # Esthiomene (disorder)
        "27278006",  # Acute empyema of sphenoidal sinus (disorder)
        "274081004",  # Bacillary dysentery (disorder)
        "274109003",  # Acute recurrent cystitis (disorder)
        "274118001",  # Venereal disease in pregnancy (disorder)
        "27460003",  # Primary syphilis of breast (disorder)
        "275376007",  # Congenital syphilitic chronic coryza (disorder)
        "275410008",  # Infected diverticulum of urinary bladder (disorder)
        "275412000",  # Cystitis of pregnancy (disorder)
        "275441009",  # Superficial bacterial infection of skin (disorder)
        "275443007",  # Traumatic blister of scrotum, infected (disorder)
        "275444001",  # Traumatic blister of heel, infected (disorder)
        "27648007",  # Congenital syphilitic periostitis (disorder)
        "276676005",  # Perinatal skin and subcutaneous infections of fetus and/or neonate (disorder)
        "276693005",  # Congenital bacterial pneumonia (disorder)
        "276695003",  # Neonatal aspiration pneumonia (disorder)
        "276700005",  # Congenital syphilitic rhinitis (disorder)
        "27681008",  # Chronic gonorrhea (disorder)
        "276877003",  # Trichomonal vaginitis (disorder)
        "277499006",  # Complicated donovanosis (disorder)
        "277869007",  # Non-tuberculous mycobacterial pneumonia (disorder)
        "278039004",  # Cutaneous African histoplasmosis (disorder)
        "27836007",  # Pertussis (disorder)
        "278480000",  # Syphilitic endocarditis of aortic valve (disorder)
        "278481001",  # Quaternary syphilis (disorder)
        "278516003",  # Lobar pneumonia (disorder)
        "27858009",  # Clostridial gastroenteritis (disorder)
        "278834008",  # Idiopathic multicentric osteolysis (disorder)
        "279324004",  # Site-specific infective disorders of skin (disorder)
        "279325003",  # Submammary monilia (disorder)
        "279499002",  # Infective vulvitis (disorder)
        "280221000009107",  # Otitis externa caused by bacteria (disorder)
        "280919009",  # Cutaneous disorders of Yaws (disorder)
        "281801000009108",  # Prostatitis caused by bacteria (disorder)
        "28198007",  # Primary syphilis of lip (disorder)
        "283347003",  # Infected insect bite of upper limb (disorder)
        "283348008",  # Infected insect bite of dorsum of hand (disorder)
        "283349000",  # Infected insect bite of palm of hand (disorder)
        "283350000",  # Infected insect bite of thumb (disorder)
        "283352008",  # Infected insect bite of genitalia (disorder)
        "283354009",  # Infected insect bite of sole of foot (disorder)
        "283876006",  # Astrovirus gastroenteritis (disorder)
        "283877002",  # Calicivirus gastroenteritis (disorder)
        "28438004",  # Gonococcal conjunctivitis neonatorum (disorder)
        "28572009",  # Chronic gonorrhea of genitourinary tract (disorder)
        "28709001",  # Cellulitis of larynx (disorder)
        "28791000119105",  # Chronic coccidioidomycotic pneumonia (disorder)
        "28867007",  # Granuloma inguinale (disorder)
        "28951000",  # Infection caused by Trichuris suis (disorder)
        "290051000119103",  # Recurrent acute suppurative otitis media of bilateral middle ears (disorder)
        "292891000119104",  # Intracranial granuloma (disorder)
        "292901000119100",  # Intracranial subdural granuloma (disorder)
        "292911000119102",  # Intracranial epidural granuloma (disorder)
        "292921000119109",  # Intraspinal granuloma (disorder)
        "292931000119107",  # Intraspinal subdural granuloma (disorder)
        "292941000119103",  # Intraspinal epidural granuloma (disorder)
        "29604006",  # Infection caused by Trichostrongylus colubriformis (disorder)
        "298128008",  # Perineal wart (disorder)
        "298129000",  # Female perineal wart (disorder)
        "29864006",  # Acute inflammation of urethra caused by Neisseria gonorrhoeae (disorder)
        "29979000",  # Infection caused by Enteromonas hominis (disorder)
        "300927001",  # Episiotomy infection (disorder)
        "300931007",  # Peritonsillar cyst (disorder)
        "300945001",  # Infected heel (disorder)
        "300999006",  # Basal pneumonia (disorder)
        "301000005",  # Left lower zone pneumonia (disorder)
        "301001009",  # Right lower zone pneumonia (disorder)
        "301002002",  # Left upper zone pneumonia (disorder)
        "301003007",  # Right middle zone pneumonia (disorder)
        "301004001",  # Right upper zone pneumonia (disorder)
        "301086002",  # Syphilitic meningitis (disorder)
        "30168008",  # Acute gonococcal epididymo-orchitis (disorder)
        "301990003",  # Gonococcal seminal vesiculitis (disorder)
        "30242009",  # Scarlet fever (disorder)
        "302812006",  # Anogenital human papillomavirus infection (disorder)
        "302813001",  # Syphilitic acoustic neuritis - quaternary stage (disorder)
        "302814007",  # Early yaws skin lesion (disorder)
        "30283007",  # Initial frambesial ulcer (disorder)
        "302905001",  # Aspergillus otomycosis (disorder)
        "303131000119109",  # Calcaneal spur of left foot (disorder)
        "303141000119100",  # Calcaneal spur of right foot (disorder)
        "30358009",  # Acquired stenosis of external ear canal secondary to surgery (disorder)
        "303672007",  # Infection caused by Physaloptera caucasica (disorder)
        "304552000",  # Infection caused by Physaloptera mordens (disorder)
        "304801000119107",  # Complete epiphyseal arrest of left distal radius (disorder)
        "304871000119102",  # Complete epiphyseal arrest of right distal radius (disorder)
        "30493003",  # Intestinal infection caused by Proteus mirabilis (disorder)
        "30595008",  # Acute necrotizing cutaneous leishmaniasis (disorder)
        "30716008",  # Infection caused by Oesophagostomum columbianum (disorder)
        "307423008",  # Gonococcal pelvic peritonitis (disorder)
        "307426000",  # Acute infective cystitis (disorder)
        "308119005",  # Infantile viral gastroenteritis (disorder)
        "30818003",  # Nonvenomous insect bite of perineum with infection (disorder)
        "308906005",  # Secondary bacterial pneumonia (disorder)
        "309734000",  # Cyst of tonsil (disorder)
        "31015008",  # Primary anal syphilis (disorder)
        "31137003",  # Early latent syphilis, positive serology, negative cerebrospinal fluid, less than 2 years after infection (disorder)
        "31201001",  # Knight's disease (disorder)
        "312088007",  # Fungal gastroenteritis (disorder)
        "312106007",  # Toxocara larva migrans of skin (disorder)
        "312108008",  # Bacterial intestinal infectious disease (disorder)
        "312109000",  # Bacterial dysentery (disorder)
        "312221005",  # Infective ureteritis (disorder)
        "312934004",  # Syphilitic chorioretinitis (disorder)
        "312955002",  # Tertiary syphilitic chorioretinitis (disorder)
        "314840009",  # Progressive locomotor ataxia (disorder)
        "31487001",  # Disseminated idiopathic skeletal hyperostosis (disorder)
        "314978007",  # Postoperative pneumonia (disorder)
        "315551000119105",  # Osteolysis of left hand (disorder)
        "315631000119100",  # Osteolysis of right hand (disorder)
        "315826004",  # Tabetic neurosyphilis (disorder)
        "31604004",  # Blister of flank with infection (disorder)
        "317371000119104",  # Partial epiphyseal arrest of left distal femur (disorder)
        "317381000119101",  # Partial epiphyseal arrest of left distal humerus (disorder)
        "317421000119105",  # Partial epiphyseal arrest of left proximal humerus (disorder)
        "317431000119108",  # Partial epiphyseal arrest of left proximal tibia (disorder)
        "317441000119104",  # Partial epiphyseal arrest of right distal femur (disorder)
        "317451000119102",  # Partial epiphyseal arrest of right distal humerus (disorder)
        "317491000119107",  # Partial epiphyseal arrest of right proximal humerus (disorder)
        "317501000119100",  # Partial epiphyseal arrest of right proximal tibia (disorder)
        "31772005",  # Cicatricial skin lesions of pinta (disorder)
        "31999004",  # Chronic gonococcal endometritis (disorder)
        "32130001000004108",  # Spur of ankle bone (disorder)
        "32204007",  # Pulmonary actinobacillosis (disorder)
        "32286006",  # Pneumonia in Q fever (disorder)
        "32298001",  # Intestinal infection caused by Trichomonas vaginalis (disorder)
        "32527003",  # Staphylococcal enterocolitis (disorder)
        "32580004",  # Enterovirus enteritis (disorder)
        "3261002",  # Migratory osteolysis (disorder)
        "32735002",  # Congenital syphilitic encephalitis (disorder)
        "33042004",  # Parasitic skin infestation (disorder)
        "33090006",  # Yabapox (disorder)
        "331151000119109",  # Syphilitic choroiditis (disorder)
        "33155001",  # Nonvenomous insect bite of flank with infection (disorder)
        "33261009",  # Abscess of tonsil (disorder)
        "33655002",  # Chronic cystitis (disorder)
        "33666009",  # Black piedra (disorder)
        "33710003",  # Trichostrongyliasis (disorder)
        "33924006",  # Abscess of larynx (disorder)
        "33934002",  # Hemorrhagic otitis externa (disorder)
        "33937009",  # Lyme arthritis (disorder)
        "34020007",  # Pneumonia caused by Streptococcus (disorder)
        "342381000119109",  # Gonococcal iritis (disorder)
        "34240005",  # Infection caused by Diphyllobothrium pacificum (disorder)
        "34403004",  # Blister of buttock with infection (disorder)
        "34441006",  # Blister of hand with infection (disorder)
        "3464007",  # Infection caused by Oesophagostomum dentatum (disorder)
        "34730008",  # Primate erythroparvovirus 1 infection (disorder)
        "34786008",  # Candidal proctitis (disorder)
        "3487004",  # Candidiasis of lung (disorder)
        "35031000119100",  # Acute aspiration pneumonia (disorder)
        "35057008",  # Nonvenomous insect bite of penis with infection (disorder)
        "35089004",  # Urogenital infection caused by Trichomonas vaginalis (disorder)
        "35168006",  # Acute empyema of ethmoidal sinus (disorder)
        "35339003",  # Primary pneumonic plague (disorder)
        "35526001",  # Chronic gonococcal epididymo-orchitis (disorder)
        "35586003",  # White piedra (disorder)
        "35742006",  # Congenital syphilis (disorder)
        "35876006",  # Gonococcal infection of eye (disorder)
        "3589003",  # Syphilitic pericarditis (disorder)
        "35923002",  # Chronic maxillary sinusitis (disorder)
        "359613008",  # Acute infectious nonbacterial gastroenteritis (disorder)
        "35974005",  # Cryptococcal gastroenteritis (disorder)
        "359795009",  # Congo floor maggot disease (disorder)
        "359799003",  # Intestinal diphyllobothriasis (disorder)
        "36001007",  # Capillaria hepatica infection (disorder)
        "36038007",  # Goundou of yaws (disorder)
        "360425009",  # Human coccidiosis (disorder)
        "36139001",  # Chronic ulcerating interstitial cystitis (disorder)
        "36188001",  # Shigellosis (disorder)
        "36276008",  # Syphilitic retrobulbar neuritis (disorder)
        "36292003",  # Otitis externa caused by Herpes zoster (disorder)
        "362990007",  # Auditory system complication of procedure (disorder)
        "36334003",  # Hymenolepis diminuta infection (disorder)
        "36529003",  # Intestinal infection caused by Morganella morganii (disorder)
        "36689008",  # Acute pyelonephritis (disorder)
        "366941000119107",  # Hematuria co-occurrent and due to acute cystitis (disorder)
        "367171000119100",  # Hematuria co-occurrent and due to cystitis (disorder)
        "367338000",  # Infective dermatitis (disorder)
        "367661000119102",  # Hematuria co-occurrent and due to chronic interstitial cystitis (disorder)
        "36789003",  # Acute infective gastroenteritis (disorder)
        "368371000119102",  # Hematuria co-occurrent and due to trigonitis (disorder)
        "36866003",  # Mucocutaneous-lymphangitic paracoccidioidomycosis (disorder)
        "368971000119101",  # Hematuria co-occurrent and due to chronic cystitis (disorder)
        "370518000",  # Enteric coccidiosis (disorder)
        "370519008",  # Cutaneous pythiosis (disorder)
        "371072008",  # Postobstructive pneumonia (disorder)
        "371237000",  # Syphilitic skin disorder (disorder)
        "371310004",  # Streptococcal skin disorder (disorder)
        "371423007",  # Isosporiasis (disorder)
        "371521007",  # Disorder of bone development (disorder)
        "37160003",  # Pintid (disorder)
        "373611003",  # Pyocystis (disorder)
        "37426002",  # Ulcerative laryngitis (disorder)
        "37430004",  # Syphilis of synovium (disorder)
        "3752003",  # Trichuriasis (disorder)
        "37702000",  # Hereditary acrodermatitis enteropathica (disorder)
        "37722001",  # Ulceroglandular tularemia (disorder)
        "37754005",  # Asymptomatic neurosyphilis (disorder)
        "37776001",  # Abrasion and/or friction burn of perineum with infection (disorder)
        "37779008",  # Pyeloureteritis cystica (disorder)
        "37832003",  # Infection caused by Metagonimus yokogawai (disorder)
        "37887002",  # Nonvenomous insect bite of neck with infection (disorder)
        "38189003",  # Nonvenomous insect bite of ankle with infection (disorder)
        "38302000",  # Infection caused by Echinostoma lindoense (disorder)
        "3841004",  # Blister of cheek with infection (disorder)
        "385093006",  # Community acquired pneumonia (disorder)
        "38523005",  # Syphilitic parkinsonism (disorder)
        "38573008",  # Diffuse cutaneous leishmaniasis (disorder)
        "386034005",  # Acute mastoiditis (disorder)
        "38699009",  # Pneumonia caused by Histoplasma capsulatum (disorder)
        "38700005",  # Madura foot caused by Actinomadura pelletieri (disorder)
        "387754006",  # Amebic dysentery (disorder)
        "38822007",  # Cystitis (disorder)
        "389274008",  # Whyte Hemingway carpal tarsal phalangeal osteolyses (disorder)
        "38961000",  # Chronic sphenoidal sinusitis (disorder)
        "38976008",  # Pneumonic plague (disorder)
        "39084003",  # Tinea cruris caused by Epidermophyton floccosum (disorder)
        "39085002",  # Secondary syphilis of mucous membrane (disorder)
        "39172002",  # Pneumonia caused by Proteus mirabilis (disorder)
        "39341005",  # Infectious colitis (disorder)
        "39439008",  # Wet form of cutaneous leishmaniasis (disorder)
        "39524009",  # Cercarial dermatitis of freshwater mammalian type (disorder)
        "396285007",  # Bronchopneumonia (disorder)
        "396286008",  # Bilateral bronchopneumonia (disorder)
        "39629007",  # Pyogenic granuloma of skin (disorder)
        "397176006",  # Splinter in skin with infection (disorder)
        "39747007",  # Enteritis necroticans (disorder)
        "397515005",  # Blepharoconjunctivitis caused by Molluscum contagiosum (disorder)
        "397552005",  # Vaccinia keratitis (disorder)
        "397810006",  # Pyosalpinx (disorder)
        "398570005",  # Diarrhea caused by staphylococcus toxin (disorder)
        "398679002",  # Bowenoid papulosis of penis (disorder)
        "399008001",  # Deep bacterial folliculitis (disorder)
        "399029005",  # Tinea cruris (disorder)
        "399172001",  # Tinea of perianal region (disorder)
        "399324007",  # Bacterial folliculitis (disorder)
        "399329002",  # Tinea barbae (disorder)
        "399599006",  # Condyloma acuminata of vagina (disorder)
        "39963006",  # Toddler diarrhea (disorder)
        "399911003",  # Bowenoid papulosis of vulva with vulval intraepithelial neoplasia grade III (disorder)
        "399929008",  # Bowenoid papulosis of anus with anal intraepithelial neoplasia grade III (disorder)
        "399990001",  # Bowenoid papulosis of vulva (disorder)
        "400079002",  # Infected eczema (disorder)
        "400086005",  # Amebic ulcer of skin (disorder)
        "400118002",  # Bowenoid papulosis of penis with penile intraepithelial neoplasia grade III (disorder)
        "400213003",  # Blister of skin with infection (disorder)
        "40055000",  # Chronic sinusitis (disorder)
        "40070004",  # Infection caused by Molluscum contagiosum (disorder)
        "40149008",  # Gonococcal keratitis (disorder)
        "4016003",  # Empyema of mastoid (disorder)
        "402122002",  # Cytomegalovirus infection of skin (disorder)
        "402126004",  # Sexually transmitted bacterial disease affecting skin (disorder)
        "402127008",  # Brucellosis of skin (disorder)
        "402128003",  # Cutaneous listeriosis (disorder)
        "402129006",  # Actinomycetoma caused by Nocardia (disorder)
        "402131002",  # Disseminated cutaneous candidosis (disorder)
        "402132009",  # Systemic candidosis with skin involvement (disorder)
        "402133004",  # Malassezia infection of skin (disorder)
        "402138008",  # Disseminated cutaneous mycosis (disorder)
        "402139000",  # Dermatosis caused by parasite (disorder)
        "402140003",  # Infestation with human nematode (disorder)
        "402141004",  # American cutaneous/mucocutaneous Leishmaniasis (disorder)
        "402142006",  # Trypanosomiasis affecting skin (disorder)
        "402143001",  # Trichomoniasis affecting skin (disorder)
        "402145008",  # Dermatosis caused by Diptera (disorder)
        "402215004",  # Infected discoid eczema (disorder)
        "402420001",  # Human herpes virus 6 infection of skin (disorder)
        "402565007",  # Human herpes virus 7 infection of skin (disorder)
        "402702009",  # Infective dermatosis of lip (disorder)
        "402705006",  # Infective dermatosis of male genitalia (disorder)
        "402706007",  # Infective dermatosis of female genitalia (disorder)
        "402709000",  # Infective dermatosis of perianal skin (disorder)
        "402796000",  # Neonatal viral infection of skin (disorder)
        "402798004",  # Neonatal fungal infection of skin (disorder)
        "402857001",  # Human herpes virus 8 infection of skin (disorder)
        "402884000",  # Vaccination site vaccinia (disorder)
        "402902002",  # Roseola infantum caused by human herpesvirus 6 (disorder)
        "402903007",  # Roseola infantum caused by human herpesvirus 7 (disorder)
        "402904001",  # Viral wart on finger (disorder)
        "402905000",  # Viral wart on nose (disorder)
        "402907008",  # Beard wart (disorder)
        "402909006",  # Viral wart on toe (disorder)
        "402910001",  # Anogenital verrucous carcinoma of Buschke-Lowenstein (disorder)
        "402911002",  # Penile verrucous carcinoma of Buschke-Lowenstein (disorder)
        "402912009",  # Vulval verrucous carcinoma of Buschke-Lowenstein (disorder)
        "402913004",  # Bowenoid papulosis (disorder)
        "402921005",  # Pseudomonas aeruginosa folliculitis (disorder)
        "402922003",  # Propionibacterium acnes folliculitis (disorder)
        "402924002",  # Recurrent erysipelas (disorder)
        "402925001",  # Erysipelas of lower limb (disorder)
        "402934006",  # Streptococcal cellulitis of external ear (disorder)
        "402938009",  # Staphylococcal infection of skin (disorder)
        "402940004",  # Syphilitic chancre of penis (disorder)
        "402941000",  # Syphilitic chancre of vulva (disorder)
        "402942007",  # Syphilitic chancre of oral mucous membranes (disorder)
        "402943002",  # Syphilitic balanitis (disorder)
        "402944008",  # Condylomata lata of perianal skin (disorder)
        "402945009",  # Condylomata lata of penis (disorder)
        "402946005",  # Condylomata lata of vulva (disorder)
        "402947001",  # Late secondary syphilis (disorder)
        "402949003",  # Nodular syphilide (disorder)
        "402950003",  # Syphilitic leukoplakia of tongue (disorder)
        "402951004",  # Oral mucous membrane lesion due to late syphilis (disorder)
        "402952006",  # Anetoderma secondary to syphilis (disorder)
        "402953001",  # Generalized cutaneous pinta (disorder)
        "402954007",  # Penile chancroid (disorder)
        "402955008",  # Vulval chancroid (disorder)
        "402956009",  # Localized cutaneous gonococcal infection (disorder)
        "402957000",  # Gonococcal bartholinitis (disorder)
        "402958005",  # Pustular vasculitis due to gonococcal bacteremia (disorder)
        "402959002",  # Leptospirosis with cutaneous involvement (disorder)
        "402968000",  # Erythema nodosum due to bacterial infection (disorder)
        "402969008",  # Erythema nodosum due to streptococcal infection (disorder)
        "402970009",  # Erythema nodosum due to Yersinia enterocolitica (disorder)
        "402976003",  # Mycobacterium kansasii infection of skin (disorder)
        "402977007",  # Mycobacterium scrofulaceum infection of skin (disorder)
        "402978002",  # Mycobacterium szulgai infection of skin (disorder)
        "402979005",  # Mycobacterium avium intracellulare infection of skin (disorder)
        "402980008",  # Mycobacterium fortuitum infection of skin (disorder)
        "402981007",  # Mycobacteroides chelonae infection of skin (disorder)
        "402982000",  # Mycobacterium gordonae infection of skin (disorder)
        "402983005",  # Mycobacterium malmoense infection of skin (disorder)
        "402984004",  # Mycobacteroides abscessus infection of skin (disorder)
        "402985003",  # Actinomycetoma caused by Nocardia brasiliensis (disorder)
        "402986002",  # Actinomycetoma caused by Nocardia asteroides (disorder)
        "402987006",  # Actinomycetoma caused by Nocardia otitidis caviarum (disorder)
        "402989009",  # Primary cutaneous nocardiosis (disorder)
        "402990000",  # Disseminated nocardiosis infection affecting skin (disorder)
        "402992008",  # Disseminated actinomycosis infection affecting skin (disorder)
        "402993003",  # Actinomycetoma caused by Actinomadura madurae (disorder)
        "402994009",  # Actinomycetoma caused by Actinomadura pelletieri (disorder)
        "402995005",  # Actinomycetoma caused by Streptomyces somaliensis (disorder)
        "402999004",  # Nodular candidiasis of diaper area (disorder)
        "403002006",  # Tinea capitis caused by Microsporum canis variant canis (disorder)
        "403003001",  # Tinea capitis caused by Microsporum canis variant distortum (disorder)
        "403004007",  # Tinea capitis caused by Microsporum gallinae (disorder)
        "403005008",  # Tinea capitis caused by Trichophyton gourvilii (disorder)
        "403006009",  # Tinea capitis caused by Microsporum equinum (disorder)
        "403007000",  # Tinea capitis caused by Microsporum ferrugineum (disorder)
        "403008005",  # Tinea capitis caused by Nannizzia fulva (disorder)
        "403009002",  # Tinea capitis caused by Trichophyton megninii (disorder)
        "403010007",  # Tinea capitis caused by Microsporum gypseum (disorder)
        "403012004",  # Tinea capitis caused by Microsporum nanum (disorder)
        "403013009",  # Tinea capitis caused by Trichophyton mentagrophytes variant erinacei (disorder)
        "403014003",  # Tinea capitis caused by Trichophyton mentagrophytes (disorder)
        "403015002",  # Tinea capitis caused by Trichophyton rubrum (disorder)
        "403016001",  # Tinea capitis caused by Trichophyton soudanense (disorder)
        "403017005",  # Tinea capitis caused by Trichophyton verrucosum (disorder)
        "403018000",  # Tinea capitis caused by Trichophyton violaceum (disorder)
        "403019008",  # Tinea capitis caused by Trichophyton yaoundei (disorder)
        "403021003",  # Tinea capitis caused by Microsporum audouinii (disorder)
        "403047007",  # Tinea manuum caused by Epidermophyton floccosum (disorder)
        "403048002",  # Tinea manuum caused by Trichophyton mentagrophytes variant interdigitale (disorder)
        "403049005",  # Tinea manuum caused by Trichophyton mentagrophytes variant erinacei (disorder)
        "403050005",  # Tinea manuum caused by Trichophyton rubrum (disorder)
        "403051009",  # Tinea manuum caused by Microsporum vanbreuseghemii (disorder)
        "403052002",  # Tinea manuum caused by Trichophyton violaceum (disorder)
        "403053007",  # Tinea cruris caused by Trichophyton gallinae (disorder)
        "403054001",  # Tinea pedis caused by Trichophyton mentagrophytes variant interdigitale (disorder)
        "403055000",  # Tinea pedis caused by Trichophyton rubrum (disorder)
        "403056004",  # Tinea pedis caused by Trichophyton violaceum (disorder)
        "403059006",  # Onychomycosis of toenails (disorder)
        "403089004",  # Tinea nigra caused by Phaeoannellomyces werneckii (disorder)
        "403090008",  # Tinea nigra caused by Stenella araguata (disorder)
        "403091007",  # Piedra caused by Trichosporon beigelii (disorder)
        "403104005",  # Distal and lateral subungual onychomycosis (disorder)
        "403105006",  # Proximal subungual onychomycosis (disorder)
        "403110005",  # Disseminated cutaneous coccidioidomycosis (disorder)
        "403111009",  # Fixed cutaneous sporotrichosis (disorder)
        "403112002",  # Mycetoma caused by Acremonium (disorder)
        "403113007",  # Mycetoma caused by Curvularia lunata (disorder)
        "403114001",  # Mycetoma caused by dermatophyte (disorder)
        "403115000",  # Mycetoma caused by Exophiala jeanselmei (disorder)
        "403116004",  # Mycetoma caused by Fusarium (disorder)
        "403117008",  # Mycetoma caused by Leptosphaeria senegalensis (disorder)
        "403118003",  # Mycetoma caused by Madurella grisea (disorder)
        "403119006",  # Mycetoma caused by Madurella mycetomatis (disorder)
        "403120000",  # Mycetoma caused by Neotestudina rosatii (disorder)
        "403121001",  # Mycetoma caused by Pseudallescheria boydii (disorder)
        "403122008",  # Mycetoma caused by Pyrenochaeta romeroi (disorder)
        "403123003",  # Disseminated cutaneous mycosis caused by Penicillium (disorder)
        "403124009",  # Disseminated cutaneous mycosis caused by Fusarium (disorder)
        "403125005",  # Disseminated cutaneous mycosis caused by Trichosporon (disorder)
        "403126006",  # Acute papular onchodermatitis (disorder)
        "403127002",  # Chronic papular onchodermatitis (disorder)
        "403128007",  # Atrophic onchodermatitis (disorder)
        "403131008",  # Echinococcosis of skin (disorder)
        "403132001",  # Cysticercosis of skin (disorder)
        "403133006",  # Sparganosis of skin (disorder)
        "403134000",  # Dermatosis caused by tapeworm (disorder)
        "403136003",  # Leishmania brasiliensis complex cutaneous leishmaniasis (disorder)
        "403138002",  # African trypanosomiasis affecting skin (disorder)
        "403139005",  # American trypanosomiasis affecting skin (disorder)
        "403143009",  # Phthiriasis of eyelashes (disorder)
        "403144003",  # Disseminated phthiriasis (disorder)
        "403167001",  # Protothecosis of skin (disorder)
        "403484002",  # Bowenoid papulosis of anus (disorder)
        "403702001",  # Parastomal pyoderma gangrenosum (disorder)
        "403840005",  # Neonatal staphylococcal infection of skin (disorder)
        "403845000",  # Neonatal eosinophilic pustular folliculitis (disorder)
        "403961004",  # Pyogenic granuloma of lip (disorder)
        "404907009",  # Toxic megacolon due to Clostridium difficile infection (disorder)
        "405635002",  # Late syphilis with clinical manifestations other than neurosyphilis (disorder)
        "40635000",  # Cystitis caused by Pseudomonas (disorder)
        "406581000",  # Gonococcal infection of the central nervous system (disorder)
        "406596001",  # Naegleria infection of the central nervous system (disorder)
        "407671000",  # Bilateral pneumonia (disorder)
        "40786001",  # Aspiration pneumonitis caused by regurgitated gastric secretions (disorder)
        "4082005",  # Myocarditis caused by Treponema pallidum (disorder)
        "408680002",  # Healthcare associated bacterial pneumonia (disorder)
        "408681003",  # Healthcare associated legionnaire's disease (disorder)
        "408682005",  # Healthcare associated pertussis (disorder)
        "409641002",  # Hemorrhagic smallpox (disorder)
        "409664000",  # Pneumonia caused by anaerobic bacteria (disorder)
        "409665004",  # Pneumonia caused by aerobic bacteria (disorder)
        "410038006",  # Eumycotic mycetoma (disorder)
        "410039003",  # Mycetoma (disorder)
        "410040001",  # Eumycotic mycetoma of foot (disorder)
        "410470003",  # Syphilitic retinitis (disorder)
        "410472006",  # Bartonella henselae neuroretinitis (disorder)
        "410473001",  # Lyme uveitis (disorder)
        "410474007",  # Lyme keratitis (disorder)
        "410475008",  # Lyme conjunctivitis (disorder)
        "410478005",  # Ocular syphilis (disorder)
        "41080008",  # Cutaneous yaws (less than 5 years) (disorder)
        "41116009",  # Spirochetal infection (disorder)
        "41232003",  # Blister of scrotum with infection (disorder)
        "41235001",  # Dry form of cutaneous leishmaniasis (disorder)
        "41381004",  # Pneumonia caused by Pseudomonas (disorder)
        "413937003",  # Cutaneous disorder caused by bovine papular stomatitis virus (disorder)
        "414492009",  # Infective eczematoid dermatitis (disorder)
        "414529006",  # Intestinal infectious disease caused by anaerobic bacteria (disorder)
        "414530001",  # Intestinal infectious disease caused by Gram-negative bacteria (disorder)
        "414600000",  # Lepromatous anterior uveitis (disorder)
        "415125002",  # Pneumocystosis jirovecii pneumonia (disorder)
        "41545003",  # Whipple's disease (disorder)
        "415822001",  # Viral gastroenteritis caused by Rotavirus (disorder)
        "416189003",  # Exostosis (disorder)
        "416205001",  # Infection caused by Bartonella peromysci (disorder)
        "416718008",  # Infection of skin of eyelid due to herpes zoster (disorder)
        "417441005",  # Creeping myiasis (disorder)
        "417583002",  # Superficial white onychomycosis (disorder)
        "417688002",  # Chronic progressive coccidioidal pneumonia (disorder)
        "41804000",  # Blister of axilla with infection (disorder)
        "419266001",  # Condyloma acuminata of cervix (disorder)
        "419468003",  # Gardnerella vaginitis (disorder)
        "41958008",  # Nonvenomous insect bite of back with infection (disorder)
        "419760006",  # Bacterial vaginosis (disorder)
        "419817006",  # Cutaneous paracoccioidomycosis (disorder)
        "42032008",  # Nonvenomous insect bite of abdominal wall with infection (disorder)
        "420544002",  # Bacterial pneumonia with acquired immunodeficiency syndrome (disorder)
        "420787001",  # Pneumococcal pneumonia with acquired immunodeficiency syndrome (disorder)
        "420877009",  # Dermatomycosis with acquired immunodeficiency syndrome (disorder)
        "42089007",  # Nonvenomous insect bite of anus with infection (disorder)
        "421047005",  # Candidiasis of lung with acquired immunodeficiency syndrome (disorder)
        "421204004",  # Blastocystis hominis infection (disorder)
        "421224000",  # Intestinal infection caused by Klebsiella aerogenes (disorder)
        "421454008",  # Infectious gastroenteritis with acquired immunodeficiency syndrome (disorder)
        "421671002",  # Pneumonia with acquired immunodeficiency syndrome (disorder)
        "42231000009109",  # Bacterial nephritis (disorder)
        "422588002",  # Aspiration pneumonia (disorder)
        "423333008",  # Exanthem due to varicella (disorder)
        "423590009",  # Clostridium difficile colitis (disorder)
        "42402006",  # Kartagener syndrome (disorder)
        "424551004",  # Bacterial cystitis (disorder)
        "425464007",  # Nosocomial pneumonia (disorder)
        "425562008",  # Recurrent bacterial cystitis (disorder)
        "425996009",  # Bilateral basal pneumonia (disorder)
        "426006006",  # Trichomycosis pubis (disorder)
        "426135001",  # Chronic prostatitis - chronic pelvic pain syndrome (disorder)
        "426137009",  # Cytomegaloviral enteritis (disorder)
        "42640003",  # Suppurative appendicitis (disorder)
        "426437004",  # Familial idiopathic pulmonary fibrosis (disorder)
        "426507006",  # Enteritis caused by Candida (disorder)
        "426637005",  # Acute lyme disease (disorder)
        "426696003",  # Lingular pneumonia (disorder)
        "42680007",  # Endogenous lipoid pneumonitis (disorder)
        "42746002",  # Infection of rectum caused by Neisseria gonorrhoeae (disorder)
        "42770003",  # Syphilis of tricuspid valve (disorder)
        "427909005",  # Chronic recurrent sinusitis (disorder)
        "428017002",  # Condyloma acuminata of vulva in pregnancy (disorder)
        "428175000",  # Primary amebic encephalitis caused by Naegleria fowleri (disorder)
        "428230005",  # Trichomonal vaginitis in pregnancy (disorder)
        "428471001",  # Nonvenomous insect bite of multiple sites with infection (disorder)
        "42849000",  # Trypanosomal chancre (disorder)
        "428762009",  # Infected insect bite of hand (disorder)
        "428874003",  # Intestinal infection caused by Escherichia coli serotype O158 (disorder)
        "428877005",  # Infection of skin of finger (disorder)
        "429271009",  # Ventilator-acquired pneumonia (disorder)
        "43017006",  # Infection caused by Gastrodiscoides hominis (disorder)
        "43021004",  # Periungual wart (disorder)
        "430348006",  # Localized infection of skin AND/OR subcutaneous tissue (disorder)
        "430395005",  # Pneumonia caused by Gram negative bacteria (disorder)
        "430969000",  # Recurrent aspiration pneumonia (disorder)
        "43114002",  # Chronic trigonitis (disorder)
        "4324001",  # Subacute cystitis (disorder)
        "432728006",  # Infection of skin of neck (disorder)
        "43373009",  # Tinea barbae caused by Microsporum canis (disorder)
        "4340003",  # Acrodermatitis atrophicans chronica (disorder)
        "435141000124102",  # Recurrent acute suppurative otitis media of right ear (disorder)
        "435201000124109",  # Recurrent acute suppurative otitis media of left ear (disorder)
        "43581009",  # Tinea pedis caused by Epidermophyton (disorder)
        "4359001",  # Early congenital syphilis (less than 2 years) (disorder)
        "43591003",  # Exanthematous infectious disease (disorder)
        "438764004",  # Aspiration pneumonia following surgical procedure (disorder)
        "43956002",  # Blister of elbow with infection (disorder)
        "441658007",  # Pneumonia caused by Staphylococcus aureus (disorder)
        "441698009",  # Osteolysis of bone adjacent to prosthesis (disorder)
        "441942006",  # Pneumonia caused by Streptococcus pyogenes (disorder)
        "442003006",  # Diffuse cutaneous leishmaniasis caused by Leishmania aethiopica (disorder)
        "442027008",  # Inflammation of skin due to parasite (disorder)
        "442081006",  # Cutaneous larva migrans caused by Uncinaria larva (disorder)
        "442094008",  # Pneumonia caused by Histoplasma (disorder)
        "44274007",  # Lymphoid interstitial pneumonitis (disorder)
        "443137009",  # Infection of pierced pinna (disorder)
        "443138004",  # Granulomatosis (disorder)
        "44352004",  # Nonvenomous insect bite of hand with infection (disorder)
        "443874006",  # Infection of tick bite (disorder)
        "44412000",  # Chronic inflammation of urethra caused by Neisseria gonorrhoeae (disorder)
        "444150000",  # Latent syphilis (disorder)
        "444451006",  # Exposure to sexually transmissible disorder (event)
        "44464006",  # Postpartum AND/OR puerperal erysipelas (disorder)
        "444760002",  # Acute osteomyelitis of vertebra (disorder)
        "444834005",  # Abscess of urethral gland caused by Neisseria gonorrhoeae (disorder)
        "445063003",  # Chronic osteomyelitis of vertebra (disorder)
        "445130008",  # Acute infective adenoiditis (disorder)
        "445152004",  # Inflammation of intestine caused by Norovirus (disorder)
        "445172009",  # Cholesterol granuloma of middle ear and mastoid (disorder)
        "44549008",  # Aspiration pneumonia following procedure (disorder)
        "44568006",  # Syphilitic interstitial keratitis (disorder)
        "445918001",  # Eosinophilic cystitis (disorder)
        "446081009",  # Small bowel bacterial overgrowth syndrome (disorder)
        "446122002",  # Intestinal infection caused by Campylobacter coli (disorder)
        "446289001",  # Infection of nail bed of finger (disorder)
        "446328009",  # Intestinal infection caused by Escherichia coli O157:H7 (disorder)
        "44653001",  # Erysipelas (disorder)
        "446642005",  # Infection of anus caused by Chlamydia trachomatis (disorder)
        "446672004",  # Intestinal infection caused by Vibrio cholerae non-O139 (disorder)
        "446754004",  # Enteritis caused by Human coxsackievirus (disorder)
        "446755003",  # Enteritis caused by Human echovirus (disorder)
        "446756002",  # Enteritis caused by Human enterovirus 71 (disorder)
        "446771003",  # Verruca vulgaris of skin of scalp (disorder)
        "446901009",  # Verruca vulgaris of skin of pinna (disorder)
        "44697002",  # Melorheostosis (disorder)
        "446985003",  # Verruca vulgaris of skin of cheek (disorder)
        "446988001",  # Intestinal infection caused by Aeromonas hydrophila (disorder)
        "447004005",  # Enteritis caused by fungus (disorder)
        "447102007",  # Cholesterol granuloma of mastoid (disorder)
        "447146004",  # Verruca vulgaris of skin of lower extremity (disorder)
        "447222001",  # Granuloma gravidarum (disorder)
        "447282003",  # Intestinal infection caused by Vibrio cholerae O1 (disorder)
        "447354007",  # Intestinal infection caused by Campylobacter jejuni (disorder)
        "447372001",  # Infection of rectum caused by Chlamydia trachomatis (disorder)
        "44743006",  # Gonococcal infection of joint (disorder)
        "4483005",  # Syphilitic punched out ulcer (disorder)
        "44917000",  # Hymenolepiasis (disorder)
        "449643001",  # Nonvenomous insect bite of lower leg with infection (disorder)
        "449688000",  # Blister of lower limb with infection (disorder)
        "449689008",  # Blister of lower leg with infection (disorder)
        "449773001",  # Interstitial glossitis (disorder)
        "449775008",  # Solitary exostosis (disorder)
        "449776009",  # Luetic glossitis (disorder)
        "4499004",  # Disseminated anergic leishmaniasis (disorder)
        "449908004",  # Aggressive periodontitis (disorder)
        "45058001",  # Syphilis of pulmonary valve (disorder)
        "45072005",  # Acne necrotica miliaris (disorder)
        "450761000124106",  # Irradiation cystitis without hematuria (disorder)
        "450991000124101",  # Spontaneous rupture of right tympanic membrane co-occurrent and due to acute suppurative otitis media (disorder)
        "45312009",  # Pneumonia in typhoid fever (disorder)
        "45377007",  # Acute gonococcal salpingitis (disorder)
        "45556008",  # Pulmonary tularemia (disorder)
        "45581006",  # Abrasion and/or friction burn of breast with infection (disorder)
        "45629003",  # Abscess of nasopharynx (disorder)
        "45666009",  # Blister of ear with infection (disorder)
        "45816000",  # Pyelonephritis (disorder)
        "45915002",  # Nonvenomous insect bite of upper arm with infection (disorder)
        "46080009",  # Abrasion and/or friction burn of abdominal wall with infection (disorder)
        "46132008",  # Blister of hip with infection (disorder)
        "46235002",  # Early latent congenital syphilis, positive serology, negative spinal fluid (disorder)
        "46672003",  # Helicobacter-associated proctitis (disorder)
        "4669001",  # Pinta, primary lesion (disorder)
        "46699001",  # Bursitis caused by Neisseria gonorrhoeae (disorder)
        "46727001",  # Disorder of zinc metabolism (disorder)
        "46752004",  # Torus palatinus (disorder)
        "46799006",  # Dysenteric diarrhea (disorder)
        "46970008",  # Pneumonia caused by Mycoplasma pneumoniae (disorder)
        "46999003",  # Tinea barbae caused by Trichophyton (disorder)
        "47027001",  # Nonvenomous insect bite of scrotum with infection (disorder)
        "471272001",  # Cavitary pneumonia (disorder)
        "47277009",  # Izumi fever (disorder)
        "47306003",  # Lobomycosis (disorder)
        "473439008",  # Necrosis of cartilage of larynx caused by ionizing radiation (disorder)
        "47452006",  # Variola major (disorder)
        "47807003",  # Cutaneous habronemiasis (disorder)
        "47941007",  # Purulent enteritis (disorder)
        "48278001",  # Diphtheritic cystitis (disorder)
        "4834000",  # Typhoid fever (disorder)
        "48631008",  # Pyonephrosis (disorder)
        "48971001",  # Tinea manus (disorder)
        "48982009",  # Bannwarth syndrome (disorder)
        "49442000",  # Initial lesion of yaws (disorder)
        "49454000",  # Abrasion and/or friction burn of flank with infection (disorder)
        "49524007",  # Abrasion and/or friction burn of groin with infection (disorder)
        "49649001",  # Infection caused by Acanthamoeba (disorder)
        "49882001",  # Viral exanthem (disorder)
        "49883006",  # Candidiasis of skin (disorder)
        "49894005",  # Trichomycosis axillaris (disorder)
        "49923008",  # Syphilis of tendon (disorder)
        "50124004",  # Nonvenomous insect bite of buttock with infection (disorder)
        "50242001",  # Tinea barbae caused by Microsporum gypseum (disorder)
        "50293005",  # Abrasion and/or friction burn of penis with infection (disorder)
        "50414000",  # Late cutaneous leishmaniasis (disorder)
        "50528008",  # Syphilis of bone (disorder)
        "5085001",  # Gonococcemia (finding)
        "5093001",  # Cystitis with actinomycosis (disorder)
        "50970007",  # Acute gonorrhea of upper genitourinary tract (disorder)
        "51152000",  # Abscess of vocal cords (disorder)
        "51212009",  # Pitted keratolysis (disorder)
        "51254007",  # Paratyphoid C fever (disorder)
        "51423006",  # Modified smallpox (disorder)
        "51530003",  # Pneumonia caused by Escherichia coli (disorder)
        "51613008",  # Infection caused by Babesia divergens (disorder)
        "5168000",  # Sporotrichotic chancre (disorder)
        "51817006",  # Infection caused by Acanthamoeba castellani (disorder)
        "51882009",  # Nonvenomous insect bite of interscapular region with infection (disorder)
        "51928006",  # General paresis - neurosyphilis (disorder)
        "51960003",  # Secondary syphilis of pharynx (disorder)
        "521000119104",  # Acute cystitis in pregnancy, antepartum (disorder)
        "52123000",  # Emphysematous cystitis (disorder)
        "52281009",  # Meningoencephalitis caused by Naegleria (disorder)
        "52404001",  # Mastoiditis (disorder)
        "52414005",  # Secondary syphilis of vulva (disorder)
        "52705001",  # Nonvenomous insect bite of finger with infection (disorder)
        "52914002",  # Anal candidiasis (disorder)
        "52918004",  # Infection caused by Echinochasmus (disorder)
        "52979002",  # Capillaria infection (disorder)
        "53084003",  # Bacterial pneumonia (disorder)
        "53243005",  # Hyperchromic lesions of pinta (disorder)
        "53316003",  # Otomycosis (disorder)
        "53529004",  # Chronic gonococcal salpingitis (disorder)
        "53664003",  # Gonococcal spondylitis (disorder)
        "53734002",  # Hyperkeratosis of yaws (disorder)
        "53991003",  # Cystitis with bilharziasis (disorder)
        "54026009",  # Cystitis with Echinococcus infestation (disorder)
        "54043007",  # Cercarial dermatitis of freshwater avian type (disorder)
        "54069001",  # Congenital syphilitic mucous patches (disorder)
        "54266002",  # Infection caused by Fasciolopsis buski (disorder)
        "54274001",  # Primary syphilis of fingers (disorder)
        "54385001",  # Exanthema subitum (disorder)
        "5441008",  # Tinea capitis (disorder)
        "54825009",  # Acute gonorrhea of lower genitourinary tract (disorder)
        "5490008",  # Tinea barbae caused by Trichophyton verrucosum (disorder)
        "55023005",  # Amebic nondysenteric colitis (disorder)
        "55184003",  # Infectious inflammation of intestine (disorder)
        "55260003",  # Calcaneal spur (finding)
        "55679008",  # Peribronchial pneumonia (disorder)
        "55768006",  # Syphilitic leukoderma (disorder)
        "55799007",  # Tinea capitis caused by Microsporum (disorder)
        "56140006",  # Progressive vaccina (disorder)
        "56200009",  # Infection caused by Trichuris globulosa (disorder)
        "5631000124102",  # Facies leprosa (disorder)
        "56892000",  # Nonvenomous insect bite of chest wall with infection (disorder)
        "56978007",  # Generalized vaccinia (disorder)
        "57540006",  # Intestinal nematode infection (disorder)
        "5765005",  # Infection caused by Leishmania tropica minor (disorder)
        "57702005",  # Unresolved pneumonia (disorder)
        "57725006",  # Balantidiasis (disorder)
        "58056005",  # Syphilis of mitral valve (disorder)
        "58213005",  # Bacillary angiomatosis (disorder)
        "58227000",  # Secondary syphilis of viscera (disorder)
        "58392004",  # Congenital syphilitic osteochondritis (disorder)
        "58695007",  # Chromoblastomycosis caused by Fonsecaea compacta (disorder)
        "58763001",  # Acute empyema of nasal sinus (disorder)
        "5891000119102",  # Clostridium difficile diarrhea (disorder)
        "59121004",  # Infection caused by Leishmania aethiopica (disorder)
        "59158004",  # Infection caused by Oesophagostomum stephanostomum (disorder)
        "59233003",  # Secondary syphilis of skin (disorder)
        "59307008",  # Syphilitic alopecia (disorder)
        "59443003",  # Nonvenomous insect bite of thigh with infection (disorder)
        "59475000",  # Pneumonia in pertussis (disorder)
        "59530001",  # Syphilis of kidney (disorder)
        "59707005",  # Abscess of pharynx (disorder)
        "59721007",  # Congenital syphilitic pemphigus (disorder)
        "59763006",  # Hyperphosphatasemia tarda (disorder)
        "59934002",  # Secondary syphilitic adenopathy (disorder)
        "60100005",  # Cyst of pharynx (disorder)
        "60130002",  # Chronic frontal sinusitis (disorder)
        "6020002",  # Tinea pedis (disorder)
        "60298004",  # Nonvenomous insect bite of forearm with infection (disorder)
        "60335002",  # Gonococcal keratosis (disorder)
        "60416001",  # Edema of nasopharynx (disorder)
        "60485005",  # Pleurobronchopneumonia (disorder)
        "60506006",  # Blister of scalp with infection (disorder)
        "60528006",  # Secondary syphilis of mouth (disorder)
        "60570001",  # Infection caused by Trichuris trichiura (disorder)
        "60600009",  # Disorder of the larynx (disorder)
        "60867007",  # Klebsiella cystitis (disorder)
        "60893000",  # Chronic gonococcal prostatitis (disorder)
        "609338009",  # Abrasion and/or friction burn of upper limb, infected (disorder)
        "609351004",  # Abrasion and/or friction burn of lower leg with infection (disorder)
        "61048000",  # Gonococcal endocarditis (disorder)
        "61099007",  # Rubeola scarlatinosa (disorder)
        "61251002",  # Plantar hyperkeratosis of yaws (disorder)
        "61341000",  # Abrasion and/or friction burn of thigh with infection (disorder)
        "61383005",  # Abrasion and/or friction burn of ear with infection (disorder)
        "61500009",  # Granulomatous prostatitis (disorder)
        "61612001",  # Syphilitic aortic incompetence (disorder)
        "61624008",  # Chronic recurrent streptococcal erysipelas (disorder)
        "61884008",  # Achromobacter pneumonia (disorder)
        "62078006",  # Infection caused by Hypodereum (disorder)
        "62199007",  # Acute trigonitis (disorder)
        "62207008",  # Syphilitic ostial coronary disease (disorder)
        "62558006",  # Dyschromic skin lesions of pinta (disorder)
        "6267005",  # Congenital syphilitic meningitis (disorder)
        "62861003",  # Secondary syphilis, relapse (untreated) (disorder)
        "63302006",  # Chronic pyelonephritis (disorder)
        "63440008",  # Verruca plantaris (disorder)
        "63605003",  # Infection caused by Naegleria gruberi (disorder)
        "63650001",  # Cholera (disorder)
        "6365004",  # Cutaneous actinomycosis (disorder)
        "63717008",  # Exostosis of orbit (disorder)
        "63751007",  # Secondary syphilis of bone (disorder)
        "63757006",  # Osler's node (disorder)
        "63848007",  # Abrasion and/or friction burn of wrist with infection (disorder)
        "64102008",  # Syphilitic gumma (disorder)
        "64479007",  # Pneumonia caused by Klebsiella pneumoniae (disorder)
        "64650008",  # Cutaneous nocardiosis (disorder)
        "64667001",  # Interstitial pneumonia (disorder)
        "64703005",  # Terminal bronchopneumonia (disorder)
        "64831005",  # Abrasion and/or friction burn of shoulder with infection (disorder)
        "64956000",  # Follicular cystitis (disorder)
        "65049003",  # Acute gonococcal seminal vesiculitis (disorder)
        "65141002",  # Foreign body pneumonia (disorder)
        "65295003",  # Acute gonococcal endometritis (disorder)
        "65332001",  # Diverticulitis of urinary bladder (disorder)
        "65696007",  # Coccidioidal granuloma (disorder)
        "65984002",  # Salpingitis follicularis (disorder)
        "66160001",  # Cryptosporidial gastroenteritis (disorder)
        "66281009",  # Chorioretinitis due to secondary syphilis (disorder)
        "66371007",  # Abrasion and/or friction burn of upper arm with infection (disorder)
        "66379009",  # Bacterial overgrowth syndrome (disorder)
        "66429007",  # Unresolved lobar pneumonia (disorder)
        "66578000",  # Abrasion and/or friction burn of cheek with infection (disorder)
        "66887000",  # Late syphilis, latent (positive serology, negative cephalospinal fluid 2 years after) (disorder)
        "67077008",  # Abrasion and/or friction burn of axilla with infection (disorder)
        "67125004",  # Latent syphilis with positive serology (disorder)
        "67167008",  # Urethrotrigonitis (disorder)
        "67391006",  # Syphilitic endocarditis (disorder)
        "67525007",  # Secondary pneumonic plague (disorder)
        "67624004",  # Abscess of Bartholin's gland (disorder)
        "67685000",  # Prostatocystitis (disorder)
        "67862004",  # Nonvenomous insect bite of elbow with infection (disorder)
        "67896006",  # Post-kala-azar dermal leishmaniasis (disorder)
        "67915005",  # Intestinal trichomoniasis (disorder)
        "67924001",  # Smallpox (disorder)
        "67974004",  # Nonvenomous insect bite of nose with infection (disorder)
        "68202005",  # Intermediate lesion of pinta (disorder)
        "68226007",  # Acute cystitis (disorder)
        "68389001",  # Mucopurulent vaginitis (disorder)
        "68409003",  # Organized pneumonia (disorder)
        "68421004",  # Epiphyseal arrest (disorder)
        "68566005",  # Urinary tract infectious disease (disorder)
        "68764005",  # Juvenile taboparesis (disorder)
        "68863007",  # Acute syphilitic meningitis (disorder)
        "689981000119103",  # Ulcer of skin caused by Cytomegalovirus (disorder)
        "69013005",  # Cutaneous draschiasis (disorder)
        "69163003",  # Taenia saginata infection (disorder)
        "69588003",  # Erythema chronica migrans (disorder)
        "69595007",  # Secondary syphilitic periostitis (disorder)
        "6964007",  # Blister of breast with infection (disorder)
        "69810009",  # Acute hemorrhagic colitis caused by Escherichia coli (disorder)
        "698628000",  # Postabortal pelvic inflammatory disease (disorder)
        "699014000",  # Recurrent pneumonia (disorder)
        "699219002",  # Spindle shaped torus palatinus (disorder)
        "699861000",  # Familial granulomatous inflammatory arthritis, dermatitis and uveitis (disorder)
        "700220003",  # Acute bacterial otitis externa (disorder)
        "700249006",  # Idiopathic interstitial pneumonitis (disorder)
        "700250006",  # Idiopathic pulmonary fibrosis (disorder)
        "70036007",  # Haemophilus influenzae pneumonia (disorder)
        "70143003",  # Tuberculoid leprosy (disorder)
        "702715008",  # Acute on chronic prostatitis (disorder)
        "70413005",  # Dermatophilosis caused by Dermatophilus congolensis (disorder)
        "704345008",  # Chronic interstitial pneumonitis (disorder)
        "7063008",  # Gangrenous pneumonia (disorder)
        "707208009",  # Familial non-obstructive reflux-associated chronic pyelonephritis (disorder)
        "707222009",  # Epidemic gastroenteritis (disorder)
        "707449006",  # Exogenous lipoid pneumonitis (disorder)
        "707503004",  # Pneumonia caused by Schistosoma mansoni (disorder)
        "707507003",  # Pneumonia caused by Schistosoma japonicum (disorder)
        "707508008",  # Pneumonia caused by Schistosoma haematobium (disorder)
        "707525005",  # Non-O1 and non-O139 Vibrio cholerae infection (disorder)
        "707552000",  # Acute exogenous lipoid pneumonitis (disorder)
        "707553005",  # Chronic exogenous lipoid pneumonitis (disorder)
        "707834009",  # Acute on chronic cystitis (disorder)
        "708031000",  # Idiopathic eosinophilic pneumonitis (disorder)
        "708126004",  # Recurrent candidiasis of vagina (disorder)
        "70880006",  # Adenoviral enteritis (disorder)
        "71011005",  # Infestation caused by Phthirus pubis (disorder)
        "71085009",  # Paratyphoid B fever (disorder)
        "71104009",  # Achromic AND hyperchromic skin lesions of pinta (disorder)
        "711167005",  # Enteritis caused by Yersinia enterocolitica (disorder)
        "711644007",  # Endophthalmitis caused by Acanthamoeba (disorder)
        "711645008",  # Corneal ulcer caused by Acanthamoeba (disorder)
        "71186008",  # Croup (disorder)
        "712583003",  # Intestinal infection caused by Plesiomonas shigelloides (disorder)
        "713249002",  # Pyogenic granuloma of gingiva due to pregnancy (disorder)
        "713251003",  # Gingival disease caused by Treponema pallidum (disorder)
        "713261005",  # Gingival disease caused by Neisseria gonorrhoeae (disorder)
        "713316008",  # Eruption of skin co-occurrent with human immunodeficiency virus infection (disorder)
        "713525001",  # Recurrent bacterial pneumonia (disorder)
        "713526000",  # Recurrent bacterial pneumonia co-occurrent with human immunodeficiency virus infection (disorder)
        "713544008",  # Bacterial pneumonia co-occurrent with human immunodeficiency virus infection (disorder)
        "713546005",  # Isosporiasis co-occurrent with human immunodeficiency virus infection (disorder)
        "713570009",  # Infectious gastroenteritis co-occurrent with human immunodeficiency virus infection (disorder)
        "7138005",  # Caries of mastoid (disorder)
        "714002",  # Abrasion and/or friction burn of toe with infection (disorder)
        "71418005",  # Cyst of nasopharynx (disorder)
        "71450000",  # Contact ulcer of vocal folds (disorder)
        "715062005",  # Infection caused by Oesophagostomum radiatum (disorder)
        "715065007",  # Infection caused by Bartonella talpae (disorder)
        "715067004",  # Ulcerating cutaneous leishmaniasis of lower extremity (disorder)
        "715266002",  # Infection of skin of eyelid and periocular region (disorder)
        "715487005",  # Autosomal recessive distal osteolysis syndrome (disorder)
        "715507005",  # Infection of central nervous system caused by Borrelia burgdorferi (disorder)
        "71572001",  # Infection caused by Enteromonas (disorder)
        "71577007",  # Deformity of eyelid due to infective dermatitis of eyelid (disorder)
        "715874005",  # Infection of skin caused by Neoscytalidium dimidiatum (disorder)
        "71590000",  # Trichomonal prostatitis (disorder)
        "716860005",  # Infection of intestine caused by cyclospora cayetanensis (disorder)
        "716868003",  # Multicentric osteolysis nodulosis arthropathy spectrum (disorder)
        "717875005",  # Infective duodenitis (disorder)
        "718373005",  # Cutaneous B-cell pseudolymphoma caused by Borrelia (disorder)
        "719218000",  # Cryptogenic organizing pneumonia (disorder)
        "719755001",  # Conjunctivitis of adulthood caused by Neisseria gonorrhoeae (disorder)
        "72074009",  # Infection caused by Trichostrongylus orientalis (disorder)
        "72083004",  # Late syphilis (disorder)
        "721267000",  # Pyogenic infection of skin and subcutaneous tissues caused by bacterium (disorder)
        "721268005",  # Superficial folliculitis caused by bacterium (disorder)
        "721270001",  # Infection of diaper area (disorder)
        "721583004",  # Mucocutaneous early congenital syphilis (disorder)
        "721587003",  # Sexually transmissible infection caused by human papillomavirus (disorder)
        "721588008",  # Hepatitis caused by sexually transmissible virus (disorder)
        "721589000",  # Sexually transmissible infection caused by Molluscum contagiosum virus (disorder)
        "721590009",  # Infection of anogenital region caused by Molluscum contagiosum virus (disorder)
        "721657003",  # Food poisoning caused by Listeria monocytogenes (disorder)
        "721659000",  # Parasitic infection causing inflammation of small intestine (disorder)
        "721681005",  # Colitis caused by bacterium (disorder)
        "721682003",  # Colitis caused by Salmonella (disorder)
        "721683008",  # Infection causing abscess of colon (disorder)
        "721684002",  # Colitis caused by protozoan (disorder)
        "721685001",  # Parasitic infection causing colitis (disorder)
        "721771003",  # Infection of intestine caused by Human cytomegalovirus 5 (disorder)
        "721808004",  # Infection of intestine caused by Entamoeba histolytica (disorder)
        "721816008",  # Infection caused by Naegleria fowleri (disorder)
        "722127006",  # Pacman dysplasia (disorder)
        "72222004",  # Papillary cystitis (disorder)
        "72225002",  # Urethral stricture due to gonococcal infection (disorder)
        "722520001",  # Non-ulcerative sexually transmitted infection caused by Chlamydia trachomatis (disorder)
        "722521002",  # Wart of anal mucosa caused by Human papillomavirus (disorder)
        "722522009",  # Female pelvic inflammatory disease caused by Mycoplasma genitalium (disorder)
        "722541007",  # Toxic megacolon due to infectious colitis (disorder)
        "722830001",  # Acute female pelvic inflammatory disease following procedure (disorder)
        "722861005",  # Dermatosis of perianal region caused by Enterobius vermicularis (disorder)
        "72294005",  # Alastrim (disorder)
        "723544007",  # Trichodysplasia spinulosa caused by Polyomavirus (disorder)
        "723580007",  # Talo-patello-scaphoid osteolysis syndrome (disorder)
        "723829000",  # Pulmonary fibrosis, hepatic hyperplasia, bone marrow hypoplasia syndrome (disorder)
        "723884008",  # Duodenal ulcer caused by bacterium (disorder)
        "724015007",  # Pyogenic arthritis, pyoderma gangrenosum, acne syndrome (disorder)
        "724498004",  # Pneumonia caused by Chlamydia pneumoniae (disorder)
        "724499007",  # Idiopathic acute eosinophilic pneumonitis (disorder)
        "724500003",  # Idiopathic chronic eosinophilic pneumonitis (disorder)
        "724525007",  # Duodenitis caused by Helicobacter pylori (disorder)
        "724528009",  # Duodenal ulcer caused by Helicobacter pylori (disorder)
        "724530006",  # Duodenal ulcer caused by Helicobacter pylori and non-steroidal anti-inflammatory agent (disorder)
        "724533008",  # Duodenal ulcer caused by virus (disorder)
        "724534002",  # Duodenal ulcer caused by fungus (disorder)
        "72512006",  # Abrasion and/or friction burn of nose with infection (disorder)
        "725916003",  # Acute sinusitis caused by fungus (disorder)
        "725917007",  # Acute sinusitis caused by virus (disorder)
        "72605008",  # Candidal vulvovaginitis (disorder)
        "726078000",  # Sporadic Blau syndrome (disorder)
        "72656004",  # Granulomatous pneumonitis (disorder)
        "72815004",  # Cystitis glandularis (disorder)
        "72854003",  # Aspiration pneumonia due to near drowning (disorder)
        "72934000",  # Candidiasis of vagina (disorder)
        "73237007",  # Chronic ethmoidal sinusitis (disorder)
        "73281004",  # Chronic zinc deficiency (disorder)
        "733051000",  # Pneumonia caused by Gram positive bacteria (disorder)
        "733148009",  # Duodenitis caused by Tropheryma whipplei (disorder)
        "733154005",  # Parasitic infection of small intestine caused by Cystoisospora belli (disorder)
        "733161009",  # Parasitic infection of colon caused by Cystoisospora belli (disorder)
        "733166004",  # Infection of skin caused by Borrelia (disorder)
        "733172004",  # Ameboma of large intestine caused by Entamoeba histolytica (disorder)
        "733187009",  # Osteolysis following surgical procedure on skeletal system (disorder)
        "73375005",  # Ulcer of larynx (disorder)
        "73532000",  # Cutaneous larva migrans caused by Ancylostoma caninum (disorder)
        "735400004",  # Localized infection of subcutaneous tissue (disorder)
        "735486007",  # Genital tract infection due to and following pregnancy with abortive outcome (disorder)
        "735487003",  # Pelvic infection due to and following pregnancy with abortive outcome (disorder)
        "735488008",  # Pelvic infection due to and following ectopic pregnancy (disorder)
        "735489000",  # Genital tract infection due to and following ectopic pregnancy (disorder)
        "735490009",  # Genital tract infection due to and following molar pregnancy (disorder)
        "735491008",  # Pelvic infection due to and following molar pregnancy (disorder)
        "735514001",  # Infection of intestine caused by Vibrio (disorder)
        "735515000",  # Symptomatic late congenital syphilis (disorder)
        "735520000",  # Disseminated infection caused by Borrelia (disorder)
        "735605000",  # Degeneration of cervical intervertebral disc co-occurrent with osteophyte of cervical vertebra (disorder)
        "735608003",  # Degeneration of thoracic intervertebral disc co-occurrent with osteophyte (disorder)
        "735611002",  # Degeneration of lumbar intervertebral disc co-occurrent with osteophyte of lumbar vertebra (disorder)
        "735619000",  # Pyometra co-occurrent and due to acute inflammatory disease of uterus (disorder)
        "735620006",  # Pyometra co-occurrent and due to chronic inflammatory disease of uterus (disorder)
        "735738004",  # Localized cutaneous leishmaniasis (disorder)
        "735739007",  # Infection of skin and subcutaneous tissue caused by fungus (disorder)
        "73594001",  # Pinta, late lesion (disorder)
        "735958007",  # Exanthem caused by Chlamydophila psittaci (disorder)
        "736686006",  # Chancre caused by Treponema pallidum (disorder)
        "736979001",  # Recurrent infection of skin (disorder)
        "737147003",  # Chronic inflammation of mastoid cavity (disorder)
        "737345002",  # Infection of large intestine (disorder)
        "73908004",  # Infection caused by Babesia microti (disorder)
        "74372003",  # Gonorrhea of pharynx (disorder)
        "74445007",  # Trigonitis (disorder)
        "7448003",  # Acute pyonephrosis with renal medullary necrosis (disorder)
        "74578003",  # Pyoderma gangrenosum (disorder)
        "74621002",  # Epidemic vomiting syndrome (disorder)
        "74631009",  # Blister of finger with infection (disorder)
        "74724001",  # Smallpox without rash (disorder)
        "7493006",  # Infection caused by Plagiorchis (disorder)
        "75073007",  # Malakoplakia of urinary bladder (disorder)
        "75169006",  # Infection caused by Oesophagostomum apiostomum (disorder)
        "75299005",  # Spastic spinal syphilitic paralysis (disorder)
        "75375008",  # Inflammation of intestine caused by bacteria (disorder)
        "75426006",  # Aspiration pneumonia caused by regurgitated food (disorder)
        "7548000",  # Rheumatic pneumonitis (disorder)
        "75498004",  # Acute bacterial sinusitis (disorder)
        "75530006",  # Streptococcal cervicitis (disorder)
        "75656006",  # Ulcerating cutaneous leishmaniasis (disorder)
        "75996005",  # Tinea pedis caused by Epidermophyton floccosum (disorder)
        "76069003",  # Disorder of bone (disorder)
        "76090006",  # Pittsburgh pneumonia (disorder)
        "76122006",  # Hyperkeratosis of pinta (disorder)
        "762257007",  # Disseminated infection caused by Neisseria gonorrhoeae (disorder)
        "762276009",  # Ulcer of duodenum due to infection (disorder)
        "762375006",  # Erythema multiforme due to viral disease (disorder)
        "76272004",  # Syphilis (disorder)
        "763888005",  # Necrotizing pneumonia caused by Panton-Valentine leukocidin producing Staphylococcus aureus (disorder)
        "76490001",  # Otomycosis externa caused by Fusarium (disorder)
        "765782003",  # Infection of skin of eyelid caused by Molluscum contagiosum (disorder)
        "765783008",  # Infection of skin caused by Molluscum contagiosum (disorder)
        "76584003",  # Nonvenomous insect bite of hip with infection (disorder)
        "76623002",  # Paratyphoid A fever (disorder)
        "76653009",  # Acute empyema of maxillary sinus (disorder)
        "766992008",  # Multicentric carpotarsal osteolysis syndrome (disorder)
        "76802005",  # Chronic inflammation of cervix uteri caused by Neisseria gonorrhoeae (disorder)
        "76813008",  # Pyogenic granuloma of oral mucosa (disorder)
        "76832002",  # Blister of wrist with infection (disorder)
        "77028001",  # Secondary syphilis of anus (disorder)
        "770652001",  # Stenosis of external auditory canal due to and following radiotherapy (disorder)
        "770761005",  # Phalangeal microgeodic syndrome (disorder)
        "771339005",  # Hyperzincemia and hypercalprotectinemia (disorder)
        "771537008",  # Infection of deep tissue at puncture site due to and following peripheral nerve block (disorder)
        "771539006",  # Abscess at puncture site due to and following peripheral nerve block (disorder)
        "773328002",  # Infantile onset panniculitis with uveitis and systemic granulomatosis (disorder)
        "773331001",  # Nestor Guillermo progeria syndrome (disorder)
        "77412003",  # Hymenolepis nana infection (disorder)
        "77470003",  # Enteroviral exanthematous fever (disorder)
        "77746001",  # Leontiasis (disorder)
        "77863005",  # Lyme carditis (disorder)
        "77874001",  # Nonvenomous insect bite of shoulder with infection (disorder)
        "77939001",  # Syphilitic disseminated retinochoroiditis (disorder)
        "781201007",  # Ascariasis caused by adult Ascaris (disorder)
        "78128001",  # Helicobacter-associated colitis (disorder)
        "782167001",  # Stewart-Morel-Morgagni syndrome (disorder)
        "78223000",  # Erythema nodosum due to coccidioidomycosis (disorder)
        "783184003",  # Encrusted cystitis (disorder)
        "783566008",  # Mucocutaneous ulcer caused by Epstein-Barr virus (disorder)
        "78360004",  # Abrasion and/or friction burn of ankle with infection (disorder)
        "783743009",  # Combined immunodeficiency with granulomatosis (disorder)
        "78420004",  # Inflammation of intestine caused by virus (disorder)
        "78487004",  # Granulomatous dermatophytosis (disorder)
        "78561001",  # Blister of foot with infection (disorder)
        "785724007",  # Pyoderma gangrenosum, acne, suppurative hidradenitis syndrome (disorder)
        "7877005",  # Infection caused by Diphyllobothrium latum (disorder)
        "78895009",  # Congenital pneumonia (disorder)
        "788974002",  # Late manifestations of infection of skin caused by Borrelia (disorder)
        "788975001",  # Late syphilis of skin (disorder)
        "788998009",  # Ocular late syphilis (disorder)
        "789052003",  # Subcutaneous infection caused by fungus (disorder)
        "789121002",  # Late syphilis of mucous membrane (disorder)
        "789229008",  # Infected wound of eyelid due to nonvenomous insect bite (disorder)
        "789230003",  # Infected wound of bilateral eyelids due to nonvenomous insect bite (disorder)
        "789574002",  # Acute exacerbation of idiopathic pulmonary fibrosis (disorder)
        "789685005",  # Recurrent infection caused by Clostridioides difficile (disorder)
        "79077003",  # Abrasion and/or friction burn of back with infection (disorder)
        "79340003",  # Abrasion and/or friction burn of interscapular region with infection (disorder)
        "79411002",  # Acute prostatitis (disorder)
        "79949009",  # Primary cutaneous coccidioidomycosis (disorder)
        "79974007",  # Cat scratch disease (disorder)
        "801006",  # Nonvenomous insect bite of foot with infection (disorder)
        "80388004",  # Chronic gonorrhea of upper genitourinary tract (disorder)
        "80604007",  # Acute gonococcal bartholinitis (disorder)
        "80645004",  # Chronic mastoiditis (disorder)
        "80770009",  # Secondary syphilis of liver (disorder)
        "80890005",  # Cutaneous sporotrichosis (disorder)
        "8098009",  # Sexually transmitted infectious disease (disorder)
        "81000006",  # Pediculosis capitis (disorder)
        "81020007",  # Cholera caused by Vibrio cholerae El Tor (disorder)
        "81125004",  # Acute hemorrhagic cystitis (disorder)
        "81244001",  # Blister of neck with infection (disorder)
        "81339006",  # Secondary syphilis of tonsil (disorder)
        "81598001",  # Trichomonal vulvovaginitis (disorder)
        "82054006",  # Hyperostosis interna frontalis (finding)
        "82074004",  # Achromic skin lesions of pinta (disorder)
        "82214002",  # Trench fever (disorder)
        "82323002",  # Late congenital syphilis (2 years or more) (disorder)
        "82355002",  # Syphilitic aortic stenosis (disorder)
        "82451005",  # Blister of upper arm with infection (disorder)
        "82512002",  # Blister of back with infection (disorder)
        "82597009",  # Tinea capitis caused by Trichophyton tonsurans (disorder)
        "827006",  # Late congenital syphilis, latent (positive serology - cerebrospinal fluid, 2 years OR more) (disorder)
        "82930004",  # Intestinal infection caused by Pseudomonas (disorder)
        "82959004",  # Dementia paralytica juvenilis (disorder)
        "83052004",  # Cutaneous screw-worm myiasis (disorder)
        "83070001",  # Friction blister with infection (disorder)
        "83448006",  # Plantar papilloma of yaws (disorder)
        "83587009",  # Nonvenomous insect bite of cheek with infection (disorder)
        "838317007",  # Bursitis due to late syphilis (disorder)
        "838353009",  # Acute pyelonephritis caused by bacterium (disorder)
        "83883001",  # Cardiovascular syphilis (disorder)
        "83937008",  # Infection caused by Babesia bovis (disorder)
        "840284009",  # Eczema coxsackium (disorder)
        "840460003",  # Colitis caused by Cryptosporidium (disorder)
        "840484006",  # Conjunctivitis caused by Acanthamoeba (disorder)
        "840503000",  # Duodenitis caused by Ancylostoma (disorder)
        "840504006",  # Duodenitis caused by Cytomegalovirus (disorder)
        "840506008",  # Duodenal candidosis (disorder)
        "84070009",  # Nonvenomous insect bite of axilla with infection (disorder)
        "84204007",  # Abrasion and/or friction burn of scrotum with infection (disorder)
        "84278006",  # Abrasion and/or friction burn of neck with infection (disorder)
        "84417007",  # Blister of forearm with infection (disorder)
        "84753008",  # Pneumonia in systemic mycosis (disorder)
        "84889008",  # Parapharyngeal abscess (disorder)
        "84980006",  # Cutaneous anthrax (disorder)
        "8549006",  # Desquamative interstitial pneumonitis (disorder)
        "85495007",  # Acute pyonephrosis without renal medullary necrosis (disorder)
        "8555001",  # Syphilis of lung (disorder)
        "85857008",  # Secondary syphilis, relapse (treated) (disorder)
        "85904008",  # Paratyphoid fever (disorder)
        "86028001",  # Syphilis of liver (disorder)
        "860867001",  # Genitocrural intertrigo caused by Candida (disorder)
        "860871003",  # Eosinophilic folliculitis due to human immunodeficiency virus infection (disorder)
        "86279000",  # Acute suppurative otitis media with spontaneous rupture of ear drum (disorder)
        "86333008",  # Subcutaneous phycomycosis caused by Basidiobolus (disorder)
        "86443005",  # Hutchinson's teeth (disorder)
        "86497005",  # Flat-type smallpox (disorder)
        "866074003",  # Infection of colon caused by Spirochaetes (disorder)
        "866115006",  # Lues maligna (disorder)
        "866117003",  # Localized cutaneous leishmaniasis co-occurrent with nodular lymphangitis (disorder)
        "866120006",  # Infection of intestine caused by Anisakis larva (disorder)
        "866122003",  # Infection of large intestine caused by Entamoeba histolytica (disorder)
        "86615009",  # Epidemic diarrhea (disorder)
        "86981007",  # Infective otitis externa (disorder)
        "870298008",  # Parasitic infestation of intestine caused by Taenia solium (disorder)
        "870300008",  # Parasitic infection of duodenum caused by Anisakis (disorder)
        "870325004",  # Parasitic infection of colon caused by Ascaris lumbricoides (disorder)
        "870326003",  # Parasitic infection of colon caused by Strongyloides (disorder)
        "870573008",  # Interstitial pneumonitis with autoimmune features (disorder)
        "870728004",  # Exanthem caused by human echovirus 16 (disorder)
        "870729007",  # Exanthem caused by human echovirus (disorder)
        "871634008",  # Warts of perianal region in infancy caused by human papillomavirus (disorder)
        "87282003",  # Intestinal parasitism (disorder)
        "87318008",  # Congenital syphilis with gumma (disorder)
        "8748000",  # Abrasion and/or friction burn of hip with infection (disorder)
        "87603003",  # Lymphocutaneous sporotrichosis (disorder)
        "87628006",  # Bacterial infectious disease (disorder)
        "87696004",  # Hemorrhagic cystitis (disorder)
        "8776008",  # Amebic cystitis (disorder)
        "880095009",  # Dermatitis due to Ornithonyssus sylviarum infestation (disorder)
        "880096005",  # Dermatitis due to Liponyssoides sanguineus infestation (disorder)
        "880097001",  # Dermatitis due to Ornithonyssus bacoti infestation (disorder)
        "880098006",  # Dermatitis due to Dermanyssus gallinae infestation (disorder)
        "880099003",  # Dermatitis due to Cheyletiella yasguri infestation (disorder)
        "880100006",  # Dermatitis due to Cheyletiella parasitivorax infestation (disorder)
        "880101005",  # Dermatitis due to Cheyletiella blakei infestation (disorder)
        "88220006",  # Pachydermoperiostosis syndrome (disorder)
        "88411000",  # Blister of anus with infection (disorder)
        "88558006",  # Cercarial dermatitis of sea water avian type (disorder)
        "88646002",  # Blister of penis with infection (disorder)
        "88813005",  # Chronic gonococcal cystitis (disorder)
        "88850006",  # Chronic pansinusitis (disorder)
        "88943008",  # Syphilis of muscle (disorder)
        "88981003",  # Pyometra (disorder)
        "890114008",  # Iris papulosa caused by Treponema pallidum (disorder)
        "890115009",  # Iris roseola caused by Treponema pallidum (disorder)
        "89478006",  # Parasitic melanoderma (disorder)
        "897657000",  # Chronic rhinosinusitis (disorder)
        "89933001",  # Protozoal intestinal disease (disorder)
        "90145001",  # Frontal bossing (disorder)
        "90271007",  # Intestinal infection caused by Arizona group (disorder)
        "90428001",  # Gonococcal pericarditis (disorder)
        "90679001",  # Mixed intestinal infection caused by Cestoda and/or Trematoda and/or Phylum Nemata (disorder)
        "90886006",  # Infestation of rectum caused by fly larvae (disorder)
        "9091006",  # Gonococcal iridocyclitis (disorder)
        "91060006",  # Nonvenomous insect bite of wrist with infection (disorder)
        "91374005",  # Carbuncle of skin and/or subcutaneous tissue (disorder)
        "91554004",  # Condyloma latum (disorder)
        "9241004",  # Gonococcal heart disease (disorder)
        "94851000119107",  # Syphilitic valvular endocarditis (disorder)
        "95333004",  # Eosinophilic pustular folliculitis (disorder)
        "95340003",  # Nail bed infection (disorder)
        "95374004",  # Application site infection (disorder)
        "95381006",  # Injection site infection (disorder)
        "95547004",  # Rupture of appendix due to suppurative appendicitis (disorder)
        "95881004",  # Mycetoma of foot (disorder)
        "95889002",  # Mycoplasmal pyelonephritis (disorder)
        "95896000",  # Protozoan infection (disorder)
        "9713002",  # Prostatitis (disorder)
        "9723006",  # Hyperphosphatasemia with bone disease (disorder)
        "9802006",  # Abrasion and/or friction burn of buttock with infection (disorder)
        "9866007",  # Infection caused by Trichuris vulpis (disorder)
        "9889004",  # Tinea barbae caused by Trichophyton violaceum (disorder)
        "9941009",  # Congenital syphilitic choroiditis (disorder)
    }

class DementiaAndMentalDegenerations(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of dementia or mental degenerations.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis for dementia and mental degeneration.

    **Exclusion Criteria:** Excludes concepts that describe dementia and mental degeneration diagnoses in remission.
    """

    VALUE_SET_NAME = "Dementia and Mental Degenerations"
    OID = "2.16.840.1.113883.3.526.3.1005"
    DEFINITION_VERSION = "20200306"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "A5217",  # General paresis
        "F0150",  # Vascular dementia, unspecified severity, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F0280",  # Dementia in other diseases classified elsewhere, unspecified severity, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F0390",  # Unspecified dementia, unspecified severity, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F05",  # Delirium due to known physiological condition
        "F068",  # Other specified mental disorders due to known physiological condition
        "G300",  # Alzheimer's disease with early onset
        "G301",  # Alzheimer's disease with late onset
        "G308",  # Other Alzheimer's disease
        "G309",  # Alzheimer's disease, unspecified
        "G3101",  # Pick's disease
        "G3109",  # Other frontotemporal neurocognitive disorder
        "G3180",  # Leukodystrophy, unspecified
        "G3183",  # Neurocognitive disorder with Lewy bodies
        "G3184",  # Mild cognitive impairment of uncertain or unknown etiology
        "G3185",  # Corticobasal degeneration
        "G3189",  # Other specified degenerative diseases of nervous system
    }

    SNOMEDCT = {
        "10349009",  # Multi-infarct dementia with delirium (disorder)
        "10532003",  # Primary degenerative dementia of the Alzheimer type, presenile onset, with depression (disorder)
        "111480006",  # Psychoactive substance-induced organic dementia (disorder)
        "12348006",  # Presenile dementia (disorder)
        "1339031006",  # Dementia caused by ionizing radiation (disorder)
        "14070001",  # Multi-infarct dementia with depression (disorder)
        "15662003",  # Senile dementia (disorder)
        "191449005",  # Uncomplicated senile dementia (disorder)
        "191451009",  # Uncomplicated presenile dementia (disorder)
        "191452002",  # Presenile dementia with delirium (disorder)
        "191454001",  # Presenile dementia with paranoia (disorder)
        "191455000",  # Presenile dementia with depression (disorder)
        "191457008",  # Senile dementia with depressive or paranoid features (disorder)
        "191458003",  # Senile dementia with paranoia (disorder)
        "191459006",  # Senile dementia with depression (disorder)
        "191461002",  # Senile dementia with delirium (disorder)
        "191463004",  # Uncomplicated arteriosclerotic dementia (disorder)
        "191464005",  # Arteriosclerotic dementia with delirium (disorder)
        "191465006",  # Arteriosclerotic dementia with paranoia (disorder)
        "191466007",  # Arteriosclerotic dementia with depression (disorder)
        "191493005",  # Dementia caused by drug (disorder)
        "22381000119105",  # Primary degenerative dementia (disorder)
        "230258005",  # Amyotrophic lateral sclerosis with dementia (disorder)
        "230270009",  # Frontotemporal dementia (disorder)
        "230282000",  # Post-traumatic dementia (disorder)
        "230285003",  # Vascular dementia of acute onset (disorder)
        "230286002",  # Subcortical vascular dementia (disorder)
        "230287006",  # Mixed cortical and subcortical vascular dementia (disorder)
        "230288001",  # Semantic dementia (disorder)
        "230289009",  # Patchy dementia (disorder)
        "25772007",  # Multi-infarct dementia with delusions (disorder)
        "26852004",  # Primary degenerative dementia of the Alzheimer type, senile onset, with depression (disorder)
        "278857002",  # Dementia of frontal lobe type (disorder)
        "279982005",  # Cerebral degeneration presenting primarily with dementia (disorder)
        "281004",  # Dementia associated with alcoholism (disorder)
        "31081000119101",  # Presenile dementia with delusions (disorder)
        "312991009",  # Senile dementia of the Lewy body type (disorder)
        "32875003",  # Inhalant-induced persisting dementia (disorder)
        "371024007",  # Senile dementia with delusion (disorder)
        "371026009",  # Senile dementia with psychosis (disorder)
        "416780008",  # Primary degenerative dementia of the Alzheimer type, presenile onset (disorder)
        "420614009",  # Organic dementia with acquired immunodeficiency syndrome (disorder)
        "421023003",  # Presenile dementia with acquired immunodeficiency syndrome (disorder)
        "421529006",  # Dementia with acquired immunodeficiency syndrome (disorder)
        "425390006",  # Dementia associated with Parkinson's Disease (disorder)
        "429458009",  # Dementia due to Creutzfeldt Jakob disease (disorder)
        "429998004",  # Vascular dementia (disorder)
        "430771000124100",  # Moderate dementia (disorder)
        "442344002",  # Dementia due to Huntington chorea (disorder)
        "4817008",  # Primary degenerative dementia of the Alzheimer type, senile onset, with delirium (disorder)
        "51928006",  # General paresis - neurosyphilis (disorder)
        "52448006",  # Dementia (disorder)
        "54502004",  # Primary degenerative dementia of the Alzheimer type, presenile onset, with delusions (disorder)
        "55009008",  # Primary degenerative dementia of the Alzheimer type, senile onset, with delusions (disorder)
        "56267009",  # Multi-infarct dementia (disorder)
        "59651006",  # Sedative, hypnotic AND/OR anxiolytic-induced persisting dementia (disorder)
        "6475002",  # Primary degenerative dementia of the Alzheimer type, presenile onset, uncomplicated (disorder)
        "65096006",  # Primary degenerative dementia of the Alzheimer type, presenile onset, with delirium (disorder)
        "66108005",  # Primary degenerative dementia of the Alzheimer type, senile onset, uncomplicated (disorder)
        "698624003",  # Dementia associated with cerebral lipidosis (disorder)
        "698625002",  # Dementia associated with normal pressure hydrocephalus (disorder)
        "698626001",  # Dementia associated with multiple sclerosis (disorder)
        "698687007",  # Post-traumatic dementia with behavioral change (disorder)
        "698725008",  # Dementia associated with neurosyphilis (disorder)
        "698726009",  # Dementia associated with viral encephalitis (disorder)
        "698781002",  # Dementia associated with cerebral anoxia (disorder)
        "702393003",  # Frontotemporal dementia with gene located on 3p11 (disorder)
        "702426001",  # GRN-related frontotemporal dementia (disorder)
        "702429008",  # Frontotemporal dementia with parkinsonism-17 (disorder)
        "703544004",  # Inclusion body myopathy with early-onset Paget disease and frontotemporal dementia (disorder)
        "70936005",  # Multi-infarct dementia, uncomplicated (disorder)
        "713488003",  # Presenile dementia co-occurrent with human immunodeficiency virus infection (disorder)
        "713844000",  # Dementia co-occurrent with human immunodeficiency virus infection (disorder)
        "715737004",  # Parkinsonism co-occurrent with dementia of Guadeloupe (disorder)
        "716667005",  # Right temporal atrophy variant frontotemporal dementia (disorder)
        "716994006",  # Behavioral variant of frontotemporal dementia (disorder)
        "722977005",  # Dementia co-occurrent and due to neurocysticercosis (disorder)
        "722978000",  # Toxic dementia (disorder)
        "722979008",  # Dementia due to metabolic abnormality (disorder)
        "722980006",  # Dementia due to chromosomal anomaly (disorder)
        "723123001",  # Ischemic vascular dementia (disorder)
        "723390000",  # Rapidly progressive dementia (disorder)
        "724776007",  # Dementia due to disorder of central nervous system (disorder)
        "724777003",  # Dementia due to infectious disease (disorder)
        "724992007",  # Epilepsy co-occurrent and due to dementia (disorder)
        "725898002",  # Delirium co-occurrent with dementia (disorder)
        "733184002",  # Dementia caused by heavy metal exposure (disorder)
        "733190003",  # Dementia due to primary malignant neoplasm of brain (disorder)
        "733191004",  # Dementia due to chronic subdural hematoma (disorder)
        "733192006",  # Dementia due to herpes encephalitis (disorder)
        "733193001",  # Dementia co-occurrent and due to progressive multifocal leukoencephalopathy (disorder)
        "733194007",  # Dementia co-occurrent and due to Down syndrome (disorder)
        "762350007",  # Dementia due to prion disease (disorder)
        "762351006",  # Dementia due to and following injury of head (disorder)
        "762707000",  # Subcortical dementia (disorder)
        "79341000119107",  # Mixed dementia (disorder)
        "82959004",  # Dementia paralytica juvenilis (disorder)
        "838276009",  # Amyotrophic lateral sclerosis, parkinsonism, dementia complex (disorder)
        "90099008",  # Subcortical leukoencephalopathy (disorder)
        "9345005",  # Dialysis dementia (disorder)
    }

class ComplicationsOfPregnancyChildbirthAndThePuerperium(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of complications related to pregnancy or childbirth.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis for complications of pregnancy or childbirth.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Complications of Pregnancy, Childbirth and the Puerperium"
    OID = "2.16.840.1.113883.3.464.1003.111.12.1012"
    DEFINITION_VERSION = "20200310"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "O0000",  # Abdominal pregnancy without intrauterine pregnancy
        "O0001",  # Abdominal pregnancy with intrauterine pregnancy
        "O00101",  # Right tubal pregnancy without intrauterine pregnancy
        "O00102",  # Left tubal pregnancy without intrauterine pregnancy
        "O00109",  # Unspecified tubal pregnancy without intrauterine pregnancy
        "O00111",  # Right tubal pregnancy with intrauterine pregnancy
        "O00112",  # Left tubal pregnancy with intrauterine pregnancy
        "O00119",  # Unspecified tubal pregnancy with intrauterine pregnancy
        "O00201",  # Right ovarian pregnancy without intrauterine pregnancy
        "O00202",  # Left ovarian pregnancy without intrauterine pregnancy
        "O00209",  # Unspecified ovarian pregnancy without intrauterine pregnancy
        "O00211",  # Right ovarian pregnancy with intrauterine pregnancy
        "O00212",  # Left ovarian pregnancy with intrauterine pregnancy
        "O00219",  # Unspecified ovarian pregnancy with intrauterine pregnancy
        "O0080",  # Other ectopic pregnancy without intrauterine pregnancy
        "O0081",  # Other ectopic pregnancy with intrauterine pregnancy
        "O0090",  # Unspecified ectopic pregnancy without intrauterine pregnancy
        "O0091",  # Unspecified ectopic pregnancy with intrauterine pregnancy
        "O010",  # Classical hydatidiform mole
        "O011",  # Incomplete and partial hydatidiform mole
        "O019",  # Hydatidiform mole, unspecified
        "O020",  # Blighted ovum and nonhydatidiform mole
        "O021",  # Missed abortion
        "O0281",  # Inappropriate change in quantitative human chorionic gonadotropin (hCG) in early pregnancy
        "O0289",  # Other abnormal products of conception
        "O029",  # Abnormal product of conception, unspecified
        "O030",  # Genital tract and pelvic infection following incomplete spontaneous abortion
        "O031",  # Delayed or excessive hemorrhage following incomplete spontaneous abortion
        "O032",  # Embolism following incomplete spontaneous abortion
        "O0330",  # Unspecified complication following incomplete spontaneous abortion
        "O0331",  # Shock following incomplete spontaneous abortion
        "O0332",  # Renal failure following incomplete spontaneous abortion
        "O0333",  # Metabolic disorder following incomplete spontaneous abortion
        "O0334",  # Damage to pelvic organs following incomplete spontaneous abortion
        "O0335",  # Other venous complications following incomplete spontaneous abortion
        "O0336",  # Cardiac arrest following incomplete spontaneous abortion
        "O0337",  # Sepsis following incomplete spontaneous abortion
        "O0338",  # Urinary tract infection following incomplete spontaneous abortion
        "O0339",  # Incomplete spontaneous abortion with other complications
        "O034",  # Incomplete spontaneous abortion without complication
        "O035",  # Genital tract and pelvic infection following complete or unspecified spontaneous abortion
        "O036",  # Delayed or excessive hemorrhage following complete or unspecified spontaneous abortion
        "O037",  # Embolism following complete or unspecified spontaneous abortion
        "O0380",  # Unspecified complication following complete or unspecified spontaneous abortion
        "O0381",  # Shock following complete or unspecified spontaneous abortion
        "O0382",  # Renal failure following complete or unspecified spontaneous abortion
        "O0383",  # Metabolic disorder following complete or unspecified spontaneous abortion
        "O0384",  # Damage to pelvic organs following complete or unspecified spontaneous abortion
        "O0385",  # Other venous complications following complete or unspecified spontaneous abortion
        "O0386",  # Cardiac arrest following complete or unspecified spontaneous abortion
        "O0387",  # Sepsis following complete or unspecified spontaneous abortion
        "O0388",  # Urinary tract infection following complete or unspecified spontaneous abortion
        "O0389",  # Complete or unspecified spontaneous abortion with other complications
        "O039",  # Complete or unspecified spontaneous abortion without complication
        "O045",  # Genital tract and pelvic infection following (induced) termination of pregnancy
        "O046",  # Delayed or excessive hemorrhage following (induced) termination of pregnancy
        "O047",  # Embolism following (induced) termination of pregnancy
        "O0480",  # (Induced) termination of pregnancy with unspecified complications
        "O0481",  # Shock following (induced) termination of pregnancy
        "O0482",  # Renal failure following (induced) termination of pregnancy
        "O0483",  # Metabolic disorder following (induced) termination of pregnancy
        "O0484",  # Damage to pelvic organs following (induced) termination of pregnancy
        "O0485",  # Other venous complications following (induced) termination of pregnancy
        "O0486",  # Cardiac arrest following (induced) termination of pregnancy
        "O0487",  # Sepsis following (induced) termination of pregnancy
        "O0488",  # Urinary tract infection following (induced) termination of pregnancy
        "O0489",  # (Induced) termination of pregnancy with other complications
        "O070",  # Genital tract and pelvic infection following failed attempted termination of pregnancy
        "O071",  # Delayed or excessive hemorrhage following failed attempted termination of pregnancy
        "O072",  # Embolism following failed attempted termination of pregnancy
        "O0730",  # Failed attempted termination of pregnancy with unspecified complications
        "O0731",  # Shock following failed attempted termination of pregnancy
        "O0732",  # Renal failure following failed attempted termination of pregnancy
        "O0733",  # Metabolic disorder following failed attempted termination of pregnancy
        "O0734",  # Damage to pelvic organs following failed attempted termination of pregnancy
        "O0735",  # Other venous complications following failed attempted termination of pregnancy
        "O0736",  # Cardiac arrest following failed attempted termination of pregnancy
        "O0737",  # Sepsis following failed attempted termination of pregnancy
        "O0738",  # Urinary tract infection following failed attempted termination of pregnancy
        "O0739",  # Failed attempted termination of pregnancy with other complications
        "O074",  # Failed attempted termination of pregnancy without complication
        "O080",  # Genital tract and pelvic infection following ectopic and molar pregnancy
        "O081",  # Delayed or excessive hemorrhage following ectopic and molar pregnancy
        "O082",  # Embolism following ectopic and molar pregnancy
        "O083",  # Shock following ectopic and molar pregnancy
        "O084",  # Renal failure following ectopic and molar pregnancy
        "O085",  # Metabolic disorders following an ectopic and molar pregnancy
        "O086",  # Damage to pelvic organs and tissues following an ectopic and molar pregnancy
        "O087",  # Other venous complications following an ectopic and molar pregnancy
        "O0881",  # Cardiac arrest following an ectopic and molar pregnancy
        "O0882",  # Sepsis following ectopic and molar pregnancy
        "O0883",  # Urinary tract infection following an ectopic and molar pregnancy
        "O0889",  # Other complications following an ectopic and molar pregnancy
        "O089",  # Unspecified complication following an ectopic and molar pregnancy
        "O0900",  # Supervision of pregnancy with history of infertility, unspecified trimester
        "O0901",  # Supervision of pregnancy with history of infertility, first trimester
        "O0902",  # Supervision of pregnancy with history of infertility, second trimester
        "O0903",  # Supervision of pregnancy with history of infertility, third trimester
        "O0910",  # Supervision of pregnancy with history of ectopic pregnancy, unspecified trimester
        "O0911",  # Supervision of pregnancy with history of ectopic pregnancy, first trimester
        "O0912",  # Supervision of pregnancy with history of ectopic pregnancy, second trimester
        "O0913",  # Supervision of pregnancy with history of ectopic pregnancy, third trimester
        "O09211",  # Supervision of pregnancy with history of pre-term labor, first trimester
        "O09212",  # Supervision of pregnancy with history of pre-term labor, second trimester
        "O09213",  # Supervision of pregnancy with history of pre-term labor, third trimester
        "O09219",  # Supervision of pregnancy with history of pre-term labor, unspecified trimester
        "O09291",  # Supervision of pregnancy with other poor reproductive or obstetric history, first trimester
        "O09292",  # Supervision of pregnancy with other poor reproductive or obstetric history, second trimester
        "O09293",  # Supervision of pregnancy with other poor reproductive or obstetric history, third trimester
        "O09299",  # Supervision of pregnancy with other poor reproductive or obstetric history, unspecified trimester
        "O0930",  # Supervision of pregnancy with insufficient antenatal care, unspecified trimester
        "O0931",  # Supervision of pregnancy with insufficient antenatal care, first trimester
        "O0932",  # Supervision of pregnancy with insufficient antenatal care, second trimester
        "O0933",  # Supervision of pregnancy with insufficient antenatal care, third trimester
        "O0940",  # Supervision of pregnancy with grand multiparity, unspecified trimester
        "O0941",  # Supervision of pregnancy with grand multiparity, first trimester
        "O0942",  # Supervision of pregnancy with grand multiparity, second trimester
        "O0943",  # Supervision of pregnancy with grand multiparity, third trimester
        "O09511",  # Supervision of elderly primigravida, first trimester
        "O09512",  # Supervision of elderly primigravida, second trimester
        "O09513",  # Supervision of elderly primigravida, third trimester
        "O09519",  # Supervision of elderly primigravida, unspecified trimester
        "O09521",  # Supervision of elderly multigravida, first trimester
        "O09522",  # Supervision of elderly multigravida, second trimester
        "O09523",  # Supervision of elderly multigravida, third trimester
        "O09529",  # Supervision of elderly multigravida, unspecified trimester
        "O09611",  # Supervision of young primigravida, first trimester
        "O09612",  # Supervision of young primigravida, second trimester
        "O09613",  # Supervision of young primigravida, third trimester
        "O09619",  # Supervision of young primigravida, unspecified trimester
        "O09621",  # Supervision of young multigravida, first trimester
        "O09622",  # Supervision of young multigravida, second trimester
        "O09623",  # Supervision of young multigravida, third trimester
        "O09629",  # Supervision of young multigravida, unspecified trimester
        "O0970",  # Supervision of high risk pregnancy due to social problems, unspecified trimester
        "O0971",  # Supervision of high risk pregnancy due to social problems, first trimester
        "O0972",  # Supervision of high risk pregnancy due to social problems, second trimester
        "O0973",  # Supervision of high risk pregnancy due to social problems, third trimester
        "O09811",  # Supervision of pregnancy resulting from assisted reproductive technology, first trimester
        "O09812",  # Supervision of pregnancy resulting from assisted reproductive technology, second trimester
        "O09813",  # Supervision of pregnancy resulting from assisted reproductive technology, third trimester
        "O09819",  # Supervision of pregnancy resulting from assisted reproductive technology, unspecified trimester
        "O09821",  # Supervision of pregnancy with history of in utero procedure during previous pregnancy, first trimester
        "O09822",  # Supervision of pregnancy with history of in utero procedure during previous pregnancy, second trimester
        "O09823",  # Supervision of pregnancy with history of in utero procedure during previous pregnancy, third trimester
        "O09829",  # Supervision of pregnancy with history of in utero procedure during previous pregnancy, unspecified trimester
        "O09891",  # Supervision of other high risk pregnancies, first trimester
        "O09892",  # Supervision of other high risk pregnancies, second trimester
        "O09893",  # Supervision of other high risk pregnancies, third trimester
        "O09899",  # Supervision of other high risk pregnancies, unspecified trimester
        "O0990",  # Supervision of high risk pregnancy, unspecified, unspecified trimester
        "O0991",  # Supervision of high risk pregnancy, unspecified, first trimester
        "O0992",  # Supervision of high risk pregnancy, unspecified, second trimester
        "O0993",  # Supervision of high risk pregnancy, unspecified, third trimester
        "O09A0",  # Supervision of pregnancy with history of molar pregnancy, unspecified trimester
        "O09A1",  # Supervision of pregnancy with history of molar pregnancy, first trimester
        "O09A2",  # Supervision of pregnancy with history of molar pregnancy, second trimester
        "O09A3",  # Supervision of pregnancy with history of molar pregnancy, third trimester
        "O10011",  # Pre-existing essential hypertension complicating pregnancy, first trimester
        "O10012",  # Pre-existing essential hypertension complicating pregnancy, second trimester
        "O10013",  # Pre-existing essential hypertension complicating pregnancy, third trimester
        "O10019",  # Pre-existing essential hypertension complicating pregnancy, unspecified trimester
        "O1002",  # Pre-existing essential hypertension complicating childbirth
        "O10111",  # Pre-existing hypertensive heart disease complicating pregnancy, first trimester
        "O10112",  # Pre-existing hypertensive heart disease complicating pregnancy, second trimester
        "O10113",  # Pre-existing hypertensive heart disease complicating pregnancy, third trimester
        "O10119",  # Pre-existing hypertensive heart disease complicating pregnancy, unspecified trimester
        "O1012",  # Pre-existing hypertensive heart disease complicating childbirth
        "O10211",  # Pre-existing hypertensive chronic kidney disease complicating pregnancy, first trimester
        "O10212",  # Pre-existing hypertensive chronic kidney disease complicating pregnancy, second trimester
        "O10213",  # Pre-existing hypertensive chronic kidney disease complicating pregnancy, third trimester
        "O10219",  # Pre-existing hypertensive chronic kidney disease complicating pregnancy, unspecified trimester
        "O1022",  # Pre-existing hypertensive chronic kidney disease complicating childbirth
        "O10311",  # Pre-existing hypertensive heart and chronic kidney disease complicating pregnancy, first trimester
        "O10312",  # Pre-existing hypertensive heart and chronic kidney disease complicating pregnancy, second trimester
        "O10313",  # Pre-existing hypertensive heart and chronic kidney disease complicating pregnancy, third trimester
        "O10319",  # Pre-existing hypertensive heart and chronic kidney disease complicating pregnancy, unspecified trimester
        "O1032",  # Pre-existing hypertensive heart and chronic kidney disease complicating childbirth
        "O10411",  # Pre-existing secondary hypertension complicating pregnancy, first trimester
        "O10412",  # Pre-existing secondary hypertension complicating pregnancy, second trimester
        "O10413",  # Pre-existing secondary hypertension complicating pregnancy, third trimester
        "O10419",  # Pre-existing secondary hypertension complicating pregnancy, unspecified trimester
        "O1042",  # Pre-existing secondary hypertension complicating childbirth
        "O10911",  # Unspecified pre-existing hypertension complicating pregnancy, first trimester
        "O10912",  # Unspecified pre-existing hypertension complicating pregnancy, second trimester
        "O10913",  # Unspecified pre-existing hypertension complicating pregnancy, third trimester
        "O10919",  # Unspecified pre-existing hypertension complicating pregnancy, unspecified trimester
        "O1092",  # Unspecified pre-existing hypertension complicating childbirth
        "O111",  # Pre-existing hypertension with pre-eclampsia, first trimester
        "O112",  # Pre-existing hypertension with pre-eclampsia, second trimester
        "O113",  # Pre-existing hypertension with pre-eclampsia, third trimester
        "O114",  # Pre-existing hypertension with pre-eclampsia, complicating childbirth
        "O119",  # Pre-existing hypertension with pre-eclampsia, unspecified trimester
        "O1200",  # Gestational edema, unspecified trimester
        "O1201",  # Gestational edema, first trimester
        "O1202",  # Gestational edema, second trimester
        "O1203",  # Gestational edema, third trimester
        "O1204",  # Gestational edema, complicating childbirth
        "O1210",  # Gestational proteinuria, unspecified trimester
        "O1211",  # Gestational proteinuria, first trimester
        "O1212",  # Gestational proteinuria, second trimester
        "O1213",  # Gestational proteinuria, third trimester
        "O1214",  # Gestational proteinuria, complicating childbirth
        "O1220",  # Gestational edema with proteinuria, unspecified trimester
        "O1221",  # Gestational edema with proteinuria, first trimester
        "O1222",  # Gestational edema with proteinuria, second trimester
        "O1223",  # Gestational edema with proteinuria, third trimester
        "O1224",  # Gestational edema with proteinuria, complicating childbirth
        "O131",  # Gestational [pregnancy-induced] hypertension without significant proteinuria, first trimester
        "O132",  # Gestational [pregnancy-induced] hypertension without significant proteinuria, second trimester
        "O133",  # Gestational [pregnancy-induced] hypertension without significant proteinuria, third trimester
        "O134",  # Gestational [pregnancy-induced] hypertension without significant proteinuria, complicating childbirth
        "O139",  # Gestational [pregnancy-induced] hypertension without significant proteinuria, unspecified trimester
        "O1400",  # Mild to moderate pre-eclampsia, unspecified trimester
        "O1402",  # Mild to moderate pre-eclampsia, second trimester
        "O1403",  # Mild to moderate pre-eclampsia, third trimester
        "O1404",  # Mild to moderate pre-eclampsia, complicating childbirth
        "O1410",  # Severe pre-eclampsia, unspecified trimester
        "O1412",  # Severe pre-eclampsia, second trimester
        "O1413",  # Severe pre-eclampsia, third trimester
        "O1414",  # Severe pre-eclampsia complicating childbirth
        "O1420",  # HELLP syndrome (HELLP), unspecified trimester
        "O1422",  # HELLP syndrome (HELLP), second trimester
        "O1423",  # HELLP syndrome (HELLP), third trimester
        "O1424",  # HELLP syndrome, complicating childbirth
        "O1490",  # Unspecified pre-eclampsia, unspecified trimester
        "O1492",  # Unspecified pre-eclampsia, second trimester
        "O1493",  # Unspecified pre-eclampsia, third trimester
        "O1494",  # Unspecified pre-eclampsia, complicating childbirth
        "O1500",  # Eclampsia complicating pregnancy, unspecified trimester
        "O1502",  # Eclampsia complicating pregnancy, second trimester
        "O1503",  # Eclampsia complicating pregnancy, third trimester
        "O151",  # Eclampsia complicating labor
        "O159",  # Eclampsia, unspecified as to time period
        "O161",  # Unspecified maternal hypertension, first trimester
        "O162",  # Unspecified maternal hypertension, second trimester
        "O163",  # Unspecified maternal hypertension, third trimester
        "O164",  # Unspecified maternal hypertension, complicating childbirth
        "O169",  # Unspecified maternal hypertension, unspecified trimester
        "O200",  # Threatened abortion
        "O208",  # Other hemorrhage in early pregnancy
        "O209",  # Hemorrhage in early pregnancy, unspecified
        "O210",  # Mild hyperemesis gravidarum
        "O211",  # Hyperemesis gravidarum with metabolic disturbance
        "O212",  # Late vomiting of pregnancy
        "O218",  # Other vomiting complicating pregnancy
        "O219",  # Vomiting of pregnancy, unspecified
        "O2200",  # Varicose veins of lower extremity in pregnancy, unspecified trimester
        "O2201",  # Varicose veins of lower extremity in pregnancy, first trimester
        "O2202",  # Varicose veins of lower extremity in pregnancy, second trimester
        "O2203",  # Varicose veins of lower extremity in pregnancy, third trimester
        "O2210",  # Genital varices in pregnancy, unspecified trimester
        "O2211",  # Genital varices in pregnancy, first trimester
        "O2212",  # Genital varices in pregnancy, second trimester
        "O2213",  # Genital varices in pregnancy, third trimester
        "O2220",  # Superficial thrombophlebitis in pregnancy, unspecified trimester
        "O2221",  # Superficial thrombophlebitis in pregnancy, first trimester
        "O2222",  # Superficial thrombophlebitis in pregnancy, second trimester
        "O2223",  # Superficial thrombophlebitis in pregnancy, third trimester
        "O2230",  # Deep phlebothrombosis in pregnancy, unspecified trimester
        "O2231",  # Deep phlebothrombosis in pregnancy, first trimester
        "O2232",  # Deep phlebothrombosis in pregnancy, second trimester
        "O2233",  # Deep phlebothrombosis in pregnancy, third trimester
        "O2240",  # Hemorrhoids in pregnancy, unspecified trimester
        "O2241",  # Hemorrhoids in pregnancy, first trimester
        "O2242",  # Hemorrhoids in pregnancy, second trimester
        "O2243",  # Hemorrhoids in pregnancy, third trimester
        "O2250",  # Cerebral venous thrombosis in pregnancy, unspecified trimester
        "O2251",  # Cerebral venous thrombosis in pregnancy, first trimester
        "O2252",  # Cerebral venous thrombosis in pregnancy, second trimester
        "O2253",  # Cerebral venous thrombosis in pregnancy, third trimester
        "O228X1",  # Other venous complications in pregnancy, first trimester
        "O228X2",  # Other venous complications in pregnancy, second trimester
        "O228X3",  # Other venous complications in pregnancy, third trimester
        "O228X9",  # Other venous complications in pregnancy, unspecified trimester
        "O2290",  # Venous complication in pregnancy, unspecified, unspecified trimester
        "O2291",  # Venous complication in pregnancy, unspecified, first trimester
        "O2292",  # Venous complication in pregnancy, unspecified, second trimester
        "O2293",  # Venous complication in pregnancy, unspecified, third trimester
        "O2300",  # Infections of kidney in pregnancy, unspecified trimester
        "O2301",  # Infections of kidney in pregnancy, first trimester
        "O2302",  # Infections of kidney in pregnancy, second trimester
        "O2303",  # Infections of kidney in pregnancy, third trimester
        "O2310",  # Infections of bladder in pregnancy, unspecified trimester
        "O2311",  # Infections of bladder in pregnancy, first trimester
        "O2312",  # Infections of bladder in pregnancy, second trimester
        "O2313",  # Infections of bladder in pregnancy, third trimester
        "O2320",  # Infections of urethra in pregnancy, unspecified trimester
        "O2321",  # Infections of urethra in pregnancy, first trimester
        "O2322",  # Infections of urethra in pregnancy, second trimester
        "O2323",  # Infections of urethra in pregnancy, third trimester
        "O2330",  # Infections of other parts of urinary tract in pregnancy, unspecified trimester
        "O2331",  # Infections of other parts of urinary tract in pregnancy, first trimester
        "O2332",  # Infections of other parts of urinary tract in pregnancy, second trimester
        "O2333",  # Infections of other parts of urinary tract in pregnancy, third trimester
        "O2340",  # Unspecified infection of urinary tract in pregnancy, unspecified trimester
        "O2341",  # Unspecified infection of urinary tract in pregnancy, first trimester
        "O2342",  # Unspecified infection of urinary tract in pregnancy, second trimester
        "O2343",  # Unspecified infection of urinary tract in pregnancy, third trimester
        "O23511",  # Infections of cervix in pregnancy, first trimester
        "O23512",  # Infections of cervix in pregnancy, second trimester
        "O23513",  # Infections of cervix in pregnancy, third trimester
        "O23519",  # Infections of cervix in pregnancy, unspecified trimester
        "O23521",  # Salpingo-oophoritis in pregnancy, first trimester
        "O23522",  # Salpingo-oophoritis in pregnancy, second trimester
        "O23523",  # Salpingo-oophoritis in pregnancy, third trimester
        "O23529",  # Salpingo-oophoritis in pregnancy, unspecified trimester
        "O23591",  # Infection of other part of genital tract in pregnancy, first trimester
        "O23592",  # Infection of other part of genital tract in pregnancy, second trimester
        "O23593",  # Infection of other part of genital tract in pregnancy, third trimester
        "O23599",  # Infection of other part of genital tract in pregnancy, unspecified trimester
        "O2390",  # Unspecified genitourinary tract infection in pregnancy, unspecified trimester
        "O2391",  # Unspecified genitourinary tract infection in pregnancy, first trimester
        "O2392",  # Unspecified genitourinary tract infection in pregnancy, second trimester
        "O2393",  # Unspecified genitourinary tract infection in pregnancy, third trimester
        "O24011",  # Pre-existing type 1 diabetes mellitus, in pregnancy, first trimester
        "O24012",  # Pre-existing type 1 diabetes mellitus, in pregnancy, second trimester
        "O24013",  # Pre-existing type 1 diabetes mellitus, in pregnancy, third trimester
        "O24019",  # Pre-existing type 1 diabetes mellitus, in pregnancy, unspecified trimester
        "O2402",  # Pre-existing type 1 diabetes mellitus, in childbirth
        "O24111",  # Pre-existing type 2 diabetes mellitus, in pregnancy, first trimester
        "O24112",  # Pre-existing type 2 diabetes mellitus, in pregnancy, second trimester
        "O24113",  # Pre-existing type 2 diabetes mellitus, in pregnancy, third trimester
        "O24119",  # Pre-existing type 2 diabetes mellitus, in pregnancy, unspecified trimester
        "O2412",  # Pre-existing type 2 diabetes mellitus, in childbirth
        "O24311",  # Unspecified pre-existing diabetes mellitus in pregnancy, first trimester
        "O24312",  # Unspecified pre-existing diabetes mellitus in pregnancy, second trimester
        "O24313",  # Unspecified pre-existing diabetes mellitus in pregnancy, third trimester
        "O24319",  # Unspecified pre-existing diabetes mellitus in pregnancy, unspecified trimester
        "O2432",  # Unspecified pre-existing diabetes mellitus in childbirth
        "O24410",  # Gestational diabetes mellitus in pregnancy, diet controlled
        "O24414",  # Gestational diabetes mellitus in pregnancy, insulin controlled
        "O24415",  # Gestational diabetes mellitus in pregnancy, controlled by oral hypoglycemic drugs
        "O24419",  # Gestational diabetes mellitus in pregnancy, unspecified control
        "O24420",  # Gestational diabetes mellitus in childbirth, diet controlled
        "O24424",  # Gestational diabetes mellitus in childbirth, insulin controlled
        "O24425",  # Gestational diabetes mellitus in childbirth, controlled by oral hypoglycemic drugs
        "O24429",  # Gestational diabetes mellitus in childbirth, unspecified control
        "O24811",  # Other pre-existing diabetes mellitus in pregnancy, first trimester
        "O24812",  # Other pre-existing diabetes mellitus in pregnancy, second trimester
        "O24813",  # Other pre-existing diabetes mellitus in pregnancy, third trimester
        "O24819",  # Other pre-existing diabetes mellitus in pregnancy, unspecified trimester
        "O2482",  # Other pre-existing diabetes mellitus in childbirth
        "O24911",  # Unspecified diabetes mellitus in pregnancy, first trimester
        "O24912",  # Unspecified diabetes mellitus in pregnancy, second trimester
        "O24913",  # Unspecified diabetes mellitus in pregnancy, third trimester
        "O24919",  # Unspecified diabetes mellitus in pregnancy, unspecified trimester
        "O2492",  # Unspecified diabetes mellitus in childbirth
        "O2510",  # Malnutrition in pregnancy, unspecified trimester
        "O2511",  # Malnutrition in pregnancy, first trimester
        "O2512",  # Malnutrition in pregnancy, second trimester
        "O2513",  # Malnutrition in pregnancy, third trimester
        "O252",  # Malnutrition in childbirth
        "O2600",  # Excessive weight gain in pregnancy, unspecified trimester
        "O2601",  # Excessive weight gain in pregnancy, first trimester
        "O2602",  # Excessive weight gain in pregnancy, second trimester
        "O2603",  # Excessive weight gain in pregnancy, third trimester
        "O2610",  # Low weight gain in pregnancy, unspecified trimester
        "O2611",  # Low weight gain in pregnancy, first trimester
        "O2612",  # Low weight gain in pregnancy, second trimester
        "O2613",  # Low weight gain in pregnancy, third trimester
        "O2620",  # Pregnancy care for patient with recurrent pregnancy loss, unspecified trimester
        "O2621",  # Pregnancy care for patient with recurrent pregnancy loss, first trimester
        "O2622",  # Pregnancy care for patient with recurrent pregnancy loss, second trimester
        "O2623",  # Pregnancy care for patient with recurrent pregnancy loss, third trimester
        "O2630",  # Retained intrauterine contraceptive device in pregnancy, unspecified trimester
        "O2631",  # Retained intrauterine contraceptive device in pregnancy, first trimester
        "O2632",  # Retained intrauterine contraceptive device in pregnancy, second trimester
        "O2633",  # Retained intrauterine contraceptive device in pregnancy, third trimester
        "O2640",  # Herpes gestationis, unspecified trimester
        "O2641",  # Herpes gestationis, first trimester
        "O2642",  # Herpes gestationis, second trimester
        "O2643",  # Herpes gestationis, third trimester
        "O2650",  # Maternal hypotension syndrome, unspecified trimester
        "O2651",  # Maternal hypotension syndrome, first trimester
        "O2652",  # Maternal hypotension syndrome, second trimester
        "O2653",  # Maternal hypotension syndrome, third trimester
        "O26611",  # Liver and biliary tract disorders in pregnancy, first trimester
        "O26612",  # Liver and biliary tract disorders in pregnancy, second trimester
        "O26613",  # Liver and biliary tract disorders in pregnancy, third trimester
        "O26619",  # Liver and biliary tract disorders in pregnancy, unspecified trimester
        "O2662",  # Liver and biliary tract disorders in childbirth
        "O26641",  # Intrahepatic cholestasis of pregnancy, first trimester
        "O26642",  # Intrahepatic cholestasis of pregnancy, second trimester
        "O26643",  # Intrahepatic cholestasis of pregnancy, third trimester
        "O26649",  # Intrahepatic cholestasis of pregnancy, unspecified trimester
        "O26711",  # Subluxation of symphysis (pubis) in pregnancy, first trimester
        "O26712",  # Subluxation of symphysis (pubis) in pregnancy, second trimester
        "O26713",  # Subluxation of symphysis (pubis) in pregnancy, third trimester
        "O26719",  # Subluxation of symphysis (pubis) in pregnancy, unspecified trimester
        "O2672",  # Subluxation of symphysis (pubis) in childbirth
        "O26811",  # Pregnancy related exhaustion and fatigue, first trimester
        "O26812",  # Pregnancy related exhaustion and fatigue, second trimester
        "O26813",  # Pregnancy related exhaustion and fatigue, third trimester
        "O26819",  # Pregnancy related exhaustion and fatigue, unspecified trimester
        "O26821",  # Pregnancy related peripheral neuritis, first trimester
        "O26822",  # Pregnancy related peripheral neuritis, second trimester
        "O26823",  # Pregnancy related peripheral neuritis, third trimester
        "O26829",  # Pregnancy related peripheral neuritis, unspecified trimester
        "O26831",  # Pregnancy related renal disease, first trimester
        "O26832",  # Pregnancy related renal disease, second trimester
        "O26833",  # Pregnancy related renal disease, third trimester
        "O26839",  # Pregnancy related renal disease, unspecified trimester
        "O26841",  # Uterine size-date discrepancy, first trimester
        "O26842",  # Uterine size-date discrepancy, second trimester
        "O26843",  # Uterine size-date discrepancy, third trimester
        "O26849",  # Uterine size-date discrepancy, unspecified trimester
        "O26851",  # Spotting complicating pregnancy, first trimester
        "O26852",  # Spotting complicating pregnancy, second trimester
        "O26853",  # Spotting complicating pregnancy, third trimester
        "O26859",  # Spotting complicating pregnancy, unspecified trimester
        "O2686",  # Pruritic urticarial papules and plaques of pregnancy (PUPPP)
        "O26872",  # Cervical shortening, second trimester
        "O26873",  # Cervical shortening, third trimester
        "O26879",  # Cervical shortening, unspecified trimester
        "O26891",  # Other specified pregnancy related conditions, first trimester
        "O26892",  # Other specified pregnancy related conditions, second trimester
        "O26893",  # Other specified pregnancy related conditions, third trimester
        "O26899",  # Other specified pregnancy related conditions, unspecified trimester
        "O2690",  # Pregnancy related conditions, unspecified, unspecified trimester
        "O2691",  # Pregnancy related conditions, unspecified, first trimester
        "O2692",  # Pregnancy related conditions, unspecified, second trimester
        "O2693",  # Pregnancy related conditions, unspecified, third trimester
        "O280",  # Abnormal hematological finding on antenatal screening of mother
        "O281",  # Abnormal biochemical finding on antenatal screening of mother
        "O282",  # Abnormal cytological finding on antenatal screening of mother
        "O283",  # Abnormal ultrasonic finding on antenatal screening of mother
        "O284",  # Abnormal radiological finding on antenatal screening of mother
        "O285",  # Abnormal chromosomal and genetic finding on antenatal screening of mother
        "O288",  # Other abnormal findings on antenatal screening of mother
        "O289",  # Unspecified abnormal findings on antenatal screening of mother
        "O29011",  # Aspiration pneumonitis due to anesthesia during pregnancy, first trimester
        "O29012",  # Aspiration pneumonitis due to anesthesia during pregnancy, second trimester
        "O29013",  # Aspiration pneumonitis due to anesthesia during pregnancy, third trimester
        "O29019",  # Aspiration pneumonitis due to anesthesia during pregnancy, unspecified trimester
        "O29021",  # Pressure collapse of lung due to anesthesia during pregnancy, first trimester
        "O29022",  # Pressure collapse of lung due to anesthesia during pregnancy, second trimester
        "O29023",  # Pressure collapse of lung due to anesthesia during pregnancy, third trimester
        "O29029",  # Pressure collapse of lung due to anesthesia during pregnancy, unspecified trimester
        "O29091",  # Other pulmonary complications of anesthesia during pregnancy, first trimester
        "O29092",  # Other pulmonary complications of anesthesia during pregnancy, second trimester
        "O29093",  # Other pulmonary complications of anesthesia during pregnancy, third trimester
        "O29099",  # Other pulmonary complications of anesthesia during pregnancy, unspecified trimester
        "O29111",  # Cardiac arrest due to anesthesia during pregnancy, first trimester
        "O29112",  # Cardiac arrest due to anesthesia during pregnancy, second trimester
        "O29113",  # Cardiac arrest due to anesthesia during pregnancy, third trimester
        "O29119",  # Cardiac arrest due to anesthesia during pregnancy, unspecified trimester
        "O29121",  # Cardiac failure due to anesthesia during pregnancy, first trimester
        "O29122",  # Cardiac failure due to anesthesia during pregnancy, second trimester
        "O29123",  # Cardiac failure due to anesthesia during pregnancy, third trimester
        "O29129",  # Cardiac failure due to anesthesia during pregnancy, unspecified trimester
        "O29191",  # Other cardiac complications of anesthesia during pregnancy, first trimester
        "O29192",  # Other cardiac complications of anesthesia during pregnancy, second trimester
        "O29193",  # Other cardiac complications of anesthesia during pregnancy, third trimester
        "O29199",  # Other cardiac complications of anesthesia during pregnancy, unspecified trimester
        "O29211",  # Cerebral anoxia due to anesthesia during pregnancy, first trimester
        "O29212",  # Cerebral anoxia due to anesthesia during pregnancy, second trimester
        "O29213",  # Cerebral anoxia due to anesthesia during pregnancy, third trimester
        "O29219",  # Cerebral anoxia due to anesthesia during pregnancy, unspecified trimester
        "O29291",  # Other central nervous system complications of anesthesia during pregnancy, first trimester
        "O29292",  # Other central nervous system complications of anesthesia during pregnancy, second trimester
        "O29293",  # Other central nervous system complications of anesthesia during pregnancy, third trimester
        "O29299",  # Other central nervous system complications of anesthesia during pregnancy, unspecified trimester
        "O293X1",  # Toxic reaction to local anesthesia during pregnancy, first trimester
        "O293X2",  # Toxic reaction to local anesthesia during pregnancy, second trimester
        "O293X3",  # Toxic reaction to local anesthesia during pregnancy, third trimester
        "O293X9",  # Toxic reaction to local anesthesia during pregnancy, unspecified trimester
        "O2940",  # Spinal and epidural anesthesia induced headache during pregnancy, unspecified trimester
        "O2941",  # Spinal and epidural anesthesia induced headache during pregnancy, first trimester
        "O2942",  # Spinal and epidural anesthesia induced headache during pregnancy, second trimester
        "O2943",  # Spinal and epidural anesthesia induced headache during pregnancy, third trimester
        "O295X1",  # Other complications of spinal and epidural anesthesia during pregnancy, first trimester
        "O295X2",  # Other complications of spinal and epidural anesthesia during pregnancy, second trimester
        "O295X3",  # Other complications of spinal and epidural anesthesia during pregnancy, third trimester
        "O295X9",  # Other complications of spinal and epidural anesthesia during pregnancy, unspecified trimester
        "O2960",  # Failed or difficult intubation for anesthesia during pregnancy, unspecified trimester
        "O2961",  # Failed or difficult intubation for anesthesia during pregnancy, first trimester
        "O2962",  # Failed or difficult intubation for anesthesia during pregnancy, second trimester
        "O2963",  # Failed or difficult intubation for anesthesia during pregnancy, third trimester
        "O298X1",  # Other complications of anesthesia during pregnancy, first trimester
        "O298X2",  # Other complications of anesthesia during pregnancy, second trimester
        "O298X3",  # Other complications of anesthesia during pregnancy, third trimester
        "O298X9",  # Other complications of anesthesia during pregnancy, unspecified trimester
        "O2990",  # Unspecified complication of anesthesia during pregnancy, unspecified trimester
        "O2991",  # Unspecified complication of anesthesia during pregnancy, first trimester
        "O2992",  # Unspecified complication of anesthesia during pregnancy, second trimester
        "O2993",  # Unspecified complication of anesthesia during pregnancy, third trimester
        "O30001",  # Twin pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, first trimester
        "O30002",  # Twin pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, second trimester
        "O30003",  # Twin pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, third trimester
        "O30009",  # Twin pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, unspecified trimester
        "O30011",  # Twin pregnancy, monochorionic/monoamniotic, first trimester
        "O30012",  # Twin pregnancy, monochorionic/monoamniotic, second trimester
        "O30013",  # Twin pregnancy, monochorionic/monoamniotic, third trimester
        "O30019",  # Twin pregnancy, monochorionic/monoamniotic, unspecified trimester
        "O30021",  # Conjoined twin pregnancy, first trimester
        "O30022",  # Conjoined twin pregnancy, second trimester
        "O30023",  # Conjoined twin pregnancy, third trimester
        "O30029",  # Conjoined twin pregnancy, unspecified trimester
        "O30031",  # Twin pregnancy, monochorionic/diamniotic, first trimester
        "O30032",  # Twin pregnancy, monochorionic/diamniotic, second trimester
        "O30033",  # Twin pregnancy, monochorionic/diamniotic, third trimester
        "O30039",  # Twin pregnancy, monochorionic/diamniotic, unspecified trimester
        "O30041",  # Twin pregnancy, dichorionic/diamniotic, first trimester
        "O30042",  # Twin pregnancy, dichorionic/diamniotic, second trimester
        "O30043",  # Twin pregnancy, dichorionic/diamniotic, third trimester
        "O30049",  # Twin pregnancy, dichorionic/diamniotic, unspecified trimester
        "O30091",  # Twin pregnancy, unable to determine number of placenta and number of amniotic sacs, first trimester
        "O30092",  # Twin pregnancy, unable to determine number of placenta and number of amniotic sacs, second trimester
        "O30093",  # Twin pregnancy, unable to determine number of placenta and number of amniotic sacs, third trimester
        "O30099",  # Twin pregnancy, unable to determine number of placenta and number of amniotic sacs, unspecified trimester
        "O30101",  # Triplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, first trimester
        "O30102",  # Triplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, second trimester
        "O30103",  # Triplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, third trimester
        "O30109",  # Triplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, unspecified trimester
        "O30111",  # Triplet pregnancy with two or more monochorionic fetuses, first trimester
        "O30112",  # Triplet pregnancy with two or more monochorionic fetuses, second trimester
        "O30113",  # Triplet pregnancy with two or more monochorionic fetuses, third trimester
        "O30119",  # Triplet pregnancy with two or more monochorionic fetuses, unspecified trimester
        "O30121",  # Triplet pregnancy with two or more monoamniotic fetuses, first trimester
        "O30122",  # Triplet pregnancy with two or more monoamniotic fetuses, second trimester
        "O30123",  # Triplet pregnancy with two or more monoamniotic fetuses, third trimester
        "O30129",  # Triplet pregnancy with two or more monoamniotic fetuses, unspecified trimester
        "O30131",  # Triplet pregnancy, trichorionic/triamniotic, first trimester
        "O30132",  # Triplet pregnancy, trichorionic/triamniotic, second trimester
        "O30133",  # Triplet pregnancy, trichorionic/triamniotic, third trimester
        "O30139",  # Triplet pregnancy, trichorionic/triamniotic, unspecified trimester
        "O30191",  # Triplet pregnancy, unable to determine number of placenta and number of amniotic sacs, first trimester
        "O30192",  # Triplet pregnancy, unable to determine number of placenta and number of amniotic sacs, second trimester
        "O30193",  # Triplet pregnancy, unable to determine number of placenta and number of amniotic sacs, third trimester
        "O30199",  # Triplet pregnancy, unable to determine number of placenta and number of amniotic sacs, unspecified trimester
        "O30201",  # Quadruplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, first trimester
        "O30202",  # Quadruplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, second trimester
        "O30203",  # Quadruplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, third trimester
        "O30209",  # Quadruplet pregnancy, unspecified number of placenta and unspecified number of amniotic sacs, unspecified trimester
        "O30211",  # Quadruplet pregnancy with two or more monochorionic fetuses, first trimester
        "O30212",  # Quadruplet pregnancy with two or more monochorionic fetuses, second trimester
        "O30213",  # Quadruplet pregnancy with two or more monochorionic fetuses, third trimester
        "O30219",  # Quadruplet pregnancy with two or more monochorionic fetuses, unspecified trimester
        "O30221",  # Quadruplet pregnancy with two or more monoamniotic fetuses, first trimester
        "O30222",  # Quadruplet pregnancy with two or more monoamniotic fetuses, second trimester
        "O30223",  # Quadruplet pregnancy with two or more monoamniotic fetuses, third trimester
        "O30229",  # Quadruplet pregnancy with two or more monoamniotic fetuses, unspecified trimester
        "O30231",  # Quadruplet pregnancy, quadrachorionic/quadra-amniotic, first trimester
        "O30232",  # Quadruplet pregnancy, quadrachorionic/quadra-amniotic, second trimester
        "O30233",  # Quadruplet pregnancy, quadrachorionic/quadra-amniotic, third trimester
        "O30239",  # Quadruplet pregnancy, quadrachorionic/quadra-amniotic, unspecified trimester
        "O30291",  # Quadruplet pregnancy, unable to determine number of placenta and number of amniotic sacs, first trimester
        "O30292",  # Quadruplet pregnancy, unable to determine number of placenta and number of amniotic sacs, second trimester
        "O30293",  # Quadruplet pregnancy, unable to determine number of placenta and number of amniotic sacs, third trimester
        "O30299",  # Quadruplet pregnancy, unable to determine number of placenta and number of amniotic sacs, unspecified trimester
        "O30801",  # Other specified multiple gestation, unspecified number of placenta and unspecified number of amniotic sacs, first trimester
        "O30802",  # Other specified multiple gestation, unspecified number of placenta and unspecified number of amniotic sacs, second trimester
        "O30803",  # Other specified multiple gestation, unspecified number of placenta and unspecified number of amniotic sacs, third trimester
        "O30809",  # Other specified multiple gestation, unspecified number of placenta and unspecified number of amniotic sacs, unspecified trimester
        "O30811",  # Other specified multiple gestation with two or more monochorionic fetuses, first trimester
        "O30812",  # Other specified multiple gestation with two or more monochorionic fetuses, second trimester
        "O30813",  # Other specified multiple gestation with two or more monochorionic fetuses, third trimester
        "O30819",  # Other specified multiple gestation with two or more monochorionic fetuses, unspecified trimester
        "O30821",  # Other specified multiple gestation with two or more monoamniotic fetuses, first trimester
        "O30822",  # Other specified multiple gestation with two or more monoamniotic fetuses, second trimester
        "O30823",  # Other specified multiple gestation with two or more monoamniotic fetuses, third trimester
        "O30829",  # Other specified multiple gestation with two or more monoamniotic fetuses, unspecified trimester
        "O30831",  # Other specified multiple gestation, number of chorions and amnions are both equal to the number of fetuses, first trimester
        "O30832",  # Other specified multiple gestation, number of chorions and amnions are both equal to the number of fetuses, second trimester
        "O30833",  # Other specified multiple gestation, number of chorions and amnions are both equal to the number of fetuses, third trimester
        "O30839",  # Other specified multiple gestation, number of chorions and amnions are both equal to the number of fetuses, unspecified trimester
        "O30891",  # Other specified multiple gestation, unable to determine number of placenta and number of amniotic sacs, first trimester
        "O30892",  # Other specified multiple gestation, unable to determine number of placenta and number of amniotic sacs, second trimester
        "O30893",  # Other specified multiple gestation, unable to determine number of placenta and number of amniotic sacs, third trimester
        "O30899",  # Other specified multiple gestation, unable to determine number of placenta and number of amniotic sacs, unspecified trimester
        "O3090",  # Multiple gestation, unspecified, unspecified trimester
        "O3091",  # Multiple gestation, unspecified, first trimester
        "O3092",  # Multiple gestation, unspecified, second trimester
        "O3093",  # Multiple gestation, unspecified, third trimester
        "O3100X0",  # Papyraceous fetus, unspecified trimester, not applicable or unspecified
        "O3100X1",  # Papyraceous fetus, unspecified trimester, fetus 1
        "O3100X2",  # Papyraceous fetus, unspecified trimester, fetus 2
        "O3100X3",  # Papyraceous fetus, unspecified trimester, fetus 3
        "O3100X4",  # Papyraceous fetus, unspecified trimester, fetus 4
        "O3100X5",  # Papyraceous fetus, unspecified trimester, fetus 5
        "O3100X9",  # Papyraceous fetus, unspecified trimester, other fetus
        "O3101X0",  # Papyraceous fetus, first trimester, not applicable or unspecified
        "O3101X1",  # Papyraceous fetus, first trimester, fetus 1
        "O3101X2",  # Papyraceous fetus, first trimester, fetus 2
        "O3101X3",  # Papyraceous fetus, first trimester, fetus 3
        "O3101X4",  # Papyraceous fetus, first trimester, fetus 4
        "O3101X5",  # Papyraceous fetus, first trimester, fetus 5
        "O3101X9",  # Papyraceous fetus, first trimester, other fetus
        "O3102X0",  # Papyraceous fetus, second trimester, not applicable or unspecified
        "O3102X1",  # Papyraceous fetus, second trimester, fetus 1
        "O3102X2",  # Papyraceous fetus, second trimester, fetus 2
        "O3102X3",  # Papyraceous fetus, second trimester, fetus 3
        "O3102X4",  # Papyraceous fetus, second trimester, fetus 4
        "O3102X5",  # Papyraceous fetus, second trimester, fetus 5
        "O3102X9",  # Papyraceous fetus, second trimester, other fetus
        "O3103X0",  # Papyraceous fetus, third trimester, not applicable or unspecified
        "O3103X1",  # Papyraceous fetus, third trimester, fetus 1
        "O3103X2",  # Papyraceous fetus, third trimester, fetus 2
        "O3103X3",  # Papyraceous fetus, third trimester, fetus 3
        "O3103X4",  # Papyraceous fetus, third trimester, fetus 4
        "O3103X5",  # Papyraceous fetus, third trimester, fetus 5
        "O3103X9",  # Papyraceous fetus, third trimester, other fetus
        "O3110X0",  # Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, not applicable or unspecified
        "O3110X1",  # Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, fetus 1
        "O3110X2",  # Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, fetus 2
        "O3110X3",  # Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, fetus 3
        "O3110X4",  # Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, fetus 4
        "O3110X5",  # Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, fetus 5
        "O3110X9",  # Continuing pregnancy after spontaneous abortion of one fetus or more, unspecified trimester, other fetus
        "O3111X0",  # Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, not applicable or unspecified
        "O3111X1",  # Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, fetus 1
        "O3111X2",  # Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, fetus 2
        "O3111X3",  # Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, fetus 3
        "O3111X4",  # Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, fetus 4
        "O3111X5",  # Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, fetus 5
        "O3111X9",  # Continuing pregnancy after spontaneous abortion of one fetus or more, first trimester, other fetus
        "O3112X0",  # Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, not applicable or unspecified
        "O3112X1",  # Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, fetus 1
        "O3112X2",  # Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, fetus 2
        "O3112X3",  # Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, fetus 3
        "O3112X4",  # Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, fetus 4
        "O3112X5",  # Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, fetus 5
        "O3112X9",  # Continuing pregnancy after spontaneous abortion of one fetus or more, second trimester, other fetus
        "O3113X0",  # Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, not applicable or unspecified
        "O3113X1",  # Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, fetus 1
        "O3113X2",  # Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, fetus 2
        "O3113X3",  # Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, fetus 3
        "O3113X4",  # Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, fetus 4
        "O3113X5",  # Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, fetus 5
        "O3113X9",  # Continuing pregnancy after spontaneous abortion of one fetus or more, third trimester, other fetus
        "O3120X0",  # Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, not applicable or unspecified
        "O3120X1",  # Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, fetus 1
        "O3120X2",  # Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, fetus 2
        "O3120X3",  # Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, fetus 3
        "O3120X4",  # Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, fetus 4
        "O3120X5",  # Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, fetus 5
        "O3120X9",  # Continuing pregnancy after intrauterine death of one fetus or more, unspecified trimester, other fetus
        "O3121X0",  # Continuing pregnancy after intrauterine death of one fetus or more, first trimester, not applicable or unspecified
        "O3121X1",  # Continuing pregnancy after intrauterine death of one fetus or more, first trimester, fetus 1
        "O3121X2",  # Continuing pregnancy after intrauterine death of one fetus or more, first trimester, fetus 2
        "O3121X3",  # Continuing pregnancy after intrauterine death of one fetus or more, first trimester, fetus 3
        "O3121X4",  # Continuing pregnancy after intrauterine death of one fetus or more, first trimester, fetus 4
        "O3121X5",  # Continuing pregnancy after intrauterine death of one fetus or more, first trimester, fetus 5
        "O3121X9",  # Continuing pregnancy after intrauterine death of one fetus or more, first trimester, other fetus
        "O3122X0",  # Continuing pregnancy after intrauterine death of one fetus or more, second trimester, not applicable or unspecified
        "O3122X1",  # Continuing pregnancy after intrauterine death of one fetus or more, second trimester, fetus 1
        "O3122X2",  # Continuing pregnancy after intrauterine death of one fetus or more, second trimester, fetus 2
        "O3122X3",  # Continuing pregnancy after intrauterine death of one fetus or more, second trimester, fetus 3
        "O3122X4",  # Continuing pregnancy after intrauterine death of one fetus or more, second trimester, fetus 4
        "O3122X5",  # Continuing pregnancy after intrauterine death of one fetus or more, second trimester, fetus 5
        "O3122X9",  # Continuing pregnancy after intrauterine death of one fetus or more, second trimester, other fetus
        "O3123X0",  # Continuing pregnancy after intrauterine death of one fetus or more, third trimester, not applicable or unspecified
        "O3123X1",  # Continuing pregnancy after intrauterine death of one fetus or more, third trimester, fetus 1
        "O3123X2",  # Continuing pregnancy after intrauterine death of one fetus or more, third trimester, fetus 2
        "O3123X3",  # Continuing pregnancy after intrauterine death of one fetus or more, third trimester, fetus 3
        "O3123X4",  # Continuing pregnancy after intrauterine death of one fetus or more, third trimester, fetus 4
        "O3123X5",  # Continuing pregnancy after intrauterine death of one fetus or more, third trimester, fetus 5
        "O3123X9",  # Continuing pregnancy after intrauterine death of one fetus or more, third trimester, other fetus
        "O3130X0",  # Continuing pregnancy after elective fetal reduction of one fetus or more, unspecified trimester, not applicable or unspecified
        "O3130X1",  # Continuing pregnancy after elective fetal reduction of one fetus or more, unspecified trimester, fetus 1
        "O3130X2",  # Continuing pregnancy after elective fetal reduction of one fetus or more, unspecified trimester, fetus 2
        "O3130X3",  # Continuing pregnancy after elective fetal reduction of one fetus or more, unspecified trimester, fetus 3
        "O3130X4",  # Continuing pregnancy after elective fetal reduction of one fetus or more, unspecified trimester, fetus 4
        "O3130X5",  # Continuing pregnancy after elective fetal reduction of one fetus or more, unspecified trimester, fetus 5
        "O3130X9",  # Continuing pregnancy after elective fetal reduction of one fetus or more, unspecified trimester, other fetus
        "O3131X0",  # Continuing pregnancy after elective fetal reduction of one fetus or more, first trimester, not applicable or unspecified
        "O3131X1",  # Continuing pregnancy after elective fetal reduction of one fetus or more, first trimester, fetus 1
        "O3131X2",  # Continuing pregnancy after elective fetal reduction of one fetus or more, first trimester, fetus 2
        "O3131X3",  # Continuing pregnancy after elective fetal reduction of one fetus or more, first trimester, fetus 3
        "O3131X4",  # Continuing pregnancy after elective fetal reduction of one fetus or more, first trimester, fetus 4
        "O3131X5",  # Continuing pregnancy after elective fetal reduction of one fetus or more, first trimester, fetus 5
        "O3131X9",  # Continuing pregnancy after elective fetal reduction of one fetus or more, first trimester, other fetus
        "O3132X0",  # Continuing pregnancy after elective fetal reduction of one fetus or more, second trimester, not applicable or unspecified
        "O3132X1",  # Continuing pregnancy after elective fetal reduction of one fetus or more, second trimester, fetus 1
        "O3132X2",  # Continuing pregnancy after elective fetal reduction of one fetus or more, second trimester, fetus 2
        "O3132X3",  # Continuing pregnancy after elective fetal reduction of one fetus or more, second trimester, fetus 3
        "O3132X4",  # Continuing pregnancy after elective fetal reduction of one fetus or more, second trimester, fetus 4
        "O3132X5",  # Continuing pregnancy after elective fetal reduction of one fetus or more, second trimester, fetus 5
        "O3132X9",  # Continuing pregnancy after elective fetal reduction of one fetus or more, second trimester, other fetus
        "O3133X0",  # Continuing pregnancy after elective fetal reduction of one fetus or more, third trimester, not applicable or unspecified
        "O3133X1",  # Continuing pregnancy after elective fetal reduction of one fetus or more, third trimester, fetus 1
        "O3133X2",  # Continuing pregnancy after elective fetal reduction of one fetus or more, third trimester, fetus 2
        "O3133X3",  # Continuing pregnancy after elective fetal reduction of one fetus or more, third trimester, fetus 3
        "O3133X4",  # Continuing pregnancy after elective fetal reduction of one fetus or more, third trimester, fetus 4
        "O3133X5",  # Continuing pregnancy after elective fetal reduction of one fetus or more, third trimester, fetus 5
        "O3133X9",  # Continuing pregnancy after elective fetal reduction of one fetus or more, third trimester, other fetus
        "O318X10",  # Other complications specific to multiple gestation, first trimester, not applicable or unspecified
        "O318X11",  # Other complications specific to multiple gestation, first trimester, fetus 1
        "O318X12",  # Other complications specific to multiple gestation, first trimester, fetus 2
        "O318X13",  # Other complications specific to multiple gestation, first trimester, fetus 3
        "O318X14",  # Other complications specific to multiple gestation, first trimester, fetus 4
        "O318X15",  # Other complications specific to multiple gestation, first trimester, fetus 5
        "O318X19",  # Other complications specific to multiple gestation, first trimester, other fetus
        "O318X20",  # Other complications specific to multiple gestation, second trimester, not applicable or unspecified
        "O318X21",  # Other complications specific to multiple gestation, second trimester, fetus 1
        "O318X22",  # Other complications specific to multiple gestation, second trimester, fetus 2
        "O318X23",  # Other complications specific to multiple gestation, second trimester, fetus 3
        "O318X24",  # Other complications specific to multiple gestation, second trimester, fetus 4
        "O318X25",  # Other complications specific to multiple gestation, second trimester, fetus 5
        "O318X29",  # Other complications specific to multiple gestation, second trimester, other fetus
        "O318X30",  # Other complications specific to multiple gestation, third trimester, not applicable or unspecified
        "O318X31",  # Other complications specific to multiple gestation, third trimester, fetus 1
        "O318X32",  # Other complications specific to multiple gestation, third trimester, fetus 2
        "O318X33",  # Other complications specific to multiple gestation, third trimester, fetus 3
        "O318X34",  # Other complications specific to multiple gestation, third trimester, fetus 4
        "O318X35",  # Other complications specific to multiple gestation, third trimester, fetus 5
        "O318X39",  # Other complications specific to multiple gestation, third trimester, other fetus
        "O318X90",  # Other complications specific to multiple gestation, unspecified trimester, not applicable or unspecified
        "O318X91",  # Other complications specific to multiple gestation, unspecified trimester, fetus 1
        "O318X92",  # Other complications specific to multiple gestation, unspecified trimester, fetus 2
        "O318X93",  # Other complications specific to multiple gestation, unspecified trimester, fetus 3
        "O318X94",  # Other complications specific to multiple gestation, unspecified trimester, fetus 4
        "O318X95",  # Other complications specific to multiple gestation, unspecified trimester, fetus 5
        "O318X99",  # Other complications specific to multiple gestation, unspecified trimester, other fetus
        "O320XX0",  # Maternal care for unstable lie, not applicable or unspecified
        "O320XX1",  # Maternal care for unstable lie, fetus 1
        "O320XX2",  # Maternal care for unstable lie, fetus 2
        "O320XX3",  # Maternal care for unstable lie, fetus 3
        "O320XX4",  # Maternal care for unstable lie, fetus 4
        "O320XX5",  # Maternal care for unstable lie, fetus 5
        "O320XX9",  # Maternal care for unstable lie, other fetus
        "O321XX0",  # Maternal care for breech presentation, not applicable or unspecified
        "O321XX1",  # Maternal care for breech presentation, fetus 1
        "O321XX2",  # Maternal care for breech presentation, fetus 2
        "O321XX3",  # Maternal care for breech presentation, fetus 3
        "O321XX4",  # Maternal care for breech presentation, fetus 4
        "O321XX5",  # Maternal care for breech presentation, fetus 5
        "O321XX9",  # Maternal care for breech presentation, other fetus
        "O322XX0",  # Maternal care for transverse and oblique lie, not applicable or unspecified
        "O322XX1",  # Maternal care for transverse and oblique lie, fetus 1
        "O322XX2",  # Maternal care for transverse and oblique lie, fetus 2
        "O322XX3",  # Maternal care for transverse and oblique lie, fetus 3
        "O322XX4",  # Maternal care for transverse and oblique lie, fetus 4
        "O322XX5",  # Maternal care for transverse and oblique lie, fetus 5
        "O322XX9",  # Maternal care for transverse and oblique lie, other fetus
        "O323XX0",  # Maternal care for face, brow and chin presentation, not applicable or unspecified
        "O323XX1",  # Maternal care for face, brow and chin presentation, fetus 1
        "O323XX2",  # Maternal care for face, brow and chin presentation, fetus 2
        "O323XX3",  # Maternal care for face, brow and chin presentation, fetus 3
        "O323XX4",  # Maternal care for face, brow and chin presentation, fetus 4
        "O323XX5",  # Maternal care for face, brow and chin presentation, fetus 5
        "O323XX9",  # Maternal care for face, brow and chin presentation, other fetus
        "O324XX0",  # Maternal care for high head at term, not applicable or unspecified
        "O324XX1",  # Maternal care for high head at term, fetus 1
        "O324XX2",  # Maternal care for high head at term, fetus 2
        "O324XX3",  # Maternal care for high head at term, fetus 3
        "O324XX4",  # Maternal care for high head at term, fetus 4
        "O324XX5",  # Maternal care for high head at term, fetus 5
        "O324XX9",  # Maternal care for high head at term, other fetus
        "O326XX0",  # Maternal care for compound presentation, not applicable or unspecified
        "O326XX1",  # Maternal care for compound presentation, fetus 1
        "O326XX2",  # Maternal care for compound presentation, fetus 2
        "O326XX3",  # Maternal care for compound presentation, fetus 3
        "O326XX4",  # Maternal care for compound presentation, fetus 4
        "O326XX5",  # Maternal care for compound presentation, fetus 5
        "O326XX9",  # Maternal care for compound presentation, other fetus
        "O328XX0",  # Maternal care for other malpresentation of fetus, not applicable or unspecified
        "O328XX1",  # Maternal care for other malpresentation of fetus, fetus 1
        "O328XX2",  # Maternal care for other malpresentation of fetus, fetus 2
        "O328XX3",  # Maternal care for other malpresentation of fetus, fetus 3
        "O328XX4",  # Maternal care for other malpresentation of fetus, fetus 4
        "O328XX5",  # Maternal care for other malpresentation of fetus, fetus 5
        "O328XX9",  # Maternal care for other malpresentation of fetus, other fetus
        "O329XX0",  # Maternal care for malpresentation of fetus, unspecified, not applicable or unspecified
        "O329XX1",  # Maternal care for malpresentation of fetus, unspecified, fetus 1
        "O329XX2",  # Maternal care for malpresentation of fetus, unspecified, fetus 2
        "O329XX3",  # Maternal care for malpresentation of fetus, unspecified, fetus 3
        "O329XX4",  # Maternal care for malpresentation of fetus, unspecified, fetus 4
        "O329XX5",  # Maternal care for malpresentation of fetus, unspecified, fetus 5
        "O329XX9",  # Maternal care for malpresentation of fetus, unspecified, other fetus
        "O330",  # Maternal care for disproportion due to deformity of maternal pelvic bones
        "O331",  # Maternal care for disproportion due to generally contracted pelvis
        "O332",  # Maternal care for disproportion due to inlet contraction of pelvis
        "O333XX0",  # Maternal care for disproportion due to outlet contraction of pelvis, not applicable or unspecified
        "O333XX1",  # Maternal care for disproportion due to outlet contraction of pelvis, fetus 1
        "O333XX2",  # Maternal care for disproportion due to outlet contraction of pelvis, fetus 2
        "O333XX3",  # Maternal care for disproportion due to outlet contraction of pelvis, fetus 3
        "O333XX4",  # Maternal care for disproportion due to outlet contraction of pelvis, fetus 4
        "O333XX5",  # Maternal care for disproportion due to outlet contraction of pelvis, fetus 5
        "O333XX9",  # Maternal care for disproportion due to outlet contraction of pelvis, other fetus
        "O334XX0",  # Maternal care for disproportion of mixed maternal and fetal origin, not applicable or unspecified
        "O334XX1",  # Maternal care for disproportion of mixed maternal and fetal origin, fetus 1
        "O334XX2",  # Maternal care for disproportion of mixed maternal and fetal origin, fetus 2
        "O334XX3",  # Maternal care for disproportion of mixed maternal and fetal origin, fetus 3
        "O334XX4",  # Maternal care for disproportion of mixed maternal and fetal origin, fetus 4
        "O334XX5",  # Maternal care for disproportion of mixed maternal and fetal origin, fetus 5
        "O334XX9",  # Maternal care for disproportion of mixed maternal and fetal origin, other fetus
        "O335XX0",  # Maternal care for disproportion due to unusually large fetus, not applicable or unspecified
        "O335XX1",  # Maternal care for disproportion due to unusually large fetus, fetus 1
        "O335XX2",  # Maternal care for disproportion due to unusually large fetus, fetus 2
        "O335XX3",  # Maternal care for disproportion due to unusually large fetus, fetus 3
        "O335XX4",  # Maternal care for disproportion due to unusually large fetus, fetus 4
        "O335XX5",  # Maternal care for disproportion due to unusually large fetus, fetus 5
        "O335XX9",  # Maternal care for disproportion due to unusually large fetus, other fetus
        "O336XX0",  # Maternal care for disproportion due to hydrocephalic fetus, not applicable or unspecified
        "O336XX1",  # Maternal care for disproportion due to hydrocephalic fetus, fetus 1
        "O336XX2",  # Maternal care for disproportion due to hydrocephalic fetus, fetus 2
        "O336XX3",  # Maternal care for disproportion due to hydrocephalic fetus, fetus 3
        "O336XX4",  # Maternal care for disproportion due to hydrocephalic fetus, fetus 4
        "O336XX5",  # Maternal care for disproportion due to hydrocephalic fetus, fetus 5
        "O336XX9",  # Maternal care for disproportion due to hydrocephalic fetus, other fetus
        "O337XX0",  # Maternal care for disproportion due to other fetal deformities, not applicable or unspecified
        "O337XX1",  # Maternal care for disproportion due to other fetal deformities, fetus 1
        "O337XX2",  # Maternal care for disproportion due to other fetal deformities, fetus 2
        "O337XX3",  # Maternal care for disproportion due to other fetal deformities, fetus 3
        "O337XX4",  # Maternal care for disproportion due to other fetal deformities, fetus 4
        "O337XX5",  # Maternal care for disproportion due to other fetal deformities, fetus 5
        "O337XX9",  # Maternal care for disproportion due to other fetal deformities, other fetus
        "O338",  # Maternal care for disproportion of other origin
        "O339",  # Maternal care for disproportion, unspecified
        "O3400",  # Maternal care for unspecified congenital malformation of uterus, unspecified trimester
        "O3401",  # Maternal care for unspecified congenital malformation of uterus, first trimester
        "O3402",  # Maternal care for unspecified congenital malformation of uterus, second trimester
        "O3403",  # Maternal care for unspecified congenital malformation of uterus, third trimester
        "O3410",  # Maternal care for benign tumor of corpus uteri, unspecified trimester
        "O3411",  # Maternal care for benign tumor of corpus uteri, first trimester
        "O3412",  # Maternal care for benign tumor of corpus uteri, second trimester
        "O3413",  # Maternal care for benign tumor of corpus uteri, third trimester
        "O34211",  # Maternal care for low transverse scar from previous cesarean delivery
        "O34212",  # Maternal care for vertical scar from previous cesarean delivery
        "O34218",  # Maternal care for other type scar from previous cesarean delivery
        "O34219",  # Maternal care for unspecified type scar from previous cesarean delivery
        "O3422",  # Maternal care for cesarean scar defect (isthmocele)
        "O3429",  # Maternal care due to uterine scar from other previous surgery
        "O3430",  # Maternal care for cervical incompetence, unspecified trimester
        "O3431",  # Maternal care for cervical incompetence, first trimester
        "O3432",  # Maternal care for cervical incompetence, second trimester
        "O3433",  # Maternal care for cervical incompetence, third trimester
        "O3440",  # Maternal care for other abnormalities of cervix, unspecified trimester
        "O3441",  # Maternal care for other abnormalities of cervix, first trimester
        "O3442",  # Maternal care for other abnormalities of cervix, second trimester
        "O3443",  # Maternal care for other abnormalities of cervix, third trimester
        "O34511",  # Maternal care for incarceration of gravid uterus, first trimester
        "O34512",  # Maternal care for incarceration of gravid uterus, second trimester
        "O34513",  # Maternal care for incarceration of gravid uterus, third trimester
        "O34519",  # Maternal care for incarceration of gravid uterus, unspecified trimester
        "O34521",  # Maternal care for prolapse of gravid uterus, first trimester
        "O34522",  # Maternal care for prolapse of gravid uterus, second trimester
        "O34523",  # Maternal care for prolapse of gravid uterus, third trimester
        "O34529",  # Maternal care for prolapse of gravid uterus, unspecified trimester
        "O34531",  # Maternal care for retroversion of gravid uterus, first trimester
        "O34532",  # Maternal care for retroversion of gravid uterus, second trimester
        "O34533",  # Maternal care for retroversion of gravid uterus, third trimester
        "O34539",  # Maternal care for retroversion of gravid uterus, unspecified trimester
        "O34591",  # Maternal care for other abnormalities of gravid uterus, first trimester
        "O34592",  # Maternal care for other abnormalities of gravid uterus, second trimester
        "O34593",  # Maternal care for other abnormalities of gravid uterus, third trimester
        "O34599",  # Maternal care for other abnormalities of gravid uterus, unspecified trimester
        "O3460",  # Maternal care for abnormality of vagina, unspecified trimester
        "O3461",  # Maternal care for abnormality of vagina, first trimester
        "O3462",  # Maternal care for abnormality of vagina, second trimester
        "O3463",  # Maternal care for abnormality of vagina, third trimester
        "O3470",  # Maternal care for abnormality of vulva and perineum, unspecified trimester
        "O3471",  # Maternal care for abnormality of vulva and perineum, first trimester
        "O3472",  # Maternal care for abnormality of vulva and perineum, second trimester
        "O3473",  # Maternal care for abnormality of vulva and perineum, third trimester
        "O3480",  # Maternal care for other abnormalities of pelvic organs, unspecified trimester
        "O3481",  # Maternal care for other abnormalities of pelvic organs, first trimester
        "O3482",  # Maternal care for other abnormalities of pelvic organs, second trimester
        "O3483",  # Maternal care for other abnormalities of pelvic organs, third trimester
        "O3490",  # Maternal care for abnormality of pelvic organ, unspecified, unspecified trimester
        "O3491",  # Maternal care for abnormality of pelvic organ, unspecified, first trimester
        "O3492",  # Maternal care for abnormality of pelvic organ, unspecified, second trimester
        "O3493",  # Maternal care for abnormality of pelvic organ, unspecified, third trimester
        "O3500X0",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, unspecified, not applicable or unspecified
        "O3500X1",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, unspecified, fetus 1
        "O3500X2",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, unspecified, fetus 2
        "O3500X3",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, unspecified, fetus 3
        "O3500X4",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, unspecified, fetus 4
        "O3500X5",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, unspecified, fetus 5
        "O3500X9",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, unspecified, other fetus
        "O3501X0",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, agenesis of the corpus callosum, not applicable or unspecified
        "O3501X1",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, agenesis of the corpus callosum, fetus 1
        "O3501X2",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, agenesis of the corpus callosum, fetus 2
        "O3501X3",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, agenesis of the corpus callosum, fetus 3
        "O3501X4",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, agenesis of the corpus callosum, fetus 4
        "O3501X5",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, agenesis of the corpus callosum, fetus 5
        "O3501X9",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, agenesis of the corpus callosum, other fetus
        "O3502X0",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, anencephaly, not applicable or unspecified
        "O3502X1",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, anencephaly, fetus 1
        "O3502X2",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, anencephaly, fetus 2
        "O3502X3",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, anencephaly, fetus 3
        "O3502X4",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, anencephaly, fetus 4
        "O3502X5",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, anencephaly, fetus 5
        "O3502X9",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, anencephaly, other fetus
        "O3503X0",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, choroid plexus cysts, not applicable or unspecified
        "O3503X1",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, choroid plexus cysts, fetus 1
        "O3503X2",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, choroid plexus cysts, fetus 2
        "O3503X3",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, choroid plexus cysts, fetus 3
        "O3503X4",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, choroid plexus cysts, fetus 4
        "O3503X5",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, choroid plexus cysts, fetus 5
        "O3503X9",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, choroid plexus cysts, other fetus
        "O3504X0",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, encephalocele, not applicable or unspecified
        "O3504X1",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, encephalocele, fetus 1
        "O3504X2",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, encephalocele, fetus 2
        "O3504X3",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, encephalocele, fetus 3
        "O3504X4",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, encephalocele, fetus 4
        "O3504X5",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, encephalocele, fetus 5
        "O3504X9",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, encephalocele, other fetus
        "O3505X0",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, holoprosencephaly, not applicable or unspecified
        "O3505X1",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, holoprosencephaly, fetus 1
        "O3505X2",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, holoprosencephaly, fetus 2
        "O3505X3",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, holoprosencephaly, fetus 3
        "O3505X4",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, holoprosencephaly, fetus 4
        "O3505X5",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, holoprosencephaly, fetus 5
        "O3505X9",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, holoprosencephaly, other fetus
        "O3506X0",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, hydrocephaly, not applicable or unspecified
        "O3506X1",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, hydrocephaly, fetus 1
        "O3506X2",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, hydrocephaly, fetus 2
        "O3506X3",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, hydrocephaly, fetus 3
        "O3506X4",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, hydrocephaly, fetus 4
        "O3506X5",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, hydrocephaly, fetus 5
        "O3506X9",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, hydrocephaly, other fetus
        "O3507X0",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, microcephaly, not applicable or unspecified
        "O3507X1",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, microcephaly, fetus 1
        "O3507X2",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, microcephaly, fetus 2
        "O3507X3",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, microcephaly, fetus 3
        "O3507X4",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, microcephaly, fetus 4
        "O3507X5",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, microcephaly, fetus 5
        "O3507X9",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, microcephaly, other fetus
        "O3508X0",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, spina bifida, not applicable or unspecified
        "O3508X1",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, spina bifida, fetus 1
        "O3508X2",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, spina bifida, fetus 2
        "O3508X3",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, spina bifida, fetus 3
        "O3508X4",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, spina bifida, fetus 4
        "O3508X5",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, spina bifida, fetus 5
        "O3508X9",  # Maternal care for (suspected) central nervous system malformation or damage in fetus, spina bifida, other fetus
        "O3509X0",  # Maternal care for (suspected) other central nervous system malformation or damage in fetus, not applicable or unspecified
        "O3509X1",  # Maternal care for (suspected) other central nervous system malformation or damage in fetus, fetus 1
        "O3509X2",  # Maternal care for (suspected) other central nervous system malformation or damage in fetus, fetus 2
        "O3509X3",  # Maternal care for (suspected) other central nervous system malformation or damage in fetus, fetus 3
        "O3509X4",  # Maternal care for (suspected) other central nervous system malformation or damage in fetus, fetus 4
        "O3509X5",  # Maternal care for (suspected) other central nervous system malformation or damage in fetus, fetus 5
        "O3509X9",  # Maternal care for (suspected) other central nervous system malformation or damage in fetus, other fetus
        "O3510X0",  # Maternal care for (suspected) chromosomal abnormality in fetus, unspecified, not applicable or unspecified
        "O3510X1",  # Maternal care for (suspected) chromosomal abnormality in fetus, unspecified, fetus 1
        "O3510X2",  # Maternal care for (suspected) chromosomal abnormality in fetus, unspecified, fetus 2
        "O3510X3",  # Maternal care for (suspected) chromosomal abnormality in fetus, unspecified, fetus 3
        "O3510X4",  # Maternal care for (suspected) chromosomal abnormality in fetus, unspecified, fetus 4
        "O3510X5",  # Maternal care for (suspected) chromosomal abnormality in fetus, unspecified, fetus 5
        "O3510X9",  # Maternal care for (suspected) chromosomal abnormality in fetus, unspecified, other fetus
        "O3511X0",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 13, not applicable or unspecified
        "O3511X1",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 13, fetus 1
        "O3511X2",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 13, fetus 2
        "O3511X3",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 13, fetus 3
        "O3511X4",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 13, fetus 4
        "O3511X5",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 13, fetus 5
        "O3511X9",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 13, other fetus
        "O3512X0",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 18, not applicable or unspecified
        "O3512X1",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 18, fetus 1
        "O3512X2",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 18, fetus 2
        "O3512X3",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 18, fetus 3
        "O3512X4",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 18, fetus 4
        "O3512X5",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 18, fetus 5
        "O3512X9",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 18, other fetus
        "O3513X0",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 21, not applicable or unspecified
        "O3513X1",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 21, fetus 1
        "O3513X2",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 21, fetus 2
        "O3513X3",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 21, fetus 3
        "O3513X4",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 21, fetus 4
        "O3513X5",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 21, fetus 5
        "O3513X9",  # Maternal care for (suspected) chromosomal abnormality in fetus, Trisomy 21, other fetus
        "O3514X0",  # Maternal care for (suspected) chromosomal abnormality in fetus, Turner Syndrome, not applicable or unspecified
        "O3514X1",  # Maternal care for (suspected) chromosomal abnormality in fetus, Turner Syndrome, fetus 1
        "O3514X2",  # Maternal care for (suspected) chromosomal abnormality in fetus, Turner Syndrome, fetus 2
        "O3514X3",  # Maternal care for (suspected) chromosomal abnormality in fetus, Turner Syndrome, fetus 3
        "O3514X4",  # Maternal care for (suspected) chromosomal abnormality in fetus, Turner Syndrome, fetus 4
        "O3514X5",  # Maternal care for (suspected) chromosomal abnormality in fetus, Turner Syndrome, fetus 5
        "O3514X9",  # Maternal care for (suspected) chromosomal abnormality in fetus, Turner Syndrome, other fetus
        "O3515X0",  # Maternal care for (suspected) chromosomal abnormality in fetus, sex chromosome abnormality, not applicable or unspecified
        "O3515X1",  # Maternal care for (suspected) chromosomal abnormality in fetus, sex chromosome abnormality, fetus 1
        "O3515X2",  # Maternal care for (suspected) chromosomal abnormality in fetus, sex chromosome abnormality, fetus 2
        "O3515X3",  # Maternal care for (suspected) chromosomal abnormality in fetus, sex chromosome abnormality, fetus 3
        "O3515X4",  # Maternal care for (suspected) chromosomal abnormality in fetus, sex chromosome abnormality, fetus 4
        "O3515X5",  # Maternal care for (suspected) chromosomal abnormality in fetus, sex chromosome abnormality, fetus 5
        "O3515X9",  # Maternal care for (suspected) chromosomal abnormality in fetus, sex chromosome abnormality, other fetus
        "O3519X0",  # Maternal care for (suspected) chromosomal abnormality in fetus, other chromosomal abnormality, not applicable or unspecified
        "O3519X1",  # Maternal care for (suspected) chromosomal abnormality in fetus, other chromosomal abnormality, fetus 1
        "O3519X2",  # Maternal care for (suspected) chromosomal abnormality in fetus, other chromosomal abnormality, fetus 2
        "O3519X3",  # Maternal care for (suspected) chromosomal abnormality in fetus, other chromosomal abnormality, fetus 3
        "O3519X4",  # Maternal care for (suspected) chromosomal abnormality in fetus, other chromosomal abnormality, fetus 4
        "O3519X5",  # Maternal care for (suspected) chromosomal abnormality in fetus, other chromosomal abnormality, fetus 5
        "O3519X9",  # Maternal care for (suspected) chromosomal abnormality in fetus, other chromosomal abnormality, other fetus
        "O352XX0",  # Maternal care for (suspected) hereditary disease in fetus, not applicable or unspecified
        "O352XX1",  # Maternal care for (suspected) hereditary disease in fetus, fetus 1
        "O352XX2",  # Maternal care for (suspected) hereditary disease in fetus, fetus 2
        "O352XX3",  # Maternal care for (suspected) hereditary disease in fetus, fetus 3
        "O352XX4",  # Maternal care for (suspected) hereditary disease in fetus, fetus 4
        "O352XX5",  # Maternal care for (suspected) hereditary disease in fetus, fetus 5
        "O352XX9",  # Maternal care for (suspected) hereditary disease in fetus, other fetus
        "O353XX0",  # Maternal care for (suspected) damage to fetus from viral disease in mother, not applicable or unspecified
        "O353XX1",  # Maternal care for (suspected) damage to fetus from viral disease in mother, fetus 1
        "O353XX2",  # Maternal care for (suspected) damage to fetus from viral disease in mother, fetus 2
        "O353XX3",  # Maternal care for (suspected) damage to fetus from viral disease in mother, fetus 3
        "O353XX4",  # Maternal care for (suspected) damage to fetus from viral disease in mother, fetus 4
        "O353XX5",  # Maternal care for (suspected) damage to fetus from viral disease in mother, fetus 5
        "O353XX9",  # Maternal care for (suspected) damage to fetus from viral disease in mother, other fetus
        "O354XX0",  # Maternal care for (suspected) damage to fetus from alcohol, not applicable or unspecified
        "O354XX1",  # Maternal care for (suspected) damage to fetus from alcohol, fetus 1
        "O354XX2",  # Maternal care for (suspected) damage to fetus from alcohol, fetus 2
        "O354XX3",  # Maternal care for (suspected) damage to fetus from alcohol, fetus 3
        "O354XX4",  # Maternal care for (suspected) damage to fetus from alcohol, fetus 4
        "O354XX5",  # Maternal care for (suspected) damage to fetus from alcohol, fetus 5
        "O354XX9",  # Maternal care for (suspected) damage to fetus from alcohol, other fetus
        "O355XX0",  # Maternal care for (suspected) damage to fetus by drugs, not applicable or unspecified
        "O355XX1",  # Maternal care for (suspected) damage to fetus by drugs, fetus 1
        "O355XX2",  # Maternal care for (suspected) damage to fetus by drugs, fetus 2
        "O355XX3",  # Maternal care for (suspected) damage to fetus by drugs, fetus 3
        "O355XX4",  # Maternal care for (suspected) damage to fetus by drugs, fetus 4
        "O355XX5",  # Maternal care for (suspected) damage to fetus by drugs, fetus 5
        "O355XX9",  # Maternal care for (suspected) damage to fetus by drugs, other fetus
        "O356XX0",  # Maternal care for (suspected) damage to fetus by radiation, not applicable or unspecified
        "O356XX1",  # Maternal care for (suspected) damage to fetus by radiation, fetus 1
        "O356XX2",  # Maternal care for (suspected) damage to fetus by radiation, fetus 2
        "O356XX3",  # Maternal care for (suspected) damage to fetus by radiation, fetus 3
        "O356XX4",  # Maternal care for (suspected) damage to fetus by radiation, fetus 4
        "O356XX5",  # Maternal care for (suspected) damage to fetus by radiation, fetus 5
        "O356XX9",  # Maternal care for (suspected) damage to fetus by radiation, other fetus
        "O357XX0",  # Maternal care for (suspected) damage to fetus by other medical procedures, not applicable or unspecified
        "O357XX1",  # Maternal care for (suspected) damage to fetus by other medical procedures, fetus 1
        "O357XX2",  # Maternal care for (suspected) damage to fetus by other medical procedures, fetus 2
        "O357XX3",  # Maternal care for (suspected) damage to fetus by other medical procedures, fetus 3
        "O357XX4",  # Maternal care for (suspected) damage to fetus by other medical procedures, fetus 4
        "O357XX5",  # Maternal care for (suspected) damage to fetus by other medical procedures, fetus 5
        "O357XX9",  # Maternal care for (suspected) damage to fetus by other medical procedures, other fetus
        "O358XX0",  # Maternal care for other (suspected) fetal abnormality and damage, not applicable or unspecified
        "O358XX1",  # Maternal care for other (suspected) fetal abnormality and damage, fetus 1
        "O358XX2",  # Maternal care for other (suspected) fetal abnormality and damage, fetus 2
        "O358XX3",  # Maternal care for other (suspected) fetal abnormality and damage, fetus 3
        "O358XX4",  # Maternal care for other (suspected) fetal abnormality and damage, fetus 4
        "O358XX5",  # Maternal care for other (suspected) fetal abnormality and damage, fetus 5
        "O358XX9",  # Maternal care for other (suspected) fetal abnormality and damage, other fetus
        "O359XX0",  # Maternal care for (suspected) fetal abnormality and damage, unspecified, not applicable or unspecified
        "O359XX1",  # Maternal care for (suspected) fetal abnormality and damage, unspecified, fetus 1
        "O359XX2",  # Maternal care for (suspected) fetal abnormality and damage, unspecified, fetus 2
        "O359XX3",  # Maternal care for (suspected) fetal abnormality and damage, unspecified, fetus 3
        "O359XX4",  # Maternal care for (suspected) fetal abnormality and damage, unspecified, fetus 4
        "O359XX5",  # Maternal care for (suspected) fetal abnormality and damage, unspecified, fetus 5
        "O359XX9",  # Maternal care for (suspected) fetal abnormality and damage, unspecified, other fetus
        "O35AXX0",  # Maternal care for other (suspected) fetal abnormality and damage, fetal facial anomalies, not applicable or unspecified
        "O35AXX1",  # Maternal care for other (suspected) fetal abnormality and damage, fetal facial anomalies, fetus 1
        "O35AXX2",  # Maternal care for other (suspected) fetal abnormality and damage, fetal facial anomalies, fetus 2
        "O35AXX3",  # Maternal care for other (suspected) fetal abnormality and damage, fetal facial anomalies, fetus 3
        "O35AXX4",  # Maternal care for other (suspected) fetal abnormality and damage, fetal facial anomalies, fetus 4
        "O35AXX5",  # Maternal care for other (suspected) fetal abnormality and damage, fetal facial anomalies, fetus 5
        "O35AXX9",  # Maternal care for other (suspected) fetal abnormality and damage, fetal facial anomalies, other fetus
        "O35BXX0",  # Maternal care for other (suspected) fetal abnormality and damage, fetal cardiac anomalies, not applicable or unspecified
        "O35BXX1",  # Maternal care for other (suspected) fetal abnormality and damage, fetal cardiac anomalies, fetus 1
        "O35BXX2",  # Maternal care for other (suspected) fetal abnormality and damage, fetal cardiac anomalies, fetus 2
        "O35BXX3",  # Maternal care for other (suspected) fetal abnormality and damage, fetal cardiac anomalies, fetus 3
        "O35BXX4",  # Maternal care for other (suspected) fetal abnormality and damage, fetal cardiac anomalies, fetus 4
        "O35BXX5",  # Maternal care for other (suspected) fetal abnormality and damage, fetal cardiac anomalies, fetus 5
        "O35BXX9",  # Maternal care for other (suspected) fetal abnormality and damage, fetal cardiac anomalies, other fetus
        "O35CXX0",  # Maternal care for other (suspected) fetal abnormality and damage, fetal pulmonary anomalies, not applicable or unspecified
        "O35CXX1",  # Maternal care for other (suspected) fetal abnormality and damage, fetal pulmonary anomalies, fetus 1
        "O35CXX2",  # Maternal care for other (suspected) fetal abnormality and damage, fetal pulmonary anomalies, fetus 2
        "O35CXX3",  # Maternal care for other (suspected) fetal abnormality and damage, fetal pulmonary anomalies, fetus 3
        "O35CXX4",  # Maternal care for other (suspected) fetal abnormality and damage, fetal pulmonary anomalies, fetus 4
        "O35CXX5",  # Maternal care for other (suspected) fetal abnormality and damage, fetal pulmonary anomalies, fetus 5
        "O35CXX9",  # Maternal care for other (suspected) fetal abnormality and damage, fetal pulmonary anomalies, other fetus
        "O35DXX0",  # Maternal care for other (suspected) fetal abnormality and damage, fetal gastrointestinal anomalies, not applicable or unspecified
        "O35DXX1",  # Maternal care for other (suspected) fetal abnormality and damage, fetal gastrointestinal anomalies, fetus 1
        "O35DXX2",  # Maternal care for other (suspected) fetal abnormality and damage, fetal gastrointestinal anomalies, fetus 2
        "O35DXX3",  # Maternal care for other (suspected) fetal abnormality and damage, fetal gastrointestinal anomalies, fetus 3
        "O35DXX4",  # Maternal care for other (suspected) fetal abnormality and damage, fetal gastrointestinal anomalies, fetus 4
        "O35DXX5",  # Maternal care for other (suspected) fetal abnormality and damage, fetal gastrointestinal anomalies, fetus 5
        "O35DXX9",  # Maternal care for other (suspected) fetal abnormality and damage, fetal gastrointestinal anomalies, other fetus
        "O35EXX0",  # Maternal care for other (suspected) fetal abnormality and damage, fetal genitourinary anomalies, not applicable or unspecified
        "O35EXX1",  # Maternal care for other (suspected) fetal abnormality and damage, fetal genitourinary anomalies, fetus 1
        "O35EXX2",  # Maternal care for other (suspected) fetal abnormality and damage, fetal genitourinary anomalies, fetus 2
        "O35EXX3",  # Maternal care for other (suspected) fetal abnormality and damage, fetal genitourinary anomalies, fetus 3
        "O35EXX4",  # Maternal care for other (suspected) fetal abnormality and damage, fetal genitourinary anomalies, fetus 4
        "O35EXX5",  # Maternal care for other (suspected) fetal abnormality and damage, fetal genitourinary anomalies, fetus 5
        "O35EXX9",  # Maternal care for other (suspected) fetal abnormality and damage, fetal genitourinary anomalies, other fetus
        "O35FXX0",  # Maternal care for other (suspected) fetal abnormality and damage, fetal musculoskeletal anomalies of trunk, not applicable or unspecified
        "O35FXX1",  # Maternal care for other (suspected) fetal abnormality and damage, fetal musculoskeletal anomalies of trunk, fetus 1
        "O35FXX2",  # Maternal care for other (suspected) fetal abnormality and damage, fetal musculoskeletal anomalies of trunk, fetus 2
        "O35FXX3",  # Maternal care for other (suspected) fetal abnormality and damage, fetal musculoskeletal anomalies of trunk, fetus 3
        "O35FXX4",  # Maternal care for other (suspected) fetal abnormality and damage, fetal musculoskeletal anomalies of trunk, fetus 4
        "O35FXX5",  # Maternal care for other (suspected) fetal abnormality and damage, fetal musculoskeletal anomalies of trunk, fetus 5
        "O35FXX9",  # Maternal care for other (suspected) fetal abnormality and damage, fetal musculoskeletal anomalies of trunk, other fetus
        "O35GXX0",  # Maternal care for other (suspected) fetal abnormality and damage, fetal upper extremities anomalies, not applicable or unspecified
        "O35GXX1",  # Maternal care for other (suspected) fetal abnormality and damage, fetal upper extremities anomalies, fetus 1
        "O35GXX2",  # Maternal care for other (suspected) fetal abnormality and damage, fetal upper extremities anomalies, fetus 2
        "O35GXX3",  # Maternal care for other (suspected) fetal abnormality and damage, fetal upper extremities anomalies, fetus 3
        "O35GXX4",  # Maternal care for other (suspected) fetal abnormality and damage, fetal upper extremities anomalies, fetus 4
        "O35GXX5",  # Maternal care for other (suspected) fetal abnormality and damage, fetal upper extremities anomalies, fetus 5
        "O35GXX9",  # Maternal care for other (suspected) fetal abnormality and damage, fetal upper extremities anomalies, other fetus
        "O35HXX0",  # Maternal care for other (suspected) fetal abnormality and damage, fetal lower extremities anomalies, not applicable or unspecified
        "O35HXX1",  # Maternal care for other (suspected) fetal abnormality and damage, fetal lower extremities anomalies, fetus 1
        "O35HXX2",  # Maternal care for other (suspected) fetal abnormality and damage, fetal lower extremities anomalies, fetus 2
        "O35HXX3",  # Maternal care for other (suspected) fetal abnormality and damage, fetal lower extremities anomalies, fetus 3
        "O35HXX4",  # Maternal care for other (suspected) fetal abnormality and damage, fetal lower extremities anomalies, fetus 4
        "O35HXX5",  # Maternal care for other (suspected) fetal abnormality and damage, fetal lower extremities anomalies, fetus 5
        "O35HXX9",  # Maternal care for other (suspected) fetal abnormality and damage, fetal lower extremities anomalies, other fetus
        "O360110",  # Maternal care for anti-D [Rh] antibodies, first trimester, not applicable or unspecified
        "O360111",  # Maternal care for anti-D [Rh] antibodies, first trimester, fetus 1
        "O360112",  # Maternal care for anti-D [Rh] antibodies, first trimester, fetus 2
        "O360113",  # Maternal care for anti-D [Rh] antibodies, first trimester, fetus 3
        "O360114",  # Maternal care for anti-D [Rh] antibodies, first trimester, fetus 4
        "O360115",  # Maternal care for anti-D [Rh] antibodies, first trimester, fetus 5
        "O360119",  # Maternal care for anti-D [Rh] antibodies, first trimester, other fetus
        "O360120",  # Maternal care for anti-D [Rh] antibodies, second trimester, not applicable or unspecified
        "O360121",  # Maternal care for anti-D [Rh] antibodies, second trimester, fetus 1
        "O360122",  # Maternal care for anti-D [Rh] antibodies, second trimester, fetus 2
        "O360123",  # Maternal care for anti-D [Rh] antibodies, second trimester, fetus 3
        "O360124",  # Maternal care for anti-D [Rh] antibodies, second trimester, fetus 4
        "O360125",  # Maternal care for anti-D [Rh] antibodies, second trimester, fetus 5
        "O360129",  # Maternal care for anti-D [Rh] antibodies, second trimester, other fetus
        "O360130",  # Maternal care for anti-D [Rh] antibodies, third trimester, not applicable or unspecified
        "O360131",  # Maternal care for anti-D [Rh] antibodies, third trimester, fetus 1
        "O360132",  # Maternal care for anti-D [Rh] antibodies, third trimester, fetus 2
        "O360133",  # Maternal care for anti-D [Rh] antibodies, third trimester, fetus 3
        "O360134",  # Maternal care for anti-D [Rh] antibodies, third trimester, fetus 4
        "O360135",  # Maternal care for anti-D [Rh] antibodies, third trimester, fetus 5
        "O360139",  # Maternal care for anti-D [Rh] antibodies, third trimester, other fetus
        "O360190",  # Maternal care for anti-D [Rh] antibodies, unspecified trimester, not applicable or unspecified
        "O360191",  # Maternal care for anti-D [Rh] antibodies, unspecified trimester, fetus 1
        "O360192",  # Maternal care for anti-D [Rh] antibodies, unspecified trimester, fetus 2
        "O360193",  # Maternal care for anti-D [Rh] antibodies, unspecified trimester, fetus 3
        "O360194",  # Maternal care for anti-D [Rh] antibodies, unspecified trimester, fetus 4
        "O360195",  # Maternal care for anti-D [Rh] antibodies, unspecified trimester, fetus 5
        "O360199",  # Maternal care for anti-D [Rh] antibodies, unspecified trimester, other fetus
        "O360910",  # Maternal care for other rhesus isoimmunization, first trimester, not applicable or unspecified
        "O360911",  # Maternal care for other rhesus isoimmunization, first trimester, fetus 1
        "O360912",  # Maternal care for other rhesus isoimmunization, first trimester, fetus 2
        "O360913",  # Maternal care for other rhesus isoimmunization, first trimester, fetus 3
        "O360914",  # Maternal care for other rhesus isoimmunization, first trimester, fetus 4
        "O360915",  # Maternal care for other rhesus isoimmunization, first trimester, fetus 5
        "O360919",  # Maternal care for other rhesus isoimmunization, first trimester, other fetus
        "O360920",  # Maternal care for other rhesus isoimmunization, second trimester, not applicable or unspecified
        "O360921",  # Maternal care for other rhesus isoimmunization, second trimester, fetus 1
        "O360922",  # Maternal care for other rhesus isoimmunization, second trimester, fetus 2
        "O360923",  # Maternal care for other rhesus isoimmunization, second trimester, fetus 3
        "O360924",  # Maternal care for other rhesus isoimmunization, second trimester, fetus 4
        "O360925",  # Maternal care for other rhesus isoimmunization, second trimester, fetus 5
        "O360929",  # Maternal care for other rhesus isoimmunization, second trimester, other fetus
        "O360930",  # Maternal care for other rhesus isoimmunization, third trimester, not applicable or unspecified
        "O360931",  # Maternal care for other rhesus isoimmunization, third trimester, fetus 1
        "O360932",  # Maternal care for other rhesus isoimmunization, third trimester, fetus 2
        "O360933",  # Maternal care for other rhesus isoimmunization, third trimester, fetus 3
        "O360934",  # Maternal care for other rhesus isoimmunization, third trimester, fetus 4
        "O360935",  # Maternal care for other rhesus isoimmunization, third trimester, fetus 5
        "O360939",  # Maternal care for other rhesus isoimmunization, third trimester, other fetus
        "O360990",  # Maternal care for other rhesus isoimmunization, unspecified trimester, not applicable or unspecified
        "O360991",  # Maternal care for other rhesus isoimmunization, unspecified trimester, fetus 1
        "O360992",  # Maternal care for other rhesus isoimmunization, unspecified trimester, fetus 2
        "O360993",  # Maternal care for other rhesus isoimmunization, unspecified trimester, fetus 3
        "O360994",  # Maternal care for other rhesus isoimmunization, unspecified trimester, fetus 4
        "O360995",  # Maternal care for other rhesus isoimmunization, unspecified trimester, fetus 5
        "O360999",  # Maternal care for other rhesus isoimmunization, unspecified trimester, other fetus
        "O361110",  # Maternal care for Anti-A sensitization, first trimester, not applicable or unspecified
        "O361111",  # Maternal care for Anti-A sensitization, first trimester, fetus 1
        "O361112",  # Maternal care for Anti-A sensitization, first trimester, fetus 2
        "O361113",  # Maternal care for Anti-A sensitization, first trimester, fetus 3
        "O361114",  # Maternal care for Anti-A sensitization, first trimester, fetus 4
        "O361115",  # Maternal care for Anti-A sensitization, first trimester, fetus 5
        "O361119",  # Maternal care for Anti-A sensitization, first trimester, other fetus
        "O361120",  # Maternal care for Anti-A sensitization, second trimester, not applicable or unspecified
        "O361121",  # Maternal care for Anti-A sensitization, second trimester, fetus 1
        "O361122",  # Maternal care for Anti-A sensitization, second trimester, fetus 2
        "O361123",  # Maternal care for Anti-A sensitization, second trimester, fetus 3
        "O361124",  # Maternal care for Anti-A sensitization, second trimester, fetus 4
        "O361125",  # Maternal care for Anti-A sensitization, second trimester, fetus 5
        "O361129",  # Maternal care for Anti-A sensitization, second trimester, other fetus
        "O361130",  # Maternal care for Anti-A sensitization, third trimester, not applicable or unspecified
        "O361131",  # Maternal care for Anti-A sensitization, third trimester, fetus 1
        "O361132",  # Maternal care for Anti-A sensitization, third trimester, fetus 2
        "O361133",  # Maternal care for Anti-A sensitization, third trimester, fetus 3
        "O361134",  # Maternal care for Anti-A sensitization, third trimester, fetus 4
        "O361135",  # Maternal care for Anti-A sensitization, third trimester, fetus 5
        "O361139",  # Maternal care for Anti-A sensitization, third trimester, other fetus
        "O361190",  # Maternal care for Anti-A sensitization, unspecified trimester, not applicable or unspecified
        "O361191",  # Maternal care for Anti-A sensitization, unspecified trimester, fetus 1
        "O361192",  # Maternal care for Anti-A sensitization, unspecified trimester, fetus 2
        "O361193",  # Maternal care for Anti-A sensitization, unspecified trimester, fetus 3
        "O361194",  # Maternal care for Anti-A sensitization, unspecified trimester, fetus 4
        "O361195",  # Maternal care for Anti-A sensitization, unspecified trimester, fetus 5
        "O361199",  # Maternal care for Anti-A sensitization, unspecified trimester, other fetus
        "O361910",  # Maternal care for other isoimmunization, first trimester, not applicable or unspecified
        "O361911",  # Maternal care for other isoimmunization, first trimester, fetus 1
        "O361912",  # Maternal care for other isoimmunization, first trimester, fetus 2
        "O361913",  # Maternal care for other isoimmunization, first trimester, fetus 3
        "O361914",  # Maternal care for other isoimmunization, first trimester, fetus 4
        "O361915",  # Maternal care for other isoimmunization, first trimester, fetus 5
        "O361919",  # Maternal care for other isoimmunization, first trimester, other fetus
        "O361920",  # Maternal care for other isoimmunization, second trimester, not applicable or unspecified
        "O361921",  # Maternal care for other isoimmunization, second trimester, fetus 1
        "O361922",  # Maternal care for other isoimmunization, second trimester, fetus 2
        "O361923",  # Maternal care for other isoimmunization, second trimester, fetus 3
        "O361924",  # Maternal care for other isoimmunization, second trimester, fetus 4
        "O361925",  # Maternal care for other isoimmunization, second trimester, fetus 5
        "O361929",  # Maternal care for other isoimmunization, second trimester, other fetus
        "O361930",  # Maternal care for other isoimmunization, third trimester, not applicable or unspecified
        "O361931",  # Maternal care for other isoimmunization, third trimester, fetus 1
        "O361932",  # Maternal care for other isoimmunization, third trimester, fetus 2
        "O361933",  # Maternal care for other isoimmunization, third trimester, fetus 3
        "O361934",  # Maternal care for other isoimmunization, third trimester, fetus 4
        "O361935",  # Maternal care for other isoimmunization, third trimester, fetus 5
        "O361939",  # Maternal care for other isoimmunization, third trimester, other fetus
        "O361990",  # Maternal care for other isoimmunization, unspecified trimester, not applicable or unspecified
        "O361991",  # Maternal care for other isoimmunization, unspecified trimester, fetus 1
        "O361992",  # Maternal care for other isoimmunization, unspecified trimester, fetus 2
        "O361993",  # Maternal care for other isoimmunization, unspecified trimester, fetus 3
        "O361994",  # Maternal care for other isoimmunization, unspecified trimester, fetus 4
        "O361995",  # Maternal care for other isoimmunization, unspecified trimester, fetus 5
        "O361999",  # Maternal care for other isoimmunization, unspecified trimester, other fetus
        "O3620X0",  # Maternal care for hydrops fetalis, unspecified trimester, not applicable or unspecified
        "O3620X1",  # Maternal care for hydrops fetalis, unspecified trimester, fetus 1
        "O3620X2",  # Maternal care for hydrops fetalis, unspecified trimester, fetus 2
        "O3620X3",  # Maternal care for hydrops fetalis, unspecified trimester, fetus 3
        "O3620X4",  # Maternal care for hydrops fetalis, unspecified trimester, fetus 4
        "O3620X5",  # Maternal care for hydrops fetalis, unspecified trimester, fetus 5
        "O3620X9",  # Maternal care for hydrops fetalis, unspecified trimester, other fetus
        "O3621X0",  # Maternal care for hydrops fetalis, first trimester, not applicable or unspecified
        "O3621X1",  # Maternal care for hydrops fetalis, first trimester, fetus 1
        "O3621X2",  # Maternal care for hydrops fetalis, first trimester, fetus 2
        "O3621X3",  # Maternal care for hydrops fetalis, first trimester, fetus 3
        "O3621X4",  # Maternal care for hydrops fetalis, first trimester, fetus 4
        "O3621X5",  # Maternal care for hydrops fetalis, first trimester, fetus 5
        "O3621X9",  # Maternal care for hydrops fetalis, first trimester, other fetus
        "O3622X0",  # Maternal care for hydrops fetalis, second trimester, not applicable or unspecified
        "O3622X1",  # Maternal care for hydrops fetalis, second trimester, fetus 1
        "O3622X2",  # Maternal care for hydrops fetalis, second trimester, fetus 2
        "O3622X3",  # Maternal care for hydrops fetalis, second trimester, fetus 3
        "O3622X4",  # Maternal care for hydrops fetalis, second trimester, fetus 4
        "O3622X5",  # Maternal care for hydrops fetalis, second trimester, fetus 5
        "O3622X9",  # Maternal care for hydrops fetalis, second trimester, other fetus
        "O3623X0",  # Maternal care for hydrops fetalis, third trimester, not applicable or unspecified
        "O3623X1",  # Maternal care for hydrops fetalis, third trimester, fetus 1
        "O3623X2",  # Maternal care for hydrops fetalis, third trimester, fetus 2
        "O3623X3",  # Maternal care for hydrops fetalis, third trimester, fetus 3
        "O3623X4",  # Maternal care for hydrops fetalis, third trimester, fetus 4
        "O3623X5",  # Maternal care for hydrops fetalis, third trimester, fetus 5
        "O3623X9",  # Maternal care for hydrops fetalis, third trimester, other fetus
        "O364XX0",  # Maternal care for intrauterine death, not applicable or unspecified
        "O364XX1",  # Maternal care for intrauterine death, fetus 1
        "O364XX2",  # Maternal care for intrauterine death, fetus 2
        "O364XX3",  # Maternal care for intrauterine death, fetus 3
        "O364XX4",  # Maternal care for intrauterine death, fetus 4
        "O364XX5",  # Maternal care for intrauterine death, fetus 5
        "O364XX9",  # Maternal care for intrauterine death, other fetus
        "O365110",  # Maternal care for known or suspected placental insufficiency, first trimester, not applicable or unspecified
        "O365111",  # Maternal care for known or suspected placental insufficiency, first trimester, fetus 1
        "O365112",  # Maternal care for known or suspected placental insufficiency, first trimester, fetus 2
        "O365113",  # Maternal care for known or suspected placental insufficiency, first trimester, fetus 3
        "O365114",  # Maternal care for known or suspected placental insufficiency, first trimester, fetus 4
        "O365115",  # Maternal care for known or suspected placental insufficiency, first trimester, fetus 5
        "O365119",  # Maternal care for known or suspected placental insufficiency, first trimester, other fetus
        "O365120",  # Maternal care for known or suspected placental insufficiency, second trimester, not applicable or unspecified
        "O365121",  # Maternal care for known or suspected placental insufficiency, second trimester, fetus 1
        "O365122",  # Maternal care for known or suspected placental insufficiency, second trimester, fetus 2
        "O365123",  # Maternal care for known or suspected placental insufficiency, second trimester, fetus 3
        "O365124",  # Maternal care for known or suspected placental insufficiency, second trimester, fetus 4
        "O365125",  # Maternal care for known or suspected placental insufficiency, second trimester, fetus 5
        "O365129",  # Maternal care for known or suspected placental insufficiency, second trimester, other fetus
        "O365130",  # Maternal care for known or suspected placental insufficiency, third trimester, not applicable or unspecified
        "O365131",  # Maternal care for known or suspected placental insufficiency, third trimester, fetus 1
        "O365132",  # Maternal care for known or suspected placental insufficiency, third trimester, fetus 2
        "O365133",  # Maternal care for known or suspected placental insufficiency, third trimester, fetus 3
        "O365134",  # Maternal care for known or suspected placental insufficiency, third trimester, fetus 4
        "O365135",  # Maternal care for known or suspected placental insufficiency, third trimester, fetus 5
        "O365139",  # Maternal care for known or suspected placental insufficiency, third trimester, other fetus
        "O365190",  # Maternal care for known or suspected placental insufficiency, unspecified trimester, not applicable or unspecified
        "O365191",  # Maternal care for known or suspected placental insufficiency, unspecified trimester, fetus 1
        "O365192",  # Maternal care for known or suspected placental insufficiency, unspecified trimester, fetus 2
        "O365193",  # Maternal care for known or suspected placental insufficiency, unspecified trimester, fetus 3
        "O365194",  # Maternal care for known or suspected placental insufficiency, unspecified trimester, fetus 4
        "O365195",  # Maternal care for known or suspected placental insufficiency, unspecified trimester, fetus 5
        "O365199",  # Maternal care for known or suspected placental insufficiency, unspecified trimester, other fetus
        "O365910",  # Maternal care for other known or suspected poor fetal growth, first trimester, not applicable or unspecified
        "O365911",  # Maternal care for other known or suspected poor fetal growth, first trimester, fetus 1
        "O365912",  # Maternal care for other known or suspected poor fetal growth, first trimester, fetus 2
        "O365913",  # Maternal care for other known or suspected poor fetal growth, first trimester, fetus 3
        "O365914",  # Maternal care for other known or suspected poor fetal growth, first trimester, fetus 4
        "O365915",  # Maternal care for other known or suspected poor fetal growth, first trimester, fetus 5
        "O365919",  # Maternal care for other known or suspected poor fetal growth, first trimester, other fetus
        "O365920",  # Maternal care for other known or suspected poor fetal growth, second trimester, not applicable or unspecified
        "O365921",  # Maternal care for other known or suspected poor fetal growth, second trimester, fetus 1
        "O365922",  # Maternal care for other known or suspected poor fetal growth, second trimester, fetus 2
        "O365923",  # Maternal care for other known or suspected poor fetal growth, second trimester, fetus 3
        "O365924",  # Maternal care for other known or suspected poor fetal growth, second trimester, fetus 4
        "O365925",  # Maternal care for other known or suspected poor fetal growth, second trimester, fetus 5
        "O365929",  # Maternal care for other known or suspected poor fetal growth, second trimester, other fetus
        "O365930",  # Maternal care for other known or suspected poor fetal growth, third trimester, not applicable or unspecified
        "O365931",  # Maternal care for other known or suspected poor fetal growth, third trimester, fetus 1
        "O365932",  # Maternal care for other known or suspected poor fetal growth, third trimester, fetus 2
        "O365933",  # Maternal care for other known or suspected poor fetal growth, third trimester, fetus 3
        "O365934",  # Maternal care for other known or suspected poor fetal growth, third trimester, fetus 4
        "O365935",  # Maternal care for other known or suspected poor fetal growth, third trimester, fetus 5
        "O365939",  # Maternal care for other known or suspected poor fetal growth, third trimester, other fetus
        "O365990",  # Maternal care for other known or suspected poor fetal growth, unspecified trimester, not applicable or unspecified
        "O365991",  # Maternal care for other known or suspected poor fetal growth, unspecified trimester, fetus 1
        "O365992",  # Maternal care for other known or suspected poor fetal growth, unspecified trimester, fetus 2
        "O365993",  # Maternal care for other known or suspected poor fetal growth, unspecified trimester, fetus 3
        "O365994",  # Maternal care for other known or suspected poor fetal growth, unspecified trimester, fetus 4
        "O365995",  # Maternal care for other known or suspected poor fetal growth, unspecified trimester, fetus 5
        "O365999",  # Maternal care for other known or suspected poor fetal growth, unspecified trimester, other fetus
        "O3660X0",  # Maternal care for excessive fetal growth, unspecified trimester, not applicable or unspecified
        "O3660X1",  # Maternal care for excessive fetal growth, unspecified trimester, fetus 1
        "O3660X2",  # Maternal care for excessive fetal growth, unspecified trimester, fetus 2
        "O3660X3",  # Maternal care for excessive fetal growth, unspecified trimester, fetus 3
        "O3660X4",  # Maternal care for excessive fetal growth, unspecified trimester, fetus 4
        "O3660X5",  # Maternal care for excessive fetal growth, unspecified trimester, fetus 5
        "O3660X9",  # Maternal care for excessive fetal growth, unspecified trimester, other fetus
        "O3661X0",  # Maternal care for excessive fetal growth, first trimester, not applicable or unspecified
        "O3661X1",  # Maternal care for excessive fetal growth, first trimester, fetus 1
        "O3661X2",  # Maternal care for excessive fetal growth, first trimester, fetus 2
        "O3661X3",  # Maternal care for excessive fetal growth, first trimester, fetus 3
        "O3661X4",  # Maternal care for excessive fetal growth, first trimester, fetus 4
        "O3661X5",  # Maternal care for excessive fetal growth, first trimester, fetus 5
        "O3661X9",  # Maternal care for excessive fetal growth, first trimester, other fetus
        "O3662X0",  # Maternal care for excessive fetal growth, second trimester, not applicable or unspecified
        "O3662X1",  # Maternal care for excessive fetal growth, second trimester, fetus 1
        "O3662X2",  # Maternal care for excessive fetal growth, second trimester, fetus 2
        "O3662X3",  # Maternal care for excessive fetal growth, second trimester, fetus 3
        "O3662X4",  # Maternal care for excessive fetal growth, second trimester, fetus 4
        "O3662X5",  # Maternal care for excessive fetal growth, second trimester, fetus 5
        "O3662X9",  # Maternal care for excessive fetal growth, second trimester, other fetus
        "O3663X0",  # Maternal care for excessive fetal growth, third trimester, not applicable or unspecified
        "O3663X1",  # Maternal care for excessive fetal growth, third trimester, fetus 1
        "O3663X2",  # Maternal care for excessive fetal growth, third trimester, fetus 2
        "O3663X3",  # Maternal care for excessive fetal growth, third trimester, fetus 3
        "O3663X4",  # Maternal care for excessive fetal growth, third trimester, fetus 4
        "O3663X5",  # Maternal care for excessive fetal growth, third trimester, fetus 5
        "O3663X9",  # Maternal care for excessive fetal growth, third trimester, other fetus
        "O3670X0",  # Maternal care for viable fetus in abdominal pregnancy, unspecified trimester, not applicable or unspecified
        "O3670X1",  # Maternal care for viable fetus in abdominal pregnancy, unspecified trimester, fetus 1
        "O3670X2",  # Maternal care for viable fetus in abdominal pregnancy, unspecified trimester, fetus 2
        "O3670X3",  # Maternal care for viable fetus in abdominal pregnancy, unspecified trimester, fetus 3
        "O3670X4",  # Maternal care for viable fetus in abdominal pregnancy, unspecified trimester, fetus 4
        "O3670X5",  # Maternal care for viable fetus in abdominal pregnancy, unspecified trimester, fetus 5
        "O3670X9",  # Maternal care for viable fetus in abdominal pregnancy, unspecified trimester, other fetus
        "O3671X0",  # Maternal care for viable fetus in abdominal pregnancy, first trimester, not applicable or unspecified
        "O3671X1",  # Maternal care for viable fetus in abdominal pregnancy, first trimester, fetus 1
        "O3671X2",  # Maternal care for viable fetus in abdominal pregnancy, first trimester, fetus 2
        "O3671X3",  # Maternal care for viable fetus in abdominal pregnancy, first trimester, fetus 3
        "O3671X4",  # Maternal care for viable fetus in abdominal pregnancy, first trimester, fetus 4
        "O3671X5",  # Maternal care for viable fetus in abdominal pregnancy, first trimester, fetus 5
        "O3671X9",  # Maternal care for viable fetus in abdominal pregnancy, first trimester, other fetus
        "O3672X0",  # Maternal care for viable fetus in abdominal pregnancy, second trimester, not applicable or unspecified
        "O3672X1",  # Maternal care for viable fetus in abdominal pregnancy, second trimester, fetus 1
        "O3672X2",  # Maternal care for viable fetus in abdominal pregnancy, second trimester, fetus 2
        "O3672X3",  # Maternal care for viable fetus in abdominal pregnancy, second trimester, fetus 3
        "O3672X4",  # Maternal care for viable fetus in abdominal pregnancy, second trimester, fetus 4
        "O3672X5",  # Maternal care for viable fetus in abdominal pregnancy, second trimester, fetus 5
        "O3672X9",  # Maternal care for viable fetus in abdominal pregnancy, second trimester, other fetus
        "O3673X0",  # Maternal care for viable fetus in abdominal pregnancy, third trimester, not applicable or unspecified
        "O3673X1",  # Maternal care for viable fetus in abdominal pregnancy, third trimester, fetus 1
        "O3673X2",  # Maternal care for viable fetus in abdominal pregnancy, third trimester, fetus 2
        "O3673X3",  # Maternal care for viable fetus in abdominal pregnancy, third trimester, fetus 3
        "O3673X4",  # Maternal care for viable fetus in abdominal pregnancy, third trimester, fetus 4
        "O3673X5",  # Maternal care for viable fetus in abdominal pregnancy, third trimester, fetus 5
        "O3673X9",  # Maternal care for viable fetus in abdominal pregnancy, third trimester, other fetus
        "O3680X0",  # Pregnancy with inconclusive fetal viability, not applicable or unspecified
        "O3680X1",  # Pregnancy with inconclusive fetal viability, fetus 1
        "O3680X2",  # Pregnancy with inconclusive fetal viability, fetus 2
        "O3680X3",  # Pregnancy with inconclusive fetal viability, fetus 3
        "O3680X4",  # Pregnancy with inconclusive fetal viability, fetus 4
        "O3680X5",  # Pregnancy with inconclusive fetal viability, fetus 5
        "O3680X9",  # Pregnancy with inconclusive fetal viability, other fetus
        "O368120",  # Decreased fetal movements, second trimester, not applicable or unspecified
        "O368121",  # Decreased fetal movements, second trimester, fetus 1
        "O368122",  # Decreased fetal movements, second trimester, fetus 2
        "O368123",  # Decreased fetal movements, second trimester, fetus 3
        "O368124",  # Decreased fetal movements, second trimester, fetus 4
        "O368125",  # Decreased fetal movements, second trimester, fetus 5
        "O368129",  # Decreased fetal movements, second trimester, other fetus
        "O368130",  # Decreased fetal movements, third trimester, not applicable or unspecified
        "O368131",  # Decreased fetal movements, third trimester, fetus 1
        "O368132",  # Decreased fetal movements, third trimester, fetus 2
        "O368133",  # Decreased fetal movements, third trimester, fetus 3
        "O368134",  # Decreased fetal movements, third trimester, fetus 4
        "O368135",  # Decreased fetal movements, third trimester, fetus 5
        "O368139",  # Decreased fetal movements, third trimester, other fetus
        "O368190",  # Decreased fetal movements, unspecified trimester, not applicable or unspecified
        "O368191",  # Decreased fetal movements, unspecified trimester, fetus 1
        "O368192",  # Decreased fetal movements, unspecified trimester, fetus 2
        "O368193",  # Decreased fetal movements, unspecified trimester, fetus 3
        "O368194",  # Decreased fetal movements, unspecified trimester, fetus 4
        "O368195",  # Decreased fetal movements, unspecified trimester, fetus 5
        "O368199",  # Decreased fetal movements, unspecified trimester, other fetus
        "O368210",  # Fetal anemia and thrombocytopenia, first trimester, not applicable or unspecified
        "O368211",  # Fetal anemia and thrombocytopenia, first trimester, fetus 1
        "O368212",  # Fetal anemia and thrombocytopenia, first trimester, fetus 2
        "O368213",  # Fetal anemia and thrombocytopenia, first trimester, fetus 3
        "O368214",  # Fetal anemia and thrombocytopenia, first trimester, fetus 4
        "O368215",  # Fetal anemia and thrombocytopenia, first trimester, fetus 5
        "O368219",  # Fetal anemia and thrombocytopenia, first trimester, other fetus
        "O368220",  # Fetal anemia and thrombocytopenia, second trimester, not applicable or unspecified
        "O368221",  # Fetal anemia and thrombocytopenia, second trimester, fetus 1
        "O368222",  # Fetal anemia and thrombocytopenia, second trimester, fetus 2
        "O368223",  # Fetal anemia and thrombocytopenia, second trimester, fetus 3
        "O368224",  # Fetal anemia and thrombocytopenia, second trimester, fetus 4
        "O368225",  # Fetal anemia and thrombocytopenia, second trimester, fetus 5
        "O368229",  # Fetal anemia and thrombocytopenia, second trimester, other fetus
        "O368230",  # Fetal anemia and thrombocytopenia, third trimester, not applicable or unspecified
        "O368231",  # Fetal anemia and thrombocytopenia, third trimester, fetus 1
        "O368232",  # Fetal anemia and thrombocytopenia, third trimester, fetus 2
        "O368233",  # Fetal anemia and thrombocytopenia, third trimester, fetus 3
        "O368234",  # Fetal anemia and thrombocytopenia, third trimester, fetus 4
        "O368235",  # Fetal anemia and thrombocytopenia, third trimester, fetus 5
        "O368239",  # Fetal anemia and thrombocytopenia, third trimester, other fetus
        "O368290",  # Fetal anemia and thrombocytopenia, unspecified trimester, not applicable or unspecified
        "O368291",  # Fetal anemia and thrombocytopenia, unspecified trimester, fetus 1
        "O368292",  # Fetal anemia and thrombocytopenia, unspecified trimester, fetus 2
        "O368293",  # Fetal anemia and thrombocytopenia, unspecified trimester, fetus 3
        "O368294",  # Fetal anemia and thrombocytopenia, unspecified trimester, fetus 4
        "O368295",  # Fetal anemia and thrombocytopenia, unspecified trimester, fetus 5
        "O368299",  # Fetal anemia and thrombocytopenia, unspecified trimester, other fetus
        "O368310",  # Maternal care for abnormalities of the fetal heart rate or rhythm, first trimester, not applicable or unspecified
        "O368311",  # Maternal care for abnormalities of the fetal heart rate or rhythm, first trimester, fetus 1
        "O368312",  # Maternal care for abnormalities of the fetal heart rate or rhythm, first trimester, fetus 2
        "O368313",  # Maternal care for abnormalities of the fetal heart rate or rhythm, first trimester, fetus 3
        "O368314",  # Maternal care for abnormalities of the fetal heart rate or rhythm, first trimester, fetus 4
        "O368315",  # Maternal care for abnormalities of the fetal heart rate or rhythm, first trimester, fetus 5
        "O368319",  # Maternal care for abnormalities of the fetal heart rate or rhythm, first trimester, other fetus
        "O368320",  # Maternal care for abnormalities of the fetal heart rate or rhythm, second trimester, not applicable or unspecified
        "O368321",  # Maternal care for abnormalities of the fetal heart rate or rhythm, second trimester, fetus 1
        "O368322",  # Maternal care for abnormalities of the fetal heart rate or rhythm, second trimester, fetus 2
        "O368323",  # Maternal care for abnormalities of the fetal heart rate or rhythm, second trimester, fetus 3
        "O368324",  # Maternal care for abnormalities of the fetal heart rate or rhythm, second trimester, fetus 4
        "O368325",  # Maternal care for abnormalities of the fetal heart rate or rhythm, second trimester, fetus 5
        "O368329",  # Maternal care for abnormalities of the fetal heart rate or rhythm, second trimester, other fetus
        "O368330",  # Maternal care for abnormalities of the fetal heart rate or rhythm, third trimester, not applicable or unspecified
        "O368331",  # Maternal care for abnormalities of the fetal heart rate or rhythm, third trimester, fetus 1
        "O368332",  # Maternal care for abnormalities of the fetal heart rate or rhythm, third trimester, fetus 2
        "O368333",  # Maternal care for abnormalities of the fetal heart rate or rhythm, third trimester, fetus 3
        "O368334",  # Maternal care for abnormalities of the fetal heart rate or rhythm, third trimester, fetus 4
        "O368335",  # Maternal care for abnormalities of the fetal heart rate or rhythm, third trimester, fetus 5
        "O368339",  # Maternal care for abnormalities of the fetal heart rate or rhythm, third trimester, other fetus
        "O368390",  # Maternal care for abnormalities of the fetal heart rate or rhythm, unspecified trimester, not applicable or unspecified
        "O368391",  # Maternal care for abnormalities of the fetal heart rate or rhythm, unspecified trimester, fetus 1
        "O368392",  # Maternal care for abnormalities of the fetal heart rate or rhythm, unspecified trimester, fetus 2
        "O368393",  # Maternal care for abnormalities of the fetal heart rate or rhythm, unspecified trimester, fetus 3
        "O368394",  # Maternal care for abnormalities of the fetal heart rate or rhythm, unspecified trimester, fetus 4
        "O368395",  # Maternal care for abnormalities of the fetal heart rate or rhythm, unspecified trimester, fetus 5
        "O368399",  # Maternal care for abnormalities of the fetal heart rate or rhythm, unspecified trimester, other fetus
        "O368910",  # Maternal care for other specified fetal problems, first trimester, not applicable or unspecified
        "O368911",  # Maternal care for other specified fetal problems, first trimester, fetus 1
        "O368912",  # Maternal care for other specified fetal problems, first trimester, fetus 2
        "O368913",  # Maternal care for other specified fetal problems, first trimester, fetus 3
        "O368914",  # Maternal care for other specified fetal problems, first trimester, fetus 4
        "O368915",  # Maternal care for other specified fetal problems, first trimester, fetus 5
        "O368919",  # Maternal care for other specified fetal problems, first trimester, other fetus
        "O368920",  # Maternal care for other specified fetal problems, second trimester, not applicable or unspecified
        "O368921",  # Maternal care for other specified fetal problems, second trimester, fetus 1
        "O368922",  # Maternal care for other specified fetal problems, second trimester, fetus 2
        "O368923",  # Maternal care for other specified fetal problems, second trimester, fetus 3
        "O368924",  # Maternal care for other specified fetal problems, second trimester, fetus 4
        "O368925",  # Maternal care for other specified fetal problems, second trimester, fetus 5
        "O368929",  # Maternal care for other specified fetal problems, second trimester, other fetus
        "O368930",  # Maternal care for other specified fetal problems, third trimester, not applicable or unspecified
        "O368931",  # Maternal care for other specified fetal problems, third trimester, fetus 1
        "O368932",  # Maternal care for other specified fetal problems, third trimester, fetus 2
        "O368933",  # Maternal care for other specified fetal problems, third trimester, fetus 3
        "O368934",  # Maternal care for other specified fetal problems, third trimester, fetus 4
        "O368935",  # Maternal care for other specified fetal problems, third trimester, fetus 5
        "O368939",  # Maternal care for other specified fetal problems, third trimester, other fetus
        "O368990",  # Maternal care for other specified fetal problems, unspecified trimester, not applicable or unspecified
        "O368991",  # Maternal care for other specified fetal problems, unspecified trimester, fetus 1
        "O368992",  # Maternal care for other specified fetal problems, unspecified trimester, fetus 2
        "O368993",  # Maternal care for other specified fetal problems, unspecified trimester, fetus 3
        "O368994",  # Maternal care for other specified fetal problems, unspecified trimester, fetus 4
        "O368995",  # Maternal care for other specified fetal problems, unspecified trimester, fetus 5
        "O368999",  # Maternal care for other specified fetal problems, unspecified trimester, other fetus
        "O3690X0",  # Maternal care for fetal problem, unspecified, unspecified trimester, not applicable or unspecified
        "O3690X1",  # Maternal care for fetal problem, unspecified, unspecified trimester, fetus 1
        "O3690X2",  # Maternal care for fetal problem, unspecified, unspecified trimester, fetus 2
        "O3690X3",  # Maternal care for fetal problem, unspecified, unspecified trimester, fetus 3
        "O3690X4",  # Maternal care for fetal problem, unspecified, unspecified trimester, fetus 4
        "O3690X5",  # Maternal care for fetal problem, unspecified, unspecified trimester, fetus 5
        "O3690X9",  # Maternal care for fetal problem, unspecified, unspecified trimester, other fetus
        "O3691X0",  # Maternal care for fetal problem, unspecified, first trimester, not applicable or unspecified
        "O3691X1",  # Maternal care for fetal problem, unspecified, first trimester, fetus 1
        "O3691X2",  # Maternal care for fetal problem, unspecified, first trimester, fetus 2
        "O3691X3",  # Maternal care for fetal problem, unspecified, first trimester, fetus 3
        "O3691X4",  # Maternal care for fetal problem, unspecified, first trimester, fetus 4
        "O3691X5",  # Maternal care for fetal problem, unspecified, first trimester, fetus 5
        "O3691X9",  # Maternal care for fetal problem, unspecified, first trimester, other fetus
        "O3692X0",  # Maternal care for fetal problem, unspecified, second trimester, not applicable or unspecified
        "O3692X1",  # Maternal care for fetal problem, unspecified, second trimester, fetus 1
        "O3692X2",  # Maternal care for fetal problem, unspecified, second trimester, fetus 2
        "O3692X3",  # Maternal care for fetal problem, unspecified, second trimester, fetus 3
        "O3692X4",  # Maternal care for fetal problem, unspecified, second trimester, fetus 4
        "O3692X5",  # Maternal care for fetal problem, unspecified, second trimester, fetus 5
        "O3692X9",  # Maternal care for fetal problem, unspecified, second trimester, other fetus
        "O3693X0",  # Maternal care for fetal problem, unspecified, third trimester, not applicable or unspecified
        "O3693X1",  # Maternal care for fetal problem, unspecified, third trimester, fetus 1
        "O3693X2",  # Maternal care for fetal problem, unspecified, third trimester, fetus 2
        "O3693X3",  # Maternal care for fetal problem, unspecified, third trimester, fetus 3
        "O3693X4",  # Maternal care for fetal problem, unspecified, third trimester, fetus 4
        "O3693X5",  # Maternal care for fetal problem, unspecified, third trimester, fetus 5
        "O3693X9",  # Maternal care for fetal problem, unspecified, third trimester, other fetus
        "O401XX0",  # Polyhydramnios, first trimester, not applicable or unspecified
        "O401XX1",  # Polyhydramnios, first trimester, fetus 1
        "O401XX2",  # Polyhydramnios, first trimester, fetus 2
        "O401XX3",  # Polyhydramnios, first trimester, fetus 3
        "O401XX4",  # Polyhydramnios, first trimester, fetus 4
        "O401XX5",  # Polyhydramnios, first trimester, fetus 5
        "O401XX9",  # Polyhydramnios, first trimester, other fetus
        "O402XX0",  # Polyhydramnios, second trimester, not applicable or unspecified
        "O402XX1",  # Polyhydramnios, second trimester, fetus 1
        "O402XX2",  # Polyhydramnios, second trimester, fetus 2
        "O402XX3",  # Polyhydramnios, second trimester, fetus 3
        "O402XX4",  # Polyhydramnios, second trimester, fetus 4
        "O402XX5",  # Polyhydramnios, second trimester, fetus 5
        "O402XX9",  # Polyhydramnios, second trimester, other fetus
        "O403XX0",  # Polyhydramnios, third trimester, not applicable or unspecified
        "O403XX1",  # Polyhydramnios, third trimester, fetus 1
        "O403XX2",  # Polyhydramnios, third trimester, fetus 2
        "O403XX3",  # Polyhydramnios, third trimester, fetus 3
        "O403XX4",  # Polyhydramnios, third trimester, fetus 4
        "O403XX5",  # Polyhydramnios, third trimester, fetus 5
        "O403XX9",  # Polyhydramnios, third trimester, other fetus
        "O409XX0",  # Polyhydramnios, unspecified trimester, not applicable or unspecified
        "O409XX1",  # Polyhydramnios, unspecified trimester, fetus 1
        "O409XX2",  # Polyhydramnios, unspecified trimester, fetus 2
        "O409XX3",  # Polyhydramnios, unspecified trimester, fetus 3
        "O409XX4",  # Polyhydramnios, unspecified trimester, fetus 4
        "O409XX5",  # Polyhydramnios, unspecified trimester, fetus 5
        "O409XX9",  # Polyhydramnios, unspecified trimester, other fetus
        "O4100X0",  # Oligohydramnios, unspecified trimester, not applicable or unspecified
        "O4100X1",  # Oligohydramnios, unspecified trimester, fetus 1
        "O4100X2",  # Oligohydramnios, unspecified trimester, fetus 2
        "O4100X3",  # Oligohydramnios, unspecified trimester, fetus 3
        "O4100X4",  # Oligohydramnios, unspecified trimester, fetus 4
        "O4100X5",  # Oligohydramnios, unspecified trimester, fetus 5
        "O4100X9",  # Oligohydramnios, unspecified trimester, other fetus
        "O4101X0",  # Oligohydramnios, first trimester, not applicable or unspecified
        "O4101X1",  # Oligohydramnios, first trimester, fetus 1
        "O4101X2",  # Oligohydramnios, first trimester, fetus 2
        "O4101X3",  # Oligohydramnios, first trimester, fetus 3
        "O4101X4",  # Oligohydramnios, first trimester, fetus 4
        "O4101X5",  # Oligohydramnios, first trimester, fetus 5
        "O4101X9",  # Oligohydramnios, first trimester, other fetus
        "O4102X0",  # Oligohydramnios, second trimester, not applicable or unspecified
        "O4102X1",  # Oligohydramnios, second trimester, fetus 1
        "O4102X2",  # Oligohydramnios, second trimester, fetus 2
        "O4102X3",  # Oligohydramnios, second trimester, fetus 3
        "O4102X4",  # Oligohydramnios, second trimester, fetus 4
        "O4102X5",  # Oligohydramnios, second trimester, fetus 5
        "O4102X9",  # Oligohydramnios, second trimester, other fetus
        "O4103X0",  # Oligohydramnios, third trimester, not applicable or unspecified
        "O4103X1",  # Oligohydramnios, third trimester, fetus 1
        "O4103X2",  # Oligohydramnios, third trimester, fetus 2
        "O4103X3",  # Oligohydramnios, third trimester, fetus 3
        "O4103X4",  # Oligohydramnios, third trimester, fetus 4
        "O4103X5",  # Oligohydramnios, third trimester, fetus 5
        "O4103X9",  # Oligohydramnios, third trimester, other fetus
        "O411010",  # Infection of amniotic sac and membranes, unspecified, first trimester, not applicable or unspecified
        "O411011",  # Infection of amniotic sac and membranes, unspecified, first trimester, fetus 1
        "O411012",  # Infection of amniotic sac and membranes, unspecified, first trimester, fetus 2
        "O411013",  # Infection of amniotic sac and membranes, unspecified, first trimester, fetus 3
        "O411014",  # Infection of amniotic sac and membranes, unspecified, first trimester, fetus 4
        "O411015",  # Infection of amniotic sac and membranes, unspecified, first trimester, fetus 5
        "O411019",  # Infection of amniotic sac and membranes, unspecified, first trimester, other fetus
        "O411020",  # Infection of amniotic sac and membranes, unspecified, second trimester, not applicable or unspecified
        "O411021",  # Infection of amniotic sac and membranes, unspecified, second trimester, fetus 1
        "O411022",  # Infection of amniotic sac and membranes, unspecified, second trimester, fetus 2
        "O411023",  # Infection of amniotic sac and membranes, unspecified, second trimester, fetus 3
        "O411024",  # Infection of amniotic sac and membranes, unspecified, second trimester, fetus 4
        "O411025",  # Infection of amniotic sac and membranes, unspecified, second trimester, fetus 5
        "O411029",  # Infection of amniotic sac and membranes, unspecified, second trimester, other fetus
        "O411030",  # Infection of amniotic sac and membranes, unspecified, third trimester, not applicable or unspecified
        "O411031",  # Infection of amniotic sac and membranes, unspecified, third trimester, fetus 1
        "O411032",  # Infection of amniotic sac and membranes, unspecified, third trimester, fetus 2
        "O411033",  # Infection of amniotic sac and membranes, unspecified, third trimester, fetus 3
        "O411034",  # Infection of amniotic sac and membranes, unspecified, third trimester, fetus 4
        "O411035",  # Infection of amniotic sac and membranes, unspecified, third trimester, fetus 5
        "O411039",  # Infection of amniotic sac and membranes, unspecified, third trimester, other fetus
        "O411090",  # Infection of amniotic sac and membranes, unspecified, unspecified trimester, not applicable or unspecified
        "O411091",  # Infection of amniotic sac and membranes, unspecified, unspecified trimester, fetus 1
        "O411092",  # Infection of amniotic sac and membranes, unspecified, unspecified trimester, fetus 2
        "O411093",  # Infection of amniotic sac and membranes, unspecified, unspecified trimester, fetus 3
        "O411094",  # Infection of amniotic sac and membranes, unspecified, unspecified trimester, fetus 4
        "O411095",  # Infection of amniotic sac and membranes, unspecified, unspecified trimester, fetus 5
        "O411099",  # Infection of amniotic sac and membranes, unspecified, unspecified trimester, other fetus
        "O411210",  # Chorioamnionitis, first trimester, not applicable or unspecified
        "O411211",  # Chorioamnionitis, first trimester, fetus 1
        "O411212",  # Chorioamnionitis, first trimester, fetus 2
        "O411213",  # Chorioamnionitis, first trimester, fetus 3
        "O411214",  # Chorioamnionitis, first trimester, fetus 4
        "O411215",  # Chorioamnionitis, first trimester, fetus 5
        "O411219",  # Chorioamnionitis, first trimester, other fetus
        "O411220",  # Chorioamnionitis, second trimester, not applicable or unspecified
        "O411221",  # Chorioamnionitis, second trimester, fetus 1
        "O411222",  # Chorioamnionitis, second trimester, fetus 2
        "O411223",  # Chorioamnionitis, second trimester, fetus 3
        "O411224",  # Chorioamnionitis, second trimester, fetus 4
        "O411225",  # Chorioamnionitis, second trimester, fetus 5
        "O411229",  # Chorioamnionitis, second trimester, other fetus
        "O411230",  # Chorioamnionitis, third trimester, not applicable or unspecified
        "O411231",  # Chorioamnionitis, third trimester, fetus 1
        "O411232",  # Chorioamnionitis, third trimester, fetus 2
        "O411233",  # Chorioamnionitis, third trimester, fetus 3
        "O411234",  # Chorioamnionitis, third trimester, fetus 4
        "O411235",  # Chorioamnionitis, third trimester, fetus 5
        "O411239",  # Chorioamnionitis, third trimester, other fetus
        "O411290",  # Chorioamnionitis, unspecified trimester, not applicable or unspecified
        "O411291",  # Chorioamnionitis, unspecified trimester, fetus 1
        "O411292",  # Chorioamnionitis, unspecified trimester, fetus 2
        "O411293",  # Chorioamnionitis, unspecified trimester, fetus 3
        "O411294",  # Chorioamnionitis, unspecified trimester, fetus 4
        "O411295",  # Chorioamnionitis, unspecified trimester, fetus 5
        "O411299",  # Chorioamnionitis, unspecified trimester, other fetus
        "O411410",  # Placentitis, first trimester, not applicable or unspecified
        "O411411",  # Placentitis, first trimester, fetus 1
        "O411412",  # Placentitis, first trimester, fetus 2
        "O411413",  # Placentitis, first trimester, fetus 3
        "O411414",  # Placentitis, first trimester, fetus 4
        "O411415",  # Placentitis, first trimester, fetus 5
        "O411419",  # Placentitis, first trimester, other fetus
        "O411420",  # Placentitis, second trimester, not applicable or unspecified
        "O411421",  # Placentitis, second trimester, fetus 1
        "O411422",  # Placentitis, second trimester, fetus 2
        "O411423",  # Placentitis, second trimester, fetus 3
        "O411424",  # Placentitis, second trimester, fetus 4
        "O411425",  # Placentitis, second trimester, fetus 5
        "O411429",  # Placentitis, second trimester, other fetus
        "O411430",  # Placentitis, third trimester, not applicable or unspecified
        "O411431",  # Placentitis, third trimester, fetus 1
        "O411432",  # Placentitis, third trimester, fetus 2
        "O411433",  # Placentitis, third trimester, fetus 3
        "O411434",  # Placentitis, third trimester, fetus 4
        "O411435",  # Placentitis, third trimester, fetus 5
        "O411439",  # Placentitis, third trimester, other fetus
        "O411490",  # Placentitis, unspecified trimester, not applicable or unspecified
        "O411491",  # Placentitis, unspecified trimester, fetus 1
        "O411492",  # Placentitis, unspecified trimester, fetus 2
        "O411493",  # Placentitis, unspecified trimester, fetus 3
        "O411494",  # Placentitis, unspecified trimester, fetus 4
        "O411495",  # Placentitis, unspecified trimester, fetus 5
        "O411499",  # Placentitis, unspecified trimester, other fetus
        "O418X10",  # Other specified disorders of amniotic fluid and membranes, first trimester, not applicable or unspecified
        "O418X11",  # Other specified disorders of amniotic fluid and membranes, first trimester, fetus 1
        "O418X12",  # Other specified disorders of amniotic fluid and membranes, first trimester, fetus 2
        "O418X13",  # Other specified disorders of amniotic fluid and membranes, first trimester, fetus 3
        "O418X14",  # Other specified disorders of amniotic fluid and membranes, first trimester, fetus 4
        "O418X15",  # Other specified disorders of amniotic fluid and membranes, first trimester, fetus 5
        "O418X19",  # Other specified disorders of amniotic fluid and membranes, first trimester, other fetus
        "O418X20",  # Other specified disorders of amniotic fluid and membranes, second trimester, not applicable or unspecified
        "O418X21",  # Other specified disorders of amniotic fluid and membranes, second trimester, fetus 1
        "O418X22",  # Other specified disorders of amniotic fluid and membranes, second trimester, fetus 2
        "O418X23",  # Other specified disorders of amniotic fluid and membranes, second trimester, fetus 3
        "O418X24",  # Other specified disorders of amniotic fluid and membranes, second trimester, fetus 4
        "O418X25",  # Other specified disorders of amniotic fluid and membranes, second trimester, fetus 5
        "O418X29",  # Other specified disorders of amniotic fluid and membranes, second trimester, other fetus
        "O418X30",  # Other specified disorders of amniotic fluid and membranes, third trimester, not applicable or unspecified
        "O418X31",  # Other specified disorders of amniotic fluid and membranes, third trimester, fetus 1
        "O418X32",  # Other specified disorders of amniotic fluid and membranes, third trimester, fetus 2
        "O418X33",  # Other specified disorders of amniotic fluid and membranes, third trimester, fetus 3
        "O418X34",  # Other specified disorders of amniotic fluid and membranes, third trimester, fetus 4
        "O418X35",  # Other specified disorders of amniotic fluid and membranes, third trimester, fetus 5
        "O418X39",  # Other specified disorders of amniotic fluid and membranes, third trimester, other fetus
        "O418X90",  # Other specified disorders of amniotic fluid and membranes, unspecified trimester, not applicable or unspecified
        "O418X91",  # Other specified disorders of amniotic fluid and membranes, unspecified trimester, fetus 1
        "O418X92",  # Other specified disorders of amniotic fluid and membranes, unspecified trimester, fetus 2
        "O418X93",  # Other specified disorders of amniotic fluid and membranes, unspecified trimester, fetus 3
        "O418X94",  # Other specified disorders of amniotic fluid and membranes, unspecified trimester, fetus 4
        "O418X95",  # Other specified disorders of amniotic fluid and membranes, unspecified trimester, fetus 5
        "O418X99",  # Other specified disorders of amniotic fluid and membranes, unspecified trimester, other fetus
        "O4190X0",  # Disorder of amniotic fluid and membranes, unspecified, unspecified trimester, not applicable or unspecified
        "O4190X1",  # Disorder of amniotic fluid and membranes, unspecified, unspecified trimester, fetus 1
        "O4190X2",  # Disorder of amniotic fluid and membranes, unspecified, unspecified trimester, fetus 2
        "O4190X3",  # Disorder of amniotic fluid and membranes, unspecified, unspecified trimester, fetus 3
        "O4190X4",  # Disorder of amniotic fluid and membranes, unspecified, unspecified trimester, fetus 4
        "O4190X5",  # Disorder of amniotic fluid and membranes, unspecified, unspecified trimester, fetus 5
        "O4190X9",  # Disorder of amniotic fluid and membranes, unspecified, unspecified trimester, other fetus
        "O4191X0",  # Disorder of amniotic fluid and membranes, unspecified, first trimester, not applicable or unspecified
        "O4191X1",  # Disorder of amniotic fluid and membranes, unspecified, first trimester, fetus 1
        "O4191X2",  # Disorder of amniotic fluid and membranes, unspecified, first trimester, fetus 2
        "O4191X3",  # Disorder of amniotic fluid and membranes, unspecified, first trimester, fetus 3
        "O4191X4",  # Disorder of amniotic fluid and membranes, unspecified, first trimester, fetus 4
        "O4191X5",  # Disorder of amniotic fluid and membranes, unspecified, first trimester, fetus 5
        "O4191X9",  # Disorder of amniotic fluid and membranes, unspecified, first trimester, other fetus
        "O4192X0",  # Disorder of amniotic fluid and membranes, unspecified, second trimester, not applicable or unspecified
        "O4192X1",  # Disorder of amniotic fluid and membranes, unspecified, second trimester, fetus 1
        "O4192X2",  # Disorder of amniotic fluid and membranes, unspecified, second trimester, fetus 2
        "O4192X3",  # Disorder of amniotic fluid and membranes, unspecified, second trimester, fetus 3
        "O4192X4",  # Disorder of amniotic fluid and membranes, unspecified, second trimester, fetus 4
        "O4192X5",  # Disorder of amniotic fluid and membranes, unspecified, second trimester, fetus 5
        "O4192X9",  # Disorder of amniotic fluid and membranes, unspecified, second trimester, other fetus
        "O4193X0",  # Disorder of amniotic fluid and membranes, unspecified, third trimester, not applicable or unspecified
        "O4193X1",  # Disorder of amniotic fluid and membranes, unspecified, third trimester, fetus 1
        "O4193X2",  # Disorder of amniotic fluid and membranes, unspecified, third trimester, fetus 2
        "O4193X3",  # Disorder of amniotic fluid and membranes, unspecified, third trimester, fetus 3
        "O4193X4",  # Disorder of amniotic fluid and membranes, unspecified, third trimester, fetus 4
        "O4193X5",  # Disorder of amniotic fluid and membranes, unspecified, third trimester, fetus 5
        "O4193X9",  # Disorder of amniotic fluid and membranes, unspecified, third trimester, other fetus
        "O4200",  # Premature rupture of membranes, onset of labor within 24 hours of rupture, unspecified weeks of gestation
        "O42011",  # Preterm premature rupture of membranes, onset of labor within 24 hours of rupture, first trimester
        "O42012",  # Preterm premature rupture of membranes, onset of labor within 24 hours of rupture, second trimester
        "O42013",  # Preterm premature rupture of membranes, onset of labor within 24 hours of rupture, third trimester
        "O42019",  # Preterm premature rupture of membranes, onset of labor within 24 hours of rupture, unspecified trimester
        "O4202",  # Full-term premature rupture of membranes, onset of labor within 24 hours of rupture
        "O4210",  # Premature rupture of membranes, onset of labor more than 24 hours following rupture, unspecified weeks of gestation
        "O42111",  # Preterm premature rupture of membranes, onset of labor more than 24 hours following rupture, first trimester
        "O42112",  # Preterm premature rupture of membranes, onset of labor more than 24 hours following rupture, second trimester
        "O42113",  # Preterm premature rupture of membranes, onset of labor more than 24 hours following rupture, third trimester
        "O42119",  # Preterm premature rupture of membranes, onset of labor more than 24 hours following rupture, unspecified trimester
        "O4212",  # Full-term premature rupture of membranes, onset of labor more than 24 hours following rupture
        "O4290",  # Premature rupture of membranes, unspecified as to length of time between rupture and onset of labor, unspecified weeks of gestation
        "O42911",  # Preterm premature rupture of membranes, unspecified as to length of time between rupture and onset of labor, first trimester
        "O42912",  # Preterm premature rupture of membranes, unspecified as to length of time between rupture and onset of labor, second trimester
        "O42913",  # Preterm premature rupture of membranes, unspecified as to length of time between rupture and onset of labor, third trimester
        "O42919",  # Preterm premature rupture of membranes, unspecified as to length of time between rupture and onset of labor, unspecified trimester
        "O4292",  # Full-term premature rupture of membranes, unspecified as to length of time between rupture and onset of labor
        "O43011",  # Fetomaternal placental transfusion syndrome, first trimester
        "O43012",  # Fetomaternal placental transfusion syndrome, second trimester
        "O43013",  # Fetomaternal placental transfusion syndrome, third trimester
        "O43019",  # Fetomaternal placental transfusion syndrome, unspecified trimester
        "O43021",  # Fetus-to-fetus placental transfusion syndrome, first trimester
        "O43022",  # Fetus-to-fetus placental transfusion syndrome, second trimester
        "O43023",  # Fetus-to-fetus placental transfusion syndrome, third trimester
        "O43029",  # Fetus-to-fetus placental transfusion syndrome, unspecified trimester
        "O43101",  # Malformation of placenta, unspecified, first trimester
        "O43102",  # Malformation of placenta, unspecified, second trimester
        "O43103",  # Malformation of placenta, unspecified, third trimester
        "O43109",  # Malformation of placenta, unspecified, unspecified trimester
        "O43111",  # Circumvallate placenta, first trimester
        "O43112",  # Circumvallate placenta, second trimester
        "O43113",  # Circumvallate placenta, third trimester
        "O43119",  # Circumvallate placenta, unspecified trimester
        "O43121",  # Velamentous insertion of umbilical cord, first trimester
        "O43122",  # Velamentous insertion of umbilical cord, second trimester
        "O43123",  # Velamentous insertion of umbilical cord, third trimester
        "O43129",  # Velamentous insertion of umbilical cord, unspecified trimester
        "O43191",  # Other malformation of placenta, first trimester
        "O43192",  # Other malformation of placenta, second trimester
        "O43193",  # Other malformation of placenta, third trimester
        "O43199",  # Other malformation of placenta, unspecified trimester
        "O43211",  # Placenta accreta, first trimester
        "O43212",  # Placenta accreta, second trimester
        "O43213",  # Placenta accreta, third trimester
        "O43219",  # Placenta accreta, unspecified trimester
        "O43221",  # Placenta increta, first trimester
        "O43222",  # Placenta increta, second trimester
        "O43223",  # Placenta increta, third trimester
        "O43229",  # Placenta increta, unspecified trimester
        "O43231",  # Placenta percreta, first trimester
        "O43232",  # Placenta percreta, second trimester
        "O43233",  # Placenta percreta, third trimester
        "O43239",  # Placenta percreta, unspecified trimester
        "O43811",  # Placental infarction, first trimester
        "O43812",  # Placental infarction, second trimester
        "O43813",  # Placental infarction, third trimester
        "O43819",  # Placental infarction, unspecified trimester
        "O43891",  # Other placental disorders, first trimester
        "O43892",  # Other placental disorders, second trimester
        "O43893",  # Other placental disorders, third trimester
        "O43899",  # Other placental disorders, unspecified trimester
        "O4390",  # Unspecified placental disorder, unspecified trimester
        "O4391",  # Unspecified placental disorder, first trimester
        "O4392",  # Unspecified placental disorder, second trimester
        "O4393",  # Unspecified placental disorder, third trimester
        "O4400",  # Complete placenta previa NOS or without hemorrhage, unspecified trimester
        "O4401",  # Complete placenta previa NOS or without hemorrhage, first trimester
        "O4402",  # Complete placenta previa NOS or without hemorrhage, second trimester
        "O4403",  # Complete placenta previa NOS or without hemorrhage, third trimester
        "O4410",  # Complete placenta previa with hemorrhage, unspecified trimester
        "O4411",  # Complete placenta previa with hemorrhage, first trimester
        "O4412",  # Complete placenta previa with hemorrhage, second trimester
        "O4413",  # Complete placenta previa with hemorrhage, third trimester
        "O4420",  # Partial placenta previa NOS or without hemorrhage, unspecified trimester
        "O4421",  # Partial placenta previa NOS or without hemorrhage, first trimester
        "O4422",  # Partial placenta previa NOS or without hemorrhage, second trimester
        "O4423",  # Partial placenta previa NOS or without hemorrhage, third trimester
        "O4430",  # Partial placenta previa with hemorrhage, unspecified trimester
        "O4431",  # Partial placenta previa with hemorrhage, first trimester
        "O4432",  # Partial placenta previa with hemorrhage, second trimester
        "O4433",  # Partial placenta previa with hemorrhage, third trimester
        "O4440",  # Low lying placenta NOS or without hemorrhage, unspecified trimester
        "O4441",  # Low lying placenta NOS or without hemorrhage, first trimester
        "O4442",  # Low lying placenta NOS or without hemorrhage, second trimester
        "O4443",  # Low lying placenta NOS or without hemorrhage, third trimester
        "O4450",  # Low lying placenta with hemorrhage, unspecified trimester
        "O4451",  # Low lying placenta with hemorrhage, first trimester
        "O4452",  # Low lying placenta with hemorrhage, second trimester
        "O4453",  # Low lying placenta with hemorrhage, third trimester
        "O45001",  # Premature separation of placenta with coagulation defect, unspecified, first trimester
        "O45002",  # Premature separation of placenta with coagulation defect, unspecified, second trimester
        "O45003",  # Premature separation of placenta with coagulation defect, unspecified, third trimester
        "O45009",  # Premature separation of placenta with coagulation defect, unspecified, unspecified trimester
        "O45011",  # Premature separation of placenta with afibrinogenemia, first trimester
        "O45012",  # Premature separation of placenta with afibrinogenemia, second trimester
        "O45013",  # Premature separation of placenta with afibrinogenemia, third trimester
        "O45019",  # Premature separation of placenta with afibrinogenemia, unspecified trimester
        "O45021",  # Premature separation of placenta with disseminated intravascular coagulation, first trimester
        "O45022",  # Premature separation of placenta with disseminated intravascular coagulation, second trimester
        "O45023",  # Premature separation of placenta with disseminated intravascular coagulation, third trimester
        "O45029",  # Premature separation of placenta with disseminated intravascular coagulation, unspecified trimester
        "O45091",  # Premature separation of placenta with other coagulation defect, first trimester
        "O45092",  # Premature separation of placenta with other coagulation defect, second trimester
        "O45093",  # Premature separation of placenta with other coagulation defect, third trimester
        "O45099",  # Premature separation of placenta with other coagulation defect, unspecified trimester
        "O458X1",  # Other premature separation of placenta, first trimester
        "O458X2",  # Other premature separation of placenta, second trimester
        "O458X3",  # Other premature separation of placenta, third trimester
        "O458X9",  # Other premature separation of placenta, unspecified trimester
        "O4590",  # Premature separation of placenta, unspecified, unspecified trimester
        "O4591",  # Premature separation of placenta, unspecified, first trimester
        "O4592",  # Premature separation of placenta, unspecified, second trimester
        "O4593",  # Premature separation of placenta, unspecified, third trimester
        "O46001",  # Antepartum hemorrhage with coagulation defect, unspecified, first trimester
        "O46002",  # Antepartum hemorrhage with coagulation defect, unspecified, second trimester
        "O46003",  # Antepartum hemorrhage with coagulation defect, unspecified, third trimester
        "O46009",  # Antepartum hemorrhage with coagulation defect, unspecified, unspecified trimester
        "O46011",  # Antepartum hemorrhage with afibrinogenemia, first trimester
        "O46012",  # Antepartum hemorrhage with afibrinogenemia, second trimester
        "O46013",  # Antepartum hemorrhage with afibrinogenemia, third trimester
        "O46019",  # Antepartum hemorrhage with afibrinogenemia, unspecified trimester
        "O46021",  # Antepartum hemorrhage with disseminated intravascular coagulation, first trimester
        "O46022",  # Antepartum hemorrhage with disseminated intravascular coagulation, second trimester
        "O46023",  # Antepartum hemorrhage with disseminated intravascular coagulation, third trimester
        "O46029",  # Antepartum hemorrhage with disseminated intravascular coagulation, unspecified trimester
        "O46091",  # Antepartum hemorrhage with other coagulation defect, first trimester
        "O46092",  # Antepartum hemorrhage with other coagulation defect, second trimester
        "O46093",  # Antepartum hemorrhage with other coagulation defect, third trimester
        "O46099",  # Antepartum hemorrhage with other coagulation defect, unspecified trimester
        "O468X1",  # Other antepartum hemorrhage, first trimester
        "O468X2",  # Other antepartum hemorrhage, second trimester
        "O468X3",  # Other antepartum hemorrhage, third trimester
        "O468X9",  # Other antepartum hemorrhage, unspecified trimester
        "O4690",  # Antepartum hemorrhage, unspecified, unspecified trimester
        "O4691",  # Antepartum hemorrhage, unspecified, first trimester
        "O4692",  # Antepartum hemorrhage, unspecified, second trimester
        "O4693",  # Antepartum hemorrhage, unspecified, third trimester
        "O4700",  # False labor before 37 completed weeks of gestation, unspecified trimester
        "O4702",  # False labor before 37 completed weeks of gestation, second trimester
        "O4703",  # False labor before 37 completed weeks of gestation, third trimester
        "O471",  # False labor at or after 37 completed weeks of gestation
        "O479",  # False labor, unspecified
        "O480",  # Post-term pregnancy
        "O481",  # Prolonged pregnancy
        "O6000",  # Preterm labor without delivery, unspecified trimester
        "O6002",  # Preterm labor without delivery, second trimester
        "O6003",  # Preterm labor without delivery, third trimester
        "O6010X0",  # Preterm labor with preterm delivery, unspecified trimester, not applicable or unspecified
        "O6010X1",  # Preterm labor with preterm delivery, unspecified trimester, fetus 1
        "O6010X2",  # Preterm labor with preterm delivery, unspecified trimester, fetus 2
        "O6010X3",  # Preterm labor with preterm delivery, unspecified trimester, fetus 3
        "O6010X4",  # Preterm labor with preterm delivery, unspecified trimester, fetus 4
        "O6010X5",  # Preterm labor with preterm delivery, unspecified trimester, fetus 5
        "O6010X9",  # Preterm labor with preterm delivery, unspecified trimester, other fetus
        "O6012X0",  # Preterm labor second trimester with preterm delivery second trimester, not applicable or unspecified
        "O6012X1",  # Preterm labor second trimester with preterm delivery second trimester, fetus 1
        "O6012X2",  # Preterm labor second trimester with preterm delivery second trimester, fetus 2
        "O6012X3",  # Preterm labor second trimester with preterm delivery second trimester, fetus 3
        "O6012X4",  # Preterm labor second trimester with preterm delivery second trimester, fetus 4
        "O6012X5",  # Preterm labor second trimester with preterm delivery second trimester, fetus 5
        "O6012X9",  # Preterm labor second trimester with preterm delivery second trimester, other fetus
        "O6013X0",  # Preterm labor second trimester with preterm delivery third trimester, not applicable or unspecified
        "O6013X1",  # Preterm labor second trimester with preterm delivery third trimester, fetus 1
        "O6013X2",  # Preterm labor second trimester with preterm delivery third trimester, fetus 2
        "O6013X3",  # Preterm labor second trimester with preterm delivery third trimester, fetus 3
        "O6013X4",  # Preterm labor second trimester with preterm delivery third trimester, fetus 4
        "O6013X5",  # Preterm labor second trimester with preterm delivery third trimester, fetus 5
        "O6013X9",  # Preterm labor second trimester with preterm delivery third trimester, other fetus
        "O6014X0",  # Preterm labor third trimester with preterm delivery third trimester, not applicable or unspecified
        "O6014X1",  # Preterm labor third trimester with preterm delivery third trimester, fetus 1
        "O6014X2",  # Preterm labor third trimester with preterm delivery third trimester, fetus 2
        "O6014X3",  # Preterm labor third trimester with preterm delivery third trimester, fetus 3
        "O6014X4",  # Preterm labor third trimester with preterm delivery third trimester, fetus 4
        "O6014X5",  # Preterm labor third trimester with preterm delivery third trimester, fetus 5
        "O6014X9",  # Preterm labor third trimester with preterm delivery third trimester, other fetus
        "O6020X0",  # Term delivery with preterm labor, unspecified trimester, not applicable or unspecified
        "O6020X1",  # Term delivery with preterm labor, unspecified trimester, fetus 1
        "O6020X2",  # Term delivery with preterm labor, unspecified trimester, fetus 2
        "O6020X3",  # Term delivery with preterm labor, unspecified trimester, fetus 3
        "O6020X4",  # Term delivery with preterm labor, unspecified trimester, fetus 4
        "O6020X5",  # Term delivery with preterm labor, unspecified trimester, fetus 5
        "O6020X9",  # Term delivery with preterm labor, unspecified trimester, other fetus
        "O6022X0",  # Term delivery with preterm labor, second trimester, not applicable or unspecified
        "O6022X1",  # Term delivery with preterm labor, second trimester, fetus 1
        "O6022X2",  # Term delivery with preterm labor, second trimester, fetus 2
        "O6022X3",  # Term delivery with preterm labor, second trimester, fetus 3
        "O6022X4",  # Term delivery with preterm labor, second trimester, fetus 4
        "O6022X5",  # Term delivery with preterm labor, second trimester, fetus 5
        "O6022X9",  # Term delivery with preterm labor, second trimester, other fetus
        "O6023X0",  # Term delivery with preterm labor, third trimester, not applicable or unspecified
        "O6023X1",  # Term delivery with preterm labor, third trimester, fetus 1
        "O6023X2",  # Term delivery with preterm labor, third trimester, fetus 2
        "O6023X3",  # Term delivery with preterm labor, third trimester, fetus 3
        "O6023X4",  # Term delivery with preterm labor, third trimester, fetus 4
        "O6023X5",  # Term delivery with preterm labor, third trimester, fetus 5
        "O6023X9",  # Term delivery with preterm labor, third trimester, other fetus
        "O610",  # Failed medical induction of labor
        "O611",  # Failed instrumental induction of labor
        "O618",  # Other failed induction of labor
        "O619",  # Failed induction of labor, unspecified
        "O620",  # Primary inadequate contractions
        "O621",  # Secondary uterine inertia
        "O622",  # Other uterine inertia
        "O623",  # Precipitate labor
        "O624",  # Hypertonic, incoordinate, and prolonged uterine contractions
        "O628",  # Other abnormalities of forces of labor
        "O629",  # Abnormality of forces of labor, unspecified
        "O630",  # Prolonged first stage (of labor)
        "O631",  # Prolonged second stage (of labor)
        "O632",  # Delayed delivery of second twin, triplet, etc.
        "O639",  # Long labor, unspecified
        "O640XX0",  # Obstructed labor due to incomplete rotation of fetal head, not applicable or unspecified
        "O640XX1",  # Obstructed labor due to incomplete rotation of fetal head, fetus 1
        "O640XX2",  # Obstructed labor due to incomplete rotation of fetal head, fetus 2
        "O640XX3",  # Obstructed labor due to incomplete rotation of fetal head, fetus 3
        "O640XX4",  # Obstructed labor due to incomplete rotation of fetal head, fetus 4
        "O640XX5",  # Obstructed labor due to incomplete rotation of fetal head, fetus 5
        "O640XX9",  # Obstructed labor due to incomplete rotation of fetal head, other fetus
        "O641XX0",  # Obstructed labor due to breech presentation, not applicable or unspecified
        "O641XX1",  # Obstructed labor due to breech presentation, fetus 1
        "O641XX2",  # Obstructed labor due to breech presentation, fetus 2
        "O641XX3",  # Obstructed labor due to breech presentation, fetus 3
        "O641XX4",  # Obstructed labor due to breech presentation, fetus 4
        "O641XX5",  # Obstructed labor due to breech presentation, fetus 5
        "O641XX9",  # Obstructed labor due to breech presentation, other fetus
        "O642XX0",  # Obstructed labor due to face presentation, not applicable or unspecified
        "O642XX1",  # Obstructed labor due to face presentation, fetus 1
        "O642XX2",  # Obstructed labor due to face presentation, fetus 2
        "O642XX3",  # Obstructed labor due to face presentation, fetus 3
        "O642XX4",  # Obstructed labor due to face presentation, fetus 4
        "O642XX5",  # Obstructed labor due to face presentation, fetus 5
        "O642XX9",  # Obstructed labor due to face presentation, other fetus
        "O643XX0",  # Obstructed labor due to brow presentation, not applicable or unspecified
        "O643XX1",  # Obstructed labor due to brow presentation, fetus 1
        "O643XX2",  # Obstructed labor due to brow presentation, fetus 2
        "O643XX3",  # Obstructed labor due to brow presentation, fetus 3
        "O643XX4",  # Obstructed labor due to brow presentation, fetus 4
        "O643XX5",  # Obstructed labor due to brow presentation, fetus 5
        "O643XX9",  # Obstructed labor due to brow presentation, other fetus
        "O644XX0",  # Obstructed labor due to shoulder presentation, not applicable or unspecified
        "O644XX1",  # Obstructed labor due to shoulder presentation, fetus 1
        "O644XX2",  # Obstructed labor due to shoulder presentation, fetus 2
        "O644XX3",  # Obstructed labor due to shoulder presentation, fetus 3
        "O644XX4",  # Obstructed labor due to shoulder presentation, fetus 4
        "O644XX5",  # Obstructed labor due to shoulder presentation, fetus 5
        "O644XX9",  # Obstructed labor due to shoulder presentation, other fetus
        "O645XX0",  # Obstructed labor due to compound presentation, not applicable or unspecified
        "O645XX1",  # Obstructed labor due to compound presentation, fetus 1
        "O645XX2",  # Obstructed labor due to compound presentation, fetus 2
        "O645XX3",  # Obstructed labor due to compound presentation, fetus 3
        "O645XX4",  # Obstructed labor due to compound presentation, fetus 4
        "O645XX5",  # Obstructed labor due to compound presentation, fetus 5
        "O645XX9",  # Obstructed labor due to compound presentation, other fetus
        "O648XX0",  # Obstructed labor due to other malposition and malpresentation, not applicable or unspecified
        "O648XX1",  # Obstructed labor due to other malposition and malpresentation, fetus 1
        "O648XX2",  # Obstructed labor due to other malposition and malpresentation, fetus 2
        "O648XX3",  # Obstructed labor due to other malposition and malpresentation, fetus 3
        "O648XX4",  # Obstructed labor due to other malposition and malpresentation, fetus 4
        "O648XX5",  # Obstructed labor due to other malposition and malpresentation, fetus 5
        "O648XX9",  # Obstructed labor due to other malposition and malpresentation, other fetus
        "O649XX0",  # Obstructed labor due to malposition and malpresentation, unspecified, not applicable or unspecified
        "O649XX1",  # Obstructed labor due to malposition and malpresentation, unspecified, fetus 1
        "O649XX2",  # Obstructed labor due to malposition and malpresentation, unspecified, fetus 2
        "O649XX3",  # Obstructed labor due to malposition and malpresentation, unspecified, fetus 3
        "O649XX4",  # Obstructed labor due to malposition and malpresentation, unspecified, fetus 4
        "O649XX5",  # Obstructed labor due to malposition and malpresentation, unspecified, fetus 5
        "O649XX9",  # Obstructed labor due to malposition and malpresentation, unspecified, other fetus
        "O650",  # Obstructed labor due to deformed pelvis
        "O651",  # Obstructed labor due to generally contracted pelvis
        "O652",  # Obstructed labor due to pelvic inlet contraction
        "O653",  # Obstructed labor due to pelvic outlet and mid-cavity contraction
        "O654",  # Obstructed labor due to fetopelvic disproportion, unspecified
        "O655",  # Obstructed labor due to abnormality of maternal pelvic organs
        "O658",  # Obstructed labor due to other maternal pelvic abnormalities
        "O659",  # Obstructed labor due to maternal pelvic abnormality, unspecified
        "O660",  # Obstructed labor due to shoulder dystocia
        "O661",  # Obstructed labor due to locked twins
        "O662",  # Obstructed labor due to unusually large fetus
        "O663",  # Obstructed labor due to other abnormalities of fetus
        "O6640",  # Failed trial of labor, unspecified
        "O6641",  # Failed attempted vaginal birth after previous cesarean delivery
        "O665",  # Attempted application of vacuum extractor and forceps
        "O666",  # Obstructed labor due to other multiple fetuses
        "O668",  # Other specified obstructed labor
        "O669",  # Obstructed labor, unspecified
        "O670",  # Intrapartum hemorrhage with coagulation defect
        "O678",  # Other intrapartum hemorrhage
        "O679",  # Intrapartum hemorrhage, unspecified
        "O68",  # Labor and delivery complicated by abnormality of fetal acid-base balance
        "O690XX0",  # Labor and delivery complicated by prolapse of cord, not applicable or unspecified
        "O690XX1",  # Labor and delivery complicated by prolapse of cord, fetus 1
        "O690XX2",  # Labor and delivery complicated by prolapse of cord, fetus 2
        "O690XX3",  # Labor and delivery complicated by prolapse of cord, fetus 3
        "O690XX4",  # Labor and delivery complicated by prolapse of cord, fetus 4
        "O690XX5",  # Labor and delivery complicated by prolapse of cord, fetus 5
        "O690XX9",  # Labor and delivery complicated by prolapse of cord, other fetus
        "O691XX0",  # Labor and delivery complicated by cord around neck, with compression, not applicable or unspecified
        "O691XX1",  # Labor and delivery complicated by cord around neck, with compression, fetus 1
        "O691XX2",  # Labor and delivery complicated by cord around neck, with compression, fetus 2
        "O691XX3",  # Labor and delivery complicated by cord around neck, with compression, fetus 3
        "O691XX4",  # Labor and delivery complicated by cord around neck, with compression, fetus 4
        "O691XX5",  # Labor and delivery complicated by cord around neck, with compression, fetus 5
        "O691XX9",  # Labor and delivery complicated by cord around neck, with compression, other fetus
        "O692XX0",  # Labor and delivery complicated by other cord entanglement, with compression, not applicable or unspecified
        "O692XX1",  # Labor and delivery complicated by other cord entanglement, with compression, fetus 1
        "O692XX2",  # Labor and delivery complicated by other cord entanglement, with compression, fetus 2
        "O692XX3",  # Labor and delivery complicated by other cord entanglement, with compression, fetus 3
        "O692XX4",  # Labor and delivery complicated by other cord entanglement, with compression, fetus 4
        "O692XX5",  # Labor and delivery complicated by other cord entanglement, with compression, fetus 5
        "O692XX9",  # Labor and delivery complicated by other cord entanglement, with compression, other fetus
        "O693XX0",  # Labor and delivery complicated by short cord, not applicable or unspecified
        "O693XX1",  # Labor and delivery complicated by short cord, fetus 1
        "O693XX2",  # Labor and delivery complicated by short cord, fetus 2
        "O693XX3",  # Labor and delivery complicated by short cord, fetus 3
        "O693XX4",  # Labor and delivery complicated by short cord, fetus 4
        "O693XX5",  # Labor and delivery complicated by short cord, fetus 5
        "O693XX9",  # Labor and delivery complicated by short cord, other fetus
        "O694XX0",  # Labor and delivery complicated by vasa previa, not applicable or unspecified
        "O694XX1",  # Labor and delivery complicated by vasa previa, fetus 1
        "O694XX2",  # Labor and delivery complicated by vasa previa, fetus 2
        "O694XX3",  # Labor and delivery complicated by vasa previa, fetus 3
        "O694XX4",  # Labor and delivery complicated by vasa previa, fetus 4
        "O694XX5",  # Labor and delivery complicated by vasa previa, fetus 5
        "O694XX9",  # Labor and delivery complicated by vasa previa, other fetus
        "O695XX0",  # Labor and delivery complicated by vascular lesion of cord, not applicable or unspecified
        "O695XX1",  # Labor and delivery complicated by vascular lesion of cord, fetus 1
        "O695XX2",  # Labor and delivery complicated by vascular lesion of cord, fetus 2
        "O695XX3",  # Labor and delivery complicated by vascular lesion of cord, fetus 3
        "O695XX4",  # Labor and delivery complicated by vascular lesion of cord, fetus 4
        "O695XX5",  # Labor and delivery complicated by vascular lesion of cord, fetus 5
        "O695XX9",  # Labor and delivery complicated by vascular lesion of cord, other fetus
        "O6981X0",  # Labor and delivery complicated by cord around neck, without compression, not applicable or unspecified
        "O6981X1",  # Labor and delivery complicated by cord around neck, without compression, fetus 1
        "O6981X2",  # Labor and delivery complicated by cord around neck, without compression, fetus 2
        "O6981X3",  # Labor and delivery complicated by cord around neck, without compression, fetus 3
        "O6981X4",  # Labor and delivery complicated by cord around neck, without compression, fetus 4
        "O6981X5",  # Labor and delivery complicated by cord around neck, without compression, fetus 5
        "O6981X9",  # Labor and delivery complicated by cord around neck, without compression, other fetus
        "O6982X0",  # Labor and delivery complicated by other cord entanglement, without compression, not applicable or unspecified
        "O6982X1",  # Labor and delivery complicated by other cord entanglement, without compression, fetus 1
        "O6982X2",  # Labor and delivery complicated by other cord entanglement, without compression, fetus 2
        "O6982X3",  # Labor and delivery complicated by other cord entanglement, without compression, fetus 3
        "O6982X4",  # Labor and delivery complicated by other cord entanglement, without compression, fetus 4
        "O6982X5",  # Labor and delivery complicated by other cord entanglement, without compression, fetus 5
        "O6982X9",  # Labor and delivery complicated by other cord entanglement, without compression, other fetus
        "O6989X0",  # Labor and delivery complicated by other cord complications, not applicable or unspecified
        "O6989X1",  # Labor and delivery complicated by other cord complications, fetus 1
        "O6989X2",  # Labor and delivery complicated by other cord complications, fetus 2
        "O6989X3",  # Labor and delivery complicated by other cord complications, fetus 3
        "O6989X4",  # Labor and delivery complicated by other cord complications, fetus 4
        "O6989X5",  # Labor and delivery complicated by other cord complications, fetus 5
        "O6989X9",  # Labor and delivery complicated by other cord complications, other fetus
        "O699XX0",  # Labor and delivery complicated by cord complication, unspecified, not applicable or unspecified
        "O699XX1",  # Labor and delivery complicated by cord complication, unspecified, fetus 1
        "O699XX2",  # Labor and delivery complicated by cord complication, unspecified, fetus 2
        "O699XX3",  # Labor and delivery complicated by cord complication, unspecified, fetus 3
        "O699XX4",  # Labor and delivery complicated by cord complication, unspecified, fetus 4
        "O699XX5",  # Labor and delivery complicated by cord complication, unspecified, fetus 5
        "O699XX9",  # Labor and delivery complicated by cord complication, unspecified, other fetus
        "O700",  # First degree perineal laceration during delivery
        "O701",  # Second degree perineal laceration during delivery
        "O7020",  # Third degree perineal laceration during delivery, unspecified
        "O7021",  # Third degree perineal laceration during delivery, IIIa
        "O7022",  # Third degree perineal laceration during delivery, IIIb
        "O7023",  # Third degree perineal laceration during delivery, IIIc
        "O703",  # Fourth degree perineal laceration during delivery
        "O704",  # Anal sphincter tear complicating delivery, not associated with third degree laceration
        "O709",  # Perineal laceration during delivery, unspecified
        "O7100",  # Rupture of uterus before onset of labor, unspecified trimester
        "O7102",  # Rupture of uterus before onset of labor, second trimester
        "O7103",  # Rupture of uterus before onset of labor, third trimester
        "O711",  # Rupture of uterus during labor
        "O713",  # Obstetric laceration of cervix
        "O714",  # Obstetric high vaginal laceration alone
        "O715",  # Other obstetric injury to pelvic organs
        "O716",  # Obstetric damage to pelvic joints and ligaments
        "O717",  # Obstetric hematoma of pelvis
        "O7181",  # Laceration of uterus, not elsewhere classified
        "O7182",  # Other specified trauma to perineum and vulva
        "O7189",  # Other specified obstetric trauma
        "O719",  # Obstetric trauma, unspecified
        "O720",  # Third-stage hemorrhage
        "O730",  # Retained placenta without hemorrhage
        "O731",  # Retained portions of placenta and membranes, without hemorrhage
        "O740",  # Aspiration pneumonitis due to anesthesia during labor and delivery
        "O741",  # Other pulmonary complications of anesthesia during labor and delivery
        "O742",  # Cardiac complications of anesthesia during labor and delivery
        "O743",  # Central nervous system complications of anesthesia during labor and delivery
        "O744",  # Toxic reaction to local anesthesia during labor and delivery
        "O745",  # Spinal and epidural anesthesia-induced headache during labor and delivery
        "O746",  # Other complications of spinal and epidural anesthesia during labor and delivery
        "O747",  # Failed or difficult intubation for anesthesia during labor and delivery
        "O748",  # Other complications of anesthesia during labor and delivery
        "O749",  # Complication of anesthesia during labor and delivery, unspecified
        "O750",  # Maternal distress during labor and delivery
        "O751",  # Shock during or following labor and delivery
        "O752",  # Pyrexia during labor, not elsewhere classified
        "O753",  # Other infection during labor
        "O754",  # Other complications of obstetric surgery and procedures
        "O755",  # Delayed delivery after artificial rupture of membranes
        "O7581",  # Maternal exhaustion complicating labor and delivery
        "O7582",  # Onset (spontaneous) of labor after 37 completed weeks of gestation but before 39 completed weeks gestation, with delivery by (planned) cesarean section
        "O7589",  # Other specified complications of labor and delivery
        "O759",  # Complication of labor and delivery, unspecified
        "O76",  # Abnormality in fetal heart rate and rhythm complicating labor and delivery
        "O770",  # Labor and delivery complicated by meconium in amniotic fluid
        "O771",  # Fetal stress in labor or delivery due to drug administration
        "O778",  # Labor and delivery complicated by other evidence of fetal stress
        "O779",  # Labor and delivery complicated by fetal stress, unspecified
        "O80",  # Encounter for full-term uncomplicated delivery
        "O82",  # Encounter for cesarean delivery without indication
        "O88011",  # Air embolism in pregnancy, first trimester
        "O88012",  # Air embolism in pregnancy, second trimester
        "O88013",  # Air embolism in pregnancy, third trimester
        "O88019",  # Air embolism in pregnancy, unspecified trimester
        "O8802",  # Air embolism in childbirth
        "O88111",  # Amniotic fluid embolism in pregnancy, first trimester
        "O88112",  # Amniotic fluid embolism in pregnancy, second trimester
        "O88113",  # Amniotic fluid embolism in pregnancy, third trimester
        "O88119",  # Amniotic fluid embolism in pregnancy, unspecified trimester
        "O8812",  # Amniotic fluid embolism in childbirth
        "O88211",  # Thromboembolism in pregnancy, first trimester
        "O88212",  # Thromboembolism in pregnancy, second trimester
        "O88213",  # Thromboembolism in pregnancy, third trimester
        "O88219",  # Thromboembolism in pregnancy, unspecified trimester
        "O8822",  # Thromboembolism in childbirth
        "O88311",  # Pyemic and septic embolism in pregnancy, first trimester
        "O88312",  # Pyemic and septic embolism in pregnancy, second trimester
        "O88313",  # Pyemic and septic embolism in pregnancy, third trimester
        "O88319",  # Pyemic and septic embolism in pregnancy, unspecified trimester
        "O8832",  # Pyemic and septic embolism in childbirth
        "O88811",  # Other embolism in pregnancy, first trimester
        "O88812",  # Other embolism in pregnancy, second trimester
        "O88813",  # Other embolism in pregnancy, third trimester
        "O88819",  # Other embolism in pregnancy, unspecified trimester
        "O8882",  # Other embolism in childbirth
        "O91011",  # Infection of nipple associated with pregnancy, first trimester
        "O91012",  # Infection of nipple associated with pregnancy, second trimester
        "O91013",  # Infection of nipple associated with pregnancy, third trimester
        "O91019",  # Infection of nipple associated with pregnancy, unspecified trimester
        "O91111",  # Abscess of breast associated with pregnancy, first trimester
        "O91112",  # Abscess of breast associated with pregnancy, second trimester
        "O91113",  # Abscess of breast associated with pregnancy, third trimester
        "O91119",  # Abscess of breast associated with pregnancy, unspecified trimester
        "O91211",  # Nonpurulent mastitis associated with pregnancy, first trimester
        "O91212",  # Nonpurulent mastitis associated with pregnancy, second trimester
        "O91213",  # Nonpurulent mastitis associated with pregnancy, third trimester
        "O91219",  # Nonpurulent mastitis associated with pregnancy, unspecified trimester
        "O92011",  # Retracted nipple associated with pregnancy, first trimester
        "O92012",  # Retracted nipple associated with pregnancy, second trimester
        "O92013",  # Retracted nipple associated with pregnancy, third trimester
        "O92019",  # Retracted nipple associated with pregnancy, unspecified trimester
        "O92111",  # Cracked nipple associated with pregnancy, first trimester
        "O92112",  # Cracked nipple associated with pregnancy, second trimester
        "O92113",  # Cracked nipple associated with pregnancy, third trimester
        "O92119",  # Cracked nipple associated with pregnancy, unspecified trimester
        "O98011",  # Tuberculosis complicating pregnancy, first trimester
        "O98012",  # Tuberculosis complicating pregnancy, second trimester
        "O98013",  # Tuberculosis complicating pregnancy, third trimester
        "O98019",  # Tuberculosis complicating pregnancy, unspecified trimester
        "O9802",  # Tuberculosis complicating childbirth
        "O98111",  # Syphilis complicating pregnancy, first trimester
        "O98112",  # Syphilis complicating pregnancy, second trimester
        "O98113",  # Syphilis complicating pregnancy, third trimester
        "O98119",  # Syphilis complicating pregnancy, unspecified trimester
        "O9812",  # Syphilis complicating childbirth
        "O98211",  # Gonorrhea complicating pregnancy, first trimester
        "O98212",  # Gonorrhea complicating pregnancy, second trimester
        "O98213",  # Gonorrhea complicating pregnancy, third trimester
        "O98219",  # Gonorrhea complicating pregnancy, unspecified trimester
        "O9822",  # Gonorrhea complicating childbirth
        "O98311",  # Other infections with a predominantly sexual mode of transmission complicating pregnancy, first trimester
        "O98312",  # Other infections with a predominantly sexual mode of transmission complicating pregnancy, second trimester
        "O98313",  # Other infections with a predominantly sexual mode of transmission complicating pregnancy, third trimester
        "O98319",  # Other infections with a predominantly sexual mode of transmission complicating pregnancy, unspecified trimester
        "O9832",  # Other infections with a predominantly sexual mode of transmission complicating childbirth
        "O98411",  # Viral hepatitis complicating pregnancy, first trimester
        "O98412",  # Viral hepatitis complicating pregnancy, second trimester
        "O98413",  # Viral hepatitis complicating pregnancy, third trimester
        "O98419",  # Viral hepatitis complicating pregnancy, unspecified trimester
        "O9842",  # Viral hepatitis complicating childbirth
        "O98511",  # Other viral diseases complicating pregnancy, first trimester
        "O98512",  # Other viral diseases complicating pregnancy, second trimester
        "O98513",  # Other viral diseases complicating pregnancy, third trimester
        "O98519",  # Other viral diseases complicating pregnancy, unspecified trimester
        "O9852",  # Other viral diseases complicating childbirth
        "O98611",  # Protozoal diseases complicating pregnancy, first trimester
        "O98612",  # Protozoal diseases complicating pregnancy, second trimester
        "O98613",  # Protozoal diseases complicating pregnancy, third trimester
        "O98619",  # Protozoal diseases complicating pregnancy, unspecified trimester
        "O9862",  # Protozoal diseases complicating childbirth
        "O98711",  # Human immunodeficiency virus [HIV] disease complicating pregnancy, first trimester
        "O98712",  # Human immunodeficiency virus [HIV] disease complicating pregnancy, second trimester
        "O98713",  # Human immunodeficiency virus [HIV] disease complicating pregnancy, third trimester
        "O98719",  # Human immunodeficiency virus [HIV] disease complicating pregnancy, unspecified trimester
        "O9872",  # Human immunodeficiency virus [HIV] disease complicating childbirth
        "O98811",  # Other maternal infectious and parasitic diseases complicating pregnancy, first trimester
        "O98812",  # Other maternal infectious and parasitic diseases complicating pregnancy, second trimester
        "O98813",  # Other maternal infectious and parasitic diseases complicating pregnancy, third trimester
        "O98819",  # Other maternal infectious and parasitic diseases complicating pregnancy, unspecified trimester
        "O9882",  # Other maternal infectious and parasitic diseases complicating childbirth
        "O98911",  # Unspecified maternal infectious and parasitic disease complicating pregnancy, first trimester
        "O98912",  # Unspecified maternal infectious and parasitic disease complicating pregnancy, second trimester
        "O98913",  # Unspecified maternal infectious and parasitic disease complicating pregnancy, third trimester
        "O98919",  # Unspecified maternal infectious and parasitic disease complicating pregnancy, unspecified trimester
        "O9892",  # Unspecified maternal infectious and parasitic disease complicating childbirth
        "O99011",  # Anemia complicating pregnancy, first trimester
        "O99012",  # Anemia complicating pregnancy, second trimester
        "O99013",  # Anemia complicating pregnancy, third trimester
        "O99019",  # Anemia complicating pregnancy, unspecified trimester
        "O9902",  # Anemia complicating childbirth
        "O99111",  # Other diseases of the blood and blood-forming organs and certain disorders involving the immune mechanism complicating pregnancy, first trimester
        "O99112",  # Other diseases of the blood and blood-forming organs and certain disorders involving the immune mechanism complicating pregnancy, second trimester
        "O99113",  # Other diseases of the blood and blood-forming organs and certain disorders involving the immune mechanism complicating pregnancy, third trimester
        "O99119",  # Other diseases of the blood and blood-forming organs and certain disorders involving the immune mechanism complicating pregnancy, unspecified trimester
        "O9912",  # Other diseases of the blood and blood-forming organs and certain disorders involving the immune mechanism complicating childbirth
        "O99210",  # Obesity complicating pregnancy, unspecified trimester
        "O99211",  # Obesity complicating pregnancy, first trimester
        "O99212",  # Obesity complicating pregnancy, second trimester
        "O99213",  # Obesity complicating pregnancy, third trimester
        "O99214",  # Obesity complicating childbirth
        "O99280",  # Endocrine, nutritional and metabolic diseases complicating pregnancy, unspecified trimester
        "O99281",  # Endocrine, nutritional and metabolic diseases complicating pregnancy, first trimester
        "O99282",  # Endocrine, nutritional and metabolic diseases complicating pregnancy, second trimester
        "O99283",  # Endocrine, nutritional and metabolic diseases complicating pregnancy, third trimester
        "O99284",  # Endocrine, nutritional and metabolic diseases complicating childbirth
        "O99310",  # Alcohol use complicating pregnancy, unspecified trimester
        "O99311",  # Alcohol use complicating pregnancy, first trimester
        "O99312",  # Alcohol use complicating pregnancy, second trimester
        "O99313",  # Alcohol use complicating pregnancy, third trimester
        "O99314",  # Alcohol use complicating childbirth
        "O99320",  # Drug use complicating pregnancy, unspecified trimester
        "O99321",  # Drug use complicating pregnancy, first trimester
        "O99322",  # Drug use complicating pregnancy, second trimester
        "O99323",  # Drug use complicating pregnancy, third trimester
        "O99324",  # Drug use complicating childbirth
        "O99330",  # Smoking (tobacco) complicating pregnancy, unspecified trimester
        "O99331",  # Smoking (tobacco) complicating pregnancy, first trimester
        "O99332",  # Smoking (tobacco) complicating pregnancy, second trimester
        "O99333",  # Smoking (tobacco) complicating pregnancy, third trimester
        "O99334",  # Smoking (tobacco) complicating childbirth
        "O99340",  # Other mental disorders complicating pregnancy, unspecified trimester
        "O99341",  # Other mental disorders complicating pregnancy, first trimester
        "O99342",  # Other mental disorders complicating pregnancy, second trimester
        "O99343",  # Other mental disorders complicating pregnancy, third trimester
        "O99344",  # Other mental disorders complicating childbirth
        "O99350",  # Diseases of the nervous system complicating pregnancy, unspecified trimester
        "O99351",  # Diseases of the nervous system complicating pregnancy, first trimester
        "O99352",  # Diseases of the nervous system complicating pregnancy, second trimester
        "O99353",  # Diseases of the nervous system complicating pregnancy, third trimester
        "O99354",  # Diseases of the nervous system complicating childbirth
        "O99411",  # Diseases of the circulatory system complicating pregnancy, first trimester
        "O99412",  # Diseases of the circulatory system complicating pregnancy, second trimester
        "O99413",  # Diseases of the circulatory system complicating pregnancy, third trimester
        "O99419",  # Diseases of the circulatory system complicating pregnancy, unspecified trimester
        "O9942",  # Diseases of the circulatory system complicating childbirth
        "O99511",  # Diseases of the respiratory system complicating pregnancy, first trimester
        "O99512",  # Diseases of the respiratory system complicating pregnancy, second trimester
        "O99513",  # Diseases of the respiratory system complicating pregnancy, third trimester
        "O99519",  # Diseases of the respiratory system complicating pregnancy, unspecified trimester
        "O9952",  # Diseases of the respiratory system complicating childbirth
        "O99611",  # Diseases of the digestive system complicating pregnancy, first trimester
        "O99612",  # Diseases of the digestive system complicating pregnancy, second trimester
        "O99613",  # Diseases of the digestive system complicating pregnancy, third trimester
        "O99619",  # Diseases of the digestive system complicating pregnancy, unspecified trimester
        "O9962",  # Diseases of the digestive system complicating childbirth
        "O99711",  # Diseases of the skin and subcutaneous tissue complicating pregnancy, first trimester
        "O99712",  # Diseases of the skin and subcutaneous tissue complicating pregnancy, second trimester
        "O99713",  # Diseases of the skin and subcutaneous tissue complicating pregnancy, third trimester
        "O99719",  # Diseases of the skin and subcutaneous tissue complicating pregnancy, unspecified trimester
        "O9972",  # Diseases of the skin and subcutaneous tissue complicating childbirth
        "O99810",  # Abnormal glucose complicating pregnancy
        "O99814",  # Abnormal glucose complicating childbirth
        "O99820",  # Streptococcus B carrier state complicating pregnancy
        "O99824",  # Streptococcus B carrier state complicating childbirth
        "O99830",  # Other infection carrier state complicating pregnancy
        "O99834",  # Other infection carrier state complicating childbirth
        "O99840",  # Bariatric surgery status complicating pregnancy, unspecified trimester
        "O99841",  # Bariatric surgery status complicating pregnancy, first trimester
        "O99842",  # Bariatric surgery status complicating pregnancy, second trimester
        "O99843",  # Bariatric surgery status complicating pregnancy, third trimester
        "O99844",  # Bariatric surgery status complicating childbirth
        "O99891",  # Other specified diseases and conditions complicating pregnancy
        "O99892",  # Other specified diseases and conditions complicating childbirth
        "O9A111",  # Malignant neoplasm complicating pregnancy, first trimester
        "O9A112",  # Malignant neoplasm complicating pregnancy, second trimester
        "O9A113",  # Malignant neoplasm complicating pregnancy, third trimester
        "O9A119",  # Malignant neoplasm complicating pregnancy, unspecified trimester
        "O9A12",  # Malignant neoplasm complicating childbirth
        "O9A211",  # Injury, poisoning and certain other consequences of external causes complicating pregnancy, first trimester
        "O9A212",  # Injury, poisoning and certain other consequences of external causes complicating pregnancy, second trimester
        "O9A213",  # Injury, poisoning and certain other consequences of external causes complicating pregnancy, third trimester
        "O9A219",  # Injury, poisoning and certain other consequences of external causes complicating pregnancy, unspecified trimester
        "O9A22",  # Injury, poisoning and certain other consequences of external causes complicating childbirth
        "O9A311",  # Physical abuse complicating pregnancy, first trimester
        "O9A312",  # Physical abuse complicating pregnancy, second trimester
        "O9A313",  # Physical abuse complicating pregnancy, third trimester
        "O9A319",  # Physical abuse complicating pregnancy, unspecified trimester
        "O9A32",  # Physical abuse complicating childbirth
        "O9A411",  # Sexual abuse complicating pregnancy, first trimester
        "O9A412",  # Sexual abuse complicating pregnancy, second trimester
        "O9A413",  # Sexual abuse complicating pregnancy, third trimester
        "O9A419",  # Sexual abuse complicating pregnancy, unspecified trimester
        "O9A42",  # Sexual abuse complicating childbirth
        "O9A511",  # Psychological abuse complicating pregnancy, first trimester
        "O9A512",  # Psychological abuse complicating pregnancy, second trimester
        "O9A513",  # Psychological abuse complicating pregnancy, third trimester
        "O9A519",  # Psychological abuse complicating pregnancy, unspecified trimester
        "O9A52",  # Psychological abuse complicating childbirth
        "Z0371",  # Encounter for suspected problem with amniotic cavity and membrane ruled out
        "Z0372",  # Encounter for suspected placental problem ruled out
        "Z0373",  # Encounter for suspected fetal anomaly ruled out
        "Z0374",  # Encounter for suspected problem with fetal growth ruled out
        "Z0375",  # Encounter for suspected cervical shortening ruled out
        "Z0379",  # Encounter for other suspected maternal and fetal conditions ruled out
        "Z331",  # Pregnant state, incidental
        "Z332",  # Encounter for elective termination of pregnancy
        "Z333",  # Pregnant state, gestational carrier
        "Z3400",  # Encounter for supervision of normal first pregnancy, unspecified trimester
        "Z3401",  # Encounter for supervision of normal first pregnancy, first trimester
        "Z3402",  # Encounter for supervision of normal first pregnancy, second trimester
        "Z3403",  # Encounter for supervision of normal first pregnancy, third trimester
        "Z3480",  # Encounter for supervision of other normal pregnancy, unspecified trimester
        "Z3481",  # Encounter for supervision of other normal pregnancy, first trimester
        "Z3482",  # Encounter for supervision of other normal pregnancy, second trimester
        "Z3483",  # Encounter for supervision of other normal pregnancy, third trimester
        "Z3490",  # Encounter for supervision of normal pregnancy, unspecified, unspecified trimester
        "Z3491",  # Encounter for supervision of normal pregnancy, unspecified, first trimester
        "Z3492",  # Encounter for supervision of normal pregnancy, unspecified, second trimester
        "Z3493",  # Encounter for supervision of normal pregnancy, unspecified, third trimester
        "Z360",  # Encounter for antenatal screening for chromosomal anomalies
        "Z361",  # Encounter for antenatal screening for raised alphafetoprotein level
        "Z362",  # Encounter for other antenatal screening follow-up
        "Z363",  # Encounter for antenatal screening for malformations
        "Z364",  # Encounter for antenatal screening for fetal growth retardation
        "Z365",  # Encounter for antenatal screening for isoimmunization
        "Z3681",  # Encounter for antenatal screening for hydrops fetalis
        "Z3682",  # Encounter for antenatal screening for nuchal translucency
        "Z3683",  # Encounter for fetal screening for congenital cardiac abnormalities
        "Z3684",  # Encounter for antenatal screening for fetal lung maturity
        "Z3685",  # Encounter for antenatal screening for Streptococcus B
        "Z3686",  # Encounter for antenatal screening for cervical length
        "Z3687",  # Encounter for antenatal screening for uncertain dates
        "Z3688",  # Encounter for antenatal screening for fetal macrosomia
        "Z3689",  # Encounter for other specified antenatal screening
        "Z368A",  # Encounter for antenatal screening for other genetic defects
        "Z369",  # Encounter for antenatal screening, unspecified
    }

    SNOMEDCT = {
        "12095001",  # Legal abortion with cardiac arrest AND/OR failure (disorder)
        "127376007",  # Cervical effacement finding (finding)
        "15394000",  # Toxemia of pregnancy (disorder)
        "163403004",  # On examination - vaginal examination - gravid uterus (finding)
        "163498004",  # On examination - gravid uterus size (finding)
        "163499007",  # On examination - fundus 12-16 week size (finding)
        "163500003",  # On examination - fundus 16-20 week size (finding)
        "163501004",  # On examination - fundus 20-24 week size (finding)
        "163502006",  # On examination - fundus 24-28 week size (finding)
        "163504007",  # On examination - fundus 28-32 week size (finding)
        "163505008",  # On examination - fundus 32-34 week size (finding)
        "163506009",  # On examination - fundus 34-36 week size (finding)
        "163507000",  # On examination - fundus 36-38 week size (finding)
        "163508005",  # On examination - fundus 38 weeks-term size (finding)
        "163509002",  # On examination - fundus = term size (finding)
        "163510007",  # On examination - fundal size = dates (finding)
        "163515002",  # On examination - oblique lie (disorder)
        "163516001",  # On examination - transverse lie (disorder)
        "163520002",  # On examination - presenting part position (finding)
        "163521003",  # On examination - presenting part left occipitoanterior (finding)
        "163522005",  # On examination - presenting right occipitoanterior (finding)
        "163524006",  # On examination - presenting part right occipitoposterior (finding)
        "163525007",  # On examination - presenting part left sacroanterior (finding)
        "163526008",  # On examination - presenting part right sacroanterior (finding)
        "163531005",  # On examination - partial engagement - 3/5 (finding)
        "163532003",  # On examination - partial engagement - 2/5 (finding)
        "163533008",  # On examination - presentation engaged-1/5 (finding)
        "163535001",  # On examination - fetal movements (finding)
        "163539007",  # On examination - fetus very active (finding)
        "163540009",  # On examination - fetal movement diminished (finding)
        "163542001",  # On examination - fetal heart heard (finding)
        "163556004",  # On examination - vaginal examination - pelvis adequate (finding)
        "163559006",  # On examination - vaginal examination - cervical ripeness (finding)
        "163561002",  # On examination - vaginal examination - cervix ripe (finding)
        "163563004",  # On examination - vaginal examination - cervical dilatation (finding)
        "163564005",  # On examination - vaginal examination - cervical os closed (finding)
        "163565006",  # On examination - vaginal examination - os 0-1 cm dilated (finding)
        "163566007",  # On examination - vaginal examination - os 1-2cm dilated (finding)
        "163567003",  # On examination - vaginal examination - os 2-4 cm dilated (finding)
        "163568008",  # On examination - vaginal examination - os 4-6cm dilated (finding)
        "163569000",  # On examination - vaginal examination - os 6-8cm dilated (finding)
        "163570004",  # On examination - vaginal examination - os 8-10cm dilated (finding)
        "163571000",  # On examination - vaginal examination - os fully dilated (finding)
        "163574008",  # On examination - vaginal examination - presenting part high (finding)
        "163575009",  # On examination - vaginal examination - presenting part in mid cavity (finding)
        "163576005",  # On examination - vaginal examination - presenting part at spines (finding)
        "163577001",  # On examination - vaginal examination - presenting part visible (finding)
        "169954003",  # Placenta normal O/E (finding)
        "199127006",  # Herpes gestationis - delivered (disorder)
        "199128001",  # Herpes gestationis - delivered with postnatal complication (disorder)
        "199129009",  # Herpes gestationis - not delivered (disorder)
        "199375007",  # High head at term - delivered (finding)
        "199958008",  # Ruptured uterus before labor (disorder)
        "239105004",  # Transplacental herpes gestationis (disorder)
        "24082006",  # Bruising of cord (disorder)
        "267271004",  # Obstetric trauma damaging pelvic joints and ligaments (disorder)
        "26741000",  # Abnormal amnion (disorder)
        "268954007",  # On examination - presenting part station (finding)
        "268955008",  # On examination - vaginal examination - descent of presenting part (finding)
        "271887000",  # On examination - obstetric (finding)
        "274297000",  # On examination - vaginal exam - obstetric (finding)
        "275969005",  # On examination - presenting part engaged (finding)
        "309670007",  # On examination - fetus (finding)
        "312354009",  # On examination - viable fetus (finding)
        "37762002",  # Face OR brow presentation of fetus (disorder)
        "388602007",  # Villitis of unknown etiology (disorder)
        "39804004",  # Abnormal products of conception (disorder)
        "408802005",  # On examination - presentation engaged- 0/5 (finding)
        "44782008",  # Molar pregnancy (disorder)
        "609460008",  # Induced termination of pregnancy complicated by cardiac arrest and/or failure (disorder)
        "69270005",  # Rupture of uterus during AND/OR after labor (disorder)
        "102874004",  # Possible pregnancy (finding)
        "111454005",  # Retained portions of placenta AND/OR membranes without hemorrhage (disorder)
        "127363001",  # Number of pregnancies, currently pregnant (finding)
        "200040005",  # Retained portion of placenta or membranes with no hemorrhage (disorder)
        "200204002",  # Varicose veins of legs in pregnancy and the puerperium (disorder)
        "200208004",  # Varicose veins of legs in pregnancy and the puerperium with antenatal complication (disorder)
        "200217004",  # Varicose veins of perineum and vulva in pregnancy and the puerperium with antenatal complication (disorder)
        "200222004",  # Superficial thrombophlebitis in pregnancy and the puerperium (disorder)
        "200226001",  # Superficial thrombophlebitis in pregnancy and the puerperium with antenatal complication (disorder)
        "200253002",  # Hemorrhoids in pregnancy and the puerperium with antenatal complication (disorder)
        "20286008",  # Gorissenne's sign (finding)
        "20956009",  # Phlebitis complicating pregnancy AND/OR puerperium (disorder)
        "237288003",  # Abnormal weight gain in pregnancy (disorder)
        "237294006",  # Retained placenta and membranes (disorder)
        "237345008",  # Varicose veins - obstetric (disorder)
        "237346009",  # Vulval obstetric varicose veins (disorder)
        "237347000",  # Perineal obstetric varicose veins (disorder)
        "249007008",  # Subpubic arch (finding)
        "267273001",  # Retained placenta or membranes with no hemorrhage (disorder)
        "26850007",  # Phlebitis AND/OR thrombosis complicating pregnancy AND/OR puerperium (disorder)
        "275427000",  # Retained membrane without hemorrhage (disorder)
        "276479009",  # Retained membrane (disorder)
        "289771002",  # Cervix paper thin (finding)
        "308133005",  # Varicose veins of legs in pregnancy (disorder)
        "308134004",  # Varicose veins of legs in the puerperium (disorder)
        "37139009",  # Varicose veins of vulva AND/OR perineum complicating pregnancy AND/OR puerperium (disorder)
        "44772007",  # Maternal obesity syndrome (disorder)
        "44918005",  # Varicose veins complicating pregnancy AND/OR puerperium (disorder)
        "47780009",  # Superficial thrombophlebitis complicating pregnancy AND/OR puerperium (disorder)
        "63040001",  # Hemorrhoids complicating pregnancy AND/OR puerperium (disorder)
        "63795001",  # Thrombosis of intracranial venous sinus of pregnancy AND/OR puerperium (disorder)
        "66231000",  # Fetal OR intrauterine asphyxia, not clear if noted before OR after onset of labor in liveborn infant (disorder)
        "67480003",  # Indication for care AND/OR intervention in labor AND/OR delivery (disorder)
        "682004",  # Thrombosis complicating pregnancy AND/OR puerperium (disorder)
        "71848002",  # Bolt's sign (finding)
        "84216001",  # Cerebral venous thrombosis of pregnancy AND/OR puerperium (disorder)
        "10835571000119102",  # Antepartum hemorrhage due to disseminated intravascular coagulation (disorder)
        "38534008",  # Aspiration of stomach contents after anesthesia AND/OR sedation in labor AND/OR delivery (disorder)
        "1010443001",  # Legal abortion complicated by cardiac arrest (disorder)
        "1010444007",  # Legal abortion complicated by cardiac failure (disorder)
        "1010446009",  # Illegal abortion complicated by cardiac arrest (disorder)
        "1010447000",  # Illegal abortion complicated by cardiac failure (disorder)
        "106007006",  # Maternal AND/OR fetal condition affecting labor AND/OR delivery (disorder)
        "111431001",  # Legal abortion complicated by damage to pelvic organs AND/OR tissues (disorder)
        "111432008",  # Legal abortion with air embolism (disorder)
        "11373009",  # Illegal abortion with sepsis (disorder)
        "12296009",  # Legal abortion with intravascular hemolysis (disorder)
        "13100007",  # Legal abortion with laceration of uterus (disorder)
        "15511008",  # Legal abortion with parametritis (disorder)
        "17335003",  # Illegal abortion with laceration of bowel (disorder)
        "18684002",  # Illegal abortion complicated by metabolic disorder (disorder)
        "18872002",  # Illegal abortion with laceration of cervix (disorder)
        "18894005",  # Legal abortion with cerebral anoxia (disorder)
        "195005",  # Illegal abortion with endometritis (disorder)
        "19564003",  # Illegal abortion with laceration of periurethral tissue (disorder)
        "198705001",  # Incomplete legal abortion with genital tract or pelvic infection (disorder)
        "198706000",  # Incomplete legal abortion with delayed or excessive hemorrhage (disorder)
        "198707009",  # Incomplete legal abortion with damage to pelvic organs or tissues (disorder)
        "198708004",  # Incomplete legal abortion with renal failure (disorder)
        "198709007",  # Incomplete legal abortion with metabolic disorder (disorder)
        "198710002",  # Incomplete legal abortion with shock (disorder)
        "198711003",  # Incomplete legal abortion with embolism (disorder)
        "198718009",  # Complete legal abortion with genital tract or pelvic infection (disorder)
        "198719001",  # Complete legal abortion with delayed or excessive hemorrhage (disorder)
        "198720007",  # Complete legal abortion with damage to pelvic organs or tissues (disorder)
        "198721006",  # Complete legal abortion with renal failure (disorder)
        "198722004",  # Complete legal abortion with metabolic disorder (disorder)
        "198723009",  # Complete legal abortion with shock (disorder)
        "198724003",  # Complete legal abortion with embolism (disorder)
        "198743003",  # Illegal abortion incomplete (disorder)
        "198744009",  # Incomplete illegal abortion with genital tract or pelvic infection (disorder)
        "198745005",  # Incomplete illegal abortion with delayed or excessive hemorrhage (disorder)
        "198746006",  # Incomplete illegal abortion with damage to pelvic organs or tissues (disorder)
        "198747002",  # Incomplete illegal abortion with renal failure (disorder)
        "198748007",  # Incomplete illegal abortion with metabolic disorder (disorder)
        "198749004",  # Incomplete illegal abortion with shock (disorder)
        "198750004",  # Incomplete illegal abortion with embolism (disorder)
        "198755009",  # Illegal abortion complete (disorder)
        "198757001",  # Complete illegal abortion with delayed or excessive hemorrhage (disorder)
        "198758006",  # Complete illegal abortion with damage to pelvic organs or tissues (disorder)
        "198759003",  # Complete illegal abortion with renal failure (disorder)
        "198760008",  # Complete illegal abortion with metabolic disorder (disorder)
        "198761007",  # Complete illegal abortion with shock (disorder)
        "198762000",  # Complete illegal abortion with embolism (disorder)
        "198861000",  # Readmission for retained products of conception, spontaneous abortion (disorder)
        "198862007",  # Readmission for retained products of conception, legal abortion (disorder)
        "198863002",  # Readmission for retained products of conception, illegal abortion (disorder)
        "199710008",  # Sepsis during labor, delivered (disorder)
        "20216003",  # Legal abortion with perforation of uterus (disorder)
        "20483002",  # Legal abortion with renal tubular necrosis (disorder)
        "206070006",  # Fetal or neonatal effect of placental damage caused by cesarean section (disorder)
        "206121009",  # Fetal or neonatal effect of forceps delivery (disorder)
        "206122002",  # Fetal or neonatal effect of vacuum extraction delivery (disorder)
        "206123007",  # Fetal or neonatal effect of cesarean section (disorder)
        "206149002",  # Fetal or neonatal effect of instrumental delivery (disorder)
        "21280005",  # Legal abortion with postoperative shock (disorder)
        "21604008",  # Illegal abortion with blood-clot embolism (disorder)
        "22046002",  # Illegal abortion with salpingitis (disorder)
        "23401002",  # Illegal abortion with perforation of bowel (disorder)
        "25404008",  # Illegal abortion with electrolyte imbalance (disorder)
        "25519006",  # Illegal abortion with perforation of vagina (disorder)
        "25691001",  # Legal abortion with septic shock (disorder)
        "26010008",  # Legal abortion with pelvic peritonitis (disorder)
        "267193004",  # Incomplete legal abortion (disorder)
        "26743002",  # Illegal abortion complicated by embolism (disorder)
        "268865003",  # Had short umbilical cord (disorder)
        "27015006",  # Legal abortion with amniotic fluid embolism (disorder)
        "27169005",  # Legal abortion complicated by shock (disorder)
        "289278007",  # Finding of measures of placenta (finding)
        "289769002",  # Cervix thinning (finding)
        "29583006",  # Illegal abortion with laceration of broad ligament (disorder)
        "29950007",  # Legal abortion with septic embolism (disorder)
        "29995000",  # Illegal abortion with fat embolism (disorder)
        "30479005",  # Legal abortion with afibrinogenemia (disorder)
        "31207002",  # Self-induced abortion (disorder)
        "3230006",  # Illegal abortion with afibrinogenemia (disorder)
        "33046001",  # Illegal abortion with urinary tract infection (disorder)
        "33561005",  # Illegal abortion with acute renal failure (disorder)
        "34500003",  # Legal abortion without complication (disorder)
        "34701001",  # Illegal abortion with postoperative shock (disorder)
        "3634007",  # Legal abortion complicated by metabolic disorder (disorder)
        "36473002",  # Legal abortion with oliguria (disorder)
        "373901007",  # Legal abortion complicated by genital-pelvic infection (disorder)
        "373902000",  # Illegal abortion complicated by genital-pelvic infection (disorder)
        "37787009",  # Legal abortion with blood-clot embolism (disorder)
        "38784004",  # Illegal abortion complicated by damage to pelvic organs AND/OR tissues (disorder)
        "38905006",  # Illegal abortion with amniotic fluid embolism (disorder)
        "39239006",  # Legal abortion with electrolyte imbalance (disorder)
        "39246002",  # Legal abortion with sepsis (disorder)
        "40405003",  # Illegal abortion with cerebral anoxia (disorder)
        "40500006",  # Illegal abortion with laceration of uterus (disorder)
        "405256006",  # Parturient paresis (disorder)
        "405736009",  # Accidental antepartum hemorrhage (disorder)
        "41806003",  # Legal abortion with laceration of broad ligament (disorder)
        "44101005",  # Illegal abortion with air embolism (disorder)
        "4451004",  # Illegal abortion with renal tubular necrosis (disorder)
        "4576001",  # Legal abortion complicated by renal failure (disorder)
        "48888007",  # Placenta previa found before labor AND delivery by cesarean section without hemorrhage (disorder)
        "4910006",  # Premature abnormal placenta (disorder)
        "49632008",  # Illegally induced abortion (disorder)
        "51096002",  # Legal abortion with pulmonary embolism (disorder)
        "51707007",  # Legal abortion with salpingo-oophoritis (disorder)
        "51953009",  # Legal abortion with perforation of broad ligament (disorder)
        "52342006",  # Legal abortion with acute renal failure (disorder)
        "53098006",  # Legal abortion with salpingitis (disorder)
        "53247006",  # Illegal abortion with laceration of bladder (disorder)
        "54044001",  # Legal abortion with laceration of vagina (disorder)
        "54155004",  # Illegal abortion with uremia (disorder)
        "55589000",  # Illegal abortion with pulmonary embolism (disorder)
        "5577003",  # Legal abortion with laceration of bowel (disorder)
        "55933000",  # Legal abortion with perforation of bowel (disorder)
        "57734001",  # Legal abortion complicated by embolism (disorder)
        "5945005",  # Legal abortion with urinary tract infection (disorder)
        "59795007",  # Short cord (disorder)
        "59859005",  # Legal abortion with laceration of bladder (disorder)
        "609446006",  # Induced termination of pregnancy with complication (disorder)
        "609485004",  # Induced termination of pregnancy complicated by sepsis (disorder)
        "6134000",  # Illegal abortion with oliguria (disorder)
        "61752008",  # Illegal abortion complicated by shock (disorder)
        "61810006",  # Illegal abortion with defibrination syndrome (disorder)
        "62888008",  # Legal abortion with perforation of bladder (disorder)
        "6647006",  # Legal abortion with defibrination syndrome (disorder)
        "6802007",  # Illegal abortion with perforation of broad ligament (disorder)
        "68189005",  # Illegal abortion with perforation of periurethral tissue (disorder)
        "698586005",  # Complete legal abortion complicated by excessive hemorrhage (disorder)
        "698587001",  # Complete illegal abortion complicated by excessive hemorrhage (disorder)
        "698632006",  # Pregnancy induced edema (disorder)
        "70823006",  # Illegal abortion with intravascular hemolysis (disorder)
        "71216006",  # Legal abortion with laceration of periurethral tissue (disorder)
        "71362000",  # Illegal abortion with septic embolism (disorder)
        "7166002",  # Legal abortion with laceration of cervix (disorder)
        "724484007",  # Incomplete legal abortion without complication (disorder)
        "73280003",  # Illegal abortion with laceration of vagina (disorder)
        "74978008",  # Illegal abortion with complication (disorder)
        "75013000",  # Legal abortion with perforation of vagina (disorder)
        "75825001",  # Legal abortion with fat embolism (disorder)
        "75947000",  # Legal abortion with endometritis (disorder)
        "76472002",  # Legal abortion with perforation of periurethral tissue (disorder)
        "77099001",  # Illegal abortion with perforation of uterus (disorder)
        "77206006",  # Puerperal pelvic sepsis (disorder)
        "77913004",  # Submammary mastitis associated with childbirth (disorder)
        "7802000",  # Illegal abortion without complication (disorder)
        "785869007",  # Secondary hemorrhage due to and following illegally induced termination of pregnancy (disorder)
        "785870008",  # Secondary hemorrhage due to and following legally induced termination of pregnancy (disorder)
        "785871007",  # Excessive hemorrhage due to and following legally induced termination of pregnancy (disorder)
        "785872000",  # Excessive hemorrhage due to and following illegally induced termination of pregnancy (disorder)
        "80438008",  # Illegal abortion with perforation of bladder (disorder)
        "80847009",  # Tubal-cornual placenta (disorder)
        "82204006",  # Illegal abortion with parametritis (disorder)
        "82338001",  # Illegal abortion with septic shock (disorder)
        "84143004",  # Illegal abortion with perforation of cervix (disorder)
        "84235001",  # Cephalic version (disorder)
        "8468007",  # Legal abortion with uremia (disorder)
        "85652000",  # Legal abortion with perforation of cervix (disorder)
        "8670007",  # Illegal abortion with salpingo-oophoritis (disorder)
        "8996006",  # Illegal abortion complicated by renal failure (disorder)
        "90450000",  # Illegal abortion with pelvic peritonitis (disorder)
        "10749691000119103",  # Obstetric anesthesia with cardiac complication in childbirth (disorder)
        "10755161000119102",  # Drug dependence in childbirth (disorder)
        "118182003",  # Finding related to amniotic fluid turnover (finding)
        "161000119101",  # Placental abnormality, antepartum (finding)
        "168097000",  # Amniotic fluid cell content OK (finding)
        "169957005",  # Placental abnormality (finding)
        "199252002",  # Drug dependence during pregnancy - baby delivered (disorder)
        "199253007",  # Drug dependence in the puerperium - baby delivered (disorder)
        "199254001",  # Drug dependence during pregnancy - baby not yet delivered (disorder)
        "199255000",  # Drug dependence in the puerperium - baby delivered during previous episode of care (disorder)
        "199461004",  # Uterine scar from previous surgery in pregnancy, childbirth and the puerperium (disorder)
        "199464007",  # Uterine scar from previous surgery in pregnancy, childbirth and the puerperium with antenatal problem (disorder)
        "199469002",  # Retroverted incarcerated gravid uterus - delivered with postnatal complication (disorder)
        "199470001",  # Retroverted incarcerated gravid uterus with antenatal problem (disorder)
        "199471002",  # Retroverted incarcerated gravid uterus with postnatal complication (disorder)
        "199711007",  # Sepsis during labor with antenatal problem (disorder)
        "199969006",  # Obstetric inversion of uterus - delivered with postnatal problem (disorder)
        "199970007",  # Obstetric inversion of uterus with postnatal problem (disorder)
        "200106008",  # Obstetric shock - delivered with postnatal problem (disorder)
        "200107004",  # Obstetric shock with antenatal problem (disorder)
        "200108009",  # Obstetric shock with postnatal problem (disorder)
        "200146002",  # Cesarean delivery - delivered (finding)
        "200351002",  # Placental polyp - delivered with postnatal complication (disorder)
        "21504004",  # Pressure collapse of lung after anesthesia AND/OR sedation in labor AND/OR delivery (disorder)
        "237228001",  # Pregnancy and drug dependence (disorder)
        "237353000",  # Obstetric disorders of breast and lactation (disorder)
        "249124004",  # Condition of amniotic membranes (finding)
        "249147003",  # State of upper segment retraction during labor (finding)
        "249175001",  # Placental vessel finding (finding)
        "267206008",  # Drug dependence during pregnancy, childbirth and the puerperium (disorder)
        "267335003",  # Fetoplacental problems (disorder)
        "271403007",  # Placenta infarcted (finding)
        "28608002",  # Choriovitelline placenta (disorder)
        "289258004",  # Finding of pattern of delivery (finding)
        "289263000",  # Large placenta (finding)
        "289264006",  # Small placenta (finding)
        "289265007",  # Placenta normal size (finding)
        "289281002",  # Placenta problem (finding)
        "289292001",  # Reduced amniotic fluid (finding)
        "289298002",  # Amniotic fluid normal (finding)
        "289745001",  # Excessive retraction of upper segment (finding)
        "289746000",  # Normal retraction of upper segment (finding)
        "34150001",  # Drug dependence in mother complicating pregnancy, childbirth AND/OR puerperium (disorder)
        "366330003",  # Finding of size of placenta (finding)
        "408780005",  # Finding of color of amniotic fluid (finding)
        "54449002",  # Forced uterine inversion (disorder)
        "65539006",  # Impetigo herpetiformis (disorder)
        "88697005",  # Papular dermatitis of pregnancy (disorder)
        "111425004",  # Genital tract AND/OR pelvic infection following molar AND/OR ectopic pregnancy (disorder)
        "112071002",  # Postmature pregnancy delivered (finding)
        "11669000",  # Postoperative shock following molar AND/OR ectopic pregnancy (disorder)
        "1220654002",  # Complete induced termination of pregnancy complicated by delayed or excessive hemorrhage (disorder)
        "16083003",  # Damage to pelvic organs AND/OR tissues following molar AND/OR ectopic pregnancy (disorder)
        "172001",  # Endometritis following molar AND/OR ectopic pregnancy (disorder)
        "18909006",  # Fetal or neonatal effect of transverse lie (disorder)
        "198644001",  # Incomplete miscarriage with genital tract or pelvic infection (disorder)
        "198645000",  # Incomplete miscarriage with delayed or excessive hemorrhage (disorder)
        "198646004",  # Incomplete miscarriage with damage to pelvic organs or tissues (disorder)
        "198899007",  # Placenta previa without hemorrhage - delivered (disorder)
        "198900002",  # Placenta previa without hemorrhage - not delivered (disorder)
        "198905007",  # Placenta previa with hemorrhage - delivered (disorder)
        "198906008",  # Placenta previa with hemorrhage - not delivered (disorder)
        "198910006",  # Placental abruption - delivered (disorder)
        "198911005",  # Placental abruption - not delivered (disorder)
        "198917009",  # Antepartum hemorrhage with coagulation defect - delivered (disorder)
        "198918004",  # Antepartum hemorrhage with coagulation defect - not delivered (disorder)
        "198922009",  # Antepartum hemorrhage with trauma - delivered (disorder)
        "198923004",  # Antepartum hemorrhage with trauma - not delivered (disorder)
        "198927003",  # Antepartum hemorrhage with uterine leiomyoma - delivered (disorder)
        "198928008",  # Antepartum hemorrhage with uterine leiomyoma - not delivered (disorder)
        "198944004",  # Benign essential hypertension complicating pregnancy, childbirth and the puerperium - delivered (disorder)
        "198945003",  # Benign essential hypertension complicating pregnancy, childbirth and the puerperium - delivered with postnatal complication (disorder)
        "198946002",  # Benign essential hypertension complicating pregnancy, childbirth and the puerperium - not delivered (disorder)
        "198951008",  # Renal hypertension complicating pregnancy, childbirth and the puerperium - delivered (disorder)
        "198952001",  # Renal hypertension complicating pregnancy, childbirth and the puerperium - delivered with postnatal complication (disorder)
        "198953006",  # Renal hypertension complicating pregnancy, childbirth and the puerperium - not delivered (disorder)
        "198965005",  # Transient hypertension of pregnancy - delivered (disorder)
        "198966006",  # Transient hypertension of pregnancy - delivered with postnatal complication (disorder)
        "198967002",  # Transient hypertension of pregnancy - not delivered (disorder)
        "198983002",  # Severe pre-eclampsia - delivered (disorder)
        "198984008",  # Severe pre-eclampsia - delivered with postnatal complication (disorder)
        "198985009",  # Severe pre-eclampsia - not delivered (disorder)
        "198990007",  # Eclampsia - delivered (disorder)
        "198991006",  # Eclampsia - delivered with postnatal complication (disorder)
        "198999008",  # Pre-eclampsia or eclampsia with pre-existing hypertension - delivered (disorder)
        "199000005",  # Pre-eclampsia or eclampsia with pre-existing hypertension - delivered with postnatal complication (disorder)
        "199002002",  # Pre-eclampsia or eclampsia with pre-existing hypertension - not delivered (disorder)
        "199022003",  # Mild hyperemesis-delivered (disorder)
        "199023008",  # Mild hyperemesis-not delivered (disorder)
        "199027009",  # Hyperemesis gravidarum with metabolic disturbance - delivered (disorder)
        "199028004",  # Hyperemesis gravidarum with metabolic disturbance - not delivered (disorder)
        "199049003",  # Threatened premature labor - not delivered (finding)
        "199063009",  # Post-term pregnancy - delivered (disorder)
        "199064003",  # Post-term pregnancy - not delivered (disorder)
        "199087006",  # Habitual aborter - delivered (finding)
        "199088001",  # Habitual aborter - not delivered (finding)
        "199093003",  # Peripheral neuritis in pregnancy - delivered (disorder)
        "199095005",  # Peripheral neuritis in pregnancy - not delivered (disorder)
        "199099004",  # Asymptomatic bacteriuria in pregnancy - delivered (disorder)
        "199100007",  # Asymptomatic bacteriuria in pregnancy - delivered with postnatal complication (disorder)
        "199101006",  # Asymptomatic bacteriuria in pregnancy - not delivered (disorder)
        "199106001",  # Genitourinary tract infection in pregnancy - delivered (disorder)
        "199107005",  # Genitourinary tract infection in pregnancy - delivered with postnatal complication (disorder)
        "199108000",  # Genitourinary tract infection in pregnancy - not delivered (disorder)
        "199117000",  # Liver disorder in pregnancy - delivered (disorder)
        "199118005",  # Liver disorder in pregnancy - not delivered (disorder)
        "199121007",  # Fatigue during pregnancy - delivered (disorder)
        "199123005",  # Fatigue during pregnancy - not delivered (disorder)
        "199134008",  # Glycosuria during pregnancy - delivered (disorder)
        "199135009",  # Glycosuria during pregnancy - delivered with postnatal complication (disorder)
        "199136005",  # Glycosuria during pregnancy - not delivered (disorder)
        "199156006",  # Maternal syphilis during pregnancy - baby delivered (disorder)
        "199157002",  # Maternal syphilis in the puerperium - baby delivered during current episode of care (disorder)
        "199158007",  # Maternal syphilis during pregnancy - baby not yet delivered (disorder)
        "199159004",  # Maternal syphilis in the puerperium - baby delivered during previous episode of care (disorder)
        "199163006",  # Maternal gonorrhea during pregnancy - baby delivered (disorder)
        "199164000",  # Maternal gonorrhea in the puerperium - baby delivered during current episode of care (disorder)
        "199165004",  # Maternal gonorrhea during pregnancy - baby not yet delivered (disorder)
        "199166003",  # Maternal gonorrhea in the puerperium - baby delivered during previous episode of care (disorder)
        "199178004",  # Tuberculosis in pregnancy, childbirth and the puerperium - delivered (disorder)
        "199179007",  # Maternal tuberculosis in the puerperium - baby delivered during current episode of care (disorder)
        "199180005",  # Maternal tuberculosis during pregnancy - baby not yet delivered (disorder)
        "199181009",  # Maternal tuberculosis in the puerperium - baby delivered during previous episode of care (disorder)
        "199185000",  # Maternal malaria during pregnancy - baby delivered (disorder)
        "199186004",  # Maternal malaria in the puerperium - baby delivered during current episode of care (disorder)
        "199187008",  # Maternal malaria during pregnancy - baby not yet delivered (disorder)
        "199188003",  # Maternal malaria in the puerperium - baby delivered during previous episode of care (disorder)
        "199192005",  # Maternal rubella during pregnancy - baby delivered (disorder)
        "199193000",  # Maternal rubella in the puerperium - baby delivered during current episode of care (disorder)
        "199194006",  # Maternal rubella during pregnancy - baby not yet delivered (disorder)
        "199195007",  # Maternal rubella in the puerperium - baby delivered during previous episode of care (disorder)
        "199225007",  # Diabetes mellitus during pregnancy - baby delivered (disorder)
        "199227004",  # Diabetes mellitus during pregnancy - baby not yet delivered (disorder)
        "199244000",  # Anemia during pregnancy - baby delivered (disorder)
        "199245004",  # Anemia in the puerperium - baby delivered during current episode of care (disorder)
        "199246003",  # Anemia during pregnancy - baby not yet delivered (disorder)
        "199247007",  # Anemia in the puerperium - baby delivered during previous episode of care (disorder)
        "199259006",  # Mental disorder during pregnancy - baby delivered (disorder)
        "199260001",  # Mental disorder in the puerperium - baby delivered (disorder)
        "199261002",  # Mental disorder during pregnancy - baby not yet delivered (disorder)
        "199262009",  # Mental disorder in the puerperium - baby delivered during previous episode of care (disorder)
        "199266007",  # Congenital cardiovascular disorder during pregnancy - baby delivered (disorder)
        "199267003",  # Congenital cardiovascular disorder in the puerperium - baby delivered during current episode of care (disorder)
        "199268008",  # Congenital cardiovascular disorder during pregnancy - baby not yet delivered (disorder)
        "199269000",  # Congenital cardiovascular disorder in the puerperium - baby delivered during previous episode of care (disorder)
        "199281001",  # Orthopedic disorder during pregnancy - baby delivered (disorder)
        "199282008",  # Orthopedic disorder in the puerperium - baby delivered during current episode of care (disorder)
        "199283003",  # Orthopedic disorder during pregnancy - baby not yet delivered (disorder)
        "199284009",  # Orthopedic disorder in the puerperium - baby delivered during previous episode of care (disorder)
        "199317008",  # Twin pregnancy - delivered (finding)
        "199321001",  # Triplet pregnancy - delivered (finding)
        "199325005",  # Quadruplet pregnancy - delivered (finding)
        "199344003",  # Unstable lie - delivered (disorder)
        "199358001",  # Oblique lie - delivered (disorder)
        "199362007",  # Transverse lie - delivered (disorder)
        "199380003",  # Multiple pregnancy with malpresentation - delivered (disorder)
        "199384007",  # Prolapsed arm - delivered (disorder)
        "199405005",  # Generally contracted pelvis - delivered (disorder)
        "199409004",  # Inlet pelvic contraction - delivered (disorder)
        "199413006",  # Outlet pelvic contraction - delivered (disorder)
        "199418002",  # Mixed feto-pelvic disproportion - delivered (disorder)
        "199422007",  # Large fetus causing disproportion - delivered (disorder)
        "199427001",  # Hydrocephalic disproportion - delivered (disorder)
        "199463001",  # Uterine scar from previous surgery in pregnancy, childbirth and the puerperium - delivered (disorder)
        "199468005",  # Retroverted incarcerated gravid uterus - delivered (disorder)
        "199482005",  # Cervical incompetence - delivered (disorder)
        "199483000",  # Cervical incompetence - delivered with postnatal complication (disorder)
        "199511005",  # Pelvic soft tissue abnormality in pregnancy, childbirth and the puerperium - delivered (disorder)
        "199512003",  # Pelvic soft tissue abnormality in pregnancy, childbirth and the puerperium - delivered with postnatal complication (disorder)
        "199577000",  # Fetal-maternal hemorrhage - delivered (disorder)
        "199582007",  # Rhesus isoimmunization - delivered (disorder)
        "199646006",  # Polyhydramnios - delivered (disorder)
        "199653002",  # Oligohydramnios - delivered (disorder)
        "199658006",  # Premature rupture of membranes - delivered (disorder)
        "199672002",  # Prolonged artificial rupture of membranes - delivered (disorder)
        "199677008",  # Amniotic cavity infection - delivered (disorder)
        "199694005",  # Failed mechanical induction - delivered (disorder)
        "199714004",  # Grand multiparity - delivered (finding)
        "199718001",  # Elderly primigravida - delivered (finding)
        "199749006",  # Obstructed labor due to fetal malposition - delivered (disorder)
        "199759007",  # Obstructed labor caused by bony pelvis - delivered (disorder)
        "199769001",  # Obstructed labor caused by pelvic soft tissues - delivered (disorder)
        "199774009",  # Deep transverse arrest - delivered (disorder)
        "199783004",  # Shoulder dystocia - delivered (disorder)
        "199787003",  # Locked twins - delivered (finding)
        "199788008",  # Locked twins with antenatal problem (finding)
        "199819004",  # Primary uterine inertia - delivered (disorder)
        "199824001",  # Secondary uterine inertia - delivered (disorder)
        "199838008",  # Hypertonic uterine inertia - delivered (disorder)
        "199847000",  # Prolonged first stage - delivered (disorder)
        "199857004",  # Prolonged second stage - delivered (disorder)
        "199862003",  # Delayed delivery second twin - delivered (disorder)
        "199869007",  # Prolapse of cord - delivered (disorder)
        "199874004",  # Umbilical cord tight around neck - delivered (disorder)
        "199880007",  # Cord tangled with compression - delivered (disorder)
        "199889008",  # Short cord - delivered (disorder)
        "199895009",  # Vasa previa - delivered (disorder)
        "199916005",  # First degree perineal tear during delivery - delivered (disorder)
        "199925004",  # Second degree perineal tear during delivery - delivered (disorder)
        "199930000",  # Third degree perineal tear during delivery - delivered (disorder)
        "199934009",  # Fourth degree perineal tear during delivery - delivered (disorder)
        "199960005",  # Rupture of uterus before labor - delivered (disorder)
        "199965000",  # Rupture of uterus during and after labor - delivered with postnatal problem (disorder)
        "199974003",  # Obstetric laceration of cervix - delivered (disorder)
        "199979008",  # Obstetric high vaginal laceration - delivered (disorder)
        "199990003",  # Obstetric damage to pelvic joints and ligaments - delivered (disorder)
        "199995008",  # Obstetric pelvic hematoma - delivered (disorder)
        "200025008",  # Secondary postpartum hemorrhage - delivered with postnatal problem (disorder)
        "200030007",  # Postpartum coagulation defects - delivered with postnatal problem (disorder)
        "200049006",  # Obstetric anesthesia with pulmonary complications - delivered (disorder)
        "200050006",  # Obstetric anesthesia with pulmonary complications - delivered with postnatal problem (disorder)
        "200056000",  # Obstetric anesthesia with cardiac complications - delivered (disorder)
        "200057009",  # Obstetric anesthesia with cardiac complications - delivered with postnatal problem (disorder)
        "200063000",  # Obstetric anesthesia with central nervous system complications - delivered (disorder)
        "200064006",  # Obstetric anesthesia with central nervous system complication - delivered with postnatal problem (disorder)
        "200099005",  # Maternal distress - delivered (disorder)
        "200100002",  # Maternal distress - delivered with postnatal problem (disorder)
        "200105007",  # Obstetric shock - delivered (disorder)
        "200111005",  # Maternal hypotension syndrome - delivered (disorder)
        "200112003",  # Maternal hypotension syndrome - delivered with postnatal problem (disorder)
        "200181000",  # Puerperal endometritis - delivered with postnatal complication (disorder)
        "200185009",  # Puerperal salpingitis - delivered with postnatal complication (disorder)
        "200190007",  # Puerperal peritonitis - delivered with postnatal complication (disorder)
        "200280009",  # Puerperal pyrexia of unknown origin - delivered with postnatal complication (disorder)
        "200288002",  # Obstetric air pulmonary embolism - delivered (disorder)
        "200294005",  # Amniotic fluid pulmonary embolism - delivered (disorder)
        "200301007",  # Obstetric blood-clot pulmonary embolism - delivered (disorder)
        "200308001",  # Obstetric pyemic and septic pulmonary embolism - delivered (disorder)
        "200331001",  # Puerperal cerebrovascular disorder - delivered with postnatal complication (disorder)
        "200342005",  # Obstetric perineal wound disruption - delivered with postnatal complication (disorder)
        "200367008",  # Obstetric nipple infection - delivered (disorder)
        "200368003",  # Obstetric nipple infection - delivered with postnatal complication (disorder)
        "200374003",  # Obstetric breast abscess - delivered (disorder)
        "200375002",  # Obstetric breast abscess - delivered with postnatal complication (disorder)
        "200381005",  # Obstetric non-purulent mastitis - delivered (disorder)
        "200382003",  # Obstetric non-purulent mastitis - delivered with postnatal complication (disorder)
        "200446009",  # Galactorrhea in pregnancy and the puerperium - delivered (disorder)
        "200447000",  # Galactorrhea in pregnancy and the puerperium - delivered with postnatal complication (disorder)
        "20084001",  # Intravascular hemolysis following molar AND/OR ectopic pregnancy (disorder)
        "206057002",  # Fetal or neonatal effect of transverse lie before labor (disorder)
        "206112008",  # Fetal or neonatal effect of maternal bony pelvis abnormality during labor and delivery (disorder)
        "206113003",  # Fetal or neonatal effect of maternal contracted pelvis during labor and delivery (disorder)
        "206114009",  # Fetal or neonatal effect of persistent occipito-posterior malposition during labor and delivery (disorder)
        "206115005",  # Fetal or neonatal effect of shoulder presentation during labor and delivery (disorder)
        "206116006",  # Fetal or neonatal effect of transverse lie during labor and delivery (disorder)
        "206117002",  # Fetal or neonatal effect of face presentation during labor and delivery (disorder)
        "206118007",  # Fetal or neonatal effect of disproportion during labor and delivery (disorder)
        "206134005",  # Fetal or neonatal effect of precipitate delivery (disorder)
        "206137003",  # Fetal or neonatal effect of maternal contraction ring (disorder)
        "206138008",  # Fetal or neonatal effect of hypertonic labor (disorder)
        "206139000",  # Fetal or neonatal effect of hypertonic uterine dysfunction (disorder)
        "206146009",  # Fetal or neonatal effect of induction of labor (disorder)
        "206148005",  # Fetal or neonatal effect of long labor (disorder)
        "20949003",  # Impacted twins (finding)
        "237188007",  # Stenosis of cervix complicating postnatal care - baby delivered during previous episode of care (disorder)
        "237189004",  # Stenosis of cervix complicating antenatal care - baby not yet delivered (disorder)
        "237190008",  # Stenosis of cervix - baby delivered with postpartum complication (disorder)
        "237198001",  # Rectocele complicating postpartum care - baby delivered during previous episode of care (disorder)
        "237201006",  # Rectocele complicating antenatal care - baby not yet delivered (disorder)
        "237202004",  # Rectocele - delivered with postpartum complication (disorder)
        "237206001",  # Rectocele - baby delivered (disorder)
        "237219002",  # Bicornuate uterus complicating postpartum care - baby delivered during previous episode of care (disorder)
        "237220008",  # Bicornuate uterus complicating antenatal care, baby not yet delivered (disorder)
        "237221007",  # Bicornuate uterus - baby delivered with postpartum complication (disorder)
        "237223005",  # Bicornuate uterus - baby delivered (disorder)
        "24051007",  # Septic shock following molar AND/OR ectopic pregnancy (disorder)
        "26235008",  # Renal tubular necrosis following molar AND/OR ectopic pregnancy (disorder)
        "267194005",  # Complete legal abortion (disorder)
        "267214002",  # Congenital abnormality of uterus - baby delivered (disorder)
        "267215001",  # Congenital abnormality of uterus - baby delivered with postpartum complication (disorder)
        "267216000",  # Congenital abnormality of uterus complicating antenatal care, baby not yet delivered (disorder)
        "267217009",  # Congenital abnormality of uterus complicating postpartum care - baby delivered during previous episode of care (disorder)
        "267266006",  # Cord tangled or knotted with compression (disorder)
        "268809007",  # Fetal or neonatal effect of malposition or disproportion during labor or delivery (disorder)
        "268811003",  # Fetal or neonatal effect of abnormal uterine contractions (disorder)
        "268812005",  # Fetal or neonatal effect of uterine inertia or dysfunction (disorder)
        "268813000",  # Fetal or neonatal effect of destructive operation to aid delivery (disorder)
        "27552001",  # Damage to bladder following molar AND/OR ectopic pregnancy (disorder)
        "28196006",  # Uremia following molar AND/OR ectopic pregnancy (disorder)
        "288259000",  # Born after multiple pregnancy (disorder)
        "2989004",  # Complication following molar AND/OR ectopic pregnancy (disorder)
        "29908007",  # Oliguria following molar AND/OR ectopic pregnancy (disorder)
        "30243004",  # Cerebral anoxia following molar AND/OR ectopic pregnancy (disorder)
        "35208003",  # Metabolic disorder following molar AND/OR ectopic pregnancy (disorder)
        "38176009",  # Urinary tract infection following molar AND/OR ectopic pregnancy (disorder)
        "39406005",  # Legally induced abortion (disorder)
        "413339006",  # Failed trial of labor - delivered (disorder)
        "413340008",  # Fetal or neonatal effect of diethylstilbestrol transmitted via placenta and/or breast milk (disorder)
        "42294002",  # Damage to cervix following molar AND/OR ectopic pregnancy (disorder)
        "42536002",  # Damage to periurethral tissue following molar AND/OR ectopic pregnancy (disorder)
        "43629001",  # Renal failure following molar AND/OR ectopic pregnancy (disorder)
        "44017002",  # Damage to bowel following molar AND/OR ectopic pregnancy (disorder)
        "46848006",  # Sepsis following molar AND/OR ectopic pregnancy (disorder)
        "471531000124103",  # At risk of miscarriage (finding)
        "47267007",  # Fetal or neonatal effect of destructive operation on live fetus to facilitate delivery (disorder)
        "4787007",  # Fetal or neonatal effect of breech delivery and extraction (disorder)
        "48433002",  # Legal abortion with complication (disorder)
        "49550006",  # Premature pregnancy delivered (finding)
        "50258003",  # Fetal or neonatal effect of hypotonic uterine dysfunction (disorder)
        "57468008",  # Salpingo-oophoritis following molar AND/OR ectopic pregnancy (disorder)
        "5984000",  # Fetal or neonatal effect of malpresentation, malposition and/or disproportion during labor and/or delivery (disorder)
        "609441001",  # Fetal or neonatal effect of breech delivery (disorder)
        "62698000",  # Defibrination syndrome following molar AND/OR ectopic pregnancy (disorder)
        "6509007",  # Electrolyte imbalance following molar AND/OR ectopic pregnancy (disorder)
        "69344006",  # Shock following molar AND/OR ectopic pregnancy (disorder)
        "698414000",  # Fetal or neonatal effect of complication of labor (disorder)
        "698415004",  # Fetal or neonatal effect of complication of delivery (disorder)
        "698497008",  # Fetal or neonatal effect of disproportion during labor (disorder)
        "698498003",  # Fetal or neonatal effect of disproportion during delivery (disorder)
        "698554000",  # Fetal or neonatal effect of malposition during labor (disorder)
        "698555004",  # Fetal or neonatal effect of malposition during delivery (disorder)
        "698582007",  # Fetal or neonatal effect of diethylstilbestrol transmitted via placenta (disorder)
        "698583002",  # Fetal or neonatal effect of diethylstilbestrol transmitted via breast milk (disorder)
        "698636009",  # Complete legal abortion with complication (disorder)
        "698791008",  # Fetal or neonatal effect of malpresentation during labor (disorder)
        "698795004",  # Fetal or neonatal effect of malpresentation during delivery (disorder)
        "71909003",  # Acute renal failure following molar AND/OR ectopic pregnancy (disorder)
        "72209007",  # Damage to vagina following molar AND/OR ectopic pregnancy (disorder)
        "722570003",  # Fetal or neonatal effect of meconium passage during delivery (disorder)
        "737326006",  # Physiological shock due to and following molar pregnancy (disorder)
        "76012002",  # Fetal or neonatal effect of complication of labor and/or delivery (disorder)
        "76843005",  # Tetanus complicating ectopic AND/OR molar pregnancy (disorder)
        "79333004",  # Damage to uterus following molar AND/OR ectopic pregnancy (disorder)
        "81448000",  # Hemorrhage in early pregnancy, delivered (disorder)
        "85796009",  # Afibrinogenemia following molar AND/OR ectopic pregnancy (disorder)
        "86709000",  # Pelvic peritonitis following molar AND/OR ectopic pregnancy (disorder)
        "87169001",  # Parametritis following molar AND/OR ectopic pregnancy (disorder)
        "89573004",  # Damage to broad ligament following molar AND/OR ectopic pregnancy (disorder)
        "9516007",  # Salpingitis following molar AND/OR ectopic pregnancy (disorder)
        "10812081000119108",  # Iron deficiency anemia in mother complicating childbirth (disorder)
        "169636004",  # Parent craft class attended (finding)
        "199205008",  # Infections of bladder in pregnancy (disorder)
        "200051005",  # Obstetric anesthesia with pulmonary complications with antenatal problem (disorder)
        "200052003",  # Obstetric anesthesia with pulmonary complications with postnatal problem (disorder)
        "200281008",  # Puerperal pyrexia of unknown origin with postnatal complication (disorder)
        "237244005",  # Single pregnancy (finding)
        "248994003",  # Obstetric pelvic finding (finding)
        "248997005",  # Position of ischial tuberosities (finding)
        "289204008",  # Finding of quantity of pregnancy (finding)
        "289450008",  # Finding of size of female pelvis (finding)
        "289459009",  # Finding of arrangement of female pelvis (finding)
        "289460004",  # Finding of proportion of female pelvis (finding)
        "289461000",  # Finding of measures of female pelvis (finding)
        "63662002",  # Purulent mastitis associated with childbirth (disorder)
        "86196005",  # Disorder of breast associated with childbirth (disorder)
        "106109006",  # Number of previous induced termination of pregnancy (finding)
        "16101000119107",  # Incomplete induced termination of pregnancy with complication (disorder)
        "169836001",  # Birth of child (finding)
        "198878009",  # Failed medical abortion, without complication (disorder)
        "199870008",  # Prolapse of cord with antenatal problem (disorder)
        "199881006",  # Cord tangled with compression with antenatal problem (disorder)
        "199890004",  # Short cord with antenatal problem (disorder)
        "237262001",  # Bleeding from umbilical cord (disorder)
        "23793007",  # Miscarriage without complication (disorder)
        "249034006",  # Position of umbilical cord during labor (finding)
        "249187008",  # Condition of umbilical cord at delivery (finding)
        "28493005",  # Two previous induced terminations of pregnancy (finding)
        "289304006",  # Finding of pulsation of cord through intact membranes (finding)
        "289310006",  # Finding of arrangement of umbilical cord (finding)
        "289314002",  # Finding of measures of umbilical cord (finding)
        "289316000",  # Umbilical cord problem (finding)
        "302945005",  # Absent blood vessel in umbilical cord (disorder)
        "307738005",  # Complete inevitable abortion without complication (disorder)
        "307748007",  # Incomplete inevitable abortion without complication (disorder)
        "31227001",  # Twelve previous induced terminations of pregnancy (finding)
        "364738009",  # Finding of outcome of delivery (finding)
        "38835003",  # One previous induced termination of pregnancy (finding)
        "40193008",  # Single vessel of umbilical cord (disorder)
        "44108004",  # Seven previous induced terminations of pregnancy (finding)
        "4426000",  # Ten previous induced terminations of pregnancy (finding)
        "44638009",  # No previous induced termination of pregnancy (finding)
        "48287005",  # Umbilical cord complication (disorder)
        "56029000",  # Three previous induced terminations of pregnancy (finding)
        "57197002",  # Nine previous induced terminations of pregnancy (finding)
        "62331000",  # Six previous induced terminations of pregnancy (finding)
        "76491002",  # Four previous induced terminations of pregnancy (finding)
        "81131001",  # Eleven previous induced terminations of pregnancy (finding)
        "83712005",  # Eight previous induced terminations of pregnancy (finding)
        "9402004",  # Five previous induced terminations of pregnancy (finding)
        "10058006",  # Miscarriage with amniotic fluid embolism (disorder)
        "100801000119107",  # Maternal tobacco use in pregnancy (finding)
        "100841000119109",  # Previous bariatric surgery complicating pregnancy (disorder)
        "100941000119100",  # Epilepsy in mother complicating pregnancy (disorder)
        "1010645004",  # Single umbilical cord artery type III (disorder)
        "1010646003",  # Single umbilical cord artery type IV (disorder)
        "1010652002",  # Single umbilical cord artery type II (disorder)
        "1010655000",  # Single umbilical cord artery type I (disorder)
        "10152009",  # Postpartum state, 4 weeks (finding)
        "10211000132109",  # Perinatal depression (disorder)
        "10217006",  # Third degree perineal laceration (disorder)
        "10231000132102",  # In-vitro fertilization pregnancy (finding)
        "102500002",  # Good neonatal condition at birth (finding)
        "102501003",  # Well male newborn (finding)
        "102502005",  # Well female newborn (finding)
        "102503000",  # Well premature newborn (finding)
        "102504006",  # Well premature male newborn (finding)
        "102505007",  # Well premature female newborn (finding)
        "102872000",  # Pregnancy on oral contraceptive (finding)
        "102875003",  # Surrogate pregnancy (finding)
        "102876002",  # Multigravida (finding)
        "102877006",  # Nulliparous (finding)
        "102878001",  # Recurrent miscarriage (disorder)
        "102879009",  # Post-term delivery (finding)
        "102881006",  # Normal placental secretion of chorionic gonadotropin (finding)
        "102882004",  # Abnormal placental secretion of chorionic gonadotropin (disorder)
        "102883009",  # Increased placental secretion of chorionic gonadotropin (finding)
        "102884003",  # Decreased placental secretion of chorionic gonadotropin (finding)
        "102885002",  # Absence of placental secretion of chorionic gonadotropin (disorder)
        "102886001",  # Increased amniotic fluid production (finding)
        "102887005",  # Decreased amniotic fluid production (finding)
        "102955006",  # Contraception failure (finding)
        "103008008",  # Postpartum headache (finding)
        "1031000119109",  # Insufficient prenatal care (finding)
        "104011000119109",  # Genital herpes simplex in mother complicating pregnancy (disorder)
        "10423003",  # Braun von Fernwald's sign (finding)
        "104851000119103",  # Postpartum major depression in remission (disorder)
        "1052284009",  # Restless leg syndrome in pregnancy (disorder)
        "10562009",  # Malignant hypertension complicating AND/OR reason for care during childbirth (disorder)
        "10573002",  # Infection of amniotic cavity (disorder)
        "106004004",  # Hemorrhagic complication of pregnancy (disorder)
        "106005003",  # Hypertension AND/OR vomiting complicating pregnancy childbirth AND/OR puerperium (disorder)
        "106008001",  # Delivery AND/OR maternal condition affecting management (disorder)
        "106009009",  # Fetal condition affecting obstetrical care of mother (disorder)
        "106010004",  # Pelvic dystocia AND/OR uterine disorder (disorder)
        "106111002",  # Clinical sign related to pregnancy (finding)
        "106281000119103",  # Pre-existing diabetes mellitus in mother complicating childbirth (disorder)
        "10629511000119102",  # Rhinitis of pregnancy (disorder)
        "10697004",  # Miscarriage complicated by renal failure (disorder)
        "10741751000119100",  # Anemia in mother complicating childbirth (disorder)
        "10741871000119101",  # Alcohol dependence in pregnancy (disorder)
        "10742121000119104",  # Asthma in mother complicating childbirth (disorder)
        "10742471000119108",  # Previous bariatric surgery in mother complicating childbirth (disorder)
        "10743001000119103",  # Anxiety disorder in mother complicating childbirth (disorder)
        "10743121000119108",  # Aortic valve disorder in mother complicating childbirth (disorder)
        "10743651000119105",  # Inflammation of cervix in pregnancy (disorder)
        "10743741000119103",  # Cardiac arrhythmia in mother complicating childbirth (disorder)
        "10743831000119100",  # Neoplasm of uterus affecting pregnancy (disorder)
        "10743881000119104",  # Suspected fetal abnormality affecting management of mother (disorder)
        "10745231000119102",  # Cardiac arrest due to administration of anesthesia in pregnancy (disorder)
        "10745291000119103",  # Malignant neoplastic disease in mother complicating childbirth (disorder)
        "10746341000119109",  # Acquired immune deficiency syndrome complicating childbirth (disorder)
        "10749581000119100",  # Blood coagulation disorder complicating childbirth (disorder)
        "10749641000119106",  # Blood coagulation disorder complicating pregnancy (disorder)
        "10749811000119108",  # Obstetric anesthesia with central nervous system complication in childbirth (disorder)
        "10749871000119100",  # Malignant neoplastic disease in pregnancy (disorder)
        "10750051000119105",  # Chlamydia trachomatis infection in mother complicating childbirth (disorder)
        "10750111000119108",  # Breast lump in pregnancy (finding)
        "10750161000119106",  # Cholestasis of pregnancy complicating childbirth (disorder)
        "10750411000119102",  # Nonpurulent mastitis associated with lactation (disorder)
        "10750551000119100",  # Obesity in mother complicating childbirth (disorder)
        "10750801000119102",  # False labor before 37 completed weeks of gestation (finding)
        "10750951000119106",  # Epilepsy in mother complicating childbirth (disorder)
        "10750991000119101",  # Cyst of ovary in pregnancy (disorder)
        "10751471000119101",  # Nutritional deficiency in mother complicating childbirth (disorder)
        "10751511000119105",  # Obstructed labor due to incomplete rotation of fetal head (disorder)
        "10751581000119104",  # Obstructed labor due to disproportion between fetus and pelvis (disorder)
        "10751631000119101",  # Obstructed labor due to abnormality of maternal pelvis (disorder)
        "10751701000119102",  # Spontaneous onset of labor between 37 and 39 weeks gestation with planned cesarean section (finding)
        "10751771000119107",  # Placental abruption due to afibrinogenemia (disorder)
        "10752251000119103",  # Galactorrhea in pregnancy (disorder)
        "10752641000119102",  # Eclampsia with pre-existing hypertension in childbirth (disorder)
        "10752771000119100",  # Renal failure after ectopic pregnancy (disorder)
        "10753101000119102",  # Heart disease in mother complicating childbirth (disorder)
        "10753181000119105",  # Premature rupture of membranes in full term pregnancy with onset of labor within 24 hours of rupture (disorder)
        "10753231000119104",  # Premature rupture of membranes in full term pregnancy with onset of labor more than 24 hours after rupture (disorder)
        "10753281000119103",  # Premature rupture of membranes in full term pregnancy with onset of labor unknown (disorder)
        "10753491000119101",  # Gestational diabetes mellitus in childbirth (disorder)
        "10754031000119105",  # Gonorrhea in mother complicating childbirth (disorder)
        "10754331000119108",  # Air embolism in childbirth (disorder)
        "10754701000119100",  # Genital herpes simplex in mother complicating childbirth (disorder)
        "10754781000119108",  # Gallbladder calculus in mother complicating childbirth (disorder)
        "10754881000119104",  # Diabetes mellitus in mother complicating childbirth (disorder)
        "10754921000119106",  # Hyperthyroidism in childbirth (disorder)
        "10755041000119100",  # Alcohol dependence in childbirth (disorder)
        "10755671000119100",  # Human immunodeficiency virus in mother complicating childbirth (disorder)
        "10755951000119102",  # Heart murmur in mother in childbirth (finding)
        "10756101000119107",  # Group B streptococcus infection in mother complicating childbirth (disorder)
        "10756261000119102",  # Physical abuse complicating childbirth (finding)
        "10756301000119105",  # Physical abuse complicating pregnancy (finding)
        "10757401000119104",  # Pre-existing hypertensive heart and chronic kidney disease in mother complicating childbirth (disorder)
        "10757441000119102",  # Pre-existing hypertensive heart disease in mother complicating childbirth (disorder)
        "10757481000119107",  # Pre-existing hypertensive heart and chronic kidney disease in mother complicating pregnancy (disorder)
        "10759031000119106",  # Pre-existing hypertensive heart disease in mother complicating pregnancy (disorder)
        "10759111000119102",  # Viral hepatitis in mother complicating childbirth (disorder)
        "10759151000119101",  # Viral hepatitis in mother complicating pregnancy (disorder)
        "10759191000119106",  # Septic thrombophlebitis in puerperium (disorder)
        "10759231000119102",  # Salpingo-oophoritis in pregnancy (disorder)
        "10759311000119104",  # Pulmonary embolism in childbirth (disorder)
        "10759351000119103",  # Sickle cell anemia in mother complicating childbirth (disorder)
        "10759401000119105",  # Sickle cell trait in mother complicating childbirth (disorder)
        "10759611000119105",  # Preterm premature rupture of membranes with onset of labor later than 24 hours after rupture (disorder)
        "10759661000119108",  # Preterm premature rupture of membranes with onset of labor unknown (disorder)
        "10759711000119103",  # Preterm premature rupture of membranes with onset of labor within 24 hours of rupture (disorder)
        "10759761000119100",  # Rubella in mother complicating childbirth (disorder)
        "10759841000119105",  # Sexually transmitted infectious disease in mother complicating childbirth (disorder)
        "10759881000119100",  # Tuberculosis in mother complicating childbirth (disorder)
        "10759921000119107",  # Syphilis in mother complicating childbirth (disorder)
        "10759961000119102",  # Viral disease in mother complicating childbirth (disorder)
        "10760001000119100",  # Thyroid disease in mother complicating childbirth (disorder)
        "10760041000119103",  # Thyroiditis in mother complicating childbirth (disorder)
        "10760081000119108",  # Pyemic and septic embolism in childbirth (disorder)
        "10760131000119108",  # Pyemic and septic embolism in pregnancy (disorder)
        "10760221000119101",  # Uterine prolapse in pregnancy (disorder)
        "10760261000119106",  # Uterine laceration during delivery (disorder)
        "10760301000119103",  # Retraction of nipple during lactation (finding)
        "10760381000119106",  # Thyroiditis in pregnancy (disorder)
        "10760421000119102",  # Psychotic disorder in mother complicating childbirth (disorder)
        "10760461000119107",  # Psychotic disorder in mother complicating pregnancy (disorder)
        "10760541000119109",  # Traumatic injury to vulva during pregnancy (disorder)
        "10760581000119104",  # Pain in round ligament in pregnancy (finding)
        "10760661000119109",  # Triplets, some live born (finding)
        "10760701000119102",  # Quadruplets, all live born (finding)
        "10760741000119100",  # Quadruplets, some live born (finding)
        "10760781000119105",  # Quintuplets, all live born (finding)
        "10760821000119100",  # Quintuplets, some live born (finding)
        "10760861000119105",  # Sextuplets, all live born (finding)
        "10760901000119104",  # Sextuplets, some live born (finding)
        "10760941000119102",  # Psychological abuse complicating childbirth (finding)
        "10760981000119107",  # Psychological abuse complicating pregnancy (finding)
        "10761021000119102",  # Sexual abuse complicating childbirth (finding)
        "10761061000119107",  # Sexual abuse complicating pregnancy (finding)
        "10761101000119105",  # Vacuum assisted vaginal delivery (finding)
        "10761141000119107",  # Preterm labor in second trimester with preterm delivery in second trimester (finding)
        "10761191000119104",  # Preterm labor in third trimester with preterm delivery in third trimester (finding)
        "10761241000119104",  # Preterm labor with preterm delivery (finding)
        "10761341000119105",  # Preterm labor without delivery (finding)
        "10761391000119102",  # Tobacco use in mother complicating childbirth (finding)
        "10805641000119102",  # Congenital cardiovascular anomaly in mother complicating childbirth (disorder)
        "10806041000119104",  # Infection of nipple associated with lactation (disorder)
        "10806241000119108",  # Thalassemia in mother complicating childbirth (disorder)
        "10807061000119103",  # Liver disorder in mother complicating childbirth (disorder)
        "10808861000119105",  # Maternal distress during childbirth (disorder)
        "10809061000119106",  # Malaria in mother complicating childbirth (disorder)
        "10809101000119109",  # Hypothyroidism in childbirth (disorder)
        "10809641000119100",  # Mitral valve disorder in mother complicating childbirth (disorder)
        "10811121000119102",  # Major depressive disorder in mother complicating childbirth (disorder)
        "10811161000119107",  # Major depressive disorder in mother complicating pregnancy (disorder)
        "10811201000119102",  # Mental disorder in mother complicating childbirth (disorder)
        "10812041000119103",  # Urinary tract infection due to incomplete miscarriage (disorder)
        "10812131000119108",  # Malnutrition in mother complicating childbirth (disorder)
        "10835621000119104",  # Congenital anomaly of vagina in mother complicating pregnancy (disorder)
        "10835701000119102",  # Congenital anomaly of vulva in mother complicating pregnancy (disorder)
        "10835781000119105",  # Amniotic fluid embolism in childbirth (disorder)
        "10835871000119104",  # Depressive disorder in mother complicating childbirth (disorder)
        "10835971000119109",  # Anti-A sensitization in pregnancy (disorder)
        "10836071000119101",  # Dislocation of symphysis pubis in pregnancy (disorder)
        "10836241000119104",  # Cracked nipple associated with lactation (disorder)
        "10853001",  # Obstetrical complication of general anesthesia (disorder)
        "10884004",  # Threatened abortion in third trimester (disorder)
        "109562007",  # Abnormal chorion (disorder)
        "109891004",  # Detached products of conception (disorder)
        "109892006",  # Detached chorionic villi (disorder)
        "109893001",  # Detached trophoblast (disorder)
        "109894007",  # Retained placenta (disorder)
        "109895008",  # Retained placental fragment (disorder)
        "110081000119109",  # Bacterial vaginosis in pregnancy (disorder)
        "110091000119107",  # Candidiasis of vagina in pregnancy (disorder)
        "11026009",  # Miscarriage with renal tubular necrosis (disorder)
        "11082009",  # Abnormal pregnancy (finding)
        "11109001",  # Miscarriage with uremia (disorder)
        "111208003",  # Melasma gravidarum (disorder)
        "111294009",  # Distended umbilical veins (disorder)
        "111424000",  # Retained products of conception not following abortion (disorder)
        "111438007",  # Hypertension secondary to renal disease in obstetric context (disorder)
        "111447004",  # Placental condition affecting management of mother (disorder)
        "111451002",  # Obstetrical injury to pelvic organ (disorder)
        "111452009",  # Postpartum afibrinogenemia with hemorrhage (disorder)
        "111453004",  # Retained placenta, without hemorrhage (disorder)
        "111458008",  # Venous thrombosis in puerperium (disorder)
        "111459000",  # Infection of nipple, associated with childbirth (disorder)
        "112070001",  # Onset of labor induced (finding)
        "112074005",  # Condition of fetal membrane (finding)
        "112075006",  # Premature birth of newborn sextuplets (finding)
        "11209007",  # Cord entanglement without compression (disorder)
        "1124005",  # Postpartum period, 6 days (finding)
        "1125006",  # Sepsis during labor (disorder)
        "11337002",  # Quickening of fetus (finding)
        "1142030006",  # Nutritional anemia of pregnancy (disorder)
        "1142048002",  # Persistent gestational proteinuria (disorder)
        "1142062009",  # Macrocytic anemia of pregnancy (disorder)
        "1142083003",  # Hemolytic anemia of pregnancy (disorder)
        "1142097006",  # Intermittent gestational proteinuria (disorder)
        "1144435009",  # Varicose veins of perineum in pregnancy (disorder)
        "1144807000",  # Varicose veins of bilateral lower limbs in pregnancy (disorder)
        "11454006",  # Failed attempted abortion with amniotic fluid embolism (disorder)
        "1148484005",  # Retained uterine membrane (disorder)
        "1148583001",  # Phlebitis in pregnancy (disorder)
        "1148584007",  # Venous thrombosis in pregnancy (disorder)
        "1148587000",  # Phlegmasia alba dolens in pregnancy (disorder)
        "1148619000",  # Retained uterine membrane without hemorrhage (disorder)
        "1148740009",  # Jorissenne sign (finding)
        "1148755000",  # Retained products of conception with no hemorrhage (disorder)
        "114881000119108",  # Maternal gonorrhea during pregnancy (disorder)
        "1148848009",  # Multiple pregnancy following assisted fertilization (finding)
        "1148941002",  # Retention of part of placenta and uterine membrane without hemorrhage (disorder)
        "1148942009",  # Retention of part of placenta without hemorrhage (disorder)
        "1148943004",  # Retention of placenta and uterine membrane (disorder)
        "1148970006",  # Higher order multiple pregnancy due to and following medically assisted reproduction (finding)
        "1149345007",  # Excessive weight gain measured during pregnancy (finding)
        "1149472000",  # Thrombophlebitis of bilateral lower limbs in pregnancy (disorder)
        "1156027008",  # Thrombus of dural sinus in pregnancy (disorder)
        "1156031002",  # Venous varices in pregnancy (disorder)
        "1156100006",  # Pregnancy with normal glucose tolerance test (finding)
        "1157306001",  # Embryo present in utero (finding)
        "11612004",  # Chorioamnionitis (disorder)
        "11618000",  # Intra-amniotic infection of fetus (disorder)
        "1162631002",  # Bilobate placenta (disorder)
        "11687002",  # Gestational diabetes mellitus (disorder)
        "11718971000119100",  # Diarrhea in pregnancy (disorder)
        "1173098001",  # Placenta previa lateralis (disorder)
        "11763009",  # Placenta increta (disorder)
        "1179359005",  # Maternal epilepsy due to perinatal stroke (disorder)
        "1179361001",  # Maternal perinatal sensorineural hearing loss (disorder)
        "1179363003",  # Maternal perinatal purpura (disorder)
        "1179366006",  # Maternal perinatal epistaxis (disorder)
        "1179398000",  # Maternal perinatal infection (disorder)
        "118178000",  # Finding of placental function (finding)
        "118179008",  # Finding related to placental secretion of chorionic gonadotropin (finding)
        "118180006",  # Finding related to amniotic fluid function (finding)
        "118181005",  # Finding related to amniotic fluid production (finding)
        "118185001",  # Finding related to pregnancy (finding)
        "118189007",  # Prenatal finding (finding)
        "118212000",  # Parity finding (finding)
        "118213005",  # Postpartum finding (finding)
        "118215003",  # Delivery finding (finding)
        "118216002",  # Labor finding (finding)
        "118781000119108",  # Pre-existing hypertensive chronic kidney disease in mother complicating pregnancy (disorder)
        "11914001",  # Transverse OR oblique presentation of fetus (disorder)
        "11942004",  # Perineal laceration involving pelvic floor (disorder)
        "1196980000",  # Injury of placenta due to cesarean section (disorder)
        "1197158009",  # Placental separation (finding)
        "1197159001",  # Placental separation with bleeding (finding)
        "119891000119105",  # Postpartum hemorrhage due to total placenta previa (disorder)
        "119901000119109",  # Hemorrhage co-occurrent and due to partial placenta previa (disorder)
        "120001000119107",  # Group B streptococcus carrier complicating pregnancy (disorder)
        "12062007",  # Puerperal pelvic cellulitis (disorder)
        "1220564001",  # Incomplete induced termination of pregnancy complicated by shock (disorder)
        "1220586000",  # Incomplete induced termination of pregnancy complicated by renal failure (disorder)
        "1220625008",  # Incomplete induced termination of pregnancy complicated by metabolic disorder (disorder)
        "1220626009",  # Incomplete induced termination of pregnancy complicated by genital-pelvic infection (disorder)
        "1220630007",  # Incomplete induced termination of pregnancy complicated by embolism (disorder)
        "1220631006",  # Incomplete induced termination of pregnancy complicated by delayed or excessive hemorrhage (disorder)
        "1220632004",  # Incomplete induced termination of pregnancy complicated by damage to pelvic organs or tissues (disorder)
        "1220641009",  # Complete induced termination of pregnancy with complication (disorder)
        "1220645000",  # Shock due to and following complete induced termination of pregnancy (disorder)
        "1220648003",  # Renal failure due to and following complete induced termination of pregnancy (disorder)
        "1220651005",  # Metabolic disease due to and following complete induced termination of pregnancy (disorder)
        "1220652003",  # Embolism due to and following complete induced termination of pregnancy (disorder)
        "1220653008",  # Excessive hemorrhage due to and following complete induced termination of pregnancy (disorder)
        "1220656000",  # Genital-pelvic infection due to and following complete induced termination of pregnancy (disorder)
        "1220657009",  # Damage to pelvic organs and/or tissue due to and following complete induced termination of pregnancy (disorder)
        "12271000132101",  # Maternal infection caused by Streptococcus agalactiae (disorder)
        "12275071000119109",  # Conjoined twin pregnancy (disorder)
        "1230040001",  # Cornual placenta accreta (disorder)
        "1230061008",  # Premature aging of placenta (disorder)
        "12349003",  # Danforth's sign (finding)
        "123667006",  # Abnormal mature chorion (disorder)
        "12394009",  # Miscarriage with cerebral anoxia (disorder)
        "124735008",  # Late onset of labor (finding)
        "125586008",  # Disorder of placenta (disorder)
        "126941005",  # Subdural intracranial hemorrhage due to birth trauma (disorder)
        "127009",  # Miscarriage with laceration of cervix (disorder)
        "12729009",  # Prolonged rupture of membranes (disorder)
        "127362006",  # Previous pregnancies (finding)
        "127364007",  # Primigravida (finding)
        "127365008",  # Gravida 2 (finding)
        "127366009",  # Gravida 3 (finding)
        "127367000",  # Gravida 4 (finding)
        "127368005",  # Gravida 5 (finding)
        "127369002",  # Gravida 6 (finding)
        "127370001",  # Gravida 7 (finding)
        "127371002",  # Gravida 8 (finding)
        "127372009",  # Gravida 9 (finding)
        "127373004",  # Gravida 10 (finding)
        "127374005",  # Gravida more than 10 (finding)
        "12803000",  # High maternal weight gain (finding)
        "128076000",  # Bone AND/OR joint disorder in mother complicating pregnancy, childbirth AND/OR puerperium (disorder)
        "128077009",  # Trauma to vagina during delivery (disorder)
        "128604004",  # Disorder of product of conception (disorder)
        "12867002",  # Fetal distress affecting management of mother (disorder)
        "129597002",  # Moderate hyperemesis gravidarum (disorder)
        "129598007",  # Severe hyperemesis gravidarum (disorder)
        "12983003",  # Failed attempted abortion with septic shock (disorder)
        "130958001",  # Disorder of placental circulatory function (disorder)
        "130959009",  # Disorder of amniotic fluid turnover (disorder)
        "130960004",  # Disorder of amniotic fluid production (disorder)
        "1323351000000104",  # First trimester bleeding (disorder)
        "13273002",  # Postpartum period, 4 days (finding)
        "13384007",  # Miscarriage complicated by metabolic disorder (disorder)
        "13404009",  # Twin-to-twin blood transfer (disorder)
        "1343000",  # Deep transverse arrest (disorder)
        "134781000119106",  # High risk pregnancy due to recurrent miscarriage (finding)
        "13763000",  # Gestation period, 34 weeks (finding)
        "13798002",  # Gestation period, 38 weeks (finding)
        "13842006",  # Subinvolution of uterus in puerperium (disorder)
        "13859001",  # Premature birth of newborn twins (finding)
        "13866000",  # Fetal acidemia affecting management of mother (disorder)
        "13868004",  # Para 5 (finding)
        "13943000",  # Failed attempted abortion complicated by embolism (disorder)
        "14022007",  # Fetal death, affecting management of mother (disorder)
        "14049007",  # Chaussier's sign (finding)
        "14094001",  # Excessive vomiting in pregnancy (disorder)
        "14136000",  # Miscarriage with perforation of broad ligament (disorder)
        "14209008",  # Para 1 (finding)
        "14418008",  # Precocious pregnancy (finding)
        "1469007",  # Miscarriage with urinary tract infection (disorder)
        "1474004",  # Hypertensive heart AND renal disease complicating AND/OR reason for care during childbirth (disorder)
        "15100005",  # Postpartum state, 2 weeks (finding)
        "151441000119105",  # Twin live born in hospital by vaginal delivery (finding)
        "15196006",  # Intraplacental hematoma (disorder)
        "152231000119106",  # Pregnancy not yet confirmed (finding)
        "15230009",  # Liver disorder in pregnancy (disorder)
        "1538006",  # Central nervous system malformation in fetus affecting obstetrical care (disorder)
        "15400003",  # Obstetrical air embolism (disorder)
        "15406009",  # Retromammary abscess associated with childbirth (disorder)
        "15467003",  # Term birth of identical twins, both living (finding)
        "15504009",  # Rupture of gravid uterus (disorder)
        "15539009",  # Hydrops fetalis due to isoimmunization (disorder)
        "15592002",  # Previous pregnancy 1 (finding)
        "156072005",  # Incomplete miscarriage (disorder)
        "156073000",  # Complete miscarriage (disorder)
        "156222001",  # Short umbilical cord (disorder)
        "15633004",  # Gestation period, 16 weeks (finding)
        "15643101000119103",  # Gastroesophageal reflux disease in pregnancy (disorder)
        "15663008",  # Placenta previa centralis (disorder)
        "15750121000119108",  # Pre-existing severe obesity in mother complicating pregnancy (disorder)
        "15809008",  # Miscarriage with perforation of uterus (disorder)
        "15898009",  # Third stage of labor (finding)
        "1592005",  # Failed attempted abortion with uremia (disorder)
        "15938005",  # Eclampsia (disorder)
        "15953341000119100",  # Fissure of right nipple during lactation (disorder)
        "15953381000119105",  # Fissure of left nipple during lactation (disorder)
        "16038004",  # Abortion due to Leptospira (disorder)
        "16111000119105",  # Incomplete spontaneous abortion due to hemorrhage (disorder)
        "1621000119101",  # Chlamydia trachomatis infection in pregnancy (disorder)
        "16253001",  # Dilated peripartum cardiomyopathy (disorder)
        "16271000119108",  # Pyelonephritis in pregnancy (disorder)
        "16320551000119109",  # Bleeding from female genital tract co-occurrent with pregnancy (disorder)
        "16356006",  # Multiple pregnancy (finding)
        "1639007",  # Abnormality of organs AND/OR soft tissues of pelvis affecting pregnancy (disorder)
        "16538005",  # Early postpartum state (finding)
        "16574001",  # Cardiac arrest after obstetrical surgery AND/OR other procedure including delivery (disorder)
        "16607004",  # Missed abortion (disorder)
        "16714009",  # Miscarriage with laceration of bowel (disorder)
        "16756008",  # Albuminuria in pregnancy without hypertension (disorder)
        "168092006",  # Amniotic fluid -meconium stain (finding)
        "16836261000119107",  # Ectopic pregnancy of left ovary (disorder)
        "16836351000119100",  # Ectopic pregnancy of right ovary (disorder)
        "16836391000119105",  # Pregnancy of left fallopian tube (disorder)
        "16836431000119100",  # Ruptured tubal pregnancy of right fallopian tube (disorder)
        "16836571000119103",  # Ruptured tubal pregnancy of left fallopian tube (disorder)
        "16836811000119108",  # Miscarriage of left tubal ectopic pregnancy (disorder)
        "16836851000119109",  # Miscarriage of right tubal ectopic pregnancy (disorder)
        "16836891000119104",  # Pregnancy of right fallopian tube (disorder)
        "169470007",  # Combined oral contraceptive pill failure (finding)
        "169471006",  # Progestogen-only pill failure (finding)
        "169488004",  # Contraceptive intrauterine device failure - pregnant (finding)
        "16950007",  # Fourth degree perineal laceration involving anal mucosa (disorder)
        "169501005",  # Pregnant, diaphragm failure (finding)
        "169508004",  # Pregnant, sheath failure (finding)
        "169524003",  # Depot contraceptive failure (finding)
        "169533001",  # Contraceptive sponge failure (finding)
        "169539002",  # Symptothermal contraception failure (finding)
        "169544009",  # Postcoital oral contraceptive pill failure (finding)
        "169545005",  # Postcoital intrauterine contraceptive device failure (finding)
        "169548007",  # Vasectomy failure (finding)
        "169550004",  # Female sterilization failure (finding)
        "169560008",  # Pregnant - urine test confirms (finding)
        "169561007",  # Pregnant - blood test confirms (finding)
        "169562000",  # Pregnant - vaginal examination confirms (finding)
        "169563005",  # Pregnant - on history (finding)
        "169564004",  # Pregnant - on abdominal palpation (finding)
        "169565003",  # Pregnant - planned (finding)
        "169566002",  # Pregnancy unplanned but wanted (finding)
        "169567006",  # Pregnancy unplanned and unwanted (finding)
        "169568001",  # Unplanned pregnancy unknown if child is wanted (finding)
        "169569009",  # Questionable if pregnancy was planned (finding)
        "169613008",  # Antenatal care provider (finding)
        "169614002",  # Antenatal care from general practitioner (finding)
        "169615001",  # Antenatal care from consultant (finding)
        "169616000",  # Antenatal - shared care (finding)
        "169620001",  # Delivery: no place booked (finding)
        "169628008",  # Delivery booking - length of stay (finding)
        "169632002",  # Parent craft classes (finding)
        "169637008",  # Parent craft -individual class (finding)
        "169638003",  # Parent craft - group class (finding)
        "169646002",  # Antenatal amniocentesis (finding)
        "169650009",  # Antenatal amniocentesis wanted (finding)
        "169651008",  # Antenatal amniocentesis - awaited (finding)
        "169657007",  # Antenatal ultrasound scan status (finding)
        "169661001",  # Antenatal ultrasound scan wanted (finding)
        "169662008",  # Antenatal ultrasound scan awaited (finding)
        "169667002",  # Antenatal ultrasound scan for slow growth (finding)
        "169686005",  # Alpha-fetoprotein blood test wanted (finding)
        "169688006",  # Alpha-fetoprotein blood sent (finding)
        "169694003",  # Rubella screening wanted (finding)
        "169695002",  # Rubella screening - blood sent (finding)
        "169696001",  # Rubella status not known (finding)
        "169734005",  # Spontaneous rupture of membranes (finding)
        "169783009",  # Postnatal examination minor problem found (finding)
        "169784003",  # Postnatal examination normal (finding)
        "169813005",  # Home birth (finding)
        "169814004",  # General practice unit birth (finding)
        "169815003",  # Consultant unit birth (finding)
        "169817006",  # Ambulance birth (finding)
        "169826009",  # Single live birth (finding)
        "169827000",  # Single stillbirth (finding)
        "169828005",  # Twins - both live born (finding)
        "169829002",  # Twins - one still and one live born (finding)
        "169830007",  # Twins - both stillborn (finding)
        "169831006",  # Triplets - all live born (finding)
        "169832004",  # Triplets - two live and one stillborn (finding)
        "169833009",  # Triplets - one live and two stillborn (finding)
        "169834003",  # Triplets - three stillborn (finding)
        "169952004",  # Placental finding (finding)
        "169960003",  # Labor details (finding)
        "169961004",  # Normal birth (finding)
        "169963001",  # Birth details not known (finding)
        "171000119107",  # Maternal obesity complicating pregnancy, childbirth and the puerperium, antepartum (disorder)
        "17263003",  # Amniotic fluid embolism (disorder)
        "17285009",  # Intraperitoneal pregnancy (disorder)
        "173300003",  # Disorder of pregnancy (disorder)
        "17333005",  # Term birth of multiple newborns (finding)
        "17369002",  # Miscarriage (disorder)
        "17380002",  # Failed attempted abortion with acute renal failure (disorder)
        "17382005",  # Cervical incompetence (disorder)
        "17433009",  # Ruptured ectopic pregnancy (disorder)
        "17532001",  # Breech malpresentation successfully converted to cephalic presentation (disorder)
        "17588005",  # Abnormal yolk sac (disorder)
        "17594002",  # Fetal bradycardia affecting management of mother (disorder)
        "17618008",  # Previous pregnancies 9 (finding)
        "17787002",  # Peripheral neuritis in pregnancy (disorder)
        "178280004",  # Postnatal infection (disorder)
        "18122002",  # Normal uterine contraction wave (finding)
        "18237006",  # Parenchymatous mastitis associated with childbirth (disorder)
        "18260003",  # Postpartum psychosis (disorder)
        "18391007",  # Elective abortion (disorder)
        "18416000",  # Essential hypertension complicating AND/OR reason for care during childbirth (disorder)
        "184339009",  # Miscarriage at 8 to 28 weeks (disorder)
        "18491000119109",  # Psychological disorder during pregnancy (disorder)
        "18606002",  # Dyscoordinate labor (disorder)
        "18613002",  # Miscarriage with septic shock (disorder)
        "18656007",  # Grand multipara (finding)
        "19099008",  # Failed attempted abortion with laceration of urinary bladder (disorder)
        "19117007",  # Placenta reflexa (disorder)
        "19169002",  # Miscarriage in first trimester (disorder)
        "19228003",  # Failed attempted abortion with perforation of urinary bladder (disorder)
        "19236007",  # Immature abnormal placenta (disorder)
        "19258009",  # Placenta extrachorales (disorder)
        "19337000",  # Fully effaced cervix (finding)
        "19363005",  # Failed attempted abortion with blood-clot embolism (disorder)
        "19569008",  # Mild hyperemesis gravidarum (disorder)
        "19729005",  # Prolapse of anterior lip of cervix obstructing labor (disorder)
        "19773009",  # Infection of the breast AND/OR nipple associated with childbirth (disorder)
        "198332009",  # Old uterine laceration due to obstetric cause (disorder)
        "198347000",  # Cicatrix (postpartum) of cervix (disorder)
        "198609003",  # Complication of pregnancy, childbirth and/or puerperium (disorder)
        "198617006",  # Delivery of viable fetus in abdominal pregnancy (disorder)
        "198620003",  # Ruptured tubal pregnancy (disorder)
        "198624007",  # Membranous pregnancy (disorder)
        "198626009",  # Mesenteric pregnancy (disorder)
        "198627000",  # Angular pregnancy (disorder)
        "198647008",  # Renal failure due to and following incomplete miscarriage (disorder)
        "198648003",  # Metabolic disorder due to incomplete miscarriage (disorder)
        "198649006",  # Shock due to incomplete miscarriage (disorder)
        "198650006",  # Embolism due to incomplete miscarriage (disorder)
        "198655001",  # Complete miscarriage with genital tract or pelvic infection (disorder)
        "198656000",  # Complete miscarriage with delayed or excessive hemorrhage (disorder)
        "198657009",  # Complete miscarriage with damage to pelvic organs or tissues (disorder)
        "198659007",  # Complete miscarriage with renal failure (disorder)
        "198660002",  # Complete miscarriage with metabolic disorder (disorder)
        "19866007",  # Previous operation to cervix affecting pregnancy (disorder)
        "198661003",  # Complete miscarriage with shock (disorder)
        "198663000",  # Complete miscarriage with embolism (disorder)
        "198806007",  # Failed attempted abortion with genital tract or pelvic infection (disorder)
        "198807003",  # Failed attempted abortion with delayed or excessive hemorrhage (disorder)
        "198808008",  # Failed attempted abortion with damage to pelvic organs or tissues (disorder)
        "198809000",  # Failed attempted abortion with renal failure (disorder)
        "198810005",  # Failed attempted abortion with metabolic disorder (disorder)
        "198811009",  # Failed attempted abortion with shock (disorder)
        "198812002",  # Failed attempted abortion with embolism (disorder)
        "198874006",  # Failed medical abortion, complicated by genital tract and pelvic infection (disorder)
        "198875007",  # Failed medical abortion, complicated by delayed or excessive hemorrhage (disorder)
        "198876008",  # Failed medical abortion, complicated by embolism (disorder)
        "198903000",  # Hemorrhage due to placenta previa (disorder)
        "198912003",  # Premature separation of placenta with coagulation defect (disorder)
        "198920001",  # Antepartum hemorrhage with trauma (disorder)
        "198925006",  # Antepartum hemorrhage with uterine leiomyoma (disorder)
        "198941007",  # Hypertension complicating pregnancy, childbirth and the puerperium (disorder)
        "198942000",  # Benign essential hypertension complicating pregnancy, childbirth and the puerperium (disorder)
        "198947006",  # Benign essential hypertension complicating pregnancy, childbirth and the puerperium with postnatal complication (disorder)
        "198949009",  # Renal hypertension complicating pregnancy, childbirth and the puerperium (disorder)
        "198968007",  # Transient hypertension of pregnancy with postnatal complication (disorder)
        "198986005",  # Severe pre-eclampsia with postnatal complication (disorder)
        "198992004",  # Eclampsia in pregnancy (disorder)
        "198993009",  # Eclampsia with postnatal complication (disorder)
        "198997005",  # Pre-eclampsia or eclampsia with pre-existing hypertension (disorder)
        "199003007",  # Pre-eclampsia or eclampsia with pre-existing hypertension with postnatal complication (disorder)
        "199005000",  # Pre-existing hypertension complicating pregnancy, childbirth and puerperium (disorder)
        "199006004",  # Pre-existing hypertensive heart disease complicating pregnancy, childbirth and the puerperium (disorder)
        "199007008",  # Pre-existing hypertensive heart and renal disease complicating pregnancy, childbirth and the puerperium (disorder)
        "199025001",  # Hyperemesis gravidarum with metabolic disturbance (disorder)
        "199047001",  # False labor (finding)
        "199050003",  # False labor at or after 37 completed weeks of gestation (finding)
        "199096006",  # Peripheral neuritis in pregnancy with postnatal complication (disorder)
        "199102004",  # Asymptomatic bacteriuria in pregnancy with postnatal complication (disorder)
        "199109008",  # Genitourinary tract infection in pregnancy with postnatal complication (disorder)
        "199110003",  # Infections of kidney in pregnancy (disorder)
        "199111004",  # Urinary tract infection following delivery (disorder)
        "199112006",  # Infections of the genital tract in pregnancy (disorder)
        "199124004",  # Fatigue during pregnancy with postnatal complication (disorder)
        "199132007",  # Pregnancy-related glycosuria (disorder)
        "199137001",  # Glycosuria during pregnancy with postnatal complication (disorder)
        "199139003",  # Pregnancy-induced edema and proteinuria without hypertension (disorder)
        "199141002",  # Gestational edema with proteinuria (disorder)
        "199154009",  # Maternal syphilis during pregnancy, childbirth and the puerperium (disorder)
        "199161008",  # Maternal gonorrhea during pregnancy, childbirth and the puerperium (disorder)
        "199175001",  # Maternal tuberculosis during pregnancy, childbirth and the puerperium (disorder)
        "199183007",  # Maternal malaria during pregnancy, childbirth and the puerperium (disorder)
        "199203001",  # Viral hepatitis complicating pregnancy, childbirth and the puerperium (disorder)
        "199257008",  # Mental disorders during pregnancy, childbirth and the puerperium (disorder)
        "199279003",  # Orthopedic disorders during pregnancy, childbirth and the puerperium (disorder)
        "199293005",  # Disease of the respiratory system complicating pregnancy, childbirth and/or the puerperium (disorder)
        "199294004",  # Disease of the digestive system complicating pregnancy, childbirth and/or the puerperium (disorder)
        "199295003",  # Disease of skin and subcutaneous tissue complicating pregnancy, childbirth and/or puerperium (disorder)
        "199296002",  # Endocrine, nutritional and metabolic disease complicating pregnancy, childbirth and puerperium (disorder)
        "199297006",  # Disease of nervous system complicating pregnancy, childbirth and puerperium (disorder)
        "199305006",  # Complications specific to multiple gestation (disorder)
        "199306007",  # Continuing pregnancy after abortion of one fetus or more (disorder)
        "199307003",  # Continuing pregnancy after intrauterine death of one or more fetuses (disorder)
        "199308008",  # Subluxation of symphysis pubis in pregnancy, childbirth and the puerperium (disorder)
        "199314001",  # Normal delivery but ante- or post- natal conditions present (disorder)
        "199318003",  # Twin pregnancy with antenatal problem (disorder)
        "199322008",  # Triplet pregnancy with antenatal problem (disorder)
        "199326006",  # Quadruplet pregnancy with antenatal problem (disorder)
        "199329004",  # Multiple delivery, all spontaneous (finding)
        "199330009",  # Multiple delivery, all by forceps and vacuum extractor (finding)
        "199331008",  # Multiple delivery, all by cesarean section (finding)
        "199345002",  # Unstable lie with antenatal problem (disorder)
        "199359009",  # Oblique lie with antenatal problem (disorder)
        "199363002",  # Transverse lie with antenatal problem (disorder)
        "199373000",  # High head at term (finding)
        "199376008",  # High head at term with antenatal problem (finding)
        "199378009",  # Multiple pregnancy with malpresentation (disorder)
        "199381004",  # Multiple pregnancy with malpresentation with antenatal problem (disorder)
        "199385008",  # Prolapsed arm with antenatal problem (disorder)
        "199397009",  # Cephalopelvic disproportion (disorder)
        "199406006",  # Generally contracted pelvis with antenatal problem (disorder)
        "199410009",  # Inlet pelvic contraction with antenatal problem (disorder)
        "199414000",  # Outlet pelvic contraction with antenatal problem (disorder)
        "199416003",  # Mixed feto-pelvic disproportion (disorder)
        "199419005",  # Mixed feto-pelvic disproportion with antenatal problem (disorder)
        "199423002",  # Large fetus causing disproportion with antenatal problem (disorder)
        "199425009",  # Hydrocephalic disproportion (disorder)
        "199428006",  # Hydrocephalic disproportion with antenatal problem (disorder)
        "199446000",  # Pelvic soft tissue abnormality in pregnancy, childbirth and the puerperium (disorder)
        "199466009",  # Retroverted incarcerated gravid uterus (disorder)
        "199484006",  # Cervical incompetence with antenatal problem (disorder)
        "199485007",  # Cervical incompetence with postnatal complication (disorder)
        "199513008",  # Pelvic soft tissue abnormality in pregnancy, childbirth and the puerperium with antenatal problem (disorder)
        "199514002",  # Pelvic soft tissue abnormality in pregnancy, childbirth and the puerperium with postnatal complication (disorder)
        "199578005",  # Fetal-maternal hemorrhage with antenatal problem (disorder)
        "199583002",  # Rhesus isoimmunization with antenatal problem (disorder)
        "199595002",  # Labor and delivery complication by meconium in amniotic fluid (disorder)
        "199596001",  # Labor and delivery complicated by fetal heart rate anomaly with meconium in amniotic fluid (disorder)
        "199597005",  # Labor and delivery complicated by biochemical evidence of fetal stress (disorder)
        "199625002",  # Placental transfusion syndromes (disorder)
        "199647002",  # Polyhydramnios with antenatal problem (disorder)
        "199654008",  # Oligohydramnios with antenatal problem (disorder)
        "199659003",  # Premature rupture of membranes with antenatal problem (disorder)
        "199660008",  # Premature rupture of membranes with onset of labor within 24 hours of the rupture (disorder)
        "199661007",  # Premature rupture of membranes, labor delayed by therapy (disorder)
        "199662000",  # Premature rupture of membranes with onset of labor after 24 hours of the rupture (disorder)
        "199670005",  # Prolonged artificial rupture of membranes (disorder)
        "199673007",  # Prolonged artificial rupture of membranes with antenatal problem (disorder)
        "199678003",  # Amniotic cavity infection with antenatal problem (disorder)
        "199695006",  # Failed mechanical induction with antenatal problem (disorder)
        "199715003",  # Grand multiparity with antenatal problem (finding)
        "199719009",  # Elderly primigravida with antenatal problem (finding)
        "199732004",  # Abnormal finding on antenatal screening of mother (finding)
        "199733009",  # Abnormal hematologic finding on antenatal screening of mother (finding)
        "199741009",  # Malnutrition in pregnancy (disorder)
        "199745000",  # Complication occurring during labor and delivery (disorder)
        "199746004",  # Obstructed labor (disorder)
        "199747008",  # Obstructed labor due to fetal malposition (disorder)
        "199750006",  # Obstructed labor due to fetal malposition with antenatal problem (disorder)
        "199751005",  # Obstructed labor due to breech presentation (disorder)
        "199752003",  # Obstructed labor due to face presentation (disorder)
        "199753008",  # Obstructed labor due to brow presentation (disorder)
        "199754002",  # Obstructed labor due to shoulder presentation (disorder)
        "199755001",  # Obstructed labor due to compound presentation (disorder)
        "199757009",  # Obstructed labor caused by bony pelvis (disorder)
        "199760002",  # Obstructed labor caused by bony pelvis with antenatal problem (disorder)
        "199761003",  # Obstructed labor due to deformed pelvis (disorder)
        "199762005",  # Obstructed labor due to generally contracted pelvis (disorder)
        "199763000",  # Obstructed labor due to pelvic inlet contraction (disorder)
        "199764006",  # Obstructed labor due to pelvic outlet and mid-cavity contraction (disorder)
        "199765007",  # Obstructed labor due to abnormality of maternal pelvic organs (disorder)
        "199767004",  # Obstructed labor caused by pelvic soft tissues (disorder)
        "199770000",  # Obstructed labor caused by pelvic soft tissues with antenatal problem (disorder)
        "199775005",  # Deep transverse arrest with antenatal problem (disorder)
        "199784005",  # Shoulder dystocia with antenatal problem (disorder)
        "199806003",  # Obstructed labor due to unusually large fetus (disorder)
        "199821009",  # Primary uterine inertia with antenatal problem (disorder)
        "199825000",  # Secondary uterine inertia with antenatal problem (disorder)
        "199839000",  # Hypertonic uterine inertia with antenatal problem (disorder)
        "199848005",  # Prolonged first stage with antenatal problem (disorder)
        "199858009",  # Prolonged second stage with antenatal problem (disorder)
        "199863008",  # Delayed delivery second twin with antenatal problem (disorder)
        "199875003",  # Umbilical cord tight around neck with antenatal problem (disorder)
        "199896005",  # Vasa previa with antenatal problem (disorder)
        "199917001",  # First degree perineal tear during delivery with postnatal problem (disorder)
        "199926003",  # Second degree perineal tear during delivery with postnatal problem (disorder)
        "199931001",  # Third degree perineal tear during delivery with postnatal problem (disorder)
        "199935005",  # Fourth degree perineal tear during delivery with postnatal problem (disorder)
        "199961009",  # Rupture of uterus before labor with antenatal problem (disorder)
        "199972004",  # Laceration of cervix due to childbirth (disorder)
        "199975002",  # Obstetric laceration of cervix with postnatal problem (disorder)
        "199977005",  # Obstetric high vaginal laceration (disorder)
        "199980006",  # Obstetric high vaginal laceration with postnatal problem (disorder)
        "199991004",  # Obstetric damage to pelvic joints and ligaments with postnatal problem (disorder)
        "199993001",  # Obstetric trauma causing pelvic hematoma (disorder)
        "199997000",  # Obstetric pelvic hematoma with postnatal problem (disorder)
        "200031006",  # Postpartum coagulation defects with postnatal problem (disorder)
        "200038000",  # Retained placenta with no hemorrhage with postnatal problem (disorder)
        "200046004",  # Complications of anesthesia during labor and delivery (disorder)
        "200054002",  # Obstetric anesthesia with cardiac complications (disorder)
        "200058004",  # Obstetric anesthesia with cardiac complications with antenatal problem (disorder)
        "200059007",  # Obstetric anesthesia with cardiac complications with postnatal problem (disorder)
        "200061003",  # Obstetric anesthesia with central nervous system complications (disorder)
        "200065007",  # Obstetric anesthesia with central nervous system complication with antenatal problem (disorder)
        "200066008",  # Obstetric anesthesia with central nervous system complication with postnatal problem (disorder)
        "200068009",  # Obstetric toxic reaction caused by local anesthesia (disorder)
        "200069001",  # Toxic reaction caused by local anesthesia during pregnancy (disorder)
        "200070000",  # Toxic reaction caused by local anesthesia during puerperium (disorder)
        "200075005",  # Toxic reaction caused by local anesthetic during labor and delivery (disorder)
        "200077002",  # Cardiac complications of anesthesia during labor and delivery (disorder)
        "200101003",  # Maternal distress with antenatal problem (disorder)
        "200102005",  # Maternal distress with postnatal problem (disorder)
        "200113008",  # Maternal hypotension syndrome with antenatal problem (disorder)
        "200114002",  # Maternal hypotension syndrome with postnatal problem (disorder)
        "200118004",  # Post-delivery acute renal failure with postnatal problem (disorder)
        "200125006",  # Infection of obstetric surgical wound (disorder)
        "200130005",  # Forceps delivery - delivered (finding)
        "200133007",  # Delivery by combination of forceps and vacuum extractor (finding)
        "200134001",  # Delivered by mid-cavity forceps with rotation (finding)
        "200138003",  # Vacuum extractor delivery - delivered (finding)
        "200142000",  # Breech extraction - delivered (finding)
        "200144004",  # Deliveries by cesarean (finding)
        "200147006",  # Cesarean section - pregnancy at term (finding)
        "200148001",  # Delivery by elective cesarean section (finding)
        "200149009",  # Delivery by emergency cesarean section (finding)
        "200150009",  # Delivery by cesarean hysterectomy (finding)
        "200151008",  # Cesarean section following previous cesarean section (finding)
        "200154000",  # Deliveries by destructive operation (finding)
        "200164009",  # Exhaustion during labor (disorder)
        "200173001",  # Intrapartum hemorrhage with coagulation defect (disorder)
        "200182007",  # Puerperal endometritis with postnatal complication (disorder)
        "200187001",  # Puerperal salpingitis with postnatal complication (disorder)
        "200191006",  # Puerperal peritonitis with postnatal complication (disorder)
        "200196001",  # Puerperal sepsis with postnatal complication (disorder)
        "200233001",  # Antenatal deep vein thrombosis with antenatal complication (disorder)
        "200249004",  # Hemorrhoids in pregnancy and puerperium (disorder)
        "200255009",  # Hemorrhoids in puerperium (disorder)
        "200259003",  # Cerebral venous thrombosis in pregnancy (disorder)
        "200277008",  # Puerperal pyrexia of unknown origin (disorder)
        "200284000",  # Obstetric pulmonary embolism (disorder)
        "200286003",  # Obstetric air pulmonary embolism (disorder)
        "200290001",  # Obstetric air pulmonary embolism with antenatal complication (disorder)
        "200291002",  # Obstetric air pulmonary embolism with postnatal complication (disorder)
        "200296007",  # Amniotic fluid pulmonary embolism with antenatal complication (disorder)
        "200297003",  # Amniotic fluid pulmonary embolism with postnatal complication (disorder)
        "200299000",  # Obstetric blood-clot pulmonary embolism (disorder)
        "200303005",  # Obstetric blood-clot pulmonary embolism with antenatal complication (disorder)
        "200304004",  # Obstetric blood-clot pulmonary embolism with postnatal complication (disorder)
        "200310004",  # Obstetric pyemic and septic pulmonary embolism with antenatal complication (disorder)
        "200311000",  # Obstetric pyemic and septic pulmonary embolism with postnatal complication (disorder)
        "200332008",  # Puerperal cerebrovascular disorder with antenatal complication (disorder)
        "200333003",  # Puerperal cerebrovascular disorder with postnatal complication (disorder)
        "200338007",  # Cesarean wound disruption with postnatal complication (disorder)
        "200343000",  # Obstetric perineal wound disruption with postnatal complication (disorder)
        "200352009",  # Placental polyp with postnatal complication (disorder)
        "200364001",  # Obstetric breast infections (disorder)
        "200369006",  # Obstetric nipple infection with antenatal complication (disorder)
        "200370007",  # Obstetric nipple infection with postnatal complication (disorder)
        "200376001",  # Obstetric breast abscess with antenatal complication (disorder)
        "200377005",  # Obstetric breast abscess with postnatal complication (disorder)
        "200383008",  # Obstetric non-purulent mastitis with antenatal complication (disorder)
        "200385001",  # Obstetric non-purulent mastitis with postnatal complication (disorder)
        "200444007",  # Galactorrhea in pregnancy and the puerperium (disorder)
        "200448005",  # Galactorrhea in pregnancy and the puerperium with antenatal complication (disorder)
        "200449002",  # Galactorrhea in pregnancy and the puerperium with postnatal complication (disorder)
        "201134009",  # Alopecia of pregnancy (disorder)
        "20236002",  # Labor established (finding)
        "20259008",  # Finding of ballottement of fetal parts (finding)
        "20272009",  # Premature birth of multiple newborns (finding)
        "20391007",  # Amniotic cyst (disorder)
        "204470001",  # Single umbilical artery (disorder)
        "206191000",  # Local intracranial subdural hematoma due to birth trauma (disorder)
        "206242003",  # Liver subcapsular hematoma due to birth trauma (disorder)
        "206244002",  # Vulval hematoma due to birth trauma (disorder)
        "20625004",  # Obstruction caused by position of fetus at onset of labor (disorder)
        "206365006",  # Clostridial intra-amniotic fetal infection (disorder)
        "206366007",  # Escherichia coli intra-amniotic fetal infection (disorder)
        "206538000",  # Idiopathic hydrops fetalis (disorder)
        "20753005",  # Hypertensive heart disease complicating AND/OR reason for care during pregnancy (disorder)
        "20845005",  # Meconium in amniotic fluid affecting management of mother (disorder)
        "20932005",  # Puerperal salpingitis (disorder)
        "21127004",  # Term birth of newborn male (finding)
        "21243004",  # Term birth of newborn (finding)
        "21334005",  # Failed attempted abortion with oliguria (disorder)
        "21346009",  # Double uterus affecting pregnancy (disorder)
        "21360006",  # Miscarriage with afibrinogenemia (disorder)
        "21737000",  # Retained fetal tissue (disorder)
        "21882006",  # Face presentation of fetus (disorder)
        "21987001",  # Delayed delivery of second of multiple births (disorder)
        "22173004",  # Excessive fetal growth affecting management of mother (disorder)
        "22178008",  # Postpartum state, 8 weeks (finding)
        "22271007",  # Abnormal fetal heart beat first noted during labor AND/OR delivery in liveborn infant (disorder)
        "22281000119101",  # Post-term pregnancy of 40 to 42 weeks (finding)
        "22288000",  # Tumor of cervix affecting pregnancy (disorder)
        "223003",  # Tumor of body of uterus affecting pregnancy (disorder)
        "2239004",  # Previous pregnancies 6 (finding)
        "22399000",  # Puerperal endometritis (disorder)
        "22514005",  # Term birth of fraternal twins, one living, one stillborn (finding)
        "2256007",  # Monozygotic twins (finding)
        "2261009",  # Obstetrical pulmonary fat embolism (disorder)
        "22753004",  # Fetal AND/OR placental disorder affecting management of mother (disorder)
        "22758008",  # Spalding-Horner sign (finding)
        "22846003",  # Hepatorenal syndrome following delivery (disorder)
        "22879003",  # Previous pregnancies 2 (finding)
        "22966008",  # Hypertensive heart AND renal disease complicating AND/OR reason for care during pregnancy (disorder)
        "23001005",  # Stenosis AND/OR stricture of cervix affecting pregnancy (disorder)
        "23037000",  # Parity ten or more (finding)
        "23128002",  # Cervical dilatation, 5cm (finding)
        "23171006",  # Delayed AND/OR secondary postpartum hemorrhage (disorder)
        "23177005",  # Crepitus uteri (finding)
        "23332002",  # Failed trial of labor (disorder)
        "234058009",  # Varicose veins of genitalia in pregnancy (disorder)
        "23431000119106",  # Rupture of uterus during labor (disorder)
        "234380002",  # Kell isoimmunization of the newborn (disorder)
        "23464008",  # Gestation period, 20 weeks (finding)
        "23508005",  # Uterine incoordination, first degree (disorder)
        "235888006",  # Cholestasis of pregnancy (disorder)
        "23652008",  # Cervical dilatation, 3cm (finding)
        "23667007",  # Term pregnancy delivered (finding)
        "237036002",  # Habitual aborter-not pregnant (finding)
        "23717007",  # Benign essential hypertension complicating AND/OR reason for care during pregnancy (disorder)
        "237186006",  # Stenosis of cervix in pregnancy, childbirth and the puerperium (disorder)
        "237205002",  # Rectocele affecting obstetric care (disorder)
        "237208000",  # Rectocele in pregnancy, childbirth and puerperium (disorder)
        "237209008",  # Pendulous abdomen in pregnancy, childbirth and/or the puerperium (disorder)
        "237224004",  # Bicornuate uterus affecting obstetric care (disorder)
        "237225003",  # Bicornuate uterus in pregnancy, childbirth and the puerperium (disorder)
        "237227006",  # Congenital heart disease in pregnancy (disorder)
        "237230004",  # Uremia in pregnancy without hypertension (disorder)
        "237233002",  # Concealed pregnancy (finding)
        "237234008",  # Undiagnosed pregnancy (finding)
        "237235009",  # Undiagnosed breech (disorder)
        "237236005",  # Undiagnosed multiple pregnancy (disorder)
        "237237001",  # Undiagnosed twin pregnancy (disorder)
        "237238006",  # Pregnancy with uncertain dates (finding)
        "237239003",  # Low risk pregnancy (finding)
        "237240001",  # Teenage pregnancy (finding)
        "237241002",  # Viable pregnancy (finding)
        "237242009",  # Non-viable pregnancy (finding)
        "237243004",  # Biochemical pregnancy (finding)
        "237245006",  # Vanishing twin syndrome (disorder)
        "237247003",  # Continuing pregnancy after abortion of sibling fetus (disorder)
        "237249000",  # Complete hydatidiform mole (disorder)
        "237250000",  # Incomplete hydatidiform mole (disorder)
        "237252008",  # Placental site trophoblastic tumor (disorder)
        "237253003",  # Viable fetus in abdominal pregnancy (disorder)
        "237254009",  # Unruptured tubal pregnancy (disorder)
        "237256006",  # Disorder of pelvic size and disproportion (disorder)
        "237257002",  # Midpelvic contraction (disorder)
        "237259004",  # Variation of placental position (disorder)
        "237261008",  # Long umbilical cord (disorder)
        "237264000",  # Spontaneous hindwater rupture of membranes (finding)
        "237265004",  # Spontaneous forewater rupture of membranes (finding)
        "237267007",  # Prolonged premature rupture of membranes (disorder)
        "237268002",  # Chorioangioma (disorder)
        "237270006",  # Antepartum hemorrhage with hypofibrinogenemia (disorder)
        "237271005",  # Antepartum hemorrhage with hyperfibrinolysis (disorder)
        "237272003",  # Antepartum hemorrhage with afibrinogenemia (disorder)
        "237273008",  # Revealed placental abruption (disorder)
        "237274002",  # Concealed placental abruption (disorder)
        "237275001",  # Mixed placental abruption (disorder)
        "237276000",  # Indeterminate antepartum hemorrhage (disorder)
        "237277009",  # Marginal placental hemorrhage (disorder)
        "237279007",  # Transient hypertension of pregnancy (disorder)
        "237281009",  # Moderate proteinuric hypertension of pregnancy (disorder)
        "237282002",  # Impending eclampsia (disorder)
        "237283007",  # Eclampsia in labor (disorder)
        "237284001",  # Symptomatic disorders in pregnancy (disorder)
        "237285000",  # Gestational edema (disorder)
        "237286004",  # Varicose veins of vulva in pregnancy (disorder)
        "237292005",  # Placental insufficiency (disorder)
        "237296008",  # Morbidly adherent placenta (disorder)
        "237297004",  # Placenta adherent to previous uterine scar (disorder)
        "237298009",  # Maternofetal transfusion (disorder)
        "237300009",  # Pregnancy with isoimmunization (disorder)
        "237302001",  # Kell isoimmunization in pregnancy (disorder)
        "237303006",  # Duffy isoimmunization in pregnancy (disorder)
        "237304000",  # Lewis isoimmunization in pregnancy (disorder)
        "237305004",  # Presentation of cord (disorder)
        "237306003",  # Cord beside head (disorder)
        "237307007",  # Umbilical cord around body (disorder)
        "237309005",  # Knot in umbilical cord (disorder)
        "237310000",  # Traumatic injury of umbilical cord (disorder)
        "237313003",  # Vaginal delivery following previous cesarean section (finding)
        "237319004",  # Failure to progress in labor (finding)
        "237320005",  # Failure to progress in first stage of labor (finding)
        "237321009",  # Delayed delivery of triplet (disorder)
        "237324001",  # Cervical dystocia (disorder)
        "237325000",  # Head entrapment during breech delivery (disorder)
        "237327008",  # Obstetric pelvic ligament damage (disorder)
        "237328003",  # Obstetric pelvic joint damage (disorder)
        "237329006",  # Urethra injury - obstetric (disorder)
        "237336007",  # Fibrinolysis - postpartum (disorder)
        "237337003",  # Afibrinogenemia - postpartum (disorder)
        "237338008",  # Blood dyscrasia puerperal (disorder)
        "237342006",  # Fat embolism - obstetric (disorder)
        "237343001",  # Phlebitis in puerperium (disorder)
        "237348005",  # Puerperal pyrexia (disorder)
        "237349002",  # Mild postnatal depression (disorder)
        "237350002",  # Severe postnatal depression (disorder)
        "237351003",  # Mild postnatal psychosis (disorder)
        "237352005",  # Severe postnatal psychosis (disorder)
        "237355007",  # Milk engorgement of breast (disorder)
        "237357004",  # Vascular engorgement of breast (disorder)
        "237364002",  # Stillbirth (finding)
        "237365001",  # Fresh stillbirth (finding)
        "237366000",  # Macerated stillbirth (finding)
        "237506002",  # Thyrotoxicosis in pregnancy (disorder)
        "237507006",  # Human chorionic gonadotrophin-induced thyrotoxicosis (disorder)
        "23786008",  # Malignant hypertension complicating AND/OR reason for care during puerperium (disorder)
        "238613007",  # Generalized pustular psoriasis of pregnancy (disorder)
        "238820002",  # Erythema multiforme of pregnancy (disorder)
        "23885003",  # Inversion of uterus during delivery (disorder)
        "239101008",  # Pregnancy eruption (disorder)
        "239102001",  # Pruritus of pregnancy (disorder)
        "239103006",  # Prurigo of pregnancy (disorder)
        "239104000",  # Pruritic folliculitis of pregnancy (disorder)
        "240154002",  # Idiopathic osteoporosis in pregnancy (disorder)
        "240160002",  # Transient osteoporosis of hip in pregnancy (disorder)
        "240167004",  # Osteomalacia secondary to pregnancy (disorder)
        "24042004",  # Chronic hypertension complicating AND/OR reason for care during puerperium (disorder)
        "24095001",  # Placenta previa partialis (disorder)
        "24146004",  # Premature birth of newborn quintuplets (finding)
        "24258008",  # Damage to pelvic joints AND/OR ligaments during delivery (disorder)
        "2437000",  # Placenta circumvallata (disorder)
        "243826008",  # Antenatal care status (finding)
        "243827004",  # Antenatal RhD antibody status (finding)
        "243828009",  # Alpha-fetoprotein blood test status (finding)
        "243829001",  # Rubella screening status (finding)
        "24444009",  # Failed attempted abortion with sepsis (disorder)
        "24699006",  # Arrested active phase of labor (disorder)
        "248896000",  # Products of conception in vagina (finding)
        "248937008",  # Products of conception at uterine os cervix (finding)
        "248982007",  # Pregnancy, childbirth and puerperium finding (finding)
        "248985009",  # Presentation of pregnancy (finding)
        "248996001",  # Transversely contracted pelvis (disorder)
        "249004001",  # Pelvic side wall finding (finding)
        "249005000",  # Convergent pelvic side walls (finding)
        "249006004",  # Divergent pelvic side walls (finding)
        "249013004",  # Pregnant abdomen finding (finding)
        "249014005",  # Finding of shape of pregnant abdomen (finding)
        "249017003",  # Pregnant uterus displaced laterally (finding)
        "249018008",  # Irritable uterus (finding)
        "249020006",  # Cervical observation during pregnancy and labor (finding)
        "249032005",  # Finding of speed of delivery (finding)
        "249037004",  # Caul membrane over baby's head at delivery (finding)
        "249064003",  # Oblique lie, head in iliac fossa (disorder)
        "249065002",  # Oblique lie, breech in iliac fossa (disorder)
        "249073006",  # Sutures touching without overlapping (finding)
        "249074000",  # Reducible suture overlap (finding)
        "249075004",  # Fixed suture overlap (finding)
        "249089009",  # Fetal mouth presenting (disorder)
        "249090000",  # Fetal ear presenting (disorder)
        "249091001",  # Fetal nose presenting (disorder)
        "249093003",  # Presentation of orbital ridges of fetus (disorder)
        "249098007",  # Knee presentation (disorder)
        "249099004",  # Single knee presentation (disorder)
        "249100007",  # Double knee presentation (disorder)
        "249104003",  # Dorsoanterior shoulder presentation (disorder)
        "249105002",  # Dorsoposterior shoulder presentation (disorder)
        "249122000",  # Baby overdue (finding)
        "249125003",  # Intact membranes (finding)
        "249127006",  # Intact membranes bulging through cervix (finding)
        "249128001",  # Hourglass membranes (finding)
        "249131000",  # Finding of amniotic fluid volume (finding)
        "249132007",  # Change in quantity of liquor (finding)
        "249134008",  # Bloodstained liquor (finding)
        "249136005",  # Fresh meconium staining liquor (finding)
        "249137001",  # Old meconium staining liquor (finding)
        "249139003",  # Thick meconium draining with liquor (finding)
        "249141002",  # Labor not established (finding)
        "249142009",  # Slow progress in first stage of labor (finding)
        "249144005",  # Rapid first stage of labor (finding)
        "249145006",  # Uterine observation in labor (finding)
        "249146007",  # Segments of uterus distinguishable in labor (finding)
        "249148008",  # Hypertonic lower uterine segment during labor (finding)
        "249149000",  # Contraction of uterus during labor (finding)
        "249150000",  # Transmission of uterine contraction wave (finding)
        "249151001",  # Reversal of uterine contraction wave (finding)
        "249161008",  # Maternal effort during second stage of labor (finding)
        "249162001",  # Desire to push in labor (finding)
        "249166003",  # Failure to progress in second stage of labor (finding)
        "249170006",  # Complete placenta at delivery (finding)
        "249172003",  # Finding of consistency of placenta (finding)
        "249173008",  # Placenta gritty (finding)
        "249174002",  # Placenta calcified (disorder)
        "249177009",  # Retroplacental clot (disorder)
        "249179007",  # Condition of membranes at delivery (finding)
        "249180005",  # Membranes absent (finding)
        "249181009",  # Membranes complete (finding)
        "249182002",  # Membranes incomplete (finding)
        "249186004",  # Blood clots in membranes (finding)
        "249189006",  # Wharton's jelly excessive (disorder)
        "249195007",  # Maternal condition during labor (finding)
        "249196008",  # Distress from pain in labor (disorder)
        "249202006",  # Flaccid uterus after delivery (finding)
        "249203001",  # Loss of fundal mass after delivery (finding)
        "249205008",  # Placental fragments in uterus (finding)
        "249206009",  # Placental fragments at cervical os (finding)
        "249207000",  # Membrane at cervical os (finding)
        "249211006",  # Lochia finding (finding)
        "249213009",  # Finding of consistency of lochia (finding)
        "249215002",  # Odor of lochia (finding)
        "249218000",  # Postpartum vulval hematoma (disorder)
        "249219008",  # Genital tear resulting from childbirth (disorder)
        "249220002",  # Vaginal tear resulting from childbirth (disorder)
        "25026004",  # Gestation period, 18 weeks (finding)
        "25032009",  # Failed attempted abortion with laceration of cervix (disorder)
        "25053000",  # Obstetrical central nervous system complication of anesthesia AND/OR sedation (disorder)
        "25113000",  # Chorea gravidarum (disorder)
        "251665008",  # Obstetric investigative finding (finding)
        "25192009",  # Premature birth of fraternal twins, both living (finding)
        "25585008",  # Placenta percreta (disorder)
        "25749005",  # Disproportion between fetus and pelvis (disorder)
        "25922000",  # Major depressive disorder, single episode with postpartum onset (disorder)
        "26078007",  # Hypertension secondary to renal disease complicating AND/OR reason for care during childbirth (disorder)
        "26158002",  # Uterine inertia (disorder)
        "26224003",  # Failed attempted abortion complicated by genital-pelvic infection (disorder)
        "26358008",  # Ectopic placenta (disorder)
        "26611006",  # Para 7 (finding)
        "26623000",  # Failed attempted termination of pregnancy complicated by delayed and/or excessive hemorrhage (disorder)
        "26690008",  # Gestation period, 8 weeks (finding)
        "267197003",  # Antepartum hemorrhage due to abruptio placentae and placenta previa (disorder)
        "267199000",  # Antepartum hemorrhage with coagulation defect (disorder)
        "267204006",  # Genitourinary tract infection in pregnancy (disorder)
        "267207004",  # Congenital cardiovascular disorders during pregnancy, childbirth and the puerperium (disorder)
        "267212003",  # Congenital abnormality of uterus in pregnancy, childbirth and the puerperium (disorder)
        "267213008",  # Congenital abnormality of uterus affecting obstetric care (disorder)
        "267257007",  # Labor and delivery complicated by fetal heart rate anomaly (disorder)
        "267262008",  # Prolonged spontaneous rupture of membranes (disorder)
        "267265005",  # Hypertonic uterine inertia (disorder)
        "267268007",  # Trauma to perineum and/or vulva during delivery (disorder)
        "267269004",  # Vulval and perineal hematoma during delivery (disorder)
        "267272006",  # Postpartum coagulation defects (disorder)
        "267276009",  # Obstetric anesthesia with pulmonary complications (disorder)
        "267278005",  # Deliveries by vacuum extractor (finding)
        "267280004",  # Venous complication of pregnancy and/or puerperium (disorder)
        "267282007",  # Varicose veins of perineum and vulva in pregnancy and the puerperium (disorder)
        "267284008",  # Obstetric pyemic and septic pulmonary embolism (disorder)
        "267289003",  # Obstetric nipple infection (disorder)
        "267317007",  # Tuberculosis in pregnancy (disorder)
        "267320004",  # Pregnancy with mental disorders (disorder)
        "267322007",  # Pregnancy with orthopedic problem (disorder)
        "267340006",  # Maternal pyrexia in labor (disorder)
        "267350007",  # Obstetric anesthesia problems (disorder)
        "26828006",  # Rectocele affecting pregnancy (disorder)
        "268479002",  # Incomplete placenta at delivery (finding)
        "268585006",  # Placental infarct (disorder)
        "26886008",  # Para 6 (finding)
        "268863005",  # Had umbilical cord around neck (disorder)
        "268864004",  # Had knot in umbilical cord (disorder)
        "270498000",  # Malposition and malpresentation of fetus (disorder)
        "270500004",  # Prolapsed cord (disorder)
        "270502007",  # Obstetric breast abscess (disorder)
        "27068000",  # Failed attempted abortion with afibrinogenemia (disorder)
        "27075004",  # Venereal disease in mother complicating pregnancy, childbirth AND/OR puerperium (disorder)
        "271368004",  # Delivered by low forceps delivery (finding)
        "271369007",  # Delivered by mid-cavity forceps delivery (finding)
        "271370008",  # Deliveries by breech extraction (finding)
        "271372000",  # Obstetric non-purulent mastitis (disorder)
        "271373005",  # Deliveries by spontaneous breech delivery (finding)
        "27152008",  # Hyperemesis gravidarum before end of 22 week gestation with carbohydrate depletion (disorder)
        "271693006",  # Pelvis adequate for delivery (finding)
        "271954000",  # Failed attempted medical abortion (disorder)
        "27214003",  # Atonic postpartum hemorrhage (disorder)
        "27215002",  # Inversion of uterus (disorder)
        "27342004",  # Anemia of pregnancy (disorder)
        "27388005",  # Intrapartum hemorrhage due to partial placenta previa (disorder)
        "273982004",  # Obstetric problem affecting fetus (disorder)
        "273983009",  # Maternal cord or placental problem (disorder)
        "273984003",  # Delivery problem for fetus (finding)
        "274118001",  # Venereal disease in pregnancy (disorder)
        "274119009",  # Rubella in pregnancy (disorder)
        "274120003",  # Thyroid disease in pregnancy (disorder)
        "274121004",  # Cardiac disease in pregnancy (disorder)
        "274122006",  # Gravid uterus - retroverted (finding)
        "274124007",  # Grand multipara in labor (finding)
        "274125008",  # Elderly primiparous with labor (finding)
        "274127000",  # Abnormal delivery (finding)
        "274128005",  # Brow delivery (finding)
        "274129002",  # Face delivery (finding)
        "274131006",  # Mastitis - obstetric (disorder)
        "275306006",  # Postnatal data (finding)
        "275412000",  # Cystitis of pregnancy (disorder)
        "275421004",  # Miscarriage with heavy bleeding (disorder)
        "275425008",  # Retained products after miscarriage (disorder)
        "275426009",  # Pelvic disproportion (disorder)
        "275429002",  # Delayed delivery of second twin (disorder)
        "275433009",  # Lymphangitis of breast - obstetric (disorder)
        "275434003",  # Stroke in the puerperium (disorder)
        "27574007",  # Fertilization in vivo (finding)
        "276367008",  # Wanted pregnancy (finding)
        "276375002",  # Offensive lochia (finding)
        "276508000",  # Hydrops fetalis (disorder)
        "276509008",  # Non-immune hydrops fetalis (disorder)
        "276580005",  # Atypical isoimmunization of newborn (disorder)
        "276641008",  # Intrauterine asphyxia (disorder)
        "276642001",  # Antepartum fetal asphyxia (disorder)
        "276881003",  # Secondary abdominal pregnancy (disorder)
        "27696007",  # True knot of umbilical cord (disorder)
        "278056007",  # Uneffaced cervix (finding)
        "278058008",  # Partially effaced cervix (finding)
        "278072004",  # Lochia rubra (finding)
        "278087009",  # Concave sacral curve (finding)
        "278088004",  # Flat sacral curve (finding)
        "278092006",  # Amniotic fluid volume within reference range (finding)
        "278094007",  # Rapid rate of delivery (finding)
        "278095008",  # Normal rate of delivery (finding)
        "278096009",  # Slow rate of delivery (finding)
        "279225001",  # Maternity blues (disorder)
        "28030000",  # Twin birth (finding)
        "280732008",  # Obstetric disorder of uterus (disorder)
        "28079008",  # Multiparous (finding)
        "281050002",  # Livebirth (finding)
        "281052005",  # Triplet birth (finding)
        "281307002",  # Uncertain viability of pregnancy (finding)
        "281687006",  # Face to pubes birth (finding)
        "282020008",  # Premature delivery (finding)
        "284075002",  # Spotting per vagina in pregnancy (finding)
        "2858002",  # Puerperal sepsis (disorder)
        "28701003",  # Low maternal weight gain (finding)
        "28773001",  # Thrombosis of vessels of cord (disorder)
        "288209009",  # Normal delivery - occipitoanterior (finding)
        "288210004",  # Abnormal head presentation delivery (finding)
        "288250001",  # Maternal hypertension (disorder)
        "288261009",  # Was malpresentation pre-labor (disorder)
        "288265000",  # Born after precipitate delivery (finding)
        "288266004",  # Born after induced labor (finding)
        "289080009",  # Able to perform breast-feeding (finding)
        "289081008",  # Unable to perform breast-feeding (finding)
        "289083006",  # Does not perform breast-feeding (finding)
        "289084000",  # Difficulty performing breast-feeding (finding)
        "289086003",  # Able to position baby at breast for feeding (finding)
        "289087007",  # Unable to position baby at breast for feeding (finding)
        "289088002",  # Does position baby at breast for feeding (finding)
        "289089005",  # Does not position baby at breast for feeding (finding)
        "289090001",  # Difficulty positioning baby at breast for feeding (finding)
        "28911003",  # Adherent placenta (disorder)
        "289203002",  # Finding of pattern of pregnancy (finding)
        "289205009",  # Finding of measures of pregnancy (finding)
        "289208006",  # Finding of viability of pregnancy (finding)
        "289210008",  # Finding of first stage of labor (finding)
        "289211007",  # First stage of labor established (finding)
        "289212000",  # First stage of labor not established (finding)
        "289213005",  # Normal length of first stage of labor (finding)
        "289214004",  # Normal first stage of labor (finding)
        "289215003",  # First stage of labor problem (finding)
        "289216002",  # Finding of second stage of labor (finding)
        "289217006",  # Second stage of labor established (finding)
        "289218001",  # Second stage of labor not established (finding)
        "289219009",  # Rapid second stage of labor (finding)
        "289220003",  # Progressing well in second stage (finding)
        "289221004",  # Normal length of second stage of labor (finding)
        "289222006",  # Second stage of labor problem (finding)
        "289223001",  # Normal second stage of labor (finding)
        "289224007",  # Finding of delivery push in labor (finding)
        "289226009",  # Pushing effectively in labor (finding)
        "289227000",  # Not pushing well in labor (finding)
        "289228005",  # Urge to push in labor (finding)
        "289229002",  # Reluctant to push in labor (finding)
        "289230007",  # Pushing voluntarily in labor (finding)
        "289231006",  # Pushing involuntarily in labor (finding)
        "289232004",  # Finding of third stage of labor (finding)
        "289233009",  # Normal length of third stage of labor (finding)
        "289234003",  # Prolonged third stage of labor (finding)
        "289235002",  # Rapid expulsion of placenta (finding)
        "289236001",  # Delayed expulsion of placenta (finding)
        "289237005",  # Normal rate of expulsion of placenta (finding)
        "289238000",  # Finding of pattern of labor (finding)
        "289239008",  # Finding of duration of labor (finding)
        "289240005",  # Long duration of labor (finding)
        "289241009",  # Short duration of labor (finding)
        "289242002",  # Finding of blood loss in labor (finding)
        "289243007",  # Maternal blood loss minimal (finding)
        "289244001",  # Maternal blood loss within normal limits (finding)
        "289245000",  # Maternal blood loss moderate (finding)
        "289246004",  # Maternal blood loss heavy (finding)
        "289247008",  # Finding of measures of labor (finding)
        "289252003",  # Estimated maternal blood loss (finding)
        "289253008",  # Device-associated finding of labor (finding)
        "289254002",  # Expulsion of intrauterine contraceptive device during third stage of labor (finding)
        "289256000",  # Mother delivered (finding)
        "289257009",  # Mother not delivered (finding)
        "289259007",  # Vaginal delivery (finding)
        "289261003",  # Delivery problem (finding)
        "289262005",  # Finding of structures of conception (finding)
        "289267004",  # Finding of completeness of placenta (finding)
        "289269001",  # Lesion of placenta (disorder)
        "289270000",  # Fresh retroplacental clot (disorder)
        "289271001",  # Old retroplacental clot (disorder)
        "289272008",  # Infectious disease of placenta (disorder)
        "289273003",  # Placenta fatty deposits (disorder)
        "289275005",  # Placenta offensive odor (finding)
        "289276006",  # Finding of color of placenta (finding)
        "289277002",  # Placenta pale (finding)
        "289279004",  # Placenta healthy (finding)
        "289280001",  # Placenta unhealthy (finding)
        "289282009",  # Amniotic membrane finding (finding)
        "289283004",  # Amniotic membranes present (finding)
        "289285006",  # Finding of completeness of membranes (finding)
        "289286007",  # Finding of number of amniotic membranes (finding)
        "289287003",  # Two placental membranes present (finding)
        "289288008",  # One placental membrane present (finding)
        "289289000",  # Lesion of amniotic membrane (disorder)
        "289290009",  # Finding of measures of amniotic membranes (finding)
        "289291008",  # Amniotic membranes normal (finding)
        "289293006",  # Amniotic fluid volume above reference range (finding)
        "289294000",  # Thick meconium stained liquor (finding)
        "289295004",  # Finding of fluid thrill of amniotic fluid (finding)
        "289296003",  # Fluid thrill of amniotic fluid present (finding)
        "289297007",  # Fluid thrill of amniotic fluid absent (finding)
        "289299005",  # Umbilical cord finding (finding)
        "289301003",  # Umbilical cord thin (finding)
        "289302005",  # Umbilical cord thick (finding)
        "289303000",  # Finding of pulsation of cord (finding)
        "289305007",  # Pulsation of umbilical cord palpable through intact membrane (finding)
        "289306008",  # Pulsation of umbilical cord not palpable through intact membranes (finding)
        "289307004",  # Finding of pulsation of prolapsed cord (finding)
        "289308009",  # Pulsation present in prolapsed cord (finding)
        "289309001",  # Pulsation absent in prolapsed cord (finding)
        "289311005",  # Umbilical cord tight around baby's neck (disorder)
        "289312003",  # Umbilical cord entangled around baby's limbs (disorder)
        "289313008",  # Umbilical cord not around baby's neck at delivery (finding)
        "289315001",  # Umbilical cord normal (finding)
        "289349009",  # Presenting part ballottable (finding)
        "289350009",  # Ballottement of fetal head abdominally (finding)
        "289351008",  # Ballottement of fetal head at fundus (finding)
        "289352001",  # Ballottement of fetal head in suprapubic area (finding)
        "289353006",  # Ballottement of fetal head vaginally (finding)
        "289354000",  # Presenting part not ballottable (finding)
        "289355004",  # Oblique lie head in right iliac fossa (disorder)
        "289356003",  # Oblique lie head in left iliac fossa (disorder)
        "289357007",  # Oblique lie breech in right iliac fossa (disorder)
        "289358002",  # Oblique lie breech in left iliac fossa (disorder)
        "289451007",  # Finding of pubic arch (finding)
        "289452000",  # Small pubic arch (finding)
        "289453005",  # Narrow pubic arch (finding)
        "289454004",  # Prominent ischial spines (finding)
        "289455003",  # Finding of sacrosciatic notch (finding)
        "289456002",  # Wide sacrosciatic notch (finding)
        "289457006",  # Narrow sacrosciatic notch (finding)
        "289458001",  # Rounded sacrosciatic notch (finding)
        "289463002",  # Pelvis not adequate for delivery (finding)
        "289464008",  # Problem of pelvis for delivery (finding)
        "289568008",  # Finding of vaginal liquor (finding)
        "289570004",  # Scanty flow of amniotic liquor (finding)
        "289571000",  # Profuse flow of amniotic liquor (finding)
        "289572007",  # No liquor observed vaginally (finding)
        "289573002",  # Finding of passing of operculum (finding)
        "289574008",  # Operculum passed (finding)
        "289575009",  # Operculum not passed (finding)
        "289576005",  # Lochia present (finding)
        "289577001",  # Lochia reducing (finding)
        "289578006",  # Lochia absent (finding)
        "289579003",  # Lochia ceased (finding)
        "289580000",  # Lochia heavy (finding)
        "289581001",  # Lochia scanty (finding)
        "289582008",  # Lochia minimal (finding)
        "289583003",  # Lochia watery (finding)
        "289584009",  # Lochia thick (finding)
        "289585005",  # Lochia normal (finding)
        "289586006",  # Lochia abnormal (finding)
        "289605002",  # Vaginal liquor pad finding (finding)
        "289606001",  # Maternity pad damp with liquor (finding)
        "289607005",  # Maternity pad wet with liquor (finding)
        "289608000",  # Maternity pad soaked with liquor (finding)
        "289675001",  # Finding of gravid uterus (finding)
        "289676000",  # Gravid uterus present (finding)
        "289677009",  # Gravid uterus absent (finding)
        "289678004",  # Gravid uterus not observed (finding)
        "289679007",  # Finding of size of gravid uterus (finding)
        "289680005",  # Gravid uterus large-for-dates (finding)
        "289681009",  # Gravid uterus small-for-dates (finding)
        "289682002",  # Finding of height of gravid uterus (finding)
        "289683007",  # Fundal height high for dates (finding)
        "289684001",  # Fundal height equal to dates (finding)
        "289685000",  # Fundal height low for dates (finding)
        "289686004",  # Ovoid pregnant abdomen (finding)
        "289687008",  # Rounded pregnant abdomen (finding)
        "289688003",  # Transversely enlarged pregnant abdomen (finding)
        "289689006",  # Finding of arrangement of gravid uterus (finding)
        "289690002",  # Gravid uterus central (finding)
        "289691003",  # Gravid uterus deviated to left (finding)
        "289692005",  # Gravid uterus deviated to right (finding)
        "289693000",  # Normal position of gravid uterus (finding)
        "289694006",  # Finding of sensation of gravid uterus (finding)
        "289695007",  # Tender scar of gravid uterus (disorder)
        "289696008",  # Tender scar of upper uterine segment (disorder)
        "289697004",  # Tender scar of lower uterine segment (disorder)
        "289699001",  # Finding of uterine contractions (finding)
        "289700000",  # Uterine contractions present (finding)
        "289701001",  # Uterine contractions absent (finding)
        "289702008",  # Uterine contractions ceased (finding)
        "289703003",  # Finding of pattern of uterine contractions (finding)
        "289704009",  # Finding of frequency of uterine contraction (finding)
        "289705005",  # Intermittent uterine contractions (finding)
        "289706006",  # Occasional uterine tightenings (finding)
        "289707002",  # Finding of regularity of uterine contraction (finding)
        "289708007",  # Regular uterine contractions (finding)
        "289709004",  # Finding of quantity of uterine contraction (finding)
        "289710009",  # Excessive uterine contraction (finding)
        "289711008",  # Reduced uterine contraction (finding)
        "289712001",  # Niggling uterine contractions (finding)
        "289714000",  # Finding of strength of uterine contraction (finding)
        "289715004",  # Mild uterine contractions (finding)
        "289716003",  # Moderate uterine contractions (finding)
        "289717007",  # Fair uterine contractions (finding)
        "289718002",  # Good uterine contractions (finding)
        "289719005",  # Strong uterine contractions (finding)
        "289720004",  # Very strong uterine contractions (finding)
        "289721000",  # Variable strength uterine contractions (finding)
        "289722007",  # Normal strength uterine contractions (finding)
        "289723002",  # Finding of duration of uterine contraction (finding)
        "289724008",  # Continuous contractions (finding)
        "289725009",  # Hypertonic contractions (finding)
        "289726005",  # Finding of effectiveness of uterine contraction (finding)
        "289727001",  # Effective uterine contractions (finding)
        "289728006",  # Non-effective uterine contractions (finding)
        "289729003",  # Expulsive uterine contractions (finding)
        "289730008",  # Finding of painfulness of uterine contraction (finding)
        "289731007",  # Painless uterine contractions (finding)
        "289732000",  # Painful uterine contractions (finding)
        "289733005",  # Premature uterine contraction (finding)
        "289734004",  # Delayed uterine contraction (finding)
        "289735003",  # Persistent uterine contraction (finding)
        "289736002",  # Inappropriate uterine contraction (finding)
        "289737006",  # Finding of measures of uterine contractions (finding)
        "289738001",  # Uterine contractions normal (finding)
        "289739009",  # Uterine contractions problem (finding)
        "289740006",  # Finding of contraction state of uterus (finding)
        "289741005",  # Uterus contracted (finding)
        "289742003",  # Uterus relaxed (finding)
        "289743008",  # Finding of upper segment retraction (finding)
        "289744002",  # Poor retraction of upper segment (finding)
        "289747009",  # Finding of measures of gravid uterus (finding)
        "289748004",  # Gravid uterus normal (finding)
        "289749007",  # Gravid uterus problem (finding)
        "289750007",  # Finding of involution of uterus (finding)
        "289751006",  # Uterus involuting (finding)
        "289752004",  # Uterus involuted (finding)
        "289753009",  # No further involution of the uterus (finding)
        "289761004",  # Finding of cervical dilatation (finding)
        "289762006",  # Cervix dilated (finding)
        "289763001",  # Rim of cervix palpable (finding)
        "289764007",  # Cervix undilated (finding)
        "289765008",  # Ripe cervix (finding)
        "289766009",  # Finding of thickness of cervix (finding)
        "289768005",  # Cervix thick (finding)
        "289770001",  # Cervix thin (finding)
        "289783000",  # Cervix central (finding)
        "289827009",  # Finding of cervical cerclage suture (finding)
        "289828004",  # Cervical cerclage suture absent (finding)
        "28996007",  # Stillbirth of premature female (1000-2499 gms.) (finding)
        "290653008",  # Postpartum hypopituitarism (disorder)
        "29123003",  # Postpartum state, 3 weeks (finding)
        "291665000",  # Postpartum intrapituitary hemorrhage (disorder)
        "29171003",  # Trapped placenta (disorder)
        "29259002",  # Malignant hypertension complicating AND/OR reason for care during pregnancy (disorder)
        "29399001",  # Elderly primigravida (finding)
        "29421008",  # Failed attempted abortion complicated by renal failure (disorder)
        "29847008",  # Obstetrical complication of anesthesia AND/OR sedation (disorder)
        "29851005",  # Halo sign (finding)
        "29997008",  # Premature birth of newborn triplets (finding)
        "300571009",  # Finding of gestational sac (finding)
        "300572002",  # Gestational sac present (finding)
        "300573007",  # Gestational sac absent (finding)
        "300927001",  # Episiotomy infection (disorder)
        "30118000",  # Postpartum period, 1 day (finding)
        "30165006",  # Premature birth of fraternal twins, one living, one stillborn (finding)
        "301801008",  # Finding of position of pregnancy (finding)
        "302078000",  # Abnormal immature chorionic villi (disorder)
        "302080006",  # Finding of birth outcome (finding)
        "302253005",  # Delivered by cesarean section - pregnancy at term (finding)
        "302254004",  # Delivered by cesarean delivery following previous cesarean delivery (finding)
        "302644007",  # Abnormal immature chorion (disorder)
        "302929008",  # Umbilical cord around neck (disorder)
        "303063000",  # Eclampsia in puerperium (disorder)
        "3033000",  # Bone AND/OR joint disorder of pelvis in mother complicating pregnancy, childbirth AND/OR puerperium (disorder)
        "304598004",  # Postnatal examination finding (finding)
        "30653008",  # Previous pregnancies 5 (finding)
        "307337003",  # Duffy isoimmunization of the newborn (disorder)
        "307338008",  # Kidd isoimmunization of the newborn (disorder)
        "307517001",  # Pregnancy-related factor VIII deficiency (disorder)
        "307534009",  # Urinary tract infection in pregnancy (disorder)
        "307632004",  # Non-proteinuric hypertension of pregnancy (disorder)
        "307733001",  # Inevitable abortion complete (disorder)
        "307734007",  # Complete inevitable abortion complicated by genital tract and pelvic infection (disorder)
        "307735008",  # Complete inevitable abortion complicated by delayed or excessive hemorrhage (disorder)
        "307737000",  # Inevitable abortion incomplete (disorder)
        "307746006",  # Incomplete inevitable abortion complicated by embolism (disorder)
        "307749004",  # Incomplete inevitable abortion complicated by genital tract and pelvic infection (disorder)
        "307750004",  # Complete inevitable abortion complicated by embolism (disorder)
        "307752007",  # Incomplete inevitable abortion complicated by delayed or excessive hemorrhage (disorder)
        "30806007",  # Miscarriage with endometritis (disorder)
        "308135003",  # Varicose veins of genitalia in puerperium (disorder)
        "308136002",  # Superficial thrombophlebitis in pregnancy (disorder)
        "308137006",  # Superficial thrombophlebitis in puerperium (disorder)
        "308140006",  # Hemorrhoids in pregnancy (disorder)
        "308187004",  # Varicose veins of vagina in puerperium (disorder)
        "309469004",  # Spontaneous vertex delivery (finding)
        "310248000",  # Antenatal care midwifery led (finding)
        "31026002",  # Obstetrical complication of sedation (disorder)
        "31041000119106",  # Obstructed labor due to deep transverse arrest and persistent occipitoposterior position (disorder)
        "31159001",  # Bluish discoloration of cervix (finding)
        "312974005",  # Preterm premature rupture of membranes (disorder)
        "313017000",  # Anhydramnios (disorder)
        "313130001",  # Uterine membrane finding (finding)
        "313178001",  # Gestation less than 24 weeks (finding)
        "313179009",  # Gestation period, 24 weeks (finding)
        "313180007",  # Gestation greater than 24 weeks (finding)
        "313291009",  # Anemia during the puerperium (disorder)
        "31383003",  # Tumor of vagina affecting pregnancy (disorder)
        "31407004",  # Pre-existing hypertension complicating AND/OR reason for care during puerperium (disorder)
        "314204000",  # Early stage of pregnancy (finding)
        "31465005",  # Cervical effacement 80 percent (finding)
        "31481000",  # Vascular anomaly of umbilical cord (disorder)
        "31516002",  # Viral disease in mother complicating pregnancy, childbirth AND/OR puerperium (disorder)
        "31563000",  # Asymptomatic bacteriuria in pregnancy (disorder)
        "31601007",  # Combined pregnancy (disorder)
        "31801000146106",  # Pre-existing maternal disease complicating pregnancy (disorder)
        "31805001",  # Fetal disproportion (disorder)
        "31821006",  # Uterine scar from previous surgery affecting pregnancy (disorder)
        "320003",  # Cervical dilatation, 1cm (finding)
        "32644009",  # Braxton Hicks contractions (finding)
        "32801000119106",  # Infectious disease in mother complicating childbirth (disorder)
        "32999002",  # Complication of obstetrical surgical wound (disorder)
        "33058001",  # Placenta accreta without hemorrhage (disorder)
        "33340004",  # Multiple conception (finding)
        "33348006",  # Previous pregnancies 7 (finding)
        "33370009",  # Hyperemesis gravidarum before end of 22 week gestation with electrolyte imbalance (disorder)
        "33490001",  # Failed attempted abortion with fat embolism (disorder)
        "33552005",  # Anomaly of placenta (disorder)
        "33627001",  # Prolonged first stage of labor (disorder)
        "34089005",  # Premature birth of newborn quadruplets (finding)
        "34100008",  # Premature birth of identical twins, both living (finding)
        "34165000",  # Gestational proteinuria (disorder)
        "34262005",  # Fourth degree perineal laceration involving rectal mucosa (disorder)
        "34270000",  # Miscarriage complicated by shock (disorder)
        "34273003",  # Hypertensive heart disease complicating AND/OR reason for care during puerperium (disorder)
        "34327003",  # Parturient hemorrhage associated with hyperfibrinolysis (disorder)
        "34367002",  # Failed attempted abortion with perforation of bowel (disorder)
        "34478009",  # Failed attempted abortion with defibrination syndrome (disorder)
        "34530006",  # Failed attempted abortion with electrolyte imbalance (disorder)
        "34614007",  # Miscarriage with complication (disorder)
        "34694006",  # Pre-existing hypertension complicating AND/OR reason for care during childbirth (disorder)
        "34801009",  # Ectopic pregnancy (disorder)
        "34842007",  # Antepartum hemorrhage (disorder)
        "34981006",  # Hypertonic uterine dysfunction (disorder)
        "35303009",  # Benign essential hypertension complicating AND/OR reason for care during puerperium (disorder)
        "35347003",  # Delayed delivery after artificial rupture of membranes (disorder)
        "35381000119101",  # Quadruplet pregnancy with loss of one or more fetuses (disorder)
        "35537000",  # Ladin's sign (finding)
        "35608009",  # Stillbirth of immature male (500-999 gms.) (finding)
        "35656003",  # Intraligamentous pregnancy (disorder)
        "35716006",  # Cracked nipple associated with childbirth (disorder)
        "35746009",  # Uterine fibroids affecting pregnancy (disorder)
        "35874009",  # Normal labor (finding)
        "35882009",  # Abnormality of forces of labor (disorder)
        "35999006",  # Blighted ovum (disorder)
        "361095003",  # Dehiscence AND/OR disruption of uterine wound in the puerperium (disorder)
        "36144008",  # McClintock's sign (finding)
        "36297009",  # Septate vagina affecting pregnancy (disorder)
        "363681007",  # Pregnancy with abortive outcome (disorder)
        "36428009",  # Gestation period, 42 weeks (finding)
        "364826005",  # Finding related to ability to perform breast-feeding (finding)
        "364827001",  # Finding related to ability to position baby at breast for feeding (finding)
        "36497006",  # Traumatic extension of episiotomy (disorder)
        "365662002",  # Finding of amniotic fluid appearance (finding)
        "36614009",  # Parity eight (finding)
        "366298006",  # Finding of color of lochia (finding)
        "366299003",  # Finding of quantity of lochia (finding)
        "366309007",  # Finding of effacement of cervix (finding)
        "366321006",  # Finding of gravida (finding)
        "366323009",  # Finding of length of gestation (finding)
        "366325002",  # Finding of progress of labor - first stage (finding)
        "366327005",  # Finding of progress of second stage of labor (finding)
        "366328000",  # Finding related to ability to push in labor (finding)
        "366329008",  # Finding of speed of delivery of placenta (finding)
        "366332006",  # Finding of odor of placenta (finding)
        "366333001",  # Finding of number of membranes (finding)
        "366334007",  # Finding of condition of amniotic fluid (finding)
        "366335008",  # Finding of consistency of liquor (finding)
        "366339002",  # Finding of general shape of pelvis (finding)
        "366340000",  # Finding of shape of sacral curve (finding)
        "366341001",  # Finding of position of ischial spines (finding)
        "366343003",  # Finding of birth details (finding)
        "366344009",  # Finding of place of birth (finding)
        "366345005",  # Finding of antenatal data (finding)
        "36664002",  # Failed attempted abortion with air embolism (disorder)
        "36697001",  # False knot of umbilical cord (disorder)
        "367390009",  # Hypertension in the obstetric context (disorder)
        "367494004",  # Premature birth of newborn (finding)
        "36798000",  # Cervical effacement 10 percent (finding)
        "36801000119105",  # Continuing triplet pregnancy after spontaneous abortion of one or more fetuses (disorder)
        "36813001",  # Placenta previa (disorder)
        "36854009",  # Inlet contraction of pelvis (disorder)
        "36916008",  # Bone AND/OR joint disorder of back in mother complicating pregnancy, childbirth AND/OR puerperium (disorder)
        "37005007",  # Gestation period, 5 weeks (finding)
        "371106008",  # Idiopathic maternal thrombocytopenia (disorder)
        "371374003",  # Retained products of conception (disorder)
        "371375002",  # Retained secundines (disorder)
        "371380006",  # Amniotic fluid leaking (disorder)
        "371614003",  # Hematoma of obstetric wound (disorder)
        "372043009",  # Hydrops of allantois (disorder)
        "372046001",  # Glucose tolerance test during pregnancy, childbirth and puerperium outside reference range (finding)
        "372048000",  # Pregnancy with abnormal glucose tolerance test (finding)
        "373663005",  # Perinatal period (finding)
        "373680009",  # Avulsion of umbilical cord (disorder)
        "373896005",  # Miscarriage complicated by genital-pelvic infection (disorder)
        "37618003",  # Chronic hypertension complicating AND/OR reason for care during pregnancy (disorder)
        "3798002",  # Premature birth of identical twins, both stillborn (finding)
        "38010008",  # Intrapartum hemorrhage (disorder)
        "38039008",  # Gestation period, 10 weeks (finding)
        "38099005",  # Failed attempted abortion with endometritis (disorder)
        "38250004",  # Arrested labor (finding)
        "38257001",  # Term birth of identical twins, one living, one stillborn (finding)
        "38451000119105",  # Failed attempted vaginal birth after previous cesarean section (disorder)
        "38720006",  # Septuplet pregnancy (finding)
        "387692004",  # Hypotonic uterine inertia (disorder)
        "387696001",  # Primary hypotonic uterine dysfunction (disorder)
        "387699008",  # Primary uterine inertia (disorder)
        "387700009",  # Prolonged latent phase of labor (disorder)
        "38802008",  # Postpartum state, 5 weeks (finding)
        "3885002",  # ABO isoimmunization affecting pregnancy (disorder)
        "388604008",  # Villitis (disorder)
        "388605009",  # Disorder of chorion (disorder)
        "38951007",  # Failed attempted abortion with laceration of bowel (disorder)
        "390959009",  # Live birth surviving more than one year (finding)
        "39101000119109",  # Pregnancy with isoimmunization from irregular blood group incompatibility (disorder)
        "39120007",  # Failed attempted abortion with salpingo-oophoritis (disorder)
        "39121000119100",  # Pelvic mass in pregnancy (disorder)
        "39199000",  # Miscarriage with laceration of urinary bladder (disorder)
        "39208009",  # Fetal hydrops causing disproportion (disorder)
        "39213008",  # Premature birth of identical twins, one living, one stillborn (finding)
        "39249009",  # Placentitis (disorder)
        "39333005",  # Cervical dilatation, 8cm (finding)
        "3944006",  # Placental sulfatase deficiency (X-linked steryl-sulfatase deficiency) in a female (disorder)
        "3950001",  # Birth (finding)
        "396544001",  # Cesarean wound disruption (disorder)
        "39727004",  # Hypertension secondary to renal disease complicating AND/OR reason for care during puerperium (disorder)
        "39763004",  # Abnormal glucose tolerance in mother complicating pregnancy, childbirth AND/OR puerperium (disorder)
        "397752008",  # Obstetric perineal wound disruption (disorder)
        "397949005",  # Poor fetal growth affecting management (disorder)
        "397952002",  # Trauma to perineum during delivery (disorder)
        "398019008",  # Perineal laceration during delivery (disorder)
        "398254007",  # Pre-eclampsia (disorder)
        "398262004",  # Disruption of episiotomy wound in the puerperium (disorder)
        "398276003",  # Placental insufficiency affecting management of mother (disorder)
        "399031001",  # Fourth degree perineal laceration (disorder)
        "399363000",  # Late fetal death affecting management of mother (disorder)
        "400170001",  # Hypocalcemia of puerperium (disorder)
        "4006006",  # Fetal tachycardia affecting management of mother (disorder)
        "40156002",  # Postpartum state, 7 weeks (finding)
        "4026005",  # Interstitial mastitis associated with childbirth (disorder)
        "402836009",  # Spider telangiectasis in association with pregnancy (disorder)
        "403528000",  # Pregnancy-related exacerbation of dermatosis (disorder)
        "40444006",  # Miscarriage with acute necrosis of liver (disorder)
        "40511000119107",  # Postpartum pre-existing essential hypertension (disorder)
        "40521000119100",  # Postpartum pregnancy-induced hypertension (disorder)
        "405733001",  # Hypocalcemia of late pregnancy and/or lactation (disorder)
        "40609001",  # Infectious disease in mother complicating pregnancy, childbirth AND/OR puerperium (disorder)
        "40691000119104",  # Infectious disease of mother complicating pregnancy (disorder)
        "40791000119105",  # Postpartum gestational diabetes mellitus (disorder)
        "40801000119106",  # Gestational diabetes mellitus complicating pregnancy (disorder)
        "408781009",  # Straw colored amniotic fluid (finding)
        "408782002",  # Purulent amniotic fluid (finding)
        "408783007",  # Antenatal Anti-D prophylaxis status (finding)
        "408792005",  # Meconium stained liquor - grade I (finding)
        "408793000",  # Meconium stained liquor - grade II (finding)
        "408794006",  # Meconium stained liquor - grade III (finding)
        "408796008",  # Stillbirth - unknown if fetal death intrapartum or prior to labor (finding)
        "408823004",  # Antenatal hepatitis B blood screening test status (finding)
        "408825006",  # Antenatal hepatitis B blood screening test sent (finding)
        "408827003",  # Antenatal human immunodeficiency virus blood screening test status (finding)
        "408828008",  # Antenatal human immunodeficiency virus blood screening test sent (finding)
        "408831009",  # Antenatal thalassemia blood screening test status (finding)
        "408833007",  # Antenatal thalassemia blood screening test sent (finding)
        "408838003",  # Unplanned home birth (finding)
        "408839006",  # Planned home birth (finding)
        "408842000",  # Antenatal human immunodeficiency virus blood screening test requested (finding)
        "408843005",  # Antenatal thalassemia blood screening test requested (finding)
        "41114007",  # Mild pre-eclampsia (disorder)
        "41215002",  # Congenital abnormality of uterus, affecting pregnancy (disorder)
        "413338003",  # Incomplete miscarriage with complication (disorder)
        "413567003",  # Aplastic anemia associated with pregnancy (disorder)
        "41438001",  # Gestation period, 21 weeks (finding)
        "415105001",  # Placental abruption (disorder)
        "415810004",  # Varix of umbilical vein (disorder)
        "41587001",  # Third trimester pregnancy (finding)
        "416402001",  # Gestational trophoblastic disease (disorder)
        "416413003",  # Advanced maternal age gravida (finding)
        "416669000",  # Invasive hydatidiform mole (disorder)
        "417006004",  # Twin reversal arterial perfusion syndrome (disorder)
        "417044008",  # Hydatidiform mole, benign (disorder)
        "417150000",  # Exaggerated placental site (disorder)
        "417364008",  # Placental site nodule (disorder)
        "4174008",  # Placenta tripartita (disorder)
        "417426005",  # Four vessels in umbilical cord (disorder)
        "417570003",  # Gestational choriocarcinoma (disorder)
        "41962002",  # Oligohydramnios sequence (disorder)
        "41991004",  # Angiectasis pregnancy (disorder)
        "42170009",  # Amniotic fluid abnormal (finding)
        "42279001",  # Para 4 (finding)
        "423445003",  # Difficulty following prenatal diet (finding)
        "423502007",  # Difficulty with postpartum rest pattern (finding)
        "423675002",  # Difficulty following postpartum exercise routine (finding)
        "423834007",  # Difficulty with prenatal rest pattern (finding)
        "424009001",  # Fears delivery procedure (finding)
        "424037008",  # Difficulty following prenatal exercise routine (finding)
        "424474002",  # Difficulty coping with postpartum changes (finding)
        "42466007",  # Cervical effacement 70 percent (finding)
        "424712007",  # Difficulty following postpartum diet (finding)
        "42537006",  # Secondary perineal tear in the puerperium (disorder)
        "42553009",  # Deficiency of placental barrier function (disorder)
        "425708006",  # Placental aromatase deficiency (disorder)
        "42571002",  # Failed induction of labor (disorder)
        "42599006",  # Trauma from instrument during delivery (disorder)
        "426403007",  # Late entry into prenatal care (finding)
        "42686001",  # Chromosomal abnormality in fetus affecting obstetrical care (disorder)
        "426997005",  # Traumatic injury during pregnancy (disorder)
        "427013000",  # Alcohol consumption during pregnancy (finding)
        "427139004",  # Third trimester bleeding (disorder)
        "42718004",  # Cervical effacement 90 percent (finding)
        "42783002",  # Spontaneous placental expulsion, Schultz mechanism (finding)
        "428017002",  # Condyloma acuminata of vulva in pregnancy (disorder)
        "428058009",  # Gestation less than 9 weeks (finding)
        "42814007",  # Mid postpartum state (finding)
        "428164004",  # Mitral valve disorder in pregnancy (disorder)
        "428165003",  # Hypothyroidism in pregnancy (disorder)
        "428230005",  # Trichomonal vaginitis in pregnancy (disorder)
        "428252001",  # Vaginitis in pregnancy (disorder)
        "428508008",  # Abortion due to Brucella abortus (disorder)
        "428511009",  # Multiple pregnancy with one fetal loss (disorder)
        "428566005",  # Gestation less than 20 weeks (finding)
        "428567001",  # Gestation 14 - 20 weeks (finding)
        "42857002",  # Second stage of labor (finding)
        "428707000",  # Nutritional deficiency in pregnancy (disorder)
        "428930004",  # Gestation 9- 13 weeks (finding)
        "429187001",  # Continuing pregnancy after intrauterine death of twin fetus (disorder)
        "429240000",  # Third trimester pregnancy less than 36 weeks (finding)
        "429715006",  # Gestation greater than 20 weeks (finding)
        "430881000",  # Second trimester bleeding (disorder)
        "430933008",  # Gravid uterus size for dates discrepancy (finding)
        "43195002",  # Tenney changes of placenta (disorder)
        "43293004",  # Failed attempted abortion with septic embolism (disorder)
        "43306002",  # Miscarriage complicated by embolism (disorder)
        "433601000124106",  # Gestation period greater than or equal to 37 weeks (finding)
        "43651009",  # Acquired stenosis of vagina affecting pregnancy (disorder)
        "43673002",  # Fetal souffle (finding)
        "43697006",  # Gestation period, 37 weeks (finding)
        "43715006",  # Secondary uterine inertia (disorder)
        "43808007",  # Cervical dilatation, 4cm (finding)
        "439311009",  # Intends to continue pregnancy (finding)
        "43970002",  # Congenital stenosis of vagina affecting pregnancy (disorder)
        "43990006",  # Sextuplet pregnancy (finding)
        "441697004",  # Thrombophilia associated with pregnancy (disorder)
        "441924001",  # Gestational age unknown (finding)
        "44216000",  # Retained products of conception, following delivery with hemorrhage (disorder)
        "44223004",  # Premature rupture of membranes (disorder)
        "442478007",  # Multiple pregnancy involving intrauterine pregnancy and tubal pregnancy (disorder)
        "443006",  # Cystocele affecting pregnancy (disorder)
        "443460007",  # Multigravida of advanced maternal age (finding)
        "44398003",  # Gestation period, 4 weeks (finding)
        "44464006",  # Postpartum AND/OR puerperal erysipelas (disorder)
        "444661007",  # High risk pregnancy due to history of preterm labor (finding)
        "445086005",  # Chemical pregnancy (disorder)
        "445122007",  # Low lying placenta (finding)
        "445149007",  # Residual trophoblastic disease (disorder)
        "445548006",  # Dead fetus in utero (finding)
        "44622003",  # Congenital hypoplasia of umbilical artery (disorder)
        "44640004",  # Failed attempted abortion with parametritis (disorder)
        "44795003",  # Rhesus isoimmunization affecting pregnancy (disorder)
        "44814008",  # Miscarriage with laceration of periurethral tissue (disorder)
        "449169008",  # Subchorionic intervillositis (disorder)
        "44979007",  # Carneous mole (disorder)
        "449807005",  # Type 3a third degree laceration of perineum (disorder)
        "449808000",  # Type 3b third degree laceration of perineum (disorder)
        "449809008",  # Type 3c third degree laceration of perineum (disorder)
        "449827006",  # Lochia alba (finding)
        "449828001",  # Lochia serosa (finding)
        "44992005",  # Failed attempted abortion with intravascular hemolysis (disorder)
        "450852006",  # Yolk sac present (finding)
        "45139008",  # Gestation period, 29 weeks (finding)
        "451881000124101",  # Tobacco use in mother complicating puerperium (finding)
        "451891000124103",  # Tobacco use in mother complicating pregnancy (finding)
        "451901000124104",  # Tobacco smoking in mother complicating puerperium (finding)
        "451911000124101",  # Tobacco smoking in mother complicating childbirth (finding)
        "451921000124109",  # Tobacco smoking in mother complicating pregnancy (finding)
        "45307008",  # Extrachorial pregnancy (finding)
        "45384004",  # Multiple birth (finding)
        "45757002",  # Labor problem (finding)
        "45759004",  # Rigid perineum affecting pregnancy (disorder)
        "457811000124103",  # Normal pregnancy in primigravida (finding)
        "457821000124106",  # Normal pregnancy in multigravida (finding)
        "45828008",  # Anemia in mother complicating pregnancy, childbirth AND/OR puerperium (disorder)
        "459166009",  # Dichorionic diamniotic twin pregnancy (finding)
        "459167000",  # Monochorionic twin pregnancy (finding)
        "459168005",  # Monochorionic diamniotic twin pregnancy (finding)
        "459169002",  # Monochorionic diamniotic twin pregnancy with similar amniotic fluid volumes (finding)
        "459170001",  # Monochorionic diamniotic twin pregnancy with dissimilar amniotic fluid volumes (disorder)
        "459171002",  # Monochorionic monoamniotic twin pregnancy (finding)
        "46022004",  # Bearing down reflex (finding)
        "46230007",  # Gestation period, 40 weeks (finding)
        "46273003",  # Abscess of nipple associated with childbirth (disorder)
        "46311005",  # Perineal laceration involving fourchette (disorder)
        "46365005",  # Failed attempted abortion with perforation of periurethral tissue (disorder)
        "46502006",  # Hematoma of vagina during delivery (disorder)
        "46764007",  # Severe pre-eclampsia (disorder)
        "46894009",  # Gestational diabetes mellitus class A2 (disorder)
        "46906003",  # Gestation period, 27 weeks (finding)
        "46914009",  # Intact fetal membranes (finding)
        "47161002",  # Failed attempted abortion with perforation of cervix (disorder)
        "47200007",  # High risk pregnancy (finding)
        "472321009",  # Continuing pregnancy after intrauterine death of one twin with intrauterine retention of dead twin (disorder)
        "47236005",  # Third stage hemorrhage (disorder)
        "472836003",  # Yolk sac absent (finding)
        "47537002",  # Miscarriage with postoperative shock (disorder)
        "47574003",  # Immature cervix (finding)
        "47821001",  # Postpartum hemorrhage (disorder)
        "4783006",  # Maternal diabetes mellitus with hypoglycemia affecting fetus OR newborn (disorder)
        "48194001",  # Pregnancy-induced hypertension (disorder)
        "48200009",  # Bone AND/OR joint disorder of lower extremities in mother complicating pregnancy, childbirth AND/OR puerperium (disorder)
        "48395006",  # Para 2 (finding)
        "48485000",  # Miscarriage in third trimester (disorder)
        "48552006",  # Hypertension secondary to renal disease complicating AND/OR reason for care during pregnancy (disorder)
        "48688005",  # Gestation period, 26 weeks (finding)
        "48739004",  # Miscarriage with cardiac arrest and/or cardiac failure (disorder)
        "48782003",  # Delivery normal (finding)
        "4886009",  # Premature birth of newborn male (finding)
        "48975005",  # Stimulated labor (finding)
        "4907004",  # Non-involution of uterus (disorder)
        "49071005",  # Postpartum period, 2 days (finding)
        "49102001",  # Hypertensive heart AND renal disease in obstetric context (disorder)
        "49171007",  # Hypertensive renal disease complicating AND/OR reason for care during childbirth (disorder)
        "49177006",  # Postpartum coagulation defect with hemorrhage (disorder)
        "49279000",  # Slow slope active phase of labor (disorder)
        "49342001",  # Stricture of vagina affecting pregnancy (disorder)
        "49364005",  # Subareolar abscess associated with childbirth (disorder)
        "49416000",  # Failed attempted abortion (disorder)
        "49561003",  # Rupture of gravid uterus before onset of labor (disorder)
        "4961000124100",  # White or yellow-green tinged lochia (finding)
        "4971000124107",  # Red or brown-green tinged lochia (finding)
        "49794002",  # Auscultatory sign (finding)
        "49815007",  # Damage to coccyx during delivery (disorder)
        "49955008",  # Para 3 (finding)
        "49956009",  # Antepartum deep phlebothrombosis (disorder)
        "49964003",  # Ectopic fetus (disorder)
        "50120008",  # Abnormal acid-base balance affecting management of mother (disorder)
        "50367001",  # Gestation period, 11 weeks (finding)
        "50404009",  # Postpartum state, 9 weeks (finding)
        "50557007",  # Healed pelvic floor repair affecting pregnancy (disorder)
        "50726009",  # Failed attempted abortion with perforation of uterus (disorder)
        "50758004",  # Term birth of newborn twins (finding)
        "50764006",  # Vomiting due to organic disease during pregnancy (disorder)
        "50770000",  # Miscarriage with defibrination syndrome (disorder)
        "50844007",  # Failed attempted abortion with pelvic peritonitis (disorder)
        "51154004",  # Obstetrical pulmonary complication of anesthesia AND/OR sedation (disorder)
        "51195001",  # Placental polyp (disorder)
        "51495008",  # Cerebral anoxia following anesthesia AND/OR sedation in labor AND/OR delivery (disorder)
        "51519001",  # Marginal insertion of umbilical cord (disorder)
        "51885006",  # Morning sickness (finding)
        "51954003",  # Miscarriage with perforation of periurethral tissue (disorder)
        "521000119104",  # Acute cystitis in pregnancy, antepartum (disorder)
        "52327008",  # Fetal disproportion due to fetal myelomeningocele (disorder)
        "52483005",  # Term birth of newborn sextuplets (finding)
        "52588004",  # Robert's sign (finding)
        "52772002",  # Postpartum thyroiditis (disorder)
        "52942000",  # Term birth of stillborn twins (finding)
        "53024001",  # Insufficient weight gain of pregnancy (disorder)
        "53111003",  # Failed attempted abortion with postoperative shock (disorder)
        "53183006",  # Miscarriage with intravascular hemolysis (disorder)
        "53419009",  # Cord entanglement (disorder)
        "53443007",  # Prolonged labor (disorder)
        "5361000124103",  # Red-green tinged lochia (finding)
        "53638009",  # Therapeutic abortion (disorder)
        "53881005",  # Gravida 0 (finding)
        "54048003",  # Threatened abortion (disorder)
        "541000119105",  # Hypertension complicating pregnancy, childbirth and the puerperium, antepartum (disorder)
        "54212005",  # Irregular uterine contractions (disorder)
        "54213000",  # Oligohydramnios without rupture of membranes (disorder)
        "54318006",  # Gestation period, 19 weeks (finding)
        "54559004",  # Uterine souffle (finding)
        "54650005",  # Premature birth of stillborn twins (finding)
        "54679003",  # Traumatic injury of urinary bladder during obstetric procedure (disorder)
        "54844002",  # Prolapse of gravid uterus (disorder)
        "55187005",  # Tarnier's sign (finding)
        "55466006",  # Missed labor (finding)
        "55472006",  # Submammary abscess associated with childbirth (disorder)
        "55543007",  # Previous pregnancies 3 (finding)
        "55565007",  # Cardiac failure after obstetrical surgery AND/OR other procedure including delivery (disorder)
        "55581002",  # Meconium in amniotic fluid first noted during labor AND/OR delivery in liveborn infant (disorder)
        "55613002",  # Engorgement of breasts associated with childbirth (disorder)
        "55639004",  # Failed attempted abortion with laceration of vagina (disorder)
        "55704005",  # Abscess of breast, associated with childbirth (disorder)
        "55861007",  # Postpartum period, 5 days (finding)
        "55976003",  # Miscarriage with blood-clot embolism (disorder)
        "56026007",  # Late postpartum state (finding)
        "56160003",  # Hydrorrhea gravidarum (finding)
        "56272000",  # Deep venous thrombosis in puerperium (disorder)
        "56313000",  # Abnormal placenta affecting management of mother (disorder)
        "56425003",  # Edema of placenta (disorder)
        "56451001",  # Failed attempted abortion with perforation of broad ligament (disorder)
        "56462001",  # Failed attempted abortion with soap embolism (disorder)
        "56624009",  # Cervical effacement 20 percent (finding)
        "57220000",  # Obstetrical complication of local anesthesia (disorder)
        "57296000",  # Incarcerated gravid uterus (disorder)
        "5740008",  # Pelvic hematoma during delivery (disorder)
        "57420002",  # Listeria abortion (disorder)
        "57469000",  # Miscarriage with acute renal failure (disorder)
        "57576007",  # Retroversion of gravid uterus (disorder)
        "57630001",  # First trimester pregnancy (finding)
        "57759005",  # First degree perineal laceration (disorder)
        "57797005",  # Induced termination of pregnancy (disorder)
        "57873008",  # Hypertensive renal disease complicating AND/OR reason for care during puerperium (disorder)
        "57907009",  # Gestation period, 36 weeks (finding)
        "58123006",  # Failed attempted abortion with pulmonary embolism (disorder)
        "58289000",  # Prodromal stage labor (finding)
        "58532003",  # Unwanted pregnancy (finding)
        "58699001",  # Cervical dilatation, 2cm (finding)
        "58703003",  # Postpartum depression (disorder)
        "58881007",  # Polyp of cervix affecting pregnancy (disorder)
        "58990004",  # Miscarriage complicated by damage to pelvic organs and/or tissues (disorder)
        "59136001",  # Thyroid disease in mother complicating pregnancy, childbirth AND/OR puerperium (disorder)
        "59204004",  # Miscarriage with septic embolism (disorder)
        "59291004",  # Undelivered in utero fetal death (finding)
        "59363009",  # Inevitable abortion (disorder)
        "5939002",  # Failed attempted abortion complicated by metabolic disorder (disorder)
        "59403008",  # Premature birth of newborn female (finding)
        "59466002",  # Second trimester pregnancy (finding)
        "59566000",  # Oligohydramnios (disorder)
        "59733002",  # Hypertensive heart disease complicating AND/OR reason for care during childbirth (disorder)
        "59919008",  # Failed attempted abortion with complication (disorder)
        "59932003",  # Cervical effacement 50 percent (finding)
        "60000008",  # Mesometric pregnancy (disorder)
        "60001007",  # Not pregnant (finding)
        "60265009",  # Miscarriage with air embolism (disorder)
        "60328005",  # Previous pregnancies 4 (finding)
        "60401000119104",  # Postpartum psychosis in remission (disorder)
        "60574005",  # Twin placenta (disorder)
        "60601008",  # Obstetrical blood clot embolism (disorder)
        "60755004",  # Persistent hymen affecting pregnancy (disorder)
        "60810003",  # Quadruplet pregnancy (finding)
        "609133009",  # Short cervical length in pregnancy (finding)
        "609204004",  # Subchorionic hematoma (disorder)
        "609443003",  # Retained products of conception following induced termination of pregnancy (disorder)
        "609447002",  # Induced termination of pregnancy complicated by damage to pelvic organs and/or tissues (disorder)
        "609449004",  # Induced termination of pregnancy complicated by embolism (disorder)
        "609450004",  # Induced termination of pregnancy complicated by genital-pelvic infection (disorder)
        "609451000",  # Induced termination of pregnancy complicated by metabolic disorder (disorder)
        "609452007",  # Induced termination of pregnancy complicated by renal failure (disorder)
        "609453002",  # Induced termination of pregnancy complicated by shock (disorder)
        "609454008",  # Induced termination of pregnancy complicated by acute necrosis of liver (disorder)
        "609455009",  # Induced termination of pregnancy complicated by acute renal failure (disorder)
        "609456005",  # Induced termination of pregnancy complicated by afibrinogenemia (disorder)
        "609457001",  # Induced termination of pregnancy complicated by air embolism (disorder)
        "609458006",  # Induced termination of pregnancy complicated by amniotic fluid embolism (disorder)
        "609459003",  # Induced termination of pregnancy complicated by blood-clot embolism (disorder)
        "609461007",  # Induced termination of pregnancy complicated by cerebral anoxia (disorder)
        "609462000",  # Induced termination of pregnancy complicated by defibrination syndrome (disorder)
        "609463005",  # Induced termination of pregnancy complicated by electrolyte imbalance (disorder)
        "609464004",  # Induced termination of pregnancy complicated by endometritis (disorder)
        "609465003",  # Induced termination of pregnancy complicated by fat embolism (disorder)
        "609466002",  # Induced termination of pregnancy complicated by intravascular hemolysis (disorder)
        "609467006",  # Induced termination of pregnancy complicated by laceration of bowel (disorder)
        "609468001",  # Induced termination of pregnancy complicated by laceration of broad ligament (disorder)
        "609469009",  # Induced termination of pregnancy complicated by laceration of cervix (disorder)
        "609470005",  # Induced termination of pregnancy complicated by laceration of uterus (disorder)
        "609471009",  # Induced termination of pregnancy complicated by laceration of vagina (disorder)
        "609472002",  # Induced termination of pregnancy complicated by acute renal failure with oliguria (disorder)
        "609473007",  # Induced termination of pregnancy complicated by parametritis (disorder)
        "609474001",  # Induced termination of pregnancy complicated by pelvic peritonitis (disorder)
        "609475000",  # Induced termination of pregnancy complicated by perforation of bowel (disorder)
        "609476004",  # Induced termination of pregnancy complicated by perforation of cervix (disorder)
        "609477008",  # Induced termination of pregnancy complicated by perforation of uterus (disorder)
        "609478003",  # Induced termination of pregnancy complicated by perforation of vagina (disorder)
        "609479006",  # Induced termination of pregnancy complicated by postoperative shock (disorder)
        "609480009",  # Induced termination of pregnancy complicated by pulmonary embolism (disorder)
        "609482001",  # Induced termination of pregnancy complicated by renal tubular necrosis (disorder)
        "609483006",  # Induced termination of pregnancy complicated by salpingitis (disorder)
        "609484000",  # Induced termination of pregnancy complicated by salpingo-oophoritis (disorder)
        "609486003",  # Induced termination of pregnancy complicated by septic embolism (disorder)
        "609487007",  # Induced termination of pregnancy complicated by septic shock (disorder)
        "609489005",  # Induced termination of pregnancy complicated by soap embolism (disorder)
        "609490001",  # Induced termination of pregnancy complicated by uremia (disorder)
        "609491002",  # Induced termination of pregnancy complicated by urinary tract infection (disorder)
        "609492009",  # Induced termination of pregnancy without complication (disorder)
        "609493004",  # Induced termination of pregnancy complicated by tetanus (disorder)
        "609494005",  # Induced termination of pregnancy complicated by pelvic disorder (disorder)
        "609496007",  # Complication occurring during pregnancy (disorder)
        "609497003",  # Venous complication of puerperium (disorder)
        "609498008",  # Induced termination of pregnancy complicated by laceration of urinary bladder (disorder)
        "609499000",  # Induced termination of pregnancy complicated by laceration of periurethral tissue (disorder)
        "609500009",  # Induced termination of pregnancy complicated by perforation of urinary bladder (disorder)
        "609501008",  # Induced termination of pregnancy complicated by perforation of broad ligament (disorder)
        "609502001",  # Induced termination of pregnancy complicated by perforation of periurethral tissue (disorder)
        "609503006",  # Induced termination of pregnancy complicated by urinary bladder damage (disorder)
        "609504000",  # Induced termination of pregnancy complicated by bowel damage (disorder)
        "609505004",  # Induced termination of pregnancy complicated by broad ligament damage (disorder)
        "609506003",  # Induced termination of pregnancy complicated by cardiac arrest (disorder)
        "609507007",  # Induced termination of pregnancy complicated by cardiac failure (disorder)
        "609508002",  # Induced termination of pregnancy complicated by cervix damage (disorder)
        "609510000",  # Induced termination of pregnancy complicated by infectious disease (disorder)
        "609511001",  # Induced termination of pregnancy complicated by periurethral tissue damage (disorder)
        "609512008",  # Induced termination of pregnancy complicated by oliguria (disorder)
        "609513003",  # Induced termination of pregnancy complicated by uterus damage (disorder)
        "609514009",  # Induced termination of pregnancy complicated by vaginal damage (disorder)
        "609515005",  # Epithelioid trophoblastic tumor (disorder)
        "609516006",  # Gestational trophoblastic lesion (disorder)
        "609519004",  # Gestational trophoblastic neoplasia (disorder)
        "609522002",  # Disorder of extraembryonic membrane (disorder)
        "609525000",  # Miscarriage of tubal ectopic pregnancy (disorder)
        "61007003",  # Separation of symphysis pubis during delivery (disorder)
        "61452007",  # Failed attempted abortion with laceration of uterus (disorder)
        "61568004",  # Miscarriage with salpingo-oophoritis (disorder)
        "61714007",  # Metabolic disturbance in labor AND/OR delivery (disorder)
        "61881000",  # Osiander's sign (finding)
        "61951009",  # Failed attempted abortion with laceration of periurethral tissue (disorder)
        "62023000",  # Arterial anomaly of umbilical cord (disorder)
        "62098001",  # Head of fetus not engaged (finding)
        "62129004",  # Contraction ring dystocia (disorder)
        "62131008",  # Couvelaire uterus (disorder)
        "62333002",  # Gestation period, 13 weeks (finding)
        "6234006",  # Second degree perineal laceration (disorder)
        "62377009",  # Postpartum cardiomyopathy (disorder)
        "62410004",  # Postpartum fibrinolysis with hemorrhage (disorder)
        "62472004",  # Cervix fully dilated (finding)
        "6251000119101",  # Induced termination of pregnancy in first trimester (disorder)
        "62531004",  # Placenta previa marginalis (disorder)
        "62583006",  # Phlegmasia alba dolens in puerperium (disorder)
        "6261000119104",  # Induced termination of pregnancy in second trimester (disorder)
        "62612003",  # Fibrosis of perineum affecting pregnancy (disorder)
        "62657007",  # Cardiac arrest AND/OR failure following anesthesia AND/OR sedation in labor AND/OR delivery (disorder)
        "62774004",  # Cervical dilatation, 9cm (finding)
        "63110000",  # Gestation period, 7 weeks (finding)
        "63287004",  # Benign essential hypertension in obstetric context (disorder)
        "63503002",  # Gestation period, 41 weeks (finding)
        "63637002",  # Failed attempted abortion with laceration of broad ligament (disorder)
        "63750008",  # Oblique lie (disorder)
        "6383007",  # Premature labor (finding)
        "64171002",  # Obstetrical cardiac complication of anesthesia AND/OR sedation (disorder)
        "64181003",  # Failed attempted abortion with perforation of vagina (disorder)
        "64229006",  # Traumatic lesion during delivery (disorder)
        "64254006",  # Triplet pregnancy (finding)
        "64541000",  # Postpartum period, 3 days (finding)
        "64814003",  # Miscarriage with electrolyte imbalance (disorder)
        "64920003",  # Gestation period, 31 weeks (finding)
        "64954002",  # Avulsion of inner symphyseal cartilage during delivery (disorder)
        "65035007",  # Gestation period, 22 weeks (finding)
        "65147003",  # Twin pregnancy (finding)
        "65336003",  # Cervical effacement 60 percent (finding)
        "65375007",  # Postpartum state, 6 weeks (finding)
        "65377004",  # Polygalactia (disorder)
        "65402008",  # Pre-existing hypertension complicating AND/OR reason for care during pregnancy (disorder)
        "65409004",  # Obstetrical complication of anesthesia (disorder)
        "65483000",  # Placenta of multiple birth higher than twin (finding)
        "65683006",  # Gestation period, 17 weeks (finding)
        "65727000",  # Intrauterine pregnancy (finding)
        "65809005",  # Cervical effacement 40 percent (finding)
        "6594005",  # Cerebrovascular disorder in the puerperium (disorder)
        "66119008",  # Disruption of perineal laceration repair in the puerperium (disorder)
        "66131005",  # Miscarriage with fat embolism (disorder)
        "66294006",  # Obstruction by abnormal pelvic soft tissues (disorder)
        "6636004",  # Congenital cardiovascular disorder in mother complicating pregnancy, childbirth AND/OR puerperium (disorder)
        "66709007",  # Hypertensive renal disease complicating AND/OR reason for care during pregnancy (disorder)
        "6678005",  # Gestation period, 15 weeks (finding)
        "66844003",  # Generalized infection during labor (disorder)
        "66892003",  # Failed attempted abortion with urinary tract infection (disorder)
        "66895001",  # Cervical dilatation, 6cm (finding)
        "66958002",  # Isoimmunization from non-ABO, non-Rh blood-group incompatibility affecting pregnancy (disorder)
        "67042008",  # Failed attempted abortion complicated by shock (disorder)
        "67229002",  # Spontaneous uterine inversion (disorder)
        "67359005",  # Pre-eclampsia added to pre-existing hypertension (disorder)
        "67465009",  # Miscarriage with sepsis (disorder)
        "67486009",  # Pelvic thrombophlebitis in puerperium (disorder)
        "67802002",  # Malpresentation other than breech, successfully converted to cephalic presentation (disorder)
        "67881007",  # Annular placenta (disorder)
        "68214002",  # Lymphangitis of breast associated with childbirth (disorder)
        "6825008",  # Perineal laceration involving rectovaginal septum (disorder)
        "68509000",  # Stillbirth of immature female (500-999 gms.) (finding)
        "68635007",  # Deficiency of placental function (disorder)
        "6867004",  # Previous pregnancies more than 10 (finding)
        "6891008",  # Uterine incoordination, second degree (disorder)
        "6893006",  # First stage of labor (finding)
        "69217004",  # Outlet contraction of pelvis (disorder)
        "69302000",  # Abortion on demand (disorder)
        "69338007",  # Kanter's sign (finding)
        "69777007",  # Interlocked twins (finding)
        "69802008",  # Cervical cerclage suture present (disorder)
        "69840006",  # Normal breast feeding (finding)
        "698638005",  # Pregnancy induced hypertension with pulmonary edema (disorder)
        "698640000",  # Hypertension in the puerperium with pulmonary edema (disorder)
        "698702007",  # Deep transverse arrest with persistent occipitoposterior position (disorder)
        "698708006",  # Antepartum hemorrhage due to placenta previa type I (disorder)
        "698709003",  # Antepartum hemorrhage due to placenta previa type II (disorder)
        "698710008",  # Antepartum hemorrhage due to placenta previa type III (disorder)
        "698711007",  # Antepartum hemorrhage due to placenta previa type IV (disorder)
        "698712000",  # Antepartum hemorrhage due to cervical polyp (disorder)
        "698713005",  # Antepartum hemorrhage due to cervical erosion (disorder)
        "698716002",  # Preterm spontaneous labor with preterm delivery (finding)
        "698717006",  # Preterm spontaneous labor with term delivery (finding)
        "69909000",  # Eclampsia added to pre-existing hypertension (disorder)
        "699240001",  # Combined intrauterine and ovarian pregnancy (disorder)
        "699949009",  # Retained placenta due to morbidly adherent placenta (disorder)
        "699950009",  # Anti-D isoimmunization affecting pregnancy (disorder)
        "700038005",  # Mastitis associated with lactation (disorder)
        "70068004",  # Persistent occipitoposterior position (disorder)
        "70112005",  # Stillbirth of mature female (2500 gms. or more) (finding)
        "70129008",  # Placenta accreta (disorder)
        "70137000",  # Deficiency of placental endocrine function (disorder)
        "702452007",  # Quadruplet birth (finding)
        "702453002",  # Quintuplet birth (finding)
        "702454008",  # Sextuplet birth (finding)
        "702711004",  # Mental disorder in mother complicating pregnancy (disorder)
        "703309000",  # Familial gestational hyperthyroidism (disorder)
        "704175009",  # Acute chorioamnionitis (disorder)
        "70425008",  # Piskacek's sign (finding)
        "70537007",  # Hegar's sign (finding)
        "70603007",  # Rupture of cord (disorder)
        "70651004",  # Calkin's sign (finding)
        "707089008",  # Genital tract infection in puerperium (disorder)
        "707207004",  # Incomplete induced termination of pregnancy (disorder)
        "707254000",  # Amniotic adhesion (disorder)
        "70964000",  # Postparturient hemoglobinuria (disorder)
        "71028008",  # Fetal-maternal hemorrhage (disorder)
        "71096001",  # Inversion of uterine contraction (finding)
        "71151000119105",  # Umbilical cord three times around neck (disorder)
        "712653003",  # Delivery by cesarean section for footling breech presentation (finding)
        "712654009",  # Delivery by cesarean section for breech presentation (finding)
        "712655005",  # Delivery by cesarean section for flexed breech presentation (finding)
        "713187004",  # Polyhydramnios due to maternal disease (disorder)
        "713191009",  # Polyhydramnios due to placental anomaly (disorder)
        "713192002",  # Oligohydramnios due to rupture of membranes (disorder)
        "713202001",  # Antepartum stillbirth (finding)
        "713204000",  # Non immune hydrops in newborn (disorder)
        "713232009",  # Prolonged second stage of labor due to poor maternal effort (disorder)
        "713249002",  # Pyogenic granuloma of gingiva due to pregnancy (disorder)
        "71355009",  # Gestation period, 30 weeks (finding)
        "713575004",  # Dizygotic twin pregnancy (finding)
        "713576003",  # Monozygotic twin pregnancy (finding)
        "71473008",  # Amnion nodosum (disorder)
        "715880002",  # Obstructed labor due to fetal abnormality (disorder)
        "71612002",  # Postpartum uterine hypertrophy (disorder)
        "716379000",  # Acute fatty liver of pregnancy (disorder)
        "71639005",  # Galactorrhea associated with childbirth (disorder)
        "717816002",  # Infection of nipple associated with childbirth with attachment difficulty (disorder)
        "717817006",  # Abscess of breast associated with childbirth with attachment difficulty (disorder)
        "717818001",  # Nonpurulent mastitis associated with childbirth with attachment difficulty (disorder)
        "717819009",  # Retraction of nipple associated with childbirth with attachment difficulty (finding)
        "717820003",  # Cracked nipple associated with childbirth with attachment difficulty (disorder)
        "717959008",  # Cardiac complication of anesthesia during the puerperium (disorder)
        "717960003",  # Central nervous system complication of anesthesia during the puerperium (disorder)
        "71874008",  # Benign essential hypertension complicating AND/OR reason for care during childbirth (disorder)
        "71901000",  # Congenital contracted pelvis (disorder)
        "72014004",  # Abnormal fetal duplication (disorder)
        "72022006",  # Essential hypertension in obstetric context (disorder)
        "721022000",  # Complication of anesthesia during the puerperium (disorder)
        "721153000",  # Congenital anomaly of mother complicating pregnancy (disorder)
        "721166000",  # Human immunodeficiency virus complicating pregnancy childbirth and the puerperium (disorder)
        "721177006",  # Injury complicating pregnancy (disorder)
        "721201001",  # Protozoal disease complicating pregnancy childbirth and the puerperium (disorder)
        "72161000119100",  # Antiphospholipid syndrome in pregnancy (disorder)
        "72271000119100",  # Hyperthyroidism in pregnancy (disorder)
        "72301000119103",  # Asthma in pregnancy (disorder)
        "72321000119107",  # Sickle cell trait in mother complicating pregnancy (disorder)
        "72331000119105",  # Thalassemia in mother complicating pregnancy (disorder)
        "723541004",  # Disease of respiratory system complicating pregnancy (disorder)
        "723665008",  # Vaginal bleeding complicating early pregnancy (disorder)
        "72417002",  # Tumultuous uterine contraction (finding)
        "724483001",  # Concern about body image related to pregnancy (finding)
        "724485008",  # Gestational proteinuria without hypertension (disorder)
        "724486009",  # Venous complication of pregnancy (disorder)
        "724488005",  # Preterm delivery following induction of labor (finding)
        "724489002",  # Preterm delivery following Cesarean section (finding)
        "724490006",  # Intrapartum hemorrhage co-occurrent and due to obstructed labor with uterine rupture (disorder)
        "724491005",  # Vesicovaginal fistula with involvement of urinary continence mechanism following normal delivery (disorder)
        "724492003",  # Vesicovaginal fistula with involvement of urinary continence mechanism due to and following obstructed labor (disorder)
        "724493008",  # Vesicovaginal fistula with involvement of urinary continence mechanism following obstetric delivery procedure (disorder)
        "724494002",  # Vesicovaginal fistula with involvement of urinary continence mechanism following termination of pregnancy procedure (disorder)
        "724496000",  # Postpartum hemorrhage co-occurrent and due to uterine rupture following obstructed labor (disorder)
        "724497009",  # Disease of circulatory system complicating pregnancy childbirth and puerperium (disorder)
        "7245003",  # Fetal dystocia (disorder)
        "72543004",  # Stillbirth of immature fetus, sex undetermined (500-999 gms.) (finding)
        "72544005",  # Gestation period, 25 weeks (finding)
        "72613009",  # Miscarriage with oliguria (disorder)
        "7266006",  # Intrapartum hemorrhage due to total placenta previa (disorder)
        "72771000119107",  # Postpartum episiotomy pain (finding)
        "72846000",  # Gestation period, 14 weeks (finding)
        "72860003",  # Disorder of amniotic cavity AND/OR membrane (disorder)
        "72892002",  # Normal pregnancy (finding)
        "72957006",  # Diamniotic-monochorionic twins (finding)
        "73030000",  # Hypertensive renal disease in obstetric context (disorder)
        "73161006",  # Transverse lie (disorder)
        "733142005",  # Sepsis following obstructed labor (disorder)
        "733839001",  # Postpartum acute renal failure (disorder)
        "734857007",  # Injury of female genital organ due to obstetric procedure (disorder)
        "735492001",  # Obstructed labor due to shoulder dystocia (disorder)
        "73621009",  # Monoamniotic twins (finding)
        "737318003",  # Delayed hemorrhage due to and following miscarriage (disorder)
        "737320000",  # Delayed hemorrhage due to and following molar pregnancy (disorder)
        "737321001",  # Excessive hemorrhage due to and following molar pregnancy (disorder)
        "737322008",  # Delayed hemorrhage due to and following ectopic pregnancy (disorder)
        "737323003",  # Excessive hemorrhage due to and following ectopic pregnancy (disorder)
        "737324009",  # Embolism due to and following ectopic pregnancy (disorder)
        "737325005",  # Embolism due to and following molar pregnancy (disorder)
        "737327002",  # Renal failure following molar pregnancy (disorder)
        "737329004",  # Damage to pelvic organs and tissues following ectopic pregnancy (disorder)
        "737330009",  # Damage to pelvic organs and tissues following molar pregnancy (disorder)
        "737331008",  # Disorder of vein following miscarriage (disorder)
        "737332001",  # Disorder of vein following ectopic pregnancy (disorder)
        "737333006",  # Disorder of vein following molar pregnancy (disorder)
        "73790007",  # Threatened abortion in first trimester (disorder)
        "73837001",  # Failed attempted abortion with cardiac arrest AND/OR failure (disorder)
        "739668007",  # Rupture of uterus due to obstetric procedure (disorder)
        "73972002",  # Postpartum neurosis (disorder)
        "740597009",  # Umbilical cord complication during labor and delivery (disorder)
        "74369005",  # Miscarriage with perforation of cervix (disorder)
        "74437002",  # Ahlfeld's sign (finding)
        "74522004",  # Cervical dilatation, 7cm (finding)
        "74952004",  # Gestation period, 3 weeks (finding)
        "74955002",  # Retraction of nipple associated with childbirth (finding)
        "75022004",  # Gestational diabetes mellitus class A1 (disorder)
        "7504005",  # Trauma to vulva during delivery (disorder)
        "75094005",  # Hydrops of placenta (disorder)
        "75150001",  # Renal disease in pregnancy AND/OR puerperium without hypertension (disorder)
        "75697004",  # Term birth of identical twins, both stillborn (finding)
        "75798003",  # Twin dichorionic diamniotic placenta (disorder)
        "75933004",  # Threatened abortion in second trimester (disorder)
        "76037007",  # Rigid cervix uteri affecting pregnancy (disorder)
        "762612009",  # Quadruplets with all four stillborn (finding)
        "762613004",  # Quintuplets with all five stillborn (finding)
        "762614005",  # Sextuplets with all six stillborn (finding)
        "76380008",  # Placenta membranacea (disorder)
        "765182005",  # Pre-eclampsia in puerperium (disorder)
        "76751001",  # Diabetes mellitus in mother complicating pregnancy, childbirth AND/OR puerperium (disorder)
        "76771005",  # Parturient hemorrhage associated with hypofibrinogenemia (disorder)
        "76871004",  # Previous surgery to vagina affecting pregnancy (disorder)
        "76889003",  # Failed attempted abortion with cerebral anoxia (disorder)
        "769087009",  # Vomiting during third trimester of pregnancy (disorder)
        "7707000",  # Gestation period, 32 weeks (finding)
        "77186001",  # Failed attempted abortion with renal tubular necrosis (disorder)
        "77259008",  # Prolonged second stage of labor (disorder)
        "77278008",  # Velamentous insertion of umbilical cord (disorder)
        "77285007",  # Placental infarct affecting management of mother (disorder)
        "77376005",  # Gestational edema without hypertension (disorder)
        "77386006",  # Pregnancy (finding)
        "77563000",  # Obstruction by bony pelvis (disorder)
        "7768008",  # Failure of cervical dilation (disorder)
        "77814006",  # Stillbirth of premature male (1000-2499 gms.) (finding)
        "77854008",  # Failed medical induction of labor (disorder)
        "7792000",  # Placenta previa without hemorrhage (disorder)
        "7809009",  # Miscarriage with laceration of uterus (disorder)
        "7822001",  # Failed attempted abortion complicated by damage to pelvic organs AND/OR tissues (disorder)
        "78381004",  # Heart disease in mother complicating pregnancy, childbirth AND/OR puerperium (disorder)
        "78395001",  # Gestation period, 33 weeks (finding)
        "78408007",  # Complication of obstetrical surgery AND/OR procedure (disorder)
        "785341006",  # Intrapartum hemorrhage due to leiomyoma (disorder)
        "785867009",  # Excessive hemorrhage due to and following induced termination of pregnancy (disorder)
        "785868004",  # Secondary hemorrhage due to and following induced termination of pregnancy (disorder)
        "7860005",  # Stillbirth of mature male (2500 gms. or more) (finding)
        "786067000",  # Intramural ectopic pregnancy of myometrium (disorder)
        "78697003",  # Nonpurulent mastitis associated with childbirth (disorder)
        "78789000",  # Mature abnormal placenta (disorder)
        "78808002",  # Essential hypertension complicating AND/OR reason for care during pregnancy (disorder)
        "788120007",  # Antenatal depression (disorder)
        "788290007",  # Hemangioma of skin in pregnancy (disorder)
        "788728009",  # Hematoma of surgical wound following cesarean section (disorder)
        "7888004",  # Term birth of newborn quadruplets (finding)
        "7910003",  # Miscarriage with salpingitis (disorder)
        "79114003",  # Fetal head molding (finding)
        "79179003",  # Desultory labor (disorder)
        "79222000",  # Compression of umbilical cord (disorder)
        "79255005",  # Mentum presentation of fetus (disorder)
        "79290002",  # Cervical pregnancy (disorder)
        "79355007",  # Hypertensive heart disease in obstetric context (disorder)
        "79414005",  # Retraction ring dystocia (disorder)
        "79586000",  # Tubal pregnancy (disorder)
        "79668009",  # Vasa previa (disorder)
        "79748007",  # Maternal dystocia (disorder)
        "79839005",  # Perineal laceration involving vulva (disorder)
        "79992004",  # Gestation period, 12 weeks (finding)
        "80002007",  # Malpresentation of fetus (disorder)
        "80113008",  # Complication of the puerperium (disorder)
        "8014007",  # Brow presentation of fetus (disorder)
        "80224003",  # Multiple gestation with one OR more fetal malpresentations (disorder)
        "80228000",  # Lightening of fetus (finding)
        "80256005",  # Intervillous thrombosis (disorder)
        "80487005",  # Gestation period, 39 weeks (finding)
        "8071005",  # Miscarriage with perforation of bowel (disorder)
        "8076000",  # Postmature abnormal placenta (disorder)
        "80818002",  # Previous surgery to perineum AND/OR vulva affecting pregnancy (disorder)
        "80856001",  # Previous pregnancies 8 (finding)
        "80997009",  # Quintuplet pregnancy (finding)
        "81328008",  # Fundal dominance of uterine contraction (finding)
        "81514003",  # Parity nine (finding)
        "81521003",  # Failed attempted abortion with salpingitis (disorder)
        "81557008",  # Parasitic disease in mother complicating pregnancy, childbirth AND/OR puerperium (disorder)
        "816148008",  # Disproportion between fetus and pelvis due to conjoined twins (disorder)
        "81626002",  # Malignant hypertension in obstetric context (disorder)
        "81677009",  # Lactation tetany (disorder)
        "816967008",  # Fetal distress due to augmentation of labor with oxytocin (disorder)
        "816969006",  # Inefficient uterine activity with oxytocin augmentation (disorder)
        "820947007",  # Third stage hemorrhage due to retention of placenta (disorder)
        "82118009",  # Gestation period, 2 weeks (finding)
        "82153002",  # Miscarriage with pulmonary embolism (disorder)
        "8218002",  # Chronic hypertension complicating AND/OR reason for care during childbirth (disorder)
        "82634006",  # Previous pregnancies 10 (finding)
        "82661006",  # Abdominal pregnancy (disorder)
        "82664003",  # Placenta succenturiata (disorder)
        "82771000119102",  # Hypertension complicating pregnancy (disorder)
        "82897000",  # Spontaneous placental expulsion, Duncan mechanism (finding)
        "83056001",  # Cervical effacement 30 percent (finding)
        "83074005",  # Unplanned pregnancy (finding)
        "83094001",  # Perineal hematoma during delivery (disorder)
        "83121003",  # Term birth of fraternal twins, both stillborn (finding)
        "83243004",  # Rigid pelvic floor affecting pregnancy (disorder)
        "8333008",  # Term birth of newborn triplets (finding)
        "833321003",  # Hydropic degeneration of chorionic villi (disorder)
        "833332003",  # Incomplete amniotic membrane (finding)
        "83741001",  # Placenta spuria (disorder)
        "83787007",  # Twin monochorionic diamniotic placenta (disorder)
        "83916000",  # Thrombophlebitis in puerperium (disorder)
        "83922009",  # Miscarriage with parametritis (disorder)
        "8393005",  # Tetanic contractions of uterus (disorder)
        "84007008",  # Shock during AND/OR following labor AND/OR delivery (disorder)
        "84032005",  # Halban's sign (finding)
        "840365009",  # Placenta circummarginate (disorder)
        "840625002",  # Gravid uterus at 12-16 weeks size (finding)
        "840626001",  # Gravid uterus at 16-20 weeks size (finding)
        "840627005",  # Gravid uterus at 20-24 weeks size (finding)
        "840628000",  # Gravid uterus at 24-28 weeks size (finding)
        "840629008",  # Gravid uterus at 28-32 weeks size (finding)
        "840630003",  # Gravid uterus at 32-34 weeks size (finding)
        "840631004",  # Gravid uterus at 34-36 weeks size (finding)
        "840632006",  # Gravid uterus at 36-38 weeks size (finding)
        "840633001",  # Gravid uterus at term size (finding)
        "84132007",  # Gestation period, 35 weeks (finding)
        "84357006",  # Twin monochorionic monoamniotic placenta (disorder)
        "84382006",  # Premature birth of fraternal twins, both stillborn (finding)
        "8445003",  # Tumor of vulva affecting pregnancy (disorder)
        "84457005",  # Spontaneous onset of labor (finding)
        "84693004",  # Intervillous hemorrhage of placenta (disorder)
        "84939004",  # Rubella in mother complicating pregnancy, childbirth AND/OR puerperium (disorder)
        "85039006",  # Postpartum amenorrhea-galactorrhea syndrome (disorder)
        "85076009",  # Placenta fenestrata (disorder)
        "85116003",  # Miscarriage in second trimester (disorder)
        "85331004",  # Miscarriage with laceration of broad ligament (disorder)
        "85467007",  # Miscarriage with perforation of urinary bladder (disorder)
        "85542007",  # Perineal laceration involving skin (disorder)
        "85632001",  # Miscarriage with laceration of vagina (disorder)
        "85991008",  # Miscarriage with pelvic peritonitis (disorder)
        "86041002",  # Pre-existing hypertension in obstetric context (disorder)
        "860603002",  # Episiotomy wound (disorder)
        "86081009",  # Herpes gestationis (disorder)
        "86203003",  # Polyhydramnios (disorder)
        "86256001",  # Hematoma of cord (disorder)
        "86356004",  # Unstable lie (disorder)
        "863897005",  # Failed attempted termination of pregnancy complicated by acute necrosis of liver (disorder)
        "86569001",  # Postpartum state (finding)
        "866229004",  # Trauma to urethra and urinary bladder during delivery (disorder)
        "86801005",  # Gestation period, 6 weeks (finding)
        "86803008",  # Term birth of newborn quintuplets (finding)
        "86883006",  # Gestation period, 23 weeks (finding)
        "87038002",  # Postpartum alopecia (disorder)
        "87043009",  # Cerebral anoxia after obstetrical surgery AND/OR other procedure including delivery (disorder)
        "871005",  # Contracted pelvis (disorder)
        "87178007",  # Gestation period, 1 week (finding)
        "87383005",  # Maternal distress (disorder)
        "87527008",  # Term pregnancy (finding)
        "87605005",  # Cornual pregnancy (disorder)
        "8762007",  # Chronic hypertension in obstetric context (disorder)
        "87621000",  # Hyperemesis gravidarum before end of 22 week gestation with dehydration (disorder)
        "87662006",  # Term birth of fraternal twins, both living (finding)
        "87814002",  # Intrapartum hemorrhage due to marginal placenta previa (disorder)
        "87840008",  # Galactocele associated with childbirth (disorder)
        "87967003",  # Miscarriage with perforation of vagina (disorder)
        "879817000",  # Disproportion between fetus and pelvis due to fetal neoplasm (disorder)
        "879819002",  # Disproportion between fetus and pelvis due to fetal sacral teratoma (disorder)
        "880126001",  # Disorder of amnion (disorder)
        "88178009",  # Puerperal peritonitis (disorder)
        "88201000119101",  # Failure of cervical dilation due to primary uterine inertia (disorder)
        "88387008",  # Postpartum period, 7 days (finding)
        "8884000",  # Fetal ascites causing disproportion (disorder)
        "88887003",  # Maternal hypotension syndrome (disorder)
        "88895004",  # Fatigue during pregnancy (disorder)
        "890096007",  # Pregnancy with implant contraceptive (finding)
        "89244003",  # Entanglement of cords of twins in monoamniotic sac (disorder)
        "89672000",  # Retromammary mastitis associated with childbirth (disorder)
        "89700002",  # Shoulder girdle dystocia (disorder)
        "89934007",  # Crowning (finding)
        "90009001",  # Abnormal umbilical cord (disorder)
        "90188009",  # Failed mechanical induction (disorder)
        "90306000",  # Trial labor (finding)
        "90325002",  # Vomiting of pregnancy (disorder)
        "904002",  # Pinard's sign (finding)
        "90532005",  # Central laceration during delivery (disorder)
        "90645002",  # Failed attempted abortion without complication (disorder)
        "90751002",  # Linea nigra (finding)
        "90797000",  # Gestation period, 28 weeks (finding)
        "90968009",  # Prolonged pregnancy (disorder)
        "9111000119104",  # Musculoskeletal problem in pregnancy (disorder)
        "91162000",  # Necrosis of liver of pregnancy (disorder)
        "9121000119106",  # Low back pain in pregnancy (finding)
        "91217009",  # Megaloblastic anemia due to pregnancy (disorder)
        "91271004",  # Superfetation (finding)
        "91484005",  # Failure of induction of labor by oxytocic drugs (disorder)
        "91948008",  # Asymptomatic human immunodeficiency virus infection in pregnancy (disorder)
        "91957002",  # Back pain complicating pregnancy (disorder)
        "92297008",  # Benign neoplasm of placenta (disorder)
        "92684002",  # Carcinoma in situ of placenta (disorder)
        "9279009",  # Extra-amniotic pregnancy (finding)
        "9293002",  # Atony of uterus (disorder)
        "9297001",  # Uterus bicornis affecting pregnancy (disorder)
        "931004",  # Gestation period, 9 weeks (finding)
        "9343003",  # Term birth of newborn female (finding)
        "9442009",  # Parturient hemorrhage associated with afibrinogenemia (disorder)
        "94631000119100",  # Depressive disorder in mother complicating pregnancy (disorder)
        "94641000119109",  # Anxiety in pregnancy (disorder)
        "95605009",  # Hemolysis-elevated liver enzymes-low platelet count syndrome (disorder)
        "95606005",  # Maternal drug exposure (disorder)
        "95607001",  # Maternal drug use (disorder)
        "95608006",  # Necrosis of placenta (disorder)
        "9686009",  # Goodell's sign (finding)
        "97101000119105",  # Viral disease of mother during pregnancy (disorder)
        "9720009",  # Cicatrix of cervix affecting pregnancy (disorder)
        "9780006",  # Presentation of prolapsed arm of fetus (disorder)
        "9864005",  # Placenta multipartita (disorder)
        "9899009",  # Ovarian pregnancy (disorder)
        "9901000",  # Essential hypertension complicating AND/OR reason for care during puerperium (disorder)
    }

class DiagnosesUsedToIndicateSexualActivity(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent diagnoses indicating history of sexual activity.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of pregnancy, postpartum, sexually transmitted infection, conditions caused by sexually transmitted infections (e.g., inflammatory disease of ovary, fallopian tubes or uterus), symptoms following intercourse (e.g., bleeding), contraceptive use (e.g., adverse effect of contraceptive).

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis of congenital sexually transmitted infections; exposure to HIV virus; inconclusive serologic evidence of HIV.
    """

    VALUE_SET_NAME = "Diagnoses Used to Indicate Sexual Activity"
    OID = "2.16.840.1.113883.3.464.1003.111.12.1018"
    DEFINITION_VERSION = "20210224"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "A34",  # Obstetrical tetanus
        "A510",  # Primary genital syphilis
        "A511",  # Primary anal syphilis
        "A512",  # Primary syphilis of other sites
        "A5131",  # Condyloma latum
        "A5132",  # Syphilitic alopecia
        "A5139",  # Other secondary syphilis of skin
        "A5141",  # Secondary syphilitic meningitis
        "A5142",  # Secondary syphilitic female pelvic disease
        "A5143",  # Secondary syphilitic oculopathy
        "A5144",  # Secondary syphilitic nephritis
        "A5145",  # Secondary syphilitic hepatitis
        "A5146",  # Secondary syphilitic osteopathy
        "A5149",  # Other secondary syphilitic conditions
        "A515",  # Early syphilis, latent
        "A519",  # Early syphilis, unspecified
        "A5200",  # Cardiovascular syphilis, unspecified
        "A5201",  # Syphilitic aneurysm of aorta
        "A5202",  # Syphilitic aortitis
        "A5203",  # Syphilitic endocarditis
        "A5204",  # Syphilitic cerebral arteritis
        "A5205",  # Other cerebrovascular syphilis
        "A5206",  # Other syphilitic heart involvement
        "A5209",  # Other cardiovascular syphilis
        "A5210",  # Symptomatic neurosyphilis, unspecified
        "A5211",  # Tabes dorsalis
        "A5212",  # Other cerebrospinal syphilis
        "A5213",  # Late syphilitic meningitis
        "A5214",  # Late syphilitic encephalitis
        "A5215",  # Late syphilitic neuropathy
        "A5216",  # Charcot's arthropathy (tabetic)
        "A5217",  # General paresis
        "A5219",  # Other symptomatic neurosyphilis
        "A522",  # Asymptomatic neurosyphilis
        "A523",  # Neurosyphilis, unspecified
        "A5271",  # Late syphilitic oculopathy
        "A5272",  # Syphilis of lung and bronchus
        "A5273",  # Symptomatic late syphilis of other respiratory organs
        "A5274",  # Syphilis of liver and other viscera
        "A5275",  # Syphilis of kidney and ureter
        "A5276",  # Other genitourinary symptomatic late syphilis
        "A5277",  # Syphilis of bone and joint
        "A5278",  # Syphilis of other musculoskeletal tissue
        "A5279",  # Other symptomatic late syphilis
        "A528",  # Late syphilis, latent
        "A529",  # Late syphilis, unspecified
        "A530",  # Latent syphilis, unspecified as early or late
        "A539",  # Syphilis, unspecified
        "A5400",  # Gonococcal infection of lower genitourinary tract, unspecified
        "A5401",  # Gonococcal cystitis and urethritis, unspecified
        "A5402",  # Gonococcal vulvovaginitis, unspecified
        "A5403",  # Gonococcal cervicitis, unspecified
        "A5409",  # Other gonococcal infection of lower genitourinary tract
        "A541",  # Gonococcal infection of lower genitourinary tract with periurethral and accessory gland abscess
        "A5421",  # Gonococcal infection of kidney and ureter
        "A5424",  # Gonococcal female pelvic inflammatory disease
        "A5429",  # Other gonococcal genitourinary infections
        "A5430",  # Gonococcal infection of eye, unspecified
        "A5431",  # Gonococcal conjunctivitis
        "A5432",  # Gonococcal iridocyclitis
        "A5433",  # Gonococcal keratitis
        "A5439",  # Other gonococcal eye infection
        "A5440",  # Gonococcal infection of musculoskeletal system, unspecified
        "A5441",  # Gonococcal spondylopathy
        "A5442",  # Gonococcal arthritis
        "A5443",  # Gonococcal osteomyelitis
        "A5449",  # Gonococcal infection of other musculoskeletal tissue
        "A545",  # Gonococcal pharyngitis
        "A546",  # Gonococcal infection of anus and rectum
        "A5481",  # Gonococcal meningitis
        "A5482",  # Gonococcal brain abscess
        "A5483",  # Gonococcal heart infection
        "A5484",  # Gonococcal pneumonia
        "A5485",  # Gonococcal peritonitis
        "A5486",  # Gonococcal sepsis
        "A5489",  # Other gonococcal infections
        "A549",  # Gonococcal infection, unspecified
        "A55",  # Chlamydial lymphogranuloma (venereum)
        "A5600",  # Chlamydial infection of lower genitourinary tract, unspecified
        "A5601",  # Chlamydial cystitis and urethritis
        "A5602",  # Chlamydial vulvovaginitis
        "A5609",  # Other chlamydial infection of lower genitourinary tract
        "A5611",  # Chlamydial female pelvic inflammatory disease
        "A5619",  # Other chlamydial genitourinary infection
        "A562",  # Chlamydial infection of genitourinary tract, unspecified
        "A563",  # Chlamydial infection of anus and rectum
        "A564",  # Chlamydial infection of pharynx
        "A568",  # Sexually transmitted chlamydial infection of other sites
        "A57",  # Chancroid
        "A58",  # Granuloma inguinale
        "A5900",  # Urogenital trichomoniasis, unspecified
        "A5901",  # Trichomonal vulvovaginitis
        "A5903",  # Trichomonal cystitis and urethritis
        "A5909",  # Other urogenital trichomoniasis
        "A598",  # Trichomoniasis of other sites
        "A599",  # Trichomoniasis, unspecified
        "A6000",  # Herpesviral infection of urogenital system, unspecified
        "A6003",  # Herpesviral cervicitis
        "A6004",  # Herpesviral vulvovaginitis
        "A6009",  # Herpesviral infection of other urogenital tract
        "A601",  # Herpesviral infection of perianal skin and rectum
        "A609",  # Anogenital herpesviral infection, unspecified
        "A630",  # Anogenital (venereal) warts
        "A638",  # Other specified predominantly sexually transmitted diseases
        "A64",  # Unspecified sexually transmitted disease
        "B20",  # Human immunodeficiency virus [HIV] disease
        "B9733",  # Human T-cell lymphotrophic virus, type I [HTLV-I] as the cause of diseases classified elsewhere
        "B9734",  # Human T-cell lymphotrophic virus, type II [HTLV-II] as the cause of diseases classified elsewhere
        "B9735",  # Human immunodeficiency virus, type 2 [HIV 2] as the cause of diseases classified elsewhere
        "B977",  # Papillomavirus as the cause of diseases classified elsewhere
        "F526",  # Dyspareunia not due to a substance or known physiological condition
        "F530",  # Postpartum depression
        "F531",  # Puerperal psychosis
        "G4482",  # Headache associated with sexual activity
        "N7001",  # Acute salpingitis
        "N7002",  # Acute oophoritis
        "N7003",  # Acute salpingitis and oophoritis
        "N7011",  # Chronic salpingitis
        "N7012",  # Chronic oophoritis
        "N7013",  # Chronic salpingitis and oophoritis
        "N7091",  # Salpingitis, unspecified
        "N7092",  # Oophoritis, unspecified
        "N7093",  # Salpingitis and oophoritis, unspecified
        "N710",  # Acute inflammatory disease of uterus
        "N711",  # Chronic inflammatory disease of uterus
        "N719",  # Inflammatory disease of uterus, unspecified
        "N930",  # Postcoital and contact bleeding
        "N9410",  # Unspecified dyspareunia
        "N9411",  # Superficial (introital) dyspareunia
        "N9412",  # Deep dyspareunia
        "N9419",  # Other specified dyspareunia
        "N96",  # Recurrent pregnancy loss
        "N970",  # Female infertility associated with anovulation
        "N971",  # Female infertility of tubal origin
        "N972",  # Female infertility of uterine origin
        "N978",  # Female infertility of other origin
        "N979",  # Female infertility, unspecified
        "O94",  # Sequelae of complication of pregnancy, childbirth, and the puerperium
        "T384X1A",  # Poisoning by oral contraceptives, accidental (unintentional), initial encounter
        "T384X1D",  # Poisoning by oral contraceptives, accidental (unintentional), subsequent encounter
        "T384X1S",  # Poisoning by oral contraceptives, accidental (unintentional), sequela
        "T384X2A",  # Poisoning by oral contraceptives, intentional self-harm, initial encounter
        "T384X2D",  # Poisoning by oral contraceptives, intentional self-harm, subsequent encounter
        "T384X2S",  # Poisoning by oral contraceptives, intentional self-harm, sequela
        "T384X3A",  # Poisoning by oral contraceptives, assault, initial encounter
        "T384X3D",  # Poisoning by oral contraceptives, assault, subsequent encounter
        "T384X3S",  # Poisoning by oral contraceptives, assault, sequela
        "T384X4A",  # Poisoning by oral contraceptives, undetermined, initial encounter
        "T384X4D",  # Poisoning by oral contraceptives, undetermined, subsequent encounter
        "T384X4S",  # Poisoning by oral contraceptives, undetermined, sequela
        "T384X5A",  # Adverse effect of oral contraceptives, initial encounter
        "T384X5D",  # Adverse effect of oral contraceptives, subsequent encounter
        "T384X5S",  # Adverse effect of oral contraceptives, sequela
        "T384X6A",  # Underdosing of oral contraceptives, initial encounter
        "T384X6D",  # Underdosing of oral contraceptives, subsequent encounter
        "T384X6S",  # Underdosing of oral contraceptives, sequela
        "T8331XA",  # Breakdown (mechanical) of intrauterine contraceptive device, initial encounter
        "T8331XD",  # Breakdown (mechanical) of intrauterine contraceptive device, subsequent encounter
        "T8331XS",  # Breakdown (mechanical) of intrauterine contraceptive device, sequela
        "T8332XA",  # Displacement of intrauterine contraceptive device, initial encounter
        "T8332XD",  # Displacement of intrauterine contraceptive device, subsequent encounter
        "T8332XS",  # Displacement of intrauterine contraceptive device, sequela
        "T8339XA",  # Other mechanical complication of intrauterine contraceptive device, initial encounter
        "T8339XD",  # Other mechanical complication of intrauterine contraceptive device, subsequent encounter
        "T8339XS",  # Other mechanical complication of intrauterine contraceptive device, sequela
        "Z202",  # Contact with and (suspected) exposure to infections with a predominantly sexual mode of transmission
        "Z21",  # Asymptomatic human immunodeficiency virus [HIV] infection status
        "Z224",  # Carrier of infections with a predominantly sexual mode of transmission
        "Z370",  # Single live birth
        "Z371",  # Single stillbirth
        "Z372",  # Twins, both liveborn
        "Z373",  # Twins, one liveborn and one stillborn
        "Z374",  # Twins, both stillborn
        "Z3750",  # Multiple births, unspecified, all liveborn
        "Z3751",  # Triplets, all liveborn
        "Z3752",  # Quadruplets, all liveborn
        "Z3753",  # Quintuplets, all liveborn
        "Z3754",  # Sextuplets, all liveborn
        "Z3759",  # Other multiple births, all liveborn
        "Z3760",  # Multiple births, unspecified, some liveborn
        "Z3761",  # Triplets, some liveborn
        "Z3762",  # Quadruplets, some liveborn
        "Z3763",  # Quintuplets, some liveborn
        "Z3764",  # Sextuplets, some liveborn
        "Z3769",  # Other multiple births, some liveborn
        "Z377",  # Other multiple births, all stillborn
        "Z379",  # Outcome of delivery, unspecified
        "Z640",  # Problems related to unwanted pregnancy
        "Z641",  # Problems related to multiparity
        "Z7251",  # High risk heterosexual behavior
        "Z7252",  # High risk homosexual behavior
        "Z7253",  # High risk bisexual behavior
        "Z793",  # Long term (current) use of hormonal contraceptives
        "Z920",  # Personal history of contraception
        "Z975",  # Presence of (intrauterine) contraceptive device
        "Z9851",  # Tubal ligation status
    }

    SNOMEDCT = {
        "10345003",  # Primary syphilis of tonsils (disorder)
        "10746341000119109",  # Acquired immune deficiency syndrome complicating childbirth (disorder)
        "10754031000119105",  # Gonorrhea in mother complicating childbirth (disorder)
        "10755671000119100",  # Human immunodeficiency virus in mother complicating childbirth (disorder)
        "1087001000119105",  # Infection of anus and rectum caused by Neisseria gonorrhoeae (disorder)
        "1087021000119101",  # Periurethral abscess caused by Neisseria gonorrhoeae (disorder)
        "1087041000119107",  # Infection of kidney and ureter caused by Neisseria gonorrhoeae (disorder)
        "1087051000119109",  # Gonococcal osteomyelitis (disorder)
        "1087061000119106",  # Gonococcal pneumonia (disorder)
        "1090381000119104",  # Recurrent cervicitis caused by human herpes simplex virus (disorder)
        "109436001",  # Moon molar teeth (disorder)
        "11056091000119105",  # Recurrent herpes simplex infection of buttock (disorder)
        "1107004",  # Early latent syphilis, positive serology, negative cerebrospinal fluid, with relapse after treatment (disorder)
        "111806005",  # Acute gonococcal prostatitis (disorder)
        "111807001",  # Gonococcal endophthalmia (disorder)
        "111880001",  # Acute human immunodeficiency virus infection (disorder)
        "112121000119105",  # Venereal disease caused by Chlamydia trachomatis (disorder)
        "11338007",  # Syphilitic episcleritis (disorder)
        "1142043006",  # Regurgitation of pulmonary valve due to late syphilis (disorder)
        "1142045004",  # Primary human immunodeficiency virus infection (disorder)
        "1142055000",  # Human immunodeficiency virus modified skin disease (disorder)
        "1142099009",  # Secondary syphilis of synovium (disorder)
        "114881000119108",  # Maternal gonorrhea during pregnancy (disorder)
        "1148858008",  # Female infertility due to genetic disease (disorder)
        "1148927006",  # Idiopathic female infertility (disorder)
        "11906007",  # Chronic gonococcal vulvovaginitis (disorder)
        "12198241000119109",  # Malposition of intrauterine contraceptive device (disorder)
        "12232008",  # Syphilitic aneurysm of aorta (disorder)
        "12373006",  # Chronic gonococcal bartholinitis (disorder)
        "1258923000",  # Infection caused by Neisseria gonorrhoeae during puerperium (disorder)
        "1260096003",  # Wasting syndrome due to acquired immunodeficiency syndrome (disorder)
        "1269475009",  # Breakage of intrauterine contraceptive device (disorder)
        "129670002",  # Herpetic cervicitis (disorder)
        "13310005",  # Taboparesis (disorder)
        "151004",  # Gonococcal meningitis (disorder)
        "15628003",  # Gonorrhea (disorder)
        "15928141000119107",  # Cognitive impairment co-occurrent and due to human immunodeficiency virus infection (disorder)
        "15993311000119106",  # Iritis of right eye due to secondary syphilis (disorder)
        "15993351000119107",  # Iritis of left eye due to secondary syphilis (disorder)
        "15993471000119105",  # Iritis of bilateral eyes due to secondary syphilis (disorder)
        "16070004",  # Syphilitic cirrhosis (disorder)
        "16217981000119107",  # Infection of upper genitourinary tract caused by Neisseria gonorrheae (disorder)
        "17305005",  # Acute gonorrhea of genitourinary tract (disorder)
        "186706006",  # Human immunodeficiency virus infection constitutional disease (disorder)
        "186707002",  # Human immunodeficiency virus infection with neurological disease (disorder)
        "186708007",  # Human immunodeficiency virus infection with secondary clinical infectious disease (disorder)
        "186846005",  # Early symptomatic syphilis (disorder)
        "186847001",  # Primary genital syphilis (disorder)
        "186850003",  # Secondary syphilis of skin and mucous membrane (disorder)
        "186854007",  # Uveitis due to secondary syphilis (disorder)
        "186861006",  # Secondary syphilis relapse (disorder)
        "186863009",  # Acute secondary syphilitic meningitis (disorder)
        "186867005",  # Latent early syphilis (disorder)
        "186868000",  # Serological relapse after treatment of latent early syphilis (disorder)
        "186875004",  # Syphilitic endocarditis of mitral valve (disorder)
        "186877007",  # Syphilitic endocarditis of tricuspid valve (disorder)
        "186878002",  # Syphilitic endocarditis of pulmonary valve (disorder)
        "186893003",  # Rupture of syphilitic cerebral aneurysm (disorder)
        "186903006",  # Late latent syphilis (disorder)
        "186915005",  # Chronic gonorrhea lower genitourinary tract (disorder)
        "186931002",  # Gonococcal anal infection (disorder)
        "186939000",  # Gonococcal peritonitis (disorder)
        "186946009",  # Lymphogranuloma venereum (disorder)
        "19206003",  # Syphilitic acoustic neuritis (disorder)
        "192647003",  # Secondary syphilitic meningitis (disorder)
        "19290004",  # Clutton's joints (disorder)
        "193786000",  # Keratitis due to syphilis (disorder)
        "194907008",  # Acute syphilitic pericarditis (disorder)
        "194910001",  # Acute gonococcal pericarditis (disorder)
        "194947001",  # Acute myocarditis caused by Treponema pallidum (disorder)
        "197305002",  # Syphilitic portal cirrhosis (disorder)
        "197347003",  # Hepatitis in late syphilis (disorder)
        "197348008",  # Hepatitis in secondary syphilis (disorder)
        "197757004",  # Late syphilis of kidney (disorder)
        "197848003",  # Gonococcal cystitis (disorder)
        "197850006",  # Trichomonal cystitis (disorder)
        "197966009",  # Syphilitic prostatitis (disorder)
        "198445009",  # Primary anovulatory infertility (disorder)
        "198446005",  # Secondary anovulatory infertility (disorder)
        "198448006",  # Female infertility of pituitary - hypothalamic origin (disorder)
        "198449003",  # Primary pituitary - hypothalamic infertility (disorder)
        "198450003",  # Secondary pituitary - hypothalamic infertility (disorder)
        "198452006",  # Primary tubal infertility (disorder)
        "198453001",  # Secondary tubal infertility (disorder)
        "198456009",  # Primary uterine infertility (disorder)
        "198457000",  # Secondary uterine infertility (disorder)
        "198459002",  # Primary cervical infertility (disorder)
        "198460007",  # Secondary cervical infertility (disorder)
        "198464003",  # Primary vaginal infertility (disorder)
        "198465002",  # Secondary vaginal infertility (disorder)
        "199742002",  # Retained intrauterine contraceptive device in pregnancy (disorder)
        "202933002",  # Bursitis caused by Treponema pallidum (disorder)
        "20735004",  # Syphilitic aortitis (disorder)
        "20943002",  # Acute inflammation of cervix uteri caused by Neisseria gonorrhoeae (disorder)
        "21523006",  # Syphilitic gumma of central nervous system (disorder)
        "22386003",  # Syphilitic optic atrophy (disorder)
        "227561004",  # Female infertility associated with anovulation (disorder)
        "230180003",  # Human immunodeficiency virus leukoencephalopathy (disorder)
        "230182006",  # Late syphilitic encephalitis (disorder)
        "230202002",  # Vacuolar myelopathy (disorder)
        "230598008",  # Neuropathy caused by human immunodeficiency virus (disorder)
        "230735006",  # Syphilitic cerebral arteritis (disorder)
        "231858009",  # Gonococcal conjunctivitis (disorder)
        "232313005",  # Endocochlear syphilis (disorder)
        "232367004",  # Nasal syphilis (disorder)
        "233849007",  # Syphilitic valve disease (disorder)
        "235032001",  # Syphilitic oral leukoplakia (disorder)
        "235062007",  # Oral syphilis (disorder)
        "235064008",  # Syphilitic gumma of oral cavity (disorder)
        "235065009",  # Syphilitic oral snail track ulcer (disorder)
        "235726002",  # Human immunodeficiency virus enteropathy (disorder)
        "235861001",  # Abscess gonococcal (disorder)
        "235863003",  # Gonococcal hepatitis (disorder)
        "236406007",  # Acquired immune deficiency syndrome-related nephropathy (disorder)
        "236682002",  # Inflammation of urethra caused by Neisseria gonorrhoeae (disorder)
        "236687008",  # Gonococcal urethral abscess (disorder)
        "236737006",  # Bleeding due to intrauterine contraceptive device (disorder)
        "236738001",  # Perforation of uterus caused by intrauterine contraceptive device (disorder)
        "236766009",  # Gonococcal epididymitis (disorder)
        "236772009",  # Gonococcal epididymo-orchitis (disorder)
        "237038001",  # Gonococcal salpingitis (disorder)
        "237042003",  # Gonococcal perihepatitis (disorder)
        "237046000",  # Gonococcal tubo-ovarian abscess (disorder)
        "237069002",  # Gonococcal endometritis (disorder)
        "237079000",  # Disorder associated with retained intrauterine contraceptive device (disorder)
        "237080002",  # Expulsion intrauterine contraceptive device (disorder)
        "237083000",  # Inflammation of cervix uteri caused by Neisseria gonorrhoeae (disorder)
        "237095000",  # Gonococcal vulvovaginitis (disorder)
        "237104007",  # Trichomonal vulvitis (disorder)
        "237139007",  # Female infertility due to ovulatory disorder (disorder)
        "237141008",  # Luteal phase defect (disorder)
        "237142001",  # Luteinized unruptured follicle syndrome (disorder)
        "237446005",  # Syphilis of breast (disorder)
        "237447001",  # Primary syphilis of nipple (disorder)
        "237800001",  # Endocrine female infertility (disorder)
        "2390000",  # Acute gonococcal vulvovaginitis (disorder)
        "23975003",  # Chronic gonococcal seminal vesiculitis (disorder)
        "239825002",  # Tabetic joint (disorder)
        "240039005",  # Gonococcal tenosynovitis (disorder)
        "240103002",  # Human immunodeficiency virus myopathy (disorder)
        "240545008",  # Anogenital papillomaviral intraepithelial neoplasia (disorder)
        "240554006",  # Hutchinson's triad (disorder)
        "240555007",  # Acquired syphilis (disorder)
        "240556008",  # Primary extragenital syphilis (disorder)
        "240557004",  # Secondary syphilis (disorder)
        "240558009",  # Macular syphilide (disorder)
        "240560006",  # Papular syphilide (disorder)
        "240561005",  # Corona veneris (disorder)
        "240562003",  # Pustular syphilide (disorder)
        "240563008",  # Syphilitic mucosal ulceration (disorder)
        "240564002",  # Secondary neurosyphilis (disorder)
        "240565001",  # Asymptomatic secondary neurosyphilis (disorder)
        "240566000",  # Gummatous neurosyphilis (disorder)
        "240567009",  # Syphilitic coronary artery disease (disorder)
        "240568004",  # Meningovascular syphilis - quaternary stage (disorder)
        "240569007",  # Syphilitic polyneuropathy (disorder)
        "240572000",  # Gonorrhea with local complication (disorder)
        "240573005",  # Gonococcal Bartholin's gland abscess (disorder)
        "240574004",  # Inflammation of paraurethral gland caused by Neisseria gonorrhoeae (disorder)
        "240575003",  # Gonococcal Tysonitis (disorder)
        "240576002",  # Inflammation of bulbourethral gland caused by Neisseria gonorrhoeae (disorder)
        "240577006",  # Inflammation of Littre gland caused by Neisseria gonorrhoeae (disorder)
        "240578001",  # Gonococcal Littre gland abscess (disorder)
        "240579009",  # Gonococcal paraurethral gland abscess (disorder)
        "240582004",  # Gonococcal synovitis (disorder)
        "240583009",  # Cutaneous gonorrhea (disorder)
        "240584003",  # Gonococcal cellulitis (disorder)
        "240585002",  # Chancroid - anogenital ulcer (disorder)
        "240586001",  # Chancroid - extragenital ulcer (disorder)
        "240587005",  # Chancroid - latent infection (disorder)
        "240602008",  # Early lymphogranuloma venereum (disorder)
        "240603003",  # Late lymphogranuloma venereum (disorder)
        "240604009",  # Latent lymphogranuloma venereum (disorder)
        "240606006",  # Donovanosis - anogenital ulcer (disorder)
        "240608007",  # Donovanosis - non-genital lesion (disorder)
        "24868007",  # Acute gonococcal cystitis (disorder)
        "26039008",  # Neurosyphilis (disorder)
        "26135000",  # Syphilitic encephalitis (disorder)
        "266127002",  # Primary syphilis (disorder)
        "266128007",  # Rash of secondary syphilis (disorder)
        "266130009",  # Acquired syphilis - early latent (disorder)
        "266133006",  # Late quaternary neurosyphilis (disorder)
        "266136003",  # Acquired syphilis - late latent (disorder)
        "266143009",  # Chancroid (disorder)
        "266609001",  # Female infertility of anovulatory origin (disorder)
        "26899006",  # Female infertility of uterine origin (disorder)
        "270524007",  # Mechanical complication of intrauterine contraceptive device (disorder)
        "271973001",  # Disorder of intrauterine contraceptive device (disorder)
        "272006008",  # Gonococcal arthritis dermatitis syndrome (disorder)
        "272262003",  # Esthiomene (disorder)
        "27420004",  # Herpetic vulvovaginitis (disorder)
        "27460003",  # Primary syphilis of breast (disorder)
        "276665006",  # Congenital acquired immune deficiency syndrome (disorder)
        "276666007",  # Congenital human immunodeficiency virus positive status syndrome (disorder)
        "27681008",  # Chronic gonorrhea (disorder)
        "276877003",  # Trichomonal vaginitis (disorder)
        "277499006",  # Complicated donovanosis (disorder)
        "278068003",  # Herpetic vesicle in vagina (disorder)
        "278480000",  # Syphilitic endocarditis of aortic valve (disorder)
        "278481001",  # Quaternary syphilis (disorder)
        "281388009",  # Human immunodeficiency virus-related sclerosing cholangitis (disorder)
        "28198007",  # Primary syphilis of lip (disorder)
        "28438004",  # Gonococcal conjunctivitis neonatorum (disorder)
        "28572009",  # Chronic gonorrhea of genitourinary tract (disorder)
        "286995008",  # Female infertility - cervical/vaginal (disorder)
        "28867007",  # Granuloma inguinale (disorder)
        "296896008",  # Oral contraceptive overdose (disorder)
        "296897004",  # Accidental oral contraceptive overdose (disorder)
        "296898009",  # Intentional oral contraceptive overdose (disorder)
        "29864006",  # Acute inflammation of urethra caused by Neisseria gonorrhoeae (disorder)
        "301086002",  # Syphilitic meningitis (disorder)
        "30168008",  # Acute gonococcal epididymo-orchitis (disorder)
        "30238006",  # Secondary female infertility (disorder)
        "302812006",  # Anogenital human papillomavirus infection (disorder)
        "302813001",  # Syphilitic acoustic neuritis - quaternary stage (disorder)
        "307423008",  # Gonococcal pelvic peritonitis (disorder)
        "30826006",  # Female infertility of vaginal origin (disorder)
        "31015008",  # Primary anal syphilis (disorder)
        "31137003",  # Early latent syphilis, positive serology, negative cerebrospinal fluid, less than 2 years after infection (disorder)
        "312934004",  # Syphilitic chorioretinitis (disorder)
        "312955002",  # Tertiary syphilitic chorioretinitis (disorder)
        "314840009",  # Progressive locomotor ataxia (disorder)
        "315019000",  # Aseptic meningitis due to human immunodeficiency virus infection (disorder)
        "315826004",  # Tabetic neurosyphilis (disorder)
        "3160009",  # Female infertility of cervical origin (disorder)
        "31999004",  # Chronic gonococcal endometritis (disorder)
        "331151000119109",  # Syphilitic choroiditis (disorder)
        "33839006",  # Genital herpes simplex (disorder)
        "342381000119109",  # Gonococcal iritis (disorder)
        "35089004",  # Urogenital infection caused by Trichomonas vaginalis (disorder)
        "35308000",  # Female infertility due to nonimplantation (disorder)
        "35526001",  # Chronic gonococcal epididymo-orchitis (disorder)
        "35876006",  # Gonococcal infection of eye (disorder)
        "3589003",  # Syphilitic pericarditis (disorder)
        "36276008",  # Syphilitic retrobulbar neuritis (disorder)
        "371237000",  # Syphilitic skin disorder (disorder)
        "37430004",  # Syphilis of synovium (disorder)
        "37754005",  # Asymptomatic neurosyphilis (disorder)
        "38523005",  # Syphilitic parkinsonism (disorder)
        "39085002",  # Secondary syphilis of mucous membrane (disorder)
        "39446004",  # Female infertility of tubal origin (disorder)
        "39591000119109",  # Female infertility due to disorder of pituitary gland (disorder)
        "397763006",  # Human immunodeficiency virus encephalopathy (disorder)
        "398329009",  # Human immunodeficiency virus encephalitis (disorder)
        "40149008",  # Gonococcal keratitis (disorder)
        "402126004",  # Sexually transmitted bacterial disease affecting skin (disorder)
        "402888002",  # Primary herpes simplex infection of genitalia (disorder)
        "402890001",  # Primary herpetic vulvovaginitis (disorder)
        "402894005",  # Recurrent genital herpes simplex (disorder)
        "402896007",  # Recurrent herpetic vulvovaginitis (disorder)
        "402901009",  # Oral hairy leukoplakia associated with human immunodeficiency virus disease (disorder)
        "402941000",  # Syphilitic chancre of vulva (disorder)
        "402942007",  # Syphilitic chancre of oral mucous membranes (disorder)
        "402943002",  # Syphilitic balanitis (disorder)
        "402944008",  # Condylomata lata of perianal skin (disorder)
        "402946005",  # Condylomata lata of vulva (disorder)
        "402947001",  # Late secondary syphilis (disorder)
        "402949003",  # Nodular syphilide (disorder)
        "402950003",  # Syphilitic leukoplakia of tongue (disorder)
        "402951004",  # Oral mucous membrane lesion due to late syphilis (disorder)
        "402952006",  # Anetoderma secondary to syphilis (disorder)
        "402955008",  # Vulval chancroid (disorder)
        "402956009",  # Localized cutaneous gonococcal infection (disorder)
        "402957000",  # Gonococcal bartholinitis (disorder)
        "402958005",  # Pustular vasculitis due to gonococcal bacteremia (disorder)
        "405631006",  # Pediatric human immunodeficiency virus infection (disorder)
        "405635002",  # Late syphilis with clinical manifestations other than neurosyphilis (disorder)
        "406581000",  # Gonococcal infection of the central nervous system (disorder)
        "40780007",  # Human immunodeficiency virus I infection (disorder)
        "4082005",  # Myocarditis caused by Treponema pallidum (disorder)
        "410470003",  # Syphilitic retinitis (disorder)
        "410478005",  # Ocular syphilis (disorder)
        "414238007",  # Female infertility due to antisperm antibody (disorder)
        "414239004",  # Female infertility due to cervical mucus dysfunction (disorder)
        "414376003",  # Hairy leukoplakia of tongue associated with human immunodeficiency virus disease (disorder)
        "414604009",  # Leukoplakia of tongue associated with human immunodeficiency virus disease (disorder)
        "416729007",  # Neutropenia with acquired immunodeficiency syndrome (disorder)
        "420244003",  # Encephalitis with acquired immunodeficiency syndrome (disorder)
        "420281004",  # Skin rash with acquired immunodeficiency syndrome (disorder)
        "420302007",  # Reticulosarcoma with acquired immunodeficiency syndrome (disorder)
        "420308006",  # Retinal vascular changes with acquired immunodeficiency syndrome (disorder)
        "420321004",  # Intestinal malabsorption with acquired immunodeficiency syndrome (disorder)
        "420384005",  # Hematopoietic system disease with acquired immunodeficiency syndrome (disorder)
        "420395004",  # Acute endocarditis with acquired immunodeficiency syndrome (disorder)
        "420403001",  # Pneumocystosis with acquired immunodeficiency syndrome (disorder)
        "420452002",  # Myelopathy with acquired immunodeficiency syndrome (disorder)
        "420524008",  # Kaposi's sarcoma with acquired immunodeficiency syndrome (disorder)
        "420543008",  # Anemia associated with acquired immunodeficiency syndrome (disorder)
        "420544002",  # Bacterial pneumonia with acquired immunodeficiency syndrome (disorder)
        "420549007",  # Salivary gland disease with acquired immunodeficiency syndrome (disorder)
        "420554003",  # Progressive multifocal leukoencephalopathy with acquired immunodeficiency syndrome (disorder)
        "420614009",  # Organic dementia with acquired immunodeficiency syndrome (disorder)
        "420658009",  # Radiculitis with acquired immunodeficiency syndrome (disorder)
        "420691000",  # Nutritional deficiency with acquired immunodeficiency syndrome (disorder)
        "420718004",  # Central nervous system demyelinating disease with acquired immunodeficiency syndrome (disorder)
        "420721002",  # Acquired immunodeficiency syndrome-associated disorder (disorder)
        "420764009",  # Salmonella infection with acquired immunodeficiency syndrome (disorder)
        "420774007",  # Organic brain syndrome with acquired immunodeficiency syndrome (disorder)
        "420787001",  # Pneumococcal pneumonia with acquired immunodeficiency syndrome (disorder)
        "420801006",  # Malaise with acquired immunodeficiency syndrome (disorder)
        "420818005",  # Mycobacteriosis with acquired immunodeficiency syndrome (disorder)
        "420877009",  # Dermatomycosis with acquired immunodeficiency syndrome (disorder)
        "420900006",  # Fatigue with acquired immunodeficiency syndrome (disorder)
        "420938005",  # Subacute endocarditis with acquired immunodeficiency syndrome (disorder)
        "420945005",  # Histoplasmosis with acquired immunodeficiency syndrome (disorder)
        "421020000",  # Microsporidiosis associated with acquired immunodeficiency syndrome (disorder)
        "421023003",  # Presenile dementia with acquired immunodeficiency syndrome (disorder)
        "421047005",  # Candidiasis of lung with acquired immunodeficiency syndrome (disorder)
        "421077004",  # Disseminated candidiasis with acquired immunodeficiency syndrome (disorder)
        "421102007",  # Aplastic anemia with acquired immunodeficiency syndrome (disorder)
        "421230000",  # Hepatomegaly with acquired immunodeficiency syndrome (disorder)
        "421272004",  # Subacute myocarditis with acquired immunodeficiency syndrome (disorder)
        "421283008",  # Primary lymphoma of brain with acquired immunodeficiency syndrome (disorder)
        "421312009",  # Agranulocytosis with acquired immunodeficiency syndrome (disorder)
        "421315006",  # Myelitis with acquired immunodeficiency syndrome (disorder)
        "421394009",  # Skin disorder with acquired immunodeficiency syndrome (disorder)
        "421403008",  # Cryptococcosis with acquired immunodeficiency syndrome (disorder)
        "421415007",  # Subacute adenoviral encephalitis with acquired immunodeficiency syndrome (disorder)
        "421431004",  # Nocardiosis with acquired immunodeficiency syndrome (disorder)
        "421454008",  # Infectious gastroenteritis with acquired immunodeficiency syndrome (disorder)
        "421460008",  # Retinopathy with acquired immunodeficiency syndrome (disorder)
        "421508002",  # Viral pneumonia with acquired immunodeficiency syndrome (disorder)
        "421529006",  # Dementia with acquired immunodeficiency syndrome (disorder)
        "421571007",  # Tuberculosis with acquired immunodeficiency syndrome (disorder)
        "421597001",  # Polyneuropathy with acquired immunodeficiency syndrome (disorder)
        "421660003",  # Failure to thrive in infant with acquired immunodeficiency syndrome (disorder)
        "421671002",  # Pneumonia with acquired immunodeficiency syndrome (disorder)
        "421706001",  # Blindness with acquired immunodeficiency syndrome (disorder)
        "421708000",  # Infective arthritis with acquired immunodeficiency syndrome (disorder)
        "421710003",  # Candidiasis of mouth with acquired immunodeficiency syndrome (disorder)
        "421766003",  # Thrombocytopenia with acquired immunodeficiency syndrome (disorder)
        "421827003",  # Encephalopathy with acquired immunodeficiency syndrome (disorder)
        "421851008",  # Acquired hemolytic anemia associated with acquired immunodeficiency syndrome (disorder)
        "421874007",  # Respiratory disorder with acquired immunodeficiency syndrome (disorder)
        "421883002",  # Strongyloidiasis with acquired immunodeficiency syndrome (disorder)
        "421929001",  # Myocarditis with acquired immunodeficiency syndrome (disorder)
        "421983003",  # Noninfectious gastroenteritis with acquired immunodeficiency syndrome (disorder)
        "421998001",  # Central nervous disorder with acquired immunodeficiency syndrome (disorder)
        "422012004",  # Neuritis with acquired immunodeficiency syndrome (disorder)
        "422089004",  # Encephalomyelitis with acquired immunodeficiency syndrome (disorder)
        "422127002",  # Herpes zoster with acquired immunodeficiency syndrome (disorder)
        "422136003",  # Neuralgia with acquired immunodeficiency syndrome (disorder)
        "422177004",  # Dyspnea with acquired immunodeficiency syndrome (disorder)
        "422189002",  # Low vision with acquired immunodeficiency syndrome (disorder)
        "422194002",  # Hyperhidrosis with acquired immunodeficiency syndrome (disorder)
        "422282000",  # Malignant neoplasm with acquired immunodeficiency syndrome (disorder)
        "422337001",  # Coccidioidomycosis with acquired immunodeficiency syndrome (disorder)
        "423391007",  # Genital herpes simplex type 2 (disorder)
        "426488001",  # Female infertility due to oligo-ovulation (disorder)
        "42746002",  # Infection of rectum caused by Neisseria gonorrhoeae (disorder)
        "427547007",  # Female infertility due to diminished ovarian reserve (disorder)
        "42770003",  # Syphilis of tricuspid valve (disorder)
        "428008002",  # Female infertility due to advanced maternal age (disorder)
        "432218001",  # History of asymptomatic human immunodeficiency virus infection (situation)
        "439912007",  # Recurrent genital Herpes simplex type 1 infection (disorder)
        "439913002",  # Recurrent genital Herpes simplex type 2 infection (disorder)
        "440714005",  # Genital Herpes simplex type 1 infection (disorder)
        "442537007",  # Non-Hodgkin lymphoma associated with Human immunodeficiency virus infection (disorder)
        "44412000",  # Chronic inflammation of urethra caused by Neisseria gonorrhoeae (disorder)
        "444150000",  # Latent syphilis (disorder)
        "444834005",  # Abscess of urethral gland caused by Neisseria gonorrhoeae (disorder)
        "44568006",  # Syphilitic interstitial keratitis (disorder)
        "445945000",  # Infectious disease with acquired immune deficiency syndrome (disorder)
        "44743006",  # Gonococcal infection of joint (disorder)
        "4483005",  # Syphilitic punched out ulcer (disorder)
        "449773001",  # Interstitial glossitis (disorder)
        "449776009",  # Luetic glossitis (disorder)
        "45058001",  # Syphilis of pulmonary valve (disorder)
        "45377007",  # Acute gonococcal salpingitis (disorder)
        "4667004",  # Female infertility due to structural congenital anomaly of vagina (disorder)
        "46699001",  # Bursitis caused by Neisseria gonorrhoeae (disorder)
        "473065001",  # Infection associated with retained intrauterine contraceptive device (disorder)
        "473076002",  # Infection associated with intrauterine contraceptive device (disorder)
        "473079009",  # Inflammation associated with retained intrauterine contraceptive device (disorder)
        "473082004",  # Infection and inflammation associated with retained intrauterine contraceptive device (disorder)
        "48794007",  # Human immunodeficiency virus infection with infectious mononucleosis-like syndrome (disorder)
        "49923008",  # Syphilis of tendon (disorder)
        "501661611000119107",  # Latent genital herpes simplex during pregnancy (disorder)
        "50528008",  # Syphilis of bone (disorder)
        "5084002",  # Primary female infertility (disorder)
        "50970007",  # Acute gonorrhea of upper genitourinary tract (disorder)
        "51928006",  # General paresis - neurosyphilis (disorder)
        "51960003",  # Secondary syphilis of pharynx (disorder)
        "52079000",  # Congenital human immunodeficiency virus infection (disorder)
        "52414005",  # Secondary syphilis of vulva (disorder)
        "53529004",  # Chronic gonococcal salpingitis (disorder)
        "53664003",  # Gonococcal spondylitis (disorder)
        "54274001",  # Primary syphilis of fingers (disorder)
        "54825009",  # Acute gonorrhea of lower genitourinary tract (disorder)
        "55768006",  # Syphilitic leukoderma (disorder)
        "58056005",  # Syphilis of mitral valve (disorder)
        "58227000",  # Secondary syphilis of viscera (disorder)
        "59233003",  # Secondary syphilis of skin (disorder)
        "59307008",  # Syphilitic alopecia (disorder)
        "59530001",  # Syphilis of kidney (disorder)
        "59819007",  # Herpetic ulceration of vulva (disorder)
        "59934002",  # Secondary syphilitic adenopathy (disorder)
        "60335002",  # Gonococcal keratosis (disorder)
        "60528006",  # Secondary syphilis of mouth (disorder)
        "60893000",  # Chronic gonococcal prostatitis (disorder)
        "61048000",  # Gonococcal endocarditis (disorder)
        "61612001",  # Syphilitic aortic incompetence (disorder)
        "62207008",  # Syphilitic ostial coronary disease (disorder)
        "62479008",  # Acquired immune deficiency syndrome (disorder)
        "62861003",  # Secondary syphilis, relapse (untreated) (disorder)
        "63751007",  # Secondary syphilis of bone (disorder)
        "64102008",  # Syphilitic gumma (disorder)
        "65049003",  # Acute gonococcal seminal vesiculitis (disorder)
        "65295003",  # Acute gonococcal endometritis (disorder)
        "65454003",  # Female infertility associated with congenital anomaly of uterus (disorder)
        "66281009",  # Chorioretinitis due to secondary syphilis (disorder)
        "66887000",  # Late syphilis, latent (positive serology, negative cephalospinal fluid 2 years after) (disorder)
        "67125004",  # Latent syphilis with positive serology (disorder)
        "6738008",  # Female infertility (disorder)
        "67391006",  # Syphilitic endocarditis (disorder)
        "68863007",  # Acute syphilitic meningitis (disorder)
        "69595007",  # Secondary syphilitic periostitis (disorder)
        "697904001",  # Pulmonary arterial hypertension associated with human immunodeficiency virus infection (disorder)
        "697965002",  # Cholangitis with acquired immunodeficiency syndrome (disorder)
        "700053002",  # Sepsis with acquired immunodeficiency syndrome (disorder)
        "707577004",  # Female infertility due to cystic fibrosis (disorder)
        "71011005",  # Infestation caused by Phthirus pubis (disorder)
        "712584009",  # Female infertility due to endometriosis (disorder)
        "712585005",  # Female infertility due to pelvic peritoneal adhesion (disorder)
        "713251003",  # Gingival disease caused by Treponema pallidum (disorder)
        "713260006",  # Subacute adenoviral encephalitis co-occurrent with human immunodeficiency virus infection (disorder)
        "713261005",  # Gingival disease caused by Neisseria gonorrhoeae (disorder)
        "713275003",  # Splenomegaly co-occurrent with human immunodeficiency virus infection (disorder)
        "713278001",  # Neuralgia co-occurrent with human immunodeficiency virus infection (disorder)
        "713297001",  # Candidiasis of esophagus co-occurrent with human immunodeficiency virus infection (disorder)
        "713298006",  # Heart disease co-occurrent with human immunodeficiency virus infection (disorder)
        "713299003",  # Disorder of eye proper co-occurrent with human immunodeficiency virus infection (disorder)
        "713300006",  # Disorder of gastrointestinal tract co-occurrent with human immunodeficiency virus infection (disorder)
        "713316008",  # Eruption of skin co-occurrent with human immunodeficiency virus infection (disorder)
        "713318009",  # Myocarditis co-occurrent with human immunodeficiency virus infection (disorder)
        "713320007",  # Radiculitis co-occurrent with human immunodeficiency virus infection (disorder)
        "713325002",  # Primary cerebral lymphoma co-occurrent with human immunodeficiency virus infection (disorder)
        "713339002",  # Infection caused by Strongyloides co-occurrent with human immunodeficiency virus infection (disorder)
        "713340000",  # Disorder of skin co-occurrent with human immunodeficiency virus infection (disorder)
        "713341001",  # Myelitis co-occurrent with human immunodeficiency virus infection (disorder)
        "713342008",  # Infection caused by Salmonella co-occurrent with human immunodeficiency virus infection (disorder)
        "713349004",  # Anemia co-occurrent with human immunodeficiency virus infection (disorder)
        "713444005",  # Hemophagocytic syndrome with human immunodeficiency virus infection (disorder)
        "713445006",  # Disseminated infection caused by Strongyloides co-occurrent with human immunodeficiency virus infection (disorder)
        "713446007",  # Chronic infection caused by herpes simplex virus co-occurrent with human immunodeficiency virus infection (disorder)
        "713483007",  # Reticulosarcoma co-occurrent with human immunodeficiency virus infection (disorder)
        "713484001",  # Disorder of respiratory system co-occurrent with human immunodeficiency virus infection (disorder)
        "713487008",  # Progressive multifocal leukoencephalopathy co-occurrent with human immunodeficiency virus infection (disorder)
        "713488003",  # Presenile dementia co-occurrent with human immunodeficiency virus infection (disorder)
        "713489006",  # Polyneuropathy co-occurrent with human immunodeficiency virus infection (disorder)
        "713490002",  # Infection caused by Pneumocystis co-occurrent with human immunodeficiency virus infection (disorder)
        "713491003",  # Organic brain syndrome co-occurrent with human immunodeficiency virus infection (disorder)
        "713497004",  # Candidiasis of mouth co-occurrent with human immunodeficiency virus infection (disorder)
        "713503007",  # Disorder of spinal cord co-occurrent with human immunodeficiency virus infection (disorder)
        "713504001",  # Disorder of kidney co-occurrent with human immunodeficiency virus infection (disorder)
        "713505000",  # Gastrointestinal malabsorption syndrome co-occurrent with human immunodeficiency virus infection (disorder)
        "713506004",  # Neuritis co-occurrent with human immunodeficiency virus infection (disorder)
        "713507008",  # Lymphadenopathy co-occurrent with human immunodeficiency virus infection (disorder)
        "713508003",  # Aplastic anemia co-occurrent with human immunodeficiency virus infection (disorder)
        "713510001",  # Enlargement of liver co-occurrent with human immunodeficiency virus infection (disorder)
        "713511002",  # Acute endocarditis co-occurrent with human immunodeficiency virus infection (disorder)
        "713523008",  # Cardiomyopathy co-occurrent with human immunodeficiency virus infection (disorder)
        "713526000",  # Recurrent bacterial pneumonia co-occurrent with human immunodeficiency virus infection (disorder)
        "713527009",  # Disorder of peripheral nervous system co-occurrent with human immunodeficiency virus infection (disorder)
        "713530002",  # Agranulocytosis co-occurrent with human immunodeficiency virus infection (disorder)
        "713531003",  # Visual impairment co-occurrent with human immunodeficiency virus infection (disorder)
        "713532005",  # Infective arthritis co-occurrent with human immunodeficiency virus infection (disorder)
        "713533000",  # Acquired hemolytic anemia co-occurrent with human immunodeficiency virus infection (disorder)
        "713543002",  # Demyelinating disease of central nervous system co-occurrent with human immunodeficiency virus infection (disorder)
        "713544008",  # Bacterial pneumonia co-occurrent with human immunodeficiency virus infection (disorder)
        "713545009",  # Infection caused by Nocardia co-occurrent with human immunodeficiency virus infection (disorder)
        "713546005",  # Isosporiasis co-occurrent with human immunodeficiency virus infection (disorder)
        "713570009",  # Infectious gastroenteritis co-occurrent with human immunodeficiency virus infection (disorder)
        "713571008",  # Disorder of central nervous system co-occurrent with human immunodeficiency virus infection (disorder)
        "713572001",  # Malignant neoplastic disease co-occurrent with human immunodeficiency virus infection (disorder)
        "713695001",  # Nephrotic syndrome co-occurrent with human immunodeficiency virus infection (disorder)
        "713696000",  # Renal failure syndrome co-occurrent with human immunodeficiency virus infection (disorder)
        "713718006",  # Diffuse non-Hodgkin immunoblastic lymphoma co-occurrent with human immunodeficiency virus infection (disorder)
        "713722001",  # Infection caused by Cytomegalovirus co-occurrent with human immunodeficiency virus infection (disorder)
        "713729005",  # Infection caused by Coccidia co-occurrent with human immunodeficiency virus infection (disorder)
        "713730000",  # Infection caused by herpes simplex virus co-occurrent with human immunodeficiency virus infection (disorder)
        "713731001",  # Pyrexia of unknown origin co-occurrent with human immunodeficiency virus infection (disorder)
        "713732008",  # Infection caused by Aspergillus co-occurrent with human immunodeficiency virus infection (disorder)
        "713733003",  # Infection caused by herpes zoster virus co-occurrent with human immunodeficiency virus infection (disorder)
        "713734009",  # Infection caused by Dermatophyte co-occurrent with human immunodeficiency virus infection (disorder)
        "713844000",  # Dementia co-occurrent with human immunodeficiency virus infection (disorder)
        "713845004",  # Infection caused by Cryptosporidium co-occurrent with human immunodeficiency virus infection (disorder)
        "713880000",  # Opportunistic mycosis co-occurrent with human immunodeficiency virus infection (disorder)
        "713881001",  # Infection caused by Microsporidia co-occurrent with human immunodeficiency virus infection (disorder)
        "713887002",  # Focal segmental glomerulosclerosis co-occurrent with human immunodeficiency virus infection (disorder)
        "713897006",  # Burkitt lymphoma co-occurrent with human immunodeficiency virus infection (disorder)
        "713964006",  # Multidermatomal infection caused by Herpes zoster co-occurrent with human immunodeficiency virus infection (disorder)
        "713967004",  # Disseminated atypical infection caused by Mycobacterium co-occurrent with human immunodeficiency virus infection (disorder)
        "714083007",  # Recurrent salmonella sepsis co-occurrent with human immunodeficiency virus infection (disorder)
        "714464009",  # Immune reconstitution inflammatory syndrome caused by human immunodeficiency virus infection (disorder)
        "719522009",  # Infection of upper respiratory tract caused by Candida co-occurrent with human immunodeficiency virus infection (disorder)
        "719755001",  # Conjunctivitis of adulthood caused by Neisseria gonorrhoeae (disorder)
        "72083004",  # Late syphilis (disorder)
        "721166000",  # Human immunodeficiency virus complicating pregnancy childbirth and the puerperium (disorder)
        "721585006",  # Sexually transmissible infection caused by Herpes simplex virus (disorder)
        "721587003",  # Sexually transmissible infection caused by human papillomavirus (disorder)
        "721588008",  # Hepatitis caused by sexually transmissible virus (disorder)
        "721589000",  # Sexually transmissible infection caused by Molluscum contagiosum virus (disorder)
        "72225002",  # Urethral stricture due to gonococcal infection (disorder)
        "722520001",  # Non-ulcerative sexually transmitted infection caused by Chlamydia trachomatis (disorder)
        "722557007",  # Parkinsonism due to human immunodeficiency virus infection (disorder)
        "733834006",  # Invasive carcinoma of uterine cervix co-occurrent with human immunodeficiency virus infection (disorder)
        "733835007",  # Extrapulmonary tuberculosis co-occurrent with human immunodeficiency virus infection (disorder)
        "735521001",  # Human immunodeficiency virus World Health Organization 2007 stage 1 co-occurrent with tuberculosis (disorder)
        "735522008",  # Human immunodeficiency virus World Health Organization 2007 stage 1 co-occurrent with malaria (disorder)
        "735523003",  # Human immunodeficiency virus World Health Organization 2007 stage 2 co-occurrent with tuberculosis (disorder)
        "735524009",  # Human immunodeficiency virus World Health Organization 2007 stage 2 co-occurrent with malaria (disorder)
        "735525005",  # Human immunodeficiency virus World Health Organization 2007 stage 3 co-occurrent with tuberculosis (disorder)
        "735526006",  # Human immunodeficiency virus World Health Organization 2007 stage 3 co-occurrent with malaria (disorder)
        "735527002",  # Human immunodeficiency virus World Health Organization 2007 stage 4 co-occurrent with tuberculosis (disorder)
        "735528007",  # Human immunodeficiency virus World Health Organization 2007 stage 4 co-occurrent with malaria (disorder)
        "736686006",  # Chancre caused by Treponema pallidum (disorder)
        "74294009",  # Female infertility due to structural congenital anomaly of cervix (disorder)
        "74372003",  # Gonorrhea of pharynx (disorder)
        "75299005",  # Spastic spinal syphilitic paralysis (disorder)
        "762257007",  # Disseminated infection caused by Neisseria gonorrhoeae (disorder)
        "76272004",  # Syphilis (disorder)
        "76802005",  # Chronic inflammation of cervix uteri caused by Neisseria gonorrhoeae (disorder)
        "77028001",  # Secondary syphilis of anus (disorder)
        "770726004",  # Female infertility due to zona pellucida defect (disorder)
        "771119002",  # Infection caused by Coccidia co-occurrent with acquired immunodeficiency syndrome (disorder)
        "771126002",  # Infection caused by Toxoplasma gondii co-occurrent with acquired immunodeficiency syndrome (disorder)
        "771127006",  # Infection caused by Isospora co-occurrent with acquired immunodeficiency syndrome (disorder)
        "77939001",  # Syphilitic disseminated retinochoroiditis (disorder)
        "788975001",  # Late syphilis of skin (disorder)
        "788998009",  # Ocular late syphilis (disorder)
        "789121002",  # Late syphilis of mucous membrane (disorder)
        "789473005",  # Migration of intrauterine contraceptive device (disorder)
        "79019005",  # Human immunodeficiency virus II infection (disorder)
        "80191000119101",  # Symptomatic human immunodeficiency virus I infection (disorder)
        "80388004",  # Chronic gonorrhea of upper genitourinary tract (disorder)
        "80604007",  # Acute gonococcal bartholinitis (disorder)
        "80770009",  # Secondary syphilis of liver (disorder)
        "8098009",  # Sexually transmitted infectious disease (disorder)
        "81000119104",  # Symptomatic human immunodeficiency virus infection (disorder)
        "81339006",  # Secondary syphilis of tonsil (disorder)
        "81598001",  # Trichomonal vulvovaginitis (disorder)
        "82355002",  # Syphilitic aortic stenosis (disorder)
        "82593008",  # Female infertility due to congenital anomaly of fallopian tube (disorder)
        "838338001",  # Bacterial infection due to human immunodeficiency virus infection (disorder)
        "83883001",  # Cardiovascular syphilis (disorder)
        "84785006",  # Female infertility due to acquired stenosis of fallopian tube (disorder)
        "85078005",  # Female infertility due to occlusion of fallopian tube (disorder)
        "8555001",  # Syphilis of lung (disorder)
        "85857008",  # Secondary syphilis, relapse (treated) (disorder)
        "86028001",  # Syphilis of liver (disorder)
        "860871003",  # Eosinophilic folliculitis due to human immunodeficiency virus infection (disorder)
        "860872005",  # Esophagitis due to human immunodeficiency virus disease (disorder)
        "86406008",  # Human immunodeficiency virus infection (disorder)
        "86443005",  # Hutchinson's teeth (disorder)
        "866044006",  # Mycosis due to human immunodeficiency virus infection (disorder)
        "870271009",  # Pruritus due to human immunodeficiency virus infection (disorder)
        "870328002",  # Papular pruritic eruption of skin due to human immunodeficiency virus infection (disorder)
        "870344006",  # Seborrheic dermatitis due to human immunodeficiency virus infection (disorder)
        "87117006",  # Human immunodeficiency virus infection with acute lymphadenitis (disorder)
        "88813005",  # Chronic gonococcal cystitis (disorder)
        "88943008",  # Syphilis of muscle (disorder)
        "90428001",  # Gonococcal pericarditis (disorder)
        "90681000119107",  # Asymptomatic human immunodeficiency virus A1 infection (disorder)
        "90691000119105",  # Asymptomatic human immunodeficiency virus A2 infection (disorder)
        "9091006",  # Gonococcal iridocyclitis (disorder)
        "91554004",  # Condyloma latum (disorder)
        "91947003",  # Asymptomatic human immunodeficiency virus infection (disorder)
        "91948008",  # Asymptomatic human immunodeficiency virus infection in pregnancy (disorder)
        "9241004",  # Gonococcal heart disease (disorder)
        "94851000119107",  # Syphilitic valvular endocarditis (disorder)
        "95892003",  # Persistent generalized lymphadenopathy (disorder)
    }

class UpperRespiratoryInfection(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of an acute upper respiratory infection.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of an acute upper respiratory infection.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Upper Respiratory Infection"
    OID = "2.16.840.1.113883.3.464.1003.102.12.1022"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "J00",  # Acute nasopharyngitis [common cold]
        "J060",  # Acute laryngopharyngitis
        "J069",  # Acute upper respiratory infection, unspecified
    }

    SNOMEDCT = {
        "43692000",  # Influenzal acute upper respiratory infection (disorder)
        "54398005",  # Acute upper respiratory infection (disorder)
        "82272006",  # Common cold (disorder)
    }

class AlcoholWithdrawal(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of alcohol withdrawal.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of alcohol withdrawal or alcohol dependence with withdrawal.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Alcohol Withdrawal"
    OID = "2.16.840.1.113883.3.464.1003.105.12.1209"
    DEFINITION_VERSION = "20210224"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "F10230",  # Alcohol dependence with withdrawal, uncomplicated
        "F10231",  # Alcohol dependence with withdrawal delirium
        "F10232",  # Alcohol dependence with withdrawal with perceptual disturbance
        "F10239",  # Alcohol dependence with withdrawal, unspecified
    }

    SNOMEDCT = {
        "85561006",  # Alcohol withdrawal syndrome without complication (disorder)
        "191476005",  # Alcohol withdrawal hallucinosis (disorder)
        "191480000",  # Alcohol withdrawal syndrome (disorder)
        "308742005",  # Alcohol withdrawal-induced convulsion (disorder)
        "723926008",  # Perceptual disturbances and seizures co-occurrent and due to alcohol withdrawal (disorder)
        "8635005",  # Alcohol withdrawal delirium (disorder)
    }

class BenzodiazepineWithdrawal(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of benzodiazepine withdrawal.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of benzodiazepine withdrawal; sedative, hypnotic or anxiolytic withdrawal; or sedative, hypnotic, or anxiolytic dependence with withdrawal.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Benzodiazepine Withdrawal"
    OID = "2.16.840.1.113883.3.464.1003.105.12.1208"
    DEFINITION_VERSION = "20210224"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "F13230",  # Sedative, hypnotic or anxiolytic dependence with withdrawal, uncomplicated
        "F13231",  # Sedative, hypnotic or anxiolytic dependence with withdrawal delirium
        "F13232",  # Sedative, hypnotic or anxiolytic dependence with withdrawal with perceptual disturbance
        "F13239",  # Sedative, hypnotic or anxiolytic dependence with withdrawal, unspecified
        "F13930",  # Sedative, hypnotic or anxiolytic use, unspecified with withdrawal, uncomplicated
        "F13931",  # Sedative, hypnotic or anxiolytic use, unspecified with withdrawal delirium
        "F13932",  # Sedative, hypnotic or anxiolytic use, unspecified with withdrawal with perceptual disturbances
        "F13939",  # Sedative, hypnotic or anxiolytic use, unspecified with withdrawal, unspecified
    }

    SNOMEDCT = {
        "361150008",  # Sedative, hypnotic AND/OR anxiolytic withdrawal delirium (disorder)
        "10028000",  # Uncomplicated sedative, hypnotic AND/OR anxiolytic withdrawal (disorder)
        "724662001",  # Sedative withdrawal syndrome without complication (disorder)
        "724664000",  # Anxiolytic withdrawal syndrome without complication (disorder)
        "23601000119102",  # Sedative withdrawal syndrome (disorder)
        "361151007",  # Delirium due to sedative withdrawal (disorder)
        "703849002",  # Benzodiazepine withdrawal syndrome (disorder)
        "703850002",  # Delirium due to benzodiazepine withdrawal (disorder)
        "724660009",  # Hypnotic withdrawal syndrome (disorder)
        "724661008",  # Anxiolytic withdrawal syndrome (disorder)
        "724663006",  # Hypnotic withdrawal syndrome without complication (disorder)
        "724665004",  # Perceptual disturbances co-occurrent and due to sedative withdrawal (disorder)
        "724666003",  # Perceptual disturbances co-occurrent and due to hypnotic withdrawal (disorder)
        "724667007",  # Perceptual disturbances co-occurrent and due to anxiolytic withdrawal (disorder)
        "724668002",  # Seizure co-occurrent and due to sedative withdrawal (disorder)
        "724669005",  # Seizure co-occurrent and due to hypnotic withdrawal (disorder)
        "724670006",  # Seizure co-occurrent and due to anxiolytic withdrawal (disorder)
        "724671005",  # Perceptual disturbances and seizures co-occurrent and due to sedative withdrawal (disorder)
        "724672003",  # Perceptual disturbances and seizures co-occurrent and due to hypnotic withdrawal (disorder)
    }

class BipolarDisorder(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses related to bipolar disorder.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of bipolar disorder.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Bipolar Disorder"
    OID = "2.16.840.1.113883.3.67.1.101.1.128"
    DEFINITION_VERSION = "20240214"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "F3010",  # Manic episode without psychotic symptoms, unspecified
        "F3011",  # Manic episode without psychotic symptoms, mild
        "F3012",  # Manic episode without psychotic symptoms, moderate
        "F3013",  # Manic episode, severe, without psychotic symptoms
        "F302",  # Manic episode, severe with psychotic symptoms
        "F303",  # Manic episode in partial remission
        "F304",  # Manic episode in full remission
        "F308",  # Other manic episodes
        "F309",  # Manic episode, unspecified
        "F310",  # Bipolar disorder, current episode hypomanic
        "F3110",  # Bipolar disorder, current episode manic without psychotic features, unspecified
        "F3111",  # Bipolar disorder, current episode manic without psychotic features, mild
        "F3112",  # Bipolar disorder, current episode manic without psychotic features, moderate
        "F3113",  # Bipolar disorder, current episode manic without psychotic features, severe
        "F312",  # Bipolar disorder, current episode manic severe with psychotic features
        "F3130",  # Bipolar disorder, current episode depressed, mild or moderate severity, unspecified
        "F3131",  # Bipolar disorder, current episode depressed, mild
        "F3132",  # Bipolar disorder, current episode depressed, moderate
        "F314",  # Bipolar disorder, current episode depressed, severe, without psychotic features
        "F315",  # Bipolar disorder, current episode depressed, severe, with psychotic features
        "F3160",  # Bipolar disorder, current episode mixed, unspecified
        "F3161",  # Bipolar disorder, current episode mixed, mild
        "F3162",  # Bipolar disorder, current episode mixed, moderate
        "F3163",  # Bipolar disorder, current episode mixed, severe, without psychotic features
        "F3164",  # Bipolar disorder, current episode mixed, severe, with psychotic features
        "F3170",  # Bipolar disorder, currently in remission, most recent episode unspecified
        "F3171",  # Bipolar disorder, in partial remission, most recent episode hypomanic
        "F3172",  # Bipolar disorder, in full remission, most recent episode hypomanic
        "F3173",  # Bipolar disorder, in partial remission, most recent episode manic
        "F3174",  # Bipolar disorder, in full remission, most recent episode manic
        "F3175",  # Bipolar disorder, in partial remission, most recent episode depressed
        "F3176",  # Bipolar disorder, in full remission, most recent episode depressed
        "F3177",  # Bipolar disorder, in partial remission, most recent episode mixed
        "F3178",  # Bipolar disorder, in full remission, most recent episode mixed
        "F3181",  # Bipolar II disorder
        "F3189",  # Other bipolar disorder
        "F319",  # Bipolar disorder, unspecified
    }

    SNOMEDCT = {
        "10875004",  # Severe mixed bipolar I disorder with psychotic features, mood-incongruent (disorder)
        "10981006",  # Severe mixed bipolar I disorder with psychotic features (disorder)
        "111485001",  # Mixed bipolar I disorder in full remission (disorder)
        "1196001",  # Chronic bipolar II disorder, most recent episode major depressive (disorder)
        "12969000",  # Severe bipolar II disorder, most recent episode major depressive, in full remission (disorder)
        "133091000119105",  # Rapid cycling bipolar I disorder (disorder)
        "13313007",  # Mild bipolar disorder (disorder)
        "13581000",  # Severe bipolar I disorder, single manic episode with psychotic features, mood-congruent (disorder)
        "13746004",  # Bipolar disorder (disorder)
        "14495005",  # Severe bipolar I disorder, single manic episode without psychotic features (disorder)
        "1499003",  # Bipolar I disorder, single manic episode with postpartum onset (disorder)
        "162004",  # Severe manic bipolar I disorder without psychotic features (disorder)
        "16238741000119105",  # Bipolar disorder caused by drug (disorder)
        "16295005",  # Bipolar II disorder, most recent episode major depressive (disorder)
        "16506000",  # Mixed bipolar I disorder (disorder)
        "17782008",  # Bipolar I disorder, most recent episode manic with catatonic features (disorder)
        "191618007",  # Bipolar affective disorder, current episode manic (disorder)
        "191620005",  # Bipolar affective disorder, currently manic, mild (disorder)
        "191621009",  # Bipolar affective disorder, currently manic, moderate (disorder)
        "191623007",  # Bipolar affective disorder, currently manic, severe, with psychosis (disorder)
        "191627008",  # Bipolar affective disorder, current episode depression (disorder)
        "191629006",  # Bipolar affective disorder, currently depressed, mild (disorder)
        "191630001",  # Bipolar affective disorder, currently depressed, moderate (disorder)
        "191636007",  # Mixed bipolar affective disorder (disorder)
        "191638008",  # Mixed bipolar affective disorder, mild (disorder)
        "191639000",  # Mixed bipolar affective disorder, moderate (disorder)
        "191641004",  # Mixed bipolar affective disorder, severe, with psychosis (disorder)
        "191643001",  # Mixed bipolar affective disorder, in full remission (disorder)
        "192362008",  # Bipolar affective disorder, current episode mixed (disorder)
        "19300006",  # Severe bipolar II disorder, most recent episode major depressive with psychotic features, mood-congruent (disorder)
        "20960007",  # Severe bipolar II disorder, most recent episode major depressive with psychotic features, mood-incongruent (disorder)
        "21900002",  # Bipolar I disorder, most recent episode depressed with catatonic features (disorder)
        "22121000",  # Depressed bipolar I disorder in full remission (disorder)
        "22407005",  # Bipolar II disorder, most recent episode major depressive with catatonic features (disorder)
        "231444002",  # Organic bipolar disorder (disorder)
        "23741000119105",  # Severe manic bipolar I disorder (disorder)
        "261000119107",  # Severe depressed bipolar I disorder (disorder)
        "26203008",  # Severe depressed bipolar I disorder with psychotic features, mood-incongruent (disorder)
        "26530004",  # Severe bipolar disorder with psychotic features, mood-incongruent (disorder)
        "271000119101",  # Severe mixed bipolar I disorder (disorder)
        "28663008",  # Severe manic bipolar I disorder with psychotic features (disorder)
        "28884001",  # Moderate bipolar I disorder, single manic episode (disorder)
        "29929003",  # Bipolar I disorder, most recent episode depressed with atypical features (disorder)
        "30520009",  # Severe bipolar II disorder, most recent episode major depressive with psychotic features (disorder)
        "30687003",  # Bipolar II disorder, most recent episode major depressive with postpartum onset (disorder)
        "30935000",  # Manic bipolar I disorder in full remission (disorder)
        "31446002",  # Bipolar I disorder, most recent episode hypomanic (disorder)
        "33380008",  # Severe manic bipolar I disorder with psychotic features, mood-incongruent (disorder)
        "34315001",  # Bipolar II disorder, most recent episode major depressive with melancholic features (disorder)
        "3530005",  # Bipolar I disorder, single manic episode, in full remission (disorder)
        "35481005",  # Mixed bipolar I disorder in remission (disorder)
        "35722002",  # Severe bipolar II disorder, most recent episode major depressive, in remission (disorder)
        "35846004",  # Moderate bipolar II disorder, most recent episode major depressive (disorder)
        "36583000",  # Mixed bipolar I disorder in partial remission (disorder)
        "371596008",  # Bipolar I disorder (disorder)
        "371599001",  # Severe bipolar I disorder (disorder)
        "371600003",  # Severe bipolar disorder (disorder)
        "371604007",  # Severe bipolar II disorder (disorder)
        "40926005",  # Moderate mixed bipolar I disorder (disorder)
        "41552001",  # Mild bipolar I disorder, single manic episode (disorder)
        "41832009",  # Severe bipolar I disorder, single manic episode with psychotic features (disorder)
        "41836007",  # Bipolar disorder in full remission (disorder)
        "43568002",  # Bipolar II disorder, most recent episode major depressive with atypical features (disorder)
        "43769008",  # Mild mixed bipolar I disorder (disorder)
        "4441000",  # Severe bipolar disorder with psychotic features (disorder)
        "45479006",  # Manic bipolar I disorder in remission (disorder)
        "46229002",  # Severe mixed bipolar I disorder without psychotic features (disorder)
        "48937005",  # Bipolar II disorder, most recent episode hypomanic (disorder)
        "49468007",  # Depressed bipolar I disorder (disorder)
        "49512000",  # Depressed bipolar I disorder in partial remission (disorder)
        "51637008",  # Chronic bipolar I disorder, most recent episode depressed (disorder)
        "53049002",  # Severe bipolar disorder without psychotic features (disorder)
        "53607008",  # Depressed bipolar I disorder in remission (disorder)
        "54761006",  # Severe depressed bipolar I disorder with psychotic features, mood-congruent (disorder)
        "55516002",  # Bipolar I disorder, most recent episode manic with postpartum onset (disorder)
        "5703000",  # Bipolar disorder in partial remission (disorder)
        "59617007",  # Severe depressed bipolar I disorder with psychotic features (disorder)
        "61403008",  # Severe depressed bipolar I disorder without psychotic features (disorder)
        "63249007",  # Manic bipolar I disorder in partial remission (disorder)
        "64731001",  # Severe mixed bipolar I disorder with psychotic features, mood-congruent (disorder)
        "65042007",  # Bipolar I disorder, most recent episode mixed with postpartum onset (disorder)
        "66631006",  # Moderate depressed bipolar I disorder (disorder)
        "67002003",  # Severe bipolar II disorder, most recent episode major depressive, in partial remission (disorder)
        "68569003",  # Manic bipolar I disorder (disorder)
        "70546001",  # Severe bipolar disorder with psychotic features, mood-congruent (disorder)
        "71294008",  # Mild bipolar II disorder, most recent episode major depressive (disorder)
        "71984005",  # Mild manic bipolar I disorder (disorder)
        "723903001",  # Bipolar type I disorder currently in full remission (disorder)
        "723905008",  # Bipolar type II disorder currently in full remission (disorder)
        "73471000",  # Bipolar I disorder, most recent episode mixed with catatonic features (disorder)
        "74686005",  # Mild depressed bipolar I disorder (disorder)
        "75360000",  # Bipolar I disorder, single manic episode, in remission (disorder)
        "75752004",  # Bipolar I disorder, most recent episode depressed with melancholic features (disorder)
        "765176007",  # Psychosis and severe depression co-occurrent and due to bipolar affective disorder (disorder)
        "767631007",  # Bipolar disorder, most recent episode depression (disorder)
        "767632000",  # Bipolar disorder, most recent episode manic (disorder)
        "767633005",  # Bipolar affective disorder, most recent episode mixed (disorder)
        "767635003",  # Bipolar I disorder, most recent episode manic (disorder)
        "767636002",  # Bipolar I disorder, most recent episode depression (disorder)
        "78269000",  # Bipolar I disorder, single manic episode, in partial remission (disorder)
        "78640000",  # Severe manic bipolar I disorder with psychotic features, mood-congruent (disorder)
        "789061003",  # Rapid cycling bipolar II disorder (disorder)
        "79584002",  # Moderate bipolar disorder (disorder)
        "81319007",  # Severe bipolar II disorder, most recent episode major depressive without psychotic features (disorder)
        "82998009",  # Moderate manic bipolar I disorder (disorder)
        "83225003",  # Bipolar II disorder (disorder)
        "85248005",  # Bipolar disorder in remission (disorder)
        "86058007",  # Severe bipolar I disorder, single manic episode with psychotic features, mood-incongruent (disorder)
        "87203005",  # Bipolar I disorder, most recent episode depressed with postpartum onset (disorder)
        "87950005",  # Bipolar I disorder, single manic episode with catatonic features (disorder)
        "9340000",  # Bipolar I disorder, single manic episode (disorder)
    }

class GeneralizedAnxietyDisorder(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of generalized anxiety disorder.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of generalized anxiety disorder, panic disorder or other anxiety disorders.

    **Exclusion Criteria:** Excludes concepts that represent a diagnosis of phobic anxiety disorders.
    """

    VALUE_SET_NAME = "Generalized Anxiety Disorder"
    OID = "2.16.840.1.113883.3.464.1003.105.12.1210"
    DEFINITION_VERSION = "20210224"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "F410",  # Panic disorder [episodic paroxysmal anxiety]
        "F411",  # Generalized anxiety disorder
        "F413",  # Other mixed anxiety disorders
        "F418",  # Other specified anxiety disorders
        "F419",  # Anxiety disorder, unspecified
    }

    SNOMEDCT = {
        "191738003",  # Obsessional neurosis (disorder)
        "192014006",  # Psychogenic rumination (disorder)
        "191737008",  # Compulsive neurosis (disorder)
        "10586006",  # Occupation-related stress disorder (disorder)
        "10743001000119103",  # Anxiety disorder in mother complicating childbirth (disorder)
        "109006",  # Anxiety disorder of childhood OR adolescence (disorder)
        "111487009",  # Dream anxiety disorder (disorder)
        "111490003",  # Panic disorder with agoraphobia, agoraphobic avoidance in partial remission AND severe panic attacks (disorder)
        "111491004",  # Panic disorder with agoraphobia, agoraphobic avoidance in full remission AND panic attacks in partial remission (disorder)
        "11806006",  # Separation anxiety disorder of childhood (disorder)
        "11941006",  # Panic disorder with agoraphobia, agoraphobic avoidance in full remission AND panic attacks in full remission (disorder)
        "12398201000119102",  # Anxiety disorder caused by methamphetamine (disorder)
        "126943008",  # Separation anxiety (disorder)
        "13438001",  # Overanxious disorder of childhood (disorder)
        "1380006",  # Agoraphobia without history of panic disorder with limited symptom attacks (disorder)
        "15277004",  # Hallucinogen-induced anxiety disorder (disorder)
        "16264621000119109",  # Recurrent mild major depressive disorder co-occurrent with anxiety (disorder)
        "16264821000119108",  # Recurrent severe major depressive disorder co-occurrent with anxiety (disorder)
        "16264901000119109",  # Recurrent moderate major depressive disorder co-occurrent with anxiety (disorder)
        "16265061000119105",  # Recurrent major depressive disorder co-occurrent with anxiety in full remission (disorder)
        "16265301000119106",  # Recurrent major depressive disorder in partial remission co-occurrent with anxiety (disorder)
        "16265951000119109",  # Mild major depressive disorder co-occurrent with anxiety single episode (disorder)
        "16266831000119100",  # Moderate major depressive disorder co-occurrent with anxiety single episode (disorder)
        "16266991000119108",  # Severe major depressive disorder co-occurrent with anxiety single episode (disorder)
        "1686006",  # Sedative, hypnotic AND/OR anxiolytic-induced anxiety disorder (disorder)
        "17496003",  # Organic anxiety disorder (disorder)
        "1816003",  # Panic disorder with agoraphobia, severe agoraphobic avoidance AND mild panic attacks (disorder)
        "191722009",  # Agoraphobia with panic attacks (disorder)
        "191724005",  # Social phobia, fear of eating in public (disorder)
        "191725006",  # Social phobia, fear of public speaking (disorder)
        "191726007",  # Social phobia, fear of public washing (disorder)
        "191736004",  # Obsessive-compulsive disorder (disorder)
        "192037000",  # Acute panic state due to acute stress reaction (disorder)
        "192038005",  # Acute fugue state due to acute stress reaction (disorder)
        "192039002",  # Acute stupor state due to acute stress reaction (disorder)
        "192041001",  # Acute situational disturbance (disorder)
        "192042008",  # Acute post-trauma stress state (disorder)
        "192044009",  # Stress reaction causing mixed disturbance of emotion and conduct (disorder)
        "192611004",  # Childhood phobic anxiety disorder (disorder)
        "197480006",  # Anxiety disorder (disorder)
        "19766004",  # Panic disorder with agoraphobia, mild agoraphobic avoidance AND severe panic attacks (disorder)
        "20876004",  # Inhalant-induced anxiety disorder (disorder)
        "21897009",  # Generalized anxiety disorder (disorder)
        "22230001",  # Panic disorder with agoraphobia, agoraphobic avoidance in partial remission AND panic attacks in full remission (disorder)
        "22621000119103",  # Anxiety disorder caused by drug (disorder)
        "231504006",  # Mixed anxiety and depressive disorder (disorder)
        "238965007",  # Venereophobia (disorder)
        "238966008",  # Syphilophobia (disorder)
        "24781009",  # Panic disorder with agoraphobia, mild agoraphobic avoidance AND panic attacks in full remission (disorder)
        "25501002",  # Social phobia (disorder)
        "268669006",  # Childhood and adolescent disturbance with elective mutism (disorder)
        "279611005",  # Shell shock (disorder)
        "30059008",  # Panic disorder with agoraphobia, severe agoraphobic avoidance AND moderate panic attacks (disorder)
        "313182004",  # Chronic post-traumatic stress disorder (disorder)
        "3158007",  # Panic disorder with agoraphobia, agoraphobic avoidance in partial remission AND panic attacks in partial remission (disorder)
        "31781004",  # Panic disorder with agoraphobia, agoraphobic avoidance in partial remission AND mild panic attacks (disorder)
        "317816007",  # Stockholm syndrome (disorder)
        "318784009",  # Posttraumatic stress disorder, delayed onset (disorder)
        "32388005",  # Panic disorder with agoraphobia, agoraphobic avoidance in partial remission AND moderate panic attacks (disorder)
        "34116005",  # Panic disorder with agoraphobia, agoraphobic avoidance in full remission AND severe panic attacks (disorder)
        "34938008",  # Anxiety disorder caused by alcohol (disorder)
        "35607004",  # Panic disorder with agoraphobia (disorder)
        "371631005",  # Panic disorder (disorder)
        "37868008",  # Anxiety disorder of adolescence (disorder)
        "37872007",  # Avoidant disorder of childhood OR adolescence (disorder)
        "38328002",  # Panic disorder with agoraphobia, severe agoraphobic avoidance AND panic attacks in full remission (disorder)
        "386810004",  # Phobic disorder (disorder)
        "39951001",  # Cannabis-induced anxiety disorder (disorder)
        "403593004",  # Phobic fear of skin cancer (disorder)
        "426174008",  # Chronic stress disorder (disorder)
        "428687006",  # Nightmares associated with chronic post-traumatic stress disorder (disorder)
        "43150009",  # Panic disorder without agoraphobia with severe panic attacks (disorder)
        "443919007",  # Complex posttraumatic stress disorder (disorder)
        "446175003",  # Acute posttraumatic stress disorder following military combat (disorder)
        "446180007",  # Delayed posttraumatic stress disorder following military combat (disorder)
        "47505003",  # Posttraumatic stress disorder (disorder)
        "4932002",  # Panic disorder with agoraphobia, moderate agoraphobic avoidance AND mild panic attacks (disorder)
        "49564006",  # Panic disorder with agoraphobia, mild agoraphobic avoidance AND moderate panic attacks (disorder)
        "50026000",  # Organic anxiety disorder caused by psychoactive substance (disorder)
        "50983008",  # Panic disorder with agoraphobia, mild agoraphobic avoidance AND panic attacks in partial remission (disorder)
        "51493001",  # Anxiety disorder caused by cocaine (disorder)
        "52910006",  # Anxiety disorder due to a general medical condition (disorder)
        "53467004",  # Anxiety disorder of childhood (disorder)
        "53956006",  # Panic disorder without agoraphobia with panic attacks in partial remission (disorder)
        "54587008",  # Simple phobia (disorder)
        "5509004",  # Panic disorder with agoraphobia AND severe panic attacks (disorder)
        "55967005",  # Anxiety disorder caused by phencyclidine (disorder)
        "56576003",  # Panic disorder without agoraphobia (disorder)
        "58535001",  # Physical AND emotional exhaustion state (disorder)
        "59923000",  # Panic disorder with agoraphobia AND panic attacks in full remission (disorder)
        "61157009",  # Combat fatigue (disorder)
        "61212007",  # Panic disorder with agoraphobia, severe agoraphobic avoidance AND severe panic attacks (disorder)
        "61569007",  # Agoraphobia without history of panic disorder (disorder)
        "62351001",  # Generalized social phobia (disorder)
        "63701002",  # Panic disorder with agoraphobia, mild agoraphobic avoidance AND mild panic attacks (disorder)
        "63909006",  # Panic disorder with agoraphobia AND panic attacks in partial remission (disorder)
        "64060000",  # Panic disorder with agoraphobia, moderate agoraphobic avoidance AND panic attacks in full remission (disorder)
        "64165008",  # Avoidant disorder of childhood (disorder)
        "65064003",  # Panic disorder without agoraphobia with moderate panic attacks (disorder)
        "67195008",  # Acute stress disorder (disorder)
        "69479009",  # Anxiety hyperventilation (disorder)
        "699241002",  # Chronic post-traumatic stress disorder following military combat (disorder)
        "70655008",  # Caffeine-induced anxiety disorder (disorder)
        "70691001",  # Agoraphobia (disorder)
        "71959007",  # Elective mutism (disorder)
        "723913009",  # Olfactory reference disorder (disorder)
        "724654009",  # Anxiety disorder caused by opioid (disorder)
        "724693000",  # Obsessive compulsive disorder caused by cocaine (disorder)
        "724708007",  # Anxiety disorder caused by methylenedioxymethamphetamine (disorder)
        "724722007",  # Anxiety disorder caused by dissociative drug (disorder)
        "724723002",  # Anxiety disorder caused by ketamine (disorder)
        "724730008",  # Obsessive compulsive disorder caused by psychoactive substance (disorder)
        "72861004",  # Panic disorder without agoraphobia with mild panic attacks (disorder)
        "737341006",  # Anxiety disorder caused by synthetic cannabinoid (disorder)
        "74010007",  # Panic disorder with agoraphobia, severe agoraphobic avoidance AND panic attacks in partial remission (disorder)
        "762331007",  # Anxiety disorder caused by stimulant (disorder)
        "762332000",  # Obsessive compulsive disorder caused by stimulant (disorder)
        "762515000",  # Anxiety disorder caused by synthetic cathinone (disorder)
        "762516004",  # Obsessive compulsive disorder caused by synthetic cathinone (disorder)
        "76812003",  # Panic disorder with agoraphobia, moderate agoraphobic avoidance AND panic attacks in partial remission (disorder)
        "76868007",  # Panic disorder with agoraphobia, agoraphobic avoidance in full remission AND mild panic attacks (disorder)
        "8185002",  # Panic disorder with agoraphobia AND moderate panic attacks (disorder)
        "82339009",  # Anxiety disorder caused by amfetamine (disorder)
        "82415003",  # Agoraphobia without history of panic disorder without limited symptom attacks (disorder)
        "82494000",  # Panic disorder without agoraphobia with panic attacks in full remission (disorder)
        "82738004",  # Panic disorder with agoraphobia, moderate agoraphobic avoidance AND moderate panic attacks (disorder)
        "83253003",  # Shyness disorder of childhood (disorder)
        "83631006",  # Panic disorder with agoraphobia, moderate agoraphobic avoidance AND severe panic attacks (disorder)
        "85061001",  # Separation anxiety disorder of childhood, early onset (disorder)
        "87798009",  # Panic disorder with agoraphobia, agoraphobic avoidance in full remission AND moderate panic attacks (disorder)
        "89948007",  # Panic disorder with agoraphobia AND mild panic attacks (disorder)
        "90790003",  # Avoidant disorder of adolescence (disorder)
    }

class RemSleepBehaviorDisorder(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of rapid eye movement (REM) sleep behavior disorder.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of REM sleep behavior disorder.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "REM Sleep Behavior Disorder"
    OID = "2.16.840.1.113883.3.464.1003.105.12.1207"
    DEFINITION_VERSION = "20210224"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "G4752",  # REM sleep behavior disorder
    }

    SNOMEDCT = {
        "415238003",  # Rapid eye movement sleep behavior disorder (disorder)
        "69020003",  # Abnormal rapid eye movement sleep (finding)
    }

class Schizophrenia(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of schizophrenia.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of schizophrenia or schizoaffective disorder.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Schizophrenia"
    OID = "2.16.840.1.113883.3.464.1003.105.12.1205"
    DEFINITION_VERSION = "20210224"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "F200",  # Paranoid schizophrenia
        "F201",  # Disorganized schizophrenia
        "F202",  # Catatonic schizophrenia
        "F203",  # Undifferentiated schizophrenia
        "F205",  # Residual schizophrenia
        "F2081",  # Schizophreniform disorder
        "F2089",  # Other schizophrenia
        "F209",  # Schizophrenia, unspecified
        "F250",  # Schizoaffective disorder, bipolar type
        "F251",  # Schizoaffective disorder, depressive type
        "F258",  # Other schizoaffective disorders
        "F259",  # Schizoaffective disorder, unspecified
    }

    SNOMEDCT = {
        "191563001",  # Acute exacerbation of subchronic latent schizophrenia (disorder)
        "111482003",  # Subchronic schizophrenia with acute exacerbations (disorder)
        "111484002",  # Undifferentiated schizophrenia (disorder)
        "1204417003",  # Early onset schizophrenia (disorder)
        "12939007",  # Chronic disorganized schizophrenia (disorder)
        "1335862003",  # Childhood-onset schizophrenia (disorder)
        "14291003",  # Subchronic disorganized schizophrenia with acute exacerbations (disorder)
        "16990005",  # Subchronic schizophrenia (disorder)
        "191526005",  # Schizophrenic disorders (disorder)
        "191527001",  # Simple schizophrenia (disorder)
        "191531007",  # Acute exacerbation of chronic schizophrenia (disorder)
        "191542003",  # Catatonic schizophrenia (disorder)
        "191547009",  # Acute exacerbation of subchronic catatonic schizophrenia (disorder)
        "191548004",  # Acute exacerbation of chronic catatonic schizophrenia (disorder)
        "191554003",  # Acute exacerbation of subchronic paranoid schizophrenia (disorder)
        "191555002",  # Acute exacerbation of chronic paranoid schizophrenia (disorder)
        "191577003",  # Cenesthopathic schizophrenia (disorder)
        "247804008",  # Schizophrenic prodrome (disorder)
        "26025008",  # Residual schizophrenia (disorder)
        "26472000",  # Paraphrenia (disorder)
        "268617001",  # Acute schizophrenic episode (disorder)
        "27387000",  # Subchronic disorganized schizophrenia (disorder)
        "29599000",  # Chronic undifferentiated schizophrenia (disorder)
        "30336007",  # Chronic residual schizophrenia with acute exacerbations (disorder)
        "31658008",  # Chronic paranoid schizophrenia (disorder)
        "35218008",  # Chronic disorganized schizophrenia with acute exacerbation (disorder)
        "35252006",  # Disorganized schizophrenia (disorder)
        "38295006",  # Involutional paraphrenia (disorder)
        "416340002",  # Late onset schizophrenia (disorder)
        "42868002",  # Subchronic catatonic schizophrenia (disorder)
        "441833000",  # Lethal catatonia (disorder)
        "58214004",  # Schizophrenia (disorder)
        "64905009",  # Paranoid schizophrenia (disorder)
        "68995007",  # Chronic catatonic schizophrenia (disorder)
        "7025000",  # Subchronic undifferentiated schizophrenia with acute exacerbations (disorder)
        "70814008",  # Subchronic residual schizophrenia with acute exacerbations (disorder)
        "71103003",  # Chronic residual schizophrenia (disorder)
        "76566000",  # Subchronic residual schizophrenia (disorder)
        "79204003",  # Chronic undifferentiated schizophrenia with acute exacerbations (disorder)
        "79866005",  # Subchronic paranoid schizophrenia (disorder)
        "83746006",  # Chronic schizophrenia (disorder)
        "85861002",  # Subchronic undifferentiated schizophrenia (disorder)
    }

class SeizureDisorder(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of a seizure disorder.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of epilepsy or recurrent seizures.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Seizure Disorder"
    OID = "2.16.840.1.113883.3.464.1003.105.12.1206"
    DEFINITION_VERSION = "20210224"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "G40001",  # Localization-related (focal) (partial) idiopathic epilepsy and epileptic syndromes with seizures of localized onset, not intractable, with status epilepticus
        "G40009",  # Localization-related (focal) (partial) idiopathic epilepsy and epileptic syndromes with seizures of localized onset, not intractable, without status epilepticus
        "G40011",  # Localization-related (focal) (partial) idiopathic epilepsy and epileptic syndromes with seizures of localized onset, intractable, with status epilepticus
        "G40019",  # Localization-related (focal) (partial) idiopathic epilepsy and epileptic syndromes with seizures of localized onset, intractable, without status epilepticus
        "G40101",  # Localization-related (focal) (partial) symptomatic epilepsy and epileptic syndromes with simple partial seizures, not intractable, with status epilepticus
        "G40109",  # Localization-related (focal) (partial) symptomatic epilepsy and epileptic syndromes with simple partial seizures, not intractable, without status epilepticus
        "G40111",  # Localization-related (focal) (partial) symptomatic epilepsy and epileptic syndromes with simple partial seizures, intractable, with status epilepticus
        "G40119",  # Localization-related (focal) (partial) symptomatic epilepsy and epileptic syndromes with simple partial seizures, intractable, without status epilepticus
        "G40201",  # Localization-related (focal) (partial) symptomatic epilepsy and epileptic syndromes with complex partial seizures, not intractable, with status epilepticus
        "G40209",  # Localization-related (focal) (partial) symptomatic epilepsy and epileptic syndromes with complex partial seizures, not intractable, without status epilepticus
        "G40211",  # Localization-related (focal) (partial) symptomatic epilepsy and epileptic syndromes with complex partial seizures, intractable, with status epilepticus
        "G40219",  # Localization-related (focal) (partial) symptomatic epilepsy and epileptic syndromes with complex partial seizures, intractable, without status epilepticus
        "G40301",  # Generalized idiopathic epilepsy and epileptic syndromes, not intractable, with status epilepticus
        "G40309",  # Generalized idiopathic epilepsy and epileptic syndromes, not intractable, without status epilepticus
        "G40311",  # Generalized idiopathic epilepsy and epileptic syndromes, intractable, with status epilepticus
        "G40319",  # Generalized idiopathic epilepsy and epileptic syndromes, intractable, without status epilepticus
        "G40401",  # Other generalized epilepsy and epileptic syndromes, not intractable, with status epilepticus
        "G40409",  # Other generalized epilepsy and epileptic syndromes, not intractable, without status epilepticus
        "G40411",  # Other generalized epilepsy and epileptic syndromes, intractable, with status epilepticus
        "G40419",  # Other generalized epilepsy and epileptic syndromes, intractable, without status epilepticus
        "G4042",  # Cyclin-Dependent Kinase-Like 5 Deficiency Disorder
        "G40501",  # Epileptic seizures related to external causes, not intractable, with status epilepticus
        "G40509",  # Epileptic seizures related to external causes, not intractable, without status epilepticus
        "G40801",  # Other epilepsy, not intractable, with status epilepticus
        "G40802",  # Other epilepsy, not intractable, without status epilepticus
        "G40803",  # Other epilepsy, intractable, with status epilepticus
        "G40804",  # Other epilepsy, intractable, without status epilepticus
        "G40811",  # Lennox-Gastaut syndrome, not intractable, with status epilepticus
        "G40812",  # Lennox-Gastaut syndrome, not intractable, without status epilepticus
        "G40813",  # Lennox-Gastaut syndrome, intractable, with status epilepticus
        "G40814",  # Lennox-Gastaut syndrome, intractable, without status epilepticus
        "G40821",  # Epileptic spasms, not intractable, with status epilepticus
        "G40822",  # Epileptic spasms, not intractable, without status epilepticus
        "G40823",  # Epileptic spasms, intractable, with status epilepticus
        "G40824",  # Epileptic spasms, intractable, without status epilepticus
        "G40833",  # Dravet syndrome, intractable, with status epilepticus
        "G40834",  # Dravet syndrome, intractable, without status epilepticus
        "G4089",  # Other seizures
        "G40901",  # Epilepsy, unspecified, not intractable, with status epilepticus
        "G40909",  # Epilepsy, unspecified, not intractable, without status epilepticus
        "G40911",  # Epilepsy, unspecified, intractable, with status epilepticus
        "G40919",  # Epilepsy, unspecified, intractable, without status epilepticus
        "G40A01",  # Absence epileptic syndrome, not intractable, with status epilepticus
        "G40A09",  # Absence epileptic syndrome, not intractable, without status epilepticus
        "G40A11",  # Absence epileptic syndrome, intractable, with status epilepticus
        "G40A19",  # Absence epileptic syndrome, intractable, without status epilepticus
        "G40B01",  # Juvenile myoclonic epilepsy, not intractable, with status epilepticus
        "G40B09",  # Juvenile myoclonic epilepsy, not intractable, without status epilepticus
        "G40B11",  # Juvenile myoclonic epilepsy, intractable, with status epilepticus
        "G40B19",  # Juvenile myoclonic epilepsy, intractable, without status epilepticus
        "G40C01",  # Lafora progressive myoclonus epilepsy, not intractable, with status epilepticus
        "G40C09",  # Lafora progressive myoclonus epilepsy, not intractable, without status epilepticus
        "G40C11",  # Lafora progressive myoclonus epilepsy, intractable, with status epilepticus
        "G40C19",  # Lafora progressive myoclonus epilepsy, intractable, without status epilepticus
    }

    SNOMEDCT = {
        "170706008",  # Epilepsy associated problems (finding)
        "401178003",  # Epilepsy care arrangement (finding)
        "407616001",  # Epilepsy severity (finding)
        "84161000119100",  # Partial epileptic seizure of parietal lobe with impairment of consciousness (disorder)
        "84171000119106",  # Partial epileptic seizure of frontal lobe with impairment of consciousness (disorder)
        "84181000119109",  # Partial epileptic seizure of occipital lobe with impairment of consciousness (disorder)
        "84191000119107",  # Partial epileptic seizure of temporal lobe with impairment of consciousness (disorder)
        "230401007",  # Non-progressive Kozhevnikow syndrome (disorder)
        "698763000",  # Postoperative status epilepticus (disorder)
        "111498005",  # Extratemporal epilepsy (disorder)
        "1163527006",  # Epilepsy due to parasitic disease (disorder)
        "1163529009",  # Epilepsy due to bacterial endocarditis (disorder)
        "116401000119105",  # Recurrent complex partial epilepsy (disorder)
        "1167371007",  # Polyhydramnios, megalencephaly, symptomatic epilepsy syndrome (disorder)
        "1172593006",  # Early-onset progressive diffuse brain atrophy, microcephaly, muscle weakness, optic atrophy syndrome (disorder)
        "1172630000",  # Global developmental delay, neuro-ophthalmological abnormalities, seizures, intellectual disability syndrome (disorder)
        "1172844009",  # Combined oxidative phosphorylation defect type 27 (disorder)
        "1179282009",  # Severe neurodevelopmental disorder with feeding difficulties, stereotypic hand movement, bilateral cataract (disorder)
        "1179359005",  # Maternal epilepsy due to perinatal stroke (disorder)
        "1179360000",  # Fetal epilepsy due to perinatal stroke (disorder)
        "1179547007",  # Neonatal epilepsy due to perinatal stroke (disorder)
        "1187042007",  # Early-onset seizures, distal limb anomalies, facial dysmorphism, global developmental delay syndrome (disorder)
        "1187210007",  # Intellectual disability, epilepsy, extrapyramidal syndrome (disorder)
        "1187250005",  # Seizures, scoliosis, macrocephaly syndrome (disorder)
        "1187278006",  # Spastic paraplegia, severe developmental delay, epilepsy syndrome (disorder)
        "1187304005",  # Macrocephaly, intellectual disability, neurodevelopmental disorder, small thorax syndrome (disorder)
        "1197587003",  # Lethal neonatal spasticity, epileptic encephalopathy syndrome (disorder)
        "1208486005",  # Multiple mitochondrial dysfunctions syndrome type 2 (disorder)
        "1208939001",  # Progressive myoclonic epilepsy type 7 (disorder)
        "1217381009",  # Neurodevelopmental delay, seizures, ophthalmic anomalies, osteopenia, cerebellar atrophy syndrome (disorder)
        "1222656005",  # Synaptic Ras GTPase activating protein 1-related developmental and epileptic encephalopathy (disorder)
        "1222659003",  # Ring finger protein 13-related severe early-onset epileptic encephalopathy (disorder)
        "1222662000",  # Neonatal epileptic encephalopathy due to deficiency of glutaminase (disorder)
        "1228857005",  # Progressive myoclonic epilepsy type 9 (disorder)
        "1230376005",  # Contactin associated protein 2-related developmental and epileptic encephalopathy (disorder)
        "1231737000",  # Rolandic epilepsy, paroxysmal exercise-induced dystonia, writer's cramp syndrome (disorder)
        "1237417007",  # Carbamoyl-phosphate synthetase 2, aspartate transcarbamylase, and dihydroorotase congenital disorder of glycosylation (disorder)
        "1237619001",  # Fatty acyl-coenzyme A reductase 1 deficiency (disorder)
        "1254960008",  # Seizure co-occurrent and due to substance withdrawal (disorder)
        "1260117009",  # Combined focal and generalized epilepsy (disorder)
        "1260368003",  # Epilepsy due to Rasmussen syndrome (disorder)
        "1260369006",  # Epilepsy due to perinatal intraventricular hemorrhage (disorder)
        "1260370007",  # Epilepsy due to perinatal periventricular hemorrhage (disorder)
        "1260371006",  # Epilepsy due to perinatal cerebral ischemia (disorder)
        "1260374003",  # Epilepsy due to congenital infectious disease (disorder)
        "1260375002",  # Epilepsy due to glucose transporter protein type 1 deficiency syndrome (disorder)
        "1260377005",  # Epilepsy due to congenital anomaly of brain (disorder)
        "1260407007",  # Genetic generalized epilepsy (disorder)
        "1269236003",  # Primary hypomagnesemia, refractory seizures, intellectual disability syndrome (disorder)
        "1275631007",  # Developmental and epileptic encephalopathy (disorder)
        "128613002",  # Seizure disorder (disorder)
        "13973009",  # Tonic-clonic status epilepticus (finding)
        "14401000119109",  # Partial frontal lobe epilepsy (disorder)
        "170709001",  # Epilepsy drug side effect (finding)
        "191714002",  # Dissociative convulsions (disorder)
        "192845009",  # Myoclonic encephalopathy (disorder)
        "192990004",  # Myoclonic epilepsy in infancy (disorder)
        "192999003",  # Partial epilepsy with impairment of consciousness (disorder)
        "193000002",  # Temporal lobe epilepsy (disorder)
        "193002005",  # Psychosensory epilepsy (disorder)
        "193003000",  # Mesiobasal limbic epilepsy (disorder)
        "193008009",  # Somatosensory epilepsy (disorder)
        "193009001",  # Partial epilepsy with autonomic symptoms (disorder)
        "193010006",  # Visual reflex epilepsy (disorder)
        "193021002",  # Cursive (running) epilepsy (disorder)
        "19598007",  # Generalized epilepsy (disorder)
        "20121000119105",  # Partial occipital lobe epilepsy (disorder)
        "21391000119102",  # Partial parietal lobe epilepsy (disorder)
        "2198002",  # Visceral epilepsy (disorder)
        "230191005",  # Rasmussen syndrome (disorder)
        "230381009",  # Focal epilepsy (disorder)
        "230389006",  # Primary inherited reading epilepsy (disorder)
        "230390002",  # Localization-related symptomatic epilepsy (disorder)
        "230391003",  # Amygdalo-hippocampal epilepsy (disorder)
        "230392005",  # Rhinencephalic epilepsy (disorder)
        "230393000",  # Lateral temporal epilepsy (disorder)
        "230394006",  # Frontal lobe epilepsy (disorder)
        "230395007",  # Supplementary motor epilepsy (disorder)
        "230396008",  # Cingulate epilepsy (disorder)
        "230397004",  # Anterior frontopolar epilepsy (disorder)
        "230398009",  # Orbitofrontal epilepsy (disorder)
        "230399001",  # Dorsolateral epilepsy (disorder)
        "230400008",  # Opercular epilepsy (disorder)
        "230403005",  # Parietal lobe epilepsy (disorder)
        "230404004",  # Occipital lobe epilepsy (disorder)
        "230406002",  # Localization-related symptomatic epilepsy with specific precipitant (disorder)
        "230407006",  # Hemiplegia-hemiconvulsion-epilepsy syndrome (disorder)
        "230408001",  # Localization-related cryptogenic epilepsy (disorder)
        "230414008",  # Epilepsy with generalized tonic-clonic seizures alone (disorder)
        "230416005",  # Cryptogenic West syndrome (disorder)
        "230417001",  # Symptomatic West syndrome (disorder)
        "230418006",  # Lennox-Gastaut syndrome (disorder)
        "230419003",  # Cryptogenic Lennox-Gastaut syndrome (disorder)
        "230420009",  # Symptomatic Lennox-Gastaut syndrome (disorder)
        "230421008",  # Epilepsy with myoclonic-atonic seizures (disorder)
        "230422001",  # Epilepsy with myoclonic absence (disorder)
        "230423006",  # Unverricht-Lundborg syndrome (disorder)
        "230425004",  # Lafora disease (disorder)
        "230426003",  # Myoclonic epilepsy with ragged red fibers (disorder)
        "230432008",  # Familial febrile convulsions (disorder)
        "230437002",  # Severe myoclonic epilepsy in infancy (disorder)
        "230438007",  # Acquired epileptic aphasia (disorder)
        "230439004",  # Epilepsy with continuous spike wave during slow-wave sleep (disorder)
        "230440002",  # Secondary reading epilepsy (disorder)
        "230443000",  # Narcotic withdrawal epilepsy (disorder)
        "230444006",  # Menstrual epilepsy (disorder)
        "230445007",  # Nocturnal epilepsy (disorder)
        "230448009",  # Writing epilepsy (disorder)
        "230450001",  # Eating epilepsy (disorder)
        "230452009",  # Toothbrushing epilepsy (disorder)
        "230453004",  # Decision-making epilepsy (disorder)
        "230454005",  # Aquagenic epilepsy (disorder)
        "230456007",  # Status epilepticus (finding)
        "230457003",  # Typical absence status epilepticus (finding)
        "230458008",  # Atypical absence status epilepticus (finding)
        "23374007",  # Atypical absence seizure (finding)
        "243857008",  # Epilepsy monitoring status (finding)
        "267581004",  # Progressive myoclonic epilepsy (disorder)
        "267592003",  # Motor cortex epilepsy (disorder)
        "278510009",  # Localization-related idiopathic epilepsy (disorder)
        "28055006",  # West syndrome (disorder)
        "290731000119106",  # Idiopathic partial status epilepticus (finding)
        "290741000119102",  # Intractable idiopathic partial epilepsy (disorder)
        "307356008",  # Motor epilepsy (disorder)
        "307357004",  # Jacksonian, focal or motor epilepsy (disorder)
        "314827004",  # Epilepsy control good (finding)
        "314828009",  # Epilepsy control poor (finding)
        "322112361000132104",  # Epilepsy due to scarring of brain (disorder)
        "361123003",  # Psychomotor epilepsy (disorder)
        "36803009",  # Idiopathic generalized epilepsy (disorder)
        "39194005",  # Visual epilepsy (disorder)
        "407623000",  # Emergency epilepsy treatment since last appointment (finding)
        "407675009",  # Focal onset impaired awareness epileptic seizure (finding)
        "413101007",  # Stress-induced epilepsy (disorder)
        "430061000124109",  # Early myoclonic encephalopathy, refractory (disorder)
        "430071000124102",  # Early myoclonic encephalopathy, non-refractory (disorder)
        "431111000124104",  # Lennox-Gastaut syndrome, refractory (disorder)
        "431121000124107",  # Lennox-Gastaut syndrome, non-refractory (disorder)
        "431141000124100",  # Myoclonic absence epilepsy, non-refractory (disorder)
        "431991000124107",  # Petit mal status, non-refractory (finding)
        "432171000124102",  # Severe myoclonic epilepsy in infancy, refractory (disorder)
        "432271000124109",  # Tonic seizures, refractory (finding)
        "432281000124107",  # Tonic seizures, non-refractory (finding)
        "432511000124103",  # Atonic seizure, non-refractory (finding)
        "432561000124100",  # Typical absence seizure, non-refractory (finding)
        "432571000124107",  # Typical absence seizure, refractory (finding)
        "434521000124102",  # Atypical absence seizure, refractory (finding)
        "434531000124104",  # Atypical absence seizure, non-refractory (finding)
        "435321000124106",  # Acquired epileptic aphasia, non-refractory (disorder)
        "435331000124109",  # Acquired epileptic aphasia, refractory (disorder)
        "435341000124104",  # Rasmussen syndrome, refractory (disorder)
        "436011000124108",  # Petit mal status, refractory (finding)
        "438113009",  # Convulsive syncope (finding)
        "438421000124109",  # Myoclonic astatic epilepsy, non-refractory (disorder)
        "44145005",  # Benign Rolandic epilepsy (disorder)
        "442151000124108",  # Autosomal dominant nocturnal frontal lobe epilepsy, refractory (disorder)
        "442161000124105",  # Autosomal dominant nocturnal frontal lobe epilepsy, non-refractory (disorder)
        "442511009",  # Progressive encephalopathy with edema, hypsarrhythmia and optic atrophy syndrome (disorder)
        "442512002",  # Status epilepticus without prominent motor symptoms (finding)
        "44423001",  # Early myoclonic encephalopathy (disorder)
        "444441000124109",  # Generalized epilepsy with febrile seizures plus, refractory (disorder)
        "444451000124106",  # Generalized epilepsy with febrile seizures plus, non-refractory (disorder)
        "460731000124105",  # Recurrent seizure (disorder)
        "5321000124109",  # Temporal lobe epilepsy with mesial temporal sclerosis (disorder)
        "68761002",  # Epileptic vertigo (disorder)
        "698021005",  # Autosomal dominant nocturnal frontal lobe epilepsy (disorder)
        "698764006",  # Post infectious grand mal epilepsy (disorder)
        "698767004",  # Post-cerebrovascular accident epilepsy (disorder)
        "699688008",  # Genetic epilepsy with febrile seizures plus (disorder)
        "702326000",  # Progressive myoclonus epilepsy with ataxia (disorder)
        "702344008",  # Pitt-Hopkins syndrome (disorder)
        "7033004",  # Absence status epilepticus (finding)
        "703524005",  # Spinal muscular atrophy with progressive myoclonic epilepsy (disorder)
        "715425000",  # Benign focal seizure of adolescence (disorder)
        "715629001",  # Generalized epilepsy and paroxysmal dyskinesia syndrome (disorder)
        "716278005",  # Epilepsy with eyelid myoclonia (disorder)
        "716706009",  # Female restricted epilepsy with intellectual disability syndrome (disorder)
        "717223008",  # X-linked epilepsy with learning disability and behavior disorder syndrome (disorder)
        "717225001",  # Benign adult familial myoclonic epilepsy (disorder)
        "717276003",  # Folinic acid responsive seizure syndrome (disorder)
        "719810000",  # X-linked intellectual disability with seizure and psoriasis syndrome (disorder)
        "720519003",  # Atherosclerosis, deafness, diabetes, epilepsy, nephropathy syndrome (disorder)
        "721088003",  # Developmental delay, epilepsy, neonatal diabetes syndrome (disorder)
        "721207002",  # Seizure, sensorineural deafness, ataxia, intellectual disability, electrolyte imbalance syndrome (disorder)
        "722110003",  # Osteogenesis imperfecta, retinopathy, seizures, intellectual disability syndrome (disorder)
        "722386009",  # Celiac disease with epilepsy and cerebral calcification syndrome (disorder)
        "722762005",  # Ganglioside GM3 synthase deficiency (disorder)
        "723125008",  # Epileptic encephalopathy (disorder)
        "723304001",  # Microcephaly, seizure, intellectual disability, heart disease syndrome (disorder)
        "723676007",  # Severe intellectual disability, epilepsy, anal anomaly, distal phalangeal hypoplasia syndrome (disorder)
        "723926008",  # Perceptual disturbances and seizures co-occurrent and due to alcohol withdrawal (disorder)
        "724274009",  # Infant epilepsy with migrant focal crisis (disorder)
        "724549005",  # Epilepsy due to infectious disease of central nervous system (disorder)
        "724576005",  # Pyridoxal 5-phosphate dependent epilepsy (disorder)
        "724668002",  # Seizure co-occurrent and due to sedative withdrawal (disorder)
        "724669005",  # Seizure co-occurrent and due to hypnotic withdrawal (disorder)
        "724670006",  # Seizure co-occurrent and due to anxiolytic withdrawal (disorder)
        "724671005",  # Perceptual disturbances and seizures co-occurrent and due to sedative withdrawal (disorder)
        "724672003",  # Perceptual disturbances and seizures co-occurrent and due to hypnotic withdrawal (disorder)
        "724727001",  # Seizure co-occurrent and due to psychoactive substance withdrawal (disorder)
        "724728006",  # Perceptual disturbances and seizures co-occurrent and due to psychoactive substance withdrawal (disorder)
        "724786008",  # Epilepsy due to perinatal anoxic-ischemic brain injury (disorder)
        "724787004",  # Epilepsy due to cerebrovascular accident (disorder)
        "724789001",  # Epilepsy due to intracranial tumor (disorder)
        "724988000",  # Epilepsy co-occurrent and due to degenerative brain disorder (disorder)
        "724989008",  # Epilepsy co-occurrent and due to mesial temporal sclerosis (disorder)
        "724990004",  # Epilepsy due to immune disorder (disorder)
        "724991000",  # Epilepsy co-occurrent and due to demyelinating disorder (disorder)
        "724992007",  # Epilepsy co-occurrent and due to dementia (disorder)
        "725163002",  # X-linked spasticity, intellectual disability, epilepsy syndrome (disorder)
        "725413002",  # Febrile infection related epilepsy syndrome (disorder)
        "726702005",  # Epileptic encephalopathy with global cerebral demyelination (disorder)
        "733031004",  # Epilepsy, microcephaly, skeletal dysplasia syndrome (disorder)
        "733032006",  # Epilepsy telangiectasia syndrome (disorder)
        "733082001",  # Early-onset Lafora body disease (disorder)
        "733195008",  # Epilepsy of infancy with migrating focal seizures (disorder)
        "733623005",  # Autism spectrum disorder, epilepsy, arthrogryposis syndrome (disorder)
        "734434007",  # Pyridoxine-dependent epilepsy (disorder)
        "75023009",  # Post-traumatic epilepsy (disorder)
        "763349002",  # Progressive myoclonic epilepsy with dystonia (disorder)
        "763534009",  # Hot water reflex epilepsy (disorder)
        "763622006",  # Thinking epilepsy (disorder)
        "763632004",  # Startle epilepsy (disorder)
        "763802009",  # Micturition induced epilepsy (disorder)
        "763827002",  # Orgasm induced epilepsy (disorder)
        "763861000",  # Pachygyria, intellectual disability, epilepsy syndrome (disorder)
        "764453009",  # Action myoclonus renal failure syndrome (disorder)
        "764522009",  # Familial focal epilepsy with variable foci (disorder)
        "765089003",  # Focal epilepsy, intellectual disability, cerebro-cerebellar malformation syndrome (disorder)
        "765093009",  # Rolandic epilepsy, speech dyspraxia syndrome (disorder)
        "765170001",  # Sodium voltage-gated channel alpha subunit 8-related epilepsy with encephalopathy (disorder)
        "765216006",  # Audiogenic epilepsy (disorder)
        "766044005",  # Acute encephalopathy with biphasic seizures and late reduced diffusion (disorder)
        "766815007",  # Perioral myoclonia with absences (disorder)
        "766932005",  # Hypothalamic hamartoma with gelastic seizure (disorder)
        "768473009",  # Purine rich element binding protein A syndrome (disorder)
        "768555009",  # 5q31.3 microdeletion syndrome (disorder)
        "770405003",  # Benign familial mesial temporal lobe epilepsy (disorder)
        "770431001",  # Early-onset epileptic encephalopathy and intellectual disability due to glutamate receptor, ionotropic, N-methyl-D-aspartate, subunit 2A mutation (disorder)
        "770623004",  # Benign occipital lobe epilepsy (disorder)
        "770624005",  # Benign partial epilepsy of infancy with complex partial seizures (disorder)
        "770643005",  # Mesial temporal lobe epilepsy with hippocampal sclerosis (disorder)
        "770678005",  # Progressive encephalopathy with edema, hypsarrhythmia, and optic atrophy-like syndrome (disorder)
        "770755007",  # Intellectual disability, seizures, hypotonia, ophthalmologic, skeletal anomalies syndrome (disorder)
        "770758009",  # New-onset refractory status epilepticus (disorder)
        "770898002",  # Autosomal recessive cerebellar ataxia, epilepsy, intellectual disability syndrome due to WW domain containing oxidoreductase deficiency (disorder)
        "771141002",  # Benign partial epilepsy with secondarily generalized seizures in infancy (disorder)
        "771303004",  # Severe neonatal onset encephalopathy with microcephaly (disorder)
        "771448004",  # Autism epilepsy syndrome due to branched chain ketoacid dehydrogenase kinase deficiency (disorder)
        "771469002",  # Early-onset spastic ataxia, myoclonic epilepsy, neuropathy syndrome (disorder)
        "771516000",  # Solute carrier family 35 member A2 congenital disorder of glycosylation (disorder)
        "773230003",  # Cyclin-dependent kinase-like 5 deficiency (disorder)
        "773497001",  # Partial corpus callosum agenesis, cerebellar vermis hypoplasia with posterior fossa cysts syndrome (disorder)
        "773498006",  # Autosomal recessive cerebellar ataxia, epilepsy, intellectual disability syndrome due to TUD deficiency (disorder)
        "773548008",  # Early-onset epileptic encephalopathy, cortical blindness, intellectual disability, facial dysmorphism syndrome (disorder)
        "773643006",  # Multiple congenital anomalies, hypotonia, seizures syndrome type 2 (disorder)
        "778001003",  # Potassium voltage-gated channel subfamily Q member 2 related epileptic encephalopathy (disorder)
        "778047006",  # Myoclonic epilepsy in non-progressive encephalopathy (disorder)
        "778063003",  # Cryptogenic late-onset epileptic spasms (disorder)
        "780827006",  # Synaptic Ras GTPase activating protein 1- related intellectual disability (disorder)
        "782737003",  # Diffuse cerebral and cerebellar atrophy, intractable seizures, progressive microcephaly syndrome (disorder)
        "782772000",  # Congenital muscular dystrophy with intellectual disability and severe epilepsy (disorder)
        "782825008",  # Primary microcephaly, epilepsy, permanent neonatal diabetes syndrome (disorder)
        "783055005",  # Progressive myoclonic epilepsy type 5 (disorder)
        "783062001",  # Progressive myoclonic epilepsy type 6 (disorder)
        "783064000",  # Progressive myoclonic epilepsy type 3 (disorder)
        "783139000",  # Progressive myoclonic epilepsy type 8 (disorder)
        "783739005",  # Familial temporal lobe epilepsy (disorder)
        "784345005",  # Malignant migrating partial seizures of infancy (disorder)
        "784372002",  # Familial mesial temporal lobe epilepsy with febrile seizures (disorder)
        "784377008",  # Autosomal dominant epilepsy with auditory features (disorder)
        "785303004",  # Multiple congenital anomalies, hypotonia, seizures syndrome (disorder)
        "785726009",  # Hyperekplexia epilepsy syndrome (disorder)
        "788417006",  # Alopecia, epilepsy, intellectual disability syndrome Moynahan type (disorder)
        "789063000",  # Primary hyperaldosteronism, seizures, neurological abnormalities syndrome (disorder)
        "79631006",  # Absence seizure (finding)
        "79745005",  # Reflex epilepsy (disorder)
        "80651009",  # Aicardi's syndrome (disorder)
        "82381000119103",  # Epileptic dementia with behavioral disturbance (disorder)
        "84211000119108",  # Intractable partial parietal lobe epilepsy with impairment of consciousness (disorder)
        "84757009",  # Epilepsy (disorder)
        "860804005",  # Epilepsy due to infectious encephalitis (disorder)
        "860806007",  # Epilepsy due to infectious meningitis (disorder)
        "860815000",  # Epilepsy due to neonatal central nervous system infection (disorder)
        "95208000",  # Photogenic epilepsy (disorder)
    }

class Cancer(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of cancer.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of cancer.

    **Exclusion Criteria:** Excludes concepts that describe in situ carcinoma.
    """

    VALUE_SET_NAME = "Cancer"
    OID = "2.16.840.1.113883.3.526.3.1010"
    DEFINITION_VERSION = "20200306"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "C000",  # Malignant neoplasm of external upper lip
        "C001",  # Malignant neoplasm of external lower lip
        "C002",  # Malignant neoplasm of external lip, unspecified
        "C003",  # Malignant neoplasm of upper lip, inner aspect
        "C004",  # Malignant neoplasm of lower lip, inner aspect
        "C005",  # Malignant neoplasm of lip, unspecified, inner aspect
        "C006",  # Malignant neoplasm of commissure of lip, unspecified
        "C008",  # Malignant neoplasm of overlapping sites of lip
        "C009",  # Malignant neoplasm of lip, unspecified
        "C01",  # Malignant neoplasm of base of tongue
        "C020",  # Malignant neoplasm of dorsal surface of tongue
        "C021",  # Malignant neoplasm of border of tongue
        "C022",  # Malignant neoplasm of ventral surface of tongue
        "C023",  # Malignant neoplasm of anterior two-thirds of tongue, part unspecified
        "C024",  # Malignant neoplasm of lingual tonsil
        "C028",  # Malignant neoplasm of overlapping sites of tongue
        "C029",  # Malignant neoplasm of tongue, unspecified
        "C030",  # Malignant neoplasm of upper gum
        "C031",  # Malignant neoplasm of lower gum
        "C039",  # Malignant neoplasm of gum, unspecified
        "C040",  # Malignant neoplasm of anterior floor of mouth
        "C041",  # Malignant neoplasm of lateral floor of mouth
        "C048",  # Malignant neoplasm of overlapping sites of floor of mouth
        "C049",  # Malignant neoplasm of floor of mouth, unspecified
        "C050",  # Malignant neoplasm of hard palate
        "C051",  # Malignant neoplasm of soft palate
        "C052",  # Malignant neoplasm of uvula
        "C058",  # Malignant neoplasm of overlapping sites of palate
        "C059",  # Malignant neoplasm of palate, unspecified
        "C060",  # Malignant neoplasm of cheek mucosa
        "C061",  # Malignant neoplasm of vestibule of mouth
        "C062",  # Malignant neoplasm of retromolar area
        "C0680",  # Malignant neoplasm of overlapping sites of unspecified parts of mouth
        "C0689",  # Malignant neoplasm of overlapping sites of other parts of mouth
        "C069",  # Malignant neoplasm of mouth, unspecified
        "C07",  # Malignant neoplasm of parotid gland
        "C080",  # Malignant neoplasm of submandibular gland
        "C081",  # Malignant neoplasm of sublingual gland
        "C089",  # Malignant neoplasm of major salivary gland, unspecified
        "C090",  # Malignant neoplasm of tonsillar fossa
        "C091",  # Malignant neoplasm of tonsillar pillar (anterior) (posterior)
        "C098",  # Malignant neoplasm of overlapping sites of tonsil
        "C099",  # Malignant neoplasm of tonsil, unspecified
        "C100",  # Malignant neoplasm of vallecula
        "C101",  # Malignant neoplasm of anterior surface of epiglottis
        "C102",  # Malignant neoplasm of lateral wall of oropharynx
        "C103",  # Malignant neoplasm of posterior wall of oropharynx
        "C104",  # Malignant neoplasm of branchial cleft
        "C108",  # Malignant neoplasm of overlapping sites of oropharynx
        "C109",  # Malignant neoplasm of oropharynx, unspecified
        "C110",  # Malignant neoplasm of superior wall of nasopharynx
        "C111",  # Malignant neoplasm of posterior wall of nasopharynx
        "C112",  # Malignant neoplasm of lateral wall of nasopharynx
        "C113",  # Malignant neoplasm of anterior wall of nasopharynx
        "C118",  # Malignant neoplasm of overlapping sites of nasopharynx
        "C119",  # Malignant neoplasm of nasopharynx, unspecified
        "C12",  # Malignant neoplasm of pyriform sinus
        "C130",  # Malignant neoplasm of postcricoid region
        "C131",  # Malignant neoplasm of aryepiglottic fold, hypopharyngeal aspect
        "C132",  # Malignant neoplasm of posterior wall of hypopharynx
        "C138",  # Malignant neoplasm of overlapping sites of hypopharynx
        "C139",  # Malignant neoplasm of hypopharynx, unspecified
        "C140",  # Malignant neoplasm of pharynx, unspecified
        "C142",  # Malignant neoplasm of Waldeyer's ring
        "C148",  # Malignant neoplasm of overlapping sites of lip, oral cavity and pharynx
        "C153",  # Malignant neoplasm of upper third of esophagus
        "C154",  # Malignant neoplasm of middle third of esophagus
        "C155",  # Malignant neoplasm of lower third of esophagus
        "C158",  # Malignant neoplasm of overlapping sites of esophagus
        "C159",  # Malignant neoplasm of esophagus, unspecified
        "C160",  # Malignant neoplasm of cardia
        "C161",  # Malignant neoplasm of fundus of stomach
        "C162",  # Malignant neoplasm of body of stomach
        "C163",  # Malignant neoplasm of pyloric antrum
        "C164",  # Malignant neoplasm of pylorus
        "C165",  # Malignant neoplasm of lesser curvature of stomach, unspecified
        "C166",  # Malignant neoplasm of greater curvature of stomach, unspecified
        "C168",  # Malignant neoplasm of overlapping sites of stomach
        "C169",  # Malignant neoplasm of stomach, unspecified
        "C170",  # Malignant neoplasm of duodenum
        "C171",  # Malignant neoplasm of jejunum
        "C172",  # Malignant neoplasm of ileum
        "C173",  # Meckel's diverticulum, malignant
        "C178",  # Malignant neoplasm of overlapping sites of small intestine
        "C179",  # Malignant neoplasm of small intestine, unspecified
        "C180",  # Malignant neoplasm of cecum
        "C181",  # Malignant neoplasm of appendix
        "C182",  # Malignant neoplasm of ascending colon
        "C183",  # Malignant neoplasm of hepatic flexure
        "C184",  # Malignant neoplasm of transverse colon
        "C185",  # Malignant neoplasm of splenic flexure
        "C186",  # Malignant neoplasm of descending colon
        "C187",  # Malignant neoplasm of sigmoid colon
        "C188",  # Malignant neoplasm of overlapping sites of colon
        "C189",  # Malignant neoplasm of colon, unspecified
        "C19",  # Malignant neoplasm of rectosigmoid junction
        "C20",  # Malignant neoplasm of rectum
        "C210",  # Malignant neoplasm of anus, unspecified
        "C211",  # Malignant neoplasm of anal canal
        "C212",  # Malignant neoplasm of cloacogenic zone
        "C218",  # Malignant neoplasm of overlapping sites of rectum, anus and anal canal
        "C220",  # Liver cell carcinoma
        "C221",  # Intrahepatic bile duct carcinoma
        "C222",  # Hepatoblastoma
        "C223",  # Angiosarcoma of liver
        "C224",  # Other sarcomas of liver
        "C227",  # Other specified carcinomas of liver
        "C228",  # Malignant neoplasm of liver, primary, unspecified as to type
        "C229",  # Malignant neoplasm of liver, not specified as primary or secondary
        "C23",  # Malignant neoplasm of gallbladder
        "C240",  # Malignant neoplasm of extrahepatic bile duct
        "C241",  # Malignant neoplasm of ampulla of Vater
        "C248",  # Malignant neoplasm of overlapping sites of biliary tract
        "C249",  # Malignant neoplasm of biliary tract, unspecified
        "C250",  # Malignant neoplasm of head of pancreas
        "C251",  # Malignant neoplasm of body of pancreas
        "C252",  # Malignant neoplasm of tail of pancreas
        "C253",  # Malignant neoplasm of pancreatic duct
        "C254",  # Malignant neoplasm of endocrine pancreas
        "C257",  # Malignant neoplasm of other parts of pancreas
        "C258",  # Malignant neoplasm of overlapping sites of pancreas
        "C259",  # Malignant neoplasm of pancreas, unspecified
        "C260",  # Malignant neoplasm of intestinal tract, part unspecified
        "C261",  # Malignant neoplasm of spleen
        "C269",  # Malignant neoplasm of ill-defined sites within the digestive system
        "C300",  # Malignant neoplasm of nasal cavity
        "C301",  # Malignant neoplasm of middle ear
        "C310",  # Malignant neoplasm of maxillary sinus
        "C311",  # Malignant neoplasm of ethmoidal sinus
        "C312",  # Malignant neoplasm of frontal sinus
        "C313",  # Malignant neoplasm of sphenoid sinus
        "C318",  # Malignant neoplasm of overlapping sites of accessory sinuses
        "C319",  # Malignant neoplasm of accessory sinus, unspecified
        "C320",  # Malignant neoplasm of glottis
        "C321",  # Malignant neoplasm of supraglottis
        "C322",  # Malignant neoplasm of subglottis
        "C323",  # Malignant neoplasm of laryngeal cartilage
        "C328",  # Malignant neoplasm of overlapping sites of larynx
        "C329",  # Malignant neoplasm of larynx, unspecified
        "C33",  # Malignant neoplasm of trachea
        "C3400",  # Malignant neoplasm of unspecified main bronchus
        "C3401",  # Malignant neoplasm of right main bronchus
        "C3402",  # Malignant neoplasm of left main bronchus
        "C3410",  # Malignant neoplasm of upper lobe, unspecified bronchus or lung
        "C3411",  # Malignant neoplasm of upper lobe, right bronchus or lung
        "C3412",  # Malignant neoplasm of upper lobe, left bronchus or lung
        "C342",  # Malignant neoplasm of middle lobe, bronchus or lung
        "C3430",  # Malignant neoplasm of lower lobe, unspecified bronchus or lung
        "C3431",  # Malignant neoplasm of lower lobe, right bronchus or lung
        "C3432",  # Malignant neoplasm of lower lobe, left bronchus or lung
        "C3480",  # Malignant neoplasm of overlapping sites of unspecified bronchus and lung
        "C3481",  # Malignant neoplasm of overlapping sites of right bronchus and lung
        "C3482",  # Malignant neoplasm of overlapping sites of left bronchus and lung
        "C3490",  # Malignant neoplasm of unspecified part of unspecified bronchus or lung
        "C3491",  # Malignant neoplasm of unspecified part of right bronchus or lung
        "C3492",  # Malignant neoplasm of unspecified part of left bronchus or lung
        "C37",  # Malignant neoplasm of thymus
        "C380",  # Malignant neoplasm of heart
        "C381",  # Malignant neoplasm of anterior mediastinum
        "C382",  # Malignant neoplasm of posterior mediastinum
        "C383",  # Malignant neoplasm of mediastinum, part unspecified
        "C384",  # Malignant neoplasm of pleura
        "C388",  # Malignant neoplasm of overlapping sites of heart, mediastinum and pleura
        "C390",  # Malignant neoplasm of upper respiratory tract, part unspecified
        "C399",  # Malignant neoplasm of lower respiratory tract, part unspecified
        "C4000",  # Malignant neoplasm of scapula and long bones of unspecified upper limb
        "C4001",  # Malignant neoplasm of scapula and long bones of right upper limb
        "C4002",  # Malignant neoplasm of scapula and long bones of left upper limb
        "C4010",  # Malignant neoplasm of short bones of unspecified upper limb
        "C4011",  # Malignant neoplasm of short bones of right upper limb
        "C4012",  # Malignant neoplasm of short bones of left upper limb
        "C4020",  # Malignant neoplasm of long bones of unspecified lower limb
        "C4021",  # Malignant neoplasm of long bones of right lower limb
        "C4022",  # Malignant neoplasm of long bones of left lower limb
        "C4030",  # Malignant neoplasm of short bones of unspecified lower limb
        "C4031",  # Malignant neoplasm of short bones of right lower limb
        "C4032",  # Malignant neoplasm of short bones of left lower limb
        "C4080",  # Malignant neoplasm of overlapping sites of bone and articular cartilage of unspecified limb
        "C4081",  # Malignant neoplasm of overlapping sites of bone and articular cartilage of right limb
        "C4082",  # Malignant neoplasm of overlapping sites of bone and articular cartilage of left limb
        "C4090",  # Malignant neoplasm of unspecified bones and articular cartilage of unspecified limb
        "C4091",  # Malignant neoplasm of unspecified bones and articular cartilage of right limb
        "C4092",  # Malignant neoplasm of unspecified bones and articular cartilage of left limb
        "C410",  # Malignant neoplasm of bones of skull and face
        "C411",  # Malignant neoplasm of mandible
        "C412",  # Malignant neoplasm of vertebral column
        "C413",  # Malignant neoplasm of ribs, sternum and clavicle
        "C414",  # Malignant neoplasm of pelvic bones, sacrum and coccyx
        "C419",  # Malignant neoplasm of bone and articular cartilage, unspecified
        "C430",  # Malignant melanoma of lip
        "C4310",  # Malignant melanoma of unspecified eyelid, including canthus
        "C43111",  # Malignant melanoma of right upper eyelid, including canthus
        "C43112",  # Malignant melanoma of right lower eyelid, including canthus
        "C43121",  # Malignant melanoma of left upper eyelid, including canthus
        "C43122",  # Malignant melanoma of left lower eyelid, including canthus
        "C4320",  # Malignant melanoma of unspecified ear and external auricular canal
        "C4321",  # Malignant melanoma of right ear and external auricular canal
        "C4322",  # Malignant melanoma of left ear and external auricular canal
        "C4330",  # Malignant melanoma of unspecified part of face
        "C4331",  # Malignant melanoma of nose
        "C4339",  # Malignant melanoma of other parts of face
        "C434",  # Malignant melanoma of scalp and neck
        "C4351",  # Malignant melanoma of anal skin
        "C4352",  # Malignant melanoma of skin of breast
        "C4359",  # Malignant melanoma of other part of trunk
        "C4360",  # Malignant melanoma of unspecified upper limb, including shoulder
        "C4361",  # Malignant melanoma of right upper limb, including shoulder
        "C4362",  # Malignant melanoma of left upper limb, including shoulder
        "C4370",  # Malignant melanoma of unspecified lower limb, including hip
        "C4371",  # Malignant melanoma of right lower limb, including hip
        "C4372",  # Malignant melanoma of left lower limb, including hip
        "C438",  # Malignant melanoma of overlapping sites of skin
        "C439",  # Malignant melanoma of skin, unspecified
        "C4400",  # Unspecified malignant neoplasm of skin of lip
        "C4401",  # Basal cell carcinoma of skin of lip
        "C4402",  # Squamous cell carcinoma of skin of lip
        "C4409",  # Other specified malignant neoplasm of skin of lip
        "C44101",  # Unspecified malignant neoplasm of skin of unspecified eyelid, including canthus
        "C441021",  # Unspecified malignant neoplasm of skin of right upper eyelid, including canthus
        "C441022",  # Unspecified malignant neoplasm of skin of right lower eyelid, including canthus
        "C441091",  # Unspecified malignant neoplasm of skin of left upper eyelid, including canthus
        "C441092",  # Unspecified malignant neoplasm of skin of left lower eyelid, including canthus
        "C44111",  # Basal cell carcinoma of skin of unspecified eyelid, including canthus
        "C441121",  # Basal cell carcinoma of skin of right upper eyelid, including canthus
        "C441122",  # Basal cell carcinoma of skin of right lower eyelid, including canthus
        "C441191",  # Basal cell carcinoma of skin of left upper eyelid, including canthus
        "C441192",  # Basal cell carcinoma of skin of left lower eyelid, including canthus
        "C44121",  # Squamous cell carcinoma of skin of unspecified eyelid, including canthus
        "C441221",  # Squamous cell carcinoma of skin of right upper eyelid, including canthus
        "C441222",  # Squamous cell carcinoma of skin of right lower eyelid, including canthus
        "C441291",  # Squamous cell carcinoma of skin of left upper eyelid, including canthus
        "C441292",  # Squamous cell carcinoma of skin of left lower eyelid, including canthus
        "C44131",  # Sebaceous cell carcinoma of skin of unspecified eyelid, including canthus
        "C441321",  # Sebaceous cell carcinoma of skin of right upper eyelid, including canthus
        "C441322",  # Sebaceous cell carcinoma of skin of right lower eyelid, including canthus
        "C441391",  # Sebaceous cell carcinoma of skin of left upper eyelid, including canthus
        "C441392",  # Sebaceous cell carcinoma of skin of left lower eyelid, including canthus
        "C44191",  # Other specified malignant neoplasm of skin of unspecified eyelid, including canthus
        "C441921",  # Other specified malignant neoplasm of skin of right upper eyelid, including canthus
        "C441922",  # Other specified malignant neoplasm of skin of right lower eyelid, including canthus
        "C441991",  # Other specified malignant neoplasm of skin of left upper eyelid, including canthus
        "C441992",  # Other specified malignant neoplasm of skin of left lower eyelid, including canthus
        "C44201",  # Unspecified malignant neoplasm of skin of unspecified ear and external auricular canal
        "C44202",  # Unspecified malignant neoplasm of skin of right ear and external auricular canal
        "C44209",  # Unspecified malignant neoplasm of skin of left ear and external auricular canal
        "C44211",  # Basal cell carcinoma of skin of unspecified ear and external auricular canal
        "C44212",  # Basal cell carcinoma of skin of right ear and external auricular canal
        "C44219",  # Basal cell carcinoma of skin of left ear and external auricular canal
        "C44221",  # Squamous cell carcinoma of skin of unspecified ear and external auricular canal
        "C44222",  # Squamous cell carcinoma of skin of right ear and external auricular canal
        "C44229",  # Squamous cell carcinoma of skin of left ear and external auricular canal
        "C44291",  # Other specified malignant neoplasm of skin of unspecified ear and external auricular canal
        "C44292",  # Other specified malignant neoplasm of skin of right ear and external auricular canal
        "C44299",  # Other specified malignant neoplasm of skin of left ear and external auricular canal
        "C44300",  # Unspecified malignant neoplasm of skin of unspecified part of face
        "C44301",  # Unspecified malignant neoplasm of skin of nose
        "C44309",  # Unspecified malignant neoplasm of skin of other parts of face
        "C44310",  # Basal cell carcinoma of skin of unspecified parts of face
        "C44311",  # Basal cell carcinoma of skin of nose
        "C44319",  # Basal cell carcinoma of skin of other parts of face
        "C44320",  # Squamous cell carcinoma of skin of unspecified parts of face
        "C44321",  # Squamous cell carcinoma of skin of nose
        "C44329",  # Squamous cell carcinoma of skin of other parts of face
        "C44390",  # Other specified malignant neoplasm of skin of unspecified parts of face
        "C44391",  # Other specified malignant neoplasm of skin of nose
        "C44399",  # Other specified malignant neoplasm of skin of other parts of face
        "C4440",  # Unspecified malignant neoplasm of skin of scalp and neck
        "C4441",  # Basal cell carcinoma of skin of scalp and neck
        "C4442",  # Squamous cell carcinoma of skin of scalp and neck
        "C4449",  # Other specified malignant neoplasm of skin of scalp and neck
        "C44500",  # Unspecified malignant neoplasm of anal skin
        "C44501",  # Unspecified malignant neoplasm of skin of breast
        "C44509",  # Unspecified malignant neoplasm of skin of other part of trunk
        "C44510",  # Basal cell carcinoma of anal skin
        "C44511",  # Basal cell carcinoma of skin of breast
        "C44519",  # Basal cell carcinoma of skin of other part of trunk
        "C44520",  # Squamous cell carcinoma of anal skin
        "C44521",  # Squamous cell carcinoma of skin of breast
        "C44529",  # Squamous cell carcinoma of skin of other part of trunk
        "C44590",  # Other specified malignant neoplasm of anal skin
        "C44591",  # Other specified malignant neoplasm of skin of breast
        "C44599",  # Other specified malignant neoplasm of skin of other part of trunk
        "C44601",  # Unspecified malignant neoplasm of skin of unspecified upper limb, including shoulder
        "C44602",  # Unspecified malignant neoplasm of skin of right upper limb, including shoulder
        "C44609",  # Unspecified malignant neoplasm of skin of left upper limb, including shoulder
        "C44611",  # Basal cell carcinoma of skin of unspecified upper limb, including shoulder
        "C44612",  # Basal cell carcinoma of skin of right upper limb, including shoulder
        "C44619",  # Basal cell carcinoma of skin of left upper limb, including shoulder
        "C44621",  # Squamous cell carcinoma of skin of unspecified upper limb, including shoulder
        "C44622",  # Squamous cell carcinoma of skin of right upper limb, including shoulder
        "C44629",  # Squamous cell carcinoma of skin of left upper limb, including shoulder
        "C44691",  # Other specified malignant neoplasm of skin of unspecified upper limb, including shoulder
        "C44692",  # Other specified malignant neoplasm of skin of right upper limb, including shoulder
        "C44699",  # Other specified malignant neoplasm of skin of left upper limb, including shoulder
        "C44701",  # Unspecified malignant neoplasm of skin of unspecified lower limb, including hip
        "C44702",  # Unspecified malignant neoplasm of skin of right lower limb, including hip
        "C44709",  # Unspecified malignant neoplasm of skin of left lower limb, including hip
        "C44711",  # Basal cell carcinoma of skin of unspecified lower limb, including hip
        "C44712",  # Basal cell carcinoma of skin of right lower limb, including hip
        "C44719",  # Basal cell carcinoma of skin of left lower limb, including hip
        "C44721",  # Squamous cell carcinoma of skin of unspecified lower limb, including hip
        "C44722",  # Squamous cell carcinoma of skin of right lower limb, including hip
        "C44729",  # Squamous cell carcinoma of skin of left lower limb, including hip
        "C44791",  # Other specified malignant neoplasm of skin of unspecified lower limb, including hip
        "C44792",  # Other specified malignant neoplasm of skin of right lower limb, including hip
        "C44799",  # Other specified malignant neoplasm of skin of left lower limb, including hip
        "C4480",  # Unspecified malignant neoplasm of overlapping sites of skin
        "C4481",  # Basal cell carcinoma of overlapping sites of skin
        "C4482",  # Squamous cell carcinoma of overlapping sites of skin
        "C4489",  # Other specified malignant neoplasm of overlapping sites of skin
        "C4490",  # Unspecified malignant neoplasm of skin, unspecified
        "C4491",  # Basal cell carcinoma of skin, unspecified
        "C4492",  # Squamous cell carcinoma of skin, unspecified
        "C4499",  # Other specified malignant neoplasm of skin, unspecified
        "C450",  # Mesothelioma of pleura
        "C451",  # Mesothelioma of peritoneum
        "C452",  # Mesothelioma of pericardium
        "C457",  # Mesothelioma of other sites
        "C459",  # Mesothelioma, unspecified
        "C460",  # Kaposi's sarcoma of skin
        "C461",  # Kaposi's sarcoma of soft tissue
        "C462",  # Kaposi's sarcoma of palate
        "C463",  # Kaposi's sarcoma of lymph nodes
        "C464",  # Kaposi's sarcoma of gastrointestinal sites
        "C4650",  # Kaposi's sarcoma of unspecified lung
        "C4651",  # Kaposi's sarcoma of right lung
        "C4652",  # Kaposi's sarcoma of left lung
        "C467",  # Kaposi's sarcoma of other sites
        "C469",  # Kaposi's sarcoma, unspecified
        "C470",  # Malignant neoplasm of peripheral nerves of head, face and neck
        "C4710",  # Malignant neoplasm of peripheral nerves of unspecified upper limb, including shoulder
        "C4711",  # Malignant neoplasm of peripheral nerves of right upper limb, including shoulder
        "C4712",  # Malignant neoplasm of peripheral nerves of left upper limb, including shoulder
        "C4720",  # Malignant neoplasm of peripheral nerves of unspecified lower limb, including hip
        "C4721",  # Malignant neoplasm of peripheral nerves of right lower limb, including hip
        "C4722",  # Malignant neoplasm of peripheral nerves of left lower limb, including hip
        "C473",  # Malignant neoplasm of peripheral nerves of thorax
        "C474",  # Malignant neoplasm of peripheral nerves of abdomen
        "C475",  # Malignant neoplasm of peripheral nerves of pelvis
        "C476",  # Malignant neoplasm of peripheral nerves of trunk, unspecified
        "C478",  # Malignant neoplasm of overlapping sites of peripheral nerves and autonomic nervous system
        "C479",  # Malignant neoplasm of peripheral nerves and autonomic nervous system, unspecified
        "C480",  # Malignant neoplasm of retroperitoneum
        "C481",  # Malignant neoplasm of specified parts of peritoneum
        "C482",  # Malignant neoplasm of peritoneum, unspecified
        "C488",  # Malignant neoplasm of overlapping sites of retroperitoneum and peritoneum
        "C490",  # Malignant neoplasm of connective and soft tissue of head, face and neck
        "C4910",  # Malignant neoplasm of connective and soft tissue of unspecified upper limb, including shoulder
        "C4911",  # Malignant neoplasm of connective and soft tissue of right upper limb, including shoulder
        "C4912",  # Malignant neoplasm of connective and soft tissue of left upper limb, including shoulder
        "C4920",  # Malignant neoplasm of connective and soft tissue of unspecified lower limb, including hip
        "C4921",  # Malignant neoplasm of connective and soft tissue of right lower limb, including hip
        "C4922",  # Malignant neoplasm of connective and soft tissue of left lower limb, including hip
        "C493",  # Malignant neoplasm of connective and soft tissue of thorax
        "C494",  # Malignant neoplasm of connective and soft tissue of abdomen
        "C495",  # Malignant neoplasm of connective and soft tissue of pelvis
        "C496",  # Malignant neoplasm of connective and soft tissue of trunk, unspecified
        "C498",  # Malignant neoplasm of overlapping sites of connective and soft tissue
        "C499",  # Malignant neoplasm of connective and soft tissue, unspecified
        "C49A0",  # Gastrointestinal stromal tumor, unspecified site
        "C49A1",  # Gastrointestinal stromal tumor of esophagus
        "C49A2",  # Gastrointestinal stromal tumor of stomach
        "C49A3",  # Gastrointestinal stromal tumor of small intestine
        "C49A4",  # Gastrointestinal stromal tumor of large intestine
        "C49A5",  # Gastrointestinal stromal tumor of rectum
        "C49A9",  # Gastrointestinal stromal tumor of other sites
        "C4A0",  # Merkel cell carcinoma of lip
        "C4A10",  # Merkel cell carcinoma of unspecified eyelid, including canthus
        "C4A111",  # Merkel cell carcinoma of right upper eyelid, including canthus
        "C4A112",  # Merkel cell carcinoma of right lower eyelid, including canthus
        "C4A121",  # Merkel cell carcinoma of left upper eyelid, including canthus
        "C4A122",  # Merkel cell carcinoma of left lower eyelid, including canthus
        "C4A20",  # Merkel cell carcinoma of unspecified ear and external auricular canal
        "C4A21",  # Merkel cell carcinoma of right ear and external auricular canal
        "C4A22",  # Merkel cell carcinoma of left ear and external auricular canal
        "C4A30",  # Merkel cell carcinoma of unspecified part of face
        "C4A31",  # Merkel cell carcinoma of nose
        "C4A39",  # Merkel cell carcinoma of other parts of face
        "C4A4",  # Merkel cell carcinoma of scalp and neck
        "C4A51",  # Merkel cell carcinoma of anal skin
        "C4A52",  # Merkel cell carcinoma of skin of breast
        "C4A59",  # Merkel cell carcinoma of other part of trunk
        "C4A60",  # Merkel cell carcinoma of unspecified upper limb, including shoulder
        "C4A61",  # Merkel cell carcinoma of right upper limb, including shoulder
        "C4A62",  # Merkel cell carcinoma of left upper limb, including shoulder
        "C4A70",  # Merkel cell carcinoma of unspecified lower limb, including hip
        "C4A71",  # Merkel cell carcinoma of right lower limb, including hip
        "C4A72",  # Merkel cell carcinoma of left lower limb, including hip
        "C4A8",  # Merkel cell carcinoma of overlapping sites
        "C4A9",  # Merkel cell carcinoma, unspecified
        "C50011",  # Malignant neoplasm of nipple and areola, right female breast
        "C50012",  # Malignant neoplasm of nipple and areola, left female breast
        "C50019",  # Malignant neoplasm of nipple and areola, unspecified female breast
        "C50021",  # Malignant neoplasm of nipple and areola, right male breast
        "C50022",  # Malignant neoplasm of nipple and areola, left male breast
        "C50029",  # Malignant neoplasm of nipple and areola, unspecified male breast
        "C50111",  # Malignant neoplasm of central portion of right female breast
        "C50112",  # Malignant neoplasm of central portion of left female breast
        "C50119",  # Malignant neoplasm of central portion of unspecified female breast
        "C50121",  # Malignant neoplasm of central portion of right male breast
        "C50122",  # Malignant neoplasm of central portion of left male breast
        "C50129",  # Malignant neoplasm of central portion of unspecified male breast
        "C50211",  # Malignant neoplasm of upper-inner quadrant of right female breast
        "C50212",  # Malignant neoplasm of upper-inner quadrant of left female breast
        "C50219",  # Malignant neoplasm of upper-inner quadrant of unspecified female breast
        "C50221",  # Malignant neoplasm of upper-inner quadrant of right male breast
        "C50222",  # Malignant neoplasm of upper-inner quadrant of left male breast
        "C50229",  # Malignant neoplasm of upper-inner quadrant of unspecified male breast
        "C50311",  # Malignant neoplasm of lower-inner quadrant of right female breast
        "C50312",  # Malignant neoplasm of lower-inner quadrant of left female breast
        "C50319",  # Malignant neoplasm of lower-inner quadrant of unspecified female breast
        "C50321",  # Malignant neoplasm of lower-inner quadrant of right male breast
        "C50322",  # Malignant neoplasm of lower-inner quadrant of left male breast
        "C50329",  # Malignant neoplasm of lower-inner quadrant of unspecified male breast
        "C50411",  # Malignant neoplasm of upper-outer quadrant of right female breast
        "C50412",  # Malignant neoplasm of upper-outer quadrant of left female breast
        "C50419",  # Malignant neoplasm of upper-outer quadrant of unspecified female breast
        "C50421",  # Malignant neoplasm of upper-outer quadrant of right male breast
        "C50422",  # Malignant neoplasm of upper-outer quadrant of left male breast
        "C50429",  # Malignant neoplasm of upper-outer quadrant of unspecified male breast
        "C50511",  # Malignant neoplasm of lower-outer quadrant of right female breast
        "C50512",  # Malignant neoplasm of lower-outer quadrant of left female breast
        "C50519",  # Malignant neoplasm of lower-outer quadrant of unspecified female breast
        "C50521",  # Malignant neoplasm of lower-outer quadrant of right male breast
        "C50522",  # Malignant neoplasm of lower-outer quadrant of left male breast
        "C50529",  # Malignant neoplasm of lower-outer quadrant of unspecified male breast
        "C50611",  # Malignant neoplasm of axillary tail of right female breast
        "C50612",  # Malignant neoplasm of axillary tail of left female breast
        "C50619",  # Malignant neoplasm of axillary tail of unspecified female breast
        "C50621",  # Malignant neoplasm of axillary tail of right male breast
        "C50622",  # Malignant neoplasm of axillary tail of left male breast
        "C50629",  # Malignant neoplasm of axillary tail of unspecified male breast
        "C50811",  # Malignant neoplasm of overlapping sites of right female breast
        "C50812",  # Malignant neoplasm of overlapping sites of left female breast
        "C50819",  # Malignant neoplasm of overlapping sites of unspecified female breast
        "C50821",  # Malignant neoplasm of overlapping sites of right male breast
        "C50822",  # Malignant neoplasm of overlapping sites of left male breast
        "C50829",  # Malignant neoplasm of overlapping sites of unspecified male breast
        "C50911",  # Malignant neoplasm of unspecified site of right female breast
        "C50912",  # Malignant neoplasm of unspecified site of left female breast
        "C50919",  # Malignant neoplasm of unspecified site of unspecified female breast
        "C50921",  # Malignant neoplasm of unspecified site of right male breast
        "C50922",  # Malignant neoplasm of unspecified site of left male breast
        "C50929",  # Malignant neoplasm of unspecified site of unspecified male breast
        "C510",  # Malignant neoplasm of labium majus
        "C511",  # Malignant neoplasm of labium minus
        "C512",  # Malignant neoplasm of clitoris
        "C518",  # Malignant neoplasm of overlapping sites of vulva
        "C519",  # Malignant neoplasm of vulva, unspecified
        "C52",  # Malignant neoplasm of vagina
        "C530",  # Malignant neoplasm of endocervix
        "C531",  # Malignant neoplasm of exocervix
        "C538",  # Malignant neoplasm of overlapping sites of cervix uteri
        "C539",  # Malignant neoplasm of cervix uteri, unspecified
        "C540",  # Malignant neoplasm of isthmus uteri
        "C541",  # Malignant neoplasm of endometrium
        "C542",  # Malignant neoplasm of myometrium
        "C543",  # Malignant neoplasm of fundus uteri
        "C548",  # Malignant neoplasm of overlapping sites of corpus uteri
        "C549",  # Malignant neoplasm of corpus uteri, unspecified
        "C55",  # Malignant neoplasm of uterus, part unspecified
        "C561",  # Malignant neoplasm of right ovary
        "C562",  # Malignant neoplasm of left ovary
        "C563",  # Malignant neoplasm of bilateral ovaries
        "C569",  # Malignant neoplasm of unspecified ovary
        "C5700",  # Malignant neoplasm of unspecified fallopian tube
        "C5701",  # Malignant neoplasm of right fallopian tube
        "C5702",  # Malignant neoplasm of left fallopian tube
        "C5710",  # Malignant neoplasm of unspecified broad ligament
        "C5711",  # Malignant neoplasm of right broad ligament
        "C5712",  # Malignant neoplasm of left broad ligament
        "C5720",  # Malignant neoplasm of unspecified round ligament
        "C5721",  # Malignant neoplasm of right round ligament
        "C5722",  # Malignant neoplasm of left round ligament
        "C573",  # Malignant neoplasm of parametrium
        "C574",  # Malignant neoplasm of uterine adnexa, unspecified
        "C577",  # Malignant neoplasm of other specified female genital organs
        "C578",  # Malignant neoplasm of overlapping sites of female genital organs
        "C579",  # Malignant neoplasm of female genital organ, unspecified
        "C58",  # Malignant neoplasm of placenta
        "C600",  # Malignant neoplasm of prepuce
        "C601",  # Malignant neoplasm of glans penis
        "C602",  # Malignant neoplasm of body of penis
        "C608",  # Malignant neoplasm of overlapping sites of penis
        "C609",  # Malignant neoplasm of penis, unspecified
        "C61",  # Malignant neoplasm of prostate
        "C6200",  # Malignant neoplasm of unspecified undescended testis
        "C6201",  # Malignant neoplasm of undescended right testis
        "C6202",  # Malignant neoplasm of undescended left testis
        "C6210",  # Malignant neoplasm of unspecified descended testis
        "C6211",  # Malignant neoplasm of descended right testis
        "C6212",  # Malignant neoplasm of descended left testis
        "C6290",  # Malignant neoplasm of unspecified testis, unspecified whether descended or undescended
        "C6291",  # Malignant neoplasm of right testis, unspecified whether descended or undescended
        "C6292",  # Malignant neoplasm of left testis, unspecified whether descended or undescended
        "C6300",  # Malignant neoplasm of unspecified epididymis
        "C6301",  # Malignant neoplasm of right epididymis
        "C6302",  # Malignant neoplasm of left epididymis
        "C6310",  # Malignant neoplasm of unspecified spermatic cord
        "C6311",  # Malignant neoplasm of right spermatic cord
        "C6312",  # Malignant neoplasm of left spermatic cord
        "C632",  # Malignant neoplasm of scrotum
        "C637",  # Malignant neoplasm of other specified male genital organs
        "C638",  # Malignant neoplasm of overlapping sites of male genital organs
        "C639",  # Malignant neoplasm of male genital organ, unspecified
        "C641",  # Malignant neoplasm of right kidney, except renal pelvis
        "C642",  # Malignant neoplasm of left kidney, except renal pelvis
        "C649",  # Malignant neoplasm of unspecified kidney, except renal pelvis
        "C651",  # Malignant neoplasm of right renal pelvis
        "C652",  # Malignant neoplasm of left renal pelvis
        "C659",  # Malignant neoplasm of unspecified renal pelvis
        "C661",  # Malignant neoplasm of right ureter
        "C662",  # Malignant neoplasm of left ureter
        "C669",  # Malignant neoplasm of unspecified ureter
        "C670",  # Malignant neoplasm of trigone of bladder
        "C671",  # Malignant neoplasm of dome of bladder
        "C672",  # Malignant neoplasm of lateral wall of bladder
        "C673",  # Malignant neoplasm of anterior wall of bladder
        "C674",  # Malignant neoplasm of posterior wall of bladder
        "C675",  # Malignant neoplasm of bladder neck
        "C676",  # Malignant neoplasm of ureteric orifice
        "C677",  # Malignant neoplasm of urachus
        "C678",  # Malignant neoplasm of overlapping sites of bladder
        "C679",  # Malignant neoplasm of bladder, unspecified
        "C680",  # Malignant neoplasm of urethra
        "C681",  # Malignant neoplasm of paraurethral glands
        "C688",  # Malignant neoplasm of overlapping sites of urinary organs
        "C689",  # Malignant neoplasm of urinary organ, unspecified
        "C6900",  # Malignant neoplasm of unspecified conjunctiva
        "C6901",  # Malignant neoplasm of right conjunctiva
        "C6902",  # Malignant neoplasm of left conjunctiva
        "C6910",  # Malignant neoplasm of unspecified cornea
        "C6911",  # Malignant neoplasm of right cornea
        "C6912",  # Malignant neoplasm of left cornea
        "C6920",  # Malignant neoplasm of unspecified retina
        "C6921",  # Malignant neoplasm of right retina
        "C6922",  # Malignant neoplasm of left retina
        "C6930",  # Malignant neoplasm of unspecified choroid
        "C6931",  # Malignant neoplasm of right choroid
        "C6932",  # Malignant neoplasm of left choroid
        "C6940",  # Malignant neoplasm of unspecified ciliary body
        "C6941",  # Malignant neoplasm of right ciliary body
        "C6942",  # Malignant neoplasm of left ciliary body
        "C6950",  # Malignant neoplasm of unspecified lacrimal gland and duct
        "C6951",  # Malignant neoplasm of right lacrimal gland and duct
        "C6952",  # Malignant neoplasm of left lacrimal gland and duct
        "C6960",  # Malignant neoplasm of unspecified orbit
        "C6961",  # Malignant neoplasm of right orbit
        "C6962",  # Malignant neoplasm of left orbit
        "C6980",  # Malignant neoplasm of overlapping sites of unspecified eye and adnexa
        "C6981",  # Malignant neoplasm of overlapping sites of right eye and adnexa
        "C6982",  # Malignant neoplasm of overlapping sites of left eye and adnexa
        "C6990",  # Malignant neoplasm of unspecified site of unspecified eye
        "C6991",  # Malignant neoplasm of unspecified site of right eye
        "C6992",  # Malignant neoplasm of unspecified site of left eye
        "C700",  # Malignant neoplasm of cerebral meninges
        "C701",  # Malignant neoplasm of spinal meninges
        "C709",  # Malignant neoplasm of meninges, unspecified
        "C710",  # Malignant neoplasm of cerebrum, except lobes and ventricles
        "C711",  # Malignant neoplasm of frontal lobe
        "C712",  # Malignant neoplasm of temporal lobe
        "C713",  # Malignant neoplasm of parietal lobe
        "C714",  # Malignant neoplasm of occipital lobe
        "C715",  # Malignant neoplasm of cerebral ventricle
        "C716",  # Malignant neoplasm of cerebellum
        "C717",  # Malignant neoplasm of brain stem
        "C718",  # Malignant neoplasm of overlapping sites of brain
        "C719",  # Malignant neoplasm of brain, unspecified
        "C720",  # Malignant neoplasm of spinal cord
        "C721",  # Malignant neoplasm of cauda equina
        "C7220",  # Malignant neoplasm of unspecified olfactory nerve
        "C7221",  # Malignant neoplasm of right olfactory nerve
        "C7222",  # Malignant neoplasm of left olfactory nerve
        "C7230",  # Malignant neoplasm of unspecified optic nerve
        "C7231",  # Malignant neoplasm of right optic nerve
        "C7232",  # Malignant neoplasm of left optic nerve
        "C7240",  # Malignant neoplasm of unspecified acoustic nerve
        "C7241",  # Malignant neoplasm of right acoustic nerve
        "C7242",  # Malignant neoplasm of left acoustic nerve
        "C7250",  # Malignant neoplasm of unspecified cranial nerve
        "C7259",  # Malignant neoplasm of other cranial nerves
        "C729",  # Malignant neoplasm of central nervous system, unspecified
        "C73",  # Malignant neoplasm of thyroid gland
        "C7400",  # Malignant neoplasm of cortex of unspecified adrenal gland
        "C7401",  # Malignant neoplasm of cortex of right adrenal gland
        "C7402",  # Malignant neoplasm of cortex of left adrenal gland
        "C7410",  # Malignant neoplasm of medulla of unspecified adrenal gland
        "C7411",  # Malignant neoplasm of medulla of right adrenal gland
        "C7412",  # Malignant neoplasm of medulla of left adrenal gland
        "C7490",  # Malignant neoplasm of unspecified part of unspecified adrenal gland
        "C7491",  # Malignant neoplasm of unspecified part of right adrenal gland
        "C7492",  # Malignant neoplasm of unspecified part of left adrenal gland
        "C750",  # Malignant neoplasm of parathyroid gland
        "C751",  # Malignant neoplasm of pituitary gland
        "C752",  # Malignant neoplasm of craniopharyngeal duct
        "C753",  # Malignant neoplasm of pineal gland
        "C754",  # Malignant neoplasm of carotid body
        "C755",  # Malignant neoplasm of aortic body and other paraganglia
        "C758",  # Malignant neoplasm with pluriglandular involvement, unspecified
        "C759",  # Malignant neoplasm of endocrine gland, unspecified
        "C760",  # Malignant neoplasm of head, face and neck
        "C761",  # Malignant neoplasm of thorax
        "C762",  # Malignant neoplasm of abdomen
        "C763",  # Malignant neoplasm of pelvis
        "C7640",  # Malignant neoplasm of unspecified upper limb
        "C7641",  # Malignant neoplasm of right upper limb
        "C7642",  # Malignant neoplasm of left upper limb
        "C7650",  # Malignant neoplasm of unspecified lower limb
        "C7651",  # Malignant neoplasm of right lower limb
        "C7652",  # Malignant neoplasm of left lower limb
        "C768",  # Malignant neoplasm of other specified ill-defined sites
        "C770",  # Secondary and unspecified malignant neoplasm of lymph nodes of head, face and neck
        "C771",  # Secondary and unspecified malignant neoplasm of intrathoracic lymph nodes
        "C772",  # Secondary and unspecified malignant neoplasm of intra-abdominal lymph nodes
        "C773",  # Secondary and unspecified malignant neoplasm of axilla and upper limb lymph nodes
        "C774",  # Secondary and unspecified malignant neoplasm of inguinal and lower limb lymph nodes
        "C775",  # Secondary and unspecified malignant neoplasm of intrapelvic lymph nodes
        "C778",  # Secondary and unspecified malignant neoplasm of lymph nodes of multiple regions
        "C779",  # Secondary and unspecified malignant neoplasm of lymph node, unspecified
        "C7800",  # Secondary malignant neoplasm of unspecified lung
        "C7801",  # Secondary malignant neoplasm of right lung
        "C7802",  # Secondary malignant neoplasm of left lung
        "C781",  # Secondary malignant neoplasm of mediastinum
        "C782",  # Secondary malignant neoplasm of pleura
        "C7830",  # Secondary malignant neoplasm of unspecified respiratory organ
        "C7839",  # Secondary malignant neoplasm of other respiratory organs
        "C784",  # Secondary malignant neoplasm of small intestine
        "C785",  # Secondary malignant neoplasm of large intestine and rectum
        "C786",  # Secondary malignant neoplasm of retroperitoneum and peritoneum
        "C787",  # Secondary malignant neoplasm of liver and intrahepatic bile duct
        "C7880",  # Secondary malignant neoplasm of unspecified digestive organ
        "C7889",  # Secondary malignant neoplasm of other digestive organs
        "C7900",  # Secondary malignant neoplasm of unspecified kidney and renal pelvis
        "C7901",  # Secondary malignant neoplasm of right kidney and renal pelvis
        "C7902",  # Secondary malignant neoplasm of left kidney and renal pelvis
        "C7910",  # Secondary malignant neoplasm of unspecified urinary organs
        "C7911",  # Secondary malignant neoplasm of bladder
        "C7919",  # Secondary malignant neoplasm of other urinary organs
        "C792",  # Secondary malignant neoplasm of skin
        "C7931",  # Secondary malignant neoplasm of brain
        "C7932",  # Secondary malignant neoplasm of cerebral meninges
        "C7940",  # Secondary malignant neoplasm of unspecified part of nervous system
        "C7949",  # Secondary malignant neoplasm of other parts of nervous system
        "C7951",  # Secondary malignant neoplasm of bone
        "C7952",  # Secondary malignant neoplasm of bone marrow
        "C7960",  # Secondary malignant neoplasm of unspecified ovary
        "C7961",  # Secondary malignant neoplasm of right ovary
        "C7962",  # Secondary malignant neoplasm of left ovary
        "C7963",  # Secondary malignant neoplasm of bilateral ovaries
        "C7970",  # Secondary malignant neoplasm of unspecified adrenal gland
        "C7971",  # Secondary malignant neoplasm of right adrenal gland
        "C7972",  # Secondary malignant neoplasm of left adrenal gland
        "C7981",  # Secondary malignant neoplasm of breast
        "C7982",  # Secondary malignant neoplasm of genital organs
        "C7989",  # Secondary malignant neoplasm of other specified sites
        "C799",  # Secondary malignant neoplasm of unspecified site
        "C7A00",  # Malignant carcinoid tumor of unspecified site
        "C7A010",  # Malignant carcinoid tumor of the duodenum
        "C7A011",  # Malignant carcinoid tumor of the jejunum
        "C7A012",  # Malignant carcinoid tumor of the ileum
        "C7A019",  # Malignant carcinoid tumor of the small intestine, unspecified portion
        "C7A020",  # Malignant carcinoid tumor of the appendix
        "C7A021",  # Malignant carcinoid tumor of the cecum
        "C7A022",  # Malignant carcinoid tumor of the ascending colon
        "C7A023",  # Malignant carcinoid tumor of the transverse colon
        "C7A024",  # Malignant carcinoid tumor of the descending colon
        "C7A025",  # Malignant carcinoid tumor of the sigmoid colon
        "C7A026",  # Malignant carcinoid tumor of the rectum
        "C7A029",  # Malignant carcinoid tumor of the large intestine, unspecified portion
        "C7A090",  # Malignant carcinoid tumor of the bronchus and lung
        "C7A091",  # Malignant carcinoid tumor of the thymus
        "C7A092",  # Malignant carcinoid tumor of the stomach
        "C7A093",  # Malignant carcinoid tumor of the kidney
        "C7A094",  # Malignant carcinoid tumor of the foregut, unspecified
        "C7A095",  # Malignant carcinoid tumor of the midgut, unspecified
        "C7A096",  # Malignant carcinoid tumor of the hindgut, unspecified
        "C7A098",  # Malignant carcinoid tumors of other sites
        "C7A1",  # Malignant poorly differentiated neuroendocrine tumors
        "C7A8",  # Other malignant neuroendocrine tumors
        "C7B00",  # Secondary carcinoid tumors, unspecified site
        "C7B01",  # Secondary carcinoid tumors of distant lymph nodes
        "C7B02",  # Secondary carcinoid tumors of liver
        "C7B03",  # Secondary carcinoid tumors of bone
        "C7B04",  # Secondary carcinoid tumors of peritoneum
        "C7B09",  # Secondary carcinoid tumors of other sites
        "C7B1",  # Secondary Merkel cell carcinoma
        "C7B8",  # Other secondary neuroendocrine tumors
        "C800",  # Disseminated malignant neoplasm, unspecified
        "C801",  # Malignant (primary) neoplasm, unspecified
        "C802",  # Malignant neoplasm associated with transplanted organ
        "C8100",  # Nodular lymphocyte predominant Hodgkin lymphoma, unspecified site
        "C8101",  # Nodular lymphocyte predominant Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8102",  # Nodular lymphocyte predominant Hodgkin lymphoma, intrathoracic lymph nodes
        "C8103",  # Nodular lymphocyte predominant Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8104",  # Nodular lymphocyte predominant Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8105",  # Nodular lymphocyte predominant Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8106",  # Nodular lymphocyte predominant Hodgkin lymphoma, intrapelvic lymph nodes
        "C8107",  # Nodular lymphocyte predominant Hodgkin lymphoma, spleen
        "C8108",  # Nodular lymphocyte predominant Hodgkin lymphoma, lymph nodes of multiple sites
        "C8109",  # Nodular lymphocyte predominant Hodgkin lymphoma, extranodal and solid organ sites
        "C8110",  # Nodular sclerosis Hodgkin lymphoma, unspecified site
        "C8111",  # Nodular sclerosis Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8112",  # Nodular sclerosis Hodgkin lymphoma, intrathoracic lymph nodes
        "C8113",  # Nodular sclerosis Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8114",  # Nodular sclerosis Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8115",  # Nodular sclerosis Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8116",  # Nodular sclerosis Hodgkin lymphoma, intrapelvic lymph nodes
        "C8117",  # Nodular sclerosis Hodgkin lymphoma, spleen
        "C8118",  # Nodular sclerosis Hodgkin lymphoma, lymph nodes of multiple sites
        "C8119",  # Nodular sclerosis Hodgkin lymphoma, extranodal and solid organ sites
        "C8120",  # Mixed cellularity Hodgkin lymphoma, unspecified site
        "C8121",  # Mixed cellularity Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8122",  # Mixed cellularity Hodgkin lymphoma, intrathoracic lymph nodes
        "C8123",  # Mixed cellularity Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8124",  # Mixed cellularity Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8125",  # Mixed cellularity Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8126",  # Mixed cellularity Hodgkin lymphoma, intrapelvic lymph nodes
        "C8127",  # Mixed cellularity Hodgkin lymphoma, spleen
        "C8128",  # Mixed cellularity Hodgkin lymphoma, lymph nodes of multiple sites
        "C8129",  # Mixed cellularity Hodgkin lymphoma, extranodal and solid organ sites
        "C8130",  # Lymphocyte depleted Hodgkin lymphoma, unspecified site
        "C8131",  # Lymphocyte depleted Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8132",  # Lymphocyte depleted Hodgkin lymphoma, intrathoracic lymph nodes
        "C8133",  # Lymphocyte depleted Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8134",  # Lymphocyte depleted Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8135",  # Lymphocyte depleted Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8136",  # Lymphocyte depleted Hodgkin lymphoma, intrapelvic lymph nodes
        "C8137",  # Lymphocyte depleted Hodgkin lymphoma, spleen
        "C8138",  # Lymphocyte depleted Hodgkin lymphoma, lymph nodes of multiple sites
        "C8139",  # Lymphocyte depleted Hodgkin lymphoma, extranodal and solid organ sites
        "C8140",  # Lymphocyte-rich Hodgkin lymphoma, unspecified site
        "C8141",  # Lymphocyte-rich Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8142",  # Lymphocyte-rich Hodgkin lymphoma, intrathoracic lymph nodes
        "C8143",  # Lymphocyte-rich Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8144",  # Lymphocyte-rich Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8145",  # Lymphocyte-rich Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8146",  # Lymphocyte-rich Hodgkin lymphoma, intrapelvic lymph nodes
        "C8147",  # Lymphocyte-rich Hodgkin lymphoma, spleen
        "C8148",  # Lymphocyte-rich Hodgkin lymphoma, lymph nodes of multiple sites
        "C8149",  # Lymphocyte-rich Hodgkin lymphoma, extranodal and solid organ sites
        "C8170",  # Other Hodgkin lymphoma, unspecified site
        "C8171",  # Other Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8172",  # Other Hodgkin lymphoma, intrathoracic lymph nodes
        "C8173",  # Other Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8174",  # Other Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8175",  # Other Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8176",  # Other Hodgkin lymphoma, intrapelvic lymph nodes
        "C8177",  # Other Hodgkin lymphoma, spleen
        "C8178",  # Other Hodgkin lymphoma, lymph nodes of multiple sites
        "C8179",  # Other Hodgkin lymphoma, extranodal and solid organ sites
        "C8190",  # Hodgkin lymphoma, unspecified, unspecified site
        "C8191",  # Hodgkin lymphoma, unspecified, lymph nodes of head, face, and neck
        "C8192",  # Hodgkin lymphoma, unspecified, intrathoracic lymph nodes
        "C8193",  # Hodgkin lymphoma, unspecified, intra-abdominal lymph nodes
        "C8194",  # Hodgkin lymphoma, unspecified, lymph nodes of axilla and upper limb
        "C8195",  # Hodgkin lymphoma, unspecified, lymph nodes of inguinal region and lower limb
        "C8196",  # Hodgkin lymphoma, unspecified, intrapelvic lymph nodes
        "C8197",  # Hodgkin lymphoma, unspecified, spleen
        "C8198",  # Hodgkin lymphoma, unspecified, lymph nodes of multiple sites
        "C8199",  # Hodgkin lymphoma, unspecified, extranodal and solid organ sites
        "C8200",  # Follicular lymphoma grade I, unspecified site
        "C8201",  # Follicular lymphoma grade I, lymph nodes of head, face, and neck
        "C8202",  # Follicular lymphoma grade I, intrathoracic lymph nodes
        "C8203",  # Follicular lymphoma grade I, intra-abdominal lymph nodes
        "C8204",  # Follicular lymphoma grade I, lymph nodes of axilla and upper limb
        "C8205",  # Follicular lymphoma grade I, lymph nodes of inguinal region and lower limb
        "C8206",  # Follicular lymphoma grade I, intrapelvic lymph nodes
        "C8207",  # Follicular lymphoma grade I, spleen
        "C8208",  # Follicular lymphoma grade I, lymph nodes of multiple sites
        "C8209",  # Follicular lymphoma grade I, extranodal and solid organ sites
        "C8210",  # Follicular lymphoma grade II, unspecified site
        "C8211",  # Follicular lymphoma grade II, lymph nodes of head, face, and neck
        "C8212",  # Follicular lymphoma grade II, intrathoracic lymph nodes
        "C8213",  # Follicular lymphoma grade II, intra-abdominal lymph nodes
        "C8214",  # Follicular lymphoma grade II, lymph nodes of axilla and upper limb
        "C8215",  # Follicular lymphoma grade II, lymph nodes of inguinal region and lower limb
        "C8216",  # Follicular lymphoma grade II, intrapelvic lymph nodes
        "C8217",  # Follicular lymphoma grade II, spleen
        "C8218",  # Follicular lymphoma grade II, lymph nodes of multiple sites
        "C8219",  # Follicular lymphoma grade II, extranodal and solid organ sites
        "C8220",  # Follicular lymphoma grade III, unspecified, unspecified site
        "C8221",  # Follicular lymphoma grade III, unspecified, lymph nodes of head, face, and neck
        "C8222",  # Follicular lymphoma grade III, unspecified, intrathoracic lymph nodes
        "C8223",  # Follicular lymphoma grade III, unspecified, intra-abdominal lymph nodes
        "C8224",  # Follicular lymphoma grade III, unspecified, lymph nodes of axilla and upper limb
        "C8225",  # Follicular lymphoma grade III, unspecified, lymph nodes of inguinal region and lower limb
        "C8226",  # Follicular lymphoma grade III, unspecified, intrapelvic lymph nodes
        "C8227",  # Follicular lymphoma grade III, unspecified, spleen
        "C8228",  # Follicular lymphoma grade III, unspecified, lymph nodes of multiple sites
        "C8229",  # Follicular lymphoma grade III, unspecified, extranodal and solid organ sites
        "C8230",  # Follicular lymphoma grade IIIa, unspecified site
        "C8231",  # Follicular lymphoma grade IIIa, lymph nodes of head, face, and neck
        "C8232",  # Follicular lymphoma grade IIIa, intrathoracic lymph nodes
        "C8233",  # Follicular lymphoma grade IIIa, intra-abdominal lymph nodes
        "C8234",  # Follicular lymphoma grade IIIa, lymph nodes of axilla and upper limb
        "C8235",  # Follicular lymphoma grade IIIa, lymph nodes of inguinal region and lower limb
        "C8236",  # Follicular lymphoma grade IIIa, intrapelvic lymph nodes
        "C8237",  # Follicular lymphoma grade IIIa, spleen
        "C8238",  # Follicular lymphoma grade IIIa, lymph nodes of multiple sites
        "C8239",  # Follicular lymphoma grade IIIa, extranodal and solid organ sites
        "C8240",  # Follicular lymphoma grade IIIb, unspecified site
        "C8241",  # Follicular lymphoma grade IIIb, lymph nodes of head, face, and neck
        "C8242",  # Follicular lymphoma grade IIIb, intrathoracic lymph nodes
        "C8243",  # Follicular lymphoma grade IIIb, intra-abdominal lymph nodes
        "C8244",  # Follicular lymphoma grade IIIb, lymph nodes of axilla and upper limb
        "C8245",  # Follicular lymphoma grade IIIb, lymph nodes of inguinal region and lower limb
        "C8246",  # Follicular lymphoma grade IIIb, intrapelvic lymph nodes
        "C8247",  # Follicular lymphoma grade IIIb, spleen
        "C8248",  # Follicular lymphoma grade IIIb, lymph nodes of multiple sites
        "C8249",  # Follicular lymphoma grade IIIb, extranodal and solid organ sites
        "C8250",  # Diffuse follicle center lymphoma, unspecified site
        "C8251",  # Diffuse follicle center lymphoma, lymph nodes of head, face, and neck
        "C8252",  # Diffuse follicle center lymphoma, intrathoracic lymph nodes
        "C8253",  # Diffuse follicle center lymphoma, intra-abdominal lymph nodes
        "C8254",  # Diffuse follicle center lymphoma, lymph nodes of axilla and upper limb
        "C8255",  # Diffuse follicle center lymphoma, lymph nodes of inguinal region and lower limb
        "C8256",  # Diffuse follicle center lymphoma, intrapelvic lymph nodes
        "C8257",  # Diffuse follicle center lymphoma, spleen
        "C8258",  # Diffuse follicle center lymphoma, lymph nodes of multiple sites
        "C8259",  # Diffuse follicle center lymphoma, extranodal and solid organ sites
        "C8260",  # Cutaneous follicle center lymphoma, unspecified site
        "C8261",  # Cutaneous follicle center lymphoma, lymph nodes of head, face, and neck
        "C8262",  # Cutaneous follicle center lymphoma, intrathoracic lymph nodes
        "C8263",  # Cutaneous follicle center lymphoma, intra-abdominal lymph nodes
        "C8264",  # Cutaneous follicle center lymphoma, lymph nodes of axilla and upper limb
        "C8265",  # Cutaneous follicle center lymphoma, lymph nodes of inguinal region and lower limb
        "C8266",  # Cutaneous follicle center lymphoma, intrapelvic lymph nodes
        "C8267",  # Cutaneous follicle center lymphoma, spleen
        "C8268",  # Cutaneous follicle center lymphoma, lymph nodes of multiple sites
        "C8269",  # Cutaneous follicle center lymphoma, extranodal and solid organ sites
        "C8280",  # Other types of follicular lymphoma, unspecified site
        "C8281",  # Other types of follicular lymphoma, lymph nodes of head, face, and neck
        "C8282",  # Other types of follicular lymphoma, intrathoracic lymph nodes
        "C8283",  # Other types of follicular lymphoma, intra-abdominal lymph nodes
        "C8284",  # Other types of follicular lymphoma, lymph nodes of axilla and upper limb
        "C8285",  # Other types of follicular lymphoma, lymph nodes of inguinal region and lower limb
        "C8286",  # Other types of follicular lymphoma, intrapelvic lymph nodes
        "C8287",  # Other types of follicular lymphoma, spleen
        "C8288",  # Other types of follicular lymphoma, lymph nodes of multiple sites
        "C8289",  # Other types of follicular lymphoma, extranodal and solid organ sites
        "C8290",  # Follicular lymphoma, unspecified, unspecified site
        "C8291",  # Follicular lymphoma, unspecified, lymph nodes of head, face, and neck
        "C8292",  # Follicular lymphoma, unspecified, intrathoracic lymph nodes
        "C8293",  # Follicular lymphoma, unspecified, intra-abdominal lymph nodes
        "C8294",  # Follicular lymphoma, unspecified, lymph nodes of axilla and upper limb
        "C8295",  # Follicular lymphoma, unspecified, lymph nodes of inguinal region and lower limb
        "C8296",  # Follicular lymphoma, unspecified, intrapelvic lymph nodes
        "C8297",  # Follicular lymphoma, unspecified, spleen
        "C8298",  # Follicular lymphoma, unspecified, lymph nodes of multiple sites
        "C8299",  # Follicular lymphoma, unspecified, extranodal and solid organ sites
        "C8300",  # Small cell B-cell lymphoma, unspecified site
        "C8301",  # Small cell B-cell lymphoma, lymph nodes of head, face, and neck
        "C8302",  # Small cell B-cell lymphoma, intrathoracic lymph nodes
        "C8303",  # Small cell B-cell lymphoma, intra-abdominal lymph nodes
        "C8304",  # Small cell B-cell lymphoma, lymph nodes of axilla and upper limb
        "C8305",  # Small cell B-cell lymphoma, lymph nodes of inguinal region and lower limb
        "C8306",  # Small cell B-cell lymphoma, intrapelvic lymph nodes
        "C8307",  # Small cell B-cell lymphoma, spleen
        "C8308",  # Small cell B-cell lymphoma, lymph nodes of multiple sites
        "C8309",  # Small cell B-cell lymphoma, extranodal and solid organ sites
        "C8310",  # Mantle cell lymphoma, unspecified site
        "C8311",  # Mantle cell lymphoma, lymph nodes of head, face, and neck
        "C8312",  # Mantle cell lymphoma, intrathoracic lymph nodes
        "C8313",  # Mantle cell lymphoma, intra-abdominal lymph nodes
        "C8314",  # Mantle cell lymphoma, lymph nodes of axilla and upper limb
        "C8315",  # Mantle cell lymphoma, lymph nodes of inguinal region and lower limb
        "C8316",  # Mantle cell lymphoma, intrapelvic lymph nodes
        "C8317",  # Mantle cell lymphoma, spleen
        "C8318",  # Mantle cell lymphoma, lymph nodes of multiple sites
        "C8319",  # Mantle cell lymphoma, extranodal and solid organ sites
        "C8330",  # Diffuse large B-cell lymphoma, unspecified site
        "C8331",  # Diffuse large B-cell lymphoma, lymph nodes of head, face, and neck
        "C8332",  # Diffuse large B-cell lymphoma, intrathoracic lymph nodes
        "C8333",  # Diffuse large B-cell lymphoma, intra-abdominal lymph nodes
        "C8334",  # Diffuse large B-cell lymphoma, lymph nodes of axilla and upper limb
        "C8335",  # Diffuse large B-cell lymphoma, lymph nodes of inguinal region and lower limb
        "C8336",  # Diffuse large B-cell lymphoma, intrapelvic lymph nodes
        "C8337",  # Diffuse large B-cell lymphoma, spleen
        "C8338",  # Diffuse large B-cell lymphoma, lymph nodes of multiple sites
        "C8339",  # Diffuse large B-cell lymphoma, extranodal and solid organ sites
        "C83390",  # Primary central nervous system lymphoma
        "C83398",  # Diffuse large B-cell lymphoma of other extranodal and solid organ sites
        "C8350",  # Lymphoblastic (diffuse) lymphoma, unspecified site
        "C8351",  # Lymphoblastic (diffuse) lymphoma, lymph nodes of head, face, and neck
        "C8352",  # Lymphoblastic (diffuse) lymphoma, intrathoracic lymph nodes
        "C8353",  # Lymphoblastic (diffuse) lymphoma, intra-abdominal lymph nodes
        "C8354",  # Lymphoblastic (diffuse) lymphoma, lymph nodes of axilla and upper limb
        "C8355",  # Lymphoblastic (diffuse) lymphoma, lymph nodes of inguinal region and lower limb
        "C8356",  # Lymphoblastic (diffuse) lymphoma, intrapelvic lymph nodes
        "C8357",  # Lymphoblastic (diffuse) lymphoma, spleen
        "C8358",  # Lymphoblastic (diffuse) lymphoma, lymph nodes of multiple sites
        "C8359",  # Lymphoblastic (diffuse) lymphoma, extranodal and solid organ sites
        "C8370",  # Burkitt lymphoma, unspecified site
        "C8371",  # Burkitt lymphoma, lymph nodes of head, face, and neck
        "C8372",  # Burkitt lymphoma, intrathoracic lymph nodes
        "C8373",  # Burkitt lymphoma, intra-abdominal lymph nodes
        "C8374",  # Burkitt lymphoma, lymph nodes of axilla and upper limb
        "C8375",  # Burkitt lymphoma, lymph nodes of inguinal region and lower limb
        "C8376",  # Burkitt lymphoma, intrapelvic lymph nodes
        "C8377",  # Burkitt lymphoma, spleen
        "C8378",  # Burkitt lymphoma, lymph nodes of multiple sites
        "C8379",  # Burkitt lymphoma, extranodal and solid organ sites
        "C8380",  # Other non-follicular lymphoma, unspecified site
        "C8381",  # Other non-follicular lymphoma, lymph nodes of head, face, and neck
        "C8382",  # Other non-follicular lymphoma, intrathoracic lymph nodes
        "C8383",  # Other non-follicular lymphoma, intra-abdominal lymph nodes
        "C8384",  # Other non-follicular lymphoma, lymph nodes of axilla and upper limb
        "C8385",  # Other non-follicular lymphoma, lymph nodes of inguinal region and lower limb
        "C8386",  # Other non-follicular lymphoma, intrapelvic lymph nodes
        "C8387",  # Other non-follicular lymphoma, spleen
        "C8388",  # Other non-follicular lymphoma, lymph nodes of multiple sites
        "C8389",  # Other non-follicular lymphoma, extranodal and solid organ sites
        "C8390",  # Non-follicular (diffuse) lymphoma, unspecified, unspecified site
        "C8391",  # Non-follicular (diffuse) lymphoma, unspecified, lymph nodes of head, face, and neck
        "C8392",  # Non-follicular (diffuse) lymphoma, unspecified, intrathoracic lymph nodes
        "C8393",  # Non-follicular (diffuse) lymphoma, unspecified, intra-abdominal lymph nodes
        "C8394",  # Non-follicular (diffuse) lymphoma, unspecified, lymph nodes of axilla and upper limb
        "C8395",  # Non-follicular (diffuse) lymphoma, unspecified, lymph nodes of inguinal region and lower limb
        "C8396",  # Non-follicular (diffuse) lymphoma, unspecified, intrapelvic lymph nodes
        "C8397",  # Non-follicular (diffuse) lymphoma, unspecified, spleen
        "C8398",  # Non-follicular (diffuse) lymphoma, unspecified, lymph nodes of multiple sites
        "C8399",  # Non-follicular (diffuse) lymphoma, unspecified, extranodal and solid organ sites
        "C8400",  # Mycosis fungoides, unspecified site
        "C8401",  # Mycosis fungoides, lymph nodes of head, face, and neck
        "C8402",  # Mycosis fungoides, intrathoracic lymph nodes
        "C8403",  # Mycosis fungoides, intra-abdominal lymph nodes
        "C8404",  # Mycosis fungoides, lymph nodes of axilla and upper limb
        "C8405",  # Mycosis fungoides, lymph nodes of inguinal region and lower limb
        "C8406",  # Mycosis fungoides, intrapelvic lymph nodes
        "C8407",  # Mycosis fungoides, spleen
        "C8408",  # Mycosis fungoides, lymph nodes of multiple sites
        "C8409",  # Mycosis fungoides, extranodal and solid organ sites
        "C8410",  # Sezary disease, unspecified site
        "C8411",  # Sezary disease, lymph nodes of head, face, and neck
        "C8412",  # Sezary disease, intrathoracic lymph nodes
        "C8413",  # Sezary disease, intra-abdominal lymph nodes
        "C8414",  # Sezary disease, lymph nodes of axilla and upper limb
        "C8415",  # Sezary disease, lymph nodes of inguinal region and lower limb
        "C8416",  # Sezary disease, intrapelvic lymph nodes
        "C8417",  # Sezary disease, spleen
        "C8418",  # Sezary disease, lymph nodes of multiple sites
        "C8419",  # Sezary disease, extranodal and solid organ sites
        "C8440",  # Peripheral T-cell lymphoma, not elsewhere classified, unspecified site
        "C8441",  # Peripheral T-cell lymphoma, not elsewhere classified, lymph nodes of head, face, and neck
        "C8442",  # Peripheral T-cell lymphoma, not elsewhere classified, intrathoracic lymph nodes
        "C8443",  # Peripheral T-cell lymphoma, not elsewhere classified, intra-abdominal lymph nodes
        "C8444",  # Peripheral T-cell lymphoma, not elsewhere classified, lymph nodes of axilla and upper limb
        "C8445",  # Peripheral T-cell lymphoma, not elsewhere classified, lymph nodes of inguinal region and lower limb
        "C8446",  # Peripheral T-cell lymphoma, not elsewhere classified, intrapelvic lymph nodes
        "C8447",  # Peripheral T-cell lymphoma, not elsewhere classified, spleen
        "C8448",  # Peripheral T-cell lymphoma, not elsewhere classified, lymph nodes of multiple sites
        "C8449",  # Peripheral T-cell lymphoma, not elsewhere classified, extranodal and solid organ sites
        "C8460",  # Anaplastic large cell lymphoma, ALK-positive, unspecified site
        "C8461",  # Anaplastic large cell lymphoma, ALK-positive, lymph nodes of head, face, and neck
        "C8462",  # Anaplastic large cell lymphoma, ALK-positive, intrathoracic lymph nodes
        "C8463",  # Anaplastic large cell lymphoma, ALK-positive, intra-abdominal lymph nodes
        "C8464",  # Anaplastic large cell lymphoma, ALK-positive, lymph nodes of axilla and upper limb
        "C8465",  # Anaplastic large cell lymphoma, ALK-positive, lymph nodes of inguinal region and lower limb
        "C8466",  # Anaplastic large cell lymphoma, ALK-positive, intrapelvic lymph nodes
        "C8467",  # Anaplastic large cell lymphoma, ALK-positive, spleen
        "C8468",  # Anaplastic large cell lymphoma, ALK-positive, lymph nodes of multiple sites
        "C8469",  # Anaplastic large cell lymphoma, ALK-positive, extranodal and solid organ sites
        "C8470",  # Anaplastic large cell lymphoma, ALK-negative, unspecified site
        "C8471",  # Anaplastic large cell lymphoma, ALK-negative, lymph nodes of head, face, and neck
        "C8472",  # Anaplastic large cell lymphoma, ALK-negative, intrathoracic lymph nodes
        "C8473",  # Anaplastic large cell lymphoma, ALK-negative, intra-abdominal lymph nodes
        "C8474",  # Anaplastic large cell lymphoma, ALK-negative, lymph nodes of axilla and upper limb
        "C8475",  # Anaplastic large cell lymphoma, ALK-negative, lymph nodes of inguinal region and lower limb
        "C8476",  # Anaplastic large cell lymphoma, ALK-negative, intrapelvic lymph nodes
        "C8477",  # Anaplastic large cell lymphoma, ALK-negative, spleen
        "C8478",  # Anaplastic large cell lymphoma, ALK-negative, lymph nodes of multiple sites
        "C8479",  # Anaplastic large cell lymphoma, ALK-negative, extranodal and solid organ sites
        "C847A",  # Anaplastic large cell lymphoma, ALK-negative, breast
        "C8490",  # Mature T/NK-cell lymphomas, unspecified, unspecified site
        "C8491",  # Mature T/NK-cell lymphomas, unspecified, lymph nodes of head, face, and neck
        "C8492",  # Mature T/NK-cell lymphomas, unspecified, intrathoracic lymph nodes
        "C8493",  # Mature T/NK-cell lymphomas, unspecified, intra-abdominal lymph nodes
        "C8494",  # Mature T/NK-cell lymphomas, unspecified, lymph nodes of axilla and upper limb
        "C8495",  # Mature T/NK-cell lymphomas, unspecified, lymph nodes of inguinal region and lower limb
        "C8496",  # Mature T/NK-cell lymphomas, unspecified, intrapelvic lymph nodes
        "C8497",  # Mature T/NK-cell lymphomas, unspecified, spleen
        "C8498",  # Mature T/NK-cell lymphomas, unspecified, lymph nodes of multiple sites
        "C8499",  # Mature T/NK-cell lymphomas, unspecified, extranodal and solid organ sites
        "C84A0",  # Cutaneous T-cell lymphoma, unspecified, unspecified site
        "C84A1",  # Cutaneous T-cell lymphoma, unspecified lymph nodes of head, face, and neck
        "C84A2",  # Cutaneous T-cell lymphoma, unspecified, intrathoracic lymph nodes
        "C84A3",  # Cutaneous T-cell lymphoma, unspecified, intra-abdominal lymph nodes
        "C84A4",  # Cutaneous T-cell lymphoma, unspecified, lymph nodes of axilla and upper limb
        "C84A5",  # Cutaneous T-cell lymphoma, unspecified, lymph nodes of inguinal region and lower limb
        "C84A6",  # Cutaneous T-cell lymphoma, unspecified, intrapelvic lymph nodes
        "C84A7",  # Cutaneous T-cell lymphoma, unspecified, spleen
        "C84A8",  # Cutaneous T-cell lymphoma, unspecified, lymph nodes of multiple sites
        "C84A9",  # Cutaneous T-cell lymphoma, unspecified, extranodal and solid organ sites
        "C84Z0",  # Other mature T/NK-cell lymphomas, unspecified site
        "C84Z1",  # Other mature T/NK-cell lymphomas, lymph nodes of head, face, and neck
        "C84Z2",  # Other mature T/NK-cell lymphomas, intrathoracic lymph nodes
        "C84Z3",  # Other mature T/NK-cell lymphomas, intra-abdominal lymph nodes
        "C84Z4",  # Other mature T/NK-cell lymphomas, lymph nodes of axilla and upper limb
        "C84Z5",  # Other mature T/NK-cell lymphomas, lymph nodes of inguinal region and lower limb
        "C84Z6",  # Other mature T/NK-cell lymphomas, intrapelvic lymph nodes
        "C84Z7",  # Other mature T/NK-cell lymphomas, spleen
        "C84Z8",  # Other mature T/NK-cell lymphomas, lymph nodes of multiple sites
        "C84Z9",  # Other mature T/NK-cell lymphomas, extranodal and solid organ sites
        "C8510",  # Unspecified B-cell lymphoma, unspecified site
        "C8511",  # Unspecified B-cell lymphoma, lymph nodes of head, face, and neck
        "C8512",  # Unspecified B-cell lymphoma, intrathoracic lymph nodes
        "C8513",  # Unspecified B-cell lymphoma, intra-abdominal lymph nodes
        "C8514",  # Unspecified B-cell lymphoma, lymph nodes of axilla and upper limb
        "C8515",  # Unspecified B-cell lymphoma, lymph nodes of inguinal region and lower limb
        "C8516",  # Unspecified B-cell lymphoma, intrapelvic lymph nodes
        "C8517",  # Unspecified B-cell lymphoma, spleen
        "C8518",  # Unspecified B-cell lymphoma, lymph nodes of multiple sites
        "C8519",  # Unspecified B-cell lymphoma, extranodal and solid organ sites
        "C8520",  # Mediastinal (thymic) large B-cell lymphoma, unspecified site
        "C8521",  # Mediastinal (thymic) large B-cell lymphoma, lymph nodes of head, face, and neck
        "C8522",  # Mediastinal (thymic) large B-cell lymphoma, intrathoracic lymph nodes
        "C8523",  # Mediastinal (thymic) large B-cell lymphoma, intra-abdominal lymph nodes
        "C8524",  # Mediastinal (thymic) large B-cell lymphoma, lymph nodes of axilla and upper limb
        "C8525",  # Mediastinal (thymic) large B-cell lymphoma, lymph nodes of inguinal region and lower limb
        "C8526",  # Mediastinal (thymic) large B-cell lymphoma, intrapelvic lymph nodes
        "C8527",  # Mediastinal (thymic) large B-cell lymphoma, spleen
        "C8528",  # Mediastinal (thymic) large B-cell lymphoma, lymph nodes of multiple sites
        "C8529",  # Mediastinal (thymic) large B-cell lymphoma, extranodal and solid organ sites
        "C8580",  # Other specified types of non-Hodgkin lymphoma, unspecified site
        "C8581",  # Other specified types of non-Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8582",  # Other specified types of non-Hodgkin lymphoma, intrathoracic lymph nodes
        "C8583",  # Other specified types of non-Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8584",  # Other specified types of non-Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8585",  # Other specified types of non-Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8586",  # Other specified types of non-Hodgkin lymphoma, intrapelvic lymph nodes
        "C8587",  # Other specified types of non-Hodgkin lymphoma, spleen
        "C8588",  # Other specified types of non-Hodgkin lymphoma, lymph nodes of multiple sites
        "C8589",  # Other specified types of non-Hodgkin lymphoma, extranodal and solid organ sites
        "C8590",  # Non-Hodgkin lymphoma, unspecified, unspecified site
        "C8591",  # Non-Hodgkin lymphoma, unspecified, lymph nodes of head, face, and neck
        "C8592",  # Non-Hodgkin lymphoma, unspecified, intrathoracic lymph nodes
        "C8593",  # Non-Hodgkin lymphoma, unspecified, intra-abdominal lymph nodes
        "C8594",  # Non-Hodgkin lymphoma, unspecified, lymph nodes of axilla and upper limb
        "C8595",  # Non-Hodgkin lymphoma, unspecified, lymph nodes of inguinal region and lower limb
        "C8596",  # Non-Hodgkin lymphoma, unspecified, intrapelvic lymph nodes
        "C8597",  # Non-Hodgkin lymphoma, unspecified, spleen
        "C8598",  # Non-Hodgkin lymphoma, unspecified, lymph nodes of multiple sites
        "C8599",  # Non-Hodgkin lymphoma, unspecified, extranodal and solid organ sites
        "C860",  # Extranodal NK/T-cell lymphoma, nasal type
        "C8600",  # Extranodal NK/T-cell lymphoma, nasal type not having achieved remission
        "C861",  # Hepatosplenic T-cell lymphoma
        "C8610",  # Hepatosplenic T-cell lymphoma not having achieved remission
        "C862",  # Enteropathy-type (intestinal) T-cell lymphoma
        "C8620",  # Enteropathy-type (intestinal) T-cell lymphoma not having achieved remission
        "C863",  # Subcutaneous panniculitis-like T-cell lymphoma
        "C8630",  # Subcutaneous panniculitis-like T-cell lymphoma not having achieved remission
        "C864",  # Blastic NK-cell lymphoma
        "C8640",  # Blastic NK-cell lymphoma not having achieved remission
        "C865",  # Angioimmunoblastic T-cell lymphoma
        "C8650",  # Angioimmunoblastic T-cell lymphoma not having achieved remission
        "C866",  # Primary cutaneous CD30-positive T-cell proliferations
        "C8660",  # Primary cutaneous CD30-positive T-cell proliferations not having achieved remission
        "C880",  # Waldenstrom macroglobulinemia
        "C8800",  # Waldenstrom macroglobulinemia not having achieved remission
        "C882",  # Heavy chain disease
        "C8820",  # Heavy chain disease not having achieved remission
        "C883",  # Immunoproliferative small intestinal disease
        "C8830",  # Immunoproliferative small intestinal disease not having achieved remission
        "C884",  # Extranodal marginal zone B-cell lymphoma of mucosa-associated lymphoid tissue [MALT-lymphoma]
        "C8840",  # Extranodal marginal zone B-cell lymphoma of mucosa-associated lymphoid tissue [MALT-lymphoma] not having achieved remission
        "C888",  # Other malignant immunoproliferative diseases
        "C8880",  # Other malignant immunoproliferative diseases not having achieved remission
        "C889",  # Malignant immunoproliferative disease, unspecified
        "C8890",  # Malignant immunoproliferative disease, unspecified not having achieved remission
        "C9000",  # Multiple myeloma not having achieved remission
        "C9001",  # Multiple myeloma in remission
        "C9002",  # Multiple myeloma in relapse
        "C9010",  # Plasma cell leukemia not having achieved remission
        "C9011",  # Plasma cell leukemia in remission
        "C9012",  # Plasma cell leukemia in relapse
        "C9020",  # Extramedullary plasmacytoma not having achieved remission
        "C9021",  # Extramedullary plasmacytoma in remission
        "C9022",  # Extramedullary plasmacytoma in relapse
        "C9030",  # Solitary plasmacytoma not having achieved remission
        "C9031",  # Solitary plasmacytoma in remission
        "C9032",  # Solitary plasmacytoma in relapse
        "C9100",  # Acute lymphoblastic leukemia not having achieved remission
        "C9101",  # Acute lymphoblastic leukemia, in remission
        "C9102",  # Acute lymphoblastic leukemia, in relapse
        "C9110",  # Chronic lymphocytic leukemia of B-cell type not having achieved remission
        "C9111",  # Chronic lymphocytic leukemia of B-cell type in remission
        "C9112",  # Chronic lymphocytic leukemia of B-cell type in relapse
        "C9130",  # Prolymphocytic leukemia of B-cell type not having achieved remission
        "C9131",  # Prolymphocytic leukemia of B-cell type, in remission
        "C9132",  # Prolymphocytic leukemia of B-cell type, in relapse
        "C9140",  # Hairy cell leukemia not having achieved remission
        "C9141",  # Hairy cell leukemia, in remission
        "C9142",  # Hairy cell leukemia, in relapse
        "C9150",  # Adult T-cell lymphoma/leukemia (HTLV-1-associated) not having achieved remission
        "C9151",  # Adult T-cell lymphoma/leukemia (HTLV-1-associated), in remission
        "C9152",  # Adult T-cell lymphoma/leukemia (HTLV-1-associated), in relapse
        "C9160",  # Prolymphocytic leukemia of T-cell type not having achieved remission
        "C9161",  # Prolymphocytic leukemia of T-cell type, in remission
        "C9162",  # Prolymphocytic leukemia of T-cell type, in relapse
        "C9190",  # Lymphoid leukemia, unspecified not having achieved remission
        "C9191",  # Lymphoid leukemia, unspecified, in remission
        "C9192",  # Lymphoid leukemia, unspecified, in relapse
        "C91A0",  # Mature B-cell leukemia Burkitt-type not having achieved remission
        "C91A1",  # Mature B-cell leukemia Burkitt-type, in remission
        "C91A2",  # Mature B-cell leukemia Burkitt-type, in relapse
        "C91Z0",  # Other lymphoid leukemia not having achieved remission
        "C91Z1",  # Other lymphoid leukemia, in remission
        "C91Z2",  # Other lymphoid leukemia, in relapse
        "C9200",  # Acute myeloblastic leukemia, not having achieved remission
        "C9201",  # Acute myeloblastic leukemia, in remission
        "C9202",  # Acute myeloblastic leukemia, in relapse
        "C9210",  # Chronic myeloid leukemia, BCR/ABL-positive, not having achieved remission
        "C9211",  # Chronic myeloid leukemia, BCR/ABL-positive, in remission
        "C9212",  # Chronic myeloid leukemia, BCR/ABL-positive, in relapse
        "C9220",  # Atypical chronic myeloid leukemia, BCR/ABL-negative, not having achieved remission
        "C9221",  # Atypical chronic myeloid leukemia, BCR/ABL-negative, in remission
        "C9222",  # Atypical chronic myeloid leukemia, BCR/ABL-negative, in relapse
        "C9230",  # Myeloid sarcoma, not having achieved remission
        "C9231",  # Myeloid sarcoma, in remission
        "C9232",  # Myeloid sarcoma, in relapse
        "C9240",  # Acute promyelocytic leukemia, not having achieved remission
        "C9241",  # Acute promyelocytic leukemia, in remission
        "C9242",  # Acute promyelocytic leukemia, in relapse
        "C9250",  # Acute myelomonocytic leukemia, not having achieved remission
        "C9251",  # Acute myelomonocytic leukemia, in remission
        "C9252",  # Acute myelomonocytic leukemia, in relapse
        "C9260",  # Acute myeloid leukemia with 11q23-abnormality not having achieved remission
        "C9261",  # Acute myeloid leukemia with 11q23-abnormality in remission
        "C9262",  # Acute myeloid leukemia with 11q23-abnormality in relapse
        "C9290",  # Myeloid leukemia, unspecified, not having achieved remission
        "C9291",  # Myeloid leukemia, unspecified in remission
        "C9292",  # Myeloid leukemia, unspecified in relapse
        "C92A0",  # Acute myeloid leukemia with multilineage dysplasia, not having achieved remission
        "C92A1",  # Acute myeloid leukemia with multilineage dysplasia, in remission
        "C92A2",  # Acute myeloid leukemia with multilineage dysplasia, in relapse
        "C92Z0",  # Other myeloid leukemia not having achieved remission
        "C92Z1",  # Other myeloid leukemia, in remission
        "C92Z2",  # Other myeloid leukemia, in relapse
        "C9300",  # Acute monoblastic/monocytic leukemia, not having achieved remission
        "C9301",  # Acute monoblastic/monocytic leukemia, in remission
        "C9302",  # Acute monoblastic/monocytic leukemia, in relapse
        "C9310",  # Chronic myelomonocytic leukemia not having achieved remission
        "C9311",  # Chronic myelomonocytic leukemia, in remission
        "C9312",  # Chronic myelomonocytic leukemia, in relapse
        "C9330",  # Juvenile myelomonocytic leukemia, not having achieved remission
        "C9331",  # Juvenile myelomonocytic leukemia, in remission
        "C9332",  # Juvenile myelomonocytic leukemia, in relapse
        "C9390",  # Monocytic leukemia, unspecified, not having achieved remission
        "C9391",  # Monocytic leukemia, unspecified in remission
        "C9392",  # Monocytic leukemia, unspecified in relapse
        "C93Z0",  # Other monocytic leukemia, not having achieved remission
        "C93Z1",  # Other monocytic leukemia, in remission
        "C93Z2",  # Other monocytic leukemia, in relapse
        "C9400",  # Acute erythroid leukemia, not having achieved remission
        "C9401",  # Acute erythroid leukemia, in remission
        "C9402",  # Acute erythroid leukemia, in relapse
        "C9420",  # Acute megakaryoblastic leukemia not having achieved remission
        "C9421",  # Acute megakaryoblastic leukemia, in remission
        "C9422",  # Acute megakaryoblastic leukemia, in relapse
        "C9430",  # Mast cell leukemia not having achieved remission
        "C9431",  # Mast cell leukemia, in remission
        "C9432",  # Mast cell leukemia, in relapse
        "C9440",  # Acute panmyelosis with myelofibrosis not having achieved remission
        "C9441",  # Acute panmyelosis with myelofibrosis, in remission
        "C9442",  # Acute panmyelosis with myelofibrosis, in relapse
        "C946",  # Myelodysplastic disease, not elsewhere classified
        "C9480",  # Other specified leukemias not having achieved remission
        "C9481",  # Other specified leukemias, in remission
        "C9482",  # Other specified leukemias, in relapse
        "C9500",  # Acute leukemia of unspecified cell type not having achieved remission
        "C9501",  # Acute leukemia of unspecified cell type, in remission
        "C9502",  # Acute leukemia of unspecified cell type, in relapse
        "C9510",  # Chronic leukemia of unspecified cell type not having achieved remission
        "C9511",  # Chronic leukemia of unspecified cell type, in remission
        "C9512",  # Chronic leukemia of unspecified cell type, in relapse
        "C9590",  # Leukemia, unspecified not having achieved remission
        "C9591",  # Leukemia, unspecified, in remission
        "C9592",  # Leukemia, unspecified, in relapse
        "C960",  # Multifocal and multisystemic (disseminated) Langerhans-cell histiocytosis
        "C9620",  # Malignant mast cell neoplasm, unspecified
        "C9621",  # Aggressive systemic mastocytosis
        "C9622",  # Mast cell sarcoma
        "C9629",  # Other malignant mast cell neoplasm
        "C964",  # Sarcoma of dendritic cells (accessory cells)
        "C965",  # Multifocal and unisystemic Langerhans-cell histiocytosis
        "C966",  # Unifocal Langerhans-cell histiocytosis
        "C969",  # Malignant neoplasm of lymphoid, hematopoietic and related tissue, unspecified
        "C96A",  # Histiocytic sarcoma
        "C96Z",  # Other specified malignant neoplasms of lymphoid, hematopoietic and related tissue
        "D3701",  # Neoplasm of uncertain behavior of lip
        "D3702",  # Neoplasm of uncertain behavior of tongue
        "D37030",  # Neoplasm of uncertain behavior of the parotid salivary glands
        "D37031",  # Neoplasm of uncertain behavior of the sublingual salivary glands
        "D37032",  # Neoplasm of uncertain behavior of the submandibular salivary glands
        "D37039",  # Neoplasm of uncertain behavior of the major salivary glands, unspecified
        "D3704",  # Neoplasm of uncertain behavior of the minor salivary glands
        "D3705",  # Neoplasm of uncertain behavior of pharynx
        "D3709",  # Neoplasm of uncertain behavior of other specified sites of the oral cavity
        "D371",  # Neoplasm of uncertain behavior of stomach
        "D372",  # Neoplasm of uncertain behavior of small intestine
        "D373",  # Neoplasm of uncertain behavior of appendix
        "D374",  # Neoplasm of uncertain behavior of colon
        "D375",  # Neoplasm of uncertain behavior of rectum
        "D376",  # Neoplasm of uncertain behavior of liver, gallbladder and bile ducts
        "D378",  # Neoplasm of uncertain behavior of other specified digestive organs
        "D379",  # Neoplasm of uncertain behavior of digestive organ, unspecified
        "D380",  # Neoplasm of uncertain behavior of larynx
        "D381",  # Neoplasm of uncertain behavior of trachea, bronchus and lung
        "D382",  # Neoplasm of uncertain behavior of pleura
        "D383",  # Neoplasm of uncertain behavior of mediastinum
        "D384",  # Neoplasm of uncertain behavior of thymus
        "D385",  # Neoplasm of uncertain behavior of other respiratory organs
        "D386",  # Neoplasm of uncertain behavior of respiratory organ, unspecified
        "D390",  # Neoplasm of uncertain behavior of uterus
        "D3910",  # Neoplasm of uncertain behavior of unspecified ovary
        "D3911",  # Neoplasm of uncertain behavior of right ovary
        "D3912",  # Neoplasm of uncertain behavior of left ovary
        "D392",  # Neoplasm of uncertain behavior of placenta
        "D398",  # Neoplasm of uncertain behavior of other specified female genital organs
        "D399",  # Neoplasm of uncertain behavior of female genital organ, unspecified
        "D400",  # Neoplasm of uncertain behavior of prostate
        "D4010",  # Neoplasm of uncertain behavior of unspecified testis
        "D4011",  # Neoplasm of uncertain behavior of right testis
        "D4012",  # Neoplasm of uncertain behavior of left testis
        "D408",  # Neoplasm of uncertain behavior of other specified male genital organs
        "D409",  # Neoplasm of uncertain behavior of male genital organ, unspecified
        "D4100",  # Neoplasm of uncertain behavior of unspecified kidney
        "D4101",  # Neoplasm of uncertain behavior of right kidney
        "D4102",  # Neoplasm of uncertain behavior of left kidney
        "D4110",  # Neoplasm of uncertain behavior of unspecified renal pelvis
        "D4111",  # Neoplasm of uncertain behavior of right renal pelvis
        "D4112",  # Neoplasm of uncertain behavior of left renal pelvis
        "D4120",  # Neoplasm of uncertain behavior of unspecified ureter
        "D4121",  # Neoplasm of uncertain behavior of right ureter
        "D4122",  # Neoplasm of uncertain behavior of left ureter
        "D413",  # Neoplasm of uncertain behavior of urethra
        "D414",  # Neoplasm of uncertain behavior of bladder
        "D418",  # Neoplasm of uncertain behavior of other specified urinary organs
        "D419",  # Neoplasm of uncertain behavior of unspecified urinary organ
        "D420",  # Neoplasm of uncertain behavior of cerebral meninges
        "D421",  # Neoplasm of uncertain behavior of spinal meninges
        "D429",  # Neoplasm of uncertain behavior of meninges, unspecified
        "D430",  # Neoplasm of uncertain behavior of brain, supratentorial
        "D431",  # Neoplasm of uncertain behavior of brain, infratentorial
        "D432",  # Neoplasm of uncertain behavior of brain, unspecified
        "D433",  # Neoplasm of uncertain behavior of cranial nerves
        "D434",  # Neoplasm of uncertain behavior of spinal cord
        "D438",  # Neoplasm of uncertain behavior of other specified parts of central nervous system
        "D439",  # Neoplasm of uncertain behavior of central nervous system, unspecified
        "D440",  # Neoplasm of uncertain behavior of thyroid gland
        "D4410",  # Neoplasm of uncertain behavior of unspecified adrenal gland
        "D4411",  # Neoplasm of uncertain behavior of right adrenal gland
        "D4412",  # Neoplasm of uncertain behavior of left adrenal gland
        "D442",  # Neoplasm of uncertain behavior of parathyroid gland
        "D443",  # Neoplasm of uncertain behavior of pituitary gland
        "D444",  # Neoplasm of uncertain behavior of craniopharyngeal duct
        "D445",  # Neoplasm of uncertain behavior of pineal gland
        "D446",  # Neoplasm of uncertain behavior of carotid body
        "D447",  # Neoplasm of uncertain behavior of aortic body and other paraganglia
        "D449",  # Neoplasm of uncertain behavior of unspecified endocrine gland
        "D45",  # Polycythemia vera
        "D460",  # Refractory anemia without ring sideroblasts, so stated
        "D461",  # Refractory anemia with ring sideroblasts
        "D4620",  # Refractory anemia with excess of blasts, unspecified
        "D4621",  # Refractory anemia with excess of blasts 1
        "D4622",  # Refractory anemia with excess of blasts 2
        "D464",  # Refractory anemia, unspecified
        "D469",  # Myelodysplastic syndrome, unspecified
        "D46A",  # Refractory cytopenia with multilineage dysplasia
        "D46B",  # Refractory cytopenia with multilineage dysplasia and ring sideroblasts
        "D46C",  # Myelodysplastic syndrome with isolated del(5q) chromosomal abnormality
        "D46Z",  # Other myelodysplastic syndromes
        "D4701",  # Cutaneous mastocytosis
        "D4702",  # Systemic mastocytosis
        "D4709",  # Other mast cell neoplasms of uncertain behavior
        "D471",  # Chronic myeloproliferative disease
        "D472",  # Monoclonal gammopathy
        "D473",  # Essential (hemorrhagic) thrombocythemia
        "D474",  # Osteomyelofibrosis
        "D479",  # Neoplasm of uncertain behavior of lymphoid, hematopoietic and related tissue, unspecified
        "D47Z1",  # Post-transplant lymphoproliferative disorder (PTLD)
        "D47Z2",  # Castleman disease
        "D47Z9",  # Other specified neoplasms of uncertain behavior of lymphoid, hematopoietic and related tissue
        "D480",  # Neoplasm of uncertain behavior of bone and articular cartilage
        "D4819",  # Other specified neoplasm of uncertain behavior of connective and other soft tissue
        "D482",  # Neoplasm of uncertain behavior of peripheral nerves and autonomic nervous system
        "D483",  # Neoplasm of uncertain behavior of retroperitoneum
        "D484",  # Neoplasm of uncertain behavior of peritoneum
        "D485",  # Neoplasm of uncertain behavior of skin
        "D4860",  # Neoplasm of uncertain behavior of unspecified breast
        "D4861",  # Neoplasm of uncertain behavior of right breast
        "D4862",  # Neoplasm of uncertain behavior of left breast
        "D487",  # Neoplasm of uncertain behavior of other specified sites
        "D489",  # Neoplasm of uncertain behavior, unspecified
        "D490",  # Neoplasm of unspecified behavior of digestive system
        "D491",  # Neoplasm of unspecified behavior of respiratory system
        "D492",  # Neoplasm of unspecified behavior of bone, soft tissue, and skin
        "D493",  # Neoplasm of unspecified behavior of breast
        "D494",  # Neoplasm of unspecified behavior of bladder
        "D49511",  # Neoplasm of unspecified behavior of right kidney
        "D49512",  # Neoplasm of unspecified behavior of left kidney
        "D49519",  # Neoplasm of unspecified behavior of unspecified kidney
        "D4959",  # Neoplasm of unspecified behavior of other genitourinary organ
        "D496",  # Neoplasm of unspecified behavior of brain
        "D497",  # Neoplasm of unspecified behavior of endocrine glands and other parts of nervous system
        "D4981",  # Neoplasm of unspecified behavior of retina and choroid
        "D4989",  # Neoplasm of unspecified behavior of other specified sites
        "D499",  # Neoplasm of unspecified behavior of unspecified site
    }

    SNOMEDCT = {
        "421835000",  # Immunoblastic lymphoma associated with acquired immunodeficiency syndrome (disorder)
        "422052002",  # Burkitt's lymphoma associated with acquired immunodeficiency syndrome (disorder)
        "432082001",  # Non-seminomatous malignant neoplasm of testis (disorder)
        "255116009",  # Secondary carcinoma of respiratory and/or digestive systems (disorder)
        "188196004",  # Malignant neoplasm of lower uterine segment (disorder)
        "188526000",  # Hodgkin's paragranuloma of intra-abdominal lymph nodes (disorder)
        "404146005",  # Primary cutaneous diffuse large cell B-cell lymphoma of the leg (disorder)
        "188251003",  # Hypernephroma (disorder)
        "235965006",  # Malignant cystic tumor of exocrine pancreas (disorder)
        "255044008",  # Malignant pinealoma (disorder)
        "373627005",  # Bronchioloalveolar carcinoma (disorder)
        "416274001",  # Primary malignant clear cell tumor of ovary (disorder)
        "93147005",  # Leukemic reticuloendotheliosis of lymph nodes of axilla AND/OR upper limb (disorder)
        "93148000",  # Leukemic reticuloendotheliosis of lymph nodes of head, face AND/OR neck (disorder)
        "93149008",  # Leukemic reticuloendotheliosis of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93450001",  # Erythroleukemia in remission (disorder)
        "188067002",  # Malignant melanoma of lower limb and hip (disorder)
        "188353002",  # Malignant neoplasm of head, neck and face (disorder)
        "188510001",  # Burkitt's lymphoma of lymph nodes of head, face and neck (disorder)
        "188534006",  # Hodgkin's granuloma of lymph nodes of head, face and neck (disorder)
        "188544008",  # Hodgkin's sarcoma of lymph nodes of head, face and neck (disorder)
        "188630009",  # Sezary's disease of lymph nodes of head, face and neck (disorder)
        "188640007",  # Malignant histiocytosis of lymph nodes of head, face and neck (disorder)
        "275419009",  # Primary vulval cancer (disorder)
        "277641001",  # Follicular malignant lymphoma - large cell (disorder)
        "404125001",  # CD-30 positive anaplastic large T-cell cutaneous lymphoma (disorder)
        "414791003",  # Myelodysplastic syndrome, unclassified by World Health Organization classification (disorder)
        "189494003",  # Neoplasm of uncertain or unknown behavior of peripheral nerves and autonomic nervous system (disorder)
        "254628007",  # Carcinoma of lung parenchyma (disorder)
        "278053004",  # Intraductal carcinoma of breast (disorder)
        "302817000",  # Malignant tumor of unknown origin or ill-defined site (disorder)
        "363397008",  # Malignant tumor of roof of nasopharynx (disorder)
        "187702003",  # Malignant tumor of nasopharyngeal soft palate surface (disorder)
        "188198003",  # Malignant neoplasm of overlapping lesion of corpus uteri (disorder)
        "254871000",  # Immature teratoma of ovary (disorder)
        "404132005",  # Pleomorphic small/medium-sized cell cutaneous T-cell lymphoma (disorder)
        "92508006",  # Burkitt's tumor of intra-abdominal lymph nodes (disorder)
        "92509003",  # Burkitt's tumor of intrapelvic lymph nodes (disorder)
        "92510008",  # Burkitt's tumor of intrathoracic lymph nodes (disorder)
        "92514004",  # Burkitt's tumor of lymph nodes of multiple sites (disorder)
        "92515003",  # Burkitt's tumor of spleen (disorder)
        "93545005",  # Hodgkin's paragranuloma of spleen (disorder)
        "187624007",  # Malignant neoplasm of overlapping lesion of lip (disorder)
        "187807003",  # Overlapping malignant lesion of retroperitoneum and peritoneum (disorder)
        "187808008",  # Malignant neoplasm of specified parts of peritoneum (disorder)
        "187838002",  # Malignant neoplasm, overlapping lesion of accessory sinuses (disorder)
        "187868006",  # Malignant neoplasm of lower lobe, bronchus or lung (disorder)
        "188023004",  # Malignant neoplasm of connective and soft tissue of sacrum or coccyx (disorder)
        "188119005",  # Malignant neoplasm of skin of upper limb and shoulder (disorder)
        "188127001",  # Malignant neoplasm of skin of lower limb and hip (disorder)
        "188459004",  # Secondary malignant neoplasm of skin of hip and leg (disorder)
        "188660004",  # Malignant mast cell tumors (disorder)
        "188663002",  # Mast cell malignancy of intrathoracic lymph nodes (disorder)
        "188664008",  # Mast cell malignancy of intra-abdominal lymph nodes (disorder)
        "188667001",  # Mast cell malignancy of intrapelvic lymph nodes (disorder)
        "188738007",  # Granulocytic sarcoma (disorder)
        "254861002",  # Malignant granulosa cell tumor of ovary (disorder)
        "255083005",  # Malignant tumor of anus and anal canal (disorder)
        "286900008",  # Carcinoma of epididymis/spermatic cord (disorder)
        "93206004",  # Malignant mast cell tumor of lymph nodes of multiple sites (disorder)
        "93207008",  # Malignant mast cell tumor of spleen (disorder)
        "93208003",  # Malignant mast cell tumor of extranodal AND/OR solid organ site (disorder)
        "128874001",  # Cutaneous CD30+ lymphoproliferative disorder (disorder)
        "187864008",  # Malignant neoplasm of middle lobe, bronchus or lung (disorder)
        "187881004",  # Malignant neoplasm of thymus, heart and mediastinum (disorder)
        "255101006",  # Sezary disease of skin (disorder)
        "372112000",  # Primary malignant neoplasm of middle lobe, bronchus or lung (disorder)
        "372113005",  # Carcinoma of middle lobe, bronchus or lung (disorder)
        "372135002",  # Primary malignant neoplasm of upper lobe, bronchus or lung (disorder)
        "372136001",  # Carcinoma of upper lobe, bronchus or lung (disorder)
        "404145009",  # Primary cutaneous anaplastic large cell B-cell lymphoma (disorder)
        "4752007",  # Fetal sacral teratoma causing disproportion (disorder)
        "95261008",  # Sezary's disease of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "109357005",  # Primary malignant neoplasm of ill-defined site (disorder)
        "109358000",  # Overlapping malignant neoplasm of ill-defined site (disorder)
        "255102004",  # Angioendotheliomatosis (disorder)
        "269617008",  # Secondary nodes - inguinal/leg (disorder)
        "363357005",  # Malignant neoplasm of ill-defined site (disorder)
        "94712003",  # Mycosis fungoides of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "187757001",  # Malignant neoplasm, overlapping lesion of colon (disorder)
        "187767006",  # Malignant neoplasm of liver and intrahepatic bile ducts (disorder)
        "187866005",  # Malignant neoplasm of middle lobe of lung (disorder)
        "188032002",  # Malignant melanoma of ear and/or external auditory canal (disorder)
        "188042000",  # Malignant melanoma of temple (disorder)
        "188729005",  # Adult T-cell leukemia (disorder)
        "254828009",  # Malignant lipomatous tumor (disorder)
        "275265005",  # Epithelioma basal cell (disorder)
        "277473004",  # B-cell chronic lymphocytic leukemia (disorder)
        "277637000",  # Large cell anaplastic lymphoma (disorder)
        "404070007",  # Round cell liposarcoma (disorder)
        "93152000",  # Leukemic reticuloendotheliosis of extranodal AND/OR solid organ site (disorder)
        "109965004",  # Diffuse non-Hodgkin's lymphoma, lymphoblastic (disorder)
        "277651000",  # Peripheral T-cell lymphoma - pleomorphic small cell (disorder)
        "277653002",  # Peripheral T-cell lymphoma - pleomorphic medium and large cell (disorder)
        "109356001",  # Primary malignant neoplasm of unspecified site (disorder)
        "188044004",  # Malignant melanoma of scalp and/or neck (disorder)
        "188478004",  # Malignant neoplasms of independent (primary) multiple sites (disorder)
        "188513004",  # Burkitt lymphoma of lymph nodes of axilla and upper limb (disorder)
        "188514005",  # Burkitt lymphoma of lymph nodes of inguinal region and lower limb (disorder)
        "188537004",  # Hodgkin's granuloma of lymph nodes of axilla and upper limb (disorder)
        "188538009",  # Hodgkin's granuloma of lymph nodes of inguinal region and lower limb (disorder)
        "188547001",  # Hodgkin's sarcoma of lymph nodes of axilla and upper limb (disorder)
        "188548006",  # Hodgkin's sarcoma of lymph nodes of inguinal region and lower limb (disorder)
        "188554007",  # Hodgkin's disease, lymphocytic-histiocytic predominance of lymph nodes of head, face and neck (disorder)
        "188558005",  # Hodgkin's disease, lymphocytic-histiocytic predominance of lymph nodes of axilla and upper limb (disorder)
        "188559002",  # Hodgkin's disease, lymphocytic-histiocytic predominance of lymph nodes of inguinal region and lower limb (disorder)
        "188565002",  # Hodgkin's disease, nodular sclerosis of lymph nodes of head, face and neck (disorder)
        "188568000",  # Hodgkin's disease, nodular sclerosis of lymph nodes of axilla and upper limb (disorder)
        "188569008",  # Hodgkin's disease, nodular sclerosis of lymph nodes of inguinal region and lower limb (disorder)
        "188575004",  # Hodgkin's disease, mixed cellularity of lymph nodes of head, face and neck (disorder)
        "188578002",  # Hodgkin's disease, mixed cellularity of lymph nodes of axilla and upper limb (disorder)
        "188579005",  # Hodgkin's disease, mixed cellularity of lymph nodes of inguinal region and lower limb (disorder)
        "188585003",  # Hodgkin's disease, lymphocytic depletion of lymph nodes of head, face and neck (disorder)
        "188589009",  # Hodgkin's disease, lymphocytic depletion of lymph nodes of axilla and upper limb (disorder)
        "188590000",  # Hodgkin's disease, lymphocytic depletion of lymph nodes of inguinal region and lower limb (disorder)
        "188642004",  # Malignant histiocytosis of lymph nodes of inguinal region and lower limb (disorder)
        "188665009",  # Mast cell malignancy of lymph nodes of axilla and upper limb (disorder)
        "188666005",  # Mast cell malignancy of lymph nodes of inguinal region and lower limb (disorder)
        "188691005",  # Malignant immunoproliferative small intestinal disease (disorder)
        "269616004",  # Secondary nodes - axilla/arm (disorder)
        "314988008",  # Metastasis from malignant tumor of skin (disorder)
        "314989000",  # Metastasis from malignant tumor of soft tissues (disorder)
        "314990009",  # Metastasis from malignant tumor of bone (disorder)
        "314991008",  # Metastasis from malignant tumor of adrenal gland (disorder)
        "314992001",  # Metastasis from malignant tumor of cervix (disorder)
        "314993006",  # Metastasis from malignant tumor of uterus (disorder)
        "314995004",  # Metastasis from malignant tumor of bladder (disorder)
        "314996003",  # Metastasis from malignant tumor of kidney (disorder)
        "314997007",  # Metastasis from malignant tumor of rectum (disorder)
        "314998002",  # Metastasis from malignant tumor of colon (disorder)
        "314999005",  # Metastasis from malignant tumor of pancreas (disorder)
        "315000005",  # Metastasis from malignant tumor of liver (disorder)
        "315001009",  # Metastasis from malignant tumor of gallbladder (disorder)
        "315002002",  # Metastasis from malignant tumor of stomach (disorder)
        "315003007",  # Metastasis from malignant tumor of esophagus (disorder)
        "315004001",  # Metastasis from malignant tumor of breast (disorder)
        "315005000",  # Metastasis from malignant tumor of bronchus (disorder)
        "315006004",  # Metastasis from malignant neoplasm of lung (disorder)
        "315007008",  # Metastasis from malignant tumor of thyroid (disorder)
        "315008003",  # Metastasis from malignant tumor of buccal cavity (disorder)
        "315009006",  # Metastasis from malignant tumor of tongue (disorder)
        "369602008",  # Malignant tumor involving an organ by separate metastasis from bladder (disorder)
        "369603003",  # Malignant tumor involving an organ by separate metastasis from endometrium (disorder)
        "369605005",  # Malignant tumor involving an organ by separate metastasis from ovary (disorder)
        "369606006",  # Malignant tumor involving an organ by separate metastasis from prostate (disorder)
        "369607002",  # Malignant tumor involving an organ by separate metastasis from uterine cervix (disorder)
        "369608007",  # Malignant tumor involving an organ by separate metastasis from uterus (disorder)
        "369609004",  # Malignant tumor involving an organ by separate metastasis from vagina (disorder)
        "92511007",  # Burkitt's tumor of lymph nodes of axilla AND/OR upper limb (disorder)
        "92512000",  # Burkitt's tumor of lymph nodes of head, face AND/OR neck (disorder)
        "92513005",  # Burkitt's tumor of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "92516002",  # Burkitt's tumor of extranodal AND/OR solid organ site (disorder)
        "93136003",  # Letterer-Siwe disease of lymph nodes of axilla AND/OR upper limb (disorder)
        "93137007",  # Letterer-Siwe disease of lymph nodes of head, face AND/OR neck (disorder)
        "93138002",  # Letterer-Siwe disease of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93141006",  # Letterer-Siwe disease of extranodal AND/OR solid organ site (disorder)
        "93185008",  # Malignant histiocytosis of lymph nodes of axilla AND/OR upper limb (disorder)
        "93186009",  # Malignant histiocytosis of lymph nodes of head, face AND/OR neck (disorder)
        "93187000",  # Malignant histiocytosis of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93190006",  # Malignant histiocytosis of extranodal AND/OR solid organ site (disorder)
        "93194002",  # Malignant lymphoma of lymph nodes of axilla AND/OR upper limb (disorder)
        "93195001",  # Malignant lymphoma of lymph nodes of head, face AND/OR neck (disorder)
        "93196000",  # Malignant lymphoma of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93203007",  # Malignant mast cell tumor of lymph nodes of axilla AND/OR upper limb (disorder)
        "93204001",  # Malignant mast cell tumor of lymph nodes of head, face AND/OR neck (disorder)
        "93205000",  # Malignant mast cell tumor of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93487009",  # Hodgkin's disease, lymphocytic depletion of lymph nodes of axilla AND/OR upper limb (disorder)
        "93488004",  # Hodgkin's disease, lymphocytic depletion of lymph nodes of head, face AND/OR neck (disorder)
        "93489007",  # Hodgkin's disease, lymphocytic depletion of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93492006",  # Hodgkin's disease, lymphocytic depletion of extranodal AND/OR solid organ site (disorder)
        "93496009",  # Hodgkin's disease, lymphocytic-histiocytic predominance of lymph nodes of axilla AND/OR upper limb (disorder)
        "93497000",  # Hodgkin's disease, lymphocytic-histiocytic predominance of lymph nodes of head, face AND/OR neck (disorder)
        "93498005",  # Hodgkin's disease, lymphocytic-histiocytic predominance of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93501005",  # Hodgkin's disease, lymphocytic-histiocytic predominance of extranodal AND/OR solid organ site (disorder)
        "93505001",  # Hodgkin's disease, mixed cellularity of lymph nodes of axilla AND/OR upper limb (disorder)
        "93506000",  # Hodgkin's disease, mixed cellularity of lymph nodes of head, face AND/OR neck (disorder)
        "93507009",  # Hodgkin's disease, mixed cellularity of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93510002",  # Hodgkin's disease, mixed cellularity of extranodal AND/OR solid organ site (disorder)
        "93514006",  # Hodgkin's disease, nodular sclerosis of lymph nodes of axilla AND/OR upper limb (disorder)
        "93515007",  # Hodgkin's disease, nodular sclerosis of lymph nodes of head, face AND/OR neck (disorder)
        "93516008",  # Hodgkin's disease, nodular sclerosis of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93519001",  # Hodgkin's disease, nodular sclerosis of extranodal AND/OR solid organ site (disorder)
        "93523009",  # Hodgkin's disease of lymph nodes of axilla AND/OR upper limb (disorder)
        "93524003",  # Hodgkin's disease of lymph nodes of head, face AND/OR neck (disorder)
        "93525002",  # Hodgkin's disease of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93528000",  # Hodgkin's disease of extranodal AND/OR solid organ site (disorder)
        "93532006",  # Hodgkin's granuloma of lymph nodes of axilla AND/OR upper limb (disorder)
        "93533001",  # Hodgkin's granuloma of lymph nodes of head, face AND/OR neck (disorder)
        "93534007",  # Hodgkin's granuloma of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93537000",  # Hodgkin's granuloma of extranodal AND/OR solid organ site (disorder)
        "93541001",  # Hodgkin's paragranuloma of lymph nodes of axilla AND/OR upper limb (disorder)
        "93542008",  # Hodgkin's paragranuloma of lymph nodes of head, face AND/OR neck (disorder)
        "93543003",  # Hodgkin's paragranuloma of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93546006",  # Hodgkin's paragranuloma of extranodal AND/OR solid organ site (disorder)
        "93550004",  # Hodgkin's sarcoma of lymph nodes of axilla AND/OR upper limb (disorder)
        "93551000",  # Hodgkin's sarcoma of lymph nodes of head, face AND/OR neck (disorder)
        "93552007",  # Hodgkin's sarcoma of lymph nodes of inguinal region AND/OR lower limb (disorder)
        "93555009",  # Hodgkin's sarcoma of extranodal AND/OR solid organ site (disorder)
        "93951006",  # Primary malignant neoplasm of pelvic bone (disorder)
        "94710006",  # Mycosis fungoides of lymph nodes of axilla AND/OR upper limb (disorder)
        "94711005",  # Mycosis fungoides of lymph nodes of head, face AND/OR neck (disorder)
        "94715001",  # Mycosis fungoides of extranodal AND/OR solid organ site (disorder)
        "95194004",  # Nodular lymphoma of extranodal AND/OR solid organ site (disorder)
        "95260009",  # Sezary's disease of lymph nodes of head, face AND/OR neck (disorder)
        "95264000",  # Sezary's disease of extranodal AND/OR solid organ site (disorder)
        "109996008",  # Myelodysplastic syndrome: Refractory anemia, without ringed sideroblasts, without excess blasts (disorder)
        "110000005",  # Refractory anemia with excess blasts in transformation (disorder)
        "187991006",  # Malignant neoplasm of connective and soft tissue of upper limb and shoulder (disorder)
        "187992004",  # Malignant neoplasm of connective and soft tissue of shoulder (disorder)
        "187993009",  # Malignant neoplasm of connective and soft tissue, upper arm (disorder)
        "187994003",  # Malignant neoplasm of connective and soft tissue of forearm (disorder)
        "187995002",  # Malignant neoplasm of connective and soft tissue of hand (disorder)
        "187996001",  # Malignant neoplasm of connective and soft tissue of finger (disorder)
        "187997005",  # Malignant neoplasm of connective and soft tissue of thumb (disorder)
        "187999008",  # Malignant neoplasm of connective and soft tissue of hip and lower limb (disorder)
        "188000002",  # Malignant neoplasm of connective and soft tissue of hip (disorder)
        "188001003",  # Malignant neoplasm of connective and soft tissue of thigh and upper leg (disorder)
        "188002005",  # Malignant neoplasm of connective and soft tissue of popliteal space (disorder)
        "188003000",  # Malignant neoplasm of connective and soft tissue of lower leg (disorder)
        "188004006",  # Malignant neoplasm of connective and soft tissue of foot (disorder)
        "188005007",  # Malignant neoplasm of connective and soft tissue of toe (disorder)
        "188006008",  # Malignant neoplasm of connective and soft tissue of great toe (disorder)
        "188010006",  # Malignant neoplasm of connective and soft tissue of axilla (disorder)
        "188013008",  # Malignant neoplasm of connective and soft tissues of thoracic spine (disorder)
        "188015001",  # Malignant neoplasm of connective and soft tissue of abdomen (disorder)
        "188016000",  # Malignant neoplasm of connective and soft tissue of abdominal wall (disorder)
        "188017009",  # Malignant neoplasm of connective and soft tissues of lumbar spine (disorder)
        "188019007",  # Malignant neoplasm of connective and soft tissue of pelvis (disorder)
        "188020001",  # Malignant neoplasm of connective and soft tissue of buttock (disorder)
        "188021002",  # Malignant neoplasm of connective and soft tissue of inguinal region (disorder)
        "188022009",  # Malignant neoplasm of connective and soft tissue of perineum (disorder)
        "188736006",  # Subacute myeloid leukemia (disorder)
        "188741003",  # Aleukemic myeloid leukemia (disorder)
        "188745007",  # Chronic monocytic leukemia (disorder)
        "188746008",  # Subacute monocytic leukemia (disorder)
        "188748009",  # Aleukemic monocytic leukemia (disorder)
        "20224008",  # Delta heavy chain disease (disorder)
        "277601005",  # Acute monoblastic leukemia (disorder)
        "285598005",  # Metastasis to trachea of unknown primary (disorder)
        "285603002",  # Metastasis to bronchus of unknown primary (disorder)
        "285604008",  # Metastasis to lung of unknown primary (disorder)
        "285605009",  # Metastasis to pleura of unknown primary (disorder)
        "285606005",  # Metastasis to heart of unknown primary (disorder)
        "285607001",  # Metastasis to mediastinum of unknown primary (disorder)
        "285608006",  # Metastasis to thymus of unknown primary (disorder)
        "285609003",  # Metastasis to small intestine of unknown primary (disorder)
        "285610008",  # Metastasis to large intestine of unknown primary (disorder)
        "285611007",  # Metastasis to colon of unknown primary (disorder)
        "285612000",  # Metastasis to rectum of unknown primary (disorder)
        "285613005",  # Metastasis to liver of unknown primary (disorder)
        "285614004",  # Metastasis to pancreas of unknown primary (disorder)
        "285615003",  # Metastasis to spleen of unknown primary (disorder)
        "285616002",  # Metastasis to peritoneum of unknown primary (disorder)
        "285617006",  # Metastasis to retroperitoneum of unknown primary (disorder)
        "285618001",  # Metastasis to bone of unknown primary (disorder)
        "285619009",  # Metastasis to vertebral column of unknown primary (disorder)
        "285631006",  # Metastasis to skin of unknown primary (disorder)
        "285633009",  # Metastasis to soft tissue of unknown primary (disorder)
        "285634003",  # Metastasis to breast of unknown primary (disorder)
        "285635002",  # Metastasis to uterus of unknown primary (disorder)
        "285637005",  # Metastasis to ovary of unknown primary (disorder)
        "285638000",  # Metastasis to vagina of unknown primary (disorder)
        "285639008",  # Metastasis to kidney of unknown primary (disorder)
        "285640005",  # Metastasis to bladder of unknown primary (disorder)
        "285641009",  # Metastasis to brain of unknown primary (disorder)
        "285642002",  # Metastasis to eye of unknown primary (disorder)
        "285643007",  # Metastasis to adrenal gland of unknown primary (disorder)
        "285644001",  # Metastasis to lymph node of unknown primary (disorder)
        "285645000",  # Disseminated malignancy of unknown primary (disorder)
        "285839005",  # Acute myelomonocytic leukemia - eosinophilic variant (disorder)
        "314987003",  # Metastasis from malignant melanoma of skin (disorder)
        "314994000",  # Metastasis from malignant tumor of prostate (disorder)
        "369455009",  # Malignant tumor involving rectum by separate metastasis from endometrium (disorder)
        "369456005",  # Malignant tumor involving rectum by separate metastasis from fallopian tube (disorder)
        "369457001",  # Malignant tumor involving rectum by separate metastasis from ovary (disorder)
        "369458006",  # Malignant tumor involving rectum by separate metastasis from prostate (disorder)
        "369459003",  # Malignant tumor involving rectum by separate metastasis from uterine cervix (disorder)
        "369460008",  # Malignant tumor involving rectum by separate metastasis from uterus (disorder)
        "369461007",  # Malignant tumor involving rectum by separate metastasis from vagina (disorder)
        "369464004",  # Malignant tumor involving ureter by separate metastasis from bladder (disorder)
        "369467006",  # Malignant tumor involving urethra by separate metastasis from bladder (disorder)
        "369468001",  # Malignant tumor involving urethra by separate metastasis from prostate (disorder)
        "369476004",  # Malignant tumor involving bladder by separate metastasis from endometrium (disorder)
        "369477008",  # Malignant tumor involving bladder by separate metastasis from fallopian tube (disorder)
        "369478003",  # Malignant tumor involving bladder by separate metastasis from ovary (disorder)
        "369479006",  # Malignant tumor involving bladder by separate metastasis from prostate (disorder)
        "369480009",  # Malignant tumor involving bladder by separate metastasis from uterine cervix (disorder)
        "369481008",  # Malignant tumor involving bladder by separate metastasis from uterus (disorder)
        "369482001",  # Malignant tumor involving bladder by separate metastasis from vagina (disorder)
        "369484000",  # Malignant tumor involving vasa deferentia by separate metastasis from prostate (disorder)
        "369486003",  # Malignant tumor involving prostate by separate metastasis from bladder (disorder)
        "369491002",  # Malignant tumor involving seminal vesicle by separate metastasis from bladder (disorder)
        "369492009",  # Malignant tumor involving seminal vesicle by separate metastasis from prostate (disorder)
        "369500009",  # Malignant tumor involving uterine cervix by separate metastasis from fallopian tube (disorder)
        "369501008",  # Malignant tumor involving uterine cervix by separate metastasis from ovary (disorder)
        "369502001",  # Malignant tumor involving uterine corpus by separate metastasis from bladder (disorder)
        "369538008",  # Malignant tumor involving left broad ligament by metastasis from ovary (disorder)
        "369540003",  # Malignant tumor involving right broad ligament by metastasis from ovary (disorder)
        "369542006",  # Malignant tumor involving left fallopian tube by separate metastasis from endometrium (disorder)
        "369543001",  # Malignant tumor involving left fallopian tube by separate metastasis from ovary (disorder)
        "369544007",  # Malignant tumor involving left fallopian tube by separate metastasis from right fallopian tube (disorder)
        "369545008",  # Malignant tumor involving left fallopian tube by separate metastasis from uterus (disorder)
        "369546009",  # Malignant tumor involving left fallopian tube by separate metastasis from vagina (disorder)
        "369553000",  # Malignant tumor involving right fallopian tube by separate metastasis from endometrium (disorder)
        "369554006",  # Malignant tumor involving right fallopian tube by separate metastasis from left fallopian tube (disorder)
        "369555007",  # Malignant tumor involving right fallopian tube by separate metastasis from ovary (disorder)
        "369556008",  # Malignant tumor involving right fallopian tube by separate metastasis from uterine cervix (disorder)
        "369557004",  # Malignant tumor involving right fallopian tube by separate metastasis from uterus (disorder)
        "369558009",  # Malignant tumor involving right fallopian tube by separate metastasis from vagina (disorder)
        "369560006",  # Malignant tumor involving left ovary by separate metastasis from endometrium (disorder)
        "369561005",  # Malignant tumor involving left ovary by separate metastasis from fallopian tube (disorder)
        "369562003",  # Malignant tumor involving left ovary by separate metastasis from right ovary (disorder)
        "369563008",  # Malignant tumor involving left ovary by separate metastasis from uterine cervix (disorder)
        "369564002",  # Malignant tumor involving left ovary by separate metastasis from vagina (disorder)
        "369565001",  # Malignant tumor involving left ovary by separate metastasis uterus (disorder)
        "369568004",  # Malignant tumor involving right ovary by separate metastasis from endometrium (disorder)
        "369569007",  # Malignant tumor involving right ovary by separate metastasis from fallopian tube (disorder)
        "369570008",  # Malignant tumor involving right ovary by separate metastasis from left ovary (disorder)
        "369571007",  # Malignant tumor involving right ovary by separate metastasis from uterine cervix (disorder)
        "369572000",  # Malignant tumor involving right ovary by separate metastasis from uterus (disorder)
        "369573005",  # Malignant tumor involving right ovary by separate metastasis from vagina (disorder)
        "369574004",  # Malignant tumor involving uterine cervix by separate metastasis from vagina (disorder)
        "369575003",  # Malignant tumor involving uterine corpus by separate metastasis from fallopian tube (disorder)
        "369576002",  # Malignant tumor involving uterine corpus by separate metastasis from ovary (disorder)
        "369577006",  # Malignant tumor involving uterine corpus by separate metastasis from uterine cervix (disorder)
        "369578001",  # Malignant tumor involving uterine corpus by separate metastasis from vagina (disorder)
        "369581006",  # Malignant tumor involving vagina by separate metastasis from bladder (disorder)
        "369582004",  # Malignant tumor involving vagina by separate metastasis from endometrium (disorder)
        "369583009",  # Malignant tumor involving vagina by separate metastasis from fallopian tube (disorder)
        "369584003",  # Malignant tumor involving vagina by separate metastasis from ovary (disorder)
        "369585002",  # Malignant tumor involving vagina by separate metastasis from uterine cervix (disorder)
        "369586001",  # Malignant tumor involving vagina by separate metastasis from uterus (disorder)
        "369588000",  # Malignant tumor involving vulva by separate metastasis from endometrium (disorder)
        "369589008",  # Malignant tumor involving vulva by separate metastasis from fallopian tube (disorder)
        "369590004",  # Malignant tumor involving vulva by separate metastasis from ovary (disorder)
        "369591000",  # Malignant tumor involving vulva by separate metastasis from uterine cervix (disorder)
        "369592007",  # Malignant tumor involving vulva by separate metastasis from uterus (disorder)
        "369593002",  # Malignant tumor involving vulva by separate metastasis from vagina (disorder)
        "369604009",  # Malignant tumor involving an organ by separate metastasis from fallopian tube (disorder)
        "369610009",  # Malignant tumor involving left fallopian tube by separate metastasis from uterine cervix (disorder)
        "403920004",  # Minimal deviation malignant melanoma (disorder)
        "413442004",  # Acute monocytic/monoblastic leukemia (disorder)
        "415285009",  # Myelodysplastic syndrome with multilineage dysplasia (disorder)
        "415286005",  # Myelodysplastic syndrome with ring sideroblasts and multilineage dysplasia (disorder)
        "422782004",  # Primary malignant neoplasm of ovary, with widespread metastatic disease (disorder)
        "423610004",  # Rhabdomyosarcoma of connective or soft tissue (disorder)
        "423987006",  # Primary malignant neoplasm of vulva, with widespread metastatic disease (disorder)
        "424887002",  # Primary malignant neoplasm of thyroid gland, metastatic to bone (disorder)
        "425749006",  # Subacute myeloid leukemia in remission (disorder)
        "60620005",  # Epsilon heavy chain disease (disorder)
        "871843002",  # Malignant neoplasm of connective and soft tissue of sacrococcygeal region (disorder)
        "1217010007",  # Human epidermal growth factor 2 expressing colon and/or rectum malignant neoplasm (disorder)
        "1236965009",  # Metastatic renal cell carcinoma to kidney except renal pelvis (disorder)
        "1240357005",  # Metastatic renal cell carcinoma to kidney (disorder)
        "1254729000",  # Metastatic lobular carcinoma to breast (disorder)
        "1254730005",  # Metastatic lobular carcinoma to left female breast (disorder)
        "1254731009",  # Metastatic lobular carcinoma to right female breast (disorder)
        "1263997005",  # Metastatic pilomatrix carcinoma to skin (disorder)
        "1263998000",  # Metastatic cystic renal cell carcinoma to kidney (disorder)
        "1264257008",  # Metastatic apocrine adenocarcinoma to skin (disorder)
        "1264521002",  # Metastatic collecting duct carcinoma to kidney (disorder)
        "1264523004",  # Metastatic cholangiocarcinoma to perihilar bile duct (disorder)
        "1264524005",  # Metastatic cholangiocarcinoma to biliary tract (disorder)
        "1268355005",  # Metastatic eccrine porocarcinoma to skin (disorder)
        "133841000119105",  # Merkel cell carcinoma of face (disorder)
        "133871000119103",  # Merkel cell carcinoma of upper limb (disorder)
        "133881000119100",  # Merkel cell carcinoma of lower limb (disorder)
        "254980001",  # Malignant tumor of acoustic vestibular nerve (disorder)
        "351461000119100",  # Merkel cell carcinoma of right lower limb (disorder)
        "352071000119105",  # Merkel cell carcinoma of left lower limb (disorder)
        "771080008",  # Hereditary site-specific ovarian cancer syndrome (disorder)
        "781076008",  # Metastatic malignant neoplasm to colon and/or rectum (disorder)
        "781382000",  # Malignant neoplasm of colon and/or rectum (disorder)
        "1080091000119100",  # Infiltrating ductal carcinoma of axillary tail of left female breast (disorder)
        "1080101000119105",  # Infiltrating duct carcinoma of left female breast (disorder)
        "1080111000119108",  # Infiltrating ductal carcinoma of central portion of left female breast (disorder)
        "1080121000119101",  # Infiltrating ductal carcinoma of lower inner quadrant of left female breast (disorder)
        "1080131000119103",  # Infiltrating ductal carcinoma of lower outer quadrant of left female breast (disorder)
        "1080151000119109",  # Infiltrating ductal carcinoma of upper inner quadrant of left female breast (disorder)
        "1080161000119106",  # Infiltrating ductal carcinoma of upper outer quadrant of left female breast (disorder)
        "1080171000119100",  # Infiltrating ductal carcinoma of axillary tail of right female breast (disorder)
        "1080181000119102",  # Infiltrating duct carcinoma of right female breast (disorder)
        "1080191000119104",  # Infiltrating ductal carcinoma of central portion of right female breast (disorder)
        "1080201000119101",  # Infiltrating ductal carcinoma of lower inner quadrant of right female breast (disorder)
        "1080211000119103",  # Infiltrating ductal carcinoma of lower outer quadrant of right female breast (disorder)
        "1080231000119108",  # Infiltrating ductal carcinoma of upper inner quadrant of right female breast (disorder)
        "1080241000119104",  # Infiltrating ductal carcinoma of upper outer quadrant of right female breast (disorder)
        "1080261000119100",  # Infiltrating lobular carcinoma of left female breast (disorder)
        "1080341000119105",  # Infiltrating lobular carcinoma of right female breast (disorder)
        "1081551000119106",  # Recurrent primary malignant neoplasm of left female breast (disorder)
        "1081561000119108",  # Recurrent primary malignant neoplasm of right female breast (disorder)
        "109886000",  # Overlapping malignant neoplasm of female breast (disorder)
        "109887009",  # Overlapping malignant neoplasm of male breast (disorder)
        "116691000119101",  # Marginal zone lymphoma of spleen (disorder)
        "1222643003",  # Hereditary malignant neuroendocrine neoplasm of small intestine (disorder)
        "123631000119103",  # Malignant poorly differentiated neuroendocrine carcinoma (disorder)
        "123691000119104",  # Malignant carcinoid tumor of sigmoid colon (disorder)
        "1237340007",  # Primary malignant neuroendocrine neoplasm of thymus gland (disorder)
        "1237491009",  # Metastatic carcinoma to male breast (disorder)
        "1255740000",  # Letterer-Siwe disease of lymph nodes of head (disorder)
        "1255741001",  # Letterer-Siwe disease of lymph nodes of neck (disorder)
        "1255742008",  # Letterer-Siwe disease of lymph nodes of head and neck (disorder)
        "1255813007",  # Letterer-Siwe disease of lymph nodes of upper limb (disorder)
        "1255885008",  # Letterer-Siwe disease of lymph nodes of lower limb (disorder)
        "1259370003",  # Primary oat cell carcinoma of lung (disorder)
        "1259455003",  # Primary infiltrating duct carcinoma of female breast (disorder)
        "1259474006",  # Primary infiltrating ductal carcinoma of central portion of left female breast (disorder)
        "1259475007",  # Primary infiltrating ductal carcinoma of central portion of right female breast (disorder)
        "1259477004",  # Primary infiltrating ductal carcinoma of upper outer quadrant of left female breast (disorder)
        "1259479001",  # Primary infiltrating ductal carcinoma of upper outer quadrant of right female breast (disorder)
        "1259481004",  # Primary infiltrating ductal carcinoma of upper inner quadrant of right female breast (disorder)
        "1259483001",  # Primary infiltrating ductal carcinoma of upper inner quadrant of left female breast (disorder)
        "1259484007",  # Primary infiltrating lobular carcinoma of left female breast (disorder)
        "1259486009",  # Primary infiltrating lobular carcinoma of right female breast (disorder)
        "1259512004",  # Primary carcinoma of male breast (disorder)
        "1259598001",  # Primary small cell carcinoma of prostate (disorder)
        "1259651001",  # Primary infiltrating ductal carcinoma of lower outer quadrant of right female breast (disorder)
        "1259653003",  # Primary infiltrating ductal carcinoma of lower outer quadrant of left female breast (disorder)
        "1259655005",  # Primary infiltrating ductal carcinoma of lower inner quadrant of right female breast (disorder)
        "1259657002",  # Primary infiltrating ductal carcinoma of lower inner quadrant of left female breast (disorder)
        "1259659004",  # Primary infiltrating ductal carcinoma of axillary tail of left female breast (disorder)
        "1259660009",  # Primary infiltrating ductal carcinoma of axillary tail of right female breast (disorder)
        "1259678004",  # Primary parafollicular cell carcinoma (disorder)
        "1264496004",  # Metastatic carcinoma to female breast (disorder)
        "1264498003",  # Metastatic infiltrating duct carcinoma to female breast (disorder)
        "133531000119104",  # Malignant neuroendocrine tumor (disorder)
        "15635761000119103",  # Infiltrating duct carcinoma of bilateral female breasts (disorder)
        "15635801000119106",  # Primary malignant neoplasm of bilateral female breasts (disorder)
        "188147009",  # Malignant neoplasm of nipple and areola of female breast (disorder)
        "188151006",  # Malignant neoplasm of central part of female breast (disorder)
        "188152004",  # Malignant neoplasm of upper-inner quadrant of female breast (disorder)
        "188153009",  # Malignant neoplasm of lower-inner quadrant of female breast (disorder)
        "188154003",  # Malignant neoplasm of upper-outer quadrant of female breast (disorder)
        "188155002",  # Malignant neoplasm of lower-outer quadrant of female breast (disorder)
        "188156001",  # Malignant neoplasm of axillary tail of female breast (disorder)
        "188159008",  # Malignant neoplasm of ectopic site of female breast (disorder)
        "188163001",  # Malignant neoplasm of nipple and areola of male breast (disorder)
        "188168005",  # Malignant neoplasm of ectopic site of male breast (disorder)
        "188679001",  # Diffuse non-Hodgkin's lymphoma undifferentiated (diffuse) (disorder)
        "253003009",  # Carcinoid bronchial adenoma (disorder)
        "253030004",  # Glomus vagale tumor (disorder)
        "254633006",  # Oat cell carcinoma of lung (disorder)
        "255038009",  # Growth hormone releasing factor-secreting tumor (disorder)
        "255040004",  # Parathyroid hormone-related peptide-secreting tumor (disorder)
        "255042007",  # Gastric inhibitory peptide-secreting tumor (disorder)
        "303017006",  # Malignant lymphoma, convoluted cell type (disorder)
        "353421000119109",  # Primary malignant neoplasm of axillary tail of left female breast (disorder)
        "353431000119107",  # Primary malignant neoplasm of female left breast (disorder)
        "353441000119103",  # Primary malignant neoplasm of central portion of female left breast (disorder)
        "353501000119104",  # Primary malignant neoplasm of axillary tail of right female breast (disorder)
        "353511000119101",  # Primary malignant neoplasm of female right breast (disorder)
        "353521000119108",  # Primary malignant neoplasm of central portion of female right breast (disorder)
        "354491000119109",  # Primary malignant neoplasm of left male breast (disorder)
        "354591000119108",  # Primary malignant neoplasm of right male breast (disorder)
        "372064008",  # Malignant neoplasm of female breast (disorder)
        "372095001",  # Malignant neoplasm of male breast (disorder)
        "372096000",  # Carcinoma of male breast (disorder)
        "396198006",  # Small cell carcinoma of prostate (disorder)
        "445238008",  # Malignant carcinoid tumor (disorder)
        "447782002",  # Malignant epithelial neoplasm of female breast (disorder)
        "447784001",  # Sarcoma of axillary tail of female breast (disorder)
        "448257000",  # Sarcoma of male breast (disorder)
        "448388003",  # Sarcoma of lower outer quadrant of female breast (disorder)
        "448408001",  # Sarcoma of upper inner quadrant of female breast (disorder)
        "448435005",  # Sarcoma lower inner quadrant of female breast (disorder)
        "448436006",  # Sarcoma of central portion of female breast (disorder)
        "448449001",  # Sarcoma of female breast (disorder)
        "448451002",  # Sarcoma upper outer quadrant of female breast (disorder)
        "448952004",  # Infiltrating duct carcinoma of female breast (disorder)
        "45221000119105",  # Primary invasive malignant neoplasm of female breast (disorder)
        "699356008",  # Endometrial stromal sarcoma (disorder)
        "705176003",  # Metastatic carcinoid tumor (disorder)
        "709517003",  # Malignant carcinoid tumor of small intestine (disorder)
        "709830006",  # Malignant carcinoid tumor of stomach (disorder)
        "713573006",  # Malignant carcinoid tumor of rectum (disorder)
        "713574000",  # Malignant carcinoid tumor of kidney (disorder)
        "717921000",  # Poorly-differentiated neuroendocrine carcinoma of thymus (disorder)
        "717922007",  # Well-differentiated neuroendocrine carcinoma of thymus (disorder)
        "726652005",  # Malignant carcinoid tumor of thymus (disorder)
        "726653000",  # Malignant carcinoid tumor of bronchus (disorder)
        "726654006",  # Malignant carcinoid tumor of colon (disorder)
        "783006001",  # Low-grade neuroendocrine neoplasm of corpus uteri (disorder)
        "789004008",  # Parafollicular cell carcinoma (disorder)
        "93133006",  # Letterer-Siwe disease of intra-abdominal lymph nodes (disorder)
        "93134000",  # Letterer-Siwe disease of intrapelvic lymph nodes (disorder)
        "93135004",  # Letterer-Siwe disease of intrathoracic lymph nodes (disorder)
        "93139005",  # Letterer-Siwe disease of lymph nodes of multiple sites (disorder)
        "93140007",  # Letterer-Siwe disease of spleen (disorder)
        "93680004",  # Primary malignant neoplasm of areola of female breast (disorder)
        "93681000",  # Primary malignant neoplasm of areola of male breast (disorder)
        "93745008",  # Primary malignant neoplasm of central portion of female breast (disorder)
        "93776002",  # Primary malignant neoplasm of ectopic female breast tissue (disorder)
        "93777006",  # Primary malignant neoplasm of ectopic male breast tissue (disorder)
        "93796005",  # Primary malignant neoplasm of female breast (disorder)
        "93874009",  # Primary malignant neoplasm of lower inner quadrant of female breast (disorder)
        "93876006",  # Primary malignant neoplasm of lower outer quadrant of female breast (disorder)
        "93884005",  # Primary malignant neoplasm of male breast (disorder)
        "93924008",  # Primary malignant neoplasm of nipple of female breast (disorder)
        "93925009",  # Primary malignant neoplasm of nipple of male breast (disorder)
        "94115006",  # Primary malignant neoplasm of upper inner quadrant of female breast (disorder)
        "94117003",  # Primary malignant neoplasm of upper outer quadrant of female breast (disorder)
        "94176003",  # Metastatic malignant neoplasm to areola of female breast (disorder)
        "94177007",  # Metastatic malignant neoplasm to areola of male breast (disorder)
        "94182000",  # Metastatic malignant neoplasm to axillary tail of female breast (disorder)
        "94244003",  # Metastatic malignant neoplasm to central portion of female breast (disorder)
        "94276008",  # Metastatic malignant neoplasm to ectopic female breast tissue (disorder)
        "94277004",  # Metastatic malignant neoplasm to ectopic male breast tissue (disorder)
        "94297009",  # Metastatic malignant neoplasm to female breast (disorder)
        "94385006",  # Metastatic malignant neoplasm to lower inner quadrant of female breast (disorder)
        "94387003",  # Metastatic malignant neoplasm to lower outer quadrant of female breast (disorder)
        "94401004",  # Metastatic malignant neoplasm to male breast (disorder)
        "94443006",  # Metastatic malignant neoplasm to nipple of female breast (disorder)
        "94444000",  # Metastatic malignant neoplasm to nipple of male breast (disorder)
        "94653000",  # Metastatic malignant neoplasm to upper inner quadrant of female breast (disorder)
        "94655007",  # Metastatic malignant neoplasm to upper outer quadrant of female breast (disorder)
        "94745003",  # Neoplasm of uncertain behavior of areola of female breast (disorder)
        "94746002",  # Neoplasm of uncertain behavior of areola of male breast (disorder)
        "94750009",  # Neoplasm of uncertain behavior of axillary tail of female breast (disorder)
        "94785009",  # Neoplasm of uncertain behavior of central portion of female breast (disorder)
        "94816009",  # Neoplasm of uncertain behavior of ectopic female breast tissue (disorder)
        "94817000",  # Neoplasm of uncertain behavior of ectopic male breast tissue (disorder)
        "94836005",  # Neoplasm of uncertain behavior of female breast (disorder)
        "94914006",  # Neoplasm of uncertain behavior of lower inner quadrant of female breast (disorder)
        "94916008",  # Neoplasm of uncertain behavior of lower outer quadrant of female breast (disorder)
        "94924003",  # Neoplasm of uncertain behavior of male breast (disorder)
        "94964004",  # Neoplasm of uncertain behavior of nipple of female breast (disorder)
        "94965003",  # Neoplasm of uncertain behavior of nipple of male breast (disorder)
        "95155000",  # Neoplasm of uncertain behavior of upper inner quadrant of female breast (disorder)
        "95157008",  # Neoplasm of uncertain behavior of upper outer quadrant of female breast (disorder)
        "96291000119105",  # Primary malignant inflammatory neoplasm of female breast (disorder)
        "100721000119109",  # High grade astrocytoma of brain (disorder)
        "1010603001",  # Primary mucoepidermoid carcinoma of nasopharynx (disorder)
        "1010607000",  # Primary lymphoepithelial carcinoma of nasopharynx (disorder)
        "1010624006",  # Primary salivary gland type neoplasm of nasopharynx (disorder)
        "1010631005",  # Primary malignant melanoma of overlapping sites of accessory sinuses (disorder)
        "1010633008",  # Primary papillary adenocarcinoma of nasopharynx (disorder)
        "1010656004",  # Adenoid squamous cell carcinoma of nasopharynx (disorder)
        "1010669000",  # Primary squamous cell carcinoma of superior wall of nasopharynx (disorder)
        "1010674008",  # Basaloid squamous cell carcinoma of nasopharynx (disorder)
        "1010675009",  # Squamous cell carcinoma of posterior wall of nasopharynx (disorder)
        "1010677001",  # Primary squamous cell carcinomas of anterior wall of nasopharynx (disorder)
        "1010679003",  # Primary squamous cell carcinoma of lateral wall of nasopharynx (disorder)
        "1010681001",  # Primary squamous cell carcinoma of overlapping sites of nasopharynx (disorder)
        "101371000119101",  # Adamantinoma of long bone of lower limb (disorder)
        "103511000119103",  # Melanoma carcinomatosis (disorder)
        "104981000119104",  # Oligodendroglioma of cerebrum (disorder)
        "105041000119109",  # Metastatic squamous cell carcinoma to lung (disorder)
        "105051000119106",  # Metastatic undifferentiated large cell carcinoma to lung (disorder)
        "105111000119109",  # Primary squamous cell carcinoma of thymus (disorder)
        "105121000119102",  # Squamous cell carcinoma of vagina (disorder)
        "10708511000119108",  # Primary malignant neoplasm of uterus (disorder)
        "10737361000119109",  # Primary adenocarcinoma of right fallopian tube (disorder)
        "10737401000119100",  # Primary adenocarcinoma of left fallopian tube (disorder)
        "10737521000119108",  # Primary mucinous cystadenocarcinoma of right ovary (disorder)
        "10737561000119103",  # Primary mucinous cystadenocarcinoma of left ovary (disorder)
        "10737601000119103",  # Primary serous papillary cystadenocarcinoma of right ovary (disorder)
        "10737641000119101",  # Primary serous papillary cystadenocarcinoma of left ovary (disorder)
        "10737681000119106",  # Malignant sex cord tumor of right ovary (disorder)
        "10737731000119102",  # Malignant sex cord tumor of left ovary (disorder)
        "10737781000119101",  # Primary endometrioid carcinoma of right ovary (disorder)
        "10737821000119106",  # Primary endometrioid carcinoma of left ovary (disorder)
        "10737861000119101",  # Malignant germ cell neoplasm of right ovary (disorder)
        "10737911000119105",  # Malignant germ cell neoplasm of left ovary (disorder)
        "10737961000119108",  # Undifferentiated carcinoma of right ovary (disorder)
        "10738001000119105",  # Undifferentiated carcinoma of left ovary (disorder)
        "10745291000119103",  # Malignant neoplastic disease in mother complicating childbirth (disorder)
        "10749871000119100",  # Malignant neoplastic disease in pregnancy (disorder)
        "107571000119101",  # Oligodendroglioma of brain stem (disorder)
        "107581000119103",  # Astrocytoma of brain stem (disorder)
        "107591000119100",  # Primary squamous cell carcinoma of urethra (disorder)
        "107601000119107",  # Primary transitional cell carcinoma of urethra (disorder)
        "107691000119101",  # Non-seminomatous germ cell neoplasm of testis (disorder)
        "107771000119106",  # Primary malignant clear cell neoplasm of endometrium (disorder)
        "107791000119107",  # Primary adenosquamous carcinoma of endometrium (disorder)
        "107811000119106",  # Metastatic small cell carcinoma to liver (disorder)
        "107831000119101",  # Metastatic squamous cell carcinoma to liver (disorder)
        "1078881000119102",  # Primary adenocarcinoma of lower lobe of left lung (disorder)
        "1078891000119104",  # Primary adenocarcinoma of left main bronchus (disorder)
        "1078901000119100",  # Primary adenocarcinoma of upper lobe of left lung (disorder)
        "1078931000119107",  # Primary adenocarcinoma of lower lobe of right lung (disorder)
        "1078941000119103",  # Primary adenocarcinoma of right main bronchus (disorder)
        "1078951000119101",  # Primary adenocarcinoma of middle lobe of right lung (disorder)
        "1078961000119104",  # Primary adenocarcinoma of upper lobe of right lung (disorder)
        "1078971000119105",  # Adenoid cystic carcinoma of left lacrimal gland (disorder)
        "1078981000119108",  # Adenoid cystic carcinoma of right lacrimal gland (disorder)
        "1079011000119109",  # Primary angiosarcoma of left upper limb (disorder)
        "1079021000119102",  # Primary angiosarcoma of left hip region (disorder)
        "1079031000119104",  # Primary angiosarcoma of left lower limb (disorder)
        "1079051000119105",  # Primary angiosarcoma of right upper limb (disorder)
        "1079061000119107",  # Primary angiosarcoma of right hip region (disorder)
        "1079071000119101",  # Primary angiosarcoma of right lower limb (disorder)
        "1079091000119100",  # Primary basal cell carcinoma of left upper limb (disorder)
        "1079101000119105",  # Primary basal cell carcinoma of left ear (disorder)
        "1079111000119108",  # Primary basal cell carcinoma of left eyelid (disorder)
        "1079121000119101",  # Primary basal cell carcinoma of skin of left hip (disorder)
        "1079131000119103",  # Primary basal cell carcinoma of left lower limb (disorder)
        "1079141000119107",  # Primary basal cell carcinoma of skin of left shoulder (disorder)
        "1079151000119109",  # Primary basal cell carcinoma of right upper limb (disorder)
        "1079161000119106",  # Primary basal cell carcinoma of right ear (disorder)
        "1079171000119100",  # Primary basal cell carcinoma of right eyelid (disorder)
        "1079181000119102",  # Primary basal cell carcinoma of skin of right hip (disorder)
        "1079191000119104",  # Primary basal cell carcinoma of right lower limb (disorder)
        "1079201000119101",  # Primary basal cell carcinoma of skin of right shoulder (disorder)
        "1079531000119106",  # Metastatic malignant neoplasm to left kidney (disorder)
        "1079541000119102",  # Metastatic malignant neoplasm to right kidney (disorder)
        "1079551000119100",  # Primary chondrosarcoma of bone of left upper limb (disorder)
        "1079561000119103",  # Primary chondrosarcoma of bone of left foot (disorder)
        "1079571000119109",  # Primary chondrosarcoma of bone of left hand (disorder)
        "1079581000119107",  # Primary chondrosarcoma of bone of left lower limb (disorder)
        "1079591000119105",  # Primary chondrosarcoma of left scapula (disorder)
        "1079601000119103",  # Primary chondrosarcoma of bone of right upper limb (disorder)
        "1079611000119100",  # Primary chondrosarcoma of bone of right foot (disorder)
        "1079621000119107",  # Primary chondrosarcoma of bone of right hand (disorder)
        "1079631000119105",  # Primary chondrosarcoma of bone of right lower limb (disorder)
        "1079641000119101",  # Primary chondrosarcoma of right scapula (disorder)
        "1079991000119101",  # Primary Ewing sarcoma of bone of left upper limb (disorder)
        "1080001000119106",  # Primary Ewing sarcoma of bone of left foot (disorder)
        "1080021000119102",  # Primary Ewing sarcoma of bone of left lower limb (disorder)
        "1080041000119108",  # Primary Ewing sarcoma of bone of right upper limb (disorder)
        "1080051000119105",  # Primary Ewing sarcoma of bone of right foot (disorder)
        "1080071000119101",  # Primary Ewing sarcoma of bone of right lower limb (disorder)
        "1080451000119101",  # Primary large cell carcinoma of lower lobe of left lung (disorder)
        "1080471000119105",  # Primary large cell carcinoma of upper lobe of left lung (disorder)
        "1080501000119104",  # Primary large cell carcinoma of lower lobe of right lung (disorder)
        "1080531000119106",  # Primary large cell carcinoma of upper lobe of right lung (disorder)
        "1080541000119102",  # Primary leiomyosarcoma of left upper limb (disorder)
        "1080551000119100",  # Primary leiomyosarcoma of left hip region (disorder)
        "1080561000119103",  # Primary leiomyosarcoma of left kidney (disorder)
        "1080571000119109",  # Primary leiomyosarcoma of left lower limb (disorder)
        "1080581000119107",  # Primary leiomyosarcoma of left shoulder region (disorder)
        "1080591000119105",  # Primary leiomyosarcoma of right upper limb (disorder)
        "1080601000119103",  # Primary leiomyosarcoma of right hip region (disorder)
        "1080611000119100",  # Primary leiomyosarcoma of right kidney (disorder)
        "1080621000119107",  # Primary leiomyosarcoma of right lower limb (disorder)
        "1080631000119105",  # Primary leiomyosarcoma of right shoulder region (disorder)
        "1080681000119106",  # Primary liposarcoma of soft tissue of left upper limb (disorder)
        "1080691000119109",  # Primary liposarcoma of soft tissue of left hip (disorder)
        "1080701000119109",  # Primary liposarcoma of soft tissue of left lower limb (disorder)
        "1080711000119107",  # Primary liposarcoma of soft tissue of left shoulder (disorder)
        "1080721000119100",  # Primary liposarcoma of soft tissue of right upper limb (disorder)
        "1080731000119102",  # Primary liposarcoma of soft tissue of right hip (disorder)
        "1080741000119106",  # Primary liposarcoma of soft tissue of right lower limb (disorder)
        "1080751000119108",  # Primary liposarcoma of soft tissue of right shoulder (disorder)
        "1080941000119109",  # Malignant melanoma of left choroid (disorder)
        "1080981000119104",  # Malignant melanoma of right choroid (disorder)
        "108101000119101",  # Malignant melanoma metastatic to kidney (disorder)
        "1081021000119109",  # Malignant melanoma of skin of left lower limb (disorder)
        "1081271000119100",  # Primary osteosarcoma of bone of left upper limb (disorder)
        "1081281000119102",  # Primary osteosarcoma of bone of left foot (disorder)
        "1081291000119104",  # Primary osteosarcoma of bone of left hand (disorder)
        "1081301000119103",  # Primary osteosarcoma of bone of left lower limb (disorder)
        "108131000119108",  # Small cell carcinoma metastatic to kidney (disorder)
        "1081311000119100",  # Primary osteosarcoma of left scapula (disorder)
        "1081321000119107",  # Primary osteosarcoma of bone of right upper limb (disorder)
        "1081331000119105",  # Primary osteosarcoma of bone of right foot (disorder)
        "1081341000119101",  # Primary osteosarcoma of bone of right hand (disorder)
        "1081351000119104",  # Primary osteosarcoma of bone of right lower limb (disorder)
        "1081361000119102",  # Primary osteosarcoma of right scapula (disorder)
        "1081571000119102",  # Clear cell carcinoma of left kidney (disorder)
        "1081581000119104",  # Clear cell carcinoma of right kidney (disorder)
        "1081591000119101",  # Primary retinoblastoma of left retina (disorder)
        "1081601000119108",  # Primary retinoblastoma of right retina (disorder)
        "1081611000119106",  # Primary rhabdomyosarcoma of left upper limb (disorder)
        "1081631000119101",  # Primary rhabdomyosarcoma of left lower limb (disorder)
        "1081651000119107",  # Primary rhabdomyosarcoma of right upper limb (disorder)
        "1081671000119103",  # Primary rhabdomyosarcoma of right lower limb (disorder)
        "1081691000119102",  # Primary sarcoma of soft tissues of left upper limb (disorder)
        "1081701000119102",  # Primary sarcoma of soft tissues of left hip (disorder)
        "1081711000119104",  # Primary sarcoma of left lower limb (disorder)
        "1081721000119106",  # Primary sarcoma of soft tissues of left shoulder (disorder)
        "1081731000119109",  # Primary sarcoma of soft tissues of right upper limb (disorder)
        "1081741000119100",  # Primary sarcoma of soft tissues of right hip (disorder)
        "1081751000119103",  # Primary sarcoma of right lower limb (disorder)
        "1081761000119101",  # Primary sarcoma of soft tissues of right shoulder (disorder)
        "1081781000119105",  # Primary seminoma of left testis (disorder)
        "1081811000119107",  # Primary seminoma of right testis (disorder)
        "1081841000119106",  # Primary small cell carcinoma of lower lobe of left lung (disorder)
        "1081851000119108",  # Primary small cell carcinoma of left main bronchus (disorder)
        "1081861000119105",  # Primary small cell carcinoma of upper lobe of left lung (disorder)
        "1081891000119103",  # Primary small cell carcinoma of lower lobe of right lung (disorder)
        "1081901000119104",  # Primary small cell carcinoma of right main bronchus (disorder)
        "1081911000119101",  # Primary small cell carcinoma of middle lobe of right lung (disorder)
        "1081921000119108",  # Primary small cell carcinoma of upper lobe of right lung (disorder)
        "108201000119105",  # Undifferentiated large cell carcinoma metastatic to kidney (disorder)
        "1082051000119106",  # Primary squamous cell carcinoma of skin of left upper limb (disorder)
        "1082061000119108",  # Primary squamous cell carcinoma of conjunctiva of left eye (disorder)
        "1082071000119102",  # Primary squamous cell carcinoma of left ear (disorder)
        "1082091000119101",  # Primary squamous cell carcinoma of skin of left hip (disorder)
        "1082101000119106",  # Primary squamous cell carcinoma of skin of left lower limb (disorder)
        "108211000119108",  # Secondary adenocarcinoma of kidney (disorder)
        "1082111000119109",  # Primary squamous cell carcinoma of lower lobe of left lung (disorder)
        "1082121000119102",  # Primary squamous cell carcinoma of left main bronchus (disorder)
        "1082131000119104",  # Primary squamous cell carcinoma of skin of left shoulder (disorder)
        "1082141000119108",  # Primary squamous cell carcinoma of upper lobe of left lung (disorder)
        "1082171000119101",  # Primary squamous cell carcinoma of skin of right upper limb (disorder)
        "1082181000119103",  # Primary squamous cell carcinoma of conjunctiva of right eye (disorder)
        "1082191000119100",  # Primary squamous cell carcinoma of right ear (disorder)
        "1082211000119104",  # Primary squamous cell carcinoma of skin of right hip (disorder)
        "1082221000119106",  # Primary squamous cell carcinoma of skin of right lower limb (disorder)
        "1082231000119109",  # Primary squamous cell carcinoma of lower lobe of right lung (disorder)
        "1082241000119100",  # Primary squamous cell carcinoma of right main bronchus (disorder)
        "1082251000119103",  # Primary squamous cell carcinoma of middle lobe of right lung (disorder)
        "1082261000119101",  # Primary squamous cell carcinoma of skin of right shoulder (disorder)
        "1082271000119107",  # Primary squamous cell carcinoma of upper lobe of right lung (disorder)
        "1082281000119105",  # Transitional cell carcinoma of left renal pelvis (disorder)
        "1082291000119108",  # Transitional cell carcinoma of left ureter (disorder)
        "1082301000119109",  # Transitional cell carcinoma of right renal pelvis (disorder)
        "1082311000119107",  # Transitional cell carcinoma of right ureter (disorder)
        "1082321000119100",  # Nephroblastoma of left kidney (disorder)
        "1082331000119102",  # Nephroblastoma of right kidney (disorder)
        "1082601000112109",  # Metastatic malignant neoplasm to viscera (disorder)
        "1082701000112100",  # Locally advanced breast cancer (disorder)
        "109264009",  # Overlapping malignant neoplasm of skin (disorder)
        "109267002",  # Overlapping malignant melanoma of skin (disorder)
        "109347009",  # Overlapping malignant neoplasm of bone and articular cartilage (disorder)
        "109348004",  # Overlapping malignant neoplasm of bone and articular cartilage of limb (disorder)
        "109349007",  # Overlapping malignant neoplasm of soft tissues (disorder)
        "109359008",  # Primary malignant neoplasm of independent multiple sites (disorder)
        "109366009",  # Overlapping malignant neoplasm of accessory sinuses (disorder)
        "109367000",  # Overlapping malignant neoplasm of nasopharynx (disorder)
        "109368005",  # Overlapping malignant neoplasm of hypopharynx (disorder)
        "109369002",  # Overlapping malignant neoplasm of larynx (disorder)
        "109370001",  # Primary malignant neoplasm of laryngeal cartilage (disorder)
        "109371002",  # Overlapping malignant neoplasm of bronchus and lung (disorder)
        "109374005",  # Overlapping malignant neoplasm of mediastinum and pleura (disorder)
        "109378008",  # Mesothelioma (malignant, clinical disorder) (disorder)
        "109383000",  # Malignant mesothelioma of pericardium (disorder)
        "109384006",  # Overlapping malignant neoplasm of heart, mediastinum and pleura (disorder)
        "109385007",  # Kaposi's sarcoma (disorder)
        "109386008",  # Kaposi's sarcoma of skin (disorder)
        "109388009",  # Kaposi's sarcoma of palate (disorder)
        "109389001",  # Kaposi's sarcoma of gastrointestinal tract (disorder)
        "109390005",  # Kaposi's sarcoma of lung (disorder)
        "109391009",  # Kaposi's sarcoma of lymph nodes (disorder)
        "109392002",  # Kaposi's sarcoma of multiple organs (disorder)
        "10976631000119101",  # Leiomyosarcoma of skin of chest (disorder)
        "10976711000119104",  # Primary primitive neuroectodermal neoplasm of central nervous system (disorder)
        "109821008",  # Overlapping malignant neoplasm of gastrointestinal tract (disorder)
        "109822001",  # Overlapping malignant neoplasm of lip (disorder)
        "109823006",  # Overlapping malignant neoplasm of tongue (disorder)
        "109824000",  # Overlapping malignant neoplasm of major salivary gland (disorder)
        "109826003",  # Neoplasm of uncertain behavior of salivary gland duct (disorder)
        "109828002",  # Primary malignant neoplasm of salivary gland duct (disorder)
        "109830000",  # Overlapping malignant neoplasm of floor of mouth (disorder)
        "109831001",  # Overlapping malignant neoplasm of palate (disorder)
        "109832008",  # Overlapping malignant neoplasm of oropharynx (disorder)
        "109833003",  # Overlapping malignant neoplasm of lip, oral cavity and/or pharynx (disorder)
        "109834009",  # Primary malignant neoplasm of branchial cleft (disorder)
        "109835005",  # Overlapping malignant neoplasm of esophagus (disorder)
        "109836006",  # Overlapping malignant neoplasm of stomach (disorder)
        "109837002",  # Overlapping malignant neoplasm of small intestine (disorder)
        "109838007",  # Overlapping malignant neoplasm of colon (disorder)
        "109839004",  # Overlapping malignant neoplasm of rectum, anus and anal canal (disorder)
        "109840002",  # Primary malignant neoplasm of cloacogenic zone (disorder)
        "109841003",  # Liver cell carcinoma (disorder)
        "109842005",  # Intrahepatic bile duct carcinoma (disorder)
        "109843000",  # Hepatoblastoma (disorder)
        "109844006",  # Angiosarcoma of liver (disorder)
        "109847004",  # Overlapping malignant neoplasm of biliary tract (disorder)
        "109848009",  # Overlapping malignant neoplasm of pancreas (disorder)
        "109851002",  # Overlapping malignant neoplasm of retroperitoneum and peritoneum (disorder)
        "109853004",  # Malignant mesothelioma of peritoneum (disorder)
        "109854005",  # Malignant mesothelioma of parietal peritoneum (disorder)
        "109855006",  # Malignant mesothelioma of pelvic peritoneum (disorder)
        "109856007",  # Malignant mesothelioma of mesentery (disorder)
        "109857003",  # Mesothelioma of mesocolon (disorder)
        "109858008",  # Malignant mesothelioma of omentum (disorder)
        "109870007",  # Overlapping malignant neoplasm of urinary system (disorder)
        "109874003",  # Overlapping malignant neoplasm of male genital organs (disorder)
        "109875002",  # Overlapping malignant neoplasm of penis (disorder)
        "109876001",  # Primary malignant neoplasm of descended testis (disorder)
        "109878000",  # Overlapping malignant neoplasm of female genital organs (disorder)
        "109879008",  # Overlapping malignant neoplasm of body of uterus (disorder)
        "109880006",  # Overlapping malignant neoplasm of uterine cervix (disorder)
        "109882003",  # Primary malignant neoplasm of fundus uteri (disorder)
        "109885001",  # Overlapping malignant neoplasm of vulva (disorder)
        "1098981000119101",  # Recurrent malignant neoplasm of prostate (disorder)
        "109911004",  # Overlapping malignant neoplasm of brain and other parts of the central nervous system (disorder)
        "109912006",  # Overlapping malignant neoplasm of brain (disorder)
        "109914007",  # Neoplasm of uncertain behavior of meninges (disorder)
        "109915008",  # Primary malignant neoplasm of meninges (disorder)
        "109917000",  # Neoplasm of uncertain behavior of peripheral nerves and peripheral autonomic nervous system (disorder)
        "109918005",  # Primary malignant neoplasm of peripheral nerves and peripheral autonomic nervous system (disorder)
        "109919002",  # Overlapping malignant neoplasm of peripheral nerves and autonomic nervous system (disorder)
        "109921007",  # Primary malignant neoplasm of peripheral nerve of head, face and/or neck (disorder)
        "109923005",  # Primary malignant neoplasm of peripheral nerves of head (disorder)
        "109925003",  # Primary malignant neoplasm of peripheral nerves of face (disorder)
        "109927006",  # Primary malignant neoplasm of peripheral nerves of neck (disorder)
        "109931000",  # Primary malignant neoplasm of peripheral nerves of upper limb (disorder)
        "109933002",  # Primary malignant neoplasm of peripheral nerves of shoulder (disorder)
        "109937001",  # Primary malignant neoplasm of peripheral nerves of lower limb (disorder)
        "109939003",  # Primary malignant neoplasm of peripheral nerves of hip (disorder)
        "109941002",  # Primary malignant neoplasm of peripheral nerves of thorax (disorder)
        "109943004",  # Primary malignant neoplasm of peripheral nerves of abdomen (disorder)
        "109945006",  # Primary malignant neoplasm of peripheral nerves of pelvis region (disorder)
        "109947003",  # Primary malignant neoplasm of peripheral nerves of trunk (disorder)
        "109948008",  # Overlapping malignant neoplasm of eye and adnexa, primary (disorder)
        "109951001",  # Overlapping malignant neoplasm of multiple endocrine glands (disorder)
        "1099571000119100",  # Primary malignant neoplasm of left lacrimal sac (disorder)
        "109958007",  # Hematologic neoplasm of uncertain behavior (disorder)
        "1099581000119102",  # Primary malignant neoplasm of right lacrimal sac (disorder)
        "109962001",  # Diffuse non-Hodgkin's lymphoma (disorder)
        "109964000",  # Diffuse non-Hodgkin's lymphoma, undifferentiated (disorder)
        "109966003",  # Diffuse non-Hodgkin's lymphoma, immunoblastic (disorder)
        "109967007",  # Diffuse non-Hodgkin's lymphoma, small cleaved cell (disorder)
        "109968002",  # Diffuse non-Hodgkin's lymphoma, small cell (disorder)
        "109969005",  # Diffuse large B-cell malignant lymphoma (disorder)
        "109970006",  # Follicular non-Hodgkin's lymphoma, small cleaved cell (disorder)
        "109971005",  # Follicular non-Hodgkin's lymphoma, mixed small cleaved cell and large cell (disorder)
        "109972003",  # Follicular non-Hodgkin's lymphoma, large cell (disorder)
        "109975001",  # T-zone lymphoma (disorder)
        "109976000",  # Lymphoepithelioid lymphoma (disorder)
        "109977009",  # Peripheral T-cell lymphoma (disorder)
        "109978004",  # T-cell lymphoma (disorder)
        "109979007",  # B-cell lymphoma (disorder)
        "109980005",  # Malignant immunoproliferative disease (disorder)
        "109982002",  # Alpha heavy chain disease (disorder)
        "109984001",  # Gamma heavy chain disease (disorder)
        "109985000",  # Immunoproliferative small intestinal disease (disorder)
        "109988003",  # Histiocytic sarcoma (disorder)
        "109989006",  # Multiple myeloma (disorder)
        "109991003",  # Acute panmyelosis with myelofibrosis (disorder)
        "109992005",  # Polycythemia vera (disorder)
        "109993000",  # Chronic myeloproliferative disorder (clinical) (disorder)
        "109994006",  # Essential thrombocythemia (disorder)
        "109995007",  # Myelodysplastic syndrome (disorder)
        "109998009",  # Myelodysplastic syndrome with ring sideroblasts and single lineage dysplasia (disorder)
        "110002002",  # Mast cell leukemia (disorder)
        "110004001",  # Acute promyelocytic leukemia, FAB M3 (disorder)
        "110005000",  # Acute myelomonocytic leukemia, FAB M4 (disorder)
        "110006004",  # Prolymphocytic leukemia (disorder)
        "110007008",  # Adult T-cell leukemia/lymphoma (disorder)
        "110013004",  # Overlapping malignant neoplasm of tonsil (disorder)
        "11010461000119101",  # Small cell carcinoma (disorder)
        "112691000112107",  # Primary malignant atypical teratoid rhabdoid neoplasm of central nervous system (disorder)
        "1137644008",  # Leiomyosarcoma of fundus of stomach (disorder)
        "1141621000",  # Malignant epithelial neoplasm of pyriform fossa (disorder)
        "1141626005",  # Adenocarcinoma of pancreas with neuregulin 1 gene fusion (disorder)
        "1141627001",  # Non-small cell lung carcinoma with neuregulin 1 gene fusion (disorder)
        "1144252005",  # Malignant neoplasm of lateral border of tongue (disorder)
        "1144303008",  # Leiomyosarcoma of pylorus of stomach (disorder)
        "1144304002",  # Leiomyosarcoma of pyloric antrum of stomach (disorder)
        "1144305001",  # Leiomyosarcoma of gastric corpus structure (disorder)
        "1144306000",  # Leiomyosarcoma of lesser curvature of stomach (disorder)
        "1144307009",  # Leiomyosarcoma of greater curvature of stomach (disorder)
        "1144764001",  # Malignant neoplasm of esophagus with neuregulin 1 gene fusion (disorder)
        "1144891007",  # Primary cholangiocarcinoma of extrahepatic bile duct (disorder)
        "1144922006",  # Thymoma type AB (disorder)
        "1144924007",  # Thymoma type B (disorder)
        "1144928005",  # Thymoma type A (disorder)
        "1144945000",  # Adnexal carcinoma of vulva (disorder)
        "1145245004",  # Carcinoma of meibomian gland (disorder)
        "11471000224106",  # Diffuse intrinsic pontine glioma (disorder)
        "1148551007",  # Infiltrating duct carcinoma of pancreas (disorder)
        "1148845007",  # Follicular lymphoma grade 3b (disorder)
        "1148851002",  # Follicular lymphoma grade 3a (disorder)
        "1148852009",  # Malignant cystic nephroma (disorder)
        "1148901006",  # Squamous cell carcinoma of male genital (disorder)
        "1148904003",  # Spermatocytic seminoma of testis (disorder)
        "1148906001",  # Acute myeloid leukemia with t(8;21)(q22;q22) RUNX1-RUNX1T1 (disorder)
        "1148931000",  # Cystic renal cell carcinoma of kidney (disorder)
        "1153336009",  # Pediatric nodal marginal zone B cell lymphoma (disorder)
        "1153341001",  # Myeloproliferative hypereosinophilic syndrome (disorder)
        "1153346006",  # Primary non-Hodgkin malignant lymphoma of uveal tract (disorder)
        "1153347002",  # Primary non-Hodgkin malignant lymphoma of vitreoretinal tract (disorder)
        "1153348007",  # Primary effusion lymphoma due to human immune deficiency virus infection (disorder)
        "1153354008",  # Malignant lymphoma of duodenum (disorder)
        "1153355009",  # Malignant lymphoma of esophagus (disorder)
        "1153356005",  # Lymphoma of anus (disorder)
        "1153357001",  # Lymphoma of appendix (disorder)
        "1153377008",  # Mixed phenotype acute leukemia with t(9;22) (q34;q11.2); BCR-ABL1 (disorder)
        "1153379006",  # Mixed phenotype acute leukemia with T-cell and myeloid lineage (disorder)
        "1153381008",  # Mixed phenotype acute leukemia with myeloid and B-cell lymphoid phenotypes (disorder)
        "1153382001",  # Malignant lymphoma of lacrimal gland (disorder)
        "1153383006",  # Malignant lymphoma of liver (disorder)
        "1153421007",  # Chondrosarcoma of sternum (disorder)
        "1153422000",  # Chondrosarcoma of mandible (disorder)
        "1153423005",  # Chondrosarcoma of rib (disorder)
        "1153424004",  # Chondrosarcoma of bone of pelvic wall (disorder)
        "1153425003",  # Chondrosarcoma of clavicle (disorder)
        "1153426002",  # Chondrosarcoma of skull (disorder)
        "1153427006",  # Chondrosarcoma of vertebral column (disorder)
        "1155991005",  # Metastatic malignant neoplasm to leptomeninges (disorder)
        "1156403002",  # Composite Hodgkin and non-Hodgkin lymphoma (disorder)
        "1156404008",  # Polyembryoma (disorder)
        "1156406005",  # Anaplastic oligodendroglioma of central nervous system (disorder)
        "1156409003",  # Anaplastic ependymoma of central nervous system (disorder)
        "1156410008",  # Anaplastic oligoastrocytoma of central nervous system (disorder)
        "1156413005",  # Gliomatosis cerebri (disorder)
        "1156414004",  # Giant cell glioblastoma of brain (disorder)
        "1156415003",  # Protoplasmic astrocytoma of brain (disorder)
        "1156416002",  # Gemistocytic astrocytoma of brain (disorder)
        "1156417006",  # Fibrillary astrocytoma of brain (disorder)
        "1156418001",  # Malignant rhabdoid tumor (disorder)
        "1156420003",  # Desmoplastic small round cell tumor (disorder)
        "1156452003",  # Mixed germ cell tumor (disorder)
        "1156453008",  # Chordoma (disorder)
        "1156454002",  # Embryonal carcinoma (disorder)
        "1156455001",  # Pleomorphic xanthoastrocytoma of brain (disorder)
        "1156456000",  # Pilomyxoid astrocytoma of brain (disorder)
        "1156459007",  # Anaplastic ganglioglioma of central nervous system (disorder)
        "1156460002",  # Desmoplastic nodular medulloblastoma of brain (disorder)
        "1156461003",  # Gliosarcoma of brain (disorder)
        "1156469001",  # Large cell medulloblastoma of brain (disorder)
        "1156471001",  # Choroid plexus carcinoma (disorder)
        "1156472008",  # Papillary tumor of pineal region (disorder)
        "1156495004",  # Dendritic cell sarcoma (disorder)
        "1156783003",  # Leiomyosarcoma of rectum (disorder)
        "1156785005",  # Leiomyosarcoma of duodenum (disorder)
        "1156786006",  # Leiomyosarcoma of esophagus (disorder)
        "1156788007",  # Leiomyosarcoma of colon (disorder)
        "1156795003",  # Kaposi sarcoma of colon (disorder)
        "1156797006",  # Kaposi sarcoma of rectum (disorder)
        "1156804004",  # Peripheral neuroectodermal neoplasm of corpus uteri (disorder)
        "1156806002",  # Primary malignant neoplasm of musculoskeletal system (disorder)
        "1156807006",  # Peripheral neuroectodermal neoplasm of cervix uteri (disorder)
        "1156809009",  # Malignant mesenchymal neoplasm of duodenum (disorder)
        "1156810004",  # Malignant mesenchymal neoplasm of esophagus (disorder)
        "1156811000",  # Malignant mesenchymal neoplasm of anus (disorder)
        "1156812007",  # Malignant mesenchymal neoplasm of appendix (disorder)
        "1156833003",  # Germinoma of central nervous system (disorder)
        "1157060001",  # Diffuse astrocytoma of brain (disorder)
        "1157061002",  # Gliosarcoma of central nervous system (disorder)
        "1157062009",  # Giant cell glioblastoma of central nervous system (disorder)
        "1157063004",  # Malignant neoplasm of vertebral column region (disorder)
        "1157064005",  # Gemistocytic astrocytoma of central nervous system (disorder)
        "1157067003",  # Malignant neoplasm of vertebra (disorder)
        "1157068008",  # Fibrillary astrocytoma of central nervous system (disorder)
        "1157070004",  # Pilomyxoid astrocytoma (disorder)
        "1157071000",  # Anaplastic ganglioglioma (disorder)
        "1157072007",  # Gliosarcoma of spinal cord (disorder)
        "1157141006",  # Astroblastoma of central nervous system (disorder)
        "1157157006",  # Acute myeloid leukemia with 11q23 abnormality (disorder)
        "1157162007",  # Intravascular large B-cell lymphoma (disorder)
        "1162264008",  # Malignant lymphoma of uveal tract (disorder)
        "1162280003",  # Primary rhabdomyosarcoma of isthmus of uterine body (disorder)
        "1162282006",  # Primary rhabdomyosarcoma of myometrium of corpus uteri (disorder)
        "1162284007",  # Primary rhabdomyosarcoma of endometrium of corpus uteri (disorder)
        "1162286009",  # Primary rhabdomyosarcoma of fundus uteri (disorder)
        "1162311007",  # Sarcoma of body of uterus (disorder)
        "1162346004",  # Osteosarcoma of sternum (disorder)
        "1162348003",  # Osteosarcoma of bone of vertebra (disorder)
        "1162349006",  # Osteosarcoma of bone of clavicle (disorder)
        "1162351005",  # Osteosarcoma of bone of rib (disorder)
        "1162352003",  # Osteosarcoma of articular cartilage of sternum (disorder)
        "1162353008",  # Osteosarcoma of articular cartilage of vertebra (disorder)
        "1162354002",  # Osteosarcoma of articular cartilage of rib (disorder)
        "1162361003",  # Osteosarcoma of articular cartilage of clavicle (disorder)
        "1162855005",  # Carcinoma of gallbladder and extrahepatic biliary tract (disorder)
        "1163357003",  # Malignant neoplasm of oral cavity and lip and salivary gland (disorder)
        "1163568002",  # Primary malignant neoplasm of overlapping sites of colon (disorder)
        "116381000119105",  # Ganglioneuroblastoma (disorder)
        "116711000119103",  # Marginal zone lymphoma of inguinal lymph node (disorder)
        "116741000119104",  # Marginal zone lymphoma of thoracic lymph node (disorder)
        "116811000119106",  # Non-Hodgkin lymphoma of central nervous system metastatic to lymph node of lower limb (disorder)
        "116821000119104",  # Non-Hodgkin lymphoma of central nervous system metastatic to lymph node of upper limb (disorder)
        "116841000119105",  # Marginal zone lymphoma of lymph nodes of multiple sites (disorder)
        "116871000119103",  # Mantle cell lymphoma of lymph nodes of multiple sites (disorder)
        "117061000119101",  # Peripheral T-cell lymphoma of spleen (disorder)
        "117091000119108",  # Peripheral T-cell lymphoma of axillary lymph node (disorder)
        "117111000119100",  # Peripheral T-cell lymphoma of thoracic lymph node (disorder)
        "11717361000119108",  # Metastatic malignant neoplasm to soft tissue of back (disorder)
        "117211000119105",  # Peripheral T-cell lymphoma of lymph nodes of multiple sites (disorder)
        "1172704005",  # High grade B-cell lymphoma with MYC and BCL2 and/or BCL6 rearrangements (disorder)
        "1172793008",  # Malignant peripheral nerve sheath tumor of gingival mucosa (disorder)
        "1179375008",  # Adenocarcinoma of parathyroid gland (disorder)
        "1179377000",  # Adenocarcinoma of pituitary gland (disorder)
        "1179389003",  # Adenocarcinoma of paraganglia (disorder)
        "1179390007",  # Adenocarcinoma of carotid body (disorder)
        "1179762006",  # Malignant neoplasm of middle lobe of right lung (disorder)
        "118599009",  # Hodgkin's disease (disorder)
        "118600007",  # Malignant lymphoma (disorder)
        "118601006",  # Non-Hodgkin's lymphoma (disorder)
        "118602004",  # Hodgkin's granuloma (disorder)
        "118605002",  # Hodgkin lymphoma, nodular lymphocyte predominance (disorder)
        "118606001",  # Hodgkin's sarcoma (disorder)
        "118607005",  # Hodgkin lymphoma, lymphocyte-rich (disorder)
        "118608000",  # Hodgkin's disease, nodular sclerosis (disorder)
        "118609008",  # Hodgkin's disease, mixed cellularity (disorder)
        "118610003",  # Hodgkin's disease, lymphocytic depletion (disorder)
        "118611004",  # Sezary's disease (disorder)
        "118612006",  # Malignant histiocytosis (disorder)
        "118613001",  # Hairy cell leukemia (disorder)
        "118614007",  # Langerhans cell histiocytosis, disseminated (disorder)
        "118615008",  # Malignant mast cell tumor (disorder)
        "118616009",  # Neoplastic disease of uncertain behavior (disorder)
        "118617000",  # Burkitt's lymphoma (disorder)
        "118618005",  # Mycosis fungoides (disorder)
        "1186612004",  # Metastatic hepatocellular carcinoma (disorder)
        "1186614003",  # Primary Kaposi sarcoma of gastrointestinal tract (disorder)
        "1186615002",  # Primary Kaposi sarcoma of palate (disorder)
        "1186616001",  # Primary Kaposi sarcoma of skin (disorder)
        "1186629001",  # Primary fibrolamellar hepatocellular carcinoma (disorder)
        "1186630006",  # Primary liver cell carcinoma (disorder)
        "1186631005",  # Primary combined hepatocellular carcinoma and cholangiocarcinoma (disorder)
        "1186735007",  # Mixed neuroendocrine-non neuroendocrine neoplasm of pancreas (disorder)
        "1186869002",  # Neuroendocrine carcinoma of pancreas (disorder)
        "1187123005",  # Mixed phenotype acute leukemia (disorder)
        "1187285005",  # Metastatic clear cell renal cell carcinoma (disorder)
        "1187464007",  # Clear cell sarcoma of kidney (disorder)
        "1187519007",  # Malignant gastrinoma of pancreas (disorder)
        "1187521002",  # Malignant glucagonoma of pancreas (disorder)
        "1187622004",  # Nuclear protein in testis carcinoma (disorder)
        "1196839006",  # Primary embryonal carcinoma of brain (disorder)
        "1196841007",  # Primary desmoplastic mesothelioma of pleura (disorder)
        "1196844004",  # Primary malignant astrocytoma of brain (disorder)
        "1196846002",  # Primary cystadenocarcinoma of intrahepatic bile duct (disorder)
        "1196853006",  # Primary carcinoma of bile duct (disorder)
        "1196854000",  # Primary carcinosarcoma of liver (disorder)
        "1196856003",  # Primary biphasic malignant mesothelioma of pleura (disorder)
        "1196860000",  # Primary anaplastic large cell medulloblastoma of brain (disorder)
        "1196874007",  # Primary adenocarcinoma of bile duct (disorder)
        "1196876009",  # Primary acinar cell carcinoma of parotid gland (disorder)
        "1196879002",  # Familial cancer of ovary (disorder)
        "1196881000",  # Primary embryonal carcinoma of liver (disorder)
        "1196883002",  # Primary epithelioid malignant mesothelioma of pleura (disorder)
        "1196898000",  # Primary malignant atypical teratoid rhabdoid neoplasm of brain (disorder)
        "1196900003",  # B cell lymphoma of small intestine (disorder)
        "1196911006",  # Primary adenocarcinoma of iris neuroepithelium (disorder)
        "1196998001",  # Malignant perivascular epithelioid cell neoplasm (disorder)
        "1197011004",  # Malignant neoplasm of intestine due to familial adenomatous polyposis (disorder)
        "1197204009",  # Adult hepatocellular carcinoma (disorder)
        "1197258000",  # Primary salivary gland type carcinoma of palate (disorder)
        "1197259008",  # Primary sarcomatoid malignant mesothelioma of pleura (disorder)
        "1197260003",  # Primary poorly differentiated endocrine carcinoma of body of uterus (disorder)
        "1197263001",  # Primary poorly differentiated endocrine carcinoma of cervix uteri (disorder)
        "1197266009",  # Primary papillary carcinoma of body of uterus (disorder)
        "1197268005",  # Primary papillary carcinoma of cervix uteri (disorder)
        "1197275006",  # Primary squamous cell carcinoma of fallopian tube (disorder)
        "1197276007",  # Primary squamous cell carcinoma of pharyngeal lymphoid ring (disorder)
        "1197280002",  # Yolk sac tumor of central nervous system (disorder)
        "1197283000",  # Primary mixed acinar endocrine-ductal carcinoma of pancreas (disorder)
        "1197285007",  # Metastatic squamous cell carcinoma to thoracic lymph node (disorder)
        "1197286008",  # Primary mixed acinar ductal carcinoma of pancreas (disorder)
        "1197292002",  # Metastatic squamous cell carcinoma of pelvic lymph node (disorder)
        "1197293007",  # Metastatic squamous cell carcinoma to lymph nodes of head and neck (disorder)
        "1197294001",  # Malignant melanoma arising in melanocytic nevus (disorder)
        "1197295000",  # Metastatic squamous cell carcinoma to lymph nodes of multiple sites (disorder)
        "1197296004",  # Primary malignant germ cell neoplasm of pleura (disorder)
        "1197297008",  # Metastatic squamous cell carcinoma to lymph node of lower limb (disorder)
        "1197298003",  # Primary malignant germ cell neoplasm of pineal gland (disorder)
        "1197299006",  # Metastatic squamous cell carcinoma to lymph node of upper limb (disorder)
        "1197307000",  # Primary neuroendocrine carcinoma of gallbladder (disorder)
        "1197310007",  # Metastatic adenocarcinoma to lymph node of lower limb (disorder)
        "1197311006",  # Metastatic adenocarcinoma to lymph node of upper limb (disorder)
        "1197312004",  # Metastatic adenocarcinoma to abdominal lymph node (disorder)
        "1197313009",  # Metastatic adenocarcinoma to lymph node of head and/or neck (disorder)
        "1197322005",  # Metastatic squamous cell carcinoma to abdominal lymph node (disorder)
        "1197323000",  # Metastatic carcinoma to lymph node of thorax (disorder)
        "1197324006",  # Metastatic malignant melanoma to skin (disorder)
        "1197325007",  # Metastatic carcinoma of lymph node of pelvis (disorder)
        "1197326008",  # Metastatic carcinoma to lymph node of head and neck (disorder)
        "1197327004",  # Metastatic carcinoma to lymph node of lower limb (disorder)
        "1197328009",  # Metastatic adenocarcinoma to thoracic lymph node (disorder)
        "1197329001",  # Metastatic carcinoma of abdominal lymph node (disorder)
        "1197330006",  # Metastatic adenocarcinoma to lymph nodes of multiple sites (disorder)
        "1197331005",  # Metastatic adenocarcinoma to pelvic lymph node (disorder)
        "1197332003",  # Primary malignant melanoma of central nervous system (disorder)
        "1197334002",  # Malignant melanoma of uveal tract (disorder)
        "1197342001",  # Primary teratocarcinoma of liver (disorder)
        "1197344000",  # T-cell lymphoma of small intestine (disorder)
        "1197349005",  # Primary malignant paraganglioma of carotid body (disorder)
        "1197354001",  # Primary malignant mesenchymal neoplasm of colon (disorder)
        "1197355000",  # Primary malignant mesenchymal neoplasm of rectum (disorder)
        "1197359006",  # Familial colorectal cancer type X (disorder)
        "1201780008",  # Chondrosarcoma of gingiva (disorder)
        "1208349005",  # Pediatric hepatocellular carcinoma (disorder)
        "1208451008",  # Transitional cell carcinoma of upper urinary tract (disorder)
        "1208457007",  # Small cell neuroendocrine carcinoma of prostate (disorder)
        "1208459005",  # Micropapillary urothelial carcinoma (disorder)
        "1208460000",  # Sarcomatoid urothelial carcinoma of urinary bladder (disorder)
        "1208461001",  # Plasmacytoid urothelial carcinoma of urinary bladder (disorder)
        "1208483002",  # Primary choriocarcinoma of central nervous system (disorder)
        "1208741006",  # SMARCA4-deficient undifferentiated neoplasm of thorax (disorder)
        "1208744003",  # Non-functioning neuroendocrine neoplasm of pancreas (disorder)
        "1208745002",  # Serotonin-producing neuroendocrine neoplasm of pancreas (disorder)
        "1217020002",  # Sarcomatoid urothelial carcinoma of ureter (disorder)
        "1217301006",  # Precursor cell lymphoblastic lymphoma (disorder)
        "1217490003",  # Kaposi sarcoma of gingiva (disorder)
        "1217617009",  # Kaposi sarcoma of internal part of mouth (disorder)
        "1222581000",  # Lipid-rich urothelial carcinoma of urinary system (disorder)
        "12235561000119105",  # Large cell carcinoma of left lung (disorder)
        "12235601000119105",  # Large cell carcinoma of right lung (disorder)
        "12240951000119107",  # Squamous cell carcinoma of left lung (disorder)
        "12240991000119102",  # Squamous cell carcinoma of right lung (disorder)
        "12241031000119108",  # Metastatic malignant neoplasm to left breast (disorder)
        "12241071000119106",  # Metastatic malignant neoplasm to right breast (disorder)
        "12246481000119106",  # Metastatic malignant neoplasm to bilateral kidneys (disorder)
        "12246561000119101",  # Metastatic malignant neoplasm to bilateral adrenal glands (disorder)
        "12246601000119101",  # Metastatic malignant neoplasm to bilateral lungs (disorder)
        "12246641000119104",  # Metastatic malignant neoplasm to bilateral breasts (disorder)
        "12281000132104",  # Relapsing acute myeloid leukemia (disorder)
        "122881000119107",  # Chronic monocytic leukemia in relapse (disorder)
        "122901000119109",  # Myeloid leukemia in relapse (disorder)
        "12291000132102",  # Refractory acute myeloid leukemia (disorder)
        "122951000119108",  # Lymphoid leukemia in relapse (disorder)
        "122961000119105",  # Chronic lymphoid leukemia in relapse (disorder)
        "122981000119101",  # Plasma cell leukemia in relapse (disorder)
        "1229871006",  # Primary squamous cell carcinoma of nasal cavity and paranasal sinus (disorder)
        "1229988003",  # Myeloid and/or lymphoid neoplasm with pericentriolar material 1-janus kinase 2 (disorder)
        "12301000132103",  # Acute lymphoid leukemia relapse (disorder)
        "1230318000",  # Serous carcinoma of body of uterus (disorder)
        "12311000132101",  # Refractory acute lymphoid leukemia (disorder)
        "1231267007",  # Primary malignant ependymoma of optic nerve (disorder)
        "1231381006",  # Clear cell urothelial carcinoma of urinary system (disorder)
        "123313007",  # Alpha heavy chain disease, enteric form (disorder)
        "123661000119106",  # Malignant carcinoid tumor of lung (disorder)
        "1236931008",  # Metastatic carcinoma to body of pancreas (disorder)
        "1236940007",  # Metastatic carcinoma to tail of pancreas (disorder)
        "1236941006",  # Metastatic carcinoma to intrahepatic bile duct (disorder)
        "1236942004",  # Metastatic carcinoma to ventral surface of tongue (disorder)
        "1236943009",  # Metastatic carcinoma to anterior two-thirds tongue ventrum (disorder)
        "1236944003",  # Metastatic carcinoma to dorsal surface of anterior two-thirds of tongue (disorder)
        "1236945002",  # Metastatic carcinoma to midline of tongue (disorder)
        "1236947005",  # Metastatic carcinoma to dorsal surface of base of tongue (disorder)
        "1236948000",  # Metastatic carcinoma to fallopian tube (disorder)
        "1236953005",  # Metastatic carcinoma to bone, connective tissue, skin and breast (disorder)
        "1236954004",  # Metastatic carcinoma to vermilion border of upper lip (disorder)
        "1236955003",  # Metastatic carcinoma to frenum of lip (disorder)
        "1236956002",  # Metastatic carcinoma to frenum of lower lip (disorder)
        "1236957006",  # Metastatic carcinoma to frenum of upper lip (disorder)
        "1236958001",  # Metastatic carcinoma to commissure of lip (disorder)
        "1236960004",  # Metastatic carcinoma to lingual tonsil (disorder)
        "1236961000",  # Metastatic carcinoma to anterior part of floor of mouth (disorder)
        "1236962007",  # Metastatic carcinoma to lateral part of floor of mouth (disorder)
        "1236963002",  # Metastatic carcinoma to hard palate (disorder)
        "1236964008",  # Metastatic carcinoma to soft palate (disorder)
        "1236966005",  # Metastatic carcinoma to upper buccal sulcus (disorder)
        "1236967001",  # Metastatic carcinoma to lower buccal sulcus (disorder)
        "1236968006",  # Metastatic carcinoma to upper labial sulcus (disorder)
        "1236969003",  # Metastatic carcinoma to lower labial sulcus (disorder)
        "123701000119104",  # Malignant carcinoid tumor of descending colon (disorder)
        "1237031003",  # Metastatic carcinoma to fundus of stomach (disorder)
        "1237032005",  # Metastatic carcinoma to thoracic part of esophagus (disorder)
        "1237033000",  # Metastatic carcinoma to sublingual gland (disorder)
        "1237034006",  # Metastatic carcinoma to submandibular gland (disorder)
        "1237035007",  # Metastatic carcinoma to parotid gland (disorder)
        "1237036008",  # Metastatic carcinoma to middle third of esophagus (disorder)
        "1237037004",  # Metastatic carcinoma to cervical part of esophagus (disorder)
        "1237038009",  # Metastatic carcinoma to abdominal part of esophagus (disorder)
        "1237039001",  # Metastatic carcinoma to upper third of esophagus (disorder)
        "1237041000",  # Metastatic carcinoma to pylorus (disorder)
        "1237042007",  # Metastatic carcinoma to pyloric antrum (disorder)
        "1237043002",  # Metastatic carcinoma to greater curve of stomach (disorder)
        "1237044008",  # Metastatic carcinoma to lesser curve of stomach (disorder)
        "1237045009",  # Metastatic carcinoma to body of stomach (disorder)
        "1237046005",  # Metastatic carcinoma to duodenum (disorder)
        "1237047001",  # Metastatic carcinoma to endocrine pancreas (disorder)
        "1237048006",  # Metastatic squamous cell carcinoma to bronchus (disorder)
        "1237050003",  # Metastatic carcinoma to cardia of stomach (disorder)
        "1237051004",  # Metastatic carcinoma to lower third of esophagus (disorder)
        "1237052006",  # Metastatic carcinoma to axillary tail of breast (disorder)
        "1237053001",  # Metastatic carcinoma to upper inner quadrant of breast (disorder)
        "1237055008",  # Metastatic carcinoma to upper outer quadrant of breast (disorder)
        "1237056009",  # Metastatic carcinoma to lower inner quadrant of breast (disorder)
        "1237057000",  # Metastatic carcinoma to lower outer quadrant of breast (disorder)
        "123721000119108",  # Malignant carcinoid tumor of ascending colon (disorder)
        "123731000119106",  # Malignant carcinoid tumor of cecum (disorder)
        "1237341006",  # Intraductal tubulopapillary malignant neoplasm of pancreas (disorder)
        "1237368006",  # Acute myeloid leukemia with BCR-ABL1 (disorder)
        "1237372005",  # Metastatic carcinoma to vagina (disorder)
        "123741000119102",  # Malignant carcinoid tumor of large intestine (disorder)
        "1237414000",  # Metastatic cystadenocarcinoma to kidney (disorder)
        "1237415004",  # Metastatic cystadenocarcinoma to ovary (disorder)
        "1237416003",  # Metastatic cystadenocarcinoma to pancreas (disorder)
        "1237422007",  # Metastatic carcinoma to prostate (disorder)
        "1237423002",  # Metastatic carcinoma to glans penis (disorder)
        "1237424008",  # Metastatic carcinoma to foreskin (disorder)
        "1237425009",  # Metastatic carcinoma to pituitary (disorder)
        "1237426005",  # Metastatic carcinoma to parathyroid (disorder)
        "1237427001",  # Metastatic carcinoma to cecum (disorder)
        "1237428006",  # Metastatic carcinoma to common bile duct (disorder)
        "1237454003",  # Metastatic carcinoma to splenic flexure (disorder)
        "1237455002",  # Metastatic carcinoma to descending colon (disorder)
        "1237456001",  # Metastatic carcinoma to hepatic flexure (disorder)
        "1237457005",  # Metastatic carcinoma to urinary bladder (disorder)
        "1237458000",  # Metastatic carcinoma to transverse colon (disorder)
        "1237460003",  # Metastatic carcinoma to ascending colon (disorder)
        "1237461004",  # Metastatic adenocarcinoma to retinal pigment epithelium (disorder)
        "1237465008",  # Metastatic adenocarcinoma to urinary bladder (disorder)
        "1237466009",  # Metastatic squamous cell carcinoma to bronchus of left upper lobe (disorder)
        "1237467000",  # Metastatic squamous cell carcinoma to bronchus of right upper lobe (disorder)
        "1237469002",  # Metastatic squamous cell carcinoma to bronchus of left lower lobe (disorder)
        "1237471002",  # Metastatic squamous cell carcinoma to bronchus of right lower lobe (disorder)
        "1237472009",  # Metastatic squamous cell carcinoma to bronchus of middle lobe of right lung (disorder)
        "1237473004",  # Metastatic carcinoma to head of pancreas (disorder)
        "1237474005",  # Metastatic carcinoma to bone of upper limb (disorder)
        "1237476007",  # Metastatic carcinoma to bone of lower limb (disorder)
        "1237477003",  # Metastatic carcinoma to skin of head (disorder)
        "1237478008",  # Metastatic carcinoma to skin of neck (disorder)
        "1237479000",  # Metastatic carcinoma to anal canal (disorder)
        "1237480002",  # Metastatic carcinoma to sigmoid colon (disorder)
        "1237481003",  # Metastatic carcinoma to cervix (disorder)
        "1237482005",  # Metastatic papillary adenocarcinoma to thyroid (disorder)
        "1237483000",  # Metastatic carcinoma to tongue (disorder)
        "1237484006",  # Metastatic carcinoma to colon (disorder)
        "1237485007",  # Metastatic carcinoma to rectosigmoid junction (disorder)
        "1237487004",  # Metastatic carcinoma to base of tongue (disorder)
        "1237488009",  # Metastatic carcinoma to palate (disorder)
        "1237489001",  # Metastatic carcinoma to tonsil (disorder)
        "1237490005",  # Metastatic carcinoma to genital organ (disorder)
        "1237492002",  # Metastatic carcinoma to endocervix (disorder)
        "1237493007",  # Metastatic carcinoma to exocervix (disorder)
        "1237494001",  # Metastatic carcinoma to extrahepatic bile duct (disorder)
        "1237495000",  # Metastatic carcinoma to glottis (disorder)
        "1237496004",  # Metastatic carcinoma to subglottis (disorder)
        "1237497008",  # Metastatic carcinoma to supraglottis (disorder)
        "1237498003",  # Metastatic carcinoma to penis (disorder)
        "123751000119100",  # Malignant carcinoid tumor of jejunum (disorder)
        "1237516000",  # Metastatic carcinoma to lower lobe of lung (disorder)
        "1237517009",  # Metastatic carcinoma to pelvic bones, sacrum and coccyx (disorder)
        "1237518004",  # Metastatic carcinoma to main bronchus (disorder)
        "1237519007",  # Metastatic carcinoma to skin of lower limb (disorder)
        "1237520001",  # Metastatic carcinoma to skin of trunk (disorder)
        "1237521002",  # Metastatic carcinoma to skin of upper limb (disorder)
        "1237522009",  # Metastatic carcinoma to gallbladder (disorder)
        "1237523004",  # Metastatic carcinoma to vocal cord (disorder)
        "1237524005",  # Metastatic carcinoma to pancreas (disorder)
        "1237525006",  # Metastatic carcinoma to stomach (disorder)
        "1237526007",  # Metastatic carcinoma to bone of rib (disorder)
        "1237527003",  # Metastatic carcinoma to bone of sternum (disorder)
        "1237528008",  # Metastatic carcinoma to bone of clavicle (disorder)
        "1237529000",  # Metastatic carcinoma to ampulla of Vater (disorder)
        "1237530005",  # Metastatic adenocarcinoma to carotid body (disorder)
        "1237531009",  # Metastatic adenocarcinoma to parathyroid (disorder)
        "1237578005",  # Primary oculocerebral non-Hodgkin lymphoma (disorder)
        "123761000119103",  # Malignant carcinoid tumor of duodenum (disorder)
        "1237621006",  # Metastatic adenocarcinoma to Moll's gland (disorder)
        "123781000119107",  # Chronic leukemia in relapse (disorder)
        "123842006",  # Endocervical adenocarcinoma (disorder)
        "123845008",  # Adenocarcinoma of endometrium (disorder)
        "1240346000",  # Metastatic adenocarcinoma to Meckel diverticulum (disorder)
        "1240351006",  # Metastatic carcinoma to lower gum (disorder)
        "1240352004",  # Metastatic carcinoma to upper gum (disorder)
        "1240353009",  # Metastatic squamous cell carcinoma to prostate (disorder)
        "1240355002",  # Metastatic squamous cell carcinoma to epididymis and spermatic cord (disorder)
        "1240359008",  # Metastatic small cell carcinoma to prostate (disorder)
        "1240362006",  # Metastatic small cell carcinoma (disorder)
        "1240363001",  # Metastatic small cell carcinoma to ovary (disorder)
        "1240370001",  # Metastatic squamous cell carcinoma to liver and intrahepatic biliary tract (disorder)
        "1240372009",  # Metastatic carcinoma to meibomian gland (disorder)
        "1251445000",  # Metastatic neuroblastoma to abdominothoracic structure (disorder)
        "1251454002",  # Metastatic neuroblastoma to adrenal gland (disorder)
        "1251456000",  # Metastatic neuroblastoma to brain (disorder)
        "1251457009",  # Metastatic neuroblastoma to central nervous system (disorder)
        "1251458004",  # Metastatic neuroblastoma to thorax (disorder)
        "1251459007",  # Metastatic neuroblastoma to paraspinal area (disorder)
        "1251460002",  # Metastatic neuroblastoma to pelvis (disorder)
        "1251465007",  # Metastatic extraskeletal osteosarcoma (disorder)
        "1251466008",  # Metastatic osteosarcoma to articular cartilage of clavicle (disorder)
        "1251467004",  # Metastatic osteosarcoma to articular cartilage of rib (disorder)
        "1251468009",  # Metastatic osteosarcoma to articular cartilage of sternum (disorder)
        "1251469001",  # Metastatic osteosarcoma to articular cartilage of vertebra (disorder)
        "1251470000",  # Metastatic osteosarcoma to bone (disorder)
        "1251471001",  # Metastatic osteosarcoma to bone of clavicle (disorder)
        "1251472008",  # Metastatic osteosarcoma to bone of head (disorder)
        "1251473003",  # Metastatic osteosarcoma to bone of rib (disorder)
        "1251474009",  # Metastatic osteosarcoma to bone of vertebra (disorder)
        "1251475005",  # Metastatic osteosarcoma to bone of sternum (disorder)
        "1251482009",  # Metastatic adenocarcinoma to pancreatic duct (disorder)
        "1251483004",  # Metastatic adenocarcinoma to paraganglia (disorder)
        "1251484005",  # Metastatic adenocarcinoma to pituitary gland (disorder)
        "1251485006",  # Metastatic adenocarcinoma to prostate (disorder)
        "1251486007",  # Metastatic carcinoma to liver (disorder)
        "1251487003",  # Metastatic carcinoma to biliary tract (disorder)
        "1251489000",  # Metastatic carcinoma to frenum linguae (disorder)
        "1251490009",  # Metastatic carcinoma to gallbladder and extrahepatic biliary tract (disorder)
        "1251491008",  # Metastatic carcinoma to larynx (disorder)
        "1251492001",  # Metastatic carcinoma to lip (disorder)
        "1251493006",  # Metastatic carcinoma to lip, oral cavity and/or pharynx (disorder)
        "1251494000",  # Metastatic carcinoma to retromolar area (disorder)
        "1251495004",  # Metastatic carcinoma to scapula (disorder)
        "1251496003",  # Metastatic carcinoma to uvula (disorder)
        "1251497007",  # Metastatic carcinoma to vermilion border of lower lip (disorder)
        "1251498002",  # Metastatic sarcoma to retroperitoneum (disorder)
        "1251564008",  # Relapsing malignant neoplastic disease (disorder)
        "1251565009",  # Refractory malignant neoplastic disease (disorder)
        "1254689008",  # Metastatic adenoid cystic carcinoma to cervix uteri (disorder)
        "1254691000",  # Metastatic adenoid cystic carcinoma to left lacrimal gland (disorder)
        "1254723004",  # Metastatic adenoid cystic carcinoma to right lacrimal gland (disorder)
        "1254724005",  # Metastatic adenoid cystic carcinoma to oropharynx (disorder)
        "1254725006",  # Metastatic adenoid cystic carcinoma to parotid gland (disorder)
        "1254726007",  # Metastatic adenoid cystic carcinoma to salivary gland (disorder)
        "1254727003",  # Metastatic adenoid cystic carcinoma to submandibular gland (disorder)
        "1254728008",  # Metastatic adenoid cystic carcinoma to trachea (disorder)
        "1254732002",  # Metastatic large cell carcinoma to lung (disorder)
        "1254733007",  # Metastatic large cell carcinoma to right lung (disorder)
        "1254734001",  # Metastatic large cell carcinoma to left lung (disorder)
        "1254735000",  # Metastatic large cell carcinoma to bilateral lungs (disorder)
        "1254740008",  # Metastatic giant cell carcinoma to lung (disorder)
        "1254745003",  # Metastatic giant cell carcinoma to left lung (disorder)
        "1254746002",  # Metastatic giant cell carcinoma to right lung (disorder)
        "1254747006",  # Metastatic giant cell carcinoma to bilateral lungs (disorder)
        "1254748001",  # Metastatic clear cell squamous cell carcinoma to skin (disorder)
        "1254749009",  # Metastatic acantholytic squamous cell carcinoma (disorder)
        "1254750009",  # Metastatic acantholytic squamous cell carcinoma to skin (disorder)
        "1254752001",  # Metastatic verrucous carcinoma to oral cavity (disorder)
        "1254753006",  # Metastatic verrucous carcinoma to skin of sole of foot (disorder)
        "1254755004",  # Metastatic basosquamous cell carcinoma to skin (disorder)
        "1254781000",  # Ewing sarcoma of bone of rib (disorder)
        "1254783002",  # Verrucous carcinoma of trachea (disorder)
        "1254785009",  # Squamous cell adenoid carcinoma of trachea (disorder)
        "1254786005",  # Spindle cell carcinoma of trachea (disorder)
        "1254787001",  # Papillary squamous cell carcinoma of trachea (disorder)
        "1254788006",  # Lymphoepithelial carcinoma of trachea (disorder)
        "1254789003",  # Clear cell squamous cell carcinoma of trachea (disorder)
        "1254790007",  # Basaloid squamous cell carcinoma of trachea (disorder)
        "1254800007",  # Squamous cell carcinoma of lateral wall of oropharynx (disorder)
        "1254801006",  # Basaloid squamous cell carcinoma of oropharynx (disorder)
        "1254803009",  # Papillary squamous cell carcinoma of oropharynx (disorder)
        "1254805002",  # Squamous cell carcinoma of posterior wall of oropharynx (disorder)
        "1254806001",  # Spindle cell squamous cell carcinoma of oropharynx (disorder)
        "1254825003",  # Primary malignant mesenchymal neoplasm of anus (disorder)
        "1254844005",  # Primary malignant mesenchymal neoplasm of bile duct (disorder)
        "1254845006",  # Primary malignant mesenchymal neoplasm of gallbladder (disorder)
        "1254846007",  # Primary malignant mesenchymal neoplasm of duodenum (disorder)
        "1254847003",  # Primary malignant mesenchymal neoplasm of appendix (disorder)
        "1254848008",  # Primary malignant mesenchymal neoplasm of esophagus (disorder)
        "1254852008",  # Squamous cell carcinoma of laryngeal cartilage (disorder)
        "1254957001",  # Malignant neoplasm of blood vessel (disorder)
        "1255074005",  # Adenocarcinoma of frontal sinus (disorder)
        "1255084006",  # Adenocarcinoma of sphenoidal sinus (disorder)
        "1255085007",  # Adenocarcinoma of trachea (disorder)
        "1255086008",  # Differentiated thyroid carcinoma (disorder)
        "1255092002",  # Adenocarcinoma of accessory sinus (disorder)
        "1255096004",  # Adenocarcinoma of ethmoidal sinus (disorder)
        "1255110007",  # Adenocarcinoma of maxillary sinus (disorder)
        "1255112004",  # Adenocarcinoma of oropharynx (disorder)
        "1255190008",  # Primary diffuse sclerosing papillary thyroid carcinoma (disorder)
        "1255191007",  # Primary papillary thyroid carcinoma (disorder)
        "1255192000",  # Primary follicular thyroid carcinoma (disorder)
        "1255193005",  # Primary Hurthle cell carcinoma of thyroid (disorder)
        "1255194004",  # Primary papillary thyroid carcinoma with renal papillary neoplasia (disorder)
        "1255270006",  # Lymphoplasmacytic lymphoma without immunoglobulin M production (disorder)
        "1255358009",  # Oligometastatic malignant neoplasm (disorder)
        "1255385002",  # Rhabdomyosarcoma of endometrium of corpus uteri (disorder)
        "1255392007",  # Undifferentiated carcinoma of trachea (disorder)
        "1255393002",  # Undifferentiated carcinoma of lung (disorder)
        "1255394008",  # Undifferentiated carcinoma of endometrium (disorder)
        "1255395009",  # Spindle cell carcinoma of lung (disorder)
        "1255628005",  # Rhabdomyosarcoma of myometrium of corpus uteri (disorder)
        "1255629002",  # Signet ring cell carcinoma of lung (disorder)
        "1255630007",  # Signet ring cell carcinoma of trachea (disorder)
        "1255631006",  # Spindle cell squamous cell carcinoma of hypopharynx (disorder)
        "1255632004",  # Spindle cell squamous cell carcinoma of larynx (disorder)
        "1255641009",  # Pleomorphic carcinoma of lung (disorder)
        "1255642002",  # Polymorphous low grade adenocarcinoma of oropharynx (disorder)
        "1255643007",  # Acinar cell carcinoma of trachea (disorder)
        "1255644001",  # Acinar cell carcinoma of lung (disorder)
        "1255645000",  # Acinar cell carcinoma of oropharynx (disorder)
        "1255646004",  # Basaloid squamous cell carcinoma of hypopharynx (disorder)
        "1255647008",  # Basaloid squamous cell carcinoma of larynx (disorder)
        "1255648003",  # Basaloid carcinoma of hypopharynx (disorder)
        "1255649006",  # Basaloid carcinoma of oropharynx (disorder)
        "1255650006",  # Basaloid carcinoma of larynx (disorder)
        "1255651005",  # Basaloid squamous cell carcinoma of lung (disorder)
        "1255652003",  # Carcinoma ex pleomorphic adenoma of oropharynx (disorder)
        "1255654002",  # Giant cell carcinoma of larynx (disorder)
        "1255655001",  # Giant cell carcinoma of trachea (disorder)
        "1255656000",  # Giant cell carcinoma of hypopharynx (disorder)
        "1255657009",  # Carcinoma of sphenoidal sinus (disorder)
        "1255658004",  # Carcinosarcoma of lung (disorder)
        "1255659007",  # Epithelial-myoepithelial carcinoma of oropharynx (disorder)
        "1255660002",  # Squamous cell carcinoma of sphenoidal sinus (disorder)
        "1255661003",  # Myoepithelial carcinoma of oropharynx (disorder)
        "1255662005",  # Myoepithelial carcinoma of lung (disorder)
        "1255663000",  # Myoepithelial carcinoma of trachea (disorder)
        "1255666008",  # Hodgkin lymphoma, lymphocyte depletion of lymph nodes of neck (disorder)
        "1255687003",  # Hodgkin lymphoma, lymphocyte depletion of lymph nodes of head and neck (disorder)
        "1255725002",  # Non-small cell adenocarcinoma of lung (disorder)
        "1255726001",  # Hodgkin granuloma of lymph nodes of head (disorder)
        "1255727005",  # Hodgkin granuloma of lymph nodes of neck (disorder)
        "1255728000",  # Hodgkin granuloma of lymph nodes of head and neck (disorder)
        "1255729008",  # Mast cell malignancy of lymph nodes of head (disorder)
        "1255730003",  # Mast cell malignancy of lymph nodes of neck (disorder)
        "1255737000",  # Hodgkin disease, lymphocytic-histiocytic predominance of lymph nodes of head (disorder)
        "1255738005",  # Hodgkin disease, lymphocytic-histiocytic predominance of lymph nodes of neck (disorder)
        "1255739002",  # Hodgkin disease, lymphocytic-histiocytic predominance of lymph nodes of head and neck (disorder)
        "1255743003",  # Hodgkin sarcoma of lymph nodes of head (disorder)
        "1255744009",  # Hodgkin sarcoma of lymph nodes of neck (disorder)
        "1255745005",  # Hodgkin sarcoma of lymph nodes of head and neck (disorder)
        "1255746006",  # Malignant histiocytosis of lymph nodes of head (disorder)
        "1255747002",  # Malignant histiocytosis of lymph nodes of neck (disorder)
        "1255748007",  # Malignant histiocytosis of lymph nodes of head and neck (disorder)
        "1255749004",  # Burkitt tumor of lymph nodes of head (disorder)
        "1255750004",  # Burkitt tumor of lymph nodes of neck (disorder)
        "1255751000",  # Burkitt tumor of lymph nodes of head and neck (disorder)
        "1255752007",  # Mycosis fungoides of lymph nodes of head (disorder)
        "1255753002",  # Mycosis fungoides of lymph nodes of neck (disorder)
        "1255754008",  # Mycosis fungoides of lymph nodes of head and neck (disorder)
        "1255784000",  # Hodgkin paragranuloma of lymph nodes of head (disorder)
        "1255785004",  # Hodgkin paragranuloma of lymph nodes of neck (disorder)
        "1255786003",  # Hodgkin paragranuloma of lymph nodes of head and neck (disorder)
        "1255788002",  # Hodgkin disease, mixed cellularity of lymph nodes of head (disorder)
        "1255789005",  # Hodgkin disease, mixed cellularity of lymph nodes of neck (disorder)
        "1255790001",  # Hodgkin disease, mixed cellularity of lymph nodes of head and neck (disorder)
        "1255791002",  # Hodgkin disease, nodular sclerosis of lymph nodes of head and neck (disorder)
        "1255792009",  # Hodgkin disease, nodular sclerosis of lymph nodes of head (disorder)
        "1255793004",  # Hodgkin disease, nodular sclerosis of lymph nodes of neck (disorder)
        "1255795006",  # Malignant histiocytosis of lymph nodes of lower limb (disorder)
        "1255796007",  # Burkitt lymphoma of lymph nodes of lower limb (disorder)
        "1255797003",  # Hodgkin disease, nodular sclerosis of lymph nodes of lower limb (disorder)
        "1255798008",  # Hodgkin granuloma of lymph nodes of upper limb (disorder)
        "1255799000",  # Sezary disease of lymph nodes of head (disorder)
        "1255800001",  # Sezary disease of lymph nodes of neck (disorder)
        "1255801002",  # Sezary disease of lymph nodes of head and neck (disorder)
        "1255802009",  # Hodgkin disease of lymph nodes of head (disorder)
        "1255803004",  # Hodgkin disease of lymph nodes of neck (disorder)
        "1255804005",  # Hodgkin disease of lymph nodes of head and neck (disorder)
        "1255805006",  # Malignant lymphoma of lymph nodes of head (disorder)
        "1255806007",  # Malignant lymphoma of lymph nodes of neck (disorder)
        "1255807003",  # Malignant lymphoma of lymph nodes of head and neck (disorder)
        "1255808008",  # Hodgkin sarcoma of lymph nodes of upper limb (disorder)
        "1255809000",  # Malignant mast cell tumor of lymph nodes of upper limb (disorder)
        "1255811009",  # Hodgkin disease, mixed cellularity of lymph nodes of upper limb (disorder)
        "1255812002",  # Hodgkin disease, nodular sclerosis of lymph nodes of upper limb (disorder)
        "1255814001",  # Hodgkin disease, lymphocytic-histiocytic predominance of lymph nodes of upper limb (disorder)
        "1255815000",  # Malignant histiocytosis of lymph nodes of upper limb (disorder)
        "1255883001",  # Hodgkin sarcoma of lymph nodes of lower limb (disorder)
        "1255886009",  # Mycosis fungoides of lymph nodes of upper limb (disorder)
        "1255887000",  # Hodgkin's paragranuloma of lymph nodes of lower limb (disorder)
        "1255888005",  # Hodgkin paragranuloma of lymph nodes of upper limb (disorder)
        "1255890006",  # Hodgkin disease, lymphocytic depletion of lymph nodes of upper limb (disorder)
        "1255892003",  # Hodgkin disease, lymphocytic depletion of lymph nodes of lower limb (disorder)
        "1255894002",  # Hodgkin granuloma of lymph nodes of lower limb (disorder)
        "1255895001",  # Burkitt lymphoma of lymph nodes of upper limb (disorder)
        "1255897009",  # Hodgkin disease, lymphocytic-histiocytic predominance of lymph nodes of lower limb (disorder)
        "1255898004",  # Hodgkin disease, mixed cellularity of lymph nodes of lower limb (disorder)
        "1255899007",  # Malignant mast cell tumor of lymph nodes of lower limb (disorder)
        "1255902005",  # Hodgkin disease of lymph nodes of lower limb (disorder)
        "1255905007",  # Hodgkin disease of lymph nodes of upper limb (disorder)
        "1255906008",  # Malignant lymphoma of lymph nodes of upper limb (disorder)
        "1255907004",  # Malignant lymphoma of lymph nodes of lower limb (disorder)
        "1259175002",  # Adenocarcinoma of neck (disorder)
        "1259176001",  # Adenocarcinoma of head (disorder)
        "1259237005",  # Recurrent malignant neoplasm (disorder)
        "1259251005",  # Primary mucoepidermoid carcinoma of skin (disorder)
        "1259260002",  # Primary mucinous eccrine carcinoma of skin (disorder)
        "1259309001",  # Primary carcinoma of body of pancreas (disorder)
        "1259310006",  # Primary carcinoma of tail of pancreas (disorder)
        "1259311005",  # Primary cystadenocarcinoma of pancreas (disorder)
        "1259312003",  # Primary fibrosarcoma of spleen (disorder)
        "1259313008",  # Primary intrahepatic bile duct carcinoma (disorder)
        "1259314002",  # Primary carcinoma ventral surface of tongue (disorder)
        "1259315001",  # Primary carcinoma of anterior two-thirds of tongue ventrum (disorder)
        "1259316000",  # Primary carcinoma of anterior two-thirds of tongue - dorsal surface (disorder)
        "1259317009",  # Primary carcinoma of midline of tongue (disorder)
        "1259318004",  # Primary carcinoma of tongue base - dorsal surface (disorder)
        "1259319007",  # Primary amelanotic malignant melanoma of skin (disorder)
        "1259320001",  # Primary carcinoma of fallopian tube (disorder)
        "1259321002",  # Primary carcinoma of larynx (disorder)
        "1259322009",  # Primary lobular carcinoma of breast (disorder)
        "1259323004",  # Primary carcinoma of vermilion border of upper lip (disorder)
        "1259324005",  # Primary carcinoma of vermilion border of lower lip (disorder)
        "1259325006",  # Primary carcinoma of frenum of lip (disorder)
        "1259326007",  # Primary carcinoma of frenum of upper lip (disorder)
        "1259327003",  # Primary carcinoma of frenum of lower lip (disorder)
        "1259328008",  # Primary carcinoma of commissure of lip (disorder)
        "1259329000",  # Primary carcinoma of frenum linguae (disorder)
        "1259330005",  # Primary carcinoma of lingual tonsil (disorder)
        "1259331009",  # Primary carcinoma of anterior part of floor of mouth (disorder)
        "1259332002",  # Primary carcinoma of lateral part of floor of mouth (disorder)
        "1259333007",  # Primary carcinoma of hard palate (disorder)
        "1259334001",  # Primary carcinoma of soft palate (disorder)
        "1259335000",  # Primary carcinoma of uvula (disorder)
        "1259336004",  # Primary carcinoma of upper buccal sulcus (disorder)
        "1259337008",  # Primary carcinoma of lower buccal sulcus (disorder)
        "1259338003",  # Primary carcinoma of upper labial sulcus (disorder)
        "1259339006",  # Primary carcinoma of lower labial sulcus (disorder)
        "1259340008",  # Primary carcinoma of retromolar area (disorder)
        "1259341007",  # Primary carcinoma of parotid gland (disorder)
        "1259342000",  # Primary carcinoma of submandibular gland (disorder)
        "1259343005",  # Primary carcinoma of sublingual gland (disorder)
        "1259344004",  # Primary carcinoma of cervical part of esophagus (disorder)
        "1259345003",  # Primary carcinoma of thoracic part of esophagus (disorder)
        "1259346002",  # Primary carcinoma of abdominal part of esophagus (disorder)
        "1259347006",  # Primary carcinoma of upper third of esophagus (disorder)
        "1259348001",  # Primary carcinoma of middle third of esophagus (disorder)
        "1259349009",  # Primary carcinoma of lower third of esophagus (disorder)
        "1259350009",  # Primary carcinoma of cardia (disorder)
        "1259351008",  # Primary carcinoma of fundus of stomach (disorder)
        "1259352001",  # Primary carcinoma of body of stomach (disorder)
        "1259353006",  # Primary carcinoma of pyloric antrum (disorder)
        "1259354000",  # Primary carcinoma of pylorus (disorder)
        "1259355004",  # Primary carcinoma of lesser curve of stomach (disorder)
        "1259356003",  # Primary carcinoma of greater curve of stomach (disorder)
        "1259357007",  # Primary carcinoma of duodenum (disorder)
        "1259358002",  # Primary carcinoma of endocrine pancreas (disorder)
        "1259366006",  # Primary adenoid cystic carcinoma of trachea (disorder)
        "1259367002",  # Primary squamous cell carcinoma of bronchus (disorder)
        "1259368007",  # Primary large cell carcinoma of lung (disorder)
        "1259369004",  # Primary giant cell carcinoma of lung (disorder)
        "1259371004",  # Primary squamous cell carcinoma of lung (disorder)
        "1259372006",  # Primary clear cell squamous cell carcinoma of skin (disorder)
        "1259373001",  # Primary spindle cell squamous carcinoma of skin (disorder)
        "1259374007",  # Primary acantholytic squamous cell carcinoma of skin (disorder)
        "1259375008",  # Primary plantar verrucous carcinoma (disorder)
        "1259376009",  # Primary basosquamous carcinoma of skin (disorder)
        "1259377000",  # Primary basal cell carcinoma with eccrine differentiation (disorder)
        "1259378005",  # Primary superficial spreading malignant melanoma of skin (disorder)
        "1259379002",  # Primary nodular malignant melanoma of skin (disorder)
        "1259380004",  # Primary acral lentiginous malignant melanoma of skin (disorder)
        "1259381000",  # Primary cutaneous fibrosarcoma (disorder)
        "1259382007",  # Primary epithelioid cell sarcoma of skin (disorder)
        "1259383002",  # Primary scirrhous carcinoma of breast (disorder)
        "1259384008",  # Primary serous papillary cystadenocarcinoma ovary (disorder)
        "1259385009",  # Primary mucinous cystadenocarcinoma of ovary (disorder)
        "1259386005",  # Primary undifferentiated carcinoma of ovary (disorder)
        "1259387001",  # Primary adenosquamous carcinoma of cervix (disorder)
        "1259388006",  # Primary carcinoma of prostate (disorder)
        "1259389003",  # Primary carcinoma of glans penis (disorder)
        "1259390007",  # Primary carcinoma of foreskin (disorder)
        "1259391006",  # Primary cystadenocarcinoma of kidney (disorder)
        "1259392004",  # Primary papillary cystadenocarcinoma of kidney (disorder)
        "1259393009",  # Primary adenoid cystic carcinoma of lacrimal gland (disorder)
        "1259394003",  # Primary carcinoma ex pleomorphic adenoma of lacrimal gland (disorder)
        "1259395002",  # Primary mixed follicular and papillary thyroid carcinoma (disorder)
        "1259396001",  # Primary carcinoma of genitourinary organ (disorder)
        "1259398000",  # Primary carcinoma of cecum (disorder)
        "1259399008",  # Primary carcinoma common bile duct (disorder)
        "1259400001",  # Primary carcinoma of urinary bladder (disorder)
        "1259401002",  # Primary transitional cell carcinoma of urinary bladder (disorder)
        "1259402009",  # Primary adenocarcinoma of urinary bladder (disorder)
        "1259403004",  # Primary carcinoma of ascending colon (disorder)
        "1259404005",  # Primary carcinoma of transverse colon (disorder)
        "1259405006",  # Primary carcinoma of descending colon (disorder)
        "1259406007",  # Primary carcinoma of hepatic flexure (disorder)
        "1259407003",  # Primary carcinoma of splenic flexure (disorder)
        "1259408008",  # Primary retinal pigment epithelial adenocarcinoma (disorder)
        "1259409000",  # Primary squamous cell carcinoma of bronchus in left lower lobe (disorder)
        "1259410005",  # Primary squamous cell carcinoma of bronchus in left upper lobe (disorder)
        "1259411009",  # Primary squamous cell carcinoma of bronchus in right lower lobe (disorder)
        "1259412002",  # Primary squamous cell carcinoma of bronchus of middle lobe of right lung (disorder)
        "1259413007",  # Primary squamous cell carcinoma of bronchus in right upper lobe (disorder)
        "1259414001",  # Primary cystadenocarcinoma of ovary (disorder)
        "1259415000",  # Primary carcinoma of head of pancreas (disorder)
        "1259416004",  # Primary sebaceous adenocarcinoma (disorder)
        "1259417008",  # Primary carcinoma of bone of lower limb (disorder)
        "1259418003",  # Primary carcinoma of breast - upper, inner quadrant (disorder)
        "1259419006",  # Primary carcinoma of breast - lower, inner quadrant (disorder)
        "1259420000",  # Primary carcinoma of upper outer quadrant of breast (disorder)
        "1259421001",  # Primary carcinoma breast - lower, outer quadrant (disorder)
        "1259422008",  # Primary carcinoma of breast - axillary tail (disorder)
        "1259423003",  # Primary transitional cell carcinoma of ureter (disorder)
        "1259424009",  # Primary lentigo maligna melanoma (disorder)
        "1259425005",  # Primary neuroblastoma of brain (disorder)
        "1259426006",  # Primary adrenal neuroblastoma (disorder)
        "1259427002",  # Primary thoracic neuroblastoma (disorder)
        "1259428007",  # Primary pelvic neuroblastoma (disorder)
        "1259429004",  # Primary paraspinal neuroblastoma (disorder)
        "1259430009",  # Primary abdominothoracic neuroblastoma (disorder)
        "1259431008",  # Primary carcinoma of anal canal (disorder)
        "1259432001",  # Primary carcinoma of sigmoid colon (disorder)
        "1259433006",  # Primary carcinoma of cervix (disorder)
        "1259434000",  # Primary carcinoma of lip (disorder)
        "1259435004",  # Primary tongue carcinoma (disorder)
        "1259436003",  # Primary carcinoma of colon (disorder)
        "1259437007",  # Primary carcinoma of rectosigmoid junction (disorder)
        "1259438002",  # Primary carcinoma of base of tongue (disorder)
        "1259439005",  # Primary palate carcinoma (disorder)
        "1259440007",  # Primary tonsil carcinoma (disorder)
        "1259441006",  # Primary odontogenic ghost cell carcinoma (disorder)
        "1259442004",  # Primary mucinous carcinoma of breast (disorder)
        "1259443009",  # Primary carcinoma of breast with ductal and lobular features (disorder)
        "1259444003",  # Primary undifferentiated carcinoma of nasal sinus (disorder)
        "1259445002",  # Primary keratinizing squamous cell carcinoma of nasopharynx (disorder)
        "1259446001",  # Primary undifferentiated nonkeratinizing squamous cell carcinoma of nasopharynx (disorder)
        "1259447005",  # Primary adenocarcinoma in adenomatous polyp (disorder)
        "1259448000",  # Primary mucinous adenocarcinoma of gastrointestinal tract (disorder)
        "1259449008",  # Primary carcinosarcoma of uterus (disorder)
        "1259450008",  # Primary transitional cell carcinoma of left ureter (disorder)
        "1259451007",  # Primary oxyphilic adenocarcinoma (disorder)
        "1259452000",  # Primary adenosarcoma of uterus (disorder)
        "1259453005",  # Primary fibrosarcoma of connective tissue (disorder)
        "1259454004",  # Primary adenosquamous carcinoma (disorder)
        "1259456002",  # Primary ganglioneuroblastoma of central nervous system (disorder)
        "1259457006",  # Primary neuroblastoma of central nervous system (disorder)
        "1259458001",  # Primary carcinosarcoma of endometrium (disorder)
        "1259459009",  # Primary nodular basal cell carcinoma of skin (disorder)
        "1259461000",  # Primary squamous cell carcinoma of right lung (disorder)
        "1259462007",  # Primary small cell carcinoma (disorder)
        "1259463002",  # Primary squamous cell carcinoma of left lung (disorder)
        "1259464008",  # Primary transitional cell carcinoma of left renal pelvis (disorder)
        "1259466005",  # Primary transitional cell carcinoma of right renal pelvis (disorder)
        "1259468006",  # Primary transitional cell carcinoma of right ureter (disorder)
        "1259470002",  # Primary nonkeratinizing carcinoma of the nasopharynx (disorder)
        "1259472005",  # Primary periosteal osteosarcoma of jaw (disorder)
        "1259487000",  # Primary carcinosarcoma of uterine adnexa (disorder)
        "1259489002",  # Primary renal cell carcinoma (disorder)
        "1259491005",  # Primary carcinoma ex pleomorphic adenoma of parotid gland (disorder)
        "1259493008",  # Primary carcinoma of uterine cervix, invasive (disorder)
        "1259495001",  # Primary adenoid cystic carcinoma of parotid gland (disorder)
        "1259497009",  # Primary mucoepidermoid carcinoma of parotid gland (disorder)
        "1259498004",  # Primary adenoid cystic carcinoma of submandibular gland (disorder)
        "1259500003",  # Primary mucoepidermoid carcinoma of submandibular gland (disorder)
        "1259502006",  # Primary adenoid cystic carcinoma of salivary gland (disorder)
        "1259504007",  # Primary polymorphous low grade adenocarcinoma of salivary gland (disorder)
        "1259505008",  # Primary mucoepidermoid carcinoma of salivary gland (disorder)
        "1259507000",  # Primary adenoid cystic carcinoma of oropharynx (disorder)
        "1259509002",  # Primary undifferentiated carcinoma of nasopharynx (disorder)
        "1259510007",  # Primary carcinoma of genital organ (disorder)
        "1259514003",  # Primary carcinoma of endocervix (disorder)
        "1259516001",  # Primary carcinoma of exocervix (disorder)
        "1259518000",  # Primary carcinoma of extrahepatic bile duct (disorder)
        "1259520002",  # Primary carcinoma of glottis (disorder)
        "1259521003",  # Primary carcinoma of subglottis (disorder)
        "1259523000",  # Primary carcinoma of supraglottis (disorder)
        "1259525007",  # Primary carcinoma of penis (disorder)
        "1259526008",  # Primary carcinoma of pelvic bones, sacrum and coccyx (disorder)
        "1259528009",  # Primary carcinoma of main bronchus (disorder)
        "1259530006",  # Primary carcinoma of skin of lower limb (disorder)
        "1259532003",  # Primary carcinoma of skin of trunk (disorder)
        "1259533008",  # Primary carcinoma of skin of upper limb (disorder)
        "1259535001",  # Primary carcinoma of gallbladder (disorder)
        "1259537009",  # Primary carcinoma of vocal cord (disorder)
        "1259539007",  # Primary carcinoma of pancreas (disorder)
        "1259541008",  # Primary carcinoma of stomach (disorder)
        "1259543006",  # Primary carcinoma of ribs and/or sternum and/or clavicle (disorder)
        "1259544000",  # Primary basal cell carcinoma - adenoid (disorder)
        "1259546003",  # Primary basal cell carcinoma - infiltrative (disorder)
        "1259548002",  # Primary basal cell carcinoma - micronodular (disorder)
        "1259549005",  # Primary verrucous carcinoma of oral cavity (disorder)
        "1259551009",  # Primary spindle cell squamous cell carcinoma (disorder)
        "1259553007",  # Primary acantholytic squamous cell carcinoma (disorder)
        "1259555000",  # Primary verrucous squamous cell carcinoma (disorder)
        "1259557008",  # Primary sclerosing morpheic basal cell carcinoma (disorder)
        "1259559006",  # Primary superficial basal cell carcinoma (disorder)
        "1259561002",  # Primary balloon cell malignant melanoma (disorder)
        "1259563004",  # Primary spindle cell malignant melanoma (disorder)
        "1259564005",  # Primary desmoplastic malignant melanoma (disorder)
        "1259566007",  # Primary trichilemmal carcinoma (disorder)
        "1259567003",  # Primary eccrine ductal carcinoma of skin (disorder)
        "1259569000",  # Primary clear cell eccrine hidradenocarcinoma of skin (disorder)
        "1259571000",  # Primary small cell eccrine carcinoma of skin (disorder)
        "1259573002",  # Primary apocrine adenocarcinoma of skin (disorder)
        "1259575009",  # Primary ceruminous gland adenocarcinoma of skin (disorder)
        "1259577001",  # Primary mixed eccrine/pilar adnexal carcinoma of skin (disorder)
        "1259578006",  # Primary undifferentiated adnexal carcinoma of skin (disorder)
        "1259580000",  # Primary infantile fibrosarcoma (disorder)
        "1259582008",  # Primary botryoid rhabdomyosarcoma (disorder)
        "1259583003",  # Primary spindle cell rhabdomyosarcoma (disorder)
        "1259585005",  # Primary extraskeletal osteosarcoma (disorder)
        "1259587002",  # Primary carcinosarcoma of skin (disorder)
        "1259589004",  # Primary transitional cell carcinoma of kidney (disorder)
        "1259590008",  # Primary infiltrating duct carcinoma of breast (disorder)
        "1259592000",  # Primary eccrine porocarcinoma of skin (disorder)
        "1259593005",  # Primary digital papillary eccrine carcinoma of skin (disorder)
        "1259594004",  # Primary adenoid cystic eccrine carcinoma of skin (disorder)
        "1259596002",  # Primary microcystic adnexal carcinoma of skin (disorder)
        "1259599009",  # Primary eccrine carcinoma of skin (disorder)
        "1259601006",  # Primary myxofibrosarcoma of skin (disorder)
        "1259602004",  # Primary carcinoma liver and/or biliary system (disorder)
        "1259610003",  # Primary carcinoma of Bartholin gland (disorder)
        "1259612006",  # Primary Moll gland adenocarcinoma (disorder)
        "1259613001",  # Primary large cell Ewing sarcoma of bone (disorder)
        "1259615008",  # Primary Ewing sarcoma of bone of pelvis (disorder)
        "1259616009",  # Primary Ewing sarcoma of bone structure of upper limb (disorder)
        "1259618005",  # Primary sebaceous adenocarcinoma of eyelid (disorder)
        "1259620008",  # Primary papillary squamous cell carcinoma of nasopharynx (disorder)
        "1259623005",  # Primary osteosarcoma of bone of head (disorder)
        "1259624004",  # Primary adenocarcinoma of carotid body (disorder)
        "1259626002",  # Primary adenocarcinoma of paraganglia (disorder)
        "1259627006",  # Primary adenocarcinoma of parathyroid gland (disorder)
        "1259629009",  # Primary adenocarcinoma of pituitary gland (disorder)
        "1259631000",  # Primary carcinoma of lower gum (disorder)
        "1259633002",  # Primary carcinoma of upper gum (disorder)
        "1259635009",  # Primary osteosarcoma of articular cartilage of rib (disorder)
        "1259637001",  # Primary osteosarcoma of articular cartilage of vertebra (disorder)
        "1259639003",  # Primary osteosarcoma of articular cartilage of sternum (disorder)
        "1259641002",  # Primary osteosarcoma of bone of clavicle (disorder)
        "1259643004",  # Primary osteosarcoma of bone of rib (disorder)
        "1259645006",  # Primary osteosarcoma of bone of vertebra (disorder)
        "1259646007",  # Primary osteosarcoma of sternum (disorder)
        "1259648008",  # Primary osteosarcoma of articular cartilage of clavicle (disorder)
        "1259650000",  # Primary renal granular cell carcinoma (disorder)
        "1259662001",  # Primary transitional cell carcinoma of right kidney (disorder)
        "1259664000",  # Primary transitional cell carcinoma of left kidney (disorder)
        "1259666003",  # Primary acinar cell cystadenocarcinoma of prostate (disorder)
        "1259668002",  # Primary acinar cell cystadenocarcinoma of lung (disorder)
        "1259669005",  # Primary infiltrating duct carcinoma of prostate (disorder)
        "1259672003",  # Primary adenocarcinoma of prostate (disorder)
        "1259674002",  # Primary endometrioid carcinoma of prostate (disorder)
        "1259676000",  # Primary nodular melanoma of mucous membrane (disorder)
        "1259680005",  # Primary carcinoma of meibomian gland (disorder)
        "1259681009",  # Primary adnexal carcinoma of vulva (disorder)
        "1259682002",  # Primary infiltrating duct carcinoma of pancreas (disorder)
        "1259684001",  # Primary cystic renal cell carcinoma of kidney (disorder)
        "1259686004",  # Primary non-small cell lung carcinoma with neuregulin 1 gene fusion (disorder)
        "1259688003",  # Primary adenoid cystic carcinoma of right lacrimal gland (disorder)
        "1259690002",  # Primary adenoid cystic carcinoma of left lacrimal gland (disorder)
        "1259692005",  # Primary gliosarcoma of brain (disorder)
        "1259693000",  # Primary gliosarcoma of spinal cord (disorder)
        "1259695007",  # Primary carcinoma of epididymis and spermatic cord (disorder)
        "1259697004",  # Primary adenocarcinoma of Meckel diverticulum (disorder)
        "1259698009",  # Primary clear cell adenocarcinoma of peritoneum (disorder)
        "1259700000",  # Primary mucinous cystic neoplasm with invasive carcinoma of pancreas (disorder)
        "1259702008",  # Primary adenoid squamous cell carcinoma of nasopharynx (disorder)
        "1259704009",  # Primary basaloid squamous cell carcinoma of nasopharynx (disorder)
        "1259705005",  # Primary sarcomatoid urothelial carcinoma of urinary bladder (disorder)
        "1259707002",  # Primary sarcomatoid urothelial carcinoma of ureter (disorder)
        "1259709004",  # Primary micropapillary urothelial carcinoma (disorder)
        "1259710009",  # Primary small cell neuroendocrine carcinoma of prostate (disorder)
        "1259711008",  # Primary transitional cell carcinoma of upper urinary tract (disorder)
        "1259712001",  # Primary plasmacytoid urothelial carcinoma of urinary bladder (disorder)
        "1259713006",  # Primary lipid-rich urothelial carcinoma of urinary system (disorder)
        "1259714000",  # Primary clear cell urothelial carcinoma of urinary system (disorder)
        "1259715004",  # Primary overlapping malignant neoplasm of nasopharynx (disorder)
        "1259716003",  # Primary malignant mesothelioma (disorder)
        "1259717007",  # Primary malignant mesothelioma of pelvic peritoneum (disorder)
        "1259718002",  # Primary malignant mesothelioma of mesentery (disorder)
        "1259719005",  # Primary malignant mesothelioma of omentum (disorder)
        "1259720004",  # Primary malignant glioma of brain (disorder)
        "1259721000",  # Primary malignant glioma of spinal cord (disorder)
        "1259722007",  # Primary glioblastoma multiforme of brain (disorder)
        "1259723002",  # Primary glioblastoma multiforme of spinal cord (disorder)
        "1259724008",  # Primary anaplastic astrocytoma of brain (disorder)
        "1259725009",  # Primary medulloblastoma of cerebellum (disorder)
        "1259726005",  # Primary epithelioid hemangioendothelioma of lung (disorder)
        "1259727001",  # Primary non-small cell lung cancer (disorder)
        "1259728006",  # Primary malignant phyllodes tumor of breast (disorder)
        "1259730008",  # Primary oligodendroglioma of brain (disorder)
        "1259731007",  # Primary astrocytoma of spinal cord (disorder)
        "1259732000",  # Primary oligodendroglioma of spinal cord (disorder)
        "1259733005",  # Primary malignant astrocytoma of optic nerve (disorder)
        "1259734004",  # Primary malignant meningioma of optic nerve sheath (disorder)
        "1259735003",  # Primary malignant fibrous histiocytoma of orbit (disorder)
        "1259736002",  # Primary malignant blue nevus of skin (disorder)
        "1259737006",  # Primary tibial adamantinoma (disorder)
        "1259738001",  # Primary mixed glioma (disorder)
        "1259739009",  # Primary malignant glioma of hypothalamus (disorder)
        "1259740006",  # Primary malignant glioma of cerebellum (disorder)
        "1259741005",  # Primary malignant glioma of eye (disorder)
        "1259742003",  # Primary malignant glioma of cerebrum (disorder)
        "1259743008",  # Primary astrocytoma of brain stem (disorder)
        "1259744002",  # Primary glioblastoma multiforme of central nervous system (disorder)
        "1259745001",  # Primary malignant glioma of central nervous system (disorder)
        "1259746000",  # Primary malignant pheochromocytoma (disorder)
        "1259747009",  # Primary malignant insulinoma (disorder)
        "1259748004",  # Primary malignant ameloblastoma of mandible (disorder)
        "1259749007",  # Primary malignant meningioma of meninges of brain (disorder)
        "1259750007",  # Primary astrocytoma of cerebrum (disorder)
        "1259751006",  # Primary malignant odontogenic neoplasm of lower jaw (disorder)
        "1259752004",  # Primary malignant odontogenic neoplasm of upper jaw (disorder)
        "1259753009",  # Primary malignant sex cord tumor of testis (disorder)
        "1259754003",  # Primary squamous non-small cell lung cancer (disorder)
        "1259755002",  # Primary chordoma of clivus (disorder)
        "1259756001",  # Primary medulloepithelioma (disorder)
        "1259757005",  # Primary nonsquamous nonsmall cell neoplasm of lung (disorder)
        "1259758000",  # Primary reactive oxygen species 1 positive non-small cell lung cancer (disorder)
        "1259759008",  # Primary malignant granulosa cell tumor of testis (disorder)
        "1259760003",  # Primary non-small cell lung cancer with mutation in epidermal growth factor receptor (disorder)
        "1259761004",  # Primary non-small cell lung cancer without mutation in epidermal growth factor receptor (disorder)
        "1259762006",  # Primary malignant optic glioma of adulthood (disorder)
        "1259763001",  # Primary primitive neuroectodermal tumor (disorder)
        "1259764007",  # Primary supratentorial primitive neuroectodermal tumor (disorder)
        "1259765008",  # Primary classic medulloblastoma (disorder)
        "1259766009",  # Primary glandular malignant peripheral nerve sheath tumor (disorder)
        "1259767000",  # Primary malignant mesothelioma of pericardium (disorder)
        "1259768005",  # Primary non-small cell lung cancer negative for epidermal growth factor receptor expression (disorder)
        "1259769002",  # Primary malignant glioma of brainstem (disorder)
        "1259770001",  # Primary anaplastic glioma of brain (disorder)
        "1259771002",  # Primary malignant mixed tumor of salivary gland (disorder)
        "1259772009",  # Primary malignant neoplasm of gum and contiguous sites (disorder)
        "1259773004",  # Primary anaplastic astrocytoma of spinal cord (disorder)
        "1259774005",  # Primary adamantinoma of femur (disorder)
        "1259775006",  # Primary malignant medulloepithelioma of ciliary body (disorder)
        "1259776007",  # Primary non-small cell lung cancer, positive for epidermal growth factor receptor expression (disorder)
        "1259777003",  # Primary malignant eccrine spiradenoma of skin (disorder)
        "1259778008",  # Primary malignant chondroid syringoma of skin (disorder)
        "1259779000",  # Primary malignant fibrous histiocytoma of skin (disorder)
        "1259780002",  # Primary superficial malignant fibrous histiocytoma of skin (disorder)
        "1259781003",  # Primary giant cell malignant fibrous histiocytoma of skin (disorder)
        "1259782005",  # Primary inflammatory malignant fibrous histiocytoma of skin (disorder)
        "1259783000",  # Primary epithelioid malignant nerve sheath tumor (disorder)
        "1259784006",  # Primary melanotic malignant nerve sheath tumor (disorder)
        "1259785007",  # Primary malignant granular cell tumor (disorder)
        "1259786008",  # Primary extrarenal rhabdoid tumor (disorder)
        "1259787004",  # Primary malignant optic glioma (disorder)
        "1259788009",  # Primary malignant mesothelioma of parietal peritoneum (disorder)
        "1259789001",  # Primary pleomorphic malignant fibrous histiocytoma of skin (disorder)
        "1259790005",  # Primary perianal Paget disease (disorder)
        "1259791009",  # Primary invasive vulval Paget disease (disorder)
        "1259792002",  # Primary malignant Leydig cell tumor of testis (disorder)
        "1259794001",  # Primary Pancoast tumor (disorder)
        "1259795000",  # Primary Paget disease of vulva (disorder)
        "1259796004",  # Primary Paget disease of anal canal (disorder)
        "1259797008",  # Primary Paget disease of skin of scrotum (disorder)
        "1259798003",  # Primary oligodendroglioma of cerebrum (disorder)
        "1259799006",  # Primary malignant glucagonoma of pancreas (disorder)
        "1259800005",  # Primary malignant gastrinoma of pancreas (disorder)
        "1259801009",  # Primary gastrointestinal stromal neoplasm of overlapping sites of small intestine (disorder)
        "1259802002",  # Primary gastrointestinal stromal neoplasm of overlapping sites of stomach (disorder)
        "1259803007",  # Primary human epidermal growth factor 2 positive gastric cancer (disorder)
        "1259804001",  # Primary chordoma of vertebral column (disorder)
        "1259805000",  # Primary malignant peripheral nerve sheath tumor of gingival mucosa (disorder)
        "1259806004",  # Primary nephroblastoma of right kidney (disorder)
        "1259807008",  # Primary nephroblastoma of left kidney (disorder)
        "1259808003",  # Primary solid neoplasm with neurotrophic receptor tyrosine kinase gene fusion (disorder)
        "1259810001",  # Primary malignant middle ear paraganglioma (disorder)
        "1259812009",  # Primary astroblastoma of brain (disorder)
        "1259813004",  # Primary anaplastic oligoastrocytoma of central nervous system (disorder)
        "1259814005",  # Primary gemistocytic astrocytoma of brain (disorder)
        "1259815006",  # Primary giant cell glioblastoma of brain (disorder)
        "1259816007",  # Primary fibrillary astrocytoma of brain (disorder)
        "1259817003",  # Primary pleomorphic xanthoastrocytoma of brain (disorder)
        "1259818008",  # Primary pilomyxoid astrocytoma of brain (disorder)
        "1259819000",  # Primary anaplastic ganglioglioma of central nervous system (disorder)
        "1259820006",  # Primary reactive oxygen species 1 negative non-small cell lung cancer (disorder)
        "1259821005",  # Primary anaplastic lymphoma kinase fusion oncogene positive non-small cell lung cancer (disorder)
        "1259822003",  # Primary anaplastic lymphoma kinase fusion oncogene negative non-small cell lung cancer (disorder)
        "1259823008",  # Primary chordoma of cervical spine (disorder)
        "1259824002",  # Primary chordoma of lumbar spine (disorder)
        "1259825001",  # Primary chordoma of thoracic spine (disorder)
        "1259826000",  # Primary epithelioid hemangioendothelioma of liver (disorder)
        "1259827009",  # Primary mixed germ cell neoplasm of central nervous system (disorder)
        "1259828004",  # Primary malignant nerve sheath neoplasm of peripheral nerve of abdomen (disorder)
        "1259829007",  # Primary malignant nerve sheath neoplasm of peripheral nerve of upper limb (disorder)
        "1259830002",  # Primary malignant nerve sheath neoplasm of peripheral nerve of head and neck (disorder)
        "1259831003",  # Primary malignant nerve sheath neoplasm of peripheral nerve of pelvis (disorder)
        "1259832005",  # Primary malignant nerve sheath neoplasm of peripheral nerve of thorax (disorder)
        "1259833000",  # Primary malignant perivascular epithelioid cell neoplasm (disorder)
        "1260018001",  # Adenoid cystic carcinoma of hypopharynx (disorder)
        "1260019009",  # Adenoid cystic carcinoma of lung (disorder)
        "1260021004",  # Adenoid squamous cell carcinoma of hypopharynx (disorder)
        "1260024007",  # Adenoid squamous cell carcinoma of larynx (disorder)
        "1260031006",  # Clear cell adenocarcinoma of oropharynx (disorder)
        "1260032004",  # Clear cell adenocarcinoma of lung (disorder)
        "1260033009",  # Clear cell adenocarcinoma of trachea (disorder)
        "1260034003",  # Basal cell adenocarcinoma of oropharynx (disorder)
        "1260035002",  # Carcinoma of ethmoidal sinus (disorder)
        "1260036001",  # Carcinoma of frontal sinus (disorder)
        "1260037005",  # Adenosquamous carcinoma of larynx (disorder)
        "1260038000",  # Adenosquamous carcinoma of trachea (disorder)
        "1260039008",  # Adenosquamous carcinoma of oropharynx (disorder)
        "1260040005",  # Adenosquamous carcinoma of hypopharynx (disorder)
        "1260042002",  # Adenosquamous carcinoma of lung (disorder)
        "1260045000",  # Cystadenocarcinoma of oropharynx (disorder)
        "1260046004",  # Squamous cell carcinoma of ethmoidal sinus (disorder)
        "1260048003",  # Squamous cell carcinoma of frontal sinus (disorder)
        "1260050006",  # Cholangiocarcinoma of intrahepatic biliary tract (disorder)
        "1260056000",  # Clear cell squamous cell carcinoma of lung (disorder)
        "1260057009",  # Mucinous adenocarcinoma of lung (disorder)
        "1260058004",  # Mucinous adenocarcinoma of oropharynx (disorder)
        "1260059007",  # Mucinous bronchiolo-alveolar carcinoma of lung (disorder)
        "1260060002",  # Mucinous adenocarcinoma of trachea (disorder)
        "1260061003",  # Mucoepidermoid carcinoma of oropharynx (disorder)
        "1260062005",  # Mucoepidermoid carcinoma of lung (disorder)
        "1260063000",  # Mucoepidermoid carcinoma of hypopharynx (disorder)
        "1260064006",  # Mucinous cystadenocarcinoma of trachea (disorder)
        "1260065007",  # Mucinous cystadenocarcinoma of lung (disorder)
        "1260066008",  # Papillary adenocarcinoma of trachea (disorder)
        "1260067004",  # Papillary adenocarcinoma of oropharynx (disorder)
        "1260068009",  # Papillary adenocarcinoma of lung (disorder)
        "1260069001",  # Oncocytic carcinoma of oropharynx (disorder)
        "1260070000",  # Non-mucinous bronchiolo-alveolar carcinoma of lung (disorder)
        "1260071001",  # Papillary squamous cell carcinoma of lung (disorder)
        "1260072008",  # Pseudosarcomatous carcinoma of lung (disorder)
        "1260073003",  # Papillary squamous cell carcinoma of larynx (disorder)
        "1260074009",  # Papillary squamous cell carcinoma of hypopharynx (disorder)
        "1260075005",  # Solid carcinoma of trachea (disorder)
        "1260076006",  # Small cell non-keratinizing squamous cell carcinoma of lung (disorder)
        "1260077002",  # Solid carcinoma of lung (disorder)
        "1260079004",  # Squamous cell carcinoma of hypopharyngeal aspect of aryepiglottic fold (disorder)
        "1260080001",  # Squamous cell carcinoma of maxillary sinus (disorder)
        "1260081002",  # Squamous cell carcinoma of postcricoid region (disorder)
        "1260082009",  # Squamous cell carcinoma of posterior wall of hypopharynx (disorder)
        "1260083004",  # Squamous cell carcinoma of piriform sinus (disorder)
        "1260084005",  # Squamous cell carcinoma of vallecula (disorder)
        "1260085006",  # Undifferentiated carcinoma of hypopharynx (disorder)
        "1260086007",  # Endometrioid carcinoma of endometrium (disorder)
        "1260087003",  # Fetal adenocarcinoma of lung (disorder)
        "1260088008",  # Infiltrating duct carcinoma of oropharynx (disorder)
        "1260089000",  # Lymphoepithelial carcinoma of hypopharynx (disorder)
        "1260090009",  # Lymphoepithelial carcinoma of larynx (disorder)
        "1260091008",  # Lymphoepithelial carcinoma of oropharynx (disorder)
        "1260092001",  # Mixed mucinous and non-mucinous bronchiolo-alveolar carcinoma of lung (disorder)
        "1260093006",  # Mixed subtype adenocarcinoma of lung (disorder)
        "1260094000",  # Verrucous carcinoma of hypopharynx (disorder)
        "1260100002",  # Salivary gland type carcinoma of hypopharynx (disorder)
        "1260110006",  # Salivary gland type carcinoma of oropharynx (disorder)
        "1260115001",  # Salivary gland type carcinoma of lung (disorder)
        "1260116000",  # Primary malignant stromal sarcoma of endometrium (disorder)
        "1260217004",  # Myelodysplastic neoplasm with low blasts and SF3B1 mutation (disorder)
        "1260268001",  # Myelodysplastic syndrome with low blasts (disorder)
        "1260270005",  # Hypoplastic myelodysplastic syndrome (disorder)
        "1260406003",  # Myelodysplastic neoplasm with biallelic TP53 inactivation (disorder)
        "1263450003",  # Malignant non-dysgerminomatous germ cell tumor of ovary (disorder)
        "1263465002",  # Malignant neoplasm of soft tissue of finger (disorder)
        "1263466001",  # Malignant neoplasm of soft tissue of toe (disorder)
        "1263467005",  # Malignant neoplasm of soft tissue of hand (disorder)
        "1263468000",  # Malignant neoplasm of soft tissue of foot (disorder)
        "1263469008",  # Malignant neoplasm of soft tissue of thumb (disorder)
        "1263474000",  # Malignant neoplasm of soft tissue of axilla (disorder)
        "1263475004",  # Malignant neoplasm of soft tissue of buttock (disorder)
        "1263476003",  # Malignant neoplasm soft tissue of lower limb (disorder)
        "1263479005",  # Malignant neoplasm of perineum (disorder)
        "1263484004",  # Metastatic endometrioid carcinoma to ovary (disorder)
        "1263601000",  # Metastatic scirrhous carcinoma to breast (disorder)
        "1263639008",  # Metastatic transitional cell carcinoma to urinary bladder (disorder)
        "1263640005",  # Metastatic transitional cell carcinoma to corpus uteri (disorder)
        "1263641009",  # Metastatic transitional cell carcinoma to kidney (disorder)
        "1263642002",  # Metastatic transitional cell carcinoma to left kidney (disorder)
        "1263643007",  # Metastatic transitional cell carcinoma to right kidney (disorder)
        "1263651005",  # Metastatic mucoepidermoid carcinoma to skin (disorder)
        "1263652003",  # Metastatic mucoepidermoid carcinoma to parotid gland (disorder)
        "1263653008",  # Metastatic mucoepidermoid carcinoma to salivary gland (disorder)
        "1263654002",  # Metastatic mucoepidermoid carcinoma to submandibular gland (disorder)
        "1263655001",  # Metastatic transitional cell carcinoma to right renal pelvis (disorder)
        "1263656000",  # Metastatic transitional cell carcinoma to left renal pelvis (disorder)
        "1263657009",  # Metastatic transitional cell carcinoma to ureter (disorder)
        "1263658004",  # Metastatic transitional cell carcinoma to left ureter (disorder)
        "1263659007",  # Metastatic transitional cell carcinoma to renal pelvis (disorder)
        "1263660002",  # Metastatic transitional cell carcinoma to upper urinary tract (disorder)
        "1263786003",  # Metastatic basaloid squamous cell carcinoma to nasopharynx (disorder)
        "1263911005",  # Metastatic carcinoma ex pleomorphic adenoma to parotid gland (disorder)
        "1263912003",  # Metastatic carcinoma ex pleomorphic adenoma to lacrimal gland (disorder)
        "1263913008",  # Metastatic chromophobe renal cell carcinoma (disorder)
        "1263914002",  # Metastatic papillary renal cell carcinoma (disorder)
        "1263999008",  # Metastatic MiT family translocation renal cell carcinoma (disorder)
        "1264066001",  # Metastatic keratinizing squamous cell carcinoma to nasopharynx (disorder)
        "1264067005",  # Metastatic acinar cell carcinoma to pancreas (disorder)
        "1264068000",  # Metastatic lymphoepithelial carcinoma (disorder)
        "1264115008",  # Metastatic metaplastic carcinoma to breast (disorder)
        "1264116009",  # Metastatic adenosquamous carcinoma (disorder)
        "1264117000",  # Metastatic adenosquamous carcinoma to cervix (disorder)
        "1264118005",  # Metastatic oxyphilic adenocarcinoma (disorder)
        "1264258003",  # Metastatic papillary squamous cell carcinoma (disorder)
        "1264340007",  # Familial gastric type 1 neuroendocrine neoplasm (disorder)
        "1264357006",  # Metastatic mucinous cystadenocarcinoma to ovary (disorder)
        "1264358001",  # Metastatic mucinous cystadenocarcinoma to pancreas (disorder)
        "1264364008",  # Metastatic ceruminous adenocarcinoma to skin (disorder)
        "1264367001",  # Metastatic sebaceous adenocarcinoma (disorder)
        "1264369003",  # Metastatic sebaceous adenocarcinoma to eyelid (disorder)
        "1264416003",  # Unclassifiable myelodysplastic syndrome (disorder)
        "1264458000",  # Acute undifferentiated leukemia (disorder)
        "1264495000",  # Metastatic carcinoma to breast (disorder)
        "1264497008",  # Metastatic infiltrating duct carcinoma to breast (disorder)
        "1264499006",  # Metastatic infiltrating duct carcinoma to pancreas (disorder)
        "1264500002",  # Metastatic infiltrating duct carcinoma to prostate (disorder)
        "1264501003",  # Metastatic papillary cystadenocarcinoma to kidney (disorder)
        "1264502005",  # Metastatic mucinous adenocarcinoma to gastrointestinal tract (disorder)
        "126667002",  # Neoplasm of respiratory tract (disorder)
        "1268348008",  # Primary small cell carcinoma of lung (disorder)
        "1268349000",  # Primary endometrioid carcinoma of ovary (disorder)
        "1268350000",  # Primary carcinoma of vagina (disorder)
        "1268351001",  # Primary clear cell carcinoma of kidney (disorder)
        "1268352008",  # Primary carcinoma of pituitary gland (disorder)
        "1268353003",  # Metastatic clear cell adenocarcinoma to ovary (disorder)
        "1268354009",  # Metastatic clear cell adenocarcinoma to peritoneum (disorder)
        "1268386004",  # Metastatic undifferentiated carcinoma to corpus uteri (disorder)
        "1268387008",  # Primary undifferentiated carcinoma of corpus uteri (disorder)
        "1268388003",  # Metastatic undifferentiated carcinoma to esophagus (disorder)
        "1268389006",  # Primary undifferentiated carcinoma of esophagus (disorder)
        "1268390002",  # Metastatic undifferentiated carcinoma to liver and intrahepatic biliary tract (disorder)
        "1268391003",  # Primary undifferentiated carcinoma of liver and intrahepatic biliary tract (disorder)
        "1268392005",  # Metastatic undifferentiated carcinoma to nasal sinus (disorder)
        "1268393000",  # Metastatic undifferentiated carcinoma to nasopharynx (disorder)
        "1268394006",  # Metastatic undifferentiated carcinoma to ovary (disorder)
        "1268395007",  # Metastatic undifferentiated carcinoma to stomach (disorder)
        "1268396008",  # Primary undifferentiated carcinoma of stomach (disorder)
        "1268458008",  # Primary anaplastic thyroid carcinoma (disorder)
        "1268459000",  # Primary adrenal cortical carcinoma (disorder)
        "1268460005",  # Primary parathyroid carcinoma (disorder)
        "1268461009",  # Primary cholangiocarcinoma of biliary tract (disorder)
        "1268462002",  # Primary osteosarcoma of bone (disorder)
        "1268484002",  # Primary synovial sarcoma (disorder)
        "1268485001",  # Primary non-seminomatous germ cell neoplasm of testis (disorder)
        "1268486000",  # Primary ganglioneuroblastoma (disorder)
        "1268487009",  # Primary carcinoma of nasopharynx (disorder)
        "1268488004",  # Primary carcinoma of stomach due to Epstein-Barr virus disease (disorder)
        "1268499002",  # Primary collecting duct carcinoma of kidney (disorder)
        "1268500006",  # Primary chromophobe renal cell carcinoma (disorder)
        "1268501005",  # Primary tubulocystic renal cell carcinoma (disorder)
        "1268502003",  # Primary embryonal sarcoma of liver (disorder)
        "1268503008",  # Primary papillary renal cell carcinoma (disorder)
        "1268518009",  # Primary small cell neuroendocrine carcinoma of urinary bladder (disorder)
        "1268519001",  # Primary neuroblastoma (disorder)
        "1268526001",  # Primary carcinosarcoma of ovary (disorder)
        "1268528000",  # Primary olfactory neuroblastoma (disorder)
        "1268531004",  # Primary alveolar soft part sarcoma (disorder)
        "1268532006",  # Primary carcinoma of ampulla of Vater (disorder)
        "1268533001",  # Primary Ewing sarcoma of bone (disorder)
        "1268534007",  # Primary Ewing sarcoma of soft tissue (disorder)
        "1268536009",  # Primary adenoid basal carcinoma of cervix uteri (disorder)
        "1268537000",  # Primary adenoid cystic carcinoma of cervix uteri (disorder)
        "1268538005",  # Primary clear cell adenocarcinoma of ovary (disorder)
        "1268539002",  # Primary metaplastic carcinoma of breast (disorder)
        "1268540000",  # Primary small cell carcinoma of ovary (disorder)
        "1268541001",  # Primary squamous cell carcinoma of liver and intrahepatic biliary tract (disorder)
        "1268542008",  # Primary invasive intraductal papillary-mucinous carcinoma of pancreas (disorder)
        "1268543003",  # Primary clear cell sarcoma of kidney (disorder)
        "1268544009",  # Primary carcinoma of gallbladder and extrahepatic biliary tract (disorder)
        "1268545005",  # Primary transitional cell carcinoma of corpus uteri (disorder)
        "1268546006",  # Primary mucinous cystadenocarcinoma of pancreas (disorder)
        "1268547002",  # Primary epithelioid sarcoma (disorder)
        "1268548007",  # Primary salivary gland type carcinoma of esophagus (disorder)
        "1268561007",  # Primary serous cystadenocarcinoma of pancreas (disorder)
        "1268563005",  # Primary acinar cell carcinoma of pancreas (disorder)
        "1268631009",  # Primary glassy cell carcinoma of cervix uteri (disorder)
        "1268632002",  # Primary squamous cell carcinoma of gallbladder and extrahepatic biliary tract (disorder)
        "1268633007",  # Primary squamous cell carcinoma of rectum (disorder)
        "1268634001",  # Primary squamous cell carcinoma of stomach (disorder)
        "1268635000",  # Primary squamous cell carcinoma of colon (disorder)
        "1268636004",  # Primary squamous cell carcinoma of small intestine (disorder)
        "1268637008",  # Primary MiT family translocation renal cell carcinoma (disorder)
        "1268638003",  # Primary adenosarcoma of cervix uteri (disorder)
        "1268639006",  # Primary carcinosarcoma of cervix uteri (disorder)
        "1268640008",  # Primary lymphoepithelial carcinoma (disorder)
        "1268641007",  # Primary renal mucinous tubular and spindle cell carcinoma (disorder)
        "1268642000",  # Primary renal medullary carcinoma (disorder)
        "1268697005",  # Primary adenosarcoma of corpus uteri (disorder)
        "1268698000",  # Primary solid pseudopapillary carcinoma of pancreas (disorder)
        "1268699008",  # Primary pilomatrix carcinoma of skin (disorder)
        "1268700009",  # Primary spermatocytic seminoma of testis (disorder)
        "1268701008",  # Primary gliosarcoma of central nervous system (disorder)
        "1268702001",  # Primary choroid plexus carcinoma (disorder)
        "1268703006",  # Primary peripheral neuroectodermal tumor (disorder)
        "1268704000",  # Primary malignant mesothelioma of pleura (disorder)
        "1268705004",  # Primary malignant sex cord tumor of ovary (disorder)
        "1268706003",  # Primary malignant granulosa cell tumor of ovary (disorder)
        "1268707007",  # Primary malignant dysgerminoma of ovary (disorder)
        "1268710000",  # Primary adamantinoma of long bone (disorder)
        "1268711001",  # Primary hepatoblastoma of liver (disorder)
        "1268712008",  # Primary malignant mesothelioma of peritoneum (disorder)
        "1268714009",  # Primary nephroblastoma (disorder)
        "1268715005",  # Primary malignant fibrous histiocytoma (disorder)
        "1268716006",  # Primary medulloblastoma (disorder)
        "1268717002",  # Primary oligodendroglioma (disorder)
        "1268718007",  # Primary malignant thymoma (disorder)
        "1268719004",  # Primary anaplastic astrocytoma of central nervous system (disorder)
        "1268720005",  # Primary malignant peripheral nerve sheath neoplasm with perineurial differentiation (disorder)
        "1268721009",  # Primary malignant seminoma of testis (disorder)
        "1268881002",  # Primary malignant melanoma of skin (disorder)
        "1268885006",  # Primary ependymoblastoma (disorder)
        "1268888008",  # Primary pineal parenchymal tumor of intermediate differentiation (disorder)
        "1268897007",  # Primary theca steroid producing cell malignant neoplasm of ovary (disorder)
        "1268898002",  # Primary mixed oligoastrocytoma (disorder)
        "1268899005",  # Primary pleuropulmonary blastoma (disorder)
        "1268900000",  # Primary linitis plastica of stomach (disorder)
        "1268901001",  # Primary retinoblastoma of retina (disorder)
        "1268902008",  # Primary pulmonary blastoma (disorder)
        "1268904009",  # Primary epithelioid hemangioendothelioma (disorder)
        "1268905005",  # Primary malignant peripheral nerve sheath tumor (disorder)
        "1268906006",  # Primary Klatskin tumor (disorder)
        "1268908007",  # Primary Paget disease of nipple (disorder)
        "1268909004",  # Primary carcinofibroma of corpus uteri (disorder)
        "1268911008",  # Primary pancreatoblastoma (disorder)
        "1268955002",  # Primary malignant Triton tumor (disorder)
        "1268956001",  # Primary extramammary Paget disease of skin (disorder)
        "1268957005",  # Primary pineoblastoma (disorder)
        "1268958000",  # Primary primitive neuroectodermal tumor of corpus uteri (disorder)
        "1268959008",  # Primary primitive neuroectodermal tumor of cervix uteri (disorder)
        "1268960003",  # Primary astroblastoma of central nervous system (disorder)
        "1268961004",  # Primary diffuse astrocytoma of brain (disorder)
        "1268962006",  # Primary polyembryoma (disorder)
        "1268964007",  # Primary anaplastic oligodendroglioma of central nervous system (disorder)
        "1268965008",  # Primary anaplastic ependymoma of central nervous system (disorder)
        "1268966009",  # Primary gliomatosis cerebri (disorder)
        "1269042005",  # Primary protoplasmic astrocytoma of brain (disorder)
        "1269043000",  # Primary malignant rhabdoid tumor (disorder)
        "1269044006",  # Primary desmoplastic small round cell tumor (disorder)
        "1269045007",  # Primary mixed germ cell tumor (disorder)
        "1269046008",  # Primary chordoma (disorder)
        "1269047004",  # Primary papillary tumor of pineal region (disorder)
        "1269048009",  # Primary diffuse intrinsic pontine glioma (disorder)
        "1269049001",  # Primary giant cell glioblastoma of central nervous system (disorder)
        "1269050001",  # Primary gemistocytic astrocytoma of central nervous system (disorder)
        "1269051002",  # Primary desmoplastic nodular medulloblastoma of brain (disorder)
        "1269052009",  # Primary large cell medulloblastoma of brain (disorder)
        "1269121005",  # Metastatic rhabdomyosarcoma (disorder)
        "1269122003",  # Metastatic carcinoma to nasopharynx (disorder)
        "1269123008",  # Metastatic squamous cell carcinoma to rectum (disorder)
        "1269125001",  # Metastatic squamous cell carcinoma to small intestine (disorder)
        "1269126000",  # Metastatic squamous cell carcinoma to stomach (disorder)
        "1269273000",  # Malignancy diagnosed during pregnancy (disorder)
        "1269347008",  # Primary carcinosarcoma of corpus uteri (disorder)
        "1269412004",  # Primary medulloepithelioma of central nervous system (disorder)
        "1269413009",  # Primary pilomyxoid astrocytoma (disorder)
        "1269414003",  # Primary fibrillary astrocytoma of central nervous system (disorder)
        "1269458003",  # Malignant peripheral nerve sheath neoplasm of peripheral nerve of lumbar spine (disorder)
        "127070008",  # Malignant histiocytic disorder (disorder)
        "127220001",  # Malignant lymphoma of lymph nodes (disorder)
        "127225006",  # Chronic myelomonocytic leukemia (disorder)
        "1279841001",  # Undifferentiated carcinoma with osteoclast-like giant cells of pancreas (disorder)
        "1279882001",  # Metastatic embryonal rhabdomyosarcoma (disorder)
        "128041000119107",  # Primary adenocarcinoma of distal third of esophagus (disorder)
        "1281812003",  # Malignant neoplasm of breast in partial remission (disorder)
        "128462008",  # Metastatic malignant neoplasm (disorder)
        "128465005",  # Metastatic malignant neoplasm to articular cartilage (disorder)
        "128466006",  # Primary malignant neoplasm of articular cartilage (disorder)
        "128467002",  # Neoplasm of uncertain behavior of articular cartilage (disorder)
        "1284849005",  # Primary malignant Sertoli-Leydig cell tumor of ovary (disorder)
        "1285321001",  # Nevoid melanoma (disorder)
        "1286863002",  # Metastatic malignant neoplasm to cecum and/or colon and/or rectum (disorder)
        "1286877004",  # Malignant neoplasm of cecum and/or colon and/or rectum (disorder)
        "1286879001",  # Malignant neoplasm of cecum and/or colon and/or rectum expressing human epidermal growth factor 2 (disorder)
        "1286896000",  # Metastatic collecting duct renal cell carcinoma (disorder)
        "1286898004",  # Metastatic cystic renal cell carcinoma (disorder)
        "1287197000",  # Primary well-differentiated neuroendocrine tumor of thymus (disorder)
        "1287332002",  # Primary Merkel cell carcinoma of skin of face (disorder)
        "1287333007",  # Primary Merkel cell carcinoma of skin of lower limb (disorder)
        "1287334001",  # Primary Merkel cell carcinoma of skin of left lower limb (disorder)
        "1287335000",  # Primary Merkel cell carcinoma of skin of right lower limb (disorder)
        "1287336004",  # Primary Merkel cell carcinoma of skin of upper limb (disorder)
        "1288003006",  # Primary medullary thyroid carcinoma (disorder)
        "1288023007",  # Hereditary well-differentiated neuroendocrine tumor of small intestine (disorder)
        "1288024001",  # Well-differentiated neuroendocrine tumor of corpus uteri (disorder)
        "1288032009",  # Well-differentiated neuroendocrine tumor of kidney (disorder)
        "1288033004",  # Well-differentiated neuroendocrine tumor of sigmoid colon (disorder)
        "1288045008",  # Well-differentiated neuroendocrine tumor (disorder)
        "128875000",  # Primary cutaneous CD30 antigen positive large T-cell lymphoma (disorder)
        "1290060007",  # Metastatic well-differentiated neuroendocrine tumor (disorder)
        "1290064003",  # Well-differentiated neuroendocrine tumor of stomach (disorder)
        "1290067005",  # Well-differentiated neuroendocrine tumor of small intestine (disorder)
        "1290068000",  # Well-differentiated neuroendocrine tumor of rectum (disorder)
        "1290116009",  # Gastric inhibitory peptide-secreting tumor of pancreas (disorder)
        "1290117000",  # Parathyroid hormone-related peptide-secreting tumor of pancreas (disorder)
        "1290125003",  # Neuroendocrine neoplasm, malignant (disorder)
        "1290274008",  # Well-differentiated neuroendocrine tumor of colon (disorder)
        "1290690001",  # Metastatic cholangiocarcinoma (disorder)
        "1290695006",  # Metastatic apocrine adenocarcinoma (disorder)
        "1290697003",  # Metastatic eccrine porocarcinoma (disorder)
        "1290698008",  # Metastatic pilomatrix carcinoma (disorder)
        "1293166002",  # Malignant neoplasm of vestibulocochlear nerve structure (disorder)
        "13048006",  # Orbital lymphoma (disorder)
        "1306293004",  # Primary carcinoma of breast (disorder)
        "1306368003",  # Sarcoma of axillary tail of breast (disorder)
        "1306369006",  # Sarcoma of central portion of breast (disorder)
        "1306370007",  # Sarcoma of lower inner quadrant of breast (disorder)
        "1306371006",  # Sarcoma of upper outer quadrant of breast (disorder)
        "1306372004",  # Sarcoma of lower outer quadrant of breast (disorder)
        "1306373009",  # Sarcoma of upper inner quadrant of breast (disorder)
        "1306374003",  # Neoplasm of uncertain behavior of axillary tail of breast (disorder)
        "1306375002",  # Neoplasm of uncertain behavior of central portion of breast (disorder)
        "1306376001",  # Neoplasm of uncertain behavior of lower inner quadrant of breast (disorder)
        "1306377005",  # Neoplasm of uncertain behavior of lower outer quadrant of breast (disorder)
        "1306385001",  # Neoplasm of uncertain behavior of upper inner quadrant of breast (disorder)
        "1306387009",  # Neoplasm of uncertain behavior of upper outer quadrant of breast (disorder)
        "1306489008",  # Primary infiltrating duct carcinoma of lower inner quadrant of right breast (disorder)
        "1306490004",  # Primary infiltrating duct carcinoma of lower outer quadrant of right breast (disorder)
        "1306491000",  # Primary infiltrating duct carcinoma of upper outer quadrant of right breast (disorder)
        "1306492007",  # Primary infiltrating duct carcinoma of upper inner quadrant of right breast (disorder)
        "1306493002",  # Primary infiltrating duct carcinoma of lower inner quadrant of left breast (disorder)
        "1306494008",  # Primary infiltrating duct carcinoma of lower outer quadrant of left breast (disorder)
        "1306495009",  # Primary infiltrating duct carcinoma of upper inner quadrant of left breast (disorder)
        "1306496005",  # Primary infiltrating duct carcinoma of upper outer quadrant of left breast (disorder)
        "1306497001",  # Primary infiltrating duct carcinoma of central portion of right breast (disorder)
        "1306498006",  # Primary infiltrating duct carcinoma of central portion of left breast (disorder)
        "1306499003",  # Primary infiltrating duct carcinoma of axillary tail of right breast (disorder)
        "1306500007",  # Primary infiltrating duct carcinoma of axillary tail of left breast (disorder)
        "1306507005",  # Primary malignant neoplasm of central portion of right breast (disorder)
        "1306508000",  # Primary malignant neoplasm of central portion of left breast (disorder)
        "1306509008",  # Primary malignant neoplasm of central portion of breast (disorder)
        "1306511004",  # Primary malignant neoplasm of bilateral breasts (disorder)
        "1306512006",  # Primary inflammatory carcinoma of breast (disorder)
        "1306513001",  # Recurrent primary malignant neoplasm of right breast (disorder)
        "1306514007",  # Recurrent primary malignant neoplasm of left breast (disorder)
        "1306516009",  # Primary lobular carcinoma of left breast (disorder)
        "1306517000",  # Primary lobular carcinoma of right breast (disorder)
        "1306541002",  # Primary malignant neoplasm of axillary tail of left breast (disorder)
        "1306542009",  # Primary malignant neoplasm of axillary tail of right breast (disorder)
        "1306543004",  # Metastatic malignant neoplasm to central portion of breast (disorder)
        "1306544005",  # Metastatic malignant neoplasm to lower inner quadrant of breast (disorder)
        "1306545006",  # Metastatic malignant neoplasm to lower outer quadrant of breast (disorder)
        "1306546007",  # Metastatic malignant neoplasm to upper inner quadrant of breast (disorder)
        "1306547003",  # Metastatic malignant neoplasm to upper outer quadrant of breast (disorder)
        "1306548008",  # Malignant neoplasm of central portion of breast (disorder)
        "1306552008",  # Infiltrating duct carcinoma of axillary tail of left breast (disorder)
        "1306554009",  # Infiltrating duct carcinoma of axillary tail of right breast (disorder)
        "1306557002",  # Infiltrating duct carcinoma of central portion of left breast (disorder)
        "1306558007",  # Infiltrating duct carcinoma of central portion of right breast (disorder)
        "1306564000",  # Lobular carcinoma of right breast (disorder)
        "1306565004",  # Lobular carcinoma of left breast (disorder)
        "1306566003",  # Infiltrating duct carcinoma of upper outer quadrant of right breast (disorder)
        "1306567007",  # Infiltrating duct carcinoma of upper outer quadrant of left breast (disorder)
        "1306570006",  # Infiltrating duct carcinoma of upper inner quadrant of right breast (disorder)
        "1306571005",  # Infiltrating duct carcinoma of upper inner quadrant of left breast (disorder)
        "1306573008",  # Infiltrating duct carcinoma of lower outer quadrant of right breast (disorder)
        "1306574002",  # Infiltrating duct carcinoma of lower outer quadrant of left breast (disorder)
        "1306575001",  # Infiltrating duct carcinoma of left breast (disorder)
        "1306576000",  # Infiltrating duct carcinoma of right breast (disorder)
        "1306579007",  # Infiltrating duct carcinoma of lower inner quadrant of right breast (disorder)
        "1306580005",  # Infiltrating duct carcinoma of lower inner quadrant of left breast (disorder)
        "130891000119106",  # Chordoma of coccyx (disorder)
        "130911000119108",  # Chordoma of pelvis (disorder)
        "130941000119107",  # Primary chordoma of bone of skull (disorder)
        "1332427007",  # Disseminated Langerhans cell histiocytosis of spleen (disorder)
        "1332428002",  # Disseminated Langerhans cell histiocytosis of lymph nodes of multiple sites (disorder)
        "1332429005",  # Disseminated Langerhans cell histiocytosis of lymph nodes of thorax (disorder)
        "1332430000",  # Disseminated Langerhans cell histiocytosis of pelvic lymph nodes (disorder)
        "1332431001",  # Disseminated Langerhans cell histiocytosis of abdominal lymph nodes (disorder)
        "1332485001",  # Metastatic endometrial stromal sarcoma (disorder)
        "133751000119102",  # Lymphoma of colon (disorder)
        "133851000119107",  # Merkel cell carcinoma of neck (disorder)
        "133861000119109",  # Merkel cell carcinoma of scalp (disorder)
        "133891000119102",  # Merkel cell carcinoma of trunk (disorder)
        "1340074000",  # Malignant vagal paraganglioma (disorder)
        "134312002",  # Odontogenic ghost cell carcinoma (disorder)
        "134335004",  # Solitary fibrous tumor (disorder)
        "134421000",  # Pathologic fracture of bone at site of metastatic neoplasm (disorder)
        "134871000119100",  # Poorly differentiated malignant carcinoid tumor (disorder)
        "135091000119106",  # Secondary neuroendocrine carcinoma of peritoneum (disorder)
        "135931000119106",  # Merkel cell carcinoma of buttock (disorder)
        "136641000119101",  # Torre-Muir syndrome co-occurrent with malignant sebaceous neoplasm (disorder)
        "143401000119106",  # Malignant carcinoid tumor of pancreas (disorder)
        "143411000119109",  # Liposarcoma of retroperitoneum (disorder)
        "145298771000119104",  # Malignant carcinoid tumor of left lung (disorder)
        "145501000119108",  # Metastatic malignant neoplasm to breast (disorder)
        "145511000119106",  # Metastatic malignant neoplasm to genital organ (disorder)
        "145831000119103",  # Primary malignant germ cell neoplasm (disorder)
        "147101000119108",  # Primary malignant astrocytoma of central nervous system (disorder)
        "147131000119101",  # Glioblastoma multiforme of central nervous system (disorder)
        "148911000119107",  # Primary malignant neoplasm of abdomen (disorder)
        "15635721000119108",  # Primary malignant neoplasm of both ovaries (disorder)
        "15636991000119103",  # Renal cell carcinoma of bilateral kidneys (disorder)
        "15930821000119105",  # Metastatic malignant neoplasm to bilateral ovaries (disorder)
        "15930861000119100",  # Carcinosarcoma of left ovary (disorder)
        "15930901000119106",  # Carcinosarcoma of right ovary (disorder)
        "15930941000119108",  # Carcinosarcoma of bilateral ovaries (disorder)
        "15931021000119108",  # Primary sarcoma of left ovary (disorder)
        "15931061000119103",  # Primary sarcoma of right ovary (disorder)
        "15950061000119105",  # Primary malignant neoplasm of skin of right breast (disorder)
        "15950221000119108",  # Primary malignant neoplasm of skin of left breast (disorder)
        "15951021000119107",  # Malignant melanoma of skin of bilateral lower limbs (disorder)
        "15951061000119102",  # Malignant melanoma of skin of bilateral upper limbs (disorder)
        "15955101000119101",  # Primary malignant neoplasm of right greater vestibular gland (disorder)
        "15955141000119104",  # Primary malignant neoplasm of left greater vestibular gland (disorder)
        "15955901000119104",  # Primary small cell carcinoma of right lung (disorder)
        "15955941000119102",  # Primary small cell carcinoma of left lung (disorder)
        "15956181000119102",  # Metastatic adenocarcinoma to bilateral lungs (disorder)
        "15956221000119105",  # Metastatic adenocarcinoma to right lung (disorder)
        "15956261000119100",  # Metastatic adenocarcinoma to left lung (disorder)
        "15956341000119105",  # Adenocarcinoma of left lung (disorder)
        "15956381000119100",  # Adenocarcinoma of right lung (disorder)
        "15956661000119102",  # Metastatic small cell carcinoma to right lung (disorder)
        "15956701000119109",  # Metastatic small cell carcinoma to left lung (disorder)
        "15956741000119106",  # Metastatic small cell carcinoma to bilateral lungs (disorder)
        "15957141000119109",  # Metastatic squamous cell carcinoma to bilateral lungs (disorder)
        "15957181000119104",  # Metastatic squamous cell carcinoma to right lung (disorder)
        "15957221000119107",  # Metastatic squamous cell carcinoma to left lung (disorder)
        "15957261000119102",  # Kaposi sarcoma of bilateral lungs (disorder)
        "15958021000119103",  # Primary sarcoma of left kidney (disorder)
        "15958101000119106",  # Primary sarcoma of right kidney (disorder)
        "15958181000119103",  # Metastatic adenocarcinoma to bilateral kidneys (disorder)
        "15958221000119106",  # Metastatic adenocarcinoma to left kidney (disorder)
        "15958261000119101",  # Metastatic adenocarcinoma to right kidney (disorder)
        "15958301000119109",  # Metastatic squamous cell carcinoma to right kidney (disorder)
        "15958381000119101",  # Metastatic squamous cell carcinoma to left kidney (disorder)
        "15958661000119103",  # Metastatic malignant melanoma to left kidney (disorder)
        "15958701000119105",  # Metastatic malignant melanoma to bilateral kidneys (disorder)
        "15958741000119107",  # Metastatic malignant melanoma to right kidney (disorder)
        "15959061000119101",  # Transitional cell carcinoma of left kidney (disorder)
        "15959101000119103",  # Transitional cell carcinoma of right kidney (disorder)
        "15977551000119100",  # Metastatic malignant neoplasm to bilateral eyes (disorder)
        "15977591000119105",  # Metastatic malignant neoplasm to right eye (disorder)
        "15977631000119105",  # Metastatic malignant neoplasm to left eye (disorder)
        "16091131000119108",  # Primary angiosarcoma of skin of forehead (disorder)
        "16091171000119106",  # Angiosarcoma of skin of cheek (disorder)
        "16091251000119109",  # Primary angiosarcoma of skin of nose (disorder)
        "16091371000119108",  # Primary angiosarcoma of skin of temporal region (disorder)
        "16091411000119109",  # Primary angiosarcoma of skin of lip (disorder)
        "16228971000119104",  # Primary malignant neoplasm of soft tissue of left hip (disorder)
        "16229011000119100",  # Primary malignant neoplasm of soft tissue of right hip (disorder)
        "16260391000119109",  # Primary malignant neoplasm of soft tissues of left shoulder (disorder)
        "16260431000119104",  # Primary malignant neoplasm of soft tissues of right shoulder (disorder)
        "16414611000119100",  # Primary malignant neoplasm of peripheral nerves of left upper limb (disorder)
        "1651000119109",  # Primary adenocarcinoma of pancreas (disorder)
        "16527441000119105",  # Primary non-small cell carcinoma of lower lobe of left lung (disorder)
        "16527481000119100",  # Primary non-small cell carcinoma of middle lobe of right lung (disorder)
        "16527521000119100",  # Primary non-small cell carcinoma of upper lobe of left lung (disorder)
        "16527561000119105",  # Primary non-small cell carcinoma of lower lobe of right lung (disorder)
        "16527601000119105",  # Primary non-small cell carcinoma of right lung (disorder)
        "16527641000119107",  # Primary non-small cell carcinoma of left lung (disorder)
        "16527721000119108",  # Primary non-small cell carcinoma of upper lobe of right lung (disorder)
        "16636051000119105",  # Primary malignant gastrointestinal stromal neoplasm of colon (disorder)
        "16636101000119105",  # Primary malignant gastrointestinal stromal neoplasm of rectum (disorder)
        "16637101000119107",  # Primary sarcoma of small intestine (disorder)
        "16699241000119100",  # Kaposi sarcoma of toe (disorder)
        "1671000119100",  # Metastatic squamous cell carcinoma to lymph node (disorder)
        "16775311000119107",  # Mycosis fungoides of facial lymph node (disorder)
        "1681000119102",  # Metastatic adenocarcinoma to lymph node (disorder)
        "16823941000119108",  # Primary malignant gastrointestinal stromal neoplasm of pancreas (disorder)
        "16845301000119108",  # Primary glioblastoma multiforme of cerebellum (disorder)
        "16845341000119105",  # Primary glioblastoma multiforme of brainstem (disorder)
        "16845381000119100",  # Primary chondrosarcoma of sternum (disorder)
        "16905141000119107",  # Primary malignant neoplasm of skin of right upper eyelid (disorder)
        "16905341000119105",  # Primary malignant neoplasm of skin of right lower eyelid (disorder)
        "16905541000119102",  # Primary malignant neoplasm of skin of left lower eyelid (disorder)
        "16905581000119107",  # Primary malignant neoplasm of skin of left upper eyelid (disorder)
        "1701000119104",  # Primary adenocarcinoma of colon (disorder)
        "18121000119104",  # Primary squamous cell carcinoma of palatine tonsil (disorder)
        "184741000119107",  # Primary adenocarcinoma of anus (disorder)
        "184781000119102",  # Primary adenocarcinoma of cervix uteri (disorder)
        "184801000119103",  # Primary adenocarcinoma of lacrimal gland (disorder)
        "184861000119102",  # Primary adenocarcinoma of nasopharynx (disorder)
        "184871000119108",  # Primary adenocarcinoma of pelvis (disorder)
        "184881000119106",  # Primary adenocarcinoma of rectosigmoid junction (disorder)
        "184891000119109",  # Primary adenocarcinoma of small intestine (disorder)
        "187601000",  # Malignant neoplasm of upper lip, lipstick area (disorder)
        "187604008",  # Malignant neoplasm of lower lip, external (disorder)
        "187606005",  # Malignant tumor of upper labial mucosa (disorder)
        "187608006",  # Malignant tumor of frenum of upper lip (disorder)
        "187613005",  # Malignant neoplasm of lower lip, buccal aspect (disorder)
        "187614004",  # Malignant tumor of frenum of lower lip (disorder)
        "187622006",  # Malignant tumor of labial mucosa (disorder)
        "187631006",  # Malignant neoplasm of base of tongue dorsal surface (disorder)
        "187633009",  # Malignant neoplasm of dorsal surface of tongue (disorder)
        "187634003",  # Malignant tumor of anterior two-thirds of tongue - dorsal surface (disorder)
        "187635002",  # Malignant neoplasm of midline of tongue (disorder)
        "187637005",  # Malignant neoplasm of tongue, tip and lateral border (disorder)
        "187640005",  # Malignant tumor of anterior two-thirds of tongue - ventral surface (disorder)
        "187641009",  # Malignant tumor of frenum linguae (disorder)
        "187644001",  # Malignant tumor of junctional zone of tongue (disorder)
        "187652003",  # Malignant tumor of anterior floor of mouth (disorder)
        "187653008",  # Malignant tumor of lateral floor of mouth (disorder)
        "187658004",  # Malignant tumor of vestibule of mouth (disorder)
        "187659007",  # Malignant tumor of upper buccal sulcus (disorder)
        "187660002",  # Malignant tumor of lower buccal sulcus (disorder)
        "187661003",  # Malignant tumor of upper labial sulcus (disorder)
        "187662005",  # Malignant tumor of lower labial sulcus (disorder)
        "187666008",  # Malignant neoplasm of junction of hard and soft palate (disorder)
        "187675005",  # Malignant tumor of tonsillar pillar (disorder)
        "187681002",  # Malignant neoplasm of anterior epiglottis (disorder)
        "187682009",  # Malignant neoplasm of epiglottis, free border (disorder)
        "187683004",  # Malignant neoplasm of glossoepiglottic fold (disorder)
        "187685006",  # Malignant neoplasm of junctional region of epiglottis (disorder)
        "187688008",  # Malignant tumor of posterior wall of oropharynx (disorder)
        "187692001",  # Malignant tumor of nasopharynx (disorder)
        "187693006",  # Malignant tumor of posterior wall of nasopharynx (disorder)
        "187694000",  # Malignant tumor of adenoid (disorder)
        "187697007",  # Malignant tumor of pharyngeal recess (disorder)
        "187698002",  # Malignant tumor of opening of auditory tube (disorder)
        "187700006",  # Malignant tumor of anterior wall of nasopharynx (disorder)
        "187701005",  # Malignant neoplasm of floor of nasopharynx (disorder)
        "187708004",  # Malignant tumor aryepiglottic fold - hypopharyngeal aspect (disorder)
        "187709007",  # Malignant neoplasm of posterior pharynx (disorder)
        "187716008",  # Malignant tumor of Waldeyer's ring (disorder)
        "187722004",  # Malignant tumor of cervical part of esophagus (disorder)
        "187723009",  # Malignant tumor of thoracic part of esophagus (disorder)
        "187724003",  # Malignant tumor of abdominal part of esophagus (disorder)
        "187725002",  # Malignant tumor of upper third of esophagus (disorder)
        "187726001",  # Malignant tumor of middle third of esophagus (disorder)
        "187727005",  # Malignant tumor of lower third of esophagus (disorder)
        "187732006",  # Malignant tumor of cardia (disorder)
        "187733001",  # Malignant neoplasm of cardiac orifice of stomach (disorder)
        "187734007",  # Malignant neoplasm of cardioesophageal junction of stomach (disorder)
        "187736009",  # Malignant tumor of pylorus (disorder)
        "187738005",  # Malignant neoplasm of pyloric canal of stomach (disorder)
        "187740000",  # Malignant tumor of pyloric antrum (disorder)
        "187741001",  # Malignant tumor of fundus of stomach (disorder)
        "187742008",  # Malignant tumor of body of stomach (disorder)
        "187752007",  # Malignant tumor of Meckel's diverticulum (disorder)
        "187760008",  # Malignant neoplasm of rectum, rectosigmoid junction and anus (disorder)
        "187769009",  # Primary carcinoma of liver (disorder)
        "187773007",  # Malignant neoplasm of interlobular bile ducts (disorder)
        "187776004",  # Malignant neoplasm of intrahepatic canaliculi (disorder)
        "187777008",  # Malignant neoplasm of intrahepatic gall duct (disorder)
        "187784000",  # Malignant neoplasm of hepatic duct (disorder)
        "187786003",  # Malignant neoplasm of sphincter of Oddi (disorder)
        "187791002",  # Malignant tumor of body of pancreas (disorder)
        "187792009",  # Malignant tumor of tail of pancreas (disorder)
        "187793004",  # Malignant tumor of pancreatic duct (disorder)
        "187794005",  # Malignant tumor of Islets of Langerhans (disorder)
        "187798008",  # Malignant neoplasm of ectopic pancreatic tissue (disorder)
        "187801002",  # Malignant tumor of peritoneum and retroperitoneum (disorder)
        "187803004",  # Malignant neoplasm of perinephric tissue (disorder)
        "187804005",  # Malignant neoplasm of retrocecal tissue (disorder)
        "187809000",  # Malignant neoplasm of mesocolon (disorder)
        "187810005",  # Malignant neoplasm of mesocecum (disorder)
        "187811009",  # Malignant neoplasm of mesorectum (disorder)
        "187814001",  # Malignant neoplasm of the pouch of Douglas (disorder)
        "187821001",  # Angiosarcoma of spleen (disorder)
        "187822008",  # Fibrosarcoma of spleen (disorder)
        "187824009",  # Malignant neoplasm of overlapping sites of digestive system (disorder)
        "187828007",  # Malignant neoplasm of nasal cavities, middle ear and accessory sinuses (disorder)
        "187829004",  # Malignant neoplasm of cartilage of nose (disorder)
        "187830009",  # Malignant neoplasm of nasal conchae (disorder)
        "187831008",  # Malignant tumor of nasal vestibule (disorder)
        "187833006",  # Malignant neoplasm of auditory tube, middle ear and mastoid air cells (disorder)
        "187834000",  # Malignant tumor of Eustachian tube (disorder)
        "187835004",  # Malignant tumor of tympanic cavity (disorder)
        "187836003",  # Malignant tumor of tympanic antrum (disorder)
        "187841006",  # Malignant tumor of glottis (disorder)
        "187842004",  # Malignant tumor of supraglottis (disorder)
        "187843009",  # Malignant neoplasm of arytenoid cartilage (disorder)
        "187844003",  # Malignant neoplasm of cricoid cartilage (disorder)
        "187845002",  # Malignant neoplasm of cuneiform cartilage (disorder)
        "187846001",  # Malignant neoplasm of thyroid cartilage (disorder)
        "187853005",  # Malignant neoplasm of cartilage of trachea (disorder)
        "187854004",  # Malignant neoplasm of mucosa of trachea (disorder)
        "187857006",  # Malignant neoplasm of carina of bronchus (disorder)
        "187861000",  # Malignant neoplasm of upper lobe bronchus (disorder)
        "187862007",  # Malignant neoplasm of upper lobe of lung (disorder)
        "187865009",  # Malignant neoplasm of middle lobe bronchus (disorder)
        "187869003",  # Malignant neoplasm of lower lobe bronchus (disorder)
        "187870002",  # Malignant neoplasm of lower lobe of lung (disorder)
        "187884007",  # Malignant neoplasm of pericardium (disorder)
        "187900002",  # Malignant neoplasm of bones of skull and face (disorder)
        "187903000",  # Malignant neoplasm of malar bone (disorder)
        "187906008",  # Malignant neoplasm of orbital bone (disorder)
        "187916000",  # Malignant neoplasm of cervical vertebra (disorder)
        "187917009",  # Malignant neoplasm of thoracic vertebra (disorder)
        "187918004",  # Malignant neoplasm of lumbar vertebra (disorder)
        "187920001",  # Malignant neoplasm of ribs and/or sternum and/or clavicle (disorder)
        "187925006",  # Malignant neoplasm of costal cartilage (disorder)
        "187926007",  # Malignant neoplasm of costovertebral joint (disorder)
        "187927003",  # Malignant neoplasm of xiphoid process (disorder)
        "187929000",  # Malignant neoplasm of scapula and long bones of upper arm (disorder)
        "187932002",  # Malignant neoplasm of humerus (disorder)
        "187937008",  # Malignant neoplasm of carpal bone - scaphoid (disorder)
        "187938003",  # Malignant neoplasm of carpal bone - lunate (disorder)
        "187939006",  # Malignant neoplasm of carpal bone - triquetrum (disorder)
        "187940008",  # Malignant neoplasm of carpal bone - pisiform (disorder)
        "187941007",  # Malignant neoplasm of carpal bone - trapezium (disorder)
        "187942000",  # Malignant neoplasm of carpal bone - trapezoid (disorder)
        "187943005",  # Malignant neoplasm of carpal bone - capitate (disorder)
        "187944004",  # Malignant neoplasm of carpal bone - hamate (disorder)
        "187945003",  # Malignant neoplasm of first metacarpal bone (disorder)
        "187946002",  # Malignant neoplasm of second metacarpal bone (disorder)
        "187947006",  # Malignant neoplasm of third metacarpal bone (disorder)
        "187948001",  # Malignant neoplasm of fourth metacarpal bone (disorder)
        "187949009",  # Malignant neoplasm of fifth metacarpal bone (disorder)
        "187950009",  # Malignant neoplasm of phalanges of hand (disorder)
        "187952001",  # Malignant neoplasm of pelvic bones, sacrum and coccyx (disorder)
        "187956003",  # Malignant neoplasm of sacral vertebra (disorder)
        "187957007",  # Malignant neoplasm of coccygeal vertebra (disorder)
        "187967002",  # Malignant neoplasm of calcaneum (disorder)
        "187968007",  # Malignant neoplasm of medial cuneiform (disorder)
        "187969004",  # Malignant neoplasm of intermediate cuneiform (disorder)
        "187970003",  # Malignant neoplasm of lateral cuneiform (disorder)
        "187972006",  # Malignant neoplasm of navicular (disorder)
        "187973001",  # Malignant neoplasm of first metatarsal bone (disorder)
        "187974007",  # Malignant neoplasm of second metatarsal bone (disorder)
        "187975008",  # Malignant neoplasm of third metatarsal bone (disorder)
        "187976009",  # Malignant neoplasm of fourth metatarsal bone (disorder)
        "187977000",  # Malignant neoplasm of fifth metatarsal bone (disorder)
        "187978005",  # Malignant neoplasm of phalanges of foot (disorder)
        "187987001",  # Malignant neoplasm of cartilage of ear (disorder)
        "187988006",  # Malignant neoplasm of tarsus of eyelid (disorder)
        "187989003",  # Malignant neoplasm soft tissues of cervical spine (disorder)
        "188029000",  # Kaposi's sarcoma of soft tissue (disorder)
        "188030005",  # Malignant melanoma of lip (disorder)
        "188033007",  # Malignant melanoma of auricle (ear) (disorder)
        "188034001",  # Malignant melanoma of external auditory meatus (disorder)
        "188038003",  # Malignant melanoma of chin (disorder)
        "188040008",  # Malignant melanoma of forehead (disorder)
        "188045003",  # Malignant melanoma of scalp (disorder)
        "188046002",  # Malignant melanoma of neck (disorder)
        "188049009",  # Malignant melanoma of axilla (disorder)
        "188050009",  # Malignant melanoma of breast (disorder)
        "188051008",  # Malignant melanoma of buttock (disorder)
        "188052001",  # Malignant melanoma of groin (disorder)
        "188054000",  # Malignant melanoma of perineum (disorder)
        "188055004",  # Malignant melanoma of umbilicus (disorder)
        "188060000",  # Malignant melanoma of shoulder (disorder)
        "188061001",  # Malignant melanoma of upper arm (disorder)
        "188062008",  # Malignant melanoma of forearm (disorder)
        "188063003",  # Malignant melanoma of hand (disorder)
        "188064009",  # Malignant melanoma of finger (disorder)
        "188065005",  # Malignant melanoma of thumb (disorder)
        "188068007",  # Malignant melanoma of hip (disorder)
        "188069004",  # Malignant melanoma of thigh (disorder)
        "188070003",  # Malignant melanoma of knee (disorder)
        "188071004",  # Malignant melanoma of popliteal fossa area (disorder)
        "188072006",  # Malignant melanoma of lower leg (disorder)
        "188073001",  # Malignant melanoma of ankle (disorder)
        "188074007",  # Malignant melanoma of heel (disorder)
        "188075008",  # Malignant melanoma of foot (disorder)
        "188076009",  # Malignant melanoma of toe (disorder)
        "188077000",  # Malignant melanoma of great toe (disorder)
        "188089003",  # Malignant neoplasm of skin of ear and external auditory canal (disorder)
        "188090007",  # Malignant neoplasm of skin of auricle (ear) (disorder)
        "188091006",  # Malignant neoplasm of skin of external auditory meatus (disorder)
        "188095002",  # Malignant neoplasm of skin of cheek, external (disorder)
        "188099008",  # Malignant neoplasm of skin of nose (external) (disorder)
        "188100000",  # Malignant neoplasm of skin of temple (disorder)
        "188102008",  # Malignant neoplasm of scalp and/or skin of neck (disorder)
        "188103003",  # Malignant neoplasm of skin of scalp (disorder)
        "188107002",  # Malignant neoplasm of skin of axillary fold (disorder)
        "188110009",  # Malignant neoplasm of skin of abdominal wall (disorder)
        "188121000",  # Malignant neoplasm of skin of upper arm (disorder)
        "188122007",  # Malignant neoplasm of skin of forearm (disorder)
        "188125009",  # Malignant neoplasm of skin of thumb (disorder)
        "188132000",  # Malignant neoplasm of skin of popliteal fossa area (disorder)
        "188133005",  # Malignant neoplasm of skin of lower leg (disorder)
        "188135003",  # Malignant neoplasm of skin of heel (disorder)
        "188138001",  # Malignant neoplasm of skin of great toe (disorder)
        "188157005",  # Malignant neoplasm of overlapping sites of breast (disorder)
        "188176007",  # Malignant neoplasm of endocervical canal (disorder)
        "188177003",  # Malignant neoplasm of endocervical gland (disorder)
        "188180002",  # Primary malignant neoplasm of overlapping sites of cervix uteri (disorder)
        "188183000",  # Malignant neoplasm of cervical stump (disorder)
        "188184006",  # Malignant neoplasm of squamocolumnar junction of cervix (disorder)
        "188188009",  # Choriocarcinoma (disorder)
        "188189001",  # Malignant neoplasm of corpus uteri, excluding isthmus (disorder)
        "188190005",  # Malignant neoplasm of cornu of corpus uteri (disorder)
        "188191009",  # Malignant neoplasm of fundus of corpus uteri (disorder)
        "188192002",  # Malignant neoplasm of endometrium of corpus uteri (disorder)
        "188193007",  # Malignant neoplasm of myometrium of corpus uteri (disorder)
        "188195000",  # Malignant neoplasm of isthmus of uterine body (disorder)
        "188204000",  # Malignant neoplasm of round ligament (disorder)
        "188208002",  # Malignant neoplasm of Gartner's duct (disorder)
        "188209005",  # Malignant neoplasm of vaginal vault (disorder)
        "188211001",  # Malignant neoplasm of greater vestibular (Bartholin's) gland (disorder)
        "188219004",  # Malignant tumor of undescended testis (disorder)
        "188220005",  # Malignant tumor of ectopic testis (disorder)
        "188230001",  # Malignant tumor of body of penis (disorder)
        "188234005",  # Malignant tumor of seminal vesicle (disorder)
        "188235006",  # Malignant tumor of tunica vaginalis (disorder)
        "188239000",  # Malignant tumor of trigone of urinary bladder (disorder)
        "188240003",  # Malignant neoplasm of dome of urinary bladder (disorder)
        "188241004",  # Malignant neoplasm of lateral wall of urinary bladder (disorder)
        "188242006",  # Malignant neoplasm of anterior wall of urinary bladder (disorder)
        "188243001",  # Malignant neoplasm of posterior wall of urinary bladder (disorder)
        "188244007",  # Malignant neoplasm of neck of urinary bladder (disorder)
        "188245008",  # Malignant tumor of ureteric orifice (disorder)
        "188247000",  # Malignant neoplasm of overlapping sites of urinary bladder (disorder)
        "188250002",  # Malignant tumor of kidney parenchyma (disorder)
        "188252005",  # Malignant tumor of renal calyx (disorder)
        "188253000",  # Malignant tumor of pelviureteric junction (disorder)
        "188256008",  # Malignant neoplasm of overlapping sites of urinary organs (disorder)
        "188261005",  # Malignant neoplasm of eyeball excluding conjunctiva, cornea, retina and choroid (disorder)
        "188263008",  # Malignant tumor of ciliary body (disorder)
        "188264002",  # Malignant tumor of iris (disorder)
        "188265001",  # Malignant neoplasm of crystalline lens (disorder)
        "188266000",  # Malignant neoplasm of sclera (disorder)
        "188268004",  # Malignant neoplasm of connective tissue of orbit (disorder)
        "188269007",  # Malignant neoplasm of extraocular muscle of orbit (disorder)
        "188272000",  # Malignant tumor of lacrimal gland (disorder)
        "188273005",  # Malignant neoplasm of lacrimal sac (disorder)
        "188274004",  # Malignant neoplasm of nasolacrimal duct (disorder)
        "188280007",  # Malignant neoplasm of cerebrum (excluding lobes and ventricles) (disorder)
        "188281006",  # Malignant neoplasm of basal ganglia (disorder)
        "188282004",  # Malignant neoplasm of cerebral cortex (disorder)
        "188283009",  # Malignant neoplasm of corpus striatum (disorder)
        "188285002",  # Malignant neoplasm of globus pallidus (disorder)
        "188286001",  # Malignant tumor of hypothalamus (disorder)
        "188287005",  # Malignant neoplasm of thalamus (disorder)
        "188289008",  # Malignant neoplasm of hippocampus (disorder)
        "188290004",  # Malignant neoplasm of uncus (disorder)
        "188292007",  # Malignant tumor of choroid plexus (disorder)
        "188293002",  # Malignant neoplasm of floor of cerebral ventricle (disorder)
        "188295009",  # Malignant neoplasm of cerebral peduncle (disorder)
        "188296005",  # Malignant neoplasm of medulla oblongata (disorder)
        "188297001",  # Malignant neoplasm of midbrain (disorder)
        "188298006",  # Malignant neoplasm of pons (disorder)
        "188301005",  # Malignant neoplasm of corpus callosum (disorder)
        "188302003",  # Malignant neoplasm of tapetum (disorder)
        "188307009",  # Malignant tumor of cranial nerve (disorder)
        "188308004",  # Malignant neoplasm of olfactory bulb (disorder)
        "188312005",  # Malignant neoplasm of cerebral dura mater (disorder)
        "188313000",  # Malignant neoplasm of cerebral arachnoid mater (disorder)
        "188315007",  # Malignant neoplasm of cerebral pia mater (disorder)
        "188317004",  # Malignant neoplasm of spinal dura mater (disorder)
        "188318009",  # Malignant neoplasm of spinal arachnoid mater (disorder)
        "188319001",  # Malignant neoplasm of spinal pia mater (disorder)
        "188321006",  # Malignant neoplasm of peripheral nerves and autonomic nervous system (disorder)
        "188322004",  # Malignant neoplasm of peripheral nerves of head, face and neck (disorder)
        "188323009",  # Malignant neoplasm of peripheral nerves of upper limb, including shoulder (disorder)
        "188324003",  # Malignant neoplasm of peripheral nerves of lower limb, including hip (disorder)
        "188325002",  # Malignant neoplasm of peripheral nerve of thorax (disorder)
        "188326001",  # Malignant neoplasm of peripheral nerve of abdomen (disorder)
        "188327005",  # Malignant neoplasm of peripheral nerve of pelvis (disorder)
        "188339002",  # Malignant neoplasm of pituitary gland and craniopharyngeal duct (disorder)
        "188340000",  # Malignant tumor of craniopharyngeal duct (disorder)
        "188361007",  # Malignant neoplasm of thorax (disorder)
        "188366002",  # Malignant neoplasm of abdomen (disorder)
        "188445006",  # Metastatic malignant neoplasm to retroperitoneum and peritoneum (disorder)
        "188454009",  # Metastatic malignant neoplasm to skin of head (disorder)
        "188458007",  # Metastatic malignant neoplasm to skin of shoulder and arm (disorder)
        "188462001",  # Metastatic malignant neoplasm to brain and spinal cord (disorder)
        "188469005",  # Metastatic malignant neoplasm to cervix uteri (disorder)
        "188471005",  # Metastatic malignant neoplasm to epididymis and vas deferens (disorder)
        "188487008",  # Lymphosarcoma and reticulosarcoma (disorder)
        "188489006",  # Reticulosarcoma of lymph nodes of head, face and neck (disorder)
        "188492005",  # Reticulosarcoma of lymph nodes of axilla and upper limb (disorder)
        "188493000",  # Reticulosarcoma of lymph nodes of inguinal region and lower limb (disorder)
        "188498009",  # Lymphosarcoma (disorder)
        "188500005",  # Lymphosarcoma of lymph nodes of head, face and neck (disorder)
        "188501009",  # Lymphosarcoma of intrathoracic lymph nodes (disorder)
        "188502002",  # Lymphosarcoma of intra-abdominal lymph nodes (disorder)
        "188503007",  # Lymphosarcoma of lymph nodes of axilla and upper limb (disorder)
        "188504001",  # Lymphosarcoma of lymph nodes of inguinal region and lower limb (disorder)
        "188505000",  # Lymphosarcoma of intrapelvic lymph nodes (disorder)
        "188506004",  # Lymphosarcoma of spleen (disorder)
        "188507008",  # Lymphosarcoma of lymph nodes of multiple sites (disorder)
        "188511002",  # Burkitt's lymphoma of intrathoracic lymph nodes (disorder)
        "188512009",  # Burkitt's lymphoma of intra-abdominal lymph nodes (disorder)
        "188515006",  # Burkitt's lymphoma of intrapelvic lymph nodes (disorder)
        "188516007",  # Burkitt's lymphoma of spleen (disorder)
        "188517003",  # Burkitt's lymphoma of lymph nodes of multiple sites (disorder)
        "188524002",  # Hodgkin's paragranuloma of intrathoracic lymph nodes (disorder)
        "188529007",  # Hodgkin's paragranuloma of intrapelvic lymph nodes (disorder)
        "188531003",  # Hodgkin's paragranuloma of lymph nodes of multiple sites (disorder)
        "188536008",  # Hodgkin's granuloma of intra-abdominal lymph nodes (disorder)
        "188541000",  # Hodgkin's granuloma of lymph nodes of multiple sites (disorder)
        "188551004",  # Hodgkin's sarcoma of lymph nodes of multiple sites (disorder)
        "188562004",  # Hodgkin's disease, lymphocytic-histiocytic predominance of lymph nodes of multiple sites (disorder)
        "188566001",  # Hodgkin's disease, nodular sclerosis of intrathoracic lymph nodes (disorder)
        "188567005",  # Hodgkin's disease, nodular sclerosis of intra-abdominal lymph nodes (disorder)
        "188570009",  # Hodgkin's disease, nodular sclerosis of intrapelvic lymph nodes (disorder)
        "188572001",  # Hodgkin's disease, nodular sclerosis of lymph nodes of multiple sites (disorder)
        "188576003",  # Hodgkin's disease, mixed cellularity of intrathoracic lymph nodes (disorder)
        "188577007",  # Hodgkin's disease, mixed cellularity of intra-abdominal lymph nodes (disorder)
        "188580008",  # Hodgkin's disease, mixed cellularity of intrapelvic lymph nodes (disorder)
        "188582000",  # Hodgkin's disease, mixed cellularity of lymph nodes of multiple sites (disorder)
        "188586002",  # Hodgkin's disease, lymphocytic depletion of intrathoracic lymph nodes (disorder)
        "188587006",  # Hodgkin's disease, lymphocytic depletion of intra-abdominal lymph nodes (disorder)
        "188591001",  # Hodgkin's disease, lymphocytic depletion of intrapelvic lymph nodes (disorder)
        "188592008",  # Hodgkin's disease, lymphocytic depletion of spleen (disorder)
        "188593003",  # Hodgkin's disease, lymphocytic depletion of lymph nodes of multiple sites (disorder)
        "188609000",  # Nodular lymphoma of lymph nodes of head, face and neck (disorder)
        "188612002",  # Nodular lymphoma of lymph nodes of axilla and upper limb (disorder)
        "188613007",  # Nodular lymphoma of lymph nodes of inguinal region and lower limb (disorder)
        "188627002",  # Mycosis fungoides of lymph nodes of multiple sites (disorder)
        "188631008",  # Sezary's disease of intrathoracic lymph nodes (disorder)
        "188632001",  # Sezary's disease of intra-abdominal lymph nodes (disorder)
        "188633006",  # Sezary's disease of lymph nodes of axilla and upper limb (disorder)
        "188634000",  # Sezary's disease of lymph nodes of inguinal region and lower limb (disorder)
        "188635004",  # Sezary's disease of intrapelvic lymph nodes (disorder)
        "188637007",  # Sezary's disease of lymph nodes of multiple sites (disorder)
        "188641006",  # Malignant histiocytosis of lymph nodes of axilla and upper limb (disorder)
        "188645002",  # Leukemic reticuloendotheliosis of lymph nodes of head, face and neck (disorder)
        "188648000",  # Leukemic reticuloendotheliosis of lymph nodes of axilla and upper limb (disorder)
        "188649008",  # Leukemic reticuloendotheliosis of lymph nodes of inguinal region and lower limb (disorder)
        "188662007",  # Mast cell malignancy of lymph nodes of head and neck (disorder)
        "188668006",  # Malignant mast cell sarcoma of spleen (disorder)
        "188669003",  # Mast cell malignancy of lymph nodes of multiple sites (disorder)
        "188672005",  # Follicular non-Hodgkin's mixed small cleaved and large cell lymphoma (disorder)
        "188674006",  # Diffuse malignant lymphoma - small non-cleaved cell (disorder)
        "188675007",  # Malignant lymphoma - small cleaved cell (disorder)
        "188676008",  # Malignant lymphoma - mixed small and large cell (disorder)
        "188718006",  # Malignant plasma cell neoplasm, extramedullary plasmacytoma (disorder)
        "188725004",  # Lymphoid leukemia (disorder)
        "188726003",  # Subacute lymphoid leukemia (disorder)
        "188728002",  # Aleukemic lymphoid leukemia (disorder)
        "188732008",  # Myeloid leukemia (disorder)
        "188733003",  # Chronic eosinophilic leukemia (disorder)
        "188734009",  # Chronic neutrophilic leukemia (disorder)
        "188737002",  # Chloroma (disorder)
        "188744006",  # Monocytic leukemia (disorder)
        "188754005",  # Megakaryocytic leukemia (disorder)
        "188768003",  # Myelomonocytic leukemia (disorder)
        "188770007",  # Subacute myelomonocytic leukemia (disorder)
        "189179009",  # Craniopharyngioma (disorder)
        "189361002",  # Neoplasm of uncertain behavior of digestive and respiratory systems (disorder)
        "189362009",  # Neoplasm of uncertain behavior of major salivary gland (disorder)
        "189365006",  # Neoplasm of uncertain behavior of submandibular gland (disorder)
        "189371000",  # Neoplasm of uncertain behavior of gums (disorder)
        "189392000",  # Neoplasm of uncertain behavior of hepatic duct (disorder)
        "189397006",  # Neoplasm of uncertain behavior of sphincter of Oddi (disorder)
        "189399009",  # Neoplasm of uncertain behavior of retroperitoneum and peritoneum (disorder)
        "189401003",  # Neoplasm of uncertain behavior of peritoneum (disorder)
        "189406008",  # Neoplasm of uncertain behavior of anal canal and sphincter (disorder)
        "189410006",  # Neoplasm of uncertain behavior of thyroid cartilage (disorder)
        "189412003",  # Neoplasm of uncertain behavior of cricoid cartilage (disorder)
        "189414002",  # Neoplasm of uncertain behavior of arytenoid cartilage (disorder)
        "189415001",  # Neoplasm of uncertain behavior of corniculate cartilage (disorder)
        "189416000",  # Neoplasm of uncertain behavior of cuneiform cartilage (disorder)
        "189417009",  # Neoplasm of uncertain behavior of aryepiglottic fold (disorder)
        "189432002",  # Neoplasm of uncertain behavior of tympanic cavity (disorder)
        "189433007",  # Neoplasm of uncertain behavior of tympanic antrum (disorder)
        "189435000",  # Neoplasm of uncertain behavior of mastoid air cells (disorder)
        "189441007",  # Neoplasm of uncertain behavior of genitourinary organs (disorder)
        "189473001",  # Neoplasm of uncertain behavior of paraganglia (disorder)
        "189484008",  # Neoplasm of uncertain behavior of brain and/or spinal cord (disorder)
        "189487001",  # Neoplasm of uncertain or unknown behavior of brain, supratentorial (disorder)
        "189488006",  # Neoplasm of uncertain or unknown behavior of brain, infratentorial (disorder)
        "189814006",  # Pancreatoblastoma (disorder)
        "189815007",  # Pulmonary blastoma (disorder)
        "189847002",  # Malignant teratoma (disorder)
        "189924002",  # Pleomorphic xanthoastrocytoma (disorder)
        "190030009",  # Compound leukemias (disorder)
        "190818004",  # Waldenstrom macroglobulinemia (disorder)
        "19090001000004101",  # Metastatic neoplasm of left basal ganglion (disorder)
        "192681000119104",  # Malignant carcinoid tumor of appendix (disorder)
        "192701000119101",  # Malignant carcinoid tumor of ileum (disorder)
        "208041000119100",  # Primary adenocarcinoma of endocervix (disorder)
        "21851000119103",  # Malignant pheochromocytoma (disorder)
        "226521000119108",  # Primary malignant neoplasm of nasopharynx (disorder)
        "230156002",  # Malignant meningitis (disorder)
        "231829006",  # Malignant neoplasm of eyelid (disorder)
        "231831002",  # Squamous cell carcinoma of eyelid (disorder)
        "231832009",  # Basal cell carcinoma of eyelid (disorder)
        "231833004",  # Sebaceous adenocarcinoma of eyelid (disorder)
        "231834005",  # Malignant melanoma of eyelid (disorder)
        "231835006",  # Kaposi's sarcoma of eyelid (disorder)
        "232075002",  # Lymphoma of retina (disorder)
        "233719000",  # Lymphangioleiomyomatosis of skin (disorder)
        "234941000119100",  # Malignant glioma of eye (disorder)
        "235966007",  # Cystadenocarcinoma of pancreas (disorder)
        "236512004",  # Leukemic infiltrate of kidney (disorder)
        "236513009",  # Lymphoma of kidney (disorder)
        "236811000119101",  # Paget disease of anal canal (disorder)
        "237252008",  # Placental site trophoblastic tumor (disorder)
        "237832001",  # Gastrointestinal hormone-secreting endocrine tumor (disorder)
        "238476000",  # Eosinophilic granuloma of bone (disorder)
        "238636003",  # Malignant acanthosis nigricans (disorder)
        "238863004",  # Giant cell fibroblastoma of skin (disorder)
        "239147000",  # Mast cell nevus (disorder)
        "240163000",  # Malignant neoplasm of nasopharyngeal wall (disorder)
        "240531002",  # African Burkitt's lymphoma (disorder)
        "241861008",  # Metastatic malignant neoplasm to nasopharynx (disorder)
        "242862004",  # Metastatic malignant neoplasm to nasopharyngeal wall (disorder)
        "243971001000119103",  # Malignant melanoma of skin of right ankle (disorder)
        "249389151000119103",  # Malignant melanoma of skin of right wrist (disorder)
        "25050002",  # Alpha heavy chain disease, respiratory form (disorder)
        "253000007",  # Neuroendocrine carcinoma (disorder)
        "253001006",  # Merkel cell carcinoma (disorder)
        "253005002",  # Vasoactive intestinal peptide-secreting tumor (disorder)
        "253006001",  # Somatostatinoma (disorder)
        "253008000",  # Apudoma (disorder)
        "253017000",  # Klatskin's tumor (disorder)
        "253018005",  # Fibrolamellar hepatocellular carcinoma (disorder)
        "253029009",  # Gangliocytic paraganglioma (disorder)
        "253031000",  # Glomus tympanicum tumor (disorder)
        "253042009",  # Myxoid dermatofibrosarcoma protuberans (disorder)
        "253096008",  # Peripheral neuroectodermal tumor (disorder)
        "254389005",  # Carcinoma of vermilion border of upper lip (disorder)
        "254390001",  # Carcinoma of vermilion border of lower lip (disorder)
        "254393004",  # Carcinoma of frenum of lip (disorder)
        "254398008",  # Carcinoma of frenum of upper lip (disorder)
        "254402004",  # Carcinoma of frenum of lower lip (disorder)
        "254404003",  # Carcinoma of commissure of lip (disorder)
        "254408000",  # Malignant tumor of anterior two-thirds of tongue - lateral margin (disorder)
        "254412006",  # Malignant tumor of tip of tongue (disorder)
        "254417000",  # Carcinoma of frenum linguae (disorder)
        "254423005",  # Carcinoma of lingual tonsil (disorder)
        "254424004",  # Carcinoma of upper gum (disorder)
        "254425003",  # Carcinoma of lower gum (disorder)
        "254427006",  # Carcinoma of anterior part of floor of mouth (disorder)
        "254431000",  # Carcinoma of lateral part of floor of mouth (disorder)
        "254434008",  # Carcinoma of hard palate (disorder)
        "254435009",  # Carcinoma of soft palate (disorder)
        "254436005",  # Carcinoma of uvula (disorder)
        "254437001",  # Squamous cell carcinoma of buccal mucosa (disorder)
        "254441002",  # Carcinoma of upper buccal sulcus (disorder)
        "254445006",  # Carcinoma of lower buccal sulcus (disorder)
        "254450000",  # Carcinoma of upper labial sulcus (disorder)
        "254454009",  # Carcinoma of lower labial sulcus (disorder)
        "254457002",  # Carcinoma of retromolar area (disorder)
        "254459004",  # Malignant tumor of anterior pillar of fauces (disorder)
        "254462001",  # Carcinoma of parotid gland (disorder)
        "254465004",  # Carcinoma of submandibular gland (disorder)
        "254466003",  # Carcinoma of sublingual gland (disorder)
        "254474002",  # Malignant tumor of nasal skeleton (disorder)
        "254478004",  # Malignant tumor of inferior turbinate (disorder)
        "254481009",  # Malignant tumor of middle turbinate (disorder)
        "254484001",  # Malignant tumor of posterior margin of nasal septum and choanae (disorder)
        "254503007",  # Malignant tumor of inferior surface of soft palate (disorder)
        "254509006",  # Malignant tumor of anterior commissure (disorder)
        "254513004",  # Malignant tumor of posterior commissure (disorder)
        "254517003",  # Malignant tumor of suprahyoid epiglottis (disorder)
        "254520006",  # Malignant tumor of infrahyoid epiglottis (disorder)
        "254526000",  # Malignant tumor of laryngeal ventricle (disorder)
        "254530002",  # Malignant tumor of parapharyngeal space (disorder)
        "254535007",  # Carcinoma of cervical part of esophagus (disorder)
        "254539001",  # Carcinoma of thoracic part of esophagus (disorder)
        "254543002",  # Carcinoma of abdominal part of esophagus (disorder)
        "254547001",  # Carcinoma of upper third of esophagus (disorder)
        "254549003",  # Carcinoma of middle third of esophagus (disorder)
        "254551004",  # Carcinoma of lower third of esophagus (disorder)
        "254553001",  # Carcinoma of cardia (disorder)
        "254555008",  # Carcinoma of fundus of stomach (disorder)
        "254557000",  # Carcinoma of body of stomach (disorder)
        "254559002",  # Carcinoma of pyloric antrum (disorder)
        "254561006",  # Carcinoma of pylorus (disorder)
        "254563009",  # Carcinoma of lesser curve of stomach (disorder)
        "254567005",  # Carcinoma of greater curve of stomach (disorder)
        "254570009",  # Carcinoma of duodenum (disorder)
        "254574000",  # Diffuse leiomyomatosis of esophagus (disorder)
        "254582000",  # Adenocarcinoma of rectum (disorder)
        "254586002",  # Malignant tumor of anorectal junction (disorder)
        "254601002",  # Sarcoma of liver (disorder)
        "254609000",  # Carcinoma of ampulla of Vater (disorder)
        "254611009",  # Malignant tumor of endocrine pancreas (disorder)
        "254612002",  # Carcinoma of endocrine pancreas (disorder)
        "254619006",  # Adenoid cystic carcinoma of trachea (disorder)
        "254620000",  # Squamous cell carcinoma of trachea (disorder)
        "254622008",  # Squamous cell carcinoma of bronchus (disorder)
        "254625005",  # Malignant tumor of lung parenchyma (disorder)
        "254626006",  # Adenocarcinoma of lung (disorder)
        "254629004",  # Large cell carcinoma of lung (disorder)
        "254631008",  # Giant cell carcinoma of lung (disorder)
        "254632001",  # Small cell carcinoma of lung (disorder)
        "254634000",  # Squamous cell carcinoma of lung (disorder)
        "254635004",  # Epithelioid hemangioendothelioma of lung (disorder)
        "254637007",  # Non-small cell lung cancer (disorder)
        "254638002",  # Pancoast tumor (disorder)
        "254643009",  # Intrapulmonary teratoma (disorder)
        "254645002",  # Malignant mesothelioma of pleura (disorder)
        "254650008",  # Malignant epithelial neoplasm of skin (disorder)
        "254651007",  # Squamous cell carcinoma of skin (disorder)
        "254652000",  # Clear cell squamous cell carcinoma of skin (disorder)
        "254653005",  # Spindle cell squamous carcinoma of skin (disorder)
        "254654004",  # Acantholytic squamous cell carcinoma of skin (disorder)
        "254655003",  # Plantar verrucous carcinoma (disorder)
        "254663002",  # Solitary keratoacanthoma (disorder)
        "254664008",  # Eruptive keratoacanthoma (disorder)
        "254701007",  # Basal cell carcinoma of skin (disorder)
        "254702000",  # Basosquamous carcinoma of skin (disorder)
        "254703005",  # Fibroepithelioma of Pinkus (disorder)
        "254707006",  # Malignant skin tumor with eccrine differentiation (disorder)
        "254708001",  # Eccrine porocarcinoma of skin (disorder)
        "254709009",  # Digital papillary eccrine carcinoma of skin (disorder)
        "254710004",  # Basal cell carcinoma with eccrine differentiation (disorder)
        "254711000",  # Adenoid cystic eccrine carcinoma of skin (disorder)
        "254712007",  # Microcystic adnexal carcinoma of skin (disorder)
        "254713002",  # Mucoepidermoid carcinoma of skin (disorder)
        "254714008",  # Mucinous eccrine carcinoma of skin (disorder)
        "254726003",  # Malignant skin tumor with apocrine differentiation (disorder)
        "254727007",  # Extramammary Paget's disease of skin (disorder)
        "254729005",  # Apudoma of skin (disorder)
        "254730000",  # Superficial spreading malignant melanoma of skin (disorder)
        "254731001",  # Nodular malignant melanoma of skin (disorder)
        "254732008",  # Acral lentiginous malignant melanoma of skin (disorder)
        "254733003",  # Malignant melanoma arising in intradermal nevus (disorder)
        "254734009",  # Malignant melanoma arising in congenital nevus (disorder)
        "254748009",  # Cutaneous fibrosarcoma (disorder)
        "254754005",  # Atypical fibroxanthoma of skin (disorder)
        "254764001",  # Peripheral neuroepithelioma (disorder)
        "254771006",  # Cutaneous leiomyosarcoma (disorder)
        "254792006",  # Proliferating angioendotheliomatosis (disorder)
        "254794007",  # Angiosarcoma of skin (disorder)
        "254796009",  # Solitary fibrous tumor of skin (disorder)
        "254797000",  # Malignant hemangiopericytoma of skin (disorder)
        "254798005",  # Hemangioendothelioma of skin (disorder)
        "254799002",  # Papillary intralymphatic angioendothelioma of childhood (disorder)
        "254800003",  # Epithelioid cell sarcoma of skin (disorder)
        "254820002",  # Follicular atrophoderma and basal cell epitheliomata (disorder)
        "254824006",  # Malignant tumor of mesothelial tissue (disorder)
        "254829001",  # Liposarcoma (disorder)
        "254837009",  # Malignant neoplasm of breast (disorder)
        "254838004",  # Carcinoma of breast (disorder)
        "254839007",  # Scirrhous carcinoma of breast (disorder)
        "254840009",  # Inflammatory carcinoma of breast (disorder)
        "254841008",  # Cancer en cuirasse (disorder)
        "254843006",  # Familial cancer of breast (disorder)
        "254844000",  # Malignant phyllodes tumor of breast (disorder)
        "254849005",  # Malignant epithelial tumor of ovary (disorder)
        "254850005",  # Serous papillary cystadenocarcinoma ovary (disorder)
        "254851009",  # Mucinous cystadenocarcinoma of ovary (disorder)
        "254852002",  # Endometrioid carcinoma ovary (disorder)
        "254855000",  # Mixed epithelial tumor of ovary (disorder)
        "254856004",  # Undifferentiated carcinoma of ovary (disorder)
        "254860001",  # Malignant sex cord tumor of ovary (disorder)
        "254863004",  # Malignant granulosa cell tumour of ovary (disorder)
        "254867003",  # Gynandroblastoma of ovary (disorder)
        "254869000",  # Malignant germ cell tumor of ovary (disorder)
        "254870004",  # Choriocarcinoma of ovary (disorder)
        "254872007",  # Embryonal carcinoma of ovary (disorder)
        "254874008",  # Malignant dysgerminoma of ovary (disorder)
        "254876005",  # Endodermal sinus tumor of ovary (disorder)
        "254877001",  # Sarcoma of uterus (disorder)
        "254878006",  # Endometrial carcinoma (disorder)
        "254883003",  # Intravenous leiomyomatosis (disorder)
        "254886006",  # Squamous cell carcinoma of cervix (disorder)
        "254887002",  # Adenocarcinoma of cervix (disorder)
        "254888007",  # Adenosquamous carcinoma of cervix (disorder)
        "254893005",  # Carcinoma of vagina (disorder)
        "254895003",  # Squamous cell carcinoma of vulva (disorder)
        "254896002",  # Malignant melanoma of vulva (disorder)
        "254897006",  # Sarcoma of vulva (disorder)
        "254898001",  # Paget's disease of vulva (disorder)
        "254900004",  # Carcinoma of prostate (disorder)
        "254904008",  # Carcinoma of glans penis (disorder)
        "254908006",  # Malignant tumor of skin of penis (disorder)
        "254909003",  # Carcinoma of foreskin (disorder)
        "254912000",  # Regressed malignant testicular tumor (disorder)
        "254915003",  # Clear cell carcinoma of kidney (disorder)
        "254916002",  # Cystadenocarcinoma of kidney (disorder)
        "254917006",  # Papillary cystadenocarcinoma of kidney (disorder)
        "254918001",  # Sarcoma of kidney (disorder)
        "254934003",  # Malignant tumor of urethral stump (disorder)
        "254938000",  # Astrocytoma of brain (disorder)
        "254940005",  # Oligodendroglioma of brain (disorder)
        "254945000",  # Embryonal tumor of brain (disorder)
        "254948003",  # Astrocytoma of spinal cord (disorder)
        "254950006",  # Oligodendroglioma of spinal cord (disorder)
        "254954002",  # Embryonal tumor of spinal cord (disorder)
        "254955001",  # Pituitary carcinoma (disorder)
        "254969001",  # Malignant tumor of olfactory tract (disorder)
        "254972008",  # Malignant tumor of optic nerve and sheath (disorder)
        "254973003",  # Malignant astrocytoma of optic nerve (disorder)
        "254974009",  # Malignant tumor of optic nerve sheath (disorder)
        "254975005",  # Malignant meningioma of optic nerve sheath (disorder)
        "254976006",  # Optic nerve glioma (disorder)
        "254983004",  # Malignant tumor of spinal nerve and sheath (disorder)
        "254986007",  # Malignant tumor of peripheral nerve (disorder)
        "254987003",  # Adenoid cystic carcinoma of lacrimal gland (disorder)
        "254988008",  # Adenocarcinoma of lacrimal gland (disorder)
        "254989000",  # Carcinoma ex pleomorphic adenoma of lacrimal gland (disorder)
        "254990009",  # Mucoepidermoid tumor of lacrimal gland (disorder)
        "254993006",  # Liposarcoma of orbit (disorder)
        "254994000",  # Rhabdomyosarcoma of orbit (disorder)
        "254995004",  # Malignant hemangiopericytoma of orbit (disorder)
        "254996003",  # Malignant fibrous histiocytoma of orbit (disorder)
        "255003007",  # Squamous cell carcinoma of conjunctiva (disorder)
        "255004001",  # Malignant melanoma of conjunctiva (disorder)
        "255008003",  # Squamous cell carcinoma of cornea (disorder)
        "255012009",  # Malignant melanoma of iris (disorder)
        "255015006",  # Malignant melanoma of ciliary body (disorder)
        "255016007",  # Adenocarcinoma of pigmented epithelium of ciliary body (disorder)
        "255017003",  # Adenocarcinoma of non-pigmented epithelium of ciliary body (disorder)
        "255021005",  # Malignant melanoma of choroid (disorder)
        "255028004",  # Follicular thyroid carcinoma (disorder)
        "255029007",  # Papillary thyroid carcinoma (disorder)
        "255030002",  # Mixed follicular and papillary thyroid carcinoma (disorder)
        "255031003",  # Anaplastic thyroid carcinoma (disorder)
        "255032005",  # Medullary thyroid carcinoma (disorder)
        "255035007",  # Adrenal carcinoma (disorder)
        "255037004",  # Parathyroid carcinoma (disorder)
        "255039001",  # Pancreatic polypeptidoma (disorder)
        "255052006",  # Malignant neoplasm of unknown origin (disorder)
        "255056009",  # Malignant tumor of head and/or neck (disorder)
        "255062004",  # Neoplasm of uncertain behavior of oral cavity (disorder)
        "255064003",  # Neoplasm of uncertain behavior of biliary system (disorder)
        "255066001",  # Carcinoma of genitourinary organ (disorder)
        "255067005",  # Sarcoma of bone and connective tissue (disorder)
        "255068000",  # Carcinoma of bone, connective tissue, skin and breast (disorder)
        "255069008",  # Carcinoma of lip, oral cavity and/or pharynx (disorder)
        "255070009",  # Overlapping malignant neoplasm of oral cavity and lip and salivary gland (disorder)
        "255071008",  # Squamous cell carcinoma of lip (disorder)
        "255072001",  # Malignant tumor of salivary gland (disorder)
        "255073006",  # Malignant tumor of ear, nose and throat (disorder)
        "255074000",  # Malignant tumor of nasal cavity and nasopharynx (disorder)
        "255075004",  # Malignant tumor of lateral nasal wall (disorder)
        "255077007",  # Malignant tumor of digestive organ (disorder)
        "255078002",  # Malignant tumor of esophagus, stomach and duodenum (disorder)
        "255081007",  # Carcinoma of cecum (disorder)
        "255084004",  # Squamous cell carcinoma of anal margin (disorder)
        "255086002",  # Carcinoma common bile duct (disorder)
        "255087006",  # Malignant polyp of biliary tract (disorder)
        "255088001",  # Malignant tumor of exocrine pancreas (disorder)
        "255090000",  # Malignant neoplasm of carpal bones (disorder)
        "255091001",  # Malignant neoplasm of metacarpal bones (disorder)
        "255093003",  # Malignant skin tumor with adnexal differentiation (disorder)
        "255096006",  # Malignant tumor of dermis (disorder)
        "255107005",  # Seminoma of testis (disorder)
        "255108000",  # Carcinoma of urinary bladder (disorder)
        "255109008",  # Transitional cell carcinoma of urinary bladder (disorder)
        "255110003",  # Adenocarcinoma of urinary bladder (disorder)
        "255111004",  # Squamous cell carcinoma of urinary bladder (disorder)
        "255112006",  # Malignant tumor of pituitary and hypothalamus (disorder)
        "255114007",  # Kaposi's sarcoma of conjunctiva (disorder)
        "255115008",  # Kaposi's sarcoma of cornea (disorder)
        "255118005",  # Metastatic lymphangitic carcinoma (disorder)
        "255119002",  # Lymphangitis carcinomatosa (disorder)
        "255121007",  # Carcinomatosis of peritoneal cavity (disorder)
        "255123005",  # Metastasis to nervous system and eye (disorder)
        "255124004",  # Metastasis to peripheral nerve (disorder)
        "255128001",  # Malignant infiltration of peripheral nerve (disorder)
        "255129009",  # Malignant infiltration of peripheral nerve plexus (disorder)
        "255191003",  # Localized malignant reticulohistiocytoma (disorder)
        "269459004",  # Malignant tumor of lesser curve of stomach (disorder)
        "269460009",  # Malignant tumor of greater curve of stomach (disorder)
        "269463006",  # Malignant tumor of middle ear and mastoid (disorder)
        "269464000",  # Malignant neoplasm of upper lobe, bronchus or lung (disorder)
        "269467007",  # Malignant neoplasm of hand bones (disorder)
        "269469005",  # Malignant neoplasm of soft tissue (disorder)
        "269473008",  # Metastatic malignant neoplasm to respiratory and digestive systems (disorder)
        "269475001",  # Malignant tumor of lymphoid, hemopoietic AND/OR related tissue (disorder)
        "269476000",  # Nodular lymphoma (disorder)
        "269496008",  # Neoplasm of uncertain behavior of endocrine glands and nervous system (disorder)
        "269497004",  # Neoplasm of uncertain behavior of breast (disorder)
        "269515006",  # Carcinoma of lip (disorder)
        "269516007",  # Tongue carcinoma (disorder)
        "269533000",  # Carcinoma of colon (disorder)
        "269544008",  # Carcinoma of rectosigmoid junction (disorder)
        "269578002",  # Malignant melanoma of head and neck (disorder)
        "269579005",  # Malignant melanoma of trunk (disorder)
        "269580008",  # Malignant melanoma of upper limb (disorder)
        "269581007",  # Malignant melanoma of lower limb (disorder)
        "271323007",  # Malignant neoplasm of lip, oral cavity and/or pharynx (disorder)
        "271326004",  # Neoplasm of uncertain behavior of lip, oral cavity and/or pharynx (disorder)
        "271467005",  # Malignant neoplasm of bone, connective tissue, skin and breast (disorder)
        "271468000",  # Malignant neoplasm of genitourinary organ (disorder)
        "271482000",  # Neoplasm of uncertain behavior of liver and/or biliary passages (disorder)
        "271568003",  # Malignant tumor of lower labial mucosa (disorder)
        "271943005",  # Carcinoma of base of tongue (disorder)
        "274084007",  # Palate carcinoma (disorder)
        "274085008",  # Tonsil carcinoma (disorder)
        "274087000",  # Malignant melanoma of eye (disorder)
        "274088005",  # Metastatic malignant neoplasm to unknown site (disorder)
        "274902006",  # Combined hepatocellular carcinoma and cholangiocarcinoma (disorder)
        "274905008",  # Malignant lymphoma - lymphocytic, intermediate differentiation (disorder)
        "275266006",  # Metastasis to digestive organs (disorder)
        "275394001",  # Carcinoma ventral surface of tongue (disorder)
        "275395000",  # Carcinoma anterior 2/3 tongue ventrum (disorder)
        "275396004",  # Carcinoma of anterior two-thirds of tongue - dorsal surface (disorder)
        "275397008",  # Carcinoma of midline of tongue (disorder)
        "275399006",  # Malignant tumor of lipstick area of lip (disorder)
        "275490009",  # Carcinoma of tongue base - dorsal surface (disorder)
        "276419004",  # Malignant tumor of corpus spongiosum (disorder)
        "276420005",  # Malignant tumor of corpus cavernosum (disorder)
        "276738009",  # Primary signet ring cell carcinoma of skin (disorder)
        "276750003",  # Malignant tumor of skin with pilar differentiation (disorder)
        "276751004",  # Amelanotic malignant melanoma of skin (disorder)
        "276797002",  # Malignant tumor of fibrous tissue (disorder)
        "276799004",  # Dermatofibrosarcoma protuberans (disorder)
        "276803003",  # Adenocarcinoma of esophagus (disorder)
        "276804009",  # Squamous cell carcinoma of esophagus (disorder)
        "276809004",  # Early gastric cancer (disorder)
        "276810009",  # Late gastric cancer (disorder)
        "276811008",  # Gastric lymphoma (disorder)
        "276815004",  # Lymphoma of intestine (disorder)
        "276821000",  # Malignant melanoma of anus (disorder)
        "276822007",  # Malignant melanoma of rectum (disorder)
        "276826005",  # Malignant glioma of brain (disorder)
        "276827001",  # Malignant glioma of spinal cord (disorder)
        "276828006",  # Glioblastoma multiforme of brain (disorder)
        "276829003",  # Glioblastoma multiforme of spinal cord (disorder)
        "276836002",  # Primary cerebral lymphoma (disorder)
        "276860003",  # Squamous cell carcinoma of scrotum (disorder)
        "276870001",  # Carcinoma of fallopian tube (disorder)
        "276876007",  # Carcinoma of Bartholin's gland (disorder)
        "276952000",  # Squamous cell carcinoma of tongue (disorder)
        "276953005",  # Squamous cell carcinoma of gum (disorder)
        "276954004",  # Squamous cell carcinoma of floor of mouth (disorder)
        "276962007",  # Squamous cell carcinoma of palate (disorder)
        "276975007",  # Carcinoma of larynx (disorder)
        "277156006",  # Malignant tumor of external ear (disorder)
        "277461004",  # Anaplastic astrocytoma of brain (disorder)
        "277474005",  # B-cell chronic lymphocytic leukemia variant (disorder)
        "277505007",  # Medulloblastoma of cerebellum (disorder)
        "277507004",  # Pilocytic astrocytoma of cerebellum (disorder)
        "277530005",  # Malignant melanoma of meninges (disorder)
        "277543005",  # Malignant white blood cell disorder (disorder)
        "277545003",  # T-cell chronic lymphocytic leukemia (disorder)
        "277549009",  # Chronic lymphocytic prolymphocytic leukemia syndrome (disorder)
        "277550009",  # Richter's syndrome (disorder)
        "277551008",  # Splenic lymphoma with villous lymphocytes (disorder)
        "277567002",  # T-cell prolymphocytic leukemia (disorder)
        "277568007",  # Hairy cell leukemia variant (disorder)
        "277569004",  # Large granular lymphocytic leukemia (disorder)
        "277570003",  # Lymphoma with spill (disorder)
        "277571004",  # B-cell acute lymphoblastic leukemia (disorder)
        "277572006",  # Precursor B-cell acute lymphoblastic leukemia (disorder)
        "277573001",  # Common acute lymphoblastic leukemia (disorder)
        "277574007",  # Null cell acute lymphoblastic leukemia (disorder)
        "277575008",  # T-lymphoblastic leukemia (disorder)
        "277577000",  # Monoclonal gammopathy of uncertain significance (disorder)
        "277579002",  # Light chain myeloma (disorder)
        "277580004",  # Non-secretory myeloma (disorder)
        "277587001",  # Juvenile chronic myeloid leukemia (disorder)
        "277589003",  # Myelodysplastic-myeloproliferative neoplasm with neutrophilia (disorder)
        "277597005",  # Myelodysplastic syndrome with isolated del(5q) (disorder)
        "277602003",  # Acute megakaryoblastic leukemia (disorder)
        "277604002",  # Malignant myeloid-lymphoid neoplasm (disorder)
        "277609007",  # Hodgkin's disease, lymphocytic predominance - diffuse (disorder)
        "277610002",  # Hodgkin's disease, nodular sclerosis - lymphocytic predominance (disorder)
        "277611003",  # Hodgkin's disease, nodular sclerosis - mixed cellularity (disorder)
        "277612005",  # Hodgkin's disease, nodular sclerosis - lymphocytic depletion (disorder)
        "277613000",  # Cutaneous/peripheral T-cell lymphoma (disorder)
        "277614006",  # Prethymic and thymic T-cell lymphoma/leukemia (disorder)
        "277615007",  # Low grade B-cell lymphoma (disorder)
        "277616008",  # Diffuse low grade B-cell lymphoma (disorder)
        "277617004",  # High grade B-cell lymphoma (disorder)
        "277618009",  # Follicular low grade B-cell lymphoma (disorder)
        "277619001",  # B-cell prolymphocytic leukemia (disorder)
        "277622004",  # Mucosa-associated lymphoma (disorder)
        "277623009",  # Monocytoid B-cell lymphoma (disorder)
        "277624003",  # Follicular malignant lymphoma - mixed cell type (disorder)
        "277625002",  # Follicular malignant lymphoma - small cleaved cell (disorder)
        "277626001",  # Diffuse high grade B-cell lymphoma (disorder)
        "277627005",  # Nodular high grade B-cell lymphoma (disorder)
        "277628000",  # Diffuse malignant lymphoma - large cleaved cell (disorder)
        "277629008",  # Diffuse malignant lymphoma - large non-cleaved cell (disorder)
        "277632006",  # Diffuse malignant lymphoma - centroblastic polymorphic (disorder)
        "277642008",  # Low grade T-cell lymphoma (disorder)
        "277643003",  # High grade T-cell lymphoma (disorder)
        "277654008",  # Enteropathy-associated T-cell lymphoma (disorder)
        "277664004",  # Malignant lymphoma of testis (disorder)
        "277782009",  # Local recurrence of malignant neoplasm of peritoneum (disorder)
        "278024000",  # Rhabdomyosarcoma of urinary bladder (disorder)
        "278042005",  # Malignant teratoma of mediastinum (disorder)
        "278043000",  # Malignant seminoma of mediastinum (disorder)
        "278044006",  # Malignant neuroma of mediastinum (disorder)
        "278046008",  # Sarcoma of urinary bladder (disorder)
        "278050001",  # Sarcoma of breast (disorder)
        "278051002",  # Malignant lymphoma of thyroid gland (disorder)
        "278052009",  # Malignant lymphoma of breast (disorder)
        "278054005",  # Lobular carcinoma of breast (disorder)
        "278055006",  # Malignant Leydig cell tumor of testis (disorder)
        "278057003",  # Sertoli cell tumor of testis (disorder)
        "278060005",  # Endometrioid carcinoma of prostate (disorder)
        "278189009",  # Hypergranular promyelocytic leukemia (disorder)
        "278433008",  # Malignant infiltration of soft tissue (disorder)
        "278453007",  # Acute biphenotypic leukemia (disorder)
        "278491007",  # Mixed seminoma teratoma of testis (disorder)
        "280959007",  # Malignant tumor of lacrimal drainage structure (disorder)
        "281560004",  # Neuroblastoma of brain (disorder)
        "281561000",  # Sacrococcygeal teratoma (disorder)
        "281562007",  # Adrenal neuroblastoma (disorder)
        "281563002",  # Thoracic neuroblastoma (disorder)
        "281564008",  # Pelvic neuroblastoma (disorder)
        "281565009",  # Paraspinal neuroblastoma (disorder)
        "281566005",  # Abdominothoracic neuroblastoma (disorder)
        "281702006",  # Tibial adamantinoma (disorder)
        "285307007",  # Squamous cell carcinoma of skin of upper lip (disorder)
        "285308002",  # Squamous cell carcinoma of skin of lower lip (disorder)
        "285309005",  # Squamous cell carcinoma of skin of cheek (disorder)
        "285310000",  # Carcinoma of anal canal (disorder)
        "285312008",  # Carcinoma of sigmoid colon (disorder)
        "285420006",  # Immunoglobulin A myeloma (disorder)
        "285421005",  # Immunoglobulin G myeloma (disorder)
        "285422003",  # Immunoglobulin D myeloma (disorder)
        "285432005",  # Carcinoma of cervix (disorder)
        "285769009",  # Acute promyelocytic leukemia - hypogranular variant (disorder)
        "285776004",  # Intermediate grade B-cell lymphoma (disorder)
        "286887005",  # Carcinoma liver and/or biliary system (disorder)
        "286889008",  # Carcinoma of upper limb bones/scapula (disorder)
        "286890004",  # Carcinoma of lower limb bones (disorder)
        "286891000",  # Carcinoma of skin of head/neck (disorder)
        "286893002",  # Carcinoma of breast - upper, inner quadrant (disorder)
        "286894008",  # Carcinoma of breast - lower, inner quadrant (disorder)
        "286895009",  # Carcinoma of upper outer quadrant of breast (disorder)
        "286896005",  # Carcinoma breast - lower, outer quadrant (disorder)
        "286897001",  # Carcinoma of breast - axillary tail (disorder)
        "286899003",  # Carcinoma of genital organ (disorder)
        "286902000",  # Secondary carcinoma of gastrointestinal tract (disorder)
        "29421000119105",  # Adenocarcinoma in adenomatous polyp (disorder)
        "29431000119108",  # Adenocarcinoma in villous adenoma (disorder)
        "300988009",  # Transitional cell carcinoma of ureter (disorder)
        "301756000",  # Adenocarcinoma of sigmoid colon (disorder)
        "302815008",  # Malignant tumor of frenum of lip (disorder)
        "302816009",  # Malignant tumor of soft tissue of head, face and neck (disorder)
        "302833002",  # Paraganglioma (disorder)
        "302834008",  # Extra-adrenal paraganglioma (disorder)
        "302837001",  # Lentigo maligna melanoma (disorder)
        "302841002",  # Malignant lymphoma - small lymphocytic (disorder)
        "302842009",  # Diffuse malignant lymphoma - centroblastic (disorder)
        "302845006",  # Nodular malignant lymphoma, lymphocytic - well differentiated (disorder)
        "302847003",  # Rhabdomyosarcoma (disorder)
        "302848008",  # Nodular malignant lymphoma, lymphocytic - intermediate differentiation (disorder)
        "302849000",  # Nephroblastoma (disorder)
        "302851001",  # Synovial sarcoma (disorder)
        "302855005",  # Subacute leukemia (disorder)
        "302856006",  # Aleukemic leukemia (disorder)
        "303012000",  # Malignant tumor of posterior wall of hypopharynx (disorder)
        "303055001",  # Malignant lymphoma, follicular center cell (disorder)
        "303056000",  # Malignant lymphoma, follicular center cell, cleaved (disorder)
        "303057009",  # Malignant lymphoma, follicular center cell, non-cleaved (disorder)
        "303194003",  # Metastasis to head and neck lymph node (disorder)
        "303201005",  # Metastatic malignant neoplasm to multiple lymph nodes (disorder)
        "304545002",  # Adenocarcinoma of ileum (disorder)
        "30664006",  # Multiple endocrine neoplasia, type 1 (disorder)
        "307216009",  # Perforated carcinoma of esophagus (disorder)
        "307219002",  # Retroperitoneal sarcoma (disorder)
        "307226002",  # Metastatic adenocarcinoma of unknown origin (disorder)
        "307340003",  # Monosomy 7 syndrome (disorder)
        "307341004",  # Atypical hairy cell leukemia (disorder)
        "307501007",  # Transglottic tumor (disorder)
        "307502000",  # Squamous cell carcinoma of mouth (disorder)
        "307576001",  # Osteosarcoma of bone (disorder)
        "307591004",  # Malignant mastocytosis (disorder)
        "307592006",  # Basophilic leukemia (disorder)
        "307593001",  # Carcinomatosis (disorder)
        "307599002",  # Sebaceous adenocarcinoma (disorder)
        "307601000",  # Pseudomyxoma peritonei (disorder)
        "307603002",  # Malignant blue nevus of skin (disorder)
        "307604008",  # Mesoblastic nephroma (disorder)
        "307608006",  # Ewing's sarcoma of bone (disorder)
        "307609003",  # Adamantinoma of long bone (disorder)
        "307610008",  # Pilomatrix carcinoma of skin (disorder)
        "307617006",  # Neutrophilic leukemia (disorder)
        "307622006",  # Prolymphocytic lymphosarcoma (disorder)
        "307623001",  # Malignant lymphoma - lymphoplasmacytic (disorder)
        "307624007",  # Diffuse malignant lymphoma - centroblastic-centrocytic (disorder)
        "307625008",  # Malignant lymphoma - centrocytic (disorder)
        "307633009",  # Hodgkin's disease, lymphocytic depletion, diffuse fibrosis (disorder)
        "307634003",  # Hodgkin's disease, lymphocytic depletion, reticular type (disorder)
        "307635002",  # Hodgkin's disease, nodular sclerosis - cellular phase (disorder)
        "307636001",  # Malignant lymphoma, mixed lymphocytic-histiocytic, nodular (disorder)
        "307637005",  # Malignant lymphoma, centroblastic-centrocytic, follicular (disorder)
        "307646004",  # Malignant lymphoma, lymphocytic, poorly differentiated, nodular (disorder)
        "307647008",  # Malignant lymphoma, centroblastic type, follicular (disorder)
        "307649006",  # Microglioma (disorder)
        "307650006",  # Histiocytic medullary reticulosis (disorder)
        "307651005",  # Primary myelofibrosis (disorder)
        "308121000",  # Follicular lymphoma (disorder)
        "309245001",  # Adenocarcinoma of uterus (disorder)
        "31047003",  # Lymphomatoid papulosis (disorder)
        "310498001",  # Malignant melanoma of back (disorder)
        "310504009",  # Primary malignant neoplasm of unknown site (disorder)
        "310526005",  # Malignant tumor of soft tissue of back (disorder)
        "310599006",  # Malignant neoplasm of canthus (disorder)
        "311779007",  # Malignant neoplasm of skin of scapular region (disorder)
        "312104005",  # Cholangiocarcinoma of biliary tract (disorder)
        "312111009",  # Carcinoma of ascending colon (disorder)
        "312112002",  # Carcinoma of transverse colon (disorder)
        "312113007",  # Carcinoma of descending colon (disorder)
        "312114001",  # Carcinoma of hepatic flexure (disorder)
        "312115000",  # Carcinoma of splenic flexure (disorder)
        "312949007",  # Retinal pigment epithelial adenocarcinoma (disorder)
        "313248004",  # Malignant melanoma of chest wall (disorder)
        "313249007",  # Malignant neoplasm of upper eyelid (disorder)
        "313250007",  # Malignant neoplasm of lower eyelid (disorder)
        "313353007",  # Squamous cell carcinoma of bronchus in left lower lobe (disorder)
        "313354001",  # Squamous cell carcinoma of bronchus in left upper lobe (disorder)
        "313355000",  # Squamous cell carcinoma of bronchus in right lower lobe (disorder)
        "313356004",  # Squamous cell carcinoma of bronchus of middle lobe of right lung (disorder)
        "313357008",  # Squamous cell carcinoma of bronchus in right upper lobe (disorder)
        "313427003",  # Lambda light chain myeloma (disorder)
        "313428008",  # Seminoma of undescended testis (disorder)
        "313429000",  # Seminoma of descended testis (disorder)
        "314191009",  # Cystadenocarcinoma of ovary (disorder)
        "314408000",  # Leukemic infiltrate of choroid (disorder)
        "314418005",  # Leukemic infiltrate of retina (disorder)
        "314419002",  # Bilateral diffuse uveal melanocytic proliferation (disorder)
        "314951005",  # Local recurrence of malignant tumor of tongue (disorder)
        "314952003",  # Local recurrence of malignant tumor of buccal cavity (disorder)
        "314953008",  # Local recurrence of malignant tumor of thyroid gland (disorder)
        "314954002",  # Local recurrence of malignant tumor of lung (disorder)
        "314955001",  # Local recurrence of malignant tumor of breast (disorder)
        "314960002",  # Local recurrence of malignant tumor of esophagus (disorder)
        "314961003",  # Local recurrence of malignant tumor of stomach (disorder)
        "314962005",  # Local recurrence of malignant tumor of gallbladder (disorder)
        "314963000",  # Local recurrence of malignant tumor of liver (disorder)
        "314964006",  # Local recurrence of malignant tumor of pancreas (disorder)
        "314965007",  # Local recurrence of malignant tumor of colon (disorder)
        "314966008",  # Local recurrence of malignant tumor of rectum (disorder)
        "314967004",  # Local recurrence of malignant tumor of kidney (disorder)
        "314968009",  # Local recurrence of malignant tumor of urinary bladder (disorder)
        "314969001",  # Local recurrence of malignant tumor of prostate (disorder)
        "314970000",  # Local recurrence of malignant tumor of cervix (disorder)
        "314973003",  # Local recurrence of malignant tumor of bone (disorder)
        "314974009",  # Local recurrence of malignant tumor of soft tissue (disorder)
        "314975005",  # Local recurrence of malignant tumor of skin (disorder)
        "314976006",  # Local recurrence of malignant melanoma of skin (disorder)
        "315058005",  # Hereditary nonpolyposis colon cancer (disorder)
        "326072005",  # Carcinoma of head of pancreas (disorder)
        "33581000119109",  # Primary malignant neoplasm of extradural spinal cord (disorder)
        "350621000119100",  # Primary malignant neoplasm of right vestibulocochlear nerve (disorder)
        "350631000119102",  # Primary malignant neoplasm of left vestibulocochlear nerve (disorder)
        "350681000119101",  # Primary malignant neoplasm of adrenal medulla of right adrenal gland (disorder)
        "350801000119105",  # Primary malignant neoplasm of left adrenal cortex (disorder)
        "350851000119109",  # Primary malignant neoplasm of adrenal medulla of left adrenal gland (disorder)
        "350861000119106",  # Primary malignant neoplasm of right adrenal gland (disorder)
        "350881000119102",  # Primary malignant neoplasm of right adrenal cortex (disorder)
        "350921000119109",  # Primary malignant neoplasm of left adrenal gland (disorder)
        "350951000119101",  # B-cell lymphoma of intra-abdominal lymph nodes (disorder)
        "351041000119109",  # Primary malignant neoplasm of left upper limb (disorder)
        "351051000119106",  # Primary malignant neoplasm of right upper limb (disorder)
        "351211000119104",  # B-cell lymphoma of lymph nodes of multiple sites (disorder)
        "351961000119109",  # Malignant melanoma of skin of right upper limb (disorder)
        "352001000119100",  # Malignant melanoma of skin of right lower limb (disorder)
        "352201000119105",  # Malignant melanoma of skin of left upper limb (disorder)
        "352231000119103",  # Primary malignant neoplasm of soft tissue of left upper extremity (disorder)
        "352251000119109",  # Small lymphocytic B-cell lymphoma of lymph nodes of multiple sites (disorder)
        "352301000119103",  # Primary malignant neoplasm of soft tissue of right upper extremity (disorder)
        "352311000119100",  # Primary malignant neoplasm of soft tissue of right lower limb (disorder)
        "352321000119107",  # Primary malignant neoplasm of soft tissue of left lower limb (disorder)
        "352331000119105",  # Primary malignant neoplasm of left spermatic cord (disorder)
        "352341000119101",  # Primary malignant neoplasm of right spermatic cord (disorder)
        "352381000119106",  # Primary malignant neoplasm of skin of left upper limb (disorder)
        "352391000119109",  # Primary malignant neoplasm of skin of left ear (disorder)
        "352411000119109",  # Small lymphocytic B-cell lymphoma of intra-abdominal lymph nodes (disorder)
        "352421000119102",  # Primary malignant neoplasm of skin of left shoulder (disorder)
        "352431000119104",  # Primary malignant neoplasm of skin of right ear (disorder)
        "352461000119107",  # Primary malignant neoplasm of skin of right lower limb (disorder)
        "352471000119101",  # Primary malignant neoplasm of skin of right shoulder (disorder)
        "352481000119103",  # Primary malignant neoplasm of skin of left lower limb (disorder)
        "352551000119106",  # Primary malignant neoplasm of left descended testis (disorder)
        "352581000119104",  # Primary malignant neoplasm of right descended testis (disorder)
        "352701000119102",  # Primary malignant neoplasm of neck of pancreas (disorder)
        "352731000119109",  # Primary malignant neoplasm of peripheral nerves of left lower limb (disorder)
        "352751000119103",  # Primary malignant neoplasm of peripheral nerves of right upper limb (disorder)
        "352771000119107",  # Primary malignant neoplasm of peripheral nerves of right lower limb (disorder)
        "352791000119108",  # Non-Hodgkin's lymphoma of lymph nodes of multiple sites (disorder)
        "352911000119101",  # Primary malignant neoplasm of left optic nerve (disorder)
        "352921000119108",  # Primary malignant neoplasm of right optic nerve (disorder)
        "352961000119103",  # Primary malignant neoplasm of right renal pelvis (disorder)
        "353061000119107",  # Primary malignant neoplasm of retina of right eye (disorder)
        "353081000119103",  # Primary malignant neoplasm of retina of left eye (disorder)
        "353111000119108",  # Primary malignant neoplasm of left renal pelvis (disorder)
        "353211000119103",  # Extramedullary plasmacytoma in remission (disorder)
        "353561000119103",  # Metastatic malignant neoplasm to right lung (disorder)
        "353571000119109",  # Metastatic malignant neoplasm to right adrenal gland (disorder)
        "353661000119102",  # Overlapping primary malignant neoplasm of bone and articular cartilage of lower limb (disorder)
        "353671000119108",  # Overlapping primary malignant neoplasm of bone and articular cartilage of right upper limb (disorder)
        "353721000119100",  # Metastatic malignant neoplasm to left adrenal gland (disorder)
        "353741000119106",  # Metastatic malignant neoplasm to left lung (disorder)
        "353761000119105",  # Primary malignant neoplasm of cornea of left eye (disorder)
        "353791000119103",  # Primary malignant neoplasm of conjunctiva of right eye (disorder)
        "353801000119102",  # Primary malignant neoplasm of choroid of right eye (disorder)
        "353811000119104",  # Primary malignant neoplasm of cornea of right eye (disorder)
        "353841000119100",  # Primary malignant neoplasm of conjunctiva of left eye (disorder)
        "354081000119101",  # Metastatic adenocarcinoma (disorder)
        "354301000119106",  # Primary malignant neoplasm of choroid of left eye (disorder)
        "354331000119104",  # Primary malignant neoplasm of left lower limb (disorder)
        "354351000119105",  # Primary malignant neoplasm of left kidney (disorder)
        "354361000119107",  # Primary malignant neoplasm of right kidney (disorder)
        "354391000119100",  # Kaposi sarcoma of left lung (disorder)
        "354421000119107",  # Primary malignant neoplasm of right lower limb (disorder)
        "354461000119102",  # Kaposi sarcoma of right lung (disorder)
        "354621000119105",  # Overlapping primary malignant neoplasm of bone and articular cartilage of left upper limb (disorder)
        "354641000119104",  # Primary malignant neoplasm of bone of left upper limb (disorder)
        "354651000119102",  # Primary malignant neoplasm of bone of left lower limb (disorder)
        "354671000119106",  # Primary malignant neoplasm of bone of right upper limb (disorder)
        "354681000119109",  # Primary malignant neoplasm of bone of right lower limb (disorder)
        "354721000119103",  # Primary malignant neoplasm of left main bronchus (disorder)
        "354731000119100",  # Primary malignant neoplasm of right main bronchus (disorder)
        "354851000119101",  # Follicular non-Hodgkin's lymphoma of lymph nodes of multiple sites (disorder)
        "355141000119106",  # Primary malignant neoplasm of skin of right upper limb (disorder)
        "35868009",  # Carcinoid syndrome (disorder)
        "359619007",  # Pinealoma (disorder)
        "359631009",  # Acute myeloid leukemia, minimal differentiation, FAB M0 (disorder)
        "359640008",  # Acute myeloid leukemia without maturation, FAB M1 (disorder)
        "359648001",  # Acute myeloid leukemia with maturation, FAB M2 (disorder)
        "359780007",  # Metastatic malignant neoplasm to lateral axillary lymph nodes (disorder)
        "359782004",  # Metastatic malignant neoplasm to apex of urinary bladder (disorder)
        "359785002",  # Metastatic malignant neoplasm to dome of urinary bladder (disorder)
        "359987004",  # Krukenberg tumor (disorder)
        "363346000",  # Malignant neoplastic disease (disorder)
        "363348004",  # Malignant tumor of lip (disorder)
        "363349007",  # Malignant tumor of stomach (disorder)
        "363350007",  # Malignant tumor of cecum (disorder)
        "363351006",  # Malignant tumor of rectum (disorder)
        "363352004",  # Malignant tumor of anal canal (disorder)
        "363353009",  # Malignant tumor of gallbladder (disorder)
        "363354003",  # Malignant tumor of cervix (disorder)
        "363355002",  # Malignant tumor of adrenal gland (disorder)
        "363358000",  # Malignant tumor of lung (disorder)
        "363359008",  # Malignant tumor of middle ear (disorder)
        "363360003",  # Malignant tumor of anterior two-thirds of tongue (disorder)
        "363363001",  # Malignant tumor of soft tissue of shoulder (disorder)
        "363364007",  # Malignant tumor of soft tissue of hip (disorder)
        "363365008",  # Malignant tumor of soft tissue of thorax (disorder)
        "363366009",  # Malignant tumor of soft tissue of pelvis (disorder)
        "363367000",  # Malignant tumor of vulva (disorder)
        "363368005",  # Carcinoma of body of pancreas (disorder)
        "363369002",  # Carcinoma of tail of pancreas (disorder)
        "363370001",  # Malignant neoplasm of mesentery (disorder)
        "363372009",  # Malignant tumor of vermilion border of upper lip (disorder)
        "363373004",  # Malignant tumor of vermilion border of lower lip (disorder)
        "363374005",  # Malignant tumor of commissure of lip (disorder)
        "363375006",  # Malignant tumor of tongue (disorder)
        "363376007",  # Malignant tumor of base of tongue (disorder)
        "363377003",  # Malignant tumor of lingual tonsil (disorder)
        "363378008",  # Malignant tumor of major salivary gland (disorder)
        "363379000",  # Malignant tumor of parotid gland (disorder)
        "363380002",  # Malignant tumor of submandibular gland (disorder)
        "363381003",  # Malignant tumor of sublingual gland (disorder)
        "363382005",  # Malignant tumor of gum (disorder)
        "363383000",  # Malignant tumor of upper gingiva (disorder)
        "363384006",  # Malignant tumor of lower gingiva (disorder)
        "363385007",  # Malignant tumor of floor of mouth (disorder)
        "363386008",  # Malignant tumor of buccal mucosa (disorder)
        "363387004",  # Malignant tumor of hard palate (disorder)
        "363388009",  # Malignant tumor of soft palate (disorder)
        "363389001",  # Malignant tumor of uvula (disorder)
        "363390005",  # Malignant tumor of palate (disorder)
        "363391009",  # Malignant tumor of retromolar area (disorder)
        "363392002",  # Malignant tumor of oropharynx (disorder)
        "363393007",  # Malignant tumor of tonsil (disorder)
        "363394001",  # Malignant tumor of tonsillar fossa (disorder)
        "363395000",  # Malignant tumor of vallecula (disorder)
        "363396004",  # Malignant tumor of branchial cleft (disorder)
        "363398003",  # Malignant tumor of lateral wall of nasopharynx (disorder)
        "363399006",  # Malignant tumor of hypopharynx (disorder)
        "363400004",  # Malignant tumor of postcricoid region (disorder)
        "363401000",  # Malignant tumor of pyriform fossa (disorder)
        "363402007",  # Malignant tumor of esophagus (disorder)
        "363403002",  # Malignant tumor of duodenum (disorder)
        "363404008",  # Malignant tumor of jejunum (disorder)
        "363405009",  # Malignant tumor of ileum (disorder)
        "363406005",  # Malignant neoplasm of colon (disorder)
        "363407001",  # Malignant tumor of hepatic flexure (disorder)
        "363408006",  # Malignant tumor of transverse colon (disorder)
        "363409003",  # Malignant tumor of descending colon (disorder)
        "363410008",  # Malignant tumor of sigmoid colon (disorder)
        "363411007",  # Malignant neoplasm of appendix (disorder)
        "363412000",  # Malignant tumor of ascending colon (disorder)
        "363413005",  # Malignant tumor of splenic flexure (disorder)
        "363414004",  # Malignant tumor of rectosigmoid junction (disorder)
        "363415003",  # Malignant tumor of biliary tract (disorder)
        "363416002",  # Malignant tumor of extrahepatic bile duct (disorder)
        "363417006",  # Malignant tumor of ampulla of Vater (disorder)
        "363418001",  # Malignant tumor of pancreas (disorder)
        "363419009",  # Malignant tumor of head of pancreas (disorder)
        "363420003",  # Malignant retroperitoneal tumor (disorder)
        "363421004",  # Malignant neoplasm of omentum (disorder)
        "363422006",  # Malignant tumor of nasal cavity (disorder)
        "363423001",  # Malignant tumor of nasal septum (disorder)
        "363424007",  # Malignant tumor of mastoid air cells (disorder)
        "363425008",  # Malignant tumor of maxillary sinus (disorder)
        "363426009",  # Malignant tumor of ethmoid sinus (disorder)
        "363427000",  # Malignant tumor of frontal sinus (disorder)
        "363428005",  # Malignant tumor of sphenoid sinus (disorder)
        "363429002",  # Malignant tumor of larynx (disorder)
        "363430007",  # Malignant tumor of subglottis (disorder)
        "363431006",  # Malignant tumor of laryngeal cartilage (disorder)
        "363432004",  # Malignant tumor of trachea (disorder)
        "363433009",  # Malignant tumor of pleura (disorder)
        "363434003",  # Malignant tumor of thymus (disorder)
        "363435002",  # Malignant tumor of heart (disorder)
        "363436001",  # Malignant tumor of endocardium (disorder)
        "363437005",  # Malignant tumor of myocardium (disorder)
        "363438000",  # Malignant neoplasm of vertebral column (disorder)
        "363439008",  # Malignant tumor of soft tissue of head (disorder)
        "363440005",  # Malignant tumor of soft tissue of face (disorder)
        "363441009",  # Malignant tumor of soft tissue of neck (disorder)
        "363443007",  # Malignant tumor of ovary (disorder)
        "363444001",  # Malignant tumor of fallopian tube (disorder)
        "363445000",  # Malignant tumor of vagina (disorder)
        "363446004",  # Malignant neoplasm of labia majora (disorder)
        "363447008",  # Malignant neoplasm of labia minora (disorder)
        "363449006",  # Malignant tumor of testis (disorder)
        "363450006",  # Malignant tumor of foreskin (disorder)
        "363451005",  # Malignant tumor of glans penis (disorder)
        "363452003",  # Malignant tumor of epididymis (disorder)
        "363453008",  # Malignant tumor of spermatic cord (disorder)
        "363454002",  # Malignant tumor of scrotum (disorder)
        "363456000",  # Malignant tumor of urachus (disorder)
        "363457009",  # Malignant tumor of renal pelvis (disorder)
        "363458004",  # Malignant tumor of ureter (disorder)
        "363459007",  # Malignant tumor of urethra (disorder)
        "363460002",  # Malignant tumor of paraurethral gland (disorder)
        "363461003",  # Malignant tumor of eye (disorder)
        "363462005",  # Malignant tumor of orbit (disorder)
        "363463000",  # Malignant tumor of conjunctiva (disorder)
        "363464006",  # Malignant tumor of cornea (disorder)
        "363465007",  # Malignant tumor of retina (disorder)
        "363466008",  # Malignant tumor of choroid (disorder)
        "363467004",  # Malignant neoplasm of frontal lobe (disorder)
        "363468009",  # Malignant neoplasm of temporal lobe (disorder)
        "363469001",  # Malignant neoplasm of parietal lobe (disorder)
        "363470000",  # Malignant neoplasm of occipital lobe (disorder)
        "363471001",  # Malignant neoplasm of cerebral ventricles (disorder)
        "363473003",  # Malignant neoplasm of brainstem (disorder)
        "363474009",  # Malignant neoplasm of cerebral meninges (disorder)
        "363475005",  # Malignant tumor of spinal cord (disorder)
        "363476006",  # Malignant neoplasm of spinal meninges (disorder)
        "363477002",  # Malignant neoplasm of cauda equina (disorder)
        "363478007",  # Malignant tumor of thyroid gland (disorder)
        "363481002",  # Malignant tumor of parathyroid gland (disorder)
        "363482009",  # Malignant tumor of pituitary gland (disorder)
        "363483004",  # Malignant tumor of pineal gland (disorder)
        "363484005",  # Malignant tumor of pelvis (disorder)
        "363485006",  # Malignant tumor of minor salivary gland (disorder)
        "363486007",  # Malignant tumor of vocal cord (disorder)
        "363487003",  # Malignant tumor of aryepiglottic fold - laryngeal aspect (disorder)
        "363488008",  # Malignant tumor of false cord (disorder)
        "363489000",  # Malignant tumor of neck (disorder)
        "363490009",  # Malignant tumor of anus (disorder)
        "363491008",  # Malignant tumor of cloacogenic zone (disorder)
        "363492001",  # Malignant tumor of peritoneum (disorder)
        "363493006",  # Malignant tumor of bronchus (disorder)
        "363494000",  # Malignant tumor of mediastinum (disorder)
        "363495004",  # Malignant tumor of muscle (disorder)
        "363496003",  # Malignant tumor of soft tissue of abdomen (disorder)
        "363497007",  # Malignant tumor of meninges (disorder)
        "363498002",  # Malignant tumor of optic nerve (disorder)
        "363499005",  # Malignant tumor of spleen (disorder)
        "363500001",  # Multiple malignancy (disorder)
        "363501002",  # Malignant tumor of face (disorder)
        "363502009",  # Malignant tumor of axilla (disorder)
        "363503004",  # Malignant tumor of upper limb (disorder)
        "363504005",  # Malignant tumor of lower limb (disorder)
        "363505006",  # Malignant tumor of oral cavity (disorder)
        "363506007",  # Malignant tumor of nasal sinuses (disorder)
        "363507003",  # Malignant neoplasm of pharynx (disorder)
        "363508008",  # Malignant tumor of intestine (disorder)
        "363509000",  # Malignant tumor of small intestine (disorder)
        "363510005",  # Malignant tumor of large intestine (disorder)
        "363514001",  # Malignant tumor of female genital organ (disorder)
        "363515000",  # Malignant tumor of male genital organ (disorder)
        "363516004",  # Malignant tumor of penis (disorder)
        "363517008",  # Malignant tumor of urinary tract proper (disorder)
        "363518003",  # Malignant tumor of kidney (disorder)
        "363745004",  # Primary malignant neoplasm of gastrointestinal tract (disorder)
        "369448007",  # Malignant tumor involving rectum by direct extension from endometrium (disorder)
        "369449004",  # Malignant tumor involving rectum by direct extension from fallopian tube (disorder)
        "369450004",  # Malignant tumor involving rectum by direct extension from ovary (disorder)
        "369451000",  # Malignant tumor involving rectum by direct extension from prostate (disorder)
        "369452007",  # Malignant tumor involving rectum by direct extension from uterine cervix (disorder)
        "369453002",  # Malignant tumor involving rectum by direct extension from uterus (disorder)
        "369454008",  # Malignant tumor involving rectum by direct extension from vagina (disorder)
        "369463005",  # Malignant tumor involving ureter by direct extension from bladder (disorder)
        "369465003",  # Malignant tumor involving urethra by direct extension from bladder (disorder)
        "369466002",  # Malignant tumor involving urethra by direct extension from prostate (disorder)
        "369469009",  # Malignant neoplasm involving urinary bladder by direct extension from endometrium (disorder)
        "369470005",  # Malignant neoplasm involving urinary bladder by direct extension from fallopian tube (disorder)
        "369471009",  # Malignant neoplasm involving urinary bladder by direct extension from ovary (disorder)
        "369472002",  # Malignant neoplasm involving urinary bladder by direct extension from prostate (disorder)
        "369473007",  # Malignant neoplasm involving urinary bladder by direct extension from uterine cervix (disorder)
        "369474001",  # Malignant neoplasm involving urinary bladder by direct extension from uterus (disorder)
        "369475000",  # Malignant neoplasm involving urinary bladder by direct extension from vagina (disorder)
        "369483006",  # Malignant tumor involving vasa deferentia by direct extension from prostate (disorder)
        "369485004",  # Malignant tumor involving prostate by direct extension from bladder (disorder)
        "369487007",  # Primary malignant neoplasm of seminal vesicle (disorder)
        "369488002",  # Metastatic malignant neoplasm to seminal vesicle (disorder)
        "369489005",  # Malignant tumor involving seminal vesicle by direct extension from bladder (disorder)
        "369490001",  # Malignant tumor involving seminal vesicle by direct extension from prostate (disorder)
        "369493004",  # Malignant tumor involving uterine corpus by direct extension from bladder (disorder)
        "369494005",  # Malignant tumor involving uterine corpus by direct extension from ovary (disorder)
        "369495006",  # Malignant tumor involving uterine corpus by direct extension from uterine cervix (disorder)
        "369496007",  # Malignant tumor involving uterine corpus by direct extension from vagina (disorder)
        "369497003",  # Malignant tumor involving uterine cervix by direct extension from fallopian tube (disorder)
        "369498008",  # Malignant tumor involving uterine cervix by direct extension from ovary (disorder)
        "369499000",  # Malignant tumor involving uterine cervix by direct extension from vagina (disorder)
        "369503006",  # Malignant tumor involving vagina by direct extension from uterus (disorder)
        "369504000",  # Malignant tumor involving vagina by direct extension from bladder (disorder)
        "369505004",  # Malignant tumor involving vagina by direct extension from endometrium (disorder)
        "369506003",  # Malignant tumor involving vagina by direct extension from fallopian tube (disorder)
        "369507007",  # Malignant tumor involving vulva by direct extension from endometrium (disorder)
        "369508002",  # Malignant tumor involving vulva by direct extension from fallopian tube (disorder)
        "369509005",  # Malignant tumor involving vulva by direct extension from ovary (disorder)
        "369510000",  # Malignant tumor involving vulva by direct extension from uterine cervix (disorder)
        "369511001",  # Malignant tumor involving vulva by direct extension from uterus (disorder)
        "369512008",  # Malignant tumor involving vagina by direct extension from ovary (disorder)
        "369513003",  # Primary malignant neoplasm of left fallopian tube (disorder)
        "369514009",  # Metastatic malignant neoplasm to left fallopian tube (disorder)
        "369515005",  # Malignant tumor involving left fallopian tube by direct extension from endometrium (disorder)
        "369516006",  # Malignant tumor involving left fallopian tube by direct extension from ovary (disorder)
        "369517002",  # Malignant tumor involving left fallopian tube by direct extension from right fallopian tube (disorder)
        "369518007",  # Malignant tumor involving left fallopian tube by direct extension from uterine cervix (disorder)
        "369519004",  # Malignant tumor involving left fallopian tube by direct extension from uterus (disorder)
        "369520005",  # Primary malignant neoplasm of right fallopian tube (disorder)
        "369521009",  # Metastatic malignant neoplasm to right fallopian tube (disorder)
        "369522002",  # Primary malignant neoplasm of left ovary (disorder)
        "369523007",  # Metastatic malignant neoplasm to left ovary (disorder)
        "369524001",  # Malignant tumor involving left ovary by direct extension from endometrium (disorder)
        "369525000",  # Malignant tumor involving left ovary by direct extension from fallopian tube (disorder)
        "369526004",  # Malignant tumor involving left ovary by direct extension from right ovary (disorder)
        "369527008",  # Malignant tumor involving left ovary by direct extension from uterine cervix (disorder)
        "369528003",  # Malignant tumor involving left ovary by direct extension from uterus (disorder)
        "369529006",  # Primary malignant neoplasm of right ovary (disorder)
        "369530001",  # Metastatic malignant neoplasm to right ovary (disorder)
        "369531002",  # Malignant tumor involving right ovary by direct extension from endometrium (disorder)
        "369532009",  # Malignant tumor involving right ovary by direct extension from fallopian tube (disorder)
        "369533004",  # Malignant tumor involving right ovary by direct extension from left ovary (disorder)
        "369534005",  # Malignant tumor involving right ovary by direct extension from uterine cervix (disorder)
        "369535006",  # Metastatic neoplasm of left broad ligament (disorder)
        "369536007",  # Metastatic neoplasm of right broad ligament (disorder)
        "369541004",  # Malignant tumor involving left fallopian tube by direct extension from vagina (disorder)
        "369547000",  # Malignant tumor involving right fallopian tube by direct extension from endometrium (disorder)
        "369548005",  # Malignant tumor involving right fallopian tube by direct extension from left fallopian tube (disorder)
        "369549002",  # Malignant tumor involving right fallopian tube by direct extension from ovary (disorder)
        "369550002",  # Malignant tumor involving right fallopian tube by direct extension from uterine cervix (disorder)
        "369551003",  # Malignant tumor involving right fallopian tube by direct extension from uterus (disorder)
        "369552005",  # Malignant tumor involving right fallopian tube by direct extension from vagina (disorder)
        "369559001",  # Malignant tumor involving left ovary by direct extension from vagina (disorder)
        "369566000",  # Malignant tumor involving right ovary by direct extension from uterus (disorder)
        "369567009",  # Malignant tumor involving right ovary by direct extension from vagina (disorder)
        "369579009",  # Malignant tumor involving uterine corpus by direct extension from fallopian tube (disorder)
        "369580007",  # Malignant tumor involving vagina by direct extension from uterine cervix (disorder)
        "369587005",  # Malignant tumor involving vulva by direct extension from vagina (disorder)
        "369594008",  # Malignant tumor involving an organ by direct extension from bladder (disorder)
        "369595009",  # Malignant tumor involving an organ by direct extension from endometrium (disorder)
        "369596005",  # Malignant tumor involving an organ by direct extension from fallopian tube (disorder)
        "369597001",  # Malignant tumor involving an organ by direct extension from ovary (disorder)
        "369598006",  # Malignant tumor involving an organ by direct extension from prostate (disorder)
        "369599003",  # Malignant tumor involving an organ by direct extension from uterine cervix (disorder)
        "369600000",  # Malignant tumor involving an organ by direct extension from uterus (disorder)
        "369601001",  # Malignant tumor involving an organ by direct extension from vagina (disorder)
        "370511006",  # Vaccine-induced fibrosarcoma (disorder)
        "370967009",  # Retinoblastoma (disorder)
        "370987005",  # Anaplastic astrocytoma of spinal cord (disorder)
        "371012000",  # Acute lymphoblastic leukemia, transitional pre-B-cell (disorder)
        "371134001",  # Malignant lymphoma, large cell, polymorphous, immunoblastic (disorder)
        "371962007",  # Primary malignant neoplasm of abdominal esophagus (disorder)
        "371963002",  # Primary malignant neoplasm of adrenal cortex (disorder)
        "371964008",  # Malignant neoplasm of adrenal cortex (disorder)
        "371965009",  # Malignant neoplasm of adrenal medulla (disorder)
        "371966005",  # Primary malignant neoplasm of adrenal medulla (disorder)
        "371967001",  # Primary malignant neoplasm of ampulla of Vater (disorder)
        "371968006",  # Primary malignant neoplasm of anterior two-thirds of tongue (disorder)
        "371969003",  # Primary malignant neoplasm of apex of urinary bladder (disorder)
        "371970002",  # Primary malignant neoplasm of biliary tract (disorder)
        "371971003",  # Primary malignant neoplasm of body of uterus (disorder)
        "371972005",  # Malignant neoplasm of body of uterus (disorder)
        "371973000",  # Malignant neoplasm of uterus (disorder)
        "371974006",  # Malignant neoplasm of border of tongue (disorder)
        "371975007",  # Primary malignant neoplasm of border of tongue (disorder)
        "371976008",  # Primary malignant neoplasm of buccal mucosa (disorder)
        "371977004",  # Primary malignant neoplasm of cecum (disorder)
        "371978009",  # Primary malignant neoplasm of cervical esophagus (disorder)
        "371979001",  # Malignant neoplasm of clitoris (disorder)
        "371980003",  # Primary malignant neoplasm of clitoris (disorder)
        "371981004",  # Primary malignant neoplasm of commissure of lip (disorder)
        "371982006",  # Malignant neoplasm of endocrine gland (disorder)
        "371983001",  # Primary malignant neoplasm of endocrine gland (disorder)
        "371984007",  # Primary malignant neoplasm of esophagus (disorder)
        "371986009",  # Primary malignant neoplasm of eye (disorder)
        "371987000",  # Primary malignant neoplasm of fallopian tube (disorder)
        "371988005",  # Primary malignant neoplasm of false vocal cord (disorder)
        "371989002",  # Primary malignant neoplasm of glans penis (disorder)
        "371990006",  # Primary malignant neoplasm of gum (disorder)
        "371991005",  # Primary malignant neoplasm of hard palate (disorder)
        "371992003",  # Primary malignant neoplasm of intestinal tract (disorder)
        "371993008",  # Primary malignant neoplasm of lacrimal gland (disorder)
        "371994002",  # Primary malignant neoplasm of laryngeal aspect of aryepiglottic fold (disorder)
        "371995001",  # Primary malignant neoplasm of larynx (disorder)
        "371996000",  # Primary malignant neoplasm of lip (disorder)
        "371997009",  # Primary malignant neoplasm of lower gum (disorder)
        "371998004",  # Primary malignant neoplasm of lower third of esophagus (disorder)
        "371999007",  # Primary malignant neoplasm of middle third of esophagus (disorder)
        "372000001",  # Primary malignant neoplasm of minor salivary gland (disorder)
        "372001002",  # Primary malignant neoplasm of oral cavity (disorder)
        "372002009",  # Primary malignant neoplasm of palate (disorder)
        "372003004",  # Primary malignant neoplasm of pancreas (disorder)
        "372004005",  # Primary malignant neoplasm of parotid gland (disorder)
        "372005006",  # Primary malignant neoplasm of penis (disorder)
        "372006007",  # Primary malignant neoplasm of prepuce (disorder)
        "372007003",  # Primary malignant neoplasm of retrocecal tissue (disorder)
        "372008008",  # Primary malignant neoplasm of scaphoid bone (disorder)
        "372009000",  # Primary malignant neoplasm of scrotum (disorder)
        "372010005",  # Primary malignant neoplasm of soft tissues (disorder)
        "372011009",  # Malignant tumor of soft tissue of upper limb (disorder)
        "372012002",  # Primary malignant neoplasm of soft tissues of upper limb (disorder)
        "372013007",  # Primary malignant neoplasm of spermatic cord (disorder)
        "372014001",  # Primary malignant neoplasm of stomach (disorder)
        "372015000",  # Primary malignant neoplasm of the mesentery (disorder)
        "372016004",  # Primary malignant neoplasm of the peritoneum (disorder)
        "372017008",  # Primary malignant neoplasm of thoracic esophagus (disorder)
        "372018003",  # Malignant neoplasm of thoracic vertebral column (disorder)
        "372019006",  # Primary malignant neoplasm of thoracic vertebral column (disorder)
        "372020000",  # Primary malignant neoplasm of tonsil (disorder)
        "372021001",  # Primary malignant neoplasm of trapezium (disorder)
        "372022008",  # Primary malignant neoplasm of upper gum (disorder)
        "372023003",  # Primary malignant neoplasm of upper third of esophagus (disorder)
        "372024009",  # Primary malignant neoplasm of uterine cervix (disorder)
        "372025005",  # Primary malignant neoplasm of vagina (disorder)
        "372026006",  # Primary malignant neoplasm of vermilion border of lower lip (disorder)
        "372027002",  # Primary malignant neoplasm of vermilion border of upper lip (disorder)
        "372028007",  # Primary malignant neoplasm of vertebral column (disorder)
        "372030009",  # Primary malignant neoplasm of vocal cord (disorder)
        "372062007",  # Malignant neoplasm of central nervous system (disorder)
        "372063002",  # Malignant neoplasm of nervous system (disorder)
        "372065009",  # Malignant neoplasm of main bronchus (disorder)
        "372087000",  # Primary malignant neoplasm (disorder)
        "372092003",  # Primary malignant neoplasm of axillary tail of breast (disorder)
        "372093008",  # Metastatic malignant neoplasm to axillary tail of breast (disorder)
        "372094002",  # Malignant neoplasm of axillary tail of breast (disorder)
        "372097009",  # Malignant neoplasm of endocervix (disorder)
        "372098004",  # Carcinoma of endocervix (disorder)
        "372099007",  # Malignant neoplasm of exocervix (disorder)
        "372100004",  # Carcinoma of exocervix (disorder)
        "372101000",  # Carcinoma of extrahepatic bile duct (disorder)
        "372103002",  # Carcinoma of glottis (disorder)
        "372104008",  # Carcinoma of subglottis (disorder)
        "372105009",  # Carcinoma of supraglottis (disorder)
        "372106005",  # Carcinoma of penis (disorder)
        "372107001",  # Primary malignant neoplasm of ribs and/or sternum and/or clavicle (disorder)
        "372108006",  # Malignant neoplasm of bone of lower limb (disorder)
        "372110008",  # Primary malignant neoplasm of lower lobe, bronchus or lung (disorder)
        "372111007",  # Carcinoma of lower lobe of lung (disorder)
        "372115003",  # Primary malignant neoplasm of pelvic bones, sacrum and coccyx (disorder)
        "372116002",  # Carcinoma of pelvic bones, sacrum and coccyx (disorder)
        "372119009",  # Primary malignant neoplasm of head of pancreas (disorder)
        "372120003",  # Carcinoma of main bronchus (disorder)
        "372121004",  # Carcinoma of ribs and/or sternum and/or clavicle (disorder)
        "372122006",  # Malignant neoplasm of skin head and neck (disorder)
        "372123001",  # Primary malignant neoplasm of skin head and neck (disorder)
        "372124007",  # Malignant neoplasm of skin of lower limb (disorder)
        "372125008",  # Carcinoma of skin of lower limb (disorder)
        "372126009",  # Malignant neoplasm of skin of trunk (disorder)
        "372127000",  # Carcinoma of skin of trunk (disorder)
        "372128005",  # Malignant neoplasm of skin of upper limb (disorder)
        "372129002",  # Carcinoma of skin of upper limb (disorder)
        "372130007",  # Malignant neoplasm of skin (disorder)
        "372131006",  # Malignant neoplasm of upper limb bones and scapula (disorder)
        "372133009",  # Primary malignant neoplasm of upper limb bones and scapula (disorder)
        "372137005",  # Primary malignant neoplasm of breast (disorder)
        "372138000",  # Carcinoma of esophagus (disorder)
        "372139008",  # Primary malignant neoplasm of gallbladder (disorder)
        "372140005",  # Carcinoma of gallbladder (disorder)
        "372141009",  # Carcinoma of vocal cord (disorder)
        "372142002",  # Carcinoma of pancreas (disorder)
        "372143007",  # Carcinoma of stomach (disorder)
        "372244006",  # Malignant melanoma (disorder)
        "373080008",  # Malignant neoplasm of lower inner quadrant of breast (disorder)
        "373081007",  # Malignant neoplasm of lower outer quadrant of breast (disorder)
        "373082000",  # Malignant neoplasm of upper inner quadrant of breast (disorder)
        "373083005",  # Malignant neoplasm of upper outer quadrant of breast (disorder)
        "373088001",  # Primary malignant neoplasm of upper outer quadrant of breast (disorder)
        "373089009",  # Primary malignant neoplasm of upper inner quadrant of breast (disorder)
        "373090000",  # Primary malignant neoplasm of lower inner quadrant of breast (disorder)
        "373091001",  # Primary malignant neoplasm of lower outer quadrant of breast (disorder)
        "373168002",  # Reticulosarcoma (disorder)
        "373888000",  # Extrauterine adenocarcinoma (disorder)
        "385478001",  # Adenoma malignum (disorder)
        "389216001",  # Diaphyseal medullary stenosis with bone malignancy (disorder)
        "393563007",  # Glioblastoma multiforme (disorder)
        "397008008",  # Aggressive lymphadenopathic mastocytosis with eosinophilia (disorder)
        "397009000",  # Mast cell malignancy (disorder)
        "397011009",  # Mast cell malignancy of lymph nodes (disorder)
        "397012002",  # Cutaneous mastocytosis (disorder)
        "397013007",  # Solitary cutaneous mastocytoma (disorder)
        "397015000",  # Systemic mastocytosis with associated clonal hematological non-mast cell lineage disease (disorder)
        "398623004",  # Myelodysplastic syndrome with excess blasts (disorder)
        "398670003",  # Pigmented dermatofibrosarcoma protuberans of skin (disorder)
        "398903003",  # Pleomorphic malignant fibrous histiocytoma of skin (disorder)
        "399068003",  # Malignant tumor of prostate (disorder)
        "399326009",  # Malignant neoplasm of urinary bladder (disorder)
        "399490008",  # Adenocarcinoma of prostate (disorder)
        "399590005",  # Squamous cell carcinoma of prostate (disorder)
        "399660006",  # Ring melanoma of ciliary body (disorder)
        "399884001",  # Primary malignant neoplasm of blood vessel of upper arm (disorder)
        "399897004",  # Basal cell carcinoma of postauricular skin (disorder)
        "399934007",  # Basal cell carcinoma of preauricular skin (disorder)
        "399954008",  # Basal cell carcinoma of earlobe (disorder)
        "399967006",  # Neoplasm of uncertain behavior of stomach (disorder)
        "399969009",  # Metastatic malignant neoplasm to blood vessel of upper arm (disorder)
        "400001003",  # Primary cutaneous lymphoma (disorder)
        "400057007",  # Basal cell carcinoma of pinnal sulcus (disorder)
        "400058002",  # Metastatic malignant neoplasm to blood vessel of lower leg (disorder)
        "400074007",  # Primary malignant neoplasm of blood vessel of lower leg (disorder)
        "400122007",  # Primary cutaneous T-cell lymphoma (disorder)
        "400173004",  # Eccrine carcinoma of skin (disorder)
        "400174005",  # Basal cell carcinoma of antihelix of ear (disorder)
        "400175006",  # Myxofibrosarcoma of skin (disorder)
        "400375351000119102",  # Malignant melanoma of skin of left ankle (disorder)
        "402494003",  # Basal cell carcinoma of lower eyelid (disorder)
        "402495002",  # Basal cell carcinoma of medial canthus (disorder)
        "402496001",  # Basal cell carcinoma of lateral canthus (disorder)
        "402497005",  # Basal cell carcinoma of root of nose (disorder)
        "402498000",  # Basal cell carcinoma of dorsum of nose (disorder)
        "402499008",  # Basal cell carcinoma of lateral side wall of nose (disorder)
        "402500004",  # Basal cell carcinoma of ala nasi (disorder)
        "402501000",  # Basal cell carcinoma of supratip of nose (disorder)
        "402502007",  # Basal cell carcinoma of tip of nose (disorder)
        "402503002",  # Basal cell carcinoma of nasal columella (disorder)
        "402504008",  # Basal cell carcinoma of nasolabial groove (disorder)
        "402505009",  # Basal cell carcinoma of upper lip (disorder)
        "402506005",  # Basal cell carcinoma of lower lip (disorder)
        "402507001",  # Basal cell carcinoma of cheek (disorder)
        "402508006",  # Basal cell carcinoma of chin (disorder)
        "402509003",  # Basal cell carcinoma of neck (disorder)
        "402510008",  # Basal cell carcinoma of helix of ear (disorder)
        "402511007",  # Basal cell carcinoma of conchal bowl of ear (disorder)
        "402512000",  # Basal cell carcinoma of antitragus (disorder)
        "402513005",  # Basal cell carcinoma of tragus (disorder)
        "402514004",  # Basal cell carcinoma of obverse of pinna (disorder)
        "402515003",  # Basal cell carcinoma of anterior chest (disorder)
        "402516002",  # Basal cell carcinoma of abdomen (disorder)
        "402517006",  # Basal cell carcinoma of upper back (disorder)
        "402518001",  # Basal cell carcinoma of lower back (disorder)
        "402519009",  # Basal cell carcinoma of face (disorder)
        "402520003",  # Basal cell carcinoma of hand (disorder)
        "402521004",  # Basal cell carcinoma of upper extremity (disorder)
        "402522006",  # Basal cell carcinoma of lower extremity (disorder)
        "402523001",  # Basal cell carcinoma of truncal skin (disorder)
        "402524007",  # Basal cell carcinoma - adamantinoid (disorder)
        "402525008",  # Basal cell carcinoma - adenoid (disorder)
        "402526009",  # Basal cell carcinoma - follicular (disorder)
        "402527000",  # Basal cell carcinoma - infiltrative (disorder)
        "402528005",  # Basal cell carcinoma - keratotic (disorder)
        "402529002",  # Basal cell carcinoma - micronodular (disorder)
        "402530007",  # Basal cell carcinoma with matrical differentiation (disorder)
        "402531006",  # Basal cell carcinoma with granular cell change (disorder)
        "402532004",  # Basal cell carcinoma with monster cells (disorder)
        "402533009",  # Basal cell carcinoma with sebaceous differentiation (disorder)
        "402534003",  # Basal cell carcinoma with signet ring change (disorder)
        "402536001",  # Basal cell carcinoma - primary (disorder)
        "402537005",  # Metastatic basal cell carcinoma (disorder)
        "402538000",  # Recurrent basal cell carcinoma (disorder)
        "402539008",  # Basal cell carcinoma recurrent following excision (disorder)
        "402540005",  # Basal cell carcinoma recurrent following Mohs' excision (disorder)
        "402541009",  # Basal cell carcinoma recurrent following curettage (disorder)
        "402542002",  # Basal cell carcinoma recurrent following cryosurgery (disorder)
        "402543007",  # Basal cell carcinoma recurrent following radiotherapy (disorder)
        "402544001",  # Basal cell carcinoma - first recurrence (disorder)
        "402545000",  # Basal cell carcinoma - second recurrence (disorder)
        "402546004",  # Basal cell carcinoma - third recurrence (disorder)
        "402547008",  # Basal cell carcinoma - multiple recurrences (more than three times) (disorder)
        "402558004",  # Malignant melanoma (vertical growth phase) (disorder)
        "402559007",  # Congenital malignant melanoma (disorder)
        "402560002",  # Materno-fetal metastatic malignant melanoma (disorder)
        "402561003",  # Malignant melanoma of soft tissues (disorder)
        "402562005",  # Malignant melanoma animal-type (disorder)
        "402563000",  # Metastatic malignant melanoma with diffuse hypermelanosis (disorder)
        "402564006",  # Multiple primary malignant melanomata (disorder)
        "402636006",  # Malignant neoplasm of nail apparatus (disorder)
        "402652009",  # Malignant vascular tumor of skin (disorder)
        "402815007",  # Squamous cell carcinoma (disorder)
        "402816008",  # Squamous cell carcinoma of anogenital area (disorder)
        "402817004",  # Squamous cell carcinoma of nail apparatus (disorder)
        "402818009",  # Basal cell carcinoma of nose (disorder)
        "402819001",  # Basal cell carcinoma of skin of lip (disorder)
        "402820007",  # Basal cell carcinoma of ear (disorder)
        "402871009",  # Malignant neoplasm of subcutaneous fibrous tissue (disorder)
        "402873007",  # Malignant fibrohistiocytic tumor of skin (disorder)
        "402876004",  # Malignant tumor of nerve sheath origin (disorder)
        "402879006",  # T-cell leukemic infiltration of skin (disorder)
        "402880009",  # Primary cutaneous large T-cell lymphoma (disorder)
        "402881008",  # Primary cutaneous B-cell lymphoma (disorder)
        "402882001",  # Hodgkin's disease affecting skin (disorder)
        "402910001",  # Anogenital verrucous carcinoma of Buschke-Lowenstein (disorder)
        "402911002",  # Penile verrucous carcinoma of Buschke-Lowenstein (disorder)
        "402912009",  # Vulval verrucous carcinoma of Buschke-Lowenstein (disorder)
        "403458008",  # Localized skin involvement by breast carcinoma (disorder)
        "403468003",  # Squamous cell carcinoma of penis (disorder)
        "403711001",  # Primary malignant neoplasm of skin due to psoralen and longwave ultraviolet radiation therapy (disorder)
        "403712008",  # Psoralen and long-wave ultraviolet radiation therapy-associated basal cell carcinoma (disorder)
        "403713003",  # Psoralen and long-wave ultraviolet radiation therapy-associated squamous cell carcinoma (disorder)
        "403714009",  # Psoralen and long-wave ultraviolet radiation therapy-associated malignant melanoma (disorder)
        "403729006",  # Malignant neoplasm of skin caused by radiation (disorder)
        "403742006",  # Malignant neoplasm of skin caused by arsenic (disorder)
        "403889000",  # Verrucous carcinoma of oral cavity (disorder)
        "403890009",  # Squamous cell carcinoma of vulva due to lichen sclerosus (disorder)
        "403891008",  # Squamous cell carcinoma of scalp (disorder)
        "403892001",  # Squamous cell carcinoma of skin of face (disorder)
        "403893006",  # Squamous cell carcinoma of forehead (disorder)
        "403894000",  # Squamous cell carcinoma of skin of ear (disorder)
        "403895004",  # Squamous cell carcinoma of foot (disorder)
        "403896003",  # Squamous cell carcinoma of hand (disorder)
        "403897007",  # Squamous cell carcinoma of upper extremity (disorder)
        "403898002",  # Squamous cell carcinoma of skin of lower extremity (disorder)
        "403899005",  # Squamous cell carcinoma of skin of trunk (disorder)
        "403900000",  # Spindle cell squamous cell carcinoma (disorder)
        "403901001",  # Acantholytic squamous cell carcinoma (disorder)
        "403902008",  # Adenosquamous carcinoma (disorder)
        "403903003",  # Signet ring squamous cell carcinoma (disorder)
        "403904009",  # Verrucous squamous cell carcinoma (disorder)
        "403905005",  # Multiple squamous cell carcinomata (disorder)
        "403906006",  # Metastatic squamous cell carcinoma (disorder)
        "403907002",  # Giant keratoacanthoma (disorder)
        "403908007",  # Subungual keratoacanthoma (disorder)
        "403909004",  # Pigmented basal cell carcinoma (disorder)
        "403910009",  # Circumscribed solid basal cell carcinoma (disorder)
        "403911008",  # Nodulo-ulcerative basal cell carcinoma (disorder)
        "403912001",  # Cystic basal cell carcinoma (disorder)
        "403913006",  # Sclerosing morpheic basal cell carcinoma (disorder)
        "403914000",  # Superficial basal cell carcinoma (disorder)
        "403915004",  # Basal cell carcinoma of scalp (disorder)
        "403916003",  # Basal cell carcinoma of forehead (disorder)
        "403917007",  # Basal cell carcinoma of temple (disorder)
        "403918002",  # Basal cell carcinoma of glabella (disorder)
        "403919005",  # Basal cell carcinoma of upper eyelid (disorder)
        "403921000",  # Borderline malignant melanoma (disorder)
        "403922007",  # Balloon cell malignant melanoma (disorder)
        "403923002",  # Spindle cell malignant melanoma (disorder)
        "403924008",  # Desmoplastic malignant melanoma (disorder)
        "403925009",  # Neurotropic malignant melanoma (disorder)
        "403926005",  # Malignant melanoma of oral cavity (disorder)
        "403927001",  # Malignant melanoma of nail apparatus (disorder)
        "403929003",  # Trichilemmal carcinoma (disorder)
        "403939009",  # Eccrine ductal carcinoma of skin (disorder)
        "403940006",  # Clear cell eccrine hidradenocarcinoma of skin (disorder)
        "403941005",  # Malignant cylindroma of skin (disorder)
        "403942003",  # Malignant eccrine spiradenoma of skin (disorder)
        "403943008",  # Malignant chondroid syringoma of skin (disorder)
        "403944002",  # Small cell eccrine carcinoma of skin (disorder)
        "403946000",  # Paget's disease of nipple (disorder)
        "403947009",  # Perianal Paget's disease (disorder)
        "403949007",  # Apocrine adenocarcinoma of skin (disorder)
        "403950007",  # Moll's gland adenocarcinoma (disorder)
        "403951006",  # Ceruminous gland adenocarcinoma of skin (disorder)
        "403954003",  # Mixed eccrine/pilar adnexal carcinoma of skin (disorder)
        "403955002",  # Undifferentiated adnexal carcinoma of skin (disorder)
        "403977003",  # Angiosarcoma (disorder)
        "403978008",  # Kaposi's sarcoma - sporadic (disorder)
        "403979000",  # Endemic Kaposi sarcoma (disorder)
        "403980002",  # Hemangioendothelioma (disorder)
        "403981003",  # Epithelioid hemangioendothelioma (disorder)
        "403982005",  # Retiform hemangioendothelioma (disorder)
        "403983000",  # Kaposiform hemangioendothelioma (disorder)
        "403984006",  # Composite hemangioendothelioma (disorder)
        "403985007",  # Intravascular angiosarcoma (disorder)
        "403986008",  # Lymphangiosarcoma (disorder)
        "403996004",  # Infantile fibrosarcoma (disorder)
        "404008001",  # Dermatofibrosarcoma protuberans with myoid differentiation (disorder)
        "404009009",  # Dermatofibrosarcoma protuberans with granular cell change (disorder)
        "404010004",  # Dermatofibrosarcoma protuberans with giant cell fibroblastoma (disorder)
        "404011000",  # Plexiform fibrohistiocytic tumor of skin (disorder)
        "404012007",  # Atypical fibroxanthoma - spindle cell type (disorder)
        "404013002",  # Angiomatoid fibrous histiocytoma of skin (disorder)
        "404014008",  # Malignant fibrous histiocytoma of skin (disorder)
        "404015009",  # Superficial malignant fibrous histiocytoma of skin (disorder)
        "404016005",  # Giant cell malignant fibrous histiocytoma of skin (disorder)
        "404017001",  # Inflammatory malignant fibrous histiocytoma of skin (disorder)
        "404037002",  # Malignant peripheral nerve sheath tumor (disorder)
        "404038007",  # Epithelioid malignant nerve sheath tumor (disorder)
        "404039004",  # Melanotic malignant nerve sheath tumor (disorder)
        "404040002",  # Malignant Triton tumor (disorder)
        "404041003",  # Malignant granular cell tumor (disorder)
        "404045007",  # Epithelioid leiomyosarcoma of skin (disorder)
        "404046008",  # Myxoid leiomyosarcoma of skin (disorder)
        "404047004",  # Cutaneous leiomyosarcoma with granular cell change (disorder)
        "404051002",  # Embryonal rhabdomyosarcoma (disorder)
        "404052009",  # Botryoid rhabdomyosarcoma (disorder)
        "404053004",  # Alveolar rhabdomyosarcoma (disorder)
        "404054005",  # Pleomorphic rhabdomyosarcoma (disorder)
        "404055006",  # Spindle cell rhabdomyosarcoma (disorder)
        "404056007",  # Alveolar soft part sarcoma (disorder)
        "404067008",  # Adipocytic liposarcoma (disorder)
        "404068003",  # Sclerosing liposarcoma (disorder)
        "404069006",  # Myxoid liposarcoma (disorder)
        "404071006",  # Pleomorphic liposarcoma (disorder)
        "404072004",  # Dedifferentiated liposarcoma (disorder)
        "404073009",  # Spindle cell liposarcoma (disorder)
        "404077005",  # Extraskeletal osteosarcoma (disorder)
        "404079008",  # Extraskeletal myxoid chondrosarcoma (disorder)
        "404080006",  # Extraskeletal mesenchymal chondrosarcoma (disorder)
        "404081005",  # Yolk sac tumor (disorder)
        "404083008",  # Angiomyxoma (disorder)
        "404084002",  # Superficial angiomyxoma (disorder)
        "404085001",  # Aggressive angiomyxoma (disorder)
        "404087009",  # Carcinosarcoma of skin (disorder)
        "404088004",  # Low-grade fibromyxoid sarcoma (disorder)
        "404089007",  # Extrarenal rhabdoid tumor (disorder)
        "404090003",  # Malignant infiltration of oral cavity by underlying tumor (disorder)
        "404091004",  # Malignant infiltration of skin by underlying tumor (disorder)
        "404092006",  # Metastatic carcinoma to skin (disorder)
        "404093001",  # Sarcomatous metastasis in skin (disorder)
        "404094007",  # Metastasis involving oral cavity (disorder)
        "404103007",  # Lymphomatoid papulosis type A (CD-30 positive type) (disorder)
        "404104001",  # Lymphomatoid papulosis type B - mycosis fungoides-like (disorder)
        "404105000",  # Lymphomatoid papulosis type C (anaplastic large-cell lymphoma-like) (disorder)
        "404106004",  # Lymphomatoid papulosis with Hodgkin's disease (disorder)
        "404107008",  # Patch/plaque stage mycosis fungoides (disorder)
        "404108003",  # Poikilodermatous mycosis fungoides (disorder)
        "404109006",  # Follicular mucinosis type mycosis fungoides (disorder)
        "404110001",  # Hypomelanotic mycosis fungoides (disorder)
        "404111002",  # Lymphomatoid papulosis-associated mycosis fungoides (disorder)
        "404112009",  # Granulomatous mycosis fungoides (disorder)
        "404113004",  # Tumor stage mycosis fungoides (disorder)
        "404114005",  # Erythrodermic mycosis fungoides (disorder)
        "404115006",  # Bullous mycosis fungoides (disorder)
        "404116007",  # Mycosis fungoides with systemic infiltration (disorder)
        "404117003",  # Spongiotic mycosis fungoides (disorder)
        "404118008",  # Syringotropic mycosis fungoides (disorder)
        "404119000",  # Pagetoid reticulosis (disorder)
        "404120006",  # Localized pagetoid reticulosis (disorder)
        "404121005",  # Generalized pagetoid reticulosis (disorder)
        "404122003",  # Leukemic infiltration of skin (chronic T-cell lymphocytic leukemia) (disorder)
        "404123008",  # Leukemic infiltration of skin (T-cell prolymphocytic leukemia) (disorder)
        "404124002",  # Leukemic infiltration of skin (T-cell lymphoblastic leukemia) (disorder)
        "404126000",  # CD-30 positive pleomorphic large T-cell cutaneous lymphoma (disorder)
        "404127009",  # CD-30 positive T-immunoblastic cutaneous lymphoma (disorder)
        "404128004",  # CD-30 negative cutaneous T-cell lymphoma (disorder)
        "404129007",  # CD-30 negative anaplastic large T-cell cutaneous lymphoma (disorder)
        "404130002",  # CD-30 negative pleomorphic large T-cell cutaneous lymphoma (disorder)
        "404131003",  # CD-30 negative T-immunoblastic cutaneous lymphoma (disorder)
        "404133000",  # Subcutaneous panniculitic cutaneous T-cell lymphoma (disorder)
        "404134006",  # Anaplastic large T-cell systemic malignant lymphoma (disorder)
        "404135007",  # Angiocentric natural killer/T-cell malignant lymphoma involving skin (disorder)
        "404136008",  # Aggressive natural killer-cell leukemia involving skin (disorder)
        "404137004",  # Precursor B-cell lymphoblastic lymphoma involving skin (disorder)
        "404138009",  # Small lymphocytic B-cell lymphoma involving skin (disorder)
        "404139001",  # Leukemic infiltration of skin in hairy-cell leukemia (disorder)
        "404140004",  # Primary cutaneous marginal zone B-cell lymphoma (disorder)
        "404141000",  # Primary cutaneous immunocytoma (disorder)
        "404142007",  # Primary cutaneous plasmacytoma (disorder)
        "404143002",  # Primary cutaneous follicular center B-cell lymphoma (disorder)
        "404144008",  # Primary cutaneous diffuse large cell B-cell lymphoma (disorder)
        "404147001",  # Follicular center B-cell lymphoma (nodal/systemic with skin involvement) (disorder)
        "404148006",  # Diffuse large B-cell lymphoma (nodal/systemic with skin involvement) (disorder)
        "404149003",  # Lymphoplasmacytic B-cell lymphoma, nodal/systemic with skin involvement (disorder)
        "404150003",  # Mantle cell B-cell lymphoma (nodal/systemic with skin involvement) (disorder)
        "404151004",  # Leukemic infiltration of skin in myeloid leukemia (disorder)
        "404152006",  # Leukemic infiltration of skin in acute myeloid leukemia (disorder)
        "404153001",  # Leukemic infiltration of skin in chronic myeloid leukemia (disorder)
        "404154007",  # Leukemic infiltration of skin in monocytic leukemia (disorder)
        "404155008",  # Granulocytic sarcoma affecting skin (disorder)
        "404156009",  # Leukemic infiltration of skin (disorder)
        "404157000",  # Specific skin infiltration in Hodgkin's disease (disorder)
        "404169008",  # Malignant histiocytosis involving skin (disorder)
        "404171008",  # Mastocytoma (disorder)
        "404172001",  # Mast cell leukemia affecting skin (disorder)
        "404653000",  # Chiasmal glioma (disorder)
        "404664002",  # Malignant optic glioma (disorder)
        "404810003",  # 5-hydroxytryptamine secreting neuroendocrine tumor (disorder)
        "405546008",  # Pericardial effusion co-occurrent and due to malignant neoplasm of pericardium (disorder)
        "405822008",  # Squamous cell carcinoma of larynx (disorder)
        "405843009",  # Widespread metastatic malignant neoplastic disease (disorder)
        "405945003",  # Malignant neoplasm of metatarsal bone of foot (disorder)
        "408642003",  # Transitional cell carcinoma of kidney (disorder)
        "408643008",  # Infiltrating duct carcinoma of breast (disorder)
        "408644002",  # Adenocarcinoma of duodenum (disorder)
        "408645001",  # Adenocarcinoma of large intestine (disorder)
        "408646000",  # Adenocarcinoma of liver (disorder)
        "408647009",  # Adenocarcinoma of stomach (disorder)
        "408648004",  # Squamous cell carcinoma of epiglottis (disorder)
        "408649007",  # Squamous cell carcinoma of pharynx (disorder)
        "413389003",  # Accelerated phase chronic myeloid leukemia (disorder)
        "413441006",  # Acute monocytic leukemia (disorder)
        "413445002",  # Adenocarcinoma of appendix (disorder)
        "413446001",  # Adenocarcinoma of cecum (disorder)
        "4135001",  # 11p partial monosomy syndrome (disorder)
        "413537009",  # Angioimmunoblastic T-cell lymphoma (disorder)
        "413587002",  # Smoldering myeloma (disorder)
        "413656006",  # Blastic phase chronic myeloid leukemia (disorder)
        "413842007",  # Chronic myeloid leukemia in lymphoid blast crisis (disorder)
        "413843002",  # Chronic myeloid leukemia in myeloid blast crisis (disorder)
        "413847001",  # Chronic phase chronic myeloid leukemia (disorder)
        "414166008",  # Extranodal natural killer/T-cell lymphoma, nasal type (disorder)
        "414553000",  # Kappa light chain myeloma (disorder)
        "414653009",  # Mast cell neoplasm (disorder)
        "414676007",  # Metastatic neuroblastoma of orbit proper (disorder)
        "414780005",  # Mucosa-associated lymphoid tissue lymphoma of orbit (disorder)
        "414785000",  # Multiple solitary plasmacytomas (disorder)
        "414950005",  # Optic nerve glioma of orbit (disorder)
        "415110002",  # Plasma cell myeloma/plasmacytoma (disorder)
        "415112005",  # Plasmacytoma (disorder)
        "415177008",  # Primary malignant neoplasm of ear, nose AND/OR throat (disorder)
        "415283002",  # Myelodysplastic syndrome with excess blasts-1 (disorder)
        "415284008",  # Myelodysplastic syndrome with excess blasts-2 (disorder)
        "415287001",  # Relapsing chronic myeloid leukemia (disorder)
        "416351002",  # Intraocular optic nerve glioma (disorder)
        "416669000",  # Invasive hydatidiform mole (disorder)
        "416769008",  # Malignant teratoma of testis (disorder)
        "416842003",  # Malignant sacral teratoma (disorder)
        "416901002",  # Malignant medulloepithelioma of ciliary body (disorder)
        "417181009",  # Hormone receptor positive malignant neoplasm of breast (disorder)
        "417417007",  # Malignant teratoma of undescended testis (disorder)
        "417554000",  # Malignant teratoma of descended testis (disorder)
        "417570003",  # Gestational choriocarcinoma (disorder)
        "417619001",  # Intracranial optic nerve glioma (disorder)
        "418040002",  # Littoral cell angioma (disorder)
        "418372008",  # Squamous cell carcinoma of mucous membrane of lower lip (disorder)
        "418529003",  # Metastatic malignant neoplasm to lacrimal drainage structure (disorder)
        "419052002",  # Malignant tumor of urinary system (disorder)
        "419240004",  # Squamous cell carcinoma of mucous membrane of upper lip (disorder)
        "419317000",  # Neoplasm of uncertain behavior of lacrimal drainage structure (disorder)
        "419842002",  # Squamous cell carcinoma of oral mucous membrane (disorder)
        "420302007",  # Reticulosarcoma with acquired immunodeficiency syndrome (disorder)
        "420519005",  # Malignant lymphoma of the eye region (disorder)
        "420524008",  # Kaposi's sarcoma with acquired immunodeficiency syndrome (disorder)
        "420788006",  # Intraocular non-Hodgkin malignant lymphoma (disorder)
        "420890002",  # T-lymphoblastic leukemia/lymphoblastic lymphoma (disorder)
        "421246008",  # T-lymphoblastic lymphoma (disorder)
        "421249001",  # Malignant tumor of vermilion border of lip (disorder)
        "421283008",  # Primary lymphoma of brain with acquired immunodeficiency syndrome (disorder)
        "422282000",  # Malignant neoplasm with acquired immunodeficiency syndrome (disorder)
        "422378004",  # Squamous cell carcinoma of bridge of nose (disorder)
        "422399001",  # Infiltrating ductal carcinoma of breast, stage 1 (disorder)
        "422479008",  # Infiltrating ductal carcinoma of breast, stage 2 (disorder)
        "422541001",  # Undifferentiated carcinoma of nasopharynx (disorder)
        "422572002",  # Squamous cell carcinoma of chin (disorder)
        "422599000",  # Squamous cell carcinoma of back (disorder)
        "422676009",  # Squamous cell carcinoma of ala nasi (disorder)
        "422679002",  # Seminoma of testis, stage 3 (disorder)
        "422691006",  # Squamous cell carcinoma of nasopharynx (disorder)
        "422736007",  # Primary malignant neoplasm of peripheral nerve (disorder)
        "422758009",  # Carcinoma of nasal meatus (disorder)
        "422833009",  # Adenoid cystic carcinoma of salivary gland (disorder)
        "422853008",  # Lymphoma of retroperitoneal space (disorder)
        "422886007",  # Olfactory neuroblastoma (disorder)
        "422968005",  # Non-small cell carcinoma of lung, TNM stage 3 (disorder)
        "423005002",  # Primary malignant neoplasm of lacrimal gland duct (disorder)
        "423032007",  # Leukemic infiltration of orbit (disorder)
        "423038006",  # Polymorphous low grade adenocarcinoma of salivary gland (disorder)
        "423050000",  # Large cell carcinoma of lung, TNM stage 2 (disorder)
        "423106003",  # Adenocarcinoma of nasopharynx (disorder)
        "423114009",  # Infiltrating ductal carcinoma of breast, stage 3 (disorder)
        "423121009",  # Non-small cell carcinoma of lung, TNM stage 4 (disorder)
        "423158009",  # Hurthle cell carcinoma of thyroid (disorder)
        "423189008",  # Adenoid cystic carcinoma of submandibular gland (disorder)
        "423195009",  # Primary malignant neoplasm of lacrimal drainage system (disorder)
        "423274005",  # Carcinoma of ovary, stage 4 (disorder)
        "423280002",  # Malignant melanoma of skin of canthus of eye (disorder)
        "423284006",  # Squamous cell carcinoma of skin of neck (disorder)
        "423294001",  # Idiopathic hypereosinophilic syndrome (disorder)
        "423295000",  # Squamous cell carcinoma of lung, TNM stage 1 (disorder)
        "423318000",  # Adenoid cystic carcinoma of oropharynx (disorder)
        "423325007",  # Basal cell carcinoma of external auditory canal (disorder)
        "423349005",  # Carcinoma of tip of nose (disorder)
        "423384009",  # Metastatic malignant neoplasm to lacrimal gland duct (disorder)
        "423424005",  # Mucoepidermoid carcinoma of submandibular gland (disorder)
        "423425006",  # Malignant neoplasm of skin of eyelid (disorder)
        "423447006",  # Malignant melanoma of skin of lower eyelid (disorder)
        "423463003",  # Basal cell carcinoma of back (disorder)
        "423464009",  # Squamous cell carcinoma of oropharynx (disorder)
        "423468007",  # Squamous cell carcinoma of lung, TNM stage 2 (disorder)
        "423480004",  # Carcinoma of ovary, stage 3 (disorder)
        "423494003",  # Malignant melanoma of skin of upper eyelid (disorder)
        "423506005",  # Squamous cell carcinoma of external auditory canal (disorder)
        "423535002",  # Basal cell carcinoma of chest wall (disorder)
        "423595004",  # Adenocarcinoma carcinomatosis (disorder)
        "423600008",  # Large cell carcinoma of lung, TNM stage 4 (disorder)
        "423607006",  # Adenocarcinoma of anus (disorder)
        "423615009",  # Adenoid cystic carcinoma of parotid gland (disorder)
        "423627007",  # Sarcoma of ovary (disorder)
        "423645007",  # Seminoma of testis, stage 2 (disorder)
        "423673009",  # Malignant melanoma of retina (disorder)
        "423691004",  # Malignant neoplasm of gum and contiguous sites (disorder)
        "423697000",  # Seminoma of testis, stage 4 (disorder)
        "423700001",  # Squamous cell carcinoma of auricle of ear (disorder)
        "423708008",  # Mucoepidermoid carcinoma of salivary gland (disorder)
        "423746001",  # Adenocarcinoma of pelvis (disorder)
        "423793008",  # Mucoepidermoid carcinoma of parotid gland (disorder)
        "423807009",  # Primary leiomyosarcoma (disorder)
        "423812005",  # Sarcoma of head and/or neck (disorder)
        "423816008",  # Seminoma of testis, stage 1 (disorder)
        "423829008",  # Invasive vulval Paget's disease (disorder)
        "423896007",  # Squamous cell carcinoma of tip of nose (disorder)
        "423973006",  # Carcinoma of uterine cervix, invasive (disorder)
        "424052001",  # Small cell carcinoma carcinomatosis (disorder)
        "424078005",  # Adenocarcinoma of lung, stage I (disorder)
        "424132000",  # Non-small cell carcinoma of lung, TNM stage 1 (disorder)
        "424151006",  # Anaplastic glioma of brain (disorder)
        "424190005",  # Malignant melanoma of unknown origin (disorder)
        "424229006",  # Infiltrating ductal carcinoma of breast, stage 4 (disorder)
        "424260006",  # Squamous cell carcinoma of nasolabial fold (disorder)
        "424276002",  # Malignant glioma of brainstem (disorder)
        "424302003",  # Malignant melanoma of skin of lower lip (disorder)
        "424334007",  # Malignant tumor of spinal cord, intramedullary (disorder)
        "424413001",  # Sarcoma (disorder)
        "424440001",  # Adenocarcinoma of small intestine (disorder)
        "424454009",  # Adenocarcinoma of lung, stage III (disorder)
        "424486004",  # Carcinoma of ovary, stage 2 (disorder)
        "424487008",  # Malignant melanoma of skin of upper lip (disorder)
        "424549003",  # Malignant tumor of spinal cord, extramedullary (disorder)
        "424600001",  # Carcinoma of ovary, stage 1 (disorder)
        "424632001",  # Adenocarcinoma of lung, stage II (disorder)
        "424666005",  # Basal cell carcinoma of auricle of ear (disorder)
        "424779008",  # Primary Kaposi's sarcoma of oral cavity (disorder)
        "424849005",  # Primary sarcoma of tongue (disorder)
        "424938000",  # Large cell carcinoma of lung, TNM stage 1 (disorder)
        "424952003",  # Sarcoma of soft tissue (disorder)
        "424954002",  # Undifferentiated large cell carcinomatosis (disorder)
        "424970000",  # Large cell carcinoma of lung, TNM stage 3 (disorder)
        "424993006",  # Adenocarcinoma of lung, stage IV (disorder)
        "425048006",  # Non-small cell carcinoma of lung, TNM stage 2 (disorder)
        "425066001",  # Carcinoma of urinary bladder, invasive (disorder)
        "425127006",  # Carcinoma ex pleomorphic adenoma of parotid gland (disorder)
        "425148008",  # Squamous cell carcinoma of temple (disorder)
        "425178004",  # Adenocarcinoma of rectosigmoid junction (disorder)
        "425225007",  # Malignant mixed tumor of salivary gland (disorder)
        "425230006",  # Squamous cell carcinoma of lung, TNM stage 3 (disorder)
        "425231005",  # Carcinoma of urinary bladder, superficial (disorder)
        "425303004",  # Squamous cell carcinomatosis (disorder)
        "425376008",  # Squamous cell carcinoma of lung, TNM stage 4 (disorder)
        "425607009",  # Hemangioendothelioma of retroperitoneum (disorder)
        "425657001",  # Osteosclerotic myeloma (disorder)
        "425688002",  # Philadelphia chromosome-positive acute lymphoblastic leukemia (disorder)
        "425869007",  # Acute promyelocytic leukemia, FAB M3, in remission (disorder)
        "425941003",  # Precursor B-cell acute lymphoblastic leukemia in remission (disorder)
        "426071002",  # Hodgkin's disease in remission (disorder)
        "426124006",  # Acute myeloid leukemia with maturation, FAB M2, in remission (disorder)
        "426217000",  # Aleukemic leukemia in remission (disorder)
        "426248008",  # Aleukemic lymphoid leukemia in remission (disorder)
        "426336007",  # Solitary osseous myeloma (disorder)
        "426370008",  # Subacute lymphoid leukemia in remission (disorder)
        "426642002",  # Erythroleukemia, FAB M6 in remission (disorder)
        "426885008",  # Hodgkin's disease, lymphocytic depletion of lymph nodes of head (disorder)
        "426964009",  # Non-small cell lung cancer, positive for epidermal growth factor receptor expression (disorder)
        "427038005",  # Non-small cell lung cancer, negative for epidermal growth factor receptor expression (disorder)
        "427056005",  # Subacute leukemia in remission (disorder)
        "427141003",  # Malignant lymphoma in remission (disorder)
        "427285006",  # Atypical lipoma of soft tissue (disorder)
        "427492003",  # Hormone refractory prostate cancer (disorder)
        "427642009",  # T-lymphoblastic leukemia in remission (disorder)
        "427658007",  # Acute myelomonocytic leukemia, FAB M4, in remission (disorder)
        "427685000",  # Human epidermal growth factor 2 positive carcinoma of breast (disorder)
        "427744002",  # Hemangioendothelioma of liver (disorder)
        "427942006",  # Phyllodes tumor of breast, borderline (disorder)
        "428061005",  # Malignant neoplasm of brain (disorder)
        "428100006",  # Malignant neoplasm of thoracic cavity structure (disorder)
        "428281000",  # Malignant neoplasm of bone (disorder)
        "428322007",  # Malignant neoplasm of uterine adnexa (disorder)
        "428905002",  # Malignant neoplasm of gastrointestinal tract (disorder)
        "429033009",  # Malignant neoplasm of cerebrum (disorder)
        "429114002",  # Malignant basal cell neoplasm of skin (disorder)
        "429176009",  # Hemangioendothelioma of abdomen (disorder)
        "429570006",  # Atypical fibrous histiocytoma of soft tissue (disorder)
        "430338009",  # Smoldering chronic lymphocytic leukemia (disorder)
        "430556008",  # Malignant neoplasm of genital structure (disorder)
        "430621000",  # Malignant neoplasm of lower respiratory tract (disorder)
        "431396003",  # Human epidermal growth factor 2 negative carcinoma of breast (disorder)
        "431911000124102",  # Gnathic osteosarcoma (disorder)
        "432131000124100",  # Malignant mesothelioma of tunica vaginalis (disorder)
        "432328008",  # Neuroblastoma (disorder)
        "433067002",  # Adamantinoma of femur (disorder)
        "438946002",  # Siewert type I adenocarcinoma of esophagogastric junction (disorder)
        "439478008",  # Siewert type III adenocarcinoma of esophagogastric junction (disorder)
        "440173001",  # Nonsquamous nonsmall cell neoplasm of lung (disorder)
        "440397000",  # Primary osteosarcoma of pelvis (disorder)
        "440422002",  # Asymptomatic multiple myeloma (disorder)
        "440501006",  # Siewert type II adenocarcinoma of esophagogastric junction (disorder)
        "440525003",  # Primary small cell neoplasm of thymus (disorder)
        "440527006",  # Primary squamous cell carcinoma of naris (disorder)
        "441313008",  # Indolent multiple myeloma (disorder)
        "441535001",  # Adenocarcinoma of head and/or neck (disorder)
        "441559006",  # Mantle cell lymphoma of spleen (disorder)
        "441962003",  # Large cell lymphoma of intrapelvic lymph nodes (disorder)
        "442537007",  # Non-Hodgkin lymphoma associated with Human immunodeficiency virus infection (disorder)
        "443042003",  # Neoplasm of uncertain behavior of skeletal system (disorder)
        "443136000",  # Malignant neoplasm of skin of face (disorder)
        "443144000",  # Metastatic sarcoma (disorder)
        "443250000",  # Malignant fibromatous neoplasm (disorder)
        "443261008",  # Oxyphilic adenocarcinoma (disorder)
        "443333004",  # Medulloblastoma (disorder)
        "443439001",  # Malignant fibrous histiocytoma (disorder)
        "443487006",  # Mantle cell lymphoma (disorder)
        "443488001",  # Malignant neoplasm of anorectum (disorder)
        "443493003",  # Metastatic malignant melanoma (disorder)
        "443520009",  # Chondrosarcoma (disorder)
        "443648003",  # Malignant neoplasm of ear (disorder)
        "443675005",  # Seminoma (disorder)
        "443679004",  # Malignant neoplasm of skeletal system (disorder)
        "443719001",  # Leiomyosarcoma (disorder)
        "443936004",  # Oligodendroglioma (disorder)
        "443937008",  # Mixed glioma (disorder)
        "443961001",  # Malignant adenomatous neoplasm (disorder)
        "444374006",  # Type C thymoma (disorder)
        "444596001",  # Malignant thymoma (disorder)
        "444597005",  # Extranodal marginal zone lymphoma of mucosa-associated lymphoid tissue of stomach (disorder)
        "444604002",  # Carcinoma of breast with ductal and lobular features (disorder)
        "444712000",  # Mucinous carcinoma of breast (disorder)
        "444910004",  # Primary mediastinal (thymic) large B-cell lymphoma (disorder)
        "444911000",  # Acute myeloid leukemia with t(9:11)(p22;q23); MLLT3-MLL (disorder)
        "445227008",  # Juvenile myelomonocytic leukemia (disorder)
        "445269007",  # Extranodal marginal zone B-cell lymphoma of mucosa-associated lymphoid tissue (disorder)
        "445406001",  # Hepatosplenic T-cell lymphoma (disorder)
        "445448008",  # Acute myeloid leukemia with myelodysplasia-related changes (disorder)
        "445738007",  # Myelodysplastic/myeloproliferative disease (disorder)
        "446022000",  # Malignant epithelial neoplasm of uterus (disorder)
        "446189008",  # Primary malignant neoplasm of extrahepatic bile duct (disorder)
        "446807009",  # Primary malignant neoplasm of perihilar bile duct (disorder)
        "446939001",  # Chordoma of clivus (disorder)
        "447100004",  # Marginal zone lymphoma (disorder)
        "447109003",  # Primary malignant neoplasm of intrahepatic bile duct (disorder)
        "44721000087109",  # Malignant neoplasm of breast in full remission (disorder)
        "447266004",  # Sarcoma of endometrium (disorder)
        "447389009",  # Leiomyosarcoma of uterus (disorder)
        "447390000",  # Adenosarcoma of uterus (disorder)
        "447416008",  # Primary malignant neoplasm of distal bile duct (disorder)
        "447596005",  # Myelodysplastic/myeloproliferative neoplasm, unclassifiable (disorder)
        "447656001",  # Lymphoma of pylorus of stomach (disorder)
        "447658000",  # Lymphoma of fundus of stomach (disorder)
        "447705002",  # Malignant germ cell neoplasm of posterior mediastinum (disorder)
        "447706001",  # Leiomyosarcoma of scalp (disorder)
        "447707005",  # Leiomyosarcoma of cardia of stomach (disorder)
        "447708000",  # Malignant germ cell neoplasm of anterior mediastinum (disorder)
        "447712006",  # Malignant melanoma of skin of anus (disorder)
        "447730004",  # Chordoma of sacrum (disorder)
        "447738006",  # Paget's disease of skin of scrotum (disorder)
        "447757002",  # Angiosarcoma of cheek (disorder)
        "447766003",  # Lymphoma of pyloric antrum of stomach (disorder)
        "447781009",  # Malignant epithelial neoplasm of peritoneum (disorder)
        "447783007",  # Sarcoma of peritoneum (disorder)
        "447785000",  # Leiomyosarcoma of stomach (disorder)
        "447792005",  # Chondrosarcoma of bone (disorder)
        "447800002",  # Adenocarcinoma of scrotum (disorder)
        "447804006",  # Leiomyosarcoma of connective tissue (disorder)
        "447805007",  # Lymphoma of greater curvature of stomach (disorder)
        "447806008",  # Lymphoma of cardia of stomach (disorder)
        "447882007",  # Malignant epithelial neoplasm of vulva (disorder)
        "447883002",  # Malignant neoplasm of carotid body (disorder)
        "447885009",  # Sarcoma of pelvis (disorder)
        "447886005",  # Adenocarcinoma of anorectum (disorder)
        "447949005",  # Malignant epithelial neoplasm of cheek (disorder)
        "447951009",  # Ewing's sarcoma of soft tissue (disorder)
        "447989004",  # Non-Hodgkin's lymphoma of extranodal site (disorder)
        "448139005",  # Fibrosarcoma of connective tissue (disorder)
        "448165009",  # Squamous cell carcinoma arising in chronic ulcer (disorder)
        "448212009",  # Anaplastic lymphoma kinase negative anaplastic large cell lymphoma (disorder)
        "448213004",  # Diffuse non-Hodgkin's lymphoma of prostate (disorder)
        "448214005",  # Malignant epithelial neoplasm of oropharynx (disorder)
        "448215006",  # Malignant epithelial neoplasm of renal pelvis (disorder)
        "448216007",  # Malignant epithelial neoplasm of thyroid (disorder)
        "448217003",  # Follicular non-Hodgkin's lymphoma of prostate (disorder)
        "448218008",  # Malignant neoplasm of cerebellopontine angle (disorder)
        "448220006",  # Non-Hodgkin's lymphoma of bone (disorder)
        "448221005",  # Sarcoma of fibula (disorder)
        "448229007",  # Leiomyosarcoma of lower esophagus (disorder)
        "448231003",  # Follicular non-Hodgkin's lymphoma of nose (disorder)
        "448233000",  # Malignant neoplasm of urinary organ (disorder)
        "448248006",  # Malignant neoplasm of axial suprasellar region of brain (disorder)
        "448250003",  # Malignant teratoma of pineal region (disorder)
        "448254007",  # Non-Hodgkin's lymphoma of central nervous system (disorder)
        "448258005",  # Sarcoma of mesentery (disorder)
        "448259002",  # Sarcoma of posterior mediastinum (disorder)
        "448269008",  # Lymphoma of lesser curvature of stomach (disorder)
        "448273006",  # Malignant melanoma of skin of scrotum (disorder)
        "448274000",  # Malignant neoplasm of connective tissue (disorder)
        "448298007",  # Malignant melanoma of skin of penis (disorder)
        "448299004",  # Malignant neoplasm of mastoid (disorder)
        "448300007",  # Malignant melanoma of skin of vulva (disorder)
        "448314007",  # Malignant epithelial neoplasm of spinal cord (disorder)
        "448315008",  # Malignant epithelial neoplasm of anus (disorder)
        "448317000",  # Follicular non-Hodgkin's lymphoma of soft tissue (disorder)
        "448319002",  # Diffuse non-Hodgkin's lymphoma of nasopharynx (disorder)
        "448354009",  # Non-Hodgkin's lymphoma of intestine (disorder)
        "448371005",  # Non-Hodgkin's lymphoma of nasopharynx (disorder)
        "448372003",  # Non-Hodgkin's lymphoma of lung (disorder)
        "448376000",  # Non-Hodgkin's lymphoma of ovary (disorder)
        "448377009",  # Sarcoma of sacrum (disorder)
        "448378004",  # Sarcoma of bone of foot (disorder)
        "448384001",  # Non-Hodgkin's lymphoma of nose (disorder)
        "448386004",  # Non-Hodgkin's lymphoma of oral cavity (disorder)
        "448387008",  # Non-Hodgkin's lymphoma of testis (disorder)
        "448401007",  # Choriocarcinoma of placenta (disorder)
        "448447004",  # Non-Hodgkin's lymphoma of skin (disorder)
        "448450001",  # Sarcoma of omentum (disorder)
        "448465000",  # Diffuse non-Hodgkin's lymphoma of testis (disorder)
        "448468003",  # Diffuse non-Hodgkin's lymphoma of oral cavity (disorder)
        "448509007",  # Transglottic malignant neoplasm of larynx (disorder)
        "448553002",  # Lymphoma of pelvis (disorder)
        "448554008",  # Liposarcoma of connective tissue (disorder)
        "448555009",  # Lymphoma of body of stomach (disorder)
        "448558006",  # Malignant neoplasm of maxillofacial bone (disorder)
        "448559003",  # Malignant teratoma of retroperitoneum (disorder)
        "448560008",  # Diffuse non-Hodgkin's lymphoma of extranodal site (disorder)
        "448561007",  # Follicular non-Hodgkin's lymphoma of extranodal site (disorder)
        "448607004",  # Diffuse non-Hodgkin's lymphoma of uterine cervix (disorder)
        "448609001",  # Diffuse non-Hodgkin's lymphoma of ovary (disorder)
        "448663003",  # Diffuse non-Hodgkin's lymphoma of stomach (disorder)
        "448664009",  # Malignant epithelial neoplasm of small intestine (disorder)
        "448665005",  # Malignant epithelial neoplasm of hypopharynx (disorder)
        "448666006",  # Follicular non-Hodgkin's lymphoma of bone (disorder)
        "448668007",  # Malignant neoplasm of mandible (disorder)
        "448669004",  # Malignant neoplasm of soft tissue of orbit (disorder)
        "448670003",  # Malignant neoplasm of posterior mediastinum (disorder)
        "448672006",  # Follicular non-Hodgkin's lymphoma of lung (disorder)
        "448674007",  # Malignant neoplasm of parametrium (disorder)
        "448675008",  # Malignant neoplasm of digestive system (disorder)
        "448709005",  # Non-Hodgkin's lymphoma of stomach (disorder)
        "448710000",  # Sarcoma of bone (disorder)
        "448711001",  # Sarcoma of back (disorder)
        "448712008",  # Sarcoma of femur (disorder)
        "448738008",  # Non-Hodgkin's lymphoma of soft tissue (disorder)
        "448774004",  # Non-Hodgkin's lymphoma of uterine cervix (disorder)
        "448775003",  # Sarcoma of radius (disorder)
        "448776002",  # Sarcoma of vertebra (disorder)
        "448863000",  # Malignant epithelial neoplasm of pineal gland (disorder)
        "448864006",  # Malignant epithelial neoplasm of ureter (disorder)
        "448865007",  # Follicular non-Hodgkin's lymphoma of skin (disorder)
        "448867004",  # Diffuse non-Hodgkin's lymphoma of lung (disorder)
        "448868009",  # Malignant neoplasm of lateral wall of oropharynx (disorder)
        "448882009",  # Malignant neoplasm of intraabdominal organ (disorder)
        "448930008",  # Squamous cell carcinoma of nose (disorder)
        "448931007",  # Squamous cell carcinoma of shoulder (disorder)
        "448954003",  # Malignant epithelial neoplasm of urethra (disorder)
        "448988009",  # Malignant epithelial neoplasm of nose (disorder)
        "448989001",  # Malignant epithelial neoplasm of brain (disorder)
        "448990005",  # Malignant epithelial neoplasm of nasal cavity (disorder)
        "448992002",  # Malignant epithelial neoplasm of appendix (disorder)
        "448993007",  # Malignant epithelial neoplasm of lung (disorder)
        "448994001",  # Malignant epithelial neoplasm of upper rectum (disorder)
        "448995000",  # Follicular non-Hodgkin's lymphoma of central nervous system (disorder)
        "449034009",  # Malignant neoplasm of anterior and lateral floor of mouth (disorder)
        "449053004",  # Lymphoma of lower esophagus (disorder)
        "449054005",  # Malignant epithelial neoplasm of fundus of uterus (disorder)
        "449055006",  # Leiomyosarcoma of cardioesophageal junction (disorder)
        "449058008",  # Follicular non-Hodgkin's lymphoma of tonsil (disorder)
        "449059000",  # Follicular non-Hodgkin's lymphoma of uterine cervix (disorder)
        "449063007",  # Follicular non-Hodgkin's lymphoma of oral cavity (disorder)
        "449065000",  # Diffuse non-Hodgkin's lymphoma of nose (disorder)
        "449066004",  # Malignant neoplasm of upper respiratory tract (disorder)
        "449067008",  # Malignant neoplasm of parietal pleura (disorder)
        "449072004",  # Lymphoma of gastrointestinal tract (disorder)
        "449073009",  # Malignant epithelial neoplasm of body of uterus (disorder)
        "449074003",  # Lymphoma of small intestine (disorder)
        "449075002",  # Lymphoma of cardioesophageal junction (disorder)
        "449077005",  # Malignant epithelial neoplasm of maxilla (disorder)
        "449096009",  # Malignant neoplasm of respiratory system (disorder)
        "449100005",  # Sarcoma of bone of pelvis (disorder)
        "449101009",  # Sarcoma of sternum (disorder)
        "449153001",  # Adenocarcinoma of lower esophagus (disorder)
        "449156009",  # Malignant epithelial neoplasm of floor of mouth (disorder)
        "449173006",  # Diffuse non-Hodgkin's lymphoma of tonsil (disorder)
        "449176003",  # Diffuse non-Hodgkin's lymphoma of intestine (disorder)
        "449177007",  # Diffuse non-Hodgkin's lymphoma of bone (disorder)
        "449206007",  # Sarcoma of tibia (disorder)
        "449207003",  # Sarcoma of humerus (disorder)
        "449208008",  # Sarcoma of coccyx (disorder)
        "449211009",  # Malignant epithelial neoplasm of face (disorder)
        "449216004",  # Diffuse non-Hodgkin's lymphoma of soft tissue (disorder)
        "449217008",  # Diffuse non-Hodgkin's lymphoma of skin (disorder)
        "449218003",  # Lymphoma of sigmoid colon (disorder)
        "449219006",  # Follicular non-Hodgkin's lymphoma of nasopharynx (disorder)
        "449220000",  # Diffuse follicle center lymphoma (disorder)
        "449221001",  # Diffuse non-Hodgkin's lymphoma of central nervous system (disorder)
        "449222008",  # Follicular non-Hodgkin's lymphoma of stomach (disorder)
        "449223003",  # Malignant neoplasm of alveolus of maxilla (disorder)
        "449224009",  # Malignant neoplasm of anterior mediastinum (disorder)
        "449248000",  # Nasopharyngeal carcinoma (disorder)
        "449253005",  # Malignant epithelial neoplasm of hypothalamus (disorder)
        "449254004",  # Malignant epithelial neoplasm of pharynx (disorder)
        "449259009",  # Malignant neoplasm of broad ligament of uterus (disorder)
        "449260004",  # Malignant neoplasm of alveolus dentalis (disorder)
        "449267001",  # Sarcoma of mandible (disorder)
        "449268006",  # Sarcoma of clavicle (disorder)
        "449269003",  # Sarcoma of rib (disorder)
        "449292003",  # Non-Hodgkin's lymphoma of tonsil (disorder)
        "449293008",  # Sarcoma of connective tissue (disorder)
        "449294002",  # Sarcoma of scapula (disorder)
        "449295001",  # Sarcoma of skull (disorder)
        "449307001",  # Follicular non-Hodgkin's lymphoma of ovary (disorder)
        "449308006",  # Malignant neoplasm of visceral pleura (disorder)
        "449309003",  # Malignant neoplasm of skin of scrotum (disorder)
        "449318001",  # Non-Hodgkin's lymphoma of prostate (disorder)
        "449377002",  # Malignant neoplasm of pelvic peritoneum (disorder)
        "449416001",  # Malignant epithelial neoplasm of skin of anus (disorder)
        "449417005",  # Malignant epithelial neoplasm of nasal septum (disorder)
        "449418000",  # Follicular non-Hodgkin's lymphoma of testis (disorder)
        "449419008",  # Follicular non-Hodgkin's lymphoma of intestine (disorder)
        "449420002",  # Malignant neoplasm of cerebellum (disorder)
        "449472007",  # Malignant epithelial neoplasm of alveolus dentalis (disorder)
        "449487002",  # Malignant epithelial neoplasm of mandible (disorder)
        "449497006",  # Sarcoma of pelvic peritoneum (disorder)
        "449578008",  # Malignant neoplasm of alveolus of mandible (disorder)
        "449627008",  # Malignant neoplasm of long bone of lower limb (disorder)
        "449628003",  # Malignant neoplasm of long bone of lower leg (disorder)
        "449629006",  # Neoplasm of uncertain behavior of lower leg (disorder)
        "449630001",  # Metastatic malignant neoplasm to skin of lower leg (disorder)
        "449631002",  # Metastatic malignant neoplasm to skin of upper arm (disorder)
        "449632009",  # Metastatic malignant neoplasm to lower leg (disorder)
        "449633004",  # Metastatic malignant neoplasm to upper arm (disorder)
        "449634005",  # Primary malignant neoplasm of skin of lower leg (disorder)
        "449635006",  # Primary malignant neoplasm of lower leg (disorder)
        "449636007",  # Malignant melanoma of skin of lower leg (disorder)
        "449637003",  # Malignant melanoma of skin of upper arm (disorder)
        "449803009",  # Primary malignant neoplasm of dome of urinary bladder (disorder)
        "450521000124109",  # Marginal zone lymphoma of axillary lymph node (disorder)
        "45061000087102",  # Malignant neoplasm of breast in remission (disorder)
        "452241000124100",  # Recurrent malignant neoplastic disease (disorder)
        "452291000124109",  # Relapse multiple myeloma (disorder)
        "454721000124105",  # Chemosensitive malignant neoplastic disease (disorder)
        "455733521000119103",  # Primary squamous cell carcinoma of skin of head and/or neck (disorder)
        "456911000124108",  # Diffuse non-Hodgkin's lymphoma Lugano stage III (disorder)
        "457721000124104",  # Metastatic non-small cell lung cancer (disorder)
        "458321000124102",  # Metastatic urothelial carcinoma (disorder)
        "458581000124106",  # Metastatic HER2 positive neoplasm of gastroesophageal junction (disorder)
        "459371000124108",  # Metastasis from gastroesophageal adenocarcinoma (disorder)
        "459381000124106",  # Metastasis from hormone-refractory prostate cancer (disorder)
        "459391000124109",  # Metastasis from human epidermal growth factor 2 positive carcinoma of breast (disorder)
        "459401000124106",  # Metastasis from pancreatic endocrine carcinoma (disorder)
        "459411000124109",  # Metastasis from collecting duct carcinoma of nipple (disorder)
        "459421000124101",  # Metastasis from malignant tumor of penis (disorder)
        "461311000124107",  # Refractory Hodgkin lymphoma (disorder)
        "461371000124103",  # Recurrent ovarian cancer (disorder)
        "46724008",  # Polyglandular activity in multiple endocrine adenomatosis (disorder)
        "53132006",  # Zollinger-Ellison syndrome (disorder)
        "550991000124107",  # Malignant neoplasm in full remission (disorder)
        "551001000124108",  # Malignant neoplasm in partial remission (disorder)
        "58961005",  # Lethal midline granuloma (disorder)
        "609519004",  # Gestational trophoblastic neoplasia (disorder)
        "61311000119104",  # Acute myeloid leukemia, minimal differentiation, FAB M0 in remission (disorder)
        "61321000119106",  # Acute myeloid leukemia without maturation, FAB M1 in remission (disorder)
        "61493004",  # Mu heavy chain disease (disorder)
        "61530001",  # Multiple endocrine neoplasia, type 2b (disorder)
        "61808009",  # Multiple endocrine neoplasia, type 2 (disorder)
        "62497000",  # Stewart-Treves syndrome (disorder)
        "63175003",  # Localized extracutaneous mastocytosis (disorder)
        "67811000119102",  # Primary small cell malignant neoplasm of lung, TNM stage 1 (disorder)
        "67821000119109",  # Primary small cell malignant neoplasm of lung, TNM stage 2 (disorder)
        "67831000119107",  # Primary small cell malignant neoplasm of lung, TNM stage 3 (disorder)
        "67841000119103",  # Primary small cell malignant neoplasm of lung, TNM stage 4 (disorder)
        "681221000124106",  # Primary malignant hormone-refractory neoplasm of prostate (disorder)
        "681591000119108",  # Primary adenocarcinoma of anterior wall of urinary bladder (disorder)
        "681601000119101",  # Primary adenocarcinoma of ascending colon (disorder)
        "681611000119103",  # Primary adenocarcinoma of neck of urinary bladder (disorder)
        "681621000119105",  # Primary adenocarcinoma of body of pancreas (disorder)
        "681631000119108",  # Primary adenocarcinoma of body of stomach (disorder)
        "681651000119102",  # Primary adenocarcinoma of descending colon (disorder)
        "681661000119100",  # Primary adenocarcinoma of dome of urinary bladder (disorder)
        "681681000119109",  # Primary adenocarcinoma of fundus of stomach (disorder)
        "681691000119107",  # Primary adenocarcinoma of gastrointestinal tract (disorder)
        "681701000119107",  # Primary adenocarcinoma of greater curvature of stomach (disorder)
        "681711000119105",  # Primary adenocarcinoma of head (disorder)
        "681721000119103",  # Primary adenocarcinoma of head of pancreas (disorder)
        "681751000119106",  # Primary adenocarcinoma of intrahepatic bile duct (disorder)
        "681781000119104",  # Primary adenocarcinoma of lesser curvature of stomach (disorder)
        "681821000119109",  # Primary adenocarcinoma of neck (disorder)
        "681831000119107",  # Primary adenocarcinoma of neck of pancreas (disorder)
        "681911000119108",  # Primary adenocarcinoma of pancreatic duct (disorder)
        "681921000119101",  # Primary adenocarcinoma of posterior wall of urinary bladder (disorder)
        "681941000119107",  # Primary adenocarcinoma of pylorus (disorder)
        "681971000119100",  # Primary adenocarcinoma of tail of pancreas (disorder)
        "681991000119104",  # Primary adenocarcinoma of trigone of urinary bladder (disorder)
        "682031000119109",  # Primary adenocarcinoma of urachus (disorder)
        "682041000119100",  # Primary adenocarcinoma of ureteral orifice (disorder)
        "682221000119103",  # Primary anaplastic astrocytoma of cerebrum (disorder)
        "682231000119100",  # Primary anaplastic astrocytoma of frontal lobe (disorder)
        "682241000119109",  # Primary anaplastic astrocytoma of occipital lobe (disorder)
        "682251000119106",  # Primary anaplastic astrocytoma of parietal lobe (disorder)
        "682261000119108",  # Primary anaplastic astrocytoma of temporal lobe (disorder)
        "682271000119102",  # Primary angiosarcoma of abdomen (disorder)
        "682291000119101",  # Primary angiosarcoma of axillary region (disorder)
        "682311000119102",  # Primary angiosarcoma of face (disorder)
        "682321000119109",  # Primary angiosarcoma of head (disorder)
        "682361000119104",  # Primary angiosarcoma of neck (disorder)
        "682381000119108",  # Primary angiosarcoma of pelvis (disorder)
        "682411000119106",  # Primary angiosarcoma of thorax (disorder)
        "682421000119104",  # Primary angiosarcoma of trunk (disorder)
        "682441000119105",  # Primary astrocytoma of cerebral ventricle (disorder)
        "682451000119107",  # Primary astrocytoma of frontal lobe (disorder)
        "682461000119109",  # Primary astrocytoma of occipital lobe (disorder)
        "682471000119103",  # Primary astrocytoma of parietal lobe (disorder)
        "682481000119100",  # Primary astrocytoma of temporal lobe (disorder)
        "68261000119102",  # Diffuse non-Hodgkin's lymphoma Lugano stage I (disorder)
        "68271000119108",  # Diffuse non-Hodgkin's lymphoma Lugano stage II (disorder)
        "68291000119109",  # Diffuse non-Hodgkin's lymphoma Lugano stage IV (disorder)
        "683031000119101",  # Primary carcinoma ex pleomorphic adenoma of salivary gland (disorder)
        "683041000119105",  # Primary carcinoma ex pleomorphic adenoma of submandibular gland (disorder)
        "683121000119103",  # Primary chondrosarcoma of mandible (disorder)
        "683131000119100",  # Primary chondrosarcoma of vertebral column (disorder)
        "683981000119101",  # Primary Ewing sarcoma of bone of spine (disorder)
        "683991000119103",  # Extensive stage primary small cell carcinoma of lung (disorder)
        "684901000119107",  # Primary glioblastoma multiforme of cerebrum (disorder)
        "684911000119105",  # Primary glioblastoma multiforme of frontal lobe (disorder)
        "684921000119103",  # Primary glioblastoma multiforme of occipital lobe (disorder)
        "684931000119100",  # Primary glioblastoma multiforme of parietal lobe (disorder)
        "684941000119109",  # Primary glioblastoma multiforme of temporal lobe (disorder)
        "685571000119105",  # Primary leiomyosarcoma of abdomen (disorder)
        "685601000119104",  # Primary leiomyosarcoma of buttock (disorder)
        "685611000119101",  # Primary leiomyosarcoma of face (disorder)
        "685621000119108",  # Primary leiomyosarcoma of head (disorder)
        "685641000119102",  # Primary leiomyosarcoma of inguinal region (disorder)
        "685671000119109",  # Primary leiomyosarcoma of neck (disorder)
        "685691000119105",  # Primary leiomyosarcoma of pelvis (disorder)
        "685701000119105",  # Primary leiomyosarcoma of perineum (disorder)
        "685721000119101",  # Primary leiomyosarcoma of thorax (disorder)
        "685731000119103",  # Primary leiomyosarcoma of trunk (disorder)
        "685831000119105",  # Primary liposarcoma of soft tissue of abdomen (disorder)
        "685851000119104",  # Primary liposarcoma of soft tissue of axilla (disorder)
        "685861000119102",  # Primary liposarcoma of soft tissue of buttock (disorder)
        "685901000119108",  # Primary liposarcoma of soft tissue of inguinal region (disorder)
        "685921000119104",  # Primary liposarcoma of soft tissue of neck (disorder)
        "685941000119105",  # Primary liposarcoma of soft tissue of pelvis (disorder)
        "685951000119107",  # Primary liposarcoma of soft tissue of perineum (disorder)
        "685971000119103",  # Primary liposarcoma of soft tissue of thorax (disorder)
        "685981000119100",  # Primary liposarcoma of soft tissue of trunk (disorder)
        "686431000119100",  # Primary malignant glioma of frontal lobe (disorder)
        "686441000119109",  # Primary malignant glioma of occipital lobe (disorder)
        "686451000119106",  # Primary malignant glioma of parietal lobe (disorder)
        "686461000119108",  # Primary malignant glioma of temporal lobe (disorder)
        "686591000119102",  # Primary malignant pheochromocytoma of left adrenal gland (disorder)
        "686601000119109",  # Primary malignant pheochromocytoma of right adrenal gland (disorder)
        "686981000119108",  # Mycosis fungoides of inguinal lymph node (disorder)
        "686991000119106",  # Mycosis fungoides of lower limb lymph node (disorder)
        "687331000119109",  # Primary oligodendroglioma of frontal lobe (disorder)
        "687351000119103",  # Primary oligodendroglioma of parietal lobe (disorder)
        "687361000119101",  # Primary oligodendroglioma of temporal lobe (disorder)
        "687401000119105",  # Primary osteosarcoma of mandible (disorder)
        "687421000119101",  # Primary osteosarcoma of vertebral column (disorder)
        "687771000119106",  # Primary rhabdomyosarcoma of abdomen (disorder)
        "687801000119108",  # Primary rhabdomyosarcoma of buttock (disorder)
        "687811000119106",  # Primary rhabdomyosarcoma of face (disorder)
        "687821000119104",  # Primary rhabdomyosarcoma of head (disorder)
        "687841000119105",  # Primary rhabdomyosarcoma of inguinal region (disorder)
        "687861000119109",  # Primary rhabdomyosarcoma of neck (disorder)
        "687881000119100",  # Primary rhabdomyosarcoma of pelvis (disorder)
        "687891000119102",  # Primary rhabdomyosarcoma of perineum (disorder)
        "687911000119100",  # Primary rhabdomyosarcoma of thorax (disorder)
        "687941000119101",  # Primary sarcoma of soft tissues of abdomen (disorder)
        "687961000119102",  # Primary sarcoma of axilla (disorder)
        "687971000119108",  # Primary sarcoma of buttock (disorder)
        "687981000119106",  # Sarcoma of face (disorder)
        "687991000119109",  # Primary sarcoma of head (disorder)
        "688011000119102",  # Primary sarcoma of inguinal region (disorder)
        "688031000119107",  # Primary sarcoma of neck (disorder)
        "688061000119104",  # Primary sarcoma of perineum (disorder)
        "688081000119108",  # Primary sarcoma of thorax (disorder)
        "688091000119106",  # Primary sarcoma of trunk (disorder)
        "688321000119107",  # Primary malignant neoplasm of skin of bridge of nose (disorder)
        "688381000119106",  # Primary malignant neoplasm of skin of ala nasi (disorder)
        "688411000119109",  # Primary malignant neoplasm of skin of tip of nose (disorder)
        "688671000119103",  # Primary squamous cell carcinoma of anterior floor of mouth (disorder)
        "688681000119100",  # Primary squamous cell carcinoma of body of tongue (disorder)
        "688691000119102",  # Primary squamous cell carcinoma of anterior wall of urinary bladder (disorder)
        "688711000119104",  # Primary squamous cell carcinoma of aryepiglottic fold (disorder)
        "688731000119109",  # Primary squamous cell carcinoma of neck of urinary bladder (disorder)
        "688751000119103",  # Primary squamous cell carcinoma of border of tongue (disorder)
        "688781000119105",  # Primary squamous cell carcinoma of commissure of lip (disorder)
        "688801000119109",  # Primary squamous cell carcinoma of dome of urinary bladder (disorder)
        "688811000119107",  # Primary squamous cell carcinoma of dorsal surface of tongue (disorder)
        "688861000119105",  # Primary squamous cell carcinoma of hard palate (disorder)
        "688891000119103",  # Primary squamous cell carcinoma of mucous membrane of lip (disorder)
        "688901000119104",  # Primary squamous cell carcinoma of mucous membrane of lower lip (disorder)
        "688911000119101",  # Primary squamous cell carcinoma of mucous membrane of upper lip (disorder)
        "688931000119106",  # Primary squamous cell carcinoma of lateral floor of mouth (disorder)
        "688941000119102",  # Primary squamous cell carcinoma of lateral wall of urinary bladder (disorder)
        "688981000119107",  # Primary squamous cell carcinoma of lower gum (disorder)
        "689211000119102",  # Primary squamous cell carcinoma of perianal skin (disorder)
        "689231000119107",  # Primary squamous cell carcinoma of posterior wall of urinary bladder (disorder)
        "689251000119101",  # Primary squamous cell carcinoma of posterior wall of nasopharynx (disorder)
        "689281000119108",  # Primary squamous cell carcinoma of retromolar area (disorder)
        "689311000119105",  # Primary squamous cell carcinoma of soft palate (disorder)
        "689361000119108",  # Primary squamous cell carcinoma of tonsillar fossa (disorder)
        "689371000119102",  # Primary squamous cell carcinoma of tonsillar pillar (disorder)
        "689401000119104",  # Primary squamous cell carcinoma of upper gum (disorder)
        "689441000119102",  # Primary squamous cell carcinoma of ureteral orifice (disorder)
        "689451000119100",  # Primary squamous cell carcinoma of uvula (disorder)
        "689471000119109",  # Primary squamous cell carcinoma of inferior surface of tongue (disorder)
        "689481000119107",  # Primary squamous cell carcinoma of vermilion border of lower lip (disorder)
        "689491000119105",  # Primary squamous cell carcinoma of vermilion border of upper lip (disorder)
        "689501000119103",  # Primary squamous cell carcinoma of vestibule of mouth (disorder)
        "689771000119103",  # Primary transitional cell carcinoma of anterior wall of urinary bladder (disorder)
        "689781000119100",  # Primary transitional cell carcinoma of neck of urinary bladder (disorder)
        "68979007",  # Heavy chain disease (disorder)
        "689791000119102",  # Primary transitional cell carcinoma of dome of urinary bladder (disorder)
        "689801000119101",  # Primary transitional cell carcinoma of lateral wall of urinary bladder (disorder)
        "689821000119105",  # Primary transitional cell carcinoma of posterior wall of urinary bladder (disorder)
        "689831000119108",  # Primary transitional cell carcinoma of trigone of urinary bladder (disorder)
        "689851000119102",  # Primary transitional cell carcinoma of ureteral orifice (disorder)
        "69408002",  # Gorlin syndrome (disorder)
        "697993003",  # Undifferentiated carcinoma of nasal sinus (disorder)
        "698011002",  # Keratinizing squamous cell carcinoma of nasopharynx (disorder)
        "698040004",  # Malignant melanoma of nasal cavity (disorder)
        "698041000",  # Malignant melanoma of vestibule of mouth (disorder)
        "698042007",  # Malignant melanoma of tongue (disorder)
        "698043002",  # Malignant melanoma of floor of mouth (disorder)
        "698044008",  # Malignant melanoma of palatine arch (disorder)
        "698045009",  # Malignant melanoma of buccal mucosa (disorder)
        "698048006",  # Undifferentiated nonkeratinizing squamous cell carcinoma of nasopharynx (disorder)
        "698200003",  # Large cell Ewing sarcoma of bone (disorder)
        "698285005",  # Malignant melanoma of ethmoid sinus (disorder)
        "698286006",  # Malignant melanoma of palate (disorder)
        "698287002",  # Malignant melanoma of gum (disorder)
        "698288007",  # Malignant melanoma of maxillary sinus (disorder)
        "699028006",  # Primitive neuroectodermal tumor (disorder)
        "699317002",  # Paratesticular malignant neoplasm (disorder)
        "699318007",  # Supratentorial primitive neuroectodermal tumor (disorder)
        "699354006",  # Sarcoma of orbit (disorder)
        "699355007",  # Leiomyosarcoma of orbit (disorder)
        "699357004",  # Low grade endometrial stromal sarcoma (disorder)
        "699358009",  # High grade endometrial stromal sarcoma (disorder)
        "699537002",  # Erdheim-Chester disease (disorder)
        "699657009",  # Hepatosplenic gamma-delta cell lymphoma (disorder)
        "699659007",  # Glandular malignant peripheral nerve sheath tumor (disorder)
        "699704002",  # Classic medulloblastoma (disorder)
        "699818003",  # T-cell large granular lymphocytic leukemia (disorder)
        "700423003",  # Adenocarcinoma of pancreas (disorder)
        "700488005",  # Malignant sex cord tumor of testis (disorder)
        "702368000",  # Carcinosarcoma of ovary (disorder)
        "702369008",  # Carcinosarcoma of uterus (disorder)
        "702391001",  # Renal cell carcinoma (disorder)
        "702392008",  # Metastatic renal cell carcinoma (disorder)
        "702405001",  # Malignant granulosa cell tumor of testis (disorder)
        "702446006",  # Core binding factor acute myeloid leukemia (disorder)
        "702467006",  # Malignant neoplasm of augmented urinary bladder (disorder)
        "702476004",  # Therapy-related myelodysplastic syndrome (disorder)
        "702785000",  # Large cell anaplastic lymphoma T cell and Null cell type (disorder)
        "702786004",  # Follicular non-Hodgkin's lymphoma diffuse follicle center sub-type grade 1 (disorder)
        "702977001",  # Follicular non-Hodgkin's lymphoma diffuse follicle center cell sub-type grade 2 (disorder)
        "703228009",  # Non-small cell lung cancer with mutation in epidermal growth factor receptor (disorder)
        "703230006",  # Non-small cell lung cancer without mutation in epidermal growth factor receptor (disorder)
        "703387000",  # Acute myeloid leukemia with normal karyotype (disorder)
        "703429003",  # Malignant optic glioma of adulthood (disorder)
        "703625002",  # Kaposi sarcoma not associated with acquired immunodeficiency syndrome (disorder)
        "703626001",  # Anaplastic large cell lymphoma, T/Null cell, primary systemic type (disorder)
        "704152002",  # Metastatic neuroblastoma (disorder)
        "705061009",  # Childhood myelodysplastic syndrome (disorder)
        "706970001",  # Triple negative malignant neoplasm of breast (disorder)
        "707337006",  # Primary adenocarcinoma of accessory sinus (disorder)
        "707339009",  # Primary adenocarcinoma of maxillary sinus (disorder)
        "707342003",  # Primary adenocarcinoma of ethmoidal sinus (disorder)
        "707343008",  # Primary adenocarcinoma of frontal sinus (disorder)
        "707344002",  # Primary adenocarcinoma of sphenoidal sinus (disorder)
        "707345001",  # Primary carcinoma of accessory sinus (disorder)
        "707346000",  # Primary carcinoma of ethmoidal sinus (disorder)
        "707347009",  # Primary carcinoma of maxillary sinus (disorder)
        "707348004",  # Primary carcinoma of sphenoidal sinus (disorder)
        "707349007",  # Primary carcinoma of frontal sinus (disorder)
        "707350007",  # Malignant melanoma of accessory sinus (disorder)
        "707353009",  # Primary squamous cell carcinoma of accessory sinus (disorder)
        "707354003",  # Primary squamous cell carcinoma of maxillary sinus (disorder)
        "707355002",  # Primary squamous cell carcinoma of sphenoidal sinus (disorder)
        "707356001",  # Primary squamous cell carcinoma of frontal sinus (disorder)
        "707357005",  # Primary squamous cell carcinoma of laryngeal cartilage (disorder)
        "707358000",  # Primary squamous cell carcinoma of larynx (disorder)
        "707359008",  # Primary squamous cell carcinoma of ethmoidal sinus (disorder)
        "707360003",  # Primary lymphoepithelial carcinoma of larynx (disorder)
        "707361004",  # Malignant melanoma of frontal sinus (disorder)
        "707362006",  # Malignant melanoma of sphenoidal sinus (disorder)
        "707377003",  # Primary signet ring cell carcinoma of trachea (disorder)
        "707378008",  # Primary myoepithelial carcinoma of trachea (disorder)
        "707379000",  # Primary mucoepidermoid carcinoma of trachea (disorder)
        "707380002",  # Primary salivary gland type carcinoma of trachea (disorder)
        "707383000",  # Primary mucinous cystadenocarcinoma of trachea (disorder)
        "707384006",  # Primary solid carcinoma of trachea (disorder)
        "707385007",  # Primary undifferentiated carcinoma of trachea (disorder)
        "707386008",  # Primary acinar cell carcinoma of trachea (disorder)
        "707388009",  # Primary squamous cell carcinoma of trachea (disorder)
        "707389001",  # Primary clear cell squamous cell carcinoma of trachea (disorder)
        "707390005",  # Primary basaloid squamous cell carcinoma of trachea (disorder)
        "707391009",  # Primary papillary squamous cell carcinoma of trachea (disorder)
        "707392002",  # Primary giant cell carcinoma of trachea (disorder)
        "707393007",  # Primary adenosquamous carcinoma of trachea (disorder)
        "707394001",  # Primary spindle cell carcinoma of trachea (disorder)
        "707395000",  # Primary adenocarcinoma of hypopharynx (disorder)
        "707396004",  # Primary oxyphilic adenocarcinoma of oropharynx (disorder)
        "707397008",  # Primary basal cell adenocarcinoma of oropharynx (disorder)
        "707398003",  # Primary polymorphous low grade adenocarcinoma of oropharynx (disorder)
        "707399006",  # Primary papillary adenocarcinoma of oropharynx (disorder)
        "707400004",  # Primary mucinous adenocarcinoma of oropharynx (disorder)
        "707401000",  # Primary clear cell adenocarcinoma of oropharynx (disorder)
        "707402007",  # Primary adenocarcinoma of oropharynx (disorder)
        "707403002",  # Primary fetal adenocarcinoma of lung (disorder)
        "707404008",  # Primary mixed subtype adenocarcinoma of lung (disorder)
        "707405009",  # Primary adenosquamous carcinoma of lung (disorder)
        "707406005",  # Primary mucoepidermoid carcinoma of hypopharynx (disorder)
        "707407001",  # Primary signet ring cell carcinoma of lung (disorder)
        "707408006",  # Primary small cell non-keratinizing squamous cell carcinoma of lung (disorder)
        "707409003",  # Primary acinar cell carcinoma of lung (disorder)
        "707410008",  # Primary solid carcinoma of lung (disorder)
        "707411007",  # Primary papillary adenocarcinoma of lung (disorder)
        "707421004",  # Primary undifferentiated carcinoma of larynx (disorder)
        "707422006",  # Primary spindle cell squamous cell carcinoma of larynx (disorder)
        "707423001",  # Primary basaloid carcinoma of larynx (disorder)
        "707424007",  # Primary adenosquamous cell carcinoma of larynx (disorder)
        "707425008",  # Primary adenoid squamous cell carcinoma of larynx (disorder)
        "707426009",  # Primary papillary squamous cell carcinoma of larynx (disorder)
        "707427000",  # Primary verrucous carcinoma of larynx (disorder)
        "707429002",  # Overlapping squamous cell carcinoma of larynx (disorder)
        "707430007",  # Overlapping squamous cell carcinoma of laryngeal cartilage (disorder)
        "707451005",  # Primary adenocarcinoma of lung (disorder)
        "707452003",  # Primary mucinous adenocarcinoma of lung (disorder)
        "707453008",  # Primary clear cell squamous cell carcinoma of lung (disorder)
        "707454002",  # Primary basaloid squamous cell carcinoma of lung (disorder)
        "707455001",  # Primary papillary squamous cell carcinoma of lung (disorder)
        "707456000",  # Primary undifferentiated carcinoma of lung (disorder)
        "707457009",  # Primary spindle cell carcinoma of lung (disorder)
        "707458004",  # Primary pleomorphic carcinoma of lung (disorder)
        "707460002",  # Primary pseudosarcomatous carcinoma of lung (disorder)
        "707464006",  # Primary myoepithelial carcinoma of lung (disorder)
        "707465007",  # Primary mucoepidermoid carcinoma of lung (disorder)
        "707466008",  # Primary adenoid cystic carcinoma of lung (disorder)
        "707467004",  # Primary salivary gland type carcinoma of lung (disorder)
        "707468009",  # Primary mixed mucinous and non-mucinous bronchiolo-alveolar carcinoma of lung (disorder)
        "707469001",  # Primary non-mucinous bronchiolo-alveolar carcinoma of lung (disorder)
        "707470000",  # Primary mucinous bronchiolo-alveolar carcinoma of lung (disorder)
        "707471001",  # Primary clear cell adenocarcinoma of trachea (disorder)
        "707472008",  # Primary papillary adenocarcinoma of trachea (disorder)
        "707473003",  # Primary mucinous adenocarcinoma of trachea (disorder)
        "707475005",  # Primary adenocarcinoma of trachea (disorder)
        "707479004",  # Primary adenocarcinoma of subglottis (disorder)
        "707481002",  # Primary basaloid squamous cell carcinoma of hypopharynx (disorder)
        "707482009",  # Primary papillary squamous cell carcinoma of hypopharynx (disorder)
        "707483004",  # Primary undifferentiated carcinoma of hypopharynx (disorder)
        "707484005",  # Primary adenoid squamous cell carcinoma of hypopharynx (disorder)
        "707485006",  # Primary adenosquamous carcinoma of hypopharynx (disorder)
        "707486007",  # Primary basaloid carcinoma of hypopharynx (disorder)
        "707487003",  # Primary giant cell carcinoma of hypopharynx (disorder)
        "707489000",  # Primary spindle cell squamous cell carcinoma of hypopharynx (disorder)
        "707490009",  # Primary verrucous carcinoma of hypopharynx (disorder)
        "707491008",  # Primary lymphoepithelial carcinoma of hypopharynx (disorder)
        "707492001",  # Primary squamous cell carcinoma of hypopharynx (disorder)
        "707493006",  # Primary lymphoepithelial carcinoma of trachea (disorder)
        "707494000",  # Primary verrucous carcinoma of trachea (disorder)
        "707495004",  # Primary squamous cell adenoid carcinoma of trachea (disorder)
        "707528007",  # Primary squamous cell carcinoma of nasopharynx (disorder)
        "707529004",  # Overlapping squamous cell carcinoma of oropharynx (disorder)
        "707531008",  # Primary squamous cell carcinoma of branchial cleft (disorder)
        "707532001",  # Primary squamous cell carcinoma of posterior wall of oropharynx (disorder)
        "707535004",  # Primary squamous cell carcinoma of lateral wall of oropharynx (disorder)
        "707537007",  # Primary squamous cell carcinoma of anterior surface of epiglottis (disorder)
        "707538002",  # Primary squamous cell carcinoma of vallecula (disorder)
        "707539005",  # Primary adenoid cystic carcinoma of hypopharynx (disorder)
        "707575007",  # Primary squamous cell carcinoma of supraglottis (disorder)
        "707576008",  # Primary squamous cell carcinoma of subglottis (disorder)
        "707579001",  # Primary basaloid squamous cell carcinoma of oropharynx (disorder)
        "707580003",  # Primary basaloid carcinoma of oropharynx (disorder)
        "707581004",  # Primary papillary squamous cell carcinoma of oropharynx (disorder)
        "707582006",  # Primary spindle cell squamous cell carcinoma of oropharynx (disorder)
        "707583001",  # Primary adenosquamous carcinoma of oropharynx (disorder)
        "707584007",  # Primary lymphoepithelial carcinoma of oropharynx (disorder)
        "707585008",  # Primary squamous cell carcinoma of oropharynx (disorder)
        "707586009",  # Primary myoepithelial carcinoma of oropharynx (disorder)
        "707587000",  # Primary carcinoma ex pleomorphic adenoma of oropharynx (disorder)
        "707588005",  # Primary epithelial-myoepithelial carcinoma of oropharynx (disorder)
        "707589002",  # Primary cystadenocarcinoma of oropharynx (disorder)
        "707590006",  # Primary acinar cell carcinoma of oropharynx (disorder)
        "707591005",  # Primary mucoepidermoid carcinoma of oropharynx (disorder)
        "707592003",  # Primary infiltrating duct carcinoma of oropharynx (disorder)
        "707593008",  # Primary salivary gland-type tumor of oropharynx (disorder)
        "707595001",  # Primary mucinous cystadenocarcinoma of lung (disorder)
        "707596000",  # Primary carcinosarcoma of lung (disorder)
        "707627009",  # Primary salivary gland type carcinoma of hypopharynx (disorder)
        "707628004",  # Overlapping squamous cell carcinoma of hypopharynx (disorder)
        "707660007",  # Primary giant cell carcinoma of larynx (disorder)
        "707662004",  # Primary basaloid squamous cell carcinoma of larynx (disorder)
        "707664003",  # Primary squamous cell carcinoma of glottis (disorder)
        "707670009",  # Pleuropulmonary blastoma (disorder)
        "707671008",  # Pleuropulmonary blastoma type I (disorder)
        "707672001",  # Pleuropulmonary blastoma type II (disorder)
        "707673006",  # Pleuropulmonary blastoma type III (disorder)
        "707674000",  # Primary malignant epithelial neoplasm of trachea (disorder)
        "707686002",  # Primary squamous cell carcinoma of posterior wall of hypopharynx (disorder)
        "707697002",  # Primary squamous cell carcinoma of hypopharyngeal aspect of aryepiglottic fold (disorder)
        "707703001",  # Primary squamous cell carcinoma of postcricoid region (disorder)
        "707704007",  # Primary squamous cell carcinoma of pyriform sinus (disorder)
        "707705008",  # Nonkeratinizing carcinoma of the nasopharynx (disorder)
        "708054009",  # Malignant neoplasm after immunosuppressive therapy (disorder)
        "708504008",  # Periosteal osteosarcoma of jaw (disorder)
        "708921005",  # Carcinoma of central portion of breast (disorder)
        "708971008",  # Diffuse sclerosing papillary thyroid carcinoma (disorder)
        "709031009",  # Malignant neoplasm of superior wall of nasopharynx (disorder)
        "70910003",  # Indolent systemic mastocytosis (disorder)
        "709285002",  # Metastatic malignant neoplasm to lumbosacral plexus (disorder)
        "710195004",  # Malignant odontogenic neoplasm of lower jaw (disorder)
        "710196003",  # Malignant odontogenic neoplasm of upper jaw (disorder)
        "71101000119106",  # Recurrent primary malignant neoplasm of vagina (disorder)
        "71111000119109",  # Recurrent primary malignant neoplasm of vulva (disorder)
        "711414003",  # Primary clear cell adenocarcinoma of lung (disorder)
        "712525007",  # Malignant neoplasm of short bone of lower limb (disorder)
        "712750007",  # Malignant neoplasm of chest wall (disorder)
        "713038003",  # Overlapping primary malignant neoplasm of bone and articular cartilage of upper limb (disorder)
        "713189001",  # Malignant insulinoma (disorder)
        "713290004",  # Malignant ameloblastoma of mandible (disorder)
        "713293002",  # Malignant germ cell neoplasm of mediastinum (disorder)
        "713325002",  # Primary cerebral lymphoma co-occurrent with human immunodeficiency virus infection (disorder)
        "713327005",  # Malignant meningioma of meninges of brain (disorder)
        "713483007",  # Reticulosarcoma co-occurrent with human immunodeficiency virus infection (disorder)
        "713516007",  # Primary effusion lymphoma (disorder)
        "713572001",  # Malignant neoplastic disease co-occurrent with human immunodeficiency virus infection (disorder)
        "713609000",  # Invasive carcinoma of breast (disorder)
        "713646001",  # Malignant germ cell tumor of testis (disorder)
        "713718006",  # Diffuse non-Hodgkin immunoblastic lymphoma co-occurrent with human immunodeficiency virus infection (disorder)
        "713897006",  # Burkitt lymphoma co-occurrent with human immunodeficiency virus infection (disorder)
        "714251006",  # Philadelphia chromosome-negative precursor B-cell acute lymphoblastic leukemia (disorder)
        "714463003",  # Primary effusion lymphoma co-occurrent with infection caused by Human herpesvirus 8 (disorder)
        "715412008",  # Familial malignant neoplasm of prostate (disorder)
        "715414009",  # Familial malignant neoplasm of pancreas (disorder)
        "715664005",  # Interdigitating dendritic cell sarcoma (disorder)
        "715901002",  # Ependymoblastoma (disorder)
        "715903004",  # Medulloepithelioma (disorder)
        "715904005",  # Pineal parenchymal tumor of intermediate differentiation (disorder)
        "715950008",  # Anaplastic lymphoma kinase positive large B-cell lymphoma (disorder)
        "716586009",  # Carcinoma of stomach due to Epstein-Barr virus disease (disorder)
        "716588005",  # Primary non-gestational choriocarcinoma of ovary (disorder)
        "716593008",  # Carcinoma of salivary gland type of breast (disorder)
        "716647001",  # Mixed oligoastrocytoma (disorder)
        "716648006",  # Embryonal sarcoma of liver (disorder)
        "716649003",  # Extraovarian primary peritoneal carcinoma (disorder)
        "716651004",  # Leiomyosarcoma of small intestine (disorder)
        "716652006",  # Primary hepatic neuroendocrine carcinoma (disorder)
        "716653001",  # Neuroendocrine carcinoma of thymus (disorder)
        "716655008",  # Aggressive systemic mastocytosis (disorder)
        "716657000",  # Familial papillary thyroid carcinoma with renal papillary neoplasia syndrome (disorder)
        "716659002",  # Squamous cell carcinoma of head and neck (disorder)
        "716788007",  # Epstein-Barr virus positive diffuse large B-cell lymphoma of elderly (disorder)
        "716855006",  # Theca steroid producing cell malignant neoplasm of ovary (disorder)
        "716859000",  # Hereditary diffuse carcinoma of stomach (disorder)
        "717731002",  # Basal cell carcinoma of vulva (disorder)
        "717734005",  # Papillary thyroid carcinoma with renal papillary neoplasia (disorder)
        "717735006",  # Renal cell carcinoma of kidney except renal pelvis (disorder)
        "717736007",  # Familial renal cell carcinoma (disorder)
        "717916003",  # Neuroendocrine carcinoma of appendix (disorder)
        "717968005",  # Melanoma and neural system tumor syndrome (disorder)
        "718200007",  # Primary pulmonary lymphoma (disorder)
        "718220008",  # Hereditary breast and ovarian cancer syndrome (disorder)
        "718604008",  # Small cell neuroendocrine carcinoma of urinary bladder (disorder)
        "719052006",  # Recurrent squamous cell carcinoma (disorder)
        "720587009",  # Donor derived melanoma (disorder)
        "721301004",  # Myeloid and lymphoid neoplasm with fibroblast growth factor receptor 1 abnormality (disorder)
        "721306009",  # Therapy related acute myeloid leukemia and myelodysplastic syndrome (disorder)
        "721308005",  # Acute leukemia of ambiguous lineage (disorder)
        "721310007",  # Aggressive natural killer-cell leukemia (disorder)
        "721311006",  # Systemic Epstein-Barr virus positive T-cell lymphoproliferative disease of childhood (disorder)
        "721313009",  # Indeterminate dendritic cell neoplasm (disorder)
        "721314003",  # Fibroblastic reticular cell neoplasm (disorder)
        "721540005",  # Primary adnexal carcinoma of skin (disorder)
        "721541009",  # Primary malignant sarcoma of skin (disorder)
        "721546004",  # Primary adenocarcinoma of ciliary epithelium (disorder)
        "721547008",  # Primary adenocarcinoma of epithelium of iris (disorder)
        "721548003",  # Primary malignant neoplasm of lacrimal apparatus (disorder)
        "721549006",  # Primary adenocarcinoma of lacrimal apparatus (disorder)
        "721555001",  # Follicular lymphoma of small intestine (disorder)
        "721556000",  # Primary adenocarcinoma of palate (disorder)
        "721557009",  # Primary adenocarcinoma of parotid gland (disorder)
        "721558004",  # Primary adenocarcinoma of cystic duct (disorder)
        "721559007",  # Primary adenocarcinoma of common bile duct (disorder)
        "721560002",  # Primary adenocarcinoma of nasal cavity (disorder)
        "721561003",  # Primary adenocarcinoma of middle ear (disorder)
        "721562005",  # Primary adenocarcinoma of overlapping sites of retroperitoneum, peritoneum and omentum (disorder)
        "721563000",  # Primary malignant melanoma of vagina (disorder)
        "721567004",  # Primary malignant neoplasm of placenta (disorder)
        "721568009",  # Primary adenocarcinoma of parametrium (disorder)
        "721569001",  # Primary adenocarcinoma of uterine ligament (disorder)
        "721574009",  # Primary rhabdomyosarcoma of male genital organ (disorder)
        "721575005",  # Primary angiosarcoma of heart (disorder)
        "721576006",  # Primary angiosarcoma of breast (disorder)
        "721577002",  # Primary liposarcoma of soft tissue of limb (disorder)
        "721578007",  # Primary liposarcoma of male genital organ (disorder)
        "721579004",  # Primary synovial sarcoma of soft tissue of limb (disorder)
        "721580001",  # Primary myosarcoma of omentum (disorder)
        "721602000",  # Primary embryonal carcinoma of testis (disorder)
        "721603005",  # Primary choriocarcinoma of testis (disorder)
        "721604004",  # Primary squamous cell carcinoma of overlapping sites of male genital organ (disorder)
        "721605003",  # Primary squamous cell carcinoma of overlapping sites of accessory sinuses (disorder)
        "721606002",  # Primary adenocarcinoma of overlapping sites of accessory sinuses (disorder)
        "721607006",  # Primary undifferentiated carcinoma of oropharynx (disorder)
        "721617001",  # Primary adenocarcinoma of lower third of esophagus due to Barrett esophagus (disorder)
        "721618006",  # Primary squamous cell carcinoma of upper third of esophagus (disorder)
        "721619003",  # Primary squamous cell carcinoma of middle third of esophagus (disorder)
        "721620009",  # Primary squamous cell carcinoma of lower third of esophagus (disorder)
        "721621008",  # Primary squamous cell carcinoma of overlapping sites of esophagus (disorder)
        "721622001",  # Primary adenocarcinoma of upper third of esophagus (disorder)
        "721623006",  # Primary adenocarcinoma of middle third of esophagus (disorder)
        "721624000",  # Primary adenocarcinoma of overlapping sites of esophagus (disorder)
        "721625004",  # Primary neuroendocrine carcinoma of esophagus (disorder)
        "721626003",  # Primary malignant neuroendocrine neoplasm of esophagus (disorder)
        "721627007",  # Malignant melanoma of esophagus (disorder)
        "721628002",  # Primary adenocarcinoma of esophagogastric junction (disorder)
        "721629005",  # Linitis plastica of stomach (disorder)
        "721630000",  # Primary adenocarcinoma of cardia of stomach (disorder)
        "721632008",  # Primary adenocarcinoma of pyloric antrum of stomach (disorder)
        "721633003",  # Primary adenocarcinoma of overlapping sites of stomach (disorder)
        "721634009",  # Primary malignant neuroendocrine neoplasm of stomach (disorder)
        "721635005",  # Primary malignant neuroendocrine neoplasm of cardia of stomach (disorder)
        "721636006",  # Primary malignant neuroendocrine neoplasm of body of stomach (disorder)
        "721637002",  # Primary malignant neuroendocrine neoplasm of pyloric antrum of stomach (disorder)
        "721638007",  # Primary neuroendocrine carcinoma of stomach (disorder)
        "721639004",  # Primary neuroendocrine carcinoma of cardia of stomach (disorder)
        "721640002",  # Primary neuroendocrine carcinoma of body of stomach (disorder)
        "721641003",  # Primary neuroendocrine carcinoma of pyloric antrum of stomach (disorder)
        "721642005",  # Primary neuroendocrine carcinoma of overlapping sites of stomach (disorder)
        "721643000",  # Primary malignant mesenchymal neoplasm of stomach (disorder)
        "721644006",  # Primary malignant neuroendocrine neoplasm of duodenum (disorder)
        "721645007",  # Primary neuroendocrine carcinoma of duodenum (disorder)
        "721669006",  # Primary adenocarcinoma of overlapping sites of small intestine (disorder)
        "721670007",  # Primary malignant neuroendocrine neoplasm of small intestine (disorder)
        "721671006",  # Primary neuroendocrine carcinoma of small intestine (disorder)
        "721672004",  # Primary mucinous adenocarcinoma of appendix (disorder)
        "721673009",  # Primary malignant neuroendocrine neoplasm of appendix (disorder)
        "721695008",  # Primary adenocarcinoma of ascending colon and right flexure (disorder)
        "721696009",  # Primary adenocarcinoma of transverse colon (disorder)
        "721697000",  # Primary neuroendocrine carcinoma of colon (disorder)
        "721698005",  # Primary malignant neuroendocrine neoplasm of colon (disorder)
        "721699002",  # Primary adenocarcinoma of descending colon and splenic flexure (disorder)
        "721700001",  # Primary malignant neuroendocrine neoplasm of rectum (disorder)
        "721701002",  # Primary neuroendocrine carcinoma of rectum (disorder)
        "721709000",  # Primary cloacogenic carcinoma of anal canal (disorder)
        "721716004",  # Primary cholangiocarcinoma of intrahepatic biliary tract (disorder)
        "721718003",  # Primary adenocarcinoma of ampulla of Vater (disorder)
        "721725005",  # Primary adenocarcinoma of peritoneum (disorder)
        "721762007",  # Adult T-cell leukemia/lymphoma of skin (disorder)
        "722103009",  # Hormone sensitive prostate cancer (disorder)
        "722112006",  # Primary squamous cell carcinoma of ear (disorder)
        "722229001",  # Primary sarcoma of retroperitoneum (disorder)
        "722377004",  # Paraganglioma and gastric stromal sarcoma syndrome (disorder)
        "722425009",  # Reactive oxygen species 1 positive non-small cell lung cancer (disorder)
        "722444002",  # Primary giant cell sarcoma of retroperitoneum (disorder)
        "722445001",  # Primary giant cell sarcoma of peritoneum (disorder)
        "722509001",  # Primary rhabdomyosarcoma of oral cavity (disorder)
        "722510006",  # Primary rhabdomyosarcoma of pharynx (disorder)
        "722511005",  # Primary rhabdomyosarcoma of respiratory organ (disorder)
        "722512003",  # Primary rhabdomyosarcoma of intrathoracic organ (disorder)
        "722513008",  # Primary leiomyosarcoma of retroperitoneum (disorder)
        "722514002",  # Primary leiomyosarcoma of peritoneum (disorder)
        "722515001",  # Primary liposarcoma of retroperitoneum (disorder)
        "722516000",  # Primary liposarcoma of peritoneum (disorder)
        "722517009",  # Primary malignant nerve sheath neoplasm of peripheral nervous system structure (disorder)
        "722518004",  # Primary malignant nerve sheath neoplasm of autonomic nerve (disorder)
        "722519007",  # Primary sarcoma of peritoneum (disorder)
        "722524005",  # Primary invasive pleomorphic lobular carcinoma of breast (disorder)
        "722527003",  # Primary malignant neuroendocrine neoplasm of bronchus (disorder)
        "722528008",  # Primary malignant neuroendocrine neoplasm of lung (disorder)
        "722529000",  # Primary malignant epithelial neoplasm of nasopharynx (disorder)
        "722530005",  # Primary squamous cell carcinoma of pharyngeal tonsil (disorder)
        "722533007",  # Primary malignant neoplasm of esophagogastric junction (disorder)
        "722542000",  # Primary squamous cell carcinoma of anal canal (disorder)
        "722543005",  # Primary malignant melanoma of anal canal (disorder)
        "722665003",  # Primary malignant melanoma of cornea (disorder)
        "722666002",  # Primary squamous cell carcinoma of lacrimal apparatus (disorder)
        "722670005",  # Primary thymic carcinoma (disorder)
        "722671009",  # Metastatic malignant neoplasm of meninges (disorder)
        "722672002",  # Primary squamous cell carcinoma of base of tongue (disorder)
        "722673007",  # Primary squamous cell carcinoma of lingual tonsil (disorder)
        "722674001",  # Primary squamous cell carcinoma of parotid gland (disorder)
        "722676004",  # Primary squamous cell carcinoma of middle ear (disorder)
        "722677008",  # Primary mesothelioma of overlapping sites of retroperitoneum, peritoneum and omentum (disorder)
        "722678003",  # Primary squamous cell carcinoma of vagina (disorder)
        "722679006",  # Primary mucinous adenocarcinoma of endometrium (disorder)
        "722680009",  # Primary serous adenocarcinoma of endometrium (disorder)
        "722681008",  # Primary mixed adenocarcinoma of endometrium (disorder)
        "722682001",  # Primary small cell carcinoma of endometrium (disorder)
        "722683006",  # Primary neuroendocrine carcinoma of cervix uteri (disorder)
        "722684000",  # Primary low grade serous adenocarcinoma of ovary (disorder)
        "722685004",  # Primary high grade serous adenocarcinoma of ovary (disorder)
        "722686003",  # Primary serous carcinoma of uterine adnexa (disorder)
        "722687007",  # Primary mucinous carcinoma of uterine adnexa (disorder)
        "722688002",  # Malignant epithelial neoplasm (disorder)
        "722707001",  # Metastatic malignant neoplasm of peripheral nervous system (disorder)
        "722712000",  # Occupational cancer of skin (disorder)
        "722714004",  # Primary mucoepidermoid carcinoma of lacrimal apparatus (disorder)
        "722718001",  # Primary malignant meningioma (disorder)
        "722795004",  # Meningeal leukemia (disorder)
        "722827008",  # Primary synovial sarcoma of respiratory organ (disorder)
        "722828003",  # Primary synovial sarcoma of intrathoracic organ (disorder)
        "722832009",  # Primary solid papillary carcinoma with invasion of breast (disorder)
        "722954005",  # B-cell lymphoma unclassifiable with features intermediate between classical Hodgkin lymphoma and diffuse large B-cell lymphoma (disorder)
        "723076008",  # Primary myxofibrosarcoma (disorder)
        "723077004",  # Primary myosarcoma of uterus (disorder)
        "723173003",  # Carcinosarcoma of uterine adnexa (disorder)
        "723182009",  # Primary squamous cell carcinoma of nasal cavity (disorder)
        "723265000",  # Primary squamous cell carcinoma of anus (disorder)
        "723281005",  # Primary malignant melanoma of anus (disorder)
        "723301009",  # Squamous non-small cell lung cancer (disorder)
        "723843005",  # Primary chondrosarcoma of bone of limb (disorder)
        "723844004",  # Primary chondrosarcoma of articular cartilage of limb (disorder)
        "723845003",  # Primary chondrosarcoma of articular cartilage of pelvis (disorder)
        "723846002",  # Primary chondrosarcoma of bone of rib (disorder)
        "723847006",  # Primary chondrosarcoma of articular cartilage of rib (disorder)
        "723848001",  # Primary osteosarcoma of bone of jaw (disorder)
        "723849009",  # Primary osteosarcoma of articular cartilage of jaw (disorder)
        "723850009",  # Primary osteosarcoma of bone of limb (disorder)
        "723851008",  # Primary osteosarcoma of articular cartilage of limb (disorder)
        "723852001",  # Primary osteosarcoma of articular cartilage of pelvis (disorder)
        "723853006",  # Primary Ewing sarcoma of bone of limb (disorder)
        "723854000",  # Primary Ewing sarcoma of articular cartilage of limb (disorder)
        "723855004",  # Primary Ewing sarcoma of articular cartilage of pelvis (disorder)
        "723856003",  # Primary Ewing sarcoma of articular cartilage of rib (disorder)
        "723879002",  # Primary adenocarcinoma of overlapping sites of urinary organ (disorder)
        "723887001",  # Primary Ewing sarcoma of bone of rib (disorder)
        "723889003",  # B lymphoblastic leukemia lymphoma with t(9:22) (q34;q11.2); BCR-ABL 1 (disorder)
        "724056005",  # Malignant neoplasm of lower lobe of right lung (disorder)
        "724058006",  # Malignant neoplasm of upper lobe of left lung (disorder)
        "724059003",  # Malignant neoplasm of lower lobe of left lung (disorder)
        "724060008",  # Malignant neoplasm of right upper lobe of lung (disorder)
        "724467001",  # Primary squamous cell carcinoma of overlapping sites of urinary organs (disorder)
        "724468006",  # Primary urothelial carcinoma of overlapping sites of urinary organs (disorder)
        "724536000",  # Primary adenocarcinoma of ileum (disorder)
        "724537009",  # Primary adenocarcinoma of jejunum (disorder)
        "724552002",  # Primary poorly differentiated carcinoma of thyroid gland (disorder)
        "724553007",  # Primary undifferentiated carcinoma of thyroid gland (disorder)
        "724554001",  # Primary malignant epithelial neoplasm of endocrine gland (disorder)
        "724642009",  # Myeloid neoplasm associated with beta-type platelet-derived growth factor receptor gene rearrangement (disorder)
        "724644005",  # Myeloid leukemia associated with Down syndrome (disorder)
        "724645006",  # T-cell histiocyte rich large B-cell lymphoma (disorder)
        "724647003",  # Diffuse large B-cell lymphoma co-occurrent with chronic inflammation caused by Epstein-Barr virus (disorder)
        "724648008",  # Plasmablastic lymphoma (disorder)
        "724649000",  # Langerhans cell sarcoma (disorder)
        "724650000",  # Primary follicular dendritic cell sarcoma (disorder)
        "724805000",  # Primary malignant neuroepitheliomatous neoplasm of peripheral nerve (disorder)
        "724806004",  # Primary malignant neuroepitheliomatous neoplasm of autonomic nervous system (disorder)
        "724866001",  # Primary malignant neoplasm of skin due to and following radiotherapy (disorder)
        "725390002",  # Acute myeloid leukemia with t(8;16)(p11;p13) translocation (disorder)
        "725437002",  # Chronic lymphocytic leukemia genetic mutation variant (disorder)
        "726019003",  # Familial malignant melanoma of skin (disorder)
        "726721002",  # Nodal marginal zone B-cell lymphoma (disorder)
        "72891000119103",  # Kaposi sarcoma of viscus (disorder)
        "732201008",  # Carcinosarcoma of endometrium (disorder)
        "733064004",  # Osteosarcoma, limb anomalies, erythroid macrocytosis syndrome (disorder)
        "733134009",  # Primary squamous cell carcinoma of paraurethral gland (disorder)
        "733135005",  # Primary urothelial carcinoma of paraurethral gland (disorder)
        "733136006",  # Primary adenocarcinoma of urethra (disorder)
        "733144006",  # Malignant epithelial neoplasm of bronchus (disorder)
        "733162002",  # Primary malignant neuroendocrine neoplasm of anus (disorder)
        "733163007",  # Primary malignant neuroendocrine neoplasm of anal canal (disorder)
        "733343005",  # Primary squamous cell carcinoma of oral cavity (disorder)
        "733344004",  # Primary squamous cell carcinoma of lip (disorder)
        "733345003",  # Primary squamous cell carcinoma of pharynx (disorder)
        "733346002",  # Primary mucinous cystic neoplasm with associated invasive carcinoma of perihilar bile duct (disorder)
        "733347006",  # Primary mucinous cystic neoplasm with associated invasive carcinoma of cystic duct (disorder)
        "733348001",  # Primary malignant neuroendocrine neoplasm of cystic duct (disorder)
        "733349009",  # Primary mucinous cystic neoplasm with associated invasive carcinoma of distal bile duct (disorder)
        "733350009",  # Primary malignant neuroendocrine neoplasm of distal bile duct (disorder)
        "733351008",  # Primary malignant neuroendocrine neoplasm of ampulla of Vater (disorder)
        "733352001",  # Primary mucinous cystic neoplasm with associated invasive carcinoma of biliary tract (disorder)
        "733353006",  # Primary malignant neuroendocrine neoplasm of biliary tract (disorder)
        "733354000",  # Primary adenocarcinoma of biliary tract (disorder)
        "733355004",  # Primary adenocarcinoma of digestive organ (disorder)
        "733356003",  # Primary mucinous carcinoma of digestive organ (disorder)
        "733357007",  # Primary squamous cell carcinoma of intrathoracic organ (disorder)
        "733358002",  # Primary squamous cell carcinoma of respiratory system (disorder)
        "733359005",  # Primary squamous cell carcinoma of endometrium (disorder)
        "733360000",  # Primary undifferentiated carcinoma of endometrium (disorder)
        "733361001",  # Primary mucinous adenocarcinoma of ovary (disorder)
        "733362008",  # Primary adenocarcinoma of paraurethral gland (disorder)
        "733470002",  # Collecting duct carcinoma of kidney (disorder)
        "733471003",  # Chromophobe renal cell carcinoma (disorder)
        "733598001",  # Acute myeloid leukemia with t(6;9)(p23;q34) translocation (disorder)
        "733603009",  # Tubulocystic renal cell carcinoma (disorder)
        "733608000",  # Papillary renal cell carcinoma (disorder)
        "733627006",  # Primary cutaneous gamma-delta-positive T-cell lymphoma (disorder)
        "733834006",  # Invasive carcinoma of uterine cervix co-occurrent with human immunodeficiency virus infection (disorder)
        "733926004",  # Ganglioneuroblastoma of central nervous system (disorder)
        "734066005",  # Diffuse large B-cell lymphoma of central nervous system (disorder)
        "734099007",  # Neuroblastoma of central nervous system (disorder)
        "735332000",  # Primary cutaneous diffuse large cell B-cell lymphoma of lower extremity (disorder)
        "735385007",  # Microsatellite instability-high solid malignant tumor (disorder)
        "735450006",  # Primary malignant neuroepitheliomatous neoplasm of nasal cavity (disorder)
        "735678002",  # Primary chondrosarcoma of articular cartilage (disorder)
        "735679005",  # Primary chondrosarcoma of bone (disorder)
        "735680008",  # Primary osteosarcoma (disorder)
        "735735001",  # Primary malignant neuroendocrine neoplasm of pancreas (disorder)
        "735757008",  # Primary ganglioneuroblastoma of brain (disorder)
        "735916009",  # Primary malignant neuroepithelial neoplasm of retina (disorder)
        "735917000",  # Primary malignant neoplasm of ocular adnexa (disorder)
        "735918005",  # Primary malignant neoplasm of lacrimal caruncle (disorder)
        "735919002",  # Primary malignant neuroepithelial neoplasm of iris (disorder)
        "735920008",  # Primary malignant neuroepithelial neoplasm of ciliary body (disorder)
        "735921007",  # Primary malignant neuroepithelial neoplasm of orbit (disorder)
        "736322001",  # Pediatric follicular lymphoma (disorder)
        "737058005",  # Microsatellite instability-high colorectal cancer (disorder)
        "737308008",  # Primary adenocarcinoma of sublingual gland (disorder)
        "737309000",  # Primary adenocarcinoma of submandibular gland (disorder)
        "737310005",  # Primary squamous cell carcinoma of submandibular gland (disorder)
        "737311009",  # Primary squamous cell carcinoma of sublingual gland (disorder)
        "737312002",  # Primary malignant neuroendocrine neoplasm of jejunum (disorder)
        "737313007",  # Primary malignant neuroendocrine neoplasm of ileum (disorder)
        "738527001",  # Myeloid and/or lymphoid neoplasm associated with platelet derived growth factor receptor alpha rearrangement (disorder)
        "738770003",  # Anaplastic lymphoma kinase positive anaplastic large cell lymphoma (disorder)
        "7391000119103",  # Primary adenoid cystic carcinoma of nasopharynx (disorder)
        "761958009",  # Malignant peripheral nerve sheath neoplasm with perineurial differentiation (disorder)
        "762315004",  # Therapy related acute myeloid leukemia due to and following administration of antineoplastic agent (disorder)
        "762457009",  # Astroblastoma of brain (disorder)
        "762458004",  # Primary endometrioid carcinoma of endometrium (disorder)
        "762665002",  # Primary malignant neuroendocrine neoplasm of perihilar bile duct (disorder)
        "762690000",  # Classical Hodgkin lymphoma (disorder)
        "763063001",  # Adenoid basal carcinoma of cervix uteri (disorder)
        "763064007",  # Adenoid cystic carcinoma of cervix uteri (disorder)
        "763101411000119100",  # Primary renal cell carcinoma of left kidney (disorder)
        "763131005",  # Clear cell adenocarcinoma of ovary (disorder)
        "763309005",  # Acute myeloid leukemia with nucleophosmin 1 somatic mutation (disorder)
        "763408003",  # Rhabdomyosarcoma of cervix uteri (disorder)
        "763409006",  # Rhabdomyosarcoma of corpus uteri (disorder)
        "763477007",  # Primary lymphoma of conjunctiva (disorder)
        "763479005",  # Metaplastic carcinoma of breast (disorder)
        "763666008",  # Splenic marginal zone B-cell lymphoma (disorder)
        "763771009",  # Leiomyosarcoma of cervix uteri (disorder)
        "763796007",  # Megakaryoblastic acute myeloid leukemia with t(1;22)(p13;q13) (disorder)
        "763884007",  # Splenic diffuse red pulp small B-cell lymphoma (disorder)
        "764694005",  # MiT family translocation renal cell carcinoma (disorder)
        "764735002",  # Squamous cell carcinoma of small intestine (disorder)
        "764737005",  # Squamous cell carcinoma of corpus uteri (disorder)
        "764791008",  # Borderline epithelial neoplasm of ovary (disorder)
        "764845008",  # Adenocarcinoma of anal canal (disorder)
        "764846009",  # Adenocarcinoma of penis (disorder)
        "764847000",  # Adenosarcoma of cervix uteri (disorder)
        "764855007",  # Acute myeloid leukemia with CCAAT/enhancer binding protein alpha somatic mutation (disorder)
        "764938007",  # Lymphoepithelial carcinoma (disorder)
        "764940002",  # Inherited acute myeloid leukemia (disorder)
        "764951002",  # Carcinosarcoma of cervix uteri (disorder)
        "764952009",  # Carcinosarcoma of corpus uteri (disorder)
        "764961009",  # Hereditary primary clear cell renal cell carcinoma (disorder)
        "764990003",  # Renal mucinous tubular and spindle cell carcinoma (disorder)
        "765095002",  # Renal medullary carcinoma (disorder)
        "765136002",  # Primary cutaneous CD8 positive aggressive epidermotropic cytotoxic T-cell lymphoma (disorder)
        "765328000",  # Classic mycosis fungoides (disorder)
        "765740002",  # Adenosarcoma of corpus uteri (disorder)
        "765741003",  # Adenocarcinoma of gallbladder and extrahepatic biliary tract (disorder)
        "766045006",  # Acute myeloid leukemia and myelodysplastic syndrome related to alkylating agent (disorder)
        "766046007",  # Acute myeloid leukemia and myelodysplastic syndrome related to topoisomerase type 2 inhibitor (disorder)
        "766048008",  # Acute myeloid leukemia and myelodysplastic syndrome related to radiation (disorder)
        "766247009",  # Primitive neuroectodermal tumor of corpus uteri (disorder)
        "766248004",  # Primitive neuroectodermal tumor of cervix uteri (disorder)
        "766757006",  # Undifferentiated carcinoma of stomach (disorder)
        "766758001",  # Undifferentiated carcinoma of corpus uteri (disorder)
        "766759009",  # Rhabdomyosarcoma of vulva and vagina (disorder)
        "766930002",  # Glassy cell carcinoma of cervix uteri (disorder)
        "766935007",  # Primary bone lymphoma (disorder)
        "766978002",  # Squamous cell carcinoma of gallbladder and extrahepatic biliary tract (disorder)
        "766979005",  # Squamous cell carcinoma of rectum (disorder)
        "766980008",  # Squamous cell carcinoma of stomach (disorder)
        "766981007",  # Squamous cell carcinoma of colon (disorder)
        "767444009",  # Germline BRCA-mutated human epidermal growth factor receptor 2 negative metastatic carcinoma of breast (disorder)
        "767448007",  # Pineoblastoma (disorder)
        "770559003",  # Leiomyosarcoma of corpus uteri (disorder)
        "770601003",  # Small cell carcinoma of ovary (disorder)
        "770602005",  # Squamous cell carcinoma of exocrine pancreas (disorder)
        "770684008",  # Squamous cell carcinoma of liver and intrahepatic biliary tract (disorder)
        "770685009",  # Undifferentiated carcinoma of liver and intrahepatic biliary tract (disorder)
        "770686005",  # Malignant germ cell neoplasm of vagina (disorder)
        "772992009",  # Primary differentiated carcinoma of thyroid gland (disorder)
        "773283006",  # Malignant germ cell neoplasm of cervix uteri (disorder)
        "773284000",  # Malignant germ cell neoplasm of corpus uteri (disorder)
        "773624006",  # Primary ameloblastic carcinoma (disorder)
        "773774000",  # High-grade neuroendocrine carcinoma of corpus uteri (disorder)
        "773775004",  # High-grade neuroendocrine carcinoma of cervix uteri (disorder)
        "773995001",  # Primary cutaneous anaplastic large cell lymphoma (disorder)
        "778066006",  # Carcinofibroma of corpus uteri (disorder)
        "78001000146109",  # Metastatic cholangiocarcinoma (morphologic abnormality)
        "780817000",  # Undifferentiated myeloproliferative disease (disorder)
        "780821007",  # Invasive intraductal papillary-mucinous carcinoma of pancreas (disorder)
        "780844005",  # Acute myeloid leukemia with inv(3)(q21q26.2) or t(3;3)(q21;q26.2); RPN1-EVI1 (disorder)
        "782672006",  # Extragonadal germinoma (disorder)
        "782697005",  # Solid pseudopapillary carcinoma of pancreas (disorder)
        "782722002",  # Global developmental delay, lung cysts, overgrowth, Wilms tumor syndrome (disorder)
        "782827000",  # Epithelioid sarcoma (disorder)
        "783056006",  # Adenocarcinoma of paratestis (disorder)
        "783155007",  # Malignant epithelial neoplasm of salivary gland (disorder)
        "783183009",  # Salivary gland type carcinoma of esophagus (disorder)
        "783704005",  # Undifferentiated carcinoma of esophagus (disorder)
        "783706007",  # Serous cystadenocarcinoma of pancreas (disorder)
        "783736003",  # Malignant melanoma of mucous membrane (disorder)
        "783771003",  # Acinar cell carcinoma of pancreas (disorder)
        "78411000119107",  # Ewing sarcoma of bone of pelvis (disorder)
        "78461000119105",  # Metastatic squamous cell carcinoma to skin (disorder)
        "785807007",  # Transitional cell carcinoma of corpus uteri (disorder)
        "785879009",  # Mucinous cystadenocarcinoma of pancreas (disorder)
        "786038001",  # Familial nonmedullary primary thyroid carcinoma (disorder)
        "786710002",  # Solid neoplasm with neurotrophic receptor tyrosine kinase gene fusion (disorder)
        "787091002",  # Adenocarcinoma of liver and intrahepatic biliary tract (disorder)
        "788674000",  # Primary cutaneous CD4 positive small/medium T-cell lymphoproliferative disorder (disorder)
        "788979007",  # Malignant pilonidal cyst (disorder)
        "788982002",  # Mesothelioma of liver (disorder)
        "789689004",  # Malignant lymphomatoid granulomatosis (disorder)
        "789690008",  # Malignant lymphomatoid granulomatosis of lung (disorder)
        "789721005",  # Malignant middle ear paraganglioma (disorder)
        "79281000119106",  # Primary adenocarcinoma of skin (disorder)
        "792907004",  # Adenocarcinoma of pancreatic duct (disorder)
        "793197981000119106",  # Malignant carcinoid tumor of right lung (disorder)
        "808558991000119105",  # Malignant melanoma of skin of left forearm (disorder)
        "80914001",  # Anaplasia of cervix (disorder)
        "811131000124105",  # Infiltrating duct carcinoma of bilateral breasts (disorder)
        "816204007",  # Nodular melanoma of mucous membrane (disorder)
        "816205008",  # Malignant pituitary blastoma (disorder)
        "818967003",  # Medulloepithelioma of central nervous system (disorder)
        "822969007",  # Acinar cell cystadenocarcinoma of lung (disorder)
        "822970008",  # Acinar cell cystadenocarcinoma of prostate (disorder)
        "823017009",  # Infiltrating duct carcinoma of prostate (disorder)
        "82501000119102",  # Anaplastic astrocytoma of central nervous system (disorder)
        "82546001",  # Reactive immunoproliferative disease (disorder)
        "827162007",  # Malignant immature teratoma of ovary (disorder)
        "827186009",  # Metastatic malignant neoplasm to skin of hip and skin of lower leg (disorder)
        "82761000119108",  # Osteosarcoma of bone of upper limb (disorder)
        "830055006",  # Anaplastic lymphoma kinase fusion oncogene negative non-small cell lung cancer (disorder)
        "830057003",  # Relapsing classical Hodgkin lymphoma (disorder)
        "830060005",  # Reactive oxygen species 1 negative non-small cell lung cancer (disorder)
        "830150003",  # Malignant melanoma with B-Raf proto-oncogene, serine/threonine kinase V600E mutation (disorder)
        "830151004",  # Anaplastic lymphoma kinase fusion oncogene positive non-small cell lung cancer (disorder)
        "830196006",  # Chordoma of cervical spine (disorder)
        "833285007",  # Chordoma of lumbar spine (disorder)
        "833295000",  # Chordoma of thoracic spine (disorder)
        "836274002",  # Carcinomatosis of peritoneum (disorder)
        "836486002",  # Lymphomatous infiltrate of kidney (disorder)
        "838340006",  # B lymphoblastic leukemia lymphoma with t(5;14)(q31;q32); IL3-IGH (disorder)
        "838341005",  # B lymphoblastic leukemia lymphoma with t(v;11q23); MLL rearranged (disorder)
        "838342003",  # B lymphoblastic leukemia lymphoma with t(12;21) (p13;q22); TEL/AML1 (ETV6-RUNX1) (disorder)
        "838343008",  # B lymphoblastic leukemia lymphoma with t(1;19)(Q23;P13.3); E2A-PBX1 (TCF3/PBX1) (disorder)
        "838344002",  # B lymphoblastic leukemia lymphoma with hypodiploidy (disorder)
        "838346000",  # B lymphoblastic leukemia lymphoma with hyperdiploidy (disorder)
        "838349007",  # Adenocarcinoma of Meckel diverticulum (disorder)
        "838355002",  # Acute myeloid leukemia with CBFB::MYH11 fusion (disorder)
        "840423002",  # Diffuse large B-cell lymphoma of small intestine (disorder)
        "840424008",  # Diffuse large B-cell lymphoma of stomach (disorder)
        "840573001",  # Carcinoma of epididymis and spermatic cord (disorder)
        "84811000119107",  # Follicular dendritic sarcoma of intraabdominal lymph node (disorder)
        "84821000119100",  # Primary follicular dendritic cell sarcoma of spleen (disorder)
        "84831000119102",  # Follicular dendritic sarcoma of lymph nodes of multiple sites (disorder)
        "860828007",  # Gastrointestinal stromal neoplasm of overlapping sites of small intestine (disorder)
        "860830009",  # Gastrointestinal stromal neoplasm of overlapping sites of stomach (disorder)
        "860831008",  # Epithelioid hemangioendothelioma of liver (disorder)
        "860832001",  # Extraskeletal myxoid chondrosarcoma of soft tissue of limb (disorder)
        "860923004",  # Primary osteosarcoma of articular cartilage of lower limb (disorder)
        "865951006",  # Cholangiocarcinoma of perihilar bile duct (disorder)
        "865954003",  # Adenocarcinoma of breast (disorder)
        "866048009",  # Mucinous cystic neoplasm with invasive carcinoma of pancreas (disorder)
        "866050001",  # Mixed germ cell neoplasm of central nervous system (disorder)
        "866052009",  # Mixed ductal-neuroendocrine carcinoma of pancreas (disorder)
        "866068003",  # Malignant nerve sheath neoplasm of overlapping sites of peripheral nerves and autonomic nervous system (disorder)
        "866069006",  # Malignant nerve sheath neoplasm of peripheral nerve of abdomen (disorder)
        "866070007",  # Malignant mesenchymal neoplasm of bile duct (disorder)
        "866071006",  # Malignant mesenchymal neoplasm of gallbladder (disorder)
        "866072004",  # Malignant melanoma of orbit (disorder)
        "866073009",  # Malignant melanoma of overlapping sites of eye and adnexa (disorder)
        "866079008",  # Malignant melanoma of lacrimal gland (disorder)
        "866080006",  # Malignant germ cell neoplasm of heart (disorder)
        "866082003",  # Malignant melanoma of lacrimal drainage system (disorder)
        "866093001",  # Malignant nerve sheath neoplasm of peripheral nerve of upper limb (disorder)
        "866094007",  # Malignant nerve sheath neoplasm of peripheral nerve of pelvis (disorder)
        "866095008",  # Malignant nerve sheath neoplasm of peripheral nerve of thorax (disorder)
        "866096009",  # Malignant nerve sheath neoplasm of peripheral nerve of head and neck (disorder)
        "866097000",  # Malignant nerve sheath neoplasm of peripheral nerve of lower limb (disorder)
        "866098005",  # Large B-cell lymphoma arising in HHV8-associated multicentric Castleman disease (disorder)
        "866176003",  # Clear cell adenocarcinoma of peritoneum (disorder)
        "870318006",  # Osteosarcoma of bone of head (disorder)
        "870327007",  # Papillary squamous cell carcinoma of nasopharynx (disorder)
        "870355006",  # Neuroendocrine carcinoma of anus (disorder)
        "87091000119101",  # Malignant glioma of cerebrum (disorder)
        "87111000119109",  # Malignant glioma of hypothalamus (disorder)
        "87121000119102",  # Malignant glioma of cerebellum (disorder)
        "87131000119104",  # Primary extramedullary malignant tumor of spinal cord (disorder)
        "87141000119108",  # Primary malignant neoplasm of intramedullary spinal cord (disorder)
        "87151000119105",  # Malignant glioma of central nervous system (disorder)
        "878805009",  # Renal granular cell carcinoma (disorder)
        "878806005",  # Second primary cancer (disorder)
        "878807001",  # Human epidermal growth factor 2 positive gastric cancer (disorder)
        "878808006",  # Nongerminomatous germ cell tumor of central nervous system (disorder)
        "878857009",  # Malignant lymphomatoid granulomatosis grade 3 (disorder)
        "878858004",  # Malignant lymphomatoid granulomatosis grade 3 of lung (disorder)
        "890528009",  # Primary malignant neoplasm of left lung (disorder)
        "890529001",  # Primary malignant neoplasm of right lung (disorder)
        "890534002",  # Primary malignant neoplasm of bilateral lungs (disorder)
        "894171571000119107",  # Primary renal cell carcinoma of right kidney (disorder)
        "895111002",  # Chordoma of vertebral column (disorder)
        "895345000",  # Primary malignant neoplasm of bilateral kidneys (disorder)
        "895354002",  # Primary malignant neoplasm of left ureter (disorder)
        "895355001",  # Primary malignant neoplasm of right ureter (disorder)
        "895356000",  # Primary malignant neoplasm of bilateral ureters (disorder)
        "895358004",  # Primary malignant neoplasm of left testis (disorder)
        "895359007",  # Primary malignant neoplasm of right testis (disorder)
        "895361003",  # Primary malignant neoplasm of bilateral testes (disorder)
        "898197001",  # Adult T-cell leukemia/lymphoma in remission (disorder)
        "898198006",  # T-cell prolymphocytic leukemia in remission (disorder)
        "90811000119100",  # Low grade malignant glioma of brain (disorder)
        "90831000119105",  # Grade 4 malignant glioma of brain (disorder)
        "91031000119108",  # Primary chondrosarcoma of bone of upper limb (disorder)
        "91041000119104",  # Ewing sarcoma of bone structure of upper limb (disorder)
        "91061000119100",  # Primary chondrosarcoma of bone of pelvis (disorder)
        "91081000119109",  # Primary chondrosarcoma of bone of lower limb (disorder)
        "91131000119109",  # Primary adenocarcinoma of vulva (disorder)
        "91141000119100",  # Primary spindle cell carcinoma of urinary bladder (disorder)
        "91151000119103",  # Primary squamous cell carcinoma of chest wall (disorder)
        "91161000119101",  # Primary adenocarcinoma of chest wall (disorder)
        "91171000119107",  # Primary undifferentiated large cell malignant neoplasm of chest wall (disorder)
        "91181000119105",  # Squamous cell carcinoma metastatic to pleura (disorder)
        "91191000119108",  # Metastatic adenocarcinoma to pleura (disorder)
        "91211000119109",  # Metastatic small cell carcinoma to pleura (disorder)
        "91221000119102",  # Metastatic malignant melanoma to pleura (disorder)
        "91231000119104",  # Metastatic adenocarcinoma to skin (disorder)
        "91251000119105",  # Small cell carcinoma metastatic to skin (disorder)
        "91271000119101",  # Metastatic squamous cell carcinoma to bone (disorder)
        "91281000119103",  # Metastatic adenocarcinoma to bone (disorder)
        "91421000119103",  # Primary undifferentiated large cell carcinoma (disorder)
        "91854005",  # Acute leukemia in remission (disorder)
        "91855006",  # Acute leukemia, disease (disorder)
        "91856007",  # Acute lymphoid leukemia in remission (disorder)
        "91857003",  # Acute lymphoid leukemia, disease (disorder)
        "91858008",  # Acute monocytic leukemia in remission (disorder)
        "91860005",  # Acute myeloid leukemia in remission (disorder)
        "91861009",  # Acute myeloid leukemia (disorder)
        "92811003",  # Chronic leukemia in remission (disorder)
        "92812005",  # Chronic leukemia, disease (disorder)
        "92813000",  # Chronic lymphoid leukemia in remission (disorder)
        "92814006",  # Chronic lymphoid leukemia, disease (disorder)
        "92817004",  # Chronic myeloid leukemia in remission (disorder)
        "92818009",  # Chronic myeloid leukemia (disorder)
        "93142004",  # Leukemia in remission (disorder)
        "93143009",  # Leukemia, disease (disorder)
        "93144003",  # Leukemic reticuloendotheliosis of intra-abdominal lymph nodes (disorder)
        "93145002",  # Leukemic reticuloendotheliosis of intrapelvic lymph nodes (disorder)
        "93146001",  # Leukemic reticuloendotheliosis of intrathoracic lymph nodes (disorder)
        "93150008",  # Leukemic reticuloendotheliosis of lymph nodes of multiple sites (disorder)
        "93151007",  # Hairy cell leukemia of spleen (disorder)
        "93169003",  # Lymphoid leukemia in remission (disorder)
        "93182006",  # Malignant histiocytosis of intra-abdominal lymph nodes (disorder)
        "93183001",  # Malignant histiocytosis of intrapelvic lymph nodes (disorder)
        "93184007",  # Malignant histiocytosis of intrathoracic lymph nodes (disorder)
        "93188005",  # Malignant histiocytosis of lymph nodes of multiple sites (disorder)
        "93189002",  # Malignant histiocytosis of spleen (disorder)
        "93191005",  # Malignant lymphoma of intra-abdominal lymph nodes (disorder)
        "93192003",  # Malignant lymphoma of intrapelvic lymph nodes (disorder)
        "93193008",  # Malignant lymphoma of intrathoracic lymph nodes (disorder)
        "93197009",  # Malignant lymphoma of lymph nodes of multiple sites (disorder)
        "93198004",  # Malignant lymphoma of spleen (disorder)
        "93199007",  # Malignant lymphoma of extranodal AND/OR solid organ site (disorder)
        "93200005",  # Malignant mast cell tumor of intra-abdominal lymph nodes (disorder)
        "93201009",  # Malignant mast cell tumor of intrapelvic lymph nodes (disorder)
        "93202002",  # Malignant mast cell tumor of intrathoracic lymph nodes (disorder)
        "93209006",  # Malignant melanoma of perianal skin (disorder)
        "93210001",  # Malignant melanoma of skin of abdomen (disorder)
        "93211002",  # Malignant melanoma of skin of ankle (disorder)
        "93213004",  # Malignant melanoma of skin of axilla (disorder)
        "93214005",  # Malignant melanoma of skin of back (disorder)
        "93215006",  # Malignant melanoma of skin of breast (disorder)
        "93216007",  # Malignant melanoma of skin of buttock (disorder)
        "93217003",  # Malignant melanoma of skin of cheek (disorder)
        "93218008",  # Malignant melanoma of skin of chest (disorder)
        "93219000",  # Malignant melanoma of skin of chin (disorder)
        "93220006",  # Malignant melanoma of skin of ear (disorder)
        "93221005",  # Malignant melanoma of skin of elbow (disorder)
        "93222003",  # Malignant melanoma of skin of external auditory canal (disorder)
        "93223008",  # Malignant melanoma of skin of eyebrow (disorder)
        "93224002",  # Malignant melanoma of skin of eyelid (disorder)
        "93225001",  # Malignant melanoma of skin of face (disorder)
        "93226000",  # Malignant melanoma of skin of finger (disorder)
        "93227009",  # Malignant melanoma of skin of foot (disorder)
        "93228004",  # Malignant melanoma of skin of forearm (disorder)
        "93229007",  # Malignant melanoma of skin of forehead (disorder)
        "93230002",  # Malignant melanoma of skin of groin (disorder)
        "93451002",  # Erythroleukemia, FAB M6 (disorder)
        "93493001",  # Hodgkin's disease, lymphocytic-histiocytic predominance of intra-abdominal lymph nodes (disorder)
        "93494007",  # Hodgkin's disease, lymphocytic-histiocytic predominance of intrapelvic lymph nodes (disorder)
        "93495008",  # Hodgkin's disease, lymphocytic-histiocytic predominance of intrathoracic lymph nodes (disorder)
        "93500006",  # Hodgkin's disease, lymphocytic-histiocytic predominance of spleen (disorder)
        "93509007",  # Hodgkin's disease, mixed cellularity of spleen (disorder)
        "93518009",  # Hodgkin's disease, nodular sclerosis of spleen (disorder)
        "93520007",  # Hodgkin's disease of intra-abdominal lymph nodes (disorder)
        "93521006",  # Hodgkin's disease of intrapelvic lymph nodes (disorder)
        "93522004",  # Hodgkin's disease of intrathoracic lymph nodes (disorder)
        "93526001",  # Hodgkin's disease of lymph nodes of multiple sites (disorder)
        "93527005",  # Hodgkin's disease of spleen (disorder)
        "93530003",  # Hodgkin's granuloma of intrapelvic lymph nodes (disorder)
        "93531004",  # Hodgkin's granuloma of intrathoracic lymph nodes (disorder)
        "93536009",  # Hodgkin's granuloma of spleen (disorder)
        "93547002",  # Hodgkin's sarcoma of intra-abdominal lymph nodes (disorder)
        "93548007",  # Hodgkin's sarcoma of intrapelvic lymph nodes (disorder)
        "93549004",  # Hodgkin's sarcoma of intrathoracic lymph nodes (disorder)
        "93554008",  # Hodgkin's sarcoma of spleen (disorder)
        "93636004",  # Malignant melanoma of skin of hand (disorder)
        "93637008",  # Malignant melanoma of skin of hip (disorder)
        "93638003",  # Malignant melanoma of skin of knee (disorder)
        "93640008",  # Malignant melanoma of skin of lip (disorder)
        "93641007",  # Malignant melanoma of skin of lower limb (disorder)
        "93642000",  # Malignant melanoma of skin of neck (disorder)
        "93643005",  # Malignant melanoma of skin of nose (disorder)
        "93644004",  # Malignant melanoma of skin of perineum (disorder)
        "93645003",  # Malignant melanoma of skin of popliteal area (disorder)
        "93646002",  # Malignant melanoma of skin of scalp (disorder)
        "93647006",  # Malignant melanoma of skin of shoulder (disorder)
        "93648001",  # Malignant melanoma of skin of temporal region (disorder)
        "93649009",  # Malignant melanoma of skin of thigh (disorder)
        "93650009",  # Malignant melanoma of skin of toe (disorder)
        "93651008",  # Malignant melanoma of skin of trunk (disorder)
        "93652001",  # Malignant melanoma of skin of umbilicus (disorder)
        "93653006",  # Malignant melanoma of skin of upper limb (disorder)
        "93654000",  # Malignant melanoma of skin of wrist (disorder)
        "93655004",  # Malignant melanoma of skin (disorder)
        "93657007",  # Primary malignant neoplasm of abducens nerve (disorder)
        "93658002",  # Primary malignant neoplasm of accessory nerve (disorder)
        "93659005",  # Primary malignant neoplasm of accessory sinus (disorder)
        "93660000",  # Primary malignant neoplasm of vestibulocochlear nerve (disorder)
        "93661001",  # Primary malignant neoplasm of acromion (disorder)
        "93662008",  # Primary malignant neoplasm of adenoid (disorder)
        "93663003",  # Primary malignant neoplasm of skin with adnexal differentiation (disorder)
        "93665005",  # Primary malignant neoplasm of adrenal gland (disorder)
        "93667002",  # Primary malignant neoplasm of alveolar ridge mucosa (disorder)
        "93669004",  # Primary malignant neoplasm of anal canal (disorder)
        "93670003",  # Primary malignant neoplasm of anterior aspect of epiglottis (disorder)
        "93671004",  # Primary malignant neoplasm of anterior mediastinum (disorder)
        "93672006",  # Primary malignant neoplasm of anterior portion of floor of mouth (disorder)
        "93674007",  # Primary malignant neoplasm of anterior wall of nasopharynx (disorder)
        "93675008",  # Primary malignant neoplasm of anterior wall of urinary bladder (disorder)
        "93676009",  # Primary malignant neoplasm of anus (disorder)
        "93677000",  # Primary malignant neoplasm of aortic body (disorder)
        "93679002",  # Primary malignant neoplasm of appendix (disorder)
        "93682007",  # Primary malignant neoplasm of upper arm (disorder)
        "93683002",  # Primary malignant neoplasm of ascending colon (disorder)
        "93684008",  # Primary malignant neoplasm of axilla (disorder)
        "93686005",  # Primary malignant neoplasm of back (disorder)
        "93687001",  # Primary malignant neoplasm of base of tongue (disorder)
        "93689003",  # Primary malignant neoplasm of urinary bladder (disorder)
        "93690007",  # Primary malignant neoplasm of blood vessel of abdomen (disorder)
        "93692004",  # Primary malignant neoplasm of blood vessel of axilla (disorder)
        "93693009",  # Primary malignant neoplasm of blood vessel of buttock (disorder)
        "93694003",  # Primary malignant neoplasm of blood vessel of face (disorder)
        "93695002",  # Primary malignant neoplasm of blood vessel of finger (disorder)
        "93696001",  # Primary malignant neoplasm of blood vessel of foot (disorder)
        "93697005",  # Primary malignant neoplasm of blood vessel of forearm (disorder)
        "93698000",  # Primary malignant neoplasm of blood vessel of hand (disorder)
        "93699008",  # Primary malignant neoplasm of blood vessel of head (disorder)
        "93700009",  # Primary malignant neoplasm of blood vessel of hip (disorder)
        "93701008",  # Primary malignant neoplasm of blood vessel of inguinal region (disorder)
        "93703006",  # Primary malignant neoplasm of blood vessel of lower limb (disorder)
        "93704000",  # Primary malignant neoplasm of blood vessel of neck (disorder)
        "93705004",  # Primary malignant neoplasm of blood vessel of pelvis (disorder)
        "93706003",  # Primary malignant neoplasm of blood vessel of perineum (disorder)
        "93707007",  # Primary malignant neoplasm of blood vessel of popliteal space (disorder)
        "93708002",  # Primary malignant neoplasm of blood vessel of shoulder (disorder)
        "93709005",  # Primary malignant neoplasm of blood vessel of thigh (disorder)
        "93710000",  # Primary malignant neoplasm of blood vessel of thorax (disorder)
        "93711001",  # Primary malignant neoplasm of blood vessel of toe (disorder)
        "93712008",  # Primary malignant neoplasm of blood vessel of trunk (disorder)
        "93713003",  # Primary malignant neoplasm of blood vessel of upper limb (disorder)
        "93714009",  # Primary malignant neoplasm of blood vessel (disorder)
        "93715005",  # Primary malignant neoplasm of body of pancreas (disorder)
        "93716006",  # Primary malignant neoplasm of body of penis (disorder)
        "93717002",  # Primary malignant neoplasm of body of stomach (disorder)
        "93720005",  # Primary malignant neoplasm of bone marrow (disorder)
        "93721009",  # Primary malignant neoplasm of bone of face (disorder)
        "93722002",  # Primary malignant neoplasm of bone of lower limb (disorder)
        "93723007",  # Primary malignant neoplasm of bone of skull (disorder)
        "93724001",  # Primary malignant neoplasm of bone of upper limb (disorder)
        "93725000",  # Primary malignant neoplasm of bone (disorder)
        "93726004",  # Primary malignant neoplasm of brain stem (disorder)
        "93727008",  # Primary malignant neoplasm of brain (disorder)
        "93728003",  # Primary malignant neoplasm of broad ligament (disorder)
        "93729006",  # Primary malignant neoplasm of bronchus of left lower lobe (disorder)
        "93730001",  # Primary malignant neoplasm of bronchus of left upper lobe (disorder)
        "93731002",  # Primary malignant neoplasm of bronchus of right lower lobe (disorder)
        "93732009",  # Primary malignant neoplasm of bronchus of middle lobe of right lung (disorder)
        "93733004",  # Primary malignant neoplasm of bronchus of right upper lobe (disorder)
        "93734005",  # Primary malignant neoplasm of bronchus (disorder)
        "93737003",  # Primary malignant neoplasm of calcaneus (disorder)
        "93738008",  # Primary malignant neoplasm of cardia of stomach (disorder)
        "93739000",  # Primary malignant neoplasm of carina (disorder)
        "93740003",  # Primary malignant neoplasm of carotid body (disorder)
        "93741004",  # Primary malignant neoplasm of carpal bone (disorder)
        "93742006",  # Primary malignant neoplasm of cartilage of nose (disorder)
        "93743001",  # Primary malignant neoplasm of cauda equina (disorder)
        "93744007",  # Primary malignant neoplasm of central nervous system (disorder)
        "93746009",  # Primary malignant neoplasm of cerebellum (disorder)
        "93747000",  # Primary malignant neoplasm of cerebral meninges (disorder)
        "93748005",  # Primary malignant neoplasm of cerebral ventricle (disorder)
        "93749002",  # Primary malignant neoplasm of cerebrum (disorder)
        "93751003",  # Primary malignant neoplasm of cervical vertebral column (disorder)
        "93753000",  # Primary malignant neoplasm of cheek (disorder)
        "93754006",  # Primary malignant neoplasm of chest wall (disorder)
        "93755007",  # Primary malignant neoplasm of choroid (disorder)
        "93756008",  # Primary malignant neoplasm of ciliary body, primary (disorder)
        "93757004",  # Primary malignant neoplasm of clavicle (disorder)
        "93759001",  # Primary malignant neoplasm of coccygeal body (disorder)
        "93760006",  # Primary malignant neoplasm of coccyx (disorder)
        "93761005",  # Primary malignant neoplasm of colon (disorder)
        "93763008",  # Primary malignant neoplasm of common bile duct (disorder)
        "93764002",  # Primary malignant neoplasm of conjunctiva of eye (disorder)
        "93766000",  # Primary malignant neoplasm of cornea of eye (disorder)
        "93767009",  # Primary malignant neoplasm of cranial nerve (disorder)
        "93768004",  # Primary malignant neoplasm of craniopharyngeal duct (disorder)
        "93769007",  # Primary malignant neoplasm of cuboid (disorder)
        "93770008",  # Primary malignant neoplasm of cystic duct (disorder)
        "93771007",  # Primary malignant neoplasm of descending colon (disorder)
        "93772000",  # Primary malignant neoplasm of diaphragm (disorder)
        "93773005",  # Primary malignant neoplasm of dorsal surface of tongue (disorder)
        "93775003",  # Primary malignant neoplasm of duodenum (disorder)
        "93778001",  # Primary malignant neoplasm of endocardium (disorder)
        "93779009",  # Primary malignant neoplasm of endocervix (disorder)
        "93781006",  # Primary malignant neoplasm of endometrium (disorder)
        "93782004",  # Primary malignant neoplasm of epicardium (disorder)
        "93783009",  # Primary malignant neoplasm of epididymis (disorder)
        "93784003",  # Primary malignant neoplasm of epiglottis (disorder)
        "93786001",  # Primary malignant neoplasm of ethmoid bone (disorder)
        "93787005",  # Primary malignant neoplasm of ethmoidal sinus (disorder)
        "93788000",  # Primary malignant neoplasm of eustachian tube (disorder)
        "93789008",  # Primary malignant neoplasm of exocervix (disorder)
        "93792007",  # Primary malignant neoplasm of face (disorder)
        "93793002",  # Primary malignant neoplasm of facial nerve (disorder)
        "93797001",  # Primary malignant neoplasm of female genital organ (disorder)
        "93798006",  # Primary malignant neoplasm of femur (disorder)
        "93799003",  # Primary malignant neoplasm of fibula (disorder)
        "93800004",  # Primary malignant neoplasm of first cuneiform bone of foot (disorder)
        "93801000",  # Primary malignant neoplasm of flank (disorder)
        "93802007",  # Primary malignant neoplasm of floor of mouth (disorder)
        "93803002",  # Primary malignant neoplasm of foot (disorder)
        "93804008",  # Primary malignant neoplasm of forearm (disorder)
        "93806005",  # Primary malignant neoplasm of frontal bone (disorder)
        "93807001",  # Primary malignant neoplasm of frontal lobe (disorder)
        "93808006",  # Primary malignant neoplasm of frontal sinus (disorder)
        "93809003",  # Primary malignant neoplasm of fundus of stomach (disorder)
        "93812000",  # Primary malignant neoplasm of gingival mucosa (disorder)
        "93814004",  # Primary malignant neoplasm of glomus jugulare (disorder)
        "93815003",  # Primary malignant neoplasm of glossopharyngeal nerve (disorder)
        "93816002",  # Primary malignant neoplasm of glottis (disorder)
        "93817006",  # Primary malignant neoplasm of great vessels (disorder)
        "93818001",  # Primary malignant neoplasm of greater curvature of stomach (disorder)
        "93820003",  # Primary malignant neoplasm of hamate bone (disorder)
        "93821004",  # Primary malignant neoplasm of hand (disorder)
        "93824007",  # Primary malignant neoplasm of head (disorder)
        "93825008",  # Primary malignant neoplasm of heart (disorder)
        "93826009",  # Primary malignant neoplasm of hepatic flexure of colon (disorder)
        "93827000",  # Primary malignant neoplasm of hilus of lung (disorder)
        "93828005",  # Primary malignant neoplasm of hypoglossal nerve (disorder)
        "93829002",  # Primary malignant neoplasm of hypopharyngeal aspect of aryepiglottic fold (disorder)
        "93830007",  # Primary malignant neoplasm of hypopharyngeal aspect of interarytenoid fold (disorder)
        "93831006",  # Primary malignant neoplasm of hypopharynx (disorder)
        "93832004",  # Primary malignant neoplasm of ileum (disorder)
        "93833009",  # Primary malignant neoplasm of ilium (disorder)
        "93834003",  # Primary malignant neoplasm of inguinal region (disorder)
        "93835002",  # Primary malignant neoplasm of inner aspect of lip (disorder)
        "93836001",  # Primary malignant neoplasm of inner aspect of lower lip (disorder)
        "93837005",  # Primary malignant neoplasm of inner aspect of upper lip (disorder)
        "93839008",  # Primary malignant neoplasm of intra-abdominal organs (disorder)
        "93841009",  # Primary malignant neoplasm of intrathoracic organs (disorder)
        "93842002",  # Primary malignant neoplasm of ischium (disorder)
        "93843007",  # Primary malignant neoplasm of islets of Langerhans (disorder)
        "93844001",  # Primary malignant neoplasm of isthmus of uterus (disorder)
        "93845000",  # Primary malignant neoplasm of jaw (disorder)
        "93846004",  # Primary malignant neoplasm of jejunum (disorder)
        "93848003",  # Primary malignant neoplasm of junctional zone of tongue (disorder)
        "93849006",  # Primary malignant neoplasm of kidney (disorder)
        "93850006",  # Primary malignant neoplasm of labia majora (disorder)
        "93851005",  # Primary malignant neoplasm of labia minora (disorder)
        "93854002",  # Primary malignant neoplasm of large intestine (disorder)
        "93857009",  # Primary malignant neoplasm of laryngeal commissure (disorder)
        "93858004",  # Primary malignant neoplasm of laryngeal surface of epiglottis (disorder)
        "93860002",  # Primary malignant neoplasm of lateral portion of floor of mouth (disorder)
        "93861003",  # Primary malignant neoplasm of lateral wall of nasopharynx (disorder)
        "93862005",  # Primary malignant neoplasm of lateral wall of oropharynx (disorder)
        "93863000",  # Primary malignant neoplasm of lateral wall of urinary bladder (disorder)
        "93864006",  # Primary malignant neoplasm of lower lobe of left lung (disorder)
        "93865007",  # Primary malignant neoplasm of upper lobe of left lung (disorder)
        "93867004",  # Primary malignant neoplasm of lesser curvature of stomach (disorder)
        "93868009",  # Primary malignant neoplasm of lingual tonsil (disorder)
        "93870000",  # Malignant neoplasm of liver (disorder)
        "93871001",  # Primary malignant neoplasm of long bone of lower limb (disorder)
        "93872008",  # Primary malignant neoplasm of long bone of upper limb (disorder)
        "93875005",  # Primary malignant neoplasm of lower limb (disorder)
        "93878007",  # Primary malignant neoplasm of lumbar vertebral column (disorder)
        "93879004",  # Primary malignant neoplasm of lunate bone (disorder)
        "93880001",  # Primary malignant neoplasm of lung (disorder)
        "93882009",  # Primary malignant neoplasm of main bronchus (disorder)
        "93883004",  # Primary malignant neoplasm of major salivary gland (disorder)
        "93885006",  # Primary malignant neoplasm of male genital organ (disorder)
        "93886007",  # Primary malignant neoplasm of mandible (disorder)
        "93887003",  # Primary malignant neoplasm of mastoid air cells (disorder)
        "93888008",  # Primary malignant neoplasm of maxilla (disorder)
        "93889000",  # Primary malignant neoplasm of maxillary sinus (disorder)
        "93890009",  # Primary malignant neoplasm of Meckel's diverticulum (disorder)
        "93891008",  # Primary malignant neoplasm of mediastinum (disorder)
        "93892001",  # Primary malignant neoplasm of metacarpal bone (disorder)
        "93893006",  # Primary malignant neoplasm of metatarsal bone (disorder)
        "93894000",  # Primary malignant neoplasm of middle ear (disorder)
        "93898002",  # Primary malignant neoplasm of multiple endocrine glands (disorder)
        "93899005",  # Primary malignant neoplasm of muscle of abdomen (disorder)
        "93900000",  # Primary malignant neoplasm of muscle of buttock (disorder)
        "93901001",  # Primary malignant neoplasm of muscle of face (disorder)
        "93902008",  # Primary malignant neoplasm of muscle of head (disorder)
        "93903003",  # Primary malignant neoplasm of muscle of hip (disorder)
        "93904009",  # Primary malignant neoplasm of muscle of inguinal region (disorder)
        "93905005",  # Primary malignant neoplasm of muscle of lower limb (disorder)
        "93906006",  # Primary malignant neoplasm of muscle of neck (disorder)
        "93907002",  # Primary malignant neoplasm of muscle of pelvis (disorder)
        "93908007",  # Primary malignant neoplasm of muscle of perineum (disorder)
        "93909004",  # Primary malignant neoplasm of muscle of shoulder (disorder)
        "93910009",  # Primary malignant neoplasm of muscle of thorax (disorder)
        "93911008",  # Primary malignant neoplasm of muscle of trunk (disorder)
        "93912001",  # Primary malignant neoplasm of muscle of upper limb (disorder)
        "93913006",  # Primary malignant neoplasm of muscle (disorder)
        "93914000",  # Primary malignant neoplasm of myocardium (disorder)
        "93915004",  # Primary malignant neoplasm of myometrium (disorder)
        "93916003",  # Primary malignant neoplasm of nasal bone (disorder)
        "93917007",  # Primary malignant neoplasm of nasal cavity (disorder)
        "93918002",  # Primary malignant neoplasm of nasal concha (disorder)
        "93920004",  # Primary malignant neoplasm of navicular bone of foot (disorder)
        "93922007",  # Primary malignant neoplasm of neck (disorder)
        "93923002",  # Primary malignant neoplasm of nervous system (disorder)
        "93926005",  # Primary malignant neoplasm of nose (disorder)
        "93927001",  # Primary malignant neoplasm of occipital bone (disorder)
        "93928006",  # Primary malignant neoplasm of occipital lobe (disorder)
        "93929003",  # Primary malignant neoplasm of oculomotor nerve (disorder)
        "93930008",  # Primary malignant neoplasm of olfactory nerve (disorder)
        "93931007",  # Primary malignant neoplasm of optic nerve (disorder)
        "93932000",  # Primary malignant neoplasm of orbit (disorder)
        "93933005",  # Primary malignant neoplasm of oropharynx (disorder)
        "93934004",  # Primary malignant neoplasm of ovary (disorder)
        "93936002",  # Primary malignant neoplasm of palatine bone (disorder)
        "93939009",  # Primary malignant neoplasm of pancreatic duct (disorder)
        "93940006",  # Primary malignant neoplasm of para-aortic body (disorder)
        "93941005",  # Primary malignant neoplasm of paraganglion (disorder)
        "93942003",  # Primary malignant neoplasm of parametrium (disorder)
        "93943008",  # Primary malignant neoplasm of parathyroid gland (disorder)
        "93944002",  # Primary malignant neoplasm of paraurethral glands (disorder)
        "93945001",  # Primary malignant neoplasm of parietal bone (disorder)
        "93946000",  # Primary malignant neoplasm of parietal lobe (disorder)
        "93947009",  # Primary malignant neoplasm of parietal peritoneum (disorder)
        "93948004",  # Primary malignant neoplasm of parietal pleura (disorder)
        "93950007",  # Primary malignant neoplasm of patella (disorder)
        "93952004",  # Primary malignant neoplasm of pelvic peritoneum (disorder)
        "93953009",  # Primary malignant neoplasm of pelvis (disorder)
        "93955002",  # Primary malignant neoplasm of periadrenal tissue (disorder)
        "93956001",  # Primary malignant neoplasm of perianal skin (disorder)
        "93957005",  # Primary malignant neoplasm of pericardium (disorder)
        "93958000",  # Primary malignant neoplasm of perirenal tissue (disorder)
        "93959008",  # Primary malignant neoplasm of phalanx of foot (disorder)
        "939595491000119108",  # Malignant melanoma of skin of right forearm (disorder)
        "93960003",  # Primary malignant neoplasm of phalanx of hand (disorder)
        "93961004",  # Primary malignant neoplasm of pharynx (disorder)
        "93962006",  # Primary malignant neoplasm of pineal gland (disorder)
        "93963001",  # Primary malignant neoplasm of pisiform bone of hand (disorder)
        "93964007",  # Primary malignant neoplasm of pituitary gland (disorder)
        "93966009",  # Primary malignant neoplasm of pleura (disorder)
        "93967000",  # Primary malignant neoplasm of postcricoid region (disorder)
        "93968005",  # Primary malignant neoplasm of posterior hypopharyngeal wall (disorder)
        "93969002",  # Primary malignant neoplasm of posterior mediastinum (disorder)
        "93970001",  # Primary malignant neoplasm of posterior wall of nasopharynx (disorder)
        "93971002",  # Primary malignant neoplasm of posterior wall of oropharynx (disorder)
        "93972009",  # Primary malignant neoplasm of posterior wall of urinary bladder (disorder)
        "93973004",  # Primary malignant neoplasm of presacral region (disorder)
        "93974005",  # Primary malignant neoplasm of prostate (disorder)
        "93975006",  # Primary malignant neoplasm of pubis (disorder)
        "93976007",  # Primary malignant neoplasm of pyloric antrum (disorder)
        "93977003",  # Primary malignant neoplasm of pylorus (disorder)
        "93978008",  # Primary malignant neoplasm of pyriform sinus (disorder)
        "93979000",  # Primary malignant neoplasm of radius (disorder)
        "93980002",  # Primary malignant neoplasm of rectosigmoid junction (disorder)
        "93981003",  # Primary malignant neoplasm of rectouterine pouch (disorder)
        "93982005",  # Primary malignant neoplasm of rectovaginal septum (disorder)
        "93983000",  # Primary malignant neoplasm of rectovesical septum (disorder)
        "93984006",  # Primary malignant neoplasm of rectum (disorder)
        "93985007",  # Primary malignant neoplasm of renal pelvis (disorder)
        "93986008",  # Primary malignant neoplasm of respiratory tract (disorder)
        "93987004",  # Primary malignant neoplasm of retina of eye (disorder)
        "93989001",  # Primary malignant neoplasm of retromolar area (disorder)
        "93990005",  # Primary malignant neoplasm of rib (disorder)
        "93991009",  # Primary malignant neoplasm of right lower lobe of lung (disorder)
        "93992002",  # Primary malignant neoplasm of middle lobe of right lung (disorder)
        "93993007",  # Primary malignant neoplasm of upper lobe of right lung (disorder)
        "93994001",  # Primary malignant neoplasm of round ligament of uterus (disorder)
        "93995000",  # Primary malignant neoplasm of sacrococcygeal region (disorder)
        "93996004",  # Primary malignant neoplasm of sacrum (disorder)
        "93997008",  # Primary malignant neoplasm of scapula (disorder)
        "93998003",  # Primary malignant neoplasm of sclera of eye (disorder)
        "94000008",  # Primary malignant neoplasm of sebaceous gland (disorder)
        "94001007",  # Primary malignant neoplasm of second cuneiform bone of foot (disorder)
        "94002000",  # Primary malignant neoplasm of septum of nose (disorder)
        "94003005",  # Primary malignant neoplasm of short bone of lower limb (disorder)
        "94004004",  # Primary malignant neoplasm of short bone of upper limb (disorder)
        "94005003",  # Primary malignant neoplasm of shoulder (disorder)
        "94006002",  # Primary malignant neoplasm of sigmoid colon (disorder)
        "94007006",  # Primary malignant neoplasm of skin of abdomen (disorder)
        "94008001",  # Primary malignant neoplasm of skin of ankle (disorder)
        "94010004",  # Primary malignant neoplasm of skin of axilla (disorder)
        "94011000",  # Primary malignant neoplasm of skin of back (disorder)
        "94012007",  # Primary malignant neoplasm of skin of breast (disorder)
        "94013002",  # Primary malignant neoplasm of skin of buttock (disorder)
        "94014008",  # Primary malignant neoplasm of skin of cheek (disorder)
        "94015009",  # Primary malignant neoplasm of skin of chest (disorder)
        "94016005",  # Primary malignant neoplasm of skin of chin (disorder)
        "94017001",  # Primary malignant neoplasm of skin of ear (disorder)
        "94018006",  # Primary malignant neoplasm of skin of elbow (disorder)
        "94019003",  # Primary malignant neoplasm of skin of external auditory canal (disorder)
        "94020009",  # Primary malignant neoplasm of skin of eyebrow (disorder)
        "94021008",  # Primary malignant neoplasm of skin of eyelid (disorder)
        "94022001",  # Primary malignant neoplasm of skin of face (disorder)
        "94023006",  # Primary malignant neoplasm of skin of finger (disorder)
        "94024000",  # Primary malignant neoplasm of skin of foot (disorder)
        "94025004",  # Primary malignant neoplasm of skin of forearm (disorder)
        "94026003",  # Primary malignant neoplasm of skin of forehead (disorder)
        "94027007",  # Primary malignant neoplasm of skin of groin (disorder)
        "94028002",  # Primary malignant neoplasm of skin of hand (disorder)
        "94029005",  # Primary malignant neoplasm of skin of hip (disorder)
        "94030000",  # Primary malignant neoplasm of skin of knee (disorder)
        "94032008",  # Primary malignant neoplasm of skin of lip (disorder)
        "94033003",  # Primary malignant neoplasm of skin of lower limb (disorder)
        "94034009",  # Primary malignant neoplasm of skin of neck (disorder)
        "94035005",  # Primary malignant neoplasm of skin of nose (disorder)
        "94036006",  # Primary malignant neoplasm of skin of perineum (disorder)
        "94037002",  # Primary malignant neoplasm of skin of popliteal area (disorder)
        "94038007",  # Primary malignant neoplasm of skin of scalp (disorder)
        "94039004",  # Primary malignant neoplasm of skin of shoulder (disorder)
        "94040002",  # Primary malignant neoplasm of skin of temporal region (disorder)
        "94041003",  # Primary malignant neoplasm of skin of thigh (disorder)
        "94042005",  # Primary malignant neoplasm of skin of toe (disorder)
        "94043000",  # Primary malignant neoplasm of skin of trunk (disorder)
        "94044006",  # Primary malignant neoplasm of skin of umbilicus (disorder)
        "94045007",  # Primary malignant neoplasm of skin of upper limb (disorder)
        "94046008",  # Primary malignant neoplasm of skin of wrist (disorder)
        "94047004",  # Primary malignant neoplasm of skin (disorder)
        "94048009",  # Primary malignant neoplasm of small intestine (disorder)
        "94049001",  # Primary malignant neoplasm of soft palate (disorder)
        "94050001",  # Primary malignant neoplasm of soft tissues of abdomen (disorder)
        "94051002",  # Primary malignant neoplasm of soft tissues of axilla (disorder)
        "94052009",  # Primary malignant neoplasm of soft tissues of buttock (disorder)
        "94053004",  # Primary malignant neoplasm of soft tissues of face (disorder)
        "94054005",  # Primary malignant neoplasm of soft tissues of head (disorder)
        "94055006",  # Primary malignant neoplasm of soft tissues of hip (disorder)
        "94056007",  # Primary malignant neoplasm of soft tissues of inguinal region (disorder)
        "94057003",  # Primary malignant neoplasm of soft tissues of lower limb (disorder)
        "94058008",  # Primary malignant neoplasm of soft tissues of neck (disorder)
        "94059000",  # Primary malignant neoplasm of soft tissues of pelvis (disorder)
        "94060005",  # Primary malignant neoplasm of soft tissues of perineum (disorder)
        "94061009",  # Primary malignant neoplasm of soft tissues of shoulder (disorder)
        "94062002",  # Primary malignant neoplasm of soft tissues of thorax (disorder)
        "94063007",  # Primary malignant neoplasm of soft tissues of trunk (disorder)
        "94066004",  # Primary malignant neoplasm of sphenoid bone (disorder)
        "94067008",  # Primary malignant neoplasm of sphenoidal sinus (disorder)
        "94068003",  # Primary malignant neoplasm of spinal cord (disorder)
        "94069006",  # Primary malignant neoplasm of spinal meninges (disorder)
        "94071006",  # Primary malignant neoplasm of spleen (disorder)
        "94072004",  # Primary malignant neoplasm of splenic flexure of colon (disorder)
        "94073009",  # Primary malignant neoplasm of sternum (disorder)
        "94075002",  # Primary malignant neoplasm of subglottis (disorder)
        "94076001",  # Primary malignant neoplasm of sublingual gland (disorder)
        "94077005",  # Primary malignant neoplasm of submaxillary gland (disorder)
        "94078000",  # Primary malignant neoplasm of superior wall of nasopharynx (disorder)
        "94079008",  # Primary malignant neoplasm of supraclavicular region (disorder)
        "94080006",  # Primary malignant neoplasm of supraglottis (disorder)
        "94081005",  # Primary malignant neoplasm of sweat gland (disorder)
        "94082003",  # Primary malignant neoplasm of tail of pancreas (disorder)
        "94083008",  # Primary malignant neoplasm of talus (disorder)
        "94084002",  # Primary malignant neoplasm of tarsal bone (disorder)
        "94085001",  # Primary malignant neoplasm of temporal bone (disorder)
        "94086000",  # Primary malignant neoplasm of temporal lobe (disorder)
        "94087009",  # Primary malignant neoplasm of testis (disorder)
        "94089007",  # Primary malignant neoplasm of the mesocolon (disorder)
        "94090003",  # Primary malignant neoplasm of omentum (disorder)
        "94092006",  # Primary malignant neoplasm of retroperitoneum (disorder)
        "94093001",  # Primary malignant neoplasm of thigh (disorder)
        "94094007",  # Primary malignant neoplasm of third cuneiform bone of foot (disorder)
        "94096009",  # Primary malignant neoplasm of thymus (disorder)
        "94097000",  # Primary malignant neoplasm of thyroglossal duct (disorder)
        "94098005",  # Primary malignant neoplasm of thyroid gland (disorder)
        "94099002",  # Primary malignant neoplasm of tibia (disorder)
        "94101009",  # Primary malignant neoplasm of tongue (disorder)
        "94102002",  # Primary malignant neoplasm of tonsillar fossa (disorder)
        "94103007",  # Primary malignant neoplasm of tonsillar pillar (disorder)
        "94104001",  # Primary malignant neoplasm of trachea (disorder)
        "94105000",  # Primary malignant neoplasm of transverse colon (disorder)
        "94107008",  # Primary malignant neoplasm of trapezoid bone (disorder)
        "94108003",  # Primary malignant neoplasm of trigeminal nerve (disorder)
        "94109006",  # Primary malignant neoplasm of trigone of urinary bladder (disorder)
        "94110001",  # Primary malignant neoplasm of trochlear nerve (disorder)
        "94111002",  # Primary malignant neoplasm of trunk (disorder)
        "94112009",  # Primary malignant neoplasm of ulna (disorder)
        "94113004",  # Primary malignant neoplasm of undescended testis (disorder)
        "94116007",  # Primary malignant neoplasm of upper limb (disorder)
        "94118008",  # Primary malignant neoplasm of upper respiratory tract (disorder)
        "94120006",  # Primary malignant neoplasm of urachus (disorder)
        "94121005",  # Primary malignant neoplasm of ureter (disorder)
        "94122003",  # Primary malignant neoplasm of ureteric orifice of urinary bladder (disorder)
        "94123008",  # Primary malignant neoplasm of urethra (disorder)
        "94124002",  # Primary malignant neoplasm of urinary bladder neck (disorder)
        "94125001",  # Primary malignant neoplasm of urinary system (disorder)
        "94126000",  # Primary malignant neoplasm of uterine adnexa (disorder)
        "94128004",  # Primary malignant neoplasm of uveal tract of eye (disorder)
        "94129007",  # Primary malignant neoplasm of uvula (disorder)
        "94131003",  # Primary malignant neoplasm of vagus nerve (disorder)
        "94132005",  # Primary malignant neoplasm of vallecula (disorder)
        "94133000",  # Primary malignant neoplasm of vas deferens (disorder)
        "94134006",  # Primary malignant neoplasm of ventral surface of tongue (disorder)
        "94135007",  # Primary malignant neoplasm of vermilion border of lip (disorder)
        "94138009",  # Primary malignant neoplasm of vestibule of mouth (disorder)
        "94139001",  # Primary malignant neoplasm of vestibule of nose (disorder)
        "94140004",  # Primary malignant neoplasm of visceral pleura (disorder)
        "94142007",  # Primary malignant neoplasm of vomer (disorder)
        "94143002",  # Primary malignant neoplasm of vulva (disorder)
        "94144008",  # Primary malignant neoplasm of Waldeyer's ring (disorder)
        "94145009",  # Primary malignant neoplasm of zygomatic bone (disorder)
        "94148006",  # Megakaryocytic leukemia in remission (disorder)
        "94152006",  # Metastatic malignant neoplasm to abdominal esophagus (disorder)
        "94153001",  # Metastatic malignant neoplasm to abducens nerve (disorder)
        "94154007",  # Metastatic malignant neoplasm to accessory nerve (disorder)
        "94155008",  # Metastatic malignant neoplasm to accessory sinus (disorder)
        "94156009",  # Metastatic malignant neoplasm of vestibulocochlear nerve (disorder)
        "94157000",  # Metastatic malignant neoplasm to acromion (disorder)
        "94158005",  # Metastatic malignant neoplasm to adenoid (disorder)
        "94159002",  # Metastatic malignant neoplasm to adnexa of skin (disorder)
        "94160007",  # Metastatic malignant neoplasm to adrenal cortex (disorder)
        "94161006",  # Metastatic malignant neoplasm to adrenal gland (disorder)
        "94162004",  # Metastatic malignant neoplasm to adrenal medulla (disorder)
        "94163009",  # Metastatic malignant neoplasm to alveolar ridge mucosa (disorder)
        "94164003",  # Metastatic malignant neoplasm to ampulla of Vater (disorder)
        "94165002",  # Metastatic malignant neoplasm to anal canal (disorder)
        "94166001",  # Metastatic malignant neoplasm to anterior aspect of epiglottis (disorder)
        "94167005",  # Metastatic malignant neoplasm to anterior mediastinum (disorder)
        "94168000",  # Metastatic malignant neoplasm to anterior portion of floor of mouth (disorder)
        "94169008",  # Metastatic malignant neoplasm to anterior two-thirds of tongue (disorder)
        "94170009",  # Metastatic malignant neoplasm to anterior wall of nasopharynx (disorder)
        "94171008",  # Metastatic malignant neoplasm to anterior wall of urinary bladder (disorder)
        "94172001",  # Metastatic malignant neoplasm to anus (disorder)
        "94173006",  # Metastatic malignant neoplasm to aortic body (disorder)
        "94175004",  # Metastatic malignant neoplasm to appendix (disorder)
        "94179005",  # Metastatic malignant neoplasm to ascending colon (disorder)
        "94180008",  # Metastatic malignant neoplasm to axilla (disorder)
        "94181007",  # Metastatic malignant neoplasm to axillary lymph nodes (disorder)
        "94183005",  # Metastatic malignant neoplasm to back (disorder)
        "94184004",  # Metastatic malignant neoplasm to base of tongue (disorder)
        "94185003",  # Metastatic malignant neoplasm to biliary tract (disorder)
        "94186002",  # Metastatic malignant neoplasm to urinary bladder (disorder)
        "94187006",  # Metastatic malignant neoplasm to blood vessel of abdomen (disorder)
        "94189009",  # Metastatic malignant neoplasm to blood vessel of axilla (disorder)
        "94190000",  # Metastatic malignant neoplasm to blood vessel of buttock (disorder)
        "94191001",  # Metastatic malignant neoplasm to blood vessel of face (disorder)
        "94192008",  # Metastatic malignant neoplasm to blood vessel of finger (disorder)
        "94193003",  # Metastatic malignant neoplasm to blood vessel of foot (disorder)
        "94194009",  # Metastatic malignant neoplasm to blood vessel of forearm (disorder)
        "94195005",  # Metastatic malignant neoplasm to blood vessel of hand (disorder)
        "94196006",  # Metastatic malignant neoplasm to blood vessel of head (disorder)
        "94197002",  # Metastatic malignant neoplasm to blood vessel of hip (disorder)
        "94198007",  # Metastatic malignant neoplasm to blood vessel of inguinal region (disorder)
        "94200001",  # Metastatic malignant neoplasm to blood vessel of lower limb (disorder)
        "94201002",  # Metastatic malignant neoplasm to blood vessel of neck (disorder)
        "94202009",  # Metastatic malignant neoplasm to blood vessel of pelvis (disorder)
        "94203004",  # Metastatic malignant neoplasm to blood vessel of perineum (disorder)
        "94204005",  # Metastatic malignant neoplasm to blood vessel of popliteal space (disorder)
        "94205006",  # Metastatic malignant neoplasm to blood vessel of shoulder (disorder)
        "94206007",  # Metastatic malignant neoplasm to blood vessel of thigh (disorder)
        "94207003",  # Metastatic malignant neoplasm to blood vessel of thorax (disorder)
        "94208008",  # Metastatic malignant neoplasm to blood vessel of toe (disorder)
        "94209000",  # Metastatic malignant neoplasm to blood vessel of trunk (disorder)
        "94210005",  # Metastatic malignant neoplasm to blood vessel of upper limb (disorder)
        "94211009",  # Metastatic malignant neoplasm to blood vessel (disorder)
        "94212002",  # Metastatic malignant neoplasm to body of pancreas (disorder)
        "94213007",  # Metastatic malignant neoplasm to body of penis (disorder)
        "94214001",  # Metastatic malignant neoplasm to body of stomach (disorder)
        "94215000",  # Metastatic malignant neoplasm to body of uterus (disorder)
        "94217008",  # Metastatic malignant neoplasm to bone marrow (disorder)
        "94218003",  # Metastatic malignant neoplasm to bone of face (disorder)
        "94219006",  # Metastatic malignant neoplasm to bone of lower limb (disorder)
        "94220000",  # Metastatic malignant neoplasm to bone of skull (disorder)
        "94221001",  # Metastatic malignant neoplasm to bone of upper limb (disorder)
        "94222008",  # Metastatic malignant neoplasm to bone (disorder)
        "94224009",  # Metastatic malignant neoplasm to brain stem (disorder)
        "94225005",  # Metastatic malignant neoplasm to brain (disorder)
        "94226006",  # Metastatic malignant neoplasm to broad ligament (disorder)
        "94227002",  # Metastatic malignant neoplasm to bronchopulmonary lymph nodes (disorder)
        "94228007",  # Metastatic malignant neoplasm to bronchus of left lower lobe (disorder)
        "94229004",  # Metastatic malignant neoplasm to bronchus of left upper lobe (disorder)
        "94230009",  # Metastatic malignant neoplasm to bronchus of right lower lobe (disorder)
        "94231008",  # Metastatic malignant neoplasm to bronchus of middle lobe of right lung (disorder)
        "94232001",  # Metastatic malignant neoplasm to bronchus of right upper lobe (disorder)
        "94233006",  # Metastatic malignant neoplasm to bronchus (disorder)
        "94234000",  # Metastatic malignant neoplasm to buccal mucosa (disorder)
        "94235004",  # Metastatic malignant neoplasm to cecum (disorder)
        "94236003",  # Metastatic malignant neoplasm to calcaneus (disorder)
        "94237007",  # Metastatic malignant neoplasm to cardia of stomach (disorder)
        "94238002",  # Metastatic malignant neoplasm to carina (disorder)
        "94239005",  # Metastatic malignant neoplasm to carotid body (disorder)
        "94240007",  # Metastatic malignant neoplasm to carpal bone (disorder)
        "94241006",  # Metastatic malignant neoplasm to cartilage of nose (disorder)
        "94242004",  # Metastatic malignant neoplasm to cauda equina (disorder)
        "94243009",  # Metastatic malignant neoplasm to central nervous system (disorder)
        "94245002",  # Metastatic malignant neoplasm to cerebellum (disorder)
        "94246001",  # Metastatic malignant neoplasm to cerebral meninges (disorder)
        "94247005",  # Metastatic malignant neoplasm to cerebral ventricle (disorder)
        "94248000",  # Metastatic malignant neoplasm to cerebrum (disorder)
        "94249008",  # Metastatic malignant neoplasm to cervical esophagus (disorder)
        "94250008",  # Metastatic malignant neoplasm to cervical vertebral column (disorder)
        "94252000",  # Metastatic malignant neoplasm to cheek (disorder)
        "94253005",  # Metastatic malignant neoplasm to chest wall (disorder)
        "94254004",  # Metastatic malignant neoplasm to choroid (disorder)
        "94255003",  # Metastatic malignant neoplasm to ciliary body (disorder)
        "94256002",  # Metastatic malignant neoplasm to clavicle (disorder)
        "94257006",  # Metastatic malignant neoplasm to clitoris (disorder)
        "94258001",  # Metastatic malignant neoplasm to coccygeal body (disorder)
        "94259009",  # Metastatic malignant neoplasm to coccyx (disorder)
        "94260004",  # Metastatic malignant neoplasm to colon (disorder)
        "94261000",  # Metastatic malignant neoplasm to commissure of lip (disorder)
        "94262007",  # Metastatic malignant neoplasm to common bile duct (disorder)
        "94263002",  # Metastatic malignant neoplasm to conjunctiva (disorder)
        "94264008",  # Metastatic malignant neoplasm to soft tissues (disorder)
        "94265009",  # Metastatic malignant neoplasm to cornea (disorder)
        "94266005",  # Metastatic malignant neoplasm to cranial nerve (disorder)
        "94267001",  # Metastatic malignant neoplasm to craniopharyngeal duct (disorder)
        "94268006",  # Metastatic malignant neoplasm to cubital lymph nodes (disorder)
        "94269003",  # Metastatic malignant neoplasm to cuboid (disorder)
        "94270002",  # Metastatic malignant neoplasm to cystic duct (disorder)
        "94271003",  # Metastatic malignant neoplasm to descending colon (disorder)
        "94272005",  # Metastatic malignant neoplasm to diaphragm (disorder)
        "94273000",  # Metastatic malignant neoplasm to dorsal surface of tongue (disorder)
        "94274006",  # Metastatic malignant neoplasm to thoracic vertebral column (disorder)
        "94275007",  # Metastatic malignant neoplasm to duodenum (disorder)
        "94278009",  # Metastatic malignant neoplasm to endocardium (disorder)
        "94279001",  # Metastatic malignant neoplasm to endocervix (disorder)
        "94280003",  # Metastatic malignant neoplasm to endocrine gland (disorder)
        "94281004",  # Metastatic malignant neoplasm to endometrium (disorder)
        "94282006",  # Metastatic malignant neoplasm to epicardium (disorder)
        "94283001",  # Metastatic malignant neoplasm to epididymis (disorder)
        "94284007",  # Metastatic malignant neoplasm to epiglottis (disorder)
        "94285008",  # Metastatic malignant neoplasm to epitrochlear lymph nodes (disorder)
        "94286009",  # Metastatic malignant neoplasm to esophagus (disorder)
        "94287000",  # Metastatic malignant neoplasm to ethmoid bone (disorder)
        "94288005",  # Metastatic malignant neoplasm to ethmoidal sinus (disorder)
        "94289002",  # Metastatic malignant neoplasm to eustachian tube (disorder)
        "94290006",  # Metastatic malignant neoplasm to exocervix (disorder)
        "94291000119103",  # Mucinous adenocarcinoma of gastrointestinal tract (disorder)
        "94291005",  # Metastatic malignant neoplasm to extrahepatic bile ducts (disorder)
        "94292003",  # Metastatic malignant neoplasm to eye (disorder)
        "94293008",  # Metastatic malignant neoplasm to face (disorder)
        "94294002",  # Metastatic malignant neoplasm to facial nerve (disorder)
        "94295001",  # Metastatic malignant neoplasm to fallopian tube (disorder)
        "94296000",  # Metastatic malignant neoplasm to false vocal cord (disorder)
        "94298004",  # Metastatic malignant neoplasm to female genital organ (disorder)
        "94299007",  # Metastatic malignant neoplasm to femoral lymph nodes (disorder)
        "94300004",  # Metastatic malignant neoplasm to femur (disorder)
        "94301000",  # Metastatic malignant neoplasm to fibula (disorder)
        "94302007",  # Metastatic malignant neoplasm to first cuneiform bone of foot (disorder)
        "94303002",  # Metastatic malignant neoplasm to flank (disorder)
        "94304008",  # Metastatic malignant neoplasm to floor of mouth (disorder)
        "94305009",  # Metastatic malignant neoplasm to foot (disorder)
        "94306005",  # Metastatic malignant neoplasm to forearm (disorder)
        "94307001",  # Metastatic malignant neoplasm to prepuce (disorder)
        "94308006",  # Metastatic malignant neoplasm to frontal bone (disorder)
        "94309003",  # Metastatic malignant neoplasm to frontal lobe (disorder)
        "94310008",  # Metastatic malignant neoplasm to frontal sinus (disorder)
        "94311007",  # Metastatic malignant neoplasm to fundus of stomach (disorder)
        "94312000",  # Metastatic malignant neoplasm to gallbladder (disorder)
        "94313005",  # Metastatic malignant neoplasm to gastrointestinal tract (disorder)
        "94314004",  # Metastatic malignant neoplasm to gingival mucosa (disorder)
        "94315003",  # Metastatic malignant neoplasm to glans penis (disorder)
        "94316002",  # Metastatic malignant neoplasm to glomus jugulare (disorder)
        "94317006",  # Metastatic malignant neoplasm to glossopharyngeal nerve (disorder)
        "94318001",  # Metastatic malignant neoplasm to glottis (disorder)
        "94319009",  # Metastatic malignant neoplasm to great vessels (disorder)
        "94320003",  # Metastatic malignant neoplasm to greater curvature of stomach (disorder)
        "94321004",  # Metastatic malignant neoplasm to gum (disorder)
        "94322006",  # Metastatic malignant neoplasm to hamate bone (disorder)
        "94323001",  # Metastatic malignant neoplasm to hand (disorder)
        "94324007",  # Metastatic malignant neoplasm to hard palate (disorder)
        "94325008",  # Metastatic malignant neoplasm to head of pancreas (disorder)
        "94326009",  # Metastatic malignant neoplasm to head (disorder)
        "94327000",  # Metastatic malignant neoplasm to heart (disorder)
        "94328005",  # Metastatic malignant neoplasm to hepatic flexure of colon (disorder)
        "94329002",  # Metastatic malignant neoplasm to hilus of lung (disorder)
        "94330007",  # Metastatic malignant neoplasm to hypogastric lymph nodes (disorder)
        "94331006",  # Metastatic malignant neoplasm to hypoglossal nerve (disorder)
        "94332004",  # Metastatic malignant neoplasm to hypopharyngeal aspect of aryepiglottic fold (disorder)
        "94333009",  # Metastatic malignant neoplasm to hypopharyngeal aspect of interarytenoid fold (disorder)
        "94334003",  # Metastatic malignant neoplasm to hypopharynx (disorder)
        "94335002",  # Metastatic malignant neoplasm to ileum (disorder)
        "94336001",  # Metastatic malignant neoplasm to iliac lymph nodes (disorder)
        "94337005",  # Metastatic malignant neoplasm to ilium (disorder)
        "94338000",  # Metastatic malignant neoplasm to infraclavicular lymph nodes (disorder)
        "94339008",  # Metastatic malignant neoplasm to inguinal lymph nodes (disorder)
        "94340005",  # Metastatic malignant neoplasm to inguinal region (disorder)
        "94341009",  # Metastatic malignant neoplasm to inner aspect of lip (disorder)
        "94342002",  # Metastatic malignant neoplasm to inner aspect of lower lip (disorder)
        "94343007",  # Metastatic malignant neoplasm to inner aspect of upper lip (disorder)
        "94344001",  # Metastatic malignant neoplasm to intercostal lymph nodes (disorder)
        "94345000",  # Metastatic malignant neoplasm to intestinal lymph nodes (disorder)
        "94346004",  # Metastatic malignant neoplasm to intestinal tract (disorder)
        "94347008",  # Metastatic malignant neoplasm to intra-abdominal lymph nodes (disorder)
        "94348003",  # Metastatic malignant neoplasm to intra-abdominal organs (disorder)
        "94349006",  # Metastatic malignant neoplasm to intrahepatic bile ducts (disorder)
        "94350006",  # Metastatic malignant neoplasm to intrapelvic lymph nodes (disorder)
        "94351005",  # Metastatic malignant neoplasm to intrathoracic lymph nodes (disorder)
        "94352003",  # Metastatic malignant neoplasm to intrathoracic organs (disorder)
        "94353008",  # Metastatic malignant neoplasm to ischium (disorder)
        "94354002",  # Metastatic malignant neoplasm to islets of Langerhans (disorder)
        "94355001",  # Metastatic malignant neoplasm to isthmus of uterus (disorder)
        "94356000",  # Metastatic malignant neoplasm to jaw (disorder)
        "94357009",  # Metastatic malignant neoplasm to jejunum (disorder)
        "94358004",  # Metastatic malignant neoplasm to junctional region of epiglottis (disorder)
        "94359007",  # Metastatic malignant neoplasm to junctional zone of tongue (disorder)
        "94360002",  # Metastatic malignant neoplasm to kidney (disorder)
        "94361003",  # Metastatic malignant neoplasm to labia majora (disorder)
        "94362005",  # Metastatic malignant neoplasm to labia minora (disorder)
        "94364006",  # Metastatic malignant neoplasm to lacrimal gland (disorder)
        "94365007",  # Metastatic malignant neoplasm to large intestine (disorder)
        "94366008",  # Metastatic malignant neoplasm to laryngeal aspect of aryepiglottic fold (disorder)
        "94367004",  # Metastatic malignant neoplasm to laryngeal aspect of interarytenoid fold (disorder)
        "94368009",  # Metastatic malignant neoplasm to laryngeal commissure (disorder)
        "94369001",  # Metastatic malignant neoplasm to laryngeal surface of epiglottis (disorder)
        "94370000",  # Metastatic malignant neoplasm to larynx (disorder)
        "94371001",  # Metastatic malignant neoplasm to lateral portion of floor of mouth (disorder)
        "94372008",  # Metastatic malignant neoplasm to lateral wall of nasopharynx (disorder)
        "94373003",  # Metastatic malignant neoplasm to lateral wall of oropharynx (disorder)
        "94374009",  # Metastatic malignant neoplasm to lateral wall of urinary bladder (disorder)
        "94375005",  # Metastatic malignant neoplasm to left lower lobe of lung (disorder)
        "94376006",  # Metastatic malignant neoplasm to left upper lobe of lung (disorder)
        "94378007",  # Metastatic malignant neoplasm to lesser curvature of stomach (disorder)
        "94379004",  # Metastatic malignant neoplasm to lingual tonsil (disorder)
        "94380001",  # Metastatic malignant neoplasm to lip (disorder)
        "94381002",  # Metastatic malignant neoplasm to liver (disorder)
        "94382009",  # Metastatic malignant neoplasm to long bone of lower limb (disorder)
        "94383004",  # Metastatic malignant neoplasm to long bone of upper limb (disorder)
        "94384005",  # Metastatic malignant neoplasm to lower gum (disorder)
        "94386007",  # Metastatic malignant neoplasm to lower limb (disorder)
        "94388008",  # Metastatic malignant neoplasm to lower third of esophagus (disorder)
        "94389000",  # Metastatic malignant neoplasm to lumbar vertebral column (disorder)
        "94390009",  # Metastatic malignant neoplasm to lunate bone (disorder)
        "94391008",  # Metastatic malignant neoplasm to lung (disorder)
        "94392001",  # Metastatic malignant neoplasm to lymph node (disorder)
        "94393006",  # Metastatic malignant neoplasm to lymph nodes of face (disorder)
        "94394000",  # Metastatic malignant neoplasm to lymph nodes of head (disorder)
        "94395004",  # Metastatic malignant neoplasm to lymph nodes of lower limb (disorder)
        "94396003",  # Metastatic malignant neoplasm to lymph nodes of multiple sites (disorder)
        "94397007",  # Metastatic malignant neoplasm to lymph nodes of neck (disorder)
        "94398002",  # Metastatic malignant neoplasm to lymph nodes of upper limb (disorder)
        "94399005",  # Metastatic malignant neoplasm to main bronchus (disorder)
        "94400003",  # Metastatic malignant neoplasm to major salivary gland (disorder)
        "94402006",  # Metastatic malignant neoplasm to male genital organ (disorder)
        "94403001",  # Metastatic malignant neoplasm to mandible (disorder)
        "94404007",  # Metastatic malignant neoplasm to mastoid air cells (disorder)
        "94405008",  # Metastatic malignant neoplasm to maxilla (disorder)
        "94406009",  # Metastatic malignant neoplasm to maxillary sinus (disorder)
        "94407000",  # Metastatic malignant neoplasm to Meckel's diverticulum (disorder)
        "94408005",  # Metastatic malignant neoplasm to mediastinal lymph nodes (disorder)
        "94409002",  # Metastatic malignant neoplasm to mediastinum (disorder)
        "94410007",  # Metastatic malignant neoplasm to mesenteric lymph nodes (disorder)
        "94411006",  # Metastatic malignant neoplasm to metacarpal bone (disorder)
        "94412004",  # Metastatic malignant neoplasm to metatarsal bone (disorder)
        "94413009",  # Metastatic malignant neoplasm to middle ear (disorder)
        "94414003",  # Metastatic malignant neoplasm to middle third of esophagus (disorder)
        "94415002",  # Metastatic malignant neoplasm to minor salivary gland (disorder)
        "94416001",  # Metastatic malignant neoplasm to mouth (disorder)
        "94417005",  # Metastatic malignant neoplasm to multiple endocrine glands (disorder)
        "94418000",  # Metastatic malignant neoplasm to muscle of abdomen (disorder)
        "94419008",  # Metastatic malignant neoplasm to muscle of buttock (disorder)
        "94420002",  # Metastatic malignant neoplasm to muscle of face (disorder)
        "94421003",  # Metastatic malignant neoplasm to muscle of head (disorder)
        "94422005",  # Metastatic malignant neoplasm to muscle of hip (disorder)
        "94423000",  # Metastatic malignant neoplasm to muscle of inguinal region (disorder)
        "94424006",  # Metastatic malignant neoplasm to muscle of lower limb (disorder)
        "94425007",  # Metastatic malignant neoplasm to muscle of neck (disorder)
        "94426008",  # Metastatic malignant neoplasm to muscle of pelvis (disorder)
        "94427004",  # Metastatic malignant neoplasm to muscle of perineum (disorder)
        "94428009",  # Metastatic malignant neoplasm to muscle of shoulder (disorder)
        "94429001",  # Metastatic malignant neoplasm to muscle of thorax (disorder)
        "94430006",  # Metastatic malignant neoplasm to muscle of trunk (disorder)
        "94431005",  # Metastatic malignant neoplasm to muscle of upper limb (disorder)
        "94432003",  # Metastatic malignant neoplasm to muscle (disorder)
        "94433008",  # Metastatic malignant neoplasm to myocardium (disorder)
        "94434002",  # Metastatic malignant neoplasm to myometrium (disorder)
        "94435001",  # Metastatic malignant neoplasm to nasal bone (disorder)
        "94436000",  # Metastatic malignant neoplasm to nasal cavity (disorder)
        "94437009",  # Metastatic malignant neoplasm to nasal concha (disorder)
        "94439007",  # Metastatic malignant neoplasm to navicular bone of foot (disorder)
        "94440009",  # Metastatic malignant neoplasm to scaphoid bone (disorder)
        "94441008",  # Metastatic malignant neoplasm to neck (disorder)
        "94442001",  # Metastatic malignant neoplasm to nervous system (disorder)
        "94445004",  # Metastatic malignant neoplasm to nose (disorder)
        "94446003",  # Metastatic malignant neoplasm to obturator lymph nodes (disorder)
        "94447007",  # Metastatic malignant neoplasm to occipital bone (disorder)
        "94448002",  # Metastatic malignant neoplasm to occipital lobe (disorder)
        "94449005",  # Metastatic malignant neoplasm to occipital lymph nodes (disorder)
        "94450005",  # Metastatic malignant neoplasm to oculomotor nerve (disorder)
        "94451009",  # Metastatic malignant neoplasm to olfactory nerve (disorder)
        "94452002",  # Metastatic malignant neoplasm to optic nerve (disorder)
        "94453007",  # Metastatic malignant neoplasm to orbit (disorder)
        "94454001",  # Metastatic malignant neoplasm to oropharynx (disorder)
        "94455000",  # Metastatic malignant neoplasm to ovary (disorder)
        "94456004",  # Metastatic malignant neoplasm to palate (disorder)
        "94457008",  # Metastatic malignant neoplasm to palatine bone (disorder)
        "94458003",  # Metastatic malignant neoplasm to tonsil (disorder)
        "94459006",  # Metastatic malignant neoplasm to pancreas (disorder)
        "94460001",  # Metastatic malignant neoplasm to pancreatic duct (disorder)
        "94461002",  # Metastatic malignant neoplasm to para-aortic body (disorder)
        "94462009",  # Metastatic malignant neoplasm to paraganglion (disorder)
        "94463004",  # Metastatic malignant neoplasm to paramammary lymph nodes (disorder)
        "94464005",  # Metastatic malignant neoplasm to parametrial lymph nodes (disorder)
        "94465006",  # Metastatic malignant neoplasm to parametrium (disorder)
        "94466007",  # Metastatic malignant neoplasm to pararectal lymph nodes (disorder)
        "94467003",  # Metastatic malignant neoplasm to parathyroid gland (disorder)
        "94468008",  # Metastatic malignant neoplasm to paraurethral glands (disorder)
        "94469000",  # Metastatic malignant neoplasm to paravaginal lymph nodes (disorder)
        "94470004",  # Metastatic malignant neoplasm to parietal bone (disorder)
        "94471000",  # Metastatic malignant neoplasm to parietal lobe (disorder)
        "94472007",  # Metastatic malignant neoplasm to parietal peritoneum (disorder)
        "94473002",  # Metastatic malignant neoplasm to parietal pleura (disorder)
        "94474008",  # Metastatic malignant neoplasm to parotid gland (disorder)
        "94475009",  # Metastatic malignant neoplasm to parotid lymph nodes (disorder)
        "94476005",  # Metastatic malignant neoplasm to patella (disorder)
        "94477001",  # Metastatic malignant neoplasm to pectoral axillary lymph nodes (disorder)
        "94478006",  # Metastatic malignant neoplasm to pelvic bone (disorder)
        "94479003",  # Metastatic malignant neoplasm to pelvic peritoneum (disorder)
        "94480000",  # Metastatic malignant neoplasm to pelvis (disorder)
        "94481001",  # Metastatic malignant neoplasm to penis (disorder)
        "94482008",  # Metastatic malignant neoplasm to periadrenal tissue (disorder)
        "94483003",  # Metastatic malignant neoplasm to perianal skin (disorder)
        "94484009",  # Metastatic malignant neoplasm to pericardium (disorder)
        "94485005",  # Metastatic malignant neoplasm to perirenal tissue (disorder)
        "94486006",  # Metastatic malignant neoplasm to phalanx of foot (disorder)
        "94487002",  # Metastatic malignant neoplasm to phalanx of hand (disorder)
        "94488007",  # Metastatic malignant neoplasm to pharynx (disorder)
        "94489004",  # Metastatic malignant neoplasm to pineal gland (disorder)
        "94490008",  # Metastatic malignant neoplasm to pisiform bone of hand (disorder)
        "94491007",  # Metastatic malignant neoplasm to pituitary gland (disorder)
        "94492000",  # Metastatic malignant neoplasm to placenta (disorder)
        "94493005",  # Metastatic malignant neoplasm to pleura (disorder)
        "94494004",  # Metastatic malignant neoplasm to popliteal lymph nodes (disorder)
        "94495003",  # Metastatic malignant neoplasm to postcricoid region (disorder)
        "94496002",  # Metastatic malignant neoplasm to posterior hypopharyngeal wall (disorder)
        "94497006",  # Metastatic malignant neoplasm to posterior mediastinum (disorder)
        "94498001",  # Metastatic malignant neoplasm to posterior wall of nasopharynx (disorder)
        "94499009",  # Metastatic malignant neoplasm to posterior wall of oropharynx (disorder)
        "94500000",  # Metastatic malignant neoplasm to posterior wall of urinary bladder (disorder)
        "94501001",  # Metastatic malignant neoplasm to preauricular lymph nodes (disorder)
        "94502008",  # Metastatic malignant neoplasm to presacral region (disorder)
        "94503003",  # Metastatic malignant neoplasm to prostate (disorder)
        "94504009",  # Metastatic malignant neoplasm to pubis (disorder)
        "94505005",  # Metastatic malignant neoplasm to pyloric antrum (disorder)
        "94506006",  # Metastatic malignant neoplasm to pylorus (disorder)
        "94507002",  # Metastatic malignant neoplasm to pyriform sinus (disorder)
        "94508007",  # Metastatic malignant neoplasm to radius (disorder)
        "94509004",  # Metastatic malignant neoplasm to rectosigmoid junction (disorder)
        "94510009",  # Metastatic malignant neoplasm to rectouterine pouch (disorder)
        "94511008",  # Metastatic malignant neoplasm to rectovaginal septum (disorder)
        "94512001",  # Metastatic malignant neoplasm to rectovesical septum (disorder)
        "94513006",  # Metastatic malignant neoplasm to rectum (disorder)
        "94514000",  # Metastatic malignant neoplasm to renal pelvis (disorder)
        "94515004",  # Metastatic malignant neoplasm to respiratory tract (disorder)
        "94516003",  # Metastatic malignant neoplasm to retina (disorder)
        "94517007",  # Metastatic malignant neoplasm to retrocecal tissue (disorder)
        "94518002",  # Metastatic malignant neoplasm to retromolar area (disorder)
        "94519005",  # Metastatic malignant neoplasm to retroperitoneal lymph nodes (disorder)
        "94520004",  # Metastatic malignant neoplasm to retropharyngeal lymph nodes (disorder)
        "94521000",  # Metastatic malignant neoplasm to rib (disorder)
        "94522007",  # Metastatic malignant neoplasm to right lower lobe of lung (disorder)
        "94523002",  # Metastatic malignant neoplasm to middle lobe of right lung (disorder)
        "94524008",  # Metastatic malignant neoplasm to right upper lobe of lung (disorder)
        "94525009",  # Metastatic malignant neoplasm to round ligament of uterus (disorder)
        "94526005",  # Metastatic malignant neoplasm to sacrococcygeal region (disorder)
        "94527001",  # Metastatic malignant neoplasm to sacrum (disorder)
        "94528006",  # Metastatic malignant neoplasm to scalene lymph nodes (disorder)
        "94529003",  # Metastatic malignant neoplasm to scapula (disorder)
        "94530008",  # Metastatic malignant neoplasm to sclera (disorder)
        "94531007",  # Metastatic malignant neoplasm to scrotum (disorder)
        "94532000",  # Metastatic malignant neoplasm to sebaceous gland (disorder)
        "94533005",  # Metastatic malignant neoplasm to second cuneiform bone of foot (disorder)
        "94534004",  # Metastatic malignant neoplasm to septum of nose (disorder)
        "94535003",  # Metastatic malignant neoplasm to short bone of lower limb (disorder)
        "94536002",  # Metastatic malignant neoplasm to short bone of upper limb (disorder)
        "94537006",  # Metastatic malignant neoplasm to shoulder (disorder)
        "94538001",  # Metastatic malignant neoplasm to sigmoid colon (disorder)
        "94539009",  # Metastatic malignant neoplasm to skin of abdomen (disorder)
        "94540006",  # Metastatic malignant neoplasm to skin of ankle (disorder)
        "94542003",  # Metastatic malignant neoplasm to skin of axilla (disorder)
        "94543008",  # Metastatic malignant neoplasm to skin of back (disorder)
        "94544002",  # Metastatic malignant neoplasm to skin of breast (disorder)
        "94545001",  # Metastatic malignant neoplasm to skin of buttock (disorder)
        "94546000",  # Metastatic malignant neoplasm to skin of cheek (disorder)
        "94547009",  # Metastatic malignant neoplasm to skin of chest (disorder)
        "94548004",  # Metastatic malignant neoplasm to skin of chin (disorder)
        "94549007",  # Metastatic malignant neoplasm to skin of ear (disorder)
        "94550007",  # Metastatic malignant neoplasm to skin of elbow (disorder)
        "94551006",  # Metastatic malignant neoplasm to skin of external auditory canal (disorder)
        "94552004",  # Metastatic malignant neoplasm to skin of eyebrow (disorder)
        "94553009",  # Metastatic malignant neoplasm to skin of eyelid (disorder)
        "94554003",  # Metastatic malignant neoplasm to skin of face (disorder)
        "94555002",  # Metastatic malignant neoplasm to skin of finger (disorder)
        "94556001",  # Metastatic malignant neoplasm to skin of foot (disorder)
        "94557005",  # Metastatic malignant neoplasm to skin of forearm (disorder)
        "94558000",  # Metastatic malignant neoplasm to skin of forehead (disorder)
        "94559008",  # Metastatic malignant neoplasm to skin of groin (disorder)
        "94560003",  # Metastatic malignant neoplasm to skin of hand (disorder)
        "94561004",  # Metastatic malignant neoplasm to skin of hip (disorder)
        "94562006",  # Metastatic malignant neoplasm to skin of knee (disorder)
        "94564007",  # Metastatic malignant neoplasm to skin of lip (disorder)
        "94565008",  # Metastatic malignant neoplasm to skin of lower limb (disorder)
        "94566009",  # Metastatic malignant neoplasm to skin of neck (disorder)
        "94567000",  # Metastatic malignant neoplasm to skin of nose (disorder)
        "94568005",  # Metastatic malignant neoplasm to skin of perineum (disorder)
        "94569002",  # Metastatic malignant neoplasm to skin of popliteal area (disorder)
        "94570001",  # Metastatic malignant neoplasm to skin of scalp (disorder)
        "94571002",  # Metastatic malignant neoplasm to skin of shoulder (disorder)
        "94572009",  # Metastatic malignant neoplasm to skin of temporal region (disorder)
        "94573004",  # Metastatic malignant neoplasm to skin of thigh (disorder)
        "94574005",  # Metastatic malignant neoplasm to skin of toe (disorder)
        "94575006",  # Metastatic malignant neoplasm to skin of trunk (disorder)
        "94576007",  # Metastatic malignant neoplasm to skin of umbilicus (disorder)
        "94577003",  # Metastatic malignant neoplasm to skin of upper limb (disorder)
        "94578008",  # Metastatic malignant neoplasm to skin of wrist (disorder)
        "94579000",  # Metastatic malignant neoplasm to skin (disorder)
        "94580002",  # Metastatic malignant neoplasm to small intestine (disorder)
        "94581003",  # Metastatic malignant neoplasm to soft palate (disorder)
        "94582005",  # Metastatic malignant neoplasm to soft tissues of abdomen (disorder)
        "94583000",  # Metastatic malignant neoplasm to soft tissues of axilla (disorder)
        "94584006",  # Metastatic malignant neoplasm to soft tissues of buttock (disorder)
        "94585007",  # Metastatic malignant neoplasm to soft tissues of face (disorder)
        "94586008",  # Metastatic malignant neoplasm to soft tissues of head (disorder)
        "94587004",  # Metastatic malignant neoplasm to soft tissues of hip (disorder)
        "94588009",  # Metastatic malignant neoplasm to soft tissues of inguinal region (disorder)
        "94589001",  # Metastatic malignant neoplasm to soft tissues of lower limb (disorder)
        "94590005",  # Metastatic malignant neoplasm to soft tissues of neck (disorder)
        "94591009",  # Metastatic malignant neoplasm to soft tissues of pelvis (disorder)
        "94592002",  # Metastatic malignant neoplasm to soft tissues of perineum (disorder)
        "94593007",  # Metastatic malignant neoplasm to soft tissues of shoulder (disorder)
        "94594001",  # Metastatic malignant neoplasm to soft tissues of thorax (disorder)
        "94595000",  # Metastatic malignant neoplasm to soft tissues of trunk (disorder)
        "94596004",  # Metastatic malignant neoplasm to soft tissues of upper limb (disorder)
        "94597008",  # Metastatic malignant neoplasm to spermatic cord (disorder)
        "94598003",  # Metastatic malignant neoplasm to sphenoid bone (disorder)
        "94599006",  # Metastatic malignant neoplasm to sphenoidal sinus (disorder)
        "94600009",  # Metastatic malignant neoplasm to spinal cord (disorder)
        "94601008",  # Metastatic malignant neoplasm to spinal meninges (disorder)
        "94602001",  # Metastatic malignant neoplasm to vertebral column (disorder)
        "94603006",  # Metastatic malignant neoplasm to spleen (disorder)
        "94604000",  # Metastatic malignant neoplasm to splenic flexure of colon (disorder)
        "94605004",  # Metastatic malignant neoplasm to sternum (disorder)
        "94606003",  # Metastatic malignant neoplasm to stomach (disorder)
        "94607007",  # Metastatic malignant neoplasm to subglottis (disorder)
        "94608002",  # Metastatic malignant neoplasm to sublingual gland (disorder)
        "94609005",  # Metastatic malignant neoplasm to submandibular lymph nodes (disorder)
        "94610000",  # Metastatic malignant neoplasm to submaxillary gland (disorder)
        "94611001",  # Metastatic malignant neoplasm to submental lymph nodes (disorder)
        "94612008",  # Metastatic malignant neoplasm to superficial inguinal lymph nodes (disorder)
        "94613003",  # Metastatic malignant neoplasm to superior wall of nasopharynx (disorder)
        "94614009",  # Metastatic malignant neoplasm to supraclavicular lymph nodes (disorder)
        "94615005",  # Metastatic malignant neoplasm to supraclavicular region (disorder)
        "94616006",  # Metastatic malignant neoplasm to supraglottis (disorder)
        "94617002",  # Metastatic malignant neoplasm to sweat gland (disorder)
        "94618007",  # Metastatic malignant neoplasm to tail of pancreas (disorder)
        "94619004",  # Metastatic malignant neoplasm to talus (disorder)
        "94620005",  # Metastatic malignant neoplasm to tarsal bone (disorder)
        "94621009",  # Metastatic malignant neoplasm to temporal bone (disorder)
        "94622002",  # Metastatic malignant neoplasm to temporal lobe (disorder)
        "94623007",  # Metastatic malignant neoplasm to testis (disorder)
        "94624001",  # Metastatic malignant neoplasm to the mesentery (disorder)
        "94625000",  # Metastatic malignant neoplasm to the mesocolon (disorder)
        "94626004",  # Metastatic malignant neoplasm to omentum (disorder)
        "94627008",  # Metastatic malignant neoplasm to peritoneum (disorder)
        "94628003",  # Metastatic malignant neoplasm to retroperitoneum (disorder)
        "94629006",  # Metastatic malignant neoplasm to thigh (disorder)
        "94630001",  # Metastatic malignant neoplasm to third cuneiform bone of foot (disorder)
        "94631002",  # Metastatic malignant neoplasm to thoracic esophagus (disorder)
        "94632009",  # Metastatic malignant neoplasm to thymus (disorder)
        "94633004",  # Metastatic malignant neoplasm to thyroglossal duct (disorder)
        "94634005",  # Metastatic malignant neoplasm to thyroid gland (disorder)
        "94635006",  # Metastatic malignant neoplasm to tibia (disorder)
        "94636007",  # Metastatic malignant neoplasm to tibial lymph nodes (disorder)
        "94637003",  # Metastatic malignant neoplasm to tip and lateral border of tongue (disorder)
        "94638008",  # Metastatic malignant neoplasm to tongue (disorder)
        "94639000",  # Metastatic malignant neoplasm to tonsillar fossa (disorder)
        "94640003",  # Metastatic malignant neoplasm to tonsillar pillar (disorder)
        "94641004",  # Metastatic malignant neoplasm to trachea (disorder)
        "94642006",  # Metastatic malignant neoplasm to tracheobronchial lymph nodes (disorder)
        "94643001",  # Metastatic malignant neoplasm to transverse colon (disorder)
        "94644007",  # Metastatic malignant neoplasm to trapezium (disorder)
        "94645008",  # Metastatic malignant neoplasm to trapezoid bone (disorder)
        "94646009",  # Metastatic malignant neoplasm to trigeminal nerve (disorder)
        "94647000",  # Metastatic malignant neoplasm to trigone of urinary bladder (disorder)
        "94648005",  # Metastatic malignant neoplasm to trochlear nerve (disorder)
        "94649002",  # Metastatic malignant neoplasm to trunk (disorder)
        "94650002",  # Metastatic malignant neoplasm to ulna (disorder)
        "94651003",  # Metastatic malignant neoplasm to undescended testis (disorder)
        "94652005",  # Metastatic malignant neoplasm to upper gum (disorder)
        "94654006",  # Metastatic malignant neoplasm to upper limb (disorder)
        "94656008",  # Metastatic malignant neoplasm to upper respiratory tract (disorder)
        "94657004",  # Metastatic malignant neoplasm to upper third of esophagus (disorder)
        "94658009",  # Metastatic malignant neoplasm to urachus (disorder)
        "94659001",  # Metastatic malignant neoplasm to ureter (disorder)
        "94660006",  # Metastatic malignant neoplasm to ureteric orifice of urinary bladder (disorder)
        "94661005",  # Metastatic malignant neoplasm to urethra (disorder)
        "94662003",  # Metastatic malignant neoplasm to urinary bladder neck (disorder)
        "94663008",  # Metastatic malignant neoplasm to urinary system (disorder)
        "94664002",  # Metastatic malignant neoplasm to uterine adnexa (disorder)
        "94665001",  # Metastatic malignant neoplasm to uterus (disorder)
        "94666000",  # Metastatic malignant neoplasm to uveal tract (disorder)
        "94667009",  # Metastatic malignant neoplasm to uvula (disorder)
        "94668004",  # Metastatic malignant neoplasm to vagina (disorder)
        "94669007",  # Metastatic malignant neoplasm to vagus nerve (disorder)
        "94670008",  # Metastatic malignant neoplasm to vallecula (disorder)
        "94671007",  # Metastatic malignant neoplasm to vas deferens (disorder)
        "94672000",  # Metastatic malignant neoplasm to ventral surface of tongue (disorder)
        "94673005",  # Metastatic malignant neoplasm to vermilion border of lip (disorder)
        "94674004",  # Metastatic malignant neoplasm to vermilion border of lower lip (disorder)
        "94675003",  # Metastatic malignant neoplasm to vermilion border of upper lip (disorder)
        "94676002",  # Metastatic malignant neoplasm to vestibule of mouth (disorder)
        "94677006",  # Metastatic malignant neoplasm to vestibule of nose (disorder)
        "94678001",  # Metastatic malignant neoplasm to visceral pleura (disorder)
        "94679009",  # Metastatic malignant neoplasm to vocal cord (disorder)
        "94680007",  # Metastatic malignant neoplasm to vomer (disorder)
        "94681006",  # Metastatic malignant neoplasm to vulva (disorder)
        "94682004",  # Metastatic malignant neoplasm to Waldeyer's ring (disorder)
        "94683009",  # Metastatic malignant neoplasm to zygomatic bone (disorder)
        "94686001",  # Mixed cell type lymphosarcoma of intra-abdominal lymph nodes (disorder)
        "94687005",  # Mixed cell type lymphosarcoma of intrapelvic lymph nodes (disorder)
        "94688000",  # Mixed cell type lymphosarcoma of intrathoracic lymph nodes (disorder)
        "94690004",  # Mixed cell type lymphosarcoma of lymph nodes of head, face, and neck (disorder)
        "94704006",  # Multiple myeloma in remission (disorder)
        "94707004",  # Mycosis fungoides of intra-abdominal lymph nodes (disorder)
        "94708009",  # Mycosis fungoides of intrapelvic lymph nodes (disorder)
        "94709001",  # Mycosis fungoides of intrathoracic lymph nodes (disorder)
        "94714002",  # Mycosis fungoides of spleen (disorder)
        "94716000",  # Myeloid leukemia in remission (disorder)
        "94718004",  # Myeloid sarcoma in remission (disorder)
        "94719007",  # Myeloid sarcoma, disease (disorder)
        "94721002",  # Neoplasm of uncertain behavior of abdominal esophagus (disorder)
        "94722009",  # Neoplasm of uncertain behavior of abducens nerve (disorder)
        "94723004",  # Neoplasm of uncertain behavior of accessory nerve (disorder)
        "94724005",  # Neoplasm of uncertain behavior of accessory sinus (disorder)
        "94725006",  # Neoplasm of uncertain behavior of vestibulocochlear nerve (disorder)
        "94726007",  # Neoplasm of uncertain behavior of acromion (disorder)
        "94727003",  # Neoplasm of uncertain behavior of adenoid (disorder)
        "94728008",  # Neoplasm of uncertain behavior of adnexa of skin (disorder)
        "94729000",  # Neoplasm of uncertain behavior of adrenal cortex (disorder)
        "94730005",  # Neoplasm of uncertain behavior of adrenal gland (disorder)
        "94731009",  # Neoplasm of uncertain behavior of adrenal medulla (disorder)
        "94732002",  # Neoplasm of uncertain behavior of alveolar ridge mucosa (disorder)
        "94733007",  # Neoplasm of uncertain behavior of ampulla of Vater (disorder)
        "94734001",  # Neoplasm of uncertain behavior of anal canal (disorder)
        "94735000",  # Neoplasm of uncertain behavior of anterior aspect of epiglottis (disorder)
        "94736004",  # Neoplasm of uncertain behavior of anterior mediastinum (disorder)
        "94737008",  # Neoplasm of uncertain behavior of anterior portion of floor of mouth (disorder)
        "94738003",  # Neoplasm of uncertain behavior of anterior two-thirds of tongue (disorder)
        "94739006",  # Neoplasm of uncertain behavior of anterior wall of nasopharynx (disorder)
        "94740008",  # Neoplasm of uncertain behavior of anterior wall of urinary bladder (disorder)
        "94741007",  # Neoplasm of uncertain behavior of anus (disorder)
        "94742000",  # Neoplasm of uncertain behavior of aortic body (disorder)
        "94743005",  # Neoplasm of uncertain behavior of apex of urinary bladder (disorder)
        "94744004",  # Neoplasm of uncertain behavior of appendix (disorder)
        "94747006",  # Neoplasm of uncertain behavior of upper arm (disorder)
        "94748001",  # Neoplasm of uncertain behavior of ascending colon (disorder)
        "94749009",  # Neoplasm of uncertain behavior of axilla (disorder)
        "94751008",  # Neoplasm of uncertain behavior of back (disorder)
        "94752001",  # Neoplasm of uncertain behavior of base of tongue (disorder)
        "94754000",  # Neoplasm of uncertain behavior of urinary bladder (disorder)
        "94755004",  # Neoplasm of uncertain behavior of blood vessel (disorder)
        "94756003",  # Neoplasm of uncertain behavior of body of pancreas (disorder)
        "94757007",  # Neoplasm of uncertain behavior of body of penis (disorder)
        "94758002",  # Neoplasm of uncertain behavior of body of stomach (disorder)
        "94759005",  # Neoplasm of uncertain behavior of body of uterus (disorder)
        "94761001",  # Neoplasm of uncertain behavior of bone of face (disorder)
        "94762008",  # Neoplasm of uncertain behavior of bone of lower limb (disorder)
        "94763003",  # Neoplasm of uncertain behavior of bone of skull (disorder)
        "94764009",  # Neoplasm of uncertain behavior of bone of upper limb (disorder)
        "94765005",  # Neoplasm of uncertain behavior of bone (disorder)
        "94766006",  # Neoplasm of uncertain behavior of brain stem (disorder)
        "94767002",  # Neoplasm of uncertain behavior of brain (disorder)
        "94768007",  # Neoplasm of uncertain behavior of broad ligament of uterus (disorder)
        "94769004",  # Neoplasm of uncertain behavior of bronchus of left lower lobe (disorder)
        "94770003",  # Neoplasm of uncertain behavior of bronchus of left upper lobe (disorder)
        "94771004",  # Neoplasm of uncertain behavior of bronchus of right lower lobe (disorder)
        "94772006",  # Neoplasm of uncertain behavior of bronchus of middle lobe of right lung (disorder)
        "94773001",  # Neoplasm of uncertain behavior of bronchus of right upper lobe (disorder)
        "94774007",  # Neoplasm of uncertain behavior of bronchus (disorder)
        "94775008",  # Neoplasm of uncertain behavior of buccal mucosa (disorder)
        "94776009",  # Neoplasm of uncertain behavior of calcaneus (disorder)
        "94777000",  # Neoplasm of uncertain behavior of cardia of stomach (disorder)
        "94778005",  # Neoplasm of uncertain behavior of carina of trachea (disorder)
        "94779002",  # Neoplasm of uncertain behavior of carotid body (disorder)
        "94780004",  # Neoplasm of uncertain behavior of carpal bone (disorder)
        "94781000",  # Neoplasm of uncertain behavior of cartilage of nose (disorder)
        "94782007",  # Neoplasm of uncertain behavior of cauda equina (disorder)
        "94783002",  # Neoplasm of uncertain behavior of cecum (disorder)
        "94784008",  # Neoplasm of uncertain behavior of central nervous system (disorder)
        "94786005",  # Neoplasm of uncertain behavior of cerebellum (disorder)
        "94787001",  # Neoplasm of uncertain behavior of cerebral meninges (disorder)
        "94788006",  # Neoplasm of uncertain behavior of cerebral ventricle (disorder)
        "94789003",  # Neoplasm of uncertain behavior of cerebrum (disorder)
        "94790007",  # Neoplasm of uncertain behavior of cervical esophagus (disorder)
        "94791006",  # Neoplasm of uncertain behavior of cervical vertebral column (disorder)
        "94792004",  # Neoplasm of uncertain behavior of uterine cervix (disorder)
        "94793009",  # Neoplasm of uncertain behavior of cheek (disorder)
        "94794003",  # Neoplasm of uncertain behavior of chest wall (disorder)
        "94795002",  # Neoplasm of uncertain behavior of choroid (disorder)
        "94796001",  # Neoplasm of uncertain behavior of ciliary body (disorder)
        "94797005",  # Neoplasm of uncertain behavior of clavicle (disorder)
        "94798000",  # Neoplasm of uncertain behavior of clitoris (disorder)
        "94799008",  # Neoplasm of uncertain behavior of coccygeal body (disorder)
        "94800007",  # Neoplasm of uncertain behavior of coccyx (disorder)
        "94801006",  # Neoplasm of uncertain behavior of colon (disorder)
        "94802004",  # Neoplasm of uncertain behavior of commissure of lip (disorder)
        "94803009",  # Neoplasm of uncertain behavior of common bile duct (disorder)
        "94804003",  # Neoplasm of uncertain behavior of conjunctiva (disorder)
        "94805002",  # Neoplasm of uncertain behavior of soft tissues (disorder)
        "94806001",  # Neoplasm of uncertain behavior of cornea (disorder)
        "94807005",  # Neoplasm of uncertain behavior of cranial nerve (disorder)
        "94808000",  # Neoplasm of uncertain behavior of craniopharyngeal duct (disorder)
        "94809008",  # Neoplasm of uncertain behavior of cuboid (disorder)
        "94810003",  # Neoplasm of uncertain behavior of cystic duct (disorder)
        "94811004",  # Neoplasm of uncertain behavior of descending colon (disorder)
        "94812006",  # Neoplasm of uncertain behavior of diaphragm (disorder)
        "94813001",  # Neoplasm of uncertain behavior of dorsal surface of tongue (disorder)
        "94814007",  # Neoplasm of uncertain behavior of thoracic vertebral column (disorder)
        "94815008",  # Neoplasm of uncertain behavior of duodenum (disorder)
        "94818005",  # Neoplasm of uncertain behavior of endocardium (disorder)
        "94819002",  # Neoplasm of uncertain behavior of endocervix (disorder)
        "94820008",  # Neoplasm of uncertain behavior of endocrine gland (disorder)
        "94821007",  # Neoplasm of uncertain behavior of endometrium (disorder)
        "94822000",  # Neoplasm of uncertain behavior of epicardium (disorder)
        "94823005",  # Neoplasm of uncertain behavior of epididymis (disorder)
        "94824004",  # Neoplasm of uncertain behavior of epiglottis (disorder)
        "94825003",  # Neoplasm of uncertain behavior of esophagus (disorder)
        "94826002",  # Neoplasm of uncertain behavior of ethmoid bone (disorder)
        "94827006",  # Neoplasm of uncertain behavior of ethmoidal sinus (disorder)
        "94828001",  # Neoplasm of uncertain behavior of eustachian tube (disorder)
        "94829009",  # Neoplasm of uncertain behavior of exocervix (disorder)
        "94830004",  # Neoplasm of uncertain behavior of extrahepatic bile ducts (disorder)
        "94831000",  # Neoplasm of uncertain behavior of eye (disorder)
        "94832007",  # Neoplasm of uncertain behavior of face (disorder)
        "94833002",  # Neoplasm of uncertain behavior of facial nerve (disorder)
        "94834008",  # Neoplasm of uncertain behavior of fallopian tube (disorder)
        "94835009",  # Neoplasm of uncertain behavior of false vocal cord (disorder)
        "94837001",  # Neoplasm of uncertain behavior of female genital organ (disorder)
        "94838006",  # Neoplasm of uncertain behavior of femur (disorder)
        "94839003",  # Neoplasm of uncertain behavior of fibula (disorder)
        "94840001",  # Neoplasm of uncertain behavior of first cuneiform bone of foot (disorder)
        "94841002",  # Neoplasm of uncertain behavior of flank (disorder)
        "94842009",  # Neoplasm of uncertain behavior of floor of mouth (disorder)
        "94843004",  # Neoplasm of uncertain behavior of foot (disorder)
        "94844005",  # Neoplasm of uncertain behavior of forearm (disorder)
        "94845006",  # Neoplasm of uncertain behavior of prepuce (disorder)
        "94846007",  # Neoplasm of uncertain behavior of frontal bone (disorder)
        "94847003",  # Neoplasm of uncertain behavior of frontal lobe (disorder)
        "94848008",  # Neoplasm of uncertain behavior of frontal sinus (disorder)
        "94849000",  # Neoplasm of uncertain behavior of fundus of stomach (disorder)
        "94850000",  # Neoplasm of uncertain behavior of gallbladder (disorder)
        "94851001",  # Neoplasm of uncertain behavior of gastrointestinal tract (disorder)
        "94852008",  # Neoplasm of uncertain behavior of gingival mucosa (disorder)
        "94853003",  # Neoplasm of uncertain behavior of glans penis (disorder)
        "94854009",  # Neoplasm of uncertain behavior of glomus jugulare (disorder)
        "94855005",  # Neoplasm of uncertain behavior of glossopharyngeal nerve (disorder)
        "94856006",  # Neoplasm of uncertain behavior of glottis (disorder)
        "94857002",  # Neoplasm of uncertain behavior of great vessels (disorder)
        "94858007",  # Neoplasm of uncertain behavior of greater curvature of stomach (disorder)
        "94860009",  # Neoplasm of uncertain behavior of hamate bone (disorder)
        "94861008",  # Neoplasm of uncertain behavior of hand (disorder)
        "94862001",  # Neoplasm of uncertain behavior of hard palate (disorder)
        "94863006",  # Neoplasm of uncertain behavior of head of pancreas (disorder)
        "94864000",  # Neoplasm of uncertain behavior of head (disorder)
        "94865004",  # Neoplasm of uncertain behavior of heart (disorder)
        "94866003",  # Neoplasm of uncertain behavior of hepatic flexure of colon (disorder)
        "94867007",  # Neoplasm of uncertain behavior of hilus of lung (disorder)
        "94868002",  # Neoplasm of uncertain behavior of hypoglossal nerve (disorder)
        "94869005",  # Neoplasm of uncertain behavior of hypopharyngeal aspect of aryepiglottic fold (disorder)
        "94870006",  # Neoplasm of uncertain behavior of hypopharyngeal aspect of interarytenoid fold (disorder)
        "94871005",  # Neoplasm of uncertain behavior of hypopharynx (disorder)
        "94872003",  # Neoplasm of uncertain behavior of ileum (disorder)
        "94873008",  # Neoplasm of uncertain behavior of ilium (disorder)
        "94874002",  # Neoplasm of uncertain behavior of inguinal region (disorder)
        "94875001",  # Neoplasm of uncertain behavior of inner aspect of lip (disorder)
        "94876000",  # Neoplasm of uncertain behavior of inner aspect of lower lip (disorder)
        "94877009",  # Neoplasm of uncertain behavior of inner aspect of upper lip (disorder)
        "94878004",  # Neoplasm of uncertain behavior of intestinal tract (disorder)
        "94879007",  # Neoplasm of uncertain behavior of intra-abdominal organs (disorder)
        "94880005",  # Neoplasm of uncertain behavior of intrahepatic bile ducts (disorder)
        "94881009",  # Neoplasm of uncertain behavior of intrathoracic organs (disorder)
        "94882002",  # Neoplasm of uncertain behavior of ischium (disorder)
        "94883007",  # Neoplasm of uncertain behavior of islets of Langerhans (disorder)
        "94884001",  # Neoplasm of uncertain behavior of isthmus of uterus (disorder)
        "94885000",  # Neoplasm of uncertain behavior of jaw (disorder)
        "94886004",  # Neoplasm of uncertain behavior of jejunum (disorder)
        "94887008",  # Neoplasm of uncertain behavior of junctional region of epiglottis (disorder)
        "94888003",  # Neoplasm of uncertain behavior of junctional zone of tongue (disorder)
        "94889006",  # Neoplasm of uncertain behavior of kidney (disorder)
        "94890002",  # Neoplasm of uncertain behavior of labia majora (disorder)
        "94891003",  # Neoplasm of uncertain behavior of labia minora (disorder)
        "94893000",  # Neoplasm of uncertain behavior of lacrimal gland (disorder)
        "94894006",  # Neoplasm of uncertain behavior of large intestine (disorder)
        "94895007",  # Neoplasm of uncertain behavior of laryngeal aspect of aryepiglottic fold (disorder)
        "94896008",  # Neoplasm of uncertain behavior of laryngeal aspect of interarytenoid fold (disorder)
        "94897004",  # Neoplasm of uncertain behavior of laryngeal commissure (disorder)
        "94898009",  # Neoplasm of uncertain behavior of laryngeal surface of epiglottis (disorder)
        "94899001",  # Neoplasm of uncertain behavior of larynx (disorder)
        "94900006",  # Neoplasm of uncertain behavior of lateral portion of floor of mouth (disorder)
        "94901005",  # Neoplasm of uncertain behavior of lateral wall of nasopharynx (disorder)
        "94902003",  # Neoplasm of uncertain behavior of lateral wall of oropharynx (disorder)
        "94903008",  # Neoplasm of uncertain behavior of lateral wall of urinary bladder (disorder)
        "94904002",  # Neoplasm of uncertain behavior of left lower lobe of lung (disorder)
        "94905001",  # Neoplasm of uncertain behavior of left upper lobe of lung (disorder)
        "94907009",  # Neoplasm of uncertain behavior of lesser curvature of stomach (disorder)
        "94908004",  # Neoplasm of uncertain behavior of lingual tonsil (disorder)
        "94909007",  # Neoplasm of uncertain behavior of lip (disorder)
        "94910002",  # Neoplasm of uncertain behavior of liver (disorder)
        "94911003",  # Neoplasm of uncertain behavior of long bone of lower limb (disorder)
        "94912005",  # Neoplasm of uncertain behavior of long bone of upper limb (disorder)
        "94913000",  # Neoplasm of uncertain behavior of lower gum (disorder)
        "94915007",  # Neoplasm of uncertain behavior of lower limb (disorder)
        "94917004",  # Neoplasm of uncertain behavior of lower third of esophagus (disorder)
        "94918009",  # Neoplasm of uncertain behavior of lumbar vertebral column (disorder)
        "94919001",  # Neoplasm of uncertain behavior of lunate bone (disorder)
        "94920007",  # Neoplasm of uncertain behavior of lung (disorder)
        "94921006",  # Neoplasm of uncertain behavior of lymph node (disorder)
        "94922004",  # Neoplasm of uncertain behavior of main bronchus (disorder)
        "94925002",  # Neoplasm of uncertain behavior of male genital organ (disorder)
        "94926001",  # Neoplasm of uncertain behavior of mandible (disorder)
        "94928000",  # Neoplasm of uncertain behavior of maxilla (disorder)
        "94929008",  # Neoplasm of uncertain behavior of maxillary sinus (disorder)
        "94930003",  # Neoplasm of uncertain behavior of Meckel's diverticulum (disorder)
        "94931004",  # Neoplasm of uncertain behavior of mediastinum (disorder)
        "94932006",  # Neoplasm of uncertain behavior of metacarpal bone (disorder)
        "94933001",  # Neoplasm of uncertain behavior of metatarsal bone (disorder)
        "94934007",  # Neoplasm of uncertain behavior of middle ear (disorder)
        "94935008",  # Neoplasm of uncertain behavior of middle third of esophagus (disorder)
        "94936009",  # Neoplasm of uncertain behavior of minor salivary gland (disorder)
        "94937000",  # Neoplasm of uncertain behavior of mouth (disorder)
        "94938005",  # Neoplasm of uncertain behavior of multiple endocrine glands (disorder)
        "94939002",  # Neoplasm of uncertain behavior of muscle of abdomen (disorder)
        "94940000",  # Neoplasm of uncertain behavior of muscle of buttock (disorder)
        "94941001",  # Neoplasm of uncertain behavior of muscle of face (disorder)
        "94942008",  # Neoplasm of uncertain behavior of muscle of head (disorder)
        "94943003",  # Neoplasm of uncertain behavior of muscle of hip (disorder)
        "94944009",  # Neoplasm of uncertain behavior of muscle of inguinal region (disorder)
        "94945005",  # Neoplasm of uncertain behavior of muscle of lower limb (disorder)
        "94946006",  # Neoplasm of uncertain behavior of muscle of neck (disorder)
        "94947002",  # Neoplasm of uncertain behavior of muscle of pelvis (disorder)
        "94948007",  # Neoplasm of uncertain behavior of muscle of perineum (disorder)
        "94949004",  # Neoplasm of uncertain behavior of muscle of shoulder (disorder)
        "94950004",  # Neoplasm of uncertain behavior of muscle of thorax (disorder)
        "94951000",  # Neoplasm of uncertain behavior of muscle of trunk (disorder)
        "94952007",  # Neoplasm of uncertain behavior of muscle of upper limb (disorder)
        "94953002",  # Neoplasm of uncertain behavior of muscle (disorder)
        "94954008",  # Neoplasm of uncertain behavior of myocardium (disorder)
        "94955009",  # Neoplasm of uncertain behavior of myometrium (disorder)
        "94956005",  # Neoplasm of uncertain behavior of nasal bone (disorder)
        "94957001",  # Neoplasm of uncertain behavior of nasal cavity (disorder)
        "94958006",  # Neoplasm of uncertain behavior of nasal concha (disorder)
        "94959003",  # Neoplasm of uncertain behavior of nasopharynx (disorder)
        "94960008",  # Neoplasm of uncertain behavior of navicular bone of foot (disorder)
        "94961007",  # Neoplasm of uncertain behavior of scaphoid bone (disorder)
        "94962000",  # Neoplasm of uncertain behavior of neck (disorder)
        "94963005",  # Neoplasm of uncertain behavior of nervous system (disorder)
        "94966002",  # Neoplasm of uncertain behavior of nose (disorder)
        "94967006",  # Neoplasm of uncertain behavior of occipital bone (disorder)
        "94968001",  # Neoplasm of uncertain behavior of occipital lobe (disorder)
        "94969009",  # Neoplasm of uncertain behavior of oculomotor nerve (disorder)
        "94970005",  # Neoplasm of uncertain behavior of olfactory nerve (disorder)
        "94971009",  # Neoplasm of uncertain behavior of optic nerve (disorder)
        "94972002",  # Neoplasm of uncertain behavior of orbit proper (disorder)
        "94973007",  # Neoplasm of uncertain behavior of oropharynx (disorder)
        "94974001",  # Neoplasm of uncertain behavior of ovary (disorder)
        "94975000",  # Neoplasm of uncertain behavior of palate (disorder)
        "94976004",  # Neoplasm of uncertain behavior of palatine bone (disorder)
        "94977008",  # Neoplasm of uncertain behavior of tonsil (disorder)
        "94978003",  # Neoplasm of uncertain behavior of pancreas (disorder)
        "94979006",  # Neoplasm of uncertain behavior of pancreatic duct (disorder)
        "94980009",  # Neoplasm of uncertain behavior of para-aortic body (disorder)
        "94982001",  # Neoplasm of uncertain behavior of parametrium (disorder)
        "94983006",  # Neoplasm of uncertain behavior of parathyroid gland (disorder)
        "94984000",  # Neoplasm of uncertain behavior of paraurethral glands (disorder)
        "94985004",  # Neoplasm of uncertain behavior of parietal bone (disorder)
        "94986003",  # Neoplasm of uncertain behavior of parietal lobe (disorder)
        "94987007",  # Neoplasm of uncertain behavior of parietal peritoneum (disorder)
        "94988002",  # Neoplasm of uncertain behavior of parietal pleura (disorder)
        "94989005",  # Neoplasm of uncertain behavior of parotid gland (disorder)
        "94990001",  # Neoplasm of uncertain behavior of patella (disorder)
        "94991002",  # Neoplasm of uncertain behavior of pelvic bone (disorder)
        "94992009",  # Neoplasm of uncertain behavior of pelvic peritoneum (disorder)
        "94993004",  # Neoplasm of uncertain behavior of pelvis (disorder)
        "94994005",  # Neoplasm of uncertain behavior of penis (disorder)
        "94995006",  # Neoplasm of uncertain behavior of periadrenal tissue (disorder)
        "94996007",  # Neoplasm of uncertain behavior of perianal skin (disorder)
        "94997003",  # Neoplasm of uncertain behavior of pericardium (disorder)
        "94998008",  # Neoplasm of uncertain behavior of perirenal tissue (disorder)
        "94999000",  # Neoplasm of uncertain behavior of phalanx of foot (disorder)
        "95000000",  # Neoplasm of uncertain behavior of phalanx of hand (disorder)
        "95001001",  # Neoplasm of uncertain behavior of pharynx (disorder)
        "95002008",  # Neoplasm of uncertain behavior of pineal gland (disorder)
        "95003003",  # Neoplasm of uncertain behavior of pisiform bone of hand (disorder)
        "95004009",  # Neoplasm of uncertain behavior of pituitary gland (disorder)
        "95005005",  # Neoplasm of uncertain behavior of placenta (disorder)
        "95006006",  # Neoplasm of uncertain behavior of pleura (disorder)
        "95007002",  # Neoplasm of uncertain behavior of postcricoid region (disorder)
        "95008007",  # Neoplasm of uncertain behavior of posterior hypopharyngeal wall (disorder)
        "95009004",  # Neoplasm of uncertain behavior of posterior mediastinum (disorder)
        "95010009",  # Neoplasm of uncertain behavior of posterior wall of nasopharynx (disorder)
        "95011008",  # Neoplasm of uncertain behavior of posterior wall of oropharynx (disorder)
        "95012001",  # Neoplasm of uncertain behavior of posterior wall of urinary bladder (disorder)
        "95013006",  # Neoplasm of uncertain behavior of presacral region (disorder)
        "95014000",  # Neoplasm of uncertain behavior of prostate (disorder)
        "95015004",  # Neoplasm of uncertain behavior of pubis (disorder)
        "95016003",  # Neoplasm of uncertain behavior of pyloric antrum (disorder)
        "95017007",  # Neoplasm of uncertain behavior of pylorus (disorder)
        "95018002",  # Neoplasm of uncertain behavior of pyriform sinus (disorder)
        "95019005",  # Neoplasm of uncertain behavior of radius (disorder)
        "95020004",  # Neoplasm of uncertain behavior of rectosigmoid junction (disorder)
        "95021000",  # Neoplasm of uncertain behavior of rectouterine pouch (disorder)
        "95022007",  # Neoplasm of uncertain behavior of rectovaginal septum (disorder)
        "95023002",  # Neoplasm of uncertain behavior of rectovesical septum (disorder)
        "95024008",  # Neoplasm of uncertain behavior of rectum (disorder)
        "95025009",  # Neoplasm of uncertain behavior of renal pelvis (disorder)
        "95026005",  # Neoplasm of uncertain behavior of respiratory tract (disorder)
        "95027001",  # Neoplasm of uncertain behavior of retina (disorder)
        "95028006",  # Neoplasm of uncertain behavior of retrocecal tissue (disorder)
        "95029003",  # Neoplasm of uncertain behavior of retromolar area (disorder)
        "95030008",  # Neoplasm of uncertain behavior of rib (disorder)
        "95031007",  # Neoplasm of uncertain behavior of right lower lobe of lung (disorder)
        "95032000",  # Neoplasm of uncertain behavior of middle lobe of right lung (disorder)
        "95033005",  # Neoplasm of uncertain behavior of right upper lobe of lung (disorder)
        "95034004",  # Neoplasm of uncertain behavior of round ligament of uterus (disorder)
        "95035003",  # Neoplasm of uncertain behavior of sacrococcygeal region (disorder)
        "95036002",  # Neoplasm of uncertain behavior of sacrum (disorder)
        "95037006",  # Neoplasm of uncertain behavior of scapula (disorder)
        "95038001",  # Neoplasm of uncertain behavior of sclera (disorder)
        "95039009",  # Neoplasm of uncertain behavior of scrotum (disorder)
        "95040006",  # Neoplasm of uncertain behavior of sebaceous gland (disorder)
        "95041005",  # Neoplasm of uncertain behavior of second cuneiform bone of foot (disorder)
        "95042003",  # Neoplasm of uncertain behavior of septum of nose (disorder)
        "95043008",  # Neoplasm of uncertain behavior of short bone of lower limb (disorder)
        "95044002",  # Neoplasm of uncertain behavior of short bone of upper limb (disorder)
        "95045001",  # Neoplasm of uncertain behavior of shoulder (disorder)
        "95046000",  # Neoplasm of uncertain behavior of sigmoid colon (disorder)
        "95047009",  # Neoplasm of uncertain behavior of skin of abdomen (disorder)
        "95048004",  # Neoplasm of uncertain behavior of skin of ankle (disorder)
        "95049007",  # Neoplasm of uncertain behavior of skin of upper arm (disorder)
        "95050007",  # Neoplasm of uncertain behavior of skin of axilla (disorder)
        "95051006",  # Neoplasm of uncertain behavior of skin of back (disorder)
        "95052004",  # Neoplasm of uncertain behavior of skin of breast (disorder)
        "95053009",  # Neoplasm of uncertain behavior of skin of buttock (disorder)
        "95054003",  # Neoplasm of uncertain behavior of skin of cheek (disorder)
        "95055002",  # Neoplasm of uncertain behavior of skin of chest (disorder)
        "95056001",  # Neoplasm of uncertain behavior of skin of chin (disorder)
        "95057005",  # Neoplasm of uncertain behavior of skin of ear (disorder)
        "95058000",  # Neoplasm of uncertain behavior of skin of elbow (disorder)
        "95059008",  # Neoplasm of uncertain behavior of skin of external auditory canal (disorder)
        "95060003",  # Neoplasm of uncertain behavior of skin of eyebrow (disorder)
        "95061004",  # Neoplasm of uncertain behavior of skin of eyelid (disorder)
        "95062006",  # Neoplasm of uncertain behavior of skin of face (disorder)
        "95063001",  # Neoplasm of uncertain behavior of skin of finger (disorder)
        "95064007",  # Neoplasm of uncertain behavior of skin of foot (disorder)
        "95065008",  # Neoplasm of uncertain behavior of skin of forearm (disorder)
        "95066009",  # Neoplasm of uncertain behavior of skin of forehead (disorder)
        "95067000",  # Neoplasm of uncertain behavior of skin of groin (disorder)
        "95068005",  # Neoplasm of uncertain behavior of skin of hand (disorder)
        "95069002",  # Neoplasm of uncertain behavior of skin of hip (disorder)
        "95070001",  # Neoplasm of uncertain behavior of skin of knee (disorder)
        "95071002",  # Neoplasm of uncertain behavior of skin of lower leg (disorder)
        "95072009",  # Neoplasm of uncertain behavior of skin of lip (disorder)
        "95073004",  # Neoplasm of uncertain behavior of skin of lower limb (disorder)
        "95074005",  # Neoplasm of uncertain behavior of skin of neck (disorder)
        "95075006",  # Neoplasm of uncertain behavior of skin of nose (disorder)
        "95076007",  # Neoplasm of uncertain behavior of skin of perineum (disorder)
        "95077003",  # Neoplasm of uncertain behavior of skin of popliteal area (disorder)
        "95078008",  # Neoplasm of uncertain behavior of skin of scalp (disorder)
        "95079000",  # Neoplasm of uncertain behavior of skin of shoulder (disorder)
        "95080002",  # Neoplasm of uncertain behavior of skin of temporal region (disorder)
        "95081003",  # Neoplasm of uncertain behavior of skin of thigh (disorder)
        "95082005",  # Neoplasm of uncertain behavior of skin of toe (disorder)
        "95083000",  # Neoplasm of uncertain behavior of skin of trunk (disorder)
        "95084006",  # Neoplasm of uncertain behavior of skin of umbilicus (disorder)
        "95085007",  # Neoplasm of uncertain behavior of skin of upper limb (disorder)
        "95086008",  # Neoplasm of uncertain behavior of skin of wrist (disorder)
        "95087004",  # Neoplasm of uncertain behavior of skin (disorder)
        "95088009",  # Neoplasm of uncertain behavior of small intestine (disorder)
        "95089001",  # Neoplasm of uncertain behavior of soft palate (disorder)
        "95090005",  # Neoplasm of uncertain behavior of soft tissues of abdomen (disorder)
        "95091009",  # Neoplasm of uncertain behavior of soft tissues of axilla (disorder)
        "95092002",  # Neoplasm of uncertain behavior of soft tissues of buttock (disorder)
        "95093007",  # Neoplasm of uncertain behavior of soft tissues of face (disorder)
        "95094001",  # Neoplasm of uncertain behavior of soft tissues of head (disorder)
        "95095000",  # Neoplasm of uncertain behavior of soft tissues of hip (disorder)
        "95096004",  # Neoplasm of uncertain behavior of soft tissues of inguinal region (disorder)
        "95097008",  # Neoplasm of uncertain behavior of soft tissues of lower limb (disorder)
        "95098003",  # Neoplasm of uncertain behavior of soft tissues of neck (disorder)
        "95099006",  # Neoplasm of uncertain behavior of soft tissues of pelvis (disorder)
        "95100003",  # Neoplasm of uncertain behavior of soft tissues of perineum (disorder)
        "95101004",  # Neoplasm of uncertain behavior of soft tissues of shoulder (disorder)
        "95102006",  # Neoplasm of uncertain behavior of soft tissues of thorax (disorder)
        "95103001",  # Neoplasm of uncertain behavior of soft tissues of trunk (disorder)
        "95104007",  # Neoplasm of uncertain behavior of soft tissues of upper limb (disorder)
        "95105008",  # Neoplasm of uncertain behavior of spermatic cord (disorder)
        "95106009",  # Neoplasm of uncertain behavior of sphenoid bone (disorder)
        "95107000",  # Neoplasm of uncertain behavior of sphenoidal sinus (disorder)
        "95108005",  # Neoplasm of uncertain behavior of spinal cord (disorder)
        "95109002",  # Neoplasm of uncertain behavior of spinal meninges (disorder)
        "95110007",  # Neoplasm of uncertain behavior of vertebral column (disorder)
        "95111006",  # Neoplasm of uncertain behavior of spleen (disorder)
        "95112004",  # Neoplasm of uncertain behavior of splenic flexure of colon (disorder)
        "95113009",  # Neoplasm of uncertain behavior of sternum (disorder)
        "95115002",  # Neoplasm of uncertain behavior of subglottis (disorder)
        "95116001",  # Neoplasm of uncertain behavior of sublingual gland (disorder)
        "95118000",  # Neoplasm of uncertain behavior of superior wall of nasopharynx (disorder)
        "95119008",  # Neoplasm of uncertain behavior of supraclavicular region (disorder)
        "95120002",  # Neoplasm of uncertain behavior of supraglottis (disorder)
        "95121003",  # Neoplasm of uncertain behavior of sweat gland (disorder)
        "95122005",  # Neoplasm of uncertain behavior of tail of pancreas (disorder)
        "95123000",  # Neoplasm of uncertain behavior of talus (disorder)
        "95124006",  # Neoplasm of uncertain behavior of tarsal bone (disorder)
        "95125007",  # Neoplasm of uncertain behavior of temporal bone (disorder)
        "95126008",  # Neoplasm of uncertain behavior of temporal lobe (disorder)
        "95127004",  # Neoplasm of uncertain behavior of testis (disorder)
        "95128009",  # Neoplasm of uncertain behavior of the mesentery (disorder)
        "95129001",  # Neoplasm of uncertain behavior of the mesocolon (disorder)
        "95130006",  # Neoplasm of uncertain behavior of omentum (disorder)
        "95131005",  # Neoplasm of uncertain behavior of the perineum (disorder)
        "95132003",  # Neoplasm of uncertain behavior of retroperitoneum (disorder)
        "95133008",  # Neoplasm of uncertain behavior of thigh (disorder)
        "95134002",  # Neoplasm of uncertain behavior of third cuneiform bone of foot (disorder)
        "95135001",  # Neoplasm of uncertain behavior of thoracic esophagus (disorder)
        "95136000",  # Neoplasm of uncertain behavior of thymus (disorder)
        "95137009",  # Neoplasm of uncertain behavior of thyroglossal duct (disorder)
        "95138004",  # Neoplasm of uncertain behavior of thyroid gland (disorder)
        "95139007",  # Neoplasm of uncertain behavior of tibia (disorder)
        "95140009",  # Neoplasm of uncertain behavior of border of tongue (disorder)
        "95141008",  # Neoplasm of uncertain behavior of tongue (disorder)
        "95142001",  # Neoplasm of uncertain behavior of tonsillar fossa (disorder)
        "95143006",  # Neoplasm of uncertain behavior of tonsillar pillar (disorder)
        "95144000",  # Neoplasm of uncertain behavior of trachea (disorder)
        "95145004",  # Neoplasm of uncertain behavior of transverse colon (disorder)
        "95146003",  # Neoplasm of uncertain behavior of trapezium (disorder)
        "95147007",  # Neoplasm of uncertain behavior of trapezoid bone (disorder)
        "95148002",  # Neoplasm of uncertain behavior of trigeminal nerve (disorder)
        "95149005",  # Neoplasm of uncertain behavior of trigone of urinary bladder (disorder)
        "95150005",  # Neoplasm of uncertain behavior of trochlear nerve (disorder)
        "95151009",  # Neoplasm of uncertain behavior of trunk (disorder)
        "95152002",  # Neoplasm of uncertain behavior of ulna (disorder)
        "95153007",  # Neoplasm of uncertain behavior of undescended testis (disorder)
        "95154001",  # Neoplasm of uncertain behavior of upper gum (disorder)
        "95156004",  # Neoplasm of uncertain behavior of upper limb (disorder)
        "95158003",  # Neoplasm of uncertain behavior of upper respiratory tract (disorder)
        "95159006",  # Neoplasm of uncertain behavior of upper third of esophagus (disorder)
        "95160001",  # Neoplasm of uncertain behavior of urachus (disorder)
        "95161002",  # Neoplasm of uncertain behavior of ureter (disorder)
        "95162009",  # Neoplasm of uncertain behavior of ureteric orifice of urinary bladder (disorder)
        "95163004",  # Neoplasm of uncertain behavior of urethra (disorder)
        "95164005",  # Neoplasm of uncertain behavior of urinary bladder neck (disorder)
        "95165006",  # Neoplasm of uncertain behavior of urinary system (disorder)
        "95166007",  # Neoplasm of uncertain behavior of uterine adnexa (disorder)
        "95167003",  # Neoplasm of uncertain behavior of uterus (disorder)
        "95168008",  # Neoplasm of uncertain behavior of uveal tract (disorder)
        "95169000",  # Neoplasm of uncertain behavior of uvula palatina (disorder)
        "95170004",  # Neoplasm of uncertain behavior of vagina (disorder)
        "95171000",  # Neoplasm of uncertain behavior of vagus nerve (disorder)
        "95172007",  # Neoplasm of uncertain behavior of vallecula of epiglottis (disorder)
        "95173002",  # Neoplasm of uncertain behavior of vas deferens (disorder)
        "95174008",  # Neoplasm of uncertain behavior of ventral surface of tongue (disorder)
        "95175009",  # Neoplasm of uncertain behavior of vermilion border of lip (disorder)
        "95176005",  # Neoplasm of uncertain behavior of vermilion border of lower lip (disorder)
        "95177001",  # Neoplasm of uncertain behavior of vermilion border of upper lip (disorder)
        "95178006",  # Neoplasm of uncertain behavior of vestibule of mouth (disorder)
        "95179003",  # Neoplasm of uncertain behavior of vestibule of nose (disorder)
        "95180000",  # Neoplasm of uncertain behavior of visceral pleura (disorder)
        "95181001",  # Neoplasm of uncertain behavior of vocal cord (disorder)
        "95182008",  # Neoplasm of uncertain behavior of vomer (disorder)
        "95183003",  # Neoplasm of uncertain behavior of vulva (disorder)
        "95184009",  # Neoplasm of uncertain behavior of Waldeyer's ring (disorder)
        "95185005",  # Neoplasm of uncertain behavior of zygomatic bone (disorder)
        "95186006",  # Nodular lymphoma of intra-abdominal lymph nodes (disorder)
        "95187002",  # Nodular lymphoma of intrapelvic lymph nodes (disorder)
        "95188007",  # Nodular lymphoma of intrathoracic lymph nodes (disorder)
        "95192000",  # Nodular lymphoma of lymph nodes of multiple sites (disorder)
        "95193005",  # Nodular lymphoma of spleen (disorder)
        "95209008",  # Plasma cell leukemia in remission (disorder)
        "95210003",  # Plasma cell leukemia, disease (disorder)
        "95214007",  # Primary malignant neoplasm of liver (disorder)
        "95224004",  # Reticulosarcoma of intra-abdominal lymph nodes (disorder)
        "95225003",  # Reticulosarcoma of intrapelvic lymph nodes (disorder)
        "95226002",  # Reticulosarcoma of intrathoracic lymph nodes (disorder)
        "95230004",  # Reticulosarcoma of lymph nodes of multiple sites (disorder)
        "95231000",  # Reticulosarcoma of spleen (disorder)
        "95263006",  # Sezary's disease of spleen (disorder)
        "9541000119105",  # Primary adenocarcinoma of gallbladder (disorder)
        "96281000119107",  # Overlapping malignant neoplasm of colon and rectum (disorder)
        "96901000119105",  # Prostate cancer metastatic to eye (disorder)
        "97051000119105",  # Malignant melanoma metastatic to pancreas (disorder)
        "985355341000119101",  # Malignant melanoma of skin of left wrist (disorder)
        "98981000119103",  # Primary malignant neoplasm of ileocecal valve (disorder)
        "99101000119101",  # Primary adenocarcinoma of fallopian tube (disorder)
        "99111000119103",  # Primary adenocarcinoma of intestinal tract (disorder)
        "99121000119105",  # Primary adenocarcinoma of vagina (disorder)
        "99131000119108",  # Astrocytoma of cerebrum (disorder)
    }

class Dysthymia(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses related to dysthymia.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of dysthymia, including early and late onset, and primary and secondary dysthymia.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Dysthymia"
    OID = "2.16.840.1.113883.3.67.1.101.1.254"
    DEFINITION_VERSION = "20220219"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "F341",  # Dysthymic disorder
    }

    SNOMEDCT = {
        "1153575004",  # Persistent depressive disorder (disorder)
        "19694002",  # Late onset dysthymia (disorder)
        "2506003",  # Early onset dysthymia (disorder)
        "3109008",  # Secondary dysthymia early onset (disorder)
        "36170009",  # Secondary dysthymia late onset (disorder)
        "38451003",  # Primary dysthymia early onset (disorder)
        "67711008",  # Primary dysthymia late onset (disorder)
        "78667006",  # Dysthymia (disorder)
        "83176005",  # Primary dysthymia (disorder)
        "85080004",  # Secondary dysthymia (disorder)
    }

class MajorDepressionIncludingRemission(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses related to major depression and depression remission.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of major depression including chronic, recurrent, and single episodes of depression

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Major Depression Including Remission"
    OID = "2.16.840.113883.3.67.1.101.3.2444"
    DEFINITION_VERSION = "20210219"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "F320",  # Major depressive disorder, single episode, mild
        "F321",  # Major depressive disorder, single episode, moderate
        "F322",  # Major depressive disorder, single episode, severe without psychotic features
        "F323",  # Major depressive disorder, single episode, severe with psychotic features
        "F324",  # Major depressive disorder, single episode, in partial remission
        "F325",  # Major depressive disorder, single episode, in full remission
        "F329",  # Major depressive disorder, single episode, unspecified
        "F330",  # Major depressive disorder, recurrent, mild
        "F331",  # Major depressive disorder, recurrent, moderate
        "F332",  # Major depressive disorder, recurrent severe without psychotic features
        "F333",  # Major depressive disorder, recurrent, severe with psychotic symptoms
        "F3340",  # Major depressive disorder, recurrent, in remission, unspecified
        "F3341",  # Major depressive disorder, recurrent, in partial remission
        "F3342",  # Major depressive disorder, recurrent, in full remission
        "F339",  # Major depressive disorder, recurrent, unspecified
    }

    SNOMEDCT = {
        "104851000119103",  # Postpartum major depression in remission (disorder)
        "10811121000119102",  # Major depressive disorder in mother complicating childbirth (disorder)
        "10811161000119107",  # Major depressive disorder in mother complicating pregnancy (disorder)
        "14183003",  # Chronic major depressive disorder, single episode (disorder)
        "15193003",  # Severe recurrent major depression with psychotic features, mood-incongruent (disorder)
        "15639000",  # Moderate major depression, single episode (disorder)
        "16264621000119109",  # Recurrent mild major depressive disorder co-occurrent with anxiety (disorder)
        "16264821000119108",  # Recurrent severe major depressive disorder co-occurrent with anxiety (disorder)
        "16264901000119109",  # Recurrent moderate major depressive disorder co-occurrent with anxiety (disorder)
        "16265951000119109",  # Mild major depressive disorder co-occurrent with anxiety single episode (disorder)
        "16266831000119100",  # Moderate major depressive disorder co-occurrent with anxiety single episode (disorder)
        "16266991000119108",  # Severe major depressive disorder co-occurrent with anxiety single episode (disorder)
        "18818009",  # Moderate recurrent major depression (disorder)
        "191604000",  # Single major depressive episode, severe, with psychosis (disorder)
        "191610000",  # Recurrent major depressive episodes, mild (disorder)
        "191611001",  # Recurrent major depressive episodes, moderate (disorder)
        "191613003",  # Recurrent major depressive episodes, severe, with psychosis (disorder)
        "19527009",  # Single episode of major depression in full remission (disorder)
        "20250007",  # Severe major depression, single episode, with psychotic features, mood-incongruent (disorder)
        "251000119105",  # Severe major depression, single episode (disorder)
        "25922000",  # Major depressive disorder, single episode with postpartum onset (disorder)
        "2618002",  # Chronic recurrent major depressive disorder (disorder)
        "268621008",  # Recurrent major depressive episodes (disorder)
        "281000119103",  # Severe recurrent major depression (disorder)
        "28475009",  # Severe recurrent major depression with psychotic features (disorder)
        "30605009",  # Major depression in partial remission (disorder)
        "319768000",  # Recurrent major depressive disorder with melancholic features (disorder)
        "320751009",  # Major depression, melancholic type (disorder)
        "33078009",  # Severe recurrent major depression with psychotic features, mood-congruent (disorder)
        "33135002",  # Recurrent major depression in partial remission (disorder)
        "33736005",  # Severe major depression with psychotic features, mood-congruent (disorder)
        "36474008",  # Severe recurrent major depression without psychotic features (disorder)
        "36923009",  # Major depression, single episode (disorder)
        "370143000",  # Major depressive disorder (disorder)
        "38694004",  # Recurrent major depressive disorder with atypical features (disorder)
        "39809009",  # Recurrent major depressive disorder with catatonic features (disorder)
        "40379007",  # Mild recurrent major depression (disorder)
        "42810003",  # Major depression in remission (disorder)
        "42925002",  # Major depressive disorder, single episode with atypical features (disorder)
        "430852001",  # Severe major depression, single episode, with psychotic features (disorder)
        "450714000",  # Severe major depression (disorder)
        "46244001",  # Recurrent major depression in full remission (disorder)
        "60099002",  # Severe major depression with psychotic features, mood-incongruent (disorder)
        "63412003",  # Major depression in full remission (disorder)
        "63778009",  # Major depressive disorder, single episode with melancholic features (disorder)
        "66344007",  # Recurrent major depression (disorder)
        "68019004",  # Recurrent major depression in remission (disorder)
        "69392006",  # Major depressive disorder, single episode with catatonic features (disorder)
        "70747007",  # Major depression single episode, in partial remission (disorder)
        "71336009",  # Recurrent major depressive disorder with postpartum onset (disorder)
        "719592004",  # Moderately severe major depression (disorder)
        "720451004",  # Minimal recurrent major depression (disorder)
        "720452006",  # Moderately severe recurrent major depression (disorder)
        "720453001",  # Moderately severe major depression single episode (disorder)
        "720454007",  # Minimal major depression single episode (disorder)
        "720455008",  # Minimal major depression (disorder)
        "726772006",  # Major depression with psychotic features (disorder)
        "73867007",  # Severe major depression with psychotic features (disorder)
        "75084000",  # Severe major depression without psychotic features (disorder)
        "76441001",  # Severe major depression, single episode, without psychotic features (disorder)
        "77911002",  # Severe major depression, single episode, with psychotic features, mood-congruent (disorder)
        "79298009",  # Mild major depression, single episode (disorder)
        "832007",  # Moderate major depression (disorder)
        "87512008",  # Mild major depression (disorder)
    }

class PersonalityDisorderEmotionallyLabile(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses related to personality disorders where the patient could not reliably complete a PHQ-9 tool due to their condition.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of personality disorder, including borderline, cyclothymic, histrionic, and factitious disorders.

    **Exclusion Criteria:** Excludes concepts that represent some personality disorders such as narcissistic or obsessive compulsive personality disorders.
    """

    VALUE_SET_NAME = "Personality Disorder Emotionally Labile"
    OID = "2.16.840.1.113883.3.67.1.101.1.246"
    DEFINITION_VERSION = "20240214"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "F340",  # Cyclothymic disorder
        "F603",  # Borderline personality disorder
        "F604",  # Histrionic personality disorder
        "F6810",  # Factitious disorder imposed on self, unspecified
        "F6811",  # Factitious disorder imposed on self, with predominantly psychological signs and symptoms
        "F6812",  # Factitious disorder imposed on self, with predominantly physical signs and symptoms
        "F6813",  # Factitious disorder imposed on self, with combined psychological and physical signs and symptoms
    }

    SNOMEDCT = {
        "191765005",  # Emotionally unstable personality disorder (disorder)
        "20010003",  # Borderline personality disorder (disorder)
        "21586000",  # Munchausen's syndrome (disorder)
        "24315006",  # Factitious disorder with combined physical AND psychological symptoms (disorder)
        "430744005",  # Factitious disorder with predominantly physical signs and symptoms (disorder)
        "430751001",  # Factitious disorder with predominantly psychological signs and symptoms (disorder)
        "55341008",  # Histrionic personality disorder (disorder)
        "76105009",  # Cyclothymia (disorder)
    }

class PervasiveDevelopmentalDisorder(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of an autistic disorder, other childhood disintegrative disorder, or other pervasive developmental disorder.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of autistic disorder, other childhood disintegrative disorders, or other pervasive developmental disorders.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Pervasive Developmental Disorder"
    OID = "2.16.840.1.113883.3.464.1003.105.12.1152"
    DEFINITION_VERSION = "20240210"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "F840",  # Autistic disorder
        "F842",  # Rett's syndrome
        "F843",  # Other childhood disintegrative disorder
        "F845",  # Asperger's syndrome
        "F848",  # Other pervasive developmental disorders
        "F849",  # Pervasive developmental disorder, unspecified
    }

    SNOMEDCT = {
        "191689008",  # Active infantile autism (disorder)
        "191690004",  # Residual infantile autism (disorder)
        "191692007",  # Active disintegrative psychoses (disorder)
        "191693002",  # Residual disintegrative psychoses (disorder)
        "231536004",  # Atypical autism (disorder)
        "23560001",  # Asperger's disorder (disorder)
        "35919005",  # Pervasive developmental disorder (disorder)
        "373618009",  # Autistic spectrum disorder with isolated skills (disorder)
        "39951000119105",  # Pervasive developmental disorder of residual state (disorder)
        "408857007",  # Infantile autism (disorder)
        "43614003",  # Autistic disorder of childhood onset (disorder)
        "442314000",  # Active but odd autism (disorder)
        "68618008",  # Rett syndrome (disorder)
        "702732007",  # High-functioning autism (disorder)
        "708037001",  # Residual Asperger's disorder (disorder)
        "718393002",  # Atypical Rett syndrome (disorder)
        "719600006",  # 1p21.3 microdeletion syndrome (disorder)
        "71961003",  # Childhood disintegrative disorder (disorder)
        "723332005",  # Isodicentric chromosome 15 syndrome (disorder)
        "733623005",  # Autism spectrum disorder, epilepsy, arthrogryposis syndrome (disorder)
        "766824003",  # Activity dependent neuroprotector homeobox related multiple congenital anomalies, intellectual disability, autism spectrum disorder (disorder)
        "770790004",  # Developmental delay with autism spectrum disorder and gait instability (disorder)
        "771448004",  # Autism epilepsy syndrome due to branched chain ketoacid dehydrogenase kinase deficiency (disorder)
        "771512003",  # Autism spectrum disorder due to AUTS2 activator of transcription and developmental regulator deficiency (disorder)
        "870260008",  # Pervasive developmental disorder with marked impairment of functional language with loss of previously acquired skills (disorder)
        "870261007",  # Pervasive developmental disorder with marked impairment of functional language without loss of previously acquired skills (disorder)
        "870262000",  # Pervasive developmental disorder with disorder of intellectual development without loss of previously acquired skills (disorder)
        "870263005",  # Pervasive developmental disorder with impairment of functional language (disorder)
        "870264004",  # Pervasive developmental disorder with disorder of intellectual development and pervasive impairment of functional language without loss of previously acquired skills (disorder)
        "870265003",  # Pervasive developmental disorder with disorder of intellectual development with loss of previously acquired skills (disorder)
        "870266002",  # Pervasive developmental disorder with disorder of intellectual development and marked impairment of functional language with loss of previously acquired skills (disorder)
        "870267006",  # Pervasive developmental disorder with disorder of intellectual development and marked impairment of functional language without loss of previously acquired skills (disorder)
        "870268001",  # Pervasive developmental disorder with disorder of intellectual development and complete impairment of functional language without loss of previously acquired skills (disorder)
        "870269009",  # Pervasive developmental disorder with disorder of intellectual development and absence of functional language with loss of previously acquired skills (disorder)
        "870270005",  # Pervasive developmental disorder with disorder of intellectual development and complete impairment of functional language with loss of previously acquired skills (disorder)
        "870280009",  # Pervasive developmental disorder with severe impairment of functional language with loss of previously acquired skills (disorder)
        "870282001",  # Pervasive developmental disorder with severe impairment of functional language without loss of previously acquired skills (disorder)
        "870303005",  # Pervasive developmental disorder with complete impairment of functional language with loss of previously acquired skills (disorder)
        "870304004",  # Pervasive developmental disorder with complete impairment of functional language without loss of previously acquired skills (disorder)
        "870305003",  # Pervasive developmental disorder with cognitive developmental delay and marked impairment of functional language (disorder)
        "870306002",  # Pervasive developmental disorder with complete impairment of functional language (disorder)
        "870307006",  # Pervasive developmental disorder with abscence of functional language (disorder)
        "870308001",  # Pervasive developmental disorder with cognitive developmental delay and complete impairment of functional language (disorder)
    }

class SchizophreniaOrPsychoticDisorder(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of all types of schizophrenia and schizoaffective disorders.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that identify a diagnosis of schizophrenia or schizoaffective disorders.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Schizophrenia or Psychotic Disorder"
    OID = "2.16.840.1.113883.3.464.1003.105.12.1104"
    DEFINITION_VERSION = "20240214"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "F200",  # Paranoid schizophrenia
        "F201",  # Disorganized schizophrenia
        "F202",  # Catatonic schizophrenia
        "F203",  # Undifferentiated schizophrenia
        "F205",  # Residual schizophrenia
        "F2081",  # Schizophreniform disorder
        "F2089",  # Other schizophrenia
        "F209",  # Schizophrenia, unspecified
        "F21",  # Schizotypal disorder
        "F23",  # Brief psychotic disorder
        "F250",  # Schizoaffective disorder, bipolar type
        "F251",  # Schizoaffective disorder, depressive type
        "F258",  # Other schizoaffective disorders
        "F259",  # Schizoaffective disorder, unspecified
        "F28",  # Other psychotic disorder not due to a substance or known physiological condition
        "F29",  # Unspecified psychosis not due to a substance or known physiological condition
    }

    SNOMEDCT = {
        "111482003",  # Subchronic schizophrenia with acute exacerbations (disorder)
        "111483008",  # Catatonic schizophrenia in remission (disorder)
        "111484002",  # Undifferentiated schizophrenia (disorder)
        "12939007",  # Chronic disorganized schizophrenia (disorder)
        "14291003",  # Subchronic disorganized schizophrenia with acute exacerbations (disorder)
        "16990005",  # Subchronic schizophrenia (disorder)
        "191526005",  # Schizophrenic disorders (disorder)
        "191527001",  # Simple schizophrenia (disorder)
        "191531007",  # Acute exacerbation of chronic schizophrenia (disorder)
        "191542003",  # Catatonic schizophrenia (disorder)
        "191547009",  # Acute exacerbation of subchronic catatonic schizophrenia (disorder)
        "191548004",  # Acute exacerbation of chronic catatonic schizophrenia (disorder)
        "191554003",  # Acute exacerbation of subchronic paranoid schizophrenia (disorder)
        "191555002",  # Acute exacerbation of chronic paranoid schizophrenia (disorder)
        "191559008",  # Latent schizophrenia (disorder)
        "191567000",  # Schizoaffective schizophrenia (disorder)
        "191569002",  # Subchronic schizoaffective schizophrenia (disorder)
        "191570001",  # Chronic schizoaffective schizophrenia (disorder)
        "191571002",  # Acute exacerbation of subchronic schizoaffective schizophrenia (disorder)
        "191572009",  # Acute exacerbation of chronic schizoaffective schizophrenia (disorder)
        "191574005",  # Schizoaffective schizophrenia in remission (disorder)
        "191577003",  # Cenesthopathic schizophrenia (disorder)
        "191680007",  # Psychogenic paranoid psychosis (disorder)
        "231437006",  # Reactive psychoses (disorder)
        "231489001",  # Acute transient psychotic disorder (disorder)
        "247804008",  # Schizophrenic prodrome (disorder)
        "26025008",  # Residual schizophrenia (disorder)
        "268617001",  # Acute schizophrenic episode (disorder)
        "268624000",  # Acute paranoid reaction (disorder)
        "270901009",  # Schizoaffective disorder, mixed type (disorder)
        "271428004",  # Schizoaffective disorder, manic type (disorder)
        "27387000",  # Subchronic disorganized schizophrenia (disorder)
        "278853003",  # Acute schizophrenia-like psychotic disorder (disorder)
        "29599000",  # Chronic undifferentiated schizophrenia (disorder)
        "30336007",  # Chronic residual schizophrenia with acute exacerbations (disorder)
        "31027006",  # Schizotypal personality disorder (disorder)
        "31373002",  # Disorganized schizophrenia in remission (disorder)
        "31658008",  # Chronic paranoid schizophrenia (disorder)
        "35218008",  # Chronic disorganized schizophrenia with acute exacerbation (disorder)
        "35252006",  # Disorganized schizophrenia (disorder)
        "36158005",  # Schizophreniform disorder with good prognostic features (disorder)
        "38368003",  # Schizoaffective disorder, bipolar type (disorder)
        "39610001",  # Undifferentiated schizophrenia in remission (disorder)
        "416340002",  # Late onset schizophrenia (disorder)
        "42868002",  # Subchronic catatonic schizophrenia (disorder)
        "441704009",  # Affective psychosis (disorder)
        "441833000",  # Lethal catatonia (disorder)
        "4926007",  # Schizophrenia in remission (disorder)
        "51133006",  # Residual schizophrenia in remission (disorder)
        "5464005",  # Brief reactive psychosis (disorder)
        "55736003",  # Schizophreniform disorder without good prognostic features (disorder)
        "58214004",  # Schizophrenia (disorder)
        "63181006",  # Paranoid schizophrenia in remission (disorder)
        "64905009",  # Paranoid schizophrenia (disorder)
        "68890003",  # Schizoaffective disorder (disorder)
        "68995007",  # Chronic catatonic schizophrenia (disorder)
        "7025000",  # Subchronic undifferentiated schizophrenia with acute exacerbations (disorder)
        "70814008",  # Subchronic residual schizophrenia with acute exacerbations (disorder)
        "71103003",  # Chronic residual schizophrenia (disorder)
        "76566000",  # Subchronic residual schizophrenia (disorder)
        "79204003",  # Chronic undifferentiated schizophrenia with acute exacerbations (disorder)
        "79866005",  # Subchronic paranoid schizophrenia (disorder)
        "83746006",  # Chronic schizophrenia (disorder)
        "84760002",  # Schizoaffective disorder, depressive type (disorder)
        "85861002",  # Subchronic undifferentiated schizophrenia (disorder)
        "88975006",  # Schizophreniform disorder (disorder)
    }

class ChronicKidneyDiseaseStage5(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of Stage 5 chronic kidney disease.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of Stage 5 of chronic kidney disease.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Chronic Kidney Disease, Stage 5"
    OID = "2.16.840.1.113883.3.526.3.1002"
    DEFINITION_VERSION = "20200307"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "N185",  # Chronic kidney disease, stage 5
    }

    SNOMEDCT = {
        "433146000",  # Chronic kidney disease stage 5 (disorder)
    }

class EndStageRenalDisease(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of end stage renal disease (ESRD).

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of end stage renal disease (ESRD).

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "End Stage Renal Disease"
    OID = "2.16.840.1.113883.3.526.3.353"
    DEFINITION_VERSION = "20200307"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "N186",  # End stage renal disease
    }

    SNOMEDCT = {
        "236434000",  # End stage renal failure untreated by renal replacement therapy (disorder)
        "236435004",  # End stage renal failure on dialysis (disorder)
        "236436003",  # End stage renal failure with renal transplant (disorder)
        "46177005",  # End-stage renal disease (disorder)
    }

class EssentialHypertension(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of essential hypertension.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of essential hypertension.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Essential Hypertension"
    OID = "2.16.840.1.113883.3.464.1003.104.12.1011"
    DEFINITION_VERSION = "20240106"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "I10",  # Essential (primary) hypertension
    }

    SNOMEDCT = {
        "1201005",  # Benign essential hypertension (disorder)
        "18416000",  # Essential hypertension complicating AND/OR reason for care during childbirth (disorder)
        "19769006",  # High-renin essential hypertension (disorder)
        "23717007",  # Benign essential hypertension complicating AND/OR reason for care during pregnancy (disorder)
        "35303009",  # Benign essential hypertension complicating AND/OR reason for care during puerperium (disorder)
        "371125006",  # Labile essential hypertension (disorder)
        "40511000119107",  # Postpartum pre-existing essential hypertension (disorder)
        "429457004",  # Systolic essential hypertension (disorder)
        "46481004",  # Low-renin essential hypertension (disorder)
        "59621000",  # Essential hypertension (disorder)
        "63287004",  # Benign essential hypertension in obstetric context (disorder)
        "71874008",  # Benign essential hypertension complicating AND/OR reason for care during childbirth (disorder)
        "72022006",  # Essential hypertension in obstetric context (disorder)
        "78808002",  # Essential hypertension complicating AND/OR reason for care during pregnancy (disorder)
        "78975002",  # Malignant essential hypertension (disorder)
        "9901000",  # Essential hypertension complicating AND/OR reason for care during puerperium (disorder)
    }

class KidneyTransplantRecipient(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of a history of kidney transplant.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of a history of kidney transplant.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Kidney Transplant Recipient"
    OID = "2.16.840.1.113883.3.464.1003.109.12.1029"
    DEFINITION_VERSION = "20240112"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "Z940",  # Kidney transplant status
    }

class DiagnosisOfHypertension(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses and disorders of hypertension.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of hypertension.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Diagnosis of Hypertension"
    OID = "2.16.840.1.113883.3.600.263"
    DEFINITION_VERSION = "20240206"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "H35031",  # Hypertensive retinopathy, right eye
        "H35032",  # Hypertensive retinopathy, left eye
        "H35033",  # Hypertensive retinopathy, bilateral
        "I10",  # Essential (primary) hypertension
        "I110",  # Hypertensive heart disease with heart failure
        "I119",  # Hypertensive heart disease without heart failure
        "I120",  # Hypertensive chronic kidney disease with stage 5 chronic kidney disease or end stage renal disease
        "I129",  # Hypertensive chronic kidney disease with stage 1 through stage 4 chronic kidney disease, or unspecified chronic kidney disease
        "I130",  # Hypertensive heart and chronic kidney disease with heart failure and stage 1 through stage 4 chronic kidney disease, or unspecified chronic kidney disease
        "I1310",  # Hypertensive heart and chronic kidney disease without heart failure, with stage 1 through stage 4 chronic kidney disease, or unspecified chronic kidney disease
        "I1311",  # Hypertensive heart and chronic kidney disease without heart failure, with stage 5 chronic kidney disease, or end stage renal disease
        "I132",  # Hypertensive heart and chronic kidney disease with heart failure and with stage 5 chronic kidney disease, or end stage renal disease
        "I150",  # Renovascular hypertension
        "I151",  # Hypertension secondary to other renal disorders
        "I152",  # Hypertension secondary to endocrine disorders
        "I158",  # Other secondary hypertension
        "I159",  # Secondary hypertension, unspecified
        "I270",  # Primary pulmonary hypertension
        "I2720",  # Pulmonary hypertension, unspecified
        "I2721",  # Secondary pulmonary arterial hypertension
        "I2722",  # Pulmonary hypertension due to left heart disease
        "I2723",  # Pulmonary hypertension due to lung diseases and hypoxia
        "I2724",  # Chronic thromboembolic pulmonary hypertension
        "I2729",  # Other secondary pulmonary hypertension
    }

    SNOMEDCT = {
        "10725009",  # Benign hypertension (disorder)
        "10964002",  # Progressive pulmonary hypertension (disorder)
        "1201005",  # Benign essential hypertension (disorder)
        "123799005",  # Renovascular hypertension (disorder)
        "123800009",  # Goldblatt hypertension (disorder)
        "14973001",  # Renal sclerosis with hypertension (disorder)
        "194783001",  # Secondary malignant renovascular hypertension (disorder)
        "194785008",  # Secondary benign hypertension (disorder)
        "194788005",  # Hypertension secondary to endocrine disorder (disorder)
        "194791005",  # Hypertension caused by drug (disorder)
        "19769006",  # High-renin essential hypertension (disorder)
        "23130000",  # Paroxysmal hypertension (disorder)
        "233815004",  # Persistent pulmonary hypertension of the newborn (disorder)
        "233941006",  # Solitary pulmonary hypertension (disorder)
        "233942004",  # Small vessel pulmonary hypertension (disorder)
        "233943009",  # Sporadic primary pulmonary hypertension (disorder)
        "233944003",  # Familial primary pulmonary hypertension (disorder)
        "233945002",  # Pulmonary hypertension caused by drug (disorder)
        "233946001",  # Large vessel pulmonary hypertension (disorder)
        "234072000",  # Venous hypertension (disorder)
        "234073005",  # Venous hypertension of upper limb (disorder)
        "234075003",  # Venous hypertension of lower limb (disorder)
        "237279007",  # Transient hypertension of pregnancy (disorder)
        "28119000",  # Renal hypertension (disorder)
        "288250001",  # Maternal hypertension (disorder)
        "31992008",  # Secondary hypertension (disorder)
        "34742003",  # Portal hypertension (disorder)
        "371125006",  # Labile essential hypertension (disorder)
        "38341003",  # Hypertensive disorder, systemic arterial (disorder)
        "39018007",  # Renal arterial hypertension (disorder)
        "397748008",  # Hypertension with albuminuria (disorder)
        "398254007",  # Pre-eclampsia (disorder)
        "4210003",  # Ocular hypertension (disorder)
        "423674003",  # Chronic peripheral venous hypertension (disorder)
        "428575007",  # Hypertension secondary to kidney transplant (disorder)
        "429198000",  # Exertional hypertension (disorder)
        "429457004",  # Systolic essential hypertension (disorder)
        "46481004",  # Low-renin essential hypertension (disorder)
        "46764007",  # Severe pre-eclampsia (disorder)
        "48146000",  # Diastolic hypertension (disorder)
        "48194001",  # Pregnancy-induced hypertension (disorder)
        "52698002",  # Transient hypertension (disorder)
        "56218007",  # Systolic hypertension (disorder)
        "57684003",  # Parenchymal renal hypertension (disorder)
        "59621000",  # Essential hypertension (disorder)
        "59720008",  # Sustained diastolic hypertension (disorder)
        "65518004",  # Labile diastolic hypertension (disorder)
        "68267002",  # Benign intracranial hypertension (disorder)
        "70272006",  # Malignant hypertension (disorder)
        "73410007",  # Benign secondary renovascular hypertension (disorder)
        "74451002",  # Secondary diastolic hypertension (disorder)
        "762463000",  # Diastolic hypertension co-occurrent with systolic hypertension (disorder)
        "78975002",  # Malignant essential hypertension (disorder)
        "81626002",  # Malignant hypertension in obstetric context (disorder)
        "84094009",  # Rebound hypertension (disorder)
        "8762007",  # Chronic hypertension in obstetric context (disorder)
        "89242004",  # Malignant secondary hypertension (disorder)
    }

class AtherosclerosisAndPeripheralArterialDisease(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of atherosclerosis or peripheral artery disease (PAD).

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent conditions that indicate arterial blood flow is restricted.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Atherosclerosis and Peripheral Arterial Disease"
    OID = "2.16.840.1.113762.1.4.1047.21"
    DEFINITION_VERSION = "20240208"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "E0851",  # Diabetes mellitus due to underlying condition with diabetic peripheral angiopathy without gangrene
        "E0852",  # Diabetes mellitus due to underlying condition with diabetic peripheral angiopathy with gangrene
        "E0951",  # Drug or chemical induced diabetes mellitus with diabetic peripheral angiopathy without gangrene
        "E0952",  # Drug or chemical induced diabetes mellitus with diabetic peripheral angiopathy with gangrene
        "I2510",  # Atherosclerotic heart disease of native coronary artery without angina pectoris
        "I25110",  # Atherosclerotic heart disease of native coronary artery with unstable angina pectoris
        "I25111",  # Atherosclerotic heart disease of native coronary artery with angina pectoris with documented spasm
        "I25112",  # Atherosclerotic heart disease of native coronary artery with refractory angina pectoris
        "I25118",  # Atherosclerotic heart disease of native coronary artery with other forms of angina pectoris
        "I25119",  # Atherosclerotic heart disease of native coronary artery with unspecified angina pectoris
        "I252",  # Old myocardial infarction
        "I255",  # Ischemic cardiomyopathy
        "I25700",  # Atherosclerosis of coronary artery bypass graft(s), unspecified, with unstable angina pectoris
        "I25701",  # Atherosclerosis of coronary artery bypass graft(s), unspecified, with angina pectoris with documented spasm
        "I25702",  # Atherosclerosis of coronary artery bypass graft(s), unspecified, with refractory angina pectoris
        "I25708",  # Atherosclerosis of coronary artery bypass graft(s), unspecified, with other forms of angina pectoris
        "I25709",  # Atherosclerosis of coronary artery bypass graft(s), unspecified, with unspecified angina pectoris
        "I25710",  # Atherosclerosis of autologous vein coronary artery bypass graft(s) with unstable angina pectoris
        "I25711",  # Atherosclerosis of autologous vein coronary artery bypass graft(s) with angina pectoris with documented spasm
        "I25712",  # Atherosclerosis of autologous vein coronary artery bypass graft(s) with refractory angina pectoris
        "I25718",  # Atherosclerosis of autologous vein coronary artery bypass graft(s) with other forms of angina pectoris
        "I25719",  # Atherosclerosis of autologous vein coronary artery bypass graft(s) with unspecified angina pectoris
        "I25720",  # Atherosclerosis of autologous artery coronary artery bypass graft(s) with unstable angina pectoris
        "I25721",  # Atherosclerosis of autologous artery coronary artery bypass graft(s) with angina pectoris with documented spasm
        "I25722",  # Atherosclerosis of autologous artery coronary artery bypass graft(s) with refractory angina pectoris
        "I25728",  # Atherosclerosis of autologous artery coronary artery bypass graft(s) with other forms of angina pectoris
        "I25729",  # Atherosclerosis of autologous artery coronary artery bypass graft(s) with unspecified angina pectoris
        "I25730",  # Atherosclerosis of nonautologous biological coronary artery bypass graft(s) with unstable angina pectoris
        "I25731",  # Atherosclerosis of nonautologous biological coronary artery bypass graft(s) with angina pectoris with documented spasm
        "I25732",  # Atherosclerosis of nonautologous biological coronary artery bypass graft(s) with refractory angina pectoris
        "I25738",  # Atherosclerosis of nonautologous biological coronary artery bypass graft(s) with other forms of angina pectoris
        "I25739",  # Atherosclerosis of nonautologous biological coronary artery bypass graft(s) with unspecified angina pectoris
        "I25750",  # Atherosclerosis of native coronary artery of transplanted heart with unstable angina
        "I25751",  # Atherosclerosis of native coronary artery of transplanted heart with angina pectoris with documented spasm
        "I25752",  # Atherosclerosis of native coronary artery of transplanted heart with refractory angina pectoris
        "I25758",  # Atherosclerosis of native coronary artery of transplanted heart with other forms of angina pectoris
        "I25759",  # Atherosclerosis of native coronary artery of transplanted heart with unspecified angina pectoris
        "I25760",  # Atherosclerosis of bypass graft of coronary artery of transplanted heart with unstable angina
        "I25761",  # Atherosclerosis of bypass graft of coronary artery of transplanted heart with angina pectoris with documented spasm
        "I25762",  # Atherosclerosis of bypass graft of coronary artery of transplanted heart with refractory angina pectoris
        "I25768",  # Atherosclerosis of bypass graft of coronary artery of transplanted heart with other forms of angina pectoris
        "I25769",  # Atherosclerosis of bypass graft of coronary artery of transplanted heart with unspecified angina pectoris
        "I25790",  # Atherosclerosis of other coronary artery bypass graft(s) with unstable angina pectoris
        "I25791",  # Atherosclerosis of other coronary artery bypass graft(s) with angina pectoris with documented spasm
        "I25792",  # Atherosclerosis of other coronary artery bypass graft(s) with refractory angina pectoris
        "I25798",  # Atherosclerosis of other coronary artery bypass graft(s) with other forms of angina pectoris
        "I25799",  # Atherosclerosis of other coronary artery bypass graft(s) with unspecified angina pectoris
        "I25810",  # Atherosclerosis of coronary artery bypass graft(s) without angina pectoris
        "I25811",  # Atherosclerosis of native coronary artery of transplanted heart without angina pectoris
        "I25812",  # Atherosclerosis of bypass graft of coronary artery of transplanted heart without angina pectoris
        "I2582",  # Chronic total occlusion of coronary artery
        "I2583",  # Coronary atherosclerosis due to lipid rich plaque
        "I2584",  # Coronary atherosclerosis due to calcified coronary lesion
        "I2585",  # Chronic coronary microvascular dysfunction
        "I2589",  # Other forms of chronic ischemic heart disease
        "I259",  # Chronic ischemic heart disease, unspecified
        "I6501",  # Occlusion and stenosis of right vertebral artery
        "I6502",  # Occlusion and stenosis of left vertebral artery
        "I6503",  # Occlusion and stenosis of bilateral vertebral arteries
        "I6509",  # Occlusion and stenosis of unspecified vertebral artery
        "I651",  # Occlusion and stenosis of basilar artery
        "I6521",  # Occlusion and stenosis of right carotid artery
        "I6522",  # Occlusion and stenosis of left carotid artery
        "I6523",  # Occlusion and stenosis of bilateral carotid arteries
        "I6529",  # Occlusion and stenosis of unspecified carotid artery
        "I658",  # Occlusion and stenosis of other precerebral arteries
        "I659",  # Occlusion and stenosis of unspecified precerebral artery
        "I672",  # Cerebral atherosclerosis
        "I700",  # Atherosclerosis of aorta
        "I701",  # Atherosclerosis of renal artery
        "I70201",  # Unspecified atherosclerosis of native arteries of extremities, right leg
        "I70202",  # Unspecified atherosclerosis of native arteries of extremities, left leg
        "I70203",  # Unspecified atherosclerosis of native arteries of extremities, bilateral legs
        "I70208",  # Unspecified atherosclerosis of native arteries of extremities, other extremity
        "I70209",  # Unspecified atherosclerosis of native arteries of extremities, unspecified extremity
        "I70211",  # Atherosclerosis of native arteries of extremities with intermittent claudication, right leg
        "I70212",  # Atherosclerosis of native arteries of extremities with intermittent claudication, left leg
        "I70213",  # Atherosclerosis of native arteries of extremities with intermittent claudication, bilateral legs
        "I70218",  # Atherosclerosis of native arteries of extremities with intermittent claudication, other extremity
        "I70219",  # Atherosclerosis of native arteries of extremities with intermittent claudication, unspecified extremity
        "I70221",  # Atherosclerosis of native arteries of extremities with rest pain, right leg
        "I70222",  # Atherosclerosis of native arteries of extremities with rest pain, left leg
        "I70223",  # Atherosclerosis of native arteries of extremities with rest pain, bilateral legs
        "I70228",  # Atherosclerosis of native arteries of extremities with rest pain, other extremity
        "I70229",  # Atherosclerosis of native arteries of extremities with rest pain, unspecified extremity
        "I70231",  # Atherosclerosis of native arteries of right leg with ulceration of thigh
        "I70232",  # Atherosclerosis of native arteries of right leg with ulceration of calf
        "I70233",  # Atherosclerosis of native arteries of right leg with ulceration of ankle
        "I70234",  # Atherosclerosis of native arteries of right leg with ulceration of heel and midfoot
        "I70235",  # Atherosclerosis of native arteries of right leg with ulceration of other part of foot
        "I70238",  # Atherosclerosis of native arteries of right leg with ulceration of other part of lower leg
        "I70239",  # Atherosclerosis of native arteries of right leg with ulceration of unspecified site
        "I70241",  # Atherosclerosis of native arteries of left leg with ulceration of thigh
        "I70242",  # Atherosclerosis of native arteries of left leg with ulceration of calf
        "I70243",  # Atherosclerosis of native arteries of left leg with ulceration of ankle
        "I70244",  # Atherosclerosis of native arteries of left leg with ulceration of heel and midfoot
        "I70245",  # Atherosclerosis of native arteries of left leg with ulceration of other part of foot
        "I70248",  # Atherosclerosis of native arteries of left leg with ulceration of other part of lower leg
        "I70249",  # Atherosclerosis of native arteries of left leg with ulceration of unspecified site
        "I7025",  # Atherosclerosis of native arteries of other extremities with ulceration
        "I70261",  # Atherosclerosis of native arteries of extremities with gangrene, right leg
        "I70262",  # Atherosclerosis of native arteries of extremities with gangrene, left leg
        "I70263",  # Atherosclerosis of native arteries of extremities with gangrene, bilateral legs
        "I70268",  # Atherosclerosis of native arteries of extremities with gangrene, other extremity
        "I70269",  # Atherosclerosis of native arteries of extremities with gangrene, unspecified extremity
        "I70291",  # Other atherosclerosis of native arteries of extremities, right leg
        "I70292",  # Other atherosclerosis of native arteries of extremities, left leg
        "I70293",  # Other atherosclerosis of native arteries of extremities, bilateral legs
        "I70298",  # Other atherosclerosis of native arteries of extremities, other extremity
        "I70299",  # Other atherosclerosis of native arteries of extremities, unspecified extremity
        "I70301",  # Unspecified atherosclerosis of unspecified type of bypass graft(s) of the extremities, right leg
        "I70302",  # Unspecified atherosclerosis of unspecified type of bypass graft(s) of the extremities, left leg
        "I70303",  # Unspecified atherosclerosis of unspecified type of bypass graft(s) of the extremities, bilateral legs
        "I70308",  # Unspecified atherosclerosis of unspecified type of bypass graft(s) of the extremities, other extremity
        "I70309",  # Unspecified atherosclerosis of unspecified type of bypass graft(s) of the extremities, unspecified extremity
        "I70311",  # Atherosclerosis of unspecified type of bypass graft(s) of the extremities with intermittent claudication, right leg
        "I70312",  # Atherosclerosis of unspecified type of bypass graft(s) of the extremities with intermittent claudication, left leg
        "I70313",  # Atherosclerosis of unspecified type of bypass graft(s) of the extremities with intermittent claudication, bilateral legs
        "I70318",  # Atherosclerosis of unspecified type of bypass graft(s) of the extremities with intermittent claudication, other extremity
        "I70319",  # Atherosclerosis of unspecified type of bypass graft(s) of the extremities with intermittent claudication, unspecified extremity
        "I70321",  # Atherosclerosis of unspecified type of bypass graft(s) of the extremities with rest pain, right leg
        "I70322",  # Atherosclerosis of unspecified type of bypass graft(s) of the extremities with rest pain, left leg
        "I70323",  # Atherosclerosis of unspecified type of bypass graft(s) of the extremities with rest pain, bilateral legs
        "I70328",  # Atherosclerosis of unspecified type of bypass graft(s) of the extremities with rest pain, other extremity
        "I70329",  # Atherosclerosis of unspecified type of bypass graft(s) of the extremities with rest pain, unspecified extremity
        "I70331",  # Atherosclerosis of unspecified type of bypass graft(s) of the right leg with ulceration of thigh
        "I70332",  # Atherosclerosis of unspecified type of bypass graft(s) of the right leg with ulceration of calf
        "I70333",  # Atherosclerosis of unspecified type of bypass graft(s) of the right leg with ulceration of ankle
        "I70334",  # Atherosclerosis of unspecified type of bypass graft(s) of the right leg with ulceration of heel and midfoot
        "I70335",  # Atherosclerosis of unspecified type of bypass graft(s) of the right leg with ulceration of other part of foot
        "I70338",  # Atherosclerosis of unspecified type of bypass graft(s) of the right leg with ulceration of other part of lower leg
        "I70339",  # Atherosclerosis of unspecified type of bypass graft(s) of the right leg with ulceration of unspecified site
        "I70341",  # Atherosclerosis of unspecified type of bypass graft(s) of the left leg with ulceration of thigh
        "I70342",  # Atherosclerosis of unspecified type of bypass graft(s) of the left leg with ulceration of calf
        "I70343",  # Atherosclerosis of unspecified type of bypass graft(s) of the left leg with ulceration of ankle
        "I70344",  # Atherosclerosis of unspecified type of bypass graft(s) of the left leg with ulceration of heel and midfoot
        "I70345",  # Atherosclerosis of unspecified type of bypass graft(s) of the left leg with ulceration of other part of foot
        "I70348",  # Atherosclerosis of unspecified type of bypass graft(s) of the left leg with ulceration of other part of lower leg
        "I70349",  # Atherosclerosis of unspecified type of bypass graft(s) of the left leg with ulceration of unspecified site
        "I7035",  # Atherosclerosis of unspecified type of bypass graft(s) of other extremity with ulceration
        "I70361",  # Atherosclerosis of unspecified type of bypass graft(s) of the extremities with gangrene, right leg
        "I70362",  # Atherosclerosis of unspecified type of bypass graft(s) of the extremities with gangrene, left leg
        "I70363",  # Atherosclerosis of unspecified type of bypass graft(s) of the extremities with gangrene, bilateral legs
        "I70368",  # Atherosclerosis of unspecified type of bypass graft(s) of the extremities with gangrene, other extremity
        "I70369",  # Atherosclerosis of unspecified type of bypass graft(s) of the extremities with gangrene, unspecified extremity
        "I70391",  # Other atherosclerosis of unspecified type of bypass graft(s) of the extremities, right leg
        "I70392",  # Other atherosclerosis of unspecified type of bypass graft(s) of the extremities, left leg
        "I70393",  # Other atherosclerosis of unspecified type of bypass graft(s) of the extremities, bilateral legs
        "I70398",  # Other atherosclerosis of unspecified type of bypass graft(s) of the extremities, other extremity
        "I70399",  # Other atherosclerosis of unspecified type of bypass graft(s) of the extremities, unspecified extremity
        "I70401",  # Unspecified atherosclerosis of autologous vein bypass graft(s) of the extremities, right leg
        "I70402",  # Unspecified atherosclerosis of autologous vein bypass graft(s) of the extremities, left leg
        "I70403",  # Unspecified atherosclerosis of autologous vein bypass graft(s) of the extremities, bilateral legs
        "I70408",  # Unspecified atherosclerosis of autologous vein bypass graft(s) of the extremities, other extremity
        "I70409",  # Unspecified atherosclerosis of autologous vein bypass graft(s) of the extremities, unspecified extremity
        "I70411",  # Atherosclerosis of autologous vein bypass graft(s) of the extremities with intermittent claudication, right leg
        "I70412",  # Atherosclerosis of autologous vein bypass graft(s) of the extremities with intermittent claudication, left leg
        "I70413",  # Atherosclerosis of autologous vein bypass graft(s) of the extremities with intermittent claudication, bilateral legs
        "I70418",  # Atherosclerosis of autologous vein bypass graft(s) of the extremities with intermittent claudication, other extremity
        "I70419",  # Atherosclerosis of autologous vein bypass graft(s) of the extremities with intermittent claudication, unspecified extremity
        "I70421",  # Atherosclerosis of autologous vein bypass graft(s) of the extremities with rest pain, right leg
        "I70422",  # Atherosclerosis of autologous vein bypass graft(s) of the extremities with rest pain, left leg
        "I70423",  # Atherosclerosis of autologous vein bypass graft(s) of the extremities with rest pain, bilateral legs
        "I70428",  # Atherosclerosis of autologous vein bypass graft(s) of the extremities with rest pain, other extremity
        "I70429",  # Atherosclerosis of autologous vein bypass graft(s) of the extremities with rest pain, unspecified extremity
        "I70431",  # Atherosclerosis of autologous vein bypass graft(s) of the right leg with ulceration of thigh
        "I70432",  # Atherosclerosis of autologous vein bypass graft(s) of the right leg with ulceration of calf
        "I70433",  # Atherosclerosis of autologous vein bypass graft(s) of the right leg with ulceration of ankle
        "I70434",  # Atherosclerosis of autologous vein bypass graft(s) of the right leg with ulceration of heel and midfoot
        "I70435",  # Atherosclerosis of autologous vein bypass graft(s) of the right leg with ulceration of other part of foot
        "I70438",  # Atherosclerosis of autologous vein bypass graft(s) of the right leg with ulceration of other part of lower leg
        "I70439",  # Atherosclerosis of autologous vein bypass graft(s) of the right leg with ulceration of unspecified site
        "I70441",  # Atherosclerosis of autologous vein bypass graft(s) of the left leg with ulceration of thigh
        "I70442",  # Atherosclerosis of autologous vein bypass graft(s) of the left leg with ulceration of calf
        "I70443",  # Atherosclerosis of autologous vein bypass graft(s) of the left leg with ulceration of ankle
        "I70444",  # Atherosclerosis of autologous vein bypass graft(s) of the left leg with ulceration of heel and midfoot
        "I70445",  # Atherosclerosis of autologous vein bypass graft(s) of the left leg with ulceration of other part of foot
        "I70448",  # Atherosclerosis of autologous vein bypass graft(s) of the left leg with ulceration of other part of lower leg
        "I70449",  # Atherosclerosis of autologous vein bypass graft(s) of the left leg with ulceration of unspecified site
        "I7045",  # Atherosclerosis of autologous vein bypass graft(s) of other extremity with ulceration
        "I70461",  # Atherosclerosis of autologous vein bypass graft(s) of the extremities with gangrene, right leg
        "I70462",  # Atherosclerosis of autologous vein bypass graft(s) of the extremities with gangrene, left leg
        "I70463",  # Atherosclerosis of autologous vein bypass graft(s) of the extremities with gangrene, bilateral legs
        "I70468",  # Atherosclerosis of autologous vein bypass graft(s) of the extremities with gangrene, other extremity
        "I70469",  # Atherosclerosis of autologous vein bypass graft(s) of the extremities with gangrene, unspecified extremity
        "I70491",  # Other atherosclerosis of autologous vein bypass graft(s) of the extremities, right leg
        "I70492",  # Other atherosclerosis of autologous vein bypass graft(s) of the extremities, left leg
        "I70493",  # Other atherosclerosis of autologous vein bypass graft(s) of the extremities, bilateral legs
        "I70498",  # Other atherosclerosis of autologous vein bypass graft(s) of the extremities, other extremity
        "I70499",  # Other atherosclerosis of autologous vein bypass graft(s) of the extremities, unspecified extremity
        "I70501",  # Unspecified atherosclerosis of nonautologous biological bypass graft(s) of the extremities, right leg
        "I70502",  # Unspecified atherosclerosis of nonautologous biological bypass graft(s) of the extremities, left leg
        "I70503",  # Unspecified atherosclerosis of nonautologous biological bypass graft(s) of the extremities, bilateral legs
        "I70508",  # Unspecified atherosclerosis of nonautologous biological bypass graft(s) of the extremities, other extremity
        "I70509",  # Unspecified atherosclerosis of nonautologous biological bypass graft(s) of the extremities, unspecified extremity
        "I70511",  # Atherosclerosis of nonautologous biological bypass graft(s) of the extremities with intermittent claudication, right leg
        "I70512",  # Atherosclerosis of nonautologous biological bypass graft(s) of the extremities with intermittent claudication, left leg
        "I70513",  # Atherosclerosis of nonautologous biological bypass graft(s) of the extremities with intermittent claudication, bilateral legs
        "I70518",  # Atherosclerosis of nonautologous biological bypass graft(s) of the extremities with intermittent claudication, other extremity
        "I70519",  # Atherosclerosis of nonautologous biological bypass graft(s) of the extremities with intermittent claudication, unspecified extremity
        "I70521",  # Atherosclerosis of nonautologous biological bypass graft(s) of the extremities with rest pain, right leg
        "I70522",  # Atherosclerosis of nonautologous biological bypass graft(s) of the extremities with rest pain, left leg
        "I70523",  # Atherosclerosis of nonautologous biological bypass graft(s) of the extremities with rest pain, bilateral legs
        "I70528",  # Atherosclerosis of nonautologous biological bypass graft(s) of the extremities with rest pain, other extremity
        "I70529",  # Atherosclerosis of nonautologous biological bypass graft(s) of the extremities with rest pain, unspecified extremity
        "I70531",  # Atherosclerosis of nonautologous biological bypass graft(s) of the right leg with ulceration of thigh
        "I70532",  # Atherosclerosis of nonautologous biological bypass graft(s) of the right leg with ulceration of calf
        "I70533",  # Atherosclerosis of nonautologous biological bypass graft(s) of the right leg with ulceration of ankle
        "I70534",  # Atherosclerosis of nonautologous biological bypass graft(s) of the right leg with ulceration of heel and midfoot
        "I70535",  # Atherosclerosis of nonautologous biological bypass graft(s) of the right leg with ulceration of other part of foot
        "I70538",  # Atherosclerosis of nonautologous biological bypass graft(s) of the right leg with ulceration of other part of lower leg
        "I70539",  # Atherosclerosis of nonautologous biological bypass graft(s) of the right leg with ulceration of unspecified site
        "I70541",  # Atherosclerosis of nonautologous biological bypass graft(s) of the left leg with ulceration of thigh
        "I70542",  # Atherosclerosis of nonautologous biological bypass graft(s) of the left leg with ulceration of calf
        "I70543",  # Atherosclerosis of nonautologous biological bypass graft(s) of the left leg with ulceration of ankle
        "I70544",  # Atherosclerosis of nonautologous biological bypass graft(s) of the left leg with ulceration of heel and midfoot
        "I70545",  # Atherosclerosis of nonautologous biological bypass graft(s) of the left leg with ulceration of other part of foot
        "I70548",  # Atherosclerosis of nonautologous biological bypass graft(s) of the left leg with ulceration of other part of lower leg
        "I70549",  # Atherosclerosis of nonautologous biological bypass graft(s) of the left leg with ulceration of unspecified site
        "I7055",  # Atherosclerosis of nonautologous biological bypass graft(s) of other extremity with ulceration
        "I70561",  # Atherosclerosis of nonautologous biological bypass graft(s) of the extremities with gangrene, right leg
        "I70562",  # Atherosclerosis of nonautologous biological bypass graft(s) of the extremities with gangrene, left leg
        "I70563",  # Atherosclerosis of nonautologous biological bypass graft(s) of the extremities with gangrene, bilateral legs
        "I70568",  # Atherosclerosis of nonautologous biological bypass graft(s) of the extremities with gangrene, other extremity
        "I70569",  # Atherosclerosis of nonautologous biological bypass graft(s) of the extremities with gangrene, unspecified extremity
        "I70591",  # Other atherosclerosis of nonautologous biological bypass graft(s) of the extremities, right leg
        "I70592",  # Other atherosclerosis of nonautologous biological bypass graft(s) of the extremities, left leg
        "I70593",  # Other atherosclerosis of nonautologous biological bypass graft(s) of the extremities, bilateral legs
        "I70598",  # Other atherosclerosis of nonautologous biological bypass graft(s) of the extremities, other extremity
        "I70599",  # Other atherosclerosis of nonautologous biological bypass graft(s) of the extremities, unspecified extremity
        "I70601",  # Unspecified atherosclerosis of nonbiological bypass graft(s) of the extremities, right leg
        "I70602",  # Unspecified atherosclerosis of nonbiological bypass graft(s) of the extremities, left leg
        "I70603",  # Unspecified atherosclerosis of nonbiological bypass graft(s) of the extremities, bilateral legs
        "I70608",  # Unspecified atherosclerosis of nonbiological bypass graft(s) of the extremities, other extremity
        "I70609",  # Unspecified atherosclerosis of nonbiological bypass graft(s) of the extremities, unspecified extremity
        "I70611",  # Atherosclerosis of nonbiological bypass graft(s) of the extremities with intermittent claudication, right leg
        "I70612",  # Atherosclerosis of nonbiological bypass graft(s) of the extremities with intermittent claudication, left leg
        "I70613",  # Atherosclerosis of nonbiological bypass graft(s) of the extremities with intermittent claudication, bilateral legs
        "I70618",  # Atherosclerosis of nonbiological bypass graft(s) of the extremities with intermittent claudication, other extremity
        "I70619",  # Atherosclerosis of nonbiological bypass graft(s) of the extremities with intermittent claudication, unspecified extremity
        "I70621",  # Atherosclerosis of nonbiological bypass graft(s) of the extremities with rest pain, right leg
        "I70622",  # Atherosclerosis of nonbiological bypass graft(s) of the extremities with rest pain, left leg
        "I70623",  # Atherosclerosis of nonbiological bypass graft(s) of the extremities with rest pain, bilateral legs
        "I70628",  # Atherosclerosis of nonbiological bypass graft(s) of the extremities with rest pain, other extremity
        "I70629",  # Atherosclerosis of nonbiological bypass graft(s) of the extremities with rest pain, unspecified extremity
        "I70631",  # Atherosclerosis of nonbiological bypass graft(s) of the right leg with ulceration of thigh
        "I70632",  # Atherosclerosis of nonbiological bypass graft(s) of the right leg with ulceration of calf
        "I70633",  # Atherosclerosis of nonbiological bypass graft(s) of the right leg with ulceration of ankle
        "I70634",  # Atherosclerosis of nonbiological bypass graft(s) of the right leg with ulceration of heel and midfoot
        "I70635",  # Atherosclerosis of nonbiological bypass graft(s) of the right leg with ulceration of other part of foot
        "I70638",  # Atherosclerosis of nonbiological bypass graft(s) of the right leg with ulceration of other part of lower leg
        "I70639",  # Atherosclerosis of nonbiological bypass graft(s) of the right leg with ulceration of unspecified site
        "I70641",  # Atherosclerosis of nonbiological bypass graft(s) of the left leg with ulceration of thigh
        "I70642",  # Atherosclerosis of nonbiological bypass graft(s) of the left leg with ulceration of calf
        "I70643",  # Atherosclerosis of nonbiological bypass graft(s) of the left leg with ulceration of ankle
        "I70644",  # Atherosclerosis of nonbiological bypass graft(s) of the left leg with ulceration of heel and midfoot
        "I70645",  # Atherosclerosis of nonbiological bypass graft(s) of the left leg with ulceration of other part of foot
        "I70648",  # Atherosclerosis of nonbiological bypass graft(s) of the left leg with ulceration of other part of lower leg
        "I70649",  # Atherosclerosis of nonbiological bypass graft(s) of the left leg with ulceration of unspecified site
        "I7065",  # Atherosclerosis of nonbiological bypass graft(s) of other extremity with ulceration
        "I70661",  # Atherosclerosis of nonbiological bypass graft(s) of the extremities with gangrene, right leg
        "I70662",  # Atherosclerosis of nonbiological bypass graft(s) of the extremities with gangrene, left leg
        "I70663",  # Atherosclerosis of nonbiological bypass graft(s) of the extremities with gangrene, bilateral legs
        "I70668",  # Atherosclerosis of nonbiological bypass graft(s) of the extremities with gangrene, other extremity
        "I70669",  # Atherosclerosis of nonbiological bypass graft(s) of the extremities with gangrene, unspecified extremity
        "I70691",  # Other atherosclerosis of nonbiological bypass graft(s) of the extremities, right leg
        "I70692",  # Other atherosclerosis of nonbiological bypass graft(s) of the extremities, left leg
        "I70693",  # Other atherosclerosis of nonbiological bypass graft(s) of the extremities, bilateral legs
        "I70698",  # Other atherosclerosis of nonbiological bypass graft(s) of the extremities, other extremity
        "I70699",  # Other atherosclerosis of nonbiological bypass graft(s) of the extremities, unspecified extremity
        "I70701",  # Unspecified atherosclerosis of other type of bypass graft(s) of the extremities, right leg
        "I70702",  # Unspecified atherosclerosis of other type of bypass graft(s) of the extremities, left leg
        "I70703",  # Unspecified atherosclerosis of other type of bypass graft(s) of the extremities, bilateral legs
        "I70708",  # Unspecified atherosclerosis of other type of bypass graft(s) of the extremities, other extremity
        "I70709",  # Unspecified atherosclerosis of other type of bypass graft(s) of the extremities, unspecified extremity
        "I70711",  # Atherosclerosis of other type of bypass graft(s) of the extremities with intermittent claudication, right leg
        "I70712",  # Atherosclerosis of other type of bypass graft(s) of the extremities with intermittent claudication, left leg
        "I70713",  # Atherosclerosis of other type of bypass graft(s) of the extremities with intermittent claudication, bilateral legs
        "I70718",  # Atherosclerosis of other type of bypass graft(s) of the extremities with intermittent claudication, other extremity
        "I70719",  # Atherosclerosis of other type of bypass graft(s) of the extremities with intermittent claudication, unspecified extremity
        "I70721",  # Atherosclerosis of other type of bypass graft(s) of the extremities with rest pain, right leg
        "I70722",  # Atherosclerosis of other type of bypass graft(s) of the extremities with rest pain, left leg
        "I70723",  # Atherosclerosis of other type of bypass graft(s) of the extremities with rest pain, bilateral legs
        "I70728",  # Atherosclerosis of other type of bypass graft(s) of the extremities with rest pain, other extremity
        "I70729",  # Atherosclerosis of other type of bypass graft(s) of the extremities with rest pain, unspecified extremity
        "I70731",  # Atherosclerosis of other type of bypass graft(s) of the right leg with ulceration of thigh
        "I70732",  # Atherosclerosis of other type of bypass graft(s) of the right leg with ulceration of calf
        "I70733",  # Atherosclerosis of other type of bypass graft(s) of the right leg with ulceration of ankle
        "I70734",  # Atherosclerosis of other type of bypass graft(s) of the right leg with ulceration of heel and midfoot
        "I70735",  # Atherosclerosis of other type of bypass graft(s) of the right leg with ulceration of other part of foot
        "I70738",  # Atherosclerosis of other type of bypass graft(s) of the right leg with ulceration of other part of lower leg
        "I70739",  # Atherosclerosis of other type of bypass graft(s) of the right leg with ulceration of unspecified site
        "I70741",  # Atherosclerosis of other type of bypass graft(s) of the left leg with ulceration of thigh
        "I70742",  # Atherosclerosis of other type of bypass graft(s) of the left leg with ulceration of calf
        "I70743",  # Atherosclerosis of other type of bypass graft(s) of the left leg with ulceration of ankle
        "I70744",  # Atherosclerosis of other type of bypass graft(s) of the left leg with ulceration of heel and midfoot
        "I70745",  # Atherosclerosis of other type of bypass graft(s) of the left leg with ulceration of other part of foot
        "I70748",  # Atherosclerosis of other type of bypass graft(s) of the left leg with ulceration of other part of lower leg
        "I70749",  # Atherosclerosis of other type of bypass graft(s) of the left leg with ulceration of unspecified site
        "I7075",  # Atherosclerosis of other type of bypass graft(s) of other extremity with ulceration
        "I70761",  # Atherosclerosis of other type of bypass graft(s) of the extremities with gangrene, right leg
        "I70762",  # Atherosclerosis of other type of bypass graft(s) of the extremities with gangrene, left leg
        "I70763",  # Atherosclerosis of other type of bypass graft(s) of the extremities with gangrene, bilateral legs
        "I70768",  # Atherosclerosis of other type of bypass graft(s) of the extremities with gangrene, other extremity
        "I70769",  # Atherosclerosis of other type of bypass graft(s) of the extremities with gangrene, unspecified extremity
        "I70791",  # Other atherosclerosis of other type of bypass graft(s) of the extremities, right leg
        "I70792",  # Other atherosclerosis of other type of bypass graft(s) of the extremities, left leg
        "I70793",  # Other atherosclerosis of other type of bypass graft(s) of the extremities, bilateral legs
        "I70798",  # Other atherosclerosis of other type of bypass graft(s) of the extremities, other extremity
        "I70799",  # Other atherosclerosis of other type of bypass graft(s) of the extremities, unspecified extremity
        "I708",  # Atherosclerosis of other arteries
        "I7090",  # Unspecified atherosclerosis
        "I7091",  # Generalized atherosclerosis
        "I7092",  # Chronic total occlusion of artery of the extremities
    }

    SNOMEDCT = {
        "127014009",  # Peripheral angiopathy due to diabetes mellitus (disorder)
        "1303693000",  # Atherosclerosis of right coronary artery (disorder)
        "13954005",  # Ischemic ulcer (disorder)
        "233955003",  # Abdominal aortic atherosclerosis (disorder)
        "233956002",  # Aortoiliac atherosclerosis (disorder)
        "238794007",  # Ischemic foot ulcer (disorder)
        "284871000119105",  # Atherosclerosis of left carotid artery (disorder)
        "284881000119108",  # Atherosclerosis of right carotid artery (disorder)
        "300920004",  # Carotid atherosclerosis (disorder)
        "39823006",  # Generalized atherosclerosis (disorder)
        "399957001",  # Peripheral arterial occlusive disease (disorder)
        "413838009",  # Chronic ischemic heart disease (disorder)
        "421365002",  # Peripheral circulatory disorder due to type 1 diabetes mellitus (disorder)
        "421895002",  # Peripheral vascular disorder due to diabetes mellitus (disorder)
        "422166005",  # Peripheral circulatory disorder due to type 2 diabetes mellitus (disorder)
        "429768000",  # Ischemic ulcer of toe (disorder)
        "441574008",  # Atherosclerosis of artery (disorder)
        "442439008",  # Atherosclerosis of bypass graft of limb (disorder)
        "442693003",  # Atherosclerosis of autologous vein bypass graft of limb (disorder)
        "442701004",  # Atherosclerosis of nonautologous biological bypass graft of limb (disorder)
        "442735001",  # Atherosclerosis of nonautologous bypass graft of limb (disorder)
        "443502000",  # Atherosclerosis of coronary artery (disorder)
        "45281005",  # Atherosclerosis of renal artery (disorder)
        "51274000",  # Atherosclerosis of arteries of the extremities (disorder)
        "55382008",  # Cerebral atherosclerosis (disorder)
        "63491006",  # Intermittent claudication (finding)
        "709584004",  # Atherosclerosis of bypass graft of lower limb (disorder)
        "709585003",  # Atherosclerosis of nonautologous biological bypass graft of lower limb (disorder)
        "792843009",  # Gangrene of limb due to atherosclerosis of artery of limb (disorder)
        "792844003",  # Limb pain at rest due to atherosclerosis of artery of lower limb (disorder)
        "792845002",  # Intermittent claudication due to atherosclerosis of artery of limb (finding)
        "8001000119106",  # Atherosclerosis of aortoiliac bypass graft (disorder)
        "81817003",  # Atherosclerosis of aorta (disorder)
    }

class Breastfeeding(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for active breastfeeding.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of lactation and the actual finding or intention of breastfeeding or nursing.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Breastfeeding"
    OID = "2.16.840.1.113762.1.4.1047.73"
    DEFINITION_VERSION = "20210220"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "O9103",  # Infection of nipple associated with lactation
        "O9113",  # Abscess of breast associated with lactation
        "O9123",  # Nonpurulent mastitis associated with lactation
        "O9203",  # Retracted nipple associated with lactation
        "O9213",  # Cracked nipple associated with lactation
        "O925",  # Suppressed lactation
        "O9270",  # Unspecified disorders of lactation
        "O9279",  # Other disorders of lactation
        "Z391",  # Encounter for care and examination of lactating mother
    }

    SNOMEDCT = {
        "1258998001",  # Partial maternal breastfeeding (finding)
        "1258999009",  # Maternal exclusive breastfeeding (finding)
        "1260078007",  # Maternal breastfeeding (finding)
        "169643005",  # Feeding intention - breast (finding)
        "169743001",  # Breastfeeding with supplement (finding)
        "169745008",  # Breastfeeding started (finding)
        "199004",  # Decreased lactation (finding)
        "200430001",  # Breastfeeding painful (finding)
        "225604004",  # Lactation established (finding)
        "247415009",  # Painful lactation (finding)
        "289084000",  # Difficulty performing breast-feeding (finding)
        "290122005",  # Finding of lactation (finding)
        "290124006",  # Finding of quantity of lactation (finding)
        "290126008",  # Finding of measures of lactation (finding)
        "58219009",  # Lactation problem (finding)
        "69840006",  # Normal breast feeding (finding)
        "82374005",  # Lactation normal (finding)
        "866041003",  # Mother currently breastfeeding (situation)
    }

class CerebrovascularDiseaseStrokeOrTia(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for stroke and high risk diagnoses associated with stroke, including transient ischemic attack (TIA) and generalized ischemic cerebrovascular disease, which would indicate a patient has atherosclerotic cardiovascular disease (ASCVD) in relation to a cerebrovascular event.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of atherosclerotic cardiovascular disease (ASCVD) of cerebrovascular origin.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Cerebrovascular Disease Stroke or TIA"
    OID = "2.16.840.1.113762.1.4.1047.44"
    DEFINITION_VERSION = "20240208"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "G450",  # Vertebro-basilar artery syndrome
        "G451",  # Carotid artery syndrome (hemispheric)
        "G452",  # Multiple and bilateral precerebral artery syndromes
        "G458",  # Other transient cerebral ischemic attacks and related syndromes
        "G459",  # Transient cerebral ischemic attack, unspecified
        "G460",  # Middle cerebral artery syndrome
        "G461",  # Anterior cerebral artery syndrome
        "G462",  # Posterior cerebral artery syndrome
        "G463",  # Brain stem stroke syndrome
        "G464",  # Cerebellar stroke syndrome
        "G465",  # Pure motor lacunar syndrome
        "G466",  # Pure sensory lacunar syndrome
        "G467",  # Other lacunar syndromes
        "G468",  # Other vascular syndromes of brain in cerebrovascular diseases
        "I6300",  # Cerebral infarction due to thrombosis of unspecified precerebral artery
        "I63011",  # Cerebral infarction due to thrombosis of right vertebral artery
        "I63012",  # Cerebral infarction due to thrombosis of left vertebral artery
        "I63013",  # Cerebral infarction due to thrombosis of bilateral vertebral arteries
        "I63019",  # Cerebral infarction due to thrombosis of unspecified vertebral artery
        "I6302",  # Cerebral infarction due to thrombosis of basilar artery
        "I63031",  # Cerebral infarction due to thrombosis of right carotid artery
        "I63032",  # Cerebral infarction due to thrombosis of left carotid artery
        "I63033",  # Cerebral infarction due to thrombosis of bilateral carotid arteries
        "I63039",  # Cerebral infarction due to thrombosis of unspecified carotid artery
        "I6309",  # Cerebral infarction due to thrombosis of other precerebral artery
        "I6310",  # Cerebral infarction due to embolism of unspecified precerebral artery
        "I63111",  # Cerebral infarction due to embolism of right vertebral artery
        "I63112",  # Cerebral infarction due to embolism of left vertebral artery
        "I63113",  # Cerebral infarction due to embolism of bilateral vertebral arteries
        "I63119",  # Cerebral infarction due to embolism of unspecified vertebral artery
        "I6312",  # Cerebral infarction due to embolism of basilar artery
        "I63131",  # Cerebral infarction due to embolism of right carotid artery
        "I63132",  # Cerebral infarction due to embolism of left carotid artery
        "I63133",  # Cerebral infarction due to embolism of bilateral carotid arteries
        "I63139",  # Cerebral infarction due to embolism of unspecified carotid artery
        "I6319",  # Cerebral infarction due to embolism of other precerebral artery
        "I6320",  # Cerebral infarction due to unspecified occlusion or stenosis of unspecified precerebral arteries
        "I63211",  # Cerebral infarction due to unspecified occlusion or stenosis of right vertebral artery
        "I63212",  # Cerebral infarction due to unspecified occlusion or stenosis of left vertebral artery
        "I63213",  # Cerebral infarction due to unspecified occlusion or stenosis of bilateral vertebral arteries
        "I63219",  # Cerebral infarction due to unspecified occlusion or stenosis of unspecified vertebral artery
        "I6322",  # Cerebral infarction due to unspecified occlusion or stenosis of basilar artery
        "I63231",  # Cerebral infarction due to unspecified occlusion or stenosis of right carotid arteries
        "I63232",  # Cerebral infarction due to unspecified occlusion or stenosis of left carotid arteries
        "I63233",  # Cerebral infarction due to unspecified occlusion or stenosis of bilateral carotid arteries
        "I63239",  # Cerebral infarction due to unspecified occlusion or stenosis of unspecified carotid artery
        "I6329",  # Cerebral infarction due to unspecified occlusion or stenosis of other precerebral arteries
        "I6330",  # Cerebral infarction due to thrombosis of unspecified cerebral artery
        "I63311",  # Cerebral infarction due to thrombosis of right middle cerebral artery
        "I63312",  # Cerebral infarction due to thrombosis of left middle cerebral artery
        "I63313",  # Cerebral infarction due to thrombosis of bilateral middle cerebral arteries
        "I63319",  # Cerebral infarction due to thrombosis of unspecified middle cerebral artery
        "I63321",  # Cerebral infarction due to thrombosis of right anterior cerebral artery
        "I63322",  # Cerebral infarction due to thrombosis of left anterior cerebral artery
        "I63323",  # Cerebral infarction due to thrombosis of bilateral anterior cerebral arteries
        "I63329",  # Cerebral infarction due to thrombosis of unspecified anterior cerebral artery
        "I63331",  # Cerebral infarction due to thrombosis of right posterior cerebral artery
        "I63332",  # Cerebral infarction due to thrombosis of left posterior cerebral artery
        "I63333",  # Cerebral infarction due to thrombosis of bilateral posterior cerebral arteries
        "I63339",  # Cerebral infarction due to thrombosis of unspecified posterior cerebral artery
        "I63341",  # Cerebral infarction due to thrombosis of right cerebellar artery
        "I63342",  # Cerebral infarction due to thrombosis of left cerebellar artery
        "I63343",  # Cerebral infarction due to thrombosis of bilateral cerebellar arteries
        "I63349",  # Cerebral infarction due to thrombosis of unspecified cerebellar artery
        "I6339",  # Cerebral infarction due to thrombosis of other cerebral artery
        "I6340",  # Cerebral infarction due to embolism of unspecified cerebral artery
        "I63411",  # Cerebral infarction due to embolism of right middle cerebral artery
        "I63412",  # Cerebral infarction due to embolism of left middle cerebral artery
        "I63413",  # Cerebral infarction due to embolism of bilateral middle cerebral arteries
        "I63419",  # Cerebral infarction due to embolism of unspecified middle cerebral artery
        "I63421",  # Cerebral infarction due to embolism of right anterior cerebral artery
        "I63422",  # Cerebral infarction due to embolism of left anterior cerebral artery
        "I63423",  # Cerebral infarction due to embolism of bilateral anterior cerebral arteries
        "I63429",  # Cerebral infarction due to embolism of unspecified anterior cerebral artery
        "I63431",  # Cerebral infarction due to embolism of right posterior cerebral artery
        "I63432",  # Cerebral infarction due to embolism of left posterior cerebral artery
        "I63433",  # Cerebral infarction due to embolism of bilateral posterior cerebral arteries
        "I63439",  # Cerebral infarction due to embolism of unspecified posterior cerebral artery
        "I63441",  # Cerebral infarction due to embolism of right cerebellar artery
        "I63442",  # Cerebral infarction due to embolism of left cerebellar artery
        "I63443",  # Cerebral infarction due to embolism of bilateral cerebellar arteries
        "I63449",  # Cerebral infarction due to embolism of unspecified cerebellar artery
        "I6349",  # Cerebral infarction due to embolism of other cerebral artery
        "I6350",  # Cerebral infarction due to unspecified occlusion or stenosis of unspecified cerebral artery
        "I63511",  # Cerebral infarction due to unspecified occlusion or stenosis of right middle cerebral artery
        "I63512",  # Cerebral infarction due to unspecified occlusion or stenosis of left middle cerebral artery
        "I63513",  # Cerebral infarction due to unspecified occlusion or stenosis of bilateral middle cerebral arteries
        "I63519",  # Cerebral infarction due to unspecified occlusion or stenosis of unspecified middle cerebral artery
        "I63521",  # Cerebral infarction due to unspecified occlusion or stenosis of right anterior cerebral artery
        "I63522",  # Cerebral infarction due to unspecified occlusion or stenosis of left anterior cerebral artery
        "I63523",  # Cerebral infarction due to unspecified occlusion or stenosis of bilateral anterior cerebral arteries
        "I63529",  # Cerebral infarction due to unspecified occlusion or stenosis of unspecified anterior cerebral artery
        "I63531",  # Cerebral infarction due to unspecified occlusion or stenosis of right posterior cerebral artery
        "I63532",  # Cerebral infarction due to unspecified occlusion or stenosis of left posterior cerebral artery
        "I63533",  # Cerebral infarction due to unspecified occlusion or stenosis of bilateral posterior cerebral arteries
        "I63539",  # Cerebral infarction due to unspecified occlusion or stenosis of unspecified posterior cerebral artery
        "I63541",  # Cerebral infarction due to unspecified occlusion or stenosis of right cerebellar artery
        "I63542",  # Cerebral infarction due to unspecified occlusion or stenosis of left cerebellar artery
        "I63543",  # Cerebral infarction due to unspecified occlusion or stenosis of bilateral cerebellar arteries
        "I63549",  # Cerebral infarction due to unspecified occlusion or stenosis of unspecified cerebellar artery
        "I6359",  # Cerebral infarction due to unspecified occlusion or stenosis of other cerebral artery
        "I636",  # Cerebral infarction due to cerebral venous thrombosis, nonpyogenic
        "I6381",  # Other cerebral infarction due to occlusion or stenosis of small artery
        "I6389",  # Other cerebral infarction
        "I639",  # Cerebral infarction, unspecified
        "I6900",  # Unspecified sequelae of nontraumatic subarachnoid hemorrhage
        "I69010",  # Attention and concentration deficit following nontraumatic subarachnoid hemorrhage
        "I69011",  # Memory deficit following nontraumatic subarachnoid hemorrhage
        "I69012",  # Visuospatial deficit and spatial neglect following nontraumatic subarachnoid hemorrhage
        "I69013",  # Psychomotor deficit following nontraumatic subarachnoid hemorrhage
        "I69014",  # Frontal lobe and executive function deficit following nontraumatic subarachnoid hemorrhage
        "I69015",  # Cognitive social or emotional deficit following nontraumatic subarachnoid hemorrhage
        "I69018",  # Other symptoms and signs involving cognitive functions following nontraumatic subarachnoid hemorrhage
        "I69019",  # Unspecified symptoms and signs involving cognitive functions following nontraumatic subarachnoid hemorrhage
        "I69020",  # Aphasia following nontraumatic subarachnoid hemorrhage
        "I69021",  # Dysphasia following nontraumatic subarachnoid hemorrhage
        "I69022",  # Dysarthria following nontraumatic subarachnoid hemorrhage
        "I69023",  # Fluency disorder following nontraumatic subarachnoid hemorrhage
        "I69028",  # Other speech and language deficits following nontraumatic subarachnoid hemorrhage
        "I69031",  # Monoplegia of upper limb following nontraumatic subarachnoid hemorrhage affecting right dominant side
        "I69032",  # Monoplegia of upper limb following nontraumatic subarachnoid hemorrhage affecting left dominant side
        "I69033",  # Monoplegia of upper limb following nontraumatic subarachnoid hemorrhage affecting right non-dominant side
        "I69034",  # Monoplegia of upper limb following nontraumatic subarachnoid hemorrhage affecting left non-dominant side
        "I69039",  # Monoplegia of upper limb following nontraumatic subarachnoid hemorrhage affecting unspecified side
        "I69041",  # Monoplegia of lower limb following nontraumatic subarachnoid hemorrhage affecting right dominant side
        "I69042",  # Monoplegia of lower limb following nontraumatic subarachnoid hemorrhage affecting left dominant side
        "I69043",  # Monoplegia of lower limb following nontraumatic subarachnoid hemorrhage affecting right non-dominant side
        "I69044",  # Monoplegia of lower limb following nontraumatic subarachnoid hemorrhage affecting left non-dominant side
        "I69049",  # Monoplegia of lower limb following nontraumatic subarachnoid hemorrhage affecting unspecified side
        "I69051",  # Hemiplegia and hemiparesis following nontraumatic subarachnoid hemorrhage affecting right dominant side
        "I69052",  # Hemiplegia and hemiparesis following nontraumatic subarachnoid hemorrhage affecting left dominant side
        "I69053",  # Hemiplegia and hemiparesis following nontraumatic subarachnoid hemorrhage affecting right non-dominant side
        "I69054",  # Hemiplegia and hemiparesis following nontraumatic subarachnoid hemorrhage affecting left non-dominant side
        "I69059",  # Hemiplegia and hemiparesis following nontraumatic subarachnoid hemorrhage affecting unspecified side
        "I69061",  # Other paralytic syndrome following nontraumatic subarachnoid hemorrhage affecting right dominant side
        "I69062",  # Other paralytic syndrome following nontraumatic subarachnoid hemorrhage affecting left dominant side
        "I69063",  # Other paralytic syndrome following nontraumatic subarachnoid hemorrhage affecting right non-dominant side
        "I69064",  # Other paralytic syndrome following nontraumatic subarachnoid hemorrhage affecting left non-dominant side
        "I69065",  # Other paralytic syndrome following nontraumatic subarachnoid hemorrhage, bilateral
        "I69069",  # Other paralytic syndrome following nontraumatic subarachnoid hemorrhage affecting unspecified side
        "I69090",  # Apraxia following nontraumatic subarachnoid hemorrhage
        "I69091",  # Dysphagia following nontraumatic subarachnoid hemorrhage
        "I69092",  # Facial weakness following nontraumatic subarachnoid hemorrhage
        "I69093",  # Ataxia following nontraumatic subarachnoid hemorrhage
        "I69098",  # Other sequelae following nontraumatic subarachnoid hemorrhage
        "I6910",  # Unspecified sequelae of nontraumatic intracerebral hemorrhage
        "I69110",  # Attention and concentration deficit following nontraumatic intracerebral hemorrhage
        "I69111",  # Memory deficit following nontraumatic intracerebral hemorrhage
        "I69112",  # Visuospatial deficit and spatial neglect following nontraumatic intracerebral hemorrhage
        "I69113",  # Psychomotor deficit following nontraumatic intracerebral hemorrhage
        "I69114",  # Frontal lobe and executive function deficit following nontraumatic intracerebral hemorrhage
        "I69115",  # Cognitive social or emotional deficit following nontraumatic intracerebral hemorrhage
        "I69118",  # Other symptoms and signs involving cognitive functions following nontraumatic intracerebral hemorrhage
        "I69119",  # Unspecified symptoms and signs involving cognitive functions following nontraumatic intracerebral hemorrhage
        "I69120",  # Aphasia following nontraumatic intracerebral hemorrhage
        "I69121",  # Dysphasia following nontraumatic intracerebral hemorrhage
        "I69122",  # Dysarthria following nontraumatic intracerebral hemorrhage
        "I69123",  # Fluency disorder following nontraumatic intracerebral hemorrhage
        "I69128",  # Other speech and language deficits following nontraumatic intracerebral hemorrhage
        "I69131",  # Monoplegia of upper limb following nontraumatic intracerebral hemorrhage affecting right dominant side
        "I69132",  # Monoplegia of upper limb following nontraumatic intracerebral hemorrhage affecting left dominant side
        "I69133",  # Monoplegia of upper limb following nontraumatic intracerebral hemorrhage affecting right non-dominant side
        "I69134",  # Monoplegia of upper limb following nontraumatic intracerebral hemorrhage affecting left non-dominant side
        "I69139",  # Monoplegia of upper limb following nontraumatic intracerebral hemorrhage affecting unspecified side
        "I69141",  # Monoplegia of lower limb following nontraumatic intracerebral hemorrhage affecting right dominant side
        "I69142",  # Monoplegia of lower limb following nontraumatic intracerebral hemorrhage affecting left dominant side
        "I69143",  # Monoplegia of lower limb following nontraumatic intracerebral hemorrhage affecting right non-dominant side
        "I69144",  # Monoplegia of lower limb following nontraumatic intracerebral hemorrhage affecting left non-dominant side
        "I69149",  # Monoplegia of lower limb following nontraumatic intracerebral hemorrhage affecting unspecified side
        "I69151",  # Hemiplegia and hemiparesis following nontraumatic intracerebral hemorrhage affecting right dominant side
        "I69152",  # Hemiplegia and hemiparesis following nontraumatic intracerebral hemorrhage affecting left dominant side
        "I69153",  # Hemiplegia and hemiparesis following nontraumatic intracerebral hemorrhage affecting right non-dominant side
        "I69154",  # Hemiplegia and hemiparesis following nontraumatic intracerebral hemorrhage affecting left non-dominant side
        "I69159",  # Hemiplegia and hemiparesis following nontraumatic intracerebral hemorrhage affecting unspecified side
        "I69161",  # Other paralytic syndrome following nontraumatic intracerebral hemorrhage affecting right dominant side
        "I69162",  # Other paralytic syndrome following nontraumatic intracerebral hemorrhage affecting left dominant side
        "I69163",  # Other paralytic syndrome following nontraumatic intracerebral hemorrhage affecting right non-dominant side
        "I69164",  # Other paralytic syndrome following nontraumatic intracerebral hemorrhage affecting left non-dominant side
        "I69165",  # Other paralytic syndrome following nontraumatic intracerebral hemorrhage, bilateral
        "I69169",  # Other paralytic syndrome following nontraumatic intracerebral hemorrhage affecting unspecified side
        "I69190",  # Apraxia following nontraumatic intracerebral hemorrhage
        "I69191",  # Dysphagia following nontraumatic intracerebral hemorrhage
        "I69192",  # Facial weakness following nontraumatic intracerebral hemorrhage
        "I69193",  # Ataxia following nontraumatic intracerebral hemorrhage
        "I69198",  # Other sequelae of nontraumatic intracerebral hemorrhage
        "I6920",  # Unspecified sequelae of other nontraumatic intracranial hemorrhage
        "I69210",  # Attention and concentration deficit following other nontraumatic intracranial hemorrhage
        "I69211",  # Memory deficit following other nontraumatic intracranial hemorrhage
        "I69212",  # Visuospatial deficit and spatial neglect following other nontraumatic intracranial hemorrhage
        "I69213",  # Psychomotor deficit following other nontraumatic intracranial hemorrhage
        "I69214",  # Frontal lobe and executive function deficit following other nontraumatic intracranial hemorrhage
        "I69215",  # Cognitive social or emotional deficit following other nontraumatic intracranial hemorrhage
        "I69218",  # Other symptoms and signs involving cognitive functions following other nontraumatic intracranial hemorrhage
        "I69219",  # Unspecified symptoms and signs involving cognitive functions following other nontraumatic intracranial hemorrhage
        "I69220",  # Aphasia following other nontraumatic intracranial hemorrhage
        "I69221",  # Dysphasia following other nontraumatic intracranial hemorrhage
        "I69222",  # Dysarthria following other nontraumatic intracranial hemorrhage
        "I69223",  # Fluency disorder following other nontraumatic intracranial hemorrhage
        "I69228",  # Other speech and language deficits following other nontraumatic intracranial hemorrhage
        "I69231",  # Monoplegia of upper limb following other nontraumatic intracranial hemorrhage affecting right dominant side
        "I69232",  # Monoplegia of upper limb following other nontraumatic intracranial hemorrhage affecting left dominant side
        "I69233",  # Monoplegia of upper limb following other nontraumatic intracranial hemorrhage affecting right non-dominant side
        "I69234",  # Monoplegia of upper limb following other nontraumatic intracranial hemorrhage affecting left non-dominant side
        "I69239",  # Monoplegia of upper limb following other nontraumatic intracranial hemorrhage affecting unspecified side
        "I69241",  # Monoplegia of lower limb following other nontraumatic intracranial hemorrhage affecting right dominant side
        "I69242",  # Monoplegia of lower limb following other nontraumatic intracranial hemorrhage affecting left dominant side
        "I69243",  # Monoplegia of lower limb following other nontraumatic intracranial hemorrhage affecting right non-dominant side
        "I69244",  # Monoplegia of lower limb following other nontraumatic intracranial hemorrhage affecting left non-dominant side
        "I69249",  # Monoplegia of lower limb following other nontraumatic intracranial hemorrhage affecting unspecified side
        "I69251",  # Hemiplegia and hemiparesis following other nontraumatic intracranial hemorrhage affecting right dominant side
        "I69252",  # Hemiplegia and hemiparesis following other nontraumatic intracranial hemorrhage affecting left dominant side
        "I69253",  # Hemiplegia and hemiparesis following other nontraumatic intracranial hemorrhage affecting right non-dominant side
        "I69254",  # Hemiplegia and hemiparesis following other nontraumatic intracranial hemorrhage affecting left non-dominant side
        "I69259",  # Hemiplegia and hemiparesis following other nontraumatic intracranial hemorrhage affecting unspecified side
        "I69261",  # Other paralytic syndrome following other nontraumatic intracranial hemorrhage affecting right dominant side
        "I69262",  # Other paralytic syndrome following other nontraumatic intracranial hemorrhage affecting left dominant side
        "I69263",  # Other paralytic syndrome following other nontraumatic intracranial hemorrhage affecting right non-dominant side
        "I69264",  # Other paralytic syndrome following other nontraumatic intracranial hemorrhage affecting left non-dominant side
        "I69265",  # Other paralytic syndrome following other nontraumatic intracranial hemorrhage, bilateral
        "I69269",  # Other paralytic syndrome following other nontraumatic intracranial hemorrhage affecting unspecified side
        "I69290",  # Apraxia following other nontraumatic intracranial hemorrhage
        "I69291",  # Dysphagia following other nontraumatic intracranial hemorrhage
        "I69292",  # Facial weakness following other nontraumatic intracranial hemorrhage
        "I69293",  # Ataxia following other nontraumatic intracranial hemorrhage
        "I69298",  # Other sequelae of other nontraumatic intracranial hemorrhage
        "I6930",  # Unspecified sequelae of cerebral infarction
        "I69310",  # Attention and concentration deficit following cerebral infarction
        "I69311",  # Memory deficit following cerebral infarction
        "I69312",  # Visuospatial deficit and spatial neglect following cerebral infarction
        "I69313",  # Psychomotor deficit following cerebral infarction
        "I69314",  # Frontal lobe and executive function deficit following cerebral infarction
        "I69315",  # Cognitive social or emotional deficit following cerebral infarction
        "I69318",  # Other symptoms and signs involving cognitive functions following cerebral infarction
        "I69319",  # Unspecified symptoms and signs involving cognitive functions following cerebral infarction
        "I69320",  # Aphasia following cerebral infarction
        "I69321",  # Dysphasia following cerebral infarction
        "I69322",  # Dysarthria following cerebral infarction
        "I69323",  # Fluency disorder following cerebral infarction
        "I69328",  # Other speech and language deficits following cerebral infarction
        "I69331",  # Monoplegia of upper limb following cerebral infarction affecting right dominant side
        "I69332",  # Monoplegia of upper limb following cerebral infarction affecting left dominant side
        "I69333",  # Monoplegia of upper limb following cerebral infarction affecting right non-dominant side
        "I69334",  # Monoplegia of upper limb following cerebral infarction affecting left non-dominant side
        "I69339",  # Monoplegia of upper limb following cerebral infarction affecting unspecified side
        "I69341",  # Monoplegia of lower limb following cerebral infarction affecting right dominant side
        "I69342",  # Monoplegia of lower limb following cerebral infarction affecting left dominant side
        "I69343",  # Monoplegia of lower limb following cerebral infarction affecting right non-dominant side
        "I69344",  # Monoplegia of lower limb following cerebral infarction affecting left non-dominant side
        "I69349",  # Monoplegia of lower limb following cerebral infarction affecting unspecified side
        "I69351",  # Hemiplegia and hemiparesis following cerebral infarction affecting right dominant side
        "I69352",  # Hemiplegia and hemiparesis following cerebral infarction affecting left dominant side
        "I69353",  # Hemiplegia and hemiparesis following cerebral infarction affecting right non-dominant side
        "I69354",  # Hemiplegia and hemiparesis following cerebral infarction affecting left non-dominant side
        "I69359",  # Hemiplegia and hemiparesis following cerebral infarction affecting unspecified side
        "I69361",  # Other paralytic syndrome following cerebral infarction affecting right dominant side
        "I69362",  # Other paralytic syndrome following cerebral infarction affecting left dominant side
        "I69363",  # Other paralytic syndrome following cerebral infarction affecting right non-dominant side
        "I69364",  # Other paralytic syndrome following cerebral infarction affecting left non-dominant side
        "I69365",  # Other paralytic syndrome following cerebral infarction, bilateral
        "I69369",  # Other paralytic syndrome following cerebral infarction affecting unspecified side
        "I69390",  # Apraxia following cerebral infarction
        "I69391",  # Dysphagia following cerebral infarction
        "I69392",  # Facial weakness following cerebral infarction
        "I69393",  # Ataxia following cerebral infarction
        "I69398",  # Other sequelae of cerebral infarction
        "I6980",  # Unspecified sequelae of other cerebrovascular disease
        "I69810",  # Attention and concentration deficit following other cerebrovascular disease
        "I69811",  # Memory deficit following other cerebrovascular disease
        "I69812",  # Visuospatial deficit and spatial neglect following other cerebrovascular disease
        "I69813",  # Psychomotor deficit following other cerebrovascular disease
        "I69814",  # Frontal lobe and executive function deficit following other cerebrovascular disease
        "I69815",  # Cognitive social or emotional deficit following other cerebrovascular disease
        "I69818",  # Other symptoms and signs involving cognitive functions following other cerebrovascular disease
        "I69819",  # Unspecified symptoms and signs involving cognitive functions following other cerebrovascular disease
        "I69820",  # Aphasia following other cerebrovascular disease
        "I69821",  # Dysphasia following other cerebrovascular disease
        "I69822",  # Dysarthria following other cerebrovascular disease
        "I69823",  # Fluency disorder following other cerebrovascular disease
        "I69828",  # Other speech and language deficits following other cerebrovascular disease
        "I69831",  # Monoplegia of upper limb following other cerebrovascular disease affecting right dominant side
        "I69832",  # Monoplegia of upper limb following other cerebrovascular disease affecting left dominant side
        "I69833",  # Monoplegia of upper limb following other cerebrovascular disease affecting right non-dominant side
        "I69834",  # Monoplegia of upper limb following other cerebrovascular disease affecting left non-dominant side
        "I69839",  # Monoplegia of upper limb following other cerebrovascular disease affecting unspecified side
        "I69841",  # Monoplegia of lower limb following other cerebrovascular disease affecting right dominant side
        "I69842",  # Monoplegia of lower limb following other cerebrovascular disease affecting left dominant side
        "I69843",  # Monoplegia of lower limb following other cerebrovascular disease affecting right non-dominant side
        "I69844",  # Monoplegia of lower limb following other cerebrovascular disease affecting left non-dominant side
        "I69849",  # Monoplegia of lower limb following other cerebrovascular disease affecting unspecified side
        "I69851",  # Hemiplegia and hemiparesis following other cerebrovascular disease affecting right dominant side
        "I69852",  # Hemiplegia and hemiparesis following other cerebrovascular disease affecting left dominant side
        "I69853",  # Hemiplegia and hemiparesis following other cerebrovascular disease affecting right non-dominant side
        "I69854",  # Hemiplegia and hemiparesis following other cerebrovascular disease affecting left non-dominant side
        "I69859",  # Hemiplegia and hemiparesis following other cerebrovascular disease affecting unspecified side
        "I69861",  # Other paralytic syndrome following other cerebrovascular disease affecting right dominant side
        "I69862",  # Other paralytic syndrome following other cerebrovascular disease affecting left dominant side
        "I69863",  # Other paralytic syndrome following other cerebrovascular disease affecting right non-dominant side
        "I69864",  # Other paralytic syndrome following other cerebrovascular disease affecting left non-dominant side
        "I69865",  # Other paralytic syndrome following other cerebrovascular disease, bilateral
        "I69869",  # Other paralytic syndrome following other cerebrovascular disease affecting unspecified side
        "I69890",  # Apraxia following other cerebrovascular disease
        "I69891",  # Dysphagia following other cerebrovascular disease
        "I69892",  # Facial weakness following other cerebrovascular disease
        "I69893",  # Ataxia following other cerebrovascular disease
        "I69898",  # Other sequelae of other cerebrovascular disease
        "I6990",  # Unspecified sequelae of unspecified cerebrovascular disease
        "I69910",  # Attention and concentration deficit following unspecified cerebrovascular disease
        "I69911",  # Memory deficit following unspecified cerebrovascular disease
        "I69912",  # Visuospatial deficit and spatial neglect following unspecified cerebrovascular disease
        "I69913",  # Psychomotor deficit following unspecified cerebrovascular disease
        "I69914",  # Frontal lobe and executive function deficit following unspecified cerebrovascular disease
        "I69915",  # Cognitive social or emotional deficit following unspecified cerebrovascular disease
        "I69918",  # Other symptoms and signs involving cognitive functions following unspecified cerebrovascular disease
        "I69919",  # Unspecified symptoms and signs involving cognitive functions following unspecified cerebrovascular disease
        "I69920",  # Aphasia following unspecified cerebrovascular disease
        "I69921",  # Dysphasia following unspecified cerebrovascular disease
        "I69922",  # Dysarthria following unspecified cerebrovascular disease
        "I69923",  # Fluency disorder following unspecified cerebrovascular disease
        "I69928",  # Other speech and language deficits following unspecified cerebrovascular disease
        "I69931",  # Monoplegia of upper limb following unspecified cerebrovascular disease affecting right dominant side
        "I69932",  # Monoplegia of upper limb following unspecified cerebrovascular disease affecting left dominant side
        "I69933",  # Monoplegia of upper limb following unspecified cerebrovascular disease affecting right non-dominant side
        "I69934",  # Monoplegia of upper limb following unspecified cerebrovascular disease affecting left non-dominant side
        "I69939",  # Monoplegia of upper limb following unspecified cerebrovascular disease affecting unspecified side
        "I69941",  # Monoplegia of lower limb following unspecified cerebrovascular disease affecting right dominant side
        "I69942",  # Monoplegia of lower limb following unspecified cerebrovascular disease affecting left dominant side
        "I69943",  # Monoplegia of lower limb following unspecified cerebrovascular disease affecting right non-dominant side
        "I69944",  # Monoplegia of lower limb following unspecified cerebrovascular disease affecting left non-dominant side
        "I69949",  # Monoplegia of lower limb following unspecified cerebrovascular disease affecting unspecified side
        "I69951",  # Hemiplegia and hemiparesis following unspecified cerebrovascular disease affecting right dominant side
        "I69952",  # Hemiplegia and hemiparesis following unspecified cerebrovascular disease affecting left dominant side
        "I69953",  # Hemiplegia and hemiparesis following unspecified cerebrovascular disease affecting right non-dominant side
        "I69954",  # Hemiplegia and hemiparesis following unspecified cerebrovascular disease affecting left non-dominant side
        "I69959",  # Hemiplegia and hemiparesis following unspecified cerebrovascular disease affecting unspecified side
        "I69961",  # Other paralytic syndrome following unspecified cerebrovascular disease affecting right dominant side
        "I69962",  # Other paralytic syndrome following unspecified cerebrovascular disease affecting left dominant side
        "I69963",  # Other paralytic syndrome following unspecified cerebrovascular disease affecting right non-dominant side
        "I69964",  # Other paralytic syndrome following unspecified cerebrovascular disease affecting left non-dominant side
        "I69965",  # Other paralytic syndrome following unspecified cerebrovascular disease, bilateral
        "I69969",  # Other paralytic syndrome following unspecified cerebrovascular disease affecting unspecified side
        "I69990",  # Apraxia following unspecified cerebrovascular disease
        "I69991",  # Dysphagia following unspecified cerebrovascular disease
        "I69992",  # Facial weakness following unspecified cerebrovascular disease
        "I69993",  # Ataxia following unspecified cerebrovascular disease
        "I69998",  # Other sequelae following unspecified cerebrovascular disease
        "Z8673",  # Personal history of transient ischemic attack (TIA), and cerebral infarction without residual deficits
    }

    SNOMEDCT = {
        "102831000119104",  # Paralytic syndrome of both lower limbs as sequela of stroke (disorder)
        "103761000119107",  # Paralytic syndrome of all four limbs as sequela of stroke (disorder)
        "108691000119102",  # Spasticity as sequela of stroke (disorder)
        "111297002",  # Nonparalytic stroke (disorder)
        "111298007",  # Chronic cerebral ischemia (disorder)
        "116288000",  # Paralytic stroke (disorder)
        "133981000119106",  # Dysarthria as late effects of cerebrovascular disease (disorder)
        "133991000119109",  # Fluency disorder as sequela of cerebrovascular disease (disorder)
        "134771000119108",  # Alteration of sensation as late effect of stroke (disorder)
        "137991000119103",  # Seizure disorder as sequela of stroke (disorder)
        "140281000119108",  # Hemiparesis as late effect of cerebrovascular disease (disorder)
        "140911000119109",  # Ischemic stroke with coma (disorder)
        "140921000119102",  # Ischemic stroke without coma (disorder)
        "145741000119101",  # Apraxia as late effect of cerebrovascular disease (disorder)
        "148871000119109",  # Weakness as a late effect of cerebrovascular accident (disorder)
        "149821000119103",  # Cerebral infarction due to carotid artery occlusion (disorder)
        "18761000119108",  # Monoplegia of arm dominant side as sequela of cerebrovascular disease (disorder)
        "192813004",  # Cerebral degeneration due to cerebrovascular disease (disorder)
        "195185009",  # Cerebral infarct due to thrombosis of precerebral arteries (disorder)
        "195186005",  # Cerebral infarction due to embolism of precerebral arteries (disorder)
        "195189003",  # Cerebral infarction due to thrombosis of cerebral arteries (disorder)
        "195190007",  # Cerebral infarction due to embolism of cerebral arteries (disorder)
        "195212005",  # Brainstem stroke syndrome (disorder)
        "195213000",  # Cerebellar stroke syndrome (disorder)
        "195230003",  # Cerebral infarction due to cerebral venous thrombosis, non-pyogenic (disorder)
        "195239002",  # Late effects of cerebrovascular disease (disorder)
        "195243003",  # Sequelae of cerebral infarction (disorder)
        "20059004",  # Occlusion of cerebral artery (disorder)
        "230690007",  # Cerebrovascular accident (disorder)
        "230693009",  # Anterior cerebral circulation infarction (disorder)
        "230694003",  # Total anterior cerebral circulation infarction (disorder)
        "230695002",  # Partial anterior cerebral circulation infarction (disorder)
        "230696001",  # Posterior cerebral circulation infarction (disorder)
        "230698000",  # Lacunar infarction (disorder)
        "230706003",  # Hemorrhagic cerebral infarction (disorder)
        "230707007",  # Anterior cerebral circulation hemorrhagic infarction (disorder)
        "230708002",  # Posterior cerebral circulation hemorrhagic infarction (disorder)
        "230713003",  # Stroke of uncertain pathology (disorder)
        "230714009",  # Anterior circulation stroke of uncertain pathology (disorder)
        "230715005",  # Posterior circulation stroke of uncertain pathology (disorder)
        "230738008",  # Asymptomatic cerebrovascular disease (disorder)
        "23671000119107",  # Sequela of ischemic cerebral infarction (disorder)
        "25133001",  # Completed stroke (disorder)
        "26021000119107",  # Vertigo as sequela of cerebrovascular disease (disorder)
        "266257000",  # Transient ischemic attack (disorder)
        "275434003",  # Stroke in the puerperium (disorder)
        "276219001",  # Occipital cerebral infarction (disorder)
        "281240008",  # Extension of cerebrovascular accident (disorder)
        "29941000119105",  # Ataxia as sequela of cerebrovascular disease (disorder)
        "302909007",  # Diffuse cerebrovascular disease (disorder)
        "307766002",  # Left sided cerebral infarction (disorder)
        "307767006",  # Right sided cerebral infarction (disorder)
        "33301000119105",  # Sequela of cardioembolic stroke (disorder)
        "33331000119103",  # Sequela of lacunar stroke (disorder)
        "34191000119104",  # Cerebral infarction due to vertebral artery occlusion (disorder)
        "361000119103",  # Paralytic syndrome on one side of the body as late effect of cerebrovascular accident (disorder)
        "371040005",  # Thrombotic stroke (disorder)
        "371041009",  # Embolic stroke (disorder)
        "373606000",  # Occlusive stroke (disorder)
        "390936003",  # Cerebral autosomal dominant arteriopathy with subcortical infarcts and leukoencephalopathy (disorder)
        "40161000119102",  # Weakness of face muscles as sequela of stroke (disorder)
        "413102000",  # Infarction of basal ganglia (disorder)
        "413758000",  # Cardioembolic stroke (disorder)
        "422504002",  # Ischemic stroke (disorder)
        "425642008",  # Monoplegia of dominant lower limb as a late effect of cerebrovascular accident (disorder)
        "425882004",  # Paralytic syndrome as late effect of stroke (disorder)
        "426033005",  # Dysphagia as a late effect of cerebrovascular accident (disorder)
        "426788002",  # Vertigo as late effect of stroke (disorder)
        "426983002",  # Infarction of medulla oblongata (disorder)
        "427065003",  # Monoplegia of dominant upper limb as a late effect of cerebrovascular accident (disorder)
        "427296003",  # Thalamic infarction (disorder)
        "427432001",  # Paralytic syndrome as late effect of thalamic stroke (disorder)
        "428668000",  # Apraxia due to cerebrovascular accident (disorder)
        "430947007",  # Paralytic syndrome of nondominant side as late effect of stroke (disorder)
        "430959006",  # Paralytic syndrome of dominant side as late effect of stroke (disorder)
        "432504007",  # Cerebral infarction (disorder)
        "433183000",  # Neurogenic urinary bladder due to and following cerebrovascular accident (disorder)
        "433891000124100",  # Cerebral infarction due to cerebral artery occlusion (disorder)
        "433911000124103",  # Cerebral infarction due to posterior cerebral artery occlusion (disorder)
        "433931000124109",  # Cerebral infarction due to internal carotid artery occlusion (disorder)
        "433941000124104",  # Occlusion and stenosis of middle cerebral artery with infarction (disorder)
        "433951000124102",  # Occlusion and stenosis of posterior cerebral artery with infarction (disorder)
        "433961000124100",  # Cerebral infarction due to cerebral venous thrombosis (disorder)
        "433971000124107",  # Cerebral venous thrombosis of cortical vein with infarction (disorder)
        "434141000124103",  # Chronic cerebrovascular accident (disorder)
        "434151000124101",  # Cerebral infarction due to anterior cerebral artery occlusion (disorder)
        "434961000124102",  # Cerebral infarction due to middle cerebral artery occlusion (disorder)
        "434991000124105",  # Cerebral infarction due to basilar artery stenosis (disorder)
        "441526008",  # Infarct of cerebrum due to iatrogenic cerebrovascular accident (disorder)
        "441529001",  # Dysphasia as late effect of cerebrovascular disease (disorder)
        "441630004",  # Aphasia as late effect of cerebrovascular disease (disorder)
        "441735003",  # Sensory disorder as a late effect of cerebrovascular disease (disorder)
        "441759008",  # Abnormal vision as a late effect of cerebrovascular disease (disorder)
        "441887006",  # Monoplegia of lower limb as late effect of cerebrovascular disease (disorder)
        "441894009",  # Monoplegia of nondominant lower limb as a late effect of cerebrovascular accident (disorder)
        "441960006",  # Speech and language deficit as late effect of cerebrovascular accident (disorder)
        "441991000",  # Hemiparesis as late effect of cerebrovascular accident (disorder)
        "442024001",  # Hemiplegia as late effect of cerebrovascular disease (disorder)
        "442097001",  # Monoplegia of upper limb as late effect of cerebrovascular disease (disorder)
        "442181008",  # Monoplegia of nondominant upper limb as a late effect of cerebrovascular accident (disorder)
        "442212003",  # Residual cognitive deficit as late effect of cerebrovascular accident (disorder)
        "442617003",  # Aphasia as late effect of cerebrovascular accident (disorder)
        "442668000",  # Hemiplegia of nondominant side as late effect of cerebrovascular disease (disorder)
        "442676003",  # Hemiplegia of dominant side as late effect of cerebrovascular disease (disorder)
        "442733008",  # Hemiplegia as late effect of cerebrovascular accident (disorder)
        "443929000",  # Small vessel cerebrovascular disease (disorder)
        "46421000119102",  # Behavior disorder as sequela of cerebral infarction (disorder)
        "48601000119107",  # Paralytic syndrome on one side of the body as effect of cerebrovascular accident (disorder)
        "5571000124103",  # Cerebrovascular accident with intracranial hemorrhage (disorder)
        "57981008",  # Progressing stroke (disorder)
        "62914000",  # Cerebrovascular disease (disorder)
        "703163006",  # Secondary cerebrovascular disease (disorder)
        "705128004",  # Cerebral infarction due to embolism of middle cerebral artery (disorder)
        "705130002",  # Cerebral infarction due to thrombosis of middle cerebral artery (disorder)
        "87551000119101",  # Visual disturbance as sequela of cerebrovascular disease (disorder)
        "91601000119109",  # Sequela of thrombotic stroke (disorder)
        "92341000119107",  # Weakness of extremities as sequela of stroke (disorder)
        "95457000",  # Brain stem infarction (disorder)
        "9901000119100",  # Occlusion of cerebral artery with stroke (disorder)
        "99451000119105",  # Cerebral infarction due to stenosis of carotid artery (disorder)
    }

class FamilialHypercholesterolemia(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for familial hypercholesterolemia.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of familial hypercholesterolemia.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Familial Hypercholesterolemia"
    OID = "2.16.840.1.113762.1.4.1047.100"
    DEFINITION_VERSION = "20210220"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "E7801",  # Familial hypercholesterolemia
    }

    SNOMEDCT = {
        "238077000",  # Polygenic hypercholesterolemia (disorder)
        "238078005",  # Familial hypercholesterolemia - homozygous (disorder)
        "238079002",  # Familial hypercholesterolemia - heterozygous (disorder)
        "398036000",  # Familial hypercholesterolemia (disorder)
        "398796005",  # Familial type 3 hyperlipoproteinemia (disorder)
        "403829002",  # Familial hypercholesterolemia due to heterozygous low density lipoprotein receptor mutation (disorder)
        "403830007",  # Familial hypercholesterolemia due to homozygous low density lipoprotein receptor mutation (disorder)
        "403831006",  # Familial hypercholesterolemia due to genetic defect of apolipoprotein B (disorder)
    }

class IschemicHeartDiseaseOrRelatedDiagnoses(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of ischemic heart disease and associated diagnoses related to atherosclerotic cardiovascular disease (ASCVD).

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of acute and chronic forms of ischemic heart disease as well as coronary occlusion, thrombosis, or rupture that can be related to ASCVD.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Ischemic Heart Disease or Related Diagnoses"
    OID = "2.16.840.1.113762.1.4.1047.46"
    DEFINITION_VERSION = "20240208"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "I230",  # Hemopericardium as current complication following acute myocardial infarction
        "I231",  # Atrial septal defect as current complication following acute myocardial infarction
        "I232",  # Ventricular septal defect as current complication following acute myocardial infarction
        "I233",  # Rupture of cardiac wall without hemopericardium as current complication following acute myocardial infarction
        "I234",  # Rupture of chordae tendineae as current complication following acute myocardial infarction
        "I235",  # Rupture of papillary muscle as current complication following acute myocardial infarction
        "I236",  # Thrombosis of atrium, auricular appendage, and ventricle as current complications following acute myocardial infarction
        "I238",  # Other current complications following acute myocardial infarction
        "I240",  # Acute coronary thrombosis not resulting in myocardial infarction
        "I2481",  # Acute coronary microvascular dysfunction
        "I2489",  # Other forms of acute ischemic heart disease
        "I249",  # Acute ischemic heart disease, unspecified
        "I252",  # Old myocardial infarction
        "I255",  # Ischemic cardiomyopathy
        "I256",  # Silent myocardial ischemia
        "I2582",  # Chronic total occlusion of coronary artery
        "I2585",  # Chronic coronary microvascular dysfunction
        "I2589",  # Other forms of chronic ischemic heart disease
        "I259",  # Chronic ischemic heart disease, unspecified
    }

    SNOMEDCT = {
        "10365005",  # Right main coronary artery thrombosis (disorder)
        "123641001",  # Left coronary artery occlusion (disorder)
        "123642008",  # Right coronary artery occlusion (disorder)
        "194821006",  # Coronary thrombosis not resulting in myocardial infarction (disorder)
        "194862000",  # Hemopericardium due to and following acute myocardial infarction (disorder)
        "194863005",  # Atrial septal defect due to and following acute myocardial infarction (disorder)
        "194865003",  # Rupture of cardiac wall without hemopericardium as current complication following acute myocardial infarction (disorder)
        "194867006",  # Rupture of papillary muscle as current complication following acute myocardial infarction (disorder)
        "194868001",  # Thrombosis of atrium, auricular appendage, and ventricle due to and following acute myocardial infarction (disorder)
        "233823002",  # Silent myocardial ischemia (disorder)
        "233846000",  # Post-infarction ventricular septal defect (disorder)
        "233847009",  # Cardiac rupture due to and following acute myocardial infarction (disorder)
        "233860003",  # Post-infarction mitral papillary muscle rupture (disorder)
        "233889001",  # Post-infarction hemopericardium (disorder)
        "28248000",  # Left anterior descending coronary artery thrombosis (disorder)
        "28931004",  # Coronary artery rupture (disorder)
        "315026000",  # Transient myocardial ischemia (disorder)
        "398274000",  # Coronary artery thrombosis (disorder)
        "408546009",  # Coronary artery bypass graft occlusion (disorder)
        "413439005",  # Acute ischemic heart disease (disorder)
        "413444003",  # Acute myocardial ischemia (disorder)
        "413838009",  # Chronic ischemic heart disease (disorder)
        "413844008",  # Chronic myocardial ischemia (disorder)
        "414545008",  # Ischemic heart disease (disorder)
        "414795007",  # Myocardial ischemia (disorder)
        "421327009",  # Coronary artery stent thrombosis (disorder)
        "428196007",  # Mixed myocardial ischemia and infarction (disorder)
        "457592003",  # Occlusion of coronary sinus (disorder)
        "46109009",  # Subendocardial ischemia (disorder)
        "461417000",  # Acquired occlusion of coronary sinus (disorder)
        "56276002",  # Left main coronary artery thrombosis (disorder)
        "63739005",  # Coronary occlusion (disorder)
        "697976003",  # Microvascular ischemia of myocardium (disorder)
        "703328007",  # Mitral valve regurgitation due to acute myocardial infarction without papillary muscle and chordal rupture (disorder)
        "703330009",  # Mitral valve regurgitation due to acute myocardial infarction with papillary muscle and chordal rupture (disorder)
        "712866001",  # Resting ischemia co-occurrent and due to ischemic heart disease (disorder)
        "713405002",  # Subacute ischemic heart disease (disorder)
        "78741000119103",  # Acute coronary artery occlusion not resulting in myocardial infarction (disorder)
    }

class LiverDisease(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of liver disease.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis for acute and chronic liver conditions that may represent the liver is failing. Examples of conditions include concepts associated with, but are not limited to, the following: alcoholic liver damage, cirrhosis, hepatitis C and E, and liver necrosis.

    **Exclusion Criteria:** Excludes concepts that represent diagnoses for Hepatitis A and Hepatitis B.
    """

    VALUE_SET_NAME = "Liver Disease"
    OID = "2.16.840.1.113762.1.4.1047.42"
    DEFINITION_VERSION = "20210220"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "B170",  # Acute delta-(super) infection of hepatitis B carrier
        "B1710",  # Acute hepatitis C without hepatic coma
        "B1711",  # Acute hepatitis C with hepatic coma
        "B172",  # Acute hepatitis E
        "B178",  # Other specified acute viral hepatitis
        "B179",  # Acute viral hepatitis, unspecified
        "B182",  # Chronic viral hepatitis C
        "B188",  # Other chronic viral hepatitis
        "B189",  # Chronic viral hepatitis, unspecified
        "B190",  # Unspecified viral hepatitis with hepatic coma
        "B1920",  # Unspecified viral hepatitis C without hepatic coma
        "B1921",  # Unspecified viral hepatitis C with hepatic coma
        "B199",  # Unspecified viral hepatitis without hepatic coma
        "K700",  # Alcoholic fatty liver
        "K7010",  # Alcoholic hepatitis without ascites
        "K7011",  # Alcoholic hepatitis with ascites
        "K702",  # Alcoholic fibrosis and sclerosis of liver
        "K7030",  # Alcoholic cirrhosis of liver without ascites
        "K7031",  # Alcoholic cirrhosis of liver with ascites
        "K7040",  # Alcoholic hepatic failure without coma
        "K7041",  # Alcoholic hepatic failure with coma
        "K709",  # Alcoholic liver disease, unspecified
        "K710",  # Toxic liver disease with cholestasis
        "K7110",  # Toxic liver disease with hepatic necrosis, without coma
        "K7111",  # Toxic liver disease with hepatic necrosis, with coma
        "K712",  # Toxic liver disease with acute hepatitis
        "K713",  # Toxic liver disease with chronic persistent hepatitis
        "K714",  # Toxic liver disease with chronic lobular hepatitis
        "K7150",  # Toxic liver disease with chronic active hepatitis without ascites
        "K7151",  # Toxic liver disease with chronic active hepatitis with ascites
        "K716",  # Toxic liver disease with hepatitis, not elsewhere classified
        "K717",  # Toxic liver disease with fibrosis and cirrhosis of liver
        "K718",  # Toxic liver disease with other disorders of liver
        "K719",  # Toxic liver disease, unspecified
        "K7200",  # Acute and subacute hepatic failure without coma
        "K7201",  # Acute and subacute hepatic failure with coma
        "K7210",  # Chronic hepatic failure without coma
        "K7211",  # Chronic hepatic failure with coma
        "K7290",  # Hepatic failure, unspecified without coma
        "K7291",  # Hepatic failure, unspecified with coma
        "K730",  # Chronic persistent hepatitis, not elsewhere classified
        "K731",  # Chronic lobular hepatitis, not elsewhere classified
        "K732",  # Chronic active hepatitis, not elsewhere classified
        "K738",  # Other chronic hepatitis, not elsewhere classified
        "K739",  # Chronic hepatitis, unspecified
        "K7400",  # Hepatic fibrosis, unspecified
        "K7401",  # Hepatic fibrosis, early fibrosis
        "K7402",  # Hepatic fibrosis, advanced fibrosis
        "K741",  # Hepatic sclerosis
        "K742",  # Hepatic fibrosis with hepatic sclerosis
        "K743",  # Primary biliary cirrhosis
        "K744",  # Secondary biliary cirrhosis
        "K745",  # Biliary cirrhosis, unspecified
        "K7460",  # Unspecified cirrhosis of liver
        "K7469",  # Other cirrhosis of liver
        "K754",  # Autoimmune hepatitis
        "O98411",  # Viral hepatitis complicating pregnancy, first trimester
        "O98412",  # Viral hepatitis complicating pregnancy, second trimester
        "O98413",  # Viral hepatitis complicating pregnancy, third trimester
        "O98419",  # Viral hepatitis complicating pregnancy, unspecified trimester
    }

    SNOMEDCT = {
        "1010616001",  # Cirrhosis of liver due to classical cystic fibrosis (disorder)
        "10295004",  # Chronic viral hepatitis (disorder)
        "10759111000119102",  # Viral hepatitis in mother complicating childbirth (disorder)
        "10759151000119101",  # Viral hepatitis in mother complicating pregnancy (disorder)
        "1087101000119109",  # Herpes simplex hepatitis (disorder)
        "109819003",  # Obstructive biliary cirrhosis (disorder)
        "111896003",  # Viral hepatitis without hepatic coma (disorder)
        "1148573000",  # Hepatic failure following surgical procedure (disorder)
        "1197150002",  # Recurrent hepatitis C virus induced liver disease following liver transplant (disorder)
        "1197704005",  # Autoantibody negative autoimmune hepatitis (disorder)
        "123604002",  # Toxic cirrhosis (disorder)
        "123605001",  # Nutritional cirrhosis (disorder)
        "123606000",  # Cholangiolitic cirrhosis (disorder)
        "123716002",  # Latent cirrhosis (disorder)
        "123717006",  # Advanced cirrhosis (disorder)
        "128241005",  # Inflammatory disease of liver (disorder)
        "128302006",  # Chronic hepatitis C (disorder)
        "146371000119104",  # Hepatic coma due to chronic hepatitis C (disorder)
        "16098491000119109",  # Chronic autoimmune hepatitis (disorder)
        "16623921000119105",  # Viral hepatitis C in mother during pregnancy (disorder)
        "1761006",  # Biliary cirrhosis (disorder)
        "186628001",  # Hepatic coma due to viral hepatitis C (disorder)
        "186698009",  # Cytomegalovirus hepatitis (disorder)
        "19682006",  # Lupus hepatitis (disorder)
        "197268000",  # Acute and subacute liver necrosis (disorder)
        "197270009",  # Acute hepatic failure (disorder)
        "197271008",  # Acute hepatitis - non-infective (disorder)
        "197284004",  # Chronic active hepatitis (disorder)
        "197286002",  # Recurrent hepatitis (disorder)
        "197293003",  # Diffuse nodular cirrhosis (disorder)
        "197310003",  # Biliary cirrhosis of children (disorder)
        "197339003",  # Coxsackie virus hepatitis (disorder)
        "197344005",  # Hepatitis caused by adenovirus (disorder)
        "197352008",  # Toxic hepatitis (disorder)
        "199203001",  # Viral hepatitis complicating pregnancy, childbirth and the puerperium (disorder)
        "19943007",  # Cirrhosis of liver (disorder)
        "206372007",  # Congenital viral hepatitis (disorder)
        "213230009",  # Hepatic failure as a complication of care (disorder)
        "21861000",  # Micronodular cirrhosis (disorder)
        "22508003",  # Hepatic failure due to a procedure (disorder)
        "235865005",  # Hepatitis D superinfection of hepatitis B carrier (disorder)
        "235866006",  # Acute hepatitis C (disorder)
        "235867002",  # Acute hepatitis E (disorder)
        "235869004",  # Chronic viral hepatitis B with hepatitis D (disorder)
        "235873001",  # Halothane hepatitis (disorder)
        "235874007",  # Enflurane hepatitis (disorder)
        "235875008",  # Alcoholic hepatitis (disorder)
        "235876009",  # Drug-induced hepatitis (disorder)
        "235881000",  # Alcoholic hepatic failure (disorder)
        "235884008",  # Fulminant hepatic failure (disorder)
        "235885009",  # Subfulminant hepatic failure (disorder)
        "235886005",  # Chronic hepatic failure (disorder)
        "235889003",  # Drug-induced chronic hepatitis (disorder)
        "235896001",  # Infectious cirrhosis (disorder)
        "235897005",  # Hypoxia-associated cirrhosis (disorder)
        "266468003",  # Cirrhosis - non-alcoholic (disorder)
        "271440004",  # Cirrhosis secondary to cholestasis (disorder)
        "27156006",  # Posthepatitic cirrhosis (disorder)
        "278929008",  # Congenital hepatitis C infection (disorder)
        "28766006",  # Posttransfusion viral hepatitis (disorder)
        "302919001",  # Epstein-Barr virus hepatitis (disorder)
        "307757001",  # Chronic alcoholic hepatitis (disorder)
        "31005002",  # Hepatorenal syndrome due to a procedure (disorder)
        "328383001",  # Chronic liver disease (disorder)
        "347891000119103",  # Chronic hepatitis C with stage 3 fibrosis (disorder)
        "370889009",  # Lymphocytic portal hepatitis (disorder)
        "371139006",  # Early cirrhosis (disorder)
        "3738000",  # Viral hepatitis (disorder)
        "37871000",  # Acute hepatitis (disorder)
        "397575003",  # Viral hepatitis, type G (disorder)
        "408335007",  # Autoimmune hepatitis (disorder)
        "41309000",  # Alcoholic liver damage (disorder)
        "41889008",  # Chronic persistent hepatitis (disorder)
        "419728003",  # Portal cirrhosis (disorder)
        "420054005",  # Alcoholic cirrhosis (disorder)
        "425413006",  # Drug-induced cirrhosis of liver (disorder)
        "426364002",  # Allergic hepatitis (disorder)
        "435091000124105",  # Hepatitis with hepatic coma (disorder)
        "435101000124104",  # Chronic viral hepatitis C with hepatic coma (disorder)
        "43904005",  # Macronodular cirrhosis (disorder)
        "450880008",  # Chronic hepatitis E (disorder)
        "4846001",  # Anicteric viral hepatitis (disorder)
        "4896000",  # Acute toxic hepatitis (disorder)
        "50325005",  # Alcoholic fatty liver (disorder)
        "50711007",  # Viral hepatitis type C (disorder)
        "51292008",  # Hepatorenal syndrome (disorder)
        "536002",  # Glissonian cirrhosis (disorder)
        "57339008",  # Chronic lobular hepatitis (disorder)
        "57412004",  # Acute viral hepatitis (disorder)
        "58282009",  # Relapsing viral hepatitis (disorder)
        "59927004",  # Hepatic failure (disorder)
        "60037002",  # Chronic persistent viral hepatitis (disorder)
        "62216007",  # Familial arthrogryposis-cholestatic hepatorenal syndrome (disorder)
        "66870002",  # Chronic active viral hepatitis (disorder)
        "702969000",  # Reactivation of hepatitis C viral hepatitis (disorder)
        "703866000",  # Chronic hepatitis C with stage 2 fibrosis (disorder)
        "707341005",  # Viral hepatitis type D (disorder)
        "708198006",  # Chronic active hepatitis C (disorder)
        "7111000119109",  # Viral hepatitis type E (disorder)
        "713181003",  # Chronic alcoholic liver disease (disorder)
        "716203000",  # Decompensated cirrhosis of liver (disorder)
        "721588008",  # Hepatitis caused by sexually transmissible virus (disorder)
        "721710005",  # Fibrosis of liver caused by alcohol (disorder)
        "72445008",  # Subacute noninfective hepatitis (disorder)
        "725938001",  # Cirrhosis of liver caused by methotrexate (disorder)
        "725939009",  # Cirrhosis of liver caused by amiodarone (disorder)
        "725940006",  # Cirrhosis of liver caused by methyldopa (disorder)
        "735733008",  # Cirrhosis of liver co-occurrent and due to primary sclerosing cholangitis (disorder)
        "74669004",  # Cardiac cirrhosis (disorder)
        "76301009",  # Florid cirrhosis (disorder)
        "767809001",  # Chronic hepatitis C caused by hepatitis C virus genotype 6 (disorder)
        "767810006",  # Chronic hepatitis C caused by hepatitis C virus genotype 5 (disorder)
        "76783007",  # Chronic hepatitis (disorder)
        "768006009",  # Chronic hepatitis C caused by Hepatitis C virus genotype 3 (disorder)
        "768125005",  # Chronic hepatitis C caused by Hepatitis C virus genotype 2 (disorder)
        "768126006",  # Chronic hepatitis C caused by Hepatitis C virus genotype 4 (disorder)
        "768127002",  # Chronic hepatitis C caused by Hepatitis C virus genotype 1 (disorder)
        "78208005",  # Pigment cirrhosis (disorder)
        "79720007",  # Chronic nonalcoholic liver disease (disorder)
        "831000119103",  # Cirrhosis of liver due to chronic hepatitis C (disorder)
        "838377003",  # Chronic hepatitis C co-occurrent with human immunodeficiency virus infection (disorder)
        "83864008",  # Drug-induced cholestatic hepatitis (disorder)
        "86454000",  # Postnecrotic cirrhosis (disorder)
        "871619002",  # Cirrhosis of liver due to and following cardiac procedure (disorder)
        "89580002",  # Cryptogenic cirrhosis (disorder)
        "89789003",  # Chronic aggressive viral hepatitis (disorder)
        "95556007",  # Cholestatic hepatitis (disorder)
        "9953008",  # Acute alcoholic liver disease (disorder)
    }

class MyocardialInfarction(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of myocardial infarction - both STEMI and NSTEMI.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of myocardial infarction.

    **Exclusion Criteria:** None
    """

    VALUE_SET_NAME = "Myocardial Infarction"
    OID = "2.16.840.1.113762.1.4.1108.101"
    DEFINITION_VERSION = "20240208"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "I2101",  # ST elevation (STEMI) myocardial infarction involving left main coronary artery
        "I2102",  # ST elevation (STEMI) myocardial infarction involving left anterior descending coronary artery
        "I2109",  # ST elevation (STEMI) myocardial infarction involving other coronary artery of anterior wall
        "I2111",  # ST elevation (STEMI) myocardial infarction involving right coronary artery
        "I2119",  # ST elevation (STEMI) myocardial infarction involving other coronary artery of inferior wall
        "I2121",  # ST elevation (STEMI) myocardial infarction involving left circumflex coronary artery
        "I2129",  # ST elevation (STEMI) myocardial infarction involving other sites
        "I213",  # ST elevation (STEMI) myocardial infarction of unspecified site
        "I214",  # Non-ST elevation (NSTEMI) myocardial infarction
        "I219",  # Acute myocardial infarction, unspecified
        "I21A9",  # Other myocardial infarction type
        "I21B",  # Myocardial infarction with coronary microvascular dysfunction
        "I220",  # Subsequent ST elevation (STEMI) myocardial infarction of anterior wall
        "I221",  # Subsequent ST elevation (STEMI) myocardial infarction of inferior wall
        "I228",  # Subsequent ST elevation (STEMI) myocardial infarction of other sites
        "I229",  # Subsequent ST elevation (STEMI) myocardial infarction of unspecified site
    }

    SNOMEDCT = {
        "10273003",  # Acute infarction of papillary muscle (disorder)
        "1163440003",  # Postoperative acute myocardial infarction (disorder)
        "1204151009",  # Acute inferior non-ST segment elevation myocardial infarction of right ventricle (disorder)
        "1204152002",  # Acute inferior non-ST segment elevation myocardial infarction (disorder)
        "1204154001",  # Acute anterior non-ST segment elevation myocardial infarction with right ventricular involvement (disorder)
        "1204155000",  # Acute anterior non-ST segment elevation myocardial infarction (disorder)
        "1204222000",  # Acute non-ST segment elevation myocardial infarction of right ventricle (disorder)
        "129574000",  # Postoperative myocardial infarction (disorder)
        "15713001000119100",  # Acute ST segment elevation myocardial infarction of atrium (disorder)
        "15990001",  # Acute myocardial infarction of posterolateral wall (disorder)
        "194802003",  # True posterior myocardial infarction (disorder)
        "194809007",  # Acute atrial infarction (disorder)
        "194856005",  # Subsequent myocardial infarction (disorder)
        "22298006",  # Myocardial infarction (disorder)
        "233835003",  # Acute widespread myocardial infarction (disorder)
        "233838001",  # Acute posterior myocardial infarction (disorder)
        "233843008",  # Silent myocardial infarction (disorder)
        "30277009",  # Rupture of ventricle due to acute myocardial infarction (disorder)
        "304914007",  # Acute Q wave myocardial infarction (disorder)
        "307140009",  # Acute non-Q wave infarction (disorder)
        "314207007",  # Non-Q wave myocardial infarction (disorder)
        "32574007",  # Past myocardial infarction diagnosed on electrocardiogram AND/OR other special investigation, but currently presenting no symptoms (disorder)
        "394710008",  # First myocardial infarction (disorder)
        "401303003",  # Acute ST segment elevation myocardial infarction (disorder)
        "401314000",  # Acute non-ST segment elevation myocardial infarction (disorder)
        "418044006",  # Myocardial infarction in recovery phase (disorder)
        "42531007",  # Microinfarct of heart (disorder)
        "428196007",  # Mixed myocardial ischemia and infarction (disorder)
        "428752002",  # Recent myocardial infarction (situation)
        "44811000087108",  # Acute ST segment elevation myocardial infarction due to occlusion of distal portion of circumflex branch of left coronary artery (disorder)
        "44821000087100",  # Acute ST segment elevation myocardial infarction due to occlusion of mid portion of circumflex branch of left coronary artery (disorder)
        "44831000087103",  # Acute ST segment elevation myocardial infarction due to occlusion of obtuse marginal branch of circumflex branch of left coronary artery (disorder)
        "44841000087109",  # Acute ST segment elevation myocardial infarction due to occlusion of posterior lateral branch of circumflex branch of left coronary artery (disorder)
        "44851000087107",  # Acute ST segment elevation myocardial infarction due to occlusion of proximal portion of circumflex branch of left coronary artery (disorder)
        "52035003",  # Acute anteroapical myocardial infarction (disorder)
        "54329005",  # Acute myocardial infarction of anterior wall (disorder)
        "57054005",  # Acute myocardial infarction (disorder)
        "58612006",  # Acute myocardial infarction of lateral wall (disorder)
        "62695002",  # Acute anteroseptal myocardial infarction (disorder)
        "65547006",  # Acute myocardial infarction of inferolateral wall (disorder)
        "70211005",  # Acute myocardial infarction of anterolateral wall (disorder)
        "703164000",  # Acute ST segment elevation myocardial infarction of anterior wall (disorder)
        "703165004",  # Acute ST segment elevation myocardial infarction of anterior wall involving right ventricle (disorder)
        "703212004",  # Acute myocardial infarction during procedure (disorder)
        "703213009",  # Acute ST segment elevation myocardial infarction of inferior wall (disorder)
        "703251009",  # Acute myocardial infarction of inferior wall involving right ventricle (disorder)
        "703252002",  # Acute myocardial infarction of anterior wall involving right ventricle (disorder)
        "703253007",  # Acute ST segment elevation myocardial infarction of inferior wall involving right ventricle (disorder)
        "70422006",  # Acute subendocardial infarction (disorder)
        "70998009",  # Acute myocardial infarction of basal inferior segment of left ventricle (disorder)
        "726499301000119105",  # Myocardial infarction due to atherothrombotic coronary artery disease (disorder)
        "73795002",  # Acute myocardial infarction of inferior wall (disorder)
        "76593002",  # Acute myocardial infarction of inferoposterior wall (disorder)
        "79009004",  # Acute myocardial infarction of septum (disorder)
        "836293000",  # Acute myocardial infarction of right ventricle (disorder)
        "836294006",  # Acute myocardial infarction of apex of heart (disorder)
        "836295007",  # Acute myocardial infarction of inferolateral wall with posterior extension (disorder)
        "840309000",  # Acute ST segment elevation myocardial infarction due to occlusion of proximal portion of anterior descending branch of left coronary artery (disorder)
        "840312002",  # Acute ST segment elevation myocardial infarction due to occlusion of mid portion of anterior descending branch of left coronary artery (disorder)
        "840316004",  # Acute ST segment elevation myocardial infarction due to occlusion of distal portion of anterior descending branch of left coronary artery (disorder)
        "840609007",  # Acute ST segment elevation myocardial infarction due to occlusion of anterior descending branch of left coronary artery (disorder)
        "840680009",  # Acute ST segment elevation myocardial infarction due to occlusion of septal branch of anterior descending branch of left coronary artery (disorder)
        "879955009",  # Myocardial infarction with non-obstructive coronary artery (disorder)
        "896689003",  # Acute myocardial infarction due to occlusion of circumflex branch of left coronary artery (disorder)
        "896691006",  # Acute ST segment elevation myocardial infarction due to occlusion of circumflex branch of left coronary artery (disorder)
        "896696001",  # Acute ST segment elevation myocardial infarction of apex of heart (disorder)
        "896697005",  # Acute ST segment elevation myocardial infarction of right ventricle (disorder)
    }

class Rhabdomyolysis(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of rhabdomyolysis.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of rhabdomyolysis.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Rhabdomyolysis"
    OID = "2.16.840.1.113762.1.4.1047.102"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "M6282",  # Rhabdomyolysis
        "T796XXA",  # Traumatic ischemia of muscle, initial encounter
        "T796XXD",  # Traumatic ischemia of muscle, subsequent encounter
        "T796XXS",  # Traumatic ischemia of muscle, sequela
    }

    SNOMEDCT = {
        "1148631003",  # Paroxysmal rhabdomyolysis (disorder)
        "23697004",  # Crush syndrome (disorder)
        "240125008",  # Muscle crush syndrome (disorder)
        "240131006",  # Rhabdomyolysis (disorder)
        "240132004",  # Non-traumatic rhabdomyolysis (disorder)
        "240133009",  # Familial paroxysmal rhabdomyolysis (disorder)
        "72960004",  # Exertional rhabdomyolysis (disorder)
    }

class StableAndUnstableAngina(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of angina.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of angina, including: stable and unstable angina, pre and post infarction angina, angina decubitus, intermediate coronary syndrome, other unspecified angina or sequelae of myocardial infarction, and other forms of angina pectoris.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Stable and Unstable Angina"
    OID = "2.16.840.1.113762.1.4.1047.47"
    DEFINITION_VERSION = "20240208"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "I200",  # Unstable angina
        "I201",  # Angina pectoris with documented spasm
        "I202",  # Refractory angina pectoris
        "I2081",  # Angina pectoris with coronary microvascular dysfunction
        "I2089",  # Other forms of angina pectoris
        "I209",  # Angina pectoris, unspecified
        "I237",  # Postinfarction angina
    }

    SNOMEDCT = {
        "15960061000119102",  # Unstable angina co-occurrent and due to coronary arteriosclerosis (disorder)
        "15960101000119104",  # Angina due to coronary arteriosclerosis in artery of transplanted heart (disorder)
        "15960141000119102",  # Angina co-occurrent and due to coronary arteriosclerosis (disorder)
        "15960341000119104",  # Unstable angina due to arteriosclerosis of coronary artery bypass graft of transplanted heart (disorder)
        "15960461000119105",  # Unstable angina due to arteriosclerosis of autologous arterial coronary artery bypass graft (disorder)
        "15960541000119107",  # Unstable angina due to arteriosclerosis of autologous vein coronary artery bypass graft (disorder)
        "15960701000119100",  # Angina due to arteriosclerosis of coronary artery bypass graft of transplanted heart (disorder)
        "15960741000119103",  # Preinfarction syndrome due to arteriosclerosis of nonautologous biological coronary artery bypass graft (disorder)
        "16754391000119100",  # Stable angina due to coronary arteriosclerosis (disorder)
        "19057007",  # Status anginosus (disorder)
        "194828000",  # Angina (disorder)
        "21470009",  # Syncope anginosa (disorder)
        "233819005",  # Stable angina (disorder)
        "233821000",  # New onset angina (disorder)
        "314116003",  # Post infarct angina (disorder)
        "4557003",  # Preinfarction syndrome (disorder)
        "59021001",  # Angina decubitus (disorder)
    }

class StatinAssociatedMuscleSymptoms(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for muscle symptoms related to statin therapy.

    **Data Element Scope:** This value set may use a model element in the QDM category of diagnosis.

    **Inclusion Criteria:** Includes concepts that describe muscle symptoms like myopathy, myositis or myalgia; the concept may or may not specify the cause being statin therapy.

    **Exclusion Criteria:** Excludes concepts that are not diagnoses or muscle symptoms that are noted to be related to another cause. This grouping does not include ICD-9-CM.
    """

    VALUE_SET_NAME = "Statin Associated Muscle Symptoms"
    OID = "2.16.840.1.113762.1.4.1108.85"
    DEFINITION_VERSION = "20240208"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "G720",  # Drug-induced myopathy
        "G729",  # Myopathy, unspecified
        "M609",  # Myositis, unspecified
        "M7910",  # Myalgia, unspecified site
    }

    SNOMEDCT = {
        "16462851000119106",  # Myalgia caused by statin (finding)
        "240101000",  # Drug-induced myopathy (disorder)
    }

class CancerRelatedPain(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of pain related to cancer.

    **Data Element Scope:** This value set may use a model element related to diagnosis.

    **Inclusion Criteria:** Includes concepts that represent concepts of pain related to cancer.

    **Exclusion Criteria:** Excludes concepts that do not represent cancer with pain.
    """

    VALUE_SET_NAME = "Cancer Related Pain"
    OID = "2.16.840.1.113762.1.4.1111.180"
    DEFINITION_VERSION = "20240215"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "G893",  # Neoplasm related pain (acute) (chronic)
    }

    SNOMEDCT = {
        "315241008",  # Pain from metastases (finding)
        "735935009",  # Chronic pain due to malignant neoplastic disease (finding)
        "762594002",  # Chronic pain following surgical procedure for cancer (disorder)
        "788891004",  # Chronic pain after cancer treatment (finding)
        "879973007",  # Breakthrough cancer pain (finding)
        "98921000119102",  # Pain due to neoplastic disease (finding)
    }

class OpioidUseDisorder(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to group concepts that indicate an opioid use disorder.

    **Data Element Scope:** This value set may use a model element related to condition.

    **Inclusion Criteria:** Includes concepts that represent an conditions for opioid use disorders.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Opioid Use Disorder"
    OID = "2.16.840.1.113762.1.4.1111.171"
    DEFINITION_VERSION = "20240215"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "F1120",  # Opioid dependence, uncomplicated
        "F11220",  # Opioid dependence with intoxication, uncomplicated
        "F11221",  # Opioid dependence with intoxication delirium
        "F11222",  # Opioid dependence with intoxication with perceptual disturbance
        "F11229",  # Opioid dependence with intoxication, unspecified
        "F1123",  # Opioid dependence with withdrawal
        "F1124",  # Opioid dependence with opioid-induced mood disorder
        "F11250",  # Opioid dependence with opioid-induced psychotic disorder with delusions
        "F11251",  # Opioid dependence with opioid-induced psychotic disorder with hallucinations
        "F11259",  # Opioid dependence with opioid-induced psychotic disorder, unspecified
        "F11281",  # Opioid dependence with opioid-induced sexual dysfunction
        "F11282",  # Opioid dependence with opioid-induced sleep disorder
        "F11288",  # Opioid dependence with other opioid-induced disorder
        "F1129",  # Opioid dependence with unspecified opioid-induced disorder
    }

    SNOMEDCT = {
        "1081000119105",  # Opioid dependence, on agonist therapy (disorder)
        "1255013006",  # Combined opioid with non-opioid substance dependence (disorder)
        "1255015004",  # Combined opioid with non-opioid substance dependence, episodic (disorder)
        "1255018002",  # Combined opioid with non-opioid substance dependence, continuous (disorder)
        "191819002",  # Continuous opioid dependence (disorder)
        "191820008",  # Episodic opioid dependence (disorder)
        "231477003",  # Dependence on diamorphine (disorder)
        "231478008",  # Methadone dependence (disorder)
        "231479000",  # Morphine dependence (disorder)
        "231480002",  # Opium dependence (disorder)
        "426001001",  # Fentanyl dependence (disorder)
        "703845008",  # Buprenorphine dependence (disorder)
        "75544000",  # Opioid dependence (disorder)
    }

class SickleCellDiseaseWithAndWithoutCrisis(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for conditions used to identify sickle cell disease with and without crisis.

    **Data Element Scope:** This value set may use the data element Diagnosis/Condition.

    **Inclusion Criteria:** Includes concepts that represent a conditions for sickle cell disease.

    **Exclusion Criteria:** Excludes hemoglobin diseases not identified as sickle cell.
    """

    VALUE_SET_NAME = "Sickle Cell Disease with and without Crisis"
    OID = "2.16.840.1.113762.1.4.1111.175"
    DEFINITION_VERSION = "20240215"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "D5700",  # Hb-SS disease with crisis, unspecified
        "D5701",  # Hb-SS disease with acute chest syndrome
        "D5702",  # Hb-SS disease with splenic sequestration
        "D571",  # Sickle-cell disease without crisis
        "D5720",  # Sickle-cell/Hb-C disease without crisis
        "D57211",  # Sickle-cell/Hb-C disease with acute chest syndrome
        "D57212",  # Sickle-cell/Hb-C disease with splenic sequestration
        "D57219",  # Sickle-cell/Hb-C disease with crisis, unspecified
        "D5740",  # Sickle-cell thalassemia without crisis
        "D57411",  # Sickle-cell thalassemia, unspecified, with acute chest syndrome
        "D57412",  # Sickle-cell thalassemia, unspecified, with splenic sequestration
        "D57419",  # Sickle-cell thalassemia, unspecified, with crisis
        "D5780",  # Other sickle-cell disorders without crisis
        "D57811",  # Other sickle-cell disorders with acute chest syndrome
        "D57812",  # Other sickle-cell disorders with splenic sequestration
        "D57819",  # Other sickle-cell disorders with crisis, unspecified
    }

    SNOMEDCT = {
        "25472008",  # Sickle cell-hemoglobin D disease (disorder)
        "35434009",  # Sickle cell-hemoglobin C disease (disorder)
        "36472007",  # Sickle cell-thalassemia disease (disorder)
        "416180004",  # Hemoglobin SS disease without crisis (disorder)
        "416214006",  # Sickle cell-hemoglobin D disease without crisis (disorder)
        "416290001",  # Hemoglobin S sickling disorder without crisis (disorder)
        "416417002",  # Hereditary hemoglobin S (disorder)
        "416484003",  # Sickle cell-hemoglobin E disease with crisis (disorder)
        "416638004",  # Sickle cell-hemoglobin E disease without crisis (disorder)
        "416826005",  # Sickle cell-thalassemia disease with crisis (disorder)
        "417048006",  # Sickle cell-thalassemia disease without crisis (disorder)
        "417279003",  # Hemoglobin S sickling disorder with crisis (disorder)
        "417357006",  # Sickling disorder due to hemoglobin S (disorder)
        "417425009",  # Hemoglobin SS disease with crisis (disorder)
        "417517009",  # Sickle cell-hemoglobin C disease with crisis (disorder)
        "417683006",  # Sickle cell-hemoglobin C disease without crisis (disorder)
        "417748003",  # Sickle cell-hemoglobin D disease with crisis (disorder)
        "427306008",  # Hereditary hemoglobinopathy (disorder)
        "440206000",  # Hemoglobin SS disease with vasoocclusive crisis (disorder)
        "444108000",  # Acute sickle cell splenic sequestration crisis (disorder)
        "47024008",  # Sickle cell-hemoglobin E disease (disorder)
    }

class LowerBodyFracturesExcludingAnkleAndFoot(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of a lower body fracture excluding fractures of the ankle and foot.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of fracture of the hip, pelvis, femur, tibia, or fibula.

    **Exclusion Criteria:** Excludes fractures of the ankle or foot.
    """

    VALUE_SET_NAME = "Lower Body Fractures Excluding Ankle and Foot"
    OID = "2.16.840.1.113883.3.464.1003.1178"
    DEFINITION_VERSION = "20230722"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "M80051A",  # Age-related osteoporosis with current pathological fracture, right femur, initial encounter for fracture
        "M80051D",  # Age-related osteoporosis with current pathological fracture, right femur, subsequent encounter for fracture with routine healing
        "M80051G",  # Age-related osteoporosis with current pathological fracture, right femur, subsequent encounter for fracture with delayed healing
        "M80051K",  # Age-related osteoporosis with current pathological fracture, right femur, subsequent encounter for fracture with nonunion
        "M80051P",  # Age-related osteoporosis with current pathological fracture, right femur, subsequent encounter for fracture with malunion
        "M80051S",  # Age-related osteoporosis with current pathological fracture, right femur, sequela
        "M80052A",  # Age-related osteoporosis with current pathological fracture, left femur, initial encounter for fracture
        "M80052D",  # Age-related osteoporosis with current pathological fracture, left femur, subsequent encounter for fracture with routine healing
        "M80052G",  # Age-related osteoporosis with current pathological fracture, left femur, subsequent encounter for fracture with delayed healing
        "M80052K",  # Age-related osteoporosis with current pathological fracture, left femur, subsequent encounter for fracture with nonunion
        "M80052P",  # Age-related osteoporosis with current pathological fracture, left femur, subsequent encounter for fracture with malunion
        "M80052S",  # Age-related osteoporosis with current pathological fracture, left femur, sequela
        "M80059A",  # Age-related osteoporosis with current pathological fracture, unspecified femur, initial encounter for fracture
        "M80059D",  # Age-related osteoporosis with current pathological fracture, unspecified femur, subsequent encounter for fracture with routine healing
        "M80059G",  # Age-related osteoporosis with current pathological fracture, unspecified femur, subsequent encounter for fracture with delayed healing
        "M80059K",  # Age-related osteoporosis with current pathological fracture, unspecified femur, subsequent encounter for fracture with nonunion
        "M80059P",  # Age-related osteoporosis with current pathological fracture, unspecified femur, subsequent encounter for fracture with malunion
        "M80059S",  # Age-related osteoporosis with current pathological fracture, unspecified femur, sequela
        "M80061A",  # Age-related osteoporosis with current pathological fracture, right lower leg, initial encounter for fracture
        "M80061D",  # Age-related osteoporosis with current pathological fracture, right lower leg, subsequent encounter for fracture with routine healing
        "M80061G",  # Age-related osteoporosis with current pathological fracture, right lower leg, subsequent encounter for fracture with delayed healing
        "M80061K",  # Age-related osteoporosis with current pathological fracture, right lower leg, subsequent encounter for fracture with nonunion
        "M80061P",  # Age-related osteoporosis with current pathological fracture, right lower leg, subsequent encounter for fracture with malunion
        "M80061S",  # Age-related osteoporosis with current pathological fracture, right lower leg, sequela
        "M80062A",  # Age-related osteoporosis with current pathological fracture, left lower leg, initial encounter for fracture
        "M80062D",  # Age-related osteoporosis with current pathological fracture, left lower leg, subsequent encounter for fracture with routine healing
        "M80062G",  # Age-related osteoporosis with current pathological fracture, left lower leg, subsequent encounter for fracture with delayed healing
        "M80062K",  # Age-related osteoporosis with current pathological fracture, left lower leg, subsequent encounter for fracture with nonunion
        "M80062P",  # Age-related osteoporosis with current pathological fracture, left lower leg, subsequent encounter for fracture with malunion
        "M80062S",  # Age-related osteoporosis with current pathological fracture, left lower leg, sequela
        "M80069A",  # Age-related osteoporosis with current pathological fracture, unspecified lower leg, initial encounter for fracture
        "M80069D",  # Age-related osteoporosis with current pathological fracture, unspecified lower leg, subsequent encounter for fracture with routine healing
        "M80069G",  # Age-related osteoporosis with current pathological fracture, unspecified lower leg, subsequent encounter for fracture with delayed healing
        "M80069K",  # Age-related osteoporosis with current pathological fracture, unspecified lower leg, subsequent encounter for fracture with nonunion
        "M80069P",  # Age-related osteoporosis with current pathological fracture, unspecified lower leg, subsequent encounter for fracture with malunion
        "M80069S",  # Age-related osteoporosis with current pathological fracture, unspecified lower leg, sequela
        "M80851A",  # Other osteoporosis with current pathological fracture, right femur, initial encounter for fracture
        "M80851D",  # Other osteoporosis with current pathological fracture, right femur, subsequent encounter for fracture with routine healing
        "M80851G",  # Other osteoporosis with current pathological fracture, right femur, subsequent encounter for fracture with delayed healing
        "M80851K",  # Other osteoporosis with current pathological fracture, right femur, subsequent encounter for fracture with nonunion
        "M80851P",  # Other osteoporosis with current pathological fracture, right femur, subsequent encounter for fracture with malunion
        "M80851S",  # Other osteoporosis with current pathological fracture, right femur, sequela
        "M80852A",  # Other osteoporosis with current pathological fracture, left femur, initial encounter for fracture
        "M80852D",  # Other osteoporosis with current pathological fracture, left femur, subsequent encounter for fracture with routine healing
        "M80852G",  # Other osteoporosis with current pathological fracture, left femur, subsequent encounter for fracture with delayed healing
        "M80852K",  # Other osteoporosis with current pathological fracture, left femur, subsequent encounter for fracture with nonunion
        "M80852P",  # Other osteoporosis with current pathological fracture, left femur, subsequent encounter for fracture with malunion
        "M80852S",  # Other osteoporosis with current pathological fracture, left femur, sequela
        "M80859A",  # Other osteoporosis with current pathological fracture, unspecified femur, initial encounter for fracture
        "M80859D",  # Other osteoporosis with current pathological fracture, unspecified femur, subsequent encounter for fracture with routine healing
        "M80859G",  # Other osteoporosis with current pathological fracture, unspecified femur, subsequent encounter for fracture with delayed healing
        "M80859K",  # Other osteoporosis with current pathological fracture, unspecified femur, subsequent encounter for fracture with nonunion
        "M80859P",  # Other osteoporosis with current pathological fracture, unspecified femur, subsequent encounter for fracture with malunion
        "M80859S",  # Other osteoporosis with current pathological fracture, unspecified femur, sequela
        "M80861A",  # Other osteoporosis with current pathological fracture, right lower leg, initial encounter for fracture
        "M80861D",  # Other osteoporosis with current pathological fracture, right lower leg, subsequent encounter for fracture with routine healing
        "M80861G",  # Other osteoporosis with current pathological fracture, right lower leg, subsequent encounter for fracture with delayed healing
        "M80861K",  # Other osteoporosis with current pathological fracture, right lower leg, subsequent encounter for fracture with nonunion
        "M80861P",  # Other osteoporosis with current pathological fracture, right lower leg, subsequent encounter for fracture with malunion
        "M80861S",  # Other osteoporosis with current pathological fracture, right lower leg, sequela
        "M80862A",  # Other osteoporosis with current pathological fracture, left lower leg, initial encounter for fracture
        "M80862D",  # Other osteoporosis with current pathological fracture, left lower leg, subsequent encounter for fracture with routine healing
        "M80862G",  # Other osteoporosis with current pathological fracture, left lower leg, subsequent encounter for fracture with delayed healing
        "M80862K",  # Other osteoporosis with current pathological fracture, left lower leg, subsequent encounter for fracture with nonunion
        "M80862P",  # Other osteoporosis with current pathological fracture, left lower leg, subsequent encounter for fracture with malunion
        "M80862S",  # Other osteoporosis with current pathological fracture, left lower leg, sequela
        "M80869A",  # Other osteoporosis with current pathological fracture, unspecified lower leg, initial encounter for fracture
        "M80869D",  # Other osteoporosis with current pathological fracture, unspecified lower leg, subsequent encounter for fracture with routine healing
        "M80869G",  # Other osteoporosis with current pathological fracture, unspecified lower leg, subsequent encounter for fracture with delayed healing
        "M80869K",  # Other osteoporosis with current pathological fracture, unspecified lower leg, subsequent encounter for fracture with nonunion
        "M80869P",  # Other osteoporosis with current pathological fracture, unspecified lower leg, subsequent encounter for fracture with malunion
        "M80869S",  # Other osteoporosis with current pathological fracture, unspecified lower leg, sequela
        "M84350A",  # Stress fracture, pelvis, initial encounter for fracture
        "M84350D",  # Stress fracture, pelvis, subsequent encounter for fracture with routine healing
        "M84350G",  # Stress fracture, pelvis, subsequent encounter for fracture with delayed healing
        "M84350K",  # Stress fracture, pelvis, subsequent encounter for fracture with nonunion
        "M84350P",  # Stress fracture, pelvis, subsequent encounter for fracture with malunion
        "M84350S",  # Stress fracture, pelvis, sequela
        "M84351A",  # Stress fracture, right femur, initial encounter for fracture
        "M84351D",  # Stress fracture, right femur, subsequent encounter for fracture with routine healing
        "M84351G",  # Stress fracture, right femur, subsequent encounter for fracture with delayed healing
        "M84351K",  # Stress fracture, right femur, subsequent encounter for fracture with nonunion
        "M84351P",  # Stress fracture, right femur, subsequent encounter for fracture with malunion
        "M84351S",  # Stress fracture, right femur, sequela
        "M84352A",  # Stress fracture, left femur, initial encounter for fracture
        "M84352D",  # Stress fracture, left femur, subsequent encounter for fracture with routine healing
        "M84352G",  # Stress fracture, left femur, subsequent encounter for fracture with delayed healing
        "M84352K",  # Stress fracture, left femur, subsequent encounter for fracture with nonunion
        "M84352P",  # Stress fracture, left femur, subsequent encounter for fracture with malunion
        "M84352S",  # Stress fracture, left femur, sequela
        "M84353A",  # Stress fracture, unspecified femur, initial encounter for fracture
        "M84353D",  # Stress fracture, unspecified femur, subsequent encounter for fracture with routine healing
        "M84353G",  # Stress fracture, unspecified femur, subsequent encounter for fracture with delayed healing
        "M84353K",  # Stress fracture, unspecified femur, subsequent encounter for fracture with nonunion
        "M84353P",  # Stress fracture, unspecified femur, subsequent encounter for fracture with malunion
        "M84353S",  # Stress fracture, unspecified femur, sequela
        "M84359A",  # Stress fracture, hip, unspecified, initial encounter for fracture
        "M84359D",  # Stress fracture, hip, unspecified, subsequent encounter for fracture with routine healing
        "M84359G",  # Stress fracture, hip, unspecified, subsequent encounter for fracture with delayed healing
        "M84359K",  # Stress fracture, hip, unspecified, subsequent encounter for fracture with nonunion
        "M84359P",  # Stress fracture, hip, unspecified, subsequent encounter for fracture with malunion
        "M84359S",  # Stress fracture, hip, unspecified, sequela
        "M84361A",  # Stress fracture, right tibia, initial encounter for fracture
        "M84361D",  # Stress fracture, right tibia, subsequent encounter for fracture with routine healing
        "M84361G",  # Stress fracture, right tibia, subsequent encounter for fracture with delayed healing
        "M84361K",  # Stress fracture, right tibia, subsequent encounter for fracture with nonunion
        "M84361P",  # Stress fracture, right tibia, subsequent encounter for fracture with malunion
        "M84361S",  # Stress fracture, right tibia, sequela
        "M84362A",  # Stress fracture, left tibia, initial encounter for fracture
        "M84362D",  # Stress fracture, left tibia, subsequent encounter for fracture with routine healing
        "M84362G",  # Stress fracture, left tibia, subsequent encounter for fracture with delayed healing
        "M84362K",  # Stress fracture, left tibia, subsequent encounter for fracture with nonunion
        "M84362P",  # Stress fracture, left tibia, subsequent encounter for fracture with malunion
        "M84362S",  # Stress fracture, left tibia, sequela
        "M84363A",  # Stress fracture, right fibula, initial encounter for fracture
        "M84363D",  # Stress fracture, right fibula, subsequent encounter for fracture with routine healing
        "M84363G",  # Stress fracture, right fibula, subsequent encounter for fracture with delayed healing
        "M84363K",  # Stress fracture, right fibula, subsequent encounter for fracture with nonunion
        "M84363P",  # Stress fracture, right fibula, subsequent encounter for fracture with malunion
        "M84363S",  # Stress fracture, right fibula, sequela
        "M84364A",  # Stress fracture, left fibula, initial encounter for fracture
        "M84364D",  # Stress fracture, left fibula, subsequent encounter for fracture with routine healing
        "M84364G",  # Stress fracture, left fibula, subsequent encounter for fracture with delayed healing
        "M84364K",  # Stress fracture, left fibula, subsequent encounter for fracture with nonunion
        "M84364P",  # Stress fracture, left fibula, subsequent encounter for fracture with malunion
        "M84364S",  # Stress fracture, left fibula, sequela
        "M84369A",  # Stress fracture, unspecified tibia and fibula, initial encounter for fracture
        "M84369D",  # Stress fracture, unspecified tibia and fibula, subsequent encounter for fracture with routine healing
        "M84369G",  # Stress fracture, unspecified tibia and fibula, subsequent encounter for fracture with delayed healing
        "M84369K",  # Stress fracture, unspecified tibia and fibula, subsequent encounter for fracture with nonunion
        "M84369P",  # Stress fracture, unspecified tibia and fibula, subsequent encounter for fracture with malunion
        "M84369S",  # Stress fracture, unspecified tibia and fibula, sequela
        "M84451A",  # Pathological fracture, right femur, initial encounter for fracture
        "M84451D",  # Pathological fracture, right femur, subsequent encounter for fracture with routine healing
        "M84451G",  # Pathological fracture, right femur, subsequent encounter for fracture with delayed healing
        "M84451K",  # Pathological fracture, right femur, subsequent encounter for fracture with nonunion
        "M84451P",  # Pathological fracture, right femur, subsequent encounter for fracture with malunion
        "M84451S",  # Pathological fracture, right femur, sequela
        "M84452A",  # Pathological fracture, left femur, initial encounter for fracture
        "M84452D",  # Pathological fracture, left femur, subsequent encounter for fracture with routine healing
        "M84452G",  # Pathological fracture, left femur, subsequent encounter for fracture with delayed healing
        "M84452K",  # Pathological fracture, left femur, subsequent encounter for fracture with nonunion
        "M84452P",  # Pathological fracture, left femur, subsequent encounter for fracture with malunion
        "M84452S",  # Pathological fracture, left femur, sequela
        "M84453A",  # Pathological fracture, unspecified femur, initial encounter for fracture
        "M84453D",  # Pathological fracture, unspecified femur, subsequent encounter for fracture with routine healing
        "M84453G",  # Pathological fracture, unspecified femur, subsequent encounter for fracture with delayed healing
        "M84453K",  # Pathological fracture, unspecified femur, subsequent encounter for fracture with nonunion
        "M84453P",  # Pathological fracture, unspecified femur, subsequent encounter for fracture with malunion
        "M84453S",  # Pathological fracture, unspecified femur, sequela
        "M84454A",  # Pathological fracture, pelvis, initial encounter for fracture
        "M84454D",  # Pathological fracture, pelvis, subsequent encounter for fracture with routine healing
        "M84454G",  # Pathological fracture, pelvis, subsequent encounter for fracture with delayed healing
        "M84454K",  # Pathological fracture, pelvis, subsequent encounter for fracture with nonunion
        "M84454P",  # Pathological fracture, pelvis, subsequent encounter for fracture with malunion
        "M84454S",  # Pathological fracture, pelvis, sequela
        "M84459A",  # Pathological fracture, hip, unspecified, initial encounter for fracture
        "M84459D",  # Pathological fracture, hip, unspecified, subsequent encounter for fracture with routine healing
        "M84459G",  # Pathological fracture, hip, unspecified, subsequent encounter for fracture with delayed healing
        "M84459K",  # Pathological fracture, hip, unspecified, subsequent encounter for fracture with nonunion
        "M84459P",  # Pathological fracture, hip, unspecified, subsequent encounter for fracture with malunion
        "M84459S",  # Pathological fracture, hip, unspecified, sequela
        "M84461A",  # Pathological fracture, right tibia, initial encounter for fracture
        "M84461D",  # Pathological fracture, right tibia, subsequent encounter for fracture with routine healing
        "M84461G",  # Pathological fracture, right tibia, subsequent encounter for fracture with delayed healing
        "M84461K",  # Pathological fracture, right tibia, subsequent encounter for fracture with nonunion
        "M84461P",  # Pathological fracture, right tibia, subsequent encounter for fracture with malunion
        "M84461S",  # Pathological fracture, right tibia, sequela
        "M84462A",  # Pathological fracture, left tibia, initial encounter for fracture
        "M84462D",  # Pathological fracture, left tibia, subsequent encounter for fracture with routine healing
        "M84462G",  # Pathological fracture, left tibia, subsequent encounter for fracture with delayed healing
        "M84462K",  # Pathological fracture, left tibia, subsequent encounter for fracture with nonunion
        "M84462P",  # Pathological fracture, left tibia, subsequent encounter for fracture with malunion
        "M84462S",  # Pathological fracture, left tibia, sequela
        "M84463A",  # Pathological fracture, right fibula, initial encounter for fracture
        "M84463D",  # Pathological fracture, right fibula, subsequent encounter for fracture with routine healing
        "M84463G",  # Pathological fracture, right fibula, subsequent encounter for fracture with delayed healing
        "M84463K",  # Pathological fracture, right fibula, subsequent encounter for fracture with nonunion
        "M84463P",  # Pathological fracture, right fibula, subsequent encounter for fracture with malunion
        "M84463S",  # Pathological fracture, right fibula, sequela
        "M84464A",  # Pathological fracture, left fibula, initial encounter for fracture
        "M84464D",  # Pathological fracture, left fibula, subsequent encounter for fracture with routine healing
        "M84464G",  # Pathological fracture, left fibula, subsequent encounter for fracture with delayed healing
        "M84464K",  # Pathological fracture, left fibula, subsequent encounter for fracture with nonunion
        "M84464P",  # Pathological fracture, left fibula, subsequent encounter for fracture with malunion
        "M84464S",  # Pathological fracture, left fibula, sequela
        "M84469A",  # Pathological fracture, unspecified tibia and fibula, initial encounter for fracture
        "M84469D",  # Pathological fracture, unspecified tibia and fibula, subsequent encounter for fracture with routine healing
        "M84469G",  # Pathological fracture, unspecified tibia and fibula, subsequent encounter for fracture with delayed healing
        "M84469K",  # Pathological fracture, unspecified tibia and fibula, subsequent encounter for fracture with nonunion
        "M84469P",  # Pathological fracture, unspecified tibia and fibula, subsequent encounter for fracture with malunion
        "M84469S",  # Pathological fracture, unspecified tibia and fibula, sequela
        "M84550A",  # Pathological fracture in neoplastic disease, pelvis, initial encounter for fracture
        "M84550D",  # Pathological fracture in neoplastic disease, pelvis, subsequent encounter for fracture with routine healing
        "M84550G",  # Pathological fracture in neoplastic disease, pelvis, subsequent encounter for fracture with delayed healing
        "M84550K",  # Pathological fracture in neoplastic disease, pelvis, subsequent encounter for fracture with nonunion
        "M84550P",  # Pathological fracture in neoplastic disease, pelvis, subsequent encounter for fracture with malunion
        "M84550S",  # Pathological fracture in neoplastic disease, pelvis, sequela
        "M84551A",  # Pathological fracture in neoplastic disease, right femur, initial encounter for fracture
        "M84551D",  # Pathological fracture in neoplastic disease, right femur, subsequent encounter for fracture with routine healing
        "M84551G",  # Pathological fracture in neoplastic disease, right femur, subsequent encounter for fracture with delayed healing
        "M84551K",  # Pathological fracture in neoplastic disease, right femur, subsequent encounter for fracture with nonunion
        "M84551P",  # Pathological fracture in neoplastic disease, right femur, subsequent encounter for fracture with malunion
        "M84551S",  # Pathological fracture in neoplastic disease, right femur, sequela
        "M84552A",  # Pathological fracture in neoplastic disease, left femur, initial encounter for fracture
        "M84552D",  # Pathological fracture in neoplastic disease, left femur, subsequent encounter for fracture with routine healing
        "M84552G",  # Pathological fracture in neoplastic disease, left femur, subsequent encounter for fracture with delayed healing
        "M84552K",  # Pathological fracture in neoplastic disease, left femur, subsequent encounter for fracture with nonunion
        "M84552P",  # Pathological fracture in neoplastic disease, left femur, subsequent encounter for fracture with malunion
        "M84552S",  # Pathological fracture in neoplastic disease, left femur, sequela
        "M84553A",  # Pathological fracture in neoplastic disease, unspecified femur, initial encounter for fracture
        "M84553D",  # Pathological fracture in neoplastic disease, unspecified femur, subsequent encounter for fracture with routine healing
        "M84553G",  # Pathological fracture in neoplastic disease, unspecified femur, subsequent encounter for fracture with delayed healing
        "M84553K",  # Pathological fracture in neoplastic disease, unspecified femur, subsequent encounter for fracture with nonunion
        "M84553P",  # Pathological fracture in neoplastic disease, unspecified femur, subsequent encounter for fracture with malunion
        "M84553S",  # Pathological fracture in neoplastic disease, unspecified femur, sequela
        "M84559A",  # Pathological fracture in neoplastic disease, hip, unspecified, initial encounter for fracture
        "M84559D",  # Pathological fracture in neoplastic disease, hip, unspecified, subsequent encounter for fracture with routine healing
        "M84559G",  # Pathological fracture in neoplastic disease, hip, unspecified, subsequent encounter for fracture with delayed healing
        "M84559K",  # Pathological fracture in neoplastic disease, hip, unspecified, subsequent encounter for fracture with nonunion
        "M84559P",  # Pathological fracture in neoplastic disease, hip, unspecified, subsequent encounter for fracture with malunion
        "M84559S",  # Pathological fracture in neoplastic disease, hip, unspecified, sequela
        "M84561A",  # Pathological fracture in neoplastic disease, right tibia, initial encounter for fracture
        "M84561D",  # Pathological fracture in neoplastic disease, right tibia, subsequent encounter for fracture with routine healing
        "M84561G",  # Pathological fracture in neoplastic disease, right tibia, subsequent encounter for fracture with delayed healing
        "M84561K",  # Pathological fracture in neoplastic disease, right tibia, subsequent encounter for fracture with nonunion
        "M84561P",  # Pathological fracture in neoplastic disease, right tibia, subsequent encounter for fracture with malunion
        "M84561S",  # Pathological fracture in neoplastic disease, right tibia, sequela
        "M84562A",  # Pathological fracture in neoplastic disease, left tibia, initial encounter for fracture
        "M84562D",  # Pathological fracture in neoplastic disease, left tibia, subsequent encounter for fracture with routine healing
        "M84562G",  # Pathological fracture in neoplastic disease, left tibia, subsequent encounter for fracture with delayed healing
        "M84562K",  # Pathological fracture in neoplastic disease, left tibia, subsequent encounter for fracture with nonunion
        "M84562P",  # Pathological fracture in neoplastic disease, left tibia, subsequent encounter for fracture with malunion
        "M84562S",  # Pathological fracture in neoplastic disease, left tibia, sequela
        "M84563A",  # Pathological fracture in neoplastic disease, right fibula, initial encounter for fracture
        "M84563D",  # Pathological fracture in neoplastic disease, right fibula, subsequent encounter for fracture with routine healing
        "M84563G",  # Pathological fracture in neoplastic disease, right fibula, subsequent encounter for fracture with delayed healing
        "M84563K",  # Pathological fracture in neoplastic disease, right fibula, subsequent encounter for fracture with nonunion
        "M84563P",  # Pathological fracture in neoplastic disease, right fibula, subsequent encounter for fracture with malunion
        "M84563S",  # Pathological fracture in neoplastic disease, right fibula, sequela
        "M84564A",  # Pathological fracture in neoplastic disease, left fibula, initial encounter for fracture
        "M84564D",  # Pathological fracture in neoplastic disease, left fibula, subsequent encounter for fracture with routine healing
        "M84564G",  # Pathological fracture in neoplastic disease, left fibula, subsequent encounter for fracture with delayed healing
        "M84564K",  # Pathological fracture in neoplastic disease, left fibula, subsequent encounter for fracture with nonunion
        "M84564P",  # Pathological fracture in neoplastic disease, left fibula, subsequent encounter for fracture with malunion
        "M84564S",  # Pathological fracture in neoplastic disease, left fibula, sequela
        "M84569A",  # Pathological fracture in neoplastic disease, unspecified tibia and fibula, initial encounter for fracture
        "M84569D",  # Pathological fracture in neoplastic disease, unspecified tibia and fibula, subsequent encounter for fracture with routine healing
        "M84569G",  # Pathological fracture in neoplastic disease, unspecified tibia and fibula, subsequent encounter for fracture with delayed healing
        "M84569K",  # Pathological fracture in neoplastic disease, unspecified tibia and fibula, subsequent encounter for fracture with nonunion
        "M84569P",  # Pathological fracture in neoplastic disease, unspecified tibia and fibula, subsequent encounter for fracture with malunion
        "M84569S",  # Pathological fracture in neoplastic disease, unspecified tibia and fibula, sequela
        "M84650A",  # Pathological fracture in other disease, pelvis, initial encounter for fracture
        "M84650D",  # Pathological fracture in other disease, pelvis, subsequent encounter for fracture with routine healing
        "M84650G",  # Pathological fracture in other disease, pelvis, subsequent encounter for fracture with delayed healing
        "M84650K",  # Pathological fracture in other disease, pelvis, subsequent encounter for fracture with nonunion
        "M84650P",  # Pathological fracture in other disease, pelvis, subsequent encounter for fracture with malunion
        "M84650S",  # Pathological fracture in other disease, pelvis, sequela
        "M84651A",  # Pathological fracture in other disease, right femur, initial encounter for fracture
        "M84651D",  # Pathological fracture in other disease, right femur, subsequent encounter for fracture with routine healing
        "M84651G",  # Pathological fracture in other disease, right femur, subsequent encounter for fracture with delayed healing
        "M84651K",  # Pathological fracture in other disease, right femur, subsequent encounter for fracture with nonunion
        "M84651P",  # Pathological fracture in other disease, right femur, subsequent encounter for fracture with malunion
        "M84651S",  # Pathological fracture in other disease, right femur, sequela
        "M84652A",  # Pathological fracture in other disease, left femur, initial encounter for fracture
        "M84652D",  # Pathological fracture in other disease, left femur, subsequent encounter for fracture with routine healing
        "M84652G",  # Pathological fracture in other disease, left femur, subsequent encounter for fracture with delayed healing
        "M84652K",  # Pathological fracture in other disease, left femur, subsequent encounter for fracture with nonunion
        "M84652P",  # Pathological fracture in other disease, left femur, subsequent encounter for fracture with malunion
        "M84652S",  # Pathological fracture in other disease, left femur, sequela
        "M84653A",  # Pathological fracture in other disease, unspecified femur, initial encounter for fracture
        "M84653D",  # Pathological fracture in other disease, unspecified femur, subsequent encounter for fracture with routine healing
        "M84653G",  # Pathological fracture in other disease, unspecified femur, subsequent encounter for fracture with delayed healing
        "M84653K",  # Pathological fracture in other disease, unspecified femur, subsequent encounter for fracture with nonunion
        "M84653P",  # Pathological fracture in other disease, unspecified femur, subsequent encounter for fracture with malunion
        "M84653S",  # Pathological fracture in other disease, unspecified femur, sequela
        "M84659A",  # Pathological fracture in other disease, hip, unspecified, initial encounter for fracture
        "M84659D",  # Pathological fracture in other disease, hip, unspecified, subsequent encounter for fracture with routine healing
        "M84659G",  # Pathological fracture in other disease, hip, unspecified, subsequent encounter for fracture with delayed healing
        "M84659K",  # Pathological fracture in other disease, hip, unspecified, subsequent encounter for fracture with nonunion
        "M84659P",  # Pathological fracture in other disease, hip, unspecified, subsequent encounter for fracture with malunion
        "M84659S",  # Pathological fracture in other disease, hip, unspecified, sequela
        "M84661A",  # Pathological fracture in other disease, right tibia, initial encounter for fracture
        "M84661D",  # Pathological fracture in other disease, right tibia, subsequent encounter for fracture with routine healing
        "M84661G",  # Pathological fracture in other disease, right tibia, subsequent encounter for fracture with delayed healing
        "M84661K",  # Pathological fracture in other disease, right tibia, subsequent encounter for fracture with nonunion
        "M84661P",  # Pathological fracture in other disease, right tibia, subsequent encounter for fracture with malunion
        "M84661S",  # Pathological fracture in other disease, right tibia, sequela
        "M84662A",  # Pathological fracture in other disease, left tibia, initial encounter for fracture
        "M84662D",  # Pathological fracture in other disease, left tibia, subsequent encounter for fracture with routine healing
        "M84662G",  # Pathological fracture in other disease, left tibia, subsequent encounter for fracture with delayed healing
        "M84662K",  # Pathological fracture in other disease, left tibia, subsequent encounter for fracture with nonunion
        "M84662P",  # Pathological fracture in other disease, left tibia, subsequent encounter for fracture with malunion
        "M84662S",  # Pathological fracture in other disease, left tibia, sequela
        "M84663A",  # Pathological fracture in other disease, right fibula, initial encounter for fracture
        "M84663D",  # Pathological fracture in other disease, right fibula, subsequent encounter for fracture with routine healing
        "M84663G",  # Pathological fracture in other disease, right fibula, subsequent encounter for fracture with delayed healing
        "M84663K",  # Pathological fracture in other disease, right fibula, subsequent encounter for fracture with nonunion
        "M84663P",  # Pathological fracture in other disease, right fibula, subsequent encounter for fracture with malunion
        "M84663S",  # Pathological fracture in other disease, right fibula, sequela
        "M84664A",  # Pathological fracture in other disease, left fibula, initial encounter for fracture
        "M84664D",  # Pathological fracture in other disease, left fibula, subsequent encounter for fracture with routine healing
        "M84664G",  # Pathological fracture in other disease, left fibula, subsequent encounter for fracture with delayed healing
        "M84664K",  # Pathological fracture in other disease, left fibula, subsequent encounter for fracture with nonunion
        "M84664P",  # Pathological fracture in other disease, left fibula, subsequent encounter for fracture with malunion
        "M84664S",  # Pathological fracture in other disease, left fibula, sequela
        "M84669A",  # Pathological fracture in other disease, unspecified tibia and fibula, initial encounter for fracture
        "M84669D",  # Pathological fracture in other disease, unspecified tibia and fibula, subsequent encounter for fracture with routine healing
        "M84669G",  # Pathological fracture in other disease, unspecified tibia and fibula, subsequent encounter for fracture with delayed healing
        "M84669K",  # Pathological fracture in other disease, unspecified tibia and fibula, subsequent encounter for fracture with nonunion
        "M84669P",  # Pathological fracture in other disease, unspecified tibia and fibula, subsequent encounter for fracture with malunion
        "M84669S",  # Pathological fracture in other disease, unspecified tibia and fibula, sequela
        "M84750A",  # Atypical femoral fracture, unspecified, initial encounter for fracture
        "M84750D",  # Atypical femoral fracture, unspecified, subsequent encounter for fracture with routine healing
        "M84750G",  # Atypical femoral fracture, unspecified, subsequent encounter for fracture with delayed healing
        "M84750K",  # Atypical femoral fracture, unspecified, subsequent encounter for fracture with nonunion
        "M84750P",  # Atypical femoral fracture, unspecified, subsequent encounter for fracture with malunion
        "M84750S",  # Atypical femoral fracture, unspecified, sequela
        "M84751A",  # Incomplete atypical femoral fracture, right leg, initial encounter for fracture
        "M84751D",  # Incomplete atypical femoral fracture, right leg, subsequent encounter for fracture with routine healing
        "M84751G",  # Incomplete atypical femoral fracture, right leg, subsequent encounter for fracture with delayed healing
        "M84751K",  # Incomplete atypical femoral fracture, right leg, subsequent encounter for fracture with nonunion
        "M84751P",  # Incomplete atypical femoral fracture, right leg, subsequent encounter for fracture with malunion
        "M84751S",  # Incomplete atypical femoral fracture, right leg, sequela
        "M84752A",  # Incomplete atypical femoral fracture, left leg, initial encounter for fracture
        "M84752D",  # Incomplete atypical femoral fracture, left leg, subsequent encounter for fracture with routine healing
        "M84752G",  # Incomplete atypical femoral fracture, left leg, subsequent encounter for fracture with delayed healing
        "M84752K",  # Incomplete atypical femoral fracture, left leg, subsequent encounter for fracture with nonunion
        "M84752P",  # Incomplete atypical femoral fracture, left leg, subsequent encounter for fracture with malunion
        "M84752S",  # Incomplete atypical femoral fracture, left leg, sequela
        "M84753A",  # Incomplete atypical femoral fracture, unspecified leg, initial encounter for fracture
        "M84753D",  # Incomplete atypical femoral fracture, unspecified leg, subsequent encounter for fracture with routine healing
        "M84753G",  # Incomplete atypical femoral fracture, unspecified leg, subsequent encounter for fracture with delayed healing
        "M84753K",  # Incomplete atypical femoral fracture, unspecified leg, subsequent encounter for fracture with nonunion
        "M84753P",  # Incomplete atypical femoral fracture, unspecified leg, subsequent encounter for fracture with malunion
        "M84753S",  # Incomplete atypical femoral fracture, unspecified leg, sequela
        "M84754A",  # Complete transverse atypical femoral fracture, right leg, initial encounter for fracture
        "M84754D",  # Complete transverse atypical femoral fracture, right leg, subsequent encounter for fracture with routine healing
        "M84754G",  # Complete transverse atypical femoral fracture, right leg, subsequent encounter for fracture with delayed healing
        "M84754K",  # Complete transverse atypical femoral fracture, right leg, subsequent encounter for fracture with nonunion
        "M84754P",  # Complete transverse atypical femoral fracture, right leg, subsequent encounter for fracture with malunion
        "M84754S",  # Complete transverse atypical femoral fracture, right leg, sequela
        "M84755A",  # Complete transverse atypical femoral fracture, left leg, initial encounter for fracture
        "M84755D",  # Complete transverse atypical femoral fracture, left leg, subsequent encounter for fracture with routine healing
        "M84755G",  # Complete transverse atypical femoral fracture, left leg, subsequent encounter for fracture with delayed healing
        "M84755K",  # Complete transverse atypical femoral fracture, left leg, subsequent encounter for fracture with nonunion
        "M84755P",  # Complete transverse atypical femoral fracture, left leg, subsequent encounter for fracture with malunion
        "M84755S",  # Complete transverse atypical femoral fracture, left leg, sequela
        "M84756A",  # Complete transverse atypical femoral fracture, unspecified leg, initial encounter for fracture
        "M84756D",  # Complete transverse atypical femoral fracture, unspecified leg, subsequent encounter for fracture with routine healing
        "M84756G",  # Complete transverse atypical femoral fracture, unspecified leg, subsequent encounter for fracture with delayed healing
        "M84756K",  # Complete transverse atypical femoral fracture, unspecified leg, subsequent encounter for fracture with nonunion
        "M84756P",  # Complete transverse atypical femoral fracture, unspecified leg, subsequent encounter for fracture with malunion
        "M84756S",  # Complete transverse atypical femoral fracture, unspecified leg, sequela
        "M84757A",  # Complete oblique atypical femoral fracture, right leg, initial encounter for fracture
        "M84757D",  # Complete oblique atypical femoral fracture, right leg, subsequent encounter for fracture with routine healing
        "M84757G",  # Complete oblique atypical femoral fracture, right leg, subsequent encounter for fracture with delayed healing
        "M84757K",  # Complete oblique atypical femoral fracture, right leg, subsequent encounter for fracture with nonunion
        "M84757P",  # Complete oblique atypical femoral fracture, right leg, subsequent encounter for fracture with malunion
        "M84757S",  # Complete oblique atypical femoral fracture, right leg, sequela
        "M84758A",  # Complete oblique atypical femoral fracture, left leg, initial encounter for fracture
        "M84758D",  # Complete oblique atypical femoral fracture, left leg, subsequent encounter for fracture with routine healing
        "M84758G",  # Complete oblique atypical femoral fracture, left leg, subsequent encounter for fracture with delayed healing
        "M84758K",  # Complete oblique atypical femoral fracture, left leg, subsequent encounter for fracture with nonunion
        "M84758P",  # Complete oblique atypical femoral fracture, left leg, subsequent encounter for fracture with malunion
        "M84758S",  # Complete oblique atypical femoral fracture, left leg, sequela
        "M84759A",  # Complete oblique atypical femoral fracture, unspecified leg, initial encounter for fracture
        "M84759D",  # Complete oblique atypical femoral fracture, unspecified leg, subsequent encounter for fracture with routine healing
        "M84759G",  # Complete oblique atypical femoral fracture, unspecified leg, subsequent encounter for fracture with delayed healing
        "M84759K",  # Complete oblique atypical femoral fracture, unspecified leg, subsequent encounter for fracture with nonunion
        "M84759P",  # Complete oblique atypical femoral fracture, unspecified leg, subsequent encounter for fracture with malunion
        "M84759S",  # Complete oblique atypical femoral fracture, unspecified leg, sequela
        "M9665",  # Fracture of pelvis following insertion of orthopedic implant, joint prosthesis, or bone plate
        "M96661",  # Fracture of femur following insertion of orthopedic implant, joint prosthesis, or bone plate, right leg
        "M96662",  # Fracture of femur following insertion of orthopedic implant, joint prosthesis, or bone plate, left leg
        "M96669",  # Fracture of femur following insertion of orthopedic implant, joint prosthesis, or bone plate, unspecified leg
        "M96671",  # Fracture of tibia or fibula following insertion of orthopedic implant, joint prosthesis, or bone plate, right leg
        "M96672",  # Fracture of tibia or fibula following insertion of orthopedic implant, joint prosthesis, or bone plate, left leg
        "M96679",  # Fracture of tibia or fibula following insertion of orthopedic implant, joint prosthesis, or bone plate, unspecified leg
        "M9669",  # Fracture of other bone following insertion of orthopedic implant, joint prosthesis, or bone plate
        "M9701XA",  # Periprosthetic fracture around internal prosthetic right hip joint, initial encounter
        "M9701XD",  # Periprosthetic fracture around internal prosthetic right hip joint, subsequent encounter
        "M9701XS",  # Periprosthetic fracture around internal prosthetic right hip joint, sequela
        "M9702XA",  # Periprosthetic fracture around internal prosthetic left hip joint, initial encounter
        "M9702XD",  # Periprosthetic fracture around internal prosthetic left hip joint, subsequent encounter
        "M9702XS",  # Periprosthetic fracture around internal prosthetic left hip joint, sequela
        "M9711XA",  # Periprosthetic fracture around internal prosthetic right knee joint, initial encounter
        "M9711XD",  # Periprosthetic fracture around internal prosthetic right knee joint, subsequent encounter
        "M9711XS",  # Periprosthetic fracture around internal prosthetic right knee joint, sequela
        "M9712XA",  # Periprosthetic fracture around internal prosthetic left knee joint, initial encounter
        "M9712XD",  # Periprosthetic fracture around internal prosthetic left knee joint, subsequent encounter
        "M9712XS",  # Periprosthetic fracture around internal prosthetic left knee joint, sequela
        "S32301A",  # Unspecified fracture of right ilium, initial encounter for closed fracture
        "S32301B",  # Unspecified fracture of right ilium, initial encounter for open fracture
        "S32301D",  # Unspecified fracture of right ilium, subsequent encounter for fracture with routine healing
        "S32301G",  # Unspecified fracture of right ilium, subsequent encounter for fracture with delayed healing
        "S32301K",  # Unspecified fracture of right ilium, subsequent encounter for fracture with nonunion
        "S32301S",  # Unspecified fracture of right ilium, sequela
        "S32302A",  # Unspecified fracture of left ilium, initial encounter for closed fracture
        "S32302B",  # Unspecified fracture of left ilium, initial encounter for open fracture
        "S32302D",  # Unspecified fracture of left ilium, subsequent encounter for fracture with routine healing
        "S32302G",  # Unspecified fracture of left ilium, subsequent encounter for fracture with delayed healing
        "S32302K",  # Unspecified fracture of left ilium, subsequent encounter for fracture with nonunion
        "S32302S",  # Unspecified fracture of left ilium, sequela
        "S32309A",  # Unspecified fracture of unspecified ilium, initial encounter for closed fracture
        "S32309B",  # Unspecified fracture of unspecified ilium, initial encounter for open fracture
        "S32309D",  # Unspecified fracture of unspecified ilium, subsequent encounter for fracture with routine healing
        "S32309G",  # Unspecified fracture of unspecified ilium, subsequent encounter for fracture with delayed healing
        "S32309K",  # Unspecified fracture of unspecified ilium, subsequent encounter for fracture with nonunion
        "S32309S",  # Unspecified fracture of unspecified ilium, sequela
        "S32311A",  # Displaced avulsion fracture of right ilium, initial encounter for closed fracture
        "S32311B",  # Displaced avulsion fracture of right ilium, initial encounter for open fracture
        "S32311D",  # Displaced avulsion fracture of right ilium, subsequent encounter for fracture with routine healing
        "S32311G",  # Displaced avulsion fracture of right ilium, subsequent encounter for fracture with delayed healing
        "S32311K",  # Displaced avulsion fracture of right ilium, subsequent encounter for fracture with nonunion
        "S32311S",  # Displaced avulsion fracture of right ilium, sequela
        "S32312A",  # Displaced avulsion fracture of left ilium, initial encounter for closed fracture
        "S32312B",  # Displaced avulsion fracture of left ilium, initial encounter for open fracture
        "S32312D",  # Displaced avulsion fracture of left ilium, subsequent encounter for fracture with routine healing
        "S32312G",  # Displaced avulsion fracture of left ilium, subsequent encounter for fracture with delayed healing
        "S32312K",  # Displaced avulsion fracture of left ilium, subsequent encounter for fracture with nonunion
        "S32312S",  # Displaced avulsion fracture of left ilium, sequela
        "S32313A",  # Displaced avulsion fracture of unspecified ilium, initial encounter for closed fracture
        "S32313B",  # Displaced avulsion fracture of unspecified ilium, initial encounter for open fracture
        "S32313D",  # Displaced avulsion fracture of unspecified ilium, subsequent encounter for fracture with routine healing
        "S32313G",  # Displaced avulsion fracture of unspecified ilium, subsequent encounter for fracture with delayed healing
        "S32313K",  # Displaced avulsion fracture of unspecified ilium, subsequent encounter for fracture with nonunion
        "S32313S",  # Displaced avulsion fracture of unspecified ilium, sequela
        "S32314A",  # Nondisplaced avulsion fracture of right ilium, initial encounter for closed fracture
        "S32314B",  # Nondisplaced avulsion fracture of right ilium, initial encounter for open fracture
        "S32314D",  # Nondisplaced avulsion fracture of right ilium, subsequent encounter for fracture with routine healing
        "S32314G",  # Nondisplaced avulsion fracture of right ilium, subsequent encounter for fracture with delayed healing
        "S32314K",  # Nondisplaced avulsion fracture of right ilium, subsequent encounter for fracture with nonunion
        "S32314S",  # Nondisplaced avulsion fracture of right ilium, sequela
        "S32315A",  # Nondisplaced avulsion fracture of left ilium, initial encounter for closed fracture
        "S32315B",  # Nondisplaced avulsion fracture of left ilium, initial encounter for open fracture
        "S32315D",  # Nondisplaced avulsion fracture of left ilium, subsequent encounter for fracture with routine healing
        "S32315G",  # Nondisplaced avulsion fracture of left ilium, subsequent encounter for fracture with delayed healing
        "S32315K",  # Nondisplaced avulsion fracture of left ilium, subsequent encounter for fracture with nonunion
        "S32315S",  # Nondisplaced avulsion fracture of left ilium, sequela
        "S32316A",  # Nondisplaced avulsion fracture of unspecified ilium, initial encounter for closed fracture
        "S32316B",  # Nondisplaced avulsion fracture of unspecified ilium, initial encounter for open fracture
        "S32316D",  # Nondisplaced avulsion fracture of unspecified ilium, subsequent encounter for fracture with routine healing
        "S32316G",  # Nondisplaced avulsion fracture of unspecified ilium, subsequent encounter for fracture with delayed healing
        "S32316K",  # Nondisplaced avulsion fracture of unspecified ilium, subsequent encounter for fracture with nonunion
        "S32316S",  # Nondisplaced avulsion fracture of unspecified ilium, sequela
        "S32391A",  # Other fracture of right ilium, initial encounter for closed fracture
        "S32391B",  # Other fracture of right ilium, initial encounter for open fracture
        "S32391D",  # Other fracture of right ilium, subsequent encounter for fracture with routine healing
        "S32391G",  # Other fracture of right ilium, subsequent encounter for fracture with delayed healing
        "S32391K",  # Other fracture of right ilium, subsequent encounter for fracture with nonunion
        "S32391S",  # Other fracture of right ilium, sequela
        "S32392A",  # Other fracture of left ilium, initial encounter for closed fracture
        "S32392B",  # Other fracture of left ilium, initial encounter for open fracture
        "S32392D",  # Other fracture of left ilium, subsequent encounter for fracture with routine healing
        "S32392G",  # Other fracture of left ilium, subsequent encounter for fracture with delayed healing
        "S32392K",  # Other fracture of left ilium, subsequent encounter for fracture with nonunion
        "S32392S",  # Other fracture of left ilium, sequela
        "S32399A",  # Other fracture of unspecified ilium, initial encounter for closed fracture
        "S32399B",  # Other fracture of unspecified ilium, initial encounter for open fracture
        "S32399D",  # Other fracture of unspecified ilium, subsequent encounter for fracture with routine healing
        "S32399G",  # Other fracture of unspecified ilium, subsequent encounter for fracture with delayed healing
        "S32399K",  # Other fracture of unspecified ilium, subsequent encounter for fracture with nonunion
        "S32399S",  # Other fracture of unspecified ilium, sequela
        "S32401A",  # Unspecified fracture of right acetabulum, initial encounter for closed fracture
        "S32401B",  # Unspecified fracture of right acetabulum, initial encounter for open fracture
        "S32401D",  # Unspecified fracture of right acetabulum, subsequent encounter for fracture with routine healing
        "S32401G",  # Unspecified fracture of right acetabulum, subsequent encounter for fracture with delayed healing
        "S32401K",  # Unspecified fracture of right acetabulum, subsequent encounter for fracture with nonunion
        "S32401S",  # Unspecified fracture of right acetabulum, sequela
        "S32402A",  # Unspecified fracture of left acetabulum, initial encounter for closed fracture
        "S32402B",  # Unspecified fracture of left acetabulum, initial encounter for open fracture
        "S32402D",  # Unspecified fracture of left acetabulum, subsequent encounter for fracture with routine healing
        "S32402G",  # Unspecified fracture of left acetabulum, subsequent encounter for fracture with delayed healing
        "S32402K",  # Unspecified fracture of left acetabulum, subsequent encounter for fracture with nonunion
        "S32402S",  # Unspecified fracture of left acetabulum, sequela
        "S32409A",  # Unspecified fracture of unspecified acetabulum, initial encounter for closed fracture
        "S32409B",  # Unspecified fracture of unspecified acetabulum, initial encounter for open fracture
        "S32409D",  # Unspecified fracture of unspecified acetabulum, subsequent encounter for fracture with routine healing
        "S32409G",  # Unspecified fracture of unspecified acetabulum, subsequent encounter for fracture with delayed healing
        "S32409K",  # Unspecified fracture of unspecified acetabulum, subsequent encounter for fracture with nonunion
        "S32409S",  # Unspecified fracture of unspecified acetabulum, sequela
        "S32411A",  # Displaced fracture of anterior wall of right acetabulum, initial encounter for closed fracture
        "S32411B",  # Displaced fracture of anterior wall of right acetabulum, initial encounter for open fracture
        "S32411D",  # Displaced fracture of anterior wall of right acetabulum, subsequent encounter for fracture with routine healing
        "S32411G",  # Displaced fracture of anterior wall of right acetabulum, subsequent encounter for fracture with delayed healing
        "S32411K",  # Displaced fracture of anterior wall of right acetabulum, subsequent encounter for fracture with nonunion
        "S32411S",  # Displaced fracture of anterior wall of right acetabulum, sequela
        "S32412A",  # Displaced fracture of anterior wall of left acetabulum, initial encounter for closed fracture
        "S32412B",  # Displaced fracture of anterior wall of left acetabulum, initial encounter for open fracture
        "S32412D",  # Displaced fracture of anterior wall of left acetabulum, subsequent encounter for fracture with routine healing
        "S32412G",  # Displaced fracture of anterior wall of left acetabulum, subsequent encounter for fracture with delayed healing
        "S32412K",  # Displaced fracture of anterior wall of left acetabulum, subsequent encounter for fracture with nonunion
        "S32412S",  # Displaced fracture of anterior wall of left acetabulum, sequela
        "S32413A",  # Displaced fracture of anterior wall of unspecified acetabulum, initial encounter for closed fracture
        "S32413B",  # Displaced fracture of anterior wall of unspecified acetabulum, initial encounter for open fracture
        "S32413D",  # Displaced fracture of anterior wall of unspecified acetabulum, subsequent encounter for fracture with routine healing
        "S32413G",  # Displaced fracture of anterior wall of unspecified acetabulum, subsequent encounter for fracture with delayed healing
        "S32413K",  # Displaced fracture of anterior wall of unspecified acetabulum, subsequent encounter for fracture with nonunion
        "S32413S",  # Displaced fracture of anterior wall of unspecified acetabulum, sequela
        "S32414A",  # Nondisplaced fracture of anterior wall of right acetabulum, initial encounter for closed fracture
        "S32414B",  # Nondisplaced fracture of anterior wall of right acetabulum, initial encounter for open fracture
        "S32414D",  # Nondisplaced fracture of anterior wall of right acetabulum, subsequent encounter for fracture with routine healing
        "S32414G",  # Nondisplaced fracture of anterior wall of right acetabulum, subsequent encounter for fracture with delayed healing
        "S32414K",  # Nondisplaced fracture of anterior wall of right acetabulum, subsequent encounter for fracture with nonunion
        "S32414S",  # Nondisplaced fracture of anterior wall of right acetabulum, sequela
        "S32415A",  # Nondisplaced fracture of anterior wall of left acetabulum, initial encounter for closed fracture
        "S32415B",  # Nondisplaced fracture of anterior wall of left acetabulum, initial encounter for open fracture
        "S32415D",  # Nondisplaced fracture of anterior wall of left acetabulum, subsequent encounter for fracture with routine healing
        "S32415G",  # Nondisplaced fracture of anterior wall of left acetabulum, subsequent encounter for fracture with delayed healing
        "S32415K",  # Nondisplaced fracture of anterior wall of left acetabulum, subsequent encounter for fracture with nonunion
        "S32415S",  # Nondisplaced fracture of anterior wall of left acetabulum, sequela
        "S32416A",  # Nondisplaced fracture of anterior wall of unspecified acetabulum, initial encounter for closed fracture
        "S32416B",  # Nondisplaced fracture of anterior wall of unspecified acetabulum, initial encounter for open fracture
        "S32416D",  # Nondisplaced fracture of anterior wall of unspecified acetabulum, subsequent encounter for fracture with routine healing
        "S32416G",  # Nondisplaced fracture of anterior wall of unspecified acetabulum, subsequent encounter for fracture with delayed healing
        "S32416K",  # Nondisplaced fracture of anterior wall of unspecified acetabulum, subsequent encounter for fracture with nonunion
        "S32416S",  # Nondisplaced fracture of anterior wall of unspecified acetabulum, sequela
        "S32421A",  # Displaced fracture of posterior wall of right acetabulum, initial encounter for closed fracture
        "S32421B",  # Displaced fracture of posterior wall of right acetabulum, initial encounter for open fracture
        "S32421D",  # Displaced fracture of posterior wall of right acetabulum, subsequent encounter for fracture with routine healing
        "S32421G",  # Displaced fracture of posterior wall of right acetabulum, subsequent encounter for fracture with delayed healing
        "S32421K",  # Displaced fracture of posterior wall of right acetabulum, subsequent encounter for fracture with nonunion
        "S32421S",  # Displaced fracture of posterior wall of right acetabulum, sequela
        "S32422A",  # Displaced fracture of posterior wall of left acetabulum, initial encounter for closed fracture
        "S32422B",  # Displaced fracture of posterior wall of left acetabulum, initial encounter for open fracture
        "S32422D",  # Displaced fracture of posterior wall of left acetabulum, subsequent encounter for fracture with routine healing
        "S32422G",  # Displaced fracture of posterior wall of left acetabulum, subsequent encounter for fracture with delayed healing
        "S32422K",  # Displaced fracture of posterior wall of left acetabulum, subsequent encounter for fracture with nonunion
        "S32422S",  # Displaced fracture of posterior wall of left acetabulum, sequela
        "S32423A",  # Displaced fracture of posterior wall of unspecified acetabulum, initial encounter for closed fracture
        "S32423B",  # Displaced fracture of posterior wall of unspecified acetabulum, initial encounter for open fracture
        "S32423D",  # Displaced fracture of posterior wall of unspecified acetabulum, subsequent encounter for fracture with routine healing
        "S32423G",  # Displaced fracture of posterior wall of unspecified acetabulum, subsequent encounter for fracture with delayed healing
        "S32423K",  # Displaced fracture of posterior wall of unspecified acetabulum, subsequent encounter for fracture with nonunion
        "S32423S",  # Displaced fracture of posterior wall of unspecified acetabulum, sequela
        "S32424A",  # Nondisplaced fracture of posterior wall of right acetabulum, initial encounter for closed fracture
        "S32424B",  # Nondisplaced fracture of posterior wall of right acetabulum, initial encounter for open fracture
        "S32424D",  # Nondisplaced fracture of posterior wall of right acetabulum, subsequent encounter for fracture with routine healing
        "S32424G",  # Nondisplaced fracture of posterior wall of right acetabulum, subsequent encounter for fracture with delayed healing
        "S32424K",  # Nondisplaced fracture of posterior wall of right acetabulum, subsequent encounter for fracture with nonunion
        "S32424S",  # Nondisplaced fracture of posterior wall of right acetabulum, sequela
        "S32425A",  # Nondisplaced fracture of posterior wall of left acetabulum, initial encounter for closed fracture
        "S32425B",  # Nondisplaced fracture of posterior wall of left acetabulum, initial encounter for open fracture
        "S32425D",  # Nondisplaced fracture of posterior wall of left acetabulum, subsequent encounter for fracture with routine healing
        "S32425G",  # Nondisplaced fracture of posterior wall of left acetabulum, subsequent encounter for fracture with delayed healing
        "S32425K",  # Nondisplaced fracture of posterior wall of left acetabulum, subsequent encounter for fracture with nonunion
        "S32425S",  # Nondisplaced fracture of posterior wall of left acetabulum, sequela
        "S32426A",  # Nondisplaced fracture of posterior wall of unspecified acetabulum, initial encounter for closed fracture
        "S32426B",  # Nondisplaced fracture of posterior wall of unspecified acetabulum, initial encounter for open fracture
        "S32426D",  # Nondisplaced fracture of posterior wall of unspecified acetabulum, subsequent encounter for fracture with routine healing
        "S32426G",  # Nondisplaced fracture of posterior wall of unspecified acetabulum, subsequent encounter for fracture with delayed healing
        "S32426K",  # Nondisplaced fracture of posterior wall of unspecified acetabulum, subsequent encounter for fracture with nonunion
        "S32426S",  # Nondisplaced fracture of posterior wall of unspecified acetabulum, sequela
        "S32431A",  # Displaced fracture of anterior column [iliopubic] of right acetabulum, initial encounter for closed fracture
        "S32431B",  # Displaced fracture of anterior column [iliopubic] of right acetabulum, initial encounter for open fracture
        "S32431D",  # Displaced fracture of anterior column [iliopubic] of right acetabulum, subsequent encounter for fracture with routine healing
        "S32431G",  # Displaced fracture of anterior column [iliopubic] of right acetabulum, subsequent encounter for fracture with delayed healing
        "S32431K",  # Displaced fracture of anterior column [iliopubic] of right acetabulum, subsequent encounter for fracture with nonunion
        "S32431S",  # Displaced fracture of anterior column [iliopubic] of right acetabulum, sequela
        "S32432A",  # Displaced fracture of anterior column [iliopubic] of left acetabulum, initial encounter for closed fracture
        "S32432B",  # Displaced fracture of anterior column [iliopubic] of left acetabulum, initial encounter for open fracture
        "S32432D",  # Displaced fracture of anterior column [iliopubic] of left acetabulum, subsequent encounter for fracture with routine healing
        "S32432G",  # Displaced fracture of anterior column [iliopubic] of left acetabulum, subsequent encounter for fracture with delayed healing
        "S32432K",  # Displaced fracture of anterior column [iliopubic] of left acetabulum, subsequent encounter for fracture with nonunion
        "S32432S",  # Displaced fracture of anterior column [iliopubic] of left acetabulum, sequela
        "S32433A",  # Displaced fracture of anterior column [iliopubic] of unspecified acetabulum, initial encounter for closed fracture
        "S32433B",  # Displaced fracture of anterior column [iliopubic] of unspecified acetabulum, initial encounter for open fracture
        "S32433D",  # Displaced fracture of anterior column [iliopubic] of unspecified acetabulum, subsequent encounter for fracture with routine healing
        "S32433G",  # Displaced fracture of anterior column [iliopubic] of unspecified acetabulum, subsequent encounter for fracture with delayed healing
        "S32433K",  # Displaced fracture of anterior column [iliopubic] of unspecified acetabulum, subsequent encounter for fracture with nonunion
        "S32433S",  # Displaced fracture of anterior column [iliopubic] of unspecified acetabulum, sequela
        "S32434A",  # Nondisplaced fracture of anterior column [iliopubic] of right acetabulum, initial encounter for closed fracture
        "S32434B",  # Nondisplaced fracture of anterior column [iliopubic] of right acetabulum, initial encounter for open fracture
        "S32434D",  # Nondisplaced fracture of anterior column [iliopubic] of right acetabulum, subsequent encounter for fracture with routine healing
        "S32434G",  # Nondisplaced fracture of anterior column [iliopubic] of right acetabulum, subsequent encounter for fracture with delayed healing
        "S32434K",  # Nondisplaced fracture of anterior column [iliopubic] of right acetabulum, subsequent encounter for fracture with nonunion
        "S32434S",  # Nondisplaced fracture of anterior column [iliopubic] of right acetabulum, sequela
        "S32435A",  # Nondisplaced fracture of anterior column [iliopubic] of left acetabulum, initial encounter for closed fracture
        "S32435B",  # Nondisplaced fracture of anterior column [iliopubic] of left acetabulum, initial encounter for open fracture
        "S32435D",  # Nondisplaced fracture of anterior column [iliopubic] of left acetabulum, subsequent encounter for fracture with routine healing
        "S32435G",  # Nondisplaced fracture of anterior column [iliopubic] of left acetabulum, subsequent encounter for fracture with delayed healing
        "S32435K",  # Nondisplaced fracture of anterior column [iliopubic] of left acetabulum, subsequent encounter for fracture with nonunion
        "S32435S",  # Nondisplaced fracture of anterior column [iliopubic] of left acetabulum, sequela
        "S32436A",  # Nondisplaced fracture of anterior column [iliopubic] of unspecified acetabulum, initial encounter for closed fracture
        "S32436B",  # Nondisplaced fracture of anterior column [iliopubic] of unspecified acetabulum, initial encounter for open fracture
        "S32436D",  # Nondisplaced fracture of anterior column [iliopubic] of unspecified acetabulum, subsequent encounter for fracture with routine healing
        "S32436G",  # Nondisplaced fracture of anterior column [iliopubic] of unspecified acetabulum, subsequent encounter for fracture with delayed healing
        "S32436K",  # Nondisplaced fracture of anterior column [iliopubic] of unspecified acetabulum, subsequent encounter for fracture with nonunion
        "S32436S",  # Nondisplaced fracture of anterior column [iliopubic] of unspecified acetabulum, sequela
        "S32441A",  # Displaced fracture of posterior column [ilioischial] of right acetabulum, initial encounter for closed fracture
        "S32441B",  # Displaced fracture of posterior column [ilioischial] of right acetabulum, initial encounter for open fracture
        "S32441D",  # Displaced fracture of posterior column [ilioischial] of right acetabulum, subsequent encounter for fracture with routine healing
        "S32441G",  # Displaced fracture of posterior column [ilioischial] of right acetabulum, subsequent encounter for fracture with delayed healing
        "S32441K",  # Displaced fracture of posterior column [ilioischial] of right acetabulum, subsequent encounter for fracture with nonunion
        "S32441S",  # Displaced fracture of posterior column [ilioischial] of right acetabulum, sequela
        "S32442A",  # Displaced fracture of posterior column [ilioischial] of left acetabulum, initial encounter for closed fracture
        "S32442B",  # Displaced fracture of posterior column [ilioischial] of left acetabulum, initial encounter for open fracture
        "S32442D",  # Displaced fracture of posterior column [ilioischial] of left acetabulum, subsequent encounter for fracture with routine healing
        "S32442G",  # Displaced fracture of posterior column [ilioischial] of left acetabulum, subsequent encounter for fracture with delayed healing
        "S32442K",  # Displaced fracture of posterior column [ilioischial] of left acetabulum, subsequent encounter for fracture with nonunion
        "S32442S",  # Displaced fracture of posterior column [ilioischial] of left acetabulum, sequela
        "S32443A",  # Displaced fracture of posterior column [ilioischial] of unspecified acetabulum, initial encounter for closed fracture
        "S32443B",  # Displaced fracture of posterior column [ilioischial] of unspecified acetabulum, initial encounter for open fracture
        "S32443D",  # Displaced fracture of posterior column [ilioischial] of unspecified acetabulum, subsequent encounter for fracture with routine healing
        "S32443G",  # Displaced fracture of posterior column [ilioischial] of unspecified acetabulum, subsequent encounter for fracture with delayed healing
        "S32443K",  # Displaced fracture of posterior column [ilioischial] of unspecified acetabulum, subsequent encounter for fracture with nonunion
        "S32443S",  # Displaced fracture of posterior column [ilioischial] of unspecified acetabulum, sequela
        "S32444A",  # Nondisplaced fracture of posterior column [ilioischial] of right acetabulum, initial encounter for closed fracture
        "S32444B",  # Nondisplaced fracture of posterior column [ilioischial] of right acetabulum, initial encounter for open fracture
        "S32444D",  # Nondisplaced fracture of posterior column [ilioischial] of right acetabulum, subsequent encounter for fracture with routine healing
        "S32444G",  # Nondisplaced fracture of posterior column [ilioischial] of right acetabulum, subsequent encounter for fracture with delayed healing
        "S32444K",  # Nondisplaced fracture of posterior column [ilioischial] of right acetabulum, subsequent encounter for fracture with nonunion
        "S32444S",  # Nondisplaced fracture of posterior column [ilioischial] of right acetabulum, sequela
        "S32445A",  # Nondisplaced fracture of posterior column [ilioischial] of left acetabulum, initial encounter for closed fracture
        "S32445B",  # Nondisplaced fracture of posterior column [ilioischial] of left acetabulum, initial encounter for open fracture
        "S32445D",  # Nondisplaced fracture of posterior column [ilioischial] of left acetabulum, subsequent encounter for fracture with routine healing
        "S32445G",  # Nondisplaced fracture of posterior column [ilioischial] of left acetabulum, subsequent encounter for fracture with delayed healing
        "S32445K",  # Nondisplaced fracture of posterior column [ilioischial] of left acetabulum, subsequent encounter for fracture with nonunion
        "S32445S",  # Nondisplaced fracture of posterior column [ilioischial] of left acetabulum, sequela
        "S32446A",  # Nondisplaced fracture of posterior column [ilioischial] of unspecified acetabulum, initial encounter for closed fracture
        "S32446B",  # Nondisplaced fracture of posterior column [ilioischial] of unspecified acetabulum, initial encounter for open fracture
        "S32446D",  # Nondisplaced fracture of posterior column [ilioischial] of unspecified acetabulum, subsequent encounter for fracture with routine healing
        "S32446G",  # Nondisplaced fracture of posterior column [ilioischial] of unspecified acetabulum, subsequent encounter for fracture with delayed healing
        "S32446K",  # Nondisplaced fracture of posterior column [ilioischial] of unspecified acetabulum, subsequent encounter for fracture with nonunion
        "S32446S",  # Nondisplaced fracture of posterior column [ilioischial] of unspecified acetabulum, sequela
        "S32451A",  # Displaced transverse fracture of right acetabulum, initial encounter for closed fracture
        "S32451B",  # Displaced transverse fracture of right acetabulum, initial encounter for open fracture
        "S32451D",  # Displaced transverse fracture of right acetabulum, subsequent encounter for fracture with routine healing
        "S32451G",  # Displaced transverse fracture of right acetabulum, subsequent encounter for fracture with delayed healing
        "S32451K",  # Displaced transverse fracture of right acetabulum, subsequent encounter for fracture with nonunion
        "S32451S",  # Displaced transverse fracture of right acetabulum, sequela
        "S32452A",  # Displaced transverse fracture of left acetabulum, initial encounter for closed fracture
        "S32452B",  # Displaced transverse fracture of left acetabulum, initial encounter for open fracture
        "S32452D",  # Displaced transverse fracture of left acetabulum, subsequent encounter for fracture with routine healing
        "S32452G",  # Displaced transverse fracture of left acetabulum, subsequent encounter for fracture with delayed healing
        "S32452K",  # Displaced transverse fracture of left acetabulum, subsequent encounter for fracture with nonunion
        "S32452S",  # Displaced transverse fracture of left acetabulum, sequela
        "S32453A",  # Displaced transverse fracture of unspecified acetabulum, initial encounter for closed fracture
        "S32453B",  # Displaced transverse fracture of unspecified acetabulum, initial encounter for open fracture
        "S32453D",  # Displaced transverse fracture of unspecified acetabulum, subsequent encounter for fracture with routine healing
        "S32453G",  # Displaced transverse fracture of unspecified acetabulum, subsequent encounter for fracture with delayed healing
        "S32453K",  # Displaced transverse fracture of unspecified acetabulum, subsequent encounter for fracture with nonunion
        "S32453S",  # Displaced transverse fracture of unspecified acetabulum, sequela
        "S32454A",  # Nondisplaced transverse fracture of right acetabulum, initial encounter for closed fracture
        "S32454B",  # Nondisplaced transverse fracture of right acetabulum, initial encounter for open fracture
        "S32454D",  # Nondisplaced transverse fracture of right acetabulum, subsequent encounter for fracture with routine healing
        "S32454G",  # Nondisplaced transverse fracture of right acetabulum, subsequent encounter for fracture with delayed healing
        "S32454K",  # Nondisplaced transverse fracture of right acetabulum, subsequent encounter for fracture with nonunion
        "S32454S",  # Nondisplaced transverse fracture of right acetabulum, sequela
        "S32455A",  # Nondisplaced transverse fracture of left acetabulum, initial encounter for closed fracture
        "S32455B",  # Nondisplaced transverse fracture of left acetabulum, initial encounter for open fracture
        "S32455D",  # Nondisplaced transverse fracture of left acetabulum, subsequent encounter for fracture with routine healing
        "S32455G",  # Nondisplaced transverse fracture of left acetabulum, subsequent encounter for fracture with delayed healing
        "S32455K",  # Nondisplaced transverse fracture of left acetabulum, subsequent encounter for fracture with nonunion
        "S32455S",  # Nondisplaced transverse fracture of left acetabulum, sequela
        "S32456A",  # Nondisplaced transverse fracture of unspecified acetabulum, initial encounter for closed fracture
        "S32456B",  # Nondisplaced transverse fracture of unspecified acetabulum, initial encounter for open fracture
        "S32456D",  # Nondisplaced transverse fracture of unspecified acetabulum, subsequent encounter for fracture with routine healing
        "S32456G",  # Nondisplaced transverse fracture of unspecified acetabulum, subsequent encounter for fracture with delayed healing
        "S32456K",  # Nondisplaced transverse fracture of unspecified acetabulum, subsequent encounter for fracture with nonunion
        "S32456S",  # Nondisplaced transverse fracture of unspecified acetabulum, sequela
        "S32461A",  # Displaced associated transverse-posterior fracture of right acetabulum, initial encounter for closed fracture
        "S32461B",  # Displaced associated transverse-posterior fracture of right acetabulum, initial encounter for open fracture
        "S32461D",  # Displaced associated transverse-posterior fracture of right acetabulum, subsequent encounter for fracture with routine healing
        "S32461G",  # Displaced associated transverse-posterior fracture of right acetabulum, subsequent encounter for fracture with delayed healing
        "S32461K",  # Displaced associated transverse-posterior fracture of right acetabulum, subsequent encounter for fracture with nonunion
        "S32461S",  # Displaced associated transverse-posterior fracture of right acetabulum, sequela
        "S32462A",  # Displaced associated transverse-posterior fracture of left acetabulum, initial encounter for closed fracture
        "S32462B",  # Displaced associated transverse-posterior fracture of left acetabulum, initial encounter for open fracture
        "S32462D",  # Displaced associated transverse-posterior fracture of left acetabulum, subsequent encounter for fracture with routine healing
        "S32462G",  # Displaced associated transverse-posterior fracture of left acetabulum, subsequent encounter for fracture with delayed healing
        "S32462K",  # Displaced associated transverse-posterior fracture of left acetabulum, subsequent encounter for fracture with nonunion
        "S32462S",  # Displaced associated transverse-posterior fracture of left acetabulum, sequela
        "S32463A",  # Displaced associated transverse-posterior fracture of unspecified acetabulum, initial encounter for closed fracture
        "S32463B",  # Displaced associated transverse-posterior fracture of unspecified acetabulum, initial encounter for open fracture
        "S32463D",  # Displaced associated transverse-posterior fracture of unspecified acetabulum, subsequent encounter for fracture with routine healing
        "S32463G",  # Displaced associated transverse-posterior fracture of unspecified acetabulum, subsequent encounter for fracture with delayed healing
        "S32463K",  # Displaced associated transverse-posterior fracture of unspecified acetabulum, subsequent encounter for fracture with nonunion
        "S32463S",  # Displaced associated transverse-posterior fracture of unspecified acetabulum, sequela
        "S32464A",  # Nondisplaced associated transverse-posterior fracture of right acetabulum, initial encounter for closed fracture
        "S32464B",  # Nondisplaced associated transverse-posterior fracture of right acetabulum, initial encounter for open fracture
        "S32464D",  # Nondisplaced associated transverse-posterior fracture of right acetabulum, subsequent encounter for fracture with routine healing
        "S32464G",  # Nondisplaced associated transverse-posterior fracture of right acetabulum, subsequent encounter for fracture with delayed healing
        "S32464K",  # Nondisplaced associated transverse-posterior fracture of right acetabulum, subsequent encounter for fracture with nonunion
        "S32464S",  # Nondisplaced associated transverse-posterior fracture of right acetabulum, sequela
        "S32465A",  # Nondisplaced associated transverse-posterior fracture of left acetabulum, initial encounter for closed fracture
        "S32465B",  # Nondisplaced associated transverse-posterior fracture of left acetabulum, initial encounter for open fracture
        "S32465D",  # Nondisplaced associated transverse-posterior fracture of left acetabulum, subsequent encounter for fracture with routine healing
        "S32465G",  # Nondisplaced associated transverse-posterior fracture of left acetabulum, subsequent encounter for fracture with delayed healing
        "S32465K",  # Nondisplaced associated transverse-posterior fracture of left acetabulum, subsequent encounter for fracture with nonunion
        "S32465S",  # Nondisplaced associated transverse-posterior fracture of left acetabulum, sequela
        "S32466A",  # Nondisplaced associated transverse-posterior fracture of unspecified acetabulum, initial encounter for closed fracture
        "S32466B",  # Nondisplaced associated transverse-posterior fracture of unspecified acetabulum, initial encounter for open fracture
        "S32466D",  # Nondisplaced associated transverse-posterior fracture of unspecified acetabulum, subsequent encounter for fracture with routine healing
        "S32466G",  # Nondisplaced associated transverse-posterior fracture of unspecified acetabulum, subsequent encounter for fracture with delayed healing
        "S32466K",  # Nondisplaced associated transverse-posterior fracture of unspecified acetabulum, subsequent encounter for fracture with nonunion
        "S32466S",  # Nondisplaced associated transverse-posterior fracture of unspecified acetabulum, sequela
        "S32471A",  # Displaced fracture of medial wall of right acetabulum, initial encounter for closed fracture
        "S32471B",  # Displaced fracture of medial wall of right acetabulum, initial encounter for open fracture
        "S32471D",  # Displaced fracture of medial wall of right acetabulum, subsequent encounter for fracture with routine healing
        "S32471G",  # Displaced fracture of medial wall of right acetabulum, subsequent encounter for fracture with delayed healing
        "S32471K",  # Displaced fracture of medial wall of right acetabulum, subsequent encounter for fracture with nonunion
        "S32471S",  # Displaced fracture of medial wall of right acetabulum, sequela
        "S32472A",  # Displaced fracture of medial wall of left acetabulum, initial encounter for closed fracture
        "S32472B",  # Displaced fracture of medial wall of left acetabulum, initial encounter for open fracture
        "S32472D",  # Displaced fracture of medial wall of left acetabulum, subsequent encounter for fracture with routine healing
        "S32472G",  # Displaced fracture of medial wall of left acetabulum, subsequent encounter for fracture with delayed healing
        "S32472K",  # Displaced fracture of medial wall of left acetabulum, subsequent encounter for fracture with nonunion
        "S32472S",  # Displaced fracture of medial wall of left acetabulum, sequela
        "S32473A",  # Displaced fracture of medial wall of unspecified acetabulum, initial encounter for closed fracture
        "S32473B",  # Displaced fracture of medial wall of unspecified acetabulum, initial encounter for open fracture
        "S32473D",  # Displaced fracture of medial wall of unspecified acetabulum, subsequent encounter for fracture with routine healing
        "S32473G",  # Displaced fracture of medial wall of unspecified acetabulum, subsequent encounter for fracture with delayed healing
        "S32473K",  # Displaced fracture of medial wall of unspecified acetabulum, subsequent encounter for fracture with nonunion
        "S32473S",  # Displaced fracture of medial wall of unspecified acetabulum, sequela
        "S32474A",  # Nondisplaced fracture of medial wall of right acetabulum, initial encounter for closed fracture
        "S32474B",  # Nondisplaced fracture of medial wall of right acetabulum, initial encounter for open fracture
        "S32474D",  # Nondisplaced fracture of medial wall of right acetabulum, subsequent encounter for fracture with routine healing
        "S32474G",  # Nondisplaced fracture of medial wall of right acetabulum, subsequent encounter for fracture with delayed healing
        "S32474K",  # Nondisplaced fracture of medial wall of right acetabulum, subsequent encounter for fracture with nonunion
        "S32474S",  # Nondisplaced fracture of medial wall of right acetabulum, sequela
        "S32475A",  # Nondisplaced fracture of medial wall of left acetabulum, initial encounter for closed fracture
        "S32475B",  # Nondisplaced fracture of medial wall of left acetabulum, initial encounter for open fracture
        "S32475D",  # Nondisplaced fracture of medial wall of left acetabulum, subsequent encounter for fracture with routine healing
        "S32475G",  # Nondisplaced fracture of medial wall of left acetabulum, subsequent encounter for fracture with delayed healing
        "S32475K",  # Nondisplaced fracture of medial wall of left acetabulum, subsequent encounter for fracture with nonunion
        "S32475S",  # Nondisplaced fracture of medial wall of left acetabulum, sequela
        "S32476A",  # Nondisplaced fracture of medial wall of unspecified acetabulum, initial encounter for closed fracture
        "S32476B",  # Nondisplaced fracture of medial wall of unspecified acetabulum, initial encounter for open fracture
        "S32476D",  # Nondisplaced fracture of medial wall of unspecified acetabulum, subsequent encounter for fracture with routine healing
        "S32476G",  # Nondisplaced fracture of medial wall of unspecified acetabulum, subsequent encounter for fracture with delayed healing
        "S32476K",  # Nondisplaced fracture of medial wall of unspecified acetabulum, subsequent encounter for fracture with nonunion
        "S32476S",  # Nondisplaced fracture of medial wall of unspecified acetabulum, sequela
        "S32481A",  # Displaced dome fracture of right acetabulum, initial encounter for closed fracture
        "S32481B",  # Displaced dome fracture of right acetabulum, initial encounter for open fracture
        "S32481D",  # Displaced dome fracture of right acetabulum, subsequent encounter for fracture with routine healing
        "S32481G",  # Displaced dome fracture of right acetabulum, subsequent encounter for fracture with delayed healing
        "S32481K",  # Displaced dome fracture of right acetabulum, subsequent encounter for fracture with nonunion
        "S32481S",  # Displaced dome fracture of right acetabulum, sequela
        "S32482A",  # Displaced dome fracture of left acetabulum, initial encounter for closed fracture
        "S32482B",  # Displaced dome fracture of left acetabulum, initial encounter for open fracture
        "S32482D",  # Displaced dome fracture of left acetabulum, subsequent encounter for fracture with routine healing
        "S32482G",  # Displaced dome fracture of left acetabulum, subsequent encounter for fracture with delayed healing
        "S32482K",  # Displaced dome fracture of left acetabulum, subsequent encounter for fracture with nonunion
        "S32482S",  # Displaced dome fracture of left acetabulum, sequela
        "S32483A",  # Displaced dome fracture of unspecified acetabulum, initial encounter for closed fracture
        "S32483B",  # Displaced dome fracture of unspecified acetabulum, initial encounter for open fracture
        "S32483D",  # Displaced dome fracture of unspecified acetabulum, subsequent encounter for fracture with routine healing
        "S32483G",  # Displaced dome fracture of unspecified acetabulum, subsequent encounter for fracture with delayed healing
        "S32483K",  # Displaced dome fracture of unspecified acetabulum, subsequent encounter for fracture with nonunion
        "S32483S",  # Displaced dome fracture of unspecified acetabulum, sequela
        "S32484A",  # Nondisplaced dome fracture of right acetabulum, initial encounter for closed fracture
        "S32484B",  # Nondisplaced dome fracture of right acetabulum, initial encounter for open fracture
        "S32484D",  # Nondisplaced dome fracture of right acetabulum, subsequent encounter for fracture with routine healing
        "S32484G",  # Nondisplaced dome fracture of right acetabulum, subsequent encounter for fracture with delayed healing
        "S32484K",  # Nondisplaced dome fracture of right acetabulum, subsequent encounter for fracture with nonunion
        "S32484S",  # Nondisplaced dome fracture of right acetabulum, sequela
        "S32485A",  # Nondisplaced dome fracture of left acetabulum, initial encounter for closed fracture
        "S32485B",  # Nondisplaced dome fracture of left acetabulum, initial encounter for open fracture
        "S32485D",  # Nondisplaced dome fracture of left acetabulum, subsequent encounter for fracture with routine healing
        "S32485G",  # Nondisplaced dome fracture of left acetabulum, subsequent encounter for fracture with delayed healing
        "S32485K",  # Nondisplaced dome fracture of left acetabulum, subsequent encounter for fracture with nonunion
        "S32485S",  # Nondisplaced dome fracture of left acetabulum, sequela
        "S32486A",  # Nondisplaced dome fracture of unspecified acetabulum, initial encounter for closed fracture
        "S32486B",  # Nondisplaced dome fracture of unspecified acetabulum, initial encounter for open fracture
        "S32486D",  # Nondisplaced dome fracture of unspecified acetabulum, subsequent encounter for fracture with routine healing
        "S32486G",  # Nondisplaced dome fracture of unspecified acetabulum, subsequent encounter for fracture with delayed healing
        "S32486K",  # Nondisplaced dome fracture of unspecified acetabulum, subsequent encounter for fracture with nonunion
        "S32486S",  # Nondisplaced dome fracture of unspecified acetabulum, sequela
        "S32491A",  # Other specified fracture of right acetabulum, initial encounter for closed fracture
        "S32491B",  # Other specified fracture of right acetabulum, initial encounter for open fracture
        "S32491D",  # Other specified fracture of right acetabulum, subsequent encounter for fracture with routine healing
        "S32491G",  # Other specified fracture of right acetabulum, subsequent encounter for fracture with delayed healing
        "S32491K",  # Other specified fracture of right acetabulum, subsequent encounter for fracture with nonunion
        "S32491S",  # Other specified fracture of right acetabulum, sequela
        "S32492A",  # Other specified fracture of left acetabulum, initial encounter for closed fracture
        "S32492B",  # Other specified fracture of left acetabulum, initial encounter for open fracture
        "S32492D",  # Other specified fracture of left acetabulum, subsequent encounter for fracture with routine healing
        "S32492G",  # Other specified fracture of left acetabulum, subsequent encounter for fracture with delayed healing
        "S32492K",  # Other specified fracture of left acetabulum, subsequent encounter for fracture with nonunion
        "S32492S",  # Other specified fracture of left acetabulum, sequela
        "S32499A",  # Other specified fracture of unspecified acetabulum, initial encounter for closed fracture
        "S32499B",  # Other specified fracture of unspecified acetabulum, initial encounter for open fracture
        "S32499D",  # Other specified fracture of unspecified acetabulum, subsequent encounter for fracture with routine healing
        "S32499G",  # Other specified fracture of unspecified acetabulum, subsequent encounter for fracture with delayed healing
        "S32499K",  # Other specified fracture of unspecified acetabulum, subsequent encounter for fracture with nonunion
        "S32499S",  # Other specified fracture of unspecified acetabulum, sequela
        "S32501A",  # Unspecified fracture of right pubis, initial encounter for closed fracture
        "S32501B",  # Unspecified fracture of right pubis, initial encounter for open fracture
        "S32501D",  # Unspecified fracture of right pubis, subsequent encounter for fracture with routine healing
        "S32501G",  # Unspecified fracture of right pubis, subsequent encounter for fracture with delayed healing
        "S32501K",  # Unspecified fracture of right pubis, subsequent encounter for fracture with nonunion
        "S32501S",  # Unspecified fracture of right pubis, sequela
        "S32502A",  # Unspecified fracture of left pubis, initial encounter for closed fracture
        "S32502B",  # Unspecified fracture of left pubis, initial encounter for open fracture
        "S32502D",  # Unspecified fracture of left pubis, subsequent encounter for fracture with routine healing
        "S32502G",  # Unspecified fracture of left pubis, subsequent encounter for fracture with delayed healing
        "S32502K",  # Unspecified fracture of left pubis, subsequent encounter for fracture with nonunion
        "S32502S",  # Unspecified fracture of left pubis, sequela
        "S32509A",  # Unspecified fracture of unspecified pubis, initial encounter for closed fracture
        "S32509B",  # Unspecified fracture of unspecified pubis, initial encounter for open fracture
        "S32509D",  # Unspecified fracture of unspecified pubis, subsequent encounter for fracture with routine healing
        "S32509G",  # Unspecified fracture of unspecified pubis, subsequent encounter for fracture with delayed healing
        "S32509K",  # Unspecified fracture of unspecified pubis, subsequent encounter for fracture with nonunion
        "S32509S",  # Unspecified fracture of unspecified pubis, sequela
        "S32511A",  # Fracture of superior rim of right pubis, initial encounter for closed fracture
        "S32511B",  # Fracture of superior rim of right pubis, initial encounter for open fracture
        "S32511D",  # Fracture of superior rim of right pubis, subsequent encounter for fracture with routine healing
        "S32511G",  # Fracture of superior rim of right pubis, subsequent encounter for fracture with delayed healing
        "S32511K",  # Fracture of superior rim of right pubis, subsequent encounter for fracture with nonunion
        "S32511S",  # Fracture of superior rim of right pubis, sequela
        "S32512A",  # Fracture of superior rim of left pubis, initial encounter for closed fracture
        "S32512B",  # Fracture of superior rim of left pubis, initial encounter for open fracture
        "S32512D",  # Fracture of superior rim of left pubis, subsequent encounter for fracture with routine healing
        "S32512G",  # Fracture of superior rim of left pubis, subsequent encounter for fracture with delayed healing
        "S32512K",  # Fracture of superior rim of left pubis, subsequent encounter for fracture with nonunion
        "S32512S",  # Fracture of superior rim of left pubis, sequela
        "S32519A",  # Fracture of superior rim of unspecified pubis, initial encounter for closed fracture
        "S32519B",  # Fracture of superior rim of unspecified pubis, initial encounter for open fracture
        "S32519D",  # Fracture of superior rim of unspecified pubis, subsequent encounter for fracture with routine healing
        "S32519G",  # Fracture of superior rim of unspecified pubis, subsequent encounter for fracture with delayed healing
        "S32519K",  # Fracture of superior rim of unspecified pubis, subsequent encounter for fracture with nonunion
        "S32519S",  # Fracture of superior rim of unspecified pubis, sequela
        "S32591A",  # Other specified fracture of right pubis, initial encounter for closed fracture
        "S32591B",  # Other specified fracture of right pubis, initial encounter for open fracture
        "S32591D",  # Other specified fracture of right pubis, subsequent encounter for fracture with routine healing
        "S32591G",  # Other specified fracture of right pubis, subsequent encounter for fracture with delayed healing
        "S32591K",  # Other specified fracture of right pubis, subsequent encounter for fracture with nonunion
        "S32591S",  # Other specified fracture of right pubis, sequela
        "S32592A",  # Other specified fracture of left pubis, initial encounter for closed fracture
        "S32592B",  # Other specified fracture of left pubis, initial encounter for open fracture
        "S32592D",  # Other specified fracture of left pubis, subsequent encounter for fracture with routine healing
        "S32592G",  # Other specified fracture of left pubis, subsequent encounter for fracture with delayed healing
        "S32592K",  # Other specified fracture of left pubis, subsequent encounter for fracture with nonunion
        "S32592S",  # Other specified fracture of left pubis, sequela
        "S32599A",  # Other specified fracture of unspecified pubis, initial encounter for closed fracture
        "S32599B",  # Other specified fracture of unspecified pubis, initial encounter for open fracture
        "S32599D",  # Other specified fracture of unspecified pubis, subsequent encounter for fracture with routine healing
        "S32599G",  # Other specified fracture of unspecified pubis, subsequent encounter for fracture with delayed healing
        "S32599K",  # Other specified fracture of unspecified pubis, subsequent encounter for fracture with nonunion
        "S32599S",  # Other specified fracture of unspecified pubis, sequela
        "S32601A",  # Unspecified fracture of right ischium, initial encounter for closed fracture
        "S32601B",  # Unspecified fracture of right ischium, initial encounter for open fracture
        "S32601D",  # Unspecified fracture of right ischium, subsequent encounter for fracture with routine healing
        "S32601G",  # Unspecified fracture of right ischium, subsequent encounter for fracture with delayed healing
        "S32601K",  # Unspecified fracture of right ischium, subsequent encounter for fracture with nonunion
        "S32601S",  # Unspecified fracture of right ischium, sequela
        "S32602A",  # Unspecified fracture of left ischium, initial encounter for closed fracture
        "S32602B",  # Unspecified fracture of left ischium, initial encounter for open fracture
        "S32602D",  # Unspecified fracture of left ischium, subsequent encounter for fracture with routine healing
        "S32602G",  # Unspecified fracture of left ischium, subsequent encounter for fracture with delayed healing
        "S32602K",  # Unspecified fracture of left ischium, subsequent encounter for fracture with nonunion
        "S32602S",  # Unspecified fracture of left ischium, sequela
        "S32609A",  # Unspecified fracture of unspecified ischium, initial encounter for closed fracture
        "S32609B",  # Unspecified fracture of unspecified ischium, initial encounter for open fracture
        "S32609D",  # Unspecified fracture of unspecified ischium, subsequent encounter for fracture with routine healing
        "S32609G",  # Unspecified fracture of unspecified ischium, subsequent encounter for fracture with delayed healing
        "S32609K",  # Unspecified fracture of unspecified ischium, subsequent encounter for fracture with nonunion
        "S32609S",  # Unspecified fracture of unspecified ischium, sequela
        "S32611A",  # Displaced avulsion fracture of right ischium, initial encounter for closed fracture
        "S32611B",  # Displaced avulsion fracture of right ischium, initial encounter for open fracture
        "S32611D",  # Displaced avulsion fracture of right ischium, subsequent encounter for fracture with routine healing
        "S32611G",  # Displaced avulsion fracture of right ischium, subsequent encounter for fracture with delayed healing
        "S32611K",  # Displaced avulsion fracture of right ischium, subsequent encounter for fracture with nonunion
        "S32611S",  # Displaced avulsion fracture of right ischium, sequela
        "S32612A",  # Displaced avulsion fracture of left ischium, initial encounter for closed fracture
        "S32612B",  # Displaced avulsion fracture of left ischium, initial encounter for open fracture
        "S32612D",  # Displaced avulsion fracture of left ischium, subsequent encounter for fracture with routine healing
        "S32612G",  # Displaced avulsion fracture of left ischium, subsequent encounter for fracture with delayed healing
        "S32612K",  # Displaced avulsion fracture of left ischium, subsequent encounter for fracture with nonunion
        "S32612S",  # Displaced avulsion fracture of left ischium, sequela
        "S32613A",  # Displaced avulsion fracture of unspecified ischium, initial encounter for closed fracture
        "S32613B",  # Displaced avulsion fracture of unspecified ischium, initial encounter for open fracture
        "S32613D",  # Displaced avulsion fracture of unspecified ischium, subsequent encounter for fracture with routine healing
        "S32613G",  # Displaced avulsion fracture of unspecified ischium, subsequent encounter for fracture with delayed healing
        "S32613K",  # Displaced avulsion fracture of unspecified ischium, subsequent encounter for fracture with nonunion
        "S32613S",  # Displaced avulsion fracture of unspecified ischium, sequela
        "S32614A",  # Nondisplaced avulsion fracture of right ischium, initial encounter for closed fracture
        "S32614B",  # Nondisplaced avulsion fracture of right ischium, initial encounter for open fracture
        "S32614D",  # Nondisplaced avulsion fracture of right ischium, subsequent encounter for fracture with routine healing
        "S32614G",  # Nondisplaced avulsion fracture of right ischium, subsequent encounter for fracture with delayed healing
        "S32614K",  # Nondisplaced avulsion fracture of right ischium, subsequent encounter for fracture with nonunion
        "S32614S",  # Nondisplaced avulsion fracture of right ischium, sequela
        "S32615A",  # Nondisplaced avulsion fracture of left ischium, initial encounter for closed fracture
        "S32615B",  # Nondisplaced avulsion fracture of left ischium, initial encounter for open fracture
        "S32615D",  # Nondisplaced avulsion fracture of left ischium, subsequent encounter for fracture with routine healing
        "S32615G",  # Nondisplaced avulsion fracture of left ischium, subsequent encounter for fracture with delayed healing
        "S32615K",  # Nondisplaced avulsion fracture of left ischium, subsequent encounter for fracture with nonunion
        "S32615S",  # Nondisplaced avulsion fracture of left ischium, sequela
        "S32616A",  # Nondisplaced avulsion fracture of unspecified ischium, initial encounter for closed fracture
        "S32616B",  # Nondisplaced avulsion fracture of unspecified ischium, initial encounter for open fracture
        "S32616D",  # Nondisplaced avulsion fracture of unspecified ischium, subsequent encounter for fracture with routine healing
        "S32616G",  # Nondisplaced avulsion fracture of unspecified ischium, subsequent encounter for fracture with delayed healing
        "S32616K",  # Nondisplaced avulsion fracture of unspecified ischium, subsequent encounter for fracture with nonunion
        "S32616S",  # Nondisplaced avulsion fracture of unspecified ischium, sequela
        "S32691A",  # Other specified fracture of right ischium, initial encounter for closed fracture
        "S32691B",  # Other specified fracture of right ischium, initial encounter for open fracture
        "S32691D",  # Other specified fracture of right ischium, subsequent encounter for fracture with routine healing
        "S32691G",  # Other specified fracture of right ischium, subsequent encounter for fracture with delayed healing
        "S32691K",  # Other specified fracture of right ischium, subsequent encounter for fracture with nonunion
        "S32691S",  # Other specified fracture of right ischium, sequela
        "S32692A",  # Other specified fracture of left ischium, initial encounter for closed fracture
        "S32692B",  # Other specified fracture of left ischium, initial encounter for open fracture
        "S32692D",  # Other specified fracture of left ischium, subsequent encounter for fracture with routine healing
        "S32692G",  # Other specified fracture of left ischium, subsequent encounter for fracture with delayed healing
        "S32692K",  # Other specified fracture of left ischium, subsequent encounter for fracture with nonunion
        "S32692S",  # Other specified fracture of left ischium, sequela
        "S32699A",  # Other specified fracture of unspecified ischium, initial encounter for closed fracture
        "S32699B",  # Other specified fracture of unspecified ischium, initial encounter for open fracture
        "S32699D",  # Other specified fracture of unspecified ischium, subsequent encounter for fracture with routine healing
        "S32699G",  # Other specified fracture of unspecified ischium, subsequent encounter for fracture with delayed healing
        "S32699K",  # Other specified fracture of unspecified ischium, subsequent encounter for fracture with nonunion
        "S32699S",  # Other specified fracture of unspecified ischium, sequela
        "S32810A",  # Multiple fractures of pelvis with stable disruption of pelvic ring, initial encounter for closed fracture
        "S32810B",  # Multiple fractures of pelvis with stable disruption of pelvic ring, initial encounter for open fracture
        "S32810D",  # Multiple fractures of pelvis with stable disruption of pelvic ring, subsequent encounter for fracture with routine healing
        "S32810G",  # Multiple fractures of pelvis with stable disruption of pelvic ring, subsequent encounter for fracture with delayed healing
        "S32810K",  # Multiple fractures of pelvis with stable disruption of pelvic ring, subsequent encounter for fracture with nonunion
        "S32810S",  # Multiple fractures of pelvis with stable disruption of pelvic ring, sequela
        "S32811A",  # Multiple fractures of pelvis with unstable disruption of pelvic ring, initial encounter for closed fracture
        "S32811B",  # Multiple fractures of pelvis with unstable disruption of pelvic ring, initial encounter for open fracture
        "S32811D",  # Multiple fractures of pelvis with unstable disruption of pelvic ring, subsequent encounter for fracture with routine healing
        "S32811G",  # Multiple fractures of pelvis with unstable disruption of pelvic ring, subsequent encounter for fracture with delayed healing
        "S32811K",  # Multiple fractures of pelvis with unstable disruption of pelvic ring, subsequent encounter for fracture with nonunion
        "S32811S",  # Multiple fractures of pelvis with unstable disruption of pelvic ring, sequela
        "S3282XA",  # Multiple fractures of pelvis without disruption of pelvic ring, initial encounter for closed fracture
        "S3282XB",  # Multiple fractures of pelvis without disruption of pelvic ring, initial encounter for open fracture
        "S3282XD",  # Multiple fractures of pelvis without disruption of pelvic ring, subsequent encounter for fracture with routine healing
        "S3282XG",  # Multiple fractures of pelvis without disruption of pelvic ring, subsequent encounter for fracture with delayed healing
        "S3282XK",  # Multiple fractures of pelvis without disruption of pelvic ring, subsequent encounter for fracture with nonunion
        "S3282XS",  # Multiple fractures of pelvis without disruption of pelvic ring, sequela
        "S3289XA",  # Fracture of other parts of pelvis, initial encounter for closed fracture
        "S3289XB",  # Fracture of other parts of pelvis, initial encounter for open fracture
        "S3289XD",  # Fracture of other parts of pelvis, subsequent encounter for fracture with routine healing
        "S3289XG",  # Fracture of other parts of pelvis, subsequent encounter for fracture with delayed healing
        "S3289XK",  # Fracture of other parts of pelvis, subsequent encounter for fracture with nonunion
        "S3289XS",  # Fracture of other parts of pelvis, sequela
        "S72001A",  # Fracture of unspecified part of neck of right femur, initial encounter for closed fracture
        "S72001B",  # Fracture of unspecified part of neck of right femur, initial encounter for open fracture type I or II
        "S72001C",  # Fracture of unspecified part of neck of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72001D",  # Fracture of unspecified part of neck of right femur, subsequent encounter for closed fracture with routine healing
        "S72001E",  # Fracture of unspecified part of neck of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72001F",  # Fracture of unspecified part of neck of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72001G",  # Fracture of unspecified part of neck of right femur, subsequent encounter for closed fracture with delayed healing
        "S72001H",  # Fracture of unspecified part of neck of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72001J",  # Fracture of unspecified part of neck of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72001K",  # Fracture of unspecified part of neck of right femur, subsequent encounter for closed fracture with nonunion
        "S72001M",  # Fracture of unspecified part of neck of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72001N",  # Fracture of unspecified part of neck of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72001P",  # Fracture of unspecified part of neck of right femur, subsequent encounter for closed fracture with malunion
        "S72001Q",  # Fracture of unspecified part of neck of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72001R",  # Fracture of unspecified part of neck of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72001S",  # Fracture of unspecified part of neck of right femur, sequela
        "S72002A",  # Fracture of unspecified part of neck of left femur, initial encounter for closed fracture
        "S72002B",  # Fracture of unspecified part of neck of left femur, initial encounter for open fracture type I or II
        "S72002C",  # Fracture of unspecified part of neck of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72002D",  # Fracture of unspecified part of neck of left femur, subsequent encounter for closed fracture with routine healing
        "S72002E",  # Fracture of unspecified part of neck of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72002F",  # Fracture of unspecified part of neck of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72002G",  # Fracture of unspecified part of neck of left femur, subsequent encounter for closed fracture with delayed healing
        "S72002H",  # Fracture of unspecified part of neck of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72002J",  # Fracture of unspecified part of neck of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72002K",  # Fracture of unspecified part of neck of left femur, subsequent encounter for closed fracture with nonunion
        "S72002M",  # Fracture of unspecified part of neck of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72002N",  # Fracture of unspecified part of neck of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72002P",  # Fracture of unspecified part of neck of left femur, subsequent encounter for closed fracture with malunion
        "S72002Q",  # Fracture of unspecified part of neck of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72002R",  # Fracture of unspecified part of neck of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72002S",  # Fracture of unspecified part of neck of left femur, sequela
        "S72009A",  # Fracture of unspecified part of neck of unspecified femur, initial encounter for closed fracture
        "S72009B",  # Fracture of unspecified part of neck of unspecified femur, initial encounter for open fracture type I or II
        "S72009C",  # Fracture of unspecified part of neck of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72009D",  # Fracture of unspecified part of neck of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72009E",  # Fracture of unspecified part of neck of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72009F",  # Fracture of unspecified part of neck of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72009G",  # Fracture of unspecified part of neck of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72009H",  # Fracture of unspecified part of neck of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72009J",  # Fracture of unspecified part of neck of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72009K",  # Fracture of unspecified part of neck of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72009M",  # Fracture of unspecified part of neck of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72009N",  # Fracture of unspecified part of neck of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72009P",  # Fracture of unspecified part of neck of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72009Q",  # Fracture of unspecified part of neck of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72009R",  # Fracture of unspecified part of neck of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72009S",  # Fracture of unspecified part of neck of unspecified femur, sequela
        "S72011A",  # Unspecified intracapsular fracture of right femur, initial encounter for closed fracture
        "S72011B",  # Unspecified intracapsular fracture of right femur, initial encounter for open fracture type I or II
        "S72011C",  # Unspecified intracapsular fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72011D",  # Unspecified intracapsular fracture of right femur, subsequent encounter for closed fracture with routine healing
        "S72011E",  # Unspecified intracapsular fracture of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72011F",  # Unspecified intracapsular fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72011G",  # Unspecified intracapsular fracture of right femur, subsequent encounter for closed fracture with delayed healing
        "S72011H",  # Unspecified intracapsular fracture of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72011J",  # Unspecified intracapsular fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72011K",  # Unspecified intracapsular fracture of right femur, subsequent encounter for closed fracture with nonunion
        "S72011M",  # Unspecified intracapsular fracture of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72011N",  # Unspecified intracapsular fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72011P",  # Unspecified intracapsular fracture of right femur, subsequent encounter for closed fracture with malunion
        "S72011Q",  # Unspecified intracapsular fracture of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72011R",  # Unspecified intracapsular fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72011S",  # Unspecified intracapsular fracture of right femur, sequela
        "S72012A",  # Unspecified intracapsular fracture of left femur, initial encounter for closed fracture
        "S72012B",  # Unspecified intracapsular fracture of left femur, initial encounter for open fracture type I or II
        "S72012C",  # Unspecified intracapsular fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72012D",  # Unspecified intracapsular fracture of left femur, subsequent encounter for closed fracture with routine healing
        "S72012E",  # Unspecified intracapsular fracture of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72012F",  # Unspecified intracapsular fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72012G",  # Unspecified intracapsular fracture of left femur, subsequent encounter for closed fracture with delayed healing
        "S72012H",  # Unspecified intracapsular fracture of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72012J",  # Unspecified intracapsular fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72012K",  # Unspecified intracapsular fracture of left femur, subsequent encounter for closed fracture with nonunion
        "S72012M",  # Unspecified intracapsular fracture of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72012N",  # Unspecified intracapsular fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72012P",  # Unspecified intracapsular fracture of left femur, subsequent encounter for closed fracture with malunion
        "S72012Q",  # Unspecified intracapsular fracture of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72012R",  # Unspecified intracapsular fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72012S",  # Unspecified intracapsular fracture of left femur, sequela
        "S72019A",  # Unspecified intracapsular fracture of unspecified femur, initial encounter for closed fracture
        "S72019B",  # Unspecified intracapsular fracture of unspecified femur, initial encounter for open fracture type I or II
        "S72019C",  # Unspecified intracapsular fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72019D",  # Unspecified intracapsular fracture of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72019E",  # Unspecified intracapsular fracture of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72019F",  # Unspecified intracapsular fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72019G",  # Unspecified intracapsular fracture of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72019H",  # Unspecified intracapsular fracture of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72019J",  # Unspecified intracapsular fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72019K",  # Unspecified intracapsular fracture of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72019M",  # Unspecified intracapsular fracture of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72019N",  # Unspecified intracapsular fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72019P",  # Unspecified intracapsular fracture of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72019Q",  # Unspecified intracapsular fracture of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72019R",  # Unspecified intracapsular fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72019S",  # Unspecified intracapsular fracture of unspecified femur, sequela
        "S72021A",  # Displaced fracture of epiphysis (separation) (upper) of right femur, initial encounter for closed fracture
        "S72021B",  # Displaced fracture of epiphysis (separation) (upper) of right femur, initial encounter for open fracture type I or II
        "S72021C",  # Displaced fracture of epiphysis (separation) (upper) of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72021D",  # Displaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for closed fracture with routine healing
        "S72021E",  # Displaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72021F",  # Displaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72021G",  # Displaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for closed fracture with delayed healing
        "S72021H",  # Displaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72021J",  # Displaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72021K",  # Displaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for closed fracture with nonunion
        "S72021M",  # Displaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72021N",  # Displaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72021P",  # Displaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for closed fracture with malunion
        "S72021Q",  # Displaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72021R",  # Displaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72021S",  # Displaced fracture of epiphysis (separation) (upper) of right femur, sequela
        "S72022A",  # Displaced fracture of epiphysis (separation) (upper) of left femur, initial encounter for closed fracture
        "S72022B",  # Displaced fracture of epiphysis (separation) (upper) of left femur, initial encounter for open fracture type I or II
        "S72022C",  # Displaced fracture of epiphysis (separation) (upper) of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72022D",  # Displaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for closed fracture with routine healing
        "S72022E",  # Displaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72022F",  # Displaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72022G",  # Displaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for closed fracture with delayed healing
        "S72022H",  # Displaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72022J",  # Displaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72022K",  # Displaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for closed fracture with nonunion
        "S72022M",  # Displaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72022N",  # Displaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72022P",  # Displaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for closed fracture with malunion
        "S72022Q",  # Displaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72022R",  # Displaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72022S",  # Displaced fracture of epiphysis (separation) (upper) of left femur, sequela
        "S72023A",  # Displaced fracture of epiphysis (separation) (upper) of unspecified femur, initial encounter for closed fracture
        "S72023B",  # Displaced fracture of epiphysis (separation) (upper) of unspecified femur, initial encounter for open fracture type I or II
        "S72023C",  # Displaced fracture of epiphysis (separation) (upper) of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72023D",  # Displaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72023E",  # Displaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72023F",  # Displaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72023G",  # Displaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72023H",  # Displaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72023J",  # Displaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72023K",  # Displaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72023M",  # Displaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72023N",  # Displaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72023P",  # Displaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72023Q",  # Displaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72023R",  # Displaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72023S",  # Displaced fracture of epiphysis (separation) (upper) of unspecified femur, sequela
        "S72024A",  # Nondisplaced fracture of epiphysis (separation) (upper) of right femur, initial encounter for closed fracture
        "S72024B",  # Nondisplaced fracture of epiphysis (separation) (upper) of right femur, initial encounter for open fracture type I or II
        "S72024C",  # Nondisplaced fracture of epiphysis (separation) (upper) of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72024D",  # Nondisplaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for closed fracture with routine healing
        "S72024E",  # Nondisplaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72024F",  # Nondisplaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72024G",  # Nondisplaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for closed fracture with delayed healing
        "S72024H",  # Nondisplaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72024J",  # Nondisplaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72024K",  # Nondisplaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for closed fracture with nonunion
        "S72024M",  # Nondisplaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72024N",  # Nondisplaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72024P",  # Nondisplaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for closed fracture with malunion
        "S72024Q",  # Nondisplaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72024R",  # Nondisplaced fracture of epiphysis (separation) (upper) of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72024S",  # Nondisplaced fracture of epiphysis (separation) (upper) of right femur, sequela
        "S72025A",  # Nondisplaced fracture of epiphysis (separation) (upper) of left femur, initial encounter for closed fracture
        "S72025B",  # Nondisplaced fracture of epiphysis (separation) (upper) of left femur, initial encounter for open fracture type I or II
        "S72025C",  # Nondisplaced fracture of epiphysis (separation) (upper) of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72025D",  # Nondisplaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for closed fracture with routine healing
        "S72025E",  # Nondisplaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72025F",  # Nondisplaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72025G",  # Nondisplaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for closed fracture with delayed healing
        "S72025H",  # Nondisplaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72025J",  # Nondisplaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72025K",  # Nondisplaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for closed fracture with nonunion
        "S72025M",  # Nondisplaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72025N",  # Nondisplaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72025P",  # Nondisplaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for closed fracture with malunion
        "S72025Q",  # Nondisplaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72025R",  # Nondisplaced fracture of epiphysis (separation) (upper) of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72025S",  # Nondisplaced fracture of epiphysis (separation) (upper) of left femur, sequela
        "S72026A",  # Nondisplaced fracture of epiphysis (separation) (upper) of unspecified femur, initial encounter for closed fracture
        "S72026B",  # Nondisplaced fracture of epiphysis (separation) (upper) of unspecified femur, initial encounter for open fracture type I or II
        "S72026C",  # Nondisplaced fracture of epiphysis (separation) (upper) of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72026D",  # Nondisplaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72026E",  # Nondisplaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72026F",  # Nondisplaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72026G",  # Nondisplaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72026H",  # Nondisplaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72026J",  # Nondisplaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72026K",  # Nondisplaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72026M",  # Nondisplaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72026N",  # Nondisplaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72026P",  # Nondisplaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72026Q",  # Nondisplaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72026R",  # Nondisplaced fracture of epiphysis (separation) (upper) of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72026S",  # Nondisplaced fracture of epiphysis (separation) (upper) of unspecified femur, sequela
        "S72031A",  # Displaced midcervical fracture of right femur, initial encounter for closed fracture
        "S72031B",  # Displaced midcervical fracture of right femur, initial encounter for open fracture type I or II
        "S72031C",  # Displaced midcervical fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72031D",  # Displaced midcervical fracture of right femur, subsequent encounter for closed fracture with routine healing
        "S72031E",  # Displaced midcervical fracture of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72031F",  # Displaced midcervical fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72031G",  # Displaced midcervical fracture of right femur, subsequent encounter for closed fracture with delayed healing
        "S72031H",  # Displaced midcervical fracture of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72031J",  # Displaced midcervical fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72031K",  # Displaced midcervical fracture of right femur, subsequent encounter for closed fracture with nonunion
        "S72031M",  # Displaced midcervical fracture of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72031N",  # Displaced midcervical fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72031P",  # Displaced midcervical fracture of right femur, subsequent encounter for closed fracture with malunion
        "S72031Q",  # Displaced midcervical fracture of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72031R",  # Displaced midcervical fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72031S",  # Displaced midcervical fracture of right femur, sequela
        "S72032A",  # Displaced midcervical fracture of left femur, initial encounter for closed fracture
        "S72032B",  # Displaced midcervical fracture of left femur, initial encounter for open fracture type I or II
        "S72032C",  # Displaced midcervical fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72032D",  # Displaced midcervical fracture of left femur, subsequent encounter for closed fracture with routine healing
        "S72032E",  # Displaced midcervical fracture of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72032F",  # Displaced midcervical fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72032G",  # Displaced midcervical fracture of left femur, subsequent encounter for closed fracture with delayed healing
        "S72032H",  # Displaced midcervical fracture of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72032J",  # Displaced midcervical fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72032K",  # Displaced midcervical fracture of left femur, subsequent encounter for closed fracture with nonunion
        "S72032M",  # Displaced midcervical fracture of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72032N",  # Displaced midcervical fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72032P",  # Displaced midcervical fracture of left femur, subsequent encounter for closed fracture with malunion
        "S72032Q",  # Displaced midcervical fracture of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72032R",  # Displaced midcervical fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72032S",  # Displaced midcervical fracture of left femur, sequela
        "S72033A",  # Displaced midcervical fracture of unspecified femur, initial encounter for closed fracture
        "S72033B",  # Displaced midcervical fracture of unspecified femur, initial encounter for open fracture type I or II
        "S72033C",  # Displaced midcervical fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72033D",  # Displaced midcervical fracture of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72033E",  # Displaced midcervical fracture of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72033F",  # Displaced midcervical fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72033G",  # Displaced midcervical fracture of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72033H",  # Displaced midcervical fracture of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72033J",  # Displaced midcervical fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72033K",  # Displaced midcervical fracture of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72033M",  # Displaced midcervical fracture of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72033N",  # Displaced midcervical fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72033P",  # Displaced midcervical fracture of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72033Q",  # Displaced midcervical fracture of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72033R",  # Displaced midcervical fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72033S",  # Displaced midcervical fracture of unspecified femur, sequela
        "S72034A",  # Nondisplaced midcervical fracture of right femur, initial encounter for closed fracture
        "S72034B",  # Nondisplaced midcervical fracture of right femur, initial encounter for open fracture type I or II
        "S72034C",  # Nondisplaced midcervical fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72034D",  # Nondisplaced midcervical fracture of right femur, subsequent encounter for closed fracture with routine healing
        "S72034E",  # Nondisplaced midcervical fracture of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72034F",  # Nondisplaced midcervical fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72034G",  # Nondisplaced midcervical fracture of right femur, subsequent encounter for closed fracture with delayed healing
        "S72034H",  # Nondisplaced midcervical fracture of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72034J",  # Nondisplaced midcervical fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72034K",  # Nondisplaced midcervical fracture of right femur, subsequent encounter for closed fracture with nonunion
        "S72034M",  # Nondisplaced midcervical fracture of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72034N",  # Nondisplaced midcervical fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72034P",  # Nondisplaced midcervical fracture of right femur, subsequent encounter for closed fracture with malunion
        "S72034Q",  # Nondisplaced midcervical fracture of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72034R",  # Nondisplaced midcervical fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72034S",  # Nondisplaced midcervical fracture of right femur, sequela
        "S72035A",  # Nondisplaced midcervical fracture of left femur, initial encounter for closed fracture
        "S72035B",  # Nondisplaced midcervical fracture of left femur, initial encounter for open fracture type I or II
        "S72035C",  # Nondisplaced midcervical fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72035D",  # Nondisplaced midcervical fracture of left femur, subsequent encounter for closed fracture with routine healing
        "S72035E",  # Nondisplaced midcervical fracture of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72035F",  # Nondisplaced midcervical fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72035G",  # Nondisplaced midcervical fracture of left femur, subsequent encounter for closed fracture with delayed healing
        "S72035H",  # Nondisplaced midcervical fracture of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72035J",  # Nondisplaced midcervical fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72035K",  # Nondisplaced midcervical fracture of left femur, subsequent encounter for closed fracture with nonunion
        "S72035M",  # Nondisplaced midcervical fracture of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72035N",  # Nondisplaced midcervical fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72035P",  # Nondisplaced midcervical fracture of left femur, subsequent encounter for closed fracture with malunion
        "S72035Q",  # Nondisplaced midcervical fracture of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72035R",  # Nondisplaced midcervical fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72035S",  # Nondisplaced midcervical fracture of left femur, sequela
        "S72036A",  # Nondisplaced midcervical fracture of unspecified femur, initial encounter for closed fracture
        "S72036B",  # Nondisplaced midcervical fracture of unspecified femur, initial encounter for open fracture type I or II
        "S72036C",  # Nondisplaced midcervical fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72036D",  # Nondisplaced midcervical fracture of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72036E",  # Nondisplaced midcervical fracture of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72036F",  # Nondisplaced midcervical fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72036G",  # Nondisplaced midcervical fracture of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72036H",  # Nondisplaced midcervical fracture of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72036J",  # Nondisplaced midcervical fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72036K",  # Nondisplaced midcervical fracture of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72036M",  # Nondisplaced midcervical fracture of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72036N",  # Nondisplaced midcervical fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72036P",  # Nondisplaced midcervical fracture of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72036Q",  # Nondisplaced midcervical fracture of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72036R",  # Nondisplaced midcervical fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72036S",  # Nondisplaced midcervical fracture of unspecified femur, sequela
        "S72041A",  # Displaced fracture of base of neck of right femur, initial encounter for closed fracture
        "S72041B",  # Displaced fracture of base of neck of right femur, initial encounter for open fracture type I or II
        "S72041C",  # Displaced fracture of base of neck of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72041D",  # Displaced fracture of base of neck of right femur, subsequent encounter for closed fracture with routine healing
        "S72041E",  # Displaced fracture of base of neck of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72041F",  # Displaced fracture of base of neck of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72041G",  # Displaced fracture of base of neck of right femur, subsequent encounter for closed fracture with delayed healing
        "S72041H",  # Displaced fracture of base of neck of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72041J",  # Displaced fracture of base of neck of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72041K",  # Displaced fracture of base of neck of right femur, subsequent encounter for closed fracture with nonunion
        "S72041M",  # Displaced fracture of base of neck of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72041N",  # Displaced fracture of base of neck of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72041P",  # Displaced fracture of base of neck of right femur, subsequent encounter for closed fracture with malunion
        "S72041Q",  # Displaced fracture of base of neck of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72041R",  # Displaced fracture of base of neck of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72041S",  # Displaced fracture of base of neck of right femur, sequela
        "S72042A",  # Displaced fracture of base of neck of left femur, initial encounter for closed fracture
        "S72042B",  # Displaced fracture of base of neck of left femur, initial encounter for open fracture type I or II
        "S72042C",  # Displaced fracture of base of neck of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72042D",  # Displaced fracture of base of neck of left femur, subsequent encounter for closed fracture with routine healing
        "S72042E",  # Displaced fracture of base of neck of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72042F",  # Displaced fracture of base of neck of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72042G",  # Displaced fracture of base of neck of left femur, subsequent encounter for closed fracture with delayed healing
        "S72042H",  # Displaced fracture of base of neck of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72042J",  # Displaced fracture of base of neck of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72042K",  # Displaced fracture of base of neck of left femur, subsequent encounter for closed fracture with nonunion
        "S72042M",  # Displaced fracture of base of neck of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72042N",  # Displaced fracture of base of neck of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72042P",  # Displaced fracture of base of neck of left femur, subsequent encounter for closed fracture with malunion
        "S72042Q",  # Displaced fracture of base of neck of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72042R",  # Displaced fracture of base of neck of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72042S",  # Displaced fracture of base of neck of left femur, sequela
        "S72043A",  # Displaced fracture of base of neck of unspecified femur, initial encounter for closed fracture
        "S72043B",  # Displaced fracture of base of neck of unspecified femur, initial encounter for open fracture type I or II
        "S72043C",  # Displaced fracture of base of neck of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72043D",  # Displaced fracture of base of neck of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72043E",  # Displaced fracture of base of neck of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72043F",  # Displaced fracture of base of neck of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72043G",  # Displaced fracture of base of neck of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72043H",  # Displaced fracture of base of neck of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72043J",  # Displaced fracture of base of neck of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72043K",  # Displaced fracture of base of neck of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72043M",  # Displaced fracture of base of neck of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72043N",  # Displaced fracture of base of neck of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72043P",  # Displaced fracture of base of neck of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72043Q",  # Displaced fracture of base of neck of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72043R",  # Displaced fracture of base of neck of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72043S",  # Displaced fracture of base of neck of unspecified femur, sequela
        "S72044A",  # Nondisplaced fracture of base of neck of right femur, initial encounter for closed fracture
        "S72044B",  # Nondisplaced fracture of base of neck of right femur, initial encounter for open fracture type I or II
        "S72044C",  # Nondisplaced fracture of base of neck of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72044D",  # Nondisplaced fracture of base of neck of right femur, subsequent encounter for closed fracture with routine healing
        "S72044E",  # Nondisplaced fracture of base of neck of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72044F",  # Nondisplaced fracture of base of neck of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72044G",  # Nondisplaced fracture of base of neck of right femur, subsequent encounter for closed fracture with delayed healing
        "S72044H",  # Nondisplaced fracture of base of neck of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72044J",  # Nondisplaced fracture of base of neck of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72044K",  # Nondisplaced fracture of base of neck of right femur, subsequent encounter for closed fracture with nonunion
        "S72044M",  # Nondisplaced fracture of base of neck of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72044N",  # Nondisplaced fracture of base of neck of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72044P",  # Nondisplaced fracture of base of neck of right femur, subsequent encounter for closed fracture with malunion
        "S72044Q",  # Nondisplaced fracture of base of neck of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72044R",  # Nondisplaced fracture of base of neck of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72044S",  # Nondisplaced fracture of base of neck of right femur, sequela
        "S72045A",  # Nondisplaced fracture of base of neck of left femur, initial encounter for closed fracture
        "S72045B",  # Nondisplaced fracture of base of neck of left femur, initial encounter for open fracture type I or II
        "S72045C",  # Nondisplaced fracture of base of neck of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72045D",  # Nondisplaced fracture of base of neck of left femur, subsequent encounter for closed fracture with routine healing
        "S72045E",  # Nondisplaced fracture of base of neck of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72045F",  # Nondisplaced fracture of base of neck of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72045G",  # Nondisplaced fracture of base of neck of left femur, subsequent encounter for closed fracture with delayed healing
        "S72045H",  # Nondisplaced fracture of base of neck of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72045J",  # Nondisplaced fracture of base of neck of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72045K",  # Nondisplaced fracture of base of neck of left femur, subsequent encounter for closed fracture with nonunion
        "S72045M",  # Nondisplaced fracture of base of neck of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72045N",  # Nondisplaced fracture of base of neck of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72045P",  # Nondisplaced fracture of base of neck of left femur, subsequent encounter for closed fracture with malunion
        "S72045Q",  # Nondisplaced fracture of base of neck of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72045R",  # Nondisplaced fracture of base of neck of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72045S",  # Nondisplaced fracture of base of neck of left femur, sequela
        "S72046A",  # Nondisplaced fracture of base of neck of unspecified femur, initial encounter for closed fracture
        "S72046B",  # Nondisplaced fracture of base of neck of unspecified femur, initial encounter for open fracture type I or II
        "S72046C",  # Nondisplaced fracture of base of neck of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72046D",  # Nondisplaced fracture of base of neck of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72046E",  # Nondisplaced fracture of base of neck of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72046F",  # Nondisplaced fracture of base of neck of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72046G",  # Nondisplaced fracture of base of neck of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72046H",  # Nondisplaced fracture of base of neck of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72046J",  # Nondisplaced fracture of base of neck of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72046K",  # Nondisplaced fracture of base of neck of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72046M",  # Nondisplaced fracture of base of neck of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72046N",  # Nondisplaced fracture of base of neck of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72046P",  # Nondisplaced fracture of base of neck of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72046Q",  # Nondisplaced fracture of base of neck of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72046R",  # Nondisplaced fracture of base of neck of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72046S",  # Nondisplaced fracture of base of neck of unspecified femur, sequela
        "S72051A",  # Unspecified fracture of head of right femur, initial encounter for closed fracture
        "S72051B",  # Unspecified fracture of head of right femur, initial encounter for open fracture type I or II
        "S72051C",  # Unspecified fracture of head of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72051D",  # Unspecified fracture of head of right femur, subsequent encounter for closed fracture with routine healing
        "S72051E",  # Unspecified fracture of head of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72051F",  # Unspecified fracture of head of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72051G",  # Unspecified fracture of head of right femur, subsequent encounter for closed fracture with delayed healing
        "S72051H",  # Unspecified fracture of head of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72051J",  # Unspecified fracture of head of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72051K",  # Unspecified fracture of head of right femur, subsequent encounter for closed fracture with nonunion
        "S72051M",  # Unspecified fracture of head of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72051N",  # Unspecified fracture of head of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72051P",  # Unspecified fracture of head of right femur, subsequent encounter for closed fracture with malunion
        "S72051Q",  # Unspecified fracture of head of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72051R",  # Unspecified fracture of head of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72051S",  # Unspecified fracture of head of right femur, sequela
        "S72052A",  # Unspecified fracture of head of left femur, initial encounter for closed fracture
        "S72052B",  # Unspecified fracture of head of left femur, initial encounter for open fracture type I or II
        "S72052C",  # Unspecified fracture of head of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72052D",  # Unspecified fracture of head of left femur, subsequent encounter for closed fracture with routine healing
        "S72052E",  # Unspecified fracture of head of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72052F",  # Unspecified fracture of head of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72052G",  # Unspecified fracture of head of left femur, subsequent encounter for closed fracture with delayed healing
        "S72052H",  # Unspecified fracture of head of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72052J",  # Unspecified fracture of head of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72052K",  # Unspecified fracture of head of left femur, subsequent encounter for closed fracture with nonunion
        "S72052M",  # Unspecified fracture of head of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72052N",  # Unspecified fracture of head of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72052P",  # Unspecified fracture of head of left femur, subsequent encounter for closed fracture with malunion
        "S72052Q",  # Unspecified fracture of head of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72052R",  # Unspecified fracture of head of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72052S",  # Unspecified fracture of head of left femur, sequela
        "S72059A",  # Unspecified fracture of head of unspecified femur, initial encounter for closed fracture
        "S72059B",  # Unspecified fracture of head of unspecified femur, initial encounter for open fracture type I or II
        "S72059C",  # Unspecified fracture of head of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72059D",  # Unspecified fracture of head of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72059E",  # Unspecified fracture of head of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72059F",  # Unspecified fracture of head of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72059G",  # Unspecified fracture of head of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72059H",  # Unspecified fracture of head of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72059J",  # Unspecified fracture of head of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72059K",  # Unspecified fracture of head of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72059M",  # Unspecified fracture of head of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72059N",  # Unspecified fracture of head of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72059P",  # Unspecified fracture of head of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72059Q",  # Unspecified fracture of head of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72059R",  # Unspecified fracture of head of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72059S",  # Unspecified fracture of head of unspecified femur, sequela
        "S72061A",  # Displaced articular fracture of head of right femur, initial encounter for closed fracture
        "S72061B",  # Displaced articular fracture of head of right femur, initial encounter for open fracture type I or II
        "S72061C",  # Displaced articular fracture of head of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72061D",  # Displaced articular fracture of head of right femur, subsequent encounter for closed fracture with routine healing
        "S72061E",  # Displaced articular fracture of head of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72061F",  # Displaced articular fracture of head of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72061G",  # Displaced articular fracture of head of right femur, subsequent encounter for closed fracture with delayed healing
        "S72061H",  # Displaced articular fracture of head of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72061J",  # Displaced articular fracture of head of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72061K",  # Displaced articular fracture of head of right femur, subsequent encounter for closed fracture with nonunion
        "S72061M",  # Displaced articular fracture of head of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72061N",  # Displaced articular fracture of head of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72061P",  # Displaced articular fracture of head of right femur, subsequent encounter for closed fracture with malunion
        "S72061Q",  # Displaced articular fracture of head of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72061R",  # Displaced articular fracture of head of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72061S",  # Displaced articular fracture of head of right femur, sequela
        "S72062A",  # Displaced articular fracture of head of left femur, initial encounter for closed fracture
        "S72062B",  # Displaced articular fracture of head of left femur, initial encounter for open fracture type I or II
        "S72062C",  # Displaced articular fracture of head of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72062D",  # Displaced articular fracture of head of left femur, subsequent encounter for closed fracture with routine healing
        "S72062E",  # Displaced articular fracture of head of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72062F",  # Displaced articular fracture of head of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72062G",  # Displaced articular fracture of head of left femur, subsequent encounter for closed fracture with delayed healing
        "S72062H",  # Displaced articular fracture of head of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72062J",  # Displaced articular fracture of head of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72062K",  # Displaced articular fracture of head of left femur, subsequent encounter for closed fracture with nonunion
        "S72062M",  # Displaced articular fracture of head of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72062N",  # Displaced articular fracture of head of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72062P",  # Displaced articular fracture of head of left femur, subsequent encounter for closed fracture with malunion
        "S72062Q",  # Displaced articular fracture of head of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72062R",  # Displaced articular fracture of head of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72062S",  # Displaced articular fracture of head of left femur, sequela
        "S72063A",  # Displaced articular fracture of head of unspecified femur, initial encounter for closed fracture
        "S72063B",  # Displaced articular fracture of head of unspecified femur, initial encounter for open fracture type I or II
        "S72063C",  # Displaced articular fracture of head of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72063D",  # Displaced articular fracture of head of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72063E",  # Displaced articular fracture of head of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72063F",  # Displaced articular fracture of head of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72063G",  # Displaced articular fracture of head of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72063H",  # Displaced articular fracture of head of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72063J",  # Displaced articular fracture of head of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72063K",  # Displaced articular fracture of head of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72063M",  # Displaced articular fracture of head of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72063N",  # Displaced articular fracture of head of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72063P",  # Displaced articular fracture of head of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72063Q",  # Displaced articular fracture of head of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72063R",  # Displaced articular fracture of head of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72063S",  # Displaced articular fracture of head of unspecified femur, sequela
        "S72064A",  # Nondisplaced articular fracture of head of right femur, initial encounter for closed fracture
        "S72064B",  # Nondisplaced articular fracture of head of right femur, initial encounter for open fracture type I or II
        "S72064C",  # Nondisplaced articular fracture of head of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72064D",  # Nondisplaced articular fracture of head of right femur, subsequent encounter for closed fracture with routine healing
        "S72064E",  # Nondisplaced articular fracture of head of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72064F",  # Nondisplaced articular fracture of head of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72064G",  # Nondisplaced articular fracture of head of right femur, subsequent encounter for closed fracture with delayed healing
        "S72064H",  # Nondisplaced articular fracture of head of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72064J",  # Nondisplaced articular fracture of head of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72064K",  # Nondisplaced articular fracture of head of right femur, subsequent encounter for closed fracture with nonunion
        "S72064M",  # Nondisplaced articular fracture of head of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72064N",  # Nondisplaced articular fracture of head of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72064P",  # Nondisplaced articular fracture of head of right femur, subsequent encounter for closed fracture with malunion
        "S72064Q",  # Nondisplaced articular fracture of head of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72064R",  # Nondisplaced articular fracture of head of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72064S",  # Nondisplaced articular fracture of head of right femur, sequela
        "S72065A",  # Nondisplaced articular fracture of head of left femur, initial encounter for closed fracture
        "S72065B",  # Nondisplaced articular fracture of head of left femur, initial encounter for open fracture type I or II
        "S72065C",  # Nondisplaced articular fracture of head of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72065D",  # Nondisplaced articular fracture of head of left femur, subsequent encounter for closed fracture with routine healing
        "S72065E",  # Nondisplaced articular fracture of head of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72065F",  # Nondisplaced articular fracture of head of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72065G",  # Nondisplaced articular fracture of head of left femur, subsequent encounter for closed fracture with delayed healing
        "S72065H",  # Nondisplaced articular fracture of head of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72065J",  # Nondisplaced articular fracture of head of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72065K",  # Nondisplaced articular fracture of head of left femur, subsequent encounter for closed fracture with nonunion
        "S72065M",  # Nondisplaced articular fracture of head of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72065N",  # Nondisplaced articular fracture of head of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72065P",  # Nondisplaced articular fracture of head of left femur, subsequent encounter for closed fracture with malunion
        "S72065Q",  # Nondisplaced articular fracture of head of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72065R",  # Nondisplaced articular fracture of head of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72065S",  # Nondisplaced articular fracture of head of left femur, sequela
        "S72066A",  # Nondisplaced articular fracture of head of unspecified femur, initial encounter for closed fracture
        "S72066B",  # Nondisplaced articular fracture of head of unspecified femur, initial encounter for open fracture type I or II
        "S72066C",  # Nondisplaced articular fracture of head of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72066D",  # Nondisplaced articular fracture of head of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72066E",  # Nondisplaced articular fracture of head of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72066F",  # Nondisplaced articular fracture of head of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72066G",  # Nondisplaced articular fracture of head of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72066H",  # Nondisplaced articular fracture of head of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72066J",  # Nondisplaced articular fracture of head of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72066K",  # Nondisplaced articular fracture of head of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72066M",  # Nondisplaced articular fracture of head of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72066N",  # Nondisplaced articular fracture of head of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72066P",  # Nondisplaced articular fracture of head of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72066Q",  # Nondisplaced articular fracture of head of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72066R",  # Nondisplaced articular fracture of head of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72066S",  # Nondisplaced articular fracture of head of unspecified femur, sequela
        "S72091A",  # Other fracture of head and neck of right femur, initial encounter for closed fracture
        "S72091B",  # Other fracture of head and neck of right femur, initial encounter for open fracture type I or II
        "S72091C",  # Other fracture of head and neck of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72091D",  # Other fracture of head and neck of right femur, subsequent encounter for closed fracture with routine healing
        "S72091E",  # Other fracture of head and neck of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72091F",  # Other fracture of head and neck of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72091G",  # Other fracture of head and neck of right femur, subsequent encounter for closed fracture with delayed healing
        "S72091H",  # Other fracture of head and neck of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72091J",  # Other fracture of head and neck of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72091K",  # Other fracture of head and neck of right femur, subsequent encounter for closed fracture with nonunion
        "S72091M",  # Other fracture of head and neck of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72091N",  # Other fracture of head and neck of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72091P",  # Other fracture of head and neck of right femur, subsequent encounter for closed fracture with malunion
        "S72091Q",  # Other fracture of head and neck of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72091R",  # Other fracture of head and neck of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72091S",  # Other fracture of head and neck of right femur, sequela
        "S72092A",  # Other fracture of head and neck of left femur, initial encounter for closed fracture
        "S72092B",  # Other fracture of head and neck of left femur, initial encounter for open fracture type I or II
        "S72092C",  # Other fracture of head and neck of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72092D",  # Other fracture of head and neck of left femur, subsequent encounter for closed fracture with routine healing
        "S72092E",  # Other fracture of head and neck of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72092F",  # Other fracture of head and neck of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72092G",  # Other fracture of head and neck of left femur, subsequent encounter for closed fracture with delayed healing
        "S72092H",  # Other fracture of head and neck of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72092J",  # Other fracture of head and neck of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72092K",  # Other fracture of head and neck of left femur, subsequent encounter for closed fracture with nonunion
        "S72092M",  # Other fracture of head and neck of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72092N",  # Other fracture of head and neck of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72092P",  # Other fracture of head and neck of left femur, subsequent encounter for closed fracture with malunion
        "S72092Q",  # Other fracture of head and neck of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72092R",  # Other fracture of head and neck of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72092S",  # Other fracture of head and neck of left femur, sequela
        "S72099A",  # Other fracture of head and neck of unspecified femur, initial encounter for closed fracture
        "S72099B",  # Other fracture of head and neck of unspecified femur, initial encounter for open fracture type I or II
        "S72099C",  # Other fracture of head and neck of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72099D",  # Other fracture of head and neck of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72099E",  # Other fracture of head and neck of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72099F",  # Other fracture of head and neck of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72099G",  # Other fracture of head and neck of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72099H",  # Other fracture of head and neck of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72099J",  # Other fracture of head and neck of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72099K",  # Other fracture of head and neck of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72099M",  # Other fracture of head and neck of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72099N",  # Other fracture of head and neck of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72099P",  # Other fracture of head and neck of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72099Q",  # Other fracture of head and neck of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72099R",  # Other fracture of head and neck of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72099S",  # Other fracture of head and neck of unspecified femur, sequela
        "S72101A",  # Unspecified trochanteric fracture of right femur, initial encounter for closed fracture
        "S72101B",  # Unspecified trochanteric fracture of right femur, initial encounter for open fracture type I or II
        "S72101C",  # Unspecified trochanteric fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72101D",  # Unspecified trochanteric fracture of right femur, subsequent encounter for closed fracture with routine healing
        "S72101E",  # Unspecified trochanteric fracture of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72101F",  # Unspecified trochanteric fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72101G",  # Unspecified trochanteric fracture of right femur, subsequent encounter for closed fracture with delayed healing
        "S72101H",  # Unspecified trochanteric fracture of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72101J",  # Unspecified trochanteric fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72101K",  # Unspecified trochanteric fracture of right femur, subsequent encounter for closed fracture with nonunion
        "S72101M",  # Unspecified trochanteric fracture of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72101N",  # Unspecified trochanteric fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72101P",  # Unspecified trochanteric fracture of right femur, subsequent encounter for closed fracture with malunion
        "S72101Q",  # Unspecified trochanteric fracture of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72101R",  # Unspecified trochanteric fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72101S",  # Unspecified trochanteric fracture of right femur, sequela
        "S72102A",  # Unspecified trochanteric fracture of left femur, initial encounter for closed fracture
        "S72102B",  # Unspecified trochanteric fracture of left femur, initial encounter for open fracture type I or II
        "S72102C",  # Unspecified trochanteric fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72102D",  # Unspecified trochanteric fracture of left femur, subsequent encounter for closed fracture with routine healing
        "S72102E",  # Unspecified trochanteric fracture of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72102F",  # Unspecified trochanteric fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72102G",  # Unspecified trochanteric fracture of left femur, subsequent encounter for closed fracture with delayed healing
        "S72102H",  # Unspecified trochanteric fracture of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72102J",  # Unspecified trochanteric fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72102K",  # Unspecified trochanteric fracture of left femur, subsequent encounter for closed fracture with nonunion
        "S72102M",  # Unspecified trochanteric fracture of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72102N",  # Unspecified trochanteric fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72102P",  # Unspecified trochanteric fracture of left femur, subsequent encounter for closed fracture with malunion
        "S72102Q",  # Unspecified trochanteric fracture of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72102R",  # Unspecified trochanteric fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72102S",  # Unspecified trochanteric fracture of left femur, sequela
        "S72109A",  # Unspecified trochanteric fracture of unspecified femur, initial encounter for closed fracture
        "S72109B",  # Unspecified trochanteric fracture of unspecified femur, initial encounter for open fracture type I or II
        "S72109C",  # Unspecified trochanteric fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72109D",  # Unspecified trochanteric fracture of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72109E",  # Unspecified trochanteric fracture of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72109F",  # Unspecified trochanteric fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72109G",  # Unspecified trochanteric fracture of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72109H",  # Unspecified trochanteric fracture of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72109J",  # Unspecified trochanteric fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72109K",  # Unspecified trochanteric fracture of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72109M",  # Unspecified trochanteric fracture of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72109N",  # Unspecified trochanteric fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72109P",  # Unspecified trochanteric fracture of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72109Q",  # Unspecified trochanteric fracture of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72109R",  # Unspecified trochanteric fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72109S",  # Unspecified trochanteric fracture of unspecified femur, sequela
        "S72111A",  # Displaced fracture of greater trochanter of right femur, initial encounter for closed fracture
        "S72111B",  # Displaced fracture of greater trochanter of right femur, initial encounter for open fracture type I or II
        "S72111C",  # Displaced fracture of greater trochanter of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72111D",  # Displaced fracture of greater trochanter of right femur, subsequent encounter for closed fracture with routine healing
        "S72111E",  # Displaced fracture of greater trochanter of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72111F",  # Displaced fracture of greater trochanter of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72111G",  # Displaced fracture of greater trochanter of right femur, subsequent encounter for closed fracture with delayed healing
        "S72111H",  # Displaced fracture of greater trochanter of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72111J",  # Displaced fracture of greater trochanter of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72111K",  # Displaced fracture of greater trochanter of right femur, subsequent encounter for closed fracture with nonunion
        "S72111M",  # Displaced fracture of greater trochanter of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72111N",  # Displaced fracture of greater trochanter of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72111P",  # Displaced fracture of greater trochanter of right femur, subsequent encounter for closed fracture with malunion
        "S72111Q",  # Displaced fracture of greater trochanter of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72111R",  # Displaced fracture of greater trochanter of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72111S",  # Displaced fracture of greater trochanter of right femur, sequela
        "S72112A",  # Displaced fracture of greater trochanter of left femur, initial encounter for closed fracture
        "S72112B",  # Displaced fracture of greater trochanter of left femur, initial encounter for open fracture type I or II
        "S72112C",  # Displaced fracture of greater trochanter of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72112D",  # Displaced fracture of greater trochanter of left femur, subsequent encounter for closed fracture with routine healing
        "S72112E",  # Displaced fracture of greater trochanter of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72112F",  # Displaced fracture of greater trochanter of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72112G",  # Displaced fracture of greater trochanter of left femur, subsequent encounter for closed fracture with delayed healing
        "S72112H",  # Displaced fracture of greater trochanter of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72112J",  # Displaced fracture of greater trochanter of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72112K",  # Displaced fracture of greater trochanter of left femur, subsequent encounter for closed fracture with nonunion
        "S72112M",  # Displaced fracture of greater trochanter of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72112N",  # Displaced fracture of greater trochanter of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72112P",  # Displaced fracture of greater trochanter of left femur, subsequent encounter for closed fracture with malunion
        "S72112Q",  # Displaced fracture of greater trochanter of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72112R",  # Displaced fracture of greater trochanter of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72112S",  # Displaced fracture of greater trochanter of left femur, sequela
        "S72113A",  # Displaced fracture of greater trochanter of unspecified femur, initial encounter for closed fracture
        "S72113B",  # Displaced fracture of greater trochanter of unspecified femur, initial encounter for open fracture type I or II
        "S72113C",  # Displaced fracture of greater trochanter of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72113D",  # Displaced fracture of greater trochanter of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72113E",  # Displaced fracture of greater trochanter of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72113F",  # Displaced fracture of greater trochanter of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72113G",  # Displaced fracture of greater trochanter of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72113H",  # Displaced fracture of greater trochanter of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72113J",  # Displaced fracture of greater trochanter of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72113K",  # Displaced fracture of greater trochanter of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72113M",  # Displaced fracture of greater trochanter of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72113N",  # Displaced fracture of greater trochanter of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72113P",  # Displaced fracture of greater trochanter of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72113Q",  # Displaced fracture of greater trochanter of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72113R",  # Displaced fracture of greater trochanter of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72113S",  # Displaced fracture of greater trochanter of unspecified femur, sequela
        "S72114A",  # Nondisplaced fracture of greater trochanter of right femur, initial encounter for closed fracture
        "S72114B",  # Nondisplaced fracture of greater trochanter of right femur, initial encounter for open fracture type I or II
        "S72114C",  # Nondisplaced fracture of greater trochanter of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72114D",  # Nondisplaced fracture of greater trochanter of right femur, subsequent encounter for closed fracture with routine healing
        "S72114E",  # Nondisplaced fracture of greater trochanter of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72114F",  # Nondisplaced fracture of greater trochanter of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72114G",  # Nondisplaced fracture of greater trochanter of right femur, subsequent encounter for closed fracture with delayed healing
        "S72114H",  # Nondisplaced fracture of greater trochanter of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72114J",  # Nondisplaced fracture of greater trochanter of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72114K",  # Nondisplaced fracture of greater trochanter of right femur, subsequent encounter for closed fracture with nonunion
        "S72114M",  # Nondisplaced fracture of greater trochanter of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72114N",  # Nondisplaced fracture of greater trochanter of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72114P",  # Nondisplaced fracture of greater trochanter of right femur, subsequent encounter for closed fracture with malunion
        "S72114Q",  # Nondisplaced fracture of greater trochanter of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72114R",  # Nondisplaced fracture of greater trochanter of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72114S",  # Nondisplaced fracture of greater trochanter of right femur, sequela
        "S72115A",  # Nondisplaced fracture of greater trochanter of left femur, initial encounter for closed fracture
        "S72115B",  # Nondisplaced fracture of greater trochanter of left femur, initial encounter for open fracture type I or II
        "S72115C",  # Nondisplaced fracture of greater trochanter of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72115D",  # Nondisplaced fracture of greater trochanter of left femur, subsequent encounter for closed fracture with routine healing
        "S72115E",  # Nondisplaced fracture of greater trochanter of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72115F",  # Nondisplaced fracture of greater trochanter of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72115G",  # Nondisplaced fracture of greater trochanter of left femur, subsequent encounter for closed fracture with delayed healing
        "S72115H",  # Nondisplaced fracture of greater trochanter of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72115J",  # Nondisplaced fracture of greater trochanter of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72115K",  # Nondisplaced fracture of greater trochanter of left femur, subsequent encounter for closed fracture with nonunion
        "S72115M",  # Nondisplaced fracture of greater trochanter of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72115N",  # Nondisplaced fracture of greater trochanter of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72115P",  # Nondisplaced fracture of greater trochanter of left femur, subsequent encounter for closed fracture with malunion
        "S72115Q",  # Nondisplaced fracture of greater trochanter of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72115R",  # Nondisplaced fracture of greater trochanter of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72115S",  # Nondisplaced fracture of greater trochanter of left femur, sequela
        "S72116A",  # Nondisplaced fracture of greater trochanter of unspecified femur, initial encounter for closed fracture
        "S72116B",  # Nondisplaced fracture of greater trochanter of unspecified femur, initial encounter for open fracture type I or II
        "S72116C",  # Nondisplaced fracture of greater trochanter of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72116D",  # Nondisplaced fracture of greater trochanter of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72116E",  # Nondisplaced fracture of greater trochanter of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72116F",  # Nondisplaced fracture of greater trochanter of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72116G",  # Nondisplaced fracture of greater trochanter of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72116H",  # Nondisplaced fracture of greater trochanter of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72116J",  # Nondisplaced fracture of greater trochanter of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72116K",  # Nondisplaced fracture of greater trochanter of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72116M",  # Nondisplaced fracture of greater trochanter of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72116N",  # Nondisplaced fracture of greater trochanter of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72116P",  # Nondisplaced fracture of greater trochanter of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72116Q",  # Nondisplaced fracture of greater trochanter of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72116R",  # Nondisplaced fracture of greater trochanter of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72116S",  # Nondisplaced fracture of greater trochanter of unspecified femur, sequela
        "S72121A",  # Displaced fracture of lesser trochanter of right femur, initial encounter for closed fracture
        "S72121B",  # Displaced fracture of lesser trochanter of right femur, initial encounter for open fracture type I or II
        "S72121C",  # Displaced fracture of lesser trochanter of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72121D",  # Displaced fracture of lesser trochanter of right femur, subsequent encounter for closed fracture with routine healing
        "S72121E",  # Displaced fracture of lesser trochanter of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72121F",  # Displaced fracture of lesser trochanter of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72121G",  # Displaced fracture of lesser trochanter of right femur, subsequent encounter for closed fracture with delayed healing
        "S72121H",  # Displaced fracture of lesser trochanter of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72121J",  # Displaced fracture of lesser trochanter of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72121K",  # Displaced fracture of lesser trochanter of right femur, subsequent encounter for closed fracture with nonunion
        "S72121M",  # Displaced fracture of lesser trochanter of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72121N",  # Displaced fracture of lesser trochanter of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72121P",  # Displaced fracture of lesser trochanter of right femur, subsequent encounter for closed fracture with malunion
        "S72121Q",  # Displaced fracture of lesser trochanter of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72121R",  # Displaced fracture of lesser trochanter of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72121S",  # Displaced fracture of lesser trochanter of right femur, sequela
        "S72122A",  # Displaced fracture of lesser trochanter of left femur, initial encounter for closed fracture
        "S72122B",  # Displaced fracture of lesser trochanter of left femur, initial encounter for open fracture type I or II
        "S72122C",  # Displaced fracture of lesser trochanter of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72122D",  # Displaced fracture of lesser trochanter of left femur, subsequent encounter for closed fracture with routine healing
        "S72122E",  # Displaced fracture of lesser trochanter of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72122F",  # Displaced fracture of lesser trochanter of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72122G",  # Displaced fracture of lesser trochanter of left femur, subsequent encounter for closed fracture with delayed healing
        "S72122H",  # Displaced fracture of lesser trochanter of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72122J",  # Displaced fracture of lesser trochanter of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72122K",  # Displaced fracture of lesser trochanter of left femur, subsequent encounter for closed fracture with nonunion
        "S72122M",  # Displaced fracture of lesser trochanter of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72122N",  # Displaced fracture of lesser trochanter of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72122P",  # Displaced fracture of lesser trochanter of left femur, subsequent encounter for closed fracture with malunion
        "S72122Q",  # Displaced fracture of lesser trochanter of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72122R",  # Displaced fracture of lesser trochanter of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72122S",  # Displaced fracture of lesser trochanter of left femur, sequela
        "S72123A",  # Displaced fracture of lesser trochanter of unspecified femur, initial encounter for closed fracture
        "S72123B",  # Displaced fracture of lesser trochanter of unspecified femur, initial encounter for open fracture type I or II
        "S72123C",  # Displaced fracture of lesser trochanter of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72123D",  # Displaced fracture of lesser trochanter of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72123E",  # Displaced fracture of lesser trochanter of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72123F",  # Displaced fracture of lesser trochanter of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72123G",  # Displaced fracture of lesser trochanter of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72123H",  # Displaced fracture of lesser trochanter of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72123J",  # Displaced fracture of lesser trochanter of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72123K",  # Displaced fracture of lesser trochanter of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72123M",  # Displaced fracture of lesser trochanter of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72123N",  # Displaced fracture of lesser trochanter of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72123P",  # Displaced fracture of lesser trochanter of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72123Q",  # Displaced fracture of lesser trochanter of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72123R",  # Displaced fracture of lesser trochanter of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72123S",  # Displaced fracture of lesser trochanter of unspecified femur, sequela
        "S72124A",  # Nondisplaced fracture of lesser trochanter of right femur, initial encounter for closed fracture
        "S72124B",  # Nondisplaced fracture of lesser trochanter of right femur, initial encounter for open fracture type I or II
        "S72124C",  # Nondisplaced fracture of lesser trochanter of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72124D",  # Nondisplaced fracture of lesser trochanter of right femur, subsequent encounter for closed fracture with routine healing
        "S72124E",  # Nondisplaced fracture of lesser trochanter of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72124F",  # Nondisplaced fracture of lesser trochanter of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72124G",  # Nondisplaced fracture of lesser trochanter of right femur, subsequent encounter for closed fracture with delayed healing
        "S72124H",  # Nondisplaced fracture of lesser trochanter of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72124J",  # Nondisplaced fracture of lesser trochanter of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72124K",  # Nondisplaced fracture of lesser trochanter of right femur, subsequent encounter for closed fracture with nonunion
        "S72124M",  # Nondisplaced fracture of lesser trochanter of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72124N",  # Nondisplaced fracture of lesser trochanter of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72124P",  # Nondisplaced fracture of lesser trochanter of right femur, subsequent encounter for closed fracture with malunion
        "S72124Q",  # Nondisplaced fracture of lesser trochanter of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72124R",  # Nondisplaced fracture of lesser trochanter of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72124S",  # Nondisplaced fracture of lesser trochanter of right femur, sequela
        "S72125A",  # Nondisplaced fracture of lesser trochanter of left femur, initial encounter for closed fracture
        "S72125B",  # Nondisplaced fracture of lesser trochanter of left femur, initial encounter for open fracture type I or II
        "S72125C",  # Nondisplaced fracture of lesser trochanter of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72125D",  # Nondisplaced fracture of lesser trochanter of left femur, subsequent encounter for closed fracture with routine healing
        "S72125E",  # Nondisplaced fracture of lesser trochanter of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72125F",  # Nondisplaced fracture of lesser trochanter of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72125G",  # Nondisplaced fracture of lesser trochanter of left femur, subsequent encounter for closed fracture with delayed healing
        "S72125H",  # Nondisplaced fracture of lesser trochanter of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72125J",  # Nondisplaced fracture of lesser trochanter of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72125K",  # Nondisplaced fracture of lesser trochanter of left femur, subsequent encounter for closed fracture with nonunion
        "S72125M",  # Nondisplaced fracture of lesser trochanter of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72125N",  # Nondisplaced fracture of lesser trochanter of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72125P",  # Nondisplaced fracture of lesser trochanter of left femur, subsequent encounter for closed fracture with malunion
        "S72125Q",  # Nondisplaced fracture of lesser trochanter of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72125R",  # Nondisplaced fracture of lesser trochanter of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72125S",  # Nondisplaced fracture of lesser trochanter of left femur, sequela
        "S72126A",  # Nondisplaced fracture of lesser trochanter of unspecified femur, initial encounter for closed fracture
        "S72126B",  # Nondisplaced fracture of lesser trochanter of unspecified femur, initial encounter for open fracture type I or II
        "S72126C",  # Nondisplaced fracture of lesser trochanter of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72126D",  # Nondisplaced fracture of lesser trochanter of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72126E",  # Nondisplaced fracture of lesser trochanter of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72126F",  # Nondisplaced fracture of lesser trochanter of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72126G",  # Nondisplaced fracture of lesser trochanter of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72126H",  # Nondisplaced fracture of lesser trochanter of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72126J",  # Nondisplaced fracture of lesser trochanter of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72126K",  # Nondisplaced fracture of lesser trochanter of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72126M",  # Nondisplaced fracture of lesser trochanter of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72126N",  # Nondisplaced fracture of lesser trochanter of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72126P",  # Nondisplaced fracture of lesser trochanter of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72126Q",  # Nondisplaced fracture of lesser trochanter of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72126R",  # Nondisplaced fracture of lesser trochanter of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72126S",  # Nondisplaced fracture of lesser trochanter of unspecified femur, sequela
        "S72131A",  # Displaced apophyseal fracture of right femur, initial encounter for closed fracture
        "S72131B",  # Displaced apophyseal fracture of right femur, initial encounter for open fracture type I or II
        "S72131C",  # Displaced apophyseal fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72131D",  # Displaced apophyseal fracture of right femur, subsequent encounter for closed fracture with routine healing
        "S72131E",  # Displaced apophyseal fracture of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72131F",  # Displaced apophyseal fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72131G",  # Displaced apophyseal fracture of right femur, subsequent encounter for closed fracture with delayed healing
        "S72131H",  # Displaced apophyseal fracture of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72131J",  # Displaced apophyseal fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72131K",  # Displaced apophyseal fracture of right femur, subsequent encounter for closed fracture with nonunion
        "S72131M",  # Displaced apophyseal fracture of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72131N",  # Displaced apophyseal fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72131P",  # Displaced apophyseal fracture of right femur, subsequent encounter for closed fracture with malunion
        "S72131Q",  # Displaced apophyseal fracture of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72131R",  # Displaced apophyseal fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72131S",  # Displaced apophyseal fracture of right femur, sequela
        "S72132A",  # Displaced apophyseal fracture of left femur, initial encounter for closed fracture
        "S72132B",  # Displaced apophyseal fracture of left femur, initial encounter for open fracture type I or II
        "S72132C",  # Displaced apophyseal fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72132D",  # Displaced apophyseal fracture of left femur, subsequent encounter for closed fracture with routine healing
        "S72132E",  # Displaced apophyseal fracture of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72132F",  # Displaced apophyseal fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72132G",  # Displaced apophyseal fracture of left femur, subsequent encounter for closed fracture with delayed healing
        "S72132H",  # Displaced apophyseal fracture of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72132J",  # Displaced apophyseal fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72132K",  # Displaced apophyseal fracture of left femur, subsequent encounter for closed fracture with nonunion
        "S72132M",  # Displaced apophyseal fracture of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72132N",  # Displaced apophyseal fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72132P",  # Displaced apophyseal fracture of left femur, subsequent encounter for closed fracture with malunion
        "S72132Q",  # Displaced apophyseal fracture of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72132R",  # Displaced apophyseal fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72132S",  # Displaced apophyseal fracture of left femur, sequela
        "S72133A",  # Displaced apophyseal fracture of unspecified femur, initial encounter for closed fracture
        "S72133B",  # Displaced apophyseal fracture of unspecified femur, initial encounter for open fracture type I or II
        "S72133C",  # Displaced apophyseal fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72133D",  # Displaced apophyseal fracture of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72133E",  # Displaced apophyseal fracture of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72133F",  # Displaced apophyseal fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72133G",  # Displaced apophyseal fracture of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72133H",  # Displaced apophyseal fracture of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72133J",  # Displaced apophyseal fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72133K",  # Displaced apophyseal fracture of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72133M",  # Displaced apophyseal fracture of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72133N",  # Displaced apophyseal fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72133P",  # Displaced apophyseal fracture of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72133Q",  # Displaced apophyseal fracture of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72133R",  # Displaced apophyseal fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72133S",  # Displaced apophyseal fracture of unspecified femur, sequela
        "S72134A",  # Nondisplaced apophyseal fracture of right femur, initial encounter for closed fracture
        "S72134B",  # Nondisplaced apophyseal fracture of right femur, initial encounter for open fracture type I or II
        "S72134C",  # Nondisplaced apophyseal fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72134D",  # Nondisplaced apophyseal fracture of right femur, subsequent encounter for closed fracture with routine healing
        "S72134E",  # Nondisplaced apophyseal fracture of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72134F",  # Nondisplaced apophyseal fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72134G",  # Nondisplaced apophyseal fracture of right femur, subsequent encounter for closed fracture with delayed healing
        "S72134H",  # Nondisplaced apophyseal fracture of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72134J",  # Nondisplaced apophyseal fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72134K",  # Nondisplaced apophyseal fracture of right femur, subsequent encounter for closed fracture with nonunion
        "S72134M",  # Nondisplaced apophyseal fracture of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72134N",  # Nondisplaced apophyseal fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72134P",  # Nondisplaced apophyseal fracture of right femur, subsequent encounter for closed fracture with malunion
        "S72134Q",  # Nondisplaced apophyseal fracture of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72134R",  # Nondisplaced apophyseal fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72134S",  # Nondisplaced apophyseal fracture of right femur, sequela
        "S72135A",  # Nondisplaced apophyseal fracture of left femur, initial encounter for closed fracture
        "S72135B",  # Nondisplaced apophyseal fracture of left femur, initial encounter for open fracture type I or II
        "S72135C",  # Nondisplaced apophyseal fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72135D",  # Nondisplaced apophyseal fracture of left femur, subsequent encounter for closed fracture with routine healing
        "S72135E",  # Nondisplaced apophyseal fracture of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72135F",  # Nondisplaced apophyseal fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72135G",  # Nondisplaced apophyseal fracture of left femur, subsequent encounter for closed fracture with delayed healing
        "S72135H",  # Nondisplaced apophyseal fracture of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72135J",  # Nondisplaced apophyseal fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72135K",  # Nondisplaced apophyseal fracture of left femur, subsequent encounter for closed fracture with nonunion
        "S72135M",  # Nondisplaced apophyseal fracture of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72135N",  # Nondisplaced apophyseal fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72135P",  # Nondisplaced apophyseal fracture of left femur, subsequent encounter for closed fracture with malunion
        "S72135Q",  # Nondisplaced apophyseal fracture of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72135R",  # Nondisplaced apophyseal fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72135S",  # Nondisplaced apophyseal fracture of left femur, sequela
        "S72136A",  # Nondisplaced apophyseal fracture of unspecified femur, initial encounter for closed fracture
        "S72136B",  # Nondisplaced apophyseal fracture of unspecified femur, initial encounter for open fracture type I or II
        "S72136C",  # Nondisplaced apophyseal fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72136D",  # Nondisplaced apophyseal fracture of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72136E",  # Nondisplaced apophyseal fracture of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72136F",  # Nondisplaced apophyseal fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72136G",  # Nondisplaced apophyseal fracture of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72136H",  # Nondisplaced apophyseal fracture of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72136J",  # Nondisplaced apophyseal fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72136K",  # Nondisplaced apophyseal fracture of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72136M",  # Nondisplaced apophyseal fracture of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72136N",  # Nondisplaced apophyseal fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72136P",  # Nondisplaced apophyseal fracture of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72136Q",  # Nondisplaced apophyseal fracture of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72136R",  # Nondisplaced apophyseal fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72136S",  # Nondisplaced apophyseal fracture of unspecified femur, sequela
        "S72141A",  # Displaced intertrochanteric fracture of right femur, initial encounter for closed fracture
        "S72141B",  # Displaced intertrochanteric fracture of right femur, initial encounter for open fracture type I or II
        "S72141C",  # Displaced intertrochanteric fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72141D",  # Displaced intertrochanteric fracture of right femur, subsequent encounter for closed fracture with routine healing
        "S72141E",  # Displaced intertrochanteric fracture of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72141F",  # Displaced intertrochanteric fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72141G",  # Displaced intertrochanteric fracture of right femur, subsequent encounter for closed fracture with delayed healing
        "S72141H",  # Displaced intertrochanteric fracture of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72141J",  # Displaced intertrochanteric fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72141K",  # Displaced intertrochanteric fracture of right femur, subsequent encounter for closed fracture with nonunion
        "S72141M",  # Displaced intertrochanteric fracture of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72141N",  # Displaced intertrochanteric fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72141P",  # Displaced intertrochanteric fracture of right femur, subsequent encounter for closed fracture with malunion
        "S72141Q",  # Displaced intertrochanteric fracture of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72141R",  # Displaced intertrochanteric fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72141S",  # Displaced intertrochanteric fracture of right femur, sequela
        "S72142A",  # Displaced intertrochanteric fracture of left femur, initial encounter for closed fracture
        "S72142B",  # Displaced intertrochanteric fracture of left femur, initial encounter for open fracture type I or II
        "S72142C",  # Displaced intertrochanteric fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72142D",  # Displaced intertrochanteric fracture of left femur, subsequent encounter for closed fracture with routine healing
        "S72142E",  # Displaced intertrochanteric fracture of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72142F",  # Displaced intertrochanteric fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72142G",  # Displaced intertrochanteric fracture of left femur, subsequent encounter for closed fracture with delayed healing
        "S72142H",  # Displaced intertrochanteric fracture of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72142J",  # Displaced intertrochanteric fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72142K",  # Displaced intertrochanteric fracture of left femur, subsequent encounter for closed fracture with nonunion
        "S72142M",  # Displaced intertrochanteric fracture of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72142N",  # Displaced intertrochanteric fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72142P",  # Displaced intertrochanteric fracture of left femur, subsequent encounter for closed fracture with malunion
        "S72142Q",  # Displaced intertrochanteric fracture of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72142R",  # Displaced intertrochanteric fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72142S",  # Displaced intertrochanteric fracture of left femur, sequela
        "S72143A",  # Displaced intertrochanteric fracture of unspecified femur, initial encounter for closed fracture
        "S72143B",  # Displaced intertrochanteric fracture of unspecified femur, initial encounter for open fracture type I or II
        "S72143C",  # Displaced intertrochanteric fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72143D",  # Displaced intertrochanteric fracture of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72143E",  # Displaced intertrochanteric fracture of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72143F",  # Displaced intertrochanteric fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72143G",  # Displaced intertrochanteric fracture of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72143H",  # Displaced intertrochanteric fracture of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72143J",  # Displaced intertrochanteric fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72143K",  # Displaced intertrochanteric fracture of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72143M",  # Displaced intertrochanteric fracture of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72143N",  # Displaced intertrochanteric fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72143P",  # Displaced intertrochanteric fracture of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72143Q",  # Displaced intertrochanteric fracture of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72143R",  # Displaced intertrochanteric fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72143S",  # Displaced intertrochanteric fracture of unspecified femur, sequela
        "S72144A",  # Nondisplaced intertrochanteric fracture of right femur, initial encounter for closed fracture
        "S72144B",  # Nondisplaced intertrochanteric fracture of right femur, initial encounter for open fracture type I or II
        "S72144C",  # Nondisplaced intertrochanteric fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72144D",  # Nondisplaced intertrochanteric fracture of right femur, subsequent encounter for closed fracture with routine healing
        "S72144E",  # Nondisplaced intertrochanteric fracture of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72144F",  # Nondisplaced intertrochanteric fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72144G",  # Nondisplaced intertrochanteric fracture of right femur, subsequent encounter for closed fracture with delayed healing
        "S72144H",  # Nondisplaced intertrochanteric fracture of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72144J",  # Nondisplaced intertrochanteric fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72144K",  # Nondisplaced intertrochanteric fracture of right femur, subsequent encounter for closed fracture with nonunion
        "S72144M",  # Nondisplaced intertrochanteric fracture of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72144N",  # Nondisplaced intertrochanteric fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72144P",  # Nondisplaced intertrochanteric fracture of right femur, subsequent encounter for closed fracture with malunion
        "S72144Q",  # Nondisplaced intertrochanteric fracture of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72144R",  # Nondisplaced intertrochanteric fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72144S",  # Nondisplaced intertrochanteric fracture of right femur, sequela
        "S72145A",  # Nondisplaced intertrochanteric fracture of left femur, initial encounter for closed fracture
        "S72145B",  # Nondisplaced intertrochanteric fracture of left femur, initial encounter for open fracture type I or II
        "S72145C",  # Nondisplaced intertrochanteric fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72145D",  # Nondisplaced intertrochanteric fracture of left femur, subsequent encounter for closed fracture with routine healing
        "S72145E",  # Nondisplaced intertrochanteric fracture of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72145F",  # Nondisplaced intertrochanteric fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72145G",  # Nondisplaced intertrochanteric fracture of left femur, subsequent encounter for closed fracture with delayed healing
        "S72145H",  # Nondisplaced intertrochanteric fracture of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72145J",  # Nondisplaced intertrochanteric fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72145K",  # Nondisplaced intertrochanteric fracture of left femur, subsequent encounter for closed fracture with nonunion
        "S72145M",  # Nondisplaced intertrochanteric fracture of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72145N",  # Nondisplaced intertrochanteric fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72145P",  # Nondisplaced intertrochanteric fracture of left femur, subsequent encounter for closed fracture with malunion
        "S72145Q",  # Nondisplaced intertrochanteric fracture of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72145R",  # Nondisplaced intertrochanteric fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72145S",  # Nondisplaced intertrochanteric fracture of left femur, sequela
        "S72146A",  # Nondisplaced intertrochanteric fracture of unspecified femur, initial encounter for closed fracture
        "S72146B",  # Nondisplaced intertrochanteric fracture of unspecified femur, initial encounter for open fracture type I or II
        "S72146C",  # Nondisplaced intertrochanteric fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72146D",  # Nondisplaced intertrochanteric fracture of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72146E",  # Nondisplaced intertrochanteric fracture of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72146F",  # Nondisplaced intertrochanteric fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72146G",  # Nondisplaced intertrochanteric fracture of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72146H",  # Nondisplaced intertrochanteric fracture of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72146J",  # Nondisplaced intertrochanteric fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72146K",  # Nondisplaced intertrochanteric fracture of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72146M",  # Nondisplaced intertrochanteric fracture of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72146N",  # Nondisplaced intertrochanteric fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72146P",  # Nondisplaced intertrochanteric fracture of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72146Q",  # Nondisplaced intertrochanteric fracture of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72146R",  # Nondisplaced intertrochanteric fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72146S",  # Nondisplaced intertrochanteric fracture of unspecified femur, sequela
        "S7221XA",  # Displaced subtrochanteric fracture of right femur, initial encounter for closed fracture
        "S7221XB",  # Displaced subtrochanteric fracture of right femur, initial encounter for open fracture type I or II
        "S7221XC",  # Displaced subtrochanteric fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S7221XD",  # Displaced subtrochanteric fracture of right femur, subsequent encounter for closed fracture with routine healing
        "S7221XE",  # Displaced subtrochanteric fracture of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S7221XF",  # Displaced subtrochanteric fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S7221XG",  # Displaced subtrochanteric fracture of right femur, subsequent encounter for closed fracture with delayed healing
        "S7221XH",  # Displaced subtrochanteric fracture of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S7221XJ",  # Displaced subtrochanteric fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S7221XK",  # Displaced subtrochanteric fracture of right femur, subsequent encounter for closed fracture with nonunion
        "S7221XM",  # Displaced subtrochanteric fracture of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S7221XN",  # Displaced subtrochanteric fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S7221XP",  # Displaced subtrochanteric fracture of right femur, subsequent encounter for closed fracture with malunion
        "S7221XQ",  # Displaced subtrochanteric fracture of right femur, subsequent encounter for open fracture type I or II with malunion
        "S7221XR",  # Displaced subtrochanteric fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S7221XS",  # Displaced subtrochanteric fracture of right femur, sequela
        "S7222XA",  # Displaced subtrochanteric fracture of left femur, initial encounter for closed fracture
        "S7222XB",  # Displaced subtrochanteric fracture of left femur, initial encounter for open fracture type I or II
        "S7222XC",  # Displaced subtrochanteric fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S7222XD",  # Displaced subtrochanteric fracture of left femur, subsequent encounter for closed fracture with routine healing
        "S7222XE",  # Displaced subtrochanteric fracture of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S7222XF",  # Displaced subtrochanteric fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S7222XG",  # Displaced subtrochanteric fracture of left femur, subsequent encounter for closed fracture with delayed healing
        "S7222XH",  # Displaced subtrochanteric fracture of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S7222XJ",  # Displaced subtrochanteric fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S7222XK",  # Displaced subtrochanteric fracture of left femur, subsequent encounter for closed fracture with nonunion
        "S7222XM",  # Displaced subtrochanteric fracture of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S7222XN",  # Displaced subtrochanteric fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S7222XP",  # Displaced subtrochanteric fracture of left femur, subsequent encounter for closed fracture with malunion
        "S7222XQ",  # Displaced subtrochanteric fracture of left femur, subsequent encounter for open fracture type I or II with malunion
        "S7222XR",  # Displaced subtrochanteric fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S7222XS",  # Displaced subtrochanteric fracture of left femur, sequela
        "S7223XA",  # Displaced subtrochanteric fracture of unspecified femur, initial encounter for closed fracture
        "S7223XB",  # Displaced subtrochanteric fracture of unspecified femur, initial encounter for open fracture type I or II
        "S7223XC",  # Displaced subtrochanteric fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S7223XD",  # Displaced subtrochanteric fracture of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S7223XE",  # Displaced subtrochanteric fracture of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S7223XF",  # Displaced subtrochanteric fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S7223XG",  # Displaced subtrochanteric fracture of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S7223XH",  # Displaced subtrochanteric fracture of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S7223XJ",  # Displaced subtrochanteric fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S7223XK",  # Displaced subtrochanteric fracture of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S7223XM",  # Displaced subtrochanteric fracture of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S7223XN",  # Displaced subtrochanteric fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S7223XP",  # Displaced subtrochanteric fracture of unspecified femur, subsequent encounter for closed fracture with malunion
        "S7223XQ",  # Displaced subtrochanteric fracture of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S7223XR",  # Displaced subtrochanteric fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S7223XS",  # Displaced subtrochanteric fracture of unspecified femur, sequela
        "S7224XA",  # Nondisplaced subtrochanteric fracture of right femur, initial encounter for closed fracture
        "S7224XB",  # Nondisplaced subtrochanteric fracture of right femur, initial encounter for open fracture type I or II
        "S7224XC",  # Nondisplaced subtrochanteric fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S7224XD",  # Nondisplaced subtrochanteric fracture of right femur, subsequent encounter for closed fracture with routine healing
        "S7224XE",  # Nondisplaced subtrochanteric fracture of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S7224XF",  # Nondisplaced subtrochanteric fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S7224XG",  # Nondisplaced subtrochanteric fracture of right femur, subsequent encounter for closed fracture with delayed healing
        "S7224XH",  # Nondisplaced subtrochanteric fracture of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S7224XJ",  # Nondisplaced subtrochanteric fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S7224XK",  # Nondisplaced subtrochanteric fracture of right femur, subsequent encounter for closed fracture with nonunion
        "S7224XM",  # Nondisplaced subtrochanteric fracture of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S7224XN",  # Nondisplaced subtrochanteric fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S7224XP",  # Nondisplaced subtrochanteric fracture of right femur, subsequent encounter for closed fracture with malunion
        "S7224XQ",  # Nondisplaced subtrochanteric fracture of right femur, subsequent encounter for open fracture type I or II with malunion
        "S7224XR",  # Nondisplaced subtrochanteric fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S7224XS",  # Nondisplaced subtrochanteric fracture of right femur, sequela
        "S7225XA",  # Nondisplaced subtrochanteric fracture of left femur, initial encounter for closed fracture
        "S7225XB",  # Nondisplaced subtrochanteric fracture of left femur, initial encounter for open fracture type I or II
        "S7225XC",  # Nondisplaced subtrochanteric fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S7225XD",  # Nondisplaced subtrochanteric fracture of left femur, subsequent encounter for closed fracture with routine healing
        "S7225XE",  # Nondisplaced subtrochanteric fracture of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S7225XF",  # Nondisplaced subtrochanteric fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S7225XG",  # Nondisplaced subtrochanteric fracture of left femur, subsequent encounter for closed fracture with delayed healing
        "S7225XH",  # Nondisplaced subtrochanteric fracture of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S7225XJ",  # Nondisplaced subtrochanteric fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S7225XK",  # Nondisplaced subtrochanteric fracture of left femur, subsequent encounter for closed fracture with nonunion
        "S7225XM",  # Nondisplaced subtrochanteric fracture of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S7225XN",  # Nondisplaced subtrochanteric fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S7225XP",  # Nondisplaced subtrochanteric fracture of left femur, subsequent encounter for closed fracture with malunion
        "S7225XQ",  # Nondisplaced subtrochanteric fracture of left femur, subsequent encounter for open fracture type I or II with malunion
        "S7225XR",  # Nondisplaced subtrochanteric fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S7225XS",  # Nondisplaced subtrochanteric fracture of left femur, sequela
        "S7226XA",  # Nondisplaced subtrochanteric fracture of unspecified femur, initial encounter for closed fracture
        "S7226XB",  # Nondisplaced subtrochanteric fracture of unspecified femur, initial encounter for open fracture type I or II
        "S7226XC",  # Nondisplaced subtrochanteric fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S7226XD",  # Nondisplaced subtrochanteric fracture of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S7226XE",  # Nondisplaced subtrochanteric fracture of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S7226XF",  # Nondisplaced subtrochanteric fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S7226XG",  # Nondisplaced subtrochanteric fracture of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S7226XH",  # Nondisplaced subtrochanteric fracture of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S7226XJ",  # Nondisplaced subtrochanteric fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S7226XK",  # Nondisplaced subtrochanteric fracture of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S7226XM",  # Nondisplaced subtrochanteric fracture of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S7226XN",  # Nondisplaced subtrochanteric fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S7226XP",  # Nondisplaced subtrochanteric fracture of unspecified femur, subsequent encounter for closed fracture with malunion
        "S7226XQ",  # Nondisplaced subtrochanteric fracture of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S7226XR",  # Nondisplaced subtrochanteric fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S7226XS",  # Nondisplaced subtrochanteric fracture of unspecified femur, sequela
        "S72301A",  # Unspecified fracture of shaft of right femur, initial encounter for closed fracture
        "S72301B",  # Unspecified fracture of shaft of right femur, initial encounter for open fracture type I or II
        "S72301C",  # Unspecified fracture of shaft of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72301D",  # Unspecified fracture of shaft of right femur, subsequent encounter for closed fracture with routine healing
        "S72301E",  # Unspecified fracture of shaft of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72301F",  # Unspecified fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72301G",  # Unspecified fracture of shaft of right femur, subsequent encounter for closed fracture with delayed healing
        "S72301H",  # Unspecified fracture of shaft of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72301J",  # Unspecified fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72301K",  # Unspecified fracture of shaft of right femur, subsequent encounter for closed fracture with nonunion
        "S72301M",  # Unspecified fracture of shaft of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72301N",  # Unspecified fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72301P",  # Unspecified fracture of shaft of right femur, subsequent encounter for closed fracture with malunion
        "S72301Q",  # Unspecified fracture of shaft of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72301R",  # Unspecified fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72301S",  # Unspecified fracture of shaft of right femur, sequela
        "S72302A",  # Unspecified fracture of shaft of left femur, initial encounter for closed fracture
        "S72302B",  # Unspecified fracture of shaft of left femur, initial encounter for open fracture type I or II
        "S72302C",  # Unspecified fracture of shaft of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72302D",  # Unspecified fracture of shaft of left femur, subsequent encounter for closed fracture with routine healing
        "S72302E",  # Unspecified fracture of shaft of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72302F",  # Unspecified fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72302G",  # Unspecified fracture of shaft of left femur, subsequent encounter for closed fracture with delayed healing
        "S72302H",  # Unspecified fracture of shaft of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72302J",  # Unspecified fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72302K",  # Unspecified fracture of shaft of left femur, subsequent encounter for closed fracture with nonunion
        "S72302M",  # Unspecified fracture of shaft of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72302N",  # Unspecified fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72302P",  # Unspecified fracture of shaft of left femur, subsequent encounter for closed fracture with malunion
        "S72302Q",  # Unspecified fracture of shaft of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72302R",  # Unspecified fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72302S",  # Unspecified fracture of shaft of left femur, sequela
        "S72309A",  # Unspecified fracture of shaft of unspecified femur, initial encounter for closed fracture
        "S72309B",  # Unspecified fracture of shaft of unspecified femur, initial encounter for open fracture type I or II
        "S72309C",  # Unspecified fracture of shaft of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72309D",  # Unspecified fracture of shaft of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72309E",  # Unspecified fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72309F",  # Unspecified fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72309G",  # Unspecified fracture of shaft of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72309H",  # Unspecified fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72309J",  # Unspecified fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72309K",  # Unspecified fracture of shaft of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72309M",  # Unspecified fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72309N",  # Unspecified fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72309P",  # Unspecified fracture of shaft of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72309Q",  # Unspecified fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72309R",  # Unspecified fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72309S",  # Unspecified fracture of shaft of unspecified femur, sequela
        "S72321A",  # Displaced transverse fracture of shaft of right femur, initial encounter for closed fracture
        "S72321B",  # Displaced transverse fracture of shaft of right femur, initial encounter for open fracture type I or II
        "S72321C",  # Displaced transverse fracture of shaft of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72321D",  # Displaced transverse fracture of shaft of right femur, subsequent encounter for closed fracture with routine healing
        "S72321E",  # Displaced transverse fracture of shaft of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72321F",  # Displaced transverse fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72321G",  # Displaced transverse fracture of shaft of right femur, subsequent encounter for closed fracture with delayed healing
        "S72321H",  # Displaced transverse fracture of shaft of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72321J",  # Displaced transverse fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72321K",  # Displaced transverse fracture of shaft of right femur, subsequent encounter for closed fracture with nonunion
        "S72321M",  # Displaced transverse fracture of shaft of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72321N",  # Displaced transverse fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72321P",  # Displaced transverse fracture of shaft of right femur, subsequent encounter for closed fracture with malunion
        "S72321Q",  # Displaced transverse fracture of shaft of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72321R",  # Displaced transverse fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72321S",  # Displaced transverse fracture of shaft of right femur, sequela
        "S72322A",  # Displaced transverse fracture of shaft of left femur, initial encounter for closed fracture
        "S72322B",  # Displaced transverse fracture of shaft of left femur, initial encounter for open fracture type I or II
        "S72322C",  # Displaced transverse fracture of shaft of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72322D",  # Displaced transverse fracture of shaft of left femur, subsequent encounter for closed fracture with routine healing
        "S72322E",  # Displaced transverse fracture of shaft of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72322F",  # Displaced transverse fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72322G",  # Displaced transverse fracture of shaft of left femur, subsequent encounter for closed fracture with delayed healing
        "S72322H",  # Displaced transverse fracture of shaft of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72322J",  # Displaced transverse fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72322K",  # Displaced transverse fracture of shaft of left femur, subsequent encounter for closed fracture with nonunion
        "S72322M",  # Displaced transverse fracture of shaft of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72322N",  # Displaced transverse fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72322P",  # Displaced transverse fracture of shaft of left femur, subsequent encounter for closed fracture with malunion
        "S72322Q",  # Displaced transverse fracture of shaft of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72322R",  # Displaced transverse fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72322S",  # Displaced transverse fracture of shaft of left femur, sequela
        "S72323A",  # Displaced transverse fracture of shaft of unspecified femur, initial encounter for closed fracture
        "S72323B",  # Displaced transverse fracture of shaft of unspecified femur, initial encounter for open fracture type I or II
        "S72323C",  # Displaced transverse fracture of shaft of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72323D",  # Displaced transverse fracture of shaft of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72323E",  # Displaced transverse fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72323F",  # Displaced transverse fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72323G",  # Displaced transverse fracture of shaft of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72323H",  # Displaced transverse fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72323J",  # Displaced transverse fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72323K",  # Displaced transverse fracture of shaft of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72323M",  # Displaced transverse fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72323N",  # Displaced transverse fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72323P",  # Displaced transverse fracture of shaft of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72323Q",  # Displaced transverse fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72323R",  # Displaced transverse fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72323S",  # Displaced transverse fracture of shaft of unspecified femur, sequela
        "S72324A",  # Nondisplaced transverse fracture of shaft of right femur, initial encounter for closed fracture
        "S72324B",  # Nondisplaced transverse fracture of shaft of right femur, initial encounter for open fracture type I or II
        "S72324C",  # Nondisplaced transverse fracture of shaft of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72324D",  # Nondisplaced transverse fracture of shaft of right femur, subsequent encounter for closed fracture with routine healing
        "S72324E",  # Nondisplaced transverse fracture of shaft of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72324F",  # Nondisplaced transverse fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72324G",  # Nondisplaced transverse fracture of shaft of right femur, subsequent encounter for closed fracture with delayed healing
        "S72324H",  # Nondisplaced transverse fracture of shaft of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72324J",  # Nondisplaced transverse fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72324K",  # Nondisplaced transverse fracture of shaft of right femur, subsequent encounter for closed fracture with nonunion
        "S72324M",  # Nondisplaced transverse fracture of shaft of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72324N",  # Nondisplaced transverse fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72324P",  # Nondisplaced transverse fracture of shaft of right femur, subsequent encounter for closed fracture with malunion
        "S72324Q",  # Nondisplaced transverse fracture of shaft of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72324R",  # Nondisplaced transverse fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72324S",  # Nondisplaced transverse fracture of shaft of right femur, sequela
        "S72325A",  # Nondisplaced transverse fracture of shaft of left femur, initial encounter for closed fracture
        "S72325B",  # Nondisplaced transverse fracture of shaft of left femur, initial encounter for open fracture type I or II
        "S72325C",  # Nondisplaced transverse fracture of shaft of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72325D",  # Nondisplaced transverse fracture of shaft of left femur, subsequent encounter for closed fracture with routine healing
        "S72325E",  # Nondisplaced transverse fracture of shaft of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72325F",  # Nondisplaced transverse fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72325G",  # Nondisplaced transverse fracture of shaft of left femur, subsequent encounter for closed fracture with delayed healing
        "S72325H",  # Nondisplaced transverse fracture of shaft of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72325J",  # Nondisplaced transverse fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72325K",  # Nondisplaced transverse fracture of shaft of left femur, subsequent encounter for closed fracture with nonunion
        "S72325M",  # Nondisplaced transverse fracture of shaft of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72325N",  # Nondisplaced transverse fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72325P",  # Nondisplaced transverse fracture of shaft of left femur, subsequent encounter for closed fracture with malunion
        "S72325Q",  # Nondisplaced transverse fracture of shaft of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72325R",  # Nondisplaced transverse fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72325S",  # Nondisplaced transverse fracture of shaft of left femur, sequela
        "S72326A",  # Nondisplaced transverse fracture of shaft of unspecified femur, initial encounter for closed fracture
        "S72326B",  # Nondisplaced transverse fracture of shaft of unspecified femur, initial encounter for open fracture type I or II
        "S72326C",  # Nondisplaced transverse fracture of shaft of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72326D",  # Nondisplaced transverse fracture of shaft of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72326E",  # Nondisplaced transverse fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72326F",  # Nondisplaced transverse fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72326G",  # Nondisplaced transverse fracture of shaft of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72326H",  # Nondisplaced transverse fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72326J",  # Nondisplaced transverse fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72326K",  # Nondisplaced transverse fracture of shaft of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72326M",  # Nondisplaced transverse fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72326N",  # Nondisplaced transverse fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72326P",  # Nondisplaced transverse fracture of shaft of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72326Q",  # Nondisplaced transverse fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72326R",  # Nondisplaced transverse fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72326S",  # Nondisplaced transverse fracture of shaft of unspecified femur, sequela
        "S72331A",  # Displaced oblique fracture of shaft of right femur, initial encounter for closed fracture
        "S72331B",  # Displaced oblique fracture of shaft of right femur, initial encounter for open fracture type I or II
        "S72331C",  # Displaced oblique fracture of shaft of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72331D",  # Displaced oblique fracture of shaft of right femur, subsequent encounter for closed fracture with routine healing
        "S72331E",  # Displaced oblique fracture of shaft of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72331F",  # Displaced oblique fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72331G",  # Displaced oblique fracture of shaft of right femur, subsequent encounter for closed fracture with delayed healing
        "S72331H",  # Displaced oblique fracture of shaft of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72331J",  # Displaced oblique fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72331K",  # Displaced oblique fracture of shaft of right femur, subsequent encounter for closed fracture with nonunion
        "S72331M",  # Displaced oblique fracture of shaft of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72331N",  # Displaced oblique fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72331P",  # Displaced oblique fracture of shaft of right femur, subsequent encounter for closed fracture with malunion
        "S72331Q",  # Displaced oblique fracture of shaft of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72331R",  # Displaced oblique fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72331S",  # Displaced oblique fracture of shaft of right femur, sequela
        "S72332A",  # Displaced oblique fracture of shaft of left femur, initial encounter for closed fracture
        "S72332B",  # Displaced oblique fracture of shaft of left femur, initial encounter for open fracture type I or II
        "S72332C",  # Displaced oblique fracture of shaft of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72332D",  # Displaced oblique fracture of shaft of left femur, subsequent encounter for closed fracture with routine healing
        "S72332E",  # Displaced oblique fracture of shaft of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72332F",  # Displaced oblique fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72332G",  # Displaced oblique fracture of shaft of left femur, subsequent encounter for closed fracture with delayed healing
        "S72332H",  # Displaced oblique fracture of shaft of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72332J",  # Displaced oblique fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72332K",  # Displaced oblique fracture of shaft of left femur, subsequent encounter for closed fracture with nonunion
        "S72332M",  # Displaced oblique fracture of shaft of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72332N",  # Displaced oblique fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72332P",  # Displaced oblique fracture of shaft of left femur, subsequent encounter for closed fracture with malunion
        "S72332Q",  # Displaced oblique fracture of shaft of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72332R",  # Displaced oblique fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72332S",  # Displaced oblique fracture of shaft of left femur, sequela
        "S72333A",  # Displaced oblique fracture of shaft of unspecified femur, initial encounter for closed fracture
        "S72333B",  # Displaced oblique fracture of shaft of unspecified femur, initial encounter for open fracture type I or II
        "S72333C",  # Displaced oblique fracture of shaft of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72333D",  # Displaced oblique fracture of shaft of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72333E",  # Displaced oblique fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72333F",  # Displaced oblique fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72333G",  # Displaced oblique fracture of shaft of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72333H",  # Displaced oblique fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72333J",  # Displaced oblique fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72333K",  # Displaced oblique fracture of shaft of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72333M",  # Displaced oblique fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72333N",  # Displaced oblique fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72333P",  # Displaced oblique fracture of shaft of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72333Q",  # Displaced oblique fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72333R",  # Displaced oblique fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72333S",  # Displaced oblique fracture of shaft of unspecified femur, sequela
        "S72334A",  # Nondisplaced oblique fracture of shaft of right femur, initial encounter for closed fracture
        "S72334B",  # Nondisplaced oblique fracture of shaft of right femur, initial encounter for open fracture type I or II
        "S72334C",  # Nondisplaced oblique fracture of shaft of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72334D",  # Nondisplaced oblique fracture of shaft of right femur, subsequent encounter for closed fracture with routine healing
        "S72334E",  # Nondisplaced oblique fracture of shaft of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72334F",  # Nondisplaced oblique fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72334G",  # Nondisplaced oblique fracture of shaft of right femur, subsequent encounter for closed fracture with delayed healing
        "S72334H",  # Nondisplaced oblique fracture of shaft of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72334J",  # Nondisplaced oblique fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72334K",  # Nondisplaced oblique fracture of shaft of right femur, subsequent encounter for closed fracture with nonunion
        "S72334M",  # Nondisplaced oblique fracture of shaft of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72334N",  # Nondisplaced oblique fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72334P",  # Nondisplaced oblique fracture of shaft of right femur, subsequent encounter for closed fracture with malunion
        "S72334Q",  # Nondisplaced oblique fracture of shaft of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72334R",  # Nondisplaced oblique fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72334S",  # Nondisplaced oblique fracture of shaft of right femur, sequela
        "S72335A",  # Nondisplaced oblique fracture of shaft of left femur, initial encounter for closed fracture
        "S72335B",  # Nondisplaced oblique fracture of shaft of left femur, initial encounter for open fracture type I or II
        "S72335C",  # Nondisplaced oblique fracture of shaft of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72335D",  # Nondisplaced oblique fracture of shaft of left femur, subsequent encounter for closed fracture with routine healing
        "S72335E",  # Nondisplaced oblique fracture of shaft of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72335F",  # Nondisplaced oblique fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72335G",  # Nondisplaced oblique fracture of shaft of left femur, subsequent encounter for closed fracture with delayed healing
        "S72335H",  # Nondisplaced oblique fracture of shaft of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72335J",  # Nondisplaced oblique fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72335K",  # Nondisplaced oblique fracture of shaft of left femur, subsequent encounter for closed fracture with nonunion
        "S72335M",  # Nondisplaced oblique fracture of shaft of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72335N",  # Nondisplaced oblique fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72335P",  # Nondisplaced oblique fracture of shaft of left femur, subsequent encounter for closed fracture with malunion
        "S72335Q",  # Nondisplaced oblique fracture of shaft of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72335R",  # Nondisplaced oblique fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72335S",  # Nondisplaced oblique fracture of shaft of left femur, sequela
        "S72336A",  # Nondisplaced oblique fracture of shaft of unspecified femur, initial encounter for closed fracture
        "S72336B",  # Nondisplaced oblique fracture of shaft of unspecified femur, initial encounter for open fracture type I or II
        "S72336C",  # Nondisplaced oblique fracture of shaft of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72336D",  # Nondisplaced oblique fracture of shaft of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72336E",  # Nondisplaced oblique fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72336F",  # Nondisplaced oblique fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72336G",  # Nondisplaced oblique fracture of shaft of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72336H",  # Nondisplaced oblique fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72336J",  # Nondisplaced oblique fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72336K",  # Nondisplaced oblique fracture of shaft of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72336M",  # Nondisplaced oblique fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72336N",  # Nondisplaced oblique fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72336P",  # Nondisplaced oblique fracture of shaft of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72336Q",  # Nondisplaced oblique fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72336R",  # Nondisplaced oblique fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72336S",  # Nondisplaced oblique fracture of shaft of unspecified femur, sequela
        "S72341A",  # Displaced spiral fracture of shaft of right femur, initial encounter for closed fracture
        "S72341B",  # Displaced spiral fracture of shaft of right femur, initial encounter for open fracture type I or II
        "S72341C",  # Displaced spiral fracture of shaft of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72341D",  # Displaced spiral fracture of shaft of right femur, subsequent encounter for closed fracture with routine healing
        "S72341E",  # Displaced spiral fracture of shaft of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72341F",  # Displaced spiral fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72341G",  # Displaced spiral fracture of shaft of right femur, subsequent encounter for closed fracture with delayed healing
        "S72341H",  # Displaced spiral fracture of shaft of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72341J",  # Displaced spiral fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72341K",  # Displaced spiral fracture of shaft of right femur, subsequent encounter for closed fracture with nonunion
        "S72341M",  # Displaced spiral fracture of shaft of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72341N",  # Displaced spiral fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72341P",  # Displaced spiral fracture of shaft of right femur, subsequent encounter for closed fracture with malunion
        "S72341Q",  # Displaced spiral fracture of shaft of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72341R",  # Displaced spiral fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72341S",  # Displaced spiral fracture of shaft of right femur, sequela
        "S72342A",  # Displaced spiral fracture of shaft of left femur, initial encounter for closed fracture
        "S72342B",  # Displaced spiral fracture of shaft of left femur, initial encounter for open fracture type I or II
        "S72342C",  # Displaced spiral fracture of shaft of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72342D",  # Displaced spiral fracture of shaft of left femur, subsequent encounter for closed fracture with routine healing
        "S72342E",  # Displaced spiral fracture of shaft of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72342F",  # Displaced spiral fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72342G",  # Displaced spiral fracture of shaft of left femur, subsequent encounter for closed fracture with delayed healing
        "S72342H",  # Displaced spiral fracture of shaft of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72342J",  # Displaced spiral fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72342K",  # Displaced spiral fracture of shaft of left femur, subsequent encounter for closed fracture with nonunion
        "S72342M",  # Displaced spiral fracture of shaft of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72342N",  # Displaced spiral fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72342P",  # Displaced spiral fracture of shaft of left femur, subsequent encounter for closed fracture with malunion
        "S72342Q",  # Displaced spiral fracture of shaft of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72342R",  # Displaced spiral fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72342S",  # Displaced spiral fracture of shaft of left femur, sequela
        "S72343A",  # Displaced spiral fracture of shaft of unspecified femur, initial encounter for closed fracture
        "S72343B",  # Displaced spiral fracture of shaft of unspecified femur, initial encounter for open fracture type I or II
        "S72343C",  # Displaced spiral fracture of shaft of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72343D",  # Displaced spiral fracture of shaft of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72343E",  # Displaced spiral fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72343F",  # Displaced spiral fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72343G",  # Displaced spiral fracture of shaft of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72343H",  # Displaced spiral fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72343J",  # Displaced spiral fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72343K",  # Displaced spiral fracture of shaft of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72343M",  # Displaced spiral fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72343N",  # Displaced spiral fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72343P",  # Displaced spiral fracture of shaft of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72343Q",  # Displaced spiral fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72343R",  # Displaced spiral fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72343S",  # Displaced spiral fracture of shaft of unspecified femur, sequela
        "S72344A",  # Nondisplaced spiral fracture of shaft of right femur, initial encounter for closed fracture
        "S72344B",  # Nondisplaced spiral fracture of shaft of right femur, initial encounter for open fracture type I or II
        "S72344C",  # Nondisplaced spiral fracture of shaft of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72344D",  # Nondisplaced spiral fracture of shaft of right femur, subsequent encounter for closed fracture with routine healing
        "S72344E",  # Nondisplaced spiral fracture of shaft of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72344F",  # Nondisplaced spiral fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72344G",  # Nondisplaced spiral fracture of shaft of right femur, subsequent encounter for closed fracture with delayed healing
        "S72344H",  # Nondisplaced spiral fracture of shaft of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72344J",  # Nondisplaced spiral fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72344K",  # Nondisplaced spiral fracture of shaft of right femur, subsequent encounter for closed fracture with nonunion
        "S72344M",  # Nondisplaced spiral fracture of shaft of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72344N",  # Nondisplaced spiral fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72344P",  # Nondisplaced spiral fracture of shaft of right femur, subsequent encounter for closed fracture with malunion
        "S72344Q",  # Nondisplaced spiral fracture of shaft of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72344R",  # Nondisplaced spiral fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72344S",  # Nondisplaced spiral fracture of shaft of right femur, sequela
        "S72345A",  # Nondisplaced spiral fracture of shaft of left femur, initial encounter for closed fracture
        "S72345B",  # Nondisplaced spiral fracture of shaft of left femur, initial encounter for open fracture type I or II
        "S72345C",  # Nondisplaced spiral fracture of shaft of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72345D",  # Nondisplaced spiral fracture of shaft of left femur, subsequent encounter for closed fracture with routine healing
        "S72345E",  # Nondisplaced spiral fracture of shaft of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72345F",  # Nondisplaced spiral fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72345G",  # Nondisplaced spiral fracture of shaft of left femur, subsequent encounter for closed fracture with delayed healing
        "S72345H",  # Nondisplaced spiral fracture of shaft of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72345J",  # Nondisplaced spiral fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72345K",  # Nondisplaced spiral fracture of shaft of left femur, subsequent encounter for closed fracture with nonunion
        "S72345M",  # Nondisplaced spiral fracture of shaft of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72345N",  # Nondisplaced spiral fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72345P",  # Nondisplaced spiral fracture of shaft of left femur, subsequent encounter for closed fracture with malunion
        "S72345Q",  # Nondisplaced spiral fracture of shaft of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72345R",  # Nondisplaced spiral fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72345S",  # Nondisplaced spiral fracture of shaft of left femur, sequela
        "S72346A",  # Nondisplaced spiral fracture of shaft of unspecified femur, initial encounter for closed fracture
        "S72346B",  # Nondisplaced spiral fracture of shaft of unspecified femur, initial encounter for open fracture type I or II
        "S72346C",  # Nondisplaced spiral fracture of shaft of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72346D",  # Nondisplaced spiral fracture of shaft of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72346E",  # Nondisplaced spiral fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72346F",  # Nondisplaced spiral fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72346G",  # Nondisplaced spiral fracture of shaft of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72346H",  # Nondisplaced spiral fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72346J",  # Nondisplaced spiral fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72346K",  # Nondisplaced spiral fracture of shaft of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72346M",  # Nondisplaced spiral fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72346N",  # Nondisplaced spiral fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72346P",  # Nondisplaced spiral fracture of shaft of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72346Q",  # Nondisplaced spiral fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72346R",  # Nondisplaced spiral fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72346S",  # Nondisplaced spiral fracture of shaft of unspecified femur, sequela
        "S72351A",  # Displaced comminuted fracture of shaft of right femur, initial encounter for closed fracture
        "S72351B",  # Displaced comminuted fracture of shaft of right femur, initial encounter for open fracture type I or II
        "S72351C",  # Displaced comminuted fracture of shaft of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72351D",  # Displaced comminuted fracture of shaft of right femur, subsequent encounter for closed fracture with routine healing
        "S72351E",  # Displaced comminuted fracture of shaft of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72351F",  # Displaced comminuted fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72351G",  # Displaced comminuted fracture of shaft of right femur, subsequent encounter for closed fracture with delayed healing
        "S72351H",  # Displaced comminuted fracture of shaft of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72351J",  # Displaced comminuted fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72351K",  # Displaced comminuted fracture of shaft of right femur, subsequent encounter for closed fracture with nonunion
        "S72351M",  # Displaced comminuted fracture of shaft of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72351N",  # Displaced comminuted fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72351P",  # Displaced comminuted fracture of shaft of right femur, subsequent encounter for closed fracture with malunion
        "S72351Q",  # Displaced comminuted fracture of shaft of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72351R",  # Displaced comminuted fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72351S",  # Displaced comminuted fracture of shaft of right femur, sequela
        "S72352A",  # Displaced comminuted fracture of shaft of left femur, initial encounter for closed fracture
        "S72352B",  # Displaced comminuted fracture of shaft of left femur, initial encounter for open fracture type I or II
        "S72352C",  # Displaced comminuted fracture of shaft of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72352D",  # Displaced comminuted fracture of shaft of left femur, subsequent encounter for closed fracture with routine healing
        "S72352E",  # Displaced comminuted fracture of shaft of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72352F",  # Displaced comminuted fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72352G",  # Displaced comminuted fracture of shaft of left femur, subsequent encounter for closed fracture with delayed healing
        "S72352H",  # Displaced comminuted fracture of shaft of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72352J",  # Displaced comminuted fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72352K",  # Displaced comminuted fracture of shaft of left femur, subsequent encounter for closed fracture with nonunion
        "S72352M",  # Displaced comminuted fracture of shaft of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72352N",  # Displaced comminuted fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72352P",  # Displaced comminuted fracture of shaft of left femur, subsequent encounter for closed fracture with malunion
        "S72352Q",  # Displaced comminuted fracture of shaft of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72352R",  # Displaced comminuted fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72352S",  # Displaced comminuted fracture of shaft of left femur, sequela
        "S72353A",  # Displaced comminuted fracture of shaft of unspecified femur, initial encounter for closed fracture
        "S72353B",  # Displaced comminuted fracture of shaft of unspecified femur, initial encounter for open fracture type I or II
        "S72353C",  # Displaced comminuted fracture of shaft of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72353D",  # Displaced comminuted fracture of shaft of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72353E",  # Displaced comminuted fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72353F",  # Displaced comminuted fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72353G",  # Displaced comminuted fracture of shaft of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72353H",  # Displaced comminuted fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72353J",  # Displaced comminuted fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72353K",  # Displaced comminuted fracture of shaft of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72353M",  # Displaced comminuted fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72353N",  # Displaced comminuted fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72353P",  # Displaced comminuted fracture of shaft of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72353Q",  # Displaced comminuted fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72353R",  # Displaced comminuted fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72353S",  # Displaced comminuted fracture of shaft of unspecified femur, sequela
        "S72354A",  # Nondisplaced comminuted fracture of shaft of right femur, initial encounter for closed fracture
        "S72354B",  # Nondisplaced comminuted fracture of shaft of right femur, initial encounter for open fracture type I or II
        "S72354C",  # Nondisplaced comminuted fracture of shaft of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72354D",  # Nondisplaced comminuted fracture of shaft of right femur, subsequent encounter for closed fracture with routine healing
        "S72354E",  # Nondisplaced comminuted fracture of shaft of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72354F",  # Nondisplaced comminuted fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72354G",  # Nondisplaced comminuted fracture of shaft of right femur, subsequent encounter for closed fracture with delayed healing
        "S72354H",  # Nondisplaced comminuted fracture of shaft of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72354J",  # Nondisplaced comminuted fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72354K",  # Nondisplaced comminuted fracture of shaft of right femur, subsequent encounter for closed fracture with nonunion
        "S72354M",  # Nondisplaced comminuted fracture of shaft of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72354N",  # Nondisplaced comminuted fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72354P",  # Nondisplaced comminuted fracture of shaft of right femur, subsequent encounter for closed fracture with malunion
        "S72354Q",  # Nondisplaced comminuted fracture of shaft of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72354R",  # Nondisplaced comminuted fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72354S",  # Nondisplaced comminuted fracture of shaft of right femur, sequela
        "S72355A",  # Nondisplaced comminuted fracture of shaft of left femur, initial encounter for closed fracture
        "S72355B",  # Nondisplaced comminuted fracture of shaft of left femur, initial encounter for open fracture type I or II
        "S72355C",  # Nondisplaced comminuted fracture of shaft of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72355D",  # Nondisplaced comminuted fracture of shaft of left femur, subsequent encounter for closed fracture with routine healing
        "S72355E",  # Nondisplaced comminuted fracture of shaft of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72355F",  # Nondisplaced comminuted fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72355G",  # Nondisplaced comminuted fracture of shaft of left femur, subsequent encounter for closed fracture with delayed healing
        "S72355H",  # Nondisplaced comminuted fracture of shaft of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72355J",  # Nondisplaced comminuted fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72355K",  # Nondisplaced comminuted fracture of shaft of left femur, subsequent encounter for closed fracture with nonunion
        "S72355M",  # Nondisplaced comminuted fracture of shaft of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72355N",  # Nondisplaced comminuted fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72355P",  # Nondisplaced comminuted fracture of shaft of left femur, subsequent encounter for closed fracture with malunion
        "S72355Q",  # Nondisplaced comminuted fracture of shaft of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72355R",  # Nondisplaced comminuted fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72355S",  # Nondisplaced comminuted fracture of shaft of left femur, sequela
        "S72356A",  # Nondisplaced comminuted fracture of shaft of unspecified femur, initial encounter for closed fracture
        "S72356B",  # Nondisplaced comminuted fracture of shaft of unspecified femur, initial encounter for open fracture type I or II
        "S72356C",  # Nondisplaced comminuted fracture of shaft of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72356D",  # Nondisplaced comminuted fracture of shaft of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72356E",  # Nondisplaced comminuted fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72356F",  # Nondisplaced comminuted fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72356G",  # Nondisplaced comminuted fracture of shaft of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72356H",  # Nondisplaced comminuted fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72356J",  # Nondisplaced comminuted fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72356K",  # Nondisplaced comminuted fracture of shaft of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72356M",  # Nondisplaced comminuted fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72356N",  # Nondisplaced comminuted fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72356P",  # Nondisplaced comminuted fracture of shaft of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72356Q",  # Nondisplaced comminuted fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72356R",  # Nondisplaced comminuted fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72356S",  # Nondisplaced comminuted fracture of shaft of unspecified femur, sequela
        "S72361A",  # Displaced segmental fracture of shaft of right femur, initial encounter for closed fracture
        "S72361B",  # Displaced segmental fracture of shaft of right femur, initial encounter for open fracture type I or II
        "S72361C",  # Displaced segmental fracture of shaft of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72361D",  # Displaced segmental fracture of shaft of right femur, subsequent encounter for closed fracture with routine healing
        "S72361E",  # Displaced segmental fracture of shaft of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72361F",  # Displaced segmental fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72361G",  # Displaced segmental fracture of shaft of right femur, subsequent encounter for closed fracture with delayed healing
        "S72361H",  # Displaced segmental fracture of shaft of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72361J",  # Displaced segmental fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72361K",  # Displaced segmental fracture of shaft of right femur, subsequent encounter for closed fracture with nonunion
        "S72361M",  # Displaced segmental fracture of shaft of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72361N",  # Displaced segmental fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72361P",  # Displaced segmental fracture of shaft of right femur, subsequent encounter for closed fracture with malunion
        "S72361Q",  # Displaced segmental fracture of shaft of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72361R",  # Displaced segmental fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72361S",  # Displaced segmental fracture of shaft of right femur, sequela
        "S72362A",  # Displaced segmental fracture of shaft of left femur, initial encounter for closed fracture
        "S72362B",  # Displaced segmental fracture of shaft of left femur, initial encounter for open fracture type I or II
        "S72362C",  # Displaced segmental fracture of shaft of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72362D",  # Displaced segmental fracture of shaft of left femur, subsequent encounter for closed fracture with routine healing
        "S72362E",  # Displaced segmental fracture of shaft of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72362F",  # Displaced segmental fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72362G",  # Displaced segmental fracture of shaft of left femur, subsequent encounter for closed fracture with delayed healing
        "S72362H",  # Displaced segmental fracture of shaft of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72362J",  # Displaced segmental fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72362K",  # Displaced segmental fracture of shaft of left femur, subsequent encounter for closed fracture with nonunion
        "S72362M",  # Displaced segmental fracture of shaft of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72362N",  # Displaced segmental fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72362P",  # Displaced segmental fracture of shaft of left femur, subsequent encounter for closed fracture with malunion
        "S72362Q",  # Displaced segmental fracture of shaft of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72362R",  # Displaced segmental fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72362S",  # Displaced segmental fracture of shaft of left femur, sequela
        "S72363A",  # Displaced segmental fracture of shaft of unspecified femur, initial encounter for closed fracture
        "S72363B",  # Displaced segmental fracture of shaft of unspecified femur, initial encounter for open fracture type I or II
        "S72363C",  # Displaced segmental fracture of shaft of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72363D",  # Displaced segmental fracture of shaft of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72363E",  # Displaced segmental fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72363F",  # Displaced segmental fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72363G",  # Displaced segmental fracture of shaft of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72363H",  # Displaced segmental fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72363J",  # Displaced segmental fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72363K",  # Displaced segmental fracture of shaft of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72363M",  # Displaced segmental fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72363N",  # Displaced segmental fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72363P",  # Displaced segmental fracture of shaft of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72363Q",  # Displaced segmental fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72363R",  # Displaced segmental fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72363S",  # Displaced segmental fracture of shaft of unspecified femur, sequela
        "S72364A",  # Nondisplaced segmental fracture of shaft of right femur, initial encounter for closed fracture
        "S72364B",  # Nondisplaced segmental fracture of shaft of right femur, initial encounter for open fracture type I or II
        "S72364C",  # Nondisplaced segmental fracture of shaft of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72364D",  # Nondisplaced segmental fracture of shaft of right femur, subsequent encounter for closed fracture with routine healing
        "S72364E",  # Nondisplaced segmental fracture of shaft of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72364F",  # Nondisplaced segmental fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72364G",  # Nondisplaced segmental fracture of shaft of right femur, subsequent encounter for closed fracture with delayed healing
        "S72364H",  # Nondisplaced segmental fracture of shaft of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72364J",  # Nondisplaced segmental fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72364K",  # Nondisplaced segmental fracture of shaft of right femur, subsequent encounter for closed fracture with nonunion
        "S72364M",  # Nondisplaced segmental fracture of shaft of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72364N",  # Nondisplaced segmental fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72364P",  # Nondisplaced segmental fracture of shaft of right femur, subsequent encounter for closed fracture with malunion
        "S72364Q",  # Nondisplaced segmental fracture of shaft of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72364R",  # Nondisplaced segmental fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72364S",  # Nondisplaced segmental fracture of shaft of right femur, sequela
        "S72365A",  # Nondisplaced segmental fracture of shaft of left femur, initial encounter for closed fracture
        "S72365B",  # Nondisplaced segmental fracture of shaft of left femur, initial encounter for open fracture type I or II
        "S72365C",  # Nondisplaced segmental fracture of shaft of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72365D",  # Nondisplaced segmental fracture of shaft of left femur, subsequent encounter for closed fracture with routine healing
        "S72365E",  # Nondisplaced segmental fracture of shaft of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72365F",  # Nondisplaced segmental fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72365G",  # Nondisplaced segmental fracture of shaft of left femur, subsequent encounter for closed fracture with delayed healing
        "S72365H",  # Nondisplaced segmental fracture of shaft of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72365J",  # Nondisplaced segmental fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72365K",  # Nondisplaced segmental fracture of shaft of left femur, subsequent encounter for closed fracture with nonunion
        "S72365M",  # Nondisplaced segmental fracture of shaft of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72365N",  # Nondisplaced segmental fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72365P",  # Nondisplaced segmental fracture of shaft of left femur, subsequent encounter for closed fracture with malunion
        "S72365Q",  # Nondisplaced segmental fracture of shaft of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72365R",  # Nondisplaced segmental fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72365S",  # Nondisplaced segmental fracture of shaft of left femur, sequela
        "S72366A",  # Nondisplaced segmental fracture of shaft of unspecified femur, initial encounter for closed fracture
        "S72366B",  # Nondisplaced segmental fracture of shaft of unspecified femur, initial encounter for open fracture type I or II
        "S72366C",  # Nondisplaced segmental fracture of shaft of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72366D",  # Nondisplaced segmental fracture of shaft of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72366E",  # Nondisplaced segmental fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72366F",  # Nondisplaced segmental fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72366G",  # Nondisplaced segmental fracture of shaft of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72366H",  # Nondisplaced segmental fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72366J",  # Nondisplaced segmental fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72366K",  # Nondisplaced segmental fracture of shaft of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72366M",  # Nondisplaced segmental fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72366N",  # Nondisplaced segmental fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72366P",  # Nondisplaced segmental fracture of shaft of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72366Q",  # Nondisplaced segmental fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72366R",  # Nondisplaced segmental fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72366S",  # Nondisplaced segmental fracture of shaft of unspecified femur, sequela
        "S72391A",  # Other fracture of shaft of right femur, initial encounter for closed fracture
        "S72391B",  # Other fracture of shaft of right femur, initial encounter for open fracture type I or II
        "S72391C",  # Other fracture of shaft of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72391D",  # Other fracture of shaft of right femur, subsequent encounter for closed fracture with routine healing
        "S72391E",  # Other fracture of shaft of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72391F",  # Other fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72391G",  # Other fracture of shaft of right femur, subsequent encounter for closed fracture with delayed healing
        "S72391H",  # Other fracture of shaft of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72391J",  # Other fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72391K",  # Other fracture of shaft of right femur, subsequent encounter for closed fracture with nonunion
        "S72391M",  # Other fracture of shaft of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72391N",  # Other fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72391P",  # Other fracture of shaft of right femur, subsequent encounter for closed fracture with malunion
        "S72391Q",  # Other fracture of shaft of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72391R",  # Other fracture of shaft of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72391S",  # Other fracture of shaft of right femur, sequela
        "S72392A",  # Other fracture of shaft of left femur, initial encounter for closed fracture
        "S72392B",  # Other fracture of shaft of left femur, initial encounter for open fracture type I or II
        "S72392C",  # Other fracture of shaft of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72392D",  # Other fracture of shaft of left femur, subsequent encounter for closed fracture with routine healing
        "S72392E",  # Other fracture of shaft of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72392F",  # Other fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72392G",  # Other fracture of shaft of left femur, subsequent encounter for closed fracture with delayed healing
        "S72392H",  # Other fracture of shaft of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72392J",  # Other fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72392K",  # Other fracture of shaft of left femur, subsequent encounter for closed fracture with nonunion
        "S72392M",  # Other fracture of shaft of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72392N",  # Other fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72392P",  # Other fracture of shaft of left femur, subsequent encounter for closed fracture with malunion
        "S72392Q",  # Other fracture of shaft of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72392R",  # Other fracture of shaft of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72392S",  # Other fracture of shaft of left femur, sequela
        "S72399A",  # Other fracture of shaft of unspecified femur, initial encounter for closed fracture
        "S72399B",  # Other fracture of shaft of unspecified femur, initial encounter for open fracture type I or II
        "S72399C",  # Other fracture of shaft of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72399D",  # Other fracture of shaft of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72399E",  # Other fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72399F",  # Other fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72399G",  # Other fracture of shaft of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72399H",  # Other fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72399J",  # Other fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72399K",  # Other fracture of shaft of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72399M",  # Other fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72399N",  # Other fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72399P",  # Other fracture of shaft of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72399Q",  # Other fracture of shaft of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72399R",  # Other fracture of shaft of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72399S",  # Other fracture of shaft of unspecified femur, sequela
        "S72401A",  # Unspecified fracture of lower end of right femur, initial encounter for closed fracture
        "S72401B",  # Unspecified fracture of lower end of right femur, initial encounter for open fracture type I or II
        "S72401C",  # Unspecified fracture of lower end of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72401D",  # Unspecified fracture of lower end of right femur, subsequent encounter for closed fracture with routine healing
        "S72401E",  # Unspecified fracture of lower end of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72401F",  # Unspecified fracture of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72401G",  # Unspecified fracture of lower end of right femur, subsequent encounter for closed fracture with delayed healing
        "S72401H",  # Unspecified fracture of lower end of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72401J",  # Unspecified fracture of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72401K",  # Unspecified fracture of lower end of right femur, subsequent encounter for closed fracture with nonunion
        "S72401M",  # Unspecified fracture of lower end of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72401N",  # Unspecified fracture of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72401P",  # Unspecified fracture of lower end of right femur, subsequent encounter for closed fracture with malunion
        "S72401Q",  # Unspecified fracture of lower end of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72401R",  # Unspecified fracture of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72401S",  # Unspecified fracture of lower end of right femur, sequela
        "S72402A",  # Unspecified fracture of lower end of left femur, initial encounter for closed fracture
        "S72402B",  # Unspecified fracture of lower end of left femur, initial encounter for open fracture type I or II
        "S72402C",  # Unspecified fracture of lower end of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72402D",  # Unspecified fracture of lower end of left femur, subsequent encounter for closed fracture with routine healing
        "S72402E",  # Unspecified fracture of lower end of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72402F",  # Unspecified fracture of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72402G",  # Unspecified fracture of lower end of left femur, subsequent encounter for closed fracture with delayed healing
        "S72402H",  # Unspecified fracture of lower end of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72402J",  # Unspecified fracture of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72402K",  # Unspecified fracture of lower end of left femur, subsequent encounter for closed fracture with nonunion
        "S72402M",  # Unspecified fracture of lower end of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72402N",  # Unspecified fracture of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72402P",  # Unspecified fracture of lower end of left femur, subsequent encounter for closed fracture with malunion
        "S72402Q",  # Unspecified fracture of lower end of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72402R",  # Unspecified fracture of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72402S",  # Unspecified fracture of lower end of left femur, sequela
        "S72409A",  # Unspecified fracture of lower end of unspecified femur, initial encounter for closed fracture
        "S72409B",  # Unspecified fracture of lower end of unspecified femur, initial encounter for open fracture type I or II
        "S72409C",  # Unspecified fracture of lower end of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72409D",  # Unspecified fracture of lower end of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72409E",  # Unspecified fracture of lower end of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72409F",  # Unspecified fracture of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72409G",  # Unspecified fracture of lower end of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72409H",  # Unspecified fracture of lower end of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72409J",  # Unspecified fracture of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72409K",  # Unspecified fracture of lower end of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72409M",  # Unspecified fracture of lower end of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72409N",  # Unspecified fracture of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72409P",  # Unspecified fracture of lower end of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72409Q",  # Unspecified fracture of lower end of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72409R",  # Unspecified fracture of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72409S",  # Unspecified fracture of lower end of unspecified femur, sequela
        "S72411A",  # Displaced unspecified condyle fracture of lower end of right femur, initial encounter for closed fracture
        "S72411B",  # Displaced unspecified condyle fracture of lower end of right femur, initial encounter for open fracture type I or II
        "S72411C",  # Displaced unspecified condyle fracture of lower end of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72411D",  # Displaced unspecified condyle fracture of lower end of right femur, subsequent encounter for closed fracture with routine healing
        "S72411E",  # Displaced unspecified condyle fracture of lower end of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72411F",  # Displaced unspecified condyle fracture of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72411G",  # Displaced unspecified condyle fracture of lower end of right femur, subsequent encounter for closed fracture with delayed healing
        "S72411H",  # Displaced unspecified condyle fracture of lower end of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72411J",  # Displaced unspecified condyle fracture of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72411K",  # Displaced unspecified condyle fracture of lower end of right femur, subsequent encounter for closed fracture with nonunion
        "S72411M",  # Displaced unspecified condyle fracture of lower end of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72411N",  # Displaced unspecified condyle fracture of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72411P",  # Displaced unspecified condyle fracture of lower end of right femur, subsequent encounter for closed fracture with malunion
        "S72411Q",  # Displaced unspecified condyle fracture of lower end of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72411R",  # Displaced unspecified condyle fracture of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72411S",  # Displaced unspecified condyle fracture of lower end of right femur, sequela
        "S72412A",  # Displaced unspecified condyle fracture of lower end of left femur, initial encounter for closed fracture
        "S72412B",  # Displaced unspecified condyle fracture of lower end of left femur, initial encounter for open fracture type I or II
        "S72412C",  # Displaced unspecified condyle fracture of lower end of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72412D",  # Displaced unspecified condyle fracture of lower end of left femur, subsequent encounter for closed fracture with routine healing
        "S72412E",  # Displaced unspecified condyle fracture of lower end of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72412F",  # Displaced unspecified condyle fracture of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72412G",  # Displaced unspecified condyle fracture of lower end of left femur, subsequent encounter for closed fracture with delayed healing
        "S72412H",  # Displaced unspecified condyle fracture of lower end of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72412J",  # Displaced unspecified condyle fracture of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72412K",  # Displaced unspecified condyle fracture of lower end of left femur, subsequent encounter for closed fracture with nonunion
        "S72412M",  # Displaced unspecified condyle fracture of lower end of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72412N",  # Displaced unspecified condyle fracture of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72412P",  # Displaced unspecified condyle fracture of lower end of left femur, subsequent encounter for closed fracture with malunion
        "S72412Q",  # Displaced unspecified condyle fracture of lower end of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72412R",  # Displaced unspecified condyle fracture of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72412S",  # Displaced unspecified condyle fracture of lower end of left femur, sequela
        "S72413A",  # Displaced unspecified condyle fracture of lower end of unspecified femur, initial encounter for closed fracture
        "S72413B",  # Displaced unspecified condyle fracture of lower end of unspecified femur, initial encounter for open fracture type I or II
        "S72413C",  # Displaced unspecified condyle fracture of lower end of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72413D",  # Displaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72413E",  # Displaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72413F",  # Displaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72413G",  # Displaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72413H",  # Displaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72413J",  # Displaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72413K",  # Displaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72413M",  # Displaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72413N",  # Displaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72413P",  # Displaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72413Q",  # Displaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72413R",  # Displaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72413S",  # Displaced unspecified condyle fracture of lower end of unspecified femur, sequela
        "S72414A",  # Nondisplaced unspecified condyle fracture of lower end of right femur, initial encounter for closed fracture
        "S72414B",  # Nondisplaced unspecified condyle fracture of lower end of right femur, initial encounter for open fracture type I or II
        "S72414C",  # Nondisplaced unspecified condyle fracture of lower end of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72414D",  # Nondisplaced unspecified condyle fracture of lower end of right femur, subsequent encounter for closed fracture with routine healing
        "S72414E",  # Nondisplaced unspecified condyle fracture of lower end of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72414F",  # Nondisplaced unspecified condyle fracture of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72414G",  # Nondisplaced unspecified condyle fracture of lower end of right femur, subsequent encounter for closed fracture with delayed healing
        "S72414H",  # Nondisplaced unspecified condyle fracture of lower end of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72414J",  # Nondisplaced unspecified condyle fracture of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72414K",  # Nondisplaced unspecified condyle fracture of lower end of right femur, subsequent encounter for closed fracture with nonunion
        "S72414M",  # Nondisplaced unspecified condyle fracture of lower end of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72414N",  # Nondisplaced unspecified condyle fracture of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72414P",  # Nondisplaced unspecified condyle fracture of lower end of right femur, subsequent encounter for closed fracture with malunion
        "S72414Q",  # Nondisplaced unspecified condyle fracture of lower end of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72414R",  # Nondisplaced unspecified condyle fracture of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72414S",  # Nondisplaced unspecified condyle fracture of lower end of right femur, sequela
        "S72415A",  # Nondisplaced unspecified condyle fracture of lower end of left femur, initial encounter for closed fracture
        "S72415B",  # Nondisplaced unspecified condyle fracture of lower end of left femur, initial encounter for open fracture type I or II
        "S72415C",  # Nondisplaced unspecified condyle fracture of lower end of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72415D",  # Nondisplaced unspecified condyle fracture of lower end of left femur, subsequent encounter for closed fracture with routine healing
        "S72415E",  # Nondisplaced unspecified condyle fracture of lower end of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72415F",  # Nondisplaced unspecified condyle fracture of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72415G",  # Nondisplaced unspecified condyle fracture of lower end of left femur, subsequent encounter for closed fracture with delayed healing
        "S72415H",  # Nondisplaced unspecified condyle fracture of lower end of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72415J",  # Nondisplaced unspecified condyle fracture of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72415K",  # Nondisplaced unspecified condyle fracture of lower end of left femur, subsequent encounter for closed fracture with nonunion
        "S72415M",  # Nondisplaced unspecified condyle fracture of lower end of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72415N",  # Nondisplaced unspecified condyle fracture of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72415P",  # Nondisplaced unspecified condyle fracture of lower end of left femur, subsequent encounter for closed fracture with malunion
        "S72415Q",  # Nondisplaced unspecified condyle fracture of lower end of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72415R",  # Nondisplaced unspecified condyle fracture of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72415S",  # Nondisplaced unspecified condyle fracture of lower end of left femur, sequela
        "S72416A",  # Nondisplaced unspecified condyle fracture of lower end of unspecified femur, initial encounter for closed fracture
        "S72416B",  # Nondisplaced unspecified condyle fracture of lower end of unspecified femur, initial encounter for open fracture type I or II
        "S72416C",  # Nondisplaced unspecified condyle fracture of lower end of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72416D",  # Nondisplaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72416E",  # Nondisplaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72416F",  # Nondisplaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72416G",  # Nondisplaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72416H",  # Nondisplaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72416J",  # Nondisplaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72416K",  # Nondisplaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72416M",  # Nondisplaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72416N",  # Nondisplaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72416P",  # Nondisplaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72416Q",  # Nondisplaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72416R",  # Nondisplaced unspecified condyle fracture of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72416S",  # Nondisplaced unspecified condyle fracture of lower end of unspecified femur, sequela
        "S72421A",  # Displaced fracture of lateral condyle of right femur, initial encounter for closed fracture
        "S72421B",  # Displaced fracture of lateral condyle of right femur, initial encounter for open fracture type I or II
        "S72421C",  # Displaced fracture of lateral condyle of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72421D",  # Displaced fracture of lateral condyle of right femur, subsequent encounter for closed fracture with routine healing
        "S72421E",  # Displaced fracture of lateral condyle of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72421F",  # Displaced fracture of lateral condyle of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72421G",  # Displaced fracture of lateral condyle of right femur, subsequent encounter for closed fracture with delayed healing
        "S72421H",  # Displaced fracture of lateral condyle of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72421J",  # Displaced fracture of lateral condyle of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72421K",  # Displaced fracture of lateral condyle of right femur, subsequent encounter for closed fracture with nonunion
        "S72421M",  # Displaced fracture of lateral condyle of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72421N",  # Displaced fracture of lateral condyle of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72421P",  # Displaced fracture of lateral condyle of right femur, subsequent encounter for closed fracture with malunion
        "S72421Q",  # Displaced fracture of lateral condyle of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72421R",  # Displaced fracture of lateral condyle of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72421S",  # Displaced fracture of lateral condyle of right femur, sequela
        "S72422A",  # Displaced fracture of lateral condyle of left femur, initial encounter for closed fracture
        "S72422B",  # Displaced fracture of lateral condyle of left femur, initial encounter for open fracture type I or II
        "S72422C",  # Displaced fracture of lateral condyle of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72422D",  # Displaced fracture of lateral condyle of left femur, subsequent encounter for closed fracture with routine healing
        "S72422E",  # Displaced fracture of lateral condyle of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72422F",  # Displaced fracture of lateral condyle of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72422G",  # Displaced fracture of lateral condyle of left femur, subsequent encounter for closed fracture with delayed healing
        "S72422H",  # Displaced fracture of lateral condyle of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72422J",  # Displaced fracture of lateral condyle of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72422K",  # Displaced fracture of lateral condyle of left femur, subsequent encounter for closed fracture with nonunion
        "S72422M",  # Displaced fracture of lateral condyle of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72422N",  # Displaced fracture of lateral condyle of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72422P",  # Displaced fracture of lateral condyle of left femur, subsequent encounter for closed fracture with malunion
        "S72422Q",  # Displaced fracture of lateral condyle of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72422R",  # Displaced fracture of lateral condyle of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72422S",  # Displaced fracture of lateral condyle of left femur, sequela
        "S72423A",  # Displaced fracture of lateral condyle of unspecified femur, initial encounter for closed fracture
        "S72423B",  # Displaced fracture of lateral condyle of unspecified femur, initial encounter for open fracture type I or II
        "S72423C",  # Displaced fracture of lateral condyle of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72423D",  # Displaced fracture of lateral condyle of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72423E",  # Displaced fracture of lateral condyle of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72423F",  # Displaced fracture of lateral condyle of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72423G",  # Displaced fracture of lateral condyle of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72423H",  # Displaced fracture of lateral condyle of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72423J",  # Displaced fracture of lateral condyle of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72423K",  # Displaced fracture of lateral condyle of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72423M",  # Displaced fracture of lateral condyle of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72423N",  # Displaced fracture of lateral condyle of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72423P",  # Displaced fracture of lateral condyle of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72423Q",  # Displaced fracture of lateral condyle of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72423R",  # Displaced fracture of lateral condyle of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72423S",  # Displaced fracture of lateral condyle of unspecified femur, sequela
        "S72424A",  # Nondisplaced fracture of lateral condyle of right femur, initial encounter for closed fracture
        "S72424B",  # Nondisplaced fracture of lateral condyle of right femur, initial encounter for open fracture type I or II
        "S72424C",  # Nondisplaced fracture of lateral condyle of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72424D",  # Nondisplaced fracture of lateral condyle of right femur, subsequent encounter for closed fracture with routine healing
        "S72424E",  # Nondisplaced fracture of lateral condyle of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72424F",  # Nondisplaced fracture of lateral condyle of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72424G",  # Nondisplaced fracture of lateral condyle of right femur, subsequent encounter for closed fracture with delayed healing
        "S72424H",  # Nondisplaced fracture of lateral condyle of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72424J",  # Nondisplaced fracture of lateral condyle of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72424K",  # Nondisplaced fracture of lateral condyle of right femur, subsequent encounter for closed fracture with nonunion
        "S72424M",  # Nondisplaced fracture of lateral condyle of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72424N",  # Nondisplaced fracture of lateral condyle of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72424P",  # Nondisplaced fracture of lateral condyle of right femur, subsequent encounter for closed fracture with malunion
        "S72424Q",  # Nondisplaced fracture of lateral condyle of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72424R",  # Nondisplaced fracture of lateral condyle of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72424S",  # Nondisplaced fracture of lateral condyle of right femur, sequela
        "S72425A",  # Nondisplaced fracture of lateral condyle of left femur, initial encounter for closed fracture
        "S72425B",  # Nondisplaced fracture of lateral condyle of left femur, initial encounter for open fracture type I or II
        "S72425C",  # Nondisplaced fracture of lateral condyle of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72425D",  # Nondisplaced fracture of lateral condyle of left femur, subsequent encounter for closed fracture with routine healing
        "S72425E",  # Nondisplaced fracture of lateral condyle of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72425F",  # Nondisplaced fracture of lateral condyle of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72425G",  # Nondisplaced fracture of lateral condyle of left femur, subsequent encounter for closed fracture with delayed healing
        "S72425H",  # Nondisplaced fracture of lateral condyle of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72425J",  # Nondisplaced fracture of lateral condyle of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72425K",  # Nondisplaced fracture of lateral condyle of left femur, subsequent encounter for closed fracture with nonunion
        "S72425M",  # Nondisplaced fracture of lateral condyle of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72425N",  # Nondisplaced fracture of lateral condyle of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72425P",  # Nondisplaced fracture of lateral condyle of left femur, subsequent encounter for closed fracture with malunion
        "S72425Q",  # Nondisplaced fracture of lateral condyle of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72425R",  # Nondisplaced fracture of lateral condyle of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72425S",  # Nondisplaced fracture of lateral condyle of left femur, sequela
        "S72426A",  # Nondisplaced fracture of lateral condyle of unspecified femur, initial encounter for closed fracture
        "S72426B",  # Nondisplaced fracture of lateral condyle of unspecified femur, initial encounter for open fracture type I or II
        "S72426C",  # Nondisplaced fracture of lateral condyle of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72426D",  # Nondisplaced fracture of lateral condyle of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72426E",  # Nondisplaced fracture of lateral condyle of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72426F",  # Nondisplaced fracture of lateral condyle of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72426G",  # Nondisplaced fracture of lateral condyle of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72426H",  # Nondisplaced fracture of lateral condyle of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72426J",  # Nondisplaced fracture of lateral condyle of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72426K",  # Nondisplaced fracture of lateral condyle of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72426M",  # Nondisplaced fracture of lateral condyle of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72426N",  # Nondisplaced fracture of lateral condyle of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72426P",  # Nondisplaced fracture of lateral condyle of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72426Q",  # Nondisplaced fracture of lateral condyle of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72426R",  # Nondisplaced fracture of lateral condyle of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72426S",  # Nondisplaced fracture of lateral condyle of unspecified femur, sequela
        "S72431A",  # Displaced fracture of medial condyle of right femur, initial encounter for closed fracture
        "S72431B",  # Displaced fracture of medial condyle of right femur, initial encounter for open fracture type I or II
        "S72431C",  # Displaced fracture of medial condyle of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72431D",  # Displaced fracture of medial condyle of right femur, subsequent encounter for closed fracture with routine healing
        "S72431E",  # Displaced fracture of medial condyle of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72431F",  # Displaced fracture of medial condyle of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72431G",  # Displaced fracture of medial condyle of right femur, subsequent encounter for closed fracture with delayed healing
        "S72431H",  # Displaced fracture of medial condyle of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72431J",  # Displaced fracture of medial condyle of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72431K",  # Displaced fracture of medial condyle of right femur, subsequent encounter for closed fracture with nonunion
        "S72431M",  # Displaced fracture of medial condyle of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72431N",  # Displaced fracture of medial condyle of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72431P",  # Displaced fracture of medial condyle of right femur, subsequent encounter for closed fracture with malunion
        "S72431Q",  # Displaced fracture of medial condyle of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72431R",  # Displaced fracture of medial condyle of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72431S",  # Displaced fracture of medial condyle of right femur, sequela
        "S72432A",  # Displaced fracture of medial condyle of left femur, initial encounter for closed fracture
        "S72432B",  # Displaced fracture of medial condyle of left femur, initial encounter for open fracture type I or II
        "S72432C",  # Displaced fracture of medial condyle of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72432D",  # Displaced fracture of medial condyle of left femur, subsequent encounter for closed fracture with routine healing
        "S72432E",  # Displaced fracture of medial condyle of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72432F",  # Displaced fracture of medial condyle of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72432G",  # Displaced fracture of medial condyle of left femur, subsequent encounter for closed fracture with delayed healing
        "S72432H",  # Displaced fracture of medial condyle of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72432J",  # Displaced fracture of medial condyle of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72432K",  # Displaced fracture of medial condyle of left femur, subsequent encounter for closed fracture with nonunion
        "S72432M",  # Displaced fracture of medial condyle of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72432N",  # Displaced fracture of medial condyle of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72432P",  # Displaced fracture of medial condyle of left femur, subsequent encounter for closed fracture with malunion
        "S72432Q",  # Displaced fracture of medial condyle of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72432R",  # Displaced fracture of medial condyle of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72432S",  # Displaced fracture of medial condyle of left femur, sequela
        "S72433A",  # Displaced fracture of medial condyle of unspecified femur, initial encounter for closed fracture
        "S72433B",  # Displaced fracture of medial condyle of unspecified femur, initial encounter for open fracture type I or II
        "S72433C",  # Displaced fracture of medial condyle of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72433D",  # Displaced fracture of medial condyle of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72433E",  # Displaced fracture of medial condyle of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72433F",  # Displaced fracture of medial condyle of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72433G",  # Displaced fracture of medial condyle of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72433H",  # Displaced fracture of medial condyle of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72433J",  # Displaced fracture of medial condyle of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72433K",  # Displaced fracture of medial condyle of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72433M",  # Displaced fracture of medial condyle of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72433N",  # Displaced fracture of medial condyle of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72433P",  # Displaced fracture of medial condyle of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72433Q",  # Displaced fracture of medial condyle of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72433R",  # Displaced fracture of medial condyle of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72433S",  # Displaced fracture of medial condyle of unspecified femur, sequela
        "S72434A",  # Nondisplaced fracture of medial condyle of right femur, initial encounter for closed fracture
        "S72434B",  # Nondisplaced fracture of medial condyle of right femur, initial encounter for open fracture type I or II
        "S72434C",  # Nondisplaced fracture of medial condyle of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72434D",  # Nondisplaced fracture of medial condyle of right femur, subsequent encounter for closed fracture with routine healing
        "S72434E",  # Nondisplaced fracture of medial condyle of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72434F",  # Nondisplaced fracture of medial condyle of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72434G",  # Nondisplaced fracture of medial condyle of right femur, subsequent encounter for closed fracture with delayed healing
        "S72434H",  # Nondisplaced fracture of medial condyle of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72434J",  # Nondisplaced fracture of medial condyle of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72434K",  # Nondisplaced fracture of medial condyle of right femur, subsequent encounter for closed fracture with nonunion
        "S72434M",  # Nondisplaced fracture of medial condyle of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72434N",  # Nondisplaced fracture of medial condyle of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72434P",  # Nondisplaced fracture of medial condyle of right femur, subsequent encounter for closed fracture with malunion
        "S72434Q",  # Nondisplaced fracture of medial condyle of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72434R",  # Nondisplaced fracture of medial condyle of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72434S",  # Nondisplaced fracture of medial condyle of right femur, sequela
        "S72435A",  # Nondisplaced fracture of medial condyle of left femur, initial encounter for closed fracture
        "S72435B",  # Nondisplaced fracture of medial condyle of left femur, initial encounter for open fracture type I or II
        "S72435C",  # Nondisplaced fracture of medial condyle of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72435D",  # Nondisplaced fracture of medial condyle of left femur, subsequent encounter for closed fracture with routine healing
        "S72435E",  # Nondisplaced fracture of medial condyle of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72435F",  # Nondisplaced fracture of medial condyle of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72435G",  # Nondisplaced fracture of medial condyle of left femur, subsequent encounter for closed fracture with delayed healing
        "S72435H",  # Nondisplaced fracture of medial condyle of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72435J",  # Nondisplaced fracture of medial condyle of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72435K",  # Nondisplaced fracture of medial condyle of left femur, subsequent encounter for closed fracture with nonunion
        "S72435M",  # Nondisplaced fracture of medial condyle of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72435N",  # Nondisplaced fracture of medial condyle of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72435P",  # Nondisplaced fracture of medial condyle of left femur, subsequent encounter for closed fracture with malunion
        "S72435Q",  # Nondisplaced fracture of medial condyle of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72435R",  # Nondisplaced fracture of medial condyle of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72435S",  # Nondisplaced fracture of medial condyle of left femur, sequela
        "S72436A",  # Nondisplaced fracture of medial condyle of unspecified femur, initial encounter for closed fracture
        "S72436B",  # Nondisplaced fracture of medial condyle of unspecified femur, initial encounter for open fracture type I or II
        "S72436C",  # Nondisplaced fracture of medial condyle of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72436D",  # Nondisplaced fracture of medial condyle of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72436E",  # Nondisplaced fracture of medial condyle of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72436F",  # Nondisplaced fracture of medial condyle of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72436G",  # Nondisplaced fracture of medial condyle of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72436H",  # Nondisplaced fracture of medial condyle of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72436J",  # Nondisplaced fracture of medial condyle of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72436K",  # Nondisplaced fracture of medial condyle of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72436M",  # Nondisplaced fracture of medial condyle of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72436N",  # Nondisplaced fracture of medial condyle of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72436P",  # Nondisplaced fracture of medial condyle of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72436Q",  # Nondisplaced fracture of medial condyle of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72436R",  # Nondisplaced fracture of medial condyle of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72436S",  # Nondisplaced fracture of medial condyle of unspecified femur, sequela
        "S72441A",  # Displaced fracture of lower epiphysis (separation) of right femur, initial encounter for closed fracture
        "S72441B",  # Displaced fracture of lower epiphysis (separation) of right femur, initial encounter for open fracture type I or II
        "S72441C",  # Displaced fracture of lower epiphysis (separation) of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72441D",  # Displaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for closed fracture with routine healing
        "S72441E",  # Displaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72441F",  # Displaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72441G",  # Displaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for closed fracture with delayed healing
        "S72441H",  # Displaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72441J",  # Displaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72441K",  # Displaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for closed fracture with nonunion
        "S72441M",  # Displaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72441N",  # Displaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72441P",  # Displaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for closed fracture with malunion
        "S72441Q",  # Displaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72441R",  # Displaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72441S",  # Displaced fracture of lower epiphysis (separation) of right femur, sequela
        "S72442A",  # Displaced fracture of lower epiphysis (separation) of left femur, initial encounter for closed fracture
        "S72442B",  # Displaced fracture of lower epiphysis (separation) of left femur, initial encounter for open fracture type I or II
        "S72442C",  # Displaced fracture of lower epiphysis (separation) of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72442D",  # Displaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for closed fracture with routine healing
        "S72442E",  # Displaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72442F",  # Displaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72442G",  # Displaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for closed fracture with delayed healing
        "S72442H",  # Displaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72442J",  # Displaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72442K",  # Displaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for closed fracture with nonunion
        "S72442M",  # Displaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72442N",  # Displaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72442P",  # Displaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for closed fracture with malunion
        "S72442Q",  # Displaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72442R",  # Displaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72442S",  # Displaced fracture of lower epiphysis (separation) of left femur, sequela
        "S72443A",  # Displaced fracture of lower epiphysis (separation) of unspecified femur, initial encounter for closed fracture
        "S72443B",  # Displaced fracture of lower epiphysis (separation) of unspecified femur, initial encounter for open fracture type I or II
        "S72443C",  # Displaced fracture of lower epiphysis (separation) of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72443D",  # Displaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72443E",  # Displaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72443F",  # Displaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72443G",  # Displaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72443H",  # Displaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72443J",  # Displaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72443K",  # Displaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72443M",  # Displaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72443N",  # Displaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72443P",  # Displaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72443Q",  # Displaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72443R",  # Displaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72443S",  # Displaced fracture of lower epiphysis (separation) of unspecified femur, sequela
        "S72444A",  # Nondisplaced fracture of lower epiphysis (separation) of right femur, initial encounter for closed fracture
        "S72444B",  # Nondisplaced fracture of lower epiphysis (separation) of right femur, initial encounter for open fracture type I or II
        "S72444C",  # Nondisplaced fracture of lower epiphysis (separation) of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72444D",  # Nondisplaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for closed fracture with routine healing
        "S72444E",  # Nondisplaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72444F",  # Nondisplaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72444G",  # Nondisplaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for closed fracture with delayed healing
        "S72444H",  # Nondisplaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72444J",  # Nondisplaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72444K",  # Nondisplaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for closed fracture with nonunion
        "S72444M",  # Nondisplaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72444N",  # Nondisplaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72444P",  # Nondisplaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for closed fracture with malunion
        "S72444Q",  # Nondisplaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72444R",  # Nondisplaced fracture of lower epiphysis (separation) of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72444S",  # Nondisplaced fracture of lower epiphysis (separation) of right femur, sequela
        "S72445A",  # Nondisplaced fracture of lower epiphysis (separation) of left femur, initial encounter for closed fracture
        "S72445B",  # Nondisplaced fracture of lower epiphysis (separation) of left femur, initial encounter for open fracture type I or II
        "S72445C",  # Nondisplaced fracture of lower epiphysis (separation) of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72445D",  # Nondisplaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for closed fracture with routine healing
        "S72445E",  # Nondisplaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72445F",  # Nondisplaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72445G",  # Nondisplaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for closed fracture with delayed healing
        "S72445H",  # Nondisplaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72445J",  # Nondisplaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72445K",  # Nondisplaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for closed fracture with nonunion
        "S72445M",  # Nondisplaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72445N",  # Nondisplaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72445P",  # Nondisplaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for closed fracture with malunion
        "S72445Q",  # Nondisplaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72445R",  # Nondisplaced fracture of lower epiphysis (separation) of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72445S",  # Nondisplaced fracture of lower epiphysis (separation) of left femur, sequela
        "S72446A",  # Nondisplaced fracture of lower epiphysis (separation) of unspecified femur, initial encounter for closed fracture
        "S72446B",  # Nondisplaced fracture of lower epiphysis (separation) of unspecified femur, initial encounter for open fracture type I or II
        "S72446C",  # Nondisplaced fracture of lower epiphysis (separation) of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72446D",  # Nondisplaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72446E",  # Nondisplaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72446F",  # Nondisplaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72446G",  # Nondisplaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72446H",  # Nondisplaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72446J",  # Nondisplaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72446K",  # Nondisplaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72446M",  # Nondisplaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72446N",  # Nondisplaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72446P",  # Nondisplaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72446Q",  # Nondisplaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72446R",  # Nondisplaced fracture of lower epiphysis (separation) of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72446S",  # Nondisplaced fracture of lower epiphysis (separation) of unspecified femur, sequela
        "S72451A",  # Displaced supracondylar fracture without intracondylar extension of lower end of right femur, initial encounter for closed fracture
        "S72451B",  # Displaced supracondylar fracture without intracondylar extension of lower end of right femur, initial encounter for open fracture type I or II
        "S72451C",  # Displaced supracondylar fracture without intracondylar extension of lower end of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72451D",  # Displaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for closed fracture with routine healing
        "S72451E",  # Displaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72451F",  # Displaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72451G",  # Displaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for closed fracture with delayed healing
        "S72451H",  # Displaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72451J",  # Displaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72451K",  # Displaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for closed fracture with nonunion
        "S72451M",  # Displaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72451N",  # Displaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72451P",  # Displaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for closed fracture with malunion
        "S72451Q",  # Displaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72451R",  # Displaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72451S",  # Displaced supracondylar fracture without intracondylar extension of lower end of right femur, sequela
        "S72452A",  # Displaced supracondylar fracture without intracondylar extension of lower end of left femur, initial encounter for closed fracture
        "S72452B",  # Displaced supracondylar fracture without intracondylar extension of lower end of left femur, initial encounter for open fracture type I or II
        "S72452C",  # Displaced supracondylar fracture without intracondylar extension of lower end of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72452D",  # Displaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for closed fracture with routine healing
        "S72452E",  # Displaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72452F",  # Displaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72452G",  # Displaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for closed fracture with delayed healing
        "S72452H",  # Displaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72452J",  # Displaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72452K",  # Displaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for closed fracture with nonunion
        "S72452M",  # Displaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72452N",  # Displaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72452P",  # Displaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for closed fracture with malunion
        "S72452Q",  # Displaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72452R",  # Displaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72452S",  # Displaced supracondylar fracture without intracondylar extension of lower end of left femur, sequela
        "S72453A",  # Displaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, initial encounter for closed fracture
        "S72453B",  # Displaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, initial encounter for open fracture type I or II
        "S72453C",  # Displaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72453D",  # Displaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72453E",  # Displaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72453F",  # Displaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72453G",  # Displaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72453H",  # Displaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72453J",  # Displaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72453K",  # Displaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72453M",  # Displaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72453N",  # Displaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72453P",  # Displaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72453Q",  # Displaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72453R",  # Displaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72453S",  # Displaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, sequela
        "S72454A",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of right femur, initial encounter for closed fracture
        "S72454B",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of right femur, initial encounter for open fracture type I or II
        "S72454C",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72454D",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for closed fracture with routine healing
        "S72454E",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72454F",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72454G",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for closed fracture with delayed healing
        "S72454H",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72454J",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72454K",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for closed fracture with nonunion
        "S72454M",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72454N",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72454P",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for closed fracture with malunion
        "S72454Q",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72454R",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72454S",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of right femur, sequela
        "S72455A",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of left femur, initial encounter for closed fracture
        "S72455B",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of left femur, initial encounter for open fracture type I or II
        "S72455C",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72455D",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for closed fracture with routine healing
        "S72455E",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72455F",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72455G",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for closed fracture with delayed healing
        "S72455H",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72455J",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72455K",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for closed fracture with nonunion
        "S72455M",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72455N",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72455P",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for closed fracture with malunion
        "S72455Q",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72455R",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72455S",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of left femur, sequela
        "S72456A",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, initial encounter for closed fracture
        "S72456B",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, initial encounter for open fracture type I or II
        "S72456C",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72456D",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72456E",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72456F",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72456G",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72456H",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72456J",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72456K",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72456M",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72456N",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72456P",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72456Q",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72456R",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72456S",  # Nondisplaced supracondylar fracture without intracondylar extension of lower end of unspecified femur, sequela
        "S72461A",  # Displaced supracondylar fracture with intracondylar extension of lower end of right femur, initial encounter for closed fracture
        "S72461B",  # Displaced supracondylar fracture with intracondylar extension of lower end of right femur, initial encounter for open fracture type I or II
        "S72461C",  # Displaced supracondylar fracture with intracondylar extension of lower end of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72461D",  # Displaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for closed fracture with routine healing
        "S72461E",  # Displaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72461F",  # Displaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72461G",  # Displaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for closed fracture with delayed healing
        "S72461H",  # Displaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72461J",  # Displaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72461K",  # Displaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for closed fracture with nonunion
        "S72461M",  # Displaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72461N",  # Displaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72461P",  # Displaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for closed fracture with malunion
        "S72461Q",  # Displaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72461R",  # Displaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72461S",  # Displaced supracondylar fracture with intracondylar extension of lower end of right femur, sequela
        "S72462A",  # Displaced supracondylar fracture with intracondylar extension of lower end of left femur, initial encounter for closed fracture
        "S72462B",  # Displaced supracondylar fracture with intracondylar extension of lower end of left femur, initial encounter for open fracture type I or II
        "S72462C",  # Displaced supracondylar fracture with intracondylar extension of lower end of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72462D",  # Displaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for closed fracture with routine healing
        "S72462E",  # Displaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72462F",  # Displaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72462G",  # Displaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for closed fracture with delayed healing
        "S72462H",  # Displaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72462J",  # Displaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72462K",  # Displaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for closed fracture with nonunion
        "S72462M",  # Displaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72462N",  # Displaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72462P",  # Displaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for closed fracture with malunion
        "S72462Q",  # Displaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72462R",  # Displaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72462S",  # Displaced supracondylar fracture with intracondylar extension of lower end of left femur, sequela
        "S72463A",  # Displaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, initial encounter for closed fracture
        "S72463B",  # Displaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, initial encounter for open fracture type I or II
        "S72463C",  # Displaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72463D",  # Displaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72463E",  # Displaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72463F",  # Displaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72463G",  # Displaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72463H",  # Displaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72463J",  # Displaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72463K",  # Displaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72463M",  # Displaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72463N",  # Displaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72463P",  # Displaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72463Q",  # Displaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72463R",  # Displaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72463S",  # Displaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, sequela
        "S72464A",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of right femur, initial encounter for closed fracture
        "S72464B",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of right femur, initial encounter for open fracture type I or II
        "S72464C",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72464D",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for closed fracture with routine healing
        "S72464E",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72464F",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72464G",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for closed fracture with delayed healing
        "S72464H",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72464J",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72464K",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for closed fracture with nonunion
        "S72464M",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72464N",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72464P",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for closed fracture with malunion
        "S72464Q",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72464R",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72464S",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of right femur, sequela
        "S72465A",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of left femur, initial encounter for closed fracture
        "S72465B",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of left femur, initial encounter for open fracture type I or II
        "S72465C",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72465D",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for closed fracture with routine healing
        "S72465E",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72465F",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72465G",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for closed fracture with delayed healing
        "S72465H",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72465J",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72465K",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for closed fracture with nonunion
        "S72465M",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72465N",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72465P",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for closed fracture with malunion
        "S72465Q",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72465R",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72465S",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of left femur, sequela
        "S72466A",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, initial encounter for closed fracture
        "S72466B",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, initial encounter for open fracture type I or II
        "S72466C",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72466D",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72466E",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72466F",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72466G",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72466H",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72466J",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72466K",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72466M",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72466N",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72466P",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72466Q",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72466R",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72466S",  # Nondisplaced supracondylar fracture with intracondylar extension of lower end of unspecified femur, sequela
        "S72471A",  # Torus fracture of lower end of right femur, initial encounter for closed fracture
        "S72471D",  # Torus fracture of lower end of right femur, subsequent encounter for fracture with routine healing
        "S72471G",  # Torus fracture of lower end of right femur, subsequent encounter for fracture with delayed healing
        "S72471K",  # Torus fracture of lower end of right femur, subsequent encounter for fracture with nonunion
        "S72471P",  # Torus fracture of lower end of right femur, subsequent encounter for fracture with malunion
        "S72471S",  # Torus fracture of lower end of right femur, sequela
        "S72472A",  # Torus fracture of lower end of left femur, initial encounter for closed fracture
        "S72472D",  # Torus fracture of lower end of left femur, subsequent encounter for fracture with routine healing
        "S72472G",  # Torus fracture of lower end of left femur, subsequent encounter for fracture with delayed healing
        "S72472K",  # Torus fracture of lower end of left femur, subsequent encounter for fracture with nonunion
        "S72472P",  # Torus fracture of lower end of left femur, subsequent encounter for fracture with malunion
        "S72472S",  # Torus fracture of lower end of left femur, sequela
        "S72479A",  # Torus fracture of lower end of unspecified femur, initial encounter for closed fracture
        "S72479D",  # Torus fracture of lower end of unspecified femur, subsequent encounter for fracture with routine healing
        "S72479G",  # Torus fracture of lower end of unspecified femur, subsequent encounter for fracture with delayed healing
        "S72479K",  # Torus fracture of lower end of unspecified femur, subsequent encounter for fracture with nonunion
        "S72479P",  # Torus fracture of lower end of unspecified femur, subsequent encounter for fracture with malunion
        "S72479S",  # Torus fracture of lower end of unspecified femur, sequela
        "S72491A",  # Other fracture of lower end of right femur, initial encounter for closed fracture
        "S72491B",  # Other fracture of lower end of right femur, initial encounter for open fracture type I or II
        "S72491C",  # Other fracture of lower end of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72491D",  # Other fracture of lower end of right femur, subsequent encounter for closed fracture with routine healing
        "S72491E",  # Other fracture of lower end of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S72491F",  # Other fracture of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72491G",  # Other fracture of lower end of right femur, subsequent encounter for closed fracture with delayed healing
        "S72491H",  # Other fracture of lower end of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72491J",  # Other fracture of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72491K",  # Other fracture of lower end of right femur, subsequent encounter for closed fracture with nonunion
        "S72491M",  # Other fracture of lower end of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S72491N",  # Other fracture of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72491P",  # Other fracture of lower end of right femur, subsequent encounter for closed fracture with malunion
        "S72491Q",  # Other fracture of lower end of right femur, subsequent encounter for open fracture type I or II with malunion
        "S72491R",  # Other fracture of lower end of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72491S",  # Other fracture of lower end of right femur, sequela
        "S72492A",  # Other fracture of lower end of left femur, initial encounter for closed fracture
        "S72492B",  # Other fracture of lower end of left femur, initial encounter for open fracture type I or II
        "S72492C",  # Other fracture of lower end of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72492D",  # Other fracture of lower end of left femur, subsequent encounter for closed fracture with routine healing
        "S72492E",  # Other fracture of lower end of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S72492F",  # Other fracture of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72492G",  # Other fracture of lower end of left femur, subsequent encounter for closed fracture with delayed healing
        "S72492H",  # Other fracture of lower end of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72492J",  # Other fracture of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72492K",  # Other fracture of lower end of left femur, subsequent encounter for closed fracture with nonunion
        "S72492M",  # Other fracture of lower end of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S72492N",  # Other fracture of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72492P",  # Other fracture of lower end of left femur, subsequent encounter for closed fracture with malunion
        "S72492Q",  # Other fracture of lower end of left femur, subsequent encounter for open fracture type I or II with malunion
        "S72492R",  # Other fracture of lower end of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72492S",  # Other fracture of lower end of left femur, sequela
        "S72499A",  # Other fracture of lower end of unspecified femur, initial encounter for closed fracture
        "S72499B",  # Other fracture of lower end of unspecified femur, initial encounter for open fracture type I or II
        "S72499C",  # Other fracture of lower end of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S72499D",  # Other fracture of lower end of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S72499E",  # Other fracture of lower end of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S72499F",  # Other fracture of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S72499G",  # Other fracture of lower end of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S72499H",  # Other fracture of lower end of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S72499J",  # Other fracture of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S72499K",  # Other fracture of lower end of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S72499M",  # Other fracture of lower end of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S72499N",  # Other fracture of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S72499P",  # Other fracture of lower end of unspecified femur, subsequent encounter for closed fracture with malunion
        "S72499Q",  # Other fracture of lower end of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S72499R",  # Other fracture of lower end of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S72499S",  # Other fracture of lower end of unspecified femur, sequela
        "S728X1A",  # Other fracture of right femur, initial encounter for closed fracture
        "S728X1B",  # Other fracture of right femur, initial encounter for open fracture type I or II
        "S728X1C",  # Other fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S728X1D",  # Other fracture of right femur, subsequent encounter for closed fracture with routine healing
        "S728X1E",  # Other fracture of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S728X1F",  # Other fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S728X1G",  # Other fracture of right femur, subsequent encounter for closed fracture with delayed healing
        "S728X1H",  # Other fracture of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S728X1J",  # Other fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S728X1K",  # Other fracture of right femur, subsequent encounter for closed fracture with nonunion
        "S728X1M",  # Other fracture of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S728X1N",  # Other fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S728X1P",  # Other fracture of right femur, subsequent encounter for closed fracture with malunion
        "S728X1Q",  # Other fracture of right femur, subsequent encounter for open fracture type I or II with malunion
        "S728X1R",  # Other fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S728X1S",  # Other fracture of right femur, sequela
        "S728X2A",  # Other fracture of left femur, initial encounter for closed fracture
        "S728X2B",  # Other fracture of left femur, initial encounter for open fracture type I or II
        "S728X2C",  # Other fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S728X2D",  # Other fracture of left femur, subsequent encounter for closed fracture with routine healing
        "S728X2E",  # Other fracture of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S728X2F",  # Other fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S728X2G",  # Other fracture of left femur, subsequent encounter for closed fracture with delayed healing
        "S728X2H",  # Other fracture of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S728X2J",  # Other fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S728X2K",  # Other fracture of left femur, subsequent encounter for closed fracture with nonunion
        "S728X2M",  # Other fracture of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S728X2N",  # Other fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S728X2P",  # Other fracture of left femur, subsequent encounter for closed fracture with malunion
        "S728X2Q",  # Other fracture of left femur, subsequent encounter for open fracture type I or II with malunion
        "S728X2R",  # Other fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S728X2S",  # Other fracture of left femur, sequela
        "S728X9A",  # Other fracture of unspecified femur, initial encounter for closed fracture
        "S728X9B",  # Other fracture of unspecified femur, initial encounter for open fracture type I or II
        "S728X9C",  # Other fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S728X9D",  # Other fracture of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S728X9E",  # Other fracture of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S728X9F",  # Other fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S728X9G",  # Other fracture of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S728X9H",  # Other fracture of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S728X9J",  # Other fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S728X9K",  # Other fracture of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S728X9M",  # Other fracture of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S728X9N",  # Other fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S728X9P",  # Other fracture of unspecified femur, subsequent encounter for closed fracture with malunion
        "S728X9Q",  # Other fracture of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S728X9R",  # Other fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S728X9S",  # Other fracture of unspecified femur, sequela
        "S7290XA",  # Unspecified fracture of unspecified femur, initial encounter for closed fracture
        "S7290XB",  # Unspecified fracture of unspecified femur, initial encounter for open fracture type I or II
        "S7290XC",  # Unspecified fracture of unspecified femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S7290XD",  # Unspecified fracture of unspecified femur, subsequent encounter for closed fracture with routine healing
        "S7290XE",  # Unspecified fracture of unspecified femur, subsequent encounter for open fracture type I or II with routine healing
        "S7290XF",  # Unspecified fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S7290XG",  # Unspecified fracture of unspecified femur, subsequent encounter for closed fracture with delayed healing
        "S7290XH",  # Unspecified fracture of unspecified femur, subsequent encounter for open fracture type I or II with delayed healing
        "S7290XJ",  # Unspecified fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S7290XK",  # Unspecified fracture of unspecified femur, subsequent encounter for closed fracture with nonunion
        "S7290XM",  # Unspecified fracture of unspecified femur, subsequent encounter for open fracture type I or II with nonunion
        "S7290XN",  # Unspecified fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S7290XP",  # Unspecified fracture of unspecified femur, subsequent encounter for closed fracture with malunion
        "S7290XQ",  # Unspecified fracture of unspecified femur, subsequent encounter for open fracture type I or II with malunion
        "S7290XR",  # Unspecified fracture of unspecified femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S7290XS",  # Unspecified fracture of unspecified femur, sequela
        "S7291XA",  # Unspecified fracture of right femur, initial encounter for closed fracture
        "S7291XB",  # Unspecified fracture of right femur, initial encounter for open fracture type I or II
        "S7291XC",  # Unspecified fracture of right femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S7291XD",  # Unspecified fracture of right femur, subsequent encounter for closed fracture with routine healing
        "S7291XE",  # Unspecified fracture of right femur, subsequent encounter for open fracture type I or II with routine healing
        "S7291XF",  # Unspecified fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S7291XG",  # Unspecified fracture of right femur, subsequent encounter for closed fracture with delayed healing
        "S7291XH",  # Unspecified fracture of right femur, subsequent encounter for open fracture type I or II with delayed healing
        "S7291XJ",  # Unspecified fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S7291XK",  # Unspecified fracture of right femur, subsequent encounter for closed fracture with nonunion
        "S7291XM",  # Unspecified fracture of right femur, subsequent encounter for open fracture type I or II with nonunion
        "S7291XN",  # Unspecified fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S7291XP",  # Unspecified fracture of right femur, subsequent encounter for closed fracture with malunion
        "S7291XQ",  # Unspecified fracture of right femur, subsequent encounter for open fracture type I or II with malunion
        "S7291XR",  # Unspecified fracture of right femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S7291XS",  # Unspecified fracture of right femur, sequela
        "S7292XA",  # Unspecified fracture of left femur, initial encounter for closed fracture
        "S7292XB",  # Unspecified fracture of left femur, initial encounter for open fracture type I or II
        "S7292XC",  # Unspecified fracture of left femur, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S7292XD",  # Unspecified fracture of left femur, subsequent encounter for closed fracture with routine healing
        "S7292XE",  # Unspecified fracture of left femur, subsequent encounter for open fracture type I or II with routine healing
        "S7292XF",  # Unspecified fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S7292XG",  # Unspecified fracture of left femur, subsequent encounter for closed fracture with delayed healing
        "S7292XH",  # Unspecified fracture of left femur, subsequent encounter for open fracture type I or II with delayed healing
        "S7292XJ",  # Unspecified fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S7292XK",  # Unspecified fracture of left femur, subsequent encounter for closed fracture with nonunion
        "S7292XM",  # Unspecified fracture of left femur, subsequent encounter for open fracture type I or II with nonunion
        "S7292XN",  # Unspecified fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S7292XP",  # Unspecified fracture of left femur, subsequent encounter for closed fracture with malunion
        "S7292XQ",  # Unspecified fracture of left femur, subsequent encounter for open fracture type I or II with malunion
        "S7292XR",  # Unspecified fracture of left femur, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S7292XS",  # Unspecified fracture of left femur, sequela
        "S79001A",  # Unspecified physeal fracture of upper end of right femur, initial encounter for closed fracture
        "S79001D",  # Unspecified physeal fracture of upper end of right femur, subsequent encounter for fracture with routine healing
        "S79001G",  # Unspecified physeal fracture of upper end of right femur, subsequent encounter for fracture with delayed healing
        "S79001K",  # Unspecified physeal fracture of upper end of right femur, subsequent encounter for fracture with nonunion
        "S79001P",  # Unspecified physeal fracture of upper end of right femur, subsequent encounter for fracture with malunion
        "S79001S",  # Unspecified physeal fracture of upper end of right femur, sequela
        "S79002A",  # Unspecified physeal fracture of upper end of left femur, initial encounter for closed fracture
        "S79002D",  # Unspecified physeal fracture of upper end of left femur, subsequent encounter for fracture with routine healing
        "S79002G",  # Unspecified physeal fracture of upper end of left femur, subsequent encounter for fracture with delayed healing
        "S79002K",  # Unspecified physeal fracture of upper end of left femur, subsequent encounter for fracture with nonunion
        "S79002P",  # Unspecified physeal fracture of upper end of left femur, subsequent encounter for fracture with malunion
        "S79002S",  # Unspecified physeal fracture of upper end of left femur, sequela
        "S79009A",  # Unspecified physeal fracture of upper end of unspecified femur, initial encounter for closed fracture
        "S79009D",  # Unspecified physeal fracture of upper end of unspecified femur, subsequent encounter for fracture with routine healing
        "S79009G",  # Unspecified physeal fracture of upper end of unspecified femur, subsequent encounter for fracture with delayed healing
        "S79009K",  # Unspecified physeal fracture of upper end of unspecified femur, subsequent encounter for fracture with nonunion
        "S79009P",  # Unspecified physeal fracture of upper end of unspecified femur, subsequent encounter for fracture with malunion
        "S79009S",  # Unspecified physeal fracture of upper end of unspecified femur, sequela
        "S79011A",  # Salter-Harris Type I physeal fracture of upper end of right femur, initial encounter for closed fracture
        "S79011D",  # Salter-Harris Type I physeal fracture of upper end of right femur, subsequent encounter for fracture with routine healing
        "S79011G",  # Salter-Harris Type I physeal fracture of upper end of right femur, subsequent encounter for fracture with delayed healing
        "S79011K",  # Salter-Harris Type I physeal fracture of upper end of right femur, subsequent encounter for fracture with nonunion
        "S79011P",  # Salter-Harris Type I physeal fracture of upper end of right femur, subsequent encounter for fracture with malunion
        "S79011S",  # Salter-Harris Type I physeal fracture of upper end of right femur, sequela
        "S79012A",  # Salter-Harris Type I physeal fracture of upper end of left femur, initial encounter for closed fracture
        "S79012D",  # Salter-Harris Type I physeal fracture of upper end of left femur, subsequent encounter for fracture with routine healing
        "S79012G",  # Salter-Harris Type I physeal fracture of upper end of left femur, subsequent encounter for fracture with delayed healing
        "S79012K",  # Salter-Harris Type I physeal fracture of upper end of left femur, subsequent encounter for fracture with nonunion
        "S79012P",  # Salter-Harris Type I physeal fracture of upper end of left femur, subsequent encounter for fracture with malunion
        "S79012S",  # Salter-Harris Type I physeal fracture of upper end of left femur, sequela
        "S79019A",  # Salter-Harris Type I physeal fracture of upper end of unspecified femur, initial encounter for closed fracture
        "S79019D",  # Salter-Harris Type I physeal fracture of upper end of unspecified femur, subsequent encounter for fracture with routine healing
        "S79019G",  # Salter-Harris Type I physeal fracture of upper end of unspecified femur, subsequent encounter for fracture with delayed healing
        "S79019K",  # Salter-Harris Type I physeal fracture of upper end of unspecified femur, subsequent encounter for fracture with nonunion
        "S79019P",  # Salter-Harris Type I physeal fracture of upper end of unspecified femur, subsequent encounter for fracture with malunion
        "S79019S",  # Salter-Harris Type I physeal fracture of upper end of unspecified femur, sequela
        "S79091A",  # Other physeal fracture of upper end of right femur, initial encounter for closed fracture
        "S79091D",  # Other physeal fracture of upper end of right femur, subsequent encounter for fracture with routine healing
        "S79091G",  # Other physeal fracture of upper end of right femur, subsequent encounter for fracture with delayed healing
        "S79091K",  # Other physeal fracture of upper end of right femur, subsequent encounter for fracture with nonunion
        "S79091P",  # Other physeal fracture of upper end of right femur, subsequent encounter for fracture with malunion
        "S79091S",  # Other physeal fracture of upper end of right femur, sequela
        "S79092A",  # Other physeal fracture of upper end of left femur, initial encounter for closed fracture
        "S79092D",  # Other physeal fracture of upper end of left femur, subsequent encounter for fracture with routine healing
        "S79092G",  # Other physeal fracture of upper end of left femur, subsequent encounter for fracture with delayed healing
        "S79092K",  # Other physeal fracture of upper end of left femur, subsequent encounter for fracture with nonunion
        "S79092P",  # Other physeal fracture of upper end of left femur, subsequent encounter for fracture with malunion
        "S79092S",  # Other physeal fracture of upper end of left femur, sequela
        "S79099A",  # Other physeal fracture of upper end of unspecified femur, initial encounter for closed fracture
        "S79099D",  # Other physeal fracture of upper end of unspecified femur, subsequent encounter for fracture with routine healing
        "S79099G",  # Other physeal fracture of upper end of unspecified femur, subsequent encounter for fracture with delayed healing
        "S79099K",  # Other physeal fracture of upper end of unspecified femur, subsequent encounter for fracture with nonunion
        "S79099P",  # Other physeal fracture of upper end of unspecified femur, subsequent encounter for fracture with malunion
        "S79099S",  # Other physeal fracture of upper end of unspecified femur, sequela
        "S79101A",  # Unspecified physeal fracture of lower end of right femur, initial encounter for closed fracture
        "S79101D",  # Unspecified physeal fracture of lower end of right femur, subsequent encounter for fracture with routine healing
        "S79101G",  # Unspecified physeal fracture of lower end of right femur, subsequent encounter for fracture with delayed healing
        "S79101K",  # Unspecified physeal fracture of lower end of right femur, subsequent encounter for fracture with nonunion
        "S79101P",  # Unspecified physeal fracture of lower end of right femur, subsequent encounter for fracture with malunion
        "S79101S",  # Unspecified physeal fracture of lower end of right femur, sequela
        "S79102A",  # Unspecified physeal fracture of lower end of left femur, initial encounter for closed fracture
        "S79102D",  # Unspecified physeal fracture of lower end of left femur, subsequent encounter for fracture with routine healing
        "S79102G",  # Unspecified physeal fracture of lower end of left femur, subsequent encounter for fracture with delayed healing
        "S79102K",  # Unspecified physeal fracture of lower end of left femur, subsequent encounter for fracture with nonunion
        "S79102P",  # Unspecified physeal fracture of lower end of left femur, subsequent encounter for fracture with malunion
        "S79102S",  # Unspecified physeal fracture of lower end of left femur, sequela
        "S79109A",  # Unspecified physeal fracture of lower end of unspecified femur, initial encounter for closed fracture
        "S79109D",  # Unspecified physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with routine healing
        "S79109G",  # Unspecified physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with delayed healing
        "S79109K",  # Unspecified physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with nonunion
        "S79109P",  # Unspecified physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with malunion
        "S79109S",  # Unspecified physeal fracture of lower end of unspecified femur, sequela
        "S79111A",  # Salter-Harris Type I physeal fracture of lower end of right femur, initial encounter for closed fracture
        "S79111D",  # Salter-Harris Type I physeal fracture of lower end of right femur, subsequent encounter for fracture with routine healing
        "S79111G",  # Salter-Harris Type I physeal fracture of lower end of right femur, subsequent encounter for fracture with delayed healing
        "S79111K",  # Salter-Harris Type I physeal fracture of lower end of right femur, subsequent encounter for fracture with nonunion
        "S79111P",  # Salter-Harris Type I physeal fracture of lower end of right femur, subsequent encounter for fracture with malunion
        "S79111S",  # Salter-Harris Type I physeal fracture of lower end of right femur, sequela
        "S79112A",  # Salter-Harris Type I physeal fracture of lower end of left femur, initial encounter for closed fracture
        "S79112D",  # Salter-Harris Type I physeal fracture of lower end of left femur, subsequent encounter for fracture with routine healing
        "S79112G",  # Salter-Harris Type I physeal fracture of lower end of left femur, subsequent encounter for fracture with delayed healing
        "S79112K",  # Salter-Harris Type I physeal fracture of lower end of left femur, subsequent encounter for fracture with nonunion
        "S79112P",  # Salter-Harris Type I physeal fracture of lower end of left femur, subsequent encounter for fracture with malunion
        "S79112S",  # Salter-Harris Type I physeal fracture of lower end of left femur, sequela
        "S79119A",  # Salter-Harris Type I physeal fracture of lower end of unspecified femur, initial encounter for closed fracture
        "S79119D",  # Salter-Harris Type I physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with routine healing
        "S79119G",  # Salter-Harris Type I physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with delayed healing
        "S79119K",  # Salter-Harris Type I physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with nonunion
        "S79119P",  # Salter-Harris Type I physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with malunion
        "S79119S",  # Salter-Harris Type I physeal fracture of lower end of unspecified femur, sequela
        "S79121A",  # Salter-Harris Type II physeal fracture of lower end of right femur, initial encounter for closed fracture
        "S79121D",  # Salter-Harris Type II physeal fracture of lower end of right femur, subsequent encounter for fracture with routine healing
        "S79121G",  # Salter-Harris Type II physeal fracture of lower end of right femur, subsequent encounter for fracture with delayed healing
        "S79121K",  # Salter-Harris Type II physeal fracture of lower end of right femur, subsequent encounter for fracture with nonunion
        "S79121P",  # Salter-Harris Type II physeal fracture of lower end of right femur, subsequent encounter for fracture with malunion
        "S79121S",  # Salter-Harris Type II physeal fracture of lower end of right femur, sequela
        "S79122A",  # Salter-Harris Type II physeal fracture of lower end of left femur, initial encounter for closed fracture
        "S79122D",  # Salter-Harris Type II physeal fracture of lower end of left femur, subsequent encounter for fracture with routine healing
        "S79122G",  # Salter-Harris Type II physeal fracture of lower end of left femur, subsequent encounter for fracture with delayed healing
        "S79122K",  # Salter-Harris Type II physeal fracture of lower end of left femur, subsequent encounter for fracture with nonunion
        "S79122P",  # Salter-Harris Type II physeal fracture of lower end of left femur, subsequent encounter for fracture with malunion
        "S79122S",  # Salter-Harris Type II physeal fracture of lower end of left femur, sequela
        "S79129A",  # Salter-Harris Type II physeal fracture of lower end of unspecified femur, initial encounter for closed fracture
        "S79129D",  # Salter-Harris Type II physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with routine healing
        "S79129G",  # Salter-Harris Type II physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with delayed healing
        "S79129K",  # Salter-Harris Type II physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with nonunion
        "S79129P",  # Salter-Harris Type II physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with malunion
        "S79129S",  # Salter-Harris Type II physeal fracture of lower end of unspecified femur, sequela
        "S79131A",  # Salter-Harris Type III physeal fracture of lower end of right femur, initial encounter for closed fracture
        "S79131D",  # Salter-Harris Type III physeal fracture of lower end of right femur, subsequent encounter for fracture with routine healing
        "S79131G",  # Salter-Harris Type III physeal fracture of lower end of right femur, subsequent encounter for fracture with delayed healing
        "S79131K",  # Salter-Harris Type III physeal fracture of lower end of right femur, subsequent encounter for fracture with nonunion
        "S79131P",  # Salter-Harris Type III physeal fracture of lower end of right femur, subsequent encounter for fracture with malunion
        "S79131S",  # Salter-Harris Type III physeal fracture of lower end of right femur, sequela
        "S79132A",  # Salter-Harris Type III physeal fracture of lower end of left femur, initial encounter for closed fracture
        "S79132D",  # Salter-Harris Type III physeal fracture of lower end of left femur, subsequent encounter for fracture with routine healing
        "S79132G",  # Salter-Harris Type III physeal fracture of lower end of left femur, subsequent encounter for fracture with delayed healing
        "S79132K",  # Salter-Harris Type III physeal fracture of lower end of left femur, subsequent encounter for fracture with nonunion
        "S79132P",  # Salter-Harris Type III physeal fracture of lower end of left femur, subsequent encounter for fracture with malunion
        "S79132S",  # Salter-Harris Type III physeal fracture of lower end of left femur, sequela
        "S79139A",  # Salter-Harris Type III physeal fracture of lower end of unspecified femur, initial encounter for closed fracture
        "S79139D",  # Salter-Harris Type III physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with routine healing
        "S79139G",  # Salter-Harris Type III physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with delayed healing
        "S79139K",  # Salter-Harris Type III physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with nonunion
        "S79139P",  # Salter-Harris Type III physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with malunion
        "S79139S",  # Salter-Harris Type III physeal fracture of lower end of unspecified femur, sequela
        "S79141A",  # Salter-Harris Type IV physeal fracture of lower end of right femur, initial encounter for closed fracture
        "S79141D",  # Salter-Harris Type IV physeal fracture of lower end of right femur, subsequent encounter for fracture with routine healing
        "S79141G",  # Salter-Harris Type IV physeal fracture of lower end of right femur, subsequent encounter for fracture with delayed healing
        "S79141K",  # Salter-Harris Type IV physeal fracture of lower end of right femur, subsequent encounter for fracture with nonunion
        "S79141P",  # Salter-Harris Type IV physeal fracture of lower end of right femur, subsequent encounter for fracture with malunion
        "S79141S",  # Salter-Harris Type IV physeal fracture of lower end of right femur, sequela
        "S79142A",  # Salter-Harris Type IV physeal fracture of lower end of left femur, initial encounter for closed fracture
        "S79142D",  # Salter-Harris Type IV physeal fracture of lower end of left femur, subsequent encounter for fracture with routine healing
        "S79142G",  # Salter-Harris Type IV physeal fracture of lower end of left femur, subsequent encounter for fracture with delayed healing
        "S79142K",  # Salter-Harris Type IV physeal fracture of lower end of left femur, subsequent encounter for fracture with nonunion
        "S79142P",  # Salter-Harris Type IV physeal fracture of lower end of left femur, subsequent encounter for fracture with malunion
        "S79142S",  # Salter-Harris Type IV physeal fracture of lower end of left femur, sequela
        "S79149A",  # Salter-Harris Type IV physeal fracture of lower end of unspecified femur, initial encounter for closed fracture
        "S79149D",  # Salter-Harris Type IV physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with routine healing
        "S79149G",  # Salter-Harris Type IV physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with delayed healing
        "S79149K",  # Salter-Harris Type IV physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with nonunion
        "S79149P",  # Salter-Harris Type IV physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with malunion
        "S79149S",  # Salter-Harris Type IV physeal fracture of lower end of unspecified femur, sequela
        "S79191A",  # Other physeal fracture of lower end of right femur, initial encounter for closed fracture
        "S79191D",  # Other physeal fracture of lower end of right femur, subsequent encounter for fracture with routine healing
        "S79191G",  # Other physeal fracture of lower end of right femur, subsequent encounter for fracture with delayed healing
        "S79191K",  # Other physeal fracture of lower end of right femur, subsequent encounter for fracture with nonunion
        "S79191P",  # Other physeal fracture of lower end of right femur, subsequent encounter for fracture with malunion
        "S79191S",  # Other physeal fracture of lower end of right femur, sequela
        "S79192A",  # Other physeal fracture of lower end of left femur, initial encounter for closed fracture
        "S79192D",  # Other physeal fracture of lower end of left femur, subsequent encounter for fracture with routine healing
        "S79192G",  # Other physeal fracture of lower end of left femur, subsequent encounter for fracture with delayed healing
        "S79192K",  # Other physeal fracture of lower end of left femur, subsequent encounter for fracture with nonunion
        "S79192P",  # Other physeal fracture of lower end of left femur, subsequent encounter for fracture with malunion
        "S79192S",  # Other physeal fracture of lower end of left femur, sequela
        "S79199A",  # Other physeal fracture of lower end of unspecified femur, initial encounter for closed fracture
        "S79199D",  # Other physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with routine healing
        "S79199G",  # Other physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with delayed healing
        "S79199K",  # Other physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with nonunion
        "S79199P",  # Other physeal fracture of lower end of unspecified femur, subsequent encounter for fracture with malunion
        "S79199S",  # Other physeal fracture of lower end of unspecified femur, sequela
        "S82001A",  # Unspecified fracture of right patella, initial encounter for closed fracture
        "S82001B",  # Unspecified fracture of right patella, initial encounter for open fracture type I or II
        "S82001C",  # Unspecified fracture of right patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82001D",  # Unspecified fracture of right patella, subsequent encounter for closed fracture with routine healing
        "S82001E",  # Unspecified fracture of right patella, subsequent encounter for open fracture type I or II with routine healing
        "S82001F",  # Unspecified fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82001G",  # Unspecified fracture of right patella, subsequent encounter for closed fracture with delayed healing
        "S82001H",  # Unspecified fracture of right patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82001J",  # Unspecified fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82001K",  # Unspecified fracture of right patella, subsequent encounter for closed fracture with nonunion
        "S82001M",  # Unspecified fracture of right patella, subsequent encounter for open fracture type I or II with nonunion
        "S82001N",  # Unspecified fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82001P",  # Unspecified fracture of right patella, subsequent encounter for closed fracture with malunion
        "S82001Q",  # Unspecified fracture of right patella, subsequent encounter for open fracture type I or II with malunion
        "S82001R",  # Unspecified fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82001S",  # Unspecified fracture of right patella, sequela
        "S82002A",  # Unspecified fracture of left patella, initial encounter for closed fracture
        "S82002B",  # Unspecified fracture of left patella, initial encounter for open fracture type I or II
        "S82002C",  # Unspecified fracture of left patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82002D",  # Unspecified fracture of left patella, subsequent encounter for closed fracture with routine healing
        "S82002E",  # Unspecified fracture of left patella, subsequent encounter for open fracture type I or II with routine healing
        "S82002F",  # Unspecified fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82002G",  # Unspecified fracture of left patella, subsequent encounter for closed fracture with delayed healing
        "S82002H",  # Unspecified fracture of left patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82002J",  # Unspecified fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82002K",  # Unspecified fracture of left patella, subsequent encounter for closed fracture with nonunion
        "S82002M",  # Unspecified fracture of left patella, subsequent encounter for open fracture type I or II with nonunion
        "S82002N",  # Unspecified fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82002P",  # Unspecified fracture of left patella, subsequent encounter for closed fracture with malunion
        "S82002Q",  # Unspecified fracture of left patella, subsequent encounter for open fracture type I or II with malunion
        "S82002R",  # Unspecified fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82002S",  # Unspecified fracture of left patella, sequela
        "S82009A",  # Unspecified fracture of unspecified patella, initial encounter for closed fracture
        "S82009B",  # Unspecified fracture of unspecified patella, initial encounter for open fracture type I or II
        "S82009C",  # Unspecified fracture of unspecified patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82009D",  # Unspecified fracture of unspecified patella, subsequent encounter for closed fracture with routine healing
        "S82009E",  # Unspecified fracture of unspecified patella, subsequent encounter for open fracture type I or II with routine healing
        "S82009F",  # Unspecified fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82009G",  # Unspecified fracture of unspecified patella, subsequent encounter for closed fracture with delayed healing
        "S82009H",  # Unspecified fracture of unspecified patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82009J",  # Unspecified fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82009K",  # Unspecified fracture of unspecified patella, subsequent encounter for closed fracture with nonunion
        "S82009M",  # Unspecified fracture of unspecified patella, subsequent encounter for open fracture type I or II with nonunion
        "S82009N",  # Unspecified fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82009P",  # Unspecified fracture of unspecified patella, subsequent encounter for closed fracture with malunion
        "S82009Q",  # Unspecified fracture of unspecified patella, subsequent encounter for open fracture type I or II with malunion
        "S82009R",  # Unspecified fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82009S",  # Unspecified fracture of unspecified patella, sequela
        "S82011A",  # Displaced osteochondral fracture of right patella, initial encounter for closed fracture
        "S82011B",  # Displaced osteochondral fracture of right patella, initial encounter for open fracture type I or II
        "S82011C",  # Displaced osteochondral fracture of right patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82011D",  # Displaced osteochondral fracture of right patella, subsequent encounter for closed fracture with routine healing
        "S82011E",  # Displaced osteochondral fracture of right patella, subsequent encounter for open fracture type I or II with routine healing
        "S82011F",  # Displaced osteochondral fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82011G",  # Displaced osteochondral fracture of right patella, subsequent encounter for closed fracture with delayed healing
        "S82011H",  # Displaced osteochondral fracture of right patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82011J",  # Displaced osteochondral fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82011K",  # Displaced osteochondral fracture of right patella, subsequent encounter for closed fracture with nonunion
        "S82011M",  # Displaced osteochondral fracture of right patella, subsequent encounter for open fracture type I or II with nonunion
        "S82011N",  # Displaced osteochondral fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82011P",  # Displaced osteochondral fracture of right patella, subsequent encounter for closed fracture with malunion
        "S82011Q",  # Displaced osteochondral fracture of right patella, subsequent encounter for open fracture type I or II with malunion
        "S82011R",  # Displaced osteochondral fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82011S",  # Displaced osteochondral fracture of right patella, sequela
        "S82012A",  # Displaced osteochondral fracture of left patella, initial encounter for closed fracture
        "S82012B",  # Displaced osteochondral fracture of left patella, initial encounter for open fracture type I or II
        "S82012C",  # Displaced osteochondral fracture of left patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82012D",  # Displaced osteochondral fracture of left patella, subsequent encounter for closed fracture with routine healing
        "S82012E",  # Displaced osteochondral fracture of left patella, subsequent encounter for open fracture type I or II with routine healing
        "S82012F",  # Displaced osteochondral fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82012G",  # Displaced osteochondral fracture of left patella, subsequent encounter for closed fracture with delayed healing
        "S82012H",  # Displaced osteochondral fracture of left patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82012J",  # Displaced osteochondral fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82012K",  # Displaced osteochondral fracture of left patella, subsequent encounter for closed fracture with nonunion
        "S82012M",  # Displaced osteochondral fracture of left patella, subsequent encounter for open fracture type I or II with nonunion
        "S82012N",  # Displaced osteochondral fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82012P",  # Displaced osteochondral fracture of left patella, subsequent encounter for closed fracture with malunion
        "S82012Q",  # Displaced osteochondral fracture of left patella, subsequent encounter for open fracture type I or II with malunion
        "S82012R",  # Displaced osteochondral fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82012S",  # Displaced osteochondral fracture of left patella, sequela
        "S82013A",  # Displaced osteochondral fracture of unspecified patella, initial encounter for closed fracture
        "S82013B",  # Displaced osteochondral fracture of unspecified patella, initial encounter for open fracture type I or II
        "S82013C",  # Displaced osteochondral fracture of unspecified patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82013D",  # Displaced osteochondral fracture of unspecified patella, subsequent encounter for closed fracture with routine healing
        "S82013E",  # Displaced osteochondral fracture of unspecified patella, subsequent encounter for open fracture type I or II with routine healing
        "S82013F",  # Displaced osteochondral fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82013G",  # Displaced osteochondral fracture of unspecified patella, subsequent encounter for closed fracture with delayed healing
        "S82013H",  # Displaced osteochondral fracture of unspecified patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82013J",  # Displaced osteochondral fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82013K",  # Displaced osteochondral fracture of unspecified patella, subsequent encounter for closed fracture with nonunion
        "S82013M",  # Displaced osteochondral fracture of unspecified patella, subsequent encounter for open fracture type I or II with nonunion
        "S82013N",  # Displaced osteochondral fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82013P",  # Displaced osteochondral fracture of unspecified patella, subsequent encounter for closed fracture with malunion
        "S82013Q",  # Displaced osteochondral fracture of unspecified patella, subsequent encounter for open fracture type I or II with malunion
        "S82013R",  # Displaced osteochondral fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82013S",  # Displaced osteochondral fracture of unspecified patella, sequela
        "S82014A",  # Nondisplaced osteochondral fracture of right patella, initial encounter for closed fracture
        "S82014B",  # Nondisplaced osteochondral fracture of right patella, initial encounter for open fracture type I or II
        "S82014C",  # Nondisplaced osteochondral fracture of right patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82014D",  # Nondisplaced osteochondral fracture of right patella, subsequent encounter for closed fracture with routine healing
        "S82014E",  # Nondisplaced osteochondral fracture of right patella, subsequent encounter for open fracture type I or II with routine healing
        "S82014F",  # Nondisplaced osteochondral fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82014G",  # Nondisplaced osteochondral fracture of right patella, subsequent encounter for closed fracture with delayed healing
        "S82014H",  # Nondisplaced osteochondral fracture of right patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82014J",  # Nondisplaced osteochondral fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82014K",  # Nondisplaced osteochondral fracture of right patella, subsequent encounter for closed fracture with nonunion
        "S82014M",  # Nondisplaced osteochondral fracture of right patella, subsequent encounter for open fracture type I or II with nonunion
        "S82014N",  # Nondisplaced osteochondral fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82014P",  # Nondisplaced osteochondral fracture of right patella, subsequent encounter for closed fracture with malunion
        "S82014Q",  # Nondisplaced osteochondral fracture of right patella, subsequent encounter for open fracture type I or II with malunion
        "S82014R",  # Nondisplaced osteochondral fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82014S",  # Nondisplaced osteochondral fracture of right patella, sequela
        "S82015A",  # Nondisplaced osteochondral fracture of left patella, initial encounter for closed fracture
        "S82015B",  # Nondisplaced osteochondral fracture of left patella, initial encounter for open fracture type I or II
        "S82015C",  # Nondisplaced osteochondral fracture of left patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82015D",  # Nondisplaced osteochondral fracture of left patella, subsequent encounter for closed fracture with routine healing
        "S82015E",  # Nondisplaced osteochondral fracture of left patella, subsequent encounter for open fracture type I or II with routine healing
        "S82015F",  # Nondisplaced osteochondral fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82015G",  # Nondisplaced osteochondral fracture of left patella, subsequent encounter for closed fracture with delayed healing
        "S82015H",  # Nondisplaced osteochondral fracture of left patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82015J",  # Nondisplaced osteochondral fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82015K",  # Nondisplaced osteochondral fracture of left patella, subsequent encounter for closed fracture with nonunion
        "S82015M",  # Nondisplaced osteochondral fracture of left patella, subsequent encounter for open fracture type I or II with nonunion
        "S82015N",  # Nondisplaced osteochondral fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82015P",  # Nondisplaced osteochondral fracture of left patella, subsequent encounter for closed fracture with malunion
        "S82015Q",  # Nondisplaced osteochondral fracture of left patella, subsequent encounter for open fracture type I or II with malunion
        "S82015R",  # Nondisplaced osteochondral fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82015S",  # Nondisplaced osteochondral fracture of left patella, sequela
        "S82016A",  # Nondisplaced osteochondral fracture of unspecified patella, initial encounter for closed fracture
        "S82016B",  # Nondisplaced osteochondral fracture of unspecified patella, initial encounter for open fracture type I or II
        "S82016C",  # Nondisplaced osteochondral fracture of unspecified patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82016D",  # Nondisplaced osteochondral fracture of unspecified patella, subsequent encounter for closed fracture with routine healing
        "S82016E",  # Nondisplaced osteochondral fracture of unspecified patella, subsequent encounter for open fracture type I or II with routine healing
        "S82016F",  # Nondisplaced osteochondral fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82016G",  # Nondisplaced osteochondral fracture of unspecified patella, subsequent encounter for closed fracture with delayed healing
        "S82016H",  # Nondisplaced osteochondral fracture of unspecified patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82016J",  # Nondisplaced osteochondral fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82016K",  # Nondisplaced osteochondral fracture of unspecified patella, subsequent encounter for closed fracture with nonunion
        "S82016M",  # Nondisplaced osteochondral fracture of unspecified patella, subsequent encounter for open fracture type I or II with nonunion
        "S82016N",  # Nondisplaced osteochondral fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82016P",  # Nondisplaced osteochondral fracture of unspecified patella, subsequent encounter for closed fracture with malunion
        "S82016Q",  # Nondisplaced osteochondral fracture of unspecified patella, subsequent encounter for open fracture type I or II with malunion
        "S82016R",  # Nondisplaced osteochondral fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82016S",  # Nondisplaced osteochondral fracture of unspecified patella, sequela
        "S82021A",  # Displaced longitudinal fracture of right patella, initial encounter for closed fracture
        "S82021B",  # Displaced longitudinal fracture of right patella, initial encounter for open fracture type I or II
        "S82021C",  # Displaced longitudinal fracture of right patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82021D",  # Displaced longitudinal fracture of right patella, subsequent encounter for closed fracture with routine healing
        "S82021E",  # Displaced longitudinal fracture of right patella, subsequent encounter for open fracture type I or II with routine healing
        "S82021F",  # Displaced longitudinal fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82021G",  # Displaced longitudinal fracture of right patella, subsequent encounter for closed fracture with delayed healing
        "S82021H",  # Displaced longitudinal fracture of right patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82021J",  # Displaced longitudinal fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82021K",  # Displaced longitudinal fracture of right patella, subsequent encounter for closed fracture with nonunion
        "S82021M",  # Displaced longitudinal fracture of right patella, subsequent encounter for open fracture type I or II with nonunion
        "S82021N",  # Displaced longitudinal fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82021P",  # Displaced longitudinal fracture of right patella, subsequent encounter for closed fracture with malunion
        "S82021Q",  # Displaced longitudinal fracture of right patella, subsequent encounter for open fracture type I or II with malunion
        "S82021R",  # Displaced longitudinal fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82021S",  # Displaced longitudinal fracture of right patella, sequela
        "S82022A",  # Displaced longitudinal fracture of left patella, initial encounter for closed fracture
        "S82022B",  # Displaced longitudinal fracture of left patella, initial encounter for open fracture type I or II
        "S82022C",  # Displaced longitudinal fracture of left patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82022D",  # Displaced longitudinal fracture of left patella, subsequent encounter for closed fracture with routine healing
        "S82022E",  # Displaced longitudinal fracture of left patella, subsequent encounter for open fracture type I or II with routine healing
        "S82022F",  # Displaced longitudinal fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82022G",  # Displaced longitudinal fracture of left patella, subsequent encounter for closed fracture with delayed healing
        "S82022H",  # Displaced longitudinal fracture of left patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82022J",  # Displaced longitudinal fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82022K",  # Displaced longitudinal fracture of left patella, subsequent encounter for closed fracture with nonunion
        "S82022M",  # Displaced longitudinal fracture of left patella, subsequent encounter for open fracture type I or II with nonunion
        "S82022N",  # Displaced longitudinal fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82022P",  # Displaced longitudinal fracture of left patella, subsequent encounter for closed fracture with malunion
        "S82022Q",  # Displaced longitudinal fracture of left patella, subsequent encounter for open fracture type I or II with malunion
        "S82022R",  # Displaced longitudinal fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82022S",  # Displaced longitudinal fracture of left patella, sequela
        "S82023A",  # Displaced longitudinal fracture of unspecified patella, initial encounter for closed fracture
        "S82023B",  # Displaced longitudinal fracture of unspecified patella, initial encounter for open fracture type I or II
        "S82023C",  # Displaced longitudinal fracture of unspecified patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82023D",  # Displaced longitudinal fracture of unspecified patella, subsequent encounter for closed fracture with routine healing
        "S82023E",  # Displaced longitudinal fracture of unspecified patella, subsequent encounter for open fracture type I or II with routine healing
        "S82023F",  # Displaced longitudinal fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82023G",  # Displaced longitudinal fracture of unspecified patella, subsequent encounter for closed fracture with delayed healing
        "S82023H",  # Displaced longitudinal fracture of unspecified patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82023J",  # Displaced longitudinal fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82023K",  # Displaced longitudinal fracture of unspecified patella, subsequent encounter for closed fracture with nonunion
        "S82023M",  # Displaced longitudinal fracture of unspecified patella, subsequent encounter for open fracture type I or II with nonunion
        "S82023N",  # Displaced longitudinal fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82023P",  # Displaced longitudinal fracture of unspecified patella, subsequent encounter for closed fracture with malunion
        "S82023Q",  # Displaced longitudinal fracture of unspecified patella, subsequent encounter for open fracture type I or II with malunion
        "S82023R",  # Displaced longitudinal fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82023S",  # Displaced longitudinal fracture of unspecified patella, sequela
        "S82024A",  # Nondisplaced longitudinal fracture of right patella, initial encounter for closed fracture
        "S82024B",  # Nondisplaced longitudinal fracture of right patella, initial encounter for open fracture type I or II
        "S82024C",  # Nondisplaced longitudinal fracture of right patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82024D",  # Nondisplaced longitudinal fracture of right patella, subsequent encounter for closed fracture with routine healing
        "S82024E",  # Nondisplaced longitudinal fracture of right patella, subsequent encounter for open fracture type I or II with routine healing
        "S82024F",  # Nondisplaced longitudinal fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82024G",  # Nondisplaced longitudinal fracture of right patella, subsequent encounter for closed fracture with delayed healing
        "S82024H",  # Nondisplaced longitudinal fracture of right patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82024J",  # Nondisplaced longitudinal fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82024K",  # Nondisplaced longitudinal fracture of right patella, subsequent encounter for closed fracture with nonunion
        "S82024M",  # Nondisplaced longitudinal fracture of right patella, subsequent encounter for open fracture type I or II with nonunion
        "S82024N",  # Nondisplaced longitudinal fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82024P",  # Nondisplaced longitudinal fracture of right patella, subsequent encounter for closed fracture with malunion
        "S82024Q",  # Nondisplaced longitudinal fracture of right patella, subsequent encounter for open fracture type I or II with malunion
        "S82024R",  # Nondisplaced longitudinal fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82024S",  # Nondisplaced longitudinal fracture of right patella, sequela
        "S82025A",  # Nondisplaced longitudinal fracture of left patella, initial encounter for closed fracture
        "S82025B",  # Nondisplaced longitudinal fracture of left patella, initial encounter for open fracture type I or II
        "S82025C",  # Nondisplaced longitudinal fracture of left patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82025D",  # Nondisplaced longitudinal fracture of left patella, subsequent encounter for closed fracture with routine healing
        "S82025E",  # Nondisplaced longitudinal fracture of left patella, subsequent encounter for open fracture type I or II with routine healing
        "S82025F",  # Nondisplaced longitudinal fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82025G",  # Nondisplaced longitudinal fracture of left patella, subsequent encounter for closed fracture with delayed healing
        "S82025H",  # Nondisplaced longitudinal fracture of left patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82025J",  # Nondisplaced longitudinal fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82025K",  # Nondisplaced longitudinal fracture of left patella, subsequent encounter for closed fracture with nonunion
        "S82025M",  # Nondisplaced longitudinal fracture of left patella, subsequent encounter for open fracture type I or II with nonunion
        "S82025N",  # Nondisplaced longitudinal fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82025P",  # Nondisplaced longitudinal fracture of left patella, subsequent encounter for closed fracture with malunion
        "S82025Q",  # Nondisplaced longitudinal fracture of left patella, subsequent encounter for open fracture type I or II with malunion
        "S82025R",  # Nondisplaced longitudinal fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82025S",  # Nondisplaced longitudinal fracture of left patella, sequela
        "S82026A",  # Nondisplaced longitudinal fracture of unspecified patella, initial encounter for closed fracture
        "S82026B",  # Nondisplaced longitudinal fracture of unspecified patella, initial encounter for open fracture type I or II
        "S82026C",  # Nondisplaced longitudinal fracture of unspecified patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82026D",  # Nondisplaced longitudinal fracture of unspecified patella, subsequent encounter for closed fracture with routine healing
        "S82026E",  # Nondisplaced longitudinal fracture of unspecified patella, subsequent encounter for open fracture type I or II with routine healing
        "S82026F",  # Nondisplaced longitudinal fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82026G",  # Nondisplaced longitudinal fracture of unspecified patella, subsequent encounter for closed fracture with delayed healing
        "S82026H",  # Nondisplaced longitudinal fracture of unspecified patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82026J",  # Nondisplaced longitudinal fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82026K",  # Nondisplaced longitudinal fracture of unspecified patella, subsequent encounter for closed fracture with nonunion
        "S82026M",  # Nondisplaced longitudinal fracture of unspecified patella, subsequent encounter for open fracture type I or II with nonunion
        "S82026N",  # Nondisplaced longitudinal fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82026P",  # Nondisplaced longitudinal fracture of unspecified patella, subsequent encounter for closed fracture with malunion
        "S82026Q",  # Nondisplaced longitudinal fracture of unspecified patella, subsequent encounter for open fracture type I or II with malunion
        "S82026R",  # Nondisplaced longitudinal fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82026S",  # Nondisplaced longitudinal fracture of unspecified patella, sequela
        "S82031A",  # Displaced transverse fracture of right patella, initial encounter for closed fracture
        "S82031B",  # Displaced transverse fracture of right patella, initial encounter for open fracture type I or II
        "S82031C",  # Displaced transverse fracture of right patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82031D",  # Displaced transverse fracture of right patella, subsequent encounter for closed fracture with routine healing
        "S82031E",  # Displaced transverse fracture of right patella, subsequent encounter for open fracture type I or II with routine healing
        "S82031F",  # Displaced transverse fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82031G",  # Displaced transverse fracture of right patella, subsequent encounter for closed fracture with delayed healing
        "S82031H",  # Displaced transverse fracture of right patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82031J",  # Displaced transverse fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82031K",  # Displaced transverse fracture of right patella, subsequent encounter for closed fracture with nonunion
        "S82031M",  # Displaced transverse fracture of right patella, subsequent encounter for open fracture type I or II with nonunion
        "S82031N",  # Displaced transverse fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82031P",  # Displaced transverse fracture of right patella, subsequent encounter for closed fracture with malunion
        "S82031Q",  # Displaced transverse fracture of right patella, subsequent encounter for open fracture type I or II with malunion
        "S82031R",  # Displaced transverse fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82031S",  # Displaced transverse fracture of right patella, sequela
        "S82032A",  # Displaced transverse fracture of left patella, initial encounter for closed fracture
        "S82032B",  # Displaced transverse fracture of left patella, initial encounter for open fracture type I or II
        "S82032C",  # Displaced transverse fracture of left patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82032D",  # Displaced transverse fracture of left patella, subsequent encounter for closed fracture with routine healing
        "S82032E",  # Displaced transverse fracture of left patella, subsequent encounter for open fracture type I or II with routine healing
        "S82032F",  # Displaced transverse fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82032G",  # Displaced transverse fracture of left patella, subsequent encounter for closed fracture with delayed healing
        "S82032H",  # Displaced transverse fracture of left patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82032J",  # Displaced transverse fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82032K",  # Displaced transverse fracture of left patella, subsequent encounter for closed fracture with nonunion
        "S82032M",  # Displaced transverse fracture of left patella, subsequent encounter for open fracture type I or II with nonunion
        "S82032N",  # Displaced transverse fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82032P",  # Displaced transverse fracture of left patella, subsequent encounter for closed fracture with malunion
        "S82032Q",  # Displaced transverse fracture of left patella, subsequent encounter for open fracture type I or II with malunion
        "S82032R",  # Displaced transverse fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82032S",  # Displaced transverse fracture of left patella, sequela
        "S82033A",  # Displaced transverse fracture of unspecified patella, initial encounter for closed fracture
        "S82033B",  # Displaced transverse fracture of unspecified patella, initial encounter for open fracture type I or II
        "S82033C",  # Displaced transverse fracture of unspecified patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82033D",  # Displaced transverse fracture of unspecified patella, subsequent encounter for closed fracture with routine healing
        "S82033E",  # Displaced transverse fracture of unspecified patella, subsequent encounter for open fracture type I or II with routine healing
        "S82033F",  # Displaced transverse fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82033G",  # Displaced transverse fracture of unspecified patella, subsequent encounter for closed fracture with delayed healing
        "S82033H",  # Displaced transverse fracture of unspecified patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82033J",  # Displaced transverse fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82033K",  # Displaced transverse fracture of unspecified patella, subsequent encounter for closed fracture with nonunion
        "S82033M",  # Displaced transverse fracture of unspecified patella, subsequent encounter for open fracture type I or II with nonunion
        "S82033N",  # Displaced transverse fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82033P",  # Displaced transverse fracture of unspecified patella, subsequent encounter for closed fracture with malunion
        "S82033Q",  # Displaced transverse fracture of unspecified patella, subsequent encounter for open fracture type I or II with malunion
        "S82033R",  # Displaced transverse fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82033S",  # Displaced transverse fracture of unspecified patella, sequela
        "S82034A",  # Nondisplaced transverse fracture of right patella, initial encounter for closed fracture
        "S82034B",  # Nondisplaced transverse fracture of right patella, initial encounter for open fracture type I or II
        "S82034C",  # Nondisplaced transverse fracture of right patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82034D",  # Nondisplaced transverse fracture of right patella, subsequent encounter for closed fracture with routine healing
        "S82034E",  # Nondisplaced transverse fracture of right patella, subsequent encounter for open fracture type I or II with routine healing
        "S82034F",  # Nondisplaced transverse fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82034G",  # Nondisplaced transverse fracture of right patella, subsequent encounter for closed fracture with delayed healing
        "S82034H",  # Nondisplaced transverse fracture of right patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82034J",  # Nondisplaced transverse fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82034K",  # Nondisplaced transverse fracture of right patella, subsequent encounter for closed fracture with nonunion
        "S82034M",  # Nondisplaced transverse fracture of right patella, subsequent encounter for open fracture type I or II with nonunion
        "S82034N",  # Nondisplaced transverse fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82034P",  # Nondisplaced transverse fracture of right patella, subsequent encounter for closed fracture with malunion
        "S82034Q",  # Nondisplaced transverse fracture of right patella, subsequent encounter for open fracture type I or II with malunion
        "S82034R",  # Nondisplaced transverse fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82034S",  # Nondisplaced transverse fracture of right patella, sequela
        "S82035A",  # Nondisplaced transverse fracture of left patella, initial encounter for closed fracture
        "S82035B",  # Nondisplaced transverse fracture of left patella, initial encounter for open fracture type I or II
        "S82035C",  # Nondisplaced transverse fracture of left patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82035D",  # Nondisplaced transverse fracture of left patella, subsequent encounter for closed fracture with routine healing
        "S82035E",  # Nondisplaced transverse fracture of left patella, subsequent encounter for open fracture type I or II with routine healing
        "S82035F",  # Nondisplaced transverse fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82035G",  # Nondisplaced transverse fracture of left patella, subsequent encounter for closed fracture with delayed healing
        "S82035H",  # Nondisplaced transverse fracture of left patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82035J",  # Nondisplaced transverse fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82035K",  # Nondisplaced transverse fracture of left patella, subsequent encounter for closed fracture with nonunion
        "S82035M",  # Nondisplaced transverse fracture of left patella, subsequent encounter for open fracture type I or II with nonunion
        "S82035N",  # Nondisplaced transverse fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82035P",  # Nondisplaced transverse fracture of left patella, subsequent encounter for closed fracture with malunion
        "S82035Q",  # Nondisplaced transverse fracture of left patella, subsequent encounter for open fracture type I or II with malunion
        "S82035R",  # Nondisplaced transverse fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82035S",  # Nondisplaced transverse fracture of left patella, sequela
        "S82036A",  # Nondisplaced transverse fracture of unspecified patella, initial encounter for closed fracture
        "S82036B",  # Nondisplaced transverse fracture of unspecified patella, initial encounter for open fracture type I or II
        "S82036C",  # Nondisplaced transverse fracture of unspecified patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82036D",  # Nondisplaced transverse fracture of unspecified patella, subsequent encounter for closed fracture with routine healing
        "S82036E",  # Nondisplaced transverse fracture of unspecified patella, subsequent encounter for open fracture type I or II with routine healing
        "S82036F",  # Nondisplaced transverse fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82036G",  # Nondisplaced transverse fracture of unspecified patella, subsequent encounter for closed fracture with delayed healing
        "S82036H",  # Nondisplaced transverse fracture of unspecified patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82036J",  # Nondisplaced transverse fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82036K",  # Nondisplaced transverse fracture of unspecified patella, subsequent encounter for closed fracture with nonunion
        "S82036M",  # Nondisplaced transverse fracture of unspecified patella, subsequent encounter for open fracture type I or II with nonunion
        "S82036N",  # Nondisplaced transverse fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82036P",  # Nondisplaced transverse fracture of unspecified patella, subsequent encounter for closed fracture with malunion
        "S82036Q",  # Nondisplaced transverse fracture of unspecified patella, subsequent encounter for open fracture type I or II with malunion
        "S82036R",  # Nondisplaced transverse fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82036S",  # Nondisplaced transverse fracture of unspecified patella, sequela
        "S82041A",  # Displaced comminuted fracture of right patella, initial encounter for closed fracture
        "S82041B",  # Displaced comminuted fracture of right patella, initial encounter for open fracture type I or II
        "S82041C",  # Displaced comminuted fracture of right patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82041D",  # Displaced comminuted fracture of right patella, subsequent encounter for closed fracture with routine healing
        "S82041E",  # Displaced comminuted fracture of right patella, subsequent encounter for open fracture type I or II with routine healing
        "S82041F",  # Displaced comminuted fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82041G",  # Displaced comminuted fracture of right patella, subsequent encounter for closed fracture with delayed healing
        "S82041H",  # Displaced comminuted fracture of right patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82041J",  # Displaced comminuted fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82041K",  # Displaced comminuted fracture of right patella, subsequent encounter for closed fracture with nonunion
        "S82041M",  # Displaced comminuted fracture of right patella, subsequent encounter for open fracture type I or II with nonunion
        "S82041N",  # Displaced comminuted fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82041P",  # Displaced comminuted fracture of right patella, subsequent encounter for closed fracture with malunion
        "S82041Q",  # Displaced comminuted fracture of right patella, subsequent encounter for open fracture type I or II with malunion
        "S82041R",  # Displaced comminuted fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82041S",  # Displaced comminuted fracture of right patella, sequela
        "S82042A",  # Displaced comminuted fracture of left patella, initial encounter for closed fracture
        "S82042B",  # Displaced comminuted fracture of left patella, initial encounter for open fracture type I or II
        "S82042C",  # Displaced comminuted fracture of left patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82042D",  # Displaced comminuted fracture of left patella, subsequent encounter for closed fracture with routine healing
        "S82042E",  # Displaced comminuted fracture of left patella, subsequent encounter for open fracture type I or II with routine healing
        "S82042F",  # Displaced comminuted fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82042G",  # Displaced comminuted fracture of left patella, subsequent encounter for closed fracture with delayed healing
        "S82042H",  # Displaced comminuted fracture of left patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82042J",  # Displaced comminuted fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82042K",  # Displaced comminuted fracture of left patella, subsequent encounter for closed fracture with nonunion
        "S82042M",  # Displaced comminuted fracture of left patella, subsequent encounter for open fracture type I or II with nonunion
        "S82042N",  # Displaced comminuted fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82042P",  # Displaced comminuted fracture of left patella, subsequent encounter for closed fracture with malunion
        "S82042Q",  # Displaced comminuted fracture of left patella, subsequent encounter for open fracture type I or II with malunion
        "S82042R",  # Displaced comminuted fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82042S",  # Displaced comminuted fracture of left patella, sequela
        "S82043A",  # Displaced comminuted fracture of unspecified patella, initial encounter for closed fracture
        "S82043B",  # Displaced comminuted fracture of unspecified patella, initial encounter for open fracture type I or II
        "S82043C",  # Displaced comminuted fracture of unspecified patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82043D",  # Displaced comminuted fracture of unspecified patella, subsequent encounter for closed fracture with routine healing
        "S82043E",  # Displaced comminuted fracture of unspecified patella, subsequent encounter for open fracture type I or II with routine healing
        "S82043F",  # Displaced comminuted fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82043G",  # Displaced comminuted fracture of unspecified patella, subsequent encounter for closed fracture with delayed healing
        "S82043H",  # Displaced comminuted fracture of unspecified patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82043J",  # Displaced comminuted fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82043K",  # Displaced comminuted fracture of unspecified patella, subsequent encounter for closed fracture with nonunion
        "S82043M",  # Displaced comminuted fracture of unspecified patella, subsequent encounter for open fracture type I or II with nonunion
        "S82043N",  # Displaced comminuted fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82043P",  # Displaced comminuted fracture of unspecified patella, subsequent encounter for closed fracture with malunion
        "S82043Q",  # Displaced comminuted fracture of unspecified patella, subsequent encounter for open fracture type I or II with malunion
        "S82043R",  # Displaced comminuted fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82043S",  # Displaced comminuted fracture of unspecified patella, sequela
        "S82044A",  # Nondisplaced comminuted fracture of right patella, initial encounter for closed fracture
        "S82044B",  # Nondisplaced comminuted fracture of right patella, initial encounter for open fracture type I or II
        "S82044C",  # Nondisplaced comminuted fracture of right patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82044D",  # Nondisplaced comminuted fracture of right patella, subsequent encounter for closed fracture with routine healing
        "S82044E",  # Nondisplaced comminuted fracture of right patella, subsequent encounter for open fracture type I or II with routine healing
        "S82044F",  # Nondisplaced comminuted fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82044G",  # Nondisplaced comminuted fracture of right patella, subsequent encounter for closed fracture with delayed healing
        "S82044H",  # Nondisplaced comminuted fracture of right patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82044J",  # Nondisplaced comminuted fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82044K",  # Nondisplaced comminuted fracture of right patella, subsequent encounter for closed fracture with nonunion
        "S82044M",  # Nondisplaced comminuted fracture of right patella, subsequent encounter for open fracture type I or II with nonunion
        "S82044N",  # Nondisplaced comminuted fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82044P",  # Nondisplaced comminuted fracture of right patella, subsequent encounter for closed fracture with malunion
        "S82044Q",  # Nondisplaced comminuted fracture of right patella, subsequent encounter for open fracture type I or II with malunion
        "S82044R",  # Nondisplaced comminuted fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82044S",  # Nondisplaced comminuted fracture of right patella, sequela
        "S82045A",  # Nondisplaced comminuted fracture of left patella, initial encounter for closed fracture
        "S82045B",  # Nondisplaced comminuted fracture of left patella, initial encounter for open fracture type I or II
        "S82045C",  # Nondisplaced comminuted fracture of left patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82045D",  # Nondisplaced comminuted fracture of left patella, subsequent encounter for closed fracture with routine healing
        "S82045E",  # Nondisplaced comminuted fracture of left patella, subsequent encounter for open fracture type I or II with routine healing
        "S82045F",  # Nondisplaced comminuted fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82045G",  # Nondisplaced comminuted fracture of left patella, subsequent encounter for closed fracture with delayed healing
        "S82045H",  # Nondisplaced comminuted fracture of left patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82045J",  # Nondisplaced comminuted fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82045K",  # Nondisplaced comminuted fracture of left patella, subsequent encounter for closed fracture with nonunion
        "S82045M",  # Nondisplaced comminuted fracture of left patella, subsequent encounter for open fracture type I or II with nonunion
        "S82045N",  # Nondisplaced comminuted fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82045P",  # Nondisplaced comminuted fracture of left patella, subsequent encounter for closed fracture with malunion
        "S82045Q",  # Nondisplaced comminuted fracture of left patella, subsequent encounter for open fracture type I or II with malunion
        "S82045R",  # Nondisplaced comminuted fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82045S",  # Nondisplaced comminuted fracture of left patella, sequela
        "S82046A",  # Nondisplaced comminuted fracture of unspecified patella, initial encounter for closed fracture
        "S82046B",  # Nondisplaced comminuted fracture of unspecified patella, initial encounter for open fracture type I or II
        "S82046C",  # Nondisplaced comminuted fracture of unspecified patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82046D",  # Nondisplaced comminuted fracture of unspecified patella, subsequent encounter for closed fracture with routine healing
        "S82046E",  # Nondisplaced comminuted fracture of unspecified patella, subsequent encounter for open fracture type I or II with routine healing
        "S82046F",  # Nondisplaced comminuted fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82046G",  # Nondisplaced comminuted fracture of unspecified patella, subsequent encounter for closed fracture with delayed healing
        "S82046H",  # Nondisplaced comminuted fracture of unspecified patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82046J",  # Nondisplaced comminuted fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82046K",  # Nondisplaced comminuted fracture of unspecified patella, subsequent encounter for closed fracture with nonunion
        "S82046M",  # Nondisplaced comminuted fracture of unspecified patella, subsequent encounter for open fracture type I or II with nonunion
        "S82046N",  # Nondisplaced comminuted fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82046P",  # Nondisplaced comminuted fracture of unspecified patella, subsequent encounter for closed fracture with malunion
        "S82046Q",  # Nondisplaced comminuted fracture of unspecified patella, subsequent encounter for open fracture type I or II with malunion
        "S82046R",  # Nondisplaced comminuted fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82046S",  # Nondisplaced comminuted fracture of unspecified patella, sequela
        "S82091A",  # Other fracture of right patella, initial encounter for closed fracture
        "S82091B",  # Other fracture of right patella, initial encounter for open fracture type I or II
        "S82091C",  # Other fracture of right patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82091D",  # Other fracture of right patella, subsequent encounter for closed fracture with routine healing
        "S82091E",  # Other fracture of right patella, subsequent encounter for open fracture type I or II with routine healing
        "S82091F",  # Other fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82091G",  # Other fracture of right patella, subsequent encounter for closed fracture with delayed healing
        "S82091H",  # Other fracture of right patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82091J",  # Other fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82091K",  # Other fracture of right patella, subsequent encounter for closed fracture with nonunion
        "S82091M",  # Other fracture of right patella, subsequent encounter for open fracture type I or II with nonunion
        "S82091N",  # Other fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82091P",  # Other fracture of right patella, subsequent encounter for closed fracture with malunion
        "S82091Q",  # Other fracture of right patella, subsequent encounter for open fracture type I or II with malunion
        "S82091R",  # Other fracture of right patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82091S",  # Other fracture of right patella, sequela
        "S82092A",  # Other fracture of left patella, initial encounter for closed fracture
        "S82092B",  # Other fracture of left patella, initial encounter for open fracture type I or II
        "S82092C",  # Other fracture of left patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82092D",  # Other fracture of left patella, subsequent encounter for closed fracture with routine healing
        "S82092E",  # Other fracture of left patella, subsequent encounter for open fracture type I or II with routine healing
        "S82092F",  # Other fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82092G",  # Other fracture of left patella, subsequent encounter for closed fracture with delayed healing
        "S82092H",  # Other fracture of left patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82092J",  # Other fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82092K",  # Other fracture of left patella, subsequent encounter for closed fracture with nonunion
        "S82092M",  # Other fracture of left patella, subsequent encounter for open fracture type I or II with nonunion
        "S82092N",  # Other fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82092P",  # Other fracture of left patella, subsequent encounter for closed fracture with malunion
        "S82092Q",  # Other fracture of left patella, subsequent encounter for open fracture type I or II with malunion
        "S82092R",  # Other fracture of left patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82092S",  # Other fracture of left patella, sequela
        "S82099A",  # Other fracture of unspecified patella, initial encounter for closed fracture
        "S82099B",  # Other fracture of unspecified patella, initial encounter for open fracture type I or II
        "S82099C",  # Other fracture of unspecified patella, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82099D",  # Other fracture of unspecified patella, subsequent encounter for closed fracture with routine healing
        "S82099E",  # Other fracture of unspecified patella, subsequent encounter for open fracture type I or II with routine healing
        "S82099F",  # Other fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82099G",  # Other fracture of unspecified patella, subsequent encounter for closed fracture with delayed healing
        "S82099H",  # Other fracture of unspecified patella, subsequent encounter for open fracture type I or II with delayed healing
        "S82099J",  # Other fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82099K",  # Other fracture of unspecified patella, subsequent encounter for closed fracture with nonunion
        "S82099M",  # Other fracture of unspecified patella, subsequent encounter for open fracture type I or II with nonunion
        "S82099N",  # Other fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82099P",  # Other fracture of unspecified patella, subsequent encounter for closed fracture with malunion
        "S82099Q",  # Other fracture of unspecified patella, subsequent encounter for open fracture type I or II with malunion
        "S82099R",  # Other fracture of unspecified patella, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82099S",  # Other fracture of unspecified patella, sequela
        "S82101A",  # Unspecified fracture of upper end of right tibia, initial encounter for closed fracture
        "S82101B",  # Unspecified fracture of upper end of right tibia, initial encounter for open fracture type I or II
        "S82101C",  # Unspecified fracture of upper end of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82101D",  # Unspecified fracture of upper end of right tibia, subsequent encounter for closed fracture with routine healing
        "S82101E",  # Unspecified fracture of upper end of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82101F",  # Unspecified fracture of upper end of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82101G",  # Unspecified fracture of upper end of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82101H",  # Unspecified fracture of upper end of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82101J",  # Unspecified fracture of upper end of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82101K",  # Unspecified fracture of upper end of right tibia, subsequent encounter for closed fracture with nonunion
        "S82101M",  # Unspecified fracture of upper end of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82101N",  # Unspecified fracture of upper end of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82101P",  # Unspecified fracture of upper end of right tibia, subsequent encounter for closed fracture with malunion
        "S82101Q",  # Unspecified fracture of upper end of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82101R",  # Unspecified fracture of upper end of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82101S",  # Unspecified fracture of upper end of right tibia, sequela
        "S82102A",  # Unspecified fracture of upper end of left tibia, initial encounter for closed fracture
        "S82102B",  # Unspecified fracture of upper end of left tibia, initial encounter for open fracture type I or II
        "S82102C",  # Unspecified fracture of upper end of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82102D",  # Unspecified fracture of upper end of left tibia, subsequent encounter for closed fracture with routine healing
        "S82102E",  # Unspecified fracture of upper end of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82102F",  # Unspecified fracture of upper end of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82102G",  # Unspecified fracture of upper end of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82102H",  # Unspecified fracture of upper end of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82102J",  # Unspecified fracture of upper end of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82102K",  # Unspecified fracture of upper end of left tibia, subsequent encounter for closed fracture with nonunion
        "S82102M",  # Unspecified fracture of upper end of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82102N",  # Unspecified fracture of upper end of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82102P",  # Unspecified fracture of upper end of left tibia, subsequent encounter for closed fracture with malunion
        "S82102Q",  # Unspecified fracture of upper end of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82102R",  # Unspecified fracture of upper end of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82102S",  # Unspecified fracture of upper end of left tibia, sequela
        "S82109A",  # Unspecified fracture of upper end of unspecified tibia, initial encounter for closed fracture
        "S82109B",  # Unspecified fracture of upper end of unspecified tibia, initial encounter for open fracture type I or II
        "S82109C",  # Unspecified fracture of upper end of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82109D",  # Unspecified fracture of upper end of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82109E",  # Unspecified fracture of upper end of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82109F",  # Unspecified fracture of upper end of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82109G",  # Unspecified fracture of upper end of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82109H",  # Unspecified fracture of upper end of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82109J",  # Unspecified fracture of upper end of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82109K",  # Unspecified fracture of upper end of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82109M",  # Unspecified fracture of upper end of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82109N",  # Unspecified fracture of upper end of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82109P",  # Unspecified fracture of upper end of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82109Q",  # Unspecified fracture of upper end of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82109R",  # Unspecified fracture of upper end of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82109S",  # Unspecified fracture of upper end of unspecified tibia, sequela
        "S82111A",  # Displaced fracture of right tibial spine, initial encounter for closed fracture
        "S82111B",  # Displaced fracture of right tibial spine, initial encounter for open fracture type I or II
        "S82111C",  # Displaced fracture of right tibial spine, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82111D",  # Displaced fracture of right tibial spine, subsequent encounter for closed fracture with routine healing
        "S82111E",  # Displaced fracture of right tibial spine, subsequent encounter for open fracture type I or II with routine healing
        "S82111F",  # Displaced fracture of right tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82111G",  # Displaced fracture of right tibial spine, subsequent encounter for closed fracture with delayed healing
        "S82111H",  # Displaced fracture of right tibial spine, subsequent encounter for open fracture type I or II with delayed healing
        "S82111J",  # Displaced fracture of right tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82111K",  # Displaced fracture of right tibial spine, subsequent encounter for closed fracture with nonunion
        "S82111M",  # Displaced fracture of right tibial spine, subsequent encounter for open fracture type I or II with nonunion
        "S82111N",  # Displaced fracture of right tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82111P",  # Displaced fracture of right tibial spine, subsequent encounter for closed fracture with malunion
        "S82111Q",  # Displaced fracture of right tibial spine, subsequent encounter for open fracture type I or II with malunion
        "S82111R",  # Displaced fracture of right tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82111S",  # Displaced fracture of right tibial spine, sequela
        "S82112A",  # Displaced fracture of left tibial spine, initial encounter for closed fracture
        "S82112B",  # Displaced fracture of left tibial spine, initial encounter for open fracture type I or II
        "S82112C",  # Displaced fracture of left tibial spine, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82112D",  # Displaced fracture of left tibial spine, subsequent encounter for closed fracture with routine healing
        "S82112E",  # Displaced fracture of left tibial spine, subsequent encounter for open fracture type I or II with routine healing
        "S82112F",  # Displaced fracture of left tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82112G",  # Displaced fracture of left tibial spine, subsequent encounter for closed fracture with delayed healing
        "S82112H",  # Displaced fracture of left tibial spine, subsequent encounter for open fracture type I or II with delayed healing
        "S82112J",  # Displaced fracture of left tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82112K",  # Displaced fracture of left tibial spine, subsequent encounter for closed fracture with nonunion
        "S82112M",  # Displaced fracture of left tibial spine, subsequent encounter for open fracture type I or II with nonunion
        "S82112N",  # Displaced fracture of left tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82112P",  # Displaced fracture of left tibial spine, subsequent encounter for closed fracture with malunion
        "S82112Q",  # Displaced fracture of left tibial spine, subsequent encounter for open fracture type I or II with malunion
        "S82112R",  # Displaced fracture of left tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82112S",  # Displaced fracture of left tibial spine, sequela
        "S82113A",  # Displaced fracture of unspecified tibial spine, initial encounter for closed fracture
        "S82113B",  # Displaced fracture of unspecified tibial spine, initial encounter for open fracture type I or II
        "S82113C",  # Displaced fracture of unspecified tibial spine, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82113D",  # Displaced fracture of unspecified tibial spine, subsequent encounter for closed fracture with routine healing
        "S82113E",  # Displaced fracture of unspecified tibial spine, subsequent encounter for open fracture type I or II with routine healing
        "S82113F",  # Displaced fracture of unspecified tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82113G",  # Displaced fracture of unspecified tibial spine, subsequent encounter for closed fracture with delayed healing
        "S82113H",  # Displaced fracture of unspecified tibial spine, subsequent encounter for open fracture type I or II with delayed healing
        "S82113J",  # Displaced fracture of unspecified tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82113K",  # Displaced fracture of unspecified tibial spine, subsequent encounter for closed fracture with nonunion
        "S82113M",  # Displaced fracture of unspecified tibial spine, subsequent encounter for open fracture type I or II with nonunion
        "S82113N",  # Displaced fracture of unspecified tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82113P",  # Displaced fracture of unspecified tibial spine, subsequent encounter for closed fracture with malunion
        "S82113Q",  # Displaced fracture of unspecified tibial spine, subsequent encounter for open fracture type I or II with malunion
        "S82113R",  # Displaced fracture of unspecified tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82113S",  # Displaced fracture of unspecified tibial spine, sequela
        "S82114A",  # Nondisplaced fracture of right tibial spine, initial encounter for closed fracture
        "S82114B",  # Nondisplaced fracture of right tibial spine, initial encounter for open fracture type I or II
        "S82114C",  # Nondisplaced fracture of right tibial spine, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82114D",  # Nondisplaced fracture of right tibial spine, subsequent encounter for closed fracture with routine healing
        "S82114E",  # Nondisplaced fracture of right tibial spine, subsequent encounter for open fracture type I or II with routine healing
        "S82114F",  # Nondisplaced fracture of right tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82114G",  # Nondisplaced fracture of right tibial spine, subsequent encounter for closed fracture with delayed healing
        "S82114H",  # Nondisplaced fracture of right tibial spine, subsequent encounter for open fracture type I or II with delayed healing
        "S82114J",  # Nondisplaced fracture of right tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82114K",  # Nondisplaced fracture of right tibial spine, subsequent encounter for closed fracture with nonunion
        "S82114M",  # Nondisplaced fracture of right tibial spine, subsequent encounter for open fracture type I or II with nonunion
        "S82114N",  # Nondisplaced fracture of right tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82114P",  # Nondisplaced fracture of right tibial spine, subsequent encounter for closed fracture with malunion
        "S82114Q",  # Nondisplaced fracture of right tibial spine, subsequent encounter for open fracture type I or II with malunion
        "S82114R",  # Nondisplaced fracture of right tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82114S",  # Nondisplaced fracture of right tibial spine, sequela
        "S82115A",  # Nondisplaced fracture of left tibial spine, initial encounter for closed fracture
        "S82115B",  # Nondisplaced fracture of left tibial spine, initial encounter for open fracture type I or II
        "S82115C",  # Nondisplaced fracture of left tibial spine, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82115D",  # Nondisplaced fracture of left tibial spine, subsequent encounter for closed fracture with routine healing
        "S82115E",  # Nondisplaced fracture of left tibial spine, subsequent encounter for open fracture type I or II with routine healing
        "S82115F",  # Nondisplaced fracture of left tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82115G",  # Nondisplaced fracture of left tibial spine, subsequent encounter for closed fracture with delayed healing
        "S82115H",  # Nondisplaced fracture of left tibial spine, subsequent encounter for open fracture type I or II with delayed healing
        "S82115J",  # Nondisplaced fracture of left tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82115K",  # Nondisplaced fracture of left tibial spine, subsequent encounter for closed fracture with nonunion
        "S82115M",  # Nondisplaced fracture of left tibial spine, subsequent encounter for open fracture type I or II with nonunion
        "S82115N",  # Nondisplaced fracture of left tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82115P",  # Nondisplaced fracture of left tibial spine, subsequent encounter for closed fracture with malunion
        "S82115Q",  # Nondisplaced fracture of left tibial spine, subsequent encounter for open fracture type I or II with malunion
        "S82115R",  # Nondisplaced fracture of left tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82115S",  # Nondisplaced fracture of left tibial spine, sequela
        "S82116A",  # Nondisplaced fracture of unspecified tibial spine, initial encounter for closed fracture
        "S82116B",  # Nondisplaced fracture of unspecified tibial spine, initial encounter for open fracture type I or II
        "S82116C",  # Nondisplaced fracture of unspecified tibial spine, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82116D",  # Nondisplaced fracture of unspecified tibial spine, subsequent encounter for closed fracture with routine healing
        "S82116E",  # Nondisplaced fracture of unspecified tibial spine, subsequent encounter for open fracture type I or II with routine healing
        "S82116F",  # Nondisplaced fracture of unspecified tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82116G",  # Nondisplaced fracture of unspecified tibial spine, subsequent encounter for closed fracture with delayed healing
        "S82116H",  # Nondisplaced fracture of unspecified tibial spine, subsequent encounter for open fracture type I or II with delayed healing
        "S82116J",  # Nondisplaced fracture of unspecified tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82116K",  # Nondisplaced fracture of unspecified tibial spine, subsequent encounter for closed fracture with nonunion
        "S82116M",  # Nondisplaced fracture of unspecified tibial spine, subsequent encounter for open fracture type I or II with nonunion
        "S82116N",  # Nondisplaced fracture of unspecified tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82116P",  # Nondisplaced fracture of unspecified tibial spine, subsequent encounter for closed fracture with malunion
        "S82116Q",  # Nondisplaced fracture of unspecified tibial spine, subsequent encounter for open fracture type I or II with malunion
        "S82116R",  # Nondisplaced fracture of unspecified tibial spine, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82116S",  # Nondisplaced fracture of unspecified tibial spine, sequela
        "S82121A",  # Displaced fracture of lateral condyle of right tibia, initial encounter for closed fracture
        "S82121B",  # Displaced fracture of lateral condyle of right tibia, initial encounter for open fracture type I or II
        "S82121C",  # Displaced fracture of lateral condyle of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82121D",  # Displaced fracture of lateral condyle of right tibia, subsequent encounter for closed fracture with routine healing
        "S82121E",  # Displaced fracture of lateral condyle of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82121F",  # Displaced fracture of lateral condyle of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82121G",  # Displaced fracture of lateral condyle of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82121H",  # Displaced fracture of lateral condyle of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82121J",  # Displaced fracture of lateral condyle of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82121K",  # Displaced fracture of lateral condyle of right tibia, subsequent encounter for closed fracture with nonunion
        "S82121M",  # Displaced fracture of lateral condyle of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82121N",  # Displaced fracture of lateral condyle of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82121P",  # Displaced fracture of lateral condyle of right tibia, subsequent encounter for closed fracture with malunion
        "S82121Q",  # Displaced fracture of lateral condyle of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82121R",  # Displaced fracture of lateral condyle of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82121S",  # Displaced fracture of lateral condyle of right tibia, sequela
        "S82122A",  # Displaced fracture of lateral condyle of left tibia, initial encounter for closed fracture
        "S82122B",  # Displaced fracture of lateral condyle of left tibia, initial encounter for open fracture type I or II
        "S82122C",  # Displaced fracture of lateral condyle of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82122D",  # Displaced fracture of lateral condyle of left tibia, subsequent encounter for closed fracture with routine healing
        "S82122E",  # Displaced fracture of lateral condyle of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82122F",  # Displaced fracture of lateral condyle of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82122G",  # Displaced fracture of lateral condyle of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82122H",  # Displaced fracture of lateral condyle of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82122J",  # Displaced fracture of lateral condyle of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82122K",  # Displaced fracture of lateral condyle of left tibia, subsequent encounter for closed fracture with nonunion
        "S82122M",  # Displaced fracture of lateral condyle of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82122N",  # Displaced fracture of lateral condyle of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82122P",  # Displaced fracture of lateral condyle of left tibia, subsequent encounter for closed fracture with malunion
        "S82122Q",  # Displaced fracture of lateral condyle of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82122R",  # Displaced fracture of lateral condyle of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82122S",  # Displaced fracture of lateral condyle of left tibia, sequela
        "S82123A",  # Displaced fracture of lateral condyle of unspecified tibia, initial encounter for closed fracture
        "S82123B",  # Displaced fracture of lateral condyle of unspecified tibia, initial encounter for open fracture type I or II
        "S82123C",  # Displaced fracture of lateral condyle of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82123D",  # Displaced fracture of lateral condyle of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82123E",  # Displaced fracture of lateral condyle of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82123F",  # Displaced fracture of lateral condyle of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82123G",  # Displaced fracture of lateral condyle of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82123H",  # Displaced fracture of lateral condyle of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82123J",  # Displaced fracture of lateral condyle of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82123K",  # Displaced fracture of lateral condyle of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82123M",  # Displaced fracture of lateral condyle of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82123N",  # Displaced fracture of lateral condyle of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82123P",  # Displaced fracture of lateral condyle of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82123Q",  # Displaced fracture of lateral condyle of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82123R",  # Displaced fracture of lateral condyle of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82123S",  # Displaced fracture of lateral condyle of unspecified tibia, sequela
        "S82124A",  # Nondisplaced fracture of lateral condyle of right tibia, initial encounter for closed fracture
        "S82124B",  # Nondisplaced fracture of lateral condyle of right tibia, initial encounter for open fracture type I or II
        "S82124C",  # Nondisplaced fracture of lateral condyle of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82124D",  # Nondisplaced fracture of lateral condyle of right tibia, subsequent encounter for closed fracture with routine healing
        "S82124E",  # Nondisplaced fracture of lateral condyle of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82124F",  # Nondisplaced fracture of lateral condyle of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82124G",  # Nondisplaced fracture of lateral condyle of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82124H",  # Nondisplaced fracture of lateral condyle of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82124J",  # Nondisplaced fracture of lateral condyle of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82124K",  # Nondisplaced fracture of lateral condyle of right tibia, subsequent encounter for closed fracture with nonunion
        "S82124M",  # Nondisplaced fracture of lateral condyle of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82124N",  # Nondisplaced fracture of lateral condyle of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82124P",  # Nondisplaced fracture of lateral condyle of right tibia, subsequent encounter for closed fracture with malunion
        "S82124Q",  # Nondisplaced fracture of lateral condyle of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82124R",  # Nondisplaced fracture of lateral condyle of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82124S",  # Nondisplaced fracture of lateral condyle of right tibia, sequela
        "S82125A",  # Nondisplaced fracture of lateral condyle of left tibia, initial encounter for closed fracture
        "S82125B",  # Nondisplaced fracture of lateral condyle of left tibia, initial encounter for open fracture type I or II
        "S82125C",  # Nondisplaced fracture of lateral condyle of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82125D",  # Nondisplaced fracture of lateral condyle of left tibia, subsequent encounter for closed fracture with routine healing
        "S82125E",  # Nondisplaced fracture of lateral condyle of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82125F",  # Nondisplaced fracture of lateral condyle of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82125G",  # Nondisplaced fracture of lateral condyle of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82125H",  # Nondisplaced fracture of lateral condyle of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82125J",  # Nondisplaced fracture of lateral condyle of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82125K",  # Nondisplaced fracture of lateral condyle of left tibia, subsequent encounter for closed fracture with nonunion
        "S82125M",  # Nondisplaced fracture of lateral condyle of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82125N",  # Nondisplaced fracture of lateral condyle of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82125P",  # Nondisplaced fracture of lateral condyle of left tibia, subsequent encounter for closed fracture with malunion
        "S82125Q",  # Nondisplaced fracture of lateral condyle of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82125R",  # Nondisplaced fracture of lateral condyle of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82125S",  # Nondisplaced fracture of lateral condyle of left tibia, sequela
        "S82126A",  # Nondisplaced fracture of lateral condyle of unspecified tibia, initial encounter for closed fracture
        "S82126B",  # Nondisplaced fracture of lateral condyle of unspecified tibia, initial encounter for open fracture type I or II
        "S82126C",  # Nondisplaced fracture of lateral condyle of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82126D",  # Nondisplaced fracture of lateral condyle of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82126E",  # Nondisplaced fracture of lateral condyle of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82126F",  # Nondisplaced fracture of lateral condyle of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82126G",  # Nondisplaced fracture of lateral condyle of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82126H",  # Nondisplaced fracture of lateral condyle of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82126J",  # Nondisplaced fracture of lateral condyle of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82126K",  # Nondisplaced fracture of lateral condyle of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82126M",  # Nondisplaced fracture of lateral condyle of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82126N",  # Nondisplaced fracture of lateral condyle of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82126P",  # Nondisplaced fracture of lateral condyle of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82126Q",  # Nondisplaced fracture of lateral condyle of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82126R",  # Nondisplaced fracture of lateral condyle of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82126S",  # Nondisplaced fracture of lateral condyle of unspecified tibia, sequela
        "S82131A",  # Displaced fracture of medial condyle of right tibia, initial encounter for closed fracture
        "S82131B",  # Displaced fracture of medial condyle of right tibia, initial encounter for open fracture type I or II
        "S82131C",  # Displaced fracture of medial condyle of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82131D",  # Displaced fracture of medial condyle of right tibia, subsequent encounter for closed fracture with routine healing
        "S82131E",  # Displaced fracture of medial condyle of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82131F",  # Displaced fracture of medial condyle of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82131G",  # Displaced fracture of medial condyle of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82131H",  # Displaced fracture of medial condyle of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82131J",  # Displaced fracture of medial condyle of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82131K",  # Displaced fracture of medial condyle of right tibia, subsequent encounter for closed fracture with nonunion
        "S82131M",  # Displaced fracture of medial condyle of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82131N",  # Displaced fracture of medial condyle of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82131P",  # Displaced fracture of medial condyle of right tibia, subsequent encounter for closed fracture with malunion
        "S82131Q",  # Displaced fracture of medial condyle of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82131R",  # Displaced fracture of medial condyle of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82131S",  # Displaced fracture of medial condyle of right tibia, sequela
        "S82132A",  # Displaced fracture of medial condyle of left tibia, initial encounter for closed fracture
        "S82132B",  # Displaced fracture of medial condyle of left tibia, initial encounter for open fracture type I or II
        "S82132C",  # Displaced fracture of medial condyle of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82132D",  # Displaced fracture of medial condyle of left tibia, subsequent encounter for closed fracture with routine healing
        "S82132E",  # Displaced fracture of medial condyle of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82132F",  # Displaced fracture of medial condyle of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82132G",  # Displaced fracture of medial condyle of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82132H",  # Displaced fracture of medial condyle of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82132J",  # Displaced fracture of medial condyle of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82132K",  # Displaced fracture of medial condyle of left tibia, subsequent encounter for closed fracture with nonunion
        "S82132M",  # Displaced fracture of medial condyle of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82132N",  # Displaced fracture of medial condyle of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82132P",  # Displaced fracture of medial condyle of left tibia, subsequent encounter for closed fracture with malunion
        "S82132Q",  # Displaced fracture of medial condyle of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82132R",  # Displaced fracture of medial condyle of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82132S",  # Displaced fracture of medial condyle of left tibia, sequela
        "S82133A",  # Displaced fracture of medial condyle of unspecified tibia, initial encounter for closed fracture
        "S82133B",  # Displaced fracture of medial condyle of unspecified tibia, initial encounter for open fracture type I or II
        "S82133C",  # Displaced fracture of medial condyle of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82133D",  # Displaced fracture of medial condyle of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82133E",  # Displaced fracture of medial condyle of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82133F",  # Displaced fracture of medial condyle of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82133G",  # Displaced fracture of medial condyle of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82133H",  # Displaced fracture of medial condyle of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82133J",  # Displaced fracture of medial condyle of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82133K",  # Displaced fracture of medial condyle of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82133M",  # Displaced fracture of medial condyle of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82133N",  # Displaced fracture of medial condyle of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82133P",  # Displaced fracture of medial condyle of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82133Q",  # Displaced fracture of medial condyle of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82133R",  # Displaced fracture of medial condyle of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82133S",  # Displaced fracture of medial condyle of unspecified tibia, sequela
        "S82134A",  # Nondisplaced fracture of medial condyle of right tibia, initial encounter for closed fracture
        "S82134B",  # Nondisplaced fracture of medial condyle of right tibia, initial encounter for open fracture type I or II
        "S82134C",  # Nondisplaced fracture of medial condyle of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82134D",  # Nondisplaced fracture of medial condyle of right tibia, subsequent encounter for closed fracture with routine healing
        "S82134E",  # Nondisplaced fracture of medial condyle of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82134F",  # Nondisplaced fracture of medial condyle of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82134G",  # Nondisplaced fracture of medial condyle of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82134H",  # Nondisplaced fracture of medial condyle of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82134J",  # Nondisplaced fracture of medial condyle of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82134K",  # Nondisplaced fracture of medial condyle of right tibia, subsequent encounter for closed fracture with nonunion
        "S82134M",  # Nondisplaced fracture of medial condyle of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82134N",  # Nondisplaced fracture of medial condyle of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82134P",  # Nondisplaced fracture of medial condyle of right tibia, subsequent encounter for closed fracture with malunion
        "S82134Q",  # Nondisplaced fracture of medial condyle of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82134R",  # Nondisplaced fracture of medial condyle of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82134S",  # Nondisplaced fracture of medial condyle of right tibia, sequela
        "S82135A",  # Nondisplaced fracture of medial condyle of left tibia, initial encounter for closed fracture
        "S82135B",  # Nondisplaced fracture of medial condyle of left tibia, initial encounter for open fracture type I or II
        "S82135C",  # Nondisplaced fracture of medial condyle of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82135D",  # Nondisplaced fracture of medial condyle of left tibia, subsequent encounter for closed fracture with routine healing
        "S82135E",  # Nondisplaced fracture of medial condyle of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82135F",  # Nondisplaced fracture of medial condyle of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82135G",  # Nondisplaced fracture of medial condyle of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82135H",  # Nondisplaced fracture of medial condyle of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82135J",  # Nondisplaced fracture of medial condyle of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82135K",  # Nondisplaced fracture of medial condyle of left tibia, subsequent encounter for closed fracture with nonunion
        "S82135M",  # Nondisplaced fracture of medial condyle of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82135N",  # Nondisplaced fracture of medial condyle of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82135P",  # Nondisplaced fracture of medial condyle of left tibia, subsequent encounter for closed fracture with malunion
        "S82135Q",  # Nondisplaced fracture of medial condyle of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82135R",  # Nondisplaced fracture of medial condyle of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82135S",  # Nondisplaced fracture of medial condyle of left tibia, sequela
        "S82136A",  # Nondisplaced fracture of medial condyle of unspecified tibia, initial encounter for closed fracture
        "S82136B",  # Nondisplaced fracture of medial condyle of unspecified tibia, initial encounter for open fracture type I or II
        "S82136C",  # Nondisplaced fracture of medial condyle of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82136D",  # Nondisplaced fracture of medial condyle of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82136E",  # Nondisplaced fracture of medial condyle of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82136F",  # Nondisplaced fracture of medial condyle of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82136G",  # Nondisplaced fracture of medial condyle of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82136H",  # Nondisplaced fracture of medial condyle of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82136J",  # Nondisplaced fracture of medial condyle of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82136K",  # Nondisplaced fracture of medial condyle of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82136M",  # Nondisplaced fracture of medial condyle of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82136N",  # Nondisplaced fracture of medial condyle of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82136P",  # Nondisplaced fracture of medial condyle of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82136Q",  # Nondisplaced fracture of medial condyle of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82136R",  # Nondisplaced fracture of medial condyle of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82136S",  # Nondisplaced fracture of medial condyle of unspecified tibia, sequela
        "S82141A",  # Displaced bicondylar fracture of right tibia, initial encounter for closed fracture
        "S82141B",  # Displaced bicondylar fracture of right tibia, initial encounter for open fracture type I or II
        "S82141C",  # Displaced bicondylar fracture of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82141D",  # Displaced bicondylar fracture of right tibia, subsequent encounter for closed fracture with routine healing
        "S82141E",  # Displaced bicondylar fracture of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82141F",  # Displaced bicondylar fracture of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82141G",  # Displaced bicondylar fracture of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82141H",  # Displaced bicondylar fracture of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82141J",  # Displaced bicondylar fracture of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82141K",  # Displaced bicondylar fracture of right tibia, subsequent encounter for closed fracture with nonunion
        "S82141M",  # Displaced bicondylar fracture of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82141N",  # Displaced bicondylar fracture of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82141P",  # Displaced bicondylar fracture of right tibia, subsequent encounter for closed fracture with malunion
        "S82141Q",  # Displaced bicondylar fracture of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82141R",  # Displaced bicondylar fracture of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82141S",  # Displaced bicondylar fracture of right tibia, sequela
        "S82142A",  # Displaced bicondylar fracture of left tibia, initial encounter for closed fracture
        "S82142B",  # Displaced bicondylar fracture of left tibia, initial encounter for open fracture type I or II
        "S82142C",  # Displaced bicondylar fracture of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82142D",  # Displaced bicondylar fracture of left tibia, subsequent encounter for closed fracture with routine healing
        "S82142E",  # Displaced bicondylar fracture of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82142F",  # Displaced bicondylar fracture of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82142G",  # Displaced bicondylar fracture of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82142H",  # Displaced bicondylar fracture of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82142J",  # Displaced bicondylar fracture of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82142K",  # Displaced bicondylar fracture of left tibia, subsequent encounter for closed fracture with nonunion
        "S82142M",  # Displaced bicondylar fracture of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82142N",  # Displaced bicondylar fracture of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82142P",  # Displaced bicondylar fracture of left tibia, subsequent encounter for closed fracture with malunion
        "S82142Q",  # Displaced bicondylar fracture of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82142R",  # Displaced bicondylar fracture of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82142S",  # Displaced bicondylar fracture of left tibia, sequela
        "S82143A",  # Displaced bicondylar fracture of unspecified tibia, initial encounter for closed fracture
        "S82143B",  # Displaced bicondylar fracture of unspecified tibia, initial encounter for open fracture type I or II
        "S82143C",  # Displaced bicondylar fracture of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82143D",  # Displaced bicondylar fracture of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82143E",  # Displaced bicondylar fracture of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82143F",  # Displaced bicondylar fracture of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82143G",  # Displaced bicondylar fracture of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82143H",  # Displaced bicondylar fracture of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82143J",  # Displaced bicondylar fracture of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82143K",  # Displaced bicondylar fracture of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82143M",  # Displaced bicondylar fracture of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82143N",  # Displaced bicondylar fracture of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82143P",  # Displaced bicondylar fracture of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82143Q",  # Displaced bicondylar fracture of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82143R",  # Displaced bicondylar fracture of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82143S",  # Displaced bicondylar fracture of unspecified tibia, sequela
        "S82144A",  # Nondisplaced bicondylar fracture of right tibia, initial encounter for closed fracture
        "S82144B",  # Nondisplaced bicondylar fracture of right tibia, initial encounter for open fracture type I or II
        "S82144C",  # Nondisplaced bicondylar fracture of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82144D",  # Nondisplaced bicondylar fracture of right tibia, subsequent encounter for closed fracture with routine healing
        "S82144E",  # Nondisplaced bicondylar fracture of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82144F",  # Nondisplaced bicondylar fracture of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82144G",  # Nondisplaced bicondylar fracture of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82144H",  # Nondisplaced bicondylar fracture of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82144J",  # Nondisplaced bicondylar fracture of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82144K",  # Nondisplaced bicondylar fracture of right tibia, subsequent encounter for closed fracture with nonunion
        "S82144M",  # Nondisplaced bicondylar fracture of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82144N",  # Nondisplaced bicondylar fracture of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82144P",  # Nondisplaced bicondylar fracture of right tibia, subsequent encounter for closed fracture with malunion
        "S82144Q",  # Nondisplaced bicondylar fracture of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82144R",  # Nondisplaced bicondylar fracture of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82144S",  # Nondisplaced bicondylar fracture of right tibia, sequela
        "S82145A",  # Nondisplaced bicondylar fracture of left tibia, initial encounter for closed fracture
        "S82145B",  # Nondisplaced bicondylar fracture of left tibia, initial encounter for open fracture type I or II
        "S82145C",  # Nondisplaced bicondylar fracture of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82145D",  # Nondisplaced bicondylar fracture of left tibia, subsequent encounter for closed fracture with routine healing
        "S82145E",  # Nondisplaced bicondylar fracture of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82145F",  # Nondisplaced bicondylar fracture of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82145G",  # Nondisplaced bicondylar fracture of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82145H",  # Nondisplaced bicondylar fracture of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82145J",  # Nondisplaced bicondylar fracture of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82145K",  # Nondisplaced bicondylar fracture of left tibia, subsequent encounter for closed fracture with nonunion
        "S82145M",  # Nondisplaced bicondylar fracture of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82145N",  # Nondisplaced bicondylar fracture of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82145P",  # Nondisplaced bicondylar fracture of left tibia, subsequent encounter for closed fracture with malunion
        "S82145Q",  # Nondisplaced bicondylar fracture of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82145R",  # Nondisplaced bicondylar fracture of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82145S",  # Nondisplaced bicondylar fracture of left tibia, sequela
        "S82146A",  # Nondisplaced bicondylar fracture of unspecified tibia, initial encounter for closed fracture
        "S82146B",  # Nondisplaced bicondylar fracture of unspecified tibia, initial encounter for open fracture type I or II
        "S82146C",  # Nondisplaced bicondylar fracture of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82146D",  # Nondisplaced bicondylar fracture of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82146E",  # Nondisplaced bicondylar fracture of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82146F",  # Nondisplaced bicondylar fracture of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82146G",  # Nondisplaced bicondylar fracture of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82146H",  # Nondisplaced bicondylar fracture of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82146J",  # Nondisplaced bicondylar fracture of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82146K",  # Nondisplaced bicondylar fracture of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82146M",  # Nondisplaced bicondylar fracture of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82146N",  # Nondisplaced bicondylar fracture of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82146P",  # Nondisplaced bicondylar fracture of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82146Q",  # Nondisplaced bicondylar fracture of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82146R",  # Nondisplaced bicondylar fracture of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82146S",  # Nondisplaced bicondylar fracture of unspecified tibia, sequela
        "S82151A",  # Displaced fracture of right tibial tuberosity, initial encounter for closed fracture
        "S82151B",  # Displaced fracture of right tibial tuberosity, initial encounter for open fracture type I or II
        "S82151C",  # Displaced fracture of right tibial tuberosity, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82151D",  # Displaced fracture of right tibial tuberosity, subsequent encounter for closed fracture with routine healing
        "S82151E",  # Displaced fracture of right tibial tuberosity, subsequent encounter for open fracture type I or II with routine healing
        "S82151F",  # Displaced fracture of right tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82151G",  # Displaced fracture of right tibial tuberosity, subsequent encounter for closed fracture with delayed healing
        "S82151H",  # Displaced fracture of right tibial tuberosity, subsequent encounter for open fracture type I or II with delayed healing
        "S82151J",  # Displaced fracture of right tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82151K",  # Displaced fracture of right tibial tuberosity, subsequent encounter for closed fracture with nonunion
        "S82151M",  # Displaced fracture of right tibial tuberosity, subsequent encounter for open fracture type I or II with nonunion
        "S82151N",  # Displaced fracture of right tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82151P",  # Displaced fracture of right tibial tuberosity, subsequent encounter for closed fracture with malunion
        "S82151Q",  # Displaced fracture of right tibial tuberosity, subsequent encounter for open fracture type I or II with malunion
        "S82151R",  # Displaced fracture of right tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82151S",  # Displaced fracture of right tibial tuberosity, sequela
        "S82152A",  # Displaced fracture of left tibial tuberosity, initial encounter for closed fracture
        "S82152B",  # Displaced fracture of left tibial tuberosity, initial encounter for open fracture type I or II
        "S82152C",  # Displaced fracture of left tibial tuberosity, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82152D",  # Displaced fracture of left tibial tuberosity, subsequent encounter for closed fracture with routine healing
        "S82152E",  # Displaced fracture of left tibial tuberosity, subsequent encounter for open fracture type I or II with routine healing
        "S82152F",  # Displaced fracture of left tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82152G",  # Displaced fracture of left tibial tuberosity, subsequent encounter for closed fracture with delayed healing
        "S82152H",  # Displaced fracture of left tibial tuberosity, subsequent encounter for open fracture type I or II with delayed healing
        "S82152J",  # Displaced fracture of left tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82152K",  # Displaced fracture of left tibial tuberosity, subsequent encounter for closed fracture with nonunion
        "S82152M",  # Displaced fracture of left tibial tuberosity, subsequent encounter for open fracture type I or II with nonunion
        "S82152N",  # Displaced fracture of left tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82152P",  # Displaced fracture of left tibial tuberosity, subsequent encounter for closed fracture with malunion
        "S82152Q",  # Displaced fracture of left tibial tuberosity, subsequent encounter for open fracture type I or II with malunion
        "S82152R",  # Displaced fracture of left tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82152S",  # Displaced fracture of left tibial tuberosity, sequela
        "S82153A",  # Displaced fracture of unspecified tibial tuberosity, initial encounter for closed fracture
        "S82153B",  # Displaced fracture of unspecified tibial tuberosity, initial encounter for open fracture type I or II
        "S82153C",  # Displaced fracture of unspecified tibial tuberosity, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82153D",  # Displaced fracture of unspecified tibial tuberosity, subsequent encounter for closed fracture with routine healing
        "S82153E",  # Displaced fracture of unspecified tibial tuberosity, subsequent encounter for open fracture type I or II with routine healing
        "S82153F",  # Displaced fracture of unspecified tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82153G",  # Displaced fracture of unspecified tibial tuberosity, subsequent encounter for closed fracture with delayed healing
        "S82153H",  # Displaced fracture of unspecified tibial tuberosity, subsequent encounter for open fracture type I or II with delayed healing
        "S82153J",  # Displaced fracture of unspecified tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82153K",  # Displaced fracture of unspecified tibial tuberosity, subsequent encounter for closed fracture with nonunion
        "S82153M",  # Displaced fracture of unspecified tibial tuberosity, subsequent encounter for open fracture type I or II with nonunion
        "S82153N",  # Displaced fracture of unspecified tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82153P",  # Displaced fracture of unspecified tibial tuberosity, subsequent encounter for closed fracture with malunion
        "S82153Q",  # Displaced fracture of unspecified tibial tuberosity, subsequent encounter for open fracture type I or II with malunion
        "S82153R",  # Displaced fracture of unspecified tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82153S",  # Displaced fracture of unspecified tibial tuberosity, sequela
        "S82154A",  # Nondisplaced fracture of right tibial tuberosity, initial encounter for closed fracture
        "S82154B",  # Nondisplaced fracture of right tibial tuberosity, initial encounter for open fracture type I or II
        "S82154C",  # Nondisplaced fracture of right tibial tuberosity, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82154D",  # Nondisplaced fracture of right tibial tuberosity, subsequent encounter for closed fracture with routine healing
        "S82154E",  # Nondisplaced fracture of right tibial tuberosity, subsequent encounter for open fracture type I or II with routine healing
        "S82154F",  # Nondisplaced fracture of right tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82154G",  # Nondisplaced fracture of right tibial tuberosity, subsequent encounter for closed fracture with delayed healing
        "S82154H",  # Nondisplaced fracture of right tibial tuberosity, subsequent encounter for open fracture type I or II with delayed healing
        "S82154J",  # Nondisplaced fracture of right tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82154K",  # Nondisplaced fracture of right tibial tuberosity, subsequent encounter for closed fracture with nonunion
        "S82154M",  # Nondisplaced fracture of right tibial tuberosity, subsequent encounter for open fracture type I or II with nonunion
        "S82154N",  # Nondisplaced fracture of right tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82154P",  # Nondisplaced fracture of right tibial tuberosity, subsequent encounter for closed fracture with malunion
        "S82154Q",  # Nondisplaced fracture of right tibial tuberosity, subsequent encounter for open fracture type I or II with malunion
        "S82154R",  # Nondisplaced fracture of right tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82154S",  # Nondisplaced fracture of right tibial tuberosity, sequela
        "S82155A",  # Nondisplaced fracture of left tibial tuberosity, initial encounter for closed fracture
        "S82155B",  # Nondisplaced fracture of left tibial tuberosity, initial encounter for open fracture type I or II
        "S82155C",  # Nondisplaced fracture of left tibial tuberosity, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82155D",  # Nondisplaced fracture of left tibial tuberosity, subsequent encounter for closed fracture with routine healing
        "S82155E",  # Nondisplaced fracture of left tibial tuberosity, subsequent encounter for open fracture type I or II with routine healing
        "S82155F",  # Nondisplaced fracture of left tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82155G",  # Nondisplaced fracture of left tibial tuberosity, subsequent encounter for closed fracture with delayed healing
        "S82155H",  # Nondisplaced fracture of left tibial tuberosity, subsequent encounter for open fracture type I or II with delayed healing
        "S82155J",  # Nondisplaced fracture of left tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82155K",  # Nondisplaced fracture of left tibial tuberosity, subsequent encounter for closed fracture with nonunion
        "S82155M",  # Nondisplaced fracture of left tibial tuberosity, subsequent encounter for open fracture type I or II with nonunion
        "S82155N",  # Nondisplaced fracture of left tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82155P",  # Nondisplaced fracture of left tibial tuberosity, subsequent encounter for closed fracture with malunion
        "S82155Q",  # Nondisplaced fracture of left tibial tuberosity, subsequent encounter for open fracture type I or II with malunion
        "S82155R",  # Nondisplaced fracture of left tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82155S",  # Nondisplaced fracture of left tibial tuberosity, sequela
        "S82156A",  # Nondisplaced fracture of unspecified tibial tuberosity, initial encounter for closed fracture
        "S82156B",  # Nondisplaced fracture of unspecified tibial tuberosity, initial encounter for open fracture type I or II
        "S82156C",  # Nondisplaced fracture of unspecified tibial tuberosity, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82156D",  # Nondisplaced fracture of unspecified tibial tuberosity, subsequent encounter for closed fracture with routine healing
        "S82156E",  # Nondisplaced fracture of unspecified tibial tuberosity, subsequent encounter for open fracture type I or II with routine healing
        "S82156F",  # Nondisplaced fracture of unspecified tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82156G",  # Nondisplaced fracture of unspecified tibial tuberosity, subsequent encounter for closed fracture with delayed healing
        "S82156H",  # Nondisplaced fracture of unspecified tibial tuberosity, subsequent encounter for open fracture type I or II with delayed healing
        "S82156J",  # Nondisplaced fracture of unspecified tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82156K",  # Nondisplaced fracture of unspecified tibial tuberosity, subsequent encounter for closed fracture with nonunion
        "S82156M",  # Nondisplaced fracture of unspecified tibial tuberosity, subsequent encounter for open fracture type I or II with nonunion
        "S82156N",  # Nondisplaced fracture of unspecified tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82156P",  # Nondisplaced fracture of unspecified tibial tuberosity, subsequent encounter for closed fracture with malunion
        "S82156Q",  # Nondisplaced fracture of unspecified tibial tuberosity, subsequent encounter for open fracture type I or II with malunion
        "S82156R",  # Nondisplaced fracture of unspecified tibial tuberosity, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82156S",  # Nondisplaced fracture of unspecified tibial tuberosity, sequela
        "S82161A",  # Torus fracture of upper end of right tibia, initial encounter for closed fracture
        "S82161D",  # Torus fracture of upper end of right tibia, subsequent encounter for fracture with routine healing
        "S82161G",  # Torus fracture of upper end of right tibia, subsequent encounter for fracture with delayed healing
        "S82161K",  # Torus fracture of upper end of right tibia, subsequent encounter for fracture with nonunion
        "S82161P",  # Torus fracture of upper end of right tibia, subsequent encounter for fracture with malunion
        "S82161S",  # Torus fracture of upper end of right tibia, sequela
        "S82162A",  # Torus fracture of upper end of left tibia, initial encounter for closed fracture
        "S82162D",  # Torus fracture of upper end of left tibia, subsequent encounter for fracture with routine healing
        "S82162G",  # Torus fracture of upper end of left tibia, subsequent encounter for fracture with delayed healing
        "S82162K",  # Torus fracture of upper end of left tibia, subsequent encounter for fracture with nonunion
        "S82162P",  # Torus fracture of upper end of left tibia, subsequent encounter for fracture with malunion
        "S82162S",  # Torus fracture of upper end of left tibia, sequela
        "S82169A",  # Torus fracture of upper end of unspecified tibia, initial encounter for closed fracture
        "S82169D",  # Torus fracture of upper end of unspecified tibia, subsequent encounter for fracture with routine healing
        "S82169G",  # Torus fracture of upper end of unspecified tibia, subsequent encounter for fracture with delayed healing
        "S82169K",  # Torus fracture of upper end of unspecified tibia, subsequent encounter for fracture with nonunion
        "S82169P",  # Torus fracture of upper end of unspecified tibia, subsequent encounter for fracture with malunion
        "S82169S",  # Torus fracture of upper end of unspecified tibia, sequela
        "S82191A",  # Other fracture of upper end of right tibia, initial encounter for closed fracture
        "S82191B",  # Other fracture of upper end of right tibia, initial encounter for open fracture type I or II
        "S82191C",  # Other fracture of upper end of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82191D",  # Other fracture of upper end of right tibia, subsequent encounter for closed fracture with routine healing
        "S82191E",  # Other fracture of upper end of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82191F",  # Other fracture of upper end of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82191G",  # Other fracture of upper end of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82191H",  # Other fracture of upper end of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82191J",  # Other fracture of upper end of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82191K",  # Other fracture of upper end of right tibia, subsequent encounter for closed fracture with nonunion
        "S82191M",  # Other fracture of upper end of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82191N",  # Other fracture of upper end of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82191P",  # Other fracture of upper end of right tibia, subsequent encounter for closed fracture with malunion
        "S82191Q",  # Other fracture of upper end of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82191R",  # Other fracture of upper end of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82191S",  # Other fracture of upper end of right tibia, sequela
        "S82192A",  # Other fracture of upper end of left tibia, initial encounter for closed fracture
        "S82192B",  # Other fracture of upper end of left tibia, initial encounter for open fracture type I or II
        "S82192C",  # Other fracture of upper end of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82192D",  # Other fracture of upper end of left tibia, subsequent encounter for closed fracture with routine healing
        "S82192E",  # Other fracture of upper end of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82192F",  # Other fracture of upper end of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82192G",  # Other fracture of upper end of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82192H",  # Other fracture of upper end of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82192J",  # Other fracture of upper end of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82192K",  # Other fracture of upper end of left tibia, subsequent encounter for closed fracture with nonunion
        "S82192M",  # Other fracture of upper end of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82192N",  # Other fracture of upper end of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82192P",  # Other fracture of upper end of left tibia, subsequent encounter for closed fracture with malunion
        "S82192Q",  # Other fracture of upper end of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82192R",  # Other fracture of upper end of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82192S",  # Other fracture of upper end of left tibia, sequela
        "S82199A",  # Other fracture of upper end of unspecified tibia, initial encounter for closed fracture
        "S82199B",  # Other fracture of upper end of unspecified tibia, initial encounter for open fracture type I or II
        "S82199C",  # Other fracture of upper end of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82199D",  # Other fracture of upper end of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82199E",  # Other fracture of upper end of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82199F",  # Other fracture of upper end of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82199G",  # Other fracture of upper end of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82199H",  # Other fracture of upper end of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82199J",  # Other fracture of upper end of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82199K",  # Other fracture of upper end of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82199M",  # Other fracture of upper end of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82199N",  # Other fracture of upper end of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82199P",  # Other fracture of upper end of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82199Q",  # Other fracture of upper end of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82199R",  # Other fracture of upper end of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82199S",  # Other fracture of upper end of unspecified tibia, sequela
        "S82201A",  # Unspecified fracture of shaft of right tibia, initial encounter for closed fracture
        "S82201B",  # Unspecified fracture of shaft of right tibia, initial encounter for open fracture type I or II
        "S82201C",  # Unspecified fracture of shaft of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82201D",  # Unspecified fracture of shaft of right tibia, subsequent encounter for closed fracture with routine healing
        "S82201E",  # Unspecified fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82201F",  # Unspecified fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82201G",  # Unspecified fracture of shaft of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82201H",  # Unspecified fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82201J",  # Unspecified fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82201K",  # Unspecified fracture of shaft of right tibia, subsequent encounter for closed fracture with nonunion
        "S82201M",  # Unspecified fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82201N",  # Unspecified fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82201P",  # Unspecified fracture of shaft of right tibia, subsequent encounter for closed fracture with malunion
        "S82201Q",  # Unspecified fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82201R",  # Unspecified fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82201S",  # Unspecified fracture of shaft of right tibia, sequela
        "S82202A",  # Unspecified fracture of shaft of left tibia, initial encounter for closed fracture
        "S82202B",  # Unspecified fracture of shaft of left tibia, initial encounter for open fracture type I or II
        "S82202C",  # Unspecified fracture of shaft of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82202D",  # Unspecified fracture of shaft of left tibia, subsequent encounter for closed fracture with routine healing
        "S82202E",  # Unspecified fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82202F",  # Unspecified fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82202G",  # Unspecified fracture of shaft of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82202H",  # Unspecified fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82202J",  # Unspecified fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82202K",  # Unspecified fracture of shaft of left tibia, subsequent encounter for closed fracture with nonunion
        "S82202M",  # Unspecified fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82202N",  # Unspecified fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82202P",  # Unspecified fracture of shaft of left tibia, subsequent encounter for closed fracture with malunion
        "S82202Q",  # Unspecified fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82202R",  # Unspecified fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82202S",  # Unspecified fracture of shaft of left tibia, sequela
        "S82209A",  # Unspecified fracture of shaft of unspecified tibia, initial encounter for closed fracture
        "S82209B",  # Unspecified fracture of shaft of unspecified tibia, initial encounter for open fracture type I or II
        "S82209C",  # Unspecified fracture of shaft of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82209D",  # Unspecified fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82209E",  # Unspecified fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82209F",  # Unspecified fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82209G",  # Unspecified fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82209H",  # Unspecified fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82209J",  # Unspecified fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82209K",  # Unspecified fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82209M",  # Unspecified fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82209N",  # Unspecified fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82209P",  # Unspecified fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82209Q",  # Unspecified fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82209R",  # Unspecified fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82209S",  # Unspecified fracture of shaft of unspecified tibia, sequela
        "S82221A",  # Displaced transverse fracture of shaft of right tibia, initial encounter for closed fracture
        "S82221B",  # Displaced transverse fracture of shaft of right tibia, initial encounter for open fracture type I or II
        "S82221C",  # Displaced transverse fracture of shaft of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82221D",  # Displaced transverse fracture of shaft of right tibia, subsequent encounter for closed fracture with routine healing
        "S82221E",  # Displaced transverse fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82221F",  # Displaced transverse fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82221G",  # Displaced transverse fracture of shaft of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82221H",  # Displaced transverse fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82221J",  # Displaced transverse fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82221K",  # Displaced transverse fracture of shaft of right tibia, subsequent encounter for closed fracture with nonunion
        "S82221M",  # Displaced transverse fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82221N",  # Displaced transverse fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82221P",  # Displaced transverse fracture of shaft of right tibia, subsequent encounter for closed fracture with malunion
        "S82221Q",  # Displaced transverse fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82221R",  # Displaced transverse fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82221S",  # Displaced transverse fracture of shaft of right tibia, sequela
        "S82222A",  # Displaced transverse fracture of shaft of left tibia, initial encounter for closed fracture
        "S82222B",  # Displaced transverse fracture of shaft of left tibia, initial encounter for open fracture type I or II
        "S82222C",  # Displaced transverse fracture of shaft of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82222D",  # Displaced transverse fracture of shaft of left tibia, subsequent encounter for closed fracture with routine healing
        "S82222E",  # Displaced transverse fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82222F",  # Displaced transverse fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82222G",  # Displaced transverse fracture of shaft of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82222H",  # Displaced transverse fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82222J",  # Displaced transverse fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82222K",  # Displaced transverse fracture of shaft of left tibia, subsequent encounter for closed fracture with nonunion
        "S82222M",  # Displaced transverse fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82222N",  # Displaced transverse fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82222P",  # Displaced transverse fracture of shaft of left tibia, subsequent encounter for closed fracture with malunion
        "S82222Q",  # Displaced transverse fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82222R",  # Displaced transverse fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82222S",  # Displaced transverse fracture of shaft of left tibia, sequela
        "S82223A",  # Displaced transverse fracture of shaft of unspecified tibia, initial encounter for closed fracture
        "S82223B",  # Displaced transverse fracture of shaft of unspecified tibia, initial encounter for open fracture type I or II
        "S82223C",  # Displaced transverse fracture of shaft of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82223D",  # Displaced transverse fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82223E",  # Displaced transverse fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82223F",  # Displaced transverse fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82223G",  # Displaced transverse fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82223H",  # Displaced transverse fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82223J",  # Displaced transverse fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82223K",  # Displaced transverse fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82223M",  # Displaced transverse fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82223N",  # Displaced transverse fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82223P",  # Displaced transverse fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82223Q",  # Displaced transverse fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82223R",  # Displaced transverse fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82223S",  # Displaced transverse fracture of shaft of unspecified tibia, sequela
        "S82224A",  # Nondisplaced transverse fracture of shaft of right tibia, initial encounter for closed fracture
        "S82224B",  # Nondisplaced transverse fracture of shaft of right tibia, initial encounter for open fracture type I or II
        "S82224C",  # Nondisplaced transverse fracture of shaft of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82224D",  # Nondisplaced transverse fracture of shaft of right tibia, subsequent encounter for closed fracture with routine healing
        "S82224E",  # Nondisplaced transverse fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82224F",  # Nondisplaced transverse fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82224G",  # Nondisplaced transverse fracture of shaft of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82224H",  # Nondisplaced transverse fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82224J",  # Nondisplaced transverse fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82224K",  # Nondisplaced transverse fracture of shaft of right tibia, subsequent encounter for closed fracture with nonunion
        "S82224M",  # Nondisplaced transverse fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82224N",  # Nondisplaced transverse fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82224P",  # Nondisplaced transverse fracture of shaft of right tibia, subsequent encounter for closed fracture with malunion
        "S82224Q",  # Nondisplaced transverse fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82224R",  # Nondisplaced transverse fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82224S",  # Nondisplaced transverse fracture of shaft of right tibia, sequela
        "S82225A",  # Nondisplaced transverse fracture of shaft of left tibia, initial encounter for closed fracture
        "S82225B",  # Nondisplaced transverse fracture of shaft of left tibia, initial encounter for open fracture type I or II
        "S82225C",  # Nondisplaced transverse fracture of shaft of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82225D",  # Nondisplaced transverse fracture of shaft of left tibia, subsequent encounter for closed fracture with routine healing
        "S82225E",  # Nondisplaced transverse fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82225F",  # Nondisplaced transverse fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82225G",  # Nondisplaced transverse fracture of shaft of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82225H",  # Nondisplaced transverse fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82225J",  # Nondisplaced transverse fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82225K",  # Nondisplaced transverse fracture of shaft of left tibia, subsequent encounter for closed fracture with nonunion
        "S82225M",  # Nondisplaced transverse fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82225N",  # Nondisplaced transverse fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82225P",  # Nondisplaced transverse fracture of shaft of left tibia, subsequent encounter for closed fracture with malunion
        "S82225Q",  # Nondisplaced transverse fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82225R",  # Nondisplaced transverse fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82225S",  # Nondisplaced transverse fracture of shaft of left tibia, sequela
        "S82226A",  # Nondisplaced transverse fracture of shaft of unspecified tibia, initial encounter for closed fracture
        "S82226B",  # Nondisplaced transverse fracture of shaft of unspecified tibia, initial encounter for open fracture type I or II
        "S82226C",  # Nondisplaced transverse fracture of shaft of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82226D",  # Nondisplaced transverse fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82226E",  # Nondisplaced transverse fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82226F",  # Nondisplaced transverse fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82226G",  # Nondisplaced transverse fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82226H",  # Nondisplaced transverse fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82226J",  # Nondisplaced transverse fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82226K",  # Nondisplaced transverse fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82226M",  # Nondisplaced transverse fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82226N",  # Nondisplaced transverse fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82226P",  # Nondisplaced transverse fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82226Q",  # Nondisplaced transverse fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82226R",  # Nondisplaced transverse fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82226S",  # Nondisplaced transverse fracture of shaft of unspecified tibia, sequela
        "S82231A",  # Displaced oblique fracture of shaft of right tibia, initial encounter for closed fracture
        "S82231B",  # Displaced oblique fracture of shaft of right tibia, initial encounter for open fracture type I or II
        "S82231C",  # Displaced oblique fracture of shaft of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82231D",  # Displaced oblique fracture of shaft of right tibia, subsequent encounter for closed fracture with routine healing
        "S82231E",  # Displaced oblique fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82231F",  # Displaced oblique fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82231G",  # Displaced oblique fracture of shaft of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82231H",  # Displaced oblique fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82231J",  # Displaced oblique fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82231K",  # Displaced oblique fracture of shaft of right tibia, subsequent encounter for closed fracture with nonunion
        "S82231M",  # Displaced oblique fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82231N",  # Displaced oblique fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82231P",  # Displaced oblique fracture of shaft of right tibia, subsequent encounter for closed fracture with malunion
        "S82231Q",  # Displaced oblique fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82231R",  # Displaced oblique fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82231S",  # Displaced oblique fracture of shaft of right tibia, sequela
        "S82232A",  # Displaced oblique fracture of shaft of left tibia, initial encounter for closed fracture
        "S82232B",  # Displaced oblique fracture of shaft of left tibia, initial encounter for open fracture type I or II
        "S82232C",  # Displaced oblique fracture of shaft of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82232D",  # Displaced oblique fracture of shaft of left tibia, subsequent encounter for closed fracture with routine healing
        "S82232E",  # Displaced oblique fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82232F",  # Displaced oblique fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82232G",  # Displaced oblique fracture of shaft of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82232H",  # Displaced oblique fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82232J",  # Displaced oblique fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82232K",  # Displaced oblique fracture of shaft of left tibia, subsequent encounter for closed fracture with nonunion
        "S82232M",  # Displaced oblique fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82232N",  # Displaced oblique fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82232P",  # Displaced oblique fracture of shaft of left tibia, subsequent encounter for closed fracture with malunion
        "S82232Q",  # Displaced oblique fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82232R",  # Displaced oblique fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82232S",  # Displaced oblique fracture of shaft of left tibia, sequela
        "S82233A",  # Displaced oblique fracture of shaft of unspecified tibia, initial encounter for closed fracture
        "S82233B",  # Displaced oblique fracture of shaft of unspecified tibia, initial encounter for open fracture type I or II
        "S82233C",  # Displaced oblique fracture of shaft of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82233D",  # Displaced oblique fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82233E",  # Displaced oblique fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82233F",  # Displaced oblique fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82233G",  # Displaced oblique fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82233H",  # Displaced oblique fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82233J",  # Displaced oblique fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82233K",  # Displaced oblique fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82233M",  # Displaced oblique fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82233N",  # Displaced oblique fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82233P",  # Displaced oblique fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82233Q",  # Displaced oblique fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82233R",  # Displaced oblique fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82233S",  # Displaced oblique fracture of shaft of unspecified tibia, sequela
        "S82234A",  # Nondisplaced oblique fracture of shaft of right tibia, initial encounter for closed fracture
        "S82234B",  # Nondisplaced oblique fracture of shaft of right tibia, initial encounter for open fracture type I or II
        "S82234C",  # Nondisplaced oblique fracture of shaft of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82234D",  # Nondisplaced oblique fracture of shaft of right tibia, subsequent encounter for closed fracture with routine healing
        "S82234E",  # Nondisplaced oblique fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82234F",  # Nondisplaced oblique fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82234G",  # Nondisplaced oblique fracture of shaft of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82234H",  # Nondisplaced oblique fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82234J",  # Nondisplaced oblique fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82234K",  # Nondisplaced oblique fracture of shaft of right tibia, subsequent encounter for closed fracture with nonunion
        "S82234M",  # Nondisplaced oblique fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82234N",  # Nondisplaced oblique fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82234P",  # Nondisplaced oblique fracture of shaft of right tibia, subsequent encounter for closed fracture with malunion
        "S82234Q",  # Nondisplaced oblique fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82234R",  # Nondisplaced oblique fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82234S",  # Nondisplaced oblique fracture of shaft of right tibia, sequela
        "S82235A",  # Nondisplaced oblique fracture of shaft of left tibia, initial encounter for closed fracture
        "S82235B",  # Nondisplaced oblique fracture of shaft of left tibia, initial encounter for open fracture type I or II
        "S82235C",  # Nondisplaced oblique fracture of shaft of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82235D",  # Nondisplaced oblique fracture of shaft of left tibia, subsequent encounter for closed fracture with routine healing
        "S82235E",  # Nondisplaced oblique fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82235F",  # Nondisplaced oblique fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82235G",  # Nondisplaced oblique fracture of shaft of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82235H",  # Nondisplaced oblique fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82235J",  # Nondisplaced oblique fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82235K",  # Nondisplaced oblique fracture of shaft of left tibia, subsequent encounter for closed fracture with nonunion
        "S82235M",  # Nondisplaced oblique fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82235N",  # Nondisplaced oblique fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82235P",  # Nondisplaced oblique fracture of shaft of left tibia, subsequent encounter for closed fracture with malunion
        "S82235Q",  # Nondisplaced oblique fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82235R",  # Nondisplaced oblique fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82235S",  # Nondisplaced oblique fracture of shaft of left tibia, sequela
        "S82236A",  # Nondisplaced oblique fracture of shaft of unspecified tibia, initial encounter for closed fracture
        "S82236B",  # Nondisplaced oblique fracture of shaft of unspecified tibia, initial encounter for open fracture type I or II
        "S82236C",  # Nondisplaced oblique fracture of shaft of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82236D",  # Nondisplaced oblique fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82236E",  # Nondisplaced oblique fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82236F",  # Nondisplaced oblique fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82236G",  # Nondisplaced oblique fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82236H",  # Nondisplaced oblique fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82236J",  # Nondisplaced oblique fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82236K",  # Nondisplaced oblique fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82236M",  # Nondisplaced oblique fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82236N",  # Nondisplaced oblique fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82236P",  # Nondisplaced oblique fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82236Q",  # Nondisplaced oblique fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82236R",  # Nondisplaced oblique fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82236S",  # Nondisplaced oblique fracture of shaft of unspecified tibia, sequela
        "S82241A",  # Displaced spiral fracture of shaft of right tibia, initial encounter for closed fracture
        "S82241B",  # Displaced spiral fracture of shaft of right tibia, initial encounter for open fracture type I or II
        "S82241C",  # Displaced spiral fracture of shaft of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82241D",  # Displaced spiral fracture of shaft of right tibia, subsequent encounter for closed fracture with routine healing
        "S82241E",  # Displaced spiral fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82241F",  # Displaced spiral fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82241G",  # Displaced spiral fracture of shaft of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82241H",  # Displaced spiral fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82241J",  # Displaced spiral fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82241K",  # Displaced spiral fracture of shaft of right tibia, subsequent encounter for closed fracture with nonunion
        "S82241M",  # Displaced spiral fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82241N",  # Displaced spiral fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82241P",  # Displaced spiral fracture of shaft of right tibia, subsequent encounter for closed fracture with malunion
        "S82241Q",  # Displaced spiral fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82241R",  # Displaced spiral fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82241S",  # Displaced spiral fracture of shaft of right tibia, sequela
        "S82242A",  # Displaced spiral fracture of shaft of left tibia, initial encounter for closed fracture
        "S82242B",  # Displaced spiral fracture of shaft of left tibia, initial encounter for open fracture type I or II
        "S82242C",  # Displaced spiral fracture of shaft of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82242D",  # Displaced spiral fracture of shaft of left tibia, subsequent encounter for closed fracture with routine healing
        "S82242E",  # Displaced spiral fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82242F",  # Displaced spiral fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82242G",  # Displaced spiral fracture of shaft of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82242H",  # Displaced spiral fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82242J",  # Displaced spiral fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82242K",  # Displaced spiral fracture of shaft of left tibia, subsequent encounter for closed fracture with nonunion
        "S82242M",  # Displaced spiral fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82242N",  # Displaced spiral fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82242P",  # Displaced spiral fracture of shaft of left tibia, subsequent encounter for closed fracture with malunion
        "S82242Q",  # Displaced spiral fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82242R",  # Displaced spiral fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82242S",  # Displaced spiral fracture of shaft of left tibia, sequela
        "S82243A",  # Displaced spiral fracture of shaft of unspecified tibia, initial encounter for closed fracture
        "S82243B",  # Displaced spiral fracture of shaft of unspecified tibia, initial encounter for open fracture type I or II
        "S82243C",  # Displaced spiral fracture of shaft of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82243D",  # Displaced spiral fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82243E",  # Displaced spiral fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82243F",  # Displaced spiral fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82243G",  # Displaced spiral fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82243H",  # Displaced spiral fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82243J",  # Displaced spiral fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82243K",  # Displaced spiral fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82243M",  # Displaced spiral fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82243N",  # Displaced spiral fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82243P",  # Displaced spiral fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82243Q",  # Displaced spiral fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82243R",  # Displaced spiral fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82243S",  # Displaced spiral fracture of shaft of unspecified tibia, sequela
        "S82244A",  # Nondisplaced spiral fracture of shaft of right tibia, initial encounter for closed fracture
        "S82244B",  # Nondisplaced spiral fracture of shaft of right tibia, initial encounter for open fracture type I or II
        "S82244C",  # Nondisplaced spiral fracture of shaft of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82244D",  # Nondisplaced spiral fracture of shaft of right tibia, subsequent encounter for closed fracture with routine healing
        "S82244E",  # Nondisplaced spiral fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82244F",  # Nondisplaced spiral fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82244G",  # Nondisplaced spiral fracture of shaft of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82244H",  # Nondisplaced spiral fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82244J",  # Nondisplaced spiral fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82244K",  # Nondisplaced spiral fracture of shaft of right tibia, subsequent encounter for closed fracture with nonunion
        "S82244M",  # Nondisplaced spiral fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82244N",  # Nondisplaced spiral fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82244P",  # Nondisplaced spiral fracture of shaft of right tibia, subsequent encounter for closed fracture with malunion
        "S82244Q",  # Nondisplaced spiral fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82244R",  # Nondisplaced spiral fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82244S",  # Nondisplaced spiral fracture of shaft of right tibia, sequela
        "S82245A",  # Nondisplaced spiral fracture of shaft of left tibia, initial encounter for closed fracture
        "S82245B",  # Nondisplaced spiral fracture of shaft of left tibia, initial encounter for open fracture type I or II
        "S82245C",  # Nondisplaced spiral fracture of shaft of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82245D",  # Nondisplaced spiral fracture of shaft of left tibia, subsequent encounter for closed fracture with routine healing
        "S82245E",  # Nondisplaced spiral fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82245F",  # Nondisplaced spiral fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82245G",  # Nondisplaced spiral fracture of shaft of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82245H",  # Nondisplaced spiral fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82245J",  # Nondisplaced spiral fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82245K",  # Nondisplaced spiral fracture of shaft of left tibia, subsequent encounter for closed fracture with nonunion
        "S82245M",  # Nondisplaced spiral fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82245N",  # Nondisplaced spiral fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82245P",  # Nondisplaced spiral fracture of shaft of left tibia, subsequent encounter for closed fracture with malunion
        "S82245Q",  # Nondisplaced spiral fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82245R",  # Nondisplaced spiral fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82245S",  # Nondisplaced spiral fracture of shaft of left tibia, sequela
        "S82246A",  # Nondisplaced spiral fracture of shaft of unspecified tibia, initial encounter for closed fracture
        "S82246B",  # Nondisplaced spiral fracture of shaft of unspecified tibia, initial encounter for open fracture type I or II
        "S82246C",  # Nondisplaced spiral fracture of shaft of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82246D",  # Nondisplaced spiral fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82246E",  # Nondisplaced spiral fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82246F",  # Nondisplaced spiral fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82246G",  # Nondisplaced spiral fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82246H",  # Nondisplaced spiral fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82246J",  # Nondisplaced spiral fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82246K",  # Nondisplaced spiral fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82246M",  # Nondisplaced spiral fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82246N",  # Nondisplaced spiral fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82246P",  # Nondisplaced spiral fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82246Q",  # Nondisplaced spiral fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82246R",  # Nondisplaced spiral fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82246S",  # Nondisplaced spiral fracture of shaft of unspecified tibia, sequela
        "S82251A",  # Displaced comminuted fracture of shaft of right tibia, initial encounter for closed fracture
        "S82251B",  # Displaced comminuted fracture of shaft of right tibia, initial encounter for open fracture type I or II
        "S82251C",  # Displaced comminuted fracture of shaft of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82251D",  # Displaced comminuted fracture of shaft of right tibia, subsequent encounter for closed fracture with routine healing
        "S82251E",  # Displaced comminuted fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82251F",  # Displaced comminuted fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82251G",  # Displaced comminuted fracture of shaft of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82251H",  # Displaced comminuted fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82251J",  # Displaced comminuted fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82251K",  # Displaced comminuted fracture of shaft of right tibia, subsequent encounter for closed fracture with nonunion
        "S82251M",  # Displaced comminuted fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82251N",  # Displaced comminuted fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82251P",  # Displaced comminuted fracture of shaft of right tibia, subsequent encounter for closed fracture with malunion
        "S82251Q",  # Displaced comminuted fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82251R",  # Displaced comminuted fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82251S",  # Displaced comminuted fracture of shaft of right tibia, sequela
        "S82252A",  # Displaced comminuted fracture of shaft of left tibia, initial encounter for closed fracture
        "S82252B",  # Displaced comminuted fracture of shaft of left tibia, initial encounter for open fracture type I or II
        "S82252C",  # Displaced comminuted fracture of shaft of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82252D",  # Displaced comminuted fracture of shaft of left tibia, subsequent encounter for closed fracture with routine healing
        "S82252E",  # Displaced comminuted fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82252F",  # Displaced comminuted fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82252G",  # Displaced comminuted fracture of shaft of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82252H",  # Displaced comminuted fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82252J",  # Displaced comminuted fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82252K",  # Displaced comminuted fracture of shaft of left tibia, subsequent encounter for closed fracture with nonunion
        "S82252M",  # Displaced comminuted fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82252N",  # Displaced comminuted fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82252P",  # Displaced comminuted fracture of shaft of left tibia, subsequent encounter for closed fracture with malunion
        "S82252Q",  # Displaced comminuted fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82252R",  # Displaced comminuted fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82252S",  # Displaced comminuted fracture of shaft of left tibia, sequela
        "S82253A",  # Displaced comminuted fracture of shaft of unspecified tibia, initial encounter for closed fracture
        "S82253B",  # Displaced comminuted fracture of shaft of unspecified tibia, initial encounter for open fracture type I or II
        "S82253C",  # Displaced comminuted fracture of shaft of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82253D",  # Displaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82253E",  # Displaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82253F",  # Displaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82253G",  # Displaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82253H",  # Displaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82253J",  # Displaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82253K",  # Displaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82253M",  # Displaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82253N",  # Displaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82253P",  # Displaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82253Q",  # Displaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82253R",  # Displaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82253S",  # Displaced comminuted fracture of shaft of unspecified tibia, sequela
        "S82254A",  # Nondisplaced comminuted fracture of shaft of right tibia, initial encounter for closed fracture
        "S82254B",  # Nondisplaced comminuted fracture of shaft of right tibia, initial encounter for open fracture type I or II
        "S82254C",  # Nondisplaced comminuted fracture of shaft of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82254D",  # Nondisplaced comminuted fracture of shaft of right tibia, subsequent encounter for closed fracture with routine healing
        "S82254E",  # Nondisplaced comminuted fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82254F",  # Nondisplaced comminuted fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82254G",  # Nondisplaced comminuted fracture of shaft of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82254H",  # Nondisplaced comminuted fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82254J",  # Nondisplaced comminuted fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82254K",  # Nondisplaced comminuted fracture of shaft of right tibia, subsequent encounter for closed fracture with nonunion
        "S82254M",  # Nondisplaced comminuted fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82254N",  # Nondisplaced comminuted fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82254P",  # Nondisplaced comminuted fracture of shaft of right tibia, subsequent encounter for closed fracture with malunion
        "S82254Q",  # Nondisplaced comminuted fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82254R",  # Nondisplaced comminuted fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82254S",  # Nondisplaced comminuted fracture of shaft of right tibia, sequela
        "S82255A",  # Nondisplaced comminuted fracture of shaft of left tibia, initial encounter for closed fracture
        "S82255B",  # Nondisplaced comminuted fracture of shaft of left tibia, initial encounter for open fracture type I or II
        "S82255C",  # Nondisplaced comminuted fracture of shaft of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82255D",  # Nondisplaced comminuted fracture of shaft of left tibia, subsequent encounter for closed fracture with routine healing
        "S82255E",  # Nondisplaced comminuted fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82255F",  # Nondisplaced comminuted fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82255G",  # Nondisplaced comminuted fracture of shaft of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82255H",  # Nondisplaced comminuted fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82255J",  # Nondisplaced comminuted fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82255K",  # Nondisplaced comminuted fracture of shaft of left tibia, subsequent encounter for closed fracture with nonunion
        "S82255M",  # Nondisplaced comminuted fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82255N",  # Nondisplaced comminuted fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82255P",  # Nondisplaced comminuted fracture of shaft of left tibia, subsequent encounter for closed fracture with malunion
        "S82255Q",  # Nondisplaced comminuted fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82255R",  # Nondisplaced comminuted fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82255S",  # Nondisplaced comminuted fracture of shaft of left tibia, sequela
        "S82256A",  # Nondisplaced comminuted fracture of shaft of unspecified tibia, initial encounter for closed fracture
        "S82256B",  # Nondisplaced comminuted fracture of shaft of unspecified tibia, initial encounter for open fracture type I or II
        "S82256C",  # Nondisplaced comminuted fracture of shaft of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82256D",  # Nondisplaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82256E",  # Nondisplaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82256F",  # Nondisplaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82256G",  # Nondisplaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82256H",  # Nondisplaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82256J",  # Nondisplaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82256K",  # Nondisplaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82256M",  # Nondisplaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82256N",  # Nondisplaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82256P",  # Nondisplaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82256Q",  # Nondisplaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82256R",  # Nondisplaced comminuted fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82256S",  # Nondisplaced comminuted fracture of shaft of unspecified tibia, sequela
        "S82261A",  # Displaced segmental fracture of shaft of right tibia, initial encounter for closed fracture
        "S82261B",  # Displaced segmental fracture of shaft of right tibia, initial encounter for open fracture type I or II
        "S82261C",  # Displaced segmental fracture of shaft of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82261D",  # Displaced segmental fracture of shaft of right tibia, subsequent encounter for closed fracture with routine healing
        "S82261E",  # Displaced segmental fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82261F",  # Displaced segmental fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82261G",  # Displaced segmental fracture of shaft of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82261H",  # Displaced segmental fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82261J",  # Displaced segmental fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82261K",  # Displaced segmental fracture of shaft of right tibia, subsequent encounter for closed fracture with nonunion
        "S82261M",  # Displaced segmental fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82261N",  # Displaced segmental fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82261P",  # Displaced segmental fracture of shaft of right tibia, subsequent encounter for closed fracture with malunion
        "S82261Q",  # Displaced segmental fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82261R",  # Displaced segmental fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82261S",  # Displaced segmental fracture of shaft of right tibia, sequela
        "S82262A",  # Displaced segmental fracture of shaft of left tibia, initial encounter for closed fracture
        "S82262B",  # Displaced segmental fracture of shaft of left tibia, initial encounter for open fracture type I or II
        "S82262C",  # Displaced segmental fracture of shaft of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82262D",  # Displaced segmental fracture of shaft of left tibia, subsequent encounter for closed fracture with routine healing
        "S82262E",  # Displaced segmental fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82262F",  # Displaced segmental fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82262G",  # Displaced segmental fracture of shaft of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82262H",  # Displaced segmental fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82262J",  # Displaced segmental fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82262K",  # Displaced segmental fracture of shaft of left tibia, subsequent encounter for closed fracture with nonunion
        "S82262M",  # Displaced segmental fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82262N",  # Displaced segmental fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82262P",  # Displaced segmental fracture of shaft of left tibia, subsequent encounter for closed fracture with malunion
        "S82262Q",  # Displaced segmental fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82262R",  # Displaced segmental fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82262S",  # Displaced segmental fracture of shaft of left tibia, sequela
        "S82263A",  # Displaced segmental fracture of shaft of unspecified tibia, initial encounter for closed fracture
        "S82263B",  # Displaced segmental fracture of shaft of unspecified tibia, initial encounter for open fracture type I or II
        "S82263C",  # Displaced segmental fracture of shaft of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82263D",  # Displaced segmental fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82263E",  # Displaced segmental fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82263F",  # Displaced segmental fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82263G",  # Displaced segmental fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82263H",  # Displaced segmental fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82263J",  # Displaced segmental fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82263K",  # Displaced segmental fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82263M",  # Displaced segmental fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82263N",  # Displaced segmental fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82263P",  # Displaced segmental fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82263Q",  # Displaced segmental fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82263R",  # Displaced segmental fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82263S",  # Displaced segmental fracture of shaft of unspecified tibia, sequela
        "S82264A",  # Nondisplaced segmental fracture of shaft of right tibia, initial encounter for closed fracture
        "S82264B",  # Nondisplaced segmental fracture of shaft of right tibia, initial encounter for open fracture type I or II
        "S82264C",  # Nondisplaced segmental fracture of shaft of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82264D",  # Nondisplaced segmental fracture of shaft of right tibia, subsequent encounter for closed fracture with routine healing
        "S82264E",  # Nondisplaced segmental fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82264F",  # Nondisplaced segmental fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82264G",  # Nondisplaced segmental fracture of shaft of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82264H",  # Nondisplaced segmental fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82264J",  # Nondisplaced segmental fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82264K",  # Nondisplaced segmental fracture of shaft of right tibia, subsequent encounter for closed fracture with nonunion
        "S82264M",  # Nondisplaced segmental fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82264N",  # Nondisplaced segmental fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82264P",  # Nondisplaced segmental fracture of shaft of right tibia, subsequent encounter for closed fracture with malunion
        "S82264Q",  # Nondisplaced segmental fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82264R",  # Nondisplaced segmental fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82264S",  # Nondisplaced segmental fracture of shaft of right tibia, sequela
        "S82265A",  # Nondisplaced segmental fracture of shaft of left tibia, initial encounter for closed fracture
        "S82265B",  # Nondisplaced segmental fracture of shaft of left tibia, initial encounter for open fracture type I or II
        "S82265C",  # Nondisplaced segmental fracture of shaft of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82265D",  # Nondisplaced segmental fracture of shaft of left tibia, subsequent encounter for closed fracture with routine healing
        "S82265E",  # Nondisplaced segmental fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82265F",  # Nondisplaced segmental fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82265G",  # Nondisplaced segmental fracture of shaft of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82265H",  # Nondisplaced segmental fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82265J",  # Nondisplaced segmental fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82265K",  # Nondisplaced segmental fracture of shaft of left tibia, subsequent encounter for closed fracture with nonunion
        "S82265M",  # Nondisplaced segmental fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82265N",  # Nondisplaced segmental fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82265P",  # Nondisplaced segmental fracture of shaft of left tibia, subsequent encounter for closed fracture with malunion
        "S82265Q",  # Nondisplaced segmental fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82265R",  # Nondisplaced segmental fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82265S",  # Nondisplaced segmental fracture of shaft of left tibia, sequela
        "S82266A",  # Nondisplaced segmental fracture of shaft of unspecified tibia, initial encounter for closed fracture
        "S82266B",  # Nondisplaced segmental fracture of shaft of unspecified tibia, initial encounter for open fracture type I or II
        "S82266C",  # Nondisplaced segmental fracture of shaft of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82266D",  # Nondisplaced segmental fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82266E",  # Nondisplaced segmental fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82266F",  # Nondisplaced segmental fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82266G",  # Nondisplaced segmental fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82266H",  # Nondisplaced segmental fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82266J",  # Nondisplaced segmental fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82266K",  # Nondisplaced segmental fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82266M",  # Nondisplaced segmental fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82266N",  # Nondisplaced segmental fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82266P",  # Nondisplaced segmental fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82266Q",  # Nondisplaced segmental fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82266R",  # Nondisplaced segmental fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82266S",  # Nondisplaced segmental fracture of shaft of unspecified tibia, sequela
        "S82291A",  # Other fracture of shaft of right tibia, initial encounter for closed fracture
        "S82291B",  # Other fracture of shaft of right tibia, initial encounter for open fracture type I or II
        "S82291C",  # Other fracture of shaft of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82291D",  # Other fracture of shaft of right tibia, subsequent encounter for closed fracture with routine healing
        "S82291E",  # Other fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82291F",  # Other fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82291G",  # Other fracture of shaft of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82291H",  # Other fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82291J",  # Other fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82291K",  # Other fracture of shaft of right tibia, subsequent encounter for closed fracture with nonunion
        "S82291M",  # Other fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82291N",  # Other fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82291P",  # Other fracture of shaft of right tibia, subsequent encounter for closed fracture with malunion
        "S82291Q",  # Other fracture of shaft of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82291R",  # Other fracture of shaft of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82291S",  # Other fracture of shaft of right tibia, sequela
        "S82292A",  # Other fracture of shaft of left tibia, initial encounter for closed fracture
        "S82292B",  # Other fracture of shaft of left tibia, initial encounter for open fracture type I or II
        "S82292C",  # Other fracture of shaft of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82292D",  # Other fracture of shaft of left tibia, subsequent encounter for closed fracture with routine healing
        "S82292E",  # Other fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82292F",  # Other fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82292G",  # Other fracture of shaft of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82292H",  # Other fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82292J",  # Other fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82292K",  # Other fracture of shaft of left tibia, subsequent encounter for closed fracture with nonunion
        "S82292M",  # Other fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82292N",  # Other fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82292P",  # Other fracture of shaft of left tibia, subsequent encounter for closed fracture with malunion
        "S82292Q",  # Other fracture of shaft of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82292R",  # Other fracture of shaft of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82292S",  # Other fracture of shaft of left tibia, sequela
        "S82299A",  # Other fracture of shaft of unspecified tibia, initial encounter for closed fracture
        "S82299B",  # Other fracture of shaft of unspecified tibia, initial encounter for open fracture type I or II
        "S82299C",  # Other fracture of shaft of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82299D",  # Other fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82299E",  # Other fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82299F",  # Other fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82299G",  # Other fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82299H",  # Other fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82299J",  # Other fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82299K",  # Other fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82299M",  # Other fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82299N",  # Other fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82299P",  # Other fracture of shaft of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82299Q",  # Other fracture of shaft of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82299R",  # Other fracture of shaft of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82299S",  # Other fracture of shaft of unspecified tibia, sequela
        "S82301A",  # Unspecified fracture of lower end of right tibia, initial encounter for closed fracture
        "S82301B",  # Unspecified fracture of lower end of right tibia, initial encounter for open fracture type I or II
        "S82301C",  # Unspecified fracture of lower end of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82301D",  # Unspecified fracture of lower end of right tibia, subsequent encounter for closed fracture with routine healing
        "S82301E",  # Unspecified fracture of lower end of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82301F",  # Unspecified fracture of lower end of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82301G",  # Unspecified fracture of lower end of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82301H",  # Unspecified fracture of lower end of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82301J",  # Unspecified fracture of lower end of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82301K",  # Unspecified fracture of lower end of right tibia, subsequent encounter for closed fracture with nonunion
        "S82301M",  # Unspecified fracture of lower end of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82301N",  # Unspecified fracture of lower end of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82301P",  # Unspecified fracture of lower end of right tibia, subsequent encounter for closed fracture with malunion
        "S82301Q",  # Unspecified fracture of lower end of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82301R",  # Unspecified fracture of lower end of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82301S",  # Unspecified fracture of lower end of right tibia, sequela
        "S82302A",  # Unspecified fracture of lower end of left tibia, initial encounter for closed fracture
        "S82302B",  # Unspecified fracture of lower end of left tibia, initial encounter for open fracture type I or II
        "S82302C",  # Unspecified fracture of lower end of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82302D",  # Unspecified fracture of lower end of left tibia, subsequent encounter for closed fracture with routine healing
        "S82302E",  # Unspecified fracture of lower end of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82302F",  # Unspecified fracture of lower end of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82302G",  # Unspecified fracture of lower end of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82302H",  # Unspecified fracture of lower end of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82302J",  # Unspecified fracture of lower end of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82302K",  # Unspecified fracture of lower end of left tibia, subsequent encounter for closed fracture with nonunion
        "S82302M",  # Unspecified fracture of lower end of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82302N",  # Unspecified fracture of lower end of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82302P",  # Unspecified fracture of lower end of left tibia, subsequent encounter for closed fracture with malunion
        "S82302Q",  # Unspecified fracture of lower end of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82302R",  # Unspecified fracture of lower end of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82302S",  # Unspecified fracture of lower end of left tibia, sequela
        "S82309A",  # Unspecified fracture of lower end of unspecified tibia, initial encounter for closed fracture
        "S82309B",  # Unspecified fracture of lower end of unspecified tibia, initial encounter for open fracture type I or II
        "S82309C",  # Unspecified fracture of lower end of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82309D",  # Unspecified fracture of lower end of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82309E",  # Unspecified fracture of lower end of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82309F",  # Unspecified fracture of lower end of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82309G",  # Unspecified fracture of lower end of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82309H",  # Unspecified fracture of lower end of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82309J",  # Unspecified fracture of lower end of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82309K",  # Unspecified fracture of lower end of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82309M",  # Unspecified fracture of lower end of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82309N",  # Unspecified fracture of lower end of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82309P",  # Unspecified fracture of lower end of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82309Q",  # Unspecified fracture of lower end of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82309R",  # Unspecified fracture of lower end of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82309S",  # Unspecified fracture of lower end of unspecified tibia, sequela
        "S82311A",  # Torus fracture of lower end of right tibia, initial encounter for closed fracture
        "S82311D",  # Torus fracture of lower end of right tibia, subsequent encounter for fracture with routine healing
        "S82311G",  # Torus fracture of lower end of right tibia, subsequent encounter for fracture with delayed healing
        "S82311K",  # Torus fracture of lower end of right tibia, subsequent encounter for fracture with nonunion
        "S82311P",  # Torus fracture of lower end of right tibia, subsequent encounter for fracture with malunion
        "S82311S",  # Torus fracture of lower end of right tibia, sequela
        "S82312A",  # Torus fracture of lower end of left tibia, initial encounter for closed fracture
        "S82312D",  # Torus fracture of lower end of left tibia, subsequent encounter for fracture with routine healing
        "S82312G",  # Torus fracture of lower end of left tibia, subsequent encounter for fracture with delayed healing
        "S82312K",  # Torus fracture of lower end of left tibia, subsequent encounter for fracture with nonunion
        "S82312P",  # Torus fracture of lower end of left tibia, subsequent encounter for fracture with malunion
        "S82312S",  # Torus fracture of lower end of left tibia, sequela
        "S82319A",  # Torus fracture of lower end of unspecified tibia, initial encounter for closed fracture
        "S82319D",  # Torus fracture of lower end of unspecified tibia, subsequent encounter for fracture with routine healing
        "S82319G",  # Torus fracture of lower end of unspecified tibia, subsequent encounter for fracture with delayed healing
        "S82319K",  # Torus fracture of lower end of unspecified tibia, subsequent encounter for fracture with nonunion
        "S82319P",  # Torus fracture of lower end of unspecified tibia, subsequent encounter for fracture with malunion
        "S82319S",  # Torus fracture of lower end of unspecified tibia, sequela
        "S82391A",  # Other fracture of lower end of right tibia, initial encounter for closed fracture
        "S82391B",  # Other fracture of lower end of right tibia, initial encounter for open fracture type I or II
        "S82391C",  # Other fracture of lower end of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82391D",  # Other fracture of lower end of right tibia, subsequent encounter for closed fracture with routine healing
        "S82391E",  # Other fracture of lower end of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82391F",  # Other fracture of lower end of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82391G",  # Other fracture of lower end of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82391H",  # Other fracture of lower end of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82391J",  # Other fracture of lower end of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82391K",  # Other fracture of lower end of right tibia, subsequent encounter for closed fracture with nonunion
        "S82391M",  # Other fracture of lower end of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82391N",  # Other fracture of lower end of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82391P",  # Other fracture of lower end of right tibia, subsequent encounter for closed fracture with malunion
        "S82391Q",  # Other fracture of lower end of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82391R",  # Other fracture of lower end of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82391S",  # Other fracture of lower end of right tibia, sequela
        "S82392A",  # Other fracture of lower end of left tibia, initial encounter for closed fracture
        "S82392B",  # Other fracture of lower end of left tibia, initial encounter for open fracture type I or II
        "S82392C",  # Other fracture of lower end of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82392D",  # Other fracture of lower end of left tibia, subsequent encounter for closed fracture with routine healing
        "S82392E",  # Other fracture of lower end of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82392F",  # Other fracture of lower end of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82392G",  # Other fracture of lower end of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82392H",  # Other fracture of lower end of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82392J",  # Other fracture of lower end of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82392K",  # Other fracture of lower end of left tibia, subsequent encounter for closed fracture with nonunion
        "S82392M",  # Other fracture of lower end of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82392N",  # Other fracture of lower end of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82392P",  # Other fracture of lower end of left tibia, subsequent encounter for closed fracture with malunion
        "S82392Q",  # Other fracture of lower end of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82392R",  # Other fracture of lower end of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82392S",  # Other fracture of lower end of left tibia, sequela
        "S82399A",  # Other fracture of lower end of unspecified tibia, initial encounter for closed fracture
        "S82399B",  # Other fracture of lower end of unspecified tibia, initial encounter for open fracture type I or II
        "S82399C",  # Other fracture of lower end of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82399D",  # Other fracture of lower end of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82399E",  # Other fracture of lower end of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82399F",  # Other fracture of lower end of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82399G",  # Other fracture of lower end of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82399H",  # Other fracture of lower end of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82399J",  # Other fracture of lower end of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82399K",  # Other fracture of lower end of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82399M",  # Other fracture of lower end of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82399N",  # Other fracture of lower end of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82399P",  # Other fracture of lower end of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82399Q",  # Other fracture of lower end of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82399R",  # Other fracture of lower end of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82399S",  # Other fracture of lower end of unspecified tibia, sequela
        "S82401A",  # Unspecified fracture of shaft of right fibula, initial encounter for closed fracture
        "S82401B",  # Unspecified fracture of shaft of right fibula, initial encounter for open fracture type I or II
        "S82401C",  # Unspecified fracture of shaft of right fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82401D",  # Unspecified fracture of shaft of right fibula, subsequent encounter for closed fracture with routine healing
        "S82401E",  # Unspecified fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82401F",  # Unspecified fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82401G",  # Unspecified fracture of shaft of right fibula, subsequent encounter for closed fracture with delayed healing
        "S82401H",  # Unspecified fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82401J",  # Unspecified fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82401K",  # Unspecified fracture of shaft of right fibula, subsequent encounter for closed fracture with nonunion
        "S82401M",  # Unspecified fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82401N",  # Unspecified fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82401P",  # Unspecified fracture of shaft of right fibula, subsequent encounter for closed fracture with malunion
        "S82401Q",  # Unspecified fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with malunion
        "S82401R",  # Unspecified fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82401S",  # Unspecified fracture of shaft of right fibula, sequela
        "S82402A",  # Unspecified fracture of shaft of left fibula, initial encounter for closed fracture
        "S82402B",  # Unspecified fracture of shaft of left fibula, initial encounter for open fracture type I or II
        "S82402C",  # Unspecified fracture of shaft of left fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82402D",  # Unspecified fracture of shaft of left fibula, subsequent encounter for closed fracture with routine healing
        "S82402E",  # Unspecified fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82402F",  # Unspecified fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82402G",  # Unspecified fracture of shaft of left fibula, subsequent encounter for closed fracture with delayed healing
        "S82402H",  # Unspecified fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82402J",  # Unspecified fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82402K",  # Unspecified fracture of shaft of left fibula, subsequent encounter for closed fracture with nonunion
        "S82402M",  # Unspecified fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82402N",  # Unspecified fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82402P",  # Unspecified fracture of shaft of left fibula, subsequent encounter for closed fracture with malunion
        "S82402Q",  # Unspecified fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with malunion
        "S82402R",  # Unspecified fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82402S",  # Unspecified fracture of shaft of left fibula, sequela
        "S82409A",  # Unspecified fracture of shaft of unspecified fibula, initial encounter for closed fracture
        "S82409B",  # Unspecified fracture of shaft of unspecified fibula, initial encounter for open fracture type I or II
        "S82409C",  # Unspecified fracture of shaft of unspecified fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82409D",  # Unspecified fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with routine healing
        "S82409E",  # Unspecified fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82409F",  # Unspecified fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82409G",  # Unspecified fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with delayed healing
        "S82409H",  # Unspecified fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82409J",  # Unspecified fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82409K",  # Unspecified fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with nonunion
        "S82409M",  # Unspecified fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82409N",  # Unspecified fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82409P",  # Unspecified fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with malunion
        "S82409Q",  # Unspecified fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with malunion
        "S82409R",  # Unspecified fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82409S",  # Unspecified fracture of shaft of unspecified fibula, sequela
        "S82421A",  # Displaced transverse fracture of shaft of right fibula, initial encounter for closed fracture
        "S82421B",  # Displaced transverse fracture of shaft of right fibula, initial encounter for open fracture type I or II
        "S82421C",  # Displaced transverse fracture of shaft of right fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82421D",  # Displaced transverse fracture of shaft of right fibula, subsequent encounter for closed fracture with routine healing
        "S82421E",  # Displaced transverse fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82421F",  # Displaced transverse fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82421G",  # Displaced transverse fracture of shaft of right fibula, subsequent encounter for closed fracture with delayed healing
        "S82421H",  # Displaced transverse fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82421J",  # Displaced transverse fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82421K",  # Displaced transverse fracture of shaft of right fibula, subsequent encounter for closed fracture with nonunion
        "S82421M",  # Displaced transverse fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82421N",  # Displaced transverse fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82421P",  # Displaced transverse fracture of shaft of right fibula, subsequent encounter for closed fracture with malunion
        "S82421Q",  # Displaced transverse fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with malunion
        "S82421R",  # Displaced transverse fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82421S",  # Displaced transverse fracture of shaft of right fibula, sequela
        "S82422A",  # Displaced transverse fracture of shaft of left fibula, initial encounter for closed fracture
        "S82422B",  # Displaced transverse fracture of shaft of left fibula, initial encounter for open fracture type I or II
        "S82422C",  # Displaced transverse fracture of shaft of left fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82422D",  # Displaced transverse fracture of shaft of left fibula, subsequent encounter for closed fracture with routine healing
        "S82422E",  # Displaced transverse fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82422F",  # Displaced transverse fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82422G",  # Displaced transverse fracture of shaft of left fibula, subsequent encounter for closed fracture with delayed healing
        "S82422H",  # Displaced transverse fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82422J",  # Displaced transverse fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82422K",  # Displaced transverse fracture of shaft of left fibula, subsequent encounter for closed fracture with nonunion
        "S82422M",  # Displaced transverse fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82422N",  # Displaced transverse fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82422P",  # Displaced transverse fracture of shaft of left fibula, subsequent encounter for closed fracture with malunion
        "S82422Q",  # Displaced transverse fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with malunion
        "S82422R",  # Displaced transverse fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82422S",  # Displaced transverse fracture of shaft of left fibula, sequela
        "S82423A",  # Displaced transverse fracture of shaft of unspecified fibula, initial encounter for closed fracture
        "S82423B",  # Displaced transverse fracture of shaft of unspecified fibula, initial encounter for open fracture type I or II
        "S82423C",  # Displaced transverse fracture of shaft of unspecified fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82423D",  # Displaced transverse fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with routine healing
        "S82423E",  # Displaced transverse fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82423F",  # Displaced transverse fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82423G",  # Displaced transverse fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with delayed healing
        "S82423H",  # Displaced transverse fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82423J",  # Displaced transverse fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82423K",  # Displaced transverse fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with nonunion
        "S82423M",  # Displaced transverse fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82423N",  # Displaced transverse fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82423P",  # Displaced transverse fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with malunion
        "S82423Q",  # Displaced transverse fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with malunion
        "S82423R",  # Displaced transverse fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82423S",  # Displaced transverse fracture of shaft of unspecified fibula, sequela
        "S82424A",  # Nondisplaced transverse fracture of shaft of right fibula, initial encounter for closed fracture
        "S82424B",  # Nondisplaced transverse fracture of shaft of right fibula, initial encounter for open fracture type I or II
        "S82424C",  # Nondisplaced transverse fracture of shaft of right fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82424D",  # Nondisplaced transverse fracture of shaft of right fibula, subsequent encounter for closed fracture with routine healing
        "S82424E",  # Nondisplaced transverse fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82424F",  # Nondisplaced transverse fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82424G",  # Nondisplaced transverse fracture of shaft of right fibula, subsequent encounter for closed fracture with delayed healing
        "S82424H",  # Nondisplaced transverse fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82424J",  # Nondisplaced transverse fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82424K",  # Nondisplaced transverse fracture of shaft of right fibula, subsequent encounter for closed fracture with nonunion
        "S82424M",  # Nondisplaced transverse fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82424N",  # Nondisplaced transverse fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82424P",  # Nondisplaced transverse fracture of shaft of right fibula, subsequent encounter for closed fracture with malunion
        "S82424Q",  # Nondisplaced transverse fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with malunion
        "S82424R",  # Nondisplaced transverse fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82424S",  # Nondisplaced transverse fracture of shaft of right fibula, sequela
        "S82425A",  # Nondisplaced transverse fracture of shaft of left fibula, initial encounter for closed fracture
        "S82425B",  # Nondisplaced transverse fracture of shaft of left fibula, initial encounter for open fracture type I or II
        "S82425C",  # Nondisplaced transverse fracture of shaft of left fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82425D",  # Nondisplaced transverse fracture of shaft of left fibula, subsequent encounter for closed fracture with routine healing
        "S82425E",  # Nondisplaced transverse fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82425F",  # Nondisplaced transverse fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82425G",  # Nondisplaced transverse fracture of shaft of left fibula, subsequent encounter for closed fracture with delayed healing
        "S82425H",  # Nondisplaced transverse fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82425J",  # Nondisplaced transverse fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82425K",  # Nondisplaced transverse fracture of shaft of left fibula, subsequent encounter for closed fracture with nonunion
        "S82425M",  # Nondisplaced transverse fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82425N",  # Nondisplaced transverse fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82425P",  # Nondisplaced transverse fracture of shaft of left fibula, subsequent encounter for closed fracture with malunion
        "S82425Q",  # Nondisplaced transverse fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with malunion
        "S82425R",  # Nondisplaced transverse fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82425S",  # Nondisplaced transverse fracture of shaft of left fibula, sequela
        "S82426A",  # Nondisplaced transverse fracture of shaft of unspecified fibula, initial encounter for closed fracture
        "S82426B",  # Nondisplaced transverse fracture of shaft of unspecified fibula, initial encounter for open fracture type I or II
        "S82426C",  # Nondisplaced transverse fracture of shaft of unspecified fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82426D",  # Nondisplaced transverse fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with routine healing
        "S82426E",  # Nondisplaced transverse fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82426F",  # Nondisplaced transverse fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82426G",  # Nondisplaced transverse fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with delayed healing
        "S82426H",  # Nondisplaced transverse fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82426J",  # Nondisplaced transverse fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82426K",  # Nondisplaced transverse fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with nonunion
        "S82426M",  # Nondisplaced transverse fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82426N",  # Nondisplaced transverse fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82426P",  # Nondisplaced transverse fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with malunion
        "S82426Q",  # Nondisplaced transverse fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with malunion
        "S82426R",  # Nondisplaced transverse fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82426S",  # Nondisplaced transverse fracture of shaft of unspecified fibula, sequela
        "S82431A",  # Displaced oblique fracture of shaft of right fibula, initial encounter for closed fracture
        "S82431B",  # Displaced oblique fracture of shaft of right fibula, initial encounter for open fracture type I or II
        "S82431C",  # Displaced oblique fracture of shaft of right fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82431D",  # Displaced oblique fracture of shaft of right fibula, subsequent encounter for closed fracture with routine healing
        "S82431E",  # Displaced oblique fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82431F",  # Displaced oblique fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82431G",  # Displaced oblique fracture of shaft of right fibula, subsequent encounter for closed fracture with delayed healing
        "S82431H",  # Displaced oblique fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82431J",  # Displaced oblique fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82431K",  # Displaced oblique fracture of shaft of right fibula, subsequent encounter for closed fracture with nonunion
        "S82431M",  # Displaced oblique fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82431N",  # Displaced oblique fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82431P",  # Displaced oblique fracture of shaft of right fibula, subsequent encounter for closed fracture with malunion
        "S82431Q",  # Displaced oblique fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with malunion
        "S82431R",  # Displaced oblique fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82431S",  # Displaced oblique fracture of shaft of right fibula, sequela
        "S82432A",  # Displaced oblique fracture of shaft of left fibula, initial encounter for closed fracture
        "S82432B",  # Displaced oblique fracture of shaft of left fibula, initial encounter for open fracture type I or II
        "S82432C",  # Displaced oblique fracture of shaft of left fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82432D",  # Displaced oblique fracture of shaft of left fibula, subsequent encounter for closed fracture with routine healing
        "S82432E",  # Displaced oblique fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82432F",  # Displaced oblique fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82432G",  # Displaced oblique fracture of shaft of left fibula, subsequent encounter for closed fracture with delayed healing
        "S82432H",  # Displaced oblique fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82432J",  # Displaced oblique fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82432K",  # Displaced oblique fracture of shaft of left fibula, subsequent encounter for closed fracture with nonunion
        "S82432M",  # Displaced oblique fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82432N",  # Displaced oblique fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82432P",  # Displaced oblique fracture of shaft of left fibula, subsequent encounter for closed fracture with malunion
        "S82432Q",  # Displaced oblique fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with malunion
        "S82432R",  # Displaced oblique fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82432S",  # Displaced oblique fracture of shaft of left fibula, sequela
        "S82433A",  # Displaced oblique fracture of shaft of unspecified fibula, initial encounter for closed fracture
        "S82433B",  # Displaced oblique fracture of shaft of unspecified fibula, initial encounter for open fracture type I or II
        "S82433C",  # Displaced oblique fracture of shaft of unspecified fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82433D",  # Displaced oblique fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with routine healing
        "S82433E",  # Displaced oblique fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82433F",  # Displaced oblique fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82433G",  # Displaced oblique fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with delayed healing
        "S82433H",  # Displaced oblique fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82433J",  # Displaced oblique fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82433K",  # Displaced oblique fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with nonunion
        "S82433M",  # Displaced oblique fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82433N",  # Displaced oblique fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82433P",  # Displaced oblique fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with malunion
        "S82433Q",  # Displaced oblique fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with malunion
        "S82433R",  # Displaced oblique fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82433S",  # Displaced oblique fracture of shaft of unspecified fibula, sequela
        "S82434A",  # Nondisplaced oblique fracture of shaft of right fibula, initial encounter for closed fracture
        "S82434B",  # Nondisplaced oblique fracture of shaft of right fibula, initial encounter for open fracture type I or II
        "S82434C",  # Nondisplaced oblique fracture of shaft of right fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82434D",  # Nondisplaced oblique fracture of shaft of right fibula, subsequent encounter for closed fracture with routine healing
        "S82434E",  # Nondisplaced oblique fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82434F",  # Nondisplaced oblique fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82434G",  # Nondisplaced oblique fracture of shaft of right fibula, subsequent encounter for closed fracture with delayed healing
        "S82434H",  # Nondisplaced oblique fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82434J",  # Nondisplaced oblique fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82434K",  # Nondisplaced oblique fracture of shaft of right fibula, subsequent encounter for closed fracture with nonunion
        "S82434M",  # Nondisplaced oblique fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82434N",  # Nondisplaced oblique fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82434P",  # Nondisplaced oblique fracture of shaft of right fibula, subsequent encounter for closed fracture with malunion
        "S82434Q",  # Nondisplaced oblique fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with malunion
        "S82434R",  # Nondisplaced oblique fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82434S",  # Nondisplaced oblique fracture of shaft of right fibula, sequela
        "S82435A",  # Nondisplaced oblique fracture of shaft of left fibula, initial encounter for closed fracture
        "S82435B",  # Nondisplaced oblique fracture of shaft of left fibula, initial encounter for open fracture type I or II
        "S82435C",  # Nondisplaced oblique fracture of shaft of left fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82435D",  # Nondisplaced oblique fracture of shaft of left fibula, subsequent encounter for closed fracture with routine healing
        "S82435E",  # Nondisplaced oblique fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82435F",  # Nondisplaced oblique fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82435G",  # Nondisplaced oblique fracture of shaft of left fibula, subsequent encounter for closed fracture with delayed healing
        "S82435H",  # Nondisplaced oblique fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82435J",  # Nondisplaced oblique fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82435K",  # Nondisplaced oblique fracture of shaft of left fibula, subsequent encounter for closed fracture with nonunion
        "S82435M",  # Nondisplaced oblique fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82435N",  # Nondisplaced oblique fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82435P",  # Nondisplaced oblique fracture of shaft of left fibula, subsequent encounter for closed fracture with malunion
        "S82435Q",  # Nondisplaced oblique fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with malunion
        "S82435R",  # Nondisplaced oblique fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82435S",  # Nondisplaced oblique fracture of shaft of left fibula, sequela
        "S82436A",  # Nondisplaced oblique fracture of shaft of unspecified fibula, initial encounter for closed fracture
        "S82436B",  # Nondisplaced oblique fracture of shaft of unspecified fibula, initial encounter for open fracture type I or II
        "S82436C",  # Nondisplaced oblique fracture of shaft of unspecified fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82436D",  # Nondisplaced oblique fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with routine healing
        "S82436E",  # Nondisplaced oblique fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82436F",  # Nondisplaced oblique fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82436G",  # Nondisplaced oblique fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with delayed healing
        "S82436H",  # Nondisplaced oblique fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82436J",  # Nondisplaced oblique fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82436K",  # Nondisplaced oblique fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with nonunion
        "S82436M",  # Nondisplaced oblique fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82436N",  # Nondisplaced oblique fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82436P",  # Nondisplaced oblique fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with malunion
        "S82436Q",  # Nondisplaced oblique fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with malunion
        "S82436R",  # Nondisplaced oblique fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82436S",  # Nondisplaced oblique fracture of shaft of unspecified fibula, sequela
        "S82441A",  # Displaced spiral fracture of shaft of right fibula, initial encounter for closed fracture
        "S82441B",  # Displaced spiral fracture of shaft of right fibula, initial encounter for open fracture type I or II
        "S82441C",  # Displaced spiral fracture of shaft of right fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82441D",  # Displaced spiral fracture of shaft of right fibula, subsequent encounter for closed fracture with routine healing
        "S82441E",  # Displaced spiral fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82441F",  # Displaced spiral fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82441G",  # Displaced spiral fracture of shaft of right fibula, subsequent encounter for closed fracture with delayed healing
        "S82441H",  # Displaced spiral fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82441J",  # Displaced spiral fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82441K",  # Displaced spiral fracture of shaft of right fibula, subsequent encounter for closed fracture with nonunion
        "S82441M",  # Displaced spiral fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82441N",  # Displaced spiral fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82441P",  # Displaced spiral fracture of shaft of right fibula, subsequent encounter for closed fracture with malunion
        "S82441Q",  # Displaced spiral fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with malunion
        "S82441R",  # Displaced spiral fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82441S",  # Displaced spiral fracture of shaft of right fibula, sequela
        "S82442A",  # Displaced spiral fracture of shaft of left fibula, initial encounter for closed fracture
        "S82442B",  # Displaced spiral fracture of shaft of left fibula, initial encounter for open fracture type I or II
        "S82442C",  # Displaced spiral fracture of shaft of left fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82442D",  # Displaced spiral fracture of shaft of left fibula, subsequent encounter for closed fracture with routine healing
        "S82442E",  # Displaced spiral fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82442F",  # Displaced spiral fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82442G",  # Displaced spiral fracture of shaft of left fibula, subsequent encounter for closed fracture with delayed healing
        "S82442H",  # Displaced spiral fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82442J",  # Displaced spiral fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82442K",  # Displaced spiral fracture of shaft of left fibula, subsequent encounter for closed fracture with nonunion
        "S82442M",  # Displaced spiral fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82442N",  # Displaced spiral fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82442P",  # Displaced spiral fracture of shaft of left fibula, subsequent encounter for closed fracture with malunion
        "S82442Q",  # Displaced spiral fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with malunion
        "S82442R",  # Displaced spiral fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82442S",  # Displaced spiral fracture of shaft of left fibula, sequela
        "S82443A",  # Displaced spiral fracture of shaft of unspecified fibula, initial encounter for closed fracture
        "S82443B",  # Displaced spiral fracture of shaft of unspecified fibula, initial encounter for open fracture type I or II
        "S82443C",  # Displaced spiral fracture of shaft of unspecified fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82443D",  # Displaced spiral fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with routine healing
        "S82443E",  # Displaced spiral fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82443F",  # Displaced spiral fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82443G",  # Displaced spiral fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with delayed healing
        "S82443H",  # Displaced spiral fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82443J",  # Displaced spiral fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82443K",  # Displaced spiral fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with nonunion
        "S82443M",  # Displaced spiral fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82443N",  # Displaced spiral fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82443P",  # Displaced spiral fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with malunion
        "S82443Q",  # Displaced spiral fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with malunion
        "S82443R",  # Displaced spiral fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82443S",  # Displaced spiral fracture of shaft of unspecified fibula, sequela
        "S82444A",  # Nondisplaced spiral fracture of shaft of right fibula, initial encounter for closed fracture
        "S82444B",  # Nondisplaced spiral fracture of shaft of right fibula, initial encounter for open fracture type I or II
        "S82444C",  # Nondisplaced spiral fracture of shaft of right fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82444D",  # Nondisplaced spiral fracture of shaft of right fibula, subsequent encounter for closed fracture with routine healing
        "S82444E",  # Nondisplaced spiral fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82444F",  # Nondisplaced spiral fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82444G",  # Nondisplaced spiral fracture of shaft of right fibula, subsequent encounter for closed fracture with delayed healing
        "S82444H",  # Nondisplaced spiral fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82444J",  # Nondisplaced spiral fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82444K",  # Nondisplaced spiral fracture of shaft of right fibula, subsequent encounter for closed fracture with nonunion
        "S82444M",  # Nondisplaced spiral fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82444N",  # Nondisplaced spiral fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82444P",  # Nondisplaced spiral fracture of shaft of right fibula, subsequent encounter for closed fracture with malunion
        "S82444Q",  # Nondisplaced spiral fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with malunion
        "S82444R",  # Nondisplaced spiral fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82444S",  # Nondisplaced spiral fracture of shaft of right fibula, sequela
        "S82445A",  # Nondisplaced spiral fracture of shaft of left fibula, initial encounter for closed fracture
        "S82445B",  # Nondisplaced spiral fracture of shaft of left fibula, initial encounter for open fracture type I or II
        "S82445C",  # Nondisplaced spiral fracture of shaft of left fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82445D",  # Nondisplaced spiral fracture of shaft of left fibula, subsequent encounter for closed fracture with routine healing
        "S82445E",  # Nondisplaced spiral fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82445F",  # Nondisplaced spiral fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82445G",  # Nondisplaced spiral fracture of shaft of left fibula, subsequent encounter for closed fracture with delayed healing
        "S82445H",  # Nondisplaced spiral fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82445J",  # Nondisplaced spiral fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82445K",  # Nondisplaced spiral fracture of shaft of left fibula, subsequent encounter for closed fracture with nonunion
        "S82445M",  # Nondisplaced spiral fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82445N",  # Nondisplaced spiral fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82445P",  # Nondisplaced spiral fracture of shaft of left fibula, subsequent encounter for closed fracture with malunion
        "S82445Q",  # Nondisplaced spiral fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with malunion
        "S82445R",  # Nondisplaced spiral fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82445S",  # Nondisplaced spiral fracture of shaft of left fibula, sequela
        "S82446A",  # Nondisplaced spiral fracture of shaft of unspecified fibula, initial encounter for closed fracture
        "S82446B",  # Nondisplaced spiral fracture of shaft of unspecified fibula, initial encounter for open fracture type I or II
        "S82446C",  # Nondisplaced spiral fracture of shaft of unspecified fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82446D",  # Nondisplaced spiral fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with routine healing
        "S82446E",  # Nondisplaced spiral fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82446F",  # Nondisplaced spiral fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82446G",  # Nondisplaced spiral fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with delayed healing
        "S82446H",  # Nondisplaced spiral fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82446J",  # Nondisplaced spiral fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82446K",  # Nondisplaced spiral fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with nonunion
        "S82446M",  # Nondisplaced spiral fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82446N",  # Nondisplaced spiral fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82446P",  # Nondisplaced spiral fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with malunion
        "S82446Q",  # Nondisplaced spiral fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with malunion
        "S82446R",  # Nondisplaced spiral fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82446S",  # Nondisplaced spiral fracture of shaft of unspecified fibula, sequela
        "S82451A",  # Displaced comminuted fracture of shaft of right fibula, initial encounter for closed fracture
        "S82451B",  # Displaced comminuted fracture of shaft of right fibula, initial encounter for open fracture type I or II
        "S82451C",  # Displaced comminuted fracture of shaft of right fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82451D",  # Displaced comminuted fracture of shaft of right fibula, subsequent encounter for closed fracture with routine healing
        "S82451E",  # Displaced comminuted fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82451F",  # Displaced comminuted fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82451G",  # Displaced comminuted fracture of shaft of right fibula, subsequent encounter for closed fracture with delayed healing
        "S82451H",  # Displaced comminuted fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82451J",  # Displaced comminuted fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82451K",  # Displaced comminuted fracture of shaft of right fibula, subsequent encounter for closed fracture with nonunion
        "S82451M",  # Displaced comminuted fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82451N",  # Displaced comminuted fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82451P",  # Displaced comminuted fracture of shaft of right fibula, subsequent encounter for closed fracture with malunion
        "S82451Q",  # Displaced comminuted fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with malunion
        "S82451R",  # Displaced comminuted fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82451S",  # Displaced comminuted fracture of shaft of right fibula, sequela
        "S82452A",  # Displaced comminuted fracture of shaft of left fibula, initial encounter for closed fracture
        "S82452B",  # Displaced comminuted fracture of shaft of left fibula, initial encounter for open fracture type I or II
        "S82452C",  # Displaced comminuted fracture of shaft of left fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82452D",  # Displaced comminuted fracture of shaft of left fibula, subsequent encounter for closed fracture with routine healing
        "S82452E",  # Displaced comminuted fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82452F",  # Displaced comminuted fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82452G",  # Displaced comminuted fracture of shaft of left fibula, subsequent encounter for closed fracture with delayed healing
        "S82452H",  # Displaced comminuted fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82452J",  # Displaced comminuted fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82452K",  # Displaced comminuted fracture of shaft of left fibula, subsequent encounter for closed fracture with nonunion
        "S82452M",  # Displaced comminuted fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82452N",  # Displaced comminuted fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82452P",  # Displaced comminuted fracture of shaft of left fibula, subsequent encounter for closed fracture with malunion
        "S82452Q",  # Displaced comminuted fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with malunion
        "S82452R",  # Displaced comminuted fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82452S",  # Displaced comminuted fracture of shaft of left fibula, sequela
        "S82453A",  # Displaced comminuted fracture of shaft of unspecified fibula, initial encounter for closed fracture
        "S82453B",  # Displaced comminuted fracture of shaft of unspecified fibula, initial encounter for open fracture type I or II
        "S82453C",  # Displaced comminuted fracture of shaft of unspecified fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82453D",  # Displaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with routine healing
        "S82453E",  # Displaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82453F",  # Displaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82453G",  # Displaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with delayed healing
        "S82453H",  # Displaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82453J",  # Displaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82453K",  # Displaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with nonunion
        "S82453M",  # Displaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82453N",  # Displaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82453P",  # Displaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with malunion
        "S82453Q",  # Displaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with malunion
        "S82453R",  # Displaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82453S",  # Displaced comminuted fracture of shaft of unspecified fibula, sequela
        "S82454A",  # Nondisplaced comminuted fracture of shaft of right fibula, initial encounter for closed fracture
        "S82454B",  # Nondisplaced comminuted fracture of shaft of right fibula, initial encounter for open fracture type I or II
        "S82454C",  # Nondisplaced comminuted fracture of shaft of right fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82454D",  # Nondisplaced comminuted fracture of shaft of right fibula, subsequent encounter for closed fracture with routine healing
        "S82454E",  # Nondisplaced comminuted fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82454F",  # Nondisplaced comminuted fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82454G",  # Nondisplaced comminuted fracture of shaft of right fibula, subsequent encounter for closed fracture with delayed healing
        "S82454H",  # Nondisplaced comminuted fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82454J",  # Nondisplaced comminuted fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82454K",  # Nondisplaced comminuted fracture of shaft of right fibula, subsequent encounter for closed fracture with nonunion
        "S82454M",  # Nondisplaced comminuted fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82454N",  # Nondisplaced comminuted fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82454P",  # Nondisplaced comminuted fracture of shaft of right fibula, subsequent encounter for closed fracture with malunion
        "S82454Q",  # Nondisplaced comminuted fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with malunion
        "S82454R",  # Nondisplaced comminuted fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82454S",  # Nondisplaced comminuted fracture of shaft of right fibula, sequela
        "S82455A",  # Nondisplaced comminuted fracture of shaft of left fibula, initial encounter for closed fracture
        "S82455B",  # Nondisplaced comminuted fracture of shaft of left fibula, initial encounter for open fracture type I or II
        "S82455C",  # Nondisplaced comminuted fracture of shaft of left fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82455D",  # Nondisplaced comminuted fracture of shaft of left fibula, subsequent encounter for closed fracture with routine healing
        "S82455E",  # Nondisplaced comminuted fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82455F",  # Nondisplaced comminuted fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82455G",  # Nondisplaced comminuted fracture of shaft of left fibula, subsequent encounter for closed fracture with delayed healing
        "S82455H",  # Nondisplaced comminuted fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82455J",  # Nondisplaced comminuted fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82455K",  # Nondisplaced comminuted fracture of shaft of left fibula, subsequent encounter for closed fracture with nonunion
        "S82455M",  # Nondisplaced comminuted fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82455N",  # Nondisplaced comminuted fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82455P",  # Nondisplaced comminuted fracture of shaft of left fibula, subsequent encounter for closed fracture with malunion
        "S82455Q",  # Nondisplaced comminuted fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with malunion
        "S82455R",  # Nondisplaced comminuted fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82455S",  # Nondisplaced comminuted fracture of shaft of left fibula, sequela
        "S82456A",  # Nondisplaced comminuted fracture of shaft of unspecified fibula, initial encounter for closed fracture
        "S82456B",  # Nondisplaced comminuted fracture of shaft of unspecified fibula, initial encounter for open fracture type I or II
        "S82456C",  # Nondisplaced comminuted fracture of shaft of unspecified fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82456D",  # Nondisplaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with routine healing
        "S82456E",  # Nondisplaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82456F",  # Nondisplaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82456G",  # Nondisplaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with delayed healing
        "S82456H",  # Nondisplaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82456J",  # Nondisplaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82456K",  # Nondisplaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with nonunion
        "S82456M",  # Nondisplaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82456N",  # Nondisplaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82456P",  # Nondisplaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with malunion
        "S82456Q",  # Nondisplaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with malunion
        "S82456R",  # Nondisplaced comminuted fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82456S",  # Nondisplaced comminuted fracture of shaft of unspecified fibula, sequela
        "S82461A",  # Displaced segmental fracture of shaft of right fibula, initial encounter for closed fracture
        "S82461B",  # Displaced segmental fracture of shaft of right fibula, initial encounter for open fracture type I or II
        "S82461C",  # Displaced segmental fracture of shaft of right fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82461D",  # Displaced segmental fracture of shaft of right fibula, subsequent encounter for closed fracture with routine healing
        "S82461E",  # Displaced segmental fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82461F",  # Displaced segmental fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82461G",  # Displaced segmental fracture of shaft of right fibula, subsequent encounter for closed fracture with delayed healing
        "S82461H",  # Displaced segmental fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82461J",  # Displaced segmental fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82461K",  # Displaced segmental fracture of shaft of right fibula, subsequent encounter for closed fracture with nonunion
        "S82461M",  # Displaced segmental fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82461N",  # Displaced segmental fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82461P",  # Displaced segmental fracture of shaft of right fibula, subsequent encounter for closed fracture with malunion
        "S82461Q",  # Displaced segmental fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with malunion
        "S82461R",  # Displaced segmental fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82461S",  # Displaced segmental fracture of shaft of right fibula, sequela
        "S82462A",  # Displaced segmental fracture of shaft of left fibula, initial encounter for closed fracture
        "S82462B",  # Displaced segmental fracture of shaft of left fibula, initial encounter for open fracture type I or II
        "S82462C",  # Displaced segmental fracture of shaft of left fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82462D",  # Displaced segmental fracture of shaft of left fibula, subsequent encounter for closed fracture with routine healing
        "S82462E",  # Displaced segmental fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82462F",  # Displaced segmental fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82462G",  # Displaced segmental fracture of shaft of left fibula, subsequent encounter for closed fracture with delayed healing
        "S82462H",  # Displaced segmental fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82462J",  # Displaced segmental fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82462K",  # Displaced segmental fracture of shaft of left fibula, subsequent encounter for closed fracture with nonunion
        "S82462M",  # Displaced segmental fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82462N",  # Displaced segmental fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82462P",  # Displaced segmental fracture of shaft of left fibula, subsequent encounter for closed fracture with malunion
        "S82462Q",  # Displaced segmental fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with malunion
        "S82462R",  # Displaced segmental fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82462S",  # Displaced segmental fracture of shaft of left fibula, sequela
        "S82463A",  # Displaced segmental fracture of shaft of unspecified fibula, initial encounter for closed fracture
        "S82463B",  # Displaced segmental fracture of shaft of unspecified fibula, initial encounter for open fracture type I or II
        "S82463C",  # Displaced segmental fracture of shaft of unspecified fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82463D",  # Displaced segmental fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with routine healing
        "S82463E",  # Displaced segmental fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82463F",  # Displaced segmental fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82463G",  # Displaced segmental fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with delayed healing
        "S82463H",  # Displaced segmental fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82463J",  # Displaced segmental fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82463K",  # Displaced segmental fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with nonunion
        "S82463M",  # Displaced segmental fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82463N",  # Displaced segmental fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82463P",  # Displaced segmental fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with malunion
        "S82463Q",  # Displaced segmental fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with malunion
        "S82463R",  # Displaced segmental fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82463S",  # Displaced segmental fracture of shaft of unspecified fibula, sequela
        "S82464A",  # Nondisplaced segmental fracture of shaft of right fibula, initial encounter for closed fracture
        "S82464B",  # Nondisplaced segmental fracture of shaft of right fibula, initial encounter for open fracture type I or II
        "S82464C",  # Nondisplaced segmental fracture of shaft of right fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82464D",  # Nondisplaced segmental fracture of shaft of right fibula, subsequent encounter for closed fracture with routine healing
        "S82464E",  # Nondisplaced segmental fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82464F",  # Nondisplaced segmental fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82464G",  # Nondisplaced segmental fracture of shaft of right fibula, subsequent encounter for closed fracture with delayed healing
        "S82464H",  # Nondisplaced segmental fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82464J",  # Nondisplaced segmental fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82464K",  # Nondisplaced segmental fracture of shaft of right fibula, subsequent encounter for closed fracture with nonunion
        "S82464M",  # Nondisplaced segmental fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82464N",  # Nondisplaced segmental fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82464P",  # Nondisplaced segmental fracture of shaft of right fibula, subsequent encounter for closed fracture with malunion
        "S82464Q",  # Nondisplaced segmental fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with malunion
        "S82464R",  # Nondisplaced segmental fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82464S",  # Nondisplaced segmental fracture of shaft of right fibula, sequela
        "S82465A",  # Nondisplaced segmental fracture of shaft of left fibula, initial encounter for closed fracture
        "S82465B",  # Nondisplaced segmental fracture of shaft of left fibula, initial encounter for open fracture type I or II
        "S82465C",  # Nondisplaced segmental fracture of shaft of left fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82465D",  # Nondisplaced segmental fracture of shaft of left fibula, subsequent encounter for closed fracture with routine healing
        "S82465E",  # Nondisplaced segmental fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82465F",  # Nondisplaced segmental fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82465G",  # Nondisplaced segmental fracture of shaft of left fibula, subsequent encounter for closed fracture with delayed healing
        "S82465H",  # Nondisplaced segmental fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82465J",  # Nondisplaced segmental fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82465K",  # Nondisplaced segmental fracture of shaft of left fibula, subsequent encounter for closed fracture with nonunion
        "S82465M",  # Nondisplaced segmental fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82465N",  # Nondisplaced segmental fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82465P",  # Nondisplaced segmental fracture of shaft of left fibula, subsequent encounter for closed fracture with malunion
        "S82465Q",  # Nondisplaced segmental fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with malunion
        "S82465R",  # Nondisplaced segmental fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82465S",  # Nondisplaced segmental fracture of shaft of left fibula, sequela
        "S82466A",  # Nondisplaced segmental fracture of shaft of unspecified fibula, initial encounter for closed fracture
        "S82466B",  # Nondisplaced segmental fracture of shaft of unspecified fibula, initial encounter for open fracture type I or II
        "S82466C",  # Nondisplaced segmental fracture of shaft of unspecified fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82466D",  # Nondisplaced segmental fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with routine healing
        "S82466E",  # Nondisplaced segmental fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82466F",  # Nondisplaced segmental fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82466G",  # Nondisplaced segmental fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with delayed healing
        "S82466H",  # Nondisplaced segmental fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82466J",  # Nondisplaced segmental fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82466K",  # Nondisplaced segmental fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with nonunion
        "S82466M",  # Nondisplaced segmental fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82466N",  # Nondisplaced segmental fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82466P",  # Nondisplaced segmental fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with malunion
        "S82466Q",  # Nondisplaced segmental fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with malunion
        "S82466R",  # Nondisplaced segmental fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82466S",  # Nondisplaced segmental fracture of shaft of unspecified fibula, sequela
        "S82491A",  # Other fracture of shaft of right fibula, initial encounter for closed fracture
        "S82491B",  # Other fracture of shaft of right fibula, initial encounter for open fracture type I or II
        "S82491C",  # Other fracture of shaft of right fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82491D",  # Other fracture of shaft of right fibula, subsequent encounter for closed fracture with routine healing
        "S82491E",  # Other fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82491F",  # Other fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82491G",  # Other fracture of shaft of right fibula, subsequent encounter for closed fracture with delayed healing
        "S82491H",  # Other fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82491J",  # Other fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82491K",  # Other fracture of shaft of right fibula, subsequent encounter for closed fracture with nonunion
        "S82491M",  # Other fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82491N",  # Other fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82491P",  # Other fracture of shaft of right fibula, subsequent encounter for closed fracture with malunion
        "S82491Q",  # Other fracture of shaft of right fibula, subsequent encounter for open fracture type I or II with malunion
        "S82491R",  # Other fracture of shaft of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82491S",  # Other fracture of shaft of right fibula, sequela
        "S82492A",  # Other fracture of shaft of left fibula, initial encounter for closed fracture
        "S82492B",  # Other fracture of shaft of left fibula, initial encounter for open fracture type I or II
        "S82492C",  # Other fracture of shaft of left fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82492D",  # Other fracture of shaft of left fibula, subsequent encounter for closed fracture with routine healing
        "S82492E",  # Other fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82492F",  # Other fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82492G",  # Other fracture of shaft of left fibula, subsequent encounter for closed fracture with delayed healing
        "S82492H",  # Other fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82492J",  # Other fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82492K",  # Other fracture of shaft of left fibula, subsequent encounter for closed fracture with nonunion
        "S82492M",  # Other fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82492N",  # Other fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82492P",  # Other fracture of shaft of left fibula, subsequent encounter for closed fracture with malunion
        "S82492Q",  # Other fracture of shaft of left fibula, subsequent encounter for open fracture type I or II with malunion
        "S82492R",  # Other fracture of shaft of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82492S",  # Other fracture of shaft of left fibula, sequela
        "S82499A",  # Other fracture of shaft of unspecified fibula, initial encounter for closed fracture
        "S82499B",  # Other fracture of shaft of unspecified fibula, initial encounter for open fracture type I or II
        "S82499C",  # Other fracture of shaft of unspecified fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82499D",  # Other fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with routine healing
        "S82499E",  # Other fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82499F",  # Other fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82499G",  # Other fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with delayed healing
        "S82499H",  # Other fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82499J",  # Other fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82499K",  # Other fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with nonunion
        "S82499M",  # Other fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82499N",  # Other fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82499P",  # Other fracture of shaft of unspecified fibula, subsequent encounter for closed fracture with malunion
        "S82499Q",  # Other fracture of shaft of unspecified fibula, subsequent encounter for open fracture type I or II with malunion
        "S82499R",  # Other fracture of shaft of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82499S",  # Other fracture of shaft of unspecified fibula, sequela
        "S82811A",  # Torus fracture of upper end of right fibula, initial encounter for closed fracture
        "S82811D",  # Torus fracture of upper end of right fibula, subsequent encounter for fracture with routine healing
        "S82811G",  # Torus fracture of upper end of right fibula, subsequent encounter for fracture with delayed healing
        "S82811K",  # Torus fracture of upper end of right fibula, subsequent encounter for fracture with nonunion
        "S82811P",  # Torus fracture of upper end of right fibula, subsequent encounter for fracture with malunion
        "S82811S",  # Torus fracture of upper end of right fibula, sequela
        "S82812A",  # Torus fracture of upper end of left fibula, initial encounter for closed fracture
        "S82812D",  # Torus fracture of upper end of left fibula, subsequent encounter for fracture with routine healing
        "S82812G",  # Torus fracture of upper end of left fibula, subsequent encounter for fracture with delayed healing
        "S82812K",  # Torus fracture of upper end of left fibula, subsequent encounter for fracture with nonunion
        "S82812P",  # Torus fracture of upper end of left fibula, subsequent encounter for fracture with malunion
        "S82812S",  # Torus fracture of upper end of left fibula, sequela
        "S82819A",  # Torus fracture of upper end of unspecified fibula, initial encounter for closed fracture
        "S82819D",  # Torus fracture of upper end of unspecified fibula, subsequent encounter for fracture with routine healing
        "S82819G",  # Torus fracture of upper end of unspecified fibula, subsequent encounter for fracture with delayed healing
        "S82819K",  # Torus fracture of upper end of unspecified fibula, subsequent encounter for fracture with nonunion
        "S82819P",  # Torus fracture of upper end of unspecified fibula, subsequent encounter for fracture with malunion
        "S82819S",  # Torus fracture of upper end of unspecified fibula, sequela
        "S82821A",  # Torus fracture of lower end of right fibula, initial encounter for closed fracture
        "S82821D",  # Torus fracture of lower end of right fibula, subsequent encounter for fracture with routine healing
        "S82821G",  # Torus fracture of lower end of right fibula, subsequent encounter for fracture with delayed healing
        "S82821K",  # Torus fracture of lower end of right fibula, subsequent encounter for fracture with nonunion
        "S82821P",  # Torus fracture of lower end of right fibula, subsequent encounter for fracture with malunion
        "S82821S",  # Torus fracture of lower end of right fibula, sequela
        "S82822A",  # Torus fracture of lower end of left fibula, initial encounter for closed fracture
        "S82822D",  # Torus fracture of lower end of left fibula, subsequent encounter for fracture with routine healing
        "S82822G",  # Torus fracture of lower end of left fibula, subsequent encounter for fracture with delayed healing
        "S82822K",  # Torus fracture of lower end of left fibula, subsequent encounter for fracture with nonunion
        "S82822P",  # Torus fracture of lower end of left fibula, subsequent encounter for fracture with malunion
        "S82822S",  # Torus fracture of lower end of left fibula, sequela
        "S82829A",  # Torus fracture of lower end of unspecified fibula, initial encounter for closed fracture
        "S82829D",  # Torus fracture of lower end of unspecified fibula, subsequent encounter for fracture with routine healing
        "S82829G",  # Torus fracture of lower end of unspecified fibula, subsequent encounter for fracture with delayed healing
        "S82829K",  # Torus fracture of lower end of unspecified fibula, subsequent encounter for fracture with nonunion
        "S82829P",  # Torus fracture of lower end of unspecified fibula, subsequent encounter for fracture with malunion
        "S82829S",  # Torus fracture of lower end of unspecified fibula, sequela
        "S82831A",  # Other fracture of upper and lower end of right fibula, initial encounter for closed fracture
        "S82831B",  # Other fracture of upper and lower end of right fibula, initial encounter for open fracture type I or II
        "S82831C",  # Other fracture of upper and lower end of right fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82831D",  # Other fracture of upper and lower end of right fibula, subsequent encounter for closed fracture with routine healing
        "S82831E",  # Other fracture of upper and lower end of right fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82831F",  # Other fracture of upper and lower end of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82831G",  # Other fracture of upper and lower end of right fibula, subsequent encounter for closed fracture with delayed healing
        "S82831H",  # Other fracture of upper and lower end of right fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82831J",  # Other fracture of upper and lower end of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82831K",  # Other fracture of upper and lower end of right fibula, subsequent encounter for closed fracture with nonunion
        "S82831M",  # Other fracture of upper and lower end of right fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82831N",  # Other fracture of upper and lower end of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82831P",  # Other fracture of upper and lower end of right fibula, subsequent encounter for closed fracture with malunion
        "S82831Q",  # Other fracture of upper and lower end of right fibula, subsequent encounter for open fracture type I or II with malunion
        "S82831R",  # Other fracture of upper and lower end of right fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82831S",  # Other fracture of upper and lower end of right fibula, sequela
        "S82832A",  # Other fracture of upper and lower end of left fibula, initial encounter for closed fracture
        "S82832B",  # Other fracture of upper and lower end of left fibula, initial encounter for open fracture type I or II
        "S82832C",  # Other fracture of upper and lower end of left fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82832D",  # Other fracture of upper and lower end of left fibula, subsequent encounter for closed fracture with routine healing
        "S82832E",  # Other fracture of upper and lower end of left fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82832F",  # Other fracture of upper and lower end of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82832G",  # Other fracture of upper and lower end of left fibula, subsequent encounter for closed fracture with delayed healing
        "S82832H",  # Other fracture of upper and lower end of left fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82832J",  # Other fracture of upper and lower end of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82832K",  # Other fracture of upper and lower end of left fibula, subsequent encounter for closed fracture with nonunion
        "S82832M",  # Other fracture of upper and lower end of left fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82832N",  # Other fracture of upper and lower end of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82832P",  # Other fracture of upper and lower end of left fibula, subsequent encounter for closed fracture with malunion
        "S82832Q",  # Other fracture of upper and lower end of left fibula, subsequent encounter for open fracture type I or II with malunion
        "S82832R",  # Other fracture of upper and lower end of left fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82832S",  # Other fracture of upper and lower end of left fibula, sequela
        "S82839A",  # Other fracture of upper and lower end of unspecified fibula, initial encounter for closed fracture
        "S82839B",  # Other fracture of upper and lower end of unspecified fibula, initial encounter for open fracture type I or II
        "S82839C",  # Other fracture of upper and lower end of unspecified fibula, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82839D",  # Other fracture of upper and lower end of unspecified fibula, subsequent encounter for closed fracture with routine healing
        "S82839E",  # Other fracture of upper and lower end of unspecified fibula, subsequent encounter for open fracture type I or II with routine healing
        "S82839F",  # Other fracture of upper and lower end of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82839G",  # Other fracture of upper and lower end of unspecified fibula, subsequent encounter for closed fracture with delayed healing
        "S82839H",  # Other fracture of upper and lower end of unspecified fibula, subsequent encounter for open fracture type I or II with delayed healing
        "S82839J",  # Other fracture of upper and lower end of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82839K",  # Other fracture of upper and lower end of unspecified fibula, subsequent encounter for closed fracture with nonunion
        "S82839M",  # Other fracture of upper and lower end of unspecified fibula, subsequent encounter for open fracture type I or II with nonunion
        "S82839N",  # Other fracture of upper and lower end of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82839P",  # Other fracture of upper and lower end of unspecified fibula, subsequent encounter for closed fracture with malunion
        "S82839Q",  # Other fracture of upper and lower end of unspecified fibula, subsequent encounter for open fracture type I or II with malunion
        "S82839R",  # Other fracture of upper and lower end of unspecified fibula, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82839S",  # Other fracture of upper and lower end of unspecified fibula, sequela
        "S82861A",  # Displaced Maisonneuve's fracture of right leg, initial encounter for closed fracture
        "S82861B",  # Displaced Maisonneuve's fracture of right leg, initial encounter for open fracture type I or II
        "S82861C",  # Displaced Maisonneuve's fracture of right leg, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82861D",  # Displaced Maisonneuve's fracture of right leg, subsequent encounter for closed fracture with routine healing
        "S82861E",  # Displaced Maisonneuve's fracture of right leg, subsequent encounter for open fracture type I or II with routine healing
        "S82861F",  # Displaced Maisonneuve's fracture of right leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82861G",  # Displaced Maisonneuve's fracture of right leg, subsequent encounter for closed fracture with delayed healing
        "S82861H",  # Displaced Maisonneuve's fracture of right leg, subsequent encounter for open fracture type I or II with delayed healing
        "S82861J",  # Displaced Maisonneuve's fracture of right leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82861K",  # Displaced Maisonneuve's fracture of right leg, subsequent encounter for closed fracture with nonunion
        "S82861M",  # Displaced Maisonneuve's fracture of right leg, subsequent encounter for open fracture type I or II with nonunion
        "S82861N",  # Displaced Maisonneuve's fracture of right leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82861P",  # Displaced Maisonneuve's fracture of right leg, subsequent encounter for closed fracture with malunion
        "S82861Q",  # Displaced Maisonneuve's fracture of right leg, subsequent encounter for open fracture type I or II with malunion
        "S82861R",  # Displaced Maisonneuve's fracture of right leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82861S",  # Displaced Maisonneuve's fracture of right leg, sequela
        "S82862A",  # Displaced Maisonneuve's fracture of left leg, initial encounter for closed fracture
        "S82862B",  # Displaced Maisonneuve's fracture of left leg, initial encounter for open fracture type I or II
        "S82862C",  # Displaced Maisonneuve's fracture of left leg, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82862D",  # Displaced Maisonneuve's fracture of left leg, subsequent encounter for closed fracture with routine healing
        "S82862E",  # Displaced Maisonneuve's fracture of left leg, subsequent encounter for open fracture type I or II with routine healing
        "S82862F",  # Displaced Maisonneuve's fracture of left leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82862G",  # Displaced Maisonneuve's fracture of left leg, subsequent encounter for closed fracture with delayed healing
        "S82862H",  # Displaced Maisonneuve's fracture of left leg, subsequent encounter for open fracture type I or II with delayed healing
        "S82862J",  # Displaced Maisonneuve's fracture of left leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82862K",  # Displaced Maisonneuve's fracture of left leg, subsequent encounter for closed fracture with nonunion
        "S82862M",  # Displaced Maisonneuve's fracture of left leg, subsequent encounter for open fracture type I or II with nonunion
        "S82862N",  # Displaced Maisonneuve's fracture of left leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82862P",  # Displaced Maisonneuve's fracture of left leg, subsequent encounter for closed fracture with malunion
        "S82862Q",  # Displaced Maisonneuve's fracture of left leg, subsequent encounter for open fracture type I or II with malunion
        "S82862R",  # Displaced Maisonneuve's fracture of left leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82862S",  # Displaced Maisonneuve's fracture of left leg, sequela
        "S82863A",  # Displaced Maisonneuve's fracture of unspecified leg, initial encounter for closed fracture
        "S82863B",  # Displaced Maisonneuve's fracture of unspecified leg, initial encounter for open fracture type I or II
        "S82863C",  # Displaced Maisonneuve's fracture of unspecified leg, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82863D",  # Displaced Maisonneuve's fracture of unspecified leg, subsequent encounter for closed fracture with routine healing
        "S82863E",  # Displaced Maisonneuve's fracture of unspecified leg, subsequent encounter for open fracture type I or II with routine healing
        "S82863F",  # Displaced Maisonneuve's fracture of unspecified leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82863G",  # Displaced Maisonneuve's fracture of unspecified leg, subsequent encounter for closed fracture with delayed healing
        "S82863H",  # Displaced Maisonneuve's fracture of unspecified leg, subsequent encounter for open fracture type I or II with delayed healing
        "S82863J",  # Displaced Maisonneuve's fracture of unspecified leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82863K",  # Displaced Maisonneuve's fracture of unspecified leg, subsequent encounter for closed fracture with nonunion
        "S82863M",  # Displaced Maisonneuve's fracture of unspecified leg, subsequent encounter for open fracture type I or II with nonunion
        "S82863N",  # Displaced Maisonneuve's fracture of unspecified leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82863P",  # Displaced Maisonneuve's fracture of unspecified leg, subsequent encounter for closed fracture with malunion
        "S82863Q",  # Displaced Maisonneuve's fracture of unspecified leg, subsequent encounter for open fracture type I or II with malunion
        "S82863R",  # Displaced Maisonneuve's fracture of unspecified leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82863S",  # Displaced Maisonneuve's fracture of unspecified leg, sequela
        "S82864A",  # Nondisplaced Maisonneuve's fracture of right leg, initial encounter for closed fracture
        "S82864B",  # Nondisplaced Maisonneuve's fracture of right leg, initial encounter for open fracture type I or II
        "S82864C",  # Nondisplaced Maisonneuve's fracture of right leg, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82864D",  # Nondisplaced Maisonneuve's fracture of right leg, subsequent encounter for closed fracture with routine healing
        "S82864E",  # Nondisplaced Maisonneuve's fracture of right leg, subsequent encounter for open fracture type I or II with routine healing
        "S82864F",  # Nondisplaced Maisonneuve's fracture of right leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82864G",  # Nondisplaced Maisonneuve's fracture of right leg, subsequent encounter for closed fracture with delayed healing
        "S82864H",  # Nondisplaced Maisonneuve's fracture of right leg, subsequent encounter for open fracture type I or II with delayed healing
        "S82864J",  # Nondisplaced Maisonneuve's fracture of right leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82864K",  # Nondisplaced Maisonneuve's fracture of right leg, subsequent encounter for closed fracture with nonunion
        "S82864M",  # Nondisplaced Maisonneuve's fracture of right leg, subsequent encounter for open fracture type I or II with nonunion
        "S82864N",  # Nondisplaced Maisonneuve's fracture of right leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82864P",  # Nondisplaced Maisonneuve's fracture of right leg, subsequent encounter for closed fracture with malunion
        "S82864Q",  # Nondisplaced Maisonneuve's fracture of right leg, subsequent encounter for open fracture type I or II with malunion
        "S82864R",  # Nondisplaced Maisonneuve's fracture of right leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82864S",  # Nondisplaced Maisonneuve's fracture of right leg, sequela
        "S82865A",  # Nondisplaced Maisonneuve's fracture of left leg, initial encounter for closed fracture
        "S82865B",  # Nondisplaced Maisonneuve's fracture of left leg, initial encounter for open fracture type I or II
        "S82865C",  # Nondisplaced Maisonneuve's fracture of left leg, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82865D",  # Nondisplaced Maisonneuve's fracture of left leg, subsequent encounter for closed fracture with routine healing
        "S82865E",  # Nondisplaced Maisonneuve's fracture of left leg, subsequent encounter for open fracture type I or II with routine healing
        "S82865F",  # Nondisplaced Maisonneuve's fracture of left leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82865G",  # Nondisplaced Maisonneuve's fracture of left leg, subsequent encounter for closed fracture with delayed healing
        "S82865H",  # Nondisplaced Maisonneuve's fracture of left leg, subsequent encounter for open fracture type I or II with delayed healing
        "S82865J",  # Nondisplaced Maisonneuve's fracture of left leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82865K",  # Nondisplaced Maisonneuve's fracture of left leg, subsequent encounter for closed fracture with nonunion
        "S82865M",  # Nondisplaced Maisonneuve's fracture of left leg, subsequent encounter for open fracture type I or II with nonunion
        "S82865N",  # Nondisplaced Maisonneuve's fracture of left leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82865P",  # Nondisplaced Maisonneuve's fracture of left leg, subsequent encounter for closed fracture with malunion
        "S82865Q",  # Nondisplaced Maisonneuve's fracture of left leg, subsequent encounter for open fracture type I or II with malunion
        "S82865R",  # Nondisplaced Maisonneuve's fracture of left leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82865S",  # Nondisplaced Maisonneuve's fracture of left leg, sequela
        "S82866A",  # Nondisplaced Maisonneuve's fracture of unspecified leg, initial encounter for closed fracture
        "S82866B",  # Nondisplaced Maisonneuve's fracture of unspecified leg, initial encounter for open fracture type I or II
        "S82866C",  # Nondisplaced Maisonneuve's fracture of unspecified leg, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82866D",  # Nondisplaced Maisonneuve's fracture of unspecified leg, subsequent encounter for closed fracture with routine healing
        "S82866E",  # Nondisplaced Maisonneuve's fracture of unspecified leg, subsequent encounter for open fracture type I or II with routine healing
        "S82866F",  # Nondisplaced Maisonneuve's fracture of unspecified leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82866G",  # Nondisplaced Maisonneuve's fracture of unspecified leg, subsequent encounter for closed fracture with delayed healing
        "S82866H",  # Nondisplaced Maisonneuve's fracture of unspecified leg, subsequent encounter for open fracture type I or II with delayed healing
        "S82866J",  # Nondisplaced Maisonneuve's fracture of unspecified leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82866K",  # Nondisplaced Maisonneuve's fracture of unspecified leg, subsequent encounter for closed fracture with nonunion
        "S82866M",  # Nondisplaced Maisonneuve's fracture of unspecified leg, subsequent encounter for open fracture type I or II with nonunion
        "S82866N",  # Nondisplaced Maisonneuve's fracture of unspecified leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82866P",  # Nondisplaced Maisonneuve's fracture of unspecified leg, subsequent encounter for closed fracture with malunion
        "S82866Q",  # Nondisplaced Maisonneuve's fracture of unspecified leg, subsequent encounter for open fracture type I or II with malunion
        "S82866R",  # Nondisplaced Maisonneuve's fracture of unspecified leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82866S",  # Nondisplaced Maisonneuve's fracture of unspecified leg, sequela
        "S82871A",  # Displaced pilon fracture of right tibia, initial encounter for closed fracture
        "S82871B",  # Displaced pilon fracture of right tibia, initial encounter for open fracture type I or II
        "S82871C",  # Displaced pilon fracture of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82871D",  # Displaced pilon fracture of right tibia, subsequent encounter for closed fracture with routine healing
        "S82871E",  # Displaced pilon fracture of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82871F",  # Displaced pilon fracture of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82871G",  # Displaced pilon fracture of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82871H",  # Displaced pilon fracture of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82871J",  # Displaced pilon fracture of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82871K",  # Displaced pilon fracture of right tibia, subsequent encounter for closed fracture with nonunion
        "S82871M",  # Displaced pilon fracture of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82871N",  # Displaced pilon fracture of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82871P",  # Displaced pilon fracture of right tibia, subsequent encounter for closed fracture with malunion
        "S82871Q",  # Displaced pilon fracture of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82871R",  # Displaced pilon fracture of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82871S",  # Displaced pilon fracture of right tibia, sequela
        "S82872A",  # Displaced pilon fracture of left tibia, initial encounter for closed fracture
        "S82872B",  # Displaced pilon fracture of left tibia, initial encounter for open fracture type I or II
        "S82872C",  # Displaced pilon fracture of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82872D",  # Displaced pilon fracture of left tibia, subsequent encounter for closed fracture with routine healing
        "S82872E",  # Displaced pilon fracture of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82872F",  # Displaced pilon fracture of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82872G",  # Displaced pilon fracture of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82872H",  # Displaced pilon fracture of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82872J",  # Displaced pilon fracture of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82872K",  # Displaced pilon fracture of left tibia, subsequent encounter for closed fracture with nonunion
        "S82872M",  # Displaced pilon fracture of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82872N",  # Displaced pilon fracture of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82872P",  # Displaced pilon fracture of left tibia, subsequent encounter for closed fracture with malunion
        "S82872Q",  # Displaced pilon fracture of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82872R",  # Displaced pilon fracture of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82872S",  # Displaced pilon fracture of left tibia, sequela
        "S82873A",  # Displaced pilon fracture of unspecified tibia, initial encounter for closed fracture
        "S82873B",  # Displaced pilon fracture of unspecified tibia, initial encounter for open fracture type I or II
        "S82873C",  # Displaced pilon fracture of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82873D",  # Displaced pilon fracture of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82873E",  # Displaced pilon fracture of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82873F",  # Displaced pilon fracture of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82873G",  # Displaced pilon fracture of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82873H",  # Displaced pilon fracture of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82873J",  # Displaced pilon fracture of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82873K",  # Displaced pilon fracture of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82873M",  # Displaced pilon fracture of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82873N",  # Displaced pilon fracture of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82873P",  # Displaced pilon fracture of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82873Q",  # Displaced pilon fracture of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82873R",  # Displaced pilon fracture of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82873S",  # Displaced pilon fracture of unspecified tibia, sequela
        "S82874A",  # Nondisplaced pilon fracture of right tibia, initial encounter for closed fracture
        "S82874B",  # Nondisplaced pilon fracture of right tibia, initial encounter for open fracture type I or II
        "S82874C",  # Nondisplaced pilon fracture of right tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82874D",  # Nondisplaced pilon fracture of right tibia, subsequent encounter for closed fracture with routine healing
        "S82874E",  # Nondisplaced pilon fracture of right tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82874F",  # Nondisplaced pilon fracture of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82874G",  # Nondisplaced pilon fracture of right tibia, subsequent encounter for closed fracture with delayed healing
        "S82874H",  # Nondisplaced pilon fracture of right tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82874J",  # Nondisplaced pilon fracture of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82874K",  # Nondisplaced pilon fracture of right tibia, subsequent encounter for closed fracture with nonunion
        "S82874M",  # Nondisplaced pilon fracture of right tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82874N",  # Nondisplaced pilon fracture of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82874P",  # Nondisplaced pilon fracture of right tibia, subsequent encounter for closed fracture with malunion
        "S82874Q",  # Nondisplaced pilon fracture of right tibia, subsequent encounter for open fracture type I or II with malunion
        "S82874R",  # Nondisplaced pilon fracture of right tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82874S",  # Nondisplaced pilon fracture of right tibia, sequela
        "S82875A",  # Nondisplaced pilon fracture of left tibia, initial encounter for closed fracture
        "S82875B",  # Nondisplaced pilon fracture of left tibia, initial encounter for open fracture type I or II
        "S82875C",  # Nondisplaced pilon fracture of left tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82875D",  # Nondisplaced pilon fracture of left tibia, subsequent encounter for closed fracture with routine healing
        "S82875E",  # Nondisplaced pilon fracture of left tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82875F",  # Nondisplaced pilon fracture of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82875G",  # Nondisplaced pilon fracture of left tibia, subsequent encounter for closed fracture with delayed healing
        "S82875H",  # Nondisplaced pilon fracture of left tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82875J",  # Nondisplaced pilon fracture of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82875K",  # Nondisplaced pilon fracture of left tibia, subsequent encounter for closed fracture with nonunion
        "S82875M",  # Nondisplaced pilon fracture of left tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82875N",  # Nondisplaced pilon fracture of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82875P",  # Nondisplaced pilon fracture of left tibia, subsequent encounter for closed fracture with malunion
        "S82875Q",  # Nondisplaced pilon fracture of left tibia, subsequent encounter for open fracture type I or II with malunion
        "S82875R",  # Nondisplaced pilon fracture of left tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82875S",  # Nondisplaced pilon fracture of left tibia, sequela
        "S82876A",  # Nondisplaced pilon fracture of unspecified tibia, initial encounter for closed fracture
        "S82876B",  # Nondisplaced pilon fracture of unspecified tibia, initial encounter for open fracture type I or II
        "S82876C",  # Nondisplaced pilon fracture of unspecified tibia, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82876D",  # Nondisplaced pilon fracture of unspecified tibia, subsequent encounter for closed fracture with routine healing
        "S82876E",  # Nondisplaced pilon fracture of unspecified tibia, subsequent encounter for open fracture type I or II with routine healing
        "S82876F",  # Nondisplaced pilon fracture of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82876G",  # Nondisplaced pilon fracture of unspecified tibia, subsequent encounter for closed fracture with delayed healing
        "S82876H",  # Nondisplaced pilon fracture of unspecified tibia, subsequent encounter for open fracture type I or II with delayed healing
        "S82876J",  # Nondisplaced pilon fracture of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82876K",  # Nondisplaced pilon fracture of unspecified tibia, subsequent encounter for closed fracture with nonunion
        "S82876M",  # Nondisplaced pilon fracture of unspecified tibia, subsequent encounter for open fracture type I or II with nonunion
        "S82876N",  # Nondisplaced pilon fracture of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82876P",  # Nondisplaced pilon fracture of unspecified tibia, subsequent encounter for closed fracture with malunion
        "S82876Q",  # Nondisplaced pilon fracture of unspecified tibia, subsequent encounter for open fracture type I or II with malunion
        "S82876R",  # Nondisplaced pilon fracture of unspecified tibia, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82876S",  # Nondisplaced pilon fracture of unspecified tibia, sequela
        "S82891A",  # Other fracture of right lower leg, initial encounter for closed fracture
        "S82891B",  # Other fracture of right lower leg, initial encounter for open fracture type I or II
        "S82891C",  # Other fracture of right lower leg, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82891D",  # Other fracture of right lower leg, subsequent encounter for closed fracture with routine healing
        "S82891E",  # Other fracture of right lower leg, subsequent encounter for open fracture type I or II with routine healing
        "S82891F",  # Other fracture of right lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82891G",  # Other fracture of right lower leg, subsequent encounter for closed fracture with delayed healing
        "S82891H",  # Other fracture of right lower leg, subsequent encounter for open fracture type I or II with delayed healing
        "S82891J",  # Other fracture of right lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82891K",  # Other fracture of right lower leg, subsequent encounter for closed fracture with nonunion
        "S82891M",  # Other fracture of right lower leg, subsequent encounter for open fracture type I or II with nonunion
        "S82891N",  # Other fracture of right lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82891P",  # Other fracture of right lower leg, subsequent encounter for closed fracture with malunion
        "S82891Q",  # Other fracture of right lower leg, subsequent encounter for open fracture type I or II with malunion
        "S82891R",  # Other fracture of right lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82891S",  # Other fracture of right lower leg, sequela
        "S82892A",  # Other fracture of left lower leg, initial encounter for closed fracture
        "S82892B",  # Other fracture of left lower leg, initial encounter for open fracture type I or II
        "S82892C",  # Other fracture of left lower leg, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82892D",  # Other fracture of left lower leg, subsequent encounter for closed fracture with routine healing
        "S82892E",  # Other fracture of left lower leg, subsequent encounter for open fracture type I or II with routine healing
        "S82892F",  # Other fracture of left lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82892G",  # Other fracture of left lower leg, subsequent encounter for closed fracture with delayed healing
        "S82892H",  # Other fracture of left lower leg, subsequent encounter for open fracture type I or II with delayed healing
        "S82892J",  # Other fracture of left lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82892K",  # Other fracture of left lower leg, subsequent encounter for closed fracture with nonunion
        "S82892M",  # Other fracture of left lower leg, subsequent encounter for open fracture type I or II with nonunion
        "S82892N",  # Other fracture of left lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82892P",  # Other fracture of left lower leg, subsequent encounter for closed fracture with malunion
        "S82892Q",  # Other fracture of left lower leg, subsequent encounter for open fracture type I or II with malunion
        "S82892R",  # Other fracture of left lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82892S",  # Other fracture of left lower leg, sequela
        "S82899A",  # Other fracture of unspecified lower leg, initial encounter for closed fracture
        "S82899B",  # Other fracture of unspecified lower leg, initial encounter for open fracture type I or II
        "S82899C",  # Other fracture of unspecified lower leg, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S82899D",  # Other fracture of unspecified lower leg, subsequent encounter for closed fracture with routine healing
        "S82899E",  # Other fracture of unspecified lower leg, subsequent encounter for open fracture type I or II with routine healing
        "S82899F",  # Other fracture of unspecified lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S82899G",  # Other fracture of unspecified lower leg, subsequent encounter for closed fracture with delayed healing
        "S82899H",  # Other fracture of unspecified lower leg, subsequent encounter for open fracture type I or II with delayed healing
        "S82899J",  # Other fracture of unspecified lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S82899K",  # Other fracture of unspecified lower leg, subsequent encounter for closed fracture with nonunion
        "S82899M",  # Other fracture of unspecified lower leg, subsequent encounter for open fracture type I or II with nonunion
        "S82899N",  # Other fracture of unspecified lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S82899P",  # Other fracture of unspecified lower leg, subsequent encounter for closed fracture with malunion
        "S82899Q",  # Other fracture of unspecified lower leg, subsequent encounter for open fracture type I or II with malunion
        "S82899R",  # Other fracture of unspecified lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S82899S",  # Other fracture of unspecified lower leg, sequela
        "S8290XA",  # Unspecified fracture of unspecified lower leg, initial encounter for closed fracture
        "S8290XB",  # Unspecified fracture of unspecified lower leg, initial encounter for open fracture type I or II
        "S8290XC",  # Unspecified fracture of unspecified lower leg, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S8290XD",  # Unspecified fracture of unspecified lower leg, subsequent encounter for closed fracture with routine healing
        "S8290XE",  # Unspecified fracture of unspecified lower leg, subsequent encounter for open fracture type I or II with routine healing
        "S8290XF",  # Unspecified fracture of unspecified lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S8290XG",  # Unspecified fracture of unspecified lower leg, subsequent encounter for closed fracture with delayed healing
        "S8290XH",  # Unspecified fracture of unspecified lower leg, subsequent encounter for open fracture type I or II with delayed healing
        "S8290XJ",  # Unspecified fracture of unspecified lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S8290XK",  # Unspecified fracture of unspecified lower leg, subsequent encounter for closed fracture with nonunion
        "S8290XM",  # Unspecified fracture of unspecified lower leg, subsequent encounter for open fracture type I or II with nonunion
        "S8290XN",  # Unspecified fracture of unspecified lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S8290XP",  # Unspecified fracture of unspecified lower leg, subsequent encounter for closed fracture with malunion
        "S8290XQ",  # Unspecified fracture of unspecified lower leg, subsequent encounter for open fracture type I or II with malunion
        "S8290XR",  # Unspecified fracture of unspecified lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S8290XS",  # Unspecified fracture of unspecified lower leg, sequela
        "S8291XA",  # Unspecified fracture of right lower leg, initial encounter for closed fracture
        "S8291XB",  # Unspecified fracture of right lower leg, initial encounter for open fracture type I or II
        "S8291XC",  # Unspecified fracture of right lower leg, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S8291XD",  # Unspecified fracture of right lower leg, subsequent encounter for closed fracture with routine healing
        "S8291XE",  # Unspecified fracture of right lower leg, subsequent encounter for open fracture type I or II with routine healing
        "S8291XF",  # Unspecified fracture of right lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S8291XG",  # Unspecified fracture of right lower leg, subsequent encounter for closed fracture with delayed healing
        "S8291XH",  # Unspecified fracture of right lower leg, subsequent encounter for open fracture type I or II with delayed healing
        "S8291XJ",  # Unspecified fracture of right lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S8291XK",  # Unspecified fracture of right lower leg, subsequent encounter for closed fracture with nonunion
        "S8291XM",  # Unspecified fracture of right lower leg, subsequent encounter for open fracture type I or II with nonunion
        "S8291XN",  # Unspecified fracture of right lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S8291XP",  # Unspecified fracture of right lower leg, subsequent encounter for closed fracture with malunion
        "S8291XQ",  # Unspecified fracture of right lower leg, subsequent encounter for open fracture type I or II with malunion
        "S8291XR",  # Unspecified fracture of right lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S8291XS",  # Unspecified fracture of right lower leg, sequela
        "S8292XA",  # Unspecified fracture of left lower leg, initial encounter for closed fracture
        "S8292XB",  # Unspecified fracture of left lower leg, initial encounter for open fracture type I or II
        "S8292XC",  # Unspecified fracture of left lower leg, initial encounter for open fracture type IIIA, IIIB, or IIIC
        "S8292XD",  # Unspecified fracture of left lower leg, subsequent encounter for closed fracture with routine healing
        "S8292XE",  # Unspecified fracture of left lower leg, subsequent encounter for open fracture type I or II with routine healing
        "S8292XF",  # Unspecified fracture of left lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with routine healing
        "S8292XG",  # Unspecified fracture of left lower leg, subsequent encounter for closed fracture with delayed healing
        "S8292XH",  # Unspecified fracture of left lower leg, subsequent encounter for open fracture type I or II with delayed healing
        "S8292XJ",  # Unspecified fracture of left lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with delayed healing
        "S8292XK",  # Unspecified fracture of left lower leg, subsequent encounter for closed fracture with nonunion
        "S8292XM",  # Unspecified fracture of left lower leg, subsequent encounter for open fracture type I or II with nonunion
        "S8292XN",  # Unspecified fracture of left lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with nonunion
        "S8292XP",  # Unspecified fracture of left lower leg, subsequent encounter for closed fracture with malunion
        "S8292XQ",  # Unspecified fracture of left lower leg, subsequent encounter for open fracture type I or II with malunion
        "S8292XR",  # Unspecified fracture of left lower leg, subsequent encounter for open fracture type IIIA, IIIB, or IIIC with malunion
        "S8292XS",  # Unspecified fracture of left lower leg, sequela
        "S89001A",  # Unspecified physeal fracture of upper end of right tibia, initial encounter for closed fracture
        "S89001D",  # Unspecified physeal fracture of upper end of right tibia, subsequent encounter for fracture with routine healing
        "S89001G",  # Unspecified physeal fracture of upper end of right tibia, subsequent encounter for fracture with delayed healing
        "S89001K",  # Unspecified physeal fracture of upper end of right tibia, subsequent encounter for fracture with nonunion
        "S89001P",  # Unspecified physeal fracture of upper end of right tibia, subsequent encounter for fracture with malunion
        "S89001S",  # Unspecified physeal fracture of upper end of right tibia, sequela
        "S89002A",  # Unspecified physeal fracture of upper end of left tibia, initial encounter for closed fracture
        "S89002D",  # Unspecified physeal fracture of upper end of left tibia, subsequent encounter for fracture with routine healing
        "S89002G",  # Unspecified physeal fracture of upper end of left tibia, subsequent encounter for fracture with delayed healing
        "S89002K",  # Unspecified physeal fracture of upper end of left tibia, subsequent encounter for fracture with nonunion
        "S89002P",  # Unspecified physeal fracture of upper end of left tibia, subsequent encounter for fracture with malunion
        "S89002S",  # Unspecified physeal fracture of upper end of left tibia, sequela
        "S89009A",  # Unspecified physeal fracture of upper end of unspecified tibia, initial encounter for closed fracture
        "S89009D",  # Unspecified physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with routine healing
        "S89009G",  # Unspecified physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with delayed healing
        "S89009K",  # Unspecified physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with nonunion
        "S89009P",  # Unspecified physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with malunion
        "S89009S",  # Unspecified physeal fracture of upper end of unspecified tibia, sequela
        "S89011A",  # Salter-Harris Type I physeal fracture of upper end of right tibia, initial encounter for closed fracture
        "S89011D",  # Salter-Harris Type I physeal fracture of upper end of right tibia, subsequent encounter for fracture with routine healing
        "S89011G",  # Salter-Harris Type I physeal fracture of upper end of right tibia, subsequent encounter for fracture with delayed healing
        "S89011K",  # Salter-Harris Type I physeal fracture of upper end of right tibia, subsequent encounter for fracture with nonunion
        "S89011P",  # Salter-Harris Type I physeal fracture of upper end of right tibia, subsequent encounter for fracture with malunion
        "S89011S",  # Salter-Harris Type I physeal fracture of upper end of right tibia, sequela
        "S89012A",  # Salter-Harris Type I physeal fracture of upper end of left tibia, initial encounter for closed fracture
        "S89012D",  # Salter-Harris Type I physeal fracture of upper end of left tibia, subsequent encounter for fracture with routine healing
        "S89012G",  # Salter-Harris Type I physeal fracture of upper end of left tibia, subsequent encounter for fracture with delayed healing
        "S89012K",  # Salter-Harris Type I physeal fracture of upper end of left tibia, subsequent encounter for fracture with nonunion
        "S89012P",  # Salter-Harris Type I physeal fracture of upper end of left tibia, subsequent encounter for fracture with malunion
        "S89012S",  # Salter-Harris Type I physeal fracture of upper end of left tibia, sequela
        "S89019A",  # Salter-Harris Type I physeal fracture of upper end of unspecified tibia, initial encounter for closed fracture
        "S89019D",  # Salter-Harris Type I physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with routine healing
        "S89019G",  # Salter-Harris Type I physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with delayed healing
        "S89019K",  # Salter-Harris Type I physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with nonunion
        "S89019P",  # Salter-Harris Type I physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with malunion
        "S89019S",  # Salter-Harris Type I physeal fracture of upper end of unspecified tibia, sequela
        "S89021A",  # Salter-Harris Type II physeal fracture of upper end of right tibia, initial encounter for closed fracture
        "S89021D",  # Salter-Harris Type II physeal fracture of upper end of right tibia, subsequent encounter for fracture with routine healing
        "S89021G",  # Salter-Harris Type II physeal fracture of upper end of right tibia, subsequent encounter for fracture with delayed healing
        "S89021K",  # Salter-Harris Type II physeal fracture of upper end of right tibia, subsequent encounter for fracture with nonunion
        "S89021P",  # Salter-Harris Type II physeal fracture of upper end of right tibia, subsequent encounter for fracture with malunion
        "S89021S",  # Salter-Harris Type II physeal fracture of upper end of right tibia, sequela
        "S89022A",  # Salter-Harris Type II physeal fracture of upper end of left tibia, initial encounter for closed fracture
        "S89022D",  # Salter-Harris Type II physeal fracture of upper end of left tibia, subsequent encounter for fracture with routine healing
        "S89022G",  # Salter-Harris Type II physeal fracture of upper end of left tibia, subsequent encounter for fracture with delayed healing
        "S89022K",  # Salter-Harris Type II physeal fracture of upper end of left tibia, subsequent encounter for fracture with nonunion
        "S89022P",  # Salter-Harris Type II physeal fracture of upper end of left tibia, subsequent encounter for fracture with malunion
        "S89022S",  # Salter-Harris Type II physeal fracture of upper end of left tibia, sequela
        "S89029A",  # Salter-Harris Type II physeal fracture of upper end of unspecified tibia, initial encounter for closed fracture
        "S89029D",  # Salter-Harris Type II physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with routine healing
        "S89029G",  # Salter-Harris Type II physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with delayed healing
        "S89029K",  # Salter-Harris Type II physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with nonunion
        "S89029P",  # Salter-Harris Type II physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with malunion
        "S89029S",  # Salter-Harris Type II physeal fracture of upper end of unspecified tibia, sequela
        "S89031A",  # Salter-Harris Type III physeal fracture of upper end of right tibia, initial encounter for closed fracture
        "S89031D",  # Salter-Harris Type III physeal fracture of upper end of right tibia, subsequent encounter for fracture with routine healing
        "S89031G",  # Salter-Harris Type III physeal fracture of upper end of right tibia, subsequent encounter for fracture with delayed healing
        "S89031K",  # Salter-Harris Type III physeal fracture of upper end of right tibia, subsequent encounter for fracture with nonunion
        "S89031P",  # Salter-Harris Type III physeal fracture of upper end of right tibia, subsequent encounter for fracture with malunion
        "S89031S",  # Salter-Harris Type III physeal fracture of upper end of right tibia, sequela
        "S89032A",  # Salter-Harris Type III physeal fracture of upper end of left tibia, initial encounter for closed fracture
        "S89032D",  # Salter-Harris Type III physeal fracture of upper end of left tibia, subsequent encounter for fracture with routine healing
        "S89032G",  # Salter-Harris Type III physeal fracture of upper end of left tibia, subsequent encounter for fracture with delayed healing
        "S89032K",  # Salter-Harris Type III physeal fracture of upper end of left tibia, subsequent encounter for fracture with nonunion
        "S89032P",  # Salter-Harris Type III physeal fracture of upper end of left tibia, subsequent encounter for fracture with malunion
        "S89032S",  # Salter-Harris Type III physeal fracture of upper end of left tibia, sequela
        "S89039A",  # Salter-Harris Type III physeal fracture of upper end of unspecified tibia, initial encounter for closed fracture
        "S89039D",  # Salter-Harris Type III physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with routine healing
        "S89039G",  # Salter-Harris Type III physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with delayed healing
        "S89039K",  # Salter-Harris Type III physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with nonunion
        "S89039P",  # Salter-Harris Type III physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with malunion
        "S89039S",  # Salter-Harris Type III physeal fracture of upper end of unspecified tibia, sequela
        "S89041A",  # Salter-Harris Type IV physeal fracture of upper end of right tibia, initial encounter for closed fracture
        "S89041D",  # Salter-Harris Type IV physeal fracture of upper end of right tibia, subsequent encounter for fracture with routine healing
        "S89041G",  # Salter-Harris Type IV physeal fracture of upper end of right tibia, subsequent encounter for fracture with delayed healing
        "S89041K",  # Salter-Harris Type IV physeal fracture of upper end of right tibia, subsequent encounter for fracture with nonunion
        "S89041P",  # Salter-Harris Type IV physeal fracture of upper end of right tibia, subsequent encounter for fracture with malunion
        "S89041S",  # Salter-Harris Type IV physeal fracture of upper end of right tibia, sequela
        "S89042A",  # Salter-Harris Type IV physeal fracture of upper end of left tibia, initial encounter for closed fracture
        "S89042D",  # Salter-Harris Type IV physeal fracture of upper end of left tibia, subsequent encounter for fracture with routine healing
        "S89042G",  # Salter-Harris Type IV physeal fracture of upper end of left tibia, subsequent encounter for fracture with delayed healing
        "S89042K",  # Salter-Harris Type IV physeal fracture of upper end of left tibia, subsequent encounter for fracture with nonunion
        "S89042P",  # Salter-Harris Type IV physeal fracture of upper end of left tibia, subsequent encounter for fracture with malunion
        "S89042S",  # Salter-Harris Type IV physeal fracture of upper end of left tibia, sequela
        "S89049A",  # Salter-Harris Type IV physeal fracture of upper end of unspecified tibia, initial encounter for closed fracture
        "S89049D",  # Salter-Harris Type IV physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with routine healing
        "S89049G",  # Salter-Harris Type IV physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with delayed healing
        "S89049K",  # Salter-Harris Type IV physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with nonunion
        "S89049P",  # Salter-Harris Type IV physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with malunion
        "S89049S",  # Salter-Harris Type IV physeal fracture of upper end of unspecified tibia, sequela
        "S89091A",  # Other physeal fracture of upper end of right tibia, initial encounter for closed fracture
        "S89091D",  # Other physeal fracture of upper end of right tibia, subsequent encounter for fracture with routine healing
        "S89091G",  # Other physeal fracture of upper end of right tibia, subsequent encounter for fracture with delayed healing
        "S89091K",  # Other physeal fracture of upper end of right tibia, subsequent encounter for fracture with nonunion
        "S89091P",  # Other physeal fracture of upper end of right tibia, subsequent encounter for fracture with malunion
        "S89091S",  # Other physeal fracture of upper end of right tibia, sequela
        "S89092A",  # Other physeal fracture of upper end of left tibia, initial encounter for closed fracture
        "S89092D",  # Other physeal fracture of upper end of left tibia, subsequent encounter for fracture with routine healing
        "S89092G",  # Other physeal fracture of upper end of left tibia, subsequent encounter for fracture with delayed healing
        "S89092K",  # Other physeal fracture of upper end of left tibia, subsequent encounter for fracture with nonunion
        "S89092P",  # Other physeal fracture of upper end of left tibia, subsequent encounter for fracture with malunion
        "S89092S",  # Other physeal fracture of upper end of left tibia, sequela
        "S89099A",  # Other physeal fracture of upper end of unspecified tibia, initial encounter for closed fracture
        "S89099D",  # Other physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with routine healing
        "S89099G",  # Other physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with delayed healing
        "S89099K",  # Other physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with nonunion
        "S89099P",  # Other physeal fracture of upper end of unspecified tibia, subsequent encounter for fracture with malunion
        "S89099S",  # Other physeal fracture of upper end of unspecified tibia, sequela
        "S89101A",  # Unspecified physeal fracture of lower end of right tibia, initial encounter for closed fracture
        "S89101D",  # Unspecified physeal fracture of lower end of right tibia, subsequent encounter for fracture with routine healing
        "S89101G",  # Unspecified physeal fracture of lower end of right tibia, subsequent encounter for fracture with delayed healing
        "S89101K",  # Unspecified physeal fracture of lower end of right tibia, subsequent encounter for fracture with nonunion
        "S89101P",  # Unspecified physeal fracture of lower end of right tibia, subsequent encounter for fracture with malunion
        "S89101S",  # Unspecified physeal fracture of lower end of right tibia, sequela
        "S89102A",  # Unspecified physeal fracture of lower end of left tibia, initial encounter for closed fracture
        "S89102D",  # Unspecified physeal fracture of lower end of left tibia, subsequent encounter for fracture with routine healing
        "S89102G",  # Unspecified physeal fracture of lower end of left tibia, subsequent encounter for fracture with delayed healing
        "S89102K",  # Unspecified physeal fracture of lower end of left tibia, subsequent encounter for fracture with nonunion
        "S89102P",  # Unspecified physeal fracture of lower end of left tibia, subsequent encounter for fracture with malunion
        "S89102S",  # Unspecified physeal fracture of lower end of left tibia, sequela
        "S89109A",  # Unspecified physeal fracture of lower end of unspecified tibia, initial encounter for closed fracture
        "S89109D",  # Unspecified physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with routine healing
        "S89109G",  # Unspecified physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with delayed healing
        "S89109K",  # Unspecified physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with nonunion
        "S89109P",  # Unspecified physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with malunion
        "S89109S",  # Unspecified physeal fracture of lower end of unspecified tibia, sequela
        "S89111A",  # Salter-Harris Type I physeal fracture of lower end of right tibia, initial encounter for closed fracture
        "S89111D",  # Salter-Harris Type I physeal fracture of lower end of right tibia, subsequent encounter for fracture with routine healing
        "S89111G",  # Salter-Harris Type I physeal fracture of lower end of right tibia, subsequent encounter for fracture with delayed healing
        "S89111K",  # Salter-Harris Type I physeal fracture of lower end of right tibia, subsequent encounter for fracture with nonunion
        "S89111P",  # Salter-Harris Type I physeal fracture of lower end of right tibia, subsequent encounter for fracture with malunion
        "S89111S",  # Salter-Harris Type I physeal fracture of lower end of right tibia, sequela
        "S89112A",  # Salter-Harris Type I physeal fracture of lower end of left tibia, initial encounter for closed fracture
        "S89112D",  # Salter-Harris Type I physeal fracture of lower end of left tibia, subsequent encounter for fracture with routine healing
        "S89112G",  # Salter-Harris Type I physeal fracture of lower end of left tibia, subsequent encounter for fracture with delayed healing
        "S89112K",  # Salter-Harris Type I physeal fracture of lower end of left tibia, subsequent encounter for fracture with nonunion
        "S89112P",  # Salter-Harris Type I physeal fracture of lower end of left tibia, subsequent encounter for fracture with malunion
        "S89112S",  # Salter-Harris Type I physeal fracture of lower end of left tibia, sequela
        "S89119A",  # Salter-Harris Type I physeal fracture of lower end of unspecified tibia, initial encounter for closed fracture
        "S89119D",  # Salter-Harris Type I physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with routine healing
        "S89119G",  # Salter-Harris Type I physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with delayed healing
        "S89119K",  # Salter-Harris Type I physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with nonunion
        "S89119P",  # Salter-Harris Type I physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with malunion
        "S89119S",  # Salter-Harris Type I physeal fracture of lower end of unspecified tibia, sequela
        "S89121A",  # Salter-Harris Type II physeal fracture of lower end of right tibia, initial encounter for closed fracture
        "S89121D",  # Salter-Harris Type II physeal fracture of lower end of right tibia, subsequent encounter for fracture with routine healing
        "S89121G",  # Salter-Harris Type II physeal fracture of lower end of right tibia, subsequent encounter for fracture with delayed healing
        "S89121K",  # Salter-Harris Type II physeal fracture of lower end of right tibia, subsequent encounter for fracture with nonunion
        "S89121P",  # Salter-Harris Type II physeal fracture of lower end of right tibia, subsequent encounter for fracture with malunion
        "S89121S",  # Salter-Harris Type II physeal fracture of lower end of right tibia, sequela
        "S89122A",  # Salter-Harris Type II physeal fracture of lower end of left tibia, initial encounter for closed fracture
        "S89122D",  # Salter-Harris Type II physeal fracture of lower end of left tibia, subsequent encounter for fracture with routine healing
        "S89122G",  # Salter-Harris Type II physeal fracture of lower end of left tibia, subsequent encounter for fracture with delayed healing
        "S89122K",  # Salter-Harris Type II physeal fracture of lower end of left tibia, subsequent encounter for fracture with nonunion
        "S89122P",  # Salter-Harris Type II physeal fracture of lower end of left tibia, subsequent encounter for fracture with malunion
        "S89122S",  # Salter-Harris Type II physeal fracture of lower end of left tibia, sequela
        "S89129A",  # Salter-Harris Type II physeal fracture of lower end of unspecified tibia, initial encounter for closed fracture
        "S89129D",  # Salter-Harris Type II physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with routine healing
        "S89129G",  # Salter-Harris Type II physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with delayed healing
        "S89129K",  # Salter-Harris Type II physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with nonunion
        "S89129P",  # Salter-Harris Type II physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with malunion
        "S89129S",  # Salter-Harris Type II physeal fracture of lower end of unspecified tibia, sequela
        "S89131A",  # Salter-Harris Type III physeal fracture of lower end of right tibia, initial encounter for closed fracture
        "S89131D",  # Salter-Harris Type III physeal fracture of lower end of right tibia, subsequent encounter for fracture with routine healing
        "S89131G",  # Salter-Harris Type III physeal fracture of lower end of right tibia, subsequent encounter for fracture with delayed healing
        "S89131K",  # Salter-Harris Type III physeal fracture of lower end of right tibia, subsequent encounter for fracture with nonunion
        "S89131P",  # Salter-Harris Type III physeal fracture of lower end of right tibia, subsequent encounter for fracture with malunion
        "S89131S",  # Salter-Harris Type III physeal fracture of lower end of right tibia, sequela
        "S89132A",  # Salter-Harris Type III physeal fracture of lower end of left tibia, initial encounter for closed fracture
        "S89132D",  # Salter-Harris Type III physeal fracture of lower end of left tibia, subsequent encounter for fracture with routine healing
        "S89132G",  # Salter-Harris Type III physeal fracture of lower end of left tibia, subsequent encounter for fracture with delayed healing
        "S89132K",  # Salter-Harris Type III physeal fracture of lower end of left tibia, subsequent encounter for fracture with nonunion
        "S89132P",  # Salter-Harris Type III physeal fracture of lower end of left tibia, subsequent encounter for fracture with malunion
        "S89132S",  # Salter-Harris Type III physeal fracture of lower end of left tibia, sequela
        "S89139A",  # Salter-Harris Type III physeal fracture of lower end of unspecified tibia, initial encounter for closed fracture
        "S89139D",  # Salter-Harris Type III physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with routine healing
        "S89139G",  # Salter-Harris Type III physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with delayed healing
        "S89139K",  # Salter-Harris Type III physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with nonunion
        "S89139P",  # Salter-Harris Type III physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with malunion
        "S89139S",  # Salter-Harris Type III physeal fracture of lower end of unspecified tibia, sequela
        "S89141A",  # Salter-Harris Type IV physeal fracture of lower end of right tibia, initial encounter for closed fracture
        "S89141D",  # Salter-Harris Type IV physeal fracture of lower end of right tibia, subsequent encounter for fracture with routine healing
        "S89141G",  # Salter-Harris Type IV physeal fracture of lower end of right tibia, subsequent encounter for fracture with delayed healing
        "S89141K",  # Salter-Harris Type IV physeal fracture of lower end of right tibia, subsequent encounter for fracture with nonunion
        "S89141P",  # Salter-Harris Type IV physeal fracture of lower end of right tibia, subsequent encounter for fracture with malunion
        "S89141S",  # Salter-Harris Type IV physeal fracture of lower end of right tibia, sequela
        "S89142A",  # Salter-Harris Type IV physeal fracture of lower end of left tibia, initial encounter for closed fracture
        "S89142D",  # Salter-Harris Type IV physeal fracture of lower end of left tibia, subsequent encounter for fracture with routine healing
        "S89142G",  # Salter-Harris Type IV physeal fracture of lower end of left tibia, subsequent encounter for fracture with delayed healing
        "S89142K",  # Salter-Harris Type IV physeal fracture of lower end of left tibia, subsequent encounter for fracture with nonunion
        "S89142P",  # Salter-Harris Type IV physeal fracture of lower end of left tibia, subsequent encounter for fracture with malunion
        "S89142S",  # Salter-Harris Type IV physeal fracture of lower end of left tibia, sequela
        "S89149A",  # Salter-Harris Type IV physeal fracture of lower end of unspecified tibia, initial encounter for closed fracture
        "S89149D",  # Salter-Harris Type IV physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with routine healing
        "S89149G",  # Salter-Harris Type IV physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with delayed healing
        "S89149K",  # Salter-Harris Type IV physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with nonunion
        "S89149P",  # Salter-Harris Type IV physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with malunion
        "S89149S",  # Salter-Harris Type IV physeal fracture of lower end of unspecified tibia, sequela
        "S89191A",  # Other physeal fracture of lower end of right tibia, initial encounter for closed fracture
        "S89191D",  # Other physeal fracture of lower end of right tibia, subsequent encounter for fracture with routine healing
        "S89191G",  # Other physeal fracture of lower end of right tibia, subsequent encounter for fracture with delayed healing
        "S89191K",  # Other physeal fracture of lower end of right tibia, subsequent encounter for fracture with nonunion
        "S89191P",  # Other physeal fracture of lower end of right tibia, subsequent encounter for fracture with malunion
        "S89191S",  # Other physeal fracture of lower end of right tibia, sequela
        "S89192A",  # Other physeal fracture of lower end of left tibia, initial encounter for closed fracture
        "S89192D",  # Other physeal fracture of lower end of left tibia, subsequent encounter for fracture with routine healing
        "S89192G",  # Other physeal fracture of lower end of left tibia, subsequent encounter for fracture with delayed healing
        "S89192K",  # Other physeal fracture of lower end of left tibia, subsequent encounter for fracture with nonunion
        "S89192P",  # Other physeal fracture of lower end of left tibia, subsequent encounter for fracture with malunion
        "S89192S",  # Other physeal fracture of lower end of left tibia, sequela
        "S89199A",  # Other physeal fracture of lower end of unspecified tibia, initial encounter for closed fracture
        "S89199D",  # Other physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with routine healing
        "S89199G",  # Other physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with delayed healing
        "S89199K",  # Other physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with nonunion
        "S89199P",  # Other physeal fracture of lower end of unspecified tibia, subsequent encounter for fracture with malunion
        "S89199S",  # Other physeal fracture of lower end of unspecified tibia, sequela
        "S89201A",  # Unspecified physeal fracture of upper end of right fibula, initial encounter for closed fracture
        "S89201D",  # Unspecified physeal fracture of upper end of right fibula, subsequent encounter for fracture with routine healing
        "S89201G",  # Unspecified physeal fracture of upper end of right fibula, subsequent encounter for fracture with delayed healing
        "S89201K",  # Unspecified physeal fracture of upper end of right fibula, subsequent encounter for fracture with nonunion
        "S89201P",  # Unspecified physeal fracture of upper end of right fibula, subsequent encounter for fracture with malunion
        "S89201S",  # Unspecified physeal fracture of upper end of right fibula, sequela
        "S89202A",  # Unspecified physeal fracture of upper end of left fibula, initial encounter for closed fracture
        "S89202D",  # Unspecified physeal fracture of upper end of left fibula, subsequent encounter for fracture with routine healing
        "S89202G",  # Unspecified physeal fracture of upper end of left fibula, subsequent encounter for fracture with delayed healing
        "S89202K",  # Unspecified physeal fracture of upper end of left fibula, subsequent encounter for fracture with nonunion
        "S89202P",  # Unspecified physeal fracture of upper end of left fibula, subsequent encounter for fracture with malunion
        "S89202S",  # Unspecified physeal fracture of upper end of left fibula, sequela
        "S89209A",  # Unspecified physeal fracture of upper end of unspecified fibula, initial encounter for closed fracture
        "S89209D",  # Unspecified physeal fracture of upper end of unspecified fibula, subsequent encounter for fracture with routine healing
        "S89209G",  # Unspecified physeal fracture of upper end of unspecified fibula, subsequent encounter for fracture with delayed healing
        "S89209K",  # Unspecified physeal fracture of upper end of unspecified fibula, subsequent encounter for fracture with nonunion
        "S89209P",  # Unspecified physeal fracture of upper end of unspecified fibula, subsequent encounter for fracture with malunion
        "S89209S",  # Unspecified physeal fracture of upper end of unspecified fibula, sequela
        "S89211A",  # Salter-Harris Type I physeal fracture of upper end of right fibula, initial encounter for closed fracture
        "S89211D",  # Salter-Harris Type I physeal fracture of upper end of right fibula, subsequent encounter for fracture with routine healing
        "S89211G",  # Salter-Harris Type I physeal fracture of upper end of right fibula, subsequent encounter for fracture with delayed healing
        "S89211K",  # Salter-Harris Type I physeal fracture of upper end of right fibula, subsequent encounter for fracture with nonunion
        "S89211P",  # Salter-Harris Type I physeal fracture of upper end of right fibula, subsequent encounter for fracture with malunion
        "S89211S",  # Salter-Harris Type I physeal fracture of upper end of right fibula, sequela
        "S89212A",  # Salter-Harris Type I physeal fracture of upper end of left fibula, initial encounter for closed fracture
        "S89212D",  # Salter-Harris Type I physeal fracture of upper end of left fibula, subsequent encounter for fracture with routine healing
        "S89212G",  # Salter-Harris Type I physeal fracture of upper end of left fibula, subsequent encounter for fracture with delayed healing
        "S89212K",  # Salter-Harris Type I physeal fracture of upper end of left fibula, subsequent encounter for fracture with nonunion
        "S89212P",  # Salter-Harris Type I physeal fracture of upper end of left fibula, subsequent encounter for fracture with malunion
        "S89212S",  # Salter-Harris Type I physeal fracture of upper end of left fibula, sequela
        "S89219A",  # Salter-Harris Type I physeal fracture of upper end of unspecified fibula, initial encounter for closed fracture
        "S89219D",  # Salter-Harris Type I physeal fracture of upper end of unspecified fibula, subsequent encounter for fracture with routine healing
        "S89219G",  # Salter-Harris Type I physeal fracture of upper end of unspecified fibula, subsequent encounter for fracture with delayed healing
        "S89219K",  # Salter-Harris Type I physeal fracture of upper end of unspecified fibula, subsequent encounter for fracture with nonunion
        "S89219P",  # Salter-Harris Type I physeal fracture of upper end of unspecified fibula, subsequent encounter for fracture with malunion
        "S89219S",  # Salter-Harris Type I physeal fracture of upper end of unspecified fibula, sequela
        "S89221A",  # Salter-Harris Type II physeal fracture of upper end of right fibula, initial encounter for closed fracture
        "S89221D",  # Salter-Harris Type II physeal fracture of upper end of right fibula, subsequent encounter for fracture with routine healing
        "S89221G",  # Salter-Harris Type II physeal fracture of upper end of right fibula, subsequent encounter for fracture with delayed healing
        "S89221K",  # Salter-Harris Type II physeal fracture of upper end of right fibula, subsequent encounter for fracture with nonunion
        "S89221P",  # Salter-Harris Type II physeal fracture of upper end of right fibula, subsequent encounter for fracture with malunion
        "S89221S",  # Salter-Harris Type II physeal fracture of upper end of right fibula, sequela
        "S89222A",  # Salter-Harris Type II physeal fracture of upper end of left fibula, initial encounter for closed fracture
        "S89222D",  # Salter-Harris Type II physeal fracture of upper end of left fibula, subsequent encounter for fracture with routine healing
        "S89222G",  # Salter-Harris Type II physeal fracture of upper end of left fibula, subsequent encounter for fracture with delayed healing
        "S89222K",  # Salter-Harris Type II physeal fracture of upper end of left fibula, subsequent encounter for fracture with nonunion
        "S89222P",  # Salter-Harris Type II physeal fracture of upper end of left fibula, subsequent encounter for fracture with malunion
        "S89222S",  # Salter-Harris Type II physeal fracture of upper end of left fibula, sequela
        "S89229A",  # Salter-Harris Type II physeal fracture of upper end of unspecified fibula, initial encounter for closed fracture
        "S89229D",  # Salter-Harris Type II physeal fracture of upper end of unspecified fibula, subsequent encounter for fracture with routine healing
        "S89229G",  # Salter-Harris Type II physeal fracture of upper end of unspecified fibula, subsequent encounter for fracture with delayed healing
        "S89229K",  # Salter-Harris Type II physeal fracture of upper end of unspecified fibula, subsequent encounter for fracture with nonunion
        "S89229P",  # Salter-Harris Type II physeal fracture of upper end of unspecified fibula, subsequent encounter for fracture with malunion
        "S89229S",  # Salter-Harris Type II physeal fracture of upper end of unspecified fibula, sequela
        "S89291A",  # Other physeal fracture of upper end of right fibula, initial encounter for closed fracture
        "S89291D",  # Other physeal fracture of upper end of right fibula, subsequent encounter for fracture with routine healing
        "S89291G",  # Other physeal fracture of upper end of right fibula, subsequent encounter for fracture with delayed healing
        "S89291K",  # Other physeal fracture of upper end of right fibula, subsequent encounter for fracture with nonunion
        "S89291P",  # Other physeal fracture of upper end of right fibula, subsequent encounter for fracture with malunion
        "S89291S",  # Other physeal fracture of upper end of right fibula, sequela
        "S89292A",  # Other physeal fracture of upper end of left fibula, initial encounter for closed fracture
        "S89292D",  # Other physeal fracture of upper end of left fibula, subsequent encounter for fracture with routine healing
        "S89292G",  # Other physeal fracture of upper end of left fibula, subsequent encounter for fracture with delayed healing
        "S89292K",  # Other physeal fracture of upper end of left fibula, subsequent encounter for fracture with nonunion
        "S89292P",  # Other physeal fracture of upper end of left fibula, subsequent encounter for fracture with malunion
        "S89292S",  # Other physeal fracture of upper end of left fibula, sequela
        "S89299A",  # Other physeal fracture of upper end of unspecified fibula, initial encounter for closed fracture
        "S89299D",  # Other physeal fracture of upper end of unspecified fibula, subsequent encounter for fracture with routine healing
        "S89299G",  # Other physeal fracture of upper end of unspecified fibula, subsequent encounter for fracture with delayed healing
        "S89299K",  # Other physeal fracture of upper end of unspecified fibula, subsequent encounter for fracture with nonunion
        "S89299P",  # Other physeal fracture of upper end of unspecified fibula, subsequent encounter for fracture with malunion
        "S89299S",  # Other physeal fracture of upper end of unspecified fibula, sequela
        "S89301A",  # Unspecified physeal fracture of lower end of right fibula, initial encounter for closed fracture
        "S89301D",  # Unspecified physeal fracture of lower end of right fibula, subsequent encounter for fracture with routine healing
        "S89301G",  # Unspecified physeal fracture of lower end of right fibula, subsequent encounter for fracture with delayed healing
        "S89301K",  # Unspecified physeal fracture of lower end of right fibula, subsequent encounter for fracture with nonunion
        "S89301P",  # Unspecified physeal fracture of lower end of right fibula, subsequent encounter for fracture with malunion
        "S89301S",  # Unspecified physeal fracture of lower end of right fibula, sequela
        "S89302A",  # Unspecified physeal fracture of lower end of left fibula, initial encounter for closed fracture
        "S89302D",  # Unspecified physeal fracture of lower end of left fibula, subsequent encounter for fracture with routine healing
        "S89302G",  # Unspecified physeal fracture of lower end of left fibula, subsequent encounter for fracture with delayed healing
        "S89302K",  # Unspecified physeal fracture of lower end of left fibula, subsequent encounter for fracture with nonunion
        "S89302P",  # Unspecified physeal fracture of lower end of left fibula, subsequent encounter for fracture with malunion
        "S89302S",  # Unspecified physeal fracture of lower end of left fibula, sequela
        "S89309A",  # Unspecified physeal fracture of lower end of unspecified fibula, initial encounter for closed fracture
        "S89309D",  # Unspecified physeal fracture of lower end of unspecified fibula, subsequent encounter for fracture with routine healing
        "S89309G",  # Unspecified physeal fracture of lower end of unspecified fibula, subsequent encounter for fracture with delayed healing
        "S89309K",  # Unspecified physeal fracture of lower end of unspecified fibula, subsequent encounter for fracture with nonunion
        "S89309P",  # Unspecified physeal fracture of lower end of unspecified fibula, subsequent encounter for fracture with malunion
        "S89309S",  # Unspecified physeal fracture of lower end of unspecified fibula, sequela
        "S89311A",  # Salter-Harris Type I physeal fracture of lower end of right fibula, initial encounter for closed fracture
        "S89311D",  # Salter-Harris Type I physeal fracture of lower end of right fibula, subsequent encounter for fracture with routine healing
        "S89311G",  # Salter-Harris Type I physeal fracture of lower end of right fibula, subsequent encounter for fracture with delayed healing
        "S89311K",  # Salter-Harris Type I physeal fracture of lower end of right fibula, subsequent encounter for fracture with nonunion
        "S89311P",  # Salter-Harris Type I physeal fracture of lower end of right fibula, subsequent encounter for fracture with malunion
        "S89311S",  # Salter-Harris Type I physeal fracture of lower end of right fibula, sequela
        "S89312A",  # Salter-Harris Type I physeal fracture of lower end of left fibula, initial encounter for closed fracture
        "S89312D",  # Salter-Harris Type I physeal fracture of lower end of left fibula, subsequent encounter for fracture with routine healing
        "S89312G",  # Salter-Harris Type I physeal fracture of lower end of left fibula, subsequent encounter for fracture with delayed healing
        "S89312K",  # Salter-Harris Type I physeal fracture of lower end of left fibula, subsequent encounter for fracture with nonunion
        "S89312P",  # Salter-Harris Type I physeal fracture of lower end of left fibula, subsequent encounter for fracture with malunion
        "S89312S",  # Salter-Harris Type I physeal fracture of lower end of left fibula, sequela
        "S89319A",  # Salter-Harris Type I physeal fracture of lower end of unspecified fibula, initial encounter for closed fracture
        "S89319D",  # Salter-Harris Type I physeal fracture of lower end of unspecified fibula, subsequent encounter for fracture with routine healing
        "S89319G",  # Salter-Harris Type I physeal fracture of lower end of unspecified fibula, subsequent encounter for fracture with delayed healing
        "S89319K",  # Salter-Harris Type I physeal fracture of lower end of unspecified fibula, subsequent encounter for fracture with nonunion
        "S89319P",  # Salter-Harris Type I physeal fracture of lower end of unspecified fibula, subsequent encounter for fracture with malunion
        "S89319S",  # Salter-Harris Type I physeal fracture of lower end of unspecified fibula, sequela
        "S89321A",  # Salter-Harris Type II physeal fracture of lower end of right fibula, initial encounter for closed fracture
        "S89321D",  # Salter-Harris Type II physeal fracture of lower end of right fibula, subsequent encounter for fracture with routine healing
        "S89321G",  # Salter-Harris Type II physeal fracture of lower end of right fibula, subsequent encounter for fracture with delayed healing
        "S89321K",  # Salter-Harris Type II physeal fracture of lower end of right fibula, subsequent encounter for fracture with nonunion
        "S89321P",  # Salter-Harris Type II physeal fracture of lower end of right fibula, subsequent encounter for fracture with malunion
        "S89321S",  # Salter-Harris Type II physeal fracture of lower end of right fibula, sequela
        "S89322A",  # Salter-Harris Type II physeal fracture of lower end of left fibula, initial encounter for closed fracture
        "S89322D",  # Salter-Harris Type II physeal fracture of lower end of left fibula, subsequent encounter for fracture with routine healing
        "S89322G",  # Salter-Harris Type II physeal fracture of lower end of left fibula, subsequent encounter for fracture with delayed healing
        "S89322K",  # Salter-Harris Type II physeal fracture of lower end of left fibula, subsequent encounter for fracture with nonunion
        "S89322P",  # Salter-Harris Type II physeal fracture of lower end of left fibula, subsequent encounter for fracture with malunion
        "S89322S",  # Salter-Harris Type II physeal fracture of lower end of left fibula, sequela
        "S89329A",  # Salter-Harris Type II physeal fracture of lower end of unspecified fibula, initial encounter for closed fracture
        "S89329D",  # Salter-Harris Type II physeal fracture of lower end of unspecified fibula, subsequent encounter for fracture with routine healing
        "S89329G",  # Salter-Harris Type II physeal fracture of lower end of unspecified fibula, subsequent encounter for fracture with delayed healing
        "S89329K",  # Salter-Harris Type II physeal fracture of lower end of unspecified fibula, subsequent encounter for fracture with nonunion
        "S89329P",  # Salter-Harris Type II physeal fracture of lower end of unspecified fibula, subsequent encounter for fracture with malunion
        "S89329S",  # Salter-Harris Type II physeal fracture of lower end of unspecified fibula, sequela
        "S89391A",  # Other physeal fracture of lower end of right fibula, initial encounter for closed fracture
        "S89391D",  # Other physeal fracture of lower end of right fibula, subsequent encounter for fracture with routine healing
        "S89391G",  # Other physeal fracture of lower end of right fibula, subsequent encounter for fracture with delayed healing
        "S89391K",  # Other physeal fracture of lower end of right fibula, subsequent encounter for fracture with nonunion
        "S89391P",  # Other physeal fracture of lower end of right fibula, subsequent encounter for fracture with malunion
        "S89391S",  # Other physeal fracture of lower end of right fibula, sequela
        "S89392A",  # Other physeal fracture of lower end of left fibula, initial encounter for closed fracture
        "S89392D",  # Other physeal fracture of lower end of left fibula, subsequent encounter for fracture with routine healing
        "S89392G",  # Other physeal fracture of lower end of left fibula, subsequent encounter for fracture with delayed healing
        "S89392K",  # Other physeal fracture of lower end of left fibula, subsequent encounter for fracture with nonunion
        "S89392P",  # Other physeal fracture of lower end of left fibula, subsequent encounter for fracture with malunion
        "S89392S",  # Other physeal fracture of lower end of left fibula, sequela
        "S89399A",  # Other physeal fracture of lower end of unspecified fibula, initial encounter for closed fracture
        "S89399D",  # Other physeal fracture of lower end of unspecified fibula, subsequent encounter for fracture with routine healing
        "S89399G",  # Other physeal fracture of lower end of unspecified fibula, subsequent encounter for fracture with delayed healing
        "S89399K",  # Other physeal fracture of lower end of unspecified fibula, subsequent encounter for fracture with nonunion
        "S89399P",  # Other physeal fracture of lower end of unspecified fibula, subsequent encounter for fracture with malunion
        "S89399S",  # Other physeal fracture of lower end of unspecified fibula, sequela
        "T84010A",  # Broken internal right hip prosthesis, initial encounter
        "T84011A",  # Broken internal left hip prosthesis, initial encounter
        "T84012A",  # Broken internal right knee prosthesis, initial encounter
        "T84013A",  # Broken internal left knee prosthesis, initial encounter
    }

    SNOMEDCT = {
        "10624611000119102",  # Stress fracture of lateral condyle of right femur (disorder)
        "10624651000119101",  # Stress fracture of lateral condyle of left femur (disorder)
        "10816221000119100",  # Open fracture of right lower leg (disorder)
        "10817141000119101",  # Open fracture of upper end of right tibia (disorder)
        "10817541000119106",  # Open fracture of proximal right fibula (disorder)
        "10817981000119109",  # Open fracture of midcervical section of right femur (disorder)
        "10818021000119107",  # Open fracture of right patella (disorder)
        "10818901000119102",  # Open trimalleolar fracture of left ankle (disorder)
        "10818981000119105",  # Open fracture of distal end of left tibia (disorder)
        "10819021000119104",  # Open fracture of shaft of left tibia (disorder)
        "10819101000119107",  # Open fracture of intercondylar spine of left tibia (disorder)
        "10819181000119104",  # Open fracture of lateral condyle of left tibia (disorder)
        "10819221000119107",  # Open fracture of medial condyle of left tibia (disorder)
        "10819301000119105",  # Open fracture of left tibia (disorder)
        "10819701000119106",  # Open fracture of left lateral malleolus of fibula (disorder)
        "10819741000119108",  # Open fracture of distal end of left fibula (disorder)
        "10820141000119108",  # Open fracture of neck of left femur (disorder)
        "10820181000119103",  # Open fracture of distal end of left femur (disorder)
        "10820891000119109",  # Open fracture of distal end of right tibia (disorder)
        "10820971000119103",  # Open fracture of distal end of right fibula (disorder)
        "10821011000119103",  # Open fracture of shaft of right fibula (disorder)
        "10821051000119102",  # Open fracture of shaft of left fibula (disorder)
        "10821091000119107",  # Open fracture of right fibula (disorder)
        "10821131000119109",  # Open fracture of left fibula (disorder)
        "10821491000119101",  # Open intertrochanteric fracture of right femur (disorder)
        "10821531000119101",  # Open intertrochanteric fracture of left femur (disorder)
        "10821691000119103",  # Open subtrochanteric fracture of left femur (disorder)
        "10821731000119105",  # Open subtrochanteric fracture of right femur (disorder)
        "10821771000119108",  # Open fracture of greater trochanter of right femur (disorder)
        "10821811000119108",  # Open fracture of greater trochanter of left femur (disorder)
        "10821851000119109",  # Open fracture of base of neck of right femur (disorder)
        "10821891000119104",  # Open fracture of base of neck of left femur (disorder)
        "10821931000119107",  # Open fracture of neck of right femur (disorder)
        "10821971000119105",  # Open fracture of shaft of right femur (disorder)
        "10822011000119106",  # Open fracture of shaft of left femur (disorder)
        "10822051000119107",  # Open supracondylar fracture of right femur (disorder)
        "10822091000119102",  # Open supracondylar fracture of left femur (disorder)
        "10822131000119100",  # Open fracture of lateral condyle of distal right femur (disorder)
        "10822171000119102",  # Open fracture of lateral condyle of distal left femur (disorder)
        "10822291000119109",  # Open fracture of condyle of right femur (disorder)
        "10822331000119103",  # Open fracture of condyle of left femur (disorder)
        "10822371000119100",  # Open fracture of distal end of right femur (disorder)
        "10822451000119103",  # Open fracture of right femur (disorder)
        "10822491000119108",  # Open fracture of left femur (disorder)
        "10824561000119107",  # Open bimalleolar fracture of left ankle (disorder)
        "10830061000119105",  # Open fracture of left lower leg (disorder)
        "10832021000119101",  # Open trimalleolar fracture of right ankle (disorder)
        "10832141000119106",  # Open fracture of shaft of right tibia (disorder)
        "10832221000119109",  # Open fracture of intercondylar spine of right tibia (disorder)
        "10832261000119104",  # Open fracture of lateral condyle of right tibia (disorder)
        "10832301000119107",  # Open fracture of medial condyle of right tibia (disorder)
        "10832341000119109",  # Open fracture of right tibia (disorder)
        "10834301000119104",  # Open subcapital fracture of right femur (disorder)
        "10835391000119109",  # Open fracture of medial malleolus of left tibia (disorder)
        "10836701000119109",  # Open bimalleolar fracture of right ankle (disorder)
        "10836741000119106",  # Open fracture of medial malleolus of right tibia (disorder)
        "10836781000119101",  # Open bicondylar fracture of upper end of right tibia (disorder)
        "10836821000119106",  # Open fracture of upper end of left tibia (disorder)
        "10837021000119104",  # Open fracture of right lateral malleolus of fibula (disorder)
        "10837061000119109",  # Open fracture of proximal left fibula (disorder)
        "10838021000119103",  # Open subcapital fracture of left femur (disorder)
        "10838061000119108",  # Open fracture of midcervical section of left femur (disorder)
        "10838101000119106",  # Open fracture of left patella (disorder)
        "10840411000119101",  # Open bicondylar fracture of upper end of left tibia (disorder)
        "10873971000119105",  # Open fracture of right tibial plateau (disorder)
        "10923401000119100",  # Closed fracture of epiphysis of distal right femur (disorder)
        "10923441000119103",  # Closed fracture of epiphysis of distal left femur (disorder)
        "10923481000119108",  # Closed fracture of epiphyseal plate of distal end of right femur (disorder)
        "10923521000119108",  # Closed fracture of epiphyseal plate of distal end of left femur (disorder)
        "10923601000119103",  # Closed fracture of proximal epiphysis of left femur (disorder)
        "10924281000119107",  # Closed trimalleolar fracture of right ankle (disorder)
        "10924321000119102",  # Closed trimalleolar fracture of left ankle (disorder)
        "10924361000119107",  # Closed bimalleolar fracture of right ankle (disorder)
        "10924401000119103",  # Closed bimalleolar fracture of left ankle (disorder)
        "10924441000119101",  # Closed fracture of medial malleolus of right distal tibia (disorder)
        "10924481000119106",  # Closed fracture of medial malleolus of left distal tibia (disorder)
        "10924521000119106",  # Closed fracture of distal end of right tibia (disorder)
        "10924561000119101",  # Closed fracture of distal end of left tibia (disorder)
        "10924601000119101",  # Closed fracture of epiphyseal plate of distal end of right tibia (disorder)
        "10924641000119104",  # Closed fracture of epiphyseal plate of distal end of left tibia (disorder)
        "10924681000119109",  # Closed fracture of tuberosity of right tibia (disorder)
        "10924721000119103",  # Closed fracture of tuberosity of left tibia (disorder)
        "10924761000119108",  # Closed fracture of lateral condyle of proximal right tibia (disorder)
        "10924801000119100",  # Closed fracture of lateral condyle of proximal left tibia (disorder)
        "10924841000119103",  # Closed fracture of medial condyle of right tibia (disorder)
        "10924881000119108",  # Closed fracture of medial condyle of left tibia (disorder)
        "10924921000119101",  # Closed bicondylar fracture of plateau of right tibia (disorder)
        "10925001000119105",  # Closed fracture of right tibial plateau (disorder)
        "10925041000119107",  # Closed fracture of left tibial plateau (disorder)
        "10925121000119100",  # Closed fracture of spine of left tibia (disorder)
        "10925161000119105",  # Closed fracture of proximal end of right tibia (disorder)
        "10925201000119100",  # Closed fracture of proximal end of left tibia (disorder)
        "10925241000119103",  # Closed fracture of epiphyseal plate of proximal right tibia (disorder)
        "10925281000119108",  # Closed fracture of epiphyseal plate of proximal left tibia (disorder)
        "10925321000119103",  # Closed fracture of shaft of right tibia (disorder)
        "10925361000119108",  # Closed fracture of shaft of left tibia (disorder)
        "10925401000119104",  # Closed fracture of right tibia (disorder)
        "10925441000119102",  # Closed fracture of left tibia (disorder)
        "10926161000119104",  # Closed fracture of lateral malleolus of right fibula (disorder)
        "10926201000119109",  # Closed fracture of lateral malleolus of left fibula (disorder)
        "10926241000119106",  # Closed fracture of distal right fibula (disorder)
        "10926281000119101",  # Closed fracture of distal left fibula (disorder)
        "10926321000119106",  # Closed fracture of epiphyseal plate of distal right fibula (disorder)
        "10926361000119101",  # Closed fracture of epiphyseal plate of distal left fibula (disorder)
        "10926401000119105",  # Closed fracture of shaft of right fibula (disorder)
        "10926441000119107",  # Closed fracture of shaft of left fibula (disorder)
        "10926481000119102",  # Closed fracture of upper end of right fibula (disorder)
        "10926521000119102",  # Closed fracture of upper end of left fibula (disorder)
        "10926561000119107",  # Closed fracture of epiphyseal plate of proximal right fibula (disorder)
        "10926601000119107",  # Closed fracture of epiphyseal plate of proximal left fibula (disorder)
        "10926641000119109",  # Closed fracture of right fibula (disorder)
        "10926681000119104",  # Closed fracture of left fibula (disorder)
        "10932281000119100",  # Closed fracture of right patella (disorder)
        "10932321000119105",  # Closed fracture of left patella (disorder)
        "10932361000119100",  # Closed fracture of midcervical region of right femur (disorder)
        "10932401000119109",  # Closed fracture of midcervical region of left femur (disorder)
        "10932441000119106",  # Closed subtrochanteric fracture of right femur (disorder)
        "10932481000119101",  # Closed subtrochanteric fracture of left femur (disorder)
        "10932521000119101",  # Closed fracture of base of neck of right femur (disorder)
        "10932561000119106",  # Closed fracture of base of neck of left femur (disorder)
        "10932601000119106",  # Closed fracture of neck of right femur (disorder)
        "10932641000119108",  # Closed fracture of neck of left femur (disorder)
        "10932681000119103",  # Closed subcapital fracture of neck of right femur (disorder)
        "10932721000119109",  # Closed subcapital fracture of neck of left femur (disorder)
        "10932761000119104",  # Closed intra-articular fracture of head of right femur (disorder)
        "10932801000119107",  # Closed intra-articular fracture of head of left femur (disorder)
        "10932841000119109",  # Closed intertrochanteric fracture of right femur (disorder)
        "10932881000119104",  # Closed intertrochanteric fracture of left femur (disorder)
        "10932921000119106",  # Closed fracture of greater trochanter of right femur (disorder)
        "10932961000119101",  # Closed fracture of greater trochanter of left femur (disorder)
        "10933001000119109",  # Closed fracture of lesser trochanter of right femur (disorder)
        "10933041000119106",  # Closed fracture of lesser trochanter of left femur (disorder)
        "10933081000119101",  # Closed fracture of apophysis of right femur (disorder)
        "10933121000119104",  # Closed fracture of apophysis of left femur (disorder)
        "10933161000119109",  # Closed fracture of epiphyseal plate of proximal end of right femur (disorder)
        "10933201000119104",  # Closed fracture of epiphyseal plate of proximal end of left femur (disorder)
        "10933241000119102",  # Closed fracture of medial condyle of right femur (disorder)
        "10933281000119107",  # Closed fracture of medial condyle of left femur (disorder)
        "10933321000119102",  # Closed fracture of lateral condyle of right femur (disorder)
        "10933361000119107",  # Closed fracture of lateral condyle of left femur (disorder)
        "10933401000119103",  # Closed fracture of condyle of right femur (disorder)
        "10933441000119101",  # Closed fracture of condyle of left femur (disorder)
        "10933481000119106",  # Closed supracondylar fracture of right femur (disorder)
        "10933521000119106",  # Closed supracondylar fracture of left femur (disorder)
        "10933561000119101",  # Closed fracture of distal end of right femur (disorder)
        "10933601000119101",  # Closed fracture of distal end of left femur (disorder)
        "10933641000119104",  # Closed fracture of shaft of right femur (disorder)
        "10933681000119109",  # Closed fracture of shaft of left femur (disorder)
        "10933721000119103",  # Closed fracture of right femur (disorder)
        "10933761000119108",  # Closed fracture of left femur (disorder)
        "10933801000119100",  # Closed fracture of right lower leg (disorder)
        "10933841000119103",  # Closed fracture of left lower leg (disorder)
        "111643005",  # Open fracture of patella (disorder)
        "111644004",  # Closed fracture of upper end of lower leg (disorder)
        "111645003",  # Open fracture of upper end of tibia (disorder)
        "111646002",  # Open fracture of shaft of fibula (disorder)
        "11304681000119102",  # Stress fracture of right tibia (disorder)
        "11304721000119108",  # Stress fracture of left tibia (disorder)
        "11304761000119103",  # Pathological fracture of right tibia due to osteoporosis (disorder)
        "11304841000119108",  # Pathological fracture of right tibia due to neoplasm (disorder)
        "11304881000119103",  # Pathological fracture of left tibia due to neoplastic disease (disorder)
        "11304921000119105",  # Pathological fracture of right tibia (disorder)
        "11304961000119100",  # Pathological fracture of left tibia (disorder)
        "11305081000119109",  # Pathological fracture of right fibula due to osteoporosis (disorder)
        "11305121000119106",  # Pathological fracture of left fibula due to osteoporosis (disorder)
        "11305241000119108",  # Pathological fracture of right fibula (disorder)
        "11305281000119103",  # Pathological fracture of left fibula (disorder)
        "11305401000119107",  # Stress fracture of right fibula (disorder)
        "11305441000119109",  # Stress fracture of left fibula (disorder)
        "11306121000119102",  # Open avulsion fracture of anterior inferior spine of right ilium (disorder)
        "11306161000119107",  # Open avulsion fracture of anterior inferior spine of left ilium (disorder)
        "11306281000119105",  # Closed avulsion fracture of anterior inferior spine of right ilium (disorder)
        "11306321000119100",  # Closed avulsion fracture of anterior inferior spine of left ilium (disorder)
        "11306361000119105",  # Closed avulsion fracture of anterior superior spine of right ilium (disorder)
        "11306401000119101",  # Closed avulsion fracture of anterior superior spine of left ilium (disorder)
        "11306441000119104",  # Closed fracture of right ilium (disorder)
        "11306481000119109",  # Closed fracture of left ilium (disorder)
        "11306521000119109",  # Open fracture of right ilium (disorder)
        "11306561000119104",  # Open fracture of left ilium (disorder)
        "11306601000119104",  # Open fracture of right pubis (disorder)
        "11306641000119102",  # Open fracture of left pubis (disorder)
        "11306681000119107",  # Stress fracture of right pubic ramus (disorder)
        "11306721000119101",  # Stress fracture of left pubic ramus (disorder)
        "11306761000119106",  # Closed fracture of right pubis (disorder)
        "11306801000119103",  # Closed fracture of left pubis (disorder)
        "11306921000119104",  # Closed fracture of right ischium (disorder)
        "11306961000119109",  # Closed fracture of left ischium (disorder)
        "11307001000119109",  # Closed fracture of posterior column of right acetabulum (disorder)
        "11307081000119101",  # Closed fracture of anterior column of right acetabulum (disorder)
        "11307121000119104",  # Closed fracture of posterior column of left acetabulum (disorder)
        "11307161000119109",  # Closed transverse and posterior wall fracture of right acetabulum (disorder)
        "11307201000119104",  # Closed fracture of right acetabulum (disorder)
        "11307241000119102",  # Closed transverse and posterior wall fracture of left acetabulum (disorder)
        "11307281000119107",  # Closed fracture of left acetabulum (disorder)
        "11307321000119102",  # Closed transverse fracture of right acetabulum (disorder)
        "11307361000119107",  # Closed transverse fracture of left acetabulum (disorder)
        "11307401000119103",  # Closed fracture of anterior column and posterior column of right acetabulum (disorder)
        "11307441000119101",  # Closed fracture of anterior column and posterior column of left acetabulum (disorder)
        "11307481000119106",  # Closed fracture of triradiate cartilage and epiphysis of right acetabulum (disorder)
        "11307521000119106",  # Closed fracture of triradiate cartilage and epiphysis of left acetabulum (disorder)
        "11307561000119101",  # Closed fracture of roof of right acetabulum (disorder)
        "11307601000119101",  # Closed fracture of roof of left acetabulum (disorder)
        "11307641000119104",  # Closed fracture of anterior wall of right acetabulum (disorder)
        "11307681000119109",  # Closed fracture of anterior wall of left acetabulum (disorder)
        "11307721000119103",  # Closed fracture of posterior wall of right acetabulum (disorder)
        "11307761000119108",  # Closed fracture of posterior wall of left acetabulum (disorder)
        "11307801000119100",  # Closed fracture of medial wall of right acetabulum (disorder)
        "11307841000119103",  # Closed fracture of medial wall of left acetabulum (disorder)
        "11308001000119108",  # Open fracture of anterior column of right acetabulum (disorder)
        "11308041000119105",  # Open fracture of anterior column of left acetabulum (disorder)
        "11308201000119103",  # Open fracture of right acetabulum (disorder)
        "11308281000119106",  # Open fracture of left acetabulum (disorder)
        "11309721000119109",  # Pathological fracture of right patella (disorder)
        "11309761000119104",  # Pathological fracture of left patella (disorder)
        "11309801000119107",  # Pathological fracture of right femur due to osteoporosis (disorder)
        "11309841000119109",  # Pathological fracture of left femur due to osteoporosis (disorder)
        "11309881000119104",  # Pathological fracture of right femur due to neoplastic disease (disorder)
        "11309921000119106",  # Pathological fracture of left femur due to neoplastic disease (disorder)
        "11309961000119101",  # Pathological fracture of right femur (disorder)
        "11310001000119107",  # Pathological fracture of left femur (disorder)
        "11310041000119109",  # Pathological fracture of neck of right femur (disorder)
        "11310081000119104",  # Pathological fracture of neck of left femur (disorder)
        "11310121000119102",  # Pathological fracture of right femur due to secondary osteoporosis (disorder)
        "11310161000119107",  # Pathological fracture of left femur due to secondary osteoporosis (disorder)
        "11314361000119101",  # Stress fracture of right femur (disorder)
        "11314401000119105",  # Stress fracture of left femur (disorder)
        "11314441000119107",  # Stress fracture of shaft of right femur (disorder)
        "11314481000119102",  # Stress fracture of shaft of left femur (disorder)
        "11314521000119102",  # Stress fracture of neck of right femur (disorder)
        "11314561000119107",  # Stress fracture of neck of left femur (disorder)
        "11314601000119107",  # Stress fracture of right hip (disorder)
        "11314641000119109",  # Stress fracture of left hip (disorder)
        "11314881000119100",  # Pathological fracture of right hip due to osteoporosis (disorder)
        "11314921000119107",  # Pathological fracture of left hip due to osteoporosis (disorder)
        "11314961000119102",  # Pathological fracture of proximal right femur due to neoplastic disease (disorder)
        "11315001000119103",  # Pathological fracture of proximal left femur due to neoplastic disease (disorder)
        "11315041000119101",  # Pathological fracture of right hip (disorder)
        "11315081000119106",  # Pathological fracture of left hip (disorder)
        "11315121000119108",  # Pathological fracture of proximal end of right femur due to secondary osteoporosis (disorder)
        "11315161000119103",  # Pathological fracture of left hip due to secondary osteoporosis (disorder)
        "1144497004",  # Pathological fracture of pelvis and femur (disorder)
        "1144498009",  # Fracture malunion of pelvis and femur (disorder)
        "123975002",  # Trimalleolar fracture (disorder)
        "127287001",  # Intertrochanteric fracture (disorder)
        "12746007",  # Open transcervical fracture of femur (disorder)
        "1290783004",  # Stress fracture of bone of hip joint region (disorder)
        "1290785006",  # Stress fracture of bone of lower leg (disorder)
        "1292974007",  # Fracture of ramus of pubis (disorder)
        "1292980004",  # Closed fracture of pubic ramus (disorder)
        "1292981000",  # Open fracture of pubic ramus (disorder)
        "1303377000",  # Fracture of bone of left femur (disorder)
        "1303378005",  # Fracture of bone of right femur (disorder)
        "1303385009",  # Fracture of bone of right ankle joint region (disorder)
        "1303386005",  # Fracture of bone of left ankle joint region (disorder)
        "1337009007",  # Open comminuted fracture of patella (disorder)
        "1337010002",  # Open stellate fracture of patella (disorder)
        "1337011003",  # Closed comminuted fracture of patella (disorder)
        "1337012005",  # Closed stellate fracture of patella (disorder)
        "1340005000",  # Fracture of distal epiphysis of femur (disorder)
        "13960001000004106",  # Fracture of pubic tubercle (disorder)
        "15385006",  # Closed fracture of medial malleolus (disorder)
        "15911931000119102",  # Open fracture of head of left femur (disorder)
        "15912091000119107",  # Open fracture of head of right femur (disorder)
        "15912331000119106",  # Closed fracture of head of left femur (disorder)
        "15912431000119100",  # Closed fracture of head of right femur (disorder)
        "15912531000119101",  # Closed fracture of trochanter of left femur (disorder)
        "15912631000119102",  # Closed fracture of trochanter of right femur (disorder)
        "15970001000004108",  # Periprosthetic fracture of proximal femur (disorder)
        "16280471000119103",  # Pathological fracture of left tibia due to osteoporosis (disorder)
        "16769891000119103",  # Closed fracture of anterior column of left acetabulum (disorder)
        "1705000",  # Closed fracture of base of neck of femur (disorder)
        "17616007",  # Closed fracture of ilium (disorder)
        "18230001000004102",  # Closed fracture of left pilon (disorder)
        "19350001000004107",  # Stress fracture of tibia and fibula (disorder)
        "19652000",  # Open fracture of femoral condyle of femur (disorder)
        "20100009",  # Closed fracture of intracapsular section of femur (disorder)
        "203374004",  # Non-traumatic acute slipped upper femoral epiphysis (disorder)
        "203375003",  # Non-traumatic acute-on-chronic slipped upper femoral epiphysis (disorder)
        "203376002",  # Non-traumatic chronic slipped upper femoral epiphysis (disorder)
        "20433007",  # Fracture of upper end of tibia (disorder)
        "206213006",  # Fracture of femur due to birth trauma (disorder)
        "208148000",  # Closed fracture acetabulum, anterior column (disorder)
        "208149008",  # Closed fracture acetabulum, posterior column (disorder)
        "208150008",  # Closed fracture acetabulum, floor (disorder)
        "208151007",  # Closed fracture acetabulum, double column transverse (disorder)
        "208157006",  # Open fracture acetabulum, anterior column (disorder)
        "208158001",  # Open fracture acetabulum, posterior column (disorder)
        "208159009",  # Open fracture acetabulum, floor (disorder)
        "208160004",  # Open fracture acetabulum, double column transverse (disorder)
        "208164008",  # Closed fracture pubis (disorder)
        "208166005",  # Closed stable fracture of multiple pubic rami (disorder)
        "208168006",  # Closed unstable fracture of multiple pubic rami (disorder)
        "208173000",  # Open stable fracture of multiple pubic rami (disorder)
        "208174006",  # Open unstable fracture of multiple pubic rami (disorder)
        "208181004",  # Closed fracture pelvis, ischial tuberosity (disorder)
        "208182006",  # Closed fracture pelvis, anterior superior iliac spine (disorder)
        "208183001",  # Closed fracture pelvis, anterior inferior iliac spine (disorder)
        "208184007",  # Closed fracture pelvis, iliac wing (disorder)
        "208185008",  # Closed vertical fracture of ilium (disorder)
        "208192003",  # Open fracture pelvis, ischial tuberosity (disorder)
        "208193008",  # Open fracture pelvis, anterior superior iliac spine (disorder)
        "208194002",  # Open fracture pelvis, anterior inferior iliac spine (disorder)
        "208195001",  # Open fracture pelvis, iliac wing (disorder)
        "208197009",  # Open vertical fracture of ilium (disorder)
        "208521006",  # Closed fracture proximal femur, transcervical (disorder)
        "208523009",  # Closed fracture proximal femur, transepiphyseal (disorder)
        "208526001",  # Closed fracture of head of femur (disorder)
        "208528000",  # Closed fracture proximal femur, subcapital, Garden grade I (disorder)
        "208529008",  # Closed fracture proximal femur, subcapital, Garden grade II (disorder)
        "208530003",  # Closed fracture proximal femur, subcapital, Garden grade III (disorder)
        "208531004",  # Closed fracture proximal femur, subcapital, Garden grade IV (disorder)
        "208536009",  # Open fracture proximal femur, transepiphyseal (disorder)
        "208541001",  # Open subcapital fracture of proximal femur Garden grade I (disorder)
        "208542008",  # Open subcapital fracture of proximal femur Garden grade II (disorder)
        "208543003",  # Open subcapital fracture of proximal femur Garden grade III (disorder)
        "208544009",  # Open subcapital fracture of proximal femur Garden grade IV (disorder)
        "208550004",  # Closed fracture proximal femur, intertrochanteric, two part (disorder)
        "208551000",  # Closed fracture proximal femur, subtrochanteric (disorder)
        "208552007",  # Closed fracture proximal femur, intertrochanteric, comminuted (disorder)
        "208557001",  # Open fracture proximal femur, intertrochanteric, two part (disorder)
        "208558006",  # Open fracture proximal femur, subtrochanteric (disorder)
        "208559003",  # Open fracture proximal femur, intertrochanteric, comminuted (disorder)
        "208562000",  # Pertrochanteric fracture (disorder)
        "208578003",  # Closed fracture of lower epiphysis of femur (disorder)
        "208579006",  # Closed fracture distal femur, medial condyle (disorder)
        "208580009",  # Closed fracture distal femur, lateral condyle (disorder)
        "208581008",  # Closed fracture distal femur, bicondylar (T-Y fracture) (disorder)
        "208582001",  # Closed fracture distal femur, comminuted/intra-articular (disorder)
        "208587007",  # Open fracture of lower epiphysis of femur (disorder)
        "208588002",  # Open fracture distal femur, medial condyle (disorder)
        "208589005",  # Open fracture distal femur, lateral condyle (disorder)
        "208590001",  # Open fracture distal femur, bicondylar (T-Y fracture) (disorder)
        "208591002",  # Open fracture distal femur, comminuted/intra-articular (disorder)
        "208596007",  # Closed fracture patella, transverse (disorder)
        "208597003",  # Closed fracture patella, proximal pole (disorder)
        "208598008",  # Closed fracture patella, distal pole (disorder)
        "208600002",  # Closed fracture patella, vertical (disorder)
        "208602005",  # Open fracture patella, transverse (disorder)
        "208603000",  # Open fracture patella, proximal pole (disorder)
        "208604006",  # Open fracture patella, distal pole (disorder)
        "208605007",  # Open fracture patella, vertical (disorder)
        "208610006",  # Closed fracture proximal tibia, medial condyle (plateau) (disorder)
        "208611005",  # Closed fracture proximal tibia, lateral condyle (plateau) (disorder)
        "208612003",  # Closed fracture proximal tibia, bicondylar (disorder)
        "208613008",  # Closed fracture intercondylar spine of tibia (disorder)
        "208615001",  # Closed fracture fibula, neck (disorder)
        "208619007",  # Open fracture proximal tibia, medial condyle (plateau) (disorder)
        "208620001",  # Open fracture proximal tibia, lateral condyle (plateau) (disorder)
        "208621002",  # Open fracture proximal tibia, bicondylar (disorder)
        "208622009",  # Open fracture intercondylar spine of tibia (disorder)
        "208624005",  # Open fracture fibula, head (disorder)
        "208625006",  # Open fracture fibula, neck (disorder)
        "208629000",  # Closed fracture of tibia and fibula, shaft (disorder)
        "208632002",  # Open fracture of tibia and fibula, shaft (disorder)
        "208634001",  # Closed fracture distal tibia (disorder)
        "208635000",  # Closed fracture distal tibia, extra-articular (disorder)
        "208636004",  # Closed fracture distal tibia, intra-articular (disorder)
        "208637008",  # Open fracture distal tibia (disorder)
        "208638003",  # Open fracture distal tibia, extra-articular (disorder)
        "208639006",  # Open fracture distal tibia, intra-articular (disorder)
        "208657007",  # Closed fracture ankle, lateral malleolus, low (disorder)
        "208658002",  # Closed fracture ankle, lateral malleolus, high (disorder)
        "208659005",  # Open fracture ankle, lateral malleolus, low (disorder)
        "208660000",  # Open fracture ankle, lateral malleolus, high (disorder)
        "208662008",  # Closed fracture ankle, bimalleolar, low fibular fracture (disorder)
        "208663003",  # Closed fracture ankle, bimalleolar, high fibular fracture (disorder)
        "208664009",  # Open fracture ankle, bimalleolar, low fibular fracture (disorder)
        "208665005",  # Open fracture ankle, bimalleolar, high fibular fracture (disorder)
        "208666006",  # Closed fracture ankle, trimalleolar, low fibular fracture (disorder)
        "208667002",  # Closed fracture ankle, trimalleolar, high fibular fracture (disorder)
        "208668007",  # Open fracture ankle, trimalleolar, low fibular fracture (disorder)
        "208669004",  # Open fracture ankle, trimalleolar, high fibular fracture (disorder)
        "208731002",  # Multiple fractures of femur (disorder)
        "209335002",  # Closed fracture dislocation of hip joint (disorder)
        "209336001",  # Open fracture dislocation of hip joint (disorder)
        "209337005",  # Closed fracture subluxation of hip joint (disorder)
        "209338000",  # Open fracture subluxation of hip joint (disorder)
        "209340005",  # Closed fracture dislocation of knee joint (disorder)
        "209341009",  # Open fracture dislocation of knee joint (disorder)
        "209342002",  # Closed fracture subluxation of knee joint (disorder)
        "209343007",  # Open fracture subluxation of knee joint (disorder)
        "209344001",  # Closed fracture dislocation of patellofemoral joint (disorder)
        "209345000",  # Open fracture dislocation of patellofemoral joint (disorder)
        "209346004",  # Closed fracture subluxation of patellofemoral joint (disorder)
        "209347008",  # Open fracture subluxation of patellofemoral joint (disorder)
        "21080001000004105",  # Subchondral insufficiency fracture of condyle of femur (disorder)
        "21640001000004108",  # Subtrochanteric fracture of left femur (disorder)
        "21867001",  # Fracture of upper end of fibula (disorder)
        "22234005",  # Open fracture of shaft of tibia (disorder)
        "22802006",  # Closed pertrochanteric fracture (disorder)
        "23900009",  # Closed fracture of upper end of tibia (disorder)
        "25415003",  # Closed fracture of femur (disorder)
        "25899002",  # Closed bimalleolar fracture (disorder)
        "263067005",  # Fracture dislocation of symphysis pubis (disorder)
        "263087006",  # Fracture dislocation of joint of lower limb (disorder)
        "263088001",  # Fracture dislocation of hip joint (disorder)
        "263089009",  # Fracture dislocation of knee joint (disorder)
        "263090000",  # Fracture dislocation of patellofemoral joint (disorder)
        "263112006",  # Fracture subluxation of joint of lower limb (disorder)
        "263113001",  # Fracture subluxation of knee joint (disorder)
        "263114007",  # Fracture subluxation of patellofemoral joint (disorder)
        "263216001",  # Vertical fracture of ilium (disorder)
        "263217005",  # Fracture of iliac crest (disorder)
        "263219008",  # Fracture of multiple pubic rami (disorder)
        "263220002",  # Fracture of ischium (disorder)
        "263221003",  # Fracture of ischial tuberosity (disorder)
        "263225007",  # Fracture of proximal end of femur (disorder)
        "263226008",  # Subcapital fracture of neck of femur (disorder)
        "263227004",  # Midcervical fracture of neck of femur (disorder)
        "263228009",  # Basicervical fracture of neck of femur (disorder)
        "263229001",  # Subtrochanteric fracture of femur (disorder)
        "263231005",  # Open fracture of base of neck of femur (disorder)
        "263232003",  # Fracture of distal end of femur (disorder)
        "263233008",  # Closed fracture of femur, distal end (disorder)
        "263234002",  # Open fracture of femur, distal end (disorder)
        "263235001",  # Supracondylar fracture femur (disorder)
        "263236000",  # Fracture of femoral condyle (disorder)
        "263237009",  # Closed fracture of tibial tuberosity (disorder)
        "263238004",  # Open fracture of tibial tuberosity (disorder)
        "263240009",  # Pilon fracture (disorder)
        "263241008",  # Tillaux fracture (disorder)
        "263242001",  # Fracture of distal end of fibula (disorder)
        "263244000",  # Bimalleolar fracture of ankle (disorder)
        "26442006",  # Closed fracture of shaft of femur (disorder)
        "26908008",  # Open bimalleolar fracture (disorder)
        "26938002",  # Open intertrochanteric fracture (disorder)
        "271577005",  # Fracture of shaft of tibia and fibula (disorder)
        "271963003",  # Fracture of iliac wing (disorder)
        "275338001",  # Closed subcapital fracture of femur (disorder)
        "275339009",  # Open subcapital fracture of femur (disorder)
        "275340006",  # Closed fracture of greater trochanter of femur (disorder)
        "275341005",  # Closed fracture of lesser trochanter of femur (disorder)
        "275342003",  # Open fracture of lesser trochanter of femur (disorder)
        "275343008",  # Open fracture of greater trochanter of femur (disorder)
        "278537006",  # Fracture of distal end of tibia (disorder)
        "28012007",  # Closed fracture of shaft of tibia (disorder)
        "281531008",  # Fracture of medial malleolus (disorder)
        "281532001",  # Fracture of posterior malleolus (disorder)
        "281533006",  # Fracture of head of fibula (disorder)
        "281534000",  # Fracture of neck of fibula (disorder)
        "281535004",  # Fracture of lateral malleolus (disorder)
        "281843000",  # Fracture of tibial spine (disorder)
        "28359007",  # Closed fracture of head of fibula (disorder)
        "28576007",  # Open fracture of femur (disorder)
        "287069001",  # Pathological fracture - lower leg (disorder)
        "30060003",  # Duverney's fracture (disorder)
        "301032005",  # Fracture of superior pubic ramus (disorder)
        "301033000",  # Fracture of inferior pubic ramus (disorder)
        "301034006",  # Fracture of greater trochanter (disorder)
        "301757009",  # Central fracture dislocation acetabulum (disorder)
        "302034009",  # Fracture subluxation of hip joint (disorder)
        "30489009",  # Open fracture of intracapsular section of femur (disorder)
        "307178006",  # Comminuted fracture of patella (disorder)
        "307727005",  # Open fracture of distal fibula (disorder)
        "308153009",  # Closed fracture of distal fibula (disorder)
        "30905007",  # Closed supracondylar fracture of femur (disorder)
        "311857007",  # Closed fractures involving multiple regions upper with lower limb (disorder)
        "315643003",  # Dupuytren's fracture dislocation ankle (disorder)
        "31978002",  # Fracture of tibia (disorder)
        "33118001",  # Closed fracture of acetabulum (disorder)
        "34268009",  # Closed fracture of lateral malleolus (disorder)
        "359817006",  # Closed fracture of hip (disorder)
        "359820003",  # Closed fracture of neck of femur (disorder)
        "361118003",  # Open fracture of hip (disorder)
        "361119006",  # Open fracture of neck of femur (disorder)
        "36127009",  # Fracture of pubis (disorder)
        "36417005",  # Closed fracture of ischium (disorder)
        "39408006",  # Closed fracture of innominate bone (disorder)
        "39551000087109",  # Fracture of medial condyle of right tibia (disorder)
        "409666003",  # Pathological fracture of neck of femur (disorder)
        "409667007",  # Pathological fracture of femur (disorder)
        "41191003",  # Open fracture of head of femur (disorder)
        "413876003",  # Closed fracture of lower leg (disorder)
        "413877007",  # Closed fracture of tibia AND fibula (disorder)
        "41392003",  # Closed multiple fractures of lower limb AND sternum (disorder)
        "414292006",  # Fracture of lower leg (disorder)
        "414293001",  # Fracture of tibia AND fibula (disorder)
        "414942001",  # Open fracture of lower leg (disorder)
        "414943006",  # Open fracture of tibia AND fibula (disorder)
        "42109006",  # Closed multiple fractures of lower limb with ribs AND sternum (disorder)
        "426382004",  # Closed avulsion fracture of greater trochanter of femur (disorder)
        "426679004",  # Closed avulsion fracture of anterior inferior iliac spine of pelvis (disorder)
        "426777008",  # Pathological fracture of patella (disorder)
        "426962008",  # Closed fracture of acetabulum with dislocation of hip (disorder)
        "427509002",  # Closed Salter-Harris type I fracture of lower epiphysis of femur (disorder)
        "427584009",  # Closed Salter-Harris type II fracture of lower epiphysis of femur (disorder)
        "427688003",  # Closed avulsion fracture of lesser trochanter of femur (disorder)
        "427826000",  # Closed Salter-Harris type IV fracture of lower epiphysis of femur (disorder)
        "428019004",  # Open fracture of bone of knee joint (disorder)
        "428020005",  # Closed osteochondral fracture of patella (disorder)
        "428112005",  # Malunion of fracture of proximal femur (disorder)
        "428151000",  # Closed fracture of bone of knee joint (disorder)
        "428232002",  # Closed Salter-Harris type V fracture of lower epiphysis of femur (disorder)
        "428256003",  # Fracture of condyle of tibia (disorder)
        "428257007",  # Fracture of tibial plateau (disorder)
        "428357005",  # Closed Salter-Harris type III fracture of lower epiphysis of femur (disorder)
        "428723000",  # Closed avulsion fracture of anterior superior iliac spine of pelvis (disorder)
        "428797006",  # Closed osteochondral fracture of proximal tibia (disorder)
        "428798001",  # Closed fracture of tibial plateau (disorder)
        "429342005",  # Closed osteochondral fracture of distal femur (disorder)
        "429766001",  # Closed fracture of triradiate cartilage and epiphysis of acetabulum (disorder)
        "431031001",  # Osteoporotic fracture of neck of femur (disorder)
        "433811000124105",  # Avulsion fracture of lateral condyle of tibia (disorder)
        "441499007",  # Stress fracture of pubis (disorder)
        "442205007",  # Stress fracture of tibia (disorder)
        "442538002",  # Stress fracture of fibula (disorder)
        "445311003",  # Open fracture of distal tibia and distal fibula (disorder)
        "445410003",  # Closed fracture of distal tibia and distal fibula (disorder)
        "445526000",  # Pathological fracture of tibia (disorder)
        "445527009",  # Pathological fracture of fibula (disorder)
        "445733003",  # Open comminuted supracondylar fracture of femur (disorder)
        "445875009",  # Fracture of trochanter of femur (disorder)
        "445899009",  # Closed fracture of trochanter of femur (disorder)
        "446298003",  # Closed pilon fracture (disorder)
        "446361000124109",  # Closed fracture of spine of right tibia (disorder)
        "446471000124105",  # Closed subcapital fracture of left femur (disorder)
        "446501000124103",  # Closed bicondylar fracture of plateau of left tibia (disorder)
        "446551000124104",  # Open fracture of left tibial plateau (disorder)
        "446561000124102",  # Open avulsion fracture of anterior inferior spine of ilium (disorder)
        "446581000124107",  # Closed fracture of left lower limb (disorder)
        "446591000124105",  # Closed fracture of right lower limb (disorder)
        "446979005",  # Open fracture of tibia (disorder)
        "446980008",  # Open fracture of tibial plateau (disorder)
        "446981007",  # Open pilon fracture (disorder)
        "447017008",  # Open fracture of fibula (disorder)
        "447139008",  # Closed fracture of tibia (disorder)
        "447394009",  # Closed comminuted supracondylar fracture of femur (disorder)
        "447395005",  # Closed fracture of fibula (disorder)
        "447931000124101",  # Closed subcapital fracture of right femur (disorder)
        "45455009",  # Multiple open fractures of lower end of femur (disorder)
        "46272008",  # Open fracture of innominate bone (disorder)
        "46391000087104",  # Fracture of distal epiphysis of left femur (disorder)
        "46401000087101",  # Fracture of distal epiphysis of right femur (disorder)
        "46481000087108",  # Fracture of neck of left femur (disorder)
        "46491000087105",  # Fracture of neck of right femur (disorder)
        "46521000087108",  # Fracture of proximal end of left fibula (disorder)
        "46531000087105",  # Fracture of proximal end of right fibula (disorder)
        "4673003",  # Open fracture of lateral malleolus (disorder)
        "47848000",  # Closed fracture of condyle of tibia (disorder)
        "51037009",  # Fracture of patella (disorder)
        "51155003",  # Closed multiple fractures of upper AND lower limbs (disorder)
        "52450003",  # Closed fracture of midcervical section of femur (disorder)
        "52603002",  # Closed fracture of lower limb (disorder)
        "53441009",  # Closed multiple fractures of lower limb AND ribs (disorder)
        "54441004",  # Fracture of shaft of femur (disorder)
        "54530004",  # Open fracture of medial malleolus (disorder)
        "55379003",  # Congenital pseudoarthrosis of tibia (disorder)
        "5913000",  # Fracture of neck of femur (disorder)
        "59556008",  # Open fracture of upper end of fibula (disorder)
        "59639009",  # Closed fracture of upper end of fibula (disorder)
        "60214001",  # Open fracture of ilium (disorder)
        "62125005",  # Closed fracture of epiphysis of femur (disorder)
        "64455005",  # Fracture of acetabulum (disorder)
        "64569008",  # Closed fractures of multiple bones of lower limb (disorder)
        "6628008",  # Open fracture of shaft of femur (disorder)
        "66483005",  # Open pertrochanteric fracture (disorder)
        "66926007",  # Closed fracture of femoral condyle of femur (disorder)
        "6698000",  # Closed trimalleolar fracture (disorder)
        "67394003",  # Fracture of shaft of fibula (disorder)
        "69166006",  # Fracture of upper end of lower leg (disorder)
        "69830008",  # Multiple closed fractures of lower end of femur (disorder)
        "6990005",  # Fracture of shaft of tibia (disorder)
        "700097003",  # Fracture of bone of hip region (disorder)
        "703994007",  # Closed intra-articular fracture of head of femur (disorder)
        "703998005",  # Closed bicondylar fracture of tibial plateau (disorder)
        "704060003",  # Stress fracture of pubic rami (disorder)
        "704061004",  # Stress fracture of lateral condyle of femur (disorder)
        "704063001",  # Stress fracture of neck of femur (disorder)
        "704067000",  # Stress fracture of femur (disorder)
        "704068005",  # Stress fracture of shaft of femur (disorder)
        "704171000",  # Pathological fracture of proximal end of femur (disorder)
        "704209008",  # Closed fracture of epiphysis of proximal fibula (disorder)
        "704245006",  # Pathologic fracture of tibia at site of neoplasm (disorder)
        "704249000",  # Pathologic fracture of proximal femur at site of neoplasm (disorder)
        "704252008",  # Pathologic fracture of fibula at site of neoplasm (disorder)
        "704253003",  # Pathologic fracture of femur at site of neoplasm (disorder)
        "704283007",  # Secondary osteoporotic fracture of tibia (disorder)
        "704287008",  # Secondary osteoporotic fracture of proximal femur (disorder)
        "704290002",  # Secondary osteoporotic fracture of fibula (disorder)
        "704291003",  # Secondary osteoporotic fracture of femur (disorder)
        "704330001",  # Osteoporotic fracture of femur (disorder)
        "704331002",  # Osteoporotic fracture of fibula (disorder)
        "704334005",  # Osteoporotic fracture of proximal femur (disorder)
        "704338008",  # Osteoporotic fracture of tibia (disorder)
        "704398006",  # Open transverse fracture of acetabulum (disorder)
        "704451004",  # Closed transverse fracture of acetabulum (disorder)
        "704483005",  # Open fracture of anterior wall of acetabulum (disorder)
        "704484004",  # Open fracture of posterior wall of acetabulum (disorder)
        "705080006",  # Closed fracture of epiphyseal plate of distal tibia (disorder)
        "705082003",  # Closed fracture of epiphyseal plate of distal fibula (disorder)
        "705090003",  # Closed fracture of apophysis of femur (disorder)
        "705091004",  # Closed fracture of epiphyseal plate of distal femur (disorder)
        "705092006",  # Closed fracture of epiphyseal plate of proximal tibia (disorder)
        "705093001",  # Open fracture of apophysis of femur (disorder)
        "705104001",  # Closed fracture of anterior wall of acetabulum (disorder)
        "705120006",  # Open fracture of medial wall of acetabulum (disorder)
        "705121005",  # Closed fracture of medial wall of acetabulum (disorder)
        "705122003",  # Open fracture of anterior column and posterior column of acetabulum (disorder)
        "705123008",  # Closed fracture of anterior column and posterior column of acetabulum (disorder)
        "705124002",  # Open intra-articular fracture of head of femur (disorder)
        "705125001",  # Closed fracture of posterior wall of acetabulum (disorder)
        "705134006",  # Open fracture of roof of acetabulum (disorder)
        "705135007",  # Closed fracture of roof of acetabulum (disorder)
        "705136008",  # Open transverse and posterior wall fracture of acetabulum (disorder)
        "705137004",  # Closed transverse and posterior wall fracture of acetabulum (disorder)
        "71220005",  # Open fracture of pubis (disorder)
        "71620000",  # Fracture of femur (disorder)
        "71830006",  # Supination-adduction injury of ankle, stage 2 (disorder)
        "733276006",  # Fracture of head of femur (disorder)
        "733295004",  # Avulsion of tibial tuberosity (disorder)
        "733296003",  # Avulsion of head of fibula (disorder)
        "735669008",  # Fracture of metaphysis of proximal tibia (disorder)
        "735671008",  # Fracture of lateral condyle of tibia (disorder)
        "735672001",  # Fracture of medial condyle of tibia (disorder)
        "735842005",  # Fracture of lateral malleolus below syndesmosis (disorder)
        "735844006",  # Avulsion of ligament with bony fragment of lateral malleolus (disorder)
        "735845007",  # Fracture of lateral malleolus at syndesmosis (disorder)
        "735846008",  # Avulsion of ligament with bony fragment of medial malleolus (disorder)
        "735847004",  # Fracture of distal fibula above syndesmosis (disorder)
        "736517000",  # Avulsion fracture of anterior fibula (disorder)
        "75591007",  # Fracture of fibula (disorder)
        "7687006",  # Fracture of ilium (disorder)
        "76911006",  # Open fracture of ischium (disorder)
        "77598002",  # Open fracture of acetabulum (disorder)
        "77803008",  # Closed fracture of shaft of fibula (disorder)
        "788892006",  # Complex fracture of lower end of femur not extending into joint (disorder)
        "788943004",  # Fracture of proximal epiphysis of femur (disorder)
        "789045000",  # Simple fracture of lower end of femur not extending into joint (disorder)
        "789058004",  # Wedge fracture of lower end of femur not extending into joint (disorder)
        "79484004",  # Closed transcervical fracture of femur (disorder)
        "79978005",  # Open supracondylar fracture of femur (disorder)
        "80756009",  # Closed fracture of patella (disorder)
        "82640004",  # Stieda's fracture (disorder)
        "84320000",  # Open fracture of midcervical section of femur (disorder)
        "8530001000004108",  # Closed Malgaigne fracture (disorder)
        "8540001000004109",  # Open Malgaigne fracture (disorder)
        "871545004",  # Open fracture of tibial condyle (disorder)
        "87173003",  # Open fracture of epiphysis of femur (disorder)
        "878852003",  # Fracture of knee (disorder)
        "87905008",  # Gosselin's fracture (disorder)
        "890095006",  # Atypical fracture of femur (disorder)
        "89820008",  # Closed intertrochanteric fracture (disorder)
        "90338005",  # Open trimalleolar fracture (disorder)
        "9344009",  # Open fracture of upper end of lower leg (disorder)
        "95853005",  # Metaphyseal fracture of bone of lower limb (disorder)
    }

class MalignantNeoplasmsOfLowerAndUnspecifiedLimbs(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of malignant neoplasms of lower and unspecified limbs.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of malignant neoplasms of lower (long bones) and unspecified limbs.

    **Exclusion Criteria:** Excludes malignant neoplasms of short bones of the lower limbs.
    """

    VALUE_SET_NAME = "Malignant Neoplasms of Lower and Unspecified Limbs"
    OID = "2.16.840.1.113883.3.464.1003.1180"
    DEFINITION_VERSION = "20230722"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "C4020",  # Malignant neoplasm of long bones of unspecified lower limb
        "C4021",  # Malignant neoplasm of long bones of right lower limb
        "C4022",  # Malignant neoplasm of long bones of left lower limb
        "C4080",  # Malignant neoplasm of overlapping sites of bone and articular cartilage of unspecified limb
        "C4081",  # Malignant neoplasm of overlapping sites of bone and articular cartilage of right limb
        "C4082",  # Malignant neoplasm of overlapping sites of bone and articular cartilage of left limb
        "C4090",  # Malignant neoplasm of unspecified bones and articular cartilage of unspecified limb
        "C4091",  # Malignant neoplasm of unspecified bones and articular cartilage of right limb
        "C4092",  # Malignant neoplasm of unspecified bones and articular cartilage of left limb
        "C414",  # Malignant neoplasm of pelvic bones, sacrum and coccyx
        "C419",  # Malignant neoplasm of bone and articular cartilage, unspecified
        "C763",  # Malignant neoplasm of pelvis
        "C7650",  # Malignant neoplasm of unspecified lower limb
        "C7651",  # Malignant neoplasm of right lower limb
        "C7652",  # Malignant neoplasm of left lower limb
        "C7951",  # Secondary malignant neoplasm of bone
        "C7952",  # Secondary malignant neoplasm of bone marrow
        "C800",  # Disseminated malignant neoplasm, unspecified
    }

    SNOMEDCT = {
        "101371000119101",  # Adamantinoma of long bone of lower limb (disorder)
        "1079561000119103",  # Primary chondrosarcoma of bone of left foot (disorder)
        "1079581000119107",  # Primary chondrosarcoma of bone of left lower limb (disorder)
        "1079611000119100",  # Primary chondrosarcoma of bone of right foot (disorder)
        "1079631000119105",  # Primary chondrosarcoma of bone of right lower limb (disorder)
        "1080001000119106",  # Primary Ewing sarcoma of bone of left foot (disorder)
        "1080021000119102",  # Primary Ewing sarcoma of bone of left lower limb (disorder)
        "1080051000119105",  # Primary Ewing sarcoma of bone of right foot (disorder)
        "1080071000119101",  # Primary Ewing sarcoma of bone of right lower limb (disorder)
        "1081281000119102",  # Primary osteosarcoma of bone of left foot (disorder)
        "1081301000119103",  # Primary osteosarcoma of bone of left lower limb (disorder)
        "1081331000119105",  # Primary osteosarcoma of bone of right foot (disorder)
        "1081351000119104",  # Primary osteosarcoma of bone of right lower limb (disorder)
        "1153424004",  # Chondrosarcoma of bone of pelvic wall (disorder)
        "1259417008",  # Primary carcinoma of bone of lower limb (disorder)
        "1259526008",  # Primary carcinoma of pelvic bones, sacrum and coccyx (disorder)
        "1259615008",  # Primary Ewing sarcoma of bone of pelvis (disorder)
        "1259737006",  # Primary tibial adamantinoma (disorder)
        "1259774005",  # Primary adamantinoma of femur (disorder)
        "1285738000",  # Primary malignant neoplasm of bone of foot (disorder)
        "1287540009",  # Primary malignant adamantinoma of long bone of lower limb (disorder)
        "1290725008",  # Malignant dedifferentiated adamantinoma of bone (disorder)
        "130911000119108",  # Chordoma of pelvis (disorder)
        "187952001",  # Malignant neoplasm of pelvic bones, sacrum and coccyx (disorder)
        "187956003",  # Malignant neoplasm of sacral vertebra (disorder)
        "187957007",  # Malignant neoplasm of coccygeal vertebra (disorder)
        "187967002",  # Malignant neoplasm of calcaneum (disorder)
        "187968007",  # Malignant neoplasm of medial cuneiform (disorder)
        "187969004",  # Malignant neoplasm of intermediate cuneiform (disorder)
        "187970003",  # Malignant neoplasm of lateral cuneiform (disorder)
        "187972006",  # Malignant neoplasm of navicular (disorder)
        "187973001",  # Malignant neoplasm of first metatarsal bone (disorder)
        "187974007",  # Malignant neoplasm of second metatarsal bone (disorder)
        "187975008",  # Malignant neoplasm of third metatarsal bone (disorder)
        "187976009",  # Malignant neoplasm of fourth metatarsal bone (disorder)
        "187977000",  # Malignant neoplasm of fifth metatarsal bone (disorder)
        "187978005",  # Malignant neoplasm of phalanges of foot (disorder)
        "265458371000119104",  # Primary malignant adamantinoma of long bone of right lower limb (disorder)
        "281702006",  # Tibial adamantinoma (disorder)
        "286890004",  # Carcinoma of lower limb bones (disorder)
        "352981000119107",  # Primary malignant neoplasm of bone of left foot (disorder)
        "353001000119106",  # Primary malignant neoplasm of bone of right foot (disorder)
        "353661000119102",  # Overlapping primary malignant neoplasm of bone and articular cartilage of lower limb (disorder)
        "354651000119102",  # Primary malignant neoplasm of bone of left lower limb (disorder)
        "354681000119109",  # Primary malignant neoplasm of bone of right lower limb (disorder)
        "372108006",  # Malignant neoplasm of bone of lower limb (disorder)
        "372115003",  # Primary malignant neoplasm of pelvic bones, sacrum and coccyx (disorder)
        "372116002",  # Carcinoma of pelvic bones, sacrum and coccyx (disorder)
        "405945003",  # Malignant neoplasm of metatarsal bone of foot (disorder)
        "433067002",  # Adamantinoma of femur (disorder)
        "440397000",  # Primary osteosarcoma of pelvis (disorder)
        "448221005",  # Sarcoma of fibula (disorder)
        "448378004",  # Sarcoma of bone of foot (disorder)
        "448712008",  # Sarcoma of femur (disorder)
        "449100005",  # Sarcoma of bone of pelvis (disorder)
        "449206007",  # Sarcoma of tibia (disorder)
        "449627008",  # Malignant neoplasm of long bone of lower limb (disorder)
        "449628003",  # Malignant neoplasm of long bone of lower leg (disorder)
        "712525007",  # Malignant neoplasm of short bone of lower limb (disorder)
        "776005371000119104",  # Primary malignant adamantinoma of long bone of left lower limb (disorder)
        "78411000119107",  # Ewing sarcoma of bone of pelvis (disorder)
        "91061000119100",  # Primary chondrosarcoma of bone of pelvis (disorder)
        "91081000119109",  # Primary chondrosarcoma of bone of lower limb (disorder)
        "93722002",  # Primary malignant neoplasm of bone of lower limb (disorder)
        "93737003",  # Primary malignant neoplasm of calcaneus (disorder)
        "93769007",  # Primary malignant neoplasm of cuboid (disorder)
        "93798006",  # Primary malignant neoplasm of femur (disorder)
        "93799003",  # Primary malignant neoplasm of fibula (disorder)
        "93800004",  # Primary malignant neoplasm of first cuneiform bone of foot (disorder)
        "93833009",  # Primary malignant neoplasm of ilium (disorder)
        "93842002",  # Primary malignant neoplasm of ischium (disorder)
        "93871001",  # Primary malignant neoplasm of long bone of lower limb (disorder)
        "93893006",  # Primary malignant neoplasm of metatarsal bone (disorder)
        "93920004",  # Primary malignant neoplasm of navicular bone of foot (disorder)
        "93950007",  # Primary malignant neoplasm of patella (disorder)
        "93959008",  # Primary malignant neoplasm of phalanx of foot (disorder)
        "93975006",  # Primary malignant neoplasm of pubis (disorder)
        "94001007",  # Primary malignant neoplasm of second cuneiform bone of foot (disorder)
        "94003005",  # Primary malignant neoplasm of short bone of lower limb (disorder)
        "94083008",  # Primary malignant neoplasm of talus (disorder)
        "94084002",  # Primary malignant neoplasm of tarsal bone (disorder)
        "94094007",  # Primary malignant neoplasm of third cuneiform bone of foot (disorder)
        "94099002",  # Primary malignant neoplasm of tibia (disorder)
        "94219006",  # Metastatic malignant neoplasm to bone of lower limb (disorder)
        "94236003",  # Metastatic malignant neoplasm to calcaneus (disorder)
        "94269003",  # Metastatic malignant neoplasm to cuboid (disorder)
        "94300004",  # Metastatic malignant neoplasm to femur (disorder)
        "94301000",  # Metastatic malignant neoplasm to fibula (disorder)
        "94302007",  # Metastatic malignant neoplasm to first cuneiform bone of foot (disorder)
        "94337005",  # Metastatic malignant neoplasm to ilium (disorder)
        "94353008",  # Metastatic malignant neoplasm to ischium (disorder)
        "94382009",  # Metastatic malignant neoplasm to long bone of lower limb (disorder)
        "94412004",  # Metastatic malignant neoplasm to metatarsal bone (disorder)
        "94439007",  # Metastatic malignant neoplasm to navicular bone of foot (disorder)
        "94476005",  # Metastatic malignant neoplasm to patella (disorder)
        "94478006",  # Metastatic malignant neoplasm to pelvic bone (disorder)
        "94486006",  # Metastatic malignant neoplasm to phalanx of foot (disorder)
        "94504009",  # Metastatic malignant neoplasm to pubis (disorder)
        "94533005",  # Metastatic malignant neoplasm to second cuneiform bone of foot (disorder)
        "94535003",  # Metastatic malignant neoplasm to short bone of lower limb (disorder)
        "94619004",  # Metastatic malignant neoplasm to talus (disorder)
        "94620005",  # Metastatic malignant neoplasm to tarsal bone (disorder)
        "94630001",  # Metastatic malignant neoplasm to third cuneiform bone of foot (disorder)
        "94635006",  # Metastatic malignant neoplasm to tibia (disorder)
    }

class MechanicalComplicationsExcludingUpperBody(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of mechanical complications (excluding those specific to the upper body).

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of mechanical complications (excluding those specific to the upper body).

    **Exclusion Criteria:** Excludes mechanical complications specific to the upper body.
    """

    VALUE_SET_NAME = "Mechanical Complications Excluding Upper Body"
    OID = "2.16.840.1.113883.3.464.1003.1182"
    DEFINITION_VERSION = "20230722"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "T84010D",  # Broken internal right hip prosthesis, subsequent encounter
        "T84010S",  # Broken internal right hip prosthesis, sequela
        "T84011D",  # Broken internal left hip prosthesis, subsequent encounter
        "T84011S",  # Broken internal left hip prosthesis, sequela
        "T84012D",  # Broken internal right knee prosthesis, subsequent encounter
        "T84012S",  # Broken internal right knee prosthesis, sequela
        "T84013D",  # Broken internal left knee prosthesis, subsequent encounter
        "T84013S",  # Broken internal left knee prosthesis, sequela
        "T84020A",  # Dislocation of internal right hip prosthesis, initial encounter
        "T84020D",  # Dislocation of internal right hip prosthesis, subsequent encounter
        "T84020S",  # Dislocation of internal right hip prosthesis, sequela
        "T84021A",  # Dislocation of internal left hip prosthesis, initial encounter
        "T84021D",  # Dislocation of internal left hip prosthesis, subsequent encounter
        "T84021S",  # Dislocation of internal left hip prosthesis, sequela
        "T84022A",  # Instability of internal right knee prosthesis, initial encounter
        "T84022D",  # Instability of internal right knee prosthesis, subsequent encounter
        "T84022S",  # Instability of internal right knee prosthesis, sequela
        "T84023A",  # Instability of internal left knee prosthesis, initial encounter
        "T84023D",  # Instability of internal left knee prosthesis, subsequent encounter
        "T84023S",  # Instability of internal left knee prosthesis, sequela
        "T84030A",  # Mechanical loosening of internal right hip prosthetic joint, initial encounter
        "T84030D",  # Mechanical loosening of internal right hip prosthetic joint, subsequent encounter
        "T84030S",  # Mechanical loosening of internal right hip prosthetic joint, sequela
        "T84031A",  # Mechanical loosening of internal left hip prosthetic joint, initial encounter
        "T84031D",  # Mechanical loosening of internal left hip prosthetic joint, subsequent encounter
        "T84031S",  # Mechanical loosening of internal left hip prosthetic joint, sequela
        "T84032A",  # Mechanical loosening of internal right knee prosthetic joint, initial encounter
        "T84032D",  # Mechanical loosening of internal right knee prosthetic joint, subsequent encounter
        "T84032S",  # Mechanical loosening of internal right knee prosthetic joint, sequela
        "T84033A",  # Mechanical loosening of internal left knee prosthetic joint, initial encounter
        "T84033D",  # Mechanical loosening of internal left knee prosthetic joint, subsequent encounter
        "T84033S",  # Mechanical loosening of internal left knee prosthetic joint, sequela
        "T84038A",  # Mechanical loosening of other internal prosthetic joint, initial encounter
        "T84038D",  # Mechanical loosening of other internal prosthetic joint, subsequent encounter
        "T84038S",  # Mechanical loosening of other internal prosthetic joint, sequela
        "T84039A",  # Mechanical loosening of unspecified internal prosthetic joint, initial encounter
        "T84039D",  # Mechanical loosening of unspecified internal prosthetic joint, subsequent encounter
        "T84039S",  # Mechanical loosening of unspecified internal prosthetic joint, sequela
        "T84050A",  # Periprosthetic osteolysis of internal prosthetic right hip joint, initial encounter
        "T84050D",  # Periprosthetic osteolysis of internal prosthetic right hip joint, subsequent encounter
        "T84050S",  # Periprosthetic osteolysis of internal prosthetic right hip joint, sequela
        "T84051A",  # Periprosthetic osteolysis of internal prosthetic left hip joint, initial encounter
        "T84051D",  # Periprosthetic osteolysis of internal prosthetic left hip joint, subsequent encounter
        "T84051S",  # Periprosthetic osteolysis of internal prosthetic left hip joint, sequela
        "T84052A",  # Periprosthetic osteolysis of internal prosthetic right knee joint, initial encounter
        "T84052D",  # Periprosthetic osteolysis of internal prosthetic right knee joint, subsequent encounter
        "T84052S",  # Periprosthetic osteolysis of internal prosthetic right knee joint, sequela
        "T84053A",  # Periprosthetic osteolysis of internal prosthetic left knee joint, initial encounter
        "T84053D",  # Periprosthetic osteolysis of internal prosthetic left knee joint, subsequent encounter
        "T84053S",  # Periprosthetic osteolysis of internal prosthetic left knee joint, sequela
        "T84060A",  # Wear of articular bearing surface of internal prosthetic right hip joint, initial encounter
        "T84060D",  # Wear of articular bearing surface of internal prosthetic right hip joint, subsequent encounter
        "T84060S",  # Wear of articular bearing surface of internal prosthetic right hip joint, sequela
        "T84061A",  # Wear of articular bearing surface of internal prosthetic left hip joint, initial encounter
        "T84061D",  # Wear of articular bearing surface of internal prosthetic left hip joint, subsequent encounter
        "T84061S",  # Wear of articular bearing surface of internal prosthetic left hip joint, sequela
        "T84062A",  # Wear of articular bearing surface of internal prosthetic right knee joint, initial encounter
        "T84062D",  # Wear of articular bearing surface of internal prosthetic right knee joint, subsequent encounter
        "T84062S",  # Wear of articular bearing surface of internal prosthetic right knee joint, sequela
        "T84063A",  # Wear of articular bearing surface of internal prosthetic left knee joint, initial encounter
        "T84063D",  # Wear of articular bearing surface of internal prosthetic left knee joint, subsequent encounter
        "T84063S",  # Wear of articular bearing surface of internal prosthetic left knee joint, sequela
        "T84068A",  # Wear of articular bearing surface of other internal prosthetic joint, initial encounter
        "T84069A",  # Wear of articular bearing surface of unspecified internal prosthetic joint, initial encounter
        "T84069D",  # Wear of articular bearing surface of unspecified internal prosthetic joint, subsequent encounter
        "T84069S",  # Wear of articular bearing surface of unspecified internal prosthetic joint, sequela
        "T84090A",  # Other mechanical complication of internal right hip prosthesis, initial encounter
        "T84090D",  # Other mechanical complication of internal right hip prosthesis, subsequent encounter
        "T84090S",  # Other mechanical complication of internal right hip prosthesis, sequela
        "T84091A",  # Other mechanical complication of internal left hip prosthesis, initial encounter
        "T84091D",  # Other mechanical complication of internal left hip prosthesis, subsequent encounter
        "T84091S",  # Other mechanical complication of internal left hip prosthesis, sequela
        "T84092A",  # Other mechanical complication of internal right knee prosthesis, initial encounter
        "T84092D",  # Other mechanical complication of internal right knee prosthesis, subsequent encounter
        "T84092S",  # Other mechanical complication of internal right knee prosthesis, sequela
        "T84093A",  # Other mechanical complication of internal left knee prosthesis, initial encounter
        "T84093D",  # Other mechanical complication of internal left knee prosthesis, subsequent encounter
        "T84093S",  # Other mechanical complication of internal left knee prosthesis, sequela
        "T84114A",  # Breakdown (mechanical) of internal fixation device of right femur, initial encounter
        "T84114D",  # Breakdown (mechanical) of internal fixation device of right femur, subsequent encounter
        "T84114S",  # Breakdown (mechanical) of internal fixation device of right femur, sequela
        "T84115A",  # Breakdown (mechanical) of internal fixation device of left femur, initial encounter
        "T84115D",  # Breakdown (mechanical) of internal fixation device of left femur, subsequent encounter
        "T84115S",  # Breakdown (mechanical) of internal fixation device of left femur, sequela
        "T84116A",  # Breakdown (mechanical) of internal fixation device of bone of right lower leg, initial encounter
        "T84116D",  # Breakdown (mechanical) of internal fixation device of bone of right lower leg, subsequent encounter
        "T84116S",  # Breakdown (mechanical) of internal fixation device of bone of right lower leg, sequela
        "T84117A",  # Breakdown (mechanical) of internal fixation device of bone of left lower leg, initial encounter
        "T84117D",  # Breakdown (mechanical) of internal fixation device of bone of left lower leg, subsequent encounter
        "T84117S",  # Breakdown (mechanical) of internal fixation device of bone of left lower leg, sequela
        "T84124A",  # Displacement of internal fixation device of right femur, initial encounter
        "T84124D",  # Displacement of internal fixation device of right femur, subsequent encounter
        "T84124S",  # Displacement of internal fixation device of right femur, sequela
        "T84125A",  # Displacement of internal fixation device of left femur, initial encounter
        "T84125D",  # Displacement of internal fixation device of left femur, subsequent encounter
        "T84125S",  # Displacement of internal fixation device of left femur, sequela
        "T84126A",  # Displacement of internal fixation device of bone of right lower leg, initial encounter
        "T84126D",  # Displacement of internal fixation device of bone of right lower leg, subsequent encounter
        "T84126S",  # Displacement of internal fixation device of bone of right lower leg, sequela
        "T84127A",  # Displacement of internal fixation device of bone of left lower leg, initial encounter
        "T84127D",  # Displacement of internal fixation device of bone of left lower leg, subsequent encounter
        "T84127S",  # Displacement of internal fixation device of bone of left lower leg, sequela
        "T84194A",  # Other mechanical complication of internal fixation device of right femur, initial encounter
        "T84194D",  # Other mechanical complication of internal fixation device of right femur, subsequent encounter
        "T84194S",  # Other mechanical complication of internal fixation device of right femur, sequela
        "T84195A",  # Other mechanical complication of internal fixation device of left femur, initial encounter
        "T84195D",  # Other mechanical complication of internal fixation device of left femur, subsequent encounter
        "T84195S",  # Other mechanical complication of internal fixation device of left femur, sequela
        "T84196A",  # Other mechanical complication of internal fixation device of bone of right lower leg, initial encounter
        "T84196D",  # Other mechanical complication of internal fixation device of bone of right lower leg, subsequent encounter
        "T84196S",  # Other mechanical complication of internal fixation device of bone of right lower leg, sequela
        "T84197A",  # Other mechanical complication of internal fixation device of bone of left lower leg, initial encounter
        "T84197D",  # Other mechanical complication of internal fixation device of bone of left lower leg, subsequent encounter
        "T84197S",  # Other mechanical complication of internal fixation device of bone of left lower leg, sequela
        "T84218A",  # Breakdown (mechanical) of internal fixation device of other bones, initial encounter
        "T84218D",  # Breakdown (mechanical) of internal fixation device of other bones, subsequent encounter
        "T84218S",  # Breakdown (mechanical) of internal fixation device of other bones, sequela
        "T84228A",  # Displacement of internal fixation device of other bones, initial encounter
        "T84228D",  # Displacement of internal fixation device of other bones, subsequent encounter
        "T84228S",  # Displacement of internal fixation device of other bones, sequela
        "T84310A",  # Breakdown (mechanical) of electronic bone stimulator, initial encounter
        "T84310D",  # Breakdown (mechanical) of electronic bone stimulator, subsequent encounter
        "T84310S",  # Breakdown (mechanical) of electronic bone stimulator, sequela
        "T84318A",  # Breakdown (mechanical) of other bone devices, implants and grafts, initial encounter
        "T84318D",  # Breakdown (mechanical) of other bone devices, implants and grafts, subsequent encounter
        "T84318S",  # Breakdown (mechanical) of other bone devices, implants and grafts, sequela
        "T84320A",  # Displacement of electronic bone stimulator, initial encounter
        "T84320D",  # Displacement of electronic bone stimulator, subsequent encounter
        "T84320S",  # Displacement of electronic bone stimulator, sequela
        "T84328A",  # Displacement of other bone devices, implants and grafts, initial encounter
        "T84328D",  # Displacement of other bone devices, implants and grafts, subsequent encounter
        "T84328S",  # Displacement of other bone devices, implants and grafts, sequela
        "T84390A",  # Other mechanical complication of electronic bone stimulator, initial encounter
        "T84390D",  # Other mechanical complication of electronic bone stimulator, subsequent encounter
        "T84390S",  # Other mechanical complication of electronic bone stimulator, sequela
        "T84398A",  # Other mechanical complication of other bone devices, implants and grafts, initial encounter
        "T84398D",  # Other mechanical complication of other bone devices, implants and grafts, subsequent encounter
        "T84398S",  # Other mechanical complication of other bone devices, implants and grafts, sequela
        "T84410A",  # Breakdown (mechanical) of muscle and tendon graft, initial encounter
        "T84410D",  # Breakdown (mechanical) of muscle and tendon graft, subsequent encounter
        "T84410S",  # Breakdown (mechanical) of muscle and tendon graft, sequela
        "T84418A",  # Breakdown (mechanical) of other internal orthopedic devices, implants and grafts, initial encounter
        "T84418D",  # Breakdown (mechanical) of other internal orthopedic devices, implants and grafts, subsequent encounter
        "T84418S",  # Breakdown (mechanical) of other internal orthopedic devices, implants and grafts, sequela
        "T84420A",  # Displacement of muscle and tendon graft, initial encounter
        "T84420D",  # Displacement of muscle and tendon graft, subsequent encounter
        "T84420S",  # Displacement of muscle and tendon graft, sequela
        "T84428A",  # Displacement of other internal orthopedic devices, implants and grafts, initial encounter
        "T84428D",  # Displacement of other internal orthopedic devices, implants and grafts, subsequent encounter
        "T84428S",  # Displacement of other internal orthopedic devices, implants and grafts, sequela
        "T84490A",  # Other mechanical complication of muscle and tendon graft, initial encounter
        "T84490D",  # Other mechanical complication of muscle and tendon graft, subsequent encounter
        "T84490S",  # Other mechanical complication of muscle and tendon graft, sequela
        "T84498A",  # Other mechanical complication of other internal orthopedic devices, implants and grafts, initial encounter
        "T84498D",  # Other mechanical complication of other internal orthopedic devices, implants and grafts, subsequent encounter
        "T84498S",  # Other mechanical complication of other internal orthopedic devices, implants and grafts, sequela
        "T8481XA",  # Embolism due to internal orthopedic prosthetic devices, implants and grafts, initial encounter
        "T8482XA",  # Fibrosis due to internal orthopedic prosthetic devices, implants and grafts, initial encounter
        "T8483XA",  # Hemorrhage due to internal orthopedic prosthetic devices, implants and grafts, initial encounter
        "T8484XA",  # Pain due to internal orthopedic prosthetic devices, implants and grafts, initial encounter
        "T8485XA",  # Stenosis due to internal orthopedic prosthetic devices, implants and grafts, initial encounter
        "T8486XA",  # Thrombosis due to internal orthopedic prosthetic devices, implants and grafts, initial encounter
        "T8489XA",  # Other specified complication of internal orthopedic prosthetic devices, implants and grafts, initial encounter
        "T849XXA",  # Unspecified complication of internal orthopedic prosthetic device, implant and graft, initial encounter
    }

    SNOMEDCT = {
        "12077001000119100",  # Dislocation of prosthesis of left hip joint (disorder)
        "12199521000119109",  # Mechanical failure of left knee prosthetic joint (disorder)
        "12199641000119102",  # Loosening of prosthesis of left knee joint (disorder)
        "12200001000119100",  # Malposition of prosthetic joint of left hip (disorder)
        "12200041000119103",  # Loosening of prosthesis of left hip joint (disorder)
        "12202141000119103",  # Recurrent dislocation of prosthesis of left hip joint (disorder)
        "12202181000119108",  # Recurrent dislocation of prosthesis of right hip joint (disorder)
        "12206701000119100",  # Dislocation of prosthesis of right hip joint (disorder)
        "12207101000119102",  # Mechanical failure of right knee prosthetic joint (disorder)
        "12207221000119102",  # Malposition of prosthetic joint of right hip (disorder)
        "12207381000119107",  # Loosening of prosthesis of right knee joint (disorder)
        "12207421000119103",  # Loosening of prosthesis of right hip joint (disorder)
        "12403221000119108",  # Instability of prosthesis of joint of left hip (disorder)
        "12403341000119108",  # Instability of prosthesis of joint of right hip (disorder)
        "1254821007",  # Instability of prosthesis of joint of hip (disorder)
        "1255235009",  # Malposition of prosthetic joint of hip (disorder)
        "1287988000",  # Dislocation of prosthesis of hip joint following total hip replacement (disorder)
        "240267008",  # Loosening of hip joint prosthesis (disorder)
        "281443007",  # Loosening of hip joint hemiarthroplasty (disorder)
        "281445000",  # Loosening of total hip replacement (disorder)
        "281446004",  # Loosening of acetabular cup of total hip replacement (disorder)
        "281447008",  # Loosening of femoral stem of total hip replacement (disorder)
        "281448003",  # Loosening of knee joint prosthesis (disorder)
        "281449006",  # Loosening of unicondylar knee replacement (disorder)
        "281450006",  # Loosening of total knee replacement (disorder)
        "281451005",  # Loosening of femoral component of total knee replacement (disorder)
        "281452003",  # Loosening of tibial component of total knee replacement (disorder)
        "314201008",  # Dislocation of hip joint prosthesis (disorder)
        "429757000",  # Recurrent dislocation of hip joint prosthesis (disorder)
        "441565006",  # Wear of articular bearing surface of prosthetic joint (disorder)
        "735602002",  # Wear of articular bearing surface of joint prosthesis of hip (disorder)
        "735603007",  # Wear of articular bearing surface of joint prosthesis of knee (disorder)
    }

class BladderCancerForUrologyCare(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to identify concepts for diagnosis of bladder cancer.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of bladder cancer.

    **Exclusion Criteria:** Excludes concepts that identify a diagnosis for cancer elsewhere in the urinary tract other than the bladder.
    """

    VALUE_SET_NAME = "Bladder Cancer for Urology Care"
    OID = "2.16.840.1.113762.1.4.1248.354"
    DEFINITION_VERSION = "20250312"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "C670",  # Malignant neoplasm of trigone of bladder
        "C671",  # Malignant neoplasm of dome of bladder
        "C672",  # Malignant neoplasm of lateral wall of bladder
        "C673",  # Malignant neoplasm of anterior wall of bladder
        "C674",  # Malignant neoplasm of posterior wall of bladder
        "C675",  # Malignant neoplasm of bladder neck
        "C676",  # Malignant neoplasm of ureteric orifice
        "C677",  # Malignant neoplasm of urachus
        "C678",  # Malignant neoplasm of overlapping sites of bladder
        "C679",  # Malignant neoplasm of bladder, unspecified
    }

class ImmunocompromisedConditions(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnosis of immunocompromising or immunodeficient conditions.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that identify a diagnosis for an immunocompromising or immunodeficient conditions.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Immunocompromised Conditions"
    OID = "2.16.840.1.113762.1.4.1248.363"
    DEFINITION_VERSION = "20250312"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "B20",  # Human immunodeficiency virus [HIV] disease
        "B9735",  # Human immunodeficiency virus, type 2 [HIV 2] as the cause of diseases classified elsewhere
        "C8100",  # Nodular lymphocyte predominant Hodgkin lymphoma, unspecified site
        "C8101",  # Nodular lymphocyte predominant Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8102",  # Nodular lymphocyte predominant Hodgkin lymphoma, intrathoracic lymph nodes
        "C8103",  # Nodular lymphocyte predominant Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8104",  # Nodular lymphocyte predominant Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8105",  # Nodular lymphocyte predominant Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8106",  # Nodular lymphocyte predominant Hodgkin lymphoma, intrapelvic lymph nodes
        "C8107",  # Nodular lymphocyte predominant Hodgkin lymphoma, spleen
        "C8108",  # Nodular lymphocyte predominant Hodgkin lymphoma, lymph nodes of multiple sites
        "C8109",  # Nodular lymphocyte predominant Hodgkin lymphoma, extranodal and solid organ sites
        "C8110",  # Nodular sclerosis Hodgkin lymphoma, unspecified site
        "C8111",  # Nodular sclerosis Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8112",  # Nodular sclerosis Hodgkin lymphoma, intrathoracic lymph nodes
        "C8113",  # Nodular sclerosis Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8114",  # Nodular sclerosis Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8115",  # Nodular sclerosis Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8116",  # Nodular sclerosis Hodgkin lymphoma, intrapelvic lymph nodes
        "C8117",  # Nodular sclerosis Hodgkin lymphoma, spleen
        "C8118",  # Nodular sclerosis Hodgkin lymphoma, lymph nodes of multiple sites
        "C8119",  # Nodular sclerosis Hodgkin lymphoma, extranodal and solid organ sites
        "C8120",  # Mixed cellularity Hodgkin lymphoma, unspecified site
        "C8121",  # Mixed cellularity Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8122",  # Mixed cellularity Hodgkin lymphoma, intrathoracic lymph nodes
        "C8123",  # Mixed cellularity Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8124",  # Mixed cellularity Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8125",  # Mixed cellularity Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8126",  # Mixed cellularity Hodgkin lymphoma, intrapelvic lymph nodes
        "C8127",  # Mixed cellularity Hodgkin lymphoma, spleen
        "C8128",  # Mixed cellularity Hodgkin lymphoma, lymph nodes of multiple sites
        "C8129",  # Mixed cellularity Hodgkin lymphoma, extranodal and solid organ sites
        "C8130",  # Lymphocyte depleted Hodgkin lymphoma, unspecified site
        "C8131",  # Lymphocyte depleted Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8132",  # Lymphocyte depleted Hodgkin lymphoma, intrathoracic lymph nodes
        "C8133",  # Lymphocyte depleted Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8134",  # Lymphocyte depleted Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8135",  # Lymphocyte depleted Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8136",  # Lymphocyte depleted Hodgkin lymphoma, intrapelvic lymph nodes
        "C8137",  # Lymphocyte depleted Hodgkin lymphoma, spleen
        "C8138",  # Lymphocyte depleted Hodgkin lymphoma, lymph nodes of multiple sites
        "C8139",  # Lymphocyte depleted Hodgkin lymphoma, extranodal and solid organ sites
        "C8140",  # Lymphocyte-rich Hodgkin lymphoma, unspecified site
        "C8141",  # Lymphocyte-rich Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8142",  # Lymphocyte-rich Hodgkin lymphoma, intrathoracic lymph nodes
        "C8143",  # Lymphocyte-rich Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8144",  # Lymphocyte-rich Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8145",  # Lymphocyte-rich Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8146",  # Lymphocyte-rich Hodgkin lymphoma, intrapelvic lymph nodes
        "C8147",  # Lymphocyte-rich Hodgkin lymphoma, spleen
        "C8148",  # Lymphocyte-rich Hodgkin lymphoma, lymph nodes of multiple sites
        "C8149",  # Lymphocyte-rich Hodgkin lymphoma, extranodal and solid organ sites
        "C8170",  # Other Hodgkin lymphoma, unspecified site
        "C8171",  # Other Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8172",  # Other Hodgkin lymphoma, intrathoracic lymph nodes
        "C8173",  # Other Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8174",  # Other Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8175",  # Other Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8176",  # Other Hodgkin lymphoma, intrapelvic lymph nodes
        "C8177",  # Other Hodgkin lymphoma, spleen
        "C8178",  # Other Hodgkin lymphoma, lymph nodes of multiple sites
        "C8179",  # Other Hodgkin lymphoma, extranodal and solid organ sites
        "C8190",  # Hodgkin lymphoma, unspecified, unspecified site
        "C8191",  # Hodgkin lymphoma, unspecified, lymph nodes of head, face, and neck
        "C8192",  # Hodgkin lymphoma, unspecified, intrathoracic lymph nodes
        "C8193",  # Hodgkin lymphoma, unspecified, intra-abdominal lymph nodes
        "C8194",  # Hodgkin lymphoma, unspecified, lymph nodes of axilla and upper limb
        "C8195",  # Hodgkin lymphoma, unspecified, lymph nodes of inguinal region and lower limb
        "C8196",  # Hodgkin lymphoma, unspecified, intrapelvic lymph nodes
        "C8197",  # Hodgkin lymphoma, unspecified, spleen
        "C8198",  # Hodgkin lymphoma, unspecified, lymph nodes of multiple sites
        "C8199",  # Hodgkin lymphoma, unspecified, extranodal and solid organ sites
        "C8200",  # Follicular lymphoma grade I, unspecified site
        "C8201",  # Follicular lymphoma grade I, lymph nodes of head, face, and neck
        "C8202",  # Follicular lymphoma grade I, intrathoracic lymph nodes
        "C8203",  # Follicular lymphoma grade I, intra-abdominal lymph nodes
        "C8204",  # Follicular lymphoma grade I, lymph nodes of axilla and upper limb
        "C8205",  # Follicular lymphoma grade I, lymph nodes of inguinal region and lower limb
        "C8206",  # Follicular lymphoma grade I, intrapelvic lymph nodes
        "C8207",  # Follicular lymphoma grade I, spleen
        "C8208",  # Follicular lymphoma grade I, lymph nodes of multiple sites
        "C8209",  # Follicular lymphoma grade I, extranodal and solid organ sites
        "C8210",  # Follicular lymphoma grade II, unspecified site
        "C8211",  # Follicular lymphoma grade II, lymph nodes of head, face, and neck
        "C8212",  # Follicular lymphoma grade II, intrathoracic lymph nodes
        "C8213",  # Follicular lymphoma grade II, intra-abdominal lymph nodes
        "C8214",  # Follicular lymphoma grade II, lymph nodes of axilla and upper limb
        "C8215",  # Follicular lymphoma grade II, lymph nodes of inguinal region and lower limb
        "C8216",  # Follicular lymphoma grade II, intrapelvic lymph nodes
        "C8217",  # Follicular lymphoma grade II, spleen
        "C8218",  # Follicular lymphoma grade II, lymph nodes of multiple sites
        "C8219",  # Follicular lymphoma grade II, extranodal and solid organ sites
        "C8220",  # Follicular lymphoma grade III, unspecified, unspecified site
        "C8221",  # Follicular lymphoma grade III, unspecified, lymph nodes of head, face, and neck
        "C8222",  # Follicular lymphoma grade III, unspecified, intrathoracic lymph nodes
        "C8223",  # Follicular lymphoma grade III, unspecified, intra-abdominal lymph nodes
        "C8224",  # Follicular lymphoma grade III, unspecified, lymph nodes of axilla and upper limb
        "C8225",  # Follicular lymphoma grade III, unspecified, lymph nodes of inguinal region and lower limb
        "C8226",  # Follicular lymphoma grade III, unspecified, intrapelvic lymph nodes
        "C8227",  # Follicular lymphoma grade III, unspecified, spleen
        "C8228",  # Follicular lymphoma grade III, unspecified, lymph nodes of multiple sites
        "C8229",  # Follicular lymphoma grade III, unspecified, extranodal and solid organ sites
        "C8230",  # Follicular lymphoma grade IIIa, unspecified site
        "C8231",  # Follicular lymphoma grade IIIa, lymph nodes of head, face, and neck
        "C8232",  # Follicular lymphoma grade IIIa, intrathoracic lymph nodes
        "C8233",  # Follicular lymphoma grade IIIa, intra-abdominal lymph nodes
        "C8234",  # Follicular lymphoma grade IIIa, lymph nodes of axilla and upper limb
        "C8235",  # Follicular lymphoma grade IIIa, lymph nodes of inguinal region and lower limb
        "C8236",  # Follicular lymphoma grade IIIa, intrapelvic lymph nodes
        "C8237",  # Follicular lymphoma grade IIIa, spleen
        "C8238",  # Follicular lymphoma grade IIIa, lymph nodes of multiple sites
        "C8239",  # Follicular lymphoma grade IIIa, extranodal and solid organ sites
        "C8240",  # Follicular lymphoma grade IIIb, unspecified site
        "C8241",  # Follicular lymphoma grade IIIb, lymph nodes of head, face, and neck
        "C8242",  # Follicular lymphoma grade IIIb, intrathoracic lymph nodes
        "C8243",  # Follicular lymphoma grade IIIb, intra-abdominal lymph nodes
        "C8244",  # Follicular lymphoma grade IIIb, lymph nodes of axilla and upper limb
        "C8245",  # Follicular lymphoma grade IIIb, lymph nodes of inguinal region and lower limb
        "C8246",  # Follicular lymphoma grade IIIb, intrapelvic lymph nodes
        "C8247",  # Follicular lymphoma grade IIIb, spleen
        "C8248",  # Follicular lymphoma grade IIIb, lymph nodes of multiple sites
        "C8249",  # Follicular lymphoma grade IIIb, extranodal and solid organ sites
        "C8250",  # Diffuse follicle center lymphoma, unspecified site
        "C8251",  # Diffuse follicle center lymphoma, lymph nodes of head, face, and neck
        "C8252",  # Diffuse follicle center lymphoma, intrathoracic lymph nodes
        "C8253",  # Diffuse follicle center lymphoma, intra-abdominal lymph nodes
        "C8254",  # Diffuse follicle center lymphoma, lymph nodes of axilla and upper limb
        "C8255",  # Diffuse follicle center lymphoma, lymph nodes of inguinal region and lower limb
        "C8256",  # Diffuse follicle center lymphoma, intrapelvic lymph nodes
        "C8257",  # Diffuse follicle center lymphoma, spleen
        "C8258",  # Diffuse follicle center lymphoma, lymph nodes of multiple sites
        "C8259",  # Diffuse follicle center lymphoma, extranodal and solid organ sites
        "C8260",  # Cutaneous follicle center lymphoma, unspecified site
        "C8261",  # Cutaneous follicle center lymphoma, lymph nodes of head, face, and neck
        "C8262",  # Cutaneous follicle center lymphoma, intrathoracic lymph nodes
        "C8263",  # Cutaneous follicle center lymphoma, intra-abdominal lymph nodes
        "C8264",  # Cutaneous follicle center lymphoma, lymph nodes of axilla and upper limb
        "C8265",  # Cutaneous follicle center lymphoma, lymph nodes of inguinal region and lower limb
        "C8266",  # Cutaneous follicle center lymphoma, intrapelvic lymph nodes
        "C8267",  # Cutaneous follicle center lymphoma, spleen
        "C8268",  # Cutaneous follicle center lymphoma, lymph nodes of multiple sites
        "C8269",  # Cutaneous follicle center lymphoma, extranodal and solid organ sites
        "C8280",  # Other types of follicular lymphoma, unspecified site
        "C8281",  # Other types of follicular lymphoma, lymph nodes of head, face, and neck
        "C8282",  # Other types of follicular lymphoma, intrathoracic lymph nodes
        "C8283",  # Other types of follicular lymphoma, intra-abdominal lymph nodes
        "C8284",  # Other types of follicular lymphoma, lymph nodes of axilla and upper limb
        "C8285",  # Other types of follicular lymphoma, lymph nodes of inguinal region and lower limb
        "C8286",  # Other types of follicular lymphoma, intrapelvic lymph nodes
        "C8287",  # Other types of follicular lymphoma, spleen
        "C8288",  # Other types of follicular lymphoma, lymph nodes of multiple sites
        "C8289",  # Other types of follicular lymphoma, extranodal and solid organ sites
        "C8290",  # Follicular lymphoma, unspecified, unspecified site
        "C8291",  # Follicular lymphoma, unspecified, lymph nodes of head, face, and neck
        "C8292",  # Follicular lymphoma, unspecified, intrathoracic lymph nodes
        "C8293",  # Follicular lymphoma, unspecified, intra-abdominal lymph nodes
        "C8294",  # Follicular lymphoma, unspecified, lymph nodes of axilla and upper limb
        "C8295",  # Follicular lymphoma, unspecified, lymph nodes of inguinal region and lower limb
        "C8296",  # Follicular lymphoma, unspecified, intrapelvic lymph nodes
        "C8297",  # Follicular lymphoma, unspecified, spleen
        "C8298",  # Follicular lymphoma, unspecified, lymph nodes of multiple sites
        "C8299",  # Follicular lymphoma, unspecified, extranodal and solid organ sites
        "C8310",  # Mantle cell lymphoma, unspecified site
        "C8311",  # Mantle cell lymphoma, lymph nodes of head, face, and neck
        "C8312",  # Mantle cell lymphoma, intrathoracic lymph nodes
        "C8313",  # Mantle cell lymphoma, intra-abdominal lymph nodes
        "C8314",  # Mantle cell lymphoma, lymph nodes of axilla and upper limb
        "C8315",  # Mantle cell lymphoma, lymph nodes of inguinal region and lower limb
        "C8316",  # Mantle cell lymphoma, intrapelvic lymph nodes
        "C8317",  # Mantle cell lymphoma, spleen
        "C8318",  # Mantle cell lymphoma, lymph nodes of multiple sites
        "C8319",  # Mantle cell lymphoma, extranodal and solid organ sites
        "C8331",  # Diffuse large B-cell lymphoma, lymph nodes of head, face, and neck
        "C8332",  # Diffuse large B-cell lymphoma, intrathoracic lymph nodes
        "C8333",  # Diffuse large B-cell lymphoma, intra-abdominal lymph nodes
        "C8334",  # Diffuse large B-cell lymphoma, lymph nodes of axilla and upper limb
        "C8335",  # Diffuse large B-cell lymphoma, lymph nodes of inguinal region and lower limb
        "C8336",  # Diffuse large B-cell lymphoma, intrapelvic lymph nodes
        "C8337",  # Diffuse large B-cell lymphoma, spleen
        "C8338",  # Diffuse large B-cell lymphoma, lymph nodes of multiple sites
        "C8339",  # Diffuse large B-cell lymphoma, extranodal and solid organ sites
        "C8370",  # Burkitt lymphoma, unspecified site
        "C8371",  # Burkitt lymphoma, lymph nodes of head, face, and neck
        "C8372",  # Burkitt lymphoma, intrathoracic lymph nodes
        "C8373",  # Burkitt lymphoma, intra-abdominal lymph nodes
        "C8374",  # Burkitt lymphoma, lymph nodes of axilla and upper limb
        "C8375",  # Burkitt lymphoma, lymph nodes of inguinal region and lower limb
        "C8376",  # Burkitt lymphoma, intrapelvic lymph nodes
        "C8377",  # Burkitt lymphoma, spleen
        "C8378",  # Burkitt lymphoma, lymph nodes of multiple sites
        "C8379",  # Burkitt lymphoma, extranodal and solid organ sites
        "C8380",  # Other non-follicular lymphoma, unspecified site
        "C8381",  # Other non-follicular lymphoma, lymph nodes of head, face, and neck
        "C8382",  # Other non-follicular lymphoma, intrathoracic lymph nodes
        "C8383",  # Other non-follicular lymphoma, intra-abdominal lymph nodes
        "C8384",  # Other non-follicular lymphoma, lymph nodes of axilla and upper limb
        "C8385",  # Other non-follicular lymphoma, lymph nodes of inguinal region and lower limb
        "C8386",  # Other non-follicular lymphoma, intrapelvic lymph nodes
        "C8387",  # Other non-follicular lymphoma, spleen
        "C8388",  # Other non-follicular lymphoma, lymph nodes of multiple sites
        "C8389",  # Other non-follicular lymphoma, extranodal and solid organ sites
        "C8440",  # Peripheral T-cell lymphoma, not elsewhere classified, unspecified site
        "C8441",  # Peripheral T-cell lymphoma, not elsewhere classified, lymph nodes of head, face, and neck
        "C8442",  # Peripheral T-cell lymphoma, not elsewhere classified, intrathoracic lymph nodes
        "C8443",  # Peripheral T-cell lymphoma, not elsewhere classified, intra-abdominal lymph nodes
        "C8444",  # Peripheral T-cell lymphoma, not elsewhere classified, lymph nodes of axilla and upper limb
        "C8445",  # Peripheral T-cell lymphoma, not elsewhere classified, lymph nodes of inguinal region and lower limb
        "C8446",  # Peripheral T-cell lymphoma, not elsewhere classified, intrapelvic lymph nodes
        "C8447",  # Peripheral T-cell lymphoma, not elsewhere classified, spleen
        "C8448",  # Peripheral T-cell lymphoma, not elsewhere classified, lymph nodes of multiple sites
        "C8449",  # Peripheral T-cell lymphoma, not elsewhere classified, extranodal and solid organ sites
        "C8460",  # Anaplastic large cell lymphoma, ALK-positive, unspecified site
        "C8461",  # Anaplastic large cell lymphoma, ALK-positive, lymph nodes of head, face, and neck
        "C8462",  # Anaplastic large cell lymphoma, ALK-positive, intrathoracic lymph nodes
        "C8463",  # Anaplastic large cell lymphoma, ALK-positive, intra-abdominal lymph nodes
        "C8464",  # Anaplastic large cell lymphoma, ALK-positive, lymph nodes of axilla and upper limb
        "C8465",  # Anaplastic large cell lymphoma, ALK-positive, lymph nodes of inguinal region and lower limb
        "C8466",  # Anaplastic large cell lymphoma, ALK-positive, intrapelvic lymph nodes
        "C8467",  # Anaplastic large cell lymphoma, ALK-positive, spleen
        "C8468",  # Anaplastic large cell lymphoma, ALK-positive, lymph nodes of multiple sites
        "C8469",  # Anaplastic large cell lymphoma, ALK-positive, extranodal and solid organ sites
        "C8470",  # Anaplastic large cell lymphoma, ALK-negative, unspecified site
        "C8471",  # Anaplastic large cell lymphoma, ALK-negative, lymph nodes of head, face, and neck
        "C8472",  # Anaplastic large cell lymphoma, ALK-negative, intrathoracic lymph nodes
        "C8473",  # Anaplastic large cell lymphoma, ALK-negative, intra-abdominal lymph nodes
        "C8474",  # Anaplastic large cell lymphoma, ALK-negative, lymph nodes of axilla and upper limb
        "C8475",  # Anaplastic large cell lymphoma, ALK-negative, lymph nodes of inguinal region and lower limb
        "C8476",  # Anaplastic large cell lymphoma, ALK-negative, intrapelvic lymph nodes
        "C8477",  # Anaplastic large cell lymphoma, ALK-negative, spleen
        "C8478",  # Anaplastic large cell lymphoma, ALK-negative, lymph nodes of multiple sites
        "C8479",  # Anaplastic large cell lymphoma, ALK-negative, extranodal and solid organ sites
        "C8490",  # Mature T/NK-cell lymphomas, unspecified, unspecified site
        "C8491",  # Mature T/NK-cell lymphomas, unspecified, lymph nodes of head, face, and neck
        "C8492",  # Mature T/NK-cell lymphomas, unspecified, intrathoracic lymph nodes
        "C8493",  # Mature T/NK-cell lymphomas, unspecified, intra-abdominal lymph nodes
        "C8494",  # Mature T/NK-cell lymphomas, unspecified, lymph nodes of axilla and upper limb
        "C8495",  # Mature T/NK-cell lymphomas, unspecified, lymph nodes of inguinal region and lower limb
        "C8496",  # Mature T/NK-cell lymphomas, unspecified, intrapelvic lymph nodes
        "C8497",  # Mature T/NK-cell lymphomas, unspecified, spleen
        "C8498",  # Mature T/NK-cell lymphomas, unspecified, lymph nodes of multiple sites
        "C8499",  # Mature T/NK-cell lymphomas, unspecified, extranodal and solid organ sites
        "C84A0",  # Cutaneous T-cell lymphoma, unspecified, unspecified site
        "C84A1",  # Cutaneous T-cell lymphoma, unspecified lymph nodes of head, face, and neck
        "C84A2",  # Cutaneous T-cell lymphoma, unspecified, intrathoracic lymph nodes
        "C84A3",  # Cutaneous T-cell lymphoma, unspecified, intra-abdominal lymph nodes
        "C84A4",  # Cutaneous T-cell lymphoma, unspecified, lymph nodes of axilla and upper limb
        "C84A5",  # Cutaneous T-cell lymphoma, unspecified, lymph nodes of inguinal region and lower limb
        "C84A6",  # Cutaneous T-cell lymphoma, unspecified, intrapelvic lymph nodes
        "C84A7",  # Cutaneous T-cell lymphoma, unspecified, spleen
        "C84A8",  # Cutaneous T-cell lymphoma, unspecified, lymph nodes of multiple sites
        "C84A9",  # Cutaneous T-cell lymphoma, unspecified, extranodal and solid organ sites
        "C84Z0",  # Other mature T/NK-cell lymphomas, unspecified site
        "C84Z1",  # Other mature T/NK-cell lymphomas, lymph nodes of head, face, and neck
        "C84Z2",  # Other mature T/NK-cell lymphomas, intrathoracic lymph nodes
        "C84Z3",  # Other mature T/NK-cell lymphomas, intra-abdominal lymph nodes
        "C84Z4",  # Other mature T/NK-cell lymphomas, lymph nodes of axilla and upper limb
        "C84Z5",  # Other mature T/NK-cell lymphomas, lymph nodes of inguinal region and lower limb
        "C84Z6",  # Other mature T/NK-cell lymphomas, intrapelvic lymph nodes
        "C84Z7",  # Other mature T/NK-cell lymphomas, spleen
        "C84Z8",  # Other mature T/NK-cell lymphomas, lymph nodes of multiple sites
        "C84Z9",  # Other mature T/NK-cell lymphomas, extranodal and solid organ sites
        "C8510",  # Unspecified B-cell lymphoma, unspecified site
        "C8511",  # Unspecified B-cell lymphoma, lymph nodes of head, face, and neck
        "C8512",  # Unspecified B-cell lymphoma, intrathoracic lymph nodes
        "C8513",  # Unspecified B-cell lymphoma, intra-abdominal lymph nodes
        "C8514",  # Unspecified B-cell lymphoma, lymph nodes of axilla and upper limb
        "C8515",  # Unspecified B-cell lymphoma, lymph nodes of inguinal region and lower limb
        "C8516",  # Unspecified B-cell lymphoma, intrapelvic lymph nodes
        "C8517",  # Unspecified B-cell lymphoma, spleen
        "C8518",  # Unspecified B-cell lymphoma, lymph nodes of multiple sites
        "C8519",  # Unspecified B-cell lymphoma, extranodal and solid organ sites
        "C8520",  # Mediastinal (thymic) large B-cell lymphoma, unspecified site
        "C8521",  # Mediastinal (thymic) large B-cell lymphoma, lymph nodes of head, face, and neck
        "C8522",  # Mediastinal (thymic) large B-cell lymphoma, intrathoracic lymph nodes
        "C8523",  # Mediastinal (thymic) large B-cell lymphoma, intra-abdominal lymph nodes
        "C8524",  # Mediastinal (thymic) large B-cell lymphoma, lymph nodes of axilla and upper limb
        "C8525",  # Mediastinal (thymic) large B-cell lymphoma, lymph nodes of inguinal region and lower limb
        "C8526",  # Mediastinal (thymic) large B-cell lymphoma, intrapelvic lymph nodes
        "C8527",  # Mediastinal (thymic) large B-cell lymphoma, spleen
        "C8528",  # Mediastinal (thymic) large B-cell lymphoma, lymph nodes of multiple sites
        "C8529",  # Mediastinal (thymic) large B-cell lymphoma, extranodal and solid organ sites
        "C8580",  # Other specified types of non-Hodgkin lymphoma, unspecified site
        "C8581",  # Other specified types of non-Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8582",  # Other specified types of non-Hodgkin lymphoma, intrathoracic lymph nodes
        "C8583",  # Other specified types of non-Hodgkin lymphoma, intra-abdominal lymph nodes
        "C8584",  # Other specified types of non-Hodgkin lymphoma, lymph nodes of axilla and upper limb
        "C8585",  # Other specified types of non-Hodgkin lymphoma, lymph nodes of inguinal region and lower limb
        "C8586",  # Other specified types of non-Hodgkin lymphoma, intrapelvic lymph nodes
        "C8587",  # Other specified types of non-Hodgkin lymphoma, spleen
        "C8588",  # Other specified types of non-Hodgkin lymphoma, lymph nodes of multiple sites
        "C8589",  # Other specified types of non-Hodgkin lymphoma, extranodal and solid organ sites
        "C8590",  # Non-Hodgkin lymphoma, unspecified, unspecified site
        "C8591",  # Non-Hodgkin lymphoma, unspecified, lymph nodes of head, face, and neck
        "C8592",  # Non-Hodgkin lymphoma, unspecified, intrathoracic lymph nodes
        "C8593",  # Non-Hodgkin lymphoma, unspecified, intra-abdominal lymph nodes
        "C8594",  # Non-Hodgkin lymphoma, unspecified, lymph nodes of axilla and upper limb
        "C8595",  # Non-Hodgkin lymphoma, unspecified, lymph nodes of inguinal region and lower limb
        "C8596",  # Non-Hodgkin lymphoma, unspecified, intrapelvic lymph nodes
        "C8597",  # Non-Hodgkin lymphoma, unspecified, spleen
        "C8598",  # Non-Hodgkin lymphoma, unspecified, lymph nodes of multiple sites
        "C8599",  # Non-Hodgkin lymphoma, unspecified, extranodal and solid organ sites
        "C860",  # Extranodal NK/T-cell lymphoma, nasal type
        "C861",  # Hepatosplenic T-cell lymphoma
        "C862",  # Enteropathy-type (intestinal) T-cell lymphoma
        "C863",  # Subcutaneous panniculitis-like T-cell lymphoma
        "C864",  # Blastic NK-cell lymphoma
        "C866",  # Primary cutaneous CD30-positive T-cell proliferations
        "C884",  # Extranodal marginal zone B-cell lymphoma of mucosa-associated lymphoid tissue [MALT-lymphoma]
        "C9000",  # Multiple myeloma not having achieved remission
        "C9002",  # Multiple myeloma in relapse
        "C9010",  # Plasma cell leukemia not having achieved remission
        "C9012",  # Plasma cell leukemia in relapse
        "C9100",  # Acute lymphoblastic leukemia not having achieved remission
        "C9102",  # Acute lymphoblastic leukemia, in relapse
        "C9130",  # Prolymphocytic leukemia of B-cell type not having achieved remission
        "C9132",  # Prolymphocytic leukemia of B-cell type, in relapse
        "C9150",  # Adult T-cell lymphoma/leukemia (HTLV-1-associated) not having achieved remission
        "C9152",  # Adult T-cell lymphoma/leukemia (HTLV-1-associated), in relapse
        "C9160",  # Prolymphocytic leukemia of T-cell type not having achieved remission
        "C9162",  # Prolymphocytic leukemia of T-cell type, in relapse
        "C9190",  # Lymphoid leukemia, unspecified not having achieved remission
        "C9192",  # Lymphoid leukemia, unspecified, in relapse
        "C91A0",  # Mature B-cell leukemia Burkitt-type not having achieved remission
        "C91A2",  # Mature B-cell leukemia Burkitt-type, in relapse
        "C91Z0",  # Other lymphoid leukemia not having achieved remission
        "C91Z2",  # Other lymphoid leukemia, in relapse
        "C9200",  # Acute myeloblastic leukemia, not having achieved remission
        "C9202",  # Acute myeloblastic leukemia, in relapse
        "C9210",  # Chronic myeloid leukemia, BCR/ABL-positive, not having achieved remission
        "C9212",  # Chronic myeloid leukemia, BCR/ABL-positive, in relapse
        "C9220",  # Atypical chronic myeloid leukemia, BCR/ABL-negative, not having achieved remission
        "C9222",  # Atypical chronic myeloid leukemia, BCR/ABL-negative, in relapse
        "C9240",  # Acute promyelocytic leukemia, not having achieved remission
        "C9242",  # Acute promyelocytic leukemia, in relapse
        "C9250",  # Acute myelomonocytic leukemia, not having achieved remission
        "C9252",  # Acute myelomonocytic leukemia, in relapse
        "C9260",  # Acute myeloid leukemia with 11q23-abnormality not having achieved remission
        "C9262",  # Acute myeloid leukemia with 11q23-abnormality in relapse
        "C9290",  # Myeloid leukemia, unspecified, not having achieved remission
        "C9292",  # Myeloid leukemia, unspecified in relapse
        "C92A0",  # Acute myeloid leukemia with multilineage dysplasia, not having achieved remission
        "C92A2",  # Acute myeloid leukemia with multilineage dysplasia, in relapse
        "C92Z0",  # Other myeloid leukemia not having achieved remission
        "C92Z2",  # Other myeloid leukemia, in relapse
        "C9300",  # Acute monoblastic/monocytic leukemia, not having achieved remission
        "C9302",  # Acute monoblastic/monocytic leukemia, in relapse
        "C9310",  # Chronic myelomonocytic leukemia not having achieved remission
        "C9312",  # Chronic myelomonocytic leukemia, in relapse
        "C9330",  # Juvenile myelomonocytic leukemia, not having achieved remission
        "C9332",  # Juvenile myelomonocytic leukemia, in relapse
        "C9390",  # Monocytic leukemia, unspecified, not having achieved remission
        "C9392",  # Monocytic leukemia, unspecified in relapse
        "C93Z0",  # Other monocytic leukemia, not having achieved remission
        "C93Z2",  # Other monocytic leukemia, in relapse
        "C9420",  # Acute megakaryoblastic leukemia not having achieved remission
        "C9422",  # Acute megakaryoblastic leukemia, in relapse
        "C9430",  # Mast cell leukemia not having achieved remission
        "C9432",  # Mast cell leukemia, in relapse
        "C9480",  # Other specified leukemias not having achieved remission
        "C9482",  # Other specified leukemias, in relapse
        "C9500",  # Acute leukemia of unspecified cell type not having achieved remission
        "C9502",  # Acute leukemia of unspecified cell type, in relapse
        "C9510",  # Chronic leukemia of unspecified cell type not having achieved remission
        "C9512",  # Chronic leukemia of unspecified cell type, in relapse
        "C9590",  # Leukemia, unspecified not having achieved remission
        "C9592",  # Leukemia, unspecified, in relapse
        "C964",  # Sarcoma of dendritic cells (accessory cells)
        "C969",  # Malignant neoplasm of lymphoid, hematopoietic and related tissue, unspecified
        "C96Z",  # Other specified malignant neoplasms of lymphoid, hematopoietic and related tissue
        "D469",  # Myelodysplastic syndrome, unspecified
        "D46Z",  # Other myelodysplastic syndromes
        "D61818",  # Other pancytopenia
        "D700",  # Congenital agranulocytosis
        "D703",  # Neutropenia due to infection
        "D704",  # Cyclic neutropenia
        "D708",  # Other neutropenia
        "D709",  # Neutropenia, unspecified
        "D7381",  # Neutropenic splenomegaly
        "D800",  # Hereditary hypogammaglobulinemia
        "D801",  # Nonfamilial hypogammaglobulinemia
        "D802",  # Selective deficiency of immunoglobulin A [IgA]
        "D807",  # Transient hypogammaglobulinemia of infancy
        "D808",  # Other immunodeficiencies with predominantly antibody defects
        "D809",  # Immunodeficiency with predominantly antibody defects, unspecified
        "D810",  # Severe combined immunodeficiency [SCID] with reticular dysgenesis
        "D811",  # Severe combined immunodeficiency [SCID] with low T- and B-cell numbers
        "D812",  # Severe combined immunodeficiency [SCID] with low or normal B-cell numbers
        "D8131",  # Severe combined immunodeficiency due to adenosine deaminase deficiency
        "D8189",  # Other combined immunodeficiencies
        "D819",  # Combined immunodeficiency, unspecified
        "D820",  # Wiskott-Aldrich syndrome
        "D823",  # Immunodeficiency following hereditary defective response to Epstein-Barr virus
        "D828",  # Immunodeficiency associated with other specified major defects
        "D829",  # Immunodeficiency associated with major defect, unspecified
        "D830",  # Common variable immunodeficiency with predominant abnormalities of B-cell numbers and function
        "D831",  # Common variable immunodeficiency with predominant immunoregulatory T-cell disorders
        "D832",  # Common variable immunodeficiency with autoantibodies to B- or T-cells
        "D838",  # Other common variable immunodeficiencies
        "D839",  # Common variable immunodeficiency, unspecified
        "D841",  # Defects in the complement system
        "D8481",  # Immunodeficiency due to conditions classified elsewhere
        "D84821",  # Immunodeficiency due to drugs
        "D84822",  # Immunodeficiency due to external causes
        "D849",  # Immunodeficiency, unspecified
        "E70330",  # Chediak-Higashi syndrome
    }

    SNOMEDCT = {
        "109964000",  # Diffuse non-Hodgkin's lymphoma, undifferentiated (disorder)
        "109966003",  # Diffuse non-Hodgkin's lymphoma, immunoblastic (disorder)
        "109967007",  # Diffuse non-Hodgkin's lymphoma, small cleaved cell (disorder)
        "109968002",  # Diffuse non-Hodgkin's lymphoma, small cell (disorder)
        "109969005",  # Diffuse large B-cell malignant lymphoma (disorder)
        "109970006",  # Follicular non-Hodgkin's lymphoma, small cleaved cell (disorder)
        "109971005",  # Follicular non-Hodgkin's lymphoma, mixed small cleaved cell and large cell (disorder)
        "109972003",  # Follicular non-Hodgkin's lymphoma, large cell (disorder)
        "109975001",  # T-zone lymphoma (disorder)
        "109976000",  # Lymphoepithelioid lymphoma (disorder)
        "109977009",  # Peripheral T-cell lymphoma (disorder)
        "109978004",  # T-cell lymphoma (disorder)
        "109979007",  # B-cell lymphoma (disorder)
        "109989006",  # Multiple myeloma (disorder)
        "110002002",  # Mast cell leukemia (disorder)
        "110004001",  # Acute promyelocytic leukemia, FAB M3 (disorder)
        "110005000",  # Acute myelomonocytic leukemia, FAB M4 (disorder)
        "110006004",  # Prolymphocytic leukemia (disorder)
        "110007008",  # Adult T-cell leukemia/lymphoma (disorder)
        "111396008",  # Chediak-Higashi syndrome (disorder)
        "111585004",  # Neutropenia associated with autoimmune disease (disorder)
        "111587007",  # Severe combined immunodeficiency due to absent interleukin-2 receptor (disorder)
        "111880001",  # Acute human immunodeficiency virus infection (disorder)
        "1162928000",  # Acute myeloid leukemia (morphologic abnormality)
        "116711000119103",  # Marginal zone lymphoma of inguinal lymph node (disorder)
        "116741000119104",  # Marginal zone lymphoma of thoracic lymph node (disorder)
        "116811000119106",  # Non-Hodgkin lymphoma of central nervous system metastatic to lymph node of lower limb (disorder)
        "116821000119104",  # Non-Hodgkin lymphoma of central nervous system metastatic to lymph node of upper limb (disorder)
        "116841000119105",  # Marginal zone lymphoma of lymph nodes of multiple sites (disorder)
        "116871000119103",  # Mantle cell lymphoma of lymph nodes of multiple sites (disorder)
        "117061000119101",  # Peripheral T-cell lymphoma of spleen (disorder)
        "117091000119108",  # Peripheral T-cell lymphoma of axillary lymph node (disorder)
        "117111000119100",  # Peripheral T-cell lymphoma of thoracic lymph node (disorder)
        "117211000119105",  # Peripheral T-cell lymphoma of lymph nodes of multiple sites (disorder)
        "118599009",  # Hodgkin's disease (disorder)
        "118600007",  # Malignant lymphoma (disorder)
        "118601006",  # Non-Hodgkin's lymphoma (disorder)
        "118605002",  # Hodgkin lymphoma, nodular lymphocyte predominance (disorder)
        "118607005",  # Hodgkin lymphoma, lymphocyte-rich (disorder)
        "118613001",  # Hairy cell leukemia (disorder)
        "118617000",  # Burkitt's lymphoma (disorder)
        "1255796007",  # Burkitt lymphoma of lymph nodes of lower limb (disorder)
        "1255805006",  # Malignant lymphoma of lymph nodes of head (disorder)
        "1255806007",  # Malignant lymphoma of lymph nodes of neck (disorder)
        "1255807003",  # Malignant lymphoma of lymph nodes of head and neck (disorder)
        "1255895001",  # Burkitt lymphoma of lymph nodes of upper limb (disorder)
        "1255906008",  # Malignant lymphoma of lymph nodes of upper limb (disorder)
        "1255907004",  # Malignant lymphoma of lymph nodes of lower limb (disorder)
        "127034005",  # Pancytopenia (disorder)
        "127220001",  # Malignant lymphoma of lymph nodes (disorder)
        "127225006",  # Chronic myelomonocytic leukemia (disorder)
        "128804002",  # Primary cutaneous CD30 antigen positive T-cell lymphoproliferative disorder (morphologic abnormality)
        "128875000",  # Primary cutaneous CD30 antigen positive large T-cell lymphoma (disorder)
        "129641006",  # Chronic benign neutropenia of childhood (disorder)
        "129643009",  # Chronic hypoplastic neutropenia (disorder)
        "13048006",  # Orbital lymphoma (disorder)
        "133751000119102",  # Lymphoma of colon (disorder)
        "17182001",  # Agranulocytosis (disorder)
        "183005",  # Autoimmune pancytopenia (disorder)
        "186706006",  # Human immunodeficiency virus infection constitutional disease (disorder)
        "186707002",  # Human immunodeficiency virus infection with neurological disease (disorder)
        "186708007",  # Human immunodeficiency virus infection with secondary clinical infectious disease (disorder)
        "188511002",  # Burkitt's lymphoma of intrathoracic lymph nodes (disorder)
        "188512009",  # Burkitt's lymphoma of intra-abdominal lymph nodes (disorder)
        "188515006",  # Burkitt's lymphoma of intrapelvic lymph nodes (disorder)
        "188516007",  # Burkitt's lymphoma of spleen (disorder)
        "188517003",  # Burkitt's lymphoma of lymph nodes of multiple sites (disorder)
        "188572001",  # Hodgkin's disease, nodular sclerosis of lymph nodes of multiple sites (disorder)
        "188609000",  # Nodular lymphoma of lymph nodes of head, face and neck (disorder)
        "188612002",  # Nodular lymphoma of lymph nodes of axilla and upper limb (disorder)
        "188613007",  # Nodular lymphoma of lymph nodes of inguinal region and lower limb (disorder)
        "188672005",  # Follicular non-Hodgkin's mixed small cleaved and large cell lymphoma (disorder)
        "188674006",  # Diffuse malignant lymphoma - small non-cleaved cell (disorder)
        "188675007",  # Malignant lymphoma - small cleaved cell (disorder)
        "188676008",  # Malignant lymphoma - mixed small and large cell (disorder)
        "188725004",  # Lymphoid leukemia (disorder)
        "188726003",  # Subacute lymphoid leukemia (disorder)
        "188728002",  # Aleukemic lymphoid leukemia (disorder)
        "188732008",  # Myeloid leukemia (disorder)
        "188733003",  # Chronic eosinophilic leukemia (disorder)
        "188734009",  # Chronic neutrophilic leukemia (disorder)
        "188744006",  # Monocytic leukemia (disorder)
        "188754005",  # Megakaryocytic leukemia (disorder)
        "188768003",  # Myelomonocytic leukemia (disorder)
        "188770007",  # Subacute myelomonocytic leukemia (disorder)
        "190030009",  # Compound leukemias (disorder)
        "190979003",  # Selective immunoglobulin A deficiency (disorder)
        "190996002",  # Severe combined immunodeficiency with reticular dysgenesis (disorder)
        "190997006",  # Severe combined immunodeficiency with low T- and B-cell numbers (disorder)
        "190998001",  # Severe combined immunodeficiency with low or normal B-cell numbers (disorder)
        "191011000",  # Common variable immunodeficiency with predominant abnormalities of B-cell numbers and functions (disorder)
        "191012007",  # Common variable immunodeficiency with predominant immunoregulatory T-cell disorders (disorder)
        "191013002",  # Common variable immunodeficiency with autoantibodies to B- or T-cells (disorder)
        "191338000",  # Primary splenic neutropenia (disorder)
        "191347008",  # Cyclical neutropenia (disorder)
        "209962000",  # Myeloma-associated amyloidosis (disorder)
        "22406001",  # Severe combined immunodeficiency due to absent lymphoid stem cells (disorder)
        "230180003",  # Human immunodeficiency virus leukoencephalopathy (disorder)
        "232075002",  # Lymphoma of retina (disorder)
        "23238000",  # Common variable agammaglobulinemia (disorder)
        "234367000",  # Pancytopenia with pancreatitis (disorder)
        "234423001",  # Chronic benign neutropenia (disorder)
        "234424007",  # Metabolic neutropenia (disorder)
        "234425008",  # Autoimmune neutropenia (disorder)
        "234576008",  # Chronic familial neutropenia (disorder)
        "234621005",  # Factor I deficiency (disorder)
        "236513009",  # Lymphoma of kidney (disorder)
        "240531002",  # African Burkitt's lymphoma (disorder)
        "240794006",  # Splenic schistosomal giant cell lymphoma (disorder)
        "24743004",  # Complement deficiency disease (disorder)
        "247860002",  # Familial neutropenia (disorder)
        "248693006",  # Chronic idiopathic neutropenia (disorder)
        "267540007",  # Neutropenia caused by ionizing radiation (disorder)
        "269476000",  # Nodular lymphoma (disorder)
        "274905008",  # Malignant lymphoma - lymphocytic, intermediate differentiation (disorder)
        "276811008",  # Gastric lymphoma (disorder)
        "276815004",  # Lymphoma of intestine (disorder)
        "276836002",  # Primary cerebral lymphoma (disorder)
        "277474005",  # B-cell chronic lymphocytic leukemia variant (disorder)
        "277545003",  # T-cell chronic lymphocytic leukemia (disorder)
        "277551008",  # Splenic lymphoma with villous lymphocytes (disorder)
        "277567002",  # T-cell prolymphocytic leukemia (disorder)
        "277568007",  # Hairy cell leukemia variant (disorder)
        "277569004",  # Large granular lymphocytic leukemia (disorder)
        "277570003",  # Lymphoma with spill (disorder)
        "277571004",  # B-cell acute lymphoblastic leukemia (disorder)
        "277572006",  # Precursor B-cell acute lymphoblastic leukemia (disorder)
        "277573001",  # Common acute lymphoblastic leukemia (disorder)
        "277574007",  # Null cell acute lymphoblastic leukemia (disorder)
        "277575008",  # T-lymphoblastic leukemia (disorder)
        "277579002",  # Light chain myeloma (disorder)
        "277580004",  # Non-secretory myeloma (disorder)
        "277587001",  # Juvenile chronic myeloid leukemia (disorder)
        "277589003",  # Myelodysplastic-myeloproliferative neoplasm with neutrophilia (disorder)
        "277602003",  # Acute megakaryoblastic leukemia (disorder)
        "277604002",  # Malignant myeloid-lymphoid neoplasm (disorder)
        "277613000",  # Cutaneous/peripheral T-cell lymphoma (disorder)
        "277614006",  # Prethymic and thymic T-cell lymphoma/leukemia (disorder)
        "277615007",  # Low grade B-cell lymphoma (disorder)
        "277616008",  # Diffuse low grade B-cell lymphoma (disorder)
        "277617004",  # High grade B-cell lymphoma (disorder)
        "277618009",  # Follicular low grade B-cell lymphoma (disorder)
        "277619001",  # B-cell prolymphocytic leukemia (disorder)
        "277622004",  # Mucosa-associated lymphoma (disorder)
        "277623009",  # Monocytoid B-cell lymphoma (disorder)
        "277624003",  # Follicular malignant lymphoma - mixed cell type (disorder)
        "277625002",  # Follicular malignant lymphoma - small cleaved cell (disorder)
        "277626001",  # Diffuse high grade B-cell lymphoma (disorder)
        "277627005",  # Nodular high grade B-cell lymphoma (disorder)
        "277628000",  # Diffuse malignant lymphoma - large cleaved cell (disorder)
        "277629008",  # Diffuse malignant lymphoma - large non-cleaved cell (disorder)
        "277632006",  # Diffuse malignant lymphoma - centroblastic polymorphic (disorder)
        "277642008",  # Low grade T-cell lymphoma (disorder)
        "277643003",  # High grade T-cell lymphoma (disorder)
        "277654008",  # Enteropathy-associated T-cell lymphoma (disorder)
        "277664004",  # Malignant lymphoma of testis (disorder)
        "278051002",  # Malignant lymphoma of thyroid gland (disorder)
        "278052009",  # Malignant lymphoma of breast (disorder)
        "278189009",  # Hypergranular promyelocytic leukemia (disorder)
        "278453007",  # Acute biphenotypic leukemia (disorder)
        "285420006",  # Immunoglobulin A myeloma (disorder)
        "285421005",  # Immunoglobulin G myeloma (disorder)
        "285422003",  # Immunoglobulin D myeloma (disorder)
        "285769009",  # Acute promyelocytic leukemia - hypogranular variant (disorder)
        "285776004",  # Intermediate grade B-cell lymphoma (disorder)
        "302841002",  # Malignant lymphoma - small lymphocytic (disorder)
        "302842009",  # Diffuse malignant lymphoma - centroblastic (disorder)
        "302845006",  # Nodular malignant lymphoma, lymphocytic - well differentiated (disorder)
        "302848008",  # Nodular malignant lymphoma, lymphocytic - intermediate differentiation (disorder)
        "302855005",  # Subacute leukemia (disorder)
        "302856006",  # Aleukemic leukemia (disorder)
        "303011007",  # Neutropenic disorder (disorder)
        "303055001",  # Malignant lymphoma, follicular center cell (disorder)
        "303056000",  # Malignant lymphoma, follicular center cell, cleaved (disorder)
        "303057009",  # Malignant lymphoma, follicular center cell, non-cleaved (disorder)
        "30575002",  # Fanconi's anemia (disorder)
        "307341004",  # Atypical hairy cell leukemia (disorder)
        "307592006",  # Basophilic leukemia (disorder)
        "307617006",  # Neutrophilic leukemia (disorder)
        "307623001",  # Malignant lymphoma - lymphoplasmacytic (disorder)
        "307624007",  # Diffuse malignant lymphoma - centroblastic-centrocytic (disorder)
        "307625008",  # Malignant lymphoma - centrocytic (disorder)
        "307636001",  # Malignant lymphoma, mixed lymphocytic-histiocytic, nodular (disorder)
        "307637005",  # Malignant lymphoma, centroblastic-centrocytic, follicular (disorder)
        "307646004",  # Malignant lymphoma, lymphocytic, poorly differentiated, nodular (disorder)
        "307647008",  # Malignant lymphoma, centroblastic type, follicular (disorder)
        "307649006",  # Microglioma (disorder)
        "308121000",  # Follicular lymphoma (disorder)
        "31323000",  # Severe combined immunodeficiency disease (disorder)
        "313427003",  # Lambda light chain myeloma (disorder)
        "32092008",  # Toxic neutropenia (disorder)
        "32278006",  # Myeloma kidney (disorder)
        "3439009",  # Severe combined immunodeficiency due to absent peripheral T cell maturation (disorder)
        "350951000119101",  # B-cell lymphoma of intra-abdominal lymph nodes (disorder)
        "351211000119104",  # B-cell lymphoma of lymph nodes of multiple sites (disorder)
        "352251000119109",  # Small lymphocytic B-cell lymphoma of lymph nodes of multiple sites (disorder)
        "352411000119109",  # Small lymphocytic B-cell lymphoma of intra-abdominal lymph nodes (disorder)
        "352791000119108",  # Non-Hodgkin's lymphoma of lymph nodes of multiple sites (disorder)
        "354851000119101",  # Follicular non-Hodgkin's lymphoma of lymph nodes of multiple sites (disorder)
        "359631009",  # Acute myeloid leukemia, minimal differentiation, FAB M0 (disorder)
        "359640008",  # Acute myeloid leukemia without maturation, FAB M1 (disorder)
        "359648001",  # Acute myeloid leukemia with maturation, FAB M2 (disorder)
        "36070007",  # Wiskott-Aldrich syndrome (disorder)
        "362993009",  # Autosomal recessive severe combined immunodeficiency disease (disorder)
        "363009005",  # Complement component deficiency (disorder)
        "371012000",  # Acute lymphoblastic leukemia, transitional pre-B-cell (disorder)
        "371134001",  # Malignant lymphoma, large cell, polymorphous, immunoblastic (disorder)
        "38970002",  # Doan-Wright syndrome (disorder)
        "400001003",  # Primary cutaneous lymphoma (disorder)
        "400122007",  # Primary cutaneous T-cell lymphoma (disorder)
        "402880009",  # Primary cutaneous large T-cell lymphoma (disorder)
        "402881008",  # Primary cutaneous B-cell lymphoma (disorder)
        "403837005",  # Wiskott-Aldrich autosomal dominant variant syndrome (disorder)
        "404111002",  # Lymphomatoid papulosis-associated mycosis fungoides (disorder)
        "404122003",  # Leukemic infiltration of skin (chronic T-cell lymphocytic leukemia) (disorder)
        "404123008",  # Leukemic infiltration of skin (T-cell prolymphocytic leukemia) (disorder)
        "404124002",  # Leukemic infiltration of skin (T-cell lymphoblastic leukemia) (disorder)
        "404126000",  # CD-30 positive pleomorphic large T-cell cutaneous lymphoma (disorder)
        "404127009",  # CD-30 positive T-immunoblastic cutaneous lymphoma (disorder)
        "404128004",  # CD-30 negative cutaneous T-cell lymphoma (disorder)
        "404129007",  # CD-30 negative anaplastic large T-cell cutaneous lymphoma (disorder)
        "404130002",  # CD-30 negative pleomorphic large T-cell cutaneous lymphoma (disorder)
        "404131003",  # CD-30 negative T-immunoblastic cutaneous lymphoma (disorder)
        "404133000",  # Subcutaneous panniculitic cutaneous T-cell lymphoma (disorder)
        "404134006",  # Anaplastic large T-cell systemic malignant lymphoma (disorder)
        "404135007",  # Angiocentric natural killer/T-cell malignant lymphoma involving skin (disorder)
        "404136008",  # Aggressive natural killer-cell leukemia involving skin (disorder)
        "404137004",  # Precursor B-cell lymphoblastic lymphoma involving skin (disorder)
        "404138009",  # Small lymphocytic B-cell lymphoma involving skin (disorder)
        "404139001",  # Leukemic infiltration of skin in hairy-cell leukemia (disorder)
        "404140004",  # Primary cutaneous marginal zone B-cell lymphoma (disorder)
        "404143002",  # Primary cutaneous follicular center B-cell lymphoma (disorder)
        "404144008",  # Primary cutaneous diffuse large cell B-cell lymphoma (disorder)
        "404147001",  # Follicular center B-cell lymphoma (nodal/systemic with skin involvement) (disorder)
        "404148006",  # Diffuse large B-cell lymphoma (nodal/systemic with skin involvement) (disorder)
        "404149003",  # Lymphoplasmacytic B-cell lymphoma, nodal/systemic with skin involvement (disorder)
        "404150003",  # Mantle cell B-cell lymphoma (nodal/systemic with skin involvement) (disorder)
        "404151004",  # Leukemic infiltration of skin in myeloid leukemia (disorder)
        "404152006",  # Leukemic infiltration of skin in acute myeloid leukemia (disorder)
        "404153001",  # Leukemic infiltration of skin in chronic myeloid leukemia (disorder)
        "404154007",  # Leukemic infiltration of skin in monocytic leukemia (disorder)
        "404172001",  # Mast cell leukemia affecting skin (disorder)
        "409089005",  # Febrile neutropenia (disorder)
        "413389003",  # Accelerated phase chronic myeloid leukemia (disorder)
        "413441006",  # Acute monocytic leukemia (disorder)
        "413537009",  # Angioimmunoblastic T-cell lymphoma (disorder)
        "413587002",  # Smoldering myeloma (disorder)
        "413656006",  # Blastic phase chronic myeloid leukemia (disorder)
        "413842007",  # Chronic myeloid leukemia in lymphoid blast crisis (disorder)
        "413843002",  # Chronic myeloid leukemia in myeloid blast crisis (disorder)
        "413847001",  # Chronic phase chronic myeloid leukemia (disorder)
        "414166008",  # Extranodal natural killer/T-cell lymphoma, nasal type (disorder)
        "414553000",  # Kappa light chain myeloma (disorder)
        "414780005",  # Mucosa-associated lymphoid tissue lymphoma of orbit (disorder)
        "415287001",  # Relapsing chronic myeloid leukemia (disorder)
        "416729007",  # Neutropenia with acquired immunodeficiency syndrome (disorder)
        "417321001",  # Human T-cell Lymphoma Virus Type -1 associated uveitis (disorder)
        "41814009",  # Neutropenia with dysgranulopoiesis (disorder)
        "420519005",  # Malignant lymphoma of the eye region (disorder)
        "420788006",  # Intraocular non-Hodgkin malignant lymphoma (disorder)
        "420890002",  # T-lymphoblastic leukemia/lymphoblastic lymphoma (disorder)
        "421246008",  # T-lymphoblastic lymphoma (disorder)
        "421283008",  # Primary lymphoma of brain with acquired immunodeficiency syndrome (disorder)
        "421312009",  # Agranulocytosis with acquired immunodeficiency syndrome (disorder)
        "422853008",  # Lymphoma of retroperitoneal space (disorder)
        "425657001",  # Osteosclerotic myeloma (disorder)
        "425688002",  # Philadelphia chromosome-positive acute lymphoblastic leukemia (disorder)
        "426336007",  # Solitary osseous myeloma (disorder)
        "427141003",  # Malignant lymphoma in remission (disorder)
        "440422002",  # Asymptomatic multiple myeloma (disorder)
        "441313008",  # Indolent multiple myeloma (disorder)
        "441559006",  # Mantle cell lymphoma of spleen (disorder)
        "441962003",  # Large cell lymphoma of intrapelvic lymph nodes (disorder)
        "442537007",  # Non-Hodgkin lymphoma associated with Human immunodeficiency virus infection (disorder)
        "443487006",  # Mantle cell lymphoma (disorder)
        "444597005",  # Extranodal marginal zone lymphoma of mucosa-associated lymphoid tissue of stomach (disorder)
        "444910004",  # Primary mediastinal (thymic) large B-cell lymphoma (disorder)
        "444911000",  # Acute myeloid leukemia with t(9:11)(p22;q23); MLLT3-MLL (disorder)
        "445227008",  # Juvenile myelomonocytic leukemia (disorder)
        "445269007",  # Extranodal marginal zone B-cell lymphoma of mucosa-associated lymphoid tissue (disorder)
        "445406001",  # Hepatosplenic T-cell lymphoma (disorder)
        "445448008",  # Acute myeloid leukemia with myelodysplasia-related changes (disorder)
        "447100004",  # Marginal zone lymphoma (disorder)
        "447656001",  # Lymphoma of pylorus of stomach (disorder)
        "447658000",  # Lymphoma of fundus of stomach (disorder)
        "447766003",  # Lymphoma of pyloric antrum of stomach (disorder)
        "447805007",  # Lymphoma of greater curvature of stomach (disorder)
        "447806008",  # Lymphoma of cardia of stomach (disorder)
        "447989004",  # Non-Hodgkin's lymphoma of extranodal site (disorder)
        "448212009",  # Anaplastic lymphoma kinase negative anaplastic large cell lymphoma (disorder)
        "448213004",  # Diffuse non-Hodgkin's lymphoma of prostate (disorder)
        "448217003",  # Follicular non-Hodgkin's lymphoma of prostate (disorder)
        "448220006",  # Non-Hodgkin's lymphoma of bone (disorder)
        "448231003",  # Follicular non-Hodgkin's lymphoma of nose (disorder)
        "448254007",  # Non-Hodgkin's lymphoma of central nervous system (disorder)
        "448269008",  # Lymphoma of lesser curvature of stomach (disorder)
        "448317000",  # Follicular non-Hodgkin's lymphoma of soft tissue (disorder)
        "448319002",  # Diffuse non-Hodgkin's lymphoma of nasopharynx (disorder)
        "448354009",  # Non-Hodgkin's lymphoma of intestine (disorder)
        "448371005",  # Non-Hodgkin's lymphoma of nasopharynx (disorder)
        "448372003",  # Non-Hodgkin's lymphoma of lung (disorder)
        "448376000",  # Non-Hodgkin's lymphoma of ovary (disorder)
        "448384001",  # Non-Hodgkin's lymphoma of nose (disorder)
        "448386004",  # Non-Hodgkin's lymphoma of oral cavity (disorder)
        "448387008",  # Non-Hodgkin's lymphoma of testis (disorder)
        "448447004",  # Non-Hodgkin's lymphoma of skin (disorder)
        "448465000",  # Diffuse non-Hodgkin's lymphoma of testis (disorder)
        "448468003",  # Diffuse non-Hodgkin's lymphoma of oral cavity (disorder)
        "448553002",  # Lymphoma of pelvis (disorder)
        "448555009",  # Lymphoma of body of stomach (disorder)
        "448560008",  # Diffuse non-Hodgkin's lymphoma of extranodal site (disorder)
        "448561007",  # Follicular non-Hodgkin's lymphoma of extranodal site (disorder)
        "448607004",  # Diffuse non-Hodgkin's lymphoma of uterine cervix (disorder)
        "448609001",  # Diffuse non-Hodgkin's lymphoma of ovary (disorder)
        "448663003",  # Diffuse non-Hodgkin's lymphoma of stomach (disorder)
        "448666006",  # Follicular non-Hodgkin's lymphoma of bone (disorder)
        "448672006",  # Follicular non-Hodgkin's lymphoma of lung (disorder)
        "448709005",  # Non-Hodgkin's lymphoma of stomach (disorder)
        "448738008",  # Non-Hodgkin's lymphoma of soft tissue (disorder)
        "448774004",  # Non-Hodgkin's lymphoma of uterine cervix (disorder)
        "448865007",  # Follicular non-Hodgkin's lymphoma of skin (disorder)
        "448867004",  # Diffuse non-Hodgkin's lymphoma of lung (disorder)
        "448995000",  # Follicular non-Hodgkin's lymphoma of central nervous system (disorder)
        "449053004",  # Lymphoma of lower esophagus (disorder)
        "449058008",  # Follicular non-Hodgkin's lymphoma of tonsil (disorder)
        "449059000",  # Follicular non-Hodgkin's lymphoma of uterine cervix (disorder)
        "449063007",  # Follicular non-Hodgkin's lymphoma of oral cavity (disorder)
        "449065000",  # Diffuse non-Hodgkin's lymphoma of nose (disorder)
        "449072004",  # Lymphoma of gastrointestinal tract (disorder)
        "449074003",  # Lymphoma of small intestine (disorder)
        "449075002",  # Lymphoma of cardioesophageal junction (disorder)
        "449173006",  # Diffuse non-Hodgkin's lymphoma of tonsil (disorder)
        "449176003",  # Diffuse non-Hodgkin's lymphoma of intestine (disorder)
        "449177007",  # Diffuse non-Hodgkin's lymphoma of bone (disorder)
        "449216004",  # Diffuse non-Hodgkin's lymphoma of soft tissue (disorder)
        "449217008",  # Diffuse non-Hodgkin's lymphoma of skin (disorder)
        "449218003",  # Lymphoma of sigmoid colon (disorder)
        "449219006",  # Follicular non-Hodgkin's lymphoma of nasopharynx (disorder)
        "449220000",  # Diffuse follicle center lymphoma (disorder)
        "449221001",  # Diffuse non-Hodgkin's lymphoma of central nervous system (disorder)
        "449222008",  # Follicular non-Hodgkin's lymphoma of stomach (disorder)
        "449292003",  # Non-Hodgkin's lymphoma of tonsil (disorder)
        "449307001",  # Follicular non-Hodgkin's lymphoma of ovary (disorder)
        "449318001",  # Non-Hodgkin's lymphoma of prostate (disorder)
        "449418000",  # Follicular non-Hodgkin's lymphoma of testis (disorder)
        "449419008",  # Follicular non-Hodgkin's lymphoma of intestine (disorder)
        "46359005",  # Neutropenia associated with infectious disease (disorder)
        "49555001",  # Severe combined immunodeficiency due to absent T cell receptor (disorder)
        "55444004",  # Transient neonatal neutropenia (disorder)
        "58639003",  # Doan-Wiseman syndrome (disorder)
        "5876000",  # Acquired pancytopenia (disorder)
        "65623009",  # Immune neutropenia (disorder)
        "65880007",  # X-linked agammaglobulinemia (disorder)
        "68261000119102",  # Diffuse non-Hodgkin's lymphoma Lugano stage I (disorder)
        "68271000119108",  # Diffuse non-Hodgkin's lymphoma Lugano stage II (disorder)
        "68291000119109",  # Diffuse non-Hodgkin's lymphoma Lugano stage IV (disorder)
        "68504005",  # Ataxia-telangiectasia syndrome (disorder)
        "699657009",  # Hepatosplenic gamma-delta cell lymphoma (disorder)
        "700058006",  # Ataxia-telangiectasia-like disorder (disorder)
        "702785000",  # Large cell anaplastic lymphoma T cell and Null cell type (disorder)
        "702786004",  # Follicular non-Hodgkin's lymphoma diffuse follicle center sub-type grade 1 (disorder)
        "702977001",  # Follicular non-Hodgkin's lymphoma diffuse follicle center cell sub-type grade 2 (disorder)
        "703626001",  # Anaplastic large cell lymphoma, T/Null cell, primary systemic type (disorder)
        "713325002",  # Primary cerebral lymphoma co-occurrent with human immunodeficiency virus infection (disorder)
        "713516007",  # Primary effusion lymphoma (disorder)
        "713530002",  # Agranulocytosis co-occurrent with human immunodeficiency virus infection (disorder)
        "713718006",  # Diffuse non-Hodgkin immunoblastic lymphoma co-occurrent with human immunodeficiency virus infection (disorder)
        "713897006",  # Burkitt lymphoma co-occurrent with human immunodeficiency virus infection (disorder)
        "714463003",  # Primary effusion lymphoma co-occurrent with infection caused by Human herpesvirus 8 (disorder)
        "715950008",  # Anaplastic lymphoma kinase positive large B-cell lymphoma (disorder)
        "716788007",  # Epstein-Barr virus positive diffuse large B-cell lymphoma of elderly (disorder)
        "718200007",  # Primary pulmonary lymphoma (disorder)
        "720853005",  # Severe combined immunodeficiency, microcephaly, growth retardation, sensitivity to ionizing radiation syndrome (disorder)
        "721555001",  # Follicular lymphoma of small intestine (disorder)
        "721762007",  # Adult T-cell leukemia/lymphoma of skin (disorder)
        "722067005",  # Severe combined immunodeficiency with hypereosinophilia (disorder)
        "722954005",  # B-cell lymphoma unclassifiable with features intermediate between classical Hodgkin lymphoma and diffuse large B-cell lymphoma (disorder)
        "723889003",  # B lymphoblastic leukemia lymphoma with t(9:22) (q34;q11.2); BCR-ABL 1 (disorder)
        "724645006",  # T-cell histiocyte rich large B-cell lymphoma (disorder)
        "724647003",  # Diffuse large B-cell lymphoma co-occurrent with chronic inflammation caused by Epstein-Barr virus (disorder)
        "724648008",  # Plasmablastic lymphoma (disorder)
        "726721002",  # Nodal marginal zone B-cell lymphoma (disorder)
        "733627006",  # Primary cutaneous gamma-delta-positive T-cell lymphoma (disorder)
        "734066005",  # Diffuse large B-cell lymphoma of central nervous system (disorder)
        "735332000",  # Primary cutaneous diffuse large cell B-cell lymphoma of lower extremity (disorder)
        "736322001",  # Pediatric follicular lymphoma (disorder)
        "738770003",  # Anaplastic lymphoma kinase positive anaplastic large cell lymphoma (disorder)
        "762690000",  # Classical Hodgkin lymphoma (disorder)
        "763065008",  # Ataxia telangiectasia variant (disorder)
        "763477007",  # Primary lymphoma of conjunctiva (disorder)
        "763666008",  # Splenic marginal zone B-cell lymphoma (disorder)
        "763719001",  # Hydroa vacciniforme-like lymphoma (disorder)
        "763884007",  # Splenic diffuse red pulp small B-cell lymphoma (disorder)
        "765136002",  # Primary cutaneous CD8 positive aggressive epidermotropic cytotoxic T-cell lymphoma (disorder)
        "766935007",  # Primary bone lymphoma (disorder)
        "771443008",  # Complement component 3 deficiency (disorder)
        "773488000",  # Combined immunodeficiency due to mucosa-associated lymphoid tissue lymphoma translocation gene 1 deficiency (disorder)
        "773995001",  # Primary cutaneous anaplastic large cell lymphoma (disorder)
        "782919005",  # Classical Hodgkin lymphoma type posttransplant lymphoproliferative disorder (morphologic abnormality)
        "783541009",  # Breast implant-associated anaplastic large-cell lymphoma (disorder)
        "78378009",  # Isoimmune neutropenia (disorder)
        "838355002",  # Acute myeloid leukemia with CBFB::MYH11 fusion (disorder)
        "89655007",  # Congenital neutropenia (disorder)
        "91855006",  # Acute leukemia, disease (disorder)
        "91857003",  # Acute lymphoid leukemia, disease (disorder)
        "91861009",  # Acute myeloid leukemia (disorder)
        "92812005",  # Chronic leukemia, disease (disorder)
        "92818009",  # Chronic myeloid leukemia (disorder)
        "93143009",  # Leukemia, disease (disorder)
        "93151007",  # Hairy cell leukemia of spleen (disorder)
        "93191005",  # Malignant lymphoma of intra-abdominal lymph nodes (disorder)
        "93192003",  # Malignant lymphoma of intrapelvic lymph nodes (disorder)
        "93193008",  # Malignant lymphoma of intrathoracic lymph nodes (disorder)
        "93197009",  # Malignant lymphoma of lymph nodes of multiple sites (disorder)
        "93198004",  # Malignant lymphoma of spleen (disorder)
        "93199007",  # Malignant lymphoma of extranodal AND/OR solid organ site (disorder)
        "94392001",  # Metastatic malignant neoplasm to lymph node (disorder)
        "94396003",  # Metastatic malignant neoplasm to lymph nodes of multiple sites (disorder)
        "95186006",  # Nodular lymphoma of intra-abdominal lymph nodes (disorder)
        "95187002",  # Nodular lymphoma of intrapelvic lymph nodes (disorder)
        "95188007",  # Nodular lymphoma of intrathoracic lymph nodes (disorder)
        "95192000",  # Nodular lymphoma of lymph nodes of multiple sites (disorder)
        "95193005",  # Nodular lymphoma of spleen (disorder)
        "95210003",  # Plasma cell leukemia, disease (disorder)
    }

class MixedHistologyUrothelialCellCarcinomaForUrologyCare(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to identify concepts of mixed histology urothelial cell abnormalities.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that identify a diagnosis of urothelial cell abnormalities.

    **Exclusion Criteria:** Excludes concepts that indicate a condition for other urothelial cell abnormalities in the urinary bladder.
    """

    VALUE_SET_NAME = "Mixed Histology Urothelial Cell Carcinoma for Urology Care"
    OID = "2.16.840.1.113762.1.4.1248.365"
    DEFINITION_VERSION = "20250312"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    SNOMEDCT = {
        "1208459005",  # Micropapillary urothelial carcinoma (disorder)
        "1208460000",  # Sarcomatoid urothelial carcinoma of urinary bladder (disorder)
        "1208461001",  # Plasmacytoid urothelial carcinoma of urinary bladder (disorder)
        "1217020002",  # Sarcomatoid urothelial carcinoma of ureter (disorder)
        "1259402009",  # Primary adenocarcinoma of urinary bladder (disorder)
        "1259705005",  # Primary sarcomatoid urothelial carcinoma of urinary bladder (disorder)
        "1259707002",  # Primary sarcomatoid urothelial carcinoma of ureter (disorder)
        "1259709004",  # Primary micropapillary urothelial carcinoma (disorder)
        "1259712001",  # Primary plasmacytoid urothelial carcinoma of urinary bladder (disorder)
        "255110003",  # Adenocarcinoma of urinary bladder (disorder)
        "255111004",  # Squamous cell carcinoma of urinary bladder (disorder)
        "278046008",  # Sarcoma of urinary bladder (disorder)
        "681591000119108",  # Primary adenocarcinoma of anterior wall of urinary bladder (disorder)
        "681611000119103",  # Primary adenocarcinoma of neck of urinary bladder (disorder)
        "681661000119100",  # Primary adenocarcinoma of dome of urinary bladder (disorder)
        "681771000119102",  # Primary adenocarcinoma of lateral wall of urinary bladder (disorder)
        "681921000119101",  # Primary adenocarcinoma of posterior wall of urinary bladder (disorder)
        "681991000119104",  # Primary adenocarcinoma of trigone of urinary bladder (disorder)
        "688691000119102",  # Primary squamous cell carcinoma of anterior wall of urinary bladder (disorder)
        "688731000119109",  # Primary squamous cell carcinoma of neck of urinary bladder (disorder)
        "688801000119109",  # Primary squamous cell carcinoma of dome of urinary bladder (disorder)
        "688941000119102",  # Primary squamous cell carcinoma of lateral wall of urinary bladder (disorder)
        "689231000119107",  # Primary squamous cell carcinoma of posterior wall of urinary bladder (disorder)
        "689441000119102",  # Primary squamous cell carcinoma of ureteral orifice (disorder)
    }

class TuberculosisForUrologyCare(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnosis of tuberculosis.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of tuberculosis.

    **Exclusion Criteria:** Excludes concepts that identify a diagnosis other than tuberculosis.
    """

    VALUE_SET_NAME = "Tuberculosis for Urology Care"
    OID = "2.16.840.1.113762.1.4.1248.351"
    DEFINITION_VERSION = "20250312"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "A150",  # Tuberculosis of lung
        "A154",  # Tuberculosis of intrathoracic lymph nodes
        "A155",  # Tuberculosis of larynx, trachea and bronchus
        "A156",  # Tuberculous pleurisy
        "A157",  # Primary respiratory tuberculosis
        "A158",  # Other respiratory tuberculosis
        "A159",  # Respiratory tuberculosis unspecified
        "A170",  # Tuberculous meningitis
        "A1789",  # Other tuberculosis of nervous system
        "A179",  # Tuberculosis of nervous system, unspecified
        "A1801",  # Tuberculosis of spine
        "A1802",  # Tuberculous arthritis of other joints
        "A1803",  # Tuberculosis of other bones
        "A1809",  # Other musculoskeletal tuberculosis
        "A1810",  # Tuberculosis of genitourinary system, unspecified
        "A1811",  # Tuberculosis of kidney and ureter
        "A1812",  # Tuberculosis of bladder
        "A1813",  # Tuberculosis of other urinary organs
        "A1814",  # Tuberculosis of prostate
        "A1815",  # Tuberculosis of other male genital organs
        "A1816",  # Tuberculosis of cervix
        "A1818",  # Tuberculosis of other female genital organs
        "A182",  # Tuberculous peripheral lymphadenopathy
        "A1831",  # Tuberculous peritonitis
        "A1832",  # Tuberculous enteritis
        "A1839",  # Retroperitoneal tuberculosis
        "A184",  # Tuberculosis of skin and subcutaneous tissue
        "A1850",  # Tuberculosis of eye, unspecified
        "A1851",  # Tuberculous episcleritis
        "A1852",  # Tuberculous keratitis
        "A1853",  # Tuberculous chorioretinitis
        "A1854",  # Tuberculous iridocyclitis
        "A1859",  # Other tuberculosis of eye
        "A186",  # Tuberculosis of (inner) (middle) ear
        "A187",  # Tuberculosis of adrenal glands
        "A1881",  # Tuberculosis of thyroid gland
        "A1882",  # Tuberculosis of other endocrine glands
        "A1883",  # Tuberculosis of digestive tract organs, not elsewhere classified
        "A1884",  # Tuberculosis of heart
        "A1885",  # Tuberculosis of spleen
        "A1889",  # Tuberculosis of other sites
        "A190",  # Acute miliary tuberculosis of a single specified site
        "A191",  # Acute miliary tuberculosis of multiple sites
        "A192",  # Acute miliary tuberculosis, unspecified
        "A198",  # Other miliary tuberculosis
        "A199",  # Miliary tuberculosis, unspecified
    }

    SNOMEDCT = {
        "10706006",  # Tuberculosis of central nervous system (disorder)
        "10893003",  # Tuberculous adenitis (disorder)
        "1092771000119104",  # Tuberculous neuritis (disorder)
        "111825008",  # Tuberculosis of conjunctiva (disorder)
        "111830007",  # Tuberculosis of urinary organs (disorder)
        "111832004",  # Primary progressive tuberculosis (disorder)
        "1144499001",  # Tuberculosis of pelvis and femur (disorder)
        "11676005",  # Tuberculous leptomeningitis (disorder)
        "12677003",  # Tuberculosis of brain (disorder)
        "129211000119108",  # Isoniazid resistant tuberculosis of lung (disorder)
        "12985005",  # Tuberculosis of seminal vesicle (disorder)
        "13570003",  # Tuberculous tenosynovitis (disorder)
        "14188007",  # Tuberculosis of hip (disorder)
        "14527007",  # Tuberculous empyema (disorder)
        "15202009",  # Tuberculoma (disorder)
        "15284007",  # Tuberculosis of esophagus (disorder)
        "154283005",  # Pulmonary tuberculosis (disorder)
        "15984191000119101",  # Tuberculosis of right ear (disorder)
        "15984231000119105",  # Tuberculosis of left ear (disorder)
        "15984271000119108",  # Tuberculosis of bilateral ears (disorder)
        "16016231000119109",  # Tuberculosis of right knee joint (disorder)
        "16016271000119107",  # Tuberculosis of bilateral knee joints (disorder)
        "16016311000119107",  # Tuberculosis of left knee joint (disorder)
        "16409009",  # Tuberculosis of retroperitoneal lymph nodes (disorder)
        "17136009",  # Tuberculosis of knee (disorder)
        "17653001",  # Tuberculosis of bones and/or joints (disorder)
        "182159002",  # Disseminated tuberculosis (disorder)
        "185113005",  # Tuberculosis of adrenal glands (disorder)
        "186172004",  # Tuberculous pleurisy in primary progressive tuberculosis (disorder)
        "186175002",  # Infiltrative lung tuberculosis (disorder)
        "186177005",  # Tuberculosis of lung with cavitation (disorder)
        "186178000",  # Tuberculosis of bronchus (disorder)
        "186182003",  # Tuberculosis of pleura (disorder)
        "186193001",  # Tuberculosis of lung, confirmed by sputum microscopy with or without culture (disorder)
        "186194007",  # Tuberculosis of lung, confirmed by culture only (disorder)
        "186195008",  # Tuberculosis of lung, confirmed histologically (disorder)
        "186199002",  # Tuberculosis of larynx, trachea and bronchus, confirmed bacteriologically and histologically (disorder)
        "186200004",  # Tuberculous pleurisy, confirmed bacteriologically and histologically (disorder)
        "186202007",  # Respiratory tuberculosis, not confirmed bacteriologically or histologically (disorder)
        "186203002",  # Tuberculosis of lung, bacteriologically and histologically negative (disorder)
        "186204008",  # Tuberculosis of lung, bacteriological and histological examination not done (disorder)
        "186225008",  # Tuberculosis of intestines, peritoneum and mesenteric glands (disorder)
        "186259007",  # Scrofulous tuberculous abscess (disorder)
        "186263000",  # Tuberculous chorioretinitis (disorder)
        "186266008",  # Tuberculous chronic iridocyclitis (disorder)
        "186267004",  # Tuberculous keratoconjunctivitis (disorder)
        "186269001",  # Tuberculosis of ear (disorder)
        "186270000",  # Tuberculous Addison's disease (disorder)
        "186272008",  # Tuberculosis of stomach (disorder)
        "186273003",  # Tuberculosis of liver (disorder)
        "186276006",  # Acute miliary tuberculosis (disorder)
        "186278007",  # Acute miliary tuberculosis of multiple sites (disorder)
        "190710003",  # Tuberculous arthritis (disorder)
        "194908003",  # Acute tuberculous pericarditis (disorder)
        "194949003",  # Acute myocarditis - tuberculous (disorder)
        "198241002",  # Female tuberculous pelvic inflammatory disease (disorder)
        "202994006",  # Tuberculous infection of tendon sheath (disorder)
        "203263006",  # Tuberculosis of cervical spine (disorder)
        "203264000",  # Tuberculosis of thoracic spine (disorder)
        "203265004",  # Tuberculosis of lumbar spine (disorder)
        "203267007",  # Tuberculosis of limb bones (disorder)
        "203269005",  # Tuberculosis of the upper arm bone (disorder)
        "203270006",  # Tuberculosis of the forearm bone (disorder)
        "203272003",  # Tuberculosis of the lower leg bone (disorder)
        "203274002",  # Tuberculosis of multiple limb bones (disorder)
        "203278004",  # Tuberculosis of bones of shoulder region (disorder)
        "203279007",  # Tuberculosis of the bones of the hand (disorder)
        "203280005",  # Tuberculosis of the bones of the ankle and/or foot (disorder)
        "203282002",  # Tuberculosis of the bones of multiple sites (disorder)
        "20372007",  # Tuberculosis of prostate (disorder)
        "22990009",  # Chronic tuberculosis (disorder)
        "233649005",  # Tuberculous chylothorax (disorder)
        "233763009",  # Silicotuberculosis (disorder)
        "233794009",  # Tuberculous bronchopleural fistula (disorder)
        "234090004",  # Tuberculous mesenteric adenitis (disorder)
        "235067001",  # Oral tuberculosis (disorder)
        "235694001",  # Tuberculous duodenitis (disorder)
        "236602004",  # Tuberculous stricture of ureter (disorder)
        "236684001",  # Tuberculous urethritis (disorder)
        "236770001",  # Tuberculous epididymo-orchitis (disorder)
        "237445009",  # Tuberculosis of breast (disorder)
        "240376003",  # Tuberculosis of gastrointestinal tract (disorder)
        "240379005",  # Tuberculosis of male genital organs (disorder)
        "240380008",  # Acute tuberculous ulcer (disorder)
        "240381007",  # Acute miliary cutaneous tuberculosis (disorder)
        "240382000",  # Cutaneous tuberculous gumma (disorder)
        "24235005",  # Tuberculous hydrothorax (disorder)
        "24810006",  # Tuberculosis orificialis of mouth (disorder)
        "25629007",  # Acute tuberculosis (disorder)
        "25738004",  # Tuberculous synovitis (disorder)
        "25831001",  # Tuberculosis of nasal septum (disorder)
        "267317007",  # Tuberculosis in pregnancy (disorder)
        "26935004",  # Tuberculosis of thyroid gland (disorder)
        "271423008",  # Tuberculosis of skin and subcutaneous tissue (disorder)
        "281623008",  # Genital tuberculosis (disorder)
        "28399005",  # Tuberculosis of spleen (disorder)
        "29731002",  # Tuberculous pneumothorax (disorder)
        "301836007",  # Tuberculosis of colon (disorder)
        "302131003",  # Tuberculosis of heart (disorder)
        "31112008",  # Tuberculous meningoencephalitis (disorder)
        "314029000",  # Tuberculous uveitis (disorder)
        "32268008",  # Tuberculosis of urinary bladder (disorder)
        "33893009",  # Tuberculosis verrucosa cutis (disorder)
        "35117008",  # Tuberculosis of small intestine (disorder)
        "35585004",  # Tuberculous endometritis (disorder)
        "35786001",  # Tuberculoma of meninges (disorder)
        "35984006",  # Tuberculosis of vertebral column (disorder)
        "37133005",  # Tuberculous pyelitis (disorder)
        "37260006",  # Congenital tuberculosis (disorder)
        "38115001",  # Tuberculosis of spinal meninges (disorder)
        "38279006",  # Tuberculosis of bone (disorder)
        "38420005",  # Tuberculosis of large intestine (disorder)
        "39306006",  # Tuberculosis of anus (disorder)
        "40258005",  # Tuberculosis of testis (disorder)
        "413556004",  # Antibiotic resistant tuberculosis (disorder)
        "413852006",  # Ciprofloxacin resistant tuberculosis (disorder)
        "414146004",  # Ethambutol resistant tuberculosis (disorder)
        "414149006",  # Ethionamide resistant tuberculosis (disorder)
        "414546009",  # Isoniazid resistant tuberculosis (disorder)
        "415222009",  # Pyrazinamide resistant tuberculosis (disorder)
        "415345001",  # Rifampicin resistant tuberculosis (disorder)
        "415622003",  # Streptomycin resistant tuberculosis (disorder)
        "421571007",  # Tuberculosis with acquired immunodeficiency syndrome (disorder)
        "422481005",  # Extreme drug resistant tuberculosis (disorder)
        "4225003",  # Tuberculosis of nose (disorder)
        "423092005",  # Multidrug resistant tuberculosis (disorder)
        "423709000",  # Drug resistant tuberculosis (disorder)
        "423997002",  # Tuberculosis, extrapulmonary (disorder)
        "427099000",  # Active tuberculosis (disorder)
        "441770001",  # Tuberculosis of external body orifice (disorder)
        "44323002",  # Tuberculosis of kidney (disorder)
        "4445009",  # Tuberculosis of genitourinary system (disorder)
        "44572005",  # Tuberculous peritonitis (disorder)
        "446753005",  # Tuberculosis of oropharynx (disorder)
        "446946005",  # Reinfection pulmonary tuberculosis (disorder)
        "446986002",  # Tuberculous pleural effusion (disorder)
        "447006007",  # Relapse pulmonary tuberculosis (disorder)
        "447012002",  # Tuberculosis of spinal cord (disorder)
        "447253004",  # Tuberculous arachnoiditis (disorder)
        "447330002",  # Tuberculosis of abdomen (disorder)
        "447371008",  # Tuberculosis of ileum (disorder)
        "449925002",  # Tuberculosis of bone of upper limb (disorder)
        "4501007",  # Tuberculous ascites (disorder)
        "47292005",  # Tuberculosis of myocardium (disorder)
        "473114006",  # History of active tuberculosis (situation)
        "473426009",  # Primary tuberculous complex confirmed by culture (disorder)
        "473427000",  # Primary tuberculous complex confirmed histologically (disorder)
        "473428005",  # Primary tuberculous complex confirmed by microscopic examination (disorder)
        "473430007",  # Primary tuberculous complex confirmed (disorder)
        "473431006",  # Primary tuberculous complex with bacteriological or histological examination results unknown (disorder)
        "47604008",  # Miliary tuberculosis (disorder)
        "48225000",  # Tuberculous disseminated chorioretinitis (disorder)
        "49107007",  # Tuberculosis of eye (disorder)
        "49558004",  # Tuberculous salpingitis (disorder)
        "49807009",  # Tuberculous dactylitis (disorder)
        "51014003",  # Reinfection tuberculosis (disorder)
        "52721006",  # Tuberculosis of nasal sinus (disorder)
        "52987001",  # Tuberculosis of endocardium (disorder)
        "54084005",  # Cervical tuberculous lymphadenitis (disorder)
        "55419007",  # Tuberculosis of glottis (disorder)
        "56498009",  # Tuberculosis of nasopharynx (disorder)
        "56717001",  # Tuberculosis (disorder)
        "58374007",  # Acute diffuse tuberculosis (disorder)
        "58437007",  # Tuberculosis of meninges (disorder)
        "58764007",  # Tuberculous osteomyelitis (disorder)
        "60136008",  # Tuberculosis of intestine (disorder)
        "62494007",  # Chronic miliary tuberculosis (disorder)
        "62819009",  # Tuberculous cervicitis (disorder)
        "63309002",  # Primary tuberculosis (disorder)
        "63808004",  # Tuberculous chancre (disorder)
        "66986005",  # Tuberculosis of skin (disorder)
        "67256000",  # Tuberculosis of pericardium (disorder)
        "700272008",  # Tuberculosis of respiratory system (disorder)
        "700273003",  # Isolated tracheobronchial tuberculosis (disorder)
        "70341005",  # Tuberculous laryngitis (disorder)
        "710106005",  # Extensively drug resistant tuberculosis (disorder)
        "71375000",  # Tuberculosis of mastoid process (disorder)
        "717697005",  # Tuberculous ulceration of vulva (disorder)
        "71943007",  # Chronic granulomatous tuberculosis (disorder)
        "724541008",  # Acute miliary tuberculosis of single site (disorder)
        "72807002",  # Tuberculous polyserositis (disorder)
        "74181004",  # Tuberculosis of female genital organs (disorder)
        "74387008",  # Tuberculosis of hilar lymph nodes (disorder)
        "7444001",  # Tuberculosis of rectum (disorder)
        "77038006",  # Tuberculosis of peripheral lymph nodes (disorder)
        "78436002",  # Tuberculosis of intrathoracic lymph nodes (disorder)
        "789056000",  # Tuberculosis of inner ear (disorder)
        "789057009",  # Tuberculosis of middle ear (disorder)
        "79158003",  # Tuberculous interstitial keratitis (disorder)
        "80003002",  # Tuberculous pneumonia (disorder)
        "80010008",  # Isolated bronchial tuberculosis (disorder)
        "80602006",  # Nodular tuberculosis of lung (disorder)
        "81133003",  # Tuberculosis of bones of lower extremity (disorder)
        "81359005",  # Tuberculosis of ureter (disorder)
        "81554001",  # Tuberculosis of lung with involvement of bronchus (disorder)
        "82495004",  # Tuberculosis of mediastinal lymph nodes (disorder)
        "8250007",  # Tuberculosis of subcutaneous cellular tissue (disorder)
        "83652003",  # Tuberculosis of epididymis (disorder)
        "83784000",  # Tuberculosis of mediastinum (disorder)
        "84194006",  # Tuberculous oophoritis (disorder)
        "84288007",  # Tuberculous episcleritis (disorder)
        "85692003",  # Tuberculosis of tracheobronchial lymph nodes (disorder)
        "85940005",  # Tuberculous otitis media (disorder)
        "8832006",  # Tuberculosis orificialis of anus (disorder)
        "88356006",  # Primary tuberculous complex (disorder)
        "90117007",  # Tuberculous fibrosis of lung (disorder)
        "90302003",  # Tuberculosis of cerebral meninges (disorder)
        "91483004",  # Tuberculous encephalitis (disorder)
    }

class OverweightOrObese(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to identify concepts for overweight or obese.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that identify a diagnosis of overweight or obese.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Overweight or Obese"
    OID = "2.16.840.1.113762.1.4.1047.502"
    DEFINITION_VERSION = "20180309"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "E6601",  # Morbid (severe) obesity due to excess calories
        "E6609",  # Other obesity due to excess calories
        "E661",  # Drug-induced obesity
        "E662",  # Morbid (severe) obesity with alveolar hypoventilation
        "E663",  # Overweight
        "E66811",  # Obesity, class 1
        "E66812",  # Obesity, class 2
        "E66813",  # Obesity, class 3
        "E6689",  # Other obesity not elsewhere classified
        "E669",  # Obesity, unspecified
    }

    SNOMEDCT = {
        "1187531009",  # Obesity due to pituitary disease (disorder)
        "162863004",  # Body mass index 25-29 - overweight (finding)
        "162864005",  # Body mass index 30+ - obesity (finding)
        "190966007",  # Extreme obesity with alveolar hypoventilation (disorder)
        "238131007",  # Overweight (finding)
        "238136002",  # Morbid obesity (disorder)
        "408512008",  # Body mass index 40+ - severely obese (finding)
        "414915002",  # Obese (finding)
        "414916001",  # Obesity (disorder)
        "415530009",  # Simple obesity (disorder)
        "443371000124107",  # Obese class I (finding)
        "443381000124105",  # Obese class II (finding)
        "722595002",  # Overweight in adulthood with body mass index of 25 or more but less than 30 (finding)
        "783549006",  # Obesity due to centrosomal protein 19 deficiency (disorder)
        "783556000",  # Severe early-onset obesity insulin resistance syndrome due to SH2B adaptor protein 1 deficiency (disorder)
        "783719006",  # Obesity due to SIM bHLH transcription factor 1 deficiency (disorder)
        "785722006",  # Obesity due to leptin receptor gene deficiency (disorder)
        "819948005",  # Obese class III (finding)
        "83911000119104",  # Severe obesity (disorder)
    }

class Underweight(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent identifying underweight patients.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category or attribute related to Diagnosis.

    **Inclusion Criteria:** Includes only relevant concepts associated with identifying patients who are currently underweight.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Underweight"
    OID = "2.16.840.1.113883.3.526.3.1563"
    DEFINITION_VERSION = "20220302"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "R636",  # Underweight
    }

    SNOMEDCT = {
        "248342006",  # Underweight (finding)
        "427090001",  # Body mass index less than 16.5 (finding)
        "717933005",  # Severe thinness in adulthood (finding)
    }

class HistoryOfAtrialAblation(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a situation of a personal history of an atrial ablation.

    **Data Element Scope:** This value set may use a model element related to a diagnosis or assessment.

    **Inclusion Criteria:** Includes concepts that identify a situation for a personal history of an atrial ablation.

    **Exclusion Criteria:** No Exclusions.
    """

    VALUE_SET_NAME = "History of Atrial Ablation"
    OID = "2.16.840.1.113762.1.4.1110.76"
    DEFINITION_VERSION = "20221115"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    SNOMEDCT = {
        "427951003",  # History of radiofrequency ablation operation for arrhythmia (situation)
        "429508000",  # History of ablation of atrioventricular node (situation)
        "429756009",  # History of radiofrequency ablation operation on left atrium for arrhythmia (situation)
    }

class IntravenousOrIntraarterialThrombolyticTpaTherapyPriorToArrival(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for procedures of intravenous or intra-arterial thrombolytic (t-PA) therapy.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that identify a procedure for intravenous or intra-arterial thrombolytic (t-PA) therapy.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Intravenous or Intraarterial Thrombolytic tPA Therapy Prior to Arrival"
    OID = "2.16.840.1.113762.1.4.1110.21"
    DEFINITION_VERSION = "20210220"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "Z9282",  # Status post administration of tPA (rtPA) in a different facility within the last 24 hours prior to admission to current facility
    }

class DentalCaries(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a diagnosis of dental caries.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of dental caries.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Dental Caries"
    OID = "2.16.840.1.113883.3.464.1003.125.12.1004"
    DEFINITION_VERSION = "20210220"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "K023",  # Arrested dental caries
        "K0251",  # Dental caries on pit and fissure surface limited to enamel
        "K0252",  # Dental caries on pit and fissure surface penetrating into dentin
        "K0253",  # Dental caries on pit and fissure surface penetrating into pulp
        "K0261",  # Dental caries on smooth surface limited to enamel
        "K0262",  # Dental caries on smooth surface penetrating into dentin
        "K0263",  # Dental caries on smooth surface penetrating into pulp
        "K027",  # Dental root caries
        "K029",  # Dental caries, unspecified
    }

    SNOMEDCT = {
        "1085951000119102",  # Dental caries on pit and fissure surface limited to enamel (disorder)
        "1085961000119100",  # Dental caries on pit and fissure surface penetrating into dentin (disorder)
        "1085971000119106",  # Dental caries on pit and fissure surface penetrating into pulp (disorder)
        "1085981000119109",  # Dental caries on smooth surface limited to enamel (disorder)
        "1085991000119107",  # Dental caries on smooth surface penetrating into dentin (disorder)
        "1086001000119108",  # Dental caries on smooth surface penetrating into pulp (disorder)
        "109564008",  # Dental caries associated with enamel hypomineralization (disorder)
        "109566005",  # Dental caries associated with enamel hypoplasia (disorder)
        "109568006",  # Dental caries secondary to developmental defects of tooth structure (disorder)
        "109569003",  # Dental caries secondary to acquired defects of tooth structure (disorder)
        "109570002",  # Primary dental caries (disorder)
        "109571003",  # Primary dental caries of indeterminate origin (disorder)
        "109572005",  # Primary dental caries, cervical origin (disorder)
        "109573000",  # Primary dental caries, proximal smooth surface origin (disorder)
        "109574006",  # Primary dental caries, nonproximal smooth surface origin (disorder)
        "109575007",  # Primary dental caries, pit and fissure origin (disorder)
        "109576008",  # Primary dental caries, root surface origin (disorder)
        "109577004",  # Primary dental caries, multisurface origin (disorder)
        "109578009",  # Caries of infancy (disorder)
        "109580003",  # Caries of infancy associated with breast feeding (disorder)
        "109581004",  # Caries of infancy associated with bottle feeding (disorder)
        "1287576005",  # Primary dental caries extending into outer half of enamel (disorder)
        "15733007",  # Incipient enamel caries (disorder)
        "196298000",  # Acute dentine dental caries (disorder)
        "196299008",  # Chronic dentine dental caries (disorder)
        "196301001",  # Acute enamel dental caries (disorder)
        "196302008",  # Chronic enamel dental caries (disorder)
        "196305005",  # Odontoclasia (disorder)
        "234975001",  # Root caries (disorder)
        "234976000",  # Rampant dental caries (disorder)
        "30512007",  # Cementum caries (disorder)
        "442231009",  # Caries involving multiple surfaces of tooth (disorder)
        "442551007",  # Dental caries extending into dentin (disorder)
        "699489009",  # Dental caries class I (disorder)
        "699490000",  # Dental caries class II (disorder)
        "699491001",  # Dental caries class III (disorder)
        "699492008",  # Dental caries class IV (disorder)
        "699494009",  # Dental caries class V (disorder)
        "699495005",  # Dental caries class VI (disorder)
        "700046006",  # Carious exposure of pulp (disorder)
        "702402003",  # Early childhood caries (disorder)
        "733939000",  # Primary active dental caries extending into outer half of enamel (disorder)
        "733940003",  # Primary active dental caries extending into inner half of enamel (disorder)
        "733941004",  # Primary active dental caries extending into outer third of dentin (disorder)
        "733942006",  # Primary active dental caries extending into middle third of dentin (disorder)
        "733943001",  # Primary active dental caries extending into inner third of dentin (disorder)
        "733944007",  # Secondary active dental caries extending into outer half of enamel (disorder)
        "733945008",  # Secondary active dental caries extending into inner half of enamel (disorder)
        "733946009",  # Secondary active dental caries extending into outer third of dentin (disorder)
        "733947000",  # Secondary active dental caries extending into middle third of dentin (disorder)
        "733948005",  # Secondary active dental caries extending into inner third of dentin (disorder)
        "733968004",  # Active dental caries (disorder)
        "733970008",  # Primary active dental caries extending into enamel (disorder)
        "733972000",  # Primary active dental caries extending into dentin (disorder)
        "733974004",  # Secondary active dental caries extending into enamel (disorder)
        "733976002",  # Secondary active dental caries extending into dentin (disorder)
        "768639006",  # Initial stage caries of crown of tooth (disorder)
        "768641007",  # Initial stage active caries of crown of tooth (disorder)
        "768643005",  # Moderate stage caries of crown of tooth (disorder)
        "768646002",  # Moderate stage active caries of crown of tooth (disorder)
        "768649009",  # Extensive stage caries of crown of tooth (disorder)
        "768653006",  # Extensive stage active caries of crown of tooth (disorder)
        "769017001",  # Active cavitated caries of tooth root (disorder)
        "769020009",  # Non-cavitated caries of tooth root (disorder)
        "769021008",  # Active non-cavitated caries of tooth root (disorder)
        "80353004",  # Enamel caries (disorder)
        "80967001",  # Dental caries (disorder)
        "95246007",  # Salivary dysfunction caries secondary to aging (disorder)
        "95247003",  # Salivary dysfunction caries caused by medication (disorder)
        "95248008",  # Salivary dysfunction caries following radiotherapy (disorder)
        "95249000",  # Salivary dysfunction dental caries (disorder)
        "95252008",  # Secondary dental caries associated with failed or defective dental restoration (disorder)
        "95253003",  # Secondary dental caries associated with local or systemic factors (disorder)
        "95254009",  # Secondary dental caries (disorder)
    }

class MorbidObesity(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to identify concepts for diagnoses of morbid obesity and body mass index (BMI) ranges relating to morbid obesity.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that describe a diagnosis of morbid obesity and associated body mass index (BMI) values.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Morbid Obesity"
    OID = "2.16.840.1.113762.1.4.1248.368"
    DEFINITION_VERSION = "20250312"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "E6601",  # Morbid (severe) obesity due to excess calories
        "E662",  # Morbid (severe) obesity with alveolar hypoventilation
        "Z6841",  # Body mass index [BMI] 40.0-44.9, adult
        "Z6842",  # Body mass index [BMI] 45.0-49.9, adult
        "Z6843",  # Body mass index [BMI] 50.0-59.9, adult
        "Z6844",  # Body mass index [BMI] 60.0-69.9, adult
        "Z6845",  # Body mass index [BMI] 70 or greater, adult
    }

    SNOMEDCT = {
        "190966007",  # Extreme obesity with alveolar hypoventilation (disorder)
        "238136002",  # Morbid obesity (disorder)
        "408512008",  # Body mass index 40+ - severely obese (finding)
        "819948005",  # Obese class III (finding)
        "83911000119104",  # Severe obesity (disorder)
    }

class UrinaryRetention(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of urinary retention related to drugs or other causes.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of urinary retention related to drugs or other causes.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Urinary Retention"
    OID = "2.16.840.1.113762.1.4.1248.372"
    DEFINITION_VERSION = "20250312"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "R330",  # Drug induced retention of urine
        "R338",  # Other retention of urine
        "R339",  # Retention of urine, unspecified
    }

    SNOMEDCT = {
        "12245681000119103",  # Postprocedural retention of urine (disorder)
        "236648008",  # Acute retention of urine (disorder)
        "236649000",  # Acute-on-chronic retention of urine (disorder)
        "236650000",  # Chronic retention of urine (disorder)
        "236651001",  # Clot retention of urine (disorder)
        "236652008",  # Postoperative retention of urine (disorder)
        "267064002",  # Retention of urine (disorder)
        "367421000119109",  # Retention of urine caused by drug (disorder)
        "430976005",  # Retention of urine due to occlusion of Foley catheter (disorder)
    }

class MalnutritionDiagnosis(ValueSet):
    """
    **Clinical Focus:** Diagnosis of malnutrition identified after a nutrition assessment of an individual.

    **Data Element Scope:** This set of values indicates a diagnosis of malnutrition by a physician or eligible provider.

    **Inclusion Criteria:** Terms related to a diagnosis of malnutrition by a physician or eligible provider.

    **Exclusion Criteria:** Terms other than those indicating a diagnosis of malnutrition by a physician or eligible provider.
    """

    VALUE_SET_NAME = "Malnutrition Diagnosis"
    OID = "2.16.840.1.113762.1.4.1095.55"
    DEFINITION_VERSION = "20180616"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "E40",  # Kwashiorkor
        "E41",  # Nutritional marasmus
        "E42",  # Marasmic kwashiorkor
        "E43",  # Unspecified severe protein-calorie malnutrition
        "E440",  # Moderate protein-calorie malnutrition
        "E45",  # Retarded development following protein-calorie malnutrition
        "E46",  # Unspecified protein-calorie malnutrition
        "E639",  # Nutritional deficiency, unspecified
        "E640",  # Sequelae of protein-calorie malnutrition
        "M6284",  # Sarcopenia
        "O2510",  # Malnutrition in pregnancy, unspecified trimester
        "O2511",  # Malnutrition in pregnancy, first trimester
        "O2512",  # Malnutrition in pregnancy, second trimester
        "O2513",  # Malnutrition in pregnancy, third trimester
        "O252",  # Malnutrition in childbirth
        "O253",  # Malnutrition in the puerperium
        "R627",  # Adult failure to thrive
        "R64",  # Cachexia
        "T730XXA",  # Starvation, initial encounter
        "T730XXD",  # Starvation, subsequent encounter
        "T730XXS",  # Starvation, sequela
    }

    SNOMEDCT = {
        "102635008",  # Acute nutritional deficiency (finding)
        "102636009",  # Chronic nutritional deficiency (finding)
        "129588001",  # Adult failure to thrive syndrome (disorder)
        "190606006",  # Moderate protein energy malnutrition (disorder)
        "238107002",  # Deficiency of macronutrients (disorder)
        "238108007",  # Cachexia (finding)
        "238111008",  # Deficiency of micronutrients (disorder)
        "2492009",  # Nutritional disorder (disorder)
        "272588001",  # Malnutrition (calorie) (disorder)
        "281583001",  # Nutritional wasting (disorder)
        "285588004",  # Nutrition deficiency due to insufficient food (disorder)
        "288517002",  # Wasting disease (disorder)
        "371597004",  # Emaciated (finding)
        "432788009",  # Pediatric failure to thrive (disorder)
        "441951000124102",  # Starvation-related malnutrition (disorder)
        "441961000124100",  # Acute disease or injury-related malnutrition (disorder)
        "441971000124107",  # Chronic disease-related malnutrition (disorder)
        "54840006",  # Failure to thrive (disorder)
        "56882008",  # Anorexia nervosa (disorder)
        "65404009",  # Undernutrition (disorder)
        "70241007",  # Nutritional deficiency disorder (disorder)
        "722959000",  # Anorexia nervosa co-occurrent with dangerously low body weight (disorder)
        "722960005",  # Anorexia nervosa co-occurrent with significantly low body weight (disorder)
        "723917005",  # Dangerously low body weight co-occurrent and due to anorexia nervosa of restricting type (disorder)
        "723919008",  # Significantly low body weight co-occurrent and due to anorexia nervosa of restricting type (disorder)
        "772791006",  # Sarcopenia (disorder)
        "77675002",  # Anorexia nervosa, restricting type (disorder)
        "788876001",  # Cachexia due to malignant neoplastic disease (finding)
    }

class ActiveBleedingOrBleedingDiathesisExcludingMenses(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for active bleeding or bleeding diathesis (excluding menses).

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes ICD-10CM and SNOMEDCT codes associated with active bleeding.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Active Bleeding or Bleeding Diathesis, Excluding Menses"
    OID = "2.16.840.1.113883.3.3157.4036"
    DEFINITION_VERSION = "20190216"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "D630",  # Anemia in neoplastic disease
        "D65",  # Disseminated intravascular coagulation [defibrination syndrome]
        "D68311",  # Acquired hemophilia
        "D68312",  # Antiphospholipid antibody with hemorrhagic disorder
        "D68318",  # Other hemorrhagic disorder due to intrinsic circulating anticoagulants, antibodies, or inhibitors
        "D6832",  # Hemorrhagic disorder due to extrinsic circulating anticoagulants
        "D698",  # Other specified hemorrhagic conditions
        "D699",  # Hemorrhagic condition, unspecified
        "D7821",  # Postprocedural hemorrhage of the spleen following a procedure on the spleen
        "D7822",  # Postprocedural hemorrhage of the spleen following other procedure
        "E89810",  # Postprocedural hemorrhage of an endocrine system organ or structure following an endocrine system procedure
        "E89811",  # Postprocedural hemorrhage of an endocrine system organ or structure following other procedure
        "G0430",  # Acute necrotizing hemorrhagic encephalopathy, unspecified
        "G0431",  # Postinfectious acute necrotizing hemorrhagic encephalopathy
        "G0432",  # Postimmunization acute necrotizing hemorrhagic encephalopathy
        "G0439",  # Other acute necrotizing hemorrhagic encephalopathy
        "G361",  # Acute and subacute hemorrhagic leukoencephalitis [Hurst]
        "G9751",  # Postprocedural hemorrhage of a nervous system organ or structure following a nervous system procedure
        "G9752",  # Postprocedural hemorrhage of a nervous system organ or structure following other procedure
        "I6000",  # Nontraumatic subarachnoid hemorrhage from unspecified carotid siphon and bifurcation
        "I6001",  # Nontraumatic subarachnoid hemorrhage from right carotid siphon and bifurcation
        "I6002",  # Nontraumatic subarachnoid hemorrhage from left carotid siphon and bifurcation
        "I6010",  # Nontraumatic subarachnoid hemorrhage from unspecified middle cerebral artery
        "I6011",  # Nontraumatic subarachnoid hemorrhage from right middle cerebral artery
        "I6012",  # Nontraumatic subarachnoid hemorrhage from left middle cerebral artery
        "I602",  # Nontraumatic subarachnoid hemorrhage from anterior communicating artery
        "I6030",  # Nontraumatic subarachnoid hemorrhage from unspecified posterior communicating artery
        "I6031",  # Nontraumatic subarachnoid hemorrhage from right posterior communicating artery
        "I6032",  # Nontraumatic subarachnoid hemorrhage from left posterior communicating artery
        "I604",  # Nontraumatic subarachnoid hemorrhage from basilar artery
        "I6050",  # Nontraumatic subarachnoid hemorrhage from unspecified vertebral artery
        "I6051",  # Nontraumatic subarachnoid hemorrhage from right vertebral artery
        "I6052",  # Nontraumatic subarachnoid hemorrhage from left vertebral artery
        "I606",  # Nontraumatic subarachnoid hemorrhage from other intracranial arteries
        "I607",  # Nontraumatic subarachnoid hemorrhage from unspecified intracranial artery
        "I608",  # Other nontraumatic subarachnoid hemorrhage
        "I609",  # Nontraumatic subarachnoid hemorrhage, unspecified
        "I610",  # Nontraumatic intracerebral hemorrhage in hemisphere, subcortical
        "I611",  # Nontraumatic intracerebral hemorrhage in hemisphere, cortical
        "I612",  # Nontraumatic intracerebral hemorrhage in hemisphere, unspecified
        "I613",  # Nontraumatic intracerebral hemorrhage in brain stem
        "I614",  # Nontraumatic intracerebral hemorrhage in cerebellum
        "I615",  # Nontraumatic intracerebral hemorrhage, intraventricular
        "I616",  # Nontraumatic intracerebral hemorrhage, multiple localized
        "I618",  # Other nontraumatic intracerebral hemorrhage
        "I619",  # Nontraumatic intracerebral hemorrhage, unspecified
        "I6200",  # Nontraumatic subdural hemorrhage, unspecified
        "I6201",  # Nontraumatic acute subdural hemorrhage
        "I6202",  # Nontraumatic subacute subdural hemorrhage
        "I6203",  # Nontraumatic chronic subdural hemorrhage
        "I621",  # Nontraumatic extradural hemorrhage
        "I629",  # Nontraumatic intracranial hemorrhage, unspecified
        "K920",  # Hematemesis
        "R041",  # Hemorrhage from throat
        "R042",  # Hemoptysis
        "R0489",  # Hemorrhage from other sites in respiratory passages
        "R049",  # Hemorrhage from respiratory passages, unspecified
        "R58",  # Hemorrhage, not elsewhere classified
        "S06340A",  # Traumatic hemorrhage of right cerebrum without loss of consciousness, initial encounter
        "S06341A",  # Traumatic hemorrhage of right cerebrum with loss of consciousness of 30 minutes or less, initial encounter
        "S06342A",  # Traumatic hemorrhage of right cerebrum with loss of consciousness of 31 minutes to 59 minutes, initial encounter
        "S06343A",  # Traumatic hemorrhage of right cerebrum with loss of consciousness of 1 hours to 5 hours 59 minutes, initial encounter
        "S06344A",  # Traumatic hemorrhage of right cerebrum with loss of consciousness of 6 hours to 24 hours, initial encounter
        "S06345A",  # Traumatic hemorrhage of right cerebrum with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter
        "S06346A",  # Traumatic hemorrhage of right cerebrum with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter
        "S06347A",  # Traumatic hemorrhage of right cerebrum with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter
        "S06348A",  # Traumatic hemorrhage of right cerebrum with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter
        "S06349A",  # Traumatic hemorrhage of right cerebrum with loss of consciousness of unspecified duration, initial encounter
        "S0634AA",  # Traumatic hemorrhage of right cerebrum with loss of consciousness status unknown, initial encounter
        "S06350A",  # Traumatic hemorrhage of left cerebrum without loss of consciousness, initial encounter
        "S06351A",  # Traumatic hemorrhage of left cerebrum with loss of consciousness of 30 minutes or less, initial encounter
        "S06352A",  # Traumatic hemorrhage of left cerebrum with loss of consciousness of 31 minutes to 59 minutes, initial encounter
        "S06353A",  # Traumatic hemorrhage of left cerebrum with loss of consciousness of 1 hours to 5 hours 59 minutes, initial encounter
        "S06354A",  # Traumatic hemorrhage of left cerebrum with loss of consciousness of 6 hours to 24 hours, initial encounter
        "S06355A",  # Traumatic hemorrhage of left cerebrum with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter
        "S06356A",  # Traumatic hemorrhage of left cerebrum with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter
        "S06357A",  # Traumatic hemorrhage of left cerebrum with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter
        "S06358A",  # Traumatic hemorrhage of left cerebrum with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter
        "S06359A",  # Traumatic hemorrhage of left cerebrum with loss of consciousness of unspecified duration, initial encounter
        "S0635AA",  # Traumatic hemorrhage of left cerebrum with loss of consciousness status unknown, initial encounter
        "S06360A",  # Traumatic hemorrhage of cerebrum, unspecified, without loss of consciousness, initial encounter
        "S06361A",  # Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness of 30 minutes or less, initial encounter
        "S06362A",  # Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness of 31 minutes to 59 minutes, initial encounter
        "S06363A",  # Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness of 1 hours to 5 hours 59 minutes, initial encounter
        "S06364A",  # Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness of 6 hours to 24 hours, initial encounter
        "S06365A",  # Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter
        "S06366A",  # Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter
        "S06367A",  # Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter
        "S06368A",  # Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter
        "S06369A",  # Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness of unspecified duration, initial encounter
        "S0636AA",  # Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness status unknown, initial encounter
        "S06370A",  # Contusion, laceration, and hemorrhage of cerebellum without loss of consciousness, initial encounter
        "S06371A",  # Contusion, laceration, and hemorrhage of cerebellum with loss of consciousness of 30 minutes or less, initial encounter
        "S06372A",  # Contusion, laceration, and hemorrhage of cerebellum with loss of consciousness of 31 minutes to 59 minutes, initial encounter
        "S06373A",  # Contusion, laceration, and hemorrhage of cerebellum with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter
        "S06374A",  # Contusion, laceration, and hemorrhage of cerebellum with loss of consciousness of 6 hours to 24 hours, initial encounter
        "S06375A",  # Contusion, laceration, and hemorrhage of cerebellum with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter
        "S06376A",  # Contusion, laceration, and hemorrhage of cerebellum with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter
        "S06377A",  # Contusion, laceration, and hemorrhage of cerebellum with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter
        "S06378A",  # Contusion, laceration, and hemorrhage of cerebellum with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter
        "S06379A",  # Contusion, laceration, and hemorrhage of cerebellum with loss of consciousness of unspecified duration, initial encounter
        "S0637AA",  # Contusion, laceration, and hemorrhage of cerebellum with loss of consciousness status unknown, initial encounter
        "S06380A",  # Contusion, laceration, and hemorrhage of brainstem without loss of consciousness, initial encounter
        "S06381A",  # Contusion, laceration, and hemorrhage of brainstem with loss of consciousness of 30 minutes or less, initial encounter
        "S06382A",  # Contusion, laceration, and hemorrhage of brainstem with loss of consciousness of 31 minutes to 59 minutes, initial encounter
        "S06383A",  # Contusion, laceration, and hemorrhage of brainstem with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter
        "S06384A",  # Contusion, laceration, and hemorrhage of brainstem with loss of consciousness of 6 hours to 24 hours, initial encounter
        "S06385A",  # Contusion, laceration, and hemorrhage of brainstem with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter
        "S06386A",  # Contusion, laceration, and hemorrhage of brainstem with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter
        "S06387A",  # Contusion, laceration, and hemorrhage of brainstem with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter
        "S06388A",  # Contusion, laceration, and hemorrhage of brainstem with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter
        "S06389A",  # Contusion, laceration, and hemorrhage of brainstem with loss of consciousness of unspecified duration, initial encounter
        "S0638AA",  # Contusion, laceration, and hemorrhage of brainstem with loss of consciousness status unknown, initial encounter
        "S064X0A",  # Epidural hemorrhage without loss of consciousness, initial encounter
        "S064X1A",  # Epidural hemorrhage with loss of consciousness of 30 minutes or less, initial encounter
        "S064X2A",  # Epidural hemorrhage with loss of consciousness of 31 minutes to 59 minutes, initial encounter
        "S064X3A",  # Epidural hemorrhage with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter
        "S064X4A",  # Epidural hemorrhage with loss of consciousness of 6 hours to 24 hours, initial encounter
        "S064X5A",  # Epidural hemorrhage with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter
        "S064X6A",  # Epidural hemorrhage with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter
        "S064X7A",  # Epidural hemorrhage with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter
        "S064X8A",  # Epidural hemorrhage with loss of consciousness of any duration with death due to other causes prior to regaining consciousness, initial encounter
        "S064X9A",  # Epidural hemorrhage with loss of consciousness of unspecified duration, initial encounter
        "S064XAA",  # Epidural hemorrhage with loss of consciousness status unknown, initial encounter
        "S065X0A",  # Traumatic subdural hemorrhage without loss of consciousness, initial encounter
        "S065X1A",  # Traumatic subdural hemorrhage with loss of consciousness of 30 minutes or less, initial encounter
        "S065X2A",  # Traumatic subdural hemorrhage with loss of consciousness of 31 minutes to 59 minutes, initial encounter
        "S065X3A",  # Traumatic subdural hemorrhage with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter
        "S065X4A",  # Traumatic subdural hemorrhage with loss of consciousness of 6 hours to 24 hours, initial encounter
        "S065X5A",  # Traumatic subdural hemorrhage with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter
        "S065X6A",  # Traumatic subdural hemorrhage with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter
        "S065X7A",  # Traumatic subdural hemorrhage with loss of consciousness of any duration with death due to brain injury before regaining consciousness, initial encounter
        "S065X8A",  # Traumatic subdural hemorrhage with loss of consciousness of any duration with death due to other cause before regaining consciousness, initial encounter
        "S065X9A",  # Traumatic subdural hemorrhage with loss of consciousness of unspecified duration, initial encounter
        "S065XAA",  # Traumatic subdural hemorrhage with loss of consciousness status unknown, initial encounter
        "S066X0A",  # Traumatic subarachnoid hemorrhage without loss of consciousness, initial encounter
        "S066X1A",  # Traumatic subarachnoid hemorrhage with loss of consciousness of 30 minutes or less, initial encounter
        "S066X2A",  # Traumatic subarachnoid hemorrhage with loss of consciousness of 31 minutes to 59 minutes, initial encounter
        "S066X3A",  # Traumatic subarachnoid hemorrhage with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter
        "S066X4A",  # Traumatic subarachnoid hemorrhage with loss of consciousness of 6 hours to 24 hours, initial encounter
        "S066X5A",  # Traumatic subarachnoid hemorrhage with loss of consciousness greater than 24 hours with return to pre-existing conscious level, initial encounter
        "S066X6A",  # Traumatic subarachnoid hemorrhage with loss of consciousness greater than 24 hours without return to pre-existing conscious level with patient surviving, initial encounter
        "S066X7A",  # Traumatic subarachnoid hemorrhage with loss of consciousness of any duration with death due to brain injury prior to regaining consciousness, initial encounter
        "S066X8A",  # Traumatic subarachnoid hemorrhage with loss of consciousness of any duration with death due to other cause prior to regaining consciousness, initial encounter
        "S066X9A",  # Traumatic subarachnoid hemorrhage with loss of consciousness of unspecified duration, initial encounter
        "S066XAA",  # Traumatic subarachnoid hemorrhage with loss of consciousness status unknown, initial encounter
        "T792XXA",  # Traumatic secondary and recurrent hemorrhage and seroma, initial encounter
        "T82837A",  # Hemorrhage due to cardiac prosthetic devices, implants and grafts, initial encounter
        "T82838A",  # Hemorrhage due to vascular prosthetic devices, implants and grafts, initial encounter
        "T8383XA",  # Hemorrhage due to genitourinary prosthetic devices, implants and grafts, initial encounter
        "T8483XA",  # Hemorrhage due to internal orthopedic prosthetic devices, implants and grafts, initial encounter
        "T85830A",  # Hemorrhage due to nervous system prosthetic devices, implants and grafts, initial encounter
        "T85838A",  # Hemorrhage due to other internal prosthetic devices, implants and grafts, initial encounter
    }

    SNOMEDCT = {
        "105604006",  # Deficiency of naturally occurring coagulation factor inhibitor (disorder)
        "110149000",  # Traumatic hemorrhage (disorder)
        "110265006",  # Postoperative hemorrhage (disorder)
        "111597003",  # Intracranial hemorrhage co-occurrent and due to open fracture of vault of skull (disorder)
        "111601003",  # Intracranial hemorrhage co-occurrent and due to closed fracture of base of skull (disorder)
        "111605007",  # Intracranial hemorrhage co-occurrent and due to open fracture of base of skull (disorder)
        "111611005",  # Intracranial hemorrhage co-occurrent and due to closed skull fracture (disorder)
        "111615001",  # Open skull fracture with intracranial hemorrhage (disorder)
        "111681000",  # Extradural hemorrhage following injury with open intracranial wound AND loss of consciousness (disorder)
        "112648003",  # Massive hemorrhage (morphologic abnormality)
        "123786007",  # Blood coagulation disorder with shortened coagulation time (disorder)
        "123787003",  # Blood coagulation disorder with prolonged coagulation time (disorder)
        "123788008",  # Blood coagulation disorder with shortened bleeding time (disorder)
        "123789000",  # Blood coagulation disorder with prolonged bleeding time (disorder)
        "123790009",  # Blood coagulation disorder with impaired clot retraction time (disorder)
        "127308007",  # Intracranial hemorrhage following injury with loss of consciousness (disorder)
        "127309004",  # Intracranial hemorrhage following injury with brief loss of consciousness (disorder)
        "127310009",  # Intracranial hemorrhage following injury with moderate loss of consciousness (disorder)
        "127311008",  # Intracranial hemorrhage following injury with prolonged loss of consciousness AND return to pre-existing conscious level (disorder)
        "127312001",  # Intracranial hemorrhage following injury with prolonged loss of consciousness without return to pre-existing conscious level (disorder)
        "128088003",  # Blood coagulation disorder, categorized by value of screening test (disorder)
        "1286003",  # Vitamin K deficiency coagulation disorder (disorder)
        "131148009",  # Bleeding (finding)
        "13289004",  # Intracranial hemorrhage following injury with open intracranial wound AND loss of consciousness (disorder)
        "1386000",  # Intracranial hemorrhage (disorder)
        "140881000119109",  # Compression of brain co-occurrent and due to spontaneous cerebral hemorrhage (disorder)
        "141091000119105",  # Compression of brain co-occurrent and due to nontraumatic subarachnoid hemorrhage (disorder)
        "142851000119103",  # Spontaneous cerebellar hemorrhage (disorder)
        "143521000119103",  # Nontraumatic intraparenchymal cerebral hemorrhage (disorder)
        "16773005",  # Drug-induced coagulation inhibitor disorder (disorder)
        "16922007",  # Hereditary coagulation factor deficiency (disorder)
        "191287000",  # Hemorrhagic disorder due to circulating anticoagulants (disorder)
        "191288005",  # Hemorrhagic disorder due to antithrombinemia (disorder)
        "191289002",  # Hemorrhagic disorder due to hyperheparinemia (disorder)
        "191290006",  # Hemorrhagic disorder due to increase in anti-8a (disorder)
        "191291005",  # Hemorrhagic disorder due to increase in anti-9a (disorder)
        "191292003",  # Hemorrhagic disorder due to increase in anti-10a (disorder)
        "191293008",  # Hemorrhagic disorder due to increase in anti-11a (disorder)
        "191296000",  # Deficiency of coagulation factor due to liver disease (disorder)
        "191297009",  # Deficiency of coagulation factor due to vitamin K deficiency (disorder)
        "195160000",  # Intracranial subarachnoid hemorrhage from vertebral artery (disorder)
        "195165005",  # Basal ganglia hemorrhage (disorder)
        "195167002",  # External capsule hemorrhage (disorder)
        "195168007",  # Intracerebral hemorrhage with intraventricular hemorrhage (disorder)
        "195169004",  # Intracerebral hemorrhage, multiple localized (disorder)
        "195176009",  # Non-traumatic intracranial subdural hemorrhage (disorder)
        "209947002",  # Closed traumatic subdural intracranial hemorrhage (disorder)
        "209956005",  # Subdural hemorrhage following open wound of head (disorder)
        "209987007",  # Traumatic subdural intracranial hemorrhage (disorder)
        "21233002",  # Meningeal hemorrhage (disorder)
        "212373009",  # Secondary and recurrent hemorrhage (disorder)
        "213133000",  # Internal prosthetic device causing hemorrhage (disorder)
        "21454007",  # Subarachnoid intracranial hemorrhage (disorder)
        "22490002",  # Bleeding of mouth (finding)
        "230706003",  # Hemorrhagic cerebral infarction (disorder)
        "230707007",  # Anterior cerebral circulation hemorrhagic infarction (disorder)
        "230708002",  # Posterior cerebral circulation hemorrhagic infarction (disorder)
        "230709005",  # Massive supratentorial cerebral hemorrhage (disorder)
        "230710000",  # Lobar cerebral hemorrhage (disorder)
        "230711001",  # Thalamic hemorrhage (disorder)
        "230712008",  # Lacunar hemorrhage (disorder)
        "230718007",  # Subarachnoid hemorrhage due to ruptured arteriovenous malformation (disorder)
        "230719004",  # Intracranial subarachnoid hemorrhage due to ruptured aneurysm (disorder)
        "234003006",  # Arterial hemorrhage (disorder)
        "234004000",  # Hemorrhage of transplant artery (disorder)
        "234076002",  # Venous hemorrhage (disorder)
        "234077006",  # Hemorrhage of transplanted vein (disorder)
        "234155009",  # Congenital arteriovenous fistula hemorrhage (disorder)
        "234201003",  # Vascular graft hemorrhage (disorder)
        "234207004",  # Arteriovenous fistula hemorrhage (disorder)
        "234463000",  # Combined coagulation factor deficiency (disorder)
        "234466008",  # Acquired coagulation disorder (disorder)
        "25904003",  # Acquired coagulation factor deficiency (disorder)
        "276277008",  # Subarachnoid hemorrhage from multiple aneurysms (disorder)
        "276278003",  # Subarachnoid hemorrhage from anterior cerebral artery aneurysm (disorder)
        "276280009",  # Subarachnoid hemorrhage from middle cerebral artery aneurysm (disorder)
        "276281008",  # Subarachnoid hemorrhage from posterior cerebral artery aneurysm (disorder)
        "276282001",  # Subarachnoid hemorrhage from anterior communicating artery aneurysm (disorder)
        "276283006",  # Subarachnoid hemorrhage from posterior communicating artery aneurysm (disorder)
        "276284000",  # Subarachnoid hemorrhage from basilar artery aneurysm (disorder)
        "276285004",  # Subarachnoid hemorrhage from posterior inferior cerebellar artery aneurysm (disorder)
        "276286003",  # Subarachnoid hemorrhage from carotid artery aneurysm (disorder)
        "276622005",  # Epicranial subaponeurotic hemorrhage (disorder)
        "276637009",  # Hemorrhagic pulmonary edema (disorder)
        "282707003",  # Acquired inhibitor of coagulation (disorder)
        "291571000119106",  # Spontaneous cerebral hemorrhage (disorder)
        "291591000119107",  # Subacute non-traumatic intracranial subdural hemorrhage (disorder)
        "35486000",  # Subdural intracranial hemorrhage (disorder)
        "403614002",  # Purpura due to drug-induced coagulation abnormality (disorder)
        "43262000",  # Subdural hemorrhage following injury with open intracranial wound AND concussion (disorder)
        "439156006",  # Acquired combined coagulation factor deficiency (disorder)
        "439157002",  # Hereditary combined coagulation factor deficiency (disorder)
        "450379002",  # Hemorrhage into extradural space of spine (disorder)
        "450410005",  # Intracranial hemorrhage following injury (disorder)
        "49177006",  # Postpartum coagulation defect with hemorrhage (disorder)
        "5251007",  # Subarachnoid hemorrhage following injury with open intracranial wound (disorder)
        "5571000124103",  # Cerebrovascular accident with intracranial hemorrhage (disorder)
        "64509006",  # Acquired coagulation factor inhibitor disorder (disorder)
        "64779008",  # Blood coagulation disorder (disorder)
        "67406007",  # Disseminated intravascular coagulation (disorder)
        "69500007",  # Blood coagulation disorder due to liver disease (disorder)
        "715559004",  # Combined deficiency of factor V and factor VIII (disorder)
        "722628000",  # Traumatic hemorrhage of subdural space of infratentorial region (disorder)
        "722630003",  # Traumatic hemorrhage of cerebral white matter (disorder)
        "722631004",  # Traumatic hemorrhage of cerebellum (disorder)
        "722632006",  # Traumatic hemorrhage of brainstem (disorder)
        "722633001",  # Multiple traumatic hemorrhages of brain tissue (disorder)
        "722703002",  # Traumatic intraventricular hemorrhage (disorder)
        "723138000",  # Primary traumatic hemorrhage of brainstem (disorder)
        "723139008",  # Secondary traumatic hemorrhage of brainstem (disorder)
        "723141009",  # Intracranial hemorrhage co-occurrent and due to complex wound of head (disorder)
        "73020009",  # Cerebral hemisphere hemorrhage (disorder)
        "77768006",  # Intracranial hemorrhage following injury with open intracranial wound (disorder)
        "78879009",  # Intracranial hemorrhage following injury with open intracranial wound AND concussion (disorder)
        "86075001",  # Coagulation factor deficiency syndrome (disorder)
    }

class AcutePepticUlcer(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for peptic ulcer.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes ICD-10 and SNOMED codes associated with active peptic ulcer.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Acute Peptic Ulcer"
    OID = "2.16.840.1.113883.3.3157.4031"
    DEFINITION_VERSION = "20190216"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "K270",  # Acute peptic ulcer, site unspecified, with hemorrhage
        "K271",  # Acute peptic ulcer, site unspecified, with perforation
        "K272",  # Acute peptic ulcer, site unspecified, with both hemorrhage and perforation
        "K273",  # Acute peptic ulcer, site unspecified, without hemorrhage or perforation
    }

    SNOMEDCT = {
        "111353003",  # Acute peptic ulcer with hemorrhage and perforation (disorder)
        "12274003",  # Acute peptic ulcer with hemorrhage (disorder)
        "196682000",  # Acute peptic ulcer (disorder)
        "196687006",  # Acute peptic ulcer with obstruction (disorder)
        "22157005",  # Acute peptic ulcer with hemorrhage but without obstruction (disorder)
        "28945005",  # Acute peptic ulcer with hemorrhage, with perforation AND with obstruction (disorder)
        "3023008",  # Acute peptic ulcer without hemorrhage, without perforation AND without obstruction (disorder)
        "34921009",  # Acute peptic ulcer with perforation but without obstruction (disorder)
        "35681000",  # Acute peptic ulcer with perforation AND obstruction (disorder)
        "43406003",  # Acute peptic ulcer with hemorrhage AND obstruction (disorder)
        "45485004",  # Acute peptic ulcer without hemorrhage AND without perforation (disorder)
        "47064007",  # Acute peptic ulcer with hemorrhage AND with perforation but without obstruction (disorder)
        "58085004",  # Acute peptic ulcer without hemorrhage AND without perforation but with obstruction (disorder)
        "79118000",  # Acute peptic ulcer with perforation (disorder)
    }

class AllergyToThrombolytics(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for allergies to thrombolytic medications used for treatment of ST-segment elevation myocardial infaction (STEMI).

    **Data Element Scope:** This value set may use a model element related to Finding.

    **Inclusion Criteria:** Includes SNOMED-CT codes that represent allergies to thrombolytic medications for STEMI treatment.

    **Exclusion Criteria:** No Exclusions.
    """

    VALUE_SET_NAME = "Allergy to thrombolytics"
    OID = "2.16.840.1.113762.1.4.1170.5"
    DEFINITION_VERSION = "20250211"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    SNOMEDCT = {
        "1300135006",  # Allergy to reteplase (finding)
        "1300136007",  # Allergy to tenecteplase (finding)
        "295108008",  # Allergy to streptokinase (finding)
        "295109000",  # Allergy to urokinase (finding)
        "295110005",  # Allergy to alteplase (finding)
        "295111009",  # Allergy to anistreplase (finding)
    }

class AorticDissectionOrRupturedAorticAneurysm(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for aortic dissection or rupture.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes ICD-10CM and SNOMEDCT codes that represent aortic dissection or rupture.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Aortic Dissection or Ruptured Aortic Aneurysm"
    OID = "2.16.840.1.113883.3.3157.4028"
    DEFINITION_VERSION = "20190216"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "I71010",  # Dissection of ascending aorta
        "I7110",  # Thoracic aortic aneurysm, ruptured, unspecified
        "I7111",  # Aneurysm of the ascending aorta, ruptured
        "I7112",  # Aneurysm of the aortic arch, ruptured
        "I7113",  # Aneurysm of the descending thoracic aorta, ruptured
        "I7150",  # Thoracoabdominal aortic aneurysm, ruptured, unspecified
        "I7151",  # Supraceliac aneurysm of the thoracoabdominal aorta, ruptured
        "I7152",  # Paravisceral aneurysm of the thoracoabdominal aorta, ruptured
        "I718",  # Aortic aneurysm of unspecified site, ruptured
    }

    SNOMEDCT = {
        "14336007",  # Ruptured abdominal aortic aneurysm (disorder)
        "195258006",  # Thoracic aortic aneurysm which has ruptured (disorder)
        "195265003",  # Thoracoabdominal aortic aneurysm, ruptured (disorder)
        "233996000",  # Type II dissection of thoracic aorta (disorder)
        "233997009",  # Dissection of distal aorta (disorder)
        "301899003",  # Dissection of proximal aorta (disorder)
        "308546005",  # Dissection of aorta (disorder)
        "314184006",  # Ruptured suprarenal aortic aneurysm (disorder)
        "448528000",  # Aneurysm of aortic sinus of Valsalva with rupture to pericardial cavity (disorder)
        "448723007",  # Aneurysm of aortic sinus of Valsalva with rupture to left atrium (disorder)
        "448724001",  # Aneurysm of aortic sinus of Valsalva with rupture to left ventricle (disorder)
        "448744007",  # Aneurysm of aortic sinus of Valsalva with rupture to pulmonary artery (disorder)
        "448745008",  # Aneurysm of aortic sinus of Valsalva with rupture to right atrium (disorder)
        "448746009",  # Aneurysm of aortic sinus of Valsalva with rupture to right ventricle (disorder)
        "73067008",  # Ruptured aortic aneurysm (disorder)
    }

class CardiopulmonaryEmergency(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for cardiopulmonary emergency.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes ICD-10CM and SNOMEDCT codes associated with cardiopulmonary emergencies.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Cardiopulmonary Emergency"
    OID = "2.16.840.1.113883.3.3157.4048"
    DEFINITION_VERSION = "20190216"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "I462",  # Cardiac arrest due to underlying cardiac condition
        "I468",  # Cardiac arrest due to other underlying condition
        "I469",  # Cardiac arrest, cause unspecified
        "I4721",  # Torsades de pointes
        "I4901",  # Ventricular fibrillation
        "I97120",  # Postprocedural cardiac arrest following cardiac surgery
        "I97121",  # Postprocedural cardiac arrest following other surgery
        "I97710",  # Intraoperative cardiac arrest during cardiac surgery
        "I97711",  # Intraoperative cardiac arrest during other surgery
        "J9600",  # Acute respiratory failure, unspecified whether with hypoxia or hypercapnia
        "J9601",  # Acute respiratory failure with hypoxia
        "J9602",  # Acute respiratory failure with hypercapnia
        "R092",  # Respiratory arrest
    }

    SNOMEDCT = {
        "213213007",  # Cardiac arrest as a complication of care (disorder)
        "233927002",  # Cardiac arrest with successful resuscitation (disorder)
        "261195002",  # Circulatory arrest (disorder)
        "276259003",  # Respiratory arrest preceding cardiac arrest (disorder)
        "309809007",  # Electromechanical dissociation with successful resuscitation (disorder)
        "31722008",  # Torsades de pointes (disorder)
        "397829000",  # Asystole (disorder)
        "405806007",  # Pulseless ventricular tachycardia (disorder)
        "410429000",  # Cardiac arrest (disorder)
        "410430005",  # Cardiorespiratory arrest (disorder)
        "422970001",  # Cardiac arrest due to trauma (disorder)
        "423168004",  # Cardiac arrest due to respiratory disorder (disorder)
        "423191000",  # Cardiac arrest due to cardiac disorder (disorder)
        "424390001",  # Cardiac arrest caused by electrocution (disorder)
        "424571008",  # Cardiac arrest due to drowning (disorder)
        "426525004",  # Sustained ventricular tachycardia (disorder)
        "426882006",  # Electrocardiographic torsades de pointes (finding)
        "426975008",  # Electrocardiogram: pulseless electrical activity (finding)
        "5561000124105",  # Idiopathic cardiac arrest (disorder)
        "62657007",  # Cardiac arrest AND/OR failure following anesthesia AND/OR sedation in labor AND/OR delivery (disorder)
        "699748007",  # Cardiorespiratory arrest with successful resuscitation (disorder)
        "703162001",  # Bradycardic cardiac arrest (disorder)
        "71908006",  # Ventricular fibrillation (disorder)
        "78240009",  # Cardiac arrest due to pacemaker failure (disorder)
        "87317003",  # Respiratory arrest (disorder)
    }

class CerebralVascularLesion(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for cerebral vascular lesion.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes ICD-10 and SNOMEDCT codes associated with cerebral vascular lesions.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Cerebral Vascular Lesion"
    OID = "2.16.840.1.113883.3.3157.4025"
    DEFINITION_VERSION = "20190216"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "C700",  # Malignant neoplasm of cerebral meninges
        "C701",  # Malignant neoplasm of spinal meninges
        "C709",  # Malignant neoplasm of meninges, unspecified
        "C710",  # Malignant neoplasm of cerebrum, except lobes and ventricles
        "C711",  # Malignant neoplasm of frontal lobe
        "C712",  # Malignant neoplasm of temporal lobe
        "C713",  # Malignant neoplasm of parietal lobe
        "C714",  # Malignant neoplasm of occipital lobe
        "C715",  # Malignant neoplasm of cerebral ventricle
        "C716",  # Malignant neoplasm of cerebellum
        "C717",  # Malignant neoplasm of brain stem
        "C718",  # Malignant neoplasm of overlapping sites of brain
        "C719",  # Malignant neoplasm of brain, unspecified
        "C760",  # Malignant neoplasm of head, face and neck
        "C768",  # Malignant neoplasm of other specified ill-defined sites
        "C770",  # Secondary and unspecified malignant neoplasm of lymph nodes of head, face and neck
        "C7931",  # Secondary malignant neoplasm of brain
        "C7932",  # Secondary malignant neoplasm of cerebral meninges
        "C7989",  # Secondary malignant neoplasm of other specified sites
        "C799",  # Secondary malignant neoplasm of unspecified site
        "C7A00",  # Malignant carcinoid tumor of unspecified site
        "C7A098",  # Malignant carcinoid tumors of other sites
        "C7B00",  # Secondary carcinoid tumors, unspecified site
        "C7B09",  # Secondary carcinoid tumors of other sites
        "C7B1",  # Secondary Merkel cell carcinoma
        "C800",  # Disseminated malignant neoplasm, unspecified
        "C801",  # Malignant (primary) neoplasm, unspecified
        "C8100",  # Nodular lymphocyte predominant Hodgkin lymphoma, unspecified site
        "C8101",  # Nodular lymphocyte predominant Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8110",  # Nodular sclerosis Hodgkin lymphoma, unspecified site
        "C8111",  # Nodular sclerosis Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8120",  # Mixed cellularity Hodgkin lymphoma, unspecified site
        "C8121",  # Mixed cellularity Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8130",  # Lymphocyte depleted Hodgkin lymphoma, unspecified site
        "C8131",  # Lymphocyte depleted Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8140",  # Lymphocyte-rich Hodgkin lymphoma, unspecified site
        "C8141",  # Lymphocyte-rich Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8170",  # Other Hodgkin lymphoma, unspecified site
        "C8171",  # Other Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8190",  # Hodgkin lymphoma, unspecified, unspecified site
        "C8191",  # Hodgkin lymphoma, unspecified, lymph nodes of head, face, and neck
        "C8200",  # Follicular lymphoma grade I, unspecified site
        "C8201",  # Follicular lymphoma grade I, lymph nodes of head, face, and neck
        "C8210",  # Follicular lymphoma grade II, unspecified site
        "C8211",  # Follicular lymphoma grade II, lymph nodes of head, face, and neck
        "C8220",  # Follicular lymphoma grade III, unspecified, unspecified site
        "C8221",  # Follicular lymphoma grade III, unspecified, lymph nodes of head, face, and neck
        "C8230",  # Follicular lymphoma grade IIIa, unspecified site
        "C8231",  # Follicular lymphoma grade IIIa, lymph nodes of head, face, and neck
        "C8240",  # Follicular lymphoma grade IIIb, unspecified site
        "C8241",  # Follicular lymphoma grade IIIb, lymph nodes of head, face, and neck
        "C8250",  # Diffuse follicle center lymphoma, unspecified site
        "C8251",  # Diffuse follicle center lymphoma, lymph nodes of head, face, and neck
        "C8260",  # Cutaneous follicle center lymphoma, unspecified site
        "C8261",  # Cutaneous follicle center lymphoma, lymph nodes of head, face, and neck
        "C8280",  # Other types of follicular lymphoma, unspecified site
        "C8281",  # Other types of follicular lymphoma, lymph nodes of head, face, and neck
        "C8290",  # Follicular lymphoma, unspecified, unspecified site
        "C8291",  # Follicular lymphoma, unspecified, lymph nodes of head, face, and neck
        "C8300",  # Small cell B-cell lymphoma, unspecified site
        "C8301",  # Small cell B-cell lymphoma, lymph nodes of head, face, and neck
        "C8310",  # Mantle cell lymphoma, unspecified site
        "C8311",  # Mantle cell lymphoma, lymph nodes of head, face, and neck
        "C8330",  # Diffuse large B-cell lymphoma, unspecified site
        "C8331",  # Diffuse large B-cell lymphoma, lymph nodes of head, face, and neck
        "C83390",  # Primary central nervous system lymphoma
        "C8350",  # Lymphoblastic (diffuse) lymphoma, unspecified site
        "C8351",  # Lymphoblastic (diffuse) lymphoma, lymph nodes of head, face, and neck
        "C8370",  # Burkitt lymphoma, unspecified site
        "C8371",  # Burkitt lymphoma, lymph nodes of head, face, and neck
        "C8380",  # Other non-follicular lymphoma, unspecified site
        "C8381",  # Other non-follicular lymphoma, lymph nodes of head, face, and neck
        "C8390",  # Non-follicular (diffuse) lymphoma, unspecified, unspecified site
        "C8391",  # Non-follicular (diffuse) lymphoma, unspecified, lymph nodes of head, face, and neck
        "C8400",  # Mycosis fungoides, unspecified site
        "C8401",  # Mycosis fungoides, lymph nodes of head, face, and neck
        "C8410",  # Sezary disease, unspecified site
        "C8411",  # Sezary disease, lymph nodes of head, face, and neck
        "C8440",  # Peripheral T-cell lymphoma, not elsewhere classified, unspecified site
        "C8441",  # Peripheral T-cell lymphoma, not elsewhere classified, lymph nodes of head, face, and neck
        "C8460",  # Anaplastic large cell lymphoma, ALK-positive, unspecified site
        "C8461",  # Anaplastic large cell lymphoma, ALK-positive, lymph nodes of head, face, and neck
        "C8470",  # Anaplastic large cell lymphoma, ALK-negative, unspecified site
        "C8471",  # Anaplastic large cell lymphoma, ALK-negative, lymph nodes of head, face, and neck
        "C8490",  # Mature T/NK-cell lymphomas, unspecified, unspecified site
        "C8491",  # Mature T/NK-cell lymphomas, unspecified, lymph nodes of head, face, and neck
        "C84A0",  # Cutaneous T-cell lymphoma, unspecified, unspecified site
        "C84A1",  # Cutaneous T-cell lymphoma, unspecified lymph nodes of head, face, and neck
        "C84Z0",  # Other mature T/NK-cell lymphomas, unspecified site
        "C84Z1",  # Other mature T/NK-cell lymphomas, lymph nodes of head, face, and neck
        "C8510",  # Unspecified B-cell lymphoma, unspecified site
        "C8511",  # Unspecified B-cell lymphoma, lymph nodes of head, face, and neck
        "C8520",  # Mediastinal (thymic) large B-cell lymphoma, unspecified site
        "C8521",  # Mediastinal (thymic) large B-cell lymphoma, lymph nodes of head, face, and neck
        "C8580",  # Other specified types of non-Hodgkin lymphoma, unspecified site
        "C8581",  # Other specified types of non-Hodgkin lymphoma, lymph nodes of head, face, and neck
        "C8590",  # Non-Hodgkin lymphoma, unspecified, unspecified site
        "C8591",  # Non-Hodgkin lymphoma, unspecified, lymph nodes of head, face, and neck
        "C9620",  # Malignant mast cell neoplasm, unspecified
        "C9621",  # Aggressive systemic mastocytosis
        "C9622",  # Mast cell sarcoma
        "C9629",  # Other malignant mast cell neoplasm
        "C964",  # Sarcoma of dendritic cells (accessory cells)
        "C965",  # Multifocal and unisystemic Langerhans-cell histiocytosis
        "C966",  # Unifocal Langerhans-cell histiocytosis
        "C96A",  # Histiocytic sarcoma
        "D038",  # Melanoma in situ of other sites
        "D039",  # Melanoma in situ, unspecified
        "D098",  # Carcinoma in situ of other specified sites
        "D099",  # Carcinoma in situ, unspecified
        "D164",  # Benign neoplasm of bones of skull and face
        "D170",  # Benign lipomatous neoplasm of skin and subcutaneous tissue of head, face and neck
        "D1779",  # Benign lipomatous neoplasm of other sites
        "D179",  # Benign lipomatous neoplasm, unspecified
        "D1800",  # Hemangioma unspecified site
        "D1802",  # Hemangioma of intracranial structures
        "D210",  # Benign neoplasm of connective and other soft tissue of head, face and neck
        "D320",  # Benign neoplasm of cerebral meninges
        "D321",  # Benign neoplasm of spinal meninges
        "D329",  # Benign neoplasm of meninges, unspecified
        "D330",  # Benign neoplasm of brain, supratentorial
        "D331",  # Benign neoplasm of brain, infratentorial
        "D332",  # Benign neoplasm of brain, unspecified
        "D367",  # Benign neoplasm of other specified sites
        "D369",  # Benign neoplasm, unspecified site
        "D3A00",  # Benign carcinoid tumor of unspecified site
        "D420",  # Neoplasm of uncertain behavior of cerebral meninges
        "D421",  # Neoplasm of uncertain behavior of spinal meninges
        "D429",  # Neoplasm of uncertain behavior of meninges, unspecified
        "D430",  # Neoplasm of uncertain behavior of brain, supratentorial
        "D431",  # Neoplasm of uncertain behavior of brain, infratentorial
        "D432",  # Neoplasm of uncertain behavior of brain, unspecified
        "D433",  # Neoplasm of uncertain behavior of cranial nerves
        "D434",  # Neoplasm of uncertain behavior of spinal cord
        "D45",  # Polycythemia vera
        "D460",  # Refractory anemia without ring sideroblasts, so stated
        "D461",  # Refractory anemia with ring sideroblasts
        "D4620",  # Refractory anemia with excess of blasts, unspecified
        "D4621",  # Refractory anemia with excess of blasts 1
        "D4622",  # Refractory anemia with excess of blasts 2
        "D464",  # Refractory anemia, unspecified
        "D46A",  # Refractory cytopenia with multilineage dysplasia
        "D46B",  # Refractory cytopenia with multilineage dysplasia and ring sideroblasts
        "D46C",  # Myelodysplastic syndrome with isolated del(5q) chromosomal abnormality
        "D4709",  # Other mast cell neoplasms of uncertain behavior
        "D473",  # Essential (hemorrhagic) thrombocythemia
        "D487",  # Neoplasm of uncertain behavior of other specified sites
        "D489",  # Neoplasm of uncertain behavior, unspecified
        "D496",  # Neoplasm of unspecified behavior of brain
        "D4989",  # Neoplasm of unspecified behavior of other specified sites
        "D499",  # Neoplasm of unspecified behavior of unspecified site
        "I608",  # Other nontraumatic subarachnoid hemorrhage
        "I609",  # Nontraumatic subarachnoid hemorrhage, unspecified
        "Q280",  # Arteriovenous malformation of precerebral vessels
        "Q282",  # Arteriovenous malformation of cerebral vessels
    }

    SNOMEDCT = {
        "100721000119109",  # High grade astrocytoma of brain (disorder)
        "100731000119107",  # Low grade astrocytoma of brain (disorder)
        "104431000119107",  # Lipomyelomeningocele (disorder)
        "104981000119104",  # Oligodendroglioma of cerebrum (disorder)
        "107561000119107",  # Ependymoma of brain stem (disorder)
        "107571000119101",  # Oligodendroglioma of brain stem (disorder)
        "107581000119103",  # Astrocytoma of brain stem (disorder)
        "109912006",  # Overlapping malignant neoplasm of brain (disorder)
        "109913001",  # Benign neoplasm of meninges (disorder)
        "109914007",  # Neoplasm of uncertain behavior of meninges (disorder)
        "109915008",  # Primary malignant neoplasm of meninges (disorder)
        "11471000224106",  # Diffuse intrinsic pontine glioma (disorder)
        "1155991005",  # Metastatic malignant neoplasm to leptomeninges (disorder)
        "1156413005",  # Gliomatosis cerebri (disorder)
        "1156414004",  # Giant cell glioblastoma of brain (disorder)
        "1156415003",  # Protoplasmic astrocytoma of brain (disorder)
        "1156416002",  # Gemistocytic astrocytoma of brain (disorder)
        "1156417006",  # Fibrillary astrocytoma of brain (disorder)
        "1156455001",  # Pleomorphic xanthoastrocytoma of brain (disorder)
        "1156456000",  # Pilomyxoid astrocytoma of brain (disorder)
        "1156457009",  # Papillary glioneuronal tumor of brain (disorder)
        "1156460002",  # Desmoplastic nodular medulloblastoma of brain (disorder)
        "1156461003",  # Gliosarcoma of brain (disorder)
        "1156469001",  # Large cell medulloblastoma of brain (disorder)
        "1156470000",  # Atypical papilloma of choroid plexus (disorder)
        "1156471001",  # Choroid plexus carcinoma (disorder)
        "1156472008",  # Papillary tumor of pineal region (disorder)
        "1156473003",  # Pineocytoma (disorder)
        "1157060001",  # Diffuse astrocytoma of brain (disorder)
        "1157161000",  # Meningioma of uncertain behavior (disorder)
        "1157249003",  # Invasive benign pituitary adenoma (disorder)
        "1179377000",  # Adenocarcinoma of pituitary gland (disorder)
        "1196837008",  # Dysembryoplastic neuroepithelial neoplasm of brain (disorder)
        "1196839006",  # Primary embryonal carcinoma of brain (disorder)
        "1196844004",  # Primary malignant astrocytoma of brain (disorder)
        "1196860000",  # Primary anaplastic large cell medulloblastoma of brain (disorder)
        "1196898000",  # Primary malignant atypical teratoid rhabdoid neoplasm of brain (disorder)
        "1196902006",  # Angiocentric glioma of brain (disorder)
        "1197298003",  # Primary malignant germ cell neoplasm of pineal gland (disorder)
        "1197306009",  # Myxopapillary ependymoma of brain (disorder)
        "1197332003",  # Primary malignant melanoma of central nervous system (disorder)
        "1197343006",  # Primary teratoma of brain (disorder)
        "1217383007",  # RELA fusion-positive supratentorial ependymoma (disorder)
        "1259720004",  # Primary malignant glioma of brain (disorder)
        "126631009",  # Neoplasm of head (disorder)
        "126947009",  # Cerebellopontine angle tumor (disorder)
        "126948004",  # Cerebellopontine angle meningioma (disorder)
        "126951006",  # Neoplasm of central nervous system (disorder)
        "126952004",  # Neoplasm of brain (disorder)
        "126959008",  # Neoplasm of cerebral meninges (disorder)
        "126961004",  # Neoplasm of brain stem (disorder)
        "126964007",  # Neoplasm of spinal meninges (disorder)
        "126965008",  # Neoplasm of meninges (disorder)
        "134209002",  # Prolactinoma (disorder)
        "135801000119109",  # Compression of brain due to focal lesion (disorder)
        "135884009",  # Benign neoplasm of pons (disorder)
        "15629411000119106",  # Subependymoma of brain (disorder)
        "15863451000119107",  # Lipoma of brain (disorder)
        "16845301000119108",  # Primary glioblastoma multiforme of cerebellum (disorder)
        "16845341000119105",  # Primary glioblastoma multiforme of brainstem (disorder)
        "16903531000119107",  # Atypical meningioma of cerebral meninges (disorder)
        "188295009",  # Malignant neoplasm of cerebral peduncle (disorder)
        "188296005",  # Malignant neoplasm of medulla oblongata (disorder)
        "188297001",  # Malignant neoplasm of midbrain (disorder)
        "188298006",  # Malignant neoplasm of pons (disorder)
        "188312005",  # Malignant neoplasm of cerebral dura mater (disorder)
        "188313000",  # Malignant neoplasm of cerebral arachnoid mater (disorder)
        "188315007",  # Malignant neoplasm of cerebral pia mater (disorder)
        "188317004",  # Malignant neoplasm of spinal dura mater (disorder)
        "188318009",  # Malignant neoplasm of spinal arachnoid mater (disorder)
        "188319001",  # Malignant neoplasm of spinal pia mater (disorder)
        "188462001",  # Metastatic malignant neoplasm to brain and spinal cord (disorder)
        "189016009",  # Lipoma of spinal canal - intradural (disorder)
        "189162003",  # Benign neoplasm of brain, supratentorial (disorder)
        "189167009",  # Spinal meningioma (disorder)
        "189179009",  # Craniopharyngioma (disorder)
        "189487001",  # Neoplasm of uncertain or unknown behavior of brain, supratentorial (disorder)
        "189488006",  # Neoplasm of uncertain or unknown behavior of brain, infratentorial (disorder)
        "19090001000004101",  # Metastatic neoplasm of left basal ganglion (disorder)
        "204493007",  # Arteriovenous malformation of precerebral vessels (disorder)
        "230156002",  # Malignant meningitis (disorder)
        "234141001",  # Congenital arteriovenous malformation (disorder)
        "234142008",  # Cerebral arteriovenous malformation (disorder)
        "237719001",  # Pituitary adenoma with extrasellar extension (disorder)
        "23853001",  # Disorder of the central nervous system (disorder)
        "246556002",  # Central nervous system finding (finding)
        "253010003",  # Microprolactinoma (disorder)
        "253011004",  # Macroprolactinoma (disorder)
        "253192007",  # Fibrolipoma of filum terminale (disorder)
        "254935002",  # Intracranial tumor (disorder)
        "254938000",  # Astrocytoma of brain (disorder)
        "254940005",  # Oligodendroglioma of brain (disorder)
        "254943007",  # Benign tumor of choroid plexus (disorder)
        "254945000",  # Embryonal tumor of brain (disorder)
        "254955001",  # Pituitary carcinoma (disorder)
        "254956000",  # Pituitary adenoma (disorder)
        "254957009",  # Somatotroph adenoma (disorder)
        "254958004",  # Corticotroph adenoma (disorder)
        "254959007",  # Thyrotroph adenoma (disorder)
        "254960002",  # Gonadotroph adenoma (disorder)
        "254961003",  # Mixed-functioning pituitary adenoma (disorder)
        "254962005",  # Functionless pituitary adenoma (disorder)
        "254963000",  # Pituitary microadenoma (disorder)
        "254964006",  # Pituitary mesoadenoma (disorder)
        "254965007",  # Pituitary macroadenoma (disorder)
        "254966008",  # Suprasellar extension of pituitary adenoma (disorder)
        "254970000",  # Benign tumor of olfactory tract (disorder)
        "255055008",  # Neoplasm of head AND/OR neck (disorder)
        "255200003",  # Benign tumor of hypothalamus (disorder)
        "25521000119106",  # Neoplasm of uncertain behavior of hypothalamus (disorder)
        "261482004",  # Spinal arteriovenous malformation (disorder)
        "264906008",  # Pituitary macroadenoma with extrasellar extension (disorder)
        "271479005",  # Benign neoplasm of pituitary gland and craniopharyngeal duct (disorder)
        "274751001",  # Mass of head and/or neck (finding)
        "275269004",  # Benign cerebral tumor (disorder)
        "276826005",  # Malignant glioma of brain (disorder)
        "276828006",  # Glioblastoma multiforme of brain (disorder)
        "277299009",  # Ruptured cerebral arteriovenous malformation (disorder)
        "277301002",  # Ruptured spinal arteriovenous malformation (disorder)
        "277461004",  # Anaplastic astrocytoma of brain (disorder)
        "277505007",  # Medulloblastoma of cerebellum (disorder)
        "277507004",  # Pilocytic astrocytoma of cerebellum (disorder)
        "277522009",  # Solitary fibrous tumor of meninges (disorder)
        "277523004",  # Primary melanocytic lesion of meninges (disorder)
        "277526007",  # Diffuse melanosis of meninges (disorder)
        "277527003",  # Melanocytoma of meninges (disorder)
        "277530005",  # Malignant melanoma of meninges (disorder)
        "281560004",  # Neuroblastoma of brain (disorder)
        "301766008",  # Lesion of brain (disorder)
        "302820008",  # Intracranial meningioma (disorder)
        "307649006",  # Microglioma (disorder)
        "350841000119107",  # Benign neoplasm of infratentorial brain (disorder)
        "359619007",  # Pinealoma (disorder)
        "363473003",  # Malignant neoplasm of brainstem (disorder)
        "363474009",  # Malignant neoplasm of cerebral meninges (disorder)
        "363476006",  # Malignant neoplasm of spinal meninges (disorder)
        "363497007",  # Malignant tumor of meninges (disorder)
        "422840005",  # Mass lesion of brain (finding)
        "424151006",  # Anaplastic glioma of brain (disorder)
        "424276002",  # Malignant glioma of brainstem (disorder)
        "425868004",  # Benign papilloma of choroid plexus (disorder)
        "428960009",  # Low grade glioma of brainstem (disorder)
        "429306002",  # Benign neoplasm of spinal intradural intramedullary space (disorder)
        "429691007",  # Benign neoplasm of intradural space of spine (disorder)
        "429765002",  # Benign neoplasm of medulla oblongata (disorder)
        "432525007",  # Mass of head (finding)
        "443333004",  # Medulloblastoma (disorder)
        "444545003",  # Glioma of brainstem (disorder)
        "446531000124106",  # Meningioma of cerebellum (disorder)
        "448135004",  # Benign teratoma of pineal region (disorder)
        "448218008",  # Malignant neoplasm of cerebellopontine angle (disorder)
        "449799008",  # Subependymal giant cell astrocytoma (disorder)
        "5251000119108",  # Acquired arteriovenous malformation (disorder)
        "55342001",  # Neoplastic disease (disorder)
        "608817003",  # Pituicytoma (disorder)
        "682221000119103",  # Primary anaplastic astrocytoma of cerebrum (disorder)
        "682231000119100",  # Primary anaplastic astrocytoma of frontal lobe (disorder)
        "682241000119109",  # Primary anaplastic astrocytoma of occipital lobe (disorder)
        "682251000119106",  # Primary anaplastic astrocytoma of parietal lobe (disorder)
        "682261000119108",  # Primary anaplastic astrocytoma of temporal lobe (disorder)
        "682441000119105",  # Primary astrocytoma of cerebral ventricle (disorder)
        "682451000119107",  # Primary astrocytoma of frontal lobe (disorder)
        "682461000119109",  # Primary astrocytoma of occipital lobe (disorder)
        "682471000119103",  # Primary astrocytoma of parietal lobe (disorder)
        "682481000119100",  # Primary astrocytoma of temporal lobe (disorder)
        "683761000119107",  # Primary ependymoma of brain ventricle (disorder)
        "683791000119100",  # Primary ependymoma of parietal lobe (disorder)
        "684901000119107",  # Primary glioblastoma multiforme of cerebrum (disorder)
        "684911000119105",  # Primary glioblastoma multiforme of frontal lobe (disorder)
        "684921000119103",  # Primary glioblastoma multiforme of occipital lobe (disorder)
        "684931000119100",  # Primary glioblastoma multiforme of parietal lobe (disorder)
        "684941000119109",  # Primary glioblastoma multiforme of temporal lobe (disorder)
        "686431000119100",  # Primary malignant glioma of frontal lobe (disorder)
        "686441000119109",  # Primary malignant glioma of occipital lobe (disorder)
        "686451000119106",  # Primary malignant glioma of parietal lobe (disorder)
        "686461000119108",  # Primary malignant glioma of temporal lobe (disorder)
        "687331000119109",  # Primary oligodendroglioma of frontal lobe (disorder)
        "687351000119103",  # Primary oligodendroglioma of parietal lobe (disorder)
        "687361000119101",  # Primary oligodendroglioma of temporal lobe (disorder)
        "699318007",  # Supratentorial primitive neuroectodermal tumor (disorder)
        "699331002",  # Granular cell tumor of neurohypophysis (disorder)
        "699704002",  # Classic medulloblastoma (disorder)
        "702375004",  # Familial isolated pituitary adenoma (disorder)
        "713327005",  # Malignant meningioma of meninges of brain (disorder)
        "715904005",  # Pineal parenchymal tumor of intermediate differentiation (disorder)
        "716592003",  # Liponeurocytoma of cerebellum (disorder)
        "717003001",  # Hereditary cavernous hemangioma of brain (disorder)
        "721535002",  # Central neurocytoma of brain (disorder)
        "722671009",  # Metastatic malignant neoplasm of meninges (disorder)
        "722718001",  # Primary malignant meningioma (disorder)
        "724171006",  # Benign meningioma (disorder)
        "735757008",  # Primary ganglioneuroblastoma of brain (disorder)
        "762457009",  # Astroblastoma of brain (disorder)
        "765202001",  # Familial multiple benign meningioma (disorder)
        "767448007",  # Pineoblastoma (disorder)
        "770628008",  # Diffuse leptomeningeal melanocytosis (disorder)
        "770682007",  # Rosette-forming glioneuronal neoplasm (disorder)
        "778046002",  # Somatomammotropinoma (disorder)
        "780822000",  # Desmoplastic infantile astrocytoma and ganglioglioma (disorder)
        "783008000",  # Pituitary dermoid and epidermoid cysts (disorder)
        "788756004",  # Spindle cell oncocytoma of posterior pituitary gland (disorder)
        "788757008",  # Pituicytoma of posterior pituitary gland (disorder)
        "788758003",  # Sellar ependymoma of posterior pituitary gland (disorder)
        "81308009",  # Disorder of brain (disorder)
        "816205008",  # Malignant pituitary blastoma (disorder)
        "830032008",  # Dermoid cyst of occipital lobe (disorder)
        "830033003",  # Dermoid cyst of brain (disorder)
        "8551000119100",  # Benign neoplasm of spinal intradural extramedullary space (disorder)
        "85974009",  # Mass of intracranial structure (finding)
        "87091000119101",  # Malignant glioma of cerebrum (disorder)
        "87111000119109",  # Malignant glioma of hypothalamus (disorder)
        "87121000119102",  # Malignant glioma of cerebellum (disorder)
        "87161000119107",  # Benign ependymoma of brain (disorder)
        "90811000119100",  # Low grade malignant glioma of brain (disorder)
        "90831000119105",  # Grade 4 malignant glioma of brain (disorder)
        "92029009",  # Benign neoplasm of brain stem (disorder)
        "92030004",  # Benign neoplasm of brain (disorder)
        "92050000",  # Benign neoplasm of cerebellum (disorder)
        "92051001",  # Benign neoplasm of cerebral meninges (disorder)
        "92052008",  # Benign neoplasm of cerebral ventricle (disorder)
        "92072003",  # Benign neoplasm of craniopharyngeal duct (disorder)
        "92114009",  # Benign neoplasm of frontal lobe (disorder)
        "92253009",  # Benign neoplasm of occipital lobe (disorder)
        "92276007",  # Benign neoplasm of parietal lobe (disorder)
        "92294001",  # Benign neoplasm of pineal gland (disorder)
        "92296004",  # Benign neoplasm of pituitary gland (disorder)
        "92406008",  # Benign neoplasm of spinal meninges (disorder)
        "92427003",  # Benign neoplasm of temporal lobe (disorder)
        "93726004",  # Primary malignant neoplasm of brain stem (disorder)
        "93727008",  # Primary malignant neoplasm of brain (disorder)
        "93746009",  # Primary malignant neoplasm of cerebellum (disorder)
        "93747000",  # Primary malignant neoplasm of cerebral meninges (disorder)
        "93748005",  # Primary malignant neoplasm of cerebral ventricle (disorder)
        "93749002",  # Primary malignant neoplasm of cerebrum (disorder)
        "93768004",  # Primary malignant neoplasm of craniopharyngeal duct (disorder)
        "93807001",  # Primary malignant neoplasm of frontal lobe (disorder)
        "93928006",  # Primary malignant neoplasm of occipital lobe (disorder)
        "93946000",  # Primary malignant neoplasm of parietal lobe (disorder)
        "93962006",  # Primary malignant neoplasm of pineal gland (disorder)
        "93964007",  # Primary malignant neoplasm of pituitary gland (disorder)
        "94069006",  # Primary malignant neoplasm of spinal meninges (disorder)
        "94086000",  # Primary malignant neoplasm of temporal lobe (disorder)
        "94224009",  # Metastatic malignant neoplasm to brain stem (disorder)
        "94225005",  # Metastatic malignant neoplasm to brain (disorder)
        "94245002",  # Metastatic malignant neoplasm to cerebellum (disorder)
        "94246001",  # Metastatic malignant neoplasm to cerebral meninges (disorder)
        "94247005",  # Metastatic malignant neoplasm to cerebral ventricle (disorder)
        "94248000",  # Metastatic malignant neoplasm to cerebrum (disorder)
        "94267001",  # Metastatic malignant neoplasm to craniopharyngeal duct (disorder)
        "94309003",  # Metastatic malignant neoplasm to frontal lobe (disorder)
        "94448002",  # Metastatic malignant neoplasm to occipital lobe (disorder)
        "94471000",  # Metastatic malignant neoplasm to parietal lobe (disorder)
        "94489004",  # Metastatic malignant neoplasm to pineal gland (disorder)
        "94491007",  # Metastatic malignant neoplasm to pituitary gland (disorder)
        "94601008",  # Metastatic malignant neoplasm to spinal meninges (disorder)
        "94622002",  # Metastatic malignant neoplasm to temporal lobe (disorder)
        "94766006",  # Neoplasm of uncertain behavior of brain stem (disorder)
        "94767002",  # Neoplasm of uncertain behavior of brain (disorder)
        "94786005",  # Neoplasm of uncertain behavior of cerebellum (disorder)
        "94787001",  # Neoplasm of uncertain behavior of cerebral meninges (disorder)
        "94788006",  # Neoplasm of uncertain behavior of cerebral ventricle (disorder)
        "94789003",  # Neoplasm of uncertain behavior of cerebrum (disorder)
        "94808000",  # Neoplasm of uncertain behavior of craniopharyngeal duct (disorder)
        "94847003",  # Neoplasm of uncertain behavior of frontal lobe (disorder)
        "94968001",  # Neoplasm of uncertain behavior of occipital lobe (disorder)
        "94986003",  # Neoplasm of uncertain behavior of parietal lobe (disorder)
        "95002008",  # Neoplasm of uncertain behavior of pineal gland (disorder)
        "95004009",  # Neoplasm of uncertain behavior of pituitary gland (disorder)
        "95109002",  # Neoplasm of uncertain behavior of spinal meninges (disorder)
        "95126008",  # Neoplasm of uncertain behavior of temporal lobe (disorder)
        "99131000119108",  # Astrocytoma of cerebrum (disorder)
    }

class ClosedHeadAndFacialTrauma(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for closed head trauma or facial trauma.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes ICD-10 and SNOMED codes associated with closed head trauma or facial trauma.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Closed Head and Facial Trauma"
    OID = "2.16.840.1.113883.3.3157.4026"
    DEFINITION_VERSION = "20190216"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "S0003XA",  # Contusion of scalp, initial encounter
        "S0033XA",  # Contusion of nose, initial encounter
        "S00431A",  # Contusion of right ear, initial encounter
        "S00432A",  # Contusion of left ear, initial encounter
        "S00439A",  # Contusion of unspecified ear, initial encounter
        "S00531A",  # Contusion of lip, initial encounter
        "S00532A",  # Contusion of oral cavity, initial encounter
        "S0083XA",  # Contusion of other part of head, initial encounter
        "S0093XA",  # Contusion of unspecified part of head, initial encounter
        "S060X0A",  # Concussion without loss of consciousness, initial encounter
        "S060X1A",  # Concussion with loss of consciousness of 30 minutes or less, initial encounter
        "S060X9A",  # Concussion with loss of consciousness of unspecified duration, initial encounter
        "S061X0A",  # Traumatic cerebral edema without loss of consciousness, initial encounter
        "S061X1A",  # Traumatic cerebral edema with loss of consciousness of 30 minutes or less, initial encounter
        "S061X2A",  # Traumatic cerebral edema with loss of consciousness of 31 minutes to 59 minutes, initial encounter
        "S061X3A",  # Traumatic cerebral edema with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter
        "S061X4A",  # Traumatic cerebral edema with loss of consciousness of 6 hours to 24 hours, initial encounter
        "S061X9A",  # Traumatic cerebral edema with loss of consciousness of unspecified duration, initial encounter
        "S062X0A",  # Diffuse traumatic brain injury without loss of consciousness, initial encounter
        "S062X1A",  # Diffuse traumatic brain injury with loss of consciousness of 30 minutes or less, initial encounter
        "S062X2A",  # Diffuse traumatic brain injury with loss of consciousness of 31 minutes to 59 minutes, initial encounter
        "S062X3A",  # Diffuse traumatic brain injury with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter
        "S062X4A",  # Diffuse traumatic brain injury with loss of consciousness of 6 hours to 24 hours, initial encounter
        "S062X9A",  # Diffuse traumatic brain injury with loss of consciousness of unspecified duration, initial encounter
        "S06300A",  # Unspecified focal traumatic brain injury without loss of consciousness, initial encounter
        "S06301A",  # Unspecified focal traumatic brain injury with loss of consciousness of 30 minutes or less, initial encounter
        "S06302A",  # Unspecified focal traumatic brain injury with loss of consciousness of 31 minutes to 59 minutes, initial encounter
        "S06303A",  # Unspecified focal traumatic brain injury with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter
        "S06304A",  # Unspecified focal traumatic brain injury with loss of consciousness of 6 hours to 24 hours, initial encounter
        "S06309A",  # Unspecified focal traumatic brain injury with loss of consciousness of unspecified duration, initial encounter
        "S06340A",  # Traumatic hemorrhage of right cerebrum without loss of consciousness, initial encounter
        "S06341A",  # Traumatic hemorrhage of right cerebrum with loss of consciousness of 30 minutes or less, initial encounter
        "S06342A",  # Traumatic hemorrhage of right cerebrum with loss of consciousness of 31 minutes to 59 minutes, initial encounter
        "S06343A",  # Traumatic hemorrhage of right cerebrum with loss of consciousness of 1 hours to 5 hours 59 minutes, initial encounter
        "S06344A",  # Traumatic hemorrhage of right cerebrum with loss of consciousness of 6 hours to 24 hours, initial encounter
        "S06349A",  # Traumatic hemorrhage of right cerebrum with loss of consciousness of unspecified duration, initial encounter
        "S06350A",  # Traumatic hemorrhage of left cerebrum without loss of consciousness, initial encounter
        "S06351A",  # Traumatic hemorrhage of left cerebrum with loss of consciousness of 30 minutes or less, initial encounter
        "S06352A",  # Traumatic hemorrhage of left cerebrum with loss of consciousness of 31 minutes to 59 minutes, initial encounter
        "S06353A",  # Traumatic hemorrhage of left cerebrum with loss of consciousness of 1 hours to 5 hours 59 minutes, initial encounter
        "S06354A",  # Traumatic hemorrhage of left cerebrum with loss of consciousness of 6 hours to 24 hours, initial encounter
        "S06359A",  # Traumatic hemorrhage of left cerebrum with loss of consciousness of unspecified duration, initial encounter
        "S06360A",  # Traumatic hemorrhage of cerebrum, unspecified, without loss of consciousness, initial encounter
        "S06361A",  # Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness of 30 minutes or less, initial encounter
        "S06362A",  # Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness of 31 minutes to 59 minutes, initial encounter
        "S06363A",  # Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness of 1 hours to 5 hours 59 minutes, initial encounter
        "S06364A",  # Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness of 6 hours to 24 hours, initial encounter
        "S06369A",  # Traumatic hemorrhage of cerebrum, unspecified, with loss of consciousness of unspecified duration, initial encounter
        "S064X0A",  # Epidural hemorrhage without loss of consciousness, initial encounter
        "S064X1A",  # Epidural hemorrhage with loss of consciousness of 30 minutes or less, initial encounter
        "S064X2A",  # Epidural hemorrhage with loss of consciousness of 31 minutes to 59 minutes, initial encounter
        "S064X3A",  # Epidural hemorrhage with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter
        "S064X4A",  # Epidural hemorrhage with loss of consciousness of 6 hours to 24 hours, initial encounter
        "S064X9A",  # Epidural hemorrhage with loss of consciousness of unspecified duration, initial encounter
        "S065X0A",  # Traumatic subdural hemorrhage without loss of consciousness, initial encounter
        "S065X1A",  # Traumatic subdural hemorrhage with loss of consciousness of 30 minutes or less, initial encounter
        "S065X2A",  # Traumatic subdural hemorrhage with loss of consciousness of 31 minutes to 59 minutes, initial encounter
        "S065X3A",  # Traumatic subdural hemorrhage with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter
        "S065X4A",  # Traumatic subdural hemorrhage with loss of consciousness of 6 hours to 24 hours, initial encounter
        "S065X9A",  # Traumatic subdural hemorrhage with loss of consciousness of unspecified duration, initial encounter
        "S066X0A",  # Traumatic subarachnoid hemorrhage without loss of consciousness, initial encounter
        "S066X1A",  # Traumatic subarachnoid hemorrhage with loss of consciousness of 30 minutes or less, initial encounter
        "S066X2A",  # Traumatic subarachnoid hemorrhage with loss of consciousness of 31 minutes to 59 minutes, initial encounter
        "S066X3A",  # Traumatic subarachnoid hemorrhage with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter
        "S066X4A",  # Traumatic subarachnoid hemorrhage with loss of consciousness of 6 hours to 24 hours, initial encounter
        "S066X9A",  # Traumatic subarachnoid hemorrhage with loss of consciousness of unspecified duration, initial encounter
        "S06810A",  # Injury of right internal carotid artery, intracranial portion, not elsewhere classified without loss of consciousness, initial encounter
        "S06811A",  # Injury of right internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of 30 minutes or less, initial encounter
        "S06812A",  # Injury of right internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of 31 minutes to 59 minutes, initial encounter
        "S06813A",  # Injury of right internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter
        "S06814A",  # Injury of right internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of 6 hours to 24 hours, initial encounter
        "S06819A",  # Injury of right internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of unspecified duration, initial encounter
        "S06820A",  # Injury of left internal carotid artery, intracranial portion, not elsewhere classified without loss of consciousness, initial encounter
        "S06821A",  # Injury of left internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of 30 minutes or less, initial encounter
        "S06822A",  # Injury of left internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of 31 minutes to 59 minutes, initial encounter
        "S06823A",  # Injury of left internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter
        "S06824A",  # Injury of left internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of 6 hours to 24 hours, initial encounter
        "S06829A",  # Injury of left internal carotid artery, intracranial portion, not elsewhere classified with loss of consciousness of unspecified duration, initial encounter
        "S06890A",  # Other specified intracranial injury without loss of consciousness, initial encounter
        "S06891A",  # Other specified intracranial injury with loss of consciousness of 30 minutes or less, initial encounter
        "S06892A",  # Other specified intracranial injury with loss of consciousness of 31 minutes to 59 minutes, initial encounter
        "S06893A",  # Other specified intracranial injury with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter
        "S06894A",  # Other specified intracranial injury with loss of consciousness of 6 hours to 24 hours, initial encounter
        "S06899A",  # Other specified intracranial injury with loss of consciousness of unspecified duration, initial encounter
        "S069X0A",  # Unspecified intracranial injury without loss of consciousness, initial encounter
        "S069X1A",  # Unspecified intracranial injury with loss of consciousness of 30 minutes or less, initial encounter
        "S069X2A",  # Unspecified intracranial injury with loss of consciousness of 31 minutes to 59 minutes, initial encounter
        "S069X3A",  # Unspecified intracranial injury with loss of consciousness of 1 hour to 5 hours 59 minutes, initial encounter
        "S069X4A",  # Unspecified intracranial injury with loss of consciousness of 6 hours to 24 hours, initial encounter
        "S069X9A",  # Unspecified intracranial injury with loss of consciousness of unspecified duration, initial encounter
        "S0911XA",  # Strain of muscle and tendon of head, initial encounter
        "S0919XA",  # Other specified injury of muscle and tendon of head, initial encounter
        "S098XXA",  # Other specified injuries of head, initial encounter
    }

    SNOMEDCT = {
        "110030002",  # Concussion injury of brain (disorder)
        "110150000",  # Spinal cord concussion (disorder)
        "110237009",  # Contusion of condylar process of mandible (disorder)
        "110238004",  # Contusion of subcondylar region of mandible (disorder)
        "110239007",  # Contusion of coronoid process of mandible (disorder)
        "110240009",  # Contusion of ramus of mandible (disorder)
        "110241008",  # Contusion of symphysis of body of mandible (disorder)
        "110242001",  # Contusion of alveolar border of body of mandible (disorder)
        "110243006",  # Contusion of maxilla (disorder)
        "110244000",  # Contusion of forehead (disorder)
        "110246003",  # Contusion of preauricular region of face (disorder)
        "110247007",  # Contusion of malar region of face (disorder)
        "110248002",  # Contusion of circumoral region of face (disorder)
        "110249005",  # Contusion of chin (disorder)
        "110253007",  # Contusion of maxillary attached gingiva (disorder)
        "110254001",  # Contusion of mandibular attached gingiva (disorder)
        "110257008",  # Contusion of buccal mucosa (disorder)
        "110262009",  # Contusion of periorbital region (disorder)
        "111703006",  # Abrasion and/or friction burn of face with infection (disorder)
        "125668004",  # Contusion of face (disorder)
        "127295002",  # Traumatic brain injury (disorder)
        "127296001",  # Intracranial injury (disorder)
        "127298000",  # Traumatic brain injury with loss of consciousness (disorder)
        "127304009",  # Cerebellar contusion (disorder)
        "127305005",  # Brain stem contusion (disorder)
        "127306006",  # Cerebellar laceration (disorder)
        "127308007",  # Intracranial hemorrhage following injury with loss of consciousness (disorder)
        "127309004",  # Intracranial hemorrhage following injury with brief loss of consciousness (disorder)
        "127310009",  # Intracranial hemorrhage following injury with moderate loss of consciousness (disorder)
        "1386000",  # Intracranial hemorrhage (disorder)
        "141151000119101",  # Nontraumatic subdural hematoma with brain compression (disorder)
        "142851000119103",  # Spontaneous cerebellar hemorrhage (disorder)
        "14460007",  # Hemorrhage in optic nerve sheaths (disorder)
        "195155004",  # Subarachnoid hemorrhage from carotid siphon and bifurcation (disorder)
        "195160000",  # Intracranial subarachnoid hemorrhage from vertebral artery (disorder)
        "195165005",  # Basal ganglia hemorrhage (disorder)
        "195167002",  # External capsule hemorrhage (disorder)
        "195168007",  # Intracerebral hemorrhage with intraventricular hemorrhage (disorder)
        "195169004",  # Intracerebral hemorrhage, multiple localized (disorder)
        "195176009",  # Non-traumatic intracranial subdural hemorrhage (disorder)
        "20908003",  # Subcortical cerebral hemorrhage (disorder)
        "210991003",  # Abrasion and/or friction burn of head, infected (disorder)
        "21454007",  # Subarachnoid intracranial hemorrhage (disorder)
        "230710000",  # Lobar cerebral hemorrhage (disorder)
        "230711001",  # Thalamic hemorrhage (disorder)
        "230712008",  # Lacunar hemorrhage (disorder)
        "230718007",  # Subarachnoid hemorrhage due to ruptured arteriovenous malformation (disorder)
        "230719004",  # Intracranial subarachnoid hemorrhage due to ruptured aneurysm (disorder)
        "232278009",  # Transverse fracture of temporal bone (disorder)
        "23276006",  # Ventricular hemorrhage (disorder)
        "23713006",  # Contusion of cerebral cortex (disorder)
        "237702003",  # Pituitary hemorrhage (disorder)
        "262637006",  # Contusion of gingivae (disorder)
        "262638001",  # Contusion of intraoral surface of lip (disorder)
        "262639009",  # Contusion of oral alveolar mucosa (disorder)
        "262688009",  # Cortex laceration and contusion (disorder)
        "262689001",  # Contusion of cerebrum (disorder)
        "262692002",  # Burst lobe of brain (disorder)
        "262952002",  # Traumatic intracranial subdural hematoma (disorder)
        "262955000",  # Subarachnoid hemorrhage due to traumatic injury (disorder)
        "26652002",  # Abrasion and/or friction burn of scalp with infection (disorder)
        "269144002",  # Cerebral laceration and contusion (disorder)
        "269209005",  # Abrasion and/or friction burn of eyelids and/or periocular area (disorder)
        "270907008",  # Spontaneous intracranial subarachnoid hemorrhage (disorder)
        "274100004",  # Cerebral hemorrhage (disorder)
        "274164006",  # Minor head injury (disorder)
        "275382005",  # Cerebral trauma (disorder)
        "276277008",  # Subarachnoid hemorrhage from multiple aneurysms (disorder)
        "276278003",  # Subarachnoid hemorrhage from anterior cerebral artery aneurysm (disorder)
        "276280009",  # Subarachnoid hemorrhage from middle cerebral artery aneurysm (disorder)
        "276281008",  # Subarachnoid hemorrhage from posterior cerebral artery aneurysm (disorder)
        "276282001",  # Subarachnoid hemorrhage from anterior communicating artery aneurysm (disorder)
        "276283006",  # Subarachnoid hemorrhage from posterior communicating artery aneurysm (disorder)
        "276284000",  # Subarachnoid hemorrhage from basilar artery aneurysm (disorder)
        "276285004",  # Subarachnoid hemorrhage from posterior inferior cerebellar artery aneurysm (disorder)
        "276286003",  # Subarachnoid hemorrhage from carotid artery aneurysm (disorder)
        "276722003",  # Intracerebellar and posterior fossa hemorrhage (disorder)
        "281864001",  # Non-traumatic intracranial subdural hematoma (disorder)
        "282450009",  # Contusion of nasal septum (disorder)
        "2825006",  # Abrasion and/or friction burn of gum without infection (disorder)
        "283049005",  # Abrasion of head (disorder)
        "28318001",  # Basilar hemorrhage (disorder)
        "291591000119107",  # Subacute non-traumatic intracranial subdural hemorrhage (disorder)
        "301764006",  # Hematoma of brain (disorder)
        "301765007",  # Cerebellar hematoma (disorder)
        "301768009",  # Contusion of supraorbital area (disorder)
        "30742008",  # Abrasion and/or friction burn of nose without infection (disorder)
        "315048006",  # Traumatic extradural hematoma without open intracranial wound (disorder)
        "32728005",  # Hemorrhage due to ruptured congenital cerebral aneurysm (disorder)
        "35486000",  # Subdural intracranial hemorrhage (disorder)
        "37955001",  # Intracranial hematoma following injury (disorder)
        "417654008",  # Contusion of ocular adnexa and periocular tissues (disorder)
        "42429001",  # Cerebromeningeal hemorrhage (disorder)
        "425957003",  # Non-traumatic intracerebral ventricular hemorrhage (disorder)
        "428561000",  # Occipital subdural hematoma (disorder)
        "431266005",  # Intraparenchymal hematoma of brain (disorder)
        "441519008",  # Contusion of infraorbital nerve (disorder)
        "447396006",  # Concussion injury of cerebrum (disorder)
        "449020009",  # Intraparenchymal hemorrhage of brain (disorder)
        "450410005",  # Intracranial hemorrhage following injury (disorder)
        "450418003",  # Cerebral hemorrhage due to trauma (disorder)
        "451000119106",  # Closed injury of head (disorder)
        "451037005",  # Hemorrhage in globus pallidus (disorder)
        "451038000",  # Hemorrhage in caudate nucleus (disorder)
        "451039008",  # Hemorrhage in putamen (disorder)
        "4821001",  # Contusion of cheek (disorder)
        "49422009",  # Cortical hemorrhage (disorder)
        "52201006",  # Internal capsule hemorrhage (disorder)
        "52945003",  # Contusion of mandibular joint area (disorder)
        "60203004",  # Abrasion and/or friction burn of gum with infection (disorder)
        "60897004",  # Contusion of nose (disorder)
        "61383005",  # Abrasion and/or friction burn of ear with infection (disorder)
        "62603000",  # Abrasion and/or friction burn of scalp without infection (disorder)
        "63811003",  # Abrasion and/or friction burn of lip without infection (disorder)
        "64458007",  # Contusion of lip (disorder)
        "66176005",  # Abrasion and/or friction burn of face without infection (disorder)
        "66578000",  # Abrasion and/or friction burn of cheek with infection (disorder)
        "6733004",  # Abrasion and/or friction burn of ear without infection (disorder)
        "698616008",  # Loss of consciousness co-occurrent and due to closed fracture of vault of skull (disorder)
        "703156006",  # Deep hemispheric cerebral hemorrhage (disorder)
        "703174002",  # Subarachnoid hemorrhage from vertebral artery aneurysm (disorder)
        "703215002",  # Non-aneurysmal subarachnoid intracranial hemorrhage (disorder)
        "703216001",  # Perimesencephalic subarachnoid hemorrhage (disorder)
        "703217005",  # Convexal subarachnoid hemorrhage (disorder)
        "72986009",  # Acute hemorrhagic leukoencephalitis (disorder)
        "73020009",  # Cerebral hemisphere hemorrhage (disorder)
        "732923001",  # Hemorrhage of medulla oblongata (disorder)
        "75038005",  # Cerebellar hemorrhage (disorder)
        "75767000",  # Abrasion and/or friction burn of cheek without infection (disorder)
        "7713009",  # Intrapontine hemorrhage (disorder)
        "82271004",  # Injury of head (disorder)
        "874937002",  # Cerebral cortex laceration with concussion (disorder)
        "874939004",  # Cerebellar laceration with concussion (disorder)
        "874940002",  # Brain stem laceration with concussion (disorder)
        "874943000",  # Brain stem laceration with loss of consciousness (disorder)
        "95453001",  # Subdural intracranial hematoma (disorder)
        "95454007",  # Brain stem hemorrhage (disorder)
        "95817008",  # Labyrinthine concussion (disorder)
    }

class DementiaAndRelatedIntracranialPathologies(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for dementia.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes ICD-10CM and SNOMEDCT codes associated with dementia.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Dementia and Related Intracranial Pathologies"
    OID = "2.16.840.1.113883.3.3157.4043"
    DEFINITION_VERSION = "20190216"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "F0150",  # Vascular dementia, unspecified severity, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F01511",  # Vascular dementia, unspecified severity, with agitation
        "F01518",  # Vascular dementia, unspecified severity, with other behavioral disturbance
        "F0152",  # Vascular dementia, unspecified severity, with psychotic disturbance
        "F0153",  # Vascular dementia, unspecified severity, with mood disturbance
        "F0154",  # Vascular dementia, unspecified severity, with anxiety
        "F01A0",  # Vascular dementia, mild, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F01A11",  # Vascular dementia, mild, with agitation
        "F01A18",  # Vascular dementia, mild, with other behavioral disturbance
        "F01A2",  # Vascular dementia, mild, with psychotic disturbance
        "F01A3",  # Vascular dementia, mild, with mood disturbance
        "F01A4",  # Vascular dementia, mild, with anxiety
        "F01B0",  # Vascular dementia, moderate, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F01B11",  # Vascular dementia, moderate, with agitation
        "F01B18",  # Vascular dementia, moderate, with other behavioral disturbance
        "F01B2",  # Vascular dementia, moderate, with psychotic disturbance
        "F01B3",  # Vascular dementia, moderate, with mood disturbance
        "F01B4",  # Vascular dementia, moderate, with anxiety
        "F01C0",  # Vascular dementia, severe, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F01C11",  # Vascular dementia, severe, with agitation
        "F01C18",  # Vascular dementia, severe, with other behavioral disturbance
        "F01C2",  # Vascular dementia, severe, with psychotic disturbance
        "F01C3",  # Vascular dementia, severe, with mood disturbance
        "F01C4",  # Vascular dementia, severe, with anxiety
        "F0280",  # Dementia in other diseases classified elsewhere, unspecified severity, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F02811",  # Dementia in other diseases classified elsewhere, unspecified severity, with agitation
        "F02818",  # Dementia in other diseases classified elsewhere, unspecified severity, with other behavioral disturbance
        "F0282",  # Dementia in other diseases classified elsewhere, unspecified severity, with psychotic disturbance
        "F0283",  # Dementia in other diseases classified elsewhere, unspecified severity, with mood disturbance
        "F0284",  # Dementia in other diseases classified elsewhere, unspecified severity, with anxiety
        "F02A0",  # Dementia in other diseases classified elsewhere, mild, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F02A11",  # Dementia in other diseases classified elsewhere, mild, with agitation
        "F02A18",  # Dementia in other diseases classified elsewhere, mild, with other behavioral disturbance
        "F02A2",  # Dementia in other diseases classified elsewhere, mild, with psychotic disturbance
        "F02A3",  # Dementia in other diseases classified elsewhere, mild, with mood disturbance
        "F02A4",  # Dementia in other diseases classified elsewhere, mild, with anxiety
        "F02B0",  # Dementia in other diseases classified elsewhere, moderate, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F02B11",  # Dementia in other diseases classified elsewhere, moderate, with agitation
        "F02B18",  # Dementia in other diseases classified elsewhere, moderate, with other behavioral disturbance
        "F02B2",  # Dementia in other diseases classified elsewhere, moderate, with psychotic disturbance
        "F02B3",  # Dementia in other diseases classified elsewhere, moderate, with mood disturbance
        "F02B4",  # Dementia in other diseases classified elsewhere, moderate, with anxiety
        "F02C0",  # Dementia in other diseases classified elsewhere, severe, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F02C11",  # Dementia in other diseases classified elsewhere, severe, with agitation
        "F02C18",  # Dementia in other diseases classified elsewhere, severe, with other behavioral disturbance
        "F02C2",  # Dementia in other diseases classified elsewhere, severe, with psychotic disturbance
        "F02C3",  # Dementia in other diseases classified elsewhere, severe, with mood disturbance
        "F02C4",  # Dementia in other diseases classified elsewhere, severe, with anxiety
        "F0390",  # Unspecified dementia, unspecified severity, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F03911",  # Unspecified dementia, unspecified severity, with agitation
        "F03918",  # Unspecified dementia, unspecified severity, with other behavioral disturbance
        "F0392",  # Unspecified dementia, unspecified severity, with psychotic disturbance
        "F0393",  # Unspecified dementia, unspecified severity, with mood disturbance
        "F0394",  # Unspecified dementia, unspecified severity, with anxiety
        "F03A0",  # Unspecified dementia, mild, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F03A11",  # Unspecified dementia, mild, with agitation
        "F03A18",  # Unspecified dementia, mild, with other behavioral disturbance
        "F03A2",  # Unspecified dementia, mild, with psychotic disturbance
        "F03A3",  # Unspecified dementia, mild, with mood disturbance
        "F03A4",  # Unspecified dementia, mild, with anxiety
        "F03B0",  # Unspecified dementia, moderate, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F03B11",  # Unspecified dementia, moderate, with agitation
        "F03B18",  # Unspecified dementia, moderate, with other behavioral disturbance
        "F03B2",  # Unspecified dementia, moderate, with psychotic disturbance
        "F03B3",  # Unspecified dementia, moderate, with mood disturbance
        "F03B4",  # Unspecified dementia, moderate, with anxiety
        "F03C0",  # Unspecified dementia, severe, without behavioral disturbance, psychotic disturbance, mood disturbance, and anxiety
        "F03C11",  # Unspecified dementia, severe, with agitation
        "F03C18",  # Unspecified dementia, severe, with other behavioral disturbance
        "F03C2",  # Unspecified dementia, severe, with psychotic disturbance
        "F03C3",  # Unspecified dementia, severe, with mood disturbance
        "F03C4",  # Unspecified dementia, severe, with anxiety
        "G300",  # Alzheimer's disease with early onset
        "G301",  # Alzheimer's disease with late onset
        "G308",  # Other Alzheimer's disease
        "G309",  # Alzheimer's disease, unspecified
        "G3109",  # Other frontotemporal neurocognitive disorder
        "G3183",  # Neurocognitive disorder with Lewy bodies
    }

    SNOMEDCT = {
        "101421000119107",  # Dementia due to Parkinson's disease (disorder)
        "10171000132106",  # Behavioral and psychological symptoms of dementia (finding)
        "10349009",  # Multi-infarct dementia with delirium (disorder)
        "10532003",  # Primary degenerative dementia of the Alzheimer type, presenile onset, with depression (disorder)
        "106021000119105",  # Multi-infarct dementia due to atherosclerosis (disorder)
        "12348006",  # Presenile dementia (disorder)
        "1259485008",  # Dementia due to cerebral vasculitis (disorder)
        "1259501004",  # Dementia due to kuru (disorder)
        "1259586006",  # Dementia due to autoimmune encephalitis (disorder)
        "1339031006",  # Dementia caused by ionizing radiation (disorder)
        "14070001",  # Multi-infarct dementia with depression (disorder)
        "142001000119106",  # Depressed mood in Alzheimer's disease (disorder)
        "15662003",  # Senile dementia (disorder)
        "1581000119101",  # Dementia of the Alzheimer type with behavioral disturbance (disorder)
        "1591000119103",  # Dementia with behavioral disturbance (disorder)
        "16219201000119101",  # Behavioral disturbance co-occurrent and due to late onset Alzheimer dementia (disorder)
        "16276361000119109",  # Vascular dementia without behavioral disturbance (disorder)
        "191449005",  # Uncomplicated senile dementia (disorder)
        "191451009",  # Uncomplicated presenile dementia (disorder)
        "191452002",  # Presenile dementia with delirium (disorder)
        "191454001",  # Presenile dementia with paranoia (disorder)
        "191455000",  # Presenile dementia with depression (disorder)
        "191457008",  # Senile dementia with depressive or paranoid features (disorder)
        "191458003",  # Senile dementia with paranoia (disorder)
        "191459006",  # Senile dementia with depression (disorder)
        "191461002",  # Senile dementia with delirium (disorder)
        "191463004",  # Uncomplicated arteriosclerotic dementia (disorder)
        "191464005",  # Arteriosclerotic dementia with delirium (disorder)
        "191465006",  # Arteriosclerotic dementia with paranoia (disorder)
        "191466007",  # Arteriosclerotic dementia with depression (disorder)
        "191519005",  # Dementia associated with another disease (disorder)
        "21921000119103",  # Dementia co-occurrent and due to Pick's disease (disorder)
        "22381000119105",  # Primary degenerative dementia (disorder)
        "230258005",  # Amyotrophic lateral sclerosis with dementia (disorder)
        "230270009",  # Frontotemporal dementia (disorder)
        "230282000",  # Post-traumatic dementia (disorder)
        "230285003",  # Vascular dementia of acute onset (disorder)
        "230286002",  # Subcortical vascular dementia (disorder)
        "230287006",  # Mixed cortical and subcortical vascular dementia (disorder)
        "230288001",  # Semantic dementia (disorder)
        "230289009",  # Patchy dementia (disorder)
        "2421000119107",  # Hallucinations co-occurrent and due to late onset dementia (disorder)
        "25772007",  # Multi-infarct dementia with delusions (disorder)
        "26852004",  # Primary degenerative dementia of the Alzheimer type, senile onset, with depression (disorder)
        "26929004",  # Alzheimer's disease (disorder)
        "278857002",  # Dementia of frontal lobe type (disorder)
        "279982005",  # Cerebral degeneration presenting primarily with dementia (disorder)
        "281004",  # Dementia associated with alcoholism (disorder)
        "288631000119104",  # Vascular dementia with behavioral disturbance (disorder)
        "31081000119101",  # Presenile dementia with delusions (disorder)
        "312991009",  # Senile dementia of the Lewy body type (disorder)
        "32875003",  # Inhalant-induced persisting dementia (disorder)
        "371024007",  # Senile dementia with delusion (disorder)
        "371026009",  # Senile dementia with psychosis (disorder)
        "416780008",  # Primary degenerative dementia of the Alzheimer type, presenile onset (disorder)
        "416975007",  # Primary degenerative dementia of the Alzheimer type, senile onset (disorder)
        "420614009",  # Organic dementia with acquired immunodeficiency syndrome (disorder)
        "421023003",  # Presenile dementia with acquired immunodeficiency syndrome (disorder)
        "421529006",  # Dementia with acquired immunodeficiency syndrome (disorder)
        "425390006",  # Dementia associated with Parkinson's Disease (disorder)
        "428051000124108",  # Mild dementia (disorder)
        "428351000124105",  # Severe dementia (disorder)
        "429161000124103",  # Primary degenerative dementia of the Alzheimer type, senile onset, with behavioral disturbance (disorder)
        "429458009",  # Dementia due to Creutzfeldt Jakob disease (disorder)
        "429998004",  # Vascular dementia (disorder)
        "430771000124100",  # Moderate dementia (disorder)
        "442344002",  # Dementia due to Huntington chorea (disorder)
        "4817008",  # Primary degenerative dementia of the Alzheimer type, senile onset, with delirium (disorder)
        "52448006",  # Dementia (disorder)
        "54502004",  # Primary degenerative dementia of the Alzheimer type, presenile onset, with delusions (disorder)
        "55009008",  # Primary degenerative dementia of the Alzheimer type, senile onset, with delusions (disorder)
        "56267009",  # Multi-infarct dementia (disorder)
        "59651006",  # Sedative, hypnotic AND/OR anxiolytic-induced persisting dementia (disorder)
        "6475002",  # Primary degenerative dementia of the Alzheimer type, presenile onset, uncomplicated (disorder)
        "65096006",  # Primary degenerative dementia of the Alzheimer type, presenile onset, with delirium (disorder)
        "66108005",  # Primary degenerative dementia of the Alzheimer type, senile onset, uncomplicated (disorder)
        "698624003",  # Dementia associated with cerebral lipidosis (disorder)
        "698625002",  # Dementia associated with normal pressure hydrocephalus (disorder)
        "698626001",  # Dementia associated with multiple sclerosis (disorder)
        "698687007",  # Post-traumatic dementia with behavioral change (disorder)
        "698725008",  # Dementia associated with neurosyphilis (disorder)
        "698726009",  # Dementia associated with viral encephalitis (disorder)
        "698781002",  # Dementia associated with cerebral anoxia (disorder)
        "698948009",  # Vascular dementia in remission (disorder)
        "698949001",  # Dementia in remission (disorder)
        "698954005",  # Primary degenerative dementia of the Alzheimer type, senile onset in remission (disorder)
        "698955006",  # Primary degenerative dementia of the Alzheimer type, presenile onset in remission (disorder)
        "702347001",  # Polycystic lipomembranous osteodysplasia with sclerosing leukoencephalopathy (disorder)
        "702393003",  # Frontotemporal dementia with gene located on 3p11 (disorder)
        "702426001",  # GRN-related frontotemporal dementia (disorder)
        "702429008",  # Frontotemporal dementia with parkinsonism-17 (disorder)
        "703544004",  # Inclusion body myopathy with early-onset Paget disease and frontotemporal dementia (disorder)
        "70936005",  # Multi-infarct dementia, uncomplicated (disorder)
        "713488003",  # Presenile dementia co-occurrent with human immunodeficiency virus infection (disorder)
        "713844000",  # Dementia co-occurrent with human immunodeficiency virus infection (disorder)
        "716667005",  # Right temporal atrophy variant frontotemporal dementia (disorder)
        "716994006",  # Behavioral variant of frontotemporal dementia (disorder)
        "722977005",  # Dementia co-occurrent and due to neurocysticercosis (disorder)
        "722978000",  # Toxic dementia (disorder)
        "722979008",  # Dementia due to metabolic abnormality (disorder)
        "722980006",  # Dementia due to chromosomal anomaly (disorder)
        "723123001",  # Ischemic vascular dementia (disorder)
        "723390000",  # Rapidly progressive dementia (disorder)
        "724776007",  # Dementia due to disorder of central nervous system (disorder)
        "725898002",  # Delirium co-occurrent with dementia (disorder)
        "733190003",  # Dementia due to primary malignant neoplasm of brain (disorder)
        "733191004",  # Dementia due to chronic subdural hematoma (disorder)
        "733192006",  # Dementia due to herpes encephalitis (disorder)
        "733193001",  # Dementia co-occurrent and due to progressive multifocal leukoencephalopathy (disorder)
        "733194007",  # Dementia co-occurrent and due to Down syndrome (disorder)
        "762350007",  # Dementia due to prion disease (disorder)
        "762351006",  # Dementia due to and following injury of head (disorder)
        "762707000",  # Subcortical dementia (disorder)
        "788898005",  # Dementia caused by volatile inhalant (disorder)
        "792004",  # Jakob-Creutzfeldt disease (disorder)
        "79341000119107",  # Mixed dementia (disorder)
        "80098002",  # Diffuse Lewy body disease (disorder)
        "82361000119107",  # Altered behavior in dementia due to Huntington chorea (disorder)
        "82371000119101",  # Dementia due to multiple sclerosis with altered behavior (disorder)
        "82381000119103",  # Epileptic dementia with behavioral disturbance (disorder)
        "833326008",  # Cortical vascular dementia (disorder)
    }

class IschemicStroke(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diagnoses of ischemic strokes.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent a diagnosis of ischemic stroke.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Ischemic Stroke"
    OID = "2.16.840.1.113883.3.117.1.7.1.247"
    DEFINITION_VERSION = "20210220"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "I6300",  # Cerebral infarction due to thrombosis of unspecified precerebral artery
        "I63011",  # Cerebral infarction due to thrombosis of right vertebral artery
        "I63012",  # Cerebral infarction due to thrombosis of left vertebral artery
        "I63013",  # Cerebral infarction due to thrombosis of bilateral vertebral arteries
        "I63019",  # Cerebral infarction due to thrombosis of unspecified vertebral artery
        "I6302",  # Cerebral infarction due to thrombosis of basilar artery
        "I63031",  # Cerebral infarction due to thrombosis of right carotid artery
        "I63032",  # Cerebral infarction due to thrombosis of left carotid artery
        "I63033",  # Cerebral infarction due to thrombosis of bilateral carotid arteries
        "I63039",  # Cerebral infarction due to thrombosis of unspecified carotid artery
        "I6309",  # Cerebral infarction due to thrombosis of other precerebral artery
        "I6310",  # Cerebral infarction due to embolism of unspecified precerebral artery
        "I63111",  # Cerebral infarction due to embolism of right vertebral artery
        "I63112",  # Cerebral infarction due to embolism of left vertebral artery
        "I63113",  # Cerebral infarction due to embolism of bilateral vertebral arteries
        "I63119",  # Cerebral infarction due to embolism of unspecified vertebral artery
        "I6312",  # Cerebral infarction due to embolism of basilar artery
        "I63131",  # Cerebral infarction due to embolism of right carotid artery
        "I63132",  # Cerebral infarction due to embolism of left carotid artery
        "I63133",  # Cerebral infarction due to embolism of bilateral carotid arteries
        "I63139",  # Cerebral infarction due to embolism of unspecified carotid artery
        "I6319",  # Cerebral infarction due to embolism of other precerebral artery
        "I6320",  # Cerebral infarction due to unspecified occlusion or stenosis of unspecified precerebral arteries
        "I63211",  # Cerebral infarction due to unspecified occlusion or stenosis of right vertebral artery
        "I63212",  # Cerebral infarction due to unspecified occlusion or stenosis of left vertebral artery
        "I63213",  # Cerebral infarction due to unspecified occlusion or stenosis of bilateral vertebral arteries
        "I63219",  # Cerebral infarction due to unspecified occlusion or stenosis of unspecified vertebral artery
        "I6322",  # Cerebral infarction due to unspecified occlusion or stenosis of basilar artery
        "I63231",  # Cerebral infarction due to unspecified occlusion or stenosis of right carotid arteries
        "I63232",  # Cerebral infarction due to unspecified occlusion or stenosis of left carotid arteries
        "I63233",  # Cerebral infarction due to unspecified occlusion or stenosis of bilateral carotid arteries
        "I63239",  # Cerebral infarction due to unspecified occlusion or stenosis of unspecified carotid artery
        "I6329",  # Cerebral infarction due to unspecified occlusion or stenosis of other precerebral arteries
        "I6330",  # Cerebral infarction due to thrombosis of unspecified cerebral artery
        "I63311",  # Cerebral infarction due to thrombosis of right middle cerebral artery
        "I63312",  # Cerebral infarction due to thrombosis of left middle cerebral artery
        "I63313",  # Cerebral infarction due to thrombosis of bilateral middle cerebral arteries
        "I63319",  # Cerebral infarction due to thrombosis of unspecified middle cerebral artery
        "I63321",  # Cerebral infarction due to thrombosis of right anterior cerebral artery
        "I63322",  # Cerebral infarction due to thrombosis of left anterior cerebral artery
        "I63323",  # Cerebral infarction due to thrombosis of bilateral anterior cerebral arteries
        "I63329",  # Cerebral infarction due to thrombosis of unspecified anterior cerebral artery
        "I63331",  # Cerebral infarction due to thrombosis of right posterior cerebral artery
        "I63332",  # Cerebral infarction due to thrombosis of left posterior cerebral artery
        "I63333",  # Cerebral infarction due to thrombosis of bilateral posterior cerebral arteries
        "I63339",  # Cerebral infarction due to thrombosis of unspecified posterior cerebral artery
        "I63341",  # Cerebral infarction due to thrombosis of right cerebellar artery
        "I63342",  # Cerebral infarction due to thrombosis of left cerebellar artery
        "I63343",  # Cerebral infarction due to thrombosis of bilateral cerebellar arteries
        "I63349",  # Cerebral infarction due to thrombosis of unspecified cerebellar artery
        "I6339",  # Cerebral infarction due to thrombosis of other cerebral artery
        "I6340",  # Cerebral infarction due to embolism of unspecified cerebral artery
        "I63411",  # Cerebral infarction due to embolism of right middle cerebral artery
        "I63412",  # Cerebral infarction due to embolism of left middle cerebral artery
        "I63413",  # Cerebral infarction due to embolism of bilateral middle cerebral arteries
        "I63419",  # Cerebral infarction due to embolism of unspecified middle cerebral artery
        "I63421",  # Cerebral infarction due to embolism of right anterior cerebral artery
        "I63422",  # Cerebral infarction due to embolism of left anterior cerebral artery
        "I63423",  # Cerebral infarction due to embolism of bilateral anterior cerebral arteries
        "I63429",  # Cerebral infarction due to embolism of unspecified anterior cerebral artery
        "I63431",  # Cerebral infarction due to embolism of right posterior cerebral artery
        "I63432",  # Cerebral infarction due to embolism of left posterior cerebral artery
        "I63433",  # Cerebral infarction due to embolism of bilateral posterior cerebral arteries
        "I63439",  # Cerebral infarction due to embolism of unspecified posterior cerebral artery
        "I63441",  # Cerebral infarction due to embolism of right cerebellar artery
        "I63442",  # Cerebral infarction due to embolism of left cerebellar artery
        "I63443",  # Cerebral infarction due to embolism of bilateral cerebellar arteries
        "I63449",  # Cerebral infarction due to embolism of unspecified cerebellar artery
        "I6349",  # Cerebral infarction due to embolism of other cerebral artery
        "I6350",  # Cerebral infarction due to unspecified occlusion or stenosis of unspecified cerebral artery
        "I63511",  # Cerebral infarction due to unspecified occlusion or stenosis of right middle cerebral artery
        "I63512",  # Cerebral infarction due to unspecified occlusion or stenosis of left middle cerebral artery
        "I63513",  # Cerebral infarction due to unspecified occlusion or stenosis of bilateral middle cerebral arteries
        "I63519",  # Cerebral infarction due to unspecified occlusion or stenosis of unspecified middle cerebral artery
        "I63521",  # Cerebral infarction due to unspecified occlusion or stenosis of right anterior cerebral artery
        "I63522",  # Cerebral infarction due to unspecified occlusion or stenosis of left anterior cerebral artery
        "I63523",  # Cerebral infarction due to unspecified occlusion or stenosis of bilateral anterior cerebral arteries
        "I63529",  # Cerebral infarction due to unspecified occlusion or stenosis of unspecified anterior cerebral artery
        "I63531",  # Cerebral infarction due to unspecified occlusion or stenosis of right posterior cerebral artery
        "I63532",  # Cerebral infarction due to unspecified occlusion or stenosis of left posterior cerebral artery
        "I63533",  # Cerebral infarction due to unspecified occlusion or stenosis of bilateral posterior cerebral arteries
        "I63539",  # Cerebral infarction due to unspecified occlusion or stenosis of unspecified posterior cerebral artery
        "I63541",  # Cerebral infarction due to unspecified occlusion or stenosis of right cerebellar artery
        "I63542",  # Cerebral infarction due to unspecified occlusion or stenosis of left cerebellar artery
        "I63543",  # Cerebral infarction due to unspecified occlusion or stenosis of bilateral cerebellar arteries
        "I63549",  # Cerebral infarction due to unspecified occlusion or stenosis of unspecified cerebellar artery
        "I6359",  # Cerebral infarction due to unspecified occlusion or stenosis of other cerebral artery
        "I636",  # Cerebral infarction due to cerebral venous thrombosis, nonpyogenic
        "I6381",  # Other cerebral infarction due to occlusion or stenosis of small artery
        "I6389",  # Other cerebral infarction
        "I639",  # Cerebral infarction, unspecified
    }

    SNOMEDCT = {
        "111297002",  # Nonparalytic stroke (disorder)
        "116288000",  # Paralytic stroke (disorder)
        "195185009",  # Cerebral infarct due to thrombosis of precerebral arteries (disorder)
        "195186005",  # Cerebral infarction due to embolism of precerebral arteries (disorder)
        "195189003",  # Cerebral infarction due to thrombosis of cerebral arteries (disorder)
        "195190007",  # Cerebral infarction due to embolism of cerebral arteries (disorder)
        "195212005",  # Brainstem stroke syndrome (disorder)
        "195213000",  # Cerebellar stroke syndrome (disorder)
        "195216008",  # Left sided cerebral hemisphere cerebrovascular accident (disorder)
        "195217004",  # Right sided cerebral hemisphere cerebrovascular accident (disorder)
        "195230003",  # Cerebral infarction due to cerebral venous thrombosis, non-pyogenic (disorder)
        "230690007",  # Cerebrovascular accident (disorder)
        "230691006",  # Cerebrovascular accident due to occlusion of cerebral artery (disorder)
        "230692004",  # Infarction - precerebral (disorder)
        "230693009",  # Anterior cerebral circulation infarction (disorder)
        "230694003",  # Total anterior cerebral circulation infarction (disorder)
        "230695002",  # Partial anterior cerebral circulation infarction (disorder)
        "230696001",  # Posterior cerebral circulation infarction (disorder)
        "230698000",  # Lacunar infarction (disorder)
        "230699008",  # Pure motor hemiparesis due to and following lacunar infarction (disorder)
        "230700009",  # Pure sensory stroke due to and following lacunar infarction (disorder)
        "230701008",  # Pure sensorimotor motor hemiparesis due to and following lacunar infarction (disorder)
        "307363008",  # Multiple lacunar infarcts (disorder)
        "422504002",  # Ischemic stroke (disorder)
        "426107000",  # Acute lacunar infarction (disorder)
    }

class MalignantIntracranialNeoplasmGroup(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for malignant intracranial neoplasm.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes ICD-10 and SNOMEDCT codes that represent malignant intracranial neoplasm diagnoses.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Malignant Intracranial Neoplasm Group"
    OID = "2.16.840.1.113762.1.4.1170.3"
    DEFINITION_VERSION = "20211005"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "C700",  # Malignant neoplasm of cerebral meninges
        "C709",  # Malignant neoplasm of meninges, unspecified
        "C710",  # Malignant neoplasm of cerebrum, except lobes and ventricles
        "C711",  # Malignant neoplasm of frontal lobe
        "C712",  # Malignant neoplasm of temporal lobe
        "C713",  # Malignant neoplasm of parietal lobe
        "C714",  # Malignant neoplasm of occipital lobe
        "C715",  # Malignant neoplasm of cerebral ventricle
        "C716",  # Malignant neoplasm of cerebellum
        "C717",  # Malignant neoplasm of brain stem
        "C718",  # Malignant neoplasm of overlapping sites of brain
        "C719",  # Malignant neoplasm of brain, unspecified
        "C7250",  # Malignant neoplasm of unspecified cranial nerve
        "C7259",  # Malignant neoplasm of other cranial nerves
        "C729",  # Malignant neoplasm of central nervous system, unspecified
        "C7931",  # Secondary malignant neoplasm of brain
        "D430",  # Neoplasm of uncertain behavior of brain, supratentorial
        "D431",  # Neoplasm of uncertain behavior of brain, infratentorial
        "D432",  # Neoplasm of uncertain behavior of brain, unspecified
        "D496",  # Neoplasm of unspecified behavior of brain
        "H47631",  # Disorders of visual cortex in (due to) neoplasm, right side of brain
        "H47632",  # Disorders of visual cortex in (due to) neoplasm, left side of brain
        "H47639",  # Disorders of visual cortex in (due to) neoplasm, unspecified side of brain
        "Z85841",  # Personal history of malignant neoplasm of brain
    }

    SNOMEDCT = {
        "109911004",  # Overlapping malignant neoplasm of brain and other parts of the central nervous system (disorder)
        "109912006",  # Overlapping malignant neoplasm of brain (disorder)
        "126952004",  # Neoplasm of brain (disorder)
        "126961004",  # Neoplasm of brain stem (disorder)
        "188462001",  # Metastatic malignant neoplasm to brain and spinal cord (disorder)
        "189484008",  # Neoplasm of uncertain behavior of brain and/or spinal cord (disorder)
        "189487001",  # Neoplasm of uncertain or unknown behavior of brain, supratentorial (disorder)
        "189488006",  # Neoplasm of uncertain or unknown behavior of brain, infratentorial (disorder)
        "254935002",  # Intracranial tumor (disorder)
        "363469001",  # Malignant neoplasm of parietal lobe (disorder)
        "428061005",  # Malignant neoplasm of brain (disorder)
        "448248006",  # Malignant neoplasm of axial suprasellar region of brain (disorder)
        "448989001",  # Malignant epithelial neoplasm of brain (disorder)
        "93726004",  # Primary malignant neoplasm of brain stem (disorder)
        "93727008",  # Primary malignant neoplasm of brain (disorder)
        "94224009",  # Metastatic malignant neoplasm to brain stem (disorder)
        "94225005",  # Metastatic malignant neoplasm to brain (disorder)
        "94766006",  # Neoplasm of uncertain behavior of brain stem (disorder)
        "94767002",  # Neoplasm of uncertain behavior of brain (disorder)
    }

class NeurologicImpairment(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for neurologic impairment.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes concepts that represent diagnoses related to neurologic impairment.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Neurologic impairment"
    OID = "2.16.840.1.113883.3.464.1003.114.12.1012"
    DEFINITION_VERSION = "20240217"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "G834",  # Cauda equina syndrome
        "K592",  # Neurogenic bowel, not elsewhere classified
        "M48062",  # Spinal stenosis, lumbar region with neurogenic claudication
        "R262",  # Difficulty in walking, not elsewhere classified
        "R292",  # Abnormal reflex
    }

    SNOMEDCT = {
        "111633007",  # Open fracture of sacrum AND/OR coccyx with complete cauda equina lesion (disorder)
        "123534001",  # Cauda equina injury co-occurrent and due to closed fracture of sacrum AND/OR coccyx (disorder)
        "123535000",  # Cauda equina injury co-occurrent and due to open fracture of sacrum AND/OR coccyx (disorder)
        "12454008",  # Cauda equina syndrome with neurogenic urinary bladder (disorder)
        "126963001",  # Neoplasm of cauda equina (disorder)
        "16650241000119101",  # Lesion of left lumbosacral nerve root (disorder)
        "16650341000119107",  # Lesion of right lumbosacral nerve root (disorder)
        "16846261000119106",  # Compression of cauda equina (disorder)
        "192970008",  # Cauda equina syndrome (disorder)
        "192971007",  # Cauda equina syndrome not affecting bladder (disorder)
        "208060001",  # Cauda equina lesion co-occurrent and due to closed spinal fracture (disorder)
        "208067003",  # Cauda equina lesion co-occurrent and due to open spinal fracture (disorder)
        "208071000",  # Complete cauda equina lesion co-occurrent and due to closed fracture of sacrum (disorder)
        "208078006",  # Complete cauda equina lesion co-occurrent and due to open fracture of sacrum (disorder)
        "209086000",  # Closed spinal dislocation with cauda equina lesion (disorder)
        "209102002",  # Open spinal dislocation with cauda equina lesion (disorder)
        "209182000",  # Closed spinal subluxation with cauda equina lesion (disorder)
        "209197002",  # Open spinal subluxation with cauda equina lesion (disorder)
        "212192004",  # Closed injury cauda equina (disorder)
        "212193009",  # Open injury cauda equina (disorder)
        "212218009",  # Lumbar nerve root injury - L1 (disorder)
        "212219001",  # Lumbar nerve root injury - L2 (disorder)
        "212220007",  # Lumbar nerve root injury - L3 (disorder)
        "212221006",  # Lumbar nerve root injury - L4 (disorder)
        "212222004",  # Lumbar nerve root injury - L5 (disorder)
        "212223009",  # Sacral nerve root injury - S1 (disorder)
        "212224003",  # Sacral nerve root injury - S2 (disorder)
        "212225002",  # Sacral nerve root injury - S3 (disorder)
        "212226001",  # Sacral nerve root injury - S4 (disorder)
        "212227005",  # Sacral nerve root injury - S5 (disorder)
        "230614002",  # Injury of cauda equina (disorder)
        "262736005",  # Contusion of lumbar nerve root (disorder)
        "262737001",  # Contusion of sacral nerve root (disorder)
        "262742009",  # Traction injury of lumbar nerve root (disorder)
        "262743004",  # Traction injury of sacral nerve root (disorder)
        "262747003",  # Avulsion of lumbar nerve root (disorder)
        "262748008",  # Avulsion of sacral nerve root (disorder)
        "267590006",  # Cauda equina syndrome with cord bladder (disorder)
        "299966003",  # Compression of lumbar nerve root (disorder)
        "299967007",  # Compression of sacral nerve root (disorder)
        "312792007",  # Spinal subluxation with cauda equina lesion (disorder)
        "363477002",  # Malignant neoplasm of cauda equina (disorder)
        "405753002",  # Cauda equina injury without spinal bone injury (disorder)
        "41455002",  # Complete cauda equina lesion co-occurrent and due to closed fracture of sacrum AND/OR coccyx (disorder)
        "449813001",  # Numbness of saddle area (finding)
        "764272007",  # Compression of cauda equina due to lumbar vertebral stenosis (disorder)
        "782388003",  # Incomplete cauda equina syndrome (disorder)
        "782390002",  # Painless urinary retention due to cauda equina syndrome (disorder)
        "860614004",  # Injury of lumbar nerve root (disorder)
        "8663007",  # Cauda equina syndrome without neurogenic bladder (disorder)
        "87900003",  # Injury of sacral nerve roots (disorder)
        "92047003",  # Benign neoplasm of cauda equina (disorder)
        "93743001",  # Primary malignant neoplasm of cauda equina (disorder)
        "94242004",  # Metastatic malignant neoplasm to cauda equina (disorder)
        "94782007",  # Neoplasm of uncertain behavior of cauda equina (disorder)
    }

class PregnantState(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for pregnant state.

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes ICD-10CM and SNOMEDCT codes associated with the state of pregnancy.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Pregnant State"
    OID = "2.16.840.1.113883.3.3157.4055"
    DEFINITION_VERSION = "20190216"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "Z331",  # Pregnant state, incidental
        "Z333",  # Pregnant state, gestational carrier
    }

    SNOMEDCT = {
        "255409004",  # Pregnant woman (person)
        "77386006",  # Pregnancy (finding)
    }

class Stemi(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for ST-segment elevation myocardial infarction (STEMI).

    **Data Element Scope:** This value set may use a model element related to Diagnosis.

    **Inclusion Criteria:** Includes ICD-10 and SNOMEDCT codes associated with the diagnosis of STEMI.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "STEMI"
    OID = "2.16.840.1.113883.3.3157.4017"
    DEFINITION_VERSION = "20190216"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "I2101",  # ST elevation (STEMI) myocardial infarction involving left main coronary artery
        "I2102",  # ST elevation (STEMI) myocardial infarction involving left anterior descending coronary artery
        "I2109",  # ST elevation (STEMI) myocardial infarction involving other coronary artery of anterior wall
        "I2111",  # ST elevation (STEMI) myocardial infarction involving right coronary artery
        "I2119",  # ST elevation (STEMI) myocardial infarction involving other coronary artery of inferior wall
        "I2121",  # ST elevation (STEMI) myocardial infarction involving left circumflex coronary artery
        "I2129",  # ST elevation (STEMI) myocardial infarction involving other sites
        "I213",  # ST elevation (STEMI) myocardial infarction of unspecified site
        "I220",  # Subsequent ST elevation (STEMI) myocardial infarction of anterior wall
        "I221",  # Subsequent ST elevation (STEMI) myocardial infarction of inferior wall
        "I228",  # Subsequent ST elevation (STEMI) myocardial infarction of other sites
        "I229",  # Subsequent ST elevation (STEMI) myocardial infarction of unspecified site
    }

    SNOMEDCT = {
        "401303003",  # Acute ST segment elevation myocardial infarction (disorder)
        "703164000",  # Acute ST segment elevation myocardial infarction of anterior wall (disorder)
        "703165004",  # Acute ST segment elevation myocardial infarction of anterior wall involving right ventricle (disorder)
        "703209002",  # Subsequent ST segment elevation myocardial infarction of inferior wall (disorder)
        "703210007",  # Subsequent ST segment elevation myocardial infarction of anterior wall (disorder)
        "703211006",  # Subsequent ST segment elevation myocardial infarction (disorder)
        "703213009",  # Acute ST segment elevation myocardial infarction of inferior wall (disorder)
        "703253007",  # Acute ST segment elevation myocardial infarction of inferior wall involving right ventricle (disorder)
    }

__exports__ = (
    "AdvancedIllness",
    "Diabetes",
    "DiabeticRetinopathy",
    "FrailtyDiagnosis",
    "HospiceDiagnosis",
    "PalliativeCareDiagnosis",
    "AtrialFibrillationOrFlutter",
    "VenousThromboembolism",
    "LimitedLifeExpectancy",
    "PrediabetesBorderlineDiabetes",
    "Pregnancy",
    "Hiv",
    "AnaphylacticReactionToDtapVaccine",
    "DisordersOfTheImmuneSystem",
    "EncephalitisDueToDiphtheriaTetanusOrPertussisVaccine",
    "HepatitisA",
    "HepatitisB",
    "Intussusception",
    "MalignantNeoplasmOfLymphaticAndHematopoieticTissue",
    "Measles",
    "Mumps",
    "Rubella",
    "SevereCombinedImmunodeficiency",
    "VaricellaZoster",
    "CongenitalOrAcquiredAbsenceOfCervix",
    "HistoryOfBilateralMastectomy",
    "StatusPostLeftMastectomy",
    "StatusPostRightMastectomy",
    "UnilateralMastectomyUnspecifiedLaterality",
    "MajorDepression",
    "PainWarrantingFurtherInvestigationForProstateCancer",
    "ProstateCancer",
    "MalignantNeoplasmOfColon",
    "AcuteAndSubacuteIridocyclitis",
    "Amblyopia",
    "BurnConfinedToEyeAndAdnexa",
    "CataractCongenital",
    "CataractMatureOrHypermature",
    "CataractPosteriorPolar",
    "CataractSecondaryToOcularDisorders",
    "CentralCornealUlcer",
    "CertainTypesOfIridocyclitis",
    "ChoroidalDegenerations",
    "ChoroidalDetachment",
    "ChoroidalHemorrhageAndRupture",
    "ChronicIridocyclitis",
    "CloudyCornea",
    "CornealEdema",
    "DegenerationOfMaculaAndPosteriorPole",
    "DegenerativeDisordersOfGlobe",
    "DiabeticMacularEdema",
    "DisordersOfCorneaIncludingCornealOpacity",
    "DisordersOfOpticChiasm",
    "DisordersOfVisualCortex",
    "DisseminatedChorioretinitisAndDisseminatedRetinochoroiditis",
    "FocalChorioretinitisAndFocalRetinochoroiditis",
    "Glaucoma",
    "GlaucomaAssociatedWithCongenitalAnomaliesAndDystrophiesAndSystemicSyndromes",
    "HereditaryChoroidalDystrophies",
    "HereditaryCornealDystrophies",
    "HereditaryRetinalDystrophies",
    "HypotonyOfEye",
    "InjuryToOpticNerveAndPathways",
    "MacularScarOfPosteriorPolar",
    "MorgagnianCataract",
    "NystagmusAndOtherIrregularEyeMovements",
    "OpenWoundOfEyeball",
    "OpticAtrophy",
    "OpticNeuritis",
    "OtherBackgroundRetinopathyAndRetinalVascularChanges",
    "OtherDisordersOfOpticNerve",
    "OtherEndophthalmitis",
    "OtherProliferativeRetinopathy",
    "OtherAndUnspecifiedFormsOfChorioretinitisAndRetinochoroiditis",
    "PathologicMyopia",
    "PosteriorLenticonus",
    "PriorPenetratingKeratoplasty",
    "PurulentEndophthalmitis",
    "RetinalDetachmentWithRetinalDefect",
    "RetinalVascularOcclusion",
    "RetrolentalFibroplasias",
    "Scleritis",
    "SeparationOfRetinalLayers",
    "TraumaticCataract",
    "Uveitis",
    "VascularDisordersOfIrisAndCiliaryBody",
    "VisualFieldDefects",
    "AllergyToAceInhibitorOrArb",
    "HeartFailure",
    "HeartTransplantComplications",
    "IntoleranceToAceInhibitorOrArb",
    "LeftVentricularAssistDeviceComplications",
    "ModerateOrSevereLvsd",
    "Narcolepsy",
    "SubstanceUseDisorder",
    "PrimaryOpenAngleGlaucoma",
    "AllergyToBetaBlockerTherapy",
    "Arrhythmia",
    "Asthma",
    "AtrioventricularBlock",
    "Bradycardia",
    "CardiacPacerInSitu",
    "Hypotension",
    "IntoleranceToBetaBlockerTherapy",
    "CoronaryArteryDiseaseNoMi",
    "InitialMyocardialInfarction",
    "AcutePharyngitis",
    "AcuteTonsillitis",
    "ComorbidConditionsForRespiratoryConditions",
    "CompetingConditionsForRespiratoryConditions",
    "DementiaAndMentalDegenerations",
    "ComplicationsOfPregnancyChildbirthAndThePuerperium",
    "DiagnosesUsedToIndicateSexualActivity",
    "UpperRespiratoryInfection",
    "AlcoholWithdrawal",
    "BenzodiazepineWithdrawal",
    "BipolarDisorder",
    "GeneralizedAnxietyDisorder",
    "RemSleepBehaviorDisorder",
    "Schizophrenia",
    "SeizureDisorder",
    "Cancer",
    "Dysthymia",
    "MajorDepressionIncludingRemission",
    "PersonalityDisorderEmotionallyLabile",
    "PervasiveDevelopmentalDisorder",
    "SchizophreniaOrPsychoticDisorder",
    "ChronicKidneyDiseaseStage5",
    "EndStageRenalDisease",
    "EssentialHypertension",
    "KidneyTransplantRecipient",
    "DiagnosisOfHypertension",
    "AtherosclerosisAndPeripheralArterialDisease",
    "Breastfeeding",
    "CerebrovascularDiseaseStrokeOrTia",
    "FamilialHypercholesterolemia",
    "IschemicHeartDiseaseOrRelatedDiagnoses",
    "LiverDisease",
    "MyocardialInfarction",
    "Rhabdomyolysis",
    "StableAndUnstableAngina",
    "StatinAssociatedMuscleSymptoms",
    "CancerRelatedPain",
    "OpioidUseDisorder",
    "SickleCellDiseaseWithAndWithoutCrisis",
    "LowerBodyFracturesExcludingAnkleAndFoot",
    "MalignantNeoplasmsOfLowerAndUnspecifiedLimbs",
    "MechanicalComplicationsExcludingUpperBody",
    "BladderCancerForUrologyCare",
    "ImmunocompromisedConditions",
    "MixedHistologyUrothelialCellCarcinomaForUrologyCare",
    "TuberculosisForUrologyCare",
    "OverweightOrObese",
    "Underweight",
    "HistoryOfAtrialAblation",
    "IntravenousOrIntraarterialThrombolyticTpaTherapyPriorToArrival",
    "DentalCaries",
    "MorbidObesity",
    "UrinaryRetention",
    "MalnutritionDiagnosis",
    "ActiveBleedingOrBleedingDiathesisExcludingMenses",
    "AcutePepticUlcer",
    "AllergyToThrombolytics",
    "AorticDissectionOrRupturedAorticAneurysm",
    "CardiopulmonaryEmergency",
    "CerebralVascularLesion",
    "ClosedHeadAndFacialTrauma",
    "DementiaAndRelatedIntracranialPathologies",
    "IschemicStroke",
    "MalignantIntracranialNeoplasmGroup",
    "NeurologicImpairment",
    "PregnantState",
    "Stemi",
)
